diff -pruN 5.0.3-1/auxiliary/cgal_create_cmake_script.1 5.1-1/auxiliary/cgal_create_cmake_script.1
--- 5.0.3-1/auxiliary/cgal_create_cmake_script.1	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/auxiliary/cgal_create_cmake_script.1	2020-09-08 12:15:53.000000000 +0000
@@ -1,4 +1,4 @@
-.TH CGAL_CREATE_CMAKE_SCRIPT "1" "August 2020" "CGAL 5.0.3" "User Commands"
+.TH CGAL_CREATE_CMAKE_SCRIPT "1" "September 2020" "CGAL 5.1" "User Commands"
 .SH NAME
 cgal_create_cmake_script \- create a cmake script for applications using CGAL
 .SH SYNOPSIS
diff -pruN 5.0.3-1/benchmarks/Optimal_bounding_box/bench_obb.cpp 5.1-1/benchmarks/Optimal_bounding_box/bench_obb.cpp
--- 5.0.3-1/benchmarks/Optimal_bounding_box/bench_obb.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/benchmarks/Optimal_bounding_box/bench_obb.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,168 @@
+#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
+
+#include <CGAL/Surface_mesh.h>
+
+#include <CGAL/Optimal_bounding_box/oriented_bounding_box.h>
+
+#include <CGAL/convex_hull_3.h>
+#include <CGAL/Timer.h>
+
+#include <CGAL/IO/OFF_reader.h>
+#include <CGAL/IO/STL_reader.h>
+#include <CGAL/IO/OBJ_reader.h>
+
+#include <iostream>
+#include <fstream>
+#include <sstream>
+#include <vector>
+
+#define CGAL_OPTIMAL_BOUNDING_BOX_DEBUG_BENCHMARK
+
+typedef CGAL::Exact_predicates_inexact_constructions_kernel            K;
+typedef K::Point_3                                                     Point_3;
+
+typedef CGAL::Surface_mesh<Point_3>                                    Surface_mesh;
+typedef typename boost::graph_traits<Surface_mesh>::vertex_descriptor  vertex_descriptor;
+
+template <typename Point>
+bool read_mesh(const std::string filename,
+               std::vector<Point>& points)
+{
+  std::ifstream in(filename.c_str());
+  if(!in.good())
+  {
+//    std::cerr << "Error: can't read file at " << filename << std::endl;
+    return false;
+  }
+
+  std::vector<std::vector<std::size_t> > unused_faces;
+
+  std::string fn(filename);
+  if(fn.substr(fn.find_last_of(".") + 1) == "stl")
+  {
+    if(!CGAL::read_STL(in, points, unused_faces))
+      return false;
+  }
+  else if(fn.substr(fn.find_last_of(".") + 1) == "obj")
+  {
+    if(!CGAL::read_OBJ(in, points, unused_faces))
+      return false;
+  }
+  else if(fn.substr(fn.find_last_of(".") + 1) == "off")
+  {
+    if(!CGAL::read_OFF(in, points, unused_faces))
+      return false;
+  }
+  else
+  {
+//    std::cerr << "Error: unsupported file format: " << filename << std::endl;
+    return false;
+  }
+
+  return true;
+}
+
+void bench_finding_obb(const std::string filename,
+                       const int iter)
+{
+  CGAL::Timer timer;
+
+  std::vector<Point_3> points;
+  read_mesh(filename, points);
+
+  std::vector<Point_3> ch_points;
+  std::array<Point_3, 8> obb_points1;
+  std::array<Point_3, 8> obb_points2;
+
+  double ch_time = 0.;
+  double obb_time = 0.;
+  double total_time_ch = 0.;
+  double total_time_no_ch = 0.;
+
+  for(int i=0; i<iter; ++i)
+  {
+//    std::cout << "Iter #" << i << std::endl;
+//    std::cout << points.size() << std::endl;
+
+    // 1) measure convex hull calculation
+    timer.reset();
+    ch_points.clear();
+
+    timer.start();
+    extreme_points_3(points, std::back_inserter(ch_points));
+    timer.stop();
+    ch_time += timer.time();
+
+    // 2) using convex hull
+    timer.reset();
+
+    timer.start();
+    CGAL::oriented_bounding_box(ch_points, obb_points1, CGAL::parameters::use_convex_hull(false));
+    timer.stop();
+    obb_time += timer.time();
+
+    // 2bis) using convex hull in one go
+    timer.reset();
+
+    timer.start();
+    CGAL::oriented_bounding_box(points, obb_points1, CGAL::parameters::use_convex_hull(true));
+    timer.stop();
+    total_time_ch += timer.time();
+
+    // 3) without convex hull
+    timer.reset();
+
+    timer.start();
+    CGAL::oriented_bounding_box(points, obb_points2, CGAL::parameters::use_convex_hull(false));
+    timer.stop();
+    total_time_no_ch += timer.time();
+
+#ifdef CGAL_OPTIMAL_BOUNDING_BOX_DEBUG_BENCHMARK
+    Surface_mesh result_mesh1;
+    CGAL::make_hexahedron(obb_points1[0], obb_points1[1], obb_points1[2], obb_points1[3],
+                          obb_points1[4], obb_points1[5], obb_points1[6], obb_points1[7],
+                          result_mesh1);
+
+    Surface_mesh result_mesh2;
+    CGAL::make_hexahedron(obb_points2[0], obb_points2[1], obb_points2[2], obb_points2[3],
+                          obb_points2[4], obb_points2[5], obb_points2[6], obb_points2[7],
+                          result_mesh2);
+
+    std::stringstream oss1;
+    oss1 << "data/obb_result1_iter_" << i << std::ends;
+    std::ofstream out1(oss1.str().c_str());
+    out1 << result_mesh1;
+
+    std::stringstream oss2;
+    oss2 << "data/obb_result2_iter_" << i << std::ends;
+    std::ofstream out2(oss2.str().c_str());
+    out2 << result_mesh2;
+#endif
+  }
+
+#if 1 // only outputs the core stuff
+  std::cout << points.size() << " "
+            << ch_points.size() << " "
+            << ch_time / iter << " "
+            << obb_time / iter << " "
+            << total_time_ch / iter << " "
+            << total_time_no_ch / iter << std::endl;
+#else
+  std::cout << "Average of " << iter << " iterations" << std::endl;
+  std::cout << points.size() << " vertices in the mesh" << std::endl;
+  std::cout << ch_points.size() << " vertices on the convex hull" << std::endl;
+  std::cout << ch_time / iter << " seconds to find the convex hull" << std::endl;
+  std::cout << obb_time / iter << " seconds to find the best rotation" << std::endl;
+  std::cout << total_time_ch / iter << " seconds to compute and find. "
+            << "Should be about equal to: " << (ch_time + obb_time) / iter << std::endl;
+  std::cout << total_time_no_ch / iter << " seconds to find the best rotation (NO CH)" << std::endl;
+#endif
+}
+
+int main(int argc, char** argv)
+{
+  const int iter = (argc > 2) ? std::atoi(argv[2]) : 5;
+  bench_finding_obb((argc > 1) ? argv[1] : "data/elephant.off", iter);
+
+  return EXIT_SUCCESS;
+}
diff -pruN 5.0.3-1/benchmarks/Optimal_bounding_box/CMakeLists.txt 5.1-1/benchmarks/Optimal_bounding_box/CMakeLists.txt
--- 5.0.3-1/benchmarks/Optimal_bounding_box/CMakeLists.txt	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/benchmarks/Optimal_bounding_box/CMakeLists.txt	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,26 @@
+# Created by the script cgal_create_CMakeLists
+# This is the CMake script for compiling a set of CGAL applications.
+
+cmake_minimum_required(VERSION 3.1...3.15)
+project( Optimal_bounding_box_Benchmark )
+
+# CGAL and its components
+find_package( CGAL QUIET )
+
+if ( NOT CGAL_FOUND )
+  message(STATUS "This project requires the CGAL library, and will not be compiled.")
+  return()
+endif()
+
+# include helper file
+include( ${CGAL_USE_FILE} )
+
+find_package(Eigen3 3.1.0 REQUIRED) #(3.1.0 or greater)
+include(CGAL_Eigen_support)
+if (NOT TARGET CGAL::Eigen_support)
+  message(STATUS "This project requires the Eigen library, and will not be compiled.")
+  return()
+endif()
+
+create_single_source_cgal_program("bench_obb.cpp")
+target_link_libraries(bench_obb PUBLIC CGAL::Eigen_support)
diff -pruN 5.0.3-1/CHANGES.md 5.1-1/CHANGES.md
--- 5.0.3-1/CHANGES.md	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/CHANGES.md	2020-09-08 12:12:19.000000000 +0000
@@ -1,6 +1,261 @@
 Release History
 ===============
 
+[Release 5.1](https://github.com/CGAL/cgal/releases/tag/releases%2FCGAL-5.1)
+-----------
+
+Release date: September 2020
+
+### [Tetrahedral Remeshing](https://doc.cgal.org/5.1/Manual/packages.html#PkgTetrahedralRemeshing) (new package)
+
+-   This package implements a tetrahedral isotropic remeshing algorithm,
+    that improves the quality of tetrahedra in terms of dihedral angles,
+    while targeting a given edge length.
+
+    See also the associated [blog entry](https://www.cgal.org/2020/08/07/Tetrahedral-remeshing/).
+
+### [Surface Mesh Topology](https://doc.cgal.org/5.1/Manual/packages.html#PkgSurfaceMeshTopologySummary) (new package)
+
+-   This package enables the computation of some topological invariants of surfaces, such as:
+    - test if two (closed) curves on a combinatorial surface are homotopic. Users can choose
+      between free homotopy and homotopy with fixed endpoints;
+    - test is a curve is contractible;
+    - compute shortest non-contractible cycles on a surface, with or without weights on edges.
+
+    See also the associated [blog entry](https://www.cgal.org/2020/05/08/Surface_mesh_topology/).
+
+### [Optimal Bounding Box](https://doc.cgal.org/5.1/Manual/packages.html#PkgOptimalBoundingBox) (new package)
+
+-   This package implements an optimization algorithm that aims to construct a close approximation
+    of the *optimal bounding box* of a mesh or a point set, which is defined as the smallest
+    (in terms of volume) bounding box that contains a given mesh or point set.
+
+    See also the associated [blog entry](https://www.cgal.org/2020/04/20/Optimal_bounding_box/).
+
+### Installation
+
+-   The CGAL\_Core library no longer requires `Boost.Thread`, even if the g++ compiler is used.
+-   The minimal supported version of Boost is now 1.66.0.
+
+### [Tutorials](https://doc.cgal.org/5.1/Manual/tutorials.html)
+
+-   Two new, detailed tutorials have been added:
+    - [Surface Reconstruction from Point Clouds](https://doc.cgal.org/5.1/Manual/tuto_reconstruction.html),
+      which goes over a typical full processing pipeline in a CGAL environment.
+    - [Geographic Information Systems (GIS)](https://doc.cgal.org/5.1/Manual/tuto_gis.html),
+      which demonstrates usage of CGAL data structures and algorithms in the context of a typical GIS application.
+
+    Both tutorials provide complete code.
+
+### [2D and 3D Linear Geometry Kernel](https://doc.cgal.org/5.1/Manual/packages.html#PkgKernel23)
+
+-   Added the functor [`CompareSignedDistanceToLine_2`](https://doc.cgal.org/5.1/Kernel_23/classKernel_1_1CompareSignedDistanceToLine__2.html)
+    to the 2D/3D [`Kernel`](https://doc.cgal.org/5.1/Kernel_23/classKernel.html) concept to compare
+    the signed distance of two points to a line, or the line passing through two given points.
+    Corresponding functors in the model ([`Compare_signed_distance_to_line_2`](https://doc.cgal.org/5.1/Kernel_23/classKernel.html#a066d07dd592ac36ba7ee90988abd349f)) are also added.
+
+### [dD Geometry Kernel](https://doc.cgal.org/5.1/Manual/packages.html#PkgKernelD)
+
+-   The kernels [`Epick_d`](https://doc.cgal.org/5.1/Kernel_d/structCGAL_1_1Epick__d.html)
+    and [`Epeck_d`](https://doc.cgal.org/5.1/Kernel_d/structCGAL_1_1Epeck__d.html) gain two new functors:
+    [`Power_side_of_bounded_power_sphere_d`](https://doc.cgal.org/5.1/Kernel_d/classCGAL_1_1Epeck__d_1_1Power__side__of__bounded__power__sphere__d.html)
+    and [`Compute_squared_radius_smallest_orthogonal_sphere_d`](https://doc.cgal.org/5.1/Kernel_d/classCGAL_1_1Epeck__d_1_1Compute__squared__radius__smallest__orthogonal__sphere__d.html).
+    Those are essential for the computation of weighted alpha-complexes.
+
+### [Surface Mesh](https://doc.cgal.org/5.1/Manual/packages.html#PkgSurfaceMesh)
+
+-   **Breaking change**: The function [`CGAL::Surface_mesh::clear()`](https://doc.cgal.org/5.1/Surface_mesh/classCGAL_1_1Surface__mesh.html#a247d4ad3e6b106ae22e5306203812642)
+    now removes all non-default properties instead of just emptying them.
+
+### [CGAL and the Boost Graph Library (BGL)](https://doc.cgal.org/5.1/Manual/packages.html#PkgBGL)
+
+-   Added the function [`CGAL::alpha_expansion_graphcut()`](https://doc.cgal.org/5.1/BGL/group__PkgBGLPartition.html#ga79c3f58b577af51d1140450729d38f22),
+    which regularizes a multi-label partition over a user-defined graph.
+-   Added the function [`CGAL::regularize_face_selection_borders()`](https://doc.cgal.org/5.1/BGL/group__PkgBGLSelectionFct.html#gac71322b0cc7d7d59447531d5e5e345b6),
+    which uses this alpha expansion graphcut to regularize the borders of a selected faces on a triangle mesh.
+-   Added the function [`CGAL::set_triangulation_ids()`](https://doc.cgal.org/5.1/BGL/group__BGLGraphExternalIndices.html#ga1a22cf8bdde32fcdf1a4a78966eed630),
+    which must be used to initialize vertex, edge, and face indices of a triangulation meant to be used with BGL algorithms.
+
+### [3D Fast Intersection and Distance Computation](https://doc.cgal.org/5.1/Manual/packages.html#PkgAABBTree)
+
+-   The behavior of the internal search tree used to accelerate distance queries has changed:
+    usage of the internal search tree will now be enabled by default, and its construction
+    will be triggered by the first distance query. Automatic construction and usage can be disabled
+    by calling [`CGAL::AABB_tree::do_not_accelerate_distance_queries()`](https://doc.cgal.org/5.1/AABB_tree/classCGAL_1_1AABB__tree.html#abde62f52ccdf411847151aa5000ba4a4)
+    before the first distance query, and the tree can be built at any moment by calling
+    [`CGAL::AABB_tree::accelerate_distance_queries()`](https://doc.cgal.org/5.1/AABB_tree/classCGAL_1_1AABB__tree.html#a5d3877d3f2afbd09341eb4b8c230080b).
+-   **Breaking change**: [`CGAL::AABB_tree::accelerate_distance_queries()`](https://doc.cgal.org/5.1/AABB_tree/classCGAL_1_1AABB__tree.html#a5d3877d3f2afbd09341eb4b8c230080b)
+    and [`CGAL::AABB_tree::do_not_accelerate_distance_queries()`](https://doc.cgal.org/5.1/AABB_tree/classCGAL_1_1AABB__tree.html#abde62f52ccdf411847151aa5000ba4a4)
+    are no longer `const` functions.
+
+### [2D Arrangements](https://doc.cgal.org/5.1/Manual/packages.html#PkgArrangementOnSurface2)
+
+ -   Changed intersection return type from legacy [`CGAL::Object`](https://doc.cgal.org/5.1/STL_Extension/classCGAL_1_1Object.html)
+     to modern `boost::variant` in all traits concepts and models.
+     As there exists an implicit conversion from `boost::variant` to `CGAL::Object`, the
+     new code is backward compatible. However, it is recommended that all calls
+     to the intersection functions are fixed to use the new return type.
+
+### [2D Regularized Boolean Set-Operations](https://doc.cgal.org/5.1/Manual/packages.html#PkgBooleanSetOperations2)
+
+ -   Changed intersection return type from legacy [`CGAL::Object`](https://doc.cgal.org/5.1/STL_Extension/classCGAL_1_1Object.html)
+     to modern `boost::variant` in the concept [`ArrDirectionalTraits::Intersect_2`](https://doc.cgal.org/5.1/Boolean_set_operations_2/namespaceArrDirectionalTraits.html)
+     and its models.
+
+### [2D Minkowski Sums](https://doc.cgal.org/5.1/Manual/packages.html#PkgMinkowskiSum2)
+
+ -   Changed intersection return type from legacy [`CGAL::Object`](https://doc.cgal.org/5.1/STL_Extension/classCGAL_1_1Object.html)
+     to modern `boost::variant` in the (internally used) model `Arr_labeled_traits_2`.
+
+### [dD Spatial Searching](https://doc.cgal.org/5.1/Manual/packages.html#PkgSpatialSearchingD)
+
+-   The kd-tree can now be built in parallel: [`CGAL::Kd_tree::build()`](https://doc.cgal.org/5.1/Spatial_searching/classCGAL_1_1Kd__tree.html#a8559dbe4d7136fbc8ebab5ee290cbe06)
+    is given an optional template parameter `ConcurrencyTag` (default
+    value remains [`CGAL::Sequential_tag`](https://doc.cgal.org/5.1/STL_Extension/structCGAL_1_1Sequential__tag.html)
+    for backward compatibility).
+-   Improved the performance of the kd-tree in some cases:
+    - Not storing the points coordinates inside the tree usually
+      generates a lot of cache misses, leading to non-optimal
+      performance. This is the case for example
+      when indices are stored inside the tree, or to a lesser extent when the points
+      coordinates are stored in a dynamically allocated array (e.g., [`Epick_d`](https://doc.cgal.org/5.1/Kernel_d/structCGAL_1_1Epick__d.html)
+      with dynamic dimension) &mdash; we says "to a lesser extent" because the points
+      are re-created by the kd-tree in a cache-friendly order after its construction,
+      so the coordinates are more likely to be stored in a near-optimal order
+      on the heap.
+      In these cases, the new `EnablePointsCache` template parameter of the
+      [`CGAL::Kd_tree`](https://doc.cgal.org/5.1/Spatial_searching/classCGAL_1_1Kd__tree.html)
+      class can be set to `CGAL::Tag_true`. The points coordinates
+      will then be cached in an optimal way. This will increase memory
+      consumption but provides better search performance. See the updated
+      [`GeneralDistance`](https://doc.cgal.org/5.1/Spatial_searching/classGeneralDistance.html)
+      and [`FuzzyQueryItem`](https://doc.cgal.org/5.1/Spatial_searching/classFuzzyQueryItem.html)
+      concepts for additional requirements when using such a cache.
+    - In most cases (e.g., Euclidean distance), the distance computation
+      algorithm knows before its end that the distance will be greater
+      than or equal to some given value. This is used in the (orthogonal)
+      k-NN search to interrupt some distance computations before its end,
+      saving precious milliseconds, in particular in medium-to-high dimension.
+
+### [Intersecting Sequences of dD Iso-oriented Boxes](https://doc.cgal.org/5.1/Manual/packages.html#PkgBoxIntersectionD)
+
+-   Added parallel versions of the functions
+    [`CGAL::box_intersection_d()`](https://doc.cgal.org/5.1/Box_intersection_d/group__PkgBoxIntersectionD__box__intersection__d.html)
+    and [`CGAL::box_self_intersection_d()`](https://doc.cgal.org/5.1/Box_intersection_d/group__PkgBoxIntersectionD__box__self__intersection__d.html).
+
+### [Spatial Sorting](https://doc.cgal.org/5.1/Manual/packages.html#PkgSpatialSorting)
+
+-   Added parallel versions of the functions
+    [`CGAL::hilbert_sort()`](https://doc.cgal.org/5.1/Spatial_sorting/group__PkgSpatialSortingFunctions.html#ga9da67204747ac19dff65f9c9ff2fca9e)
+    and [`CGAL::spatial_sort()`](https://doc.cgal.org/5.1/Spatial_sorting/group__PkgSpatialSortingFunctions.html#ga7c597c11a3b3859234ff68526cead84d)
+    in 2D and 3D when the median policy is used.
+    The parallel versions use up to four threads in 2D, and up to eight threads in 3D.
+
+### [3D Convex Hulls](https://doc.cgal.org/5.1/Manual/packages.html#PkgConvexHull3)
+
+-   A new overload for [`CGAL::convex_hull_3()`](https://doc.cgal.org/5.1/Convex_hull_3/group__PkgConvexHull3Functions.html#gaa02a3013808fc9a2e5e2f42b9fde8e30)
+    that takes a model of [`VertexListGraph`](https://doc.cgal.org/5.1/BGL/classVertexListGraph.html) has been added.
+-   The long-deprecated function `CGAL::convex_hull_3_to_polyhedron_3()` has been removed.
+    The function [`CGAL::convex_hull_3_to_face_graph()`](https://doc.cgal.org/5.1/Convex_hull_3/group__PkgConvexHull3Functions.html#ga2750f7f197588ed643679835c748c671)
+    should be used instead.
+
+### [Polygon Mesh Processing](https://doc.cgal.org/5.1/Manual/packages.html#PkgPolygonMeshProcessing)
+
+-   Added the function [`CGAL::Polygon_mesh_processing::volume_connected_component()`](https://doc.cgal.org/5.1/Polygon_mesh_processing/group__PMP__orientation__grp.html#ga133e58280959c152770525f27bb42b91),
+    which can be used to get information about the nesting of the connected components of a given triangle mesh and about
+    the volumes defined.
+-   Added the function [`CGAL::Polygon_mesh_processing::remove_connected_components_of_negligible_size()`](https://doc.cgal.org/5.1/Polygon_mesh_processing/group__PMP__repairing__grp.html#gac544fcaba1d59d330a3a1536caff392a),
+    which can be used to remove connected components whose area or volume is under a certain threshold.
+    Area and volume thresholds are either specified by the user or deduced from the bounding box of the mesh.
+-   Added a new named parameter for [`CGAL::Polygon_mesh_processing::keep_large_connected_components()`](https://doc.cgal.org/5.1/Polygon_mesh_processing/group__keep__connected__components__grp.html#ga48e7b3e6922ee78cf8ce801e3e325d9a)
+    and [`CGAL::Polygon_mesh_processing::remove_connected_components_of_negligible_size`](https://doc.cgal.org/5.1/Polygon_mesh_processing/group__PMP__repairing__grp.html#gac544fcaba1d59d330a3a1536caff392a),
+    which can be used to perform a dry run of the operation, meaning that the function will return the number of connected
+    components that would be removed with the specified threshold, but without actually removing them.
+-   Added the function [`CGAL::Polygon_mesh_processing::split()`](https://doc.cgal.org/5.1/Polygon_mesh_processing/group__PMP__corefinement__grp.html#gaa491feee9e41f725332bea0ea1215578),
+    which can be used to split meshes along a mesh or a plane.
+-   Added the function [`CGAL::Polygon_mesh_processing::split_connected_components()`](https://doc.cgal.org/5.1/Polygon_mesh_processing/group__keep__connected__components__grp.html#ga9ddd1e4b915a4232b1ce5611985302aa)
+    to split a single mesh containing several connected components into several meshes containing one connected component.
+-   Added the functions [`CGAL::Polygon_mesh_processing::merge_reversible_connected_components()`](https://doc.cgal.org/5.1/Polygon_mesh_processing/group__PMP__orientation__grp.html#gae25c1198a89c53d5df2f29dd57fda5ca),
+    [`CGAL::Polygon_mesh_processing::duplicate_non_manifold_edges_in_polygon_soup()`](https://doc.cgal.org/5.1/Polygon_mesh_processing/group__PMP__orientation__grp.html#ga2aa4f7b500dc51d1fc4747705a050946),
+    and [`CGAL::Polygon_mesh_processing::orient_triangle_soup_with_reference_triangle_mesh()`](https://doc.cgal.org/5.1/Polygon_mesh_processing/group__PMP__orientation__grp.html#ga31779672b3afd660664fc9a6c4fdf74d),
+    which can be helpful when repairing a polygon soup.
+-   Added the function [`CGAL::Polygon_mesh_processing::sample_triangle_soup()`](https://doc.cgal.org/5.1/Polygon_mesh_processing/group__PMP__distance__grp.html#gac7af41d13bf1a7c30852be266ac81db5),
+    which generates points on a triangle soup surface.
+-   Added parallel versions of the functions [`CGAL::Polygon_mesh_processing::does_self_intersect()`](https://doc.cgal.org/5.1/Polygon_mesh_processing/group__PMP__intersection__grp.html#gad9fe5d8b433545b69154f43935a11a3b)
+    and [`CGAL::Polygon_mesh_processing::self_intersections()`](https://doc.cgal.org/5.1/Polygon_mesh_processing/group__PMP__intersection__grp.html#gaf19c80ec12cbff7ebe9e69453f1d40b8).
+-   The function [`CGAL::Polygon_mesh_processing::stitch_borders()`](https://doc.cgal.org/5.1/Polygon_mesh_processing/group__PMP__repairing__grp.html#ga8ae4352e67d2b099994ac8990c13bd41)
+    now returns the number of halfedge pairs that were stitched.
+-   Added the function [`CGAL::Polygon_mesh_processing::polygon_mesh_to_polygon_soup()`](https://doc.cgal.org/5.1/Polygon_mesh_processing/group__PMP__repairing__grp.html#ga76648a509409ff3c3ad3f71eff8ce9d9).
+-   The function [`CGAL::Polygon_mesh_processing::polygon_soup_to_polygon_mesh`](https://doc.cgal.org/5.1/Polygon_mesh_processing/group__PMP__repairing__grp.html#ga0dec58e8a0112791f72ebbe77bac074b)
+    now allows passing a point map (for the point range) and a vertex point map (for the polygon mesh) via named parameters.
+
+### [Point Set Processing](https://doc.cgal.org/5.1/Manual/packages.html#PkgPointSetProcessing3)
+
+-   **Breaking change:** [`CGAL::remove_outliers()`](https://doc.cgal.org/5.1/Point_set_processing_3/group__PkgPointSetProcessing3Algorithms.html#ga1ab1dcee59caadde50572c5a504cc41a)
+    has been parallelized and thus has a new template parameter `ConcurrencyTag`.
+    To update your code simply add as first template parameter `CGAL::Sequential_tag` or `CGAL::Parallel_tag`
+    when calling this function.
+-   Add a function [`CGAL::cluster_point_set()`](https://doc.cgal.org/5.1/Point_set_processing_3/group__PkgPointSetProcessing3Algorithms.html#gafee41d60b5a257ae034e9157d0af8e46)
+    that segments a point cloud into connected components based on a distance threshold.
+-   Added wrapper functions for registration:
+    - [`CGAL::OpenGR::compute_registration_transformation()`](https://doc.cgal.org/5.1/Point_set_processing_3/group__PkgPointSetProcessing3Algorithms.html#gab81663c718960780ddb176aad845e8cd),
+      which computes the registration transformation for two point sets using the Super4PCS algorithm
+      implemented in the third party library [OpenGR](https://storm-irit.github.io/OpenGR/index.html).
+    - [`CGAL::OpenGR::register_point_sets()`](https://doc.cgal.org/5.1/Point_set_processing_3/group__PkgPointSetProcessing3Algorithms.html#ga6194087f512e4e23dd945a9364d0931d),
+      which computes the registration transformation for two point sets using the Super4PCS algorithm
+      implemented in the third party library [OpenGR](https://storm-irit.github.io/OpenGR/index.html),
+      and registers the points sets by transforming the data point set using the computed transformation.
+    - [`CGAL::pointmatcher::compute_registration_transformation()`](https://doc.cgal.org/5.1/Point_set_processing_3/group__PkgPointSetProcessing3Algorithms.html#gaf75af5c1634fa83fa05a33e95570b127)
+      computes the registration transformation for two point sets using ICP algorithm implemented
+      in the third party library [libpointmatcher](https://github.com/ethz-asl/libpointmatcher).
+    - [`CGAL::pointmatcher::register_point_sets()`](https://doc.cgal.org/5.1/Point_set_processing_3/group__PkgPointSetProcessing3Algorithms.html#gaa222278e20a3ce41930d37326cd54ef9),
+      which computes the registration transformation for two point sets using ICP algorithm implemented
+      in the third party library [libpointmatcher](https://github.com/ethz-asl/libpointmatcher), and registers
+      the points sets by transforming the data point set using the computed transformation.
+
+### [2D Triangulations](https://doc.cgal.org/5.1/Manual/packages.html#PkgTriangulation2)
+
+-   To fix an inconsistency between code and documentation and to clarify which types of intersections
+    are truly allowed in constrained Delaunay triangulations, the tag [`CGAL::No_intersection_tag`](https://doc.cgal.org/5.1/Triangulation_2/structCGAL_1_1No__intersection__tag.html)
+    has been deprecated in favor of two new tags: [`CGAL::No_constraint_intersection_tag`](https://doc.cgal.org/5.1/Triangulation_2/structCGAL_1_1No__constraint__intersection__tag.html)
+    and [`CGAL::No_constraint_intersection_requiring_constructions_tag`](https://doc.cgal.org/5.1/Triangulation_2/structCGAL_1_1No__constraint__intersection__requiring__constructions__tag.html).
+    The latter is equivalent to the now-deprecated `CGAL::No_intersection_tag`, and allows constraints
+    to intersect as long as no new point has to be created to represent that intersection (for example,
+    the intersection of two constraint segments in a 'T'-like junction is an existing point
+    and as such does not require any new construction). The former tag, `CGAL::No_constraint_intersection_tag`,
+    does not allow any intersection, except for the configuration of two constraints having a single
+    common endpoints, for convience.
+-   Added the function [`CGAL::split_subconstraint_graph_into_constraints()`](https://doc.cgal.org/5.1/Triangulation_2/classCGAL_1_1Constrained__triangulation__plus__2.html#adea77f5db5cd4dfae302e4502f1caa85)
+    to [`Constrained_triangulation_plus_2`](https://doc.cgal.org/5.1/Triangulation_2/classCGAL_1_1Constrained__triangulation__plus__2.html) to initialize the constraints
+    from a soup of disconnected segments that should first be split into polylines.
+
+### [3D Triangulations](https://doc.cgal.org/5.1/Manual/packages.html#PkgTriangulation3)
+
+-   The member function [`CGAL::Triangulation_3::file_input()`](https://doc.cgal.org/5.1/Triangulation_3/group__PkgIOTriangulation3.html#gadd94d0613e2dd9cdd2e88d2c74d5b1c8)
+    have been added. It allows to load a [`CGAL::Triangulation_3`](https://doc.cgal.org/5.1/Triangulation_3/classCGAL_1_1Triangulation__3.html)
+    from an input stream, using functors to create vertices and cells.
+
+### [3D Triangulation Data Structure](https://doc.cgal.org/5.1/Manual/packages.html#PkgTDS3)
+
+-   The member function [`CGAL::TDS_3::file_input()`](https://doc.cgal.org/5.1/TDS_3/group__PkgIOTDS3.html#ga381446a02a9240cc83e79c48b37cd119)
+    have been added. It allows to load a [`CGAL::Triangulation_data_structure_3`](https://doc.cgal.org/5.1/TDS_3/classCGAL_1_1Triangulation__data__structure__3.html)
+    from an input stream, using functors to create vertices and cells.
+
+### [Surface Mesh Simplification](https://doc.cgal.org/5.1/Manual/packages.html#PkgSurfaceMeshSimplification)
+
+-   Added a [new simplification method](https://doc.cgal.org/5.1/Surface_mesh_simplification/classCGAL_1_1Surface__mesh__simplification_1_1GarlandHeckbert__policies.html)
+    based on the quadric error defined by Garland and Heckbert.
+-   The concept `EdgeProfile` has been removed. This concept was not actually in use as the CGAL-provided model [`CGAL::Edge_profile`](https://doc.cgal.org/5.1/Surface_mesh_simplification/classCGAL_1_1Surface__mesh__simplification_1_1Edge__profile.html)
+    was imposed to the user. Other concepts have been clarified to reflect the fact that the API uses this particular class.
+
+### [STL Extensions for CGAL](https://doc.cgal.org/5.1/Manual/packages.html#PkgSTLExtension)
+
+-   Added a new concurrency tag: [`CGAL::Parallel_if_available_tag`](https://doc.cgal.org/5.1/STL_Extension/structCGAL_1_1Parallel__if__available__tag.html).
+    This tag is a convenience typedef to [`CGAL::Parallel_tag`](https://doc.cgal.org/5.1/STL_Extension/structCGAL_1_1Parallel__tag.html)
+    if the third party library TBB has been found and linked with, and to
+    [`CGAL::Sequential_tag`](https://doc.cgal.org/5.1/STL_Extension/structCGAL_1_1Sequential__tag.html) otherwise.
+
+
 [Release 5.0](https://github.com/CGAL/cgal/releases/tag/releases%2FCGAL-5.0)
 -----------
 
@@ -25,13 +280,13 @@ Release date: November 2019
 
 ### [Polygonal Surface Reconstruction](https://doc.cgal.org/5.0/Manual/packages.html#PkgPolygonalSurfaceReconstruction) (new package)
 
- -   This package provides a method for piecewise planar object reconstruction from point clouds. 
-     The method takes as input an unordered point set sampled from a piecewise planar object 
-     and outputs a compact and watertight surface mesh interpolating the input point set. 
-     The method assumes that all necessary major planes are provided (or can be extracted from 
-     the input point set using the shape detection method described in Point Set Shape Detection, 
-     or any other alternative methods).The method can handle arbitrary piecewise planar objects 
-     and is capable of recovering sharp features and is robust to noise and outliers. See also 
+ -   This package provides a method for piecewise planar object reconstruction from point clouds.
+     The method takes as input an unordered point set sampled from a piecewise planar object
+     and outputs a compact and watertight surface mesh interpolating the input point set.
+     The method assumes that all necessary major planes are provided (or can be extracted from
+     the input point set using the shape detection method described in Point Set Shape Detection,
+     or any other alternative methods).The method can handle arbitrary piecewise planar objects
+     and is capable of recovering sharp features and is robust to noise and outliers. See also
      the associated [blog entry](https://www.cgal.org/2019/08/05/Polygonal_surface_reconstruction/).
 
 ### [Shape Detection](https://doc.cgal.org/5.0/Manual/packages.html#PkgShapeDetection) (major changes)
@@ -46,11 +301,11 @@ Release date: November 2019
 ### [2D and 3D Linear Geometry Kernel](https://doc.cgal.org/5.0/Manual/packages.html#PkgKernel23)
  -   Added a new concept, [`ComputeApproximateAngle_3`](https://doc.cgal.org/5.0/Kernel_23/classKernel_1_1ComputeApproximateAngle__3.html),
      to the 3D Kernel concepts to compute the approximate angle between two 3D vectors. Corresponding functors
-     in the model ([`Compute_approximate_angle_3`](https://doc.cgal.org/5.0/Kernel_23/classKernel.html#a183c9ac358a4ccddc04e680f8ed16c0b)) 
+     in the model ([`Compute_approximate_angle_3`](https://doc.cgal.org/5.0/Kernel_23/classKernel.html#a183c9ac358a4ccddc04e680f8ed16c0b))
      and free function ([`approximate_angle`](https://doc.cgal.org/5.0/Kernel_23/group__approximate__angle__grp.html))
      have also been added.
- -   The following objects are now hashable and thus trivially usable 
-     with [`std::unordered_set`](https://en.cppreference.com/w/cpp/container/unordered_set) 
+ -   The following objects are now hashable and thus trivially usable
+     with [`std::unordered_set`](https://en.cppreference.com/w/cpp/container/unordered_set)
      and [`std::unordered_map`](https://en.cppreference.com/w/cpp/header/unordered_map):
      `CGAL::Aff_transformation_2`, `CGAL::Aff_transformation_3`,
      `CGAL::Bbox_2`, `CGAL::Bbox_3`, `CGAL::Circle_2`,
@@ -60,11 +315,11 @@ Release date: November 2019
      `CGAL::Weighted_point_2` and `CGAL::Weighted_point_3`.
 
 ### [Polygon Mesh Processing](https://doc.cgal.org/latest/Manual/packages.html#PkgPolygonMeshProcessing)
- -   Introduced a [wide range of new functions](https://doc.cgal.org/5.0/Polygon_mesh_processing/index.html#title36) 
+ -   Introduced a [wide range of new functions](https://doc.cgal.org/5.0/Polygon_mesh_processing/index.html#title36)
      related to location of queries on a triangle mesh,
      such as [`CGAL::Polygon_mesh_processing::locate(Point, Mesh)`](https://doc.cgal.org/5.0/Polygon_mesh_processing/group__PMP__locate__grp.html#gada09bd8740ba69ead9deca597d53cf15).
      The location of a point on a triangle mesh is expressed as the pair of a face and the barycentric
-     coordinates of the point in this face, enabling robust manipulation of locations 
+     coordinates of the point in this face, enabling robust manipulation of locations
      (for example, intersections of two 3D segments living within the same face).
  -   Added the mesh smoothing function [`smooth_mesh()`](https://doc.cgal.org/5.0/Polygon_mesh_processing/group__PMP__meshing__grp.html#gaa0551d546f6ab2cd9402bea12d8332a3),
      which can be used to improve the quality of triangle elements based on various geometric characteristics.
@@ -87,14 +342,14 @@ Release date: November 2019
      or vertices appearing in multiple umbrellas) of a mesh.
 
 ### [3D Point Set](https://doc.cgal.org/5.0/Manual/packages.html#PkgPointSet3)
- -   The [PLY IO functions](https://doc.cgal.org/5.0/Point_set_3/group__PkgPointSet3IO.html) now take an additional optional parameter to 
+ -   The [PLY IO functions](https://doc.cgal.org/5.0/Point_set_3/group__PkgPointSet3IO.html) now take an additional optional parameter to
      read/write comments from/in the PLY header.
 
 ### [Point Set Processing](https://doc.cgal.org/latest/Manual/packages.html#PkgPointSetProcessing3)
  -   **Breaking change**: the API using iterators and overloads for optional parameters (deprecated since
      CGAL 4.12) has been removed. The current (and now only) API uses ranges and Named Parameters.
  -   Added the possibility to use the named parameter
-     [`neighbor_radius`](https://doc.cgal.org/5.0/Point_set_processing_3/group__psp__namedparameters.html#PSP_neighbor_radius) 
+     [`neighbor_radius`](https://doc.cgal.org/5.0/Point_set_processing_3/group__psp__namedparameters.html#PSP_neighbor_radius)
      to use spherical neighbor queries instead of K-nearest neighbors queries for the following functions:
      [`CGAL::bilateral_smooth_point_set()`](https://doc.cgal.org/5.0/Point_set_processing_3/group__PkgPointSetProcessing3Algorithms.html#ga4f82723e2f0bb33f3677e29e0208a256),
      [`CGAL::jet_estimate_normals()`](https://doc.cgal.org/5.0/Point_set_processing_3/group__PkgPointSetProcessing3Algorithms.html#ga0cd0f87de690d4edf82740e856efa491),
@@ -137,7 +392,7 @@ Release date: November 2019
 ### [3D Triangulations](https://doc.cgal.org/5.0/Manual/packages.html#PkgTriangulation3)
  -   **Breaking change**: The [constructor](https://doc.cgal.org/5.0/Triangulation_3/classCGAL_1_1Triangulation__3.html#a63f67cf6aaadcee14318cf56a36d247a)
      and the [`insert()`](https://doc.cgal.org/5.0/Triangulation_3/classCGAL_1_1Triangulation__3.html#ad3353128386bbb51f79d0263e7f67337)
-     function of [`CGAL::Triangulation_3`](https://doc.cgal.org/5.0/Triangulation_3/classCGAL_1_1Triangulation__3.html) 
+     function of [`CGAL::Triangulation_3`](https://doc.cgal.org/5.0/Triangulation_3/classCGAL_1_1Triangulation__3.html)
      which take a range of points as argument are now guaranteed to
      insert the points following the order of `InputIterator`. Note
      that this change only affects the base class `Triangulation_3`
@@ -149,14 +404,14 @@ Release date: November 2019
 
 ### [Surface Mesh](https://doc.cgal.org/5.0/Manual/packages.html#PkgSurfaceMesh)
  -   Introduced new functions to read and write using the PLY format,
-     [`CGAL::read_ply()`](https://doc.cgal.org/5.0/Surface_mesh/group__PkgSurface__mesh.html#ga42f6ad486ddab74e13d3dc53f511c343) 
-     and [`CGAL::write_ply()`](https://doc.cgal.org/5.0/Surface_mesh/group__PkgSurface__mesh.html#ga77bbb79d449c981895eedb6c3c23bd14), 
+     [`CGAL::read_ply()`](https://doc.cgal.org/5.0/Surface_mesh/group__PkgSurface__mesh.html#ga42f6ad486ddab74e13d3dc53f511c343)
+     and [`CGAL::write_ply()`](https://doc.cgal.org/5.0/Surface_mesh/group__PkgSurface__mesh.html#ga77bbb79d449c981895eedb6c3c23bd14),
      enabling users to save and load additional property maps of the surface mesh.
 
 ###  [CGAL and Solvers](https://doc.cgal.org/5.0/Manual/packages.html#PkgSolverInterface)
  -   Added [concepts](https://doc.cgal.org/5.0/Solver_interface/group__PkgSolverInterfaceConcepts.html)
-     and [models](https://doc.cgal.org/5.0/Solver_interface/group__PkgSolverInterfaceRef.html) 
-     for solving Mixed Integer Programming (MIP) problems with or without constraints. 
+     and [models](https://doc.cgal.org/5.0/Solver_interface/group__PkgSolverInterfaceRef.html)
+     for solving Mixed Integer Programming (MIP) problems with or without constraints.
 
 ### [3D Boolean Operations on Nef Polyhedra](https://doc.cgal.org/5.0/Manual/packages.html#PkgNef3)
  -   Added a function to convert a Nef_polyhedron_3 to a polygon soup: [`CGAL::convert_nef_polyhedron_to_polygon_soup()`](https://doc.cgal.org/5.0/Nef_3/group__PkgNef3IOFunctions.html#ga28a9eb4da0cd6153f0c16f7f9eaf6665)
@@ -185,7 +440,7 @@ Release 4.14
 Release date: March 2019
 
 ### 2D Periodic Hyperbolic Triangulations (new package)
- 
+
  -   This package allows the computation of Delaunay triangulations of
      the Bolza surface.  The Bolza surface is the most symmetric
      hyperbolic surface of genus 2. Its fundamental domain is the
diff -pruN 5.0.3-1/cmake/modules/CGAL_Boost_iostreams_support.cmake 5.1-1/cmake/modules/CGAL_Boost_iostreams_support.cmake
--- 5.0.3-1/cmake/modules/CGAL_Boost_iostreams_support.cmake	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/cmake/modules/CGAL_Boost_iostreams_support.cmake	2020-09-08 12:12:19.000000000 +0000
@@ -0,0 +1,44 @@
+if(Boost_IOSTREAMS_FOUND AND NOT TARGET CGAL::Boost_iostreams_support)
+
+  if( WIN32 )
+    # to avoid a warning with old cmake
+    set(_Boost_BZIP2_HEADERS             "boost/iostreams/filter/bzip2.hpp")
+    set(_Boost_ZLIB_HEADERS              "boost/iostreams/filter/zlib.hpp")
+    find_package( Boost OPTIONAL_COMPONENTS bzip2 zlib)
+    if (Boost_ZLIB_FOUND AND Boost_BZIP2_FOUND)
+      set(ZLIB_LIBS ${Boost_ZLIB_LIBRARY} ${Boost_BZIP2_LIBRARY})
+    else()
+      message(STATUS "NOTICE: This project requires Boost ZLIB and Boost BZIP2, and will not be compiled.")
+      return()
+    endif()
+
+  else()
+
+    find_package(ZLIB QUIET)
+    if(ZLIB_FOUND)
+      set(ZLIB_LIBS ZLIB::ZLIB)
+    else()
+      message(STATUS "NOTICE: This project requires ZLIB, and will not be compiled.")
+      return()
+    endif()
+
+  endif()
+
+  if(TARGET Boost::iostreams)
+    set(Boost_LIB Boost::iostreams)
+  else()
+    set(Boost_LIB  ${Boost_IOSTREAMS_LIBRARY})
+  endif()
+
+  add_library(CGAL::Boost_iostreams_support INTERFACE IMPORTED)
+
+  if(CMAKE_VERSION VERSION_LESS 3.11)
+    set_target_properties(CGAL::Boost_iostreams_support PROPERTIES
+      INTERFACE_COMPILE_DEFINITIONS "CGAL_LINKED_WITH_BOOST_IOSTREAMS"
+      INTERFACE_LINK_LIBRARIES "${Boost_LIB};${ZLIB_LIBS}")
+  else()
+    set_target_properties(CGAL::Boost_iostreams_support PROPERTIES
+      INTERFACE_COMPILE_DEFINITIONS "CGAL_LINKED_WITH_BOOST_IOSTREAMS")
+    target_link_libraries(CGAL::Boost_iostreams_support INTERFACE ${Boost_LIB} ${ZLIB_LIBS})
+  endif()
+endif()
diff -pruN 5.0.3-1/cmake/modules/CGAL_Boost_serialization_support.cmake 5.1-1/cmake/modules/CGAL_Boost_serialization_support.cmake
--- 5.0.3-1/cmake/modules/CGAL_Boost_serialization_support.cmake	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/cmake/modules/CGAL_Boost_serialization_support.cmake	2020-09-08 12:12:19.000000000 +0000
@@ -0,0 +1,12 @@
+if(Boost_SERIALIZATION_FOUND AND NOT TARGET CGAL::Boost_serialization_support)
+  if(TARGET Boost::serialization)
+    set(Boost_LIB Boost::serialization)
+  else()
+    set(Boost_LIB  ${Boost_SERIALIZATION_LIBRARY})
+  endif()
+
+  add_library(CGAL::Boost_serialization_support INTERFACE IMPORTED)
+  set_target_properties(CGAL::Boost_serialization_support PROPERTIES
+    INTERFACE_COMPILE_DEFINITIONS "CGAL_LINKED_WITH_BOOST_SERIALIZATION"
+    INTERFACE_LINK_LIBRARIES ${Boost_LIB})
+endif()
diff -pruN 5.0.3-1/cmake/modules/CGAL_Common.cmake 5.1-1/cmake/modules/CGAL_Common.cmake
--- 5.0.3-1/cmake/modules/CGAL_Common.cmake	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/cmake/modules/CGAL_Common.cmake	2020-09-08 12:12:19.000000000 +0000
@@ -23,7 +23,7 @@ if( NOT CGAL_COMMON_FILE_INCLUDED )
   else()
     set( CMAKE_2_6_3_OR_ABOVE FALSE )
   endif()
-    
+
   if ( CGAL_BUILDING_LIBS )
     option(BUILD_SHARED_LIBS "Build shared libraries" ON)
     set(CGAL_BUILD_SHARED_LIBS ${BUILD_SHARED_LIBS})
@@ -34,7 +34,7 @@ if( NOT CGAL_COMMON_FILE_INCLUDED )
       message( STATUS "Building static libraries" )
     endif()
   endif()
-  
+
   if ( WIN32 )
     find_program(CMAKE_UNAME uname /bin /usr/bin /usr/local/bin )
     if(CMAKE_UNAME)
@@ -61,5 +61,4 @@ if( NOT CGAL_COMMON_FILE_INCLUDED )
   # set use-file for Eigen3 (needed to have default solvers)
   set(EIGEN3_USE_FILE "UseEigen3")
 
-  include(${CMAKE_CURRENT_LIST_DIR}/CGAL_target_use_TBB.cmake)
 endif()
diff -pruN 5.0.3-1/cmake/modules/CGAL_Eigen_support.cmake 5.1-1/cmake/modules/CGAL_Eigen_support.cmake
--- 5.0.3-1/cmake/modules/CGAL_Eigen_support.cmake	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/cmake/modules/CGAL_Eigen_support.cmake	2020-09-08 12:12:19.000000000 +0000
@@ -0,0 +1,9 @@
+if(EIGEN3_FOUND AND NOT TARGET CGAL::Eigen_support)
+  if(NOT TARGET Threads::Threads)
+    find_package(Threads REQUIRED)
+  endif()
+  add_library(CGAL::Eigen_support INTERFACE IMPORTED)
+  set_target_properties(CGAL::Eigen_support PROPERTIES
+    INTERFACE_COMPILE_DEFINITIONS "CGAL_EIGEN3_ENABLED"
+    INTERFACE_INCLUDE_DIRECTORIES "${EIGEN3_INCLUDE_DIR}")
+endif()
diff -pruN 5.0.3-1/cmake/modules/CGAL_GLPK_support.cmake 5.1-1/cmake/modules/CGAL_GLPK_support.cmake
--- 5.0.3-1/cmake/modules/CGAL_GLPK_support.cmake	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/cmake/modules/CGAL_GLPK_support.cmake	2020-09-08 12:12:19.000000000 +0000
@@ -0,0 +1,7 @@
+if(GLPK_FOUND AND NOT TARGET CGAL::GLPK_support)
+  add_library(CGAL::GLPK_support INTERFACE IMPORTED)
+  set_target_properties(CGAL::GLPK_support PROPERTIES
+    INTERFACE_COMPILE_DEFINITIONS "CGAL_USE_GLPK"
+    INTERFACE_INCLUDE_DIRECTORIES "${GLPK_INCLUDE_DIR}"
+    INTERFACE_LINK_LIBRARIES "${GLPK_LIBRARIES}")
+endif()
diff -pruN 5.0.3-1/cmake/modules/CGAL_LASLIB_support.cmake 5.1-1/cmake/modules/CGAL_LASLIB_support.cmake
--- 5.0.3-1/cmake/modules/CGAL_LASLIB_support.cmake	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/cmake/modules/CGAL_LASLIB_support.cmake	2020-09-08 12:12:19.000000000 +0000
@@ -0,0 +1,7 @@
+if(LASLIB_FOUND AND NOT TARGET CGAL::LASLIB_support)
+  add_library(CGAL::LASLIB_support INTERFACE IMPORTED)
+  set_target_properties(CGAL::LASLIB_support PROPERTIES
+    INTERFACE_COMPILE_DEFINITIONS "CGAL_LINKED_WITH_LASLIB"
+    INTERFACE_INCLUDE_DIRECTORIES "${LASLIB_INCLUDE_DIR};${LASZIP_INCLUDE_DIR}"
+    INTERFACE_LINK_LIBRARIES "${LASLIB_LIBRARIES}")
+endif()
diff -pruN 5.0.3-1/cmake/modules/CGAL_Macros.cmake 5.1-1/cmake/modules/CGAL_Macros.cmake
--- 5.0.3-1/cmake/modules/CGAL_Macros.cmake	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/cmake/modules/CGAL_Macros.cmake	2020-09-08 12:12:19.000000000 +0000
@@ -145,6 +145,7 @@ if( NOT CGAL_MACROS_FILE_INCLUDED )
       message("Search dirs:")
       message("${search_dirs}")
     endif()
+    message( STATUS "USING COMPILER_VERSION = '${CMAKE_CXX_COMPILER_ID} ${CMAKE_CXX_COMPILER_VERSION}'" )
   endfunction()
 
   macro( get_dependency_version LIB )
@@ -323,7 +324,7 @@ if( NOT CGAL_MACROS_FILE_INCLUDED )
         endif()
       else()
 
-        if (NOT WITH_CGAL_${component}) 
+        if (NOT WITH_CGAL_${component})
           message(STATUS "NOTICE: The CGAL_${component} library seems to be required but is not build. Thus, it is expected that some executables will not be compiled.")
         endif()
 
@@ -702,4 +703,3 @@ function(process_CGAL_subdirectory entry
     message(STATUS "${subdir}/${ENTRY_DIR_NAME} is in dont_submit")
   endif()
 endfunction()
-
diff -pruN 5.0.3-1/cmake/modules/CGAL_OpenCV_support.cmake 5.1-1/cmake/modules/CGAL_OpenCV_support.cmake
--- 5.0.3-1/cmake/modules/CGAL_OpenCV_support.cmake	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/cmake/modules/CGAL_OpenCV_support.cmake	2020-09-08 12:12:19.000000000 +0000
@@ -0,0 +1,7 @@
+if(OpenCV_FOUND AND NOT TARGET CGAL::OpenCV_support)
+  add_library(CGAL::OpenCV_support INTERFACE IMPORTED)
+  set_target_properties(CGAL::OpenCV_support PROPERTIES
+    INTERFACE_COMPILE_DEFINITIONS "CGAL_LINKED_WITH_OPENCV"
+    INTERFACE_INCLUDE_DIRECTORIES "${OpenCV_INCLUDE_DIRS}"
+    INTERFACE_LINK_LIBRARIES "${OpenCV_LIBS}")
+endif()
diff -pruN 5.0.3-1/cmake/modules/CGAL_OpenGR_support.cmake 5.1-1/cmake/modules/CGAL_OpenGR_support.cmake
--- 5.0.3-1/cmake/modules/CGAL_OpenGR_support.cmake	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/cmake/modules/CGAL_OpenGR_support.cmake	2020-09-08 12:12:19.000000000 +0000
@@ -0,0 +1,6 @@
+if(OpenGR_FOUND AND NOT TARGET CGAL::OpenGR_support)
+  add_library(CGAL::OpenGR_support INTERFACE IMPORTED)
+  set_target_properties(CGAL::OpenGR_support PROPERTIES
+    INTERFACE_COMPILE_DEFINITIONS "CGAL_LINKED_WITH_OPENGR"
+    INTERFACE_INCLUDE_DIRECTORIES "${OpenGR_INCLUDE_DIR}")
+endif()
diff -pruN 5.0.3-1/cmake/modules/CGAL_parse_version_h.cmake 5.1-1/cmake/modules/CGAL_parse_version_h.cmake
--- 5.0.3-1/cmake/modules/CGAL_parse_version_h.cmake	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/cmake/modules/CGAL_parse_version_h.cmake	2020-09-08 12:12:19.000000000 +0000
@@ -14,5 +14,6 @@ function(cgal_parse_version_h version_h_
   MATH(EXPR ${ARGV3} "${CMAKE_MATCH_5}")        # minor version without leading 0
   set(${ARGV3} "${${ARGV3}}" PARENT_SCOPE)
   set(${ARGV4} "${CMAKE_MATCH_6}" PARENT_SCOPE)                 # patch number
-  set(${ARGV5} "${CMAKE_MATCH_7}" PARENT_SCOPE)                 # build number
+  MATH(EXPR ${ARGV5} "${CMAKE_MATCH_7}")        # build number version without leading 0
+  set(${ARGV5} "${${ARGV5}}" PARENT_SCOPE)                      # build number
 endfunction()
diff -pruN 5.0.3-1/cmake/modules/CGAL_pointmatcher_support.cmake 5.1-1/cmake/modules/CGAL_pointmatcher_support.cmake
--- 5.0.3-1/cmake/modules/CGAL_pointmatcher_support.cmake	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/cmake/modules/CGAL_pointmatcher_support.cmake	2020-09-08 12:12:19.000000000 +0000
@@ -0,0 +1,7 @@
+if(libpointmatcher_FOUND AND NOT TARGET CGAL::pointmatcher_support)
+  add_library(CGAL::pointmatcher_support INTERFACE IMPORTED)
+  set_target_properties(CGAL::pointmatcher_support PROPERTIES
+    INTERFACE_COMPILE_DEFINITIONS "CGAL_LINKED_WITH_POINTMATCHER"
+    INTERFACE_INCLUDE_DIRECTORIES "${libpointmatcher_INCLUDE_DIR}"
+    INTERFACE_LINK_LIBRARIES "${libpointmatcher_LIBRARIES}")
+endif()
diff -pruN 5.0.3-1/cmake/modules/CGAL_SCIP_support.cmake 5.1-1/cmake/modules/CGAL_SCIP_support.cmake
--- 5.0.3-1/cmake/modules/CGAL_SCIP_support.cmake	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/cmake/modules/CGAL_SCIP_support.cmake	2020-09-08 12:12:19.000000000 +0000
@@ -0,0 +1,7 @@
+if(SCIP_FOUND AND NOT TARGET CGAL::SCIP_support)
+  add_library(CGAL::SCIP_support INTERFACE IMPORTED)
+  set_target_properties(CGAL::SCIP_support PROPERTIES
+    INTERFACE_COMPILE_DEFINITIONS "CGAL_USE_SCIP"
+    INTERFACE_INCLUDE_DIRECTORIES "${SCIP_INCLUDE_DIRS}"
+    INTERFACE_LINK_LIBRARIES "${SCIP_LIBRARIES}")
+endif()
diff -pruN 5.0.3-1/cmake/modules/CGAL_SetupCGAL_CoreDependencies.cmake 5.1-1/cmake/modules/CGAL_SetupCGAL_CoreDependencies.cmake
--- 5.0.3-1/cmake/modules/CGAL_SetupCGAL_CoreDependencies.cmake	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/cmake/modules/CGAL_SetupCGAL_CoreDependencies.cmake	2020-09-08 12:12:19.000000000 +0000
@@ -53,13 +53,6 @@ endif()
 #   keyword, or ``PUBLIC`` otherwise.
 #
 
-# See the release notes of CGAL-4.10: CGAL_Core now requires
-# Boost.Thread, with GNU G++ < 9.1.
-if (CMAKE_CXX_COMPILER_ID STREQUAL GNU AND CMAKE_CXX_COMPILER_VERSION VERSION_LESS 9.1)
-  include(${CMAKE_CURRENT_LIST_DIR}/CGAL_TweakFindBoost.cmake)
-  find_package( Boost 1.48 REQUIRED COMPONENTS thread system )
-endif()
-
 function(CGAL_setup_CGAL_Core_dependencies target)
   if(ARGV1 STREQUAL INTERFACE)
     set(keyword INTERFACE)
@@ -71,20 +64,4 @@ function(CGAL_setup_CGAL_Core_dependenci
   target_compile_definitions(${target} ${keyword} CGAL_USE_CORE=1)
   target_link_libraries( CGAL_Core ${keyword} CGAL::CGAL )
 
-  # See the release notes of CGAL-4.10: CGAL_Core now requires
-  # Boost.Thread, with GNU G++.
-  if (CMAKE_CXX_COMPILER_ID STREQUAL GNU AND CMAKE_CXX_COMPILER_VERSION VERSION_LESS 9.1)
-    if(TARGET Boost::thread)
-      target_link_libraries( CGAL_Core ${keyword} Boost::thread)
-    else()
-      # Note that `find_package( Boost...)` must be called in the
-      # function `CGAL_setup_CGAL_Core_dependencies()` because the
-      # calling `CMakeLists.txt` may also call `find_package(Boost)`
-      # between the inclusion of this module, and the call to this
-      # function. That resets `Boost_LIBRARIES`.
-      find_package( Boost 1.48 REQUIRED thread system )
-      target_link_libraries( CGAL_Core ${keyword} ${Boost_LIBRARIES})
-    endif()
-  endif()
-
 endfunction()
diff -pruN 5.0.3-1/cmake/modules/CGAL_SetupGMP.cmake 5.1-1/cmake/modules/CGAL_SetupGMP.cmake
--- 5.0.3-1/cmake/modules/CGAL_SetupGMP.cmake	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/cmake/modules/CGAL_SetupGMP.cmake	2020-09-08 12:12:19.000000000 +0000
@@ -23,12 +23,12 @@ set(CMAKE_MODULE_PATH ${CMAKE_MODULE_PAT
 
 find_package(GMP REQUIRED)
 find_package(MPFR REQUIRED)
+find_package(GMPXX QUIET)
 
-if(NOT DEFINED WITH_GMPXX)
+if(NOT GMPXX_FOUND)
   option(CGAL_WITH_GMPXX "Use CGAL with GMPXX: use C++ classes of GNU MP instead of CGAL wrappers" OFF)
-endif()
-if(WITH_GMPXX OR CGAL_WITH_GMPXX)
-  find_package(GMPXX REQUIRED)
+else()
+  option(CGAL_WITH_GMPXX "Use CGAL with GMPXX: use C++ classes of GNU MP instead of CGAL wrappers" ON)
 endif()
 
 #.rst:
diff -pruN 5.0.3-1/cmake/modules/CGAL_target_use_TBB.cmake 5.1-1/cmake/modules/CGAL_target_use_TBB.cmake
--- 5.0.3-1/cmake/modules/CGAL_target_use_TBB.cmake	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/cmake/modules/CGAL_target_use_TBB.cmake	2020-09-08 12:12:19.000000000 +0000
@@ -6,7 +6,9 @@ set(CGAL_target_use_TBB_included TRUE)
 set(TBB_USE_FILE "UseTBB")
 
 function(CGAL_target_use_TBB target)
-  if(NOT TARGET Threads::Threads)
+  message(DEPRECATION "This file CGAL_target_use_TBB.cmake is deprecated, and the imported target `CGAL::TBB_support` from CGAL_TBB_support.cmake should be used instead.")
+  if(NOT TARGET
+      Threads::Threads)
     find_package(Threads REQUIRED)
   endif()
   target_link_libraries( ${target} PUBLIC TBB::tbb TBB::tbbmalloc Threads::Threads)
diff -pruN 5.0.3-1/cmake/modules/CGAL_TBB_support.cmake 5.1-1/cmake/modules/CGAL_TBB_support.cmake
--- 5.0.3-1/cmake/modules/CGAL_TBB_support.cmake	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/cmake/modules/CGAL_TBB_support.cmake	2020-09-08 12:12:19.000000000 +0000
@@ -0,0 +1,10 @@
+if(TBB_FOUND AND NOT TARGET CGAL::TBB_support)
+  if(NOT TARGET Threads::Threads)
+    find_package(Threads REQUIRED)
+  endif()
+  add_library(CGAL::TBB_support INTERFACE IMPORTED)
+  set_target_properties(CGAL::TBB_support PROPERTIES
+    INTERFACE_COMPILE_DEFINITIONS "CGAL_LINKED_WITH_TBB;NOMINMAX"
+    INTERFACE_INCLUDE_DIRECTORIES "${TBB_INCLUDE_DIRS}"
+    INTERFACE_LINK_LIBRARIES "TBB::tbb;TBB::tbbmalloc;Threads::Threads")
+endif()
diff -pruN 5.0.3-1/cmake/modules/CGAL_TensorFlow_support.cmake 5.1-1/cmake/modules/CGAL_TensorFlow_support.cmake
--- 5.0.3-1/cmake/modules/CGAL_TensorFlow_support.cmake	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/cmake/modules/CGAL_TensorFlow_support.cmake	2020-09-08 12:12:19.000000000 +0000
@@ -0,0 +1,7 @@
+if(TensorFlow_FOUND AND NOT TARGET CGAL::TensorFlow_support)
+  add_library(CGAL::TensorFlow_support INTERFACE IMPORTED)
+  set_target_properties(CGAL::TensorFlow_support PROPERTIES
+    INTERFACE_COMPILE_DEFINITIONS "CGAL_LINKED_WITH_TENSORFLOW"
+    INTERFACE_INCLUDE_DIRECTORIES "${TensorFlow_INCLUDE_DIR}"
+    INTERFACE_LINK_LIBRARIES "${TensorFlow_LIBRARY}")
+endif()
diff -pruN 5.0.3-1/cmake/modules/FindLibSSH.cmake 5.1-1/cmake/modules/FindLibSSH.cmake
--- 5.0.3-1/cmake/modules/FindLibSSH.cmake	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/cmake/modules/FindLibSSH.cmake	2020-09-08 12:12:19.000000000 +0000
@@ -0,0 +1,39 @@
+# - Try to find the LibSSH libraries
+# This module defines:
+#  LIBSSH_FOUND             - system has LibSSH lib
+#  LIBSSH_INCLUDE_DIR       - the LibSSH include directory
+#  LIBSSH_LIBRARIES_DIR     - directory where the LibSSH libraries are located
+#  LIBSSH_LIBRARIES         - Link these to use LibSSH
+
+
+include(FindPackageHandleStandardArgs)
+include(${CMAKE_CURRENT_LIST_DIR}/CGAL_GeneratorSpecificSettings.cmake)
+
+if(LIBSSH_INCLUDE_DIR)
+  set(LIBSSH_in_cache TRUE)
+else()
+  set(LIBSSH_in_cache FALSE)
+endif()
+if(NOT LIBSSH_LIBRARIES)
+  set(LIBSSH_in_cache FALSE)
+endif()
+
+# Is it already configured?
+if( NOT LIBSSH_in_cache )
+
+  find_path(LIBSSH_INCLUDE_DIR
+    NAMES "libssh/libssh.h"
+    )
+
+  find_library(LIBSSH_LIBRARIES NAMES ssh libssh
+    HINTS "/usr/lib"
+    "usr/lib/x86_64-linux-gnu"
+    PATH_SUFFIXES lib
+    DOC "Path to the LIBSSH library"
+    )
+endif()
+
+SET(LIBSSH_FOUND TRUE)
+if( NOT LIBSSH_LIBRARIES OR NOT LIBSSH_INCLUDE_DIR)
+  SET(LIBSSH_FOUND FALSE)
+endif()
diff -pruN 5.0.3-1/cmake/modules/FindMPFI.cmake 5.1-1/cmake/modules/FindMPFI.cmake
--- 5.0.3-1/cmake/modules/FindMPFI.cmake	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/cmake/modules/FindMPFI.cmake	2020-09-08 12:12:19.000000000 +0000
@@ -1,4 +1,5 @@
 find_package( GMP QUIET )
+find_package( MPFR QUIET )
 
 if( GMP_FOUND AND MPFR_FOUND )
 
diff -pruN 5.0.3-1/cmake/modules/FindOpenMesh.cmake 5.1-1/cmake/modules/FindOpenMesh.cmake
--- 5.0.3-1/cmake/modules/FindOpenMesh.cmake	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/cmake/modules/FindOpenMesh.cmake	2020-09-08 12:12:19.000000000 +0000
@@ -6,6 +6,8 @@
 #  OPENMESH_LIBRARIES - OpenMesh libraries
 #
 
+find_package(OpenMesh NO_MODULE QUIET)
+
 # Is it already configured?
 if (NOT OpenMesh_FOUND)
 
@@ -44,7 +46,34 @@ endif()
 
 include( FindPackageHandleStandardArgs )
 
-FIND_PACKAGE_HANDLE_STANDARD_ARGS(OpenMesh
-        REQUIRED_VARS OPENMESH_INCLUDE_DIR OPENMESH_LIBRARIES
-        FOUND_VAR OpenMesh_FOUND
-        )
+find_package_handle_standard_args(OpenMesh
+  REQUIRED_VARS OPENMESH_INCLUDE_DIR OPENMESH_LIBRARIES
+  FOUND_VAR OpenMesh_FOUND
+  )
+
+if(OpenMesh_FOUND AND NOT TARGET OpenMesh::OpenMesh)
+  add_library(OpenMesh::OpenMesh UNKNOWN IMPORTED)
+
+  if(TARGET OpenMeshCore)
+    target_link_libraries(OpenMesh::OpenMesh PUBLIC OpenMeshCore)
+    return()
+  endif()
+
+  set_target_properties(OpenMesh::OpenMesh PROPERTIES
+    INTERFACE_COMPILE_DEFINITIONS "CGAL_USE_OPENMESH;NOMINMAX;_USE_MATH_DEFINES"
+    INTERFACE_INCLUDE_DIRECTORIES  "${OPENMESH_INCLUDE_DIR}")
+
+  if(OPENMESH_LIBRARY_RELEASE)
+    set_property(TARGET OpenMesh::OpenMesh APPEND PROPERTY
+      IMPORTED_CONFIGURATIONS RELEASE)
+    set_target_properties(OpenMesh::OpenMesh PROPERTIES
+      IMPORTED_LOCATION_RELEASE "${OPENMESH_LIBRARY_RELEASE}")
+  endif()
+
+  if(OPENMESH_LIBRARY_DEBUG)
+    set_property(TARGET OpenMesh::OpenMesh APPEND PROPERTY
+      IMPORTED_CONFIGURATIONS DEBUG)
+    set_target_properties(OpenMesh::OpenMesh PROPERTIES
+      IMPORTED_LOCATION_DEBUG "${OPENMESH_LIBRARY_DEBUG}")
+  endif()
+endif()
diff -pruN 5.0.3-1/cmake/modules/list_of_whitelisted_headers.cmake 5.1-1/cmake/modules/list_of_whitelisted_headers.cmake
--- 5.0.3-1/cmake/modules/list_of_whitelisted_headers.cmake	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/cmake/modules/list_of_whitelisted_headers.cmake	2020-09-08 12:12:19.000000000 +0000
@@ -30,6 +30,7 @@ set(list_of_whitelisted_headers_txt [=[
   CGAL/Three/Triangle_container.h
   CGAL/Three/Viewer_config.h
   CGAL/Three/Viewer_interface.h
+  CGAL/Three/Three.h
 
 ]=])
 
diff -pruN 5.0.3-1/cmake/modules/UseEigen3.cmake 5.1-1/cmake/modules/UseEigen3.cmake
--- 5.0.3-1/cmake/modules/UseEigen3.cmake	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/cmake/modules/UseEigen3.cmake	2020-09-08 12:12:19.000000000 +0000
@@ -7,3 +7,5 @@ include_directories( SYSTEM ${EIGEN3_INC
 add_definitions(-DCGAL_EIGEN3_ENABLED)
 
 set (EIGEN3_SETUP TRUE)
+
+message(DEPRECATION "This file UseEigen.cmake is deprecated, and the imported target `CGAL::Eigen_support` from CGAL_Eigen_support.cmake should be used instead.")
diff -pruN 5.0.3-1/cmake/modules/UseLASLIB.cmake 5.1-1/cmake/modules/UseLASLIB.cmake
--- 5.0.3-1/cmake/modules/UseLASLIB.cmake	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/cmake/modules/UseLASLIB.cmake	2020-09-08 12:12:19.000000000 +0000
@@ -2,3 +2,5 @@
 # It assumes that find_package(LASLIB) was already called.
 
 add_definitions(-DCGAL_LINKED_WITH_LASLIB)
+
+message(DEPRECATION "This file UseLASLIB.cmake is deprecated, and the imported target `CGAL::TBB_support` from CGAL_LASLIB_support.cmake should be used instead.")
diff -pruN 5.0.3-1/cmake/modules/UseTBB.cmake 5.1-1/cmake/modules/UseTBB.cmake
--- 5.0.3-1/cmake/modules/UseTBB.cmake	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/cmake/modules/UseTBB.cmake	2020-09-08 12:12:19.000000000 +0000
@@ -5,4 +5,4 @@ include_directories ( ${TBB_INCLUDE_DIRS
 link_directories( ${TBB_LIBRARY_DIRS} )
 add_definitions( -DNOMINMAX -DCGAL_LINKED_WITH_TBB )
 
-message(DEPRECATION "This file UseTBB.cmake is deprecated, and the function `CGAL_target_use_TBB` from CGAL_target_use_TBB.cmake should be used instead.")
+message(DEPRECATION "This file UseTBB.cmake is deprecated, and the imported target `CGAL::TBB_support` from CGAL_TBB_support.cmake should be used instead.")
diff -pruN 5.0.3-1/CMakeLists.txt 5.1-1/CMakeLists.txt
--- 5.0.3-1/CMakeLists.txt	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/CMakeLists.txt	2020-09-08 12:12:19.000000000 +0000
@@ -382,8 +382,6 @@ include(${CGAL_MODULES_DIR}/CGAL_Macros.
 include(${CGAL_MODULES_DIR}/CGAL_enable_end_of_configuration_hook.cmake)
 cgal_setup_module_path()
 
-message( STATUS "CGAL_REFERENCE_CACHE_DIR=${CGAL_REFERENCE_CACHE_DIR}" )
-
 if ( RUNNING_CGAL_AUTO_TEST )
   message(STATUS "Operating system:")
   execute_process(COMMAND uname -a
@@ -392,41 +390,6 @@ if ( RUNNING_CGAL_AUTO_TEST )
     ERROR_VARIABLE uname_a)
   message(STATUS "${uname_a}")
   CGAL_display_compiler_version()
-  if ( NOT "${CGAL_REFERENCE_CACHE_DIR}" STREQUAL "" )
-    if ( EXISTS ${CGAL_REFERENCE_CACHE_DIR} )
-      if ( EXISTS ${CGAL_REFERENCE_CACHE_DIR}/CMakeCache.txt )
-        message( STATUS "Loading reference cache from ${CGAL_REFERENCE_CACHE_DIR}" )
-        load_cache( ${CGAL_REFERENCE_CACHE_DIR}
-          EXCLUDE CGAL_Core_LIBRARY
-                  CGAL_CORE_PACKAGE_DIR
-                  WITH_CGAL_Core
-                  CGAL_INSTALLATION_PACKAGE_DIR
-                  CGAL_MAINTENANCE_PACKAGE_DIR
-                  CGAL_PDB_BINARY_DIR
-                  CGAL_PDB_SOURCE_DIR
-                  CGAL_BINARY_DIR
-                  CGAL_SOURCE_DIR)
-#        message("List of cache variables:")
-
-        ## The following lines removes nasty loaded cache values. We do not
-        ## want that the current build tree depends on binaries that were
-        ## build in the reference build tree.
-        get_property(cache_variables DIRECTORY PROPERTY CACHE_VARIABLES)
-        foreach(var ${cache_variables})
-#          get_property(var_value CACHE ${var} PROPERTY VALUE)
-#          get_property(type CACHE ${var} PROPERTY TYPE)
-          string(REGEX MATCH "^CGAL(_.*_(DEPENDS|BINARY_DIR)|_.*LIBRARY)$" var_name_matches ${var})
-          if(var_name_matches)
-            unset(${var} CACHE)
-#          else()
-#            message("${var}:${var_type}=${var_value}")
-          endif()
-        endforeach()
-
-
-      endif()
-    endif()
-  endif()
 endif()
 
 include(CGAL_Common)
@@ -564,10 +527,6 @@ message("== Detect external libraries ==
 #  Coin is used in KDS, but no FindCoin or FindCOIN exists
 #  There exists FindIPE, FindMKL, but they are only used to support supporting libs
 list (INSERT CGAL_SUPPORTING_3RD_PARTY_LIBRARIES 0 GMP MPFR ZLIB OpenGL LEDA MPFI RS RS3 OpenNL Eigen3 ESBTL Coin3D NTL IPE)
-if (NOT WIN32)
-  # GMPXX is not supported on WIN32 machines
-  list (INSERT CGAL_SUPPORTING_3RD_PARTY_LIBRARIES 1 GMPXX)
-endif()
 
 # Where CMake is run several times, to avoid duplicates
 list(REMOVE_DUPLICATES CGAL_SUPPORTING_3RD_PARTY_LIBRARIES)
@@ -607,8 +566,6 @@ if(CGAL_DISABLE_GMP)
   unset(WITH_GMP CACHE)
   unset(CGAL_USE_GMP)
   unset(CGAL_USE_GMP CACHE)
-  unset(WITH_GMPXX)
-  unset(WITH_GMPXX CACHE)
   unset(WITH_MPFR)
   unset(WITH_MPFR CACHE)
 
@@ -617,9 +574,6 @@ if(CGAL_DISABLE_GMP)
     "#error GMP is disabled by the CMake option CGAL_DISABLE_GMP")
 else()
   list(APPEND      CGAL_ESSENTIAL_3RD_PARTY_LIBRARIES GMP MPFR)
-  if(WITH_GMPXX)
-    list(APPEND      CGAL_ESSENTIAL_3RD_PARTY_LIBRARIES GMPXX)
-  endif()
 
   # When CMake is run several times, to avoid duplicates
   list (REMOVE_DUPLICATES CGAL_ESSENTIAL_3RD_PARTY_LIBRARIES)
@@ -639,7 +593,7 @@ foreach (lib ${CGAL_SUPPORTING_3RD_PARTY
   # add option
   list( FIND CGAL_ESSENTIAL_3RD_PARTY_LIBRARIES "${lib}" POSITION )
   if ( "${POSITION}" STRGREATER "-1" ) # if lib is essential
-   option(WITH_${lib} "Select external library ${lib}" ON)
+    option(WITH_${lib} "Select external library ${lib}" ON)
   else()
     option(WITH_${lib} "Select external library ${lib}" OFF)
   endif()
@@ -925,6 +879,7 @@ if(NOT CGAL_HEADER_ONLY)
 else()
   install(FILES
     ${CMAKE_CURRENT_LIST_DIR}/lib/cmake/CGAL/CGALConfig.cmake
+    ${CMAKE_CURRENT_LIST_DIR}/lib/cmake/CGAL/CGALConfigBuildVersion.cmake
     ${CMAKE_CURRENT_LIST_DIR}/lib/cmake/CGAL/CGALConfigVersion.cmake
     DESTINATION ${CGAL_INSTALL_CMAKE_DIR})
   if(NOT CGAL_INSTALL_CMAKE_DIR STREQUAL "${CGAL_INSTALL_LIB_DIR}/cmake/CGAL")
@@ -1194,7 +1149,7 @@ You must disable CGAL_ENABLE_CHECK_HEADE
   file(GLOB html_files RELATIVE "${DOC_DIR}/doc_output/" "${DOC_DIR}/doc_output/*/*.html")
   file(GLOB example_files RELATIVE "${CMAKE_SOURCE_DIR}/" "${CMAKE_SOURCE_DIR}/*/examples/*/*.cpp")
   find_program(AWK awk)
-  set(awk_arguments [=[{ match($0, /# *include *(&lt;|[<"])(CGAL\/[^>&"]*)([>"]|&gt;)/,arr); if(arr[2]!="") print arr[2] }]=])
+  set(awk_arguments [=[{ match($0, /# *include *(&lt;|[<"])(CGAL\/[^>&"]*)([>"]|&gt;)| (CGAL\/[^>&"]*\.h)/,arr); if(arr[2]!="") print arr[2]; if(arr[4]!="") print arr[4] }]=])
   message("listing headers from html files")
   foreach(f ${html_files})
     execute_process(COMMAND "${AWK}" "${awk_arguments}" "${DOC_DIR}/doc_output/${f}"
diff -pruN 5.0.3-1/debian/changelog 5.1-1/debian/changelog
--- 5.0.3-1/debian/changelog	2020-08-03 15:57:06.000000000 +0000
+++ 5.1-1/debian/changelog	2020-09-08 20:26:11.000000000 +0000
@@ -1,6 +1,15 @@
+cgal (5.1-1) unstable; urgency=medium
+
+  * New upstream release.
+  * Remove patch fix-fpu-rounding-mode-control-mipsel-and-mips64el,
+    applied upstream.
+
+ -- Joachim Reichel <reichel@debian.org>  Tue, 08 Sep 2020 22:26:11 +0200
+
 cgal (5.0.3-1) unstable; urgency=medium
 
   * New upstream release.
+  * Adapt to renaming of /usr/share/doc/CGAL-x.y.z to /usr/share/doc/CGAL.
   * Add Suggests: libceres-dev to libcal-demo.
   * Remove duplicate Suggests: libglpk-dev for libcgal-demo.
 
diff -pruN 5.0.3-1/debian/patches/fix-fpu-rounding-mode-control-mipsel-and-mips64el 5.1-1/debian/patches/fix-fpu-rounding-mode-control-mipsel-and-mips64el
--- 5.0.3-1/debian/patches/fix-fpu-rounding-mode-control-mipsel-and-mips64el	2020-02-09 11:54:04.000000000 +0000
+++ 5.1-1/debian/patches/fix-fpu-rounding-mode-control-mipsel-and-mips64el	1970-01-01 00:00:00.000000000 +0000
@@ -1,13 +0,0 @@
-Index: cgal-5.0.1/include/CGAL/FPU.h
-===================================================================
---- cgal-5.0.1.orig/include/CGAL/FPU.h
-+++ cgal-5.0.1/include/CGAL/FPU.h
-@@ -408,7 +408,7 @@ typedef unsigned int FPU_CW_t;
- 
- #elif defined __mips__
- #define CGAL_IA_SETFPCW(CW) asm volatile ("ctc1 %0,$31" : :"r" (CW))
--#define CGAL_IA_GETFPCW(CW) asm volatile ("cfc1 %0,$31" : "=r" (CW))
-+#define CGAL_IA_GETFPCW(CW) asm volatile ("cfc1 %0,$31" : "=r" (CW)); CW &= 0x3
- typedef unsigned int FPU_CW_t;
- #define CGAL_FE_TONEAREST    (0x0)
- #define CGAL_FE_TOWARDZERO   (0x1)
diff -pruN 5.0.3-1/debian/patches/series 5.1-1/debian/patches/series
--- 5.0.3-1/debian/patches/series	2020-02-09 11:52:48.000000000 +0000
+++ 5.1-1/debian/patches/series	2020-09-08 20:26:11.000000000 +0000
@@ -1 +1 @@
-fix-fpu-rounding-mode-control-mipsel-and-mips64el
+# currently empty
diff -pruN 5.0.3-1/debian/README.source 5.1-1/debian/README.source
--- 5.0.3-1/debian/README.source	2014-10-16 19:39:59.000000000 +0000
+++ 5.1-1/debian/README.source	2020-09-08 20:26:11.000000000 +0000
@@ -7,5 +7,5 @@ applied during the build.
 
 See /usr/share/doc/quilt/README.source for a detailed explanation.
 
- -- Joachim Reichel <joachim.reichel@gmx.de>  Sun, 15 Jan 2012 14:38:04 +0100
+ -- Joachim Reichel <reichel@debian.org>  Sun, 15 Jan 2012 14:38:04 +0100
 
diff -pruN 5.0.3-1/demo/AABB_tree/benchmarks.cpp 5.1-1/demo/AABB_tree/benchmarks.cpp
--- 5.0.3-1/demo/AABB_tree/benchmarks.cpp	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/demo/AABB_tree/benchmarks.cpp	2020-09-08 12:12:18.000000000 +0000
@@ -77,7 +77,6 @@ void Scene::benchmark_distances(const do
     timer.start();
     std::cout << "Construct AABB tree and internal KD tree...";
     Facet_tree tree(faces(*m_pPolyhedron).first, faces(*m_pPolyhedron).second,*m_pPolyhedron);
-    tree.accelerate_distance_queries();
     std::cout << "done (" << timer.time() << " s)" << std::endl;
 
     // benchmark
@@ -123,7 +122,7 @@ void Scene::bench_memory()
         typedef CGAL::Memory_sizer::size_type size_type;
         size_type before = CGAL::Memory_sizer().virtual_size();
         Facet_tree tree(faces(*m_pPolyhedron).first, faces(*m_pPolyhedron).second,*m_pPolyhedron);
-        // tree.accelerate_distance_queries(); // 150 vs 61 bytes per primitive!
+        tree.do_not_accelerate_distance_queries(); // 150 vs 61 bytes per primitive!
 
         size_type after = CGAL::Memory_sizer().virtual_size();
         size_type bytes = after - before; // in Bytes
@@ -165,7 +164,6 @@ void Scene::bench_construction()
         CGAL::Timer time2;
         time2.start();
         Facet_tree tree2(faces(*m_pPolyhedron).first, faces(*m_pPolyhedron).second,*m_pPolyhedron);
-        tree2.accelerate_distance_queries();
         double duration_construction_and_kdtree = time2.time();
 
         std::cout << m_pPolyhedron->size_of_facets() << "\t"
@@ -248,7 +246,6 @@ void Scene::bench_distances_vs_nbt()
 
         // constructs tree (out of timing)
         Facet_tree tree(faces(*m_pPolyhedron).first, faces(*m_pPolyhedron).second, *m_pPolyhedron);
-        tree.accelerate_distance_queries();
 
         // calls queries
         CGAL::Timer timer;
diff -pruN 5.0.3-1/demo/AABB_tree/Scene.cpp 5.1-1/demo/AABB_tree/Scene.cpp
--- 5.0.3-1/demo/AABB_tree/Scene.cpp	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/demo/AABB_tree/Scene.cpp	2020-09-08 12:12:18.000000000 +0000
@@ -45,8 +45,7 @@ Scene::Scene()
     m_blue_ramp.build_blue();
     m_max_distance_function = (FT)0.0;
     texture = new Texture(m_grid_size,m_grid_size);
-    startTimer(0);
-    ready_to_cut = false;
+    ready_to_cut = true;
     are_buffers_initialized = false;
     gl_init = false;
 
@@ -519,6 +518,7 @@ void Scene::changed()
         compute_elements(_UNSIGNED);
     else
         compute_elements(_SIGNED);
+    ready_to_cut=false;
     are_buffers_initialized = false;
 
 }
@@ -806,7 +806,6 @@ void Scene::build_facet_tree()
     timer.start();
     std::cout << "Construct Facet AABB tree...";
     m_facet_tree.rebuild(faces(*m_pPolyhedron).first, faces(*m_pPolyhedron).second,*m_pPolyhedron);
-    m_facet_tree.accelerate_distance_queries();
     std::cout << "done (" << timer.time() << " s)" << std::endl;
 }
 
@@ -826,7 +825,6 @@ void Scene::build_edge_tree()
     timer.start();
     std::cout << "Construct Edge AABB tree...";
     m_edge_tree.rebuild(edges(*m_pPolyhedron).first,edges(*m_pPolyhedron).second,*m_pPolyhedron);
-    m_edge_tree.accelerate_distance_queries();
     std::cout << "done (" << timer.time() << " s)" << std::endl;
 }
 
@@ -1228,12 +1226,16 @@ void Scene::cut_segment_plane()
     m_cut_plane = CUT_SEGMENTS;
     changed();
 }
+void Scene::updateCutPlane()
+{
+  ready_to_cut = true;
+       QTimer::singleShot(0,this,SLOT(cutting_plane()));
+}
 
 void Scene::cutting_plane(bool override)
 {
     if(ready_to_cut || override)
     {
-        ready_to_cut = false;
         switch( m_cut_plane )
         {
         case UNSIGNED_FACETS:
@@ -1306,13 +1308,13 @@ void Scene::refine_loop()
 
 void Scene::activate_cutting_plane()
 {
-    connect(m_frame, SIGNAL(modified()), this, SLOT(cutting_plane()));
+    connect(m_frame, SIGNAL(modified()), this, SLOT(updateCutPlane()));
     m_view_plane = true;
 }
 
 void Scene::deactivate_cutting_plane()
 {
-    disconnect(m_frame, SIGNAL(modified()), this, SLOT(cutting_plane()));
+    disconnect(m_frame, SIGNAL(modified()), this, SLOT(updateCutPlane()));
     m_view_plane = false;
 }
 void Scene::initGL()
@@ -1328,10 +1330,3 @@ void Scene::initGL()
     compile_shaders();
     gl_init = true;
 }
-
-void Scene::timerEvent(QTimerEvent *)
-{
-    if(manipulatedFrame()->isSpinning())
-        set_fast_distance(true);
-    ready_to_cut = true;
-}
diff -pruN 5.0.3-1/demo/AABB_tree/Scene.h 5.1-1/demo/AABB_tree/Scene.h
--- 5.0.3-1/demo/AABB_tree/Scene.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/demo/AABB_tree/Scene.h	2020-09-08 12:14:29.000000000 +0000
@@ -173,6 +173,8 @@ private:
     void attrib_buffers(CGAL::QGLViewer*);
     void compile_shaders();
     void compute_texture(int, int, Color_ramp, Color_ramp);
+private slots:
+    void updateCutPlane();
 
 public:
     // file menu
@@ -251,8 +253,6 @@ public:
     void activate_cutting_plane();
     void deactivate_cutting_plane();
 
-    //timer sends a top when all the events are finished
-    void timerEvent(QTimerEvent *);
 
 
 public slots:
diff -pruN 5.0.3-1/demo/Alpha_shapes_2/Alpha_shapes_2.cpp 5.1-1/demo/Alpha_shapes_2/Alpha_shapes_2.cpp
--- 5.0.3-1/demo/Alpha_shapes_2/Alpha_shapes_2.cpp	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/demo/Alpha_shapes_2/Alpha_shapes_2.cpp	2020-09-08 12:12:19.000000000 +0000
@@ -138,7 +138,7 @@ MainWindow::MainWindow()
   this->graphicsView->setMouseTracking(true);
 
   // Turn the vertical axis upside down
-  this->graphicsView->matrix().scale(1, -1);
+  this->graphicsView->transform().scale(1, -1);
 
   // The navigation adds zooming and translation functionality to the
   // QGraphicsView
diff -pruN 5.0.3-1/demo/Apollonius_graph_2/Apollonius_graph_2.cpp 5.1-1/demo/Apollonius_graph_2/Apollonius_graph_2.cpp
--- 5.0.3-1/demo/Apollonius_graph_2/Apollonius_graph_2.cpp	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/demo/Apollonius_graph_2/Apollonius_graph_2.cpp	2020-09-08 12:12:19.000000000 +0000
@@ -125,7 +125,7 @@ MainWindow::MainWindow()
   this->graphicsView->setMouseTracking(true);
 
   // Turn the vertical axis upside down
-  this->graphicsView->matrix().scale(1, -1);
+  this->graphicsView->transform().scale(1, -1);
 
   // The navigation adds zooming and translation functionality to the
   // QGraphicsView
diff -pruN 5.0.3-1/demo/Arrangement_on_surface_2/ArrangementCurveInputCallback.h 5.1-1/demo/Arrangement_on_surface_2/ArrangementCurveInputCallback.h
--- 5.0.3-1/demo/Arrangement_on_surface_2/ArrangementCurveInputCallback.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/demo/Arrangement_on_surface_2/ArrangementCurveInputCallback.h	2020-09-08 12:14:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/ArrangementCurveInputCallback.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/ArrangementCurveInputCallback.h $
 // $Id: ArrangementCurveInputCallback.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Arrangement_on_surface_2/ArrangementDemoGraphicsView.cpp 5.1-1/demo/Arrangement_on_surface_2/ArrangementDemoGraphicsView.cpp
--- 5.0.3-1/demo/Arrangement_on_surface_2/ArrangementDemoGraphicsView.cpp	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/demo/Arrangement_on_surface_2/ArrangementDemoGraphicsView.cpp	2020-09-08 12:12:18.000000000 +0000
@@ -22,8 +22,8 @@ ArrangementDemoGraphicsView::Arrangement
   gridColor( ::Qt::black ),
   backgroundColor( ::Qt::white )
 {
-  QMatrix m( 1.0, 0.0, 0.0, -1.0, 0.0, 0.0 );
-  this->setMatrix( m );
+  QTransform m( 1.0, 0.0, 0.0, -1.0, 0.0, 0.0 );
+  this->setTransform( m );
   this->setBackgroundBrush( QBrush( backgroundColor ) );
 }
 
diff -pruN 5.0.3-1/demo/Arrangement_on_surface_2/ArrangementDemoGraphicsView.h 5.1-1/demo/Arrangement_on_surface_2/ArrangementDemoGraphicsView.h
--- 5.0.3-1/demo/Arrangement_on_surface_2/ArrangementDemoGraphicsView.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/demo/Arrangement_on_surface_2/ArrangementDemoGraphicsView.h	2020-09-08 12:14:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/ArrangementDemoGraphicsView.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/ArrangementDemoGraphicsView.h $
 // $Id: ArrangementDemoGraphicsView.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Arrangement_on_surface_2/ArrangementDemoPropertiesDialog.h 5.1-1/demo/Arrangement_on_surface_2/ArrangementDemoPropertiesDialog.h
--- 5.0.3-1/demo/Arrangement_on_surface_2/ArrangementDemoPropertiesDialog.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/demo/Arrangement_on_surface_2/ArrangementDemoPropertiesDialog.h	2020-09-08 12:14:34.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/ArrangementDemoPropertiesDialog.h $
-// $Id: ArrangementDemoPropertiesDialog.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/ArrangementDemoPropertiesDialog.h $
+// $Id: ArrangementDemoPropertiesDialog.h 3003fb3 2020-06-18T13:35:40+02:00 Maxime Gimeno
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Alex Tsui <alextsui05@gmail.com>
@@ -43,7 +43,7 @@ class ArrangementDemoPropertiesDialog :
   };
 
   ArrangementDemoPropertiesDialog( ArrangementDemoWindow* parent_ = 0,
-                                   Qt::WindowFlags f = 0 );
+                                   Qt::WindowFlags f  = Qt::WindowType(0));
   QVariant property( int index );
 
 protected:
diff -pruN 5.0.3-1/demo/Arrangement_on_surface_2/ArrangementDemoTab.h 5.1-1/demo/Arrangement_on_surface_2/ArrangementDemoTab.h
--- 5.0.3-1/demo/Arrangement_on_surface_2/ArrangementDemoTab.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/demo/Arrangement_on_surface_2/ArrangementDemoTab.h	2020-09-08 12:14:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/ArrangementDemoTab.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/ArrangementDemoTab.h $
 // $Id: ArrangementDemoTab.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Arrangement_on_surface_2/ArrangementSegmentInputCallback.h 5.1-1/demo/Arrangement_on_surface_2/ArrangementSegmentInputCallback.h
--- 5.0.3-1/demo/Arrangement_on_surface_2/ArrangementSegmentInputCallback.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/demo/Arrangement_on_surface_2/ArrangementSegmentInputCallback.h	2020-09-08 12:14:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/ArrangementSegmentInputCallback.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/ArrangementSegmentInputCallback.h $
 // $Id: ArrangementSegmentInputCallback.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Arrangement_on_surface_2/Callback.h 5.1-1/demo/Arrangement_on_surface_2/Callback.h
--- 5.0.3-1/demo/Arrangement_on_surface_2/Callback.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/demo/Arrangement_on_surface_2/Callback.h	2020-09-08 12:14:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/Callback.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/Callback.h $
 // $Id: Callback.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Arrangement_on_surface_2/Conic_reader.h 5.1-1/demo/Arrangement_on_surface_2/Conic_reader.h
--- 5.0.3-1/demo/Arrangement_on_surface_2/Conic_reader.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/demo/Arrangement_on_surface_2/Conic_reader.h	2020-09-08 12:14:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/Conic_reader.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/Conic_reader.h $
 // $Id: Conic_reader.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Arrangement_on_surface_2/CurveGraphicsItem.h 5.1-1/demo/Arrangement_on_surface_2/CurveGraphicsItem.h
--- 5.0.3-1/demo/Arrangement_on_surface_2/CurveGraphicsItem.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/demo/Arrangement_on_surface_2/CurveGraphicsItem.h	2020-09-08 12:14:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/CurveGraphicsItem.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/CurveGraphicsItem.h $
 // $Id: CurveGraphicsItem.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Arrangement_on_surface_2/DeleteCurveCallback.h 5.1-1/demo/Arrangement_on_surface_2/DeleteCurveCallback.h
--- 5.0.3-1/demo/Arrangement_on_surface_2/DeleteCurveCallback.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/demo/Arrangement_on_surface_2/DeleteCurveCallback.h	2020-09-08 12:14:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/DeleteCurveCallback.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/DeleteCurveCallback.h $
 // $Id: DeleteCurveCallback.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Arrangement_on_surface_2/DeleteCurveMode.h 5.1-1/demo/Arrangement_on_surface_2/DeleteCurveMode.h
--- 5.0.3-1/demo/Arrangement_on_surface_2/DeleteCurveMode.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/demo/Arrangement_on_surface_2/DeleteCurveMode.h	2020-09-08 12:14:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/DeleteCurveMode.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/DeleteCurveMode.h $
 // $Id: DeleteCurveMode.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Arrangement_on_surface_2/DeleteCurveModeItemEditor.h 5.1-1/demo/Arrangement_on_surface_2/DeleteCurveModeItemEditor.h
--- 5.0.3-1/demo/Arrangement_on_surface_2/DeleteCurveModeItemEditor.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/demo/Arrangement_on_surface_2/DeleteCurveModeItemEditor.h	2020-09-08 12:14:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/DeleteCurveModeItemEditor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/DeleteCurveModeItemEditor.h $
 // $Id: DeleteCurveModeItemEditor.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Arrangement_on_surface_2/EnvelopeCallback.h 5.1-1/demo/Arrangement_on_surface_2/EnvelopeCallback.h
--- 5.0.3-1/demo/Arrangement_on_surface_2/EnvelopeCallback.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/demo/Arrangement_on_surface_2/EnvelopeCallback.h	2020-09-08 12:14:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/EnvelopeCallback.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/EnvelopeCallback.h $
 // $Id: EnvelopeCallback.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Arrangement_on_surface_2/FillFaceCallback.h 5.1-1/demo/Arrangement_on_surface_2/FillFaceCallback.h
--- 5.0.3-1/demo/Arrangement_on_surface_2/FillFaceCallback.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/demo/Arrangement_on_surface_2/FillFaceCallback.h	2020-09-08 12:14:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/FillFaceCallback.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/FillFaceCallback.h $
 // $Id: FillFaceCallback.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Arrangement_on_surface_2/GraphicsViewCurveInput.h 5.1-1/demo/Arrangement_on_surface_2/GraphicsViewCurveInput.h
--- 5.0.3-1/demo/Arrangement_on_surface_2/GraphicsViewCurveInput.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/demo/Arrangement_on_surface_2/GraphicsViewCurveInput.h	2020-09-08 12:14:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/GraphicsViewCurveInput.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/GraphicsViewCurveInput.h $
 // $Id: GraphicsViewCurveInput.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Arrangement_on_surface_2/GraphicsViewSegmentInput.h 5.1-1/demo/Arrangement_on_surface_2/GraphicsViewSegmentInput.h
--- 5.0.3-1/demo/Arrangement_on_surface_2/GraphicsViewSegmentInput.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/demo/Arrangement_on_surface_2/GraphicsViewSegmentInput.h	2020-09-08 12:14:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/GraphicsViewSegmentInput.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/GraphicsViewSegmentInput.h $
 // $Id: GraphicsViewSegmentInput.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Arrangement_on_surface_2/ISnappable.h 5.1-1/demo/Arrangement_on_surface_2/ISnappable.h
--- 5.0.3-1/demo/Arrangement_on_surface_2/ISnappable.h	2020-08-01 19:02:11.000000000 +0000
+++ 5.1-1/demo/Arrangement_on_surface_2/ISnappable.h	2020-09-08 12:14:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/ISnappable.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/ISnappable.h $
 // $Id: ISnappable.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Arrangement_on_surface_2/MergeEdgeCallback.h 5.1-1/demo/Arrangement_on_surface_2/MergeEdgeCallback.h
--- 5.0.3-1/demo/Arrangement_on_surface_2/MergeEdgeCallback.h	2020-08-01 19:02:11.000000000 +0000
+++ 5.1-1/demo/Arrangement_on_surface_2/MergeEdgeCallback.h	2020-09-08 12:14:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/MergeEdgeCallback.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/MergeEdgeCallback.h $
 // $Id: MergeEdgeCallback.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Arrangement_on_surface_2/NewTabDialog.cpp 5.1-1/demo/Arrangement_on_surface_2/NewTabDialog.cpp
--- 5.0.3-1/demo/Arrangement_on_surface_2/NewTabDialog.cpp	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/demo/Arrangement_on_surface_2/NewTabDialog.cpp	2020-09-08 12:12:18.000000000 +0000
@@ -14,7 +14,7 @@
 #include "ui_NewTabDialog.h"
 #include <QButtonGroup>
 
-NewTabDialog::NewTabDialog( QWidget* parent, Qt::WindowFlags f ) :
+NewTabDialog::NewTabDialog( QWidget* parent, Qt::WindowFlags f) :
   QDialog( parent, f ),
   ui( new Ui::NewTabDialog ),
   buttonGroup( new QButtonGroup )
diff -pruN 5.0.3-1/demo/Arrangement_on_surface_2/NewTabDialog.h 5.1-1/demo/Arrangement_on_surface_2/NewTabDialog.h
--- 5.0.3-1/demo/Arrangement_on_surface_2/NewTabDialog.h	2020-08-01 19:02:11.000000000 +0000
+++ 5.1-1/demo/Arrangement_on_surface_2/NewTabDialog.h	2020-09-08 12:14:34.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/NewTabDialog.h $
-// $Id: NewTabDialog.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/NewTabDialog.h $
+// $Id: NewTabDialog.h 5976ee9 2020-06-16T09:06:34+02:00 Maxime Gimeno
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Alex Tsui <alextsui05@gmail.com>
@@ -23,7 +23,7 @@ namespace Ui
 class NewTabDialog : public QDialog
 {
 public:
-  NewTabDialog( QWidget* parent = 0, Qt::WindowFlags f = 0 );
+  NewTabDialog( QWidget* parent = 0, Qt::WindowFlags f = Qt::WindowType(0)  );
   int checkedId( ) const;
 
 protected:
diff -pruN 5.0.3-1/demo/Arrangement_on_surface_2/OverlayDialog.h 5.1-1/demo/Arrangement_on_surface_2/OverlayDialog.h
--- 5.0.3-1/demo/Arrangement_on_surface_2/OverlayDialog.h	2020-08-01 19:02:11.000000000 +0000
+++ 5.1-1/demo/Arrangement_on_surface_2/OverlayDialog.h	2020-09-08 12:14:34.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/OverlayDialog.h $
-// $Id: OverlayDialog.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/OverlayDialog.h $
+// $Id: OverlayDialog.h 3003fb3 2020-06-18T13:35:40+02:00 Maxime Gimeno
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Alex Tsui <alextsui05@gmail.com>
@@ -29,7 +29,7 @@ class OverlayDialog : public QDialog
     ARRANGEMENT = 32
   } OverlayDialogRole;
 
-  OverlayDialog( ArrangementDemoWindow* parent, Qt::WindowFlags f = 0 );
+  OverlayDialog( ArrangementDemoWindow* parent, Qt::WindowFlags f = Qt::WindowType(0)  );
 
   std::vector< CGAL::Object > selectedArrangements( ) const;
 
diff -pruN 5.0.3-1/demo/Arrangement_on_surface_2/PointLocationCallback.h 5.1-1/demo/Arrangement_on_surface_2/PointLocationCallback.h
--- 5.0.3-1/demo/Arrangement_on_surface_2/PointLocationCallback.h	2020-08-01 19:02:11.000000000 +0000
+++ 5.1-1/demo/Arrangement_on_surface_2/PointLocationCallback.h	2020-09-08 12:14:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/PointLocationCallback.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/PointLocationCallback.h $
 // $Id: PointLocationCallback.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Arrangement_on_surface_2/PointsGraphicsItem.h 5.1-1/demo/Arrangement_on_surface_2/PointsGraphicsItem.h
--- 5.0.3-1/demo/Arrangement_on_surface_2/PointsGraphicsItem.h	2020-08-01 19:02:11.000000000 +0000
+++ 5.1-1/demo/Arrangement_on_surface_2/PointsGraphicsItem.h	2020-09-08 12:14:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/PointsGraphicsItem.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/PointsGraphicsItem.h $
 // $Id: PointsGraphicsItem.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Arrangement_on_surface_2/PropertyValueDelegate.h 5.1-1/demo/Arrangement_on_surface_2/PropertyValueDelegate.h
--- 5.0.3-1/demo/Arrangement_on_surface_2/PropertyValueDelegate.h	2020-08-01 19:02:11.000000000 +0000
+++ 5.1-1/demo/Arrangement_on_surface_2/PropertyValueDelegate.h	2020-09-08 12:14:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/PropertyValueDelegate.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/PropertyValueDelegate.h $
 // $Id: PropertyValueDelegate.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Arrangement_on_surface_2/SplitEdgeCallback.h 5.1-1/demo/Arrangement_on_surface_2/SplitEdgeCallback.h
--- 5.0.3-1/demo/Arrangement_on_surface_2/SplitEdgeCallback.h	2020-08-01 19:02:11.000000000 +0000
+++ 5.1-1/demo/Arrangement_on_surface_2/SplitEdgeCallback.h	2020-09-08 12:14:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/SplitEdgeCallback.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/SplitEdgeCallback.h $
 // $Id: SplitEdgeCallback.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Arrangement_on_surface_2/VerticalRayGraphicsItem.h 5.1-1/demo/Arrangement_on_surface_2/VerticalRayGraphicsItem.h
--- 5.0.3-1/demo/Arrangement_on_surface_2/VerticalRayGraphicsItem.h	2020-08-01 19:02:11.000000000 +0000
+++ 5.1-1/demo/Arrangement_on_surface_2/VerticalRayGraphicsItem.h	2020-09-08 12:14:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/VerticalRayGraphicsItem.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/VerticalRayGraphicsItem.h $
 // $Id: VerticalRayGraphicsItem.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Arrangement_on_surface_2/VerticalRayShootCallback.h 5.1-1/demo/Arrangement_on_surface_2/VerticalRayShootCallback.h
--- 5.0.3-1/demo/Arrangement_on_surface_2/VerticalRayShootCallback.h	2020-08-01 19:02:11.000000000 +0000
+++ 5.1-1/demo/Arrangement_on_surface_2/VerticalRayShootCallback.h	2020-09-08 12:14:34.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/VerticalRayShootCallback.h $
-// $Id: VerticalRayShootCallback.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/demo/Arrangement_on_surface_2/VerticalRayShootCallback.h $
+// $Id: VerticalRayShootCallback.h ea6f94a 2020-04-23T22:54:46+03:00 Efi Fogel
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Alex Tsui <alextsui05@gmail.com>
@@ -127,7 +127,11 @@ protected:
   void highlightPointLocation( QGraphicsSceneMouseEvent *event );
   Face_const_handle getFace( const CGAL::Object& o );
   CGAL::Object rayShootUp( const Kernel_point_2& point );
+  CGAL::Object rayShootUp( const Kernel_point_2& point, CGAL::Tag_true );
+  CGAL::Object rayShootUp( const Kernel_point_2& point, CGAL::Tag_false );
   CGAL::Object rayShootDown( const Kernel_point_2& point );
+  CGAL::Object rayShootDown( const Kernel_point_2& point, CGAL::Tag_true );
+  CGAL::Object rayShootDown( const Kernel_point_2& point, CGAL::Tag_false );
 
   using Superclass::scene;
   using Superclass::shootingUp;
@@ -305,6 +309,14 @@ template < typename Arr_ >
 CGAL::Object
 VerticalRayShootCallback< Arr_ >::rayShootUp( const Kernel_point_2& pt )
 {
+  typename Supports_landmarks< Arrangement >::Tag supportsLandmarks;
+  return this->rayShootUp( pt, supportsLandmarks );
+}
+
+template < typename Arr_ >
+CGAL::Object VerticalRayShootCallback< Arr_ >::rayShootUp( const Kernel_point_2& pt,
+                                                           CGAL::Tag_true )
+{
   CGAL::Object pointLocationResult;
   Walk_pl_strategy* walkStrategy;
   TrapezoidPointLocationStrategy* trapezoidStrategy;
@@ -335,9 +347,48 @@ VerticalRayShootCallback< Arr_ >::raySho
 }
 
 template < typename Arr_ >
+CGAL::Object VerticalRayShootCallback< Arr_ >::rayShootUp( const Kernel_point_2& pt,
+                                                           CGAL::Tag_false )
+{
+  CGAL::Object pointLocationResult;
+  Walk_pl_strategy* walkStrategy;
+  TrapezoidPointLocationStrategy* trapezoidStrategy;
+  SimplePointLocationStrategy* simpleStrategy;
+
+  Point_2 point = this->toArrPoint( pt );
+
+  if ( CGAL::assign( walkStrategy, this->pointLocationStrategy ) )
+  {
+    pointLocationResult = walkStrategy->ray_shoot_up( point );
+  }
+  else if ( CGAL::assign( trapezoidStrategy, this->pointLocationStrategy ) )
+  {
+    pointLocationResult = trapezoidStrategy->ray_shoot_up( point );
+  }
+  else if ( CGAL::assign( simpleStrategy, this->pointLocationStrategy ) )
+  {
+    pointLocationResult = simpleStrategy->ray_shoot_up( point );
+  }
+  else
+  {
+    std::cout<<"Didn't find the right strategy\n";
+  }
+
+  return pointLocationResult;
+}
+
+template < typename Arr_ >
 CGAL::Object
 VerticalRayShootCallback< Arr_ >::rayShootDown( const Kernel_point_2& pt )
 {
+  typename Supports_landmarks< Arrangement >::Tag supportsLandmarks;
+  return this->rayShootDown( pt, supportsLandmarks );
+}
+
+template < typename Arr_ >
+CGAL::Object
+VerticalRayShootCallback< Arr_ >::rayShootDown( const Kernel_point_2& pt, CGAL::Tag_true )
+{
   CGAL::Object pointLocationResult;
   Walk_pl_strategy* walkStrategy;
   TrapezoidPointLocationStrategy* trapezoidStrategy;
@@ -366,5 +417,31 @@ VerticalRayShootCallback< Arr_ >::raySho
   }
   return pointLocationResult;
 }
+
+template < typename Arr_ >
+CGAL::Object
+VerticalRayShootCallback< Arr_ >::rayShootDown( const Kernel_point_2& pt, CGAL::Tag_false )
+{
+  CGAL::Object pointLocationResult;
+  Walk_pl_strategy* walkStrategy;
+  TrapezoidPointLocationStrategy* trapezoidStrategy;
+  SimplePointLocationStrategy* simpleStrategy;
+
+  Point_2 point = this->toArrPoint( pt );
+
+  if ( CGAL::assign( walkStrategy, this->pointLocationStrategy ) )
+  {
+    pointLocationResult = walkStrategy->ray_shoot_down( point );
+  }
+  else if ( CGAL::assign( trapezoidStrategy, this->pointLocationStrategy ) )
+  {
+    pointLocationResult = trapezoidStrategy->ray_shoot_down( point );
+  }
+  else if ( CGAL::assign( simpleStrategy, this->pointLocationStrategy ) )
+  {
+    pointLocationResult = simpleStrategy->ray_shoot_down( point );
+  }
+  return pointLocationResult;
+}
 
 #endif // VERTICAL_RAY_SHOOT_CALLBACK_H
diff -pruN 5.0.3-1/demo/Bounding_volumes/Bounding_volumes.cpp 5.1-1/demo/Bounding_volumes/Bounding_volumes.cpp
--- 5.0.3-1/demo/Bounding_volumes/Bounding_volumes.cpp	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/demo/Bounding_volumes/Bounding_volumes.cpp	2020-09-08 12:12:19.000000000 +0000
@@ -208,7 +208,7 @@ MainWindow::MainWindow()
   this->graphicsView->setMouseTracking(true);
 
   // Turn the vertical axis upside down
-  this->graphicsView->matrix().scale(1, -1);
+  this->graphicsView->transform().scale(1, -1);
 
   // The navigation adds zooming and translation functionality to the
   // QGraphicsView
diff -pruN 5.0.3-1/demo/CGAL_ipelets/CMakeLists.txt 5.1-1/demo/CGAL_ipelets/CMakeLists.txt
--- 5.0.3-1/demo/CGAL_ipelets/CMakeLists.txt	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/demo/CGAL_ipelets/CMakeLists.txt	2020-09-08 12:12:19.000000000 +0000
@@ -27,9 +27,8 @@ if ( CGAL_FOUND )
   include(${CGAL_USE_FILE})
 
   find_package(Eigen3 3.1.0) #(requires 3.1.0 or greater)
-  if (EIGEN3_FOUND)
-    include( ${EIGEN3_USE_FILE} )
-  else()
+  include(CGAL_Eigen_support)
+  if (NOT TARGET CGAL::Eigen_support)
     message(STATUS "NOTICE: This project requires the Eigen library, and will not be compiled.")
     return()
   endif()
@@ -142,7 +141,7 @@ if ( CGAL_FOUND )
     foreach(IPELET ${CGAL_IPELETS})
       add_library(CGAL_${IPELET} MODULE ${IPELET}.cpp)
       add_to_cached_list(CGAL_EXECUTABLE_TARGETS CGAL_${IPELET})
-      target_link_libraries(CGAL_${IPELET} PRIVATE CGAL::CGAL ${IPE_LIBRARIES})
+      target_link_libraries(CGAL_${IPELET} PRIVATE CGAL::CGAL CGAL::Eigen_support ${IPE_LIBRARIES})
       if ( IPELET_INSTALL_DIR )
         install(TARGETS CGAL_${IPELET} DESTINATION ${IPELET_INSTALL_DIR})
         if (WITH_IPE_7)
@@ -152,13 +151,12 @@ if ( CGAL_FOUND )
       cgal_add_compilation_test(CGAL_${IPELET})
     endforeach(IPELET)
     if(CGAL_Core_FOUND)
-      target_link_libraries(CGAL_cone_spanners PRIVATE CGAL::CGAL_Core)
+      target_link_libraries(CGAL_cone_spanners PRIVATE CGAL::CGAL_Core CGAL::Eigen_support)
     endif()
     #example in doc not installed
     add_library(simple_triangulation MODULE simple_triangulation.cpp)
     add_to_cached_list(CGAL_EXECUTABLE_TARGETS simple_triangulation)
-    target_link_libraries(simple_triangulation ${IPE_LIBRARIES})
-
+    target_link_libraries(simple_triangulation CGAL::Eigen_support ${IPE_LIBRARIES})
     cgal_add_compilation_test(simple_triangulation)
 
   else()
diff -pruN 5.0.3-1/demo/CGAL_ipelets/cone_spanners.cpp 5.1-1/demo/CGAL_ipelets/cone_spanners.cpp
--- 5.0.3-1/demo/CGAL_ipelets/cone_spanners.cpp	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/demo/CGAL_ipelets/cone_spanners.cpp	2020-09-08 12:12:19.000000000 +0000
@@ -60,7 +60,7 @@ private:
 void Cone_spanners_ipelet::protected_run(int fn)
 {
   std::vector<Point_2> lst;
-  int number_of_cones;
+  int number_of_cones = 0;
   switch (fn){
     case 0:
     case 1:
diff -pruN 5.0.3-1/demo/CGAL_ipelets/include/CGAL_ipelets/k_delaunay.h 5.1-1/demo/CGAL_ipelets/include/CGAL_ipelets/k_delaunay.h
--- 5.0.3-1/demo/CGAL_ipelets/include/CGAL_ipelets/k_delaunay.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/demo/CGAL_ipelets/include/CGAL_ipelets/k_delaunay.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ipelets/demo/CGAL_ipelets/include/CGAL_ipelets/k_delaunay.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ipelets/demo/CGAL_ipelets/include/CGAL_ipelets/k_delaunay.h $
 // $Id: k_delaunay.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/CGAL_ipelets/include/CGAL_ipelets/pencils.h 5.1-1/demo/CGAL_ipelets/include/CGAL_ipelets/pencils.h
--- 5.0.3-1/demo/CGAL_ipelets/include/CGAL_ipelets/pencils.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/demo/CGAL_ipelets/include/CGAL_ipelets/pencils.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ipelets/demo/CGAL_ipelets/include/CGAL_ipelets/pencils.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ipelets/demo/CGAL_ipelets/include/CGAL_ipelets/pencils.h $
 // $Id: pencils.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Hyperbolic_triangulation_2/CMakeLists.txt 5.1-1/demo/Hyperbolic_triangulation_2/CMakeLists.txt
--- 5.0.3-1/demo/Hyperbolic_triangulation_2/CMakeLists.txt	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/demo/Hyperbolic_triangulation_2/CMakeLists.txt	2020-09-08 12:12:19.000000000 +0000
@@ -14,6 +14,9 @@ endif()
 find_package(CGAL REQUIRED QUIET OPTIONAL_COMPONENTS Core Qt5)
 find_package(LEDA QUIET)
 
+# Find Qt5 itself
+find_package(Qt5 QUIET COMPONENTS OpenGL Gui)
+
 if(CGAL_FOUND AND CGAL_Qt5_FOUND AND Qt5_FOUND AND (CGAL_Core_FOUND OR LEDA_FOUND))
   include_directories( BEFORE ./ ./include )
   # ui files, created with Qt Designer
diff -pruN 5.0.3-1/demo/Hyperbolic_triangulation_2/HDT2.cpp 5.1-1/demo/Hyperbolic_triangulation_2/HDT2.cpp
--- 5.0.3-1/demo/Hyperbolic_triangulation_2/HDT2.cpp	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/demo/Hyperbolic_triangulation_2/HDT2.cpp	2020-09-08 12:12:19.000000000 +0000
@@ -207,7 +207,7 @@ MainWindow::MainWindow()
   this->graphicsView->shear(230, 230);
 
   // Turn the vertical axis upside down
-  this->graphicsView->matrix().scale(1, -1);
+  this->graphicsView->transform().scale(1, -1);
 
   // The navigation adds zooming and translation functionality to the
   // QGraphicsView
diff -pruN 5.0.3-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/HyperbolicPainterOstream.h 5.1-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/HyperbolicPainterOstream.h
--- 5.0.3-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/HyperbolicPainterOstream.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/HyperbolicPainterOstream.h	2020-09-08 12:14:54.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Hyperbolic_triangulation_2/demo/Hyperbolic_triangulation_2/include/internal/Qt/HyperbolicPainterOstream.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Hyperbolic_triangulation_2/demo/Hyperbolic_triangulation_2/include/internal/Qt/HyperbolicPainterOstream.h $
 // $Id: HyperbolicPainterOstream.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/HyperbolicVoronoiGraphicsItem.h 5.1-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/HyperbolicVoronoiGraphicsItem.h
--- 5.0.3-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/HyperbolicVoronoiGraphicsItem.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/HyperbolicVoronoiGraphicsItem.h	2020-09-08 12:14:54.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Hyperbolic_triangulation_2/demo/Hyperbolic_triangulation_2/include/internal/Qt/HyperbolicVoronoiGraphicsItem.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Hyperbolic_triangulation_2/demo/Hyperbolic_triangulation_2/include/internal/Qt/HyperbolicVoronoiGraphicsItem.h $
 // $Id: HyperbolicVoronoiGraphicsItem.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationCircumcircle.h 5.1-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationCircumcircle.h
--- 5.0.3-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationCircumcircle.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationCircumcircle.h	2020-09-08 12:14:54.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Hyperbolic_triangulation_2/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationCircumcircle.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Hyperbolic_triangulation_2/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationCircumcircle.h $
 // $Id: TriangulationCircumcircle.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationConflictZone.h 5.1-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationConflictZone.h
--- 5.0.3-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationConflictZone.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationConflictZone.h	2020-09-08 12:14:54.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Hyperbolic_triangulation_2/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationConflictZone.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Hyperbolic_triangulation_2/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationConflictZone.h $
 // $Id: TriangulationConflictZone.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationGraphicsItem.h 5.1-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationGraphicsItem.h
--- 5.0.3-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationGraphicsItem.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationGraphicsItem.h	2020-09-08 12:14:54.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Hyperbolic_triangulation_2/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationGraphicsItem.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Hyperbolic_triangulation_2/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationGraphicsItem.h $
 // $Id: TriangulationGraphicsItem.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationGraphicsItemWithColorInfo.h 5.1-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationGraphicsItemWithColorInfo.h
--- 5.0.3-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationGraphicsItemWithColorInfo.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationGraphicsItemWithColorInfo.h	2020-09-08 12:14:54.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Hyperbolic_triangulation_2/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationGraphicsItemWithColorInfo.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Hyperbolic_triangulation_2/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationGraphicsItemWithColorInfo.h $
 // $Id: TriangulationGraphicsItemWithColorInfo.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationMovingPoint.h 5.1-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationMovingPoint.h
--- 5.0.3-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationMovingPoint.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationMovingPoint.h	2020-09-08 12:14:54.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Hyperbolic_triangulation_2/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationMovingPoint.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Hyperbolic_triangulation_2/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationMovingPoint.h $
 // $Id: TriangulationMovingPoint.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationPointInputAndConflictZone.h 5.1-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationPointInputAndConflictZone.h
--- 5.0.3-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationPointInputAndConflictZone.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationPointInputAndConflictZone.h	2020-09-08 12:14:54.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Hyperbolic_triangulation_2/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationPointInputAndConflictZone.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Hyperbolic_triangulation_2/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationPointInputAndConflictZone.h $
 // $Id: TriangulationPointInputAndConflictZone.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationRemoveVertex.h 5.1-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationRemoveVertex.h
--- 5.0.3-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationRemoveVertex.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationRemoveVertex.h	2020-09-08 12:14:54.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Hyperbolic_triangulation_2/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationRemoveVertex.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Hyperbolic_triangulation_2/demo/Hyperbolic_triangulation_2/include/internal/Qt/TriangulationRemoveVertex.h $
 // $Id: TriangulationRemoveVertex.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/L1_Voronoi_diagram_2/include/CGAL/L1_voronoi_traits_2.h 5.1-1/demo/L1_Voronoi_diagram_2/include/CGAL/L1_voronoi_traits_2.h
--- 5.0.3-1/demo/L1_Voronoi_diagram_2/include/CGAL/L1_voronoi_traits_2.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/demo/L1_Voronoi_diagram_2/include/CGAL/L1_voronoi_traits_2.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/demo/L1_Voronoi_diagram_2/include/CGAL/L1_voronoi_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/demo/L1_Voronoi_diagram_2/include/CGAL/L1_voronoi_traits_2.h $
 // $Id: L1_voronoi_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/L1_Voronoi_diagram_2/include/CGAL/Qt/ArrangementGraphicsItem.h 5.1-1/demo/L1_Voronoi_diagram_2/include/CGAL/Qt/ArrangementGraphicsItem.h
--- 5.0.3-1/demo/L1_Voronoi_diagram_2/include/CGAL/Qt/ArrangementGraphicsItem.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/demo/L1_Voronoi_diagram_2/include/CGAL/Qt/ArrangementGraphicsItem.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/demo/L1_Voronoi_diagram_2/include/CGAL/Qt/ArrangementGraphicsItem.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/demo/L1_Voronoi_diagram_2/include/CGAL/Qt/ArrangementGraphicsItem.h $
 // $Id: ArrangementGraphicsItem.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/L1_Voronoi_diagram_2/include/CGAL/Qt/SetGraphicsItem.h 5.1-1/demo/L1_Voronoi_diagram_2/include/CGAL/Qt/SetGraphicsItem.h
--- 5.0.3-1/demo/L1_Voronoi_diagram_2/include/CGAL/Qt/SetGraphicsItem.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/demo/L1_Voronoi_diagram_2/include/CGAL/Qt/SetGraphicsItem.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/demo/L1_Voronoi_diagram_2/include/CGAL/Qt/SetGraphicsItem.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/demo/L1_Voronoi_diagram_2/include/CGAL/Qt/SetGraphicsItem.h $
 // $Id: SetGraphicsItem.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/L1_Voronoi_diagram_2/L1_voronoi_diagram_2.cpp 5.1-1/demo/L1_Voronoi_diagram_2/L1_voronoi_diagram_2.cpp
--- 5.0.3-1/demo/L1_Voronoi_diagram_2/L1_voronoi_diagram_2.cpp	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/demo/L1_Voronoi_diagram_2/L1_voronoi_diagram_2.cpp	2020-09-08 12:12:19.000000000 +0000
@@ -178,7 +178,7 @@ MainWindow::MainWindow()
   this->graphicsView->setMouseTracking(true);
 
   // Turn the vertical axis upside down
-  this->graphicsView->matrix().scale(1, -1);
+  this->graphicsView->transform().scale(1, -1);
 
   // The navigation adds zooming and translation functionality to the
   // QGraphicsView
diff -pruN 5.0.3-1/demo/Linear_cell_complex/import_moka.h 5.1-1/demo/Linear_cell_complex/import_moka.h
--- 5.0.3-1/demo/Linear_cell_complex/import_moka.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/demo/Linear_cell_complex/import_moka.h	2020-09-08 12:15:05.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Linear_cell_complex/demo/Linear_cell_complex/import_moka.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Linear_cell_complex/demo/Linear_cell_complex/import_moka.h $
 // $Id: import_moka.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Linear_cell_complex/MainWindow.h 5.1-1/demo/Linear_cell_complex/MainWindow.h
--- 5.0.3-1/demo/Linear_cell_complex/MainWindow.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/demo/Linear_cell_complex/MainWindow.h	2020-09-08 12:15:05.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Linear_cell_complex/demo/Linear_cell_complex/MainWindow.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Linear_cell_complex/demo/Linear_cell_complex/MainWindow.h $
 // $Id: MainWindow.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Linear_cell_complex/typedefs.h 5.1-1/demo/Linear_cell_complex/typedefs.h
--- 5.0.3-1/demo/Linear_cell_complex/typedefs.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/demo/Linear_cell_complex/typedefs.h	2020-09-08 12:15:05.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Linear_cell_complex/demo/Linear_cell_complex/typedefs.h $
-// $Id: typedefs.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Linear_cell_complex/demo/Linear_cell_complex/typedefs.h $
+// $Id: typedefs.h cf83a22 2019-12-02T16:32:29+01:00 Mael
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
@@ -185,8 +185,7 @@ typedef CGAL::Triangulation_face_base_wi
 
 typedef CGAL::Constrained_triangulation_face_base_2<P_traits, Fb1>    Fb;
 typedef CGAL::Triangulation_data_structure_2<Vb,Fb>                   TDS;
-// typedef CGAL::No_intersection_tag                                     Itag;
- typedef CGAL::Exact_predicates_tag Itag;
+typedef CGAL::Exact_predicates_tag                                    Itag;
 typedef CGAL::Constrained_Delaunay_triangulation_2<P_traits, TDS,
                                                    Itag>              CDT;
 
diff -pruN 5.0.3-1/demo/Linear_cell_complex/Viewer.h 5.1-1/demo/Linear_cell_complex/Viewer.h
--- 5.0.3-1/demo/Linear_cell_complex/Viewer.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/demo/Linear_cell_complex/Viewer.h	2020-09-08 12:15:05.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Linear_cell_complex/demo/Linear_cell_complex/Viewer.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Linear_cell_complex/demo/Linear_cell_complex/Viewer.h $
 // $Id: Viewer.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Optimal_transportation_reconstruction_2/glviewer.cpp 5.1-1/demo/Optimal_transportation_reconstruction_2/glviewer.cpp
--- 5.0.3-1/demo/Optimal_transportation_reconstruction_2/glviewer.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Optimal_transportation_reconstruction_2/glviewer.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -87,7 +87,7 @@ void GlViewer::paintGL()
 void GlViewer::wheelEvent(QWheelEvent *event)
 {
   if (!m_scene) return;
-  m_scale += 0.05 * (event->delta() / 120);
+  m_scale += 0.05 * (event->angleDelta().y() / 120);
   if (m_scale <= 0.0) m_scale = 0.0;
   update();
 }
diff -pruN 5.0.3-1/demo/Optimal_transportation_reconstruction_2/scene.h 5.1-1/demo/Optimal_transportation_reconstruction_2/scene.h
--- 5.0.3-1/demo/Optimal_transportation_reconstruction_2/scene.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/demo/Optimal_transportation_reconstruction_2/scene.h	2020-09-08 12:15:17.000000000 +0000
@@ -18,9 +18,11 @@
 #define cimg_display 0 // To avoid X11 or Windows-GDI dependency
 #include <CImg.h>
 #endif
-#include <CGAL/Random.h>
 #include <utility>      // std::pair
 #include <vector>
+
+#include <CGAL/number_type_config.h>
+#include <CGAL/Random.h>
 #include <CGAL/property_map.h>
 #include <CGAL/value_type_traits.h>
 #include <CGAL/compute_average_spacing.h>
@@ -405,7 +407,7 @@ public:
                                                          Point_3_from_sample()),
                          boost::make_transform_iterator (m_samples.end(),
                                                          Point_3_from_sample())),
-       3);
+       3, CGAL::parameters::point_map (CGAL::Identity_property_map_no_lvalue<K::Point_3>()));
     std::cerr << "Average spacing = " << spacing << std::endl;
   }
 
@@ -953,7 +955,7 @@ public:
 
   void append_star(const int nb_branches, const int density) {
     std::cerr << "append star...";
-    const double deg_in_rad = 3.1415926535897932384626 / 180.0;
+    const double deg_in_rad = CGAL_PI / 180.0;
     const double incr = 180.0 / nb_branches;
     double angle = 0.0;
     const Point center(0.5, 0.5);
@@ -970,7 +972,7 @@ public:
 
   void append_predefined_increasingly_sharp_angles(const int density,
     const double min_angle) {
-    const double deg_in_rad = 3.1415926535897932384626 / 180.0;
+    const double deg_in_rad = CGAL_PI / 180.0;
     double prev_angle = 0.0;
     double curr_angle = min_angle;
     double incr = min_angle;
diff -pruN 5.0.3-1/demo/Periodic_2_triangulation_2/include/CGAL/Qt/PeriodicTriangulationGraphicsItem.h 5.1-1/demo/Periodic_2_triangulation_2/include/CGAL/Qt/PeriodicTriangulationGraphicsItem.h
--- 5.0.3-1/demo/Periodic_2_triangulation_2/include/CGAL/Qt/PeriodicTriangulationGraphicsItem.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/demo/Periodic_2_triangulation_2/include/CGAL/Qt/PeriodicTriangulationGraphicsItem.h	2020-09-08 12:14:52.000000000 +0000
@@ -11,7 +11,7 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/demo/Periodic_2_triangulation_2/include/CGAL/Qt/PeriodicTriangulationGraphicsItem.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/demo/Periodic_2_triangulation_2/include/CGAL/Qt/PeriodicTriangulationGraphicsItem.h $
 // $Id: PeriodicTriangulationGraphicsItem.h 590612a 2020-03-26T13:25:16+01:00 Sébastien Loriot
 //
 //
diff -pruN 5.0.3-1/demo/Periodic_2_triangulation_2/include/CGAL/Qt/PeriodicVoronoiGraphicsItem.h 5.1-1/demo/Periodic_2_triangulation_2/include/CGAL/Qt/PeriodicVoronoiGraphicsItem.h
--- 5.0.3-1/demo/Periodic_2_triangulation_2/include/CGAL/Qt/PeriodicVoronoiGraphicsItem.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/demo/Periodic_2_triangulation_2/include/CGAL/Qt/PeriodicVoronoiGraphicsItem.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/demo/Periodic_2_triangulation_2/include/CGAL/Qt/PeriodicVoronoiGraphicsItem.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/demo/Periodic_2_triangulation_2/include/CGAL/Qt/PeriodicVoronoiGraphicsItem.h $
 // $Id: PeriodicVoronoiGraphicsItem.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Periodic_2_triangulation_2/Periodic_2_Delaunay_triangulation_2.cpp 5.1-1/demo/Periodic_2_triangulation_2/Periodic_2_Delaunay_triangulation_2.cpp
--- 5.0.3-1/demo/Periodic_2_triangulation_2/Periodic_2_Delaunay_triangulation_2.cpp	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/demo/Periodic_2_triangulation_2/Periodic_2_Delaunay_triangulation_2.cpp	2020-09-08 12:12:19.000000000 +0000
@@ -181,7 +181,7 @@ MainWindow::MainWindow()
   this->graphicsView->setMouseTracking(true);
 
   // Turn the vertical axis upside down
-  this->graphicsView->matrix().scale(1, -1);
+  this->graphicsView->transform().scale(1, -1);
 
   // The navigation adds zooming and translation functionality to the
   // QGraphicsView
diff -pruN 5.0.3-1/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/hyperbolic_free_motion_animation.h 5.1-1/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/hyperbolic_free_motion_animation.h
--- 5.0.3-1/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/hyperbolic_free_motion_animation.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/hyperbolic_free_motion_animation.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_4_hyperbolic_triangulation_2/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/hyperbolic_free_motion_animation.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_4_hyperbolic_triangulation_2/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/hyperbolic_free_motion_animation.h $
 // $Id: hyperbolic_free_motion_animation.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/HyperbolicPainterOstream.h 5.1-1/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/HyperbolicPainterOstream.h
--- 5.0.3-1/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/HyperbolicPainterOstream.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/HyperbolicPainterOstream.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_4_hyperbolic_triangulation_2/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/HyperbolicPainterOstream.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_4_hyperbolic_triangulation_2/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/HyperbolicPainterOstream.h $
 // $Id: HyperbolicPainterOstream.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/TriangulationCircumcircle.h 5.1-1/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/TriangulationCircumcircle.h
--- 5.0.3-1/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/TriangulationCircumcircle.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/TriangulationCircumcircle.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_4_hyperbolic_triangulation_2/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/TriangulationCircumcircle.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_4_hyperbolic_triangulation_2/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/TriangulationCircumcircle.h $
 // $Id: TriangulationCircumcircle.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/TriangulationConflictZone.h 5.1-1/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/TriangulationConflictZone.h
--- 5.0.3-1/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/TriangulationConflictZone.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/TriangulationConflictZone.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_4_hyperbolic_triangulation_2/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/TriangulationConflictZone.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_4_hyperbolic_triangulation_2/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/TriangulationConflictZone.h $
 // $Id: TriangulationConflictZone.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/TriangulationGraphicsItem.h 5.1-1/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/TriangulationGraphicsItem.h
--- 5.0.3-1/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/TriangulationGraphicsItem.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/TriangulationGraphicsItem.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_4_hyperbolic_triangulation_2/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/TriangulationGraphicsItem.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_4_hyperbolic_triangulation_2/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/TriangulationGraphicsItem.h $
 // $Id: TriangulationGraphicsItem.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/TriangulationPointInput.h 5.1-1/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/TriangulationPointInput.h
--- 5.0.3-1/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/TriangulationPointInput.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/TriangulationPointInput.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_4_hyperbolic_triangulation_2/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/TriangulationPointInput.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_4_hyperbolic_triangulation_2/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/TriangulationPointInput.h $
 // $Id: TriangulationPointInput.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/VoronoiGraphicsItem.h 5.1-1/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/VoronoiGraphicsItem.h
--- 5.0.3-1/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/VoronoiGraphicsItem.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/VoronoiGraphicsItem.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_4_hyperbolic_triangulation_2/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/VoronoiGraphicsItem.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_4_hyperbolic_triangulation_2/demo/Periodic_4_hyperbolic_triangulation_2/include/internal/Qt/VoronoiGraphicsItem.h $
 // $Id: VoronoiGraphicsItem.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Polygon/CMakeLists.txt 5.1-1/demo/Polygon/CMakeLists.txt
--- 5.0.3-1/demo/Polygon/CMakeLists.txt	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/demo/Polygon/CMakeLists.txt	2020-09-08 12:12:19.000000000 +0000
@@ -16,9 +16,8 @@ endif()
 find_package(CGAL COMPONENTS Qt5 Core)
 
 find_package(Eigen3 3.1.0) #(requires 3.1.0 or greater)
-if (EIGEN3_FOUND)
-  include( ${EIGEN3_USE_FILE} )
-else()
+include(CGAL_Eigen_support)
+if (NOT TARGET CGAL::Eigen_support)
   message(STATUS "NOTICE: This project requires the Eigen library, and will not be compiled.")
   return()
 endif()
@@ -52,7 +51,7 @@ if ( CGAL_FOUND AND CGAL_Qt5_FOUND AND Q
   add_to_cached_list( CGAL_EXECUTABLE_TARGETS Polygon_2 )
 
   target_link_libraries( Polygon_2 PRIVATE
-    CGAL::CGAL CGAL::CGAL_Qt5 Qt5::Gui )
+    CGAL::CGAL CGAL::CGAL_Qt5 CGAL::Eigen_support Qt5::Gui )
 
   include(${CGAL_MODULES_DIR}/CGAL_add_test.cmake)
   cgal_add_compilation_test(Polygon_2)
diff -pruN 5.0.3-1/demo/Polyhedron/C3t3_type.h 5.1-1/demo/Polyhedron/C3t3_type.h
--- 5.0.3-1/demo/Polyhedron/C3t3_type.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/demo/Polyhedron/C3t3_type.h	2020-09-08 12:15:24.000000000 +0000
@@ -1,6 +1,9 @@
 #ifndef CGAL_DEMO_MESH_3_C3T3_TYPE_H
 #define CGAL_DEMO_MESH_3_C3T3_TYPE_H
 
+#include "config.h"
+#include "Plugins/Mesh_3/config_mesh_3.h"
+
 // include this to get #define BOOST_PARAMETER_MAX_ARITY 12
 // as otherwise it gets set via inclusion of Polyhedron_3.h
 #include <CGAL/boost/parameter.h>
diff -pruN 5.0.3-1/demo/Polyhedron/CGAL_double_edit.cpp 5.1-1/demo/Polyhedron/CGAL_double_edit.cpp
--- 5.0.3-1/demo/Polyhedron/CGAL_double_edit.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/demo/Polyhedron/CGAL_double_edit.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,78 @@
+#include "CGAL_double_edit.h"
+#include <iostream>
+#include <QDoubleValidator>
+
+class DoubleValidator : public QDoubleValidator
+{
+public:
+  DoubleValidator(QObject* parent = nullptr)
+    : QDoubleValidator(parent)
+  {
+    setLocale(QLocale::C);
+  }
+
+  void fixup ( QString & input ) const
+  {
+    input.replace(".", locale().decimalPoint());
+    input.replace(",", locale().decimalPoint());
+    QDoubleValidator::fixup(input);
+  }
+  QValidator::State validate ( QString & input, int & pos ) const
+  {
+    fixup(input);
+    return QDoubleValidator::validate(input, pos);
+  }
+};
+
+  DoubleEdit::DoubleEdit(QWidget *parent)
+    : QLineEdit(parent)
+  {
+    validator = new DoubleValidator(this);
+    this->setValidator(validator);
+  }
+
+  DoubleEdit::~DoubleEdit()
+  {
+    delete validator;
+  }
+
+
+  double DoubleEdit::value() const
+  {
+    return this->text().toDouble();
+  }
+
+  void DoubleEdit::setValue(double d)
+  {
+    this->setText(tr("%1").arg(d));
+  }
+
+  void DoubleEdit::setMinimum(double d)
+  {
+    this->validator->setBottom(d);
+  }
+
+  void DoubleEdit::setMaximum(double d)
+  {
+    this->validator->setTop(d);
+  }
+
+  void DoubleEdit::setRange(double rmin, double rmax)
+  {
+    this->validator->setRange(rmin, rmax, this->validator->decimals());
+  }
+
+  double DoubleEdit::getValue()
+  {
+    return this->value();
+  }
+
+  double DoubleEdit::getMinimum()
+  {
+    return this->validator->bottom();
+  }
+
+  double DoubleEdit::getMaximum()
+  {
+    return this->validator->top();
+  }
diff -pruN 5.0.3-1/demo/Polyhedron/CGAL_double_edit.h 5.1-1/demo/Polyhedron/CGAL_double_edit.h
--- 5.0.3-1/demo/Polyhedron/CGAL_double_edit.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/demo/Polyhedron/CGAL_double_edit.h	2020-09-08 12:15:24.000000000 +0000
@@ -0,0 +1,33 @@
+#ifndef CGAL_DOUBLE_EDIT_H
+#define CGAL_DOUBLE_EDIT_H
+
+
+
+#include <QObject>
+#include <QtCore/qglobal.h>
+#include <QLineEdit>
+#include "Scene_config.h"
+
+
+class DoubleValidator;
+class SCENE_EXPORT DoubleEdit : public QLineEdit {
+
+  Q_OBJECT
+  Q_PROPERTY(double value READ getValue WRITE setValue)
+  Q_PROPERTY(double minimum READ getMinimum WRITE setMinimum)
+  Q_PROPERTY(double maximum READ getMaximum WRITE setMaximum)
+public:
+  DoubleEdit(QWidget* parent = nullptr);
+  ~DoubleEdit();
+  double value() const;
+  void setValue(double d);
+  void setMinimum(double d);
+  void setMaximum(double d);
+  void setRange(double rmin, double rmax);
+  double getValue();
+  double getMinimum();
+  double getMaximum();
+private:
+  DoubleValidator* validator;
+};
+#endif // CGAL_DOUBLE_EDIT_H
diff -pruN 5.0.3-1/demo/Polyhedron/CMakeLists.txt 5.1-1/demo/Polyhedron/CMakeLists.txt
--- 5.0.3-1/demo/Polyhedron/CMakeLists.txt	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/CMakeLists.txt	2020-09-08 12:12:20.000000000 +0000
@@ -64,7 +64,7 @@ include(${CGAL_USE_FILE})
 find_package(Qt5
              QUIET
              COMPONENTS            OpenGL Script
-             OPTIONAL_COMPONENTS   ScriptTools)
+             OPTIONAL_COMPONENTS   ScriptTools WebSockets)
 
 if(Qt5_FOUND)
 
@@ -73,9 +73,7 @@ if(Qt5_FOUND)
 endif(Qt5_FOUND)
 
 find_package(Eigen3 3.2.0) #(requires 3.2.0 or greater)
-if (EIGEN3_FOUND)
-  include( ${EIGEN3_USE_FILE} )
-endif(EIGEN3_FOUND)
+include(CGAL_Eigen_support)
 
 find_package( METIS )
 
@@ -90,11 +88,17 @@ option(POLYHEDRON_DEMO_ACTIVATE_CONCURRE
 
 if( POLYHEDRON_DEMO_ACTIVATE_CONCURRENCY )
   find_package( TBB )
-  if( NOT TBB_FOUND )
+  include(CGAL_TBB_support)
+  if( NOT TARGET CGAL::TBB_support )
     message( STATUS "NOTICE: Intel TBB was not found. Bilateral smoothing and WLOP plugins are faster if TBB is linked." )
   endif()
 endif()
 
+#find libssh for scene sharing
+find_package(LibSSH)
+if( NOT LIBSSH_FOUND )
+  message("NOTICE : The SSH features will be disabled.")
+endif()
 
 # Activate concurrency ? (turned OFF by default)
 option(CGAL_ACTIVATE_CONCURRENT_MESH_3
@@ -106,7 +110,8 @@ if( CGAL_ACTIVATE_CONCURRENT_MESH_3 OR E
   add_definitions( -DCGAL_CONCURRENT_MESH_3 )
   if(NOT TBB_FOUND)
     find_package( TBB REQUIRED )
-    if( NOT TBB_FOUND )
+    include( CGAL_TBB_support)
+    if( NOT TARGET CGAL::TBB_support )
       message(STATUS "NOTICE: Intel TBB was not found. Mesh_3 is faster if TBB is linked.")
     endif()
   endif()
@@ -117,12 +122,10 @@ else( CGAL_ACTIVATE_CONCURRENT_MESH_3 OR
     ON)
   if( LINK_WITH_TBB )
     find_package( TBB )
+    include( CGAL_TBB_support)
   endif( LINK_WITH_TBB )
 endif()
 
-
-
-
 if(CGAL_Qt5_FOUND AND Qt5_FOUND)
   include(${CGAL_USE_FILE})
 
@@ -131,7 +134,7 @@ if(CGAL_Qt5_FOUND AND Qt5_FOUND)
   qt5_wrap_ui( statisticsUI_FILES Statistics_on_item_dialog.ui)
   qt5_wrap_ui( FileLoaderDialogUI_files FileLoaderDialog.ui )
   qt5_wrap_ui( Show_point_dialogUI_FILES Show_point_dialog.ui )
-  qt5_wrap_ui( PreferencesUI_FILES   Preferences.ui Details.ui)
+  qt5_wrap_ui( PreferencesUI_FILES   Preferences.ui Details.ui SSH_dialog.ui)
   qt5_wrap_ui( Show_point_dialogUI_FILES Show_point_dialog.ui )
   qt5_wrap_ui( ViewerUI_FILES  LightingDialog.ui)
   qt5_generate_moc( "File_loader_dialog.h" "${CMAKE_CURRENT_BINARY_DIR}/File_loader_dialog_moc.cpp" )
@@ -190,10 +193,16 @@ if(CGAL_Qt5_FOUND AND Qt5_FOUND)
     Scene_item_rendering_helper.cpp
     Scene_item_rendering_helper_moc.cpp
     Primitive_container.cpp
-    Polyhedron_demo_plugin_helper.cpp)
+    Polyhedron_demo_plugin_helper.cpp
+    CGAL_double_edit.cpp)
   target_link_libraries(demo_framework
     PUBLIC Qt5::OpenGL Qt5::Widgets Qt5::Gui Qt5::Script
     )
+  if(TARGET Qt5::WebSockets)
+    target_link_libraries(demo_framework PUBLIC Qt5::WebSockets)
+    message(STATUS "Qt5WebSockets was found. Using WebSockets is therefore possible.")
+  endif()
+
   cgal_add_compilation_test(demo_framework)
   # Let's define `three_EXPORT` during the compilation of `demo_framework`,
   # in addition of `demo_framework_EXPORT` (defined automatically by
@@ -243,14 +252,21 @@ if(CGAL_Qt5_FOUND AND Qt5_FOUND)
 
   add_item(scene_c3t3_item Scene_c3t3_item.cpp)
   target_link_libraries(scene_c3t3_item PUBLIC scene_surface_mesh_item scene_polygon_soup_item scene_basic_objects ${TBB_LIBRARIES})
-  if(TBB_FOUND)
-    CGAL_target_use_TBB(scene_c3t3_item)
+  if(TARGET CGAL::TBB_support)
+    target_link_libraries(scene_c3t3_item PUBLIC CGAL::TBB_support)
+  endif()
+  if(COMMAND target_precompile_headers)
+    # Support for precompiled headers, for Mesh_3 (since CMake 3.16)
+    target_precompile_headers(scene_c3t3_item PUBLIC [["C3t3_type.h"]])
   endif()
   add_item(scene_transform_item Plugins/PCA/Scene_facegraph_transform_item.cpp )
   add_item(scene_edit_box_item Plugins/PCA/Scene_edit_box_item.cpp )
   add_item(scene_image_item Scene_image_item.cpp)
   add_item(scene_surface_mesh_item Scene_surface_mesh_item.cpp)
 
+  if(TBB_FOUND)
+    target_link_libraries(scene_surface_mesh_item PUBLIC CGAL::TBB_support)
+  endif()
   # special
 
   add_item(scene_item_decorator Scene_polyhedron_item_decorator.cpp )
@@ -261,20 +277,25 @@ if(CGAL_Qt5_FOUND AND Qt5_FOUND)
 
   add_item(scene_selection_item Scene_polyhedron_selection_item.cpp)
   target_link_libraries(scene_selection_item PUBLIC scene_item_decorator  scene_k_ring_selection)
+ if(TBB_FOUND)
+    target_link_libraries(scene_selection_item PUBLIC CGAL::TBB_support)
+  endif()
 
   add_item(scene_shortest_path_item Plugins/Surface_mesh/Scene_polyhedron_shortest_path_item.cpp)
   target_link_libraries(scene_shortest_path_item PUBLIC scene_item_decorator scene_surface_mesh_item scene_polylines_item)
 
   add_item(scene_movable_sm_item Plugins/AABB_tree/Scene_movable_sm_item.cpp)
 
-  if(EIGEN3_FOUND )
+  if(TARGET CGAL::Eigen_support )
     add_item(scene_textured_item Scene_textured_surface_mesh_item.cpp texture.cpp)
+    target_link_libraries(scene_textured_item PUBLIC CGAL::Eigen_support)
     qt5_wrap_ui( editionUI_FILES Plugins/Surface_mesh_deformation/Deform_mesh.ui )
     add_item(scene_edit_item Plugins/Surface_mesh_deformation/Scene_edit_polyhedron_item.cpp
       ${editionUI_FILES})
-    target_link_libraries(scene_edit_item PUBLIC scene_surface_mesh_item scene_k_ring_selection
+    target_link_libraries(scene_edit_item PUBLIC CGAL::Eigen_support scene_surface_mesh_item scene_k_ring_selection
       scene_basic_objects)
     add_item(scene_mcf_item Plugins/PMP/Scene_mcf_item.cpp)
+    target_link_libraries(scene_mcf_item PUBLIC CGAL::Eigen_support)
   endif()
 
   add_item(scene_implicit_function_item Scene_implicit_function_item.cpp )
@@ -286,21 +307,22 @@ if(CGAL_Qt5_FOUND AND Qt5_FOUND)
   add_item(scene_nef_polyhedron_item Scene_nef_polyhedron_item.cpp)
   target_link_libraries(scene_nef_polyhedron_item PUBLIC scene_surface_mesh_item)
 
+  add_item(scene_points_with_normal_item Scene_points_with_normal_item.cpp)
+  if (TARGET CGAL::Eigen_support)
+    target_link_libraries(scene_points_with_normal_item PUBLIC CGAL::Eigen_support)
+  endif()
+
   find_package(LASLIB)
-  if (LASLIB_FOUND)
-    include(${LASLIB_USE_FILE})
-    include_directories(${LASLIB_INCLUDE_DIR})
-    include_directories(${LASZIP_INCLUDE_DIR})
-    add_item(scene_points_with_normal_item Scene_points_with_normal_item.cpp)
+  include(CGAL_LASLIB_support)
+  if (TARGET CGAL::LASLIB_support)
+    target_link_libraries(scene_points_with_normal_item PUBLIC CGAL::LASLIB_support)
     if (MSVC)
       target_compile_definitions( scene_points_with_normal_item PUBLIC "-D_CRT_SECURE_NO_DEPRECATE -D_CRT_SECURE_NO_WARNINGS")
     endif()
-    target_link_libraries( scene_points_with_normal_item PUBLIC ${LASLIB_LIBRARIES})
-  else()
-    add_item(scene_points_with_normal_item Scene_points_with_normal_item.cpp)
   endif()
+
   if(TBB_FOUND)
-    CGAL_target_use_TBB(scene_points_with_normal_item)
+    target_link_libraries(scene_points_with_normal_item PUBLIC CGAL::TBB_support)
   endif()
 
   add_item(scene_polylines_item Scene_polylines_item.cpp)
@@ -322,11 +344,20 @@ if(CGAL_Qt5_FOUND AND Qt5_FOUND)
     MainWindow.cpp
     Polyhedron_demo.cpp
     File_loader_dialog_moc.cpp
+    Use_ssh.cpp
     ${CGAL_Qt5_RESOURCE_FILES} ${CGAL_Qt5_MOC_FILES}
     ${FileLoaderDialogUI_files} ${MainWindowUI_files} ${PreferencesUI_FILES}
     ${statisticsUI_FILES} ${SubViewerUI_files})
   target_link_libraries(polyhedron_demo PUBLIC
     demo_framework point_dialog Qt5::Gui Qt5::OpenGL Qt5::Widgets Qt5::Script)
+  if(LIBSSH_FOUND)
+    add_definitions(-DCGAL_USE_SSH)
+    target_link_libraries(polyhedron_demo PUBLIC ${LIBSSH_LIBRARIES})
+  endif() #libssh
+  if(TARGET Qt5::WebSockets)
+    add_definitions(-DCGAL_USE_WEBSOCKETS)
+    target_link_libraries(polyhedron_demo PUBLIC Qt5::WebSockets)
+  endif()
   add_executable  ( Polyhedron_3 Polyhedron_3.cpp )
   target_link_libraries( Polyhedron_3 PRIVATE polyhedron_demo )
   add_to_cached_list( CGAL_EXECUTABLE_TARGETS Polyhedron_3 )
@@ -375,6 +406,13 @@ add_executable  ( CGAL_PMP PMP.cpp )
 add_dependencies(CGAL_PMP PMP)
 target_link_libraries( CGAL_PMP PRIVATE polyhedron_demo )
 add_to_cached_list( CGAL_EXECUTABLE_TARGETS CGAL_PMP )
+
+#WS Server
+if(TARGET Qt5::WebSockets)
+  add_executable  (WS_server Server_ws.cpp)
+  target_link_libraries(WS_server PUBLIC Qt5::WebSockets)
+  message(STATUS "Qt5WebSockets was found. Using WebSockets is therefore possible.")
+endif()
 #
 # Exporting
 #
diff -pruN 5.0.3-1/demo/Polyhedron/Color_map.h 5.1-1/demo/Polyhedron/Color_map.h
--- 5.0.3-1/demo/Polyhedron/Color_map.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Color_map.h	2020-09-08 12:15:24.000000000 +0000
@@ -8,7 +8,7 @@ inline QColor generate_color(double h, d
   std::size_t s_max=255;
   if(h >0.8 && h < 0.95) //span of ugly pink, desaturates make it less garish IMO
     s_max = 160;
-  std::size_t s = std::rand() % (s_max-static_cast<std::size_t>(s_min*255)) +s_min*255;
+  std::size_t s = std::rand() % (s_max-static_cast<std::size_t>(s_min*255)) + static_cast<int>(s_min*255);
   return QColor::fromHsvF(h,s/255.0,1.0);
 }
 
diff -pruN 5.0.3-1/demo/Polyhedron/implicit_functions/Implicit_function_interface.h 5.1-1/demo/Polyhedron/implicit_functions/Implicit_function_interface.h
--- 5.0.3-1/demo/Polyhedron/implicit_functions/Implicit_function_interface.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/demo/Polyhedron/implicit_functions/Implicit_function_interface.h	2020-09-08 12:15:27.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron/demo/Polyhedron/implicit_functions/Implicit_function_interface.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron/demo/Polyhedron/implicit_functions/Implicit_function_interface.h $
 // $Id: Implicit_function_interface.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Polyhedron/include/CGAL/IO/read_surf_trianglemesh.h 5.1-1/demo/Polyhedron/include/CGAL/IO/read_surf_trianglemesh.h
--- 5.0.3-1/demo/Polyhedron/include/CGAL/IO/read_surf_trianglemesh.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/demo/Polyhedron/include/CGAL/IO/read_surf_trianglemesh.h	2020-09-08 12:15:27.000000000 +0000
@@ -72,228 +72,501 @@ bool line_starts_with(const std::string&
     return (line.compare(fnws, strlen(cstr), cstr) == 0);
   return false;
 }
-
-/*!
- * read_surf reads a file which extension is .surf and fills `output` with one `Mesh` per patch.
- * Mesh is a model of FaceListGraph.
- */
-template<class Mesh, typename DuplicatedPointsOutIterator, class NamedParameters>
-bool read_surf(std::istream& input, std::vector<Mesh>& output,
-    std::vector<MaterialData>& metadata,
-    CGAL::Bbox_3& grid_box,
-    std::array<unsigned int, 3>& grid_size,
-    DuplicatedPointsOutIterator out,
-    const NamedParameters&)
+namespace IO{
+namespace internal{
+bool treat_surf_materials(std::istream& input,
+                          std::vector<material>& materials,
+                          int& material_id)
 {
-  typedef typename CGAL::Polygon_mesh_processing::GetGeomTraits<Mesh,
-      NamedParameters>::type Kernel;
-  typedef typename Kernel::Point_3 Point_3;
-  std::vector<Point_3> points;
   std::string line;
-  std::istringstream iss;
-  std::size_t nb_vertices(0);
-  std::vector<material> materials;
-  //ignore header
-  int material_id = 0;
   while(std::getline(input, line))
   {
-    if (line_starts_with(line, "Materials"))
-    {
-      while(std::getline(input, line))
-      {
-        if(line_starts_with(line, "}"))
-          break;
-        else
-        {
-          material _material;
-          if (!get_material_metadata(input, line, _material, material_id++))
-            return false;
-          materials.push_back(_material);
-        }
-      }
+    if(line_starts_with(line, "}"))
       break;
+    else
+    {
+      material _material;
+      if (!get_material_metadata(input, line, _material, material_id++))
+        return false;
+      materials.push_back(_material);
     }
   }
+  return true;
+}
+
+void treat_surf_grid_box(const std::string& line,
+                        CGAL::Bbox_3& grid_box)
+{
+  std::istringstream iss;
+  iss.str(line);
+  std::string dump;
+  double xmin, xmax, ymin, ymax, zmin, zmax;
+  iss >> dump >> xmin >> xmax >> ymin >> ymax >> zmin >> zmax;
+  grid_box = CGAL::Bbox_3(xmin, ymin, zmin, xmax, ymax, zmax);
+}
 
-  //get grid box
-  while (std::getline(input, line))
+void treat_surf_grid_size(const std::string& line,
+                         std::array<unsigned int, 3>& grid_size)
+{
+  std::istringstream iss;
+  iss.str(line);
+  std::string dump;
+  iss >> dump >> grid_size[0] >> grid_size[1] >> grid_size[2];
+}
+
+template<typename Point_3>
+void treat_surf_vertices(std::istream& input,
+                         const std::string& input_line,
+                         std::size_t& nb_vertices,
+                         std::vector<Point_3>& points)
+{
+  std::istringstream iss;
+  iss.str(input_line);
+  std::string dump;
+  iss >> dump >> nb_vertices;
+  points.reserve(nb_vertices);
+  std::cout<<nb_vertices<<" vertices"<<std::endl;
+
+  std::string line;
+  //get vertices
+  while(std::getline(input, line))
   {
-    if (line_starts_with(line, "GridBox"))
-    {
-      iss.clear();
-      iss.str(line);
-      std::string dump;
-      double xmin, xmax, ymin, ymax, zmin, zmax;
-      iss >> dump >> xmin >> xmax >> ymin >> ymax >> zmin >> zmax;
-      grid_box = CGAL::Bbox_3(xmin, ymin, zmin, xmax, ymax, zmax);
+    if (line_starts_with(line, "NBranchingPoints"))
       break;
-    }
+    double x(0),y(0),z(0);
+    iss.clear();
+    iss.str(line);
+    iss >> CGAL::iformat(x) >> CGAL::iformat(y) >> CGAL::iformat(z);
+    points.push_back(Point_3(x,y,z));
   }
+}
+
+template<typename MaterialData, typename Mesh>
+void get_surf_patches(const std::string& line,
+                      int& nb_patches,
+                      std::vector<MaterialData>& metadata,
+                      std::vector<Mesh>& output)
+{
+  std::istringstream iss;
+  iss.str(line);
+  std::string dump;
+  iss >> dump >> nb_patches;
+  std::cout<<nb_patches<<" patch(es)"<<std::endl;
+  metadata.resize(nb_patches);
+  output.resize(nb_patches);
+}
 
-  //get grid size
-  while (std::getline(input, line))
+void treat_surf_inner_region(std::istream& input,
+                             const std::string& input_line,
+                             std::vector<material>& materials,
+                             std::vector<MaterialData>& metadata,
+                             const int& i)
+{
+  std::istringstream iss;
+  std::string name;
+  iss.clear();
+  iss.str(input_line);
+  std::string dump;
+  iss >> dump >> name;
+  name.erase(std::remove(name.begin(), name.end(), '"'), name.end());
+  name.erase(std::remove(name.begin(), name.end(), ','), name.end());
+  for(std::size_t j=0; j<materials.size(); ++j)
   {
-    if (line_starts_with(line, "GridSize"))
+    if(materials[j].second.compare(name) == 0)
     {
-      iss.clear();
-      iss.str(line);
-      std::string dump;
-      iss >> dump >> grid_size[0] >> grid_size[1] >> grid_size[2];
+      metadata[i].innerRegion=materials[j];
       break;
     }
   }
-
-  //get number of vertices
-  while(std::getline(input, line))
+  std::string line;
+  std::getline(input, line);
+  iss.clear();
+  iss.str(line);
+  iss >> dump >> name;
+  for(std::size_t j=0; j<materials.size(); ++j)
   {
-    if (line_starts_with(line, "Vertices"))
+    if(materials[j].second.compare(name) == 0)
     {
-      iss.clear();
-      iss.str(line);
-      std::string dump;
-      iss >> dump >> nb_vertices;
-      points.reserve(nb_vertices);
+      metadata[i].outerRegion=materials[j];
       break;
     }
   }
-  //get vertices
+}
+
+void treat_surf_triangles(const std::string& line,
+                          std::size_t& nb_triangles)
+{
+  std::istringstream iss;
+  iss.str(line);
+  std::string dump;
+  iss >> dump >> nb_triangles;
+}
+
+void connect_surf_triangles(std::istream& input,
+                            const std::size_t& nb_triangles,
+                            std::vector<std::array<std::size_t, 3> >& polygons)
+{
+  typedef std::array<std::size_t, 3> Triangle_ind;
+
+  polygons.reserve(nb_triangles);
+  std::string line;
+  std::istringstream iss;
   while(std::getline(input, line))
   {
-    if (line_starts_with(line, "NBranchingPoints"))
+    std::size_t fnws=line.find_first_not_of(" \t");
+    if(fnws != std::string::npos)
+      line.erase(0, fnws);
+    std::size_t index[3];
+    if(line.compare(0, 1, "}") == 0)
+    {
       break;
-    double x(0),y(0),z(0);
+    }
     iss.clear();
     iss.str(line);
-    iss >> CGAL::iformat(x) >> CGAL::iformat(y) >> CGAL::iformat(z);
-    points.push_back(Point_3(x,y,z));
+    iss >> index[0] >> index[1] >> index[2];
+
+    Triangle_ind polygon;
+    polygon[0] = index[0] - 1;
+    polygon[1] = index[1] - 1;
+    polygon[2] = index[2] - 1;
+    polygons.push_back(polygon);
   }
-  std::cout<<nb_vertices<<" vertices"<<std::endl;
-  int nb_patches = 0;
-  //get number of patches
-  while(std::getline(input, line))
+  CGAL_assertion(nb_triangles == polygons.size());
+}
+
+template<typename Point_3, typename DuplicatedPointsOutIterator, typename Mesh>
+void build_surf_patch(DuplicatedPointsOutIterator out,
+                      std::vector<Mesh>& output,
+                      std::vector<Point_3> points,
+                      std::vector<std::array<std::size_t, 3> >& polygons,
+                      const int& i)
+{
+  namespace PMP = CGAL::Polygon_mesh_processing;
+  typedef std::array<std::size_t, 3> Triangle_ind;
+  if (!PMP::is_polygon_soup_a_polygon_mesh(polygons))
   {
-    if (line_starts_with(line, "Patches"))
+    std::cout << "Orientation of patch #" << (i + 1) << "...";
+    std::cout.flush();
+
+    const std::size_t nbp_init = points.size();
+    bool no_duplicates =
+        PMP::orient_polygon_soup(points, polygons);//returns false if some points
+    //were duplicated
+
+    std::cout << "\rOrientation of patch #" << (i + 1) << " done";
+
+    if(!no_duplicates) //collect duplicates
     {
-      iss.clear();
-      iss.str(line);
-      std::string dump;
-      iss >> dump >> nb_patches;
-      break;
+      for (std::size_t i = nbp_init; i < points.size(); ++i)
+        *out++ = points[i];
+      std::cout << " (non manifold -> "
+                << (points.size() - nbp_init) << " duplicated vertices)";
     }
+    std::cout << "." << std::endl;
   }
-  std::cout<<nb_patches<<" patch(es)"<<std::endl;
-  metadata.resize(nb_patches);
-  output.resize(nb_patches);
 
-  for(int i=0; i < nb_patches; ++i)
+  Mesh& mesh = output[i];
+
+  PMP::internal::PS_to_PM_converter<std::vector<Point_3>, std::vector<Triangle_ind> > converter(points, polygons);
+  converter(mesh, false/*insert_isolated_vertices*/);
+
+  CGAL_assertion(PMP::remove_isolated_vertices(mesh) == 0);
+  CGAL_assertion(is_valid_polygon_mesh(mesh));
+}
+
+template<class Point>
+bool fill_binary_vertices(std::istream& is, std::vector<Point>& points, const std::size_t& nb_points)
+{
+  float f[3];
+  std::size_t pos = 0;
+  while(pos < nb_points)
+  {
+    is.read(reinterpret_cast<char*>(&f[0]), sizeof(f[0]));
+    if(!is.good())
+      return false;
+    is.read(reinterpret_cast<char*>(&f[1]), sizeof(f[1]));
+    if(!is.good())
+      return false;
+    is.read(reinterpret_cast<char*>(&f[2]), sizeof(f[2]));
+    if(!is.good())
+      return false;
+    points.push_back(Point(f[0], f[1], f[2]));
+    ++pos;
+  }
+  return true;
+}
+
+bool fill_binary_triangles(std::istream& is, std::vector<std::array<std::size_t, 3> >& triangles, const std::size_t& nb_trs)
+{
+  int tr[7];
+  std::size_t pos = 0;
+  while(pos < nb_trs)
+  {
+    for(int i = 0; i < 7; ++i)
+    {
+      is.read(reinterpret_cast<char*>(&tr[i]), sizeof(tr[i]));
+      if(!is.good())
+        return false;
+    }
+    std::array<std::size_t, 3> tri;
+    for(int i = 0; i < 3; ++i)
+      tri[i] = tr[i];
+    triangles.push_back(tri);
+    ++pos;
+  }
+  return true;
+}
+
+bool fill_binary_patch(std::istream& is, std::vector<std::size_t>& patch, std::size_t size)
+{
+  int id = 0;
+  std::size_t pos = 0;
+  while(pos < size)
+  {
+    is.read(reinterpret_cast<char*>(&id), sizeof(int));
+    if(!is.good())
+      return false;
+    patch.push_back(id);
+    ++pos;
+  }
+  return true;
+}
+
+template<typename Point_3, typename DuplicatedPointsOutIterator, typename Mesh>
+bool build_binary_surf_patch(DuplicatedPointsOutIterator& out,
+                             std::vector<Mesh>& output,
+                             std::vector<Point_3>& points,
+                             const std::vector<std::array<std::size_t, 3> >& polygons,
+                             const std::vector<std::size_t>& patch,
+                             const std::size_t i)
+{
+  namespace PMP = CGAL::Polygon_mesh_processing;
+  typedef std::array<std::size_t, 3> Triangle_ind;
+  std::vector<Triangle_ind> triangles;
+  triangles.reserve(patch.size());
+  for(const std::size_t& id : patch)
+  {
+    triangles.push_back(polygons[id]);
+  }
+  if (!PMP::is_polygon_soup_a_polygon_mesh(triangles))
+  {
+    std::cout << "Orientation of patch #" << (i + 1) << "...";
+    std::cout.flush();
+
+    const std::size_t nbp_init = points.size();
+    bool no_duplicates =
+        PMP::orient_polygon_soup(points, triangles);//returns false if some points
+    //were duplicated
+
+    std::cout << "\rOrientation of patch #" << (i + 1) << " done";
+
+    if(!no_duplicates) //collect duplicates
+    {
+      for (std::size_t i = nbp_init; i < points.size(); ++i)
+        *out++ = points[i];
+      std::cout << " (non manifold -> "
+                << (points.size() - nbp_init) << " duplicated vertices)";
+    }
+    std::cout << "." << std::endl;
+  }
+
+  Mesh& mesh = output[i];
+
+  PMP::internal::PS_to_PM_converter<std::vector<Point_3>, std::vector<Triangle_ind> > converter(points, triangles);
+  converter(mesh, false/*insert_isolated_vertices*/);
+
+  CGAL_assertion(PMP::remove_isolated_vertices(mesh) == 0);
+  CGAL_assertion(is_valid_polygon_mesh(mesh));
+  return true;
+}
+}//end internal
+}//end IO
+
+/*!
+ * read_surf reads a file which extension is .surf and fills `output` with one `Mesh` per patch.
+ * Mesh is a model of FaceListGraph.
+ */
+template<class Mesh, typename DuplicatedPointsOutIterator, class NamedParameters>
+bool read_surf(std::istream& input, std::vector<Mesh>& output,
+    std::vector<MaterialData>& metadata,
+    CGAL::Bbox_3& grid_box,
+    std::array<unsigned int, 3>& grid_size,
+    DuplicatedPointsOutIterator out,
+    const NamedParameters&)
+{
+  typedef typename CGAL::GetGeomTraits<Mesh, NamedParameters>::type Kernel;
+  typedef typename Kernel::Point_3 Point_3;
+
+  std::vector<Point_3> points;
+  std::string line;
+  std::size_t nb_vertices(0);
+  std::vector<material> materials;
+  //ignore header
+  int material_id = 0;
+  int nb_patches = 0;
+  if(!std::getline(input, line))
+    return false;
+  bool binary = (line.find("BINARY") != std::string::npos);
+  if(!binary)
   {
-    //get metada
     while(std::getline(input, line))
     {
-      if (line_starts_with(line, "InnerRegion"))
+      if (line_starts_with(line, "Materials"))
       {
-        std::string name;
-        iss.clear();
-        iss.str(line);
-        std::string dump;
-        iss >> dump >> name;
-        for(std::size_t j=0; j<materials.size(); ++j)
+        if(!IO::internal::treat_surf_materials(input, materials, material_id))
+          return false;
+      }
+
+      //get grid box
+      if (line_starts_with(line, "GridBox"))
+      {
+        IO::internal::treat_surf_grid_box(line, grid_box);
+      }
+
+      //get grid size
+      if (line_starts_with(line, "GridSize"))
+      {
+        IO::internal::treat_surf_grid_size(line, grid_size);
+      }
+
+      //get number of vertices
+      if (line_starts_with(line, "Vertices"))
+      {
+        IO::internal::treat_surf_vertices(input, line, nb_vertices, points);
+      }
+
+      //get number of patches
+      if (line_starts_with(line, "Patches"))
+      {
+        IO::internal::get_surf_patches(line, nb_patches, metadata, output);
+        break;
+      }
+    }
+
+    for(int i=0; i < nb_patches; ++i)
+    {
+      std::size_t nb_triangles(0);
+      //get metada
+      while(std::getline(input, line))
+      {
+        if (line_starts_with(line, "InnerRegion"))
         {
-          if(materials[j].second.compare(name) == 0)
-          {
-            metadata[i].innerRegion=materials[j];
-            break;
-          }
+          IO::internal::treat_surf_inner_region(input, line, materials, metadata, i);
         }
-        std::getline(input, line);
-        iss.clear();
-        iss.str(line);
-        iss >> dump >> name;
-        for(std::size_t j=0; j<materials.size(); ++j)
+        if (line_starts_with(line, "Triangles"))
         {
-          if(materials[j].second.compare(name) == 0)
-          {
-            metadata[i].outerRegion=materials[j];
-            break;
-          }
+          IO::internal::treat_surf_triangles(line, nb_triangles);
+          break;
         }
-        break;
       }
-    }
-
-    std::size_t nb_triangles(0);
+      //connect triangles
+      typedef std::array<std::size_t, 3> Triangle_ind;
+      std::vector<Triangle_ind> polygons;
+      IO::internal::connect_surf_triangles(input, nb_triangles, polygons);
+
+      //build patch
+      IO::internal::build_surf_patch(out, output, points, polygons, i);
+    } // end loop on patches
+  }
+  else
+  {
+    int nTriangles = 0;
+    typedef std::array<std::size_t, 3> Triangle_ind;
+    std::vector<Triangle_ind> triangles;
+    std::vector<int> tr_per_patches;
+    int patch_counter = -1;
     while(std::getline(input, line))
     {
-      if (line_starts_with(line, "Triangles"))
+      //get nb vertices
+      if (line_starts_with(line, "nVertices"))
       {
-        iss.clear();
+        std::istringstream iss;
         iss.str(line);
         std::string dump;
-        iss >> dump >> nb_triangles;
-        break;
+        if(!(iss >> dump >> nb_vertices))
+           return false;
       }
-    }
-
-    //connect triangles
-    typedef std::array<std::size_t, 3> Triangle_ind;
-    std::vector<Triangle_ind> polygons;
-    polygons.reserve(nb_triangles);
-    while(std::getline(input, line))
-    {
-      std::size_t fnws=line.find_first_not_of(" \t");
-      if(fnws != std::string::npos)
-        line.erase(0, fnws);
-      std::size_t index[3];
-      if(line.compare(0, 1, "}") == 0)
+    //get nb triangles
+      if (line_starts_with(line, "nTriangles"))
       {
-        break;
+        std::istringstream iss;
+        iss.str(line);
+        std::string dump;
+        if(!(iss >> dump >> nTriangles))
+          return false;
+        std::cout<<nTriangles<<" triangles."<<std::endl;
+      }
+    //get nb patches and nb triangles per patches
+      if (line_starts_with(line, "define Patches"))
+      {
+        std::istringstream iss;
+        iss.str(line);
+        std::string dump;
+        if(patch_counter <0)
+        {
+          if(!(iss >> dump >> dump >> nb_patches))
+            return false;
+          std::cout<<nb_patches<<" patches."<<std::endl;
+          ++patch_counter;
+          metadata.resize(nb_patches);
+          output.resize(nb_patches);
+        }
+        else if(patch_counter < nb_patches)
+        {
+          int nb_tr = 0;
+          if(!(iss >> dump >> dump >> nb_tr))
+            return false;
+          tr_per_patches.push_back(nb_tr);
+          ++patch_counter;
+        }
+        else
+        {
+          std::cerr<<"Error in input file. Incoherent number of materials."<<std::endl;
+          return false;
+        }
+        //reset patch_counter
+        if(patch_counter == nb_patches)
+          patch_counter = 0;
       }
-      iss.clear();
-      iss.str(line);
-      iss >> index[0] >> index[1] >> index[2];
-
-      Triangle_ind polygon;
-      polygon[0] = index[0] - 1;
-      polygon[1] = index[1] - 1;
-      polygon[2] = index[2] - 1;
-      polygons.push_back(polygon);
-    }
-    CGAL_assertion(nb_triangles == polygons.size());
-
-    //build patch
-    namespace PMP = CGAL::Polygon_mesh_processing;
-    if (!PMP::is_polygon_soup_a_polygon_mesh(polygons))
-    {
-      std::cout << "Orientation of patch #" << (i + 1) << "...";
-      std::cout.flush();
-
-      const std::size_t nbp_init = points.size();
-      bool no_duplicates =
-      PMP::orient_polygon_soup(points, polygons);//returns false if some points
-                                                 //were duplicated
 
-      std::cout << "\rOrientation of patch #" << (i + 1) << " done";
+    //get materials Ids
+      if (line_starts_with(line, "Materials"))
+      {
+        if(!IO::internal::treat_surf_materials(input, materials, material_id))
+          return false;
+      }
+    //get patches  InnerRegion
+      if (line_starts_with(line, "InnerRegion"))
+      {
+        IO::internal::treat_surf_inner_region(input, line, materials,metadata, patch_counter);
+      }
+      if (line_starts_with(line, "GridBox"))
+      {
+        IO::internal::treat_surf_grid_box(line, grid_box);
+      }
 
-      if(!no_duplicates) //collect duplicates
+      //get grid size
+      if (line_starts_with(line, "GridSize"))
+      {
+        IO::internal::treat_surf_grid_size(line, grid_size);
+      }
+      //fill vertices
+      if (line_starts_with(line, "@1"))
+        IO::internal::fill_binary_vertices(input, points, nb_vertices);
+      //fill triangles
+      else if (line_starts_with(line, "@2"))
       {
-        for (std::size_t i = nbp_init; i < points.size(); ++i)
-          *out++ = points[i];
-        std::cout << " (non manifold -> "
-                  << (points.size() - nbp_init) << " duplicated vertices)";
+        IO::internal::fill_binary_triangles(input, triangles, nTriangles);
+      }
+      //fill patches
+      else if (line_starts_with(line, "@"))
+      {
+        std::vector<std::size_t> patch;
+        IO::internal::fill_binary_patch(input, patch, tr_per_patches[patch_counter]);
+        IO::internal::build_binary_surf_patch(out, output, points, triangles, patch, patch_counter++);
       }
-      std::cout << "." << std::endl;
     }
-
-    Mesh& mesh = output[i];
-
-    PMP::internal::Polygon_soup_to_polygon_mesh<Mesh, std::vector<Point_3>, std::vector<Triangle_ind>>
-      converter(points, polygons);
-    converter(mesh, false/*insert_isolated_vertices*/);
-
-    CGAL_assertion(PMP::remove_isolated_vertices(mesh) == 0);
-    CGAL_assertion(is_valid_polygon_mesh(mesh));
-  } // end loop on patches
-
+  }
   return true;
 }
 
diff -pruN 5.0.3-1/demo/Polyhedron/include/CGAL/Triangulation_file_input.h 5.1-1/demo/Polyhedron/include/CGAL/Triangulation_file_input.h
--- 5.0.3-1/demo/Polyhedron/include/CGAL/Triangulation_file_input.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/demo/Polyhedron/include/CGAL/Triangulation_file_input.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,103 +0,0 @@
-// Copyright (c) 1997-2010  INRIA Sophia-Antipolis (France).
-// Copyright (c) 2011       GeometryFactory Sarl (France)
-// All rights reserved.
-//
-// This file is part of CGAL (www.cgal.org); you may redistribute it under
-// the terms of the Q Public License version 1.0.
-// See the file LICENSE.QPL distributed with CGAL.
-//
-// Licensees holding a valid commercial license may use this file in
-// accordance with the commercial license agreement provided with the software.
-//
-// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
-//
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron/demo/Polyhedron/include/CGAL/Triangulation_file_input.h $
-// $Id: Triangulation_file_input.h 590612a 2020-03-26T13:25:16+01:00 Sébastien Loriot
-//
-// Author(s)     : Laurent Rineau
-//
-
-// Adapted from operator>>(std::istream&, Triangulation_3&) from
-// <CGAL/Triangulation_3.h>
-
-#ifndef CGAL_TRIANGULATION_FILE_INPUT_3_H
-#define CGAL_TRIANGULATION_FILE_INPUT_3_H
-
-#include <CGAL/basic.h>
-
-namespace CGAL {
-
-template <typename Tr1,
-          typename Tr2,
-          typename Update_vertex,
-          typename Update_cell>
-std::istream& file_input(std::istream& is, Tr2 &tr,
-                         Update_vertex update_vertex = Update_vertex(),
-                         Update_cell update_cell = Update_cell())
-  // reads
-  // the dimension
-  // the number of finite vertices
-  // the non combinatorial information on vertices (point, etc)
-  // the number of cells
-  // the cells by the indices of their vertices in the preceding list
-  // of vertices, plus the non combinatorial information on each cell
-  // the neighbors of each cell by their index in the preceding list of cells
-  // when dimension < 3 : the same with faces of maximal dimension
-{
-  typedef Tr2 Triangulation;
-  typedef typename Triangulation::Vertex_handle  Vertex_handle;
-  typedef typename Triangulation::Cell_handle    Cell_handle;
-
-  typedef typename Tr1::Vertex Vertex1;
-  typedef typename Tr1::Cell Cell1;
-
-  tr.clear();
-  tr.tds().cells().clear();
-
-  std::size_t n;
-  int d;
-  if(is_ascii(is))
-     is >> d >> n;
-  else {
-    read(is, d);
-    read(is, n);
-  }
-  if(!is) return is;
-  tr.tds().set_dimension(d);
-
-  std::vector< Vertex_handle > V(n+1);
-  V[0] = tr.infinite_vertex();
-  // the infinite vertex is numbered 0
-
-  for (std::size_t i=1; i <= n; i++) {
-    V[i] = tr.tds().create_vertex();
-    Vertex1 v;
-    if(!(is >> v)) return is;
-    if(!update_vertex(v, *V[i])) {
-      is.setstate(std::ios_base::failbit);
-      return is;
-    }
-  }
-
-  std::vector< Cell_handle > C;
-
-  std::size_t m;
-  tr.tds().read_cells(is, V, m, C);
-
-  for (std::size_t j=0 ; j < m; j++) {
-    Cell1 c;
-    if(!(is >> c)) return is;
-    if(!update_cell(c, *(C[j]))) {
-      is.setstate(std::ios_base::failbit);
-      return is;
-    }
-  }
-
-  CGAL_triangulation_assertion( tr.is_valid(false) );
-  return is;
-}
-
-} // end namespace CGAL
-
-#endif // CGAL_TRIANGULATION_FILE_INPUT_3_H
diff -pruN 5.0.3-1/demo/Polyhedron/include/CGAL/Use_ssh.h 5.1-1/demo/Polyhedron/include/CGAL/Use_ssh.h
--- 5.0.3-1/demo/Polyhedron/include/CGAL/Use_ssh.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/demo/Polyhedron/include/CGAL/Use_ssh.h	2020-09-08 12:15:27.000000000 +0000
@@ -0,0 +1,38 @@
+#ifndef USE_SSH_H
+#define USE_SSH_H
+#include <libssh/libsshpp.hpp>
+#include <vector>
+
+class QStringList;
+
+namespace CGAL{
+namespace ssh_internal{
+//should be used inside a try/catch(ssh::SshException e)
+
+//give an unitialized session.
+bool establish_ssh_session(ssh_session& session,
+                           const char *user,
+                           const char *server,
+                           const char *pub_key_path,
+                           const char *priv_key_path,
+                           const char *priv_key_password);
+
+bool establish_ssh_session_from_agent(ssh_session& session,
+                                      const char *user,
+                                      const char *server,
+                                      const char *pub_key_path);
+
+void close_connection(ssh_session& session);
+
+bool push_file(ssh_session& session,
+               const char* dest_path,
+               const char* filepath);
+bool pull_file(ssh_session &session,
+               const char *from_path,
+               const char *to_path);
+
+bool explore_the_galaxy(ssh_session &session,
+                        QStringList &files);
+
+}}
+#endif
diff -pruN 5.0.3-1/demo/Polyhedron/include/Point_set_3.h 5.1-1/demo/Polyhedron/include/Point_set_3.h
--- 5.0.3-1/demo/Polyhedron/include/Point_set_3.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/demo/Polyhedron/include/Point_set_3.h	2020-09-08 12:15:27.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron/demo/Polyhedron/include/Point_set_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron/demo/Polyhedron/include/Point_set_3.h $
 // $Id: Point_set_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Polyhedron/include/run_with_qprogressdialog.h 5.1-1/demo/Polyhedron/include/run_with_qprogressdialog.h
--- 5.0.3-1/demo/Polyhedron/include/run_with_qprogressdialog.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/demo/Polyhedron/include/run_with_qprogressdialog.h	2020-09-08 12:15:27.000000000 +0000
@@ -7,12 +7,7 @@
 
 #include "Callback_signaler.h"
 
-#ifdef CGAL_LINKED_WITH_TBB
-typedef CGAL::Parallel_tag Concurrency_tag;
-#else
-typedef CGAL::Sequential_tag Concurrency_tag;
-#endif
-
+typedef CGAL::Parallel_if_available_tag Concurrency_tag;
 
 class Signal_callback
 {
diff -pruN 5.0.3-1/demo/Polyhedron/MainWindow.cpp 5.1-1/demo/Polyhedron/MainWindow.cpp
--- 5.0.3-1/demo/Polyhedron/MainWindow.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/MainWindow.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -1,3 +1,6 @@
+#ifdef CGAL_USE_SSH
+#  include "CGAL/Use_ssh.h"
+#endif
 #include <cmath>
 
 #include "config.h"
@@ -6,7 +9,9 @@
 #include <CGAL/Three/Scene_item.h>
 #include <CGAL/Three/TextRenderer.h>
 #include <CGAL/Three/exceptions.h>
+#include <CGAL/Three/Three.h>
 #include <CGAL/Qt/debug.h>
+#include <CGAL/double.h>
 
 #include <QJsonArray>
 #include <QtDebug>
@@ -42,6 +47,7 @@
 #include <QWidgetAction>
 #include <QJsonArray>
 #include <QSequentialIterable>
+#include <QDir>
 #ifdef QT_SCRIPT_LIB
 #  include <QScriptValue>
 #  ifdef QT_SCRIPTTOOLS_LIB
@@ -58,6 +64,7 @@
 #include "ui_Preferences.h"
 #include "ui_Details.h"
 #include "ui_Statistics_on_item_dialog.h"
+#include "ui_SSH_dialog.h"
 #include "Show_point_dialog.h"
 #include "File_loader_dialog.h"
 #include "Viewer.h"
@@ -69,6 +76,8 @@
 #  include <QScriptEngine>
 #  include <QScriptValue>
 #include "Color_map.h"
+
+
 using namespace CGAL::Three;
 QScriptValue
 myScene_itemToScriptValue(QScriptEngine *engine,
@@ -150,6 +159,7 @@ MainWindow::MainWindow(const QStringList
   // remove the Load Script menu entry, when the demo has not been compiled with QT_SCRIPT_LIB
 #if !defined(QT_SCRIPT_LIB)
   ui->menuBar->removeAction(ui->actionLoadScript);
+  ui->menuBar->removeAction(ui->on_actionLoad_a_Scene_from_a_Script_File);
 #endif
   // Save some pointers from ui, for latter use.
   sceneView = ui->sceneView;
@@ -392,32 +402,25 @@ MainWindow::MainWindow(const QStringList
                                                 objectValue);
     }
   }
+  filterOperations(true);
   // debugger->action(QScriptEngineDebugger::InterruptAction)->trigger();
 #endif
 }
 
 void addActionToMenu(QAction* action, QMenu* menu)
 {
-  bool added = false;
-  for(QAction* it : menu->actions())
-  {
-    QString atxt = action->text().remove("&"),
-        btxt = it->text().remove("&");
-    int i = 0;
-    while(atxt[i] == btxt[i]
-          && i < atxt.size()
-          && i < btxt.size())
-      ++i;
-    bool res = (atxt[i] < btxt[i]);
-    if (res)
-    {
-      menu->insertAction(it, action);
-      added = true;
-      break;
-    }
-  }
-  if(!added)
+  auto actions = menu->actions();
+  auto it = std::lower_bound(actions.begin(), actions.end(),
+                             action->text().remove("&"),
+                             [](QAction* a, QString text) {
+                               return a->text().remove("&").compare(text) < 0;
+                             });
+  if(it == actions.end()) {
     menu->addAction(action);
+  }
+  else {
+    menu->insertAction(*it, action);
+  }
 }
 
 //Recursive function that do a pass over a menu and its sub-menus(etc.) and hide them when they are empty
@@ -481,12 +484,12 @@ void MainWindow::filterOperations(bool)
         menu->removeAction(action);
     }
   }
+
   Q_FOREACH(QAction* action, action_menu_map.keys())
   {
     QMenu* menu = action_menu_map[action];
     addActionToMenu(action, menu);
   }
-
   QString filter=operationSearchBar.text();
   Q_FOREACH(const PluginNamePair& p, plugins) {
     Q_FOREACH(QAction* action, p.first->actions()) {
@@ -756,7 +759,7 @@ void MainWindow::loadPlugins()
     qputenv("PATH", new_path);
 #endif
     Q_FOREACH (QString pluginsDir,
-               env_path.split(separator, QString::SkipEmptyParts)) {
+               env_path.split(separator, CGAL_QT_SKIP_EMPTY_PARTS)) {
       QDir dir(pluginsDir);
       if(dir.isReadable())
         plugins_directories << dir;
@@ -1008,12 +1011,17 @@ void MainWindow::computeViewerBBox(CGAL:
 
   CGAL::qglviewer::Vec bbox_center((xmin+xmax)/2, (ymin+ymax)/2, (zmin+zmax)/2);
 
+  double bbox_diag = CGAL::approximate_sqrt(
+        CGAL::square(xmax - xmin)
+        + CGAL::square(ymax - ymin)
+        + CGAL::square(zmax - zmin));
+
   CGAL::qglviewer::Vec offset(0,0,0);
 
   double l_dist = (std::max)((std::abs)(bbox_center.x - viewer->offset().x),
                              (std::max)((std::abs)(bbox_center.y - viewer->offset().y),
                                         (std::abs)(bbox_center.z - viewer->offset().z)));
-  if((std::log2)(l_dist) > 13.0 )
+  if((std::log2)(l_dist/bbox_diag) > 11.0 )
     for(int i=0; i<3; ++i)
     {
       offset[i] = -bbox_center[i];
@@ -1315,6 +1323,8 @@ void MainWindow::selectSceneItem(int i)
   else {
     QItemSelection s =
         proxyModel->mapSelectionFromSource(scene->createSelection(i));
+    if(s.empty())
+      return;
     QModelIndex mi = proxyModel->mapFromSource(scene->getModelIndexFromId(i).first());
     sceneView->setCurrentIndex(mi);
     sceneView->selectionModel()->select(s,
@@ -1339,7 +1349,8 @@ void MainWindow::selectSceneItems(QList<
     sceneView->setCurrentIndex(i);
     sceneView->selectionModel()->select(s,
                                         QItemSelectionModel::ClearAndSelect);
-    sceneView->scrollTo(s.indexes().first());
+    if(!s.empty())
+      sceneView->scrollTo(s.indexes().first());
   }
 }
 
@@ -1384,10 +1395,7 @@ void MainWindow::removeSceneItemFromSele
 
 void MainWindow::selectAll()
 {
-  QItemSelection s =
-      proxyModel->mapSelectionFromSource(scene->createSelectionAll());
-  sceneView->selectionModel()->select(s,
-                                      QItemSelectionModel::ClearAndSelect);
+  sceneView->selectAll();
 }
 
 int MainWindow::getSelectedSceneItemIndex() const
@@ -1781,12 +1789,12 @@ void MainWindow::updateInfo() {
     CGAL::Bbox_3 bbox = item->bbox();
     if(bbox !=CGAL::Bbox_3())
       item_text += QString("<div>Bounding box: min (%1,%2,%3), max (%4,%5,%6)</div>")
-          .arg(bbox.xmin())
-          .arg(bbox.ymin())
-          .arg(bbox.zmin())
-          .arg(bbox.xmax())
-          .arg(bbox.ymax())
-          .arg(bbox.zmax());
+          .arg(bbox.xmin(),0, 'g', 17)
+          .arg(bbox.ymin(),0, 'g', 17)
+          .arg(bbox.zmin(),0, 'g', 17)
+          .arg(bbox.xmax(),0, 'g', 17)
+          .arg(bbox.ymax(),0, 'g', 17)
+          .arg(bbox.zmax(),0, 'g', 17);
     if(!item_filename.isEmpty()) {
       item_text += QString("<div>File:<i> %1</div>").arg(item_filename);
     }
@@ -1827,6 +1835,7 @@ void MainWindow::readSettings()
     this->default_point_size = settings.value("points_size").toInt();
     this->default_normal_length = settings.value("normals_length").toInt();
     this->default_lines_width = settings.value("lines_width").toInt();
+    setProperty("ws_url", settings.value("ws_server_url").toString());
 }
 
 void MainWindow::writeSettings()
@@ -1901,14 +1910,7 @@ void MainWindow::throw_exception() {
 void MainWindow::on_actionLoadScript_triggered()
 {
 #if defined(QT_SCRIPT_LIB)
-  QString filename = QFileDialog::getOpenFileName(
-        this,
-        tr("Select a script to run..."),
-        ".",
-        "QTScripts (*.js);;All Files (*)");
-  if(filename.isEmpty())
-    return;
-  loadScript(QFileInfo(filename));
+
 #endif
 }
 
@@ -2211,7 +2213,6 @@ void MainWindow::on_actionPreferences_tr
   QDialog dialog(this);
   Ui::PreferencesDialog prefdiag;
   prefdiag.setupUi(&dialog);
-
   float lineWidth[2];
   if(!viewer->isOpenGL_4_3())
     viewer->glGetFloatv(GL_LINE_WIDTH_RANGE, lineWidth);
@@ -2289,6 +2290,11 @@ void MainWindow::on_actionPreferences_tr
     this->s_defaultPSRM = CGAL::Three::Three::modeFromName(text);
   });
 
+  connect(prefdiag.backFrontColor_pushButton, &QPushButton::clicked,
+          this, [](){
+    qobject_cast<Viewer*>(CGAL::Three::Three::activeViewer())->setBackFrontColors();
+  });
+
   std::vector<QTreeWidgetItem*> items;
   QBrush successBrush(Qt::green),
       errorBrush(Qt::red),
@@ -2350,6 +2356,70 @@ void MainWindow::on_actionPreferences_tr
     });
     dialog.exec();
   });
+  connect(prefdiag.sshButton, &QPushButton::clicked,
+          this, [this](){
+    QDialog dialog(this);
+    Ui::SSHDialog sshdiag;
+    sshdiag.setupUi(&dialog);
+
+#ifdef CGAL_USE_SSH
+    sshdiag.userBox->setEnabled(true);
+    sshdiag.serverBox->setEnabled(true);
+    sshdiag.pkBox->setEnabled(true);
+    sshdiag.privkBox->setEnabled(true);
+    sshdiag.userEdit->setText(settings.value("ssh_user", QString()).toString());
+    sshdiag.serverEdit->setText(settings.value("ssh_server", QString()).toString());
+    sshdiag.publicEdit->setText(settings.value("ssh_public_key", QString()).toString());
+    sshdiag.privkEdit->setText(settings.value("ssh_priv_key", QString()).toString());
+    connect(sshdiag.pubButton, &QPushButton::clicked,
+            this, [this, sshdiag](){
+      QFileDialog diag(this,
+                       "Public Key",
+                       "",
+                       "All Files (*)");
+      diag.setFilter(QDir::Hidden|QDir::Files|QDir::Dirs|QDir::NoDotAndDotDot);
+      if(!diag.exec())
+        return;
+      sshdiag.publicEdit->setText(diag.selectedFiles().front());
+    });
+    connect(sshdiag.privButton, &QPushButton::clicked,
+            this, [this, sshdiag](){
+      QFileDialog diag(this,
+                       "Private Key",
+                       "",
+                       "All Files (*)");
+      diag.setFilter(QDir::Hidden|QDir::Files|QDir::Dirs|QDir::NoDotAndDotDot);
+      if(!diag.exec())
+        return;
+      sshdiag.privkEdit->setText(diag.selectedFiles().front());
+    });
+#else
+    sshdiag.userBox->setEnabled(false);
+    sshdiag.serverBox->setEnabled(false);
+    sshdiag.pkBox->setEnabled(false);
+    sshdiag.privkBox->setEnabled(false);
+#endif
+    sshdiag.wsEdit->setText(settings.value("ws_server_url", QString()).toString());
+
+    dialog.exec();
+    if ( dialog.result() )
+    {
+#ifdef CGAL_USE_SSH
+      settings.setValue("ssh_user",
+                        sshdiag.userEdit->text());
+      settings.setValue("ssh_server",
+                        sshdiag.serverEdit->text());
+      settings.setValue("ssh_public_key",
+                        sshdiag.publicEdit->text());
+      settings.setValue("ssh_priv_key",
+                        sshdiag.privkEdit->text());
+#endif
+      settings.setValue("ws_server_url",
+                        sshdiag.wsEdit->text());
+      setProperty("ws_url", sshdiag.wsEdit->text());
+    }
+  });
+
   dialog.exec();
 
   if ( dialog.result() )
@@ -2422,8 +2492,8 @@ void MainWindow::viewerShowObject()
     CGAL::qglviewer::Vec min((float)bbox.xmin()+viewer->offset().x, (float)bbox.ymin()+viewer->offset().y, (float)bbox.zmin()+viewer->offset().z),
         max((float)bbox.xmax()+viewer->offset().x, (float)bbox.ymax()+viewer->offset().y, (float)bbox.zmax()+viewer->offset().z);
     viewer->setSceneBoundingBox(min, max);
-    viewerShow(min.x, min.y, min.z,
-               max.x, max.y, max.z);
+    viewerShow((float)min.x, (float)min.y, (float)min.z,
+               (float)max.x, (float)max.y, (float)max.z);
   }
 }
 /* to check
@@ -2791,52 +2861,169 @@ void MainWindow::propagate_action()
   }
 }
 
+QString make_fullpath(const QString& filename, bool duplicate = false)
+{
+  QString fullpath = QString("%1/%2").arg(QDir::tempPath()).arg(filename);
+  QString tmp_fullpath = fullpath;
+  if(duplicate)
+  {
+    int i=0;
+    while(QFileInfo(tmp_fullpath).exists())
+    {
+      QString basename = QFileInfo(tmp_fullpath).baseName();
+      QString dir = QFileInfo(tmp_fullpath).dir().path();
+      QString suffix= QFileInfo(fullpath).completeSuffix();
+      tmp_fullpath=QString("%1/%2%3.%4").arg(dir).arg(basename).arg(++i).arg(suffix);
+    }
+  }
+  return tmp_fullpath;
+}
+/*
+ The two following functions allow to create files from string and strings from files.
+ This is used as a workaround of the absence of stream management in our IO system.
+ The whole to/from Base64 is used to avoid problems with binary formats. Everything is written
+ as a base64 binary string, and converted back to what it was.
+*/
+QByteArray file_to_string(const char* filename)
+{
+  std::ifstream f(filename, std::ifstream::binary);
+  // get size of file
+  f.seekg (0,f.end);
+  long size = f.tellg();
+  f.seekg (0);
+  std::ostringstream ss;
+  // allocate memory for file content
+    char* buffer = new char[size];
+
+    // read content of infile
+    f.read(buffer,size);
+
+    // write to outfile
+    ss.write(buffer,size);
+    // release dynamically-allocated memory
+    delete[] buffer;
+  //ss.write( << f.rdbuf(); // reading data
+  f.close();
+  std::string st = ss.str();
+  QByteArray ba(st.c_str(), static_cast<int>(st.size()));
+  return ba;
+}
+
+QString MainWindow::write_string_to_file(const QString& str, const QString &filename)
+{
+  QString fullpath = make_fullpath(filename);
+  std::ofstream f(fullpath.toStdString().c_str(), std::ofstream::binary);
+  QByteArray compressed_item(str.toStdString().c_str());
+  QByteArray item = qUncompress(QByteArray::fromBase64(compressed_item));
+  QByteArray bb = item;
+  f.write(bb.constData(),bb.toStdString().size());
+  f.close();
+  return fullpath;
+}
+
 void MainWindow::on_actionSa_ve_Scene_as_Script_triggered()
 {
-  QString filename =
-      QFileDialog::getSaveFileName(this,
-                                   "Save the Scene as a Script File",
-                                   last_saved_dir,
-                                   "Qt Script files (*.js)");
-  std::ofstream os(filename.toUtf8());
+  if(scene->numberOfEntries() == 0)
+    return;
+  bool do_upload = false;
+#ifdef CGAL_USE_SSH
+  QString user = settings.value("ssh_user", QString()).toString();
+  if(!user.isEmpty())
+  {
+    QMessageBox::StandardButton doyou =
+        QMessageBox::question(this, tr("Upload ?"), tr("Do you wish to upload the scene"
+                                                       " using the SSH preferences ?"));
+    do_upload = (doyou == QMessageBox::Yes);
+  }
+#endif
+
+  QString filename;
+
+  if(do_upload){
+    filename = QString("%1/save_scene.js").arg(QDir::tempPath());
+  }else{
+    filename = QFileDialog::getSaveFileName(this,
+                                            "Save the Scene as a Script File",
+                                            last_saved_dir,
+                                            "Qt Script files (*.js)");
+  }
+  std::ofstream os(filename.toUtf8(), std::ofstream::binary);
   if(!os)
     return;
-  std::vector<QString> names;
-  std::vector<QString> loaders;
+  CGAL::Three::Three::CursorScopeGuard cs(Qt::WaitCursor);
+  std::vector<std::pair<QString, QString> > names;
+  std::vector<std::pair<QString, QString> > loaders;
   std::vector<QColor> colors;
   std::vector<int> rendering_modes;
   QStringList not_saved;
   for(int i = 0; i < scene->numberOfEntries(); ++i)
   {
     Scene_item* item = scene->item(i);
-    QString loader = item->property("loader_name").toString();
-    QString source = item->property("source filename").toString();
+    QString loader;// = item->property("loader_name").toString();
+    QString ext;
+    for(Polyhedron_demo_io_plugin_interface* iop : io_plugins)
+    {
+      if(iop->isDefaultLoader(item))
+      {
+        QString sf = iop->saveNameFilters().split(";;").first();
+        //OFF Files (*.off)
+        QRegularExpression re("\\(\\*\\.(.*)\\)");
+        QRegularExpressionMatch rem = re.match(sf);
+        if(!rem.hasMatch())
+          continue;
+        ext = rem.captured(1);
+        QList<Scene_item*>to_save;
+        to_save.append(item);
+        QString savename(tr("%1.%2").arg(item->name()).arg(ext));
+        QString fullpath = make_fullpath(savename, true);
+        savename = QFileInfo(fullpath).fileName();
+        iop->save(QFileInfo(fullpath), to_save);
+        names.push_back(std::make_pair(savename, item->name()));
+        loader=iop->name();
+        break;
+      }
+    }
     if(loader.isEmpty())
     {
-      not_saved.push_back(item->name());
+      QMessageBox::warning(this, "", tr("No plugin found for %1. Not saved.").arg(item->name()));
       continue;
     }
-    names.push_back(source);
-    loaders.push_back(loader);
+    loaders.push_back(std::make_pair(loader, ext));
     colors.push_back(item->color());
     rendering_modes.push_back(item->renderingMode());
   }
+  if(loaders.empty())
+    return;
   //path
   os << "var camera = \""<<viewer->dumpCameraCoordinates().toStdString()<<"\";\n";
   os << "var items = [";
   for(std::size_t i = 0; i< names.size() -1; ++i)
   {
-    os << "\'" << names[i].toStdString() << "\', ";
-  }
-  os<<"\'"<<names.back().toStdString()<<"\'];\n";
+    QString fullpath = make_fullpath(names[i].first);
 
+    QByteArray item = file_to_string(fullpath.toStdString().c_str());
+    os<<"[\'";
+    os<<qCompress(item, 9).toBase64().toStdString().c_str();
+    os << "\', \'"<<names[i].second.toStdString().c_str()<<"\']," ;
+    //delete temp file
+    QFile tmp_file(fullpath);
+    tmp_file.remove();
+  }
+  QString fullpath = make_fullpath(names.back().first);
+  QByteArray item = file_to_string(fullpath.toStdString().c_str());
+  os<<"[\'";
+  os<<qCompress(item, 9).toBase64().toStdString().c_str();
+  os << "\', \'"<<names.back().second.toStdString().c_str()<<"\']];\n";
+  //delete temp file
+  QFile tmp_file(fullpath);
+  tmp_file.remove();
   //plugin
   os << "var loaders = [";
   for(std::size_t i = 0; i< names.size() -1; ++i)
   {
-    os << "\'" << loaders[i].toStdString() << "\', ";
+    os << "[\'" << loaders[i].first.toStdString() << "\', \'"<<loaders[i].second.toStdString()<< "\'],";
   }
-  os<<"\'"<<loaders.back().toStdString()<<"\'];\n";
+  os << "[\'" << loaders.back().first.toStdString() << "\', \'"<<loaders.back().second.toStdString()<< "\']];\n";
 
   //color
   os << "var colors = [";
@@ -2853,10 +3040,13 @@ void MainWindow::on_actionSa_ve_Scene_as
     os << rendering_modes[i] << ", ";
   }
   os << rendering_modes.back()<<"];\n";
-  os <<"var initial_scene_size = scene.numberOfEntries;\n";
   os << "items.forEach(function(item, index, array){\n";
-  os << "        main_window.open(item, loaders[index]);\n";
-  os << "        var it = scene.item(initial_scene_size+index);\n";
+  os<<"          var path=items[index][1];\n";
+  os<<"          path+='.';\n";
+  os<<"          path+=loaders[index][1];\n";
+  os<<"          var fullpath = main_window.write_string_to_file(item[0], path);\n";
+  os<<"          main_window.open(fullpath,loaders[index][0]);\n";
+  os << "        var it = scene.item(scene.numberOfEntries-1);\n";
   os << "        var r = colors[index][0];\n";
   os << "        var g = colors[index][1];\n";
   os << "        var b = colors[index][2];\n";
@@ -2870,6 +3060,81 @@ void MainWindow::on_actionSa_ve_Scene_as
                          "Items Not  Saved",
                          QString("The following items could not be saved: %1").arg(
                            not_saved.join(", ")));
+#ifdef CGAL_USE_SSH
+  using namespace CGAL::ssh_internal;
+  if(do_upload)
+  {
+    QString server = settings.value("ssh_server", QString()).toString();
+    QString pk = settings.value("ssh_public_key", QString()).toString();
+    QString privK = settings.value("ssh_priv_key", QString()).toString();
+    user = user.trimmed();
+    server = server.trimmed();
+    pk = pk.trimmed();
+    privK=privK.trimmed();
+    if(user.isEmpty()){
+      return;
+    }
+    QString path;
+    path = QInputDialog::getText(this,
+                                 "",
+                                 tr("Enter the name of your scene file."));
+    if(path.isEmpty())
+      return;
+    if(!path.contains("Polyhedron_demo_"))
+      path.prepend("Polyhedron_demo_");
+    try{
+      ssh_session session;
+      bool res = establish_ssh_session_from_agent(session,
+                                                  user.toStdString().c_str(),
+                                                  server.toStdString().c_str(),
+                                                  pk.toStdString().c_str());
+
+      if(!res)
+      {
+        bool ok;
+        QString pass;
+        pass = QInputDialog::getText(this, "SSH Password",
+                                     "Enter ssh key password:",
+                                     QLineEdit::Password,
+                                     tr(""),
+                                     &ok);
+        if(!ok)
+          return;
+        pass = pass.trimmed();
+        res = establish_ssh_session(session,
+                                    user.toStdString().c_str(),
+                                    server.toStdString().c_str(),
+                                    pk.toStdString().c_str(),
+                                    privK.toStdString().c_str(),
+                                    pass.toStdString().c_str());
+      }
+
+      if(!res)
+      {
+        QMessageBox::warning(this,
+                             "Error",
+                             "The SSH session could not be started.");
+        return;
+      }
+      res = push_file(session,path.toStdString().c_str(), filename.toStdString().c_str());
+      if(!res)
+      {
+        QMessageBox::warning(this,
+                             "Error",
+                             "The file could not be uploaded. Check your console for more information.");
+        close_connection(session);
+        return;
+      }
+      close_connection(session);
+      QFile tmp_file(filename);
+      tmp_file.remove();
+    } catch( ssh::SshException e )
+    {
+      std::cout << "Error during connection : ";
+      std::cout << e.getError() << std::endl;
+    }
+  }
+#endif
 }
 void MainWindow::setTransparencyPasses(int val)
 {
@@ -2973,6 +3238,9 @@ void MainWindow::setupViewer(Viewer* vie
     }
     viewer->setTotalPass(nb);
   });
+  action= subviewer->findChild<QAction*>("actionBackFrontShading");
+  connect(action, SIGNAL(toggled(bool)),
+          viewer, SLOT(setBackFrontShading(bool)));
   connect(viewer, SIGNAL(requestContextMenu(QPoint)),
           this, SLOT(contextMenuRequested(QPoint)));
   connect(viewer, SIGNAL(selected(int)),
@@ -2990,6 +3258,33 @@ void MainWindow::setupViewer(Viewer* vie
     information(s);
   });
 
+#ifdef CGAL_USE_WEBSOCKETS
+  action= subviewer->viewer->findChild<QAction*>("actionShareCamera");
+  connect(action, &QAction::toggled,
+          this, [this, viewer](bool b)
+  {
+    if(!viewer){
+      return;
+    }
+    QString session;
+    if(b){
+      bool ok;
+      session = QInputDialog::getText(
+            this,"Session",
+            "Please enter the session name.\n"
+            "Only the machines that enter the same session name will be connected.\n"
+            "Several sessions can run simultaneously on a same server. ",
+            QLineEdit::Normal, QString(), &ok);
+      if(session.isEmpty() || !ok)
+      {
+        viewer->setShareCam(false, session);
+        return;
+      }
+    }
+    viewer->setShareCam(b, session);
+  });
+#endif
+
 }
 
 void MainWindow::on_actionAdd_Viewer_triggered()
@@ -3141,6 +3436,20 @@ SubViewer::SubViewer(QWidget *parent, Ma
   QAction* actionTotalPass = new QAction("Set Transparency Pass &Number...",this);
   actionTotalPass->setObjectName("actionTotalPass");
   viewMenu->addAction(actionTotalPass);
+#ifdef CGAL_USE_WEBSOCKETS
+  QAction* actionShareCamera= new QAction("Join &WS Server",viewer);
+  actionShareCamera->setObjectName("actionShareCamera");
+  actionShareCamera->setCheckable(true);
+  actionShareCamera->setChecked(false);
+  viewMenu->addAction(actionShareCamera);
+#endif
+
+  QAction* actionBackFrontShading = new QAction("Activate Back/Front shading.",this);
+  actionBackFrontShading->setObjectName("actionBackFrontShading");
+  actionBackFrontShading->setCheckable(true);
+  actionBackFrontShading->setChecked(false);
+  viewMenu->addAction(actionBackFrontShading);
+
   if(mainviewer)
     setAttribute(Qt::WA_DeleteOnClose);
   setWindowIcon(QIcon(":/cgal/icons/resources/menu.png"));
@@ -3267,3 +3576,113 @@ void MainWindow::on_action_Save_triggere
     }
   }
 }
+
+void MainWindow::on_actionLoad_a_Scene_from_a_Script_File_triggered()
+{
+  bool do_download = false;
+  QString filename;
+
+#ifdef CGAL_USE_SSH
+  QString user = settings.value("ssh_user", QString()).toString();
+
+  if(!user.isEmpty())
+  {
+    QMessageBox::StandardButton doyou =
+        QMessageBox::question(this, tr("Download ?"), tr("Do you wish to download the scene"
+                                                         " using the SSH preferences ?"));
+    do_download= (doyou == QMessageBox::Yes);
+  }
+#endif
+
+  if(do_download)
+  {
+    #ifdef CGAL_USE_SSH
+    using namespace CGAL::ssh_internal;
+    QString server = settings.value("ssh_server", QString()).toString();
+    QString pk = settings.value("ssh_public_key", QString()).toString();
+    QString privK = settings.value("ssh_priv_key", QString()).toString();
+    user = user.trimmed();
+    server = server.trimmed();
+    pk = pk.trimmed();
+    privK=privK.trimmed();
+
+    try{
+      ssh_session session;
+      bool res = establish_ssh_session_from_agent(session,
+                                                  user.toStdString().c_str(),
+                                                  server.toStdString().c_str(),
+                                                  pk.toStdString().c_str());
+      if(!res){
+        bool ok;
+        QString pass= QInputDialog::getText(this, "SSH Password",
+                                     "Enter ssh key password:",
+                                     QLineEdit::Password,
+                                     tr(""),
+                                     &ok);
+        if(!ok)
+          return;
+        pass = pass.trimmed();
+        res = establish_ssh_session(session,
+                                    user.toStdString().c_str(),
+                                    server.toStdString().c_str(),
+                                    pk.toStdString().c_str(),
+                                    privK.toStdString().c_str(),
+                                    pass.toStdString().c_str());
+      }
+      if(!res)
+      {
+        QMessageBox::warning(this,
+                             "Error",
+                             "The SSH session could not be started.");
+        return;
+      }
+      QStringList names;
+      if(!CGAL::ssh_internal::explore_the_galaxy(session, names))
+      {
+        QMessageBox::warning(this,
+                             "Error",
+                             "Could not find remote directory.");
+      }
+      QString path;
+      path = QInputDialog::getItem(this,
+                                   "Choose a file",
+                                   tr("Choose the scene file."),
+                                   names);
+      filename = QString("%1/load_scene.js").arg(QDir::tempPath());
+      if(path.isEmpty())
+        return;
+      path.prepend("Polyhedron_demo_");
+      path = tr("/tmp/%2").arg(path);
+      res = pull_file(session,path.toStdString().c_str(), filename.toStdString().c_str());
+      if(!res)
+      {
+        QMessageBox::warning(this,
+                             "Error",
+                             "The file could not be fetched. Check your console for more info.");
+        close_connection(session);
+        return;
+      }
+      close_connection(session);
+    } catch( ssh::SshException e )
+    {
+      std::cout << "Error during connection : ";
+      std::cout << e.getError() << std::endl;
+    }
+    #endif
+  }
+  else
+  {
+    filename =  QFileDialog::getOpenFileName(
+          this,
+          tr("Select a Whole Scene file..."),
+          ".",
+          "Whole Scene files (*.js)");
+    if(filename.isEmpty())
+      return;
+  }
+  loadScript(QFileInfo(filename));
+  if(do_download){
+    QFile tmp_file(filename);
+    tmp_file.remove();
+  }
+}
diff -pruN 5.0.3-1/demo/Polyhedron/MainWindow.h 5.1-1/demo/Polyhedron/MainWindow.h
--- 5.0.3-1/demo/Polyhedron/MainWindow.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/demo/Polyhedron/MainWindow.h	2020-09-08 12:15:24.000000000 +0000
@@ -138,6 +138,8 @@ public Q_SLOTS:
    This slot is for use by scripts.*/
   bool open(QString filename, QString loader_name);
 
+  QString write_string_to_file(const QString &str, const QString& filename);
+
   /*! Reloads an item. Expects to be called by a QAction with the
    index of the item to be reloaded as data attached to the action.
    The index must identify a valid `Scene_item`.*/
@@ -447,6 +449,7 @@ public:
 #endif
 public Q_SLOTS:
   void on_actionSa_ve_Scene_as_Script_triggered();
+  void on_actionLoad_a_Scene_from_a_Script_File_triggered();
   void toggleFullScreen();
   void setDefaultSaveDir();
   void invalidate_bbox(bool do_recenter);
@@ -494,4 +497,5 @@ protected:
 private:
   bool is_main;
 };
+
 #endif // ifndef MAINWINDOW_H
diff -pruN 5.0.3-1/demo/Polyhedron/MainWindow.ui 5.1-1/demo/Polyhedron/MainWindow.ui
--- 5.0.3-1/demo/Polyhedron/MainWindow.ui	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/MainWindow.ui	2020-09-08 12:12:20.000000000 +0000
@@ -58,6 +58,7 @@
     <addaction name="action_Save"/>
     <addaction name="actionSaveAs"/>
     <addaction name="actionSa_ve_Scene_as_Script"/>
+    <addaction name="actionLoad_a_Scene_from_a_Script_File"/>
     <addaction name="actionSaveSnapshot"/>
     <addaction name="separator"/>
     <addaction name="actionLoadScript"/>
@@ -463,6 +464,11 @@
     <string>&amp;Save</string>
    </property>
   </action>
+  <action name="actionLoad_a_Scene_from_a_Script_File">
+   <property name="text">
+    <string>Load a Scene &amp;from a Script File...</string>
+   </property>
+  </action>
  </widget>
  <resources>
   <include location="Polyhedron_3.qrc"/>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/AABB_tree/CMakeLists.txt 5.1-1/demo/Polyhedron/Plugins/AABB_tree/CMakeLists.txt
--- 5.0.3-1/demo/Polyhedron/Plugins/AABB_tree/CMakeLists.txt	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/AABB_tree/CMakeLists.txt	2020-09-08 12:12:20.000000000 +0000
@@ -5,6 +5,6 @@ target_link_libraries(do_trees_intersect
 
 polyhedron_demo_plugin(cut_plugin Cut_plugin )
 target_link_libraries(cut_plugin PUBLIC scene_surface_mesh_item scene_basic_objects scene_color_ramp)
-if(TBB_FOUND)
-  CGAL_target_use_TBB(cut_plugin)
+if(TARGET CGAL::TBB_support)
+  target_link_libraries(cut_plugin PUBLIC CGAL::TBB_support)
 endif()
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/AABB_tree/Cut_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/AABB_tree/Cut_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/AABB_tree/Cut_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/AABB_tree/Cut_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -860,6 +860,12 @@ public:
     return ok;
   }
 
+  bool isDefaultLoader(const Scene_item* item) const Q_DECL_OVERRIDE{
+    if(qobject_cast<const Scene_edges_item*>(item))
+      return true;
+    return false;
+  }
+
   using Polyhedron_demo_io_plugin_interface::init;
   void init(QMainWindow* mainWindow, CGAL::Three::Scene_interface* scene_interface,
             Messages_interface* m) override;
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Classification/Classification_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Classification/Classification_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Classification/Classification_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Classification/Classification_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -38,7 +38,7 @@
 #include <QInputDialog>
 #include <QMessageBox>
 #include <QSpinBox>
-#include <QDoubleSpinBox>
+#include "CGAL_double_edit.h"
 #include <QSlider>
 
 #include <map>
@@ -619,10 +619,7 @@ public Q_SLOTS:
     scales->setRange (1, 99);
     scales->setValue (5);
 
-    QDoubleSpinBox* voxel_size = dialog.add<QDoubleSpinBox> ("Voxel size (0 for automatic):");
-    voxel_size->setRange (0.0, 10000.0);
-    voxel_size->setValue (0.0);
-    voxel_size->setSingleStep (0.01);
+    DoubleEdit* voxel_size = dialog.add<DoubleEdit> ("Voxel size (0 for automatic):");
 
     if (dialog.exec() != QDialog::Accepted)
       return;
@@ -921,10 +918,9 @@ public Q_SLOTS:
     subdivisions->setRange (1, 9999);
     subdivisions->setValue (16);
 
-    QDoubleSpinBox* smoothing = dialog.add<QDoubleSpinBox> ("Regularization weight: ");
-    smoothing->setRange (0.0, 100.0);
-    smoothing->setValue (0.5);
-    smoothing->setSingleStep (0.1);
+    DoubleEdit* smoothing = dialog.add<DoubleEdit> ("Regularization weight: ");
+
+    smoothing->setText(tr("%1").arg(0.5));
 
     if (dialog.exec() != QDialog::Accepted)
       return;
@@ -1371,10 +1367,9 @@ public Q_SLOTS:
       QSpinBox* trials = dialog.add<QSpinBox> ("Number of trials: ", "trials");
       trials->setRange (1, 99999);
       trials->setValue (500);
-      QDoubleSpinBox* rate = dialog.add<QDoubleSpinBox> ("Learning rate: ", "learning_rate");
+      DoubleEdit* rate = dialog.add<DoubleEdit> ("Learning rate: ", "learning_rate");
       rate->setRange (0.00001, 10000.0);
       rate->setValue (0.001);
-      rate->setDecimals (5);
       QSpinBox* batch = dialog.add<QSpinBox> ("Batch size: ", "batch_size");
       batch->setRange (1, 2000000000);
       batch->setValue (1000);
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Classification/Cluster_classification.cpp 5.1-1/demo/Polyhedron/Plugins/Classification/Cluster_classification.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Classification/Cluster_classification.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Classification/Cluster_classification.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -910,7 +910,7 @@ void Cluster_classification::train(int c
     m_neural_network->train (training,
                              dialog.get<QCheckBox>("restart")->isChecked(),
                              dialog.get<QSpinBox>("trials")->value(),
-                             dialog.get<QDoubleSpinBox>("learning_rate")->value(),
+                             dialog.get<DoubleEdit>("learning_rate")->value(),
                              dialog.get<QSpinBox>("batch_size")->value(),
                              hidden_layers);
 
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Classification/Cluster_classification.h 5.1-1/demo/Polyhedron/Plugins/Classification/Cluster_classification.h
--- 5.0.3-1/demo/Polyhedron/Plugins/Classification/Cluster_classification.h	2020-08-01 19:02:42.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Classification/Cluster_classification.h	2020-09-08 12:15:24.000000000 +0000
@@ -15,12 +15,7 @@
 
 #include <iostream>
 
-#ifdef CGAL_LINKED_WITH_TBB
-typedef CGAL::Parallel_tag Concurrency_tag;
-#else
-typedef CGAL::Sequential_tag Concurrency_tag;
-#endif
-
+typedef CGAL::Parallel_if_available_tag Concurrency_tag;
 
 class Cluster_classification : public Item_classification_base
 {
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Classification/CMakeLists.txt 5.1-1/demo/Polyhedron/Plugins/Classification/CMakeLists.txt
--- 5.0.3-1/demo/Polyhedron/Plugins/Classification/CMakeLists.txt	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Classification/CMakeLists.txt	2020-09-08 12:12:20.000000000 +0000
@@ -1,76 +1,61 @@
 include( polyhedron_demo_macros )
 
-if(EIGEN3_FOUND)
+if(TARGET CGAL::Eigen_support)
 
-  set(classification_linked_libraries)
-  set(classification_compile_definitions)
+  set(Classification_dependencies_met TRUE)
 
-  find_package(Boost OPTIONAL_COMPONENTS serialization iostreams)
+  find_package( Boost OPTIONAL_COMPONENTS serialization iostreams )
+  include(CGAL_Boost_serialization_support)
+  include(CGAL_Boost_iostreams_support)
+  if (NOT TARGET CGAL::Boost_serialization_support)
+    message(STATUS "NOTICE: Boost Serialization not found. Classification plugin won't be available.")
+    set(Classification_dependencies_met FALSE)
+  endif()
 
-  if( WIN32 )
-# to avoid a warning with old cmake
-    set(_Boost_BZIP2_HEADERS             "boost/iostreams/filter/bzip2.hpp")
-    set(_Boost_ZLIB_HEADERS              "boost/iostreams/filter/zlib.hpp")
-    find_package( Boost OPTIONAL_COMPONENTS serialization iostreams bzip2 zlib)
-    if( Boost_ZLIB_FOUND AND Boost_BZIP2_FOUND)
-      if(TARGET Boost::bzip2 AND TARGET Boost::zlib)
-        set(classification_linked_libraries ${classification_linked_libraries}
-          Boost::bzip2 Boost::zlib)
-      else()
-        set(classification_linked_libraries ${classification_linked_libraries}
-          ${Boost_ZLIB_LIBRARY} ${Boost_BZIP2_LIBRARY})
-      endif()
-    endif()
+  if (NOT TARGET CGAL::Boost_iostreams_support)
+    message(STATUS "NOTICE: Boost IOStreams not found. Classification plugin won't be available.")
+    set(Classification_dependencies_met FALSE)
+  endif()
+
+  find_package(OpenCV QUIET COMPONENTS core ml) # Need core + machine learning
+  include(CGAL_OpenCV_support)
+  if (NOT TARGET CGAL::OpenCV_support)
+    message(STATUS "NOTICE: OpenCV was not found. OpenCV random forest predicate for classification won't be available.")
+  endif()
+
+  find_package(TensorFlow QUIET)
+  include(CGAL_TensorFlow_support)
+  if (NOT TARGET CGAL::TensorFlow_support)
+    message(STATUS "NOTICE: TensorFlow not found, Neural Network predicate for classification won't be available.")
   endif()
 
-  if (Boost_SERIALIZATION_FOUND AND Boost_IOSTREAMS_FOUND AND (NOT WIN32 OR Boost_ZLIB_FOUND))
+  if (Classification_dependencies_met)
     qt5_wrap_ui( classificationUI_FILES Classification_widget.ui Classification_advanced_widget.ui )
     polyhedron_demo_plugin(classification_plugin Classification_plugin Point_set_item_classification Cluster_classification Surface_mesh_item_classification ${classificationUI_FILES} KEYWORDS Classification)
+    target_link_libraries(classification_plugin PUBLIC
+      scene_points_with_normal_item
+      scene_polylines_item
+      scene_polygon_soup_item
+      scene_surface_mesh_item
+      scene_selection_item
+      scene_color_ramp
+      CGAL::Eigen_support
+      CGAL::Boost_serialization_support
+      CGAL::Boost_iostreams_support)
 
-    if(TARGET Boost::serialization AND TARGET Boost::iostreams)
-      set(classification_linked_libraries ${classification_linked_libraries}
-        Boost::serialization Boost::iostreams)
-    else()
-      set(classification_linked_libraries ${classification_linked_libraries}
-        ${Boost_SERIALIZATION_LIBRARY}
-        ${Boost_IOSTREAMS_LIBRARY})
+    if(OpenCV_FOUND)
+      target_link_libraries(classification_plugin PUBLIC CGAL::OpenCV_support)
     endif()
-
-    find_package(OpenCV QUIET COMPONENTS core ml) # Need core + machine learning
-    if (OpenCV_FOUND)
-      message(STATUS "Found OpenCV ${OpenCV_VERSION}")
-      include_directories(${OpenCV_INCLUDE_DIRS})
-      set(classification_linked_libraries ${classification_linked_libraries}
-        ${OpenCV_LIBS})
-      set(classification_compile_definitions ${classification_compile_definitions}
-        "-DCGAL_LINKED_WITH_OPENCV")
-    else()
-      message(STATUS "NOTICE: OpenCV was not found. OpenCV random forest predicate for classification won't be available.")
+    if(TensorFlow_FOUND)
+      target_link_libraries(classification_plugin PUBLIC CGAL::TensorFlow_support)
     endif()
-
-    find_package(TensorFlow QUIET)
-    if (TensorFlow_FOUND)
-      message(STATUS "Found TensorFlow")
-      set(classification_linked_libraries ${classification_linked_libraries}
-        ${TensorFlow_LIBRARY})
-      set(classification_compile_definitions ${classification_compile_definitions}
-        "-DCGAL_LINKED_WITH_TENSORFLOW")
-      include_directories( ${TensorFlow_INCLUDE_DIR} )
-    else()
-      message(STATUS "NOTICE: TensorFlow not found, Neural Network predicate for classification won't be available.")
+    if(TBB_FOUND)
+      target_link_libraries(classification_plugin PUBLIC CGAL::TBB_support)
     endif()
 
-    target_link_libraries(classification_plugin PUBLIC scene_points_with_normal_item
-      scene_polylines_item scene_polygon_soup_item scene_surface_mesh_item scene_selection_item scene_color_ramp ${classification_linked_libraries})
     add_dependencies(classification_plugin point_set_selection_plugin selection_plugin)
-    target_compile_definitions(classification_plugin PUBLIC ${classification_compile_definitions})
-  else()
-    message(STATUS "NOTICE: Boost Serialization or IO Streams or ZLIB not found. Classification plugin won't be available.")
   endif()
 
-
-else(EIGEN3_FOUND)
+else()
   message(STATUS "NOTICE: Eigen 3.1 (or greater) was not found. Classification plugin will not be available.")
 endif()
-
-
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Classification/Point_set_item_classification.cpp 5.1-1/demo/Polyhedron/Plugins/Classification/Point_set_item_classification.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Classification/Point_set_item_classification.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Classification/Point_set_item_classification.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -800,7 +800,7 @@ void Point_set_item_classification::trai
     m_neural_network->train (training,
                              dialog.get<QCheckBox>("restart")->isChecked(),
                              dialog.get<QSpinBox>("trials")->value(),
-                             dialog.get<QDoubleSpinBox>("learning_rate")->value(),
+                             dialog.get<DoubleEdit>("learning_rate")->value(),
                              dialog.get<QSpinBox>("batch_size")->value(),
                              hidden_layers);
 
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Classification/Point_set_item_classification.h 5.1-1/demo/Polyhedron/Plugins/Classification/Point_set_item_classification.h
--- 5.0.3-1/demo/Polyhedron/Plugins/Classification/Point_set_item_classification.h	2020-08-01 19:02:42.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Classification/Point_set_item_classification.h	2020-09-08 12:15:24.000000000 +0000
@@ -16,13 +16,7 @@
 
 #include <iostream>
 
-#ifdef CGAL_LINKED_WITH_TBB
-typedef CGAL::Parallel_tag Concurrency_tag;
-#else
-typedef CGAL::Sequential_tag Concurrency_tag;
-#endif
-
-
+typedef CGAL::Parallel_if_available_tag Concurrency_tag;
 
 // This class represents a point set in the OpenGL scene
 class Point_set_item_classification : public Item_classification_base
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Classification/Surface_mesh_item_classification.cpp 5.1-1/demo/Polyhedron/Plugins/Classification/Surface_mesh_item_classification.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Classification/Surface_mesh_item_classification.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Classification/Surface_mesh_item_classification.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -366,7 +366,7 @@ void Surface_mesh_item_classification::t
     m_neural_network->train (training,
                              dialog.get<QCheckBox>("restart")->isChecked(),
                              dialog.get<QSpinBox>("trials")->value(),
-                             dialog.get<QDoubleSpinBox>("learning_rate")->value(),
+                             dialog.get<DoubleEdit>("learning_rate")->value(),
                              dialog.get<QSpinBox>("batch_size")->value(),
                              hidden_layers);
 
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Classification/Surface_mesh_item_classification.h 5.1-1/demo/Polyhedron/Plugins/Classification/Surface_mesh_item_classification.h
--- 5.0.3-1/demo/Polyhedron/Plugins/Classification/Surface_mesh_item_classification.h	2020-08-01 19:02:42.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Classification/Surface_mesh_item_classification.h	2020-09-08 12:15:24.000000000 +0000
@@ -16,11 +16,7 @@
 
 #include <iostream>
 
-#ifdef CGAL_LINKED_WITH_TBB
-typedef CGAL::Parallel_tag Concurrency_tag;
-#else
-typedef CGAL::Sequential_tag Concurrency_tag;
-#endif
+typedef CGAL::Parallel_if_available_tag Concurrency_tag;
 
 class Surface_mesh_item_classification : public Item_classification_base
 {
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Convex_hull/Convex_hull_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Convex_hull/Convex_hull_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Convex_hull/Convex_hull_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Convex_hull/Convex_hull_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -80,28 +80,15 @@ void Polyhedron_demo_convex_hull_plugin:
 
     // add convex hull as new polyhedron
     SMesh *pConvex_hull  = new SMesh;
-    typedef boost::property_map<SMesh,CGAL::vertex_point_t>::type Vpmap;
-    typedef CGAL::Property_map_to_unary_function<Vpmap> Vpmap_fct;
     if(selection_item) {
       SMesh* pMesh = selection_item->polyhedron();
-      Vpmap vpm = get(CGAL::vertex_point,*pMesh);
-
-      Vpmap_fct v2p(vpm);
       CGAL::convex_hull_3(
-        boost::make_transform_iterator(selection_item->selected_vertices.begin(), v2p),
-        boost::make_transform_iterator(selection_item->selected_vertices.end(), v2p),
-        *pConvex_hull);
+            *pMesh,
+            *pConvex_hull);
     }
     else if ( sm_item ){
       SMesh* pMesh = sm_item->polyhedron();
-      Vpmap vpm = get(CGAL::vertex_point,*pMesh);
-
-      Vpmap_fct v2p(vpm);
-      boost::graph_traits<SMesh>::vertex_iterator b,e;
-      boost::tie(b,e) = vertices(*pMesh);
-
-      CGAL::convex_hull_3(boost::make_transform_iterator(b,v2p),
-                          boost::make_transform_iterator(e,v2p),
+      CGAL::convex_hull_3(*pMesh,
                           *pConvex_hull);
 
     }
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Display/CMakeLists.txt 5.1-1/demo/Polyhedron/Plugins/Display/CMakeLists.txt
--- 5.0.3-1/demo/Polyhedron/Plugins/Display/CMakeLists.txt	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Display/CMakeLists.txt	2020-09-08 12:12:20.000000000 +0000
@@ -1,6 +1,6 @@
 include( polyhedron_demo_macros )
-if(EIGEN3_FOUND)
+if(TARGET CGAL::Eigen_support)
   qt5_wrap_ui( display_propertyUI_FILES Display_property.ui )
   polyhedron_demo_plugin(display_property_plugin Display_property_plugin ${display_propertyUI_FILES})
-  target_link_libraries(display_property_plugin PUBLIC scene_surface_mesh_item scene_points_with_normal_item scene_color_ramp)
-endif(EIGEN3_FOUND)
+  target_link_libraries(display_property_plugin PUBLIC scene_surface_mesh_item scene_points_with_normal_item scene_color_ramp CGAL::Eigen_support)
+endif()
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Display/Display_property_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Display/Display_property_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Display/Display_property_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Display/Display_property_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -443,9 +443,6 @@ public:
     connect(dock_widget->deleteButton, &QPushButton::clicked,
             this, &DisplayPropertyPlugin::delete_group);
 
-    connect(dock_widget->resetButton, &QPushButton::pressed,
-            this, &DisplayPropertyPlugin::resetRampExtremas);
-
     dock_widget->zoomToMaxButton->setEnabled(false);
     dock_widget->zoomToMinButton->setEnabled(false);
     Scene* scene_obj =static_cast<Scene*>(scene);
@@ -464,29 +461,6 @@ private Q_SLOTS:
       dock_widget->raise(); }
   }
 
-  void resetRampExtremas()
-  {
-    Scene_surface_mesh_item* item =
-        qobject_cast<Scene_surface_mesh_item*>(scene->item(scene->mainSelectionIndex()));
-    if(!item)
-      return;
-    QApplication::setOverrideCursor(Qt::WaitCursor);
-    item->face_graph()->collect_garbage();
-    bool ok;
-    switch(dock_widget->propertyBox->currentIndex())
-    {
-    case 0:
-      ok = resetAngles(item);
-      break;
-    default:
-      ok = resetScaledJacobian(item);
-      break;
-    }
-    QApplication::restoreOverrideCursor();
-    if(!ok)
-      QMessageBox::warning(mw, "Error", "You must first run colorize once to initialize the values.");
-  }
-
   void colorize()
   {
     Scene_heat_item* h_item = nullptr;
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Display/Display_property.ui 5.1-1/demo/Polyhedron/Plugins/Display/Display_property.ui
--- 5.0.3-1/demo/Polyhedron/Plugins/Display/Display_property.ui	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Display/Display_property.ui	2020-09-08 12:12:20.000000000 +0000
@@ -110,44 +110,18 @@
           </property>
           <layout class="QVBoxLayout" name="verticalLayout_2">
            <item>
-            <widget class="QDoubleSpinBox" name="minBox"/>
-           </item>
-           <item>
-            <widget class="QDoubleSpinBox" name="maxBox">
-             <property name="value">
-              <double>2.000000000000000</double>
+            <widget class="DoubleEdit" name="minBox">
+             <property name="text">
+              <string>0.00</string>
              </property>
             </widget>
            </item>
            <item>
-            <layout class="QHBoxLayout" name="horizontalLayout_2" stretch="0,0">
-             <item>
-              <spacer name="horizontalSpacer">
-               <property name="orientation">
-                <enum>Qt::Horizontal</enum>
-               </property>
-               <property name="sizeHint" stdset="0">
-                <size>
-                 <width>40</width>
-                 <height>20</height>
-                </size>
-               </property>
-              </spacer>
-             </item>
-             <item>
-              <widget class="QPushButton" name="resetButton">
-               <property name="toolTip">
-                <string>Compute Ramp Extremas</string>
-               </property>
-               <property name="text">
-                <string>Reset</string>
-               </property>
-               <property name="autoRepeat">
-                <bool>false</bool>
-               </property>
-              </widget>
-             </item>
-            </layout>
+            <widget class="DoubleEdit" name="maxBox">
+             <property name="text">
+              <string>2.00</string>
+             </property>
+            </widget>
            </item>
           </layout>
          </widget>
@@ -223,6 +197,13 @@
    </layout>
   </widget>
  </widget>
+ <customwidgets>
+  <customwidget>
+   <class>DoubleEdit</class>
+   <extends>QLineEdit</extends>
+   <header>CGAL_double_edit.h</header>
+  </customwidget>
+ </customwidgets>
  <resources/>
  <connections/>
 </ui>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/IO/3mf_io_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/IO/3mf_io_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/IO/3mf_io_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/IO/3mf_io_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -78,61 +78,6 @@ class Io_3mf_plugin:
     std::vector<std::string> names;
     QList<Scene_item*> result;
     std::vector<std::vector<CGAL::Color> > all_colors;
-    int nb_polylines =
-        CGAL::read_polylines_from_3mf(fileinfo.filePath().toUtf8().toStdString(),
-                                      all_points, all_colors, names);
-    if(nb_polylines < 0 )
-    {
-      ok = false;
-      std::cerr << "Error in reading of meshes."<<std::endl;
-      return result;
-    }
-
-    for(int i=0; i< nb_polylines; ++i)
-    {
-      Scene_polylines_item* pol_item = new Scene_polylines_item();
-      PolylineRange& polylines = pol_item->polylines;
-      polylines.push_back(all_points[i]);
-      pol_item->setName(names[i].data());
-      pol_item->invalidateOpenGLBuffers();
-      CGAL::Color c = all_colors[i].front();
-      pol_item->setColor(QColor(c.red(), c.green(), c.blue()));
-      pol_item->setProperty("already_colord", true);
-      result << pol_item;
-      if(add_to_scene)
-        CGAL::Three::Three::scene()->addItem(pol_item);
-    }
-    all_points.clear();
-    all_colors.clear();
-    names.clear();
-    int nb_point_sets =
-        CGAL::read_point_clouds_from_3mf(fileinfo.filePath().toUtf8().toStdString(),
-                                         all_points, all_colors, names);
-    if(nb_point_sets < 0 )
-    {
-      ok = false;
-      std::cerr << "Error in reading of meshes."<<std::endl;
-      return result;
-    }
-    for(int i=0; i< nb_point_sets; ++i)
-    {
-      Scene_points_with_normal_item* pts_item = new Scene_points_with_normal_item();
-      for(std::size_t j = 0; j < all_points[i].size(); ++j)
-      {
-        pts_item->point_set()->insert(all_points[i][j]);
-      }
-      pts_item->setName(names[i].data());
-      pts_item->invalidateOpenGLBuffers();
-      CGAL::Color c = all_colors[i].front();
-      pts_item->setColor(QColor(c.red(), c.green(), c.blue()));
-      pts_item->setProperty("already_colord", true);
-      result << pts_item;
-      if(add_to_scene)
-        CGAL::Three::Three::scene()->addItem(pts_item);
-    }
-    all_points.clear();
-    names.clear();
-    all_colors.clear();
     int nb_meshes =
         CGAL::read_triangle_soups_from_3mf(fileinfo.filePath().toUtf8().toStdString(),
                                   all_points, all_polygons, all_colors, names);
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/IO/CMakeLists.txt 5.1-1/demo/Polyhedron/Plugins/IO/CMakeLists.txt
--- 5.0.3-1/demo/Polyhedron/Plugins/IO/CMakeLists.txt	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/IO/CMakeLists.txt	2020-09-08 12:12:20.000000000 +0000
@@ -19,6 +19,9 @@ target_link_libraries(off_to_nef_plugin
 polyhedron_demo_plugin(polylines_io_plugin Polylines_io_plugin KEYWORDS IO Mesh_3)
 target_link_libraries(polylines_io_plugin PUBLIC scene_polylines_item)
 
+polyhedron_demo_plugin(wkt_plugin WKT_io_plugin KEYWORDS IO PointSetProcessing Mesh_3)
+target_link_libraries(wkt_plugin PUBLIC scene_polylines_item)
+
 polyhedron_demo_plugin(stl_plugin STL_io_plugin KEYWORDS IO PMP)
 target_link_libraries(stl_plugin PUBLIC scene_surface_mesh_item scene_polygon_soup_item)
 
@@ -69,9 +72,9 @@ else()
   target_link_libraries(ply_plugin PUBLIC scene_points_with_normal_item scene_polygon_soup_item scene_surface_mesh_item scene_textured_item)
   target_compile_features(ply_plugin PRIVATE ${needed_cxx_features})
 
-  if (LASLIB_FOUND)
+  if (TARGET CGAL::LASLIB_support)
     polyhedron_demo_plugin(las_plugin LAS_io_plugin KEYWORDS IO PointSetProcessing Classification)
-    target_link_libraries(las_plugin PUBLIC scene_points_with_normal_item ${LASLIB_LIBRARIES})
+    target_link_libraries(las_plugin PUBLIC scene_points_with_normal_item CGAL::LASLIB_support)
     target_compile_features(las_plugin PRIVATE ${needed_cxx_features})
   else()
     message(STATUS "NOTICE : the LAS IO plugin needs LAS libraries and will not be compiled.")
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/IO/Implicit_function_io_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/IO/Implicit_function_io_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/IO/Implicit_function_io_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/IO/Implicit_function_io_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -96,7 +96,7 @@ init(QMainWindow* mainWindow, CGAL::Thre
   QMenu* menu = menuFile->findChild<QMenu*>("menuGenerateObject");
   if(!menu){
     QAction* actionLoad = mw->findChild<QAction*>("actionLoadPlugin");
-    menu = new QMenu(tr("Generate &Objet"), menuFile);
+    menu = new QMenu(tr("Generate &Object"), menuFile);
     menu->setObjectName("menuGenerateObject");
     menuFile->insertMenu(actionLoad, menu);
   }
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/IO/lcc_io_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/IO/lcc_io_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/IO/lcc_io_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/IO/lcc_io_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -17,24 +17,24 @@ class LCC_io_plugin :
   Q_PLUGIN_METADATA(IID "com.geometryfactory.PolyhedronDemo.IOPluginInterface/1.90" FILE "lcc_io_plugin.json")
 
 public:
-  bool isDefaultLoader(const CGAL::Three::Scene_item *item) const
+  bool isDefaultLoader(const CGAL::Three::Scene_item *item) const override
   {
     if(qobject_cast<const Scene_lcc_item*>(item))
       return true;
     return false;
   }
-  QString name() const { return "lcc_plugin"; }
-  QString nameFilters() const { return
+  QString name() const override{ return "lcc_plugin"; }
+  QString nameFilters() const override{ return
         "OFF files (*.off);;"
         "3-map files (*.3map)"; }
 
-  QString saveNameFilters() const {
+  QString saveNameFilters() const override{
     return
         "3-map files (*.3map)";
   }
 
-  bool canLoad(QFileInfo) const { return true; }
-  QList<CGAL::Three::Scene_item*> load(QFileInfo fileinfo, bool& ok, bool add_to_scene=true){
+  bool canLoad(QFileInfo) const override{ return true; }
+  QList<CGAL::Three::Scene_item*> load(QFileInfo fileinfo, bool& ok, bool add_to_scene=true) override{
     // Open file
     std::ifstream ifs(fileinfo.filePath().toUtf8());
     if(!ifs) {
@@ -67,8 +67,8 @@ public:
   }
 
 
-  bool canSave(const CGAL::Three::Scene_item*){return false;}
-  bool save(QFileInfo, QList<CGAL::Three::Scene_item*>& ){
+  bool canSave(const CGAL::Three::Scene_item*)override{return false;}
+  bool save(QFileInfo, QList<CGAL::Three::Scene_item*>& )override{
     return false;
   }
 
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/IO/Nef_io_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/IO/Nef_io_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/IO/Nef_io_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/IO/Nef_io_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -15,13 +15,18 @@ class Polyhedron_demo_io_nef_plugin :
   Q_PLUGIN_METADATA(IID "com.geometryfactory.PolyhedronDemo.PluginInterface/1.0" FILE "nef_io_plugin.json")
 
 public:
-  QString nameFilters() const;
-  QString name() const { return "io_nef_plugin"; }
-  bool canLoad(QFileInfo) const;
-  QList<Scene_item*> load(QFileInfo fileinfo, bool& ok, bool add_to_scene=true);
+  QString nameFilters() const override;
+  QString name() const override { return "io_nef_plugin"; }
+  bool canLoad(QFileInfo) const override;
+  QList<Scene_item*> load(QFileInfo fileinfo, bool& ok, bool add_to_scene=true) override;
 
-  bool canSave(const CGAL::Three::Scene_item*);
-  bool save(QFileInfo fileinfo,QList<CGAL::Three::Scene_item*>& items);
+  bool canSave(const CGAL::Three::Scene_item*) override;
+  bool save(QFileInfo fileinfo,QList<CGAL::Three::Scene_item*>& items) override;
+  bool isDefaultLoader(const Scene_item* item) const override{
+    if(qobject_cast<const Scene_nef_polyhedron_item*>(item))
+      return true;
+    return false;
+  }
 };
 
 QString Polyhedron_demo_io_nef_plugin::nameFilters() const {
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/IO/OFF_io_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/IO/OFF_io_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/IO/OFF_io_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/IO/OFF_io_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -9,13 +9,16 @@
 
 #include <CGAL/Three/Polyhedron_demo_io_plugin_interface.h>
 #include <CGAL/Three/Three.h>
-#include <fstream>
 
+#include <CGAL/exceptions.h>
 #include <CGAL/IO/File_scanner_OFF.h>
 #include <CGAL/IO/OBJ_reader.h>
 #include <QMessageBox>
 #include <QApplication>
 
+#include <iostream>
+#include <fstream>
+
 using namespace CGAL::Three;
 class Polyhedron_demo_off_plugin :
   public QObject,
@@ -26,28 +29,28 @@ class Polyhedron_demo_off_plugin :
   Q_PLUGIN_METADATA(IID "com.geometryfactory.PolyhedronDemo.IOPluginInterface/1.90" FILE "off_io_plugin.json")
 
 public:
-  bool isDefaultLoader(const Scene_item *item) const
+  bool isDefaultLoader(const Scene_item *item) const override
   {
     if(qobject_cast<const Scene_surface_mesh_item*>(item)
        || qobject_cast<const Scene_polygon_soup_item*>(item))
       return true;
     return false;
   }
-  bool isDefaultLoader(const QString& name) const
+  bool isDefaultLoader(const QString& name) const override
   {
     if(name == QString("off"))
       return true;
     return false;
   }
-  QString name() const { return "off_plugin"; }
-  QString nameFilters() const { return "OFF files (*.off);;Wavefront OBJ (*.obj)"; }
-  bool canLoad(QFileInfo fileinfo) const;
-  QList<Scene_item*> load(QFileInfo fileinfo, bool& ok, bool add_to_scene=true);
+  QString name() const override{ return "off_plugin"; }
+  QString nameFilters() const override { return "OFF files (*.off);;Wavefront OBJ (*.obj)"; }
+  bool canLoad(QFileInfo fileinfo) const override;
+  QList<Scene_item*> load(QFileInfo fileinfo, bool& ok, bool add_to_scene=true) override;
   CGAL::Three::Scene_item* load_off(QFileInfo fileinfo);
   CGAL::Three::Scene_item* load_obj(QFileInfo fileinfo);
 
-  bool canSave(const CGAL::Three::Scene_item*);
-  bool save(QFileInfo fileinfo,QList<CGAL::Three::Scene_item*>& );
+  bool canSave(const CGAL::Three::Scene_item*) override;
+  bool save(QFileInfo fileinfo,QList<CGAL::Three::Scene_item*>& ) override;
 };
 
 bool Polyhedron_demo_off_plugin::canLoad(QFileInfo) const {
@@ -189,7 +192,7 @@ Polyhedron_demo_off_plugin::load_off(QFi
   try{
     CGAL::Polygon_mesh_processing::non_manifold_vertices(*surface_mesh, OutputIterator());
   }
-  catch( CGAL::internal::Throw_at_output::Throw_at_output_exception& )
+  catch( CGAL::internal::Throw_at_output_exception& )
   {
 
     QApplication::restoreOverrideCursor();
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/IO/PLY_io_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/IO/PLY_io_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/IO/PLY_io_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/IO/PLY_io_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -23,19 +23,19 @@ class Polyhedron_demo_ply_plugin :
   Q_PLUGIN_METADATA(IID "com.geometryfactory.PolyhedronDemo.IOPluginInterface/1.90" FILE "ply_io_plugin.json")
 
 public:
-  bool isDefaultLoader(const CGAL::Three::Scene_item *item) const
+  bool isDefaultLoader(const CGAL::Three::Scene_item *item) const override
   {
     if(qobject_cast<const Scene_points_with_normal_item*>(item))
       return true;
     return false;
   }
-  QString name() const { return "ply_plugin"; }
-  QString nameFilters() const { return "PLY files (*.ply)"; }
-  bool canLoad(QFileInfo fileinfo) const;
-  QList<Scene_item*> load(QFileInfo fileinfo, bool& ok, bool add_to_scene=true);
+  QString name() const override{ return "ply_plugin"; }
+  QString nameFilters() const override{ return "PLY files (*.ply)"; }
+  bool canLoad(QFileInfo fileinfo) const override;
+  QList<Scene_item*> load(QFileInfo fileinfo, bool& ok, bool add_to_scene=true)override;
 
-  bool canSave(const CGAL::Three::Scene_item*);
-  bool save(QFileInfo fileinfo,QList<CGAL::Three::Scene_item*>&);
+  bool canSave(const CGAL::Three::Scene_item*)override;
+  bool save(QFileInfo fileinfo,QList<CGAL::Three::Scene_item*>&)override;
 
 };
 
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/IO/Polylines_io_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/IO/Polylines_io_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/IO/Polylines_io_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/IO/Polylines_io_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -75,6 +75,12 @@ public:
     return QList<QAction*>()<<actionSplit_polylines
                             <<actionJoin_polylines;
   }
+
+  bool isDefaultLoader(const Scene_item* item) const override{
+    if(qobject_cast<const Scene_polylines_item*>(item))
+      return true;
+    return false;
+  }
   protected Q_SLOTS:
   //!Splits the selected Scene_polylines_item in multiple items all containing a single polyline.
   void split();
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/IO/STL_io_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/IO/STL_io_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/IO/STL_io_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/IO/STL_io_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -6,19 +6,24 @@
 
 #include <CGAL/Three/Polyhedron_demo_io_plugin_interface.h>
 #include <CGAL/Three/Three.h>
-#include <fstream>
 
 #include <CGAL/IO/Polyhedron_builder_from_STL.h>
 #include <CGAL/IO/STL_writer.h>
 #include <CGAL/Polygon_mesh_processing/polygon_soup_to_polygon_mesh.h>
 
-
 #include <QColor>
 #include <QString>
 #include <QStringList>
 #include <QMainWindow>
 #include <QInputDialog>
+
+#include <boost/property_map/function_property_map.hpp>
+
 #include <cstdint>
+#include <fstream>
+#include <iostream>
+#include <limits>
+#include <vector>
 
 using namespace CGAL::Three;
 class Polyhedron_demo_stl_plugin :
@@ -79,11 +84,19 @@ load(QFileInfo fileinfo, bool& ok, bool
     return QList<Scene_item*>();
   }
 
-  try{
+  try
+  {
     // Try building a surface_mesh
     SMesh* SM = new SMesh();
     if (CGAL::Polygon_mesh_processing::is_polygon_soup_a_polygon_mesh(triangles))
-      CGAL::Polygon_mesh_processing::polygon_soup_to_polygon_mesh(points, triangles, *SM);
+    {
+      auto pmap = boost::make_function_property_map<std::array<double, 3>, Point_3>(
+                    [](const std::array<double, 3>& a) { return Point_3(a[0], a[1], a[2]); });
+      CGAL::Polygon_mesh_processing::polygon_soup_to_polygon_mesh(points, triangles, *SM,
+                                                                  CGAL::parameters::point_map(pmap),
+                                                                  CGAL::parameters::all_default());
+    }
+
     if(!SM->is_valid() || SM->is_empty()){
       std::cerr << "Error: Invalid facegraph" << std::endl;
     }
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/IO/Surf_io_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/IO/Surf_io_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/IO/Surf_io_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/IO/Surf_io_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -27,7 +27,7 @@ class Surf_io_plugin:
 public:
 
   QString name() const { return "surf_io_plugin"; }
-  QString nameFilters() const { return "Amira files (*.surf)"; }
+  QString nameFilters() const { return "Amira files (*.surf);;Amira binary files (*.surf.am)"; }
   bool canLoad(QFileInfo) const{ return true; }
   template<class FaceGraphItem>
   CGAL::Three::Scene_item* actual_load(QFileInfo fileinfo);
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/IO/VTK_io_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/IO/VTK_io_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/IO/VTK_io_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/IO/VTK_io_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -290,10 +290,11 @@ public:
 
 
   QString nameFilters() const {
-    return "VTK PolyData files (*.vtk);; VTK XML PolyData (*.vtp);; VTK XML UnstructuredGrid (*.vtu)"; }
+    return "VTK XML UnstructuredGrid (*.vtu);;VTK PolyData files (*.vtk);; VTK XML PolyData (*.vtp)"; }
   QString name() const { return "vtk_plugin"; }
   bool canSave(const CGAL::Three::Scene_item* item)
   {
+
     return (qobject_cast<const Scene_facegraph_item*>(item)
             || qobject_cast<const Scene_c3t3_item*>(item));
   }
@@ -502,8 +503,8 @@ public:
 
     if (is_c3t3)
     {
-      typedef boost::array<int, 3> Facet; // 3 = id
-      typedef boost::array<int, 5> Tet_with_ref; // first 4 = id, fifth = reference
+      typedef std::array<int, 3> Facet; // 3 = id
+      typedef std::array<int, 5> Tet_with_ref; // first 4 = id, fifth = reference
       Scene_c3t3_item* c3t3_item = new Scene_c3t3_item();
       c3t3_item->set_valid(false);
       //build a triangulation from data:
@@ -546,7 +547,9 @@ public:
           std::swap(finite_cells[i][1], finite_cells[i][3]);
         }
       }
-      CGAL::build_triangulation<Tr, true>(c3t3_item->c3t3().triangulation(), points, finite_cells, border_facets);
+      std::vector<typename Tr::Vertex_handle> new_vertices;
+      CGAL::build_triangulation<Tr, true>(c3t3_item->c3t3().triangulation(),
+        points, finite_cells, border_facets, new_vertices);
 
       for( C3t3::Triangulation::Finite_cells_iterator
            cit = c3t3_item->c3t3().triangulation().finite_cells_begin();
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/IO/WKT_io_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/IO/WKT_io_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/IO/WKT_io_plugin.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/IO/WKT_io_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,113 @@
+#include "Scene_polylines_item.h"
+
+#include <CGAL/Three/Polyhedron_demo_io_plugin_interface.h>
+#include <CGAL/Three/Three.h>
+#include <QInputDialog>
+#include <QApplication>
+#include <fstream>
+
+#include <CGAL/IO/WKT.h>
+#include <QMessageBox>
+
+using namespace CGAL::Three;
+class Polyhedron_demo_wkt_plugin :
+  public QObject,
+  public CGAL::Three::Polyhedron_demo_io_plugin_interface
+{
+  Q_OBJECT
+  Q_INTERFACES(CGAL::Three::Polyhedron_demo_io_plugin_interface)
+  Q_PLUGIN_METADATA(IID "com.geometryfactory.PolyhedronDemo.IOPluginInterface/1.90" FILE "wkt_io_plugin.json")
+
+public:
+  bool isDefaultLoader(const CGAL::Three::Scene_item *item) const
+  {
+    if(qobject_cast<const Scene_polylines_item*>(item))
+      return true;
+    return false;
+  }
+
+  QString name() const { return "wkt_plugin"; }
+
+  // To change if we end up supporting other objects in the plugin
+  QString nameFilters() const { return "WKT polylines (*.wkt)"; }
+  bool canLoad(QFileInfo fileinfo) const;
+  QList<Scene_item*> load(QFileInfo fileinfo, bool& ok, bool add_to_scene=true);
+
+  bool canSave(const CGAL::Three::Scene_item*);
+  bool save(QFileInfo fileinfo,QList<CGAL::Three::Scene_item*>&);
+};
+
+bool Polyhedron_demo_wkt_plugin::
+canLoad(QFileInfo) const {
+  return true;
+}
+
+QList<Scene_item*>
+Polyhedron_demo_wkt_plugin::
+load(QFileInfo fileinfo, bool& ok, bool add_to_scene) {
+  std::ifstream in(fileinfo.filePath().toUtf8(), std::ios_base::binary);
+
+  if(!in)
+    std::cerr << "Error!\n";
+
+  QApplication::setOverrideCursor(Qt::WaitCursor);
+
+  if(fileinfo.size() == 0)
+  {
+    CGAL::Three::Three::warning( tr("The file you are trying to load is empty."));
+    ok = false;
+    return QList<Scene_item*>();
+  }
+
+  std::list<std::vector<Scene_polylines_item::Point_3> > polylines;
+  CGAL::read_multi_linestring_WKT (in, polylines);
+
+  Scene_polylines_item* item = new Scene_polylines_item;
+  item->polylines = polylines;
+  item->setName(fileinfo.baseName());
+  item->setColor(Qt::black);
+  std::cerr << "Number of polylines in item: " << item->polylines.size() << std::endl;
+  item->invalidateOpenGLBuffers();
+  ok = true;
+  if(add_to_scene)
+    CGAL::Three::Three::scene()->addItem(item);
+
+  QApplication::restoreOverrideCursor();
+
+  return QList<Scene_item*>() << item;
+}
+
+bool Polyhedron_demo_wkt_plugin::canSave(const CGAL::Three::Scene_item* item)
+{
+  // This plugin supports polylines
+  return (qobject_cast<const Scene_polylines_item*>(item));
+}
+
+bool Polyhedron_demo_wkt_plugin::
+save(QFileInfo fileinfo,QList<CGAL::Three::Scene_item*>& items)
+{
+  Scene_item* item = items.front();
+  // Check extension (quietly)
+  std::string extension = fileinfo.suffix().toUtf8().data();
+  std::cerr << extension << std::endl;
+  if (extension != "wkt" && extension != "WKT")
+    return false;
+
+  std::ofstream out(fileinfo.filePath().toUtf8().data(), std::ios::binary);
+  out.precision (std::numeric_limits<double>::digits10 + 2);
+
+  // This plugin supports point sets
+  Scene_polylines_item* polylines_item =
+    qobject_cast<Scene_polylines_item*>(item);
+  if (polylines_item)
+  {
+    CGAL::write_multi_linestring_WKT (out, polylines_item->polylines);
+    items.pop_front();
+    return true;
+  }
+
+  return false;
+}
+
+
+#include "WKT_io_plugin.moc"
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Mesh_2/mesh_2_dialog.ui 5.1-1/demo/Polyhedron/Plugins/Mesh_2/mesh_2_dialog.ui
--- 5.0.3-1/demo/Polyhedron/Plugins/Mesh_2/mesh_2_dialog.ui	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Mesh_2/mesh_2_dialog.ui	2020-09-08 12:12:20.000000000 +0000
@@ -10,7 +10,7 @@
     <x>0</x>
     <y>0</y>
     <width>424</width>
-    <height>301</height>
+    <height>302</height>
    </rect>
   </property>
   <property name="windowTitle">
@@ -38,9 +38,6 @@
         <property name="alignment">
          <set>Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter</set>
         </property>
-        <property name="buddy">
-         <cstring>edgeLength_dspinbox</cstring>
-        </property>
        </widget>
       </item>
       <item row="4" column="0">
@@ -56,22 +53,6 @@
         </property>
        </widget>
       </item>
-      <item row="1" column="1">
-       <widget class="QDoubleSpinBox" name="edgeLength_dspinbox">
-        <property name="minimumSize">
-         <size>
-          <width>110</width>
-          <height>0</height>
-         </size>
-        </property>
-        <property name="maximum">
-         <double>1000.000000000000000</double>
-        </property>
-        <property name="singleStep">
-         <double>0.100000000000000</double>
-        </property>
-       </widget>
-      </item>
       <item row="0" column="0">
        <widget class="QLabel" name="runMesh2_label">
         <property name="text">
@@ -157,6 +138,13 @@
         </property>
        </widget>
       </item>
+      <item row="1" column="1">
+       <widget class="DoubleEdit" name="edgeLength_dspinbox">
+        <property name="text">
+         <string>0.00</string>
+        </property>
+       </widget>
+      </item>
      </layout>
     </widget>
    </item>
@@ -185,6 +173,13 @@
    </item>
   </layout>
  </widget>
+ <customwidgets>
+  <customwidget>
+   <class>DoubleEdit</class>
+   <extends>QLineEdit</extends>
+   <header>CGAL_double_edit.h</header>
+  </customwidget>
+ </customwidgets>
  <tabstops>
   <tabstop>buttonBox</tabstop>
  </tabstops>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Mesh_2/Mesh_2_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Mesh_2/Mesh_2_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Mesh_2/Mesh_2_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Mesh_2/Mesh_2_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -195,8 +195,6 @@ private:
             ui.edgeLength_dspinbox, SLOT(setEnabled(bool)));
 
     //Set default parameter edge length
-    ui.edgeLength_dspinbox->setDecimals(3);
-    ui.edgeLength_dspinbox->setSingleStep(0.001);
     ui.edgeLength_dspinbox->setRange(1e-6 * diag_length, //min
                                      2.   * diag_length);//max
     ui.edgeLength_dspinbox->setValue(0.05 * diag_length);
@@ -256,15 +254,15 @@ private:
 
     // wait cursor
     QApplication::setOverrideCursor(Qt::WaitCursor);
-    typedef ProjectionTraits                                             Gt;
-    typedef CGAL::Delaunay_mesh_vertex_base_2<Gt>                        Vb;
-    typedef CGAL::Delaunay_mesh_face_base_2<Gt>                          Fm;
-    typedef CGAL::Triangulation_face_base_with_info_2<FaceInfo2,Gt,Fm>   Fb;
-    typedef CGAL::Triangulation_data_structure_2<Vb, Fb>                TDS;
-    typedef CGAL::No_intersection_tag                                   Tag;
-    typedef CGAL::Constrained_Delaunay_triangulation_2<Gt, TDS, Tag>    CDT;
-    typedef CGAL::Delaunay_mesh_size_criteria_2<CDT>               Criteria;
-    typedef CGAL::Delaunay_mesher_2<CDT, Criteria>                   Mesher;
+    typedef ProjectionTraits                                              Gt;
+    typedef CGAL::Delaunay_mesh_vertex_base_2<Gt>                         Vb;
+    typedef CGAL::Delaunay_mesh_face_base_2<Gt>                           Fm;
+    typedef CGAL::Triangulation_face_base_with_info_2<FaceInfo2,Gt,Fm>    Fb;
+    typedef CGAL::Triangulation_data_structure_2<Vb, Fb>                  TDS;
+    typedef CGAL::No_constraint_intersection_requiring_constructions_tag  Tag;
+    typedef CGAL::Constrained_Delaunay_triangulation_2<Gt, TDS, Tag>      CDT;
+    typedef CGAL::Delaunay_mesh_size_criteria_2<CDT>                      Criteria;
+    typedef CGAL::Delaunay_mesher_2<CDT, Criteria>                        Mesher;
 
     QElapsedTimer time; // global timer
     time.start();
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/C3t3_io_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Mesh_3/C3t3_io_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/C3t3_io_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Mesh_3/C3t3_io_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -18,15 +18,20 @@ class Polyhedron_demo_c3t3_binary_io_plu
     Q_PLUGIN_METADATA(IID "com.geometryfactory.PolyhedronDemo.IOPluginInterface/1.90" FILE "c3t3_io_plugin.json")
 
 public:
-  QString name() const { return "C3t3_io_plugin"; }
-  QString nameFilters() const { return "binary files (*.cgal);;ascii (*.mesh);;maya (*.ma)"; }
-  QString saveNameFilters() const { return "binary files (*.cgal);;ascii (*.mesh);;maya (*.ma);;avizo (*.am);;OFF files (*.off)"; }
-  QString loadNameFilters() const { return "binary files (*.cgal);;ascii (*.mesh)"; }
-  bool canLoad(QFileInfo) const;
-  QList<Scene_item*> load(QFileInfo fileinfo, bool& ok, bool add_to_scene=true);
-
-  bool canSave(const CGAL::Three::Scene_item*);
-  bool save(QFileInfo fileinfo,QList<CGAL::Three::Scene_item*>& );
+  QString name() const override { return "C3t3_io_plugin"; }
+  QString nameFilters() const override { return "binary files (*.cgal);;ascii (*.mesh);;maya (*.ma)"; }
+  QString saveNameFilters() const override { return "binary files (*.cgal);;ascii (*.mesh);;maya (*.ma);;avizo (*.am);;OFF files (*.off)"; }
+  QString loadNameFilters() const override { return "binary files (*.cgal);;ascii (*.mesh)"; }
+  bool canLoad(QFileInfo) const override;
+  QList<Scene_item*> load(QFileInfo fileinfo, bool& ok, bool add_to_scene=true) override;
+
+  bool canSave(const CGAL::Three::Scene_item*) override;
+  bool save(QFileInfo fileinfo,QList<CGAL::Three::Scene_item*>& ) override;
+  bool isDefaultLoader(const Scene_item* item) const override{
+    if(qobject_cast<const Scene_c3t3_item*>(item))
+      return true;
+    return false;
+  }
 
 private:
   bool try_load_other_binary_format(std::istream& in, C3t3& c3t3);
@@ -124,13 +129,14 @@ Polyhedron_demo_c3t3_binary_io_plugin::l
 
       if(CGAL::build_triangulation_from_file<C3t3::Triangulation, true>(in, item->c3t3().triangulation()))
       {
+        item->c3t3().rescan_after_load_of_triangulation();
         for( C3t3::Triangulation::Finite_cells_iterator
              cit = item->c3t3().triangulation().finite_cells_begin();
              cit != item->c3t3().triangulation().finite_cells_end();
              ++cit)
         {
-            CGAL_assertion(cit->info() >= 0);
-            item->c3t3().add_to_complex(cit, cit->info());
+            if(cit->subdomain_index() != C3t3::Triangulation::Cell::Subdomain_index())
+              item->c3t3().add_to_complex(cit, cit->subdomain_index());
             for(int i=0; i < 4; ++i)
             {
               if(cit->surface_patch_index(i)>0)
@@ -401,6 +407,11 @@ struct Update_vertex
   typedef typename Tr2::Vertex                  V2;
   typedef typename Tr2::Point                   Point;
 
+  V2 operator()(const V1&)
+  {
+    return V2();
+  }
+
   bool operator()(const V1& v1, V2& v2)
   {
     v2.set_point(Point(v1.point()));
@@ -413,7 +424,7 @@ struct Update_vertex
       const Sp_index sp_index = boost::get<Sp_index>(index);
       v2.set_index((std::max)(sp_index.first, sp_index.second));
     }
-    break;
+      break;
     default:// -1, 0, 1, 3
       v2.set_index(boost::get<int>(v1.index()));
     }
@@ -422,7 +433,9 @@ struct Update_vertex
 }; // end struct Update_vertex
 
 struct Update_cell {
+
   typedef Fake_mesh_domain::Surface_patch_index Sp_index;
+
   template <typename C1, typename C2>
   bool operator()(const C1& c1, C2& c2) {
     c2.set_subdomain_index(c1.subdomain_index());
@@ -437,7 +450,6 @@ struct Update_cell {
   }
 }; // end struct Update_cell
 
-#include <CGAL/Triangulation_file_input.h>
 
 template <typename Tr1, typename Tr2>
 struct Update_vertex_from_CDT_3 {
@@ -447,24 +459,28 @@ struct Update_vertex_from_CDT_3 {
   typedef typename Tr2::Vertex          V2;
   typedef typename Tr2::Point           Point;
 
-  bool operator()(const V1& v1, V2& v2)
+   V2 operator()(const V1&)
+  {
+    return V2();
+  }
+  void operator()(const V1& v1, V2& v2)
   {
     v2.set_point(Point(v1.point()));
     v2.set_dimension(2);
     v2.set_special(false);
-    return true;
   }
 }; // end struct Update_vertex
 
 struct Update_cell_from_CDT_3 {
+
   typedef Fake_mesh_domain::Surface_patch_index Sp_index;
-  template <typename C1, typename C2>
-  bool operator()(const C1& c1, C2& c2) {
+
+  template <typename C1,typename C2>
+  void operator()(const C1& c1, C2& c2) {
     c2.set_subdomain_index(1);
     for(int i = 0; i < 4; ++i) {
       c2.set_surface_patch_index(i, c1.constrained_facet[i]);
     }
-    return true;
   }
 }; // end struct Update_cell
 
@@ -495,11 +511,10 @@ try_load_a_cdt_3(std::istream& is, C3t3&
     }
   }
   if(binary) CGAL::set_binary_mode(is);
-  if(CGAL::file_input<
+  if(c3t3.triangulation().file_input<
        Fake_CDT_3,
-       C3t3::Triangulation,
        Update_vertex_from_CDT_3<Fake_CDT_3, C3t3::Triangulation>,
-       Update_cell_from_CDT_3>(is, c3t3.triangulation()))
+       Update_cell_from_CDT_3>(is))
   {
     c3t3.rescan_after_load_of_triangulation();
     std::cerr << "Try load a CDT_3... DONE";
@@ -539,11 +554,10 @@ try_load_other_binary_format(std::istrea
   }
   if(binary) CGAL::set_binary_mode(is);
   else CGAL::set_ascii_mode(is);
-  std::istream& f_is = CGAL::file_input<
+  std::istream& f_is = c3t3.triangulation().file_input<
                          Fake_c3t3::Triangulation,
-                         C3t3::Triangulation,
                          Update_vertex<Fake_c3t3::Triangulation, C3t3::Triangulation>,
-                         Update_cell>(is, c3t3.triangulation());
+                         Update_cell>(is);
 
   c3t3.rescan_after_load_of_triangulation();
   return f_is.good();
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/C3t3_rib_exporter_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Mesh_3/C3t3_rib_exporter_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/C3t3_rib_exporter_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Mesh_3/C3t3_rib_exporter_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -305,6 +305,8 @@ get_parameters_from_dialog()
   QDialog dialog(mw);
   Ui::Rib_dialog ui;
   ui.setupUi(&dialog);
+  ui.ambientIntensity->setMaximum(1.0);
+  ui.shadowIntensity->setMaximum(1.0);
 
   connect(ui.buttonBox, SIGNAL(accepted()), &dialog, SLOT(accept()));
   connect(ui.buttonBox, SIGNAL(rejected()), &dialog, SLOT(reject()));
@@ -753,8 +755,8 @@ write_surface_cells(const C3t3& c3t3, co
 
     //const int TRANSPARENCY_ALPHA_VALUE = 100;
     CGAL::Bbox_3 bbox = c3t3.bbox();
-    float relPos = (c->weighted_circumcenter().x() - bbox.xmin())
-                   / (bbox.xmax() - bbox.xmin());
+    float relPos = static_cast<float>((c->weighted_circumcenter().x() - bbox.xmin())
+                                      / (bbox.xmax() - bbox.xmin()));
     float TRANSPARENCY_ALPHA_VALUE =
       1.f -
       (relPos < 0.25f ?
@@ -840,7 +842,7 @@ write_surface_cells(const C3t3& c3t3, co
       {
         if (i == last_2D_vertex_index)
         {
-          edgecolor.setAlpha(TRANSPARENCY_ALPHA_VALUE);
+          edgecolor.setAlphaF(TRANSPARENCY_ALPHA_VALUE);
         }
         else
         {
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/CMakeLists.txt 5.1-1/demo/Polyhedron/Plugins/Mesh_3/CMakeLists.txt
--- 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/CMakeLists.txt	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Mesh_3/CMakeLists.txt	2020-09-08 12:12:20.000000000 +0000
@@ -10,6 +10,7 @@ polyhedron_demo_plugin(mesh_3_plugin Mes
   ${meshingUI_FILES} KEYWORDS Mesh_3)
 target_link_libraries(mesh_3_plugin PUBLIC scene_polygon_soup_item scene_polylines_item scene_implicit_function_item scene_image_item
   scene_surface_mesh_item scene_c3t3_item ${OPENGL_gl_LIBRARY} )
+target_include_directories(mesh_3_plugin PRIVATE include)
 
 find_package(VTK QUIET COMPONENTS
   vtkImagingGeneral  vtkIOImage NO_MODULE)
@@ -21,9 +22,7 @@ if (VTK_FOUND)
     if(TARGET VTK::IOImage)
       set(VTK_LIBRARIES VTK::IOImage)
     endif()
-    if(VTK_LIBRARIES)
-      add_definitions(-DCGAL_USE_VTK)
-    else()
+    if(NOT VTK_LIBRARIES)
       message(STATUS "NOTICE : the DICOM files (.dcm) need VTK libraries to be open and will not be able to.")
     endif()
   else()
@@ -33,13 +32,16 @@ else()
   message(STATUS "NOTICE : the DICOM files (.dcm) need VTK libraries to be open and will not be able to.")
 endif()
 
-find_package(Boost QUIET OPTIONAL_COMPONENTS filesystem)
+find_package(Boost QUIET OPTIONAL_COMPONENTS filesystem system)
 if(Boost_FILESYSTEM_FOUND)
   qt5_wrap_ui( imgUI_FILES Image_res_dialog.ui raw_image.ui)
   polyhedron_demo_plugin(io_image_plugin Io_image_plugin Volume_plane_intersection.cpp Raw_image_dialog.cpp ${imgUI_FILES} ${VOLUME_MOC_OUTFILES} KEYWORDS IO Mesh_3)
   target_link_libraries(io_image_plugin PUBLIC scene_image_item ${VTK_LIBRARIES} CGAL::CGAL_ImageIO)
+  if(VTK_LIBRARIES)
+    target_compile_definitions(io_image_plugin PRIVATE CGAL_USE_VTK)
+  endif()
   if(TARGET Boost::filesystem)
-    target_link_libraries(io_image_plugin PUBLIC Boost::filesystem)
+    target_link_libraries(io_image_plugin PUBLIC Boost::filesystem Boost::system)
   else()
     target_link_libraries(io_image_plugin PUBLIC ${Boost_LIBRARIES})
   endif()
@@ -48,10 +50,18 @@ else()
 endif()
 polyhedron_demo_plugin(mesh_3_optimization_plugin Optimization_plugin
   Optimization_plugin_cgal_code.cpp Optimizer_thread.cpp
-  ${meshingUI_FILES} KEYWORDS IO Mesh_3)
+  ${meshingUI_FILES} KEYWORDS Mesh_3)
 target_link_libraries(mesh_3_optimization_plugin PUBLIC scene_c3t3_item scene_surface_mesh_item scene_image_item scene_implicit_function_item )
 
-
+# Use Eigen
+ find_package(Eigen3 3.1.0 REQUIRED) #(3.1.0 or greater)
+ include(CGAL_Eigen_support)
+
+ if (TARGET CGAL::Eigen_support)
+   target_link_libraries(mesh_3_optimization_plugin PUBLIC CGAL::Eigen_support)
+ else() #eigen
+   message(STATUS "The Mesh_3_optimization_plugin requires Eigen, which was not found, and will use a deprecated class to replace it. Warnings are to be expected.")
+ endif()#eigen
 
 polyhedron_demo_plugin(c3t3_io_plugin C3t3_io_plugin KEYWORDS IO Mesh_3)
 target_link_libraries(c3t3_io_plugin PUBLIC scene_c3t3_item)
@@ -61,8 +71,8 @@ polyhedron_demo_plugin(c3t3_rib_exporter
 target_link_libraries(c3t3_rib_exporter_plugin PUBLIC scene_c3t3_item)
 
 if(TBB_FOUND)
-  CGAL_target_use_TBB(mesh_3_plugin)
-  CGAL_target_use_TBB(mesh_3_optimization_plugin)
-  CGAL_target_use_TBB(c3t3_io_plugin)
-  CGAL_target_use_TBB(c3t3_rib_exporter_plugin)
+  target_link_libraries(mesh_3_plugin PUBLIC CGAL::TBB_support)
+  target_link_libraries(mesh_3_optimization_plugin PUBLIC CGAL::TBB_support)
+  target_link_libraries(c3t3_io_plugin PUBLIC CGAL::TBB_support)
+  target_link_libraries(c3t3_rib_exporter_plugin PUBLIC CGAL::TBB_support)
 endif()
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/config_mesh_3.h 5.1-1/demo/Polyhedron/Plugins/Mesh_3/config_mesh_3.h
--- 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/config_mesh_3.h	2020-08-01 19:02:42.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Mesh_3/config_mesh_3.h	2020-09-08 12:15:24.000000000 +0000
@@ -3,6 +3,15 @@
 
 #define  BOOST_PARAMETER_MAX_ARITY 12
 
+// CGAL_MESH_3_PROTECTION_DEBUG:
+// -  1 : display debug messages
+// -  2 : dump file `polylines_graph.polylines.txt` and
+//        `edges-graph.polylines.txt` from polyhedral and image domains
+// -  4 : dump c3t3 in case of a bug
+// -  8 : dump c3t3 at various stages of the protection
+// - 16 : more precise debug messages
+//#define CGAL_MESH_3_PROTECTION_DEBUG 255
+
 
 //#define CGAL_PROFILE
 
@@ -31,7 +40,7 @@
 #  define CGAL_POLYHEDRON_DEMO_USE_SURFACE_MESHER
 #endif
 
-#define CGAL_MESH_3_DEMO_BIGGER_HISTOGRAM_WITH_WHITE_BACKGROUNG
+//#define CGAL_MESH_3_DEMO_BIGGER_HISTOGRAM_WITH_WHITE_BACKGROUNG
 
 // If you define this, implicit function and segmented images won't be available
 //#define CGAL_MESH_3_DEMO_ACTIVATE_SHARP_FEATURES_IN_POLYHEDRAL_DOMAIN
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/Facet_extra_criterion.h 5.1-1/demo/Polyhedron/Plugins/Mesh_3/Facet_extra_criterion.h
--- 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/Facet_extra_criterion.h	2020-08-01 19:02:42.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Mesh_3/Facet_extra_criterion.h	2020-09-08 12:15:24.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron/demo/Polyhedron/Plugins/Mesh_3/Facet_extra_criterion.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron/demo/Polyhedron/Plugins/Mesh_3/Facet_extra_criterion.h $
 // $Id: Facet_extra_criterion.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/include/gsl/gsl_assert 5.1-1/demo/Polyhedron/Plugins/Mesh_3/include/gsl/gsl_assert
--- 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/include/gsl/gsl_assert	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Mesh_3/include/gsl/gsl_assert	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,167 @@
+///////////////////////////////////////////////////////////////////////////////
+//
+// Copyright (c) 2015 Microsoft Corporation. All rights reserved.
+//
+// This code is licensed under the MIT License (MIT).
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+// THE SOFTWARE.
+//
+///////////////////////////////////////////////////////////////////////////////
+
+#ifndef GSL_CONTRACTS_H
+#define GSL_CONTRACTS_H
+
+#include <exception>
+#include <stdexcept> // for logic_error
+
+//
+// make suppress attributes parse for some compilers
+// Hopefully temporary until suppresion standardization occurs
+//
+#if defined (_MSC_VER)
+#define GSL_SUPPRESS(x) [[gsl::suppress(x)]]
+#else
+#if defined(__clang__)
+#define GSL_SUPPRESS(x) [[gsl::suppress("x")]]
+#else
+#define GSL_SUPPRESS(x)
+#endif // __clang__
+#endif // _MSC_VER
+
+//
+// Temporary until MSVC STL supports no-exceptions mode.
+// Currently terminate is a no-op in this mode, so we add termination behavior back
+//
+#if defined(_MSC_VER) && defined(_HAS_EXCEPTIONS) && !_HAS_EXCEPTIONS
+#define GSL_MSVC_USE_STL_NOEXCEPTION_WORKAROUND
+#include <intrin.h>
+#define RANGE_CHECKS_FAILURE 0
+#endif
+
+//
+// There are three configuration options for this GSL implementation's behavior
+// when pre/post conditions on the GSL types are violated:
+//
+// 1. GSL_TERMINATE_ON_CONTRACT_VIOLATION: std::terminate will be called (default)
+// 2. GSL_THROW_ON_CONTRACT_VIOLATION: a gsl::fail_fast exception will be thrown
+// 3. GSL_UNENFORCED_ON_CONTRACT_VIOLATION: nothing happens
+//
+#if !(defined(GSL_THROW_ON_CONTRACT_VIOLATION) || defined(GSL_TERMINATE_ON_CONTRACT_VIOLATION) ||  \
+      defined(GSL_UNENFORCED_ON_CONTRACT_VIOLATION))
+#define GSL_TERMINATE_ON_CONTRACT_VIOLATION
+#endif
+
+#define GSL_STRINGIFY_DETAIL(x) #x
+#define GSL_STRINGIFY(x) GSL_STRINGIFY_DETAIL(x)
+
+#if defined(__clang__) || defined(__GNUC__)
+#define GSL_LIKELY(x) __builtin_expect(!!(x), 1)
+#define GSL_UNLIKELY(x) __builtin_expect(!!(x), 0)
+#else
+#define GSL_LIKELY(x) (!!(x))
+#define GSL_UNLIKELY(x) (!!(x))
+#endif
+
+//
+// GSL_ASSUME(cond)
+//
+// Tell the optimizer that the predicate cond must hold. It is unspecified
+// whether or not cond is actually evaluated.
+//
+#ifdef _MSC_VER
+#define GSL_ASSUME(cond) __assume(cond)
+#elif defined(__GNUC__)
+#define GSL_ASSUME(cond) ((cond) ? static_cast<void>(0) : __builtin_unreachable())
+#else
+#define GSL_ASSUME(cond) static_cast<void>((cond) ? 0 : 0)
+#endif
+
+//
+// GSL.assert: assertions
+//
+
+namespace gsl
+{
+struct fail_fast : public std::logic_error
+{
+    explicit fail_fast(char const* const message) : std::logic_error(message) {}
+};
+
+namespace details
+{
+#if defined(GSL_MSVC_USE_STL_NOEXCEPTION_WORKAROUND)
+
+    typedef void  (__cdecl *terminate_handler)();
+    
+    GSL_SUPPRESS(f.6) // NO-FORMAT: attribute
+    [[noreturn]] inline void __cdecl default_terminate_handler()
+    {
+        __fastfail(RANGE_CHECKS_FAILURE);
+    }
+
+    inline gsl::details::terminate_handler& get_terminate_handler() noexcept
+    {
+        static terminate_handler handler = &default_terminate_handler;
+        return handler;
+    }
+
+#endif
+
+    [[noreturn]] inline void terminate() noexcept
+    {
+#if defined(GSL_MSVC_USE_STL_NOEXCEPTION_WORKAROUND)
+        (*gsl::details::get_terminate_handler())();
+#else
+        std::terminate();
+#endif
+    }
+
+#if defined(GSL_TERMINATE_ON_CONTRACT_VIOLATION)
+
+    template <typename Exception>
+    [[noreturn]] void throw_exception(Exception&&) noexcept
+    {
+        gsl::details::terminate();
+    }
+
+#else
+
+    template <typename Exception>
+    [[noreturn]] void throw_exception(Exception&& exception)
+    {
+        throw std::forward<Exception>(exception);
+    }
+
+#endif
+
+} // namespace details
+} // namespace gsl
+
+#if defined(GSL_THROW_ON_CONTRACT_VIOLATION)
+
+#define GSL_CONTRACT_CHECK(type, cond)                                                             \
+    (GSL_LIKELY(cond) ? static_cast<void>(0)                                                       \
+                      : gsl::details::throw_exception(gsl::fail_fast(                              \
+                            "GSL: " type " failure at " __FILE__ ": " GSL_STRINGIFY(__LINE__))))
+
+#elif defined(GSL_TERMINATE_ON_CONTRACT_VIOLATION)
+
+#define GSL_CONTRACT_CHECK(type, cond)                                                             \
+    (GSL_LIKELY(cond) ? static_cast<void>(0) : gsl::details::terminate())
+
+#elif defined(GSL_UNENFORCED_ON_CONTRACT_VIOLATION)
+
+#define GSL_CONTRACT_CHECK(type, cond) GSL_ASSUME(cond)
+
+#endif
+
+#define Expects(cond) GSL_CONTRACT_CHECK("Precondition", cond)
+#define Ensures(cond) GSL_CONTRACT_CHECK("Postcondition", cond)
+
+#endif // GSL_CONTRACTS_H
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/include/gsl/pointers 5.1-1/demo/Polyhedron/Plugins/Mesh_3/include/gsl/pointers
--- 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/include/gsl/pointers	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Mesh_3/include/gsl/pointers	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,198 @@
+///////////////////////////////////////////////////////////////////////////////
+//
+// Copyright (c) 2015 Microsoft Corporation. All rights reserved.
+//
+// This code is licensed under the MIT License (MIT).
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+// THE SOFTWARE.
+//
+///////////////////////////////////////////////////////////////////////////////
+
+#ifndef GSL_POINTERS_H
+#define GSL_POINTERS_H
+
+#include <gsl/gsl_assert>  // for Ensures, Expects
+
+#include <algorithm>    // for forward
+#include <iosfwd>       // for ptrdiff_t, nullptr_t, ostream, size_t
+#include <memory>       // for shared_ptr, unique_ptr
+#include <system_error> // for hash
+#include <type_traits>  // for enable_if_t, is_convertible, is_assignable
+
+#if defined(_MSC_VER) && _MSC_VER < 1910
+#pragma push_macro("constexpr")
+#define constexpr /*constexpr*/
+
+#endif                          // defined(_MSC_VER) && _MSC_VER < 1910
+
+namespace gsl
+{
+
+//
+// GSL.owner: ownership pointers
+//
+using std::unique_ptr;
+using std::shared_ptr;
+
+//
+// owner
+//
+// owner<T> is designed as a bridge for code that must deal directly with owning pointers for some reason
+//
+// T must be a pointer type
+// - disallow construction from any type other than pointer type
+//
+template <class T, class = std::enable_if_t<std::is_pointer<T>::value>>
+using owner = T;
+
+//
+// not_null
+//
+// Restricts a pointer or smart pointer to only hold non-null values.
+//
+// Has zero size overhead over T.
+//
+// If T is a pointer (i.e. T == U*) then
+// - allow construction from U*
+// - disallow construction from nullptr_t
+// - disallow default construction
+// - ensure construction from null U* fails
+// - allow implicit conversion to U*
+//
+template <class T>
+class not_null
+{
+public:
+    static_assert(std::is_assignable<T&, std::nullptr_t>::value, "T cannot be assigned nullptr.");
+
+    template <typename U, typename = std::enable_if_t<std::is_convertible<U, T>::value>>
+    constexpr explicit not_null(U&& u) : ptr_(std::forward<U>(u))
+    {
+        Expects(ptr_ != nullptr);
+    }
+
+    template <typename = std::enable_if_t<!std::is_same<std::nullptr_t, T>::value>>
+    constexpr explicit not_null(T u) : ptr_(u)
+    {
+        Expects(ptr_ != nullptr);
+    }
+
+    template <typename U, typename = std::enable_if_t<std::is_convertible<U, T>::value>>
+    constexpr not_null(const not_null<U>& other) : not_null(other.get())
+    {
+    }
+
+    not_null(not_null&& other) = default;
+    not_null(const not_null& other) = default;
+    not_null& operator=(const not_null& other) = default;
+
+    constexpr T get() const
+    {
+        Ensures(ptr_ != nullptr);
+        return ptr_;
+    }
+
+    constexpr operator T() const { return get(); }
+    constexpr T operator->() const { return get(); }
+    constexpr decltype(auto) operator*() const { return *get(); } 
+
+    // prevents compilation when someone attempts to assign a null pointer constant
+    not_null(std::nullptr_t) = delete;
+    not_null& operator=(std::nullptr_t) = delete;
+
+    // unwanted operators...pointers only point to single objects!
+    not_null& operator++() = delete;
+    not_null& operator--() = delete;
+    not_null operator++(int) = delete;
+    not_null operator--(int) = delete;
+    not_null& operator+=(std::ptrdiff_t) = delete;
+    not_null& operator-=(std::ptrdiff_t) = delete;
+    void operator[](std::ptrdiff_t) const = delete;
+
+private:
+    T ptr_;
+};
+
+template <class T>
+auto make_not_null(T&& t) {
+    return gsl::not_null<std::remove_cv_t<std::remove_reference_t<T>>>{std::forward<T>(t)};
+}
+
+template <class T>
+std::ostream& operator<<(std::ostream& os, const not_null<T>& val)
+{
+    os << val.get();
+    return os;
+}
+
+template <class T, class U>
+auto operator==(const not_null<T>& lhs, const not_null<U>& rhs) -> decltype(lhs.get() == rhs.get())
+{
+    return lhs.get() == rhs.get();
+}
+
+template <class T, class U>
+auto operator!=(const not_null<T>& lhs, const not_null<U>& rhs) -> decltype(lhs.get() != rhs.get())
+{
+    return lhs.get() != rhs.get();
+}
+
+template <class T, class U>
+auto operator<(const not_null<T>& lhs, const not_null<U>& rhs) -> decltype(lhs.get() < rhs.get())
+{
+    return lhs.get() < rhs.get();
+}
+
+template <class T, class U>
+auto operator<=(const not_null<T>& lhs, const not_null<U>& rhs) -> decltype(lhs.get() <= rhs.get())
+{
+    return lhs.get() <= rhs.get();
+}
+
+template <class T, class U>
+auto operator>(const not_null<T>& lhs, const not_null<U>& rhs) -> decltype(lhs.get() > rhs.get())
+{
+    return lhs.get() > rhs.get();
+}
+
+template <class T, class U>
+auto operator>=(const not_null<T>& lhs, const not_null<U>& rhs) -> decltype(lhs.get() >= rhs.get())
+{
+    return lhs.get() >= rhs.get();
+}
+
+// more unwanted operators
+template <class T, class U>
+std::ptrdiff_t operator-(const not_null<T>&, const not_null<U>&) = delete;
+template <class T>
+not_null<T> operator-(const not_null<T>&, std::ptrdiff_t) = delete;
+template <class T>
+not_null<T> operator+(const not_null<T>&, std::ptrdiff_t) = delete;
+template <class T>
+not_null<T> operator+(std::ptrdiff_t, const not_null<T>&) = delete;
+
+} // namespace gsl
+
+namespace std
+{
+template <class T>
+struct hash<gsl::not_null<T>>
+{
+    std::size_t operator()(const gsl::not_null<T>& value) const { return hash<T>{}(value); }
+};
+
+} // namespace std
+
+#if defined(_MSC_VER) && _MSC_VER < 1910
+#undef constexpr
+#pragma pop_macro("constexpr")
+
+#endif // defined(_MSC_VER) && _MSC_VER < 1910
+
+#endif // GSL_POINTERS_H
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/Io_image_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Mesh_3/Io_image_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/Io_image_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Mesh_3/Io_image_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -296,6 +296,11 @@ public:
       items.pop_front();
     return ok;
   }
+  bool isDefaultLoader(const Scene_item* item) const override{
+    if(qobject_cast<const Scene_image_item*>(item))
+      return true;
+    return false;
+  }
   QString name() const override{ return "segmented images"; }
 
 
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/Local_optimizers_dialog.ui 5.1-1/demo/Polyhedron/Plugins/Mesh_3/Local_optimizers_dialog.ui
--- 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/Local_optimizers_dialog.ui	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Mesh_3/Local_optimizers_dialog.ui	2020-09-08 12:12:20.000000000 +0000
@@ -6,7 +6,7 @@
    <rect>
     <x>0</x>
     <y>0</y>
-    <width>546</width>
+    <width>556</width>
     <height>317</height>
    </rect>
   </property>
@@ -50,85 +50,52 @@
      <property name="title">
       <string>Parameters</string>
      </property>
-     <layout class="QGridLayout" name="gridLayout_2" columnstretch="0,1,0">
-      <item row="0" column="0">
-       <widget class="QLabel" name="timeLabel">
+     <layout class="QGridLayout" name="gridLayout_2" columnstretch="0,1,0,0">
+      <item row="1" column="3">
+       <widget class="QCheckBox" name="noBound">
         <property name="text">
-         <string>Max CPU running time (s)</string>
-        </property>
-        <property name="alignment">
-         <set>Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter</set>
-        </property>
-        <property name="buddy">
-         <cstring>maxTime</cstring>
+         <string>No angle bound</string>
         </property>
        </widget>
       </item>
-      <item row="0" column="1">
-       <widget class="QDoubleSpinBox" name="maxTime">
-        <property name="minimumSize">
-         <size>
-          <width>100</width>
-          <height>0</height>
-         </size>
-        </property>
-        <property name="decimals">
-         <number>1</number>
-        </property>
-        <property name="maximum">
-         <double>9999.000000000000000</double>
+      <item row="1" column="0">
+       <widget class="QLabel" name="sliverLabel">
+        <property name="text">
+         <string>Min. dihedral angle (degree)</string>
         </property>
-        <property name="value">
-         <double>60.000000000000000</double>
+        <property name="alignment">
+         <set>Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter</set>
         </property>
        </widget>
       </item>
-      <item row="0" column="2">
+      <item row="0" column="3">
        <widget class="QCheckBox" name="noTimeLimit">
         <property name="text">
          <string>No time limit</string>
         </property>
        </widget>
       </item>
-      <item row="1" column="0">
-       <widget class="QLabel" name="sliverLabel">
+      <item row="0" column="0">
+       <widget class="QLabel" name="timeLabel">
         <property name="text">
-         <string>Min. dihedral angle (degree)</string>
+         <string>Max CPU running time (s)</string>
         </property>
         <property name="alignment">
          <set>Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter</set>
         </property>
-        <property name="buddy">
-         <cstring>sliverBound</cstring>
-        </property>
        </widget>
       </item>
-      <item row="1" column="1">
-       <widget class="QDoubleSpinBox" name="sliverBound">
-        <property name="minimumSize">
-         <size>
-          <width>100</width>
-          <height>0</height>
-         </size>
-        </property>
-        <property name="readOnly">
-         <bool>false</bool>
-        </property>
-        <property name="decimals">
-         <number>1</number>
-        </property>
-        <property name="maximum">
-         <double>180.000000000000000</double>
-        </property>
-        <property name="value">
-         <double>8.000000000000000</double>
+      <item row="0" column="2">
+       <widget class="DoubleEdit" name="maxTime">
+        <property name="text">
+         <string>60.0</string>
         </property>
        </widget>
       </item>
       <item row="1" column="2">
-       <widget class="QCheckBox" name="noBound">
+       <widget class="DoubleEdit" name="sliverBound">
         <property name="text">
-         <string>No angle bound</string>
+         <string>8.0</string>
         </property>
        </widget>
       </item>
@@ -180,6 +147,13 @@
    </item>
   </layout>
  </widget>
+ <customwidgets>
+  <customwidget>
+   <class>DoubleEdit</class>
+   <extends>QLineEdit</extends>
+   <header>CGAL_double_edit.h</header>
+  </customwidget>
+ </customwidgets>
  <resources/>
  <connections>
   <connection>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/Mesh_3_plugin_cgal_code.cpp 5.1-1/demo/Polyhedron/Plugins/Mesh_3/Mesh_3_plugin_cgal_code.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/Mesh_3_plugin_cgal_code.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Mesh_3/Mesh_3_plugin_cgal_code.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -87,6 +87,14 @@ Meshing_thread* cgal_code_mesh_3(QList<c
   std::cerr << " done (" << timer.time() * 1000 << " ms)" << std::endl;
 
   Scene_c3t3_item* p_new_item = new Scene_c3t3_item(surface_only);
+  if(polylines.empty()) {
+    if(protect_features) {
+      p_new_item->set_sharp_edges_angle(sharp_edges_angle);
+    }
+    else if (protect_borders) {
+      p_new_item->set_detect_borders(true);
+    }
+  }
 
   QString tooltip = QString("<div>From \"") + filename +
     QString("\" with the following mesh parameters"
@@ -161,6 +169,9 @@ Meshing_thread* cgal_code_mesh_3(const I
   param.tet_shape = tet_shape;
   param.edge_sizing = edge_size;
   param.manifold = manifold;
+  param.detect_connected_components = false; // to avoid random values
+                                             // in the debug displays
+
 
   typedef ::Mesh_function<Function_mesh_domain,
                           Mesh_fnt::Implicit_domain_tag> Mesh_function;
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/Mesh_3_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Mesh_3/Mesh_3_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/Mesh_3_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Mesh_3/Mesh_3_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -19,6 +19,16 @@
 #include <QUrl>
 #include <fstream>
 
+#include <gsl/pointers>
+
+// Small addition from GSL v2.0.0:
+template <class T>
+auto make_not_null(T&& t) {
+    return gsl::not_null<std::remove_cv_t<std::remove_reference_t<T>>>{std::forward<T>(t)};
+}
+
+#include <boost/variant/variant.hpp>
+#include <boost/optional/optional.hpp>
 #include "Scene_polylines_item.h"
 
 #ifdef CGAL_MESH_3_DEMO_ACTIVATE_IMPLICIT_FUNCTIONS
@@ -26,6 +36,7 @@
 #endif
 #ifdef CGAL_MESH_3_DEMO_ACTIVATE_SEGMENTED_IMAGES
 #include "Scene_image_item.h"
+#include "Image_type.h"
 #endif
 
 #include "Meshing_thread.h"
@@ -39,6 +50,7 @@ const QColor default_mesh_color(45,169,7
 
 #include "Mesh_3_plugin_cgal_code.h" // declare functions `cgal_code_mesh_3`
 #include "split_polylines.h"
+#include <CGAL/Mesh_facet_topology.h>
 
 class Mesh_3_plugin :
   public QObject,
@@ -48,6 +60,25 @@ class Mesh_3_plugin :
   Q_INTERFACES(CGAL::Three::Polyhedron_demo_plugin_interface)
   Q_PLUGIN_METADATA(IID "com.geometryfactory.PolyhedronDemo.PluginInterface/1.0" FILE "mesh_3_plugin.json")
 
+  Q_PROPERTY(double angle READ get_angle WRITE set_angle);
+  Q_PROPERTY(double sharp_edges_angle_bound
+             READ get_sharp_edges_angle_bound
+             WRITE set_sharp_edges_angle_bound);
+  Q_PROPERTY(double edges_sizing READ get_edges_sizing WRITE set_edges_sizing);
+  Q_PROPERTY(double facets_sizing READ get_facets_sizing WRITE set_facets_sizing);
+  Q_PROPERTY(double approx READ get_approx WRITE set_approx);
+  Q_PROPERTY(double tets_sizing READ get_tets_sizing WRITE set_tets_sizing);
+  Q_PROPERTY(double tets_shape READ get_tets_shape WRITE set_tets_shape);
+  Q_PROPERTY(bool protect_features READ get_protect_features WRITE set_protect_features);
+  Q_PROPERTY(bool protect_borders READ get_protect_borders WRITE set_protect_borders);
+  Q_PROPERTY(bool manifold_criterion READ get_manifold_criterion WRITE set_manifold_criterion);
+
+  typedef CGAL::Mesh_facet_topology Mesh_facet_topology;
+  Q_ENUMS(Mesh_facet_topology)
+  Q_PROPERTY(Mesh_facet_topology facet_topology
+             READ get_facet_topology
+             WRITE set_facet_topology)
+
 public:
   void init(QMainWindow* mainWindow,
             CGAL::Three::Scene_interface* scene_interface,
@@ -91,37 +122,61 @@ public:
       return qobject_cast<Scene_polylines_item*>
         (scene->item(scene->mainSelectionIndex())) != nullptr;
     }
-#ifdef CGAL_MESH_3_DEMO_ACTIVATE_IMPLICIT_FUNCTIONS
-    if(qobject_cast<Scene_implicit_function_item*>
-       (scene->item(scene->mainSelectionIndex())) != nullptr) {
-      return true;
-    }
-#endif
-#ifdef CGAL_MESH_3_DEMO_ACTIVATE_SEGMENTED_IMAGES
-    if( qobject_cast<Scene_image_item*>
-        (scene->item(scene->mainSelectionIndex())) != nullptr ) {
-      return true;
-    }
-#endif
-    for(int ind: scene->selectionIndices()){
-      Scene_surface_mesh_item* sm_item
-          = qobject_cast<Scene_surface_mesh_item*>(scene->item(ind));
-      if(nullptr == sm_item)
-        return false;
-    }
-    return true;
+    return !get_items_or_return_error_string();
   }
 
 public Q_SLOTS:
+  boost::optional<QString> get_items_or_return_error_string() const;
+  void set_defaults();
   void mesh_3_volume();
   void mesh_3_surface();
-  void mesh_3_surface_with_defaults() { mesh_3(true, true); }
+  void mesh_3_surface_with_defaults() {
+    mesh_3(Mesh_type::SURFACE_ONLY, Dialog_choice::NO_DIALOG);
+  }
+  void mesh_3_volume_with_defaults() {
+    mesh_3(Mesh_type::VOLUME, Dialog_choice::NO_DIALOG);
+  }
+  void mesh_3(bool with_dialog) { // compatibility with old Qt Scripts
+    return mesh_3(
+        Mesh_type::VOLUME,
+        with_dialog ? Dialog_choice::DIALOG : Dialog_choice::NO_DIALOG);
+  }
   void splitPolylines();
   void meshing_done(Meshing_thread* t);
   void status_report(QString str);
 
+public Q_SLOTS:
+  void set_angle(const double v) { angle = v; };
+  void set_sharp_edges_angle_bound(const double v) {
+    sharp_edges_angle_bound = v;
+  }
+  void set_edges_sizing(const double v) { edges_sizing = v; };
+  void set_facets_sizing(const double v) { facets_sizing = v; };
+  void set_approx(const double v) { approx = v; };
+  void set_tets_sizing(const double v) { tets_sizing = v; };
+  void set_tets_shape(const double v) { tets_shape = v; };
+  void set_manifold_criterion(const bool v) { manifold_criterion = v; }
+  void set_facet_topology(const CGAL::Mesh_facet_topology v) {  facet_topology = v; }
+  void set_protect_features(const bool v) { protect_features = v; };
+  void set_protect_borders(const bool v) { protect_borders = v; };
+
+  double get_angle() { return angle; };
+  double get_sharp_edges_angle_bound() { return sharp_edges_angle_bound; }
+  double get_edges_sizing() { return edges_sizing; };
+  double get_facets_sizing() { return facets_sizing; };
+  double get_approx() { return approx; };
+  double get_tets_sizing() { return tets_sizing; };
+  double get_tets_shape() { return tets_shape; };
+  bool get_manifold_criterion() { return manifold_criterion; };
+  CGAL::Mesh_facet_topology get_facet_topology() { return facet_topology; };
+  bool get_protect_features() { return protect_features; };
+  bool get_protect_borders() { return protect_borders; };
+
+
 private:
-  void mesh_3(const bool surface_only, const bool use_defaults = false);
+  enum class Mesh_type : bool { VOLUME, SURFACE_ONLY };
+  enum class Dialog_choice : bool { NO_DIALOG, DIALOG };
+  void mesh_3(const Mesh_type mesh_type, const Dialog_choice dialog = Dialog_choice::DIALOG);
   void launch_thread(Meshing_thread* mesh_thread);
   void treat_result(Scene_item& source_item, Scene_c3t3_item* result_item) const;
 
@@ -136,6 +191,48 @@ private:
   CGAL::Three::Scene_interface* scene;
   QMainWindow* mw;
   bool as_facegraph;
+
+  double angle;
+  double sharp_edges_angle_bound;
+  int sizing_decimals;
+  double approx;
+  int approx_decimals;
+  double edges_sizing;
+  double facets_sizing;
+  double tets_sizing;
+  double tets_shape;
+  bool manifold_criterion;
+  CGAL::Mesh_facet_topology facet_topology;
+  bool protect_features;
+  bool protect_borders;
+
+  struct Polyhedral_mesh_items {
+    Polyhedral_mesh_items() noexcept
+      : sm_items{}, bounding_sm_item(nullptr), polylines_item(nullptr) {}
+    QList<gsl::not_null<Scene_surface_mesh_item*>> sm_items;
+    Scene_surface_mesh_item* bounding_sm_item;
+    Scene_polylines_item* polylines_item;
+  };
+  struct Image_mesh_items {
+    Image_mesh_items(gsl::not_null<Scene_image_item*> ptr) : image_item(ptr) {}
+    gsl::not_null<Scene_image_item*> image_item;
+    Scene_polylines_item* polylines_item = nullptr;
+  };
+  struct Implicit_mesh_items {
+    gsl::not_null<Scene_implicit_function_item*> function_item;
+  };
+  enum Item_types {
+    POLYHEDRAL_MESH_ITEMS,
+    IMAGE_MESH_ITEMS,
+    IMPLICIT_MESH_ITEMS
+  };
+  mutable boost::optional<boost::variant<Polyhedral_mesh_items,
+                                         Image_mesh_items,
+                                         Implicit_mesh_items>>
+      items;
+  mutable bool features_protection_available = false;
+  mutable Scene_item* item = nullptr;
+  mutable CGAL::Three::Scene_interface::Bbox bbox = {};
 }; // end class Mesh_3_plugin
 
 double
@@ -168,124 +265,181 @@ void Mesh_3_plugin::splitPolylines() {
 
 void Mesh_3_plugin::mesh_3_surface()
 {
-  mesh_3(true);
+  mesh_3(Mesh_type::SURFACE_ONLY);
 }
 void Mesh_3_plugin::mesh_3_volume()
 {
-  mesh_3(false);
+  mesh_3(Mesh_type::VOLUME);
 }
 
-void Mesh_3_plugin::mesh_3(const bool surface_only, const bool use_defaults)
-{
-  QList<Scene_surface_mesh_item*> sm_items;
-  Scene_surface_mesh_item* bounding_sm_item = nullptr;
-  Scene_implicit_function_item* function_item = nullptr;
-  Scene_image_item* image_item = nullptr;
-  Scene_polylines_item* polylines_item = nullptr;
-
-  for(int ind: scene->selectionIndices()) {
-    Scene_surface_mesh_item* sm_item =
-      qobject_cast<Scene_surface_mesh_item*>(scene->item(ind));
-    if(sm_item) {
-      sm_items.push_back(sm_item);
-      if(is_closed(*sm_item->polyhedron())) {
-        bounding_sm_item = sm_item;
+boost::optional<QString> Mesh_3_plugin::get_items_or_return_error_string() const {
+  using boost::get;
+  items = {};
+  features_protection_available = false;
+  item = nullptr;
+  for (int ind : scene->selectionIndices()) {
+    try {
+      if (auto sm_item =
+              qobject_cast<Scene_surface_mesh_item*>(scene->item(ind))) {
+        if (!items) items = Polyhedral_mesh_items{};
+        auto& poly_items = get<Polyhedral_mesh_items>(*items);
+        auto& sm_items = poly_items.sm_items;
+        sm_items.push_back(make_not_null(sm_item));
+        if (is_closed(*sm_item->polyhedron())) {
+          poly_items.bounding_sm_item = sm_item;
+        }
       }
-    }
-#ifdef CGAL_MESH_3_DEMO_ACTIVATE_IMPLICIT_FUNCTIONS
-    else if(function_item == nullptr &&
-            nullptr !=
-            (function_item = qobject_cast<Scene_implicit_function_item*>(scene->item(ind))))
-    {}
-#endif
-#ifdef CGAL_MESH_3_DEMO_ACTIVATE_SEGMENTED_IMAGES
-    else if(image_item == nullptr &&
-            nullptr != (image_item = qobject_cast<Scene_image_item*>(scene->item(ind))))
-    {}
-#endif
-    else if(polylines_item == nullptr &&
-            nullptr != (polylines_item = qobject_cast<Scene_polylines_item*>(scene->item(ind))))
-    {}
-    else {
-      QMessageBox::warning(mw, tr("Mesh_3 plugin"),
-                           tr("Wrong selection of items"));
-      return;
-    }
-  }
-  Scene_item* item = nullptr;
-  const bool more_than_one_item = sm_items.size() > 1;
-  bool features_protection_available = false;
-  if(!sm_items.empty())
-  {
-    for(auto sm_item : sm_items) {
-      if(nullptr == sm_item->polyhedron()) {
-        QApplication::restoreOverrideCursor();
-        QMessageBox::critical(mw, tr("Mesh_3 plugin"),
-                              tr("ERROR: no data in selected item %1").arg(sm_item->name()));
-        return;
+#  ifdef CGAL_MESH_3_DEMO_ACTIVATE_IMPLICIT_FUNCTIONS
+      else if (auto function_item = qobject_cast<Scene_implicit_function_item*>(
+                   scene->item(ind))) {
+        if (!items)
+          items = Implicit_mesh_items{make_not_null(function_item)};
+        else
+          return tr(
+              "An implicit function cannot be mixed with other items type");
+      }
+#  endif
+#  ifdef CGAL_MESH_3_DEMO_ACTIVATE_SEGMENTED_IMAGES
+      else if (auto image_item =
+                   qobject_cast<Scene_image_item*>(scene->item(ind))) {
+        if (!items)
+          items = Image_mesh_items{make_not_null(image_item)};
+        else
+          return tr("An image items cannot be mixed with other items type");
+      }
+#  endif
+      else if (auto polylines_item =
+                   qobject_cast<Scene_polylines_item*>(scene->item(ind))) {
+        if (!items) items = Polyhedral_mesh_items{};
+        auto poly_items_ptr = get<Polyhedral_mesh_items>(&*items);
+        if(poly_items_ptr) {
+          if (poly_items_ptr->polylines_item) {
+            return tr("Only one polyline item is accepted");
+          } else {
+            poly_items_ptr->polylines_item = polylines_item;
+          }
+        } else {
+          auto image_items = get<Image_mesh_items>(*items);
+          if (image_items.polylines_item) {
+            return tr("Only one polyline item is accepted");
+          } else {
+            image_items.polylines_item = polylines_item;
+          }
+        }
+      } else {
+        return tr("Wrong selection of items");
       }
-      if (!is_triangle_mesh(*sm_item->polyhedron()))
-      {
-        QApplication::restoreOverrideCursor();
-        QMessageBox::warning(mw, tr("Mesh_3 plugin"),
-                             tr("Selected Scene_surface_mesh_item %1 is not triangulated.")
-                             .arg(sm_item->name()));
-        return;
+    } catch (const boost::bad_get&) { return tr("Wrong selection of items"); }
+  } // end for loop on selected items
+  if (!items) { return tr("Selected objects can't be meshed"); }
+  item = nullptr;
+  features_protection_available = false;
+  if (auto poly_items = get<Polyhedral_mesh_items>(&*items)) {
+    auto& sm_items = poly_items->sm_items;
+    for (auto sm_item : sm_items) {
+      if (nullptr == sm_item->polyhedron()) {
+        return tr("ERROR: no data in selected item %1").arg(sm_item->name());
       }
-      if(sm_item->getNbIsolatedvertices() != 0)
-      {
-        QApplication::restoreOverrideCursor();
-        QMessageBox::critical(mw, tr(""), tr("ERROR: there are isolated vertices in this mesh."));
-        return;
+      if (!is_triangle_mesh(*sm_item->polyhedron())) {
+        return tr("Selected Scene_surface_mesh_item %1 is not triangulated.")
+            .arg(sm_item->name());
+      }
+      if (sm_item->getNbIsolatedvertices() != 0) {
+        return tr("ERROR: there are isolated vertices in this mesh.");
       }
     }
-    item = sm_items.front();
+    if (!sm_items.empty()) item = sm_items.front();
     features_protection_available = true;
   }
-#ifdef CGAL_MESH_3_DEMO_ACTIVATE_IMPLICIT_FUNCTIONS
-  else if (nullptr != function_item) { item = function_item; }
-#endif
-#ifdef CGAL_MESH_3_DEMO_ACTIVATE_SEGMENTED_IMAGES
-  else if (nullptr != image_item)
-  {
+#  ifdef CGAL_MESH_3_DEMO_ACTIVATE_IMPLICIT_FUNCTIONS
+  else if (auto implicit_mesh_items = get<Implicit_mesh_items>(&*items)) {
+    item = implicit_mesh_items->function_item;
+  }
+#  endif
+#  ifdef CGAL_MESH_3_DEMO_ACTIVATE_SEGMENTED_IMAGES
+  else if (auto image_mesh_items = get<Image_mesh_items>(&*items)) {
+    auto& image_item = image_mesh_items->image_item;
     item = image_item;
     features_protection_available = true;
 
     bool fit_wrdtp = true;
     std::size_t img_wdim = image_item->image()->image()->wdim;
     WORD_KIND img_wordKind = image_item->image()->image()->wordKind;
-    //check if the word type fits the hardcoded values in the plugin
-    if(image_item->isGray())
-    {
-      if(img_wordKind != WK_FLOAT)
+    // check if the word type fits the hardcoded values in the plugin
+    if (image_item->isGray()) {
+      if (img_wordKind != WK_FLOAT)
         fit_wrdtp = false;
-      else
-        if(img_wdim != 4)
-          fit_wrdtp = false;
-    }
-    else
-    {
-      if(img_wordKind != WK_FIXED)
+      else if (img_wdim != 4)
         fit_wrdtp = false;
-      else
-        if(img_wdim != 1)
-          fit_wrdtp = false;
-    }
-    if(!fit_wrdtp)
-    {
-      QMessageBox::warning(mw, tr(""),
-                           tr("Selected object can't be meshed because the image's word type is not supported by this plugin."));
-      return;
+    } else {
+      if (img_wordKind != WK_FIXED)
+        fit_wrdtp = false;
+      else if (img_wdim != 1)
+        fit_wrdtp = false;
+    }
+    if (!fit_wrdtp) {
+      return tr(
+          "Selected object can't be meshed because the image's word type is "
+          "not supported by this plugin.");
     }
   }
-#endif
+#  endif
+
+  if(item) {
+    bbox = item->bbox();
+    if (auto poly_items = get<Polyhedral_mesh_items>(&*items)) {
+      for (auto it : poly_items->sm_items) {
+        bbox = bbox + it->bbox();
+      }
+      if (poly_items->polylines_item)
+        bbox = bbox + poly_items->polylines_item->bbox();
+    }
+  }
+  return {};
+}
 
-  if (nullptr == item)
-  {
-    QMessageBox::warning(mw, tr(""),
-                         tr("Selected object can't be meshed"));
+void Mesh_3_plugin::set_defaults() {
+  auto error = get_items_or_return_error_string();
+  if(error) return;
+  double diag = CGAL::sqrt((bbox.xmax()-bbox.xmin())*(bbox.xmax()-bbox.xmin()) + (bbox.ymax()-bbox.ymin())*(bbox.ymax()-bbox.ymin()) + (bbox.zmax()-bbox.zmin())*(bbox.zmax()-bbox.zmin()));
+  facets_sizing = get_approximate(diag * 0.05, 2, sizing_decimals);
+  edges_sizing = facets_sizing;
+  tets_sizing = facets_sizing;
+  angle = 25.;
+  sharp_edges_angle_bound = 60.;
+  approx = get_approximate(diag * 0.005, 2, approx_decimals);
+}
+
+void Mesh_3_plugin::mesh_3(const Mesh_type mesh_type,
+                           const Dialog_choice dialog_choice) {
+  CGAL_assertion(static_cast<bool>(items));
+  auto error_string = get_items_or_return_error_string();
+  if (error_string) {
+    QApplication::restoreOverrideCursor();
+    QMessageBox::warning(mw, tr("Mesh_3 plugin"), *error_string);
     return;
   }
+  using boost::get;
+  const bool more_than_one_item =
+      get<Polyhedral_mesh_items>(&*items) &&
+      (get<Polyhedral_mesh_items>(&*items)->sm_items.size() > 1);
+
+  Scene_image_item* image_item =
+      get<Image_mesh_items>(&*items)
+          ? get<Image_mesh_items>(&*items)->image_item.get()
+          : nullptr;
+  Scene_surface_mesh_item* bounding_sm_item =
+      get<Polyhedral_mesh_items>(&*items)
+          ? get<Polyhedral_mesh_items>(&*items)->bounding_sm_item
+          : nullptr;
+  Scene_polylines_item* polylines_item =
+      get<Polyhedral_mesh_items>(&*items)
+          ? get<Polyhedral_mesh_items>(&*items)->polylines_item
+          : nullptr;
+  Scene_implicit_function_item* function_item =
+      get<Implicit_mesh_items>(&*items)
+          ? get<Implicit_mesh_items>(&*items)->function_item.get()
+          : nullptr;
   // -----------------------------------
   // Create Mesh dialog
   // -----------------------------------
@@ -293,137 +447,144 @@ void Mesh_3_plugin::mesh_3(const bool su
   Ui::Meshing_dialog ui;
   ui.setupUi(&dialog);
 
+  ui.facetAngle->setRange(0.0, 30.0);
+  ui.facetAngle->setValue(25.0);
+  ui.edgeSizing->setMinimum(0.0);
+  ui.sharpEdgesAngle->setMaximum(180);
+  ui.iso_value_spinBox->setRange(-65536.0, 65536.0);
+  ui.tetShape->setMinimum(1.0);
+
   ui.advanced->setVisible(false);
   connect(ui.facetTopologyLabel,
           &QLabel::linkActivated,
           &QDesktopServices::openUrl);
 
-  dialog.setWindowFlags(Qt::Dialog|Qt::CustomizeWindowHint|Qt::WindowCloseButtonHint);
-  connect(ui.buttonBox, SIGNAL(accepted()),
-          &dialog, SLOT(accept()));
-  connect(ui.buttonBox, SIGNAL(rejected()),
-          &dialog, SLOT(reject()));
+  dialog.setWindowFlags(Qt::Dialog | Qt::CustomizeWindowHint |
+                        Qt::WindowCloseButtonHint);
+  connect(ui.buttonBox, SIGNAL(accepted()), &dialog, SLOT(accept()));
+  connect(ui.buttonBox, SIGNAL(rejected()), &dialog, SLOT(reject()));
 
   // Connect checkboxes to spinboxes
-  connect(ui.noApprox, SIGNAL(toggled(bool)),
-          ui.approx,   SLOT(setEnabled(bool)));
-
-  connect(ui.noFacetSizing, SIGNAL(toggled(bool)),
-          ui.facetSizing,   SLOT(setEnabled(bool)));
-
-  connect(ui.noAngle,    SIGNAL(toggled(bool)),
-          ui.facetAngle, SLOT(setEnabled(bool)));
-
-  connect(ui.noTetSizing, SIGNAL(toggled(bool)),
-          ui.tetSizing,   SLOT(setEnabled(bool)));
-
-  connect(ui.noTetShape, SIGNAL(toggled(bool)),
-          ui.tetShape,   SLOT(setEnabled(bool)));
-
-  connect(ui.protect, SIGNAL(toggled(bool)),
-          ui.noEdgeSizing,   SLOT(setEnabled(bool)));
-
-  connect(ui.protect, SIGNAL(toggled(bool)),
-          ui.noEdgeSizing,   SLOT(setChecked(bool)));
-
-  connect(ui.noEdgeSizing, SIGNAL(toggled(bool)),
-          ui.edgeLabel,   SLOT(setEnabled(bool)));
-
-  connect(ui.noEdgeSizing, SIGNAL(toggled(bool)),
-          ui.edgeSizing,   SLOT(setEnabled(bool)));
+  connect(
+      ui.noApprox, SIGNAL(toggled(bool)), ui.approx, SLOT(setEnabled(bool)));
 
-  connect(ui.protect, SIGNAL(toggled(bool)),
-          ui.sharpEdgesAngle, SLOT(setEnabled(bool)));
+  connect(ui.noFacetSizing,
+          SIGNAL(toggled(bool)),
+          ui.facetSizing,
+          SLOT(setEnabled(bool)));
+
+  connect(
+      ui.noAngle, SIGNAL(toggled(bool)), ui.facetAngle, SLOT(setEnabled(bool)));
+
+  connect(ui.noTetSizing,
+          SIGNAL(toggled(bool)),
+          ui.tetSizing,
+          SLOT(setEnabled(bool)));
+
+  connect(ui.noTetShape,
+          SIGNAL(toggled(bool)),
+          ui.tetShape,
+          SLOT(setEnabled(bool)));
+
+  connect(ui.protect,
+          SIGNAL(toggled(bool)),
+          ui.noEdgeSizing,
+          SLOT(setEnabled(bool)));
+
+  connect(ui.protect,
+          SIGNAL(toggled(bool)),
+          ui.noEdgeSizing,
+          SLOT(setChecked(bool)));
+
+  connect(ui.noEdgeSizing,
+          SIGNAL(toggled(bool)),
+          ui.edgeLabel,
+          SLOT(setEnabled(bool)));
+
+  connect(ui.noEdgeSizing,
+          SIGNAL(toggled(bool)),
+          ui.edgeSizing,
+          SLOT(setEnabled(bool)));
+
+  connect(ui.protect,
+          SIGNAL(toggled(bool)),
+          ui.sharpEdgesAngle,
+          SLOT(setEnabled(bool)));
+
+  connect(ui.protect,
+          SIGNAL(toggled(bool)),
+          ui.sharpEdgesAngleLabel,
+          SLOT(setEnabled(bool)));
+
+  connect(ui.protect,
+          SIGNAL(toggled(bool)),
+          ui.protectEdges,
+          SLOT(setEnabled(bool)));
 
-  connect(ui.protect, SIGNAL(toggled(bool)),
-          ui.sharpEdgesAngleLabel, SLOT(setEnabled(bool)));
+  QString item_name =
+      more_than_one_item ? QString("%1...").arg(item->name()) : item->name();
 
-  connect(ui.protect, SIGNAL(toggled(bool)),
-          ui.protectEdges, SLOT(setEnabled(bool)));
-
-  QString item_name = more_than_one_item ?
-    QString("%1...").arg(item->name()) :
-    item->name();
-
-  // Set default parameters
-  CGAL::Three::Scene_interface::Bbox bbox = item->bbox();
-  if(more_than_one_item) {
-    for(auto it: sm_items) {
-      bbox = bbox + it->bbox();
-    }
-  }
   ui.objectName->setText(item_name);
   ui.objectNameSize->setText(tr("Object bbox size (w,h,d):  <b>%1</b>,  <b>%2</b>,  <b>%3</b>")
                              .arg(bbox.xmax() - bbox.xmin(),0,'g',3)
                              .arg(bbox.ymax() - bbox.ymin(),0,'g',3)
                              .arg(bbox.zmax() - bbox.zmin(),0,'g',3) );
 
+  set_defaults();
   double diag = CGAL::sqrt((bbox.xmax()-bbox.xmin())*(bbox.xmax()-bbox.xmin()) + (bbox.ymax()-bbox.ymin())*(bbox.ymax()-bbox.ymin()) + (bbox.zmax()-bbox.zmin())*(bbox.zmax()-bbox.zmin()));
-  int decimals = 0;
-  double sizing_default = get_approximate(diag * 0.05, 2, decimals);
-  ui.facetSizing->setDecimals(-decimals+2);
-  ui.facetSizing->setSingleStep(std::pow(10.,decimals));
   ui.facetSizing->setRange(diag * 10e-6, // min
                            diag); // max
-  ui.facetSizing->setValue(sizing_default); // default value
-  ui.edgeSizing->setValue(sizing_default);
+  ui.facetSizing->setValue(facets_sizing);
+  ui.edgeSizing->setValue(edges_sizing);
 
-  ui.tetSizing->setDecimals(-decimals+2);
-  ui.tetSizing->setSingleStep(std::pow(10.,decimals));
   ui.tetSizing->setRange(diag * 10e-6, // min
-                         diag); // max
-  ui.tetSizing->setValue(sizing_default); // default value
+                         diag);        // max
+  ui.tetSizing->setValue(tets_sizing); // default value
 
-  double approx_default = get_approximate(diag * 0.005, 2, decimals);
-  ui.approx->setDecimals(-decimals+2);
-  ui.approx->setSingleStep(std::pow(10.,decimals));
   ui.approx->setRange(diag * 10e-7, // min
-                      diag); // max
-  ui.approx->setValue(approx_default);
+                      diag);        // max
+  ui.approx->setValue(approx);
 
   ui.protect->setEnabled(features_protection_available);
   ui.protect->setChecked(features_protection_available);
   ui.protectEdges->setEnabled(features_protection_available);
 
-  ui.facegraphCheckBox->setVisible(surface_only);
-  ui.initializationGroup->setVisible(image_item != nullptr && !image_item->isGray());
+  ui.facegraphCheckBox->setVisible(mesh_type == Mesh_type::SURFACE_ONLY);
+  ui.initializationGroup->setVisible(image_item != nullptr &&
+                                     !image_item->isGray());
   ui.grayImgGroup->setVisible(image_item != nullptr && image_item->isGray());
-  if (!sm_items.empty())
-      ui.volumeGroup->setVisible(!surface_only && nullptr != bounding_sm_item);
+  if (items->which() == POLYHEDRAL_MESH_ITEMS)
+    ui.volumeGroup->setVisible(mesh_type == Mesh_type::VOLUME &&
+                               nullptr != bounding_sm_item);
   else
-    ui.volumeGroup->setVisible(!surface_only);
-  if ((!sm_items.empty())|| polylines_item != nullptr) {
+    ui.volumeGroup->setVisible(mesh_type == Mesh_type::VOLUME);
+  ui.sharpEdgesAngle->setValue(sharp_edges_angle_bound);
+  if (items->which() != POLYHEDRAL_MESH_ITEMS || polylines_item != nullptr) {
     ui.sharpEdgesAngleLabel->setVisible(false);
     ui.sharpEdgesAngle->setVisible(false);
 
     ui.facetTopology->setEnabled(false);
-    ui.facetTopology->setToolTip(tr("<b>Notice:</b> "
-                                    "This option is only available with a"
-                                    " polyhedron or a surface mesh, when features are detected"
-                                    " automatically"));
+    ui.facetTopology->setToolTip(
+        tr("<b>Notice:</b> "
+           "This option is only available with a"
+           " polyhedron or a surface mesh, when features are detected"
+           " automatically"));
   }
   ui.noEdgeSizing->setChecked(ui.protect->isChecked());
   ui.edgeLabel->setEnabled(ui.noEdgeSizing->isChecked());
   ui.edgeSizing->setEnabled(ui.noEdgeSizing->isChecked());
 
-  if (features_protection_available)
-  {
-    if (!sm_items.empty())
-    {
-      if (surface_only)
-      {
+  if (features_protection_available) {
+    if (items->which() == POLYHEDRAL_MESH_ITEMS) {
+      if (mesh_type == Mesh_type::SURFACE_ONLY) {
         ui.protectEdges->addItem(QString("Sharp and Boundary edges"));
         ui.protectEdges->addItem(QString("Boundary edges only"));
-      }
-      else
+      } else
         ui.protectEdges->addItem(QString("Sharp edges"));
-    }
-    else if(nullptr != image_item)
-    {
-      if(polylines_item != nullptr)
+    } else if (items->which() == IMAGE_MESH_ITEMS) {
+      if (polylines_item != nullptr)
         ui.protectEdges->addItem(QString("Input polylines"));
-      else
-      {
+      else {
         ui.protectEdges->addItem(QString("Polylines on cube"));
       }
     }
@@ -432,38 +593,47 @@ void Mesh_3_plugin::mesh_3(const bool su
   // Get values
   // -----------------------------------
 
-  //reset cursor from the code for the scripts
+  // reset cursor from the code for the scripts
   QApplication::restoreOverrideCursor();
-  if(!use_defaults) {
+  if (dialog_choice == Dialog_choice::DIALOG) {
     int i = dialog.exec();
-    if( i == QDialog::Rejected ) { return; }
+    if (i == QDialog::Rejected) { return; }
   }
 
   // 0 means parameter is not considered
-  const double angle = !ui.noAngle->isChecked() ? 0 : ui.facetAngle->value();
-  const double approx = !ui.noApprox->isChecked() ? 0 : ui.approx->value();
-  const double facet_sizing = !ui.noFacetSizing->isChecked() ? 0 : ui.facetSizing->value();
-  const double radius_edge = !ui.noTetShape->isChecked() ? 0 : ui.tetShape->value();
-  const double tet_sizing = !ui.noTetSizing->isChecked() ? 0  : ui.tetSizing->value();
-  const double edge_size = !ui.noEdgeSizing->isChecked() ? DBL_MAX : ui.edgeSizing->value();
-  const bool protect_features = ui.protect->isChecked() && (ui.protectEdges->currentIndex() == 0);
-  const bool protect_borders = ui.protect->isChecked() && (ui.protectEdges->currentIndex() == 1);
-  const double sharp_edges_angle = ui.sharpEdgesAngle->value();
+  angle = !ui.noAngle->isChecked() ? 0 : ui.facetAngle->value();
+  sharp_edges_angle_bound = ui.sharpEdgesAngle->value();
+  std::cerr << "sharp_edges_angle_bound: " << sharp_edges_angle_bound << '\n';
+  edges_sizing =
+      !ui.noEdgeSizing->isChecked() ? DBL_MAX : ui.edgeSizing->value();
+  facets_sizing = !ui.noFacetSizing->isChecked() ? 0 : ui.facetSizing->value();
+  approx = !ui.noApprox->isChecked() ? 0 : ui.approx->value();
+  tets_shape = !ui.noTetShape->isChecked() ? 0 : ui.tetShape->value();
+  tets_sizing = !ui.noTetSizing->isChecked() ? 0 : ui.tetSizing->value();
+  protect_features =
+      ui.protect->isChecked() && (ui.protectEdges->currentIndex() == 0);
+  protect_borders =
+      ui.protect->isChecked() && (ui.protectEdges->currentIndex() == 1);
   const bool detect_connected_components = ui.detectComponents->isChecked();
-  const int manifold =
-    (ui.manifoldCheckBox->isChecked() ? 1 : 0)
-    + (ui.facetTopology->isChecked() ? 2 : 0);
+  const int manifold = (ui.manifoldCheckBox->isChecked() ? 1 : 0) +
+                       (ui.facetTopology->isChecked() ? 2 : 0);
   const float iso_value = float(ui.iso_value_spinBox->value());
   const float value_outside = float(ui.value_outside_spinBox->value());
-  const float inside_is_less =  float(ui.inside_is_less_checkBox->isChecked());
-  as_facegraph = surface_only ? ui.facegraphCheckBox->isChecked() : false;
+  const bool inside_is_less = ui.inside_is_less_checkBox->isChecked();
+  as_facegraph = (mesh_type == Mesh_type::SURFACE_ONLY)
+                     ? ui.facegraphCheckBox->isChecked()
+                     : false;
 
   Meshing_thread* thread = nullptr;
-  if (!sm_items.empty())
-  {
+  switch (items->which()) {
+  case POLYHEDRAL_MESH_ITEMS: {
+    auto& poly_items = get<Polyhedral_mesh_items>(*items);
+    auto& sm_items = poly_items.sm_items;
+    const auto bounding_sm_item = poly_items.bounding_sm_item;
+    const auto polylines_item = poly_items.polylines_item;
     QList<const SMesh*> polyhedrons;
-    if(!surface_only) {
-      sm_items.removeAll(bounding_sm_item);
+    if(mesh_type != Mesh_type::SURFACE_ONLY) {
+      sm_items.removeAll(make_not_null(bounding_sm_item));
     }
     std::transform(sm_items.begin(), sm_items.end(),
                    std::back_inserter(polyhedrons),
@@ -471,81 +641,87 @@ void Mesh_3_plugin::mesh_3(const bool su
                      return item->polyhedron();
                    });
     Scene_polylines_item::Polylines_container plc;
-    SMesh *bounding_polyhedron =
-      (bounding_sm_item == nullptr) ? nullptr : bounding_sm_item->polyhedron();
-
-    thread =    cgal_code_mesh_3(polyhedrons,
-                                 (polylines_item == nullptr)?plc:polylines_item->polylines,
-                                 bounding_polyhedron,
-                                 item_name,
-                                 angle,
-                                 facet_sizing,
-                                 approx,
-                                 tet_sizing,
-                                 edge_size,
-                                 radius_edge,
-                                 protect_features,
-                                 protect_borders,
-                                 sharp_edges_angle,
-                                 manifold,
-                                 surface_only);
+    SMesh* bounding_polyhedron = (bounding_sm_item == nullptr)
+                                     ? nullptr
+                                     : bounding_sm_item->polyhedron();
+
+    thread = cgal_code_mesh_3(
+        polyhedrons,
+        (polylines_item == nullptr) ? plc : polylines_item->polylines,
+        bounding_polyhedron,
+        item_name,
+        angle,
+        facets_sizing,
+        approx,
+        tets_sizing,
+        edges_sizing,
+        tets_shape,
+        protect_features,
+        protect_borders,
+        sharp_edges_angle_bound,
+        manifold,
+        mesh_type == Mesh_type::SURFACE_ONLY);
+    break;
   }
   // Image
-#ifdef CGAL_MESH_3_DEMO_ACTIVATE_IMPLICIT_FUNCTIONS
-  else if (nullptr != function_item)
-  {
+#  ifdef CGAL_MESH_3_DEMO_ACTIVATE_IMPLICIT_FUNCTIONS
+  case IMPLICIT_MESH_ITEMS: {
     const Implicit_function_interface* pFunction = function_item->function();
-    if (nullptr == pFunction)
-    {
+    if (nullptr == pFunction) {
       QMessageBox::critical(mw, tr(""), tr("ERROR: no data in selected item"));
       return;
     }
 
-    thread =    cgal_code_mesh_3(pFunction,
-                                 angle,
-                                 facet_sizing,
-                                 approx,
-                                 tet_sizing,
-                                 edge_size,
-                                 radius_edge,
-                                 manifold,
-                                 surface_only);
-  }
-#endif
-#ifdef CGAL_MESH_3_DEMO_ACTIVATE_SEGMENTED_IMAGES
-  else if (nullptr != image_item)
-  {
+    thread = cgal_code_mesh_3(pFunction,
+                              angle,
+                              facets_sizing,
+                              approx,
+                              tets_sizing,
+                              edges_sizing,
+                              tets_shape,
+                              manifold,
+                              mesh_type == Mesh_type::SURFACE_ONLY);
+    break;
+  }
+#  endif
+#  ifdef CGAL_MESH_3_DEMO_ACTIVATE_SEGMENTED_IMAGES
+  case IMAGE_MESH_ITEMS: {
     const Image* pImage = image_item->image();
-    if (nullptr == pImage)
-    {
+    if (nullptr == pImage) {
       QMessageBox::critical(mw, tr(""), tr("ERROR: no data in selected item"));
       return;
     }
 
     Scene_polylines_item::Polylines_container plc;
 
-    thread =    cgal_code_mesh_3(pImage,
-                                 (polylines_item == nullptr)?plc:polylines_item->polylines,
-                                 angle,
-                                 facet_sizing,
-                                 approx,
-                                 tet_sizing,
-                                 edge_size,
-                                 radius_edge,
-                                 protect_features,
-                                 manifold,
-                                 surface_only,
-                                 detect_connected_components,
-                                 image_item->isGray(),
-                                 iso_value,
-                                 value_outside,
-                                 inside_is_less);
-  }
-#endif
+    thread = cgal_code_mesh_3(
+        pImage,
+        (polylines_item == nullptr) ? plc : polylines_item->polylines,
+        angle,
+        facets_sizing,
+        approx,
+        tets_sizing,
+        edges_sizing,
+        tets_shape,
+        protect_features,
+        manifold,
+        mesh_type == Mesh_type::SURFACE_ONLY,
+        detect_connected_components,
+        image_item->isGray(),
+        iso_value,
+        value_outside,
+        inside_is_less);
+    break;
+  }
+  default:
+    CGAL::Three::Three::error(tr("Mesh_3 plugin"),
+                              tr("This type of item is not handled!"));
+    return;
+  } // end switch
+#  endif
 
-  if ( nullptr == thread )
-  {
-    QMessageBox::critical(mw,tr(""),tr("ERROR: no thread created"));
+  if (nullptr == thread) {
+    QMessageBox::critical(mw, tr(""), tr("ERROR: no thread created"));
     return;
   }
 
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/Mesh_function.h 5.1-1/demo/Polyhedron/Plugins/Mesh_3/Mesh_function.h
--- 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/Mesh_function.h	2020-08-01 19:02:42.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Mesh_3/Mesh_function.h	2020-09-08 12:15:24.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron/demo/Polyhedron/Plugins/Mesh_3/Mesh_function.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron/demo/Polyhedron/Plugins/Mesh_3/Mesh_function.h $
 // $Id: Mesh_function.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/Meshing_dialog.ui 5.1-1/demo/Polyhedron/Plugins/Mesh_3/Meshing_dialog.ui
--- 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/Meshing_dialog.ui	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Mesh_3/Meshing_dialog.ui	2020-09-08 12:12:20.000000000 +0000
@@ -100,9 +100,23 @@
      <property name="title">
       <string>Sharp features</string>
      </property>
-     <layout class="QGridLayout" name="gridLayout" columnstretch="0,1,0">
-      <item row="1" column="1">
-       <widget class="QComboBox" name="protectEdges"/>
+     <layout class="QGridLayout" name="gridLayout" columnstretch="0,0,0">
+      <item row="3" column="1">
+       <widget class="DoubleEdit" name="sharpEdgesAngle">
+        <property name="text">
+         <string>60.00</string>
+        </property>
+       </widget>
+      </item>
+      <item row="1" column="2">
+       <widget class="QCheckBox" name="protect">
+        <property name="text">
+         <string/>
+        </property>
+        <property name="checked">
+         <bool>true</bool>
+        </property>
+       </widget>
       </item>
       <item row="3" column="0">
        <widget class="QLabel" name="sharpEdgesAngleLabel">
@@ -120,19 +134,6 @@
         </property>
        </widget>
       </item>
-      <item row="7" column="0">
-       <widget class="QLabel" name="edgeLabel">
-        <property name="text">
-         <string>Edge &amp;size</string>
-        </property>
-        <property name="alignment">
-         <set>Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter</set>
-        </property>
-        <property name="buddy">
-         <cstring>edgeSizing</cstring>
-        </property>
-       </widget>
-      </item>
       <item row="7" column="2">
        <widget class="QCheckBox" name="noEdgeSizing">
         <property name="text">
@@ -140,28 +141,8 @@
         </property>
        </widget>
       </item>
-      <item row="7" column="1">
-       <widget class="QDoubleSpinBox" name="edgeSizing">
-        <property name="decimals">
-         <number>4</number>
-        </property>
-        <property name="minimum">
-         <double>0.000000000000000</double>
-        </property>
-        <property name="value">
-         <double>0.000000000000000</double>
-        </property>
-       </widget>
-      </item>
-      <item row="3" column="1">
-       <widget class="QDoubleSpinBox" name="sharpEdgesAngle">
-        <property name="maximum">
-         <double>180.000000000000000</double>
-        </property>
-        <property name="value">
-         <double>60.000000000000000</double>
-        </property>
-       </widget>
+      <item row="1" column="1">
+       <widget class="QComboBox" name="protectEdges"/>
       </item>
       <item row="1" column="0">
        <widget class="QLabel" name="protectLabel">
@@ -176,13 +157,23 @@
         </property>
        </widget>
       </item>
-      <item row="1" column="2">
-       <widget class="QCheckBox" name="protect">
+      <item row="7" column="1">
+       <widget class="DoubleEdit" name="edgeSizing">
         <property name="text">
-         <string/>
+         <string>0.0</string>
         </property>
-        <property name="checked">
-         <bool>true</bool>
+       </widget>
+      </item>
+      <item row="7" column="0">
+       <widget class="QLabel" name="edgeLabel">
+        <property name="text">
+         <string>Edge &amp;size</string>
+        </property>
+        <property name="alignment">
+         <set>Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter</set>
+        </property>
+        <property name="buddy">
+         <cstring>edgeSizing</cstring>
         </property>
        </widget>
       </item>
@@ -234,16 +225,6 @@
         </property>
        </widget>
       </item>
-      <item row="1" column="1">
-       <widget class="QDoubleSpinBox" name="approx">
-        <property name="minimumSize">
-         <size>
-          <width>110</width>
-          <height>0</height>
-         </size>
-        </property>
-       </widget>
-      </item>
       <item row="3" column="0">
        <widget class="QLabel" name="angleLabel">
         <property name="text">
@@ -257,19 +238,6 @@
         </property>
        </widget>
       </item>
-      <item row="2" column="1">
-       <widget class="QDoubleSpinBox" name="facetSizing">
-        <property name="minimumSize">
-         <size>
-          <width>110</width>
-          <height>0</height>
-         </size>
-        </property>
-        <property name="decimals">
-         <number>4</number>
-        </property>
-       </widget>
-      </item>
       <item row="9" column="0" colspan="3">
        <widget class="QFrame" name="advanced">
         <property name="visible">
@@ -364,22 +332,27 @@
         </property>
        </widget>
       </item>
-      <item row="3" column="1">
-       <widget class="QDoubleSpinBox" name="facetAngle">
-        <property name="minimumSize">
-         <size>
-          <width>110</width>
-          <height>0</height>
-         </size>
+      <item row="2" column="1">
+       <widget class="DoubleEdit" name="facetSizing">
+        <property name="text">
+         <string>0.065</string>
         </property>
-        <property name="minimum">
-         <double>0.000000000000000</double>
+       </widget>
+      </item>
+      <item row="3" column="1">
+       <widget class="DoubleEdit" name="facetAngle">
+        <property name="inputMask">
+         <string>25,00</string>
         </property>
-        <property name="maximum">
-         <double>30.000000000000000</double>
+        <property name="text">
+         <string>25,00</string>
         </property>
-        <property name="value">
-         <double>25.000000000000000</double>
+       </widget>
+      </item>
+      <item row="1" column="1">
+       <widget class="DoubleEdit" name="approx">
+        <property name="text">
+         <string>0.0017</string>
         </property>
        </widget>
       </item>
@@ -418,35 +391,6 @@
         </property>
        </widget>
       </item>
-      <item row="1" column="1">
-       <widget class="QDoubleSpinBox" name="tetShape">
-        <property name="minimumSize">
-         <size>
-          <width>110</width>
-          <height>0</height>
-         </size>
-        </property>
-        <property name="minimum">
-         <double>1.000000000000000</double>
-        </property>
-        <property name="value">
-         <double>3.000000000000000</double>
-        </property>
-       </widget>
-      </item>
-      <item row="0" column="1">
-       <widget class="QDoubleSpinBox" name="tetSizing">
-        <property name="minimumSize">
-         <size>
-          <width>110</width>
-          <height>0</height>
-         </size>
-        </property>
-        <property name="decimals">
-         <number>4</number>
-        </property>
-       </widget>
-      </item>
       <item row="1" column="0">
        <widget class="QLabel" name="label_2">
         <property name="text">
@@ -455,9 +399,6 @@
         <property name="alignment">
          <set>Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter</set>
         </property>
-        <property name="buddy">
-         <cstring>tetShape</cstring>
-        </property>
        </widget>
       </item>
       <item row="0" column="0">
@@ -473,6 +414,20 @@
         </property>
        </widget>
       </item>
+      <item row="0" column="1">
+       <widget class="DoubleEdit" name="tetSizing">
+        <property name="text">
+         <string>0.00</string>
+        </property>
+       </widget>
+      </item>
+      <item row="1" column="1">
+       <widget class="DoubleEdit" name="tetShape">
+        <property name="text">
+         <string>3.00</string>
+        </property>
+       </widget>
+      </item>
      </layout>
     </widget>
    </item>
@@ -504,22 +459,6 @@
         </property>
        </widget>
       </item>
-      <item row="1" column="1">
-       <widget class="QDoubleSpinBox" name="iso_value_spinBox">
-        <property name="minimum">
-         <double>-65536.000000000000000</double>
-        </property>
-        <property name="maximum">
-         <double>65536.000000000000000</double>
-        </property>
-        <property name="value">
-         <double>3.000000000000000</double>
-        </property>
-       </widget>
-      </item>
-      <item row="0" column="1">
-       <widget class="QDoubleSpinBox" name="value_outside_spinBox"/>
-      </item>
       <item row="0" column="0">
        <widget class="QLabel" name="label_4">
         <property name="toolTip">
@@ -559,6 +498,20 @@
         </property>
        </widget>
       </item>
+      <item row="0" column="1">
+       <widget class="DoubleEdit" name="value_outside_spinBox">
+        <property name="text">
+         <string>0.00</string>
+        </property>
+       </widget>
+      </item>
+      <item row="1" column="1">
+       <widget class="DoubleEdit" name="iso_value_spinBox">
+        <property name="text">
+         <string>3.00</string>
+        </property>
+       </widget>
+      </item>
      </layout>
     </widget>
    </item>
@@ -597,16 +550,18 @@
    </item>
   </layout>
  </widget>
+ <customwidgets>
+  <customwidget>
+   <class>DoubleEdit</class>
+   <extends>QLineEdit</extends>
+   <header location="global">CGAL_double_edit.h</header>
+  </customwidget>
+ </customwidgets>
  <tabstops>
-  <tabstop>approx</tabstop>
   <tabstop>noApprox</tabstop>
-  <tabstop>facetSizing</tabstop>
   <tabstop>noFacetSizing</tabstop>
-  <tabstop>facetAngle</tabstop>
   <tabstop>noAngle</tabstop>
-  <tabstop>tetSizing</tabstop>
   <tabstop>noTetSizing</tabstop>
-  <tabstop>tetShape</tabstop>
   <tabstop>noTetShape</tabstop>
   <tabstop>checkBox</tabstop>
   <tabstop>manifoldCheckBox</tabstop>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/Meshing_thread.h 5.1-1/demo/Polyhedron/Plugins/Mesh_3/Meshing_thread.h
--- 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/Meshing_thread.h	2020-08-01 19:02:42.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Mesh_3/Meshing_thread.h	2020-09-08 12:15:24.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron/demo/Polyhedron/Plugins/Mesh_3/Meshing_thread.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron/demo/Polyhedron/Plugins/Mesh_3/Meshing_thread.h $
 // $Id: Meshing_thread.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/Optimization_plugin_cgal_code.cpp 5.1-1/demo/Polyhedron/Plugins/Mesh_3/Optimization_plugin_cgal_code.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/Optimization_plugin_cgal_code.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Mesh_3/Optimization_plugin_cgal_code.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -141,6 +141,10 @@ Optimizer_thread* cgal_code_optimization
       return NULL;
     }
     Polyhedral_mesh_domain* sm_domain = new Polyhedral_mesh_domain(*smesh);
+    if(c3t3_item.get_sharp_edges_angle() != -1 )
+      sm_domain->detect_features(c3t3_item.get_sharp_edges_angle());
+    else if(c3t3_item.get_detect_borders())
+      sm_domain->detect_borders();
 
     // Create thread
     typedef Optimization_function<Polyhedral_mesh_domain,Parameters> Opt_function;
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/Optimization_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Mesh_3/Optimization_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/Optimization_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Mesh_3/Optimization_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -16,7 +16,6 @@
 #include "ui_Local_optimizers_dialog.h"
 
 #include "Scene_c3t3_item.h"
-#include "C3t3_type.h"
 
 #include "Optimizer_thread.h"
 
@@ -227,6 +226,9 @@ Mesh_3_optimization_plugin::odt()
   QDialog dialog(mw);
   Ui::Smoother_dialog ui;
   ui.setupUi(&dialog);
+  ui.convergenceRatio->setMaximum(1.0);
+  ui.convergenceRatio->setMinimum(0.0001);
+  ui.freezeRatio->setMinimum(0.0);
   dialog.setWindowFlags(Qt::Dialog|Qt::CustomizeWindowHint|Qt::WindowCloseButtonHint);
   dialog.setWindowTitle(tr("ODT-smoothing parameters"));
 
@@ -362,6 +364,7 @@ Mesh_3_optimization_plugin::perturb()
   QDialog dialog(mw);
   Ui::LocalOptim_dialog ui;
   ui.setupUi(&dialog);
+  ui.sliverBound->setRange(0,180);
   dialog.setWindowFlags(Qt::Dialog|Qt::CustomizeWindowHint|Qt::WindowCloseButtonHint);
   dialog.setWindowTitle(tr("Sliver perturbation parameters"));
 
@@ -517,14 +520,13 @@ treat_result(Scene_c3t3_item& source_ite
   if ( &source_item != &result_item)
   {
     const Scene_item::Bbox& bbox = result_item.bbox();
-    result_item.setPosition((bbox.xmin() + bbox.xmax())/2.f,
-                            (bbox.ymin() + bbox.ymax())/2.f,
-                            (bbox.zmin() + bbox.zmax())/2.f);
+    result_item.setPosition(static_cast<float>(bbox.xmin() + bbox.xmax())/2.f,
+                            static_cast<float>(bbox.ymin() + bbox.ymax())/2.f,
+                            static_cast<float>(bbox.zmin() + bbox.zmax())/2.f);
 
-    result_item.setColor(QColor(59,74,226));
+    result_item.setColor(source_item.color());
     result_item.setRenderingMode(source_item.renderingMode());
     result_item.set_data_item(source_item.data_item());
-
     source_item.setVisible(false);
 
     const Scene_interface::Item_id index = scene->mainSelectionIndex();
@@ -536,6 +538,7 @@ treat_result(Scene_c3t3_item& source_ite
   else
   {
     result_item.update_histogram();
+    result_item.invalidateOpenGLBuffers();
 
     const Scene_interface::Item_id index = scene->mainSelectionIndex();
     scene->itemChanged(index);
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/Optimizer_thread.h 5.1-1/demo/Polyhedron/Plugins/Mesh_3/Optimizer_thread.h
--- 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/Optimizer_thread.h	2020-08-01 19:02:42.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Mesh_3/Optimizer_thread.h	2020-09-08 12:15:24.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron/demo/Polyhedron/Plugins/Mesh_3/Optimizer_thread.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron/demo/Polyhedron/Plugins/Mesh_3/Optimizer_thread.h $
 // $Id: Optimizer_thread.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/Rib_dialog.ui 5.1-1/demo/Polyhedron/Plugins/Mesh_3/Rib_dialog.ui
--- 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/Rib_dialog.ui	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Mesh_3/Rib_dialog.ui	2020-09-08 12:12:20.000000000 +0000
@@ -22,20 +22,6 @@
      <layout class="QVBoxLayout" name="verticalLayout_2">
       <item>
        <layout class="QGridLayout" name="gridLayout">
-        <item row="1" column="1">
-         <widget class="QDoubleSpinBox" name="sphereRadius">
-          <property name="decimals">
-           <number>4</number>
-          </property>
-         </widget>
-        </item>
-        <item row="2" column="1">
-         <widget class="QDoubleSpinBox" name="cylinderRadius">
-          <property name="decimals">
-           <number>4</number>
-          </property>
-         </widget>
-        </item>
         <item row="2" column="0">
          <widget class="QLabel" name="cylinderRadiusLabel">
           <property name="text">
@@ -50,6 +36,20 @@
           </property>
          </widget>
         </item>
+        <item row="1" column="2">
+         <widget class="DoubleEdit" name="sphereRadius">
+          <property name="text">
+           <string>0.00</string>
+          </property>
+         </widget>
+        </item>
+        <item row="2" column="2">
+         <widget class="DoubleEdit" name="cylinderRadius">
+          <property name="text">
+           <string>0.00</string>
+          </property>
+         </widget>
+        </item>
        </layout>
       </item>
       <item>
@@ -70,13 +70,6 @@
      <layout class="QVBoxLayout" name="verticalLayout_3">
       <item>
        <layout class="QGridLayout" name="gridLayout_2">
-        <item row="1" column="0">
-         <widget class="QLabel" name="ambientLabel">
-          <property name="text">
-           <string>Ambient Light</string>
-          </property>
-         </widget>
-        </item>
         <item row="1" column="2">
          <widget class="QCheckBox" name="isAmbientOn">
           <property name="text">
@@ -87,29 +80,10 @@
           </property>
          </widget>
         </item>
-        <item row="1" column="1">
-         <widget class="QDoubleSpinBox" name="ambientIntensity">
-          <property name="maximum">
-           <double>1.000000000000000</double>
-          </property>
-          <property name="singleStep">
-           <double>0.010000000000000</double>
-          </property>
-          <property name="value">
-           <double>0.150000000000000</double>
-          </property>
-         </widget>
-        </item>
-        <item row="2" column="1">
-         <widget class="QDoubleSpinBox" name="shadowIntensity">
-          <property name="maximum">
-           <double>1.000000000000000</double>
-          </property>
-          <property name="singleStep">
-           <double>0.010000000000000</double>
-          </property>
-          <property name="value">
-           <double>0.850000000000000</double>
+        <item row="2" column="0">
+         <widget class="QLabel" name="shadowLabel">
+          <property name="text">
+           <string>Shadow Light</string>
           </property>
          </widget>
         </item>
@@ -123,10 +97,24 @@
           </property>
          </widget>
         </item>
-        <item row="2" column="0">
-         <widget class="QLabel" name="shadowLabel">
+        <item row="1" column="0">
+         <widget class="QLabel" name="ambientLabel">
           <property name="text">
-           <string>Shadow Light</string>
+           <string>Ambient Light</string>
+          </property>
+         </widget>
+        </item>
+        <item row="1" column="1">
+         <widget class="DoubleEdit" name="ambientIntensity">
+          <property name="text">
+           <string>0.15</string>
+          </property>
+         </widget>
+        </item>
+        <item row="2" column="1">
+         <widget class="DoubleEdit" name="shadowIntensity">
+          <property name="text">
+           <string>0.85</string>
           </property>
          </widget>
         </item>
@@ -245,6 +233,13 @@
    </item>
   </layout>
  </widget>
+ <customwidgets>
+  <customwidget>
+   <class>DoubleEdit</class>
+   <extends>QLineEdit</extends>
+   <header>CGAL_double_edit.h</header>
+  </customwidget>
+ </customwidgets>
  <resources/>
  <connections>
   <connection>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/Smoother_dialog.ui 5.1-1/demo/Polyhedron/Plugins/Mesh_3/Smoother_dialog.ui
--- 5.0.3-1/demo/Polyhedron/Plugins/Mesh_3/Smoother_dialog.ui	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Mesh_3/Smoother_dialog.ui	2020-09-08 12:12:20.000000000 +0000
@@ -7,7 +7,7 @@
     <x>0</x>
     <y>0</y>
     <width>507</width>
-    <height>328</height>
+    <height>331</height>
    </rect>
   </property>
   <property name="sizePolicy">
@@ -50,7 +50,7 @@
      <property name="title">
       <string>Parameters</string>
      </property>
-     <layout class="QGridLayout" name="gridLayout_2" rowstretch="0,0" columnstretch="0,1,0">
+     <layout class="QGridLayout" name="gridLayout_2" rowstretch="0,0" columnstretch="0,0,0">
       <property name="topMargin">
        <number>8</number>
       </property>
@@ -68,9 +68,6 @@
         <property name="alignment">
          <set>Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter</set>
         </property>
-        <property name="buddy">
-         <cstring>maxTime</cstring>
-        </property>
        </widget>
       </item>
       <item row="1" column="2">
@@ -84,21 +81,9 @@
        </widget>
       </item>
       <item row="1" column="1">
-       <widget class="QDoubleSpinBox" name="maxTime">
-        <property name="minimumSize">
-         <size>
-          <width>100</width>
-          <height>0</height>
-         </size>
-        </property>
-        <property name="decimals">
-         <number>1</number>
-        </property>
-        <property name="maximum">
-         <double>9999.000000000000000</double>
-        </property>
-        <property name="value">
-         <double>60.000000000000000</double>
+       <widget class="DoubleEdit" name="maxTime">
+        <property name="text">
+         <string>60.0</string>
         </property>
        </widget>
       </item>
@@ -122,7 +107,7 @@
      <property name="checkable">
       <bool>false</bool>
      </property>
-     <layout class="QGridLayout" name="gridLayout_3" columnstretch="0,1">
+     <layout class="QGridLayout" name="gridLayout_3" columnstretch="0,1,0">
       <property name="topMargin">
        <number>8</number>
       </property>
@@ -148,19 +133,6 @@
         </property>
        </widget>
       </item>
-      <item row="0" column="1">
-       <widget class="QDoubleSpinBox" name="maxIterationNb">
-        <property name="decimals">
-         <number>0</number>
-        </property>
-        <property name="maximum">
-         <double>200.000000000000000</double>
-        </property>
-        <property name="value">
-         <double>100.000000000000000</double>
-        </property>
-       </widget>
-      </item>
       <item row="1" column="0">
        <widget class="QLabel" name="convergenceLabel">
         <property name="font">
@@ -177,22 +149,6 @@
         </property>
        </widget>
       </item>
-      <item row="1" column="1">
-       <widget class="QDoubleSpinBox" name="convergenceRatio">
-        <property name="decimals">
-         <number>4</number>
-        </property>
-        <property name="minimum">
-         <double>0.000100000000000</double>
-        </property>
-        <property name="maximum">
-         <double>1.000000000000000</double>
-        </property>
-        <property name="singleStep">
-         <double>0.010000000000000</double>
-        </property>
-       </widget>
-      </item>
       <item row="2" column="0">
        <widget class="QLabel" name="freezeLabel">
         <property name="font">
@@ -207,27 +163,29 @@
         <property name="alignment">
          <set>Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter</set>
         </property>
-        <property name="buddy">
-         <cstring>freezeRatio</cstring>
-        </property>
        </widget>
       </item>
-      <item row="2" column="1">
-       <widget class="QDoubleSpinBox" name="freezeRatio">
-        <property name="decimals">
-         <number>4</number>
+      <item row="1" column="2">
+       <widget class="DoubleEdit" name="convergenceRatio">
+        <property name="text">
+         <string>0.001</string>
         </property>
-        <property name="minimum">
-         <double>0.000000000000000</double>
+       </widget>
+      </item>
+      <item row="2" column="2">
+       <widget class="DoubleEdit" name="freezeRatio">
+        <property name="text">
+         <string>0.000</string>
         </property>
+       </widget>
+      </item>
+      <item row="0" column="2">
+       <widget class="QSpinBox" name="maxIterationNb">
         <property name="maximum">
-         <double>1.000000000000000</double>
-        </property>
-        <property name="singleStep">
-         <double>0.001000000000000</double>
+         <number>200</number>
         </property>
         <property name="value">
-         <double>0.000000000000000</double>
+         <number>100</number>
         </property>
        </widget>
       </item>
@@ -279,6 +237,13 @@
    </item>
   </layout>
  </widget>
+ <customwidgets>
+  <customwidget>
+   <class>DoubleEdit</class>
+   <extends>QLineEdit</extends>
+   <header>CGAL_double_edit.h</header>
+  </customwidget>
+ </customwidgets>
  <resources/>
  <connections>
   <connection>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Operations_on_polyhedra/PartitionDialog.ui 5.1-1/demo/Polyhedron/Plugins/Operations_on_polyhedra/PartitionDialog.ui
--- 5.0.3-1/demo/Polyhedron/Plugins/Operations_on_polyhedra/PartitionDialog.ui	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Operations_on_polyhedra/PartitionDialog.ui	2020-09-08 12:12:20.000000000 +0000
@@ -67,7 +67,7 @@
 p, li { white-space: pre-wrap; }
 &lt;/style&gt;&lt;/head&gt;&lt;body style=&quot; font-family:'Sans Serif'; font-size:9pt; font-weight:400; font-style:normal;&quot;&gt;
 &lt;p style=&quot; margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;Tip: To split your mesh according to the result of this operation, you can click on &lt;/p&gt;
-&lt;p style=&quot; margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-weight:600;&quot;&gt;Operations-&amp;gt;Operations on Polyhedra-&amp;gt;Split Selected Polyhedra&lt;/span&gt;&lt;/p&gt;&lt;/body&gt;&lt;/html&gt;</string>
+&lt;p style=&quot; margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-weight:600;&quot;&gt;Operations-&amp;gt;Polygon Mesh Processing-&amp;gt;Split Selected Polyhedra&lt;/span&gt;&lt;/p&gt;&lt;/body&gt;&lt;/html&gt;</string>
      </property>
     </widget>
    </item>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/PCA/Affine_transform_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/PCA/Affine_transform_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/PCA/Affine_transform_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/PCA/Affine_transform_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -450,7 +450,7 @@ void Polyhedron_demo_affine_transform_pl
             Kernel::Aff_transformation_3 trans(CGAL::TRANSLATION, Kernel::Vector_3(i*x_t,j*y_t,k*z_t));
             CGAL::Polygon_mesh_processing::transform(trans, e);
             Facegraph_item* t_item = new Facegraph_item(e);
-            t_item->setName(tr("%1 %2%3%4")
+            t_item->setName(tr("%1 %2,%3,%4")
                             .arg(item->name())
                             .arg(i)
                             .arg(j)
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/PCA/Basic_generator_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/PCA/Basic_generator_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/PCA/Basic_generator_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/PCA/Basic_generator_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -15,6 +15,7 @@
 #include "Scene_polylines_item.h"
 #include <CGAL/subdivision_method_3.h>
 #include <CGAL/Kernel_traits.h>
+#include <CGAL/Three/Three.h>
 #include "ui_Basic_generator_widget.h"
 
 class GeneratorWidget :
@@ -59,7 +60,7 @@ public :
     QMenu* menu = menuFile->findChild<QMenu*>("menuGenerateObject");
     if(!menu){
       QAction* actionLoad = mw->findChild<QAction*>("actionLoadPlugin");
-      menu = new QMenu(tr("Generate &Objet"), menuFile);
+      menu = new QMenu(tr("Generate &Object"), menuFile);
       menu->setObjectName("menuGenerateObject");
       menuFile->insertMenu(actionLoad, menu);
     }
@@ -350,8 +351,7 @@ void Basic_generator_plugin::generateCub
 
     for(int i=0; i<8; ++i)
     {
-
-      QStringList list = point_texts[i].split(QRegExp("\\s+"), QString::SkipEmptyParts);
+      QStringList list = point_texts[i].split(QRegExp("\\s+"), CGAL_QT_SKIP_EMPTY_PARTS);
       if (list.isEmpty()) return;
       if (list.size()!=3){
         QMessageBox *msgBox = new QMessageBox;
@@ -392,7 +392,7 @@ void Basic_generator_plugin::generateCub
   else
   {
     QString text = dock_widget->extremaEdit->text();
-    QStringList list = text.split(QRegExp("\\s+"), QString::SkipEmptyParts);
+    QStringList list = text.split(QRegExp("\\s+"), CGAL_QT_SKIP_EMPTY_PARTS);
     if (list.isEmpty()) return;
     if (list.size()!=6){
       QMessageBox *msgBox = new QMessageBox;
@@ -443,7 +443,7 @@ void Basic_generator_plugin::generatePri
   bool is_closed = dock_widget->prismCheckBox->isChecked();
 
   QString text = dock_widget->prism_lineEdit->text();
-  QStringList list = text.split(QRegExp("\\s+"), QString::SkipEmptyParts);
+  QStringList list = text.split(QRegExp("\\s+"), CGAL_QT_SKIP_EMPTY_PARTS);
   if (list.isEmpty()) return;
   if (list.size()!=3){
     QMessageBox *msgBox = new QMessageBox;
@@ -490,7 +490,7 @@ void Basic_generator_plugin::generatePyr
   bool is_closed = dock_widget->pyramidCheckBox->isChecked();
 
   QString text = dock_widget->pyramid_lineEdit->text();
-  QStringList list = text.split(QRegExp("\\s+"), QString::SkipEmptyParts);
+  QStringList list = text.split(QRegExp("\\s+"), CGAL_QT_SKIP_EMPTY_PARTS);
   if (list.isEmpty()) return;
   if (list.size()!=3){
     QMessageBox *msgBox = new QMessageBox;
@@ -533,7 +533,7 @@ void Basic_generator_plugin::generateSph
 {
   int precision = dock_widget->SphereSpinBox->value();
   QString text = dock_widget->center_radius_lineEdit->text();
-  QStringList list = text.split(QRegExp("\\s+"), QString::SkipEmptyParts);
+  QStringList list = text.split(QRegExp("\\s+"), CGAL_QT_SKIP_EMPTY_PARTS);
   if (list.isEmpty()) return;
   if (list.size()!=4){
     QMessageBox *msgBox = new QMessageBox;
@@ -582,8 +582,7 @@ void Basic_generator_plugin::generateTet
 
   for(int i=0; i<4; ++i)
   {
-
-    QStringList list = point_texts[i].split(QRegExp("\\s+"), QString::SkipEmptyParts);
+    QStringList list = point_texts[i].split(QRegExp("\\s+"), CGAL_QT_SKIP_EMPTY_PARTS);
     if (list.isEmpty()) return;
     if (list.size()!=3){
       QMessageBox *msgBox = new QMessageBox;
@@ -624,7 +623,7 @@ void Basic_generator_plugin::generatePoi
 {
   QString text = dock_widget->point_textEdit->toPlainText();
   Scene_points_with_normal_item* item = new Scene_points_with_normal_item();
-  QStringList list = text.split(QRegExp("\\s+"), QString::SkipEmptyParts);
+  QStringList list = text.split(QRegExp("\\s+"), CGAL_QT_SKIP_EMPTY_PARTS);
   int counter = 0;
   double coord[3];
   bool ok = true;
@@ -682,7 +681,8 @@ void Basic_generator_plugin::generateLin
   polylines.resize(polylines.size()+1);
   std::vector<Scene_polylines_item::Point_3>& polyline = *(polylines.rbegin());
   QStringList polylines_metadata;
-  QStringList list = text.split(QRegExp("\\s+"), QString::SkipEmptyParts);
+
+  QStringList list = text.split(QRegExp("\\s+"), CGAL_QT_SKIP_EMPTY_PARTS);
   int counter = 0;
   double coord[3];
   bool ok = true;
@@ -777,11 +777,12 @@ void Basic_generator_plugin::generateGri
 
   QString points_text;
   Point extrema[2];
-  typename boost::graph_traits<Face_graph>::vertices_size_type nb_cells[2];
+  using size_type = typename boost::graph_traits<Face_graph>::vertices_size_type;
+  size_type nb_cells[2];
   bool triangulated = dock_widget->grid_checkBox->isChecked();
   points_text= dock_widget->grid_lineEdit->text();
 
-  QStringList list = points_text.split(QRegExp("\\s+"), QString::SkipEmptyParts);
+  QStringList list = points_text.split(QRegExp("\\s+"), CGAL_QT_SKIP_EMPTY_PARTS);
   if (list.isEmpty()) return;
   if (list.size()!=6){
     QMessageBox *msgBox = new QMessageBox;
@@ -806,8 +807,8 @@ void Basic_generator_plugin::generateGri
   }
   extrema[0] = Point(coords[0], coords[1], coords[2]);
   extrema[1] = Point(coords[3], coords[4], coords[5]);
-  nb_cells[0] = static_cast<std::size_t>(dock_widget->gridX_spinBox->value());
-  nb_cells[1] = static_cast<std::size_t>(dock_widget->gridY_spinBox->value());
+  nb_cells[0] = static_cast<size_type>(dock_widget->gridX_spinBox->value());
+  nb_cells[1] = static_cast<size_type>(dock_widget->gridY_spinBox->value());
 
   //nb_points = nb_cells+1
   Point_generator point_gen(nb_cells[0]+1, nb_cells[1]+1, extrema[0], extrema[1]);
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/PCA/Basic_generator_widget.ui 5.1-1/demo/Polyhedron/Plugins/PCA/Basic_generator_widget.ui
--- 5.0.3-1/demo/Polyhedron/Plugins/PCA/Basic_generator_widget.ui	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/PCA/Basic_generator_widget.ui	2020-09-08 12:12:20.000000000 +0000
@@ -18,7 +18,7 @@
     <item>
      <widget class="QTabWidget" name="selector_tabWidget">
       <property name="currentIndex">
-       <number>0</number>
+       <number>2</number>
       </property>
       <widget class="QWidget" name="prismTab">
        <attribute name="title">
@@ -31,7 +31,7 @@
            <string/>
           </property>
           <property name="pixmap">
-           <pixmap resource="../../Polyhedron_3.qrc">:/cgal/Polyhedron_3/resources/prism.png</pixmap>
+           <pixmap>:/cgal/Polyhedron_3/resources/prism.png</pixmap>
           </property>
           <property name="scaledContents">
            <bool>false</bool>
@@ -136,15 +136,9 @@ QGroupBox::title {
                 </widget>
                </item>
                <item>
-                <widget class="QDoubleSpinBox" name="prismBaseSpinBox">
-                 <property name="decimals">
-                  <number>5</number>
-                 </property>
-                 <property name="maximum">
-                  <double>1000000000000000000.000000000000000</double>
-                 </property>
-                 <property name="value">
-                  <double>1.000000000000000</double>
+                <widget class="DoubleEdit" name="prismBaseSpinBox">
+                 <property name="text">
+                  <string>1.00</string>
                  </property>
                 </widget>
                </item>
@@ -163,15 +157,9 @@ QGroupBox::title {
                 </widget>
                </item>
                <item>
-                <widget class="QDoubleSpinBox" name="prismHeightSpinBox">
-                 <property name="decimals">
-                  <number>5</number>
-                 </property>
-                 <property name="maximum">
-                  <double>1000000000000000000.000000000000000</double>
-                 </property>
-                 <property name="value">
-                  <double>1.000000000000000</double>
+                <widget class="DoubleEdit" name="prismHeightSpinBox">
+                 <property name="text">
+                  <string>1.00</string>
                  </property>
                 </widget>
                </item>
@@ -235,7 +223,7 @@ QGroupBox::title {
            <string/>
           </property>
           <property name="pixmap">
-           <pixmap resource="../../Polyhedron_3.qrc">:/cgal/Polyhedron_3/resources/icosphere.png</pixmap>
+           <pixmap>:/cgal/Polyhedron_3/resources/icosphere.png</pixmap>
           </property>
          </widget>
         </item>
@@ -424,18 +412,15 @@ QGroupBox::title {
                 </widget>
                </item>
                <item>
-                <widget class="QDoubleSpinBox" name="pyramidHeightSpinBox">
-                 <property name="decimals">
-                  <number>5</number>
-                 </property>
-                 <property name="minimum">
-                  <double>-999999.000000000000000</double>
-                 </property>
-                 <property name="maximum">
-                  <double>999999.000000000000000</double>
+                <widget class="DoubleEdit" name="pyramidHeightSpinBox">
+                 <property name="sizePolicy">
+                  <sizepolicy hsizetype="Minimum" vsizetype="Fixed">
+                   <horstretch>0</horstretch>
+                   <verstretch>0</verstretch>
+                  </sizepolicy>
                  </property>
-                 <property name="value">
-                  <double>1.000000000000000</double>
+                 <property name="text">
+                  <string>1.00</string>
                  </property>
                 </widget>
                </item>
@@ -469,7 +454,7 @@ QGroupBox::title {
               </layout>
              </item>
              <item>
-              <layout class="QHBoxLayout" name="horizontalLayout_13" stretch="0,0">
+              <layout class="QHBoxLayout" name="horizontalLayout_13" stretch="1,0">
                <item>
                 <widget class="QLabel" name="label_13">
                  <property name="toolTip">
@@ -481,18 +466,15 @@ QGroupBox::title {
                 </widget>
                </item>
                <item>
-                <widget class="QDoubleSpinBox" name="pyramidBaseSpinBox">
-                 <property name="decimals">
-                  <number>5</number>
+                <widget class="DoubleEdit" name="pyramidBaseSpinBox">
+                 <property name="sizePolicy">
+                  <sizepolicy hsizetype="Minimum" vsizetype="Fixed">
+                   <horstretch>0</horstretch>
+                   <verstretch>0</verstretch>
+                  </sizepolicy>
                  </property>
-                 <property name="minimum">
-                  <double>-999999.000000000000000</double>
-                 </property>
-                 <property name="maximum">
-                  <double>999999.000000000000000</double>
-                 </property>
-                 <property name="value">
-                  <double>1.000000000000000</double>
+                 <property name="text">
+                  <string>1.00</string>
                  </property>
                 </widget>
                </item>
@@ -558,7 +540,7 @@ QGroupBox::title {
              <string/>
             </property>
             <property name="pixmap">
-             <pixmap resource="../../Polyhedron_3.qrc">:/cgal/Polyhedron_3/resources/hexahedron.png</pixmap>
+             <pixmap>:/cgal/Polyhedron_3/resources/hexahedron.png</pixmap>
             </property>
             <property name="scaledContents">
              <bool>false</bool>
@@ -768,7 +750,7 @@ QGroupBox::title {
            <string/>
           </property>
           <property name="pixmap">
-           <pixmap resource="../../Polyhedron_3.qrc">:/cgal/Polyhedron_3/resources/tetrahedron.png</pixmap>
+           <pixmap>:/cgal/Polyhedron_3/resources/tetrahedron.png</pixmap>
           </property>
          </widget>
         </item>
@@ -899,7 +881,7 @@ QGroupBox::title {
              <string/>
             </property>
             <property name="pixmap">
-             <pixmap resource="../../Polyhedron_3.qrc">:/cgal/Polyhedron_3/resources/grid.png</pixmap>
+             <pixmap>:/cgal/Polyhedron_3/resources/grid.png</pixmap>
             </property>
            </widget>
           </item>
@@ -1068,7 +1050,7 @@ QGroupBox::title {
 &lt;html&gt;&lt;head&gt;&lt;meta name=&quot;qrichtext&quot; content=&quot;1&quot; /&gt;&lt;style type=&quot;text/css&quot;&gt;
 p, li { white-space: pre-wrap; }
 &lt;/style&gt;&lt;/head&gt;&lt;body style=&quot; font-family:'Sans Serif'; font-size:9pt; font-weight:400; font-style:normal;&quot;&gt;
-&lt;p style=&quot;-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:10pt;&quot;&gt;&lt;br /&gt;&lt;/p&gt;&lt;/body&gt;&lt;/html&gt;</string>
+&lt;p style=&quot;-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:'Sans Serif'; font-size:10pt;&quot;&gt;&lt;br /&gt;&lt;/p&gt;&lt;/body&gt;&lt;/html&gt;</string>
           </property>
           <property name="overwriteMode">
            <bool>false</bool>
@@ -1105,7 +1087,7 @@ p, li { white-space: pre-wrap; }
 &lt;html&gt;&lt;head&gt;&lt;meta name=&quot;qrichtext&quot; content=&quot;1&quot; /&gt;&lt;style type=&quot;text/css&quot;&gt;
 p, li { white-space: pre-wrap; }
 &lt;/style&gt;&lt;/head&gt;&lt;body style=&quot; font-family:'Sans Serif'; font-size:9pt; font-weight:400; font-style:normal;&quot;&gt;
-&lt;p style=&quot;-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:10pt;&quot;&gt;&lt;br /&gt;&lt;/p&gt;&lt;/body&gt;&lt;/html&gt;</string>
+&lt;p style=&quot;-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:'Sans Serif'; font-size:10pt;&quot;&gt;&lt;br /&gt;&lt;/p&gt;&lt;/body&gt;&lt;/html&gt;</string>
           </property>
           <property name="overwriteMode">
            <bool>false</bool>
@@ -1183,11 +1165,13 @@ p, li { white-space: pre-wrap; }
    </layout>
   </widget>
  </widget>
- <resources>
-  <include location="../../Polyhedron_3.qrc"/>
-  <include location="../../Polyhedron_3.qrc"/>
-  <include location="../../Polyhedron_3.qrc"/>
-  <include location="../../Polyhedron_3.qrc"/>
- </resources>
+ <customwidgets>
+  <customwidget>
+   <class>DoubleEdit</class>
+   <extends>QLineEdit</extends>
+   <header>CGAL_double_edit.h</header>
+  </customwidget>
+ </customwidgets>
+ <resources/>
  <connections/>
 </ui>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/PCA/CMakeLists.txt 5.1-1/demo/Polyhedron/Plugins/PCA/CMakeLists.txt
--- 5.0.3-1/demo/Polyhedron/Plugins/PCA/CMakeLists.txt	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/PCA/CMakeLists.txt	2020-09-08 12:12:20.000000000 +0000
@@ -17,6 +17,9 @@ target_link_libraries(clipping_box_plugi
 polyhedron_demo_plugin(create_bbox_mesh_plugin Create_bbox_mesh_plugin)
 target_link_libraries(create_bbox_mesh_plugin PUBLIC scene_surface_mesh_item)
 
+polyhedron_demo_plugin(create_obb_mesh_plugin Create_obb_mesh_plugin)
+target_link_libraries(create_obb_mesh_plugin PUBLIC scene_surface_mesh_item scene_selection_item scene_points_with_normal_item)
+
 qt5_wrap_ui( volumesUI_FILES Basic_generator_widget.ui)
 polyhedron_demo_plugin(basic_generator_plugin Basic_generator_plugin ${volumesUI_FILES} KEYWORDS PolygonMesh PointSetProcessing)
 target_link_libraries(basic_generator_plugin PUBLIC scene_surface_mesh_item scene_points_with_normal_item scene_polylines_item)
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/PCA/Create_obb_mesh_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/PCA/Create_obb_mesh_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/PCA/Create_obb_mesh_plugin.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/PCA/Create_obb_mesh_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,154 @@
+#include <QtCore/qglobal.h>
+
+#include <CGAL/Three/Scene_item.h>
+#include <CGAL/Three/Scene_interface.h>
+
+#include <QAction>
+#include <QMainWindow>
+#include <QMessageBox>
+#include <QApplication>
+
+#include "Scene_surface_mesh_item.h"
+#include "Scene_polyhedron_selection_item.h"
+#include "Scene_points_with_normal_item.h"
+
+#include <CGAL/Three/Polyhedron_demo_plugin_interface.h>
+
+#include <CGAL/optimal_bounding_box.h>
+
+using namespace CGAL::Three;
+
+typedef Scene_surface_mesh_item Scene_facegraph_item;
+
+class Create_obb_mesh_plugin :
+  public QObject,
+  public CGAL::Three::Polyhedron_demo_plugin_interface
+{
+  Q_OBJECT
+  Q_INTERFACES(CGAL::Three::Polyhedron_demo_plugin_interface)
+  Q_PLUGIN_METADATA(IID "com.geometryfactory.PolyhedronDemo.PluginInterface/1.0")
+
+public:
+  void init(QMainWindow* mainWindow, Scene_interface* scene_interface, Messages_interface*);
+  QList<QAction*> actions() const;
+
+  bool applicable(QAction*) const
+  {
+    if(scene->mainSelectionIndex() != -1
+       && scene->item(scene->mainSelectionIndex())->isFinite())
+      return true;
+    return false;
+  }
+
+protected:
+  void gather_mesh_points(std::vector<Point_3>& points);
+  void obb();
+
+public Q_SLOTS:
+  void createObb()
+  {
+    QApplication::setOverrideCursor(Qt::WaitCursor);
+    obb();
+    QApplication::restoreOverrideCursor();
+  }
+
+private:
+  Scene_interface* scene;
+  QMainWindow* mw;
+  QAction* actionObb;
+}; // end Create_obb_mesh_plugin class
+
+void Create_obb_mesh_plugin::init(QMainWindow* mainWindow, Scene_interface* scene_interface, Messages_interface*)
+{
+  scene = scene_interface;
+  mw = mainWindow;
+  actionObb = new QAction(tr("Create &Optimal Bbox Mesh"), mainWindow);
+  actionObb->setObjectName("createObbMeshAction");
+  connect(actionObb, SIGNAL(triggered()), this, SLOT(createObb()));
+}
+
+QList<QAction*> Create_obb_mesh_plugin::actions() const
+{
+  return QList<QAction*>() << actionObb;
+}
+
+void Create_obb_mesh_plugin::gather_mesh_points(std::vector<Point_3>& points)
+{
+  const Scene_interface::Item_id index = scene->mainSelectionIndex();
+
+  Scene_facegraph_item* item = qobject_cast<Scene_facegraph_item*>(scene->item(index));
+
+  Scene_polyhedron_selection_item* selection_item =
+    qobject_cast<Scene_polyhedron_selection_item*>(scene->item(index));
+
+  Scene_points_with_normal_item* point_set_item =
+    qobject_cast<Scene_points_with_normal_item*>(scene->item(index));
+
+  if(item || selection_item)
+  {
+    typedef typename boost::property_map<FaceGraph, boost::vertex_point_t>::type PointPMap;
+    typedef typename boost::graph_traits<FaceGraph>::vertex_descriptor vertex_descriptor;
+    typedef typename boost::graph_traits<FaceGraph>::face_descriptor face_descriptor;
+
+    std::vector<vertex_descriptor> selected_vertices;
+
+    if(item != NULL)
+    {
+      FaceGraph& pmesh = *item->polyhedron();
+      selected_vertices.assign(vertices(pmesh).begin(), vertices(pmesh).end());
+      PointPMap pmap = get(CGAL::vertex_point, pmesh);
+      for(vertex_descriptor v : selected_vertices)
+        points.push_back(get(pmap, v));
+
+    }
+    else if(selection_item != NULL) // using selection of faces
+    {
+      FaceGraph& pmesh = *selection_item->polyhedron();
+      for(face_descriptor f : selection_item->selected_facets)
+      {
+        for(vertex_descriptor v : vertices_around_face(halfedge(f, pmesh), pmesh))
+          selected_vertices.push_back(v);
+      }
+
+      PointPMap pmap = get(CGAL::vertex_point, pmesh);
+      for(vertex_descriptor v : selected_vertices)
+        points.push_back(get(pmap, v));
+    }
+
+    CGAL_assertion(points.size() >= 3);
+  }
+
+  if(point_set_item)
+  {
+    Point_set* points_set = point_set_item->point_set();
+    if(points_set == NULL)
+        return;
+
+    std::cout << "points_set->size()= " << points_set->size() << std::endl;
+    for(const Point_3& p : points_set->points())
+      points.push_back(p);
+  }
+}
+
+void Create_obb_mesh_plugin::obb()
+{
+  // gather point coordinates
+  std::vector<Point_3> points;
+  gather_mesh_points(points);
+
+  // find obb
+  std::array<Point_3, 8> obb_points;
+  CGAL::oriented_bounding_box(points, obb_points);
+
+  Scene_facegraph_item* item;
+  SMesh* p = new SMesh;
+  CGAL::make_hexahedron(obb_points[0], obb_points[1], obb_points[2], obb_points[3],
+                        obb_points[4], obb_points[5], obb_points[6], obb_points[7], *p);
+  item = new Scene_facegraph_item(p);
+
+  item->setName("Optimal bbox mesh");
+  item->setRenderingMode(Wireframe);
+  scene->addItem(item);
+}
+
+#include "Create_obb_mesh_plugin.moc"
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/PCA/MeshOnGrid_dialog.ui 5.1-1/demo/Polyhedron/Plugins/PCA/MeshOnGrid_dialog.ui
--- 5.0.3-1/demo/Polyhedron/Plugins/PCA/MeshOnGrid_dialog.ui	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/PCA/MeshOnGrid_dialog.ui	2020-09-08 12:12:20.000000000 +0000
@@ -26,13 +26,6 @@
       <string>Along X</string>
      </property>
      <layout class="QGridLayout" name="gridLayout_2">
-      <item row="1" column="1">
-       <widget class="QDoubleSpinBox" name="x_space_doubleSpinBox">
-        <property name="maximum">
-         <double>99999999999999999475366575191804932315794610450682175621941694731908308538307845136842752.000000000000000</double>
-        </property>
-       </widget>
-      </item>
       <item row="1" column="0">
        <widget class="QLabel" name="label_4">
         <property name="text">
@@ -57,6 +50,13 @@
         </property>
        </widget>
       </item>
+      <item row="1" column="1">
+       <widget class="DoubleEdit" name="x_space_doubleSpinBox">
+        <property name="text">
+         <string>0.00</string>
+        </property>
+       </widget>
+      </item>
      </layout>
     </widget>
    </item>
@@ -66,13 +66,6 @@
       <string>Along Y</string>
      </property>
      <layout class="QGridLayout" name="gridLayout_3">
-      <item row="1" column="1">
-       <widget class="QDoubleSpinBox" name="y_space_doubleSpinBox">
-        <property name="maximum">
-         <double>99999999999999999475366575191804932315794610450682175621941694731908308538307845136842752.000000000000000</double>
-        </property>
-       </widget>
-      </item>
       <item row="1" column="0">
        <widget class="QLabel" name="label_5">
         <property name="text">
@@ -97,6 +90,13 @@
         </property>
        </widget>
       </item>
+      <item row="1" column="1">
+       <widget class="DoubleEdit" name="y_space_doubleSpinBox">
+        <property name="text">
+         <string>0.00</string>
+        </property>
+       </widget>
+      </item>
      </layout>
     </widget>
    </item>
@@ -106,13 +106,6 @@
       <string>Along Z</string>
      </property>
      <layout class="QGridLayout" name="gridLayout">
-      <item row="1" column="1">
-       <widget class="QDoubleSpinBox" name="z_space_doubleSpinBox">
-        <property name="maximum">
-         <double>99999999999999999475366575191804932315794610450682175621941694731908308538307845136842752.000000000000000</double>
-        </property>
-       </widget>
-      </item>
       <item row="0" column="0">
        <widget class="QLabel" name="label_3">
         <property name="text">
@@ -137,6 +130,13 @@
         </property>
        </widget>
       </item>
+      <item row="1" column="1">
+       <widget class="DoubleEdit" name="z_space_doubleSpinBox">
+        <property name="text">
+         <string>0.00</string>
+        </property>
+       </widget>
+      </item>
      </layout>
     </widget>
    </item>
@@ -165,6 +165,13 @@
    </item>
   </layout>
  </widget>
+ <customwidgets>
+  <customwidget>
+   <class>DoubleEdit</class>
+   <extends>QLineEdit</extends>
+   <header>CGAL_double_edit.h</header>
+  </customwidget>
+ </customwidgets>
  <resources/>
  <connections>
   <connection>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/PMP/CMakeLists.txt 5.1-1/demo/Polyhedron/Plugins/PMP/CMakeLists.txt
--- 5.0.3-1/demo/Polyhedron/Plugins/PMP/CMakeLists.txt	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/PMP/CMakeLists.txt	2020-09-08 12:12:20.000000000 +0000
@@ -1,26 +1,29 @@
 include( polyhedron_demo_macros )
-if(EIGEN3_FOUND)
+if(TARGET CGAL::Eigen_support)
 
   polyhedron_demo_plugin(jet_fitting_plugin Jet_fitting_plugin)
-  target_link_libraries(jet_fitting_plugin PUBLIC scene_surface_mesh_item scene_polylines_item)
+  target_link_libraries(jet_fitting_plugin PUBLIC scene_surface_mesh_item scene_polylines_item CGAL::Eigen_support)
 
-else(EIGEN3_FOUND)
+else()
   message(STATUS "NOTICE: Eigen 3.1 (or greater) was not found. Jet fitting plugin will not be available.")
-endif(EIGEN3_FOUND)
+endif()
+
+polyhedron_demo_plugin(extrude_plugin Extrude_plugin KEYWORDS PMP)
+target_link_libraries(extrude_plugin PUBLIC scene_surface_mesh_item scene_selection_item)
 
-if(EIGEN3_FOUND)
+if(TARGET CGAL::Eigen_support)
   if("${EIGEN3_VERSION}" VERSION_GREATER "3.1.90")
 
     qt5_wrap_ui( hole_fillingUI_FILES Hole_filling_widget.ui)
     polyhedron_demo_plugin(hole_filling_plugin Hole_filling_plugin ${hole_fillingUI_FILES} KEYWORDS PMP)
-    target_link_libraries(hole_filling_plugin PUBLIC scene_surface_mesh_item scene_polylines_item scene_selection_item)
+    target_link_libraries(hole_filling_plugin PUBLIC scene_surface_mesh_item scene_polylines_item scene_selection_item CGAL::Eigen_support)
 
     qt5_wrap_ui( fairingUI_FILES Fairing_widget.ui)
     polyhedron_demo_plugin(fairing_plugin Fairing_plugin ${fairingUI_FILES} KEYWORDS PMP)
-    target_link_libraries(fairing_plugin PUBLIC scene_selection_item)
+    target_link_libraries(fairing_plugin PUBLIC scene_selection_item CGAL::Eigen_support)
 
     polyhedron_demo_plugin(hole_filling_polyline_plugin Hole_filling_polyline_plugin )
-    target_link_libraries(hole_filling_polyline_plugin PUBLIC scene_surface_mesh_item scene_polylines_item)
+    target_link_libraries(hole_filling_polyline_plugin PUBLIC scene_surface_mesh_item scene_polylines_item CGAL::Eigen_support)
 
     qt5_wrap_ui( Mean_curvature_flow_skeleton_pluginUI_FILES  Mean_curvature_flow_skeleton_plugin.ui)
     polyhedron_demo_plugin(mean_curvature_flow_skeleton_plugin Mean_curvature_flow_skeleton_plugin ${Mean_curvature_flow_skeleton_pluginUI_FILES})
@@ -30,7 +33,19 @@ if(EIGEN3_FOUND)
         scene_points_with_normal_item
         scene_polylines_item
         scene_mcf_item
-        demo_framework)
+        demo_framework
+        CGAL::Eigen_support)
+      # The smoothing plugin can still do some things, even if Ceres is not found
+      qt5_wrap_ui( smoothingUI_FILES Smoothing_plugin.ui)
+      polyhedron_demo_plugin(smoothing_plugin Smoothing_plugin ${smoothingUI_FILES})
+      target_link_libraries(smoothing_plugin PUBLIC scene_surface_mesh_item scene_selection_item CGAL::Eigen_support)
+      find_package(Ceres QUIET)
+      if(TARGET ceres)
+        target_compile_definitions( smoothing_plugin PRIVATE CGAL_PMP_USE_CERES_SOLVER )
+        target_link_libraries(smoothing_plugin PUBLIC ceres)
+      endif()
+
+      target_link_libraries(extrude_plugin PUBLIC CGAL::Eigen_support)
   else()
     message(STATUS "NOTICE: The hole filling and fairing plugins require Eigen 3.2 (or higher) and will not be available.")
   endif()
@@ -78,24 +93,24 @@ target_link_libraries(corefinement_plugi
 polyhedron_demo_plugin(surface_intersection_plugin Surface_intersection_plugin KEYWORDS PMP)
 target_link_libraries(surface_intersection_plugin PUBLIC scene_surface_mesh_item scene_polylines_item scene_points_with_normal_item)
 
-polyhedron_demo_plugin(repair_polyhedron_plugin Repair_polyhedron_plugin KEYWORDS PMP)
+qt5_wrap_ui( repairUI_FILES RemoveNeedlesDialog.ui)
+polyhedron_demo_plugin(repair_polyhedron_plugin Repair_polyhedron_plugin  ${repairUI_FILES} KEYWORDS PMP)
 target_link_libraries(repair_polyhedron_plugin PUBLIC scene_surface_mesh_item)
 
 qt5_wrap_ui( isotropicRemeshingUI_FILES Isotropic_remeshing_dialog.ui)
 polyhedron_demo_plugin(isotropic_remeshing_plugin Isotropic_remeshing_plugin  ${isotropicRemeshingUI_FILES} KEYWORDS PMP)
 target_link_libraries(isotropic_remeshing_plugin PUBLIC scene_surface_mesh_item scene_selection_item)
 
-if(TBB_FOUND)
-  CGAL_target_use_TBB(isotropic_remeshing_plugin)
+if(TARGET CGAL::TBB_support)
+  target_link_libraries(isotropic_remeshing_plugin PUBLIC CGAL::TBB_support)
 endif()
 
 polyhedron_demo_plugin(distance_plugin Distance_plugin KEYWORDS PMP)
 target_link_libraries(distance_plugin PUBLIC scene_surface_mesh_item  scene_color_ramp)
-if(TBB_FOUND)
-  CGAL_target_use_TBB(distance_plugin)
+if(TARGET CGAL::TBB_support)
+  target_link_libraries(distance_plugin PUBLIC CGAL::TBB_support)
 endif()
 
-
 polyhedron_demo_plugin(detect_sharp_edges_plugin Detect_sharp_edges_plugin KEYWORDS IO Mesh_3 PMP)
 target_link_libraries(detect_sharp_edges_plugin PUBLIC scene_surface_mesh_item)
 
@@ -109,17 +124,3 @@ target_link_libraries(degenerated_faces_
 qt5_wrap_ui( engravUI_FILES Engrave_dock_widget.ui )
 polyhedron_demo_plugin(engrave_text_plugin Engrave_text_plugin ${engravUI_FILES})
 target_link_libraries(engrave_text_plugin PUBLIC scene_surface_mesh_item scene_selection_item scene_polylines_item)
-
-polyhedron_demo_plugin(extrude_plugin Extrude_plugin KEYWORDS PMP)
-target_link_libraries(extrude_plugin PUBLIC scene_surface_mesh_item scene_selection_item)
-
-# The smoothing plugin can still do some things, even if Ceres is not found
-qt5_wrap_ui( smoothingUI_FILES Smoothing_plugin.ui)
-polyhedron_demo_plugin(smoothing_plugin Smoothing_plugin ${smoothingUI_FILES})
-target_link_libraries(smoothing_plugin PUBLIC scene_surface_mesh_item scene_selection_item)
-
-find_package(Ceres QUIET)
-if(TARGET ceres)
-  target_compile_definitions( smoothing_plugin PRIVATE CGAL_PMP_USE_CERES_SOLVER )
-  target_link_libraries(smoothing_plugin PUBLIC ceres)
-endif()
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/PMP/Distance_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/PMP/Distance_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/PMP/Distance_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/PMP/Distance_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -34,6 +34,7 @@ typedef Scene_surface_mesh_item Scene_fa
 typedef Scene_face_graph_item::Face_graph Face_graph;
 
 #if defined(CGAL_LINKED_WITH_TBB)
+#include <tbb/parallel_for.h>
 template <class AABB_tree, class Point_3>
 struct Distance_computation{
   const AABB_tree& tree;
@@ -133,7 +134,6 @@ private:
     typedef CGAL::AABB_tree< Traits > Tree;
 
     Tree tree( faces(m).first, faces(m).second, m);
-    tree.accelerate_distance_queries();
     tree.build();
     boost::graph_traits<Face_graph>::vertex_descriptor vd = *(vertices(m).first);
     Traits::Point_3 hint = get(CGAL::vertex_point,m, vd);
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/PMP/Engrave_text_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/PMP/Engrave_text_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/PMP/Engrave_text_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/PMP/Engrave_text_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -105,11 +105,11 @@ struct Top
 
 typedef EPICK                                                        Gt;
 typedef CGAL::Triangulation_vertex_base_2<Gt>                        Vb;
-typedef CGAL::Triangulation_face_base_with_info_2<FaceInfo2,Gt >    Fbb;
+typedef CGAL::Triangulation_face_base_with_info_2<FaceInfo2,Gt >     Fbb;
 typedef CGAL::Constrained_triangulation_face_base_2<Gt,Fbb>          Fb;
-typedef CGAL::Triangulation_data_structure_2<Vb, Fb>                TDS;
-typedef CGAL::No_intersection_tag                                   Tag;
-typedef CGAL::Constrained_Delaunay_triangulation_2<Gt, TDS, Tag>    CDT;
+typedef CGAL::Triangulation_data_structure_2<Vb, Fb>                 TDS;
+typedef CGAL::No_constraint_intersection_requiring_constructions_tag Tag;
+typedef CGAL::Constrained_Delaunay_triangulation_2<Gt, TDS, Tag>     CDT;
 
 //Parameterization and text displaying
 class ParamItem : public QGraphicsItem
@@ -249,12 +249,17 @@ protected:
     }
     case QEvent::Wheel: {
       QWheelEvent* event = static_cast<QWheelEvent*>(ev);
-      QPointF old_pos = v->mapToScene(event->pos());
-      if(event->delta() <0)
+#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
+      QPoint pos = event->pos();
+#else
+      QPointF pos = event->position();
+#endif
+      QPointF old_pos = v->mapToScene(pos.x(), pos.y());
+      if(event->angleDelta().y() <0)
         v->scale(1.2, 1.2);
       else
         v->scale(0.8, 0.8);
-      QPointF new_pos = v->mapToScene(event->pos());
+      QPointF new_pos = v->mapToScene(pos.x(), pos.y());
       QPointF delta = new_pos - old_pos;
       v->translate(delta.x(), delta.y());
       v->update();
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/PMP/Extrude_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/PMP/Extrude_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/PMP/Extrude_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/PMP/Extrude_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -116,7 +116,7 @@ public :
     if(event->type() == QEvent::Wheel && ctrl_pressing)
     {
       QWheelEvent *mouseEvent = static_cast<QWheelEvent*>(event);
-      int steps = mouseEvent->delta() / 120;
+      int steps = mouseEvent->angleDelta().y() / 120;
       if (steps > 0)
         length_+=tick;
       else
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/PMP/Fairing_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/PMP/Fairing_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/PMP/Fairing_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/PMP/Fairing_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -64,6 +64,7 @@ public:
     dock_widget->setVisible(false);
 
     ui_widget.setupUi(dock_widget);
+    ui_widget.Density_control_factor_spin_box->setMaximum(96.989999999999995);
     addDockWidget(dock_widget);
     dock_widget->setWindowTitle(tr(
                                   "Fairing "
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/PMP/Fairing_widget.ui 5.1-1/demo/Polyhedron/Plugins/PMP/Fairing_widget.ui
--- 5.0.3-1/demo/Polyhedron/Plugins/PMP/Fairing_widget.ui	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/PMP/Fairing_widget.ui	2020-09-08 12:12:20.000000000 +0000
@@ -99,15 +99,9 @@
           </widget>
          </item>
          <item>
-          <widget class="QDoubleSpinBox" name="Density_control_factor_spin_box">
-           <property name="maximum">
-            <double>96.989999999999995</double>
-           </property>
-           <property name="singleStep">
-            <double>0.200000000000000</double>
-           </property>
-           <property name="value">
-            <double>2.410000000000000</double>
+          <widget class="DoubleEdit" name="Density_control_factor_spin_box">
+           <property name="text">
+            <string>2.41</string>
            </property>
           </widget>
          </item>
@@ -139,6 +133,13 @@
    </layout>
   </widget>
  </widget>
+ <customwidgets>
+  <customwidget>
+   <class>DoubleEdit</class>
+   <extends>QLineEdit</extends>
+   <header>CGAL_double_edit.h</header>
+  </customwidget>
+ </customwidgets>
  <resources/>
  <connections/>
 </ui>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/PMP/Hole_filling_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/PMP/Hole_filling_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/PMP/Hole_filling_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/PMP/Hole_filling_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -443,6 +443,7 @@ void Polyhedron_demo_hole_filling_plugin
   dock_widget->installEventFilter(this);
 
   ui_widget.setupUi(dock_widget);
+  ui_widget.Density_control_factor_spin_box->setMaximum(96.989999999999995);
   ui_widget.Accept_button->setVisible(false);
   ui_widget.Reject_button->setVisible(false);
 
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/PMP/Hole_filling_widget.ui 5.1-1/demo/Polyhedron/Plugins/PMP/Hole_filling_widget.ui
--- 5.0.3-1/demo/Polyhedron/Plugins/PMP/Hole_filling_widget.ui	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/PMP/Hole_filling_widget.ui	2020-09-08 12:12:20.000000000 +0000
@@ -87,18 +87,9 @@
           </widget>
          </item>
          <item>
-          <widget class="QDoubleSpinBox" name="vertices_threshold_spin_box">
-           <property name="decimals">
-            <number>0</number>
-           </property>
-           <property name="maximum">
-            <double>999999999.000000000000000</double>
-           </property>
-           <property name="singleStep">
-            <double>1.000000000000000</double>
-           </property>
-           <property name="value">
-            <double>10.000000000000000</double>
+          <widget class="DoubleEdit" name="vertices_threshold_spin_box">
+           <property name="text">
+            <string>10.0</string>
            </property>
           </widget>
          </item>
@@ -123,15 +114,18 @@
           </widget>
          </item>
          <item>
-          <widget class="QDoubleSpinBox" name="Density_control_factor_spin_box">
-           <property name="maximum">
-            <double>96.989999999999995</double>
+          <widget class="DoubleEdit" name="Density_control_factor_spin_box">
+           <property name="sizePolicy">
+            <sizepolicy hsizetype="Minimum" vsizetype="Fixed">
+             <horstretch>0</horstretch>
+             <verstretch>0</verstretch>
+            </sizepolicy>
            </property>
-           <property name="singleStep">
-            <double>0.200000000000000</double>
+           <property name="text">
+            <string>1.41</string>
            </property>
-           <property name="value">
-            <double>1.410000000000000</double>
+           <property name="alignment">
+            <set>Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter</set>
            </property>
           </widget>
          </item>
@@ -333,6 +327,13 @@
    </layout>
   </widget>
  </widget>
+ <customwidgets>
+  <customwidget>
+   <class>DoubleEdit</class>
+   <extends>QLineEdit</extends>
+   <header>CGAL_double_edit.h</header>
+  </customwidget>
+ </customwidgets>
  <resources/>
  <connections/>
 </ui>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/PMP/Isotropic_remeshing_dialog.ui 5.1-1/demo/Polyhedron/Plugins/PMP/Isotropic_remeshing_dialog.ui
--- 5.0.3-1/demo/Polyhedron/Plugins/PMP/Isotropic_remeshing_dialog.ui	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/PMP/Isotropic_remeshing_dialog.ui	2020-09-08 12:12:20.000000000 +0000
@@ -10,7 +10,7 @@
     <x>0</x>
     <y>0</y>
     <width>376</width>
-    <height>368</height>
+    <height>369</height>
    </rect>
   </property>
   <property name="windowTitle">
@@ -98,9 +98,6 @@
         <property name="alignment">
          <set>Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter</set>
         </property>
-        <property name="buddy">
-         <cstring>edgeLength_dspinbox</cstring>
-        </property>
        </widget>
       </item>
       <item row="4" column="0">
@@ -176,22 +173,6 @@
         </property>
        </widget>
       </item>
-      <item row="0" column="1">
-       <widget class="QDoubleSpinBox" name="edgeLength_dspinbox">
-        <property name="minimumSize">
-         <size>
-          <width>110</width>
-          <height>0</height>
-         </size>
-        </property>
-        <property name="maximum">
-         <double>1000.000000000000000</double>
-        </property>
-        <property name="singleStep">
-         <double>0.100000000000000</double>
-        </property>
-       </widget>
-      </item>
       <item row="3" column="0">
        <spacer name="verticalSpacer_3">
         <property name="orientation">
@@ -208,6 +189,22 @@
         </property>
        </spacer>
       </item>
+      <item row="0" column="1">
+       <widget class="DoubleEdit" name="edgeLength_dspinbox">
+        <property name="inputMethodHints">
+         <set>Qt::ImhNone</set>
+        </property>
+        <property name="inputMask">
+         <string/>
+        </property>
+        <property name="text">
+         <string/>
+        </property>
+        <property name="placeholderText">
+         <string>0.00</string>
+        </property>
+       </widget>
+      </item>
      </layout>
     </widget>
    </item>
@@ -239,9 +236,15 @@
    </item>
   </layout>
  </widget>
+ <customwidgets>
+  <customwidget>
+   <class>DoubleEdit</class>
+   <extends>QLineEdit</extends>
+   <header>CGAL_double_edit.h</header>
+  </customwidget>
+ </customwidgets>
  <tabstops>
   <tabstop>splitEdgesOnly_checkbox</tabstop>
-  <tabstop>edgeLength_dspinbox</tabstop>
   <tabstop>nbIterations_spinbox</tabstop>
   <tabstop>nbSmoothing_spinbox</tabstop>
   <tabstop>protect_checkbox</tabstop>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/PMP/Isotropic_remeshing_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/PMP/Isotropic_remeshing_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/PMP/Isotropic_remeshing_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/PMP/Isotropic_remeshing_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -51,6 +51,7 @@ typedef Scene_surface_mesh_item Scene_fa
 typedef Scene_facegraph_item::Face_graph FaceGraph;
 typedef boost::graph_traits<FaceGraph>::face_descriptor face_descriptor;
 
+
 // give a halfedge and a target edge length, put in `out` points
 // which the edge equally spaced such that splitting the edge
 // using the sequence of points make the edges shorter than
@@ -903,13 +904,8 @@ private:
     double diago_length = CGAL::sqrt((bbox.xmax()-bbox.xmin())*(bbox.xmax()-bbox.xmin())
                                    + (bbox.ymax()-bbox.ymin())*(bbox.ymax()-bbox.ymin())
                                    + (bbox.zmax()-bbox.zmin())*(bbox.zmax()-bbox.zmin()));
-    double log = std::log10(diago_length);
-    unsigned int nb_decimals = (log > 0) ? 5 : (std::ceil(-log)+3);
 
-    ui.edgeLength_dspinbox->setDecimals(nb_decimals);
-    ui.edgeLength_dspinbox->setSingleStep(1e-3);
-    ui.edgeLength_dspinbox->setRange(1e-6 * diago_length, //min
-                                     2.   * diago_length);//max
+
     ui.edgeLength_dspinbox->setValue(0.05 * diago_length);
 
     std::ostringstream oss;
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/PMP/Join_and_split_polyhedra_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/PMP/Join_and_split_polyhedra_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/PMP/Join_and_split_polyhedra_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/PMP/Join_and_split_polyhedra_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -51,7 +51,7 @@ public:
     actionJoinPolyhedra->setObjectName("actionJoinPolyhedra");
 
     actionSplitPolyhedra= new QAction(tr("Split Selected Polyhedra"), mainWindow);
-    actionSplitPolyhedra->setProperty("subMenuName", "Operations on Polyhedra");
+    actionSplitPolyhedra->setProperty("subMenuName", "Polygon Mesh Processing");
     actionSplitPolyhedra->setObjectName("actionSplitPolyhedra");
 
     actionColorConnectedComponents = new QAction(tr("Color Each Connected Component"), mainWindow);
@@ -130,9 +130,9 @@ struct Polyhedron_appender{
 };
 
 struct Compare{
-bool operator()(FaceGraph* mesh1, FaceGraph* mesh2)
+bool operator()(const FaceGraph& mesh1, const FaceGraph& mesh2)
 {
-  return num_faces(*mesh1) < num_faces(*mesh2);
+  return num_faces(mesh1) < num_faces(mesh2);
 }
 };
 
@@ -144,28 +144,15 @@ void Polyhedron_demo_join_and_split_poly
     if(item)
     {
       QApplication::setOverrideCursor(Qt::WaitCursor);
-      std::list<FaceGraph*> new_polyhedra;
-      typedef boost::property_map<FaceGraph,CGAL::face_patch_id_t<int> >::type PatchIDMap;
-      PatchIDMap pidmap = get(CGAL::face_patch_id_t<int>(), *item->face_graph());
-      int nb_patches = item->property("NbPatchIds").toInt();
-       if(nb_patches == 0)
-         nb_patches = CGAL::Polygon_mesh_processing::connected_components(*item->face_graph(),
-                                                                          pidmap);
-
-      for(int i=0; i<nb_patches; ++i)
-      {
-        CGAL::Face_filtered_graph<FaceGraph> filter_graph(*item->face_graph(), i, pidmap);
-        FaceGraph* new_graph = new FaceGraph();
-        CGAL::copy_face_graph(filter_graph, *new_graph);
-        new_polyhedra.push_back(new_graph);
-      }
+      std::vector<FaceGraph> new_polyhedra;
+      CGAL::Polygon_mesh_processing::split_connected_components(*item->face_graph(),
+                                                                new_polyhedra);
       //sort polyhedra by number of faces
-      new_polyhedra.sort(Compare());
+      std::sort(new_polyhedra.begin(), new_polyhedra.end(), Compare());
 
 
       if (new_polyhedra.size()==1)
       {
-        delete new_polyhedra.front();
         CGAL::Three::Three::information( tr("%1 has only one connected component").arg(item->name()) );
         QApplication::restoreOverrideCursor();
         continue;
@@ -179,21 +166,15 @@ void Polyhedron_demo_join_and_split_poly
         compute_color_map(item->color(), new_polyhedra.size(), std::back_inserter(color_map));
       Scene_group_item *group = new Scene_group_item("CC");
        scene->addItem(group);
-      for(FaceGraph* polyhedron_ptr : new_polyhedra)
+      for(FaceGraph& poly : new_polyhedra)
       {
-        Scene_facegraph_item* new_item=new Scene_facegraph_item(polyhedron_ptr);
+        Scene_facegraph_item* new_item=new Scene_facegraph_item(poly);
         new_item->setName(tr("%1 - CC %2").arg(item->name()).arg(cc));
         new_item->setColor(color_map[cc]);
         ++cc;
         scene->addItem(new_item);
         scene->changeGroup(new_item, group);
       }
-
-      if(!item->hasPatchIds())//still valid bc the item has not been invalidated.
-      {
-        //remove f:patch_id map to avoid tricking the isMulticolor system.
-        item->face_graph()->remove_property_map(pidmap);
-      }
       item->setVisible(false);
       QApplication::restoreOverrideCursor();
     }
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/PMP/Mean_curvature_flow_skeleton_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/PMP/Mean_curvature_flow_skeleton_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/PMP/Mean_curvature_flow_skeleton_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/PMP/Mean_curvature_flow_skeleton_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -189,17 +189,9 @@ public:
                                             "to a surface mesh item. The generated mcf group must be selected in "
                                             "order to continue an on-going set of operations. "));});
     ui->omega_H->setValue(0.1);
-    ui->omega_H->setSingleStep(0.1);
-    ui->omega_H->setDecimals(3);
     ui->omega_P->setValue(0.2);
-    ui->omega_P->setSingleStep(0.1);
-    ui->omega_P->setDecimals(3);
-    ui->min_edge_length->setDecimals(7);
     ui->min_edge_length->setValue(0.002 * diag);
-    ui->min_edge_length->setSingleStep(0.0000001);
-    ui->delta_area->setDecimals(7);
     ui->delta_area->setValue(1e-4);
-    ui->delta_area->setSingleStep(1e-5);
     ui->is_medially_centered->setChecked(false);
 
     ui->label_omega_H->setToolTip(QString("omega_H controls the velocity of movement and approximation quality"));
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/PMP/Mean_curvature_flow_skeleton_plugin.ui 5.1-1/demo/Polyhedron/Plugins/PMP/Mean_curvature_flow_skeleton_plugin.ui
--- 5.0.3-1/demo/Polyhedron/Plugins/PMP/Mean_curvature_flow_skeleton_plugin.ui	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/PMP/Mean_curvature_flow_skeleton_plugin.ui	2020-09-08 12:12:20.000000000 +0000
@@ -6,7 +6,7 @@
    <rect>
     <x>0</x>
     <y>0</y>
-    <width>483</width>
+    <width>498</width>
     <height>279</height>
    </rect>
   </property>
@@ -51,7 +51,7 @@
       <item>
        <layout class="QVBoxLayout" name="verticalLayout_2">
         <item>
-         <layout class="QHBoxLayout" name="horizontalLayout_2">
+         <layout class="QHBoxLayout" name="horizontalLayout_2" stretch="1,0">
           <item>
            <widget class="QLabel" name="label_omega_H">
             <property name="text">
@@ -60,12 +60,22 @@
            </widget>
           </item>
           <item>
-           <widget class="QDoubleSpinBox" name="omega_H"/>
+           <widget class="DoubleEdit" name="omega_H">
+            <property name="sizePolicy">
+             <sizepolicy hsizetype="Minimum" vsizetype="Fixed">
+              <horstretch>0</horstretch>
+              <verstretch>0</verstretch>
+             </sizepolicy>
+            </property>
+            <property name="text">
+             <string>0.00</string>
+            </property>
+           </widget>
           </item>
          </layout>
         </item>
         <item>
-         <layout class="QHBoxLayout" name="horizontalLayout_3">
+         <layout class="QHBoxLayout" name="horizontalLayout_3" stretch="1,0">
           <item>
            <widget class="QLabel" name="label_omega_P">
             <property name="text">
@@ -74,12 +84,22 @@
            </widget>
           </item>
           <item>
-           <widget class="QDoubleSpinBox" name="omega_P"/>
+           <widget class="DoubleEdit" name="omega_P">
+            <property name="sizePolicy">
+             <sizepolicy hsizetype="Minimum" vsizetype="Fixed">
+              <horstretch>0</horstretch>
+              <verstretch>0</verstretch>
+             </sizepolicy>
+            </property>
+            <property name="text">
+             <string>0.00</string>
+            </property>
+           </widget>
           </item>
          </layout>
         </item>
         <item>
-         <layout class="QHBoxLayout" name="horizontalLayout_4">
+         <layout class="QHBoxLayout" name="horizontalLayout_4" stretch="1,0">
           <item>
            <widget class="QLabel" name="label_min_edge_length">
             <property name="text">
@@ -88,12 +108,16 @@
            </widget>
           </item>
           <item>
-           <widget class="QDoubleSpinBox" name="min_edge_length"/>
+           <widget class="DoubleEdit" name="min_edge_length">
+            <property name="text">
+             <string>0.00</string>
+            </property>
+           </widget>
           </item>
          </layout>
         </item>
         <item>
-         <layout class="QHBoxLayout" name="horizontalLayout_5">
+         <layout class="QHBoxLayout" name="horizontalLayout_5" stretch="1,0">
           <item>
            <widget class="QLabel" name="label_delta_area">
             <property name="text">
@@ -102,7 +126,17 @@
            </widget>
           </item>
           <item>
-           <widget class="QDoubleSpinBox" name="delta_area"/>
+           <widget class="DoubleEdit" name="delta_area">
+            <property name="sizePolicy">
+             <sizepolicy hsizetype="Minimum" vsizetype="Fixed">
+              <horstretch>0</horstretch>
+              <verstretch>0</verstretch>
+             </sizepolicy>
+            </property>
+            <property name="text">
+             <string>0.00</string>
+            </property>
+           </widget>
           </item>
          </layout>
         </item>
@@ -203,6 +237,13 @@
    </layout>
   </widget>
  </widget>
+ <customwidgets>
+  <customwidget>
+   <class>DoubleEdit</class>
+   <extends>QLineEdit</extends>
+   <header>CGAL_double_edit.h</header>
+  </customwidget>
+ </customwidgets>
  <resources/>
  <connections/>
  <slots>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/PMP/Random_perturbation_dialog.ui 5.1-1/demo/Polyhedron/Plugins/PMP/Random_perturbation_dialog.ui
--- 5.0.3-1/demo/Polyhedron/Plugins/PMP/Random_perturbation_dialog.ui	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/PMP/Random_perturbation_dialog.ui	2020-09-08 12:12:20.000000000 +0000
@@ -10,7 +10,7 @@
     <x>0</x>
     <y>0</y>
     <width>377</width>
-    <height>292</height>
+    <height>293</height>
    </rect>
   </property>
   <property name="windowTitle">
@@ -55,9 +55,6 @@
         <property name="alignment">
          <set>Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter</set>
         </property>
-        <property name="buddy">
-         <cstring>moveSize_dspinbox</cstring>
-        </property>
        </widget>
       </item>
       <item row="1" column="0">
@@ -70,22 +67,6 @@
         </property>
        </widget>
       </item>
-      <item row="0" column="1">
-       <widget class="QDoubleSpinBox" name="moveSize_dspinbox">
-        <property name="minimumSize">
-         <size>
-          <width>110</width>
-          <height>0</height>
-         </size>
-        </property>
-        <property name="maximum">
-         <double>1000.000000000000000</double>
-        </property>
-        <property name="singleStep">
-         <double>0.100000000000000</double>
-        </property>
-       </widget>
-      </item>
       <item row="1" column="1">
        <widget class="QCheckBox" name="project_checkbox">
         <property name="text">
@@ -96,6 +77,19 @@
         </property>
        </widget>
       </item>
+      <item row="0" column="1">
+       <widget class="DoubleEdit" name="moveSize_dspinbox">
+        <property name="sizePolicy">
+         <sizepolicy hsizetype="Minimum" vsizetype="Fixed">
+          <horstretch>0</horstretch>
+          <verstretch>0</verstretch>
+         </sizepolicy>
+        </property>
+        <property name="text">
+         <string>0.00</string>
+        </property>
+       </widget>
+      </item>
      </layout>
     </widget>
    </item>
@@ -198,8 +192,14 @@
    </item>
   </layout>
  </widget>
+ <customwidgets>
+  <customwidget>
+   <class>DoubleEdit</class>
+   <extends>QLineEdit</extends>
+   <header>CGAL_double_edit.h</header>
+  </customwidget>
+ </customwidgets>
  <tabstops>
-  <tabstop>moveSize_dspinbox</tabstop>
   <tabstop>project_checkbox</tabstop>
   <tabstop>buttonBox</tabstop>
  </tabstops>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/PMP/Random_perturbation_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/PMP/Random_perturbation_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/PMP/Random_perturbation_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/PMP/Random_perturbation_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -196,12 +196,8 @@ public Q_SLOTS:
     double diago_length = CGAL::sqrt((bbox.xmax() - bbox.xmin())*(bbox.xmax() - bbox.xmin())
       + (bbox.ymax() - bbox.ymin())*(bbox.ymax() - bbox.ymin())
       + (bbox.zmax() - bbox.zmin())*(bbox.zmax() - bbox.zmin()));
-    double log = std::log10(diago_length);
-    unsigned int nb_decimals = (log > 0) ? 5 : (std::ceil(-log) + 3);
 
     //parameters
-    ui.moveSize_dspinbox->setDecimals(nb_decimals);
-    ui.moveSize_dspinbox->setSingleStep(1e-3);
     ui.moveSize_dspinbox->setRange(0., diago_length);
     ui.moveSize_dspinbox->setValue(0.05 * diago_length);
 
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/PMP/RemoveNeedlesDialog.ui 5.1-1/demo/Polyhedron/Plugins/PMP/RemoveNeedlesDialog.ui
--- 5.0.3-1/demo/Polyhedron/Plugins/PMP/RemoveNeedlesDialog.ui	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/PMP/RemoveNeedlesDialog.ui	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,164 @@
+<?xml version="1.0" encoding="UTF-8"?>
+<ui version="4.0">
+ <class>NeedleDialog</class>
+ <widget class="QDialog" name="NeedleDialog">
+  <property name="geometry">
+   <rect>
+    <x>0</x>
+    <y>0</y>
+    <width>607</width>
+    <height>368</height>
+   </rect>
+  </property>
+  <property name="windowTitle">
+   <string>Remove Needles and Cap</string>
+  </property>
+  <layout class="QVBoxLayout" name="verticalLayout_2">
+   <item>
+    <layout class="QVBoxLayout" name="verticalLayout">
+     <item>
+      <widget class="QGroupBox" name="Box1">
+       <property name="toolTip">
+        <string>Threshold in degrees</string>
+       </property>
+       <property name="title">
+        <string>Cap threshold (max. angle allowed within a triangle): </string>
+       </property>
+       <layout class="QHBoxLayout" name="horizontalLayout">
+        <item>
+         <layout class="QVBoxLayout" name="verticalLayout_3">
+          <item>
+           <widget class="QDoubleSpinBox" name="capBox">
+            <property name="toolTip">
+             <string>Angle in degrees</string>
+            </property>
+            <property name="suffix">
+             <string/>
+            </property>
+            <property name="decimals">
+             <number>2</number>
+            </property>
+            <property name="maximum">
+             <double>360.000000000000000</double>
+            </property>
+            <property name="value">
+             <double>160.000000000000000</double>
+            </property>
+           </widget>
+          </item>
+         </layout>
+        </item>
+       </layout>
+      </widget>
+     </item>
+     <item>
+      <widget class="QGroupBox" name="Box2">
+       <property name="toolTip">
+        <string>(size+big edge )/(size+small edge)</string>
+       </property>
+       <property name="title">
+        <string>Needle threshold (max. edge length ratio (longest/shortest) allowed within a triangle):</string>
+       </property>
+       <layout class="QHBoxLayout" name="horizontalLayout_2">
+        <item>
+         <layout class="QVBoxLayout" name="verticalLayout_4">
+          <item>
+           <widget class="QDoubleSpinBox" name="needleBox">
+            <property name="toolTip">
+             <string>Length Ratio</string>
+            </property>
+            <property name="decimals">
+             <number>2</number>
+            </property>
+            <property name="maximum">
+             <double>1000.000000000000000</double>
+            </property>
+            <property name="value">
+             <double>4.000000000000000</double>
+            </property>
+           </widget>
+          </item>
+         </layout>
+        </item>
+       </layout>
+      </widget>
+     </item>
+     <item>
+      <widget class="QGroupBox" name="Box3">
+       <property name="toolTip">
+        <string>Do not collapse edges begger than this threshold.</string>
+       </property>
+       <property name="title">
+        <string>Do not collapse needles (according to the criterion above) whose shortest edge has a length greater than:</string>
+       </property>
+       <layout class="QHBoxLayout" name="horizontalLayout_3">
+        <item>
+         <layout class="QVBoxLayout" name="verticalLayout_5">
+          <item>
+           <widget class="QDoubleSpinBox" name="collapseBox">
+            <property name="toolTip">
+             <string>Length</string>
+            </property>
+            <property name="decimals">
+             <number>25</number>
+            </property>
+            <property name="maximum">
+             <double>99999999.000000000000000</double>
+            </property>
+           </widget>
+          </item>
+         </layout>
+        </item>
+       </layout>
+      </widget>
+     </item>
+    </layout>
+   </item>
+   <item>
+    <widget class="QDialogButtonBox" name="buttonBox">
+     <property name="orientation">
+      <enum>Qt::Horizontal</enum>
+     </property>
+     <property name="standardButtons">
+      <set>QDialogButtonBox::Cancel|QDialogButtonBox::Ok</set>
+     </property>
+    </widget>
+   </item>
+  </layout>
+ </widget>
+ <resources/>
+ <connections>
+  <connection>
+   <sender>buttonBox</sender>
+   <signal>accepted()</signal>
+   <receiver>NeedleDialog</receiver>
+   <slot>accept()</slot>
+   <hints>
+    <hint type="sourcelabel">
+     <x>248</x>
+     <y>254</y>
+    </hint>
+    <hint type="destinationlabel">
+     <x>157</x>
+     <y>274</y>
+    </hint>
+   </hints>
+  </connection>
+  <connection>
+   <sender>buttonBox</sender>
+   <signal>rejected()</signal>
+   <receiver>NeedleDialog</receiver>
+   <slot>reject()</slot>
+   <hints>
+    <hint type="sourcelabel">
+     <x>316</x>
+     <y>260</y>
+    </hint>
+    <hint type="destinationlabel">
+     <x>286</x>
+     <y>274</y>
+    </hint>
+   </hints>
+  </connection>
+ </connections>
+</ui>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/PMP/Repair_polyhedron_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/PMP/Repair_polyhedron_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/PMP/Repair_polyhedron_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/PMP/Repair_polyhedron_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -15,8 +15,10 @@
 #include <CGAL/Polygon_mesh_processing/repair.h>
 #include <CGAL/Polygon_mesh_processing/internal/repair_extra.h>
 #include <CGAL/Polygon_mesh_processing/corefinement.h>
+#include <CGAL/Polygon_mesh_processing/repair_degeneracies.h>
 #include <CGAL/Polygon_mesh_processing/merge_border_vertices.h>
 
+#include "ui_RemoveNeedlesDialog.h"
 #include <cmath>
 #include <limits>
 #include <vector>
@@ -48,6 +50,7 @@ public:
     actionMergeDuplicatedVerticesOnBoundaryCycles = new QAction(tr("Merge Duplicated Vertices on Boundary Cycles"), mw);
     actionAutorefine = new QAction(tr("Autorefine Mesh"), mw);
     actionAutorefineAndRMSelfIntersections = new QAction(tr("Autorefine and Remove Self-Intersections"), mw);
+    actionRemoveNeedlesAndCaps = new QAction(tr("Remove Needles And Caps"));
 
     actionRemoveIsolatedVertices->setObjectName("actionRemoveIsolatedVertices");
     actionRemoveDegenerateFaces->setObjectName("actionRemoveDegenerateFaces");
@@ -57,6 +60,7 @@ public:
     actionMergeDuplicatedVerticesOnBoundaryCycles->setObjectName("actionMergeDuplicatedVerticesOnBoundaryCycles");
     actionAutorefine->setObjectName("actionAutorefine");
     actionAutorefineAndRMSelfIntersections->setObjectName("actionAutorefineAndRMSelfIntersections");
+    actionRemoveNeedlesAndCaps->setObjectName("actionRemoveNeedlesAndCaps");
 
     actionRemoveDegenerateFaces->setProperty("subMenuName", "Polygon Mesh Processing/Repair/Experimental");
     actionStitchCloseBorderHalfedges->setProperty("subMenuName", "Polygon Mesh Processing/Repair/Experimental");
@@ -66,6 +70,7 @@ public:
     actionMergeDuplicatedVerticesOnBoundaryCycles->setProperty("subMenuName", "Polygon Mesh Processing/Repair");
     actionAutorefine->setProperty("subMenuName", "Polygon Mesh Processing/Repair/Experimental");
     actionAutorefineAndRMSelfIntersections->setProperty("subMenuName", "Polygon Mesh Processing/Repair/Experimental");
+    actionRemoveNeedlesAndCaps->setProperty("subMenuName", "Polygon Mesh Processing/Repair/Experimental");
 
     autoConnectActions();
   }
@@ -79,7 +84,8 @@ public:
                              << actionDuplicateNMVertices
                              << actionMergeDuplicatedVerticesOnBoundaryCycles
                              << actionAutorefine
-                             << actionAutorefineAndRMSelfIntersections;
+                             << actionAutorefineAndRMSelfIntersections
+                             << actionRemoveNeedlesAndCaps;
   }
 
   bool applicable(QAction*) const
@@ -113,6 +119,7 @@ public Q_SLOTS:
   void on_actionMergeDuplicatedVerticesOnBoundaryCycles_triggered();
   void on_actionAutorefine_triggered();
   void on_actionAutorefineAndRMSelfIntersections_triggered();
+  void on_actionRemoveNeedlesAndCaps_triggered();
 
 private:
   QAction* actionRemoveIsolatedVertices;
@@ -123,6 +130,7 @@ private:
   QAction* actionMergeDuplicatedVerticesOnBoundaryCycles;
   QAction* actionAutorefine;
   QAction* actionAutorefineAndRMSelfIntersections;
+  QAction* actionRemoveNeedlesAndCaps;
 
   Messages_interface* messages;
 }; // end Polyhedron_demo_repair_polyhedron_plugin
@@ -152,6 +160,32 @@ void Polyhedron_demo_repair_polyhedron_p
   QApplication::restoreOverrideCursor();
 }
 
+void Polyhedron_demo_repair_polyhedron_plugin::on_actionRemoveNeedlesAndCaps_triggered()
+{
+  QCursor tmp_cursor(Qt::WaitCursor);
+  CGAL::Three::Three::CursorScopeGuard guard(tmp_cursor);
+
+  const Scene_interface::Item_id index = scene->mainSelectionIndex();
+  Scene_surface_mesh_item* sm_item = qobject_cast<Scene_surface_mesh_item*>(scene->item(index));
+  if(!sm_item)
+  {
+    return;
+  }
+
+  QDialog dialog;
+  Ui::NeedleDialog ui;
+  ui.setupUi(&dialog);
+  ui.collapseBox->setValue(sm_item->diagonalBbox()*0.01);
+  if(dialog.exec() != QDialog::Accepted)
+    return;
+  CGAL::Polygon_mesh_processing::experimental::remove_almost_degenerate_faces(*sm_item->face_graph(),
+                                                                               std::cos((ui.capBox->value()/180.0) * CGAL_PI),
+                                                                              ui.needleBox->value(),
+                                                                              ui.collapseBox->value());
+  sm_item->invalidateOpenGLBuffers();
+  sm_item->itemChanged();
+}
+
 template <typename Item>
 void Polyhedron_demo_repair_polyhedron_plugin::on_actionRemoveDegenerateFaces_triggered(Scene_interface::Item_id index)
 {
@@ -159,10 +193,10 @@ void Polyhedron_demo_repair_polyhedron_p
     qobject_cast<Item*>(scene->item(index));
   if (poly_item)
   {
-    std::size_t nbv = num_faces(*poly_item->polyhedron());
+    std::size_t nbv = faces(*poly_item->polyhedron()).size();
       CGAL::Polygon_mesh_processing::remove_degenerate_faces(
       *poly_item->polyhedron());
-    nbv -= num_faces(*poly_item->polyhedron());
+    nbv -= faces(*poly_item->polyhedron()).size();
     poly_item->invalidateOpenGLBuffers();
     Q_EMIT poly_item->itemChanged();
     CGAL::Three::Three::information(tr(" %1 degenerate faces have been removed.")
@@ -186,7 +220,7 @@ void Polyhedron_demo_repair_polyhedron_p
   if (poly_item)
   {
     bool solved =
-      CGAL::Polygon_mesh_processing::remove_self_intersections(
+      CGAL::Polygon_mesh_processing::experimental::remove_self_intersections(
       *poly_item->polyhedron());
     if (!solved)
       CGAL::Three::Three::information(tr("Some self-intersection could not be fixed"));
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/PMP/Selection_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/PMP/Selection_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/PMP/Selection_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/PMP/Selection_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -1,5 +1,6 @@
 #include <QtCore/qglobal.h>
 #include <QMessageBox>
+#include <QInputDialog>
 
 #include "Kernel_type.h"
 #include "Scene_surface_mesh_item.h"
@@ -18,6 +19,10 @@
 #include <QMainWindow>
 #include <QApplication>
 
+#include <QMultipleInputDialog.h>
+#include <QDoubleSpinBox>
+#include <QCheckBox>
+
 #include <map>
 
 #include <boost/graph/adjacency_list.hpp>
@@ -127,6 +132,12 @@ public:
       return res;
   }
 
+  bool isDefaultLoader(const Scene_item* item) const override{
+    if(qobject_cast<const Scene_polyhedron_selection_item*>(item))
+      return true;
+    return false;
+  }
+
   bool applicable(QAction* action) const override {
     if(action == actionSelfIntersection)
       return qobject_cast<Scene_face_graph_item*>(scene->item(scene->mainSelectionIndex()));
@@ -181,6 +192,7 @@ public:
     connect(ui_widget.Select_all_NTButton,  SIGNAL(clicked()), this, SLOT(on_Select_all_NTButton_clicked()));
     connect(ui_widget.Select_boundaryButton,  SIGNAL(clicked()), this, SLOT(on_Select_boundaryButton_clicked()));
     connect(ui_widget.Add_to_selection_button,  SIGNAL(clicked()), this, SLOT(on_Add_to_selection_button_clicked()));
+    connect(ui_widget.Regularize_button,  SIGNAL(clicked()), this, SLOT(on_Regularize_button_clicked()));
     connect(ui_widget.Clear_button,  SIGNAL(clicked()), this, SLOT(on_Clear_button_clicked()));
     connect(ui_widget.Clear_all_button,  SIGNAL(clicked()), this, SLOT(on_Clear_all_button_clicked()));
     connect(ui_widget.Inverse_selection_button,  SIGNAL(clicked()), this, SLOT(on_Inverse_selection_button_clicked()));
@@ -363,6 +375,81 @@ public Q_SLOTS:
     selection_item->add_to_selection();
     filter_operations();
   }
+  // Regularize selection using graph cut
+  void on_Regularize_button_clicked() {
+    Scene_polyhedron_selection_item* selection_item = getSelectedItem<Scene_polyhedron_selection_item>();
+    if(!selection_item) {
+      print_message("Error: there is no selected polyhedron selection item!");
+      return;
+    }
+
+    QMultipleInputDialog dialog ("Regularize Selection Border", mw);
+    QDoubleSpinBox* weight = dialog.add<QDoubleSpinBox> ("Weight:");
+    weight->setDecimals (6);
+    weight->setRange (0., 0.999999); // [0;1[
+    weight->setValue (0.5);
+
+    QCheckBox* prevent_unselection = dialog.add<QCheckBox> ("Prevent unselection");
+
+    if (dialog.exec() != QDialog::Accepted)
+      return;
+
+    boost::unordered_map<fg_face_descriptor, bool> is_selected_map;
+    for(fg_face_descriptor fh : faces(*selection_item->polyhedron()))
+    {
+      if(selection_item->selected_facets.find(fh)
+         == selection_item->selected_facets.end())
+        is_selected_map[fh]=false;
+      else
+      {
+        is_selected_map[fh]=true;
+      }
+    }
+
+    auto border_length =
+      [&]() -> double
+      {
+        double out = 0.;
+        for(Scene_polyhedron_selection_item::fg_edge_descriptor ed : edges(*selection_item->polyhedron()))
+        {
+          fg_face_descriptor f0 = face (halfedge (ed, *selection_item->polyhedron()),
+                                        *selection_item->polyhedron());
+          fg_face_descriptor f1 = face (opposite(halfedge (ed, *selection_item->polyhedron()),
+                                                 *selection_item->polyhedron()),
+                                        *selection_item->polyhedron());
+          if (is_selected_map[f0] == is_selected_map[f1])
+            continue;
+
+          fg_vertex_descriptor esource = source(ed, *selection_item->polyhedron());
+          fg_vertex_descriptor etarget = target(ed, *selection_item->polyhedron());
+
+          out += std::sqrt(CGAL::squared_distance (get (get(CGAL::vertex_point,*selection_item->polyhedron()), esource),
+                                                   get (get(CGAL::vertex_point,*selection_item->polyhedron()), etarget)));
+        }
+        return out;
+      };
+
+    std::cerr << "[Selection Regularization] Weight = " << weight->value() << std::endl;
+
+    std::cerr << "Length of border before regularization = " << border_length() << std::endl;
+
+    CGAL::regularize_face_selection_borders (*selection_item->polyhedron(),
+                                             boost::make_assoc_property_map(is_selected_map),
+                                             weight->value(),
+                                             CGAL::parameters::prevent_unselection (prevent_unselection->isChecked()));
+
+    std::cerr << "Length of border after regularization = " << border_length() << std::endl;
+
+    selection_item->selected_facets.clear();
+
+    for(fg_face_descriptor fh : faces(*selection_item->polyhedron()))
+    {
+      if (is_selected_map[fh])
+        selection_item->selected_facets.insert(fh);
+    }
+    selection_item->invalidateOpenGLBuffers();
+    selection_item->itemChanged();
+  }
   // Clear selection
   void on_Clear_button_clicked() {
     Scene_polyhedron_selection_item* selection_item = getSelectedItem<Scene_polyhedron_selection_item>();
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/PMP/Selection_widget.ui 5.1-1/demo/Polyhedron/Plugins/PMP/Selection_widget.ui
--- 5.0.3-1/demo/Polyhedron/Plugins/PMP/Selection_widget.ui	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/PMP/Selection_widget.ui	2020-09-08 12:12:20.000000000 +0000
@@ -7,7 +7,7 @@
     <x>0</x>
     <y>0</y>
     <width>613</width>
-    <height>365</height>
+    <height>409</height>
    </rect>
   </property>
   <property name="windowTitle">
@@ -425,6 +425,13 @@
              </layout>
             </item>
             <item>
+             <widget class="QPushButton" name="Regularize_button">
+              <property name="text">
+               <string>Regularize</string>
+              </property>
+             </widget>
+            </item>
+            <item>
              <widget class="QPushButton" name="Clear_button">
               <property name="toolTip">
                <string extracomment="Clear selection for Selection Type"/>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/PMP/Smoothing_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/PMP/Smoothing_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/PMP/Smoothing_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/PMP/Smoothing_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -95,7 +95,6 @@ public:
   void init_ui()
   {
     ui_widget.time_step_spinBox->setValue(0.00001);
-    ui_widget.time_step_spinBox->setSingleStep(0.00001);
     ui_widget.time_step_spinBox->setMinimum(1e-6);
 
     ui_widget.smooth_iter_spinBox->setValue(1);
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/PMP/Smoothing_plugin.ui 5.1-1/demo/Polyhedron/Plugins/PMP/Smoothing_plugin.ui
--- 5.0.3-1/demo/Polyhedron/Plugins/PMP/Smoothing_plugin.ui	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/PMP/Smoothing_plugin.ui	2020-09-08 12:12:20.000000000 +0000
@@ -31,13 +31,13 @@
        <string>Shape Smoothing</string>
       </property>
       <layout class="QGridLayout" name="gridLayout">
-       <item row="1" column="2">
-        <widget class="QDoubleSpinBox" name="time_step_spinBox">
-         <property name="decimals">
-          <number>6</number>
+       <item row="1" column="0">
+        <widget class="QLabel" name="time_step_label">
+         <property name="toolTip">
+          <string>The time step should not be too large or the smoothing will be unsatisfactory</string>
          </property>
-         <property name="minimum">
-          <double>0.000001000000000</double>
+         <property name="text">
+          <string>Time Step:</string>
          </property>
         </widget>
        </item>
@@ -48,13 +48,16 @@
          </property>
         </widget>
        </item>
-       <item row="1" column="0">
-        <widget class="QLabel" name="time_step_label">
-         <property name="toolTip">
-          <string>The time step should not be too large or the smoothing will be unsatisfactory</string>
+       <item row="1" column="2">
+        <widget class="DoubleEdit" name="time_step_spinBox">
+         <property name="sizePolicy">
+          <sizepolicy hsizetype="Minimum" vsizetype="Fixed">
+           <horstretch>0</horstretch>
+           <verstretch>0</verstretch>
+          </sizepolicy>
          </property>
          <property name="text">
-          <string>Time Step:</string>
+          <string>0.000001</string>
          </property>
         </widget>
        </item>
@@ -173,6 +176,13 @@
    </layout>
   </widget>
  </widget>
+ <customwidgets>
+  <customwidget>
+   <class>DoubleEdit</class>
+   <extends>QLineEdit</extends>
+   <header>CGAL_double_edit.h</header>
+  </customwidget>
+ </customwidgets>
  <resources/>
  <connections/>
 </ui>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Point_set/CMakeLists.txt 5.1-1/demo/Polyhedron/Plugins/Point_set/CMakeLists.txt
--- 5.0.3-1/demo/Polyhedron/Plugins/Point_set/CMakeLists.txt	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Point_set/CMakeLists.txt	2020-09-08 12:12:20.000000000 +0000
@@ -1,48 +1,64 @@
 include( polyhedron_demo_macros )
-if(EIGEN3_FOUND)
-
-  set(polyfit_linked_libraries)
-  set(polyfit_compile_definitions)
+if(TARGET CGAL::Eigen_support)
 
   find_package(SCIP QUIET)
-  if(SCIP_FOUND)
-    include_directories(BEFORE ${SCIP_INCLUDE_DIRS})
-    set(polyfit_linked_libraries ${polyfit_linked_libraries} ${SCIP_LIBRARIES})
-    set(polyfit_compile_definitions ${polyfit_compile_definitions} "-DCGAL_USE_SCIP")
-  endif()
-  find_package(GLPK QUIET)
-  if(GLPK_FOUND)
-    include_directories(BEFORE ${GLPK_INCLUDE_DIR})
-    set(polyfit_linked_libraries ${polyfit_linked_libraries} ${GLPK_LIBRARIES})
-    set(polyfit_compile_definitions ${polyfit_compile_definitions} "-DCGAL_USE_GLPK")
+  include(CGAL_SCIP_support)
+  if(NOT TARGET CGAL::SCIP_support)
+    find_package(GLPK QUIET)
+    include(CGAL_GLPK_support)
   endif()
 
-  if(NOT SCIP_FOUND AND NOT GLPK_FOUND)
+  if(NOT TARGET CGAL::SCIP_support AND NOT TARGET CGAL::GLPK_support)
     message(STATUS "NOTICE: SCIP and GLPK were not found. Polygonal surface reconstruction will not be available.")
   endif()
 
   qt5_wrap_ui( surface_reconstructionUI_FILES Surface_reconstruction_plugin.ui)
   polyhedron_demo_plugin(surface_reconstruction_plugin Surface_reconstruction_plugin Surface_reconstruction_poisson_impl Surface_reconstruction_advancing_front_impl Surface_reconstruction_scale_space_impl Surface_reconstruction_polygonal_impl ${surface_reconstructionUI_FILES} KEYWORDS PointSetProcessing)
-  target_link_libraries(surface_reconstruction_plugin PUBLIC scene_polygon_soup_item scene_surface_mesh_item scene_points_with_normal_item ${polyfit_linked_libraries})
-  target_compile_definitions(surface_reconstruction_plugin PRIVATE ${polyfit_compile_definitions})
+  target_link_libraries(surface_reconstruction_plugin PUBLIC scene_polygon_soup_item scene_surface_mesh_item scene_points_with_normal_item CGAL::Eigen_support)
+
+  if (TARGET CGAL::SCIP_support)
+    target_link_libraries(surface_reconstruction_plugin PUBLIC CGAL::SCIP_support)
+  elseif(TARGET CGAL::GLPK_support)
+    target_link_libraries(surface_reconstruction_plugin PUBLIC CGAL::GLPK_support)
+  endif()
 
   qt5_wrap_ui( point_set_normal_estimationUI_FILES Point_set_normal_estimation_plugin.ui)
   polyhedron_demo_plugin(point_set_normal_estimation_plugin Point_set_normal_estimation_plugin ${point_set_normal_estimationUI_FILES} KEYWORDS PointSetProcessing Classification)
-  target_link_libraries(point_set_normal_estimation_plugin PUBLIC scene_points_with_normal_item scene_callback_signaler)
+  target_link_libraries(point_set_normal_estimation_plugin PUBLIC scene_points_with_normal_item scene_callback_signaler CGAL::Eigen_support)
 
   qt5_wrap_ui( features_detection_pluginUI_FILES Features_detection_plugin.ui)
   polyhedron_demo_plugin(features_detection_plugin Features_detection_plugin ${features_detection_pluginUI_FILES} KEYWORDS PointSetProcessing)
-  target_link_libraries(features_detection_plugin PUBLIC scene_points_with_normal_item)
+  target_link_libraries(features_detection_plugin PUBLIC scene_points_with_normal_item CGAL::Eigen_support)
 
   polyhedron_demo_plugin(point_set_smoothing_plugin Point_set_smoothing_plugin KEYWORDS PointSetProcessing)
-  target_link_libraries(point_set_smoothing_plugin PUBLIC scene_points_with_normal_item scene_callback_signaler)
+  target_link_libraries(point_set_smoothing_plugin PUBLIC scene_points_with_normal_item scene_callback_signaler CGAL::Eigen_support)
 
   polyhedron_demo_plugin(point_set_average_spacing_plugin Point_set_average_spacing_plugin KEYWORDS PointSetProcessing Classification)
-  target_link_libraries(point_set_average_spacing_plugin PUBLIC scene_points_with_normal_item scene_callback_signaler)
+  target_link_libraries(point_set_average_spacing_plugin PUBLIC scene_points_with_normal_item scene_callback_signaler CGAL::Eigen_support)
 
   qt5_wrap_ui(point_set_shape_detectionUI_FILES Point_set_shape_detection_plugin.ui)
   polyhedron_demo_plugin(point_set_shape_detection_plugin Point_set_shape_detection_plugin ${point_set_shape_detectionUI_FILES} KEYWORDS PointSetProcessing Classification)
-  target_link_libraries(point_set_shape_detection_plugin PUBLIC scene_surface_mesh_item scene_points_with_normal_item scene_polygon_soup_item scene_callback_signaler)
+  target_link_libraries(point_set_shape_detection_plugin PUBLIC scene_surface_mesh_item scene_points_with_normal_item scene_polygon_soup_item scene_callback_signaler CGAL::Eigen_support)
+
+  find_package(OpenGR QUIET)
+  include(CGAL_OpenGR_support)
+  find_package(libpointmatcher QUIET)
+  include(CGAL_pointmatcher_support)
+
+  if (TARGET CGAL::OpenGR_support OR CGAL::pointmatcher_support)
+    qt5_wrap_ui(register_point_setsUI_FILES Register_point_sets_plugin.ui)
+    polyhedron_demo_plugin(register_point_sets_plugin Register_point_sets_plugin ${register_point_setsUI_FILES} KEYWORDS PointSetProcessing)
+    target_link_libraries(register_point_sets_plugin PUBLIC scene_points_with_normal_item)
+    target_link_libraries(register_point_sets_plugin PUBLIC CGAL::Eigen_support)
+    if (TARGET CGAL::OpenGR_support)
+      target_link_libraries(register_point_sets_plugin PUBLIC CGAL::OpenGR_support)
+    endif()
+    if (TARGET CGAL::pointmatcher_support)
+      target_link_libraries(register_point_sets_plugin PUBLIC CGAL::pointmatcher_support)
+    endif()
+  else()
+    message(STATUS "NOTICE: OpenGR and libpointmatcher were not found. Registration plugin will not be available.")
+  endif()
 
 else(EIGEN3_FOUND)
   message(STATUS "NOTICE: Eigen 3.1 (or greater) was not found. Surface reconstruction plugin will not be available.")
@@ -50,7 +66,6 @@ else(EIGEN3_FOUND)
   message(STATUS "NOTICE: Eigen 3.1 (or greater) was not found. Smoothing plugin will not be available.")
   message(STATUS "NOTICE: Eigen 3.1 (or greater) was not found. Average spacing plugin will not be available.")
   message(STATUS "NOTICE: Eigen 3.1 (or greater) was not found. Feature detection plugin will not be available.")
-
 endif()
 
   qt5_wrap_ui(point_set_bilateral_smoothingUI_FILES Point_set_bilateral_smoothing_plugin.ui)
@@ -77,6 +92,9 @@ endif()
   polyhedron_demo_plugin(point_set_wlop_plugin Point_set_wlop_plugin ${point_set_wlopFILES} KEYWORDS PointSetProcessing)
   target_link_libraries(point_set_wlop_plugin PUBLIC scene_points_with_normal_item scene_callback_signaler)
 
+  polyhedron_demo_plugin(point_set_clustering_plugin Point_set_clustering_plugin KEYWORDS PointSetProcessing)
+  target_link_libraries(point_set_clustering_plugin PUBLIC scene_points_with_normal_item scene_callback_signaler)
+
   polyhedron_demo_plugin(merge_point_sets_plugin Merge_point_sets_plugin KEYWORDS PointSetProcessing Classification)
   target_link_libraries(merge_point_sets_plugin PUBLIC scene_points_with_normal_item)
 
@@ -91,7 +109,7 @@ endif()
   polyhedron_demo_plugin(point_set_to_mesh_distance_plugin Point_set_to_mesh_distance_plugin ${distanceUI_FILES} KEYWORDS PointSetProcessing)
   target_link_libraries(point_set_to_mesh_distance_plugin PUBLIC scene_points_with_normal_item scene_surface_mesh_item scene_color_ramp)
 
-  if(TBB_FOUND)
+  if(TARGET CGAL::TBB_support)
     foreach(plugin
         surface_reconstruction_plugin
         point_set_normal_estimation_plugin
@@ -110,7 +128,7 @@ endif()
         alpha_shape_plugin
         point_set_to_mesh_distance_plugin)
       if(TARGET ${plugin})
-        CGAL_target_use_TBB(${plugin})
+        target_link_libraries(${plugin} PUBLIC CGAL::TBB_support)
       endif()
     endforeach()
   endif()
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Features_detection_plugin.ui 5.1-1/demo/Polyhedron/Plugins/Point_set/Features_detection_plugin.ui
--- 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Features_detection_plugin.ui	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Point_set/Features_detection_plugin.ui	2020-09-08 12:12:20.000000000 +0000
@@ -6,8 +6,8 @@
    <rect>
     <x>0</x>
     <y>0</y>
-    <width>254</width>
-    <height>153</height>
+    <width>271</width>
+    <height>141</height>
    </rect>
   </property>
   <property name="windowTitle">
@@ -15,7 +15,7 @@
   </property>
   <layout class="QVBoxLayout" name="verticalLayout">
    <item>
-    <layout class="QHBoxLayout" name="horizontalLayout" stretch="0,1">
+    <layout class="QHBoxLayout" name="horizontalLayout" stretch="1,0">
      <item>
       <widget class="QLabel" name="label">
        <property name="text">
@@ -24,22 +24,22 @@
       </widget>
      </item>
      <item>
-      <widget class="QDoubleSpinBox" name="m_inputOffsetRadius">
-       <property name="decimals">
-        <number>3</number>
+      <widget class="DoubleEdit" name="m_inputOffsetRadius">
+       <property name="sizePolicy">
+        <sizepolicy hsizetype="Minimum" vsizetype="Fixed">
+         <horstretch>0</horstretch>
+         <verstretch>0</verstretch>
+        </sizepolicy>
        </property>
-       <property name="singleStep">
-        <double>0.050000000000000</double>
-       </property>
-       <property name="value">
-        <double>0.100000000000000</double>
+       <property name="text">
+        <string>0.100</string>
        </property>
       </widget>
      </item>
     </layout>
    </item>
    <item>
-    <layout class="QHBoxLayout" name="horizontalLayout_3" stretch="0,1">
+    <layout class="QHBoxLayout" name="horizontalLayout_3" stretch="1,0">
      <item>
       <widget class="QLabel" name="label_2">
        <property name="text">
@@ -48,19 +48,22 @@
       </widget>
      </item>
      <item>
-      <widget class="QDoubleSpinBox" name="m_inputConvolveRadius">
-       <property name="decimals">
-        <number>3</number>
+      <widget class="DoubleEdit" name="m_inputConvolveRadius">
+       <property name="sizePolicy">
+        <sizepolicy hsizetype="Minimum" vsizetype="Fixed">
+         <horstretch>0</horstretch>
+         <verstretch>0</verstretch>
+        </sizepolicy>
        </property>
-       <property name="singleStep">
-        <double>0.050000000000000</double>
+       <property name="text">
+        <string>0.00</string>
        </property>
       </widget>
      </item>
     </layout>
    </item>
    <item>
-    <layout class="QHBoxLayout" name="horizontalLayout_2" stretch="0,1">
+    <layout class="QHBoxLayout" name="horizontalLayout_2" stretch="1,0">
      <item>
       <widget class="QLabel" name="label_3">
        <property name="text">
@@ -69,15 +72,15 @@
       </widget>
      </item>
      <item>
-      <widget class="QDoubleSpinBox" name="m_inputFeaturesThreshold">
-       <property name="decimals">
-        <number>3</number>
+      <widget class="DoubleEdit" name="m_inputFeaturesThreshold">
+       <property name="sizePolicy">
+        <sizepolicy hsizetype="Minimum" vsizetype="Fixed">
+         <horstretch>0</horstretch>
+         <verstretch>0</verstretch>
+        </sizepolicy>
        </property>
-       <property name="singleStep">
-        <double>0.020000000000000</double>
-       </property>
-       <property name="value">
-        <double>0.160000000000000</double>
+       <property name="text">
+        <string>0.160</string>
        </property>
       </widget>
      </item>
@@ -108,6 +111,13 @@
    </item>
   </layout>
  </widget>
+ <customwidgets>
+  <customwidget>
+   <class>DoubleEdit</class>
+   <extends>QLineEdit</extends>
+   <header>CGAL_double_edit.h</header>
+  </customwidget>
+ </customwidgets>
  <resources/>
  <connections>
   <connection>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_average_spacing_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_average_spacing_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_average_spacing_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_average_spacing_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -20,11 +20,7 @@
 #include "run_with_qprogressdialog.h"
 
 // Concurrency
-#ifdef CGAL_LINKED_WITH_TBB
-typedef CGAL::Parallel_tag Concurrency_tag;
-#else
-typedef CGAL::Sequential_tag Concurrency_tag;
-#endif
+typedef CGAL::Parallel_if_available_tag Concurrency_tag;
 
 struct Compute_average_spacing_functor
   : public Functor_with_signal_callback
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_bilateral_smoothing_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_bilateral_smoothing_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_bilateral_smoothing_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_bilateral_smoothing_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -21,11 +21,7 @@
 #include "ui_Point_set_bilateral_smoothing_plugin.h"
 
 // Concurrency
-#ifdef CGAL_LINKED_WITH_TBB
-typedef CGAL::Parallel_tag Concurrency_tag;
-#else
-typedef CGAL::Sequential_tag Concurrency_tag;
-#endif
+typedef CGAL::Parallel_if_available_tag Concurrency_tag;
 
 struct Bilateral_smoothing_functor
   : public Functor_with_signal_callback
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_clustering_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_clustering_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_clustering_plugin.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_clustering_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,241 @@
+#include "config.h"
+#include "Scene_points_with_normal_item.h"
+#include <CGAL/Three/Polyhedron_demo_plugin_helper.h>
+#include <CGAL/Three/Polyhedron_demo_plugin_interface.h>
+#include <CGAL/Three/Scene_group_item.h>
+
+#include <CGAL/cluster_point_set.h>
+#include <CGAL/Timer.h>
+#include <CGAL/Memory_sizer.h>
+#include <CGAL/Random.h>
+
+#include <QObject>
+#include <QAction>
+#include <QMainWindow>
+#include <QApplication>
+#include <QtPlugin>
+#include <QMessageBox>
+#include <QSpinBox>
+#include <QDoubleSpinBox>
+#include <QCheckBox>
+
+#include <QMultipleInputDialog.h>
+
+#include "run_with_qprogressdialog.h"
+
+struct Clustering_functor
+  : public Functor_with_signal_callback
+{
+  Point_set* points;
+  Point_set::Property_map<std::size_t> cluster_map;
+  const double neighbor_radius;
+  boost::shared_ptr<std::size_t> result;
+
+  Clustering_functor (Point_set* points,
+                      const double neighbor_radius,
+                      Point_set::Property_map<std::size_t> cluster_map)
+    : points (points), cluster_map (cluster_map),
+      neighbor_radius (neighbor_radius),
+      result (new std::size_t(0)) { }
+
+  void operator()()
+  {
+    *result = CGAL::cluster_point_set (*points, cluster_map,
+                                       points->parameters().neighbor_radius(neighbor_radius).
+                                       callback(*(this->callback())));
+  }
+};
+
+using namespace CGAL::Three;
+
+class Polyhedron_demo_point_set_clustering_plugin :
+  public QObject,
+  public Polyhedron_demo_plugin_helper
+{
+  Q_OBJECT
+  Q_INTERFACES(CGAL::Three::Polyhedron_demo_plugin_interface)
+  Q_PLUGIN_METADATA(IID "com.geometryfactory.PolyhedronDemo.PluginInterface/1.0")
+
+  QAction* actionCluster;
+
+public:
+  void init(QMainWindow* mainWindow, CGAL::Three::Scene_interface* scene_interface, Messages_interface*) {
+
+    scene = scene_interface;
+    mw = mainWindow;
+    actionCluster = new QAction(tr("Cluster Point Set"), mainWindow);
+    actionCluster->setObjectName("actionCluster");
+    actionCluster->setProperty("subMenuName","Point Set Processing");
+    autoConnectActions();
+  }
+
+  QList<QAction*> actions() const {
+    return QList<QAction*>() << actionCluster;
+  }
+
+  bool applicable(QAction*) const {
+    Scene_points_with_normal_item* item = qobject_cast<Scene_points_with_normal_item*>(scene->item(scene->mainSelectionIndex()));
+    return item;
+  }
+
+public Q_SLOTS:
+  void on_actionCluster_triggered();
+
+}; // end
+
+void Polyhedron_demo_point_set_clustering_plugin::on_actionCluster_triggered()
+{
+  const CGAL::Three::Scene_interface::Item_id index = scene->mainSelectionIndex();
+
+  Scene_points_with_normal_item* item =
+    qobject_cast<Scene_points_with_normal_item*>(scene->item(index));
+
+  if(item)
+  {
+    // Gets point set
+    Point_set* points = item->point_set();
+    if(points == NULL)
+        return;
+
+    QMultipleInputDialog dialog ("Clustering", mw);
+    QDoubleSpinBox* neighbor_radius = dialog.add<QDoubleSpinBox> ("Neighbor radius (0 = automatic):");
+    neighbor_radius->setRange (0, 10000000);
+    neighbor_radius->setValue (0);
+    QSpinBox* min_nb = dialog.add<QSpinBox> ("Minimum number of points per cluster:");
+    min_nb->setRange (1, 10000000);
+    min_nb->setValue (1);
+
+    QCheckBox* add_property = dialog.add<QCheckBox> ("Add a \"cluster\" property to the input item");
+    add_property->setChecked (true);
+
+    QCheckBox* gen_color = dialog.add<QCheckBox> ("Generate one colored point set");
+    gen_color->setChecked (true);
+
+    QCheckBox* gen_sub = dialog.add<QCheckBox> ("Generate N point subsets");
+    gen_sub->setChecked (false);
+
+    if (!dialog.exec())
+      return;
+
+    QApplication::setOverrideCursor(Qt::BusyCursor);
+    QApplication::processEvents();
+    CGAL::Real_timer task_timer; task_timer.start();
+
+    Point_set::Property_map<std::size_t> cluster_map;
+
+    if (add_property->isChecked())
+      cluster_map = points->add_property_map<std::size_t> ("cluster_map").first;
+    else
+      // Use long name to avoid overwriting potentially existing map
+      cluster_map = points->add_property_map<std::size_t> ("cluster_point_set_property_map").first;
+
+    // Default value
+    if (neighbor_radius->value() == 0)
+    {
+      neighbor_radius->setRange (-1, 10000000);
+      neighbor_radius->setValue(-1);
+    }
+
+    // Computes average spacing
+    Clustering_functor functor (points, neighbor_radius->value(), cluster_map);
+    run_with_qprogressdialog (functor, "Clustering...", mw);
+
+    std::size_t nb_clusters = *functor.result;
+
+    Scene_group_item* group = nullptr;
+    std::vector<Scene_points_with_normal_item*> new_items;
+
+    if (gen_sub->isChecked())
+    {
+      group = new Scene_group_item(QString("%1 (clusters)").arg(item->name()));
+      scene->addItem(group);
+      new_items.reserve (nb_clusters);
+      for (std::size_t i = 0; i < nb_clusters; ++ i)
+      {
+        Scene_points_with_normal_item* new_item = new Scene_points_with_normal_item;
+        new_item->point_set()->copy_properties (*points);
+        CGAL::Random rand((unsigned int)(i));
+        unsigned char r, g, b;
+        r = static_cast<unsigned char>(64 + rand.get_int(0, 192));
+        g = static_cast<unsigned char>(64 + rand.get_int(0, 192));
+        b = static_cast<unsigned char>(64 + rand.get_int(0, 192));
+        new_item->setRgbColor(r, g, b);
+        new_item->setName (QString("Cluster %1 of %2").arg(i).arg(item->name()));
+        new_items.push_back (new_item);
+      }
+    }
+
+    std::vector<std::size_t> cluster_size (nb_clusters, 0);
+    for (Point_set::Index idx : *points)
+    {
+      if (gen_sub->isChecked())
+        new_items[cluster_map[idx]]->point_set()->insert (*points, idx);
+      cluster_size[cluster_map[idx]] ++;
+    }
+
+    if (gen_color->isChecked())
+    {
+      Scene_points_with_normal_item* colored;
+      Point_set::Property_map<unsigned char> red, green, blue;
+
+      colored = new Scene_points_with_normal_item;
+      colored->setName (QString("%1 (clustering)").arg(item->name()));
+
+      red = colored->point_set()->add_property_map<unsigned char>("red", 0).first;
+      green = colored->point_set()->add_property_map<unsigned char>("green", 0).first;
+      blue = colored->point_set()->add_property_map<unsigned char>("blue", 0).first;
+      colored->point_set()->check_colors();
+
+      colored->point_set()->reserve (points->size());
+
+      for (Point_set::Index idx : *points)
+      {
+        Point_set::Index iidx = *(colored->point_set()->insert (points->point(idx)));
+        if (cluster_size[cluster_map[idx]] >= std::size_t(min_nb->value()))
+        {
+          CGAL::Random rand((unsigned int)(cluster_map[idx] + 1));
+          unsigned char r, g, b;
+          r = static_cast<unsigned char>(64 + rand.get_int(0, 192));
+          g = static_cast<unsigned char>(64 + rand.get_int(0, 192));
+          b = static_cast<unsigned char>(64 + rand.get_int(0, 192));
+          red[iidx] = r;
+          green[iidx] = g;
+          blue[iidx] = b;
+        }
+      }
+      scene->addItem(colored);
+    }
+
+    if (gen_sub->isChecked())
+    {
+      for (Scene_points_with_normal_item* new_item : new_items)
+      {
+        if (new_item->point_set()->size() >= std::size_t(min_nb->value()))
+        {
+          scene->addItem(new_item);
+          scene->changeGroup (new_item, group);
+        }
+        else
+          delete new_item;
+      }
+    }
+
+    if (!add_property->isChecked())
+      points->remove_property_map (cluster_map);
+
+    std::size_t memory = CGAL::Memory_sizer().virtual_size();
+    std::cerr << "Number of clusters = " << nb_clusters << " ("
+              << task_timer.time() << " seconds, "
+              << (memory>>20) << " Mb allocated)"
+              << std::endl;
+    QApplication::restoreOverrideCursor();
+
+    item->setVisible (false);
+    item->invalidateOpenGLBuffers();
+    scene->itemChanged(item);
+  }
+}
+
+
+
+#include "Point_set_clustering_plugin.moc"
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_interference_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_interference_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_interference_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_interference_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -16,11 +16,8 @@
 #include <QMainWindow>
 
 using namespace CGAL::Three;
-#ifdef CGAL_LINKED_WITH_TBB
-typedef CGAL::Parallel_tag Concurrency_tag;
-#else
-typedef CGAL::Sequential_tag Concurrency_tag;
-#endif
+
+typedef CGAL::Parallel_if_available_tag Concurrency_tag;
 
 class  Point_set_interference_plugin:
     public QObject,
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_normal_estimation_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_normal_estimation_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_normal_estimation_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_normal_estimation_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -25,11 +25,7 @@
 #include "ui_Point_set_normal_estimation_plugin.h"
 
 // Concurrency
-#ifdef CGAL_LINKED_WITH_TBB
-typedef CGAL::Parallel_tag Concurrency_tag;
-#else
-typedef CGAL::Sequential_tag Concurrency_tag;
-#endif
+typedef CGAL::Parallel_if_available_tag Concurrency_tag;
 
 struct PCA_estimate_normals_functor
   : public Functor_with_signal_callback
@@ -152,6 +148,7 @@ class Point_set_demo_normal_estimation_d
     Point_set_demo_normal_estimation_dialog(QWidget* /*parent*/ = 0)
     {
       setupUi(this);
+      m_offset_radius->setMinimum(0.01);
     }
 
   int pca_neighbors() const { return m_pca_neighbors->value(); }
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_normal_estimation_plugin.ui 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_normal_estimation_plugin.ui
--- 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_normal_estimation_plugin.ui	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_normal_estimation_plugin.ui	2020-09-08 12:12:20.000000000 +0000
@@ -6,7 +6,7 @@
    <rect>
     <x>0</x>
     <y>0</y>
-    <width>301</width>
+    <width>330</width>
     <height>372</height>
    </rect>
   </property>
@@ -139,22 +139,6 @@
         </property>
        </widget>
       </item>
-      <item row="0" column="1">
-       <widget class="QDoubleSpinBox" name="m_offset_radius">
-        <property name="minimum">
-         <double>0.010000000000000</double>
-        </property>
-        <property name="singleStep">
-         <double>0.010000000000000</double>
-        </property>
-        <property name="value">
-         <double>0.100000000000000</double>
-        </property>
-       </widget>
-      </item>
-      <item row="1" column="1">
-       <widget class="QDoubleSpinBox" name="m_convolution_radius"/>
-      </item>
       <item row="1" column="0">
        <widget class="QRadioButton" name="buttonRadius">
         <property name="text">
@@ -185,6 +169,20 @@
         </property>
        </widget>
       </item>
+      <item row="0" column="1">
+       <widget class="DoubleEdit" name="m_offset_radius">
+        <property name="text">
+         <string>0.10</string>
+        </property>
+       </widget>
+      </item>
+      <item row="1" column="1">
+       <widget class="DoubleEdit" name="m_convolution_radius">
+        <property name="text">
+         <string>0.00</string>
+        </property>
+       </widget>
+      </item>
      </layout>
     </widget>
    </item>
@@ -213,6 +211,13 @@
    </item>
   </layout>
  </widget>
+ <customwidgets>
+  <customwidget>
+   <class>DoubleEdit</class>
+   <extends>QLineEdit</extends>
+   <header>CGAL_double_edit.h</header>
+  </customwidget>
+ </customwidgets>
  <resources/>
  <connections>
   <connection>
@@ -262,22 +267,6 @@
      <y>245</y>
     </hint>
    </hints>
-  </connection>
-  <connection>
-   <sender>buttonRadius</sender>
-   <signal>toggled(bool)</signal>
-   <receiver>m_convolution_radius</receiver>
-   <slot>setEnabled(bool)</slot>
-   <hints>
-    <hint type="sourcelabel">
-     <x>90</x>
-     <y>242</y>
-    </hint>
-    <hint type="destinationlabel">
-     <x>285</x>
-     <y>245</y>
-    </hint>
-   </hints>
   </connection>
   <connection>
    <sender>buttonNeighbors</sender>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_outliers_removal_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_outliers_removal_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_outliers_removal_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_outliers_removal_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -40,12 +40,13 @@ struct Outlier_removal_functor
   {
     // Computes outliers
     *result =
-      CGAL::remove_outliers(*points,
-                            nb_neighbors,
-                            points->parameters().
-                            threshold_percent(removed_percentage).
-                            threshold_distance(distance_threshold).
-                            callback (*(this->callback())));
+      CGAL::remove_outliers<CGAL::Parallel_if_available_tag>
+      (*points,
+       nb_neighbors,
+       points->parameters().
+       threshold_percent(removed_percentage).
+       threshold_distance(distance_threshold).
+       callback (*(this->callback())));
   }
 };
 
@@ -92,6 +93,7 @@ class Point_set_demo_outlier_removal_dia
     Point_set_demo_outlier_removal_dialog(QWidget * /*parent*/ = 0)
     {
       setupUi(this);
+      m_distanceThreshold->setMinimum(0.0);
     }
 
     double percentage() const { return m_inputPercentage->value(); }
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_outliers_removal_plugin.ui 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_outliers_removal_plugin.ui
--- 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_outliers_removal_plugin.ui	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_outliers_removal_plugin.ui	2020-09-08 12:12:20.000000000 +0000
@@ -91,26 +91,21 @@
     </widget>
    </item>
    <item row="0" column="1">
-    <widget class="QDoubleSpinBox" name="m_distanceThreshold">
-     <property name="decimals">
-      <number>6</number>
-     </property>
-     <property name="minimum">
-      <double>0.000000000000000</double>
-     </property>
-     <property name="maximum">
-      <double>100000.000000000000000</double>
-     </property>
-     <property name="singleStep">
-      <double>0.100000000000000</double>
-     </property>
-     <property name="value">
-      <double>0.500000000000000</double>
+    <widget class="DoubleEdit" name="m_distanceThreshold">
+     <property name="text">
+      <string>0.5</string>
      </property>
     </widget>
    </item>
   </layout>
  </widget>
+ <customwidgets>
+  <customwidget>
+   <class>DoubleEdit</class>
+   <extends>QLineEdit</extends>
+   <header>CGAL_double_edit.h</header>
+  </customwidget>
+ </customwidgets>
  <resources/>
  <connections>
   <connection>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_selection_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_selection_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_selection_plugin.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_selection_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -30,9 +30,10 @@
 #include <QKeyEvent>
 #include <QMouseEvent>
 #include <QMessageBox>
+#include <QSpinBox>
 
 #include <QMultipleInputDialog.h>
-#include <QDoubleSpinBox>
+#include "CGAL_double_edit.h"
 
 #include <map>
 #include <fstream>
@@ -707,7 +708,7 @@ protected:
       {
         QApplication::setOverrideCursor(Qt::WaitCursor);
         QWheelEvent *mouseEvent = static_cast<QWheelEvent*>(event);
-        int steps = mouseEvent->delta() / 120;
+        int steps = mouseEvent->angleDelta().y() / 120;
         if (steps > 0)
           neighborhood.point_set (point_set_item).expand();
         else
@@ -821,19 +822,15 @@ protected Q_SLOTS:
       return;
 
     QMultipleInputDialog dialog ("Region Selection Parameters", mw);
-    QDoubleSpinBox* epsilon = dialog.add<QDoubleSpinBox> ("Epsilon: ");
-    epsilon->setRange (0.00001, 1000000.);
-    epsilon->setDecimals (5);
+    DoubleEdit* epsilon = dialog.add<DoubleEdit> ("Epsilon: ");
     if (rg_epsilon < 0.)
       rg_epsilon = (std::max)(0.00001, 0.005 * scene->len_diagonal());
-    epsilon->setValue (rg_epsilon);
+    epsilon->setValue(rg_epsilon);
 
-    QDoubleSpinBox* cluster_epsilon = dialog.add<QDoubleSpinBox> ("Cluster epsilon: ");
-    cluster_epsilon->setRange (0.00001, 1000000.);
-    cluster_epsilon->setDecimals (5);
+    DoubleEdit* cluster_epsilon = dialog.add<DoubleEdit> ("Cluster epsilon: ");
     if (rg_cluster_epsilon < 0.)
       rg_cluster_epsilon = (std::max)(0.00001, 0.03 * scene->len_diagonal());
-    cluster_epsilon->setValue (rg_cluster_epsilon);
+    cluster_epsilon->setText(tr("%1").arg(rg_cluster_epsilon));
 
     QSpinBox* normal_threshold = dialog.add<QSpinBox> ("Normal threshold: ");
     normal_threshold->setRange (0, 90);
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_shape_detection_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_shape_detection_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_shape_detection_plugin.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_shape_detection_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -79,6 +79,11 @@ public:
   Point_set_demo_point_set_shape_detection_dialog(QWidget * /*parent*/ = 0)
   {
     setupUi(this);
+    m_normal_tolerance_field->setMaximum(1.0);
+    m_probability_field->setRange(0.00001, 1.0);
+    m_epsilon_field->setMinimum(0.000001);
+    m_normal_tolerance_field->setMinimum(0.01);
+    m_cluster_epsilon_field->setMinimum(0.000001);
   }
 
   bool region_growing() const { return m_region_growing->isChecked(); }
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_shape_detection_plugin.ui 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_shape_detection_plugin.ui
--- 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_shape_detection_plugin.ui	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_shape_detection_plugin.ui	2020-09-08 12:12:20.000000000 +0000
@@ -124,7 +124,7 @@
      </property>
      <layout class="QVBoxLayout" name="verticalLayout_3">
       <item>
-       <layout class="QHBoxLayout" name="horizontalLayout_2" stretch="0,1">
+       <layout class="QHBoxLayout" name="horizontalLayout_2" stretch="0,0">
         <item>
          <widget class="QLabel" name="label_2">
           <property name="text">
@@ -133,31 +133,22 @@
          </widget>
         </item>
         <item>
-         <widget class="QDoubleSpinBox" name="m_epsilon_field">
-          <property name="toolTip">
-           <string>Fitting tolerance in Euclidean distance</string>
-          </property>
-          <property name="decimals">
-           <number>8</number>
-          </property>
-          <property name="minimum">
-           <double>0.000001000000000</double>
-          </property>
-          <property name="maximum">
-           <double>1000000.000000000000000</double>
-          </property>
-          <property name="singleStep">
-           <double>0.001000000000000</double>
+         <widget class="DoubleEdit" name="m_epsilon_field">
+          <property name="sizePolicy">
+           <sizepolicy hsizetype="Minimum" vsizetype="Fixed">
+            <horstretch>0</horstretch>
+            <verstretch>0</verstretch>
+           </sizepolicy>
           </property>
-          <property name="value">
-           <double>0.002000000000000</double>
+          <property name="text">
+           <string>0.002</string>
           </property>
          </widget>
         </item>
        </layout>
       </item>
       <item>
-       <layout class="QHBoxLayout" name="horizontalLayout" stretch="0,1">
+       <layout class="QHBoxLayout" name="horizontalLayout" stretch="0,0">
         <item>
          <widget class="QLabel" name="label">
           <property name="text">
@@ -166,28 +157,22 @@
          </widget>
         </item>
         <item>
-         <widget class="QDoubleSpinBox" name="m_normal_tolerance_field">
-          <property name="toolTip">
-           <string>Normal angle deviation tolerance as cosine of the angle</string>
-          </property>
-          <property name="minimum">
-           <double>0.010000000000000</double>
-          </property>
-          <property name="maximum">
-           <double>1.000000000000000</double>
+         <widget class="DoubleEdit" name="m_normal_tolerance_field">
+          <property name="sizePolicy">
+           <sizepolicy hsizetype="Minimum" vsizetype="Fixed">
+            <horstretch>0</horstretch>
+            <verstretch>0</verstretch>
+           </sizepolicy>
           </property>
-          <property name="singleStep">
-           <double>0.010000000000000</double>
-          </property>
-          <property name="value">
-           <double>0.900000000000000</double>
+          <property name="text">
+           <string>0.90</string>
           </property>
          </widget>
         </item>
        </layout>
       </item>
       <item>
-       <layout class="QHBoxLayout" name="horizontalLayout_3" stretch="0,1">
+       <layout class="QHBoxLayout" name="horizontalLayout_3" stretch="0,0">
         <item>
          <widget class="QLabel" name="label_3">
           <property name="text">
@@ -214,7 +199,7 @@
        </layout>
       </item>
       <item>
-       <layout class="QHBoxLayout" name="horizontalLayout_4" stretch="0,1">
+       <layout class="QHBoxLayout" name="horizontalLayout_4" stretch="0,0">
         <item>
          <widget class="QLabel" name="label_4">
           <property name="text">
@@ -223,31 +208,22 @@
          </widget>
         </item>
         <item>
-         <widget class="QDoubleSpinBox" name="m_cluster_epsilon_field">
-          <property name="toolTip">
-           <string>Maximum world distance between points on a shape to be considered as connected</string>
-          </property>
-          <property name="decimals">
-           <number>8</number>
-          </property>
-          <property name="minimum">
-           <double>0.000001000000000</double>
+         <widget class="DoubleEdit" name="m_cluster_epsilon_field">
+          <property name="sizePolicy">
+           <sizepolicy hsizetype="Minimum" vsizetype="Fixed">
+            <horstretch>0</horstretch>
+            <verstretch>0</verstretch>
+           </sizepolicy>
           </property>
-          <property name="maximum">
-           <double>1000000.000000000000000</double>
-          </property>
-          <property name="singleStep">
-           <double>0.010000000000000</double>
-          </property>
-          <property name="value">
-           <double>0.020000000000000</double>
+          <property name="text">
+           <string>0.002</string>
           </property>
          </widget>
         </item>
        </layout>
       </item>
       <item>
-       <layout class="QHBoxLayout" name="horizontalLayout_5" stretch="0,1">
+       <layout class="QHBoxLayout" name="horizontalLayout_5" stretch="0,0">
         <item>
          <widget class="QLabel" name="label_5">
           <property name="enabled">
@@ -259,27 +235,15 @@
          </widget>
         </item>
         <item>
-         <widget class="QDoubleSpinBox" name="m_probability_field">
-          <property name="enabled">
-           <bool>false</bool>
-          </property>
-          <property name="toolTip">
-           <string>Probability to overlook the largest primitive in one extraction iteration</string>
+         <widget class="DoubleEdit" name="m_probability_field">
+          <property name="sizePolicy">
+           <sizepolicy hsizetype="Minimum" vsizetype="Fixed">
+            <horstretch>0</horstretch>
+            <verstretch>0</verstretch>
+           </sizepolicy>
           </property>
-          <property name="decimals">
-           <number>5</number>
-          </property>
-          <property name="minimum">
-           <double>0.000010000000000</double>
-          </property>
-          <property name="maximum">
-           <double>1.000000000000000</double>
-          </property>
-          <property name="singleStep">
-           <double>0.010000000000000</double>
-          </property>
-          <property name="value">
-           <double>0.050000000000000</double>
+          <property name="text">
+           <string>0.05</string>
           </property>
          </widget>
         </item>
@@ -356,6 +320,13 @@
    </item>
   </layout>
  </widget>
+ <customwidgets>
+  <customwidget>
+   <class>DoubleEdit</class>
+   <extends>QLineEdit</extends>
+   <header>CGAL_double_edit.h</header>
+  </customwidget>
+ </customwidgets>
  <resources/>
  <connections>
   <connection>
@@ -485,22 +456,6 @@
      <y>214</y>
     </hint>
    </hints>
-  </connection>
-  <connection>
-   <sender>ransac</sender>
-   <signal>toggled(bool)</signal>
-   <receiver>m_probability_field</receiver>
-   <slot>setEnabled(bool)</slot>
-   <hints>
-    <hint type="sourcelabel">
-     <x>70</x>
-     <y>46</y>
-    </hint>
-    <hint type="destinationlabel">
-     <x>352</x>
-     <y>214</y>
-    </hint>
-   </hints>
   </connection>
  </connections>
 </ui>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_simplification_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_simplification_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_simplification_plugin.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_simplification_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -22,11 +22,7 @@
 #include "ui_Point_set_simplification_plugin.h"
 
 // Concurrency
-#ifdef CGAL_LINKED_WITH_TBB
-typedef CGAL::Parallel_tag Concurrency_tag;
-#else
-typedef CGAL::Sequential_tag Concurrency_tag;
-#endif
+typedef CGAL::Parallel_if_available_tag Concurrency_tag;
 
 struct Compute_average_spacing_functor
   : public Functor_with_signal_callback
@@ -134,6 +130,7 @@ class Point_set_demo_point_set_simplific
     Point_set_demo_point_set_simplification_dialog(QWidget * /*parent*/ = 0)
     {
       setupUi(this);
+      m_maximumSurfaceVariation->setRange(0.000010, 0.33330);
     }
 
   unsigned int simplificationMethod() const
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_simplification_plugin.ui 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_simplification_plugin.ui
--- 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_simplification_plugin.ui	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_simplification_plugin.ui	2020-09-08 12:12:20.000000000 +0000
@@ -7,24 +7,20 @@
     <x>0</x>
     <y>0</y>
     <width>392</width>
-    <height>248</height>
+    <height>249</height>
    </rect>
   </property>
   <property name="windowTitle">
    <string>Simplification</string>
   </property>
   <layout class="QGridLayout" columnstretch="0,1" columnminimumwidth="0,1">
-   <item row="7" column="0">
-    <widget class="QLabel" name="label_4">
+   <item row="2" column="0">
+    <widget class="QRadioButton" name="Random">
      <property name="text">
-      <string>Maximum Cluster Size</string>
+      <string>Random</string>
      </property>
-    </widget>
-   </item>
-   <item row="4" column="0">
-    <widget class="QRadioButton" name="Grid">
-     <property name="text">
-      <string>Grid</string>
+     <property name="checked">
+      <bool>true</bool>
      </property>
     </widget>
    </item>
@@ -44,37 +40,6 @@
      </property>
     </widget>
    </item>
-   <item row="8" column="0">
-    <widget class="QLabel" name="label_5">
-     <property name="text">
-      <string>Maximum Surface Variation</string>
-     </property>
-    </widget>
-   </item>
-   <item row="2" column="0">
-    <widget class="QRadioButton" name="Random">
-     <property name="text">
-      <string>Random</string>
-     </property>
-     <property name="checked">
-      <bool>true</bool>
-     </property>
-    </widget>
-   </item>
-   <item row="6" column="0">
-    <widget class="QRadioButton" name="Hierarchy">
-     <property name="text">
-      <string>Hierarchy</string>
-     </property>
-    </widget>
-   </item>
-   <item row="3" column="0">
-    <widget class="QLabel" name="label_2">
-     <property name="text">
-      <string>Points to Remove Randomly</string>
-     </property>
-    </widget>
-   </item>
    <item row="5" column="0">
     <widget class="QLabel" name="label_3">
      <property name="text">
@@ -82,16 +47,6 @@
      </property>
     </widget>
    </item>
-   <item row="10" column="0" colspan="2">
-    <widget class="QDialogButtonBox" name="buttonBox">
-     <property name="orientation">
-      <enum>Qt::Horizontal</enum>
-     </property>
-     <property name="standardButtons">
-      <set>QDialogButtonBox::Cancel|QDialogButtonBox::Ok</set>
-     </property>
-    </widget>
-   </item>
    <item row="3" column="1">
     <widget class="QDoubleSpinBox" name="m_randomSimplificationPercentage">
      <property name="suffix">
@@ -114,28 +69,10 @@
      </property>
     </widget>
    </item>
-   <item row="8" column="1">
-    <widget class="QDoubleSpinBox" name="m_maximumSurfaceVariation">
-     <property name="enabled">
-      <bool>false</bool>
-     </property>
-     <property name="suffix">
-      <string/>
-     </property>
-     <property name="decimals">
-      <number>5</number>
-     </property>
-     <property name="minimum">
-      <double>0.000010000000000</double>
-     </property>
-     <property name="maximum">
-      <double>0.333330000000000</double>
-     </property>
-     <property name="singleStep">
-      <double>0.012340000000000</double>
-     </property>
-     <property name="value">
-      <double>0.333330000000000</double>
+   <item row="8" column="0">
+    <widget class="QLabel" name="label_5">
+     <property name="text">
+      <string>Maximum Surface Variation</string>
      </property>
     </widget>
    </item>
@@ -177,8 +114,60 @@
      </property>
     </spacer>
    </item>
+   <item row="7" column="0">
+    <widget class="QLabel" name="label_4">
+     <property name="text">
+      <string>Maximum Cluster Size</string>
+     </property>
+    </widget>
+   </item>
+   <item row="3" column="0">
+    <widget class="QLabel" name="label_2">
+     <property name="text">
+      <string>Points to Remove Randomly</string>
+     </property>
+    </widget>
+   </item>
+   <item row="4" column="0">
+    <widget class="QRadioButton" name="Grid">
+     <property name="text">
+      <string>Grid</string>
+     </property>
+    </widget>
+   </item>
+   <item row="6" column="0">
+    <widget class="QRadioButton" name="Hierarchy">
+     <property name="text">
+      <string>Hierarchy</string>
+     </property>
+    </widget>
+   </item>
+   <item row="10" column="0" colspan="2">
+    <widget class="QDialogButtonBox" name="buttonBox">
+     <property name="orientation">
+      <enum>Qt::Horizontal</enum>
+     </property>
+     <property name="standardButtons">
+      <set>QDialogButtonBox::Cancel|QDialogButtonBox::Ok</set>
+     </property>
+    </widget>
+   </item>
+   <item row="8" column="1">
+    <widget class="DoubleEdit" name="m_maximumSurfaceVariation">
+     <property name="text">
+      <string>0.333333</string>
+     </property>
+    </widget>
+   </item>
   </layout>
  </widget>
+ <customwidgets>
+  <customwidget>
+   <class>DoubleEdit</class>
+   <extends>QLineEdit</extends>
+   <header>CGAL_double_edit.h</header>
+  </customwidget>
+ </customwidgets>
  <resources/>
  <connections>
   <connection>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_smoothing_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_smoothing_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_smoothing_plugin.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_smoothing_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -15,11 +15,7 @@
 #include "run_with_qprogressdialog.h"
 
 // Concurrency
-#ifdef CGAL_LINKED_WITH_TBB
-typedef CGAL::Parallel_tag Concurrency_tag;
-#else
-typedef CGAL::Sequential_tag Concurrency_tag;
-#endif
+typedef CGAL::Parallel_if_available_tag Concurrency_tag;
 
 struct Jet_smoothing_functor
   : public Functor_with_signal_callback
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_to_mesh_distance_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_to_mesh_distance_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_to_mesh_distance_plugin.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_to_mesh_distance_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -77,7 +77,6 @@ double compute_distances(const Mesh& m,
   typedef CGAL::AABB_tree< Traits > Tree;
 
   Tree tree( faces(m).first, faces(m).second, m);
-  tree.accelerate_distance_queries();
   tree.build();
   typedef typename boost::property_map<Mesh, boost::vertex_point_t>::const_type VPMap;
   VPMap vpmap = get(boost::vertex_point, m);
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_to_mesh_distance_widget.ui 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_to_mesh_distance_widget.ui
--- 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_to_mesh_distance_widget.ui	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_to_mesh_distance_widget.ui	2020-09-08 12:12:20.000000000 +0000
@@ -140,16 +140,6 @@
         <layout class="QHBoxLayout" name="horizontalLayout_2">
          <item>
           <layout class="QGridLayout" name="gridLayout_2" columnstretch="1,0">
-           <item row="0" column="1">
-            <widget class="QDoubleSpinBox" name="distance_spinbox">
-             <property name="toolTip">
-              <string>Points further than this distance will be selected.</string>
-             </property>
-             <property name="decimals">
-              <number>6</number>
-             </property>
-            </widget>
-           </item>
            <item row="1" column="1">
             <widget class="QPushButton" name="select_button">
              <property name="toolTip">
@@ -167,6 +157,13 @@
              </property>
             </widget>
            </item>
+           <item row="0" column="1">
+            <widget class="DoubleEdit" name="distance_spinbox">
+             <property name="text">
+              <string>0.00</string>
+             </property>
+            </widget>
+           </item>
           </layout>
          </item>
         </layout>
@@ -190,6 +187,13 @@
    </layout>
   </widget>
  </widget>
+ <customwidgets>
+  <customwidget>
+   <class>DoubleEdit</class>
+   <extends>QLineEdit</extends>
+   <header>CGAL_double_edit.h</header>
+  </customwidget>
+ </customwidgets>
  <resources/>
  <connections/>
 </ui>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_upsampling_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_upsampling_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_upsampling_plugin.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_upsampling_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -19,11 +19,7 @@
 
 #include "ui_Point_set_upsampling_plugin.h"
 // Concurrency
-#ifdef CGAL_LINKED_WITH_TBB
-typedef CGAL::Parallel_tag Concurrency_tag;
-#else
-typedef CGAL::Sequential_tag Concurrency_tag;
-#endif
+typedef CGAL::Parallel_if_available_tag Concurrency_tag;
 
 using namespace CGAL::Three;
 class Polyhedron_demo_point_set_upsampling_plugin :
@@ -67,6 +63,10 @@ public:
   Point_set_demo_point_set_upsampling_dialog(QWidget * /*parent*/ = 0)
   {
     setupUi(this);
+    m_edgeSensitivity->setMaximum(1.0);
+    m_neighborhoodRadius->setRange(0.1, 10.0);
+
+
   }
 
   unsigned int sharpness_angle () const { return m_sharpnessAngle->value(); }
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_upsampling_plugin.ui 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_upsampling_plugin.ui
--- 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_upsampling_plugin.ui	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_upsampling_plugin.ui	2020-09-08 12:12:20.000000000 +0000
@@ -7,13 +7,36 @@
     <x>0</x>
     <y>0</y>
     <width>348</width>
-    <height>167</height>
+    <height>168</height>
    </rect>
   </property>
   <property name="windowTitle">
    <string>Edge Aware Upsampling</string>
   </property>
   <layout class="QGridLayout" columnstretch="0,1">
+   <item row="6" column="0" colspan="2">
+    <widget class="QDialogButtonBox" name="buttonBox">
+     <property name="orientation">
+      <enum>Qt::Horizontal</enum>
+     </property>
+     <property name="standardButtons">
+      <set>QDialogButtonBox::Cancel|QDialogButtonBox::Ok</set>
+     </property>
+    </widget>
+   </item>
+   <item row="5" column="0">
+    <spacer name="verticalSpacer">
+     <property name="orientation">
+      <enum>Qt::Vertical</enum>
+     </property>
+     <property name="sizeHint" stdset="0">
+      <size>
+       <width>20</width>
+       <height>40</height>
+      </size>
+     </property>
+    </spacer>
+   </item>
    <item row="4" column="0">
     <widget class="QLabel" name="label_2">
      <property name="text">
@@ -21,45 +44,30 @@
      </property>
     </widget>
    </item>
-   <item row="3" column="1">
-    <widget class="QDoubleSpinBox" name="m_neighborhoodRadius">
-     <property name="suffix">
-      <string> * average spacing</string>
-     </property>
-     <property name="decimals">
-      <number>2</number>
-     </property>
-     <property name="minimum">
-      <double>0.100000000000000</double>
-     </property>
-     <property name="maximum">
-      <double>10.000000000000000</double>
-     </property>
-     <property name="singleStep">
-      <double>0.100000000000000</double>
-     </property>
-     <property name="value">
-      <double>3.000000000000000</double>
+   <item row="3" column="0">
+    <widget class="QLabel" name="label">
+     <property name="text">
+      <string>Neighborhood Radius</string>
      </property>
     </widget>
    </item>
-   <item row="6" column="0" colspan="2">
-    <widget class="QDialogButtonBox" name="buttonBox">
-     <property name="orientation">
-      <enum>Qt::Horizontal</enum>
-     </property>
-     <property name="standardButtons">
-      <set>QDialogButtonBox::Cancel|QDialogButtonBox::Ok</set>
+   <item row="2" column="0">
+    <widget class="QLabel" name="label_5">
+     <property name="text">
+      <string>Edge Sensitivity</string>
      </property>
     </widget>
    </item>
-   <item row="2" column="1">
-    <widget class="QDoubleSpinBox" name="m_edgeSensitivity">
+   <item row="0" column="1">
+    <widget class="QSpinBox" name="m_sharpnessAngle">
+     <property name="suffix">
+      <string>°</string>
+     </property>
      <property name="maximum">
-      <double>1.000000000000000</double>
+      <number>90</number>
      </property>
-     <property name="singleStep">
-      <double>0.010000000000000</double>
+     <property name="value">
+      <number>25</number>
      </property>
     </widget>
    </item>
@@ -82,13 +90,6 @@
      </property>
     </widget>
    </item>
-   <item row="2" column="0">
-    <widget class="QLabel" name="label_5">
-     <property name="text">
-      <string>Edge Sensitivity</string>
-     </property>
-    </widget>
-   </item>
    <item row="0" column="0">
     <widget class="QLabel" name="label_4">
      <property name="text">
@@ -96,41 +97,44 @@
      </property>
     </widget>
    </item>
-   <item row="3" column="0">
-    <widget class="QLabel" name="label">
-     <property name="text">
-      <string>Neighborhood Radius</string>
-     </property>
-    </widget>
-   </item>
-   <item row="0" column="1">
-    <widget class="QSpinBox" name="m_sharpnessAngle">
+   <item row="3" column="1">
+    <widget class="QDoubleSpinBox" name="m_neighborhoodRadius">
      <property name="suffix">
-      <string>°</string>
+      <string> * average spacing</string>
+     </property>
+     <property name="decimals">
+      <number>2</number>
+     </property>
+     <property name="minimum">
+      <double>0.100000000000000</double>
      </property>
      <property name="maximum">
-      <number>90</number>
+      <double>10.000000000000000</double>
+     </property>
+     <property name="singleStep">
+      <double>0.100000000000000</double>
      </property>
      <property name="value">
-      <number>25</number>
+      <double>3.000000000000000</double>
      </property>
     </widget>
    </item>
-   <item row="5" column="0">
-    <spacer name="verticalSpacer">
-     <property name="orientation">
-      <enum>Qt::Vertical</enum>
-     </property>
-     <property name="sizeHint" stdset="0">
-      <size>
-       <width>20</width>
-       <height>40</height>
-      </size>
+   <item row="2" column="1">
+    <widget class="DoubleEdit" name="m_edgeSensitivity">
+     <property name="text">
+      <string>0.00</string>
      </property>
-    </spacer>
+    </widget>
    </item>
   </layout>
  </widget>
+ <customwidgets>
+  <customwidget>
+   <class>DoubleEdit</class>
+   <extends>QLineEdit</extends>
+   <header>CGAL_double_edit.h</header>
+  </customwidget>
+ </customwidgets>
  <resources/>
  <connections>
   <connection>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_wlop_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_wlop_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Point_set_wlop_plugin.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Point_set/Point_set_wlop_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -20,11 +20,7 @@
 #include "ui_Point_set_wlop_plugin.h"
 
 // Concurrency
-#ifdef CGAL_LINKED_WITH_TBB
-typedef CGAL::Parallel_tag Concurrency_tag;
-#else
-typedef CGAL::Sequential_tag Concurrency_tag;
-#endif
+typedef CGAL::Parallel_if_available_tag Concurrency_tag;
 
 struct Compute_average_spacing_functor
   : public Functor_with_signal_callback
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Register_point_sets_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Point_set/Register_point_sets_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Register_point_sets_plugin.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Point_set/Register_point_sets_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,228 @@
+#include "config.h"
+#include "Scene_points_with_normal_item.h"
+#include <CGAL/Three/Polyhedron_demo_plugin_helper.h>
+#include <CGAL/Three/Polyhedron_demo_plugin_interface.h>
+
+#ifdef CGAL_LINKED_WITH_OPENGR
+#define CGAL_OPENGR_VERBOSE
+#include <CGAL/OpenGR/register_point_sets.h>
+#endif
+
+#ifdef CGAL_LINKED_WITH_POINTMATCHER
+#include <CGAL/pointmatcher/register_point_sets.h>
+#endif
+
+#include <sstream>
+
+#include <CGAL/Timer.h>
+#include <CGAL/Memory_sizer.h>
+
+#include <QObject>
+#include <QAction>
+#include <QMainWindow>
+#include <QApplication>
+#include <QtPlugin>
+#include <QInputDialog>
+#include <QMessageBox>
+#include <QRadioButton>
+#include <QLabel>
+#include <QSpinBox>
+#include <QDoubleSpinBox>
+
+#include "ui_Register_point_sets_plugin.h"
+
+
+using namespace CGAL::Three;
+
+
+class Point_set_demo_register_dialog : public QDialog, private Ui::RegisterPointSetsDialog
+{
+  Q_OBJECT
+public:
+  Point_set_demo_register_dialog(const std::vector<Scene_points_with_normal_item*>& items,
+                                 QWidget* /*parent*/ = 0)
+  {
+    setupUi(this);
+
+    for (std::size_t i = 0; i < items.size(); ++ i)
+    {
+      QRadioButton* button = new QRadioButton(items[i]->name().toStdString().c_str(), this);
+      buttons.push_back (button);
+      pointSets->addRow (button);
+      if (i == 0)
+        button->setChecked(true);
+    }
+
+#ifndef CGAL_LINKED_WITH_OPENGR
+    coarseRegistration->setText("Coarse registration (disabled, requires OpenGR)");
+    coarseRegistration->setChecked(false);
+    coarseRegistration->setEnabled(false);
+    opengr_frame->setEnabled(false);
+#endif
+#ifndef CGAL_LINKED_WITH_POINTMATCHER
+    fineRegistration->setText("Fine registration (disabled, requires libpointmatcher)");
+    fineRegistration->setChecked(false);
+    fineRegistration->setEnabled(false);
+    pointmatcher_frame->setEnabled(false);
+#endif
+  }
+
+  std::size_t ref_index() const
+  {
+    for (std::size_t i = 0; i < buttons.size(); ++ i)
+      if (buttons[i]->isChecked())
+        return i;
+    return std::size_t(-1);
+  }
+
+  bool coarse_registration() const { return coarseRegistration->isChecked(); }
+  bool fine_registration() const { return fineRegistration->isChecked(); }
+
+  int nb_samples() const { return numberOfSamplesSpinBox->value(); }
+  double accuracy() const { return accuracyDoubleSpinBox->value(); }
+  double overlap() const { return double(overlapSpinBox->value()) / 100.0; }
+  int max_time() const { return maximumRunningTimeSpinBox->value(); }
+
+  std::string pointmatcher_config() const { return config->toPlainText().toStdString(); }
+
+private:
+
+  std::vector<QRadioButton*> buttons;
+
+};
+
+
+
+class Polyhedron_demo_register_point_sets_plugin :
+  public QObject,
+  public Polyhedron_demo_plugin_helper
+{
+  Q_OBJECT
+  Q_INTERFACES(CGAL::Three::Polyhedron_demo_plugin_interface)
+  Q_PLUGIN_METADATA(IID "com.geometryfactory.PolyhedronDemo.PluginInterface/1.0")
+
+private:
+  QAction* actionRegisterPointSets;
+
+public:
+  void init(QMainWindow* mainWindow, CGAL::Three::Scene_interface* scene_interface, Messages_interface*) {
+    scene = scene_interface;
+    mw = mainWindow;
+    actionRegisterPointSets = new QAction(tr("Register point sets"), mainWindow);
+    actionRegisterPointSets->setObjectName("actionRegisterPointSets");
+    connect(actionRegisterPointSets, SIGNAL(triggered()), this, SLOT(on_actionRegisterPointSets_triggered()));
+  }
+
+  QList<QAction*> actions() const {
+    return QList<QAction*>() << actionRegisterPointSets;
+  }
+
+  bool applicable(QAction*) const {
+    return get_point_set_items().size() >= 2;
+  }
+
+  std::vector<Scene_points_with_normal_item*> get_point_set_items() const
+  {
+    std::vector<Scene_points_with_normal_item*> items;
+
+    Q_FOREACH(int index, scene->selectionIndices())
+    {
+      Scene_points_with_normal_item* item =
+        qobject_cast<Scene_points_with_normal_item*>(scene->item(index));
+      if(item && item->point_set()->has_normal_map())
+        items.push_back (item);
+    }
+
+    return items;
+  }
+
+public Q_SLOTS:
+  void on_actionRegisterPointSets_triggered();
+private :
+  Scene_interface *scene;
+}; // end Polyhedron_demo_register_point_sets_plugin
+
+void Polyhedron_demo_register_point_sets_plugin::on_actionRegisterPointSets_triggered()
+{
+  std::vector<Scene_points_with_normal_item*> items = get_point_set_items();
+
+  Point_set_demo_register_dialog dialog(items);
+  if(!dialog.exec())
+    return;
+
+  QApplication::setOverrideCursor(Qt::WaitCursor);
+
+  std::size_t ref = dialog.ref_index();
+
+  for (std::size_t i = 0; i < items.size(); ++ i)
+  {
+    if (i == ref)
+      continue;
+
+    std::cerr << "Registering " << items[i]->name().toStdString() << " with " << items[ref]->name().toStdString() << std::endl;
+
+    Point_set& ps1 = *(items[ref]->point_set());
+    Point_set& ps2 = *(items[i]->point_set());
+
+#ifdef CGAL_LINKED_WITH_OPENGR
+    if (dialog.coarse_registration())
+    {
+      std::cerr << "* Coarse registration:" << std::endl;
+      CGAL::Timer task_timer; task_timer.start();
+      double score =
+        CGAL::OpenGR::register_point_sets(ps1, ps2,
+                                          CGAL::parameters::point_map(ps1.point_map())
+                                          .normal_map(ps1.normal_map())
+                                          .number_of_samples(dialog.nb_samples())
+                                          .maximum_running_time(dialog.max_time())
+                                          .accuracy(dialog.accuracy())
+                                          .overlap(dialog.overlap()),
+                                          CGAL::parameters::point_map(ps2.point_map())
+                                          .normal_map(ps2.normal_map()));
+
+      std::size_t memory = CGAL::Memory_sizer().virtual_size();
+      std::cerr << " -> Registration score = " << score << " ("
+                << task_timer.time() << " seconds, "
+                << (memory>>20) << " Mb allocated)"
+                << std::endl;
+    }
+#endif
+#ifdef CGAL_LINKED_WITH_POINTMATCHER
+    if (dialog.fine_registration())
+    {
+      std::cerr << "* Fine registration: " << std::endl;
+
+      std::istringstream ss (dialog.pointmatcher_config());
+
+      CGAL::Timer task_timer; task_timer.start();
+      if (CGAL::pointmatcher::register_point_sets(ps1, ps2,
+                                            CGAL::parameters::point_map(ps1.point_map())
+                                            .normal_map(ps1.normal_map())
+                                            .pointmatcher_config(&ss),
+                                            CGAL::parameters::point_map(ps2.point_map())
+                                            .normal_map(ps2.normal_map())))
+        std::cerr << " -> Success";
+      else
+        std::cerr << " -> Failure";
+
+      std::size_t memory = CGAL::Memory_sizer().virtual_size();
+      std::cerr << " ("
+                << task_timer.time() << " seconds, "
+                << (memory>>20) << " Mb allocated)"
+                << std::endl;
+    }
+#endif
+
+  }
+
+  for (std::size_t i = 0; i < items.size(); ++ i)
+  {
+    items[i]->invalidateOpenGLBuffers();
+    scene->itemChanged(items[i]);
+  }
+
+  QApplication::restoreOverrideCursor();
+}
+
+
+#include "Register_point_sets_plugin.moc"
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Register_point_sets_plugin.ui 5.1-1/demo/Polyhedron/Plugins/Point_set/Register_point_sets_plugin.ui
--- 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Register_point_sets_plugin.ui	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Point_set/Register_point_sets_plugin.ui	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,320 @@
+<?xml version="1.0" encoding="UTF-8"?>
+<ui version="4.0">
+ <class>RegisterPointSetsDialog</class>
+ <widget class="QDialog" name="RegisterPointSetsDialog">
+  <property name="geometry">
+   <rect>
+    <x>0</x>
+    <y>0</y>
+    <width>418</width>
+    <height>515</height>
+   </rect>
+  </property>
+  <property name="windowTitle">
+   <string>Register point sets</string>
+  </property>
+  <layout class="QVBoxLayout" name="verticalLayout_2">
+   <item>
+    <widget class="QGroupBox" name="groupBox">
+     <property name="title">
+      <string>Which point set is the reference? (others will be altered)</string>
+     </property>
+     <layout class="QVBoxLayout" name="verticalLayout">
+      <item>
+       <layout class="QFormLayout" name="pointSets"/>
+      </item>
+     </layout>
+    </widget>
+   </item>
+   <item>
+    <widget class="QCheckBox" name="coarseRegistration">
+     <property name="text">
+      <string>Coarse registration using OpenGR</string>
+     </property>
+     <property name="checked">
+      <bool>true</bool>
+     </property>
+    </widget>
+   </item>
+   <item>
+    <widget class="QFrame" name="opengr_frame">
+     <property name="frameShape">
+      <enum>QFrame::StyledPanel</enum>
+     </property>
+     <property name="frameShadow">
+      <enum>QFrame::Raised</enum>
+     </property>
+     <layout class="QHBoxLayout" name="horizontalLayout">
+      <item>
+       <layout class="QFormLayout" name="formLayout">
+        <item row="0" column="0">
+         <widget class="QLabel" name="numberOfSamplesLabel">
+          <property name="text">
+           <string>Number of samples:</string>
+          </property>
+         </widget>
+        </item>
+        <item row="0" column="1">
+         <widget class="QSpinBox" name="numberOfSamplesSpinBox">
+          <property name="minimum">
+           <number>3</number>
+          </property>
+          <property name="maximum">
+           <number>10000000</number>
+          </property>
+          <property name="value">
+           <number>200</number>
+          </property>
+         </widget>
+        </item>
+        <item row="1" column="0">
+         <widget class="QLabel" name="accuracyLabel">
+          <property name="text">
+           <string>Accuracy:</string>
+          </property>
+         </widget>
+        </item>
+        <item row="1" column="1">
+         <widget class="QDoubleSpinBox" name="accuracyDoubleSpinBox">
+          <property name="decimals">
+           <number>5</number>
+          </property>
+          <property name="minimum">
+           <double>0.000010000000000</double>
+          </property>
+          <property name="maximum">
+           <double>100000.000000000000000</double>
+          </property>
+          <property name="singleStep">
+           <double>0.100000000000000</double>
+          </property>
+          <property name="value">
+           <double>5.000000000000000</double>
+          </property>
+         </widget>
+        </item>
+        <item row="2" column="0">
+         <widget class="QLabel" name="overlapLabel">
+          <property name="text">
+           <string>Overlap:</string>
+          </property>
+         </widget>
+        </item>
+        <item row="3" column="0">
+         <widget class="QLabel" name="maximumRunningTimeLabel">
+          <property name="text">
+           <string>Maximum running time:</string>
+          </property>
+         </widget>
+        </item>
+        <item row="3" column="1">
+         <widget class="QSpinBox" name="maximumRunningTimeSpinBox">
+          <property name="suffix">
+           <string> s</string>
+          </property>
+          <property name="minimum">
+           <number>1</number>
+          </property>
+          <property name="maximum">
+           <number>36000</number>
+          </property>
+          <property name="value">
+           <number>60</number>
+          </property>
+         </widget>
+        </item>
+        <item row="2" column="1">
+         <widget class="QSpinBox" name="overlapSpinBox">
+          <property name="suffix">
+           <string> %</string>
+          </property>
+          <property name="minimum">
+           <number>1</number>
+          </property>
+          <property name="maximum">
+           <number>100</number>
+          </property>
+          <property name="singleStep">
+           <number>10</number>
+          </property>
+          <property name="value">
+           <number>20</number>
+          </property>
+         </widget>
+        </item>
+       </layout>
+      </item>
+     </layout>
+    </widget>
+   </item>
+   <item>
+    <widget class="QCheckBox" name="fineRegistration">
+     <property name="text">
+      <string>Fine registration using libpointmatcher</string>
+     </property>
+     <property name="checked">
+      <bool>true</bool>
+     </property>
+    </widget>
+   </item>
+   <item>
+    <widget class="QScrollArea" name="pointmatcher_frame">
+     <property name="sizePolicy">
+      <sizepolicy hsizetype="MinimumExpanding" vsizetype="Expanding">
+       <horstretch>0</horstretch>
+       <verstretch>0</verstretch>
+      </sizepolicy>
+     </property>
+     <property name="minimumSize">
+      <size>
+       <width>400</width>
+       <height>200</height>
+      </size>
+     </property>
+     <property name="widgetResizable">
+      <bool>true</bool>
+     </property>
+     <widget class="QWidget" name="scrollAreaWidgetContents">
+      <property name="geometry">
+       <rect>
+        <x>0</x>
+        <y>0</y>
+        <width>398</width>
+        <height>198</height>
+       </rect>
+      </property>
+      <layout class="QHBoxLayout" name="horizontalLayout_2">
+       <item>
+        <widget class="QTextEdit" name="config">
+         <property name="html">
+          <string>&lt;!DOCTYPE HTML PUBLIC &quot;-//W3C//DTD HTML 4.0//EN&quot; &quot;http://www.w3.org/TR/REC-html40/strict.dtd&quot;&gt;
+&lt;html&gt;&lt;head&gt;&lt;meta name=&quot;qrichtext&quot; content=&quot;1&quot; /&gt;&lt;style type=&quot;text/css&quot;&gt;
+p, li { white-space: pre-wrap; }
+&lt;/style&gt;&lt;/head&gt;&lt;body style=&quot; font-family:'Noto Sans'; font-size:9pt; font-weight:400; font-style:normal;&quot;&gt;
+&lt;p style=&quot; margin-top:12px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:'Courier New,courier';&quot;&gt;readingDataPointsFilters:&lt;/span&gt;&lt;/p&gt;
+&lt;p style=&quot; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:'Courier New,courier';&quot;&gt;  - RandomSamplingDataPointsFilter:&lt;/span&gt;&lt;/p&gt;
+&lt;p style=&quot; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:'Courier New,courier';&quot;&gt;      prob: 0.5&lt;/span&gt;&lt;/p&gt;
+&lt;p style=&quot;-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:'Courier New,courier';&quot;&gt;&lt;br /&gt;&lt;/p&gt;
+&lt;p style=&quot; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:'Courier New,courier';&quot;&gt;referenceDataPointsFilters:&lt;/span&gt;&lt;/p&gt;
+&lt;p style=&quot; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:'Courier New,courier';&quot;&gt;  - SamplingSurfaceNormalDataPointsFilter:&lt;/span&gt;&lt;/p&gt;
+&lt;p style=&quot; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:'Courier New,courier';&quot;&gt;      knn: 10&lt;/span&gt;&lt;/p&gt;
+&lt;p style=&quot;-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:'Courier New,courier';&quot;&gt;&lt;br /&gt;&lt;/p&gt;
+&lt;p style=&quot; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:'Courier New,courier';&quot;&gt;matcher:&lt;/span&gt;&lt;/p&gt;
+&lt;p style=&quot; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:'Courier New,courier';&quot;&gt;  KDTreeMatcher:&lt;/span&gt;&lt;/p&gt;
+&lt;p style=&quot; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:'Courier New,courier';&quot;&gt;    knn: 1&lt;/span&gt;&lt;/p&gt;
+&lt;p style=&quot; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:'Courier New,courier';&quot;&gt;    epsilon: 0 &lt;/span&gt;&lt;/p&gt;
+&lt;p style=&quot;-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:'Courier New,courier';&quot;&gt;&lt;br /&gt;&lt;/p&gt;
+&lt;p style=&quot; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:'Courier New,courier';&quot;&gt;outlierFilters:&lt;/span&gt;&lt;/p&gt;
+&lt;p style=&quot; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:'Courier New,courier';&quot;&gt;  - TrimmedDistOutlierFilter:&lt;/span&gt;&lt;/p&gt;
+&lt;p style=&quot; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:'Courier New,courier';&quot;&gt;      ratio: 0.75&lt;/span&gt;&lt;/p&gt;
+&lt;p style=&quot;-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:'Courier New,courier';&quot;&gt;&lt;br /&gt;&lt;/p&gt;
+&lt;p style=&quot; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:'Courier New,courier';&quot;&gt;errorMinimizer:&lt;/span&gt;&lt;/p&gt;
+&lt;p style=&quot; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:'Courier New,courier';&quot;&gt;  PointToPlaneErrorMinimizer&lt;/span&gt;&lt;/p&gt;
+&lt;p style=&quot;-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:'Courier New,courier';&quot;&gt;&lt;br /&gt;&lt;/p&gt;
+&lt;p style=&quot; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:'Courier New,courier';&quot;&gt;transformationCheckers:&lt;/span&gt;&lt;/p&gt;
+&lt;p style=&quot; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:'Courier New,courier';&quot;&gt;  - CounterTransformationChecker:&lt;/span&gt;&lt;/p&gt;
+&lt;p style=&quot; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:'Courier New,courier';&quot;&gt;      maxIterationCount: 40&lt;/span&gt;&lt;/p&gt;
+&lt;p style=&quot; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:'Courier New,courier';&quot;&gt;  - DifferentialTransformationChecker:&lt;/span&gt;&lt;/p&gt;
+&lt;p style=&quot; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:'Courier New,courier';&quot;&gt;      minDiffRotErr: 0.001&lt;/span&gt;&lt;/p&gt;
+&lt;p style=&quot; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:'Courier New,courier';&quot;&gt;      minDiffTransErr: 0.01&lt;/span&gt;&lt;/p&gt;
+&lt;p style=&quot; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:'Courier New,courier';&quot;&gt;      smoothLength: 4   &lt;/span&gt;&lt;/p&gt;
+&lt;p style=&quot;-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:'Courier New,courier';&quot;&gt;&lt;br /&gt;&lt;/p&gt;
+&lt;p style=&quot; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:'Courier New,courier';&quot;&gt;inspector:&lt;/span&gt;&lt;/p&gt;
+&lt;p style=&quot; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:'Courier New,courier';&quot;&gt; VTKFileInspector:&lt;/span&gt;&lt;/p&gt;
+&lt;p style=&quot; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:'Courier New,courier';&quot;&gt;     baseFileName: pointmatcher-run1&lt;/span&gt;&lt;/p&gt;
+&lt;p style=&quot; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:'Courier New,courier';&quot;&gt;     dumpPerfOnExit: 0&lt;/span&gt;&lt;/p&gt;
+&lt;p style=&quot; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:'Courier New,courier';&quot;&gt;     dumpStats: 0&lt;/span&gt;&lt;/p&gt;
+&lt;p style=&quot; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:'Courier New,courier';&quot;&gt;     dumpIterationInfo: 1 &lt;/span&gt;&lt;/p&gt;
+&lt;p style=&quot; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:'Courier New,courier';&quot;&gt;     dumpDataLinks: 0 &lt;/span&gt;&lt;/p&gt;
+&lt;p style=&quot; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:'Courier New,courier';&quot;&gt;     dumpReading: 0 &lt;/span&gt;&lt;/p&gt;
+&lt;p style=&quot; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:'Courier New,courier';&quot;&gt;     dumpReference: 0 &lt;/span&gt;&lt;/p&gt;
+&lt;p style=&quot;-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:'Courier New,courier';&quot;&gt;&lt;br /&gt;&lt;/p&gt;
+&lt;p style=&quot; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:'Courier New,courier';&quot;&gt;logger:&lt;/span&gt;&lt;/p&gt;
+&lt;p style=&quot; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;&quot;&gt;&lt;span style=&quot; font-family:'Courier New,courier';&quot;&gt;  NullLogger&lt;/span&gt;&lt;/p&gt;&lt;/body&gt;&lt;/html&gt;</string>
+         </property>
+        </widget>
+       </item>
+      </layout>
+     </widget>
+    </widget>
+   </item>
+   <item>
+    <widget class="QDialogButtonBox" name="buttonBox">
+     <property name="orientation">
+      <enum>Qt::Horizontal</enum>
+     </property>
+     <property name="standardButtons">
+      <set>QDialogButtonBox::Cancel|QDialogButtonBox::Ok</set>
+     </property>
+    </widget>
+   </item>
+  </layout>
+ </widget>
+ <resources/>
+ <connections>
+  <connection>
+   <sender>buttonBox</sender>
+   <signal>accepted()</signal>
+   <receiver>RegisterPointSetsDialog</receiver>
+   <slot>accept()</slot>
+   <hints>
+    <hint type="sourcelabel">
+     <x>248</x>
+     <y>254</y>
+    </hint>
+    <hint type="destinationlabel">
+     <x>157</x>
+     <y>274</y>
+    </hint>
+   </hints>
+  </connection>
+  <connection>
+   <sender>buttonBox</sender>
+   <signal>rejected()</signal>
+   <receiver>RegisterPointSetsDialog</receiver>
+   <slot>reject()</slot>
+   <hints>
+    <hint type="sourcelabel">
+     <x>316</x>
+     <y>260</y>
+    </hint>
+    <hint type="destinationlabel">
+     <x>286</x>
+     <y>274</y>
+    </hint>
+   </hints>
+  </connection>
+  <connection>
+   <sender>coarseRegistration</sender>
+   <signal>toggled(bool)</signal>
+   <receiver>opengr_frame</receiver>
+   <slot>setEnabled(bool)</slot>
+   <hints>
+    <hint type="sourcelabel">
+     <x>208</x>
+     <y>63</y>
+    </hint>
+    <hint type="destinationlabel">
+     <x>208</x>
+     <y>158</y>
+    </hint>
+   </hints>
+  </connection>
+  <connection>
+   <sender>fineRegistration</sender>
+   <signal>toggled(bool)</signal>
+   <receiver>pointmatcher_frame</receiver>
+   <slot>setEnabled(bool)</slot>
+   <hints>
+    <hint type="sourcelabel">
+     <x>208</x>
+     <y>253</y>
+    </hint>
+    <hint type="destinationlabel">
+     <x>208</x>
+     <y>370</y>
+    </hint>
+   </hints>
+  </connection>
+ </connections>
+</ui>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Surface_reconstruction_plugin.ui 5.1-1/demo/Polyhedron/Plugins/Point_set/Surface_reconstruction_plugin.ui
--- 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Surface_reconstruction_plugin.ui	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Point_set/Surface_reconstruction_plugin.ui	2020-09-08 12:12:20.000000000 +0000
@@ -6,7 +6,7 @@
    <rect>
     <x>0</x>
     <y>0</y>
-    <width>404</width>
+    <width>427</width>
     <height>482</height>
    </rect>
   </property>
@@ -736,7 +736,6 @@
         </widget>
        </item>
       </layout>
-      <zorder>tabWidget_2</zorder>
      </widget>
      <widget class="QWidget" name="tab_6">
       <attribute name="title">
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Surface_reconstruction_poisson_impl.cpp 5.1-1/demo/Polyhedron/Plugins/Point_set/Surface_reconstruction_poisson_impl.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Surface_reconstruction_poisson_impl.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Point_set/Surface_reconstruction_poisson_impl.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -25,14 +25,8 @@
 #include "SMesh_type.h"
 #include "Scene_points_with_normal_item.h"
 
-
-
 // Concurrency
-#ifdef CGAL_LINKED_WITH_TBB
-typedef CGAL::Parallel_tag Concurrency_tag;
-#else
-typedef CGAL::Sequential_tag Concurrency_tag;
-#endif
+typedef CGAL::Parallel_if_available_tag Concurrency_tag;
 
 template <typename Triangulation>
 class Marching_tets
@@ -358,7 +352,6 @@ SMesh* poisson_reconstruct(Point_set& po
   // Constructs AABB tree and computes internal KD-tree
   // data structure to accelerate distance queries
   AABB_tree tree(faces(*mesh).first, faces(*mesh).second, *mesh);
-  tree.accelerate_distance_queries();
 
   // Computes distance from each input point to reconstructed mesh
   double max_distance = DBL_MIN;
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Surface_reconstruction_scale_space_impl.cpp 5.1-1/demo/Polyhedron/Plugins/Point_set/Surface_reconstruction_scale_space_impl.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Point_set/Surface_reconstruction_scale_space_impl.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Point_set/Surface_reconstruction_scale_space_impl.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -13,11 +13,7 @@
 
 
 // Concurrency
-#ifdef CGAL_LINKED_WITH_TBB
-typedef CGAL::Parallel_tag Concurrency_tag;
-#else
-typedef CGAL::Sequential_tag Concurrency_tag;
-#endif
+typedef CGAL::Parallel_if_available_tag Concurrency_tag;
 
 typedef CGAL::Scale_space_surface_reconstruction_3<Kernel> ScaleSpace;
 typedef CGAL::Scale_space_reconstruction_3::Advancing_front_mesher<Kernel> ScaleSpaceAFM;
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Surface_mesh/CMakeLists.txt 5.1-1/demo/Polyhedron/Plugins/Surface_mesh/CMakeLists.txt
--- 5.0.3-1/demo/Polyhedron/Plugins/Surface_mesh/CMakeLists.txt	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Surface_mesh/CMakeLists.txt	2020-09-08 12:12:20.000000000 +0000
@@ -4,17 +4,17 @@ if(POLICY CMP0074)
   cmake_policy(SET CMP0074 NEW)
 endif()
 
-if(EIGEN3_FOUND)
+if(TARGET CGAL::Eigen_support)
   find_package(CGAL COMPONENTS Core)
 
   include(${CGAL_USE_FILE})
 
   qt5_wrap_ui(parameterizationUI_FILES Parameterization_widget.ui OTE_dialog.ui)
   polyhedron_demo_plugin(parameterization_plugin Parameterization_plugin ${parameterizationUI_FILES})
-  target_link_libraries(parameterization_plugin PUBLIC scene_surface_mesh_item scene_textured_item scene_selection_item)
-else(EIGEN3_FOUND)
+  target_link_libraries(parameterization_plugin PUBLIC scene_surface_mesh_item scene_textured_item scene_selection_item CGAL::Eigen_support)
+else()
   message(STATUS "NOTICE: Eigen 3.1 (or greater) was not found. The Parameterization plugin will not be available.")
-endif(EIGEN3_FOUND)
+endif()
 
 qt5_wrap_ui( segmentationUI_FILES Mesh_segmentation_widget.ui)
 polyhedron_demo_plugin(mesh_segmentation_plugin Mesh_segmentation_plugin ${segmentationUI_FILES})
@@ -31,9 +31,11 @@ target_link_libraries(mesh_simplificatio
 qt5_wrap_ui( remeshingUI_FILES  Remeshing_dialog.ui)
 polyhedron_demo_plugin(offset_meshing_plugin Offset_meshing_plugin ${remeshingUI_FILES})
 target_link_libraries(offset_meshing_plugin PUBLIC scene_surface_mesh_item scene_polygon_soup_item)
-
-if(TBB_FOUND)
-  CGAL_target_use_TBB(offset_meshing_plugin)
+if(TARGET CGAL::Eigen_support)
+  target_link_libraries(offset_meshing_plugin PUBLIC CGAL::Eigen_support)
+endif()
+if(TARGET CGAL::TBB_support)
+  target_link_libraries(offset_meshing_plugin PUBLIC CGAL::TBB_support)
 endif()
 
 qt5_wrap_ui( shortestPathUI_FILES Shortest_path_widget.ui )
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Surface_mesh/Mesh_plane_detection_dialog.ui 5.1-1/demo/Polyhedron/Plugins/Surface_mesh/Mesh_plane_detection_dialog.ui
--- 5.0.3-1/demo/Polyhedron/Plugins/Surface_mesh/Mesh_plane_detection_dialog.ui	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Surface_mesh/Mesh_plane_detection_dialog.ui	2020-09-08 12:12:20.000000000 +0000
@@ -23,25 +23,6 @@
        </property>
       </widget>
      </item>
-     <item row="0" column="1">
-      <widget class="QDoubleSpinBox" name="minimumAreaDoubleSpinBox">
-       <property name="decimals">
-        <number>5</number>
-       </property>
-       <property name="minimum">
-        <double>0.000010000000000</double>
-       </property>
-       <property name="maximum">
-        <double>1000000.000000000000000</double>
-       </property>
-       <property name="singleStep">
-        <double>0.010000000000000</double>
-       </property>
-       <property name="value">
-        <double>0.010000000000000</double>
-       </property>
-      </widget>
-     </item>
      <item row="1" column="0">
       <widget class="QLabel" name="maximumDeviationFromNormalLabel">
        <property name="text">
@@ -65,6 +46,13 @@
        </property>
       </widget>
      </item>
+     <item row="0" column="1">
+      <widget class="DoubleEdit" name="minimumAreaDoubleSpinBox">
+       <property name="text">
+        <string>0.01</string>
+       </property>
+      </widget>
+     </item>
     </layout>
    </item>
    <item>
@@ -92,6 +80,13 @@
    </item>
   </layout>
  </widget>
+ <customwidgets>
+  <customwidget>
+   <class>DoubleEdit</class>
+   <extends>QLineEdit</extends>
+   <header>CGAL_double_edit.h</header>
+  </customwidget>
+ </customwidgets>
  <resources/>
  <connections>
   <connection>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Surface_mesh/Mesh_plane_detection_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Surface_mesh/Mesh_plane_detection_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Surface_mesh/Mesh_plane_detection_plugin.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Surface_mesh/Mesh_plane_detection_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -191,7 +191,7 @@ void Polyhedron_demo_mesh_plane_detectio
       QDialog dialog(mw);
       Ui::Mesh_plane_detection_dialog ui;
       ui.setupUi(&dialog);
-
+      ui.minimumAreaDoubleSpinBox->setMinimum(0.00001);
       // check user cancellation
       if(dialog.exec() == QDialog::Rejected)
         return;
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Surface_mesh/Mesh_segmentation_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Surface_mesh/Mesh_segmentation_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Surface_mesh/Mesh_segmentation_plugin.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Surface_mesh/Mesh_segmentation_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -92,6 +92,7 @@ public:
         dock_widget = new QDockWidget("Mesh segmentation parameters", mw);
         dock_widget->setVisible(false); // do not show at the beginning
         ui_widget.setupUi(dock_widget);
+        ui_widget.Smoothness_spin_box->setMaximum(10.0);
         mw->addDockWidget(Qt::LeftDockWidgetArea, dock_widget);
 
         connect(ui_widget.Partition_button,  SIGNAL(clicked()), this, SLOT(on_Partition_button_clicked()));
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Surface_mesh/Mesh_segmentation_widget.ui 5.1-1/demo/Polyhedron/Plugins/Surface_mesh/Mesh_segmentation_widget.ui
--- 5.0.3-1/demo/Polyhedron/Plugins/Surface_mesh/Mesh_segmentation_widget.ui	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Surface_mesh/Mesh_segmentation_widget.ui	2020-09-08 12:12:20.000000000 +0000
@@ -31,13 +31,6 @@
          </property>
         </widget>
        </item>
-       <item row="0" column="1">
-        <widget class="QSpinBox" name="Number_of_rays_spin_box">
-         <property name="value">
-          <number>25</number>
-         </property>
-        </widget>
-       </item>
        <item row="1" column="0">
         <widget class="QLabel" name="Cone_angle_label">
          <property name="text">
@@ -58,6 +51,13 @@
          </property>
         </widget>
        </item>
+       <item row="0" column="1">
+        <widget class="QSpinBox" name="Number_of_rays_spin_box">
+         <property name="value">
+          <number>25</number>
+         </property>
+        </widget>
+       </item>
       </layout>
      </widget>
     </item>
@@ -100,19 +100,6 @@
          </property>
         </widget>
        </item>
-       <item row="1" column="1">
-        <widget class="QDoubleSpinBox" name="Smoothness_spin_box">
-         <property name="maximum">
-          <double>10.000000000000000</double>
-         </property>
-         <property name="singleStep">
-          <double>0.010000000000000</double>
-         </property>
-         <property name="value">
-          <double>0.260000000000000</double>
-         </property>
-        </widget>
-       </item>
        <item row="2" column="0">
         <widget class="QLabel" name="label">
          <property name="layoutDirection">
@@ -136,6 +123,13 @@
          </property>
         </widget>
        </item>
+       <item row="1" column="1">
+        <widget class="DoubleEdit" name="Smoothness_spin_box">
+         <property name="text">
+          <string>0.26</string>
+         </property>
+        </widget>
+       </item>
       </layout>
      </widget>
     </item>
@@ -194,6 +188,13 @@
    </layout>
   </widget>
  </widget>
+ <customwidgets>
+  <customwidget>
+   <class>DoubleEdit</class>
+   <extends>QLineEdit</extends>
+   <header>CGAL_double_edit.h</header>
+  </customwidget>
+ </customwidgets>
  <resources/>
  <connections/>
 </ui>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Surface_mesh/Mesh_simplification_dialog.ui 5.1-1/demo/Polyhedron/Plugins/Surface_mesh/Mesh_simplification_dialog.ui
--- 5.0.3-1/demo/Polyhedron/Plugins/Surface_mesh/Mesh_simplification_dialog.ui	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Surface_mesh/Mesh_simplification_dialog.ui	2020-09-08 12:12:20.000000000 +0000
@@ -105,15 +105,9 @@
       </widget>
      </item>
      <item>
-      <widget class="QDoubleSpinBox" name="m_edge_length">
-       <property name="enabled">
-        <bool>true</bool>
-       </property>
-       <property name="decimals">
-        <number>5</number>
-       </property>
-       <property name="maximum">
-        <double>10000.000000000000000</double>
+      <widget class="DoubleEdit" name="m_edge_length">
+       <property name="text">
+        <string>0.000</string>
        </property>
       </widget>
      </item>
@@ -144,6 +138,13 @@
    </item>
   </layout>
  </widget>
+ <customwidgets>
+  <customwidget>
+   <class>DoubleEdit</class>
+   <extends>QLineEdit</extends>
+   <header>CGAL_double_edit.h</header>
+  </customwidget>
+ </customwidgets>
  <resources/>
  <connections>
   <connection>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Surface_mesh/Mesh_simplification_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Surface_mesh/Mesh_simplification_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Surface_mesh/Mesh_simplification_plugin.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Surface_mesh/Mesh_simplification_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -9,7 +9,6 @@
 #include <QElapsedTimer>
 #include <QAction>
 
-#include <CGAL/Surface_mesh_simplification/HalfedgeGraph_Polyhedron_3.h>
 #include <CGAL/Surface_mesh_simplification/edge_collapse.h>
 #include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Count_stop_predicate.h>
 #include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_length_stop_predicate.h>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Surface_mesh/Offset_meshing_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Surface_mesh/Offset_meshing_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Surface_mesh/Offset_meshing_plugin.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Surface_mesh/Offset_meshing_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -58,7 +58,6 @@ public:
     , m_is_closed( is_closed(tm) )
   {
     CGAL_assertion(!m_tree_ptr->empty());
-    m_tree_ptr->accelerate_distance_queries();
   }
 
   double operator()(const typename GeomTraits::Point_3& p) const
@@ -188,7 +187,6 @@ public:
     , m_offset_distance(offset_distance)
   {
     CGAL_assertion(! m_tree_ptr->empty() );
-    m_tree_ptr->accelerate_distance_queries();
   }
 
   double operator()(const EPICK::Point_3& p) const
@@ -452,17 +450,16 @@ void Polyhedron_demo_offset_meshing_plug
   QDialog dialog(mw);
   Ui::Remeshing_dialog ui;
   ui.setupUi(&dialog);
+  ui.angle->setRange(1.0, 30.0);
   connect(ui.buttonBox, SIGNAL(accepted()),
           &dialog, SLOT(accept()));
   connect(ui.buttonBox, SIGNAL(rejected()),
           &dialog, SLOT(reject()));
 
-  ui.sizing->setDecimals(4);
   ui.sizing->setRange(diag * 10e-6, // min
                       diag); // max
   ui.sizing->setValue(diag * 0.05); // default value
 
-  ui.approx->setDecimals(6);
   ui.approx->setRange(diag * 10e-7, // min
                       diag); // max
   ui.approx->setValue(diag * 0.005);
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Surface_mesh/Parameterization_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Surface_mesh/Parameterization_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Surface_mesh/Parameterization_plugin.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Surface_mesh/Parameterization_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -143,12 +143,17 @@ protected:
     }
     case QEvent::Wheel: {
       QWheelEvent* event = static_cast<QWheelEvent*>(ev);
-      QPointF old_pos = v->mapToScene(event->pos());
-      if(event->delta() <0)
+#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
+      QPoint pos = event->pos();
+#else
+      QPointF pos = event->position();
+#endif
+      QPointF old_pos = v->mapToScene(pos.x(), pos.y());
+      if(event->angleDelta().y() <0)
         v->scale(1.2, 1.2);
       else
         v->scale(0.8, 0.8);
-      QPointF new_pos = v->mapToScene(event->pos());
+      QPointF new_pos = v->mapToScene(pos.x(), pos.y());
       QPointF delta = new_pos - old_pos;
       v->translate(delta.x(), delta.y());
       v->update();
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Surface_mesh/Remeshing_dialog.ui 5.1-1/demo/Polyhedron/Plugins/Surface_mesh/Remeshing_dialog.ui
--- 5.0.3-1/demo/Polyhedron/Plugins/Surface_mesh/Remeshing_dialog.ui	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Surface_mesh/Remeshing_dialog.ui	2020-09-08 12:12:20.000000000 +0000
@@ -15,7 +15,33 @@
   </property>
   <layout class="QVBoxLayout" name="verticalLayout">
    <item>
-    <layout class="QGridLayout" name="gridLayout" columnstretch="0,1">
+    <layout class="QGridLayout" name="gridLayout" columnstretch="0,0">
+     <item row="1" column="1">
+      <widget class="DoubleEdit" name="sizing">
+       <property name="text">
+        <string>0.00</string>
+       </property>
+      </widget>
+     </item>
+     <item row="3" column="1">
+      <widget class="QComboBox" name="tags">
+       <item>
+        <property name="text">
+         <string>Non manifold</string>
+        </property>
+       </item>
+       <item>
+        <property name="text">
+         <string>Manifold</string>
+        </property>
+       </item>
+       <item>
+        <property name="text">
+         <string>Manifold with boundaries</string>
+        </property>
+       </item>
+      </widget>
+     </item>
      <item row="0" column="0">
       <widget class="QLabel" name="angleLabel">
        <property name="text">
@@ -24,22 +50,6 @@
        <property name="alignment">
         <set>Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter</set>
        </property>
-       <property name="buddy">
-        <cstring>angle</cstring>
-       </property>
-      </widget>
-     </item>
-     <item row="0" column="1">
-      <widget class="QDoubleSpinBox" name="angle">
-       <property name="minimum">
-        <double>1.000000000000000</double>
-       </property>
-       <property name="maximum">
-        <double>30.000000000000000</double>
-       </property>
-       <property name="value">
-        <double>25.000000000000000</double>
-       </property>
       </widget>
      </item>
      <item row="1" column="0">
@@ -50,16 +60,6 @@
        <property name="alignment">
         <set>Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter</set>
        </property>
-       <property name="buddy">
-        <cstring>sizing</cstring>
-       </property>
-      </widget>
-     </item>
-     <item row="1" column="1">
-      <widget class="QDoubleSpinBox" name="sizing">
-       <property name="decimals">
-        <number>4</number>
-       </property>
       </widget>
      </item>
      <item row="2" column="0">
@@ -70,31 +70,6 @@
        <property name="alignment">
         <set>Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter</set>
        </property>
-       <property name="buddy">
-        <cstring>approx</cstring>
-       </property>
-      </widget>
-     </item>
-     <item row="2" column="1">
-      <widget class="QDoubleSpinBox" name="approx"/>
-     </item>
-     <item row="3" column="1">
-      <widget class="QComboBox" name="tags">
-       <item>
-        <property name="text">
-         <string>Non manifold</string>
-        </property>
-       </item>
-       <item>
-        <property name="text">
-         <string>Manifold</string>
-        </property>
-       </item>
-       <item>
-        <property name="text">
-         <string>Manifold with boundaries</string>
-        </property>
-       </item>
       </widget>
      </item>
      <item row="3" column="0">
@@ -110,6 +85,20 @@
        </property>
       </widget>
      </item>
+     <item row="2" column="1">
+      <widget class="DoubleEdit" name="approx">
+       <property name="text">
+        <string>0.00</string>
+       </property>
+      </widget>
+     </item>
+     <item row="0" column="1">
+      <widget class="DoubleEdit" name="angle">
+       <property name="text">
+        <string>25.0</string>
+       </property>
+      </widget>
+     </item>
     </layout>
    </item>
    <item>
@@ -137,6 +126,13 @@
    </item>
   </layout>
  </widget>
+ <customwidgets>
+  <customwidget>
+   <class>DoubleEdit</class>
+   <extends>QLineEdit</extends>
+   <header>CGAL_double_edit.h</header>
+  </customwidget>
+ </customwidgets>
  <resources/>
  <connections>
   <connection>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Surface_mesh/Surface_mesh_approximation_dockwidget.ui 5.1-1/demo/Polyhedron/Plugins/Surface_mesh/Surface_mesh_approximation_dockwidget.ui
--- 5.0.3-1/demo/Polyhedron/Plugins/Surface_mesh/Surface_mesh_approximation_dockwidget.ui	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Surface_mesh/Surface_mesh_approximation_dockwidget.ui	2020-09-08 12:12:20.000000000 +0000
@@ -6,7 +6,7 @@
    <rect>
     <x>0</x>
     <y>0</y>
-    <width>667</width>
+    <width>652</width>
     <height>360</height>
    </rect>
   </property>
@@ -64,20 +64,19 @@
        </item>
        <item>
         <layout class="QGridLayout" name="gridLayout_2">
-         <item row="2" column="0">
-          <widget class="QLabel" name="label_9">
-           <property name="toolTip">
-            <string/>
+         <item row="4" column="2">
+          <widget class="QSpinBox" name="nb_iterations">
+           <property name="alignment">
+            <set>Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter</set>
            </property>
-           <property name="text">
-            <string>Error drop</string>
+           <property name="buttonSymbols">
+            <enum>QAbstractSpinBox::NoButtons</enum>
            </property>
-          </widget>
-         </item>
-         <item row="0" column="0">
-          <widget class="QLabel" name="label_8">
-           <property name="text">
-            <string>#Proxies</string>
+           <property name="maximum">
+            <number>999</number>
+           </property>
+           <property name="value">
+            <number>20</number>
            </property>
           </widget>
          </item>
@@ -88,19 +87,10 @@
            </property>
           </widget>
          </item>
-         <item row="0" column="2">
-          <widget class="QSpinBox" name="nb_proxies">
-           <property name="alignment">
-            <set>Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter</set>
-           </property>
-           <property name="buttonSymbols">
-            <enum>QAbstractSpinBox::NoButtons</enum>
-           </property>
-           <property name="maximum">
-            <number>999</number>
-           </property>
-           <property name="value">
-            <number>20</number>
+         <item row="2" column="1">
+          <widget class="QCheckBox" name="enable_error_drop">
+           <property name="text">
+            <string/>
            </property>
           </widget>
          </item>
@@ -117,34 +107,36 @@
            </property>
           </spacer>
          </item>
-         <item row="2" column="2">
-          <widget class="QDoubleSpinBox" name="error_drop">
-           <property name="enabled">
-            <bool>false</bool>
-           </property>
-           <property name="toolTip">
-            <string>Negative value is ignored</string>
-           </property>
+         <item row="0" column="2">
+          <widget class="QSpinBox" name="nb_proxies">
            <property name="alignment">
             <set>Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter</set>
            </property>
            <property name="buttonSymbols">
             <enum>QAbstractSpinBox::NoButtons</enum>
            </property>
-           <property name="decimals">
-            <number>4</number>
+           <property name="maximum">
+            <number>999</number>
            </property>
-           <property name="minimum">
-            <double>0.001000000000000</double>
+           <property name="value">
+            <number>20</number>
            </property>
-           <property name="maximum">
-            <double>1.000000000000000</double>
+          </widget>
+         </item>
+         <item row="2" column="0">
+          <widget class="QLabel" name="label_9">
+           <property name="toolTip">
+            <string/>
            </property>
-           <property name="singleStep">
-            <double>0.010000000000000</double>
+           <property name="text">
+            <string>Error drop</string>
            </property>
-           <property name="value">
-            <double>0.100000000000000</double>
+          </widget>
+         </item>
+         <item row="0" column="0">
+          <widget class="QLabel" name="label_8">
+           <property name="text">
+            <string>#Proxies</string>
            </property>
           </widget>
          </item>
@@ -168,26 +160,10 @@
            </property>
           </widget>
          </item>
-         <item row="4" column="2">
-          <widget class="QSpinBox" name="nb_iterations">
-           <property name="alignment">
-            <set>Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter</set>
-           </property>
-           <property name="buttonSymbols">
-            <enum>QAbstractSpinBox::NoButtons</enum>
-           </property>
-           <property name="maximum">
-            <number>999</number>
-           </property>
-           <property name="value">
-            <number>20</number>
-           </property>
-          </widget>
-         </item>
-         <item row="2" column="1">
-          <widget class="QCheckBox" name="enable_error_drop">
+         <item row="2" column="2">
+          <widget class="DoubleEdit" name="error_drop">
            <property name="text">
-            <string/>
+            <string>0.1</string>
            </property>
           </widget>
          </item>
@@ -410,25 +386,6 @@
               </property>
              </spacer>
             </item>
-            <item row="1" column="2">
-             <widget class="QDoubleSpinBox" name="chord_error">
-              <property name="alignment">
-               <set>Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter</set>
-              </property>
-              <property name="buttonSymbols">
-               <enum>QAbstractSpinBox::NoButtons</enum>
-              </property>
-              <property name="maximum">
-               <double>10.000000000000000</double>
-              </property>
-              <property name="singleStep">
-               <double>0.100000000000000</double>
-              </property>
-              <property name="value">
-               <double>3.000000000000000</double>
-              </property>
-             </widget>
-            </item>
             <item row="1" column="0">
              <widget class="QLabel" name="label_3">
               <property name="text">
@@ -436,6 +393,19 @@
               </property>
              </widget>
             </item>
+            <item row="1" column="2">
+             <widget class="DoubleEdit" name="chord_error">
+              <property name="sizePolicy">
+               <sizepolicy hsizetype="Minimum" vsizetype="Fixed">
+                <horstretch>0</horstretch>
+                <verstretch>0</verstretch>
+               </sizepolicy>
+              </property>
+              <property name="text">
+               <string>3.00</string>
+              </property>
+             </widget>
+            </item>
            </layout>
           </item>
          </layout>
@@ -491,23 +461,13 @@
    </layout>
   </widget>
  </widget>
+ <customwidgets>
+  <customwidget>
+   <class>DoubleEdit</class>
+   <extends>QLineEdit</extends>
+   <header>CGAL_double_edit.h</header>
+  </customwidget>
+ </customwidgets>
  <resources/>
- <connections>
-  <connection>
-   <sender>enable_error_drop</sender>
-   <signal>toggled(bool)</signal>
-   <receiver>error_drop</receiver>
-   <slot>setEnabled(bool)</slot>
-   <hints>
-    <hint type="sourcelabel">
-     <x>127</x>
-     <y>168</y>
-    </hint>
-    <hint type="destinationlabel">
-     <x>153</x>
-     <y>167</y>
-    </hint>
-   </hints>
-  </connection>
- </connections>
+ <connections/>
 </ui>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Surface_mesh/Surface_mesh_approximation_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Surface_mesh/Surface_mesh_approximation_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Surface_mesh/Surface_mesh_approximation_plugin.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Surface_mesh/Surface_mesh_approximation_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -81,6 +81,8 @@ public:
     dock_widget = new QDockWidget("Mesh approximation parameters", mw);
     dock_widget->setVisible(false);
     ui_widget.setupUi(dock_widget);
+    ui_widget.chord_error->setRange(0.0, 10.0);
+    ui_widget.error_drop->setRange(0.01, 1.0);
     mw->addDockWidget(Qt::LeftDockWidgetArea, dock_widget);
 
     // connect ui actions
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Surface_mesh/UVProjector.h 5.1-1/demo/Polyhedron/Plugins/Surface_mesh/UVProjector.h
--- 5.0.3-1/demo/Polyhedron/Plugins/Surface_mesh/UVProjector.h	2020-08-01 19:02:42.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Surface_mesh/UVProjector.h	2020-09-08 12:15:25.000000000 +0000
@@ -23,7 +23,7 @@ struct State{
 class UVProjector:public QWidget
 {
 public:
-  UVProjector(QWidget* parent = 0, Qt::WindowFlags flags =0)
+  UVProjector(QWidget* parent = 0, Qt::WindowFlags flags = Qt::WindowType(0))
     :QWidget(parent,flags)
   {
     setMouseTracking(true);
@@ -108,7 +108,7 @@ protected:
   }
   void wheelEvent(QWheelEvent *event)
   {
-   if(event->delta() >0)
+   if(event->angleDelta().y() >0)
      translation[2] *= 1.2;
    else
     translation[2] /= 1.2;
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Surface_mesh/VSA_wrapper.h 5.1-1/demo/Polyhedron/Plugins/Surface_mesh/VSA_wrapper.h
--- 5.0.3-1/demo/Polyhedron/Plugins/Surface_mesh/VSA_wrapper.h	2020-08-01 19:02:42.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Surface_mesh/VSA_wrapper.h	2020-09-08 12:15:25.000000000 +0000
@@ -38,23 +38,13 @@ class VSA_WRAPPER_EXPORT VSA_wrapper {
   typedef boost::property_map<SMesh, Face_center_tag>::type Face_center_map;
   typedef boost::property_map<SMesh, Face_area_tag>::type Face_area_map;
 
-#ifdef CGAL_LINKED_WITH_TBB
   typedef CGAL::Variational_shape_approximation<SMesh, Vertex_point_map,
-    CGAL::Default, EPICK, CGAL::Parallel_tag> L21_approx;
-#else
-  typedef CGAL::Variational_shape_approximation<SMesh, Vertex_point_map,
-    CGAL::Default, EPICK> L21_approx;
-#endif
+            CGAL::Default, EPICK, CGAL::Parallel_if_available_tag> L21_approx;
   typedef L21_approx::Error_metric L21_metric;
 
   typedef VSA::L2_metric_plane_proxy<SMesh> L2_metric;
-#ifdef CGAL_LINKED_WITH_TBB
-  typedef CGAL::Variational_shape_approximation<SMesh, Vertex_point_map,
-    L2_metric, EPICK, CGAL::Parallel_tag> L2_approx;
-#else
   typedef CGAL::Variational_shape_approximation<SMesh, Vertex_point_map,
-    L2_metric, EPICK> L2_approx;
-#endif
+            L2_metric, EPICK, CGAL::Parallel_if_available_tag> L2_approx;
 
   // user defined point-wise compact metric
   struct Compact_metric_point_proxy {
@@ -89,14 +79,8 @@ class VSA_WRAPPER_EXPORT VSA_wrapper {
   };
   typedef Compact_metric_point_proxy Compact_metric;
 
-#ifdef CGAL_LINKED_WITH_TBB
   typedef CGAL::Variational_shape_approximation<SMesh, Vertex_point_map,
-    Compact_metric, EPICK, CGAL::Parallel_tag> Compact_approx;
-#else
-  typedef CGAL::Variational_shape_approximation<SMesh, Vertex_point_map,
-    Compact_metric, EPICK> Compact_approx;
-#endif
-
+            Compact_metric, EPICK, CGAL::Parallel_if_available_tag> Compact_approx;
 
 public:
   enum Metric { L21, L2, Compact };
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Surface_mesh_deformation/Deform_mesh.ui 5.1-1/demo/Polyhedron/Plugins/Surface_mesh_deformation/Deform_mesh.ui
--- 5.0.3-1/demo/Polyhedron/Plugins/Surface_mesh_deformation/Deform_mesh.ui	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Surface_mesh_deformation/Deform_mesh.ui	2020-09-08 12:12:20.000000000 +0000
@@ -199,7 +199,7 @@
           <item>
            <layout class="QVBoxLayout" name="verticalLayout_6">
             <item>
-             <layout class="QHBoxLayout" name="horizontalLayout_7" stretch="0,1">
+             <layout class="QHBoxLayout" name="horizontalLayout_7" stretch="0,0">
               <item>
                <widget class="QCheckBox" name="remeshingEdgeLengthInput_checkBox">
                 <property name="toolTip">
@@ -214,7 +214,11 @@
                </widget>
               </item>
               <item>
-               <widget class="QDoubleSpinBox" name="remeshing_edge_length_spinbox"/>
+               <widget class="DoubleEdit" name="remeshing_edge_length_spinbox">
+                <property name="text">
+                 <string>0.00</string>
+                </property>
+               </widget>
               </item>
              </layout>
             </item>
@@ -435,6 +439,13 @@
    </layout>
   </widget>
  </widget>
+ <customwidgets>
+  <customwidget>
+   <class>DoubleEdit</class>
+   <extends>QLineEdit</extends>
+   <header>CGAL_double_edit.h</header>
+  </customwidget>
+ </customwidgets>
  <resources/>
  <connections/>
 </ui>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Surface_mesh_deformation/Edit_polyhedron_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Surface_mesh_deformation/Edit_polyhedron_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Surface_mesh_deformation/Edit_polyhedron_plugin.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Surface_mesh_deformation/Edit_polyhedron_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -175,7 +175,8 @@ void Polyhedron_demo_edit_polyhedron_plu
 // what they do is simply transmitting required 'action' to selected scene_edit_polyhedron_item object
 void Polyhedron_demo_edit_polyhedron_plugin::on_AddCtrlVertPushButton_clicked()
 {
-  int item_id = scene->mainSelectionIndex();
+
+  int item_id = scene->selectionIndices().front();
   Scene_edit_polyhedron_item* edit_item = qobject_cast<Scene_edit_polyhedron_item*>(scene->item(item_id));
   if(!edit_item) return;                             // the selected item is not of the right type
 
@@ -183,7 +184,7 @@ void Polyhedron_demo_edit_polyhedron_plu
 }
 void Polyhedron_demo_edit_polyhedron_plugin::on_PrevCtrlVertPushButton_clicked()
 {
-  int item_id = scene->mainSelectionIndex();
+ int item_id = scene->selectionIndices().front();
   Scene_edit_polyhedron_item* edit_item = qobject_cast<Scene_edit_polyhedron_item*>(scene->item(item_id));
   if(!edit_item) return;                             // the selected item is not of the right type
 
@@ -193,7 +194,7 @@ void Polyhedron_demo_edit_polyhedron_plu
 }
 void Polyhedron_demo_edit_polyhedron_plugin::on_NextCtrlVertPushButton_clicked()
 {
-  int item_id = scene->mainSelectionIndex();
+  int item_id = scene->selectionIndices().front();
   Scene_edit_polyhedron_item* edit_item = qobject_cast<Scene_edit_polyhedron_item*>(scene->item(item_id));
   if(!edit_item) return;                             // the selected item is not of the right type
 
@@ -203,7 +204,7 @@ void Polyhedron_demo_edit_polyhedron_plu
 }
 void Polyhedron_demo_edit_polyhedron_plugin::on_SelectAllVerticesPushButton_clicked()
 {
-  int item_id = scene->mainSelectionIndex();
+  int item_id = scene->selectionIndices().front();
   Scene_edit_polyhedron_item* edit_item = qobject_cast<Scene_edit_polyhedron_item*>(scene->item(item_id));
   if(!edit_item) return;                             // the selected item is not of the right type
 
@@ -213,7 +214,7 @@ void Polyhedron_demo_edit_polyhedron_plu
 }
 void Polyhedron_demo_edit_polyhedron_plugin::on_DeleteCtrlVertPushButton_clicked()
 {
-  int item_id = scene->mainSelectionIndex();
+  int item_id = scene->selectionIndices().front();
   Scene_edit_polyhedron_item* edit_item = qobject_cast<Scene_edit_polyhedron_item*>(scene->item(item_id));
   if(!edit_item) return;                             // the selected item is not of the right type
 
@@ -224,7 +225,7 @@ void Polyhedron_demo_edit_polyhedron_plu
 }
 void Polyhedron_demo_edit_polyhedron_plugin::on_ClearROIPushButton_clicked()
 {
-  int item_id = scene->mainSelectionIndex();
+  int item_id = scene->selectionIndices().front();
   Scene_edit_polyhedron_item* edit_item = qobject_cast<Scene_edit_polyhedron_item*>(scene->item(item_id));
   if(!edit_item) return;                             // the selected item is not of the right type
 
@@ -238,7 +239,7 @@ void Polyhedron_demo_edit_polyhedron_plu
 }
 void Polyhedron_demo_edit_polyhedron_plugin::on_DiscardChangesPushButton_clicked()
 {
-  int item_id = scene->mainSelectionIndex();
+  int item_id = scene->selectionIndices().front();
   Scene_edit_polyhedron_item* edit_item = qobject_cast<Scene_edit_polyhedron_item*>(scene->item(item_id));
   if (!edit_item) return;                             // the selected item is not of the right type
 
@@ -297,14 +298,14 @@ void Polyhedron_demo_edit_polyhedron_plu
 }
 void Polyhedron_demo_edit_polyhedron_plugin::on_OverwritePushButton_clicked()
 {
-  int item_id = scene->mainSelectionIndex();
+  int item_id = scene->selectionIndices().front();
   Scene_edit_polyhedron_item* edit_item = qobject_cast<Scene_edit_polyhedron_item*>(scene->item(item_id));
   if(!edit_item) return;                             // the selected item is not of the right type
 
   edit_item->overwrite_deform_object();
 }
 void Polyhedron_demo_edit_polyhedron_plugin::on_Select_isolated_components_button_clicked() {
-  int item_id = scene->mainSelectionIndex();
+  int item_id = scene->selectionIndices().front();
   Scene_edit_polyhedron_item* edit_item = qobject_cast<Scene_edit_polyhedron_item*>(scene->item(item_id));
   if(!edit_item) return;                             // the selected item is not of the right type
 
@@ -316,7 +317,7 @@ void Polyhedron_demo_edit_polyhedron_plu
 }
 
 void Polyhedron_demo_edit_polyhedron_plugin::on_Get_minimum_button_clicked() {
-  int item_id = scene->mainSelectionIndex();
+  int item_id = scene->selectionIndices().front();
   Scene_edit_polyhedron_item* edit_item = qobject_cast<Scene_edit_polyhedron_item*>(scene->item(item_id));
   if(!edit_item) return;                             // the selected item is not of the right type
 
@@ -328,7 +329,7 @@ void Polyhedron_demo_edit_polyhedron_plu
 
 void Polyhedron_demo_edit_polyhedron_plugin::on_SaveROIPushButton_clicked()
 {
-  int item_id = scene->mainSelectionIndex();
+  int item_id = scene->selectionIndices().front();
   Scene_edit_polyhedron_item* edit_item = qobject_cast<Scene_edit_polyhedron_item*>(scene->item(item_id));
   if(!edit_item) return;
 
@@ -340,7 +341,7 @@ void Polyhedron_demo_edit_polyhedron_plu
 }
 void Polyhedron_demo_edit_polyhedron_plugin::on_ReadROIPushButton_clicked()
 {
-  int item_id = scene->mainSelectionIndex();
+  int item_id = scene->selectionIndices().front();
   Scene_edit_polyhedron_item* edit_item = qobject_cast<Scene_edit_polyhedron_item*>(scene->item(item_id));
   if(!edit_item) return;
 
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Surface_mesh_deformation/Scene_edit_polyhedron_item.cpp 5.1-1/demo/Polyhedron/Plugins/Surface_mesh_deformation/Scene_edit_polyhedron_item.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Surface_mesh_deformation/Scene_edit_polyhedron_item.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Surface_mesh_deformation/Scene_edit_polyhedron_item.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -99,7 +99,6 @@ struct Scene_edit_polyhedron_item_priv
   mutable std::vector<GLfloat> color_lines;
   mutable std::vector<GLfloat> ROI_points;
   mutable std::vector<GLfloat> control_points;
-  mutable std::vector<GLfloat> ROI_color;
   mutable std::vector<GLfloat> control_color;
   mutable std::vector<GLfloat> normals;
   mutable std::vector<GLfloat> pos_bbox;
@@ -360,14 +359,11 @@ void Scene_edit_polyhedron_item_priv::co
             {
               CGAL::Color c(0,255,0);
               EPICK::Point_3 point(p.x()+offset.x, p.y()+offset.y, p.z()+offset.z);
-              spheres->add_sphere(EPICK::Sphere_3(point, length_of_axis/15.0*length_of_axis/15.0), c);
+              spheres->add_sphere(EPICK::Sphere_3(point, length_of_axis/15.0*length_of_axis/15.0), 0, c);
             }
         }
 
     }
-    ROI_color.assign(ROI_points.size(),0);
-    for(std::size_t i=0; i<ROI_color.size()/3; i++)
-      ROI_color[3*i+1]=1.0;
 
     for(typename std::list<Control_vertices_data<Mesh> >::const_iterator hgb_data = fs.get_ctrl_vertex_frame_map(mesh).begin(); hgb_data != fs.get_ctrl_vertex_frame_map(mesh).end(); ++hgb_data)
     {
@@ -400,7 +396,7 @@ void Scene_edit_polyhedron_item_priv::co
               EPICK::Point_3 center(p.x()+offset.x,
                                      p.y()+offset.y,
                                      p.z()+offset.z);
-              spheres_ctrl->add_sphere(EPICK::Sphere_3(center, length_of_axis/15.0*length_of_axis/15.0), c);
+              spheres_ctrl->add_sphere(EPICK::Sphere_3(center, length_of_axis/15.0*length_of_axis/15.0), 0, c);
             }
         }
     }
@@ -889,7 +885,7 @@ bool Scene_edit_polyhedron_item::eventFi
      &&d->state.shift_pressing)
   {
     QWheelEvent *w_event = static_cast<QWheelEvent*>(event);
-    int steps = w_event->delta() / 120;
+    int steps = w_event->angleDelta().y() / 120;
     d->expand_or_reduce(steps, d->sm_item->polyhedron());
   }
   if(event->type() == QEvent::MouseButtonPress || event->type() == QEvent::MouseButtonRelease)
@@ -1273,7 +1269,7 @@ void Scene_edit_polyhedron_item::ShowAsS
   {
     if(!d->spheres)
     {
-      d->spheres = new Scene_spheres_item(this, false);
+      d->spheres = new Scene_spheres_item(this, 0, false, false);
       d->spheres->setName("ROI spheres");
       d->spheres->setRenderingMode(Gouraud);
       connect(d->spheres, SIGNAL(destroyed()), this, SLOT(reset_spheres()));
@@ -1285,7 +1281,7 @@ void Scene_edit_polyhedron_item::ShowAsS
     }
     if(!d->spheres_ctrl)
     {
-      d->spheres_ctrl = new Scene_spheres_item(this, false);
+      d->spheres_ctrl = new Scene_spheres_item(this, false, false);
       d->spheres_ctrl->setName("Control spheres");
       d->spheres_ctrl->setRenderingMode(Gouraud);
       connect(d->spheres_ctrl, &QObject::destroyed, this, Reset_spheres_ctrl(d) );
@@ -1778,6 +1774,10 @@ Scene_surface_mesh_item* Scene_edit_poly
 void Scene_edit_polyhedron_item::computeElements() const
 {
   d->compute_normals_and_vertices(sm_item()->face_graph());
+  if(d->spheres)
+    d->spheres->computeElements();
+  if(d->spheres_ctrl)
+    d->spheres_ctrl->computeElements();
   std::vector<GLfloat> vertices;
   std::vector<GLfloat> *vertices_ptr;
   const CGAL::qglviewer::Vec offset = Three::mainViewer()->offset();
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Tetrahedral_remeshing/CMakeLists.txt 5.1-1/demo/Polyhedron/Plugins/Tetrahedral_remeshing/CMakeLists.txt
--- 5.0.3-1/demo/Polyhedron/Plugins/Tetrahedral_remeshing/CMakeLists.txt	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Tetrahedral_remeshing/CMakeLists.txt	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,12 @@
+include( polyhedron_demo_macros )
+
+remove_definitions(-DQT_STATICPLUGIN)
+
+qt5_wrap_cpp( VOLUME_MOC_OUTFILES ${CMAKE_CURRENT_SOURCE_DIR}/Volume_plane_thread.h )
+qt5_wrap_cpp( VOLUME_MOC_OUTFILES ${CMAKE_CURRENT_SOURCE_DIR}/Volume_plane_interface.h )
+
+qt5_wrap_ui( tetRemeshingUI_FILES  Tetrahedral_remeshing_dialog.ui)
+polyhedron_demo_plugin(tetrahedral_remeshing_plugin Tetrahedral_remeshing_plugin ${tetRemeshingUI_FILES}
+                       KEYWORDS Tetrahedral_remeshing)
+target_link_libraries(tetrahedral_remeshing_plugin
+                                  PUBLIC scene_c3t3_item ${OPENGL_gl_LIBRARY})
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Tetrahedral_remeshing/Tetrahedral_remeshing_dialog.ui 5.1-1/demo/Polyhedron/Plugins/Tetrahedral_remeshing/Tetrahedral_remeshing_dialog.ui
--- 5.0.3-1/demo/Polyhedron/Plugins/Tetrahedral_remeshing/Tetrahedral_remeshing_dialog.ui	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Tetrahedral_remeshing/Tetrahedral_remeshing_dialog.ui	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,220 @@
+<?xml version="1.0" encoding="UTF-8"?>
+<ui version="4.0">
+ <class>Tetrahedral_remeshing_dialog</class>
+ <widget class="QDialog" name="Tetrahedral_remeshing_dialog">
+  <property name="enabled">
+   <bool>true</bool>
+  </property>
+  <property name="geometry">
+   <rect>
+    <x>0</x>
+    <y>0</y>
+    <width>376</width>
+    <height>259</height>
+   </rect>
+  </property>
+  <property name="windowTitle">
+   <string>Isotropic remeshing criteria</string>
+  </property>
+  <layout class="QVBoxLayout" name="verticalLayout">
+   <item>
+    <widget class="QLabel" name="objectName">
+     <property name="font">
+      <font>
+       <pointsize>15</pointsize>
+       <weight>75</weight>
+       <bold>true</bold>
+      </font>
+     </property>
+     <property name="text">
+      <string>NO OBJECT</string>
+     </property>
+    </widget>
+   </item>
+   <item>
+    <widget class="QLabel" name="objectNameSize">
+     <property name="text">
+      <string>No size</string>
+     </property>
+    </widget>
+   </item>
+   <item>
+    <widget class="QGroupBox" name="groupBox">
+     <property name="title">
+      <string>Tetrahedral remeshing</string>
+     </property>
+     <layout class="QGridLayout" name="gridLayout_2" rowstretch="0,0,0,0,0" columnstretch="0,0">
+      <item row="1" column="1">
+       <widget class="QSpinBox" name="nbIterations_spinbox">
+        <property name="minimumSize">
+         <size>
+          <width>110</width>
+          <height>0</height>
+         </size>
+        </property>
+        <property name="value">
+         <number>1</number>
+        </property>
+       </widget>
+      </item>
+      <item row="3" column="1">
+       <widget class="QCheckBox" name="protect_checkbox">
+        <property name="text">
+         <string>Protect boundaries</string>
+        </property>
+        <property name="checked">
+         <bool>false</bool>
+        </property>
+       </widget>
+      </item>
+      <item row="1" column="0">
+       <widget class="QLabel" name="nbIterations_label">
+        <property name="text">
+         <string>Number of Main iterations</string>
+        </property>
+        <property name="alignment">
+         <set>Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter</set>
+        </property>
+        <property name="buddy">
+         <cstring>nbIterations_spinbox</cstring>
+        </property>
+       </widget>
+      </item>
+      <item row="2" column="0">
+       <spacer name="verticalSpacer_3">
+        <property name="orientation">
+         <enum>Qt::Vertical</enum>
+        </property>
+        <property name="sizeType">
+         <enum>QSizePolicy::Maximum</enum>
+        </property>
+        <property name="sizeHint" stdset="0">
+         <size>
+          <width>20</width>
+          <height>40</height>
+         </size>
+        </property>
+       </spacer>
+      </item>
+      <item row="0" column="1">
+       <widget class="QDoubleSpinBox" name="edgeLength_dspinbox">
+        <property name="minimumSize">
+         <size>
+          <width>110</width>
+          <height>0</height>
+         </size>
+        </property>
+        <property name="maximum">
+         <double>1000.000000000000000</double>
+        </property>
+        <property name="singleStep">
+         <double>0.100000000000000</double>
+        </property>
+       </widget>
+      </item>
+      <item row="0" column="0">
+       <widget class="QLabel" name="edgeLength_label">
+        <property name="text">
+         <string>Target edge length</string>
+        </property>
+        <property name="alignment">
+         <set>Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter</set>
+        </property>
+        <property name="buddy">
+         <cstring>edgeLength_dspinbox</cstring>
+        </property>
+       </widget>
+      </item>
+      <item row="4" column="1">
+       <widget class="QCheckBox" name="smoothEdges_checkBox">
+        <property name="toolTip">
+         <string extracomment="Smoothing of constrained polylines.\nWarning : this may reduce quality of dihedral angles."/>
+        </property>
+        <property name="text">
+         <string>Smooth constrained edges</string>
+        </property>
+       </widget>
+      </item>
+      <item row="3" column="0">
+       <widget class="QLabel" name="options_label">
+        <property name="text">
+         <string>Options :</string>
+        </property>
+        <property name="alignment">
+         <set>Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter</set>
+        </property>
+       </widget>
+      </item>
+     </layout>
+    </widget>
+   </item>
+   <item>
+    <spacer name="verticalSpacer_2">
+     <property name="orientation">
+      <enum>Qt::Vertical</enum>
+     </property>
+     <property name="sizeType">
+      <enum>QSizePolicy::MinimumExpanding</enum>
+     </property>
+     <property name="sizeHint" stdset="0">
+      <size>
+       <width>0</width>
+       <height>0</height>
+      </size>
+     </property>
+    </spacer>
+   </item>
+   <item>
+    <widget class="QDialogButtonBox" name="buttonBox">
+     <property name="orientation">
+      <enum>Qt::Horizontal</enum>
+     </property>
+     <property name="standardButtons">
+      <set>QDialogButtonBox::Cancel|QDialogButtonBox::Ok</set>
+     </property>
+    </widget>
+   </item>
+  </layout>
+ </widget>
+ <tabstops>
+  <tabstop>edgeLength_dspinbox</tabstop>
+  <tabstop>nbIterations_spinbox</tabstop>
+  <tabstop>protect_checkbox</tabstop>
+  <tabstop>buttonBox</tabstop>
+ </tabstops>
+ <resources/>
+ <connections>
+  <connection>
+   <sender>buttonBox</sender>
+   <signal>accepted()</signal>
+   <receiver>Tetrahedral_remeshing_dialog</receiver>
+   <slot>accept()</slot>
+   <hints>
+    <hint type="sourcelabel">
+     <x>397</x>
+     <y>333</y>
+    </hint>
+    <hint type="destinationlabel">
+     <x>157</x>
+     <y>195</y>
+    </hint>
+   </hints>
+  </connection>
+  <connection>
+   <sender>buttonBox</sender>
+   <signal>rejected()</signal>
+   <receiver>Tetrahedral_remeshing_dialog</receiver>
+   <slot>reject()</slot>
+   <hints>
+    <hint type="sourcelabel">
+     <x>397</x>
+     <y>333</y>
+    </hint>
+    <hint type="destinationlabel">
+     <x>286</x>
+     <y>195</y>
+    </hint>
+   </hints>
+  </connection>
+ </connections>
+</ui>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Tetrahedral_remeshing/Tetrahedral_remeshing_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Tetrahedral_remeshing/Tetrahedral_remeshing_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Tetrahedral_remeshing/Tetrahedral_remeshing_plugin.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Tetrahedral_remeshing/Tetrahedral_remeshing_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,222 @@
+#define CGAL_TETRAHEDRAL_REMESHING_VERBOSE
+//#define CGAL_DUMP_REMESHING_STEPS
+//#define CGAL_TETRAHEDRAL_REMESHING_DEBUG
+//#define CGAL_TETRAHEDRAL_REMESHING_VERBOSE_PROGRESS
+//#define CGAL_TETRAHEDRAL_REMESHING_PROFILE
+
+#include <QtCore/qglobal.h>
+
+#include <CGAL/Three/Polyhedron_demo_plugin_interface.h>
+
+#include "Scene_c3t3_item.h"
+#include "C3t3_type.h"
+
+#include <CGAL/tetrahedral_remeshing.h>
+
+#include <unordered_map>
+#include <memory>
+
+#include <QElapsedTimer>
+#include <QAction>
+#include <QMainWindow>
+#include <QApplication>
+#include <QString>
+#include <QInputDialog>
+#include <QtPlugin>
+#include <QMessageBox>
+
+#include "ui_Tetrahedral_remeshing_dialog.h"
+
+#ifdef CGAL_DEBUG_TET_REMESHING_IN_PLUGIN
+std::size_t nb_topology_test = 0;
+std::size_t nb_impossible = 0;
+std::size_t nb_valid_collapse = 0;
+std::size_t nb_invalid_collapse = 0;
+std::size_t nb_invalid_lengths = 0;
+std::size_t nb_invalid_collapse_short = 0;
+std::size_t nb_orientation_v0 = 0;
+std::size_t nb_orientation_v1 = 0;
+std::size_t nb_orientation_midpoint = 0;
+std::size_t nb_test_v0 = 0;
+std::size_t nb_test_v1 = 0;
+std::size_t nb_test_midpoint = 0;
+#endif
+
+using namespace CGAL::Three;
+class Polyhedron_demo_tetrahedral_remeshing_plugin :
+  public QObject,
+  public Polyhedron_demo_plugin_interface
+{
+  Q_OBJECT
+  Q_INTERFACES(CGAL::Three::Polyhedron_demo_plugin_interface)
+  Q_PLUGIN_METADATA(IID "com.geometryfactory.PolyhedronDemo.PluginInterface/1.0" FILE "tetrahedral_remeshing_plugin.json")
+
+public:
+  void init(QMainWindow* mainWindow, Scene_interface* scene_interface, Messages_interface*)
+  {
+    this->scene = scene_interface;
+    this->mw = mainWindow;
+
+    actionTetrahedralRemeshing_ = new QAction("Tetrahedral Remeshing", mw);
+    if (actionTetrahedralRemeshing_) {
+      connect(actionTetrahedralRemeshing_, SIGNAL(triggered()),
+        this, SLOT(tetrahedral_remeshing()));
+    }
+  }
+
+  QList<QAction*> actions() const {
+    return QList<QAction*>() << actionTetrahedralRemeshing_;
+  }
+
+  bool applicable(QAction*) const
+  {
+    return qobject_cast<Scene_c3t3_item*>(scene->item(scene->mainSelectionIndex()));
+  }
+
+public Q_SLOTS:
+  void tetrahedral_remeshing()
+  {
+    const Scene_interface::Item_id index = scene->mainSelectionIndex();
+
+    Scene_c3t3_item* c3t3_item =
+      qobject_cast<Scene_c3t3_item*>(scene->item(index));
+
+    if (c3t3_item)
+    {
+      // Create dialog box
+      QDialog dialog(mw);
+      Ui::Tetrahedral_remeshing_dialog ui
+        = tet_remeshing_dialog(&dialog, c3t3_item);
+
+      // Get values
+      int i = dialog.exec();
+      if (i == QDialog::Rejected)
+      {
+        std::cout << "Remeshing aborted" << std::endl;
+        return;
+      }
+      double target_length = ui.edgeLength_dspinbox->value();
+      unsigned int nb_iter = ui.nbIterations_spinbox->value();
+      bool protect = ui.protect_checkbox->isChecked();
+      bool smooth_edges = ui.smoothEdges_checkBox->isChecked();
+
+      // wait cursor
+      QApplication::setOverrideCursor(Qt::WaitCursor);
+
+      QElapsedTimer time;
+      time.start();
+
+      CGAL::tetrahedral_isotropic_remeshing(
+          c3t3_item->c3t3(),
+          target_length,
+          CGAL::parameters::remesh_boundaries(!protect)
+          .number_of_iterations(nb_iter)
+          .smooth_constrained_edges(smooth_edges));
+
+      std::cout << "Remeshing done (" << time.elapsed() << " ms)" << std::endl;
+
+      c3t3_item->invalidateOpenGLBuffers();
+      c3t3_item->c3t3_changed();
+      this->scene->itemChanged(index);
+
+      // default cursor
+      QApplication::restoreOverrideCursor();
+
+#ifdef CGAL_DEBUG_TET_REMESHING_IN_PLUGIN
+      std::cout << "nb_topology_test = " << nb_topology_test << std::endl;
+      std::cout << "nb_impossible = " << nb_impossible << std::endl;
+
+      std::cout << "nb_invalid_collapse_short = " << nb_invalid_collapse_short << std::endl;
+
+      std::cout << "nb_orientation_fail_v0 = " << nb_orientation_v0 << std::endl;
+      std::cout << "nb_orientation_fail_v1 = " << nb_orientation_v1 << std::endl;
+      std::cout << "nb_orientation_fail_midpoint = " << nb_orientation_midpoint << std::endl;
+
+      std::cout << "nb_test_v0 = " << nb_test_v0 << std::endl;
+      std::cout << "nb_test_v1 = " << nb_test_v1 << std::endl;
+      std::cout << "nb_test_midpoint = " << nb_test_midpoint << std::endl;
+      std::cout << std::endl;
+
+      if (nb_test_v0 > 0)
+        std::cout << "nb_orientation_v0 / nb_test_v0  = "
+        << ((float)nb_orientation_v0 / (float)nb_test_v0) << std::endl;
+      if (nb_test_v1 > 1)
+        std::cout << "nb_orientation_v1 / nb_test_v1  = "
+        << ((float)nb_orientation_v1 / (float)nb_test_v1) << std::endl;
+      if (nb_test_midpoint > 0)
+        std::cout << "nb_orientation_midpoint / nb_test_midpoint  = "
+        << ((float)nb_orientation_midpoint / (float)nb_test_midpoint) << std::endl;
+
+      std::cout << std::endl;
+      std::cout << "nb_valid_collapse = " << nb_valid_collapse << std::endl;
+      std::cout << "nb_invalid_collapse = " << nb_invalid_collapse << std::endl;
+      std::cout << "nb_invalid_lengths = " << nb_invalid_lengths << std::endl;
+#endif
+    }
+    else
+    {
+      std::cout << "Can't remesh that type of thing" << std::endl;
+    }
+  }
+
+private:
+  Scene_interface *scene;
+  QMainWindow* mw;
+
+  Ui::Tetrahedral_remeshing_dialog
+  tet_remeshing_dialog(QDialog* dialog,
+                       Scene_c3t3_item* c3t3_item)
+  {
+    Ui::Tetrahedral_remeshing_dialog ui;
+    ui.setupUi(dialog);
+    connect(ui.buttonBox, SIGNAL(accepted()), dialog, SLOT(accept()));
+    connect(ui.buttonBox, SIGNAL(rejected()), dialog, SLOT(reject()));
+
+    //Set default parameters
+    Scene_interface::Bbox bbox = c3t3_item->bbox();
+    ui.objectName->setText(c3t3_item->name());
+
+    ui.objectNameSize->setText(
+      tr("Object bbox size (w,h,d):  <b>%1</b>,  <b>%2</b>,  <b>%3</b>")
+      .arg(bbox.xmax()-bbox.xmin(), 0, 'g', 3)
+      .arg(bbox.ymax()-bbox.ymin(), 0, 'g', 3)
+      .arg(bbox.zmax()-bbox.zmin(), 0, 'g', 3));
+
+    double diago_length = CGAL::sqrt((bbox.xmax()-bbox.xmin())*(bbox.xmax()-bbox.xmin())
+                                   + (bbox.ymax()-bbox.ymin())*(bbox.ymax()-bbox.ymin())
+                                   + (bbox.zmax()-bbox.zmin())*(bbox.zmax()-bbox.zmin()));
+    double log = std::log10(diago_length);
+    unsigned int nb_decimals = (log > 0) ? 5 : (std::ceil(-log)+3);
+
+    ui.edgeLength_dspinbox->setDecimals(nb_decimals);
+    ui.edgeLength_dspinbox->setSingleStep(1e-3);
+    ui.edgeLength_dspinbox->setRange(1e-6 * diago_length, //min
+                                     2.   * diago_length);//max
+    ui.edgeLength_dspinbox->setValue(0.05 * diago_length);
+
+    std::ostringstream oss;
+    oss << "Diagonal length of the Bbox of the triangulation to remesh is ";
+    oss << diago_length << "." << std::endl;
+    oss << "Default is 5% of it" << std::endl;
+    ui.edgeLength_dspinbox->setToolTip(QString::fromStdString(oss.str()));
+
+    ui.nbIterations_spinbox->setSingleStep(1);
+    ui.nbIterations_spinbox->setRange(1/*min*/, 1000/*max*/);
+    ui.nbIterations_spinbox->setValue(1);
+
+    ui.protect_checkbox->setChecked(false);
+    ui.smoothEdges_checkBox->setChecked(false);
+
+    connect(ui.protect_checkbox, SIGNAL(toggled(bool)),
+            ui.smoothEdges_checkBox, SLOT(setDisabled(bool)));
+
+    return ui;
+  }
+
+
+private:
+  QAction* actionTetrahedralRemeshing_;
+
+}; // end Polyhedron_demo_tetrahedral_remeshing_plugin
+
+#include "Tetrahedral_remeshing_plugin.moc"
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Three_examples/Basic_dialog.ui 5.1-1/demo/Polyhedron/Plugins/Three_examples/Basic_dialog.ui
--- 5.0.3-1/demo/Polyhedron/Plugins/Three_examples/Basic_dialog.ui	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Three_examples/Basic_dialog.ui	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,91 @@
+<?xml version="1.0" encoding="UTF-8"?>
+<ui version="4.0">
+ <class>BasicDialog</class>
+ <widget class="QDialog" name="BasicDialog">
+  <property name="geometry">
+   <rect>
+    <x>0</x>
+    <y>0</y>
+    <width>400</width>
+    <height>300</height>
+   </rect>
+  </property>
+  <property name="windowTitle">
+   <string>Dialog</string>
+  </property>
+  <layout class="QVBoxLayout" name="verticalLayout_2">
+   <item>
+    <layout class="QHBoxLayout" name="horizontalLayout">
+     <item>
+      <widget class="QLabel" name="label">
+       <property name="text">
+        <string>Number to display :</string>
+       </property>
+      </widget>
+     </item>
+     <item>
+      <spacer name="horizontalSpacer">
+       <property name="orientation">
+        <enum>Qt::Horizontal</enum>
+       </property>
+       <property name="sizeHint" stdset="0">
+        <size>
+         <width>40</width>
+         <height>20</height>
+        </size>
+       </property>
+      </spacer>
+     </item>
+     <item>
+      <widget class="QLineEdit" name="lineEdit"/>
+     </item>
+    </layout>
+   </item>
+   <item>
+    <widget class="QDialogButtonBox" name="buttonBox">
+     <property name="orientation">
+      <enum>Qt::Horizontal</enum>
+     </property>
+     <property name="standardButtons">
+      <set>QDialogButtonBox::Cancel|QDialogButtonBox::Ok</set>
+     </property>
+    </widget>
+   </item>
+  </layout>
+ </widget>
+ <resources/>
+ <connections>
+  <connection>
+   <sender>buttonBox</sender>
+   <signal>accepted()</signal>
+   <receiver>BasicDialog</receiver>
+   <slot>accept()</slot>
+   <hints>
+    <hint type="sourcelabel">
+     <x>248</x>
+     <y>254</y>
+    </hint>
+    <hint type="destinationlabel">
+     <x>157</x>
+     <y>274</y>
+    </hint>
+   </hints>
+  </connection>
+  <connection>
+   <sender>buttonBox</sender>
+   <signal>rejected()</signal>
+   <receiver>BasicDialog</receiver>
+   <slot>reject()</slot>
+   <hints>
+    <hint type="sourcelabel">
+     <x>316</x>
+     <y>260</y>
+    </hint>
+    <hint type="destinationlabel">
+     <x>286</x>
+     <y>274</y>
+    </hint>
+   </hints>
+  </connection>
+ </connections>
+</ui>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Three_examples/Basic_dock_widget.ui 5.1-1/demo/Polyhedron/Plugins/Three_examples/Basic_dock_widget.ui
--- 5.0.3-1/demo/Polyhedron/Plugins/Three_examples/Basic_dock_widget.ui	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Three_examples/Basic_dock_widget.ui	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,61 @@
+<?xml version="1.0" encoding="UTF-8"?>
+<ui version="4.0">
+ <class>BasicDockWidget</class>
+ <widget class="QDockWidget" name="BasicDockWidget">
+  <property name="geometry">
+   <rect>
+    <x>0</x>
+    <y>0</y>
+    <width>392</width>
+    <height>302</height>
+   </rect>
+  </property>
+  <property name="windowTitle">
+   <string>DockWidget</string>
+  </property>
+  <widget class="QWidget" name="dockWidgetContents">
+   <layout class="QHBoxLayout" name="horizontalLayout_2">
+    <item>
+     <layout class="QVBoxLayout" name="verticalLayout">
+      <item>
+       <layout class="QHBoxLayout" name="horizontalLayout">
+        <item>
+         <widget class="QLabel" name="label">
+          <property name="text">
+           <string>Number to display :</string>
+          </property>
+         </widget>
+        </item>
+        <item>
+         <spacer name="horizontalSpacer">
+          <property name="orientation">
+           <enum>Qt::Horizontal</enum>
+          </property>
+          <property name="sizeHint" stdset="0">
+           <size>
+            <width>40</width>
+            <height>20</height>
+           </size>
+          </property>
+         </spacer>
+        </item>
+        <item>
+         <widget class="QSpinBox" name="spinBox"/>
+        </item>
+       </layout>
+      </item>
+      <item>
+       <widget class="QPushButton" name="pushButton">
+        <property name="text">
+         <string>Print</string>
+        </property>
+       </widget>
+      </item>
+     </layout>
+    </item>
+   </layout>
+  </widget>
+ </widget>
+ <resources/>
+ <connections/>
+</ui>
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Three_examples/Basic_item_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Three_examples/Basic_item_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Three_examples/Basic_item_plugin.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Three_examples/Basic_item_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,87 @@
+#include <CGAL/Three/Polyhedron_demo_plugin_helper.h>
+#include <QApplication>
+#include <QObject>
+#include <QAction>
+#include <QMainWindow>
+#include <QInputDialog>
+#include "CGAL/Three/Scene_group_item.h"
+#include "CGAL/Three/Three.h"
+#include "Scene_plane_item.h"
+
+//This plugin crates an action in Operations that displays the name of the selected item,
+//adds a scene_plane_item to the scene, and adds the selected item and the plane to a new group.
+class BasicItemPlugin :
+    public QObject,
+    public CGAL::Three::Polyhedron_demo_plugin_interface
+{
+  Q_OBJECT
+  Q_INTERFACES(CGAL::Three::Polyhedron_demo_plugin_interface)
+  Q_PLUGIN_METADATA(IID "com.geometryfactory.PolyhedronDemo.PluginInterface/1.0")
+public:
+  //! [applicable]
+  //This plugin is only applicable if there is exactly one selected item.
+  bool applicable(QAction*) const Q_DECL_OVERRIDE
+  {
+    return scene->selectionIndices().size() ==1;
+  }
+  //! [applicable]
+  //the list of the actions of the plugin.
+  QList<QAction*> actions() const Q_DECL_OVERRIDE
+  {
+    return _actions;
+  }
+  //this acts like a constructor for the plugin. It gets the references to the mainwindow and the scene, and connects the action.
+  void init(QMainWindow* mw, CGAL::Three::Scene_interface* sc, Messages_interface*) Q_DECL_OVERRIDE
+  {
+    //get the references
+    this->scene = sc;
+    this->mw = mw;
+
+    //creates the action
+    QAction *actionHelloWorld= new QAction(QString("Create a group"), mw);
+    //specifies the subMenu
+    actionHelloWorld->setProperty("submenuName", "Basic");
+    //links the action
+    if(actionHelloWorld) {
+      connect(actionHelloWorld, SIGNAL(triggered()),
+              this, SLOT(helloWorld()));
+      _actions << actionHelloWorld;
+    }
+  }
+private Q_SLOTS:
+
+
+  void helloWorld()
+  { //! [use]
+    //get a reference to the selected item.
+    CGAL::Three::Scene_item *item = scene->item(scene->mainSelectionIndex());
+    CGAL::Three::Three::information(QString("The selected item's name is  : %1").arg(item->name()));
+    //! [use]
+    //! [additem]
+    //creates a plane item
+    Scene_plane_item *new_item = new Scene_plane_item(scene);
+    new_item->setName("Trivial Plane");
+    new_item->setColor(Qt::blue);
+    new_item->setNormal(0.0,0.0,1.0);
+    scene->addItem(new_item);
+    //! [additem]
+    //! [group]
+    //Create a new group
+    Scene_group_item *group = new Scene_group_item("New group");
+    //add it to the scene
+    scene->addItem(group);
+    //Then give it its children
+    scene->changeGroup(item, group);
+    scene->changeGroup(new_item,group);
+    //! [group]
+  }
+
+private:
+  QList<QAction*> _actions;
+  //The reference to the scene
+  CGAL::Three::Scene_interface* scene;
+  //The reference to the main window
+  QMainWindow* mw;
+};
+
+#include "Basic_item_plugin.moc"
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Three_examples/Basic_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Three_examples/Basic_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Three_examples/Basic_plugin.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Three_examples/Basic_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,138 @@
+/*
+Change the value of EXAMPLE_COMPLEXITY in the first line to change the behavior :
+  - 0 : prints "Hello World!" in the Info and console widgets
+  - 1 : pops-up a simple dialog asking to enter an integer , then prints it in the Info and console widgets
+  - 2 : pops-up a little more elaborated dialog asking to enter an integer , then prints it in the Info and console widgets if it was indeed an integer, else pops-up an error message box.
+  */
+#define EXAMPLE_COMPLEXITY 0
+#include "ui_Basic_dialog.h"
+//! [headers_plugin]
+#include <CGAL/Three/Polyhedron_demo_plugin_interface.h>
+#include <QApplication>
+#include <QObject>
+#include <QAction>
+#include <QMainWindow>
+#include <QInputDialog>
+#include <QMessageBox>
+#include "CGAL/Three/Three.h"
+//! [headers_plugin]
+//! [dialog_plugin]
+class ComplexDialog :
+    public QDialog,
+    public Ui::BasicDialog
+{
+  Q_OBJECT
+public:
+  ComplexDialog(QWidget* =0)
+  {
+    setupUi(this);
+
+  }
+
+};
+//! [dialog_plugin]
+//! [opening_plugin]
+
+//This plugin creates an action in Operations depending on EXAMPLE_COMPLEXITY.
+class BasicPlugin :
+    public QObject,
+    public CGAL::Three::Polyhedron_demo_plugin_interface
+{
+  Q_OBJECT
+  Q_INTERFACES(CGAL::Three::Polyhedron_demo_plugin_interface)
+  Q_PLUGIN_METADATA(IID "com.geometryfactory.PolyhedronDemo.PluginInterface/1.0")
+public:
+  //decides if the plugin's actions will be displayed or not.
+  bool applicable(QAction*) const Q_DECL_OVERRIDE
+  {
+    return true;
+  }
+  //the list of the actions of the plugin.
+  QList<QAction*> actions() const Q_DECL_OVERRIDE
+  {
+    return _actions;
+  }
+  //this acts like a constructor for the plugin. It gets the references to the main window and the scene, and connects the action.
+  void init(QMainWindow* mainWindow, CGAL::Three::Scene_interface* sc, Messages_interface*) Q_DECL_OVERRIDE
+  {
+    //get the references
+    this->scene = sc;
+    this->mw = mainWindow;
+
+    //creates the action
+    QAction *actionHelloWorld= new QAction(QString("Hello World"), mw);
+    //specifies the subMenu
+    actionHelloWorld->setProperty("submenuName", "Basic");
+    //links the action
+    if(actionHelloWorld) {
+      connect(actionHelloWorld, SIGNAL(triggered()),
+              this, SLOT(helloWorld()));
+      _actions << actionHelloWorld;
+    }
+  }
+private Q_SLOTS:
+
+  //! [opening_plugin]
+#if EXAMPLE_COMPLEXITY == 0
+  //! [basic_plugin]
+  void helloWorld()
+  {
+    CGAL::Three::Three::information(QString("Hello World!"));
+  }
+  //! [basic_plugin]
+#elif EXAMPLE_COMPLEXITY == 1
+  //! [basic_dialog_plugin]
+  void helloWorld()
+  {
+    bool ok = false;
+    const unsigned int parameter =
+        QInputDialog::getInt((QWidget*)mw,
+                             tr("Hello World"), // dialog title
+                             tr("Hello dear user! What integer would you want me to display for you ? "), // field label
+                             10, // default value = fast
+                             0, // min
+                             100, // max
+                             1, // step
+                             &ok);
+    if(!ok) return;
+    messageInterface->information(QString("You asked me to display %1, so here it is : %1").arg(parameter));
+  }
+  //! [basic_dialog_plugin]
+#elif EXAMPLE_COMPLEXITY == 2
+  //! [complex_dialog_plugin]
+  void helloWorld()
+  {
+    //creates a new dialog
+    ComplexDialog *dialog = new ComplexDialog();
+    //opens the dialog
+    if(!dialog->exec())
+      return;
+    //! [warningbox]
+    QString result = dialog->lineEdit->text();
+    bool ok = false;
+    int int_res = result.toInt(&ok);
+    if(!ok)
+    {
+      QMessageBox::warning(mw,
+                           "ERROR",
+                           tr("This is not an integer !")
+                           );
+      return;
+    }
+    //! [warningbox]
+    messageInterface->information(QString("You asked me to display %1, so here it is : %1").arg(int_res));
+  }
+  //! [complex_dialog_plugin]
+#endif
+  //! [ending_plugin]
+private:
+  QList<QAction*> _actions;
+  //The reference to the scene
+  CGAL::Three::Scene_interface* scene;
+  //The reference to the main window
+  QMainWindow* mw;
+};
+
+#include "Basic_plugin.moc"
+//! [ending_plugin]
+
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Three_examples/CMakeLists.txt 5.1-1/demo/Polyhedron/Plugins/Three_examples/CMakeLists.txt
--- 5.0.3-1/demo/Polyhedron/Plugins/Three_examples/CMakeLists.txt	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Three_examples/CMakeLists.txt	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,36 @@
+cmake_minimum_required(VERSION 3.1...3.15)
+project( Three_examples )
+
+if(NOT POLICY CMP0070 AND POLICY CMP0053)
+  # Only set CMP0053 to OLD with CMake<3.10, otherwise there is a warning.
+  cmake_policy(SET CMP0053 OLD)
+endif()
+# Find includes in corresponding build directories
+set(CMAKE_INCLUDE_CURRENT_DIR ON)
+# Instruct CMake to run moc automatically when needed.
+set(CMAKE_AUTOMOC ON)
+#Find CGAL
+find_package(CGAL COMPONENTS Qt5 ImageIO)
+# Find Qt5 itself
+find_package(Qt5
+             QUIET
+             COMPONENTS            OpenGL Script Svg Xml
+             OPTIONAL_COMPONENTS   ScriptTools WebSockets)
+
+if(RUNNING_CGAL_AUTO_TEST)
+  if(Qt5_FOUND AND CGAL_FOUND)
+    include(${CGAL_USE_FILE})
+  endif()
+  polyhedron_demo_plugin(example_plugin Example_plugin)
+
+  qt5_wrap_ui( basicUI_FILES Basic_dialog.ui  )
+  polyhedron_demo_plugin(basic_plugin Basic_plugin ${basicUI_FILES})
+
+  qt5_wrap_ui( dockUI_FILES Basic_dock_widget.ui )
+  polyhedron_demo_plugin(dock_widget_plugin Dock_widget_plugin ${dockUI_FILES})
+
+  polyhedron_demo_plugin(basic_item_plugin Basic_item_plugin)
+
+#  links the library containing the scene_plane_item with the plugin
+  target_link_libraries(basic_item_plugin PUBLIC scene_basic_objects)
+endif()
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Three_examples/Dock_widget_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Three_examples/Dock_widget_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Three_examples/Dock_widget_plugin.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Three_examples/Dock_widget_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,99 @@
+#include "ui_Basic_dock_widget.h"
+#include <CGAL/Three/Polyhedron_demo_plugin_helper.h>
+#include <QApplication>
+#include <QObject>
+#include <QAction>
+#include <QMainWindow>
+#include <CGAL/Three/Three.h>
+
+//! [dock]
+class DockWidget :
+    public QDockWidget,
+    public Ui::BasicDockWidget
+{
+public:
+  DockWidget(QString name, QWidget *parent)
+    :QDockWidget(name,parent)
+  {
+   setupUi(this);
+  }
+};
+//! [dock]
+//This plugin crates an action in Operations that creates a DOckWidget to display a number in the 'console' dockwidet.
+class BasicPlugin :
+    public QObject,
+    public CGAL::Three::Polyhedron_demo_plugin_helper
+{
+  Q_OBJECT
+  Q_INTERFACES(CGAL::Three::Polyhedron_demo_plugin_interface)
+  Q_PLUGIN_METADATA(IID "com.geometryfactory.PolyhedronDemo.PluginInterface/1.0")
+public:
+  //decides if the plugin's actions will be displayed or not.
+  bool applicable(QAction*) const Q_DECL_OVERRIDE
+  {
+    return true;
+  }
+  //the list of the actions of the plugin.
+  QList<QAction*> actions() const Q_DECL_OVERRIDE
+  {
+    return _actions;
+  }
+  //! [init]
+  //this acts like a constructor for the plugin. It gets the references to the mainwindow and the scene, and connects the action.
+  void init(QMainWindow* mw, CGAL::Three::Scene_interface* sc, Messages_interface*) Q_DECL_OVERRIDE
+  {
+    //get the references
+    this->scene = sc;
+    this->mw = mw;
+
+    //creates the action
+    QAction *actionHelloWorld= new QAction(QString("Open Dock Widget"), mw);
+    //specifies the subMenu
+    actionHelloWorld->setProperty("submenuName", "Basic");
+    //links the action
+    if(actionHelloWorld) {
+      connect(actionHelloWorld, SIGNAL(triggered()),
+              this, SLOT(helloWorld()));
+      _actions << actionHelloWorld;
+    }
+
+    dock_widget = new DockWidget("Print a number", mw);
+    dock_widget->setVisible(false); // do not show at the beginning
+
+    addDockWidget(dock_widget);
+
+    connect(dock_widget->pushButton, SIGNAL(clicked(bool)),
+            this, SLOT(on_dock_button_clicked()));
+  }
+  //! [init]
+
+private Q_SLOTS:
+//! [action]
+  void helloWorld()
+  {
+    // dock widget should be instancied in init()
+    if(dock_widget->isVisible()) { dock_widget->hide(); }
+    else                         { dock_widget->show(); }
+  }
+
+  void on_dock_button_clicked()
+  {
+      CGAL::Three::Three::information(QString("Here is your number :%1").arg(dock_widget->spinBox->value()));
+
+  }
+  //! [action]
+  //! [closure]
+  void closure()Q_DECL_OVERRIDE
+  {
+    dock_widget->hide();
+  }
+  //! [closure]
+private:
+  QList<QAction*> _actions;
+  DockWidget* dock_widget;
+};
+
+#include "Dock_widget_plugin.moc"
+
+
+
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Three_examples/Example_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Three_examples/Example_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Three_examples/Example_plugin.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Three_examples/Example_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,193 @@
+//! \file Polyhedron_demo_example_plugin.cpp
+
+#include <QApplication>
+#include <QMainWindow>
+#include <QAction>
+#include <QVector>
+#include <CGAL/Three/Scene_item_rendering_helper.h>
+#include <CGAL/Three/Viewer_interface.h>
+#include <CGAL/Three/Scene_group_item.h>
+
+#include <CGAL/Three/Triangle_container.h>
+
+//! [itemdeclaration]
+// The special Scene_item only for triangles
+
+using namespace CGAL::Three;
+typedef Triangle_container Tc;
+typedef Viewer_interface VI;
+
+
+class Scene_triangle_item
+    : public CGAL::Three::Scene_item_rendering_helper
+{
+
+  Q_OBJECT
+public :
+  Scene_triangle_item(double ax,double ay, double az,
+                      double bx,double by, double bz,
+                      double cx,double cy, double cz);
+
+  // Indicates if rendering mode is supported
+  bool supportsRenderingMode(RenderingMode m) const Q_DECL_OVERRIDE {
+    return (m == Flat);
+  }
+
+  //Displays the item
+  void draw(CGAL::Three::Viewer_interface* viewer) const Q_DECL_OVERRIDE;
+
+  //Specifies that the buffers need to be initialized again.
+  //Is mostly called after a change of geometry in the data.
+  void invalidateOpenGLBuffers() Q_DECL_OVERRIDE;
+
+  //fills the std::vector
+  void computeElements() const Q_DECL_OVERRIDE;
+
+  Scene_item* clone() const Q_DECL_OVERRIDE {return 0;}
+  QString toolTip() const Q_DECL_OVERRIDE {return QString();}
+  void compute_bbox() const Q_DECL_OVERRIDE { _bbox = Bbox(); }
+
+private:
+  //contains the data
+  mutable std::vector<float> vertices;
+  mutable std::size_t nb_pos;
+  //Fills the buffers with data. The buffers allow us to give data to the shaders.
+  void initializeBuffers(Viewer_interface *viewer) const Q_DECL_OVERRIDE;
+}; //end of class Scene_triangle_item
+//! [itemdeclaration]
+Scene_triangle_item::Scene_triangle_item(double ax,double ay, double az,
+                                         double bx,double by, double bz,
+                                         double cx,double cy, double cz)
+{
+  //! [creation]
+  //Prepare a single TriangleContainer, as we will only use one program.
+  setTriangleContainer(0, new Tc(VI::PROGRAM_NO_SELECTION,
+                                 false));
+  //! [creation]
+//! [computeelements]
+  //Fills the position vector with data.
+  nb_pos = 0;
+  vertices.resize(9);
+  vertices[0] = ax; vertices[1] = ay; vertices[2] = az;
+  vertices[3] = bx; vertices[4] = by; vertices[5] = bz;
+  vertices[6] = cx; vertices[7] = cy; vertices[8] = cz;
+  nb_pos=vertices.size();
+//! [computeelements]
+  //be sure the data will be computed next draw call
+  invalidateOpenGLBuffers();
+}
+
+
+//prepare the TriangleContainer with the computed data.
+void Scene_triangle_item::computeElements()const
+{
+//! [allocateelements]
+getTriangleContainer(0)->allocate(Tc::Flat_vertices, vertices.data(),
+                                  static_cast<int>(vertices.size()
+                                                   * sizeof(float)));
+setBuffersFilled(true);
+//! [allocateelements]
+}
+
+//! [draw]
+
+void Scene_triangle_item::draw(CGAL::Three::Viewer_interface* viewer) const
+{
+  //Initializes the OpenGL context for `viewer` if needed.
+  if(!isInit(viewer))
+    initGL(viewer);
+  if ( getBuffersFilled() &&
+       ! getBuffersInit(viewer))
+  {
+    initializeBuffers(viewer);
+    setBuffersInit(viewer, true);
+  }
+  if(!getBuffersFilled())
+  {
+    computeElements();
+    initializeBuffers(viewer);
+  }
+
+  //set the uniform properties for the TriangleContainer.
+  //Uniform values are setted at each draw call and are defined for the whole item.
+  //Values per simplex are computed as buffers in ComputeElements() and bound in initializeBuffers().
+  getTriangleContainer(0)->setColor(this->color());
+  getTriangleContainer(0)->draw(viewer, true);
+
+}
+//! [draw]
+//Specifies that the buffers need to be initialized again.
+//Is mostly called after a change of geometry in the data.
+void Scene_triangle_item::invalidateOpenGLBuffers()
+{
+  setBuffersFilled(false);
+  getTriangleContainer(0)->reset_vbos(ALL);
+}
+
+
+//! [fillbuffers]
+void Scene_triangle_item::initializeBuffers(CGAL::Three::Viewer_interface *viewer)const
+{
+//Bind the buffers filled in ComputeElements() for the TriangleContainer.
+  getTriangleContainer(0)->initializeBuffers(viewer);
+  getTriangleContainer(0)->setFlatDataSize(nb_pos);
+
+//once the buffers are bound, we can clear the vectors to optimize memory consumption
+  vertices.clear();
+  vertices.shrink_to_fit();
+}
+//! [fillbuffers]
+#include <CGAL/Three/Polyhedron_demo_plugin_helper.h>
+//The actual plugin
+using namespace CGAL::Three;
+class Q_DECL_EXPORT Polyhedron_demo_example_plugin :
+    public QObject,
+    public Polyhedron_demo_plugin_helper
+{
+  //Configures CMake to use MOC correctly
+  Q_OBJECT
+  Q_INTERFACES(CGAL::Three::Polyhedron_demo_plugin_interface)
+  Q_PLUGIN_METADATA(IID "com.geometryfactory.PolyhedronDemo.PluginInterface/1.0")
+
+
+public :
+  // Adds an action to the menu and configures the widget
+  void init(QMainWindow* mainWindow, CGAL::Three::Scene_interface* scene_interface, Messages_interface*) Q_DECL_OVERRIDE{
+    //get the references
+    this->scene = scene_interface;
+    this->mw = mainWindow;
+
+    //creates and link the actions
+    QAction* actionDrawTriangle= new QAction("Draw Triangle", mw);
+    if(actionDrawTriangle) {
+      connect(actionDrawTriangle, SIGNAL(triggered()),
+              this, SLOT(draw_triangle()));
+      _actions << actionDrawTriangle;
+    }
+  }
+  bool applicable(QAction*) const Q_DECL_OVERRIDE
+  {
+    return true;
+  }
+  QList<QAction*> actions() const Q_DECL_OVERRIDE{
+    return _actions;
+  }
+
+public Q_SLOTS:
+
+
+  void draw_triangle() {
+
+    triangle = new Scene_triangle_item(0, 0, 0,
+                                       1, 0, 0,
+                                       0.5, 0.5, 0);
+    triangle->setName(QString("Basic triangle"));
+    scene->addItem(triangle);
+  }
+private:
+  CGAL::Three::Scene_item* triangle;
+  QList<QAction*> _actions;
+
+}; //end of class Polyhedron_demo_example_plugin
+#include "Example_plugin.moc"
+
diff -pruN 5.0.3-1/demo/Polyhedron/Plugins/Three_examples/Group_item_plugin.cpp 5.1-1/demo/Polyhedron/Plugins/Three_examples/Group_item_plugin.cpp
--- 5.0.3-1/demo/Polyhedron/Plugins/Three_examples/Group_item_plugin.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Plugins/Three_examples/Group_item_plugin.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,82 @@
+#include <CGAL/Three/Polyhedron_demo_plugin_interface.h>
+#include <QApplication>
+#include <QObject>
+#include <QAction>
+#include <QMainWindow>
+#include <QInputDialog>
+#include "Scene_plane_item.h"
+#include <CGAL/Three/Scene_group_item.h>
+
+//This plugin crates an action in Operations that displays "Hello World" in the 'console' dockwidet.
+class GroupItemPlugin :
+    public QObject,
+    public CGAL::Three::Polyhedron_demo_plugin_interface
+{
+  Q_OBJECT
+  Q_INTERFACES(CGAL::Three::Polyhedron_demo_plugin_interface)
+  Q_PLUGIN_METADATA(IID "com.geometryfactory.PolyhedronDemo.PluginInterface/1.0")
+public:
+  //This plugin is only applicable if there is at exactly one selected item.
+  bool applicable(QAction*) const
+  {
+    return true;
+  }
+  //the list of the actions of the plugin.
+  QList<QAction*> actions() const
+  {
+    return _actions;
+  }
+  //this acts like a constructor for the plugin. It gets the references to the mainwindow and the scene, and connects the action.
+  void init(QMainWindow* mainWindow, CGAL::Three::Scene_interface* sc  )
+  {
+    //get the references
+    this->scene = sc;
+    this->mw = mainWindow;
+
+    //creates the action
+    QAction *actionHelloWorld= new QAction(QString("Hello World"), mw);
+    //specifies the subMenu
+    actionHelloWorld->setProperty("submenuName", "Basic");
+    //links the action
+    if(actionHelloWorld) {
+      connect(actionHelloWorld, SIGNAL(triggered()),
+              this, SLOT(helloWorld()));
+      _actions << actionHelloWorld;
+    }
+  }
+
+  void init(QMainWindow* mw, CGAL::Three::Scene_interface* sc, Messages_interface*)
+  {
+    init(mw, sc);
+  }
+private Q_SLOTS:
+
+
+  void helloWorld()
+  {
+    //creates an item
+    Scene_plane_item *child = new Scene_plane_item(scene);
+    child->setName("Trivial Plane");
+    child->setColor(Qt::blue);
+    child->setNormal(0.0,0.0,1.0);
+    scene->addItem(child);
+    //clears the selection to avoid adding unwanted items to the group.
+    scene->setSelectedItem(-1);
+    //Creates a new group
+    Scene_group_item *group = new Scene_group_item("New group");
+    //Then gives it its children
+    scene->changeGroup(child, group);
+    //adds it to the scene
+    scene->addGroup(group);
+
+  }
+
+private:
+  QList<QAction*> _actions;
+  //The reference to the scene
+  CGAL::Three::Scene_interface* scene;
+  //The reference to the main window
+  QMainWindow* mw;
+};
+
+#include "Group_item_plugin.moc"
diff -pruN 5.0.3-1/demo/Polyhedron/polyhedron_demo_macros.cmake 5.1-1/demo/Polyhedron/polyhedron_demo_macros.cmake
--- 5.0.3-1/demo/Polyhedron/polyhedron_demo_macros.cmake	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/polyhedron_demo_macros.cmake	2020-09-08 12:12:20.000000000 +0000
@@ -14,7 +14,7 @@ include(${CGAL_MODULES_DIR}/CGAL_add_tes
     else()
       set(other_sources ${ARGN_TAIL})
     endif()
-    if("${option}" STREQUAL "NO_MOC") 
+    if("${option}" STREQUAL "NO_MOC")
       set(option "")
       set(moc_file_name "")
     else()
@@ -62,11 +62,11 @@ include(${CGAL_MODULES_DIR}/CGAL_add_tes
         add_dependencies( ${keyword} ${plugin_name})
       endforeach()
       LIST(LENGTH keywords size)
-      math(EXPR size "${size} - 1") 
+      math(EXPR size "${size} - 1")
       LIST(GET keywords -1 last_element)
       LIST(REMOVE_AT keywords ${size})
       STRING(LENGTH ${last_element} size)
-      math(EXPR size "${size} - 2") 
+      math(EXPR size "${size} - 2")
       STRING(SUBSTRING ${last_element} 0 ${size} last_element)
       LIST(APPEND keywords ${last_element})
       foreach(keyword ${keywords})
diff -pruN 5.0.3-1/demo/Polyhedron/Preferences.ui 5.1-1/demo/Polyhedron/Preferences.ui
--- 5.0.3-1/demo/Polyhedron/Preferences.ui	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Preferences.ui	2020-09-08 12:12:20.000000000 +0000
@@ -7,7 +7,7 @@
     <x>0</x>
     <y>0</y>
     <width>631</width>
-    <height>526</height>
+    <height>628</height>
    </rect>
   </property>
   <property name="sizePolicy">
@@ -59,8 +59,8 @@
             <rect>
              <x>0</x>
              <y>0</y>
-             <width>278</width>
-             <height>499</height>
+             <width>275</width>
+             <height>534</height>
             </rect>
            </property>
            <layout class="QVBoxLayout" name="verticalLayout_3">
@@ -285,6 +285,23 @@
               </property>
              </widget>
             </item>
+            <item>
+             <widget class="QPushButton" name="sshButton">
+              <property name="enabled">
+               <bool>true</bool>
+              </property>
+              <property name="text">
+               <string>Network Settings</string>
+              </property>
+             </widget>
+            </item>
+            <item>
+             <widget class="QPushButton" name="backFrontColor_pushButton">
+              <property name="text">
+               <string>Change Back/Front Colors...</string>
+              </property>
+             </widget>
+            </item>
             <item>
              <spacer name="verticalSpacer">
               <property name="orientation">
diff -pruN 5.0.3-1/demo/Polyhedron/resources/compatibility_shaders/shader_c3t3.frag 5.1-1/demo/Polyhedron/resources/compatibility_shaders/shader_c3t3.frag
--- 5.0.3-1/demo/Polyhedron/resources/compatibility_shaders/shader_c3t3.frag	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/resources/compatibility_shaders/shader_c3t3.frag	2020-09-08 12:12:20.000000000 +0000
@@ -40,7 +40,10 @@ void main(void) {
       highp vec3 V = -fP.xyz; 
       highp vec3 N; 
       if(fN == vec3(0.0,0.0,0.0))
-          N = vec3(0.0,0.0,0.0);
+      {
+        gl_FragColor = my_color;
+        return;
+      }
       else 
           N = normalize(fN); 
       L = normalize(L); 
diff -pruN 5.0.3-1/demo/Polyhedron/resources/compatibility_shaders/shader_with_light.frag 5.1-1/demo/Polyhedron/resources/compatibility_shaders/shader_with_light.frag
--- 5.0.3-1/demo/Polyhedron/resources/compatibility_shaders/shader_with_light.frag	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/resources/compatibility_shaders/shader_with_light.frag	2020-09-08 12:12:20.000000000 +0000
@@ -3,6 +3,8 @@ varying highp vec4 color;
 varying highp vec4 fP;
 varying highp vec3 fN;
 varying highp float dist[6];
+uniform highp vec4 front_color;
+uniform highp vec4 back_color;
 uniform highp vec4 light_pos;
 uniform highp vec4 light_diff;
 uniform highp vec4 light_spec;
@@ -17,6 +19,7 @@ uniform highp float width;
 uniform highp float height;
 uniform bool comparing;
 uniform bool writing;
+uniform bool back_front_shading;
 uniform sampler2D sampler;
 uniform highp float alpha;
 
@@ -44,18 +47,30 @@ void main(void) {
     gl_FragColor = vec4(d,d,d,1.0);
   else
   {
-    highp vec4 my_color = vec4(color.xyz, 1.0);
     highp vec3 L = light_pos.xyz - fP.xyz;
     highp vec3 V = -fP.xyz;
     highp vec3 N;
-    if(fN == vec3(0.0,0.0,0.0))
-      N = vec3(0.0,0.0,0.0);
-    else
-      N = normalize(fN);
+    highp vec4 my_color = highp vec4(color.xyz, 1.0);
+    if(fN ==  vec3(0.0,0.0,0.0))
+    {
+      out_color = my_color;
+      return;
+    }
+    N = normalize(fN);
     L = normalize(L);
     V = normalize(V);
     highp vec3 R = reflect(-L, N);
     highp vec4 diffuse;
+    float dot_prod = dot(N,L);
+    
+    if(back_front_shading)
+    {
+      if (dot_prod > 0)
+        my_color = front_color;
+      else
+        my_color = back_color;
+    }
+    
     if(is_two_side == 1)
       diffuse = abs(dot(N,L)) * light_diff * color;
     else
diff -pruN 5.0.3-1/demo/Polyhedron/resources/shader_c3t3.frag 5.1-1/demo/Polyhedron/resources/shader_c3t3.frag
--- 5.0.3-1/demo/Polyhedron/resources/shader_c3t3.frag	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/resources/shader_c3t3.frag	2020-09-08 12:12:20.000000000 +0000
@@ -42,7 +42,10 @@ void main(void) {
       vec3 V = -fP.xyz; 
       vec3 N; 
       if(fN == vec3(0.0,0.0,0.0))
-          N = vec3(0.0,0.0,0.0); 
+      {
+        out_color = my_color;
+        return;
+      }
       else 
           N = normalize(fN); 
       L = normalize(L); 
diff -pruN 5.0.3-1/demo/Polyhedron/resources/shader_with_light.frag 5.1-1/demo/Polyhedron/resources/shader_with_light.frag
--- 5.0.3-1/demo/Polyhedron/resources/shader_with_light.frag	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/resources/shader_with_light.frag	2020-09-08 12:12:20.000000000 +0000
@@ -3,6 +3,8 @@ in vec4 color;
 in vec4 fP;
 in vec3 fN;
 in float dist[6];
+uniform vec4 front_color;
+uniform vec4 back_color;
 uniform vec4 light_pos;
 uniform vec4 light_diff;
 uniform vec4 light_spec;
@@ -17,6 +19,7 @@ uniform float width;
 uniform float height;
 uniform bool comparing;
 uniform bool writing;
+uniform bool back_front_shading;
 uniform sampler2D sampler;
 uniform float alpha;
 out vec4 out_color;
@@ -45,22 +48,39 @@ void main(void) {
     out_color = vec4(d,d,d,1.0);
   else
   {
-    vec4 my_color = vec4(color.xyz, 1.0);
     vec3 L = light_pos.xyz - fP.xyz;
     vec3 V = -fP.xyz;
     vec3 N;
+    vec4 my_color = vec4(color.xyz, 1.0);
     if(fN ==  vec3(0.0,0.0,0.0))
-      N =  vec3(0.0,0.0,0.0);
+    {
+      out_color = my_color;
+      return;
+    }
     else
       N = normalize(fN);
     L = normalize(L);
     V = normalize(V);
     vec3 R = reflect(-L, N);
     vec4 diffuse;
+    float dot_prod = dot(N,L);
+    
+    if(back_front_shading)
+    {
+      if (dot_prod > 0)
+        my_color = front_color;
+      else
+        my_color = back_color;
+    }
+    else
+    {
+      my_color = vec4(color.xyz, 1.0);
+    }
     if(is_two_side == 1)
-      diffuse = abs(dot(N,L)) * light_diff * color;
+      diffuse = abs(dot_prod) * light_diff * color;
     else
-      diffuse = max(dot(N,L), 0.0) * light_diff * my_color;
+      diffuse = max(dot_prod, 0.0) * light_diff * my_color;
+
     vec4 specular = pow(max(dot(R,V), 0.0), spec_power) * light_spec;
     vec4 ret_color = vec4((my_color*light_amb).xyz + diffuse.xyz + specular.xyz,1);
     if(is_selected)
diff -pruN 5.0.3-1/demo/Polyhedron/Scene_c3t3_item.cpp 5.1-1/demo/Polyhedron/Scene_c3t3_item.cpp
--- 5.0.3-1/demo/Polyhedron/Scene_c3t3_item.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Scene_c3t3_item.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -50,6 +50,14 @@ typedef Edge_container Ec;
 typedef Point_container Pc;
 typedef Viewer_interface Vi;
 
+QVector4D cgal_plane_to_vector4d(EPICK::Plane_3 plane) {
+  return {
+    static_cast<float>(-plane.a()),
+    static_cast<float>(-plane.b()),
+    static_cast<float>(-plane.c()),
+    static_cast<float>(-plane.d()) };
+}
+
 // The special Scene_item only for triangles
 class Scene_intersection_item : public CGAL::Three::Scene_item_rendering_helper
 {
@@ -145,11 +153,11 @@ public :
       alphaSlider->setMaximum(255);
       alphaSlider->setValue(255);
     }
-    viewer->makeCurrent();
+    //viewer->makeCurrent();
     const EPICK::Plane_3& plane = qobject_cast<Scene_c3t3_item*>(this->parent())->plane();
     float shrink_factor = qobject_cast<Scene_c3t3_item*>(this->parent())->getShrinkFactor();
-    QVector4D cp(-plane.a(), -plane.b(), -plane.c(), -plane.d());
-    getTriangleContainer(0)->setPlane(cp);
+    QVector4D cp = cgal_plane_to_vector4d(plane);
+    getTriangleContainer(0)->setPlane(-cp);
     getTriangleContainer(0)->setShrinkFactor(shrink_factor);
     // positions_poly is also used for the faces in the cut plane
     // and changes when the cut plane is moved
@@ -161,7 +169,7 @@ public :
     if(is_fast)
       return;
     const EPICK::Plane_3& plane = qobject_cast<Scene_c3t3_item*>(this->parent())->plane();
-    QVector4D cp(-plane.a(), -plane.b(), -plane.c(), -plane.d());
+    QVector4D cp = cgal_plane_to_vector4d(plane);
     getEdgeContainer(0)->setPlane(cp);
     getEdgeContainer(0)->setColor(QColor(Qt::black));
     getEdgeContainer(0)->draw(viewer, true);
@@ -180,48 +188,38 @@ public :
     Geom_traits::Vector_3 n = cross_product(pb - pa, pc - pa);
     n = n / CGAL::sqrt(n*n);
 
+    auto push_normal = [this](auto n) {
+      normals->push_back(static_cast<float>(n.x()));
+      normals->push_back(static_cast<float>(n.y()));
+      normals->push_back(static_cast<float>(n.z()));
+    };
+
+    auto push_vertex = [this, &offset](const auto& p) {
+      this->vertices->push_back(static_cast<float>(p.x()+offset.x));
+      this->vertices->push_back(static_cast<float>(p.y()+offset.y));
+      this->vertices->push_back(static_cast<float>(p.z()+offset.z));
+    };
+
+    auto push_edge = [this, &offset](const auto& pa, const auto& pb) {
+      this->edges->push_back(static_cast<float>(pa.x()+offset.x));
+      this->edges->push_back(static_cast<float>(pa.y()+offset.y));
+      this->edges->push_back(static_cast<float>(pa.z()+offset.z));
+      this->edges->push_back(static_cast<float>(pb.x()+offset.x));
+      this->edges->push_back(static_cast<float>(pb.y()+offset.y));
+      this->edges->push_back(static_cast<float>(pb.z()+offset.z));
+    };
+
     for (int i = 0; i<3; i++)
     {
-      normals->push_back(n.x());
-      normals->push_back(n.y());
-      normals->push_back(n.z());
-    }
-
-    vertices->push_back(pa.x()+offset.x);
-    vertices->push_back(pa.y()+offset.y);
-    vertices->push_back(pa.z()+offset.z);
-
-    vertices->push_back(pb.x()+offset.x);
-    vertices->push_back(pb.y()+offset.y);
-    vertices->push_back(pb.z()+offset.z);
-
-    vertices->push_back(pc.x()+offset.x);
-    vertices->push_back(pc.y()+offset.y);
-    vertices->push_back(pc.z()+offset.z);
-
-    edges->push_back(pa.x()+offset.x);
-    edges->push_back(pa.y()+offset.y);
-    edges->push_back(pa.z()+offset.z);
-
-    edges->push_back(pb.x()+offset.x);
-    edges->push_back(pb.y()+offset.y);
-    edges->push_back(pb.z()+offset.z);
-
-    edges->push_back(pb.x()+offset.x);
-    edges->push_back(pb.y()+offset.y);
-    edges->push_back(pb.z()+offset.z);
-
-    edges->push_back(pc.x()+offset.x);
-    edges->push_back(pc.y()+offset.y);
-    edges->push_back(pc.z()+offset.z);
-
-    edges->push_back(pc.x()+offset.x);
-    edges->push_back(pc.y()+offset.y);
-    edges->push_back(pc.z()+offset.z);
-
-    edges->push_back(pa.x()+offset.x);
-    edges->push_back(pa.y()+offset.y);
-    edges->push_back(pa.z()+offset.z);
+      push_normal(n);
+    }
+    push_vertex(pa);
+    push_vertex(pb);
+    push_vertex(pc);
+
+    push_edge(pa, pb);
+    push_edge(pb, pc);
+    push_edge(pc, pa);
 
     for(int i=0; i<3; i++)
     {
@@ -229,9 +227,9 @@ public :
       colors->push_back((float)color.green()/255);
       colors->push_back((float)color.blue()/255);
 
-      barycenters->push_back((pa[0]+pb[0]+pc[0])/3.0 + offset.x);
-      barycenters->push_back((pa[1]+pb[1]+pc[1])/3.0 + offset.y);
-      barycenters->push_back((pa[2]+pb[2]+pc[2])/3.0 + offset.z);
+      barycenters->push_back(static_cast<float>((pa[0]+pb[0]+pc[0])/3.0 + offset.x));
+      barycenters->push_back(static_cast<float>((pa[1]+pb[1]+pc[1])/3.0 + offset.y));
+      barycenters->push_back(static_cast<float>((pa[2]+pb[2]+pc[2])/3.0 + offset.z));
     }
   }
 
@@ -349,6 +347,8 @@ struct Scene_c3t3_item_priv {
     cnc_are_shown = false;
     is_aabb_tree_built = false;
     alphaSlider = NULL;
+    sharp_edges_angle = -1;
+    detect_borders = false;
   }
   void computeIntersection(const Primitive& facet);
   void fill_aabb_tree() {
@@ -441,6 +441,7 @@ struct Scene_c3t3_item_priv {
     return false;
   }
   Scene_spheres_item *spheres;
+  std::vector<Tr::Vertex> tr_vertices;
   Scene_intersection_item *intersection;
   bool spheres_are_shown;
   const Scene_item* data_item_;
@@ -502,6 +503,31 @@ struct Scene_c3t3_item_priv {
   bool is_valid;
   bool is_surface;
   bool last_intersection;
+  //only for optimizers
+  double sharp_edges_angle;
+  bool detect_borders;
+
+  void push_normal(std::vector<float>& normals, const EPICK::Vector_3& n) const
+  {
+    normals.push_back(static_cast<float>(n.x()));
+    normals.push_back(static_cast<float>(n.y()));
+    normals.push_back(static_cast<float>(n.z()));
+  }
+  void push_point(std::vector<float>& points, const EPICK::Point_3& p,
+                  const CGAL::qglviewer::Vec& offset) const
+  {
+    points.push_back(static_cast<float>(p.x()+offset.x));
+    points.push_back(static_cast<float>(p.y()+offset.y));
+    points.push_back(static_cast<float>(p.z()+offset.z));
+  }
+  void push_edge(std::vector<float>& edges,
+                 const EPICK::Point_3& pa,
+                 const EPICK::Point_3& pb,
+                 const CGAL::qglviewer::Vec& offset) const
+  {
+    push_point(edges, pa, offset);
+    push_point(edges, pb, offset);
+  }
 };
 
 struct Set_show_tetrahedra {
@@ -548,8 +574,10 @@ Scene_c3t3_item::Scene_c3t3_item(const C
   : Scene_group_item("unnamed")
   , d(new Scene_c3t3_item_priv(c3t3, this))
 {
-  d->reset_cut_plane();
   common_constructor(is_surface);
+  d->reset_cut_plane();
+  c3t3_changed();
+  changed();
 }
 
 Scene_c3t3_item::~Scene_c3t3_item()
@@ -747,8 +775,8 @@ Scene_c3t3_item::build_histogram()
   const int text_margin = 3;
   const int text_height = 34;
 
-  histogram_ = QPixmap(width, height + text_height);
-  histogram_.fill(QColor(255, 255, 255));
+  d->histogram_ = QPixmap(width, height + text_height);
+  d->histogram_.fill(QColor(255, 255, 255));
 #else
   // Create an histogram_ and display it
   const int height = 140;
@@ -932,7 +960,7 @@ void Scene_c3t3_item::draw(CGAL::Three::
   if(renderingMode() == Flat ||
      renderingMode() == FlatPlusEdges)
   {
-    QVector4D cp(this->plane().a(),this->plane().b(),this->plane().c(),this->plane().d());
+    QVector4D cp = cgal_plane_to_vector4d(this->plane());
     getTriangleContainer(C3t3_faces)->setPlane(cp);
     float shrink_factor = getShrinkFactor();
     getTriangleContainer(C3t3_faces)->setShrinkFactor(shrink_factor);
@@ -967,7 +995,7 @@ void Scene_c3t3_item::draw(CGAL::Three::
     getEdgeContainer(Grid_edges)->setColor(QColor(Qt::black));
     QMatrix4x4 f_mat;
     for (int i = 0; i<16; i++)
-      f_mat.data()[i] = d->frame->matrix()[i];
+      f_mat.data()[i] = static_cast<float>(d->frame->matrix()[i]);
     getEdgeContainer(Grid_edges)->setFrameMatrix(f_mat);
     getEdgeContainer(Grid_edges)->draw(viewer, true);
   }
@@ -1004,12 +1032,12 @@ void Scene_c3t3_item::drawEdges(CGAL::Th
      getEdgeContainer(Grid_edges)->setColor(QColor(Qt::black));
       QMatrix4x4 f_mat;
       for (int i = 0; i<16; i++)
-        f_mat.data()[i] = d->frame->matrix()[i];
+        f_mat.data()[i] = static_cast<float>(d->frame->matrix()[i]);
       getEdgeContainer(Grid_edges)->setFrameMatrix(f_mat);
       getEdgeContainer(Grid_edges)->draw(viewer, true);
     }
 
-    QVector4D cp(this->plane().a(),this->plane().b(),this->plane().c(),this->plane().d());
+    QVector4D cp = cgal_plane_to_vector4d(this->plane());
     getEdgeContainer(C3t3_edges)->setPlane(cp);
     getEdgeContainer(C3t3_edges)->setIsSurface(d->is_surface);
     getEdgeContainer(C3t3_edges)->setColor(QColor(Qt::black));
@@ -1059,7 +1087,7 @@ void Scene_c3t3_item::drawPoints(CGAL::T
     }
 
 
-    QVector4D cp(this->plane().a(),this->plane().b(),this->plane().c(),this->plane().d());
+    QVector4D cp = cgal_plane_to_vector4d(this->plane());
     getPointContainer(C3t3_points)->setPlane(cp);
     getPointContainer(C3t3_points)->setIsSurface(d->is_surface);
     getPointContainer(C3t3_points)->setColor(this->color());
@@ -1070,7 +1098,7 @@ void Scene_c3t3_item::drawPoints(CGAL::T
       getEdgeContainer(Grid_edges)->setColor(QColor(Qt::black));
       QMatrix4x4 f_mat;
       for (int i = 0; i<16; i++)
-        f_mat.data()[i] = d->frame->matrix()[i];
+        f_mat.data()[i] = static_cast<float>(d->frame->matrix()[i]);
       getEdgeContainer(Grid_edges)->setFrameMatrix(f_mat);
       getEdgeContainer(Grid_edges)->draw(viewer, true);
     }
@@ -1085,100 +1113,41 @@ void Scene_c3t3_item_priv::draw_triangle
                                          const Tr::Bare_point& pb,
                                          const Tr::Bare_point& pc) const
 {
-#undef darker
   Geom_traits::Vector_3 n = cross_product(pb - pa, pc - pa);
   n = n / CGAL::sqrt(n*n);
   const CGAL::qglviewer::Vec offset = Three::mainViewer()->offset();
 
   for (int i = 0; i<3; i++)
   {
-    normals.push_back(n.x());
-    normals.push_back(n.y());
-    normals.push_back(n.z());
-  }
-  positions_poly.push_back(pa.x()+offset.x);
-  positions_poly.push_back(pa.y()+offset.y);
-  positions_poly.push_back(pa.z()+offset.z);
-
-  positions_poly.push_back(pb.x()+offset.x);
-  positions_poly.push_back(pb.y()+offset.y);
-  positions_poly.push_back(pb.z()+offset.z);
-
-  positions_poly.push_back(pc.x()+offset.x);
-  positions_poly.push_back(pc.y()+offset.y);
-  positions_poly.push_back(pc.z()+offset.z);
+    push_normal(normals, n);
+  }
+  push_point(positions_poly, pa, offset);
+  push_point(positions_poly, pb, offset);
+  push_point(positions_poly, pc, offset);
 
   for(int i=0; i<3; ++i)
   {
-   positions_barycenter.push_back((pa[0]+pb[0]+pc[0])/3.0 + offset.x);
-   positions_barycenter.push_back((pa[1]+pb[1]+pc[1])/3.0 + offset.y);
-   positions_barycenter.push_back((pa[2]+pb[2]+pc[2])/3.0 + offset.z);
+    push_point(positions_barycenter, CGAL::centroid(pa, pb, pc), offset);
   }
-
-
 }
 
 void Scene_c3t3_item_priv::draw_triangle_edges(const Tr::Bare_point& pa,
                                                const Tr::Bare_point& pb,
                                                const Tr::Bare_point& pc) const
 {
-#undef darker
   const CGAL::qglviewer::Vec offset = Three::mainViewer()->offset();
-  positions_lines.push_back(pa.x()+offset.x);
-  positions_lines.push_back(pa.y()+offset.y);
-  positions_lines.push_back(pa.z()+offset.z);
-
-  positions_lines.push_back(pb.x()+offset.x);
-  positions_lines.push_back(pb.y()+offset.y);
-  positions_lines.push_back(pb.z()+offset.z);
-
-  positions_lines.push_back(pb.x()+offset.x);
-  positions_lines.push_back(pb.y()+offset.y);
-  positions_lines.push_back(pb.z()+offset.z);
-
-  positions_lines.push_back(pc.x()+offset.x);
-  positions_lines.push_back(pc.y()+offset.y);
-  positions_lines.push_back(pc.z()+offset.z);
-
-  positions_lines.push_back(pc.x()+offset.x);
-  positions_lines.push_back(pc.y()+offset.y);
-  positions_lines.push_back(pc.z()+offset.z);
-
-  positions_lines.push_back(pa.x()+offset.x);
-  positions_lines.push_back(pa.y()+offset.y);
-  positions_lines.push_back(pa.z()+offset.z);
-
+  push_edge(positions_lines, pa, pb, offset);
+  push_edge(positions_lines, pb, pc, offset);
+  push_edge(positions_lines, pc, pa, offset);
 }
 void Scene_c3t3_item_priv::draw_triangle_edges_cnc(const Tr::Bare_point& pa,
                                                    const Tr::Bare_point& pb,
                                                    const Tr::Bare_point& pc) const
 {
-#undef darker
   const CGAL::qglviewer::Vec offset = Three::mainViewer()->offset();
-  positions_lines_not_in_complex.push_back(pa.x()+offset.x);
-  positions_lines_not_in_complex.push_back(pa.y()+offset.y);
-  positions_lines_not_in_complex.push_back(pa.z()+offset.z);
-
-  positions_lines_not_in_complex.push_back(pb.x()+offset.x);
-  positions_lines_not_in_complex.push_back(pb.y()+offset.y);
-  positions_lines_not_in_complex.push_back(pb.z()+offset.z);
-
-  positions_lines_not_in_complex.push_back(pb.x()+offset.x);
-  positions_lines_not_in_complex.push_back(pb.y()+offset.y);
-  positions_lines_not_in_complex.push_back(pb.z()+offset.z);
-
-  positions_lines_not_in_complex.push_back(pc.x()+offset.x);
-  positions_lines_not_in_complex.push_back(pc.y()+offset.y);
-  positions_lines_not_in_complex.push_back(pc.z()+offset.z);
-
-  positions_lines_not_in_complex.push_back(pc.x()+offset.x);
-  positions_lines_not_in_complex.push_back(pc.y()+offset.y);
-  positions_lines_not_in_complex.push_back(pc.z()+offset.z);
-
-  positions_lines_not_in_complex.push_back(pa.x()+offset.x);
-  positions_lines_not_in_complex.push_back(pa.y()+offset.y);
-  positions_lines_not_in_complex.push_back(pa.z()+offset.z);
-
+  push_edge(positions_lines_not_in_complex, pa, pb, offset);
+  push_edge(positions_lines_not_in_complex, pb, pc, offset);
+  push_edge(positions_lines_not_in_complex, pc, pa, offset);
 }
 
 double Scene_c3t3_item_priv::complex_diag() const {
@@ -1390,7 +1359,7 @@ void Scene_c3t3_item_priv::computeSphere
 
   if(!spheres)
     return;
-
+  int s_id = 0;
   for(Tr::Finite_vertices_iterator
       vit = c3t3.triangulation().finite_vertices_begin(),
       end =  c3t3.triangulation().finite_vertices_end();
@@ -1434,10 +1403,12 @@ void Scene_c3t3_item_priv::computeSphere
     Tr::Bare_point center(wp2p(vit->point()).x() + offset.x,
                           wp2p(vit->point()).y() + offset.y,
                           wp2p(vit->point()).z() + offset.z);
-    float radius = vit->point().weight() ;
+    double radius = vit->point().weight() ;
     typedef unsigned char UC;
-    spheres->add_sphere(Geom_traits::Sphere_3(center, radius),
+    tr_vertices.push_back(*vit);
+    spheres->add_sphere(Geom_traits::Sphere_3(center, radius),s_id++,
                         CGAL::Color(UC(c.red()), UC(c.green()), UC(c.blue())));
+
   }
   spheres->invalidateOpenGLBuffers();
 }
@@ -1464,30 +1435,31 @@ void Scene_c3t3_item_priv::computeElemen
 
   //The grid
   {
+    positions_grid.resize(0);
 
-    float x = (2 * (float)complex_diag()) / 10.0;
-    float y = (2 * (float)complex_diag()) / 10.0;
+    float x = (2 * (float)complex_diag()) / 10.0f;
+    float y = (2 * (float)complex_diag()) / 10.0f;
     for (float u = 0; u < 11; u += 1.f)
     {
 
       positions_grid.push_back(-(float)complex_diag() + x* u);
       positions_grid.push_back(-(float)complex_diag());
-      positions_grid.push_back(0.0);
+      positions_grid.push_back(0.0f);
 
       positions_grid.push_back(-(float)complex_diag() + x* u);
       positions_grid.push_back((float)complex_diag());
-      positions_grid.push_back(0.0);
+      positions_grid.push_back(0.0f);
     }
     for (float v = 0; v<11; v += 1.f)
     {
 
       positions_grid.push_back(-(float)complex_diag());
       positions_grid.push_back(-(float)complex_diag() + v * y);
-      positions_grid.push_back(0.0);
+      positions_grid.push_back(0.0f);
 
       positions_grid.push_back((float)complex_diag());
       positions_grid.push_back(-(float)complex_diag() + v * y);
-      positions_grid.push_back(0.0);
+      positions_grid.push_back(0.0f);
     }
   }
 
@@ -1508,9 +1480,9 @@ void Scene_c3t3_item_priv::computeElemen
       const Tr::Bare_point& pc = wp2p(cell->vertex((index + 3) & 3)->point());
 
       QColor color = colors[cell->surface_patch_index(index)];
-      f_colors.push_back(color.redF());f_colors.push_back(color.greenF());f_colors.push_back(color.blueF());
-      f_colors.push_back(color.redF());f_colors.push_back(color.greenF());f_colors.push_back(color.blueF());
-      f_colors.push_back(color.redF());f_colors.push_back(color.greenF());f_colors.push_back(color.blueF());
+      f_colors.push_back((float)color.redF());f_colors.push_back((float)color.greenF());f_colors.push_back((float)color.blueF());
+      f_colors.push_back((float)color.redF());f_colors.push_back((float)color.greenF());f_colors.push_back((float)color.blueF());
+      f_colors.push_back((float)color.redF());f_colors.push_back((float)color.greenF());f_colors.push_back((float)color.blueF());
       if ((index % 2 == 1) == c3t3.is_in_complex(cell))
         draw_triangle(pb, pa, pc);
       else draw_triangle(pa, pb, pc);
@@ -1603,7 +1575,17 @@ void Scene_c3t3_item::show_spheres(bool
     contextMenu()->findChild<QAction*>("actionShowSpheres")->setChecked(b);
     if(b && !d->spheres)
     {
-      d->spheres = new Scene_spheres_item(this, true);
+      d->spheres = new Scene_spheres_item(this, d->c3t3.number_of_vertices_in_complex(), true);
+      connect(d->spheres, &Scene_spheres_item::picked,
+              this, [this](std::size_t id)
+      {
+        if(id == (std::size_t)(-1))
+          return;
+        QString msg = QString("Vertex's index : %1; Vertex's in dimension: %2.").arg(d->tr_vertices[id].index()).arg(d->tr_vertices[id].in_dimension());
+        CGAL::Three::Three::information(msg);
+        CGAL::Three::Three::mainViewer()->displayMessage(msg, 5000);
+
+      });
       d->spheres->setName("Protecting spheres");
       d->spheres->setRenderingMode(Gouraud);
       connect(d->spheres, SIGNAL(destroyed()), this, SLOT(reset_spheres()));
@@ -1620,7 +1602,6 @@ void Scene_c3t3_item::show_spheres(bool
     }
     Q_EMIT redraw();
   }
-
 }
 void Scene_c3t3_item::show_intersection(bool b)
 {
@@ -1772,10 +1753,10 @@ QString Scene_c3t3_item::computeStats(in
 
   if(!d->computed_stats)
   {
-    float nb_edges = 0;
-    float total_edges = 0;
-    float nb_angle = 0;
-    float total_angle = 0;
+    double nb_edges = 0;
+    double total_edges = 0;
+    double nb_angle = 0;
+    double total_angle = 0;
 
     for (C3t3::Facet_iterator
       fit = d->c3t3.facets_begin(),
@@ -1787,19 +1768,19 @@ QString Scene_c3t3_item::computeStats(in
       const Tr::Bare_point& pa = wp2p(cell->vertex((index + 1) & 3)->point());
       const Tr::Bare_point& pb = wp2p(cell->vertex((index + 2) & 3)->point());
       const Tr::Bare_point& pc = wp2p(cell->vertex((index + 3) & 3)->point());
-      float edges[3];
+      double edges[3];
       edges[0]=(std::sqrt(CGAL::squared_distance(pa, pb)));
       edges[1]=(std::sqrt(CGAL::squared_distance(pa, pc)));
       edges[2]=(std::sqrt(CGAL::squared_distance(pb, pc)));
       for(int i=0; i<3; ++i)
       {
-        if(edges[i] < d->min_edges_length){ d->min_edges_length = edges[i]; }
-        if(edges[i] > d->max_edges_length){ d->max_edges_length = edges[i]; }
+        if(edges[i] < d->min_edges_length){ d->min_edges_length = static_cast<float>(edges[i]); }
+        if(edges[i] > d->max_edges_length){ d->max_edges_length = static_cast<float>(edges[i]); }
         total_edges+=edges[i];
         ++nb_edges;
       }
     }
-    d->mean_edges_length = total_edges/(float)nb_edges;
+    d->mean_edges_length = static_cast<float>(total_edges/nb_edges);
     for(Tr::Finite_vertices_iterator
         vit = d->c3t3.triangulation().finite_vertices_begin(),
         end =  d->c3t3.triangulation().finite_vertices_end();
@@ -1847,10 +1828,10 @@ QString Scene_c3t3_item::computeStats(in
       const Tr::Bare_point& p1 = wp2p(cit->vertex(1)->point());
       const Tr::Bare_point& p2 = wp2p(cit->vertex(2)->point());
       const Tr::Bare_point& p3 = wp2p(cit->vertex(3)->point());
-      float v = std::abs(CGAL::volume(p0, p1, p2, p3));
-      float circumradius = std::sqrt(CGAL::squared_radius(p0, p1, p2, p3));
+      double v = std::abs(CGAL::volume(p0, p1, p2, p3));
+      double circumradius = std::sqrt(CGAL::squared_radius(p0, p1, p2, p3));
       //find smallest edge
-      float edges[6];
+      double edges[6];
       edges[0] = std::sqrt(CGAL::squared_distance(p0, p1));
       edges[1] = std::sqrt(CGAL::squared_distance(p0, p2));
       edges[2] = std::sqrt(CGAL::squared_distance(p0, p3));
@@ -1858,60 +1839,59 @@ QString Scene_c3t3_item::computeStats(in
       edges[4] = std::sqrt(CGAL::squared_distance(p2, p3));
       edges[5] = std::sqrt(CGAL::squared_distance(p1, p3));
 
-      float min_edge = edges[0];
+      double min_edge = edges[0];
       for(int i=1; i<6; ++i)
       {
        if(edges[i]<min_edge)
          min_edge=edges[i];
       }
-      float sumar = std::sqrt(CGAL::squared_area(p0,p1,p2))+std::sqrt(CGAL::squared_area(p1,p2,p3))+
+      double sumar = std::sqrt(CGAL::squared_area(p0,p1,p2))+std::sqrt(CGAL::squared_area(p1,p2,p3))+
           std::sqrt(CGAL::squared_area(p2,p3,p0)) + std::sqrt(CGAL::squared_area(p3,p1,p0));
-      float inradius = 3*v/sumar;
-      float smallest_edge_radius = min_edge/circumradius*std::sqrt(6)/4.0;//*sqrt(6)/4 so that the perfect tet ratio is 1
-      float smallest_radius_radius = inradius/circumradius*3; //*3 so that the perfect tet ratio is 1 instead of 1/3
-      float biggest_v_sma_cube = v/std::pow(min_edge,3)*6*std::sqrt(2);//*6*sqrt(2) so that the perfect tet ratio is 1 instead
+      double inradius = 3*v/sumar;
+      double smallest_edge_radius = min_edge/circumradius*std::sqrt(6)/4.0;//*sqrt(6)/4 so that the perfect tet ratio is 1
+      double smallest_radius_radius = inradius/circumradius*3; //*3 so that the perfect tet ratio is 1 instead of 1/3
+      double biggest_v_sma_cube = v/std::pow(min_edge,3)*6*std::sqrt(2);//*6*sqrt(2) so that the perfect tet ratio is 1 instead
 
       if(smallest_edge_radius < d->smallest_edge_radius)
-        d->smallest_edge_radius = smallest_edge_radius;
+        d->smallest_edge_radius = static_cast<float>(smallest_edge_radius);
 
       if(smallest_radius_radius < d->smallest_radius_radius)
-        d->smallest_radius_radius = smallest_radius_radius;
+        d->smallest_radius_radius = static_cast<float>(smallest_radius_radius);
 
       if(biggest_v_sma_cube > d->biggest_v_sma_cube)
-        d->biggest_v_sma_cube = biggest_v_sma_cube;
+        d->biggest_v_sma_cube = static_cast<float>(biggest_v_sma_cube);
+
+      auto update_min_max_dihedral_angle = [this](double a) {
+        if(a < this->d->min_dihedral_angle) { this->d->min_dihedral_angle = static_cast<float>(a); }
+        if(a > this->d->max_dihedral_angle) { this->d->max_dihedral_angle = static_cast<float>(a); }
+      };
 
       double a = CGAL::to_double(CGAL::abs(approx_dihedral_angle(p0, p1, p2, p3)));
-      if(a < d->min_dihedral_angle) { d->min_dihedral_angle = a; }
-      if(a > d->max_dihedral_angle) { d->max_dihedral_angle = a; }
+      update_min_max_dihedral_angle(a);
       total_angle+=a;
       ++nb_angle;
       a = CGAL::to_double(CGAL::abs(approx_dihedral_angle(p0, p2, p1, p3)));
-      if(a < d->min_dihedral_angle) { d->min_dihedral_angle = a; }
-      if(a > d->max_dihedral_angle) { d->max_dihedral_angle = a; }
+      update_min_max_dihedral_angle(a);
       total_angle+=a;
       ++nb_angle;
       a = CGAL::to_double(CGAL::abs(approx_dihedral_angle(p0, p3, p1, p2)));
-      if(a < d->min_dihedral_angle) { d->min_dihedral_angle = a; }
-      if(a > d->max_dihedral_angle) { d->max_dihedral_angle = a; }
+      update_min_max_dihedral_angle(a);
       total_angle+=a;
       ++nb_angle;
       a = CGAL::to_double(CGAL::abs(approx_dihedral_angle(p1, p2, p0, p3)));
-      if(a < d->min_dihedral_angle) { d->min_dihedral_angle = a; }
-      if(a > d->max_dihedral_angle) { d->max_dihedral_angle = a; }
+      update_min_max_dihedral_angle(a);
       total_angle+=a;
       ++nb_angle;
       a = CGAL::to_double(CGAL::abs(approx_dihedral_angle(p1, p3, p0, p2)));
-      if(a < d->min_dihedral_angle) { d->min_dihedral_angle = a; }
-      if(a > d->max_dihedral_angle) { d->max_dihedral_angle = a; }
+      update_min_max_dihedral_angle(a);
       total_angle+=a;
       ++nb_angle;
       a = CGAL::to_double(CGAL::abs(approx_dihedral_angle(p2, p3, p0, p1)));
-      if(a < d->min_dihedral_angle) { d->min_dihedral_angle = a; }
-      if(a > d->max_dihedral_angle) { d->max_dihedral_angle = a; }
+      update_min_max_dihedral_angle(a);
       total_angle+=a;
       ++nb_angle;
     }
-    d->mean_dihedral_angle = total_angle/(float)nb_angle;
+    d->mean_dihedral_angle = static_cast<float>(total_angle/nb_angle);
     d->nb_subdomains = sub_ids.size();
     d->nb_vertices = d->c3t3.number_of_vertices_in_complex();
     d->nb_tets = d->c3t3.number_of_cells();
@@ -2125,5 +2105,18 @@ void Scene_c3t3_item::newViewer(Viewer_i
     d->computeIntersections(viewer);
   }
 }
+
+Scene_c3t3_item* Scene_c3t3_item::clone() const
+{
+  return new Scene_c3t3_item(d->c3t3, d->is_surface);
+}
+
+void Scene_c3t3_item::set_sharp_edges_angle(double a) { d->sharp_edges_angle = a; }
+double Scene_c3t3_item::get_sharp_edges_angle() { return d->sharp_edges_angle; }
+
+void Scene_c3t3_item::set_detect_borders(bool b) { d->detect_borders = b;}
+bool Scene_c3t3_item::get_detect_borders() { return d->detect_borders; }
+
+
 #include "Scene_c3t3_item.moc"
 
diff -pruN 5.0.3-1/demo/Polyhedron/Scene_c3t3_item.h 5.1-1/demo/Polyhedron/Scene_c3t3_item.h
--- 5.0.3-1/demo/Polyhedron/Scene_c3t3_item.h	2020-08-01 19:02:43.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Scene_c3t3_item.h	2020-09-08 12:15:25.000000000 +0000
@@ -101,9 +101,8 @@ public:
   {
       return Scene_item::bbox();
   }
-  Scene_c3t3_item* clone() const  Q_DECL_OVERRIDE{
-    return 0;
-  }
+
+  Scene_c3t3_item* clone() const  Q_DECL_OVERRIDE;
 
   bool load_binary(std::istream& is);
 
@@ -156,6 +155,12 @@ public:
 
   QColor get_histogram_color(const double v) const;
 
+  void set_sharp_edges_angle(double d);
+  double get_sharp_edges_angle();
+
+  void set_detect_borders(bool b);
+  bool get_detect_borders();
+
   void itemAboutToBeDestroyed(Scene_item *) Q_DECL_OVERRIDE;
 
   void initializeBuffers(Viewer_interface *) const Q_DECL_OVERRIDE;
diff -pruN 5.0.3-1/demo/Polyhedron/Scene.cpp 5.1-1/demo/Polyhedron/Scene.cpp
--- 5.0.3-1/demo/Polyhedron/Scene.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Scene.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -704,7 +704,22 @@ Scene::draw_aux(bool with_names, CGAL::T
     Q_FOREACH(Item_id id, children)
     {
       Scene_item* item = m_entries[id];
-      if(item->alpha() == 1.0f)
+      Scene_group_item* group = qobject_cast<Scene_group_item*>(item);
+      bool is_transparent=false;
+      if(item->alpha() != 1.0f)
+        is_transparent = true;
+      else if(group)
+      {
+        for(const auto& child : group->getChildren())
+        {
+          if(group->getChild(child)->alpha() < 1.0f)
+          {
+            is_transparent = true;
+            break;
+          }
+        }
+      }
+      if(!is_transparent)
         opaque_items.push_back(id);
       else
         transparent_items.push_back(id);
@@ -834,9 +849,9 @@ Scene::draw_aux(bool with_names, CGAL::T
       //blending
       program.bind();
       vaos[viewer]->bind();
-      viewer->glClearColor(background.redF(),
-                           background.greenF(),
-                           background.blueF(),
+      viewer->glClearColor((GLclampf)background.redF(),
+                           (GLclampf)background.greenF(),
+                           (GLclampf)background.blueF(),
                            0.0f);
       viewer->glDisable(GL_DEPTH_TEST);
       viewer->glClear(GL_COLOR_BUFFER_BIT);
@@ -1200,6 +1215,8 @@ bool Scene::sort_lists(QVector<QList<int
 }
 void Scene::moveRowUp()
 {
+  if(selectionIndices().isEmpty())
+    return;
   QVector<QList<int> >sorted_lists(1);
   QList<int> to_select;
   //sort lists according to the indices of each item in its container (scene or group)
@@ -1251,6 +1268,8 @@ void Scene::moveRowUp()
 }
 void Scene::moveRowDown()
 {
+  if(selectionIndices().isEmpty())
+    return;
   QVector<QList<int> >sorted_lists(1);
   QList<int> to_select;
   //sort lists according to the indices of each item in its container (scene or group)
@@ -1300,7 +1319,9 @@ void Scene::moveRowDown()
       }
     }
   }
-  selectionChanged(to_select);
+  if(!to_select.isEmpty()){
+    selectionChanged(to_select);
+  }
 }
 Scene::Item_id Scene::mainSelectionIndex() const {
     return (selectionIndices().size() == 1) ? selected_item : -1;
diff -pruN 5.0.3-1/demo/Polyhedron/Scene_implicit_function_item.cpp 5.1-1/demo/Polyhedron/Scene_implicit_function_item.cpp
--- 5.0.3-1/demo/Polyhedron/Scene_implicit_function_item.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Scene_implicit_function_item.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -102,9 +102,9 @@ void Scene_implicit_function_item_priv::
 
     const CGAL::Three::Scene_item::Bbox& b = item->bbox();
     float x,y,z;
-    z = (b.zmax()+b.zmin())/2.0;
-    x = (b.xmax()+b.xmin())/2.0;
-    y = (b.ymax()+b.ymin())/2.0;
+    z = static_cast<float>(b.zmax()+b.zmin())/2.0f;
+    x = static_cast<float>(b.xmax()+b.xmin())/2.0f;
+    y = static_cast<float>(b.ymax()+b.ymin())/2.0f;
     // The Quad
     {
 
diff -pruN 5.0.3-1/demo/Polyhedron/Scene_item_rendering_helper.cpp 5.1-1/demo/Polyhedron/Scene_item_rendering_helper.cpp
--- 5.0.3-1/demo/Polyhedron/Scene_item_rendering_helper.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Scene_item_rendering_helper.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -78,9 +78,9 @@ void PRIV::compute_diag_bbox()
 {
   const Scene_item::Bbox& b_box = item->bbox();
   _diag_bbox = CGAL::approximate_sqrt(
-          (b_box.xmax() - b_box.xmin())*(b_box.xmax() - b_box.xmin())
-        + (b_box.ymax() - b_box.ymin())*(b_box.ymax() - b_box.ymin())
-        + (b_box.zmax() - b_box.zmin())*(b_box.zmax() - b_box.zmin())
+          CGAL::square(b_box.xmax() - b_box.xmin())
+        + CGAL::square(b_box.ymax() - b_box.ymin())
+        + CGAL::square(b_box.zmax() - b_box.zmin())
         );
 }
 
diff -pruN 5.0.3-1/demo/Polyhedron/Scene_polygon_soup_item.cpp 5.1-1/demo/Polyhedron/Scene_polygon_soup_item.cpp
--- 5.0.3-1/demo/Polyhedron/Scene_polygon_soup_item.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Scene_polygon_soup_item.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -1,6 +1,3 @@
-#include <vector>
-#include <queue>
-
 #include "Scene_polygon_soup_item.h"
 #include "Scene_surface_mesh_item.h"
 #include <CGAL/Three/Viewer_interface.h>
@@ -13,9 +10,6 @@
 #include <QApplication>
 #include <QtDebug>
 
-#include <set>
-#include <stack>
-#include <algorithm>
 #include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
 
 #include <CGAL/IO/OFF_reader.h>
@@ -23,6 +17,7 @@
 #include <CGAL/version.h>
 
 #include <CGAL/Polygon_mesh_processing/polygon_soup_to_polygon_mesh.h>
+#include <CGAL/Polygon_mesh_processing/polygon_mesh_to_polygon_soup.h>
 #include <CGAL/Polygon_mesh_processing/orient_polygon_soup.h>
 #include <CGAL/Polygon_mesh_processing/orientation.h>
 #include <CGAL/Polygon_mesh_processing/repair.h>
@@ -43,8 +38,14 @@
 #include <boost/accumulators/statistics/max.hpp>
 #include <boost/accumulators/statistics/median.hpp>
 
+#include <algorithm>
+#include <iostream>
 #include <map>
+#include <queue>
+#include <set>
+#include <stack>
 #include <streambuf>
+#include <vector>
 
 using namespace CGAL::Three;
 typedef Viewer_interface Vi;
@@ -366,40 +367,14 @@ void Scene_polygon_soup_item::init_polyg
   d->oriented = false;
 }
 
-
-#include <iostream>
 template<class PolygonMesh>
 void polygon_mesh_to_soup(PolygonMesh& mesh, Polygon_soup& soup)
 {
   soup.clear();
-  typedef typename boost::property_map<PolygonMesh, boost::vertex_point_t>::type VPMap;
-  VPMap vpmap = get(boost::vertex_point, mesh);
-  std::map<typename boost::graph_traits<PolygonMesh>::vertex_descriptor, int> vim;
-  int index=0;
-  //fill points
-  for(typename boost::graph_traits<PolygonMesh>::vertex_iterator vit =
-      vertices(mesh).begin(); vit != vertices(mesh).end(); ++vit)
-  {
-    soup.points.push_back(get(vpmap, *vit));
-    vim.insert(std::make_pair(*vit, index++));
-  }
-  //fill triangles
-  for(typename boost::graph_traits<PolygonMesh>::face_iterator fit =
-      faces(mesh).begin(); fit != faces(mesh).end(); ++fit)
-  {
-    Polygon_soup::Polygon_3 polygon;
-    for(typename boost::graph_traits<PolygonMesh>::halfedge_descriptor hd :
-                  CGAL::halfedges_around_face(halfedge(*fit, mesh), mesh))
-    {
-      polygon.push_back(vim[target(hd, mesh)]);
-    }
-    soup.polygons.push_back(polygon);
-  }
+  CGAL::Polygon_mesh_processing::polygon_mesh_to_polygon_soup(mesh, soup.points, soup.polygons);
   soup.fill_edges();
-
 }
 
-
 void Scene_polygon_soup_item::load(Scene_surface_mesh_item* sm_item) {
   if(!sm_item) return;
   if(!sm_item->face_graph()) return;
@@ -537,7 +512,7 @@ Scene_polygon_soup_item::toolTip() const
     return QString();
 
   QString str = QObject::tr("<p><b>%1</b> (mode: %5, color: %6)<br />"
-                     "<i>Polygons soup</i></p>"
+                     "<i>Polygon soup</i></p>"
                      "<p>Number of vertices: %2<br />"
                      "Number of polygons: %3</p>")
     .arg(this->name())
@@ -571,12 +546,14 @@ Scene_polygon_soup_item::draw(CGAL::Thre
 
       if(d->soup->fcolors.empty())
         getTriangleContainer(Priv::Flat_facets)->setColor(this->color());
+      getTriangleContainer(Priv::Flat_facets)->setAlpha(alpha());
       getTriangleContainer(Priv::Flat_facets)->draw(viewer, d->soup->fcolors.empty());
     }
     else if(renderingMode() == Gouraud)
     {
       if(d->soup->vcolors.empty())
         getTriangleContainer(Priv::Smooth_facets)->setColor(this->color());
+      getTriangleContainer(Priv::Smooth_facets)->setAlpha(alpha());
       getTriangleContainer(Priv::Smooth_facets)->draw(viewer, d->soup->vcolors.empty());
     }
   }
diff -pruN 5.0.3-1/demo/Polyhedron/Scene_polyhedron_selection_item.cpp 5.1-1/demo/Polyhedron/Scene_polyhedron_selection_item.cpp
--- 5.0.3-1/demo/Polyhedron/Scene_polyhedron_selection_item.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Scene_polyhedron_selection_item.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -2497,7 +2497,7 @@ QString Scene_polyhedron_selection_item:
       return QString("n/a");
     if(is_triangle_mesh(*d->poly)){
       bool self_intersect
-          = CGAL::Polygon_mesh_processing::does_self_intersect(*(d->poly));
+        = CGAL::Polygon_mesh_processing::does_self_intersect<CGAL::Parallel_if_available_tag>(*(d->poly));
       if (self_intersect)
         return QString("Yes");
       else
@@ -2610,3 +2610,17 @@ CGAL::Three::Scene_item::Header_data Sce
   data.titles.append(QString("Average"));
   return data;
 }
+
+
+void Scene_polyhedron_selection_item::updateDisplayedIds(QEvent* e)
+{
+  if(e->type() == QEvent::MouseButtonRelease )
+  {
+    QMouseEvent* mouse_event = static_cast<QMouseEvent*>(e);
+    if((mouse_event->button() == Qt::RightButton || mouse_event->button() == Qt::MiddleButton)
+       && temp_selected_vertices.size() == 1) {
+      fg_vertex_descriptor vh = *temp_selected_vertices.begin();
+      poly_item->updateIds(vh);
+    }
+  }
+}
diff -pruN 5.0.3-1/demo/Polyhedron/Scene_polyhedron_selection_item.h 5.1-1/demo/Polyhedron/Scene_polyhedron_selection_item.h
--- 5.0.3-1/demo/Polyhedron/Scene_polyhedron_selection_item.h	2020-08-01 19:02:44.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Scene_polyhedron_selection_item.h	2020-09-08 12:15:26.000000000 +0000
@@ -799,6 +799,7 @@ public:
   }
 
   void selection_changed(bool);
+  void updateDisplayedIds(QEvent *e);
 
 Q_SIGNALS:
   void updateInstructions(QString);
@@ -875,6 +876,8 @@ protected:
       }
     }
 
+    updateDisplayedIds(gen_event);
+
     if(!visible() || !k_ring_selector.state.shift_pressing) { return false; }
     if(gen_event->type() == QEvent::Wheel)
     {
diff -pruN 5.0.3-1/demo/Polyhedron/Scene_polylines_item.cpp 5.1-1/demo/Polyhedron/Scene_polylines_item.cpp
--- 5.0.3-1/demo/Polyhedron/Scene_polylines_item.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Scene_polylines_item.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -218,6 +218,7 @@ Scene_polylines_item_private::computeSph
       // At this point, 'corner_polyline_nb' gives the multiplicity of all
       // corners.
       //Finds the centers of the spheres and their color
+      int s_id = 0;
       for(iterator
           p_it = corner_polyline_nb.begin(),
           end = corner_polyline_nb.end();
@@ -253,7 +254,7 @@ Scene_polylines_item_private::computeSph
           }
 
           CGAL::Color c(colors[0], colors[1], colors[2]);
-          spheres->add_sphere(K::Sphere_3(center+offset, spheres_drawn_square_radius), c);
+          spheres->add_sphere(K::Sphere_3(center+offset, spheres_drawn_square_radius),s_id++, c);
       }
       spheres->setToolTip(
             QString("<p>Legend of endpoints colors: <ul>"
@@ -560,7 +561,7 @@ void Scene_polylines_item::change_corner
         d->draw_extremities = (r > 0);
         if(r>0 && !d->spheres)
         {
-          d->spheres = new Scene_spheres_item(this, false);
+          d->spheres = new Scene_spheres_item(this, 0, false);
           d->spheres->setName("Corner spheres");
           d->spheres->setRenderingMode(Gouraud);
           connect(d->spheres, SIGNAL(destroyed()), this, SLOT(reset_spheres()));
diff -pruN 5.0.3-1/demo/Polyhedron/Scene_spheres_item.cpp 5.1-1/demo/Polyhedron/Scene_spheres_item.cpp
--- 5.0.3-1/demo/Polyhedron/Scene_spheres_item.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Scene_spheres_item.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -1,4 +1,5 @@
 #include "Scene_spheres_item.h"
+#include<fstream>
 #include <CGAL/Three/Triangle_container.h>
 #include <CGAL/Three/Edge_container.h>
 #include <QApplication>
@@ -9,14 +10,25 @@ typedef Viewer_interface Vi;
 typedef Triangle_container Tc;
 typedef Edge_container Ec;
 
+QVector4D cgal_plane_to_vector4d(CGAL::Epick::Plane_3 plane) {
+  return {
+    static_cast<float>(-plane.a()),
+    static_cast<float>(-plane.b()),
+    static_cast<float>(-plane.c()),
+    static_cast<float>(-plane.d()) };
+}
+
 struct Scene_spheres_item_priv
 {
   typedef CGAL::Exact_predicates_inexact_constructions_kernel Kernel;
+  typedef CGAL::Sphere_3<Kernel> Sphere;
+  typedef std::pair<Sphere, CGAL::Color> Sphere_pair;
+  typedef std::vector<std::vector<Sphere_pair> > Spheres_container;
 
-  Scene_spheres_item_priv(bool planed, Scene_spheres_item* parent)
+  Scene_spheres_item_priv(bool planed, std::size_t max_index, Scene_spheres_item* parent, bool pickable)
     :precision(36)
     ,has_plane(planed)
-
+    ,pickable(pickable)
   {
     item = parent;
     create_flat_and_wire_sphere(1.0f,vertices,normals, edges);
@@ -24,6 +36,7 @@ struct Scene_spheres_item_priv
     edges_colors.clear();
     centers.clear();
     radius.clear();
+    spheres.resize(max_index + 1);
     nb_centers = 0;
     nb_edges = 0;
     nb_vertices = 0;
@@ -33,6 +46,9 @@ struct Scene_spheres_item_priv
   ~Scene_spheres_item_priv()
   {
   }
+
+  void pick(int &id)const;
+
   void initializeBuffers(CGAL::Three::Viewer_interface *viewer)const;
 
   int precision;
@@ -44,6 +60,7 @@ struct Scene_spheres_item_priv
   mutable std::vector<float> edges;
   mutable std::vector<float> colors;
   mutable std::vector<float> edges_colors;
+  mutable std::vector<float> picking_colors;
   mutable std::vector<float> centers;
   mutable std::vector<float> radius;
   mutable QOpenGLShaderProgram *program;
@@ -53,12 +70,61 @@ struct Scene_spheres_item_priv
   mutable bool model_sphere_is_up;
   Scene_spheres_item* item;
   QString tooltip;
+  mutable Spheres_container spheres;
+  const bool pickable;
 
 };
-Scene_spheres_item::Scene_spheres_item(Scene_group_item* parent, bool planed)
+
+void Scene_spheres_item_priv::pick(int& id) const
+{
+  if(!pickable)
+    return;
+
+  if(id >= static_cast<int>(spheres.size()))
+  {
+    id = -1;
+  }
+
+  int offset = 0;
+  float color[4];
+  for(std::size_t i=0; i<spheres.size(); ++i)
+  {
+   for( std::size_t j = 0; j< spheres[i].size(); ++j)
+   {
+     if(id == -1 || i != static_cast<std::size_t>(id))
+     {
+       color[0]=spheres[i][j].second.red()/255.0;
+       color[1]=spheres[i][j].second.green()/255.0;
+       color[2]=spheres[i][j].second.blue()/255.0;
+     }
+     else
+     {
+       color[0]=1.0f;
+       color[1]=1.0f;
+       color[2]=0.0f;
+     }
+     Vbo* color_vbo = item->getTriangleContainer(0)->getVbo(Tc::FColors);
+     color_vbo->bind();
+     color_vbo->vbo.write(offset*3*sizeof(float), color, 3*sizeof(float));
+     color_vbo->release();
+     ++offset;
+   }
+  }
+}
+
+Scene_spheres_item::Scene_spheres_item(Scene_group_item* parent, std::size_t max_index, bool planed, bool pickable)
 {
   setParent(parent);
-  d = new Scene_spheres_item_priv(planed, this);
+  d = new Scene_spheres_item_priv(planed, max_index, this, pickable);
+  if(pickable)
+  {
+  //for picking
+  setTriangleContainer(1,
+                       new Tc(planed ? Vi::PROGRAM_CUTPLANE_SPHERES
+                                     : Vi::PROGRAM_SPHERES
+                                       ,false));
+  }
+  //for drawing
   setTriangleContainer(0,
                        new Tc(planed ? Vi::PROGRAM_CUTPLANE_SPHERES
                                      : Vi::PROGRAM_SPHERES
@@ -79,6 +145,12 @@ void Scene_spheres_item_priv::initialize
   item->getTriangleContainer(0)->initializeBuffers(viewer);
   item->getTriangleContainer(0)->setFlatDataSize(nb_vertices);
   item->getTriangleContainer(0)->setCenterSize(nb_centers);
+  if(pickable)
+  {
+    item->getTriangleContainer(1)->initializeBuffers(viewer);
+    item->getTriangleContainer(1)->setFlatDataSize(nb_vertices);
+    item->getTriangleContainer(1)->setCenterSize(nb_centers);
+  }
   vertices.clear();
   vertices.shrink_to_fit();
   item->getEdgeContainer(0)->initializeBuffers(viewer);
@@ -91,6 +163,8 @@ void Scene_spheres_item_priv::initialize
   centers.swap(centers);
   colors.clear();
   colors.swap(colors);
+  picking_colors.clear();
+  picking_colors.shrink_to_fit();
   radius.clear();
   radius.swap(radius);
   edges_colors.clear();
@@ -113,12 +187,39 @@ void Scene_spheres_item::draw(Viewer_int
     setBuffersFilled(true);
     setBuffersInit(viewer, true);
   }
-  if(d->has_plane)
-  {
-    QVector4D cp(d->plane.a(),d->plane.b(),d->plane.c(),d->plane.d());
-    getTriangleContainer(0)->setPlane(cp);
-  }
-  getTriangleContainer(0)->draw(viewer, false);
+  int deviceWidth = viewer->camera()->screenWidth();
+  int deviceHeight = viewer->camera()->screenHeight();
+    if(d->has_plane)
+    {
+      QVector4D cp = cgal_plane_to_vector4d(d->plane);
+      getTriangleContainer(0)->setPlane(cp);
+      if(d->pickable)
+        getTriangleContainer(1)->setPlane(cp);
+    }
+    if(d->pickable && (d->spheres.size() > 1 && viewer->inDrawWithNames()))
+    {
+      getTriangleContainer(1)->getVao(viewer)->program->setAttributeValue("normals", QVector3D(0,0,0));
+      getTriangleContainer(1)->draw(viewer, false);
+    }
+    else
+    {
+      getTriangleContainer(0)->draw(viewer, false);
+    }
+    if(d->pickable && (d->spheres.size() > 1 && viewer->inDrawWithNames()))
+    {
+      int rowLength = deviceWidth * 4; // data asked in RGBA,so 4 bytes.
+      const static int dataLength = rowLength * deviceHeight;
+      GLubyte* buffer = new GLubyte[dataLength];
+      // Qt uses upper corner for its origin while GL uses the lower corner.
+      QPoint picking_target = viewer->mapFromGlobal(QCursor::pos());
+      viewer->glReadPixels(picking_target.x(), deviceHeight-1-picking_target.y(), 1, 1, GL_RGBA, GL_UNSIGNED_BYTE, buffer);
+      int ID = (buffer[0] + buffer[1] * 256 +buffer[2] * 256*256) ;
+      if(buffer[0]*buffer[1]*buffer[2] < 255*255*255)
+      {
+        d->pick(ID);
+        picked(ID);
+      }
+    }
 }
 
 void Scene_spheres_item::drawEdges(Viewer_interface *viewer) const
@@ -140,35 +241,53 @@ void Scene_spheres_item::drawEdges(Viewe
 
   if(d->has_plane)
   {
-    QVector4D cp(d->plane.a(),d->plane.b(),d->plane.c(),d->plane.d());
+    QVector4D cp = cgal_plane_to_vector4d(d->plane);
     getEdgeContainer(0)->setPlane(cp);
   }
   getEdgeContainer(0)->draw(viewer, false);
 
 }
-void Scene_spheres_item::add_sphere(const CGAL::Sphere_3<Kernel>& sphere, CGAL::Color color)
+void Scene_spheres_item::add_sphere(const Sphere &sphere, std::size_t index,  CGAL::Color color)
 {
-    d->colors.push_back((float)color.red()/255);
-    d->colors.push_back((float)color.green()/255);
-    d->colors.push_back((float)color.blue()/255);
-
-    d->edges_colors.push_back((float)color.red()/255);
-    d->edges_colors.push_back((float)color.green()/255);
-    d->edges_colors.push_back((float)color.blue()/255);
+  if((int)index > (int)d->spheres.size() - 1)
+    d->spheres.resize(index+1);
 
-    d->centers.push_back(sphere.center().x());
-    d->centers.push_back(sphere.center().y());
-    d->centers.push_back(sphere.center().z());
+  d->spheres[index].push_back(std::make_pair(sphere, color));
 
-    d->radius.push_back(CGAL::sqrt(sphere.squared_radius()));
+  d->colors.push_back((float)color.red()/255);
+  d->colors.push_back((float)color.green()/255);
+  d->colors.push_back((float)color.blue()/255);
+  if(d->pickable)
+  {
+    int R = (index & 0x000000FF) >>  0;
+    int G = (index & 0x0000FF00) >>  8;
+    int B = (index & 0x00FF0000) >> 16;
+    float r = R/255.0;
+    float g = G/255.0;
+    float b = B/255.0;
+    d->picking_colors.push_back(r);
+    d->picking_colors.push_back(g);
+    d->picking_colors.push_back(b);
+  }
+  d->edges_colors.push_back((float)color.red()/255);
+  d->edges_colors.push_back((float)color.green()/255);
+  d->edges_colors.push_back((float)color.blue()/255);
+
+  d->centers.push_back(sphere.center().x());
+  d->centers.push_back(sphere.center().y());
+  d->centers.push_back(sphere.center().z());
+  d->radius.push_back(CGAL::sqrt(sphere.squared_radius()));
 }
 
 void Scene_spheres_item::clear_spheres()
 {
+  d->spheres.clear();
   d->colors.clear();
   d->edges_colors.clear();
   d->centers.clear();
   d->radius.clear();
+  d->picking_colors.clear();
+  setBuffersFilled(false);
 }
 void Scene_spheres_item::setPrecision(int prec) { d->precision = prec; }
 void Scene_spheres_item::setPlane(Kernel::Plane_3 p_plane) { d->plane = p_plane; }
@@ -176,6 +295,11 @@ void Scene_spheres_item::invalidateOpenG
 {
   setBuffersFilled(false);
   getTriangleContainer(0)->reset_vbos(NOT_INSTANCED);
+  getTriangleContainer(0)->reset_vbos(COLORS);
+  if(d->pickable){
+    getTriangleContainer(1)->reset_vbos(NOT_INSTANCED);
+    getTriangleContainer(1)->reset_vbos(COLORS);
+  }
   getEdgeContainer(0)->reset_vbos(NOT_INSTANCED);
 }
 
@@ -209,6 +333,11 @@ void Scene_spheres_item::computeElements
 
     getTriangleContainer(0)->allocate(Tc::Flat_normals, d->normals.data(),
                                       static_cast<int>(d->normals.size()*sizeof(float)));
+
+    if(d->pickable)
+      getTriangleContainer(1)->allocate(Tc::Flat_vertices, d->vertices.data(),
+                                        static_cast<int>(d->vertices.size()*sizeof(float)));
+
     d->nb_vertices = d->vertices.size();
 
   }
@@ -216,9 +345,19 @@ void Scene_spheres_item::computeElements
                                     static_cast<int>(d->colors.size()*sizeof(float)));
   getTriangleContainer(0)->allocate(Tc::Radius, d->radius.data(),
                                     static_cast<int>(d->radius.size()*sizeof(float)));
-
   getTriangleContainer(0)->allocate(Tc::Facet_centers, d->centers.data(),
                                     static_cast<int>(d->centers.size()*sizeof(float)));
+
+  if(d->pickable)
+  {
+    getTriangleContainer(1)->allocate(Tc::FColors, d->picking_colors.data(),
+                                      static_cast<int>(d->picking_colors.size()*sizeof(float)));
+    getTriangleContainer(1)->allocate(Tc::Radius, d->radius.data(),
+                                      static_cast<int>(d->radius.size()*sizeof(float)));
+    getTriangleContainer(1)->allocate(Tc::Facet_centers, d->centers.data(),
+                                      static_cast<int>(d->centers.size()*sizeof(float)));
+  }
+
   if(!d->model_sphere_is_up)
   {
     getEdgeContainer(0)->allocate(Ec::Vertices, d->edges.data(),
@@ -237,7 +376,6 @@ void Scene_spheres_item::computeElements
 
   getEdgeContainer(0)->allocate(Ec::Centers, d->centers.data(),
                                 static_cast<int>(d->centers.size()*sizeof(float)));
-
   d->nb_centers = d->centers.size();
 
 }
@@ -254,3 +392,40 @@ void Scene_spheres_item::gl_initializati
     setBuffersFilled(true);
   }
 }
+
+void Scene_spheres_item::compute_bbox() const
+{
+  Bbox box = Bbox();
+  for(std::size_t id=0; id<d->spheres.size(); ++id)
+  {
+    for(Sphere_pair pair : d->spheres[id])
+    {
+      box += pair.first.bbox();
+    }
+  }
+  _bbox = box;
+}
+
+bool Scene_spheres_item::save(const std::string& file_name)const
+{
+
+  std::ofstream out(file_name.c_str());
+  if(!out) { return false; }
+
+  std::size_t nb_spheres=0;
+  for(std::size_t i = 0; i<d->spheres.size(); ++i)
+    nb_spheres += d->spheres[i].size();
+
+  out<<nb_spheres << " " << d->spheres.size() -1<<"\n";
+
+  for(std::size_t i = 0; i<d->spheres.size(); ++i)
+  {
+    for(const Sphere_pair& pair : d->spheres[i])
+    {
+      out << i << " " << pair.first.center() << " " << CGAL::sqrt(pair.first.squared_radius())<<"\n";
+    }
+  }
+  out << "\n";
+  return true;
+}
+
diff -pruN 5.0.3-1/demo/Polyhedron/Scene_spheres_item.h 5.1-1/demo/Polyhedron/Scene_spheres_item.h
--- 5.0.3-1/demo/Polyhedron/Scene_spheres_item.h	2020-08-01 19:02:44.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Scene_spheres_item.h	2020-09-08 12:15:27.000000000 +0000
@@ -15,27 +15,37 @@
 #include <QList>
 #include <vector>
 struct Scene_spheres_item_priv;
+
+/* This item contains spheres and associated colors. They are kept in a Spheres_container,
+ * sorted by the value of their "index". This item also has an internal picking mechanism that
+ * colorizes all the spheres that has the same index as the one that has been picked.
+ * The picking is only usable if several indices exist.
+ * If all the spheres have the index 0, they can have independant colors (generally used by the items that
+ * have a Scene_spheres_item child).
+*/
 class SCENE_BASIC_OBJECTS_EXPORT Scene_spheres_item
     : public CGAL::Three::Scene_item_rendering_helper
 {
   Q_OBJECT
 public:
   typedef CGAL::Exact_predicates_inexact_constructions_kernel Kernel;
-  typedef std::pair<CGAL::Sphere_3<Kernel>*, CGAL::Color> Sphere_pair ;
+  typedef CGAL::Sphere_3<Kernel> Sphere;
+  typedef std::pair<Sphere, CGAL::Color> Sphere_pair;
+  typedef std::vector<std::vector<Sphere_pair> > Spheres_container;
 
-  Scene_spheres_item(Scene_group_item* parent, bool planed = false);
+  Scene_spheres_item(Scene_group_item* parent, std::size_t max_index = 0, bool planed = false, bool pickable = true);
 
   ~Scene_spheres_item();
 
-  bool isFinite() const Q_DECL_OVERRIDE{ return false; }
+  bool isFinite() const Q_DECL_OVERRIDE{ return true; }
   bool isEmpty() const Q_DECL_OVERRIDE{ return false; }
   Scene_item* clone() const Q_DECL_OVERRIDE{return 0;}
   QString toolTip() const Q_DECL_OVERRIDE;
   bool supportsRenderingMode(RenderingMode m) const Q_DECL_OVERRIDE{
     return (m == Gouraud || m == Wireframe);
   }
-  void compute_bbox() const Q_DECL_OVERRIDE{ _bbox = Bbox(); }
-  void add_sphere(const CGAL::Sphere_3<Kernel> &sphere, CGAL::Color = CGAL::Color(120,120,120));
+  void compute_bbox() const Q_DECL_OVERRIDE;
+  void add_sphere(const Sphere &sphere, std::size_t index = 0, CGAL::Color = CGAL::Color(120,120,120));
   void clear_spheres();
   void setPrecision(int prec);
   void gl_initialization(CGAL::Three::Viewer_interface* viewer);
@@ -45,10 +55,14 @@ public:
   void setPlane(Kernel::Plane_3 p_plane);
   void setToolTip(QString s);
   void setColor(QColor c) Q_DECL_OVERRIDE;
+  bool save(const std::string &file_name) const;
+
+
   void initializeBuffers(Viewer_interface *) const Q_DECL_OVERRIDE;
   void computeElements() const Q_DECL_OVERRIDE;
 Q_SIGNALS:
   void on_color_changed();
+  void picked(std::size_t id) const;
 protected:
   friend struct Scene_spheres_item_priv;
   Scene_spheres_item_priv* d;
diff -pruN 5.0.3-1/demo/Polyhedron/Scene_surface_mesh_item.cpp 5.1-1/demo/Polyhedron/Scene_surface_mesh_item.cpp
--- 5.0.3-1/demo/Polyhedron/Scene_surface_mesh_item.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Scene_surface_mesh_item.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -33,6 +33,7 @@
 #include <CGAL/Polygon_mesh_processing/polygon_soup_to_polygon_mesh.h>
 #include "triangulate_primitive.h"
 
+#include <CGAL/exceptions.h>
 #include <CGAL/IO/File_writer_wavefront.h>
 #include <CGAL/IO/generic_copy_OFF.h>
 #include <CGAL/IO/OBJ_reader.h>
@@ -135,6 +136,7 @@ struct Scene_surface_mesh_item_priv{
                               << GouraudPlusEdges
                               << Points;
     item->setProperty("classname", QString("surface_mesh"));
+    ids_need_update = false;
   }
 
   Scene_surface_mesh_item_priv(SMesh* sm, Scene_surface_mesh_item *parent):
@@ -167,7 +169,8 @@ struct Scene_surface_mesh_item_priv{
                               << Gouraud
                                  << GouraudPlusEdges
                               << Points;
-    item->setProperty("classname", QString("surface_mesh"));
+    item->setProperty("classname", QString("surface_mesh"));\
+    ids_need_update = false;
   }
 
   ~Scene_surface_mesh_item_priv()
@@ -223,7 +226,6 @@ struct Scene_surface_mesh_item_priv{
   mutable bool edges_displayed;
   mutable bool faces_displayed;
   mutable bool all_displayed;
-  mutable QList<double> text_ids;
   mutable std::vector<TextItem*> targeted_id;
 
   std::string comments;
@@ -238,7 +240,6 @@ struct Scene_surface_mesh_item_priv{
   mutable bool isinit;
   mutable std::vector<unsigned int> idx_data_;
   mutable std::size_t idx_data_size;
-  mutable std::map<unsigned int, unsigned int> current_indices; //map im values to ghosts-free values
   mutable std::vector<unsigned int> idx_edge_data_;
   mutable std::size_t idx_edge_data_size;
   mutable std::vector<unsigned int> idx_feature_edge_data_;
@@ -266,6 +267,7 @@ struct Scene_surface_mesh_item_priv{
   bool has_nm_vertices;
   int genus;
   bool self_intersect;
+  bool ids_need_update;
   mutable QSlider* alphaSlider;
   QList<RenderingMode> supported_rendering_modes;
 };
@@ -1581,7 +1583,7 @@ QString Scene_surface_mesh_item::compute
     try{
       CGAL::Polygon_mesh_processing::non_manifold_vertices(*d->smesh_, OutputIterator());
     }
-    catch( CGAL::internal::Throw_at_output::Throw_at_output_exception& )
+    catch( CGAL::internal::Throw_at_output_exception& )
     {
       d->has_nm_vertices = true;
     }
@@ -1659,7 +1661,7 @@ QString Scene_surface_mesh_item::compute
   {
     //todo : add a test about cache validity
     if(is_triangle_mesh(*d->smesh_))
-      d->self_intersect = CGAL::Polygon_mesh_processing::does_self_intersect(*(d->smesh_));
+      d->self_intersect = CGAL::Polygon_mesh_processing::does_self_intersect<CGAL::Parallel_if_available_tag>(*(d->smesh_));
     if (d->self_intersect)
       return QString("Yes");
     else if(is_triangle_mesh(*d->smesh_))
@@ -2016,6 +2018,7 @@ void Scene_surface_mesh_item_priv::fillT
                                                  CGAL::Three::Viewer_interface *viewer,
                                                  const CGAL::qglviewer::Vec& offset)
 {
+  all_displayed = false;
   compute_displayed_ids(*smesh_,
                         viewer,
                         selected_fh,
@@ -2367,9 +2370,31 @@ void Scene_surface_mesh_item::updateVert
             getTriangleContainer(0)->getVbo(Tri::Smooth_normals),
             new_point,id);
     }
+    d->ids_need_update = true;
+    redraw();
+  }
+}
+
+void Scene_surface_mesh_item::updateIds(vertex_descriptor vh)
+{
+  if(d->ids_need_update &&
+     (d->faces_displayed || d->vertices_displayed || d->edges_displayed))
+  {
+    invalidate_aabb_tree();
+
+    if(d->all_displayed)
+    {
+      d->killIds();
+      d->all_displayed = true;
+      ::printVertexIds(*d->smesh_, d->textVItems);
+    }
+    else
+    {
+      d->fillTargetedIds(face(halfedge(vh, *d->smesh_), *d->smesh_),
+                         face_graph()->point(vh), CGAL::Three::Three::mainViewer(), CGAL::Three::Three::mainViewer()->offset());
+    }
+    d->ids_need_update = false;
   }
-  invalidate_aabb_tree();
-  redraw();
 }
 
 void Scene_surface_mesh_item::switchToGouraudPlusEdge(bool b)
diff -pruN 5.0.3-1/demo/Polyhedron/Scene_surface_mesh_item.h 5.1-1/demo/Polyhedron/Scene_surface_mesh_item.h
--- 5.0.3-1/demo/Polyhedron/Scene_surface_mesh_item.h	2020-08-01 19:02:44.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Scene_surface_mesh_item.h	2020-09-08 12:15:27.000000000 +0000
@@ -156,6 +156,7 @@ public:
   void computeElements() const Q_DECL_OVERRIDE;
   void initializeBuffers(CGAL::Three::Viewer_interface*)const Q_DECL_OVERRIDE;
   void updateVertex(vertex_descriptor vh);
+  void updateIds(vertex_descriptor vh);
   void switchToGouraudPlusEdge(bool b); //replace flatPlusEdge by gouraudPlusEdge and ban Flat.
 Q_SIGNALS:
   void item_is_about_to_be_changed();
diff -pruN 5.0.3-1/demo/Polyhedron/Server_ws.cpp 5.1-1/demo/Polyhedron/Server_ws.cpp
--- 5.0.3-1/demo/Polyhedron/Server_ws.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Server_ws.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,100 @@
+#include <QtCore/QCoreApplication>
+#include <QtCore/QCommandLineParser>
+#include <QtCore/QCommandLineOption>
+#include <QNetworkInterface>
+#include <QApplication>
+#include <QMessageBox>
+#include "Server_ws.h"
+
+#include <iostream>
+
+
+EchoServer::EchoServer(quint16 port) :
+  QObject(),
+  m_pWebSocketServer(new QWebSocketServer(QStringLiteral("Echo Server"),
+                                          QWebSocketServer::NonSecureMode, this))
+{
+  if (m_pWebSocketServer->listen(QHostAddress::Any, port)) {
+    connect(m_pWebSocketServer, &QWebSocketServer::newConnection,
+            this, &EchoServer::onNewConnection);
+    connect(m_pWebSocketServer, &QWebSocketServer::closed, this, &EchoServer::closed);
+  }
+  QHostAddress local_host("0.0.0.0");
+
+  //to avoid printing 127.0.0.1. Not realy sure it won't ever print the external ipv4 though.
+  const QHostAddress &localhost = QHostAddress(QHostAddress::LocalHost);
+  for (const QHostAddress &address: QNetworkInterface::allAddresses()) {
+    if (address.protocol() == QAbstractSocket::IPv4Protocol && address != localhost)
+    {
+      local_host= address;
+      break;
+    }
+  }
+  QMessageBox mb(QMessageBox::NoIcon, "WS Server",
+                 tr("WebSockets Server started.\nEnter the following address in\nyour Network Preferences to be able to join it :\n"
+                    "ws://%1:%2").arg(local_host.toString()).arg(port), QMessageBox::Ok);
+  mb.setTextInteractionFlags(Qt::TextSelectableByMouse);
+  mb.exec();
+}
+
+EchoServer::~EchoServer()
+{
+  m_pWebSocketServer->close();
+  qDeleteAll(m_clients.begin(), m_clients.end());
+}
+
+void EchoServer::onNewConnection()
+{
+  QWebSocket *pSocket = m_pWebSocketServer->nextPendingConnection();
+
+  connect(pSocket, &QWebSocket::textMessageReceived, this, &EchoServer::processTextMessage);
+  connect(pSocket, &QWebSocket::binaryMessageReceived, this, &EchoServer::processBinaryMessage);
+  connect(pSocket, &QWebSocket::disconnected, this, &EchoServer::socketDisconnected);
+
+  m_clients << pSocket;
+}
+
+void EchoServer::processTextMessage(QString message)
+{
+  QWebSocket *pClient = qobject_cast<QWebSocket *>(sender());
+  for(auto *client : m_clients) {
+    if(client != pClient)
+      client->sendTextMessage(message);
+  }
+}
+
+void EchoServer::processBinaryMessage(QByteArray message)
+{
+  QWebSocket *pClient = qobject_cast<QWebSocket *>(sender());
+  if (pClient) {
+    pClient->sendBinaryMessage(message);
+  }
+}
+
+void EchoServer::socketDisconnected()
+{
+  QWebSocket *pClient = qobject_cast<QWebSocket *>(sender());
+  if (pClient) {
+    m_clients.removeAll(pClient);
+    pClient->deleteLater();
+  }
+}
+
+int main(int argc, char *argv[])
+{
+  QApplication a(argc, argv);
+  QCommandLineParser parser;
+  parser.setApplicationDescription("WS Server");
+  parser.addHelpOption();
+  QCommandLineOption portOption(QStringList() << "p" << "port",
+                                QCoreApplication::translate("main", "Port for echoserver [default: 1234]."),
+                                QCoreApplication::translate("main", "port"), QLatin1Literal("1234"));
+  parser.addOption(portOption);
+  parser.process(a);
+  int port = parser.value(portOption).toInt();
+  EchoServer *server = new EchoServer(port);
+  QObject::connect(server, &EchoServer::closed, &a, &QCoreApplication::quit);
+
+  return a.exec();
+}
+
diff -pruN 5.0.3-1/demo/Polyhedron/Server_ws.h 5.1-1/demo/Polyhedron/Server_ws.h
--- 5.0.3-1/demo/Polyhedron/Server_ws.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Server_ws.h	2020-09-08 12:15:27.000000000 +0000
@@ -0,0 +1,30 @@
+#ifndef SERVER_WS_H
+#define SERVER_WS_H
+
+#include  <QObject>
+#include <QWebSocketServer>
+#include <QWebSocket>
+
+class EchoServer : public QObject
+{
+  Q_OBJECT
+public:
+  explicit EchoServer(quint16 port);
+  ~EchoServer();
+
+
+Q_SIGNALS:
+  void closed();
+
+private Q_SLOTS:
+  void onNewConnection();
+  void processTextMessage(QString message);
+  void processBinaryMessage(QByteArray message);
+  void socketDisconnected();
+
+private:
+  QWebSocketServer *m_pWebSocketServer;
+  QList<QWebSocket *> m_clients;
+};
+
+#endif // SERVER_WS_H
diff -pruN 5.0.3-1/demo/Polyhedron/SSH_dialog.ui 5.1-1/demo/Polyhedron/SSH_dialog.ui
--- 5.0.3-1/demo/Polyhedron/SSH_dialog.ui	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/demo/Polyhedron/SSH_dialog.ui	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,151 @@
+<?xml version="1.0" encoding="UTF-8"?>
+<ui version="4.0">
+ <class>SSHDialog</class>
+ <widget class="QDialog" name="SSHDialog">
+  <property name="geometry">
+   <rect>
+    <x>0</x>
+    <y>0</y>
+    <width>415</width>
+    <height>441</height>
+   </rect>
+  </property>
+  <property name="windowTitle">
+   <string>SSH Settings</string>
+  </property>
+  <layout class="QVBoxLayout" name="verticalLayout_2">
+   <item>
+    <layout class="QVBoxLayout" name="verticalLayout">
+     <item>
+      <widget class="QGroupBox" name="groupBox_2">
+       <property name="title">
+        <string>SSH Preferences</string>
+       </property>
+       <layout class="QVBoxLayout" name="verticalLayout_3">
+        <item>
+         <widget class="QGroupBox" name="userBox">
+          <property name="title">
+           <string>User</string>
+          </property>
+          <layout class="QHBoxLayout" name="horizontalLayout">
+           <item>
+            <widget class="QLineEdit" name="userEdit"/>
+           </item>
+          </layout>
+         </widget>
+        </item>
+        <item>
+         <widget class="QGroupBox" name="serverBox">
+          <property name="title">
+           <string>Server</string>
+          </property>
+          <layout class="QHBoxLayout" name="horizontalLayout_2">
+           <item>
+            <widget class="QLineEdit" name="serverEdit"/>
+           </item>
+          </layout>
+         </widget>
+        </item>
+        <item>
+         <widget class="QGroupBox" name="pkBox">
+          <property name="title">
+           <string>Public Key </string>
+          </property>
+          <layout class="QHBoxLayout" name="horizontalLayout_3">
+           <item>
+            <widget class="QLineEdit" name="publicEdit"/>
+           </item>
+           <item>
+            <widget class="QPushButton" name="pubButton">
+             <property name="text">
+              <string>Browse...</string>
+             </property>
+            </widget>
+           </item>
+          </layout>
+         </widget>
+        </item>
+        <item>
+         <widget class="QGroupBox" name="privkBox">
+          <property name="title">
+           <string>Private Key</string>
+          </property>
+          <layout class="QHBoxLayout" name="horizontalLayout_4">
+           <item>
+            <widget class="QLineEdit" name="privkEdit"/>
+           </item>
+           <item>
+            <widget class="QPushButton" name="privButton">
+             <property name="text">
+              <string>Browse...</string>
+             </property>
+            </widget>
+           </item>
+          </layout>
+         </widget>
+        </item>
+       </layout>
+      </widget>
+     </item>
+     <item>
+      <widget class="QGroupBox" name="groupBox">
+       <property name="title">
+        <string>Camera Synchronization Server</string>
+       </property>
+       <layout class="QVBoxLayout" name="verticalLayout_4">
+        <item>
+         <widget class="QLineEdit" name="wsEdit"/>
+        </item>
+       </layout>
+      </widget>
+     </item>
+    </layout>
+   </item>
+   <item>
+    <widget class="QDialogButtonBox" name="buttonBox">
+     <property name="orientation">
+      <enum>Qt::Horizontal</enum>
+     </property>
+     <property name="standardButtons">
+      <set>QDialogButtonBox::Cancel|QDialogButtonBox::Ok</set>
+     </property>
+    </widget>
+   </item>
+  </layout>
+ </widget>
+ <resources/>
+ <connections>
+  <connection>
+   <sender>buttonBox</sender>
+   <signal>accepted()</signal>
+   <receiver>SSHDialog</receiver>
+   <slot>accept()</slot>
+   <hints>
+    <hint type="sourcelabel">
+     <x>248</x>
+     <y>254</y>
+    </hint>
+    <hint type="destinationlabel">
+     <x>157</x>
+     <y>274</y>
+    </hint>
+   </hints>
+  </connection>
+  <connection>
+   <sender>buttonBox</sender>
+   <signal>rejected()</signal>
+   <receiver>SSHDialog</receiver>
+   <slot>reject()</slot>
+   <hints>
+    <hint type="sourcelabel">
+     <x>316</x>
+     <y>260</y>
+    </hint>
+    <hint type="destinationlabel">
+     <x>286</x>
+     <y>274</y>
+    </hint>
+   </hints>
+  </connection>
+ </connections>
+</ui>
diff -pruN 5.0.3-1/demo/Polyhedron/Three.cpp 5.1-1/demo/Polyhedron/Three.cpp
--- 5.0.3-1/demo/Polyhedron/Three.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Three.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -5,6 +5,7 @@
 #include <CGAL/Three/Polyhedron_demo_plugin_interface.h>
 #include <QMdiArea>
 #include <QMdiSubWindow>
+#include <QMessageBox>
 #include "Messages_interface.h"
 using namespace CGAL::Three;
 
@@ -175,6 +176,18 @@ void Three::error(QString s)
 {
   qobject_cast<Messages_interface*>(mainWindow())->message_error(s);
 }
+void Three::information(QString title, QString s)
+{
+  QMessageBox::information(mainWindow(), title, s);
+}
+void Three::warning(QString title, QString s)
+{
+  QMessageBox::warning(mainWindow(), title, s);
+}
+void Three::error(QString title, QString s)
+{
+  QMessageBox::critical(mainWindow(), title, s);
+}
 RenderingMode Three::defaultSurfaceMeshRenderingMode()
 {
   return s_defaultSMRM;
diff -pruN 5.0.3-1/demo/Polyhedron/Use_ssh.cpp 5.1-1/demo/Polyhedron/Use_ssh.cpp
--- 5.0.3-1/demo/Polyhedron/Use_ssh.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Use_ssh.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,418 @@
+// Copyright (c) 2020  GeometryFactory (France).  All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL$
+// $Id$
+// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Maxime Gimeno
+
+#ifdef CGAL_USE_SSH
+
+#include <CGAL/Three/Three.h>
+
+#include "CGAL/Use_ssh.h"
+#include <iostream>
+#include <fstream>
+#include <cstring>
+#include <vector>
+#include <chrono>
+#include <thread>
+#include <sstream>
+
+#include <QMessageBox>
+#include <QStringList>
+
+bool test_result(int res)
+{
+  switch(res){
+
+  case SSH_AUTH_ERROR:
+  {
+    std::cerr<<"Auth failed with error: "<<std::endl;
+    return false;
+  }
+  case SSH_AUTH_DENIED:
+  {
+    std::cerr<<"The server doesn't accept that public key as an authentication token. Try another key or another method."<<std::endl;
+    return false;
+  }
+  case SSH_AUTH_PARTIAL :
+  {
+    std::cerr<<"You've been partially authenticated, you still have to use another method."<<std::endl;
+    return false;
+  }
+  case SSH_OK:
+    return true;
+  case SSH_EOF:
+    std::cerr<<"key doesn't exist."<<std::endl;
+    return false;
+  default:
+    return false;
+  }
+  return true;
+}
+namespace CGAL{
+namespace ssh_internal{
+
+bool establish_ssh_session(ssh_session &session,
+                           const char* user,
+                           const char* server,
+                           const char* pub_key_path,
+                           const char* priv_key_path,
+                           const char* priv_key_password)
+{
+  int port = 22;
+
+  //Can use SSH_LOG_PROTOCOL here for verbose output
+  int verbosity = SSH_LOG_NOLOG;
+  int res;
+  //retry 4 times max each time the connection asks to be retried.
+  for(int k = 0; k < 4; ++k)
+  {
+    session = ssh_new();
+    ssh_options_set( session, SSH_OPTIONS_LOG_VERBOSITY, &verbosity );
+    ssh_options_set( session, SSH_OPTIONS_PORT, &port );
+    ssh_options_set( session, SSH_OPTIONS_USER, user );
+    ssh_options_set( session, SSH_OPTIONS_HOST, server);
+
+    ssh_connect(session);
+#if LIBSSH_VERSION_MAJOR <1 && LIBSSH_VERSION_MINOR < 8
+    if( ssh_is_server_known(session) != SSH_SERVER_KNOWN_OK )
+#else
+      if( ssh_session_is_known_server(session) != SSH_KNOWN_HOSTS_OK )
+#endif
+    {
+      if(QMessageBox::warning(CGAL::Three::Three::mainWindow(), QString("Unknown Server"),
+                              QString ("The server you are trying to join is not known.\n"
+                                       "Do you wish to add it to the known servers list and continue?"),
+                              QMessageBox::Yes | QMessageBox::No) != QMessageBox::Yes)
+      {
+        return false;
+      }
+#if LIBSSH_VERSION_MAJOR <1 && LIBSSH_VERSION_MINOR < 8
+      if( ssh_write_knownhost(session) != SSH_OK )
+#else
+      if( ssh_session_update_known_hosts(session) != SSH_OK )
+#endif
+      {
+        std::cerr << "writeKnownHost failed" << std::endl;
+        return false;
+      }
+      else
+      {
+        ssh_connect(session);
+      }
+    }
+    ssh_key pubkey = ssh_key_new();
+    ssh_pki_import_pubkey_file(pub_key_path, &pubkey);
+    res = ssh_userauth_try_publickey(session, NULL, pubkey);
+    if(res == SSH_AUTH_AGAIN)
+      ssh_disconnect(session);
+    else
+      break;
+  }
+
+  if(!test_result(res))
+  {
+    ssh_disconnect(session);
+    return false;
+  }
+
+  ssh_key privkey = ssh_key_new();
+  res = ssh_pki_import_privkey_file(priv_key_path, priv_key_password, NULL, NULL, &privkey);
+  if (!test_result(res))
+  {
+    ssh_disconnect(session);
+    return false;
+  }
+  res = ssh_userauth_publickey(session, NULL, privkey);
+  if(!test_result(res))
+  {
+    ssh_disconnect(session);
+    return false;
+  }
+  return true;
+}
+
+
+bool establish_ssh_session_from_agent(ssh_session& session,
+                                      const char *user,
+                                      const char *server,
+                                      const char *pub_key_path)
+{
+  int port = 22;
+
+  //Can use SSH_LOG_PROTOCOL here for verbose output
+  int verbosity = SSH_LOG_NOLOG;
+  int res;
+  //retry 4 times max each time the connection asks to be retried.
+  for(int k = 0; k < 4; ++k)
+  {
+    session = ssh_new();
+    ssh_options_set( session, SSH_OPTIONS_LOG_VERBOSITY, &verbosity );
+    ssh_options_set( session, SSH_OPTIONS_PORT, &port );
+    ssh_options_set( session, SSH_OPTIONS_USER, user );
+    ssh_options_set( session, SSH_OPTIONS_HOST, server);
+
+    ssh_connect(session);
+#if LIBSSH_VERSION_MAJOR <1 && LIBSSH_VERSION_MINOR < 8
+    if( ssh_is_server_known(session) != SSH_SERVER_KNOWN_OK )
+#else
+      if( ssh_session_is_known_server(session) != SSH_KNOWN_HOSTS_OK )
+#endif
+    {
+      if(QMessageBox::warning(CGAL::Three::Three::mainWindow(), QString("Unknown Server"),
+                              QString ("The server you are trying to join is not known.\n"
+                                       "Do you wish to add it to the known servers list and continue?"),
+                              QMessageBox::Yes | QMessageBox::No) != QMessageBox::Yes)
+      {
+        return false;
+      }
+#if LIBSSH_VERSION_MAJOR <1 && LIBSSH_VERSION_MINOR < 8
+      if( ssh_write_knownhost(session) != SSH_OK )
+#else
+      if( ssh_session_update_known_hosts(session) != SSH_OK )
+#endif
+      {
+        std::cerr << "writeKnownHost failed" << std::endl;
+        return false;
+      }
+      else
+      {
+        ssh_connect(session);
+      }
+    }
+      ssh_key pubkey = ssh_key_new();
+      ssh_pki_import_pubkey_file(pub_key_path, &pubkey);
+      res = ssh_userauth_try_publickey(session, NULL, pubkey);
+      if(res == SSH_AUTH_AGAIN)
+        ssh_disconnect(session);
+      else
+        break;
+    }
+
+
+  if(!test_result(res))
+  {
+    ssh_disconnect(session);
+    return false;
+  }
+
+  res = ssh_userauth_agent(session, user);
+  if(!test_result(res))
+  {
+    ssh_disconnect(session);
+    return false;
+  }
+  return true;
+}
+
+void close_connection(ssh_session &session)
+{
+  ssh_disconnect(session);
+}
+
+bool push_file(ssh_session &session,
+               const char* dest_path,
+               const char* filepath)
+{
+  //copy a file
+  ssh_scp scp = ssh_scp_new(
+        session, SSH_SCP_WRITE | SSH_SCP_RECURSIVE, "/tmp");
+  if (scp == NULL)
+  {
+    std::cerr<<"Error allocating scp session: %s\n"
+            << ssh_get_error(session)<<std::endl;
+    return false;
+  }
+  int res = ssh_scp_init(scp);
+  if(res != SSH_OK)
+  {
+    std::cerr<< "Error initializing scp session: %s\n"
+             << ssh_get_error(session)<<std::endl;
+    ssh_scp_free(scp);
+    ssh_disconnect(session);
+    return false;
+  }
+  //read a file into a buffer
+  std::ifstream file(filepath, std::ios::binary | std::ios::ate);
+  if(!file)
+  {
+    std::cerr<<"File not found."<<std::endl;
+    ssh_scp_free(scp);
+    ssh_disconnect(session);
+    return false;
+  }
+  std::streamsize size = file.tellg();
+  file.seekg(0, std::ios::beg);
+
+  std::vector<char> buffer(size);
+  if (!file.read(buffer.data(), size))
+  {
+    std::cerr<<"error while reading file."<<std::endl;
+    ssh_disconnect(session);
+    return false;
+  }
+  //push a file to /tmp
+  res = ssh_scp_push_directory(scp, ".", 0755);
+  if (res != SSH_OK)
+  {
+    std::cerr<<"Can't create remote directory: %s\n"
+            <<ssh_get_error(session)<<std::endl;
+    ssh_disconnect(session);
+    return false;
+  }
+  res = ssh_scp_push_file
+      (scp, dest_path, size, 0644);
+  if (res != SSH_OK)
+  {
+    std::cerr<< "Can't open remote file: %s\n"
+             << ssh_get_error(session)<<std::endl;
+    ssh_disconnect(session);
+    return false;
+  }
+  res = ssh_scp_write(scp, buffer.data(), size);
+  //some versions of libssh don't copy everything without this.
+  //This is the case for the official version on Ubuntu 18.04
+  std::chrono::duration<int, std::micro> timespan(size);
+  std::this_thread::sleep_for(timespan);
+  if (res != SSH_OK)
+  {
+    std::cerr<< "Can't write to remote file: %s\n"
+             << ssh_get_error(session)<<std::endl;
+    ssh_disconnect(session);
+    return false;
+  }
+  return true;
+}
+
+bool pull_file(ssh_session &session,
+               const char* from_path,
+               const char* to_path)
+{
+  int rc;
+  std::size_t size;
+  std::size_t processed = 0;
+  std::vector<char> buffer;
+
+  ssh_scp scp = ssh_scp_new(
+        session, SSH_SCP_READ | SSH_SCP_RECURSIVE, from_path);
+  if (scp == NULL)
+  {
+    std::cerr<<"Error allocating scp session: %s\n"
+            << ssh_get_error(session)<<std::endl;
+    return false;
+  }
+  int res = ssh_scp_init(scp);
+  if(res != SSH_OK)
+  {
+    std::cerr<< "Error initializing scp session: %s\n"
+             << ssh_get_error(session)<<std::endl;
+    ssh_scp_free(scp);
+    ssh_disconnect(session);
+    return false;
+  }
+  rc = ssh_scp_pull_request(scp);
+  if (rc != SSH_SCP_REQUEST_NEWFILE)
+  {
+    std::cerr<< "Error receiving information about file: %s\n"
+             << ssh_get_error(session)<<std::endl;
+    ssh_scp_free(scp);
+    ssh_disconnect(session);
+    return false;
+  }
+  size = ssh_scp_request_get_size64(scp);
+  buffer.resize(size);
+  if(ssh_scp_accept_request(scp) != SSH_OK)
+  {
+    std::cerr<< "Could not accept request."<<std::endl;
+    ssh_scp_free(scp);
+    ssh_disconnect(session);
+    return false;
+  }
+
+  do{
+    rc = ssh_scp_read(scp, buffer.data() + processed, size-processed);
+    if (rc == SSH_ERROR)
+    {
+      std::cerr<< "Error receiving file data: %s\n"<< ssh_get_error(session)<<std::endl;
+      //free(buffer);
+      ssh_scp_free(scp);
+      ssh_disconnect(session);
+      return false;
+    }
+    else
+      processed += rc;
+  }while(processed != size);
+  std::ofstream file(to_path, std::ios::binary |std::ios::trunc);
+  if(!file.write(buffer.data(), size))
+  {
+    std::cerr<<"Error while writing file."<<std::endl;
+  }
+  file.close();
+  ssh_scp_free(scp);
+  return true;
+}
+
+bool explore_the_galaxy(ssh_session &session,
+                        QStringList& files)
+{
+  ssh_channel channel;
+  int rc;
+  channel = ssh_channel_new(session);
+  if (channel == NULL) return false;
+  rc = ssh_channel_open_session(channel);
+  if (rc != SSH_OK)
+  {
+    ssh_channel_free(channel);
+    return rc;
+  }
+  rc = ssh_channel_request_exec(channel, "ls /tmp");
+  if (rc != SSH_OK)
+  {
+    ssh_channel_close(channel);
+    ssh_channel_free(channel);
+    return rc;
+  }
+
+  char buffer[256];
+  int nbytes;
+  nbytes = ssh_channel_read(channel, buffer, sizeof(buffer), 0);
+  while (nbytes > 0)
+  {
+
+    std::string sbuf(buffer, nbytes);
+    if(sbuf.find("Polyhedron_demo_") != std::string::npos)
+    {
+      std::istringstream iss(sbuf);
+      std::string file;
+      while(iss >> file)
+      {
+        if(file.find("Polyhedron_demo_") != std::string::npos)
+        {
+          QString name(file.c_str());
+          files.push_back(name.remove("Polyhedron_demo_"));
+        }
+      }
+    }
+
+    nbytes = ssh_channel_read(channel, buffer, sizeof(buffer), 0);
+  }
+  if (nbytes < 0)
+  {
+    ssh_channel_close(channel);
+    ssh_channel_free(channel);
+    return false;
+  }
+  ssh_channel_send_eof(channel);
+  ssh_channel_close(channel);
+  ssh_channel_free(channel);
+  return true;
+}
+
+}// end of ssh_internal
+}// end of CGAL
+#endif
diff -pruN 5.0.3-1/demo/Polyhedron/Viewer.cpp 5.1-1/demo/Polyhedron/Viewer.cpp
--- 5.0.3-1/demo/Polyhedron/Viewer.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Viewer.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -16,6 +16,11 @@
 #include <QApplication>
 #include <QOpenGLDebugLogger>
 #include <QStyleFactory>
+#include <QAction>
+#include <QRegularExpressionMatch>
+#ifdef CGAL_USE_WEBSOCKETS
+#include <QtWebSockets/QWebSocket>
+#endif
 
 #include <CGAL/Three/Three.h>
 
@@ -40,6 +45,7 @@ public:
   bool inDrawWithNames;
   bool clipping;
   bool projection_is_ortho;
+  bool cam_sharing;
   GLfloat gl_point_size;
   QVector4D clipbox[6];
   QPainter *painter;
@@ -51,6 +57,10 @@ public:
   QVector4D specular;
   float spec_power;
 
+  //Back and Front Colors
+  QColor front_color;
+  QColor back_color;
+
   // M e s s a g e s
   QString message;
   bool _displayMessage;
@@ -107,6 +117,12 @@ public:
   {
     return shader_programs;
   }
+#ifdef CGAL_USE_WEBSOCKETS
+  QWebSocket m_webSocket;
+#endif
+  bool is_connected;
+  QString session;
+  QUrl m_url;
 };
 
 class LightingDialog :
@@ -124,23 +140,23 @@ public:
                                .arg(d->position.y())
                                .arg(d->position.z()));
     QPalette palette;
-    ambient=QColor(255*d->ambient.x(),
-                   255*d->ambient.y(),
-                   255*d->ambient.z());
+    ambient=QColor::fromRgbF(d->ambient.x(),
+                             d->ambient.y(),
+                             d->ambient.z());
     palette.setColor(QPalette::Button,ambient);
     ambientButton->setPalette(palette);
     ambientButton->setStyle(QStyleFactory::create("Fusion"));
 
-    diffuse=QColor(255*d->diffuse.x(),
-                   255*d->diffuse.y(),
-                   255*d->diffuse.z());
+    diffuse=QColor::fromRgbF(d->diffuse.x(),
+                             d->diffuse.y(),
+                             d->diffuse.z());
     palette.setColor(QPalette::Button,diffuse);
     diffuseButton->setPalette(palette);
     diffuseButton->setStyle(QStyleFactory::create("Fusion"));
 
-    specular=QColor(255*d->specular.x(),
-                    255*d->specular.y(),
-                    255*d->specular.z());
+    specular=QColor::fromRgbF(d->specular.x(),
+                              d->specular.y(),
+                              d->specular.z());
     palette.setColor(QPalette::Button,specular);
     specularButton->setPalette(palette);
     specularButton->setStyle(QStyleFactory::create("Fusion"));
@@ -251,11 +267,23 @@ void Viewer::doBindings()
                           specular.split(",").at(2).toFloat(),
                           1.0f);
 
+  QString front_color = viewer_settings.value("front_color", QString("1.0,0.0,0.0")).toString();
+  d->front_color= QColor::fromRgbF(front_color.split(",").at(0).toFloat(),
+                                   front_color.split(",").at(1).toFloat(),
+                                   front_color.split(",").at(2).toFloat(),
+                         1.0f);
+  QString back_color = viewer_settings.value("back_color", QString("0.0,0.0,1.0")).toString();
+  d->back_color= QColor::fromRgbF( back_color.split(",").at(0).toFloat(),
+                                   back_color.split(",").at(1).toFloat(),
+                                   back_color.split(",").at(2).toFloat(),
+                         1.0f);
   d->spec_power = viewer_settings.value("spec_power", 51.8).toFloat();
   d->scene = 0;
   d->projection_is_ortho = false;
+  d->cam_sharing = false;
   d->twosides = false;
   this->setProperty("draw_two_sides", false);
+  this->setProperty("back_front_shading", false);
   d->macro_mode = false;
   d->inFastDrawing = true;
   d->inDrawWithNames = false;
@@ -263,6 +291,7 @@ void Viewer::doBindings()
   d->shader_programs.resize(NB_OF_PROGRAMS);
   d->textRenderer = new TextRenderer();
   d->is_2d_selection_mode = false;
+  d->is_connected = false;
 
   connect( d->textRenderer, SIGNAL(sendMessage(QString,int)),
            this, SLOT(printMessage(QString,int)) );
@@ -333,6 +362,7 @@ Viewer::Viewer(QWidget* parent,
   is_sharing = true;
   d->antialiasing = antialiasing;
   this->setProperty("draw_two_sides", false);
+  this->setProperty("back_front_shading", false);
   this->setProperty("helpText", QString("This is a sub-viewer. It displays the scene "
                                         "from another point of view. \n "));
   is_ogl_4_3 = sharedWidget->is_ogl_4_3;
@@ -367,6 +397,18 @@ Viewer::~Viewer()
                              .arg(d->specular.z()));
     viewer_settings.setValue("spec_power",
                              d->spec_power);
+    viewer_settings.setValue("front_color",
+                             QString("%1,%2,%3")
+                             .arg(d->front_color.redF())
+                             .arg(d->front_color.greenF())
+                             .arg(d->front_color.blueF()));
+    viewer_settings.setValue("back_color",
+                             QString("%1,%2,%3")
+                             .arg(d->back_color.redF())
+                             .arg(d->back_color.greenF())
+                             .arg(d->back_color.blueF()));
+
+    d->vao.destroy();
     if(d->_recentFunctions)
       delete d->_recentFunctions;
     if(d->painter)
@@ -401,6 +443,13 @@ void Viewer::setTwoSides(bool b)
 }
 
 
+void Viewer::setBackFrontShading(bool b)
+{
+  this->setProperty("back_front_shading", b);
+  update();
+}
+
+
 void Viewer::setFastDrawing(bool b)
 {
   d->inFastDrawing = b;
@@ -589,7 +638,7 @@ void Viewer::mousePressEvent(QMouseEvent
       d->showDistance(event->pos());
       event->accept();
   }
-  else {
+  else{
     makeCurrent();
     CGAL::QGLViewer::mousePressEvent(event);
   }
@@ -804,7 +853,7 @@ void Viewer::postSelection(const QPoint&
 }
 bool CGAL::Three::Viewer_interface::readFrame(QString s, CGAL::qglviewer::Frame& frame)
 {
-  QStringList list = s.split(" ", QString::SkipEmptyParts);
+  QStringList list = s.split(" ", CGAL_QT_SKIP_EMPTY_PARTS);
   if(list.size() != 7)
     return false;
   float vec[3];
@@ -947,7 +996,10 @@ void Viewer::attribBuffers(int program_n
         program->setUniformValue("light_spec", d->specular);
         program->setUniformValue("light_amb", d->ambient);
         program->setUniformValue("spec_power", d->spec_power);
+        program->setUniformValue("front_color", d->front_color);
+        program->setUniformValue("back_color", d->back_color);
         program->setUniformValue("is_two_side", d->twosides);
+        program->setUniformValue("back_front_shading", this->property("back_front_shading").toBool());
         break;
     }
     switch(program_name)
@@ -1371,7 +1423,7 @@ void Viewer::wheelEvent(QWheelEvent* e)
 {
   if(e->modifiers().testFlag(Qt::ShiftModifier))
   {
-    double delta = e->delta();
+    double delta = e->angleDelta().y();
     if(delta>0)
     {
       switch(camera()->type())
@@ -1508,12 +1560,18 @@ void Viewer_impl::showDistance(QPoint pi
         TextItem *ACoord = new TextItem(float(APoint.x),
                                         float(APoint.y),
                                         float(APoint.z),
-                                        QString("A(%1,%2,%3)").arg(APoint.x-viewer->offset().x).arg(APoint.y-viewer->offset().y).arg(APoint.z-viewer->offset().z), true, font, Qt::red, true);
+                                        QString("A(%1,%2,%3)")
+                                        .arg(APoint.x-viewer->offset().x, 0, 'g', 10)
+                                        .arg(APoint.y-viewer->offset().y, 0, 'g', 10)
+                                        .arg(APoint.z-viewer->offset().z, 0, 'g', 10), true, font, Qt::red, true);
         distance_text.append(ACoord);
         TextItem *BCoord = new TextItem(float(BPoint.x),
                                         float(BPoint.y),
                                         float(BPoint.z),
-                                        QString("B(%1,%2,%3)").arg(BPoint.x-viewer->offset().x).arg(BPoint.y-viewer->offset().y).arg(BPoint.z-viewer->offset().z), true, font, Qt::red, true);
+                                        QString("B(%1,%2,%3)")
+                                        .arg(BPoint.x-viewer->offset().x, 0, 'g', 10)
+                                        .arg(BPoint.y-viewer->offset().y, 0, 'g', 10)
+                                        .arg(BPoint.z-viewer->offset().z, 0, 'g', 10), true, font, Qt::red, true);
         distance_text.append(BCoord);
         CGAL::qglviewer::Vec centerPoint = 0.5*(BPoint+APoint);
         TextItem *centerCoord = new TextItem(float(centerPoint.x),
@@ -1531,7 +1589,7 @@ void Viewer_impl::showDistance(QPoint pi
                   .arg(BPoint.x-viewer->offset().x)
                   .arg(BPoint.y-viewer->offset().y)
                   .arg(BPoint.z-viewer->offset().z)
-                  .arg(dist)));
+                  .arg(dist, 0, 'g', 10)));
     }
 
 }
@@ -1639,6 +1697,12 @@ void Viewer::setTotalPass(int p)
 
 void Viewer::messageLogged(QOpenGLDebugMessage msg)
 {
+  //filter out useless warning
+  // From those two links, we decided we didn't care for this warning:
+  // https://community.khronos.org/t/vertex-shader-in-program-2-is-being-recompiled-based-on-gl-state/76019
+  // https://stackoverflow.com/questions/12004396/opengl-debug-context-performance-warning
+  if(msg.message().contains("is being recompiled"))
+    return;
   QString error;
 
   // Format based on severity
@@ -1724,7 +1788,7 @@ void Viewer::setLighting()
   connect(dialog->position_lineEdit, &QLineEdit::editingFinished,
           [this, dialog]()
   {
-    QStringList list = dialog->position_lineEdit->text().split(QRegExp(","), QString::SkipEmptyParts);
+    QStringList list = dialog->position_lineEdit->text().split(QRegExp(","), CGAL_QT_SKIP_EMPTY_PARTS);
     if (list.isEmpty()) return;
     if (list.size()!=3){
       QMessageBox *msgBox = new QMessageBox;
@@ -1733,7 +1797,7 @@ void Viewer::setLighting()
       msgBox->exec();
       return;
     }
-    double coords[3];
+    float coords[3];
     for(int j=0; j<3; ++j)
     {
       bool ok;
@@ -1755,9 +1819,9 @@ void Viewer::setLighting()
   //set ambient
   connect(dialog, &LightingDialog::s_ambient_changed,
           [this, dialog](){
-    d->ambient=QVector4D(dialog->ambient.redF(),
-                         dialog->ambient.greenF(),
-                         dialog->ambient.blueF(),
+    d->ambient=QVector4D((float)dialog->ambient.redF(),
+                         (float)dialog->ambient.greenF(),
+                         (float)dialog->ambient.blueF(),
                          1.0f);
     update();
   });
@@ -1765,18 +1829,18 @@ void Viewer::setLighting()
   //set diffuse
   connect(dialog, &LightingDialog::s_diffuse_changed,
           [this, dialog](){
-    d->diffuse=QVector4D(dialog->diffuse.redF(),
-                         dialog->diffuse.greenF(),
-                         dialog->diffuse.blueF(),
+    d->diffuse=QVector4D((float)dialog->diffuse.redF(),
+                         (float)dialog->diffuse.greenF(),
+                         (float)dialog->diffuse.blueF(),
                          1.0f);
     update();
   });
   //set specular
   connect(dialog, &LightingDialog::s_specular_changed,
           [this, dialog](){
-    d->specular=QVector4D(dialog->specular.redF() ,
-                         dialog->specular.greenF(),
-                         dialog->specular.blueF() ,
+    d->specular=QVector4D((float)dialog->specular.redF(),
+                          (float)dialog->specular.greenF(),
+                          (float)dialog->specular.blueF(),
                          1.0f);
     update();
 
@@ -1786,8 +1850,8 @@ void Viewer::setLighting()
   connect(dialog->buttonBox->button(QDialogButtonBox::StandardButton::RestoreDefaults), &QPushButton::clicked,
           [this](){
     d->position = QVector4D(0,0,1,1);
-    d->ambient=QVector4D(77.0/255,77.0/255,77.0/255, 1.0);
-    d->diffuse=QVector4D(204.0/255,204.0/255,204.0/255,1.0);
+    d->ambient=QVector4D(77.0f/255,77.0f/255,77.0f/255, 1.0);
+    d->diffuse=QVector4D(204.0f/255,204.0f/255,204.0f/255,1.0);
     d->specular=QVector4D(0,0,0,1.0);
     d->spec_power = 51;
     update();
@@ -1805,6 +1869,65 @@ void Viewer::setLighting()
   }
 }
 
+void Viewer::setBackFrontColors()
+{
+
+  //save current settings;
+
+  QColor prev_front_color = d->front_color;
+  QColor prev_back_color = d->back_color;
+  QDialog *dialog = new QDialog(this);
+  QDialogButtonBox* buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok
+                                   | QDialogButtonBox::Cancel, dialog);
+
+  connect(buttonBox, &QDialogButtonBox::accepted, dialog, &QDialog::accept);
+  connect(buttonBox, &QDialogButtonBox::rejected, dialog, &QDialog::reject);
+
+  QGridLayout* layout = new QGridLayout(dialog);
+  layout->addWidget(new QLabel("Front color: ",dialog),0,0);
+  QPalette front_palette;
+  front_palette.setColor(QPalette::Button, d->front_color);
+  QPushButton* frontButton = new QPushButton(dialog);
+  frontButton->setPalette(front_palette);
+  QPalette back_palette;
+  back_palette.setColor(QPalette::Button, d->back_color);
+  QPushButton* backButton = new QPushButton(dialog);
+  backButton->setPalette(back_palette);
+  layout->addWidget(frontButton,0,1);
+  layout->addWidget(new QLabel("Back color: ",dialog),1,0);
+  layout->addWidget(backButton,1,1);
+  layout->addWidget(buttonBox);
+  dialog->setLayout(layout);
+  connect(frontButton, &QPushButton::clicked,
+          [this, dialog, frontButton](){
+    QColorDialog *color_dial = new QColorDialog(dialog);
+    color_dial->exec();
+    QColor front_color = color_dial->selectedColor();
+    QPalette palette;
+    palette.setColor(QPalette::Button, front_color);
+    frontButton->setPalette(palette);
+    d->front_color= front_color;
+  });
+  connect(backButton, &QPushButton::clicked,
+          [this, dialog, backButton](){
+    QColorDialog *color_dial = new QColorDialog(dialog);
+    color_dial->exec();
+    QColor back_color = color_dial->selectedColor();
+    QPalette palette;
+    palette.setColor(QPalette::Button, back_color);
+    backButton->setPalette(palette);
+    d->back_color= back_color;
+
+  });
+  if(!dialog->exec())
+  {
+    //restore previous settings
+    d->front_color= prev_front_color;
+    d->back_color= prev_back_color;
+    return;
+  }
+}
+
 void Viewer::setGlPointSize(const GLfloat &p) { d->gl_point_size = p; }
 
 const GLfloat& Viewer::getGlPointSize() const { return d->gl_point_size; }
@@ -1835,6 +1958,87 @@ bool Viewer::isClipping() const
 {
   return d->clipping;
 }
+#ifdef CGAL_USE_WEBSOCKETS
+void Viewer::setShareCam(bool b, QString session)
+{
+  static bool init = false;
+  if(b)
+  {
+    d->cam_sharing = b;
+    d->session = session;
+    QString ws_url
+        = CGAL::Three::Three::mainWindow()->property("ws_url").toString();
+    if(ws_url.isEmpty())
+    {
+      QMessageBox::warning(this, "Error", "No Server configured. Please go to Edit->Preferences->Network Settings and fill the \"Camera Synchronization Server\" Field.");
+    }
+    else{
+      if(!init)
+      {
+        connect(&d->m_webSocket, &QWebSocket::connected, this, &Viewer::onSocketConnected);
+        connect(&d->m_webSocket, &QWebSocket::disconnected, this,[this]()
+        {
+          d->is_connected = false;
+          Viewer::socketClosed();
+        });
+        init = true;
+      }
+      d->m_webSocket.open(QUrl(ws_url));
+      QApplication::setOverrideCursor(Qt::WaitCursor);
+      QTimer::singleShot(1000, this, [this](){
+        QApplication::restoreOverrideCursor();
+        if(!d->is_connected){
+          QMessageBox::warning(CGAL::Three::Three::mainWindow(),
+                               "Connection failure",
+                               "The requested server was not found.");
+          setShareCam(false, "");
+        }
+      });
+    }
+  }
+  else
+  {
+    QAction* action = findChild<QAction*>("actionShareCamera");
+    action->setChecked(false);
+    d->m_webSocket.close();
+  }
+}
 
-#include "Viewer.moc"
+void Viewer::onSocketConnected()
+{
+  connect(&d->m_webSocket, &QWebSocket::textMessageReceived,
+          this, &Viewer::onTextMessageSocketReceived);
+  connect(camera()->frame(), &CGAL::qglviewer::ManipulatedCameraFrame::manipulated,
+          this, [this](){
+    if(d->cam_sharing){
+      QString cam_state = QString("[%1] %2").arg(d->session).arg(dumpCameraCoordinates());
+      //send to server
+      d->m_webSocket.sendTextMessage(cam_state);
+    }
+  });
+  d->is_connected = true;
+}
 
+void Viewer::onTextMessageSocketReceived(QString message)
+{
+  QString session;
+  QString position;
+  QRegularExpression re("\\[(.*)\\] (.*)");
+  QRegularExpressionMatch match = re.match(message);
+  session = match.captured(1);
+  position = match.captured(2);
+  if(session != d->session){
+    return;
+  }
+  QStringList sl = position.split(" ");
+  if(sl.size() != 7)
+    return;
+
+  CGAL::qglviewer::Vec pos(sl[0].toDouble(),sl[1].toDouble(),sl[2].toDouble());
+  CGAL::qglviewer::Quaternion q(sl[3].toDouble(),sl[4].toDouble(),
+      sl[5].toDouble(),sl[6].toDouble());
+  camera()->frame()->setPositionAndOrientation(pos, q);
+  update();
+}
+#endif
+#include "Viewer.moc"
diff -pruN 5.0.3-1/demo/Polyhedron/Viewer.h 5.1-1/demo/Polyhedron/Viewer.h
--- 5.0.3-1/demo/Polyhedron/Viewer.h	2020-08-01 19:02:44.000000000 +0000
+++ 5.1-1/demo/Polyhedron/Viewer.h	2020-09-08 12:15:27.000000000 +0000
@@ -95,12 +95,14 @@ public:
 Q_SIGNALS:
   void sendMessage(QString);
   void doneInitGL(CGAL::Three::Viewer_interface*);
+  void socketClosed();
 public Q_SLOTS:
   //! Sets the antialiasing to true or false.
   void setAntiAliasing(bool b) Q_DECL_OVERRIDE;
   //! If b is true, facets will be ligted from both internal and external sides.
   //! If b is false, only the side that is exposed to the light source will be lighted.
   void setTwoSides(bool b) Q_DECL_OVERRIDE;
+  void setBackFrontShading(bool b) Q_DECL_OVERRIDE;
   void SetOrthoProjection( bool b) Q_DECL_OVERRIDE;
   //! If b is true, some items are displayed in a simplified version when moving the camera.
   //! If b is false, items display is never altered, even when moving.
@@ -127,8 +129,14 @@ public Q_SLOTS:
   }
 
   void setLighting();
+  void setBackFrontColors();
 
   void messageLogged(QOpenGLDebugMessage);
+#ifdef CGAL_USE_WEBSOCKETS
+  void setShareCam(bool, QString);
+  void onSocketConnected();
+  void onTextMessageSocketReceived(QString message);
+#endif
 
 protected:
   void paintEvent(QPaintEvent *)Q_DECL_OVERRIDE;
diff -pruN 5.0.3-1/demo/Polyline_simplification_2/include/CGAL/Qt/Polyline_simplification_2_graphics_item.h 5.1-1/demo/Polyline_simplification_2/include/CGAL/Qt/Polyline_simplification_2_graphics_item.h
--- 5.0.3-1/demo/Polyline_simplification_2/include/CGAL/Qt/Polyline_simplification_2_graphics_item.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/demo/Polyline_simplification_2/include/CGAL/Qt/Polyline_simplification_2_graphics_item.h	2020-09-08 12:15:28.000000000 +0000
@@ -11,7 +11,7 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyline_simplification_2/demo/Polyline_simplification_2/include/CGAL/Qt/Polyline_simplification_2_graphics_item.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyline_simplification_2/demo/Polyline_simplification_2/include/CGAL/Qt/Polyline_simplification_2_graphics_item.h $
 // $Id: Polyline_simplification_2_graphics_item.h 590612a 2020-03-26T13:25:16+01:00 Sébastien Loriot
 //
 //
diff -pruN 5.0.3-1/demo/Polyline_simplification_2/include/CGAL/Qt/TriangulationArrangementGraphicsItem.h 5.1-1/demo/Polyline_simplification_2/include/CGAL/Qt/TriangulationArrangementGraphicsItem.h
--- 5.0.3-1/demo/Polyline_simplification_2/include/CGAL/Qt/TriangulationArrangementGraphicsItem.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/demo/Polyline_simplification_2/include/CGAL/Qt/TriangulationArrangementGraphicsItem.h	2020-09-08 12:15:28.000000000 +0000
@@ -11,7 +11,7 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyline_simplification_2/demo/Polyline_simplification_2/include/CGAL/Qt/TriangulationArrangementGraphicsItem.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyline_simplification_2/demo/Polyline_simplification_2/include/CGAL/Qt/TriangulationArrangementGraphicsItem.h $
 // $Id: TriangulationArrangementGraphicsItem.h 590612a 2020-03-26T13:25:16+01:00 Sébastien Loriot
 //
 //
diff -pruN 5.0.3-1/demo/Polyline_simplification_2/Polyline_simplification_2.cpp 5.1-1/demo/Polyline_simplification_2/Polyline_simplification_2.cpp
--- 5.0.3-1/demo/Polyline_simplification_2/Polyline_simplification_2.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyline_simplification_2/Polyline_simplification_2.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -207,9 +207,6 @@ MainWindow::MainWindow()
 
   this->addRecentFiles(this->menuFile, this->actionQuit);
   connect(this, SIGNAL(openRecentFile(QString)), this, SLOT(open(QString)));
-
-//  QObject::connect(thresholdSlider, SIGNAL(valueChanged(int)), this, SLOT(set_Threshold(int)));
-
 }
 
 
diff -pruN 5.0.3-1/demo/Polyline_simplification_2/Polyline_simplification_2.ui 5.1-1/demo/Polyline_simplification_2/Polyline_simplification_2.ui
--- 5.0.3-1/demo/Polyline_simplification_2/Polyline_simplification_2.ui	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Polyline_simplification_2/Polyline_simplification_2.ui	2020-09-08 12:12:20.000000000 +0000
@@ -279,14 +279,6 @@
   <include location="../icons/Triangulation_2.qrc" />
   <include location="../icons/Input.qrc" />
  </resources>
- <connections>
-  <connection>
-   <sender>thresholdSlider</sender>
-   <signal>valueChanged(int)</signal>
-   <receiver>MainWindow</receiver>
-   <slot>setThreshold(int)</slot>
-  </connection>
- </connections>
  <slots>
   <slot>setThreshold(int)</slot>
  </slots>
diff -pruN 5.0.3-1/demo/Principal_component_analysis/CMakeLists.txt 5.1-1/demo/Principal_component_analysis/CMakeLists.txt
--- 5.0.3-1/demo/Principal_component_analysis/CMakeLists.txt	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Principal_component_analysis/CMakeLists.txt	2020-09-08 12:12:20.000000000 +0000
@@ -19,9 +19,8 @@ include_directories( ./ )
 find_package(CGAL COMPONENTS Qt5)
 
 find_package(Eigen3 3.1.0) #(requires 3.1.0 or greater)
-if (EIGEN3_FOUND)
-  include( ${EIGEN3_USE_FILE} )
-else()
+include(CGAL_Eigen_support)
+if (NOT TARGET CGAL::Eigen_support)
   message(STATUS "NOTICE: This project requires the Eigen library, and will not be compiled.")
   return()
 endif()
@@ -49,7 +48,7 @@ if(CGAL_Qt5_FOUND AND Qt5_FOUND )
   add_executable  ( PCA_demo PCA_demo.cpp ${UI_FILES} ${CGAL_Qt5_RESOURCE_FILES} ${CGAL_Qt5_MOC_FILES})
 
   target_link_libraries( PCA_demo PRIVATE
-    CGAL::CGAL CGAL::CGAL_Qt5 Qt5::Gui)
+    CGAL::CGAL CGAL::CGAL_Qt5 CGAL::Eigen_support Qt5::Gui)
 
   add_to_cached_list( CGAL_EXECUTABLE_TARGETS PCA_demo )
 
diff -pruN 5.0.3-1/demo/Segment_Delaunay_graph_2/svd-typedefs.h 5.1-1/demo/Segment_Delaunay_graph_2/svd-typedefs.h
--- 5.0.3-1/demo/Segment_Delaunay_graph_2/svd-typedefs.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/demo/Segment_Delaunay_graph_2/svd-typedefs.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/demo/Segment_Delaunay_graph_2/svd-typedefs.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/demo/Segment_Delaunay_graph_2/svd-typedefs.h $
 // $Id: svd-typedefs.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/demo/Snap_rounding_2/Snap_rounding_2.cpp 5.1-1/demo/Snap_rounding_2/Snap_rounding_2.cpp
--- 5.0.3-1/demo/Snap_rounding_2/Snap_rounding_2.cpp	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/demo/Snap_rounding_2/Snap_rounding_2.cpp	2020-09-08 12:12:19.000000000 +0000
@@ -132,7 +132,7 @@ MainWindow::MainWindow()
   scene.setItemIndexMethod(QGraphicsScene::NoIndex);
   this->graphicsView->setScene(&scene);
   // Turn the vertical axis upside down
-  this->graphicsView->matrix().scale(1, -1);
+  this->graphicsView->transform().scale(1, -1);
   this->graphicsView->setMouseTracking(true);
 
   rgi = new CGAL::Qt::RegularGridGraphicsItem<K>(delta, delta);
diff -pruN 5.0.3-1/demo/Stream_lines_2/Stream_lines_2.cpp 5.1-1/demo/Stream_lines_2/Stream_lines_2.cpp
--- 5.0.3-1/demo/Stream_lines_2/Stream_lines_2.cpp	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/demo/Stream_lines_2/Stream_lines_2.cpp	2020-09-08 12:12:19.000000000 +0000
@@ -106,7 +106,7 @@ MainWindow::MainWindow()
   this->graphicsView->setScene(&scene);
 
   // Turn the vertical axis upside down
-  this->graphicsView->matrix().scale(1, -1);
+  this->graphicsView->transform().scale(1, -1);
 
   // The navigation adds zooming and translation functionality to the
   // QGraphicsView
diff -pruN 5.0.3-1/demo/Surface_mesh_deformation/CMakeLists.txt 5.1-1/demo/Surface_mesh_deformation/CMakeLists.txt
--- 5.0.3-1/demo/Surface_mesh_deformation/CMakeLists.txt	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Surface_mesh_deformation/CMakeLists.txt	2020-09-08 12:12:20.000000000 +0000
@@ -17,11 +17,10 @@ find_package(CGAL QUIET)
 if ( CGAL_FOUND )
 
   find_package(Eigen3 3.1.91) #(requires 3.2.0 or greater)
-  if (EIGEN3_FOUND)
-    include( ${EIGEN3_USE_FILE} )
-
+  include(CGAL_Eigen_support)
+  if (TARGET CGAL::Eigen_support)
     create_single_source_cgal_program( "deform_mesh_for_botsch08_format.cpp" )
-
+    target_link_libraries(deform_mesh_for_botsch08_format PUBLIC CGAL::Eigen_support)
   else()
     message(STATUS "NOTICE: This program requires the Eigen library, version 3.2 or later and will not be compiled.")
   endif()
@@ -30,4 +29,3 @@ else()
     message(STATUS "NOTICE: This program requires the CGAL library, and will not be compiled.")
 
 endif()
-
diff -pruN 5.0.3-1/demo/Surface_mesher/File_XT.h 5.1-1/demo/Surface_mesher/File_XT.h
--- 5.0.3-1/demo/Surface_mesher/File_XT.h	2020-08-01 19:02:56.000000000 +0000
+++ 5.1-1/demo/Surface_mesher/File_XT.h	2020-09-08 12:15:46.000000000 +0000
@@ -14,6 +14,20 @@
 
 namespace CGAL {
 namespace Total {
+namespace internal {
+
+//To avoid "fopen may be unsafe" on Visual.
+bool open_file(FILE **fin, const char* filename)
+{
+#if defined(BOOST_MSVC)
+  return (fopen_s(fin, filename, "rw") == 0);
+#else
+  *fin = fopen(filename, "rw");
+  return (fin != NULL);
+#endif
+}
+
+} // namespace internal
 
 void permuteLong(char *a)
 {
@@ -99,7 +113,7 @@ int lire_longueur_trace(const char * nom
     FILE *fin;
     int flag=0;
 
-if ( (fin=fopen(nomfich,"rw"))!=NULL )
+if ( internal::open_file(&fin, nomfich) )
  {
     fseek(fin,4,0);
     if (fread(long_trace,4,1,fin) != 1) {
@@ -118,7 +132,7 @@ int lire_nb_trace(const char * nomfich,
     FILE *fin;
     int flag=0;
 
-if ( (fin=fopen(nomfich,"rw"))!=NULL )
+if ( internal::open_file(&fin, nomfich) )
  {
     fseek(fin,8,0);
     if (fread(nb_trace,4,1,fin) != 1) {
@@ -137,7 +151,7 @@ int lire_nb_plan(const char * nomfich, l
     FILE *fin;
     int flag=0;
 
-if ( (fin=fopen(nomfich,"rw"))!=NULL )
+if ( internal::open_file(&fin, nomfich) )
  {
     fseek(fin,12,0);
     if (fread(nb_plan,4,1,fin) != 1) {
@@ -157,7 +171,7 @@ int lire_nb_octet(const char * nomfich,
     FILE *fin;
     int flag=0;
 
-if ( (fin=fopen(nomfich,"rw"))!=NULL )
+if ( internal::open_file(&fin, nomfich) )
  {
     fseek(fin,36,0);
     if (fread(nb_octet,4,1,fin) != 1) {
@@ -177,7 +191,7 @@ int lire_longueur_entete(const char * no
     FILE *fin;
     int flag=0;
 
-if ( (fin=fopen(nomfich,"rw"))!=NULL )
+if ( internal::open_file(&fin, nomfich) )
  {
     fseek(fin,72,0);
     if (fread(long_entete,4,1,fin) != 1) {
diff -pruN 5.0.3-1/demo/Surface_mesher/values_list.cpp 5.1-1/demo/Surface_mesher/values_list.cpp
--- 5.0.3-1/demo/Surface_mesher/values_list.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Surface_mesher/values_list.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -20,6 +20,9 @@
 #include <QUrl>
 #include <QLineEdit>
 #include <QDoubleValidator>
+#if QT_VERSION >= QT_VERSION_CHECK(5, 10, 0)
+#include <QRandomGenerator>
+#endif
 
 Values_delegate::Values_delegate(QWidget* parent) : QItemDelegate(parent) {}
 void Values_delegate::paint(QPainter * painter, const QStyleOptionViewItem & option, const QModelIndex & index) const
@@ -240,7 +243,12 @@ void Values_list::addValue(const double
   newItem->setData(Value, Qt::CheckStateRole, Qt::Checked);
   newItem->setData(Value, Qt::DisplayRole, i);
   QStringList colors = QColor::colorNames();
+#if QT_VERSION < QT_VERSION_CHECK(5, 10, 0)
   const int color_index = qrand() % colors.size();
+#else
+const int color_index = QRandomGenerator::global()->generate() % colors.size();
+#endif
+
   QColor color = QColor(colors[color_index]);
   newItem->setData(Color, Qt::DisplayRole, color);
   newItem->setData(Name, Qt::DisplayRole, "");
diff -pruN 5.0.3-1/demo/Three/CMakeLists.txt 5.1-1/demo/Three/CMakeLists.txt
--- 5.0.3-1/demo/Three/CMakeLists.txt	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Three/CMakeLists.txt	1970-01-01 00:00:00.000000000 +0000
@@ -1,20 +0,0 @@
-# Created by the script cgal_create_cmake_script
-# This is the CMake script for compiling a CGAL application.
-
-
-cmake_minimum_required(VERSION 3.1...3.15)
-project( Three_Demo )
-
-
-set(CMAKE_CXX_STANDARD 14)
-
-find_package(CGAL QUIET COMPONENTS Core )
-
-if ( CGAL_FOUND )
-    include(${CGAL_USE_FILE})
-else()
-
-    message(STATUS "This program requires the CGAL library, and will not be compiled.")
-
-endif()
-
diff -pruN 5.0.3-1/demo/Three/Example_plugin/Basic_dialog.ui 5.1-1/demo/Three/Example_plugin/Basic_dialog.ui
--- 5.0.3-1/demo/Three/Example_plugin/Basic_dialog.ui	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Three/Example_plugin/Basic_dialog.ui	1970-01-01 00:00:00.000000000 +0000
@@ -1,91 +0,0 @@
-<?xml version="1.0" encoding="UTF-8"?>
-<ui version="4.0">
- <class>BasicDialog</class>
- <widget class="QDialog" name="BasicDialog">
-  <property name="geometry">
-   <rect>
-    <x>0</x>
-    <y>0</y>
-    <width>400</width>
-    <height>300</height>
-   </rect>
-  </property>
-  <property name="windowTitle">
-   <string>Dialog</string>
-  </property>
-  <layout class="QVBoxLayout" name="verticalLayout_2">
-   <item>
-    <layout class="QHBoxLayout" name="horizontalLayout">
-     <item>
-      <widget class="QLabel" name="label">
-       <property name="text">
-        <string>Number to display :</string>
-       </property>
-      </widget>
-     </item>
-     <item>
-      <spacer name="horizontalSpacer">
-       <property name="orientation">
-        <enum>Qt::Horizontal</enum>
-       </property>
-       <property name="sizeHint" stdset="0">
-        <size>
-         <width>40</width>
-         <height>20</height>
-        </size>
-       </property>
-      </spacer>
-     </item>
-     <item>
-      <widget class="QLineEdit" name="lineEdit"/>
-     </item>
-    </layout>
-   </item>
-   <item>
-    <widget class="QDialogButtonBox" name="buttonBox">
-     <property name="orientation">
-      <enum>Qt::Horizontal</enum>
-     </property>
-     <property name="standardButtons">
-      <set>QDialogButtonBox::Cancel|QDialogButtonBox::Ok</set>
-     </property>
-    </widget>
-   </item>
-  </layout>
- </widget>
- <resources/>
- <connections>
-  <connection>
-   <sender>buttonBox</sender>
-   <signal>accepted()</signal>
-   <receiver>BasicDialog</receiver>
-   <slot>accept()</slot>
-   <hints>
-    <hint type="sourcelabel">
-     <x>248</x>
-     <y>254</y>
-    </hint>
-    <hint type="destinationlabel">
-     <x>157</x>
-     <y>274</y>
-    </hint>
-   </hints>
-  </connection>
-  <connection>
-   <sender>buttonBox</sender>
-   <signal>rejected()</signal>
-   <receiver>BasicDialog</receiver>
-   <slot>reject()</slot>
-   <hints>
-    <hint type="sourcelabel">
-     <x>316</x>
-     <y>260</y>
-    </hint>
-    <hint type="destinationlabel">
-     <x>286</x>
-     <y>274</y>
-    </hint>
-   </hints>
-  </connection>
- </connections>
-</ui>
diff -pruN 5.0.3-1/demo/Three/Example_plugin/Basic_dock_widget.ui 5.1-1/demo/Three/Example_plugin/Basic_dock_widget.ui
--- 5.0.3-1/demo/Three/Example_plugin/Basic_dock_widget.ui	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Three/Example_plugin/Basic_dock_widget.ui	1970-01-01 00:00:00.000000000 +0000
@@ -1,61 +0,0 @@
-<?xml version="1.0" encoding="UTF-8"?>
-<ui version="4.0">
- <class>BasicDockWidget</class>
- <widget class="QDockWidget" name="BasicDockWidget">
-  <property name="geometry">
-   <rect>
-    <x>0</x>
-    <y>0</y>
-    <width>392</width>
-    <height>302</height>
-   </rect>
-  </property>
-  <property name="windowTitle">
-   <string>DockWidget</string>
-  </property>
-  <widget class="QWidget" name="dockWidgetContents">
-   <layout class="QHBoxLayout" name="horizontalLayout_2">
-    <item>
-     <layout class="QVBoxLayout" name="verticalLayout">
-      <item>
-       <layout class="QHBoxLayout" name="horizontalLayout">
-        <item>
-         <widget class="QLabel" name="label">
-          <property name="text">
-           <string>Number to display :</string>
-          </property>
-         </widget>
-        </item>
-        <item>
-         <spacer name="horizontalSpacer">
-          <property name="orientation">
-           <enum>Qt::Horizontal</enum>
-          </property>
-          <property name="sizeHint" stdset="0">
-           <size>
-            <width>40</width>
-            <height>20</height>
-           </size>
-          </property>
-         </spacer>
-        </item>
-        <item>
-         <widget class="QSpinBox" name="spinBox"/>
-        </item>
-       </layout>
-      </item>
-      <item>
-       <widget class="QPushButton" name="pushButton">
-        <property name="text">
-         <string>Print</string>
-        </property>
-       </widget>
-      </item>
-     </layout>
-    </item>
-   </layout>
-  </widget>
- </widget>
- <resources/>
- <connections/>
-</ui>
diff -pruN 5.0.3-1/demo/Three/Example_plugin/Basic_item_plugin.cpp 5.1-1/demo/Three/Example_plugin/Basic_item_plugin.cpp
--- 5.0.3-1/demo/Three/Example_plugin/Basic_item_plugin.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Three/Example_plugin/Basic_item_plugin.cpp	1970-01-01 00:00:00.000000000 +0000
@@ -1,90 +0,0 @@
-#include <CGAL/Three/Polyhedron_demo_plugin_interface.h>
-#include <QApplication>
-#include <QObject>
-#include <QAction>
-#include <QMainWindow>
-#include <QInputDialog>
-#include "Messages_interface.h"
-#include "CGAL/Three/Scene_group_item.h"
-#include "Scene_plane_item.h"
-
-//This plugin crates an action in Operations that displays the name of the selected item,
-//adds a scene_plane_item to the scene, and adds the selected item and the plane to a new group.
-class BasicItemPlugin :
-    public QObject,
-    public CGAL::Three::Polyhedron_demo_plugin_interface
-{
-  Q_OBJECT
-  Q_INTERFACES(CGAL::Three::Polyhedron_demo_plugin_interface)
-  Q_PLUGIN_METADATA(IID "com.geometryfactory.PolyhedronDemo.PluginInterface/1.0")
-public:
-  //! [applicable]
-  //This plugin is only applicable if there is exactly one selected item.
-  bool applicable(QAction*) const Q_DECL_OVERRIDE
-  {
-    return scene->selectionIndices().size() ==1;
-  }
-  //! [applicable]
-  //the list of the actions of the plugin.
-  QList<QAction*> actions() const Q_DECL_OVERRIDE
-  {
-    return _actions;
-  }
-  //this acts like a constructor for the plugin. It gets the references to the mainwindow and the scene, and connects the action.
-  void init(QMainWindow* mw, CGAL::Three::Scene_interface* sc, Messages_interface* mi) Q_DECL_OVERRIDE
-  {
-    //gets the reference to the message interface, to display text in the console widget
-    this->messageInterface = mi;
-    //get the references
-    this->scene = sc;
-    this->mw = mw;
-
-    //creates the action
-    QAction *actionHelloWorld= new QAction(QString("Create a group"), mw);
-    //specifies the subMenu
-    actionHelloWorld->setProperty("submenuName", "Basic");
-    //links the action
-    if(actionHelloWorld) {
-      connect(actionHelloWorld, SIGNAL(triggered()),
-              this, SLOT(helloWorld()));
-      _actions << actionHelloWorld;
-    }
-  }
-private Q_SLOTS:
-
-
-  void helloWorld()
-  { //! [use]
-    //get a reference to the selected item.
-    CGAL::Three::Scene_item *item = scene->item(scene->mainSelectionIndex());
-    messageInterface->information(QString("The selected item's name is  : %1").arg(item->name()));
-    //! [use]
-    //! [additem]
-    //creates a plane item
-    Scene_plane_item *new_item = new Scene_plane_item(scene);
-    new_item->setName("Trivial Plane");
-    new_item->setColor(Qt::blue);
-    new_item->setNormal(0.0,0.0,1.0);
-    scene->addItem(new_item);
-    //! [additem]
-    //! [group]
-    //Create a new group
-    Scene_group_item *group = new Scene_group_item("New group");
-    //add it to the scene
-    scene->addItem(group);
-    //Then give it its children
-    scene->changeGroup(item, group);
-    scene->changeGroup(new_item,group);
-    //! [group]
-  }
-
-private:
-  QList<QAction*> _actions;
-  Messages_interface* messageInterface;
-  //The reference to the scene
-  CGAL::Three::Scene_interface* scene;
-  //The reference to the main window
-  QMainWindow* mw;
-};
-
-#include "Basic_item_plugin.moc"
diff -pruN 5.0.3-1/demo/Three/Example_plugin/Basic_plugin.cpp 5.1-1/demo/Three/Example_plugin/Basic_plugin.cpp
--- 5.0.3-1/demo/Three/Example_plugin/Basic_plugin.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Three/Example_plugin/Basic_plugin.cpp	1970-01-01 00:00:00.000000000 +0000
@@ -1,141 +0,0 @@
-/*
-Change the value of EXAMPLE_COMPLEXITY in the first line to change the behavior :
-  - 0 : prints "Hello World!" in the Info and console widgets
-  - 1 : pops-up a simple dialog asking to enter an integer , then prints it in the Info and console widgets
-  - 2 : pops-up a little more elaborated dialog asking to enter an integer , then prints it in the Info and console widgets if it was indeed an integer, else pops-up an error message box.
-  */
-#define EXAMPLE_COMPLEXITY 0
-#include "ui_Basic_dialog.h"
-//! [headers_plugin]
-#include <CGAL/Three/Polyhedron_demo_plugin_interface.h>
-#include <QApplication>
-#include <QObject>
-#include <QAction>
-#include <QMainWindow>
-#include <QInputDialog>
-#include <QMessageBox>
-#include "Messages_interface.h"
-//! [headers_plugin]
-//! [dialog_plugin]
-class ComplexDialog :
-    public QDialog,
-    public Ui::BasicDialog
-{
-  Q_OBJECT
-public:
-  ComplexDialog(QWidget* =0)
-  {
-    setupUi(this);
-
-  }
-
-};
-//! [dialog_plugin]
-//! [opening_plugin]
-
-//This plugin creates an action in Operations depending on EXAMPLE_COMPLEXITY.
-class BasicPlugin :
-    public QObject,
-    public CGAL::Three::Polyhedron_demo_plugin_interface
-{
-  Q_OBJECT
-  Q_INTERFACES(CGAL::Three::Polyhedron_demo_plugin_interface)
-  Q_PLUGIN_METADATA(IID "com.geometryfactory.PolyhedronDemo.PluginInterface/1.0")
-public:
-  //decides if the plugin's actions will be displayed or not.
-  bool applicable(QAction*) const Q_DECL_OVERRIDE
-  {
-    return true;
-  }
-  //the list of the actions of the plugin.
-  QList<QAction*> actions() const Q_DECL_OVERRIDE
-  {
-    return _actions;
-  }
-  //this acts like a constructor for the plugin. It gets the references to the main window and the scene, and connects the action.
-  void init(QMainWindow* mainWindow, CGAL::Three::Scene_interface* sc, Messages_interface* mi) Q_DECL_OVERRIDE
-  {
-    //gets the reference to the message interface, to display text in the console widget
-    this->messageInterface = mi;
-    //get the references
-    this->scene = sc;
-    this->mw = mainWindow;
-
-    //creates the action
-    QAction *actionHelloWorld= new QAction(QString("Hello World"), mw);
-    //specifies the subMenu
-    actionHelloWorld->setProperty("submenuName", "Basic");
-    //links the action
-    if(actionHelloWorld) {
-      connect(actionHelloWorld, SIGNAL(triggered()),
-              this, SLOT(helloWorld()));
-      _actions << actionHelloWorld;
-    }
-  }
-private Q_SLOTS:
-
-  //! [opening_plugin]
-#if EXAMPLE_COMPLEXITY == 0
-  //! [basic_plugin]
-  void helloWorld()
-  {
-    messageInterface->information(QString("Hello World!"));
-  }
-  //! [basic_plugin]
-#elif EXAMPLE_COMPLEXITY == 1
-  //! [basic_dialog_plugin]
-  void helloWorld()
-  {
-    bool ok = false;
-    const unsigned int parameter =
-        QInputDialog::getInt((QWidget*)mw,
-                             tr("Hello World"), // dialog title
-                             tr("Hello dear user! What integer would you want me to display for you ? "), // field label
-                             10, // default value = fast
-                             0, // min
-                             100, // max
-                             1, // step
-                             &ok);
-    if(!ok) return;
-    messageInterface->information(QString("You asked me to display %1, so here it is : %1").arg(parameter));
-  }
-  //! [basic_dialog_plugin]
-#elif EXAMPLE_COMPLEXITY == 2
-  //! [complex_dialog_plugin]
-  void helloWorld()
-  {
-    //creates a new dialog
-    ComplexDialog *dialog = new ComplexDialog();
-    //opens the dialog
-    if(!dialog->exec())
-      return;
-    //! [warningbox]
-    QString result = dialog->lineEdit->text();
-    bool ok = false;
-    int int_res = result.toInt(&ok);
-    if(!ok)
-    {
-      QMessageBox::warning(mw,
-                           "ERROR",
-                           tr("This is not an integer !")
-                           );
-      return;
-    }
-    //! [warningbox]
-    messageInterface->information(QString("You asked me to display %1, so here it is : %1").arg(int_res));
-  }
-  //! [complex_dialog_plugin]
-#endif
-  //! [ending_plugin]
-private:
-  QList<QAction*> _actions;
-  Messages_interface* messageInterface;
-  //The reference to the scene
-  CGAL::Three::Scene_interface* scene;
-  //The reference to the main window
-  QMainWindow* mw;
-};
-
-#include "Basic_plugin.moc"
-//! [ending_plugin]
-
diff -pruN 5.0.3-1/demo/Three/Example_plugin/CMakeLists.txt 5.1-1/demo/Three/Example_plugin/CMakeLists.txt
--- 5.0.3-1/demo/Three/Example_plugin/CMakeLists.txt	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Three/Example_plugin/CMakeLists.txt	1970-01-01 00:00:00.000000000 +0000
@@ -1,42 +0,0 @@
-cmake_minimum_required(VERSION 3.1...3.15)
-project( Example_plugin )
-
-if(NOT POLICY CMP0070 AND POLICY CMP0053)
-  # Only set CMP0053 to OLD with CMake<3.10, otherwise there is a warning.
-  cmake_policy(SET CMP0053 OLD)
-endif()
-# Find includes in corresponding build directories
-set(CMAKE_INCLUDE_CURRENT_DIR ON)
-# Instruct CMake to run moc automatically when needed.
-set(CMAKE_AUTOMOC ON)
-#Find CGAL
-find_package(CGAL COMPONENTS Qt5)
-# Find Qt5 itself
-find_package(Qt5
-             QUIET
-             COMPONENTS            OpenGL Script Svg Xml
-             OPTIONAL_COMPONENTS   ScriptTools)
-
-if(Qt5_FOUND AND CGAL_FOUND)
-  find_package(CGAL_polyhedron_demo)
-
-include( ${CGAL_POLYHEDRON_DEMO_USE_FILE} )
-
-# Let plugins be compiled in the same directory as the executable.
-set(CMAKE_LIBRARY_OUTPUT_DIRECTORY "${CMAKE_RUNTIME_OUTPUT_DIRECTORY}")
-
-
-  polyhedron_demo_plugin(example_plugin Example_plugin)
-
-  qt5_wrap_ui( basicUI_FILES Basic_dialog.ui  )
-  polyhedron_demo_plugin(basic_plugin Basic_plugin ${basicUI_FILES})
-
-  qt5_wrap_ui( dockUI_FILES Basic_dock_widget.ui )
-  polyhedron_demo_plugin(dock_widget_plugin Dock_widget_plugin ${dockUI_FILES})
-
-  polyhedron_demo_plugin(basic_item_plugin Basic_item_plugin)
-
-#  links the library containing the scene_plane_item with the plugin
-  target_link_libraries(basic_item_plugin PUBLIC Polyhedron_scene_basic_objects)
-
-endif()
diff -pruN 5.0.3-1/demo/Three/Example_plugin/Dock_widget_plugin.cpp 5.1-1/demo/Three/Example_plugin/Dock_widget_plugin.cpp
--- 5.0.3-1/demo/Three/Example_plugin/Dock_widget_plugin.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Three/Example_plugin/Dock_widget_plugin.cpp	1970-01-01 00:00:00.000000000 +0000
@@ -1,102 +0,0 @@
-#include "ui_Basic_dock_widget.h"
-#include <CGAL/Three/Polyhedron_demo_plugin_helper.h>
-#include <QApplication>
-#include <QObject>
-#include <QAction>
-#include <QMainWindow>
-#include "Messages_interface.h"
-
-//! [dock]
-class DockWidget :
-    public QDockWidget,
-    public Ui::BasicDockWidget
-{
-public:
-  DockWidget(QString name, QWidget *parent)
-    :QDockWidget(name,parent)
-  {
-   setupUi(this);
-  }
-};
-//! [dock]
-//This plugin crates an action in Operations that creates a DOckWidget to display a number in the 'console' dockwidet.
-class BasicPlugin :
-    public QObject,
-    public CGAL::Three::Polyhedron_demo_plugin_helper
-{
-  Q_OBJECT
-  Q_INTERFACES(CGAL::Three::Polyhedron_demo_plugin_interface)
-  Q_PLUGIN_METADATA(IID "com.geometryfactory.PolyhedronDemo.PluginInterface/1.0")
-public:
-  //decides if the plugin's actions will be displayed or not.
-  bool applicable(QAction*) const Q_DECL_OVERRIDE
-  {
-    return true;
-  }
-  //the list of the actions of the plugin.
-  QList<QAction*> actions() const Q_DECL_OVERRIDE
-  {
-    return _actions;
-  }
-  //! [init]
-  //this acts like a constructor for the plugin. It gets the references to the mainwindow and the scene, and connects the action.
-  void init(QMainWindow* mw, CGAL::Three::Scene_interface* sc, Messages_interface* mi) Q_DECL_OVERRIDE
-  {
-    //gets the reference to the message interface, to display text in the console widget
-    this->messageInterface = mi;
-    //get the references
-    this->scene = sc;
-    this->mw = mw;
-
-    //creates the action
-    QAction *actionHelloWorld= new QAction(QString("Open Dock Widget"), mw);
-    //specifies the subMenu
-    actionHelloWorld->setProperty("submenuName", "Basic");
-    //links the action
-    if(actionHelloWorld) {
-      connect(actionHelloWorld, SIGNAL(triggered()),
-              this, SLOT(helloWorld()));
-      _actions << actionHelloWorld;
-    }
-
-    dock_widget = new DockWidget("Print a number", mw);
-    dock_widget->setVisible(false); // do not show at the beginning
-
-    addDockWidget(dock_widget);
-
-    connect(dock_widget->pushButton, SIGNAL(clicked(bool)),
-            this, SLOT(on_dock_button_clicked()));
-  }
-  //! [init]
-
-private Q_SLOTS:
-//! [action]
-  void helloWorld()
-  {
-    // dock widget should be instancied in init()
-    if(dock_widget->isVisible()) { dock_widget->hide(); }
-    else                         { dock_widget->show(); }
-  }
-
-  void on_dock_button_clicked()
-  {
-      messageInterface->information(QString("Here is your number :%1").arg(dock_widget->spinBox->value()));
-
-  }
-  //! [action]
-  //! [closure]
-  void closure()Q_DECL_OVERRIDE
-  {
-    dock_widget->hide();
-  }
-  //! [closure]
-private:
-  QList<QAction*> _actions;
-  Messages_interface* messageInterface;
-  DockWidget* dock_widget;
-};
-
-#include "Dock_widget_plugin.moc"
-
-
-
diff -pruN 5.0.3-1/demo/Three/Example_plugin/Example_plugin.cpp 5.1-1/demo/Three/Example_plugin/Example_plugin.cpp
--- 5.0.3-1/demo/Three/Example_plugin/Example_plugin.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Three/Example_plugin/Example_plugin.cpp	1970-01-01 00:00:00.000000000 +0000
@@ -1,209 +0,0 @@
-//! \file Polyhedron_demo_example_plugin.cpp
-
-#include <QApplication>
-#include <QMainWindow>
-#include <QAction>
-#include <QVector>
-#include <CGAL/Three/Scene_item.h>
-#include <CGAL/Three/Viewer_interface.h>
-#include <CGAL/Three/Scene_group_item.h>
-
-//! [itemdeclaration]
-// The special Scene_item only for triangles
-
-//this is used by the Qt's MOC system to manage the metadata.
-#ifdef scene_triangle_item_EXPORTS
-#  define SCENE_TRIANGLE_ITEM_EXPORT Q_DECL_EXPORT
-#else
-#  define SCENE_TRIANGLE_ITEM_EXPORT Q_DECL_IMPORT
-#endif
-
-class Scene_triangle_item : public CGAL::Three::Scene_item
-{
-
-  Q_OBJECT
-public :
-  Scene_triangle_item(double ax,double ay, double az,
-                      double bx,double by, double bz,
-                      double cx,double cy, double cz);
-
-  // Indicates if rendering mode is supported
-  bool supportsRenderingMode(RenderingMode m) const Q_DECL_OVERRIDE {
-    return (m == Flat);
-  }
-
-  //Displays the item
-  void draw(CGAL::Three::Viewer_interface* viewer) const Q_DECL_OVERRIDE;
-
-  //Specifies that the buffers need to be initialized again.
-  //Is mostly called after a change of geometry in the data.
-  void invalidateOpenGLBuffers() Q_DECL_OVERRIDE;
-
-  //fills the std::vector
-  void computeElements(double ax,double ay, double az,
-                        double bx,double by, double bz,
-                        double cx,double cy, double cz) const Q_DECL_OVERRIDE;
-
-  Scene_item* clone() const Q_DECL_OVERRIDE {return 0;}
-  QString toolTip() const Q_DECL_OVERRIDE {return QString();}
-
-
-private:
-  //contains the data
-  mutable std::vector<float> vertices;
-  mutable int nb_pos;
-  mutable QOpenGLShaderProgram *program;
-  using CGAL::Three::Scene_item::initializeBuffers;
-  //Fills the buffers with data. The buffers allow us to give data to the shaders.
-  void initializeBuffers(CGAL::Three::Viewer_interface *viewer)const Q_DECL_OVERRIDE;
-}; //end of class Scene_triangle_item
-//! [itemdeclaration]
-Scene_triangle_item::Scene_triangle_item(double ax,double ay, double az,
-                                         double bx,double by, double bz,
-                                         double cx,double cy, double cz)
-  :  CGAL::Three::Scene_item(1,1)
-{
-  nb_pos = 0;
-  are_buffers_filled = false;
-  computeElements(ax, ay, az,
-                   bx, by, bz,
-                   cx, cy, cz);
-  invalidateOpenGLBuffers();
-}
-
-//! [computeelements]
-//Fills the position vector with data.
-void Scene_triangle_item::computeElements(double ax, double ay, double az,
-                                           double bx, double by, double bz,
-                                           double cx, double cy, double cz)const
-{
-  vertices.resize(9);
-  vertices[0] = ax; vertices[1] = ay; vertices[2] = az;
-  vertices[3] = bx; vertices[4] = by; vertices[5] = bz;
-  vertices[6] = cx; vertices[7] = cy; vertices[8] = cz;
-}
-
-//! [computeelements]
-//! [draw]
-
-void Scene_triangle_item::draw(CGAL::Three::Viewer_interface* viewer) const
-{
-  //The filling of the buffers should be performed in this function, because it needs a valid openGL context, and we are certain to have one in this function.
-  if(!are_buffers_filled)
-  {
-    computeElements(0, 0, 0,
-                     1, 0, 0,
-                     0.5, 0.5, 0);
-    initializeBuffers(viewer);
-  }
-  //Binds the vao corresponding to the type of data we are drawing.
-  vaos[0]->bind();
-  //Gets the program corresponding to the type of data we are drawing.
-  //Here we want triangles with light effects.
-  program = getShaderProgram(PROGRAM_WITH_LIGHT);
-  //Gives most of the uniform values to the shaders.
-  attribBuffers(viewer, PROGRAM_WITH_LIGHT);
-  //Binds the program chosen before to use the right shaders.
-  program->bind();
-  //Gives the wanted color to the fragment shader as uniform value.
-  program->setAttributeValue("colors", this->color());
-  //Draws the items
-  viewer->glDrawArrays(GL_TRIANGLES, 0, static_cast<GLsizei>(nb_pos/3));
-  //clean up
-  vaos[0]->release();
-  program->release();
-
-}
-//! [draw]
-//Specifies that the buffers need to be initialized again.
-//Is mostly called after a change of geometry in the data.
-void Scene_triangle_item::invalidateOpenGLBuffers()
-{
-  are_buffers_filled = false;
-}
-
-
-//! [fillbuffers]
-void Scene_triangle_item::initializeBuffers(CGAL::Three::Viewer_interface *viewer)const
-{
-
-  //vao containing the data for the facets
-  {
-    program = getShaderProgram(PROGRAM_WITH_LIGHT, viewer);
-    program->bind();
-
-    vaos[0]->bind();
-    buffers[0].bind();
-    buffers[0].allocate(vertices.data(),
-                        static_cast<GLsizei>(vertices.size()*sizeof(float)));
-    program->enableAttributeArray("vertex");
-    program->setAttributeBuffer("vertex",GL_FLOAT,0,3);
-    buffers[0].release();
-
-    vaos[0]->release();
-    program->release();
-
-  }
-
-//once the buffers are fill, we can empty the vectors to optimize memory consumption
-  nb_pos = vertices.size();
-  vertices.resize(0);
-  //"Swap trick" insures that the memory is indeed freed and not kept available
-  std::vector<float>(vertices).swap(vertices);
-  are_buffers_filled = true;
-}
-//! [fillbuffers]
-#include <CGAL/Three/Polyhedron_demo_plugin_helper.h>
-//The actual plugin
-using namespace CGAL::Three;
-class Q_DECL_EXPORT Polyhedron_demo_example_plugin :
-    public QObject,
-    public Polyhedron_demo_plugin_helper
-{
-  //Configures CMake to use MOC correctly
-  Q_OBJECT
-  Q_INTERFACES(CGAL::Three::Polyhedron_demo_plugin_interface)
-  Q_PLUGIN_METADATA(IID "com.geometryfactory.PolyhedronDemo.PluginInterface/1.0")
-
-
-public :
-  // Adds an action to the menu and configures the widget
-  void init(QMainWindow* mainWindow, CGAL::Three::Scene_interface* scene_interface, Messages_interface*) Q_DECL_OVERRIDE{
-    //get the references
-    this->scene = scene_interface;
-    this->mw = mainWindow;
-
-    //creates and link the actions
-    QAction* actionDrawTriangle= new QAction("Draw Triangle", mw);
-    if(actionDrawTriangle) {
-      connect(actionDrawTriangle, SIGNAL(triggered()),
-              this, SLOT(draw_triangle()));
-      _actions << actionDrawTriangle;
-    }
-  }
-  bool applicable(QAction*) const Q_DECL_OVERRIDE
-  {
-    return true;
-  }
-  QList<QAction*> actions() const Q_DECL_OVERRIDE{
-    return _actions;
-  }
-
-public Q_SLOTS:
-
-
-  void draw_triangle() {
-
-    triangle = new Scene_triangle_item(0, 0, 0,
-                                       1, 0, 0,
-                                       0.5, 0.5, 0);
-    triangle->setName(QString("Basic triangle"));
-    scene->addItem(triangle);
-  }
-private:
-  CGAL::Three::Scene_item* triangle;
-  QList<QAction*> _actions;
-
-}; //end of class Polyhedron_demo_example_plugin
-#include "Example_plugin.moc"
-
diff -pruN 5.0.3-1/demo/Three/Example_plugin/Group_item_plugin.cpp 5.1-1/demo/Three/Example_plugin/Group_item_plugin.cpp
--- 5.0.3-1/demo/Three/Example_plugin/Group_item_plugin.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Three/Example_plugin/Group_item_plugin.cpp	1970-01-01 00:00:00.000000000 +0000
@@ -1,86 +0,0 @@
-#include <CGAL/Three/Polyhedron_demo_plugin_interface.h>
-#include <QApplication>
-#include <QObject>
-#include <QAction>
-#include <QMainWindow>
-#include <QInputDialog>
-#include "Messages_interface.h"
-#include "Scene_plane_item.h"
-#include <CGAL/Three/Scene_group_item.h>
-
-//This plugin crates an action in Operations that displays "Hello World" in the 'console' dockwidet.
-class GroupItemPlugin :
-    public QObject,
-    public CGAL::Three::Polyhedron_demo_plugin_interface
-{
-  Q_OBJECT
-  Q_INTERFACES(CGAL::Three::Polyhedron_demo_plugin_interface)
-  Q_PLUGIN_METADATA(IID "com.geometryfactory.PolyhedronDemo.PluginInterface/1.0")
-public:
-  //This plugin is only applicable if there is at exactly one selected item.
-  bool applicable(QAction*) const
-  {
-    return true;
-  }
-  //the list of the actions of the plugin.
-  QList<QAction*> actions() const
-  {
-    return _actions;
-  }
-  //this acts like a constructor for the plugin. It gets the references to the mainwindow and the scene, and connects the action.
-  void init(QMainWindow* mainWindow, CGAL::Three::Scene_interface* sc  )
-  {
-    //get the references
-    this->scene = sc;
-    this->mw = mainWindow;
-
-    //creates the action
-    QAction *actionHelloWorld= new QAction(QString("Hello World"), mw);
-    //specifies the subMenu
-    actionHelloWorld->setProperty("submenuName", "Basic");
-    //links the action
-    if(actionHelloWorld) {
-      connect(actionHelloWorld, SIGNAL(triggered()),
-              this, SLOT(helloWorld()));
-      _actions << actionHelloWorld;
-    }
-  }
-
-  void init(QMainWindow* mw, CGAL::Three::Scene_interface* sc, Messages_interface* mi)
-  {
-    //gets the reference to the message interface, to display text in the console widget
-    this->messageInterface = mi;
-    init(mw, sc);
-  }
-private Q_SLOTS:
-
-
-  void helloWorld()
-  {
-    //creates an item
-    Scene_plane_item *child = new Scene_plane_item(scene);
-    child->setName("Trivial Plane");
-    child->setColor(Qt::blue);
-    child->setNormal(0.0,0.0,1.0);
-    scene->addItem(child);
-    //clears the selection to avoid adding unwanted items to the group.
-    scene->setSelectedItem(-1);
-    //Creates a new group
-    Scene_group_item *group = new Scene_group_item("New group");
-    //Then gives it its children
-    scene->changeGroup(child, group);
-    //adds it to the scene
-    scene->addGroup(group);
-
-  }
-
-private:
-  QList<QAction*> _actions;
-  Messages_interface* messageInterface;
-  //The reference to the scene
-  CGAL::Three::Scene_interface* scene;
-  //The reference to the main window
-  QMainWindow* mw;
-};
-
-#include "Group_item_plugin.moc"
diff -pruN 5.0.3-1/demo/Triangulation_2/Delaunay_triangulation_2.cpp 5.1-1/demo/Triangulation_2/Delaunay_triangulation_2.cpp
--- 5.0.3-1/demo/Triangulation_2/Delaunay_triangulation_2.cpp	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/demo/Triangulation_2/Delaunay_triangulation_2.cpp	2020-09-08 12:12:19.000000000 +0000
@@ -171,7 +171,7 @@ MainWindow::MainWindow()
   this->graphicsView->setMouseTracking(true);
 
   // Turn the vertical axis upside down
-  this->graphicsView->matrix().scale(1, -1);
+  this->graphicsView->transform().scale(1, -1);
 
   // The navigation adds zooming and translation functionality to the
   // QGraphicsView
diff -pruN 5.0.3-1/demo/Triangulation_2/Regular_triangulation_2.cpp 5.1-1/demo/Triangulation_2/Regular_triangulation_2.cpp
--- 5.0.3-1/demo/Triangulation_2/Regular_triangulation_2.cpp	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/demo/Triangulation_2/Regular_triangulation_2.cpp	2020-09-08 12:12:19.000000000 +0000
@@ -142,7 +142,7 @@ MainWindow::MainWindow()
   this->graphicsView->setMouseTracking(true);
 
   // Turn the vertical axis upside down
-  this->graphicsView->matrix().scale(1, -1);
+  this->graphicsView->transform().scale(1, -1);
 
   // The navigation adds zooming and translation functionality to the
   // QGraphicsView
diff -pruN 5.0.3-1/demo/Triangulation_3/CMakeLists.txt 5.1-1/demo/Triangulation_3/CMakeLists.txt
--- 5.0.3-1/demo/Triangulation_3/CMakeLists.txt	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Triangulation_3/CMakeLists.txt	2020-09-08 12:12:21.000000000 +0000
@@ -37,12 +37,14 @@ option(CGAL_ACTIVATE_CONCURRENT_TRIANGUL
 if( CGAL_ACTIVATE_CONCURRENT_TRIANGULATION_3 )
   add_definitions( -DCGAL_CONCURRENT_TRIANGULATION_3 )
   find_package( TBB REQUIRED )
+  include(CGAL_TBB_support)
 else( CGAL_ACTIVATE_CONCURRENT_TRIANGULATION_3 )
   option( LINK_WITH_TBB
     "Link with TBB anyway so we can use TBB timers for profiling"
     ON)
   if( LINK_WITH_TBB )
     find_package( TBB )
+    include(CGAL_TBB_support)
   endif( LINK_WITH_TBB )
 endif()
 
@@ -67,8 +69,8 @@ if ( CGAL_FOUND AND CGAL_Qt5_FOUND AND Q
 
   target_link_libraries( T3_demo PRIVATE CGAL::CGAL CGAL::CGAL_Qt5)
   target_link_libraries( T3_demo PRIVATE Qt5::OpenGL Qt5::Xml)
-  if(TBB_FOUND)
-    CGAL_target_use_TBB(T3_demo)
+  if(TARGET CGAL::TBB_support)
+    target_link_libraries(T3_demo PUBLIC CGAL::TBB_support)
   endif()
 
   include(${CGAL_MODULES_DIR}/CGAL_add_test.cmake)
diff -pruN 5.0.3-1/demo/Triangulation_3/Viewer.cpp 5.1-1/demo/Triangulation_3/Viewer.cpp
--- 5.0.3-1/demo/Triangulation_3/Viewer.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/demo/Triangulation_3/Viewer.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -2151,7 +2151,7 @@ void Viewer::wheelEvent(QWheelEvent *eve
         //  note: most mouse types work in steps of 15 degrees
         //  positive value: rotate forwards away from the user;
         //  negative value: rotate backwards toward the user.
-        m_fRadius += (event->delta()*1.f / m_iStep ); // inc-/decrease by 0.1 per step
+        m_fRadius += (event->angleDelta().y()*1.f / m_iStep ); // inc-/decrease by 0.1 per step
         if( m_fRadius < 0.1f )
             m_fRadius = 0.1f;
 
@@ -2166,7 +2166,7 @@ void Viewer::wheelEvent(QWheelEvent *eve
         //  positive value: rotate forwards away from the user;
         //  negative value: rotate backwards toward the user.
         float origR = m_fRadius;
-        m_fRadius += (event->delta()*1.f / m_iStep ); // inc-/decrease by 0.1 per step
+        m_fRadius += (event->angleDelta().y()*1.f / m_iStep ); // inc-/decrease by 0.1 per step
         if( m_fRadius < 0.1f )
             m_fRadius = 0.1f;
         // update the new point and its conflict region
@@ -2185,7 +2185,7 @@ void Viewer::wheelEvent(QWheelEvent *eve
     // resize the trackball when moving a point
     else if( m_curMode == MOVE && modifiers == Qt::SHIFT && m_isMoving ) {
         float origR = m_fRadius;
-        m_fRadius += (event->delta()*1.f / m_iStep ); // inc-/decrease by 0.1 per step
+        m_fRadius += (event->angleDelta().y()*1.f / m_iStep ); // inc-/decrease by 0.1 per step
         if( m_fRadius < 0.1f )
             m_fRadius = 0.1f;
         origR = m_fRadius / origR;
diff -pruN 5.0.3-1/examples/AABB_tree/AABB_face_graph_triangle_example.cpp 5.1-1/examples/AABB_tree/AABB_face_graph_triangle_example.cpp
--- 5.0.3-1/examples/AABB_tree/AABB_face_graph_triangle_example.cpp	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/examples/AABB_tree/AABB_face_graph_triangle_example.cpp	2020-09-08 12:12:18.000000000 +0000
@@ -25,7 +25,6 @@ void run(const FaceGraph& graph){
   // constructs the AABB tree and the internal search tree for
   // efficient distance queries.
   Tree tree( faces(graph).first, faces(graph).second, graph);
-  tree.accelerate_distance_queries();
 
   // counts #intersections with a triangle query
   Segment segment_query(p,q);
diff -pruN 5.0.3-1/examples/AABB_tree/AABB_halfedge_graph_edge_example.cpp 5.1-1/examples/AABB_tree/AABB_halfedge_graph_edge_example.cpp
--- 5.0.3-1/examples/AABB_tree/AABB_halfedge_graph_edge_example.cpp	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/examples/AABB_tree/AABB_halfedge_graph_edge_example.cpp	2020-09-08 12:12:18.000000000 +0000
@@ -27,7 +27,6 @@ void run(const HalfedgeGraph& graph){
   // efficient distance queries.
   Tree tree( CGAL::edges(graph).first,
              CGAL::edges(graph).second, graph);
-  tree.accelerate_distance_queries();
 
   // counts #intersections with a triangle query
   Triangle triangle_query(p,q,r);
diff -pruN 5.0.3-1/examples/AABB_tree/AABB_insertion_example.cpp 5.1-1/examples/AABB_tree/AABB_insertion_example.cpp
--- 5.0.3-1/examples/AABB_tree/AABB_insertion_example.cpp	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/examples/AABB_tree/AABB_insertion_example.cpp	2020-09-08 12:12:18.000000000 +0000
@@ -37,8 +37,6 @@ int main()
     // data structure to accelerate distance queries
     Tree tree(faces(polyhedron1).first, faces(polyhedron1).second, polyhedron1);
 
-    tree.accelerate_distance_queries();
-
     tree.insert(faces(polyhedron2).first, faces(polyhedron2).second, polyhedron2);
 
     // query point
diff -pruN 5.0.3-1/examples/AABB_tree/AABB_polyhedron_edge_example.cpp 5.1-1/examples/AABB_tree/AABB_polyhedron_edge_example.cpp
--- 5.0.3-1/examples/AABB_tree/AABB_polyhedron_edge_example.cpp	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/examples/AABB_tree/AABB_polyhedron_edge_example.cpp	2020-09-08 12:12:18.000000000 +0000
@@ -31,7 +31,6 @@ int main()
     Tree tree( CGAL::edges(polyhedron).first,
                CGAL::edges(polyhedron).second,
                polyhedron);
-    tree.accelerate_distance_queries();
 
     // counts #intersections with a triangle query
     Triangle triangle_query(p,q,r);
diff -pruN 5.0.3-1/examples/AABB_tree/AABB_polyhedron_facet_distance_example.cpp 5.1-1/examples/AABB_tree/AABB_polyhedron_facet_distance_example.cpp
--- 5.0.3-1/examples/AABB_tree/AABB_polyhedron_facet_distance_example.cpp	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/examples/AABB_tree/AABB_polyhedron_facet_distance_example.cpp	2020-09-08 12:12:18.000000000 +0000
@@ -31,7 +31,6 @@ int main()
     // constructs AABB tree and computes internal KD-tree
     // data structure to accelerate distance queries
     Tree tree(faces(polyhedron).first, faces(polyhedron).second, polyhedron);
-    tree.accelerate_distance_queries();
 
     // query point
     Point query(0.0, 0.0, 3.0);
diff -pruN 5.0.3-1/examples/AABB_tree/AABB_segment_3_example.cpp 5.1-1/examples/AABB_tree/AABB_segment_3_example.cpp
--- 5.0.3-1/examples/AABB_tree/AABB_segment_3_example.cpp	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/examples/AABB_tree/AABB_segment_3_example.cpp	2020-09-08 12:12:18.000000000 +0000
@@ -36,7 +36,6 @@ int main()
     // constructs the AABB tree and the internal search tree for
     // efficient distance computations.
     Tree tree(segments.begin(),segments.end());
-    tree.accelerate_distance_queries();
 
     // counts #intersections with a plane query
     Plane plane_query(a,b,d);
diff -pruN 5.0.3-1/examples/Advancing_front_surface_reconstruction/reconstruction_fct.cpp 5.1-1/examples/Advancing_front_surface_reconstruction/reconstruction_fct.cpp
--- 5.0.3-1/examples/Advancing_front_surface_reconstruction/reconstruction_fct.cpp	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/examples/Advancing_front_surface_reconstruction/reconstruction_fct.cpp	2020-09-08 12:12:18.000000000 +0000
@@ -60,6 +60,8 @@ int main(int argc, char* argv[])
 {
   std::ifstream in((argc>1)?argv[1]:"data/half.xyz");
   double per = (argc>2)?boost::lexical_cast<double>(argv[2]):0;
+  double radius_ratio_bound = (argc>3)?boost::lexical_cast<double>(argv[3]):5.0;
+
   std::vector<Point_3> points;
   std::vector<Facet> facets;
 
@@ -71,7 +73,8 @@ int main(int argc, char* argv[])
   CGAL::advancing_front_surface_reconstruction(points.begin(),
                                                points.end(),
                                                std::back_inserter(facets),
-                                               perimeter);
+                                               perimeter,
+                                               radius_ratio_bound);
 
   std::cout << "OFF\n" << points.size() << " " << facets.size() << " 0\n";
   std::copy(points.begin(),
diff -pruN 5.0.3-1/examples/Advancing_front_surface_reconstruction/reconstruction_structured.cpp 5.1-1/examples/Advancing_front_surface_reconstruction/reconstruction_structured.cpp
--- 5.0.3-1/examples/Advancing_front_surface_reconstruction/reconstruction_structured.cpp	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/examples/Advancing_front_surface_reconstruction/reconstruction_structured.cpp	2020-09-08 12:12:18.000000000 +0000
@@ -1,6 +1,7 @@
 #include <iostream>
 #include <fstream>
 #include <algorithm>
+#include <array>
 
 #include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
 #include <CGAL/Shape_detection/Efficient_RANSAC.h>
@@ -36,7 +37,7 @@ typedef CGAL::Triangulation_data_structu
 typedef CGAL::Delaunay_triangulation_3<Kernel,Tds> Triangulation_3;
 typedef Triangulation_3::Vertex_handle Vertex_handle;
 
-typedef CGAL::cpp11::array<std::size_t,3> Facet;
+typedef std::array<std::size_t,3> Facet;
 
 
 // Functor to init the advancing front algorithm with indexed points
diff -pruN 5.0.3-1/examples/Advancing_front_surface_reconstruction/reconstruction_surface_mesh.cpp 5.1-1/examples/Advancing_front_surface_reconstruction/reconstruction_surface_mesh.cpp
--- 5.0.3-1/examples/Advancing_front_surface_reconstruction/reconstruction_surface_mesh.cpp	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/examples/Advancing_front_surface_reconstruction/reconstruction_surface_mesh.cpp	2020-09-08 12:12:18.000000000 +0000
@@ -1,10 +1,10 @@
 #include <iostream>
 #include <fstream>
 #include <algorithm>
+#include <array>
 #include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
 #include <CGAL/Advancing_front_surface_reconstruction.h>
 #include <CGAL/Surface_mesh.h>
-#include <CGAL/array.h>
 #include <CGAL/disable_warnings.h>
 
 typedef std::array<std::size_t,3> Facet;
diff -pruN 5.0.3-1/examples/Alpha_shapes_3/ex_weighted_periodic_alpha_shapes_3.cpp 5.1-1/examples/Alpha_shapes_3/ex_weighted_periodic_alpha_shapes_3.cpp
--- 5.0.3-1/examples/Alpha_shapes_3/ex_weighted_periodic_alpha_shapes_3.cpp	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/examples/Alpha_shapes_3/ex_weighted_periodic_alpha_shapes_3.cpp	2020-09-08 12:12:18.000000000 +0000
@@ -41,11 +41,11 @@ int main()
   int n;
   is >> n;
   std::cout << "Reading " << n << " points " << std::endl;
-  Bare_point bp;
-  for( ; n>0 ; n--) {
-    is >> bp;
-    Weighted_point p(bp, 0.0001 * random.get_double(0., 0.015625)); // arbitrary weights
-    pts.push_back(p);
+  for( ; n>0 ; n--)
+  {
+    Bare_point bp;
+    if(is >> bp)
+      pts.emplace_back(bp, 0.0001 * random.get_double(0., 0.015625)); // arbitrary weights
   }
 
   // Define the periodic cube
diff -pruN 5.0.3-1/examples/Approximate_min_ellipsoid_d/CMakeLists.txt 5.1-1/examples/Approximate_min_ellipsoid_d/CMakeLists.txt
--- 5.0.3-1/examples/Approximate_min_ellipsoid_d/CMakeLists.txt	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/examples/Approximate_min_ellipsoid_d/CMakeLists.txt	2020-09-08 12:12:19.000000000 +0000
@@ -8,15 +8,19 @@ if ( CGAL_FOUND )
 
   # Use Eigen
   find_package(Eigen3 3.1.0 QUIET) #(3.1.0 or greater)
-  if (EIGEN3_FOUND)
-    include( ${EIGEN3_USE_FILE} )
-  endif()
+  include(CGAL_Eigen_support)
 
   # create a target per cppfile
   file(GLOB cppfiles RELATIVE ${CMAKE_CURRENT_SOURCE_DIR} ${CMAKE_CURRENT_SOURCE_DIR}/*.cpp)
   foreach(cppfile ${cppfiles})
-    if(NOT (${cppfile} STREQUAL "ellipsoid.cpp") OR EIGEN3_FOUND)
-      create_single_source_cgal_program( "${cppfile}" )
+    if(NOT (${cppfile} STREQUAL "ellipsoid.cpp") OR TARGET CGAL::Eigen_support)
+      get_filename_component(target ${cppfile} NAME_WE)
+      add_executable(${target} ${cppfile})
+      if (TARGET CGAL::Eigen_support)
+        target_link_libraries(${target} CGAL::CGAL CGAL::Eigen_support)
+      else()
+        target_link_libraries(${target} CGAL::CGAL)
+      endif()
     endif()
   endforeach()
 
@@ -25,4 +29,3 @@ else()
     message(STATUS "This program requires the CGAL library, and will not be compiled.")
 
 endif()
-
diff -pruN 5.0.3-1/examples/BGL_graphcut/alpha_expansion_example.cpp 5.1-1/examples/BGL_graphcut/alpha_expansion_example.cpp
--- 5.0.3-1/examples/BGL_graphcut/alpha_expansion_example.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/BGL_graphcut/alpha_expansion_example.cpp	2020-09-08 12:12:19.000000000 +0000
@@ -0,0 +1,96 @@
+#include <CGAL/boost/graph/alpha_expansion_graphcut.h>
+#include <boost/graph/adjacency_list.hpp>
+
+struct Vertex_property
+{
+  int label;
+  std::vector<double> cost;
+};
+
+struct Edge_property
+{
+  double weight;
+};
+
+using Graph = boost::adjacency_list <boost::setS,
+                                     boost::vecS,
+                                     boost::undirectedS,
+                                     Vertex_property,
+                                     Edge_property>;
+using GT = boost::graph_traits<Graph>;
+using vertex_descriptor = GT::vertex_descriptor;
+using edge_descriptor = GT::edge_descriptor;
+
+int main()
+{
+  std::array<char, 3> labels = { 'X', ' ', 'O' };
+
+  std::array<std::array<int, 6>, 5> input
+    = { { { 0, 2, 0, 1, 1, 1 },
+          { 0, 0, 1, 0, 1, 2 },
+          { 2, 0, 1, 1, 2, 2 },
+          { 0, 1, 1, 2, 2, 0 },
+          { 1, 1, 2, 0, 2, 2 } } };
+
+  std::array<std::array<vertex_descriptor, 6>, 5> vertices;
+
+  // Init vertices from values
+  Graph g;
+  for (std::size_t i = 0; i < input.size(); ++ i)
+    for (std::size_t j = 0; j < input[i].size(); ++ j)
+    {
+      vertices[i][j] = boost::add_vertex(g);
+      g[vertices[i][j]].label = input[i][j];
+
+      // Cost of assigning this vertex to any label is positive except
+      // for current label which is 0 (favor init solution)
+      g[vertices[i][j]].cost.resize(3, 1);
+      g[vertices[i][j]].cost[std::size_t(input[i][j])] = 0;
+    }
+
+  // Display input values
+  std::cerr << "Input:" << std::endl;
+  for (std::size_t i = 0; i < vertices.size(); ++ i)
+  {
+    for (std::size_t j = 0; j < vertices[i].size(); ++ j)
+      std::cerr << labels[std::size_t(g[vertices[i][j]].label)];
+    std::cerr << std::endl;
+  }
+
+  // Init adjacency
+  double weight = 0.5;
+  for (std::size_t i = 0; i < vertices.size(); ++ i)
+    for (std::size_t j = 0; j < vertices[i].size(); ++ j)
+    {
+      // Neighbor vertices are connected
+      if (i < vertices.size() - 1)
+      {
+        edge_descriptor ed = boost::add_edge (vertices[i][j], vertices[i+1][j], g).first;
+        g[ed].weight = weight;
+      }
+      if (j < vertices[i].size() - 1)
+      {
+        edge_descriptor ed = boost::add_edge (vertices[i][j], vertices[i][j+1], g).first;
+        g[ed].weight = weight;
+      }
+    }
+
+  std::cerr << std::endl << "Alpha expansion..." << std::endl << std::endl;
+  CGAL::alpha_expansion_graphcut (g,
+                                  get (&Edge_property::weight, g),
+                                  get (&Vertex_property::cost, g),
+                                  get (&Vertex_property::label, g),
+                                  CGAL::parameters::vertex_index_map (get (boost::vertex_index, g)));
+
+
+  // Display output graph
+  std::cerr << "Output:" << std::endl;
+  for (std::size_t i = 0; i < vertices.size(); ++ i)
+  {
+    for (std::size_t j = 0; j < vertices[i].size(); ++ j)
+      std::cerr << labels[std::size_t(g[vertices[i][j]].label)];
+    std::cerr << std::endl;
+  }
+
+  return 0;
+}
diff -pruN 5.0.3-1/examples/BGL_graphcut/CMakeLists.txt 5.1-1/examples/BGL_graphcut/CMakeLists.txt
--- 5.0.3-1/examples/BGL_graphcut/CMakeLists.txt	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/BGL_graphcut/CMakeLists.txt	2020-09-08 12:12:19.000000000 +0000
@@ -0,0 +1,43 @@
+# Created by the script cgal_create_CMakeLists
+# This is the CMake script for compiling a set of CGAL applications.
+
+cmake_minimum_required(VERSION 3.1...3.15)
+
+project( BGL_graphcut_Examples )
+
+
+# CGAL and its components
+find_package( CGAL QUIET COMPONENTS  )
+
+if ( NOT CGAL_FOUND )
+
+  message(STATUS "This project requires the CGAL library, and will not be compiled.")
+  return()
+
+endif()
+
+
+# Boost and its components
+find_package( Boost REQUIRED )
+
+if ( NOT Boost_FOUND )
+
+  message(STATUS "This project requires the Boost library, and will not be compiled.")
+
+  return()
+
+endif()
+
+# include for local directory
+
+# include for local package
+
+
+# Creating entries for all C++ files with "main" routine
+# ##########################################################
+
+
+create_single_source_cgal_program( "alpha_expansion_example.cpp" )
+create_single_source_cgal_program( "face_selection_borders_regularization_example.cpp" )
+
+
diff -pruN 5.0.3-1/examples/BGL_graphcut/data/blobby.off 5.1-1/examples/BGL_graphcut/data/blobby.off
--- 5.0.3-1/examples/BGL_graphcut/data/blobby.off	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/BGL_graphcut/data/blobby.off	2020-09-08 12:12:19.000000000 +0000
@@ -0,0 +1,6081 @@
+OFF
+2027 4050 0
+
+0.0958494 0.00115985 -0.166522
+-0.0890262 -0.0253551 -0.184829
+0.027537 -0.144185 0.125459
+-0.0957876 -0.130678 0.145836
+-0.2052 0.0911582 -0.0713736
+0.23334 0.157643 -0.0790664
+0.23333 0.183679 0.0314829
+0.0116254 0.0348884 0.195738
+0.0616047 0.162666 0.15098
+0.131725 0.211075 -0.0224619
+-0.15771 -0.0621695 -0.114255
+-0.325953 -0.116315 0.0363017
+-0.258277 -9.96328e-005 -0.066962
+0.134113 -0.042284 0.121878
+0.302143 0.124693 0.0295703
+0.133766 0.130061 -0.0945151
+0.183739 0.124065 0.130482
+-0.0399734 -0.0945666 -0.188625
+-0.394842 0.0627338 0.00536269
+-0.137998 0.173848 0.0967775
+-0.0386748 0.229049 0.0361862
+-0.0392952 -0.165982 -0.115822
+-0.171824 -0.176912 -0.00865227
+-0.150346 -0.0322864 0.1629
+-0.219923 -0.105035 -0.044157
+-0.172879 0.0784669 0.134243
+0.016567 0.217651 -0.0195546
+0.0794101 -0.0895981 -0.0472013
+-0.313698 0.0380774 0.094855
+0.318547 0.125087 -0.0198694
+0.23227 0.0225103 -0.0218715
+0.288602 0.0623503 -0.00286261
+0.232539 0.0337645 0.0884656
+-0.257774 -0.0579123 0.114719
+0.0563849 0.229622 0.0738682
+0.278436 0.167832 -0.00539504
+-0.314713 0.0915738 -0.0607805
+0.0853491 0.0667002 -0.136862
+0.0238762 -0.124001 -0.17752
+0.0259901 -0.039927 -0.194952
+-0.105554 0.0704281 -0.157699
+0.289374 0.136378 -0.0715411
+0.235568 0.0765716 -0.0823253
+-0.245592 -0.0554978 -0.0581248
+-0.391169 -0.0634443 -0.086141
+-0.319473 -0.0445621 -0.0706851
+-0.294197 -0.111617 -0.0494176
+-0.374549 0.0271624 -0.0883772
+-0.253729 -0.173014 -0.0128907
+-0.21636 -0.14897 0.0823854
+-0.066871 0.112543 0.163308
+-0.134132 -0.136482 -0.0757672
+-0.0682104 -0.196549 -0.0186103
+0.0280116 0.03795 -0.182732
+-0.217265 0.0367722 -0.0761751
+-0.162115 0.056706 -0.114965
+-0.259921 0.0739938 -0.0650284
+-0.263827 0.132437 0.0345987
+-0.244668 0.134276 -0.0327609
+-0.147003 0.169249 -0.0355379
+-0.200611 0.169171 0.0171526
+-0.204369 0.131258 0.0856347
+0.161721 0.0518293 -0.109485
+-0.303072 0.0361583 -0.0731545
+-0.179865 -0.0862429 -0.0728761
+0.226968 0.189503 -0.0282298
+0.173201 0.184456 -0.067608
+0.182543 0.205347 0.00852821
+0.175562 0.186628 0.0841909
+0.125387 0.218763 0.0470426
+0.111015 0.194442 0.111774
+-0.17503 -0.129411 -0.0459342
+0.171469 -0.0162885 0.0309691
+0.0620033 -0.1101 0.0438386
+0.000828558 -0.181486 0.0433994
+-0.091978 -0.221835 0.0852608
+-0.213686 -0.148588 -0.0242963
+-0.255516 -0.132659 -0.0348916
+-0.299158 -0.157094 -0.0280815
+-0.371903 -0.129233 -0.0453233
+-0.289351 -0.156961 0.0265973
+-0.271491 -0.114067 0.0733012
+-0.319216 -0.0636803 0.0818392
+-0.418849 -0.0484997 0.0253466
+-0.24013 -0.181604 0.0419435
+-0.168769 -0.211101 0.0641759
+-0.0313729 0.186201 -0.0797752
+0.0310134 0.152072 -0.11733
+-0.0477668 0.131896 -0.143761
+0.122424 0.139982 0.145627
+0.137703 0.0451028 0.167145
+0.0708263 0.0877104 0.177143
+-0.258701 0.0840235 0.0876754
+-0.245925 0.0201307 0.137908
+-0.31407 0.0875519 0.0473752
+-0.480337 -0.0353406 -0.045931
+-0.442794 0.0105823 -0.100726
+0.0754016 0.224229 0.00641394
+0.00534235 0.12164 0.175352
+-0.0349476 0.200792 0.131192
+0.072181 -0.109688 -0.12035
+0.119341 -0.0534773 0.0565959
+0.0867885 -0.090323 0.105552
+0.0632331 -0.04399 0.184927
+-0.0392556 -0.0539762 0.184882
+-0.361374 0.0477886 0.0558292
+-0.361193 -0.0107018 0.0908968
+-0.416327 0.0142746 0.0438039
+-0.455557 0.0276546 -0.0170243
+-0.0323993 -0.150381 -0.162142
+-0.0970416 -0.12747 -0.137748
+0.0150389 -0.155421 -0.130476
+0.0394782 -0.13985 -0.0691441
+0.278587 0.072168 0.0614076
+0.240141 0.0960435 0.10983
+0.196467 0.0560711 0.139083
+0.182764 -0.000831763 0.111317
+-0.21378 -0.185125 -0.000724208
+-0.182684 -0.206575 0.0223814
+-0.116864 -0.211741 0.0233662
+0.128798 -0.0476846 -0.0709653
+-0.151224 -0.159278 0.110503
+-0.186717 -0.0912332 0.126278
+-0.136046 0.197824 0.0292673
+-0.0799701 0.202495 -0.0219019
+-0.0908148 0.214911 0.0746841
+0.221852 0.150137 0.0928415
+0.268648 0.127162 0.0737685
+-0.100355 0.157268 -0.0946836
+-0.151142 0.117293 -0.0953014
+-0.0598251 -0.213443 0.0369284
+-0.0321557 -0.203064 0.0927271
+-0.0758939 -0.184077 0.128448
+-0.0311162 -0.14381 0.158087
+0.0178807 -0.0969897 0.176592
+0.0474164 -0.125379 -0.0118837
+0.137437 -0.0346598 0.0136343
+0.177953 -0.0103553 -0.035624
+0.139109 -0.0349729 -0.0347612
+0.158771 -0.024122 -0.0659726
+0.141486 -0.020496 -0.127385
+0.197282 0.0147799 -0.0836013
+0.0989259 -0.0612015 -0.152448
+0.146216 -0.0331176 -0.0962158
+0.115695 -0.0620378 -0.111119
+0.168295 -0.00144778 -0.105241
+0.143284 0.0207434 -0.129543
+0.123336 0.0565421 -0.124097
+0.107295 0.0970325 -0.111084
+0.0830278 0.144804 -0.105192
+0.116715 0.182192 -0.0775793
+0.0649378 0.108018 -0.128027
+0.00886149 0.108014 -0.155996
+-0.0439072 0.0766944 -0.178622
+0.0437013 0.0767872 -0.155644
+0.102639 0.0339353 -0.147441
+0.0664848 0.0308188 -0.168122
+0.0574185 -0.00862201 -0.187686
+0.0659222 -0.0512212 -0.179973
+0.0352888 -0.0823282 -0.186738
+-0.00514964 -0.0699335 -0.195786
+-0.0296251 -0.0133509 -0.196151
+-0.0434445 -0.0528929 -0.192773
+-0.0872762 -0.076468 -0.180079
+-0.129147 -0.0500857 -0.158465
+-0.129284 0.0124896 -0.155284
+0.0695584 -0.0898679 -0.162988
+0.147444 0.0903834 -0.0996231
+0.188674 0.125502 -0.0870381
+0.190514 0.0814446 -0.0929528
+0.0543565 0.190818 -0.0744677
+-0.0676336 0.0264182 -0.185549
+0.0183536 0.000577163 -0.197389
+-0.0166393 0.031641 -0.195381
+-0.001435 0.0694683 -0.181212
+0.248561 0.0317378 0.0328905
+-0.128762 -0.0922393 -0.136755
+-0.1478 -0.10176 -0.0985596
+-0.183769 0.000809428 -0.0981379
+-0.142413 -0.159393 -0.0379004
+-0.0934853 -0.171508 -0.0612999
+-0.039723 -0.182589 -0.0634592
+-0.0116406 -0.185728 -0.0129543
+-0.130894 -0.187892 -0.00872955
+-0.000937363 -0.169029 -0.0876581
+-0.366824 -0.0724282 0.0453077
+-0.394642 -0.0954812 -0.00194395
+-0.440674 -0.0692916 -0.0230396
+-0.458899 -0.0268371 0.000520617
+-0.449781 -0.0715781 -0.0816362
+-0.416087 -0.102536 -0.0556142
+-0.310063 0.120373 -0.00558273
+-0.359557 0.0973339 -0.0246998
+-0.408598 0.0712299 -0.0566547
+0.210226 0.0065473 0.0528917
+0.10443 -0.0645713 -0.0512281
+0.0928885 -0.0838549 -0.0852733
+0.0647924 -0.112421 -0.0731805
+-0.25816 -0.0952338 -0.0476423
+-0.29017 -0.0715026 -0.0590899
+-0.334066 -0.0900194 -0.0653012
+-0.0881154 0.18359 0.121824
+-0.116958 0.131991 0.135245
+-0.115528 0.0740867 0.158033
+-0.0562597 0.0460874 0.182682
+-0.154707 0.0281229 0.162592
+-0.101345 0.00172185 0.183998
+-0.200295 -0.00554548 0.159778
+-0.055075 0.159282 0.150294
+-0.00513261 0.168012 0.15663
+0.0256509 0.210455 0.123931
+-0.000484079 0.23337 0.0785256
+0.0243063 0.232033 0.0312676
+0.297468 0.0856477 -0.0518916
+0.264291 0.0516813 -0.0431796
+-0.000333759 -0.14362 -0.17393
+-0.00811779 -0.111725 -0.189145
+0.0233168 -0.144538 -0.156498
+0.0520569 -0.121215 -0.153498
+0.0403492 -0.139089 -0.132197
+0.0307313 -0.148756 -0.10091
+0.0520152 -0.129366 -0.0938706
+-0.0358469 -0.125283 -0.1814
+-0.0719963 -0.112037 -0.174525
+-0.0579005 -0.136109 -0.165268
+-0.053519 -0.152974 -0.139859
+-0.0826048 -0.155575 -0.104309
+-0.102837 -0.101678 -0.161026
+-0.0801971 -0.127893 -0.156609
+-0.0767072 -0.142872 -0.136199
+0.00103891 -0.155684 -0.151643
+-0.0126392 -0.163805 -0.12728
+-0.153213 -0.0200729 -0.132827
+-0.157003 0.0192245 -0.127469
+-0.11382 -0.0744379 -0.166512
+-0.106384 -0.047927 -0.177108
+-0.117555 -0.0169507 -0.168029
+-0.0991594 0.0131089 -0.174764
+-0.0913706 0.0448592 -0.172496
+-0.0756406 0.0736176 -0.170824
+-0.0873468 0.110478 -0.147509
+-0.0594532 0.101489 -0.163026
+-0.0244708 0.103135 -0.166028
+-0.127423 0.0986322 -0.129408
+-0.118816 0.0419869 -0.157553
+-0.133878 0.0664189 -0.13894
+-0.0153011 0.128439 -0.147224
+-0.0252286 0.160621 -0.116778
+0.00635037 0.1777 -0.0954873
+0.00400885 0.200533 -0.0580222
+-0.0317462 0.211172 -0.0259917
+-0.0811961 -0.0509828 -0.185782
+-0.0616152 -0.0307688 -0.191254
+-0.0704314 -0.00277796 -0.188487
+-0.0629541 -0.0698913 -0.187795
+-0.0455164 0.0102212 -0.193522
+-0.0430043 0.0373487 -0.190547
+-0.0241987 0.0567533 -0.188576
+-0.063225 0.0525651 -0.181777
+-0.0655626 0.155373 -0.115695
+-0.0687568 0.176717 -0.0835132
+-0.104738 0.179668 -0.054078
+-0.0566616 0.195811 -0.052082
+-0.0949674 0.135982 -0.123239
+-0.123404 0.134453 -0.104745
+-0.141589 0.148908 -0.070839
+-0.189965 0.138072 -0.0480893
+-0.139982 0.0339539 -0.142608
+-0.0698644 -0.0908595 -0.182455
+-0.0869951 -0.0980032 -0.173608
+-0.0888347 -0.113272 -0.163583
+-0.100216 -0.115422 -0.150258
+-0.116397 -0.113163 -0.131517
+-0.109588 -0.135899 -0.107769
+-0.113596 -0.103854 -0.146711
+-0.115309 -0.0895474 -0.15719
+-0.129883 -0.0730398 -0.149787
+-0.144897 -0.0545486 -0.136877
+-0.141243 -0.0752655 -0.131382
+-0.140529 -0.0929734 -0.117774
+-0.129819 -0.114322 -0.110492
+-0.154957 -0.0820572 -0.10525
+-0.17319 -0.0660304 -0.0911662
+-0.205716 -0.0543781 -0.0686289
+-0.174187 -0.0328375 -0.103341
+-0.163372 -0.0911572 -0.0868618
+-0.155341 -0.116364 -0.0743424
+-0.198099 -0.0244193 -0.0815967
+-0.209496 0.00564775 -0.0793296
+-0.104811 -0.153707 -0.0834025
+-0.193364 0.0274989 -0.0905073
+-0.195098 0.0613981 -0.0852378
+-0.174866 0.0906393 -0.0907104
+-0.183042 0.115353 -0.0707315
+-0.209624 0.117075 -0.0568091
+-0.23628 0.0972374 -0.0606186
+-0.27478 0.110265 -0.0497172
+-0.228367 0.0668769 -0.0698484
+-0.251112 0.040523 -0.068888
+-0.216608 0.137171 -0.0376511
+-0.227411 0.154531 -0.00555484
+-0.194777 0.158609 -0.0193378
+-0.164962 0.178815 -0.000179778
+-0.231456 0.152575 0.0323689
+-0.199826 0.160548 0.0572605
+-0.236965 0.128656 0.0633833
+-0.17123 0.182733 0.0404826
+-0.128564 0.190448 -0.00837419
+-0.0991129 0.20863 0.0142043
+-0.168524 0.169871 0.0755126
+-0.163588 0.136428 0.107596
+-0.141392 0.192471 0.0652526
+-0.0624533 0.217825 0.00842925
+-0.073927 0.221866 0.0429573
+-0.0492884 0.227715 0.0784747
+-0.262211 0.142443 -0.00109806
+-0.266844 0.107888 0.0613638
+-0.28881 0.0827382 0.0705891
+-0.282314 0.0571684 0.0961857
+-0.244984 0.0555902 0.115396
+-0.28409 0.0230004 0.120573
+-0.20539 0.04444 0.13869
+-0.216657 0.0913735 0.106246
+-0.321301 -0.0037008 0.113436
+-0.269434 -0.018038 0.132429
+-0.227907 -0.0342978 0.138796
+-0.316898 0.0628548 0.0711805
+-0.295857 -0.0407173 0.108514
+-0.339188 0.0416768 0.077048
+-0.366857 0.0205973 0.0768175
+-0.394153 -0.0072429 0.0688749
+-0.337377 0.0176792 0.0971164
+-0.372548 -0.0402007 0.0673096
+-0.388725 0.0294257 0.0537187
+-0.404944 0.0403335 0.0272473
+-0.426273 0.0449791 -0.00258157
+-0.436411 0.0526146 -0.0384857
+-0.471392 0.0283371 -0.0641431
+-0.434831 0.0477246 -0.0809856
+-0.441717 0.016121 0.0160487
+-0.378625 0.0566371 0.0313463
+-0.346799 0.0750443 0.0354924
+-0.367064 0.0785269 0.0104993
+-0.336435 0.0992608 0.009509
+-0.43494 -0.0160295 0.0332624
+-0.38249 0.0807931 -0.0150041
+-0.380719 0.0863079 -0.0474863
+-0.41036 0.065654 -0.0233953
+-0.377973 0.0638507 -0.078275
+-0.341264 0.0474669 -0.0785868
+-0.336052 -0.000329239 -0.0797136
+-0.348501 0.0926189 -0.0565738
+-0.328397 0.110798 -0.0340867
+-0.377367 -0.0192039 -0.089504
+-0.421433 -0.0305453 -0.0985605
+-0.114682 0.207742 0.0527045
+-0.116467 0.199227 0.0825376
+-0.0938153 0.201811 0.100205
+-0.0638427 0.205699 0.114819
+-0.0372253 0.220532 0.106709
+-0.00713486 0.21589 0.120472
+0.00412912 0.194028 0.140734
+0.0286461 0.172559 0.151816
+0.0369435 0.139115 0.166063
+0.0759809 0.12629 0.163777
+0.112766 0.0995206 0.164051
+0.0519201 0.191606 0.135347
+0.0585701 0.215447 0.107917
+0.0916601 0.217851 0.0867493
+0.0858377 0.181441 0.133122
+0.08665 0.226782 0.0485108
+0.0377801 0.104182 0.178026
+0.00202303 0.0782792 0.1876
+0.0395654 0.0658429 0.187567
+0.0855167 0.0342036 0.183976
+0.118664 -0.0089809 0.165937
+0.116016 0.168844 0.131562
+0.155925 0.155755 0.124628
+0.139187 0.180044 0.113112
+0.135622 0.205066 0.082699
+0.161426 0.206082 0.0537547
+0.199219 0.191721 0.0517405
+0.029606 0.22756 0.0947735
+0.150049 0.109534 0.149415
+-0.0301278 0.101389 0.177374
+-0.0279917 0.137845 0.166428
+0.180989 0.0895417 0.14411
+0.209834 0.100628 0.127196
+0.152129 0.0757484 0.158321
+0.223766 0.123301 0.109789
+0.200021 0.141994 0.112242
+0.196231 0.166697 0.0946543
+0.216873 0.17303 0.0709274
+0.247026 0.154665 0.0683708
+0.275262 0.149044 0.0426982
+0.249149 0.11998 0.094882
+0.265608 0.0907272 0.0873119
+0.247289 0.0635133 0.0993065
+0.223522 0.0693513 0.123084
+0.214216 0.0371396 0.116998
+0.187747 0.02422 0.133089
+0.154819 -0.00119342 0.143999
+0.168326 0.0434847 0.152936
+0.170987 0.169879 0.105953
+-0.174282 0.0232202 -0.10807
+-0.284597 -0.0732055 0.0931726
+-0.25108 -0.09202 0.097698
+-0.222464 -0.0753609 0.11957
+-0.214213 -0.113101 0.103683
+-0.302903 -0.0945116 0.0686402
+-0.195428 -0.0556839 0.139432
+-0.156829 -0.0700418 0.146298
+-0.0976748 -0.0658008 0.166048
+-0.144421 -0.115749 0.132619
+-0.29612 -0.123273 0.0514632
+-0.266231 -0.145856 0.0540206
+0.0488831 0.0294674 0.192113
+0.023936 -0.0145482 0.198863
+0.0717513 -0.00410704 0.187615
+-0.0287241 -0.00156414 0.194867
+-0.152398 -0.135765 -0.057189
+-0.100872 -0.0864911 -0.170917
+-0.0593818 0.184347 0.134966
+-0.035312 0.178094 0.146741
+-0.0776543 0.165473 0.138
+-0.105632 0.159255 0.127513
+-0.0766796 0.138244 0.150939
+-0.0968478 0.113293 0.151863
+-0.0853482 0.0853291 0.164915
+-0.0917353 0.0496009 0.172492
+-0.126566 0.10163 0.143708
+-0.133447 0.150712 0.116875
+-0.144857 0.120635 0.126973
+-0.173737 0.107421 0.118943
+-0.1614 -0.146542 -0.0390305
+-0.186603 -0.146865 -0.0282327
+-0.201541 -0.125228 -0.0379031
+-0.193134 -0.103124 -0.0539853
+-0.211049 -0.0803921 -0.0567836
+-0.159914 -0.163462 -0.0232774
+-0.227528 -0.128247 -0.0340061
+-0.238879 -0.149045 -0.0253663
+-0.226397 -0.167794 -0.0140136
+-0.201369 -0.167497 -0.0134536
+-0.262857 -0.15434 -0.0259797
+-0.280568 -0.171091 -0.0144786
+-0.281744 -0.138503 -0.0368599
+-0.309473 -0.135166 -0.0434532
+-0.340134 -0.141114 -0.00457478
+-0.269395 -0.179854 0.0148325
+-0.305326 -0.162726 0.00125739
+-0.315329 -0.141054 0.0209936
+-0.328044 -0.152371 -0.0287465
+-0.340488 -0.131656 -0.0493684
+-0.321044 -0.1131 -0.0550007
+-0.356693 -0.110151 -0.0633657
+-0.36424 -0.0799503 -0.0753809
+-0.387596 -0.110687 -0.0644618
+-0.362559 -0.0476361 -0.081757
+-0.396237 -0.113781 -0.0353861
+-0.371596 -0.120753 -0.0109802
+-0.364576 -0.10081 0.0203935
+-0.417619 -0.0918209 -0.0251753
+-0.418352 -0.0719487 0.00161618
+-0.393997 -0.0714926 0.0251311
+-0.394772 -0.0475977 0.0459488
+-0.402686 -0.0895353 -0.0784898
+-0.42084 -0.0681017 -0.0897032
+-0.4404 -0.0509827 0.00126062
+-0.459655 -0.0472284 -0.0203999
+-0.460671 -0.0638923 -0.050193
+-0.475196 -0.0484471 -0.0775462
+-0.489471 -0.00806259 -0.0776671
+-0.475138 -0.0247761 -0.0213132
+-0.486424 -0.00153525 -0.0440281
+-0.468403 0.00156711 -0.00678311
+-0.242074 -0.191329 0.00830808
+-0.429783 -0.0877853 -0.0735719
+-0.43628 -0.0850827 -0.0461373
+-0.191731 -0.183481 -0.00157165
+-0.201183 -0.19839 0.0124415
+-0.207449 -0.198571 0.0495067
+-0.173223 -0.193455 0.00663966
+-0.149821 -0.209649 0.0233901
+-0.134115 -0.223229 0.055024
+-0.136108 -0.199724 0.0945508
+-0.152801 -0.187823 -0.00205719
+-0.18838 -0.178751 0.0798359
+-0.0995335 -0.223011 0.0513334
+-0.114673 -0.175248 0.119217
+-0.440477 -0.0499869 -0.09565
+-0.451957 -0.0204046 -0.101857
+-0.0735325 0.215767 0.0934543
+-0.234693 0.0151621 -0.0705707
+-0.232313 -0.0142483 -0.068254
+-0.256967 -0.0290178 -0.0629623
+-0.292996 -0.0166728 -0.06866
+-0.201581 -0.207306 0.0313934
+-0.223725 -0.199636 0.0229592
+-0.183119 -0.212742 0.045317
+-0.188846 -0.19935 0.0634693
+-0.160028 -0.218047 0.0431693
+-0.204584 -0.184112 0.0658921
+-0.172989 -0.194432 0.0779813
+-0.166513 -0.17455 0.0946992
+-0.18471 -0.141617 0.103157
+-0.153237 -0.206081 0.080472
+-0.13192 -0.217895 0.078765
+-0.113273 -0.211482 0.0952368
+-0.0876939 -0.205117 0.110143
+-0.0617909 -0.215381 0.0941536
+-0.0467071 -0.213033 0.0660721
+-0.0272 -0.201713 0.0396055
+-0.0428995 -0.202112 0.00684775
+-0.0500405 -0.19896 0.118331
+-0.00923828 -0.174641 0.123557
+-0.0448164 -0.172256 0.141435
+0.0162634 -0.165534 0.0849788
+0.0489655 -0.130695 0.0849369
+0.0328257 -0.147003 0.0438674
+-0.0648791 -0.145555 0.150587
+-0.0537054 -0.100021 0.169641
+-0.0147565 -0.194348 0.0677628
+-0.00796641 -0.18555 0.0956339
+0.016581 -0.163544 0.00884718
+0.0177079 -0.160734 -0.0359242
+0.058778 -0.116217 0.123118
+0.0778076 -0.081498 0.154441
+0.0352324 -0.119032 0.151956
+0.00114088 -0.126919 0.163291
+-0.0183444 -0.100065 0.178246
+-0.00235339 -0.0663502 0.190792
+0.0341041 -0.066022 0.187462
+0.0492495 -0.089232 0.169327
+-0.0128114 -0.0329744 0.194855
+-0.115415 -0.225031 0.0662711
+-0.0730773 -0.220943 0.0605258
+-0.343357 -0.0395192 0.0853781
+-0.0113899 -0.18949 0.0169894
+-0.118843 -0.193531 0.107193
+-0.138023 -0.179018 0.106649
+-0.100643 -0.19026 0.116541
+-0.0947661 -0.172586 0.128428
+-0.111739 -0.149926 0.13206
+-0.0760933 -0.163142 0.139445
+-0.234903 -0.085372 -0.0497887
+-0.029894 0.2223 0.00506028
+-0.0077 0.230474 0.027082
+-0.0166405 0.233414 0.05448
+0.0101539 0.234325 0.0539289
+0.0372094 0.233262 0.0550673
+0.0555455 0.230342 0.0298477
+0.04283 0.224072 -0.000217558
+0.067148 0.21097 -0.0360407
+0.011472 0.226074 0.00687303
+-0.0558591 -0.10463 -0.182934
+-0.0565936 -0.121689 -0.17642
+0.121759 -0.0457408 -0.0178067
+-0.0530737 0.134352 0.160908
+0.123973 0.0718871 0.168299
+0.0221523 0.233586 0.0737068
+-0.00353417 -0.152521 0.145381
+-0.157503 -0.0407249 -0.123137
+-0.402744 0.0442574 -0.0877635
+-0.405641 0.00799179 -0.0966936
+-0.121506 -0.157661 -0.0590498
+-0.111905 -0.177728 -0.0341975
+-0.0778988 0.0229642 0.18259
+-0.0653335 -0.0142125 0.187109
+-0.049684 0.0182208 0.189225
+-0.0227026 0.0405432 0.191262
+-0.10136 -0.0715673 -0.175466
+-0.30971 0.108636 0.0239712
+0.284328 0.102013 0.0657535
+0.30034 0.0876156 0.0350277
+0.313093 0.101645 0.00557695
+0.281734 0.059859 0.0301131
+0.31369 0.0939172 -0.0255194
+0.313301 0.115146 -0.0514087
+0.305361 0.148098 -0.0427014
+0.266831 0.169885 -0.0514279
+0.28652 0.1262 0.0529506
+0.302138 0.149778 -0.0123269
+0.29091 0.14998 0.0162535
+0.26822 0.166638 0.0212854
+0.247534 0.182808 -0.00123423
+0.215545 0.195039 0.00400645
+0.253566 0.165453 0.0442228
+0.287285 0.162995 -0.0300387
+0.264833 0.175412 -0.0261528
+0.246183 0.18101 -0.0421722
+0.216699 0.181738 -0.0604516
+0.191444 0.197063 -0.0374138
+0.197618 0.159773 -0.0814181
+0.162626 0.151044 -0.0869195
+0.246413 0.171038 -0.0640185
+0.262152 0.14856 -0.0768471
+0.236906 0.117298 -0.0837704
+0.266508 0.119186 -0.0801675
+0.291373 0.107261 -0.070559
+0.268727 0.0804238 -0.0713972
+0.157844 0.20099 -0.0419927
+0.127221 0.199769 -0.0524808
+0.0921871 0.199144 -0.0582807
+0.160558 0.208675 -0.0124965
+0.136234 0.216589 0.0128056
+0.106226 0.209253 -0.0354265
+0.105789 0.21885 -0.00422888
+0.0987139 0.224454 0.0227998
+0.310132 0.1309 0.00482773
+0.264335 0.0486808 0.050986
+0.242897 0.0320616 0.0613799
+0.255315 0.0509231 0.0765592
+0.223445 0.0189006 0.0703454
+0.198446 0.00226779 0.0829469
+0.159642 -0.0264743 0.0770455
+0.184659 -0.00995121 0.0581628
+0.295808 0.0728961 -0.0273998
+0.304133 0.0814612 -0.00568077
+0.296666 0.0746018 0.0156567
+0.278322 0.0564608 -0.0240055
+0.262042 0.0406401 0.00220734
+0.259669 0.0411964 -0.0218671
+0.243852 0.034963 -0.0421001
+0.215266 0.0162635 -0.0527158
+0.245815 0.0518064 -0.0675671
+0.214942 0.046667 -0.0864707
+0.282632 0.0654666 -0.0447211
+0.268523 0.0633842 -0.0596897
+0.281296 0.0760982 -0.0592187
+0.283254 0.0910852 -0.0692596
+0.26702 0.0993994 -0.0774233
+0.252339 0.0866404 -0.080311
+0.235073 0.0969385 -0.0840247
+0.210826 0.109417 -0.0864937
+0.214308 0.0858398 -0.0880213
+0.214047 0.137226 -0.0844927
+0.25352 0.0685975 -0.0733756
+0.252369 0.108439 -0.0822663
+0.205928 0.0659372 -0.0916771
+0.188313 0.0431929 -0.0992933
+0.170591 0.0255543 -0.111447
+0.194227 -0.00344713 0.0344716
+0.186031 -0.00641741 -0.000610057
+0.221621 0.0132033 0.0205145
+0.158466 -0.0217212 -0.0111899
+0.203821 0.00534862 -0.0246978
+-0.0873673 -0.213787 0.0258875
+-0.0988088 -0.199309 -0.00314121
+-0.0250407 0.231884 0.077572
+-0.0132887 0.227702 0.0992761
+0.00907169 0.224567 0.10582
+-0.0381223 0.231393 0.0596562
+-0.0597913 0.227196 0.0590599
+-0.068062 0.222998 0.0762016
+-0.0768204 0.221895 0.0606255
+-0.0944706 0.215866 0.048674
+-0.107247 0.209847 0.0333512
+-0.119042 0.202294 0.0174804
+-0.110264 0.199934 -0.00206571
+-0.108772 0.189998 -0.0282832
+-0.0885764 0.215953 0.0295549
+-0.135337 0.194063 0.00985829
+-0.156117 0.187793 0.0203953
+-0.177021 0.179366 0.0189113
+-0.299224 0.119756 -0.0341043
+-0.285395 0.131717 -0.0127806
+-0.281551 0.130928 0.0140859
+-0.285947 0.115513 0.0369868
+-0.150405 -0.218482 0.06527
+0.0152028 -0.0430482 0.197139
+0.0402834 -0.0349374 0.194561
+-0.0851073 -0.18549 -0.0382905
+-0.0673308 -0.181154 -0.0594061
+-0.0570762 -0.172068 -0.0890807
+-0.0287352 -0.17445 -0.0907527
+-0.0159017 -0.177558 -0.069441
+-0.0204618 -0.184795 -0.0411489
+0.00873811 -0.167111 -0.0615609
+-0.0381433 -0.194183 -0.0204349
+-0.051034 -0.189416 -0.0417802
+-0.35122 -0.0598012 0.0662197
+-0.33698 -0.0880177 0.0525415
+-0.414438 -0.029614 0.0437536
+-0.396684 -0.0282297 0.0588966
+-0.410734 -0.0120685 0.0558495
+-0.40436 0.00706511 0.0586068
+-0.386161 0.0132369 0.0692927
+-0.377761 -0.00208539 0.0810993
+-0.373236 -0.0223653 0.0783082
+-0.357341 -0.0277509 0.0855502
+-0.337893 -0.0190526 0.100516
+-0.32019 -0.038641 0.0982197
+-0.310347 -0.0235748 0.112107
+-0.295255 -0.0032073 0.126315
+-0.290437 -0.0236301 0.120756
+-0.27395 -0.0406672 0.11799
+-0.25102 -0.0377029 0.128663
+-0.227421 -0.0666674 -0.0569025
+-0.226022 -0.0447003 -0.0636812
+-0.210588 -0.0356727 -0.0716031
+-0.214081 -0.0168631 -0.0739229
+-0.194807 -0.0430067 -0.0792776
+-0.188472 -0.062926 -0.0770752
+-0.221809 -0.00259957 -0.0725998
+-0.22142 0.0115402 -0.0742428
+-0.209966 0.0222645 -0.0797481
+-0.203742 0.0357387 -0.083051
+-0.208844 0.0522034 -0.0787474
+-0.190822 0.0450951 -0.0909107
+-0.177425 0.0611661 -0.0991239
+-0.209154 0.071997 -0.0756773
+-0.16361 0.0781189 -0.106493
+-0.192643 0.0804217 -0.0817311
+-0.188553 0.0983129 -0.0769631
+-0.158207 0.0981986 -0.101697
+-0.14589 0.0877094 -0.119136
+-0.14156 0.107293 -0.111634
+-0.123827 0.117911 -0.119323
+-0.108028 0.110926 -0.135713
+-0.112326 0.0896534 -0.145444
+-0.0919879 0.0895894 -0.157183
+-0.354407 -0.140129 -0.0276509
+0.0608099 -0.102747 0.147637
+0.0795974 -0.0939192 0.130946
+0.106273 -0.0662511 0.133449
+0.114315 -0.0633548 0.0946988
+0.0911756 -0.080818 0.0664564
+0.0945934 -0.0696213 0.0242046
+0.094466 -0.0495519 0.164901
+0.103223 -0.059939 -0.0150141
+0.107659 -0.0557411 0.00875241
+0.112343 -0.0548812 0.0329695
+0.13301 -0.0399767 0.0388538
+0.140194 -0.0391239 0.064311
+0.140474 -0.0411727 0.0934195
+0.15845 -0.0224544 0.11313
+0.0643705 -0.103524 -0.0168157
+0.0586643 -0.110442 0.0129414
+0.0751879 -0.0909703 0.026571
+0.0799955 -0.0839899 -0.0182911
+0.0805977 -0.0822607 0.00527421
+-0.199652 -0.0706452 -0.0659354
+-0.19524 -0.0870565 -0.0610569
+-0.363652 0.00674479 0.087702
+-0.348592 0.00177698 0.0991638
+-0.352099 0.0205518 0.0871438
+-0.333961 0.00443922 0.106489
+-0.315086 0.0187442 0.108631
+-0.00316359 0.0489169 -0.190227
+0.00828848 0.0264747 -0.194092
+-0.00891282 0.00524446 -0.198566
+-0.000837179 -0.0247146 -0.198369
+-0.024708 -0.191035 0.113136
+-0.028144 -0.180474 0.13121
+-0.0241438 -0.162896 0.145613
+-0.0451245 -0.186443 0.130327
+-0.0607705 -0.19034 0.127269
+-0.0688358 -0.203401 0.115117
+-0.062611 -0.17513 0.137084
+-0.0574099 -0.160773 0.145842
+-0.0476802 -0.14828 0.153745
+-0.04727 -0.124358 0.16362
+-0.0742399 -0.116376 0.158165
+-0.103323 -0.097917 0.154292
+-0.022878 -0.122805 0.168832
+0.027446 0.0183553 -0.190554
+0.0472879 0.0249841 -0.18012
+0.047031 0.0480852 -0.168168
+0.0245826 0.0638303 -0.173037
+0.0689779 0.0532592 -0.153469
+0.039009 0.000694828 -0.192586
+0.0399169 -0.0220207 -0.193298
+0.0592175 -0.0308582 -0.186299
+0.0881819 -0.0293513 -0.172655
+0.11654 -0.0237422 -0.15226
+0.0457321 -0.0484051 -0.189
+0.0283473 -0.0619206 -0.192344
+0.00884709 -0.0532585 -0.196777
+-0.0167742 -0.0459663 -0.196316
+0.0146735 -0.0790357 -0.193471
+0.0207706 -0.101966 -0.18762
+0.0449927 -0.105242 -0.174844
+0.0651623 0.0109671 -0.179127
+0.0781854 -0.00786363 -0.178011
+0.081928 0.0167058 -0.168007
+0.0839486 0.0345127 -0.157031
+0.0127453 0.0857324 -0.167693
+0.0293959 0.0958981 -0.153813
+0.030408 0.12443 -0.136266
+-0.00876124 0.0888046 -0.17281
+-0.0206715 0.0751227 -0.181354
+0.0533308 0.133749 -0.12095
+0.0571218 0.158463 -0.105692
+0.0821399 0.175144 -0.0882636
+0.0339587 0.172826 -0.0984761
+0.0250338 0.190467 -0.0767026
+0.0361198 0.205898 -0.0486562
+0.1337 0.159717 -0.0885808
+0.153192 0.173725 -0.0795409
+0.150357 0.191093 -0.0628394
+0.000183964 -0.174592 -0.0415267
+0.00626444 -0.172179 -0.0191669
+0.0228004 -0.155988 -0.0130102
+0.0351251 -0.141473 0.00863422
+0.0360237 -0.140713 -0.0316276
+0.00202874 -0.177272 0.00120065
+0.00544145 -0.176177 0.0227339
+0.017425 -0.164748 0.0398744
+0.0235388 -0.159186 0.0627028
+0.0538022 -0.119801 -0.0390143
+0.0025606 -0.0950454 -0.192557
+-0.0188238 -0.0933552 -0.192037
+-0.0309944 -0.0730462 -0.192797
+-0.0281068 -0.109252 -0.187754
+-0.075043 -0.21301 0.103434
+-0.0894248 -0.215145 0.0987529
+-0.101426 -0.217246 0.0934193
+-0.112609 -0.221304 0.0820512
+-0.102007 -0.206618 0.104586
+-0.333709 0.0734688 -0.0709339
+-0.3055 0.0652843 -0.0701277
+-0.357922 0.075518 -0.0710404
+-0.358936 0.057208 -0.0795117
+0.0591707 0.23129 0.0518182
+0.203248 0.0154733 0.105733
+-0.472546 0.0131591 -0.0288556
+-0.461471 0.0322821 -0.0411949
+-0.451311 0.0441704 -0.0615887
+-0.456508 0.0336736 -0.0849523
+-0.475353 0.013652 -0.0871544
+-0.48302 -0.0285378 -0.0911039
+-0.489181 -0.0301113 -0.0677967
+-0.463262 -0.0404697 -0.0963726
+-0.432771 0.0573181 -0.0604522
+-0.41555 0.0601452 -0.0761121
+-0.397075 0.0676922 -0.0728843
+-0.382154 0.0783063 -0.065485
+-0.366919 0.0871881 -0.0598129
+-0.364506 0.0945902 -0.0426634
+-0.346367 0.102863 -0.0390517
+-0.341911 0.1063 -0.0193301
+-0.352907 0.0953717 -0.00411107
+-0.330555 0.100814 -0.0516651
+-0.31083 0.108258 -0.0479398
+-0.292529 0.0971517 -0.0568892
+-0.376027 0.088421 -0.0301143
+-0.395431 0.078534 -0.0331188
+-0.271862 0.0913249 -0.0598455
+-0.283283 0.0717778 -0.0661175
+-0.422152 0.0632947 -0.0456245
+-0.333323 0.0882853 -0.0627279
+-0.437052 0.0306019 -0.0942911
+-0.420832 0.0396817 -0.0908281
+-0.420208 0.0209628 -0.0962905
+-0.424603 0.00147628 -0.100178
+-0.409238 -0.01306 -0.0971805
+-0.397117 -0.0349289 -0.0925506
+-0.402307 0.0263118 -0.0936004
+-0.387819 0.0136986 -0.0921055
+-0.364576 0.00324772 -0.0872997
+-0.391817 -0.00528513 -0.0934207
+-0.388899 0.0353587 -0.08989
+-0.373861 0.0459462 -0.0841546
+-0.356581 0.0354061 -0.0840319
+-0.335616 0.0239191 -0.079938
+-0.312686 0.0114246 -0.0749035
+-0.283821 0.014154 -0.0699887
+-0.388663 0.0501317 -0.0854246
+-0.400625 0.0570771 -0.0817872
+-0.277312 0.0395577 -0.0695801
+-0.263595 0.0224806 -0.0681152
+-0.411494 0.0521117 -0.0843958
+-0.423315 0.0520061 -0.0810506
+-0.4266 0.0583367 -0.0710367
+-0.420528 0.0643365 -0.0597794
+-0.437215 0.0529979 -0.0705989
+-0.447466 0.0442731 -0.0753282
+-0.459224 0.0370869 -0.071264
+-0.468361 0.0282442 -0.0784113
+-0.478009 0.0179968 -0.0729376
+-0.479951 0.0140616 -0.0538839
+-0.463083 0.0214952 -0.0907124
+-0.463457 0.0029889 -0.101217
+-0.485691 0.00647281 -0.0788714
+-0.490194 0.00123743 -0.0619417
+-0.489027 -0.0161994 -0.0553181
+-0.483623 -0.0188975 -0.0375911
+-0.478319 -0.00529318 -0.0240312
+-0.483932 -0.00270692 -0.0898923
+-0.481107 -0.0153955 -0.0935496
+-0.46911 -0.0252159 -0.0996541
+-0.488301 -0.019441 -0.0836338
+-0.490506 -0.0186274 -0.0716706
+-0.487468 -0.0311436 -0.0799346
+-0.479204 -0.0391906 -0.0852952
+-0.482273 -0.039567 -0.0722516
+-0.473588 -0.0502137 -0.05761
+-0.482362 -0.0375783 -0.0589935
+-0.467989 -0.0491582 -0.038458
+-0.464881 -0.0634395 -0.0695957
+-0.451164 -0.0752001 -0.062683
+-0.461144 -0.0570308 -0.0871617
+-0.47021 -0.0467699 -0.0885606
+-0.454868 -0.06127 -0.0324684
+-0.443099 0.03929 -0.0866065
+-0.473886 -0.0349722 -0.094979
+-0.487452 -0.028162 -0.0529376
+-0.46882 -0.0106219 -0.0998229
+-0.448274 0.0309025 -0.0923286
+-0.443498 0.0213393 -0.0966431
+-0.45456 0.0148388 -0.0976978
+-0.431899 0.0165694 -0.0983475
+-0.474696 0.000410854 -0.0972292
+-0.469485 0.00963021 -0.095246
+0.296955 0.106656 0.0472498
+0.291078 0.0883167 0.0527806
+0.288813 0.0737593 0.0408672
+0.277877 0.0614053 0.0473432
+0.270351 0.0591299 0.0606321
+0.266187 0.0697888 0.0793687
+0.305647 0.105588 0.0299097
+0.312126 0.116415 0.0165124
+0.317369 0.116749 0.00104001
+0.316163 0.104461 -0.011578
+0.318211 0.110014 -0.0273182
+0.313357 0.101027 -0.0413356
+0.318559 0.12438 -0.0374211
+0.311096 0.133216 -0.0514791
+0.303008 0.123002 -0.0663721
+0.313352 0.139423 -0.0292404
+0.305168 0.100598 -0.0568232
+0.300383 0.145261 -0.0600217
+0.280531 0.15626 -0.0639751
+0.291353 0.156796 -0.049708
+0.305426 0.0872955 -0.0378317
+0.311948 0.137809 -0.0138391
+0.307419 0.142673 -0.00177921
+0.295409 0.151976 0.00151047
+0.302664 0.141484 0.0124035
+0.29664 0.139673 0.0272525
+0.303353 0.1511 -0.0276759
+-0.0355418 -0.15595 0.151401
+0.245876 0.184257 -0.021774
+0.231974 0.189434 -0.00989912
+0.214634 0.19537 -0.0149903
+0.197855 0.201136 -0.00456193
+0.150815 0.022535 0.157779
+0.124646 0.0192251 0.170169
+0.0987978 0.00829006 0.179252
+0.0935369 -0.0207146 0.176347
+0.137567 0.00340953 0.158961
+0.135309 -0.0228522 0.146302
+0.124465 -0.0438634 -0.132424
+-0.336318 -0.127906 0.0165096
+-0.332396 -0.00896673 0.107708
+-0.324334 -0.0217195 0.106662
+-0.343027 -0.00837195 0.10223
+-0.349979 -0.0172219 0.0947275
+-0.351563 -0.0075692 0.0970012
+-0.357567 0.000498455 0.0937007
+-0.352358 0.0106962 0.0930097
+-0.365061 -0.00241845 0.0889268
+-0.371378 -0.0116853 0.084559
+-0.381501 -0.0133038 0.0778076
+-0.38504 -0.0253475 0.0687138
+-0.265221 0.135804 -0.0227982
+-0.248347 0.143818 -0.0155162
+-0.229624 0.145057 -0.023862
+0.0837147 0.203074 0.114313
+0.191547 0.000613976 -0.058867
+0.179572 -0.00455891 -0.081695
+-0.0179285 0.191125 0.141825
+0.074671 0.125449 -0.115605
+0.100378 0.123257 -0.105614
+0.109501 0.148191 -0.0967001
+0.0861875 0.104971 -0.118148
+0.120268 0.113011 -0.101312
+0.2304 0.033054 -0.0627787
+-0.0175023 0.205236 0.13064
+-0.0246945 0.215588 0.118203
+-0.0402513 0.210836 0.119478
+-0.0520517 0.199692 0.126629
+-0.0520467 0.215163 0.109544
+-0.0491364 0.223281 0.0948895
+-0.0330574 0.227876 0.0919853
+-0.00155887 0.205641 0.13137
+0.00942581 0.214006 0.121922
+0.0140483 0.203393 0.132712
+0.0230254 0.188966 0.14258
+0.0283783 0.199611 0.13371
+0.0443375 0.205985 0.125045
+0.0645793 0.202014 0.122581
+0.0382707 0.218421 0.110452
+0.0377545 0.189657 0.139859
+0.0495097 0.17628 0.146073
+0.0673532 0.180327 0.139283
+0.0802708 0.165958 0.14453
+0.0765894 0.146779 0.155161
+0.0990867 0.134004 0.155061
+0.0432616 0.161058 0.155485
+0.0556193 0.145405 0.160331
+0.053992 0.124489 0.168818
+0.0245753 0.154745 0.161164
+0.0478735 0.224578 0.0958366
+0.0684131 0.222118 0.0900175
+0.078493 0.225737 0.0698939
+0.100876 0.221763 0.065965
+0.113688 0.212274 0.0826634
+0.104011 0.15383 0.144328
+0.126494 0.214189 0.0668955
+-0.0665517 0.193981 0.124882
+-0.0804722 0.198127 0.113743
+-0.074614 0.181052 0.130455
+0.0202763 0.219591 0.113828
+0.122288 0.201537 0.097016
+0.140391 0.193084 0.0979898
+0.155928 0.19652 0.0827522
+0.103478 0.207222 0.0987704
+0.158633 0.184228 0.0986656
+0.14837 0.206869 0.0681355
+0.143772 0.213494 0.0504607
+0.157944 0.211701 0.0277668
+0.137828 0.216728 0.0322073
+0.119203 0.220974 0.0239402
+0.181201 0.203466 0.0367049
+0.203994 0.196614 0.0269394
+0.108119 0.223088 0.0395942
+0.169816 0.19757 0.0691349
+0.188327 0.189312 0.06867
+0.181082 0.198755 0.0545559
+0.0880282 0.211392 0.101128
+-0.0466273 -0.132423 -0.174157
+-0.0318814 -0.139512 -0.174361
+-0.0157739 -0.129794 -0.182809
+-0.0169605 -0.147684 -0.170043
+-0.0171565 -0.156499 -0.153941
+-0.0342955 -0.157638 -0.143482
+0.00392453 -0.126602 -0.183463
+-0.0425012 -0.142934 -0.1669
+-0.0454602 -0.150295 -0.15321
+-0.0603212 -0.144284 -0.151597
+-0.0561048 -0.208349 0.107134
+-0.0466283 -0.209963 0.094217
+-0.0395136 -0.209195 0.0798734
+-0.0300619 -0.204605 0.0643616
+-0.0542184 -0.215608 0.0800121
+-0.0723947 -0.220944 0.0794647
+-0.0398445 -0.202712 0.106677
+-0.026282 -0.201856 0.0787931
+-0.0191106 -0.195036 0.0906651
+-0.0602593 -0.21837 0.0670154
+-0.0547895 -0.214918 0.0522095
+-0.0431681 -0.209025 0.0404074
+-0.0492054 -0.207652 0.023149
+-0.0688374 -0.205485 0.0059795
+-0.0318623 -0.201338 0.021692
+-0.0250427 -0.195297 0.00476074
+-0.0783989 -0.219366 0.0917231
+-0.036841 -0.195334 0.117821
+-0.0855943 -0.223925 0.0704585
+-0.101036 -0.225034 0.0722946
+0.00797548 -0.111749 -0.188238
+0.198289 0.201344 0.0120569
+0.129707 -0.000104893 -0.143391
+0.122803 0.022642 -0.141595
+0.131204 0.0379353 -0.129884
+0.0652331 -0.0683161 0.174012
+0.169741 0.0223577 0.146053
+0.174405 0.0061381 0.13108
+0.168622 -0.00867675 0.12098
+0.153687 -0.0164829 0.131427
+0.171817 -0.0137639 0.103162
+0.160136 -0.0252577 0.0952122
+0.172869 -0.0168205 0.0851703
+0.185212 -0.00560576 0.0932736
+-0.266685 -0.165547 0.0366709
+0.275879 0.0838846 0.0745082
+-0.0937132 0.19223 0.111342
+-0.106997 0.176659 0.115898
+-0.11095 0.190463 0.10029
+-0.126606 0.187093 0.0915943
+-0.142085 0.184033 0.0817885
+-0.154062 0.170706 0.0875104
+-0.169139 0.15327 0.0921543
+-0.149253 0.156395 0.103116
+-0.186962 0.154848 0.0764578
+-0.122296 0.176425 0.105839
+-0.1187 0.164061 0.117324
+-0.205086 0.146206 0.0712823
+-0.220448 0.149222 0.0532732
+-0.211079 0.163023 0.0370243
+-0.219494 0.161558 0.018494
+-0.237511 0.153161 0.0125252
+-0.249528 0.144798 0.0267993
+-0.246387 0.136378 0.0460156
+-0.264557 0.139198 0.0171736
+-0.247159 0.148788 3.98313e-005
+-0.250069 0.147707 0.0134031
+0.0553132 0.17621 -0.0917443
+0.173324 0.167907 -0.0804984
+0.189782 0.175871 -0.073497
+0.190862 0.188891 -0.0568903
+0.207697 0.172638 -0.0733973
+0.224967 0.17092 -0.0713981
+0.215941 0.158803 -0.0801729
+-0.484824 -0.0268311 -0.0434494
+-0.477138 -0.0315994 -0.0330875
+-0.468515 -0.036879 -0.0234582
+-0.460896 -0.0360471 -0.0108418
+-0.450483 -0.0395823 -0.000216449
+-0.437022 -0.0355125 0.0184559
+-0.00684894 0.10379 -0.163319
+-0.186018 0.123881 0.102748
+0.0137157 -0.136103 -0.175143
+0.0264414 -0.134708 -0.167362
+0.0388561 -0.134535 -0.153306
+0.0395308 -0.121548 -0.167842
+0.0111569 -0.146653 -0.164895
+-0.000750878 -0.15146 -0.163697
+0.0111868 0.056744 -0.182724
+0.233115 0.173929 0.0533539
+-0.141639 -0.218195 0.0382904
+-0.122474 -0.219682 0.0400409
+-0.105452 -0.218012 0.0353912
+0.240064 0.0618588 -0.0784205
+0.230353 0.0462174 -0.0761554
+0.223175 0.063332 -0.085684
+0.218898 0.0326369 -0.0763137
+0.204732 0.0311913 -0.0883317
+0.189904 0.0246699 -0.0976091
+0.183997 0.00806982 -0.0937799
+0.207877 0.0225787 -0.0765582
+0.200938 0.0102395 -0.0682652
+-0.299431 0.0820184 -0.065027
+-0.313951 0.0776969 -0.0671838
+-0.320939 0.064638 -0.0719511
+-0.312324 0.0502342 -0.0734035
+-0.320949 0.0344246 -0.076761
+-0.295312 0.0513818 -0.0707959
+-0.173537 -0.106557 -0.064935
+-0.454017 -0.00539478 0.0124898
+0.211161 0.0175973 -0.0649929
+0.00655894 -0.169937 0.106317
+0.0243113 -0.154347 0.104277
+0.041125 -0.137241 0.106784
+0.0330204 -0.148498 0.0826849
+0.00773306 -0.159576 0.126719
+0.0138807 -0.145963 0.139351
+0.0049516 -0.139462 0.152362
+-0.0126907 -0.13936 0.15942
+0.0184802 -0.126785 0.155856
+0.0241502 -0.111242 0.16547
+0.0286893 -0.13331 0.141105
+0.0429057 -0.130217 0.126263
+0.0491251 -0.118004 0.139231
+0.0120257 0.0695559 -0.176558
+0.0834693 0.217201 -0.0170596
+0.0421476 0.21581 -0.0251167
+0.178941 0.01749 -0.103193
+0.169163 0.0113257 -0.111613
+0.153011 -0.00104932 -0.122574
+0.157342 0.0197994 -0.120215
+0.148636 0.0375252 -0.120607
+0.154906 -0.0180664 -0.109226
+0.162003 -0.0193091 -0.0878111
+0.176321 0.00628563 -0.10222
+0.174375 -0.00362072 -0.0937611
+0.16558 -0.012255 -0.0982933
+0.169898 -0.0118878 -0.0883476
+0.167427 -0.0161977 -0.074634
+0.172483 -0.0136953 -0.0550104
+0.158573 -0.0228562 -0.038755
+0.178268 -0.0090176 -0.06772
+0.143631 -0.0308124 -0.113389
+0.131029 -0.0471009 -0.104742
+0.11914 -0.0584212 -0.0887558
+-0.477443 0.00727968 -0.0926757
+0.200075 0.181902 -0.0654523
+0.205429 0.188398 -0.0529035
+0.220716 0.187539 -0.044447
+0.233543 0.180658 -0.0547301
+-0.0393246 -0.0266683 0.189718
+-0.274121 0.00556367 0.133906
+-0.25281 -0.00309231 0.143108
+-0.226366 0.00430057 0.152124
+-0.204148 0.0203161 0.151609
+-0.180106 0.036869 0.151163
+-0.155849 0.0570393 0.149975
+-0.129284 0.0463575 0.162585
+-0.127312 0.0167731 0.173798
+-0.126864 -0.0148933 0.175188
+-0.0962677 -0.0321701 0.176177
+-0.177678 0.0113344 0.163511
+-0.152021 0.000899388 0.173289
+-0.174367 -0.0169592 0.162846
+-0.24422 -0.0216099 0.139744
+-0.22626 -0.0160268 0.148336
+-0.203186 -0.0300506 0.148803
+0.312169 0.0910623 -0.0119132
+0.304722 0.0829976 -0.0216259
+0.310662 0.0902705 0.000616455
+0.306807 0.0903994 0.0183261
+0.303433 0.081803 0.00726197
+0.296626 0.0724205 0.000646811
+0.294731 0.0699752 -0.0133786
+0.288401 0.0639221 -0.022482
+0.258399 0.0760553 0.093872
+0.245932 0.0798803 0.10576
+0.253728 0.0939036 0.0994145
+0.260498 0.107331 0.0902369
+0.273431 0.103919 0.079149
+0.236537 0.0670613 0.112172
+0.234206 0.0477322 0.10414
+0.233527 0.0825384 0.115799
+0.225279 0.0977603 0.119174
+0.216992 0.0848531 0.127225
+0.199159 0.0862903 0.136279
+0.207765 0.0697112 0.133124
+0.190919 0.105206 0.134975
+0.17057 0.110355 0.141543
+0.156373 0.132508 0.137569
+0.211033 0.0519714 0.128015
+0.22651 0.0551342 0.116173
+0.188768 0.0725313 0.143551
+0.170666 0.0742871 0.151443
+0.161818 0.0927778 0.150652
+0.180456 0.0567016 0.14836
+0.183634 0.0390989 0.143398
+0.198485 0.0390369 0.1327
+0.161023 0.0597683 0.157114
+0.217859 0.111881 0.119231
+0.201322 0.116662 0.125298
+0.23116 0.109891 0.11167
+0.277718 0.11536 0.0697436
+0.286702 0.114003 0.0593743
+0.292923 0.11704 0.0501709
+0.293917 0.127809 0.0408313
+0.299874 0.11642 0.0392668
+0.284942 0.138435 0.0451881
+0.271506 0.139451 0.0590869
+0.262203 0.153376 0.0536744
+0.265562 0.158963 0.041355
+0.275182 0.15707 0.0302267
+-0.0745698 0.208428 0.105718
+-0.285535 0.0857761 -0.0627339
+-0.27473 0.0802749 -0.0641795
+-0.271141 0.0683627 -0.0662598
+-0.258023 0.0567015 -0.0678506
+-0.280992 0.0567367 -0.0686977
+-0.244837 0.0669674 -0.0668579
+-0.237033 0.0513323 -0.0702584
+-0.234541 0.0333375 -0.0709573
+-0.245976 0.0835589 -0.0633754
+-0.254841 0.0992484 -0.0576819
+-0.241877 0.116915 -0.048652
+-0.18667 0.140069 0.0904389
+0.295036 0.0935836 -0.0627684
+-0.00926267 0.0174249 0.196426
+-0.00327775 -0.00841017 0.198497
+0.0150689 0.00949618 0.198029
+0.0405031 0.00510915 0.19529
+0.0594433 0.0118965 0.19127
+-0.481953 -0.0263701 -0.036238
+-0.480202 -0.0205423 -0.0285568
+-0.476382 -0.0147766 -0.0212279
+-0.468767 -0.0190157 -0.00845524
+-0.472476 -0.00650871 -0.014068
+-0.472283 0.00563921 -0.0183851
+-0.462142 0.0149039 -0.0113607
+-0.449381 0.0227484 0.00012384
+-0.457305 0.00865369 0.0030441
+-0.481587 -0.0110502 -0.031619
+-0.482475 0.000430245 -0.0332655
+-0.480347 0.00948865 -0.041986
+-0.472473 0.0222522 -0.0436525
+-0.466776 0.0245091 -0.0320448
+-0.485126 0.00627116 -0.0502161
+-0.0370303 -0.109778 0.171268
+-0.0368659 -0.0892024 0.177455
+-0.0606401 -0.0718449 0.175116
+-0.0188292 -0.0791852 0.184518
+-0.066591 -0.0431507 0.180288
+0.00121365 -0.0861995 0.184132
+0.0158369 -0.0735019 0.187985
+0.000163277 -0.103552 0.176731
+0.0940848 -0.08384 -0.126867
+0.0912612 -0.0790954 0.139096
+0.0975527 -0.0787616 0.119196
+0.0915109 -0.0667976 0.154999
+0.106504 -0.0546283 0.14974
+0.112281 -0.0325408 0.160141
+0.120223 -0.0509425 0.134623
+0.120697 -0.057035 0.114118
+0.130314 -0.0372564 0.138035
+0.118329 -0.0419346 0.148167
+0.078692 -0.0689693 0.164835
+0.0765218 -0.0531401 0.175006
+0.0796376 -0.0342708 0.179856
+0.0640642 -0.0234706 0.18856
+0.0539097 -0.00800927 0.192643
+0.0685062 -0.0790748 0.165097
+0.0660068 -0.0910629 0.155978
+0.0750122 -0.091819 0.144585
+0.053982 -0.0979508 0.159145
+0.038619 -0.102088 0.165471
+0.0480016 -0.109575 0.151663
+0.0334277 -0.0895897 0.176126
+0.0439807 -0.0770085 0.179527
+0.0516381 -0.060695 0.184191
+0.0787845 -0.0183292 0.183534
+0.0876456 -0.00406453 0.181976
+0.0800526 0.0145518 0.185934
+-0.0308081 0.0885405 -0.174657
+0.155665 0.212315 0.00724246
+0.141239 0.21351 -0.00570578
+0.292693 0.1093 0.0546663
+0.29272 0.0988077 0.053019
+0.297462 0.0958888 0.0427528
+0.053781 -0.0859189 -0.176765
+0.0684574 -0.0710688 -0.172827
+0.0839519 -0.0559622 -0.167953
+0.0516696 -0.0668944 -0.183321
+0.0985799 -0.0431579 -0.161885
+0.111766 -0.0468476 -0.147448
+0.109403 -0.0636708 -0.132674
+-0.294316 0.110657 -0.0472239
+-0.283128 0.119926 -0.0388336
+-0.262748 0.12402 -0.0397682
+-0.288362 0.12687 -0.0266383
+-0.302183 0.124187 -0.0189976
+-0.318458 0.117372 -0.0209314
+-0.327255 0.11023 -0.00445948
+-0.316206 0.112645 0.00853129
+-0.298838 0.121181 0.0130465
+-0.314127 0.115203 -0.0350952
+-0.291031 0.128808 0.00166
+-0.277171 0.135902 -0.000114267
+-0.273248 0.136459 -0.0125396
+0.097695 -0.0737844 -0.140551
+0.0827347 -0.0895802 -0.145093
+0.0868385 -0.0748488 -0.155466
+0.0679703 -0.106364 -0.147753
+0.0603532 -0.120556 -0.134077
+0.0588304 -0.104422 -0.163233
+-0.322674 0.00868483 0.110672
+-0.307263 0.00676205 0.118571
+-0.308005 -0.00861718 0.119522
+-0.293581 0.0114665 0.12301
+-0.299329 0.0239726 0.112823
+-0.286154 0.0398423 0.107825
+-0.26321 0.0378974 0.119208
+-0.13457 0.121943 -0.106906
+-0.139297 0.134665 -0.0899912
+-0.160382 0.130812 -0.0746394
+-0.121714 0.150283 -0.0872128
+-0.163756 0.148893 -0.0525948
+-0.109282 0.166513 -0.0740295
+-0.128648 0.16718 -0.0552288
+-0.167255 0.113836 -0.0834247
+-0.394393 -0.0186811 0.0656127
+-0.227174 0.111419 0.0862083
+-0.296227 0.127846 -0.00917536
+-0.293367 0.128193 -0.0184368
+-0.238147 -0.00559741 0.148474
+-0.239063 0.00917146 0.145641
+-0.224146 0.0262704 0.141961
+-0.250836 0.00914945 0.141379
+-0.262479 0.0186069 0.132247
+-0.262323 0.00421726 0.138666
+-0.263219 -0.00847911 0.138113
+-0.256828 -0.0198991 0.136053
+-0.263054 -0.0300816 0.128267
+-0.276828 -0.027548 0.124217
+-0.275008 -0.00693954 0.134137
+-0.281459 -0.0163763 0.128391
+-0.259073 0.112394 -0.0498989
+-0.320782 0.109631 -0.0425131
+-0.330688 0.10691 -0.0433542
+-0.339343 0.102362 -0.0465497
+-0.348883 0.0986778 -0.0481839
+-0.358069 0.093759 -0.051441
+-0.136873 -0.0120468 -0.151363
+-0.141221 -0.0340507 -0.146516
+-0.128382 -0.0309686 -0.160513
+-0.115237 -0.0338872 -0.171647
+-0.10415 -0.0221273 -0.177401
+-0.0925264 -0.00544945 -0.180794
+-0.106711 -0.006873 -0.173788
+-0.119908 0.000277304 -0.164218
+-0.114456 0.0181659 -0.165215
+-0.0825173 0.0101783 -0.18266
+-0.118672 -0.048041 -0.168781
+-0.121553 -0.0618819 -0.163771
+-0.110894 -0.0624358 -0.172199
+-0.39573 0.0793655 -0.0497941
+-0.397364 0.075223 -0.0621397
+-0.389269 0.0806101 -0.0580425
+-0.379676 0.0841545 -0.0574472
+-0.372813 0.0884948 -0.0523992
+-0.373143 0.0902016 -0.0428662
+-0.405287 0.0698332 -0.0665904
+-0.0509752 -0.0107302 -0.193683
+0.0684379 0.0587582 0.184268
+0.0950817 0.0700393 0.176254
+0.111866 0.0484329 0.175538
+0.0135438 0.23103 0.0887354
+-0.148114 -0.221117 0.051071
+-0.159772 -0.21833 0.0562015
+-0.171141 -0.215827 0.049958
+-0.172124 -0.214009 0.0349009
+-0.183868 -0.211625 0.0333021
+-0.48067 0.00147563 -0.0941531
+-0.48325 0.00511008 -0.0871288
+-0.478886 -0.00670064 -0.0953813
+-0.488509 -0.000247977 -0.083197
+-0.490106 0.000173262 -0.0745092
+-0.486384 0.0077296 -0.0687938
+-0.490839 -0.00729847 -0.0675525
+-0.487573 -0.00769379 -0.0851859
+0.305413 0.115443 0.0310865
+0.303067 0.109776 0.0361991
+0.301528 0.100881 0.0364219
+0.304542 0.0950282 0.0278378
+0.308585 0.101812 0.0193478
+0.30202 0.0858542 0.0255498
+0.294651 0.0776239 0.0308248
+-0.13973 -0.120127 -0.0894371
+0.0774121 -0.101688 -0.13441
+0.08428 -0.0964758 -0.11861
+0.0757635 -0.104018 -0.0952178
+0.0953267 -0.0839993 -0.106017
+0.00293945 0.190013 -0.0774414
+-0.0158096 0.195067 -0.0665582
+-0.0158703 0.205666 -0.0441793
+0.0913039 -0.0749531 -0.0365181
+-0.243097 0.150078 0.0206614
+-0.23361 0.154351 0.0222242
+-0.0716637 -0.194344 0.123031
+-0.0862633 -0.191679 0.120669
+-0.079351 -0.199669 0.116734
+0.0905437 -0.0720831 -0.0082291
+-0.442375 -0.0805602 -0.0729391
+-0.434684 -0.074604 -0.0854765
+-0.437054 -0.0869675 -0.0607547
+-0.42771 -0.0945168 -0.0528257
+-0.418352 -0.0983054 -0.0400735
+-0.423279 -0.0966001 -0.0655131
+-0.408659 -0.101487 -0.0697238
+-0.416431 -0.0909254 -0.0763075
+-0.411779 -0.0788541 -0.0841388
+-0.40598 -0.0655492 -0.0884204
+-0.416055 -0.0491274 -0.0936407
+-0.400038 -0.0506823 -0.0905303
+-0.381421 -0.0453955 -0.0869276
+-0.402243 -0.110281 -0.0579508
+-0.390216 -0.11874 -0.0501108
+-0.383494 -0.123101 -0.0364651
+-0.386125 -0.115131 -0.0203428
+-0.369628 -0.130374 -0.0274223
+-0.401456 -0.102315 -0.0198679
+-0.394171 -0.100274 -0.0727645
+-0.374041 -0.0974336 -0.0718592
+-0.385665 -0.0860135 -0.0779307
+-0.359712 -0.131969 -0.0141493
+-0.355735 -0.121067 0.00589521
+-0.407755 -0.107591 -0.0445125
+-0.442105 -0.0634097 -0.0906833
+-0.422825 -0.0823939 -0.081556
+-0.45077 -0.0554598 -0.0929264
+-0.451195 -0.043705 -0.0972223
+-0.437782 -0.0340365 -0.100162
+-0.433036 -0.0155961 -0.100753
+-0.454922 -0.0324231 -0.100841
+-0.458814 -0.0492048 -0.0932669
+-0.44563 -0.00538073 -0.102567
+-0.445825 -0.028368 -0.102242
+0.187567 0.0115241 0.121869
+0.199923 0.0237015 0.120686
+0.209608 0.0257357 0.11161
+0.217483 0.0222293 0.0938709
+0.22097 0.0324773 0.104357
+0.150551 0.0108782 -0.126188
+0.141789 0.000110033 -0.133433
+0.159708 0.00816767 -0.119042
+0.161309 -0.00244862 -0.114514
+0.154667 -0.0107676 -0.117543
+0.147178 -0.0103624 -0.12647
+0.148617 -0.0204088 -0.118466
+0.138524 -0.011451 -0.134851
+0.128907 -0.0262587 -0.139176
+0.125896 -0.0139325 -0.146588
+0.113147 -0.000537483 -0.155555
+0.103133 0.0164001 -0.156394
+0.104298 -0.0149856 -0.16309
+0.12017 -0.0355985 -0.145343
+0.110453 -0.0355441 -0.154722
+-0.0440265 0.0567607 -0.185688
+0.279992 0.165929 -0.0437039
+-0.215359 -0.201377 0.036184
+-0.223693 -0.188967 0.0478234
+-0.22968 -0.165617 0.0632744
+-0.243184 -0.140528 0.0722798
+-0.228035 -0.194952 0.0356759
+-0.238496 -0.19363 0.0236924
+-0.254124 -0.186606 0.0200977
+-0.257716 -0.185407 0.00294716
+-0.272871 -0.179363 -0.00179042
+-0.287583 -0.170182 0.0102401
+-0.244128 -0.182217 -0.00440503
+-0.229531 -0.186515 -5.59474e-005
+-0.220475 -0.194692 0.00897812
+-0.262511 -0.176872 0.0277986
+-0.253263 -0.17311 0.0399939
+-0.242546 -0.168318 0.0523859
+-0.257115 -0.159426 0.0493283
+-0.275877 -0.169308 0.0245016
+-0.245871 -0.15378 0.0613383
+-0.251727 -0.181928 0.0313647
+-0.212684 -0.200277 0.0170569
+-0.203523 -0.203845 0.0214033
+-0.192659 -0.203121 0.0182344
+-0.188052 -0.194159 0.0076752
+-0.212268 -0.20365 0.0267033
+-0.231299 -0.195988 0.0142458
+-0.196414 -0.206554 0.0248662
+-0.19286 -0.209888 0.0326474
+-0.198432 -0.206747 0.0442575
+-0.285561 -0.174648 -0.00239336
+-0.297363 -0.168082 -0.0129349
+-0.315441 -0.160352 -0.0156542
+-0.322426 -0.150566 0.00154516
+-0.27946 -0.176559 0.00652693
+-0.329078 -0.152562 -0.0128228
+-0.341452 -0.147331 -0.0215596
+-0.339347 -0.143911 -0.0378978
+-0.325782 -0.137289 -0.0445101
+-0.316703 -0.146788 -0.0370359
+-0.315576 -0.155996 -0.0279132
+-0.352319 -0.13774 -0.042353
+-0.356277 -0.125511 -0.0545457
+-0.371876 -0.118612 -0.0595906
+-0.362453 -0.135739 -0.0357733
+-0.307601 -0.160835 -0.0228167
+-0.298673 -0.163768 -0.0213152
+-0.283795 -0.159885 -0.0252424
+-0.288901 -0.16657 -0.018883
+-0.271431 -0.163684 -0.0212057
+-0.267306 -0.173463 -0.0127247
+-0.271834 -0.180462 0.00664249
+-0.26405 -0.184059 0.0090717
+-0.256203 -0.187567 0.0114726
+-0.248391 -0.190551 0.0145387
+-0.190153 -0.206614 0.0534964
+-0.442555 0.000265523 0.0267302
+-0.430352 0.014091 0.0310689
+-0.424721 0.0323378 0.0173275
+-0.427036 -0.00112736 0.0417282
+-0.423392 -0.0164383 0.0442389
+-0.416537 0.00176833 0.051877
+0.280404 0.135093 0.054526
+0.278761 0.125803 0.0628293
+-0.3705 -0.131378 -0.0365069
+-0.0820636 -0.222416 0.0826194
+0.0171345 0.16532 -0.108665
+0.00250375 0.146528 -0.128943
+-0.00481984 0.165963 -0.110419
+-0.0228077 0.175077 -0.0985159
+-0.0463747 0.167896 -0.10472
+0.00785902 0.127897 -0.142484
+0.0187708 0.138019 -0.131449
+-0.0288613 0.144009 -0.134253
+-0.0449952 0.153536 -0.122945
+0.290283 0.121714 -0.0740144
+0.27745 0.12898 -0.0769404
+0.276832 0.144257 -0.0755745
+0.263655 0.133985 -0.0803847
+0.243962 0.140125 -0.0819406
+0.251818 0.124002 -0.0820614
+0.279041 0.112623 -0.0763696
+0.271158 0.153665 -0.0709006
+0.257921 0.162187 -0.0701532
+0.269864 0.163795 -0.0626536
+0.248208 0.153711 -0.0775922
+0.259474 0.169139 -0.0606852
+0.254806 0.17483 -0.0498519
+0.263115 0.174352 -0.0393777
+0.242498 0.163999 -0.0735165
+0.286047 0.147304 -0.0690663
+0.290117 0.152194 -0.0598459
+0.294561 0.14326 -0.0673603
+0.30189 0.134539 -0.0645447
+0.307831 0.128351 -0.0595143
+0.309435 0.118425 -0.0601117
+0.300896 0.111367 -0.0646074
+0.313574 0.124342 -0.0532657
+0.318389 0.120863 -0.046066
+0.317393 0.113308 -0.0379339
+0.320268 0.118282 -0.0307619
+0.320218 0.116784 -0.0221264
+0.319481 0.11809 -0.0104565
+0.31556 0.126648 -0.00415804
+0.316602 0.113926 -0.0450935
+0.311777 0.106867 -0.0480733
+0.308275 0.109186 -0.056207
+0.308505 0.0983412 -0.0486102
+0.303144 0.0925952 -0.0531051
+0.309601 0.0934764 -0.0417938
+0.313156 0.0959646 -0.0340974
+0.296983 0.117415 -0.0699211
+0.320685 0.122446 -0.0255896
+0.316942 0.130978 -0.0270583
+0.314854 0.133023 -0.0360942
+0.310847 0.141503 -0.0383109
+0.309318 0.141241 -0.0473689
+0.304008 0.147351 -0.0517981
+0.307045 0.147922 -0.034494
+0.298268 0.153813 -0.0368445
+0.309649 0.146194 -0.0271281
+0.305563 0.148274 -0.019965
+0.297494 0.154475 -0.0200693
+0.292877 0.15678 -0.00921317
+0.286071 0.16011 0.000622243
+0.275595 0.166073 0.0087612
+0.260224 0.174589 0.0077726
+0.250622 0.175429 0.0254095
+0.311613 0.140756 -0.0214101
+0.308518 0.145026 -0.0132653
+0.296427 0.156548 -0.0286188
+0.290888 0.159898 -0.0225221
+0.280072 0.168055 -0.0192366
+0.00514285 -0.0262337 0.19953
+0.000257161 -0.0403876 0.197028
+-0.014043 -0.051569 0.191857
+0.0198521 -0.0289458 0.199008
+-0.449915 0.00669424 0.0132997
+0.189521 0.183354 -0.0659039
+0.181083 0.188202 -0.0613191
+0.166139 0.19374 -0.0550867
+0.180401 0.19548 -0.0478811
+0.173792 0.202184 -0.0307436
+0.014203 -0.120681 -0.18353
+0.0238414 -0.113012 -0.183101
+0.0334543 -0.103091 -0.182186
+0.0330355 -0.116504 -0.176425
+0.0318734 -0.125101 -0.172367
+0.0356839 -0.130691 -0.164289
+0.0436843 -0.125824 -0.158725
+0.0488374 -0.116976 -0.16343
+-0.0735369 0.130882 -0.137882
+-0.0601936 0.143412 -0.130466
+-0.0624708 0.118584 -0.151005
+-0.041212 0.115328 -0.157398
+-0.0420797 0.100294 -0.167006
+0.186298 0.15686 0.109529
+-0.0106882 -0.179034 -0.053539
+-0.0257532 -0.183285 -0.05601
+-0.446546 -0.0201472 0.0199448
+-0.455322 -0.0183989 0.0101318
+-0.462739 -0.00927772 0.00133495
+-0.447924 -0.0304864 0.0103481
+-0.327773 -0.145984 -0.0378562
+-0.481411 0.0150496 -0.0638608
+0.320421 0.119931 -0.0165344
+0.317042 0.125637 -0.0119931
+0.318454 0.112192 -0.0157929
+0.0929399 0.159812 -0.0958943
+0.110625 0.166917 -0.0892709
+0.124473 0.171136 -0.0845853
+0.136186 0.183251 -0.0742106
+0.13866 0.171692 -0.0823798
+0.148566 0.160826 -0.0859647
+0.142872 0.144712 -0.0905597
+0.160795 0.122901 -0.0906349
+0.126088 0.145302 -0.0936953
+0.115879 0.130871 -0.0990572
+0.124327 0.191494 -0.0663769
+0.106391 0.192491 -0.0670656
+0.0883872 0.188262 -0.0745766
+0.0719507 0.196806 -0.0642255
+0.108844 0.201665 -0.051653
+0.0984999 0.177737 -0.0836605
+-0.12026 -0.223902 0.053195
+-0.109446 -0.224931 0.0565831
+-0.111541 -0.221619 0.0451678
+-0.242046 -0.187623 0.0325776
+-0.326234 0.0938195 0.0291499
+-0.0102421 -0.165045 0.13696
+-0.373588 0.0100989 0.0801669
+0.00996937 -0.0129898 0.200095
+-0.463877 -0.0324637 -0.100365
+-0.460224 -0.0241828 -0.10134
+-0.459739 -0.0142811 -0.101377
+0.133208 -0.0412449 -0.119383
+0.123352 -0.0526797 -0.120194
+0.132637 -0.0343741 -0.130094
+-0.164742 -0.122178 -0.0595052
+0.044515 -0.137297 -0.111748
+-0.0612791 -0.162566 -0.109688
+-0.0710129 -0.164834 -0.0958253
+-0.0735533 -0.172186 -0.0768106
+-0.0876452 -0.16161 -0.0867929
+-0.476659 -0.0271538 -0.0964533
+-0.474429 -0.0190092 -0.0973528
+0.0739718 -0.0242315 -0.180703
+0.162304 -0.00560673 0.133058
+-0.479103 -0.0320985 -0.0940027
+-0.481417 -0.0340858 -0.0884057
+-0.486127 -0.0306435 -0.0860583
+-0.486018 -0.0242883 -0.0872577
+-0.482104 -0.0219439 -0.0922364
+-0.483886 -0.0355832 -0.0827419
+-0.480962 -0.0402868 -0.0788522
+-0.488368 -0.0267363 -0.0830009
+-0.489744 -0.0252165 -0.0755797
+-0.49086 -0.0244246 -0.0695498
+-0.489653 -0.0247292 -0.0607899
+-0.490316 -0.0194905 -0.0649848
+-0.490364 -0.0284268 -0.0721027
+-0.486281 -0.0339622 -0.0727452
+-0.484931 -0.0357981 -0.0657032
+-0.480417 -0.0422452 -0.0650589
+-0.486623 -0.0314942 -0.0598312
+-0.47196 -0.030399 -0.0984255
+-0.0669025 0.00647931 0.189048
+-0.00259196 0.0546073 0.192242
+-0.0285477 0.0708007 0.184734
+-0.0057345 0.0374844 0.19471
+-0.0268013 0.159534 0.157979
+-0.0096283 0.148792 0.164972
+-0.0781263 -0.119784 -0.166335
+-0.0693606 -0.129505 -0.164109
+-0.0688752 -0.121624 -0.171204
+-0.301484 0.103619 -0.053148
+-0.293096 0.10467 -0.0530655
+-0.281535 0.101848 -0.0551465
+-0.285102 0.10937 -0.0500982
+-0.270178 0.101819 -0.0557902
+0.146413 0.215601 0.0217432
+-0.0558768 -0.200581 -0.0058347
+-0.121903 -0.222398 0.0753827
+-0.131484 -0.222972 0.0672943
+-0.366143 0.0142557 0.0832656
+-0.0113622 -0.139899 -0.177586
+-0.461084 -0.0182082 0.00293327
+-0.464563 -0.0224324 -0.00273456
+-0.465956 -0.0279422 -0.0104269
+-0.0796578 0.1856 -0.0608717
+-0.408982 -0.00209341 0.057713
+-0.402494 -0.00941199 0.0623365
+-0.403353 -0.0196112 0.058583
+-0.405762 -0.0286267 0.0515338
+-0.405618 -0.0398167 0.0440031
+-0.404736 -0.0529218 0.0334363
+-0.409831 -0.0636201 0.0189125
+-0.402566 -0.0785474 0.0097687
+-0.384311 -0.0886192 0.0159024
+-0.399923 0.000206371 0.0648148
+-0.395259 0.0101254 0.0639619
+-0.399409 0.0198184 0.0541902
+-0.401015 0.0317499 0.0417548
+-0.386013 0.0426196 0.0409759
+-0.390555 0.0202045 0.0613875
+-0.376091 0.028226 0.0648869
+-0.359969 0.0354058 0.0689777
+-0.390478 0.00307787 0.0712524
+-0.414992 0.0273589 0.0326168
+-0.0822249 -0.219415 0.0436093
+0.320548 0.121146 -0.0209076
+-0.0307118 0.209112 0.125044
+0.244694 0.0716962 -0.0789439
+-0.322786 -0.150478 -0.0340602
+-0.142918 -0.221898 0.0595833
+-0.141055 -0.219727 0.0695305
+-0.148384 -0.213823 0.0747687
+-0.140352 -0.208727 0.0848566
+-0.159369 -0.210758 0.0714338
+-0.12697 -0.210548 0.09034
+-0.123362 -0.217457 0.0840045
+0.291319 0.148647 -0.0653
+0.285923 0.152334 -0.064874
+0.280206 0.15113 -0.0687143
+-0.011495 -0.128161 0.165796
+-0.0104489 -0.114268 0.172787
+-0.456328 0.00238289 0.00911999
+-0.462716 0.00053365 0.00188537
+-0.452719 0.000801432 0.0144475
+-0.447711 -0.00566448 0.020379
+-0.447072 0.00296473 0.0202268
+-0.439977 0.00940057 0.0233511
+-0.441315 -0.0101266 0.0273906
+-0.434861 -0.0036457 0.0339979
+-0.435623 0.00541616 0.0309225
+-0.397014 -0.107359 -0.0661138
+-0.394501 -0.112676 -0.0601278
+-0.39739 -0.114494 -0.052232
+-0.386945 -0.116865 -0.0581355
+-0.379673 -0.123432 -0.0518067
+-0.379349 -0.114175 -0.0625035
+-0.368845 -0.10815 -0.0658517
+-0.359899 -0.0973536 -0.069139
+-0.379424 -0.105627 -0.0684125
+-0.345533 -0.0999729 -0.0649614
+-0.340448 -0.115776 -0.057915
+-0.348994 -0.0845838 -0.0704153
+-0.332742 -0.0656731 -0.0705226
+-0.350994 -0.0664547 -0.0754087
+-0.341818 -0.0470471 -0.0761957
+-0.351962 -0.0231594 -0.0823818
+-0.39552 -0.115847 -0.0438253
+-0.370874 -0.124971 -0.0532356
+-0.363364 -0.129762 -0.049668
+-0.33009 -0.123253 -0.052732
+-0.316778 -0.125001 -0.0493635
+-0.302639 -0.123107 -0.0473696
+-0.287039 -0.124882 -0.0436812
+-0.273759 -0.109 -0.046197
+-0.270469 -0.126021 -0.0397525
+-0.254738 -0.113942 -0.0416881
+-0.278345 -0.0886894 -0.0527072
+-0.266377 -0.0678443 -0.0560345
+-0.281535 -0.0441861 -0.0626876
+-0.300226 -0.0912717 -0.0564075
+-0.312467 -0.0733818 -0.0634877
+-0.263538 -0.0493458 -0.0593802
+-0.384834 -0.123009 -0.0449511
+-0.378696 -0.127011 -0.0416145
+-0.37682 -0.126976 -0.0335983
+-0.38166 -0.12148 -0.0267217
+-0.45038 -0.0749298 -0.0724473
+-0.45796 -0.0684169 -0.0757915
+-0.463668 -0.0613132 -0.0789114
+-0.470167 -0.0560323 -0.0735728
+-0.456908 -0.0647581 -0.0842542
+-0.468123 -0.0541536 -0.0821805
+-0.355365 0.0854034 -0.0639831
+-0.345495 0.0784018 -0.0686681
+-0.345566 0.0642705 -0.0754244
+-0.345752 0.0869151 -0.0639626
+-0.340409 0.0921156 -0.0595821
+-0.320658 -0.156155 -0.00735903
+0.0310287 -0.0498547 0.193641
+0.0198894 -0.141937 -0.165459
+-0.228047 -0.00611597 0.151539
+-0.21379 -0.0101939 0.154678
+-0.21124 0.00524665 0.156381
+-0.196797 0.0093087 0.159224
+-0.188163 0.0229398 0.155281
+-0.186748 -0.000713447 0.1644
+-0.171469 -0.00241387 0.168893
+-0.15893 -0.0134394 0.168193
+-0.219561 -0.00229464 0.154558
+-0.211896 -0.00271869 0.156825
+0.241886 0.140309 0.0860302
+-0.254378 -0.180608 -0.00572631
+-0.0256362 -0.192795 -0.0108483
+-0.0213145 -0.188013 -0.0254959
+-0.00679366 -0.180573 -0.0278423
+0.265879 0.0793336 0.0857758
+-0.178895 -0.21406 0.0388483
+-0.1723 -0.215898 0.0423547
+-0.244753 0.00128651 0.146437
+-0.237347 0.0022348 0.148985
+0.25157 0.0457267 -0.0527654
+-0.475261 -0.00121759 -0.0185581
+-0.477838 0.00416433 -0.0259061
+-0.490474 0.00258014 -0.0683864
+0.196069 0.0347717 -0.0957831
+0.201615 0.0491382 -0.0939952
+0.18803 0.0621831 -0.0972113
+0.167364 0.0756427 -0.0996243
+0.172397 0.0998559 -0.0921219
+0.187663 0.033633 -0.100368
+0.174757 0.0400994 -0.106834
+0.197053 0.0262793 -0.0913859
+0.202678 0.0226549 -0.0845706
+0.210031 0.0556486 -0.0907929
+0.192507 0.0527045 -0.097814
+0.180216 0.0529828 -0.102021
+0.175563 0.0652764 -0.100425
+0.162926 0.0640423 -0.104369
+0.142127 0.0624953 -0.113056
+0.151939 0.0756284 -0.103791
+0.132789 0.0797865 -0.10854
+-0.348441 -0.142527 -0.0348869
+-0.342301 -0.14637 -0.0301304
+0.148711 0.183289 -0.0726251
+0.161656 0.181236 -0.0729202
+0.163216 0.188442 -0.064501
+-0.238556 -0.0310506 -0.0635487
+-0.00291307 0.0352651 -0.19427
+0.0108075 0.0413253 -0.188804
+-0.00553357 0.0203276 -0.197456
+-0.0214647 0.0159016 -0.196991
+0.0071277 0.0113207 -0.197326
+0.0294013 -0.148697 -0.119916
+0.0138806 -0.159193 -0.109312
+0.0172829 -0.159017 -0.0887735
+0.0277653 -0.147799 -0.13545
+0.00273327 -0.1618 -0.121337
+-0.011111 -0.168484 -0.106465
+-0.0235051 -0.167364 -0.116293
+0.0242164 -0.154659 -0.0691485
+-0.0143519 -0.172252 -0.092658
+-0.010652 0.0604636 -0.187016
+0.100536 0.224554 0.0517985
+0.114136 0.219745 0.0578874
+0.091313 0.225287 0.0595263
+0.0797026 0.227767 0.0581816
+0.0723327 0.229027 0.0453777
+0.0829269 0.227441 0.030913
+0.0668447 0.229241 0.0632039
+0.114338 0.216906 0.0705535
+0.103966 0.217771 0.0772861
+0.27314 0.0904289 -0.0742729
+0.27865 0.100969 -0.0740164
+-0.0318201 -0.172097 0.140491
+-0.150995 0.191569 0.0449039
+-0.162861 0.18261 0.0597716
+-0.181673 0.17237 0.0564074
+-0.190646 0.173251 0.0363386
+-0.133419 0.199622 0.048713
+0.30869 0.146192 -0.039147
+0.310112 0.144647 -0.0351315
+0.312128 0.140484 -0.0337784
+0.312846 0.137243 -0.0372711
+0.313626 0.133 -0.043947
+0.311506 0.138025 -0.0426565
+0.314828 0.128729 -0.0486676
+-0.39891 0.0356498 -0.091534
+-0.410483 0.0346101 -0.0927351
+-0.394039 0.04254 -0.0886511
+-0.0878188 0.211246 0.0894536
+-0.102222 0.20704 0.0831881
+-0.105324 0.210562 0.066741
+-0.119615 0.203259 0.0673147
+-0.0114042 -0.153427 -0.162654
+-0.0216924 -0.152759 -0.162414
+-0.0276497 -0.155233 -0.154041
+-0.0229191 -0.159385 -0.144364
+-0.028222 -0.162884 -0.130294
+-0.00911545 -0.159755 -0.143127
+0.00381773 -0.158174 -0.138235
+0.0158629 -0.151971 -0.145374
+-0.043042 -0.159811 -0.131283
+-0.0578498 -0.157583 -0.12483
+-0.0730611 -0.153821 -0.118257
+-0.0918654 -0.142271 -0.119321
+-0.0865458 0.151799 -0.110721
+-0.0747559 0.164903 -0.0996003
+-0.0877131 0.169611 -0.0847258
+-0.104298 0.14448 -0.108615
+-0.111502 0.130777 -0.117599
+-0.290598 0.122665 -0.0344017
+-0.283048 0.125286 -0.0326538
+-0.2771 0.131561 -0.0234897
+-0.273262 0.124078 -0.0371466
+-0.27542 0.117855 -0.0440148
+-0.186493 0.17034 -0.00116882
+-0.17177 0.166823 -0.0203714
+-0.180684 0.153502 -0.0354437
+-0.205992 0.162616 -0.00205048
+-0.198324 0.14709 -0.0338166
+-0.00592029 0.100685 0.181441
+-0.0167332 0.118964 0.174829
+0.0159467 0.103079 0.18061
+0.0284124 0.0849417 0.184432
+-0.442518 -0.0709443 -0.0863691
+-0.448856 -0.0662201 -0.0874812
+-0.134667 0.000943154 0.177011
+-0.00310122 0.234913 0.0628362
+-0.46231 -0.0586249 -0.0404296
+0.227538 0.0369226 -0.0721269
+0.236821 0.0423656 -0.0668035
+0.240605 0.0371232 -0.0549899
+0.229841 0.0265624 -0.0470128
+-0.463607 0.0286461 -0.0857262
+-0.456726 0.027279 -0.0906909
+-0.469667 0.0222263 -0.0848045
+-0.450827 0.0237483 -0.0946958
+0.216321 0.138978 0.105626
+0.21636 0.176018 -0.068996
+0.224724 0.177449 -0.0640927
+0.23515 0.172803 -0.0661681
+0.260778 0.097572 0.0930733
+0.253697 0.103791 0.0969488
+0.000426256 0.00615835 0.198705
+-0.0143538 0.00308408 0.196759
+-0.0259448 0.0152079 0.193781
+0.00496704 0.0199039 0.19777
+-0.0130132 0.231981 0.0855148
+-0.000488198 0.230115 0.092734
+-0.486599 0.00442738 -0.0834039
+-0.483174 0.00834215 -0.0830378
+-0.481531 0.012535 -0.0769011
+-0.479231 0.00931576 -0.0872037
+-0.479136 0.0130796 -0.0824478
+-0.474567 0.0182278 -0.0820571
+-0.488599 0.00268956 -0.0794002
+-0.489312 -0.00209098 -0.0787543
+-0.487879 0.00455228 -0.0748952
+-0.484573 0.00890921 -0.0740255
+-0.481621 0.0133511 -0.0706356
+-0.486157 0.0013552 -0.0870001
+0.074114 -0.0959285 -0.152444
+0.0788079 -0.0848095 -0.156079
+0.0776661 -0.0751258 -0.164638
+0.307857 0.12345 -0.0629205
+0.305632 0.118432 -0.0643039
+0.306506 0.11278 -0.0612846
+0.303699 0.106552 -0.0605951
+0.296786 0.103863 -0.0648658
+0.290509 0.0987054 -0.0684109
+0.30475 0.128034 -0.0643101
+0.297851 0.128749 -0.0681357
+-0.339388 0.107776 -0.0308211
+-0.331422 0.111016 -0.0225385
+0.1951 0.198278 0.038898
+0.20821 0.19181 0.0401465
+0.215469 0.182876 0.0549909
+0.218888 0.190238 0.0280165
+0.230673 0.188298 0.0132706
+-0.467314 -0.059895 -0.0588218
+-0.459485 -0.0677804 -0.0607044
+0.307129 0.146219 -0.0474896
+-0.422251 0.0576286 -0.0767091
+-0.42861 0.0541642 -0.076605
+3  671 417 1634
+3  944 65 945
+3  835 2025 874
+3  369 997 1009
+3  1597 927 1598
+3  1357 128 1355
+3  294 1255 299
+3  294 299 266
+3  146 1162 1482
+3  611 612 32
+3  313 20 312
+3  67 1314 1022
+3  1147 1148 561
+3  1122 483 119
+3  1715 1714 1718
+3  1718 832 1715
+3  1648 1117 782
+3  991 992 367
+3  546 547 554
+3  31 619 576
+3  650 985 359
+3  359 980 650
+3  969 367 992
+3  673 1700 674
+3  181 673 674
+3  660 59 307
+3  854 855 912
+3  1630 1498 589
+3  246 88 1571
+3  349 1137 1138
+3  1136 1137 349
+3  1426 917 576
+3  37 154 151
+3  643 642 72
+3  1285 530 1783
+3  12 494 495
+3  496 12 495
+3  426 202 427
+3  380 1021 1020
+3  192 846 345
+3  1526 118 1411
+3  1811 79 1810
+3  1540 1811 1810
+3  427 202 430
+3  827 337 1275
+3  1685 84 1518
+3  132 759 544
+3  444 1547 1545
+3  1762 1758 687
+3  416 373 7
+3  1326 844 665
+3  1326 665 1953
+3  378 70 376
+3  1325 1339 142
+3  469 187 468
+3  332 681 537
+3  1231 389 1232
+3  240 1651 241
+3  44 456 1463
+3  1356 1358 265
+3  2003 1412 889
+3  625 637 628
+3  184 1900 1896
+3  501 1860 1410
+3  1362 1336 666
+3  1086 1089 1256
+3  1086 1256 1084
+3  1824 1821 495
+3  1406 651 382
+3  405 81 406
+3  1412 1413 1177
+3  369 376 70
+3  239 241 153
+3  601 800 602
+3  1638 800 601
+3  336 827 108
+3  273 110 1947
+3  1188 1189 203
+3  205 1189 1188
+3  871 867 870
+3  1028 380 1020
+3  1017 1028 1020
+3  151 973 976
+3  35 589 1624
+3  1624 589 585
+3  478 187 904
+3  462 187 478
+3  170 1678 794
+3  794 1099 170
+3  958 690 537
+3  788 789 151
+3  806 182 802
+3  1022 1023 1021
+3  802 524 806
+3  558 426 50
+3  1976 1978 879
+3  882 1978 1976
+3  161 751 752
+3  1726 570 1725
+3  1458 460 186
+3  1458 186 1460
+3  1852 1844 207
+3  1852 207 1845
+3  1467 1469 490
+3  1720 1719 896
+3  668 57 667
+3  1870 1871 169
+3  1022 1314 1738
+3  98 1964 1963
+3  780 160 778
+3  57 1096 667
+3  1094 1096 57
+3  377 378 376
+3  1321 158 774
+3  282 64 703
+3  703 64 742
+3  854 1930 858
+3  1767 536 1060
+3  181 676 1656
+3  354 856 857
+3  1015 1008 1018
+3  632 637 1770
+3  1770 42 632
+3  15 1674 1672
+3  1011 1013 422
+3  572 94 668
+3  824 550 34
+3  1244 1012 358
+3  289 51 273
+3  1253 1250 56
+3  297 1250 1253
+3  1636 1178 1101
+3  1102 1178 1636
+3  1097 1098 1093
+3  404 233 178
+3  1139 63 1137
+3  766 53 767
+3  385 1964 1729
+3  479 1522 480
+3  117 479 480
+3  527 1301 1302
+3  527 1302 1303
+3  962 744 688
+3  688 744 1688
+3  611 32 613
+3  737 27 740
+3  740 27 1435
+3  357 1080 1078
+3  1502 406 81
+3  82 327 692
+3  570 1988 1258
+3  570 1258 1727
+3  853 338 873
+3  928 1614 1927
+3  1927 1926 928
+3  121 489 543
+3  386 1220 1226
+3  505 487 504
+3  1218 115 1222
+3  983 1244 358
+3  492 1244 983
+3  49 505 408
+3  879 880 337
+3  181 674 675
+3  1283 530 1285
+3  1847 1186 1846
+3  1292 725 1293
+3  1290 725 1292
+3  147 155 37
+3  158 776 773
+3  1669 1668 150
+3  1086 309 1919
+3  410 1195 1198
+3  161 162 252
+3  981 1769 980
+3  1946 1947 229
+3  214 623 1863
+3  1863 623 1974
+3  1288 726 1293
+3  1288 1293 725
+3  250 312 546
+3  1016 1017 379
+3  619 1204 1203
+3  1217 1224 1218
+3  650 980 360
+3  105 340 1761
+3  307 662 659
+3  659 662 658
+3  1452 490 1471
+3  408 406 1502
+3  1494 0 1492
+3  966 1956 1955
+3  1955 1956 1954
+3  55 245 267
+3  554 26 546
+3  1676 1677 1680
+3  446 1817 77
+3  660 307 659
+3  2007 1595 1593
+3  1209 114 1208
+3  1208 114 1214
+3  365 364 91
+3  1629 588 1630
+3  89 377 376
+3  482 22 486
+3  137 970 646
+3  26 250 546
+3  1340 2004 2005
+3  164 276 277
+3  693 323 956
+3  108 1269 1270
+3  1676 150 1677
+3  1427 286 177
+3  139 138 120
+3  1352 129 1353
+3  64 282 285
+3  343 1333 1332
+3  595 1581 1587
+3  769 768 53
+3  1499 1503 498
+3  71 434 435
+3  822 348 837
+3  823 348 822
+3  637 632 600
+3  637 600 628
+3  486 439 183
+3  1674 798 1672
+3  1624 585 1625
+3  636 597 634
+3  1392 235 1394
+3  171 1391 238
+3  1988 419 1987
+3  1687 1916 754
+3  1687 754 515
+3  372 1965 1963
+3  1054 1053 130
+3  486 22 439
+3  451 80 414
+3  414 11 451
+3  278 277 276
+3  1030 380 1028
+3  332 684 465
+3  1582 580 1498
+3  881 337 1662
+3  1728 208 558
+3  385 1728 558
+3  1897 184 1896
+3  1384 1383 1382
+3  977 15 167
+3  25 430 432
+3  432 430 202
+3  2004 1342 1344
+3  1897 220 1902
+3  224 556 1032
+3  517 1144 1143
+3  1375 324 1374
+3  1330 191 1362
+3  1343 1342 1428
+3  1343 1428 100
+3  737 740 741
+3  1975 1974 623
+3  638 1578 598
+3  1672 594 1673
+3  1673 15 1672
+3  937 938 939
+3  939 609 937
+3  967 966 315
+3  58 966 967
+3  1503 1500 84
+3  1503 84 1685
+3  507 1740 1778
+3  907 1722 898
+3  327 696 695
+3  71 420 434
+3  1330 1362 1363
+3  796 797 170
+3  1098 315 1096
+3  1097 315 1098
+3  821 849 1139
+3  1139 849 1249
+3  869 868 563
+3  1105 5 636
+3  1104 5 1105
+3  1394 234 1393
+3  386 1219 1220
+3  1207 1208 397
+3  397 1208 1212
+3  674 21 675
+3  1620 941 1628
+3  1228 1227 402
+3  114 1215 1214
+3  1214 1215 1216
+3  1034 814 216
+3  1650 88 1649
+3  1211 1234 127
+3  765 133 1150
+3  765 1150 1782
+3  364 1002 91
+3  313 656 655
+3  313 655 653
+3  984 359 985
+3  1454 458 44
+3  1154 1153 2
+3  1155 1153 1154
+3  1525 1526 497
+3  497 1526 1527
+3  808 807 524
+3  998 8 1001
+3  730 195 557
+3  1028 1017 68
+3  409 81 405
+3  845 36 1733
+3  1134 36 845
+3  1559 1558 685
+3  204 570 1726
+3  505 121 413
+3  1894 750 766
+3  1453 1454 44
+3  377 403 378
+3  1654 403 377
+3  1296 1301 527
+3  1067 1301 1296
+3  321 25 322
+3  468 463 83
+3  83 463 1754
+3  174 1156 1120
+3  1934 125 656
+3  129 1354 1353
+3  1543 78 1544
+3  226 1947 1946
+3  1372 1373 696
+3  231 1941 1942
+3  450 1529 1528
+3  1528 1508 450
+3  1404 91 1403
+3  1408 85 1409
+3  1409 85 1553
+3  744 961 1742
+3  556 223 555
+3  973 974 976
+3  173 257 749
+3  331 1345 748
+3  129 715 1359
+3  1088 425 1079
+3  740 1435 1441
+3  1006 368 1007
+3  69 1906 1010
+3  1038 1743 1034
+3  1014 993 382
+3  109 1039 1033
+3  1188 203 430
+3  625 1974 1973
+3  1332 841 842
+3  842 343 1332
+3  505 413 122
+3  1012 1011 358
+3  435 76 436
+3  1949 1950 260
+3  1918 311 1917
+3  1918 1917 306
+3  1962 266 299
+3  1086 1919 304
+3  110 228 229
+3  682 681 185
+3  1457 1828 459
+3  459 1828 1458
+3  1966 372 373
+3  1334 191 1333
+3  1692 883 1475
+3  75 1563 1058
+3  1221 383 1220
+3  994 366 991
+3  999 364 365
+3  1031 1005 367
+3  1031 367 969
+3  1404 1403 374
+3  426 208 424
+3  270 227 269
+3  1258 1987 1986
+3  1693 953 1695
+3  934 1498 588
+3  1807 1805 1806
+3  195 120 138
+3  1180 591 1179
+3  644 643 646
+3  64 743 742
+3  742 743 438
+3  303 1436 1094
+3  4 713 711
+3  291 713 710
+3  409 405 82
+3  1154 526 1155
+3  693 327 695
+3  93 1367 1365
+3  1441 1435 730
+3  413 543 3
+3  1965 98 1963
+3  325 697 407
+3  1348 320 1349
+3  1936 1937 1035
+3  1036 1937 1936
+3  517 1143 523
+3  1812 1536 453
+3  1003 363 1000
+3  721 720 240
+3  240 720 719
+3  390 1654 16
+3  781 780 159
+3  52 672 680
+3  611 610 612
+3  828 336 834
+3  2018 381 2019
+3  1002 1001 363
+3  1000 8 995
+3  1774 1776 669
+3  371 363 98
+3  1826 79 1562
+3  1562 79 1542
+3  357 1078 1012
+3  174 791 790
+3  1863 625 628
+3  63 866 1138
+3  354 857 1452
+3  1495 1496 775
+3  1538 452 1771
+3  421 275 234
+3  19 1082 1083
+3  313 312 661
+3  222 814 1034
+3  970 1171 1173
+3  1899 1942 111
+3  757 759 132
+3  407 122 410
+3  1816 1817 1815
+3  566 179 565
+3  1128 1142 1132
+3  718 264 1952
+3  1240 393 127
+3  270 228 271
+3  1430 100 1429
+3  228 1730 1731
+3  1272 1264 1265
+3  110 273 272
+3  699 698 43
+3  283 698 699
+3  779 162 161
+3  24 545 438
+3  627 213 617
+3  617 213 935
+3  1208 1214 1212
+3  1212 1214 398
+3  1221 1220 16
+3  899 897 1971
+3  1219 16 1220
+3  1548 1854 1507
+3  398 1216 1218
+3  653 652 20
+3  86 1567 1568
+3  570 204 569
+3  117 442 443
+3  414 415 81
+3  1216 1217 1218
+3  788 151 154
+3  1904 257 791
+3  1166 145 1160
+3  1167 145 1166
+3  274 227 271
+3  368 1018 1913
+3  543 542 544
+3  847 345 846
+3  551 212 550
+3  421 227 275
+3  370 1027 1910
+3  494 493 704
+3  562 284 232
+3  20 652 548
+3  999 998 364
+3  431 1088 1087
+3  867 866 63
+3  255 171 256
+3  989 994 990
+3  571 251 163
+3  1273 888 1865
+3  1865 888 1864
+3  1904 791 174
+3  715 717 716
+3  307 59 302
+3  467 1451 1450
+3  124 250 262
+3  796 795 248
+3  1356 59 1358
+3  1615 2024 1614
+3  1291 950 729
+3  729 1290 1291
+3  1747 260 1950
+3  362 1003 1000
+3  366 995 996
+3  565 179 420
+3  565 420 51
+3  1317 915 1316
+3  1017 1016 1019
+3  1017 1019 68
+3  482 479 22
+3  755 133 942
+3  1150 133 755
+3  1945 225 1944
+3  1012 201 1011
+3  1067 1309 1308
+3  950 1298 729
+3  729 1298 1297
+3  1093 1098 1436
+3  1436 1098 1094
+3  1294 952 1295
+3  1102 1636 1637
+3  89 383 1221
+3  91 1966 373
+3  741 739 738
+3  737 741 738
+3  292 715 712
+3  1767 1060 488
+3  644 646 30
+3  16 1219 1232
+3  360 986 987
+3  1431 1430 1429
+3  195 27 196
+3  557 645 136
+3  557 138 645
+3  1170 1165 1169
+3  1393 276 164
+3  636 634 168
+3  509 815 1440
+3  789 1570 87
+3  547 212 554
+3  562 277 10
+3  676 1903 184
+3  1297 1067 1296
+3  546 312 20
+3  1100 1671 799
+3  799 1671 1670
+3  1033 1034 1743
+3  1343 218 1342
+3  1342 218 1344
+3  510 1043 1042
+3  250 1434 262
+3  1854 1506 1507
+3  95 1263 1106
+3  1106 1263 887
+3  402 90 947
+3  249 1432 1433
+3  1225 1224 386
+3  944 585 943
+3  1743 1038 215
+3  1741 1773 1772
+3  1741 1772 484
+3  120 1176 1175
+3  120 1175 143
+3  1358 660 261
+3  1884 1539 1535
+3  1535 1539 453
+3  1051 536 1052
+3  147 37 148
+3  637 625 1125
+3  1125 625 1126
+3  638 633 597
+3  620 214 627
+3  44 458 456
+3  1867 1130 1872
+3  1568 259 1949
+3  717 243 716
+3  61 1113 1256
+3  60 1092 1091
+3  1961 1092 60
+3  245 716 243
+3  1539 1542 1811
+3  1811 1542 79
+3  177 286 285
+3  1880 1882 1870
+3  1236 1235 1316
+3  718 243 717
+3  370 1907 1905
+3  1871 168 634
+3  1084 1256 310
+3  1937 1938 109
+3  421 163 269
+3  64 437 743
+3  386 1224 1217
+3  752 751 172
+3  227 270 271
+3  269 163 268
+3  1 252 251
+3  162 251 252
+3  1948 128 1949
+3  842 192 345
+3  345 342 842
+3  1868 1129 1867
+3  626 1129 1868
+3  168 1871 1673
+3  154 37 770
+3  1142 978 624
+3  1128 978 1142
+3  120 143 139
+3  1321 774 1323
+3  478 1446 462
+3  1892 1894 751
+3  862 858 1929
+3  245 40 244
+3  270 1730 228
+3  180 289 1701
+3  129 1352 717
+3  673 181 680
+3  598 1578 1576
+3  1798 1541 1797
+3  1798 1797 1796
+3  160 812 813
+3  1700 180 1701
+3  356 1935 311
+3  441 442 48
+3  48 444 441
+3  1729 1003 209
+3  2001 1994 884
+3  652 314 649
+3  957 691 955
+3  836 348 869
+3  702 283 700
+3  1521 1522 118
+3  1959 301 1958
+3  54 705 1252
+3  489 541 542
+3  220 1895 1697
+3  10 282 284
+3  1091 1090 304
+3  1950 128 1357
+3  1820 198 1819
+3  1551 476 1552
+3  607 608 1024
+3  172 771 772
+3  332 537 690
+3  553 1157 606
+3  464 185 465
+3  260 262 86
+3  1716 907 886
+3  1722 907 1716
+3  1013 424 422
+3  187 462 463
+3  1517 1501 1502
+3  1476 1470 1473
+3  1689 417 1260
+3  343 842 342
+3  1834 903 902
+3  1931 563 868
+3  816 1058 815
+3  816 815 509
+3  161 1893 751
+3  1441 728 741
+3  741 728 739
+3  817 818 75
+3  1043 131 1048
+3  907 1106 886
+3  489 542 543
+3  557 731 730
+3  1309 1841 532
+3  235 251 571
+3  426 424 425
+3  204 428 429
+3  151 789 792
+3  11 954 451
+3  461 954 11
+3  39 172 772
+3  772 773 776
+3  162 254 251
+3  254 163 251
+3  1478 1229 400
+3  1633 1281 531
+3  854 912 852
+3  1898 1895 111
+3  548 547 20
+3  407 33 406
+3  1809 1825 1457
+3  1457 1825 1826
+3  239 153 258
+3  601 9 604
+3  1278 530 1279
+3  1286 144 1431
+3  1119 1118 230
+3  1116 218 219
+3  219 218 1343
+3  375 1291 952
+3  837 348 836
+3  1925 1924 1612
+3  238 244 40
+3  331 961 745
+3  173 256 257
+3  541 1439 542
+3  783 156 785
+3  390 126 391
+3  202 431 432
+3  1385 236 1386
+3  1465 461 460
+3  768 156 767
+3  1172 138 139
+3  770 768 154
+3  44 1451 1453
+3  714 4 294
+3  293 714 294
+3  1659 1785 475
+3  1894 172 751
+3  515 1143 1147
+3  1047 1046 510
+3  789 87 792
+3  792 87 793
+3  406 408 407
+3  1518 84 1513
+3  455 1800 1802
+3  1808 458 353
+3  285 282 281
+3  1894 766 172
+3  1530 450 1840
+3  1895 219 1697
+3  763 764 3
+3  1187 321 1847
+3  1847 321 1186
+3  1316 1235 573
+3  1316 573 1317
+3  51 420 286
+3  1223 1212 398
+3  8 998 997
+3  568 206 1192
+3  1282 568 1192
+3  1881 62 1163
+3  1163 1066 1881
+3  423 99 422
+3  1127 1126 626
+3  1366 1185 1845
+3  1366 1845 1186
+3  317 94 326
+3  206 1190 1969
+3  797 1679 170
+3  796 249 797
+3  547 546 20
+3  290 178 288
+3  813 17 254
+3  1891 749 1120
+3  482 483 118
+3  362 1000 995
+3  161 752 779
+3  624 1975 30
+3  624 30 646
+3  1716 886 1717
+3  1745 1746 188
+3  1266 1746 1745
+3  1331 191 1330
+3  1901 231 1900
+3  1978 1997 879
+3  793 87 795
+3  855 1472 1475
+3  704 701 494
+3  505 504 121
+3  288 701 704
+3  577 924 1199
+3  1924 1618 930
+3  1923 1618 1924
+3  1194 1193 1849
+3  1194 1849 1850
+3  1534 722 1885
+3  434 439 435
+3  1156 787 769
+3  1427 177 280
+3  1279 1280 521
+3  1849 1195 1850
+3  862 863 47
+3  47 863 864
+3  1721 1720 896
+3  1115 1114 38
+3  38 1114 1641
+3  1056 512 1053
+3  1722 832 1720
+3  1143 1144 1147
+3  1144 1145 2
+3  1875 1132 141
+3  1826 1562 1827
+3  602 1680 606
+3  45 1805 1807
+3  1698 1945 21
+3  1044 131 1043
+3  1943 1118 217
+3  67 1063 946
+3  915 1317 1318
+3  1051 511 1046
+3  604 9 1315
+3  1474 1469 902
+3  253 171 255
+3  732 731 136
+3  99 423 972
+3  1430 196 197
+3  77 444 446
+3  958 691 957
+3  1814 454 46
+3  783 785 784
+3  784 785 0
+3  673 680 672
+3  115 1227 1228
+3  1623 1624 584
+3  21 1944 1940
+3  567 569 204
+3  1007 368 1913
+3  986 988 987
+3  316 57 668
+3  5 1587 1583
+3  5 1583 1577
+3  260 86 1568
+3  1432 248 1567
+3  827 336 828
+3  1733 36 844
+3  200 1823 1822
+3  913 1177 914
+3  112 221 197
+3  770 37 786
+3  1635 1784 1786
+3  1786 1788 1635
+3  221 100 1430
+3  766 767 771
+3  771 767 783
+3  531 1281 1283
+3  1114 215 1038
+3  1938 1036 1939
+3  186 460 461
+3  675 676 181
+3  1035 1937 109
+3  180 673 672
+3  273 1947 226
+3  226 289 273
+3  229 1041 225
+3  811 216 812
+3  1892 1890 750
+3  158 1704 774
+3  719 718 1952
+3  1699 226 1698
+3  807 806 524
+3  1037 225 1040
+3  511 1045 1044
+3  164 277 1383
+3  1245 1735 848
+3  848 1735 1737
+3  48 1509 1854
+3  270 223 1730
+3  277 278 10
+3  1502 81 415
+3  216 1038 1034
+3  1042 815 510
+3  510 815 1058
+3  1446 1460 462
+3  1806 458 1807
+3  237 1389 1390
+3  1538 78 1543
+3  1543 1530 1538
+3  847 846 346
+3  346 1395 847
+3  858 564 854
+3  1357 261 1950
+3  1540 1803 453
+3  176 278 276
+3  234 276 1393
+3  969 369 70
+3  992 369 969
+3  1386 235 1385
+3  342 18 340
+3  74 517 522
+3  52 680 679
+3  249 26 797
+3  21 1940 1901
+3  792 793 149
+3  792 149 973
+3  1323 1324 142
+3  1960 1959 59
+3  59 1356 1960
+3  1895 1896 111
+3  1049 1045 522
+3  568 1182 419
+3  621 620 31
+3  643 644 642
+3  1986 1987 1259
+3  1170 1173 1171
+3  957 955 747
+3  1319 1322 1320
+3  597 1577 1578
+3  227 421 269
+3  875 874 834
+3  274 272 176
+3  1520 480 1521
+3  1109 188 1746
+3  271 272 274
+3  229 225 1945
+3  392 391 126
+3  4 711 297
+3  230 1118 1943
+3  21 1901 675
+3  2022 900 2023
+3  678 184 1897
+3  743 437 438
+3  438 437 24
+3  1819 1816 46
+3  1030 1029 381
+3  201 1013 1011
+3  686 1559 1748
+3  107 1559 686
+3  1087 1079 1080
+3  1587 1581 1583
+3  268 163 254
+3  1325 1286 1339
+3  1112 246 152
+3  152 246 1569
+3  1397 1395 346
+3  1392 1384 1385
+3  1874 141 1130
+3  326 328 28
+3  1403 91 373
+3  268 555 223
+3  307 302 662
+3  243 719 720
+3  552 26 554
+3  893 892 1714
+3  491 1476 1473
+3  787 152 788
+3  1559 685 1748
+3  40 721 239
+3  1681 1677 150
+3  710 713 292
+3  712 710 292
+3  120 195 1176
+3  1638 1637 66
+3  1639 1637 1638
+3  1572 1568 247
+3  1347 694 1346
+3  1347 1346 323
+3  1484 1161 1482
+3  269 223 270
+3  981 980 359
+3  981 359 983
+3  171 258 256
+3  1 253 252
+3  1184 1371 1196
+3  1184 1196 1364
+3  754 753 515
+3  1059 753 754
+3  1356 266 1960
+3  1960 266 1962
+3  1174 1693 1695
+3  1174 1695 140
+3  1498 934 933
+3  1881 1882 1880
+3  16 1654 377
+3  171 238 258
+3  235 1392 1385
+3  1530 1840 1533
+3  62 1873 1163
+3  1878 1873 62
+3  496 350 866
+3  1247 849 1246
+3  259 1948 1949
+3  654 314 653
+3  296 1736 1957
+3  296 1957 1376
+3  1354 293 266
+3  266 293 294
+3  711 291 708
+3  711 708 297
+3  173 749 1890
+3  267 244 165
+3  724 526 102
+3  724 102 1288
+3  281 278 279
+3  271 228 110
+3  405 696 327
+3  1394 1393 1392
+3  463 462 186
+3  186 462 1460
+3  110 272 271
+3  1107 95 899
+3  227 274 275
+3  274 176 275
+3  278 176 279
+3  280 272 273
+3  668 94 317
+3  668 317 316
+3  276 275 176
+3  329 1763 687
+3  329 687 1688
+3  329 1688 1742
+3  1772 1773 669
+3  275 276 234
+3  1461 1462 1801
+3  458 1808 1807
+3  463 1755 1754
+3  280 176 272
+3  279 176 280
+3  565 180 566
+3  1631 1259 534
+3  1689 1259 1631
+3  549 550 212
+3  1537 1536 447
+3  703 283 702
+3  282 703 702
+3  177 279 280
+3  1818 1816 198
+3  1916 516 756
+3  1307 1308 532
+3  245 244 267
+3  189 1833 1968
+3  178 284 287
+3  1033 1032 222
+3  1039 1032 1033
+3  1340 1286 1428
+3  76 442 441
+3  1545 1546 1544
+3  1544 78 1545
+3  1225 388 1230
+3  314 985 649
+3  652 649 548
+3  284 178 232
+3  1556 1555 1766
+3  1507 445 1548
+3  1528 445 1507
+3  292 713 714
+3  92 318 319
+3  92 319 322
+3  870 298 871
+3  781 1642 1062
+3  1445 1444 1447
+3  405 33 696
+3  875 850 193
+3  1451 44 1450
+3  317 326 318
+3  1916 942 516
+3  710 712 55
+3  1568 1567 247
+3  299 58 968
+3  764 411 413
+3  265 1355 1353
+3  346 846 1400
+3  467 1452 1451
+3  300 1961 301
+3  301 1961 1958
+3  665 844 1335
+3  1335 844 1377
+3  716 245 55
+3  414 409 11
+3  612 919 920
+3  610 919 612
+3  872 869 563
+3  71 435 436
+3  294 4 295
+3  161 255 1893
+3  867 496 866
+3  291 709 708
+3  713 4 714
+3  1189 1190 429
+3  308 124 659
+3  988 990 210
+3  1751 1750 685
+3  417 1300 1261
+3  864 863 823
+3  250 124 312
+3  326 28 318
+3  308 312 124
+3  654 653 655
+3  872 853 873
+3  1948 1951 128
+3  1082 1081 356
+3  1275 1274 826
+3  1276 1275 826
+3  24 437 436
+3  24 436 440
+3  337 878 879
+3  1957 1736 1327
+3  552 551 97
+3  212 551 552
+3  1090 1089 304
+3  113 917 916
+3  916 917 574
+3  1197 1843 1364
+3  1844 1843 1197
+3  1080 1081 1087
+3  1087 1081 19
+3  407 410 325
+3  93 1368 1367
+3  326 341 105
+3  326 105 328
+3  208 423 422
+3  297 295 4
+3  1647 1117 1648
+3  979 360 980
+3  695 1347 693
+3  696 33 697
+3  1893 256 173
+3  1588 1779 1590
+3  1590 41 1588
+3  545 698 438
+3  1092 1437 303
+3  316 305 1095
+3  493 494 12
+3  483 486 183
+3  482 486 483
+3  483 183 119
+3  1572 88 1650
+3  489 539 541
+3  400 1068 1069
+3  1477 400 1069
+3  1042 1043 1048
+3  1048 514 1042
+3  531 1283 1284
+3  520 763 3
+3  108 827 1276
+3  94 341 326
+3  335 1556 334
+3  571 421 234
+3  924 925 1665
+3  1785 1659 1141
+3  868 862 1931
+3  885 1866 1418
+3  1389 165 1390
+3  859 858 862
+3  347 336 335
+3  1947 110 229
+3  1139 1249 870
+3  1444 477 1447
+3  1442 477 1444
+3  1931 862 1929
+3  1929 563 1931
+3  247 1571 1572
+3  448 1533 1531
+3  1531 1533 1840
+3  1499 481 1500
+3  1503 1499 1500
+3  861 564 859
+3  346 1400 1399
+3  1399 1400 839
+3  857 1454 1453
+3  1571 88 1572
+3  1860 1859 1410
+3  298 870 1248
+3  1174 1175 1693
+3  143 1175 1174
+3  824 551 550
+3  821 1135 1134
+3  684 1750 1751
+3  1332 1333 191
+3  1121 6 2019
+3  1890 1892 173
+3  1348 1346 694
+3  1183 1348 694
+3  334 1760 1761
+3  334 340 18
+3  861 353 857
+3  860 353 861
+3  790 242 1112
+3  857 856 861
+3  163 421 571
+3  1270 335 108
+3  653 314 652
+3  678 1897 1902
+3  1276 826 1269
+3  1276 1269 108
+3  564 861 856
+3  1111 83 683
+3  291 711 713
+3  709 710 404
+3  986 360 979
+3  450 1508 80
+3  1207 920 1858
+3  396 1207 1858
+3  1027 69 1024
+3  1032 556 222
+3  488 1683 1684
+3  1229 115 1228
+3  23 410 411
+3  210 990 991
+3  362 995 994
+3  989 988 361
+3  159 1319 1643
+3  386 1226 1225
+3  1225 1226 388
+3  1598 1599 925
+3  1965 1966 371
+3  1132 1142 1133
+3  1132 1133 141
+3  903 1834 471
+3  218 1116 1647
+3  1658 1657 1141
+3  1660 1657 1658
+3  1027 1906 69
+3  1311 949 1312
+3  749 257 1904
+3  990 994 991
+3  1909 551 824
+3  1726 384 428
+3  397 1212 1213
+3  1213 1212 1223
+3  127 1853 395
+3  115 1218 1224
+3  115 1224 1227
+3  877 829 878
+3  429 428 203
+3  429 203 1189
+3  126 393 392
+3  1739 679 513
+3  1853 393 126
+3  387 1232 1219
+3  457 1461 1801
+3  1801 1798 457
+3  1983 1104 1982
+3  1982 1181 1983
+3  1232 390 16
+3  194 644 175
+3  175 611 194
+3  55 267 233
+3  710 55 404
+3  1412 1414 889
+3  795 87 1564
+3  1421 1238 915
+3  729 1296 1289
+3  1297 1296 729
+3  1420 1238 1421
+3  1878 1869 1877
+3  640 1878 1877
+3  395 389 1233
+3  1109 1108 469
+3  388 559 90
+3  90 559 1405
+3  1222 1223 398
+3  1217 387 1219
+3  1219 386 1217
+3  612 397 1213
+3  1213 32 612
+3  948 949 375
+3  375 949 950
+3  1807 1808 45
+3  33 405 406
+3  451 1531 450
+3  1822 46 454
+3  183 439 179
+3  414 80 415
+3  504 506 485
+3  408 505 122
+3  408 122 407
+3  1550 1551 1505
+3  1775 1774 507
+3  1775 507 1777
+3  1183 1369 1368
+3  544 542 132
+3  705 288 704
+3  217 1842 1115
+3  993 367 1004
+3  528 1151 1153
+3  1153 1151 1148
+3  503 506 504
+3  122 413 411
+3  122 411 410
+3  1820 199 1821
+3  1534 1533 448
+3  1534 448 1464
+3  1545 445 1546
+3  465 185 332
+3  1513 1512 1518
+3  747 955 323
+3  409 414 81
+3  781 1062 811
+3  566 180 672
+3  1414 913 908
+3  1728 1729 209
+3  429 567 204
+3  1364 1196 1197
+3  208 422 424
+3  356 1081 1080
+3  1372 696 697
+3  322 25 433
+3  201 1078 1079
+3  1082 19 1081
+3  203 427 430
+3  426 425 202
+3  1192 206 1191
+3  1187 25 321
+3  205 1188 1187
+3  205 1193 1194
+3  205 1194 1190
+3  310 433 432
+3  432 431 310
+3  433 25 432
+3  425 431 202
+3  1861 1862 1365
+3  1861 1365 1367
+3  319 321 322
+3  43 698 545
+3  233 404 55
+3  420 179 434
+3  1160 1485 1484
+3  1873 640 1872
+3  853 872 563
+3  708 709 707
+3  1355 265 1358
+3  968 58 967
+3  1900 184 1903
+3  1341 142 1339
+3  1689 1986 1259
+3  268 17 555
+3  1107 899 1108
+3  440 441 77
+3  1107 473 1264
+3  1263 1107 1264
+3  1273 474 1272
+3  1265 473 1266
+3  1264 473 1265
+3  76 440 436
+3  1457 1827 1828
+3  1828 1827 1459
+3  443 442 76
+3  343 1686 1333
+3  1522 482 118
+3  444 77 441
+3  476 1510 1524
+3  1524 1510 1511
+3  434 179 439
+3  76 435 443
+3  435 439 22
+3  435 22 443
+3  443 22 479
+3  464 1753 1754
+3  1546 445 1529
+3  1529 445 1528
+3  281 282 10
+3  1818 1817 1816
+3  1504 476 1524
+3  1756 461 185
+3  464 1756 185
+3  1794 1793 457
+3  1552 476 1504
+3  451 450 80
+3  1469 1470 490
+3  1145 1154 2
+3  1407 1408 501
+3  1804 456 1806
+3  1544 1546 1529
+3  1818 77 1817
+3  1817 446 1815
+3  49 487 505
+3  199 1822 1823
+3  753 523 515
+3  515 523 1143
+3  44 466 1450
+3  461 1756 186
+3  1557 1558 1559
+3  107 1557 1559
+3  1515 1514 1517
+3  1754 1753 83
+3  828 834 876
+3  71 1140 1696
+3  747 331 745
+3  1808 350 45
+3  780 781 811
+3  165 1389 1382
+3  1805 1823 200
+3  454 1803 200
+3  200 1803 1802
+3  1826 1797 79
+3  1825 1797 1826
+3  1464 1465 460
+3  448 1465 1464
+3  190 1466 1446
+3  958 106 690
+3  245 243 720
+3  186 1756 1755
+3  1537 78 1538
+3  1472 491 1475
+3  468 187 463
+3  190 1445 1447
+3  1557 344 1558
+3  1749 1757 1748
+3  1748 1757 686
+3  909 905 852
+3  344 1111 683
+3  1930 853 563
+3  909 829 905
+3  989 990 988
+3  1724 567 206
+3  1520 1521 1525
+3  552 1157 1158
+3  890 1419 889
+3  95 907 898
+3  1589 1780 933
+3  1397 837 1396
+3  1396 1395 1397
+3  1463 1461 466
+3  1536 1812 1813
+3  1552 1504 1505
+3  1551 1552 1505
+3  1409 1553 499
+3  500 503 487
+3  480 1522 1521
+3  899 904 469
+3  1971 904 899
+3  1040 225 1041
+3  952 401 951
+3  1120 53 1891
+3  1501 49 1502
+3  119 183 648
+3  1822 1819 46
+3  779 813 162
+3  3 544 520
+3  354 1452 1471
+3  390 391 1654
+3  882 830 1978
+3  890 892 1419
+3  709 290 707
+3  1 235 1386
+3  1432 249 796
+3  1080 357 1933
+3  211 1991 1406
+3  1990 1991 211
+3  288 706 290
+3  863 348 823
+3  868 348 863
+3  290 404 178
+3  1255 294 295
+3  71 436 437
+3  954 461 1465
+3  1795 1809 1466
+3  1501 502 49
+3  49 502 487
+3  825 1478 1477
+3  209 423 1728
+3  487 503 504
+3  1365 1185 1366
+3  502 500 487
+3  49 408 1502
+3  1129 1128 1132
+3  1129 1132 1875
+3  806 807 538
+3  1057 1056 513
+3  449 1516 1532
+3  1532 1516 1508
+3  500 502 481
+3  1059 754 756
+3  540 504 485
+3  697 1371 1372
+3  1580 933 1781
+3  1774 506 1776
+3  267 165 233
+3  121 504 540
+3  1148 2 1153
+3  1540 455 1803
+3  603 1678 1679
+3  1677 1678 603
+3  811 160 780
+3  756 516 759
+3  756 759 757
+3  1439 1440 1438
+3  818 1061 75
+3  517 523 522
+3  540 539 489
+3  544 759 760
+3  759 516 760
+3  1903 675 1900
+3  771 157 772
+3  1763 329 1764
+3  512 74 522
+3  750 53 766
+3  1126 1128 626
+3  626 1128 1129
+3  104 1182 1282
+3  430 25 1188
+3  1188 25 1187
+3  521 764 763
+3  1198 1197 325
+3  325 410 1198
+3  135 738 804
+3  1674 15 1675
+3  1049 522 1050
+3  74 809 517
+3  1182 568 1282
+3  1431 144 1176
+3  1272 474 887
+3  1751 683 1752
+3  104 1280 1279
+3  526 518 102
+3  819 508 817
+3  1148 1147 2
+3  1308 1307 533
+3  1224 1225 1227
+3  1227 1225 1230
+3  1071 952 1294
+3  1294 13 1071
+3  121 540 489
+3  1124 1123 119
+3  508 1777 1778
+3  1117 1645 1644
+3  1644 1645 38
+3  540 485 539
+3  67 604 1314
+3  1390 165 244
+3  755 942 1916
+3  256 1497 257
+3  164 1392 1393
+3  258 238 239
+3  238 40 239
+3  268 223 269
+3  1388 1386 236
+3  1392 164 1384
+3  1390 244 238
+3  169 1871 634
+3  926 1597 925
+3  778 752 39
+3  399 1229 1478
+3  1478 1479 399
+3  251 235 1
+3  254 17 268
+3  131 1050 753
+3  162 813 254
+3  17 813 812
+3  1039 109 1040
+3  221 112 220
+3  228 1041 229
+3  1071 401 952
+3  375 950 1291
+3  1652 242 1653
+3  318 28 1350
+3  770 156 768
+3  1732 223 556
+3  708 54 1251
+3  708 1251 297
+3  745 961 960
+3  960 959 745
+3  1368 1369 1367
+3  559 1404 1405
+3  682 11 409
+3  461 11 682
+3  461 682 185
+3  681 682 82
+3  82 682 409
+3  413 121 543
+3  1026 2018 2020
+3  1255 58 299
+3  1328 58 1255
+3  745 957 747
+3  1843 1185 1862
+3  1851 1185 1843
+3  314 984 985
+3  549 212 547
+3  689 963 964
+3  964 963 688
+3  529 1782 1150
+3  1150 1149 529
+3  212 552 554
+3  1103 1101 1178
+3  1103 1178 1981
+3  797 1158 553
+3  896 1719 1712
+3  994 995 366
+3  97 607 1157
+3  752 172 39
+3  263 240 719
+3  674 1699 1698
+3  1902 220 112
+3  1841 1634 670
+3  671 1634 1841
+3  1067 533 1301
+3  767 53 768
+3  804 519 524
+3  777 39 776
+3  974 977 148
+3  1884 722 1542
+3  1539 1884 1542
+3  1565 1570 1569
+3  1865 1274 1273
+3  1904 174 1120
+3  749 1904 1120
+3  1600 1664 1601
+3  631 638 598
+3  1784 1271 1785
+3  1072 1074 1075
+3  1322 1319 159
+3  1412 913 1414
+3  1618 941 1619
+3  728 731 732
+3  1677 603 1680
+3  313 653 20
+3  906 1690 833
+3  1679 553 603
+3  922 921 1424
+3  939 938 583
+3  1935 1921 311
+3  1487 140 1489
+3  1487 1489 1483
+3  106 960 962
+3  963 106 962
+3  1431 1429 1286
+3  249 1434 26
+3  1433 1434 249
+3  1684 1682 1123
+3  1127 626 1876
+3  639 1127 1876
+3  1666 149 793
+3  1666 793 794
+3  1675 977 974
+3  1913 1008 1912
+3  1264 887 1263
+3  959 960 106
+3  1159 1130 1131
+3  1872 1130 1159
+3  139 1165 1170
+3  1133 970 971
+3  971 970 1173
+3  814 812 216
+3  969 1018 1031
+3  1830 189 1829
+3  115 1229 1222
+3  1404 374 1405
+3  1230 90 402
+3  975 149 1666
+3  1418 1866 1416
+3  632 42 633
+3  398 1218 1222
+3  549 560 550
+3  616 194 614
+3  914 1177 830
+3  644 194 642
+3  1287 1288 725
+3  1289 1287 725
+3  1513 1514 1515
+3  1515 1076 1513
+3  982 1011 422
+3  1443 1468 477
+3  1443 477 1442
+3  388 90 1230
+3  1226 1220 383
+3  141 1874 1875
+3  1233 389 1231
+3  1002 371 91
+3  1001 1002 364
+3  1028 68 1029
+3  9 601 602
+3  34 1004 1005
+3  1450 1449 1468
+3  175 644 621
+3  1697 219 1343
+3  68 1019 403
+3  584 1624 1625
+3  1008 1015 379
+3  1018 368 1031
+3  940 1243 394
+3  611 613 194
+3  996 997 369
+3  997 998 1009
+3  369 1009 376
+3  694 1374 1183
+3  95 1107 1263
+3  195 138 557
+3  1435 27 195
+3  1127 1125 1126
+3  1574 1575 41
+3  139 1171 1172
+3  1172 137 645
+3  1828 1459 460
+3  460 1459 1464
+3  1068 947 401
+3  401 947 951
+3  818 508 1778
+3  1781 933 1780
+3  1410 483 501
+3  1122 501 483
+3  1677 1681 1678
+3  1910 608 97
+3  97 551 1910
+3  1072 736 1073
+3  116 1069 1070
+3  1653 1313 153
+3  744 1742 1688
+3  761 762 520
+3  388 365 559
+3  29 1664 1663
+3  222 555 814
+3  556 555 222
+3  558 384 385
+3  641 1160 1162
+3  1098 1096 1094
+3  1150 755 561
+3  1150 561 1149
+3  1062 216 811
+3  847 347 345
+3  1165 143 1164
+3  1164 1168 1165
+3  1796 1456 1795
+3  544 760 520
+3  10 284 562
+3  1353 1354 265
+3  478 901 1444
+3  1445 478 1444
+3  1252 298 1251
+3  54 1252 1251
+3  236 1385 1384
+3  47 859 862
+3  875 834 850
+3  1075 825 116
+3  264 1352 1353
+3  206 567 429
+3  1374 324 1370
+3  1599 1598 1610
+3  1184 1364 1861
+3  183 179 566
+3  443 479 117
+3  565 51 289
+3  180 565 289
+3  470 901 478
+3  2023 901 470
+3  867 63 870
+3  1254 1737 1376
+3  1930 854 853
+3  1136 1135 821
+3  1335 1331 1330
+3  665 1335 1330
+3  2021 2020 6
+3  576 621 31
+3  32 1481 1480
+3  897 1721 1832
+3  897 1832 2022
+3  635 169 634
+3  583 1623 1243
+3  615 1073 735
+3  1287 724 1288
+3  1626 1611 930
+3  930 1611 1612
+3  636 5 1577
+3  1168 1169 1165
+3  31 1206 1205
+3  620 1206 31
+3  641 1872 1159
+3  156 786 785
+3  1622 1623 583
+3  576 610 175
+3  621 576 175
+3  1663 1665 1599
+3  1179 591 1178
+3  1159 1131 1166
+3  946 945 592
+3  1888 66 1887
+3  1888 1887 1886
+3  793 1099 794
+3  1646 1116 1115
+3  1115 1116 217
+3  1566 1564 1565
+3  616 642 194
+3  632 638 631
+3  157 783 784
+3  1207 396 1209
+3  1209 396 1984
+3  1175 1694 1693
+3  1066 146 1065
+3  633 635 634
+3  634 597 633
+3  1675 15 977
+3  1671 594 1672
+3  578 1593 1595
+3  760 516 942
+3  1103 1105 593
+3  1626 936 1611
+3  1868 1867 640
+3  1570 1565 87
+3  1669 1670 1668
+3  1713 894 1714
+3  533 1307 1305
+3  447 78 1537
+3  9 602 606
+3  1576 1578 1577
+3  1576 1577 596
+3  608 607 97
+3  1709 892 1710
+3  892 890 1710
+3  1069 401 1705
+3  103 1067 1297
+3  735 736 13
+3  1073 736 735
+3  394 1241 1240
+3  575 1424 1202
+3  918 919 610
+3  113 919 918
+3  1243 584 1242
+3  194 613 614
+3  1513 84 1514
+3  1514 84 1500
+3  604 1640 601
+3  947 90 948
+3  1495 1490 1695
+3  1695 1490 140
+3  733 136 72
+3  734 733 72
+3  101 733 734
+3  932 928 1591
+3  628 600 629
+3  629 600 630
+3  1205 1200 618
+3  1561 127 1234
+3  1240 127 1561
+3  1508 1528 1532
+3  1104 1105 1103
+3  1973 978 1972
+3  1974 978 1973
+3  589 943 585
+3  621 30 622
+3  1493 1065 1492
+3  1064 1483 1489
+3  1491 1064 1489
+3  1242 1241 394
+3  633 42 635
+3  644 30 621
+3  647 1055 130
+3  1172 1171 137
+3  642 616 72
+3  786 37 155
+3  628 629 627
+3  622 214 620
+3  724 1287 1303
+3  633 638 632
+3  628 627 214
+3  593 168 594
+3  636 168 593
+3  1867 1874 1130
+3  587 1242 584
+3  213 627 629
+3  1201 1202 1203
+3  575 1202 1201
+3  1610 927 1611
+3  1994 1993 884
+3  148 977 1883
+3  1010 1021 69
+3  944 943 65
+3  30 623 622
+3  617 1200 1205
+3  1579 598 1574
+3  1579 1574 1573
+3  1146 809 519
+3  829 1976 879
+3  62 1880 1878
+3  1055 1054 130
+3  74 538 807
+3  145 1167 1168
+3  645 643 72
+3  643 645 137
+3  1170 1171 139
+3  136 645 72
+3  197 27 810
+3  137 646 643
+3  1610 29 1768
+3  1610 1768 1599
+3  1902 112 805
+3  183 566 648
+3  946 604 67
+3  1999 1998 1415
+3  1415 1419 1999
+3  889 1419 1415
+3  650 360 651
+3  369 992 996
+3  362 994 989
+3  1951 1952 264
+3  654 655 125
+3  427 203 428
+3  242 1652 246
+3  265 1354 1356
+3  429 1190 206
+3  113 1077 920
+3  1082 311 1918
+3  660 659 124
+3  658 657 308
+3  661 312 308
+3  1572 1650 259
+3  1568 1572 259
+3  663 662 302
+3  657 661 308
+3  564 856 855
+3  26 1158 797
+3  664 60 1920
+3  306 664 1920
+3  311 1082 356
+3  1922 2024 579
+3  306 663 664
+3  579 2024 1615
+3  664 663 302
+3  1427 280 273
+3  1851 1844 1852
+3  1851 1852 1845
+3  28 1349 1350
+3  1574 41 1573
+3  1573 41 2014
+3  1658 188 1660
+3  1409 501 1408
+3  1048 753 1059
+3  1200 617 935
+3  1114 1842 1118
+3  648 672 52
+3  672 648 566
+3  1645 1115 38
+3  1517 415 1515
+3  1701 289 226
+3  538 74 512
+3  675 1903 676
+3  1730 1732 1731
+3  698 283 742
+3  742 283 703
+3  242 246 1112
+3  742 438 698
+3  496 1821 45
+3  82 537 681
+3  1898 219 1895
+3  860 1808 353
+3  185 681 332
+3  80 1516 1076
+3  693 1347 323
+3  683 83 1752
+3  332 690 689
+3  683 685 1558
+3  1751 685 683
+3  92 322 1361
+3  963 689 690
+3  82 405 327
+3  689 964 965
+3  1689 1631 1634
+3  695 694 1347
+3  959 958 957
+3  106 958 959
+3  691 537 692
+3  537 82 692
+3  959 957 745
+3  1373 1372 324
+3  1374 1370 1369
+3  1183 1374 1369
+3  1760 107 1759
+3  407 697 33
+3  706 288 705
+3  288 178 287
+3  288 287 701
+3  103 671 1309
+3  287 702 700
+3  284 702 287
+3  700 283 699
+3  284 282 702
+3  1456 1825 1809
+3  493 705 704
+3  1253 295 297
+3  291 710 709
+3  54 706 705
+3  712 716 55
+3  871 298 1252
+3  493 871 1252
+3  709 404 290
+3  107 686 1759
+3  1708 1707 1711
+3  830 1177 1995
+3  336 347 850
+3  1752 465 684
+3  1751 1752 684
+3  1255 295 1254
+3  1135 36 1134
+3  715 716 712
+3  1399 1381 838
+3  129 717 715
+3  40 720 721
+3  719 243 718
+3  245 720 40
+3  1235 1234 573
+3  1260 1261 416
+3  569 1988 570
+3  729 1289 1290
+3  1300 671 1299
+3  541 819 509
+3  181 1656 680
+3  810 112 197
+3  1287 1289 527
+3  375 952 951
+3  7 1725 1727
+3  7 1727 1989
+3  1037 1940 1944
+3  130 1053 1052
+3  1798 1796 457
+3  1900 1899 1896
+3  765 762 133
+3  538 512 1056
+3  1040 1041 224
+3  1922 1613 1614
+3  1432 796 248
+3  474 1277 885
+3  390 1980 126
+3  801 678 525
+3  1085 19 1083
+3  1939 1036 1941
+3  1698 1946 1945
+3  1289 1296 527
+3  1741 507 1773
+3  485 1777 539
+3  286 1696 1140
+3  1016 378 1019
+3  1696 420 71
+3  134 1307 1284
+3  1284 1307 532
+3  951 947 948
+3  375 951 948
+3  819 816 509
+3  1507 1506 1550
+3  1550 1506 1551
+3  209 361 972
+3  615 735 734
+3  1074 615 616
+3  518 727 102
+3  102 727 726
+3  1435 195 730
+3  1262 416 1261
+3  1260 417 1261
+3  72 616 615
+3  734 72 615
+3  727 732 101
+3  727 728 732
+3  557 136 731
+3  101 735 726
+3  732 136 733
+3  732 733 101
+3  101 726 727
+3  818 535 1061
+3  1740 535 818
+3  101 734 735
+3  1072 1075 116
+3  680 1656 677
+3  680 677 679
+3  1068 401 1069
+3  737 738 135
+3  739 728 727
+3  73 738 739
+3  73 739 727
+3  1442 1444 901
+3  1829 1442 901
+3  1433 1432 86
+3  1101 1103 593
+3  1100 1101 593
+3  1667 150 1668
+3  391 68 403
+3  873 2025 835
+3  1058 1047 510
+3  1548 48 1854
+3  1939 231 1940
+3  1044 1049 131
+3  1866 885 1417
+3  1798 1801 1799
+3  1799 1541 1798
+3  237 1387 1388
+3  1386 1388 1387
+3  419 534 1259
+3  903 906 833
+3  1773 507 1774
+3  329 1742 746
+3  1625 585 2021
+3  6 1625 2021
+3  1017 1020 379
+3  1663 1664 1600
+3  418 1312 1262
+3  1941 1036 230
+3  1898 1943 217
+3  1901 1940 231
+3  1499 497 1527
+3  1527 481 1499
+3  466 1449 1450
+3  885 1662 1417
+3  691 956 955
+3  330 964 1765
+3  425 201 1079
+3  328 746 331
+3  28 328 331
+3  960 744 962
+3  960 961 744
+3  1014 987 210
+3  331 748 28
+3  1120 769 53
+3  446 78 447
+3  1891 53 750
+3  1705 401 1071
+3  344 1557 1791
+3  1791 1790 344
+3  520 762 763
+3  1440 815 758
+3  752 778 779
+3  159 780 777
+3  1730 223 1732
+3  1917 123 663
+3  1043 510 1046
+3  1439 1438 132
+3  756 757 514
+3  514 757 758
+3  730 731 1441
+3  520 760 761
+3  371 98 1965
+3  1278 1279 521
+3  1175 144 1694
+3  133 762 761
+3  1193 205 1187
+3  412 411 764
+3  23 411 412
+3  764 413 3
+3  1642 781 1643
+3  1644 1642 1643
+3  771 172 766
+3  768 769 154
+3  258 153 1497
+3  773 772 157
+3  1322 776 158
+3  776 39 772
+3  1325 1324 953
+3  1916 756 754
+3  700 701 287
+3  778 160 779
+3  224 1039 1040
+3  1114 1118 215
+3  830 1995 1996
+3  157 771 783
+3  1583 1581 596
+3  1642 1641 1062
+3  784 0 1494
+3  1494 774 784
+3  767 156 783
+3  396 1211 1984
+3  869 348 868
+3  156 770 786
+3  769 787 154
+3  552 97 1157
+3  9 606 607
+3  607 606 1157
+3  1770 1125 42
+3  42 1125 1127
+3  788 152 789
+3  154 787 788
+3  95 1106 907
+3  1566 248 1564
+3  1564 248 795
+3  790 787 174
+3  859 564 858
+3  945 1180 592
+3  65 1180 945
+3  1158 26 552
+3  150 1676 1669
+3  507 1778 1777
+3  1565 1564 87
+3  1106 887 886
+3  1416 1999 472
+3  1868 1877 639
+3  1876 1868 639
+3  1003 362 209
+3  50 426 427
+3  1317 573 916
+3  423 209 972
+3  1423 1422 574
+3  574 1425 1423
+3  547 548 549
+3  45 1823 1805
+3  560 382 34
+3  1406 382 560
+3  1260 7 1989
+3  1934 656 355
+3  125 655 656
+3  428 384 50
+3  1176 144 1175
+3  656 313 661
+3  1126 1973 1972
+3  1935 355 1921
+3  1133 971 141
+3  1084 1083 309
+3  1926 1612 927
+3  123 658 662
+3  663 123 662
+3  1190 1189 205
+3  3 543 544
+3  1194 1850 1191
+3  1083 1082 309
+3  38 1641 1642
+3  714 1359 292
+3  1644 1643 782
+3  1117 1644 782
+3  602 1676 1680
+3  1086 1084 309
+3  1470 833 1473
+3  915 1238 1236
+3  1316 915 1236
+3  418 1311 1312
+3  302 59 1959
+3  317 318 92
+3  1350 1349 320
+3  677 1856 679
+3  723 1304 1306
+3  368 1006 1005
+3  339 1635 1789
+3  1768 29 1663
+3  1768 1663 1599
+3  1055 513 1054
+3  1438 757 132
+3  339 1270 1635
+3  1635 1270 1271
+3  1783 530 765
+3  473 1746 1266
+3  1517 1514 1501
+3  648 647 119
+3  1735 296 1737
+3  1736 296 1735
+3  1279 1281 104
+3  1779 1589 932
+3  1655 1656 676
+3  657 355 656
+3  140 1490 1489
+3  316 317 92
+3  1655 678 801
+3  1097 967 315
+3  1772 669 1407
+3  1368 93 1351
+3  676 184 678
+3  678 1655 676
+3  1122 484 1407
+3  1593 1604 2009
+3  327 693 692
+3  1704 784 774
+3  841 192 842
+3  14 1237 1238
+3  182 1856 1857
+3  1910 1909 370
+3  975 974 149
+3  1679 1678 170
+3  1162 1160 1484
+3  976 148 37
+3  182 538 1057
+3  1776 85 1408
+3  180 1700 673
+3  1946 1698 226
+3  1857 801 802
+3  1656 1655 677
+3  182 1857 802
+3  1311 418 1310
+3  1310 418 1299
+3  738 519 804
+3  73 519 738
+3  524 802 803
+3  73 518 519
+3  1736 1326 1327
+3  518 73 727
+3  112 810 805
+3  808 809 74
+3  541 509 1439
+3  1260 416 7
+3  1052 1053 511
+3  511 1053 1045
+3  1049 1050 131
+3  1280 1282 412
+3  412 1282 1192
+3  810 27 737
+3  519 518 1146
+3  809 1146 517
+3  104 1282 1280
+3  1989 1727 1258
+3  229 1945 1946
+3  340 105 341
+3  525 803 802
+3  834 874 876
+3  1349 28 748
+3  1271 1269 475
+3  1785 1271 475
+3  1846 1845 207
+3  1848 1846 207
+3  561 1687 1147
+3  519 809 808
+3  888 1273 1272
+3  802 801 525
+3  677 801 1857
+3  811 812 160
+3  936 1626 1627
+3  1627 1626 1619
+3  803 525 805
+3  803 804 524
+3  135 804 803
+3  135 803 805
+3  526 723 1155
+3  724 723 526
+3  1717 1715 1716
+3  807 808 74
+3  519 808 524
+3  805 810 135
+3  536 1051 1047
+3  345 347 18
+3  814 17 812
+3  514 1059 756
+3  1682 1741 484
+3  484 1122 1123
+3  1123 1124 1684
+3  488 1684 1124
+3  1910 1027 608
+3  1326 1953 1327
+3  1741 1740 507
+3  1740 1741 535
+3  1682 535 1741
+3  817 75 816
+3  819 817 816
+3  539 819 541
+3  34 550 560
+3  1249 849 1247
+3  468 83 1111
+3  893 472 892
+3  1476 1472 1471
+3  491 1472 1476
+3  848 56 1246
+3  714 293 1359
+3  715 292 1359
+3  839 1381 1399
+3  1214 1216 398
+3  1480 825 614
+3  614 825 1075
+3  71 437 1140
+3  12 496 867
+3  845 1733 1734
+3  1642 1644 38
+3  256 1893 255
+3  290 706 707
+3  1248 1250 1251
+3  1251 1250 297
+3  1481 32 1213
+3  381 2017 1030
+3  1030 2017 1025
+3  886 885 1418
+3  1969 1191 206
+3  886 887 474
+3  471 895 903
+3  338 2026 873
+3  873 2026 2025
+3  773 157 1704
+3  471 1832 1721
+3  1710 1702 831
+3  1275 337 881
+3  854 564 855
+3  2026 874 2025
+3  1121 587 6
+3  1885 452 1534
+3  721 240 241
+3  260 1568 1949
+3  239 721 241
+3  241 1653 153
+3  343 342 341
+3  301 1959 1960
+3  1580 1781 1575
+3  659 658 308
+3  665 1329 1953
+3  1875 1874 1129
+3  853 852 905
+3  467 1467 490
+3  1468 1443 467
+3  1450 1468 467
+3  1110 188 1109
+3  876 877 828
+3  1556 339 1555
+3  1270 339 1556
+3  1556 335 1270
+3  956 323 955
+3  1360 330 1749
+3  335 336 108
+3  338 905 877
+3  943 590 65
+3  1239 394 1560
+3  1560 581 1239
+3  1650 1649 1948
+3  1649 88 1651
+3  356 1933 1934
+3  1951 1355 128
+3  1137 63 1138
+3  1201 1199 924
+3  658 123 657
+3  302 1959 1958
+3  1739 513 1055
+3  338 877 876
+3  1381 839 1380
+3  828 337 827
+3  862 868 863
+3  1405 374 949
+3  949 374 1312
+3  1379 1380 840
+3  95 897 899
+3  1331 352 2016
+3  1335 352 1331
+3  1422 1423 921
+3  845 1734 1735
+3  1735 1734 1736
+3  1377 844 843
+3  843 1378 1377
+3  865 1138 866
+3  1709 1710 831
+3  1478 400 1477
+3  851 1135 820
+3  36 1135 851
+3  838 837 1398
+3  1397 1398 837
+3  56 1247 1246
+3  1245 849 1134
+3  1246 849 1245
+3  300 967 1097
+3  192 839 846
+3  36 851 843
+3  469 904 187
+3  338 876 2026
+3  834 336 850
+3  850 347 847
+3  847 193 850
+3  1396 193 1395
+3  860 861 859
+3  232 1383 562
+3  1778 1740 818
+3  1136 820 1135
+3  1781 1588 1575
+3  2019 392 1121
+3  253 1391 171
+3  1029 392 2019
+3  1831 1834 902
+3  902 1833 1831
+3  469 468 1110
+3  469 1110 1109
+3  1266 1267 1265
+3  914 883 913
+3  911 883 914
+3  1445 190 1446
+3  1836 822 1835
+3  1453 1451 1452
+3  1554 1790 1791
+3  1377 1378 352
+3  1335 1377 352
+3  864 860 47
+3  1706 1707 831
+3  831 1702 1706
+3  1479 1478 825
+3  852 853 854
+3  1992 884 1993
+3  1274 881 1277
+3  1277 474 1274
+3  865 866 350
+3  252 1402 161
+3  335 334 18
+3  330 1360 964
+3  1827 1562 1459
+3  47 860 859
+3  347 335 18
+3  252 253 1402
+3  1929 1930 563
+3  865 350 860
+3  864 865 860
+3  823 349 864
+3  864 349 865
+3  342 345 18
+3  233 232 178
+3  871 12 867
+3  493 12 871
+3  872 835 869
+3  286 1140 285
+3  1140 64 285
+3  1414 908 1703
+3  891 1691 1690
+3  872 873 835
+3  835 874 875
+3  253 255 1402
+3  878 337 828
+3  96 883 911
+3  878 828 877
+3  473 1107 1108
+3  1560 394 1240
+3  2000 2001 884
+3  905 829 877
+3  889 1414 890
+3  1994 1997 1996
+3  1996 1997 830
+3  1471 1470 1476
+3  913 883 908
+3  874 2026 876
+3  1704 157 784
+3  853 905 338
+3  1320 1321 2006
+3  1470 1474 833
+3  1020 1021 1010
+3  900 1830 2023
+3  1069 116 1477
+3  897 95 898
+3  855 856 1472
+3  491 1692 1475
+3  829 1977 1976
+3  898 1722 1720
+3  790 1112 787
+3  1108 899 469
+3  354 1471 1472
+3  1832 1834 1831
+3  1832 1831 900
+3  2023 1830 901
+3  2023 470 2022
+3  1462 1463 456
+3  722 1534 1464
+3  1455 1448 1793
+3  1794 1455 1793
+3  470 478 904
+3  895 906 903
+3  891 1702 1703
+3  1703 1710 890
+3  1703 908 891
+3  885 886 474
+3  855 96 912
+3  910 912 96
+3  1999 1419 472
+3  2001 1417 2002
+3  2000 1866 1417
+3  1416 1866 2000
+3  1747 262 260
+3  1418 1416 472
+3  827 1275 1276
+3  910 1979 909
+3  2002 1662 880
+3  1996 1993 1994
+3  1660 188 1110
+3  1996 1995 1993
+3  829 909 1977
+3  1954 1953 1329
+3  1327 1953 1954
+3  1272 1265 888
+3  286 420 1696
+3  879 878 829
+3  2003 1413 1412
+3  1413 1995 1177
+3  910 909 852
+3  911 910 96
+3  1997 880 879
+3  912 910 852
+3  914 830 882
+3  1983 1587 1104
+3  911 914 882
+3  851 1836 1838
+3  1838 1839 851
+3  835 836 869
+3  1839 1838 351
+3  351 1838 1835
+3  1331 2016 1332
+3  851 820 1836
+3  304 1919 1920
+3  697 1196 1371
+3  657 656 661
+3  1336 191 1334
+3  667 1336 1334
+3  1837 820 1136
+3  1136 349 1837
+3  1728 385 1729
+3  1799 1800 455
+3  1462 1800 1799
+3  1362 666 1363
+3  170 1099 796
+3  1088 1079 1087
+3  1210 395 1985
+3  1092 1961 300
+3  1131 1167 1166
+3  1944 225 1037
+3  2024 1922 1614
+3  1553 85 500
+3  1452 857 1453
+3  1601 609 923
+3  1508 1516 80
+3  1256 1089 61
+3  1077 1211 396
+3  858 1930 1929
+3  747 323 1345
+3  467 490 1452
+3  1449 466 1448
+3  1236 581 1235
+3  923 609 922
+3  922 575 923
+3  921 1421 1422
+3  257 1497 153
+3  257 153 791
+3  394 1243 1242
+3  1422 1318 574
+3  256 258 1497
+3  575 924 923
+3  14 609 939
+3  575 1201 924
+3  1241 1242 587
+3  932 1615 928
+3  1045 512 522
+3  2012 1915 599
+3  2012 599 2011
+3  1585 595 1181
+3  1181 590 1585
+3  1613 1924 1925
+3  410 23 1195
+3  1599 1665 925
+3  581 1237 1239
+3  1598 927 1610
+3  29 1610 1611
+3  1203 1204 618
+3  1202 1423 1425
+3  761 942 133
+3  1202 1424 1423
+3  1697 1343 100
+3  942 761 760
+3  922 609 14
+3  599 1594 2011
+3  599 1609 1594
+3  1573 1609 599
+3  1309 1067 103
+3  937 582 938
+3  939 583 940
+3  940 14 939
+3  935 213 1606
+3  1606 1607 935
+3  1928 1596 1595
+3  1595 1596 578
+3  1584 595 1585
+3  1619 1620 582
+3  941 1618 1616
+3  892 1709 1713
+3  1713 1714 892
+3  527 1303 1287
+3  589 590 943
+3  647 1124 119
+3  253 1387 1391
+3  946 586 945
+3  586 944 945
+3  1340 1339 1286
+3  1341 1339 1340
+3  1907 1007 1905
+3  1091 303 1090
+3  1092 303 1091
+3  1099 795 796
+3  204 1726 428
+3  1726 1725 372
+3  373 1725 7
+3  1353 1355 264
+3  1952 1951 263
+3  91 371 1966
+3  1920 1091 304
+3  1646 1645 1117
+3  1115 1645 1646
+3  1080 1079 1078
+3  1291 1290 1295
+3  1295 1292 1294
+3  1793 1448 1461
+3  442 117 1510
+3  447 1536 1813
+3  1547 445 1545
+3  1147 1144 2
+3  2004 166 2005
+3  1277 881 1662
+3  885 1277 1662
+3  1540 1810 1541
+3  1122 119 1123
+3  444 48 1547
+3  477 1449 1447
+3  1447 1449 1448
+3  1466 1455 1795
+3  190 1455 1466
+3  1685 1504 1503
+3  1255 1254 1376
+3  1042 758 815
+3  397 920 1207
+3  612 920 397
+3  814 555 17
+3  1738 1314 605
+3  511 1044 1046
+3  1531 1840 450
+3  1158 1157 553
+3  228 1731 1041
+3  224 1731 1732
+3  67 1025 1063
+3  1063 1025 1026
+3  941 1617 1628
+3  1600 923 924
+3  215 1118 1119
+3  1022 1025 67
+3  1402 255 161
+3  782 1643 1319
+3  224 1032 1039
+3  196 27 197
+3  453 1803 1812
+3  777 778 39
+3  777 780 778
+3  1320 158 1321
+3  1322 777 776
+3  151 792 973
+3  635 42 1127
+3  159 777 1322
+3  475 1268 1864
+3  1864 1267 475
+3  236 1382 1389
+3  43 545 1820
+3  1815 1814 46
+3  1463 1462 1461
+3  985 650 1990
+3  900 2022 1832
+3  556 224 1732
+3  1303 1302 723
+3  359 984 983
+3  134 1152 1305
+3  594 168 1673
+3  378 403 1019
+3  210 991 993
+3  1125 1770 637
+3  972 979 99
+3  1582 933 1580
+3  1256 1113 310
+3  1811 1540 1539
+3  1491 775 1492
+3  1941 230 1942
+3  261 660 1747
+3  1054 1056 1053
+3  1376 1328 1255
+3  741 740 1441
+3  1786 1787 1788
+3  160 813 779
+3  217 1116 1898
+3  525 1902 805
+3  220 1897 1896
+3  419 569 1724
+3  419 1724 568
+3  166 2004 1344
+3  665 1330 1329
+3  488 1060 1061
+3  538 182 806
+3  522 523 1050
+3  1060 75 1061
+3  817 508 818
+3  1731 224 1041
+3  514 758 1042
+3  1044 1043 1046
+3  1693 1694 953
+3  1810 79 1797
+3  691 958 537
+3  691 692 956
+3  1185 1365 1862
+3  693 956 692
+3  962 688 963
+3  319 1351 93
+3  331 746 961
+3  417 1689 1634
+3  92 1361 316
+3  332 965 684
+3  1766 107 1760
+3  963 690 106
+3  965 332 689
+3  328 1764 746
+3  1764 329 746
+3  966 1338 315
+3  315 1338 1337
+3  1354 266 1356
+3  300 968 967
+3  301 968 300
+3  1760 1759 333
+3  333 1759 1762
+3  686 1757 1758
+3  1758 1757 1765
+3  330 1765 1757
+3  964 688 1765
+3  965 1360 684
+3  1764 105 1763
+3  321 1366 1186
+3  1337 666 1336
+3  1350 320 1351
+3  318 1351 319
+3  1350 1351 318
+3  870 63 1139
+3  1820 545 198
+3  844 36 843
+3  1880 1879 1878
+3  357 1012 1244
+3  921 1420 1421
+3  822 837 838
+3  822 838 1835
+3  384 558 50
+3  1933 125 1934
+3  970 624 646
+3  1873 1872 641
+3  1511 117 480
+3  1519 1511 480
+3  1257 213 629
+3  1974 625 1863
+3  1887 1100 799
+3  973 149 974
+3  151 976 37
+3  974 148 976
+3  1979 1977 909
+3  1998 884 1992
+3  882 1976 1977
+3  1950 261 1747
+3  849 821 1134
+3  983 984 492
+3  982 422 99
+3  358 982 983
+3  983 982 981
+3  845 1245 1134
+3  1035 1743 215
+3  654 984 314
+3  986 361 988
+3  654 492 984
+3  986 979 972
+3  361 986 972
+3  360 987 651
+3  987 988 210
+3  1204 1205 618
+3  209 989 361
+3  209 362 989
+3  998 1001 364
+3  996 8 997
+3  991 367 993
+3  210 993 1014
+3  991 366 992
+3  1009 998 999
+3  8 996 995
+3  996 992 366
+3  1008 379 1010
+3  630 1915 2012
+3  1002 363 371
+3  383 89 365
+3  365 89 999
+3  1000 363 1001
+3  1000 1001 8
+3  98 363 1003
+3  558 208 426
+3  34 382 1004
+3  34 1005 1006
+3  1013 201 424
+3  424 201 425
+3  1015 378 1016
+3  1912 1008 1010
+3  358 1011 982
+3  89 1009 999
+3  70 378 1015
+3  376 1009 89
+3  651 987 1014
+3  382 651 1014
+3  382 993 1004
+3  1015 1016 379
+3  1029 391 392
+3  700 1889 701
+3  70 1015 1018
+3  70 1018 969
+3  1005 1031 368
+3  68 391 1029
+3  1913 1018 1008
+3  608 1027 1024
+3  2021 585 944
+3  380 1022 1021
+3  1958 664 302
+3  1025 1022 380
+3  1023 1024 69
+3  1908 1911 1006
+3  1006 1911 34
+3  1030 1028 1029
+3  392 393 1121
+3  1004 367 1005
+3  940 583 1243
+3  1207 1209 1208
+3  1037 1938 1939
+3  109 1033 1035
+3  222 1034 1033
+3  1841 670 1284
+3  2006 1341 2005
+3  387 1217 1216
+3  1213 1223 399
+3  231 1942 1899
+3  1211 127 1210
+3  916 573 1077
+3  1077 113 916
+3  919 113 920
+3  309 1082 1918
+3  1918 1919 309
+3  1841 1284 532
+3  542 1439 132
+3  1044 1045 1049
+3  131 753 1048
+3  509 1440 1439
+3  1440 758 1438
+3  1441 731 728
+3  1052 536 130
+3  511 1051 1052
+3  1051 1046 1047
+3  1616 1618 1923
+3  1613 1923 1924
+3  1053 512 1045
+3  513 1056 1054
+3  52 1055 648
+3  648 1055 647
+3  1855 1856 182
+3  1056 1057 538
+3  816 75 1058
+3  1048 1059 514
+3  536 1047 1060
+3  1062 1038 216
+3  508 819 539
+3  1063 586 946
+3  1026 586 1063
+3  1838 1836 1835
+3  232 1382 1383
+3  350 1808 860
+3  331 747 1345
+3  1458 1828 460
+3  1477 116 825
+3  1471 490 1470
+3  1466 1809 459
+3  459 1809 1457
+3  44 1463 466
+3  353 1454 857
+3  1793 1461 457
+3  189 1968 1967
+3  833 1474 903
+3  902 1469 1833
+3  1833 1469 1968
+3  908 883 1692
+3  1147 1687 515
+3  193 847 1395
+3  1114 1038 1641
+3  1123 1682 484
+3  1613 1922 1923
+3  1661 1535 1536
+3  1536 1537 1661
+3  1725 570 1727
+3  1858 920 1077
+3  396 1858 1077
+3  423 208 1728
+3  1643 781 159
+3  1641 1038 1062
+3  1880 1870 1879
+3  1536 1535 453
+3  1184 1861 1367
+3  1184 1367 1369
+3  561 755 1687
+3  1917 663 306
+3  953 1495 1695
+3  1064 1492 1065
+3  1064 1065 146
+3  1066 155 147
+3  1065 155 1066
+3  1563 1047 1058
+3  1298 103 1297
+3  402 947 1068
+3  1228 1068 400
+3  1228 400 1229
+3  1293 13 1292
+3  419 1988 569
+3  116 1070 1072
+3  1070 736 1072
+3  80 1076 415
+3  13 736 1071
+3  1295 1290 1292
+3  615 1074 1073
+3  1075 1074 616
+3  614 1075 616
+3  1074 1072 1073
+3  1515 415 1076
+3  1012 1078 201
+3  431 425 1088
+3  1245 845 1735
+3  310 1085 1084
+3  1035 215 1119
+3  1035 1119 1936
+3  1616 1922 579
+3  230 1036 1936
+3  1761 340 334
+3  306 1920 1919
+3  1089 1090 305
+3  431 1085 310
+3  1083 1084 1085
+3  431 19 1085
+3  1094 57 1095
+3  717 1352 718
+3  1918 306 1919
+3  60 1091 1920
+3  419 1182 534
+3  688 1688 1765
+3  1763 1761 333
+3  431 1087 19
+3  304 1089 1086
+3  1436 1437 1093
+3  303 1437 1436
+3  1090 1095 305
+3  1089 305 61
+3  1092 300 1093
+3  1092 1093 1437
+3  1321 142 1341
+3  303 1095 1090
+3  1099 793 795
+3  1100 593 594
+3  1093 300 1097
+3  1671 1672 798
+3  1095 57 316
+3  303 1094 1095
+3  1096 315 1337
+3  966 1328 1956
+3  1972 1128 1126
+3  1675 975 1674
+3  591 1180 1181
+3  590 1181 1180
+3  590 1180 65
+3  1587 5 1104
+3  636 593 1105
+3  1211 1210 1984
+3  1982 1104 1981
+3  1722 1716 832
+3  1112 152 787
+3  1609 929 2008
+3  2008 1594 1609
+3  1113 61 322
+3  433 1113 322
+3  2005 1341 1340
+3  310 1113 433
+3  1193 1187 1847
+3  218 1648 1344
+3  1649 240 263
+3  393 587 1121
+3  746 1742 961
+3  1776 1408 669
+3  669 1408 1407
+3  485 506 1775
+3  639 635 1127
+3  1131 971 1167
+3  141 971 1131
+3  630 2012 1257
+3  1163 641 1162
+3  1867 1129 1874
+3  1867 1872 640
+3  1167 1169 1168
+3  1171 970 137
+3  1523 1520 497
+3  1523 497 1499
+3  971 1169 1167
+3  1131 1130 141
+3  1482 1483 146
+3  971 1170 1169
+3  1165 139 143
+3  1882 167 1870
+3  970 1133 624
+3  1510 117 1511
+3  572 1686 94
+3  498 1523 1499
+3  1326 1733 844
+3  1331 1332 191
+3  1138 865 349
+3  437 64 1140
+3  821 1139 1137
+3  821 1137 1136
+3  1420 14 1238
+3  1248 56 1250
+3  56 1248 1247
+3  24 440 1818
+3  495 1821 496
+3  1747 124 262
+3  1964 384 1963
+3  1975 623 30
+3  1142 624 1133
+3  1145 518 526
+3  1144 1146 1145
+3  517 1146 1144
+3  1120 1156 769
+3  1783 765 1782
+3  174 787 1156
+3  1284 1283 134
+3  1151 1149 1148
+3  1148 1149 561
+3  1154 1145 526
+3  1151 529 1149
+3  1152 529 1151
+3  1155 528 1153
+3  1309 532 1308
+3  533 1067 1308
+3  1301 533 1302
+3  1719 1720 832
+3  1717 886 1418
+3  1660 1111 1657
+3  1747 660 124
+3  1719 894 1711
+3  1702 1723 906
+3  1706 1702 906
+3  1659 1658 1141
+3  1744 1658 1659
+3  261 1357 1358
+3  719 1952 263
+3  641 1159 1160
+3  1482 1161 1483
+3  146 1483 1064
+3  1066 1163 146
+3  1163 1162 146
+3  1164 143 1174
+3  1613 1925 1927
+3  1126 625 1973
+3  1485 1164 1486
+3  1159 1166 1160
+3  169 635 639
+3  1170 971 1173
+3  1172 645 138
+3  1325 953 1694
+3  196 1176 195
+3  1180 1179 592
+3  66 1101 1100
+3  1178 1102 1179
+3  592 1179 1102
+3  560 211 1406
+3  630 1257 629
+3  926 925 1608
+3  1647 1116 1646
+3  1647 1646 1117
+3  530 1283 1281
+3  23 1191 1850
+3  695 1373 1375
+3  1376 1737 296
+3  1345 323 1346
+3  762 521 763
+3  758 757 1438
+3  1279 530 1281
+3  1698 21 674
+3  1346 748 1345
+3  1348 1183 320
+3  1372 1371 324
+3  320 1368 1351
+3  1734 1733 1326
+3  1576 596 1575
+3  1575 596 1580
+3  1850 1195 23
+3  1186 1845 1846
+3  928 1592 1591
+3  1635 1271 1784
+3  1845 1185 1851
+3  325 1196 697
+3  1193 1846 1848
+3  1847 1846 1193
+3  1369 1370 1184
+3  1633 104 1281
+3  1843 1844 1851
+3  134 1285 1152
+3  1152 1285 529
+3  412 1192 23
+3  23 1192 1191
+3  427 428 50
+3  325 1197 1196
+3  1201 618 1199
+3  619 1426 576
+3  1200 1199 618
+3  1200 577 1199
+3  924 577 925
+3  1636 1101 66
+3  1636 66 1637
+3  921 922 1420
+3  1201 1203 618
+3  931 1604 1603
+3  494 1889 495
+3  937 936 1627
+3  1205 1204 31
+3  627 617 1206
+3  1206 620 627
+3  2016 2015 841
+3  1314 604 1315
+3  52 1739 1055
+3  1926 1927 1925
+3  1620 1619 941
+3  1422 1421 915
+3  381 2018 2017
+3  13 1293 735
+3  1480 1479 825
+3  387 1216 1215
+3  1228 402 1068
+3  1981 1104 1103
+3  1221 16 377
+3  89 1221 377
+3  399 1223 1222
+3  383 365 388
+3  383 388 1226
+3  1230 402 1227
+3  399 1222 1229
+3  1215 1231 387
+3  1215 1233 1231
+3  587 393 1241
+3  387 1231 1232
+3  1215 114 1233
+3  1241 393 1240
+3  113 918 917
+3  1211 573 1234
+3  1077 573 1211
+3  247 1566 1565
+3  1237 14 940
+3  940 394 1239
+3  1239 1237 940
+3  1626 930 1618
+3  576 918 610
+3  917 918 576
+3  31 1204 619
+3  43 1820 1824
+3  356 1934 1935
+3  1795 1456 1809
+3  1820 1821 1824
+3  1819 198 1816
+3  1819 199 1820
+3  965 964 1360
+3  1245 848 1246
+3  1252 705 493
+3  1247 1248 1249
+3  1249 1248 870
+3  298 1248 1251
+3  708 707 54
+3  707 706 54
+3  1253 56 1254
+3  1254 56 848
+3  295 1253 1254
+3  1338 1955 666
+3  1337 1338 666
+3  1665 1663 1600
+3  528 1152 1151
+3  1306 528 1155
+3  1307 134 1305
+3  533 1305 1304
+3  1285 134 1283
+3  765 530 1278
+3  83 1753 1752
+3  753 1050 523
+3  810 737 135
+3  1303 723 724
+3  1300 418 1262
+3  418 1300 1299
+3  374 416 1262
+3  153 1313 791
+3  374 1262 1312
+3  1109 1746 1108
+3  888 1265 1267
+3  1272 887 1264
+3  475 1269 1268
+3  1268 1269 826
+3  1271 1270 1269
+3  1273 1274 474
+3  1791 1557 1792
+3  1792 1554 1791
+3  826 1274 1865
+3  529 1783 1782
+3  1275 881 1274
+3  1416 1998 1999
+3  1789 1555 339
+3  765 1278 762
+3  762 1278 521
+3  764 1280 412
+3  521 1280 764
+3  1284 670 531
+3  529 1285 1783
+3  1911 824 34
+3  798 1667 1668
+3  1674 1667 798
+3  1906 1912 1010
+3  603 553 606
+3  1236 1237 581
+3  1679 797 553
+3  1639 601 1640
+3  1638 601 1639
+3  1102 1639 592
+3  1639 1102 1637
+3  1362 191 1336
+3  1668 1670 798
+3  1433 86 262
+3  1986 1689 1260
+3  1986 1260 1989
+3  1434 250 26
+3  1681 150 1667
+3  1682 1684 1683
+3  1750 684 1360
+3  333 1761 1760
+3  1914 600 632
+3  632 631 1914
+3  913 1412 1177
+3  263 1951 1948
+3  1344 1319 166
+3  197 221 1430
+3  1290 1289 725
+3  102 726 1288
+3  1070 1071 736
+3  13 1294 1292
+3  1300 417 671
+3  613 32 1480
+3  1291 1295 952
+3  726 735 1293
+3  1299 671 103
+3  1261 1300 1262
+3  403 1654 391
+3  1302 1304 723
+3  1607 1605 926
+3  935 577 1200
+3  1608 577 935
+3  1302 533 1304
+3  723 1306 1155
+3  1152 528 1305
+3  1306 1305 528
+3  1304 1305 1306
+3  1298 1299 103
+3  1310 1299 1298
+3  950 1310 1298
+3  950 949 1311
+3  950 1311 1310
+3  791 1313 790
+3  790 1313 242
+3  525 678 1902
+3  1314 1315 605
+3  607 1024 605
+3  605 1315 607
+3  981 982 99
+3  1237 1236 1238
+3  1935 1934 355
+3  985 1990 649
+3  1970 1990 211
+3  1989 1258 1986
+3  1781 1780 1588
+3  142 1324 1325
+3  574 1318 916
+3  1318 1317 916
+3  619 1203 1202
+3  1425 619 1202
+3  1320 166 1319
+3  1592 1595 2007
+3  1320 1322 158
+3  196 1431 1176
+3  1321 1323 142
+3  1428 1286 1429
+3  100 1428 1429
+3  1021 1023 69
+3  1326 1736 1734
+3  666 1329 1363
+3  1955 1329 666
+3  1254 848 1737
+3  1328 966 58
+3  2016 841 1332
+3  1328 1957 1956
+3  1738 1023 1022
+3  1334 1333 572
+3  1334 572 668
+3  667 1334 668
+3  1329 1330 1363
+3  667 1337 1336
+3  1096 1337 667
+3  2006 1321 1341
+3  1344 782 1319
+3  1391 237 238
+3  1861 1364 1862
+3  1364 1843 1862
+3  1349 1346 1348
+3  748 1346 1349
+3  1905 1007 1906
+3  718 1352 264
+3  1359 293 1354
+3  1357 1355 1358
+3  129 1359 1354
+3  686 1758 1759
+3  1759 1758 1762
+3  1358 59 660
+3  351 1381 1380
+3  305 316 1361
+3  1361 61 305
+3  322 61 1361
+3  1655 801 677
+3  1057 513 1855
+3  321 319 1366
+3  1366 319 93
+3  1855 679 1856
+3  93 1365 1366
+3  1373 324 1375
+3  320 1183 1368
+3  1370 324 1371
+3  1371 1184 1370
+3  1373 695 696
+3  1375 1374 694
+3  695 1375 694
+3  843 1379 1378
+3  1380 839 840
+3  2015 2016 352
+3  839 192 840
+3  1400 846 839
+3  1382 232 233
+3  233 165 1382
+3  236 1384 1382
+3  164 1383 1384
+3  1383 277 562
+3  1 1386 1387
+3  236 1389 1388
+3  1 1387 253
+3  1388 1389 237
+3  237 1390 238
+3  1387 237 1391
+3  571 234 1394
+3  1394 235 571
+3  836 835 1401
+3  836 1396 837
+3  1397 346 1398
+3  838 1398 1399
+3  1399 1398 346
+3  193 1401 875
+3  1396 1401 193
+3  836 1401 1396
+3  835 875 1401
+3  949 948 1405
+3  1403 373 416
+3  374 1403 416
+3  365 1404 559
+3  365 91 1404
+3  90 1405 948
+3  1894 1892 750
+3  1355 1951 264
+3  888 1267 1864
+3  1122 1407 501
+3  1409 499 1860
+3  1530 1543 1529
+3  1529 450 1530
+3  1804 1806 1805
+3  485 1775 1777
+3  1550 1549 1507
+3  483 1410 118
+3  1860 501 1409
+3  503 500 85
+3  1419 892 472
+3  1995 1413 1993
+3  922 14 1420
+3  1318 1422 915
+3  921 1423 1424
+3  922 1424 575
+3  1426 619 1425
+3  1426 1425 574
+3  917 1426 574
+3  1427 273 51
+3  286 1427 51
+3  144 1286 1325
+3  1430 1431 196
+3  1434 1433 262
+3  1746 473 1108
+3  1449 477 1468
+3  1446 478 1445
+3  467 1443 1467
+3  1448 190 1447
+3  855 1475 96
+3  1475 883 96
+3  458 1454 353
+3  1455 190 1448
+3  1474 1470 1469
+3  452 1538 1530
+3  370 1909 1908
+3  1460 459 1458
+3  1448 466 1461
+3  722 1464 1459
+3  1542 722 1459
+3  1539 1540 453
+3  1460 1466 459
+3  1460 1446 1466
+3  456 458 1806
+3  1797 1541 1810
+3  1472 856 354
+3  613 1480 614
+3  1707 1708 831
+3  1481 1479 1480
+3  1481 399 1479
+3  1213 399 1481
+3  2021 944 586
+3  1561 581 1560
+3  1561 1234 1235
+3  1603 1605 931
+3  701 1889 494
+3  1971 897 2022
+3  2022 470 1971
+3  1611 927 1612
+3  2020 586 1026
+3  587 1625 6
+3  389 395 1853
+3  981 99 1769
+3  35 1630 589
+3  1622 35 1623
+3  1640 604 946
+3  946 592 1640
+3  582 937 1627
+3  588 1498 1630
+3  1315 9 607
+3  589 1498 1586
+3  786 155 1493
+3  1492 1064 1491
+3  1493 155 1065
+3  1490 1495 775
+3  596 1581 1580
+3  1615 1589 934
+3  127 393 1853
+3  1795 1455 1794
+3  582 1620 1621
+3  1604 578 1603
+3  928 1926 1928
+3  1607 1606 1605
+3  933 1582 1498
+3  937 609 1601
+3  1601 936 937
+3  699 1889 700
+3  2011 1594 2010
+3  1605 1606 931
+3  590 1586 1585
+3  152 1569 789
+3  933 934 1589
+3  1501 1514 1500
+3  1495 1324 1496
+3  1160 145 1485
+3  2017 2018 1026
+3  1581 1582 1580
+3  1378 1379 2015
+3  1162 1484 1482
+3  1486 1164 1488
+3  1488 1487 1486
+3  1487 1488 140
+3  1486 1161 1485
+3  1623 35 1624
+3  2018 2019 2020
+3  2020 2019 6
+3  1602 578 1596
+3  582 1627 1619
+3  1210 127 395
+3  926 1603 1602
+3  1597 926 1602
+3  1030 1025 380
+3  220 1697 221
+3  1607 1608 935
+3  611 175 610
+3  925 577 1608
+3  622 623 214
+3  621 622 620
+3  1889 43 495
+3  587 584 1625
+3  1205 1206 617
+3  1209 1985 114
+3  1984 1985 1209
+3  1519 498 1511
+3  1561 1560 1240
+3  52 679 1739
+3  1906 1027 1905
+3  1164 1174 1488
+3  1915 630 1914
+3  1486 1487 1161
+3  1914 631 1915
+3  2013 2007 929
+3  1579 599 1915
+3  1484 1485 1161
+3  1168 1485 145
+3  1164 1485 1168
+3  1491 1489 1490
+3  144 1325 1694
+3  775 1496 1494
+3  1487 1483 1161
+3  1490 775 1491
+3  953 1324 1495
+3  1493 785 786
+3  1494 1323 774
+3  785 1493 0
+3  1493 1492 0
+3  1494 1492 775
+3  1323 1496 1324
+3  1494 1496 1323
+3  1504 498 1503
+3  502 1500 481
+3  502 1501 1500
+3  1686 341 94
+3  98 1003 1729
+3  1509 48 442
+3  1076 1512 1513
+3  1509 476 1506
+3  1506 476 1551
+3  1910 551 1909
+3  444 1545 446
+3  1532 1528 1507
+3  1507 1549 1532
+3  1553 500 481
+3  481 1527 1553
+3  1505 1518 1512
+3  449 1505 1512
+3  1519 480 1520
+3  1510 476 1509
+3  1525 1521 118
+3  1517 1502 415
+3  1516 1512 1076
+3  449 1512 1516
+3  441 440 76
+3  1510 1509 442
+3  1519 1520 1523
+3  498 1519 1523
+3  1525 118 1526
+3  1410 1411 118
+3  479 482 1522
+3  1524 1511 498
+3  1504 1524 498
+3  497 1520 1525
+3  1860 499 1859
+3  499 1527 1526
+3  1526 1411 499
+3  1686 572 1333
+3  1534 452 1533
+3  1533 452 1530
+3  954 1531 451
+3  448 1531 954
+3  1465 448 954
+3  1662 337 880
+3  1885 722 1884
+3  1543 1544 1529
+3  1146 518 1145
+3  1804 200 1802
+3  484 1772 1407
+3  75 1060 1563
+3  1801 1462 1799
+3  1047 1563 1060
+3  1532 1549 449
+3  446 1545 78
+3  1547 48 1548
+3  1547 1548 445
+3  449 1550 1505
+3  449 1549 1550
+3  499 1553 1527
+3  683 1558 344
+3  1749 330 1757
+3  1557 107 1555
+3  1561 1235 581
+3  1794 1796 1795
+3  1488 1174 140
+3  1562 1542 1459
+3  1829 189 1442
+3  1474 902 903
+3  1593 2009 2008
+3  471 1834 1832
+3  86 1432 1567
+3  594 1671 1100
+3  1567 248 1566
+3  1854 1509 1506
+3  932 1589 1615
+3  1615 1614 928
+3  1923 1922 1616
+3  1566 247 1567
+3  246 1565 1569
+3  1638 1888 800
+3  1603 578 1602
+3  934 579 1615
+3  1596 927 1597
+3  1926 1925 1612
+3  45 199 1823
+3  1571 247 1565
+3  1571 1565 246
+3  1595 1592 928
+3  1593 578 1604
+3  1574 1576 1575
+3  1586 580 1585
+3  1574 598 1576
+3  1581 595 1584
+3  597 1578 638
+3  597 636 1577
+3  631 598 1579
+3  1769 979 980
+3  1577 1583 596
+3  1498 580 1586
+3  1584 580 1582
+3  1581 1584 1582
+3  590 589 1586
+3  1769 99 979
+3  580 1584 1585
+3  1590 932 1591
+3  1591 41 1590
+3  1589 1779 1780
+3  344 1790 1657
+3  1575 1588 41
+3  1780 1779 1588
+3  2010 1604 931
+3  1613 1927 1614
+3  841 2015 192
+3  192 2015 840
+3  599 1579 1573
+3  1597 1598 925
+3  1606 213 1257
+3  582 1621 938
+3  1603 926 1605
+3  931 1606 1257
+3  1600 924 1665
+3  1601 923 1600
+3  1602 1596 1597
+3  931 2011 2010
+3  1257 2011 931
+3  2009 1594 2008
+3  1608 1607 926
+3  1664 936 1601
+3  1928 927 1596
+3  936 29 1611
+3  1926 927 1928
+3  1619 1626 1618
+3  352 1378 2015
+3  1379 840 2015
+3  588 1617 934
+3  1628 1629 1620
+3  1617 588 1628
+3  1628 588 1629
+3  941 1616 1617
+3  1617 1616 579
+3  1617 579 934
+3  1621 1629 1630
+3  1621 35 1622
+3  938 1621 1622
+3  1621 1630 35
+3  583 938 1622
+3  1620 1629 1621
+3  1623 584 1243
+3  534 1632 1631
+3  670 1634 1631
+3  1631 1632 670
+3  670 1632 531
+3  1118 1842 217
+3  1633 531 1632
+3  1632 534 1633
+3  1182 1633 534
+3  1182 104 1633
+3  1908 1909 1911
+3  1790 1787 1657
+3  1309 671 1841
+3  1638 66 1888
+3  1887 66 1100
+3  1888 1886 800
+3  1639 1640 592
+3  1657 1111 344
+3  1537 1771 1661
+3  452 1661 1771
+3  218 1647 1648
+3  1648 782 1344
+3  1652 1651 88
+3  241 1651 1652
+3  240 1649 1651
+3  88 246 1652
+3  1653 242 1313
+3  241 1652 1653
+3  1587 1983 595
+3  342 340 341
+3  1859 499 1411
+3  1676 602 800
+3  1859 1411 1410
+3  1855 513 679
+3  1659 1267 1266
+3  475 1267 1659
+3  1745 188 1744
+3  1744 188 1658
+3  468 1111 1660
+3  1110 468 1660
+3  1767 488 1124
+3  29 936 1664
+3  975 1666 1667
+3  1682 1683 535
+3  1569 1570 789
+3  1671 798 1670
+3  975 1667 1674
+3  167 15 1673
+3  452 1535 1661
+3  1676 800 1669
+3  974 975 1675
+3  1680 603 606
+3  1681 1667 1666
+3  1666 794 1681
+3  1681 794 1678
+3  488 1061 1683
+3  1683 1061 535
+3  1505 1685 1518
+3  1505 1504 1685
+3  343 341 1686
+3  1141 1787 1786
+3  1687 755 1916
+3  569 567 1724
+3  833 1690 1473
+3  1691 491 1473
+3  1690 1691 1473
+3  893 1714 1715
+3  1691 1692 491
+3  891 1692 1691
+3  891 908 1692
+3  158 773 1704
+3  1456 1796 1797
+3  1885 1535 452
+3  1535 1885 1884
+3  1007 1907 1006
+3  1906 1007 1912
+3  221 1697 100
+3  674 1700 1699
+3  1723 891 1690
+3  1700 1701 1699
+3  1699 1701 226
+3  1414 1703 890
+3  1711 1707 895
+3  906 1723 1690
+3  1070 1069 1705
+3  1705 1071 1070
+3  1706 906 1707
+3  1709 831 1708
+3  896 1712 471
+3  1710 1703 1702
+3  895 1707 906
+3  1712 1711 895
+3  1708 1711 894
+3  894 1719 1718
+3  891 1723 1702
+3  894 1713 1708
+3  1712 895 471
+3  1713 1709 1708
+3  1724 206 568
+3  1716 1715 832
+3  1714 894 1718
+3  1721 896 471
+3  893 1418 472
+3  1717 1418 893
+3  1715 1717 893
+3  1831 1833 1830
+3  1721 898 1720
+3  1712 1719 1711
+3  1718 1719 832
+3  898 1721 897
+3  1020 1010 379
+3  373 372 1725
+3  384 1964 385
+3  548 649 1970
+3  1023 605 1024
+3  1738 605 1023
+3  281 279 177
+3  281 177 285
+3  10 278 281
+3  105 1761 1763
+3  328 105 1764
+3  1743 1035 1033
+3  1924 930 1612
+3  1659 1266 1745
+3  1659 1745 1744
+3  1749 1748 685
+3  1750 1749 685
+3  1750 1360 1749
+3  1752 1753 465
+3  186 1755 463
+3  464 465 1753
+3  1555 107 1766
+3  1754 1755 464
+3  464 1755 1756
+3  1763 1762 687
+3  536 1767 130
+3  1688 687 1765
+3  334 1766 1760
+3  1556 1766 334
+3  333 1762 1763
+3  687 1758 1765
+3  1767 1124 647
+3  647 130 1767
+3  1982 591 1181
+3  1538 1771 1537
+3  1649 263 1948
+3  1775 506 1774
+3  1776 506 503
+3  85 1776 503
+3  669 1773 1774
+3  1777 508 539
+3  1590 1779 932
+3  1787 1141 1657
+3  1789 1635 1788
+3  1141 1786 1784
+3  1141 1784 1785
+3  1554 1788 1787
+3  1789 1788 1554
+3  1790 1554 1787
+3  1792 1557 1555
+3  1792 1555 1789
+3  1554 1792 1789
+3  1799 455 1541
+3  1541 455 1540
+3  1796 1794 457
+3  1462 456 1800
+3  1803 455 1802
+3  1813 1812 454
+3  1812 1803 454
+3  1802 1800 1804
+3  456 1804 1800
+3  1804 1805 200
+3  45 350 496
+3  1813 1814 447
+3  1816 1815 46
+3  1813 454 1814
+3  199 45 1821
+3  447 1815 446
+3  447 1814 1815
+3  198 545 1818
+3  1818 545 24
+3  1818 440 77
+3  200 1822 454
+3  199 1819 1822
+3  43 1824 495
+3  1825 1456 1797
+3  1826 1827 1457
+3  1967 1968 1467
+3  1830 1829 901
+3  1467 1968 1469
+3  900 1831 1830
+3  1833 189 1830
+3  1836 1837 822
+3  820 1837 1836
+3  822 1837 823
+3  349 823 1837
+3  838 1381 1835
+3  1381 351 1835
+3  843 1839 1379
+3  1380 1839 351
+3  1379 1839 1380
+3  843 851 1839
+3  1115 1842 1114
+3  1848 207 1195
+3  1848 1195 1849
+3  1193 1848 1849
+3  1863 628 214
+3  1198 1844 1197
+3  1198 207 1844
+3  1198 1195 207
+3  560 549 1970
+3  1970 211 560
+3  1970 549 548
+3  1987 1258 1988
+3  2009 1604 2010
+3  2010 1594 2009
+3  1856 677 1857
+3  1855 182 1057
+3  978 1974 1975
+3  1975 624 978
+3  826 1865 1268
+3  1268 1865 1864
+3  1673 1871 167
+3  1869 169 639
+3  1877 1869 639
+3  1870 167 1871
+3  1879 1870 169
+3  1876 626 1868
+3  1873 641 1163
+3  1877 1868 640
+3  1878 640 1873
+3  1869 1879 169
+3  1879 1869 1878
+3  1883 977 167
+3  147 1881 1066
+3  1881 1883 1882
+3  147 1883 1881
+3  62 1881 1880
+3  1883 147 148
+3  167 1882 1883
+3  1905 1027 370
+3  1886 799 1670
+3  1886 1670 1669
+3  800 1886 1669
+3  1886 1887 799
+3  699 43 1889
+3  1890 749 1891
+3  1890 1891 750
+3  1893 173 1892
+3  751 1893 1892
+3  111 1942 1943
+3  220 1896 1895
+3  219 1898 1116
+3  1900 231 1899
+3  1896 1899 111
+3  1901 1900 675
+3  370 1908 1907
+3  1908 1006 1907
+3  1909 824 1911
+3  1958 60 664
+3  1007 1913 1912
+3  630 600 1914
+3  2012 2011 1257
+3  1579 1915 631
+3  230 1936 1119
+3  1921 355 657
+3  123 1921 657
+3  1921 123 1917
+3  1921 1917 311
+3  928 1928 1595
+3  1932 492 654
+3  125 1932 654
+3  1244 1932 357
+3  492 1932 1244
+3  1080 1933 356
+3  1932 1933 357
+3  125 1933 1932
+3  1036 1938 1937
+3  1938 1037 1040
+3  1037 1939 1940
+3  1938 1040 109
+3  231 1939 1941
+3  111 1943 1898
+3  1942 230 1943
+3  1945 1944 21
+3  1650 1948 259
+3  1949 128 1950
+3  1956 1957 1327
+3  1338 966 1955
+3  1376 1957 1328
+3  1329 1955 1954
+3  1956 1327 1954
+3  1958 1961 60
+3  301 1960 1962
+3  301 1962 968
+3  1962 299 968
+3  1963 1726 372
+3  384 1726 1963
+3  1967 1443 1442
+3  1729 1964 98
+3  189 1967 1442
+3  372 1966 1965
+3  1194 1191 1969
+3  1190 1194 1969
+3  1443 1967 1467
+3  470 904 1971
+3  1972 978 1128
+3  1979 911 882
+3  1979 882 1977
+3  1979 910 911
+3  1232 389 1980
+3  390 1232 1980
+3  1853 126 1980
+3  1980 389 1853
+3  1981 1178 591
+3  595 1983 1181
+3  591 1982 1981
+3  1984 1210 1985
+3  395 1233 1985
+3  1985 1233 114
+3  1970 649 1990
+3  419 1259 1987
+3  1406 1991 651
+3  650 651 1991
+3  1990 650 1991
+3  1978 830 1997
+3  1413 1992 1993
+3  2000 1998 1416
+3  2002 880 1994
+3  2002 1994 2001
+3  1994 880 1997
+3  2002 1417 1662
+3  1415 2003 889
+3  2000 884 1998
+3  1998 1992 1415
+3  2000 1417 2001
+3  1992 1413 2003
+3  1415 1992 2003
+3  2004 1340 1428
+3  1342 2004 1428
+3  166 2006 2005
+3  166 1320 2006
+3  2007 1593 2008
+3  2007 2008 929
+3  1592 2013 1591
+3  1592 2007 2013
+3  41 1591 2014
+3  2014 1591 2013
+3  929 2014 2013
+3  2014 929 1609
+3  2014 1609 1573
+3  2017 1026 1025
+3  381 1029 2019
+3  2020 2021 586
+
diff -pruN 5.0.3-1/examples/BGL_graphcut/face_selection_borders_regularization_example.cpp 5.1-1/examples/BGL_graphcut/face_selection_borders_regularization_example.cpp
--- 5.0.3-1/examples/BGL_graphcut/face_selection_borders_regularization_example.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/BGL_graphcut/face_selection_borders_regularization_example.cpp	2020-09-08 12:12:19.000000000 +0000
@@ -0,0 +1,52 @@
+#include <CGAL/Simple_cartesian.h>
+#include <CGAL/Surface_mesh.h>
+#include <CGAL/boost/graph/selection.h>
+
+#include <fstream>
+#include <iostream>
+
+using Kernel = CGAL::Simple_cartesian<double>;
+using Mesh = CGAL::Surface_mesh<Kernel::Point_3>;
+using Face_index = Mesh::Face_index;
+
+int main(int argc, char** argv)
+{
+  std::ifstream in((argc>1) ? argv[1] : "data/blobby.off");
+
+  if(!in)
+  {
+    std::cerr << "Error: could not read input file" << std::endl;
+    return EXIT_FAILURE;
+  }
+
+  Mesh mesh;
+  CGAL::read_off (in, mesh);
+
+  boost::unordered_map<Face_index, bool> is_selected_map;
+
+  // randomly select 1/3 of faces
+  std::size_t nb_selected_before = 0;
+  CGAL::Random rand;
+  for (Face_index fi : faces(mesh))
+  {
+    bool selected = (rand.get_double() < 1. / 3.);
+    is_selected_map[fi] = selected;
+    if (selected)
+      nb_selected_before ++;
+  }
+
+  std::cerr << nb_selected_before << " selected before regularization" << std::endl;
+
+  CGAL::regularize_face_selection_borders (mesh,
+                                           boost::make_assoc_property_map(is_selected_map),
+                                           0.5); // using weight = 0.5
+
+  std::size_t nb_selected_after = 0;
+  for (const auto& sel : is_selected_map)
+    if (sel.second)
+      ++ nb_selected_after;
+
+  std::cerr << nb_selected_after << " selected after regularization" << std::endl;
+
+  return EXIT_SUCCESS;
+}
diff -pruN 5.0.3-1/examples/BGL_LCC/copy_lcc.cpp 5.1-1/examples/BGL_LCC/copy_lcc.cpp
--- 5.0.3-1/examples/BGL_LCC/copy_lcc.cpp	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/examples/BGL_LCC/copy_lcc.cpp	2020-09-08 12:12:19.000000000 +0000
@@ -55,6 +55,7 @@ int main(int argc, char* argv[])
     CGAL::copy_face_graph(T1, S, CGAL::parameters::vertex_to_vertex_output_iterator(std::inserter(v2v, v2v.end()))
                                                   .halfedge_to_halfedge_output_iterator(std::inserter(h2h, h2h.end())));
     std::ofstream out("reverse.off");
+    out.precision(17);
     out << S;
   }
   return 0;
diff -pruN 5.0.3-1/examples/BGL_surface_mesh/write_inp.cpp 5.1-1/examples/BGL_surface_mesh/write_inp.cpp
--- 5.0.3-1/examples/BGL_surface_mesh/write_inp.cpp	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/examples/BGL_surface_mesh/write_inp.cpp	2020-09-08 12:12:19.000000000 +0000
@@ -20,6 +20,7 @@ int main()
   CGAL::make_quad(Point(0,0,1), Point(1,0,1),Point(1,1,1),Point(0,1,1), sm);
 
   std::ofstream out("out.inp");
+  out.precision(17);
   CGAL::write_inp(out, sm, "out.inp", "S4R");
   return 0;
 }
diff -pruN 5.0.3-1/examples/BGL_triangulation_2/emst_regular.cpp 5.1-1/examples/BGL_triangulation_2/emst_regular.cpp
--- 5.0.3-1/examples/BGL_triangulation_2/emst_regular.cpp	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/examples/BGL_triangulation_2/emst_regular.cpp	2020-09-08 12:12:19.000000000 +0000
@@ -2,8 +2,8 @@
 
 #include <CGAL/Regular_triangulation_2.h>
 #include <CGAL/boost/graph/graph_traits_Regular_triangulation_2.h>
-#include <CGAL/internal/boost/function_property_map.hpp>
 
+#include <boost/property_map/function_property_map.hpp>
 #include <boost/graph/kruskal_min_spanning_tree.hpp>
 
 #include <fstream>
@@ -75,7 +75,7 @@ int main(int argc,char* argv[])
   std::list<edge_descriptor> mst;
   boost::kruskal_minimum_spanning_tree(tr, std::back_inserter(mst),
                                        vertex_index_map(vertex_index_pmap)
-                                       .weight_map(CGAL::internal::boost_::make_function_property_map<
+                                       .weight_map(boost::make_function_property_map<
                                           edge_descriptor, FT, Edge_weight_functor>(Edge_weight_functor(tr))));
 
   std::cout << "The edges of the Euclidean mimimum spanning tree:" << std::endl;
diff -pruN 5.0.3-1/examples/Classification/CMakeLists.txt 5.1-1/examples/Classification/CMakeLists.txt
--- 5.0.3-1/examples/Classification/CMakeLists.txt	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/examples/Classification/CMakeLists.txt	2020-09-08 12:12:19.000000000 +0000
@@ -5,7 +5,6 @@ cmake_minimum_required(VERSION 3.1...3.1
 project( Classification_Examples )
 
 
-
 # CGAL and its components
 find_package( CGAL QUIET COMPONENTS  )
 
@@ -22,127 +21,82 @@ if ( NOT Boost_FOUND )
   return()
 endif()
 
+set(Classification_dependencies_met TRUE)
+
 find_package( Boost OPTIONAL_COMPONENTS serialization iostreams )
+include(CGAL_Boost_serialization_support)
+include(CGAL_Boost_iostreams_support)
 
-if( WIN32 )
-# to avoid a warning with old cmake
-  set(_Boost_BZIP2_HEADERS             "boost/iostreams/filter/bzip2.hpp")
-  set(_Boost_ZLIB_HEADERS              "boost/iostreams/filter/zlib.hpp")
-  find_package( Boost OPTIONAL_COMPONENTS bzip2 zlib)
-else()
-  find_package(ZLIB QUIET)
+if (NOT TARGET CGAL::Boost_serialization_support)
+  message(STATUS "NOTICE: This project requires Boost Serialization, and will not be compiled.")
+  set(Classification_dependencies_met FALSE)
+endif()
+if (NOT TARGET CGAL::Boost_iostreams_support)
+  message(STATUS "NOTICE: This project requires Boost IO Streams, and will not be compiled.")
+  set(Classification_dependencies_met FALSE)
 endif()
 
-find_package(TBB QUIET)
-
 find_package(OpenCV QUIET COMPONENTS core ml) # Need core + machine learning
-
-# Use Eigen
-find_package(Eigen3 3.1.0 REQUIRED) #(3.1.0 or greater)
-if (NOT EIGEN3_FOUND)
-  message(STATUS "This project requires the Eigen library, and will not be compiled.")
-  return()
-else()
-  include( ${EIGEN3_USE_FILE} )
+include(CGAL_OpenCV_support)
+if (NOT TARGET CGAL::OpenCV_support)
+  message(STATUS "NOTICE: OpenCV was not found. OpenCV random forest predicate for classification won't be available.")
 endif()
 
-
-# Creating entries for all C++ files with "main" routine
-# ##########################################################
-
-# Classification examples
-set(targets
-  example_classification
-  example_ethz_random_forest
-  example_feature
-  example_generation_and_training
-  example_mesh_classification
-  example_cluster_classification)
-
-# Classification requires some C++11 features
-set(needed_cxx_features cxx_rvalue_references cxx_variadic_templates)
-
-# Libraries and flags
-set(classification_linked_libraries)
-set(classification_compile_definitions)
-
-
-if (Boost_SERIALIZATION_FOUND AND Boost_IOSTREAMS_FOUND)
-  if(TARGET Boost::serialization AND TARGET Boost::iostreams)
-    set(classification_linked_libraries Boost::serialization Boost::iostreams)
-  else()
-    set(classification_linked_libraries ${classification_linked_libraries}
-      ${Boost_SERIALIZATION_LIBRARY}
-      ${Boost_IOSTREAMS_LIBRARY})
-  endif()
-else()
-  if (NOT Boost_SERIALIZATION_FOUND)
-    message(STATUS "NOTICE: This project requires Boost Serialization, and will not be compiled.")
-  endif()
-  if (NOT Boost_IOSTREAMS_FOUND)
-    message(STATUS "NOTICE: This project requires Boost IO Streams, and will not be compiled.")
-  endif()
-  return()
+find_package(TensorFlow QUIET)
+include(CGAL_TensorFlow_support)
+if (NOT TARGET CGAL::TensorFlow_support)
+  message(STATUS "NOTICE: TensorFlow was not found. TensorFlow neural network predicate for classification won't be available.")
 endif()
 
-if( WIN32 )
-  if (Boost_ZLIB_FOUND AND Boost_BZIP2_FOUND)
-    set(classification_linked_libraries ${classification_linked_libraries}
-      ${Boost_ZLIB_LIBRARY} ${Boost_BZIP2_LIBRARY})
-  else()
-    message(STATUS "NOTICE: This project requires Boost ZLIB and Boost BZIP2, and will not be compiled.")
-    return()
-  endif()
-else()
-  if(ZLIB_FOUND)
-    set(classification_linked_libraries ${classification_linked_libraries} ZLIB::ZLIB)
-  else()
-    message(STATUS "NOTICE: This project requires ZLIB, and will not be compiled.")
-    return()
-  endif()
+find_package(Eigen3 3.1.0 REQUIRED) #(3.1.0 or greater)
+include(CGAL_Eigen_support)
+if (NOT TARGET CGAL::Eigen_support)
+  message(STATUS "This project requires the Eigen library, and will not be compiled.")
+  set(Classification_dependencies_met FALSE)
 endif()
 
-if (OpenCV_FOUND)
-  message(STATUS "Found OpenCV ${OpenCV_VERSION}")
-  include_directories(${OpenCV_INCLUDE_DIRS})
-  set(classification_linked_libraries ${classification_linked_libraries}
-    ${OpenCV_LIBS})
-  set(classification_compile_definitions ${classification_compile_definitions}
-    "-DCGAL_LINKED_WITH_OPENCV")
-
-  set(targets ${targets} example_opencv_random_forest)
-else()
-  message(STATUS "NOTICE: OpenCV was not found. OpenCV random forest predicate for classification won't be available.")
-endif()
+find_package(TBB QUIET)
+include(CGAL_TBB_support)
 
-find_package(TensorFlow QUIET)
-if (TensorFlow_FOUND)
-  message(STATUS "Found TensorFlow")
-  include_directories( ${TensorFlow_INCLUDE_DIR} )
-  set(classification_linked_libraries ${classification_linked_libraries}
-    ${TensorFlow_LIBRARY})
-  set(classification_compile_definitions ${classification_compile_definitions}
-    "-DCGAL_LINKED_WITH_TENSORFLOW")
-
-  set(targets ${targets} example_tensorflow_neural_network)
-else()
-  message(STATUS "NOTICE: TensorFlow not found, Neural Network predicate for classification won't be available.")
+if (NOT Classification_dependencies_met)
+  return()
 endif()
 
-
-# Creating targets with correct libraries and flags
-foreach(target ${targets})
-  create_single_source_cgal_program( "${target}.cpp" CXX_FEATURES ${needed_cxx_features} )
+create_single_source_cgal_program( "example_classification.cpp" )
+create_single_source_cgal_program( "example_ethz_random_forest.cpp" )
+create_single_source_cgal_program( "example_feature.cpp" )
+create_single_source_cgal_program( "example_generation_and_training.cpp" )
+create_single_source_cgal_program( "example_mesh_classification.cpp" )
+create_single_source_cgal_program( "example_cluster_classification.cpp" )
+create_single_source_cgal_program( "gis_tutorial_example.cpp" )
+
+if (TARGET CGAL::OpenCV_support)
+  create_single_source_cgal_program( "example_opencv_random_forest.cpp" )
+  target_link_libraries(example_opencv_random_forest PUBLIC CGAL::OpenCV_support)
+endif()
+
+if (TARGET CGAL::TensorFlow_support)
+  create_single_source_cgal_program( "example_tensorflow_neural_network.cpp" )
+  target_link_libraries(example_opencv_random_forest PUBLIC CGAL::TensorFlow_support)
+endif()
+
+foreach(target
+    example_classification
+    example_ethz_random_forest
+    example_feature
+    example_generation_and_training
+    example_mesh_classification
+    example_cluster_classification
+    example_opencv_random_forest
+    example_tensorflow_neural_network
+    gis_tutorial_example)
   if(TARGET ${target})
-    target_link_libraries(${target} PUBLIC ${classification_linked_libraries})
-    target_compile_definitions(${target} PUBLIC ${classification_compile_definitions})
-    if(TBB_FOUND)
-      CGAL_target_use_TBB(${target})
+    target_link_libraries(${target} PUBLIC
+      CGAL::Eigen_support
+      CGAL::Boost_iostreams_support
+      CGAL::Boost_serialization_support)
+    if(TARGET CGAL::TBB_support)
+      target_link_libraries(${target} PUBLIC CGAL::TBB_support)
     endif()
   endif()
 endforeach()
-
-
-
-
-
diff -pruN 5.0.3-1/examples/Classification/example_classification.cpp 5.1-1/examples/Classification/example_classification.cpp
--- 5.0.3-1/examples/Classification/example_classification.cpp	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/examples/Classification/example_classification.cpp	2020-09-08 12:12:19.000000000 +0000
@@ -11,16 +11,12 @@
 #include <CGAL/Simple_cartesian.h>
 #include <CGAL/Classification.h>
 #include <CGAL/bounding_box.h>
-
+#include <CGAL/tags.h>
 #include <CGAL/IO/read_ply_points.h>
 
 #include <CGAL/Real_timer.h>
 
-#ifdef CGAL_LINKED_WITH_TBB
-typedef CGAL::Parallel_tag Concurrency_tag;
-#else
-typedef CGAL::Sequential_tag Concurrency_tag;
-#endif
+typedef CGAL::Parallel_if_available_tag Concurrency_tag;
 
 typedef CGAL::Simple_cartesian<double> Kernel;
 typedef Kernel::Point_3 Point;
diff -pruN 5.0.3-1/examples/Classification/example_cluster_classification.cpp 5.1-1/examples/Classification/example_cluster_classification.cpp
--- 5.0.3-1/examples/Classification/example_cluster_classification.cpp	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/examples/Classification/example_cluster_classification.cpp	2020-09-08 12:12:19.000000000 +0000
@@ -18,11 +18,7 @@
 #include <CGAL/Shape_detection/Region_growing.h>
 #include <CGAL/Real_timer.h>
 
-#ifdef CGAL_LINKED_WITH_TBB
-typedef CGAL::Parallel_tag   Concurrency_tag;
-#else
-typedef CGAL::Sequential_tag Concurrency_tag;
-#endif
+typedef CGAL::Parallel_if_available_tag Concurrency_tag;
 
 typedef CGAL::Simple_cartesian<double> Kernel;
 typedef Kernel::Point_3                Point;
diff -pruN 5.0.3-1/examples/Classification/gis_tutorial_example.cmd 5.1-1/examples/Classification/gis_tutorial_example.cmd
--- 5.0.3-1/examples/Classification/gis_tutorial_example.cmd	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Classification/gis_tutorial_example.cmd	2020-09-08 12:12:19.000000000 +0000
@@ -0,0 +1 @@
+data/b9_training.ply
diff -pruN 5.0.3-1/examples/Classification/gis_tutorial_example.cpp 5.1-1/examples/Classification/gis_tutorial_example.cpp
--- 5.0.3-1/examples/Classification/gis_tutorial_example.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Classification/gis_tutorial_example.cpp	2020-09-08 12:12:19.000000000 +0000
@@ -0,0 +1,749 @@
+#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
+#include <CGAL/Projection_traits_xy_3.h>
+
+#include <CGAL/Delaunay_triangulation_2.h>
+#include <CGAL/Triangulation_vertex_base_with_info_2.h>
+#include <CGAL/Triangulation_face_base_with_info_2.h>
+
+#include <CGAL/boost/graph/graph_traits_Delaunay_triangulation_2.h>
+#include <CGAL/boost/graph/copy_face_graph.h>
+
+#include <CGAL/Point_set_3.h>
+#include <CGAL/Point_set_3/IO.h>
+#include <CGAL/compute_average_spacing.h>
+
+#include <CGAL/Surface_mesh.h>
+#include <CGAL/Polygon_mesh_processing/locate.h>
+
+#include <CGAL/Polygon_mesh_processing/triangulate_hole.h>
+#include <CGAL/Polygon_mesh_processing/border.h>
+#include <CGAL/Polygon_mesh_processing/remesh.h>
+
+#include <boost/graph/adjacency_list.hpp>
+#include <CGAL/boost/graph/split_graph_into_polylines.h>
+
+#include <CGAL/IO/WKT.h>
+
+#include <CGAL/Constrained_Delaunay_triangulation_2.h>
+#include <CGAL/Constrained_triangulation_plus_2.h>
+
+#include <CGAL/Polyline_simplification_2/simplify.h>
+#include <CGAL/Polyline_simplification_2/Squared_distance_cost.h>
+
+#include <CGAL/Classification.h>
+
+#include <CGAL/Random.h>
+
+#include <fstream>
+#include <queue>
+
+#include "include/Color_ramp.h"
+
+///////////////////////////////////////////////////////////////////
+//! [TIN DS]
+
+using Kernel = CGAL::Exact_predicates_inexact_constructions_kernel;
+using Projection_traits = CGAL::Projection_traits_xy_3<Kernel>;
+using Point_2 = Kernel::Point_2;
+using Point_3 = Kernel::Point_3;
+using Segment_3 = Kernel::Segment_3;
+
+// Triangulated Irregular Network
+using TIN = CGAL::Delaunay_triangulation_2<Projection_traits>;
+
+//! [TIN DS]
+///////////////////////////////////////////////////////////////////
+
+///////////////////////////////////////////////////////////////////
+//! [TIN_with_info DS]
+
+// Triangulated Irregular Network (with info)
+using Point_set = CGAL::Point_set_3<Point_3>;
+using Vbi = CGAL::Triangulation_vertex_base_with_info_2 <Point_set::Index, Projection_traits>;
+using Fbi = CGAL::Triangulation_face_base_with_info_2<int, Projection_traits>;
+using TDS = CGAL::Triangulation_data_structure_2<Vbi, Fbi>;
+using TIN_with_info = CGAL::Delaunay_triangulation_2<Projection_traits, TDS>;
+
+//! [TIN_with_info DS]
+///////////////////////////////////////////////////////////////////
+
+namespace Classification = CGAL::Classification;
+
+#ifdef CGAL_LINKED_WITH_TBB
+using Concurrency_tag = CGAL::Parallel_tag;
+#else
+using Concurrency_tag = CGAL::Sequential_tag;
+#endif
+
+///////////////////////////////////////////////////////////////////
+//! [Contouring functions]
+
+bool face_has_isovalue (TIN::Face_handle fh, double isovalue)
+{
+  bool above = false, below = false;
+  for (int i = 0; i < 3; ++ i)
+  {
+    // Face has isovalue if one of its vertices is above and another
+    // one below
+    if (fh->vertex(i)->point().z() > isovalue)
+      above = true;
+    if (fh->vertex(i)->point().z() < isovalue)
+      below = true;
+  }
+
+  return (above && below);
+}
+
+Segment_3 isocontour_in_face (TIN::Face_handle fh, double isovalue)
+{
+  Point_3 source;
+  Point_3 target;
+  bool source_found = false;
+
+  for (int i = 0; i < 3; ++ i)
+  {
+    Point_3 p0 = fh->vertex((i+1) % 3)->point();
+    Point_3 p1 = fh->vertex((i+2) % 3)->point();
+
+    // Check if the isovalue crosses segment (p0,p1)
+    if ((p0.z() - isovalue) * (p1.z() - isovalue) > 0)
+      continue;
+
+    double zbottom = p0.z();
+    double ztop = p1.z();
+    if (zbottom > ztop)
+    {
+      std::swap (zbottom, ztop);
+      std::swap (p0, p1);
+    }
+
+    // Compute position of segment vertex
+    double ratio = (isovalue - zbottom) / (ztop - zbottom);
+    Point_3 p = CGAL::barycenter (p0, (1 - ratio), p1,ratio);
+
+    if (source_found)
+      target = p;
+    else
+    {
+      source = p;
+      source_found = true;
+    }
+  }
+
+  return Segment_3 (source, target);
+}
+
+
+//! [Contouring functions]
+///////////////////////////////////////////////////////////////////
+
+///////////////////////////////////////////////////////////////////
+//! [Contouring visitor]
+
+template <typename Graph>
+class Polylines_visitor
+{
+private:
+  std::vector<std::vector<Point_3> >& polylines;
+  Graph& graph;
+
+public:
+
+  Polylines_visitor (Graph& graph, std::vector<std::vector<Point_3> >& polylines)
+    : polylines (polylines), graph(graph) { }
+
+  void start_new_polyline()
+  {
+    polylines.push_back (std::vector<Point_3>());
+  }
+
+  void add_node (typename Graph::vertex_descriptor vd)
+  {
+    polylines.back().push_back (graph[vd]);
+  }
+
+  void end_polyline()
+  {
+    // filter small polylines
+    if (polylines.back().size() < 50)
+      polylines.pop_back();
+  }
+};
+
+//! [Contouring visitor]
+///////////////////////////////////////////////////////////////////
+
+///////////////////////////////////////////////////////////////////
+//! [CDT]
+
+namespace PS = CGAL::Polyline_simplification_2;
+using CDT_vertex_base = PS::Vertex_base_2<Projection_traits>;
+using CDT_face_base = CGAL::Constrained_triangulation_face_base_2<Projection_traits>;
+using CDT_TDS = CGAL::Triangulation_data_structure_2<CDT_vertex_base, CDT_face_base>;
+using CDT = CGAL::Constrained_Delaunay_triangulation_2<Projection_traits, CDT_TDS>;
+using CTP = CGAL::Constrained_triangulation_plus_2<CDT>;
+
+//! [CDT]
+///////////////////////////////////////////////////////////////////
+
+int main (int argc, char** argv)
+{
+  if (argc != 2)
+  {
+    std::cerr << "Usage: " << argv[0] << " points.ply" << std::endl;
+    return EXIT_FAILURE;
+  }
+
+  ///////////////////////////////////////////////////////////////////
+  //! [Init DSM]
+
+  // Read points
+  std::ifstream ifile (argv[1], std::ios_base::binary);
+  CGAL::Point_set_3<Point_3> points;
+  ifile >> points;
+  std::cerr << points.size() << " point(s) read" << std::endl;
+
+  // Create DSM
+  TIN dsm (points.points().begin(), points.points().end());
+
+  //! [Init DSM]
+  ///////////////////////////////////////////////////////////////////
+
+  ///////////////////////////////////////////////////////////////////
+  //! [Save DSM]
+
+  using Mesh = CGAL::Surface_mesh<Point_3>;
+
+  Mesh dsm_mesh;
+  CGAL::copy_face_graph (dsm, dsm_mesh);
+  std::ofstream dsm_ofile ("dsm.ply", std::ios_base::binary);
+  CGAL::set_binary_mode (dsm_ofile);
+  CGAL::write_ply (dsm_ofile, dsm_mesh);
+  dsm_ofile.close();
+
+  //! [Save DSM]
+  ///////////////////////////////////////////////////////////////////
+
+  ///////////////////////////////////////////////////////////////////
+  //! [TIN_with_info]
+
+  auto idx_to_point_with_info
+    = [&](const Point_set::Index& idx) -> std::pair<Point_3, Point_set::Index>
+      {
+        return std::make_pair (points.point(idx), idx);
+      };
+
+  TIN_with_info tin_with_info
+    (boost::make_transform_iterator (points.begin(), idx_to_point_with_info),
+     boost::make_transform_iterator (points.end(), idx_to_point_with_info));
+
+  //! [TIN_with_info]
+  ///////////////////////////////////////////////////////////////////
+
+  ///////////////////////////////////////////////////////////////////
+  //! [Components]
+
+  double spacing = CGAL::compute_average_spacing<Concurrency_tag>(points, 6);
+  spacing *= 2;
+
+  auto face_height
+    = [&](const TIN_with_info::Face_handle fh) -> double
+      {
+        double out = 0.;
+        for (int i = 0; i < 3; ++ i)
+          out = (std::max) (out, CGAL::abs(fh->vertex(i)->point().z() - fh->vertex((i+1)%3)->point().z()));
+        return out;
+      };
+
+  // Initialize faces info
+  for (TIN_with_info::Face_handle fh : tin_with_info.all_face_handles())
+    if (tin_with_info.is_infinite(fh) || face_height(fh) > spacing) // Filtered faces are given info() = -2
+      fh->info() = -2;
+    else // Pending faces are given info() = -1;
+      fh->info() = -1;
+
+  // Flooding algorithm
+  std::vector<int> component_size;
+  for (TIN_with_info::Face_handle fh : tin_with_info.finite_face_handles())
+  {
+    if (fh->info() != -1)
+      continue;
+
+    std::queue<TIN_with_info::Face_handle> todo;
+    todo.push(fh);
+
+    int size = 0;
+    while (!todo.empty())
+    {
+      TIN_with_info::Face_handle current = todo.front();
+      todo.pop();
+
+      if (current->info() != -1)
+        continue;
+      current->info() = int(component_size.size());
+      ++ size;
+
+      for (int i = 0; i < 3; ++ i)
+        todo.push (current->neighbor(i));
+    }
+
+    component_size.push_back (size);
+  }
+
+  std::cerr << component_size.size() << " connected component(s) found" << std::endl;
+
+  //! [Components]
+  ///////////////////////////////////////////////////////////////////
+
+  ///////////////////////////////////////////////////////////////////
+  //! [Save TIN with info]
+
+  Mesh tin_colored_mesh;
+
+  Mesh::Property_map<Mesh::Face_index, CGAL::Color>
+    color_map = tin_colored_mesh.add_property_map<Mesh::Face_index, CGAL::Color>("f:color").first;
+
+  CGAL::copy_face_graph (tin_with_info, tin_colored_mesh,
+                         CGAL::parameters::face_to_face_output_iterator
+                         (boost::make_function_output_iterator
+                          ([&](const std::pair<TIN_with_info::Face_handle, Mesh::Face_index>& ff)
+                           {
+                             // Color unassigned faces gray
+                             if (ff.first->info() < 0)
+                               color_map[ff.second] = CGAL::Color(128, 128, 128);
+                             else
+                             {
+                               // Random color seeded by the component ID
+                               CGAL::Random r (ff.first->info());
+                               color_map[ff.second] = CGAL::Color (r.get_int(64, 192),
+                                                                   r.get_int(64, 192),
+                                                                   r.get_int(64, 192));
+                             }
+                           })));
+
+  std::ofstream tin_colored_ofile ("colored_tin.ply", std::ios_base::binary);
+  CGAL::set_binary_mode (tin_colored_ofile);
+  CGAL::write_ply (tin_colored_ofile, tin_colored_mesh);
+  tin_colored_ofile.close();
+
+  //! [Save TIN with info]
+  ///////////////////////////////////////////////////////////////////
+
+  ///////////////////////////////////////////////////////////////////
+  //! [Filtering]
+
+  int min_size = int(points.size() / 2);
+
+  std::vector<TIN_with_info::Vertex_handle> to_remove;
+  for (TIN_with_info::Vertex_handle vh : tin_with_info.finite_vertex_handles())
+  {
+    TIN_with_info::Face_circulator circ = tin_with_info.incident_faces (vh),
+      start = circ;
+
+    // Remove a vertex if it's only adjacent to components smaller than threshold
+    bool keep = false;
+    do
+    {
+      if (circ->info() >= 0 && component_size[std::size_t(circ->info())] > min_size)
+      {
+        keep = true;
+        break;
+      }
+    }
+    while (++ circ != start);
+
+    if (!keep)
+      to_remove.push_back (vh);
+  }
+
+  std::cerr << to_remove.size() << " vertices(s) will be removed after filtering" << std::endl;
+  for (TIN_with_info::Vertex_handle vh : to_remove)
+    tin_with_info.remove (vh);
+
+  //! [Filtering]
+  ///////////////////////////////////////////////////////////////////
+
+  ///////////////////////////////////////////////////////////////////
+  //! [Hole filling]
+
+  // Copy and keep track of overly large faces
+  Mesh dtm_mesh;
+
+  std::vector<Mesh::Face_index> face_selection;
+  Mesh::Property_map<Mesh::Face_index, bool> face_selection_map
+   = dtm_mesh.add_property_map<Mesh::Face_index, bool>("is_selected", false).first;
+
+  double limit = CGAL::square (5 * spacing);
+  CGAL::copy_face_graph (tin_with_info, dtm_mesh,
+                         CGAL::parameters::face_to_face_output_iterator
+                         (boost::make_function_output_iterator
+                          ([&](const std::pair<TIN_with_info::Face_handle, Mesh::Face_index>& ff)
+                           {
+                             double longest_edge = 0.;
+                             bool border = false;
+                             for (int i = 0; i < 3; ++ i)
+                             {
+                               longest_edge = (std::max)(longest_edge, CGAL::squared_distance
+                                                         (ff.first->vertex((i+1)%3)->point(),
+                                                          ff.first->vertex((i+2)%3)->point()));
+
+                               TIN_with_info::Face_circulator circ
+                                 = tin_with_info.incident_faces (ff.first->vertex(i)),
+                                 start = circ;
+                               do
+                               {
+                                 if (tin_with_info.is_infinite (circ))
+                                 {
+                                   border = true;
+                                   break;
+                                 }
+                               }
+                               while (++ circ != start);
+
+                               if (border)
+                                 break;
+                             }
+
+                             // Select if face is too big AND it's not
+                             // on the border (to have closed holes)
+                             if (!border && longest_edge > limit)
+                             {
+                               face_selection_map[ff.second] = true;
+                               face_selection.push_back (ff.second);
+                             }
+                           })));
+
+  // Save original DTM
+  std::ofstream dtm_ofile ("dtm.ply", std::ios_base::binary);
+  CGAL::set_binary_mode (dtm_ofile);
+  CGAL::write_ply (dtm_ofile, dtm_mesh);
+  dtm_ofile.close();
+
+  std::cerr << face_selection.size() << " face(s) are selected for removal" << std::endl;
+
+  // Expand face selection to keep a well formed 2-manifold mesh after removal
+  CGAL::expand_face_selection_for_removal (face_selection, dtm_mesh, face_selection_map);
+  face_selection.clear();
+  for (Mesh::Face_index fi : faces(dtm_mesh))
+    if (face_selection_map[fi])
+      face_selection.push_back(fi);
+
+  std::cerr << face_selection.size() << " face(s) are selected for removal after expansion" << std::endl;
+
+  for (Mesh::Face_index fi : face_selection)
+    CGAL::Euler::remove_face (halfedge(fi, dtm_mesh), dtm_mesh);
+  dtm_mesh.collect_garbage();
+
+  if (!dtm_mesh.is_valid())
+    std::cerr << "Invalid mesh!" << std::endl;
+
+  // Save filtered DTM
+  std::ofstream dtm_holes_ofile ("dtm_with_holes.ply", std::ios_base::binary);
+  CGAL::set_binary_mode (dtm_holes_ofile);
+  CGAL::write_ply (dtm_holes_ofile, dtm_mesh);
+  dtm_holes_ofile.close();
+
+  // Get all holes
+  std::vector<Mesh::Halfedge_index> holes;
+  CGAL::Polygon_mesh_processing::extract_boundary_cycles (dtm_mesh, std::back_inserter (holes));
+
+  std::cerr << holes.size() << " hole(s) identified" << std::endl;
+
+  // Identify outer hull (hole with maximum size)
+  double max_size = 0.;
+  Mesh::Halfedge_index outer_hull;
+  for (Mesh::Halfedge_index hi : holes)
+  {
+    CGAL::Bbox_3 hole_bbox;
+    for (Mesh::Halfedge_index haf : CGAL::halfedges_around_face(hi, dtm_mesh))
+    {
+      const Point_3& p = dtm_mesh.point(target(haf, dtm_mesh));
+      hole_bbox += p.bbox();
+    }
+    double size = CGAL::squared_distance (Point_2(hole_bbox.xmin(), hole_bbox.ymin()),
+                                          Point_2(hole_bbox.xmax(), hole_bbox.ymax()));
+    if (size > max_size)
+    {
+      max_size = size;
+      outer_hull = hi;
+    }
+  }
+
+  // Fill all holes except the bigest (which is the outer hull of the mesh)
+  for (Mesh::Halfedge_index hi : holes)
+    if (hi != outer_hull)
+      CGAL::Polygon_mesh_processing::triangulate_refine_and_fair_hole
+        (dtm_mesh, hi, CGAL::Emptyset_iterator(), CGAL::Emptyset_iterator());
+
+  // Save DTM with holes filled
+  std::ofstream dtm_filled_ofile ("dtm_filled.ply", std::ios_base::binary);
+  CGAL::set_binary_mode (dtm_filled_ofile);
+  CGAL::write_ply (dtm_filled_ofile, dtm_mesh);
+  dtm_filled_ofile.close();
+
+  //! [Hole filling]
+  ///////////////////////////////////////////////////////////////////
+
+  ///////////////////////////////////////////////////////////////////
+  //! [Remeshing]
+
+  CGAL::Polygon_mesh_processing::isotropic_remeshing (faces(dtm_mesh), spacing, dtm_mesh);
+
+  std::ofstream dtm_remeshed_ofile ("dtm_remeshed.ply", std::ios_base::binary);
+  CGAL::set_binary_mode (dtm_remeshed_ofile);
+  CGAL::write_ply (dtm_remeshed_ofile, dtm_mesh);
+  dtm_remeshed_ofile.close();
+
+  //! [Remeshing]
+  ///////////////////////////////////////////////////////////////////
+
+  TIN dtm_clean (dtm_mesh.points().begin(), dtm_mesh.points().end());
+
+  ///////////////////////////////////////////////////////////////////
+  //! [Rastering]
+
+  CGAL::Bbox_3 bbox = CGAL::bbox_3 (points.points().begin(), points.points().end());
+
+  // Generate raster image 1920-pixels large
+  std::size_t width = 1920;
+  std::size_t height = std::size_t((bbox.ymax() - bbox.ymin()) * 1920 / (bbox.xmax() - bbox.xmin()));
+
+  std::cerr << "Rastering with resolution " << width << "x" << height << std::endl;
+
+  // Use PPM format (Portable PixMap) for simplicity
+  std::ofstream raster_ofile ("raster.ppm", std::ios_base::binary);
+
+  // PPM header
+  raster_ofile << "P6" << std::endl // magic number
+               << width << " " << height << std::endl // dimensions of the image
+               << 255 << std::endl; // maximum color value
+
+  // Use rainbow color ramp output
+  Color_ramp color_ramp;
+
+  // Keeping track of location from one point to its neighbor allows
+  // for fast locate in DT
+  TIN::Face_handle location;
+
+  // Query each pixel of the image
+  for (std::size_t y = 0; y < height; ++ y)
+    for (std::size_t x = 0; x < width; ++ x)
+    {
+      Point_3 query (bbox.xmin() + x * (bbox.xmax() - bbox.xmin()) / double(width),
+                     bbox.ymin() + (height-y) * (bbox.ymax() - bbox.ymin()) / double(height),
+                     0); // not relevant for location in 2D
+
+      location = dtm_clean.locate (query, location);
+
+      // Points outside the convex hull will be colored black
+      std::array<unsigned char, 3> colors { 0, 0, 0 };
+      if (!dtm_clean.is_infinite(location))
+      {
+        std::array<double, 3> barycentric_coordinates
+          = CGAL::Polygon_mesh_processing::barycentric_coordinates
+          (Point_2 (location->vertex(0)->point().x(), location->vertex(0)->point().y()),
+           Point_2 (location->vertex(1)->point().x(), location->vertex(1)->point().y()),
+           Point_2 (location->vertex(2)->point().x(), location->vertex(2)->point().y()),
+           Point_2 (query.x(), query.y()),
+           Kernel());
+
+        double height_at_query
+          = (barycentric_coordinates[0] * location->vertex(0)->point().z()
+             + barycentric_coordinates[1] * location->vertex(1)->point().z()
+             + barycentric_coordinates[2] * location->vertex(2)->point().z());
+
+        // Color ramp generates a color depending on a value from 0 to 1
+        double height_ratio = (height_at_query - bbox.zmin()) / (bbox.zmax() - bbox.zmin());
+        colors = color_ramp.get(height_ratio);
+      }
+      raster_ofile.write ((char*)(&colors), 3);
+    }
+
+  raster_ofile.close();
+
+  //! [Rastering]
+  ///////////////////////////////////////////////////////////////////
+
+  // Smooth heights with 5 successive Gaussian filters
+  double gaussian_variance = 4 * spacing * spacing;
+  for (TIN::Vertex_handle vh : dtm_clean.finite_vertex_handles())
+  {
+    double z = vh->point().z();
+    double total_weight = 1;
+
+    TIN::Vertex_circulator circ = dtm_clean.incident_vertices (vh),
+      start = circ;
+
+    do
+    {
+      if (!dtm_clean.is_infinite(circ))
+      {
+        double sq_dist = CGAL::squared_distance (vh->point(), circ->point());
+
+        double weight = std::exp(- sq_dist / gaussian_variance);
+        z += weight * circ->point().z();
+        total_weight += weight;
+      }
+    }
+    while (++ circ != start);
+
+    z /= total_weight;
+
+    vh->point() = Point_3 (vh->point().x(), vh->point().y(), z);
+  }
+
+  ///////////////////////////////////////////////////////////////////
+  //! [Contouring extraction]
+
+  std::array<double, 50> isovalues; // Contour 50 isovalues
+  for (std::size_t i = 0; i < isovalues.size(); ++ i)
+    isovalues[i] = bbox.zmin() + ((i+1) * (bbox.zmax() - bbox.zmin()) / (isovalues.size() - 2));
+
+  // First find on each face if they are crossed by some isovalues and
+  // extract segments in a graph
+  using Segment_graph = boost::adjacency_list<boost::listS, boost::vecS, boost::undirectedS, Point_3>;
+  Segment_graph graph;
+  using Map_p2v = std::map<Point_3, Segment_graph::vertex_descriptor>;
+  Map_p2v map_p2v;
+  for (TIN::Face_handle vh : dtm_clean.finite_face_handles())
+    for (double iv : isovalues)
+      if (face_has_isovalue (vh, iv))
+      {
+        Segment_3 segment = isocontour_in_face (vh, iv);
+        for (const Point_3& p : { segment.source(), segment.target() })
+        {
+          // Only insert end points of segments once to get a well connected graph
+          Map_p2v::iterator iter;
+          bool inserted;
+          std::tie (iter, inserted) = map_p2v.insert (std::make_pair (p, Segment_graph::vertex_descriptor()));
+          if (inserted)
+          {
+            iter->second = boost::add_vertex (graph);
+            graph[iter->second] = p;
+          }
+        }
+        boost::add_edge (map_p2v[segment.source()], map_p2v[segment.target()], graph);
+      }
+
+  //! [Contouring extraction]
+  ///////////////////////////////////////////////////////////////////
+
+  ///////////////////////////////////////////////////////////////////
+  //! [Contouring split]
+
+  // Split segments into polylines
+  std::vector<std::vector<Point_3> > polylines;
+  Polylines_visitor<Segment_graph> visitor (graph, polylines);
+  CGAL::split_graph_into_polylines (graph, visitor);
+
+  std::cerr << polylines.size() << " polylines computed, with "
+            << map_p2v.size() << " vertices in total" << std::endl;
+
+  // Output to WKT file
+  std::ofstream contour_ofile ("contour.wkt");
+  contour_ofile.precision(18);
+  CGAL::write_multi_linestring_WKT (contour_ofile, polylines);
+  contour_ofile.close();
+
+  //! [Contouring split]
+  ///////////////////////////////////////////////////////////////////
+
+  ///////////////////////////////////////////////////////////////////
+  //! [Contouring simplify]
+
+  // Construct constrained Delaunay triangulation with polylines as constraints
+  CTP ctp;
+  for (const std::vector<Point_3>& poly : polylines)
+    ctp.insert_constraint (poly.begin(), poly.end());
+
+  // Simplification algorithm with limit on distance
+  PS::simplify (ctp, PS::Squared_distance_cost(), PS::Stop_above_cost_threshold (16 * spacing * spacing));
+
+  polylines.clear();
+  for (CTP::Constraint_id cid : ctp.constraints())
+  {
+    polylines.push_back (std::vector<Point_3>());
+    polylines.back().reserve (ctp.vertices_in_constraint (cid).size());
+    for (CTP::Vertex_handle vh : ctp.vertices_in_constraint(cid))
+      polylines.back().push_back (vh->point());
+  }
+
+  std::size_t nb_vertices
+    = std::accumulate (polylines.begin(), polylines.end(), 0,
+                       [](std::size_t size, const std::vector<Point_3>& poly) -> std::size_t
+                       { return size + poly.size(); });
+
+  std::cerr << nb_vertices
+            << " vertices remaining after simplification ("
+            << 100. * (nb_vertices / double(map_p2v.size())) << "%)" << std::endl;
+
+  // Output to WKT file
+  std::ofstream simplified_ofile ("simplified.wkt");
+  simplified_ofile.precision(18);
+  CGAL::write_multi_linestring_WKT (simplified_ofile, polylines);
+  simplified_ofile.close();
+
+  //! [Contouring simplify]
+  ///////////////////////////////////////////////////////////////////
+
+  ///////////////////////////////////////////////////////////////////
+  //! [Classification]
+
+  // Get training from input
+  Point_set::Property_map<int> training_map;
+  bool training_found;
+  std::tie (training_map, training_found) = points.property_map<int>("training");
+
+  if (training_found)
+  {
+    std::cerr << "Classifying ground/vegetation/building" << std::endl;
+
+    // Create labels
+    Classification::Label_set labels ({ "ground", "vegetation", "building" });
+
+    // Generate features
+    Classification::Feature_set features;
+    Classification::Point_set_feature_generator<Kernel, Point_set, Point_set::Point_map>
+      generator (points, points.point_map(), 5); // 5 scales
+
+#ifdef CGAL_LINKED_WITH_TBB
+    // If TBB is used, features can be computed in parallel
+    features.begin_parallel_additions();
+    generator.generate_point_based_features (features);
+    features.end_parallel_additions();
+#else
+    generator.generate_point_based_features (features);
+#endif
+
+    // Train a random forest classifier
+    Classification::ETHZ::Random_forest_classifier classifier (labels, features);
+    classifier.train (points.range(training_map));
+
+    // Classify with graphcut regularization
+    Point_set::Property_map<int> label_map = points.add_property_map<int>("labels").first;
+    Classification::classify_with_graphcut<Concurrency_tag>
+      (points, points.point_map(), labels, classifier,
+       generator.neighborhood().k_neighbor_query(12), // regularize on 12-neighbors graph
+       0.5f, // graphcut weight
+       12, // Subdivide to speed-up process
+       label_map);
+
+    // Evaluate
+    std::cerr << "Mean IoU on training data = "
+              << Classification::Evaluation(labels,
+                                            points.range(training_map),
+                                            points.range(label_map)).mean_intersection_over_union() << std::endl;
+
+    // Save the classified point set
+    std::ofstream classified_ofile ("classified.ply");
+    CGAL::set_binary_mode (classified_ofile);
+    classified_ofile << points;
+    classified_ofile.close();
+  }
+
+  //! [Classification]
+  ///////////////////////////////////////////////////////////////////
+
+
+  return EXIT_SUCCESS;
+}
diff -pruN 5.0.3-1/examples/Classification/include/Color_ramp.h 5.1-1/examples/Classification/include/Color_ramp.h
--- 5.0.3-1/examples/Classification/include/Color_ramp.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Classification/include/Color_ramp.h	2020-09-08 12:14:47.000000000 +0000
@@ -0,0 +1,45 @@
+#ifndef COLOR_RAMP_H
+#define COLOR_RAMP_H
+
+class Color_ramp
+{
+  typedef std::array<unsigned char, 3> Color;
+  typedef std::pair<double, Color> Step;
+
+  std::vector<Step> m_steps;
+
+public:
+
+  Color_ramp()
+  {
+    m_steps.push_back (std::make_pair (0, Color{ 192, 192, 255}));
+    m_steps.push_back (std::make_pair (0.2, Color{ 0, 0, 255}));
+    m_steps.push_back (std::make_pair (0.4, Color{ 0, 255, 0}));
+    m_steps.push_back (std::make_pair (0.6, Color{ 255, 255, 0}));
+    m_steps.push_back (std::make_pair (0.8, Color{ 255, 0, 0}));
+    m_steps.push_back (std::make_pair (1.0, Color{ 128, 0, 0}));
+  }
+
+  Color get (double value) const
+  {
+    std::size_t idx = 0;
+    while (m_steps[idx+1].first < value)
+      ++ idx;
+
+    double v0 = m_steps[idx].first;
+    double v1 = m_steps[idx+1].first;
+    const Color& c0 = m_steps[idx].second;
+    const Color& c1 = m_steps[idx+1].second;
+
+    double ratio = (value - v0) / (v1 - v0);
+
+    Color out;
+    for (std::size_t i = 0; i < 3; ++ i)
+      out[i] = (unsigned char)((1 - ratio) * c0[i] + ratio * c1[i]);
+
+    return out;
+  }
+
+};
+
+#endif // COLOR_RAMP_H
diff -pruN 5.0.3-1/examples/Convex_hull_3/CMakeLists.txt 5.1-1/examples/Convex_hull_3/CMakeLists.txt
--- 5.0.3-1/examples/Convex_hull_3/CMakeLists.txt	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/examples/Convex_hull_3/CMakeLists.txt	2020-09-08 12:12:19.000000000 +0000
@@ -54,6 +54,9 @@ create_single_source_cgal_program( "lloy
 
 create_single_source_cgal_program( "quickhull_3.cpp" )
 
+create_single_source_cgal_program( "graph_hull_3.cpp" )
+
+
 create_single_source_cgal_program( "quickhull_any_dim_3.cpp" )
 
 create_single_source_cgal_program( "extreme_points_3_sm.cpp" )
diff -pruN 5.0.3-1/examples/Convex_hull_3/graph_hull_3.cpp 5.1-1/examples/Convex_hull_3/graph_hull_3.cpp
--- 5.0.3-1/examples/Convex_hull_3/graph_hull_3.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Convex_hull_3/graph_hull_3.cpp	2020-09-08 12:12:19.000000000 +0000
@@ -0,0 +1,31 @@
+#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
+#include <CGAL/Polyhedron_3.h>
+#include <CGAL/Surface_mesh.h>
+#include <CGAL/convex_hull_3.h>
+#include <vector>
+#include <fstream>
+
+
+typedef CGAL::Exact_predicates_inexact_constructions_kernel  K;
+typedef K::Point_3                                Point_3;
+typedef CGAL::Surface_mesh<Point_3>               Surface_mesh;
+
+
+int main(int argc, char* argv[])
+{
+  std::ifstream in( (argc>1)? argv[1] : "data/star.off");
+
+  Surface_mesh poly;
+  if(!(in >> poly))
+  {
+    std::cerr<<"Could not find input file."<<std::endl;
+    return 1;
+  }
+
+  Surface_mesh chull;
+  // compute convex hull
+  CGAL::convex_hull_3(poly, chull);
+
+  std::cout << "The convex hull contains " << chull.number_of_vertices() << " vertices" << std::endl;
+  return 0;
+}
diff -pruN 5.0.3-1/examples/Filtered_kernel/Cartesian_I.h 5.1-1/examples/Filtered_kernel/Cartesian_I.h
--- 5.0.3-1/examples/Filtered_kernel/Cartesian_I.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/examples/Filtered_kernel/Cartesian_I.h	2020-09-08 12:14:50.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/examples/Filtered_kernel/Cartesian_I.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/examples/Filtered_kernel/Cartesian_I.h $
 // $Id: Cartesian_I.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/examples/Generator/CMakeLists.txt 5.1-1/examples/Generator/CMakeLists.txt
--- 5.0.3-1/examples/Generator/CMakeLists.txt	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/examples/Generator/CMakeLists.txt	2020-09-08 12:12:19.000000000 +0000
@@ -12,17 +12,19 @@ if ( CGAL_FOUND )
 
   # Use Eigen
   find_package(Eigen3 3.1.0 QUIET) #(3.1.0 or greater)
-  if (EIGEN3_FOUND)
-    include( ${EIGEN3_USE_FILE} )
-  endif()
+  include(CGAL_Eigen_support)
 
   # create a target per cppfile
   file(GLOB cppfiles RELATIVE ${CMAKE_CURRENT_SOURCE_DIR} ${CMAKE_CURRENT_SOURCE_DIR}/*.cpp)
   foreach(cppfile ${cppfiles})
     if(NOT (${cppfile} STREQUAL "random_points_in_tetrahedral_mesh_3.cpp")
         OR NOT (${cppfile} STREQUAL "random_points_on_tetrahedral_mesh_3.cpp")
-        OR EIGEN3_FOUND)
+        OR TARGET CGAL::Eigen_support)
       create_single_source_cgal_program( "${cppfile}" )
+      if (TARGET CGAL::Eigen_support)
+        get_filename_component(target ${cppfile} NAME_WE)
+        target_link_libraries(${target} PUBLIC CGAL::Eigen_support)
+      endif()
     endif()
   endforeach()
 
@@ -31,4 +33,3 @@ else()
     message(STATUS "This program requires the CGAL library, and will not be compiled.")
 
 endif()
-
diff -pruN 5.0.3-1/examples/Heat_method_3/CMakeLists.txt 5.1-1/examples/Heat_method_3/CMakeLists.txt
--- 5.0.3-1/examples/Heat_method_3/CMakeLists.txt	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/examples/Heat_method_3/CMakeLists.txt	2020-09-08 12:12:19.000000000 +0000
@@ -33,12 +33,10 @@ endif()
 
 
 find_package(Eigen3 3.3.0)
+include(CGAL_Eigen_support)
 
-if (EIGEN3_FOUND)
-  include( ${EIGEN3_USE_FILE} )
-else()
+if (NOT TARGET CGAL::Eigen_support)
   message(STATUS "This project requires the Eigen library (3.3 or greater), and will not be compiled.")
-
   return()
 endif()
 
@@ -54,6 +52,10 @@ include_directories( BEFORE include )
 include( CGAL_CreateSingleSourceCGALProgram )
 
 create_single_source_cgal_program( "heat_method.cpp" )
+target_link_libraries(heat_method PUBLIC CGAL::Eigen_support)
 create_single_source_cgal_program( "heat_method_polyhedron.cpp" )
+target_link_libraries(heat_method_polyhedron PUBLIC CGAL::Eigen_support)
 create_single_source_cgal_program( "heat_method_surface_mesh.cpp" )
+target_link_libraries(heat_method_surface_mesh PUBLIC CGAL::Eigen_support)
 create_single_source_cgal_program( "heat_method_surface_mesh_direct.cpp" )
+target_link_libraries(heat_method_surface_mesh_direct PUBLIC CGAL::Eigen_support)
diff -pruN 5.0.3-1/examples/Jet_fitting_3/CMakeLists.txt 5.1-1/examples/Jet_fitting_3/CMakeLists.txt
--- 5.0.3-1/examples/Jet_fitting_3/CMakeLists.txt	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Jet_fitting_3/CMakeLists.txt	2020-09-08 12:12:19.000000000 +0000
@@ -13,8 +13,8 @@ if ( CGAL_FOUND )
 
   # use Eigen
   find_package(Eigen3 3.1.0) #(requires 3.1.0 or greater)
-  if (EIGEN3_FOUND)
-    include( ${EIGEN3_USE_FILE} )
+  include(CGAL_Eigen_support)
+  if (TARGET CGAL::Eigen_support)
     # Link with Boost.ProgramOptions (optional)
     find_package(Boost QUIET COMPONENTS program_options)
     if(Boost_PROGRAM_OPTIONS_FOUND)
@@ -30,8 +30,10 @@ if ( CGAL_FOUND )
       list(APPEND CGAL_3RD_PARTY_LIBRARIES ${Boost_PROGRAM_OPTIONS_LIBRARY})
    endif()
 
-    create_single_source_cgal_program( "Mesh_estimation.cpp" )
-    create_single_source_cgal_program( "Single_estimation.cpp" )
+   create_single_source_cgal_program( "Mesh_estimation.cpp" )
+   target_link_libraries(Mesh_estimation PUBLIC CGAL::Eigen_support)
+   create_single_source_cgal_program( "Single_estimation.cpp" )
+   target_link_libraries(Single_estimation PUBLIC CGAL::Eigen_support)
 
   else()
     message(STATUS "NOTICE: This program requires Eigen 3.1 (or greater) and will not be compiled.")
@@ -42,4 +44,3 @@ else()
   message(STATUS "NOTICE: This program requires the CGAL library, and will not be compiled.")
 
 endif()
-
diff -pruN 5.0.3-1/examples/Linear_cell_complex/basic_viewer.h 5.1-1/examples/Linear_cell_complex/basic_viewer.h
--- 5.0.3-1/examples/Linear_cell_complex/basic_viewer.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/examples/Linear_cell_complex/basic_viewer.h	2020-09-08 12:15:05.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Linear_cell_complex/examples/Linear_cell_complex/basic_viewer.h $
-// $Id: basic_viewer.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Linear_cell_complex/examples/Linear_cell_complex/basic_viewer.h $
+// $Id: basic_viewer.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
@@ -215,7 +215,6 @@ class Basic_viewer : public CGAL::QGLVie
 
   typedef CGAL::Constrained_triangulation_face_base_2<P_traits, Fb1>    Fb;
   typedef CGAL::Triangulation_data_structure_2<Vb,Fb>                   TDS;
-  // typedef CGAL::No_intersection_tag                                     Itag;
   typedef CGAL::Exact_predicates_tag                                    Itag;
   typedef CGAL::Constrained_Delaunay_triangulation_2<P_traits, TDS,
                                                      Itag>              CDT;
diff -pruN 5.0.3-1/examples/Linear_cell_complex/CMakeLists.txt 5.1-1/examples/Linear_cell_complex/CMakeLists.txt
--- 5.0.3-1/examples/Linear_cell_complex/CMakeLists.txt	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Linear_cell_complex/CMakeLists.txt	2020-09-08 12:12:19.000000000 +0000
@@ -24,7 +24,7 @@ endif()
 # set(CMAKE_EXE_LINKER_FLAGS "${CMAKE_EXE_LINKER_FLAGS} -pg")
 
 # To use valgrind, we must disable rounding math ckeck.
-# add_definition(-DCGAL_DISABLE_ROUNDING_MATH_CHECK)
+# add_definitions(-DCGAL_DISABLE_ROUNDING_MATH_CHECK)
 
 if (CGAL_FOUND)
 
diff -pruN 5.0.3-1/examples/Mesh_3/CMakeLists.txt 5.1-1/examples/Mesh_3/CMakeLists.txt
--- 5.0.3-1/examples/Mesh_3/CMakeLists.txt	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Mesh_3/CMakeLists.txt	2020-09-08 12:12:19.000000000 +0000
@@ -1,148 +1,158 @@
-# Created by the script cgal_create_cmake_script
-# This is the CMake script for compiling a CGAL application.
-
-
 cmake_minimum_required(VERSION 3.1...3.15)
 project( Mesh_3_Examples )
 
-
-
 add_definitions(-DCGAL_MESH_3_NO_DEPRECATED_SURFACE_INDEX
                 -DCGAL_MESH_3_NO_DEPRECATED_C3T3_ITERATORS)
 
-if ( MESH_3_VERBOSE )
+if ( CGAL_MESH_3_VERBOSE )
   add_definitions(-DCGAL_MESH_3_VERBOSE)
 endif()
 
-if(POLICY CMP0074)
-  cmake_policy(SET CMP0074 NEW)
-endif()
-
 find_package(CGAL COMPONENTS ImageIO)
+find_package(Boost)
 
-if ( CGAL_FOUND )
-  include( ${CGAL_USE_FILE} )
+option(CGAL_ACTIVATE_CONCURRENT_MESH_3 "Activate parallelism in Mesh_3" OFF)
 
-  find_package(Boost)
+if( CGAL_ACTIVATE_CONCURRENT_MESH_3 OR ENV{CGAL_ACTIVATE_CONCURRENT_MESH_3} )
+  add_definitions( -DCGAL_CONCURRENT_MESH_3 )
+  find_package( TBB REQUIRED )
+  include(CGAL_TBB_support)
+endif()
 
-  # Activate concurrency ? (turned OFF by default)
-  option(CGAL_ACTIVATE_CONCURRENT_MESH_3
-    "Activate parallelism in Mesh_3"
-    OFF)
-
-  # And add -DCGAL_CONCURRENT_MESH_3 if that option is ON
-  if( CGAL_ACTIVATE_CONCURRENT_MESH_3 OR ENV{CGAL_ACTIVATE_CONCURRENT_MESH_3} )
-    add_definitions( -DCGAL_CONCURRENT_MESH_3 )
-    find_package( TBB REQUIRED )
-  else( CGAL_ACTIVATE_CONCURRENT_MESH_3 OR ENV{CGAL_ACTIVATE_CONCURRENT_MESH_3} )
-    option( LINK_WITH_TBB
-      "Link with TBB anyway so we can use TBB timers for profiling"
-      ON)
-    if( LINK_WITH_TBB )
-      find_package( TBB )
-    endif( LINK_WITH_TBB )
+# Use Eigen
+find_package(Eigen3 3.1.0 REQUIRED) #(3.1.0 or greater)
+include(CGAL_Eigen_support)
+
+find_package(VTK QUIET COMPONENTS vtkImagingGeneral vtkIOImage NO_MODULE)
+if(VTK_FOUND)
+  if(VTK_USE_FILE)
+    include(${VTK_USE_FILE})
   endif()
-
-  # Use Eigen
-  find_package(Eigen3 3.1.0 REQUIRED) #(3.1.0 or greater)
-  if (NOT EIGEN3_FOUND)
-    message(STATUS "This project requires the Eigen library, and will not be compiled.")
-    return()
+  if ("${VTK_VERSION_MAJOR}" GREATER "5" OR VTK_VERSION VERSION_GREATER 5)
+    message(STATUS "VTK found")
+    if(TARGET VTK::IOImage)
+      set(VTK_LIBRARIES VTK::ImagingGeneral VTK::IOImage)
+    endif()
   else()
-    include( ${EIGEN3_USE_FILE} )
+    message(STATUS "VTK version 6.0 or greater is required")
   endif()
+else()
+  message(STATUS "VTK was not found")
+endif()
 
-  find_package(VTK QUIET COMPONENTS vtkImagingGeneral vtkIOImage NO_MODULE)
-  if(VTK_FOUND)
-    if(VTK_USE_FILE)
-      include(${VTK_USE_FILE})
-    endif()
-    if ("${VTK_VERSION_MAJOR}" GREATER "5" OR VTK_VERSION VERSION_GREATER 5)
-      message(STATUS "VTK found")
-      if(TARGET VTK::IOImage)
-        set(VTK_LIBRARIES VTK::ImagingGeneral VTK::IOImage)
-      endif()
-    else()
-      message(STATUS "VTK version 6.0 or greater is required")
-      endif()
-    else()
-      message(STATUS "VTK was not found")
-    endif()
+create_single_source_cgal_program( "mesh_hybrid_mesh_domain.cpp" )
+target_link_libraries(mesh_hybrid_mesh_domain PUBLIC CGAL::Eigen_support)
 
+create_single_source_cgal_program( "mesh_implicit_sphere.cpp" )
+target_link_libraries(mesh_implicit_sphere PUBLIC CGAL::Eigen_support)
 
-    # Compilable examples
-    create_single_source_cgal_program( "mesh_hybrid_mesh_domain.cpp" )
-    create_single_source_cgal_program( "mesh_implicit_sphere.cpp" )
-    create_single_source_cgal_program( "mesh_implicit_sphere_variable_size.cpp" )
-    create_single_source_cgal_program( "mesh_two_implicit_spheres_with_balls.cpp" )
-    create_single_source_cgal_program( "mesh_implicit_domains_2.cpp" "implicit_functions.cpp" )
-    create_single_source_cgal_program( "mesh_cubes_intersection.cpp" )
-    create_single_source_cgal_program( "mesh_cubes_intersection_with_features.cpp" )
-    create_single_source_cgal_program( "mesh_implicit_domains.cpp" "implicit_functions.cpp" )
-    create_single_source_cgal_program( "mesh_polyhedral_domain.cpp" )
-    create_single_source_cgal_program( "mesh_polyhedral_domain_sm.cpp" )
-    create_single_source_cgal_program( "mesh_polyhedral_domain_with_surface_inside.cpp" )
-    create_single_source_cgal_program( "remesh_polyhedral_surface.cpp" )
-    create_single_source_cgal_program( "remesh_polyhedral_surface_sm.cpp" )
-    create_single_source_cgal_program( "mesh_polyhedral_domain_with_features.cpp" )
-    create_single_source_cgal_program( "mesh_polyhedral_domain_with_features_sm.cpp" )
-    create_single_source_cgal_program( "mesh_polyhedral_domain_with_lipschitz_sizing.cpp" )
-    create_single_source_cgal_program( "mesh_polyhedral_complex.cpp" )
-    create_single_source_cgal_program( "mesh_polyhedral_complex_sm.cpp" )
-    if( WITH_CGAL_ImageIO )
-      if( VTK_FOUND AND ("${VTK_VERSION_MAJOR}" GREATER "5" OR VTK_VERSION VERSION_GREATER 5) )
-        add_executable ( mesh_3D_gray_vtk_image mesh_3D_gray_vtk_image.cpp )
-        target_link_libraries(  mesh_3D_gray_vtk_image ${CGAL_LIBRARIES} ${CGAL_3RD_PARTY_LIBRARIES} ${VTK_LIBRARIES})
-        cgal_add_test( mesh_3D_gray_vtk_image )
-        add_to_cached_list( CGAL_EXECUTABLE_TARGETS mesh_3D_gray_vtk_image )
-      endif()
-
-      create_single_source_cgal_program( "mesh_3D_gray_image.cpp" )
-      create_single_source_cgal_program( "mesh_3D_gray_image_multiple_values.cpp" )
-      create_single_source_cgal_program( "mesh_3D_image_with_features.cpp" )
-      if( CGAL_ImageIO_USE_ZLIB )
-        create_single_source_cgal_program( "mesh_optimization_example.cpp" )
-
-        create_single_source_cgal_program( "mesh_optimization_lloyd_example.cpp" )
-
-        create_single_source_cgal_program( "mesh_3D_image.cpp" )
-        create_single_source_cgal_program( "mesh_3D_image_with_custom_initialization.cpp" )
-
-        create_single_source_cgal_program( "mesh_3D_image_variable_size.cpp" )
-      else()
-        message( STATUS "NOTICE: The examples mesh_3D_image.cpp, mesh_3D_image_variable_size.cpp, mesh_optimization_example.cpp and mesh_optimization_lloyd_example.cpp need CGAL_ImageIO to be configured with ZLIB support, and will not be compiled." )
-      endif()
-    else()
-      message( STATUS "NOTICE: Some examples need the CGAL_ImageIO library, and will not be compiled." )
-    endif()
-#    create_single_source_cgal_program( "mesh_polyhedral_implicit_function.cpp" )
-#    create_single_source_cgal_program( "mesh_polyhedral_surface_tolerance_region.cpp" )
-#    create_single_source_cgal_program( "mesh_polyhedral_edge_tolerance_region.cpp" )
-
-    if(CGAL_ACTIVATE_CONCURRENT_MESH_3 AND TBB_FOUND)
-      foreach(target
-        mesh_3D_image
-        mesh_3D_image_variable_size
-        mesh_3D_image_with_custom_initialization
-        mesh_3D_image_with_features
-        mesh_implicit_sphere
-        mesh_implicit_sphere_variable_size
-        mesh_optimization_example
-        mesh_optimization_lloyd_example
-        mesh_polyhedral_complex
-        mesh_polyhedral_complex_sm
-        mesh_polyhedral_domain
-        mesh_polyhedral_domain_sm
-        mesh_polyhedral_domain_with_features
-        mesh_polyhedral_domain_with_features_sm
-        mesh_polyhedral_domain_with_lipschitz_sizing
-        mesh_two_implicit_spheres_with_balls)
-        if(TARGET ${target})
-          CGAL_target_use_TBB(${target})
-        endif()
-      endforeach()
-    endif()
+create_single_source_cgal_program( "mesh_implicit_sphere_variable_size.cpp" )
+target_link_libraries(mesh_implicit_sphere_variable_size PUBLIC CGAL::Eigen_support)
+
+create_single_source_cgal_program( "mesh_two_implicit_spheres_with_balls.cpp" )
+target_link_libraries(mesh_two_implicit_spheres_with_balls PUBLIC CGAL::Eigen_support)
+
+create_single_source_cgal_program( "mesh_implicit_domains_2.cpp" "implicit_functions.cpp" )
+target_link_libraries(mesh_implicit_domains_2 PUBLIC CGAL::Eigen_support)
+
+create_single_source_cgal_program( "mesh_cubes_intersection.cpp" )
+target_link_libraries(mesh_cubes_intersection PUBLIC CGAL::Eigen_support)
+
+create_single_source_cgal_program( "mesh_cubes_intersection_with_features.cpp" )
+target_link_libraries(mesh_cubes_intersection_with_features PUBLIC CGAL::Eigen_support)
+
+create_single_source_cgal_program( "mesh_implicit_domains.cpp" "implicit_functions.cpp" )
+target_link_libraries(mesh_implicit_domains PUBLIC CGAL::Eigen_support)
+
+create_single_source_cgal_program( "mesh_polyhedral_domain.cpp" )
+target_link_libraries(mesh_polyhedral_domain PUBLIC CGAL::Eigen_support)
+
+create_single_source_cgal_program( "mesh_polyhedral_domain_sm.cpp" )
+target_link_libraries(mesh_polyhedral_domain_sm PUBLIC CGAL::Eigen_support)
 
+create_single_source_cgal_program( "mesh_polyhedral_domain_with_surface_inside.cpp" )
+target_link_libraries(mesh_polyhedral_domain_with_surface_inside PUBLIC CGAL::Eigen_support)
+
+create_single_source_cgal_program( "remesh_polyhedral_surface.cpp" )
+target_link_libraries(remesh_polyhedral_surface PUBLIC CGAL::Eigen_support)
+
+create_single_source_cgal_program( "remesh_polyhedral_surface_sm.cpp" )
+target_link_libraries(remesh_polyhedral_surface_sm PUBLIC CGAL::Eigen_support)
+
+create_single_source_cgal_program( "mesh_polyhedral_domain_with_features.cpp" )
+target_link_libraries(mesh_polyhedral_domain_with_features PUBLIC CGAL::Eigen_support)
+
+create_single_source_cgal_program( "mesh_polyhedral_domain_with_features_sm.cpp" )
+target_link_libraries(mesh_polyhedral_domain_with_features_sm PUBLIC CGAL::Eigen_support)
+
+create_single_source_cgal_program( "mesh_polyhedral_domain_with_lipschitz_sizing.cpp" )
+target_link_libraries(mesh_polyhedral_domain_with_lipschitz_sizing PUBLIC CGAL::Eigen_support)
+
+create_single_source_cgal_program( "mesh_polyhedral_complex.cpp" )
+target_link_libraries(mesh_polyhedral_complex PUBLIC CGAL::Eigen_support)
+
+create_single_source_cgal_program( "mesh_polyhedral_complex_sm.cpp" )
+target_link_libraries(mesh_polyhedral_complex_sm PUBLIC CGAL::Eigen_support)
+
+if( TARGET CGAL::CGAL_ImageIO)
+  if( VTK_FOUND AND ("${VTK_VERSION_MAJOR}" GREATER "5" OR VTK_VERSION VERSION_GREATER 5) )
+    add_executable ( mesh_3D_gray_vtk_image mesh_3D_gray_vtk_image.cpp )
+    target_link_libraries(  mesh_3D_gray_vtk_image PUBLIC CGAL::Eigen_support CGAL::CGAL CGAL::CGAL_ImageIO ${CGAL_LIBRARIES} ${CGAL_3RD_PARTY_LIBRARIES} ${VTK_LIBRARIES})
+    cgal_add_test( mesh_3D_gray_vtk_image )
+    add_to_cached_list( CGAL_EXECUTABLE_TARGETS mesh_3D_gray_vtk_image )
+  endif()
+
+  create_single_source_cgal_program( "mesh_3D_gray_image.cpp" )
+  target_link_libraries(mesh_3D_gray_image PUBLIC CGAL::Eigen_support)
+
+  create_single_source_cgal_program( "mesh_3D_gray_image_multiple_values.cpp" )
+  target_link_libraries(mesh_3D_gray_image_multiple_values PUBLIC CGAL::Eigen_support)
+
+  create_single_source_cgal_program( "mesh_3D_image_with_features.cpp" )
+  target_link_libraries(mesh_3D_image_with_features PUBLIC CGAL::Eigen_support)
+
+  if( CGAL_ImageIO_USE_ZLIB )
+    create_single_source_cgal_program( "mesh_optimization_example.cpp" )
+    target_link_libraries(mesh_optimization_example PUBLIC CGAL::Eigen_support)
+
+    create_single_source_cgal_program( "mesh_optimization_lloyd_example.cpp" )
+    target_link_libraries(mesh_optimization_lloyd_example PUBLIC CGAL::Eigen_support)
+
+    create_single_source_cgal_program( "mesh_3D_image.cpp" )
+    target_link_libraries(mesh_3D_image PUBLIC CGAL::Eigen_support)
+
+    create_single_source_cgal_program( "mesh_3D_image_with_custom_initialization.cpp" )
+    target_link_libraries(mesh_3D_image_with_custom_initialization PUBLIC CGAL::Eigen_support)
+
+    create_single_source_cgal_program( "mesh_3D_image_variable_size.cpp" )
+    target_link_libraries(mesh_3D_image_variable_size PUBLIC CGAL::Eigen_support)
+  else()
+    message( STATUS "NOTICE: The examples mesh_3D_image.cpp, mesh_3D_image_variable_size.cpp, mesh_optimization_example.cpp and mesh_optimization_lloyd_example.cpp need CGAL_ImageIO to be configured with ZLIB support, and will not be compiled." )
+  endif()
 else()
-  message(STATUS "This program requires the CGAL library, and will not be compiled.")
+  message( STATUS "NOTICE: Some examples need the CGAL_ImageIO library, and will not be compiled." )
+endif()
+
+if(CGAL_ACTIVATE_CONCURRENT_MESH_3 AND TARGET CGAL::TBB_support)
+  foreach(target
+      mesh_3D_image
+      mesh_3D_image_variable_size
+      mesh_3D_image_with_custom_initialization
+      mesh_3D_image_with_features
+      mesh_implicit_sphere
+      mesh_implicit_sphere_variable_size
+      mesh_optimization_example
+      mesh_optimization_lloyd_example
+      mesh_polyhedral_complex
+      mesh_polyhedral_complex_sm
+      mesh_polyhedral_domain
+      mesh_polyhedral_domain_sm
+      mesh_polyhedral_domain_with_features
+      mesh_polyhedral_domain_with_features_sm
+      mesh_polyhedral_domain_with_lipschitz_sizing
+      mesh_two_implicit_spheres_with_balls)
+    if(TARGET ${target})
+      target_link_libraries(${target} PUBLIC CGAL::TBB_support)
+    endif()
+  endforeach()
 endif()
diff -pruN 5.0.3-1/examples/Nef_3/CMakeLists.txt 5.1-1/examples/Nef_3/CMakeLists.txt
--- 5.0.3-1/examples/Nef_3/CMakeLists.txt	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Nef_3/CMakeLists.txt	2020-09-08 12:12:19.000000000 +0000
@@ -6,7 +6,11 @@ cmake_minimum_required(VERSION 3.1...3.1
 project( Nef_3_Examples )
 
 
-find_package(CGAL QUIET)
+find_package(CGAL COMPONENTS Qt5)
+
+if(CGAL_Qt5_FOUND)
+        add_definitions(-DCGAL_USE_BASIC_VIEWER -DQT_NO_KEYWORDS)
+endif()
 
 if ( CGAL_FOUND )
 
@@ -16,6 +20,10 @@ if ( CGAL_FOUND )
     create_single_source_cgal_program( "${cppfile}" )
   endforeach()
 
+  if(CGAL_Qt5_FOUND)
+          target_link_libraries(draw_nef_3 PUBLIC CGAL::CGAL_Qt5)
+  endif()
+
 else()
 
     message(STATUS "This program requires the CGAL library, and will not be compiled.")
diff -pruN 5.0.3-1/examples/Nef_3/data/cross.off 5.1-1/examples/Nef_3/data/cross.off
--- 5.0.3-1/examples/Nef_3/data/cross.off	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Nef_3/data/cross.off	2020-09-08 12:12:19.000000000 +0000
@@ -0,0 +1,85 @@
+OFF
+40 38 0
+
+0 3 0
+1 3 0
+2 3 0
+2 4 0
+2 5 0
+3 5 0
+3 4 0
+3 3 0
+4 3 0
+5 3 0
+5 2 0
+4 2 0
+3 2 0
+3 1 0
+3 0 0
+2 0 0
+2 1 0
+2 2 0
+1 2 0
+0 2 0
+
+0 3 1
+1 3 1
+2 3 1
+2 4 1
+2 5 1
+3 5 1
+3 4 1
+3 3 1
+4 3 1
+5 3 1
+5 2 1
+4 2 1
+3 2 1
+3 1 1
+3 0 1
+2 0 1
+2 1 1
+2 2 1
+1 2 1
+0 2 1
+
+4 0 1 18 19
+4 1 2 17 18
+4 2 7 12 17
+4 2 3 6 7
+4 3 4 5 6
+4 7 8 11 12
+4 8 9 10 11
+4 12 13 16 17
+4 13 14 15 16
+
+4 1 0 20 21
+4 2 1 21 22
+4 3 2 22 23
+4 4 3 23 24
+4 5 4 24 25
+4 6 5 25 26
+4 7 6 26 27
+4 8 7 27 28
+4 9 8 28 29
+4 10 9 29 30
+4 11 10 30 31
+4 12 11 31 32
+4 13 12 32 33
+4 14 13 33 34
+4 15 14 34 35
+4 16 15 35 36
+4 17 16 36 37
+4 18 17 37 38
+4 19 18 38 39
+4 0 19 39 20
+
+4 39 38 21 20
+4 38 37 22 21
+4 37 32 27 22
+4 27 26 23 22
+4 26 25 24 23
+4 32 31 28 27
+4 31 30 29 28
+4 37 36 33 32
+4 36 35 34 33
diff -pruN 5.0.3-1/examples/Nef_3/draw_nef_3.cpp 5.1-1/examples/Nef_3/draw_nef_3.cpp
--- 5.0.3-1/examples/Nef_3/draw_nef_3.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Nef_3/draw_nef_3.cpp	2020-09-08 12:12:19.000000000 +0000
@@ -0,0 +1,27 @@
+#include <CGAL/Exact_predicates_exact_constructions_kernel.h>
+#include <CGAL/Nef_polyhedron_3.h>
+#include <CGAL/Polyhedron_3.h>
+#include <CGAL/draw_nef_3.h>
+
+#include <fstream>
+#include <iostream>
+
+typedef CGAL::Exact_predicates_exact_constructions_kernel   Kernel;
+typedef CGAL::Polyhedron_3<Kernel>                          Polyhedron;
+typedef CGAL::Nef_polyhedron_3<Kernel>                      Nef_polyhedron;
+
+int main(int argc, char *argv[])
+{
+  // read OFF file into a polyhedron
+  Polyhedron P;
+  std::ifstream ifs((argc > 1) ? argv[1] : "data/cross.off");
+  ifs >> P;
+
+  // initialize nef from polyhedron
+  Nef_polyhedron N(P);
+
+  // draw Nef Polyhedron
+  CGAL::draw(N);
+
+  return EXIT_SUCCESS;
+}
diff -pruN 5.0.3-1/examples/Optimal_bounding_box/CMakeLists.txt 5.1-1/examples/Optimal_bounding_box/CMakeLists.txt
--- 5.0.3-1/examples/Optimal_bounding_box/CMakeLists.txt	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Optimal_bounding_box/CMakeLists.txt	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,32 @@
+# Created by the script cgal_create_cmake_script
+# This is the CMake script for compiling a CGAL application.
+
+cmake_minimum_required(VERSION 3.1...3.15)
+project( Optimal_bounding_box_Examples )
+
+find_package(CGAL QUIET)
+
+if (NOT CGAL_FOUND)
+  message(STATUS "This project requires the CGAL library, and will not be compiled.")
+  return()
+endif()
+
+include( ${CGAL_USE_FILE} )
+
+find_package(Eigen3 3.1.0 REQUIRED) #(3.1.0 or greater)
+include(CGAL_Eigen_support)
+if (NOT TARGET CGAL::Eigen_support)
+  message(STATUS "This project requires the Eigen library, and will not be compiled.")
+  return()
+endif()
+
+create_single_source_cgal_program("obb_example.cpp")
+create_single_source_cgal_program("obb_with_point_maps_example.cpp")
+create_single_source_cgal_program("rotated_aabb_tree_example.cpp")
+
+foreach(target
+        obb_example
+        obb_with_point_maps_example
+        rotated_aabb_tree_example)
+  target_link_libraries(${target} PUBLIC CGAL::Eigen_support)
+endforeach()
diff -pruN 5.0.3-1/examples/Optimal_bounding_box/data/pig.off 5.1-1/examples/Optimal_bounding_box/data/pig.off
--- 5.0.3-1/examples/Optimal_bounding_box/data/pig.off	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Optimal_bounding_box/data/pig.off	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,1361 @@
+OFF
+468 891 0
+0.063974 0.101970 -0.415827
+0.028523 0.114259 -0.442328
+0.063412 0.118087 -0.415381
+0.077614 0.162174 -0.435627
+0.128990 0.122892 -0.393541
+0.122864 -0.147913 0.401369
+0.178320 -0.063881 0.376630
+0.131227 -0.089511 0.441084
+0.164639 -0.008344 0.415245
+0.103820 -0.043044 -0.278399
+0.091834 -0.084101 -0.204746
+0.079859 -0.055424 -0.310790
+0.118637 -0.026027 -0.235186
+0.099707 0.153598 0.166104
+0.094930 0.196480 0.152483
+0.115864 0.191688 0.136248
+0.171734 0.181103 0.136002
+0.193585 0.183376 0.154622
+0.164526 0.109059 0.214381
+-0.153221 0.150924 -0.394866
+-0.081493 0.167834 -0.436769
+-0.072517 0.124239 -0.414300
+-0.161186 0.176848 -0.378966
+-0.176790 0.132941 -0.334836
+-0.171249 0.164538 -0.303896
+-0.080475 0.201851 -0.109296
+-0.071494 0.183497 -0.140195
+-0.097522 0.199020 -0.144832
+-0.056652 0.190914 -0.054368
+-0.039241 0.197263 -0.153247
+-0.141604 0.199711 -0.154047
+-0.129425 0.157767 -0.137661
+-0.165583 0.197728 -0.148208
+-0.190944 0.201695 -0.114791
+-0.184883 0.199199 -0.132786
+-0.178540 0.131594 -0.070966
+-0.153766 0.018107 -0.272151
+-0.130012 -0.023104 -0.280793
+-0.113295 -0.035252 -0.248969
+-0.199246 0.193683 0.195561
+-0.180750 0.123251 0.255085
+-0.181651 0.208804 0.233547
+-0.193764 0.183287 0.154611
+-0.183901 0.083655 0.279856
+-0.171726 0.068282 0.231479
+0.149160 0.094349 0.371830
+0.173037 0.085349 0.334114
+0.168678 0.151411 0.318997
+-0.168351 0.090355 0.342446
+-0.191982 0.004460 0.346603
+0.010306 -0.068937 0.495666
+-0.054048 -0.045698 0.495964
+-0.036648 -0.113382 0.498431
+0.049415 -0.137244 0.485860
+-0.135590 0.204770 -0.026654
+-0.164959 0.204458 -0.052698
+-0.119997 0.140111 0.025175
+0.172825 0.080263 -0.317843
+0.160628 0.062369 -0.340044
+0.177177 0.143953 -0.350070
+0.118606 0.021487 -0.375861
+0.165354 0.008939 -0.353730
+0.237839 0.022829 -0.306004
+0.212030 0.037710 -0.318516
+0.235958 0.004327 -0.346347
+0.269290 -0.012961 -0.323921
+0.177418 -0.011732 -0.301843
+0.134912 0.006718 -0.402388
+0.237840 -0.015554 -0.414292
+0.177088 -0.010878 -0.417488
+0.234583 -0.001111 -0.396570
+0.165512 -0.038754 -0.394103
+0.274848 -0.015963 -0.391667
+0.284810 -0.026997 -0.376984
+0.281300 -0.029101 -0.338206
+0.217331 -0.034592 -0.352687
+0.196409 -0.028130 -0.326693
+0.146613 0.129225 0.183979
+0.125633 0.114296 0.184366
+0.168417 0.063049 0.228672
+0.131279 0.094069 0.157481
+0.093427 0.144615 0.148946
+0.151538 0.030944 0.165340
+0.040532 0.197143 -0.154026
+0.056681 0.209038 -0.212225
+0.002775 0.206856 -0.166610
+0.117369 0.153354 -0.205966
+0.124488 0.200090 -0.152498
+0.086066 0.183902 -0.142107
+0.091309 0.198441 -0.138090
+0.064272 0.187025 -0.133132
+0.080301 0.201940 -0.109295
+0.087605 0.207177 -0.050256
+0.071100 0.186484 -0.043141
+0.144787 0.163143 -0.404482
+0.163037 0.176611 -0.376916
+0.093064 0.199241 -0.432443
+0.133133 0.208266 -0.382604
+0.143249 0.191538 -0.272444
+0.158095 0.192739 -0.314330
+-0.139488 0.186720 -0.261262
+-0.151500 0.199626 -0.324260
+-0.098372 0.223627 -0.288685
+-0.078536 0.189639 -0.197355
+-0.039805 0.227673 -0.260155
+0.097127 0.229527 -0.341482
+0.062369 0.228756 -0.395152
+0.036595 0.231173 -0.450000
+-0.028016 0.125004 -0.448876
+-0.036461 0.096667 -0.434533
+-0.083077 0.090894 -0.414311
+-0.082268 0.070800 -0.428081
+-0.123341 -0.003528 -0.217671
+-0.111649 -0.041351 -0.203434
+-0.143481 0.129458 -0.120457
+-0.166571 0.125111 -0.100292
+-0.091445 0.155603 -0.139780
+-0.108949 0.126603 -0.132705
+-0.155764 -0.002392 -0.117269
+-0.109909 -0.047484 -0.302472
+-0.091801 -0.053960 -0.282808
+-0.111273 -0.076464 -0.152548
+-0.089774 -0.101654 -0.175408
+-0.131191 -0.018764 -0.145092
+-0.117239 -0.118603 -0.106400
+-0.186719 -0.019496 0.218873
+0.094316 -0.206017 0.232131
+0.055117 -0.224837 0.203521
+0.079552 -0.189464 0.070678
+0.040403 -0.204104 0.063479
+0.071493 -0.170167 -0.083686
+-0.180726 -0.068980 0.362373
+-0.182633 -0.099623 0.308202
+-0.158938 -0.138266 0.334999
+0.085658 0.200413 0.176418
+0.065852 0.170004 0.177725
+-0.001292 0.195544 -0.033434
+-0.285400 -0.031989 -0.344981
+-0.271603 -0.015198 -0.325428
+-0.278372 -0.020235 -0.389284
+-0.206668 0.006555 -0.349794
+-0.225044 -0.007684 -0.420462
+-0.238803 0.004342 -0.347955
+-0.158634 0.013703 -0.350809
+-0.228727 0.004252 -0.318223
+-0.223154 0.035457 -0.308571
+-0.193549 0.057123 -0.317739
+-0.210450 -0.029622 -0.405169
+0.172645 -0.045708 -0.037575
+0.158143 -0.075581 -0.062747
+0.161129 -0.053779 -0.087832
+0.151586 -0.077141 0.017952
+0.172046 0.216581 0.250518
+0.160045 0.220693 0.263343
+0.199056 0.193772 0.195561
+0.181331 0.122923 0.257005
+0.183238 0.081029 0.279971
+0.186394 0.009376 0.360522
+0.200356 -0.009475 0.302375
+0.186432 0.204012 -0.088367
+0.166519 0.185025 -0.035652
+0.178625 0.148640 -0.039831
+0.162273 -0.038823 0.020282
+0.164324 0.000657 0.028308
+0.191989 -0.031903 0.230015
+0.188950 -0.072032 0.237843
+0.159438 -0.018081 0.152399
+0.179961 -0.012368 0.206350
+0.154184 -0.071463 0.150146
+0.157446 -0.054932 0.128257
+0.146606 -0.094660 0.103787
+-0.085574 0.204703 0.235486
+-0.100152 0.207801 0.255866
+-0.059913 0.174502 0.263217
+-0.177675 0.138122 0.302573
+-0.166640 0.138235 0.331535
+-0.160235 0.220604 0.263332
+0.065130 0.176233 0.262848
+0.085351 0.204781 0.235497
+0.062759 0.177657 0.222293
+0.099947 0.207890 0.255911
+0.029319 0.167713 0.235458
+-0.007308 0.176521 0.212726
+0.018184 0.155832 0.282328
+0.016196 0.134242 0.331153
+-0.115478 0.114685 0.373666
+-0.138087 0.098705 0.375545
+-0.107498 0.047438 0.447839
+-0.136533 0.029923 0.438192
+-0.151444 0.150957 0.333934
+-0.128669 0.172997 0.320075
+-0.141960 0.217696 0.269060
+-0.070700 0.154566 0.313918
+-0.089073 0.135316 0.352129
+0.113012 -0.180473 0.171463
+0.103166 -0.168475 0.072737
+0.137034 -0.171125 0.287599
+0.153718 -0.147893 0.232349
+0.149223 0.117836 -0.382727
+0.122958 0.086674 -0.404777
+0.070966 0.060503 -0.432879
+0.046513 0.085855 -0.431266
+0.092912 0.013673 -0.404533
+0.090952 -0.016720 -0.385023
+0.077405 -0.042537 -0.356280
+0.058093 -0.059147 -0.335116
+0.068427 -0.123215 -0.187213
+0.101335 -0.132006 -0.117248
+0.097814 -0.149292 -0.079001
+0.122020 -0.125228 -0.016951
+0.126097 -0.138388 0.097790
+0.126318 -0.145156 0.141802
+0.174282 -0.112276 0.330051
+0.190029 -0.084340 0.294626
+0.058942 -0.222818 0.301995
+0.098418 -0.193164 0.340401
+0.058809 -0.206985 0.375651
+0.021923 -0.222862 0.367343
+0.014505 -0.238117 0.299646
+0.122283 0.121443 0.075831
+0.113763 0.148142 0.020311
+0.140282 0.123337 0.018261
+0.027980 -0.177863 -0.119253
+0.026130 -0.193179 -0.070708
+-0.001321 -0.211350 0.079165
+-0.001523 -0.197311 -0.066313
+0.166925 0.094073 -0.266778
+0.163733 0.142170 -0.259072
+0.127467 0.088406 -0.181953
+0.150376 0.077937 0.039615
+0.190766 0.201784 -0.114791
+0.178517 0.136767 -0.071037
+0.184704 0.199288 -0.132775
+0.166379 0.133552 -0.102642
+0.165222 0.047122 -0.102044
+0.078264 0.226830 -0.278641
+0.019112 0.233265 -0.274429
+-0.039110 -0.233486 0.287357
+-0.021609 -0.223013 0.367699
+0.001943 -0.188166 0.439819
+-0.066652 -0.213379 0.338708
+0.027726 -0.233930 0.210941
+-0.000203 -0.237188 0.219039
+-0.136376 -0.172085 0.288837
+-0.099814 -0.201744 0.258297
+-0.027582 -0.234160 0.217991
+-0.060662 -0.218919 0.186226
+-0.055734 -0.201562 0.080354
+-0.029186 -0.209148 0.081362
+-0.029001 -0.192501 -0.071644
+-0.160342 0.063827 0.024713
+-0.177523 0.015890 -0.001911
+-0.162105 0.020607 0.040724
+-0.180080 0.125890 -0.032858
+-0.185297 0.023823 -0.038772
+-0.175803 -0.030161 -0.021666
+-0.149501 -0.083919 0.026313
+-0.160461 -0.041091 0.033786
+-0.156993 -0.049877 0.148952
+-0.179046 0.029177 -0.073993
+-0.177954 -0.024883 -0.058372
+-0.151976 0.126423 -0.229667
+-0.163106 0.092405 -0.253555
+-0.121432 0.099265 -0.173188
+-0.111794 0.137703 -0.178117
+-0.166077 0.091402 -0.340017
+-0.161312 0.070001 -0.345025
+-0.167665 0.059706 -0.282855
+-0.171035 0.078014 -0.296062
+-0.126978 0.030384 -0.171087
+-0.129126 0.050437 -0.192177
+-0.131368 0.059086 -0.126361
+-0.134950 0.025663 -0.131342
+-0.143437 -0.069595 -0.109286
+-0.148269 -0.039929 -0.120515
+-0.163978 -0.065334 -0.065543
+-0.167235 -0.061629 -0.042774
+0.002745 0.189510 0.110554
+0.036689 0.188942 0.019665
+-0.036527 0.174422 0.177147
+-0.087796 0.207088 -0.050265
+-0.076524 0.172540 0.006138
+-0.087508 0.177933 -0.017043
+-0.108099 0.206330 -0.029115
+-0.072629 0.172458 0.181190
+-0.090040 0.162292 0.168261
+-0.089665 0.198396 0.161152
+-0.063826 0.178856 0.211964
+-0.149794 0.182785 0.130788
+-0.116064 0.191599 0.136236
+-0.150418 0.160583 0.151346
+-0.171912 0.181013 0.135924
+-0.138972 0.123679 0.183787
+-0.105455 0.169777 0.154956
+-0.106937 0.141313 0.167043
+-0.064997 0.162208 0.148704
+-0.089200 0.146757 0.141467
+-0.116460 0.121274 0.141967
+0.130312 0.041995 0.435860
+0.129704 -0.002210 0.453987
+0.119563 0.095019 0.389830
+0.106258 -0.067817 0.474757
+0.072458 -0.092968 0.490348
+0.035744 -0.100524 0.501598
+0.090915 -0.018706 0.481902
+0.063954 -0.018620 0.488394
+0.012687 0.019863 0.473158
+-0.066639 0.014188 0.477250
+-0.001853 -0.014358 0.486138
+-0.073991 0.051995 0.453618
+0.015992 0.056915 0.445325
+-0.072272 0.083609 0.419400
+0.125713 0.148591 0.340006
+0.151075 0.147959 0.335541
+0.120626 0.212102 0.266920
+0.069491 0.149561 0.322080
+0.091434 0.136280 0.349720
+0.076196 0.038920 0.461948
+0.080702 0.078897 0.423801
+0.006477 0.083890 0.414722
+0.003928 0.102802 0.385224
+-0.068084 0.039517 -0.430070
+-0.017676 0.023776 -0.427482
+0.193970 0.042648 -0.293997
+0.172856 0.064667 -0.285834
+0.158578 0.030796 -0.272331
+0.140866 -0.005417 -0.270488
+0.134432 0.029296 -0.214306
+0.126251 0.015555 -0.188439
+0.132124 0.003792 -0.136979
+0.125810 -0.029504 -0.153529
+0.129423 0.061526 -0.128092
+0.108063 0.127431 -0.132395
+0.091794 0.155522 -0.139751
+0.124048 0.145248 -0.131126
+0.184444 0.016147 -0.030529
+0.181489 -0.003989 -0.053023
+0.141425 0.199800 -0.154047
+0.165405 0.197817 -0.148141
+0.144876 0.183940 -0.146208
+0.154368 0.144703 -0.122262
+0.146068 0.119123 -0.117735
+0.154841 -0.033503 -0.111889
+0.143738 -0.032315 -0.128787
+0.113871 -0.078221 -0.157136
+0.112005 -0.054404 -0.183556
+0.010090 -0.146944 0.487883
+0.011570 -0.146473 0.485554
+0.010090 -0.136782 0.493756
+-0.015165 -0.128494 0.494118
+-0.010143 -0.136180 0.492642
+-0.030104 -0.153530 0.480762
+-0.010143 -0.146454 0.487048
+-0.006480 -0.157970 0.476349
+0.045456 -0.167686 0.453108
+0.029848 -0.154023 0.480267
+0.030579 -0.132753 -0.220866
+0.021176 -0.090002 -0.310105
+0.181147 0.115479 -0.036001
+0.175183 0.070269 -0.003253
+0.161619 0.042238 0.028286
+0.156945 0.037056 0.086395
+0.051155 0.182427 -0.491632
+0.031566 0.180896 -0.501667
+0.002176 0.193366 -0.489790
+-0.037122 0.180476 -0.500027
+-0.025020 0.194670 -0.492815
+0.055556 0.214167 -0.473439
+0.066526 0.214610 -0.467774
+-0.082413 0.208049 -0.429210
+-0.065055 0.215687 -0.469354
+-0.062206 0.204472 -0.479930
+-0.036973 0.231911 -0.450571
+-0.036983 0.226530 -0.458574
+-0.125722 0.187261 -0.415111
+-0.091376 0.197028 -0.433439
+-0.109535 0.220672 -0.374442
+-0.132871 0.206436 -0.383456
+-0.068993 0.234380 -0.324450
+-0.054731 0.231939 -0.379365
+-0.015007 0.238836 -0.387236
+0.010198 0.238083 -0.347126
+-0.171832 0.010661 0.394832
+-0.164229 -0.033313 0.411342
+-0.071226 -0.093884 0.490608
+-0.099723 -0.021444 0.479401
+-0.106460 -0.079934 0.470702
+-0.126287 -0.023902 0.459235
+-0.151985 -0.067142 0.420950
+-0.147699 -0.107939 0.402674
+-0.109438 -0.119557 0.446423
+-0.020037 -0.195034 0.426407
+-0.045003 -0.203863 0.398142
+-0.079909 -0.190588 0.383979
+-0.073176 -0.153947 0.447608
+-0.060832 -0.134709 0.479062
+-0.132591 -0.160951 0.350157
+-0.119567 -0.148332 0.402715
+-0.065793 -0.128601 -0.175205
+-0.097272 -0.145676 -0.099799
+-0.002565 -0.181714 -0.122217
+-0.024515 -0.179484 -0.119506
+-0.064586 -0.176266 -0.074826
+-0.059683 -0.167369 -0.114486
+-0.099658 -0.163716 0.022325
+-0.040368 -0.148020 -0.173886
+-0.152092 0.033745 0.173508
+-0.159192 -0.012910 0.157435
+-0.186610 0.203923 -0.088370
+-0.183267 0.184340 -0.063974
+-0.136575 0.103373 0.069442
+-0.143137 0.116556 0.022602
+-0.200449 -0.005081 0.293234
+-0.198541 -0.044298 0.289997
+-0.176233 -0.086749 0.211940
+-0.129605 -0.146690 0.159205
+-0.172832 -0.114168 0.232984
+-0.146311 -0.094428 0.150409
+-0.147372 -0.095199 0.103658
+-0.110455 -0.178169 0.151965
+-0.128236 -0.127687 0.039765
+-0.128792 -0.120162 -0.025065
+0.064160 0.162938 0.145558
+0.035172 0.172321 0.185202
+-0.006127 0.202885 -0.099910
+-0.034296 0.191647 -0.029673
+-0.131196 0.116627 -0.390365
+-0.142598 0.108734 -0.385698
+-0.118388 0.084666 -0.406795
+-0.122705 0.099621 -0.397329
+-0.118280 0.017285 -0.383763
+-0.138200 0.021970 -0.357392
+-0.083219 -0.042897 -0.356408
+-0.171574 -0.015949 -0.418570
+-0.079870 -0.020831 -0.382774
+-0.146033 -0.002291 -0.413268
+-0.099733 0.026973 -0.404345
+-0.098251 0.013991 -0.401317
+0.044369 -0.016396 -0.405838
+0.042389 0.006708 -0.423304
+-0.042891 -0.004873 -0.416215
+-0.012107 -0.020909 -0.408756
+-0.205436 -0.024781 -0.328409
+-0.223220 -0.034022 -0.353666
+-0.149992 -0.036689 -0.391942
+-0.164885 -0.044415 -0.367829
+-0.091425 -0.051851 -0.329759
+-0.067188 -0.061962 -0.316028
+0.032718 -0.045780 -0.374356
+0.001142 -0.045018 -0.382344
+-0.008169 -0.088329 -0.317598
+-0.061474 -0.052657 -0.346404
+-0.036931 -0.052082 -0.363104
+-0.163689 -0.139768 0.263934
+-0.134461 -0.171429 0.220297
+-0.093110 -0.202067 0.185697
+-0.083335 -0.193614 0.109481
+-0.161181 0.096630 0.214117
+-0.169578 0.129140 0.197122
+-0.136316 0.091909 0.186973
+-0.137551 0.081713 0.154719
+0.129236 0.172656 -0.007278
+0.135409 0.204859 -0.026639
+0.107912 0.206419 -0.029102
+0.080873 0.173368 -0.015229
+0.100091 0.179921 -0.016858
+-0.000035 -0.119696 -0.260509
+-0.020383 -0.117167 -0.258779
+3 0 1 2
+3 2 3 4
+3 1 3 2
+3 5 6 7
+3 6 8 7
+3 9 10 11
+3 9 12 10
+3 13 14 15
+3 15 16 13
+3 17 18 16
+3 19 20 21
+3 22 23 24
+3 23 22 19
+3 25 26 27
+3 28 29 26
+3 30 31 32
+3 27 26 30
+3 30 26 31
+3 33 34 35
+3 36 37 38
+3 39 40 41
+3 42 40 39
+3 43 40 44
+3 45 46 47
+3 48 43 49
+3 50 51 52
+3 5 7 53
+3 54 55 56
+3 57 58 59
+3 60 58 61
+3 62 63 57
+3 63 64 61
+3 63 61 58
+3 63 58 57
+3 65 64 63
+3 65 63 62
+3 66 65 62
+3 61 67 60
+3 68 69 70
+3 71 69 68
+3 69 67 70
+3 70 67 61
+3 61 64 70
+3 72 73 68
+3 72 68 70
+3 72 70 64
+3 72 65 73
+3 72 64 65
+3 73 65 74
+3 68 73 71
+3 75 74 76
+3 74 65 76
+3 76 65 66
+3 76 66 9
+3 75 76 11
+3 76 9 11
+3 75 11 71
+3 71 73 75
+3 73 74 75
+3 77 16 18
+3 78 77 18
+3 78 18 79
+3 78 80 81
+3 78 13 77
+3 16 77 13
+3 13 78 81
+3 78 82 80
+3 82 78 79
+3 83 84 85
+3 83 86 84
+3 85 29 28
+3 87 88 89
+3 89 88 90
+3 91 89 90
+3 92 91 93
+3 90 93 91
+3 94 95 59
+3 95 94 96
+3 95 96 97
+3 3 96 94
+3 86 98 84
+3 59 95 99
+3 99 95 97
+3 100 101 24
+3 101 22 24
+3 101 100 102
+3 103 102 100
+3 102 103 104
+3 103 29 104
+3 104 29 85
+3 99 105 98
+3 105 97 106
+3 99 97 105
+3 96 106 97
+3 106 96 107
+3 21 108 109
+3 21 20 108
+3 110 21 109
+3 110 109 111
+3 112 38 113
+3 112 36 38
+3 114 115 32
+3 114 32 31
+3 115 35 34
+3 34 32 115
+3 116 117 114
+3 116 114 31
+3 31 26 116
+3 29 116 26
+3 116 29 103
+3 114 118 115
+3 117 118 114
+3 119 120 37
+3 38 37 120
+3 121 113 122
+3 38 122 113
+3 38 120 122
+3 123 113 121
+3 122 124 121
+3 44 125 43
+3 126 127 128
+3 128 129 130
+3 129 128 127
+3 131 132 133
+3 134 14 135
+3 135 14 13
+3 13 81 135
+3 136 90 83
+3 137 138 139
+3 140 141 142
+3 139 142 141
+3 139 138 142
+3 140 143 141
+3 144 36 145
+3 138 144 145
+3 143 145 146
+3 143 140 145
+3 140 142 145
+3 138 145 142
+3 36 144 37
+3 147 137 139
+3 141 147 139
+3 148 149 150
+3 151 149 148
+3 152 153 47
+3 154 152 47
+3 47 46 154
+3 18 155 156
+3 79 18 156
+3 18 17 155
+3 154 155 17
+3 156 155 46
+3 154 46 155
+3 157 6 158
+3 156 157 158
+3 46 157 156
+3 157 46 45
+3 157 45 8
+3 8 6 157
+3 159 160 161
+3 162 148 163
+3 151 148 162
+3 164 156 158
+3 164 158 165
+3 166 82 167
+3 167 79 164
+3 79 156 164
+3 82 79 167
+3 168 169 166
+3 168 170 169
+3 167 168 166
+3 164 168 167
+3 164 165 168
+3 170 151 169
+3 162 169 151
+3 163 169 162
+3 163 166 169
+3 171 172 173
+3 41 174 175
+3 40 174 41
+3 40 43 174
+3 175 174 48
+3 48 174 43
+3 41 175 176
+3 177 178 179
+3 177 180 178
+3 134 179 178
+3 134 135 179
+3 181 182 183
+3 179 181 183
+3 183 182 173
+3 183 184 177
+3 177 179 183
+3 179 135 181
+3 185 186 187
+3 48 186 175
+3 188 186 48
+3 188 187 186
+3 176 189 190
+3 190 189 185
+3 176 175 189
+3 189 175 186
+3 186 185 189
+3 191 176 190
+3 192 193 184
+3 192 184 183
+3 173 192 183
+3 192 173 172
+3 193 192 191
+3 192 172 191
+3 191 190 193
+3 190 185 193
+3 194 128 195
+3 128 130 195
+3 196 194 197
+3 196 126 194
+3 126 128 194
+3 198 199 0
+3 4 198 0
+3 198 4 3
+3 198 3 94
+3 94 59 198
+3 58 198 59
+3 198 58 199
+3 60 199 58
+3 200 201 199
+3 109 108 201
+3 201 108 1
+3 201 1 0
+3 199 201 0
+3 202 67 203
+3 199 202 200
+3 202 199 60
+3 67 202 60
+3 67 69 203
+3 71 203 69
+3 203 204 202
+3 71 204 203
+3 205 204 11
+3 11 10 205
+3 204 71 11
+3 130 206 207
+3 10 207 206
+3 206 205 10
+3 207 150 149
+3 195 208 209
+3 151 170 209
+3 195 130 208
+3 208 130 207
+3 149 209 208
+3 149 151 209
+3 208 207 149
+3 209 210 195
+3 170 210 209
+3 210 168 211
+3 210 170 168
+3 168 165 211
+3 197 211 165
+3 195 210 211
+3 194 211 197
+3 194 195 211
+3 212 197 213
+3 213 197 165
+3 213 165 158
+3 213 6 212
+3 213 158 6
+3 5 212 6
+3 5 196 212
+3 212 196 197
+3 214 215 216
+3 215 126 196
+3 5 215 196
+3 5 216 215
+3 216 217 214
+3 218 214 217
+3 214 218 127
+3 215 214 126
+3 127 126 214
+3 219 220 81
+3 80 219 81
+3 220 219 221
+3 221 161 160
+3 222 130 223
+3 223 130 129
+3 129 224 223
+3 223 225 222
+3 225 223 224
+3 226 227 228
+3 226 59 227
+3 226 57 59
+3 59 99 227
+3 227 99 98
+3 227 98 86
+3 228 227 86
+3 161 221 229
+3 221 219 229
+3 229 219 80
+3 161 230 159
+3 230 161 231
+3 232 230 233
+3 233 231 234
+3 230 231 233
+3 235 105 236
+3 98 105 235
+3 104 85 236
+3 235 236 84
+3 235 84 98
+3 84 236 85
+3 237 218 238
+3 217 238 218
+3 217 239 238
+3 237 238 240
+3 224 241 242
+3 129 241 224
+3 127 241 129
+3 242 241 218
+3 241 127 218
+3 218 237 242
+3 240 243 244
+3 240 244 237
+3 242 245 224
+3 246 245 237
+3 237 244 246
+3 245 242 237
+3 247 248 246
+3 248 247 249
+3 248 249 225
+3 224 248 225
+3 245 248 224
+3 245 246 248
+3 250 251 252
+3 253 251 250
+3 251 254 255
+3 254 251 253
+3 253 35 254
+3 256 257 255
+3 257 258 252
+3 252 251 257
+3 255 257 251
+3 259 118 260
+3 35 259 254
+3 35 115 259
+3 118 259 115
+3 254 259 260
+3 260 255 254
+3 261 24 262
+3 261 100 24
+3 24 23 262
+3 263 264 261
+3 264 263 117
+3 117 116 264
+3 116 103 264
+3 264 103 100
+3 261 264 100
+3 265 266 146
+3 23 19 265
+3 267 268 146
+3 267 262 268
+3 268 262 23
+3 268 23 265
+3 146 268 265
+3 145 267 146
+3 145 36 267
+3 267 36 112
+3 269 263 270
+3 270 112 269
+3 270 263 261
+3 270 261 262
+3 267 270 262
+3 270 267 112
+3 112 113 269
+3 123 269 113
+3 271 272 118
+3 123 272 269
+3 117 271 118
+3 272 271 263
+3 272 263 269
+3 263 271 117
+3 121 273 274
+3 124 273 121
+3 274 260 118
+3 272 274 118
+3 272 123 274
+3 274 123 121
+3 275 276 260
+3 276 256 255
+3 276 255 260
+3 275 260 274
+3 275 274 273
+3 275 273 124
+3 136 277 278
+3 277 136 279
+3 277 279 182
+3 93 90 278
+3 278 90 136
+3 28 26 280
+3 25 280 26
+3 281 282 56
+3 281 28 282
+3 280 282 28
+3 283 282 280
+3 283 54 282
+3 56 282 54
+3 284 285 286
+3 284 286 287
+3 171 287 286
+3 171 173 287
+3 182 287 173
+3 279 284 287
+3 279 287 182
+3 288 289 290
+3 288 290 291
+3 290 292 291
+3 293 294 290
+3 292 290 294
+3 289 293 290
+3 289 286 293
+3 293 285 294
+3 286 285 293
+3 295 281 296
+3 284 279 295
+3 56 296 281
+3 56 297 296
+3 297 294 296
+3 295 296 285
+3 284 295 285
+3 285 296 294
+3 298 299 8
+3 8 45 298
+3 45 300 298
+3 7 8 301
+3 299 301 8
+3 53 301 302
+3 301 53 7
+3 303 53 302
+3 301 304 302
+3 299 304 301
+3 298 304 299
+3 304 305 303
+3 304 303 302
+3 303 305 50
+3 306 307 308
+3 305 306 308
+3 51 308 307
+3 50 308 51
+3 308 50 305
+3 309 307 306
+3 310 309 306
+3 193 185 311
+3 311 187 309
+3 309 187 307
+3 185 187 311
+3 153 312 313
+3 313 47 153
+3 153 314 312
+3 313 312 45
+3 45 47 313
+3 300 45 312
+3 315 180 177
+3 315 177 184
+3 300 312 316
+3 316 314 315
+3 314 180 315
+3 314 316 312
+3 317 318 310
+3 310 306 317
+3 305 317 306
+3 305 304 317
+3 298 317 304
+3 317 298 318
+3 298 300 318
+3 318 300 316
+3 319 320 311
+3 315 184 320
+3 320 184 193
+3 193 311 320
+3 319 311 309
+3 310 319 309
+3 318 319 310
+3 320 319 316
+3 316 315 320
+3 318 316 319
+3 321 111 322
+3 109 322 111
+3 201 322 109
+3 201 200 322
+3 62 323 66
+3 57 324 323
+3 62 57 323
+3 324 57 226
+3 66 325 326
+3 9 326 12
+3 66 326 9
+3 323 325 66
+3 323 324 325
+3 324 226 325
+3 327 228 328
+3 328 12 327
+3 12 326 327
+3 326 325 327
+3 226 327 325
+3 226 228 327
+3 329 330 328
+3 329 328 228
+3 331 329 228
+3 331 228 332
+3 333 332 86
+3 332 228 86
+3 333 86 83
+3 83 90 333
+3 90 88 333
+3 333 88 334
+3 335 148 336
+3 335 163 148
+3 148 150 336
+3 336 231 335
+3 336 234 231
+3 337 338 339
+3 88 339 334
+3 337 339 87
+3 87 339 88
+3 340 341 334
+3 339 340 334
+3 340 339 338
+3 340 338 232
+3 233 340 232
+3 233 341 340
+3 233 234 341
+3 341 332 333
+3 341 333 334
+3 234 336 342
+3 343 342 150
+3 336 150 342
+3 331 343 329
+3 331 332 343
+3 332 341 343
+3 342 343 234
+3 343 330 329
+3 343 341 234
+3 330 344 345
+3 328 345 12
+3 328 330 345
+3 330 343 344
+3 344 343 150
+3 344 150 207
+3 345 344 10
+3 12 345 10
+3 207 10 344
+3 346 347 348
+3 303 347 53
+3 347 303 348
+3 349 350 303
+3 348 303 350
+3 50 349 303
+3 50 52 349
+3 52 351 349
+3 352 349 353
+3 352 350 349
+3 349 351 353
+3 351 239 353
+3 346 352 353
+3 353 347 346
+3 354 53 355
+3 354 355 239
+3 347 355 53
+3 347 353 355
+3 355 353 239
+3 354 239 217
+3 216 354 217
+3 5 354 216
+3 5 53 354
+3 205 356 357
+3 356 205 206
+3 356 206 130
+3 222 356 130
+3 335 231 358
+3 161 358 231
+3 358 161 359
+3 359 161 229
+3 335 358 359
+3 335 359 163
+3 360 361 163
+3 359 360 163
+3 360 229 361
+3 359 229 360
+3 80 361 229
+3 361 80 82
+3 361 82 166
+3 163 361 166
+3 3 362 96
+3 362 1 363
+3 1 362 3
+3 364 363 365
+3 363 1 365
+3 1 108 365
+3 364 365 366
+3 20 365 108
+3 367 107 368
+3 96 368 107
+3 368 96 362
+3 363 367 368
+3 364 367 363
+3 362 363 368
+3 369 370 371
+3 371 366 365
+3 365 20 371
+3 372 373 370
+3 107 373 372
+3 369 372 370
+3 107 367 373
+3 373 367 364
+3 364 366 373
+3 366 371 373
+3 371 370 373
+3 20 374 375
+3 375 369 371
+3 375 371 20
+3 19 374 20
+3 374 19 22
+3 376 369 377
+3 369 375 377
+3 377 101 376
+3 377 375 374
+3 374 22 377
+3 101 377 22
+3 101 102 376
+3 378 379 376
+3 369 379 372
+3 376 379 369
+3 376 102 378
+3 102 104 378
+3 380 378 381
+3 379 380 372
+3 379 378 380
+3 378 104 381
+3 236 381 104
+3 105 381 236
+3 105 106 381
+3 381 107 380
+3 380 107 372
+3 381 106 107
+3 382 383 188
+3 383 382 131
+3 188 48 382
+3 48 49 382
+3 49 131 382
+3 384 52 51
+3 384 51 385
+3 307 385 51
+3 386 385 387
+3 387 385 188
+3 386 384 385
+3 187 385 307
+3 187 188 385
+3 387 188 383
+3 388 131 389
+3 131 133 389
+3 389 390 388
+3 388 390 386
+3 386 387 388
+3 387 383 388
+3 388 383 131
+3 238 391 392
+3 392 393 240
+3 240 238 392
+3 239 391 238
+3 351 392 391
+3 391 239 351
+3 392 394 393
+3 351 395 394
+3 392 351 394
+3 52 395 351
+3 384 395 52
+3 395 384 386
+3 395 390 394
+3 395 386 390
+3 396 393 397
+3 393 394 397
+3 390 397 394
+3 389 397 390
+3 396 397 133
+3 389 133 397
+3 243 396 133
+3 243 240 396
+3 240 393 396
+3 124 398 399
+3 124 122 398
+3 122 120 398
+3 400 356 222
+3 400 222 225
+3 249 401 400
+3 225 249 400
+3 402 399 403
+3 399 402 404
+3 398 403 399
+3 398 405 403
+3 405 401 403
+3 249 402 403
+3 247 402 249
+3 403 401 249
+3 406 252 407
+3 407 252 258
+3 407 258 125
+3 406 407 44
+3 125 44 407
+3 408 33 409
+3 408 409 55
+3 33 35 409
+3 409 35 253
+3 250 410 411
+3 250 252 410
+3 411 410 56
+3 56 410 297
+3 411 56 55
+3 411 55 409
+3 253 411 409
+3 411 253 250
+3 412 125 413
+3 412 413 49
+3 413 132 131
+3 413 131 49
+3 412 49 43
+3 412 43 125
+3 414 415 416
+3 414 125 258
+3 416 413 414
+3 414 413 125
+3 416 132 413
+3 415 417 418
+3 414 417 415
+3 418 417 258
+3 414 258 417
+3 418 258 257
+3 256 418 257
+3 404 419 420
+3 415 420 419
+3 418 420 415
+3 418 256 420
+3 404 420 421
+3 420 256 421
+3 421 256 276
+3 275 421 276
+3 124 421 275
+3 124 399 421
+3 404 421 399
+3 422 135 81
+3 81 220 422
+3 422 278 423
+3 423 278 277
+3 277 182 423
+3 182 181 423
+3 135 422 423
+3 423 181 135
+3 136 424 425
+3 281 295 425
+3 295 279 425
+3 425 279 136
+3 424 136 83
+3 424 83 85
+3 425 424 28
+3 28 281 425
+3 424 85 28
+3 426 19 21
+3 265 427 266
+3 265 19 427
+3 19 426 427
+3 428 266 427
+3 429 428 427
+3 426 429 427
+3 429 110 428
+3 110 111 428
+3 430 143 431
+3 146 431 143
+3 266 431 146
+3 141 143 430
+3 432 433 434
+3 141 435 433
+3 141 430 435
+3 141 433 147
+3 431 436 430
+3 431 266 436
+3 266 428 436
+3 111 436 428
+3 436 111 321
+3 430 436 437
+3 436 321 437
+3 434 433 437
+3 435 437 433
+3 435 430 437
+3 438 439 202
+3 202 204 438
+3 322 200 439
+3 439 200 202
+3 437 440 434
+3 437 321 440
+3 322 440 321
+3 441 440 439
+3 441 439 438
+3 322 439 440
+3 442 144 443
+3 443 119 442
+3 138 443 144
+3 137 443 138
+3 147 443 137
+3 119 37 442
+3 37 144 442
+3 444 432 445
+3 147 444 445
+3 444 147 433
+3 432 444 433
+3 147 445 443
+3 446 447 119
+3 119 443 446
+3 445 446 443
+3 432 446 445
+3 398 447 405
+3 120 447 398
+3 120 119 447
+3 438 448 449
+3 449 448 357
+3 448 438 204
+3 448 204 205
+3 357 448 205
+3 450 449 357
+3 449 441 438
+3 451 405 447
+3 451 447 446
+3 432 451 446
+3 451 434 452
+3 451 432 434
+3 452 434 440
+3 441 452 440
+3 449 452 441
+3 451 452 450
+3 449 450 452
+3 133 453 243
+3 133 132 453
+3 416 454 453
+3 416 453 132
+3 416 415 454
+3 419 454 415
+3 243 453 454
+3 454 244 243
+3 455 456 247
+3 246 455 247
+3 244 455 246
+3 244 454 455
+3 455 419 456
+3 419 455 454
+3 404 456 419
+3 402 456 404
+3 402 247 456
+3 292 457 458
+3 458 457 40
+3 457 44 40
+3 458 40 42
+3 291 458 42
+3 292 458 291
+3 44 457 459
+3 459 457 292
+3 294 459 292
+3 460 406 459
+3 459 297 460
+3 459 406 44
+3 459 294 297
+3 297 410 460
+3 410 252 460
+3 406 460 252
+3 461 160 462
+3 462 463 461
+3 220 221 461
+3 221 160 461
+3 159 462 160
+3 422 464 278
+3 464 220 465
+3 464 422 220
+3 461 465 220
+3 463 465 461
+3 463 92 465
+3 464 465 93
+3 464 93 278
+3 93 465 92
+3 466 357 356
+3 400 466 356
+3 466 400 467
+3 400 401 467
+3 467 401 405
+3 405 451 467
+3 466 467 450
+3 466 450 357
+3 450 467 451
diff -pruN 5.0.3-1/examples/Optimal_bounding_box/obb_example.cpp 5.1-1/examples/Optimal_bounding_box/obb_example.cpp
--- 5.0.3-1/examples/Optimal_bounding_box/obb_example.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Optimal_bounding_box/obb_example.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,50 @@
+#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
+#include <CGAL/Surface_mesh.h>
+
+#include <CGAL/optimal_bounding_box.h>
+#include <CGAL/Polygon_mesh_processing/triangulate_faces.h>
+#include <CGAL/Polygon_mesh_processing/measure.h>
+
+#include <CGAL/Real_timer.h>
+
+#include <fstream>
+#include <iostream>
+
+namespace PMP = CGAL::Polygon_mesh_processing;
+
+typedef CGAL::Exact_predicates_inexact_constructions_kernel    K;
+typedef K::Point_3                                             Point;
+
+typedef CGAL::Surface_mesh<Point>                              Surface_mesh;
+
+int main(int argc, char** argv)
+{
+  std::ifstream input((argc > 1) ? argv[1] : "data/pig.off");
+  Surface_mesh sm;
+  if(!input || !(input >> sm) || sm.is_empty())
+  {
+    std::cerr << "Invalid input file." << std::endl;
+    return EXIT_FAILURE;
+  }
+
+  CGAL::Real_timer timer;
+  timer.start();
+
+  // Compute the extreme points of the mesh, and then a tightly fitted oriented bounding box
+  std::array<Point, 8> obb_points;
+  CGAL::oriented_bounding_box(sm, obb_points,
+                              CGAL::parameters::use_convex_hull(true));
+
+  std::cout << "Elapsed time: " << timer.time() << std::endl;
+
+  // Make a mesh out of the oriented bounding box
+  Surface_mesh obb_sm;
+  CGAL::make_hexahedron(obb_points[0], obb_points[1], obb_points[2], obb_points[3],
+                        obb_points[4], obb_points[5], obb_points[6], obb_points[7], obb_sm);
+  std::ofstream("obb.off") << obb_sm;
+
+  PMP::triangulate_faces(obb_sm);
+  std::cout << "Volume: " << PMP::volume(obb_sm) << std::endl;
+
+  return EXIT_SUCCESS;
+}
diff -pruN 5.0.3-1/examples/Optimal_bounding_box/obb_with_point_maps_example.cpp 5.1-1/examples/Optimal_bounding_box/obb_with_point_maps_example.cpp
--- 5.0.3-1/examples/Optimal_bounding_box/obb_with_point_maps_example.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Optimal_bounding_box/obb_with_point_maps_example.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,58 @@
+#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
+#include <CGAL/Surface_mesh.h>
+
+#include <CGAL/optimal_bounding_box.h>
+
+#include <array>
+#include <fstream>
+#include <iostream>
+#include <map>
+#include <unordered_map>
+
+typedef CGAL::Exact_predicates_inexact_constructions_kernel    K;
+typedef K::Point_3                                             Point;
+typedef K::Vector_3                                            Vector;
+
+typedef CGAL::Surface_mesh<Point>                              Surface_mesh;
+typedef boost::graph_traits<Surface_mesh>::vertex_descriptor   vertex_descriptor;
+
+namespace CP = CGAL::parameters;
+
+int main(int argc, char** argv)
+{
+  std::ifstream input((argc > 1) ? argv[1] : "data/pig.off");
+  Surface_mesh sm;
+  if (!input || !(input >> sm) || sm.is_empty())
+  {
+    std::cerr << "Invalid input file." << std::endl;
+    return EXIT_FAILURE;
+  }
+
+  // a typical call
+  std::array<Point, 8> obb_points;
+  CGAL::oriented_bounding_box(sm, obb_points);
+
+  // one can associate positions to the vertices of the mesh without changing the mesh
+  std::unordered_map<vertex_descriptor, Point> translated_positions;
+  for(const vertex_descriptor v : vertices(sm))
+    translated_positions[v] = sm.point(v) + Vector(1, 2, 3);
+
+  CGAL::oriented_bounding_box(sm, obb_points,
+                              CP::vertex_point_map(boost::make_assoc_property_map(translated_positions)));
+
+  // using a range of points
+  std::vector<Point> points;
+  for(const vertex_descriptor v : vertices(sm))
+    points.push_back(sm.point(v));
+  CGAL::oriented_bounding_box(points, obb_points);
+
+  // one can associate positions to the range without changing the range
+  std::map<Point, Point> scaled_positions;
+  for(const Point& p : points)
+    scaled_positions[p] = p + (p - CGAL::ORIGIN);
+
+  CGAL::oriented_bounding_box(points, obb_points,
+                              CP::point_map(boost::make_assoc_property_map(scaled_positions)));
+
+  return EXIT_SUCCESS;
+}
diff -pruN 5.0.3-1/examples/Optimal_bounding_box/rotated_aabb_tree_example.cpp 5.1-1/examples/Optimal_bounding_box/rotated_aabb_tree_example.cpp
--- 5.0.3-1/examples/Optimal_bounding_box/rotated_aabb_tree_example.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Optimal_bounding_box/rotated_aabb_tree_example.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,58 @@
+#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
+#include <CGAL/Surface_mesh.h>
+
+#include <CGAL/AABB_tree.h>
+#include <CGAL/AABB_traits.h>
+#include <CGAL/AABB_face_graph_triangle_primitive.h>
+#include <CGAL/optimal_bounding_box.h>
+
+#include <boost/property_map/function_property_map.hpp>
+
+#include <fstream>
+#include <iostream>
+
+typedef CGAL::Exact_predicates_inexact_constructions_kernel    K;
+typedef K::Point_3                                             Point;
+typedef K::Aff_transformation_3                                Aff_transformation;
+
+typedef CGAL::Surface_mesh<Point>                              Surface_mesh;
+typedef boost::graph_traits<Surface_mesh>::vertex_descriptor   vertex_descriptor;
+
+struct Aff_tr_fct
+{
+  Aff_tr_fct() : m_at(nullptr), m_sm(nullptr) { }
+  Aff_tr_fct(const Aff_transformation& at, const Surface_mesh& sm) : m_at(&at), m_sm(&sm) { }
+
+  Point operator()(const vertex_descriptor v) const { return m_at->transform(m_sm->point(v)); }
+
+private:
+  const Aff_transformation* m_at;
+  const Surface_mesh* m_sm;
+};
+
+int main(int argc, char** argv)
+{
+  std::ifstream input((argc > 1) ? argv[1] : "data/pig.off");
+  Surface_mesh sm;
+  if (!input || !(input >> sm) || sm.is_empty())
+  {
+    std::cerr << "Invalid input file." << std::endl;
+    return EXIT_FAILURE;
+  }
+
+  // get the transformation that yields the optimal bounding box
+  Aff_transformation at;
+  CGAL::oriented_bounding_box(sm, at);
+
+  // functor to apply the affine transformation to a vertex of the mesh
+  Aff_tr_fct aff_tr_fct(at, sm);
+  auto aff_tr_vpm = boost::make_function_property_map<vertex_descriptor>(aff_tr_fct);
+
+  // rotated AABB tree
+  typedef CGAL::AABB_face_graph_triangle_primitive<Surface_mesh, decltype(aff_tr_vpm)> AABB_face_graph_primitive;
+  typedef CGAL::AABB_traits<K, AABB_face_graph_primitive>                              AABB_face_graph_traits;
+
+  CGAL::AABB_tree<AABB_face_graph_traits> tree(faces(sm).begin(), faces(sm).end(), sm, aff_tr_vpm);
+
+  return EXIT_SUCCESS;
+}
diff -pruN 5.0.3-1/examples/Periodic_2_triangulation_2/CMakeLists.txt 5.1-1/examples/Periodic_2_triangulation_2/CMakeLists.txt
--- 5.0.3-1/examples/Periodic_2_triangulation_2/CMakeLists.txt	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Periodic_2_triangulation_2/CMakeLists.txt	2020-09-08 12:12:20.000000000 +0000
@@ -6,7 +6,11 @@ cmake_minimum_required(VERSION 3.1...3.1
 project( Periodic_2_triangulation_2_Examples )
 
 
-find_package(CGAL QUIET)
+find_package(CGAL COMPONENTS Qt5)
+
+if(CGAL_Qt5_FOUND)
+  add_definitions(-DCGAL_USE_BASIC_VIEWER -DQT_NO_KEYWORDS)
+endif()
 
 if ( CGAL_FOUND )
 
@@ -16,6 +20,9 @@ if ( CGAL_FOUND )
     create_single_source_cgal_program( "${cppfile}" )
   endforeach()
 
+  if(CGAL_Qt5_FOUND)
+    target_link_libraries(draw_periodic_2_triangulation_2 PUBLIC CGAL::CGAL_Qt5)
+  endif()
 else()
 
     message(STATUS "This program requires the CGAL library, and will not be compiled.")
diff -pruN 5.0.3-1/examples/Periodic_2_triangulation_2/data/data1.dt.cin 5.1-1/examples/Periodic_2_triangulation_2/data/data1.dt.cin
--- 5.0.3-1/examples/Periodic_2_triangulation_2/data/data1.dt.cin	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Periodic_2_triangulation_2/data/data1.dt.cin	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,20 @@
+0.192592592592593	0.037037037037037
+0.17037037037037	0.177777777777778
+0.451851851851852	0.111111111111111
+0.274074074074074	0.37037037037037
+0.133333333333333	0.592592592592593
+0.037037037037037	0.688888888888889
+0.177777777777778	0.844444444444444
+0.288888888888889	0.674074074074074
+0.318518518518519	0.925925925925926
+0.540740740740741	0.844444444444444
+0.637037037037037	0.703703703703704
+0.511111111111111	0.496296296296296
+0.725925925925926	0.237037037037037
+0.933333333333333	0.22962962962963
+0.733333333333333	0.511111111111111
+0.925925925925926	0.459259259259259
+0.866666666666667	0.777777777777778
+0.77037037037037	0.925925925925926
+0.881481481481482	0.977777777777778
+0.990000000000000       0.911111111111111
diff -pruN 5.0.3-1/examples/Periodic_2_triangulation_2/draw_periodic_2_triangulation_2.cpp 5.1-1/examples/Periodic_2_triangulation_2/draw_periodic_2_triangulation_2.cpp
--- 5.0.3-1/examples/Periodic_2_triangulation_2/draw_periodic_2_triangulation_2.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Periodic_2_triangulation_2/draw_periodic_2_triangulation_2.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,36 @@
+#include <CGAL/Periodic_2_Delaunay_triangulation_2.h>
+#include <CGAL/Periodic_2_Delaunay_triangulation_traits_2.h>
+#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
+#include <CGAL/draw_periodic_2_triangulation_2.h>
+
+#include <fstream>
+
+typedef CGAL::Exact_predicates_inexact_constructions_kernel K;
+typedef CGAL::Periodic_2_Delaunay_triangulation_traits_2<K> GT;
+typedef CGAL::Periodic_2_Delaunay_triangulation_2<GT>       PDT;
+
+typedef PDT::Point                                          Point;
+
+int main(int argc, char* argv[])
+{
+  // Declare periodic triangulation 2D
+  PDT T;
+
+  // Read points and insert in T
+  Point p;
+  std::ifstream ifs((argc > 1) ? argv[1] : "data/data1.dt.cin");
+  if (ifs)
+  {
+    while (ifs >> p)
+    { T.insert(p); }
+    CGAL_assertion(T.is_valid());
+
+    if( T.is_triangulation_in_1_sheet())
+    { T.convert_to_9_sheeted_covering(); }
+
+    // Draw the periodic triangulation
+    CGAL::draw(T);
+  }
+
+  return EXIT_SUCCESS;
+}
diff -pruN 5.0.3-1/examples/Periodic_3_mesh_3/CMakeLists.txt 5.1-1/examples/Periodic_3_mesh_3/CMakeLists.txt
--- 5.0.3-1/examples/Periodic_3_mesh_3/CMakeLists.txt	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Periodic_3_mesh_3/CMakeLists.txt	2020-09-08 12:12:20.000000000 +0000
@@ -23,11 +23,10 @@ endif()
 
 # Use Eigen
 find_package(Eigen3 3.1.0 REQUIRED) #(3.1.0 or greater)
-if (NOT EIGEN3_FOUND)
+include(CGAL_Eigen_support)
+if (NOT TARGET CGAL::Eigen_support)
   message(STATUS "This project requires the Eigen library, and will not be compiled.")
   return()
-else()
-  include( ${EIGEN3_USE_FILE} )
 endif()
 
 # Boost and its components
@@ -46,3 +45,13 @@ create_single_source_cgal_program( "mesh
 create_single_source_cgal_program( "mesh_implicit_shape_with_subdomains.cpp" )
 create_single_source_cgal_program( "mesh_implicit_shape_with_optimizers.cpp" )
 create_single_source_cgal_program( "mesh_implicit_shape_with_features.cpp" )
+
+
+foreach(target
+    mesh_implicit_shape
+    mesh_implicit_multi_domain
+    mesh_implicit_shape_with_subdomains
+    mesh_implicit_shape_with_optimizers
+    mesh_implicit_shape_with_features)
+  target_link_libraries(${target} PUBLIC CGAL::Eigen_support)
+endforeach()
diff -pruN 5.0.3-1/examples/Point_set_3/CMakeLists.txt 5.1-1/examples/Point_set_3/CMakeLists.txt
--- 5.0.3-1/examples/Point_set_3/CMakeLists.txt	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Point_set_3/CMakeLists.txt	2020-09-08 12:12:20.000000000 +0000
@@ -44,14 +44,10 @@ set(needed_cxx_features cxx_rvalue_refer
 create_single_source_cgal_program( "point_set_read_ply.cpp" CXX_FEATURES ${needed_cxx_features} )
 
 find_package(Eigen3 3.1.0) #(requires 3.1.0 or greater)
-if (NOT EIGEN3_FOUND)
-  find_package(LAPACK)
-  if(LAPACK_FOUND)
-    include( ${LAPACK_USE_FILE} )
-  endif(LAPACK_FOUND)
-else()
-  include( ${EIGEN3_USE_FILE} )
+include(CGAL_Eigen_support)
+if (EIGEN3_FOUND)
   create_single_source_cgal_program( "point_set_algo.cpp" )
+  target_link_libraries(point_set_algo PUBLIC CGAL::Eigen_support)
 endif()
 
 create_single_source_cgal_program("draw_point_set_3.cpp" )
diff -pruN 5.0.3-1/examples/Point_set_processing_3/average_spacing_example.cpp 5.1-1/examples/Point_set_processing_3/average_spacing_example.cpp
--- 5.0.3-1/examples/Point_set_processing_3/average_spacing_example.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Point_set_processing_3/average_spacing_example.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -16,12 +16,7 @@ typedef Kernel::Point_3 Point;
 typedef boost::tuple<int, Point, int, int, int> IndexedPointWithColorTuple;
 
 // Concurrency
-#ifdef CGAL_LINKED_WITH_TBB
-typedef CGAL::Parallel_tag Concurrency_tag;
-#else
-typedef CGAL::Sequential_tag Concurrency_tag;
-#endif
-
+typedef CGAL::Parallel_if_available_tag Concurrency_tag;
 
 int main(int argc, char*argv[])
 {
diff -pruN 5.0.3-1/examples/Point_set_processing_3/bilateral_smooth_point_set_example.cpp 5.1-1/examples/Point_set_processing_3/bilateral_smooth_point_set_example.cpp
--- 5.0.3-1/examples/Point_set_processing_3/bilateral_smooth_point_set_example.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Point_set_processing_3/bilateral_smooth_point_set_example.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -17,12 +17,7 @@ typedef Kernel::Vector_3 Vector;
 typedef std::pair<Point, Vector> PointVectorPair;
 
 // Concurrency
-#ifdef CGAL_LINKED_WITH_TBB
-typedef CGAL::Parallel_tag Concurrency_tag;
-#else
-typedef CGAL::Sequential_tag Concurrency_tag;
-#endif
-
+typedef CGAL::Parallel_if_available_tag Concurrency_tag;
 
 int main(int argc, char*argv[])
 {
diff -pruN 5.0.3-1/examples/Point_set_processing_3/callback_example.cpp 5.1-1/examples/Point_set_processing_3/callback_example.cpp
--- 5.0.3-1/examples/Point_set_processing_3/callback_example.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Point_set_processing_3/callback_example.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -18,12 +18,7 @@ typedef Kernel::Point_3 Point;
 typedef CGAL::Random_points_on_sphere_3<Point> Generator;
 
 // Concurrency
-#ifdef CGAL_LINKED_WITH_TBB
-typedef CGAL::Parallel_tag Concurrency_tag;
-#else
-typedef CGAL::Sequential_tag Concurrency_tag;
-#endif
-
+typedef CGAL::Parallel_if_available_tag Concurrency_tag;
 
 // instance of std::function<bool(double)>
 struct Progress_to_std_cerr_callback
diff -pruN 5.0.3-1/examples/Point_set_processing_3/clustering_example.cpp 5.1-1/examples/Point_set_processing_3/clustering_example.cpp
--- 5.0.3-1/examples/Point_set_processing_3/clustering_example.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Point_set_processing_3/clustering_example.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,62 @@
+#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
+#include <CGAL/Point_set_3.h>
+#include <CGAL/Point_set_3/IO.h>
+#include <CGAL/cluster_point_set.h>
+#include <CGAL/compute_average_spacing.h>
+
+#include <CGAL/Random.h>
+#include <CGAL/Real_timer.h>
+
+#include <fstream>
+
+using Kernel = CGAL::Exact_predicates_inexact_constructions_kernel;
+using Point_3 = Kernel::Point_3;
+using Point_set = CGAL::Point_set_3<Point_3>;
+
+int main (int argc, char** argv)
+{
+  // Read input file
+  std::ifstream ifile ((argc > 1) ? argv[1] : "data/hippo1.ply", std::ios_base::binary);
+  Point_set points;
+  ifile >> points;
+
+  // Add a cluster map
+  Point_set::Property_map<int> cluster_map = points.add_property_map<int> ("cluster", -1).first;
+
+  // Compute average spacing
+  double spacing = CGAL::compute_average_spacing<CGAL::Parallel_if_available_tag> (points, 12);
+  std::cerr << "Spacing = " << spacing << std::endl;
+
+  // Adjacencies stored in vector
+  std::vector<std::pair<std::size_t, std::size_t> > adjacencies;
+
+  // Compute clusters
+  CGAL::Real_timer t;
+  t.start();
+  std::size_t nb_clusters
+    = CGAL::cluster_point_set (points, cluster_map,
+                               points.parameters().neighbor_radius(spacing).
+                               adjacencies (std::back_inserter (adjacencies)));
+  t.stop();
+  std::cerr << "Found " << nb_clusters << " clusters with " << adjacencies.size()
+            << " adjacencies in " << t.time() << " seconds" << std::endl;
+
+  // Output a colored PLY file
+  Point_set::Property_map<unsigned char> red = points.add_property_map<unsigned char>("red", 0).first;
+  Point_set::Property_map<unsigned char> green = points.add_property_map<unsigned char>("green", 0).first;
+  Point_set::Property_map<unsigned char> blue = points.add_property_map<unsigned char>("blue", 0).first;
+  for (Point_set::Index idx : points)
+  {
+    // One color per cluster
+    CGAL::Random rand (cluster_map[idx]);
+    red[idx] = rand.get_int(64, 192);
+    green[idx] = rand.get_int(64, 192);
+    blue[idx] = rand.get_int(64, 192);
+  }
+
+  std::ofstream ofile ("out.ply", std::ios_base::binary);
+  CGAL::set_binary_mode (ofile);
+  ofile << points;
+
+  return EXIT_SUCCESS;
+}
diff -pruN 5.0.3-1/examples/Point_set_processing_3/CMakeLists.txt 5.1-1/examples/Point_set_processing_3/CMakeLists.txt
--- 5.0.3-1/examples/Point_set_processing_3/CMakeLists.txt	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Point_set_processing_3/CMakeLists.txt	2020-09-08 12:12:20.000000000 +0000
@@ -3,8 +3,6 @@
 cmake_minimum_required(VERSION 3.1...3.15)
 project( Point_set_processing_3_Examples )
 
-
-
 # Find CGAL
 find_package(CGAL QUIET)
 
@@ -33,70 +31,100 @@ if ( CGAL_FOUND )
          "Enable concurrency"
          OFF)
 
+  set(CGAL_libs CGAL::CGAL)
   if( CGAL_ACTIVATE_CONCURRENT_PSP3 OR ENV{CGAL_ACTIVATE_CONCURRENT_PSP3} )
     find_package( TBB REQUIRED )
+    include(CGAL_TBB_support)
+    if (TARGET CGAL::TBB_support)
+      set(CGAL_TBB_target ${CGAL_libs} CGAL::TBB_support)
+    endif()
   endif()
 
   # Executables that do *not* require EIGEN
-  create_single_source_cgal_program( "average_spacing_example.cpp" )
-  create_single_source_cgal_program( "bilateral_smooth_point_set_example.cpp" )
-  create_single_source_cgal_program( "grid_simplification_example.cpp" )
-  create_single_source_cgal_program( "grid_simplify_indices.cpp" )
-  create_single_source_cgal_program( "hierarchy_simplification_example.cpp" )
-  create_single_source_cgal_program( "normals_example.cpp" )
-  create_single_source_cgal_program( "property_map.cpp" )
-  create_single_source_cgal_program( "random_simplification_example.cpp" )
-  create_single_source_cgal_program( "read_write_xyz_point_set_example.cpp" )
-  create_single_source_cgal_program( "remove_outliers_example.cpp" )
-  create_single_source_cgal_program( "scale_estimation_example.cpp" )
-  create_single_source_cgal_program( "scale_estimation_2d_example.cpp" )
-  create_single_source_cgal_program( "wlop_simplify_and_regularize_point_set_example.cpp" )
-  create_single_source_cgal_program( "edge_aware_upsample_point_set_example.cpp" )
-  create_single_source_cgal_program( "structuring_example.cpp" )
-  create_single_source_cgal_program( "callback_example.cpp" )
-
-  set(needed_cxx_features cxx_rvalue_references cxx_variadic_templates)
-  create_single_source_cgal_program( "read_ply_points_with_colors_example.cpp" CXX_FEATURES ${needed_cxx_features} )
-  create_single_source_cgal_program( "write_ply_points_example.cpp" CXX_FEATURES ${needed_cxx_features} )
+  foreach(target
+      average_spacing_example
+      bilateral_smooth_point_set_example
+      grid_simplification_example
+      grid_simplify_indices
+      property_map
+      random_simplification_example
+      read_write_xyz_point_set_example
+      remove_outliers_example
+      wlop_simplify_and_regularize_point_set_example
+      edge_aware_upsample_point_set_example
+      structuring_example
+      read_ply_points_with_colors_example
+      write_ply_points_example)
+    add_executable(${target} "${target}.cpp")
+    target_link_libraries(${target} ${CGAL_libs})
+  endforeach()
 
   find_package(LASLIB)
-  if (LASLIB_FOUND)
-    include(${LASLIB_USE_FILE})
-    include_directories(${LASLIB_INCLUDE_DIR})
-    include_directories(${LASZIP_INCLUDE_DIR})
-    create_single_source_cgal_program( "read_las_example.cpp" CXX_FEATURES ${needed_cxx_features} )
-    target_link_libraries(read_las_example PRIVATE ${LASLIB_LIBRARIES})
+  include(CGAL_LASLIB_support)
+  if (TARGET CGAL::LASLIB_support)
+    add_executable( read_las_example "read_las_example.cpp" )
+    target_link_libraries(read_las_example ${CGAL_libs} CGAL::LASLIB_support)
   else()
     message(STATUS "NOTICE : the LAS reader test requires LASlib and will not be compiled.")
   endif()
 
   # Use Eigen
   find_package(Eigen3 3.1.0) #(requires 3.1.0 or greater)
-  if (EIGEN3_FOUND)
-    include( ${EIGEN3_USE_FILE} )
+  include(CGAL_Eigen_support)
+  if (TARGET CGAL::Eigen_support)
+    set(CGAL_libs ${CGAL_libs} CGAL::Eigen_support)
+
     # Executables that require Eigen
-    create_single_source_cgal_program( "jet_smoothing_example.cpp" )
-    create_single_source_cgal_program( "normal_estimation.cpp" )
-    create_single_source_cgal_program( "edges_example.cpp" )
-  else()
+    foreach(target
+        jet_smoothing_example
+        normal_estimation
+        clustering_example
+        edges_example
+        callback_example
+        scale_estimation_example
+        scale_estimation_2d_example
+        hierarchy_simplification_example
+        normals_example)
+      add_executable(${target} "${target}.cpp")
+      target_link_libraries(${target} ${CGAL_libs})
+    endforeach()
+
+    # Executables that require libpointmatcher
+    find_package(libpointmatcher QUIET)
+    include(CGAL_pointmatcher_support)
+    if (TARGET CGAL::pointmatcher_support)
+      add_executable(registration_with_pointmatcher "registration_with_pointmatcher.cpp")
+      target_link_libraries(registration_with_pointmatcher
+        ${CGAL_libs} CGAL::pointmatcher_support)
+    else()
+      message(STATUS "NOTICE : the registration_with_pointmatcher test requires libpointmatcher and will not be compiled.")
+    endif()
 
-    message(STATUS "NOTICE: Some of the executables in this directory need Eigen 3.1 (or greater) and will not be compiled.")
-  endif()
+    # Executables that require OpenGR
+    find_package(OpenGR QUIET)
+    include(CGAL_OpenGR_support)
+    if (TARGET CGAL::OpenGR_support)
+      add_executable(registration_with_OpenGR "registration_with_OpenGR.cpp" )
+      target_link_libraries(registration_with_OpenGR
+        ${CGAL_libs} CGAL::OpenGR_support)
+    else()
+      message(STATUS "NOTICE : registration_with_OpenGR requires OpenGR, and will not be compiled.")
+    endif()
 
-  foreach(target
-      scale_estimation_example
-      wlop_simplify_and_regularize_point_set_example
-      bilateral_smooth_point_set_example
-      edge_aware_upsample_point_set_example
-      average_spacing_example
-      normals_example
-      jet_smoothing_example
-      normal_estimation
-      callback_example)
-    if(TBB_FOUND AND TARGET ${target})
-      CGAL_target_use_TBB(${target})
+    # Executables that require both libpointmatcher and OpenGR
+    if (TARGET CGAL::pointmatcher_support AND
+        TARGET CGAL::OpenGR_support)
+      add_executable(registration_with_opengr_pointmatcher_pipeline
+        "registration_with_opengr_pointmatcher_pipeline.cpp" )
+      target_link_libraries(registration_with_opengr_pointmatcher_pipeline
+        ${CGAL_libs} CGAL::pointmatcher_support CGAL::OpenGR_support)
+    else()
+      message(STATUS "NOTICE : registration_with_opengr_pointmatcher_pipeline requires libpointmatcher and OpenGR, and will not be compiled.")
     endif()
-  endforeach()
+
+  else()
+    message(STATUS "NOTICE: Some of the executables in this directory need Eigen 3.1 (or greater) and will not be compiled.")
+  endif()
 
 else()
     message(STATUS "NOTICE: This program requires the CGAL library, and will not be compiled.")
Binary files 5.0.3-1/examples/Point_set_processing_3/data/hippo1.ply and 5.1-1/examples/Point_set_processing_3/data/hippo1.ply differ
Binary files 5.0.3-1/examples/Point_set_processing_3/data/hippo2.ply and 5.1-1/examples/Point_set_processing_3/data/hippo2.ply differ
diff -pruN 5.0.3-1/examples/Point_set_processing_3/edge_aware_upsample_point_set_example.cpp 5.1-1/examples/Point_set_processing_3/edge_aware_upsample_point_set_example.cpp
--- 5.0.3-1/examples/Point_set_processing_3/edge_aware_upsample_point_set_example.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Point_set_processing_3/edge_aware_upsample_point_set_example.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -16,12 +16,7 @@ typedef Kernel::Vector_3 Vector;
 typedef std::pair<Point, Vector> PointVectorPair;
 
 // Concurrency
-#ifdef CGAL_LINKED_WITH_TBB
-typedef CGAL::Parallel_tag Concurrency_tag;
-#else
-typedef CGAL::Sequential_tag Concurrency_tag;
-#endif
-
+typedef CGAL::Parallel_if_available_tag Concurrency_tag;
 
 int main(int argc, char* argv[])
 {
diff -pruN 5.0.3-1/examples/Point_set_processing_3/jet_smoothing_example.cpp 5.1-1/examples/Point_set_processing_3/jet_smoothing_example.cpp
--- 5.0.3-1/examples/Point_set_processing_3/jet_smoothing_example.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Point_set_processing_3/jet_smoothing_example.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -8,12 +8,7 @@ typedef CGAL::Exact_predicates_inexact_c
 typedef Kernel::Point_3 Point;
 
 // Concurrency
-#ifdef CGAL_LINKED_WITH_TBB
-typedef CGAL::Parallel_tag Concurrency_tag;
-#else
-typedef CGAL::Sequential_tag Concurrency_tag;
-#endif
-
+typedef CGAL::Parallel_if_available_tag Concurrency_tag;
 
 int main(void)
 {
diff -pruN 5.0.3-1/examples/Point_set_processing_3/normal_estimation.cpp 5.1-1/examples/Point_set_processing_3/normal_estimation.cpp
--- 5.0.3-1/examples/Point_set_processing_3/normal_estimation.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Point_set_processing_3/normal_estimation.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -48,12 +48,7 @@ typedef std::pair<Point, Vector> PointVe
 typedef std::vector<PointVectorPair> PointList;
 
 // Concurrency
-#ifdef CGAL_LINKED_WITH_TBB
-typedef CGAL::Parallel_tag Concurrency_tag;
-#else
-typedef CGAL::Sequential_tag Concurrency_tag;
-#endif
-
+typedef CGAL::Parallel_if_available_tag Concurrency_tag;
 
 // ----------------------------------------------------------------------------
 // Private functions
diff -pruN 5.0.3-1/examples/Point_set_processing_3/normals_example.cpp 5.1-1/examples/Point_set_processing_3/normals_example.cpp
--- 5.0.3-1/examples/Point_set_processing_3/normals_example.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Point_set_processing_3/normals_example.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -18,11 +18,7 @@ typedef Kernel::Vector_3 Vector;
 typedef std::pair<Point, Vector> PointVectorPair;
 
 // Concurrency
-#ifdef CGAL_LINKED_WITH_TBB
-typedef CGAL::Parallel_tag Concurrency_tag;
-#else
-typedef CGAL::Sequential_tag Concurrency_tag;
-#endif
+typedef CGAL::Parallel_if_available_tag Concurrency_tag;
 
 int main(int argc, char*argv[])
 {
diff -pruN 5.0.3-1/examples/Point_set_processing_3/registration_with_OpenGR.cpp 5.1-1/examples/Point_set_processing_3/registration_with_OpenGR.cpp
--- 5.0.3-1/examples/Point_set_processing_3/registration_with_OpenGR.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Point_set_processing_3/registration_with_OpenGR.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,87 @@
+#include <CGAL/Simple_cartesian.h>
+#include <CGAL/IO/read_ply_points.h>
+#include <CGAL/IO/write_ply_points.h>
+#include <CGAL/property_map.h>
+
+#include <CGAL/OpenGR/compute_registration_transformation.h>
+#include <CGAL/OpenGR/register_point_sets.h>
+
+#include <fstream>
+#include <iostream>
+#include <utility>
+
+typedef CGAL::Simple_cartesian<double> K;
+typedef K::Point_3 Point_3;
+typedef K::Vector_3 Vector_3;
+typedef std::pair<Point_3, Vector_3> Pwn;
+typedef CGAL::First_of_pair_property_map<Pwn> Point_map;
+typedef CGAL::Second_of_pair_property_map<Pwn> Normal_map;
+
+namespace params = CGAL::parameters;
+
+int main(int argc, const char** argv)
+{
+  const char* fname1 = (argc>1)?argv[1]:"data/hippo1.ply";
+  const char* fname2 = (argc>2)?argv[2]:"data/hippo2.ply";
+
+  std::vector<Pwn> pwns1, pwns2;
+  std::ifstream input(fname1);
+  if (!input ||
+      !CGAL::read_ply_points(input, std::back_inserter(pwns1),
+            CGAL::parameters::point_map (CGAL::First_of_pair_property_map<Pwn>()).
+            normal_map (Normal_map())))
+  {
+    std::cerr << "Error: cannot read file " << fname1 << std::endl;
+    return EXIT_FAILURE;
+  }
+  input.close();
+
+  input.open(fname2);
+  if (!input ||
+      !CGAL::read_ply_points(input, std::back_inserter(pwns2),
+            CGAL::parameters::point_map (Point_map()).
+            normal_map (Normal_map())))
+  {
+    std::cerr << "Error: cannot read file " << fname2 << std::endl;
+    return EXIT_FAILURE;
+  }
+  input.close();
+
+  // EITHER call the registration method Super4PCS from OpenGR to get the transformation to apply to pwns2
+  // std::pair<K::Aff_transformation_3, double> res =
+    CGAL::OpenGR::compute_registration_transformation(pwns1, pwns2,
+                                                      params::point_map(Point_map())
+                                                      .normal_map(Normal_map())
+                                                      .number_of_samples(200)
+                                                      .maximum_running_time(60)
+                                                      .accuracy(0.01),
+                                                      params::point_map(Point_map())
+                                                      .normal_map(Normal_map()));
+
+  // OR call the registration method Super4PCS from OpenGR and apply the transformation to pwn2
+  double score =
+    CGAL::OpenGR::register_point_sets(pwns1, pwns2,
+                                      params::point_map(Point_map())
+                                      .normal_map(Normal_map())
+                                      .number_of_samples(200)
+                                      .maximum_running_time(60)
+                                      .accuracy(0.01),
+                                      params::point_map(Point_map())
+                                      .normal_map(Normal_map()));
+
+  std::ofstream out("pwns2_aligned.ply");
+  if (!out ||
+      !CGAL::write_ply_points(
+        out, pwns2,
+        CGAL::parameters::point_map(Point_map()).
+        normal_map(Normal_map())))
+  {
+    return EXIT_FAILURE;
+  }
+
+  std::cout << "Registration score: " << score << ".\n"
+            << "Transformed version of " << fname2
+            << " written to pwn2_aligned.ply.\n";
+
+  return EXIT_SUCCESS;
+}
diff -pruN 5.0.3-1/examples/Point_set_processing_3/registration_with_opengr_pointmatcher_pipeline.cpp 5.1-1/examples/Point_set_processing_3/registration_with_opengr_pointmatcher_pipeline.cpp
--- 5.0.3-1/examples/Point_set_processing_3/registration_with_opengr_pointmatcher_pipeline.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Point_set_processing_3/registration_with_opengr_pointmatcher_pipeline.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,88 @@
+#include <CGAL/Simple_cartesian.h>
+#include <CGAL/IO/read_ply_points.h>
+#include <CGAL/IO/write_ply_points.h>
+#include <CGAL/property_map.h>
+#include <CGAL/Aff_transformation_3.h>
+
+#include <CGAL/pointmatcher/register_point_sets.h>
+
+#include <CGAL/OpenGR/compute_registration_transformation.h>
+
+#include <fstream>
+#include <iostream>
+#include <utility>
+#include <vector>
+
+typedef CGAL::Simple_cartesian<double> K;
+typedef K::Point_3 Point_3;
+typedef K::Vector_3 Vector_3;
+typedef std::pair<Point_3, Vector_3> Pwn;
+typedef CGAL::First_of_pair_property_map<Pwn> Point_map;
+typedef CGAL::Second_of_pair_property_map<Pwn> Normal_map;
+
+namespace params = CGAL::parameters;
+
+int main(int argc, const char** argv)
+{
+  const char* fname1 = (argc>1)?argv[1]:"data/hippo1.ply";
+  const char* fname2 = (argc>2)?argv[2]:"data/hippo2.ply";
+
+  std::vector<Pwn> pwns1, pwns2;
+  std::ifstream input(fname1);
+  if (!input ||
+      !CGAL::read_ply_points(input, std::back_inserter(pwns1),
+            CGAL::parameters::point_map (CGAL::First_of_pair_property_map<Pwn>()).
+            normal_map (Normal_map())))
+  {
+    std::cerr << "Error: cannot read file " << fname1 << std::endl;
+    return EXIT_FAILURE;
+  }
+  input.close();
+
+  input.open(fname2);
+  if (!input ||
+      !CGAL::read_ply_points(input, std::back_inserter(pwns2),
+            CGAL::parameters::point_map (Point_map()).
+            normal_map (Normal_map())))
+  {
+    std::cerr << "Error: cannot read file " << fname2 << std::endl;
+    return EXIT_FAILURE;
+  }
+  input.close();
+
+  std::cerr << "Computing registration transformation using OpenGR Super4PCS.." << std::endl;
+  // First, compute registration transformation using OpenGR Super4PCS
+  K::Aff_transformation_3 res =
+    std::get<0>( // get first of pair, which is the transformation
+      CGAL::OpenGR::compute_registration_transformation
+        (pwns1, pwns2,
+         params::point_map(Point_map()).normal_map(Normal_map()),
+         params::point_map(Point_map()).normal_map(Normal_map()))
+    );
+
+  std::cerr << "Computing registration transformation using PointMatcher ICP, "
+            << "taking transformation computed by OpenGR Super4PCS as initial transformation.." << std::endl;
+  // Then, compute registration transformation using PointMatcher ICP, taking transformation computed
+  // by OpenGR as initial transformation, and apply the transformation to pwns2
+  // bool converged =
+    CGAL::pointmatcher::register_point_sets
+      (pwns1, pwns2,
+       params::point_map(Point_map()).normal_map(Normal_map()),
+       params::point_map(Point_map()).normal_map(Normal_map())
+       .transformation(res));
+
+  std::ofstream out("pwns2_aligned.ply");
+  if (!out ||
+      !CGAL::write_ply_points(
+        out, pwns2,
+        CGAL::parameters::point_map(Point_map()).
+        normal_map(Normal_map())))
+  {
+    return EXIT_FAILURE;
+  }
+
+  std::cerr << "Transformed version of " << fname2
+            << " written to pwn2_aligned.ply.\n";
+
+  return EXIT_SUCCESS;
+}
diff -pruN 5.0.3-1/examples/Point_set_processing_3/registration_with_pointmatcher.cpp 5.1-1/examples/Point_set_processing_3/registration_with_pointmatcher.cpp
--- 5.0.3-1/examples/Point_set_processing_3/registration_with_pointmatcher.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Point_set_processing_3/registration_with_pointmatcher.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,158 @@
+#include <CGAL/Simple_cartesian.h>
+#include <CGAL/IO/read_ply_points.h>
+#include <CGAL/IO/write_ply_points.h>
+#include <CGAL/property_map.h>
+#include <CGAL/Aff_transformation_3.h>
+#include <CGAL/aff_transformation_tags.h>
+
+#include <CGAL/pointmatcher/compute_registration_transformation.h>
+#include <CGAL/pointmatcher/register_point_sets.h>
+
+#include <fstream>
+#include <iostream>
+#include <utility>
+#include <vector>
+
+typedef CGAL::Simple_cartesian<double> K;
+typedef K::Point_3 Point_3;
+typedef K::Vector_3 Vector_3;
+typedef std::pair<Point_3, Vector_3> Pwn;
+typedef CGAL::First_of_pair_property_map<Pwn> Point_map;
+typedef CGAL::Second_of_pair_property_map<Pwn> Normal_map;
+
+namespace params = CGAL::parameters;
+
+int main(int argc, const char** argv)
+{
+  const char* fname1 = (argc>1)?argv[1]:"data/hippo1.ply";
+  const char* fname2 = (argc>2)?argv[2]:"data/hippo2.ply";
+
+  std::vector<Pwn> pwns1, pwns2;
+  std::ifstream input(fname1);
+  if (!input ||
+      !CGAL::read_ply_points(input, std::back_inserter(pwns1),
+            CGAL::parameters::point_map (CGAL::First_of_pair_property_map<Pwn>()).
+            normal_map (Normal_map())))
+  {
+    std::cerr << "Error: cannot read file " << fname1 << std::endl;
+    return EXIT_FAILURE;
+  }
+  input.close();
+
+  input.open(fname2);
+  if (!input ||
+      !CGAL::read_ply_points(input, std::back_inserter(pwns2),
+            CGAL::parameters::point_map (Point_map()).
+            normal_map (Normal_map())))
+  {
+    std::cerr << "Error: cannot read file " << fname2 << std::endl;
+    return EXIT_FAILURE;
+  }
+  input.close();
+
+  //
+  // Prepare ICP config
+  //
+  using CGAL::pointmatcher::ICP_config;
+
+  // Possible config modules/components: https://libpointmatcher.readthedocs.io/en/latest/Configuration/#configuration-of-an-icp-chain
+  // See documentation of optional named parameters for CGAL PM ICP configuration / pointmatcher config module mapping
+
+  // Prepare point set 1 filters (PM::ReferenceDataPointsFilters)
+  std::vector<ICP_config> point_set_1_filters;
+  point_set_1_filters.push_back( ICP_config { /*.name=*/"MinDistDataPointsFilter"       , /*.params=*/{ {"minDist", "0.5" }}  } );
+  point_set_1_filters.push_back( ICP_config { /*.name=*/"RandomSamplingDataPointsFilter", /*.params=*/{ {"prob"   , "0.05"}}  } );
+
+  // Prepare point set 2 filters (PM::ReadingDataPointsFilters)
+  std::vector<ICP_config> point_set_2_filters;
+  point_set_2_filters.push_back( ICP_config { /*.name=*/"MinDistDataPointsFilter"       , /*.params=*/{ {"minDist", "0.5" }}  } );
+  point_set_2_filters.push_back( ICP_config { /*.name=*/"RandomSamplingDataPointsFilter", /*.params=*/{ {"prob"   , "0.05"}}  } );
+
+        // Prepare matcher function
+  ICP_config matcher { /*.name=*/"KDTreeMatcher", /*.params=*/{ {"knn", "1"}, {"epsilon", "3.16"} } };
+
+  // Prepare outlier filters
+  std::vector<ICP_config> outlier_filters;
+  outlier_filters.push_back( ICP_config { /*.name=*/"TrimmedDistOutlierFilter", /*.params=*/{ {"ratio", "0.75" }}  } );
+
+  // Prepare error minimizer
+  ICP_config error_minimizer { /*.name=*/"PointToPointErrorMinimizer"};
+
+  // Prepare transformation checker
+  std::vector<ICP_config> transformation_checkers;
+  transformation_checkers.push_back( ICP_config { /*.name=*/"CounterTransformationChecker", /*.params=*/{ {"maxIterationCount", "150" }}  } );
+  transformation_checkers.push_back( ICP_config { /*.name=*/"DifferentialTransformationChecker", /*.params=*/{ {"minDiffRotErr"  , "0.001" },
+                                                                                                       {"minDiffTransErr", "0.01"  },
+                                                                                                       {"smoothLength"   , "4"     } }
+                                                } );
+  // Prepare inspector
+  ICP_config inspector { /*.name=*/"NullInspector" };
+
+  // Prepare logger
+  ICP_config logger { /*.name=*/"FileLogger" };
+
+  const K::Aff_transformation_3 identity_transform = K::Aff_transformation_3(CGAL::Identity_transformation());
+
+  // EITHER call the ICP registration method pointmatcher to get the transformation to apply to pwns2
+  std::pair<K::Aff_transformation_3, bool> res =
+  CGAL::pointmatcher::compute_registration_transformation
+    (pwns1, pwns2,
+     params::point_map(Point_map()).normal_map(Normal_map())
+     .point_set_filters(point_set_1_filters)
+     .matcher(matcher)
+     .outlier_filters(outlier_filters)
+     .error_minimizer(error_minimizer)
+     .transformation_checkers(transformation_checkers)
+     .inspector(inspector)
+     .logger(logger),
+     params::point_map(Point_map()).normal_map(Normal_map())
+     .point_set_filters(point_set_2_filters)
+     .transformation(identity_transform) /* initial transform for pwns2.
+                                          * default value is already identity transform.
+                                          * a proper initial transform could be given, for example,
+                                          * a transform returned from a coarse registration algorithm.
+                                          * */
+     );
+
+  // OR call the ICP registration method from pointmatcher and apply the transformation to pwn2
+  bool converged =
+  CGAL::pointmatcher::register_point_sets
+    (pwns1, pwns2,
+     params::point_map(Point_map()).normal_map(Normal_map())
+     .point_set_filters(point_set_1_filters)
+     .matcher(matcher)
+     .outlier_filters(outlier_filters)
+     .error_minimizer(error_minimizer)
+     .transformation_checkers(transformation_checkers)
+     .inspector(inspector)
+     .logger(logger),
+     params::point_map(Point_map()).normal_map(Normal_map())
+     .point_set_filters(point_set_2_filters)
+     .transformation(res.first) /* pass the above computed transformation as initial transformation.
+                                * as a result, the registration will require less iterations to converge.
+                                * */
+     );
+
+  if (converged)
+    std::cerr << "Success" << std::endl;
+  else
+  {
+    std::cerr << "Failure" << std::endl;
+    return EXIT_FAILURE;
+  }
+
+  std::ofstream out("pwns2_aligned.ply");
+  if (!out ||
+      !CGAL::write_ply_points(
+        out, pwns2,
+        CGAL::parameters::point_map(Point_map()).
+        normal_map(Normal_map())))
+  {
+    return EXIT_FAILURE;
+  }
+
+  std::cout << "Transformed version of " << fname2
+            << " written to pwn2_aligned.ply.\n";
+
+  return EXIT_SUCCESS;
+}
diff -pruN 5.0.3-1/examples/Point_set_processing_3/remove_outliers_example.cpp 5.1-1/examples/Point_set_processing_3/remove_outliers_example.cpp
--- 5.0.3-1/examples/Point_set_processing_3/remove_outliers_example.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Point_set_processing_3/remove_outliers_example.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -39,7 +39,7 @@ int main(int argc, char*argv[])
   // FIRST OPTION //
   // I don't know the ratio of outliers present in the point set
   std::vector<Point>::iterator first_to_remove
-    = CGAL::remove_outliers
+    = CGAL::remove_outliers<CGAL::Parallel_if_available_tag>
     (points,
      nb_neighbors,
      CGAL::parameters::threshold_percent (100.). // No limit on the number of outliers to remove
@@ -55,7 +55,7 @@ int main(int argc, char*argv[])
   // I know the ratio of outliers present in the point set
   const double removed_percentage = 5.0; // percentage of points to remove
 
-  points.erase(CGAL::remove_outliers
+  points.erase(CGAL::remove_outliers<CGAL::Parallel_if_available_tag>
                (points,
                 nb_neighbors,
                 CGAL::parameters::threshold_percent(removed_percentage). // Minimum percentage to remove
diff -pruN 5.0.3-1/examples/Point_set_processing_3/scale_estimation_example.cpp 5.1-1/examples/Point_set_processing_3/scale_estimation_example.cpp
--- 5.0.3-1/examples/Point_set_processing_3/scale_estimation_example.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Point_set_processing_3/scale_estimation_example.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -12,11 +12,7 @@
 #include <fstream>
 
 // Concurrency
-#ifdef CGAL_LINKED_WITH_TBB
-typedef CGAL::Parallel_tag Concurrency_tag;
-#else
-typedef CGAL::Sequential_tag Concurrency_tag;
-#endif
+typedef CGAL::Parallel_if_available_tag Concurrency_tag;
 
 // Types
 typedef CGAL::Exact_predicates_inexact_constructions_kernel Kernel;
diff -pruN 5.0.3-1/examples/Point_set_processing_3/wlop_simplify_and_regularize_point_set_example.cpp 5.1-1/examples/Point_set_processing_3/wlop_simplify_and_regularize_point_set_example.cpp
--- 5.0.3-1/examples/Point_set_processing_3/wlop_simplify_and_regularize_point_set_example.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Point_set_processing_3/wlop_simplify_and_regularize_point_set_example.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -12,11 +12,7 @@ typedef CGAL::Simple_cartesian<double> K
 typedef Kernel::Point_3 Point;
 
 // Concurrency
-#ifdef CGAL_LINKED_WITH_TBB
-typedef CGAL::Parallel_tag Concurrency_tag;
-#else
-typedef CGAL::Sequential_tag Concurrency_tag;
-#endif
+typedef CGAL::Parallel_if_available_tag Concurrency_tag;
 
 int main(int argc, char** argv)
 {
diff -pruN 5.0.3-1/examples/Poisson_surface_reconstruction_3/CMakeLists.txt 5.1-1/examples/Poisson_surface_reconstruction_3/CMakeLists.txt
--- 5.0.3-1/examples/Poisson_surface_reconstruction_3/CMakeLists.txt	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Poisson_surface_reconstruction_3/CMakeLists.txt	2020-09-08 12:12:20.000000000 +0000
@@ -23,15 +23,17 @@ if ( CGAL_FOUND )
 
   # Find Eigen3 (requires 3.1.0 or greater)
   find_package(Eigen3 3.1.0)
-  if (EIGEN3_FOUND)
-    include( ${EIGEN3_USE_FILE} )
-  endif()
-
-  if (EIGEN3_FOUND)
+  include(CGAL_Eigen_support)
+  if (TARGET CGAL::Eigen_support)
     # Executables that require Eigen 3
     create_single_source_cgal_program( "poisson_reconstruction_example.cpp" )
+    target_link_libraries(poisson_reconstruction_example PUBLIC CGAL::Eigen_support)
     create_single_source_cgal_program( "poisson_reconstruction.cpp" )
+    target_link_libraries(poisson_reconstruction PUBLIC CGAL::Eigen_support)
     create_single_source_cgal_program( "poisson_reconstruction_function.cpp" )
+    target_link_libraries(poisson_reconstruction_function PUBLIC CGAL::Eigen_support)
+    create_single_source_cgal_program( "tutorial_example.cpp" )
+    target_link_libraries(tutorial_example PUBLIC CGAL::Eigen_support)
   else()
     message(STATUS "NOTICE: The examples need Eigen 3.1 (or greater) will not be compiled.")
   endif()
@@ -42,4 +44,3 @@ else()
     message(STATUS "NOTICE: This program requires the CGAL library, and will not be compiled.")
 
 endif()
-
diff -pruN 5.0.3-1/examples/Poisson_surface_reconstruction_3/poisson_reconstruction.cpp 5.1-1/examples/Poisson_surface_reconstruction_3/poisson_reconstruction.cpp
--- 5.0.3-1/examples/Poisson_surface_reconstruction_3/poisson_reconstruction.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Poisson_surface_reconstruction_3/poisson_reconstruction.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -360,7 +360,6 @@ int main(int argc, char * argv[])
     // Constructs AABB tree and computes internal KD-tree
     // data structure to accelerate distance queries
     AABB_tree tree(faces(output_mesh).first, faces(output_mesh).second, output_mesh);
-    tree.accelerate_distance_queries();
 
     // Computes distance from each input point to reconstructed mesh
     double max_distance = DBL_MIN;
diff -pruN 5.0.3-1/examples/Poisson_surface_reconstruction_3/tutorial_example.cmd 5.1-1/examples/Poisson_surface_reconstruction_3/tutorial_example.cmd
--- 5.0.3-1/examples/Poisson_surface_reconstruction_3/tutorial_example.cmd	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Poisson_surface_reconstruction_3/tutorial_example.cmd	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,3 @@
+data/kitten.xyz
+data/kitten.xyz 1
+data/kitten.xyz 2
diff -pruN 5.0.3-1/examples/Poisson_surface_reconstruction_3/tutorial_example.cpp 5.1-1/examples/Poisson_surface_reconstruction_3/tutorial_example.cpp
--- 5.0.3-1/examples/Poisson_surface_reconstruction_3/tutorial_example.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Poisson_surface_reconstruction_3/tutorial_example.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,224 @@
+#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
+
+#include <CGAL/Point_set_3.h>
+#include <CGAL/Point_set_3/IO.h>
+
+#include <CGAL/remove_outliers.h>
+#include <CGAL/grid_simplify_point_set.h>
+#include <CGAL/jet_smooth_point_set.h>
+#include <CGAL/jet_estimate_normals.h>
+#include <CGAL/mst_orient_normals.h>
+
+#include <CGAL/poisson_surface_reconstruction.h>
+#include <CGAL/Advancing_front_surface_reconstruction.h>
+#include <CGAL/Scale_space_surface_reconstruction_3.h>
+#include <CGAL/Scale_space_reconstruction_3/Jet_smoother.h>
+#include <CGAL/Scale_space_reconstruction_3/Advancing_front_mesher.h>
+
+#include <CGAL/Surface_mesh.h>
+#include <CGAL/Polygon_mesh_processing/polygon_soup_to_polygon_mesh.h>
+
+#include <cstdlib>
+#include <vector>
+#include <fstream>
+
+// types
+typedef CGAL::Exact_predicates_inexact_constructions_kernel Kernel;
+typedef Kernel::FT FT;
+typedef Kernel::Point_3 Point_3;
+typedef Kernel::Vector_3 Vector_3;
+typedef Kernel::Sphere_3 Sphere_3;
+typedef CGAL::Point_set_3<Point_3, Vector_3> Point_set;
+
+int main(int argc, char*argv[])
+{
+  ///////////////////////////////////////////////////////////////////
+  //! [Reading input]
+
+  Point_set points;
+
+  if (argc < 2)
+  {
+    std::cerr << "Usage: " << argv[0] << " [input.xyz/off/ply/las]" << std::endl;
+    return EXIT_FAILURE;
+  }
+
+  const char* input_file = argv[1];
+  std::ifstream stream (input_file, std::ios_base::binary);
+  if (!stream)
+  {
+    std::cerr << "Error: cannot read file " << input_file << std::endl;
+    return EXIT_FAILURE;
+  }
+
+  stream >> points;
+
+  std::cout << "Read " << points.size () << " point(s)" << std::endl;
+  if (points.empty())
+    return EXIT_FAILURE;
+
+  //! [Reading input]
+  ///////////////////////////////////////////////////////////////////
+
+  ///////////////////////////////////////////////////////////////////
+  //! [Outlier removal]
+
+  CGAL::remove_outliers<CGAL::Sequential_tag>
+    (points,
+     24, // Number of neighbors considered for evaluation
+     points.parameters().threshold_percent (5.0)); // Percentage of points to remove
+
+  std::cout << points.number_of_removed_points()
+            << " point(s) are outliers." << std::endl;
+
+  // Applying point set processing algorithm to a CGAL::Point_set_3
+  // object does not erase the points from memory but place them in
+  // the garbage of the object: memory can be freeed by the user.
+  points.collect_garbage();
+
+  //! [Outlier removal]
+  ///////////////////////////////////////////////////////////////////
+
+  ///////////////////////////////////////////////////////////////////
+  //! [Simplification]
+
+  // Compute average spacing using neighborhood of 6 points
+  double spacing = CGAL::compute_average_spacing<CGAL::Sequential_tag> (points, 6);
+
+  // Simplify using a grid of size 2 * average spacing
+  CGAL::grid_simplify_point_set (points, 2. * spacing);
+
+  std::cout << points.number_of_removed_points()
+            << " point(s) removed after simplification." << std::endl;
+
+  points.collect_garbage();
+
+  //! [Simplification]
+  ///////////////////////////////////////////////////////////////////
+
+  ///////////////////////////////////////////////////////////////////
+  //! [Smoothing]
+
+  CGAL::jet_smooth_point_set<CGAL::Sequential_tag> (points, 24);
+
+  //! [Smoothing]
+  ///////////////////////////////////////////////////////////////////
+
+  unsigned int reconstruction_choice
+    = (argc < 3 ? 0 : atoi(argv[2]));
+
+  if (reconstruction_choice == 0) // Poisson
+  {
+    ///////////////////////////////////////////////////////////////////
+    //! [Normal estimation]
+
+    CGAL::jet_estimate_normals<CGAL::Sequential_tag>
+      (points, 24); // Use 24 neighbors
+
+    // Orientation of normals, returns iterator to first unoriented point
+    typename Point_set::iterator unoriented_points_begin =
+      CGAL::mst_orient_normals(points, 24); // Use 24 neighbors
+
+    points.remove (unoriented_points_begin, points.end());
+
+    //! [Normal estimation]
+    ///////////////////////////////////////////////////////////////////
+
+    ///////////////////////////////////////////////////////////////////
+    //! [Poisson reconstruction]
+
+    CGAL::Surface_mesh<Point_3> output_mesh;
+    CGAL::poisson_surface_reconstruction_delaunay
+      (points.begin(), points.end(),
+       points.point_map(), points.normal_map(),
+       output_mesh, spacing);
+
+    //! [Poisson reconstruction]
+    ///////////////////////////////////////////////////////////////////
+
+    ///////////////////////////////////////////////////////////////////
+    //! [Output poisson]
+
+    std::ofstream f ("out.ply", std::ios_base::binary);
+    CGAL::set_binary_mode (f);
+    CGAL::write_ply (f, output_mesh);
+    f.close ();
+
+    //! [Output poisson]
+    ///////////////////////////////////////////////////////////////////
+  }
+  else if (reconstruction_choice == 1) // Advancing front
+  {
+    ///////////////////////////////////////////////////////////////////
+    //! [Advancing front reconstruction]
+
+    typedef std::array<std::size_t, 3> Facet; // Triple of indices
+
+    std::vector<Facet> facets;
+
+    // The function is called using directly the points raw iterators
+    CGAL::advancing_front_surface_reconstruction(points.points().begin(),
+                                                 points.points().end(),
+                                                 std::back_inserter(facets));
+    std::cout << facets.size ()
+              << " facet(s) generated by reconstruction." << std::endl;
+
+    //! [Advancing front reconstruction]
+    ///////////////////////////////////////////////////////////////////
+
+    ///////////////////////////////////////////////////////////////////
+    //! [Output advancing front]
+
+    // copy points for random access
+    std::vector<Point_3> vertices;
+    vertices.reserve (points.size());
+    std::copy (points.points().begin(), points.points().end(), std::back_inserter (vertices));
+
+    CGAL::Surface_mesh<Point_3> output_mesh;
+    CGAL::Polygon_mesh_processing::polygon_soup_to_polygon_mesh (vertices, facets, output_mesh);
+    std::ofstream f ("out.off");
+    f << output_mesh;
+    f.close ();
+
+    //! [Output advancing front]
+    ///////////////////////////////////////////////////////////////////
+  }
+  else if (reconstruction_choice == 2) // Scale space
+  {
+    ///////////////////////////////////////////////////////////////////
+    //! [Scale space reconstruction]
+
+    CGAL::Scale_space_surface_reconstruction_3<Kernel> reconstruct
+      (points.points().begin(), points.points().end());
+
+    // Smooth using 4 iterations of Jet Smoothing
+    reconstruct.increase_scale (4, CGAL::Scale_space_reconstruction_3::Jet_smoother<Kernel>());
+    // Mesh with the Advancing Front mesher with a maximum facet length of 0.5
+    reconstruct.reconstruct_surface (CGAL::Scale_space_reconstruction_3::Advancing_front_mesher<Kernel>(0.5));
+
+    //! [Scale space reconstruction]
+    ///////////////////////////////////////////////////////////////////
+
+    ///////////////////////////////////////////////////////////////////
+    //! [Output scale space]
+
+    std::ofstream f ("out.off");
+    f << "OFF" << std::endl << points.size () << " "
+      << reconstruct.number_of_facets() << " 0" << std::endl;
+    for (Point_set::Index idx : points)
+      f << points.point (idx) << std::endl;
+    for (const auto& facet : CGAL::make_range (reconstruct.facets_begin(), reconstruct.facets_end()))
+      f << "3 "<< facet << std::endl;
+    f.close ();
+
+    //! [Output scale space]
+    ///////////////////////////////////////////////////////////////////
+  }
+  else // Handle error
+  {
+    std::cerr << "Error: invalid reconstruction id: " << reconstruction_choice << std::endl;
+    return EXIT_FAILURE;
+  }
+
+  return EXIT_SUCCESS;
+}
diff -pruN 5.0.3-1/examples/Polygonal_surface_reconstruction/CMakeLists.txt 5.1-1/examples/Polygonal_surface_reconstruction/CMakeLists.txt
--- 5.0.3-1/examples/Polygonal_surface_reconstruction/CMakeLists.txt	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Polygonal_surface_reconstruction/CMakeLists.txt	2020-09-08 12:12:20.000000000 +0000
@@ -36,58 +36,38 @@ endif()
 
 include( CGAL_CreateSingleSourceCGALProgram )
 
+find_package(Eigen3 3.1.0) #(requires 3.1.0 or greater)
+include(CGAL_Eigen_support)
+if(NOT TARGET CGAL::Eigen_support)
+  message(STATUS "NOTICE: This project requires Eigen 3.1 (or greater) and will not be compiled.")
+  return()
+endif()
+
+find_package(SCIP QUIET)
+include(CGAL_SCIP_support)
+if (NOT TARGET CGAL::SCIP_support )
+  find_package( GLPK QUIET)
+  include(CGAL_GLPK_support)
+  if (NOT TARGET CGAL::GLPK_support )
+    message( STATUS "NOTICE: This project requires either SCIP or GLPK, and will not be compiled.")
+    return()
+  endif()
+endif()
+
 create_single_source_cgal_program( "polyfit_example_without_input_planes.cpp" )
 create_single_source_cgal_program( "polyfit_example_user_provided_planes.cpp" )
 create_single_source_cgal_program( "polyfit_example_model_complexty_control.cpp" )
 create_single_source_cgal_program( "polyfit_example_with_region_growing.cpp" )
 
-find_package(Eigen3 3.1.0) #(requires 3.1.0 or greater)
-if(EIGEN3_FOUND)
-  include( ${EIGEN3_USE_FILE} )
-
-  find_package( SCIP QUIET)
-
-  if ( NOT SCIP_FOUND )
-      find_package( GLPK QUIET)
-
-      if ( NOT GLPK_FOUND )
-        message( STATUS "NOTICE: This project requires either SCIP or GLPK, and will not be compiled. "
-                        "Please provide either 'SCIP_DIR' or 'GLPK_INCLUDE_DIR' and 'GLPK_LIBRARIES'")
-      else()
-
-
-          include_directories( BEFORE ${GLPK_INCLUDE_DIR} )
-
-          target_link_libraries( polyfit_example_without_input_planes PRIVATE ${GLPK_LIBRARIES} )
-          target_compile_definitions(polyfit_example_without_input_planes PRIVATE -DCGAL_USE_GLPK)
-          target_link_libraries( polyfit_example_user_provided_planes PRIVATE ${GLPK_LIBRARIES} )
-          target_compile_definitions(polyfit_example_user_provided_planes PRIVATE -DCGAL_USE_GLPK)
-          target_link_libraries( polyfit_example_model_complexty_control PRIVATE ${GLPK_LIBRARIES} )
-          target_compile_definitions(polyfit_example_model_complexty_control PRIVATE -DCGAL_USE_GLPK)
-          target_link_libraries( polyfit_example_with_region_growing PRIVATE ${GLPK_LIBRARIES} )
-          target_compile_definitions(polyfit_example_with_region_growing PRIVATE -DCGAL_USE_GLPK)
-
-          message("GLPK found and used")
-
-      endif()
-
+foreach(target
+    polyfit_example_without_input_planes
+    polyfit_example_user_provided_planes
+    polyfit_example_model_complexty_control
+    polyfit_example_with_region_growing)
+  target_link_libraries(${target} PUBLIC CGAL::Eigen_support)
+  if (TARGET CGAL::SCIP_support)
+    target_link_libraries(${target} PUBLIC CGAL::SCIP_support)
   else()
-
-      include_directories( BEFORE ${SCIP_INCLUDE_DIRS} )
-
-      target_link_libraries( polyfit_example_without_input_planes PRIVATE ${SCIP_LIBRARIES} )
-      target_compile_definitions(polyfit_example_without_input_planes PRIVATE -DCGAL_USE_SCIP)
-      target_link_libraries( polyfit_example_user_provided_planes PRIVATE ${SCIP_LIBRARIES} )
-      target_compile_definitions(polyfit_example_user_provided_planes PRIVATE -DCGAL_USE_SCIP)
-      target_link_libraries( polyfit_example_model_complexty_control PRIVATE ${SCIP_LIBRARIES} )
-      target_compile_definitions(polyfit_example_model_complexty_control PRIVATE -DCGAL_USE_SCIP)
-      target_link_libraries( polyfit_example_with_region_growing PRIVATE ${SCIP_LIBRARIES} )
-      target_compile_definitions(polyfit_example_with_region_growing PRIVATE -DCGAL_USE_SCIP)
-
-      message("SCIP found and used")
-
+    target_link_libraries(${target} PUBLIC CGAL::GLPK_support)
   endif()
-
-else()
-  message(STATUS "NOTICE: Some of the executables in this directory need Eigen 3.1 (or greater) and will not be compiled.")
-endif()
+endforeach()
diff -pruN 5.0.3-1/examples/Polygonal_surface_reconstruction/polyfit_example_model_complexty_control.cpp 5.1-1/examples/Polygonal_surface_reconstruction/polyfit_example_model_complexty_control.cpp
--- 5.0.3-1/examples/Polygonal_surface_reconstruction/polyfit_example_model_complexty_control.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Polygonal_surface_reconstruction/polyfit_example_model_complexty_control.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -5,10 +5,10 @@
 #include <CGAL/Surface_mesh.h>
 #include <CGAL/Polygonal_surface_reconstruction.h>
 
-#ifdef CGAL_USE_SCIP
+#ifdef CGAL_USE_SCIP // defined (or not) by CMake scripts, do not define by hand
 #include <CGAL/SCIP_mixed_integer_program_traits.h>
 typedef CGAL::SCIP_mixed_integer_program_traits<double>                        MIP_Solver;
-#elif defined(CGAL_USE_GLPK)
+#elif defined(CGAL_USE_GLPK) // defined (or not) by CMake scripts, do not define by hand
 #include <CGAL/GLPK_mixed_integer_program_traits.h>
 typedef CGAL::GLPK_mixed_integer_program_traits<double>                        MIP_Solver;
 #endif
@@ -95,8 +95,11 @@ int main()
         else {
        const std::string& output_file = "data/building_result-0.05.off";
        std::ofstream output_stream(output_file.c_str());
-       if (output_stream && CGAL::write_off(output_stream, model))
+       if (output_stream && CGAL::write_off(output_stream, model)) {
+                        // flush the buffer
+                        output_stream << std::flush;
                         std::cout << " Done. Saved to " << output_file << ". Time: " << t.time() << " sec." << std::endl;
+       }
                 else {
            std::cerr << " Failed saving file." << std::endl;
            return EXIT_FAILURE;
@@ -113,8 +116,11 @@ int main()
         else {
        const std::string& output_file = "data/building_result-0.5.off";
        std::ofstream output_stream(output_file.c_str());
-       if (output_stream && CGAL::write_off(output_stream, model))
+       if (output_stream && CGAL::write_off(output_stream, model)) {
+                        // flush the buffer
+                        output_stream << std::flush;
                         std::cout << " Done. Saved to " << output_file << ". Time: " << t.time() << " sec." << std::endl;
+       }
                 else {
            std::cerr << " Failed saving file." << std::endl;
            return EXIT_FAILURE;
@@ -131,8 +137,11 @@ int main()
         else {
                 const std::string& output_file = "data/building_result-0.7.off";
                 std::ofstream output_stream(output_file.c_str());
-                if (output_stream && CGAL::write_off(output_stream, model))
+                if (output_stream && CGAL::write_off(output_stream, model)) {
+                        // flush the buffer
+                        output_stream << std::flush;
                         std::cout << " Done. Saved to " << output_file << ". Time: " << t.time() << " sec." << std::endl;
+                }
                 else {
                         std::cerr << " Failed saving file." << std::endl;
                         return EXIT_FAILURE;
diff -pruN 5.0.3-1/examples/Polygonal_surface_reconstruction/polyfit_example_user_provided_planes.cpp 5.1-1/examples/Polygonal_surface_reconstruction/polyfit_example_user_provided_planes.cpp
--- 5.0.3-1/examples/Polygonal_surface_reconstruction/polyfit_example_user_provided_planes.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Polygonal_surface_reconstruction/polyfit_example_user_provided_planes.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -5,10 +5,10 @@
 #include <CGAL/Surface_mesh.h>
 #include <CGAL/Polygonal_surface_reconstruction.h>
 
-#ifdef CGAL_USE_SCIP
+#ifdef CGAL_USE_SCIP  // defined (or not) by CMake scripts, do not define by hand
 #include <CGAL/SCIP_mixed_integer_program_traits.h>
 typedef CGAL::SCIP_mixed_integer_program_traits<double>                        MIP_Solver;
-#elif defined(CGAL_USE_GLPK)
+#elif defined(CGAL_USE_GLPK)  // defined (or not) by CMake scripts, do not define by hand
 #include <CGAL/GLPK_mixed_integer_program_traits.h>
 typedef CGAL::GLPK_mixed_integer_program_traits<double>                        MIP_Solver;
 #endif
@@ -94,8 +94,11 @@ int main()
         // Saves the mesh model
     const std::string& output_file("data/ball_result.off");
     std::ofstream output_stream(output_file.c_str());
-    if (output_stream && CGAL::write_off(output_stream, model))
+    if (output_stream && CGAL::write_off(output_stream, model)) {
+                // flush the buffer
+                output_stream << std::flush;
                 std::cout << " Done. Saved to " << output_file << ". Time: " << t.time() << " sec." << std::endl;
+    }
         else {
         std::cerr << " Failed saving file." << std::endl;
         return EXIT_FAILURE;
diff -pruN 5.0.3-1/examples/Polygonal_surface_reconstruction/polyfit_example_without_input_planes.cpp 5.1-1/examples/Polygonal_surface_reconstruction/polyfit_example_without_input_planes.cpp
--- 5.0.3-1/examples/Polygonal_surface_reconstruction/polyfit_example_without_input_planes.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Polygonal_surface_reconstruction/polyfit_example_without_input_planes.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -6,12 +6,12 @@
 #include <CGAL/Shape_detection/Efficient_RANSAC.h>
 #include <CGAL/Polygonal_surface_reconstruction.h>
 
-#ifdef CGAL_USE_SCIP
+#ifdef CGAL_USE_SCIP  // defined (or not) by CMake scripts, do not define by hand
 
 #include <CGAL/SCIP_mixed_integer_program_traits.h>
 typedef CGAL::SCIP_mixed_integer_program_traits<double>                        MIP_Solver;
 
-#elif defined(CGAL_USE_GLPK)
+#elif defined(CGAL_USE_GLPK)  // defined (or not) by CMake scripts, do not define by hand
 
 #include <CGAL/GLPK_mixed_integer_program_traits.h>
 typedef CGAL::GLPK_mixed_integer_program_traits<double>                        MIP_Solver;
@@ -129,8 +129,11 @@ int main()
 
         const std::string& output_file("data/cube_result.off");
         std::ofstream output_stream(output_file.c_str());
-        if (output_stream && CGAL::write_off(output_stream, model))
+        if (output_stream && CGAL::write_off(output_stream, model)) {
+                // flush the buffer
+                output_stream << std::flush;
                 std::cout << " Done. Saved to " << output_file << ". Time: " << t.time() << " sec." << std::endl;
+        }
         else {
                 std::cerr << " Failed saving file." << std::endl;
                 return EXIT_FAILURE;
@@ -143,8 +146,11 @@ int main()
         algo.output_candidate_faces(candidate_faces);
         const std::string& candidate_faces_file("data/cube_candidate_faces.off");
         std::ofstream candidate_stream(candidate_faces_file.c_str());
-        if (candidate_stream && CGAL::write_off(candidate_stream, candidate_faces))
+        if (candidate_stream && CGAL::write_off(candidate_stream, candidate_faces)) {
+                // flush the buffer
+                output_stream << std::flush;
                 std::cout << "Candidate faces saved to " << candidate_faces_file << "." << std::endl;
+        }
 
         return EXIT_SUCCESS;
 }
diff -pruN 5.0.3-1/examples/Polygonal_surface_reconstruction/polyfit_example_with_region_growing.cpp 5.1-1/examples/Polygonal_surface_reconstruction/polyfit_example_with_region_growing.cpp
--- 5.0.3-1/examples/Polygonal_surface_reconstruction/polyfit_example_with_region_growing.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Polygonal_surface_reconstruction/polyfit_example_with_region_growing.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -7,12 +7,12 @@
 #include <CGAL/Shape_detection/Region_growing/Region_growing_on_point_set.h>
 #include <CGAL/Polygonal_surface_reconstruction.h>
 
-#ifdef CGAL_USE_SCIP
+#ifdef CGAL_USE_SCIP  // defined (or not) by CMake scripts, do not define by hand
 
 #include <CGAL/SCIP_mixed_integer_program_traits.h>
 typedef CGAL::SCIP_mixed_integer_program_traits<double> MIP_Solver;
 
-#elif defined(CGAL_USE_GLPK)
+#elif defined(CGAL_USE_GLPK)  // defined (or not) by CMake scripts, do not define by hand
 
 #include <CGAL/GLPK_mixed_integer_program_traits.h>
 typedef CGAL::GLPK_mixed_integer_program_traits<double>        MIP_Solver;
@@ -178,8 +178,11 @@ int main()
   t.reset();
   const std::string& output_file("data/cube_result.off");
   std::ofstream output_stream(output_file.c_str());
-  if (output_stream && CGAL::write_off(output_stream, model))
+  if (output_stream && CGAL::write_off(output_stream, model)) {
+    // flush the buffer
+    output_stream << std::flush;
     std::cout << " Done. Saved to " << output_file << ". Time: " << t.time() << " sec." << std::endl;
+  }
   else {
     std::cerr << " Failed saving file." << std::endl;
     return EXIT_FAILURE;
@@ -196,4 +199,4 @@ int main(int, char**)
     return EXIT_SUCCESS;
 }
 
-#endif  // defined(CGAL_USE_GLPK) || defined(CGAL_USE_SCIP)
\ No newline at end of file
+#endif  // defined(CGAL_USE_GLPK) || defined(CGAL_USE_SCIP)
diff -pruN 5.0.3-1/examples/Polygon_mesh_processing/CMakeLists.txt 5.1-1/examples/Polygon_mesh_processing/CMakeLists.txt
--- 5.0.3-1/examples/Polygon_mesh_processing/CMakeLists.txt	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Polygon_mesh_processing/CMakeLists.txt	2020-09-08 12:12:20.000000000 +0000
@@ -38,6 +38,7 @@ endif()
 
 # include for local package
 find_package(Eigen3 3.2.0) #(requires 3.2.0 or greater)
+include(CGAL_Eigen_support)
 
 
 # Creating entries for all .cpp/.C files with "main" routine
@@ -45,16 +46,20 @@ find_package(Eigen3 3.2.0) #(requires 3.
 
 create_single_source_cgal_program( "hausdorff_distance_remeshing_example.cpp")
 
-if (EIGEN3_FOUND)
-  # Executables that require Eigen 3.2
-  include( ${EIGEN3_USE_FILE} )
-
+if (TARGET CGAL::Eigen_support)
   create_single_source_cgal_program( "hole_filling_example.cpp" )
+  target_link_libraries(hole_filling_example PUBLIC CGAL::Eigen_support)
   create_single_source_cgal_program( "hole_filling_example_SM.cpp" )
+  target_link_libraries(hole_filling_example_SM PUBLIC CGAL::Eigen_support)
   create_single_source_cgal_program( "refine_fair_example.cpp")
+  target_link_libraries(refine_fair_example PUBLIC CGAL::Eigen_support)
   create_single_source_cgal_program( "shape_smoothing_example.cpp")
-
-endif(EIGEN3_FOUND)
+  target_link_libraries(shape_smoothing_example PUBLIC CGAL::Eigen_support)
+  create_single_source_cgal_program( "hole_filling_example_LCC.cpp" )
+  target_link_libraries(hole_filling_example_LCC PUBLIC CGAL::Eigen_support)
+  create_single_source_cgal_program( "mesh_smoothing_example.cpp")
+  target_link_libraries(mesh_smoothing_example PUBLIC CGAL::Eigen_support)
+endif()
 
 create_single_source_cgal_program( "self_intersections_example.cpp" )
 create_single_source_cgal_program( "stitch_borders_example.cpp" )
@@ -80,19 +85,25 @@ create_single_source_cgal_program( "core
 create_single_source_cgal_program( "corefinement_polyhedron_union.cpp" )
 create_single_source_cgal_program( "random_perturbation_SM_example.cpp" )
 create_single_source_cgal_program( "corefinement_LCC.cpp")
-create_single_source_cgal_program( "hole_filling_example_LCC.cpp" )
 create_single_source_cgal_program( "detect_features_example.cpp" )
+create_single_source_cgal_program( "volume_connected_components.cpp" )
 create_single_source_cgal_program( "manifoldness_repair_example.cpp" )
 create_single_source_cgal_program( "repair_polygon_soup_example.cpp" )
-create_single_source_cgal_program( "mesh_smoothing_example.cpp")
 create_single_source_cgal_program( "locate_example.cpp")
+create_single_source_cgal_program( "orientation_pipeline_example.cpp")
+#create_single_source_cgal_program( "self_snapping_example.cpp")
+#create_single_source_cgal_program( "snapping_example.cpp")
+
 
 if(OpenMesh_FOUND)
+
 create_single_source_cgal_program( "compute_normals_example_OM.cpp" )
 target_link_libraries( compute_normals_example_OM PRIVATE ${OPENMESH_LIBRARIES} )
 
-create_single_source_cgal_program( "hole_filling_example_OM.cpp" )
-target_link_libraries( hole_filling_example_OM PRIVATE ${OPENMESH_LIBRARIES} )
+if (TARGET CGAL::Eigen_support)
+  create_single_source_cgal_program( "hole_filling_example_OM.cpp" )
+  target_link_libraries( hole_filling_example_OM PRIVATE CGAL::Eigen_support ${OPENMESH_LIBRARIES} )
+endif()
 
 create_single_source_cgal_program( "point_inside_example_OM.cpp")
 target_link_libraries( point_inside_example_OM PRIVATE ${OPENMESH_LIBRARIES} )
@@ -100,16 +111,18 @@ target_link_libraries( point_inside_exam
 create_single_source_cgal_program( "stitch_borders_example_OM.cpp" )
 target_link_libraries( stitch_borders_example_OM PRIVATE ${OPENMESH_LIBRARIES} )
 
-#create_single_source_cgal_program( "remove_degeneracies_example_OM.cpp")
-#target_link_libraries( remove_degeneracies_example_OM PRIVATE ${OPENMESH_LIBRARIES} )
+#create_single_source_cgal_program( "remove_degeneracies_example.cpp")
+#target_link_libraries( remove_degeneracies_example PRIVATE ${OPENMESH_LIBRARIES} )
 
 create_single_source_cgal_program( "triangulate_faces_example_OM.cpp")
 target_link_libraries( triangulate_faces_example_OM PRIVATE ${OPENMESH_LIBRARIES} )
 endif(OpenMesh_FOUND)
 
 find_package( TBB )
-if( TBB_FOUND )
-  CGAL_target_use_TBB(hausdorff_distance_remeshing_example)
+include(CGAL_TBB_support)
+if( TARGET CGAL::TBB_support )
+  target_link_libraries(self_intersections_example PUBLIC CGAL::TBB_support)
+  target_link_libraries(hausdorff_distance_remeshing_example PUBLIC CGAL::TBB_support)
 else()
   message( STATUS "NOTICE: Intel TBB was not found. Sequential code will be used." )
 endif()
@@ -119,4 +132,3 @@ if(TARGET ceres)
   target_compile_definitions( mesh_smoothing_example PRIVATE CGAL_PMP_USE_CERES_SOLVER )
   target_link_libraries( mesh_smoothing_example PRIVATE ceres )
 endif(TARGET ceres)
-
diff -pruN 5.0.3-1/examples/Polygon_mesh_processing/compute_normals_example.cpp 5.1-1/examples/Polygon_mesh_processing/compute_normals_example.cpp
--- 5.0.3-1/examples/Polygon_mesh_processing/compute_normals_example.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Polygon_mesh_processing/compute_normals_example.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -6,13 +6,16 @@
 #include <iostream>
 #include <fstream>
 
-typedef CGAL::Exact_predicates_inexact_constructions_kernel K;
-typedef K::Point_3 Point;
-typedef K::Vector_3 Vector;
-
-typedef CGAL::Surface_mesh<Point> Surface_mesh;
-typedef boost::graph_traits<Surface_mesh>::vertex_descriptor vertex_descriptor;
-typedef boost::graph_traits<Surface_mesh>::face_descriptor   face_descriptor;
+namespace PMP = CGAL::Polygon_mesh_processing;
+
+typedef CGAL::Exact_predicates_inexact_constructions_kernel       K;
+
+typedef K::Point_3                                                Point;
+typedef K::Vector_3                                               Vector;
+
+typedef CGAL::Surface_mesh<Point>                                 Surface_mesh;
+typedef boost::graph_traits<Surface_mesh>::vertex_descriptor      vertex_descriptor;
+typedef boost::graph_traits<Surface_mesh>::face_descriptor        face_descriptor;
 
 int main(int argc, char* argv[])
 {
@@ -25,24 +28,18 @@ int main(int argc, char* argv[])
     return 1;
   }
 
-  auto fnormals = mesh.add_property_map<face_descriptor, Vector>
-      ("f:normals", CGAL::NULL_VECTOR).first;
-  auto vnormals = mesh.add_property_map<vertex_descriptor, Vector>
-      ("v:normals", CGAL::NULL_VECTOR).first;
-
-  CGAL::Polygon_mesh_processing::compute_normals(mesh,
-        vnormals,
-        fnormals,
-        CGAL::Polygon_mesh_processing::parameters::vertex_point_map(mesh.points()).
-        geom_traits(K()));
+  auto vnormals = mesh.add_property_map<vertex_descriptor, Vector>("v:normals", CGAL::NULL_VECTOR).first;
+  auto fnormals = mesh.add_property_map<face_descriptor, Vector>("f:normals", CGAL::NULL_VECTOR).first;
+
+  PMP::compute_normals(mesh, vnormals, fnormals);
 
-  std::cout << "Face normals :" << std::endl;
-  for(face_descriptor fd: faces(mesh)){
-    std::cout << fnormals[fd] << std::endl;
-  }
   std::cout << "Vertex normals :" << std::endl;
-  for(vertex_descriptor vd: vertices(mesh)){
+  for(vertex_descriptor vd: vertices(mesh))
     std::cout << vnormals[vd] << std::endl;
-  }
+
+  std::cout << "Face normals :" << std::endl;
+  for(face_descriptor fd: faces(mesh))
+    std::cout << fnormals[fd] << std::endl;
+
   return 0;
 }
diff -pruN 5.0.3-1/examples/Polygon_mesh_processing/corefinement_consecutive_bool_op.cpp 5.1-1/examples/Polygon_mesh_processing/corefinement_consecutive_bool_op.cpp
--- 5.0.3-1/examples/Polygon_mesh_processing/corefinement_consecutive_bool_op.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Polygon_mesh_processing/corefinement_consecutive_bool_op.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -16,7 +16,7 @@ typedef Mesh::Property_map<vertex_descri
 namespace PMP = CGAL::Polygon_mesh_processing;
 namespace params = PMP::parameters;
 
-struct Coref_point_map
+struct Exact_vertex_point_map
 {
   // typedef for the property map
   typedef boost::property_traits<Exact_point_map>::value_type value_type;
@@ -25,64 +25,39 @@ struct Coref_point_map
   typedef boost::property_traits<Exact_point_map>::key_type key_type;
 
   // exterior references
-  Exact_point_computed* exact_point_computed_ptr;
-  Exact_point_map* exact_point_ptr;
-  Mesh* mesh_ptr;
-
-  Exact_point_computed& exact_point_computed() const
-  {
-    CGAL_assertion(exact_point_computed_ptr!=NULL);
-    return *exact_point_computed_ptr;
-  }
-
-  Exact_point_map& exact_point() const
-  {
-    CGAL_assertion(exact_point_ptr!=NULL);
-    return *exact_point_ptr;
-  }
-
-  Mesh& mesh() const
-  {
-    CGAL_assertion(mesh_ptr!=NULL);
-    return *mesh_ptr;
-  }
+  Exact_point_map exact_point_map;
+  Mesh* tm_ptr;
 
   // Converters
   CGAL::Cartesian_converter<K, EK> to_exact;
   CGAL::Cartesian_converter<EK, K> to_input;
 
-  Coref_point_map()
-    : exact_point_computed_ptr(NULL)
-    , exact_point_ptr(NULL)
-    , mesh_ptr(NULL)
+  Exact_vertex_point_map()
+    : tm_ptr(nullptr)
   {}
 
-  Coref_point_map(Exact_point_map& ep,
-                  Exact_point_computed& epc,
-                  Mesh& m)
-    : exact_point_computed_ptr(&epc)
-    , exact_point_ptr(&ep)
-    , mesh_ptr(&m)
-  {}
+  Exact_vertex_point_map(const Exact_point_map& ep, Mesh& tm)
+    : exact_point_map(ep)
+    , tm_ptr(&tm)
+  {
+    for (Mesh::Vertex_index v : vertices(tm))
+      exact_point_map[v]=to_exact(tm.point(v));
+  }
 
   friend
-  reference get(const Coref_point_map& map, key_type k)
+  reference get(const Exact_vertex_point_map& map, key_type k)
   {
-    // create exact point if it does not exist
-    if (!map.exact_point_computed()[k]){
-      map.exact_point()[k]=map.to_exact(map.mesh().point(k));
-      map.exact_point_computed()[k]=true;
-    }
-    return map.exact_point()[k];
+    CGAL_precondition(map.tm_ptr!=nullptr);
+    return map.exact_point_map[k];
   }
 
   friend
-  void put(const Coref_point_map& map, key_type k, const EK::Point_3& p)
+  void put(const Exact_vertex_point_map& map, key_type k, const EK::Point_3& p)
   {
-    map.exact_point_computed()[k]=true;
-    map.exact_point()[k]=p;
+    CGAL_precondition(map.tm_ptr!=nullptr);
+    map.exact_point_map[k]=p;
     // create the input point from the exact one
-    map.mesh().point(k)=map.to_input(p);
+    map.tm_ptr->point(k)=map.to_input(p);
   }
 };
 
@@ -109,33 +84,30 @@ int main(int argc, char* argv[])
 
   Exact_point_map mesh1_exact_points =
     mesh1.add_property_map<vertex_descriptor,EK::Point_3>("e:exact_point").first;
-  Exact_point_computed mesh1_exact_points_computed =
-    mesh1.add_property_map<vertex_descriptor,bool>("e:exact_points_computed").first;
 
   Exact_point_map mesh2_exact_points =
     mesh2.add_property_map<vertex_descriptor,EK::Point_3>("e:exact_point").first;
-  Exact_point_computed mesh2_exact_points_computed =
-    mesh2.add_property_map<vertex_descriptor,bool>("e:exact_points_computed").first;
 
-  Coref_point_map mesh1_pm(mesh1_exact_points, mesh1_exact_points_computed, mesh1);
-  Coref_point_map mesh2_pm(mesh2_exact_points, mesh2_exact_points_computed, mesh2);
+  Exact_vertex_point_map mesh1_vpm(mesh1_exact_points, mesh1);
+  Exact_vertex_point_map mesh2_vpm(mesh2_exact_points, mesh2);
 
   if ( PMP::corefine_and_compute_intersection(mesh1,
                                               mesh2,
                                               mesh1,
-                                              params::vertex_point_map(mesh1_pm),
-                                              params::vertex_point_map(mesh2_pm),
-                                              params::vertex_point_map(mesh1_pm) ) )
+                                              params::vertex_point_map(mesh1_vpm),
+                                              params::vertex_point_map(mesh2_vpm),
+                                              params::vertex_point_map(mesh1_vpm) ) )
   {
     if ( PMP::corefine_and_compute_union(mesh1,
                                          mesh2,
                                          mesh2,
-                                         params::vertex_point_map(mesh1_pm),
-                                         params::vertex_point_map(mesh2_pm),
-                                         params::vertex_point_map(mesh2_pm) ) )
+                                         params::vertex_point_map(mesh1_vpm),
+                                         params::vertex_point_map(mesh2_vpm),
+                                         params::vertex_point_map(mesh2_vpm) ) )
     {
       std::cout << "Intersection and union were successfully computed\n";
       std::ofstream output("inter_union.off");
+      output.precision(17);
       output << mesh2;
       return 0;
     }
diff -pruN 5.0.3-1/examples/Polygon_mesh_processing/corefinement_difference_remeshed.cpp 5.1-1/examples/Polygon_mesh_processing/corefinement_difference_remeshed.cpp
--- 5.0.3-1/examples/Polygon_mesh_processing/corefinement_difference_remeshed.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Polygon_mesh_processing/corefinement_difference_remeshed.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -67,6 +67,7 @@ int main(int argc, char* argv[])
   {
     std::cout << "Difference was successfully computed\n";
     std::ofstream output("difference.off");
+    output.precision(17);
     output << mesh1;
   }
   else{
@@ -111,6 +112,7 @@ int main(int argc, char* argv[])
     params::edge_is_constrained_map(is_constrained_map) );
 
   std::ofstream output("difference_remeshed.off");
+  output.precision(17);
   output << mesh1;
 
   return 0;
diff -pruN 5.0.3-1/examples/Polygon_mesh_processing/corefinement_LCC.cpp 5.1-1/examples/Polygon_mesh_processing/corefinement_LCC.cpp
--- 5.0.3-1/examples/Polygon_mesh_processing/corefinement_LCC.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Polygon_mesh_processing/corefinement_LCC.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -34,6 +34,7 @@ int main(int argc, char* argv[])
             << num_vertices(mesh2) << "\n";
 
   std::ofstream output("mesh1_refined.off");
+  output.precision(17);
   CGAL::write_off(output, mesh1);
   output.close();
   output.open("mesh2_refined.off");
diff -pruN 5.0.3-1/examples/Polygon_mesh_processing/corefinement_mesh_union_and_intersection.cpp 5.1-1/examples/Polygon_mesh_processing/corefinement_mesh_union_and_intersection.cpp
--- 5.0.3-1/examples/Polygon_mesh_processing/corefinement_mesh_union_and_intersection.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Polygon_mesh_processing/corefinement_mesh_union_and_intersection.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -55,6 +55,7 @@ int main(int argc, char* argv[])
   {
     std::cout << "Union was successfully computed\n";
     std::ofstream output("union.off");
+    output.precision(17);
     output << out_union;
   }
   else
@@ -64,6 +65,7 @@ int main(int argc, char* argv[])
   {
     std::cout << "Intersection was successfully computed\n";
     std::ofstream output("intersection.off");
+    output.precision(17);
     output << out_intersection;
   }
   else
diff -pruN 5.0.3-1/examples/Polygon_mesh_processing/corefinement_mesh_union.cpp 5.1-1/examples/Polygon_mesh_processing/corefinement_mesh_union.cpp
--- 5.0.3-1/examples/Polygon_mesh_processing/corefinement_mesh_union.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Polygon_mesh_processing/corefinement_mesh_union.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -37,6 +37,7 @@ int main(int argc, char* argv[])
   {
     std::cout << "Union was successfully computed\n";
     std::ofstream output("union.off");
+    output.precision(17);
     output << out;
     return 0;
   }
diff -pruN 5.0.3-1/examples/Polygon_mesh_processing/corefinement_mesh_union_with_attributes.cpp 5.1-1/examples/Polygon_mesh_processing/corefinement_mesh_union_with_attributes.cpp
--- 5.0.3-1/examples/Polygon_mesh_processing/corefinement_mesh_union_with_attributes.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Polygon_mesh_processing/corefinement_mesh_union_with_attributes.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -94,6 +94,7 @@ int main(int argc, char* argv[])
   {
     std::cout << "Union was successfully computed\n";
     std::ofstream output("union.off");
+    output.precision(17);
     output << out;
     return 0;
   }
diff -pruN 5.0.3-1/examples/Polygon_mesh_processing/corefinement_polyhedron_union.cpp 5.1-1/examples/Polygon_mesh_processing/corefinement_polyhedron_union.cpp
--- 5.0.3-1/examples/Polygon_mesh_processing/corefinement_polyhedron_union.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Polygon_mesh_processing/corefinement_polyhedron_union.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -39,6 +39,7 @@ int main(int argc, char* argv[])
   {
     std::cout << "Union was successfully computed\n";
     std::ofstream output("union.off");
+    output.precision(17);
     output << out;
     return 0;
   }
diff -pruN 5.0.3-1/examples/Polygon_mesh_processing/corefinement_SM.cpp 5.1-1/examples/Polygon_mesh_processing/corefinement_SM.cpp
--- 5.0.3-1/examples/Polygon_mesh_processing/corefinement_SM.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Polygon_mesh_processing/corefinement_SM.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -41,6 +41,7 @@ int main(int argc, char* argv[])
             << num_vertices(mesh2) << "\n";
 
   std::ofstream output("mesh1_refined.off");
+  output.precision(17);
   output << mesh1;
   output.close();
   output.open("mesh2_refined.off");
diff -pruN 5.0.3-1/examples/Polygon_mesh_processing/data/blobby-shuffled.off 5.1-1/examples/Polygon_mesh_processing/data/blobby-shuffled.off
--- 5.0.3-1/examples/Polygon_mesh_processing/data/blobby-shuffled.off	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Polygon_mesh_processing/data/blobby-shuffled.off	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,6104 @@
+# Output of a CGAL tool
+#CBP
+# polyhedral_surface 0
+# halfedges 0
+# triangulated 0
+# non_empty_facets 0
+# terrain 0
+# normalized_to_sphere 0
+# radius 0
+# rounded 0
+# rounded_bits 0
+# ENDCBP
+
+OFF
+2027 4050 0
+
+# 2027 vertices
+# ------------------------------------------
+
+
+0.095849400000000001 0.00115985 -0.166522
+-0.0890262 -0.025355099999999998 -0.18482899999999999
+0.027536999999999999 -0.14418500000000001 0.12545899999999999
+-0.095787600000000001 -0.13067799999999999 0.14583599999999999
+-0.20519999999999999 0.091158199999999995 -0.071373599999999995
+0.23333999999999999 0.15764300000000001 -0.079066399999999995
+0.23333000000000001 0.18367900000000001 0.031482900000000001
+0.011625399999999999 0.0348884 0.195738
+0.061604699999999998 0.16266600000000001 0.15098
+0.13172500000000001 0.21107500000000001 -0.0224619
+-0.15770999999999999 -0.062169500000000003 -0.114255
+-0.32595299999999999 -0.116315 0.036301699999999999
+-0.25827699999999998 -9.9632799999999999e-05 -0.066961999999999994
+0.13411300000000001 -0.042284000000000002 0.121878
+0.30214299999999999 0.124693 0.029570300000000001
+0.133766 0.13006100000000001 -0.094515100000000005
+0.18373900000000001 0.12406499999999999 0.13048199999999999
+-0.039973399999999999 -0.094566600000000001 -0.18862499999999999
+-0.39484200000000003 0.062733800000000006 0.0053626899999999998
+-0.13799800000000001 0.173848 0.096777500000000002
+-0.038674800000000002 0.229049 0.036186200000000002
+-0.039295200000000002 -0.16598199999999999 -0.11582199999999999
+-0.171824 -0.17691200000000001 -0.0086522700000000001
+-0.15034600000000001 -0.0322864 0.16289999999999999
+-0.21992300000000001 -0.105035 -0.044157000000000002
+-0.172879 0.078466900000000006 0.134243
+0.016566999999999998 0.21765100000000001 -0.019554599999999998
+0.079410099999999997 -0.0895981 -0.047201300000000002
+-0.31369799999999998 0.038077399999999997 0.094854999999999995
+0.31854700000000002 0.125087 -0.019869399999999999
+0.23227 0.0225103 -0.021871499999999999
+0.28860200000000003 0.062350299999999997 -0.0028626099999999998
+0.232539 0.033764500000000003 0.088465600000000005
+-0.257774 -0.0579123 0.114719
+0.056384900000000002 0.22962199999999999 0.073868199999999995
+0.27843600000000002 0.16783200000000001 -0.0053950400000000003
+-0.31471300000000002 0.091573799999999997 -0.060780500000000001
+0.085349099999999997 0.066700200000000001 -0.13686200000000001
+0.0238762 -0.124001 -0.17752000000000001
+0.025990099999999999 -0.039926999999999997 -0.19495199999999999
+-0.105554 0.070428099999999993 -0.15769900000000001
+0.28937400000000002 0.136378 -0.071541099999999996
+0.235568 0.076571600000000004 -0.082325300000000004
+-0.245592 -0.0554978 -0.058124799999999997
+-0.39116899999999999 -0.063444299999999995 -0.086140999999999995
+-0.31947300000000001 -0.0445621 -0.070685100000000001
+-0.29419699999999999 -0.11161699999999999 -0.049417599999999999
+-0.37454900000000002 0.0271624 -0.088377200000000003
+-0.25372899999999998 -0.173014 -0.0128907
+-0.21636 -0.14896999999999999 0.082385399999999998
+-0.066871 0.112543 0.16330800000000001
+-0.134132 -0.13648199999999999 -0.075767200000000007
+-0.068210400000000004 -0.196549 -0.0186103
+0.028011600000000001 0.037949999999999998 -0.18273200000000001
+-0.21726500000000001 0.036772199999999998 -0.076175099999999996
+-0.16211500000000001 0.056705999999999999 -0.114965
+-0.25992100000000001 0.073993799999999998 -0.0650284
+-0.26382699999999998 0.132437 0.034598700000000003
+-0.244668 0.13427600000000001 -0.032760900000000003
+-0.14700299999999999 0.16924900000000001 -0.035537899999999997
+-0.20061100000000001 0.16917099999999999 0.0171526
+-0.204369 0.13125800000000001 0.085634699999999994
+0.161721 0.051829300000000002 -0.109485
+-0.30307200000000001 0.036158299999999997 -0.073154499999999997
+-0.179865 -0.086242899999999997 -0.072876099999999999
+0.226968 0.189503 -0.028229799999999999
+0.17320099999999999 0.18445600000000001 -0.067608000000000001
+0.18254300000000001 0.205347 0.0085282099999999996
+0.175562 0.18662799999999999 0.084190899999999999
+0.125387 0.21876300000000001 0.047042599999999997
+0.111015 0.194442 0.111774
+-0.17502999999999999 -0.129411 -0.045934200000000001
+0.17146900000000001 -0.016288500000000001 0.030969099999999999
+0.062003299999999997 -0.1101 0.043838599999999998
+0.00082855799999999996 -0.18148600000000001 0.043399399999999998
+-0.091978000000000004 -0.221835 0.085260799999999998
+-0.21368599999999999 -0.148588 -0.0242963
+-0.25551600000000002 -0.132659 -0.034891600000000002
+-0.29915799999999998 -0.15709400000000001 -0.028081499999999999
+-0.37190299999999998 -0.12923299999999999 -0.045323299999999997
+-0.28935100000000002 -0.15696099999999999 0.026597300000000001
+-0.27149099999999998 -0.114067 0.073301199999999997
+-0.319216 -0.063680299999999995 0.081839200000000001
+-0.41884900000000003 -0.0484997 0.0253466
+-0.24013000000000001 -0.18160399999999999 0.041943500000000002
+-0.168769 -0.21110100000000001 0.064175899999999994
+-0.031372900000000002 0.18620100000000001 -0.079775200000000004
+0.0310134 0.15207200000000001 -0.11733
+-0.047766799999999998 0.13189600000000001 -0.143761
+0.12242400000000001 0.139982 0.14562700000000001
+0.13770299999999999 0.045102799999999998 0.16714499999999999
+0.070826299999999995 0.087710399999999994 0.17714299999999999
+-0.25870100000000001 0.084023500000000001 0.087675400000000001
+-0.245925 0.020130700000000001 0.137908
+-0.31407000000000002 0.087551900000000002 0.047375199999999999
+-0.48033700000000001 -0.0353406 -0.045931
+-0.44279400000000002 0.010582299999999999 -0.100726
+0.075401599999999999 0.22422900000000001 0.0064139399999999999
+0.0053423500000000001 0.12164 0.17535200000000001
+-0.034947600000000002 0.200792 0.131192
+0.072180999999999995 -0.10968799999999999 -0.12035
+0.119341 -0.053477299999999998 0.056595899999999998
+0.086788500000000005 -0.090323000000000001 0.10555200000000001
+0.0632331 -0.043990000000000001 0.18492700000000001
+-0.039255600000000002 -0.053976200000000002 0.18488199999999999
+-0.36137399999999997 0.047788600000000001 0.055829200000000002
+-0.36119299999999999 -0.010701799999999999 0.0908968
+-0.416327 0.0142746 0.0438039
+-0.45555699999999999 0.027654600000000001 -0.017024299999999999
+-0.032399299999999999 -0.15038099999999999 -0.16214200000000001
+-0.097041600000000006 -0.12747 -0.13774800000000001
+0.015038899999999999 -0.155421 -0.13047600000000001
+0.039478199999999998 -0.13985 -0.0691441
+0.27858699999999997 0.072167999999999996 0.0614076
+0.24014099999999999 0.096043500000000004 0.10983
+0.196467 0.056071099999999999 0.13908300000000001
+0.18276400000000001 -0.00083176300000000001 0.111317
+-0.21378 -0.18512500000000001 -0.00072420799999999995
+-0.18268400000000001 -0.20657500000000001 0.022381399999999999
+-0.116864 -0.21174100000000001 0.0233662
+0.128798 -0.047684600000000001 -0.070965299999999995
+-0.151224 -0.159278 0.110503
+-0.18671699999999999 -0.0912332 0.126278
+-0.136046 0.197824 0.0292673
+-0.079970100000000002 0.20249500000000001 -0.021901899999999998
+-0.090814800000000001 0.21491099999999999 0.074684100000000003
+0.22185199999999999 0.15013699999999999 0.092841499999999993
+0.268648 0.127162 0.073768500000000001
+-0.100355 0.15726799999999999 -0.094683600000000007
+-0.151142 0.11729299999999999 -0.095301399999999994
+-0.059825099999999999 -0.21344299999999999 0.0369284
+-0.032155700000000002 -0.20306399999999999 0.092727100000000007
+-0.0758939 -0.18407699999999999 0.12844800000000001
+-0.0311162 -0.14380999999999999 0.15808700000000001
+0.017880699999999999 -0.096989699999999998 0.176592
+0.047416399999999997 -0.12537899999999999 -0.011883700000000001
+0.137437 -0.034659799999999998 0.0136343
+0.177953 -0.0103553 -0.035624000000000003
+0.13910900000000001 -0.034972900000000001 -0.034761199999999999
+0.158771 -0.024122000000000001 -0.065972600000000006
+0.141486 -0.020496 -0.127385
+0.19728200000000001 0.0147799 -0.083601300000000003
+0.098925899999999997 -0.061201499999999999 -0.152448
+0.14621600000000001 -0.033117599999999997 -0.096215800000000004
+0.11569500000000001 -0.062037799999999997 -0.111119
+0.168295 -0.0014477800000000001 -0.105241
+0.14328399999999999 0.020743399999999999 -0.12954299999999999
+0.123336 0.056542099999999998 -0.124097
+0.107295 0.097032499999999994 -0.111084
+0.083027799999999999 0.14480399999999999 -0.10519199999999999
+0.116715 0.18219199999999999 -0.077579300000000004
+0.064937800000000004 0.108018 -0.128027
+0.0088614899999999996 0.108014 -0.155996
+-0.0439072 0.076694399999999996 -0.178622
+0.043701299999999998 0.0767872 -0.155644
+0.10263899999999999 0.033935300000000002 -0.14744099999999999
+0.066484799999999997 0.0308188 -0.16812199999999999
+0.057418499999999997 -0.0086220099999999994 -0.18768599999999999
+0.0659222 -0.051221200000000001 -0.17997299999999999
+0.035288800000000002 -0.082328200000000004 -0.18673799999999999
+-0.0051496399999999996 -0.069933499999999996 -0.19578599999999999
+-0.029625100000000001 -0.013350900000000001 -0.19615099999999999
+-0.043444499999999997 -0.0528929 -0.192773
+-0.087276199999999998 -0.076467999999999994 -0.18007899999999999
+-0.12914700000000001 -0.050085699999999997 -0.15846499999999999
+-0.12928400000000001 0.0124896 -0.15528400000000001
+0.069558400000000006 -0.089867900000000001 -0.16298799999999999
+0.14744399999999999 0.090383400000000003 -0.099623100000000006
+0.18867400000000001 0.125502 -0.087038099999999993
+0.19051399999999999 0.081444600000000006 -0.092952800000000002
+0.054356500000000002 0.19081799999999999 -0.074467699999999998
+-0.067633600000000002 0.026418199999999999 -0.18554899999999999
+0.018353600000000001 0.000577163 -0.19738900000000001
+-0.016639299999999999 0.031641000000000002 -0.195381
+-0.0014350000000000001 0.069468299999999997 -0.18121200000000001
+0.248561 0.031737799999999997 0.032890500000000003
+-0.12876199999999999 -0.092239299999999996 -0.13675499999999999
+-0.14779999999999999 -0.10176 -0.098559599999999997
+-0.18376899999999999 0.00080942799999999995 -0.0981379
+-0.14241300000000001 -0.15939300000000001 -0.037900400000000001
+-0.093485299999999993 -0.17150799999999999 -0.061299899999999997
+-0.039723000000000001 -0.182589 -0.063459199999999993
+-0.011640599999999999 -0.185728 -0.0129543
+-0.13089400000000001 -0.187892 -0.0087295500000000009
+-0.00093736299999999998 -0.16902900000000001 -0.087658100000000003
+-0.36682399999999998 -0.072428199999999998 0.045307699999999999
+-0.39464199999999999 -0.095481200000000002 -0.0019439500000000001
+-0.44067400000000001 -0.069291599999999995 -0.0230396
+-0.458899 -0.026837099999999999 0.00052061700000000002
+-0.44978099999999999 -0.071578100000000006 -0.081636200000000006
+-0.41608699999999998 -0.102536 -0.055614200000000003
+-0.31006299999999998 0.12037299999999999 -0.0055827300000000002
+-0.35955700000000002 0.097333900000000001 -0.024699800000000001
+-0.40859800000000002 0.071229899999999999 -0.056654700000000002
+0.210226 0.0065472999999999998 0.0528917
+0.10443 -0.064571299999999998 -0.051228099999999999
+0.092888499999999999 -0.083854899999999996 -0.085273299999999996
+0.0647924 -0.11242099999999999 -0.073180499999999996
+-0.25816 -0.095233799999999993 -0.047642299999999999
+-0.29016999999999998 -0.071502599999999999 -0.059089900000000001
+-0.33406599999999997 -0.090019399999999999 -0.065301200000000004
+-0.088115399999999997 0.18359 0.121824
+-0.11695800000000001 0.131991 0.135245
+-0.11552800000000001 0.074086700000000005 0.15803300000000001
+-0.056259700000000003 0.046087400000000001 0.18268200000000001
+-0.15470700000000001 0.028122899999999999 0.16259199999999999
+-0.101345 0.00172185 0.18399799999999999
+-0.200295 -0.0055454800000000002 0.159778
+-0.055074999999999999 0.15928200000000001 0.15029400000000001
+-0.0051326100000000001 0.16801199999999999 0.15662999999999999
+0.025650900000000001 0.210455 0.123931
+-0.000484079 0.23336999999999999 0.078525600000000001
+0.024306299999999999 0.23203299999999999 0.0312676
+0.29746800000000001 0.085647699999999993 -0.051891600000000003
+0.264291 0.051681299999999999 -0.043179599999999999
+-0.00033375899999999999 -0.14362 -0.17393
+-0.0081177899999999997 -0.111725 -0.18914500000000001
+0.023316799999999999 -0.144538 -0.156498
+0.052056900000000003 -0.121215 -0.153498
+0.040349200000000002 -0.13908899999999999 -0.13219700000000001
+0.0307313 -0.148756 -0.10091
+0.052015199999999998 -0.12936600000000001 -0.093870599999999998
+-0.035846900000000001 -0.12528300000000001 -0.18140000000000001
+-0.071996299999999999 -0.112037 -0.17452500000000001
+-0.057900500000000001 -0.13610900000000001 -0.165268
+-0.053518999999999997 -0.152974 -0.13985900000000001
+-0.082604800000000006 -0.15557499999999999 -0.104309
+-0.102837 -0.101678 -0.161026
+-0.080197099999999993 -0.12789300000000001 -0.156609
+-0.076707200000000003 -0.142872 -0.13619899999999999
+0.0010389100000000001 -0.15568399999999999 -0.151643
+-0.0126392 -0.16380500000000001 -0.12728
+-0.15321299999999999 -0.020072900000000001 -0.132827
+-0.157003 0.019224499999999999 -0.127469
+-0.11382 -0.074437900000000001 -0.16651199999999999
+-0.10638400000000001 -0.047926999999999997 -0.17710799999999999
+-0.11755500000000001 -0.016950699999999999 -0.16802900000000001
+-0.099159399999999995 0.0131089 -0.174764
+-0.091370599999999996 0.044859200000000002 -0.17249600000000001
+-0.075640600000000002 0.073617600000000005 -0.170824
+-0.087346800000000002 0.11047800000000001 -0.147509
+-0.059453199999999998 0.101489 -0.163026
+-0.024470800000000001 0.103135 -0.16602800000000001
+-0.12742300000000001 0.098632200000000003 -0.129408
+-0.118816 0.041986900000000001 -0.157553
+-0.133878 0.066418900000000003 -0.13894000000000001
+-0.0153011 0.128439 -0.14722399999999999
+-0.0252286 0.16062100000000001 -0.11677800000000001
+0.0063503700000000001 0.1777 -0.095487299999999997
+0.0040088499999999996 0.20053299999999999 -0.058022200000000003
+-0.031746200000000002 0.211172 -0.0259917
+-0.081196099999999993 -0.050982800000000002 -0.185782
+-0.061615200000000002 -0.030768799999999999 -0.19125400000000001
+-0.070431400000000005 -0.0027779599999999999 -0.18848699999999999
+-0.062954099999999999 -0.069891300000000003 -0.18779499999999999
+-0.045516399999999999 0.0102212 -0.193522
+-0.043004300000000002 0.037348699999999999 -0.19054699999999999
+-0.0241987 0.0567533 -0.18857599999999999
+-0.063225000000000003 0.052565099999999997 -0.18177699999999999
+-0.065562599999999999 0.15537300000000001 -0.11569500000000001
+-0.068756800000000007 0.17671700000000001 -0.083513199999999996
+-0.104738 0.17966799999999999 -0.054078000000000001
+-0.056661599999999999 0.19581100000000001 -0.052082000000000003
+-0.094967399999999993 0.13598199999999999 -0.123239
+-0.123404 0.13445299999999999 -0.104745
+-0.14158899999999999 0.14890800000000001 -0.070838999999999999
+-0.18996499999999999 0.138072 -0.048089300000000001
+-0.139982 0.033953900000000002 -0.14260800000000001
+-0.069864399999999993 -0.090859499999999996 -0.18245500000000001
+-0.086995100000000006 -0.098003199999999999 -0.17360800000000001
+-0.088834700000000003 -0.113272 -0.16358300000000001
+-0.100216 -0.115422 -0.150258
+-0.116397 -0.113163 -0.131517
+-0.109588 -0.13589899999999999 -0.107769
+-0.113596 -0.103854 -0.14671100000000001
+-0.11530899999999999 -0.089547399999999999 -0.15719
+-0.129883 -0.073039800000000002 -0.149787
+-0.144897 -0.054548600000000003 -0.136877
+-0.14124300000000001 -0.075265499999999999 -0.131382
+-0.14052899999999999 -0.092973399999999998 -0.117774
+-0.12981899999999999 -0.11432199999999999 -0.11049200000000001
+-0.15495700000000001 -0.082057199999999997 -0.10525
+-0.17319000000000001 -0.066030400000000003 -0.091166200000000003
+-0.20571600000000001 -0.054378099999999999 -0.068628900000000007
+-0.17418700000000001 -0.032837499999999999 -0.103341
+-0.16337199999999999 -0.091157199999999994 -0.086861800000000003
+-0.15534100000000001 -0.116364 -0.074342400000000003
+-0.198099 -0.024419300000000001 -0.081596699999999994
+-0.20949599999999999 0.00564775 -0.0793296
+-0.104811 -0.15370700000000001 -0.083402500000000004
+-0.19336400000000001 0.0274989 -0.090507299999999999
+-0.19509799999999999 0.061398099999999997 -0.085237800000000002
+-0.17486599999999999 0.090639300000000006 -0.090710399999999997
+-0.18304200000000001 0.115353 -0.070731500000000003
+-0.209624 0.117075 -0.056809100000000001
+-0.23627999999999999 0.097237400000000002 -0.060618600000000002
+-0.27478000000000002 0.110265 -0.049717200000000003
+-0.22836699999999999 0.066876900000000003 -0.069848400000000005
+-0.251112 0.040523000000000003 -0.068888000000000005
+-0.21660799999999999 0.13717099999999999 -0.0376511
+-0.227411 0.154531 -0.0055548400000000001
+-0.19477700000000001 0.158609 -0.019337799999999999
+-0.164962 0.178815 -0.00017977800000000001
+-0.231456 0.15257499999999999 0.032368899999999999
+-0.199826 0.160548 0.057260499999999999
+-0.23696500000000001 0.12865599999999999 0.063383300000000004
+-0.17122999999999999 0.18273300000000001 0.040482600000000001
+-0.12856400000000001 0.19044800000000001 -0.0083741900000000001
+-0.099112900000000004 0.20863000000000001 0.0142043
+-0.16852400000000001 0.16987099999999999 0.075512599999999999
+-0.16358800000000001 0.13642799999999999 0.107596
+-0.14139199999999999 0.192471 0.065252599999999994
+-0.062453300000000003 0.21782499999999999 0.0084292499999999992
+-0.073927000000000007 0.22186600000000001 0.042957299999999997
+-0.049288400000000003 0.227715 0.078474699999999994
+-0.26221100000000003 0.14244299999999999 -0.0010980600000000001
+-0.26684400000000003 0.107888 0.061363800000000003
+-0.28881000000000001 0.082738199999999998 0.070589100000000002
+-0.28231400000000001 0.057168400000000001 0.096185699999999999
+-0.24498400000000001 0.055590199999999999 0.115396
+-0.28409000000000001 0.023000400000000001 0.120573
+-0.20538999999999999 0.04444 0.13869000000000001
+-0.21665699999999999 0.091373499999999996 0.10624599999999999
+-0.321301 -0.0037008000000000002 0.113436
+-0.26943400000000001 -0.018037999999999998 0.13242899999999999
+-0.227907 -0.034297800000000003 0.138796
+-0.31689800000000001 0.062854800000000002 0.071180499999999994
+-0.29585699999999998 -0.040717299999999998 0.108514
+-0.33918799999999999 0.0416768 0.077048000000000005
+-0.36685699999999999 0.020597299999999999 0.076817499999999997
+-0.39415299999999998 -0.0072429 0.068874900000000003
+-0.33737699999999998 0.017679199999999999 0.097116400000000005
+-0.37254799999999999 -0.040200699999999999 0.067309599999999997
+-0.38872499999999999 0.029425699999999999 0.053718700000000001
+-0.40494400000000003 0.040333500000000001 0.027247299999999999
+-0.42627300000000001 0.044979100000000001 -0.00258157
+-0.43641099999999999 0.052614599999999997 -0.038485699999999998
+-0.47139199999999998 0.0283371 -0.064143099999999995
+-0.43483100000000002 0.047724599999999999 -0.080985600000000005
+-0.44171700000000003 0.016121 0.016048699999999999
+-0.37862499999999999 0.056637100000000003 0.031346300000000001
+-0.34679900000000002 0.075044299999999994 0.0354924
+-0.367064 0.078526899999999997 0.0104993
+-0.33643499999999998 0.099260799999999996 0.0095090000000000001
+-0.43493999999999999 -0.016029499999999999 0.033262399999999998
+-0.38249 0.080793100000000007 -0.015004099999999999
+-0.38071899999999997 0.086307900000000007 -0.047486300000000002
+-0.41036 0.065654000000000004 -0.023395300000000001
+-0.377973 0.063850699999999996 -0.078274999999999997
+-0.34126400000000001 0.047466899999999999 -0.078586799999999998
+-0.33605200000000002 -0.000329239 -0.079713599999999996
+-0.34850100000000001 0.092618900000000004 -0.0565738
+-0.32839699999999999 0.11079799999999999 -0.034086699999999998
+-0.37736700000000001 -0.019203899999999999 -0.089504
+-0.421433 -0.030545300000000001 -0.098560499999999995
+-0.11468200000000001 0.20774200000000001 0.052704500000000001
+-0.116467 0.19922699999999999 0.082537600000000003
+-0.093815300000000004 0.20181099999999999 0.100205
+-0.063842700000000002 0.20569899999999999 0.114819
+-0.037225300000000003 0.22053200000000001 0.106709
+-0.0071348599999999998 0.21589 0.120472
+0.00412912 0.19402800000000001 0.140734
+0.028646100000000001 0.17255899999999999 0.15181600000000001
+0.036943499999999997 0.13911499999999999 0.16606299999999999
+0.075980900000000004 0.12629000000000001 0.16377700000000001
+0.11276600000000001 0.099520600000000001 0.164051
+0.051920099999999997 0.191606 0.135347
+0.0585701 0.215447 0.107917
+0.091660099999999994 0.21785099999999999 0.086749300000000001
+0.085837700000000003 0.18144099999999999 0.13312199999999999
+0.086650000000000005 0.22678200000000001 0.0485108
+0.037780099999999997 0.104182 0.17802599999999999
+0.0020230299999999999 0.078279199999999993 0.18759999999999999
+0.039565400000000001 0.065842899999999996 0.18756700000000001
+0.085516700000000001 0.034203600000000001 0.183976
+0.11866400000000001 -0.0089809 0.165937
+0.11601599999999999 0.16884399999999999 0.13156200000000001
+0.15592500000000001 0.155755 0.124628
+0.13918700000000001 0.18004400000000001 0.113112
+0.13562199999999999 0.205066 0.082698999999999995
+0.16142599999999999 0.20608199999999999 0.053754700000000002
+0.19921900000000001 0.191721 0.051740500000000002
+0.029606 0.22756000000000001 0.094773499999999997
+0.15004899999999999 0.10953400000000001 0.14941499999999999
+-0.0301278 0.10138900000000001 0.177374
+-0.027991700000000001 0.137845 0.16642799999999999
+0.18098900000000001 0.089541700000000002 0.14410999999999999
+0.20983399999999999 0.100628 0.127196
+0.15212899999999999 0.075748399999999994 0.15832099999999999
+0.22376599999999999 0.12330099999999999 0.109789
+0.200021 0.14199400000000001 0.11224199999999999
+0.19623099999999999 0.16669700000000001 0.094654299999999997
+0.21687300000000001 0.17302999999999999 0.070927400000000002
+0.247026 0.154665 0.068370799999999995
+0.27526200000000001 0.14904400000000001 0.042698199999999999
+0.24914900000000001 0.11998 0.094881999999999994
+0.26560800000000001 0.090727199999999994 0.087311899999999998
+0.24728900000000001 0.063513299999999995 0.099306500000000006
+0.223522 0.069351300000000005 0.123084
+0.21421599999999999 0.037139600000000002 0.116998
+0.187747 0.024219999999999998 0.13308900000000001
+0.15481900000000001 -0.0011934199999999999 0.14399899999999999
+0.168326 0.043484700000000001 0.15293599999999999
+0.170987 0.169879 0.10595300000000001
+-0.17428199999999999 0.0232202 -0.10807
+-0.28459699999999999 -0.073205500000000007 0.093172599999999994
+-0.25108000000000003 -0.092020000000000005 0.097697999999999993
+-0.222464 -0.075360899999999995 0.11957
+-0.21421299999999999 -0.11310099999999999 0.103683
+-0.30290299999999998 -0.094511600000000001 0.068640199999999998
+-0.19542799999999999 -0.055683900000000001 0.139432
+-0.156829 -0.070041800000000001 0.14629800000000001
+-0.097674800000000006 -0.065800800000000007 0.166048
+-0.14442099999999999 -0.115749 0.13261899999999999
+-0.29611999999999999 -0.12327299999999999 0.051463200000000001
+-0.266231 -0.14585600000000001 0.054020600000000002
+0.048883099999999999 0.029467400000000001 0.19211300000000001
+0.023935999999999999 -0.014548200000000001 0.19886300000000001
+0.071751300000000004 -0.0041070400000000002 0.187615
+-0.028724099999999999 -0.0015641400000000001 0.19486700000000001
+-0.15239800000000001 -0.135765 -0.057188999999999997
+-0.100872 -0.086491100000000001 -0.17091700000000001
+-0.059381799999999998 0.18434700000000001 0.134966
+-0.035312000000000003 0.178094 0.14674100000000001
+-0.077654299999999996 0.16547300000000001 0.13800000000000001
+-0.105632 0.15925500000000001 0.12751299999999999
+-0.076679600000000001 0.13824400000000001 0.15093899999999999
+-0.096847799999999998 0.113293 0.151863
+-0.085348199999999999 0.085329100000000005 0.16491500000000001
+-0.091735300000000006 0.049600900000000003 0.17249200000000001
+-0.12656600000000001 0.10163 0.143708
+-0.13344700000000001 0.15071200000000001 0.11687500000000001
+-0.14485700000000001 0.12063500000000001 0.126973
+-0.173737 0.107421 0.11894299999999999
+-0.16139999999999999 -0.14654200000000001 -0.039030500000000003
+-0.18660299999999999 -0.146865 -0.028232699999999999
+-0.201541 -0.12522800000000001 -0.037903100000000002
+-0.193134 -0.10312399999999999 -0.0539853
+-0.21104899999999999 -0.080392099999999994 -0.056783599999999997
+-0.159914 -0.163462 -0.0232774
+-0.22752800000000001 -0.128247 -0.034006099999999997
+-0.23887900000000001 -0.14904500000000001 -0.025366300000000001
+-0.22639699999999999 -0.167794 -0.014013599999999999
+-0.20136899999999999 -0.16749700000000001 -0.0134536
+-0.26285700000000001 -0.15434 -0.025979700000000001
+-0.28056799999999998 -0.17109099999999999 -0.014478599999999999
+-0.28174399999999999 -0.13850299999999999 -0.036859900000000001
+-0.309473 -0.13516600000000001 -0.043453199999999997
+-0.34013399999999999 -0.14111399999999999 -0.0045747799999999996
+-0.269395 -0.17985400000000001 0.0148325
+-0.30532599999999999 -0.16272600000000001 0.0012573899999999999
+-0.31532900000000003 -0.14105400000000001 0.020993600000000001
+-0.328044 -0.15237100000000001 -0.028746500000000001
+-0.34048800000000001 -0.131656 -0.0493684
+-0.321044 -0.11310000000000001 -0.0550007
+-0.35669299999999998 -0.110151 -0.063365699999999997
+-0.36424000000000001 -0.079950300000000002 -0.075380900000000001
+-0.387596 -0.11068699999999999 -0.0644618
+-0.36255900000000002 -0.047636100000000001 -0.081756999999999996
+-0.39623700000000001 -0.11378099999999999 -0.035386099999999997
+-0.37159599999999998 -0.120753 -0.010980200000000001
+-0.36457600000000001 -0.10081 0.020393499999999998
+-0.41761900000000002 -0.091820899999999997 -0.025175300000000001
+-0.418352 -0.071948700000000004 0.00161618
+-0.39399699999999999 -0.071492600000000003 0.0251311
+-0.39477200000000001 -0.0475977 0.045948799999999998
+-0.40268599999999999 -0.089535299999999998 -0.078489799999999998
+-0.42083999999999999 -0.068101700000000001 -0.089703199999999997
+-0.44040000000000001 -0.050982699999999999 0.0012606200000000001
+-0.45965499999999998 -0.047228399999999997 -0.020399899999999999
+-0.460671 -0.063892299999999999 -0.050193000000000002
+-0.47519600000000001 -0.0484471 -0.077546199999999996
+-0.48947099999999999 -0.0080625899999999997 -0.077667100000000003
+-0.475138 -0.024776099999999999 -0.021313200000000001
+-0.48642400000000002 -0.0015352499999999999 -0.044028100000000001
+-0.46840300000000001 0.00156711 -0.0067831100000000002
+-0.24207400000000001 -0.191329 0.0083080800000000007
+-0.42978300000000003 -0.087785299999999997 -0.073571899999999996
+-0.43628 -0.085082699999999997 -0.046137299999999999
+-0.19173100000000001 -0.18348100000000001 -0.00157165
+-0.201183 -0.19839000000000001 0.012441499999999999
+-0.20744899999999999 -0.198571 0.049506700000000001
+-0.17322299999999999 -0.19345499999999999 0.0066396600000000004
+-0.14982100000000001 -0.209649 0.0233901
+-0.13411500000000001 -0.22322900000000001 0.055024000000000003
+-0.13610800000000001 -0.19972400000000001 0.094550800000000004
+-0.15280099999999999 -0.18782299999999999 -0.00205719
+-0.18837999999999999 -0.17875099999999999 0.079835900000000001
+-0.099533499999999997 -0.22301099999999999 0.051333400000000001
+-0.114673 -0.17524799999999999 0.119217
+-0.44047700000000001 -0.049986900000000001 -0.095649999999999999
+-0.451957 -0.020404599999999998 -0.101857
+-0.073532500000000001 0.21576699999999999 0.093454300000000004
+-0.23469300000000001 0.0151621 -0.0705707
+-0.23231299999999999 -0.0142483 -0.068253999999999995
+-0.256967 -0.0290178 -0.062962299999999999
+-0.29299599999999998 -0.016672800000000002 -0.068659999999999999
+-0.20158100000000001 -0.20730599999999999 0.031393400000000002
+-0.22372500000000001 -0.19963600000000001 0.022959199999999999
+-0.183119 -0.21274199999999999 0.045317000000000003
+-0.18884600000000001 -0.19935 0.063469300000000006
+-0.160028 -0.21804699999999999 0.043169300000000001
+-0.20458399999999999 -0.184112 0.065892099999999995
+-0.172989 -0.19443199999999999 0.077981300000000003
+-0.16651299999999999 -0.17455000000000001 0.094699199999999997
+-0.18471000000000001 -0.14161699999999999 0.103157
+-0.15323700000000001 -0.20608099999999999 0.080472000000000002
+-0.13192000000000001 -0.21789500000000001 0.078765000000000002
+-0.113273 -0.211482 0.095236799999999996
+-0.087693900000000005 -0.20511699999999999 0.110143
+-0.061790900000000003 -0.21538099999999999 0.094153600000000004
+-0.046707100000000001 -0.213033 0.066072099999999995
+-0.027199999999999998 -0.201713 0.039605500000000002
+-0.0428995 -0.20211200000000001 0.0068477499999999997
+-0.050040500000000002 -0.19896 0.11833100000000001
+-0.0092382799999999998 -0.17464099999999999 0.123557
+-0.044816399999999999 -0.17225599999999999 0.14143500000000001
+0.016263400000000001 -0.16553399999999999 0.084978799999999993
+0.048965500000000002 -0.13069500000000001 0.084936899999999996
+0.032825699999999999 -0.14700299999999999 0.043867400000000001
+-0.064879099999999995 -0.14555499999999999 0.150587
+-0.0537054 -0.100021 0.16964099999999999
+-0.014756500000000001 -0.19434799999999999 0.067762799999999998
+-0.0079664100000000002 -0.18554999999999999 0.095633899999999994
+0.016580999999999999 -0.16354399999999999 0.0088471799999999996
+0.017707899999999999 -0.16073399999999999 -0.035924200000000003
+0.058777999999999997 -0.116217 0.12311800000000001
+0.077807600000000005 -0.081498000000000001 0.15444099999999999
+0.035232399999999997 -0.119032 0.15195600000000001
+0.0011408799999999999 -0.126919 0.16329099999999999
+-0.0183444 -0.100065 0.17824599999999999
+-0.0023533899999999999 -0.066350199999999998 0.19079199999999999
+0.034104099999999998 -0.066021999999999997 0.18746199999999999
+0.049249500000000002 -0.089232000000000006 0.16932700000000001
+-0.012811400000000001 -0.032974400000000001 0.194855
+-0.115415 -0.22503100000000001 0.066271099999999999
+-0.073077299999999998 -0.220943 0.060525799999999998
+-0.34335700000000002 -0.039519199999999997 0.085378099999999998
+-0.0113899 -0.18948999999999999 0.016989400000000002
+-0.118843 -0.19353100000000001 0.107193
+-0.13802300000000001 -0.17901800000000001 0.10664899999999999
+-0.100643 -0.19026000000000001 0.11654100000000001
+-0.094766100000000006 -0.17258599999999999 0.12842799999999999
+-0.111739 -0.149926 0.13206000000000001
+-0.076093300000000003 -0.16314200000000001 0.13944500000000001
+-0.234903 -0.085372000000000003 -0.049788699999999998
+-0.029894 0.2223 0.0050602800000000003
+-0.0077000000000000002 0.23047400000000001 0.027081999999999998
+-0.016640499999999999 0.23341400000000001 0.054480000000000001
+0.0101539 0.23432500000000001 0.053928900000000002
+0.037209399999999997 0.233262 0.0550673
+0.055545499999999998 0.23034199999999999 0.029847700000000001
+0.04283 0.22407199999999999 -0.00021755799999999999
+0.067147999999999999 0.21096999999999999 -0.036040700000000002
+0.011472 0.226074 0.0068730299999999996
+-0.055859100000000002 -0.10463 -0.18293400000000001
+-0.056593600000000001 -0.12168900000000001 -0.17641999999999999
+0.12175900000000001 -0.045740799999999998 -0.017806700000000002
+-0.053073700000000001 0.134352 0.160908
+0.123973 0.071887099999999995 0.168299
+0.0221523 0.23358599999999999 0.073706800000000003
+-0.0035341700000000001 -0.15252099999999999 0.14538100000000001
+-0.157503 -0.040724900000000001 -0.123137
+-0.40274399999999999 0.044257400000000002 -0.087763499999999994
+-0.40564099999999997 0.0079917900000000004 -0.096693600000000005
+-0.121506 -0.157661 -0.0590498
+-0.111905 -0.177728 -0.034197499999999999
+-0.077898800000000004 0.022964200000000001 0.18259
+-0.065333500000000003 -0.014212499999999999 0.187109
+-0.049683999999999999 0.018220799999999999 0.189225
+-0.0227026 0.040543200000000001 0.19126199999999999
+-0.10136000000000001 -0.0715673 -0.17546600000000001
+-0.30970999999999999 0.108636 0.023971200000000002
+0.28432800000000003 0.10201300000000001 0.065753500000000006
+0.30034 0.087615600000000002 0.035027700000000002
+0.31309300000000001 0.101645 0.0055769499999999998
+0.28173399999999998 0.059859000000000002 0.0301131
+0.31369000000000002 0.093917200000000006 -0.025519400000000001
+0.313301 0.115146 -0.051408700000000002
+0.30536099999999999 0.14809800000000001 -0.0427014
+0.26683099999999998 0.16988500000000001 -0.051427899999999999
+0.28652 0.12620000000000001 0.0529506
+0.30213800000000002 0.14977799999999999 -0.0123269
+0.29091 0.14998 0.016253500000000001
+0.26822000000000001 0.16663800000000001 0.021285399999999999
+0.247534 0.182808 -0.00123423
+0.21554499999999999 0.19503899999999999 0.0040064499999999999
+0.25356600000000001 0.16545299999999999 0.0442228
+0.28728500000000001 0.162995 -0.030038700000000002
+0.26483299999999999 0.17541200000000001 -0.0261528
+0.24618300000000001 0.18101 -0.0421722
+0.216699 0.18173800000000001 -0.060451600000000001
+0.191444 0.19706299999999999 -0.037413799999999997
+0.19761799999999999 0.159773 -0.081418099999999993
+0.16262599999999999 0.15104400000000001 -0.086919499999999997
+0.24641299999999999 0.171038 -0.064018500000000006
+0.262152 0.14856 -0.076847100000000002
+0.23690600000000001 0.117298 -0.083770399999999995
+0.26650800000000002 0.119186 -0.080167500000000003
+0.29137299999999999 0.107261 -0.070558999999999997
+0.26872699999999999 0.080423800000000004 -0.071397199999999994
+0.15784400000000001 0.20099 -0.041992700000000001
+0.127221 0.199769 -0.052480800000000001
+0.092187099999999994 0.19914399999999999 -0.058280699999999998
+0.16055800000000001 0.208675 -0.012496500000000001
+0.13623399999999999 0.216589 0.0128056
+0.106226 0.20925299999999999 -0.0354265
+0.10578899999999999 0.21884999999999999 -0.00422888
+0.098713899999999993 0.22445399999999999 0.022799799999999999
+0.31013200000000002 0.13089999999999999 0.0048277299999999997
+0.26433499999999999 0.048680800000000003 0.050985999999999997
+0.242897 0.032061600000000003 0.061379900000000001
+0.25531500000000001 0.050923099999999999 0.076559199999999994
+0.223445 0.0189006 0.070345400000000002
+0.19844600000000001 0.00226779 0.082946900000000004
+0.15964200000000001 -0.026474299999999999 0.077045500000000003
+0.18465899999999999 -0.0099512100000000003 0.058162800000000001
+0.29580800000000002 0.072896100000000005 -0.027399799999999998
+0.30413299999999999 0.081461199999999998 -0.0056807699999999999
+0.29666599999999999 0.074601799999999996 0.015656699999999999
+0.27832200000000001 0.056460799999999998 -0.024005499999999999
+0.262042 0.040640099999999998 0.0022073399999999999
+0.25966899999999998 0.041196400000000001 -0.0218671
+0.24385200000000001 0.034963000000000001 -0.042100100000000001
+0.21526600000000001 0.0162635 -0.0527158
+0.24581500000000001 0.051806400000000002 -0.067567100000000005
+0.21494199999999999 0.046667 -0.086470699999999998
+0.28263199999999999 0.0654666 -0.0447211
+0.26852300000000001 0.063384200000000002 -0.059689699999999998
+0.28129599999999999 0.076098200000000005 -0.059218699999999999
+0.28325400000000001 0.091085200000000005 -0.069259600000000004
+0.26701999999999998 0.099399399999999999 -0.0774233
+0.25233899999999998 0.086640400000000006 -0.080310999999999994
+0.235073 0.096938499999999997 -0.084024699999999994
+0.21082600000000001 0.109417 -0.086493700000000007
+0.214308 0.085839799999999994 -0.088021299999999997
+0.21404699999999999 0.13722599999999999 -0.084492700000000004
+0.25352000000000002 0.068597500000000006 -0.073375599999999999
+0.25236900000000001 0.10843899999999999 -0.082266300000000001
+0.205928 0.065937200000000001 -0.091677099999999997
+0.18831300000000001 0.043192899999999999 -0.099293300000000001
+0.17059099999999999 0.025554299999999999 -0.111447
+0.19422700000000001 -0.0034471300000000001 0.034471599999999998
+0.186031 -0.0064174100000000001 -0.000610057
+0.22162100000000001 0.013203299999999999 0.020514500000000001
+0.158466 -0.0217212 -0.011189899999999999
+0.203821 0.0053486200000000001 -0.024697799999999999
+-0.087367299999999995 -0.213787 0.025887500000000001
+-0.098808800000000002 -0.19930899999999999 -0.0031412100000000002
+-0.025040699999999999 0.23188400000000001 0.077572000000000002
+-0.013288700000000001 0.22770199999999999 0.099276100000000006
+0.0090716900000000003 0.22456699999999999 0.10582
+-0.038122299999999998 0.23139299999999999 0.0596562
+-0.059791299999999999 0.22719600000000001 0.059059899999999999
+-0.068061999999999998 0.222998 0.076201599999999994
+-0.076820399999999997 0.22189500000000001 0.060625499999999999
+-0.094470600000000002 0.215866 0.048674000000000002
+-0.107247 0.20984700000000001 0.033351199999999998
+-0.119042 0.202294 0.0174804
+-0.110264 0.199934 -0.0020657100000000001
+-0.10877199999999999 0.189998 -0.028283200000000001
+-0.0885764 0.21595300000000001 0.029554899999999999
+-0.13533700000000001 0.19406300000000001 0.0098582900000000005
+-0.15611700000000001 0.18779299999999999 0.020395300000000002
+-0.17702100000000001 0.179366 0.018911299999999999
+-0.29922399999999999 0.119756 -0.034104299999999997
+-0.28539500000000001 0.131717 -0.0127806
+-0.281551 0.13092799999999999 0.0140859
+-0.28594700000000001 0.115513 0.0369868
+-0.15040500000000001 -0.21848200000000001 0.065269999999999995
+0.015202800000000001 -0.043048200000000002 0.19713900000000001
+0.040283399999999997 -0.0349374 0.19456100000000001
+-0.085107299999999997 -0.18548999999999999 -0.038290499999999998
+-0.067330799999999996 -0.18115400000000001 -0.059406100000000003
+-0.057076200000000001 -0.172068 -0.089080699999999999
+-0.028735199999999999 -0.17444999999999999 -0.090752700000000006
+-0.015901700000000001 -0.17755799999999999 -0.069441000000000003
+-0.020461799999999999 -0.18479499999999999 -0.041148900000000002
+0.0087381100000000003 -0.16711100000000001 -0.061560900000000002
+-0.038143299999999998 -0.19418299999999999 -0.020434899999999999
+-0.051034000000000003 -0.189416 -0.041780200000000003
+-0.35121999999999998 -0.059801199999999999 0.066219700000000006
+-0.33698 -0.088017700000000004 0.052541499999999998
+-0.41443799999999997 -0.029614000000000001 0.043753599999999997
+-0.39668399999999998 -0.0282297 0.0588966
+-0.41073399999999999 -0.012068499999999999 0.055849500000000003
+-0.40436 0.0070651100000000003 0.058606800000000001
+-0.38616099999999998 0.013236899999999999 0.069292699999999999
+-0.37776100000000001 -0.0020853899999999999 0.081099299999999999
+-0.37323600000000001 -0.022365300000000001 0.078308199999999994
+-0.35734100000000002 -0.027750899999999998 0.085550200000000007
+-0.337893 -0.019052599999999999 0.10051599999999999
+-0.32018999999999997 -0.038641000000000002 0.098219699999999993
+-0.31034699999999998 -0.0235748 0.112107
+-0.29525499999999999 -0.0032073000000000002 0.12631500000000001
+-0.290437 -0.023630100000000001 0.120756
+-0.27395000000000003 -0.040667200000000001 0.11799
+-0.25102000000000002 -0.037702899999999998 0.128663
+-0.22742100000000001 -0.066667400000000002 -0.056902500000000002
+-0.226022 -0.044700299999999998 -0.063681199999999993
+-0.210588 -0.035672700000000002 -0.071603100000000003
+-0.21408099999999999 -0.016863099999999999 -0.0739229
+-0.19480700000000001 -0.043006700000000002 -0.079277600000000004
+-0.188472 -0.062925999999999996 -0.077075199999999996
+-0.22180900000000001 -0.0025995699999999998 -0.072599800000000006
+-0.22142000000000001 0.0115402 -0.074242799999999998
+-0.20996600000000001 0.0222645 -0.079748100000000002
+-0.20374200000000001 0.035738699999999998 -0.083051
+-0.208844 0.052203399999999997 -0.078747399999999995
+-0.19082199999999999 0.045095099999999999 -0.090910699999999997
+-0.177425 0.061166100000000001 -0.099123900000000001
+-0.20915400000000001 0.071997000000000005 -0.075677300000000003
+-0.16361000000000001 0.078118900000000005 -0.106493
+-0.19264300000000001 0.080421699999999999 -0.081731100000000001
+-0.188553 0.098312899999999995 -0.076963100000000007
+-0.15820699999999999 0.098198599999999997 -0.101697
+-0.14588999999999999 0.087709400000000007 -0.11913600000000001
+-0.14155999999999999 0.107293 -0.111634
+-0.12382700000000001 0.117911 -0.119323
+-0.108028 0.110926 -0.135713
+-0.112326 0.089653399999999994 -0.14544399999999999
+-0.091987899999999997 0.0895894 -0.15718299999999999
+-0.35440700000000003 -0.140129 -0.027650899999999999
+0.0608099 -0.102747 0.14763699999999999
+0.079597399999999999 -0.093919199999999994 0.13094600000000001
+0.10627300000000001 -0.066251099999999993 0.13344900000000001
+0.114315 -0.063354800000000003 0.0946988
+0.091175599999999996 -0.080818000000000001 0.066456399999999999
+0.094593399999999994 -0.069621299999999997 0.0242046
+0.094465999999999994 -0.049551900000000003 0.16490099999999999
+0.103223 -0.059938999999999999 -0.015014100000000001
+0.107659 -0.055741100000000002 0.0087524100000000004
+0.112343 -0.054881199999999998 0.032969499999999999
+0.13300999999999999 -0.039976699999999997 0.038853800000000001
+0.14019400000000001 -0.039123900000000003 0.064310999999999993
+0.14047399999999999 -0.0411727 0.093419500000000003
+0.15845000000000001 -0.022454399999999999 0.11312999999999999
+0.064370499999999997 -0.103524 -0.016815699999999999
+0.058664300000000003 -0.110442 0.0129414
+0.075187900000000002 -0.090970300000000004 0.026571000000000001
+0.079995499999999997 -0.083989900000000006 -0.018291100000000001
+0.080597699999999994 -0.082260700000000006 0.0052742099999999997
+-0.199652 -0.070645200000000005 -0.065935400000000005
+-0.19524 -0.087056499999999995 -0.061056899999999997
+-0.36365199999999998 0.0067447899999999996 0.087702000000000002
+-0.34859200000000001 0.0017769800000000001 0.099163799999999996
+-0.352099 0.020551799999999999 0.087143799999999993
+-0.33396100000000001 0.0044392199999999998 0.106489
+-0.31508599999999998 0.018744199999999999 0.10863100000000001
+-0.00316359 0.048916899999999999 -0.19022700000000001
+0.0082884800000000008 0.0264747 -0.19409199999999999
+-0.0089128200000000001 0.0052444600000000003 -0.19856599999999999
+-0.00083717899999999996 -0.0247146 -0.19836899999999999
+-0.024708000000000001 -0.19103500000000001 0.113136
+-0.028143999999999999 -0.180474 0.13120999999999999
+-0.0241438 -0.16289600000000001 0.14561299999999999
+-0.045124499999999998 -0.186443 0.130327
+-0.060770499999999998 -0.19034000000000001 0.12726899999999999
+-0.068835800000000003 -0.203401 0.115117
+-0.062611 -0.17513000000000001 0.13708400000000001
+-0.0574099 -0.160773 0.145842
+-0.047680199999999999 -0.14828 0.15374499999999999
+-0.04727 -0.124358 0.16361999999999999
+-0.074239899999999998 -0.11637599999999999 0.158165
+-0.103323 -0.097917000000000004 0.15429200000000001
+-0.022877999999999999 -0.122805 0.16883200000000001
+0.027446000000000002 0.018355300000000001 -0.190554
+0.047287900000000001 0.024984099999999999 -0.18012
+0.047031000000000003 0.048085200000000002 -0.16816800000000001
+0.0245826 0.063830300000000006 -0.173037
+0.068977899999999995 0.0532592 -0.15346899999999999
+0.039009000000000002 0.00069482799999999998 -0.19258600000000001
+0.039916899999999998 -0.022020700000000001 -0.193298
+0.059217499999999999 -0.030858199999999999 -0.18629899999999999
+0.088181899999999994 -0.0293513 -0.172655
+0.11654 -0.023742200000000001 -0.15226000000000001
+0.045732099999999998 -0.0484051 -0.189
+0.028347299999999999 -0.061920599999999999 -0.19234399999999999
+0.0088470900000000002 -0.0532585 -0.19677700000000001
+-0.0167742 -0.045966300000000002 -0.19631599999999999
+0.014673500000000001 -0.0790357 -0.193471
+0.0207706 -0.101966 -0.18762000000000001
+0.044992699999999997 -0.105242 -0.174844
+0.065162300000000006 0.0109671 -0.17912700000000001
+0.078185400000000002 -0.0078636299999999999 -0.178011
+0.081928000000000001 0.0167058 -0.16800699999999999
+0.083948599999999998 0.0345127 -0.157031
+0.012745299999999999 0.0857324 -0.16769300000000001
+0.029395899999999999 0.0958981 -0.15381300000000001
+0.030408000000000001 0.12443 -0.136266
+-0.00876124 0.088804599999999997 -0.17280999999999999
+-0.020671499999999999 0.075122700000000001 -0.18135399999999999
+0.053330799999999998 0.13374900000000001 -0.12095
+0.0571218 0.15846299999999999 -0.10569199999999999
+0.082139900000000002 0.17514399999999999 -0.088263599999999998
+0.033958700000000001 0.17282600000000001 -0.098476099999999997
+0.025033799999999998 0.190467 -0.076702599999999996
+0.036119800000000001 0.205898 -0.048656199999999997
+0.13370000000000001 0.159717 -0.088580800000000001
+0.15319199999999999 0.17372499999999999 -0.079540899999999998
+0.15035699999999999 0.19109300000000001 -0.062839400000000004
+0.000183964 -0.174592 -0.0415267
+0.0062644399999999996 -0.172179 -0.019166900000000001
+0.022800399999999998 -0.15598799999999999 -0.0130102
+0.035125099999999999 -0.14147299999999999 0.0086342199999999997
+0.036023699999999999 -0.140713 -0.031627599999999999
+0.0020287399999999998 -0.17727200000000001 0.0012006499999999999
+0.0054414499999999996 -0.176177 0.022733900000000001
+0.017425 -0.16474800000000001 0.039874399999999997
+0.023538799999999999 -0.15918599999999999 0.062702800000000003
+0.053802200000000001 -0.119801 -0.039014300000000002
+0.0025606000000000001 -0.095045400000000002 -0.19255700000000001
+-0.018823800000000002 -0.093355199999999999 -0.19203700000000001
+-0.030994399999999998 -0.073046200000000006 -0.192797
+-0.028106800000000001 -0.109252 -0.187754
+-0.075042999999999999 -0.21301 0.103434
+-0.089424799999999999 -0.215145 0.098752900000000005
+-0.101426 -0.21724599999999999 0.093419299999999997
+-0.112609 -0.221304 0.082051200000000005
+-0.102007 -0.206618 0.104586
+-0.33370899999999998 0.073468800000000001 -0.070933899999999994
+-0.30549999999999999 0.065284300000000003 -0.070127700000000001
+-0.35792200000000002 0.075518000000000002 -0.071040400000000004
+-0.35893599999999998 0.057208000000000002 -0.079511700000000005
+0.0591707 0.23129 0.051818200000000002
+0.20324800000000001 0.015473300000000001 0.10573299999999999
+-0.47254600000000002 0.0131591 -0.028855599999999999
+-0.46147100000000002 0.032282100000000001 -0.0411949
+-0.45131100000000002 0.044170399999999999 -0.061588700000000003
+-0.45650800000000002 0.033673599999999998 -0.084952299999999994
+-0.47535300000000003 0.013651999999999999 -0.087154400000000007
+-0.48302 -0.028537799999999999 -0.091103900000000002
+-0.48918099999999998 -0.030111300000000001 -0.067796700000000001
+-0.46326200000000001 -0.040469699999999997 -0.096372600000000003
+-0.43277100000000002 0.057318099999999997 -0.060452199999999998
+-0.41554999999999997 0.060145200000000003 -0.076112100000000002
+-0.39707500000000001 0.067692199999999994 -0.072884299999999999
+-0.38215399999999999 0.078306299999999995 -0.065485000000000002
+-0.366919 0.087188100000000004 -0.059812900000000002
+-0.364506 0.094590199999999999 -0.042663399999999997
+-0.34636699999999998 0.102863 -0.039051700000000002
+-0.34191100000000002 0.10630000000000001 -0.019330099999999999
+-0.35290700000000003 0.095371700000000004 -0.0041110699999999997
+-0.33055499999999999 0.100814 -0.051665099999999999
+-0.31083 0.10825799999999999 -0.047939799999999998
+-0.29252899999999998 0.097151699999999994 -0.056889200000000001
+-0.376027 0.088421 -0.0301143
+-0.39543099999999998 0.078534000000000007 -0.033118799999999997
+-0.27186199999999999 0.091324900000000001 -0.059845500000000003
+-0.28328300000000001 0.071777800000000003 -0.066117499999999996
+-0.42215200000000003 0.063294699999999995 -0.045624499999999998
+-0.33332299999999998 0.088285299999999997 -0.062727900000000003
+-0.437052 0.030601900000000001 -0.094291100000000003
+-0.42083199999999998 0.0396817 -0.090828099999999995
+-0.42020800000000003 0.0209628 -0.096290500000000001
+-0.42460300000000001 0.00147628 -0.100178
+-0.40923799999999999 -0.01306 -0.097180500000000003
+-0.397117 -0.034928899999999999 -0.092550599999999997
+-0.40230700000000003 0.0263118 -0.0936004
+-0.38781900000000002 0.0136986 -0.092105500000000007
+-0.36457600000000001 0.0032477199999999999 -0.087299699999999994
+-0.39181700000000003 -0.0052851299999999999 -0.093420699999999995
+-0.38889899999999999 0.0353587 -0.089889999999999998
+-0.373861 0.0459462 -0.084154599999999996
+-0.35658099999999998 0.035406100000000003 -0.084031900000000007
+-0.33561600000000003 0.023919099999999999 -0.079937999999999995
+-0.31268600000000002 0.0114246 -0.074903499999999998
+-0.28382099999999999 0.014154 -0.069988700000000001
+-0.38866299999999998 0.050131700000000001 -0.085424600000000003
+-0.40062500000000001 0.057077099999999999 -0.081787200000000004
+-0.277312 0.039557700000000001 -0.069580100000000006
+-0.26359500000000002 0.0224806 -0.068115200000000001
+-0.41149400000000003 0.052111699999999997 -0.084395800000000007
+-0.423315 0.0520061 -0.0810506
+-0.42659999999999998 0.058336699999999998 -0.071036699999999994
+-0.42052800000000001 0.064336500000000005 -0.059779400000000003
+-0.43721500000000002 0.052997900000000001 -0.070598900000000006
+-0.44746599999999997 0.044273100000000003 -0.075328199999999998
+-0.45922400000000002 0.037086899999999999 -0.071263999999999994
+-0.46836100000000003 0.028244200000000001 -0.078411300000000003
+-0.47800900000000002 0.0179968 -0.072937600000000005
+-0.47995100000000002 0.0140616 -0.053883899999999998
+-0.46308300000000002 0.021495199999999999 -0.090712399999999999
+-0.46345700000000001 0.0029889000000000001 -0.101217
+-0.48569099999999998 0.0064728099999999998 -0.078871399999999994
+-0.49019400000000002 0.0012374300000000001 -0.061941700000000002
+-0.48902699999999999 -0.016199399999999999 -0.055318100000000002
+-0.48362300000000003 -0.018897500000000001 -0.037591100000000002
+-0.47831899999999999 -0.0052931799999999998 -0.024031199999999999
+-0.48393199999999997 -0.0027069199999999998 -0.089892299999999994
+-0.48110700000000001 -0.015395499999999999 -0.093549599999999997
+-0.46911000000000003 -0.025215899999999999 -0.099654099999999995
+-0.48830099999999999 -0.019441 -0.083633799999999994
+-0.490506 -0.018627399999999999 -0.071670600000000001
+-0.48746800000000001 -0.0311436 -0.079934599999999995
+-0.47920400000000002 -0.039190599999999999 -0.085295200000000002
+-0.48227300000000001 -0.039566999999999998 -0.072251599999999999
+-0.47358800000000001 -0.0502137 -0.057610000000000001
+-0.48236200000000001 -0.037578300000000002 -0.058993499999999997
+-0.46798899999999999 -0.049158199999999999 -0.038457999999999999
+-0.46488099999999999 -0.063439499999999996 -0.069595699999999996
+-0.45116400000000001 -0.075200100000000006 -0.062683000000000003
+-0.461144 -0.0570308 -0.087161699999999995
+-0.47021000000000002 -0.046769900000000003 -0.088560600000000003
+-0.45486799999999999 -0.061269999999999998 -0.032468400000000001
+-0.44309900000000002 0.039289999999999999 -0.086606500000000003
+-0.47388599999999997 -0.034972200000000002 -0.094978999999999994
+-0.487452 -0.028162 -0.052937600000000001
+-0.46882000000000001 -0.0106219 -0.099822900000000006
+-0.44827400000000001 0.030902499999999999 -0.092328599999999997
+-0.443498 0.021339299999999999 -0.096643099999999996
+-0.45456000000000002 0.014838799999999999 -0.097697800000000001
+-0.43189899999999998 0.016569400000000001 -0.098347500000000004
+-0.47469600000000001 0.00041085399999999999 -0.097229200000000002
+-0.46948499999999999 0.0096302100000000002 -0.095245999999999997
+0.29695500000000002 0.106656 0.047249800000000002
+0.291078 0.088316699999999998 0.052780599999999997
+0.28881299999999999 0.0737593 0.040867199999999999
+0.27787699999999999 0.061405300000000003 0.047343200000000002
+0.27035100000000001 0.059129899999999999 0.060632100000000001
+0.26618700000000001 0.069788799999999998 0.0793687
+0.305647 0.105588 0.029909700000000001
+0.31212600000000001 0.116415 0.0165124
+0.31736900000000001 0.11674900000000001 0.0010400100000000001
+0.31616300000000003 0.104461 -0.011578
+0.31821100000000002 0.110014 -0.027318200000000001
+0.313357 0.10102700000000001 -0.0413356
+0.31855899999999998 0.12438 -0.037421099999999999
+0.31109599999999998 0.133216 -0.0514791
+0.303008 0.123002 -0.066372100000000003
+0.31335200000000002 0.13942299999999999 -0.0292404
+0.30516799999999999 0.10059800000000001 -0.056823199999999997
+0.30038300000000001 0.145261 -0.060021699999999997
+0.28053099999999997 0.15626000000000001 -0.063975099999999993
+0.29135299999999997 0.15679599999999999 -0.049708000000000002
+0.30542599999999998 0.087295499999999998 -0.037831700000000003
+0.311948 0.13780899999999999 -0.0138391
+0.307419 0.14267299999999999 -0.00177921
+0.29540899999999998 0.151976 0.0015104700000000001
+0.30266399999999999 0.141484 0.0124035
+0.29664000000000001 0.13967299999999999 0.027252499999999999
+0.30335299999999998 0.15110000000000001 -0.0276759
+-0.035541799999999998 -0.15595000000000001 0.15140100000000001
+0.24587600000000001 0.184257 -0.021774000000000002
+0.23197400000000001 0.18943399999999999 -0.0098991200000000008
+0.21463399999999999 0.19536999999999999 -0.0149903
+0.197855 0.20113600000000001 -0.0045619299999999996
+0.150815 0.022534999999999999 0.157779
+0.12464600000000001 0.019225099999999998 0.17016899999999999
+0.098797800000000005 0.0082900600000000001 0.17925199999999999
+0.093536900000000006 -0.0207146 0.176347
+0.13756699999999999 0.00340953 0.15896099999999999
+0.13530900000000001 -0.0228522 0.14630199999999999
+0.12446500000000001 -0.043863399999999997 -0.13242399999999999
+-0.33631800000000001 -0.12790599999999999 0.016509599999999999
+-0.33239600000000002 -0.0089667299999999991 0.107708
+-0.32433400000000001 -0.021719499999999999 0.10666200000000001
+-0.34302700000000003 -0.0083719499999999995 0.10223
+-0.34997899999999998 -0.017221899999999998 0.094727500000000006
+-0.35156300000000001 -0.0075691999999999999 0.097001199999999996
+-0.35756700000000002 0.00049845499999999997 0.093700699999999998
+-0.352358 0.010696199999999999 0.093009700000000001
+-0.36506100000000002 -0.0024184499999999999 0.0889268
+-0.37137799999999999 -0.011685299999999999 0.084558999999999995
+-0.38150099999999998 -0.013303799999999999 0.077807600000000005
+-0.38503999999999999 -0.025347499999999999 0.068713800000000005
+-0.26522099999999998 0.13580400000000001 -0.022798200000000001
+-0.24834700000000001 0.143818 -0.015516200000000001
+-0.22962399999999999 0.14505699999999999 -0.023862000000000001
+0.083714700000000003 0.203074 0.114313
+0.191547 0.00061397600000000002 -0.058867000000000003
+0.17957200000000001 -0.0045589100000000002 -0.081695000000000004
+-0.0179285 0.19112499999999999 0.14182500000000001
+0.074671000000000001 0.125449 -0.115605
+0.100378 0.12325700000000001 -0.105614
+0.109501 0.14819099999999999 -0.096700099999999997
+0.0861875 0.10497099999999999 -0.118148
+0.120268 0.113011 -0.101312
+0.23039999999999999 0.033054 -0.062778700000000007
+-0.017502299999999998 0.205236 0.13064000000000001
+-0.024694500000000001 0.215588 0.118203
+-0.040251299999999997 0.210836 0.119478
+-0.052051699999999999 0.19969200000000001 0.12662899999999999
+-0.052046700000000001 0.21516299999999999 0.109544
+-0.049136399999999997 0.22328100000000001 0.094889500000000002
+-0.033057400000000001 0.227876 0.091985300000000006
+-0.00155887 0.20564099999999999 0.13136999999999999
+0.0094258099999999997 0.214006 0.121922
+0.0140483 0.20339299999999999 0.132712
+0.023025400000000001 0.188966 0.14258000000000001
+0.028378299999999999 0.19961100000000001 0.13371
+0.044337500000000002 0.205985 0.12504499999999999
+0.064579300000000006 0.202014 0.122581
+0.038270699999999998 0.218421 0.11045199999999999
+0.037754500000000003 0.18965699999999999 0.13985900000000001
+0.049509699999999997 0.17627999999999999 0.14607300000000001
+0.067353200000000002 0.18032699999999999 0.13928299999999999
+0.080270800000000003 0.16595799999999999 0.14452999999999999
+0.076589400000000002 0.14677899999999999 0.15516099999999999
+0.0990867 0.13400400000000001 0.155061
+0.043261599999999997 0.16105800000000001 0.15548500000000001
+0.055619300000000003 0.14540500000000001 0.160331
+0.053991999999999998 0.124489 0.168818
+0.024575300000000001 0.15474499999999999 0.161164
+0.047873499999999999 0.224578 0.095836599999999994
+0.068413100000000004 0.22211800000000001 0.0900175
+0.078492999999999993 0.22573699999999999 0.069893899999999995
+0.10087599999999999 0.22176299999999999 0.065964999999999996
+0.113688 0.21227399999999999 0.082663399999999998
+0.10401100000000001 0.15382999999999999 0.14432800000000001
+0.126494 0.21418899999999999 0.066895499999999997
+-0.066551700000000005 0.19398099999999999 0.12488200000000001
+-0.080472199999999994 0.198127 0.113743
+-0.074614 0.18105199999999999 0.13045499999999999
+0.020276300000000001 0.21959100000000001 0.113828
+0.12228799999999999 0.20153699999999999 0.097016000000000005
+0.14039099999999999 0.19308400000000001 0.097989800000000002
+0.15592800000000001 0.19652 0.082752199999999998
+0.103478 0.20722199999999999 0.098770399999999994
+0.158633 0.184228 0.098665600000000006
+0.14837 0.206869 0.068135500000000002
+0.14377200000000001 0.21349399999999999 0.050460699999999997
+0.157944 0.211701 0.027766800000000001
+0.13782800000000001 0.216728 0.032207300000000001
+0.119203 0.220974 0.023940199999999998
+0.181201 0.20346600000000001 0.036704899999999999
+0.20399400000000001 0.19661400000000001 0.026939399999999999
+0.10811900000000001 0.22308800000000001 0.039594200000000003
+0.16981599999999999 0.19757 0.069134899999999999
+0.18832699999999999 0.18931200000000001 0.068669999999999995
+0.18108199999999999 0.19875499999999999 0.054555899999999997
+0.088028200000000001 0.211392 0.101128
+-0.046627300000000003 -0.13242300000000001 -0.17415700000000001
+-0.031881399999999997 -0.139512 -0.17436099999999999
+-0.0157739 -0.12979399999999999 -0.182809
+-0.0169605 -0.14768400000000001 -0.170043
+-0.017156500000000002 -0.156499 -0.15394099999999999
+-0.0342955 -0.157638 -0.143482
+0.0039245299999999999 -0.12660199999999999 -0.18346299999999999
+-0.042501200000000003 -0.14293400000000001 -0.16689999999999999
+-0.045460199999999999 -0.15029500000000001 -0.15321000000000001
+-0.060321199999999998 -0.144284 -0.15159700000000001
+-0.056104800000000003 -0.20834900000000001 0.10713399999999999
+-0.046628299999999998 -0.20996300000000001 0.094216999999999995
+-0.039513600000000003 -0.20919499999999999 0.079873399999999997
+-0.030061899999999999 -0.20460500000000001 0.064361600000000005
+-0.0542184 -0.21560799999999999 0.080012100000000003
+-0.072394700000000006 -0.220944 0.079464699999999999
+-0.039844499999999998 -0.202712 0.10667699999999999
+-0.026282 -0.20185600000000001 0.078793100000000005
+-0.019110599999999998 -0.19503599999999999 0.090665099999999998
+-0.060259300000000002 -0.21837000000000001 0.067015400000000003
+-0.054789499999999998 -0.214918 0.052209499999999999
+-0.043168100000000001 -0.20902499999999999 0.040407400000000003
+-0.049205400000000003 -0.207652 0.023148999999999999
+-0.068837400000000007 -0.205485 0.0059794999999999996
+-0.031862300000000003 -0.20133799999999999 0.021691999999999999
+-0.025042700000000001 -0.195297 0.0047607400000000003
+-0.078398899999999994 -0.21936600000000001 0.091723100000000002
+-0.036840999999999999 -0.19533400000000001 0.117821
+-0.085594299999999998 -0.22392500000000001 0.070458499999999993
+-0.101036 -0.22503400000000001 0.072294600000000001
+0.0079754800000000001 -0.111749 -0.18823799999999999
+0.19828899999999999 0.201344 0.012056900000000001
+0.12970699999999999 -0.000104893 -0.14339099999999999
+0.122803 0.022641999999999999 -0.141595
+0.13120399999999999 0.037935299999999998 -0.129884
+0.065233100000000002 -0.068316100000000005 0.174012
+0.169741 0.022357700000000001 0.14605299999999999
+0.174405 0.0061380999999999996 0.13108
+0.16862199999999999 -0.0086767500000000004 0.12098
+0.15368699999999999 -0.016482900000000002 0.13142699999999999
+0.171817 -0.013763900000000001 0.103162
+0.160136 -0.025257700000000001 0.095212199999999997
+0.17286899999999999 -0.016820499999999999 0.085170300000000004
+0.18521199999999999 -0.0056057600000000004 0.093273599999999998
+-0.26668500000000001 -0.165547 0.036670899999999999
+0.27587899999999999 0.083884600000000004 0.074508199999999997
+-0.093713199999999997 0.19223000000000001 0.111342
+-0.10699699999999999 0.17665900000000001 0.115898
+-0.11094999999999999 0.19046299999999999 0.10029
+-0.126606 0.18709300000000001 0.091594300000000003
+-0.14208499999999999 0.184033 0.0817885
+-0.154062 0.170706 0.087510400000000002
+-0.16913900000000001 0.15326999999999999 0.092154299999999995
+-0.149253 0.15639500000000001 0.103116
+-0.18696199999999999 0.15484800000000001 0.076457800000000006
+-0.122296 0.176425 0.105839
+-0.1187 0.16406100000000001 0.117324
+-0.20508599999999999 0.146206 0.071282300000000007
+-0.22044800000000001 0.14922199999999999 0.0532732
+-0.21107899999999999 0.163023 0.037024300000000003
+-0.21949399999999999 0.16155800000000001 0.018494
+-0.237511 0.15316099999999999 0.0125252
+-0.249528 0.14479800000000001 0.026799300000000002
+-0.24638699999999999 0.136378 0.046015599999999997
+-0.26455699999999999 0.13919799999999999 0.017173600000000001
+-0.24715899999999999 0.148788 3.9831299999999999e-05
+-0.25006899999999999 0.147707 0.013403099999999999
+0.0553132 0.17621000000000001 -0.091744300000000001
+0.17332400000000001 0.167907 -0.080498399999999998
+0.18978200000000001 0.175871 -0.073497000000000007
+0.190862 0.188891 -0.056890299999999998
+0.20769699999999999 0.17263800000000001 -0.073397299999999999
+0.224967 0.17091999999999999 -0.071398100000000006
+0.21594099999999999 0.158803 -0.080172900000000005
+-0.48482399999999998 -0.0268311 -0.043449399999999999
+-0.47713800000000001 -0.0315994 -0.033087499999999999
+-0.46851500000000001 -0.036879000000000002 -0.023458199999999998
+-0.46089599999999997 -0.036047099999999999 -0.0108418
+-0.45048300000000002 -0.039582300000000001 -0.00021644899999999999
+-0.43702200000000002 -0.035512500000000002 0.018455900000000001
+-0.0068489400000000004 0.10378999999999999 -0.16331899999999999
+-0.18601799999999999 0.12388100000000001 0.10274800000000001
+0.013715700000000001 -0.136103 -0.17514299999999999
+0.0264414 -0.13470799999999999 -0.16736200000000001
+0.038856099999999998 -0.13453499999999999 -0.153306
+0.039530799999999998 -0.121548 -0.16784199999999999
+0.011156900000000001 -0.14665300000000001 -0.16489500000000001
+-0.00075087799999999996 -0.15146000000000001 -0.16369700000000001
+0.0111868 0.056744000000000003 -0.182724
+0.23311499999999999 0.173929 0.053353900000000003
+-0.14163899999999999 -0.218195 0.038290400000000002
+-0.122474 -0.21968199999999999 0.040040899999999997
+-0.105452 -0.21801200000000001 0.035391199999999998
+0.240064 0.061858799999999999 -0.078420500000000004
+0.230353 0.046217399999999999 -0.076155399999999998
+0.22317500000000001 0.063331999999999999 -0.085683999999999996
+0.21889800000000001 0.032636900000000003 -0.076313699999999998
+0.204732 0.031191300000000002 -0.088331699999999999
+0.18990399999999999 0.024669900000000002 -0.097609100000000004
+0.18399699999999999 0.0080698200000000001 -0.093779899999999999
+0.20787700000000001 0.0225787 -0.076558200000000007
+0.20093800000000001 0.0102395 -0.068265199999999998
+-0.299431 0.082018400000000005 -0.065027000000000001
+-0.31395099999999998 0.077696899999999999 -0.067183800000000002
+-0.32093899999999997 0.064638000000000001 -0.071951100000000004
+-0.31232399999999999 0.0502342 -0.073403499999999997
+-0.32094899999999998 0.0344246 -0.076760999999999996
+-0.29531200000000002 0.051381799999999998 -0.070795899999999995
+-0.173537 -0.106557 -0.064935000000000007
+-0.454017 -0.0053947800000000001 0.012489800000000001
+0.21116099999999999 0.0175973 -0.064992900000000006
+0.0065589400000000001 -0.169937 0.10631699999999999
+0.024311300000000001 -0.15434700000000001 0.10427699999999999
+0.041125000000000002 -0.137241 0.106784
+0.033020399999999998 -0.14849799999999999 0.082684900000000006
+0.0077330599999999999 -0.159576 0.126719
+0.013880699999999999 -0.14596300000000001 0.139351
+0.0049515999999999996 -0.139462 0.152362
+-0.012690699999999999 -0.13936000000000001 0.15942000000000001
+0.018480199999999999 -0.12678500000000001 0.15585599999999999
+0.0241502 -0.11124199999999999 0.16547000000000001
+0.028689300000000001 -0.13331000000000001 0.14110500000000001
+0.042905699999999998 -0.130217 0.12626299999999999
+0.049125099999999998 -0.118004 0.13923099999999999
+0.0120257 0.069555900000000004 -0.17655799999999999
+0.083469299999999996 0.21720100000000001 -0.017059600000000001
+0.0421476 0.21581 -0.025116699999999999
+0.17894099999999999 0.017489999999999999 -0.10319299999999999
+0.16916300000000001 0.011325699999999999 -0.111613
+0.15301100000000001 -0.0010493200000000001 -0.122574
+0.15734200000000001 0.019799400000000002 -0.120215
+0.14863599999999999 0.037525200000000002 -0.12060700000000001
+0.15490599999999999 -0.0180664 -0.109226
+0.16200300000000001 -0.019309099999999999 -0.087811100000000003
+0.17632100000000001 0.0062856300000000004 -0.10222000000000001
+0.174375 -0.00362072 -0.0937611
+0.16558 -0.012255 -0.0982933
+0.16989799999999999 -0.0118878 -0.088347599999999998
+0.16742699999999999 -0.016197699999999999 -0.074634000000000006
+0.172483 -0.013695300000000001 -0.055010400000000001
+0.15857299999999999 -0.0228562 -0.038754999999999998
+0.17826800000000001 -0.0090176000000000006 -0.067720000000000002
+0.14363100000000001 -0.0308124 -0.113389
+0.13102900000000001 -0.047100900000000001 -0.104742
+0.11914 -0.0584212 -0.088755799999999996
+-0.47744300000000001 0.0072796800000000002 -0.0926757
+0.200075 0.18190200000000001 -0.065452300000000005
+0.205429 0.18839800000000001 -0.052903499999999999
+0.220716 0.18753900000000001 -0.044447
+0.233543 0.18065800000000001 -0.054730099999999997
+-0.039324600000000001 -0.026668299999999999 0.189718
+-0.274121 0.0055636699999999997 0.133906
+-0.25280999999999998 -0.00309231 0.14310800000000001
+-0.22636600000000001 0.0043005700000000001 0.15212400000000001
+-0.204148 0.0203161 0.15160899999999999
+-0.18010599999999999 0.036868999999999999 0.15116299999999999
+-0.15584899999999999 0.057039300000000001 0.149975
+-0.12928400000000001 0.046357500000000003 0.16258500000000001
+-0.12731200000000001 0.016773099999999999 0.17379800000000001
+-0.126864 -0.0148933 0.17518800000000001
+-0.096267699999999998 -0.0321701 0.176177
+-0.177678 0.0113344 0.16351099999999999
+-0.15202099999999999 0.00089938799999999999 0.173289
+-0.17436699999999999 -0.016959200000000001 0.16284599999999999
+-0.24421999999999999 -0.021609900000000001 0.13974400000000001
+-0.22625999999999999 -0.016026800000000001 0.148336
+-0.20318600000000001 -0.0300506 0.14880299999999999
+0.31216899999999997 0.091062299999999999 -0.011913200000000001
+0.30472199999999999 0.082997600000000005 -0.0216259
+0.31066199999999999 0.090270500000000004 0.00061645500000000002
+0.306807 0.090399400000000005 0.018326100000000001
+0.30343300000000001 0.081803000000000001 0.0072619700000000004
+0.296626 0.072420499999999999 0.00064681099999999996
+0.29473100000000002 0.069975200000000001 -0.013378599999999999
+0.28840100000000002 0.063922099999999996 -0.022481999999999999
+0.25839899999999999 0.076055300000000006 0.093871999999999997
+0.24593200000000001 0.079880300000000001 0.10576000000000001
+0.25372800000000001 0.093903600000000004 0.099414500000000003
+0.26049800000000001 0.107331 0.090236899999999995
+0.27343099999999998 0.103919 0.079148999999999997
+0.236537 0.067061300000000004 0.11217199999999999
+0.234206 0.047732200000000002 0.10414
+0.23352700000000001 0.082538399999999998 0.115799
+0.22527900000000001 0.097760299999999994 0.119174
+0.21699199999999999 0.084853100000000001 0.127225
+0.199159 0.0862903 0.13627900000000001
+0.20776500000000001 0.069711200000000001 0.13312399999999999
+0.19091900000000001 0.10520599999999999 0.13497500000000001
+0.17057 0.11035499999999999 0.141543
+0.15637300000000001 0.13250799999999999 0.137569
+0.211033 0.051971400000000001 0.12801499999999999
+0.22650999999999999 0.055134200000000001 0.116173
+0.18876799999999999 0.072531300000000007 0.14355100000000001
+0.17066600000000001 0.074287099999999995 0.15144299999999999
+0.16181799999999999 0.092777799999999994 0.15065200000000001
+0.18045600000000001 0.056701599999999998 0.14835999999999999
+0.18363399999999999 0.039098899999999999 0.143398
+0.19848499999999999 0.039036899999999999 0.13270000000000001
+0.161023 0.059768300000000003 0.157114
+0.217859 0.11188099999999999 0.119231
+0.201322 0.116662 0.12529799999999999
+0.23116 0.109891 0.11167000000000001
+0.27771800000000002 0.11536 0.069743600000000003
+0.28670200000000001 0.11400299999999999 0.059374299999999998
+0.29292299999999999 0.11704000000000001 0.050170899999999997
+0.29391699999999998 0.12780900000000001 0.040831300000000001
+0.29987399999999997 0.11642 0.039266799999999998
+0.28494199999999997 0.138435 0.045188100000000002
+0.27150600000000003 0.13945099999999999 0.059086899999999998
+0.26220300000000002 0.15337600000000001 0.053674399999999997
+0.26556200000000002 0.15896299999999999 0.041355000000000003
+0.27518199999999998 0.15706999999999999 0.030226699999999999
+-0.074569800000000006 0.208428 0.10571800000000001
+-0.28553499999999998 0.085776099999999994 -0.062733899999999995
+-0.27472999999999997 0.080274899999999996 -0.0641795
+-0.27114100000000002 0.068362699999999998 -0.066259799999999994
+-0.258023 0.056701500000000002 -0.067850599999999997
+-0.28099200000000002 0.056736700000000001 -0.0686977
+-0.244837 0.066967399999999996 -0.066857899999999998
+-0.23703299999999999 0.051332299999999997 -0.070258399999999999
+-0.234541 0.033337499999999999 -0.070957300000000001
+-0.245976 0.083558900000000005 -0.063375399999999998
+-0.25484099999999998 0.099248400000000001 -0.057681900000000001
+-0.24187700000000001 0.11691500000000001 -0.048652000000000001
+-0.18667 0.140069 0.090438900000000003
+0.29503600000000002 0.093583600000000003 -0.062768400000000002
+-0.0092626700000000006 0.0174249 0.19642599999999999
+-0.0032777499999999998 -0.0084101699999999998 0.19849700000000001
+0.0150689 0.0094961799999999999 0.19802900000000001
+0.0405031 0.0051091499999999998 0.19528999999999999
+0.059443299999999998 0.011896500000000001 0.19127
+-0.48195300000000002 -0.0263701 -0.036237999999999999
+-0.48020200000000002 -0.020542299999999999 -0.0285568
+-0.47638200000000003 -0.014776600000000001 -0.021227900000000001
+-0.46876699999999999 -0.0190157 -0.0084552399999999993
+-0.47247600000000001 -0.00650871 -0.014068000000000001
+-0.47228300000000001 0.0056392100000000004 -0.018385100000000001
+-0.462142 0.014903899999999999 -0.0113607
+-0.44938099999999997 0.022748399999999998 0.00012384000000000001
+-0.45730500000000002 0.0086536900000000003 0.0030441000000000001
+-0.48158699999999999 -0.0110502 -0.031619000000000001
+-0.48247499999999999 0.000430245 -0.033265500000000003
+-0.48034700000000002 0.0094886499999999995 -0.041986000000000002
+-0.47247299999999998 0.0222522 -0.043652499999999997
+-0.46677600000000002 0.024509099999999999 -0.032044799999999998
+-0.485126 0.0062711599999999996 -0.0502161
+-0.037030300000000002 -0.109778 0.171268
+-0.0368659 -0.089202400000000001 0.177455
+-0.060640100000000002 -0.071844900000000003 0.17511599999999999
+-0.018829200000000001 -0.079185199999999997 0.18451799999999999
+-0.066590999999999997 -0.0431507 0.180288
+0.0012136499999999999 -0.086199499999999998 0.18413199999999999
+0.015836900000000001 -0.073501899999999995 0.18798500000000001
+0.000163277 -0.10355200000000001 0.176731
+0.094084799999999996 -0.083839999999999998 -0.12686700000000001
+0.091261200000000001 -0.079095399999999996 0.139096
+0.097552700000000006 -0.078761600000000001 0.119196
+0.091510900000000006 -0.066797599999999999 0.154999
+0.106504 -0.054628299999999998 0.14974000000000001
+0.11228100000000001 -0.032540800000000002 0.16014100000000001
+0.120223 -0.050942500000000002 0.13462299999999999
+0.120697 -0.057035000000000002 0.114118
+0.13031400000000001 -0.037256400000000002 0.13803499999999999
+0.118329 -0.041934600000000002 0.14816699999999999
+0.078691999999999998 -0.068969299999999997 0.16483500000000001
+0.076521800000000001 -0.053140100000000003 0.17500599999999999
+0.079637600000000003 -0.034270799999999997 0.17985599999999999
+0.064064200000000002 -0.023470600000000001 0.18856000000000001
+0.053909699999999998 -0.0080092700000000006 0.19264300000000001
+0.068506200000000003 -0.079074800000000001 0.16509699999999999
+0.066006800000000004 -0.091062900000000002 0.15597800000000001
+0.075012200000000001 -0.091818999999999998 0.14458499999999999
+0.053982000000000002 -0.097950800000000005 0.15914500000000001
+0.038619000000000001 -0.102088 0.16547100000000001
+0.048001599999999998 -0.10957500000000001 0.15166299999999999
+0.033427699999999998 -0.089589699999999994 0.176126
+0.043980699999999998 -0.077008499999999994 0.17952699999999999
+0.051638099999999999 -0.060694999999999999 0.18419099999999999
+0.078784499999999993 -0.0183292 0.183534
+0.087645600000000004 -0.0040645300000000002 0.181976
+0.080052600000000002 0.0145518 0.18593399999999999
+-0.030808100000000001 0.088540499999999994 -0.17465700000000001
+0.155665 0.212315 0.00724246
+0.141239 0.21351000000000001 -0.0057057799999999997
+0.29269299999999998 0.10929999999999999 0.054666300000000001
+0.29271999999999998 0.098807699999999998 0.053018999999999997
+0.297462 0.095888799999999996 0.042752800000000001
+0.053781000000000002 -0.085918900000000006 -0.17676500000000001
+0.068457400000000002 -0.071068800000000001 -0.17282700000000001
+0.083951899999999996 -0.055962199999999997 -0.16795299999999999
+0.051669600000000003 -0.066894400000000007 -0.18332100000000001
+0.098579899999999998 -0.043157899999999999 -0.161885
+0.111766 -0.046847600000000003 -0.147448
+0.109403 -0.0636708 -0.13267399999999999
+-0.29431600000000002 0.11065700000000001 -0.047223899999999999
+-0.28312799999999999 0.119926 -0.038833600000000003
+-0.26274799999999998 0.12402000000000001 -0.039768199999999997
+-0.28836200000000001 0.12687000000000001 -0.0266383
+-0.30218299999999998 0.12418700000000001 -0.0189976
+-0.31845800000000002 0.117372 -0.020931399999999999
+-0.32725500000000002 0.11022999999999999 -0.00445948
+-0.31620599999999999 0.112645 0.0085312900000000004
+-0.29883799999999999 0.121181 0.013046500000000001
+-0.31412699999999999 0.115203 -0.0350952
+-0.29103099999999998 0.12880800000000001 0.00166
+-0.277171 0.135902 -0.000114267
+-0.27324799999999999 0.136459 -0.0125396
+0.097695000000000004 -0.0737844 -0.14055100000000001
+0.082734699999999994 -0.089580199999999999 -0.145093
+0.086838499999999999 -0.074848799999999993 -0.15546599999999999
+0.067970299999999997 -0.106364 -0.147753
+0.060353200000000003 -0.120556 -0.134077
+0.058830399999999998 -0.104422 -0.16323299999999999
+-0.32267400000000002 0.0086848299999999993 0.11067200000000001
+-0.30726300000000001 0.0067620500000000004 0.118571
+-0.30800499999999997 -0.0086171800000000003 0.119522
+-0.29358099999999998 0.011466499999999999 0.12300999999999999
+-0.29932900000000001 0.0239726 0.11282300000000001
+-0.28615400000000002 0.039842299999999997 0.107825
+-0.26321 0.037897399999999998 0.11920799999999999
+-0.13457 0.121943 -0.106906
+-0.139297 0.13466500000000001 -0.089991199999999993
+-0.160382 0.13081200000000001 -0.074639399999999995
+-0.121714 0.150283 -0.087212799999999993
+-0.16375600000000001 0.148893 -0.052594799999999997
+-0.109282 0.16651299999999999 -0.074029499999999998
+-0.12864800000000001 0.16718 -0.055228800000000002
+-0.16725499999999999 0.11383600000000001 -0.083424700000000004
+-0.39439299999999999 -0.018681099999999999 0.065612699999999996
+-0.22717399999999999 0.111419 0.086208300000000002
+-0.29622700000000002 0.12784599999999999 -0.0091753600000000005
+-0.29336699999999999 0.128193 -0.0184368
+-0.238147 -0.0055974099999999997 0.14847399999999999
+-0.239063 0.0091714599999999993 0.14564099999999999
+-0.22414600000000001 0.026270399999999999 0.141961
+-0.250836 0.0091494499999999999 0.141379
+-0.26247900000000002 0.018606899999999999 0.132247
+-0.26232299999999997 0.0042172599999999996 0.13866600000000001
+-0.26321899999999998 -0.0084791099999999998 0.13811300000000001
+-0.256828 -0.019899099999999999 0.13605300000000001
+-0.26305400000000001 -0.0300816 0.12826699999999999
+-0.27682800000000002 -0.027548 0.12421699999999999
+-0.27500799999999997 -0.0069395400000000001 0.13413700000000001
+-0.28145900000000001 -0.0163763 0.12839100000000001
+-0.259073 0.11239399999999999 -0.049898900000000003
+-0.32078200000000001 0.10963100000000001 -0.042513099999999998
+-0.33068799999999998 0.10691000000000001 -0.043354200000000002
+-0.33934300000000001 0.10236199999999999 -0.046549699999999999
+-0.348883 0.098677799999999996 -0.048183900000000002
+-0.35806900000000003 0.093758999999999995 -0.051441000000000001
+-0.13687299999999999 -0.0120468 -0.151363
+-0.14122100000000001 -0.034050700000000003 -0.14651600000000001
+-0.128382 -0.030968599999999999 -0.16051299999999999
+-0.11523700000000001 -0.033887199999999999 -0.17164699999999999
+-0.10415000000000001 -0.022127299999999999 -0.177401
+-0.092526399999999995 -0.0054494499999999998 -0.18079400000000001
+-0.106711 -0.0068729999999999998 -0.173788
+-0.119908 0.00027730400000000003 -0.164218
+-0.114456 0.018165899999999999 -0.165215
+-0.082517300000000002 0.0101783 -0.18265999999999999
+-0.118672 -0.048041 -0.16878099999999999
+-0.12155299999999999 -0.061881899999999997 -0.163771
+-0.11089400000000001 -0.0624358 -0.17219899999999999
+-0.39573000000000003 0.079365500000000005 -0.049794100000000001
+-0.39736399999999999 0.075222999999999998 -0.062139699999999999
+-0.38926899999999998 0.080610100000000004 -0.058042499999999997
+-0.37967600000000001 0.084154499999999993 -0.057447199999999997
+-0.37281300000000001 0.088494799999999998 -0.0523992
+-0.373143 0.090201600000000007 -0.0428662
+-0.40528700000000001 0.069833199999999998 -0.066590399999999994
+-0.050975199999999998 -0.010730200000000001 -0.19368299999999999
+0.068437899999999996 0.058758199999999997 0.18426799999999999
+0.095081700000000005 0.070039299999999999 0.17625399999999999
+0.11186599999999999 0.048432900000000001 0.175538
+0.0135438 0.23103000000000001 0.088735400000000006
+-0.148114 -0.22111700000000001 0.051070999999999998
+-0.159772 -0.21833 0.056201500000000001
+-0.17114099999999999 -0.21582699999999999 0.049958000000000002
+-0.172124 -0.214009 0.034900899999999999
+-0.183868 -0.21162500000000001 0.033302100000000001
+-0.48066999999999999 0.0014756299999999999 -0.094153100000000003
+-0.48325000000000001 0.0051100800000000004 -0.087128800000000006
+-0.47888599999999998 -0.0067006399999999999 -0.095381300000000002
+-0.48850900000000003 -0.00024797699999999997 -0.083196999999999993
+-0.49010599999999999 0.00017326200000000001 -0.074509199999999998
+-0.48638399999999998 0.0077295999999999997 -0.068793800000000002
+-0.49083900000000003 -0.0072984699999999996 -0.067552500000000001
+-0.48757299999999998 -0.0076937899999999998 -0.085185899999999995
+0.30541299999999999 0.115443 0.0310865
+0.30306699999999998 0.109776 0.036199099999999998
+0.30152800000000002 0.100881 0.0364219
+0.30454199999999998 0.095028199999999993 0.027837799999999999
+0.308585 0.101812 0.019347799999999998
+0.30202000000000001 0.085854200000000006 0.025549800000000001
+0.294651 0.077623899999999996 0.030824799999999999
+-0.13972999999999999 -0.120127 -0.089437100000000005
+0.077412099999999998 -0.101688 -0.13441
+0.084279999999999994 -0.0964758 -0.11860999999999999
+0.075763499999999998 -0.104018 -0.095217800000000005
+0.0953267 -0.083999299999999999 -0.106017
+0.0029394500000000001 0.19001299999999999 -0.077441399999999994
+-0.0158096 0.19506699999999999 -0.066558199999999998
+-0.0158703 0.20566599999999999 -0.044179299999999998
+0.091303899999999993 -0.074953099999999995 -0.036518099999999998
+-0.24309700000000001 0.15007799999999999 0.0206614
+-0.23361000000000001 0.15435099999999999 0.0222242
+-0.071663699999999997 -0.19434399999999999 0.123031
+-0.086263300000000001 -0.19167899999999999 0.120669
+-0.079351000000000005 -0.19966900000000001 0.116734
+0.090543700000000005 -0.072083099999999997 -0.0082290999999999996
+-0.44237500000000002 -0.080560199999999998 -0.072939100000000007
+-0.43468400000000001 -0.074604000000000004 -0.085476499999999997
+-0.437054 -0.086967500000000003 -0.060754700000000002
+-0.42770999999999998 -0.094516799999999998 -0.052825700000000003
+-0.418352 -0.098305400000000001 -0.040073499999999998
+-0.42327900000000002 -0.096600099999999994 -0.065513100000000005
+-0.40865899999999999 -0.10148699999999999 -0.069723800000000002
+-0.416431 -0.090925400000000003 -0.0763075
+-0.41177900000000001 -0.078854099999999996 -0.0841388
+-0.40598000000000001 -0.065549200000000002 -0.088420399999999996
+-0.41605500000000001 -0.049127400000000002 -0.093640699999999993
+-0.400038 -0.0506823 -0.090530299999999994
+-0.38142100000000001 -0.045395499999999998 -0.086927599999999994
+-0.40224300000000002 -0.110281 -0.057950799999999997
+-0.39021600000000001 -0.11874 -0.050110799999999997
+-0.383494 -0.123101 -0.0364651
+-0.386125 -0.115131 -0.020342800000000001
+-0.36962800000000001 -0.13037399999999999 -0.0274223
+-0.40145599999999998 -0.102315 -0.019867900000000001
+-0.39417099999999999 -0.100274 -0.072764499999999996
+-0.37404100000000001 -0.097433599999999995 -0.071859199999999998
+-0.38566499999999998 -0.086013500000000007 -0.077930700000000006
+-0.35971199999999998 -0.131969 -0.0141493
+-0.35573500000000002 -0.12106699999999999 0.0058952099999999997
+-0.40775499999999998 -0.10759100000000001 -0.044512500000000003
+-0.44210500000000003 -0.063409699999999999 -0.090683299999999994
+-0.42282500000000001 -0.082393900000000006 -0.081556000000000003
+-0.45077 -0.055459799999999997 -0.092926400000000006
+-0.45119500000000001 -0.043705000000000001 -0.097222299999999998
+-0.437782 -0.034036499999999997 -0.100162
+-0.43303599999999998 -0.0155961 -0.100753
+-0.45492199999999999 -0.032423100000000003 -0.100841
+-0.458814 -0.0492048 -0.0932669
+-0.44563000000000003 -0.0053807300000000002 -0.10256700000000001
+-0.44582500000000003 -0.028368000000000001 -0.102242
+0.18756700000000001 0.011524100000000001 0.12186900000000001
+0.19992299999999999 0.0237015 0.120686
+0.20960799999999999 0.0257357 0.11161
+0.21748300000000001 0.0222293 0.093870899999999993
+0.22097 0.032477300000000001 0.10435700000000001
+0.15055099999999999 0.010878199999999999 -0.12618799999999999
+0.141789 0.00011003299999999999 -0.133433
+0.15970799999999999 0.0081676700000000001 -0.119042
+0.16130900000000001 -0.0024486199999999999 -0.114514
+0.154667 -0.0107676 -0.11754299999999999
+0.147178 -0.010362400000000001 -0.12647
+0.148617 -0.020408800000000001 -0.118466
+0.13852400000000001 -0.011450999999999999 -0.134851
+0.12890699999999999 -0.026258699999999999 -0.13917599999999999
+0.12589600000000001 -0.0139325 -0.146588
+0.113147 -0.00053748299999999995 -0.155555
+0.103133 0.016400100000000001 -0.15639400000000001
+0.104298 -0.0149856 -0.16309000000000001
+0.12017 -0.035598499999999998 -0.145343
+0.110453 -0.035544100000000002 -0.154722
+-0.044026500000000003 0.056760699999999997 -0.18568799999999999
+0.27999200000000002 0.16592899999999999 -0.043703899999999997
+-0.21535899999999999 -0.201377 0.036184000000000001
+-0.223693 -0.188967 0.047823400000000002
+-0.22968 -0.16561699999999999 0.063274399999999995
+-0.24318400000000001 -0.14052799999999999 0.072279800000000005
+-0.22803499999999999 -0.19495199999999999 0.035675900000000003
+-0.23849600000000001 -0.19363 0.023692399999999999
+-0.25412400000000002 -0.18660599999999999 0.0200977
+-0.257716 -0.18540699999999999 0.0029471599999999999
+-0.27287099999999997 -0.17936299999999999 -0.00179042
+-0.28758299999999998 -0.170182 0.0102401
+-0.24412800000000001 -0.18221699999999999 -0.0044050299999999999
+-0.22953100000000001 -0.18651499999999999 -5.5947399999999999e-05
+-0.220475 -0.194692 0.0089781199999999992
+-0.26251099999999999 -0.176872 0.0277986
+-0.25326300000000002 -0.17311000000000001 0.039993899999999999
+-0.24254600000000001 -0.168318 0.052385899999999999
+-0.25711499999999998 -0.15942600000000001 0.049328299999999999
+-0.27587699999999998 -0.16930799999999999 0.024501599999999998
+-0.24587100000000001 -0.15378 0.061338299999999998
+-0.25172699999999998 -0.18192800000000001 0.031364700000000002
+-0.21268400000000001 -0.20027700000000001 0.0170569
+-0.20352300000000001 -0.203845 0.0214033
+-0.192659 -0.203121 0.018234400000000001
+-0.188052 -0.194159 0.0076752000000000001
+-0.21226800000000001 -0.20365 0.026703299999999999
+-0.231299 -0.195988 0.014245799999999999
+-0.19641400000000001 -0.20655399999999999 0.024866200000000001
+-0.19286 -0.20988799999999999 0.0326474
+-0.198432 -0.20674699999999999 0.044257499999999998
+-0.28556100000000001 -0.174648 -0.0023933600000000002
+-0.29736299999999999 -0.16808200000000001 -0.012934899999999999
+-0.31544100000000003 -0.16035199999999999 -0.0156542
+-0.32242599999999999 -0.15056600000000001 0.0015451600000000001
+-0.27945999999999999 -0.17655899999999999 0.0065269300000000002
+-0.32907799999999998 -0.152562 -0.012822800000000001
+-0.34145199999999998 -0.14733099999999999 -0.021559600000000002
+-0.33934700000000001 -0.14391100000000001 -0.037897800000000002
+-0.32578200000000002 -0.13728899999999999 -0.044510099999999997
+-0.31670300000000001 -0.146788 -0.037035899999999997
+-0.31557600000000002 -0.155996 -0.027913199999999999
+-0.35231899999999999 -0.13774 -0.042353000000000002
+-0.35627700000000001 -0.12551100000000001 -0.054545700000000003
+-0.37187599999999998 -0.118612 -0.059590600000000001
+-0.36245300000000003 -0.135739 -0.035773300000000001
+-0.30760100000000001 -0.16083500000000001 -0.022816699999999999
+-0.29867300000000002 -0.163768 -0.021315199999999999
+-0.28379500000000002 -0.159885 -0.025242400000000002
+-0.28890100000000002 -0.16657 -0.018883
+-0.27143099999999998 -0.163684 -0.021205700000000001
+-0.26730599999999999 -0.17346300000000001 -0.0127247
+-0.27183400000000002 -0.18046200000000001 0.00664249
+-0.26405000000000001 -0.184059 0.0090717000000000002
+-0.25620300000000001 -0.18756700000000001 0.011472599999999999
+-0.248391 -0.190551 0.0145387
+-0.19015299999999999 -0.20661399999999999 0.0534964
+-0.44255499999999998 0.00026552299999999999 0.026730199999999999
+-0.43035200000000001 0.014090999999999999 0.0310689
+-0.42472100000000002 0.0323378 0.017327499999999999
+-0.42703600000000003 -0.00112736 0.0417282
+-0.42339199999999999 -0.016438299999999999 0.044238899999999998
+-0.41653699999999999 0.00176833 0.051877
+0.28040399999999999 0.13509299999999999 0.054525999999999998
+0.27876099999999998 0.125803 0.062829300000000005
+-0.3705 -0.13137799999999999 -0.036506900000000002
+-0.0820636 -0.222416 0.082619399999999996
+0.0171345 0.16531999999999999 -0.108665
+0.0025037499999999999 0.14652799999999999 -0.128943
+-0.0048198399999999997 0.165963 -0.110419
+-0.0228077 0.17507700000000001 -0.098515900000000003
+-0.046374699999999998 0.16789599999999999 -0.10471999999999999
+0.0078590199999999995 0.12789700000000001 -0.142484
+0.018770800000000001 0.138019 -0.13144900000000001
+-0.028861299999999999 0.144009 -0.13425300000000001
+-0.044995199999999999 0.15353600000000001 -0.122945
+0.29028300000000001 0.121714 -0.074014399999999994
+0.27744999999999997 0.12898000000000001 -0.076940400000000006
+0.27683200000000002 0.144257 -0.075574500000000003
+0.26365499999999997 0.13398499999999999 -0.080384700000000003
+0.24396200000000001 0.140125 -0.081940600000000002
+0.25181799999999999 0.124002 -0.082061400000000007
+0.27904099999999998 0.112623 -0.076369599999999996
+0.27115800000000001 0.153665 -0.070900599999999994
+0.25792100000000001 0.162187 -0.070153199999999999
+0.26986399999999999 0.163795 -0.062653600000000004
+0.24820800000000001 0.15371099999999999 -0.0775922
+0.25947399999999998 0.16913900000000001 -0.060685200000000002
+0.25480599999999998 0.17483000000000001 -0.049851899999999998
+0.26311499999999999 0.17435200000000001 -0.039377700000000002
+0.24249799999999999 0.16399900000000001 -0.073516499999999999
+0.286047 0.14730399999999999 -0.069066299999999997
+0.29011700000000001 0.152194 -0.0598459
+0.29456100000000002 0.14326 -0.067360299999999998
+0.30188999999999999 0.13453899999999999 -0.064544699999999997
+0.30783100000000002 0.12835099999999999 -0.059514299999999999
+0.30943500000000002 0.118425 -0.060111699999999997
+0.300896 0.11136699999999999 -0.064607399999999995
+0.31357400000000002 0.12434199999999999 -0.053265699999999999
+0.31838899999999998 0.120863 -0.046066000000000003
+0.31739299999999998 0.11330800000000001 -0.0379339
+0.320268 0.118282 -0.030761899999999998
+0.320218 0.116784 -0.022126400000000001
+0.31948100000000001 0.11809 -0.010456500000000001
+0.31556000000000001 0.12664800000000001 -0.00415804
+0.31660199999999999 0.113926 -0.045093500000000002
+0.31177700000000003 0.106867 -0.048073299999999999
+0.30827500000000002 0.10918600000000001 -0.056207
+0.30850499999999997 0.098341200000000004 -0.048610199999999999
+0.30314400000000002 0.092595200000000003 -0.053105100000000002
+0.30960100000000002 0.093476400000000001 -0.041793799999999999
+0.31315599999999999 0.095964599999999997 -0.0340974
+0.296983 0.11741500000000001 -0.0699211
+0.320685 0.122446 -0.025589600000000001
+0.316942 0.13097800000000001 -0.0270583
+0.31485400000000002 0.133023 -0.0360942
+0.31084699999999998 0.14150299999999999 -0.038310900000000002
+0.30931799999999998 0.14124100000000001 -0.047368899999999999
+0.304008 0.14735100000000001 -0.0517981
+0.30704500000000001 0.147922 -0.034493999999999997
+0.29826799999999998 0.15381300000000001 -0.036844500000000002
+0.30964900000000001 0.14619399999999999 -0.027128099999999999
+0.30556299999999997 0.14827399999999999 -0.019965
+0.29749399999999998 0.154475 -0.020069300000000002
+0.292877 0.15678 -0.0092131699999999997
+0.28607100000000002 0.16011 0.00062224299999999997
+0.27559499999999998 0.166073 0.0087612000000000002
+0.26022400000000001 0.17458899999999999 0.0077726000000000002
+0.25062200000000001 0.175429 0.025409500000000002
+0.31161299999999997 0.14075599999999999 -0.021410100000000001
+0.30851800000000001 0.14502599999999999 -0.013265300000000001
+0.296427 0.15654799999999999 -0.0286188
+0.29088799999999998 0.15989800000000001 -0.0225221
+0.28007199999999999 0.16805500000000001 -0.0192366
+0.00514285 -0.026233699999999999 0.19953000000000001
+0.000257161 -0.040387600000000003 0.19702800000000001
+-0.014043 -0.051568999999999997 0.191857
+0.019852100000000001 -0.028945800000000001 0.19900799999999999
+-0.44991500000000001 0.0066942399999999997 0.013299699999999999
+0.189521 0.18335399999999999 -0.065903900000000001
+0.18108299999999999 0.18820200000000001 -0.061319100000000001
+0.16613900000000001 0.19374 -0.055086700000000002
+0.18040100000000001 0.19547999999999999 -0.047881100000000003
+0.173792 0.202184 -0.030743599999999999
+0.014203 -0.120681 -0.18353
+0.023841399999999999 -0.113012 -0.18310100000000001
+0.033454299999999999 -0.103091 -0.18218599999999999
+0.033035500000000002 -0.116504 -0.176425
+0.031873400000000003 -0.12510099999999999 -0.17236699999999999
+0.035683899999999998 -0.130691 -0.16428899999999999
+0.043684300000000002 -0.12582399999999999 -0.158725
+0.048837400000000003 -0.116976 -0.16342999999999999
+-0.073536900000000002 0.130882 -0.137882
+-0.0601936 0.14341200000000001 -0.130466
+-0.0624708 0.11858399999999999 -0.151005
+-0.041211999999999999 0.115328 -0.15739800000000001
+-0.042079699999999998 0.10029399999999999 -0.16700599999999999
+0.18629799999999999 0.15686 0.109529
+-0.0106882 -0.179034 -0.053539000000000003
+-0.0257532 -0.183285 -0.056009999999999997
+-0.446546 -0.020147200000000001 0.019944799999999999
+-0.455322 -0.018398899999999999 0.0101318
+-0.46273900000000001 -0.0092777199999999997 0.0013349499999999999
+-0.44792399999999999 -0.0304864 0.010348100000000001
+-0.32777299999999998 -0.145984 -0.0378562
+-0.48141099999999998 0.0150496 -0.063860799999999995
+0.32042100000000001 0.119931 -0.016534400000000001
+0.31704199999999999 0.125637 -0.0119931
+0.31845400000000001 0.112192 -0.015792899999999999
+0.092939900000000006 0.15981200000000001 -0.095894300000000002
+0.110625 0.16691700000000001 -0.0892709
+0.124473 0.17113600000000001 -0.084585300000000002
+0.136186 0.183251 -0.074210600000000002
+0.13866000000000001 0.17169200000000001 -0.082379800000000003
+0.148566 0.160826 -0.085964700000000005
+0.142872 0.14471200000000001 -0.090559700000000007
+0.16079499999999999 0.122901 -0.090634900000000004
+0.12608800000000001 0.14530199999999999 -0.093695299999999995
+0.115879 0.13087099999999999 -0.099057199999999998
+0.12432699999999999 0.191494 -0.066376900000000003
+0.106391 0.192491 -0.067065600000000003
+0.088387199999999999 0.18826200000000001 -0.074576600000000007
+0.071950700000000006 0.19680600000000001 -0.064225500000000005
+0.108844 0.20166500000000001 -0.051652999999999998
+0.098499900000000001 0.17773700000000001 -0.083660499999999999
+-0.12026000000000001 -0.22390199999999999 0.053194999999999999
+-0.109446 -0.22493099999999999 0.056583099999999997
+-0.111541 -0.22161900000000001 0.045167800000000001
+-0.24204600000000001 -0.18762300000000001 0.032577599999999998
+-0.32623400000000002 0.0938195 0.029149899999999999
+-0.010242100000000001 -0.165045 0.13696
+-0.37358799999999998 0.010098899999999999 0.080166899999999999
+0.00996937 -0.012989799999999999 0.200095
+-0.46387699999999998 -0.032463699999999998 -0.100365
+-0.46022400000000002 -0.024182800000000001 -0.10134
+-0.45973900000000001 -0.0142811 -0.10137699999999999
+0.13320799999999999 -0.041244900000000001 -0.119383
+0.123352 -0.052679700000000003 -0.120194
+0.132637 -0.034374099999999998 -0.13009399999999999
+-0.164742 -0.12217799999999999 -0.059505200000000001
+0.044514999999999999 -0.137297 -0.111748
+-0.061279100000000003 -0.16256599999999999 -0.10968799999999999
+-0.071012900000000004 -0.16483400000000001 -0.095825300000000002
+-0.073553300000000002 -0.17218600000000001 -0.076810600000000007
+-0.087645200000000006 -0.16161 -0.086792900000000006
+-0.476659 -0.027153799999999999 -0.096453300000000006
+-0.47442899999999999 -0.0190092 -0.097352800000000003
+0.073971800000000004 -0.0242315 -0.180703
+0.162304 -0.0056067299999999999 0.13305800000000001
+-0.479103 -0.032098500000000002 -0.094002699999999995
+-0.48141699999999998 -0.034085799999999999 -0.088405700000000004
+-0.48612699999999998 -0.030643500000000001 -0.086058300000000004
+-0.48601800000000001 -0.024288299999999999 -0.087257699999999994
+-0.48210399999999998 -0.021943899999999999 -0.092236399999999996
+-0.48388599999999998 -0.035583200000000002 -0.082741899999999993
+-0.480962 -0.040286799999999998 -0.078852199999999997
+-0.48836800000000002 -0.026736300000000001 -0.083000900000000002
+-0.48974400000000001 -0.025216499999999999 -0.0755797
+-0.49086000000000002 -0.024424600000000001 -0.069549799999999995
+-0.489653 -0.0247292 -0.060789900000000001
+-0.49031599999999997 -0.019490500000000001 -0.064984799999999995
+-0.49036400000000002 -0.028426799999999999 -0.072102700000000006
+-0.48628100000000002 -0.033962199999999998 -0.072745199999999996
+-0.484931 -0.035798099999999999 -0.065703200000000003
+-0.48041699999999998 -0.042245199999999997 -0.065058900000000003
+-0.48662300000000003 -0.0314942 -0.059831200000000001
+-0.47195999999999999 -0.030398999999999999 -0.098425499999999999
+-0.066902500000000004 0.0064793100000000003 0.18904799999999999
+-0.0025919599999999999 0.054607299999999998 0.192242
+-0.028547699999999999 0.070800699999999994 0.18473400000000001
+-0.0057345 0.037484400000000001 0.19470999999999999
+-0.0268013 0.15953400000000001 0.15797900000000001
+-0.0096282999999999994 0.14879200000000001 0.16497200000000001
+-0.078126299999999996 -0.119784 -0.16633500000000001
+-0.069360599999999994 -0.12950500000000001 -0.164109
+-0.068875199999999998 -0.121624 -0.171204
+-0.30148399999999997 0.103619 -0.053148000000000001
+-0.29309600000000002 0.10467 -0.053065500000000002
+-0.28153499999999998 0.10184799999999999 -0.055146500000000001
+-0.28510200000000002 0.10936999999999999 -0.050098200000000002
+-0.27017799999999997 0.10181900000000001 -0.055790199999999998
+0.14641299999999999 0.21560099999999999 0.021743200000000001
+-0.055876799999999997 -0.20058100000000001 -0.0058347
+-0.121903 -0.22239800000000001 0.075382699999999997
+-0.13148399999999999 -0.222972 0.067294300000000001
+-0.366143 0.0142557 0.083265599999999995
+-0.011362199999999999 -0.139899 -0.17758599999999999
+-0.46108399999999999 -0.018208200000000001 0.00293327
+-0.464563 -0.022432400000000002 -0.00273456
+-0.46595599999999998 -0.0279422 -0.010426899999999999
+-0.079657800000000001 0.18559999999999999 -0.060871700000000001
+-0.40898200000000001 -0.00209341 0.057713
+-0.40249400000000002 -0.0094119900000000003 0.062336500000000003
+-0.40335300000000002 -0.019611199999999999 0.058583000000000003
+-0.40576200000000001 -0.028626700000000001 0.051533799999999998
+-0.40561799999999998 -0.039816699999999997 0.044003100000000003
+-0.40473599999999998 -0.052921799999999998 0.033436300000000002
+-0.409831 -0.063620099999999999 0.018912499999999999
+-0.40256599999999998 -0.078547400000000003 0.0097687
+-0.38431100000000001 -0.088619199999999995 0.015902400000000001
+-0.39992299999999997 0.00020637100000000001 0.064814800000000006
+-0.39525900000000003 0.0101254 0.063961900000000002
+-0.39940900000000001 0.0198184 0.054190200000000001
+-0.40101500000000001 0.031749899999999998 0.041754800000000002
+-0.386013 0.042619600000000001 0.040975900000000003
+-0.39055499999999999 0.0202045 0.061387499999999998
+-0.37609100000000001 0.028226000000000001 0.064886899999999997
+-0.35996899999999998 0.035405800000000001 0.068977700000000003
+-0.39047799999999999 0.0030778699999999999 0.071252399999999994
+-0.41499200000000003 0.027358899999999998 0.032616800000000001
+-0.082224900000000004 -0.219415 0.043609299999999997
+0.320548 0.121146 -0.020907599999999998
+-0.030711800000000001 0.20911199999999999 0.12504399999999999
+0.24469399999999999 0.071696200000000002 -0.078943899999999997
+-0.32278600000000002 -0.150478 -0.034060199999999999
+-0.14291799999999999 -0.22189800000000001 0.059583299999999999
+-0.14105500000000001 -0.21972700000000001 0.069530499999999995
+-0.14838399999999999 -0.21382300000000001 0.074768699999999993
+-0.140352 -0.208727 0.084856600000000004
+-0.15936900000000001 -0.210758 0.071433800000000006
+-0.12697 -0.21054800000000001 0.090340000000000004
+-0.123362 -0.21745700000000001 0.084004499999999996
+0.29131899999999999 0.148647 -0.065299999999999997
+0.28592299999999998 0.152334 -0.064874000000000001
+0.28020600000000001 0.15112999999999999 -0.068714300000000006
+-0.011495 -0.128161 0.165796
+-0.010448900000000001 -0.11426799999999999 0.172787
+-0.45632800000000001 0.0023828899999999999 0.0091199899999999997
+-0.46271600000000002 0.00053364999999999999 0.0018853699999999999
+-0.45271899999999998 0.00080143200000000003 0.0144475
+-0.44771100000000003 -0.0056644800000000004 0.020379000000000001
+-0.44707200000000002 0.0029647300000000001 0.0202268
+-0.43997700000000001 0.0094005700000000005 0.0233511
+-0.44131500000000001 -0.010126599999999999 0.027390600000000001
+-0.434861 -0.0036457 0.033997899999999998
+-0.43562299999999998 0.0054161599999999997 0.030922499999999999
+-0.39701399999999998 -0.107359 -0.0661138
+-0.39450099999999999 -0.112676 -0.060127800000000002
+-0.39739000000000002 -0.114494 -0.052232000000000001
+-0.38694499999999998 -0.116865 -0.0581355
+-0.37967299999999998 -0.123432 -0.051806699999999997
+-0.37934899999999999 -0.114175 -0.062503500000000004
+-0.36884499999999998 -0.10815 -0.065851699999999999
+-0.35989900000000002 -0.097353599999999998 -0.069139000000000006
+-0.37942399999999998 -0.105627 -0.068412500000000001
+-0.34553299999999998 -0.099972900000000003 -0.064961400000000002
+-0.34044799999999997 -0.115776 -0.057915000000000001
+-0.34899400000000003 -0.084583800000000001 -0.0704153
+-0.33274199999999998 -0.065673099999999998 -0.070522600000000005
+-0.35099399999999997 -0.066454700000000005 -0.075408699999999995
+-0.34181800000000001 -0.047047100000000001 -0.076195700000000005
+-0.351962 -0.0231594 -0.082381800000000005
+-0.39551999999999998 -0.11584700000000001 -0.043825299999999998
+-0.37087399999999998 -0.124971 -0.053235600000000001
+-0.36336400000000002 -0.12976199999999999 -0.049667999999999997
+-0.33008999999999999 -0.123253 -0.052732000000000001
+-0.316778 -0.125001 -0.049363499999999998
+-0.30263899999999999 -0.12310699999999999 -0.047369599999999998
+-0.28703899999999999 -0.12488200000000001 -0.043681200000000003
+-0.27375899999999997 -0.109 -0.046197000000000002
+-0.27046900000000001 -0.12602099999999999 -0.039752500000000003
+-0.25473800000000002 -0.113942 -0.041688099999999999
+-0.27834500000000001 -0.088689400000000002 -0.052707200000000003
+-0.26637699999999997 -0.067844299999999996 -0.056034500000000001
+-0.28153499999999998 -0.044186099999999999 -0.062687599999999996
+-0.30022599999999999 -0.091271699999999997 -0.056407499999999999
+-0.31246699999999999 -0.073381799999999997 -0.063487699999999994
+-0.26353799999999999 -0.049345800000000002 -0.059380200000000001
+-0.38483400000000001 -0.12300899999999999 -0.044951100000000001
+-0.37869599999999998 -0.12701100000000001 -0.041614499999999999
+-0.37681999999999999 -0.12697600000000001 -0.033598299999999998
+-0.38166 -0.12148 -0.026721700000000001
+-0.45038 -0.074929800000000005 -0.072447300000000006
+-0.45795999999999998 -0.068416900000000003 -0.075791499999999998
+-0.46366800000000002 -0.061313199999999998 -0.078911400000000007
+-0.470167 -0.0560323 -0.073572799999999994
+-0.45690799999999998 -0.064758099999999999 -0.084254200000000001
+-0.46812300000000001 -0.054153600000000003 -0.082180500000000004
+-0.35536499999999999 0.085403400000000004 -0.063983100000000001
+-0.345495 0.078401799999999994 -0.068668099999999996
+-0.34556599999999998 0.064270499999999994 -0.075424400000000003
+-0.345752 0.086915099999999995 -0.063962599999999994
+-0.34040900000000002 0.092115600000000006 -0.059582099999999999
+-0.320658 -0.15615499999999999 -0.0073590299999999999
+0.031028699999999999 -0.049854700000000002 0.19364100000000001
+0.019889400000000002 -0.14193700000000001 -0.16545899999999999
+-0.228047 -0.0061159700000000001 0.15153900000000001
+-0.21379000000000001 -0.010193900000000001 0.15467800000000001
+-0.21124000000000001 0.0052466500000000003 0.15638099999999999
+-0.196797 0.0093086999999999996 0.159224
+-0.188163 0.0229398 0.155281
+-0.186748 -0.00071344699999999997 0.16439999999999999
+-0.17146900000000001 -0.0024138699999999998 0.16889299999999999
+-0.15892999999999999 -0.013439400000000001 0.16819300000000001
+-0.21956100000000001 -0.0022946400000000001 0.154558
+-0.211896 -0.0027186900000000002 0.15682499999999999
+0.24188599999999999 0.14030899999999999 0.086030200000000001
+-0.25437799999999999 -0.18060799999999999 -0.0057263100000000001
+-0.025636200000000001 -0.19279499999999999 -0.0108483
+-0.0213145 -0.18801300000000001 -0.025495899999999998
+-0.00679366 -0.18057300000000001 -0.0278423
+0.26587899999999998 0.079333600000000004 0.085775799999999999
+-0.178895 -0.21406 0.038848300000000002
+-0.17230000000000001 -0.21589800000000001 0.042354700000000002
+-0.244753 0.00128651 0.14643700000000001
+-0.237347 0.0022347999999999999 0.14898500000000001
+0.25157000000000002 0.045726700000000002 -0.052765399999999997
+-0.47526099999999999 -0.0012175899999999999 -0.018558100000000001
+-0.47783799999999998 0.0041643299999999999 -0.025906100000000001
+-0.49047400000000002 0.0025801399999999999 -0.0683864
+0.19606899999999999 0.034771700000000003 -0.095783099999999996
+0.20161499999999999 0.0491382 -0.093995200000000001
+0.18803 0.062183099999999998 -0.0972113
+0.16736400000000001 0.075642699999999993 -0.099624299999999999
+0.17239699999999999 0.099855899999999997 -0.092121900000000007
+0.187663 0.033633000000000003 -0.100368
+0.174757 0.0400994 -0.106834
+0.19705300000000001 0.026279299999999998 -0.091385900000000006
+0.202678 0.022654899999999999 -0.084570599999999996
+0.210031 0.055648599999999999 -0.090792899999999996
+0.19250700000000001 0.052704500000000001 -0.097813999999999998
+0.18021599999999999 0.052982799999999997 -0.102021
+0.175563 0.065276399999999998 -0.100425
+0.16292599999999999 0.064042299999999996 -0.104369
+0.142127 0.062495299999999997 -0.113056
+0.15193899999999999 0.075628399999999998 -0.10379099999999999
+0.13278899999999999 0.079786499999999996 -0.10854
+-0.348441 -0.14252699999999999 -0.034886899999999998
+-0.34230100000000002 -0.14637 -0.030130400000000002
+0.14871100000000001 0.18328900000000001 -0.072625099999999998
+0.16165599999999999 0.18123600000000001 -0.072920200000000004
+0.163216 0.188442 -0.064501000000000003
+-0.23855599999999999 -0.031050600000000001 -0.0635487
+-0.0029130699999999998 0.035265100000000001 -0.19427
+0.010807499999999999 0.041325300000000002 -0.188804
+-0.0055335699999999998 0.020327600000000001 -0.19745599999999999
+-0.0214647 0.015901599999999998 -0.196991
+0.0071276999999999998 0.0113207 -0.197326
+0.029401299999999998 -0.148697 -0.11991599999999999
+0.0138806 -0.159193 -0.10931200000000001
+0.0172829 -0.15901699999999999 -0.088773500000000005
+0.0277653 -0.14779900000000001 -0.13544999999999999
+0.0027332699999999999 -0.1618 -0.121337
+-0.011110999999999999 -0.16848399999999999 -0.106465
+-0.023505100000000001 -0.16736400000000001 -0.11629299999999999
+0.024216399999999999 -0.15465899999999999 -0.069148500000000002
+-0.014351900000000001 -0.17225199999999999 -0.092658000000000004
+-0.010652 0.060463599999999999 -0.18701599999999999
+0.100536 0.224554 0.051798499999999997
+0.114136 0.219745 0.057887399999999999
+0.091313000000000005 0.22528699999999999 0.059526299999999997
+0.079702599999999998 0.227767 0.0581816
+0.0723327 0.22902700000000001 0.0453777
+0.082926899999999998 0.227441 0.030913
+0.066844700000000007 0.229241 0.063203899999999993
+0.114338 0.21690599999999999 0.070553500000000005
+0.103966 0.21777099999999999 0.077286099999999996
+0.27313999999999999 0.090428900000000006 -0.074272900000000003
+0.27865000000000001 0.100969 -0.074016399999999996
+-0.031820099999999997 -0.172097 0.140491
+-0.15099499999999999 0.19156899999999999 0.044903899999999997
+-0.16286100000000001 0.18260999999999999 0.059771600000000001
+-0.181673 0.17237 0.056407400000000003
+-0.19064600000000001 0.17325099999999999 0.036338599999999999
+-0.13341900000000001 0.19962199999999999 0.048712999999999999
+0.30869000000000002 0.14619199999999999 -0.039147000000000001
+0.310112 0.144647 -0.035131500000000003
+0.31212800000000002 0.140484 -0.0337784
+0.31284600000000001 0.137243 -0.037271100000000001
+0.31362600000000002 0.13300000000000001 -0.043947
+0.31150600000000001 0.13802500000000001 -0.0426565
+0.314828 0.12872900000000001 -0.048667599999999998
+-0.39890999999999999 0.035649800000000002 -0.091534000000000004
+-0.41048299999999999 0.034610099999999998 -0.092735100000000001
+-0.39403899999999997 0.042540000000000001 -0.088651099999999997
+-0.087818800000000002 0.21124599999999999 0.089453599999999994
+-0.10222199999999999 0.20704 0.083188100000000001
+-0.105324 0.210562 0.066740999999999995
+-0.119615 0.203259 0.067314700000000005
+-0.0114042 -0.15342700000000001 -0.16265399999999999
+-0.021692400000000001 -0.15275900000000001 -0.162414
+-0.027649699999999999 -0.15523300000000001 -0.15404100000000001
+-0.022919100000000001 -0.159385 -0.14436399999999999
+-0.028222000000000001 -0.162884 -0.13029399999999999
+-0.0091154500000000006 -0.15975500000000001 -0.143127
+0.0038177300000000001 -0.15817400000000001 -0.138235
+0.015862899999999999 -0.15197099999999999 -0.145374
+-0.043041999999999997 -0.15981100000000001 -0.13128300000000001
+-0.0578498 -0.157583 -0.12483
+-0.073061100000000004 -0.15382100000000001 -0.118257
+-0.0918654 -0.14227100000000001 -0.119321
+-0.086545800000000006 0.15179899999999999 -0.110721
+-0.0747559 0.16490299999999999 -0.099600300000000003
+-0.087713100000000002 0.16961100000000001 -0.084725800000000004
+-0.104298 0.14448 -0.108615
+-0.111502 0.130777 -0.117599
+-0.29059800000000002 0.122665 -0.0344017
+-0.28304800000000002 0.12528600000000001 -0.032653799999999997
+-0.27710000000000001 0.13156100000000001 -0.023489699999999999
+-0.273262 0.12407799999999999 -0.037146600000000002
+-0.27542 0.117855 -0.0440148
+-0.18649299999999999 0.17033999999999999 -0.0011688200000000001
+-0.17177000000000001 0.166823 -0.020371400000000001
+-0.18068400000000001 0.153502 -0.035443700000000002
+-0.20599200000000001 0.16261600000000001 -0.0020504799999999999
+-0.198324 0.14709 -0.033816600000000002
+-0.0059202899999999999 0.100685 0.18144099999999999
+-0.0167332 0.118964 0.17482900000000001
+0.015946700000000001 0.103079 0.18060999999999999
+0.028412400000000001 0.084941699999999995 0.18443200000000001
+-0.44251800000000002 -0.070944300000000002 -0.086369100000000004
+-0.44885599999999998 -0.066220100000000004 -0.087481199999999995
+-0.13466700000000001 0.00094315399999999998 0.177011
+-0.00310122 0.23491300000000001 0.062836199999999995
+-0.46231 -0.058624900000000001 -0.040429600000000003
+0.22753799999999999 0.0369226 -0.072126899999999994
+0.236821 0.042365600000000003 -0.066803500000000002
+0.24060500000000001 0.037123200000000002 -0.054989900000000001
+0.22984099999999999 0.0265624 -0.0470128
+-0.46360699999999999 0.028646100000000001 -0.085726200000000002
+-0.45672600000000002 0.027279000000000001 -0.090690900000000005
+-0.469667 0.022226300000000001 -0.084804500000000005
+-0.45082699999999998 0.0237483 -0.094695799999999997
+0.21632100000000001 0.13897799999999999 0.105626
+0.21636 0.17601800000000001 -0.068996000000000002
+0.22472400000000001 0.177449 -0.064092700000000002
+0.23515 0.17280300000000001 -0.066168099999999994
+0.26077800000000001 0.097572000000000006 0.093073299999999998
+0.25369700000000001 0.10379099999999999 0.096948800000000002
+0.00042625600000000002 0.0061583499999999999 0.19870499999999999
+-0.0143538 0.0030840799999999999 0.19675899999999999
+-0.025944800000000001 0.0152079 0.19378100000000001
+0.0049670399999999998 0.019903899999999999 0.19777
+-0.013013200000000001 0.23198099999999999 0.085514800000000002
+-0.00048819800000000002 0.23011499999999999 0.092733999999999997
+-0.486599 0.0044273799999999999 -0.083403900000000003
+-0.48317399999999999 0.0083421499999999996 -0.083037799999999995
+-0.48153099999999999 0.012534999999999999 -0.0769011
+-0.47923100000000002 0.0093157599999999993 -0.087203699999999995
+-0.47913600000000001 0.0130796 -0.082447800000000002
+-0.47456700000000002 0.018227799999999999 -0.082057099999999994
+-0.48859900000000001 0.0026895600000000001 -0.079400200000000004
+-0.48931200000000002 -0.0020909800000000001 -0.078754299999999999
+-0.48787900000000001 0.0045522799999999997 -0.074895199999999995
+-0.48457299999999998 0.0089092100000000007 -0.074025499999999994
+-0.48162100000000002 0.013351099999999999 -0.070635600000000007
+-0.48615700000000001 0.0013552 -0.087000099999999997
+0.074113999999999999 -0.0959285 -0.152444
+0.0788079 -0.084809499999999996 -0.156079
+0.077666100000000002 -0.075125800000000006 -0.16463800000000001
+0.30785699999999999 0.12345 -0.062920500000000004
+0.30563200000000001 0.118432 -0.064303899999999997
+0.306506 0.11278000000000001 -0.061284600000000002
+0.303699 0.10655199999999999 -0.060595099999999999
+0.29678599999999999 0.103863 -0.064865800000000001
+0.29050900000000002 0.098705399999999999 -0.068410899999999997
+0.30475000000000002 0.12803400000000001 -0.064310099999999995
+0.29785099999999998 0.128749 -0.068135699999999993
+-0.33938800000000002 0.107776 -0.030821100000000001
+-0.33142199999999999 0.111016 -0.0225385
+0.1951 0.19827800000000001 0.038898000000000002
+0.20821000000000001 0.19181000000000001 0.040146500000000002
+0.21546899999999999 0.18287600000000001 0.054990900000000002
+0.218888 0.19023799999999999 0.0280165
+0.23067299999999999 0.18829799999999999 0.0132706
+-0.46731400000000001 -0.059894999999999997 -0.0588218
+-0.45948499999999998 -0.067780400000000005 -0.060704399999999999
+0.30712899999999999 0.14621899999999999 -0.0474896
+-0.42225099999999999 0.057628600000000002 -0.076709100000000002
+-0.42860999999999999 0.054164200000000003 -0.076605000000000006
+
+# 4050 facets
+# ------------------------------------------
+
+3  671 417 1634
+3  945 65 944
+3  874 2025 835
+3  1009 997 369
+3  1598 927 1597
+3  1355 128 1357
+3  294 1255 299
+3  266 299 294
+3  1482 1162 146
+3  611 612 32
+3  313 20 312
+3  67 1314 1022
+3  1147 1148 561
+3  119 483 1122
+3  1715 1714 1718
+3  1718 832 1715
+3  782 1117 1648
+3  991 992 367
+3  554 547 546
+3  31 619 576
+3  359 985 650
+3  650 980 359
+3  969 367 992
+3  674 1700 673
+3  181 673 674
+3  660 59 307
+3  854 855 912
+3  1630 1498 589
+3  246 88 1571
+3  1138 1137 349
+3  349 1137 1136
+3  1426 917 576
+3  151 154 37
+3  72 642 643
+3  1285 530 1783
+3  495 494 12
+3  496 12 495
+3  427 202 426
+3  380 1021 1020
+3  192 846 345
+3  1526 118 1411
+3  1810 79 1811
+3  1540 1811 1810
+3  427 202 430
+3  1275 337 827
+3  1518 84 1685
+3  544 759 132
+3  1545 1547 444
+3  1762 1758 687
+3  7 373 416
+3  665 844 1326
+3  1953 665 1326
+3  376 70 378
+3  1325 1339 142
+3  468 187 469
+3  332 681 537
+3  1232 389 1231
+3  240 1651 241
+3  1463 456 44
+3  265 1358 1356
+3  2003 1412 889
+3  628 637 625
+3  184 1900 1896
+3  501 1860 1410
+3  1362 1336 666
+3  1256 1089 1086
+3  1086 1256 1084
+3  495 1821 1824
+3  382 651 1406
+3  406 81 405
+3  1412 1413 1177
+3  70 376 369
+3  239 241 153
+3  602 800 601
+3  1638 800 601
+3  336 827 108
+3  1947 110 273
+3  1188 1189 203
+3  1188 1189 205
+3  870 867 871
+3  1028 380 1020
+3  1020 1028 1017
+3  976 973 151
+3  35 589 1624
+3  1624 589 585
+3  904 187 478
+3  462 187 478
+3  794 1678 170
+3  794 1099 170
+3  958 690 537
+3  151 789 788
+3  802 182 806
+3  1021 1023 1022
+3  802 524 806
+3  558 426 50
+3  1976 1978 879
+3  882 1978 1976
+3  752 751 161
+3  1726 570 1725
+3  1458 460 186
+3  1458 186 1460
+3  207 1844 1852
+3  1852 207 1845
+3  490 1469 1467
+3  1720 1719 896
+3  667 57 668
+3  1870 1871 169
+3  1022 1314 1738
+3  98 1964 1963
+3  780 160 778
+3  57 1096 667
+3  57 1096 1094
+3  377 378 376
+3  1321 158 774
+3  703 64 282
+3  703 64 742
+3  854 1930 858
+3  1767 536 1060
+3  181 676 1656
+3  857 856 354
+3  1018 1008 1015
+3  632 637 1770
+3  632 42 1770
+3  1672 1674 15
+3  422 1013 1011
+3  572 94 668
+3  824 550 34
+3  1244 1012 358
+3  273 51 289
+3  56 1250 1253
+3  1253 1250 297
+3  1636 1178 1101
+3  1102 1178 1636
+3  1093 1098 1097
+3  404 233 178
+3  1137 63 1139
+3  767 53 766
+3  1729 1964 385
+3  479 1522 480
+3  480 479 117
+3  1302 1301 527
+3  527 1302 1303
+3  688 744 962
+3  688 744 1688
+3  613 32 611
+3  740 27 737
+3  1435 27 740
+3  1078 1080 357
+3  1502 406 81
+3  82 327 692
+3  1258 1988 570
+3  1727 1258 570
+3  873 338 853
+3  928 1614 1927
+3  928 1926 1927
+3  543 489 121
+3  1226 1220 386
+3  504 487 505
+3  1218 115 1222
+3  358 1244 983
+3  983 1244 492
+3  408 505 49
+3  879 880 337
+3  181 674 675
+3  1285 530 1283
+3  1846 1186 1847
+3  1292 725 1293
+3  1292 725 1290
+3  147 155 37
+3  158 776 773
+3  150 1668 1669
+3  1086 309 1919
+3  1198 1195 410
+3  161 162 252
+3  980 1769 981
+3  229 1947 1946
+3  214 623 1863
+3  1863 623 1974
+3  1293 726 1288
+3  1288 1293 725
+3  546 312 250
+3  1016 1017 379
+3  619 1204 1203
+3  1217 1224 1218
+3  360 980 650
+3  105 340 1761
+3  307 662 659
+3  658 662 659
+3  1452 490 1471
+3  408 406 1502
+3  1494 0 1492
+3  1955 1956 966
+3  1955 1956 1954
+3  267 245 55
+3  554 26 546
+3  1676 1677 1680
+3  446 1817 77
+3  660 307 659
+3  2007 1595 1593
+3  1208 114 1209
+3  1214 114 1208
+3  91 364 365
+3  1629 588 1630
+3  376 377 89
+3  482 22 486
+3  137 970 646
+3  546 250 26
+3  2005 2004 1340
+3  277 276 164
+3  693 323 956
+3  108 1269 1270
+3  1676 150 1677
+3  177 286 1427
+3  120 138 139
+3  1353 129 1352
+3  285 282 64
+3  1332 1333 343
+3  595 1581 1587
+3  53 768 769
+3  1499 1503 498
+3  435 434 71
+3  822 348 837
+3  823 348 822
+3  637 632 600
+3  628 600 637
+3  486 439 183
+3  1672 798 1674
+3  1624 585 1625
+3  634 597 636
+3  1394 235 1392
+3  171 1391 238
+3  1987 419 1988
+3  754 1916 1687
+3  515 754 1687
+3  1963 1965 372
+3  1054 1053 130
+3  486 22 439
+3  414 80 451
+3  414 11 451
+3  276 277 278
+3  1030 380 1028
+3  332 684 465
+3  1582 580 1498
+3  881 337 1662
+3  1728 208 558
+3  385 1728 558
+3  1897 184 1896
+3  1384 1383 1382
+3  167 15 977
+3  432 430 25
+3  432 430 202
+3  2004 1342 1344
+3  1897 220 1902
+3  1032 556 224
+3  1143 1144 517
+3  1375 324 1374
+3  1330 191 1362
+3  1343 1342 1428
+3  100 1428 1343
+3  737 740 741
+3  1975 1974 623
+3  638 1578 598
+3  1672 594 1673
+3  1673 15 1672
+3  937 938 939
+3  939 609 937
+3  315 966 967
+3  58 966 967
+3  84 1500 1503
+3  1503 84 1685
+3  507 1740 1778
+3  907 1722 898
+3  695 696 327
+3  434 420 71
+3  1330 1362 1363
+3  170 797 796
+3  1098 315 1096
+3  1098 315 1097
+3  821 849 1139
+3  1139 849 1249
+3  563 868 869
+3  636 5 1105
+3  1105 5 1104
+3  1393 234 1394
+3  386 1219 1220
+3  1207 1208 397
+3  397 1208 1212
+3  675 21 674
+3  1628 941 1620
+3  402 1227 1228
+3  1214 1215 114
+3  1214 1215 1216
+3  1034 814 216
+3  1650 88 1649
+3  127 1234 1211
+3  765 133 1150
+3  765 1150 1782
+3  91 1002 364
+3  655 656 313
+3  313 655 653
+3  984 359 985
+3  1454 458 44
+3  2 1153 1154
+3  1154 1153 1155
+3  1525 1526 497
+3  497 1526 1527
+3  524 807 808
+3  1001 8 998
+3  557 195 730
+3  1028 1017 68
+3  409 81 405
+3  1733 36 845
+3  845 36 1134
+3  685 1558 1559
+3  1726 570 204
+3  505 121 413
+3  1894 750 766
+3  44 1454 1453
+3  377 403 378
+3  1654 403 377
+3  527 1301 1296
+3  1067 1301 1296
+3  322 25 321
+3  468 463 83
+3  83 463 1754
+3  1120 1156 174
+3  656 125 1934
+3  1353 1354 129
+3  1544 78 1543
+3  1946 1947 226
+3  1372 1373 696
+3  231 1941 1942
+3  450 1529 1528
+3  1528 1508 450
+3  1404 91 1403
+3  1409 85 1408
+3  1409 85 1553
+3  744 961 1742
+3  555 223 556
+3  973 974 976
+3  173 257 749
+3  748 1345 331
+3  129 715 1359
+3  1088 425 1079
+3  1441 1435 740
+3  1007 368 1006
+3  1010 1906 69
+3  1038 1743 1034
+3  382 993 1014
+3  109 1039 1033
+3  430 203 1188
+3  1973 1974 625
+3  1332 841 842
+3  1332 343 842
+3  505 413 122
+3  358 1011 1012
+3  436 76 435
+3  260 1950 1949
+3  1917 311 1918
+3  306 1917 1918
+3  299 266 1962
+3  1086 1919 304
+3  110 228 229
+3  185 681 682
+3  459 1828 1457
+3  459 1828 1458
+3  373 372 1966
+3  1334 191 1333
+3  1475 883 1692
+3  75 1563 1058
+3  1220 383 1221
+3  994 366 991
+3  999 364 365
+3  367 1005 1031
+3  969 367 1031
+3  1404 1403 374
+3  424 208 426
+3  269 227 270
+3  1986 1987 1258
+3  1695 953 1693
+3  588 1498 934
+3  1806 1805 1807
+3  138 120 195
+3  1180 591 1179
+3  646 643 644
+3  64 743 742
+3  742 743 438
+3  1094 1436 303
+3  4 713 711
+3  291 713 710
+3  82 405 409
+3  1155 526 1154
+3  695 327 693
+3  93 1367 1365
+3  730 1435 1441
+3  3 543 413
+3  1963 98 1965
+3  407 697 325
+3  1349 320 1348
+3  1936 1937 1035
+3  1036 1937 1936
+3  523 1143 517
+3  453 1536 1812
+3  1000 363 1003
+3  240 720 721
+3  240 720 719
+3  390 1654 16
+3  159 780 781
+3  52 672 680
+3  611 610 612
+3  834 336 828
+3  2018 381 2019
+3  1002 1001 363
+3  995 8 1000
+3  669 1776 1774
+3  371 363 98
+3  1562 79 1826
+3  1562 79 1542
+3  357 1078 1012
+3  174 791 790
+3  628 625 1863
+3  63 866 1138
+3  354 857 1452
+3  775 1496 1495
+3  1538 452 1771
+3  421 275 234
+3  19 1082 1083
+3  313 312 661
+3  1034 814 222
+3  970 1171 1173
+3  111 1942 1899
+3  757 759 132
+3  407 122 410
+3  1816 1817 1815
+3  565 179 566
+3  1132 1142 1128
+3  1952 264 718
+3  1240 393 127
+3  271 228 270
+3  1430 100 1429
+3  228 1730 1731
+3  1272 1264 1265
+3  110 273 272
+3  43 698 699
+3  283 698 699
+3  779 162 161
+3  438 545 24
+3  627 213 617
+3  617 213 935
+3  1208 1214 1212
+3  1212 1214 398
+3  1221 1220 16
+3  899 897 1971
+3  1220 16 1219
+3  1507 1854 1548
+3  1218 1216 398
+3  20 652 653
+3  86 1567 1568
+3  570 204 569
+3  443 442 117
+3  81 415 414
+3  1218 1217 1216
+3  788 151 154
+3  1904 257 791
+3  1166 145 1160
+3  1167 145 1166
+3  274 227 271
+3  368 1018 1913
+3  543 542 544
+3  846 345 847
+3  550 212 551
+3  275 227 421
+3  370 1027 1910
+3  494 493 704
+3  232 284 562
+3  548 652 20
+3  999 998 364
+3  1087 1088 431
+3  63 866 867
+3  256 171 255
+3  990 994 989
+3  571 251 163
+3  1273 888 1865
+3  1865 888 1864
+3  1904 791 174
+3  715 717 716
+3  307 59 302
+3  467 1451 1450
+3  262 250 124
+3  248 795 796
+3  1356 59 1358
+3  1614 2024 1615
+3  729 950 1291
+3  729 1290 1291
+3  1950 260 1747
+3  362 1003 1000
+3  996 995 366
+3  565 179 420
+3  51 420 565
+3  1317 915 1316
+3  1017 1016 1019
+3  1017 1019 68
+3  482 479 22
+3  942 133 755
+3  755 133 1150
+3  1944 225 1945
+3  1012 201 1011
+3  1067 1309 1308
+3  729 1298 950
+3  729 1298 1297
+3  1093 1098 1436
+3  1094 1098 1436
+3  1295 952 1294
+3  1102 1636 1637
+3  89 383 1221
+3  91 1966 373
+3  738 739 741
+3  738 741 737
+3  292 715 712
+3  1767 1060 488
+3  644 646 30
+3  1232 1219 16
+3  360 986 987
+3  1431 1430 1429
+3  195 27 196
+3  557 645 136
+3  557 138 645
+3  1169 1165 1170
+3  1393 276 164
+3  636 634 168
+3  509 815 1440
+3  87 1570 789
+3  554 212 547
+3  562 277 10
+3  676 1903 184
+3  1297 1067 1296
+3  546 312 20
+3  799 1671 1100
+3  1670 1671 799
+3  1033 1034 1743
+3  1343 218 1342
+3  1344 218 1342
+3  1042 1043 510
+3  250 1434 262
+3  1854 1506 1507
+3  95 1263 1106
+3  887 1263 1106
+3  402 90 947
+3  1433 1432 249
+3  1225 1224 386
+3  943 585 944
+3  1743 1038 215
+3  1772 1773 1741
+3  484 1772 1741
+3  1175 1176 120
+3  120 1175 143
+3  1358 660 261
+3  1884 1539 1535
+3  1535 1539 453
+3  1052 536 1051
+3  147 37 148
+3  637 625 1125
+3  1125 625 1126
+3  638 633 597
+3  627 214 620
+3  456 458 44
+3  1872 1130 1867
+3  1568 259 1949
+3  716 243 717
+3  61 1113 1256
+3  1091 1092 60
+3  1961 1092 60
+3  245 716 243
+3  1811 1542 1539
+3  79 1542 1811
+3  285 286 177
+3  1870 1882 1880
+3  1316 1235 1236
+3  718 243 717
+3  1905 1907 370
+3  634 168 1871
+3  310 1256 1084
+3  1937 1938 109
+3  269 163 421
+3  743 437 64
+3  386 1224 1217
+3  752 751 172
+3  227 270 271
+3  269 163 268
+3  1 252 251
+3  252 251 162
+3  1949 128 1948
+3  345 192 842
+3  345 342 842
+3  1868 1129 1867
+3  1868 1129 626
+3  168 1871 1673
+3  770 37 154
+3  1142 978 624
+3  1128 978 1142
+3  120 143 139
+3  1321 774 1323
+3  462 1446 478
+3  751 1894 1892
+3  1929 858 862
+3  244 40 245
+3  228 1730 270
+3  1701 289 180
+3  717 1352 129
+3  673 181 680
+3  1576 1578 598
+3  1797 1541 1798
+3  1798 1797 1796
+3  160 812 813
+3  1701 180 1700
+3  356 1935 311
+3  48 442 441
+3  441 444 48
+3  1729 1003 209
+3  2001 1994 884
+3  652 314 649
+3  957 691 955
+3  836 348 869
+3  702 283 700
+3  118 1522 1521
+3  1959 301 1958
+3  54 705 1252
+3  489 541 542
+3  220 1895 1697
+3  284 282 10
+3  304 1090 1091
+3  1357 128 1950
+3  1820 198 1819
+3  1552 476 1551
+3  1024 608 607
+3  172 771 772
+3  690 537 332
+3  606 1157 553
+3  464 185 465
+3  86 262 260
+3  1716 907 886
+3  1722 907 1716
+3  1013 424 422
+3  187 462 463
+3  1502 1501 1517
+3  1476 1470 1473
+3  1260 417 1689
+3  343 842 342
+3  1834 903 902
+3  1931 563 868
+3  815 1058 816
+3  509 815 816
+3  751 1893 161
+3  741 728 1441
+3  741 728 739
+3  75 818 817
+3  1043 131 1048
+3  886 1106 907
+3  489 542 543
+3  730 731 557
+3  532 1841 1309
+3  235 251 571
+3  425 424 426
+3  429 428 204
+3  151 789 792
+3  451 954 11
+3  461 954 11
+3  39 172 772
+3  776 773 772
+3  162 254 251
+3  251 163 254
+3  1478 1229 400
+3  531 1281 1633
+3  854 912 852
+3  111 1895 1898
+3  548 547 20
+3  406 33 407
+3  1809 1825 1457
+3  1826 1825 1457
+3  258 153 239
+3  604 9 601
+3  1279 530 1278
+3  1431 144 1286
+3  1119 1118 230
+3  219 218 1116
+3  1343 218 219
+3  375 1291 952
+3  837 348 836
+3  1612 1924 1925
+3  238 244 40
+3  331 961 745
+3  173 256 257
+3  542 1439 541
+3  783 156 785
+3  390 126 391
+3  432 431 202
+3  1385 236 1386
+3  460 461 1465
+3  768 156 767
+3  1172 138 139
+3  770 768 154
+3  44 1451 1453
+3  714 4 294
+3  294 714 293
+3  475 1785 1659
+3  1894 172 751
+3  515 1143 1147
+3  1047 1046 510
+3  792 87 789
+3  792 87 793
+3  406 408 407
+3  1518 84 1513
+3  455 1800 1802
+3  1808 458 353
+3  281 282 285
+3  172 766 1894
+3  1530 450 1840
+3  1895 219 1697
+3  763 764 3
+3  1847 321 1187
+3  1186 321 1847
+3  1316 1235 573
+3  1316 573 1317
+3  51 420 286
+3  398 1212 1223
+3  997 998 8
+3  568 206 1192
+3  1192 568 1282
+3  1163 62 1881
+3  1881 1066 1163
+3  423 99 422
+3  626 1126 1127
+3  1845 1185 1366
+3  1186 1845 1366
+3  326 94 317
+3  1969 1190 206
+3  170 1679 797
+3  796 249 797
+3  547 546 20
+3  288 178 290
+3  254 17 813
+3  1120 749 1891
+3  482 483 118
+3  362 1000 995
+3  779 752 161
+3  624 1975 30
+3  624 30 646
+3  1716 886 1717
+3  1745 1746 188
+3  1266 1746 1745
+3  1331 191 1330
+3  1901 231 1900
+3  879 1997 1978
+3  795 87 793
+3  1475 1472 855
+3  494 701 704
+3  121 504 505
+3  288 701 704
+3  1199 924 577
+3  1924 1618 930
+3  1924 1618 1923
+3  1849 1193 1194
+3  1194 1849 1850
+3  1885 722 1534
+3  435 439 434
+3  1156 787 769
+3  280 177 1427
+3  521 1280 1279
+3  1850 1195 1849
+3  47 863 862
+3  47 863 864
+3  896 1720 1721
+3  38 1114 1115
+3  1641 1114 38
+3  1056 512 1053
+3  1720 832 1722
+3  1147 1144 1143
+3  1144 1145 2
+3  141 1132 1875
+3  1826 1562 1827
+3  606 1680 602
+3  45 1805 1807
+3  21 1945 1698
+3  1043 131 1044
+3  217 1118 1943
+3  67 1063 946
+3  1318 1317 915
+3  1046 511 1051
+3  1315 9 604
+3  1474 1469 902
+3  253 171 255
+3  136 731 732
+3  99 423 972
+3  197 196 1430
+3  446 444 77
+3  958 691 957
+3  1814 454 46
+3  783 785 784
+3  0 785 784
+3  673 680 672
+3  115 1227 1228
+3  1623 1624 584
+3  21 1944 1940
+3  567 569 204
+3  1007 368 1913
+3  986 988 987
+3  668 57 316
+3  5 1587 1583
+3  1577 1583 5
+3  1568 86 260
+3  1567 248 1432
+3  828 336 827
+3  1733 36 844
+3  200 1823 1822
+3  914 1177 913
+3  112 221 197
+3  786 37 770
+3  1786 1784 1635
+3  1635 1788 1786
+3  1430 100 221
+3  766 767 771
+3  771 767 783
+3  531 1281 1283
+3  1038 215 1114
+3  1939 1036 1938
+3  186 460 461
+3  181 676 675
+3  1035 1937 109
+3  672 673 180
+3  226 1947 273
+3  226 289 273
+3  229 1041 225
+3  811 216 812
+3  750 1890 1892
+3  158 1704 774
+3  1952 718 719
+3  1699 226 1698
+3  807 806 524
+3  1040 225 1037
+3  511 1045 1044
+3  164 277 1383
+3  848 1735 1245
+3  848 1735 1737
+3  1854 1509 48
+3  270 223 1730
+3  277 278 10
+3  415 81 1502
+3  216 1038 1034
+3  510 815 1042
+3  1058 815 510
+3  462 1460 1446
+3  1806 458 1807
+3  237 1389 1390
+3  1538 78 1543
+3  1538 1530 1543
+3  847 846 346
+3  847 1395 346
+3  854 564 858
+3  1357 261 1950
+3  1540 1803 453
+3  276 278 176
+3  1393 276 234
+3  70 369 969
+3  992 369 969
+3  1385 235 1386
+3  342 18 340
+3  74 517 522
+3  679 680 52
+3  797 26 249
+3  21 1940 1901
+3  792 793 149
+3  792 149 973
+3  142 1324 1323
+3  1960 1959 59
+3  1960 1356 59
+3  1895 1896 111
+3  522 1045 1049
+3  419 1182 568
+3  621 620 31
+3  642 644 643
+3  1259 1987 1986
+3  1170 1173 1171
+3  747 955 957
+3  1319 1322 1320
+3  1578 1577 597
+3  269 421 227
+3  875 874 834
+3  176 272 274
+3  1521 480 1520
+3  1746 188 1109
+3  271 272 274
+3  1945 225 229
+3  126 391 392
+3  297 711 4
+3  230 1118 1943
+3  675 1901 21
+3  2022 900 2023
+3  1897 184 678
+3  438 437 743
+3  24 437 438
+3  46 1816 1819
+3  381 1029 1030
+3  201 1013 1011
+3  1748 1559 686
+3  107 1559 686
+3  1087 1079 1080
+3  1587 1581 1583
+3  268 163 254
+3  1325 1286 1339
+3  152 246 1112
+3  1569 246 152
+3  1397 1395 346
+3  1392 1384 1385
+3  1130 141 1874
+3  326 328 28
+3  1403 91 373
+3  223 555 268
+3  662 302 307
+3  720 719 243
+3  554 26 552
+3  1714 892 893
+3  491 1476 1473
+3  788 152 787
+3  1748 685 1559
+3  40 721 239
+3  1681 1677 150
+3  292 713 710
+3  712 710 292
+3  120 195 1176
+3  1638 1637 66
+3  1639 1637 1638
+3  1572 1568 247
+3  1347 694 1346
+3  323 1346 1347
+3  1484 1161 1482
+3  270 223 269
+3  359 980 981
+3  981 359 983
+3  171 258 256
+3  1 253 252
+3  1184 1371 1196
+3  1184 1196 1364
+3  754 753 515
+3  1059 753 754
+3  1356 266 1960
+3  1962 266 1960
+3  1695 1693 1174
+3  1174 1695 140
+3  933 934 1498
+3  1880 1882 1881
+3  16 1654 377
+3  258 238 171
+3  1385 1392 235
+3  1530 1840 1533
+3  1163 1873 62
+3  1878 1873 62
+3  866 350 496
+3  1246 849 1247
+3  1949 1948 259
+3  653 314 654
+3  296 1736 1957
+3  296 1957 1376
+3  1354 293 266
+3  294 293 266
+3  708 291 711
+3  711 708 297
+3  1890 749 173
+3  165 244 267
+3  102 526 724
+3  1288 102 724
+3  281 278 279
+3  271 228 110
+3  405 696 327
+3  1392 1393 1394
+3  186 462 463
+3  186 462 1460
+3  271 272 110
+3  1107 95 899
+3  275 274 227
+3  275 176 274
+3  278 176 279
+3  273 272 280
+3  317 94 668
+3  668 317 316
+3  276 275 176
+3  687 1763 329
+3  329 687 1688
+3  1742 1688 329
+3  1772 1773 669
+3  275 276 234
+3  1801 1462 1461
+3  1807 1808 458
+3  463 1755 1754
+3  280 176 272
+3  279 176 280
+3  565 180 566
+3  534 1259 1631
+3  1689 1259 1631
+3  549 550 212
+3  447 1536 1537
+3  703 283 702
+3  702 703 282
+3  280 279 177
+3  1818 1816 198
+3  1916 516 756
+3  532 1308 1307
+3  267 244 245
+3  189 1833 1968
+3  178 284 287
+3  222 1032 1033
+3  1033 1032 1039
+3  1340 1286 1428
+3  441 442 76
+3  1544 1546 1545
+3  1545 78 1544
+3  1230 388 1225
+3  314 985 649
+3  652 649 548
+3  232 178 284
+3  1766 1555 1556
+3  1507 445 1548
+3  1507 445 1528
+3  714 713 292
+3  319 318 92
+3  92 319 322
+3  870 298 871
+3  781 1642 1062
+3  1447 1444 1445
+3  405 33 696
+3  193 850 875
+3  1450 44 1451
+3  318 326 317
+3  516 942 1916
+3  55 712 710
+3  1568 1567 247
+3  299 58 968
+3  764 411 413
+3  265 1355 1353
+3  346 846 1400
+3  1451 1452 467
+3  301 1961 300
+3  301 1961 1958
+3  665 844 1335
+3  1377 844 1335
+3  716 245 55
+3  11 409 414
+3  920 919 612
+3  612 919 610
+3  872 869 563
+3  436 435 71
+3  295 4 294
+3  1893 255 161
+3  866 496 867
+3  291 709 708
+3  714 4 713
+3  429 1190 1189
+3  659 124 308
+3  988 990 210
+3  685 1750 1751
+3  417 1300 1261
+3  864 863 823
+3  312 124 250
+3  318 28 326
+3  124 312 308
+3  654 653 655
+3  872 853 873
+3  1948 1951 128
+3  356 1081 1082
+3  1275 1274 826
+3  826 1275 1276
+3  436 437 24
+3  24 436 440
+3  337 878 879
+3  1327 1736 1957
+3  552 551 97
+3  212 551 552
+3  304 1089 1090
+3  916 917 113
+3  916 917 574
+3  1197 1843 1364
+3  1197 1843 1844
+3  1087 1081 1080
+3  19 1081 1087
+3  407 410 325
+3  93 1368 1367
+3  105 341 326
+3  326 105 328
+3  208 423 422
+3  4 295 297
+3  1647 1117 1648
+3  979 360 980
+3  695 1347 693
+3  696 33 697
+3  1893 256 173
+3  1588 1779 1590
+3  1588 41 1590
+3  438 698 545
+3  1092 1437 303
+3  1095 305 316
+3  493 494 12
+3  483 486 183
+3  482 486 483
+3  119 183 483
+3  1572 88 1650
+3  489 539 541
+3  1069 1068 400
+3  1069 400 1477
+3  1042 1043 1048
+3  1042 514 1048
+3  531 1283 1284
+3  3 763 520
+3  1276 827 108
+3  94 341 326
+3  334 1556 335
+3  571 421 234
+3  1665 925 924
+3  1785 1659 1141
+3  1931 862 868
+3  1418 1866 885
+3  1389 165 1390
+3  859 858 862
+3  347 336 335
+3  229 110 1947
+3  1139 1249 870
+3  1447 477 1444
+3  1444 477 1442
+3  1931 862 1929
+3  1931 563 1929
+3  247 1571 1572
+3  1531 1533 448
+3  1840 1533 1531
+3  1500 481 1499
+3  1503 1499 1500
+3  859 564 861
+3  346 1400 1399
+3  839 1400 1399
+3  1453 1454 857
+3  1571 88 1572
+3  1860 1859 1410
+3  1248 870 298
+3  1174 1175 1693
+3  143 1175 1174
+3  824 551 550
+3  1134 1135 821
+3  1751 1750 684
+3  191 1333 1332
+3  1121 6 2019
+3  1890 1892 173
+3  1348 1346 694
+3  1183 1348 694
+3  1761 1760 334
+3  18 340 334
+3  857 353 861
+3  861 353 860
+3  790 242 1112
+3  861 856 857
+3  571 421 163
+3  1270 335 108
+3  652 314 653
+3  678 1897 1902
+3  1276 826 1269
+3  108 1269 1276
+3  856 861 564
+3  683 83 1111
+3  291 711 713
+3  709 710 404
+3  979 360 986
+3  450 1508 80
+3  1207 920 1858
+3  396 1207 1858
+3  1024 69 1027
+3  222 556 1032
+3  1684 1683 488
+3  1229 115 1228
+3  411 410 23
+3  991 990 210
+3  362 995 994
+3  989 988 361
+3  159 1319 1643
+3  1225 1226 386
+3  388 1226 1225
+3  1598 1599 925
+3  1965 1966 371
+3  1133 1142 1132
+3  141 1133 1132
+3  471 1834 903
+3  1647 1116 218
+3  1141 1657 1658
+3  1660 1657 1658
+3  1027 1906 69
+3  1312 949 1311
+3  1904 257 749
+3  990 994 991
+3  824 551 1909
+3  428 384 1726
+3  1213 1212 397
+3  1213 1212 1223
+3  395 1853 127
+3  1224 1218 115
+3  1227 1224 115
+3  877 829 878
+3  203 428 429
+3  1189 203 429
+3  392 393 126
+3  1739 679 513
+3  1853 393 126
+3  1219 1232 387
+3  1801 1461 457
+3  457 1798 1801
+3  1982 1104 1983
+3  1982 1181 1983
+3  1232 390 16
+3  194 644 175
+3  194 611 175
+3  55 267 233
+3  710 55 404
+3  889 1414 1412
+3  1564 87 795
+3  915 1238 1421
+3  1289 1296 729
+3  1297 1296 729
+3  1421 1238 1420
+3  1877 1869 1878
+3  1877 1878 640
+3  1233 389 395
+3  1109 1108 469
+3  388 559 90
+3  90 559 1405
+3  1222 1223 398
+3  1217 387 1219
+3  1217 386 1219
+3  612 397 1213
+3  1213 32 612
+3  375 949 948
+3  375 949 950
+3  45 1808 1807
+3  33 405 406
+3  451 1531 450
+3  454 46 1822
+3  179 439 183
+3  414 80 415
+3  504 506 485
+3  408 505 122
+3  408 122 407
+3  1550 1551 1505
+3  507 1774 1775
+3  1777 507 1775
+3  1368 1369 1183
+3  132 542 544
+3  704 288 705
+3  1115 1842 217
+3  993 367 1004
+3  1153 1151 528
+3  1148 1151 1153
+3  504 506 503
+3  411 413 122
+3  122 411 410
+3  1820 199 1821
+3  448 1533 1534
+3  1534 448 1464
+3  1546 445 1545
+3  332 185 465
+3  1518 1512 1513
+3  747 955 323
+3  81 414 409
+3  781 1062 811
+3  566 180 672
+3  908 913 1414
+3  209 1729 1728
+3  429 567 204
+3  1197 1196 1364
+3  424 422 208
+3  356 1081 1080
+3  1372 696 697
+3  322 25 433
+3  1079 1078 201
+3  1082 19 1081
+3  203 427 430
+3  202 425 426
+3  1191 206 1192
+3  321 25 1187
+3  1187 1188 205
+3  205 1193 1194
+3  1190 1194 205
+3  310 433 432
+3  432 431 310
+3  433 25 432
+3  202 431 425
+3  1365 1862 1861
+3  1861 1365 1367
+3  319 321 322
+3  545 698 43
+3  233 404 55
+3  420 179 434
+3  1160 1485 1484
+3  1873 640 1872
+3  563 872 853
+3  707 709 708
+3  1355 265 1358
+3  968 58 967
+3  1903 184 1900
+3  1341 142 1339
+3  1689 1986 1259
+3  268 17 555
+3  1108 899 1107
+3  77 441 440
+3  1107 473 1264
+3  1264 1107 1263
+3  1273 474 1272
+3  1266 473 1265
+3  1265 473 1264
+3  76 440 436
+3  1828 1827 1457
+3  1828 1827 1459
+3  76 442 443
+3  343 1686 1333
+3  1522 482 118
+3  444 77 441
+3  1524 1510 476
+3  1524 1510 1511
+3  439 179 434
+3  76 435 443
+3  22 439 435
+3  435 22 443
+3  443 22 479
+3  464 1753 1754
+3  1529 445 1546
+3  1529 445 1528
+3  10 282 281
+3  1816 1817 1818
+3  1524 476 1504
+3  185 461 1756
+3  464 1756 185
+3  1794 1793 457
+3  1552 476 1504
+3  80 450 451
+3  1469 1470 490
+3  2 1154 1145
+3  501 1408 1407
+3  1804 456 1806
+3  1529 1546 1544
+3  1817 77 1818
+3  1815 446 1817
+3  49 487 505
+3  199 1822 1823
+3  515 523 753
+3  1143 523 515
+3  1450 466 44
+3  461 1756 186
+3  1559 1558 1557
+3  1559 1557 107
+3  1515 1514 1517
+3  83 1753 1754
+3  876 834 828
+3  1696 1140 71
+3  747 331 745
+3  1808 350 45
+3  780 781 811
+3  1382 1389 165
+3  1805 1823 200
+3  454 1803 200
+3  200 1803 1802
+3  1826 1797 79
+3  1826 1797 1825
+3  460 1465 1464
+3  1464 1465 448
+3  1446 1466 190
+3  690 106 958
+3  245 243 720
+3  1755 1756 186
+3  1537 78 1538
+3  1472 491 1475
+3  463 187 468
+3  190 1445 1447
+3  1558 344 1557
+3  1748 1757 1749
+3  686 1757 1748
+3  909 905 852
+3  683 1111 344
+3  1930 853 563
+3  909 829 905
+3  988 990 989
+3  1724 567 206
+3  1520 1521 1525
+3  1158 1157 552
+3  890 1419 889
+3  898 907 95
+3  933 1780 1589
+3  1396 837 1397
+3  1396 1395 1397
+3  1463 1461 466
+3  1536 1812 1813
+3  1505 1504 1552
+3  1551 1552 1505
+3  499 1553 1409
+3  487 503 500
+3  480 1522 1521
+3  469 904 899
+3  899 904 1971
+3  1041 225 1040
+3  952 401 951
+3  1120 53 1891
+3  1502 49 1501
+3  119 183 648
+3  1822 1819 46
+3  162 813 779
+3  3 544 520
+3  354 1452 1471
+3  390 391 1654
+3  1978 830 882
+3  1419 892 890
+3  709 290 707
+3  1386 235 1
+3  1432 249 796
+3  1933 357 1080
+3  211 1991 1406
+3  1990 1991 211
+3  288 706 290
+3  863 348 823
+3  868 348 863
+3  178 404 290
+3  295 294 1255
+3  71 436 437
+3  1465 461 954
+3  1795 1809 1466
+3  1501 502 49
+3  487 502 49
+3  825 1478 1477
+3  209 423 1728
+3  487 503 504
+3  1365 1185 1366
+3  487 500 502
+3  1502 408 49
+3  1129 1128 1132
+3  1875 1132 1129
+3  806 807 538
+3  513 1056 1057
+3  449 1516 1532
+3  1508 1516 1532
+3  500 502 481
+3  1059 754 756
+3  540 504 485
+3  1372 1371 697
+3  1781 933 1580
+3  1776 506 1774
+3  267 165 233
+3  121 504 540
+3  1148 2 1153
+3  1803 455 1540
+3  1679 1678 603
+3  1677 1678 603
+3  780 160 811
+3  759 516 756
+3  757 759 756
+3  1438 1440 1439
+3  75 1061 818
+3  517 523 522
+3  489 539 540
+3  544 759 760
+3  760 516 759
+3  1903 675 1900
+3  772 157 771
+3  1763 329 1764
+3  522 74 512
+3  766 53 750
+3  1126 1128 626
+3  626 1128 1129
+3  1282 1182 104
+3  1188 25 430
+3  1187 25 1188
+3  521 764 763
+3  325 1197 1198
+3  325 410 1198
+3  804 738 135
+3  1675 15 1674
+3  1049 522 1050
+3  74 809 517
+3  1182 568 1282
+3  1431 144 1176
+3  1272 474 887
+3  1751 683 1752
+3  1279 1280 104
+3  526 518 102
+3  819 508 817
+3  2 1147 1148
+3  1308 1307 533
+3  1224 1225 1227
+3  1227 1225 1230
+3  1071 952 1294
+3  1071 13 1294
+3  121 540 489
+3  1124 1123 119
+3  508 1777 1778
+3  1644 1645 1117
+3  1644 1645 38
+3  540 485 539
+3  67 604 1314
+3  244 165 1390
+3  755 942 1916
+3  257 1497 256
+3  1393 1392 164
+3  239 238 258
+3  239 40 238
+3  268 223 269
+3  236 1386 1388
+3  1384 164 1392
+3  1390 244 238
+3  169 1871 634
+3  926 1597 925
+3  39 752 778
+3  1478 1229 399
+3  399 1479 1478
+3  1 235 251
+3  254 17 268
+3  131 1050 753
+3  254 813 162
+3  812 813 17
+3  1039 109 1040
+3  221 112 220
+3  229 1041 228
+3  1071 401 952
+3  1291 950 375
+3  1652 242 1653
+3  1350 28 318
+3  770 156 768
+3  556 223 1732
+3  708 54 1251
+3  297 1251 708
+3  960 961 745
+3  745 959 960
+3  1367 1369 1368
+3  559 1404 1405
+3  682 11 409
+3  682 11 461
+3  185 682 461
+3  681 682 82
+3  409 682 82
+3  413 121 543
+3  1026 2018 2020
+3  1255 58 299
+3  1328 58 1255
+3  745 957 747
+3  1843 1185 1862
+3  1851 1185 1843
+3  314 984 985
+3  549 212 547
+3  689 963 964
+3  964 963 688
+3  1150 1782 529
+3  529 1149 1150
+3  212 552 554
+3  1103 1101 1178
+3  1103 1178 1981
+3  553 1158 797
+3  1712 1719 896
+3  366 995 994
+3  1157 607 97
+3  752 172 39
+3  719 240 263
+3  674 1699 1698
+3  112 220 1902
+3  1841 1634 670
+3  1841 1634 671
+3  1067 533 1301
+3  767 53 768
+3  524 519 804
+3  776 39 777
+3  148 977 974
+3  1542 722 1884
+3  1539 1884 1542
+3  1565 1570 1569
+3  1865 1274 1273
+3  1904 174 1120
+3  749 1904 1120
+3  1600 1664 1601
+3  598 638 631
+3  1785 1271 1784
+3  1075 1074 1072
+3  1322 1319 159
+3  1414 913 1412
+3  1618 941 1619
+3  728 731 732
+3  1677 603 1680
+3  20 653 313
+3  906 1690 833
+3  1679 553 603
+3  1424 921 922
+3  583 938 939
+3  1935 1921 311
+3  1489 140 1487
+3  1487 1489 1483
+3  106 960 962
+3  963 106 962
+3  1431 1429 1286
+3  26 1434 249
+3  1433 1434 249
+3  1684 1682 1123
+3  1876 626 1127
+3  639 1127 1876
+3  1666 149 793
+3  794 793 1666
+3  974 977 1675
+3  1913 1008 1912
+3  1264 887 1263
+3  959 960 106
+3  1131 1130 1159
+3  1872 1130 1159
+3  139 1165 1170
+3  971 970 1133
+3  1173 970 971
+3  814 812 216
+3  969 1018 1031
+3  1830 189 1829
+3  115 1229 1222
+3  1404 374 1405
+3  1230 90 402
+3  1666 149 975
+3  1418 1866 1416
+3  632 42 633
+3  398 1218 1222
+3  550 560 549
+3  614 194 616
+3  830 1177 914
+3  644 194 642
+3  1287 1288 725
+3  725 1287 1289
+3  1515 1514 1513
+3  1513 1076 1515
+3  982 1011 422
+3  1443 1468 477
+3  1442 477 1443
+3  388 90 1230
+3  383 1220 1226
+3  141 1874 1875
+3  1231 389 1233
+3  1002 371 91
+3  1001 1002 364
+3  1028 68 1029
+3  9 601 602
+3  34 1004 1005
+3  1450 1449 1468
+3  621 644 175
+3  1697 219 1343
+3  68 1019 403
+3  584 1624 1625
+3  379 1015 1008
+3  1031 368 1018
+3  394 1243 940
+3  194 613 611
+3  996 997 369
+3  997 998 1009
+3  376 1009 369
+3  694 1374 1183
+3  1263 1107 95
+3  195 138 557
+3  195 27 1435
+3  1126 1125 1127
+3  41 1575 1574
+3  1172 1171 139
+3  1172 137 645
+3  1828 1459 460
+3  1464 1459 460
+3  401 947 1068
+3  401 947 951
+3  818 508 1778
+3  1780 933 1781
+3  1410 483 501
+3  483 501 1122
+3  1677 1681 1678
+3  97 608 1910
+3  1910 551 97
+3  1072 736 1073
+3  1070 1069 116
+3  1653 1313 153
+3  1688 1742 744
+3  520 762 761
+3  388 365 559
+3  1663 1664 29
+3  814 555 222
+3  556 555 222
+3  558 384 385
+3  641 1160 1162
+3  1098 1096 1094
+3  561 755 1150
+3  1150 561 1149
+3  1062 216 811
+3  345 347 847
+3  1164 143 1165
+3  1164 1168 1165
+3  1795 1456 1796
+3  544 760 520
+3  562 284 10
+3  1353 1354 265
+3  1444 901 478
+3  1444 478 1445
+3  1251 298 1252
+3  1251 1252 54
+3  236 1385 1384
+3  47 859 862
+3  850 834 875
+3  1075 825 116
+3  264 1352 1353
+3  206 567 429
+3  1374 324 1370
+3  1610 1598 1599
+3  1184 1364 1861
+3  183 179 566
+3  443 479 117
+3  565 51 289
+3  180 565 289
+3  470 901 478
+3  470 901 2023
+3  867 63 870
+3  1376 1737 1254
+3  853 854 1930
+3  821 1135 1136
+3  1335 1331 1330
+3  1330 1335 665
+3  6 2020 2021
+3  31 621 576
+3  1480 1481 32
+3  1832 1721 897
+3  897 1832 2022
+3  635 169 634
+3  1243 1623 583
+3  615 1073 735
+3  1288 724 1287
+3  930 1611 1626
+3  1612 1611 930
+3  636 5 1577
+3  1165 1169 1168
+3  31 1206 1205
+3  31 1206 620
+3  641 1872 1159
+3  785 786 156
+3  583 1623 1622
+3  175 610 576
+3  175 576 621
+3  1663 1665 1599
+3  1179 591 1178
+3  1159 1131 1166
+3  592 945 946
+3  1888 66 1887
+3  1886 1887 1888
+3  793 1099 794
+3  1646 1116 1115
+3  1115 1116 217
+3  1565 1564 1566
+3  616 642 194
+3  632 638 631
+3  157 783 784
+3  1209 396 1207
+3  1209 396 1984
+3  1693 1694 1175
+3  1066 146 1065
+3  633 635 634
+3  634 597 633
+3  1675 15 977
+3  1672 594 1671
+3  578 1593 1595
+3  942 516 760
+3  593 1105 1103
+3  1611 936 1626
+3  640 1867 1868
+3  87 1565 1570
+3  1668 1670 1669
+3  1714 894 1713
+3  1305 1307 533
+3  1537 78 447
+3  606 602 9
+3  1577 1578 1576
+3  596 1577 1576
+3  97 607 608
+3  1709 892 1710
+3  1710 890 892
+3  1069 401 1705
+3  103 1067 1297
+3  735 736 13
+3  735 736 1073
+3  1240 1241 394
+3  575 1424 1202
+3  918 919 610
+3  918 919 113
+3  1242 584 1243
+3  194 613 614
+3  1514 84 1513
+3  1500 84 1514
+3  601 1640 604
+3  947 90 948
+3  1695 1490 1495
+3  1695 1490 140
+3  72 136 733
+3  72 733 734
+3  101 733 734
+3  1591 928 932
+3  629 600 628
+3  629 600 630
+3  1205 1200 618
+3  1561 127 1234
+3  1561 127 1240
+3  1508 1528 1532
+3  1104 1105 1103
+3  1972 978 1973
+3  1974 978 1973
+3  585 943 589
+3  622 30 621
+3  1493 1065 1492
+3  1064 1483 1489
+3  1489 1064 1491
+3  1242 1241 394
+3  635 42 633
+3  644 30 621
+3  130 1055 647
+3  137 1171 1172
+3  642 616 72
+3  786 37 155
+3  627 629 628
+3  622 214 620
+3  724 1287 1303
+3  632 638 633
+3  628 627 214
+3  594 168 593
+3  593 168 636
+3  1130 1874 1867
+3  587 1242 584
+3  629 627 213
+3  1203 1202 1201
+3  1201 1202 575
+3  1610 927 1611
+3  1994 1993 884
+3  1883 977 148
+3  69 1021 1010
+3  65 943 944
+3  622 623 30
+3  1205 1200 617
+3  1579 598 1574
+3  1579 1574 1573
+3  1146 809 519
+3  879 1976 829
+3  62 1880 1878
+3  130 1054 1055
+3  807 538 74
+3  1168 1167 145
+3  72 643 645
+3  643 645 137
+3  1170 1171 139
+3  136 645 72
+3  197 27 810
+3  137 646 643
+3  1768 29 1610
+3  1610 1768 1599
+3  805 112 1902
+3  648 566 183
+3  946 604 67
+3  1415 1998 1999
+3  1415 1419 1999
+3  1415 1419 889
+3  650 360 651
+3  369 992 996
+3  362 994 989
+3  264 1952 1951
+3  125 655 654
+3  428 203 427
+3  242 1652 246
+3  1356 1354 265
+3  206 1190 429
+3  920 1077 113
+3  1082 311 1918
+3  660 659 124
+3  308 657 658
+3  308 312 661
+3  1572 1650 259
+3  1568 1572 259
+3  302 662 663
+3  657 661 308
+3  855 856 564
+3  797 1158 26
+3  1920 60 664
+3  306 664 1920
+3  311 1082 356
+3  579 2024 1922
+3  306 663 664
+3  579 2024 1615
+3  302 663 664
+3  273 280 1427
+3  1852 1844 1851
+3  1851 1852 1845
+3  1350 1349 28
+3  1574 41 1573
+3  2014 41 1573
+3  1660 188 1658
+3  1409 501 1408
+3  1059 753 1048
+3  935 617 1200
+3  1118 1842 1114
+3  648 672 52
+3  566 648 672
+3  38 1115 1645
+3  1515 415 1517
+3  1701 289 226
+3  538 74 512
+3  675 1903 676
+3  1731 1732 1730
+3  742 283 698
+3  742 283 703
+3  242 246 1112
+3  698 438 742
+3  496 1821 45
+3  82 537 681
+3  1898 219 1895
+3  353 1808 860
+3  185 681 332
+3  1076 1516 80
+3  323 1347 693
+3  683 83 1752
+3  332 690 689
+3  1558 685 683
+3  683 685 1751
+3  92 322 1361
+3  963 689 690
+3  327 405 82
+3  689 964 965
+3  1634 1631 1689
+3  695 694 1347
+3  959 958 957
+3  106 958 959
+3  692 537 691
+3  537 82 692
+3  959 957 745
+3  324 1372 1373
+3  1369 1370 1374
+3  1369 1374 1183
+3  1760 107 1759
+3  33 697 407
+3  705 288 706
+3  288 178 287
+3  288 287 701
+3  1309 671 103
+3  700 702 287
+3  287 702 284
+3  699 283 700
+3  284 282 702
+3  1456 1825 1809
+3  704 705 493
+3  1253 295 297
+3  709 710 291
+3  54 706 705
+3  712 716 55
+3  1252 298 871
+3  493 871 1252
+3  290 404 709
+3  107 686 1759
+3  1711 1707 1708
+3  1995 1177 830
+3  850 347 336
+3  684 465 1752
+3  1751 1752 684
+3  1254 295 1255
+3  1135 36 1134
+3  715 716 712
+3  838 1381 1399
+3  715 717 129
+3  40 720 721
+3  718 243 719
+3  40 720 245
+3  573 1234 1235
+3  1260 1261 416
+3  570 1988 569
+3  1290 1289 729
+3  1300 671 1299
+3  509 819 541
+3  680 1656 181
+3  197 112 810
+3  527 1289 1287
+3  375 952 951
+3  7 1725 1727
+3  1989 1727 7
+3  1944 1940 1037
+3  1052 1053 130
+3  1798 1796 457
+3  1900 1899 1896
+3  133 762 765
+3  1056 512 538
+3  224 1041 1040
+3  1922 1613 1614
+3  248 796 1432
+3  885 1277 474
+3  126 1980 390
+3  525 678 801
+3  1085 19 1083
+3  1939 1036 1941
+3  1945 1946 1698
+3  1289 1296 527
+3  1741 507 1773
+3  539 1777 485
+3  1140 1696 286
+3  1016 378 1019
+3  1696 420 71
+3  1284 1307 134
+3  1284 1307 532
+3  951 947 948
+3  948 951 375
+3  509 816 819
+3  1507 1506 1550
+3  1551 1506 1550
+3  209 361 972
+3  615 735 734
+3  1074 615 616
+3  518 727 102
+3  102 727 726
+3  730 195 1435
+3  1262 416 1261
+3  1261 417 1260
+3  615 616 72
+3  734 72 615
+3  727 732 101
+3  732 728 727
+3  557 136 731
+3  101 735 726
+3  733 136 732
+3  101 733 732
+3  101 726 727
+3  1061 535 818
+3  818 535 1740
+3  735 734 101
+3  1072 1075 116
+3  677 1656 680
+3  680 677 679
+3  1069 401 1068
+3  135 738 737
+3  727 728 739
+3  73 738 739
+3  727 739 73
+3  901 1444 1442
+3  1829 1442 901
+3  1433 1432 86
+3  1101 1103 593
+3  593 1101 1100
+3  1668 150 1667
+3  391 68 403
+3  873 2025 835
+3  510 1047 1058
+3  1854 48 1548
+3  1939 231 1940
+3  131 1049 1044
+3  1866 885 1417
+3  1799 1801 1798
+3  1798 1541 1799
+3  1388 1387 237
+3  1387 1388 1386
+3  1259 534 419
+3  833 906 903
+3  1773 507 1774
+3  746 1742 329
+3  2021 585 1625
+3  6 1625 2021
+3  1017 1020 379
+3  1600 1664 1663
+3  1262 1312 418
+3  230 1036 1941
+3  217 1943 1898
+3  231 1940 1901
+3  1499 497 1527
+3  1527 481 1499
+3  466 1449 1450
+3  1417 1662 885
+3  955 956 691
+3  1765 964 330
+3  1079 201 425
+3  331 746 328
+3  28 328 331
+3  962 744 960
+3  744 961 960
+3  1014 987 210
+3  331 748 28
+3  53 769 1120
+3  447 78 446
+3  750 53 1891
+3  1071 401 1705
+3  1791 1557 344
+3  344 1790 1791
+3  763 762 520
+3  1440 815 758
+3  752 778 779
+3  777 780 159
+3  1730 223 1732
+3  663 123 1917
+3  1046 510 1043
+3  132 1438 1439
+3  514 757 756
+3  758 757 514
+3  1441 731 730
+3  761 760 520
+3  371 98 1965
+3  1278 1279 521
+3  1175 144 1694
+3  761 762 133
+3  1193 205 1187
+3  764 411 412
+3  412 411 23
+3  764 413 3
+3  1642 781 1643
+3  1643 1642 1644
+3  771 172 766
+3  768 769 154
+3  1497 153 258
+3  773 772 157
+3  1322 776 158
+3  772 39 776
+3  1325 1324 953
+3  754 756 1916
+3  287 701 700
+3  778 160 779
+3  224 1039 1040
+3  1114 1118 215
+3  1996 1995 830
+3  783 771 157
+3  1583 1581 596
+3  1062 1641 1642
+3  1494 0 784
+3  1494 774 784
+3  783 156 767
+3  396 1211 1984
+3  869 348 868
+3  156 770 786
+3  154 787 769
+3  552 97 1157
+3  9 606 607
+3  1157 606 607
+3  1770 1125 42
+3  1127 1125 42
+3  788 152 789
+3  788 787 154
+3  907 1106 95
+3  1566 248 1564
+3  1564 248 795
+3  174 787 790
+3  858 564 859
+3  945 1180 592
+3  945 1180 65
+3  552 26 1158
+3  150 1676 1669
+3  1777 1778 507
+3  1565 1564 87
+3  886 887 1106
+3  1416 1999 472
+3  1868 1877 639
+3  639 1868 1876
+3  209 362 1003
+3  50 426 427
+3  1317 573 916
+3  423 209 972
+3  1423 1422 574
+3  1423 1425 574
+3  547 548 549
+3  45 1823 1805
+3  34 382 560
+3  560 382 1406
+3  1989 7 1260
+3  355 656 1934
+3  125 655 656
+3  50 384 428
+3  1175 144 1176
+3  656 313 661
+3  1126 1973 1972
+3  1935 355 1921
+3  1133 971 141
+3  1084 1083 309
+3  1926 1612 927
+3  662 658 123
+3  662 123 663
+3  1190 1189 205
+3  3 543 544
+3  1191 1850 1194
+3  1083 1082 309
+3  38 1641 1642
+3  292 1359 714
+3  782 1643 1644
+3  1117 1644 782
+3  1680 1676 602
+3  309 1084 1086
+3  1473 833 1470
+3  1236 1238 915
+3  1316 915 1236
+3  1312 1311 418
+3  302 59 1959
+3  92 318 317
+3  1350 1349 320
+3  677 1856 679
+3  723 1304 1306
+3  368 1006 1005
+3  1789 1635 339
+3  1768 29 1663
+3  1768 1663 1599
+3  1055 513 1054
+3  132 757 1438
+3  1635 1270 339
+3  1271 1270 1635
+3  765 530 1783
+3  1266 1746 473
+3  1501 1514 1517
+3  119 647 648
+3  1735 296 1737
+3  1736 296 1735
+3  104 1281 1279
+3  932 1589 1779
+3  676 1656 1655
+3  656 355 657
+3  140 1490 1489
+3  92 317 316
+3  1655 678 801
+3  1097 967 315
+3  1407 669 1772
+3  1368 93 1351
+3  678 184 676
+3  678 1655 676
+3  1407 484 1122
+3  1593 1604 2009
+3  692 693 327
+3  1704 784 774
+3  842 192 841
+3  14 1237 1238
+3  182 1856 1857
+3  1910 1909 370
+3  975 974 149
+3  170 1678 1679
+3  1484 1160 1162
+3  976 148 37
+3  182 538 1057
+3  1408 85 1776
+3  180 1700 673
+3  1946 1698 226
+3  802 801 1857
+3  1656 1655 677
+3  182 1857 802
+3  1310 418 1311
+3  1310 418 1299
+3  738 519 804
+3  73 519 738
+3  803 802 524
+3  73 518 519
+3  1327 1326 1736
+3  518 73 727
+3  805 810 112
+3  808 809 74
+3  1439 509 541
+3  1260 416 7
+3  1052 1053 511
+3  511 1053 1045
+3  1049 1050 131
+3  412 1282 1280
+3  412 1282 1192
+3  737 27 810
+3  519 518 1146
+3  517 1146 809
+3  104 1282 1280
+3  1989 1727 1258
+3  1946 1945 229
+3  340 105 341
+3  802 803 525
+3  876 874 834
+3  748 28 1349
+3  475 1269 1271
+3  475 1271 1785
+3  1846 1845 207
+3  1848 1846 207
+3  561 1687 1147
+3  519 809 808
+3  1272 1273 888
+3  525 801 802
+3  677 801 1857
+3  160 812 811
+3  1627 1626 936
+3  1627 1626 1619
+3  805 525 803
+3  803 804 524
+3  135 804 803
+3  135 803 805
+3  1155 723 526
+3  526 723 724
+3  1716 1715 1717
+3  74 808 807
+3  519 808 524
+3  135 810 805
+3  536 1051 1047
+3  345 347 18
+3  812 17 814
+3  756 1059 514
+3  484 1741 1682
+3  484 1122 1123
+3  1123 1124 1684
+3  1124 1684 488
+3  1910 1027 608
+3  1326 1953 1327
+3  1741 1740 507
+3  1740 1741 535
+3  1741 535 1682
+3  816 75 817
+3  816 817 819
+3  541 819 539
+3  560 550 34
+3  1249 849 1247
+3  1111 83 468
+3  892 472 893
+3  1471 1472 1476
+3  1476 1472 491
+3  848 56 1246
+3  714 293 1359
+3  715 292 1359
+3  839 1381 1399
+3  1214 1216 398
+3  1480 825 614
+3  614 825 1075
+3  1140 437 71
+3  867 496 12
+3  1734 1733 845
+3  38 1644 1642
+3  255 1893 256
+3  707 706 290
+3  1251 1250 1248
+3  1251 1250 297
+3  1481 32 1213
+3  1030 2017 381
+3  1025 2017 1030
+3  1418 885 886
+3  1969 1191 206
+3  886 887 474
+3  903 895 471
+3  338 2026 873
+3  873 2026 2025
+3  773 157 1704
+3  471 1832 1721
+3  1710 1702 831
+3  1275 337 881
+3  854 564 855
+3  2026 874 2025
+3  6 587 1121
+3  1534 452 1885
+3  721 240 241
+3  260 1568 1949
+3  239 721 241
+3  153 1653 241
+3  343 342 341
+3  1960 1959 301
+3  1580 1781 1575
+3  659 658 308
+3  1953 1329 665
+3  1875 1874 1129
+3  853 852 905
+3  467 1467 490
+3  1468 1443 467
+3  467 1468 1450
+3  1109 188 1110
+3  876 877 828
+3  1555 339 1556
+3  1270 339 1556
+3  1556 335 1270
+3  955 323 956
+3  1360 330 1749
+3  108 336 335
+3  338 905 877
+3  65 590 943
+3  1560 394 1239
+3  1239 581 1560
+3  1948 1649 1650
+3  1649 88 1651
+3  1934 1933 356
+3  128 1355 1951
+3  1138 63 1137
+3  1201 1199 924
+3  658 123 657
+3  302 1959 1958
+3  1739 513 1055
+3  876 877 338
+3  1380 839 1381
+3  827 337 828
+3  862 868 863
+3  949 374 1405
+3  949 374 1312
+3  840 1380 1379
+3  95 897 899
+3  2016 352 1331
+3  1331 352 1335
+3  921 1423 1422
+3  1735 1734 845
+3  1735 1734 1736
+3  1377 844 843
+3  843 1378 1377
+3  865 1138 866
+3  831 1710 1709
+3  1478 400 1477
+3  820 1135 851
+3  36 1135 851
+3  838 837 1398
+3  1397 1398 837
+3  56 1247 1246
+3  1245 849 1134
+3  1246 849 1245
+3  300 967 1097
+3  192 839 846
+3  843 851 36
+3  187 904 469
+3  2026 876 338
+3  834 336 850
+3  850 347 847
+3  850 193 847
+3  1395 193 1396
+3  859 861 860
+3  232 1383 562
+3  1778 1740 818
+3  1136 820 1135
+3  1575 1588 1781
+3  1121 392 2019
+3  253 1391 171
+3  2019 392 1029
+3  902 1834 1831
+3  1831 1833 902
+3  469 468 1110
+3  469 1110 1109
+3  1266 1267 1265
+3  913 883 914
+3  914 883 911
+3  1446 190 1445
+3  1836 822 1835
+3  1452 1451 1453
+3  1554 1790 1791
+3  352 1378 1377
+3  1335 1377 352
+3  47 860 864
+3  1706 1707 831
+3  1706 1702 831
+3  825 1478 1479
+3  852 853 854
+3  1993 884 1992
+3  1274 881 1277
+3  1274 474 1277
+3  865 866 350
+3  252 1402 161
+3  335 334 18
+3  964 1360 330
+3  1827 1562 1459
+3  859 860 47
+3  18 335 347
+3  252 253 1402
+3  1929 1930 563
+3  865 350 860
+3  864 865 860
+3  864 349 823
+3  865 349 864
+3  18 345 342
+3  233 232 178
+3  867 12 871
+3  871 12 493
+3  872 835 869
+3  286 1140 285
+3  285 64 1140
+3  1703 908 1414
+3  891 1691 1690
+3  872 873 835
+3  875 874 835
+3  1402 255 253
+3  878 337 828
+3  911 883 96
+3  877 828 878
+3  1108 1107 473
+3  1240 394 1560
+3  884 2001 2000
+3  905 829 877
+3  889 1414 890
+3  1994 1997 1996
+3  1996 1997 830
+3  1476 1470 1471
+3  913 883 908
+3  874 2026 876
+3  1704 157 784
+3  853 905 338
+3  2006 1321 1320
+3  833 1474 1470
+3  1020 1021 1010
+3  900 1830 2023
+3  1477 116 1069
+3  898 95 897
+3  855 856 1472
+3  491 1692 1475
+3  1976 1977 829
+3  898 1722 1720
+3  790 1112 787
+3  469 899 1108
+3  1472 1471 354
+3  1832 1834 1831
+3  1832 1831 900
+3  2023 1830 901
+3  2022 470 2023
+3  1462 1463 456
+3  722 1534 1464
+3  1793 1448 1455
+3  1794 1455 1793
+3  470 478 904
+3  895 906 903
+3  891 1702 1703
+3  1703 1710 890
+3  891 908 1703
+3  885 886 474
+3  912 96 855
+3  96 912 910
+3  1999 1419 472
+3  2001 1417 2002
+3  1417 1866 2000
+3  1416 1866 2000
+3  260 262 1747
+3  1418 1416 472
+3  1276 1275 827
+3  910 1979 909
+3  2002 1662 880
+3  1996 1993 1994
+3  1660 188 1110
+3  1993 1995 1996
+3  1977 909 829
+3  1954 1953 1329
+3  1954 1953 1327
+3  1272 1265 888
+3  1696 420 286
+3  879 878 829
+3  1412 1413 2003
+3  1177 1995 1413
+3  852 909 910
+3  96 910 911
+3  1997 880 879
+3  852 910 912
+3  882 830 914
+3  1104 1587 1983
+3  911 914 882
+3  1838 1836 851
+3  851 1839 1838
+3  835 836 869
+3  351 1838 1839
+3  351 1838 1835
+3  1331 2016 1332
+3  851 820 1836
+3  1920 1919 304
+3  697 1196 1371
+3  661 656 657
+3  1334 191 1336
+3  1334 1336 667
+3  1837 820 1136
+3  1136 349 1837
+3  1728 385 1729
+3  1799 1800 455
+3  1799 1800 1462
+3  1362 666 1363
+3  796 1099 170
+3  1088 1079 1087
+3  1210 395 1985
+3  1092 1961 300
+3  1131 1167 1166
+3  1944 225 1037
+3  1614 1922 2024
+3  500 85 1553
+3  1452 857 1453
+3  923 609 1601
+3  80 1516 1508
+3  1256 1089 61
+3  1077 1211 396
+3  1929 1930 858
+3  747 323 1345
+3  1452 490 467
+3  1449 466 1448
+3  1235 581 1236
+3  923 609 922
+3  923 575 922
+3  921 1421 1422
+3  257 1497 153
+3  791 153 257
+3  394 1243 1242
+3  1422 1318 574
+3  1497 258 256
+3  575 924 923
+3  14 609 939
+3  575 1201 924
+3  1241 1242 587
+3  928 1615 932
+3  1045 512 522
+3  599 1915 2012
+3  2012 599 2011
+3  1585 595 1181
+3  1585 590 1181
+3  1613 1924 1925
+3  410 23 1195
+3  925 1665 1599
+3  1239 1237 581
+3  1598 927 1610
+3  1611 1610 29
+3  1203 1204 618
+3  1425 1423 1202
+3  133 942 761
+3  1202 1424 1423
+3  100 1343 1697
+3  942 761 760
+3  922 609 14
+3  2011 1594 599
+3  599 1609 1594
+3  599 1609 1573
+3  1309 1067 103
+3  938 582 937
+3  940 583 939
+3  939 14 940
+3  1606 213 935
+3  1606 1607 935
+3  1928 1596 1595
+3  1595 1596 578
+3  1585 595 1584
+3  1619 1620 582
+3  941 1618 1616
+3  1713 1709 892
+3  1713 1714 892
+3  1287 1303 527
+3  943 590 589
+3  119 1124 647
+3  253 1387 1391
+3  945 586 946
+3  945 944 586
+3  1286 1339 1340
+3  1340 1339 1341
+3  1907 1007 1905
+3  1090 303 1091
+3  1092 303 1091
+3  796 795 1099
+3  428 1726 204
+3  372 1725 1726
+3  7 1725 373
+3  264 1355 1353
+3  263 1951 1952
+3  1966 371 91
+3  1920 1091 304
+3  1117 1645 1646
+3  1646 1645 1115
+3  1078 1079 1080
+3  1295 1290 1291
+3  1295 1292 1294
+3  1793 1448 1461
+3  442 117 1510
+3  447 1536 1813
+3  1547 445 1545
+3  2 1144 1147
+3  2005 166 2004
+3  1662 881 1277
+3  1662 1277 885
+3  1541 1810 1540
+3  1123 119 1122
+3  444 48 1547
+3  477 1449 1447
+3  1448 1449 1447
+3  1466 1455 1795
+3  190 1455 1466
+3  1685 1504 1503
+3  1255 1254 1376
+3  815 758 1042
+3  1207 920 397
+3  397 920 612
+3  814 555 17
+3  605 1314 1738
+3  1046 1044 511
+3  1531 1840 450
+3  553 1157 1158
+3  1041 1731 228
+3  224 1731 1732
+3  67 1025 1063
+3  1026 1025 1063
+3  941 1617 1628
+3  924 923 1600
+3  215 1118 1119
+3  67 1025 1022
+3  161 255 1402
+3  1319 1643 782
+3  224 1032 1039
+3  197 27 196
+3  1812 1803 453
+3  777 778 39
+3  778 780 777
+3  1321 158 1320
+3  1322 777 776
+3  151 792 973
+3  1127 42 635
+3  1322 777 159
+3  475 1268 1864
+3  475 1267 1864
+3  1389 1382 236
+3  43 545 1820
+3  46 1814 1815
+3  1461 1462 1463
+3  1990 650 985
+3  1832 2022 900
+3  1732 224 556
+3  1303 1302 723
+3  983 984 359
+3  1305 1152 134
+3  1673 168 594
+3  378 403 1019
+3  993 991 210
+3  1125 1770 637
+3  99 979 972
+3  1580 933 1582
+3  310 1113 1256
+3  1539 1540 1811
+3  1492 775 1491
+3  1941 230 1942
+3  1747 660 261
+3  1054 1056 1053
+3  1255 1328 1376
+3  1441 740 741
+3  1786 1787 1788
+3  160 813 779
+3  1898 1116 217
+3  805 1902 525
+3  1896 1897 220
+3  419 569 1724
+3  568 1724 419
+3  1344 2004 166
+3  1329 1330 665
+3  1061 1060 488
+3  806 182 538
+3  522 523 1050
+3  1061 75 1060
+3  818 508 817
+3  1041 224 1731
+3  1042 758 514
+3  1044 1043 1046
+3  1693 1694 953
+3  1797 79 1810
+3  691 958 537
+3  956 692 691
+3  1862 1365 1185
+3  693 956 692
+3  963 688 962
+3  93 1351 319
+3  961 746 331
+3  417 1689 1634
+3  316 1361 92
+3  332 965 684
+3  1760 107 1766
+3  106 690 963
+3  689 332 965
+3  328 1764 746
+3  746 329 1764
+3  315 1338 966
+3  315 1338 1337
+3  1354 266 1356
+3  967 968 300
+3  301 968 300
+3  333 1759 1760
+3  1762 1759 333
+3  686 1757 1758
+3  1758 1757 1765
+3  1757 1765 330
+3  1765 688 964
+3  965 1360 684
+3  1764 105 1763
+3  321 1366 1186
+3  1336 666 1337
+3  1350 320 1351
+3  319 1351 318
+3  1350 1351 318
+3  870 63 1139
+3  198 545 1820
+3  844 36 843
+3  1880 1879 1878
+3  1244 1012 357
+3  921 1420 1421
+3  822 837 838
+3  822 838 1835
+3  384 558 50
+3  1933 125 1934
+3  970 624 646
+3  641 1872 1873
+3  480 117 1511
+3  480 1511 1519
+3  629 213 1257
+3  1974 625 1863
+3  799 1100 1887
+3  973 149 974
+3  151 976 37
+3  974 148 976
+3  1979 1977 909
+3  1992 884 1998
+3  882 1976 1977
+3  1950 261 1747
+3  1134 821 849
+3  492 984 983
+3  99 422 982
+3  983 982 358
+3  983 982 981
+3  1134 1245 845
+3  1035 1743 215
+3  314 984 654
+3  988 361 986
+3  654 492 984
+3  986 979 972
+3  972 986 361
+3  651 987 360
+3  210 988 987
+3  1204 1205 618
+3  209 989 361
+3  989 362 209
+3  998 1001 364
+3  996 8 997
+3  993 367 991
+3  210 993 1014
+3  991 366 992
+3  1009 998 999
+3  8 996 995
+3  996 992 366
+3  1008 379 1010
+3  2012 1915 630
+3  371 363 1002
+3  383 89 365
+3  999 89 365
+3  1000 363 1001
+3  1000 1001 8
+3  1003 363 98
+3  558 208 426
+3  1004 382 34
+3  1006 1005 34
+3  1013 201 424
+3  424 201 425
+3  1016 378 1015
+3  1010 1008 1912
+3  982 1011 358
+3  999 1009 89
+3  1015 378 70
+3  89 1009 376
+3  1014 987 651
+3  1014 651 382
+3  382 993 1004
+3  379 1016 1015
+3  1029 391 392
+3  700 1889 701
+3  1018 1015 70
+3  969 1018 70
+3  368 1031 1005
+3  1029 391 68
+3  1913 1018 1008
+3  608 1027 1024
+3  2021 585 944
+3  1021 1022 380
+3  1958 664 302
+3  380 1022 1025
+3  69 1024 1023
+3  1006 1911 1908
+3  34 1911 1006
+3  1029 1028 1030
+3  392 393 1121
+3  1005 367 1004
+3  1243 583 940
+3  1207 1209 1208
+3  1037 1938 1939
+3  109 1033 1035
+3  222 1034 1033
+3  1284 670 1841
+3  2006 1341 2005
+3  387 1217 1216
+3  399 1223 1213
+3  231 1942 1899
+3  1211 127 1210
+3  1077 573 916
+3  916 113 1077
+3  920 113 919
+3  309 1082 1918
+3  309 1919 1918
+3  532 1284 1841
+3  542 1439 132
+3  1049 1045 1044
+3  1048 753 131
+3  509 1440 1439
+3  1440 758 1438
+3  728 731 1441
+3  130 536 1052
+3  1052 1051 511
+3  1047 1046 1051
+3  1616 1618 1923
+3  1924 1923 1613
+3  1045 512 1053
+3  1054 1056 513
+3  648 1055 52
+3  648 1055 647
+3  1855 1856 182
+3  1056 1057 538
+3  1058 75 816
+3  514 1059 1048
+3  536 1047 1060
+3  1062 1038 216
+3  539 819 508
+3  946 586 1063
+3  1063 586 1026
+3  1838 1836 1835
+3  1383 1382 232
+3  860 1808 350
+3  1345 747 331
+3  1458 1828 460
+3  1477 116 825
+3  1470 490 1471
+3  459 1809 1466
+3  459 1809 1457
+3  466 1463 44
+3  353 1454 857
+3  1793 1461 457
+3  1967 1968 189
+3  903 1474 833
+3  902 1469 1833
+3  1968 1469 1833
+3  908 883 1692
+3  1147 1687 515
+3  193 847 1395
+3  1114 1038 1641
+3  1123 1682 484
+3  1923 1922 1613
+3  1661 1535 1536
+3  1661 1537 1536
+3  1727 570 1725
+3  1858 920 1077
+3  1077 1858 396
+3  423 208 1728
+3  1643 781 159
+3  1062 1038 1641
+3  1880 1870 1879
+3  1536 1535 453
+3  1367 1861 1184
+3  1369 1367 1184
+3  561 755 1687
+3  1917 663 306
+3  953 1495 1695
+3  1064 1492 1065
+3  1064 1065 146
+3  147 155 1066
+3  1066 155 1065
+3  1563 1047 1058
+3  1298 103 1297
+3  1068 947 402
+3  1228 1068 400
+3  1229 400 1228
+3  1293 13 1292
+3  569 1988 419
+3  116 1070 1072
+3  1072 736 1070
+3  80 1076 415
+3  1071 736 13
+3  1292 1290 1295
+3  1073 1074 615
+3  1075 1074 616
+3  616 1075 614
+3  1073 1072 1074
+3  1515 415 1076
+3  1012 1078 201
+3  431 425 1088
+3  1245 845 1735
+3  1084 1085 310
+3  1119 215 1035
+3  1035 1119 1936
+3  579 1922 1616
+3  230 1036 1936
+3  334 340 1761
+3  306 1920 1919
+3  1089 1090 305
+3  431 1085 310
+3  1085 1084 1083
+3  431 19 1085
+3  1095 57 1094
+3  717 1352 718
+3  1918 306 1919
+3  60 1091 1920
+3  534 1182 419
+3  1765 1688 688
+3  1763 1761 333
+3  431 1087 19
+3  1086 1089 304
+3  1436 1437 1093
+3  1436 1437 303
+3  305 1095 1090
+3  1089 305 61
+3  1092 300 1093
+3  1437 1093 1092
+3  1321 142 1341
+3  1090 1095 303
+3  1099 793 795
+3  1100 593 594
+3  1093 300 1097
+3  798 1672 1671
+3  1095 57 316
+3  1095 1094 303
+3  1337 315 1096
+3  966 1328 1956
+3  1126 1128 1972
+3  1674 975 1675
+3  1181 1180 591
+3  590 1181 1180
+3  65 1180 590
+3  1587 5 1104
+3  1105 593 636
+3  1211 1210 1984
+3  1981 1104 1982
+3  1722 1716 832
+3  1112 152 787
+3  1609 929 2008
+3  2008 1594 1609
+3  1113 61 322
+3  433 1113 322
+3  1340 1341 2005
+3  310 1113 433
+3  1847 1187 1193
+3  218 1648 1344
+3  263 240 1649
+3  1121 587 393
+3  746 1742 961
+3  669 1408 1776
+3  1407 1408 669
+3  1775 506 485
+3  639 635 1127
+3  1167 971 1131
+3  141 971 1131
+3  630 2012 1257
+3  1162 641 1163
+3  1874 1129 1867
+3  1867 1872 640
+3  1168 1169 1167
+3  137 970 1171
+3  497 1520 1523
+3  1499 497 1523
+3  971 1169 1167
+3  141 1130 1131
+3  146 1483 1482
+3  1169 1170 971
+3  143 139 1165
+3  1882 167 1870
+3  970 1133 624
+3  1510 117 1511
+3  94 1686 572
+3  1499 1523 498
+3  1326 1733 844
+3  1331 1332 191
+3  349 865 1138
+3  1140 64 437
+3  821 1139 1137
+3  821 1137 1136
+3  1420 14 1238
+3  1248 56 1250
+3  56 1248 1247
+3  24 440 1818
+3  496 1821 495
+3  1747 124 262
+3  1964 384 1963
+3  1975 623 30
+3  1142 624 1133
+3  1145 518 526
+3  1145 1146 1144
+3  1144 1146 517
+3  769 1156 1120
+3  1783 765 1782
+3  1156 787 174
+3  1284 1283 134
+3  1151 1149 1148
+3  561 1149 1148
+3  1154 1145 526
+3  1151 529 1149
+3  1151 529 1152
+3  1153 528 1155
+3  1308 532 1309
+3  1308 1067 533
+3  1301 533 1302
+3  1719 1720 832
+3  1418 886 1717
+3  1660 1111 1657
+3  124 660 1747
+3  1719 894 1711
+3  906 1723 1702
+3  1706 1702 906
+3  1659 1658 1141
+3  1744 1658 1659
+3  261 1357 1358
+3  719 1952 263
+3  1160 1159 641
+3  1483 1161 1482
+3  146 1483 1064
+3  146 1163 1066
+3  1163 1162 146
+3  1164 143 1174
+3  1927 1925 1613
+3  1126 625 1973
+3  1486 1164 1485
+3  1160 1166 1159
+3  169 635 639
+3  1173 971 1170
+3  1172 645 138
+3  1694 953 1325
+3  196 1176 195
+3  1180 1179 592
+3  66 1101 1100
+3  1179 1102 1178
+3  592 1179 1102
+3  1406 211 560
+3  630 1257 629
+3  926 925 1608
+3  1647 1116 1646
+3  1647 1646 1117
+3  530 1283 1281
+3  1850 1191 23
+3  695 1373 1375
+3  296 1737 1376
+3  1345 323 1346
+3  763 521 762
+3  758 757 1438
+3  1281 530 1279
+3  674 21 1698
+3  1345 748 1346
+3  320 1183 1348
+3  1372 1371 324
+3  1351 1368 320
+3  1326 1733 1734
+3  1576 596 1575
+3  1580 596 1575
+3  23 1195 1850
+3  1846 1845 1186
+3  1591 1592 928
+3  1635 1271 1784
+3  1851 1185 1845
+3  697 1196 325
+3  1848 1846 1193
+3  1847 1846 1193
+3  1184 1370 1369
+3  1281 104 1633
+3  1843 1844 1851
+3  134 1285 1152
+3  1152 1285 529
+3  412 1192 23
+3  23 1192 1191
+3  50 428 427
+3  325 1197 1196
+3  1199 618 1201
+3  576 1426 619
+3  618 1199 1200
+3  1199 577 1200
+3  925 577 924
+3  66 1101 1636
+3  1637 66 1636
+3  1420 922 921
+3  618 1203 1201
+3  931 1604 1603
+3  495 1889 494
+3  1627 936 937
+3  31 1204 1205
+3  627 617 1206
+3  1206 620 627
+3  841 2015 2016
+3  1314 604 1315
+3  1055 1739 52
+3  1926 1927 1925
+3  1620 1619 941
+3  1422 1421 915
+3  2017 2018 381
+3  13 1293 735
+3  1480 1479 825
+3  1215 1216 387
+3  1228 402 1068
+3  1103 1104 1981
+3  1221 16 377
+3  377 1221 89
+3  399 1223 1222
+3  388 365 383
+3  1226 388 383
+3  1230 402 1227
+3  1229 1222 399
+3  387 1231 1215
+3  1215 1233 1231
+3  587 393 1241
+3  1232 1231 387
+3  1233 114 1215
+3  1241 393 1240
+3  113 918 917
+3  1234 573 1211
+3  1211 573 1077
+3  1565 1566 247
+3  1237 14 940
+3  940 394 1239
+3  940 1237 1239
+3  1626 930 1618
+3  576 918 610
+3  917 918 576
+3  619 1204 31
+3  1824 1820 43
+3  1935 1934 356
+3  1809 1456 1795
+3  1820 1821 1824
+3  1816 198 1819
+3  1819 199 1820
+3  1360 964 965
+3  1245 848 1246
+3  1252 705 493
+3  1247 1248 1249
+3  1249 1248 870
+3  298 1248 1251
+3  54 707 708
+3  707 706 54
+3  1253 56 1254
+3  848 56 1254
+3  1254 1253 295
+3  1338 1955 666
+3  666 1338 1337
+3  1600 1663 1665
+3  1151 1152 528
+3  1155 528 1306
+3  1305 134 1307
+3  1304 1305 533
+3  1283 134 1285
+3  765 530 1278
+3  83 1753 1752
+3  753 1050 523
+3  135 737 810
+3  724 723 1303
+3  1300 418 1262
+3  418 1300 1299
+3  1262 416 374
+3  153 1313 791
+3  1312 1262 374
+3  1108 1746 1109
+3  1267 1265 888
+3  1264 887 1272
+3  475 1269 1268
+3  1268 1269 826
+3  1271 1270 1269
+3  474 1274 1273
+3  1792 1557 1791
+3  1791 1554 1792
+3  1865 1274 826
+3  529 1783 1782
+3  1274 881 1275
+3  1999 1998 1416
+3  339 1555 1789
+3  762 1278 765
+3  521 1278 762
+3  764 1280 412
+3  764 1280 521
+3  531 670 1284
+3  529 1285 1783
+3  34 824 1911
+3  798 1667 1668
+3  798 1667 1674
+3  1906 1912 1010
+3  606 553 603
+3  1236 1237 581
+3  1679 797 553
+3  1639 601 1640
+3  1638 601 1639
+3  592 1639 1102
+3  1639 1102 1637
+3  1336 191 1362
+3  1668 1670 798
+3  262 86 1433
+3  1986 1689 1260
+3  1989 1260 1986
+3  1434 250 26
+3  1667 150 1681
+3  1682 1684 1683
+3  1750 684 1360
+3  1760 1761 333
+3  632 600 1914
+3  632 631 1914
+3  1177 1412 913
+3  263 1951 1948
+3  1344 1319 166
+3  197 221 1430
+3  725 1289 1290
+3  1288 726 102
+3  1070 1071 736
+3  13 1294 1292
+3  671 417 1300
+3  613 32 1480
+3  952 1295 1291
+3  1293 735 726
+3  1299 671 103
+3  1261 1300 1262
+3  403 1654 391
+3  723 1304 1302
+3  926 1605 1607
+3  935 577 1200
+3  1608 577 935
+3  1304 533 1302
+3  1155 1306 723
+3  1152 528 1305
+3  528 1305 1306
+3  1304 1305 1306
+3  1298 1299 103
+3  1298 1299 1310
+3  1298 1310 950
+3  1311 949 950
+3  1310 1311 950
+3  790 1313 791
+3  242 1313 790
+3  1902 678 525
+3  1314 1315 605
+3  607 1024 605
+3  605 1315 607
+3  981 982 99
+3  1237 1236 1238
+3  1935 1934 355
+3  985 1990 649
+3  211 1990 1970
+3  1986 1258 1989
+3  1781 1780 1588
+3  1325 1324 142
+3  574 1318 916
+3  916 1317 1318
+3  619 1203 1202
+3  1425 619 1202
+3  1319 166 1320
+3  2007 1595 1592
+3  158 1322 1320
+3  196 1431 1176
+3  1321 1323 142
+3  1429 1286 1428
+3  100 1428 1429
+3  69 1023 1021
+3  1326 1736 1734
+3  666 1329 1363
+3  666 1329 1955
+3  1254 848 1737
+3  1328 966 58
+3  1332 841 2016
+3  1328 1957 1956
+3  1738 1023 1022
+3  572 1333 1334
+3  1334 572 668
+3  667 1334 668
+3  1329 1330 1363
+3  1336 1337 667
+3  667 1337 1096
+3  1341 1321 2006
+3  1319 782 1344
+3  238 237 1391
+3  1861 1364 1862
+3  1862 1843 1364
+3  1349 1346 1348
+3  748 1346 1349
+3  1905 1007 1906
+3  264 1352 718
+3  1354 293 1359
+3  1358 1355 1357
+3  1354 1359 129
+3  686 1758 1759
+3  1759 1758 1762
+3  660 59 1358
+3  1380 1381 351
+3  305 316 1361
+3  1361 61 305
+3  322 61 1361
+3  1655 801 677
+3  1057 513 1855
+3  1366 319 321
+3  1366 319 93
+3  1856 679 1855
+3  93 1365 1366
+3  1373 324 1375
+3  320 1183 1368
+3  1371 324 1370
+3  1371 1184 1370
+3  696 695 1373
+3  694 1374 1375
+3  695 1375 694
+3  1378 1379 843
+3  840 839 1380
+3  2015 2016 352
+3  839 192 840
+3  1400 846 839
+3  233 232 1382
+3  1382 165 233
+3  236 1384 1382
+3  1384 1383 164
+3  562 277 1383
+3  1387 1386 1
+3  236 1389 1388
+3  1 1387 253
+3  237 1389 1388
+3  237 1390 238
+3  1391 237 1387
+3  1394 234 571
+3  571 235 1394
+3  836 835 1401
+3  836 1396 837
+3  1398 346 1397
+3  838 1398 1399
+3  1399 1398 346
+3  875 1401 193
+3  193 1401 1396
+3  836 1401 1396
+3  1401 875 835
+3  949 948 1405
+3  1403 373 416
+3  374 1403 416
+3  365 1404 559
+3  365 91 1404
+3  90 1405 948
+3  750 1892 1894
+3  264 1951 1355
+3  888 1267 1864
+3  501 1407 1122
+3  1409 499 1860
+3  1529 1543 1530
+3  1529 450 1530
+3  1804 1806 1805
+3  1777 1775 485
+3  1507 1549 1550
+3  483 1410 118
+3  1860 501 1409
+3  85 500 503
+3  1419 892 472
+3  1995 1413 1993
+3  1420 14 922
+3  1318 1422 915
+3  1424 1423 921
+3  922 1424 575
+3  1425 619 1426
+3  1426 1425 574
+3  917 1426 574
+3  51 273 1427
+3  51 1427 286
+3  144 1286 1325
+3  196 1431 1430
+3  1434 1433 262
+3  1108 473 1746
+3  1449 477 1468
+3  1446 478 1445
+3  1467 1443 467
+3  1448 190 1447
+3  855 1475 96
+3  1475 883 96
+3  458 1454 353
+3  1448 190 1455
+3  1474 1470 1469
+3  1530 1538 452
+3  1908 1909 370
+3  1458 459 1460
+3  1448 466 1461
+3  722 1464 1459
+3  1542 722 1459
+3  1539 1540 453
+3  459 1466 1460
+3  1466 1446 1460
+3  456 458 1806
+3  1797 1541 1810
+3  354 856 1472
+3  614 1480 613
+3  1707 1708 831
+3  1481 1479 1480
+3  1481 399 1479
+3  1481 399 1213
+3  586 944 2021
+3  1561 581 1560
+3  1235 1234 1561
+3  1603 1605 931
+3  701 1889 494
+3  1971 897 2022
+3  1971 470 2022
+3  1611 927 1612
+3  1026 586 2020
+3  6 1625 587
+3  1853 395 389
+3  981 99 1769
+3  589 1630 35
+3  1623 35 1622
+3  946 604 1640
+3  1640 592 946
+3  1627 937 582
+3  588 1498 1630
+3  1315 9 607
+3  589 1498 1586
+3  1493 155 786
+3  1491 1064 1492
+3  1493 155 1065
+3  1490 1495 775
+3  596 1581 1580
+3  1615 1589 934
+3  1853 393 127
+3  1795 1455 1794
+3  1621 1620 582
+3  1604 578 1603
+3  928 1926 1928
+3  1605 1606 1607
+3  1498 1582 933
+3  937 609 1601
+3  937 936 1601
+3  700 1889 699
+3  2010 1594 2011
+3  931 1606 1605
+3  590 1586 1585
+3  789 1569 152
+3  1589 934 933
+3  1500 1514 1501
+3  1495 1324 1496
+3  1160 145 1485
+3  1026 2018 2017
+3  1581 1582 1580
+3  1378 1379 2015
+3  1482 1484 1162
+3  1488 1164 1486
+3  1488 1487 1486
+3  1487 1488 140
+3  1485 1161 1486
+3  1623 35 1624
+3  2020 2019 2018
+3  6 2019 2020
+3  1596 578 1602
+3  582 1627 1619
+3  395 127 1210
+3  926 1603 1602
+3  1597 926 1602
+3  1030 1025 380
+3  221 1697 220
+3  935 1608 1607
+3  610 175 611
+3  925 577 1608
+3  214 623 622
+3  620 622 621
+3  1889 43 495
+3  1625 584 587
+3  1205 1206 617
+3  1209 1985 114
+3  1984 1985 1209
+3  1519 498 1511
+3  1561 1560 1240
+3  1739 679 52
+3  1906 1027 1905
+3  1488 1174 1164
+3  1915 630 1914
+3  1486 1487 1161
+3  1914 631 1915
+3  929 2007 2013
+3  1915 599 1579
+3  1484 1485 1161
+3  1168 1485 145
+3  1168 1485 1164
+3  1491 1489 1490
+3  1694 1325 144
+3  775 1496 1494
+3  1487 1483 1161
+3  1490 775 1491
+3  953 1324 1495
+3  1493 785 786
+3  774 1323 1494
+3  785 1493 0
+3  1493 1492 0
+3  1494 1492 775
+3  1323 1496 1324
+3  1494 1496 1323
+3  1504 498 1503
+3  502 1500 481
+3  1500 1501 502
+3  94 341 1686
+3  1729 1003 98
+3  442 48 1509
+3  1076 1512 1513
+3  1506 476 1509
+3  1506 476 1551
+3  1909 551 1910
+3  446 1545 444
+3  1532 1528 1507
+3  1507 1549 1532
+3  481 500 1553
+3  481 1527 1553
+3  1512 1518 1505
+3  1512 1505 449
+3  1520 480 1519
+3  1509 476 1510
+3  118 1521 1525
+3  1517 1502 415
+3  1516 1512 1076
+3  449 1512 1516
+3  76 440 441
+3  442 1509 1510
+3  1519 1520 1523
+3  498 1519 1523
+3  1525 118 1526
+3  1410 1411 118
+3  1522 482 479
+3  498 1511 1524
+3  498 1524 1504
+3  1525 1520 497
+3  1860 499 1859
+3  1526 1527 499
+3  499 1411 1526
+3  1686 572 1333
+3  1533 452 1534
+3  1533 452 1530
+3  451 1531 954
+3  954 1531 448
+3  1465 448 954
+3  1662 337 880
+3  1885 722 1884
+3  1543 1544 1529
+3  1146 518 1145
+3  1804 200 1802
+3  484 1772 1407
+3  75 1060 1563
+3  1801 1462 1799
+3  1047 1563 1060
+3  449 1549 1532
+3  446 1545 78
+3  1547 48 1548
+3  1547 1548 445
+3  449 1550 1505
+3  1550 1549 449
+3  1527 1553 499
+3  683 1558 344
+3  1749 330 1757
+3  1557 107 1555
+3  1561 1235 581
+3  1794 1796 1795
+3  1488 1174 140
+3  1459 1542 1562
+3  1442 189 1829
+3  1474 902 903
+3  1593 2009 2008
+3  1832 1834 471
+3  1567 1432 86
+3  1100 1671 594
+3  1566 248 1567
+3  1854 1509 1506
+3  932 1589 1615
+3  1615 1614 928
+3  1923 1922 1616
+3  1567 247 1566
+3  1569 1565 246
+3  800 1888 1638
+3  1603 578 1602
+3  1615 579 934
+3  1597 927 1596
+3  1612 1925 1926
+3  45 199 1823
+3  1571 247 1565
+3  1571 1565 246
+3  928 1592 1595
+3  1593 578 1604
+3  1574 1576 1575
+3  1586 580 1585
+3  1574 598 1576
+3  1584 595 1581
+3  597 1578 638
+3  1577 636 597
+3  631 598 1579
+3  1769 979 980
+3  596 1583 1577
+3  1498 580 1586
+3  1584 580 1582
+3  1582 1584 1581
+3  590 589 1586
+3  1769 99 979
+3  1585 1584 580
+3  1590 932 1591
+3  1590 41 1591
+3  1589 1779 1780
+3  1657 1790 344
+3  1575 1588 41
+3  1588 1779 1780
+3  931 1604 2010
+3  1614 1927 1613
+3  192 2015 841
+3  840 2015 192
+3  1573 1579 599
+3  1597 1598 925
+3  1606 213 1257
+3  582 1621 938
+3  1603 926 1605
+3  931 1606 1257
+3  1600 924 1665
+3  1601 923 1600
+3  1597 1596 1602
+3  931 2011 2010
+3  931 2011 1257
+3  2008 1594 2009
+3  1608 1607 926
+3  1664 936 1601
+3  1928 927 1596
+3  1611 29 936
+3  1928 927 1926
+3  1619 1626 1618
+3  352 1378 2015
+3  1379 840 2015
+3  934 1617 588
+3  1620 1629 1628
+3  1628 588 1617
+3  1628 588 1629
+3  1617 1616 941
+3  1617 1616 579
+3  1617 579 934
+3  1621 1629 1630
+3  1621 35 1622
+3  938 1621 1622
+3  1621 1630 35
+3  1622 938 583
+3  1621 1629 1620
+3  1623 584 1243
+3  534 1632 1631
+3  1631 1634 670
+3  670 1632 1631
+3  531 1632 670
+3  1118 1842 217
+3  1632 531 1633
+3  1633 534 1632
+3  1182 1633 534
+3  1633 104 1182
+3  1908 1909 1911
+3  1657 1787 1790
+3  1309 671 1841
+3  1638 66 1888
+3  1887 66 1100
+3  1888 1886 800
+3  592 1640 1639
+3  344 1111 1657
+3  1537 1771 1661
+3  452 1661 1771
+3  1648 1647 218
+3  1344 782 1648
+3  1652 1651 88
+3  241 1651 1652
+3  1651 1649 240
+3  1652 246 88
+3  1653 242 1313
+3  241 1652 1653
+3  595 1983 1587
+3  342 340 341
+3  1411 499 1859
+3  1676 602 800
+3  1410 1411 1859
+3  679 513 1855
+3  1659 1267 1266
+3  475 1267 1659
+3  1745 188 1744
+3  1744 188 1658
+3  468 1111 1660
+3  1660 468 1110
+3  1767 488 1124
+3  1664 936 29
+3  1667 1666 975
+3  1682 1683 535
+3  1569 1570 789
+3  1671 798 1670
+3  1674 1667 975
+3  167 15 1673
+3  1661 1535 452
+3  1676 800 1669
+3  1675 975 974
+3  606 603 1680
+3  1681 1667 1666
+3  1681 794 1666
+3  1678 794 1681
+3  488 1061 1683
+3  535 1061 1683
+3  1505 1685 1518
+3  1505 1504 1685
+3  1686 341 343
+3  1786 1787 1141
+3  1687 755 1916
+3  569 567 1724
+3  1473 1690 833
+3  1691 491 1473
+3  1473 1691 1690
+3  893 1714 1715
+3  491 1692 1691
+3  891 1692 1691
+3  1692 908 891
+3  1704 773 158
+3  1456 1796 1797
+3  452 1535 1885
+3  1884 1885 1535
+3  1006 1907 1007
+3  1906 1007 1912
+3  100 1697 221
+3  1699 1700 674
+3  1723 891 1690
+3  1699 1701 1700
+3  1699 1701 226
+3  1414 1703 890
+3  895 1707 1711
+3  1690 1723 906
+3  1070 1069 1705
+3  1070 1071 1705
+3  1707 906 1706
+3  1708 831 1709
+3  896 1712 471
+3  1710 1703 1702
+3  906 1707 895
+3  1712 1711 895
+3  894 1711 1708
+3  1718 1719 894
+3  1702 1723 891
+3  894 1713 1708
+3  471 895 1712
+3  1713 1709 1708
+3  1724 206 568
+3  832 1715 1716
+3  1718 894 1714
+3  1721 896 471
+3  893 1418 472
+3  893 1418 1717
+3  1715 1717 893
+3  1831 1833 1830
+3  1720 898 1721
+3  1712 1719 1711
+3  1718 1719 832
+3  898 1721 897
+3  1020 1010 379
+3  1725 372 373
+3  385 1964 384
+3  1970 649 548
+3  1024 605 1023
+3  1023 605 1738
+3  177 279 281
+3  285 177 281
+3  281 278 10
+3  1763 1761 105
+3  328 105 1764
+3  1033 1035 1743
+3  1924 930 1612
+3  1659 1266 1745
+3  1744 1745 1659
+3  1749 1748 685
+3  685 1749 1750
+3  1750 1360 1749
+3  1752 1753 465
+3  186 1755 463
+3  464 465 1753
+3  1555 107 1766
+3  464 1755 1754
+3  1756 1755 464
+3  1763 1762 687
+3  130 1767 536
+3  1688 687 1765
+3  334 1766 1760
+3  1556 1766 334
+3  1763 1762 333
+3  1765 1758 687
+3  1767 1124 647
+3  1767 130 647
+3  1982 591 1181
+3  1538 1771 1537
+3  1948 263 1649
+3  1775 506 1774
+3  1776 506 503
+3  85 1776 503
+3  669 1773 1774
+3  1777 508 539
+3  932 1779 1590
+3  1787 1141 1657
+3  1789 1635 1788
+3  1784 1786 1141
+3  1785 1784 1141
+3  1787 1788 1554
+3  1554 1788 1789
+3  1790 1554 1787
+3  1792 1557 1555
+3  1789 1555 1792
+3  1789 1792 1554
+3  1541 455 1799
+3  1540 455 1541
+3  457 1794 1796
+3  1800 456 1462
+3  1802 455 1803
+3  1813 1812 454
+3  1812 1803 454
+3  1804 1800 1802
+3  456 1804 1800
+3  200 1805 1804
+3  45 350 496
+3  1813 1814 447
+3  1816 1815 46
+3  1814 454 1813
+3  199 45 1821
+3  446 1815 447
+3  447 1814 1815
+3  1818 545 198
+3  1818 545 24
+3  1818 440 77
+3  200 1822 454
+3  1822 1819 199
+3  495 1824 43
+3  1825 1456 1797
+3  1457 1827 1826
+3  1467 1968 1967
+3  1830 1829 901
+3  1467 1968 1469
+3  1830 1831 900
+3  1833 189 1830
+3  1836 1837 822
+3  1836 1837 820
+3  823 1837 822
+3  349 823 1837
+3  1835 1381 838
+3  1835 351 1381
+3  1379 1839 843
+3  351 1839 1380
+3  1380 1839 1379
+3  843 851 1839
+3  1114 1842 1115
+3  1848 207 1195
+3  1849 1195 1848
+3  1193 1848 1849
+3  1863 628 214
+3  1197 1844 1198
+3  1198 207 1844
+3  1198 1195 207
+3  1970 549 560
+3  560 211 1970
+3  548 549 1970
+3  1987 1258 1988
+3  2010 1604 2009
+3  2009 1594 2010
+3  1857 677 1856
+3  1855 182 1057
+3  1975 1974 978
+3  978 624 1975
+3  1268 1865 826
+3  1864 1865 1268
+3  1673 1871 167
+3  639 169 1869
+3  1877 1869 639
+3  1870 167 1871
+3  1879 1870 169
+3  1876 626 1868
+3  1163 641 1873
+3  640 1868 1877
+3  1878 640 1873
+3  169 1879 1869
+3  1879 1869 1878
+3  1883 977 167
+3  1066 1881 147
+3  1882 1883 1881
+3  1881 1883 147
+3  62 1881 1880
+3  148 147 1883
+3  167 1882 1883
+3  370 1027 1905
+3  1670 799 1886
+3  1886 1670 1669
+3  1669 1886 800
+3  799 1887 1886
+3  1889 43 699
+3  1891 749 1890
+3  750 1891 1890
+3  1893 173 1892
+3  751 1893 1892
+3  1943 1942 111
+3  220 1896 1895
+3  219 1898 1116
+3  1899 231 1900
+3  1896 1899 111
+3  675 1900 1901
+3  370 1908 1907
+3  1908 1006 1907
+3  1911 824 1909
+3  1958 60 664
+3  1007 1913 1912
+3  630 600 1914
+3  1257 2011 2012
+3  631 1915 1579
+3  230 1936 1119
+3  657 355 1921
+3  657 1921 123
+3  1921 123 1917
+3  1921 1917 311
+3  1595 1928 928
+3  1932 492 654
+3  654 1932 125
+3  1244 1932 357
+3  1244 1932 492
+3  356 1933 1080
+3  1932 1933 357
+3  1932 1933 125
+3  1036 1938 1937
+3  1938 1037 1040
+3  1037 1939 1940
+3  109 1040 1938
+3  1941 1939 231
+3  111 1943 1898
+3  1942 230 1943
+3  21 1944 1945
+3  259 1948 1650
+3  1950 128 1949
+3  1956 1957 1327
+3  1338 966 1955
+3  1328 1957 1376
+3  1329 1955 1954
+3  1954 1327 1956
+3  1958 1961 60
+3  301 1960 1962
+3  301 1962 968
+3  968 299 1962
+3  1963 1726 372
+3  384 1726 1963
+3  1442 1443 1967
+3  98 1964 1729
+3  1442 1967 189
+3  372 1966 1965
+3  1969 1191 1194
+3  1190 1194 1969
+3  1443 1967 1467
+3  470 904 1971
+3  1128 978 1972
+3  882 911 1979
+3  1977 882 1979
+3  1979 910 911
+3  1232 389 1980
+3  1980 1232 390
+3  1853 126 1980
+3  1853 389 1980
+3  1981 1178 591
+3  595 1983 1181
+3  591 1982 1981
+3  1984 1210 1985
+3  395 1233 1985
+3  114 1233 1985
+3  1970 649 1990
+3  1987 1259 419
+3  1406 1991 651
+3  650 651 1991
+3  1990 650 1991
+3  1997 830 1978
+3  1413 1992 1993
+3  1416 1998 2000
+3  2002 880 1994
+3  2002 1994 2001
+3  1997 880 1994
+3  2002 1417 1662
+3  889 2003 1415
+3  2000 884 1998
+3  1415 1992 1998
+3  2001 1417 2000
+3  2003 1413 1992
+3  2003 1992 1415
+3  1428 1340 2004
+3  1428 2004 1342
+3  2005 2006 166
+3  166 1320 2006
+3  2008 1593 2007
+3  2007 2008 929
+3  1591 2013 1592
+3  1592 2007 2013
+3  41 1591 2014
+3  2014 1591 2013
+3  929 2014 2013
+3  1609 929 2014
+3  2014 1609 1573
+3  1025 1026 2017
+3  2019 1029 381
+3  586 2021 2020
+
+
+# End of OFF #
diff -pruN 5.0.3-1/examples/Polygon_mesh_processing/hausdorff_distance_remeshing_example.cpp 5.1-1/examples/Polygon_mesh_processing/hausdorff_distance_remeshing_example.cpp
--- 5.0.3-1/examples/Polygon_mesh_processing/hausdorff_distance_remeshing_example.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Polygon_mesh_processing/hausdorff_distance_remeshing_example.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -3,11 +3,7 @@
 #include <CGAL/Polygon_mesh_processing/distance.h>
 #include <CGAL/Polygon_mesh_processing/remesh.h>
 
-#if defined(CGAL_LINKED_WITH_TBB)
-#define TAG CGAL::Parallel_tag
-#else
-#define TAG CGAL::Sequential_tag
-#endif
+#define TAG CGAL::Parallel_if_available_tag
 
 typedef CGAL::Exact_predicates_inexact_constructions_kernel K;
 typedef K::Point_3                     Point;
diff -pruN 5.0.3-1/examples/Polygon_mesh_processing/hole_filling_example_SM.cpp 5.1-1/examples/Polygon_mesh_processing/hole_filling_example_SM.cpp
--- 5.0.3-1/examples/Polygon_mesh_processing/hole_filling_example_SM.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Polygon_mesh_processing/hole_filling_example_SM.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -2,58 +2,96 @@
 #include <CGAL/Surface_mesh.h>
 
 #include <CGAL/Polygon_mesh_processing/triangulate_hole.h>
+#include <CGAL/Polygon_mesh_processing/border.h>
+
+#include <boost/lexical_cast.hpp>
 
 #include <iostream>
 #include <fstream>
 #include <vector>
+#include <set>
 
 typedef CGAL::Exact_predicates_inexact_constructions_kernel Kernel;
 typedef Kernel::Point_3 Point;
 typedef CGAL::Surface_mesh<Point> Mesh;
 
-typedef boost::graph_traits<Mesh>::halfedge_descriptor    halfedge_descriptor;
+typedef boost::graph_traits<Mesh>::halfedge_descriptor   halfedge_descriptor;
 typedef boost::graph_traits<Mesh>::face_descriptor       face_descriptor;
-typedef boost::graph_traits<Mesh>::vertex_descriptor      vertex_descriptor;
+typedef boost::graph_traits<Mesh>::vertex_descriptor     vertex_descriptor;
+
+bool is_small_hole(halfedge_descriptor h, Mesh & mesh,
+                   double max_hole_diam, int max_num_hole_edges)
+{
+  int num_hole_edges = 0;
+  CGAL::Bbox_3 hole_bbox;
+  for (halfedge_descriptor hc : CGAL::halfedges_around_face(h, mesh))
+  {
+    const Point& p = mesh.point(target(hc, mesh));
+
+    hole_bbox += p.bbox();
+    ++num_hole_edges;
+
+    // Exit early, to avoid unnecessary traversal of large holes
+    if (num_hole_edges > max_num_hole_edges) return false;
+    if (hole_bbox.xmax() - hole_bbox.xmin() > max_hole_diam) return false;
+    if (hole_bbox.ymax() - hole_bbox.ymin() > max_hole_diam) return false;
+    if (hole_bbox.zmax() - hole_bbox.zmin() > max_hole_diam) return false;
+  }
+
+  return true;
+}
+
+// Incrementally fill the holes that are no larger than given diameter
+// and with no more than a given number of edges (if specified).
 
 int main(int argc, char* argv[])
 {
   const char* filename = (argc > 1) ? argv[1] : "data/mech-holes-shark.off";
-  std::ifstream input(filename);
 
+  // Both of these must be positive in order to be considered
+  double max_hole_diam   = (argc > 2) ? boost::lexical_cast<double>(argv[2]): -1.0;
+  int max_num_hole_edges = (argc > 3) ? boost::lexical_cast<int>(argv[3]) : -1;
+
+  std::ifstream input(filename);
   Mesh mesh;
   if ( !input || !(input >> mesh) ) {
     std::cerr << "Not a valid off file." << std::endl;
     return 1;
   }
 
-  // Incrementally fill the holes
   unsigned int nb_holes = 0;
-  for(halfedge_descriptor h : halfedges(mesh))
+  std::vector<halfedge_descriptor> border_cycles;
+
+  // collect one halfedge per boundary cycle
+  CGAL::Polygon_mesh_processing::extract_boundary_cycles(mesh, std::back_inserter(border_cycles));
+
+  for(halfedge_descriptor h : border_cycles)
   {
-    if(is_border(h,mesh))
-    {
-      std::vector<face_descriptor>  patch_facets;
-      std::vector<vertex_descriptor> patch_vertices;
-      bool success = std::get<0>(
-        CGAL::Polygon_mesh_processing::triangulate_refine_and_fair_hole(
-                  mesh,
-                  h,
-                  std::back_inserter(patch_facets),
-                  std::back_inserter(patch_vertices),
-     CGAL::Polygon_mesh_processing::parameters::vertex_point_map(get(CGAL::vertex_point, mesh)).
-                  geom_traits(Kernel())) );
-
-      std::cout << "* Number of facets in constructed patch: " << patch_facets.size() << std::endl;
-      std::cout << "  Number of vertices in constructed patch: " << patch_vertices.size() << std::endl;
-      std::cout << "  Is fairing successful: " << success << std::endl;
-      nb_holes++;
-    }
+    if(max_hole_diam > 0 && max_num_hole_edges > 0 &&
+       !is_small_hole(h, mesh, max_hole_diam, max_num_hole_edges))
+      continue;
+
+    std::vector<face_descriptor>  patch_facets;
+    std::vector<vertex_descriptor> patch_vertices;
+    bool success = std::get<0>(
+      CGAL::Polygon_mesh_processing::triangulate_refine_and_fair_hole(
+                mesh,
+                h,
+                std::back_inserter(patch_facets),
+                std::back_inserter(patch_vertices)) );
+
+    std::cout << "* Number of facets in constructed patch: " << patch_facets.size() << std::endl;
+    std::cout << "  Number of vertices in constructed patch: " << patch_vertices.size() << std::endl;
+    std::cout << "  Is fairing successful: " << success << std::endl;
+    ++nb_holes;
   }
 
   std::cout << std::endl;
   std::cout << nb_holes << " holes have been filled" << std::endl;
 
-  std::ofstream out("filled_SM.off");
+  std::string outfile = "filled_SM.off";
+  std::ofstream out(outfile.c_str());
+  std::cout << "Mesh written to: " << outfile << std::endl;
   out.precision(17);
   out << mesh << std::endl;
   return 0;
diff -pruN 5.0.3-1/examples/Polygon_mesh_processing/isotropic_remeshing_of_patch_example.cpp 5.1-1/examples/Polygon_mesh_processing/isotropic_remeshing_of_patch_example.cpp
--- 5.0.3-1/examples/Polygon_mesh_processing/isotropic_remeshing_of_patch_example.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Polygon_mesh_processing/isotropic_remeshing_of_patch_example.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -74,6 +74,7 @@ int main(int argc, char* argv[])
                            );
 
   std::ofstream out("out.off");
+  out.precision(17);
   out << mesh;
   std::cout << "Remeshing done." << std::endl;
 
diff -pruN 5.0.3-1/examples/Polygon_mesh_processing/manifoldness_repair_example.cpp 5.1-1/examples/Polygon_mesh_processing/manifoldness_repair_example.cpp
--- 5.0.3-1/examples/Polygon_mesh_processing/manifoldness_repair_example.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Polygon_mesh_processing/manifoldness_repair_example.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -39,7 +39,7 @@ int main(int argc, char* argv[])
   Mesh mesh;
   if(!input || !(input >> mesh) || num_vertices(mesh) == 0)
   {
-    std::cerr << filename << " is not a valid off file.\n";
+    std::cerr << filename << " is not a valid off file." << std::endl;
     return EXIT_FAILURE;
   }
 
diff -pruN 5.0.3-1/examples/Polygon_mesh_processing/mesh_smoothing_example.cpp 5.1-1/examples/Polygon_mesh_processing/mesh_smoothing_example.cpp
--- 5.0.3-1/examples/Polygon_mesh_processing/mesh_smoothing_example.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Polygon_mesh_processing/mesh_smoothing_example.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -48,6 +48,7 @@ int main(int argc, char** argv)
                                          .edge_is_constrained_map(eif));
 
   std::ofstream output("mesh_smoothed.off");
+  output.precision(17);
   output << mesh;
 
   std::cout << "Done!" << std::endl;
diff -pruN 5.0.3-1/examples/Polygon_mesh_processing/orientation_pipeline_example.cpp 5.1-1/examples/Polygon_mesh_processing/orientation_pipeline_example.cpp
--- 5.0.3-1/examples/Polygon_mesh_processing/orientation_pipeline_example.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Polygon_mesh_processing/orientation_pipeline_example.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,114 @@
+#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
+
+#include <CGAL/Surface_mesh.h>
+
+#include <CGAL/Polygon_mesh_processing/orientation.h>
+#include <CGAL/Polygon_mesh_processing/polygon_soup_to_polygon_mesh.h>
+#include <CGAL/Polygon_mesh_processing/orient_polygon_soup_extension.h>
+
+
+#include <CGAL/algorithm.h>
+#include <CGAL/Timer.h>
+
+#include <fstream>
+#include <algorithm>
+#include <cstdlib>
+
+int main(int argc, char** argv)
+{
+  typedef CGAL::Exact_predicates_inexact_constructions_kernel K;
+  typedef K::Point_3 Point_3;
+  typedef CGAL::Surface_mesh<Point_3> Mesh;
+
+  std::vector<Point_3> points;
+  std::vector< std::vector<std::size_t> > polygons;
+  Mesh ref1;
+  const char* input_filename = argc<2 ? "data/blobby-shuffled.off" : argv[1];
+  std::ifstream input(input_filename);
+  if ( !input){
+    std::cerr << "Error: can not read input file.\n";
+    return 1;
+  }
+  typedef  K::Point_3 Point_3;
+  CGAL::File_scanner_OFF scanner(input);
+  points.resize(scanner.size_of_vertices());
+  polygons.resize(scanner.size_of_facets());
+  //read points
+  for (std::size_t i = 0; i < scanner.size_of_vertices(); ++i)
+  {
+    double x, y, z, w;
+    scanner.scan_vertex( x, y, z, w);
+    points[i] = Point_3(x, y, z, w);
+    scanner.skip_to_next_vertex( i);
+  }
+  //read triangles
+  for (std::size_t i = 0; i < scanner.size_of_facets(); ++i)
+   {
+     std::size_t no;
+     scanner.scan_facet( no, i);
+     polygons[i].resize(no);
+
+     for(std::size_t j = 0; j < no; ++j) {
+       std::size_t id;
+       scanner.scan_facet_vertex_index(id, i);
+       if(id < scanner.size_of_vertices())
+       {
+         polygons[i][j] = id;
+       }
+       else
+       {
+         std::cerr << "Error: input file not valid.\n";
+         return 1;
+       }
+     }
+   }
+  input.close();
+
+  if(points.size() == 0 || polygons.size()==0)
+  {
+    std::cerr << "Error: input file not valid.\n";
+    return 1;
+  }
+
+  const char* reference_filename = argc<2 ? "data/blobby.off" : argv[2];
+  input.open(reference_filename);
+
+  if ( !input || !(input >> ref1)){
+    std::cerr << "Error: can not read reference file.\n";
+    return 1;
+  }
+  input.close();
+
+  std::cout << "Is the soup a polygon mesh ? : "
+            << CGAL::Polygon_mesh_processing::is_polygon_soup_a_polygon_mesh(polygons)
+            << std::endl;
+
+  CGAL::Polygon_mesh_processing::orient_triangle_soup_with_reference_triangle_mesh<CGAL::Sequential_tag>(ref1, points, polygons);
+
+  std::cout << "And now ? : "
+            << CGAL::Polygon_mesh_processing::is_polygon_soup_a_polygon_mesh(polygons)
+            << std::endl;
+
+  CGAL::Polygon_mesh_processing::duplicate_non_manifold_edges_in_polygon_soup(points, polygons);
+
+  std::cout << "And now ? : "
+            << CGAL::Polygon_mesh_processing::is_polygon_soup_a_polygon_mesh(polygons)
+            << std::endl;
+
+  Mesh poly;
+  CGAL::Polygon_mesh_processing::polygon_soup_to_polygon_mesh(
+        points, polygons, poly);
+
+  typedef boost::property_map<Mesh, CGAL::dynamic_face_property_t<std::size_t> >::type Fccmap;
+  Fccmap fccmap = get(CGAL::dynamic_face_property_t<std::size_t>(), poly);
+
+  std::cout<<CGAL::Polygon_mesh_processing::
+         connected_components(poly, fccmap)<<" CCs before merge."<<std::endl;
+  CGAL::Polygon_mesh_processing::
+      merge_reversible_connected_components(poly);
+
+
+  std::cout<<CGAL::Polygon_mesh_processing::
+         connected_components(poly, fccmap)<<" remaining CCs."<<std::endl;
+  return 0;
+}
diff -pruN 5.0.3-1/examples/Polygon_mesh_processing/orient_polygon_soup_example.cpp 5.1-1/examples/Polygon_mesh_processing/orient_polygon_soup_example.cpp
--- 5.0.3-1/examples/Polygon_mesh_processing/orient_polygon_soup_example.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Polygon_mesh_processing/orient_polygon_soup_example.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -44,11 +44,13 @@ int main(int argc, char* argv[])
     CGAL::Polygon_mesh_processing::orient_to_bound_a_volume(mesh);
 
   std::ofstream out("tet-oriented1.off");
+  out.precision(17);
   out << mesh;
   out.close();
 
   CGAL::Polygon_mesh_processing::reverse_face_orientations(mesh);
   std::ofstream out2("tet-oriented2.off");
+  out2.precision(17);
   out2 << mesh;
   out2.close();
 
diff -pruN 5.0.3-1/examples/Polygon_mesh_processing/random_perturbation_SM_example.cpp 5.1-1/examples/Polygon_mesh_processing/random_perturbation_SM_example.cpp
--- 5.0.3-1/examples/Polygon_mesh_processing/random_perturbation_SM_example.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Polygon_mesh_processing/random_perturbation_SM_example.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -33,6 +33,7 @@ int main(int argc, char* argv[])
     PMP::parameters::vertex_point_map(mesh.points()).geom_traits(K()));
 
   std::ofstream out("data/eight_perturbed.off");
+  out.precision(17);
   out << mesh;
   out.close();
 
diff -pruN 5.0.3-1/examples/Polygon_mesh_processing/refine_fair_example.cpp 5.1-1/examples/Polygon_mesh_processing/refine_fair_example.cpp
--- 5.0.3-1/examples/Polygon_mesh_processing/refine_fair_example.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Polygon_mesh_processing/refine_fair_example.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -62,6 +62,7 @@ int main(int argc, char* argv[])
                   CGAL::Polygon_mesh_processing::parameters::density_control_factor(2.));
 
   std::ofstream refined_off("refined.off");
+  refined_off.precision(17);
   refined_off << poly;
   refined_off.close();
   std::cout << "Refinement added " << new_vertices.size() << " vertices." << std::endl;
@@ -75,6 +76,7 @@ int main(int argc, char* argv[])
   std::cout << "Fairing : " << (success ? "succeeded" : "failed") << std::endl;
 
   std::ofstream faired_off("faired.off");
+  faired_off.precision(17);
   faired_off << poly;
   faired_off.close();
 
diff -pruN 5.0.3-1/examples/Polygon_mesh_processing/repair_polygon_soup_example.cpp 5.1-1/examples/Polygon_mesh_processing/repair_polygon_soup_example.cpp
--- 5.0.3-1/examples/Polygon_mesh_processing/repair_polygon_soup_example.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Polygon_mesh_processing/repair_polygon_soup_example.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -5,30 +5,55 @@
 #include <CGAL/Polygon_mesh_processing/orient_polygon_soup.h>
 #include <CGAL/Polygon_mesh_processing/polygon_soup_to_polygon_mesh.h>
 
+#include <algorithm>
+#include <array>
 #include <iostream>
 #include <vector>
 
 typedef CGAL::Exact_predicates_inexact_constructions_kernel     K;
+typedef K::FT                                                   FT;
 typedef K::Point_3                                              Point_3;
 
-typedef std::vector<std::size_t>                                CGAL_Polygon;
 typedef CGAL::Surface_mesh<Point_3>                             Mesh;
 
+typedef std::array<FT, 3>                                       Custom_point;
+typedef std::vector<std::size_t>                                CGAL_Polygon;
+
 namespace PMP = CGAL::Polygon_mesh_processing;
 
+struct Array_traits
+{
+  struct Equal_3
+  {
+    bool operator()(const Custom_point& p, const Custom_point& q) const {
+      return (p == q);
+    }
+  };
+
+  struct Less_xyz_3
+  {
+    bool operator()(const Custom_point& p, const Custom_point& q) const {
+      return std::lexicographical_compare(p.begin(), p.end(), q.begin(), q.end());
+    }
+  };
+
+  Equal_3 equal_3_object() const { return Equal_3(); }
+  Less_xyz_3 less_xyz_3_object() const { return Less_xyz_3(); }
+};
+
 int main(int, char**)
 {
   // First, construct a polygon soup with some problems
-  std::vector<Point_3> points;
+  std::vector<std::array<FT, 3> > points;
   std::vector<CGAL_Polygon> polygons;
 
-  points.push_back(Point_3(0,0,0));
-  points.push_back(Point_3(1,0,0));
-  points.push_back(Point_3(0,1,0));
-  points.push_back(Point_3(-1,0,0));
-  points.push_back(Point_3(0,-1,0));
-  points.push_back(Point_3(0,1,0)); // duplicate point
-  points.push_back(Point_3(0,-2,0)); // unused point
+  points.push_back(CGAL::make_array<FT>(0,0,0));
+  points.push_back(CGAL::make_array<FT>(1,0,0));
+  points.push_back(CGAL::make_array<FT>(0,1,0));
+  points.push_back(CGAL::make_array<FT>(-1,0,0));
+  points.push_back(CGAL::make_array<FT>(0,-1,0));
+  points.push_back(CGAL::make_array<FT>(0,1,0)); // duplicate point
+  points.push_back(CGAL::make_array<FT>(0,-2,0)); // unused point
 
   CGAL_Polygon p;
   p.push_back(0); p.push_back(1); p.push_back(2);
@@ -67,10 +92,12 @@ int main(int, char**)
   p.push_back(4); p.push_back(3); p.push_back(2); p.push_back(1);
   polygons.push_back(p);
 
-  PMP::repair_polygon_soup(points, polygons);
-  PMP::orient_polygon_soup(points, polygons);
+  std::cout << "Before reparation, the soup has " << points.size() << " vertices and " << polygons.size() << " faces" << std::endl;
+  PMP::repair_polygon_soup(points, polygons, CGAL::parameters::geom_traits(Array_traits()));
+  std::cout << "After reparation, the soup has " << points.size() << " vertices and " << polygons.size() << " faces" << std::endl;
 
   Mesh mesh;
+  PMP::orient_polygon_soup(points, polygons);
   PMP::polygon_soup_to_polygon_mesh(points, polygons, mesh);
 
   std::cout << "Mesh has " << num_vertices(mesh) << " vertices and " << num_faces(mesh) << " faces" << std::endl;
diff -pruN 5.0.3-1/examples/Polygon_mesh_processing/self_intersections_example.cpp 5.1-1/examples/Polygon_mesh_processing/self_intersections_example.cpp
--- 5.0.3-1/examples/Polygon_mesh_processing/self_intersections_example.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Polygon_mesh_processing/self_intersections_example.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -2,12 +2,15 @@
 #include <CGAL/Surface_mesh.h>
 
 #include <CGAL/Polygon_mesh_processing/self_intersections.h>
+#include <CGAL/Real_timer.h>
+#include <CGAL/tags.h>
 
+#include <iostream>
 #include <fstream>
 
 typedef CGAL::Exact_predicates_inexact_constructions_kernel K;
-typedef CGAL::Surface_mesh<K::Point_3>             Mesh;
-typedef boost::graph_traits<Mesh>::face_descriptor face_descriptor;
+typedef CGAL::Surface_mesh<K::Point_3>                      Mesh;
+typedef boost::graph_traits<Mesh>::face_descriptor          face_descriptor;
 
 namespace PMP = CGAL::Polygon_mesh_processing;
 
@@ -20,20 +23,25 @@ int main(int argc, char* argv[])
   if (!input || !(input >> mesh) || !CGAL::is_triangle_mesh(mesh))
   {
     std::cerr << "Not a valid input file." << std::endl;
-    return 1;
+    return EXIT_FAILURE;
   }
 
-  bool intersecting = PMP::does_self_intersect(mesh,
-      PMP::parameters::vertex_point_map(get(CGAL::vertex_point, mesh)));
+  std::cout << "Using parallel mode? " << std::is_same<CGAL::Parallel_if_available_tag, CGAL::Parallel_tag>::value << std::endl;
 
-  std::cout
-    << (intersecting ? "There are self-intersections." : "There is no self-intersection.")
-    << std::endl;
+  CGAL::Real_timer timer;
+  timer.start();
 
-  std::vector<std::pair<face_descriptor, face_descriptor> > intersected_tris;
-  PMP::self_intersections(mesh, std::back_inserter(intersected_tris));
+  bool intersecting = PMP::does_self_intersect<CGAL::Parallel_if_available_tag>(mesh, CGAL::parameters::vertex_point_map(get(CGAL::vertex_point, mesh)));
+  std::cout << (intersecting ? "There are self-intersections." : "There is no self-intersection.") << std::endl;
+  std::cout << "Elapsed time (does self intersect): " << timer.time() << std::endl;
+
+  timer.reset();
 
+  std::vector<std::pair<face_descriptor, face_descriptor> > intersected_tris;
+  PMP::self_intersections<CGAL::Parallel_if_available_tag>(faces(mesh), mesh, std::back_inserter(intersected_tris));
   std::cout << intersected_tris.size() << " pairs of triangles intersect." << std::endl;
 
-  return 0;
+  std::cout << "Elapsed time (self intersections): " << timer.time() << std::endl;
+
+  return EXIT_SUCCESS;
 }
diff -pruN 5.0.3-1/examples/Polygon_mesh_processing/shape_smoothing_example.cpp 5.1-1/examples/Polygon_mesh_processing/shape_smoothing_example.cpp
--- 5.0.3-1/examples/Polygon_mesh_processing/shape_smoothing_example.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Polygon_mesh_processing/shape_smoothing_example.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -41,6 +41,7 @@ int main(int argc, char* argv[])
                                                 .vertex_is_constrained_map(vcmap));
 
   std::ofstream output("mesh_shape_smoothed.off");
+  output.precision(17);
   output << mesh;
 
   std::cout << "Done!" << std::endl;
diff -pruN 5.0.3-1/examples/Polygon_mesh_processing/stitch_borders_example.cpp 5.1-1/examples/Polygon_mesh_processing/stitch_borders_example.cpp
--- 5.0.3-1/examples/Polygon_mesh_processing/stitch_borders_example.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Polygon_mesh_processing/stitch_borders_example.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -34,6 +34,7 @@ int main(int argc, char* argv[])
   std::cout << "\t Number of facets    :\t" << mesh.size_of_facets() << std::endl;
 
   std::ofstream output("mesh_stitched.off");
+  output.precision(17);
   output << std::setprecision(17) << mesh;
 
   return 0;
diff -pruN 5.0.3-1/examples/Polygon_mesh_processing/surface_mesh_intersection.cpp 5.1-1/examples/Polygon_mesh_processing/surface_mesh_intersection.cpp
--- 5.0.3-1/examples/Polygon_mesh_processing/surface_mesh_intersection.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Polygon_mesh_processing/surface_mesh_intersection.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -24,7 +24,7 @@ int main(int argc, char* argv[])
 
   Mesh mesh1;
   if ( !input || !(input >> mesh1) ) {
-    std::cerr << filename1 << " is not a valid off file.\n";
+    std::cerr << filename1 << " is not a valid off file." << std::endl;
     return 1;
   }
   input.close();
@@ -34,7 +34,7 @@ int main(int argc, char* argv[])
 
   Mesh mesh2;
   if ( !input || !(input >> mesh2) ) {
-    std::cerr << filename2 << " is not a valid off file.\n";
+    std::cerr << filename2 << " is not a valid off file." << std::endl;
     return 1;
   }
 
@@ -43,6 +43,7 @@ int main(int argc, char* argv[])
 
   //dump polylines
   std::ofstream output("intersection_polylines.cgal");
+  output.precision(17);
   for(const std::vector<Point>& polyline : polylines)
   {
     output << polyline.size() << " ";
diff -pruN 5.0.3-1/examples/Polygon_mesh_processing/triangulate_faces_example.cpp 5.1-1/examples/Polygon_mesh_processing/triangulate_faces_example.cpp
--- 5.0.3-1/examples/Polygon_mesh_processing/triangulate_faces_example.cpp	2020-08-01 19:00:33.000000000 +0000
+++ 5.1-1/examples/Polygon_mesh_processing/triangulate_faces_example.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -32,6 +32,7 @@ int main(int argc, char* argv[])
       std::cerr << "Error: non-triangular face left in mesh." << std::endl;
 
   std::ofstream cube_off(outfilename);
+  cube_off.precision(17);
   cube_off << mesh;
 
   return 0;
diff -pruN 5.0.3-1/examples/Polygon_mesh_processing/volume_connected_components.cpp 5.1-1/examples/Polygon_mesh_processing/volume_connected_components.cpp
--- 5.0.3-1/examples/Polygon_mesh_processing/volume_connected_components.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Polygon_mesh_processing/volume_connected_components.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,53 @@
+#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
+#include <CGAL/Surface_mesh.h>
+#include <CGAL/Polygon_mesh_processing/orientation.h>
+#include <CGAL/boost/graph/Face_filtered_graph.h>
+
+#include <iostream>
+#include <fstream>
+#include <sstream>
+
+typedef CGAL::Exact_predicates_inexact_constructions_kernel Kernel;
+typedef CGAL::Surface_mesh<Kernel::Point_3> Surface_mesh;
+
+namespace PMP = CGAL::Polygon_mesh_processing;
+namespace params = CGAL::parameters;
+
+int main(int argc, char** argv)
+{
+  const char* filename = (argc > 1) ? argv[1] : "data/blobby.off";
+  std::ifstream input(filename);
+  assert(input);
+  Surface_mesh sm;
+  input >> sm;
+
+  // property map to assign a volume id for each face
+  Surface_mesh::Property_map<Surface_mesh::Face_index, std::size_t> vol_id_map =
+    sm.add_property_map<Surface_mesh::Face_index, std::size_t>().first;
+
+  std::vector<PMP::Volume_error_code> err_codes;
+  // fill the volume id map
+  std::size_t nb_vol =
+    PMP::volume_connected_components(sm,
+                                     vol_id_map,
+                                     params::error_codes(std::ref(err_codes)));
+
+  std::cout << "Found " << nb_vol << " volumes\n";
+
+  // write each volume in an OFF file
+  typedef CGAL::Face_filtered_graph<Surface_mesh> Filtered_graph;
+  Filtered_graph vol_mesh(sm, 0, vol_id_map);
+  for(std::size_t id = 0; id < nb_vol; ++id)
+  {
+    if (err_codes[id] != PMP::VALID_VOLUME)
+      std::cerr << "There is an issue with volume #" << id << "\n";
+    if(id > 0)
+      vol_mesh.set_selected_faces(id, vol_id_map);
+    Surface_mesh out;
+    CGAL::copy_face_graph(vol_mesh, out);
+    std::ostringstream oss;
+    oss << "vol_" << id <<".off";
+    std::ofstream os(oss.str().data());
+    os << out;
+  }
+}
diff -pruN 5.0.3-1/examples/Polyhedron/polyhedron_prog_cut_cube.cpp 5.1-1/examples/Polyhedron/polyhedron_prog_cut_cube.cpp
--- 5.0.3-1/examples/Polyhedron/polyhedron_prog_cut_cube.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Polyhedron/polyhedron_prog_cut_cube.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -66,6 +66,6 @@ int main() {
                       Point( 0.0, 0.0, 1.5),
                       Point( 0.0, 0.5, 0.0));
     CGAL::polyhedron_cut_plane_3( P, h, pl);
-    std::cout << P;
+    std::cout << std::setprecision(17)<< P;
     return 0;
 }
diff -pruN 5.0.3-1/examples/Polyhedron/polyhedron_prog_subdiv.cpp 5.1-1/examples/Polyhedron/polyhedron_prog_subdiv.cpp
--- 5.0.3-1/examples/Polyhedron/polyhedron_prog_subdiv.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Polyhedron/polyhedron_prog_subdiv.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -102,6 +102,6 @@ int main(int argc, char* argv[]) {
         std::exit(1);
     }
     subdiv( P);
-    std::cout << P;
+    std::cout << std::setprecision(17) << P;
     return 0;
 }
diff -pruN 5.0.3-1/examples/Polyhedron/polyhedron_prog_subdiv_with_boundary.cpp 5.1-1/examples/Polyhedron/polyhedron_prog_subdiv_with_boundary.cpp
--- 5.0.3-1/examples/Polyhedron/polyhedron_prog_subdiv_with_boundary.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Polyhedron/polyhedron_prog_subdiv_with_boundary.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -196,6 +196,6 @@ int main( int argc, char* argv[]) {
         if ( i & 1)
             subdiv_border( P);
     }
-    cout << P;
+    cout << std::setprecision(17) << P;
     return 0;
 }
diff -pruN 5.0.3-1/examples/Polyhedron/polyhedron_prog_vertex_normal.cpp 5.1-1/examples/Polyhedron/polyhedron_prog_vertex_normal.cpp
--- 5.0.3-1/examples/Polyhedron/polyhedron_prog_vertex_normal.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Polyhedron/polyhedron_prog_vertex_normal.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -50,7 +50,12 @@ class My_vertex : public CGAL::HalfedgeD
     Norm  norm;
 public:
     My_vertex() {} // repeat mandatory constructors
-    My_vertex( const P& pt) : CGAL::HalfedgeDS_vertex_base<Refs, T, P>(pt) {}
+    My_vertex( const P& pt)
+      : CGAL::HalfedgeDS_vertex_base<Refs, T, P>(pt)
+    {
+      //initialization of the normal
+      norm= CGAL::NULL_VECTOR;
+    }
     typedef Norm Normal_3;
     Normal_3&       normal()       { return norm; }
     const Normal_3& normal() const { return norm; }
diff -pruN 5.0.3-1/examples/Principal_component_analysis/CMakeLists.txt 5.1-1/examples/Principal_component_analysis/CMakeLists.txt
--- 5.0.3-1/examples/Principal_component_analysis/CMakeLists.txt	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Principal_component_analysis/CMakeLists.txt	2020-09-08 12:12:20.000000000 +0000
@@ -12,14 +12,16 @@ if ( CGAL_FOUND )
 
   # Use Eigen
   find_package(Eigen3 3.1.0) #(requires 3.1.0 or greater)
-  if (EIGEN3_FOUND)
-    include( ${EIGEN3_USE_FILE} )
-  endif()
+  include(CGAL_Eigen_support)
 
   # create a target per cppfile
   file(GLOB cppfiles RELATIVE ${CMAKE_CURRENT_SOURCE_DIR} ${CMAKE_CURRENT_SOURCE_DIR}/*.cpp)
   foreach(cppfile ${cppfiles})
     create_single_source_cgal_program( "${cppfile}" )
+    if (TARGET CGAL::Eigen_support)
+      get_filename_component(target ${cppfile} NAME_WE)
+      target_link_libraries(${target} PUBLIC CGAL::Eigen_support)
+    endif()
   endforeach()
 
 else()
@@ -27,4 +29,3 @@ else()
     message(STATUS "This program requires the CGAL library, and will not be compiled.")
 
 endif()
-
diff -pruN 5.0.3-1/examples/Property_map/CMakeLists.txt 5.1-1/examples/Property_map/CMakeLists.txt
--- 5.0.3-1/examples/Property_map/CMakeLists.txt	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Property_map/CMakeLists.txt	2020-09-08 12:12:20.000000000 +0000
@@ -32,4 +32,9 @@ endif()
 
 create_single_source_cgal_program( "dynamic_properties.cpp" )
 
-
+find_package(Eigen3 3.1.0) #(requires 3.1.0 or greater)
+include(CGAL_Eigen_support)
+if (TARGET CGAL::Eigen_support)
+  create_single_source_cgal_program( "custom_property_map.cpp" )
+  target_link_libraries(custom_property_map PUBLIC CGAL::Eigen_support)
+endif()
diff -pruN 5.0.3-1/examples/Property_map/custom_property_map.cpp 5.1-1/examples/Property_map/custom_property_map.cpp
--- 5.0.3-1/examples/Property_map/custom_property_map.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Property_map/custom_property_map.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,112 @@
+#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
+#include <CGAL/point_generators_3.h>
+#include <CGAL/jet_estimate_normals.h>
+#include <CGAL/mst_orient_normals.h>
+
+using Kernel = CGAL::Exact_predicates_inexact_constructions_kernel;
+using Point_3 = Kernel::Point_3;
+using Vector_3 = Kernel::Vector_3;
+using Generator = CGAL::Random_points_on_sphere_3<Point_3>;
+
+// Example of readable property map to get CGAL::Point_3 objects from
+// 3 coordinate arrays
+struct Custom_point_map
+{
+  using key_type = std::size_t; // The iterator's value type is an index
+  using value_type = Point_3;   // The object manipulated by the algorithm is a Point_3
+  using reference = Point_3;    // The object does not exist in memory, so there's no reference
+  using category = boost::readable_property_map_tag; // The property map is only used for reading
+
+  double *x, *y, *z;
+
+  Custom_point_map (double* x = nullptr, double* y = nullptr, double* z = nullptr)
+    : x(x), y(y), z(z) { }
+
+  // The get() function returns the object expected by the algorithm (here, Point_3)
+  friend Point_3 get (const Custom_point_map& map, std::size_t idx)
+  {
+    return Point_3 (map.x[idx], map.y[idx], map.z[idx]);
+  }
+};
+
+// Example of read-write property map to get CGAL::Vector_3 objects from
+// a buffer array and put CGAL::Vector_3 values in this buffer
+struct Custom_normal_map
+{
+  using key_type = std::size_t; // The iterator's value type is an index
+  using value_type = Vector_3;  // The object manipulated by the algorithm is a Vector_3
+  using reference = Vector_3;   // The object does not exist in memory, so there's no reference
+  using category = boost::read_write_property_map_tag; // The property map is used both
+                                                       // for reading and writing data
+  double *buffer;
+
+  Custom_normal_map (double* buffer = nullptr)
+    : buffer (buffer) { }
+
+  // The get() function returns the object expected by the algorithm (here, Vector_3)
+  friend Vector_3 get (const Custom_normal_map& map, std::size_t idx)
+  {
+    return Vector_3 (map.buffer[idx * 3    ],
+                     map.buffer[idx * 3 + 1],
+                     map.buffer[idx * 3 + 2]);
+  }
+
+  // The put() function updated the user's data structure from the
+  // object handled by the algorithm (here Vector_3)
+  friend void put (const Custom_normal_map& map, std::size_t idx, const Vector_3& vector_3)
+  {
+    map.buffer[idx * 3    ] = vector_3.x();
+    map.buffer[idx * 3 + 1] = vector_3.y();
+    map.buffer[idx * 3 + 2] = vector_3.z();
+  }
+};
+
+
+int main()
+{
+  constexpr std::size_t nb_points = 1000;
+
+  // in this example, points are stored as separate coordinate arrays
+  double x[nb_points];
+  double y[nb_points];
+  double z[nb_points];
+
+  // generate random points
+  Generator generator;
+  for (std::size_t i = 0; i < nb_points; ++ i)
+  {
+    Point_3 p = *(generator ++ );
+    x[i] = p.x();
+    y[i] = p.y();
+    z[i] = p.z();
+  }
+
+  // normals are stored as a contiguous double array
+  double normals[3 *nb_points];
+
+  // we use a vector of indices to access arrays
+  std::vector<std::size_t> indices;
+  indices.reserve (nb_points);
+  for (std::size_t i = 0; i < nb_points; ++ i)
+    indices.push_back(i);
+
+  // estimate and orient normals using directly user's data structure
+  // instead of creating deep copies using Point_3 and Vector_3
+  CGAL::jet_estimate_normals<CGAL::Sequential_tag>
+    (indices, 12,
+     CGAL::parameters::point_map (Custom_point_map(x,y,z)).
+     normal_map (Custom_normal_map(normals)));
+
+  CGAL::mst_orient_normals
+    (indices, 12,
+     CGAL::parameters::point_map (Custom_point_map(x,y,z)).
+     normal_map (Custom_normal_map(normals)));
+
+  // Display first 10 points+normals
+  for (std::size_t i = 0; i < 10; ++ i)
+    std::cerr << "Point(" << i << ") = " << x[i] << " " << y[i] << " " << z[i]
+              << "\tNormal(" << i << ") = "
+              << normals[3*i] << " " << normals[3*i+1] << " " << normals[3*i+2] << std::endl;
+
+  return EXIT_SUCCESS;
+}
diff -pruN 5.0.3-1/examples/RangeSegmentTrees/include/Tree_Traits.h 5.1-1/examples/RangeSegmentTrees/include/Tree_Traits.h
--- 5.0.3-1/examples/RangeSegmentTrees/include/Tree_Traits.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/examples/RangeSegmentTrees/include/Tree_Traits.h	2020-09-08 12:15:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/SearchStructures/examples/RangeSegmentTrees/include/Tree_Traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/SearchStructures/examples/RangeSegmentTrees/include/Tree_Traits.h $
 // $Id: Tree_Traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/examples/Ridges_3/CMakeLists.txt 5.1-1/examples/Ridges_3/CMakeLists.txt
--- 5.0.3-1/examples/Ridges_3/CMakeLists.txt	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Ridges_3/CMakeLists.txt	2020-09-08 12:12:20.000000000 +0000
@@ -10,8 +10,8 @@ if ( CGAL_FOUND )
 
   # use either Eigen
   find_package(Eigen3 3.1.0) #(requires 3.1.0 or greater)
-  if (EIGEN3_FOUND)
-    include( ${EIGEN3_USE_FILE} )
+  include(CGAL_Eigen_support)
+  if (TARGET CGAL::Eigen_support)
 
     # Link with Boost.ProgramOptions (optional)
     find_package(Boost QUIET COMPONENTS program_options)
@@ -29,8 +29,11 @@ if ( CGAL_FOUND )
    endif()
 
    create_single_source_cgal_program( Compute_Ridges_Umbilics.cpp)
+   target_link_libraries(Compute_Ridges_Umbilics PUBLIC CGAL::Eigen_support)
    create_single_source_cgal_program( Ridges_Umbilics_SM.cpp)
+   target_link_libraries(Ridges_Umbilics_SM PUBLIC CGAL::Eigen_support)
    create_single_source_cgal_program( Ridges_Umbilics_LCC.cpp)
+   target_link_libraries(Ridges_Umbilics_LCC PUBLIC CGAL::Eigen_support)
 
   else()
 
diff -pruN 5.0.3-1/examples/Scale_space_reconstruction_3/CMakeLists.txt 5.1-1/examples/Scale_space_reconstruction_3/CMakeLists.txt
--- 5.0.3-1/examples/Scale_space_reconstruction_3/CMakeLists.txt	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Scale_space_reconstruction_3/CMakeLists.txt	2020-09-08 12:12:20.000000000 +0000
@@ -13,23 +13,28 @@ if ( CGAL_FOUND )
 
   if( ACTIVATE_CONCURRENCY )
     find_package( TBB )
-    if( NOT TBB_FOUND )
+    include(CGAL_TBB_support)
+    if( NOT TARGET CGAL::TBB_support )
       message( STATUS "NOTICE: Intel TBB NOT found! The example is faster if TBB is linked." )
     endif()
   endif()
 
   find_package( Eigen3 3.1.0 )
-  if( EIGEN3_FOUND )
-    include( ${EIGEN3_USE_FILE} )
+  include(CGAL_Eigen_support)
+  if( TARGET CGAL::Eigen_support )
     create_single_source_cgal_program( "scale_space.cpp" )
+    target_link_libraries(scale_space PUBLIC CGAL::Eigen_support)
     create_single_source_cgal_program( "scale_space_incremental.cpp" )
+    target_link_libraries(scale_space_incremental PUBLIC CGAL::Eigen_support)
     create_single_source_cgal_program( "scale_space_manifold.cpp" )
+    target_link_libraries(scale_space_manifold PUBLIC CGAL::Eigen_support)
     create_single_source_cgal_program( "scale_space_advancing_front.cpp" )
-    if(ACTIVATE_CONCURRENCY AND TBB_FOUND)
-      CGAL_target_use_TBB(scale_space)
-      CGAL_target_use_TBB(scale_space_incremental)
-      CGAL_target_use_TBB(scale_space_manifold)
-      CGAL_target_use_TBB(scale_space_advancing_front)
+    target_link_libraries(scale_space_advancing_front PUBLIC CGAL::Eigen_support)
+    if(ACTIVATE_CONCURRENCY AND TARGET CGAL::TBB_support)
+      target_link_libraries(scale_space PUBLIC CGAL::TBB_support)
+      target_link_libraries(scale_space_incremental PUBLIC CGAL::TBB_support)
+      target_link_libraries(scale_space_manifold PUBLIC CGAL::TBB_support)
+      target_link_libraries(scale_space_advancing_front PUBLIC CGAL::TBB_support)
     endif()
   else()
     message( STATUS "NOTICE: The example needs Eigen 3.1 (or greater) and will not be compiled." )
diff -pruN 5.0.3-1/examples/Shape_detection/CMakeLists.txt 5.1-1/examples/Shape_detection/CMakeLists.txt
--- 5.0.3-1/examples/Shape_detection/CMakeLists.txt	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Shape_detection/CMakeLists.txt	2020-09-08 12:12:20.000000000 +0000
@@ -14,9 +14,8 @@ if(CGAL_FOUND)
 
   # Use Eigen.
   find_package(Eigen3 3.1.0 QUIET) # (3.1.0 or greater)
-  if(EIGEN3_FOUND)
-    include(${EIGEN3_USE_FILE})
-
+  include(CGAL_Eigen_support)
+  if(TARGET CGAL::Eigen_support)
     create_single_source_cgal_program(
       "efficient_RANSAC_basic.cpp")
     create_single_source_cgal_program(
@@ -35,6 +34,19 @@ if(CGAL_FOUND)
 
     create_single_source_cgal_program(
       "shape_detection_basic_deprecated.cpp")
+
+    foreach(target
+        efficient_RANSAC_basic
+        efficient_RANSAC_with_callback
+        efficient_RANSAC_and_plane_regularization
+        region_growing_on_point_set_2
+        region_growing_on_point_set_3
+        region_growing_on_polygon_mesh
+        region_growing_with_custom_classes
+        shape_detection_basic_deprecated)
+      target_link_libraries(${target} PUBLIC CGAL::Eigen_support)
+    endforeach()
+
   endif()
 
   create_single_source_cgal_program(
diff -pruN 5.0.3-1/examples/Solver_interface/CMakeLists.txt 5.1-1/examples/Solver_interface/CMakeLists.txt
--- 5.0.3-1/examples/Solver_interface/CMakeLists.txt	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Solver_interface/CMakeLists.txt	2020-09-08 12:12:20.000000000 +0000
@@ -12,40 +12,35 @@ if ( CGAL_FOUND )
 
   # Use Eigen
   find_package(Eigen3 3.1.0) #(requires 3.1.0 or greater)
+  include(CGAL_Eigen_support)
 
-  if (EIGEN3_FOUND)
-    include( ${EIGEN3_USE_FILE} )
+  if (TARGET CGAL::Eigen_support)
     create_single_source_cgal_program( "singular_value_decomposition.cpp" )
+    target_link_libraries(singular_value_decomposition PUBLIC CGAL::Eigen_support)
     create_single_source_cgal_program( "sparse_solvers.cpp" )
+    target_link_libraries(sparse_solvers PUBLIC CGAL::Eigen_support)
+    create_single_source_cgal_program( "diagonalize_matrix.cpp" )
+    target_link_libraries(diagonalize_matrix PUBLIC CGAL::Eigen_support)
   endif()
 
-  create_single_source_cgal_program( "diagonalize_matrix.cpp" )
   create_single_source_cgal_program( "mixed_integer_program.cpp" )
 
   find_package( SCIP QUIET)
+  include(CGAL_SCIP_support)
 
-  if (SCIP_FOUND)
-
-    include_directories( BEFORE ${SCIP_INCLUDE_DIRS} )
-
-    target_link_libraries( mixed_integer_program PRIVATE ${SCIP_LIBRARIES} )
-
-    target_compile_definitions(mixed_integer_program PRIVATE -DCGAL_USE_SCIP)
+  if (TARGET CGAL::SCIP_support)
 
+    target_link_libraries(mixed_integer_program PUBLIC CGAL::SCIP_support)
     message("SCIP found and used")
 
   else()
 
-      find_package( GLPK QUIET)
-
-      if (GLPK_FOUND)
+    find_package( GLPK QUIET)
+    include(CGAL_GLPK_support)
 
-        include_directories( BEFORE ${GLPK_INCLUDE_DIR} )
-
-        target_link_libraries( mixed_integer_program PRIVATE ${GLPK_LIBRARIES} )
-
-        target_compile_definitions(mixed_integer_program PRIVATE -DCGAL_USE_GLPK)
+      if (TARGET CGAL::GLPK_support)
 
+        target_link_libraries(mixed_integer_program PUBLIC CGAL::GLPK_support)
         message("GLPK found and used")
 
       else()
@@ -62,4 +57,3 @@ else()
     message(STATUS "NOTICE: This program requires the CGAL library, and will not be compiled.")
 
 endif()
-
diff -pruN 5.0.3-1/examples/Spatial_searching/CMakeLists.txt 5.1-1/examples/Spatial_searching/CMakeLists.txt
--- 5.0.3-1/examples/Spatial_searching/CMakeLists.txt	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Spatial_searching/CMakeLists.txt	2020-09-08 12:12:20.000000000 +0000
@@ -18,9 +18,7 @@ if ( NOT CGAL_FOUND )
 endif()
 
 find_package(Eigen3 3.1.91) #(requires 3.2.0 or greater)
-if (EIGEN3_FOUND)
-  include( ${EIGEN3_USE_FILE} )
-endif()
+include(CGAL_Eigen_support)
 
 if (MSVC)
     # Turn off VC++ warning
@@ -65,11 +63,13 @@ create_single_source_cgal_program( "weig
 
 
 
-if (EIGEN3_FOUND)
+if (TARGET CGAL::Eigen_support)
 
   create_single_source_cgal_program( "fuzzy_range_query.cpp" )
+  target_link_libraries(fuzzy_range_query PUBLIC CGAL::Eigen_support)
 
   create_single_source_cgal_program( "general_neighbor_searching.cpp" )
+  target_link_libraries(general_neighbor_searching PUBLIC CGAL::Eigen_support)
 
 else()
 
@@ -77,3 +77,13 @@ else()
   message(STATUS "will not be compiled as they use CGAL::Epick_d which requires the Eigen library.")
 
 endif()
+
+find_package( TBB QUIET )
+include(CGAL_TBB_support)
+if(TARGET CGAL::TBB_support)
+  create_single_source_cgal_program( "parallel_kdtree.cpp" )
+  target_link_libraries(parallel_kdtree PUBLIC CGAL::TBB_support)
+else()
+  message(STATUS "parallel_kdtree.cpp requires TBB and will not be compiled")
+endif()
+
diff -pruN 5.0.3-1/examples/Spatial_searching/Distance.h 5.1-1/examples/Spatial_searching/Distance.h
--- 5.0.3-1/examples/Spatial_searching/Distance.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/examples/Spatial_searching/Distance.h	2020-09-08 12:15:39.000000000 +0000
@@ -1,5 +1,6 @@
 struct Distance {
   typedef Point Query_item;
+  typedef Point Point_d;
   typedef double FT;
   typedef CGAL::Dimension_tag<3> D;
 
diff -pruN 5.0.3-1/examples/Spatial_searching/parallel_kdtree.cpp 5.1-1/examples/Spatial_searching/parallel_kdtree.cpp
--- 5.0.3-1/examples/Spatial_searching/parallel_kdtree.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Spatial_searching/parallel_kdtree.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,56 @@
+#include <CGAL/Simple_cartesian.h>
+#include <CGAL/point_generators_3.h>
+#include <CGAL/Orthogonal_k_neighbor_search.h>
+#include <CGAL/Search_traits_3.h>
+
+#include <tbb/blocked_range.h>
+#include <tbb/parallel_for.h>
+
+using Kernel = CGAL::Simple_cartesian<double>;
+using Point_3 = Kernel::Point_3;
+
+using Traits = CGAL::Search_traits_3<Kernel>;
+using Neighbor_search = CGAL::Orthogonal_k_neighbor_search<Traits>;
+using Tree = Neighbor_search::Tree;
+using Point_with_distance = Neighbor_search::Point_with_transformed_distance;
+
+using Generator = CGAL::Random_points_in_sphere_3<Point_3>;
+
+int main()
+{
+  const unsigned int N = 1000;
+  const unsigned int k = 6;
+
+  // Generate N points in a sphere
+  std::vector<Point_3> points;
+  points.reserve (N);
+  Generator generator;
+  for (unsigned int i = 0; i < N; ++ i)
+    points.push_back (*(generator++));
+
+  // Build tree in parallel
+  Tree tree(points.begin(), points.end());
+  tree.build<CGAL::Parallel_tag>();
+
+  // Query tree in parallel
+  std::vector<std::vector<Point_3> > neighbors (points.size());
+  tbb::parallel_for (tbb::blocked_range<std::size_t> (0, points.size()),
+                     [&](const tbb::blocked_range<std::size_t>& r)
+                     {
+                       for (std::size_t s = r.begin(); s != r.end(); ++ s)
+                       {
+                         // Neighbor search can be instantiated from
+                         // several threads at the same time
+                         Neighbor_search search (tree, points[s], k);
+                         neighbors[s].reserve(k);
+
+                         // neighbor search returns a set of pair of
+                         // point and distance <Point_3,FT>, here we
+                         // keep the points only
+                         for (const Point_with_distance& pwd : search)
+                           neighbors[s].push_back (pwd.first);
+                       }
+                     });
+
+  return 0;
+}
diff -pruN 5.0.3-1/examples/Spatial_sorting/CMakeLists.txt 5.1-1/examples/Spatial_sorting/CMakeLists.txt
--- 5.0.3-1/examples/Spatial_sorting/CMakeLists.txt	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Spatial_sorting/CMakeLists.txt	2020-09-08 12:12:20.000000000 +0000
@@ -1,4 +1,3 @@
-# Created by the script cgal_create_cmake_script
 # This is the CMake script for compiling a CGAL application.
 
 
@@ -16,9 +15,13 @@ if ( CGAL_FOUND )
     create_single_source_cgal_program( "${cppfile}" )
   endforeach()
 
+  find_package( TBB QUIET )
+  include(CGAL_TBB_support)
+  if( TARGET CGAL::TBB_support )
+    target_link_libraries( parallel_spatial_sort_3 PUBLIC CGAL::TBB_support)
+ endif()
 else()
 
     message(STATUS "This program requires the CGAL library, and will not be compiled.")
 
 endif()
-
diff -pruN 5.0.3-1/examples/Spatial_sorting/parallel_spatial_sort_3.cpp 5.1-1/examples/Spatial_sorting/parallel_spatial_sort_3.cpp
--- 5.0.3-1/examples/Spatial_sorting/parallel_spatial_sort_3.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Spatial_sorting/parallel_spatial_sort_3.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -0,0 +1,33 @@
+#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
+
+#include <CGAL/point_generators_3.h>
+#include <CGAL/Random.h>
+#include <CGAL/spatial_sort.h>
+
+#include <iostream>
+#include <vector>
+
+typedef CGAL::Exact_predicates_inexact_constructions_kernel       K;
+typedef K::Point_3                                                Point_3;
+typedef CGAL::Creator_uniform_3<double,Point_3>                   Creator_3;
+
+int main()
+{
+  std::size_t pt_nb = 10000;
+  std::vector<Point_3> points;
+  points.reserve(pt_nb);
+
+  CGAL::Random_points_in_cube_3<Point_3> gen(1.0);
+
+  for(std::size_t i=0; i<pt_nb; ++i)
+    points.push_back(*gen++);
+
+  // By default sequential
+  CGAL::spatial_sort(points.begin(),points.end());
+
+  // Add the template argument to switch on parallelism if available
+  // You can also use Parallel_tag if you know that TBB is enabled
+  CGAL::spatial_sort<CGAL::Parallel_if_available_tag>(points.begin(), points.end());
+
+  return 0;
+}
diff -pruN 5.0.3-1/examples/Surface_mesh/CMakeLists.txt 5.1-1/examples/Surface_mesh/CMakeLists.txt
--- 5.0.3-1/examples/Surface_mesh/CMakeLists.txt	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Surface_mesh/CMakeLists.txt	2020-09-08 12:12:20.000000000 +0000
@@ -5,7 +5,6 @@
 cmake_minimum_required(VERSION 3.1...3.15)
 project( Surface_mesh_Examples )
 
-
 if(POLICY CMP0053)
   # Only set CMP0053 to OLD with CMake<3.10, otherwise there is a warning.
   if(NOT POLICY CMP0070)
@@ -14,13 +13,16 @@ if(POLICY CMP0053)
     cmake_policy(SET CMP0053 NEW)
   endif()
 endif()
+
 if(POLICY CMP0071)
     cmake_policy(SET CMP0071 NEW)
 endif()
 
-find_package(CGAL COMPONENTS Qt5)
+#CGAL_Qt5 is needed for the drawing.
+find_package(CGAL REQUIRED OPTIONAL_COMPONENTS Qt5)
 
 if(CGAL_Qt5_FOUND)
+  #required to use basic_viewer
   add_definitions(-DCGAL_USE_BASIC_VIEWER -DQT_NO_KEYWORDS)
 endif()
 
@@ -37,10 +39,15 @@ if ( CGAL_FOUND )
   create_single_source_cgal_program( "sm_memory.cpp" )
   create_single_source_cgal_program( "sm_properties.cpp" )
 
+  #create the executable of the application
+
   create_single_source_cgal_program("draw_surface_mesh.cpp")
   create_single_source_cgal_program("sm_draw_small_faces.cpp")
   create_single_source_cgal_program("check_orientation.cpp")
   if(CGAL_Qt5_FOUND )
+
+    #link it with the required CGAL libraries
+
     target_link_libraries(draw_surface_mesh PUBLIC CGAL::CGAL_Qt5)
     target_link_libraries(sm_draw_small_faces PUBLIC CGAL::CGAL_Qt5)
   endif()
@@ -50,4 +57,4 @@ else()
     message(STATUS "This program requires the CGAL library, and will not be compiled.")
 
 endif()
-
+#end of the file
diff -pruN 5.0.3-1/examples/Surface_mesh/draw_surface_mesh_small_faces.h 5.1-1/examples/Surface_mesh/draw_surface_mesh_small_faces.h
--- 5.0.3-1/examples/Surface_mesh/draw_surface_mesh_small_faces.h	2020-08-01 19:02:54.000000000 +0000
+++ 5.1-1/examples/Surface_mesh/draw_surface_mesh_small_faces.h	2020-09-08 12:15:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh/examples/Surface_mesh/draw_surface_mesh_small_faces.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh/examples/Surface_mesh/draw_surface_mesh_small_faces.h $
 // $Id: draw_surface_mesh_small_faces.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/examples/Surface_mesh/sm_join.cpp 5.1-1/examples/Surface_mesh/sm_join.cpp
--- 5.0.3-1/examples/Surface_mesh/sm_join.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Surface_mesh/sm_join.cpp	2020-09-08 12:12:20.000000000 +0000
@@ -34,5 +34,5 @@ int main(int argc, char* argv[])
   }
 
 
-  std::cout << sm1 << std::endl;
+  std::cout << std::setprecision(17)<< sm1 << std::endl;
 }
diff -pruN 5.0.3-1/examples/Surface_mesh_approximation/CMakeLists.txt 5.1-1/examples/Surface_mesh_approximation/CMakeLists.txt
--- 5.0.3-1/examples/Surface_mesh_approximation/CMakeLists.txt	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_approximation/CMakeLists.txt	2020-09-08 12:12:20.000000000 +0000
@@ -28,11 +28,10 @@ endif()
 
 # Use Eigen (for PCA)
 find_package(Eigen3 3.1.0 REQUIRED) #(3.1.0 or greater)
-if (NOT EIGEN3_FOUND)
+include(CGAL_Eigen_support)
+if (NOT TARGET CGAL::Eigen_support)
   message(STATUS "This project requires the Eigen library, and will not be compiled.")
   return()
-else()
-  include( ${EIGEN3_USE_FILE} )
 endif()
 
 # Creating entries for all C++ files with "main" routine
@@ -41,14 +40,19 @@ endif()
 include( CGAL_CreateSingleSourceCGALProgram )
 
 create_single_source_cgal_program( "vsa_approximation_2_example.cpp" )
+target_link_libraries(vsa_approximation_2_example PUBLIC CGAL::Eigen_support)
 
 create_single_source_cgal_program( "vsa_approximation_example.cpp" )
+target_link_libraries(vsa_approximation_example PUBLIC CGAL::Eigen_support)
 
 create_single_source_cgal_program( "vsa_class_interface_example.cpp" )
+target_link_libraries(vsa_class_interface_example PUBLIC CGAL::Eigen_support)
 
 create_single_source_cgal_program( "vsa_isotropic_metric_example.cpp" )
+target_link_libraries(vsa_isotropic_metric_example PUBLIC CGAL::Eigen_support)
 
 create_single_source_cgal_program( "vsa_segmentation_example.cpp" )
+target_link_libraries(vsa_segmentation_example PUBLIC CGAL::Eigen_support)
 
 create_single_source_cgal_program( "vsa_simple_approximation_example.cpp" )
-
+target_link_libraries(vsa_simple_approximation_example PUBLIC CGAL::Eigen_support)
diff -pruN 5.0.3-1/examples/Surface_mesh_deformation/CMakeLists.txt 5.1-1/examples/Surface_mesh_deformation/CMakeLists.txt
--- 5.0.3-1/examples/Surface_mesh_deformation/CMakeLists.txt	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_deformation/CMakeLists.txt	2020-09-08 12:12:20.000000000 +0000
@@ -12,9 +12,8 @@ find_package(CGAL QUIET)
 if ( CGAL_FOUND )
 
   find_package(Eigen3 3.1.91) #(requires 3.2.0 or greater)
-  if (EIGEN3_FOUND)
-    include( ${EIGEN3_USE_FILE} )
-
+  include(CGAL_Eigen_support)
+  if (TARGET CGAL::Eigen_support)
     create_single_source_cgal_program( "all_roi_assign_example.cpp" )
     create_single_source_cgal_program( "all_roi_assign_example_custom_polyhedron.cpp" )
     create_single_source_cgal_program( "all_roi_assign_example_Surface_mesh.cpp" )
@@ -24,11 +23,23 @@ if ( CGAL_FOUND )
     create_single_source_cgal_program( "k_ring_roi_translate_rotate_Surface_mesh.cpp" )
     create_single_source_cgal_program( "deform_mesh_for_botsch08_format_sre_arap.cpp" )
 
+    foreach(target
+        all_roi_assign_example
+        all_roi_assign_example_custom_polyhedron
+        all_roi_assign_example_Surface_mesh
+        custom_weight_for_edges_example
+        deform_polyhedron_with_custom_pmap_example
+        k_ring_roi_translate_rotate_example
+        k_ring_roi_translate_rotate_Surface_mesh
+        deform_mesh_for_botsch08_format_sre_arap)
+      target_link_libraries(${target} PUBLIC CGAL::Eigen_support)
+    endforeach()
+
     find_package( OpenMesh QUIET )
     if ( OpenMesh_FOUND )
     include( UseOpenMesh )
       create_single_source_cgal_program( "all_roi_assign_example_with_OpenMesh.cpp" )
-      target_link_libraries( all_roi_assign_example_with_OpenMesh PRIVATE ${OPENMESH_LIBRARIES} )
+      target_link_libraries( all_roi_assign_example_with_OpenMesh PRIVATE ${OPENMESH_LIBRARIES} CGAL::Eigen_support)
     else()
       message(STATUS "Example that use OpenMesh will not be compiled.")
     endif()
@@ -41,4 +52,3 @@ else()
     message(STATUS "NOTICE: These exmaples require the CGAL library, and will not be compiled.")
 
 endif()
-
diff -pruN 5.0.3-1/examples/Surface_mesh_parameterization/CMakeLists.txt 5.1-1/examples/Surface_mesh_parameterization/CMakeLists.txt
--- 5.0.3-1/examples/Surface_mesh_parameterization/CMakeLists.txt	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_parameterization/CMakeLists.txt	2020-09-08 12:12:21.000000000 +0000
@@ -12,9 +12,9 @@ find_package(CGAL QUIET)
 if ( CGAL_FOUND )
 
   find_package(Eigen3 3.1.0) #(requires 3.1.0 or greater)
-  if (EIGEN3_FOUND)
+  include(CGAL_Eigen_support)
+  if (TARGET CGAL::Eigen_support)
     # Executables that require Eigen 3.1
-    include( ${EIGEN3_USE_FILE} )
 
 # ------------------------------------------------------------------
 # Detect SuiteSparse libraries:
@@ -52,23 +52,28 @@ if ( CGAL_FOUND )
 # ------------------------------------------------------------------
 
     create_single_source_cgal_program( "discrete_authalic.cpp" )
+    target_link_libraries(discrete_authalic PUBLIC CGAL::Eigen_support)
     create_single_source_cgal_program( "lscm.cpp" )
+    target_link_libraries(lscm PUBLIC CGAL::Eigen_support)
     create_single_source_cgal_program( "orbifold.cpp" )
+    target_link_libraries(orbifold PUBLIC CGAL::Eigen_support)
     create_single_source_cgal_program( "seam_Polyhedron_3.cpp" )
+    target_link_libraries(seam_Polyhedron_3 PUBLIC CGAL::Eigen_support)
     create_single_source_cgal_program( "simple_parameterization.cpp" )
+    target_link_libraries(simple_parameterization PUBLIC CGAL::Eigen_support)
     create_single_source_cgal_program( "square_border_parameterizer.cpp" )
+    target_link_libraries(square_border_parameterizer PUBLIC CGAL::Eigen_support)
 
     if(SuiteSparse_FOUND)
       target_link_libraries(orbifold PRIVATE ${SuiteSparse_LIBRARIES})
     endif()
 
-  else(EIGEN3_FOUND)
+  else()
     message(STATUS "NOTICE: The examples require Eigen 3.1 (or greater) and will not be compiled.")
-  endif(EIGEN3_FOUND)
+  endif()
 
 else()
 
   message(STATUS "NOTICE: This program requires the CGAL library, and will not be compiled.")
 
 endif()
-
diff -pruN 5.0.3-1/examples/Surface_mesh_shortest_path/CMakeLists.txt 5.1-1/examples/Surface_mesh_shortest_path/CMakeLists.txt
--- 5.0.3-1/examples/Surface_mesh_shortest_path/CMakeLists.txt	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_shortest_path/CMakeLists.txt	2020-09-08 12:12:21.000000000 +0000
@@ -17,6 +17,7 @@ if ( CGAL_FOUND )
   create_single_source_cgal_program( "shortest_paths_no_id.cpp" )
   create_single_source_cgal_program( "shortest_paths_with_id.cpp" )
   create_single_source_cgal_program( "shortest_paths.cpp" )
+  create_single_source_cgal_program( "shortest_path_with_locate.cpp" )
 
   find_package( OpenMesh QUIET )
   if ( OpenMesh_FOUND )
diff -pruN 5.0.3-1/examples/Surface_mesh_shortest_path/shortest_path_with_locate.cpp 5.1-1/examples/Surface_mesh_shortest_path/shortest_path_with_locate.cpp
--- 5.0.3-1/examples/Surface_mesh_shortest_path/shortest_path_with_locate.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_shortest_path/shortest_path_with_locate.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,78 @@
+#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
+
+#include <CGAL/Surface_mesh_shortest_path.h>
+
+#include <CGAL/AABB_face_graph_triangle_primitive.h>
+#include <CGAL/AABB_traits.h>
+#include <CGAL/AABB_tree.h>
+#include <CGAL/Surface_mesh.h>
+
+#include <boost/lexical_cast.hpp>
+
+#include <cstdlib>
+#include <iostream>
+#include <fstream>
+
+typedef CGAL::Exact_predicates_inexact_constructions_kernel             Kernel;
+typedef Kernel::Point_3                                                 Point_3;
+typedef Kernel::Ray_3                                                   Ray_3;
+
+typedef CGAL::Surface_mesh<Point_3>                                     Triangle_mesh;
+typedef CGAL::Surface_mesh_shortest_path_traits<Kernel, Triangle_mesh>  Traits;
+typedef CGAL::Surface_mesh_shortest_path<Traits>                        Surface_mesh_shortest_path;
+
+typedef boost::graph_traits<Triangle_mesh>                              Graph_traits;
+typedef Graph_traits::vertex_iterator                                   vertex_iterator;
+typedef Graph_traits::face_iterator                                     face_iterator;
+
+typedef typename Surface_mesh_shortest_path::Barycentric_coordinates    Barycentric_coordinates;
+typedef typename Surface_mesh_shortest_path::Face_location              Face_location;
+
+typedef CGAL::AABB_face_graph_triangle_primitive<Triangle_mesh>         AABB_face_graph_primitive;
+typedef CGAL::AABB_traits<Kernel, AABB_face_graph_primitive>            AABB_face_graph_traits;
+typedef CGAL::AABB_tree<AABB_face_graph_traits>                         AABB_tree;
+
+int main(int argc, char** argv)
+{
+  Triangle_mesh tmesh;
+  std::ifstream input((argc>1) ? argv[1] : "data/elephant.off");
+  input >> tmesh;
+
+  Surface_mesh_shortest_path shortest_paths(tmesh);
+
+  // The source point is a 3D point. We must convert it to a face location (that is,
+  // a face ID and three barycentric coordinates)
+  const Point_3 source_pt(-0.06, -0.145, 0.182);
+  Face_location source_loc = shortest_paths.locate<AABB_face_graph_traits>(source_pt); // this builds an AABB tree of the mesh
+
+  // Add the source point
+  shortest_paths.add_source_point(source_loc.first, source_loc.second);
+
+  // The target will be defined by the first intersection of the following ray with the input mesh
+  const Ray_3 ray(Point_3(0.126, 0.387, 0.324), Point_3(0.126, 0.387, 0.124));
+
+  // If you intend to call `locate` many times, you should cache the AABB tree
+  // that is used internally to avoid it being recomputed at each call, as follows:
+  AABB_tree tree;
+  shortest_paths.build_aabb_tree(tree);
+  Face_location target_loc = shortest_paths.locate<AABB_face_graph_traits>(ray, tree);
+
+  if(target_loc.first == boost::graph_traits<Triangle_mesh>::null_face())
+  {
+    std::cerr << "They ray does not intersect the mesh!" << std::endl;
+    return EXIT_FAILURE;
+  }
+
+  // Compute the shortest path between the source and the target
+  std::vector<Point_3> points;
+  shortest_paths.shortest_path_points_to_source_points(target_loc.first, target_loc.second,
+                                                       std::back_inserter(points));
+
+  // Print the points
+  std::cout << points.size() << " ";
+  for (std::size_t i = 0; i < points.size(); ++i)
+    std::cout << " " << points[i];
+  std::cout << std::endl;
+
+  return EXIT_SUCCESS;
+}
diff -pruN 5.0.3-1/examples/Surface_mesh_simplification/CMakeLists.txt 5.1-1/examples/Surface_mesh_simplification/CMakeLists.txt
--- 5.0.3-1/examples/Surface_mesh_simplification/CMakeLists.txt	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_simplification/CMakeLists.txt	2020-09-08 12:12:21.000000000 +0000
@@ -4,32 +4,22 @@
 cmake_minimum_required(VERSION 3.1...3.15)
 project( Surface_mesh_simplification_Examples )
 
-
-
-
-
 # CGAL and its components
 find_package( CGAL QUIET COMPONENTS  )
-
 if ( NOT CGAL_FOUND )
-
   message(STATUS "This project requires the CGAL library, and will not be compiled.")
   return()
-
 endif()
 
+include( ${CGAL_USE_FILE} )
+
 # Boost and its components
 find_package( Boost )
-
 if ( NOT Boost_FOUND )
-
   message(STATUS "This project requires the Boost library, and will not be compiled.")
-
   return()
-
 endif()
 
-
 find_package( OpenMesh QUIET )
 
 if ( OpenMesh_FOUND )
@@ -38,10 +28,6 @@ else()
   message(STATUS "Examples that use OpenMesh will not be compiled.")
 endif()
 
-# include for local directory
-
-# include for local package
-
 # Creating entries for all .cpp/.C files with "main" routine
 # ##########################################################
 
@@ -56,8 +42,16 @@ create_single_source_cgal_program( "edge
 create_single_source_cgal_program( "edge_collapse_bounded_normal_change.cpp" )
 create_single_source_cgal_program( "edge_collapse_visitor_surface_mesh.cpp" )
 
+find_package(Eigen3 3.1.0 QUIET) #(3.1.0 or greater)
+include(CGAL_Eigen_support)
+if( TARGET CGAL::Eigen_support )
+  create_single_source_cgal_program( "edge_collapse_garland_heckbert.cpp" )
+  target_link_libraries(edge_collapse_garland_heckbert PUBLIC CGAL::Eigen_support)
+else ()
+  message(STATUS "Garland-Heckbert polices require the Eigen library, which has not been found; related examples will not be compiled.")
+endif()
+
 if(OpenMesh_FOUND)
   create_single_source_cgal_program( "edge_collapse_OpenMesh.cpp" )
-
   target_link_libraries( edge_collapse_OpenMesh PRIVATE ${OPENMESH_LIBRARIES} )
 endif()
diff -pruN 5.0.3-1/examples/Surface_mesh_simplification/cube-meshed.off 5.1-1/examples/Surface_mesh_simplification/cube-meshed.off
--- 5.0.3-1/examples/Surface_mesh_simplification/cube-meshed.off	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_simplification/cube-meshed.off	1970-01-01 00:00:00.000000000 +0000
@@ -1,2598 +0,0 @@
-OFF
-866 1728 0
-
--0.875 1 -0.875
--0.75 1 -0.75
--0.875 1 -0.625
--1 0.75 0.5
--1 0.75 0.25
--1 0.875 0.375
--0.875 -0.375 1
--0.875 -0.125 1
--0.75 -0.25 1
-0.25 -1 -0
-0.375 -1 -0.125
-0.25 -1 -0.25
-1 -0.25 1
-0.875 -0.375 1
-0.875 -0.125 1
-0.25 1 0.75
-0.25 1 1
-0.125 1 0.875
--0 1 1
--0 0.75 1
--0.125 0.875 1
--1 0.875 0.625
--1 1 0.5
-0.375 1 -0.875
-0.25 1 -0.75
-0.25 1 -1
-0.875 -1 0.875
-0.875 -1 1
-1 -1 0.875
--0.625 -0.125 1
--0.5 -0 1
--0.5 -0.25 1
--1 -1 -0.375
--1 -0.875 -0.375
--1 -1 -0.25
--0.875 -1 -0.375
-0.75 -0.5 -1
-0.75 -0.25 -1
-0.625 -0.375 -1
-1 -0.25 0.75
-1 -0.375 0.875
-1 0.375 -0.875
-1 0.5 -0.75
-1 0.5 -1
-0.875 -1 -0.625
-0.875 -1 -0.875
-0.75 -1 -0.75
-1 -0.25 -0.25
-1 -0.125 -0.375
-1 -0.25 -0.5
--0.5 -1 1
--0.375 -1 0.875
--0.5 -1 0.75
-1 -0 -0.75
-1 -0.125 -0.625
-1 -0 -0.5
-0.75 -0.75 -1
-0.875 -0.625 -1
-1 -1 -0
-1 -0.875 0.125
-1 -0.875 -0.125
--0.75 0.5 1
--0.875 0.625 1
--0.75 0.75 1
-1 0.5 -0
-1 0.375 0.125
-1 0.5 0.25
--1 -0.375 0.125
--1 -0.25 -0
--1 -0.25 0.25
-1 -0.75 -1
-1 -0.75 -0.75
-1 -0.625 -0.875
--0.375 0.625 -1
--0.25 0.5 -1
--0.25 0.75 -1
--0.25 -1 1
--0.25 -1 0.75
-0.125 1 -1
-0.125 1 -0.875
--0 1 -1
-0.125 0.875 -1
-1 -0.625 0.125
-1 -0.75 0.25
-1 -0.625 0.375
--1 -0.875 0.125
--1 -1 -0
--1 -0.875 -0.125
--1 -0.375 -0.625
--1 -0.25 -0.75
--1 -0.25 -0.5
--0.875 -0.625 1
--0.75 -0.75 1
--0.875 -0.875 1
--0.625 -0.375 1
-0.75 -0 -1
-0.625 0.125 -1
-0.625 -0.125 -1
--0.375 0.625 1
--0.25 0.5 1
--0.375 0.375 1
-0.625 1 -0.375
-0.625 1 -0.125
-0.5 1 -0.25
--1 -0.625 -0.875
--1 -0.5 -0.75
--1 -0.625 -0.625
--0.375 -1 -0.625
--0.5 -1 -0.75
--0.5 -1 -0.5
--0.125 0.875 -1
--0.25 1 -1
-1 0.125 0.625
-1 -0 0.5
-1 -0 0.75
--0 -1 -0.5
--0.125 -1 -0.625
--0.125 -1 -0.375
-0.75 1 -0.25
-0.75 1 -0
-1 -0.5 -0
-1 -0.5 0.25
--1 0.25 -0.5
--1 0.375 -0.375
--1 0.25 -0.25
--0.375 -1 -0.875
--1 0.125 -0.625
--1 0.125 -0.875
--1 0.25 -0.75
--0 -1 0.75
--0 -1 0.5
--0.125 -1 0.625
--1 -0.75 1
--1 -0.75 0.75
--1 -0.625 0.875
-0.5 -1 0.5
-0.5 -1 0.75
-0.625 -1 0.625
-1 0.75 -0.25
-1 0.625 -0.375
-1 0.625 -0.125
-1 0.625 -0.875
--0.5 -1 -0.25
--0.625 -1 -0.375
--1 1 0.625
--0.875 1 0.625
--0.25 -1 -0.75
--0.25 -1 -0.5
-1 -0.75 -0
-1 -0.75 -0.25
--0.75 1 -1
--0.75 0.75 -1
--0.625 0.875 -1
--0.625 1 -1
--0.5 1 -1
--0.625 1 -0.875
-0.125 -1 0.875
-0.125 -1 0.625
--0.5 1 -0.75
--0.5 1 -0.5
--0.625 1 -0.625
--0.625 0.125 -1
--0.625 0.375 -1
--0.75 0.25 -1
-1 0.625 0.875
-1 0.75 0.75
-1 0.625 0.625
-0.5 -0.75 -1
-0.375 -0.875 -1
-0.5 -1 -1
--0.75 -1 -0.5
--0.875 -1 -0.625
--0.375 -0.375 1
--0.5 -0.5 1
--0.75 -1 -0.75
--0.625 -1 -0.625
-1 -0.25 0.5
-1 -0.375 0.625
--1 -0.375 0.875
--1 -0.25 0.75
--1 -0.25 1
-1 0.25 -0
-1 0.25 0.25
--0.125 0.625 1
--0.25 0.75 1
--0.25 -0.75 1
--0.25 -0.5 1
--0.125 -0.625 1
-1 0.875 -0.375
-1 0.875 -0.125
-0.5 -0.25 -1
--0.625 0.125 1
--0.875 1 -0.375
--0.875 1 -0.125
--1 1 -0.25
--0.375 -1 -0.375
--0.375 -1 -0.125
--0.625 0.375 1
--0.5 0.25 1
-1 0.75 -0
-1 0.875 0.125
-1 0.375 1
-0.875 0.375 1
-1 0.5 1
-1 0.375 0.875
--0.125 -0.375 1
--0.125 -0.125 1
--0 -0.25 1
--0 1 -0.75
--0.125 1 -0.875
--1 0.875 0.125
--1 0.75 -0
--1 0.875 -0.125
--0 1 0.5
--0.125 1 0.625
--0.125 1 0.375
-1 -0.125 0.125
-1 -0.25 0.25
-1 -0 0.25
--1 -0.25 -0.25
--1 -0.375 -0.125
-0.125 -0.875 1
-0.25 -0.75 1
-0.25 -1 1
--0.125 -1 0.875
--0 -1 1
--0.375 -0.625 1
--1 0.375 -1
--1 0.5 -1
--1 0.375 -0.875
--0.875 0.375 -1
-1 -1 -0.5
-1 -0.875 -0.625
-1 -1 -0.625
-1 0.75 0.25
-1 0.875 -0.625
-1 0.75 -0.5
--0.375 1 1
--0.375 0.875 1
--0.5 1 1
--0.375 1 0.875
--0.5 0.75 1
--0.625 0.625 1
--0.625 0.875 1
-1 0.75 -0.75
-1 0.875 -0.875
-1 0.75 -1
-0.375 -0.875 1
-0.375 -0.625 1
-0.5 -0.75 1
-0.5 -1 1
-0.625 -1 0.875
--1 -0.125 -0.625
--1 -0.125 -0.875
--1 -0 -0.75
-1 0.625 0.375
-1 0.75 0.5
--0.375 0.875 -1
-1 0.75 1
-0.875 1 -0.375
-1 1 -0.25
-0.875 1 -0.125
-0.75 1 -0.75
-0.875 1 -0.875
-0.875 1 -0.625
-0.75 0.75 -1
-0.75 1 -1
-0.625 0.875 -1
--1 0.5 1
--1 0.5 0.75
--1 0.625 0.875
-1 -0.75 -0.5
-1 0.375 -0.625
-1 0.375 -0.375
-1 0.5 -0.5
--0.75 1 -0
--0.75 1 -0.25
--0.625 1 -0.125
--0.25 -0 -1
--0.25 0.25 -1
--0.375 0.125 -1
--1 0.25 0.75
--1 0.125 0.625
--1 0.25 0.5
--1 0.625 1
--1 0.75 1
-1 -0.125 -1
-1 -0 -1
-0.875 -0.125 -1
-1 -0.125 -0.875
-1 0.25 -0.75
-1 0.25 -0.5
-0.375 -0.625 -1
-0.25 -0.75 -1
--0 0.5 1
--0.125 0.375 1
-0.625 -0.375 1
-0.75 -0.25 1
-0.75 -0.5 1
--0.875 0.375 1
--0.625 -0.875 -1
--0.75 -0.75 -1
--0.75 -1 -1
--0.75 -0 1
--0.75 0.25 1
-0.5 1 -0.75
-0.5 1 -1
--0.75 -1 1
--0.625 -1 0.875
--0.75 -1 0.75
-0.5 -1 0.25
-0.375 -1 0.125
-0.375 -1 0.375
--0.25 1 -0.25
--0.25 1 -0
--0.375 1 -0.125
--1 1 0.375
--0.875 1 0.375
--1 0.5 -0.5
--1 0.375 -0.625
--1 0.5 -0.75
--0.625 -0.875 1
--0.5 -0.75 1
-1 -0.125 1
-1 -0.125 0.875
--1 -0.625 0.625
--1 -0.625 0.375
--1 -0.5 0.5
--1 0.5 0.5
--1 0.625 0.625
--0 -1 -0.75
--0 -1 -1
--0.125 -1 -0.875
--1 1 -0.625
--1 1 -0.5
--1 0.875 -0.625
-0.625 0.875 1
-0.625 0.625 1
-0.5 0.75 1
-0.375 0.625 -1
-0.5 0.75 -1
-0.375 0.875 -1
-0.875 0.875 -1
-0.875 0.625 1
-0.75 0.75 1
-0.875 0.875 1
--0 -0.75 1
--0 -0.5 1
-0.125 -0.625 1
--0.25 -0.25 1
--0.375 -0.125 1
--0.375 0.375 -1
-1 0.625 -0.625
--1 -1 0.875
--0.875 -1 0.875
--1 -1 0.75
--1 -0.875 0.875
-1 -0.625 -0.375
-1 -0.5 -0.25
-1 -0.5 -0.5
-0.875 0.125 -1
--0.75 -1 -0.25
--0.875 -1 1
--1 -0.5 -0.25
--1 -0.625 -0.375
--1 -0.5 -0.5
-0.875 -0.375 -1
-0.375 -1 0.875
-0.375 -1 0.625
--0.125 0.375 -1
--0.125 0.125 -1
-0.25 -1 -1
-0.25 0.25 -1
-0.375 0.125 -1
-0.375 0.375 -1
-0.875 0.125 1
-0.75 -0 1
-0.75 0.25 1
--1 1 -0.375
--1 0.875 -0.375
-0.875 -0.625 1
-1 -0.5 1
-0.75 1 0.5
-0.875 1 0.375
-0.875 1 0.625
--0.25 1 0.75
--0.25 1 1
--1 0.125 -0.125
--1 0.25 -0
--1 -0 -0
-1 -0.125 0.375
-0.375 1 -0.375
-0.375 1 -0.125
-0.25 1 -0.25
--1 0.75 -0.5
--0.25 -1 -0.25
--0.25 -1 -0
--0.125 -1 -0.125
--1 -0.75 0.5
--0 -1 0.25
-0.125 -1 0.375
-0.125 -1 0.125
-0.75 -1 0.25
-0.875 -1 0.125
-0.75 -1 -0
--0.375 0.125 1
-0.625 -0.875 1
--0.5 0.5 1
--1 -0.875 1
-1 -0.625 1
-1 -0.75 1
-1 -0.625 0.875
-1 -0.125 0.625
-0.25 0.75 -1
-0.25 0.5 -1
--0.625 1 0.125
--0.625 1 0.375
--0.75 1 0.25
--0.25 -1 0.5
--0.25 -1 0.25
--0.375 -1 0.375
-1 0.875 -1
-0.5 -1 -0.5
-0.5 -1 -0.75
-0.375 -1 -0.625
--1 -0.625 -0.125
-0.75 0.25 -1
--0.5 -0.75 -1
--0.625 -0.625 -1
-0.625 1 0.875
-0.5 1 0.75
-0.625 1 0.625
-1 -0.875 -0.375
-0.5 1 1
-0.375 1 0.875
-0.375 -0.375 -1
-0.375 -0.125 -1
-0.375 0.875 1
-0.125 1 0.375
-0.25 1 0.25
-0.25 1 0.5
--0.625 -0.625 1
-1 -0.5 -0.75
-1 -0.5 -1
--1 -1 1
-1 -1 -0.875
-1 -1 -0.75
-1 -0.875 -0.875
-0.125 -0.125 -1
-0.25 -0.25 -1
-0.25 -0 -1
-1 0.25 1
-0.5 1 0.5
-0.375 1 0.625
--0.75 1 -0.5
-1 0.375 -1
-0.875 0.375 -1
--1 -1 -0.125
--0.875 -1 -0.125
--0 -0.75 -1
--0.125 -0.875 -1
--0.125 -0.375 -1
--0.25 -0.5 -1
--0.125 -0.625 -1
--1 1 -0
-1 -0.5 0.75
-1 0.5 -0.25
-1 0.125 0.875
--1 0.25 0.25
--1 0.125 0.375
--1 0.125 0.125
-0.75 -1 -0.25
-0.875 -1 -0.375
-0.75 -1 -0.5
-0.75 -1 0.5
-0.875 -1 0.375
-0.375 -1 -0.875
-1 0.625 0.125
--0.5 -1 -1
--0.625 -1 -0.875
-0.625 1 0.375
-0.75 1 0.25
-1 -0.625 -0.625
--0.375 1 -0.375
--0.25 1 -0.5
-0.25 -1 0.5
-1 0.25 -0.25
-1 0.125 -0.125
-0.125 -0.625 -1
-0.125 -0.875 -1
--1 -0.125 0.125
--1 -0.125 0.375
-0.25 1 -0
-1 1 -0.75
--1 0.375 0.875
--1 0.75 -0.25
-1 0.875 0.375
-1 -0.375 0.125
-1 -0.25 -0
--0.875 -0.375 -1
--1 -0.5 -1
--0.875 -0.625 -1
--0.125 -0.875 1
--0.625 -0.375 -1
--0.5 -0.25 -1
--0.625 -0.125 -1
--0.75 1 0.75
--0.75 1 1
--0.875 1 0.875
--0.75 -1 0.25
--0.75 -1 0.5
--0.625 -1 0.375
--0.5 -1 -0
-1 -0.375 -0.125
-1 -0.375 -0.375
--0.875 -1 0.375
--0.875 -1 0.125
--1 -0.75 -0.75
--1 -0.75 -0.5
--1 -0.875 -0.625
-0.5 -0.5 1
--0.5 0.75 -1
--0.75 -0.25 -1
--1 -0.5 0.75
-1 1 -0.5
--0 0.25 1
--0 -0 1
--0.125 0.125 1
-1 -0 1
--1 0.625 -0.125
-0.75 -1 1
-0.75 -1 0.75
-0.5 -0.25 1
-0.625 -0.125 1
-0.75 1 0.75
-0.75 1 1
--1 -0.875 -0.875
--1 -1 -0.75
--0 1 0.25
--1 -1 0.25
--1 -0.875 0.375
-0.25 -0.5 1
-1 -1 1
-1 -0.875 0.875
-0.75 0.5 1
-0.625 0.375 1
--1 0.5 -0.25
--1 0.5 -0
--0.625 1 -0.375
--0.5 1 -0.25
--1 0.125 -0.375
-0.625 1 -0.875
-1 -0.75 0.75
--0.75 -0.5 -1
-0.5 -1 -0.25
-0.625 -1 -0.375
--1 0.375 1
--1 0.25 1
-0.625 0.375 -1
-0.625 0.625 -1
-0.5 0.5 -1
--1 -0.375 -0.375
--0.875 0.875 1
-0.5 -0 -1
-0.875 1 0.125
-1 1 0.5
-1 0.875 0.625
-1 1 0.625
--1 0.75 0.75
--1 -0.75 -0.25
--0.125 1 0.875
--0.25 -0 1
--1 -0.125 0.875
--1 -0 0.75
--1 -0 1
--0.875 -1 0.625
--0.375 1 0.125
--0.25 1 0.25
--0.375 1 0.375
-0.25 -1 0.25
--0.125 1 0.125
--0.125 1 1
-0.125 1 -0.625
--0.375 -1 0.125
-1 0.5 0.5
-1 0.375 0.375
-0.25 1 -0.5
--0.5 0.25 -1
--0.75 -0.5 1
-1 -0.75 0.5
-0.625 -1 0.125
-0.5 -1 -0
-0.375 -0.375 1
--1 0.25 -1
--1 -1 0.125
-1 -0.25 -0.75
-0.875 0.625 -1
--0.875 0.125 -1
--1 -0 -1
--0.875 -0.125 -1
--1 -0.5 -0
--0.375 1 -1
--0.375 1 -0.875
-0.625 -1 0.375
--0 1 0.75
--0.375 1 0.625
--0.25 -0.75 -1
--0.375 -0.625 -1
-0.625 1 -1
--1 -0 0.25
-0.125 1 -0.125
-0.375 1 -0.625
--0.625 -1 0.625
--0.5 -1 0.5
--1 -0.375 1
--0.875 -1 -0.875
--1 0.375 -0.125
-0.625 -0.625 1
-1 0.125 -0.875
-1 0.125 -0.625
-1 0.125 -1
--1 -0.5 0.25
--0.25 -0.25 -1
--0.125 -0.125 -1
--1 0.75 -0.75
--1 0.75 -1
--1 0.875 -0.875
-0.875 -1 0.625
--1 -0.375 -0.875
-1 -0 -0
-1 0.125 0.125
-0.625 -1 -0.875
-0.625 -1 -0.625
--0.5 -0.5 -1
-0.125 1 0.125
--0.375 -0.375 -1
--1 -1 -0.625
--1 -1 -0.5
--1 0.625 0.375
--1 -0.25 -1
-0.75 -1 -1
--1 -0.125 0.625
--1 -0 0.5
-1 -1 -0.125
-1 -1 -0.25
-0.875 -1 -0.125
--1 -0.125 -0.375
--1 -0 -0.5
--1 0.875 0.875
--0.5 1 0.5
--0.5 1 0.75
-1 0.125 -0.375
-1 -1 0.25
-1 -0.875 0.375
--0.25 0.25 1
-1 1 -0.625
--1 -0.25 0.5
--1 -0.375 0.375
--0.125 -1 -1
--0.25 1 -0.75
-1 -0.375 0.375
-0.125 1 0.625
-1 0.375 0.625
-1 0.5 0.75
-1 1 -0.125
-0.25 -0 1
-0.375 -0.125 1
-0.25 -0.25 1
-1 -1 -0.375
--1 -0 -0.25
--0.125 -1 1
--1 0.125 0.875
-1 -0.5 0.5
-1 -0.625 0.625
-0.125 0.625 -1
--0.625 1 0.625
--0.625 1 0.875
-0.25 -1 0.75
-0.125 1 -0.375
--0 1 -0.25
--0 1 -0.5
-0.5 1 -0.5
--1 -0.125 -0.125
--0.75 -1 -0
-0.125 -0.375 1
--0.125 1 -0.625
--0.875 1 0.125
--0.75 1 0.5
--0.75 0.5 -1
--0.875 0.625 -1
-0.625 0.125 1
--1 0.375 0.125
--1 0.375 0.375
--0.25 -1 -1
--0 -0.5 -1
--0.875 0.875 -1
-1 0.125 0.375
--0.375 -1 -1
--0.375 -0.875 -1
--0.375 -0.125 -1
--0.5 -0 -1
-0.5 -0 1
-0.125 0.875 1
-0.125 0.625 1
-1 0.875 0.875
-1 1 0.75
--0.625 -1 -0.125
-0.5 0.25 -1
-0.375 1 0.375
-0.5 1 0.25
-0.875 1 0.875
-0.125 -1 -0.125
-0.125 -1 -0.375
-0.625 -1 -0.125
-0.25 -0.5 -1
-1 -1 0.75
-0.25 -1 -0.5
-0.375 -1 -0.375
--0.375 -0.875 1
-0.375 0.625 1
-0.25 0.75 1
--0.25 1 0.5
--0 -0.25 -1
-0.5 0.25 1
--0 0.75 -1
--1 -0.125 -1
-0.375 0.375 1
-0.5 0.5 1
--0.125 1 -1
-0.125 -1 -0.875
-1 -0.375 -0.875
-1 -0.875 -1
-0.875 -0.875 -1
--1 -0.5 1
-0.125 -0.375 -1
--0.625 -1 0.125
--0 -1 -0.25
-1 0.875 1
-0.125 0.375 1
--0 1 -0
--0.5 1 0.25
-0.125 0.125 -1
-0.125 0.375 -1
--0 0.25 -1
-1 0.25 -1
-1 -0.375 -0.625
-0.75 1 -0.5
-0.125 0.125 1
--1 -0.625 1
-0.375 1 -1
--0 0.5 -1
-1 -0.875 0.625
--1 0.625 -0.375
-1 1 0.25
-1 1 0.375
--0 -1 -0
-0.125 -1 -0.625
-0.875 -0.875 1
--1 -0.375 0.625
-0.625 1 0.125
-0.625 1 -0.625
--1 0.125 -1
--1 1 0.125
--1 1 0.25
--1 -0.75 -1
--1 -0.875 -1
--0.875 -0.875 -1
--0.875 0.125 1
-1 1 0.125
-0.25 -1 -0.75
-1 -0.25 -1
-0.375 0.125 1
-0.25 0.25 1
-0.5 -0.5 -1
--1 0.625 -0.625
--1 -1 0.625
--1 -0.875 0.625
--0.5 -1 0.25
-0.375 1 1
-0.75 0.5 -1
--1 0.625 0.125
-1 -0.625 -0.125
--1 1 0.75
--1 1 0.875
-0.125 -0.125 1
-0.375 -1 -1
-1 0.375 -0.125
--1 0.875 -1
--1 1 -1
--1 0.5 0.25
--0.5 1 -0
-1 -0.125 -0.125
--0.375 -1 0.625
--0.125 1 -0.375
--0.125 1 -0.125
--1 1 -0.125
--0.125 -1 0.125
-1 -0 -0.25
-1 1 0.875
--1 1 -0.75
--1 1 -0.875
-0.25 0.5 1
-1 0.25 0.5
-0.75 -0.75 1
-1 1 -0
--0.75 -0 -1
-1 1 -1
-1 1 -0.875
-1 -0.875 1
-0.875 -1 -1
-1 -1 -1
-0.875 1 -1
--0.125 -1 0.375
-0.625 -0.875 -1
-1 0.25 0.75
--1 0.625 -0.875
--1 -0.375 -1
--1 -0.75 -0
-0.625 -1 1
-0.375 -1 1
-0.625 -0.625 -1
-0.875 1 1
--0.875 1 -1
-1 -0.375 1
-1 0.125 1
--0 -0 -1
--1 0.625 -1
--1 0.375 0.625
-1 -0.375 -1
--1 -0.125 1
--0.375 -1 1
-0.625 -1 -1
--1 -0.75 0.25
--1 -0.625 0.125
-0.5 1 -0
-0.375 1 0.125
-1 -0.625 -1
--0.625 -1 1
--0.5 0.5 -1
--0.625 0.625 -1
-0.125 1 1
-1 0.625 -1
-1 -1 0.375
-1 -1 0.5
-0.125 -1 1
-0.125 -1 -1
-0.625 1 1
--0.375 1 -0.625
--0.875 1 1
--1 1 1
--0.125 0.625 -1
--1 0.125 1
--1 -1 0.5
-1 0.625 1
--1 -1 -0.875
--1 -1 -1
--0.875 -1 -1
--1 -0.625 -1
--0.625 -1 -1
-1 1 -0.375
--1 -1 0.375
--1 0.875 1
-1 -1 0.625
-1 1 1
--0.625 1 1
-1 -1 0.125
-3  0 2 1
-3  3 5 4
-3  6 8 7
-3  9 11 10
-3  12 14 13
-3  15 17 16
-3  18 20 19
-3  21 22 5
-3  23 25 24
-3  26 28 27
-3  29 31 30
-3  32 34 33
-3  35 34 32
-3  36 38 37
-3  39 12 40
-3  41 43 42
-3  44 46 45
-3  47 49 48
-3  50 52 51
-3  53 55 54
-3  56 36 57
-3  58 60 59
-3  61 63 62
-3  64 66 65
-3  67 69 68
-3  70 72 71
-3  73 75 74
-3  51 77 76
-3  78 80 79
-3  81 80 78
-3  25 79 24
-3  82 84 83
-3  85 87 86
-3  88 90 89
-3  91 93 92
-3  8 94 29
-3  95 97 96
-3  98 100 99
-3  101 103 102
-3  104 106 105
-3  107 109 108
-3  110 75 111
-3  112 114 113
-3  115 117 116
-3  118 102 119
-3  120 121 82
-3  122 124 123
-3  125 107 108
-3  126 128 127
-3  129 131 130
-3  132 134 133
-3  135 137 136
-3  138 140 139
-3  42 43 141
-3  142 143 109
-3  22 21 144
-3  22 144 145
-3  116 147 146
-3  60 149 148
-3  150 152 151
-3  153 155 154
-3  153 154 152
-3  156 129 157
-3  158 160 159
-3  161 163 162
-3  164 166 165
-3  167 169 168
-3  170 35 171
-3  172 31 173
-3  174 175 170
-3  176 39 177
-3  178 180 179
-3  181 65 182
-3  20 184 183
-3  185 187 186
-3  188 189 138
-3  38 190 97
-3  29 30 191
-3  192 194 193
-3  195 196 142
-3  197 191 198
-3  189 200 199
-3  201 203 202
-3  201 204 203
-3  180 6 7
-3  205 207 206
-3  80 209 208
-3  210 212 211
-3  213 215 214
-3  216 218 217
-3  219 220 68
-3  221 223 222
-3  224 129 225
-3  185 186 226
-3  227 229 228
-3  230 227 228
-3  231 233 232
-3  231 44 233
-3  199 200 234
-3  235 188 236
-3  237 239 238
-3  240 239 237
-3  241 243 242
-3  195 142 109
-3  191 30 198
-3  244 246 245
-3  247 249 248
-3  250 136 251
-3  252 254 253
-3  255 256 166
-3  257 75 73
-3  165 258 164
-3  67 68 220
-3  259 261 260
-3  262 264 263
-3  265 267 266
-3  268 270 269
-3  271 232 71
-3  272 274 273
-3  275 277 276
-3  278 280 279
-3  281 283 282
-3  284 62 285
-3  284 285 270
-3  83 148 82
-3  286 288 287
-3  289 286 287
-3  272 291 290
-3  292 168 293
-3  294 183 295
-3  296 298 297
-3  268 299 62
-3  300 302 301
-3  303 191 304
-3  305 306 23
-3  304 191 197
-3  307 309 308
-3  310 312 311
-3  313 315 314
-3  316 22 317
-3  5 22 316
-3  318 320 319
-3  321 50 322
-3  323 14 12
-3  324 323 12
-3  325 327 326
-3  328 269 329
-3  330 332 331
-3  333 334 2
-3  333 335 334
-3  336 338 337
-3  339 341 340
-3  263 266 262
-3  342 265 266
-3  235 236 244
-3  343 345 344
-3  346 348 347
-3  349 350 172
-3  74 351 73
-3  236 352 244
-3  353 355 354
-3  353 356 355
-3  357 359 358
-3  288 95 360
-3  361 35 170
-3  362 307 93
-3  362 354 307
-3  363 365 364
-3  79 80 208
-3  36 37 366
-3  367 368 136
-3  369 370 279
-3  371 293 168
-3  372 374 373
-3  375 377 376
-3  378 194 192
-3  378 379 194
-3  249 247 250
-3  380 381 13
-3  382 384 383
-3  385 240 386
-3  387 389 388
-3  218 390 217
-3  391 393 392
-3  335 394 379
-3  395 397 396
-3  325 326 398
-3  399 401 400
-3  402 404 403
-3  405 100 198
-3  250 406 249
-3  198 407 197
-3  408 132 356
-3  408 93 132
-3  409 380 410
-3  411 409 410
-3  412 114 324
-3  386 238 184
-3  339 414 413
-3  415 417 416
-3  418 420 419
-3  223 247 222
-3  421 246 342
-3  245 246 421
-3  20 386 184
-3  157 130 400
-3  422 424 423
-3  425 363 364
-3  95 96 426
-3  427 300 428
-3  224 77 131
-3  429 431 430
-3  231 232 432
-3  430 434 433
-3  435 436 190
-3  433 437 338
-3  438 440 439
-3  322 173 441
-3  118 101 102
-3  72 443 442
-3  444 353 354
-3  444 356 353
-3  445 447 446
-3  45 445 446
-3  448 450 449
-3  451 202 375
-3  452 453 430
-3  454 192 276
-3  455 43 41
-3  455 456 43
-3  457 86 87
-3  458 86 457
-3  459 331 460
-3  291 272 273
-3  461 463 462
-3  210 464 212
-3  465 177 40
-3  466 140 64
-3  467 114 112
-3  468 470 469
-3  471 473 472
-3  132 93 91
-3  202 377 375
-3  402 475 474
-3  183 184 99
-3  423 476 169
-3  66 477 255
-3  478 108 479
-3  480 382 481
-3  482 357 271
-3  483 313 484
-3  157 400 485
-3  486 181 487
-3  488 489 459
-3  69 491 490
-3  492 392 393
-3  478 125 108
-3  493 263 264
-3  233 446 232
-3  233 44 446
-3  268 269 494
-3  212 495 211
-3  200 496 234
-3  497 498 217
-3  499 501 500
-3  225 346 502
-3  334 335 379
-3  503 505 504
-3  427 478 300
-3  506 508 507
-3  509 511 510
-3  142 196 512
-3  513 358 514
-3  515 516 509
-3  517 519 518
-3  248 249 520
-3  152 154 521
-3  503 522 505
-3  325 523 327
-3  105 365 88
-3  235 524 188
-3  525 527 526
-3  528 375 14
-3  495 529 211
-3  14 297 13
-3  413 25 341
-3  530 531 26
-3  14 376 297
-3  532 296 533
-3  267 340 306
-3  379 495 212
-3  534 429 535
-3  167 168 292
-3  536 537 519
-3  538 213 438
-3  320 228 229
-3  539 540 85
-3  376 533 297
-3  222 541 348
-3  28 543 542
-3  544 337 545
-3  199 234 477
-3  150 1 155
-3  350 31 172
-3  352 42 141
-3  150 155 153
-3  150 153 152
-3  546 547 529
-3  277 549 548
-3  550 124 122
-3  551 306 305
-3  552 410 543
-3  549 483 159
-3  446 447 232
-3  553 501 499
-3  345 535 344
-3  104 105 500
-3  422 555 554
-3  556 557 299
-3  556 494 557
-3  558 560 559
-3  220 561 363
-3  360 95 426
-3  62 562 285
-3  296 532 520
-3  97 563 96
-3  564 481 383
-3  565 567 566
-3  565 384 567
-3  270 285 568
-3  243 507 63
-3  518 569 364
-3  99 184 98
-3  29 94 31
-3  385 386 570
-3  571 206 527
-3  572 574 573
-3  367 136 250
-3  354 575 309
-3  576 578 577
-3  311 312 579
-3  577 215 580
-3  581 386 20
-3  570 386 581
-3  79 582 24
-3  441 321 322
-3  512 196 583
-3  584 585 66
-3  391 586 393
-3  513 497 120
-3  134 523 325
-3  161 162 587
-3  91 92 588
-3  84 589 83
-3  590 310 591
-3  592 541 248
-3  127 128 593
-3  0 1 150
-3  126 122 128
-3  539 594 516
-3  539 85 594
-3  595 289 54
-3  246 596 342
-3  236 138 139
-3  597 599 598
-3  600 67 220
-3  257 111 75
-3  601 111 257
-3  601 602 111
-3  310 590 603
-3  604 18 17
-3  605 240 385
-3  550 122 126
-3  606 607 462
-3  569 425 364
-3  485 400 579
-3  608 306 551
-3  267 306 608
-3  491 609 490
-3  492 393 610
-3  391 611 586
-3  124 387 388
-3  511 613 612
-3  614 6 180
-3  614 180 178
-3  302 174 615
-3  616 124 388
-3  617 520 249
-3  618 290 619
-3  287 620 618
-3  287 360 620
-3  453 440 15
-3  326 327 621
-3  622 278 623
-3  624 626 625
-3  319 123 318
-3  474 627 531
-3  241 242 407
-3  472 473 44
-3  500 105 628
-3  245 235 244
-3  487 630 629
-3  479 174 302
-3  46 632 631
-3  633 427 428
-3  492 610 634
-3  130 399 400
-3  462 635 622
-3  568 21 3
-3  6 91 588
-3  415 275 417
-3  636 637 519
-3  171 637 636
-3  3 4 638
-3  639 628 89
-3  239 243 241
-3  46 631 640
-3  641 642 491
-3  214 570 604
-3  503 553 522
-3  467 528 114
-3  438 439 634
-3  643 644 60
-3  643 645 644
-3  21 5 3
-3  646 647 252
-3  188 260 189
-3  646 252 90
-3  21 568 648
-3  649 650 605
-3  612 52 308
-3  368 135 136
-3  55 619 651
-3  633 428 503
-3  59 653 652
-3  100 654 99
-3  655 524 235
-3  655 264 524
-3  656 69 657
-3  331 658 460
-3  331 332 658
-3  111 602 659
-3  89 252 253
-3  584 66 255
-3  497 217 660
-3  213 604 661
-3  662 663 204
-3  447 70 71
-3  584 663 662
-3  260 664 189
-3  260 261 664
-3  206 571 349
-3  96 558 426
-3  665 667 666
-3  644 668 432
-3  644 472 668
-3  646 669 647
-3  256 496 566
-3  308 52 50
-3  470 388 389
-3  595 54 49
-3  670 225 502
-3  670 224 225
-3  463 606 462
-3  259 118 261
-3  575 515 510
-3  574 671 573
-3  106 517 518
-3  672 465 673
-3  413 674 81
-3  675 676 650
-3  677 485 368
-3  678 680 679
-3  94 441 173
-3  305 611 681
-3  219 682 669
-3  516 683 509
-3  207 347 684
-3  484 685 659
-3  686 193 464
-3  281 557 494
-3  603 590 402
-3  687 145 506
-3  604 570 18
-3  688 689 151
-3  376 377 690
-3  339 413 341
-3  691 468 692
-3  693 146 125
-3  552 411 410
-3  377 545 690
-3  694 463 461
-3  684 347 348
-3  660 217 176
-3  363 561 365
-3  607 635 462
-3  400 401 579
-3  451 201 202
-3  451 204 201
-3  667 541 592
-3  695 150 151
-3  112 113 696
-3  54 289 53
-3  469 470 609
-3  697 693 125
-3  698 693 697
-3  100 405 654
-3  197 242 61
-3  160 548 159
-3  504 700 699
-3  532 533 701
-3  19 703 702
-3  611 391 681
-3  566 705 704
-3  706 142 512
-3  628 88 89
-3  663 166 164
-3  332 146 693
-3  470 468 388
-3  473 46 44
-3  135 310 603
-3  96 563 707
-3  213 661 438
-3  140 477 64
-3  560 339 340
-3  137 531 251
-3  689 625 695
-3  708 452 709
-3  710 534 535
-3  711 712 11
-3  591 554 713
-3  714 292 293
-3  715 28 26
-3  715 543 28
-3  64 477 66
-3  661 15 440
-3  424 717 716
-3  718 76 185
-3  160 454 548
-3  566 704 165
-3  437 720 719
-3  215 577 721
-3  276 277 548
-3  722 694 461
-3  545 723 690
-3  81 724 80
-3  725 253 598
-3  599 725 598
-3  506 507 676
-3  723 727 726
-3  709 452 480
-3  328 329 638
-3  728 111 209
-3  728 110 111
-3  65 585 182
-3  572 641 179
-3  330 331 729
-3  94 173 31
-3  143 361 170
-3  346 347 187
-3  442 443 730
-3  490 609 389
-3  77 418 131
-3  731 732 70
-3  447 731 70
-3  412 39 176
-3  733 178 523
-3  483 315 313
-3  190 436 563
-3  234 496 256
-3  288 37 95
-3  106 364 365
-3  488 734 714
-3  485 579 312
-3  80 724 110
-3  429 430 433
-3  68 490 682
-3  68 69 490
-3  139 140 466
-3  295 183 99
-3  8 29 303
-3  509 735 511
-3  347 207 205
-3  736 115 712
-3  113 114 412
-3  509 683 735
-3  209 685 208
-3  518 33 569
-3  616 691 547
-3  258 737 345
-3  258 704 737
-3  294 738 703
-3  179 641 656
-3  278 279 370
-3  202 343 544
-3  642 469 609
-3  10 717 554
-3  610 679 739
-3  740 578 576
-3  741 743 742
-3  230 597 593
-3  218 113 390
-3  744 290 618
-3  360 426 456
-3  595 745 730
-3  262 746 264
-3  503 428 553
-3  439 440 708
-3  190 563 97
-3  461 462 622
-3  525 747 738
-3  132 748 134
-3  132 91 748
-3  582 79 208
-3  53 618 619
-3  472 44 231
-3  198 100 407
-3  133 134 325
-3  432 232 271
-3  416 417 687
-3  432 271 149
-3  749 25 23
-3  341 25 749
-3  119 564 261
-3  742 743 750
-3  332 116 146
-3  461 622 623
-3  589 552 751
-3  635 699 622
-3  752 318 546
-3  647 550 126
-3  187 205 186
-3  340 559 560
-3  413 81 25
-3  592 520 532
-3  689 695 151
-3  753 754 496
-3  753 383 754
-3  19 20 183
-3  399 755 401
-3  572 573 641
-3  712 115 756
-3  410 380 757
-3  672 660 177
-3  482 72 442
-3  9 311 579
-3  178 758 523
-3  759 480 481
-3  745 442 730
-3  337 338 727
-3  305 23 611
-3  294 703 19
-3  590 713 404
-3  590 591 713
-3  101 760 681
-3  149 271 357
-3  761 127 593
-3  597 761 593
-3  610 678 679
-3  60 432 149
-3  622 699 278
-3  762 763 686
-3  210 763 762
-3  764 765 536
-3  764 766 765
-3  471 555 473
-3  148 83 59
-3  423 169 631
-3  767 304 299
-3  768 753 200
-3  768 564 753
-3  358 359 514
-3  424 769 476
-3  595 770 289
-3  195 395 196
-3  10 11 717
-3  98 184 238
-3  771 772 665
-3  15 16 434
-3  342 596 265
-3  47 498 513
-3  65 66 585
-3  482 442 359
-3  38 773 190
-3  156 677 223
-3  225 129 156
-3  573 282 642
-3  584 166 663
-3  31 350 30
-3  232 447 71
-3  296 297 533
-3  586 24 582
-3  454 276 548
-3  774 318 752
-3  355 775 575
-3  355 776 775
-3  419 420 583
-3  420 777 583
-3  778 16 437
-3  434 16 778
-3  583 196 396
-3  311 591 310
-3  426 558 779
-3  309 510 612
-3  780 529 547
-3  136 137 251
-3  615 171 537
-3  619 290 291
-3  143 170 175
-3  33 87 569
-3  781 120 82
-3  511 777 613
-3  644 432 60
-3  244 141 246
-3  548 549 159
-3  241 98 238
-3  598 254 127
-3  428 301 553
-3  420 613 777
-3  595 730 770
-3  681 391 103
-3  366 37 288
-3  651 291 486
-3  253 254 598
-3  615 174 171
-3  782 783 508
-3  648 783 782
-3  282 469 642
-3  134 733 523
-3  632 423 631
-3  784 667 665
-3  69 656 491
-3  491 642 609
-3  477 234 255
-3  283 468 469
-3  299 61 62
-3  785 168 169
-3  476 785 169
-3  374 372 414
-3  488 293 489
-3  242 197 407
-3  298 13 297
-3  435 190 773
-3  65 786 64
-3  466 64 786
-3  527 654 571
-3  787 788 695
-3  626 788 787
-3  539 516 515
-3  115 330 756
-3  789 780 547
-3  19 702 18
-3  646 219 669
-3  716 756 769
-3  637 33 519
-3  790 549 277
-3  791 498 47
-3  671 557 281
-3  549 790 315
-3  767 7 303
-3  791 216 498
-3  649 605 578
-3  644 645 472
-3  792 77 51
-3  226 172 173
-3  691 789 547
-3  519 33 518
-3  366 288 770
-3  496 565 566
-3  189 199 138
-3  122 123 319
-3  793 680 685
-3  208 685 680
-3  645 58 403
-3  616 388 691
-3  468 691 388
-3  214 721 385
-3  117 397 395
-3  219 68 682
-3  146 147 107
-3  668 231 432
-3  668 472 231
-3  344 336 337
-3  431 452 430
-3  793 313 794
-3  275 193 686
-3  515 509 510
-3  795 464 193
-3  795 212 464
-3  144 21 782
-3  144 782 145
-3  337 544 344
-3  458 361 683
-3  584 255 166
-3  796 396 397
-3  307 321 92
-3  308 309 612
-3  30 405 198
-3  248 541 222
-3  797 55 651
-3  798 705 710
-3  704 705 798
-3  525 738 294
-3  492 634 439
-3  107 147 195
-3  220 219 561
-3  799 0 800
-3  799 800 626
-3  801 719 720
-3  264 259 524
-3  701 723 771
-3  158 154 155
-3  755 736 711
-3  104 764 517
-3  315 576 314
-3  802 696 182
-3  714 449 435
-3  406 530 803
-3  261 564 804
-3  251 531 530
-3  751 552 543
-3  714 734 449
-3  790 415 740
-3  779 265 596
-3  161 805 163
-3  433 338 336
-3  383 384 565
-3  806 807 245
-3  806 263 807
-3  808 757 542
-3  543 808 542
-3  527 206 526
-3  174 170 171
-3  809 732 810
-3  45 809 810
-3  117 395 147
-3  384 534 710
-3  811 266 263
-3  342 266 811
-3  241 238 239
-3  384 710 705
-3  650 676 239
-3  617 803 298
-3  113 412 390
-3  723 726 771
-3  714 293 488
-3  575 510 309
-3  638 329 3
-3  3 329 568
-3  769 729 371
-3  786 181 486
-3  750 743 369
-3  518 364 106
-3  682 389 387
-3  321 441 92
-3  544 545 377
-3  130 812 399
-3  418 419 812
-3  236 188 138
-3  640 813 56
-3  258 345 343
-3  804 768 200
-3  804 564 768
-3  474 475 627
-3  589 751 653
-3  764 536 517
-3  301 302 766
-3  736 117 115
-3  465 40 381
-3  218 629 630
-3  758 179 656
-3  108 109 175
-3  207 526 206
-3  449 450 436
-3  672 673 84
-3  298 380 13
-3  411 465 381
-3  449 436 435
-3  482 71 72
-3  380 298 803
-3  196 395 396
-3  777 512 583
-3  460 693 606
-3  651 619 291
-3  7 8 303
-3  2 454 1
-3  607 427 633
-3  814 662 204
-3  422 423 632
-3  797 791 48
-3  624 625 815
-3  372 741 742
-3  128 122 319
-3  679 680 793
-3  299 304 61
-3  681 103 101
-3  466 273 274
-3  143 175 109
-3  707 374 560
-3  186 205 349
-3  52 612 613
-3  647 254 252
-3  92 441 588
-3  736 712 711
-3  754 565 496
-3  754 383 565
-3  456 426 779
-3  744 41 290
-3  360 456 744
-3  816 628 639
-3  499 816 639
-3  726 772 771
-3  490 389 682
-3  546 616 547
-3  275 276 193
-3  594 86 516
-3  594 85 86
-3  85 817 87
-3  818 530 406
-3  818 251 530
-3  264 746 259
-3  119 759 481
-3  819 250 247
-3  819 367 250
-3  522 553 499
-3  820 813 167
-3  821 710 535
-3  345 821 535
-3  813 820 56
-3  820 36 56
-3  14 375 376
-3  470 389 609
-3  39 412 324
-3  36 366 57
-3  57 366 443
-3  357 482 359
-3  283 692 468
-3  248 222 247
-3  695 788 822
-3  800 822 788
-3  634 538 438
-3  629 218 216
-3  772 747 665
-3  525 526 747
-3  650 239 240
-3  340 341 306
-3  823 13 381
-3  40 823 381
-3  807 493 245
-3  807 263 493
-3  824 375 528
-3  467 824 528
-3  536 519 517
-3  219 646 90
-3  19 183 294
-3  694 459 463
-3  489 293 371
-3  330 116 332
-3  463 459 460
-3  393 586 678
-3  641 573 642
-3  722 623 825
-3  129 130 157
-3  826 625 689
-3  815 625 826
-3  255 234 256
-3  586 582 678
-3  189 804 200
-3  254 647 126
-3  606 698 607
-3  318 123 546
-3  437 16 720
-3  50 718 322
-3  661 17 15
-3  760 551 305
-3  760 305 681
-3  779 558 559
-3  461 623 722
-3  307 354 309
-3  504 699 635
-3  584 662 585
-3  663 164 203
-3  814 204 451
-3  199 140 138
-3  15 434 453
-3  338 719 727
-3  706 512 735
-3  767 299 557
-3  281 827 283
-3  709 480 759
-3  707 560 558
-3  385 570 214
-3  197 61 304
-3  739 538 634
-3  338 437 719
-3  633 503 504
-3  453 434 430
-3  270 568 329
-3  270 329 269
-3  684 667 784
-3  256 566 165
-3  99 654 295
-3  772 726 801
-3  712 716 11
-3  180 572 179
-3  262 760 746
-3  226 718 185
-3  828 366 770
-3  730 828 770
-3  119 481 564
-3  169 167 813
-3  71 482 271
-3  404 645 403
-3  404 471 645
-3  318 774 320
-3  624 815 774
-3  774 815 320
-3  624 774 394
-3  335 624 394
-3  261 118 119
-3  76 502 185
-3  746 760 101
-3  829 7 574
-3  829 574 572
-3  672 84 121
-3  830 76 718
-3  830 51 76
-3  113 218 696
-3  620 744 618
-3  620 360 744
-3  592 248 520
-3  307 92 93
-3  132 133 356
-3  831 813 640
-3  631 831 640
-3  74 369 279
-3  192 193 276
-3  379 394 495
-3  394 752 495
-3  334 192 2
-3  672 177 465
-3  256 165 166
-3  382 534 384
-3  700 161 587
-3  792 420 418
-3  51 52 792
-3  52 613 792
-3  226 186 172
-3  792 613 420
-3  349 172 186
-3  274 42 352
-3  244 352 141
-3  202 544 377
-3  769 371 476
-3  423 424 476
-3  723 701 690
-3  16 702 720
-3  85 832 817
-3  832 833 817
-3  834 835 709
-3  484 313 793
-3  686 763 317
-3  786 486 273
-3  70 836 72
-3  70 57 836
-3  833 621 600
-3  61 242 63
-3  139 466 274
-3  735 512 777
-3  466 786 273
-3  474 531 137
-3  552 589 673
-3  837 308 50
-3  837 50 321
-3  380 803 757
-3  459 694 488
-3  567 705 566
-3  567 384 705
-3  348 541 684
-3  207 784 526
-3  413 414 674
-3  303 304 767
-3  616 123 124
-3  546 123 616
-3  427 698 478
-3  606 693 698
-3  747 526 784
-3  629 216 791
-3  121 120 497
-3  734 448 449
-3  747 784 665
-3  358 513 120
-3  58 643 60
-3  58 645 643
-3  523 758 327
-3  588 94 8
-3  245 421 806
-3  27 542 757
-3  27 28 542
-3  295 654 527
-3  792 418 77
-3  322 718 226
-3  322 226 173
-3  73 351 838
-3  587 838 351
-3  669 124 550
-3  647 669 550
-3  774 752 394
-3  645 471 472
-3  368 312 135
-3  121 497 660
-3  672 121 660
-3  147 395 195
-3  485 312 368
-3  2 192 454
-3  393 678 610
-3  431 534 382
-3  480 431 382
-3  480 452 431
-3  611 24 586
-3  611 23 24
-3  560 374 339
-3  374 414 339
-3  399 796 755
-3  605 721 578
-3  328 638 789
-3  675 687 506
-3  506 676 675
-3  649 675 650
-3  578 721 577
-3  98 241 407
-3  649 578 740
-3  789 691 692
-3  223 677 367
-3  677 368 367
-3  561 219 90
-3  633 504 635
-3  561 90 88
-3  365 561 88
-3  11 9 711
-3  521 73 838
-3  722 825 448
-3  722 448 734
-3  460 606 463
-3  115 116 330
-3  559 340 267
-3  213 214 604
-3  588 8 6
-3  734 488 694
-3  712 756 716
-3  722 734 694
-3  551 760 262
-3  266 551 262
-3  218 630 696
-3  182 696 630
-3  487 181 630
-3  835 492 439
-3  181 182 630
-3  406 803 617
-3  617 249 406
-3  493 655 235
-3  493 264 655
-3  355 575 354
-3  356 133 776
-3  776 133 398
-3  133 325 398
-3  356 776 355
-3  486 291 273
-3  530 757 803
-3  627 26 531
-3  627 715 26
-3  751 543 715
-3  165 704 258
-3  117 147 116
-3  534 431 429
-3  535 336 344
-3  265 559 267
-3  600 621 67
-3  673 411 552
-3  48 54 55
-3  274 272 42
-3  559 265 779
-3  585 662 802
-3  814 802 662
-3  545 337 727
-3  673 465 411
-3  209 111 659
-3  658 693 460
-3  658 332 693
-3  151 839 688
-3  163 805 597
-3  604 17 661
-3  254 126 127
-3  152 839 151
-3  152 521 839
-3  840 18 702
-3  17 18 840
-3  799 2 0
-3  335 799 626
-3  624 335 626
-3  841 43 596
-3  141 43 841
-3  359 442 745
-3  125 146 107
-3  698 427 607
-3  842 843 475
-3  842 653 843
-3  505 522 805
-3  105 88 628
-3  379 212 194
-3  664 804 189
-3  664 261 804
-3  680 582 208
-3  545 727 723
-3  724 674 750
-3  674 724 81
-3  814 451 467
-3  422 632 555
-3  632 473 555
-3  39 324 12
-3  114 528 324
-3  844 223 221
-3  844 156 223
-3  167 773 820
-3  473 632 46
-3  571 654 405
-3  772 801 738
-3  703 738 801
-3  634 610 739
-3  747 772 738
-3  178 179 758
-3  331 459 489
-3  769 756 729
-3  756 330 729
-3  845 489 371
-3  729 845 371
-3  680 678 582
-3  414 372 742
-3  242 243 63
-3  755 796 397
-3  414 742 674
-3  674 742 750
-3  736 397 117
-3  755 397 736
-3  677 157 485
-3  295 527 525
-3  685 209 659
-3  607 633 635
-3  107 195 109
-3  703 801 720
-3  438 661 440
-3  702 703 720
-3  521 838 839
-3  158 155 160
-3  155 1 160
-3  454 160 1
-3  758 656 657
-3  327 758 657
-3  588 441 94
-3  721 214 215
-3  538 215 213
-3  525 294 295
-3  733 91 6
-3  59 60 148
-3  590 404 402
-3  76 77 224
-3  225 221 346
-3  348 346 221
-3  677 156 157
-3  348 221 222
-3  739 580 538
-3  538 580 215
-3  682 387 669
-3  577 580 314
-3  780 4 211
-3  4 210 211
-3  275 686 417
-3  574 7 767
-3  671 282 573
-3  281 282 671
-3  401 711 9
-3  755 711 401
-3  217 498 216
-3  791 47 48
-3  351 74 279
-3  549 315 483
-3  791 797 629
-3  629 797 487
-3  741 372 450
-3  448 741 450
-3  448 825 741
-3  743 741 825
-3  721 605 385
-3  419 396 796
-3  650 240 605
-3  475 843 627
-3  35 637 171
-3  846 433 336
-3  429 433 846
-3  653 751 843
-3  471 713 555
-3  404 713 471
-3  713 554 555
-3  514 47 513
-3  514 49 47
-3  716 769 424
-3  717 11 716
-3  554 717 422
-3  282 283 469
-3  84 673 589
-3  521 257 73
-3  659 847 484
-3  483 484 847
-3  847 159 483
-3  158 159 847
-3  145 687 317
-3  436 450 373
-3  450 372 373
-3  317 22 145
-3  417 317 687
-3  82 121 84
-3  135 312 310
-3  135 603 137
-3  603 474 137
-3  686 317 417
-3  5 763 210
-3  4 5 210
-3  149 781 148
-3  149 357 781
-3  781 358 120
-3  148 781 82
-3  797 48 55
-3  373 707 563
-3  436 373 563
-3  33 34 87
-3  62 63 562
-3  145 508 506
-3  568 285 648
-3  848 849 562
-3  508 849 848
-3  21 648 782
-3  145 782 508
-3  507 562 63
-3  253 639 89
-3  724 850 110
-3  851 767 557
-3  851 557 671
-3  599 499 639
-3  522 499 599
-3  522 599 805
-3  805 599 597
-3  850 75 110
-3  850 74 75
-3  850 369 74
-3  373 374 707
-3  343 344 544
-3  602 158 847
-3  602 847 659
-3  204 663 203
-3  202 203 343
-3  683 706 735
-3  154 158 602
-3  154 257 521
-3  415 416 740
-3  692 328 789
-3  649 740 416
-3  649 416 675
-3  416 687 675
-3  541 667 684
-3  349 571 350
-3  789 638 780
-3  638 4 780
-3  405 350 571
-3  350 405 30
-3  142 706 143
-3  743 825 370
-3  784 207 684
-3  600 220 363
-3  452 708 453
-3  802 182 585
-3  29 191 303
-3  656 641 491
-3  453 708 440
-3  801 726 719
-3  835 708 709
-3  835 439 708
-3  576 577 314
-3  726 727 719
-3  315 790 576
-3  740 576 790
-3  739 794 580
-3  177 660 176
-3  177 39 40
-3  162 839 838
-3  587 162 838
-3  163 688 162
-3  679 793 794
-3  679 794 739
-3  580 794 314
-3  313 314 794
-3  617 296 520
-3  617 298 296
-3  412 176 390
-3  834 392 835
-3  392 492 835
-3  229 593 128
-3  391 392 103
-3  102 103 834
-3  103 392 834
-3  65 181 786
-3  53 619 55
-3  707 558 96
-3  41 272 290
-3  41 42 272
-3  780 211 529
-3  752 546 529
-3  752 529 495
-3  790 277 415
-3  415 277 275
-3  511 735 777
-3  154 601 257
-3  154 602 601
-3  327 657 621
-3  163 597 230
-3  163 230 688
-3  230 689 688
-3  230 228 689
-3  320 815 228
-3  319 229 128
-3  320 229 319
-3  540 832 85
-3  852 515 575
-3  776 398 540
-3  540 398 832
-3  776 540 852
-3  836 443 72
-3  836 57 443
-3  493 235 245
-3  532 666 592
-3  666 532 701
-3  131 418 812
-3  311 9 10
-3  591 311 10
-3  513 498 497
-3  579 401 9
-3  666 771 665
-3  666 667 592
-3  224 131 129
-3  745 595 49
-3  514 359 745
-3  102 759 119
-3  102 834 759
-3  834 709 759
-3  140 199 477
-3  853 343 203
-3  164 853 203
-3  46 640 45
-3  640 56 732
-3  854 615 537
-3  536 854 537
-3  855 856 854
-3  765 855 854
-3  765 856 855
-3  44 45 446
-3  494 827 281
-3  494 269 827
-3  328 827 269
-3  280 351 279
-3  283 827 692
-3  827 328 692
-3  280 587 351
-3  706 361 143
-3  690 701 533
-3  699 280 278
-3  376 690 533
-3  699 700 280
-3  700 587 280
-3  817 425 569
-3  833 600 425
-3  833 425 817
-3  361 706 683
-3  764 104 857
-3  501 764 857
-3  820 38 36
-3  820 773 38
-3  857 104 500
-3  501 857 500
-3  37 97 95
-3  48 49 54
-3  744 455 41
-3  744 456 455
-3  657 69 67
-3  657 67 621
-3  583 396 419
-3  289 287 53
-3  53 287 618
-3  288 360 287
-3  167 292 773
-3  478 858 300
-3  478 479 858
-3  732 57 70
-3  732 56 57
-3  858 302 300
-3  858 479 302
-3  402 403 475
-3  475 403 652
-3  524 859 188
-3  524 259 859
-3  361 458 35
-3  481 382 383
-3  859 260 188
-3  859 259 260
-3  753 564 383
-3  200 753 496
-3  458 34 35
-3  292 714 435
-3  292 435 773
-3  86 458 516
-3  852 860 515
-3  852 540 860
-3  422 717 424
-3  817 569 87
-3  832 398 326
-3  832 326 833
-3  326 621 833
-3  352 139 274
-3  106 365 105
-3  187 347 205
-3  90 252 89
-3  130 131 812
-3  502 346 187
-3  860 539 515
-3  860 540 539
-3  83 589 653
-3  603 402 474
-3  59 83 653
-3  516 458 683
-3  479 108 175
-3  479 175 174
-3  300 301 428
-3  553 301 501
-3  104 517 106
-3  301 766 501
-3  501 766 764
-3  530 27 757
-3  530 26 27
-3  161 505 805
-3  700 505 161
-3  504 505 700
-3  502 187 185
-3  217 390 176
-3  701 771 666
-3  685 484 793
-3  750 369 850
-3  724 750 850
-3  651 486 487
-3  797 651 487
-3  781 357 358
-3  591 10 554
-3  100 98 407
-3  812 419 796
-3  205 206 349
-3  399 812 796
-3  779 596 456
-3  456 596 43
-3  802 112 696
-3  814 112 802
-3  814 467 112
-3  425 600 363
-3  743 370 369
-3  623 370 825
-3  623 278 370
-3  669 387 124
-3  688 839 162
-3  510 511 612
-3  352 236 139
-3  746 118 259
-3  746 101 118
-3  38 97 37
-3  514 745 49
-3  856 302 615
-3  766 302 856
-3  640 732 809
-3  45 640 809
-3  258 343 853
-3  164 258 853
-3  268 62 284
-3  268 284 270
-3  285 562 861
-3  285 861 648
-3  507 848 562
-3  508 848 507
-3  861 562 849
-3  861 849 783
-3  783 849 508
-3  861 783 648
-3  537 636 519
-3  171 636 537
-3  535 846 336
-3  429 846 535
-3  228 826 689
-3  815 826 228
-3  748 733 134
-3  748 91 733
-3  626 800 788
-3  799 333 2
-3  799 335 333
-3  246 841 596
-3  141 841 246
-3  381 380 409
-3  411 381 409
-3  715 627 862
-3  751 715 862
-3  862 627 843
-3  751 862 843
-3  625 787 695
-3  626 787 625
-3  345 737 821
-3  695 822 150
-3  0 150 822
-3  775 852 575
-3  775 776 852
-3  421 342 811
-3  806 811 263
-3  421 811 806
-3  250 818 406
-3  250 251 818
-3  266 608 551
-3  267 608 266
-3  410 757 808
-3  543 410 808
-3  307 308 837
-3  307 837 321
-3  863 798 821
-3  737 863 821
-3  737 798 863
-3  737 704 798
-3  821 798 710
-3  800 0 822
-3  864 507 243
-3  676 507 864
-3  239 864 243
-3  676 864 239
-3  169 813 831
-3  631 169 831
-3  865 59 652
-3  403 865 652
-3  58 59 865
-3  403 58 865
-3  362 93 408
-3  444 408 356
-3  444 362 408
-3  444 354 362
-3  45 810 445
-3  810 732 731
-3  445 810 731
-3  445 731 447
-3  536 765 854
-3  856 615 854
-3  765 766 856
-3  194 795 193
-3  194 212 795
-3  464 762 686
-3  210 762 464
-3  34 457 87
-3  458 457 34
-3  639 253 725
-3  599 639 725
-3  574 767 851
-3  574 851 671
-3  76 670 502
-3  76 224 670
-3  180 7 829
-3  180 829 572
-3  18 581 20
-3  570 581 18
-3  16 840 702
-3  17 840 16
-3  598 127 761
-3  597 598 761
-3  80 728 209
-3  80 110 728
-3  331 489 845
-3  729 331 845
-3  451 375 824
-3  467 451 824
-3  770 288 286
-3  289 770 286
-3  25 78 79
-3  81 78 25
-3  528 14 323
-3  324 528 323
-3  225 844 221
-3  225 156 844
-3  500 628 816
-3  499 500 816
-3  652 842 475
-3  652 653 842
-3  593 229 227
-3  230 593 227
-3  268 556 299
-3  268 494 556
-3  763 316 317
-3  5 316 763
-3  637 32 33
-3  35 32 637
-3  733 6 614
-3  733 614 178
-3  443 366 828
-3  730 443 828
-3  223 819 247
-3  223 367 819
-3  386 237 238
-3  240 237 386
-3  371 168 785
-3  476 371 785
-3  334 378 192
-3  334 379 378
-3  478 697 125
-3  698 697 478
-3  433 778 437
-3  434 778 433
-3  50 830 718
-3  50 51 830
-3  306 749 23
-3  341 749 306
-3  12 13 823
-3  40 12 823
-
diff -pruN 5.0.3-1/examples/Surface_mesh_simplification/cube.off 5.1-1/examples/Surface_mesh_simplification/cube.off
--- 5.0.3-1/examples/Surface_mesh_simplification/cube.off	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_simplification/cube.off	1970-01-01 00:00:00.000000000 +0000
@@ -1,24 +0,0 @@
-OFF
-8 12 0
-
--0.274878 -0.274878 -0.274878
--0.274878 0.274878 -0.274878
-0.274878 0.274878 -0.274878
-0.274878 -0.274878 -0.274878
--0.274878 -0.274878 0.274878
--0.274878 0.274878 0.274878
-0.274878 0.274878 0.274878
-0.274878 -0.274878 0.274878
-3  0 1 3
-3  3 1 2
-3  0 4 1
-3  1 4 5
-3  3 2 7
-3  7 2 6
-3  4 0 3
-3  7 4 3
-3  6 4 7
-3  6 5 4
-3  1 5 6
-3  2 1 6
-
diff -pruN 5.0.3-1/examples/Surface_mesh_simplification/data/cube-meshed.off 5.1-1/examples/Surface_mesh_simplification/data/cube-meshed.off
--- 5.0.3-1/examples/Surface_mesh_simplification/data/cube-meshed.off	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_simplification/data/cube-meshed.off	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,2598 @@
+OFF
+866 1728 0
+
+-0.875 1 -0.875
+-0.75 1 -0.75
+-0.875 1 -0.625
+-1 0.75 0.5
+-1 0.75 0.25
+-1 0.875 0.375
+-0.875 -0.375 1
+-0.875 -0.125 1
+-0.75 -0.25 1
+0.25 -1 -0
+0.375 -1 -0.125
+0.25 -1 -0.25
+1 -0.25 1
+0.875 -0.375 1
+0.875 -0.125 1
+0.25 1 0.75
+0.25 1 1
+0.125 1 0.875
+-0 1 1
+-0 0.75 1
+-0.125 0.875 1
+-1 0.875 0.625
+-1 1 0.5
+0.375 1 -0.875
+0.25 1 -0.75
+0.25 1 -1
+0.875 -1 0.875
+0.875 -1 1
+1 -1 0.875
+-0.625 -0.125 1
+-0.5 -0 1
+-0.5 -0.25 1
+-1 -1 -0.375
+-1 -0.875 -0.375
+-1 -1 -0.25
+-0.875 -1 -0.375
+0.75 -0.5 -1
+0.75 -0.25 -1
+0.625 -0.375 -1
+1 -0.25 0.75
+1 -0.375 0.875
+1 0.375 -0.875
+1 0.5 -0.75
+1 0.5 -1
+0.875 -1 -0.625
+0.875 -1 -0.875
+0.75 -1 -0.75
+1 -0.25 -0.25
+1 -0.125 -0.375
+1 -0.25 -0.5
+-0.5 -1 1
+-0.375 -1 0.875
+-0.5 -1 0.75
+1 -0 -0.75
+1 -0.125 -0.625
+1 -0 -0.5
+0.75 -0.75 -1
+0.875 -0.625 -1
+1 -1 -0
+1 -0.875 0.125
+1 -0.875 -0.125
+-0.75 0.5 1
+-0.875 0.625 1
+-0.75 0.75 1
+1 0.5 -0
+1 0.375 0.125
+1 0.5 0.25
+-1 -0.375 0.125
+-1 -0.25 -0
+-1 -0.25 0.25
+1 -0.75 -1
+1 -0.75 -0.75
+1 -0.625 -0.875
+-0.375 0.625 -1
+-0.25 0.5 -1
+-0.25 0.75 -1
+-0.25 -1 1
+-0.25 -1 0.75
+0.125 1 -1
+0.125 1 -0.875
+-0 1 -1
+0.125 0.875 -1
+1 -0.625 0.125
+1 -0.75 0.25
+1 -0.625 0.375
+-1 -0.875 0.125
+-1 -1 -0
+-1 -0.875 -0.125
+-1 -0.375 -0.625
+-1 -0.25 -0.75
+-1 -0.25 -0.5
+-0.875 -0.625 1
+-0.75 -0.75 1
+-0.875 -0.875 1
+-0.625 -0.375 1
+0.75 -0 -1
+0.625 0.125 -1
+0.625 -0.125 -1
+-0.375 0.625 1
+-0.25 0.5 1
+-0.375 0.375 1
+0.625 1 -0.375
+0.625 1 -0.125
+0.5 1 -0.25
+-1 -0.625 -0.875
+-1 -0.5 -0.75
+-1 -0.625 -0.625
+-0.375 -1 -0.625
+-0.5 -1 -0.75
+-0.5 -1 -0.5
+-0.125 0.875 -1
+-0.25 1 -1
+1 0.125 0.625
+1 -0 0.5
+1 -0 0.75
+-0 -1 -0.5
+-0.125 -1 -0.625
+-0.125 -1 -0.375
+0.75 1 -0.25
+0.75 1 -0
+1 -0.5 -0
+1 -0.5 0.25
+-1 0.25 -0.5
+-1 0.375 -0.375
+-1 0.25 -0.25
+-0.375 -1 -0.875
+-1 0.125 -0.625
+-1 0.125 -0.875
+-1 0.25 -0.75
+-0 -1 0.75
+-0 -1 0.5
+-0.125 -1 0.625
+-1 -0.75 1
+-1 -0.75 0.75
+-1 -0.625 0.875
+0.5 -1 0.5
+0.5 -1 0.75
+0.625 -1 0.625
+1 0.75 -0.25
+1 0.625 -0.375
+1 0.625 -0.125
+1 0.625 -0.875
+-0.5 -1 -0.25
+-0.625 -1 -0.375
+-1 1 0.625
+-0.875 1 0.625
+-0.25 -1 -0.75
+-0.25 -1 -0.5
+1 -0.75 -0
+1 -0.75 -0.25
+-0.75 1 -1
+-0.75 0.75 -1
+-0.625 0.875 -1
+-0.625 1 -1
+-0.5 1 -1
+-0.625 1 -0.875
+0.125 -1 0.875
+0.125 -1 0.625
+-0.5 1 -0.75
+-0.5 1 -0.5
+-0.625 1 -0.625
+-0.625 0.125 -1
+-0.625 0.375 -1
+-0.75 0.25 -1
+1 0.625 0.875
+1 0.75 0.75
+1 0.625 0.625
+0.5 -0.75 -1
+0.375 -0.875 -1
+0.5 -1 -1
+-0.75 -1 -0.5
+-0.875 -1 -0.625
+-0.375 -0.375 1
+-0.5 -0.5 1
+-0.75 -1 -0.75
+-0.625 -1 -0.625
+1 -0.25 0.5
+1 -0.375 0.625
+-1 -0.375 0.875
+-1 -0.25 0.75
+-1 -0.25 1
+1 0.25 -0
+1 0.25 0.25
+-0.125 0.625 1
+-0.25 0.75 1
+-0.25 -0.75 1
+-0.25 -0.5 1
+-0.125 -0.625 1
+1 0.875 -0.375
+1 0.875 -0.125
+0.5 -0.25 -1
+-0.625 0.125 1
+-0.875 1 -0.375
+-0.875 1 -0.125
+-1 1 -0.25
+-0.375 -1 -0.375
+-0.375 -1 -0.125
+-0.625 0.375 1
+-0.5 0.25 1
+1 0.75 -0
+1 0.875 0.125
+1 0.375 1
+0.875 0.375 1
+1 0.5 1
+1 0.375 0.875
+-0.125 -0.375 1
+-0.125 -0.125 1
+-0 -0.25 1
+-0 1 -0.75
+-0.125 1 -0.875
+-1 0.875 0.125
+-1 0.75 -0
+-1 0.875 -0.125
+-0 1 0.5
+-0.125 1 0.625
+-0.125 1 0.375
+1 -0.125 0.125
+1 -0.25 0.25
+1 -0 0.25
+-1 -0.25 -0.25
+-1 -0.375 -0.125
+0.125 -0.875 1
+0.25 -0.75 1
+0.25 -1 1
+-0.125 -1 0.875
+-0 -1 1
+-0.375 -0.625 1
+-1 0.375 -1
+-1 0.5 -1
+-1 0.375 -0.875
+-0.875 0.375 -1
+1 -1 -0.5
+1 -0.875 -0.625
+1 -1 -0.625
+1 0.75 0.25
+1 0.875 -0.625
+1 0.75 -0.5
+-0.375 1 1
+-0.375 0.875 1
+-0.5 1 1
+-0.375 1 0.875
+-0.5 0.75 1
+-0.625 0.625 1
+-0.625 0.875 1
+1 0.75 -0.75
+1 0.875 -0.875
+1 0.75 -1
+0.375 -0.875 1
+0.375 -0.625 1
+0.5 -0.75 1
+0.5 -1 1
+0.625 -1 0.875
+-1 -0.125 -0.625
+-1 -0.125 -0.875
+-1 -0 -0.75
+1 0.625 0.375
+1 0.75 0.5
+-0.375 0.875 -1
+1 0.75 1
+0.875 1 -0.375
+1 1 -0.25
+0.875 1 -0.125
+0.75 1 -0.75
+0.875 1 -0.875
+0.875 1 -0.625
+0.75 0.75 -1
+0.75 1 -1
+0.625 0.875 -1
+-1 0.5 1
+-1 0.5 0.75
+-1 0.625 0.875
+1 -0.75 -0.5
+1 0.375 -0.625
+1 0.375 -0.375
+1 0.5 -0.5
+-0.75 1 -0
+-0.75 1 -0.25
+-0.625 1 -0.125
+-0.25 -0 -1
+-0.25 0.25 -1
+-0.375 0.125 -1
+-1 0.25 0.75
+-1 0.125 0.625
+-1 0.25 0.5
+-1 0.625 1
+-1 0.75 1
+1 -0.125 -1
+1 -0 -1
+0.875 -0.125 -1
+1 -0.125 -0.875
+1 0.25 -0.75
+1 0.25 -0.5
+0.375 -0.625 -1
+0.25 -0.75 -1
+-0 0.5 1
+-0.125 0.375 1
+0.625 -0.375 1
+0.75 -0.25 1
+0.75 -0.5 1
+-0.875 0.375 1
+-0.625 -0.875 -1
+-0.75 -0.75 -1
+-0.75 -1 -1
+-0.75 -0 1
+-0.75 0.25 1
+0.5 1 -0.75
+0.5 1 -1
+-0.75 -1 1
+-0.625 -1 0.875
+-0.75 -1 0.75
+0.5 -1 0.25
+0.375 -1 0.125
+0.375 -1 0.375
+-0.25 1 -0.25
+-0.25 1 -0
+-0.375 1 -0.125
+-1 1 0.375
+-0.875 1 0.375
+-1 0.5 -0.5
+-1 0.375 -0.625
+-1 0.5 -0.75
+-0.625 -0.875 1
+-0.5 -0.75 1
+1 -0.125 1
+1 -0.125 0.875
+-1 -0.625 0.625
+-1 -0.625 0.375
+-1 -0.5 0.5
+-1 0.5 0.5
+-1 0.625 0.625
+-0 -1 -0.75
+-0 -1 -1
+-0.125 -1 -0.875
+-1 1 -0.625
+-1 1 -0.5
+-1 0.875 -0.625
+0.625 0.875 1
+0.625 0.625 1
+0.5 0.75 1
+0.375 0.625 -1
+0.5 0.75 -1
+0.375 0.875 -1
+0.875 0.875 -1
+0.875 0.625 1
+0.75 0.75 1
+0.875 0.875 1
+-0 -0.75 1
+-0 -0.5 1
+0.125 -0.625 1
+-0.25 -0.25 1
+-0.375 -0.125 1
+-0.375 0.375 -1
+1 0.625 -0.625
+-1 -1 0.875
+-0.875 -1 0.875
+-1 -1 0.75
+-1 -0.875 0.875
+1 -0.625 -0.375
+1 -0.5 -0.25
+1 -0.5 -0.5
+0.875 0.125 -1
+-0.75 -1 -0.25
+-0.875 -1 1
+-1 -0.5 -0.25
+-1 -0.625 -0.375
+-1 -0.5 -0.5
+0.875 -0.375 -1
+0.375 -1 0.875
+0.375 -1 0.625
+-0.125 0.375 -1
+-0.125 0.125 -1
+0.25 -1 -1
+0.25 0.25 -1
+0.375 0.125 -1
+0.375 0.375 -1
+0.875 0.125 1
+0.75 -0 1
+0.75 0.25 1
+-1 1 -0.375
+-1 0.875 -0.375
+0.875 -0.625 1
+1 -0.5 1
+0.75 1 0.5
+0.875 1 0.375
+0.875 1 0.625
+-0.25 1 0.75
+-0.25 1 1
+-1 0.125 -0.125
+-1 0.25 -0
+-1 -0 -0
+1 -0.125 0.375
+0.375 1 -0.375
+0.375 1 -0.125
+0.25 1 -0.25
+-1 0.75 -0.5
+-0.25 -1 -0.25
+-0.25 -1 -0
+-0.125 -1 -0.125
+-1 -0.75 0.5
+-0 -1 0.25
+0.125 -1 0.375
+0.125 -1 0.125
+0.75 -1 0.25
+0.875 -1 0.125
+0.75 -1 -0
+-0.375 0.125 1
+0.625 -0.875 1
+-0.5 0.5 1
+-1 -0.875 1
+1 -0.625 1
+1 -0.75 1
+1 -0.625 0.875
+1 -0.125 0.625
+0.25 0.75 -1
+0.25 0.5 -1
+-0.625 1 0.125
+-0.625 1 0.375
+-0.75 1 0.25
+-0.25 -1 0.5
+-0.25 -1 0.25
+-0.375 -1 0.375
+1 0.875 -1
+0.5 -1 -0.5
+0.5 -1 -0.75
+0.375 -1 -0.625
+-1 -0.625 -0.125
+0.75 0.25 -1
+-0.5 -0.75 -1
+-0.625 -0.625 -1
+0.625 1 0.875
+0.5 1 0.75
+0.625 1 0.625
+1 -0.875 -0.375
+0.5 1 1
+0.375 1 0.875
+0.375 -0.375 -1
+0.375 -0.125 -1
+0.375 0.875 1
+0.125 1 0.375
+0.25 1 0.25
+0.25 1 0.5
+-0.625 -0.625 1
+1 -0.5 -0.75
+1 -0.5 -1
+-1 -1 1
+1 -1 -0.875
+1 -1 -0.75
+1 -0.875 -0.875
+0.125 -0.125 -1
+0.25 -0.25 -1
+0.25 -0 -1
+1 0.25 1
+0.5 1 0.5
+0.375 1 0.625
+-0.75 1 -0.5
+1 0.375 -1
+0.875 0.375 -1
+-1 -1 -0.125
+-0.875 -1 -0.125
+-0 -0.75 -1
+-0.125 -0.875 -1
+-0.125 -0.375 -1
+-0.25 -0.5 -1
+-0.125 -0.625 -1
+-1 1 -0
+1 -0.5 0.75
+1 0.5 -0.25
+1 0.125 0.875
+-1 0.25 0.25
+-1 0.125 0.375
+-1 0.125 0.125
+0.75 -1 -0.25
+0.875 -1 -0.375
+0.75 -1 -0.5
+0.75 -1 0.5
+0.875 -1 0.375
+0.375 -1 -0.875
+1 0.625 0.125
+-0.5 -1 -1
+-0.625 -1 -0.875
+0.625 1 0.375
+0.75 1 0.25
+1 -0.625 -0.625
+-0.375 1 -0.375
+-0.25 1 -0.5
+0.25 -1 0.5
+1 0.25 -0.25
+1 0.125 -0.125
+0.125 -0.625 -1
+0.125 -0.875 -1
+-1 -0.125 0.125
+-1 -0.125 0.375
+0.25 1 -0
+1 1 -0.75
+-1 0.375 0.875
+-1 0.75 -0.25
+1 0.875 0.375
+1 -0.375 0.125
+1 -0.25 -0
+-0.875 -0.375 -1
+-1 -0.5 -1
+-0.875 -0.625 -1
+-0.125 -0.875 1
+-0.625 -0.375 -1
+-0.5 -0.25 -1
+-0.625 -0.125 -1
+-0.75 1 0.75
+-0.75 1 1
+-0.875 1 0.875
+-0.75 -1 0.25
+-0.75 -1 0.5
+-0.625 -1 0.375
+-0.5 -1 -0
+1 -0.375 -0.125
+1 -0.375 -0.375
+-0.875 -1 0.375
+-0.875 -1 0.125
+-1 -0.75 -0.75
+-1 -0.75 -0.5
+-1 -0.875 -0.625
+0.5 -0.5 1
+-0.5 0.75 -1
+-0.75 -0.25 -1
+-1 -0.5 0.75
+1 1 -0.5
+-0 0.25 1
+-0 -0 1
+-0.125 0.125 1
+1 -0 1
+-1 0.625 -0.125
+0.75 -1 1
+0.75 -1 0.75
+0.5 -0.25 1
+0.625 -0.125 1
+0.75 1 0.75
+0.75 1 1
+-1 -0.875 -0.875
+-1 -1 -0.75
+-0 1 0.25
+-1 -1 0.25
+-1 -0.875 0.375
+0.25 -0.5 1
+1 -1 1
+1 -0.875 0.875
+0.75 0.5 1
+0.625 0.375 1
+-1 0.5 -0.25
+-1 0.5 -0
+-0.625 1 -0.375
+-0.5 1 -0.25
+-1 0.125 -0.375
+0.625 1 -0.875
+1 -0.75 0.75
+-0.75 -0.5 -1
+0.5 -1 -0.25
+0.625 -1 -0.375
+-1 0.375 1
+-1 0.25 1
+0.625 0.375 -1
+0.625 0.625 -1
+0.5 0.5 -1
+-1 -0.375 -0.375
+-0.875 0.875 1
+0.5 -0 -1
+0.875 1 0.125
+1 1 0.5
+1 0.875 0.625
+1 1 0.625
+-1 0.75 0.75
+-1 -0.75 -0.25
+-0.125 1 0.875
+-0.25 -0 1
+-1 -0.125 0.875
+-1 -0 0.75
+-1 -0 1
+-0.875 -1 0.625
+-0.375 1 0.125
+-0.25 1 0.25
+-0.375 1 0.375
+0.25 -1 0.25
+-0.125 1 0.125
+-0.125 1 1
+0.125 1 -0.625
+-0.375 -1 0.125
+1 0.5 0.5
+1 0.375 0.375
+0.25 1 -0.5
+-0.5 0.25 -1
+-0.75 -0.5 1
+1 -0.75 0.5
+0.625 -1 0.125
+0.5 -1 -0
+0.375 -0.375 1
+-1 0.25 -1
+-1 -1 0.125
+1 -0.25 -0.75
+0.875 0.625 -1
+-0.875 0.125 -1
+-1 -0 -1
+-0.875 -0.125 -1
+-1 -0.5 -0
+-0.375 1 -1
+-0.375 1 -0.875
+0.625 -1 0.375
+-0 1 0.75
+-0.375 1 0.625
+-0.25 -0.75 -1
+-0.375 -0.625 -1
+0.625 1 -1
+-1 -0 0.25
+0.125 1 -0.125
+0.375 1 -0.625
+-0.625 -1 0.625
+-0.5 -1 0.5
+-1 -0.375 1
+-0.875 -1 -0.875
+-1 0.375 -0.125
+0.625 -0.625 1
+1 0.125 -0.875
+1 0.125 -0.625
+1 0.125 -1
+-1 -0.5 0.25
+-0.25 -0.25 -1
+-0.125 -0.125 -1
+-1 0.75 -0.75
+-1 0.75 -1
+-1 0.875 -0.875
+0.875 -1 0.625
+-1 -0.375 -0.875
+1 -0 -0
+1 0.125 0.125
+0.625 -1 -0.875
+0.625 -1 -0.625
+-0.5 -0.5 -1
+0.125 1 0.125
+-0.375 -0.375 -1
+-1 -1 -0.625
+-1 -1 -0.5
+-1 0.625 0.375
+-1 -0.25 -1
+0.75 -1 -1
+-1 -0.125 0.625
+-1 -0 0.5
+1 -1 -0.125
+1 -1 -0.25
+0.875 -1 -0.125
+-1 -0.125 -0.375
+-1 -0 -0.5
+-1 0.875 0.875
+-0.5 1 0.5
+-0.5 1 0.75
+1 0.125 -0.375
+1 -1 0.25
+1 -0.875 0.375
+-0.25 0.25 1
+1 1 -0.625
+-1 -0.25 0.5
+-1 -0.375 0.375
+-0.125 -1 -1
+-0.25 1 -0.75
+1 -0.375 0.375
+0.125 1 0.625
+1 0.375 0.625
+1 0.5 0.75
+1 1 -0.125
+0.25 -0 1
+0.375 -0.125 1
+0.25 -0.25 1
+1 -1 -0.375
+-1 -0 -0.25
+-0.125 -1 1
+-1 0.125 0.875
+1 -0.5 0.5
+1 -0.625 0.625
+0.125 0.625 -1
+-0.625 1 0.625
+-0.625 1 0.875
+0.25 -1 0.75
+0.125 1 -0.375
+-0 1 -0.25
+-0 1 -0.5
+0.5 1 -0.5
+-1 -0.125 -0.125
+-0.75 -1 -0
+0.125 -0.375 1
+-0.125 1 -0.625
+-0.875 1 0.125
+-0.75 1 0.5
+-0.75 0.5 -1
+-0.875 0.625 -1
+0.625 0.125 1
+-1 0.375 0.125
+-1 0.375 0.375
+-0.25 -1 -1
+-0 -0.5 -1
+-0.875 0.875 -1
+1 0.125 0.375
+-0.375 -1 -1
+-0.375 -0.875 -1
+-0.375 -0.125 -1
+-0.5 -0 -1
+0.5 -0 1
+0.125 0.875 1
+0.125 0.625 1
+1 0.875 0.875
+1 1 0.75
+-0.625 -1 -0.125
+0.5 0.25 -1
+0.375 1 0.375
+0.5 1 0.25
+0.875 1 0.875
+0.125 -1 -0.125
+0.125 -1 -0.375
+0.625 -1 -0.125
+0.25 -0.5 -1
+1 -1 0.75
+0.25 -1 -0.5
+0.375 -1 -0.375
+-0.375 -0.875 1
+0.375 0.625 1
+0.25 0.75 1
+-0.25 1 0.5
+-0 -0.25 -1
+0.5 0.25 1
+-0 0.75 -1
+-1 -0.125 -1
+0.375 0.375 1
+0.5 0.5 1
+-0.125 1 -1
+0.125 -1 -0.875
+1 -0.375 -0.875
+1 -0.875 -1
+0.875 -0.875 -1
+-1 -0.5 1
+0.125 -0.375 -1
+-0.625 -1 0.125
+-0 -1 -0.25
+1 0.875 1
+0.125 0.375 1
+-0 1 -0
+-0.5 1 0.25
+0.125 0.125 -1
+0.125 0.375 -1
+-0 0.25 -1
+1 0.25 -1
+1 -0.375 -0.625
+0.75 1 -0.5
+0.125 0.125 1
+-1 -0.625 1
+0.375 1 -1
+-0 0.5 -1
+1 -0.875 0.625
+-1 0.625 -0.375
+1 1 0.25
+1 1 0.375
+-0 -1 -0
+0.125 -1 -0.625
+0.875 -0.875 1
+-1 -0.375 0.625
+0.625 1 0.125
+0.625 1 -0.625
+-1 0.125 -1
+-1 1 0.125
+-1 1 0.25
+-1 -0.75 -1
+-1 -0.875 -1
+-0.875 -0.875 -1
+-0.875 0.125 1
+1 1 0.125
+0.25 -1 -0.75
+1 -0.25 -1
+0.375 0.125 1
+0.25 0.25 1
+0.5 -0.5 -1
+-1 0.625 -0.625
+-1 -1 0.625
+-1 -0.875 0.625
+-0.5 -1 0.25
+0.375 1 1
+0.75 0.5 -1
+-1 0.625 0.125
+1 -0.625 -0.125
+-1 1 0.75
+-1 1 0.875
+0.125 -0.125 1
+0.375 -1 -1
+1 0.375 -0.125
+-1 0.875 -1
+-1 1 -1
+-1 0.5 0.25
+-0.5 1 -0
+1 -0.125 -0.125
+-0.375 -1 0.625
+-0.125 1 -0.375
+-0.125 1 -0.125
+-1 1 -0.125
+-0.125 -1 0.125
+1 -0 -0.25
+1 1 0.875
+-1 1 -0.75
+-1 1 -0.875
+0.25 0.5 1
+1 0.25 0.5
+0.75 -0.75 1
+1 1 -0
+-0.75 -0 -1
+1 1 -1
+1 1 -0.875
+1 -0.875 1
+0.875 -1 -1
+1 -1 -1
+0.875 1 -1
+-0.125 -1 0.375
+0.625 -0.875 -1
+1 0.25 0.75
+-1 0.625 -0.875
+-1 -0.375 -1
+-1 -0.75 -0
+0.625 -1 1
+0.375 -1 1
+0.625 -0.625 -1
+0.875 1 1
+-0.875 1 -1
+1 -0.375 1
+1 0.125 1
+-0 -0 -1
+-1 0.625 -1
+-1 0.375 0.625
+1 -0.375 -1
+-1 -0.125 1
+-0.375 -1 1
+0.625 -1 -1
+-1 -0.75 0.25
+-1 -0.625 0.125
+0.5 1 -0
+0.375 1 0.125
+1 -0.625 -1
+-0.625 -1 1
+-0.5 0.5 -1
+-0.625 0.625 -1
+0.125 1 1
+1 0.625 -1
+1 -1 0.375
+1 -1 0.5
+0.125 -1 1
+0.125 -1 -1
+0.625 1 1
+-0.375 1 -0.625
+-0.875 1 1
+-1 1 1
+-0.125 0.625 -1
+-1 0.125 1
+-1 -1 0.5
+1 0.625 1
+-1 -1 -0.875
+-1 -1 -1
+-0.875 -1 -1
+-1 -0.625 -1
+-0.625 -1 -1
+1 1 -0.375
+-1 -1 0.375
+-1 0.875 1
+1 -1 0.625
+1 1 1
+-0.625 1 1
+1 -1 0.125
+3  0 2 1
+3  3 5 4
+3  6 8 7
+3  9 11 10
+3  12 14 13
+3  15 17 16
+3  18 20 19
+3  21 22 5
+3  23 25 24
+3  26 28 27
+3  29 31 30
+3  32 34 33
+3  35 34 32
+3  36 38 37
+3  39 12 40
+3  41 43 42
+3  44 46 45
+3  47 49 48
+3  50 52 51
+3  53 55 54
+3  56 36 57
+3  58 60 59
+3  61 63 62
+3  64 66 65
+3  67 69 68
+3  70 72 71
+3  73 75 74
+3  51 77 76
+3  78 80 79
+3  81 80 78
+3  25 79 24
+3  82 84 83
+3  85 87 86
+3  88 90 89
+3  91 93 92
+3  8 94 29
+3  95 97 96
+3  98 100 99
+3  101 103 102
+3  104 106 105
+3  107 109 108
+3  110 75 111
+3  112 114 113
+3  115 117 116
+3  118 102 119
+3  120 121 82
+3  122 124 123
+3  125 107 108
+3  126 128 127
+3  129 131 130
+3  132 134 133
+3  135 137 136
+3  138 140 139
+3  42 43 141
+3  142 143 109
+3  22 21 144
+3  22 144 145
+3  116 147 146
+3  60 149 148
+3  150 152 151
+3  153 155 154
+3  153 154 152
+3  156 129 157
+3  158 160 159
+3  161 163 162
+3  164 166 165
+3  167 169 168
+3  170 35 171
+3  172 31 173
+3  174 175 170
+3  176 39 177
+3  178 180 179
+3  181 65 182
+3  20 184 183
+3  185 187 186
+3  188 189 138
+3  38 190 97
+3  29 30 191
+3  192 194 193
+3  195 196 142
+3  197 191 198
+3  189 200 199
+3  201 203 202
+3  201 204 203
+3  180 6 7
+3  205 207 206
+3  80 209 208
+3  210 212 211
+3  213 215 214
+3  216 218 217
+3  219 220 68
+3  221 223 222
+3  224 129 225
+3  185 186 226
+3  227 229 228
+3  230 227 228
+3  231 233 232
+3  231 44 233
+3  199 200 234
+3  235 188 236
+3  237 239 238
+3  240 239 237
+3  241 243 242
+3  195 142 109
+3  191 30 198
+3  244 246 245
+3  247 249 248
+3  250 136 251
+3  252 254 253
+3  255 256 166
+3  257 75 73
+3  165 258 164
+3  67 68 220
+3  259 261 260
+3  262 264 263
+3  265 267 266
+3  268 270 269
+3  271 232 71
+3  272 274 273
+3  275 277 276
+3  278 280 279
+3  281 283 282
+3  284 62 285
+3  284 285 270
+3  83 148 82
+3  286 288 287
+3  289 286 287
+3  272 291 290
+3  292 168 293
+3  294 183 295
+3  296 298 297
+3  268 299 62
+3  300 302 301
+3  303 191 304
+3  305 306 23
+3  304 191 197
+3  307 309 308
+3  310 312 311
+3  313 315 314
+3  316 22 317
+3  5 22 316
+3  318 320 319
+3  321 50 322
+3  323 14 12
+3  324 323 12
+3  325 327 326
+3  328 269 329
+3  330 332 331
+3  333 334 2
+3  333 335 334
+3  336 338 337
+3  339 341 340
+3  263 266 262
+3  342 265 266
+3  235 236 244
+3  343 345 344
+3  346 348 347
+3  349 350 172
+3  74 351 73
+3  236 352 244
+3  353 355 354
+3  353 356 355
+3  357 359 358
+3  288 95 360
+3  361 35 170
+3  362 307 93
+3  362 354 307
+3  363 365 364
+3  79 80 208
+3  36 37 366
+3  367 368 136
+3  369 370 279
+3  371 293 168
+3  372 374 373
+3  375 377 376
+3  378 194 192
+3  378 379 194
+3  249 247 250
+3  380 381 13
+3  382 384 383
+3  385 240 386
+3  387 389 388
+3  218 390 217
+3  391 393 392
+3  335 394 379
+3  395 397 396
+3  325 326 398
+3  399 401 400
+3  402 404 403
+3  405 100 198
+3  250 406 249
+3  198 407 197
+3  408 132 356
+3  408 93 132
+3  409 380 410
+3  411 409 410
+3  412 114 324
+3  386 238 184
+3  339 414 413
+3  415 417 416
+3  418 420 419
+3  223 247 222
+3  421 246 342
+3  245 246 421
+3  20 386 184
+3  157 130 400
+3  422 424 423
+3  425 363 364
+3  95 96 426
+3  427 300 428
+3  224 77 131
+3  429 431 430
+3  231 232 432
+3  430 434 433
+3  435 436 190
+3  433 437 338
+3  438 440 439
+3  322 173 441
+3  118 101 102
+3  72 443 442
+3  444 353 354
+3  444 356 353
+3  445 447 446
+3  45 445 446
+3  448 450 449
+3  451 202 375
+3  452 453 430
+3  454 192 276
+3  455 43 41
+3  455 456 43
+3  457 86 87
+3  458 86 457
+3  459 331 460
+3  291 272 273
+3  461 463 462
+3  210 464 212
+3  465 177 40
+3  466 140 64
+3  467 114 112
+3  468 470 469
+3  471 473 472
+3  132 93 91
+3  202 377 375
+3  402 475 474
+3  183 184 99
+3  423 476 169
+3  66 477 255
+3  478 108 479
+3  480 382 481
+3  482 357 271
+3  483 313 484
+3  157 400 485
+3  486 181 487
+3  488 489 459
+3  69 491 490
+3  492 392 393
+3  478 125 108
+3  493 263 264
+3  233 446 232
+3  233 44 446
+3  268 269 494
+3  212 495 211
+3  200 496 234
+3  497 498 217
+3  499 501 500
+3  225 346 502
+3  334 335 379
+3  503 505 504
+3  427 478 300
+3  506 508 507
+3  509 511 510
+3  142 196 512
+3  513 358 514
+3  515 516 509
+3  517 519 518
+3  248 249 520
+3  152 154 521
+3  503 522 505
+3  325 523 327
+3  105 365 88
+3  235 524 188
+3  525 527 526
+3  528 375 14
+3  495 529 211
+3  14 297 13
+3  413 25 341
+3  530 531 26
+3  14 376 297
+3  532 296 533
+3  267 340 306
+3  379 495 212
+3  534 429 535
+3  167 168 292
+3  536 537 519
+3  538 213 438
+3  320 228 229
+3  539 540 85
+3  376 533 297
+3  222 541 348
+3  28 543 542
+3  544 337 545
+3  199 234 477
+3  150 1 155
+3  350 31 172
+3  352 42 141
+3  150 155 153
+3  150 153 152
+3  546 547 529
+3  277 549 548
+3  550 124 122
+3  551 306 305
+3  552 410 543
+3  549 483 159
+3  446 447 232
+3  553 501 499
+3  345 535 344
+3  104 105 500
+3  422 555 554
+3  556 557 299
+3  556 494 557
+3  558 560 559
+3  220 561 363
+3  360 95 426
+3  62 562 285
+3  296 532 520
+3  97 563 96
+3  564 481 383
+3  565 567 566
+3  565 384 567
+3  270 285 568
+3  243 507 63
+3  518 569 364
+3  99 184 98
+3  29 94 31
+3  385 386 570
+3  571 206 527
+3  572 574 573
+3  367 136 250
+3  354 575 309
+3  576 578 577
+3  311 312 579
+3  577 215 580
+3  581 386 20
+3  570 386 581
+3  79 582 24
+3  441 321 322
+3  512 196 583
+3  584 585 66
+3  391 586 393
+3  513 497 120
+3  134 523 325
+3  161 162 587
+3  91 92 588
+3  84 589 83
+3  590 310 591
+3  592 541 248
+3  127 128 593
+3  0 1 150
+3  126 122 128
+3  539 594 516
+3  539 85 594
+3  595 289 54
+3  246 596 342
+3  236 138 139
+3  597 599 598
+3  600 67 220
+3  257 111 75
+3  601 111 257
+3  601 602 111
+3  310 590 603
+3  604 18 17
+3  605 240 385
+3  550 122 126
+3  606 607 462
+3  569 425 364
+3  485 400 579
+3  608 306 551
+3  267 306 608
+3  491 609 490
+3  492 393 610
+3  391 611 586
+3  124 387 388
+3  511 613 612
+3  614 6 180
+3  614 180 178
+3  302 174 615
+3  616 124 388
+3  617 520 249
+3  618 290 619
+3  287 620 618
+3  287 360 620
+3  453 440 15
+3  326 327 621
+3  622 278 623
+3  624 626 625
+3  319 123 318
+3  474 627 531
+3  241 242 407
+3  472 473 44
+3  500 105 628
+3  245 235 244
+3  487 630 629
+3  479 174 302
+3  46 632 631
+3  633 427 428
+3  492 610 634
+3  130 399 400
+3  462 635 622
+3  568 21 3
+3  6 91 588
+3  415 275 417
+3  636 637 519
+3  171 637 636
+3  3 4 638
+3  639 628 89
+3  239 243 241
+3  46 631 640
+3  641 642 491
+3  214 570 604
+3  503 553 522
+3  467 528 114
+3  438 439 634
+3  643 644 60
+3  643 645 644
+3  21 5 3
+3  646 647 252
+3  188 260 189
+3  646 252 90
+3  21 568 648
+3  649 650 605
+3  612 52 308
+3  368 135 136
+3  55 619 651
+3  633 428 503
+3  59 653 652
+3  100 654 99
+3  655 524 235
+3  655 264 524
+3  656 69 657
+3  331 658 460
+3  331 332 658
+3  111 602 659
+3  89 252 253
+3  584 66 255
+3  497 217 660
+3  213 604 661
+3  662 663 204
+3  447 70 71
+3  584 663 662
+3  260 664 189
+3  260 261 664
+3  206 571 349
+3  96 558 426
+3  665 667 666
+3  644 668 432
+3  644 472 668
+3  646 669 647
+3  256 496 566
+3  308 52 50
+3  470 388 389
+3  595 54 49
+3  670 225 502
+3  670 224 225
+3  463 606 462
+3  259 118 261
+3  575 515 510
+3  574 671 573
+3  106 517 518
+3  672 465 673
+3  413 674 81
+3  675 676 650
+3  677 485 368
+3  678 680 679
+3  94 441 173
+3  305 611 681
+3  219 682 669
+3  516 683 509
+3  207 347 684
+3  484 685 659
+3  686 193 464
+3  281 557 494
+3  603 590 402
+3  687 145 506
+3  604 570 18
+3  688 689 151
+3  376 377 690
+3  339 413 341
+3  691 468 692
+3  693 146 125
+3  552 411 410
+3  377 545 690
+3  694 463 461
+3  684 347 348
+3  660 217 176
+3  363 561 365
+3  607 635 462
+3  400 401 579
+3  451 201 202
+3  451 204 201
+3  667 541 592
+3  695 150 151
+3  112 113 696
+3  54 289 53
+3  469 470 609
+3  697 693 125
+3  698 693 697
+3  100 405 654
+3  197 242 61
+3  160 548 159
+3  504 700 699
+3  532 533 701
+3  19 703 702
+3  611 391 681
+3  566 705 704
+3  706 142 512
+3  628 88 89
+3  663 166 164
+3  332 146 693
+3  470 468 388
+3  473 46 44
+3  135 310 603
+3  96 563 707
+3  213 661 438
+3  140 477 64
+3  560 339 340
+3  137 531 251
+3  689 625 695
+3  708 452 709
+3  710 534 535
+3  711 712 11
+3  591 554 713
+3  714 292 293
+3  715 28 26
+3  715 543 28
+3  64 477 66
+3  661 15 440
+3  424 717 716
+3  718 76 185
+3  160 454 548
+3  566 704 165
+3  437 720 719
+3  215 577 721
+3  276 277 548
+3  722 694 461
+3  545 723 690
+3  81 724 80
+3  725 253 598
+3  599 725 598
+3  506 507 676
+3  723 727 726
+3  709 452 480
+3  328 329 638
+3  728 111 209
+3  728 110 111
+3  65 585 182
+3  572 641 179
+3  330 331 729
+3  94 173 31
+3  143 361 170
+3  346 347 187
+3  442 443 730
+3  490 609 389
+3  77 418 131
+3  731 732 70
+3  447 731 70
+3  412 39 176
+3  733 178 523
+3  483 315 313
+3  190 436 563
+3  234 496 256
+3  288 37 95
+3  106 364 365
+3  488 734 714
+3  485 579 312
+3  80 724 110
+3  429 430 433
+3  68 490 682
+3  68 69 490
+3  139 140 466
+3  295 183 99
+3  8 29 303
+3  509 735 511
+3  347 207 205
+3  736 115 712
+3  113 114 412
+3  509 683 735
+3  209 685 208
+3  518 33 569
+3  616 691 547
+3  258 737 345
+3  258 704 737
+3  294 738 703
+3  179 641 656
+3  278 279 370
+3  202 343 544
+3  642 469 609
+3  10 717 554
+3  610 679 739
+3  740 578 576
+3  741 743 742
+3  230 597 593
+3  218 113 390
+3  744 290 618
+3  360 426 456
+3  595 745 730
+3  262 746 264
+3  503 428 553
+3  439 440 708
+3  190 563 97
+3  461 462 622
+3  525 747 738
+3  132 748 134
+3  132 91 748
+3  582 79 208
+3  53 618 619
+3  472 44 231
+3  198 100 407
+3  133 134 325
+3  432 232 271
+3  416 417 687
+3  432 271 149
+3  749 25 23
+3  341 25 749
+3  119 564 261
+3  742 743 750
+3  332 116 146
+3  461 622 623
+3  589 552 751
+3  635 699 622
+3  752 318 546
+3  647 550 126
+3  187 205 186
+3  340 559 560
+3  413 81 25
+3  592 520 532
+3  689 695 151
+3  753 754 496
+3  753 383 754
+3  19 20 183
+3  399 755 401
+3  572 573 641
+3  712 115 756
+3  410 380 757
+3  672 660 177
+3  482 72 442
+3  9 311 579
+3  178 758 523
+3  759 480 481
+3  745 442 730
+3  337 338 727
+3  305 23 611
+3  294 703 19
+3  590 713 404
+3  590 591 713
+3  101 760 681
+3  149 271 357
+3  761 127 593
+3  597 761 593
+3  610 678 679
+3  60 432 149
+3  622 699 278
+3  762 763 686
+3  210 763 762
+3  764 765 536
+3  764 766 765
+3  471 555 473
+3  148 83 59
+3  423 169 631
+3  767 304 299
+3  768 753 200
+3  768 564 753
+3  358 359 514
+3  424 769 476
+3  595 770 289
+3  195 395 196
+3  10 11 717
+3  98 184 238
+3  771 772 665
+3  15 16 434
+3  342 596 265
+3  47 498 513
+3  65 66 585
+3  482 442 359
+3  38 773 190
+3  156 677 223
+3  225 129 156
+3  573 282 642
+3  584 166 663
+3  31 350 30
+3  232 447 71
+3  296 297 533
+3  586 24 582
+3  454 276 548
+3  774 318 752
+3  355 775 575
+3  355 776 775
+3  419 420 583
+3  420 777 583
+3  778 16 437
+3  434 16 778
+3  583 196 396
+3  311 591 310
+3  426 558 779
+3  309 510 612
+3  780 529 547
+3  136 137 251
+3  615 171 537
+3  619 290 291
+3  143 170 175
+3  33 87 569
+3  781 120 82
+3  511 777 613
+3  644 432 60
+3  244 141 246
+3  548 549 159
+3  241 98 238
+3  598 254 127
+3  428 301 553
+3  420 613 777
+3  595 730 770
+3  681 391 103
+3  366 37 288
+3  651 291 486
+3  253 254 598
+3  615 174 171
+3  782 783 508
+3  648 783 782
+3  282 469 642
+3  134 733 523
+3  632 423 631
+3  784 667 665
+3  69 656 491
+3  491 642 609
+3  477 234 255
+3  283 468 469
+3  299 61 62
+3  785 168 169
+3  476 785 169
+3  374 372 414
+3  488 293 489
+3  242 197 407
+3  298 13 297
+3  435 190 773
+3  65 786 64
+3  466 64 786
+3  527 654 571
+3  787 788 695
+3  626 788 787
+3  539 516 515
+3  115 330 756
+3  789 780 547
+3  19 702 18
+3  646 219 669
+3  716 756 769
+3  637 33 519
+3  790 549 277
+3  791 498 47
+3  671 557 281
+3  549 790 315
+3  767 7 303
+3  791 216 498
+3  649 605 578
+3  644 645 472
+3  792 77 51
+3  226 172 173
+3  691 789 547
+3  519 33 518
+3  366 288 770
+3  496 565 566
+3  189 199 138
+3  122 123 319
+3  793 680 685
+3  208 685 680
+3  645 58 403
+3  616 388 691
+3  468 691 388
+3  214 721 385
+3  117 397 395
+3  219 68 682
+3  146 147 107
+3  668 231 432
+3  668 472 231
+3  344 336 337
+3  431 452 430
+3  793 313 794
+3  275 193 686
+3  515 509 510
+3  795 464 193
+3  795 212 464
+3  144 21 782
+3  144 782 145
+3  337 544 344
+3  458 361 683
+3  584 255 166
+3  796 396 397
+3  307 321 92
+3  308 309 612
+3  30 405 198
+3  248 541 222
+3  797 55 651
+3  798 705 710
+3  704 705 798
+3  525 738 294
+3  492 634 439
+3  107 147 195
+3  220 219 561
+3  799 0 800
+3  799 800 626
+3  801 719 720
+3  264 259 524
+3  701 723 771
+3  158 154 155
+3  755 736 711
+3  104 764 517
+3  315 576 314
+3  802 696 182
+3  714 449 435
+3  406 530 803
+3  261 564 804
+3  251 531 530
+3  751 552 543
+3  714 734 449
+3  790 415 740
+3  779 265 596
+3  161 805 163
+3  433 338 336
+3  383 384 565
+3  806 807 245
+3  806 263 807
+3  808 757 542
+3  543 808 542
+3  527 206 526
+3  174 170 171
+3  809 732 810
+3  45 809 810
+3  117 395 147
+3  384 534 710
+3  811 266 263
+3  342 266 811
+3  241 238 239
+3  384 710 705
+3  650 676 239
+3  617 803 298
+3  113 412 390
+3  723 726 771
+3  714 293 488
+3  575 510 309
+3  638 329 3
+3  3 329 568
+3  769 729 371
+3  786 181 486
+3  750 743 369
+3  518 364 106
+3  682 389 387
+3  321 441 92
+3  544 545 377
+3  130 812 399
+3  418 419 812
+3  236 188 138
+3  640 813 56
+3  258 345 343
+3  804 768 200
+3  804 564 768
+3  474 475 627
+3  589 751 653
+3  764 536 517
+3  301 302 766
+3  736 117 115
+3  465 40 381
+3  218 629 630
+3  758 179 656
+3  108 109 175
+3  207 526 206
+3  449 450 436
+3  672 673 84
+3  298 380 13
+3  411 465 381
+3  449 436 435
+3  482 71 72
+3  380 298 803
+3  196 395 396
+3  777 512 583
+3  460 693 606
+3  651 619 291
+3  7 8 303
+3  2 454 1
+3  607 427 633
+3  814 662 204
+3  422 423 632
+3  797 791 48
+3  624 625 815
+3  372 741 742
+3  128 122 319
+3  679 680 793
+3  299 304 61
+3  681 103 101
+3  466 273 274
+3  143 175 109
+3  707 374 560
+3  186 205 349
+3  52 612 613
+3  647 254 252
+3  92 441 588
+3  736 712 711
+3  754 565 496
+3  754 383 565
+3  456 426 779
+3  744 41 290
+3  360 456 744
+3  816 628 639
+3  499 816 639
+3  726 772 771
+3  490 389 682
+3  546 616 547
+3  275 276 193
+3  594 86 516
+3  594 85 86
+3  85 817 87
+3  818 530 406
+3  818 251 530
+3  264 746 259
+3  119 759 481
+3  819 250 247
+3  819 367 250
+3  522 553 499
+3  820 813 167
+3  821 710 535
+3  345 821 535
+3  813 820 56
+3  820 36 56
+3  14 375 376
+3  470 389 609
+3  39 412 324
+3  36 366 57
+3  57 366 443
+3  357 482 359
+3  283 692 468
+3  248 222 247
+3  695 788 822
+3  800 822 788
+3  634 538 438
+3  629 218 216
+3  772 747 665
+3  525 526 747
+3  650 239 240
+3  340 341 306
+3  823 13 381
+3  40 823 381
+3  807 493 245
+3  807 263 493
+3  824 375 528
+3  467 824 528
+3  536 519 517
+3  219 646 90
+3  19 183 294
+3  694 459 463
+3  489 293 371
+3  330 116 332
+3  463 459 460
+3  393 586 678
+3  641 573 642
+3  722 623 825
+3  129 130 157
+3  826 625 689
+3  815 625 826
+3  255 234 256
+3  586 582 678
+3  189 804 200
+3  254 647 126
+3  606 698 607
+3  318 123 546
+3  437 16 720
+3  50 718 322
+3  661 17 15
+3  760 551 305
+3  760 305 681
+3  779 558 559
+3  461 623 722
+3  307 354 309
+3  504 699 635
+3  584 662 585
+3  663 164 203
+3  814 204 451
+3  199 140 138
+3  15 434 453
+3  338 719 727
+3  706 512 735
+3  767 299 557
+3  281 827 283
+3  709 480 759
+3  707 560 558
+3  385 570 214
+3  197 61 304
+3  739 538 634
+3  338 437 719
+3  633 503 504
+3  453 434 430
+3  270 568 329
+3  270 329 269
+3  684 667 784
+3  256 566 165
+3  99 654 295
+3  772 726 801
+3  712 716 11
+3  180 572 179
+3  262 760 746
+3  226 718 185
+3  828 366 770
+3  730 828 770
+3  119 481 564
+3  169 167 813
+3  71 482 271
+3  404 645 403
+3  404 471 645
+3  318 774 320
+3  624 815 774
+3  774 815 320
+3  624 774 394
+3  335 624 394
+3  261 118 119
+3  76 502 185
+3  746 760 101
+3  829 7 574
+3  829 574 572
+3  672 84 121
+3  830 76 718
+3  830 51 76
+3  113 218 696
+3  620 744 618
+3  620 360 744
+3  592 248 520
+3  307 92 93
+3  132 133 356
+3  831 813 640
+3  631 831 640
+3  74 369 279
+3  192 193 276
+3  379 394 495
+3  394 752 495
+3  334 192 2
+3  672 177 465
+3  256 165 166
+3  382 534 384
+3  700 161 587
+3  792 420 418
+3  51 52 792
+3  52 613 792
+3  226 186 172
+3  792 613 420
+3  349 172 186
+3  274 42 352
+3  244 352 141
+3  202 544 377
+3  769 371 476
+3  423 424 476
+3  723 701 690
+3  16 702 720
+3  85 832 817
+3  832 833 817
+3  834 835 709
+3  484 313 793
+3  686 763 317
+3  786 486 273
+3  70 836 72
+3  70 57 836
+3  833 621 600
+3  61 242 63
+3  139 466 274
+3  735 512 777
+3  466 786 273
+3  474 531 137
+3  552 589 673
+3  837 308 50
+3  837 50 321
+3  380 803 757
+3  459 694 488
+3  567 705 566
+3  567 384 705
+3  348 541 684
+3  207 784 526
+3  413 414 674
+3  303 304 767
+3  616 123 124
+3  546 123 616
+3  427 698 478
+3  606 693 698
+3  747 526 784
+3  629 216 791
+3  121 120 497
+3  734 448 449
+3  747 784 665
+3  358 513 120
+3  58 643 60
+3  58 645 643
+3  523 758 327
+3  588 94 8
+3  245 421 806
+3  27 542 757
+3  27 28 542
+3  295 654 527
+3  792 418 77
+3  322 718 226
+3  322 226 173
+3  73 351 838
+3  587 838 351
+3  669 124 550
+3  647 669 550
+3  774 752 394
+3  645 471 472
+3  368 312 135
+3  121 497 660
+3  672 121 660
+3  147 395 195
+3  485 312 368
+3  2 192 454
+3  393 678 610
+3  431 534 382
+3  480 431 382
+3  480 452 431
+3  611 24 586
+3  611 23 24
+3  560 374 339
+3  374 414 339
+3  399 796 755
+3  605 721 578
+3  328 638 789
+3  675 687 506
+3  506 676 675
+3  649 675 650
+3  578 721 577
+3  98 241 407
+3  649 578 740
+3  789 691 692
+3  223 677 367
+3  677 368 367
+3  561 219 90
+3  633 504 635
+3  561 90 88
+3  365 561 88
+3  11 9 711
+3  521 73 838
+3  722 825 448
+3  722 448 734
+3  460 606 463
+3  115 116 330
+3  559 340 267
+3  213 214 604
+3  588 8 6
+3  734 488 694
+3  712 756 716
+3  722 734 694
+3  551 760 262
+3  266 551 262
+3  218 630 696
+3  182 696 630
+3  487 181 630
+3  835 492 439
+3  181 182 630
+3  406 803 617
+3  617 249 406
+3  493 655 235
+3  493 264 655
+3  355 575 354
+3  356 133 776
+3  776 133 398
+3  133 325 398
+3  356 776 355
+3  486 291 273
+3  530 757 803
+3  627 26 531
+3  627 715 26
+3  751 543 715
+3  165 704 258
+3  117 147 116
+3  534 431 429
+3  535 336 344
+3  265 559 267
+3  600 621 67
+3  673 411 552
+3  48 54 55
+3  274 272 42
+3  559 265 779
+3  585 662 802
+3  814 802 662
+3  545 337 727
+3  673 465 411
+3  209 111 659
+3  658 693 460
+3  658 332 693
+3  151 839 688
+3  163 805 597
+3  604 17 661
+3  254 126 127
+3  152 839 151
+3  152 521 839
+3  840 18 702
+3  17 18 840
+3  799 2 0
+3  335 799 626
+3  624 335 626
+3  841 43 596
+3  141 43 841
+3  359 442 745
+3  125 146 107
+3  698 427 607
+3  842 843 475
+3  842 653 843
+3  505 522 805
+3  105 88 628
+3  379 212 194
+3  664 804 189
+3  664 261 804
+3  680 582 208
+3  545 727 723
+3  724 674 750
+3  674 724 81
+3  814 451 467
+3  422 632 555
+3  632 473 555
+3  39 324 12
+3  114 528 324
+3  844 223 221
+3  844 156 223
+3  167 773 820
+3  473 632 46
+3  571 654 405
+3  772 801 738
+3  703 738 801
+3  634 610 739
+3  747 772 738
+3  178 179 758
+3  331 459 489
+3  769 756 729
+3  756 330 729
+3  845 489 371
+3  729 845 371
+3  680 678 582
+3  414 372 742
+3  242 243 63
+3  755 796 397
+3  414 742 674
+3  674 742 750
+3  736 397 117
+3  755 397 736
+3  677 157 485
+3  295 527 525
+3  685 209 659
+3  607 633 635
+3  107 195 109
+3  703 801 720
+3  438 661 440
+3  702 703 720
+3  521 838 839
+3  158 155 160
+3  155 1 160
+3  454 160 1
+3  758 656 657
+3  327 758 657
+3  588 441 94
+3  721 214 215
+3  538 215 213
+3  525 294 295
+3  733 91 6
+3  59 60 148
+3  590 404 402
+3  76 77 224
+3  225 221 346
+3  348 346 221
+3  677 156 157
+3  348 221 222
+3  739 580 538
+3  538 580 215
+3  682 387 669
+3  577 580 314
+3  780 4 211
+3  4 210 211
+3  275 686 417
+3  574 7 767
+3  671 282 573
+3  281 282 671
+3  401 711 9
+3  755 711 401
+3  217 498 216
+3  791 47 48
+3  351 74 279
+3  549 315 483
+3  791 797 629
+3  629 797 487
+3  741 372 450
+3  448 741 450
+3  448 825 741
+3  743 741 825
+3  721 605 385
+3  419 396 796
+3  650 240 605
+3  475 843 627
+3  35 637 171
+3  846 433 336
+3  429 433 846
+3  653 751 843
+3  471 713 555
+3  404 713 471
+3  713 554 555
+3  514 47 513
+3  514 49 47
+3  716 769 424
+3  717 11 716
+3  554 717 422
+3  282 283 469
+3  84 673 589
+3  521 257 73
+3  659 847 484
+3  483 484 847
+3  847 159 483
+3  158 159 847
+3  145 687 317
+3  436 450 373
+3  450 372 373
+3  317 22 145
+3  417 317 687
+3  82 121 84
+3  135 312 310
+3  135 603 137
+3  603 474 137
+3  686 317 417
+3  5 763 210
+3  4 5 210
+3  149 781 148
+3  149 357 781
+3  781 358 120
+3  148 781 82
+3  797 48 55
+3  373 707 563
+3  436 373 563
+3  33 34 87
+3  62 63 562
+3  145 508 506
+3  568 285 648
+3  848 849 562
+3  508 849 848
+3  21 648 782
+3  145 782 508
+3  507 562 63
+3  253 639 89
+3  724 850 110
+3  851 767 557
+3  851 557 671
+3  599 499 639
+3  522 499 599
+3  522 599 805
+3  805 599 597
+3  850 75 110
+3  850 74 75
+3  850 369 74
+3  373 374 707
+3  343 344 544
+3  602 158 847
+3  602 847 659
+3  204 663 203
+3  202 203 343
+3  683 706 735
+3  154 158 602
+3  154 257 521
+3  415 416 740
+3  692 328 789
+3  649 740 416
+3  649 416 675
+3  416 687 675
+3  541 667 684
+3  349 571 350
+3  789 638 780
+3  638 4 780
+3  405 350 571
+3  350 405 30
+3  142 706 143
+3  743 825 370
+3  784 207 684
+3  600 220 363
+3  452 708 453
+3  802 182 585
+3  29 191 303
+3  656 641 491
+3  453 708 440
+3  801 726 719
+3  835 708 709
+3  835 439 708
+3  576 577 314
+3  726 727 719
+3  315 790 576
+3  740 576 790
+3  739 794 580
+3  177 660 176
+3  177 39 40
+3  162 839 838
+3  587 162 838
+3  163 688 162
+3  679 793 794
+3  679 794 739
+3  580 794 314
+3  313 314 794
+3  617 296 520
+3  617 298 296
+3  412 176 390
+3  834 392 835
+3  392 492 835
+3  229 593 128
+3  391 392 103
+3  102 103 834
+3  103 392 834
+3  65 181 786
+3  53 619 55
+3  707 558 96
+3  41 272 290
+3  41 42 272
+3  780 211 529
+3  752 546 529
+3  752 529 495
+3  790 277 415
+3  415 277 275
+3  511 735 777
+3  154 601 257
+3  154 602 601
+3  327 657 621
+3  163 597 230
+3  163 230 688
+3  230 689 688
+3  230 228 689
+3  320 815 228
+3  319 229 128
+3  320 229 319
+3  540 832 85
+3  852 515 575
+3  776 398 540
+3  540 398 832
+3  776 540 852
+3  836 443 72
+3  836 57 443
+3  493 235 245
+3  532 666 592
+3  666 532 701
+3  131 418 812
+3  311 9 10
+3  591 311 10
+3  513 498 497
+3  579 401 9
+3  666 771 665
+3  666 667 592
+3  224 131 129
+3  745 595 49
+3  514 359 745
+3  102 759 119
+3  102 834 759
+3  834 709 759
+3  140 199 477
+3  853 343 203
+3  164 853 203
+3  46 640 45
+3  640 56 732
+3  854 615 537
+3  536 854 537
+3  855 856 854
+3  765 855 854
+3  765 856 855
+3  44 45 446
+3  494 827 281
+3  494 269 827
+3  328 827 269
+3  280 351 279
+3  283 827 692
+3  827 328 692
+3  280 587 351
+3  706 361 143
+3  690 701 533
+3  699 280 278
+3  376 690 533
+3  699 700 280
+3  700 587 280
+3  817 425 569
+3  833 600 425
+3  833 425 817
+3  361 706 683
+3  764 104 857
+3  501 764 857
+3  820 38 36
+3  820 773 38
+3  857 104 500
+3  501 857 500
+3  37 97 95
+3  48 49 54
+3  744 455 41
+3  744 456 455
+3  657 69 67
+3  657 67 621
+3  583 396 419
+3  289 287 53
+3  53 287 618
+3  288 360 287
+3  167 292 773
+3  478 858 300
+3  478 479 858
+3  732 57 70
+3  732 56 57
+3  858 302 300
+3  858 479 302
+3  402 403 475
+3  475 403 652
+3  524 859 188
+3  524 259 859
+3  361 458 35
+3  481 382 383
+3  859 260 188
+3  859 259 260
+3  753 564 383
+3  200 753 496
+3  458 34 35
+3  292 714 435
+3  292 435 773
+3  86 458 516
+3  852 860 515
+3  852 540 860
+3  422 717 424
+3  817 569 87
+3  832 398 326
+3  832 326 833
+3  326 621 833
+3  352 139 274
+3  106 365 105
+3  187 347 205
+3  90 252 89
+3  130 131 812
+3  502 346 187
+3  860 539 515
+3  860 540 539
+3  83 589 653
+3  603 402 474
+3  59 83 653
+3  516 458 683
+3  479 108 175
+3  479 175 174
+3  300 301 428
+3  553 301 501
+3  104 517 106
+3  301 766 501
+3  501 766 764
+3  530 27 757
+3  530 26 27
+3  161 505 805
+3  700 505 161
+3  504 505 700
+3  502 187 185
+3  217 390 176
+3  701 771 666
+3  685 484 793
+3  750 369 850
+3  724 750 850
+3  651 486 487
+3  797 651 487
+3  781 357 358
+3  591 10 554
+3  100 98 407
+3  812 419 796
+3  205 206 349
+3  399 812 796
+3  779 596 456
+3  456 596 43
+3  802 112 696
+3  814 112 802
+3  814 467 112
+3  425 600 363
+3  743 370 369
+3  623 370 825
+3  623 278 370
+3  669 387 124
+3  688 839 162
+3  510 511 612
+3  352 236 139
+3  746 118 259
+3  746 101 118
+3  38 97 37
+3  514 745 49
+3  856 302 615
+3  766 302 856
+3  640 732 809
+3  45 640 809
+3  258 343 853
+3  164 258 853
+3  268 62 284
+3  268 284 270
+3  285 562 861
+3  285 861 648
+3  507 848 562
+3  508 848 507
+3  861 562 849
+3  861 849 783
+3  783 849 508
+3  861 783 648
+3  537 636 519
+3  171 636 537
+3  535 846 336
+3  429 846 535
+3  228 826 689
+3  815 826 228
+3  748 733 134
+3  748 91 733
+3  626 800 788
+3  799 333 2
+3  799 335 333
+3  246 841 596
+3  141 841 246
+3  381 380 409
+3  411 381 409
+3  715 627 862
+3  751 715 862
+3  862 627 843
+3  751 862 843
+3  625 787 695
+3  626 787 625
+3  345 737 821
+3  695 822 150
+3  0 150 822
+3  775 852 575
+3  775 776 852
+3  421 342 811
+3  806 811 263
+3  421 811 806
+3  250 818 406
+3  250 251 818
+3  266 608 551
+3  267 608 266
+3  410 757 808
+3  543 410 808
+3  307 308 837
+3  307 837 321
+3  863 798 821
+3  737 863 821
+3  737 798 863
+3  737 704 798
+3  821 798 710
+3  800 0 822
+3  864 507 243
+3  676 507 864
+3  239 864 243
+3  676 864 239
+3  169 813 831
+3  631 169 831
+3  865 59 652
+3  403 865 652
+3  58 59 865
+3  403 58 865
+3  362 93 408
+3  444 408 356
+3  444 362 408
+3  444 354 362
+3  45 810 445
+3  810 732 731
+3  445 810 731
+3  445 731 447
+3  536 765 854
+3  856 615 854
+3  765 766 856
+3  194 795 193
+3  194 212 795
+3  464 762 686
+3  210 762 464
+3  34 457 87
+3  458 457 34
+3  639 253 725
+3  599 639 725
+3  574 767 851
+3  574 851 671
+3  76 670 502
+3  76 224 670
+3  180 7 829
+3  180 829 572
+3  18 581 20
+3  570 581 18
+3  16 840 702
+3  17 840 16
+3  598 127 761
+3  597 598 761
+3  80 728 209
+3  80 110 728
+3  331 489 845
+3  729 331 845
+3  451 375 824
+3  467 451 824
+3  770 288 286
+3  289 770 286
+3  25 78 79
+3  81 78 25
+3  528 14 323
+3  324 528 323
+3  225 844 221
+3  225 156 844
+3  500 628 816
+3  499 500 816
+3  652 842 475
+3  652 653 842
+3  593 229 227
+3  230 593 227
+3  268 556 299
+3  268 494 556
+3  763 316 317
+3  5 316 763
+3  637 32 33
+3  35 32 637
+3  733 6 614
+3  733 614 178
+3  443 366 828
+3  730 443 828
+3  223 819 247
+3  223 367 819
+3  386 237 238
+3  240 237 386
+3  371 168 785
+3  476 371 785
+3  334 378 192
+3  334 379 378
+3  478 697 125
+3  698 697 478
+3  433 778 437
+3  434 778 433
+3  50 830 718
+3  50 51 830
+3  306 749 23
+3  341 749 306
+3  12 13 823
+3  40 12 823
+
diff -pruN 5.0.3-1/examples/Surface_mesh_simplification/data/cube.off 5.1-1/examples/Surface_mesh_simplification/data/cube.off
--- 5.0.3-1/examples/Surface_mesh_simplification/data/cube.off	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_simplification/data/cube.off	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,24 @@
+OFF
+8 12 0
+
+-0.274878 -0.274878 -0.274878
+-0.274878 0.274878 -0.274878
+0.274878 0.274878 -0.274878
+0.274878 -0.274878 -0.274878
+-0.274878 -0.274878 0.274878
+-0.274878 0.274878 0.274878
+0.274878 0.274878 0.274878
+0.274878 -0.274878 0.274878
+3  0 1 3
+3  3 1 2
+3  0 4 1
+3  1 4 5
+3  3 2 7
+3  7 2 6
+3  4 0 3
+3  7 4 3
+3  6 4 7
+3  6 5 4
+3  1 5 6
+3  2 1 6
+
diff -pruN 5.0.3-1/examples/Surface_mesh_simplification/data/mesh_with_border.off 5.1-1/examples/Surface_mesh_simplification/data/mesh_with_border.off
--- 5.0.3-1/examples/Surface_mesh_simplification/data/mesh_with_border.off	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_simplification/data/mesh_with_border.off	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,1566 @@
+OFF
+548 1014 0
+
+92.0896999051 77.41989209583 -1.916832684148
+86.39100962126 72.78664496906 15.78278094865
+77.09376525879 89.87973022461 14.55043506622
+90.98168945312 96.21558380127 3.65913438797
+82.71570084509 81.86220870549 16.44397321995
+92.31691908737 86.77964878854 1.043011456447
+89.37290527712 74.91208629563 6.905433125042
+84.03774261475 93.04766845703 9.104772567749
+87.91614428888 83.98367629501 8.885449941462
+92.08308562722 91.51009411435 2.561394143404
+79.93436140773 85.95867809642 15.71871673187
+92.16810072909 82.08661851225 -0.4813870638146
+84.95054383138 77.42281772836 16.37560573549
+84.22521839805 85.24324921259 12.54660366583
+87.28833145411 78.56473572586 11.06770318225
+88.53858617073 89.67622996838 7.078736038751
+90.21472024103 80.41696412654 4.846493577097
+87.81428016343 73.7864628996 11.33105836088
+90.80739519912 76.15358418737 2.490105437257
+90.23304320467 85.72684363719 5.157361043466
+80.56578826904 91.463722229 11.8276386261
+87.50969696045 94.63160705566 6.381914138794
+85.28558449034 88.85493917666 10.09354883143
+85.27758279004 81.37161616953 13.2681149885
+82.42035268114 88.37159009099 12.28321454594
+88.64910272783 80.44739317924 8.19157083558
+91.34705696923 83.71941907076 2.518356602206
+86.54332632431 76.21153148326 13.50411389672
+91.67848926288 93.87171420986 3.215234177501
+92.10816413641 79.75050243058 -1.206542332568
+92.29622765622 89.13884414744 1.826720300829
+89.70542995812 92.32964625679 5.425094029631
+90.59106423297 89.29868580891 4.596301969103
+91.17972407486 79.36567116545 1.91073903372
+86.33903036171 91.18877364247 8.487076777961
+87.52474663172 87.12023386306 8.676494963161
+78.49607673255 87.91733917969 15.15035040429
+86.99901132535 81.62634788796 10.68441922028
+88.58945681826 74.33088989454 9.115347731003
+88.62578378798 77.40042441075 8.552444264328
+89.49626856147 82.8803041145 6.515565583687
+90.11337785606 75.52304884182 4.696506235514
+82.30175018311 92.25574493408 10.46619224548
+86.19309520102 85.47015639124 10.53423349659
+83.62519737065 83.26930177161 14.30213272629
+82.0111827497 86.17046138772 13.90531750866
+85.62502595062 78.76016915029 14.06135119224
+78.82978057861 90.67166137695 13.1890039444
+89.45795733185 78.72045615552 6.562115009714
+85.6876237692 83.36697320818 11.86120221103
+90.44759176655 78.12728045561 3.939338198359
+80.48132290292 88.08344582991 13.73552924292
+81.36934673359 83.95779586913 16.1769257545
+85.75871853097 75.11699965987 16.11420674811
+83.88932229025 79.65919077394 16.47804590138
+92.26242604584 84.4270568131 0.271220829177
+87.82876868723 76.19362041729 10.58641505821
+87.06673968633 73.27518730634 13.55175677477
+91.45894636523 76.78940059593 0.2867129567336
+89.1360658561 87.2730913458 6.776971477474
+91.27271853367 87.17795258214 3.336373635784
+87.86365825812 92.32249801315 6.989370325119
+89.24565124512 95.42362213135 5.020479202271
+85.77369689941 93.83967590332 7.74334526062
+84.1530047653 90.8891150393 10.01020383373
+84.7473737152 87.01795663413 11.27019134425
+90.49631836036 82.69742911034 4.431726418335
+89.14774062234 85.0408944843 6.999973531134
+91.46289377915 81.61758663607 1.630732294048
+82.3132325122 90.30474632652 11.36096779886
+86.90111626841 89.15160207096 8.72203909917
+84.11950356501 81.16699817827 15.0140687604
+86.28939352036 79.79002703214 12.40403490101
+80.46573968642 89.74182843265 12.71429924485
+89.60865143194 76.76812793407 6.178829509292
+90.89270834008 91.26374173479 4.176033399443
+86.80521922435 83.70802005279 10.38215960895
+89.48178847126 80.95757614811 6.534276165285
+88.11169455465 79.04731345898 9.395346300457
+88.583493635 82.36236771915 8.124783175307
+90.84324437988 81.26551998519 3.330565985771
+83.87247641686 88.5955192875 11.20891472132
+83.19459245527 86.7501647852 12.6283671037
+90.40385047626 87.51383281538 4.886414759175
+86.16951796704 87.23207527998 9.953337633655
+91.60399930071 85.59586026518 2.336331753486
+90.75668901041 84.4323814451 4.044852809319
+87.29901191592 75.34268635555 12.09818461047
+86.57400034054 77.89258327773 12.67355564934
+85.79080099281 76.54570082726 15.07525566634
+91.77179896274 80.57408744558 0.281974535061
+91.5475556147 89.95641279222 3.1931760263
+90.3442663496 93.92285130091 4.585233593477
+86.45794665313 74.70284569722 14.49144487954
+79.05721494196 89.10127736985 13.9682290256
+92.09571263016 78.58472307811 -1.56269428567
+91.61324652266 78.65370278208 0.2413144683142
+88.95158085097 93.87160551314 5.76487891664
+88.93172774873 91.16524184997 6.380833459336
+83.1217686748 84.84727135728 13.80594048831
+89.60708732727 88.78765137181 5.981576801907
+90.92546945959 77.800373802 2.393472143492
+89.88360646754 84.27551327628 5.789320637377
+87.82352645612 80.66325786631 9.616310711501
+80.65753881345 84.96774500172 15.96810193465
+81.91399561614 84.87065850033 14.91313114612
+90.704329777 79.64984094159 3.428700832615
+84.55592036845 83.70275449477 13.0115763795
+92.21273042203 83.25630400946 -0.1087730360537
+88.21238360604 88.22000875015 7.707421885366
+88.15959122892 85.57676575574 8.297069621225
+88.81394466763 78.89692552288 7.991139144449
+87.59941965207 82.59759096357 9.584046736566
+86.06838273588 81.83162256321 11.96328549717
+87.03192126414 80.10893594721 11.0553375274
+88.46537703634 75.92217371895 9.137222873303
+87.94183853539 77.63062238945 10.0069676505
+84.73188438726 79.65427763894 15.09106477496
+89.99572259739 90.6013158789 5.326148605559
+89.08109552443 76.28510581649 7.571875818205
+87.45358755135 90.32287833068 7.925755467503
+85.80653793821 90.02205488241 9.287945386385
+84.90573883057 93.44362640381 8.424060821533
+85.02462957381 91.82100314902 9.080210762303
+82.53828964184 83.53532004812 15.31098692782
+87.24565314739 85.21590788642 9.462906429098
+88.76907792079 83.77269954437 7.686026722444
+87.26020862849 77.16918094688 11.56186126234
+79.21183662051 86.93976318432 15.44173627227
+80.6772357635 86.72750384054 14.55134866499
+89.96968019319 81.91329385452 5.544516060987
+91.04107866369 92.74354582432 3.959510149799
+89.93672918584 77.98593262991 5.347731087816
+91.02614651348 85.78683581428 3.633030004003
+85.13548572186 85.65452792654 11.50140805416
+86.58175312226 92.53969079587 7.807716427241
+91.73194481166 83.03368102701 1.268949655066
+91.6054686603 88.52647150388 2.928124991176
+82.0440222159 87.34001414513 13.1416907498
+85.52842652024 80.03896830107 13.54193677916
+85.36404016149 77.88755226405 15.12055517891
+86.0765084065 77.46013649534 13.89021585768
+90.46661968935 75.83685629421 3.592188671691
+90.19681065692 76.89784761499 4.538397099543
+84.56138644799 82.48691782071 13.65137294073
+83.30403327714 82.36071585563 15.29489338667
+86.64169311523 94.23564147949 7.062632083893
+89.15075042396 77.62070306986 7.311016382239
+91.77578816761 77.10568331543 -0.8146003184522
+92.31458355115 87.95769882393 1.431302885641
+84.27631718922 89.64398620348 10.47724464253
+91.16950185257 82.39458802729 2.677383847529
+86.87516040998 74.96862991509 13.27991264815
+89.18314931403 79.84626795266 7.164286304372
+84.44360901566 78.54784372864 16.4455387666
+85.99928322555 84.35958928423 11.12543897065
+90.96268806848 88.23215442693 3.99433103563
+91.79917398165 86.86770228776 2.211270347125
+91.35660160611 95.04584327368 3.454739655752
+90.11364746094 95.81958770752 4.33976650238
+91.8114226106 84.52104497723 1.537204226449
+89.92268062963 79.29372508096 5.47162553997
+87.80022427991 74.99940383723 10.98461017787
+88.98391421327 74.61712962187 8.011383213137
+88.41189669657 86.76113359695 7.762849859456
+89.83990585597 86.78305575769 5.804147791076
+86.47708307705 82.70982487065 11.10320850079
+85.05990301504 84.41623279402 12.11437941264
+89.32975770952 89.98303370895 6.172890167703
+87.43364445809 73.52682079937 12.44056066412
+81.26149028386 88.68493750348 12.8445643021
+79.59530025292 87.7552167202 14.55929982062
+81.56095704279 89.65865012342 12.12054795257
+83.74650335188 87.5173532649 11.81482990264
+82.05580562551 82.9262281906 16.33684179161
+77.79110462112 88.89988096489 14.85615961682
+85.38491825728 76.27397332034 16.26002634304
+83.30952273933 80.76277415836 16.47482934558
+86.09057899059 73.95418022421 15.9528063478
+92.22202721814 90.32419016346 2.207048185808
+92.13015228425 80.91751070159 -0.8477875752077
+91.91128748784 92.69412778407 2.905822503365
+92.30352960777 85.60185389102 0.6571861560139
+88.80397882864 92.35845957816 6.230165128969
+87.56350554293 93.48546833814 6.794415435958
+86.77515477948 87.99073532884 9.163124693164
+91.2921995218 77.76012886914 1.141073203928
+87.24563285669 91.51266122799 7.717013849969
+91.05693732674 80.38765116816 2.536987922097
+88.09868991582 81.66739940773 8.994784785804
+88.20079182886 74.0545900789 10.22315122432
+91.13838295799 76.47160421784 1.38837785964
+89.74903229875 75.21462640485 5.800678292534
+86.7248444178 73.03032641041 14.66755655857
+91.42198188351 80.44639618037 1.409354415978
+86.70248244291 86.20631702261 9.772311187611
+85.32357668702 82.51604988631 12.67539856556
+83.34415171148 90.3563044528 10.73296744307
+83.16972351074 92.65168762207 9.785503387451
+83.16370643034 91.45261184562 10.33452960078
+88.3776473999 95.02765655518 5.701191902161
+79.6978225708 91.06776428223 12.50832748413
+81.43376159668 91.85966491699 11.14694881439
+77.96181488037 90.27571105957 13.8696937561
+82.88320612077 89.23701510152 11.54197182646
+90.59709590487 77.0431090884 3.326233951924
+90.4385053569 81.50740377165 4.417289906702
+90.92399955002 83.20459553336 3.474223453509
+90.04153909139 83.12197296545 5.464420913909
+84.84455473976 80.53727723718 14.34532209905
+88.97115981828 81.39918470952 7.526398209826
+88.19184958435 90.76480154662 7.16373228865
+88.95207671701 88.40527714282 6.846825372554
+86.85188444752 76.78765884766 12.56411815235
+91.82868788219 82.06718063471 0.6263483943344
+87.83397570175 89.26724425275 7.860215245696
+85.63889122331 86.49196601989 10.70444370687
+84.289256929 86.25422337269 12.00150866214
+85.28132021495 87.75990875058 10.52113336767
+87.61493912667 79.70302658741 10.17261837774
+89.34878250085 83.96799192039 6.745304265489
+89.71060304788 85.34737396029 6.077644732881
+89.04670719873 86.17050776147 7.033811940758
+83.33616210627 85.71360239737 13.08930299415
+80.90981870091 85.73270169067 15.12615535105
+91.21497714171 84.80822967201 3.070330870078
+90.64792720154 86.4992805173 4.432689133307
+90.32498894608 79.39071047162 4.441071424401
+86.59113065075 81.0079681309 11.48087340359
+88.20529733344 75.14799817208 9.928778947458
+91.46932714668 91.82413160531 3.426693900819
+84.05625152588 92.04026031494 9.551273345947
+81.34168250145 90.67680250169 11.74296930792
+91.55259019351 79.61359965956 0.734537667934
+86.14928903969 88.82644504097 9.435758125875
+86.63480151744 90.17649516787 8.626415734013
+85.61112519056 92.54511608244 8.415443520635
+90.77235413367 78.64450546717 3.044471825873
+86.03015612476 80.84246483408 12.39859916442
+84.09084387379 80.31011079893 15.65381057678
+87.42963074907 86.1429197771 9.024362828801
+84.58304230312 81.54684584424 14.13916022685
+82.71823885608 84.27191904384 14.58204533128
+86.14318337822 78.875724971 13.05390603302
+87.26466369629 79.24705505371 11.27718162537
+87.46308063449 88.31600007416 8.437690516399
+81.19461313203 87.2774180299 13.78930881295
+90.33868384664 83.96273511603 4.894040729647
+86.24349253624 75.76941990672 14.45146228254
+89.62070466303 79.96751622986 6.215618617518
+89.86045031553 87.81767110116 5.714444063781
+90.32493653663 92.86181087998 4.742895505028
+91.83660492574 79.70232495475 -0.2327343256045
+87.5598870147 81.60536240291 9.845685702366
+90.24277677565 91.55416058234 4.951816512248
+90.62251838342 90.33539527205 4.550076062657
+88.3684628344 84.57954172437 8.178119845641
+84.91706095676 90.18792368357 9.834589545214
+83.9286208171 84.51052198464 13.22924134799
+89.42617545693 75.90524494088 6.691736706803
+89.91402634483 76.30110923329 5.324186755154
+89.57647684262 77.76935141695 6.265239627438
+90.26302274 88.48759436779 5.08800413955
+85.56081184425 90.97427099583 9.095988527362
+80.00626334278 89.03147537546 13.41898396371
+79.37286329558 89.82291469539 13.32549985972
+88.14755221554 76.67312076792 9.744962597196
+89.65531497262 93.32206911079 5.323482453556
+91.22225608436 89.12207284975 3.629908520633
+83.96795217196 82.30680594897 14.49836357631
+83.45877795223 81.4494251123 15.73631635087
+89.94620514167 89.59487371469 5.467634763391
+82.29238891602 91.29810333252 10.90618038177
+90.811639497 82.22302301049 3.582510020635
+80.52007293701 90.59206390381 12.24966716766
+88.29109077723 78.09038216257 9.190419558197
+87.71177444011 78.74996062053 10.22093064536
+88.46730306671 79.57337952733 8.628829838468
+88.33761245464 83.26134698128 8.394543381393
+89.36645737065 81.92077744538 6.769106527411
+89.89907582446 80.92451760593 5.635455680689
+89.00589702596 82.86301063417 7.377557640286
+90.59027668966 94.87526374646 4.171108393016
+87.42512737492 76.17020838692 11.52442944207
+91.02426936631 93.72302643884 3.916591269877
+89.8870358383 94.74290090133 4.809697443233
+87.05838461737 82.52158227934 10.3681851657
+86.56171579354 84.61481200814 10.40923667174
+87.28978037789 84.22135089334 9.650016622116
+91.99796250152 85.97382150086 1.519676157958
+91.41983478308 86.34049730935 2.903012247065
+86.98268430651 78.05139466641 11.81754825277
+91.81560657137 78.14260723897 -0.6524242830686
+86.8722058784 87.03288787543 9.354827421718
+91.35672928113 78.7639907204 1.17125485171
+91.53968604339 77.7202798909 0.2403441317094
+84.86545927303 78.71558791114 15.51626244516
+86.79722653072 93.3408647828 7.342636373913
+91.63020782938 90.91409015372 3.14840834059
+87.25531824172 74.47592155632 12.52927221104
+86.07425696252 75.21362038574 15.21297464765
+91.16106283305 81.48663482954 2.492502407732
+90.5757893716 80.50375765165 3.922765152667
+82.67168107606 86.33491475752 13.27365472624
+82.96487921529 87.60913204281 12.329282444
+87.82636242729 84.88753643638 8.842631330747
+84.35474740005 87.83056859068 11.21352843057
+84.55876620841 88.79983575754 10.6423089934
+83.17465723672 88.40634089228 11.76835493646
+90.85284906504 87.33686142924 4.121887080868
+86.72724152045 73.8489141689 14.2127483518
+86.42355444031 73.80296018704 15.09471583991
+86.93659826144 75.89680908807 12.73505114447
+83.54500604627 89.52185961323 10.99503048512
+88.83978565047 75.58107846602 8.260375606895
+88.72239336258 76.48510243038 8.436356181968
+91.66303103922 87.64914656063 2.668165150909
+86.12595505386 76.63627073986 14.24302571059
+90.63286939882 85.3266640223 4.382207547975
+85.1824586822 79.69308786923 14.30335998377
+90.22312494777 84.82135828186 5.152478488357
+91.95415466013 83.85535866474 0.9179829688677
+88.23374725874 92.97600033614 6.514255895342
+88.15189788739 93.96486724046 6.24411450713
+85.3800757181 76.98502795314 15.73048748873
+84.80319976807 92.65283203125 8.842200279236
+89.09641550807 89.22223592452 6.551811823289
+85.8175845876 88.16254355025 9.930269660873
+87.66547683905 77.09918437201 10.72152242862
+78.94045741667 88.39228989734 14.52370677822
+79.71511631602 88.43273633661 14.0000620674
+89.61954054135 91.41099822565 5.649216608215
+89.23067474365 94.6247253418 5.391794204712
+91.14098067071 90.46179999326 3.826403976189
+91.93963698747 89.54916561137 2.519897373028
+90.67907607165 92.08784992503 4.414110286577
+92.00748229705 88.36647480616 2.15883145893
+87.31342900202 80.76548024745 10.42083525356
+88.54108873981 81.32345228217 8.303818225533
+86.29197840378 83.55138824339 11.06344748072
+91.63576602179 81.14048734557 0.926761868349
+91.96373053199 81.33067011421 -0.1021323030729
+88.10058326017 82.54123945134 8.854680095962
+87.02030891857 74.10770397135 13.29008336436
+88.66114033185 85.22844041822 7.687983749868
+83.37451380026 84.06156417283 14.06219752259
+82.3656683929 85.31267926977 14.18556991487
+81.62659745583 85.67233020136 14.57361705313
+79.91235379794 86.99394195408 14.90861860817
+91.92134843838 79.00929111616 -0.7694692860489
+87.93559569912 86.4273596092 8.393204653578
+90.64911812118 83.58820518874 4.189330441393
+90.19847423323 78.53839085091 4.690054556929
+82.70591734945 82.54285737196 15.89561516193
+87.55644665574 74.60603473099 11.72233372584
+86.59650472634 78.69950252896 12.30020046608
+85.76543352653 77.28645400239 14.66293197255
+88.30923622 91.57787766767 6.847835587868
+88.78585426569 90.38592908654 6.684616642745
+81.39871557718 87.91580271512 13.23167464013
+77.44239816095 89.3917582412 14.70647816142
+78.2346321736 89.45395679512 14.22778685506
+89.93552467668 77.14410101136 5.302068334108
+84.85307911288 90.96940089197 9.550021626143
+91.07435434956 77.22910817763 1.777355579159
+91.53472676551 84.75175611001 2.296705553252
+91.07428743866 84.0006697594 3.253891270141
+87.57206861003 83.44109160119 9.452703099751
+86.46837704385 77.01830345632 13.28299203037
+88.24534904025 80.33910657831 8.94283369881
+86.03946980836 78.22531162763 13.56218249175
+85.12085057396 83.64329737652 12.4032963563
+83.99859345837 83.65910767374 13.64917308958
+91.1068447306 78.5125448281 1.950304716625
+88.93743904592 77.0416921546 7.862316649276
+81.89763387706 84.16300891638 15.47949979105
+84.17142822011 79.10427843247 16.46661667861
+90.1977196189 77.71826121309 4.613359081808
+91.46686301969 82.4573958955 1.86560046256
+85.17639263798 78.97291532426 14.76262265383
+81.25829074791 85.01580337653 15.40024671834
+85.57806257522 75.69618258949 16.18963166867
+90.6740474846 77.84868862773 3.208271283977
+88.39299437575 88.93655007712 7.376023566561
+88.02443178926 87.46291572177 8.066227217816
+88.69549876852 87.75415760413 7.255780630642
+90.83584527911 76.98060581503 2.533464600951
+90.24894299365 86.54477235102 5.141658921075
+89.6976072794 83.56866623825 6.139232569616
+90.95815006486 79.5860525514 2.65996163632
+90.54331172733 78.9237771769 3.77410403802
+85.45053954275 84.87164856816 11.51553144517
+89.01834595117 80.58443177613 7.473181777199
+88.84104627055 78.03470716764 8.002824388128
+85.67184761911 78.03529306301 14.39524142803
+86.53578569197 81.78319067054 11.31637757513
+85.60993633696 81.75701177328 12.63336621461
+85.80387144362 82.58918845105 12.02841449852
+88.50429153833 75.12400624369 9.178192908443
+84.48876628209 84.50364974112 12.66249526191
+89.41742707428 86.59282506324 6.459802357741
+89.14239643036 75.46130015491 7.488269950964
+88.8400388208 93.12543610622 6.022451468792
+84.35601236007 79.54349955601 15.80498719226
+85.72661361084 91.66288450582 8.710852466457
+81.35632656989 86.57100524067 14.14937345639
+80.30996143596 86.3295074551 15.12856879422
+80.51503942099 87.4160358727 14.17386056988
+88.82555358781 79.70698776571 7.908609004999
+88.47215841331 78.74225769992 8.723597569818
+89.15110921007 79.03160706289 7.254502463971
+89.52260410248 84.66512629317 6.423342033217
+86.62964224798 80.05172781141 11.73751161387
+87.28315315953 79.35507964116 10.84288117454
+78.39582061768 90.47370910645 13.52938842773
+89.42167039058 88.09489616508 6.301769902343
+88.59964322506 86.00871702093 7.658460101584
+82.20010929418 89.13086851198 12.01494409867
+89.92675339147 80.15382280797 5.522754060304
+87.41138994867 77.77647847548 11.06513801808
+89.32906590509 76.65549889054 6.913354804829
+89.41991350218 90.72714883996 5.963601242846
+83.84750712961 85.91736606737 12.54230305646
+84.70610177702 77.98854431145 16.41686504562
+86.55716047843 75.25385523287 13.94340207575
+79.82708740234 90.28623199463 12.78842067719
+86.00337982178 93.08626556396 7.952734470367
+86.60580976688 91.80148730574 8.072677376784
+87.22050151166 92.6603694961 7.33227675047
+88.16716263207 75.91256782034 9.852927479006
+86.72900679298 85.25690869653 10.0289813322
+88.26398948523 77.33772345921 9.36442753796
+84.48457755837 80.3785157358 15.0009805203
+86.90911612862 90.76068403985 8.224517995328
+87.99764207043 89.9437017199 7.547420421772
+87.17673875088 89.73814630364 8.325225356834
+91.27346915349 80.9035465747 2.006769025922
+87.96093644471 79.91600342135 9.518832922801
+89.77981626992 94.04055669295 5.089906242403
+82.77049394816 85.64393889715 13.61366577784
+86.36565938384 89.5909865994 9.02122595877
+85.37867446359 89.55990884413 9.762504200646
+82.93187929823 89.91362149273 11.18538301618
+90.40184120368 77.36763120377 3.978408478545
+83.05097214393 83.5778676545 14.72926797176
+89.72248932475 78.42898288903 5.918966931824
+91.67516532539 83.73576836354 1.656707294069
+78.71974914613 89.79709073339 13.73515288393
+90.92734145902 89.78390610645 4.112839472319
+89.00168562649 84.3598607859 7.279939209434
+81.71748951367 83.44541399728 16.26328327578
+85.63151466028 80.6783179693 13.07487257959
+85.2061604245 80.70675069058 13.70207637819
+89.67707012829 76.04729367448 5.994030004774
+91.80352195174 93.28461363862 3.066921973103
+92.09239955624 78.00218956873 -1.739866962382
+86.92069711747 77.41143533536 12.17691558281
+88.95079812341 82.13893390599 7.523756483162
+91.95468325146 82.85228212281 0.526891242366
+90.81599675904 80.36183194305 3.237449743645
+90.54766845703 96.017578125 3.999451637268
+92.0055241551 85.22912795386 1.269253743002
+89.88338535679 86.02225332199 5.764867970561
+81.84564932064 88.23991492601 12.73434009247
+85.93024321957 74.5364087842 16.03535734115
+84.82059474593 82.96593066196 13.08379381789
+89.68150719318 82.24151111624 6.159472595437
+90.19728828303 82.48156059828 5.093752428947
+89.71125544255 81.47605972064 6.071317743465
+83.72882271397 81.77689831853 15.14467408305
+86.25218261707 74.5079949549 15.1626753312
+85.00988855546 78.03025378603 15.74992136855
+90.18405860601 81.13314612602 4.994330226107
+85.65257806689 85.78328208136 10.96127222822
+84.83030951908 86.3240240171 11.50096113237
+84.61082060889 85.59183835254 12.02018475677
+86.18189079471 86.17576795044 10.30566166154
+86.74048774938 74.51238814105 13.8384304986
+90.4003943129 76.54491258221 3.884703534253
+81.81327505325 86.83154980488 13.62882318444
+91.31694022014 87.95524252007 3.365571118591
+91.11867506879 86.49884415014 3.536760369568
+91.31518166869 85.6281458942 2.994287968664
+90.94835188199 85.05695431771 3.705049918602
+88.43889418727 76.69555098254 9.075219277086
+89.43228854681 85.86706706099 6.500004845721
+82.73513628171 90.71773976847 10.92107545621
+91.98887439509 87.6326927391 2.007190497912
+91.52793371682 94.46079285745 3.343908937295
+86.09082407967 90.54507374952 8.898060742125
+80.63731155247 88.65047179233 13.26168529876
+91.35347577036 83.01191364709 2.327252349276
+85.6788758969 76.12079563979 15.63082402037
+89.21793908226 78.32106678605 7.123794200307
+81.89286506116 90.88075220467 11.33043282293
+80.96296280172 90.02565655652 12.27699603282
+87.18562304132 83.17319429571 10.03197094628
+85.94523835859 79.61159497471 13.06151633859
+89.54586658536 87.39575929373 6.202245926172
+84.19560615902 86.93932821561 11.75398955827
+82.64923765358 87.17546494488 12.80072555853
+85.57182703976 79.40049630906 13.80412587682
+92.11813422526 80.3336940627 -1.0274672359
+91.58969915559 89.26351826489 3.053117287076
+82.90513702073 82.93644120087 15.35797196151
+83.51087357062 82.76432473619 14.76703486348
+90.13510669594 76.2169664233 4.66747957696
+90.64913923384 93.33554679517 4.353233685676
+84.13267337331 83.00396580987 13.88201394276
+84.87243712271 81.85866336116 13.57925886949
+89.28944999749 91.98966703195 5.879106347835
+87.81999722862 91.35373114943 7.320547757033
+91.93708444299 80.37746807551 -0.360749318184
+85.84211427154 85.13059367186 11.01535006381
+85.56820387932 84.18880644955 11.66829122503
+91.48388065108 92.79963299821 3.436939740116
+83.75276774305 86.54362591673 12.30003839505
+90.33403260788 89.84300780106 4.947890651582
+91.17155781479 95.62999233702 3.555330232582
+81.08768683738 86.17984327111 14.63971208729
+90.38594006628 83.29347411891 4.750388981447
+87.98665421745 78.28535011375 9.784775030141
+82.3679546364 89.67867529519 11.635015703
+83.74534240316 85.1538186026 13.04163202392
+91.63708732988 82.17328836847 1.27095837172
+85.97074285508 75.8696789085 15.06787457704
+88.78623129313 74.47325883927 8.562521367548
+84.53518886933 91.56446841495 9.484550907027
+84.32967128058 90.25555108377 10.18170123086
+84.85811913064 89.38922071855 10.19417118041
+92.05118344848 84.5308652651 0.9058592816885
+88.24781393992 81.01711913492 8.844436451774
+85.27588195651 87.09941342014 10.79102642662
+88.3946809784 83.91013776736 8.227404481239
+87.55246737728 75.47893998843 11.44680253484
+87.72693552863 85.56194386854 8.827585540933
+82.36266194973 87.7872771424 12.65064169419
+90.99625808495 81.84789400481 3.029675030284
+87.12534937579 76.26893668453 12.16272998574
+90.51427577132 90.99750450867 4.669163752209
+88.75231293904 87.09091301546 7.292276779629
+81.77196144489 88.83586075055 12.43768637401
+80.99975585938 91.66167449951 11.48725414276
+80.78927139352 89.16541907359 12.85188992007
+87.66350506837 78.08952304598 10.47636582897
+91.93329295926 77.26298047189 -1.365480028674
+83.43715667725 91.99025726318 9.931365013123
+83.88444128096 91.49158214368 9.898426233423
+3  110 256 305
+3  43 473 476
+3  316 137 336
+3  240 195 293
+3  238 228 412
+3  218 84 532
+3  15 383 434
+3  119 374 420
+3  227 16 302
+3  283 127 328
+3  106 302 459
+3  207 66 351
+3  344 110 416
+3  31 254 331
+3  199 197 486
+3  144 196 509
+3  65 474 499
+3  348 171 407
+3  119 314 315
+3  278 8 533
+3  301 151 378
+3  88 291 456
+3  324 140 356
+3  66 207 273
+3  11 214 458
+3  251 31 267
+3  75 255 539
+3  181 9 230
+3  83 165 250
+3  15 326 383
+3  161 132 445
+3  297 184 428
+3  34 427 433
+3  226 133 318
+3  256 110 344
+3  265 47 425
+3  405 246 479
+3  189 79 338
+3  276 78 521
+3  266 116 431
+3  74 261 362
+3  116 275 431
+3  190 162 229
+3  40 279 281
+3  208 40 388
+3  260 74 362
+3  150 307 313
+3  166 49 339
+3  112 189 253
+3  346 45 439
+3  99 242 346
+3  117 209 319
+3  128 36 171
+3  14 276 544
+3  466 130 468
+3  107 371 465
+3  49 371 514
+3  161 227 352
+3  352 227 390
+3  264 73 543
+3  81 173 308
+3  246 51 359
+3  239 71 432
+3  46 243 370
+3  273 151 537
+3  365 85 461
+3  276 14 413
+3  283 87 538
+3  93 248 424
+3  169 299 354
+3  100 250 415
+3  59 415 498
+3  208 102 247
+3  19 226 318
+3  254 118 331
+3  282 92 285
+3  34 404 427
+3  121 257 263
+3  257 441 529
+3  204 24 417
+3  65 218 532
+3  80 188 459
+3  207 151 273
+3  102 208 388
+3  79 278 281
+3  108 11 458
+3  73 265 425
+3  304 24 308
+3  218 22 327
+3  64 257 528
+3  241 71 269
+3  241 144 509
+3  37 166 286
+3  228 114 412
+3  264 51 330
+3  143 260 362
+3  131 284 515
+3  288 287 430
+3  126 278 533
+3  48 161 445
+3  408 153 410
+3  103 219 337
+3  275 78 409
+3  114 228 337
+3  278 126 281
+3  302 80 459
+3  80 273 537
+3  307 81 313
+3  81 204 313
+3  82 173 516
+3  82 223 303
+3  226 83 309
+3  226 309 481
+3  234 185 327
+3  195 84 293
+3  289 182 461
+3  85 365 482
+3  133 482 483
+3  207 351 366
+3  87 312 538
+3  464 300 470
+3  312 213 538
+3  213 88 456
+3  381 176 492
+3  248 89 317
+3  96 233 252
+3  32 271 517
+3  268 91 503
+3  31 251 254
+3  284 131 507
+3  248 27 424
+3  90 340 341
+3  194 90 233
+3  292 148 295
+3  294 186 373
+3  322 323 402
+3  146 21 184
+3  100 168 271
+3  109 212 385
+3  242 99 345
+3  258 107 372
+3  331 118 421
+3  118 168 421
+3  364 191 386
+3  221 102 411
+3  247 102 320
+3  288 367 496
+3  278 79 342
+3  224 347 380
+3  242 345 444
+3  105 242 375
+3  188 33 389
+3  227 106 390
+3  99 258 345
+3  167 371 399
+3  182 55 530
+3  70 185 234
+3  433 120 435
+3  35 240 293
+3  125 195 240
+3  25 392 408
+3  111 408 410
+3  166 76 286
+3  256 8 305
+3  72 238 412
+3  396 196 397
+3  328 419 544
+3  114 219 413
+3  314 119 401
+3  116 328 544
+3  328 127 419
+3  12 423 471
+3  255 118 539
+3  271 118 517
+3  115 314 398
+3  115 398 429
+3  184 61 428
+3  98 211 357
+3  234 121 440
+3  22 234 327
+3  236 122 426
+3  198 7 231
+3  122 236 325
+3  135 236 426
+3  124 242 444
+3  177 4 270
+3  76 287 288
+3  125 288 430
+3  220 126 449
+3  126 256 449
+3  419 14 544
+3  56 283 328
+3  129 348 407
+3  128 171 348
+3  104 10 224
+3  347 224 519
+3  220 40 281
+3  247 351 520
+3  251 131 335
+3  161 16 227
+3  132 161 352
+3  133 226 481
+3  482 225 483
+3  76 166 339
+3  474 134 475
+3  235 34 433
+3  135 187 427
+3  321 136 446
+3  378 151 491
+3  309 60 481
+3  173 82 304
+3  45 346 347
+3  23 241 509
+3  139 209 452
+3  376 54 403
+3  243 88 370
+3  27 213 312
+3  205 382 386
+3  18 142 205
+3  143 443 478
+3  205 142 478
+3  319 139 501
+3  209 71 241
+3  52 375 450
+3  345 44 444
+3  146 184 297
+3  135 297 428
+3  147 393 493
+3  153 48 410
+3  316 336 487
+3  30 149 336
+3  150 528 529
+3  528 257 529
+3  26 446 491
+3  68 301 378
+3  299 169 343
+3  77 210 279
+3  161 48 249
+3  423 154 471
+3  167 391 514
+3  134 391 475
+3  156 268 480
+3  60 309 480
+3  157 316 487
+3  336 149 487
+3  62 159 285
+3  159 282 285
+3  160 321 446
+3  26 365 446
+3  16 280 472
+3  161 249 418
+3  17 162 190
+3  6 163 401
+3  163 314 401
+3  164 384 540
+3  212 59 385
+3  67 221 411
+3  385 59 540
+3  113 166 395
+3  196 49 397
+3  107 258 399
+3  371 107 399
+3  168 118 271
+3  168 100 326
+3  299 87 354
+3  17 169 354
+3  170 490 543
+3  138 246 359
+3  51 171 330
+3  94 264 330
+3  274 232 495
+3  69 172 232
+3  173 304 308
+3  218 65 306
+3  174 124 353
+3  4 174 353
+3  264 94 265
+3  171 36 329
+3  89 324 356
+3  324 89 492
+3  71 239 270
+3  117 296 403
+3  248 93 300
+3  310 193 311
+3  91 334 503
+3  230 9 298
+3  233 90 252
+3  11 180 341
+3  454 181 515
+3  160 365 461
+3  5 182 289
+3  98 331 421
+3  61 357 511
+3  183 61 322
+3  61 184 322
+3  350 35 384
+3  35 185 245
+3  96 186 294
+3  148 58 295
+3  187 61 511
+3  357 211 511
+3  301 68 436
+3  33 188 194
+3  279 210 457
+3  210 25 338
+3  38 229 398
+3  38 190 229
+3  186 191 364
+3  186 58 191
+3  261 74 420
+3  41 260 506
+3  53 300 464
+3  57 193 310
+3  33 294 373
+3  33 194 233
+3  287 43 430
+3  216 84 476
+3  371 167 514
+3  113 396 397
+3  42 199 272
+3  64 197 199
+3  64 199 547
+3  202 42 272
+3  199 546 547
+3  198 231 546
+3  97 332 438
+3  184 21 323
+3  73 274 495
+3  73 264 265
+3  202 494 542
+3  20 232 274
+3  360 2 361
+3  417 172 522
+3  204 417 522
+3  352 50 377
+3  364 101 373
+3  280 130 472
+3  351 66 520
+3  208 247 520
+3  351 86 366
+3  130 466 467
+3  279 40 466
+3  209 139 319
+3  117 239 432
+3  210 77 392
+3  25 210 392
+3  326 15 358
+3  215 120 434
+3  100 212 326
+3  212 100 415
+3  88 368 370
+3  88 213 368
+3  180 502 512
+3  458 214 524
+3  212 109 383
+3  185 70 245
+3  84 216 532
+3  474 217 499
+3  217 474 475
+3  13 422 475
+3  441 22 529
+3  22 218 307
+3  219 114 337
+3  219 103 437
+3  126 220 281
+3  67 411 449
+3  400 462 485
+3  19 221 462
+3  222 400 485
+3  462 221 485
+3  223 82 422
+3  99 223 523
+3  346 105 347
+3  104 224 380
+3  365 160 446
+3  26 225 365
+3  165 83 387
+3  83 226 387
+3  106 227 302
+3  50 352 390
+3  113 228 238
+3  166 37 395
+3  162 56 229
+3  314 115 315
+3  255 75 333
+3  75 230 298
+3  123 231 325
+3  257 64 363
+3  202 272 494
+3  292 96 349
+3  96 292 295
+3  257 121 441
+3  70 234 440
+3  235 433 435
+3  70 235 435
+3  123 236 404
+3  236 123 325
+3  237 101 382
+3  50 237 382
+3  238 72 451
+3  23 396 451
+3  54 177 239
+3  239 177 270
+3  240 35 350
+3  125 240 535
+3  465 144 508
+3  144 241 269
+3  242 124 375
+3  242 105 346
+3  139 497 501
+3  72 243 497
+3  291 88 355
+3  88 243 355
+3  70 215 245
+3  215 109 245
+3  129 246 405
+3  171 51 407
+3  19 318 320
+3  366 86 483
+3  213 27 368
+3  27 248 317
+3  48 153 249
+3  153 77 249
+3  250 100 262
+3  83 250 262
+3  332 285 438
+3  92 251 267
+3  29 252 502
+3  189 103 253
+3  228 37 337
+3  254 251 335
+3  75 254 335
+3  333 91 448
+3  156 32 268
+3  305 125 535
+3  110 305 535
+3  123 263 363
+3  263 257 363
+3  258 99 523
+3  13 258 523
+3  41 192 260
+3  192 6 259
+3  259 74 453
+3  74 260 453
+3  147 261 420
+3  410 48 493
+3  32 156 262
+3  156 83 262
+3  263 34 489
+3  121 263 489
+3  490 264 543
+3  51 264 490
+3  2 203 361
+3  266 115 429
+3  56 266 429
+3  97 267 402
+3  323 97 402
+3  255 333 448
+3  118 255 517
+3  145 269 469
+3  44 269 505
+3  145 353 504
+3  269 71 469
+3  32 262 271
+3  262 100 271
+3  272 199 486
+3  272 486 494
+3  80 206 273
+3  206 66 273
+3  47 201 425
+3  201 20 274
+3  111 393 409
+3  374 39 393
+3  219 276 413
+3  219 78 276
+3  369 277 437
+3  25 277 369
+3  288 8 367
+3  8 278 367
+3  77 279 468
+3  279 466 468
+3  249 280 418
+3  249 77 280
+3  281 279 457
+3  79 281 457
+3  92 282 284
+3  92 284 507
+3  283 56 534
+3  87 283 534
+3  28 284 488
+3  267 97 438
+3  62 285 332
+3  112 253 286
+3  253 37 286
+3  391 155 514
+3  473 43 513
+3  367 112 496
+3  76 288 496
+3  85 157 289
+3  157 5 289
+3  60 157 290
+3  157 85 290
+3  244 291 355
+3  244 14 291
+3  29 95 349
+3  95 292 349
+3  84 185 293
+3  185 35 293
+3  33 233 294
+3  233 96 294
+3  58 186 295
+3  186 96 295
+3  46 319 501
+3  319 46 379
+3  122 63 426
+3  63 146 297
+3  9 179 298
+3  179 91 298
+3  152 299 343
+3  152 87 299
+3  93 310 311
+3  188 301 436
+3  188 80 301
+3  16 206 302
+3  206 80 302
+3  246 138 479
+3  304 82 500
+3  500 138 536
+3  138 359 463
+3  8 288 305
+3  288 125 305
+3  173 499 516
+3  173 81 306
+3  218 306 307
+3  306 81 307
+3  24 204 308
+3  204 81 308
+3  83 156 309
+3  309 156 480
+3  57 310 343
+3  169 57 343
+3  193 1 311
+3  1 178 311
+3  87 152 312
+3  152 27 312
+3  197 313 442
+3  197 150 313
+3  315 39 374
+3  119 315 374
+3  39 315 484
+3  275 39 431
+3  157 60 316
+3  268 137 480
+3  317 89 356
+3  141 317 356
+3  86 247 320
+3  318 86 320
+3  296 117 379
+3  117 319 379
+3  102 221 320
+3  221 19 320
+3  55 108 321
+3  136 458 524
+3  322 184 323
+3  183 322 402
+3  21 200 323
+3  200 97 323
+3  176 324 492
+3  176 12 324
+3  231 7 325
+3  7 122 325
+3  326 212 383
+3  168 326 358
+3  185 84 327
+3  84 218 327
+3  116 266 328
+3  266 56 328
+3  36 175 329
+3  175 360 361
+3  171 329 330
+3  329 94 330
+3  331 98 510
+3  31 331 510
+3  97 200 332
+3  200 62 332
+3  75 298 333
+3  298 91 333
+3  91 179 334
+3  179 30 334
+3  131 230 335
+3  230 75 335
+3  137 334 336
+3  334 30 336
+3  37 253 337
+3  253 103 337
+3  103 369 437
+3  369 103 531
+3  155 287 339
+3  287 76 339
+3  90 194 340
+3  68 340 436
+3  340 214 341
+3  214 11 341
+3  79 189 342
+3  189 112 342
+3  343 310 477
+3  152 343 477
+3  222 344 416
+3  222 67 344
+3  345 258 372
+3  44 345 372
+3  223 99 439
+3  99 346 439
+3  45 347 405
+3  45 405 479
+3  224 10 406
+3  10 128 406
+3  96 252 349
+3  252 29 349
+3  164 350 384
+3  110 350 416
+3  247 86 351
+3  225 366 483
+3  143 362 377
+3  362 132 377
+3  145 270 353
+3  270 4 353
+3  56 162 534
+3  162 17 354
+3  243 72 355
+3  72 244 355
+3  46 370 394
+3  370 141 394
+3  61 183 357
+3  183 98 357
+3  15 211 358
+3  211 98 358
+3  304 500 536
+3  24 304 536
+3  94 329 361
+3  329 175 361
+3  414 47 447
+3  47 265 447
+3  261 132 362
+3  132 352 377
+3  363 64 527
+3  123 363 527
+3  382 101 386
+3  186 364 373
+3  365 225 482
+3  85 289 461
+3  225 26 366
+3  26 207 366
+3  278 342 367
+3  342 112 367
+3  27 317 368
+3  317 141 368
+3  338 25 531
+3  25 369 531
+3  368 141 370
+3  141 356 394
+3  49 196 371
+3  371 196 465
+3  196 144 465
+3  44 372 508
+3  101 237 373
+3  33 373 389
+3  147 374 393
+3  374 147 420
+3  105 375 380
+3  347 105 380
+3  296 376 403
+3  296 154 376
+3  377 50 443
+3  143 377 443
+3  378 136 524
+3  68 378 524
+3  356 140 394
+3  140 296 379
+3  375 52 380
+3  52 104 380
+3  300 381 492
+3  300 53 381
+3  101 364 386
+3  50 382 443
+3  109 215 383
+3  383 215 434
+3  35 245 384
+3  245 109 384
+3  384 385 540
+3  384 109 385
+3  191 18 386
+3  18 205 386
+3  226 19 387
+3  400 165 462
+3  40 220 388
+3  220 102 388
+3  373 237 389
+3  237 106 389
+3  106 237 390
+3  237 50 390
+3  473 216 476
+3  155 391 513
+3  77 153 392
+3  392 153 408
+3  39 275 393
+3  393 275 409
+3  140 379 394
+3  379 46 394
+3  37 228 395
+3  228 113 395
+3  113 238 396
+3  396 238 451
+3  49 166 397
+3  166 113 397
+3  38 398 526
+3  258 13 399
+3  13 167 399
+3  222 59 400
+3  250 165 498
+3  119 259 401
+3  259 6 401
+3  267 31 402
+3  98 183 510
+3  54 239 403
+3  239 117 403
+3  34 263 404
+3  263 123 404
+3  405 347 519
+3  129 405 519
+3  128 348 406
+3  348 129 406
+3  51 246 407
+3  246 129 407
+3  111 277 408
+3  277 25 408
+3  78 277 409
+3  277 111 409
+3  393 111 493
+3  111 410 493
+3  102 220 411
+3  411 220 449
+3  114 244 412
+3  244 72 412
+3  14 244 413
+3  244 114 413
+3  361 203 447
+3  203 414 447
+3  415 250 498
+3  59 212 415
+3  350 164 416
+3  164 222 416
+3  359 170 463
+3  463 170 541
+3  280 16 418
+3  16 161 418
+3  127 291 419
+3  291 14 419
+3  74 259 420
+3  259 119 420
+3  168 358 421
+3  358 98 421
+3  499 217 516
+3  422 217 475
+3  140 324 471
+3  324 12 471
+3  27 152 424
+3  310 93 477
+3  201 274 425
+3  274 73 425
+3  63 297 426
+3  297 135 426
+3  404 236 427
+3  236 135 427
+3  61 187 428
+3  187 135 428
+3  398 229 429
+3  229 56 429
+3  84 195 476
+3  195 125 430
+3  431 39 484
+3  266 431 484
+3  71 209 432
+3  209 117 432
+3  427 187 433
+3  187 120 433
+3  120 211 434
+3  211 15 434
+3  120 215 435
+3  215 70 435
+3  340 194 436
+3  194 188 436
+3  277 78 437
+3  78 219 437
+3  285 92 438
+3  92 267 438
+3  45 303 439
+3  303 223 439
+3  34 235 489
+3  235 70 440
+3  121 234 441
+3  234 22 441
+3  313 204 442
+3  172 69 522
+3  382 205 443
+3  443 205 478
+3  124 444 504
+3  353 124 504
+3  132 261 445
+3  261 48 445
+3  136 378 491
+3  446 136 491
+3  265 94 447
+3  94 361 447
+3  91 268 448
+3  268 32 448
+3  256 344 449
+3  344 67 449
+3  375 124 450
+3  124 174 450
+3  451 139 452
+3  23 451 452
+3  209 241 452
+3  241 23 452
+3  260 192 453
+3  192 259 453
+3  284 28 515
+3  28 454 515
+3  148 292 545
+3  292 95 455
+3  291 127 456
+3  127 213 456
+3  210 338 457
+3  338 79 457
+3  136 321 458
+3  321 108 458
+3  188 389 459
+3  389 106 459
+3  158 282 518
+3  282 159 460
+3  461 182 530
+3  160 461 530
+3  165 387 462
+3  387 19 462
+3  417 24 541
+3  24 463 541
+3  311 178 470
+3  178 464 470
+3  372 107 508
+3  107 465 508
+3  40 208 466
+3  466 208 467
+3  66 206 467
+3  206 130 467
+3  130 280 468
+3  280 77 468
+3  71 270 469
+3  270 145 469
+3  300 93 470
+3  93 311 470
+3  154 296 471
+3  296 140 471
+3  130 206 472
+3  206 16 472
+3  391 134 513
+3  134 473 513
+3  216 473 474
+3  473 134 474
+3  391 167 475
+3  167 13 475
+3  195 430 476
+3  430 43 476
+3  93 424 477
+3  424 152 477
+3  143 478 506
+3  260 143 506
+3  138 303 479
+3  303 45 479
+3  137 316 480
+3  316 60 480
+3  60 290 481
+3  290 133 481
+3  133 290 482
+3  290 85 482
+3  86 318 483
+3  318 133 483
+3  315 115 484
+3  115 266 484
+3  221 67 485
+3  67 222 485
+3  197 442 486
+3  442 69 486
+3  149 5 487
+3  5 157 487
+3  284 282 488
+3  282 158 488
+3  235 440 489
+3  440 121 489
+3  170 359 490
+3  359 51 490
+3  151 207 491
+3  207 26 491
+3  492 89 525
+3  300 492 525
+3  48 261 493
+3  261 147 493
+3  486 69 494
+3  69 232 494
+3  232 172 495
+3  73 495 543
+3  112 286 496
+3  286 76 496
+3  139 451 497
+3  451 72 497
+3  165 400 498
+3  400 59 498
+3  173 306 499
+3  306 65 499
+3  82 303 500
+3  303 138 500
+3  497 243 501
+3  243 46 501
+3  90 341 512
+3  341 180 512
+3  334 137 503
+3  137 268 503
+3  145 504 505
+3  269 145 505
+3  504 444 505
+3  444 44 505
+3  478 142 506
+3  142 41 506
+3  131 251 507
+3  251 92 507
+3  144 269 508
+3  269 44 508
+3  196 396 509
+3  396 23 509
+3  183 402 510
+3  402 31 510
+3  211 120 511
+3  120 187 511
+3  502 252 512
+3  252 90 512
+3  43 287 513
+3  287 155 513
+3  155 339 514
+3  339 49 514
+3  181 230 515
+3  230 131 515
+3  217 422 516
+3  422 82 516
+3  255 448 517
+3  448 32 517
+3  282 460 518
+3  460 3 518
+3  224 406 519
+3  406 129 519
+3  66 467 520
+3  467 208 520
+3  78 275 521
+3  275 116 521
+3  69 442 522
+3  442 204 522
+3  223 422 523
+3  422 13 523
+3  214 340 524
+3  340 68 524
+3  89 248 525
+3  248 300 525
+3  398 314 526
+3  314 163 526
+3  546 231 547
+3  231 123 527
+3  150 197 528
+3  197 64 528
+3  22 307 529
+3  307 150 529
+3  55 321 530
+3  321 160 530
+3  103 189 531
+3  189 338 531
+3  216 474 532
+3  474 65 532
+3  8 256 533
+3  256 126 533
+3  162 354 534
+3  354 87 534
+3  240 350 535
+3  350 110 535
+3  138 463 536
+3  463 24 536
+3  151 301 537
+3  301 80 537
+3  213 127 538
+3  127 283 538
+3  118 254 539
+3  254 75 539
+3  59 222 540
+3  222 164 540
+3  495 172 543
+3  172 417 541
+3  494 232 542
+3  232 20 542
+3  172 541 543
+3  541 170 543
+3  276 521 544
+3  521 116 544
+3  292 455 545
+3  455 0 545
+3  199 42 546
+3  42 198 546
+3  231 527 547
+3  527 64 547
+
diff -pruN 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_all_short_edges.cmd 5.1-1/examples/Surface_mesh_simplification/edge_collapse_all_short_edges.cmd
--- 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_all_short_edges.cmd	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_simplification/edge_collapse_all_short_edges.cmd	1970-01-01 00:00:00.000000000 +0000
@@ -1 +0,0 @@
-cube-meshed.off 0.2
diff -pruN 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_all_short_edges.cpp 5.1-1/examples/Surface_mesh_simplification/edge_collapse_all_short_edges.cpp
--- 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_all_short_edges.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_simplification/edge_collapse_all_short_edges.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -1,6 +1,3 @@
-#include <iostream>
-#include <fstream>
-
 #include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
 #include <CGAL/Polyhedron_3.h>
 
@@ -12,45 +9,48 @@
 #include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_length_stop_predicate.h>
 #include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Midpoint_placement.h>
 
+#include <iostream>
+#include <fstream>
 
 typedef CGAL::Exact_predicates_inexact_constructions_kernel Kernel;
-typedef CGAL::Polyhedron_3<Kernel> Surface_mesh;
+typedef CGAL::Polyhedron_3<Kernel>                          Surface_mesh;
 
-namespace SMS = CGAL::Surface_mesh_simplification ;
+namespace SMS = CGAL::Surface_mesh_simplification;
 
-int main( int argc, char** argv )
+int main(int argc, char** argv)
 {
-  if (argc<3)
+  Surface_mesh surface_mesh;
+  const char* filename = (argc > 1) ? argv[1] : "data/cube-meshed.off";
+  std::ifstream is(filename);
+  if(!is || !(is >> surface_mesh))
   {
-    std::cerr << "Usage: " << argv[0] << " input.off minimal_edge_length [out.off]\n";
+    std::cerr << "Failed to read input mesh: " << filename << std::endl;
     return EXIT_FAILURE;
   }
 
-  Surface_mesh surface_mesh;
-
-  std::ifstream is(argv[1]) ; is >> surface_mesh ;
-  double threshold = atof(argv[2]);
-
-  if (!CGAL::is_triangle_mesh(surface_mesh)){
+  if(!CGAL::is_triangle_mesh(surface_mesh))
+  {
     std::cerr << "Input geometry is not triangulated." << std::endl;
     return EXIT_FAILURE;
   }
 
-  int r = SMS::edge_collapse
-            (surface_mesh
-             , CGAL::Surface_mesh_simplification::Edge_length_stop_predicate<double>(threshold)
-             , CGAL::parameters::vertex_index_map(get(CGAL::vertex_external_index,surface_mesh))
-                               .halfedge_index_map  (get(CGAL::halfedge_external_index  ,surface_mesh))
-                               .get_cost (SMS::Edge_length_cost <Surface_mesh>())
-                               .get_placement(SMS::Midpoint_placement<Surface_mesh>())
-            );
+  const double threshold = (argc > 2) ? std::atof(argv[2]) : 0.2;
+
+  std::cout << "Collapsing edges with length larger than " << threshold
+            << " of mesh: " << filename << "..." << std::endl;
+  int r = SMS::edge_collapse(surface_mesh,
+                             CGAL::Surface_mesh_simplification::Edge_length_stop_predicate<double>(threshold),
+                             CGAL::parameters::vertex_index_map(get(CGAL::vertex_external_index, surface_mesh))
+                                              .halfedge_index_map(get(CGAL::halfedge_external_index  ,surface_mesh))
+                                              .get_cost(SMS::Edge_length_cost <Surface_mesh>())
+                                              .get_placement(SMS::Midpoint_placement<Surface_mesh>()));
 
-  std::cout << "\nFinished...\n" << r << " edges removed.\n"
-            << (surface_mesh.size_of_halfedges()/2) << " final edges.\n" ;
+  std::cout << "\nFinished!\n" << r << " edges removed.\n"
+            << (surface_mesh.size_of_halfedges()/2) << " final edges.\n";
 
-  std::ofstream os( argc > 3 ? argv[3] : "out.off" ) ;
+  std::ofstream os(argc > 3 ? argv[3] : "out.off");
   os.precision(17);
-  os << surface_mesh ;
+  os << surface_mesh;
 
-  return EXIT_SUCCESS ;
+  return EXIT_SUCCESS;
 }
diff -pruN 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_bounded_normal_change.cpp 5.1-1/examples/Surface_mesh_simplification/edge_collapse_bounded_normal_change.cpp
--- 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_bounded_normal_change.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_simplification/edge_collapse_bounded_normal_change.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -9,38 +9,46 @@
 #include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/LindstromTurk_placement.h>
 #include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Bounded_normal_change_placement.h>
 
-
 typedef CGAL::Simple_cartesian<double> Kernel;
 typedef CGAL::Surface_mesh<Kernel::Point_3> Surface_mesh;
 
 namespace SMS = CGAL::Surface_mesh_simplification;
 
-int main( int argc, char** argv )
+int main(int argc, char** argv)
 {
   Surface_mesh surface_mesh;
-
-  std::ifstream is(argc > 1 ? argv[1] : "data/fold.off");
-  is >> surface_mesh;
+  const char* filename = (argc > 1) ? argv[1] : "data/fold.off";
+  std::ifstream is(filename);
+  if(!is || !(is >> surface_mesh))
+  {
+    std::cerr << "Failed to read input mesh: " << filename << std::endl;
+    return EXIT_FAILURE;
+  }
+
+  if(!CGAL::is_triangle_mesh(surface_mesh))
+  {
+    std::cerr << "Input geometry is not triangulated." << std::endl;
+    return EXIT_FAILURE;
+  }
 
   // This is a stop predicate (defines when the algorithm terminates).
   // In this example, the simplification stops when the number of undirected edges
-  // left in the surface mesh drops below the specified number (1000)
-  SMS::Count_stop_predicate<Surface_mesh> stop(num_halfedges(surface_mesh)/2 - 1);
+  // left in the surface mesh drops below the specified number
+  const std::size_t stop_n = (argc > 2) ? std::stoi(argv[2]) : num_halfedges(surface_mesh)/2 - 1;
+  SMS::Count_stop_predicate<Surface_mesh> stop(stop_n);
 
   typedef SMS::Bounded_normal_change_placement<SMS::LindstromTurk_placement<Surface_mesh> > Placement;
 
-
   // This the actual call to the simplification algorithm.
   // The surface mesh and stop conditions are mandatory arguments.
   // The index maps are needed because the vertices and edges
   // of this surface mesh lack an "id()" field.
-  SMS::edge_collapse( surface_mesh,
-                      stop,
-                      CGAL::parameters::get_cost (SMS::LindstromTurk_cost<Surface_mesh>())
-                      .get_placement(Placement())
-                      );
+  std::cout << "Collapsing edges of mesh: " << filename << ", aiming for " << stop_n << " final edges..." << std::endl;
+  SMS::edge_collapse(surface_mesh, stop,
+                     CGAL::parameters::get_cost(SMS::LindstromTurk_cost<Surface_mesh>())
+                                      .get_placement(Placement()));
 
-  std::ofstream os( argc > 2 ? argv[2] : "out.off" );
+  std::ofstream os((argc > 3) ? argv[3] : "out.off");
   os.precision(17);
   os << surface_mesh;
 
diff -pruN 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_constrained_border_polyhedron.cmd 5.1-1/examples/Surface_mesh_simplification/edge_collapse_constrained_border_polyhedron.cmd
--- 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_constrained_border_polyhedron.cmd	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_simplification/edge_collapse_constrained_border_polyhedron.cmd	1970-01-01 00:00:00.000000000 +0000
@@ -1 +0,0 @@
-mesh_with_border.off
diff -pruN 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_constrained_border_polyhedron.cpp 5.1-1/examples/Surface_mesh_simplification/edge_collapse_constrained_border_polyhedron.cpp
--- 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_constrained_border_polyhedron.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_simplification/edge_collapse_constrained_border_polyhedron.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -1,7 +1,3 @@
-#include <iostream>
-#include <fstream>
-#include <map>
-
 #include <CGAL/Simple_cartesian.h>
 #include <CGAL/Polyhedron_3.h>
 
@@ -17,113 +13,107 @@
 // Stop-condition policy
 #include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Count_stop_predicate.h>
 
-typedef CGAL::Simple_cartesian<double> Kernel;
-typedef Kernel::Point_3 Point_3;
-typedef CGAL::Polyhedron_3<Kernel> Surface_mesh;
+#include <iostream>
+#include <fstream>
+#include <map>
+
+typedef CGAL::Simple_cartesian<double>                Kernel;
+typedef Kernel::Point_3                               Point_3;
+typedef CGAL::Polyhedron_3<Kernel>                    Surface_mesh;
 
-namespace SMS = CGAL::Surface_mesh_simplification ;
+namespace SMS = CGAL::Surface_mesh_simplification;
 
-//
 // BGL property map which indicates whether an edge is marked as non-removable
-//
-struct Border_is_constrained_edge_map{
+struct Border_is_constrained_edge_map
+{
   const Surface_mesh* sm_ptr;
   typedef boost::graph_traits<Surface_mesh>::edge_descriptor key_type;
-  typedef bool value_type;
-  typedef value_type reference;
-  typedef boost::readable_property_map_tag category;
-
-  Border_is_constrained_edge_map(const Surface_mesh& sm)
-    : sm_ptr(&sm)
-  {}
+  typedef bool                                               value_type;
+  typedef value_type                                         reference;
+  typedef boost::readable_property_map_tag                   category;
+
+  Border_is_constrained_edge_map(const Surface_mesh& sm) : sm_ptr(&sm) {}
 
   friend bool get(Border_is_constrained_edge_map m, const key_type& edge) {
     return CGAL::is_border(edge, *m.sm_ptr);
   }
 };
 
-//
 // Placement class
-//
 typedef SMS::Constrained_placement<SMS::Midpoint_placement<Surface_mesh>,
                                    Border_is_constrained_edge_map > Placement;
 
-int main( int argc, char** argv )
+int main(int argc, char** argv)
 {
   Surface_mesh surface_mesh;
-
-  if (argc!=2){
-    std::cerr << "Usage: " << argv[0] << " input.off\n";
-    return EXIT_FAILURE;
-  }
-
-  std::ifstream is(argv[1]);
-  if(!is){
-    std::cerr << "Filename provided is invalid\n";
+  const char* filename = (argc > 1) ? argv[1] : "data/mesh_with_border.off";
+  std::ifstream is(filename);
+  if(!is || !(is >> surface_mesh))
+  {
+    std::cerr << "Failed to read input mesh: " << filename << std::endl;
     return EXIT_FAILURE;
   }
 
-  is >> surface_mesh  ;
-  if (!CGAL::is_triangle_mesh(surface_mesh)){
+  if(!CGAL::is_triangle_mesh(surface_mesh))
+  {
     std::cerr << "Input geometry is not triangulated." << std::endl;
     return EXIT_FAILURE;
   }
 
   // map used to check that constrained_edges and the points of its vertices
   // are preserved at the end of the simplification
-  std::map<Surface_mesh::Halfedge_handle,std::pair<Point_3, Point_3> >constrained_edges;
+  std::map<Surface_mesh::Halfedge_handle, std::pair<Point_3, Point_3> >constrained_edges;
   std::size_t nb_border_edges=0;
 
-  for (Surface_mesh::Halfedge_iterator hit=surface_mesh.halfedges_begin(),
-                                       hit_end=surface_mesh.halfedges_end();
-                                       hit!=hit_end; ++hit )
+  for(Surface_mesh::Halfedge_iterator hit=surface_mesh.halfedges_begin(),
+                                      hit_end=surface_mesh.halfedges_end();
+                                      hit!=hit_end; ++hit)
   {
-    if ( hit->is_border() ){
-      constrained_edges[hit]=std::make_pair( hit->opposite()->vertex()->point(),
-                                             hit->vertex()->point() );
+    if(hit->is_border())
+    {
+      constrained_edges[hit] = std::make_pair(hit->opposite()->vertex()->point(),
+                                              hit->vertex()->point());
       ++nb_border_edges;
     }
   }
 
   // Contract the surface mesh as much as possible
   SMS::Count_stop_predicate<Surface_mesh> stop(0);
-
   Border_is_constrained_edge_map bem(surface_mesh);
 
-
   // This the actual call to the simplification algorithm.
   // The surface mesh and stop conditions are mandatory arguments.
   // The index maps are needed because the vertices and edges
   // of this surface mesh lack an "id()" field.
-  int r = SMS::edge_collapse
-            (surface_mesh
-            ,stop
-             ,CGAL::parameters::vertex_index_map(get(CGAL::vertex_external_index,surface_mesh))
-                               .halfedge_index_map  (get(CGAL::halfedge_external_index  ,surface_mesh))
-                               .edge_is_constrained_map(bem)
-                               .get_placement(Placement(bem))
-            );
-
-  std::cout << "\nFinished...\n" << r << " edges removed.\n"
-            << (surface_mesh.size_of_halfedges()/2) << " final edges.\n" ;
-
-  std::ofstream os( argc > 2 ? argv[2] : "out.off" ) ;
-  os.precision(17) ;
-  os << surface_mesh ;
+  std::cout << "Collapsing as many edges of mesh: " << filename << " as possible..." << std::endl;
+  int r = SMS::edge_collapse(surface_mesh,
+                             stop,
+                             CGAL::parameters::vertex_index_map(get(CGAL::vertex_external_index,surface_mesh))
+                                              .halfedge_index_map(get(CGAL::halfedge_external_index  ,surface_mesh))
+                                              .edge_is_constrained_map(bem)
+                                              .get_placement(Placement(bem)));
+
+  std::cout << "\nFinished!\n" << r << " edges removed.\n"
+            << (surface_mesh.size_of_halfedges()/2) << " final edges.\n";
+
+  std::ofstream os(argc > 2 ? argv[2] : "out.off");
+  os.precision(17);
+  os << surface_mesh;
 
   // now check!
-  for (Surface_mesh::Halfedge_iterator hit=surface_mesh.halfedges_begin(),
-                                       hit_end=surface_mesh.halfedges_end();
-                                       hit!=hit_end; ++hit )
+  for(Surface_mesh::Halfedge_iterator hit=surface_mesh.halfedges_begin(),
+                                      hit_end=surface_mesh.halfedges_end();
+                                      hit!=hit_end; ++hit)
   {
-    if (hit->is_border()){
+    if(hit->is_border())
+    {
       --nb_border_edges;
-      assert( constrained_edges[hit] ==
-              std::make_pair( hit->opposite()->vertex()->point(),
-                              hit->vertex()->point() ) );
+      assert(constrained_edges[hit] == std::make_pair(hit->opposite()->vertex()->point(),
+                                                      hit->vertex()->point()));
     }
   }
-  assert( nb_border_edges==0 );
 
-  return EXIT_SUCCESS ;
+  assert(nb_border_edges == 0);
+
+  return EXIT_SUCCESS;
 }
diff -pruN 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_constrained_border_surface_mesh.cmd 5.1-1/examples/Surface_mesh_simplification/edge_collapse_constrained_border_surface_mesh.cmd
--- 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_constrained_border_surface_mesh.cmd	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_simplification/edge_collapse_constrained_border_surface_mesh.cmd	1970-01-01 00:00:00.000000000 +0000
@@ -1 +0,0 @@
-mesh_with_border.off
diff -pruN 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_constrained_border_surface_mesh.cpp 5.1-1/examples/Surface_mesh_simplification/edge_collapse_constrained_border_surface_mesh.cpp
--- 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_constrained_border_surface_mesh.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_simplification/edge_collapse_constrained_border_surface_mesh.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -1,6 +1,3 @@
-#include <iostream>
-#include <fstream>
-
 #include <CGAL/Simple_cartesian.h>
 #include <CGAL/Surface_mesh.h>
 
@@ -16,107 +13,101 @@
 // Stop-condition policy
 #include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Count_stop_predicate.h>
 
-typedef CGAL::Simple_cartesian<double> Kernel;
-typedef Kernel::Point_3 Point_3;
+#include <iostream>
+#include <fstream>
+
+typedef CGAL::Simple_cartesian<double>                          Kernel;
+typedef Kernel::Point_3                                         Point_3;
 
-typedef CGAL::Surface_mesh<Point_3> Surface_mesh;
-typedef boost::graph_traits<Surface_mesh>::halfedge_descriptor halfedge_descriptor;
-typedef boost::graph_traits<Surface_mesh>::edge_descriptor edge_descriptor;
+typedef CGAL::Surface_mesh<Point_3>                             Surface_mesh;
+typedef boost::graph_traits<Surface_mesh>::halfedge_descriptor  halfedge_descriptor;
+typedef boost::graph_traits<Surface_mesh>::edge_descriptor      edge_descriptor;
 
 namespace SMS = CGAL::Surface_mesh_simplification;
 
-//
 // BGL property map which indicates whether an edge is marked as non-removable
-//
-struct Border_is_constrained_edge_map{
+struct Border_is_constrained_edge_map
+{
   const Surface_mesh* sm_ptr;
-  typedef edge_descriptor key_type;
-  typedef bool value_type;
-  typedef value_type reference;
-  typedef boost::readable_property_map_tag category;
-
-  Border_is_constrained_edge_map(const Surface_mesh& sm)
-    : sm_ptr(&sm)
-  {}
+  typedef edge_descriptor                                       key_type;
+  typedef bool                                                  value_type;
+  typedef value_type                                            reference;
+  typedef boost::readable_property_map_tag                      category;
+
+  Border_is_constrained_edge_map(const Surface_mesh& sm) : sm_ptr(&sm) {}
 
   friend bool get(Border_is_constrained_edge_map m, const key_type& edge) {
-    return  CGAL::is_border(edge, *m.sm_ptr);
+    return CGAL::is_border(edge, *m.sm_ptr);
   }
 };
 
-//
 // Placement class
-//
 typedef SMS::Constrained_placement<SMS::Midpoint_placement<Surface_mesh>,
                                    Border_is_constrained_edge_map > Placement;
 
-int main( int argc, char** argv )
+int main(int argc, char** argv)
 {
   Surface_mesh surface_mesh;
-
-  if (argc!=2){
-    std::cerr << "Usage: " << argv[0] << " input.off\n";
-    return EXIT_FAILURE;
-  }
-
-  std::ifstream is(argv[1]);
-  if(!is){
-    std::cerr << "Filename provided is invalid\n";
+  const char* filename = (argc > 1) ? argv[1] : "data/mesh_with_border.off";
+  std::ifstream is(filename);
+  if(!is || !(is >> surface_mesh))
+  {
+    std::cerr << "Failed to read input mesh: " << filename << std::endl;
     return EXIT_FAILURE;
   }
 
-  is >> surface_mesh ;
-  if (!CGAL::is_triangle_mesh(surface_mesh)){
+  if(!CGAL::is_triangle_mesh(surface_mesh))
+  {
     std::cerr << "Input geometry is not triangulated." << std::endl;
     return EXIT_FAILURE;
   }
 
-  Surface_mesh::Property_map<halfedge_descriptor,std::pair<Point_3, Point_3> > constrained_halfedges;
-
+  Surface_mesh::Property_map<halfedge_descriptor, std::pair<Point_3, Point_3> > constrained_halfedges;
   constrained_halfedges = surface_mesh.add_property_map<halfedge_descriptor,std::pair<Point_3, Point_3> >("h:vertices").first;
 
   std::size_t nb_border_edges=0;
-  for(halfedge_descriptor hd : halfedges(surface_mesh)){
-    if(CGAL::is_border(hd,surface_mesh)){
-      constrained_halfedges[hd] = std::make_pair(surface_mesh.point(source(hd,surface_mesh)),
-                                                 surface_mesh.point(target(hd,surface_mesh)));
+  for(halfedge_descriptor hd : halfedges(surface_mesh))
+  {
+    if(CGAL::is_border(hd, surface_mesh))
+    {
+      constrained_halfedges[hd] = std::make_pair(surface_mesh.point(source(hd, surface_mesh)),
+                                                 surface_mesh.point(target(hd, surface_mesh)));
       ++nb_border_edges;
     }
   }
 
   // Contract the surface mesh as much as possible
   SMS::Count_stop_predicate<Surface_mesh> stop(0);
-
   Border_is_constrained_edge_map bem(surface_mesh);
 
   // This the actual call to the simplification algorithm.
   // The surface mesh and stop conditions are mandatory arguments.
-  int r = SMS::edge_collapse
-            (surface_mesh
-            ,stop
-             ,CGAL::parameters::edge_is_constrained_map(bem)
-                               .get_placement(Placement(bem))
-            );
+  std::cout << "Collapsing as many edges of mesh: " << filename << " as possible..." << std::endl;
+  int r = SMS::edge_collapse(surface_mesh, stop,
+                             CGAL::parameters::edge_is_constrained_map(bem)
+                                              .get_placement(Placement(bem)));
 
-  std::cout << "\nFinished...\n" << r << " edges removed.\n"
+  std::cout << "\nFinished!\n" << r << " edges removed.\n"
             << surface_mesh.number_of_edges() << " final edges.\n";
 
-  std::ofstream os( argc > 2 ? argv[2] : "out.off" );
+  std::ofstream os(argc > 2 ? argv[2] : "out.off");
   os.precision(17);
   os << surface_mesh;
 
   // now check!
-  for(halfedge_descriptor hd : halfedges(surface_mesh)){
-    if(CGAL::is_border(hd,surface_mesh)){
+  for(halfedge_descriptor hd : halfedges(surface_mesh))
+  {
+    if(CGAL::is_border(hd,surface_mesh))
+    {
       --nb_border_edges;
-      if(constrained_halfedges[hd] != std::make_pair(surface_mesh.point(source(hd,surface_mesh)),
-                                                     surface_mesh.point(target(hd,surface_mesh)))){
+      if(constrained_halfedges[hd] != std::make_pair(surface_mesh.point(source(hd, surface_mesh)),
+                                                     surface_mesh.point(target(hd, surface_mesh))))
+      {
         std::cerr << "oops. send us a bug report\n";
       }
-
     }
   }
-  assert( nb_border_edges==0 );
+  assert(nb_border_edges==0);
 
   return EXIT_SUCCESS;
 }
diff -pruN 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_constrain_sharp_edges.cmd 5.1-1/examples/Surface_mesh_simplification/edge_collapse_constrain_sharp_edges.cmd
--- 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_constrain_sharp_edges.cmd	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_simplification/edge_collapse_constrain_sharp_edges.cmd	1970-01-01 00:00:00.000000000 +0000
@@ -1 +0,0 @@
-cube-meshed.off
diff -pruN 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_constrain_sharp_edges.cpp 5.1-1/examples/Surface_mesh_simplification/edge_collapse_constrain_sharp_edges.cpp
--- 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_constrain_sharp_edges.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_simplification/edge_collapse_constrain_sharp_edges.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -1,6 +1,3 @@
-#include <iostream>
-#include <fstream>
-
 #include <CGAL/Simple_cartesian.h>
 #include <CGAL/Polyhedron_3.h>
 
@@ -10,21 +7,25 @@
 #include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Count_stop_predicate.h>
 #include <CGAL/Unique_hash_map.h>
 #include <CGAL/property_map.h>
+
 #include <cmath>
+#include <iostream>
+#include <fstream>
 
-typedef CGAL::Simple_cartesian<double> Kernel;
-typedef Kernel::Point_3 Point_3;
-typedef CGAL::Polyhedron_3<Kernel> Surface_mesh;
-typedef boost::graph_traits<Surface_mesh>::vertex_descriptor vertex_descriptor;
-typedef boost::graph_traits<Surface_mesh>::halfedge_descriptor halfedge_descriptor;
-typedef boost::graph_traits<Surface_mesh>::edge_descriptor edge_descriptor;
-typedef boost::graph_traits<Surface_mesh>::edge_iterator edge_iterator;
+typedef CGAL::Simple_cartesian<double>                          Kernel;
+typedef Kernel::Point_3                                         Point_3;
+typedef CGAL::Polyhedron_3<Kernel>                              Surface_mesh;
+
+typedef boost::graph_traits<Surface_mesh>::vertex_descriptor    vertex_descriptor;
+typedef boost::graph_traits<Surface_mesh>::halfedge_descriptor  halfedge_descriptor;
+typedef boost::graph_traits<Surface_mesh>::edge_descriptor      edge_descriptor;
+typedef boost::graph_traits<Surface_mesh>::edge_iterator        edge_iterator;
 
-namespace SMS = CGAL::Surface_mesh_simplification ;
+namespace SMS = CGAL::Surface_mesh_simplification;
 
-//
 // BGL property map which indicates whether an edge is marked as non-removable
-struct Constrained_edge_map : public boost::put_get_helper<bool,Constrained_edge_map>
+struct Constrained_edge_map
+  : public boost::put_get_helper<bool, Constrained_edge_map>
 {
   typedef boost::readable_property_map_tag      category;
   typedef bool                                  value_type;
@@ -35,11 +36,9 @@ struct Constrained_edge_map : public boo
     : mConstraints(aConstraints)
   {}
 
-  reference operator[](key_type const& e) const { return  is_constrained(e); }
+  reference operator[](const key_type& e) const { return  is_constrained(e); }
 
-  bool is_constrained( key_type const& e ) const {
-    return mConstraints.is_defined(e);
-  }
+  bool is_constrained(const key_type& e) const { return mConstraints.is_defined(e); }
 
 private:
   const CGAL::Unique_hash_map<key_type,bool>& mConstraints;
@@ -47,8 +46,8 @@ private:
 
 bool is_border (edge_descriptor e, const Surface_mesh& sm)
 {
-  return (face(halfedge(e,sm),sm) == boost::graph_traits<Surface_mesh>::null_face() )
-    || (face(opposite(halfedge(e,sm),sm),sm) == boost::graph_traits<Surface_mesh>::null_face() );
+  return (face(halfedge(e,sm),sm) == boost::graph_traits<Surface_mesh>::null_face()) ||
+         (face(opposite(halfedge(e,sm),sm),sm) == boost::graph_traits<Surface_mesh>::null_face());
 }
 
 Point_3 point(vertex_descriptor vd,  const Surface_mesh& sm)
@@ -56,30 +55,24 @@ Point_3 point(vertex_descriptor vd,  con
   return get(CGAL::vertex_point, sm, vd);
 }
 
-int main( int argc, char** argv )
+int main(int argc, char** argv)
 {
-  CGAL::Unique_hash_map<edge_descriptor,bool> constraint_hmap(false);
-
   Surface_mesh surface_mesh;
-
-  if (argc < 2){
-    std::cerr << "Usage: " << argv[0] << " input.off [out.off]\n";
-    return EXIT_FAILURE;
-  }
-
-  std::ifstream is(argv[1]);
-  if(!is){
-    std::cerr << "Filename provided is invalid\n";
+  const char* filename = (argc > 1) ? argv[1] : "data/cube-meshed.off";
+  std::ifstream is(filename);
+  if(!is || !(is >> surface_mesh))
+  {
+    std::cerr << "Failed to read input mesh: " << filename << std::endl;
     return EXIT_FAILURE;
   }
 
-  is >> surface_mesh  ;
-
-  if (!CGAL::is_triangle_mesh(surface_mesh)){
+  if(!CGAL::is_triangle_mesh(surface_mesh))
+  {
     std::cerr << "Input geometry is not triangulated." << std::endl;
     return EXIT_FAILURE;
   }
 
+  CGAL::Unique_hash_map<edge_descriptor, bool> constraint_hmap(false);
   Constrained_edge_map constraints_map(constraint_hmap);
   SMS::Constrained_placement<SMS::Midpoint_placement<Surface_mesh>,
                              Constrained_edge_map > placement(constraints_map);
@@ -90,32 +83,34 @@ int main( int argc, char** argv )
   //          be far from the real value and could influence the detection of sharp
   //          edges after the simplification
   std::map<edge_descriptor,std::pair<Point_3, Point_3> >constrained_edges;
-  std::size_t nb_sharp_edges=0;
+  std::size_t nb_sharp_edges = 0;
 
   // detect sharp edges
   std::ofstream cst_output("constrained_edges.cgal");
-  edge_iterator eb,ee;
-  for(boost::tie(eb,ee) = edges(surface_mesh); eb != ee ; ++eb )
+  for(edge_descriptor ed : edges(surface_mesh))
   {
-    halfedge_descriptor hd = halfedge(*eb,surface_mesh);
-    if ( is_border(*eb,surface_mesh) ){
+    halfedge_descriptor hd = halfedge(ed, surface_mesh);
+    if(is_border(ed, surface_mesh))
+    {
       std::cerr << "border" << std::endl;
       ++nb_sharp_edges;
-      constraint_hmap[*eb]=true;
-      constrained_edges[*eb]=std::make_pair(point(source(hd,surface_mesh),surface_mesh),
-                                            point(target(hd,surface_mesh),surface_mesh));
+      constraint_hmap[ed] = true;
+      constrained_edges[ed] = std::make_pair(point(source(hd, surface_mesh), surface_mesh),
+                                             point(target(hd, surface_mesh), surface_mesh));
     }
-    else{
-      double angle = CGAL::approximate_dihedral_angle(point(target(opposite(hd,surface_mesh),surface_mesh),surface_mesh),
-                                                      point(target(hd,surface_mesh),surface_mesh),
-                                                      point(target(next(hd,surface_mesh),surface_mesh),surface_mesh),
-                                                      point(target(next(opposite(hd,surface_mesh),surface_mesh),surface_mesh),surface_mesh));
-      if ( CGAL::abs(angle)<100 ){
+    else
+    {
+      double angle = CGAL::approximate_dihedral_angle(point(target(opposite(hd, surface_mesh), surface_mesh), surface_mesh),
+                                                      point(target(hd, surface_mesh), surface_mesh),
+                                                      point(target(next(hd, surface_mesh), surface_mesh), surface_mesh),
+                                                      point(target(next(opposite(hd, surface_mesh), surface_mesh), surface_mesh), surface_mesh));
+      if(CGAL::abs(angle) < 100)
+      {
         ++nb_sharp_edges;
-        constraint_hmap[*eb]=true;
-        Point_3 p = point(source(hd,surface_mesh),surface_mesh);
-        Point_3 q = point(target(hd,surface_mesh),surface_mesh);
-        constrained_edges[*eb]=std::make_pair(p,q);
+        constraint_hmap[ed] = true;
+        Point_3 p = point(source(hd, surface_mesh), surface_mesh);
+        Point_3 q = point(target(hd, surface_mesh), surface_mesh);
+        constrained_edges[ed] = std::make_pair(p,q);
         cst_output << "2 " << p << " "  << q << "\n";
       }
     }
@@ -127,60 +122,59 @@ int main( int argc, char** argv )
   // Contract the surface mesh as much as possible
   SMS::Count_stop_predicate<Surface_mesh> stop(0);
 
-  int r
-  = SMS::edge_collapse(surface_mesh
-                       ,stop
-                       ,CGAL::parameters::vertex_index_map(get(CGAL::vertex_external_index, surface_mesh))
-                                         .halfedge_index_map(get(CGAL::halfedge_external_index, surface_mesh))
-                                         .edge_is_constrained_map(constraints_map)
-                                         .get_placement(placement)
-   );
-
-  std::cout << "\nFinished...\n" << r << " edges removed.\n"
-            << num_edges(surface_mesh) << " final edges.\n" ;
-  std::ofstream os(argc > 2 ? argv[2] : "out.off") ; os << surface_mesh ;
+  std::cout << "Collapsing as many non-sharp edges of mesh: " << filename << " as possible..." << std::endl;
+  int r = SMS::edge_collapse(surface_mesh, stop,
+                             CGAL::parameters::vertex_index_map(get(CGAL::vertex_external_index, surface_mesh))
+                                              .halfedge_index_map(get(CGAL::halfedge_external_index, surface_mesh))
+                                              .edge_is_constrained_map(constraints_map)
+                                              .get_placement(placement));
+
+  std::cout << "\nFinished!\n" << r << " edges removed.\n"
+            << num_edges(surface_mesh) << " final edges.\n";
+  std::ofstream os(argc > 2 ? argv[2] : "out.off"); os << surface_mesh;
 
   std::cout  << "Checking sharped edges were preserved...\n";
   // check sharp edges were preserved
-  for(boost::tie(eb,ee) = edges(surface_mesh); eb != ee ; ++eb )
+  for(edge_descriptor ed : edges(surface_mesh))
   {
-    halfedge_descriptor hd = halfedge(*eb,surface_mesh);
-    if ( is_border(*eb,surface_mesh) ){
+    halfedge_descriptor hd = halfedge(ed, surface_mesh);
+    if(is_border(ed, surface_mesh))
+    {
       --nb_sharp_edges;
-      assert(
-             constrained_edges[*eb]==std::make_pair( point(source(hd,surface_mesh),surface_mesh),
-                                                     point(target(hd,surface_mesh),surface_mesh)));
+      assert(constrained_edges[ed] == std::make_pair(point(source(hd, surface_mesh), surface_mesh),
+                                                     point(target(hd, surface_mesh), surface_mesh)));
     }
-    else{
-      double angle = approximate_dihedral_angle(point(target(opposite(hd,surface_mesh),surface_mesh),surface_mesh),
-                                                point(target(hd,surface_mesh),surface_mesh),
-                                                point(target(next(hd,surface_mesh),surface_mesh),surface_mesh),
-                                                point(target(next(opposite(hd,surface_mesh),surface_mesh),surface_mesh),surface_mesh));
-      if ( CGAL::abs(angle)<100 ){
+    else
+    {
+      double angle = approximate_dihedral_angle(point(target(opposite(hd, surface_mesh), surface_mesh), surface_mesh),
+                                                point(target(hd, surface_mesh), surface_mesh),
+                                                point(target(next(hd, surface_mesh), surface_mesh), surface_mesh),
+                                                point(target(next(opposite(hd, surface_mesh), surface_mesh), surface_mesh), surface_mesh));
+      if(CGAL::abs(angle) < 100)
+      {
         --nb_sharp_edges;
-      assert(
-        constrained_edges[*eb]==std::make_pair( point(source(hd,surface_mesh),surface_mesh),
-                                                point(target(hd,surface_mesh),surface_mesh)));
+        assert(constrained_edges[ed] == std::make_pair(point(source(hd, surface_mesh), surface_mesh),
+                                                       point(target(hd, surface_mesh), surface_mesh)));
       }
     }
   }
+
   std::cout  << "OK\n";
-  std::cerr << "# sharp edges = " << nb_sharp_edges << std::endl;
 
   std::cout << "Check that no removable edge has been forgotten..." << std::endl;
-  r = SMS::edge_collapse(surface_mesh
-                         ,stop
-                         ,CGAL::parameters::vertex_index_map(get(CGAL::vertex_external_index, surface_mesh))
-                                           .halfedge_index_map  (get(CGAL::halfedge_external_index, surface_mesh))
-                                           .edge_is_constrained_map(constraints_map)
-                                           .get_placement(placement)
-   );
+  r = SMS::edge_collapse(surface_mesh,
+                         stop,
+                         CGAL::parameters::vertex_index_map(get(CGAL::vertex_external_index, surface_mesh))
+                                          .halfedge_index_map(get(CGAL::halfedge_external_index, surface_mesh))
+                                          .edge_is_constrained_map(constraints_map)
+                                          .get_placement(placement)
+  );
 
-  assert(r==0);
+  assert(r == 0);
 
-  if ( r==0 )
+  if(r == 0) {
     std::cout << "OK\n";
-  else{
+  } else {
     std::cout << "ERROR! " << r << " edges removed!\n";
     return EXIT_FAILURE;
   }
diff -pruN 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_enriched_polyhedron.cmd 5.1-1/examples/Surface_mesh_simplification/edge_collapse_enriched_polyhedron.cmd
--- 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_enriched_polyhedron.cmd	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_simplification/edge_collapse_enriched_polyhedron.cmd	1970-01-01 00:00:00.000000000 +0000
@@ -1 +0,0 @@
-cube.off
diff -pruN 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_enriched_polyhedron.cpp 5.1-1/examples/Surface_mesh_simplification/edge_collapse_enriched_polyhedron.cpp
--- 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_enriched_polyhedron.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_simplification/edge_collapse_enriched_polyhedron.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -1,6 +1,3 @@
-#include <iostream>
-#include <fstream>
-
 #include <CGAL/Simple_cartesian.h>
 #include <CGAL/Polyhedron_3.h>
 // Extended polyhedron items which include an id() field
@@ -9,26 +6,33 @@
 #include <CGAL/Surface_mesh_simplification/edge_collapse.h>
 #include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Count_ratio_stop_predicate.h>
 
+#include <iostream>
+#include <fstream>
 
-typedef CGAL::Simple_cartesian<double> Kernel;
-typedef Kernel::Point_3 Point;
+typedef CGAL::Simple_cartesian<double>                              Kernel;
+typedef Kernel::Point_3                                             Point;
 
 // Setup an enriched polyhedron type which stores an id() field in the items
 typedef CGAL::Polyhedron_3<Kernel,CGAL::Polyhedron_items_with_id_3> Surface_mesh;
 
-typedef boost::graph_traits<Surface_mesh>::halfedge_descriptor halfedge_descriptor;
-typedef boost::graph_traits<Surface_mesh>::vertex_descriptor vertex_descriptor;
+typedef boost::graph_traits<Surface_mesh>::vertex_descriptor        vertex_descriptor;
+typedef boost::graph_traits<Surface_mesh>::halfedge_descriptor      halfedge_descriptor;
 
 namespace SMS = CGAL::Surface_mesh_simplification;
 
-
-int main( int argc, char** argv )
+int main(int argc, char** argv)
 {
   Surface_mesh surface_mesh;
+  const char* filename = (argc > 1) ? argv[1] : "data/cube.off";
+  std::ifstream is(filename);
+  if(!is || !(is >> surface_mesh))
+  {
+    std::cerr << "Failed to read input mesh: " << filename << std::endl;
+    return EXIT_FAILURE;
+  }
 
-  std::ifstream is(argv[1]);
-  is >> surface_mesh;
-  if (!CGAL::is_triangle_mesh(surface_mesh)){
+  if(!CGAL::is_triangle_mesh(surface_mesh))
+  {
     std::cerr << "Input geometry is not triangulated." << std::endl;
     return EXIT_FAILURE;
   }
@@ -39,35 +43,32 @@ int main( int argc, char** argv )
   // However, the Polyhedron_3 class doesn't assign any value to
   // this id(), so we must do it here:
   int index = 0;
-
-  for(halfedge_descriptor hd : halfedges(surface_mesh)){
+  for(halfedge_descriptor hd : halfedges(surface_mesh))
     hd->id() = index++;
-}
   index = 0;
 
-  for(vertex_descriptor vd : vertices(surface_mesh)){
+  for(vertex_descriptor vd : vertices(surface_mesh))
     vd->id() = index++;
-}
 
   // In this example, the simplification stops when the number of undirected edges
-  // drops below 10% of the initial count
-  SMS::Count_ratio_stop_predicate<Surface_mesh> stop(0.1);
-
+  // drops below xx% of the initial count
+  const double ratio = (argc > 2) ? std::stod(argv[2]) : 0.1;
+  SMS::Count_ratio_stop_predicate<Surface_mesh> stop(ratio);
 
   // The index maps are not explicitelty passed as in the previous
   // example because the surface mesh items have a proper id() field.
   // On the other hand, we pass here explicit cost and placement
   // function which differ from the default policies, ommited in
   // the previous example.
+  std::cout << "Collapsing edges of mesh: " << filename << ", aiming for " << 100 * ratio << "% of the input edges..." << std::endl;
   int r = SMS::edge_collapse(surface_mesh, stop);
 
-  std::cout << "\nFinished...\n" << r << " edges removed.\n"
+  std::cout << "\nFinished!\n" << r << " edges removed.\n"
             << (surface_mesh.size_of_halfedges()/2) << " final edges.\n";
 
-  std::ofstream os( argc > 2 ? argv[2] : "out.off" );
+  std::ofstream os((argc > 3) ? argv[3] : "out.off");
   os.precision(17);
   os << surface_mesh;
 
   return EXIT_SUCCESS;
 }
-
diff -pruN 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_garland_heckbert.cpp 5.1-1/examples/Surface_mesh_simplification/edge_collapse_garland_heckbert.cpp
--- 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_garland_heckbert.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_simplification/edge_collapse_garland_heckbert.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,76 @@
+#include <CGAL/Simple_cartesian.h>
+#include <CGAL/Surface_mesh.h>
+
+#include <CGAL/Surface_mesh_simplification/edge_collapse.h>
+#include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Count_ratio_stop_predicate.h>
+#include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Bounded_normal_change_placement.h>
+#include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/GarlandHeckbert_policies.h>
+
+#include <chrono>
+#include <iostream>
+#include <fstream>
+#include <vector>
+
+typedef CGAL::Simple_cartesian<double>                          Kernel;
+typedef Kernel::FT                                              FT;
+typedef Kernel::Point_3                                         Point_3;
+typedef CGAL::Surface_mesh<Point_3>                             Surface_mesh;
+
+namespace SMS = CGAL::Surface_mesh_simplification;
+
+int main(int argc, char** argv)
+{
+  Surface_mesh surface_mesh;
+  const char* filename = (argc > 1) ? argv[1] : "data/cube-meshed.off";
+  std::ifstream is(filename);
+  if(!is || !(is >> surface_mesh))
+  {
+    std::cerr << "Failed to read input mesh: " << filename << std::endl;
+    return EXIT_FAILURE;
+  }
+
+  if(!CGAL::is_triangle_mesh(surface_mesh))
+  {
+    std::cerr << "Input geometry is not triangulated." << std::endl;
+    return EXIT_FAILURE;
+  }
+
+  std::cout << "Input mesh has " << num_vertices(surface_mesh) << " nv "
+                                 << num_edges(surface_mesh) << " ne "
+                                 << num_faces(surface_mesh) << " nf" << std::endl;
+
+  std::chrono::steady_clock::time_point start_time = std::chrono::steady_clock::now();
+
+  const double ratio = (argc > 2) ? std::stod(argv[2]) : 0.2;
+  SMS::Count_ratio_stop_predicate<Surface_mesh> stop(ratio);
+
+  // Garland&Heckbert simplification policies
+  typedef typename SMS::GarlandHeckbert_policies<Surface_mesh, Kernel>          GH_policies;
+  typedef typename GH_policies::Get_cost                                        GH_cost;
+  typedef typename GH_policies::Get_placement                                   GH_placement;
+  typedef SMS::Bounded_normal_change_placement<GH_placement>                    Bounded_GH_placement;
+
+  GH_policies gh_policies(surface_mesh);
+  const GH_cost& gh_cost = gh_policies.get_cost();
+  const GH_placement& gh_placement = gh_policies.get_placement();
+  Bounded_GH_placement placement(gh_placement);
+
+  std::cout << "Collapsing edges of mesh: " << filename << ", aiming for " << 100 * ratio << "% of the input edges..." << std::endl;
+  int r = SMS::edge_collapse(surface_mesh, stop,
+                             CGAL::parameters::get_cost(gh_cost)
+                                              .get_placement(placement));
+
+  std::chrono::steady_clock::time_point end_time = std::chrono::steady_clock::now();
+
+  std::cout << "Time elapsed: "
+            << std::chrono::duration_cast<std::chrono::milliseconds>(end_time - start_time).count()
+            << "ms" << std::endl;
+
+  std::cout << "\nFinished!\n" << r << " edges removed.\n" << surface_mesh.number_of_edges() << " final edges.\n";
+
+  std::ofstream os(argc > 3 ? argv[3] : "out.off");
+  os.precision(17);
+  os << surface_mesh;
+
+  return EXIT_SUCCESS;
+}
diff -pruN 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_linear_cell_complex.cmd 5.1-1/examples/Surface_mesh_simplification/edge_collapse_linear_cell_complex.cmd
--- 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_linear_cell_complex.cmd	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_simplification/edge_collapse_linear_cell_complex.cmd	1970-01-01 00:00:00.000000000 +0000
@@ -1 +0,0 @@
-cube.off
diff -pruN 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_linear_cell_complex.cpp 5.1-1/examples/Surface_mesh_simplification/edge_collapse_linear_cell_complex.cpp
--- 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_linear_cell_complex.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_simplification/edge_collapse_linear_cell_complex.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -1,55 +1,64 @@
-#include <iostream>
-#include <fstream>
 #include <CGAL/boost/graph/graph_traits_Linear_cell_complex_for_combinatorial_map.h>
+
 // Simplification function
 #include <CGAL/Surface_mesh_simplification/edge_collapse.h>
+
 // Stop-condition policy
 #include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Count_stop_predicate.h>
 #include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_length_cost.h>
 #include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Midpoint_placement.h>
 
-typedef CGAL::Simple_cartesian<double> Kernel;
-typedef CGAL::Linear_cell_complex_traits<3, Kernel> MyTraits;
-typedef CGAL::Linear_cell_complex_for_bgl_combinatorial_map_helper
-         <2, 3, MyTraits>::type LCC;
-namespace SMS = CGAL::Surface_mesh_simplification ;
+#include <iostream>
+#include <fstream>
+
+typedef CGAL::Simple_cartesian<double>                                                   Kernel;
+typedef CGAL::Linear_cell_complex_traits<3, Kernel>                                      MyTraits;
+typedef CGAL::Linear_cell_complex_for_bgl_combinatorial_map_helper<2, 3, MyTraits>::type LCC;
+
+namespace SMS = CGAL::Surface_mesh_simplification;
 
-int main( int argc, char** argv )
+int main(int argc, char** argv)
 {
-  if (argc<2 || argc>3)
+  LCC lcc;
+  const char* filename = (argc > 1) ? argv[1] : "data/cube-meshed.off";
+  std::ifstream is(filename);
+  if(!is || !(CGAL::read_off(filename, lcc)))
   {
-    std::cout<<"Usage: simplification_Linear_cell_complex inofffile [outofffile]"<<std::endl;
+    std::cerr << "Failed to read input mesh: " << filename << std::endl;
     return EXIT_FAILURE;
   }
 
-  LCC lcc;
-  CGAL::read_off(argv[1], lcc);
+  if(!CGAL::is_triangle_mesh(lcc))
+  {
+    std::cerr << "Input geometry is not triangulated." << std::endl;
+    return EXIT_FAILURE;
+  }
 
-  lcc.display_characteristics(std::cout)<<", is_valid="<<CGAL::is_valid(lcc)<<std::endl;
+  lcc.display_characteristics(std::cout) << ", is_valid=" << CGAL::is_valid(lcc) << std::endl;
 
   // This is a stop predicate (defines when the algorithm terminates).
   // In this example, the simplification stops when the number of undirected edges
-  // left in the surface mesh drops below the specified number (1000)
-  SMS::Count_stop_predicate<LCC> stop(1000);
+  // left in the surface mesh drops below the specified number (1000 by default)
+  const std::size_t edge_count_treshold = (argc > 2) ? std::stoi(argv[2]) : 1000;
+  SMS::Count_stop_predicate<LCC> stop(edge_count_treshold);
+
   // This the actual call to the simplification algorithm.
   // The surface mesh and stop conditions are mandatory arguments.
   // The index maps are needed because the vertices and edges
   // of this surface mesh lack an "id()" field.
-  int r = SMS::edge_collapse
-    (lcc
-     ,stop
-     ,CGAL::parameters::halfedge_index_map(get(CGAL::halfedge_index, lcc))
-          .vertex_index_map(get(boost::vertex_index, lcc))
-          .get_cost(SMS::Edge_length_cost<LCC>())
-          .get_placement(SMS::Midpoint_placement<LCC>())
-     );
+  std::cout << "Collapsing edges of LCC: " << filename << ", aiming for " << edge_count_treshold << " final edges..." << std::endl;
+  int r = SMS::edge_collapse(lcc, stop,
+                             CGAL::parameters::halfedge_index_map(get(CGAL::halfedge_index, lcc))
+                                              .vertex_index_map(get(boost::vertex_index, lcc))
+                                              .get_cost(SMS::Edge_length_cost<LCC>())
+                                              .get_placement(SMS::Midpoint_placement<LCC>()));
+
+  std::cout << "\nFinished!\n" << r << " edges removed.\n"
+            << (lcc.number_of_darts()/2) << " final edges.\n";
 
-  std::cout << "\nFinished...\n" << r << " edges removed.\n"
-            << (lcc.number_of_darts()/2) << " final edges.\n" ;
+  lcc.display_characteristics(std::cout) << ", is_valid=" << CGAL::is_valid(lcc) << std::endl;
 
-  lcc.display_characteristics(std::cout)<<", is_valid="<<CGAL::is_valid(lcc)<<std::endl;
+  CGAL::write_off((argc > 3 ? argv[3] : "out.off"), lcc);
 
-  CGAL::write_off((argc > 2 ? argv[2] : "out.off"), lcc);
   return EXIT_SUCCESS;
 }
-// EOF //
diff -pruN 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_OpenMesh.cpp 5.1-1/examples/Surface_mesh_simplification/edge_collapse_OpenMesh.cpp
--- 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_OpenMesh.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_simplification/edge_collapse_OpenMesh.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -1,8 +1,3 @@
-#include <iostream>
-#include <fstream>
-
-#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
-
 #include <OpenMesh/Core/IO/MeshIO.hh>
 #include <OpenMesh/Core/Mesh/PolyMesh_ArrayKernelT.hh>
 
@@ -14,12 +9,12 @@
 #include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_length_cost.h>
 #include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Midpoint_placement.h>
 
-typedef CGAL::Exact_predicates_inexact_constructions_kernel K;
+#include <iostream>
+#include <fstream>
 
-typedef OpenMesh::PolyMesh_ArrayKernelT</* MyTraits*/> Surface_mesh;
+typedef OpenMesh::PolyMesh_ArrayKernelT</* default traits*/>     Surface_mesh;
 
-typedef boost::graph_traits<Surface_mesh>::edge_descriptor edge_descriptor;
-typedef boost::graph_traits<Surface_mesh>::edge_iterator edge_iterator;
+typedef boost::graph_traits<Surface_mesh>::edge_descriptor       edge_descriptor;
 
 class Constrained_edge_map
 {
@@ -51,51 +46,45 @@ private:
   OpenMesh::EPropHandleT<bool> constraint;
 };
 
+namespace SMS = CGAL::Surface_mesh_simplification;
 
-
-namespace SMS = CGAL::Surface_mesh_simplification ;
-
-int main( int argc, char** argv )
+int main(int argc, char** argv)
 {
   Surface_mesh surface_mesh;
   Constrained_edge_map constraints_map(surface_mesh);
-  if (argc==2)
-    OpenMesh::IO::read_mesh(surface_mesh, argv[1]);
-  else
-    OpenMesh::IO::read_mesh(surface_mesh, "cube.off");
 
-  if (!CGAL::is_triangle_mesh(surface_mesh)){
+  const char* filename = (argc > 1) ? argv[1] : "data/cube-meshed.off";
+  OpenMesh::IO::read_mesh(surface_mesh, filename);
+
+  if(!CGAL::is_triangle_mesh(surface_mesh)){
     std::cerr << "Input geometry is not triangulated." << std::endl;
     return EXIT_FAILURE;
   }
 
-  // For the pupose of the example we mark 10 edges as constrained edges
-  edge_iterator b,e;
-  int count=0;
-  for(boost::tie(b,e) = edges(surface_mesh); b!= e; ++b){
-      put(constraints_map,*b,(count++ <100));
-  }
+  // For the pupose of the example we mark 100 edges as constrained edges
+  int count = 0;
+  for(edge_descriptor e : edges(surface_mesh))
+    put(constraints_map, e, (count++ < 100));
+
   // This is a stop predicate (defines when the algorithm terminates).
   // In this example, the simplification stops when the number of undirected edges
   // left in the surface mesh drops below the specified number (1000)
-  SMS::Count_stop_predicate<Surface_mesh> stop(0);
+  const std::size_t stop_n = (argc > 2) ? std::stoi(argv[2]) : 1000;
+  SMS::Count_stop_predicate<Surface_mesh> stop(stop_n);
 
   // This the actual call to the simplification algorithm.
   // The surface mesh and stop conditions are mandatory arguments.
-
-  int r = SMS::edge_collapse
-            (surface_mesh
-            ,stop
-             ,CGAL::parameters::halfedge_index_map  (get(CGAL::halfedge_index  ,surface_mesh))
-                               .vertex_point_map(get(boost::vertex_point, surface_mesh))
-                               .edge_is_constrained_map(constraints_map)
-             );
+  std::cout << "Collapsing edges of mesh: " << filename << ", aiming for " << stop_n << " final edges..." << std::endl;
+  int r = SMS::edge_collapse(surface_mesh, stop,
+                             CGAL::parameters::halfedge_index_map(get(CGAL::halfedge_index,surface_mesh))
+                                              .vertex_point_map(get(boost::vertex_point, surface_mesh))
+                                              .edge_is_constrained_map(constraints_map));
 
   surface_mesh.garbage_collection();
-  std::cout << "\nFinished...\n" << r << " edges removed.\n"
-            << num_edges(surface_mesh) << " final edges.\n" ;
+  std::cout << "\nFinished!\n" << r << " edges removed.\n"
+            << num_edges(surface_mesh) << " final edges.\n";
 
-   OpenMesh::IO::write_mesh(surface_mesh, "out.off");
+  OpenMesh::IO::write_mesh(surface_mesh, "out.off");
 
   return EXIT_SUCCESS;
 }
diff -pruN 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_polyhedron.cmd 5.1-1/examples/Surface_mesh_simplification/edge_collapse_polyhedron.cmd
--- 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_polyhedron.cmd	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_simplification/edge_collapse_polyhedron.cmd	1970-01-01 00:00:00.000000000 +0000
@@ -1 +0,0 @@
-cube.off
diff -pruN 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_polyhedron.cpp 5.1-1/examples/Surface_mesh_simplification/edge_collapse_polyhedron.cpp
--- 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_polyhedron.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_simplification/edge_collapse_polyhedron.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -1,6 +1,3 @@
-#include <iostream>
-#include <fstream>
-
 #include <CGAL/Simple_cartesian.h>
 #include <CGAL/Polyhedron_3.h>
 
@@ -10,19 +7,27 @@
 // Stop-condition policy
 #include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Count_stop_predicate.h>
 
+#include <iostream>
+#include <fstream>
 
-typedef CGAL::Simple_cartesian<double> Kernel;
-typedef CGAL::Polyhedron_3<Kernel> Surface_mesh;
+typedef CGAL::Simple_cartesian<double>                      Kernel;
+typedef CGAL::Polyhedron_3<Kernel>                          Surface_mesh;
 
 namespace SMS = CGAL::Surface_mesh_simplification;
 
-int main( int argc, char** argv )
+int main(int argc, char** argv)
 {
   Surface_mesh surface_mesh;
+  const char* filename = (argc > 1) ? argv[1] : "data/cube.off";
+  std::ifstream is(filename);
+  if(!is || !(is >> surface_mesh))
+  {
+    std::cerr << "Failed to read input mesh: " << filename << std::endl;
+    return EXIT_FAILURE;
+  }
 
-  std::ifstream is(argv[1]);
-  is >> surface_mesh;
-  if (!CGAL::is_triangle_mesh(surface_mesh)){
+  if(!CGAL::is_triangle_mesh(surface_mesh))
+  {
     std::cerr << "Input geometry is not triangulated." << std::endl;
     return EXIT_FAILURE;
   }
@@ -30,23 +35,22 @@ int main( int argc, char** argv )
   // This is a stop predicate (defines when the algorithm terminates).
   // In this example, the simplification stops when the number of undirected edges
   // left in the surface mesh drops below the specified number (1000)
-  SMS::Count_stop_predicate<Surface_mesh> stop(1000);
+  const std::size_t edge_count_treshold = (argc > 2) ? std::stoi(argv[2]) : 1000;
+  SMS::Count_stop_predicate<Surface_mesh> stop(edge_count_treshold);
 
   // This the actual call to the simplification algorithm.
   // The surface mesh and stop conditions are mandatory arguments.
   // The index maps are needed because the vertices and edges
   // of this surface mesh lack an "id()" field.
-  int r = SMS::edge_collapse
-            (surface_mesh
-            ,stop
-             ,CGAL::parameters::vertex_index_map(get(CGAL::vertex_external_index, surface_mesh))
-                               .halfedge_index_map  (get(CGAL::halfedge_external_index, surface_mesh))
-            );
+  std::cout << "Collapsing edges of Polyhedron: " << filename << ", aiming for " << edge_count_treshold << " final edges..." << std::endl;
+  int r = SMS::edge_collapse(surface_mesh, stop,
+                             CGAL::parameters::vertex_index_map(get(CGAL::vertex_external_index, surface_mesh))
+                                              .halfedge_index_map(get(CGAL::halfedge_external_index, surface_mesh)));
 
-  std::cout << "\nFinished...\n" << r << " edges removed.\n"
+  std::cout << "\nFinished!\n" << r << " edges removed.\n"
             << (surface_mesh.size_of_halfedges()/2) << " final edges.\n";
 
-  std::ofstream os( argc > 2 ? argv[2] : "out.off" );
+  std::ofstream os(argc > 3 ? argv[3] : "out.off");
   os.precision(17);
   os << surface_mesh;
 
diff -pruN 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_surface_mesh.cmd 5.1-1/examples/Surface_mesh_simplification/edge_collapse_surface_mesh.cmd
--- 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_surface_mesh.cmd	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_simplification/edge_collapse_surface_mesh.cmd	1970-01-01 00:00:00.000000000 +0000
@@ -1 +0,0 @@
-cube.off
diff -pruN 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_surface_mesh.cpp 5.1-1/examples/Surface_mesh_simplification/edge_collapse_surface_mesh.cpp
--- 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_surface_mesh.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_simplification/edge_collapse_surface_mesh.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -1,39 +1,51 @@
-#include <iostream>
-#include <fstream>
-
 #include <CGAL/Simple_cartesian.h>
 #include <CGAL/Surface_mesh.h>
+
 #include <CGAL/Surface_mesh_simplification/edge_collapse.h>
 #include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Count_ratio_stop_predicate.h>
 
-typedef CGAL::Simple_cartesian<double> Kernel;
-typedef Kernel::Point_3 Point_3;
-typedef CGAL::Surface_mesh<Point_3> Surface_mesh;
+#include <chrono>
+#include <fstream>
+#include <iostream>
+
+typedef CGAL::Simple_cartesian<double>               Kernel;
+typedef Kernel::Point_3                              Point_3;
+typedef CGAL::Surface_mesh<Point_3>                  Surface_mesh;
 
 namespace SMS = CGAL::Surface_mesh_simplification;
 
-
-int main( int argc, char** argv )
+int main(int argc, char** argv)
 {
   Surface_mesh surface_mesh;
+  const char* filename = (argc > 1) ? argv[1] : "data/cube-meshed.off";
+  std::ifstream is(filename);
+  if(!is || !(is >> surface_mesh))
+  {
+    std::cerr << "Failed to read input mesh: " << filename << std::endl;
+    return EXIT_FAILURE;
+  }
 
-  std::ifstream is(argv[1]);
-  is >> surface_mesh;
-  if (!CGAL::is_triangle_mesh(surface_mesh)){
+  if(!CGAL::is_triangle_mesh(surface_mesh))
+  {
     std::cerr << "Input geometry is not triangulated." << std::endl;
     return EXIT_FAILURE;
   }
 
+  std::chrono::steady_clock::time_point start_time = std::chrono::steady_clock::now();
+
   // In this example, the simplification stops when the number of undirected edges
   // drops below 10% of the initial count
-  SMS::Count_ratio_stop_predicate<Surface_mesh> stop(0.1);
+  double stop_ratio = (argc > 2) ? std::stod(argv[2]) : 0.1;
+  SMS::Count_ratio_stop_predicate<Surface_mesh> stop(stop_ratio);
 
   int r = SMS::edge_collapse(surface_mesh, stop);
 
-  std::cout << "\nFinished...\n" << r << " edges removed.\n"
-            << surface_mesh.number_of_edges() << " final edges.\n";
+  std::chrono::steady_clock::time_point end_time = std::chrono::steady_clock::now();
+
+  std::cout << "\nFinished!\n" << r << " edges removed.\n" << surface_mesh.number_of_edges() << " final edges.\n";
+  std::cout << "Time elapsed: " << std::chrono::duration_cast<std::chrono::milliseconds>(end_time - start_time).count() << "ms" << std::endl;
 
-  std::ofstream os( argc > 2 ? argv[2] : "out.off" );
+  std::ofstream os(argc > 3 ? argv[3] : "out.off");
   os.precision(17);
   os << surface_mesh;
 
diff -pruN 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_visitor_surface_mesh.cmd 5.1-1/examples/Surface_mesh_simplification/edge_collapse_visitor_surface_mesh.cmd
--- 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_visitor_surface_mesh.cmd	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_simplification/edge_collapse_visitor_surface_mesh.cmd	1970-01-01 00:00:00.000000000 +0000
@@ -1 +0,0 @@
-cube.off
diff -pruN 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_visitor_surface_mesh.cpp 5.1-1/examples/Surface_mesh_simplification/edge_collapse_visitor_surface_mesh.cpp
--- 5.0.3-1/examples/Surface_mesh_simplification/edge_collapse_visitor_surface_mesh.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_simplification/edge_collapse_visitor_surface_mesh.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -1,6 +1,3 @@
-#include <iostream>
-#include <fstream>
-
 #include <CGAL/Simple_cartesian.h>
 #include <CGAL/Surface_mesh.h>
 
@@ -13,18 +10,20 @@
 // Stop-condition policy
 #include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Count_ratio_stop_predicate.h>
 
-typedef CGAL::Simple_cartesian<double> Kernel;
-typedef Kernel::Point_3 Point_3;
+#include <iostream>
+#include <fstream>
 
-typedef CGAL::Surface_mesh<Point_3> Surface_mesh;
+typedef CGAL::Simple_cartesian<double>                                  Kernel;
+typedef Kernel::Point_3                                                 Point_3;
 
-typedef boost::graph_traits<Surface_mesh>::halfedge_descriptor halfedge_descriptor;
-typedef boost::graph_traits<Surface_mesh>::vertex_descriptor vertex_descriptor;
+typedef CGAL::Surface_mesh<Point_3>                                     Surface_mesh;
 
-namespace SMS = CGAL::Surface_mesh_simplification;
+typedef boost::graph_traits<Surface_mesh>::halfedge_descriptor          halfedge_descriptor;
+typedef boost::graph_traits<Surface_mesh>::vertex_descriptor            vertex_descriptor;
 
-typedef SMS::Edge_profile<Surface_mesh> Profile;
+namespace SMS = CGAL::Surface_mesh_simplification;
 
+typedef SMS::Edge_profile<Surface_mesh>                                 Profile;
 
 // The following is a Visitor that keeps track of the simplification process.
 // In this example the progress is printed real-time and a few statistics are
@@ -32,96 +31,90 @@ typedef SMS::Edge_profile<Surface_mesh>
 //
 struct Stats
 {
-  Stats()
-    : collected(0)
-    , processed(0)
-    , collapsed(0)
-    , non_collapsable(0)
-    , cost_uncomputable(0)
-    , placement_uncomputable(0)
-  {}
-
-  std::size_t collected;
-  std::size_t processed;
-  std::size_t collapsed;
-  std::size_t non_collapsable;
-  std::size_t cost_uncomputable ;
-  std::size_t placement_uncomputable;
+  std::size_t collected = 0;
+  std::size_t processed = 0;
+  std::size_t collapsed = 0;
+  std::size_t non_collapsable = 0;
+  std::size_t cost_uncomputable = 0;
+  std::size_t placement_uncomputable = 0;
 };
 
 struct My_visitor : SMS::Edge_collapse_visitor_base<Surface_mesh>
 {
-  My_visitor( Stats* s) : stats(s){}
+  My_visitor(Stats* s) : stats(s) {}
 
   // Called during the collecting phase for each edge collected.
-  void OnCollected( Profile const&, boost::optional<double> const& )
+  void OnCollected(const Profile&, const boost::optional<double>&)
   {
-    ++ stats->collected;
+    ++(stats->collected);
     std::cerr << "\rEdges collected: " << stats->collected << std::flush;
   }
 
   // Called during the processing phase for each edge selected.
   // If cost is absent the edge won't be collapsed.
-  void OnSelected(Profile const&
-                 ,boost::optional<double> cost
-                 ,std::size_t             initial
-                 ,std::size_t             current
-                 )
-  {
-    ++ stats->processed;
-    if ( !cost )
-      ++ stats->cost_uncomputable;
+  void OnSelected(const Profile&,
+                  boost::optional<double> cost,
+                  std::size_t initial,
+                  std::size_t current)
+  {
+    ++(stats->processed);
+    if(!cost)
+      ++(stats->cost_uncomputable);
 
-    if ( current == initial )
+    if(current == initial)
       std::cerr << "\n" << std::flush;
     std::cerr << "\r" << current << std::flush;
   }
 
   // Called during the processing phase for each edge being collapsed.
   // If placement is absent the edge is left uncollapsed.
-  void OnCollapsing(Profile const&
-                   ,boost::optional<Point>  placement
-                   )
+  void OnCollapsing(const Profile&,
+                    boost::optional<Point> placement)
   {
-    if ( !placement )
-      ++ stats->placement_uncomputable;
+    if(!placement)
+      ++(stats->placement_uncomputable);
   }
 
   // Called for each edge which failed the so called link-condition,
   // that is, which cannot be collapsed because doing so would
   // turn the surface mesh into a non-manifold.
-  void OnNonCollapsable( Profile const& )
+  void OnNonCollapsable(const Profile&)
   {
-    ++ stats->non_collapsable;
+    ++(stats->non_collapsable);
   }
 
   // Called after each edge has been collapsed
-  void OnCollapsed( Profile const&, vertex_descriptor )
+  void OnCollapsed(const Profile&, vertex_descriptor)
   {
-    ++ stats->collapsed;
+    ++(stats->collapsed);
   }
 
   Stats* stats;
 };
 
-
-int main( int argc, char** argv )
+int main(int argc, char** argv)
 {
   Surface_mesh surface_mesh;
+  const char* filename = (argc > 1) ? argv[1] : "data/cube.off";
+  std::ifstream is(filename);
+  if(!is || !(is >> surface_mesh))
+  {
+    std::cerr << "Failed to read input mesh: " << filename << std::endl;
+    return EXIT_FAILURE;
+  }
 
-  std::ifstream is(argv[1]);
-  is >> surface_mesh;
-  if (!CGAL::is_triangle_mesh(surface_mesh)){
+  if(!CGAL::is_triangle_mesh(surface_mesh))
+  {
     std::cerr << "Input geometry is not triangulated." << std::endl;
     return EXIT_FAILURE;
   }
 
   // In this example, the simplification stops when the number of undirected edges
-  // drops below 10% of the initial count
-  SMS::Count_ratio_stop_predicate<Surface_mesh> stop(0.1);
+  // drops below xx% of the initial count
+  const double ratio = (argc > 2) ? std::stod(argv[2]) : 0.1;
+  SMS::Count_ratio_stop_predicate<Surface_mesh> stop(ratio);
 
   Stats stats;
-
   My_visitor vis(&stats);
 
   // The index maps are not explicitelty passed as in the previous
@@ -129,11 +122,7 @@ int main( int argc, char** argv )
   // On the other hand, we pass here explicit cost and placement
   // function which differ from the default policies, ommited in
   // the previous example.
-  int r = SMS::edge_collapse
-           (surface_mesh
-           ,stop
-           ,CGAL::parameters::visitor(vis)
-           );
+  int r = SMS::edge_collapse(surface_mesh, stop, CGAL::parameters::visitor(vis));
 
   std::cout << "\nEdges collected: "  << stats.collected
             << "\nEdges proccessed: " << stats.processed
@@ -144,10 +133,10 @@ int main( int argc, char** argv )
             << "\nEdge not collapsed due to placement computation constraints: " << stats.placement_uncomputable
             << std::endl;
 
-  std::cout << "\nFinished...\n" << r << " edges removed.\n"
+  std::cout << "\nFinished!\n" << r << " edges removed.\n"
             << surface_mesh.number_of_edges() << " final edges.\n";
 
-  std::ofstream os( argc > 2 ? argv[2] : "out.off" );
+  std::ofstream os(argc > 3 ? argv[3] : "out.off");
   os.precision(17);
   os << surface_mesh;
 
diff -pruN 5.0.3-1/examples/Surface_mesh_simplification/mesh_with_border.off 5.1-1/examples/Surface_mesh_simplification/mesh_with_border.off
--- 5.0.3-1/examples/Surface_mesh_simplification/mesh_with_border.off	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_simplification/mesh_with_border.off	1970-01-01 00:00:00.000000000 +0000
@@ -1,1566 +0,0 @@
-OFF
-548 1014 0
-
-92.0896999051 77.41989209583 -1.916832684148
-86.39100962126 72.78664496906 15.78278094865
-77.09376525879 89.87973022461 14.55043506622
-90.98168945312 96.21558380127 3.65913438797
-82.71570084509 81.86220870549 16.44397321995
-92.31691908737 86.77964878854 1.043011456447
-89.37290527712 74.91208629563 6.905433125042
-84.03774261475 93.04766845703 9.104772567749
-87.91614428888 83.98367629501 8.885449941462
-92.08308562722 91.51009411435 2.561394143404
-79.93436140773 85.95867809642 15.71871673187
-92.16810072909 82.08661851225 -0.4813870638146
-84.95054383138 77.42281772836 16.37560573549
-84.22521839805 85.24324921259 12.54660366583
-87.28833145411 78.56473572586 11.06770318225
-88.53858617073 89.67622996838 7.078736038751
-90.21472024103 80.41696412654 4.846493577097
-87.81428016343 73.7864628996 11.33105836088
-90.80739519912 76.15358418737 2.490105437257
-90.23304320467 85.72684363719 5.157361043466
-80.56578826904 91.463722229 11.8276386261
-87.50969696045 94.63160705566 6.381914138794
-85.28558449034 88.85493917666 10.09354883143
-85.27758279004 81.37161616953 13.2681149885
-82.42035268114 88.37159009099 12.28321454594
-88.64910272783 80.44739317924 8.19157083558
-91.34705696923 83.71941907076 2.518356602206
-86.54332632431 76.21153148326 13.50411389672
-91.67848926288 93.87171420986 3.215234177501
-92.10816413641 79.75050243058 -1.206542332568
-92.29622765622 89.13884414744 1.826720300829
-89.70542995812 92.32964625679 5.425094029631
-90.59106423297 89.29868580891 4.596301969103
-91.17972407486 79.36567116545 1.91073903372
-86.33903036171 91.18877364247 8.487076777961
-87.52474663172 87.12023386306 8.676494963161
-78.49607673255 87.91733917969 15.15035040429
-86.99901132535 81.62634788796 10.68441922028
-88.58945681826 74.33088989454 9.115347731003
-88.62578378798 77.40042441075 8.552444264328
-89.49626856147 82.8803041145 6.515565583687
-90.11337785606 75.52304884182 4.696506235514
-82.30175018311 92.25574493408 10.46619224548
-86.19309520102 85.47015639124 10.53423349659
-83.62519737065 83.26930177161 14.30213272629
-82.0111827497 86.17046138772 13.90531750866
-85.62502595062 78.76016915029 14.06135119224
-78.82978057861 90.67166137695 13.1890039444
-89.45795733185 78.72045615552 6.562115009714
-85.6876237692 83.36697320818 11.86120221103
-90.44759176655 78.12728045561 3.939338198359
-80.48132290292 88.08344582991 13.73552924292
-81.36934673359 83.95779586913 16.1769257545
-85.75871853097 75.11699965987 16.11420674811
-83.88932229025 79.65919077394 16.47804590138
-92.26242604584 84.4270568131 0.271220829177
-87.82876868723 76.19362041729 10.58641505821
-87.06673968633 73.27518730634 13.55175677477
-91.45894636523 76.78940059593 0.2867129567336
-89.1360658561 87.2730913458 6.776971477474
-91.27271853367 87.17795258214 3.336373635784
-87.86365825812 92.32249801315 6.989370325119
-89.24565124512 95.42362213135 5.020479202271
-85.77369689941 93.83967590332 7.74334526062
-84.1530047653 90.8891150393 10.01020383373
-84.7473737152 87.01795663413 11.27019134425
-90.49631836036 82.69742911034 4.431726418335
-89.14774062234 85.0408944843 6.999973531134
-91.46289377915 81.61758663607 1.630732294048
-82.3132325122 90.30474632652 11.36096779886
-86.90111626841 89.15160207096 8.72203909917
-84.11950356501 81.16699817827 15.0140687604
-86.28939352036 79.79002703214 12.40403490101
-80.46573968642 89.74182843265 12.71429924485
-89.60865143194 76.76812793407 6.178829509292
-90.89270834008 91.26374173479 4.176033399443
-86.80521922435 83.70802005279 10.38215960895
-89.48178847126 80.95757614811 6.534276165285
-88.11169455465 79.04731345898 9.395346300457
-88.583493635 82.36236771915 8.124783175307
-90.84324437988 81.26551998519 3.330565985771
-83.87247641686 88.5955192875 11.20891472132
-83.19459245527 86.7501647852 12.6283671037
-90.40385047626 87.51383281538 4.886414759175
-86.16951796704 87.23207527998 9.953337633655
-91.60399930071 85.59586026518 2.336331753486
-90.75668901041 84.4323814451 4.044852809319
-87.29901191592 75.34268635555 12.09818461047
-86.57400034054 77.89258327773 12.67355564934
-85.79080099281 76.54570082726 15.07525566634
-91.77179896274 80.57408744558 0.281974535061
-91.5475556147 89.95641279222 3.1931760263
-90.3442663496 93.92285130091 4.585233593477
-86.45794665313 74.70284569722 14.49144487954
-79.05721494196 89.10127736985 13.9682290256
-92.09571263016 78.58472307811 -1.56269428567
-91.61324652266 78.65370278208 0.2413144683142
-88.95158085097 93.87160551314 5.76487891664
-88.93172774873 91.16524184997 6.380833459336
-83.1217686748 84.84727135728 13.80594048831
-89.60708732727 88.78765137181 5.981576801907
-90.92546945959 77.800373802 2.393472143492
-89.88360646754 84.27551327628 5.789320637377
-87.82352645612 80.66325786631 9.616310711501
-80.65753881345 84.96774500172 15.96810193465
-81.91399561614 84.87065850033 14.91313114612
-90.704329777 79.64984094159 3.428700832615
-84.55592036845 83.70275449477 13.0115763795
-92.21273042203 83.25630400946 -0.1087730360537
-88.21238360604 88.22000875015 7.707421885366
-88.15959122892 85.57676575574 8.297069621225
-88.81394466763 78.89692552288 7.991139144449
-87.59941965207 82.59759096357 9.584046736566
-86.06838273588 81.83162256321 11.96328549717
-87.03192126414 80.10893594721 11.0553375274
-88.46537703634 75.92217371895 9.137222873303
-87.94183853539 77.63062238945 10.0069676505
-84.73188438726 79.65427763894 15.09106477496
-89.99572259739 90.6013158789 5.326148605559
-89.08109552443 76.28510581649 7.571875818205
-87.45358755135 90.32287833068 7.925755467503
-85.80653793821 90.02205488241 9.287945386385
-84.90573883057 93.44362640381 8.424060821533
-85.02462957381 91.82100314902 9.080210762303
-82.53828964184 83.53532004812 15.31098692782
-87.24565314739 85.21590788642 9.462906429098
-88.76907792079 83.77269954437 7.686026722444
-87.26020862849 77.16918094688 11.56186126234
-79.21183662051 86.93976318432 15.44173627227
-80.6772357635 86.72750384054 14.55134866499
-89.96968019319 81.91329385452 5.544516060987
-91.04107866369 92.74354582432 3.959510149799
-89.93672918584 77.98593262991 5.347731087816
-91.02614651348 85.78683581428 3.633030004003
-85.13548572186 85.65452792654 11.50140805416
-86.58175312226 92.53969079587 7.807716427241
-91.73194481166 83.03368102701 1.268949655066
-91.6054686603 88.52647150388 2.928124991176
-82.0440222159 87.34001414513 13.1416907498
-85.52842652024 80.03896830107 13.54193677916
-85.36404016149 77.88755226405 15.12055517891
-86.0765084065 77.46013649534 13.89021585768
-90.46661968935 75.83685629421 3.592188671691
-90.19681065692 76.89784761499 4.538397099543
-84.56138644799 82.48691782071 13.65137294073
-83.30403327714 82.36071585563 15.29489338667
-86.64169311523 94.23564147949 7.062632083893
-89.15075042396 77.62070306986 7.311016382239
-91.77578816761 77.10568331543 -0.8146003184522
-92.31458355115 87.95769882393 1.431302885641
-84.27631718922 89.64398620348 10.47724464253
-91.16950185257 82.39458802729 2.677383847529
-86.87516040998 74.96862991509 13.27991264815
-89.18314931403 79.84626795266 7.164286304372
-84.44360901566 78.54784372864 16.4455387666
-85.99928322555 84.35958928423 11.12543897065
-90.96268806848 88.23215442693 3.99433103563
-91.79917398165 86.86770228776 2.211270347125
-91.35660160611 95.04584327368 3.454739655752
-90.11364746094 95.81958770752 4.33976650238
-91.8114226106 84.52104497723 1.537204226449
-89.92268062963 79.29372508096 5.47162553997
-87.80022427991 74.99940383723 10.98461017787
-88.98391421327 74.61712962187 8.011383213137
-88.41189669657 86.76113359695 7.762849859456
-89.83990585597 86.78305575769 5.804147791076
-86.47708307705 82.70982487065 11.10320850079
-85.05990301504 84.41623279402 12.11437941264
-89.32975770952 89.98303370895 6.172890167703
-87.43364445809 73.52682079937 12.44056066412
-81.26149028386 88.68493750348 12.8445643021
-79.59530025292 87.7552167202 14.55929982062
-81.56095704279 89.65865012342 12.12054795257
-83.74650335188 87.5173532649 11.81482990264
-82.05580562551 82.9262281906 16.33684179161
-77.79110462112 88.89988096489 14.85615961682
-85.38491825728 76.27397332034 16.26002634304
-83.30952273933 80.76277415836 16.47482934558
-86.09057899059 73.95418022421 15.9528063478
-92.22202721814 90.32419016346 2.207048185808
-92.13015228425 80.91751070159 -0.8477875752077
-91.91128748784 92.69412778407 2.905822503365
-92.30352960777 85.60185389102 0.6571861560139
-88.80397882864 92.35845957816 6.230165128969
-87.56350554293 93.48546833814 6.794415435958
-86.77515477948 87.99073532884 9.163124693164
-91.2921995218 77.76012886914 1.141073203928
-87.24563285669 91.51266122799 7.717013849969
-91.05693732674 80.38765116816 2.536987922097
-88.09868991582 81.66739940773 8.994784785804
-88.20079182886 74.0545900789 10.22315122432
-91.13838295799 76.47160421784 1.38837785964
-89.74903229875 75.21462640485 5.800678292534
-86.7248444178 73.03032641041 14.66755655857
-91.42198188351 80.44639618037 1.409354415978
-86.70248244291 86.20631702261 9.772311187611
-85.32357668702 82.51604988631 12.67539856556
-83.34415171148 90.3563044528 10.73296744307
-83.16972351074 92.65168762207 9.785503387451
-83.16370643034 91.45261184562 10.33452960078
-88.3776473999 95.02765655518 5.701191902161
-79.6978225708 91.06776428223 12.50832748413
-81.43376159668 91.85966491699 11.14694881439
-77.96181488037 90.27571105957 13.8696937561
-82.88320612077 89.23701510152 11.54197182646
-90.59709590487 77.0431090884 3.326233951924
-90.4385053569 81.50740377165 4.417289906702
-90.92399955002 83.20459553336 3.474223453509
-90.04153909139 83.12197296545 5.464420913909
-84.84455473976 80.53727723718 14.34532209905
-88.97115981828 81.39918470952 7.526398209826
-88.19184958435 90.76480154662 7.16373228865
-88.95207671701 88.40527714282 6.846825372554
-86.85188444752 76.78765884766 12.56411815235
-91.82868788219 82.06718063471 0.6263483943344
-87.83397570175 89.26724425275 7.860215245696
-85.63889122331 86.49196601989 10.70444370687
-84.289256929 86.25422337269 12.00150866214
-85.28132021495 87.75990875058 10.52113336767
-87.61493912667 79.70302658741 10.17261837774
-89.34878250085 83.96799192039 6.745304265489
-89.71060304788 85.34737396029 6.077644732881
-89.04670719873 86.17050776147 7.033811940758
-83.33616210627 85.71360239737 13.08930299415
-80.90981870091 85.73270169067 15.12615535105
-91.21497714171 84.80822967201 3.070330870078
-90.64792720154 86.4992805173 4.432689133307
-90.32498894608 79.39071047162 4.441071424401
-86.59113065075 81.0079681309 11.48087340359
-88.20529733344 75.14799817208 9.928778947458
-91.46932714668 91.82413160531 3.426693900819
-84.05625152588 92.04026031494 9.551273345947
-81.34168250145 90.67680250169 11.74296930792
-91.55259019351 79.61359965956 0.734537667934
-86.14928903969 88.82644504097 9.435758125875
-86.63480151744 90.17649516787 8.626415734013
-85.61112519056 92.54511608244 8.415443520635
-90.77235413367 78.64450546717 3.044471825873
-86.03015612476 80.84246483408 12.39859916442
-84.09084387379 80.31011079893 15.65381057678
-87.42963074907 86.1429197771 9.024362828801
-84.58304230312 81.54684584424 14.13916022685
-82.71823885608 84.27191904384 14.58204533128
-86.14318337822 78.875724971 13.05390603302
-87.26466369629 79.24705505371 11.27718162537
-87.46308063449 88.31600007416 8.437690516399
-81.19461313203 87.2774180299 13.78930881295
-90.33868384664 83.96273511603 4.894040729647
-86.24349253624 75.76941990672 14.45146228254
-89.62070466303 79.96751622986 6.215618617518
-89.86045031553 87.81767110116 5.714444063781
-90.32493653663 92.86181087998 4.742895505028
-91.83660492574 79.70232495475 -0.2327343256045
-87.5598870147 81.60536240291 9.845685702366
-90.24277677565 91.55416058234 4.951816512248
-90.62251838342 90.33539527205 4.550076062657
-88.3684628344 84.57954172437 8.178119845641
-84.91706095676 90.18792368357 9.834589545214
-83.9286208171 84.51052198464 13.22924134799
-89.42617545693 75.90524494088 6.691736706803
-89.91402634483 76.30110923329 5.324186755154
-89.57647684262 77.76935141695 6.265239627438
-90.26302274 88.48759436779 5.08800413955
-85.56081184425 90.97427099583 9.095988527362
-80.00626334278 89.03147537546 13.41898396371
-79.37286329558 89.82291469539 13.32549985972
-88.14755221554 76.67312076792 9.744962597196
-89.65531497262 93.32206911079 5.323482453556
-91.22225608436 89.12207284975 3.629908520633
-83.96795217196 82.30680594897 14.49836357631
-83.45877795223 81.4494251123 15.73631635087
-89.94620514167 89.59487371469 5.467634763391
-82.29238891602 91.29810333252 10.90618038177
-90.811639497 82.22302301049 3.582510020635
-80.52007293701 90.59206390381 12.24966716766
-88.29109077723 78.09038216257 9.190419558197
-87.71177444011 78.74996062053 10.22093064536
-88.46730306671 79.57337952733 8.628829838468
-88.33761245464 83.26134698128 8.394543381393
-89.36645737065 81.92077744538 6.769106527411
-89.89907582446 80.92451760593 5.635455680689
-89.00589702596 82.86301063417 7.377557640286
-90.59027668966 94.87526374646 4.171108393016
-87.42512737492 76.17020838692 11.52442944207
-91.02426936631 93.72302643884 3.916591269877
-89.8870358383 94.74290090133 4.809697443233
-87.05838461737 82.52158227934 10.3681851657
-86.56171579354 84.61481200814 10.40923667174
-87.28978037789 84.22135089334 9.650016622116
-91.99796250152 85.97382150086 1.519676157958
-91.41983478308 86.34049730935 2.903012247065
-86.98268430651 78.05139466641 11.81754825277
-91.81560657137 78.14260723897 -0.6524242830686
-86.8722058784 87.03288787543 9.354827421718
-91.35672928113 78.7639907204 1.17125485171
-91.53968604339 77.7202798909 0.2403441317094
-84.86545927303 78.71558791114 15.51626244516
-86.79722653072 93.3408647828 7.342636373913
-91.63020782938 90.91409015372 3.14840834059
-87.25531824172 74.47592155632 12.52927221104
-86.07425696252 75.21362038574 15.21297464765
-91.16106283305 81.48663482954 2.492502407732
-90.5757893716 80.50375765165 3.922765152667
-82.67168107606 86.33491475752 13.27365472624
-82.96487921529 87.60913204281 12.329282444
-87.82636242729 84.88753643638 8.842631330747
-84.35474740005 87.83056859068 11.21352843057
-84.55876620841 88.79983575754 10.6423089934
-83.17465723672 88.40634089228 11.76835493646
-90.85284906504 87.33686142924 4.121887080868
-86.72724152045 73.8489141689 14.2127483518
-86.42355444031 73.80296018704 15.09471583991
-86.93659826144 75.89680908807 12.73505114447
-83.54500604627 89.52185961323 10.99503048512
-88.83978565047 75.58107846602 8.260375606895
-88.72239336258 76.48510243038 8.436356181968
-91.66303103922 87.64914656063 2.668165150909
-86.12595505386 76.63627073986 14.24302571059
-90.63286939882 85.3266640223 4.382207547975
-85.1824586822 79.69308786923 14.30335998377
-90.22312494777 84.82135828186 5.152478488357
-91.95415466013 83.85535866474 0.9179829688677
-88.23374725874 92.97600033614 6.514255895342
-88.15189788739 93.96486724046 6.24411450713
-85.3800757181 76.98502795314 15.73048748873
-84.80319976807 92.65283203125 8.842200279236
-89.09641550807 89.22223592452 6.551811823289
-85.8175845876 88.16254355025 9.930269660873
-87.66547683905 77.09918437201 10.72152242862
-78.94045741667 88.39228989734 14.52370677822
-79.71511631602 88.43273633661 14.0000620674
-89.61954054135 91.41099822565 5.649216608215
-89.23067474365 94.6247253418 5.391794204712
-91.14098067071 90.46179999326 3.826403976189
-91.93963698747 89.54916561137 2.519897373028
-90.67907607165 92.08784992503 4.414110286577
-92.00748229705 88.36647480616 2.15883145893
-87.31342900202 80.76548024745 10.42083525356
-88.54108873981 81.32345228217 8.303818225533
-86.29197840378 83.55138824339 11.06344748072
-91.63576602179 81.14048734557 0.926761868349
-91.96373053199 81.33067011421 -0.1021323030729
-88.10058326017 82.54123945134 8.854680095962
-87.02030891857 74.10770397135 13.29008336436
-88.66114033185 85.22844041822 7.687983749868
-83.37451380026 84.06156417283 14.06219752259
-82.3656683929 85.31267926977 14.18556991487
-81.62659745583 85.67233020136 14.57361705313
-79.91235379794 86.99394195408 14.90861860817
-91.92134843838 79.00929111616 -0.7694692860489
-87.93559569912 86.4273596092 8.393204653578
-90.64911812118 83.58820518874 4.189330441393
-90.19847423323 78.53839085091 4.690054556929
-82.70591734945 82.54285737196 15.89561516193
-87.55644665574 74.60603473099 11.72233372584
-86.59650472634 78.69950252896 12.30020046608
-85.76543352653 77.28645400239 14.66293197255
-88.30923622 91.57787766767 6.847835587868
-88.78585426569 90.38592908654 6.684616642745
-81.39871557718 87.91580271512 13.23167464013
-77.44239816095 89.3917582412 14.70647816142
-78.2346321736 89.45395679512 14.22778685506
-89.93552467668 77.14410101136 5.302068334108
-84.85307911288 90.96940089197 9.550021626143
-91.07435434956 77.22910817763 1.777355579159
-91.53472676551 84.75175611001 2.296705553252
-91.07428743866 84.0006697594 3.253891270141
-87.57206861003 83.44109160119 9.452703099751
-86.46837704385 77.01830345632 13.28299203037
-88.24534904025 80.33910657831 8.94283369881
-86.03946980836 78.22531162763 13.56218249175
-85.12085057396 83.64329737652 12.4032963563
-83.99859345837 83.65910767374 13.64917308958
-91.1068447306 78.5125448281 1.950304716625
-88.93743904592 77.0416921546 7.862316649276
-81.89763387706 84.16300891638 15.47949979105
-84.17142822011 79.10427843247 16.46661667861
-90.1977196189 77.71826121309 4.613359081808
-91.46686301969 82.4573958955 1.86560046256
-85.17639263798 78.97291532426 14.76262265383
-81.25829074791 85.01580337653 15.40024671834
-85.57806257522 75.69618258949 16.18963166867
-90.6740474846 77.84868862773 3.208271283977
-88.39299437575 88.93655007712 7.376023566561
-88.02443178926 87.46291572177 8.066227217816
-88.69549876852 87.75415760413 7.255780630642
-90.83584527911 76.98060581503 2.533464600951
-90.24894299365 86.54477235102 5.141658921075
-89.6976072794 83.56866623825 6.139232569616
-90.95815006486 79.5860525514 2.65996163632
-90.54331172733 78.9237771769 3.77410403802
-85.45053954275 84.87164856816 11.51553144517
-89.01834595117 80.58443177613 7.473181777199
-88.84104627055 78.03470716764 8.002824388128
-85.67184761911 78.03529306301 14.39524142803
-86.53578569197 81.78319067054 11.31637757513
-85.60993633696 81.75701177328 12.63336621461
-85.80387144362 82.58918845105 12.02841449852
-88.50429153833 75.12400624369 9.178192908443
-84.48876628209 84.50364974112 12.66249526191
-89.41742707428 86.59282506324 6.459802357741
-89.14239643036 75.46130015491 7.488269950964
-88.8400388208 93.12543610622 6.022451468792
-84.35601236007 79.54349955601 15.80498719226
-85.72661361084 91.66288450582 8.710852466457
-81.35632656989 86.57100524067 14.14937345639
-80.30996143596 86.3295074551 15.12856879422
-80.51503942099 87.4160358727 14.17386056988
-88.82555358781 79.70698776571 7.908609004999
-88.47215841331 78.74225769992 8.723597569818
-89.15110921007 79.03160706289 7.254502463971
-89.52260410248 84.66512629317 6.423342033217
-86.62964224798 80.05172781141 11.73751161387
-87.28315315953 79.35507964116 10.84288117454
-78.39582061768 90.47370910645 13.52938842773
-89.42167039058 88.09489616508 6.301769902343
-88.59964322506 86.00871702093 7.658460101584
-82.20010929418 89.13086851198 12.01494409867
-89.92675339147 80.15382280797 5.522754060304
-87.41138994867 77.77647847548 11.06513801808
-89.32906590509 76.65549889054 6.913354804829
-89.41991350218 90.72714883996 5.963601242846
-83.84750712961 85.91736606737 12.54230305646
-84.70610177702 77.98854431145 16.41686504562
-86.55716047843 75.25385523287 13.94340207575
-79.82708740234 90.28623199463 12.78842067719
-86.00337982178 93.08626556396 7.952734470367
-86.60580976688 91.80148730574 8.072677376784
-87.22050151166 92.6603694961 7.33227675047
-88.16716263207 75.91256782034 9.852927479006
-86.72900679298 85.25690869653 10.0289813322
-88.26398948523 77.33772345921 9.36442753796
-84.48457755837 80.3785157358 15.0009805203
-86.90911612862 90.76068403985 8.224517995328
-87.99764207043 89.9437017199 7.547420421772
-87.17673875088 89.73814630364 8.325225356834
-91.27346915349 80.9035465747 2.006769025922
-87.96093644471 79.91600342135 9.518832922801
-89.77981626992 94.04055669295 5.089906242403
-82.77049394816 85.64393889715 13.61366577784
-86.36565938384 89.5909865994 9.02122595877
-85.37867446359 89.55990884413 9.762504200646
-82.93187929823 89.91362149273 11.18538301618
-90.40184120368 77.36763120377 3.978408478545
-83.05097214393 83.5778676545 14.72926797176
-89.72248932475 78.42898288903 5.918966931824
-91.67516532539 83.73576836354 1.656707294069
-78.71974914613 89.79709073339 13.73515288393
-90.92734145902 89.78390610645 4.112839472319
-89.00168562649 84.3598607859 7.279939209434
-81.71748951367 83.44541399728 16.26328327578
-85.63151466028 80.6783179693 13.07487257959
-85.2061604245 80.70675069058 13.70207637819
-89.67707012829 76.04729367448 5.994030004774
-91.80352195174 93.28461363862 3.066921973103
-92.09239955624 78.00218956873 -1.739866962382
-86.92069711747 77.41143533536 12.17691558281
-88.95079812341 82.13893390599 7.523756483162
-91.95468325146 82.85228212281 0.526891242366
-90.81599675904 80.36183194305 3.237449743645
-90.54766845703 96.017578125 3.999451637268
-92.0055241551 85.22912795386 1.269253743002
-89.88338535679 86.02225332199 5.764867970561
-81.84564932064 88.23991492601 12.73434009247
-85.93024321957 74.5364087842 16.03535734115
-84.82059474593 82.96593066196 13.08379381789
-89.68150719318 82.24151111624 6.159472595437
-90.19728828303 82.48156059828 5.093752428947
-89.71125544255 81.47605972064 6.071317743465
-83.72882271397 81.77689831853 15.14467408305
-86.25218261707 74.5079949549 15.1626753312
-85.00988855546 78.03025378603 15.74992136855
-90.18405860601 81.13314612602 4.994330226107
-85.65257806689 85.78328208136 10.96127222822
-84.83030951908 86.3240240171 11.50096113237
-84.61082060889 85.59183835254 12.02018475677
-86.18189079471 86.17576795044 10.30566166154
-86.74048774938 74.51238814105 13.8384304986
-90.4003943129 76.54491258221 3.884703534253
-81.81327505325 86.83154980488 13.62882318444
-91.31694022014 87.95524252007 3.365571118591
-91.11867506879 86.49884415014 3.536760369568
-91.31518166869 85.6281458942 2.994287968664
-90.94835188199 85.05695431771 3.705049918602
-88.43889418727 76.69555098254 9.075219277086
-89.43228854681 85.86706706099 6.500004845721
-82.73513628171 90.71773976847 10.92107545621
-91.98887439509 87.6326927391 2.007190497912
-91.52793371682 94.46079285745 3.343908937295
-86.09082407967 90.54507374952 8.898060742125
-80.63731155247 88.65047179233 13.26168529876
-91.35347577036 83.01191364709 2.327252349276
-85.6788758969 76.12079563979 15.63082402037
-89.21793908226 78.32106678605 7.123794200307
-81.89286506116 90.88075220467 11.33043282293
-80.96296280172 90.02565655652 12.27699603282
-87.18562304132 83.17319429571 10.03197094628
-85.94523835859 79.61159497471 13.06151633859
-89.54586658536 87.39575929373 6.202245926172
-84.19560615902 86.93932821561 11.75398955827
-82.64923765358 87.17546494488 12.80072555853
-85.57182703976 79.40049630906 13.80412587682
-92.11813422526 80.3336940627 -1.0274672359
-91.58969915559 89.26351826489 3.053117287076
-82.90513702073 82.93644120087 15.35797196151
-83.51087357062 82.76432473619 14.76703486348
-90.13510669594 76.2169664233 4.66747957696
-90.64913923384 93.33554679517 4.353233685676
-84.13267337331 83.00396580987 13.88201394276
-84.87243712271 81.85866336116 13.57925886949
-89.28944999749 91.98966703195 5.879106347835
-87.81999722862 91.35373114943 7.320547757033
-91.93708444299 80.37746807551 -0.360749318184
-85.84211427154 85.13059367186 11.01535006381
-85.56820387932 84.18880644955 11.66829122503
-91.48388065108 92.79963299821 3.436939740116
-83.75276774305 86.54362591673 12.30003839505
-90.33403260788 89.84300780106 4.947890651582
-91.17155781479 95.62999233702 3.555330232582
-81.08768683738 86.17984327111 14.63971208729
-90.38594006628 83.29347411891 4.750388981447
-87.98665421745 78.28535011375 9.784775030141
-82.3679546364 89.67867529519 11.635015703
-83.74534240316 85.1538186026 13.04163202392
-91.63708732988 82.17328836847 1.27095837172
-85.97074285508 75.8696789085 15.06787457704
-88.78623129313 74.47325883927 8.562521367548
-84.53518886933 91.56446841495 9.484550907027
-84.32967128058 90.25555108377 10.18170123086
-84.85811913064 89.38922071855 10.19417118041
-92.05118344848 84.5308652651 0.9058592816885
-88.24781393992 81.01711913492 8.844436451774
-85.27588195651 87.09941342014 10.79102642662
-88.3946809784 83.91013776736 8.227404481239
-87.55246737728 75.47893998843 11.44680253484
-87.72693552863 85.56194386854 8.827585540933
-82.36266194973 87.7872771424 12.65064169419
-90.99625808495 81.84789400481 3.029675030284
-87.12534937579 76.26893668453 12.16272998574
-90.51427577132 90.99750450867 4.669163752209
-88.75231293904 87.09091301546 7.292276779629
-81.77196144489 88.83586075055 12.43768637401
-80.99975585938 91.66167449951 11.48725414276
-80.78927139352 89.16541907359 12.85188992007
-87.66350506837 78.08952304598 10.47636582897
-91.93329295926 77.26298047189 -1.365480028674
-83.43715667725 91.99025726318 9.931365013123
-83.88444128096 91.49158214368 9.898426233423
-3  110 256 305
-3  43 473 476
-3  316 137 336
-3  240 195 293
-3  238 228 412
-3  218 84 532
-3  15 383 434
-3  119 374 420
-3  227 16 302
-3  283 127 328
-3  106 302 459
-3  207 66 351
-3  344 110 416
-3  31 254 331
-3  199 197 486
-3  144 196 509
-3  65 474 499
-3  348 171 407
-3  119 314 315
-3  278 8 533
-3  301 151 378
-3  88 291 456
-3  324 140 356
-3  66 207 273
-3  11 214 458
-3  251 31 267
-3  75 255 539
-3  181 9 230
-3  83 165 250
-3  15 326 383
-3  161 132 445
-3  297 184 428
-3  34 427 433
-3  226 133 318
-3  256 110 344
-3  265 47 425
-3  405 246 479
-3  189 79 338
-3  276 78 521
-3  266 116 431
-3  74 261 362
-3  116 275 431
-3  190 162 229
-3  40 279 281
-3  208 40 388
-3  260 74 362
-3  150 307 313
-3  166 49 339
-3  112 189 253
-3  346 45 439
-3  99 242 346
-3  117 209 319
-3  128 36 171
-3  14 276 544
-3  466 130 468
-3  107 371 465
-3  49 371 514
-3  161 227 352
-3  352 227 390
-3  264 73 543
-3  81 173 308
-3  246 51 359
-3  239 71 432
-3  46 243 370
-3  273 151 537
-3  365 85 461
-3  276 14 413
-3  283 87 538
-3  93 248 424
-3  169 299 354
-3  100 250 415
-3  59 415 498
-3  208 102 247
-3  19 226 318
-3  254 118 331
-3  282 92 285
-3  34 404 427
-3  121 257 263
-3  257 441 529
-3  204 24 417
-3  65 218 532
-3  80 188 459
-3  207 151 273
-3  102 208 388
-3  79 278 281
-3  108 11 458
-3  73 265 425
-3  304 24 308
-3  218 22 327
-3  64 257 528
-3  241 71 269
-3  241 144 509
-3  37 166 286
-3  228 114 412
-3  264 51 330
-3  143 260 362
-3  131 284 515
-3  288 287 430
-3  126 278 533
-3  48 161 445
-3  408 153 410
-3  103 219 337
-3  275 78 409
-3  114 228 337
-3  278 126 281
-3  302 80 459
-3  80 273 537
-3  307 81 313
-3  81 204 313
-3  82 173 516
-3  82 223 303
-3  226 83 309
-3  226 309 481
-3  234 185 327
-3  195 84 293
-3  289 182 461
-3  85 365 482
-3  133 482 483
-3  207 351 366
-3  87 312 538
-3  464 300 470
-3  312 213 538
-3  213 88 456
-3  381 176 492
-3  248 89 317
-3  96 233 252
-3  32 271 517
-3  268 91 503
-3  31 251 254
-3  284 131 507
-3  248 27 424
-3  90 340 341
-3  194 90 233
-3  292 148 295
-3  294 186 373
-3  322 323 402
-3  146 21 184
-3  100 168 271
-3  109 212 385
-3  242 99 345
-3  258 107 372
-3  331 118 421
-3  118 168 421
-3  364 191 386
-3  221 102 411
-3  247 102 320
-3  288 367 496
-3  278 79 342
-3  224 347 380
-3  242 345 444
-3  105 242 375
-3  188 33 389
-3  227 106 390
-3  99 258 345
-3  167 371 399
-3  182 55 530
-3  70 185 234
-3  433 120 435
-3  35 240 293
-3  125 195 240
-3  25 392 408
-3  111 408 410
-3  166 76 286
-3  256 8 305
-3  72 238 412
-3  396 196 397
-3  328 419 544
-3  114 219 413
-3  314 119 401
-3  116 328 544
-3  328 127 419
-3  12 423 471
-3  255 118 539
-3  271 118 517
-3  115 314 398
-3  115 398 429
-3  184 61 428
-3  98 211 357
-3  234 121 440
-3  22 234 327
-3  236 122 426
-3  198 7 231
-3  122 236 325
-3  135 236 426
-3  124 242 444
-3  177 4 270
-3  76 287 288
-3  125 288 430
-3  220 126 449
-3  126 256 449
-3  419 14 544
-3  56 283 328
-3  129 348 407
-3  128 171 348
-3  104 10 224
-3  347 224 519
-3  220 40 281
-3  247 351 520
-3  251 131 335
-3  161 16 227
-3  132 161 352
-3  133 226 481
-3  482 225 483
-3  76 166 339
-3  474 134 475
-3  235 34 433
-3  135 187 427
-3  321 136 446
-3  378 151 491
-3  309 60 481
-3  173 82 304
-3  45 346 347
-3  23 241 509
-3  139 209 452
-3  376 54 403
-3  243 88 370
-3  27 213 312
-3  205 382 386
-3  18 142 205
-3  143 443 478
-3  205 142 478
-3  319 139 501
-3  209 71 241
-3  52 375 450
-3  345 44 444
-3  146 184 297
-3  135 297 428
-3  147 393 493
-3  153 48 410
-3  316 336 487
-3  30 149 336
-3  150 528 529
-3  528 257 529
-3  26 446 491
-3  68 301 378
-3  299 169 343
-3  77 210 279
-3  161 48 249
-3  423 154 471
-3  167 391 514
-3  134 391 475
-3  156 268 480
-3  60 309 480
-3  157 316 487
-3  336 149 487
-3  62 159 285
-3  159 282 285
-3  160 321 446
-3  26 365 446
-3  16 280 472
-3  161 249 418
-3  17 162 190
-3  6 163 401
-3  163 314 401
-3  164 384 540
-3  212 59 385
-3  67 221 411
-3  385 59 540
-3  113 166 395
-3  196 49 397
-3  107 258 399
-3  371 107 399
-3  168 118 271
-3  168 100 326
-3  299 87 354
-3  17 169 354
-3  170 490 543
-3  138 246 359
-3  51 171 330
-3  94 264 330
-3  274 232 495
-3  69 172 232
-3  173 304 308
-3  218 65 306
-3  174 124 353
-3  4 174 353
-3  264 94 265
-3  171 36 329
-3  89 324 356
-3  324 89 492
-3  71 239 270
-3  117 296 403
-3  248 93 300
-3  310 193 311
-3  91 334 503
-3  230 9 298
-3  233 90 252
-3  11 180 341
-3  454 181 515
-3  160 365 461
-3  5 182 289
-3  98 331 421
-3  61 357 511
-3  183 61 322
-3  61 184 322
-3  350 35 384
-3  35 185 245
-3  96 186 294
-3  148 58 295
-3  187 61 511
-3  357 211 511
-3  301 68 436
-3  33 188 194
-3  279 210 457
-3  210 25 338
-3  38 229 398
-3  38 190 229
-3  186 191 364
-3  186 58 191
-3  261 74 420
-3  41 260 506
-3  53 300 464
-3  57 193 310
-3  33 294 373
-3  33 194 233
-3  287 43 430
-3  216 84 476
-3  371 167 514
-3  113 396 397
-3  42 199 272
-3  64 197 199
-3  64 199 547
-3  202 42 272
-3  199 546 547
-3  198 231 546
-3  97 332 438
-3  184 21 323
-3  73 274 495
-3  73 264 265
-3  202 494 542
-3  20 232 274
-3  360 2 361
-3  417 172 522
-3  204 417 522
-3  352 50 377
-3  364 101 373
-3  280 130 472
-3  351 66 520
-3  208 247 520
-3  351 86 366
-3  130 466 467
-3  279 40 466
-3  209 139 319
-3  117 239 432
-3  210 77 392
-3  25 210 392
-3  326 15 358
-3  215 120 434
-3  100 212 326
-3  212 100 415
-3  88 368 370
-3  88 213 368
-3  180 502 512
-3  458 214 524
-3  212 109 383
-3  185 70 245
-3  84 216 532
-3  474 217 499
-3  217 474 475
-3  13 422 475
-3  441 22 529
-3  22 218 307
-3  219 114 337
-3  219 103 437
-3  126 220 281
-3  67 411 449
-3  400 462 485
-3  19 221 462
-3  222 400 485
-3  462 221 485
-3  223 82 422
-3  99 223 523
-3  346 105 347
-3  104 224 380
-3  365 160 446
-3  26 225 365
-3  165 83 387
-3  83 226 387
-3  106 227 302
-3  50 352 390
-3  113 228 238
-3  166 37 395
-3  162 56 229
-3  314 115 315
-3  255 75 333
-3  75 230 298
-3  123 231 325
-3  257 64 363
-3  202 272 494
-3  292 96 349
-3  96 292 295
-3  257 121 441
-3  70 234 440
-3  235 433 435
-3  70 235 435
-3  123 236 404
-3  236 123 325
-3  237 101 382
-3  50 237 382
-3  238 72 451
-3  23 396 451
-3  54 177 239
-3  239 177 270
-3  240 35 350
-3  125 240 535
-3  465 144 508
-3  144 241 269
-3  242 124 375
-3  242 105 346
-3  139 497 501
-3  72 243 497
-3  291 88 355
-3  88 243 355
-3  70 215 245
-3  215 109 245
-3  129 246 405
-3  171 51 407
-3  19 318 320
-3  366 86 483
-3  213 27 368
-3  27 248 317
-3  48 153 249
-3  153 77 249
-3  250 100 262
-3  83 250 262
-3  332 285 438
-3  92 251 267
-3  29 252 502
-3  189 103 253
-3  228 37 337
-3  254 251 335
-3  75 254 335
-3  333 91 448
-3  156 32 268
-3  305 125 535
-3  110 305 535
-3  123 263 363
-3  263 257 363
-3  258 99 523
-3  13 258 523
-3  41 192 260
-3  192 6 259
-3  259 74 453
-3  74 260 453
-3  147 261 420
-3  410 48 493
-3  32 156 262
-3  156 83 262
-3  263 34 489
-3  121 263 489
-3  490 264 543
-3  51 264 490
-3  2 203 361
-3  266 115 429
-3  56 266 429
-3  97 267 402
-3  323 97 402
-3  255 333 448
-3  118 255 517
-3  145 269 469
-3  44 269 505
-3  145 353 504
-3  269 71 469
-3  32 262 271
-3  262 100 271
-3  272 199 486
-3  272 486 494
-3  80 206 273
-3  206 66 273
-3  47 201 425
-3  201 20 274
-3  111 393 409
-3  374 39 393
-3  219 276 413
-3  219 78 276
-3  369 277 437
-3  25 277 369
-3  288 8 367
-3  8 278 367
-3  77 279 468
-3  279 466 468
-3  249 280 418
-3  249 77 280
-3  281 279 457
-3  79 281 457
-3  92 282 284
-3  92 284 507
-3  283 56 534
-3  87 283 534
-3  28 284 488
-3  267 97 438
-3  62 285 332
-3  112 253 286
-3  253 37 286
-3  391 155 514
-3  473 43 513
-3  367 112 496
-3  76 288 496
-3  85 157 289
-3  157 5 289
-3  60 157 290
-3  157 85 290
-3  244 291 355
-3  244 14 291
-3  29 95 349
-3  95 292 349
-3  84 185 293
-3  185 35 293
-3  33 233 294
-3  233 96 294
-3  58 186 295
-3  186 96 295
-3  46 319 501
-3  319 46 379
-3  122 63 426
-3  63 146 297
-3  9 179 298
-3  179 91 298
-3  152 299 343
-3  152 87 299
-3  93 310 311
-3  188 301 436
-3  188 80 301
-3  16 206 302
-3  206 80 302
-3  246 138 479
-3  304 82 500
-3  500 138 536
-3  138 359 463
-3  8 288 305
-3  288 125 305
-3  173 499 516
-3  173 81 306
-3  218 306 307
-3  306 81 307
-3  24 204 308
-3  204 81 308
-3  83 156 309
-3  309 156 480
-3  57 310 343
-3  169 57 343
-3  193 1 311
-3  1 178 311
-3  87 152 312
-3  152 27 312
-3  197 313 442
-3  197 150 313
-3  315 39 374
-3  119 315 374
-3  39 315 484
-3  275 39 431
-3  157 60 316
-3  268 137 480
-3  317 89 356
-3  141 317 356
-3  86 247 320
-3  318 86 320
-3  296 117 379
-3  117 319 379
-3  102 221 320
-3  221 19 320
-3  55 108 321
-3  136 458 524
-3  322 184 323
-3  183 322 402
-3  21 200 323
-3  200 97 323
-3  176 324 492
-3  176 12 324
-3  231 7 325
-3  7 122 325
-3  326 212 383
-3  168 326 358
-3  185 84 327
-3  84 218 327
-3  116 266 328
-3  266 56 328
-3  36 175 329
-3  175 360 361
-3  171 329 330
-3  329 94 330
-3  331 98 510
-3  31 331 510
-3  97 200 332
-3  200 62 332
-3  75 298 333
-3  298 91 333
-3  91 179 334
-3  179 30 334
-3  131 230 335
-3  230 75 335
-3  137 334 336
-3  334 30 336
-3  37 253 337
-3  253 103 337
-3  103 369 437
-3  369 103 531
-3  155 287 339
-3  287 76 339
-3  90 194 340
-3  68 340 436
-3  340 214 341
-3  214 11 341
-3  79 189 342
-3  189 112 342
-3  343 310 477
-3  152 343 477
-3  222 344 416
-3  222 67 344
-3  345 258 372
-3  44 345 372
-3  223 99 439
-3  99 346 439
-3  45 347 405
-3  45 405 479
-3  224 10 406
-3  10 128 406
-3  96 252 349
-3  252 29 349
-3  164 350 384
-3  110 350 416
-3  247 86 351
-3  225 366 483
-3  143 362 377
-3  362 132 377
-3  145 270 353
-3  270 4 353
-3  56 162 534
-3  162 17 354
-3  243 72 355
-3  72 244 355
-3  46 370 394
-3  370 141 394
-3  61 183 357
-3  183 98 357
-3  15 211 358
-3  211 98 358
-3  304 500 536
-3  24 304 536
-3  94 329 361
-3  329 175 361
-3  414 47 447
-3  47 265 447
-3  261 132 362
-3  132 352 377
-3  363 64 527
-3  123 363 527
-3  382 101 386
-3  186 364 373
-3  365 225 482
-3  85 289 461
-3  225 26 366
-3  26 207 366
-3  278 342 367
-3  342 112 367
-3  27 317 368
-3  317 141 368
-3  338 25 531
-3  25 369 531
-3  368 141 370
-3  141 356 394
-3  49 196 371
-3  371 196 465
-3  196 144 465
-3  44 372 508
-3  101 237 373
-3  33 373 389
-3  147 374 393
-3  374 147 420
-3  105 375 380
-3  347 105 380
-3  296 376 403
-3  296 154 376
-3  377 50 443
-3  143 377 443
-3  378 136 524
-3  68 378 524
-3  356 140 394
-3  140 296 379
-3  375 52 380
-3  52 104 380
-3  300 381 492
-3  300 53 381
-3  101 364 386
-3  50 382 443
-3  109 215 383
-3  383 215 434
-3  35 245 384
-3  245 109 384
-3  384 385 540
-3  384 109 385
-3  191 18 386
-3  18 205 386
-3  226 19 387
-3  400 165 462
-3  40 220 388
-3  220 102 388
-3  373 237 389
-3  237 106 389
-3  106 237 390
-3  237 50 390
-3  473 216 476
-3  155 391 513
-3  77 153 392
-3  392 153 408
-3  39 275 393
-3  393 275 409
-3  140 379 394
-3  379 46 394
-3  37 228 395
-3  228 113 395
-3  113 238 396
-3  396 238 451
-3  49 166 397
-3  166 113 397
-3  38 398 526
-3  258 13 399
-3  13 167 399
-3  222 59 400
-3  250 165 498
-3  119 259 401
-3  259 6 401
-3  267 31 402
-3  98 183 510
-3  54 239 403
-3  239 117 403
-3  34 263 404
-3  263 123 404
-3  405 347 519
-3  129 405 519
-3  128 348 406
-3  348 129 406
-3  51 246 407
-3  246 129 407
-3  111 277 408
-3  277 25 408
-3  78 277 409
-3  277 111 409
-3  393 111 493
-3  111 410 493
-3  102 220 411
-3  411 220 449
-3  114 244 412
-3  244 72 412
-3  14 244 413
-3  244 114 413
-3  361 203 447
-3  203 414 447
-3  415 250 498
-3  59 212 415
-3  350 164 416
-3  164 222 416
-3  359 170 463
-3  463 170 541
-3  280 16 418
-3  16 161 418
-3  127 291 419
-3  291 14 419
-3  74 259 420
-3  259 119 420
-3  168 358 421
-3  358 98 421
-3  499 217 516
-3  422 217 475
-3  140 324 471
-3  324 12 471
-3  27 152 424
-3  310 93 477
-3  201 274 425
-3  274 73 425
-3  63 297 426
-3  297 135 426
-3  404 236 427
-3  236 135 427
-3  61 187 428
-3  187 135 428
-3  398 229 429
-3  229 56 429
-3  84 195 476
-3  195 125 430
-3  431 39 484
-3  266 431 484
-3  71 209 432
-3  209 117 432
-3  427 187 433
-3  187 120 433
-3  120 211 434
-3  211 15 434
-3  120 215 435
-3  215 70 435
-3  340 194 436
-3  194 188 436
-3  277 78 437
-3  78 219 437
-3  285 92 438
-3  92 267 438
-3  45 303 439
-3  303 223 439
-3  34 235 489
-3  235 70 440
-3  121 234 441
-3  234 22 441
-3  313 204 442
-3  172 69 522
-3  382 205 443
-3  443 205 478
-3  124 444 504
-3  353 124 504
-3  132 261 445
-3  261 48 445
-3  136 378 491
-3  446 136 491
-3  265 94 447
-3  94 361 447
-3  91 268 448
-3  268 32 448
-3  256 344 449
-3  344 67 449
-3  375 124 450
-3  124 174 450
-3  451 139 452
-3  23 451 452
-3  209 241 452
-3  241 23 452
-3  260 192 453
-3  192 259 453
-3  284 28 515
-3  28 454 515
-3  148 292 545
-3  292 95 455
-3  291 127 456
-3  127 213 456
-3  210 338 457
-3  338 79 457
-3  136 321 458
-3  321 108 458
-3  188 389 459
-3  389 106 459
-3  158 282 518
-3  282 159 460
-3  461 182 530
-3  160 461 530
-3  165 387 462
-3  387 19 462
-3  417 24 541
-3  24 463 541
-3  311 178 470
-3  178 464 470
-3  372 107 508
-3  107 465 508
-3  40 208 466
-3  466 208 467
-3  66 206 467
-3  206 130 467
-3  130 280 468
-3  280 77 468
-3  71 270 469
-3  270 145 469
-3  300 93 470
-3  93 311 470
-3  154 296 471
-3  296 140 471
-3  130 206 472
-3  206 16 472
-3  391 134 513
-3  134 473 513
-3  216 473 474
-3  473 134 474
-3  391 167 475
-3  167 13 475
-3  195 430 476
-3  430 43 476
-3  93 424 477
-3  424 152 477
-3  143 478 506
-3  260 143 506
-3  138 303 479
-3  303 45 479
-3  137 316 480
-3  316 60 480
-3  60 290 481
-3  290 133 481
-3  133 290 482
-3  290 85 482
-3  86 318 483
-3  318 133 483
-3  315 115 484
-3  115 266 484
-3  221 67 485
-3  67 222 485
-3  197 442 486
-3  442 69 486
-3  149 5 487
-3  5 157 487
-3  284 282 488
-3  282 158 488
-3  235 440 489
-3  440 121 489
-3  170 359 490
-3  359 51 490
-3  151 207 491
-3  207 26 491
-3  492 89 525
-3  300 492 525
-3  48 261 493
-3  261 147 493
-3  486 69 494
-3  69 232 494
-3  232 172 495
-3  73 495 543
-3  112 286 496
-3  286 76 496
-3  139 451 497
-3  451 72 497
-3  165 400 498
-3  400 59 498
-3  173 306 499
-3  306 65 499
-3  82 303 500
-3  303 138 500
-3  497 243 501
-3  243 46 501
-3  90 341 512
-3  341 180 512
-3  334 137 503
-3  137 268 503
-3  145 504 505
-3  269 145 505
-3  504 444 505
-3  444 44 505
-3  478 142 506
-3  142 41 506
-3  131 251 507
-3  251 92 507
-3  144 269 508
-3  269 44 508
-3  196 396 509
-3  396 23 509
-3  183 402 510
-3  402 31 510
-3  211 120 511
-3  120 187 511
-3  502 252 512
-3  252 90 512
-3  43 287 513
-3  287 155 513
-3  155 339 514
-3  339 49 514
-3  181 230 515
-3  230 131 515
-3  217 422 516
-3  422 82 516
-3  255 448 517
-3  448 32 517
-3  282 460 518
-3  460 3 518
-3  224 406 519
-3  406 129 519
-3  66 467 520
-3  467 208 520
-3  78 275 521
-3  275 116 521
-3  69 442 522
-3  442 204 522
-3  223 422 523
-3  422 13 523
-3  214 340 524
-3  340 68 524
-3  89 248 525
-3  248 300 525
-3  398 314 526
-3  314 163 526
-3  546 231 547
-3  231 123 527
-3  150 197 528
-3  197 64 528
-3  22 307 529
-3  307 150 529
-3  55 321 530
-3  321 160 530
-3  103 189 531
-3  189 338 531
-3  216 474 532
-3  474 65 532
-3  8 256 533
-3  256 126 533
-3  162 354 534
-3  354 87 534
-3  240 350 535
-3  350 110 535
-3  138 463 536
-3  463 24 536
-3  151 301 537
-3  301 80 537
-3  213 127 538
-3  127 283 538
-3  118 254 539
-3  254 75 539
-3  59 222 540
-3  222 164 540
-3  495 172 543
-3  172 417 541
-3  494 232 542
-3  232 20 542
-3  172 541 543
-3  541 170 543
-3  276 521 544
-3  521 116 544
-3  292 455 545
-3  455 0 545
-3  199 42 546
-3  42 198 546
-3  231 527 547
-3  527 64 547
-
diff -pruN 5.0.3-1/examples/Surface_mesh_skeletonization/CMakeLists.txt 5.1-1/examples/Surface_mesh_skeletonization/CMakeLists.txt
--- 5.0.3-1/examples/Surface_mesh_skeletonization/CMakeLists.txt	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_skeletonization/CMakeLists.txt	2020-09-08 12:12:21.000000000 +0000
@@ -12,10 +12,9 @@ find_package(CGAL QUIET)
 if ( CGAL_FOUND )
 
   find_package(Eigen3 3.2.0) #(requires 3.2.0 or greater)
+  include(CGAL_Eigen_support)
 
-  if(EIGEN3_FOUND)
-    include( ${EIGEN3_USE_FILE} )
-
+  if(TARGET CGAL::Eigen_support)
     create_single_source_cgal_program( "simple_mcfskel_example.cpp" )
     create_single_source_cgal_program( "simple_mcfskel_sm_example.cpp" )
     create_single_source_cgal_program( "simple_mcfskel_LCC_example.cpp" )
@@ -23,6 +22,16 @@ if ( CGAL_FOUND )
     create_single_source_cgal_program( "MCF_Skeleton_sm_example.cpp" )
     create_single_source_cgal_program( "MCF_Skeleton_LCC_example.cpp" )
     create_single_source_cgal_program( "segmentation_example.cpp" )
+    foreach(target
+        simple_mcfskel_example
+        simple_mcfskel_sm_example
+        simple_mcfskel_LCC_example
+        MCF_Skeleton_example
+        MCF_Skeleton_sm_example
+        MCF_Skeleton_LCC_example
+        segmentation_example)
+      target_link_libraries(${target} PUBLIC CGAL::Eigen_support)
+    endforeach()
   else()
     message(STATUS "These programs require the Eigen library (3.2 or greater), and will not be compiled.")
   endif()
@@ -32,4 +41,3 @@ else()
   message(STATUS "This program requires the CGAL library, and will not be compiled.")
 
 endif()
-
diff -pruN 5.0.3-1/examples/Surface_mesh_topology/CMakeLists.txt 5.1-1/examples/Surface_mesh_topology/CMakeLists.txt
--- 5.0.3-1/examples/Surface_mesh_topology/CMakeLists.txt	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_topology/CMakeLists.txt	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,60 @@
+cmake_minimum_required(VERSION 3.1...3.15)
+project(Surface_mesh_topology_Examples)
+
+if(NOT POLICY CMP0070 AND POLICY CMP0053)
+  # Only set CMP0053 to OLD with CMake<3.10, otherwise there is a warning.
+  cmake_policy(SET CMP0053 OLD)
+endif()
+
+if(POLICY CMP0071)
+  cmake_policy(SET CMP0071 NEW)
+endif()
+
+find_package(CGAL QUIET COMPONENTS Qt5)
+if(CGAL_Qt5_FOUND)
+    add_definitions(-DCGAL_USE_BASIC_VIEWER -DQT_NO_KEYWORDS)
+endif()
+
+if ( NOT CGAL_FOUND )
+  message(FATAL_ERROR "This program requires the CGAL library, and will not be compiled.")
+endif()
+
+# add_definitions(-DCGAL_TRACE_PATH_TESTS)
+# add_definitions(-DCGAL_TRACE_CMAP_TOOLS)
+# add_definitions("-Wall -Wextra")
+# add_definitions("-D_GLIBCXX_DEBUG")
+
+set(SOURCE_FILES
+  edgewidth_lcc.cpp
+  edgewidth_surface_mesh.cpp
+  facewidth.cpp
+  map_2_constructor.cpp
+  open_path_homotopy.cpp
+  path_homotopy.cpp
+  path_homotopy_double_torus.cpp
+  path_homotopy_torus.cpp
+  path_homotopy_with_sm_and_polyhedron.cpp
+  path_homotopy_with_symbols_2.cpp
+  path_homotopy_with_symbols.cpp
+  shortest_noncontractible_cycle_2.cpp
+  shortest_noncontractible_cycle.cpp
+  unsew_edgewidth_repeatedly.cpp
+  )
+
+foreach(cppfile ${SOURCE_FILES})
+  create_single_source_cgal_program("${cppfile}")
+endforeach()
+
+if(CGAL_Qt5_FOUND)
+  target_link_libraries(edgewidth_lcc PUBLIC CGAL::CGAL_Qt5)
+  target_link_libraries(edgewidth_surface_mesh PUBLIC CGAL::CGAL_Qt5)
+  target_link_libraries(facewidth PUBLIC CGAL::CGAL_Qt5)
+  target_link_libraries(open_path_homotopy PUBLIC CGAL::CGAL_Qt5)
+  target_link_libraries(path_homotopy PUBLIC CGAL::CGAL_Qt5)
+  target_link_libraries(path_homotopy_double_torus PUBLIC CGAL::CGAL_Qt5)
+  target_link_libraries(path_homotopy_torus PUBLIC CGAL::CGAL_Qt5)
+  target_link_libraries(path_homotopy_with_sm_and_polyhedron PUBLIC CGAL::CGAL_Qt5)
+  target_link_libraries(shortest_noncontractible_cycle_2 PUBLIC CGAL::CGAL_Qt5)
+  target_link_libraries(shortest_noncontractible_cycle PUBLIC CGAL::CGAL_Qt5)
+  target_link_libraries(unsew_edgewidth_repeatedly PUBLIC CGAL::CGAL_Qt5)
+endif()
diff -pruN 5.0.3-1/examples/Surface_mesh_topology/data/3torus.off 5.1-1/examples/Surface_mesh_topology/data/3torus.off
--- 5.0.3-1/examples/Surface_mesh_topology/data/3torus.off	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_topology/data/3torus.off	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,44 @@
+OFF
+19 23 0
+-0.32821 0.728949 0.785375
+-0.253234 0.54351 0.0549907
+-0.844014 0.631231 -0.911234
+-1.50977 0.904391 -1.14708
+-0.390354 -0.366081 -0.611454
+-0.60245 -1.09023 -0.547515
+-0.888454 0.366475 0.427312
+-1.47923 0.454195 -0.538913
+-1.02557 -0.543117 -0.239133
+1.3173 0.119589 -0.65761
+0.876658 -0.199807 -0.131025
+-0.083217 -0.804717 -0.0759778
+0.053903 0.104874 0.590467
+0.569521 0.23883 -0.666502
+0.938527 -0.503249 0.961862
+1.44104 -0.487296 1.52816
+1.25068 0.439809 0.0253912
+0.427929 0.74449 0.746884
+1.31255 0.136367 1.11828
+4 0 1 2 3 
+4 3 2 4 5 
+4 1 6 7 2 
+4 2 7 8 4 
+4 4 8 6 1 
+4 6 0 3 7 
+4 7 3 5 8 
+4 8 5 0 6 
+4 9 10 11 5 
+4 5 11 12 0 
+4 10 13 4 11 
+4 11 4 1 12 
+4 12 1 13 10 
+4 13 9 5 4 
+4 1 0 9 13 
+4 0 12 14 15 
+4 15 14 10 9 
+4 10 16 17 12 
+4 12 17 18 14 
+4 14 18 16 10 
+4 16 9 0 17 
+4 17 0 15 18 
+4 18 15 9 16 
diff -pruN 5.0.3-1/examples/Surface_mesh_topology/data/double-torus-3-holes.off 5.1-1/examples/Surface_mesh_topology/data/double-torus-3-holes.off
--- 5.0.3-1/examples/Surface_mesh_topology/data/double-torus-3-holes.off	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_topology/data/double-torus-3-holes.off	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,431 @@
+OFF
+228 201 0
+1.62473 1.92519 0.645116
+1.59856 2.45453 0.105686
+1.36768 2.78362 0.507687
+1.44616 2.17973 0.956047
+2.17131 1.2071 0.731432
+1.62939 1.58636 0.364889
+2.16529 1.64517 1.09374
+2.90357 1.7785 1.18875
+2.54642 2.28757 1.81061
+2.92369 2.93613 1.71623
+3.40711 2.24707 0.874502
+2.57252 0.388793 0.240254
+1.95128 0.746801 0.480583
+2.91289 1.10083 0.628296
+3.41972 1.32982 0.115897
+3.38094 2.77641 0.335073
+3.39492 1.75991 -0.50561
+2.11625 -0.453389 -0.597175
+1.94988 -0.216593 0.101481
+2.95902 0.366033 -0.409339
+0.778032 -0.784465 -1.08886
+1.05833 -0.966152 -0.410409
+1.95041 -0.216224 -1.29583
+-1.05316 0.711603 -0.210338
+-1.28047 0.320835 -0.153469
+-0.980005 -0.0806305 -0.505709
+-0.97055 -0.110718 -0.522061
+-0.571926 -0.261335 -0.874081
+-0.418901 0.471953 -0.770442
+-0.672026 1.354 0.506539
+-1.41031 1.22067 0.411524
+-0.441144 1.02491 0.104538
+-0.136133 1.41281 1.23539
+-0.678049 1.79208 0.868843
+-0.131474 1.07398 0.955159
+0.108702 1.44301 2.10774
+0.034051 1.76883 1.42941
+-0.111327 0.982712 1.85689
+0.344842 2.07115 2.89611
+0.511204 1.83436 2.19745
+-0.277795 1.46577 2.75733
+1.00166 3.15596 2.99873
+1.12643 2.44998 2.75813
+-0.0562639 2.6432 3.1855
+1.89383 3.89949 2.10802
+2.06519 3.26051 2.47436
+0.715229 3.78364 2.68913
+1.49846 4.06987 1.48631
+2.35696 3.74549 0.728177
+2.84521 3.54002 1.26787
+-0.0555178 2.64372 1.2094
+0.34537 2.07152 1.49879
+0.736163 2.26094 1.4298
+0.449736 2.88861 1.1202
+-1.90166 1.23926 2.10589
+-1.90141 1.221 2.09078
+-1.92107 1.27736 1.16019
+-1.92646 1.66935 1.48438
+-1.35974 0.859992 2.47243
+-0.618163 0.753727 2.36929
+-0.608846 0.076062 1.80884
+-1.04378 -0.00248243 1.75286
+-1.0988 0.14376 1.8749
+-1.32934 0.452132 2.1346
+-1.35435 0.467997 2.14823
+2.3923 1.1754 -1.39739
+2.88437 0.691855 -1.08767
+2.853 2.13918 -0.872153
+2.11142 2.24544 -0.769017
+1.60459 2.01646 -0.256618
+1.38456 1.55616 -0.507466
+1.77106 1.5334 -1.15706
+1.14842 0.928019 -1.29583
+1.54952 0.355974 -1.58522
+0.982056 1.16481 -0.597175
+0.476802 0.919923 -0.507977
+0.366832 0.549191 -1.15785
+1.45921 1.23034 0.170868
+1.14789 0.927649 0.101481
+1.54878 0.355452 0.390874
+1.04352 0.11056 0.480072
+0.757098 0.738236 0.170472
+0.53559 0.0606295 0.730511
+0.125578 0.21555 1.09259
+0.0471035 0.819441 0.644228
+0.364231 0.699617 0.364176
+1.1683 -0.595421 0.239464
+0.382565 -0.672659 0.626872
+-0.00520413 -1.0707 0.113969
+-0.403829 -0.92008 0.46599
+-0.405416 -0.901236 0.496048
+-0.620442 -0.621786 0.997254
+-0.25169 -0.433009 1.18697
+-0.105304 0.544637 1.49459
+-0.653543 -0.51838 1.12035
+-1.39111 2.30732 2.68795
+-0.899037 1.82378 2.99766
+-1.46576 2.63314 2.00961
+-0.622986 3.45256 2.19745
+-0.457151 3.2154 2.89611
+-1.41963 1.89834 0.971979
+-1.07926 2.61038 1.36002
+-0.45802 2.25237 1.11969
+-0.456624 3.21576 1.49879
+0.324963 3.59459 1.36081
+0.434933 3.96532 2.01068
+0.957671 2.93854 0.869764
+1.1107 3.67183 0.973404
+-0.031132 0.869992 -0.257539
+0.491605 -0.156789 -1.39846
+1.12903 2.29956 1.2361
+1.74495 3.43218 0.413301
+2.10211 2.92311 -0.208562
+2.84039 3.05644 -0.113547
+-0.400567 -0.900322 -0.507746
+-0.446838 -0.882839 -0.466885
+-0.396754 -0.907847 -0.229717
+-0.561015 -0.673257 -0.516099
+-1.70695 0.944633 0.59665
+-1.3247 0.440138 -0.0497789
+-1.90392 1.27273 1.13283
+-1.74516 1.06687 0.748005
+1.01646 2.07925 2.10825
+1.91216 2.52722 2.37072
+-2.23628 -0.701564 0.707986
+-2.26637 -0.154647 0.171488
+-2.52431 0.161632 0.57852
+-2.43579 -0.456935 1.02281
+-1.94973 -0.658965 1.58028
+-2.38146 -0.524773 1.93784
+-2.0187 -0.118527 2.47749
+-1.35085 -0.32611 1.92438
+-1.65871 -1.41097 0.797358
+-2.2107 -1.03858 0.417248
+-1.69178 -0.975244 1.17325
+-0.951822 -0.815367 1.29474
+-1.2075 -2.2147 0.295123
+-1.85706 -1.87983 0.527054
+-0.900662 -1.4894 0.713261
+-0.38062 -1.22792 0.21422
+-1.61356 -3.06634 -0.584539
+-1.81155 -2.84761 0.123141
+-0.795936 -2.20965 -0.351752
+-2.94201 -3.4398 -1.13026
+-2.67155 -3.6283 -0.438035
+-1.76955 -2.81704 -1.29261
+-4.33495 -2.96364 -0.94302
+-4.14862 -3.61261 -0.579731
+-3.24614 -2.80771 -1.4398
+-4.87804 -2.00991 -0.265205
+-5.23892 -2.68267 -0.19436
+-4.21019 -2.2175 -0.818312
+-4.53711 -1.36078 0.485927
+-5.27707 -1.52065 0.364442
+-4.27916 -1.67706 0.0788946
+-4.01819 -1.29744 1.24193
+-4.57018 -0.925055 0.861821
+-3.99261 -1.63446 0.951193
+-3.79724 -1.27678 2.13389
+-3.86477 -0.934793 1.451
+-3.99559 -1.74565 1.86358
+-3.60475 -0.648909 2.95275
+-3.40675 -0.867637 2.24507
+-4.2088 -1.28183 2.78076
+-2.98275 0.471686 3.09898
+-2.82311 -0.233732 2.84451
+-4.04074 -0.0902782 3.24548
+-2.08027 1.27659 2.23891
+-1.89394 0.627623 2.6022
+-3.28688 1.10378 2.78944
+-2.46853 1.44822 1.60043
+-3.98134 -0.0470402 1.2433
+-3.56274 -0.618335 1.53699
+-3.17192 -0.410747 1.48287
+-3.47605 0.221349 1.17333
+-1.37052 -1.38906 -1.35352
+-0.863465 -1.86766 -1.03464
+-0.955208 -0.407325 -0.787544
+-1.71325 -0.328896 -0.703447
+-2.23329 -0.59037 -0.204406
+-2.43165 -1.05924 -0.47471
+-2.02009 -1.0542 -1.12159
+-2.62414 -1.68711 -1.29357
+-2.18815 -2.24574 -1.5863
+-2.82214 -1.46838 -0.585887
+-3.32743 -1.73677 -0.515917
+-3.40578 -2.10229 -1.18533
+-2.36412 -1.40123 0.208179
+-2.66615 -1.71769 0.122188
+-2.24755 -2.28898 0.415877
+-2.75284 -2.55737 0.485847
+-3.05697 -1.92527 0.176307
+-3.45917 -1.98288 0.359816
+-3.84743 -1.81125 -0.27866
+-3.27284 -2.63185 0.723104
+-3.70458 -2.49765 1.08066
+-3.7931 -1.87909 0.636369
+-2.59319 -3.26279 0.231378
+-3.3976 -3.37799 0.598396
+-3.76036 -3.78424 0.0587442
+-4.66433 -3.50327 0.807404
+-4.06545 -3.17041 1.1515
+-4.46448 -2.68115 1.78115
+-3.96252 -2.18138 1.48769
+-5.20443 -2.84103 1.65966
+-5.27368 -1.9287 2.44672
+-4.51564 -2.00713 2.36263
+-5.1504 -3.30124 0.249929
+-5.74894 -2.56735 1.1944
+-5.36542 -0.468358 2.69382
+-4.85836 -0.946965 3.0127
+-5.82568 -1.55631 2.06661
+-5.43295 -0.126368 2.01093
+-5.84827 -1.1081 1.44496
+-5.32823 -0.846626 0.945918
+-5.02139 -0.121326 1.36406
+-4.37183 -0.456187 1.13212
+-4.41734 0.511591 1.53604
+-4.61533 0.730319 2.24372
+-3.6357 0.926767 1.4278
+-3.55734 1.29228 2.09721
+-2.95605 0.295824 0.936075
+-2.83129 1.04197 1.06078
+-2.76972 -0.353144 1.29936
+-2.90146 -0.599248 2.1751
+-2.16344 0.83439 0.507675
+-1.76441 0.345132 -0.121971
+-5.77902 -2.02043 0.6579
+4 0 1 2 3 
+4 4 5 0 6 
+4 7 8 9 10 
+4 11 12 4 13 
+4 14 10 15 16 
+4 17 18 11 19 
+4 20 21 17 22 
+6 23 24 25 26 27 28 
+4 29 30 23 31 
+4 32 33 29 34 
+4 35 36 32 37 
+4 38 39 35 40 
+4 41 42 38 43 
+4 44 45 41 46 
+4 47 48 49 44 
+4 50 51 52 53 
+4 54 55 56 57 
+7 58 59 60 61 62 63 64 
+4 65 66 16 67 
+4 68 69 70 71 
+4 65 67 68 71 
+4 72 73 65 71 
+4 74 75 76 72 
+4 70 74 72 71 
+4 74 70 77 78 
+4 78 79 80 81 
+4 75 74 78 81 
+4 82 83 84 85 
+4 80 82 85 81 
+4 82 80 86 87 
+6 87 88 89 90 91 92 
+4 83 82 87 92 
+4 60 93 83 92 
+5 92 91 94 61 60 
+4 58 64 55 54 
+4 95 96 58 54 
+4 97 98 99 95 
+4 57 97 95 54 
+4 97 57 100 101 
+4 101 102 50 103 
+4 98 97 101 103 
+4 104 105 98 103 
+4 53 104 103 50 
+4 104 53 106 107 
+4 105 104 107 47 
+4 46 105 47 44 
+4 99 98 105 46 
+4 43 99 46 41 
+4 96 95 99 43 
+4 40 96 43 38 
+4 59 58 96 40 
+4 37 59 40 35 
+4 93 60 59 37 
+4 34 93 37 32 
+4 84 83 93 34 
+4 28 108 31 23 
+4 73 72 76 109 
+4 22 73 109 20 
+4 66 65 73 22 
+4 19 66 22 17 
+4 14 16 66 19 
+4 13 14 19 11 
+4 7 10 14 13 
+4 6 7 13 4 
+4 106 53 52 110 
+4 2 106 110 3 
+4 107 106 2 111 
+4 48 47 107 111 
+4 112 68 67 113 
+4 112 113 48 111 
+4 112 111 2 1 
+4 69 68 112 1 
+4 69 1 0 5 
+4 77 70 69 5 
+4 77 5 4 12 
+4 79 78 77 12 
+4 79 12 11 18 
+4 86 80 79 18 
+4 86 18 17 21 
+4 88 87 86 21 
+4 88 21 20 114 
+5 88 114 115 116 89 
+5 27 26 117 115 114 
+5 30 118 119 24 23 
+7 100 57 56 120 121 118 30 
+4 100 30 29 33 
+4 102 101 100 33 
+4 102 33 32 36 
+4 51 50 102 36 
+4 51 36 35 39 
+4 122 52 51 39 
+4 122 39 38 42 
+4 123 42 41 45 
+4 9 8 123 45 
+4 9 45 44 49 
+4 15 10 9 49 
+4 15 49 48 113 
+4 67 16 15 113 
+4 124 125 126 127 
+4 128 129 130 131 
+4 132 133 124 134 
+5 135 131 62 61 94 
+4 136 137 132 138 
+4 139 90 89 116 
+4 140 141 136 142 
+4 143 144 140 145 
+4 146 147 143 148 
+4 149 150 146 151 
+4 152 153 149 154 
+4 155 156 152 157 
+4 158 159 155 160 
+4 161 162 158 163 
+4 164 165 161 166 
+4 167 168 164 169 
+5 170 120 56 55 167 
+4 171 172 173 174 
+5 175 176 115 117 177 
+4 178 179 180 181 
+4 175 177 178 181 
+4 182 183 175 181 
+4 184 185 186 182 
+4 180 184 182 181 
+4 184 180 187 188 
+4 188 189 190 191 
+4 185 184 188 191 
+4 192 193 185 191 
+4 194 195 196 192 
+4 190 194 192 191 
+4 194 190 197 198 
+4 198 199 200 201 
+4 195 194 198 201 
+4 202 203 195 201 
+4 204 205 206 202 
+4 200 204 202 201 
+4 204 200 207 208 
+4 209 210 205 211 
+4 212 213 214 215 
+4 215 216 171 217 
+4 218 212 215 217 
+4 219 220 218 217 
+4 174 219 217 171 
+4 219 174 221 222 
+4 220 219 222 170 
+4 169 220 170 167 
+4 163 210 166 161 
+4 206 205 210 163 
+4 160 206 163 158 
+4 203 202 206 160 
+4 157 203 160 155 
+4 196 195 203 157 
+4 154 196 157 152 
+4 193 192 196 154 
+4 151 193 154 149 
+4 186 185 193 151 
+4 148 186 151 146 
+4 183 182 186 148 
+4 145 183 148 143 
+4 176 175 183 145 
+4 142 176 145 140 
+5 139 116 115 176 142 
+4 138 139 142 136 
+6 135 94 91 90 139 138 
+4 134 135 138 132 
+4 128 131 135 134 
+4 127 128 134 124 
+4 129 128 127 223 
+4 223 173 224 129 
+4 221 174 173 223 
+4 126 221 223 127 
+4 222 221 126 225 
+5 170 222 225 121 120 
+6 226 178 177 25 24 119 
+5 226 119 118 121 225 
+4 226 225 126 125 
+4 179 178 226 125 
+4 179 125 124 133 
+4 187 180 179 133 
+4 187 133 132 137 
+4 189 188 187 137 
+4 189 137 136 141 
+4 197 190 189 141 
+4 197 141 140 144 
+4 199 198 197 144 
+4 199 144 143 147 
+4 207 200 199 147 
+4 207 147 146 150 
+4 227 208 207 150 
+4 227 150 149 153 
+4 214 213 227 153 
+4 214 153 152 156 
+4 216 215 214 156 
+4 216 156 155 159 
+4 172 171 216 159 
+4 172 159 158 162 
+4 224 173 172 162 
+4 224 162 161 165 
+4 130 129 224 165 
+4 130 165 164 168 
+5 131 130 168 63 62 
+5 167 55 64 63 168 
+4 177 117 26 25 
diff -pruN 5.0.3-1/examples/Surface_mesh_topology/data/double-torus.off 5.1-1/examples/Surface_mesh_topology/data/double-torus.off
--- 5.0.3-1/examples/Surface_mesh_topology/data/double-torus.off	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_topology/data/double-torus.off	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,453 @@
+OFF
+231 220 0
+-0.955208 -0.407325 -0.787544
+-2.08027 1.27659 2.23891
+-1.89394 0.627623 2.6022
+-1.35085 -0.32611 1.92438
+-2.0187 -0.118527 2.47749
+-2.98275 0.471686 3.09898
+-2.82311 -0.233732 2.84451
+-2.38146 -0.524773 1.93784
+-2.90146 -0.599248 2.1751
+-3.60475 -0.648909 2.95275
+-3.40675 -0.867637 2.24507
+-3.17192 -0.410747 1.48287
+-3.56274 -0.618335 1.53699
+-3.79724 -1.27678 2.13389
+-3.86477 -0.934793 1.451
+-3.98134 -0.0470402 1.2433
+-4.37183 -0.456187 1.13212
+-4.01819 -1.29744 1.24193
+-4.57018 -0.925055 0.861821
+-5.02139 -0.121326 1.36406
+-5.32823 -0.846626 0.945918
+-4.53711 -1.36078 0.485927
+-5.27707 -1.52065 0.364442
+-5.84827 -1.1081 1.44496
+-5.77902 -2.02043 0.6579
+-4.87804 -2.00991 -0.265205
+-5.23892 -2.68267 -0.19436
+-5.74894 -2.56735 1.1944
+-5.1504 -3.30124 0.249929
+-4.33495 -2.96364 -0.94302
+-4.14862 -3.61261 -0.579731
+-4.66433 -3.50327 0.807404
+-3.76036 -3.78424 0.0587442
+-2.94201 -3.4398 -1.13026
+-2.67155 -3.6283 -0.438035
+-3.3976 -3.37799 0.598396
+-2.59319 -3.26279 0.231378
+-1.61356 -3.06634 -0.584539
+-1.81155 -2.84761 0.123141
+-2.75284 -2.55737 0.485847
+-2.24755 -2.28898 0.415877
+-1.2075 -2.2147 0.295123
+-1.85706 -1.87983 0.527054
+-2.66615 -1.71769 0.122188
+-2.36412 -1.40123 0.208179
+-1.65871 -1.41097 0.797358
+-2.2107 -1.03858 0.417248
+-2.43165 -1.05924 -0.47471
+-2.23329 -0.59037 -0.204406
+-2.23628 -0.701564 0.707986
+-2.26637 -0.154647 0.171488
+-1.71325 -0.328896 -0.703447
+-1.76441 0.345132 -0.121971
+-2.52431 0.161632 0.57852
+-2.16344 0.83439 0.507675
+-2.46853 1.44822 1.60043
+-2.83129 1.04197 1.06078
+-2.95605 0.295824 0.936075
+-2.43579 -0.456935 1.02281
+-2.76972 -0.353144 1.29936
+-3.47605 0.221349 1.17333
+-1.94973 -0.658965 1.58028
+-1.69178 -0.975244 1.17325
+-0.951822 -0.815367 1.29474
+-0.900662 -1.4894 0.713261
+-0.38062 -1.22792 0.21422
+-0.795936 -2.20965 -0.351752
+-0.863465 -1.86766 -1.03464
+-1.76955 -2.81704 -1.29261
+-1.37052 -1.38906 -1.35352
+-2.18815 -2.24574 -1.5863
+-3.24614 -2.80771 -1.4398
+-2.62414 -1.68711 -1.29357
+-3.40578 -2.10229 -1.18533
+-4.21019 -2.2175 -0.818312
+-3.32743 -1.73677 -0.515917
+-3.84743 -1.81125 -0.27866
+-4.27916 -1.67706 0.0788946
+-3.45917 -1.98288 0.359816
+-3.7931 -1.87909 0.636369
+-3.99261 -1.63446 0.951193
+-3.70458 -2.49765 1.08066
+-3.96252 -2.18138 1.48769
+-3.99559 -1.74565 1.86358
+-4.46448 -2.68115 1.78115
+-4.51564 -2.00713 2.36263
+-4.2088 -1.28183 2.78076
+-5.27368 -1.9287 2.44672
+-4.85836 -0.946965 3.0127
+-4.04074 -0.0902782 3.24548
+-5.36542 -0.468358 2.69382
+-4.45934 0.481017 2.95179
+-3.28688 1.10378 2.78944
+-4.61533 0.730319 2.24372
+-3.55734 1.29228 2.09721
+-3.6357 0.926767 1.4278
+-4.41734 0.511591 1.53604
+-5.43295 -0.126368 2.01093
+-5.82568 -1.55631 2.06661
+-5.20443 -2.84103 1.65966
+-4.06545 -3.17041 1.1515
+-3.27284 -2.63185 0.723104
+-3.05697 -1.92527 0.176307
+-2.82214 -1.46838 -0.585887
+-2.02009 -1.0542 -1.12159
+2.84039 3.05644 -0.113547
+2.853 2.13918 -0.872153
+3.39492 1.75991 -0.50561
+3.38094 2.77641 0.335073
+2.35696 3.74549 0.728177
+2.84521 3.54002 1.26787
+3.40711 2.24707 0.874502
+2.92369 2.93613 1.71623
+1.89383 3.89949 2.10802
+2.06519 3.26051 2.47436
+2.54642 2.28757 1.81061
+1.91216 2.52722 2.37072
+1.00166 3.15596 2.99873
+1.12643 2.44998 2.75813
+1.52439 2.12918 1.85781
+1.01646 2.07925 2.10825
+0.344842 2.07115 2.89611
+0.511204 1.83436 2.19745
+0.736163 2.26094 1.4298
+0.34537 2.07152 1.49879
+0.108702 1.44301 2.10774
+0.034051 1.76883 1.42941
+-0.0555178 2.64372 1.2094
+-0.45802 2.25237 1.11969
+-0.136133 1.41281 1.23539
+-0.678049 1.79208 0.868843
+-1.07926 2.61038 1.36002
+-1.41963 1.89834 0.971979
+-0.672026 1.354 0.506539
+-1.41031 1.22067 0.411524
+-1.92646 1.66935 1.48438
+-1.05316 0.711603 -0.210338
+-0.571926 -0.261335 -0.874081
+-0.400567 -0.900322 -0.507746
+-0.00520413 -1.0707 0.113969
+0.778032 -0.784465 -1.08886
+1.05833 -0.966152 -0.410409
+0.382565 -0.672659 0.626872
+1.1683 -0.595421 0.239464
+2.11625 -0.453389 -0.597175
+1.94988 -0.216593 0.101481
+1.04352 0.11056 0.480072
+1.54878 0.355452 0.390874
+2.57252 0.388793 0.240254
+1.95128 0.746801 0.480583
+1.14789 0.927649 0.101481
+1.45921 1.23034 0.170868
+2.17131 1.2071 0.731432
+1.62939 1.58636 0.364889
+1.38456 1.55616 -0.507466
+1.60459 2.01646 -0.256618
+1.62473 1.92519 0.645116
+1.59856 2.45453 0.105686
+2.11142 2.24544 -0.769017
+2.10211 2.92311 -0.208562
+1.36768 2.78362 0.507687
+1.74495 3.43218 0.413301
+1.49846 4.06987 1.48631
+1.1107 3.67183 0.973404
+0.957671 2.93854 0.869764
+1.44616 2.17973 0.956047
+1.12903 2.29956 1.2361
+0.449736 2.88861 1.1202
+1.9344 1.97426 1.49574
+2.16529 1.64517 1.09374
+2.90357 1.7785 1.18875
+2.91289 1.10083 0.628296
+3.41972 1.32982 0.115897
+2.95902 0.366033 -0.409339
+2.88437 0.691855 -1.08767
+1.95041 -0.216224 -1.29583
+2.3923 1.1754 -1.39739
+1.54952 0.355974 -1.58522
+0.491605 -0.156789 -1.39846
+1.14842 0.928019 -1.29583
+0.366832 0.549191 -1.15785
+-0.418901 0.471953 -0.770442
+0.476802 0.919923 -0.507977
+-0.031132 0.869992 -0.257539
+-0.441144 1.02491 0.104538
+0.364231 0.699617 0.364176
+0.0471035 0.819441 0.644228
+-0.131474 1.07398 0.955159
+0.125578 0.21555 1.09259
+-0.105304 0.544637 1.49459
+-0.111327 0.982712 1.85689
+-0.608846 0.076062 1.80884
+-0.618163 0.753727 2.36929
+-0.277795 1.46577 2.75733
+-1.35974 0.859992 2.47243
+-0.899037 1.82378 2.99766
+-0.0562639 2.6432 3.1855
+-1.39111 2.30732 2.68795
+-0.457151 3.2154 2.89611
+0.715229 3.78364 2.68913
+-0.622986 3.45256 2.19745
+0.434933 3.96532 2.01068
+0.324963 3.59459 1.36081
+-0.456624 3.21576 1.49879
+-1.46576 2.63314 2.00961
+-1.90166 1.23926 2.10589
+-0.25169 -0.433009 1.18697
+0.53559 0.0606295 0.730511
+0.757098 0.738236 0.170472
+0.982056 1.16481 -0.597175
+1.77106 1.5334 -1.15706
+-0.561015 -0.673257 -0.516099
+-0.97055 -0.110718 -0.522061
+-1.90141 1.221 2.09078
+-1.35435 0.467997 2.14823
+-1.32934 0.452132 2.1346
+-1.0988 0.14376 1.8749
+-1.74516 1.06687 0.748005
+-1.70695 0.944633 0.59665
+-1.3247 0.440138 -0.0497789
+-1.28047 0.320835 -0.153469
+-0.446838 -0.882839 -0.466885
+-0.396754 -0.907847 -0.229717
+-0.653543 -0.51838 1.12035
+-0.620442 -0.621786 0.997254
+-0.403829 -0.92008 0.46599
+-1.90392 1.27273 1.13283
+-1.92107 1.27736 1.16019
+-1.04378 -0.00248243 1.75286
+-0.980005 -0.0806305 -0.505709
+-0.405416 -0.901236 0.496048
+4 0 229 212 211 
+5 1 2 215 214 213 
+5 3 216 215 2 4 
+4 4 2 5 6 
+4 4 6 8 7 
+4 8 6 9 10 
+4 8 10 12 11 
+4 12 10 13 14 
+4 12 14 16 15 
+4 16 14 17 18 
+4 16 18 20 19 
+4 20 18 21 22 
+4 20 22 24 23 
+4 24 22 25 26 
+4 24 26 28 27 
+4 28 26 29 30 
+4 28 30 32 31 
+4 32 30 33 34 
+4 32 34 36 35 
+4 36 34 37 38 
+4 36 38 40 39 
+4 40 38 41 42 
+4 40 42 44 43 
+4 44 42 45 46 
+4 44 46 48 47 
+4 48 46 49 50 
+4 48 50 52 51 
+4 52 50 53 54 
+5 52 54 217 218 219 
+6 52 219 220 229 0 51 
+5 55 226 217 54 56 
+4 56 54 53 57 
+4 53 58 59 57 
+4 57 59 11 60 
+4 59 7 8 11 
+4 7 59 58 61 
+4 58 49 62 61 
+4 61 62 63 3 
+4 62 45 64 63 
+6 63 64 65 230 224 223 
+4 64 41 66 65 
+5 65 66 67 221 222 
+4 66 37 68 67 
+4 67 68 70 69 
+4 68 33 71 70 
+4 70 71 73 72 
+4 71 29 74 73 
+4 73 74 76 75 
+4 74 25 77 76 
+4 76 77 79 78 
+4 77 21 80 79 
+4 79 80 82 81 
+4 80 17 83 82 
+4 82 83 85 84 
+4 83 13 86 85 
+4 85 86 88 87 
+4 86 9 89 88 
+4 88 89 91 90 
+4 89 5 92 91 
+4 91 92 94 93 
+4 92 1 55 94 
+4 94 55 56 95 
+4 95 56 57 60 
+4 60 15 96 95 
+4 95 96 93 94 
+4 93 96 19 97 
+4 19 96 15 16 
+4 97 19 20 23 
+4 23 98 90 97 
+4 97 90 91 93 
+4 90 98 87 88 
+4 87 98 27 99 
+4 27 98 23 24 
+4 99 27 28 31 
+4 31 100 84 99 
+4 99 84 85 87 
+4 84 100 81 82 
+4 81 100 35 101 
+4 35 100 31 32 
+4 101 35 36 39 
+4 39 102 78 101 
+4 101 78 79 81 
+4 78 102 75 76 
+4 75 102 43 103 
+4 43 102 39 40 
+4 103 43 44 47 
+4 47 104 72 103 
+4 103 72 73 75 
+4 72 104 69 70 
+4 69 104 51 0 
+4 51 104 47 48 
+5 69 0 211 221 67 
+4 15 60 11 12 
+5 55 1 213 227 226 
+4 1 92 5 2 
+4 5 89 9 6 
+4 9 86 13 10 
+4 13 83 17 14 
+4 17 80 21 18 
+4 21 77 25 22 
+4 25 74 29 26 
+4 29 71 33 30 
+4 33 68 37 34 
+4 37 66 41 38 
+4 65 222 225 230 
+4 41 64 45 42 
+5 63 223 228 216 3 
+4 45 62 49 46 
+4 61 3 4 7 
+4 49 58 53 50 
+4 106 105 108 107 
+4 108 105 109 110 
+4 108 110 112 111 
+4 112 110 113 114 
+4 112 114 116 115 
+4 116 114 117 118 
+4 116 118 120 119 
+4 120 118 121 122 
+4 120 122 124 123 
+4 124 122 125 126 
+4 124 126 128 127 
+4 128 126 129 130 
+4 128 130 132 131 
+4 132 130 133 134 
+7 132 134 218 217 226 227 135 
+5 134 136 220 219 218 
+5 137 138 221 211 212 
+5 139 225 222 221 138 
+4 139 138 140 141 
+4 139 141 143 142 
+4 143 141 144 145 
+4 143 145 147 146 
+4 147 145 148 149 
+4 147 149 151 150 
+4 151 149 152 153 
+4 151 153 155 154 
+4 155 153 156 157 
+4 155 157 159 158 
+4 159 157 160 161 
+4 159 161 109 105 
+4 159 105 106 158 
+4 109 161 163 162 
+4 163 161 160 164 
+4 160 165 166 164 
+4 164 166 123 167 
+4 166 119 120 123 
+4 119 166 165 168 
+4 165 156 169 168 
+4 168 169 170 115 
+4 169 152 171 170 
+4 170 171 172 111 
+4 171 148 173 172 
+4 172 173 174 107 
+4 173 144 175 174 
+4 174 175 177 176 
+4 175 140 178 177 
+4 177 178 180 179 
+4 178 137 181 180 
+4 180 181 183 182 
+4 181 136 184 183 
+4 183 184 186 185 
+4 184 133 187 186 
+4 186 187 189 188 
+4 187 129 190 189 
+4 189 190 192 191 
+4 190 125 193 192 
+4 192 193 195 194 
+4 193 121 196 195 
+4 195 196 198 197 
+4 196 117 199 198 
+4 198 199 201 200 
+4 199 113 162 201 
+4 201 162 163 202 
+4 202 163 164 167 
+4 167 127 203 202 
+4 202 203 200 201 
+4 200 203 131 204 
+4 131 203 127 128 
+4 204 131 132 135 
+4 135 205 197 204 
+4 204 197 198 200 
+4 197 205 194 195 
+4 194 205 213 214 
+5 206 191 228 223 224 
+4 191 206 188 189 
+4 188 206 142 207 
+6 142 206 224 230 225 139 
+4 207 142 143 146 
+4 146 208 185 207 
+4 207 185 186 188 
+4 185 208 182 183 
+4 182 208 150 209 
+4 150 208 146 147 
+4 209 150 151 154 
+4 154 210 179 209 
+4 209 179 180 182 
+4 179 210 176 177 
+4 176 210 158 106 
+4 158 210 154 155 
+4 176 106 107 174 
+7 194 214 215 216 228 191 192 
+4 205 135 227 213 
+4 127 167 123 124 
+4 162 113 110 109 
+4 113 199 117 114 
+4 117 196 121 118 
+4 121 193 125 122 
+4 125 190 129 126 
+4 129 187 133 130 
+4 133 184 136 134 
+6 136 181 137 212 229 220 
+4 137 178 140 138 
+4 140 175 144 141 
+4 144 173 148 145 
+4 172 107 108 111 
+4 148 171 152 149 
+4 170 111 112 115 
+4 152 169 156 153 
+4 168 115 116 119 
+4 156 165 160 157 
diff -pruN 5.0.3-1/examples/Surface_mesh_topology/data/elephant.off 5.1-1/examples/Surface_mesh_topology/data/elephant.off
--- 5.0.3-1/examples/Surface_mesh_topology/data/elephant.off	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_topology/data/elephant.off	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,8337 @@
+OFF
+2775 5558 0
+
+0.262933 0.102269 0.138247
+0.0843142 0.0418575 -0.0419302
+0.0676609 -0.0308717 0.133371
+0.202895 0.468475 0.0802072
+0.113075 -0.465378 -0.0546734
+0.225577 -0.277149 -0.193776
+-0.146525 -0.22403 -0.169286
+-0.0249865 -0.129931 -0.13238
+-0.160965 -0.480027 -0.0707824
+0.0794063 -0.415277 -0.0839096
+0.0469116 -0.050008 0.252355
+-0.0998372 -0.193745 -0.16268
+-0.111131 -0.104825 -0.14166
+-0.061459 -0.0977343 -0.133353
+-0.247783 -0.131539 0.0722694
+-0.145972 -0.486627 -0.0633275
+0.0916759 0.0333604 -0.133002
+-0.184954 -0.325468 -0.133853
+0.0847778 -0.432659 -0.0978527
+-0.210776 -0.299151 0.0751516
+-0.151539 -0.388715 0.01282
+-0.154564 -0.438244 -0.0249412
+-0.123411 -0.0182656 0.0135824
+0.00773278 -0.053391 -0.0428874
+-0.0774524 -0.292395 -0.106335
+0.0201594 -0.263586 0.119859
+-0.105646 -0.0365155 -0.0770751
+-0.113391 -0.41896 -0.0906771
+-0.128213 -0.46259 -0.0994907
+-0.160526 -0.468057 -0.0393445
+0.197609 -0.127492 -0.0574767
+0.0757306 -0.449539 -0.0614557
+0.147148 -0.412461 -0.0801639
+0.103782 -0.333007 0.0288926
+0.108917 -0.401981 -0.00841926
+0.11392 -0.408931 -0.0973654
+0.129733 -0.31331 -0.0672325
+0.0904388 -0.356531 -0.0803906
+0.0359798 -0.255445 -0.113562
+-0.22312 -0.107007 -0.0611904
+-0.163408 -0.433458 -0.0704785
+-0.0915185 -0.473885 -0.0165533
+-0.141417 -0.474418 -0.0834065
+-0.0944539 -0.479454 -0.0663683
+-0.085985 -0.101282 0.158093
+-0.0225438 -0.0889567 -0.0848587
+0.092707 -0.109496 -0.114561
+-0.159213 -0.159272 -0.138793
+-0.24541 -0.230248 -0.110578
+0.183744 -0.231931 0.0176086
+0.294273 -0.0955008 0.116328
+-0.16477 -0.385872 -0.0413331
+-0.185303 -0.313456 -0.00374653
+-0.297084 -0.262835 -0.00415653
+0.167756 -0.128776 0.197921
+-0.0273811 0.0608812 -0.0989637
+0.208303 0.00452459 -0.0746544
+-0.150831 -0.448909 -0.0901866
+-0.0720206 -0.147118 -0.154796
+-0.0197986 -0.210658 -0.154154
+-0.130593 -0.498558 -0.021161
+0.129662 -0.373473 -0.0794411
+-0.120692 -0.481168 -0.0685411
+-0.108692 -0.494274 -0.0429275
+-0.0970902 -0.252938 -0.14843
+-0.0098054 -0.335319 -0.0147292
+0.0769246 -0.410346 -0.0391239
+-0.132409 -0.464154 -0.0197707
+-0.102806 -0.4217 0.00855496
+-0.0760332 -0.34128 0.100617
+-0.104435 -0.377391 0.0553722
+-0.14788 -0.33188 0.0922673
+-0.151409 -0.218926 0.165812
+-0.111518 -0.286925 0.148566
+-0.0458796 -0.210094 0.18627
+0.0439919 -0.147905 0.137964
+-0.0358575 -0.361749 0.0457868
+0.0284346 -0.311464 0.0537885
+0.0409609 -0.0869988 -0.085353
+0.118462 -0.0465078 -0.0483438
+-0.141108 -0.422593 -0.0846889
+-0.129145 -0.379198 -0.0712284
+-0.149426 -0.329507 -0.0473837
+-0.212723 -0.300322 -0.0706413
+-0.156151 -0.403775 -0.0651516
+-0.077704 -0.400019 -0.0437424
+-0.0783601 -0.342543 -0.0687412
+-0.0202922 -0.309739 -0.0758353
+0.0501325 -0.290557 -0.0430839
+0.0954448 -0.256567 0.0733017
+0.144565 -0.158299 0.0913557
+0.0562207 -0.179498 -0.140905
+0.16863 -0.175124 -0.176771
+0.121523 -0.241503 -0.138152
+0.196749 0.136407 0.00365942
+0.14057 0.277481 0.154966
+0.156168 -0.385879 -0.0324162
+-0.146564 -0.288069 -0.168907
+-0.212533 -0.256019 -0.170893
+0.0775241 -0.353084 -0.0221894
+-0.161054 -0.48311 -0.0516067
+-0.151386 -0.488726 -0.0297486
+-0.167299 -0.464731 -0.058517
+-0.167494 -0.44542 -0.0440266
+-0.166175 -0.416312 -0.0405929
+-0.155665 -0.409399 -0.0112037
+-0.130777 -0.428881 -0.00549876
+-0.162855 -0.460869 -0.0783452
+-0.0949104 -0.0628065 -0.118829
+-0.171364 -0.0681205 -0.104606
+-0.189411 -0.0391257 -0.0301772
+-0.201329 -0.0450332 0.0672375
+-0.0621659 -0.0595357 -0.0819357
+-0.0724451 -0.0386843 -0.0212262
+-0.0317016 -0.00827391 0.0812953
+0.125617 -0.445138 -0.086335
+0.16914 -0.45638 -0.0514974
+-0.188699 -0.102786 0.151505
+-0.127982 -0.406882 0.0143939
+-0.130706 -0.384 0.0373578
+-0.17253 -0.34721 0.0442322
+-0.136499 -0.357829 0.0655808
+-0.101729 -0.398953 0.0314689
+-0.06477 -0.393705 0.00175031
+-0.0683606 -0.382507 0.0422398
+-0.0619729 -0.361557 0.0725343
+-0.0129886 -0.329893 0.099078
+-0.0485077 -0.306718 0.14972
+-0.0586671 -0.359537 -0.0297503
+-0.105132 -0.354471 0.0834642
+-0.111953 -0.324339 0.110194
+-0.146952 -0.298663 0.122985
+-0.147935 -0.259899 0.146585
+-0.224685 -0.230598 0.12386
+-0.194785 -0.274899 0.125615
+-0.1144 -0.250007 0.185149
+-0.10665 -0.185071 0.204627
+-0.155805 -0.154327 0.174517
+-0.215755 -0.165279 0.142947
+-0.0743299 -0.277016 0.17853
+-0.0218198 -0.257755 0.16445
+0.000800113 -0.199671 0.138749
+-0.0254547 -0.133829 0.140633
+0.0635315 -0.20852 0.111271
+-0.0256804 -0.0569419 0.140514
+-0.0948894 -0.0414189 0.110395
+-0.0705488 -0.0374163 0.0415111
+-0.00760713 -0.0195458 0.0164934
+0.0499878 0.0289775 0.0673286
+0.140466 0.0718009 0.144428
+-0.00188983 0.0792593 -0.003093
+0.0559872 -0.0140882 -0.011506
+-0.227407 -0.0888195 0.0117762
+-0.273528 -0.167121 -0.0102922
+-0.0402782 -0.263586 -0.140793
+-0.137564 -0.296782 -0.109847
+-0.166109 -0.381665 -0.0120175
+-0.16983 -0.360755 0.0137933
+-0.16456 -0.354211 -0.0308766
+-0.185156 -0.334386 0.0190529
+-0.207978 -0.298552 0.0310885
+-0.252507 -0.243462 0.0511336
+-0.223388 -0.268692 -0.019703
+-0.193938 -0.322649 0.048456
+-0.175095 -0.332532 0.0736524
+-0.176111 -0.310734 0.103586
+-0.147337 -0.494175 -0.0467278
+-0.129359 -0.490613 -0.0537016
+-0.148829 -0.364106 -0.0543963
+-0.120087 -0.343197 -0.0662243
+-0.130345 -0.305692 -0.0722136
+-0.171529 -0.299424 -0.0833938
+-0.181343 -0.292949 -0.0416997
+0.207831 -0.217496 -0.0966998
+0.180453 0.174523 0.151623
+0.0997558 -0.44202 -0.0211251
+0.141829 -0.427452 -0.0246256
+0.231902 -0.0518172 0.0262484
+0.222165 -0.00765217 0.131632
+0.257605 0.0468554 0.0496674
+0.134457 -0.362329 0.00455646
+0.162364 -0.298449 0.0119315
+0.167632 -0.334853 -0.0258239
+0.166781 -0.258058 -0.0451734
+0.196618 -0.203737 -0.0367216
+0.204966 -0.148702 0.029823
+0.188775 -0.0899398 0.0842549
+0.118735 -0.0897053 0.120666
+0.108518 -0.0623755 0.19827
+0.0943505 -0.134977 0.262277
+0.0987744 0.0186293 0.19077
+0.164162 -0.016078 0.17849
+0.217401 -0.0732307 0.174029
+0.244384 -0.159022 0.158735
+0.0938974 -0.414078 -0.0986619
+0.0882094 -0.386204 -0.0874415
+0.110237 -0.433985 -0.106491
+0.0752909 -0.377302 -0.0545033
+0.136893 -0.424401 -0.103861
+0.101623 -0.446793 -0.0879738
+0.0803589 -0.447974 -0.0828166
+0.0955794 -0.458697 -0.0690652
+0.0852254 -0.472952 -0.043714
+0.0423481 -0.131534 -0.117698
+0.0102534 -0.163639 -0.135844
+-0.0294235 -0.16777 -0.154302
+-0.0581903 -0.19678 -0.162941
+-0.0495263 -0.23206 -0.157295
+0.0213258 -0.204668 -0.142847
+0.0591534 -0.221375 -0.128414
+0.094939 -0.204956 -0.158559
+0.102368 -0.149902 -0.152017
+0.161838 -0.127189 -0.119413
+0.0883157 -0.255475 -0.0965547
+0.00337956 -0.243572 -0.138992
+-0.00441584 -0.28038 -0.109802
+0.139454 -0.244447 -0.0884262
+0.17799 -0.253819 -0.13106
+0.239512 -0.249475 -0.139746
+0.239769 -0.199663 -0.201225
+0.139899 -0.220537 -0.184787
+0.188104 -0.214287 -0.208834
+0.241446 -0.186766 -0.139849
+0.209446 -0.161297 -0.101029
+-0.199048 -0.317265 -0.100601
+-0.293252 -0.31357 -0.122798
+-0.243299 -0.355691 -0.130502
+-0.247594 -0.319033 -0.0988683
+-0.291668 -0.287409 -0.0637038
+-0.248832 -0.272185 -0.0865389
+-0.259174 -0.220646 -0.0469428
+-0.282126 -0.263379 -0.124759
+-0.269731 -0.294935 -0.18724
+-0.247297 -0.165961 -0.0866554
+0.0601914 -0.0464014 -0.0549033
+-0.211085 -0.18579 -0.128927
+-0.200721 -0.132547 -0.108259
+-0.0634953 -0.15489 0.179525
+0.153912 -0.0887253 -0.0733484
+0.185145 -0.0471144 -0.0260532
+0.211443 0.0144377 0.0032337
+0.139575 0.0029693 0.000737671
+0.166751 0.0710484 -0.0323169
+0.107842 0.107181 0.0410238
+0.241648 0.0800111 -0.0150955
+-0.140262 -0.4992 -0.033581
+-0.123763 -0.497978 -0.0364981
+-0.108133 -0.495253 -0.0202341
+-0.1219 -0.481038 -0.0158085
+-0.110456 -0.461331 -0.014079
+-0.087635 -0.441699 -0.0445804
+-0.0916102 -0.451765 -0.0239283
+-0.0811529 -0.421697 -0.020421
+-0.0723115 -0.470105 -0.0423155
+-0.102577 -0.44033 -0.0062072
+-0.139118 -0.480546 -0.0199347
+-0.149266 -0.466768 -0.0254061
+0.222317 -0.0925366 -0.0161067
+-0.0417433 -0.361447 0.00632817
+-0.00965295 -0.347119 0.0218202
+0.0216724 -0.318231 0.010836
+-0.00474667 -0.341486 0.0613973
+0.0698488 -0.285166 0.0205835
+-0.108371 -0.29389 -0.0941984
+-0.11094 -0.279255 -0.127432
+-0.0852691 -0.31423 -0.083395
+-0.0516331 -0.309713 -0.0908278
+-0.0463843 -0.329032 -0.0615785
+-0.13427 -0.295211 -0.140976
+-0.16029 -0.312528 -0.151535
+-0.208046 -0.313472 -0.188061
+-0.177638 -0.29982 -0.178676
+-0.173332 -0.259819 -0.179118
+-0.186949 -0.225139 -0.161631
+-0.121077 -0.271737 -0.158365
+-0.222098 -0.229029 -0.143015
+-0.254895 -0.254361 -0.158387
+-0.119307 -0.241249 -0.168619
+-0.0940058 -0.224454 -0.161034
+-0.119452 -0.213541 -0.164903
+-0.133444 -0.181194 -0.153714
+-0.123212 -0.142937 -0.146434
+-0.154852 -0.114002 -0.127562
+-0.16919 -0.193253 -0.151916
+-0.202578 -0.281459 -0.18846
+-0.238459 -0.276537 -0.185897
+-0.240282 -0.308392 -0.197824
+-0.31592 -0.411023 -0.222975
+-0.28556 -0.354091 -0.210885
+-0.235315 -0.349789 -0.176342
+-0.247249 -0.413591 -0.205119
+-0.313616 -0.39085 -0.145549
+-0.265064 -0.38952 -0.162643
+-0.137308 -0.0765217 -0.126585
+-0.138269 -0.043252 -0.101811
+-0.149787 -0.024537 -0.0569204
+-0.190238 -0.325155 -0.164883
+-0.214126 -0.342005 -0.145019
+-0.11169 -0.0249744 -0.0351987
+-0.154494 -0.0206052 -0.0150451
+-0.170024 -0.0235376 0.033341
+-0.149964 -0.0212297 0.0915499
+-0.146292 -0.0570396 0.143171
+-0.204533 -0.050976 0.0222898
+-0.222839 -0.0748301 0.0497298
+-0.218112 -0.0836084 0.106186
+-0.241131 -0.10804 0.0423425
+-0.253857 -0.126023 0.000914005
+-0.268045 -0.155579 0.0359862
+-0.254096 -0.191499 0.0837602
+-0.284127 -0.206805 0.0257642
+-0.255388 -0.139415 -0.0455141
+-0.266486 -0.179542 -0.0496151
+-0.255448 -0.200689 -0.0776309
+-0.238672 -0.192995 -0.107883
+-0.225097 -0.163824 -0.109937
+-0.22995 -0.135947 -0.0896828
+-0.209528 -0.114818 -0.0862766
+-0.197715 -0.0771741 -0.074156
+-0.18152 -0.102481 -0.104911
+-0.212552 -0.0754645 -0.0336012
+-0.179069 -0.0426745 -0.0757348
+-0.191113 -0.0400122 0.118805
+0.0761558 -0.344791 -0.0536197
+0.0734921 -0.316433 -0.0291825
+0.0865869 -0.298841 -0.0691012
+0.0805876 -0.328744 0.0029047
+0.0952029 -0.361348 0.00820626
+0.11906 -0.273469 -0.0673767
+0.0841053 -0.310717 0.0264162
+0.125408 -0.292597 0.041066
+0.0878905 -0.285295 0.052224
+0.0491288 -0.272854 0.0806291
+0.143869 -0.242882 0.0528378
+0.117789 -0.207199 0.0870858
+0.169353 -0.193132 0.0552639
+0.0969256 -0.166289 0.115505
+0.054006 -0.288058 0.0505697
+0.0199581 -0.301066 0.0971587
+-0.00766116 -0.296894 0.132807
+0.0766599 -0.302889 0.00259846
+0.0459981 -0.299327 0.00649094
+0.0275534 -0.307808 -0.0222832
+0.0149271 -0.300169 -0.0594622
+0.0407712 -0.276916 -0.0798141
+0.0598138 -0.291021 -0.0166231
+0.105499 -0.310063 0.042044
+0.129965 -0.318676 0.0271739
+-0.0843085 -0.494264 -0.0321529
+-0.0752323 -0.238787 0.196776
+-0.295928 -0.43397 -0.176216
+0.0851801 -0.382062 -0.0130588
+0.0187394 -0.0952484 0.145146
+0.0700063 -0.10227 0.135872
+0.0221841 -0.0461712 0.144279
+0.0234739 0.0145751 0.123876
+-0.00955997 -0.0145334 0.135796
+-0.0455413 -0.0274983 0.116817
+-0.0630042 -0.0646849 0.123712
+-0.0996182 -0.0685448 0.139479
+-0.129134 -0.0937854 0.159793
+-0.112763 -0.133107 0.183753
+-0.0586283 -0.0384282 0.0801443
+-0.0976008 -0.0306336 0.0712047
+-0.187313 -0.236737 0.146886
+-0.186919 -0.194456 0.158247
+-0.276732 -0.200888 -0.0224537
+-0.291326 -0.23573 -0.0313163
+-0.262172 -0.26119 -0.0228289
+-0.244304 -0.258186 0.0138536
+-0.238049 -0.253808 -0.053252
+-0.278468 -0.245353 0.0237178
+-0.250374 -0.233381 -0.0762153
+-0.317786 -0.266287 -0.0397057
+-0.29694 -0.227134 0.00135872
+-0.28761 -0.282597 -0.0302299
+-0.0768305 -0.203891 0.202176
+-0.107975 -0.220055 0.202264
+-0.134773 -0.20066 0.190676
+-0.13214 -0.167949 0.192848
+-0.157713 -0.187173 0.17141
+-0.0792541 -0.17391 0.197354
+-0.103166 -0.157466 0.196315
+-0.0861693 -0.139966 0.183699
+-0.0642112 -0.126048 0.16286
+-0.0525585 -0.0978366 0.139755
+0.173523 -0.0454835 0.124705
+0.124762 -0.0100876 0.132612
+0.0801162 0.0231847 0.117816
+0.0903158 0.0691509 0.0824377
+0.111706 0.138719 0.113497
+0.109897 0.0476799 0.0291314
+0.0568194 0.0789592 0.018431
+0.166721 0.186565 0.0672199
+0.252858 0.160254 0.0700128
+0.103948 0.0891733 -0.0142249
+0.151331 0.103958 0.00831307
+0.156258 0.148984 0.0332697
+0.195526 0.176469 0.0301312
+0.246249 0.159404 0.0147221
+0.272985 0.107792 0.0403664
+0.220496 0.208803 0.0718273
+0.172006 0.242405 0.105809
+0.284857 0.213191 0.163319
+0.220004 0.262975 0.168971
+0.243752 0.187223 0.124992
+0.303317 0.156118 0.132187
+0.124557 0.160014 0.070217
+0.145476 0.178627 0.113721
+0.143822 0.145597 0.146983
+0.199699 0.112576 0.148784
+0.221259 0.0552492 0.134196
+0.124553 0.109697 0.139987
+0.100062 0.0751807 0.125633
+0.107686 0.0453443 0.15701
+0.152453 0.0251604 0.154394
+0.160741 0.1045 0.158385
+0.183054 0.0708926 0.1447
+0.188656 0.0314673 0.13741
+0.286002 0.0789154 0.0896861
+0.044874 0.0868553 -0.0397086
+0.0292084 0.0351428 -0.0773123
+-0.00652383 0.0868322 -0.0563984
+0.153395 -0.330946 0.00521793
+0.165687 0.227811 0.159326
+0.176713 -0.253714 -0.180764
+0.276141 0.126578 0.0974557
+-0.0329659 -0.0648403 -0.0508016
+-0.022748 -0.0460692 -0.0136176
+-0.0397109 -0.0394184 0.0195973
+0.00993129 -0.0278328 -0.0155697
+0.0270531 -0.00832198 0.0106523
+0.0103826 0.00500549 0.0538795
+0.0666135 0.0125544 0.0261568
+0.103369 0.00889595 0.155654
+0.11659 -0.0298082 0.170544
+0.153029 -0.0696813 0.168481
+0.113461 -0.0186162 0.216944
+0.10035 -0.0580327 0.251516
+0.150235 -0.084971 0.233939
+0.0675669 -0.0946043 0.253879
+0.0719166 -0.0391569 0.214813
+0.10687 -0.103697 0.22877
+0.136543 -0.144876 0.235107
+0.147147 -0.0351617 0.149371
+0.1197 -0.0491892 0.122959
+0.148505 -0.0696344 0.112799
+0.152678 -0.114026 0.101195
+0.181641 -0.136376 0.0700095
+0.177794 -0.0667748 0.102185
+0.219377 -0.0856118 0.124697
+0.231869 -0.0572983 0.0801841
+0.250938 -0.00881912 0.0578761
+0.198815 -0.0628459 0.124187
+0.177363 -0.0676828 0.147605
+0.177944 -0.101552 0.169756
+0.213844 -0.114113 0.199683
+0.250347 -0.109904 0.165803
+0.259893 -0.130195 0.122038
+0.180633 -0.0723966 0.203599
+0.114076 -0.0955963 0.272717
+0.201886 -0.0333539 0.161782
+0.240288 -0.0485439 0.141813
+0.261312 -0.0227503 0.104643
+0.273982 -0.0593402 0.119899
+0.280362 -0.074792 0.0701015
+0.238595 0.0177583 0.0951404
+0.14643 -0.0478943 0.212489
+-0.227331 -0.265687 0.0979085
+-0.244646 -0.230401 0.0887962
+-0.293688 -0.268968 -0.164669
+-0.297979 -0.308788 -0.162348
+-0.312868 -0.346598 -0.144794
+-0.344082 -0.367401 -0.192127
+-0.317302 -0.337357 -0.184462
+-0.280828 -0.350074 -0.123607
+0.105148 0.105807 0.112878
+0.112708 0.122082 0.0774818
+0.133565 0.128779 0.0503619
+-0.153371 -0.370618 0.0324641
+0.254239 -0.0926281 0.0998171
+-0.0128437 0.0136567 0.10826
+0.0175667 0.0217155 0.0871281
+0.0477136 0.0340255 0.104217
+0.0750182 0.0489044 0.100742
+0.0776037 0.0433948 0.0700673
+0.0973389 0.0603657 0.0539823
+0.0861821 0.0686274 0.0252903
+0.0735784 0.0589072 -0.0094551
+0.0829016 0.102631 0.0164944
+0.0811061 0.0911963 0.0569078
+0.0940457 0.0476479 0.121838
+0.105428 0.0231317 0.131003
+0.0916425 -0.00738665 0.126455
+0.0604145 5.34629e-005 0.128347
+0.10359 0.0595398 0.00049955
+0.144344 0.0457444 0.00327488
+0.122523 0.0419513 -0.0300499
+0.11811 0.0162342 -0.0830375
+0.170091 0.0155571 -0.145681
+0.138389 0.0626357 -0.0743287
+0.165069 0.0235027 -0.0532363
+0.177768 0.0409007 -0.100742
+0.136707 0.0380317 -0.122381
+0.124141 0.00775387 -0.157586
+0.154959 -0.000810753 -0.105169
+0.105591 0.0562623 -0.100272
+0.0594242 0.0548004 -0.106957
+0.201162 -0.0168583 -0.120783
+0.0976411 0.0972697 0.0801317
+0.0943337 0.0848541 0.102457
+0.0890479 0.0650811 0.109825
+0.0389773 0.0745551 -0.0779593
+0.0138551 0.0593589 -0.108474
+0.0773146 0.0765993 -0.0629692
+0.118409 0.00125651 -0.11931
+0.14578 -0.0101392 -0.137264
+0.178622 -0.0192175 -0.148664
+0.15461 0.0388676 -0.0266866
+0.193622 0.0322602 -0.0272748
+0.1942 0.050466 -0.0642106
+0.173634 0.0303747 -0.00069076
+0.179101 -0.0110744 -0.00523936
+0.230507 0.0422098 -0.0173425
+0.202418 0.070871 -0.0135348
+0.22302 0.0184239 -0.0441485
+0.215157 -0.0248187 0.00288885
+0.233855 -0.00344678 0.0259658
+0.238719 0.033163 0.0205233
+0.25481 0.0672767 0.0179763
+0.222129 -0.0581209 -0.00731524
+0.227145 -0.0819987 0.0145053
+0.219253 -0.118982 0.0109807
+0.211654 -0.158762 -0.0192499
+0.210611 -0.101269 0.0490966
+0.200451 -0.190104 0.0116714
+0.203938 -0.0712158 -0.0282028
+0.173774 -0.0735362 -0.0475631
+0.2027 -0.10102 -0.0400541
+0.212795 -0.129107 -0.0298945
+0.206091 -0.151217 -0.0455168
+0.202467 -0.150775 -0.0745887
+0.209378 -0.187531 -0.0743719
+0.181149 -0.126003 -0.0892617
+0.205972 -0.177843 -0.0383337
+0.188301 -0.142819 -0.114464
+0.208779 -0.164491 -0.140437
+0.171227 -0.151151 -0.147387
+0.136617 -0.138704 -0.143839
+0.131975 -0.166643 -0.169617
+0.128263 -0.113939 -0.11894
+0.116875 -0.0858965 -0.091059
+0.0786764 -0.0829677 -0.090746
+0.187859 -0.214057 -0.0686071
+0.175679 -0.237132 -0.0957975
+0.178163 -0.10588 -0.0665832
+0.269129 0.0763722 0.0451229
+0.27889 0.058723 0.0679827
+0.267652 0.0453551 0.105512
+0.2613 0.0363273 0.0775663
+0.250278 0.0199214 0.0566543
+0.250269 0.00638094 0.0769677
+0.262967 -0.0160504 0.0796628
+0.285344 -0.0433845 0.0872574
+0.24971 -0.0386027 0.0640826
+0.229012 -0.0528564 0.0533874
+0.210073 -0.0746435 0.0657137
+0.221889 -0.0775407 0.0410909
+0.302572 -0.0710887 0.0949218
+0.236871 -0.0290995 0.0413907
+0.21774 0.0486965 -0.0445342
+0.224846 0.0339602 -0.0740774
+0.244042 0.0044793 -0.10874
+0.254336 0.102378 0.0100739
+0.220262 0.108437 -0.00385435
+0.184431 0.103867 -0.0063665
+0.227766 0.138272 0.00117268
+0.21603 0.168687 0.00560537
+0.214962 0.252952 4.36931e-005
+0.24674 0.211475 0.00674743
+0.208103 0.206716 0.00898043
+0.198252 0.239152 0.0425261
+0.241235 0.183275 -0.00372162
+0.24846 0.187657 0.0253371
+0.236173 0.217937 0.0406376
+0.208251 0.202717 0.0430183
+0.190765 0.204953 0.0676283
+0.199822 0.228771 0.0907818
+0.212055 0.262964 0.121847
+0.167888 0.20936 0.0941323
+0.230768 0.216285 0.106419
+0.226441 0.220307 0.1523
+0.160904 0.217518 0.126833
+0.151535 0.251202 0.133586
+0.158786 0.306935 0.0984538
+0.193903 0.396086 0.195106
+0.119767 0.364922 0.154966
+0.158434 0.284691 0.125614
+0.188265 0.327264 0.175224
+0.139712 0.323472 0.144742
+0.247398 0.226772 0.208026
+0.162628 0.357991 0.158091
+0.132463 0.334186 0.215838
+0.184892 0.418747 0.128073
+0.148158 0.405567 0.1589
+0.116952 0.392105 0.208798
+0.128904 0.307582 0.18276
+0.173163 0.280669 0.197839
+0.237808 0.190239 0.0542196
+0.242415 0.187695 0.0885314
+0.257589 0.159811 0.105872
+0.25799 0.149475 0.1552
+0.249199 0.167605 0.0439231
+0.270589 0.141138 0.0478035
+0.291007 0.120776 0.0668204
+0.27986 0.0943457 0.0648346
+0.262028 0.128688 0.0220064
+0.283888 0.104244 0.08796
+0.278117 0.09277 0.114653
+0.296211 0.119268 0.134225
+0.25745 0.0686993 0.129638
+0.231499 0.0874258 0.144645
+0.23624 0.121986 0.150481
+0.21822 0.158716 0.152133
+0.276736 0.0681718 0.110294
+0.286087 0.0545245 0.0921962
+0.114064 0.342002 0.184102
+0.184346 0.361594 0.187357
+0.161147 0.378013 0.229421
+0.186402 0.32651 0.232026
+0.224199 0.278314 0.231623
+0.178684 0.387524 0.165454
+0.207571 0.416517 0.162223
+0.115261 0.360345 0.210089
+0.111327 0.375408 0.183082
+0.123148 0.405595 0.17979
+0.161317 0.441861 0.200022
+0.134075 0.42215 0.208124
+0.166581 0.413272 0.221996
+0.184888 0.467555 0.155941
+0.139501 0.435526 0.173911
+0.158717 0.435906 0.144337
+0.167498 0.45215 0.103084
+0.137763 0.359978 0.22921
+0.165611 0.348074 0.227085
+0.15984 0.318857 0.217125
+0.185554 0.304169 0.211919
+0.205103 0.278784 0.204981
+0.244841 0.279219 0.196675
+0.226891 0.25345 0.209087
+0.216714 0.31142 0.225032
+0.18164 0.33693 0.20427
+0.203702 0.303717 0.18726
+0.193627 0.296232 0.146589
+0.191637 0.257284 0.18429
+0.157803 0.257523 0.177293
+0.175634 0.311565 0.125155
+0.161107 0.457828 0.173386
+0.194169 0.447882 0.185268
+0.194408 0.477711 0.118315
+0.219599 0.449898 0.13857
+0.135294 0.387364 0.228733
+0.145147 0.282789 0.187345
+0.143956 0.303159 0.203618
+0.177446 0.366384 0.212315
+0.179329 0.391491 0.217312
+0.18672 0.412483 0.210145
+0.202606 0.421792 0.189409
+0.174085 0.42914 0.210627
+0.153641 0.426813 0.21312
+0.144424 0.408706 0.223131
+0.190464 0.428693 0.201747
+0.178837 0.441581 0.19921
+0.169704 0.452082 0.187978
+0.180705 0.459908 0.173619
+0.202088 0.458198 0.166952
+0.151672 0.449754 0.186725
+0.142475 0.436806 0.198199
+0.127991 0.425256 0.188222
+0.210178 0.437176 0.172385
+0.119636 0.410278 0.199562
+0.206557 0.46857 0.145869
+0.215386 0.468035 0.123827
+0.207969 0.436894 0.106764
+0.212477 0.477963 0.10116
+0.128023 0.4054 0.215288
+0.223073 0.454408 0.0921651
+0.184017 0.321535 0.149004
+0.164058 0.339436 0.133054
+0.141763 0.356765 0.138374
+0.138767 0.331395 0.111494
+0.18387 0.5 0.0894472
+0.178253 0.341112 0.15974
+0.158692 0.397227 0.229447
+0.261987 0.258572 0.22454
+0.209557 0.191204 0.156985
+0.19703 0.220129 0.168363
+0.114158 0.395299 0.191384
+0.237163 0.239354 0.0176536
+0.22873 0.228755 -0.0079498
+0.204237 0.229854 -0.000985107
+0.191898 0.248869 0.0164149
+0.195144 0.299533 0.0501168
+0.211408 0.277777 0.0261684
+0.181525 0.269092 0.042102
+0.232338 -0.0296485 0.018274
+0.220704 0.448894 0.117889
+0.211836 0.430361 0.128709
+0.200007 0.22745 0.0221073
+0.206622 0.221816 0.0395427
+0.223655 0.239447 0.0484776
+0.206846 0.263872 0.061817
+0.192412 0.289403 0.0880222
+0.200823 -0.0669613 0.089238
+0.1998 0.487191 0.0879627
+0.178418 0.478766 0.0726805
+0.171907 0.478 0.098484
+0.179607 0.4432 0.070784
+0.215421 0.44036 0.0572748
+0.206111 0.438517 0.081171
+0.184196 0.432818 0.0953164
+0.172787 0.433405 0.115219
+0.16681 0.454422 0.128591
+0.158037 0.463512 0.080819
+0.300027 -0.106441 0.0798503
+0.301565 -0.138608 0.110073
+0.223785 -0.0676095 0.104371
+0.244386 -0.0720528 0.0937201
+0.25624 -0.0594904 0.0758547
+0.271472 -0.0437947 0.0690266
+0.283677 -0.0568988 0.0744319
+0.294284 -0.0698767 0.0795338
+0.293906 -0.0887216 0.0736412
+0.275743 -0.101071 0.0877101
+0.309738 -0.0894098 0.0874741
+0.287014 -0.123527 0.0945964
+0.311125 -0.118131 0.0991123
+0.298899 -0.117613 0.118193
+0.276193 -0.109289 0.135451
+0.270493 -0.137621 0.153632
+0.286312 -0.136798 0.132025
+0.257826 -0.0797919 0.144654
+0.303792 -0.0864306 0.104131
+0.28817 -0.0747287 0.113826
+0.276632 -0.0878111 0.128274
+0.30695 -0.103289 0.107245
+0.287682 -0.0559229 0.10414
+0.272738 -0.0407993 0.108633
+0.253435 -0.0365139 0.124117
+0.295415 -0.0573592 0.0898843
+0.270105 -0.0629299 0.0693418
+0.263375 -0.0783235 0.0820667
+-0.216962 -0.20051 0.140681
+-0.236047 -0.180496 0.115013
+-0.230292 -0.136771 0.112494
+-0.247625 -0.159292 0.0914591
+-0.209896 -0.129892 0.141237
+0.222682 0.288177 0.190351
+0.233778 0.296853 0.212089
+0.209365 0.287061 0.167624
+0.208618 0.271128 0.146657
+0.223776 0.247151 0.14266
+0.250594 0.280585 0.221005
+0.215525 0.238928 0.164607
+0.248724 0.243405 0.176599
+0.282927 0.23674 0.194658
+0.253819 0.208937 0.178714
+0.265424 0.184312 0.155732
+0.308614 0.169998 0.183237
+0.273365 0.179729 0.192265
+0.265912 0.204786 0.204674
+0.300509 0.203464 0.194384
+0.281969 0.151677 0.177526
+0.279246 0.127373 0.158699
+0.31079 0.143896 0.162421
+0.30954 0.171009 0.155179
+0.288086 0.1804 0.137744
+0.264265 0.175268 0.132545
+0.241184 0.168571 0.143536
+0.282052 0.162312 0.123541
+0.290218 0.138764 0.11928
+0.232003 0.191432 0.146429
+0.237199 0.211128 0.131059
+0.175486 0.13591 0.157401
+0.244193 0.0453066 0.121153
+0.216838 0.0295154 0.115567
+0.0778181 0.0182774 -0.0959304
+0.132697 0.385267 0.165833
+0.155812 0.38306 0.160495
+-0.00373338 0.0386319 -0.0871428
+0.0052284 0.0508015 -0.0521262
+-0.0272532 0.0521944 -0.0650671
+-0.0417118 0.0760468 -0.0274796
+0.0432101 0.0478592 -0.0430105
+0.0360437 0.064037 -0.0095129
+0.0264403 0.0878588 0.0105855
+0.0200841 0.0963175 -0.0204482
+0.0508265 0.0939603 -0.0091335
+0.0753367 0.087282 -0.0290458
+-0.0114666 0.0989277 -0.0268583
+0.189464 0.426182 0.111762
+-0.04038 -0.0265907 0.0536548
+0.188037 0.19051 0.048384
+0.170897 0.170857 0.0404072
+0.180803 0.154042 0.0195245
+0.168583 0.128396 0.0100026
+0.150344 0.161847 0.0580756
+0.146195 0.173828 0.0846654
+0.123104 0.163389 0.100752
+0.131952 0.158423 0.126057
+0.154039 0.169296 0.137953
+0.163282 0.191526 0.127822
+0.170691 0.206066 0.147249
+0.123979 0.136658 0.135
+0.136161 0.125537 0.148878
+0.153818 0.131557 0.161379
+0.111897 0.12133 0.126074
+0.111889 0.144276 0.0890707
+0.11658 0.140768 0.0690434
+0.119959 0.124948 0.0613596
+0.107779 0.107117 0.0626571
+0.122618 0.115267 0.0466942
+0.127454 0.104238 0.0219653
+0.136258 0.119892 0.0320023
+0.129073 0.0915077 -0.00265103
+0.130797 0.0780035 -0.0369633
+0.10768 0.094992 0.00979378
+0.163926 0.154671 0.152149
+0.0894836 0.0909923 0.00058556
+0.0689505 0.0963924 0.00171312
+0.0612997 0.100634 0.0224348
+0.0675451 0.0846698 0.038694
+0.0795109 0.103357 0.0384133
+0.0848094 0.0754581 0.0444653
+0.110567 0.10366 0.130086
+0.12281 0.0864143 0.139975
+0.117855 0.062854 0.143513
+0.13666 0.0472165 0.155281
+0.128164 0.0235742 0.176647
+0.163067 0.0498951 0.143567
+0.143932 0.0949004 0.145284
+0.179917 0.317727 0.0742859
+0.183725 0.275085 0.0676723
+0.16838 0.29297 0.0787056
+0.0930951 0.102542 0.05002
+0.100339 0.0681106 0.0373411
+0.102886 0.0622715 0.0197467
+0.121511 0.0540863 0.0117598
+0.124719 0.0242285 0.0166428
+0.0967861 -0.00310471 -0.0020113
+0.12138 0.0519179 -0.0102922
+0.0990646 0.0492208 -0.022422
+0.0873807 -0.0275369 -0.03209
+0.200694 -0.191636 -0.0546067
+0.206298 -0.170055 -0.0598788
+0.209964 -0.168421 -0.0791806
+0.221182 -0.183261 -0.0963771
+0.222775 -0.172837 -0.120159
+0.235715 -0.195921 -0.115182
+0.253933 -0.218526 -0.134037
+0.311213 -0.253911 -0.191866
+0.279294 -0.244732 -0.16099
+0.266185 -0.201338 -0.169529
+0.285572 -0.216415 -0.213382
+0.273765 -0.285731 -0.187819
+0.259679 -0.265381 -0.248632
+0.24894 -0.227823 -0.231771
+0.232153 -0.25966 -0.225227
+0.254118 -0.290735 -0.220386
+0.336364 -0.328047 -0.241676
+0.281317 -0.319577 -0.26697
+0.295033 -0.317038 -0.218433
+0.327766 -0.263669 -0.27537
+0.320681 -0.238904 -0.235706
+0.333487 -0.28367 -0.222752
+0.25789 -0.299076 -0.25318
+0.280382 -0.278404 -0.287734
+0.262726 -0.334272 -0.234674
+0.315714 -0.303377 -0.28762
+0.358898 -0.298323 -0.270079
+0.292961 -0.250812 -0.263064
+0.260427 0.269097 0.206442
+0.273912 0.251948 0.207483
+0.274266 0.226866 0.218876
+0.254508 0.262332 0.186312
+0.268737 0.247011 0.182676
+0.278883 0.230014 0.174886
+0.292676 0.216891 0.181537
+0.301666 0.196568 0.170643
+0.235991 0.25839 0.179999
+0.216996 0.262302 0.191107
+0.233602 0.240223 0.192553
+0.26623 0.217767 0.166603
+0.291208 0.219735 0.206357
+0.285626 0.200003 0.208179
+0.295054 0.181857 0.198439
+-0.119559 -0.0454446 0.130205
+-0.148541 -0.0288065 0.124554
+-0.122421 -0.0280036 0.104512
+-0.169628 -0.0428483 0.136658
+-0.192691 -0.0700149 0.138018
+-0.165949 -0.0805689 0.151606
+-0.157867 -0.111652 0.162619
+-0.182289 -0.134815 0.160033
+-0.171616 -0.0265274 0.119564
+-0.182821 -0.0294707 0.089096
+-0.207158 -0.0941133 0.130893
+-0.0813687 -0.408143 0.0168626
+-0.0493082 -0.255289 0.183439
+-0.0417823 -0.281988 0.16825
+-0.0232624 -0.242141 -0.150317
+0.237084 0.148575 0.150278
+0.21825 0.135883 0.152701
+0.196547 0.147262 0.152063
+0.248839 0.134889 0.149793
+0.25417 0.116897 0.146677
+0.154738 -0.248351 -0.113516
+0.149894 -0.252291 -0.14374
+0.127807 -0.247316 -0.112579
+0.100037 -0.239188 -0.118127
+0.171952 -0.258325 -0.155783
+0.206243 -0.267544 -0.164319
+0.152779 -0.244265 -0.170212
+0.238869 -0.271194 -0.164355
+0.19948 -0.240785 -0.114164
+0.228533 -0.228656 -0.117975
+0.0806348 -0.448964 -0.0364622
+0.092817 -0.46403 -0.0236687
+0.131465 -0.464547 -0.0186627
+0.111576 -0.45856 -0.0162136
+0.12236 -0.437795 -0.0167687
+0.116113 -0.473014 -0.0333379
+0.141834 -0.466374 -0.0462667
+0.15629 -0.45187 -0.0265272
+0.162053 -0.430562 -0.0436087
+0.170805 -0.433786 -0.074571
+0.150694 -0.440322 -0.089161
+0.142403 -0.453672 -0.0687084
+0.20922 0.0225383 -0.118012
+0.245897 0.00269769 -0.0710137
+-0.0868896 -0.445579 -0.0827631
+-0.0899978 -0.418668 -0.0662628
+-0.0919895 -0.382051 -0.0731611
+-0.286076 -0.18977 0.00251657
+0.166397 -0.235956 -0.0665238
+0.18289 -0.231659 -0.0402536
+0.183601 -0.256036 -0.0114407
+0.19304 -0.222737 -0.0114233
+0.168396 -0.264129 0.0198747
+0.175145 -0.292863 -0.0261367
+0.159612 -0.311932 -0.0502102
+0.151795 -0.349231 -0.058414
+0.168467 0.120276 0.165442
+0.179322 0.109989 0.151163
+0.191745 0.091503 0.1476
+0.207409 0.0731218 0.143583
+0.170472 0.088013 0.148441
+0.198308 0.0526608 0.137187
+-0.288444 -0.322548 -0.196751
+-0.258254 -0.336596 -0.201797
+-0.260706 -0.370334 -0.191889
+-0.262012 -0.38355 -0.234182
+0.169409 0.331718 0.106522
+-0.0883279 -0.427369 -0.00320489
+-0.0757242 -0.410706 -0.00350047
+-0.0694098 -0.396348 -0.0215868
+-0.339105 -0.28249 -0.133907
+0.14338 -0.190029 -0.185968
+0.113197 -0.189729 -0.178573
+0.161752 -0.208101 -0.1989
+0.163143 -0.233988 -0.192556
+0.187542 -0.24244 -0.20457
+0.214342 -0.231174 -0.221761
+0.200695 -0.263551 -0.191549
+0.0888974 -0.174918 -0.165344
+0.0728578 -0.155488 -0.148655
+0.0857975 -0.13271 -0.133069
+0.0496654 -0.153477 -0.133288
+0.208417 -0.252602 -0.211864
+0.214499 -0.20684 -0.209109
+0.212326 -0.182246 -0.176825
+0.196622 -0.193456 -0.194925
+-0.0366034 0.0848157 -0.0747335
+-0.0106036 0.0767347 -0.0825468
+-0.248014 -0.143811 -0.0711582
+0.156176 -0.353723 -0.00967102
+0.161881 -0.35946 -0.0354879
+0.154192 -0.374021 -0.054565
+0.153835 -0.401954 -0.0551512
+0.147106 -0.376782 -0.0111704
+0.141013 -0.401853 -0.0175381
+0.127378 -0.38782 -0.00428773
+0.152558 -0.410563 -0.0345712
+0.144573 -0.387551 -0.0699167
+0.129797 -0.395951 -0.0860393
+0.110844 -0.383365 -0.0877166
+0.111358 -0.362136 -0.0828181
+0.10863 -0.332992 -0.0757964
+0.131091 -0.348484 -0.0736181
+0.114528 -0.372564 0.00601769
+0.116893 -0.350867 0.0177725
+0.143657 -0.369483 -0.0686154
+0.0433039 -0.239647 0.0998892
+-0.318832 -0.357055 -0.211401
+-0.299837 -0.377374 -0.238049
+-0.340344 -0.383626 -0.224893
+-0.356366 -0.419986 -0.188103
+-0.285529 -0.404192 -0.228972
+-0.356375 -0.393121 -0.201407
+-0.349321 -0.392013 -0.165399
+-0.328811 -0.42272 -0.163607
+-0.345548 -0.417553 -0.216974
+-0.322795 -0.427865 -0.195551
+-0.33518 -0.363207 -0.161311
+-0.0812327 -0.396788 0.0342935
+-0.065289 -0.38943 0.0224745
+-0.0508718 -0.371639 0.0298172
+-0.260668 -0.216401 0.0653687
+-0.2704 -0.185201 0.0538295
+0.187032 0.486356 0.0996338
+0.279593 -0.136382 0.110973
+0.26837 -0.117918 0.105466
+0.248285 -0.109463 0.116456
+0.232397 -0.125931 0.141691
+0.210603 -0.141776 0.171116
+0.137574 -0.108705 0.207737
+0.169921 0.29167 0.0522744
+0.00992085 -0.113945 -0.0964453
+0.258261 -0.257702 -0.154872
+0.275321 -0.267702 -0.17038
+0.295633 -0.272896 -0.184932
+0.295815 -0.294739 -0.20199
+0.314713 -0.27743 -0.201437
+0.327332 -0.256197 -0.214678
+0.340385 -0.261017 -0.241446
+0.313356 -0.232789 -0.209684
+0.296344 -0.226073 -0.182212
+0.31592 -0.301772 -0.216769
+0.318635 -0.326842 -0.222796
+0.307178 -0.325937 -0.251416
+0.274745 -0.303144 -0.21295
+0.263287 -0.308346 -0.232527
+0.255382 -0.319889 -0.248754
+0.332002 -0.310475 -0.229091
+0.353636 -0.307458 -0.245121
+0.335859 -0.313851 -0.265914
+0.340499 -0.300896 -0.286697
+0.344989 -0.280039 -0.276203
+0.327449 -0.28026 -0.295194
+0.304942 -0.268737 -0.28762
+0.34983 -0.283684 -0.251927
+0.343136 -0.265469 -0.261534
+0.326713 -0.248963 -0.256716
+0.307917 -0.24034 -0.252462
+0.279159 -0.231625 -0.241522
+0.300353 -0.229376 -0.234338
+0.311358 -0.251222 -0.26759
+0.299024 -0.289572 -0.301481
+0.279907 -0.305571 -0.290358
+0.263002 -0.29286 -0.276914
+0.260616 -0.313748 -0.268771
+0.275922 -0.322371 -0.224156
+0.284975 -0.33259 -0.244221
+0.303023 -0.336946 -0.233483
+0.289605 -0.333838 -0.221905
+0.166435 0.39886 0.154509
+0.189152 0.404825 0.153461
+0.197133 0.400723 0.1737
+0.17424 0.413608 0.143911
+0.169142 0.426834 0.132439
+0.188725 0.386035 0.180698
+0.186219 0.378081 0.198788
+-0.267025 -0.372115 -0.138391
+-0.248022 -0.367522 -0.158602
+0.174505 0.292586 0.0990952
+0.186251 0.309263 0.0928235
+0.181697 0.30513 0.109614
+0.189499 0.279776 0.119317
+0.172647 0.294457 0.120787
+0.158057 0.303956 0.130265
+0.143476 0.291331 0.139139
+0.131312 0.30031 0.159002
+0.186095 0.298863 0.131041
+0.198975 0.28677 0.132576
+0.172236 0.277192 0.117866
+0.184313 0.260452 0.109718
+0.15755 0.265746 0.122578
+0.145983 0.269471 0.137935
+0.151234 0.256355 0.155644
+0.124293 0.323596 0.164277
+0.126678 0.342635 0.150186
+0.129524 0.341544 0.128525
+0.146576 0.349856 0.118779
+0.192269 0.30374 0.0759625
+0.202728 0.285813 0.0690781
+0.210479 0.281567 0.0483558
+0.222196 0.26209 0.0407369
+0.224073 0.261141 0.0186008
+0.162756 0.306191 0.114529
+0.149682 0.318744 0.121136
+0.152677 0.341974 0.100993
+0.164071 0.331208 0.0841361
+0.146523 0.323278 0.093798
+0.15695 0.312461 0.0714443
+0.22266 -0.203518 -0.100877
+0.262076 -0.292278 -0.202356
+0.249754 -0.28212 -0.183581
+0.242451 -0.284733 -0.203885
+0.229725 -0.273053 -0.211906
+0.243386 -0.275428 -0.225419
+0.255999 -0.283472 -0.239546
+-0.261873 -0.405224 -0.222848
+-0.280772 -0.420055 -0.201182
+-0.274389 -0.414773 -0.173401
+-0.298626 -0.411672 -0.158377
+-0.28738 -0.389898 -0.148508
+-0.300008 -0.369107 -0.135836
+-0.257381 -0.39915 -0.185758
+-0.257531 -0.421709 -0.186727
+-0.319983 -0.369357 -0.146016
+-0.25404 -0.39271 -0.206532
+-0.269186 -0.375948 -0.213104
+0.171129 0.312683 0.0555484
+0.186538 0.309469 0.0612947
+0.17697 0.322584 0.0897939
+0.180094 0.317456 0.102692
+0.0389455 -0.294719 0.0707663
+0.19085 0.129482 0.148059
+0.26893 -0.330546 -0.250405
+0.261495 -0.324416 -0.260179
+0.163955 0.0845671 -0.00775852
+0.172992 0.467003 0.114773
+0.17962 0.47069 0.135115
+0.167392 0.460661 0.148013
+0.0927702 -0.0102964 0.178794
+0.0791092 -0.00358862 0.211868
+0.0484002 -0.0727004 0.143042
+0.0857054 -0.0664246 0.132462
+0.170606 0.462905 0.162683
+0.107346 -0.291576 0.0507084
+0.123054 -0.26548 0.0555752
+0.1033 -0.273351 0.0618915
+-0.217347 -0.0684085 0.0793768
+-0.232534 -0.1003 0.0785864
+0.160705 0.203815 0.112095
+0.157448 0.189802 0.0951937
+0.163855 0.222961 0.107992
+0.178039 0.226994 0.0939715
+0.157779 0.236558 0.121407
+0.156862 0.233096 0.141593
+0.254115 0.147478 0.0328869
+0.246739 0.139758 0.0163119
+-0.313249 -0.26088 -0.138114
+-0.319034 -0.272336 -0.0954566
+-0.312031 -0.286413 -0.151154
+-0.318615 -0.301412 -0.127758
+-0.31358 -0.30952 -0.0911544
+-0.342054 -0.297563 -0.104135
+-0.285707 -0.289103 -0.098473
+-0.332554 -0.290038 -0.0650158
+0.224391 0.444149 0.0748945
+0.224127 0.466497 0.0733316
+0.00933378 -0.0890982 -0.073455
+-0.196836 -0.0544369 -0.0547609
+-0.268852 -0.35939 -0.204575
+-0.134821 -0.144762 0.184037
+-0.134018 -0.119846 0.172991
+-0.108849 -0.110436 0.169417
+-0.142893 -0.258813 -0.176858
+0.163435 0.422628 0.144619
+0.149105 0.425301 0.157986
+0.151653 0.445126 0.160854
+0.205634 0.453218 0.0682861
+0.196116 0.437078 0.0613117
+0.305429 0.136514 0.131635
+0.304223 0.128705 0.150462
+0.294739 0.1352 0.16485
+0.29983 0.148475 0.176204
+0.293633 0.16134 0.186506
+0.312816 0.146868 0.144611
+0.290574 0.119539 0.149687
+0.280449 0.10933 0.137413
+0.285959 0.112117 0.117297
+0.154398 0.116961 0.162863
+0.147042 0.108372 0.152274
+0.179394 0.214392 0.162515
+0.185651 0.194518 0.157404
+0.180628 0.232354 0.173507
+0.198969 0.238329 0.174545
+0.207694 0.253512 0.177641
+0.203869 0.264161 0.186147
+0.189501 0.273156 0.193533
+0.17312 0.263127 0.188581
+-0.206119 -0.0619918 0.116346
+-0.201545 -0.0465532 0.095691
+0.256266 0.152102 0.0503785
+0.264034 0.143148 0.0672501
+0.26342 0.152022 0.0867622
+0.269601 0.144159 0.103885
+0.281713 0.137095 0.0629023
+0.296165 0.127981 0.0376256
+0.300117 0.13462 0.0559043
+0.287642 0.140563 0.0456708
+0.278661 0.132147 0.0309413
+0.271646 0.118273 0.028754
+0.262921 0.1025 0.0261376
+0.261118 0.0855637 0.0169037
+0.25422 0.0767023 0.000233527
+0.237362 0.0575218 0.00198963
+0.283388 0.119704 0.0383643
+0.282941 0.112062 0.0540402
+0.249937 0.0937398 -0.00575339
+0.231931 0.0954578 -0.0107377
+0.210437 0.0914748 -0.0140972
+0.239252 0.109782 0.000233887
+0.191402 0.0870306 -0.0134669
+0.184376 0.0704441 -0.0225342
+0.184719 0.0606504 -0.044543
+0.166145 0.0578967 -0.0668264
+0.200251 0.061117 -0.0313226
+0.217304 0.057967 -0.0199728
+0.227686 0.12158 0.00370932
+0.20937 0.123042 0.000300618
+0.244333 0.124524 0.00810813
+0.295429 0.120597 0.0522453
+0.178132 0.0791963 -0.0112798
+0.177197 -0.279627 -0.000252101
+0.173639 -0.299787 -0.00606886
+0.172196 -0.313975 -0.0223573
+0.166117 -0.326159 -0.00858114
+0.168079 -0.321719 -0.0379096
+0.162521 -0.339731 -0.0445028
+0.151373 -0.32895 -0.0576036
+0.312369 -0.105021 0.0899766
+0.306 -0.11652 0.0866674
+0.301418 -0.129182 0.0942967
+0.290875 -0.135236 0.101483
+0.289966 -0.143698 0.1109
+0.295915 -0.13953 0.122573
+0.279108 -0.148197 0.122497
+0.27841 -0.152315 0.142258
+0.265867 -0.157983 0.158785
+0.256687 -0.14661 0.137703
+0.25422 -0.140442 0.172759
+0.232237 -0.161116 0.192798
+0.192807 -0.160413 0.202508
+0.178601 -0.140578 0.237092
+0.154384 -0.117167 0.24831
+0.13004 -0.134226 0.269277
+0.17845 -0.105479 0.223698
+0.247464 -0.158843 0.179106
+0.226891 -0.158442 0.171794
+0.209982 -0.159856 0.187271
+0.217821 -0.141328 0.207646
+0.23745 -0.141109 0.186416
+0.229503 -0.142546 0.156329
+0.214657 -0.125289 0.155947
+0.193847 -0.12199 0.171057
+0.217246 -0.105649 0.140104
+0.198069 -0.0856194 0.142621
+0.193693 -0.14343 0.18682
+0.198045 -0.145975 0.222277
+0.178989 -0.164931 0.225321
+0.158831 -0.146831 0.218292
+0.158889 -0.159611 0.244735
+0.138513 -0.154491 0.257842
+0.156487 -0.13924 0.256434
+-0.290841 -0.208347 -0.00770324
+-0.29533 -0.224009 -0.017137
+-0.30561 -0.24462 -0.0155011
+-0.278365 -0.220452 -0.0277869
+-0.267819 -0.241114 -0.0357093
+-0.286737 -0.259306 -0.0462551
+-0.310093 -0.264404 -0.0203987
+-0.307167 -0.250649 -0.0341839
+-0.307612 -0.281242 -0.0310235
+-0.321036 -0.286281 -0.0471544
+-0.310491 -0.270102 -0.0639157
+-0.311345 -0.301149 -0.0633156
+-0.297683 -0.291436 -0.0450671
+-0.294949 -0.304108 -0.0790095
+-0.285459 -0.27779 -0.0488974
+-0.296472 -0.275126 -0.0168658
+-0.276222 -0.270671 -0.00788631
+-0.246454 -0.248267 -0.0329014
+-0.244456 -0.252303 -0.00937036
+-0.304286 -0.259682 -0.0492577
+-0.294933 -0.27203 -0.0604957
+-0.300998 -0.278796 -0.0743602
+-0.316674 -0.274623 -0.0789273
+-0.337714 -0.280945 -0.0853892
+-0.325825 -0.275251 -0.0655092
+-0.332977 -0.302425 -0.0833069
+-0.293564 -0.290291 -0.0772666
+-0.298024 -0.281309 -0.0905074
+-0.293634 -0.272694 -0.109468
+-0.270428 -0.274072 -0.107657
+-0.256077 -0.253991 -0.120812
+-0.261078 -0.296954 -0.0999473
+-0.2381 -0.297373 -0.0849839
+-0.232752 -0.278906 -0.0645746
+-0.272041 -0.319246 -0.10842
+-0.21032 -0.284441 -0.0448701
+-0.256436 -0.341131 -0.11181
+-0.225098 -0.336223 -0.113241
+-0.200175 -0.288996 -0.0158483
+-0.223009 -0.317757 -0.0905879
+-0.301096 -0.262202 -0.123506
+-0.32405 -0.263071 -0.116908
+-0.292477 -0.257052 -0.14263
+-0.270149 -0.255918 -0.142108
+-0.275486 -0.258003 -0.162669
+-0.259523 -0.2674 -0.178244
+-0.251069 -0.250018 -0.0970302
+-0.24056 -0.260706 -0.172311
+-0.232499 -0.247239 -0.154361
+-0.34182 -0.274562 -0.107339
+-0.294559 -0.307923 -0.0994756
+-0.275844 -0.269552 -0.0355918
+-0.227344 -0.270897 -0.045083
+0.25093 0.170443 0.120028
+0.248384 0.184009 0.106523
+0.235777 0.198062 0.103222
+0.23641 0.211305 0.086191
+0.221152 0.226047 0.0898201
+0.223243 0.24123 0.111495
+0.203883 0.250541 0.105061
+0.226958 0.225862 0.123117
+0.219737 0.240254 0.12789
+0.214138 0.253162 0.132941
+0.232275 0.224561 0.138088
+0.109391 -0.247931 -0.0995264
+0.104952 -0.262437 -0.0820427
+0.080763 -0.274056 -0.0783487
+0.0604155 -0.262392 -0.0950373
+-0.227936 -0.329497 -0.190147
+-0.214125 -0.337031 -0.170051
+0.263325 -0.0528405 0.0685009
+0.251926 -0.0484757 0.0715634
+0.237277 -0.0480579 0.0658708
+0.244491 -0.055646 0.0790843
+0.239659 -0.0616442 0.0888399
+0.226162 -0.0616059 0.0920824
+0.21522 -0.0631162 0.0781895
+0.212545 -0.0629682 0.0966232
+0.196781 -0.0599901 0.105959
+0.210127 -0.0657511 0.11188
+0.250495 -0.0660959 0.0846797
+0.234672 -0.0671026 0.0978575
+0.237261 -0.0846396 0.108485
+0.25989 -0.0423809 0.0666642
+0.259987 -0.0257531 0.0664908
+0.247883 -0.0247631 0.0554477
+0.242689 -0.0135836 0.0440572
+0.246291 0.00496067 0.0457059
+0.245187 0.0234669 0.0372413
+0.250061 0.17416 0.133813
+0.259114 0.167602 0.148828
+0.26511 0.166925 0.171546
+0.238618 0.181672 0.138204
+0.229669 0.176644 0.149535
+0.262049 0.187134 0.176237
+0.261795 0.200868 0.165371
+0.276513 0.199318 0.153854
+0.292677 0.199982 0.157209
+0.302715 0.186361 0.157188
+0.308755 0.181779 0.171774
+0.305926 0.190608 0.185482
+0.312423 0.167782 0.169221
+0.31381 0.157653 0.158921
+0.310783 0.154703 0.171639
+0.301108 0.177938 0.144841
+0.293264 0.189309 0.147256
+0.279322 0.188458 0.145304
+0.262483 0.193127 0.192418
+0.274289 0.192768 0.20161
+0.254841 0.206509 0.193978
+0.245154 0.222549 0.189318
+0.253012 0.22552 0.174345
+0.265429 0.23121 0.17081
+-0.253206 -0.236908 -0.0443888
+0.149428 0.386275 0.232451
+0.147867 0.371017 0.232576
+0.152926 0.356705 0.229375
+0.149032 0.338169 0.225131
+0.166114 0.363727 0.223417
+0.172256 0.353426 0.215568
+0.179541 0.338962 0.221969
+0.181625 0.354174 0.201921
+0.186426 0.343494 0.187951
+0.192075 0.328517 0.215563
+0.192453 0.320313 0.195502
+0.17815 0.351274 0.174302
+0.17655 0.370022 0.172318
+0.165276 0.33179 0.225541
+0.00939661 -0.323412 0.0804555
+0.117561 -0.144032 0.250595
+0.120353 -0.125008 0.233582
+0.111251 -0.145731 0.271784
+0.11093 -0.120815 0.285658
+0.0754192 -0.0921393 0.301481
+0.0769558 -0.124739 0.290267
+0.0748528 -0.117401 0.265004
+0.0426218 -0.103655 0.282975
+0.0504123 -0.0659143 0.287583
+0.0836935 -0.0669285 0.279976
+0.0588543 -0.11803 0.279938
+0.0586609 -0.105277 0.297458
+0.0557269 -0.0839063 0.29826
+0.0381914 -0.083198 0.286361
+0.0371995 -0.0680895 0.266535
+0.047335 -0.0888232 0.265922
+0.05563 -0.0732275 0.247968
+0.0616189 -0.0543441 0.232231
+0.0811821 -0.0784737 0.231059
+0.0764784 -0.0353683 0.254905
+0.0689625 -0.0703705 0.29266
+0.0661415 -0.0548095 0.278244
+0.0608765 -0.028919 0.235586
+0.087271 -0.018263 0.234864
+0.0756435 -0.10953 0.298971
+0.0951645 -0.0988538 0.288259
+0.0930654 -0.116235 0.295574
+0.0947803 -0.133863 0.284699
+0.0966033 -0.0830131 0.273711
+0.113798 -0.076403 0.258582
+0.127442 -0.0600385 0.234949
+0.132794 -0.0953943 0.252522
+0.0489111 -0.0556973 0.272686
+0.0606107 -0.0431898 0.263976
+0.0971312 -0.036293 0.245012
+0.110687 -0.0488392 0.23675
+0.126176 -0.0414728 0.221378
+0.136057 -0.0243951 0.201716
+0.122078 0.00127519 0.196707
+0.142291 0.00027007 0.178375
+0.0664164 -0.0191578 0.217877
+0.0802696 -0.0234954 0.199319
+0.0910082 -0.0454502 0.20083
+0.0798776 -0.132726 0.273919
+0.102454 0.00166634 0.210465
+0.0802091 -0.00534582 0.192641
+0.0863635 0.00814252 0.179895
+0.0946306 0.0282399 0.168655
+0.0981336 0.029523 0.148764
+0.101777 0.0395859 0.135111
+0.102082 0.0581642 0.134881
+0.0956437 0.0321468 0.122829
+0.0938658 0.0161985 0.122282
+0.107901 0.00369122 0.128399
+0.078491 0.00576055 0.124425
+0.0841199 0.0382034 0.11313
+0.0659972 0.0331947 0.109714
+0.0520056 0.0196069 0.12098
+0.108345 -0.0142416 0.126349
+0.0915975 -0.0313276 0.129913
+0.122833 -0.0287943 0.124051
+0.146418 -0.0426624 0.119895
+0.142658 -0.0228961 0.131632
+0.132595 -0.0194368 0.150249
+0.160419 -0.0327892 0.133975
+0.163125 -0.050543 0.147173
+0.060233 0.0422739 0.0970401
+0.0469107 0.0353785 0.0851224
+0.0330985 0.0252247 0.0760721
+0.0344131 0.0145341 0.0525338
+0.0185411 0.0141735 0.0691085
+-0.00651953 0.00766464 0.081092
+0.0314926 0.0314677 0.0951056
+0.0155266 0.0258059 0.107793
+0.11957 0.0050674 0.144577
+0.11348 -0.010764 0.15919
+0.0864989 0.00930931 0.196192
+0.112679 0.0343949 0.172228
+0.110056 0.0172326 0.144505
+-0.231861 -0.256194 -0.0373807
+-0.233847 -0.249852 -0.0220477
+-0.230235 -0.258784 -0.00973726
+-0.217151 -0.280061 0.00814621
+-0.228026 -0.273051 0.0389012
+-0.222798 -0.271648 -0.00409819
+-0.229682 -0.268052 0.0118625
+-0.212876 -0.279385 -0.0140325
+-0.214439 -0.277583 -0.0303969
+-0.199886 -0.285356 -0.0320197
+-0.187179 -0.295224 -0.0255156
+-0.167219 -0.315028 -0.0304058
+-0.156845 -0.302235 -0.0544314
+-0.236758 -0.254924 0.00227314
+0.0884351 -0.404348 -0.0195924
+0.0722252 -0.268984 0.0670772
+0.070147 -0.245252 0.0862941
+0.0966471 -0.229612 0.0882203
+0.0926053 -0.203806 0.103764
+0.0777673 -0.186021 0.118999
+0.0417862 -0.182948 0.126604
+0.0709209 -0.158179 0.128709
+0.095465 -0.128791 0.124296
+0.0102764 -0.16375 0.133447
+-0.0233099 -0.171526 0.153541
+-0.0499934 -0.0465621 -0.00582837
+-0.0749973 -0.0355191 0.0101971
+-0.0645299 -0.324704 -0.077104
+0.221779 0.470802 0.0892895
+0.219761 0.464582 0.1032
+0.246597 0.0246927 0.0795416
+0.253586 0.0320272 0.0977802
+0.252394 0.0305168 0.0670227
+0.261064 0.0424817 0.0637394
+0.267205 0.0556508 0.058182
+0.262106 0.0615289 0.0436471
+0.250683 0.0500973 0.0298185
+0.272972 0.0454164 0.0731281
+0.274554 0.0385023 0.090004
+0.285852 0.0505507 0.0780368
+0.287544 0.066644 0.0791877
+0.280213 0.0764884 0.0661465
+0.226162 0.0537579 -0.00800313
+0.231726 0.0372941 0.00212281
+0.226833 0.070796 -0.0125112
+-0.0228881 0.0822405 -0.0141151
+-0.0164496 0.062439 -0.0353807
+-0.0311204 0.09353 -0.0291259
+-0.0468617 0.0890411 -0.0503417
+-0.0192694 0.0958145 -0.0445491
+-0.0451838 0.0631149 -0.0497516
+-0.0349277 0.0970377 -0.0439657
+-0.0281121 0.0908295 -0.0597278
+-0.0422149 0.0875756 -0.0360098
+-0.0493018 0.0748281 -0.0407497
+-0.0508974 0.0756362 -0.0555946
+-0.045787 0.0616401 -0.0714534
+-0.0463878 0.0860458 -0.0643425
+-0.0469 0.0754389 -0.0734442
+-0.0361876 0.0721397 -0.0877453
+-0.0344753 0.0506229 -0.0835042
+-0.0209274 0.0429352 -0.0951495
+-0.00835098 0.0548898 -0.109767
+-0.0439408 0.0675081 -0.0809412
+-0.0384794 0.0604865 -0.0888934
+-0.0325578 0.0529818 -0.0938199
+-0.0250287 0.0507859 -0.100857
+-0.0171358 0.0586173 -0.103485
+-0.00108771 0.0669554 -0.09638
+0.0141047 0.0764389 -0.0789872
+-0.0166767 0.0687863 -0.0931008
+-0.0154196 0.0443974 -0.106822
+0.00804033 0.0334573 -0.118139
+-0.00353356 0.0420783 -0.115203
+0.00267945 0.0333406 -0.10199
+0.0284162 0.0260832 -0.102463
+0.0140719 0.0342408 -0.0876383
+0.01206 0.042083 -0.0690778
+0.0262722 0.0834922 -0.057317
+0.052314 0.0257381 -0.0888721
+0.0550064 0.0153199 -0.121681
+0.0742668 0.0307726 -0.0684202
+0.0254542 0.0462258 -0.0537161
+0.0192751 0.0579365 -0.0299961
+0.043388 0.0389601 -0.0621497
+0.0632571 0.0407552 -0.0511604
+0.0628168 0.0511918 -0.0295138
+-0.00769957 0.0468719 -0.0674097
+0.0356439 0.036944 -0.129564
+0.00985644 0.0486694 -0.11871
+0.0293481 0.0500299 -0.116265
+0.034304 0.0639803 -0.0982281
+0.0203799 0.0394103 -0.126099
+0.0274107 0.0238815 -0.120304
+-0.00989932 0.038173 -0.0977698
+0.0409915 0.0204434 -0.130582
+0.0603476 0.0361414 -0.135064
+0.0745924 0.0111822 -0.144256
+0.0995383 0.0164619 -0.152082
+0.079494 0.0098491 -0.119596
+0.109969 -0.00178903 -0.14121
+0.128369 -0.00667529 -0.145234
+0.147176 -0.000628591 -0.159936
+0.143478 0.0207956 -0.143584
+0.15945 -0.0140125 -0.151421
+0.167046 -0.013357 -0.130058
+0.175024 -0.000463673 -0.161193
+0.200548 0.00730904 -0.144289
+0.228188 0.00606999 -0.13371
+0.232374 -0.0225206 -0.12062
+0.217352 -0.0205681 -0.149989
+0.196446 -0.0094088 -0.159916
+0.0468892 0.0318746 -0.136603
+0.0588847 0.0194871 -0.141629
+0.0497176 0.0425705 -0.12546
+0.067316 0.0466181 -0.121528
+0.0829023 0.0554408 -0.10632
+0.0702884 0.0671402 -0.0864572
+0.0736048 0.0280122 -0.141966
+-0.0393368 0.0927115 -0.058201
+-0.074253 -0.462729 -0.0674302
+-0.0882268 -0.46748 -0.0836924
+-0.10265 -0.454789 -0.102517
+-0.100302 -0.434985 -0.0953132
+-0.123508 -0.439573 -0.0964816
+0.30376 -0.329009 -0.220778
+-0.34576 -0.287787 -0.1184
+-0.333412 -0.299886 -0.118936
+-0.318278 -0.306436 -0.109371
+-0.342798 -0.273541 -0.122349
+-0.330946 -0.265681 -0.13448
+-0.32759 -0.278136 -0.147142
+-0.313986 -0.269795 -0.152301
+-0.304937 -0.277784 -0.160852
+-0.289134 -0.290053 -0.171285
+-0.300398 -0.29313 -0.159904
+-0.305136 -0.304081 -0.143109
+-0.303823 -0.326091 -0.148763
+-0.297334 -0.338307 -0.131821
+-0.0297344 -0.320795 0.126249
+-0.0440833 -0.337561 0.102751
+-0.0635963 -0.3222 0.124646
+-0.0820192 -0.303394 0.145977
+-0.0280108 -0.346215 0.0792887
+0.190023 -0.166658 -0.160984
+0.187606 0.489479 0.0788413
+0.169798 0.492699 0.0805537
+0.160239 0.479513 0.0724411
+0.160997 0.477019 0.0876359
+0.167909 0.48647 0.0907839
+0.168839 0.465921 0.0685027
+0.185848 0.461592 0.0756332
+0.163713 0.449815 0.0719867
+0.171034 0.442192 0.0868531
+0.172622 0.439121 0.100638
+0.176648 -0.0540189 0.138439
+0.191763 0.435033 0.0761478
+0.150024 0.450095 0.173316
+0.142684 0.444609 0.180888
+0.134925 0.435737 0.185852
+0.0901553 -0.0455623 0.257862
+0.0764097 -0.0467149 0.266788
+0.218445 0.190081 -0.00453558
+0.212983 0.187132 0.0158667
+0.234912 0.203072 -0.0090602
+0.217724 0.208098 -0.00630956
+0.209095 0.231498 0.0538229
+0.223398 0.222537 0.0510417
+0.229495 0.205818 0.051858
+0.241614 0.200114 0.0387668
+0.245196 0.212463 0.0252865
+0.232616 0.199508 0.066443
+0.245867 0.187445 0.0704899
+0.218046 0.211438 0.0580939
+0.206013 0.207163 0.0667587
+0.208167 0.217137 0.0808695
+0.189985 0.216171 0.0819923
+-0.328288 -0.282754 -0.0562593
+-0.321215 -0.270892 -0.0527491
+-0.321885 -0.294532 -0.0580289
+-0.324301 -0.301836 -0.070993
+-0.309665 -0.294095 -0.0515043
+-0.298437 -0.298535 -0.0585809
+-0.293755 -0.297175 -0.0685565
+-0.301865 -0.303693 -0.0670723
+-0.312242 -0.307406 -0.0760424
+0.113855 0.0103494 0.135894
+0.117356 -0.000872108 0.134011
+0.24192 0.231796 0.179613
+0.236568 0.24336 0.180529
+0.226028 0.250706 0.186633
+0.280884 -0.220307 -0.163899
+0.26654 -0.211351 -0.150233
+0.265056 -0.233891 -0.14678
+0.318482 -0.331697 -0.239506
+0.324395 -0.322699 -0.253479
+0.31194 -0.314679 -0.269415
+0.329739 -0.334727 -0.229668
+0.336703 -0.323152 -0.229076
+0.344428 -0.313085 -0.23571
+0.342498 -0.293727 -0.236916
+0.350543 -0.322922 -0.2465
+0.353252 -0.314 -0.261759
+0.347047 -0.326986 -0.235121
+0.0827363 -0.465408 -0.0592863
+0.0757733 -0.45735 -0.0483001
+0.0758332 -0.442944 -0.0486429
+0.0763754 -0.427162 -0.064874
+0.0810297 -0.429275 -0.0354104
+0.0760881 -0.402196 -0.0628009
+0.0803331 -0.462498 -0.0366385
+0.075746 -0.427719 -0.0496393
+0.213629 0.454927 0.154222
+0.216172 0.433851 0.152191
+0.213875 -0.163106 0.208849
+0.228479 0.0837143 -0.0171959
+0.214862 0.0777937 -0.0178642
+0.203914 0.0820668 -0.0169156
+0.213551 0.066976 -0.0133462
+-0.291696 -0.291685 -0.053913
+-0.288445 -0.286967 -0.0453936
+-0.283457 -0.280518 -0.0392925
+0.198493 0.267303 0.113918
+-0.00816198 -0.315253 0.11742
+0.312224 -0.337685 -0.224702
+0.0776115 -0.0114015 0.225043
+0.094616 -0.00802053 0.222738
+0.212523 -0.00317223 -0.153514
+0.228944 -0.0069181 -0.146501
+0.230166 -0.0223035 -0.138067
+0.212902 -0.0238532 -0.132712
+0.194522 -0.0219296 -0.138137
+0.240306 -0.00819778 -0.130311
+0.24497 -0.0122967 -0.113875
+0.221562 -0.0094445 -0.0973672
+0.242491 -0.00970392 -0.0927411
+-0.195428 -0.293105 -0.0588105
+-0.174639 -0.293714 -0.0615636
+-0.159198 -0.295096 -0.0708856
+-0.150745 -0.297027 -0.0916319
+-0.165094 -0.309633 -0.116063
+-0.129532 -0.293794 -0.0901166
+-0.12154 -0.29002 -0.105668
+-0.103655 -0.284754 -0.110478
+-0.083304 -0.273535 -0.126518
+-0.126229 -0.291149 -0.124142
+-0.0551206 -0.282297 -0.123125
+-0.0279253 -0.280704 -0.123568
+-0.0254074 -0.296918 -0.100527
+-0.143636 -0.296672 -0.0765836
+0.228772 0.249576 0.00807469
+0.246793 0.197559 -0.000456927
+0.249351 0.185655 0.00965958
+0.077471 -0.38535 -0.0716786
+0.0780138 -0.366405 -0.0698751
+0.0796117 -0.349597 -0.0694382
+0.0817673 -0.324046 -0.067713
+0.0920794 -0.339278 -0.0766687
+0.0745961 -0.308674 -0.0513846
+0.0686559 -0.285979 -0.0584538
+0.0701691 -0.296007 -0.0346939
+0.0958416 -0.324563 -0.0739969
+0.111686 -0.315208 -0.0712996
+0.118212 -0.295045 -0.0671537
+0.145789 -0.285221 -0.0572786
+0.142444 -0.258951 -0.0651148
+0.125807 -0.256448 -0.0777825
+0.0725557 -0.302564 -0.0188272
+0.0757738 -0.318738 -0.0113816
+0.0755625 -0.334153 -0.022547
+0.0716925 -0.292684 -0.00740334
+0.0574002 -0.292216 -0.00131701
+0.0442788 -0.29974 -0.0118635
+0.0322898 -0.309073 -0.00126775
+0.0158934 -0.319374 -0.00927168
+0.00270388 -0.318254 -0.036966
+0.00321031 -0.33353 0.006095
+0.0101246 -0.330737 0.0331595
+0.0571801 -0.291727 0.0142738
+0.0411549 -0.302632 0.0323113
+0.0568056 -0.290026 0.0325413
+0.0726196 -0.279494 0.0399911
+0.0843075 -0.293947 0.0355701
+0.0802412 -0.296732 0.0208207
+0.0757523 -0.290551 0.00911073
+0.0904109 -0.307387 0.0401159
+0.0937218 -0.321431 0.0333001
+0.0876185 -0.332554 0.0196986
+-0.261918 -0.356193 -0.123562
+-0.274572 -0.363058 -0.128675
+-0.283885 -0.374671 -0.137095
+0.0993425 -0.475253 -0.0345906
+0.105878 -0.470302 -0.022596
+0.161209 0.00758193 -0.157082
+0.162861 -0.00496129 -0.160125
+0.14622 0.0234899 0.00861402
+0.16407 0.00688456 0.00334424
+0.187476 0.0108016 -0.000104135
+0.198618 0.0267239 -0.00776275
+0.213159 0.032032 -0.0211609
+0.229764 0.0271876 -0.0288068
+0.229938 0.0432028 -0.0342052
+0.231973 0.0361819 -0.05417
+0.241585 0.0230077 -0.0674939
+0.235826 0.0200922 -0.0913851
+0.229532 0.00831192 -0.0604593
+0.228417 0.0182383 -0.11087
+0.214387 0.0313739 -0.095916
+0.23826 0.0319925 -0.0407356
+0.240603 0.0153722 -0.0478262
+0.185434 0.0390225 -0.0120783
+0.174046 0.035224 -0.0278333
+0.183039 0.0254835 -0.0435084
+0.185088 0.013208 -0.0664572
+0.184517 -0.00115634 -0.0944301
+0.15883 0.0112792 -0.0789407
+0.166768 0.0454452 -0.0125857
+0.149871 0.0473393 -0.0127261
+0.24097 0.0273212 -0.0523021
+0.244689 0.0183228 -0.0578172
+0.239121 0.00955592 -0.0564169
+0.23067 0.0133196 -0.0511149
+0.217815 0.013095 -0.0555127
+0.202953 0.0215885 -0.0445658
+0.223255 0.0309758 -0.00933388
+0.246338 0.00680161 -0.0920968
+-0.29666 -0.251794 -0.0420083
+-0.282377 -0.246563 -0.0382484
+-0.274248 -0.257163 -0.0411355
+0.121069 0.3764 0.220244
+0.124178 0.361607 0.222425
+0.12316 0.347289 0.21517
+0.120818 0.335014 0.200629
+0.131773 0.318764 0.203442
+0.240354 -0.189104 -0.174254
+0.226426 -0.17846 -0.157236
+0.25469 -0.197929 -0.186668
+0.262527 -0.207499 -0.20859
+0.275709 -0.206748 -0.189755
+0.268196 -0.218714 -0.226562
+0.252109 -0.214264 -0.221034
+0.231966 -0.219863 -0.222417
+0.16439 -0.290409 -0.045741
+0.174232 0.353361 0.161991
+0.167416 0.346038 0.151615
+0.153417 0.351454 0.147309
+0.141711 0.365653 0.160008
+0.174784 0.33351 0.146976
+0.172695 0.32064 0.137876
+0.171538 0.327653 0.122668
+-0.153467 -0.465689 -0.0917447
+-0.142244 -0.458217 -0.0978282
+-0.137315 -0.444624 -0.0945322
+-0.142546 -0.468457 -0.0929801
+-0.130113 -0.470071 -0.0887414
+-0.111868 -0.466578 -0.0912306
+-0.129822 -0.476932 -0.0773011
+-0.116192 -0.47396 -0.0799073
+-0.102072 -0.471199 -0.0816089
+0.0648327 -0.288276 0.006469
+0.290523 -0.14969 0.120852
+0.286341 -0.149084 0.132395
+0.276561 -0.154417 0.131433
+0.266874 -0.144863 0.128327
+0.266123 -0.155511 0.139202
+0.254682 -0.156501 0.14847
+0.243072 -0.144724 0.147014
+0.246658 -0.132891 0.13506
+0.271484 -0.158645 0.148826
+0.262513 -0.161728 0.150556
+0.255383 -0.163883 0.160093
+0.258182 -0.154155 0.170424
+0.247063 -0.165211 0.168804
+0.236375 -0.166699 0.177267
+0.223809 -0.16596 0.182974
+0.219971 -0.167303 0.19665
+0.224247 -0.15536 0.203023
+0.206877 -0.166582 0.198203
+0.201256 -0.167208 0.210293
+0.187949 -0.165778 0.214542
+0.175727 -0.151043 0.207983
+0.205839 -0.156778 0.218275
+0.191606 -0.158183 0.226132
+0.180969 -0.154585 0.235065
+0.170507 -0.150398 0.244702
+0.167678 -0.135398 0.246604
+0.174695 -0.121433 0.234619
+0.192199 -0.123695 0.219574
+0.161002 -0.150315 0.252813
+0.149901 -0.156196 0.25282
+0.145756 -0.15618 0.238445
+0.158231 -0.157867 0.22927
+0.148019 -0.147104 0.261177
+0.14318 -0.132144 0.260892
+0.131504 -0.114456 0.261736
+0.136414 -0.145181 0.266602
+0.124033 -0.152215 0.261533
+0.128756 -0.151616 0.248741
+0.169715 -0.162656 0.235355
+0.125332 -0.145025 0.270918
+0.118124 -0.13431 0.277651
+0.122928 -0.121301 0.271952
+0.113017 -0.148351 0.260498
+0.105868 -0.137752 0.254904
+0.0970603 -0.116879 0.246491
+0.113377 -0.13188 0.244126
+0.102989 -0.14407 0.26402
+0.09993 -0.141861 0.276203
+-0.208269 -0.22401 0.141633
+-0.20692 -0.250284 0.132437
+0.0965268 -0.427234 -0.105996
+0.0991486 -0.439673 -0.0998914
+0.19126 -0.17619 0.0390751
+0.184715 -0.204433 0.0324441
+0.164304 -0.221583 0.0465757
+0.1444 -0.208807 0.0686986
+0.0226172 -0.0147867 0.137076
+-0.142255 -0.49298 -0.0216335
+0.233691 0.0208001 -0.0385404
+-0.192672 -0.315988 0.0839294
+-0.202286 -0.295403 0.104035
+-0.212462 -0.276423 0.111373
+-0.218066 -0.285356 0.0914372
+-0.230517 -0.270625 0.068595
+0.136055 0.0355608 0.0131192
+0.121886 0.0399552 0.0198872
+0.106186 0.0275039 0.0255554
+0.0928691 0.0373146 0.0446539
+0.107832 0.0113031 0.0106037
+0.0853288 0.00897116 0.0135994
+0.0648776 -0.00152148 0.00684991
+0.301901 0.1917 0.196837
+0.304498 0.181718 0.192274
+0.29757 0.171529 0.192207
+0.283611 0.170157 0.190984
+0.17589 0.332048 0.230978
+0.176274 0.318104 0.221305
+0.189786 0.308195 0.22975
+0.199452 0.291101 0.221444
+0.210647 0.278153 0.221165
+0.212403 0.290739 0.232707
+0.229872 0.295702 0.229994
+0.238851 0.283146 0.23072
+0.237981 0.260514 0.227763
+0.248013 0.243086 0.221386
+0.245976 0.271423 0.230129
+0.237201 0.240055 0.208931
+0.239318 0.292509 0.222434
+0.244312 0.28734 0.210021
+0.234086 0.289095 0.199595
+0.221914 0.298769 0.201947
+0.1778 0.322602 0.229777
+0.232474 0.299285 0.221376
+0.223334 0.304746 0.213608
+0.207337 0.311267 0.204764
+0.224642 0.304397 0.226583
+0.214619 0.302742 0.233384
+0.203769 0.313465 0.232857
+0.204968 0.319702 0.218324
+0.201735 0.298965 0.232154
+-0.0492057 -0.0812756 -0.10551
+-0.0710597 -0.075701 -0.12067
+-0.0879497 -0.0905516 -0.13677
+-0.0843511 -0.119489 -0.147588
+-0.0762899 -0.059326 -0.102542
+-0.0985707 -0.0477827 -0.098696
+-0.118275 -0.0536394 -0.113913
+-0.11441 -0.0717159 -0.128365
+0.179503 0.303256 0.0436704
+0.192013 0.290229 0.0290644
+0.17689 0.287515 0.0370015
+0.192038 0.271729 0.0156116
+0.209167 0.267708 0.00969983
+0.200133 0.255704 0.00478026
+-0.247504 -0.392344 -0.224787
+-0.247477 -0.405986 -0.218212
+-0.243099 -0.400741 -0.206484
+-0.246703 -0.405341 -0.193269
+-0.249065 -0.417066 -0.193806
+-0.263856 -0.417281 -0.202583
+-0.253854 -0.410374 -0.185389
+-0.264275 -0.407223 -0.177693
+-0.272044 -0.40206 -0.166216
+-0.284666 -0.409799 -0.163624
+-0.28689 -0.423019 -0.170791
+-0.283285 -0.428723 -0.186577
+-0.301536 -0.424549 -0.197331
+-0.276684 -0.426319 -0.176604
+-0.270204 -0.426002 -0.18843
+-0.267033 -0.421163 -0.178871
+-0.28572 -0.431162 -0.176917
+0.0984334 0.0428629 0.147655
+0.0984512 0.0370662 0.158757
+-0.324734 -0.278381 -0.0492733
+-0.319049 -0.278035 -0.0388895
+-0.314693 -0.271902 -0.0298211
+-0.0814975 -0.486596 -0.0514846
+-0.0716057 -0.47534 -0.0576231
+-0.0807984 -0.475501 -0.0687174
+-0.0782273 -0.46782 -0.0773567
+-0.0794515 -0.454639 -0.076562
+-0.0865219 -0.442019 -0.0638219
+-0.0826697 -0.456554 -0.0896798
+-0.0913604 -0.446861 -0.0969099
+-0.0924184 -0.432044 -0.0753911
+-0.098273 -0.416607 -0.0817744
+-0.111942 -0.395187 -0.0818225
+-0.0923505 -0.401008 -0.0727607
+-0.110463 -0.376831 -0.0757945
+-0.0955645 -0.358515 -0.0727168
+-0.0748788 -0.367775 -0.053732
+-0.099584 -0.336657 -0.0718408
+-0.0617229 -0.346951 -0.0508932
+-0.0372908 -0.340191 -0.0358611
+-0.187209 -0.0514446 0.134371
+-0.172275 -0.062062 0.144345
+-0.181772 -0.0387772 0.12984
+0.27325 0.225257 0.167666
+0.283037 0.221467 0.169903
+0.141164 -0.00931766 -0.150578
+0.223335 -0.262546 -0.149684
+0.199958 -0.258684 -0.141852
+0.218212 -0.249649 -0.131577
+-0.0730224 -0.485274 -0.0383372
+-0.0765985 -0.473674 -0.025717
+-0.0833487 -0.485599 -0.0202509
+-0.0969784 -0.487241 -0.0157338
+-0.106328 -0.475595 -0.0156679
+-0.107519 -0.486132 -0.0147883
+-0.117398 -0.491224 -0.0155421
+-0.11866 -0.497871 -0.0255495
+0.192205 -0.0198585 -0.151725
+0.281772 0.238327 0.210562
+0.269826 0.243309 0.222088
+0.243244 0.207816 -0.00325363
+0.239729 0.221932 -0.000908316
+0.243256 0.225044 0.0142927
+0.235434 0.23543 0.00346349
+0.158954 -0.447934 -0.074517
+0.164 -0.440163 -0.0849532
+0.160645 -0.422992 -0.0839209
+0.160099 -0.41995 -0.0633914
+0.170063 -0.432118 -0.0580684
+0.170775 -0.442201 -0.0470561
+0.167356 -0.452883 -0.036848
+0.157537 -0.464023 -0.0432353
+0.154265 -0.458207 -0.0582632
+0.146921 -0.466798 -0.0291891
+0.17099 -0.445942 -0.0649823
+0.152813 -0.426396 -0.0985179
+0.142833 -0.435406 -0.0989063
+0.126573 -0.436557 -0.100225
+0.120949 -0.42189 -0.103936
+0.130685 -0.411647 -0.0938972
+0.160351 -0.460975 -0.0324222
+0.164884 -0.460693 -0.0398836
+0.167083 -0.430709 -0.0834012
+0.161089 -0.432508 -0.0931652
+0.165433 -0.424956 -0.0749657
+0.155909 -0.417913 -0.0743018
+0.149825 -0.407366 -0.067413
+0.142015 -0.399818 -0.0764637
+0.153043 -0.435416 -0.0966096
+0.148842 -0.431897 -0.101759
+0.145045 -0.424265 -0.10134
+0.147732 -0.418014 -0.0911084
+0.138977 -0.417451 -0.0965307
+0.139635 -0.411036 -0.0878314
+0.14174 -0.430413 -0.104765
+0.134784 -0.434536 -0.101822
+0.135086 -0.440748 -0.0918112
+0.13766 -0.447456 -0.0800542
+0.122938 -0.455334 -0.0698495
+0.127914 -0.428797 -0.106691
+0.135671 -0.430083 -0.106069
+-0.0268041 -0.304928 0.142542
+-0.0182344 -0.281255 0.151927
+0.000262015 -0.258244 0.142866
+0.0151742 -0.229959 0.128466
+-0.00970849 -0.227931 0.154073
+0.0337917 -0.209923 0.115845
+0.0199495 -0.192987 0.125832
+-0.154293 -0.0335699 -0.082135
+-0.129631 -0.0324487 -0.0813475
+-0.120097 -0.027431 -0.0586998
+-0.0956922 -0.0340394 -0.0533561
+-0.0814148 -0.0448428 -0.0722969
+-0.0594432 -0.0515596 -0.0534184
+-0.160793 -0.0482086 -0.0989707
+-0.166155 -0.0307425 -0.0663998
+-0.169924 -0.0270352 -0.0414151
+-0.183311 -0.0375758 -0.0551581
+-0.174206 -0.0274939 -0.0203147
+-0.192353 -0.0397338 -0.00141151
+-0.170447 -0.0249801 0.0063437
+-0.211587 -0.0636911 -0.00501259
+-0.0018753 -0.187141 -0.149775
+0.0183103 -0.182965 -0.142326
+0.0322217 -0.167313 -0.134641
+0.0236675 -0.146992 -0.123167
+-0.00232452 -0.142332 -0.126149
+0.0375806 -0.18533 -0.140019
+0.0530379 -0.201545 -0.137283
+0.0772348 -0.20845 -0.140694
+0.0988175 -0.224384 -0.140468
+0.119251 -0.222512 -0.162731
+0.03788 -0.218276 -0.135529
+0.0772845 -0.19139 -0.155355
+0.080882 -0.225907 -0.127445
+0.0653619 -0.242778 -0.113036
+0.0731805 -0.173068 -0.155239
+0.0897045 -0.191329 -0.166141
+0.102707 -0.199007 -0.171074
+0.119058 -0.208637 -0.176942
+0.127514 -0.196293 -0.185172
+0.13998 -0.205302 -0.190755
+0.149824 -0.215626 -0.194546
+0.161245 -0.221969 -0.199237
+0.175364 -0.230409 -0.20401
+0.173584 -0.215336 -0.204853
+0.178202 -0.198362 -0.197173
+0.188397 -0.22907 -0.211289
+0.202734 -0.22056 -0.215065
+0.199745 -0.239634 -0.214151
+0.0815106 -0.18364 -0.162988
+-0.172104 -0.359269 -0.00938238
+-0.172319 -0.335226 -0.0164663
+-0.16873 -0.368903 -0.0231312
+-0.292266 -0.291505 -0.0889456
+-0.288266 -0.299574 -0.0955502
+-0.280983 -0.308012 -0.105167
+-0.278654 -0.297571 -0.101297
+-0.274336 -0.285615 -0.103446
+-0.260134 -0.28158 -0.0989442
+-0.257005 -0.265144 -0.10215
+-0.26942 -0.305285 -0.10276
+-0.257003 -0.309674 -0.100476
+-0.244993 -0.306014 -0.0938734
+-0.249351 -0.292113 -0.0926885
+-0.25954 -0.322424 -0.104282
+-0.267093 -0.332079 -0.111051
+-0.283312 -0.328944 -0.119574
+-0.249017 -0.331591 -0.10481
+-0.232981 -0.32784 -0.100164
+-0.240291 -0.342062 -0.113719
+-0.229688 -0.345883 -0.126712
+-0.23058 -0.352629 -0.145077
+-0.21352 -0.337371 -0.127344
+-0.269191 -0.344874 -0.117105
+-0.208623 -0.327937 -0.112241
+-0.191793 -0.321843 -0.117022
+-0.180909 -0.311277 -0.104708
+0.11012 0.10505 0.0238496
+0.213679 0.221732 0.163906
+-0.0357839 -0.0025294 0.108473
+-0.0312254 -0.0135193 0.128152
+-0.0238807 -0.033229 0.139313
+-0.00300831 -0.046529 0.144036
+-0.00364169 -0.0760125 0.145155
+-0.0103288 -0.10643 0.141831
+0.015326 -0.129347 0.142131
+-0.041062 -0.0443202 0.130625
+-0.0555252 -0.0465254 0.114753
+-0.0556686 -0.0325657 0.0996413
+-0.0768736 -0.0422105 0.0949058
+-0.0167984 0.000564353 0.123722
+0.00524698 0.0020139 0.129964
+-0.0281137 -0.0861213 0.139333
+-0.0785841 -0.0379469 0.0747431
+-0.0762529 -0.0505618 0.114297
+-0.032521 -0.108383 0.136839
+-0.0633754 -0.0458183 0.101476
+-0.0250298 0.00663901 0.112981
+-0.0219675 0.00393164 0.0935556
+-0.147404 -0.304789 -0.127071
+0.192111 0.473304 0.143665
+0.202701 0.475169 0.131787
+0.206558 0.475874 0.120017
+0.202492 0.480449 0.108775
+0.157654 -0.366957 -0.0205798
+0.26661 -0.307414 -0.281977
+0.270077 -0.295571 -0.288815
+0.283263 -0.291236 -0.297392
+0.290598 -0.279448 -0.297082
+0.304158 -0.276932 -0.29882
+0.315035 -0.2885 -0.301158
+0.307588 -0.298676 -0.297006
+0.297613 -0.307939 -0.283621
+0.293787 -0.301201 -0.295424
+0.318389 -0.297707 -0.296483
+0.328066 -0.301032 -0.289042
+0.324582 -0.309109 -0.276338
+0.33579 -0.291676 -0.2964
+0.346867 -0.288071 -0.287976
+0.353956 -0.299169 -0.28317
+0.345495 -0.307423 -0.274703
+0.352866 -0.288693 -0.277818
+0.351312 -0.284395 -0.265224
+0.355354 -0.294774 -0.257468
+0.360217 -0.304818 -0.260578
+0.35682 -0.308388 -0.269609
+0.359106 -0.312264 -0.252967
+0.356474 -0.316127 -0.245135
+0.351445 -0.319298 -0.237976
+-0.26647 -0.314705 -0.199507
+-0.27426 -0.329739 -0.204261
+-0.290128 -0.337563 -0.206145
+-0.303723 -0.332243 -0.195363
+-0.303858 -0.323407 -0.176279
+-0.302757 -0.349394 -0.210447
+-0.304665 -0.364526 -0.223447
+-0.321319 -0.375627 -0.232938
+-0.285437 -0.371465 -0.224735
+-0.28011 -0.37957 -0.242654
+-0.314609 -0.397575 -0.24523
+-0.31498 -0.344154 -0.199865
+-0.330892 -0.351901 -0.189469
+-0.332902 -0.388937 -0.243068
+-0.348245 -0.402805 -0.233911
+-0.328232 -0.404959 -0.235266
+-0.273541 -0.396265 -0.240028
+-0.293049 -0.39498 -0.245656
+-0.355214 -0.402159 -0.217135
+-0.360108 -0.415844 -0.205721
+-0.346381 -0.42668 -0.201491
+-0.340374 -0.436455 -0.180032
+-0.360217 -0.406031 -0.193698
+-0.356506 -0.391255 -0.182727
+-0.356512 -0.407116 -0.177427
+-0.347384 -0.421732 -0.172779
+-0.348669 -0.37532 -0.17511
+-0.341266 -0.408591 -0.160307
+-0.332592 -0.391247 -0.153464
+-0.323849 -0.407723 -0.153687
+-0.349384 -0.43169 -0.189135
+-0.335815 -0.43119 -0.192046
+-0.324454 -0.435806 -0.181733
+-0.331465 -0.433295 -0.170921
+-0.314528 -0.431132 -0.168412
+-0.260177 -0.397767 -0.235
+-0.252953 -0.401301 -0.227678
+-0.247407 -0.394723 -0.238053
+-0.24372 -0.401046 -0.225417
+-0.243948 -0.396615 -0.216223
+-0.258003 -0.38952 -0.247437
+-0.272162 -0.387844 -0.252537
+-0.287147 -0.384539 -0.255755
+-0.302942 -0.384129 -0.252391
+-0.315505 -0.382766 -0.24459
+-0.323099 -0.390444 -0.249836
+-0.312399 -0.390336 -0.253745
+-0.328503 -0.399215 -0.245058
+-0.340635 -0.398589 -0.24271
+-0.266346 -0.382624 -0.244488
+-0.267321 -0.391914 -0.245578
+-0.0266812 0.0695328 -0.0242573
+-0.00773299 0.0681739 -0.0184911
+0.0122858 0.0669077 -0.0123781
+0.0150035 0.0767227 0.00239352
+0.0141467 0.0954062 -0.00215996
+0.0325338 0.098411 -0.00617133
+0.0450676 0.0983028 0.010086
+0.0314473 0.0730983 0.00401189
+0.0505593 0.0686309 0.00292132
+0.0698817 0.067505 0.00832925
+0.145383 0.180744 0.0984363
+0.132189 0.17376 0.0925198
+0.121241 0.164951 0.0850023
+0.133425 0.169934 0.0774819
+0.11505 0.153676 0.07879
+-0.083942 -0.368893 -0.0674225
+-0.0809876 -0.385027 -0.0581697
+-0.0723248 -0.382058 -0.0416794
+-0.00904893 0.0914446 -0.0120745
+0.00504523 0.0987359 -0.0150796
+0.0060609 0.0932522 -0.034057
+0.0248461 0.0873188 -0.0377031
+0.0363994 0.089055 -0.023789
+0.00213821 0.0914225 -0.00482177
+0.0092558 0.0863088 0.00212053
+0.106375 0.0274106 0.14138
+-0.263884 -0.385451 -0.252252
+0.258755 0.24689 0.225661
+0.259085 0.23306 0.219814
+0.249971 0.25591 0.228242
+-0.322841 -0.345115 -0.164492
+-0.323706 -0.355326 -0.152393
+-0.279169 -0.265328 -0.0439542
+-0.285416 -0.267917 -0.0516616
+-0.294745 -0.263175 -0.0517725
+0.23393 -0.0149701 -0.10397
+0.219738 -0.0165453 -0.112039
+0.204608 -0.00981825 -0.104246
+0.187184 -0.00954937 -0.110855
+0.228145 0.230452 0.102316
+0.214447 0.238029 0.0983297
+0.23229 0.220992 0.0943503
+0.215398 0.247623 0.105271
+0.217938 0.25177 0.117669
+0.210276 0.258003 0.111405
+0.220949 0.241286 0.103103
+0.290344 -0.337843 -0.233955
+0.276226 -0.337233 -0.22831
+0.296573 -0.339782 -0.226215
+0.227663 0.461812 0.0838481
+0.234265 0.455281 0.0718225
+0.229698 0.445794 0.0603386
+0.225781 0.470182 0.0813133
+0.214396 0.4698 0.0788369
+0.208406 0.478123 0.0862021
+0.214031 0.460896 0.0711899
+0.217085 0.451741 0.0628259
+0.219354 0.474333 0.0827819
+0.21619 0.47758 0.0903862
+0.217994 0.472178 0.097233
+0.209525 0.481852 0.0939712
+0.227208 0.456115 0.0633709
+0.234329 0.451682 0.0642008
+0.23166 0.462658 0.0759848
+0.273713 -0.249314 -0.252993
+0.274317 -0.268417 -0.267071
+0.263304 -0.282613 -0.260934
+-0.240326 -0.404033 -0.2139
+-0.241182 -0.408607 -0.207233
+0.243855 0.194683 0.113624
+0.235959 0.206195 0.114609
+-0.329827 -0.30598 -0.098469
+-0.057071 -0.0425853 0.0117122
+-0.0551192 -0.0420888 0.0309046
+-0.0534835 -0.0402863 0.0500454
+-0.0435993 -0.0469165 0.00748095
+-0.0255629 -0.0374196 0.00481763
+-0.00817324 -0.0328811 -0.0061182
+-0.00350439 -0.0437548 -0.0255784
+-0.0349503 -0.0490115 -0.00492533
+-0.0419875 -0.0536475 -0.0293419
+-0.0139014 -0.0607747 -0.0378374
+-0.0105205 -0.0780801 -0.0585195
+-0.0335249 -0.0540562 -0.0180409
+-0.0278411 -0.0595083 -0.0313207
+0.193253 0.49569 0.0858481
+0.189805 0.494474 0.0949255
+0.179559 0.491992 0.0947812
+0.19709 0.487808 0.0978913
+0.174576 0.497081 0.0879734
+0.179584 0.496616 0.0784813
+0.175502 0.48892 0.0725035
+0.169475 0.481635 0.069206
+0.164994 0.474033 0.0669237
+0.155945 0.469227 0.0714265
+0.159639 0.459314 0.0671634
+0.160451 -0.0544036 0.113535
+0.0454737 -0.0938547 0.293886
+0.0481458 -0.104952 0.292926
+0.0550885 -0.114349 0.29062
+0.0654216 -0.120788 0.289843
+0.0691263 -0.126298 0.278339
+0.0372417 -0.0943784 0.286279
+0.0377118 -0.088702 0.274623
+0.179652 -0.262284 0.0054219
+0.186087 -0.244472 0.00347757
+0.246807 -0.00615903 -0.103541
+0.242297 -0.0135206 -0.100916
+0.240802 -0.0172339 -0.108388
+0.232593 -0.0188607 -0.112191
+0.240348 -0.0194541 -0.117278
+0.238974 -0.018297 -0.127651
+-0.164632 -0.397326 -0.024693
+0.0813645 -0.458079 -0.0716845
+0.302397 0.127887 0.0470846
+0.298473 0.138148 0.0455108
+0.292313 0.136477 0.0386335
+0.288054 0.130079 0.0326142
+0.28271 0.137427 0.0379629
+0.271939 0.136706 0.038053
+0.261008 0.14116 0.0404912
+0.28194 0.12422 0.0312108
+0.300876 0.126279 0.0551937
+0.295306 0.129324 0.0632115
+0.286846 0.130603 0.0706512
+0.282604 0.11846 0.0804122
+0.273418 0.134767 0.0747422
+0.296205 0.121995 0.0600976
+0.289869 0.116278 0.0576534
+0.283444 0.108666 0.0679441
+0.208186 0.435058 0.0678801
+0.218992 0.437774 0.0675136
+0.20517 0.444041 0.0615
+0.195457 0.447135 0.0675955
+0.224157 0.438328 0.0597838
+0.221367 0.446069 0.0584788
+0.168291 -0.443724 -0.076199
+-0.239169 -0.248023 -0.0426243
+-0.246509 -0.244281 -0.0523825
+-0.245933 -0.250709 -0.0701381
+-0.252213 -0.23035 -0.0598084
+-0.256922 -0.212794 -0.063622
+-0.26443 -0.20039 -0.048456
+-0.236601 -0.248665 -0.0331221
+-0.248238 -0.244973 -0.04299
+-0.257005 -0.243536 -0.0363368
+-0.264619 -0.253098 -0.0324677
+-0.260909 -0.234911 -0.0387305
+-0.270856 -0.228969 -0.0335016
+-0.268106 -0.214873 -0.0367573
+-0.255525 -0.250409 -0.0291707
+-0.246354 -0.252152 -0.0213798
+-0.25421 -0.258055 -0.0140562
+-0.253371 -0.258976 0.00135493
+-0.263391 -0.256528 0.0180325
+-0.264412 -0.265611 -0.0106557
+-0.268835 -0.263918 0.00476582
+-0.24972 -0.252323 0.0327963
+-0.239732 -0.259608 0.0493553
+-0.242639 -0.251027 0.0706418
+-0.27192 -0.270836 -0.02103
+-0.264888 -0.241199 0.0366373
+-0.279792 -0.22631 0.033251
+-0.274206 -0.207933 0.0474852
+-0.283361 -0.276288 -0.0174295
+-0.267659 -0.226048 0.0482271
+0.202151 0.274483 0.19338
+0.194908 0.283204 0.198963
+-0.157532 -0.273615 -0.179435
+-0.176899 -0.279729 -0.184503
+-0.188947 -0.290942 -0.187096
+-0.192598 -0.3074 -0.18306
+-0.203551 -0.297799 -0.190929
+-0.222085 -0.288246 -0.191352
+-0.217908 -0.303036 -0.194268
+-0.355074 -0.426501 -0.195973
+-0.354866 -0.423993 -0.205337
+-0.355569 -0.419108 -0.214528
+-0.353763 -0.412061 -0.224887
+-0.346029 -0.286085 -0.1062
+-0.341227 -0.288967 -0.0947058
+-0.336277 -0.278132 -0.0971269
+-0.328988 -0.269127 -0.105169
+-0.340297 -0.292656 -0.0831052
+-0.335578 -0.266756 -0.115188
+-0.339311 -0.299368 -0.0917431
+0.212569 0.261061 0.18216
+0.216886 0.255854 0.175009
+0.219484 0.251322 0.162982
+0.212289 0.247584 0.170006
+0.218513 0.26322 0.154269
+0.225667 0.261532 0.178216
+0.218436 0.276817 0.178562
+0.234042 0.273996 0.185236
+0.223426 0.242038 0.154678
+0.21181 0.288948 0.181391
+0.220498 0.257755 0.18435
+0.211254 0.266563 0.187703
+0.211739 0.26954 0.199826
+0.278409 -0.209413 -0.174692
+0.233056 0.457965 0.0658843
+0.227063 0.46182 0.0682472
+0.220168 0.458063 0.0666597
+-0.0481392 -0.0447802 0.0166181
+0.131516 0.0530135 0.000672445
+0.12038 0.0567042 0.000376152
+0.134766 0.046581 0.0097546
+0.0956782 -0.141364 0.26837
+0.0877085 -0.13595 0.269242
+0.0854698 -0.124959 0.261926
+0.0839071 -0.111836 0.253439
+0.0904091 -0.099649 0.239147
+0.0872053 -0.136949 0.279095
+0.085161 -0.130401 0.287023
+0.0763801 -0.13103 0.282851
+-0.00884361 -0.0890856 -0.0728998
+-0.00654069 -0.102279 -0.0895079
+-0.0290648 -0.104665 -0.111248
+-0.0100257 -0.116287 -0.107029
+0.0875054 -0.104584 0.297054
+0.191386 -0.264049 -0.175252
+0.190045 -0.262732 -0.156889
+0.204589 -0.269071 -0.178679
+0.222111 -0.273266 -0.172895
+0.189235 0.0753918 -0.0129238
+0.0782752 -0.467624 -0.0498343
+0.0759673 -0.46177 -0.0555898
+0.0772195 -0.460605 -0.0642783
+0.151932 0.323656 0.079912
+0.153175 0.313215 0.0881136
+0.161272 0.302381 0.0857396
+0.163146 0.324848 0.0718597
+0.151557 0.334415 0.0880604
+0.142886 0.334889 0.0972586
+0.140662 0.34411 0.107021
+0.133598 0.347717 0.117582
+0.131314 0.355467 0.129074
+0.127988 0.361743 0.142546
+0.123763 0.348981 0.138438
+0.119822 0.353337 0.149718
+0.116288 0.351928 0.168204
+0.226419 0.174912 -0.00671405
+0.232006 0.15884 0.00199041
+0.243933 0.169091 0.00253819
+0.237571 0.172786 -0.00612936
+0.183717 0.142245 0.147062
+0.183533 0.157982 0.153224
+0.200684 0.169917 0.154336
+0.175927 0.14817 0.154877
+0.164064 0.143191 0.159694
+0.181391 0.132744 0.149775
+0.177453 0.123521 0.157312
+0.108424 0.0602759 0.0311611
+0.101691 0.0511896 0.042029
+0.297187 0.12858 0.124591
+0.286869 0.125817 0.115842
+0.279264 0.13651 0.11071
+0.277464 0.150893 0.114839
+0.267972 0.163015 0.116989
+0.29066 -0.215317 -0.195858
+-0.0439761 -0.143405 -0.149346
+0.0959309 0.0199379 0.158053
+0.0941358 0.00844127 0.16726
+0.273991 -0.158318 0.138404
+0.280108 -0.155995 0.136908
+0.28311 -0.154668 0.131232
+0.287165 -0.152142 0.126309
+0.291082 -0.145525 0.127381
+0.258354 -0.329753 -0.2515
+0.256649 -0.327468 -0.240856
+0.265642 -0.321399 -0.233195
+0.269005 -0.32965 -0.227653
+0.204877 0.287044 0.0357487
+0.289139 -0.339472 -0.226774
+0.282728 -0.335989 -0.224475
+0.283065 -0.327384 -0.221193
+0.28398 -0.316486 -0.219293
+0.289461 -0.305296 -0.210924
+0.2738 -0.310998 -0.221733
+0.2646 -0.301502 -0.221281
+0.282981 -0.339471 -0.231684
+0.275544 -0.336339 -0.239462
+0.259131 -0.2954 -0.232139
+0.281853 -0.296955 -0.202405
+0.287258 -0.287693 -0.192682
+0.301236 -0.282638 -0.194913
+0.23745 0.0270265 -0.0333549
+0.234865 0.0358956 -0.0292661
+0.240774 0.0243245 -0.0402136
+0.034599 -0.0884904 0.28182
+0.0345637 -0.0787252 0.277243
+0.0422003 -0.0728232 0.283477
+0.048607 -0.0763619 0.292696
+0.0395236 -0.0802562 0.266836
+0.0486856 -0.0788086 0.257578
+0.044992 -0.0647291 0.254151
+0.0587204 -0.0731238 0.296598
+0.068389 -0.0812067 0.300077
+0.0829644 -0.0808872 0.290453
+0.0435251 -0.0559756 0.262078
+0.20354 0.276522 0.016541
+-0.0980428 -0.240155 0.197738
+-0.0924965 -0.26196 0.186819
+-0.109853 -0.270124 0.168775
+-0.253582 -0.386742 -0.238773
+-0.0267016 0.0982672 -0.0374627
+0.214024 0.433945 0.0622105
+0.204736 0.432758 0.058829
+0.201109 0.433103 0.0655996
+0.201809 0.436011 0.073894
+0.193477 0.433855 0.0682907
+0.185218 0.436354 0.0703184
+0.180836 0.436291 0.0819631
+0.191166 0.440882 0.0659291
+0.187348 0.447071 0.070626
+0.179215 0.453739 0.0726364
+0.193028 0.454826 0.0736221
+0.173421 0.440644 0.0776765
+-0.147031 -0.496444 -0.028386
+-0.151597 -0.495421 -0.0374969
+-0.157627 -0.484775 -0.0397619
+-0.140246 -0.499465 -0.0256815
+-0.132401 -0.5 -0.0296698
+-0.132703 -0.497498 -0.0384881
+-0.126331 -0.494492 -0.0452588
+-0.11646 -0.490117 -0.0524448
+-0.101295 -0.487303 -0.0547567
+-0.136101 -0.494007 -0.0471871
+-0.13938 -0.490039 -0.0561432
+-0.133381 -0.483866 -0.0661423
+-0.15577 -0.492035 -0.0446482
+-0.153261 -0.490282 -0.0555144
+0.197755 0.272342 0.0690149
+0.190382 0.263313 0.0560052
+0.0686632 -0.292912 -0.0237205
+0.056243 -0.29127 -0.0303266
+0.0398126 -0.298058 -0.030698
+0.0315681 -0.295788 -0.0489563
+0.043708 -0.285681 -0.061727
+0.0621136 -0.289132 -0.040881
+0.0722108 -0.295077 -0.0484755
+0.0577034 -0.286988 -0.0524253
+0.0569935 -0.281989 -0.0659264
+0.064236 -0.290328 -0.0328163
+-0.0127838 0.0757233 -0.00921143
+0.0935192 0.0772038 0.0642915
+0.169714 0.302879 0.0497006
+0.163659 0.300165 0.0640716
+0.172929 0.295611 0.0434924
+0.049865 0.0913802 0.0221499
+0.0418831 0.0808731 0.013212
+0.0368549 0.0913191 0.0145569
+0.0319565 0.0968433 0.00544037
+0.310435 0.13526 0.142507
+0.026474 0.0305763 -0.129196
+0.017822 0.0292426 -0.122273
+0.0163904 0.0280919 -0.108702
+0.0350495 0.0278097 -0.132911
+0.178361 0.286185 0.0866978
+0.184473 0.288229 0.0959947
+0.0746028 -0.0119842 0.202652
+0.0770488 -0.00198153 0.201878
+0.0861829 0.00359659 0.206228
+0.0964676 0.00912576 0.20305
+0.110414 0.00821695 0.200752
+0.119478 0.0159283 0.18886
+-0.0749585 -0.470198 -0.0703816
+-0.0722579 -0.469101 -0.0627931
+-0.0775597 -0.45771 -0.0519849
+-0.0725204 -0.466379 -0.0530837
+-0.0822617 -0.458336 -0.0360309
+0.11796 -0.00196684 -0.151498
+0.110489 0.008862 -0.155734
+0.119387 0.0273131 -0.141295
+0.100036 0.00317869 -0.149099
+0.0946498 0.00360487 -0.130289
+0.0996271 0.00897841 -0.108462
+0.0876406 0.00969553 -0.149119
+0.0889673 0.0239205 -0.144401
+0.103773 0.0275171 -0.140968
+0.112143 0.0407687 -0.122665
+0.128275 -0.00210722 -0.155193
+0.136944 0.00690927 -0.158099
+0.1315 0.01712 -0.150046
+0.148823 0.0111196 -0.154092
+0.137878 -0.00286061 -0.157253
+0.0812501 0.0180999 -0.148671
+0.0723702 0.0199576 -0.146504
+0.0894465 0.0177562 -0.14994
+-0.244247 -0.411744 -0.197734
+0.0532101 -0.0425986 0.239458
+0.0614299 -0.034607 0.250277
+0.0718515 -0.0264935 0.244569
+0.0612036 -0.0408317 0.227838
+0.211416 0.21985 0.0506815
+0.209629 0.209187 0.0516229
+0.197715 0.200596 0.0531448
+0.210711 0.212673 0.041983
+0.209533 0.205981 0.0261421
+0.207685 0.214484 0.0320655
+0.204793 0.215907 0.019768
+0.207322 0.190012 0.0316877
+0.210424 0.206724 0.0353988
+0.209086 0.197885 0.0355421
+0.19948 0.191764 0.0420701
+0.206287 0.1798 0.0239909
+0.199532 0.162119 0.0159658
+0.0889829 0.0153312 0.187549
+0.0895058 0.0194699 0.175832
+0.0997882 0.0258376 0.180178
+0.0912633 -0.43583 -0.104962
+0.0893849 -0.44209 -0.0966632
+0.0818551 -0.438899 -0.0891003
+0.0775492 -0.435915 -0.0759439
+0.0805228 -0.426773 -0.087989
+0.0848008 -0.421093 -0.0971385
+0.0844397 -0.410175 -0.0930928
+0.0796444 -0.399052 -0.082153
+0.096306 -0.399151 -0.0928503
+0.0992866 -0.434977 -0.10667
+-0.038871 -0.095203 0.134909
+-0.0453136 -0.074362 0.132403
+-0.0642973 -0.0816285 0.135216
+0.128958 0.371237 0.164377
+0.114324 0.368213 0.169385
+0.110394 0.358489 0.182698
+0.119359 0.382173 0.172221
+0.244566 0.0274799 0.091132
+0.236517 0.0321023 0.108593
+0.228129 0.0154777 0.110489
+0.243652 -0.0044318 0.106704
+0.215089 0.0116198 0.126655
+0.193164 -0.00203925 0.149761
+0.295901 -0.145148 0.11649
+0.181002 0.326878 0.162405
+0.192626 0.308402 0.164847
+0.180295 0.335983 0.171005
+0.215019 0.222685 -0.0085752
+0.216226 0.238327 -0.00679645
+0.204842 0.243639 -0.00108745
+0.197086 0.237151 0.00943393
+0.207737 0.21475 -0.00125832
+0.200663 0.225332 0.00994825
+0.227239 0.239308 -0.00361834
+0.210369 0.205349 -0.000451507
+0.212421 0.195338 0.00658041
+0.0965367 0.0723068 0.0490018
+-0.263237 -0.382248 -0.20077
+-0.334935 -0.395374 -0.246716
+0.0666968 0.0983382 0.0352739
+0.0735768 0.104043 0.0256427
+0.0854137 0.105714 0.0280147
+0.0966684 0.103408 0.0197312
+-0.293219 -0.246559 0.00868918
+-0.284829 -0.260543 0.00656712
+0.231694 -0.239358 -0.229883
+0.249173 -0.248287 -0.23972
+-0.313753 -0.278534 -0.156686
+-0.167921 -0.0222432 0.100354
+-0.166557 -0.0217484 0.0804222
+-0.137191 -0.0189498 0.0544508
+-0.183157 -0.0297007 0.0652393
+-0.193717 -0.0386652 0.0428719
+-0.162262 -0.0205653 0.0563587
+-0.148465 -0.0201145 0.071316
+-0.122512 -0.0229086 0.0762312
+-0.112808 -0.0225311 0.0535636
+-0.15755 -0.0225071 0.112728
+-0.13968 -0.0242143 0.109228
+-0.128528 -0.0323847 0.121144
+-0.137944 -0.0426964 0.133983
+0.338353 -0.331842 -0.233
+0.197716 0.0369013 -0.0158252
+0.225598 0.0269623 0.107608
+0.227611 0.0352699 0.116368
+0.216481 0.0416846 0.126455
+0.20366 0.0291575 0.127785
+0.197706 0.0139181 0.138946
+0.177004 0.0154396 0.15063
+0.196436 0.0406475 0.130552
+0.183582 0.0465791 0.137895
+0.171962 0.0340634 0.144092
+0.209838 0.0350513 0.1214
+0.122018 -0.0147207 0.202688
+0.134625 -0.010294 0.190699
+0.150345 -0.0204184 0.190322
+0.171467 -0.0443352 0.191435
+0.124137 -0.0275222 0.21069
+0.115004 -0.0324307 0.220668
+0.103488 -0.0235525 0.230568
+-0.234966 -0.250492 -0.00657503
+0.230136 -0.0629922 0.010065
+0.22781 -0.0439028 0.0052695
+0.226758 -0.0758146 -0.00318988
+0.218119 -0.0755566 -0.0185995
+0.210925 -0.0881128 -0.0299059
+0.195391 -0.0835698 -0.0397083
+0.187049 -0.0970928 -0.0513544
+0.215909 -0.10623 -0.0275177
+0.221663 -0.111792 -0.00835326
+0.21689 -0.135734 -0.006698
+0.187289 -0.0668273 -0.0361071
+0.170089 -0.0549841 -0.0384156
+0.158135 -0.0292105 -0.0224101
+0.144698 -0.0631652 -0.0561794
+0.210349 -0.156804 0.00619425
+0.122465 -0.0210506 -0.020971
+0.25102 0.0827579 -0.00901225
+0.246076 0.0717907 -0.00732438
+0.248146 0.0653429 0.00439784
+0.245544 0.0538898 0.0151974
+0.255748 0.0868229 -0.000826293
+-0.125725 -0.258433 -0.170214
+0.151089 -0.0268375 0.140451
+0.244155 0.0131187 -0.0533056
+0.246127 0.0105937 -0.0612737
+0.239403 0.00492409 -0.0645919
+0.228547 -0.00190445 -0.0800819
+0.236042 0.000460551 -0.073323
+0.244455 -0.00400961 -0.0816292
+0.225173 0.00380285 -0.0703884
+0.247885 -0.00234363 -0.0902777
+0.247838 0.00379159 -0.0839495
+0.243353 0.0148187 -0.0838177
+0.236031 0.0244971 -0.0791546
+0.225616 0.0291453 -0.0875646
+-0.29518 -0.390079 -0.254006
+-0.303646 -0.394416 -0.248506
+-0.302477 -0.403327 -0.233435
+-0.298023 -0.412267 -0.217588
+-0.29425 -0.38151 -0.250683
+-0.302074 -0.390767 -0.254186
+0.191342 0.435898 0.0636941
+-0.0964353 -0.460913 -0.0144457
+0.137532 -0.0116873 0.139128
+0.343547 -0.294973 -0.292923
+0.137424 0.0947211 0.0118894
+0.147115 0.0902563 -0.00271409
+0.147874 0.0802052 -0.0226835
+0.255027 -0.310713 -0.257667
+0.257498 -0.300887 -0.266482
+0.220835 -0.0105204 -0.15317
+0.210403 -0.014095 -0.156518
+0.204562 -0.0228168 -0.148776
+-0.117774 -0.20207 0.202016
+3  575 1215 1225
+3  902 137 1166
+3  2122 17 2125
+3  1333 1332 328
+3  1031 1037 1032
+3  1235 1234 736
+3  986 1231 182
+3  532 534 185
+3  534 448 185
+3  1588 1570 1587
+3  674 675 639
+3  1130 1225 242
+3  279 6 280
+3  6 283 280
+3  621 954 955
+3  235 273 275
+3  2565 1747 2566
+3  2225 2224 2223
+3  11 206 278
+3  922 93 2080
+3  530 2723 2724
+3  2144 2146 2139
+3  1374 1376 766
+3  1361 1374 766
+3  1775 525 1893
+3  2626 1649 2625
+3  2338 2265 2339
+3  992 96 990
+3  103 21 104
+3  1681 1682 31
+3  1686 1682 1681
+3  2064 2065 295
+3  747 748 464
+3  597 1084 1086
+3  1003 2191 2190
+3  2184 1003 2190
+3  214 38 215
+3  2202 2205 473
+3  2725 257 2728
+3  216 554 944
+3  554 553 944
+3  1457 1456 445
+3  986 2154 96
+3  458 1022 1021
+3  961 1949 2534
+3  625 1514 1515
+3  1786 1791 518
+3  1586 2773 2006
+3  52 1485 1486
+3  2101 52 1486
+3  2368 2374 1380
+3  2054 1499 74
+3  329 326 1762
+3  933 934 176
+3  527 569 705
+3  2004 247 2003
+3  414 1966 1967
+3  853 544 184
+3  1136 354 352
+3  1896 1895 134
+3  5 978 1108
+3  1344 1342 726
+3  80 27 1599
+3  1489 1683 175
+3  1952 1953 2619
+3  21 105 104
+3  21 103 29
+3  2180 959 1164
+3  288 2180 1164
+3  294 1940 1941
+3  1263 1865 443
+3  1755 1756 337
+3  696 1187 1185
+3  1186 696 1185
+3  1489 66 1683
+3  68 907 122
+3  260 1748 341
+3  1340 1341 451
+3  806 397 478
+3  1337 750 728
+3  1516 419 1515
+3  279 11 278
+3  277 279 278
+3  226 2120 2119
+3  258 259 76
+3  686 683 719
+3  1232 950 1230
+3  1597 1598 1978
+3  91 2073 977
+3  724 734 1233
+3  724 732 734
+3  47 281 280
+3  85 1982 2246
+3  207 154 64
+3  2000 1998 1999
+3  553 853 184
+3  76 261 1618
+3  259 261 76
+3  682 706 1504
+3  660 706 682
+3  296 17 297
+3  17 2122 297
+3  554 216 916
+3  381 383 382
+3  271 269 296
+3  269 17 296
+3  1082 656 1823
+3  115 199 196
+3  1056 1049 880
+3  1520 2574 2248
+3  212 550 238
+3  1980 1598 27
+3  364 72 365
+3  674 657 673
+3  82 168 158
+3  1133 641 1171
+3  440 1406 1411
+3  804 803 398
+3  178 2662 461
+3  159 2100 157
+3  1657 1659 1658
+3  1659 1280 1658
+3  1499 141 1498
+3  2144 2137 2146
+3  358 2137 2144
+3  567 534 531
+3  64 264 274
+3  137 365 380
+3  379 137 380
+3  117 903 902
+3  377 376 136
+3  88 2566 2567
+3  83 1299 1306
+3  1819 692 1822
+3  1642 584 710
+3  645 602 1808
+3  503 500 502
+3  1855 1263 54
+3  1706 1580 517
+3  1169 1069 1068
+3  42 1825 1828
+3  21 67 106
+3  81 80 84
+3  20 105 118
+3  20 156 105
+3  67 21 256
+3  1015 258 1016
+3  259 258 65
+3  649 890 1665
+3  2021 116 2020
+3  1146 402 1147
+3  126 1615 1618
+3  164 1894 163
+3  120 164 163
+3  2774 378 377
+3  1796 1893 525
+3  510 511 413
+3  1143 754 305
+3  1257 1024 1260
+3  866 1816 1815
+3  2685 1816 866
+3  574 1222 1221
+3  1673 1038 1672
+3  158 168 51
+3  793 1560 1561
+3  750 730 465
+3  729 730 750
+3  996 998 61
+3  280 281 11
+3  919 93 918
+3  590 2300 1322
+3  1305 1478 1482
+3  68 122 118
+3  238 537 555
+3  138 365 903
+3  1536 1562 1535
+3  264 1720 268
+3  2079 2080 93
+3  6 273 283
+3  2554 63 246
+3  236 317 319
+3  40 84 80
+3  84 40 104
+3  778 1357 1356
+3  1359 778 1356
+3  170 263 265
+3  1986 170 265
+3  21 106 105
+3  944 183 1740
+3  944 945 183
+3  1894 164 165
+3  67 249 106
+3  283 235 47
+3  63 2005 246
+3  314 235 275
+3  435 436 188
+3  1746 1747 345
+3  1161 2431 2432
+3  2459 594 1102
+3  1484 1305 1482
+3  2 1136 1137
+3  1062 2510 2511
+3  1811 219 1809
+3  1736 997 1732
+3  264 268 274
+3  439 1427 1247
+3  526 2722 530
+3  2722 2721 530
+3  118 105 106
+3  164 120 121
+3  71 131 165
+3  63 247 2005
+3  1894 165 1895
+3  2054 2052 2053
+3  2285 2432 2286
+3  254 106 249
+3  254 68 106
+3  685 661 670
+3  1498 75 2135
+3  989 180 991
+3  263 24 265
+3  2021 2020 932
+3  1724 170 1487
+3  124 1016 76
+3  277 278 64
+3  431 151 430
+3  2225 2757 2189
+3  818 407 478
+3  1499 237 74
+3  1994 1576 2611
+3  2697 904 897
+3  904 899 897
+3  377 378 135
+3  132 72 364
+3  282 109 293
+3  109 282 319
+3  224 83 1306
+3  74 381 376
+3  2054 2053 141
+3  2226 2678 2224
+3  1134 1438 1437
+3  412 814 813
+3  2624 1641 709
+3  126 1618 261
+3  999 34 991
+3  899 1991 1989
+3  98 1315 275
+3  928 1767 931
+3  144 2131 2136
+3  23 234 78
+3  603 800 707
+3  337 1490 332
+3  1490 1491 332
+3  1348 1341 727
+3  1591 16 1590
+3  547 92 549
+3  2194 2226 2189
+3  2236 2582 2581
+3  378 72 135
+3  1796 1784 1893
+3  278 206 207
+3  170 82 1487
+3  265 24 266
+3  308 153 943
+3  310 308 943
+3  569 565 177
+3  267 1987 1502
+3  236 316 317
+3  1895 165 134
+3  1898 2388 1479
+3  163 159 120
+3  1464 1468 1463
+3  756 903 117
+3  138 903 756
+3  951 1231 986
+3  987 951 986
+3  2670 700 2672
+3  445 187 446
+3  1722 154 910
+3  256 21 29
+3  84 168 81
+3  1283 2386 2684
+3  2736 79 852
+3  146 363 2143
+3  1323 1324 2271
+3  68 118 106
+3  1043 1038 1673
+3  2773 1585 1705
+3  1231 1232 1230
+3  1337 729 750
+3  922 917 93
+3  825 395 824
+3  510 389 511
+3  763 1188 2128
+3  833 831 487
+3  560 1355 179
+3  560 1354 1355
+3  899 904 1991
+3  639 1133 1138
+3  674 639 1138
+3  1250 1243 1254
+3  2715 2714 1433
+3  502 499 503
+3  1690 1213 1212
+3  1342 451 1341
+3  1397 1025 443
+3  929 928 930
+3  1855 1854 1262
+3  1854 1857 1262
+3  2733 2736 241
+3  1001 987 993
+3  2662 2708 191
+3  1470 2764 1459
+3  264 1717 1720
+3  264 1718 1717
+3  346 330 1139
+3  1137 1136 353
+3  1514 557 1515
+3  2045 115 2026
+3  143 1494 1495
+3  1487 1713 1724
+3  211 974 975
+3  319 318 109
+3  319 317 318
+3  2081 2077 209
+3  1015 123 258
+3  207 59 910
+3  1770 241 848
+3  857 222 1810
+3  546 857 1810
+3  87 343 1750
+3  621 915 622
+3  393 803 806
+3  977 976 975
+3  2016 992 934
+3  992 176 934
+3  1794 2744 2745
+3  278 207 64
+3  524 1692 1691
+3  2772 1702 2771
+3  1702 2772 1586
+3  1107 1105 1106
+3  190 2593 2594
+3  802 393 586
+3  1777 1783 1793
+3  476 816 390
+3  2159 1049 1048
+3  516 515 1576
+3  321 2057 2063
+3  476 477 509
+3  1180 773 1181
+3  553 184 945
+3  1227 1228 949
+3  1892 255 101
+3  1919 600 2258
+3  124 1014 1015
+3  273 98 275
+3  225 1612 1613
+3  48 313 314
+3  539 2728 2729
+3  2728 539 538
+3  1897 1896 468
+3  1810 1809 980
+3  794 2238 2237
+3  307 39 311
+3  39 307 320
+3  1531 1533 1538
+3  1583 1702 1582
+3  410 954 621
+3  2519 2333 2332
+3  541 854 855
+3  459 439 1249
+3  529 1511 2740
+3  1259 450 1258
+3  1677 2170 2175
+3  509 389 510
+3  1704 1703 1707
+3  1703 1583 1707
+3  1644 1645 584
+3  463 562 2660
+3  175 929 930
+3  262 1756 1755
+3  1756 262 1757
+3  380 72 378
+3  365 72 380
+3  128 2247 1985
+3  2532 135 2533
+3  128 1985 1987
+3  1121 2187 288
+3  891 1663 1377
+3  135 132 2533
+3  76 1016 258
+3  117 902 901
+3  132 135 72
+3  908 139 909
+3  125 76 1618
+3  76 125 124
+3  737 736 745
+3  30 541 543
+3  30 540 541
+3  395 826 824
+3  260 1751 1749
+3  341 1746 1753
+3  173 553 554
+3  1141 331 1139
+3  1756 1490 337
+3  2668 2667 700
+3  1751 1752 259
+3  230 1380 2377
+3  281 282 12
+3  2137 2138 2146
+3  1689 1852 1850
+3  588 1697 1083
+3  1069 1169 641
+3  1149 424 1088
+3  11 279 280
+3  919 1331 213
+3  1626 715 3
+3  2374 2368 2367
+3  178 2661 2707
+3  2094 2093 2092
+3  320 318 39
+3  318 317 39
+3  754 14 755
+3  6 272 273
+3  2172 1047 1051
+3  1058 2163 2162
+3  237 1499 142
+3  1799 523 1518
+3  591 695 781
+3  696 695 2128
+3  234 23 430
+3  800 720 683
+3  236 282 47
+3  235 236 47
+3  23 1162 2312
+3  2192 2678 2227
+3  2681 2680 832
+3  236 319 282
+3  817 390 808
+3  509 510 476
+3  1661 1474 492
+3  1278 1659 1657
+3  191 2715 2716
+3  1138 1133 1171
+3  835 840 412
+3  840 1184 412
+3  1898 1897 468
+3  138 756 754
+3  22 2069 300
+3  309 755 1018
+3  1651 586 589
+3  1147 1651 589
+3  1771 2733 241
+3  2733 1771 522
+3  379 380 378
+3  1749 1751 65
+3  1729 1728 197
+3  2092 2093 970
+3  300 2069 2068
+3  903 137 902
+3  25 332 1002
+3  236 235 315
+3  1162 78 1027
+3  1163 318 320
+3  147 429 801
+3  360 359 302
+3  2285 1172 3
+3  637 669 677
+3  754 1143 14
+3  2706 2709 418
+3  1213 575 574
+3  1215 575 1213
+3  2186 2223 2192
+3  2364 718 2365
+3  535 184 544
+3  610 1197 394
+3  1401 2331 2444
+3  839 149 837
+3  956 149 839
+3  181 330 347
+3  674 1138 657
+3  340 326 329
+3  1430 1415 1635
+3  2664 1822 692
+3  173 554 924
+3  944 553 945
+3  535 544 533
+3  1740 216 944
+3  334 90 336
+3  2475 783 2480
+3  927 1767 929
+3  1267 310 943
+3  2479 952 783
+3  1144 592 1146
+3  929 175 927
+3  323 1731 1730
+3  2280 2292 2430
+3  1811 1813 1812
+3  1733 1735 2570
+3  222 857 858
+3  346 1139 1760
+3  809 808 390
+3  292 1073 1072
+3  1113 1959 1962
+3  1765 1764 1116
+3  2285 2286 1172
+3  1961 1012 287
+3  1754 77 1752
+3  509 820 844
+3  417 839 2710
+3  1260 1024 1252
+3  1463 1468 483
+3  2734 2732 537
+3  568 742 734
+3  2060 2059 298
+3  187 1137 353
+3  1438 1414 441
+3  469 133 753
+3  901 902 360
+3  902 1166 360
+3  592 1144 811
+3  151 234 430
+3  2321 715 2322
+3  1648 1643 1642
+3  498 786 2606
+3  1501 2696 146
+3  2699 145 898
+3  373 1970 1273
+3  1229 985 182
+3  384 237 142
+3  1025 1263 443
+3  1273 1275 1282
+3  900 1989 1193
+3  381 237 383
+3  839 837 415
+3  755 753 754
+3  138 754 753
+3  2719 437 1701
+3  789 1562 1536
+3  2604 2607 1573
+3  820 509 477
+3  780 2484 2485
+3  1184 840 416
+3  2717 1433 2713
+3  437 2717 2713
+3  1216 1219 1217
+3  394 1195 612
+3  238 550 551
+3  1206 529 1207
+3  1442 1472 2637
+3  1472 190 2637
+3  1778 1794 2745
+3  1082 1078 656
+3  1709 2266 2265
+3  498 497 1
+3  1235 735 1234
+3  526 527 705
+3  624 620 558
+3  393 589 586
+3  2128 1188 696
+3  501 497 498
+3  616 1204 1205
+3  825 514 798
+3  293 2063 294
+3  215 343 87
+3  853 854 544
+3  854 853 542
+3  2769 1060 1129
+3  212 547 548
+3  121 479 119
+3  784 620 411
+3  1320 610 1321
+3  419 618 624
+3  1518 2740 528
+3  410 953 954
+3  957 2709 2705
+3  1662 1470 1661
+3  619 2484 1174
+3  2672 708 2670
+3  893 771 887
+3  1418 2622 1700
+3  2306 2307 428
+3  1348 1342 1341
+3  527 528 1355
+3  455 436 454
+3  1704 1705 1585
+3  1703 1704 1585
+3  1900 848 1901
+3  1259 1257 455
+3  401 1323 1646
+3  422 2250 1553
+3  2250 422 1524
+3  668 666 671
+3  1469 482 481
+3  1449 1662 1661
+3  622 912 410
+3  844 2681 832
+3  454 1630 453
+3  463 748 747
+3  467 1433 2717
+3  735 1021 733
+3  909 2051 140
+3  1216 524 1219
+3  840 956 416
+3  408 811 1144
+3  739 740 738
+3  449 446 447
+3  2731 536 2726
+3  576 1223 1221
+3  461 2716 192
+3  598 2665 2664
+3  1144 1146 589
+3  627 1392 1393
+3  569 177 705
+3  1390 1391 651
+3  1799 1518 240
+3  1089 606 626
+3  1776 1220 523
+3  462 461 192
+3  1505 561 560
+3  192 459 456
+3  701 581 704
+3  673 676 636
+3  1022 1349 480
+3  2722 177 2721
+3  462 178 461
+3  611 1358 1357
+3  625 624 558
+3  2322 1621 2321
+3  1630 1460 386
+3  2727 537 2726
+3  748 178 462
+3  480 733 1021
+3  983 1543 1545
+3  531 534 532
+3  778 611 1357
+3  2607 2604 2605
+3  1463 485 148
+3  567 565 566
+3  748 462 464
+3  412 816 834
+3  743 746 464
+3  566 534 567
+3  606 663 1808
+3  1770 1771 241
+3  834 835 412
+3  420 798 514
+3  1454 1662 1449
+3  2306 428 2309
+3  763 2128 591
+3  625 558 1513
+3  477 476 390
+3  817 477 390
+3  1526 1523 1528
+3  1562 1521 791
+3  551 46 552
+3  700 2670 2669
+3  497 501 518
+3  790 1562 1552
+3  1212 574 1214
+3  1588 1587 1571
+3  2736 849 241
+3  2039 2024 2038
+3  123 128 258
+3  128 123 965
+3  1726 1727 582
+3  550 46 551
+3  274 2742 64
+3  1779 1781 2755
+3  1781 1782 2755
+3  503 506 500
+3  709 1641 581
+3  1563 1589 1587
+3  505 498 515
+3  2238 2239 392
+3  807 2243 2241
+3  818 477 817
+3  1589 1565 507
+3  1736 325 1737
+3  1530 1531 1525
+3  872 1052 1051
+3  1800 2753 2752
+3  534 186 448
+3  603 1069 721
+3  2665 653 687
+3  1513 559 1512
+3  1782 520 571
+3  2473 1727 399
+3  2748 2750 2747
+3  240 1773 1799
+3  1773 1774 1799
+3  753 752 138
+3  1 497 851
+3  1070 627 1393
+3  1071 627 1070
+3  744 743 464
+3  803 397 806
+3  1086 1084 1085
+3  402 1086 1085
+3  1020 458 1021
+3  854 540 544
+3  540 533 544
+3  811 810 174
+3  729 562 563
+3  1351 562 729
+3  731 749 568
+3  749 746 568
+3  1338 564 1350
+3  1337 1338 1350
+3  455 1257 54
+3  1198 2486 426
+3  817 2244 818
+3  239 530 536
+3  242 1225 1216
+3  1582 499 938
+3  50 737 745
+3  735 724 1234
+3  540 539 533
+3  539 540 30
+3  901 1990 900
+3  223 541 855
+3  541 540 854
+3  715 1620 3
+3  212 548 550
+3  545 543 541
+3  223 545 541
+3  2456 2455 1680
+3  545 546 547
+3  223 546 545
+3  1163 110 2066
+3  212 545 547
+3  945 946 183
+3  549 548 547
+3  2748 2747 2749
+3  217 917 920
+3  173 924 925
+3  855 542 856
+3  855 854 542
+3  46 203 552
+3  1081 95 662
+3  173 542 553
+3  149 956 840
+3  1505 560 1507
+3  742 50 745
+3  2749 1710 2751
+3  561 562 452
+3  179 1509 1508
+3  1507 560 179
+3  2574 150 2248
+3  665 638 693
+3  571 1778 2754
+3  727 751 480
+3  705 177 2722
+3  885 882 765
+3  177 565 567
+3  402 1326 587
+3  556 1516 1509
+3  1690 1691 1213
+3  742 745 734
+3  725 1235 736
+3  1374 769 1375
+3  1361 769 1374
+3  1343 566 565
+3  451 1343 565
+3  707 1066 603
+3  707 632 1066
+3  1076 656 1078
+3  584 1643 1644
+3  705 2722 526
+3  1326 402 1085
+3  1148 1086 402
+3  598 652 2665
+3  2636 2635 577
+3  773 611 914
+3  611 773 772
+3  914 622 915
+3  1925 1924 757
+3  680 697 605
+3  663 646 645
+3  732 724 733
+3  833 490 831
+3  677 669 636
+3  618 617 1182
+3  617 618 419
+3  844 490 509
+3  697 680 635
+3  566 713 186
+3  1070 595 1071
+3  1825 107 57
+3  814 409 813
+3  636 672 673
+3  2349 2348 1203
+3  900 117 901
+3  1206 573 1205
+3  658 679 675
+3  2653 1821 596
+3  410 621 622
+3  2738 2737 1207
+3  649 1665 891
+3  620 0 621
+3  680 678 635
+3  674 658 675
+3  598 2664 2666
+3  1807 602 1806
+3  698 710 584
+3  710 698 1096
+3  395 798 828
+3  1365 1372 1364
+3  1371 1372 1365
+3  409 2479 827
+3  1321 610 609
+3  1151 616 1223
+3  2177 2176 1044
+3  0 915 621
+3  2664 2665 687
+3  2672 2671 580
+3  733 724 735
+3  2737 1211 2741
+3  1346 453 1345
+3  2351 1209 1204
+3  845 2676 833
+3  2323 715 1625
+3  1151 1150 616
+3  2352 2344 1201
+3  2273 2274 588
+3  615 1516 556
+3  557 1516 1515
+3  744 50 743
+3  1145 1144 589
+3  1181 1182 619
+3  1182 1181 618
+3  2546 2545 1626
+3  620 621 411
+3  1374 1375 770
+3  405 1356 1320
+3  1359 1356 405
+3  145 2144 2139
+3  1353 452 1352
+3  1469 2141 355
+3  137 379 1165
+3  784 558 620
+3  670 693 638
+3  668 671 672
+3  2753 1779 2754
+3  1069 722 721
+3  654 1187 1188
+3  637 678 680
+3  1687 1688 660
+3  813 390 816
+3  671 667 658
+3  667 679 658
+3  2350 1195 613
+3  607 646 663
+3  665 595 666
+3  638 666 668
+3  711 1096 1095
+3  769 1361 1358
+3  758 1924 1925
+3  597 1086 1087
+3  1989 900 1990
+3  1183 1184 416
+3  708 581 701
+3  1420 1407 2330
+3  637 680 685
+3  672 671 658
+3  669 670 638
+3  636 669 668
+3  669 638 668
+3  637 670 669
+3  1919 1921 600
+3  816 412 813
+3  693 628 665
+3  1082 1077 1078
+3  636 668 672
+3  666 595 667
+3  1641 1642 710
+3  884 648 881
+3  1688 707 660
+3  637 685 670
+3  145 359 2144
+3  1908 1907 895
+3  706 660 707
+3  1861 1247 1860
+3  731 568 734
+3  721 722 642
+3  606 1089 1081
+3  1352 452 1351
+3  2152 682 1504
+3  857 546 223
+3  717 1627 2545
+3  1780 1797 2750
+3  747 746 563
+3  881 694 882
+3  1133 639 1132
+3  489 829 2680
+3  1548 1537 1564
+3  965 964 252
+3  490 844 832
+3  1175 1176 1180
+3  1557 1559 793
+3  2547 2542 1628
+3  1726 1639 2009
+3  992 990 176
+3  859 222 858
+3  1667 222 859
+3  732 465 731
+3  465 730 731
+3  566 186 534
+3  357 2130 2129
+3  465 732 733
+3  1366 768 1368
+3  746 749 563
+3  731 734 732
+3  743 568 746
+3  568 743 742
+3  1579 516 1994
+3  878 2164 2165
+3  773 1176 772
+3  956 839 417
+3  50 742 743
+3  1233 736 1234
+3  751 733 480
+3  903 365 137
+3  465 733 751
+3  751 728 750
+3  465 751 750
+3  752 365 138
+3  1922 1923 758
+3  1389 2666 1392
+3  575 805 94
+3  881 762 694
+3  1442 2492 1134
+3  887 886 765
+3  406 776 1371
+3  954 417 955
+3  590 1322 1323
+3  2661 178 2659
+3  137 1165 1166
+3  773 1180 1176
+3  622 914 911
+3  439 467 1426
+3  782 591 781
+3  765 893 887
+3  413 835 834
+3  809 813 409
+3  805 823 397
+3  1356 777 1320
+3  1020 1236 1237
+3  793 1558 1557
+3  2686 2685 866
+3  965 123 964
+3  703 1096 1097
+3  2317 1019 2316
+3  1557 421 1559
+3  286 2179 232
+3  1676 2176 2177
+3  354 2133 352
+3  393 802 803
+3  407 806 478
+3  954 956 417
+3  777 2488 1320
+3  1212 1211 244
+3  2238 2580 2237
+3  813 809 390
+3  1184 1183 814
+3  835 149 840
+3  478 821 819
+3  392 2579 2580
+3  1127 2481 953
+3  819 821 820
+3  818 819 477
+3  818 478 819
+3  805 396 823
+3  819 820 477
+3  1211 573 2741
+3  397 823 478
+3  395 825 798
+3  2236 829 797
+3  831 392 487
+3  2681 489 2680
+3  798 829 828
+3  956 953 416
+3  953 956 954
+3  2128 695 591
+3  1184 814 412
+3  1943 2576 1122
+3  487 846 845
+3  1904 1903 849
+3  542 853 553
+3  16 2608 1593
+3  1048 2160 2159
+3  1586 2772 2773
+3  856 223 855
+3  978 867 1108
+3  2194 2189 2758
+3  239 536 2731
+3  2295 2296 865
+3  2170 2166 1046
+3  1138 1171 657
+3  873 1055 1035
+3  2046 2045 936
+3  2134 2145 142
+3  1167 44 360
+3  875 2296 2770
+3  2165 2164 2167
+3  1051 1052 1034
+3  914 915 773
+3  306 14 1143
+3  623 913 912
+3  2695 2694 301
+3  2353 2354 614
+3  694 762 1920
+3  1906 1907 1367
+3  1142 304 1143
+3  906 117 900
+3  756 117 906
+3  931 1767 1766
+3  754 756 906
+3  214 910 59
+3  911 912 622
+3  1129 870 1128
+3  77 337 1126
+3  917 217 916
+3  917 916 918
+3  1001 998 951
+3  1392 1818 1393
+3  1739 1740 183
+3  1075 1124 1125
+3  1414 440 1412
+3  964 963 252
+3  1494 1496 1495
+3  2708 2662 2707
+3  1881 1882 2437
+3  396 2766 822
+3  823 396 822
+3  2711 415 2708
+3  417 2710 957
+3  1008 473 1005
+3  289 1073 960
+3  71 130 131
+3  2223 1004 2222
+3  2050 339 2051
+3  1625 715 1626
+3  2520 2333 2519
+3  2407 2197 2408
+3  2197 2407 2198
+3  1226 2334 948
+3  2678 2192 2224
+3  2433 2305 2302
+3  124 1015 1016
+3  248 255 60
+3  473 1003 1005
+3  2191 1003 473
+3  2348 613 1202
+3  2036 993 2035
+3  123 907 964
+3  907 123 1015
+3  1207 2739 2738
+3  1242 1838 1839
+3  926 1681 1680
+3  1109 865 1110
+3  868 1109 1110
+3  2320 1621 2319
+3  1037 1038 1043
+3  2768 2767 1130
+3  1986 265 86
+3  1984 1986 86
+3  2107 1298 2108
+3  2377 2375 1271
+3  1380 2375 2377
+3  2195 2229 2214
+3  1294 1153 1289
+3  995 61 994
+3  1253 1856 1689
+3  2113 1299 229
+3  2423 889 2421
+3  1274 1273 1269
+3  373 1273 1274
+3  1610 1611 1154
+3  1610 471 1611
+3  2120 226 2121
+3  2535 1524 1526
+3  1375 894 770
+3  2072 204 2073
+3  2601 2604 2602
+3  2601 1575 2604
+3  981 980 979
+3  981 979 221
+3  922 425 920
+3  425 922 970
+3  2142 2145 2134
+3  2650 2145 2142
+3  970 922 220
+3  113 2062 2060
+3  2451 921 1996
+3  1996 217 2451
+3  217 920 2451
+3  57 1826 1825
+3  2168 2171 2169
+3  208 2081 214
+3  770 2258 600
+3  2258 770 883
+3  2085 975 974
+3  549 974 211
+3  335 1888 1887
+3  1334 2084 213
+3  976 548 211
+3  2079 2078 210
+3  1755 337 1754
+3  1332 213 1331
+3  869 1672 1669
+3  281 58 11
+3  58 281 1938
+3  1815 1816 219
+3  1816 979 219
+3  1812 1815 219
+3  998 1232 951
+3  1997 1996 1995
+3  2094 2096 2093
+3  2096 2094 221
+3  2098 978 971
+3  978 973 971
+3  2093 2096 971
+3  2096 2098 971
+3  2098 2096 2097
+3  1619 547 546
+3  1996 921 1995
+3  2096 221 2097
+3  1821 601 1820
+3  1769 1577 1768
+3  1581 1769 1768
+3  925 924 1997
+3  984 316 233
+3  303 2692 2068
+3  2692 300 2068
+3  2129 2130 2147
+3  1529 1525 792
+3  1530 1529 1523
+3  729 563 730
+3  2728 257 2729
+3  2673 699 2668
+3  699 2667 2668
+3  1541 1540 55
+3  1536 1540 1541
+3  1533 1531 1530
+3  348 1971 1998
+3  1523 1532 1530
+3  2250 2251 1553
+3  2251 2250 796
+3  676 657 1632
+3  1633 676 1632
+3  1527 422 983
+3  2248 150 2253
+3  2249 2248 2253
+3  2693 2689 2694
+3  2693 2691 2689
+3  22 298 299
+3  2656 697 635
+3  787 2656 635
+3  2599 1972 2597
+3  1972 1973 2597
+3  316 39 317
+3  316 984 39
+3  984 311 39
+3  1393 1818 601
+3  37 195 1729
+3  2052 2054 140
+3  1120 2218 1949
+3  989 990 96
+3  990 989 991
+3  1684 9 1682
+3  998 1001 61
+3  66 1686 1683
+3  990 930 176
+3  930 990 34
+3  930 933 176
+3  298 2059 295
+3  444 1460 1461
+3  444 2743 1460
+3  195 996 995
+3  2158 1049 2159
+3  1677 1676 1045
+3  985 180 989
+3  180 985 423
+3  333 89 1140
+3  333 1492 89
+3  695 1186 2477
+3  1186 174 2477
+3  2062 2061 2060
+3  991 34 990
+3  1699 1063 1669
+3  930 928 933
+3  33 347 346
+3  33 1000 347
+3  1030 1032 860
+3  1030 2515 1032
+3  1886 2640 2649
+3  2640 1885 2649
+3  1161 2432 2285
+3  993 987 988
+3  987 96 988
+3  143 1491 1492
+3  1493 143 1492
+3  1493 334 336
+3  734 745 1233
+3  38 1334 344
+3  927 926 1685
+3  1867 1265 1864
+3  1679 4 202
+3  2050 127 1614
+3  127 1616 1614
+3  922 920 917
+3  195 37 996
+3  1731 1736 1732
+3  1757 329 1760
+3  351 34 999
+3  327 351 999
+3  774 1179 1369
+3  1233 745 736
+3  1491 1002 332
+3  2565 88 2569
+3  323 197 99
+3  996 997 998
+3  999 991 180
+3  1000 999 180
+3  1000 327 999
+3  2140 356 2141
+3  1743 1742 324
+3  1000 180 423
+3  347 1000 423
+3  33 327 1000
+3  987 1001 951
+3  351 99 197
+3  1029 864 1030
+3  2773 1706 2006
+3  2605 2606 1574
+3  960 292 1117
+3  1073 292 960
+3  1959 1958 1114
+3  2272 1326 2274
+3  2002 249 248
+3  249 67 248
+3  2201 1006 2203
+3  2200 2210 2211
+3  1939 1935 1936
+3  1939 112 1935
+3  1533 982 1534
+3  1538 1533 1534
+3  2206 2203 2204
+3  70 124 125
+3  1014 124 70
+3  2028 994 2042
+3  2041 2028 2042
+3  2192 1005 2186
+3  1962 1960 1963
+3  1965 350 1960
+3  2187 961 2188
+3  77 1395 261
+3  41 2000 1999
+3  1014 907 1015
+3  122 907 1014
+3  70 122 1014
+3  350 1114 2213
+3  2209 2200 2204
+3  2209 2204 1006
+3  2668 700 2669
+3  141 2053 2056
+3  2321 2320 1620
+3  2321 1621 2320
+3  1010 2213 2208
+3  2299 1321 1322
+3  405 1321 2299
+3  706 707 683
+3  2301 1157 1602
+3  831 490 2679
+3  490 832 2679
+3  287 1012 1011
+3  1010 2206 2204
+3  1013 2191 473
+3  2205 1013 473
+3  2561 166 2560
+3  2201 2202 1008
+3  2203 2202 2201
+3  2369 2368 2370
+3  1896 1897 1895
+3  102 40 107
+3  2394 1283 53
+3  1283 2394 2390
+3  985 2154 986
+3  2076 2072 2073
+3  2076 208 2072
+3  308 310 1018
+3  755 308 1018
+3  308 755 14
+3  1848 1849 1251
+3  2345 1200 2346
+3  737 740 1238
+3  740 737 738
+3  1020 1021 735
+3  738 737 50
+3  50 744 738
+3  454 1259 455
+3  1337 1350 729
+3  480 1021 1022
+3  738 744 741
+3  457 738 741
+3  1841 1840 193
+3  1840 1841 1242
+3  1693 1691 1519
+3  1025 442 188
+3  1257 1256 1024
+3  453 450 1259
+3  928 931 2022
+3  931 932 2022
+3  2055 2056 2053
+3  2056 2055 1495
+3  336 1497 1496
+3  2701 1678 1673
+3  1672 2701 1673
+3  436 455 1025
+3  188 436 1025
+3  459 467 439
+3  702 1123 1093
+3  1026 842 843
+3  1093 1123 841
+3  1945 2578 1943
+3  2684 2384 371
+3  2384 2684 2386
+3  1526 1524 1527
+3  2537 2538 1173
+3  2252 796 2235
+3  1076 1125 656
+3  691 2316 2315
+3  704 842 1026
+3  342 1747 1748
+3  1747 341 1748
+3  1163 320 110
+3  190 2639 2637
+3  2065 2064 2066
+3  2079 93 919
+3  2078 2077 2082
+3  2077 91 2082
+3  1231 1230 182
+3  1678 2178 1674
+3  1048 2168 2167
+3  1676 2177 2178
+3  2177 1044 2178
+3  2500 2508 1041
+3  190 2594 2595
+3  377 136 2774
+3  874 1033 1032
+3  1710 2265 2337
+3  2336 1710 2337
+3  2158 1057 2157
+3  1057 2163 2157
+3  1236 735 1235
+3  1815 1812 1814
+3  499 1578 503
+3  2315 2316 714
+3  1620 2315 714
+3  382 1165 379
+3  1033 873 1035
+3  1033 1034 873
+3  1238 725 737
+3  861 1030 860
+3  1036 861 860
+3  1035 1036 860
+3  1938 2490 58
+3  861 1029 1030
+3  1285 2720 1488
+3  1801 1286 1274
+3  367 1801 1274
+3  937 2013 2021
+3  1673 1674 1043
+3  1415 1430 2622
+3  1430 1419 2622
+3  2161 1057 2160
+3  1037 871 1038
+3  28 1599 1597
+3  1830 28 1597
+3  2025 2044 2043
+3  2161 2162 2163
+3  2173 2172 1050
+3  2172 2173 879
+3  1095 703 2502
+3  1169 604 1170
+3  1169 1065 604
+3  971 425 970
+3  658 674 673
+3  788 604 1065
+3  631 788 1065
+3  659 2153 1019
+3  2153 2318 1019
+3  425 971 973
+3  1059 2155 2770
+3  868 1105 1107
+3  1109 868 1107
+3  35 2028 2027
+3  2028 198 2027
+3  2132 356 2131
+3  331 1756 1757
+3  2014 2032 2031
+3  2032 2015 2031
+3  1047 2171 2168
+3  1046 2166 2165
+3  1048 1049 872
+3  1834 1759 262
+3  873 1053 1055
+3  1715 155 1714
+3  2509 2508 1040
+3  2731 2726 537
+3  1705 1704 1584
+3  616 2349 1203
+3  238 543 212
+3  1047 872 1051
+3  2171 2172 879
+3  1050 1051 1034
+3  1708 1707 572
+3  2090 2089 967
+3  1681 31 1680
+3  2513 1031 2514
+3  1031 2515 2514
+3  2508 2500 1061
+3  2500 2501 1061
+3  1033 874 1034
+3  1034 1052 873
+3  1052 1053 873
+3  1052 1056 1053
+3  1055 863 1035
+3  1032 1033 860
+3  1033 1035 860
+3  1054 863 1055
+3  1049 2295 880
+3  2295 1049 876
+3  1967 1444 2491
+3  1056 1052 872
+3  1706 508 2268
+3  1580 1706 2268
+3  1814 863 1054
+3  1788 1790 1789
+3  1056 880 1053
+3  2016 2017 2033
+3  1054 1053 880
+3  2194 1011 2193
+3  2095 969 967
+3  2091 2092 970
+3  2091 969 2092
+3  2618 2616 2608
+3  2618 2607 2616
+3  2167 1046 2165
+3  2162 2161 878
+3  2644 2643 9
+3  2025 2038 2037
+3  2008 2257 2258
+3  883 2008 2258
+3  1004 2185 2187
+3  673 672 658
+3  940 1979 1976
+3  199 1886 196
+3  2769 875 2770
+3  1106 864 1029
+3  923 1106 1029
+3  2506 2508 1061
+3  640 1171 1170
+3  1171 641 1170
+3  2224 2192 2223
+3  2770 2155 1060
+3  2155 870 1060
+3  240 528 527
+3  1518 528 240
+3  220 2088 2090
+3  2088 2089 2090
+3  2035 993 988
+3  2099 2085 974
+3  2564 1742 1745
+3  2165 2166 878
+3  2013 2046 936
+3  937 2046 2013
+3  1684 1686 66
+3  631 1065 1066
+3  1067 631 1066
+3  667 1067 632
+3  1067 1066 632
+3  595 1067 667
+3  1091 2468 2466
+3  1170 641 1169
+3  1066 1068 603
+3  1066 1065 1068
+3  1170 635 640
+3  604 635 1170
+3  1934 1932 1912
+3  1928 1927 758
+3  1067 595 1070
+3  1068 1065 1169
+3  471 1612 1611
+3  1612 225 1611
+3  1067 1070 631
+3  2399 271 2400
+3  665 664 1071
+3  664 665 628
+3  2125 2126 224
+3  102 103 40
+3  712 1075 2589
+3  2588 712 2589
+3  612 399 583
+3  399 612 1150
+3  1076 1078 1074
+3  673 657 676
+3  1084 597 1078
+3  1115 1765 1116
+3  291 1115 1116
+3  653 1083 1082
+3  687 653 1082
+3  656 1824 1823
+3  1077 1082 1083
+3  1075 1125 1076
+3  1094 1093 712
+3  711 710 1096
+3  2274 2273 2272
+3  1094 702 1093
+3  584 2011 698
+3  2011 584 1645
+3  1087 1080 597
+3  1080 1087 95
+3  593 1087 1086
+3  593 1088 1087
+3  1078 1077 1084
+3  1179 1174 406
+3  1911 1394 645
+3  1911 1926 1394
+3  690 2463 1102
+3  1076 1074 2589
+3  1822 1823 688
+3  173 1104 542
+3  1091 1090 2468
+3  1099 599 1091
+3  690 1099 1091
+3  1529 1528 1523
+3  594 1098 1099
+3  1077 1085 1084
+3  2570 325 1733
+3  597 1080 1079
+3  1963 1960 1112
+3  424 655 1088
+3  1079 1080 599
+3  1080 1081 599
+3  2423 2422 757
+3  2422 2423 2421
+3  404 2422 2421
+3  760 653 759
+3  760 1083 653
+3  48 314 275
+3  1149 1088 593
+3  2589 1075 1076
+3  2588 2589 1074
+3  1085 1077 1697
+3  1146 1148 402
+3  1146 592 1148
+3  1641 711 581
+3  711 2563 581
+3  591 782 1330
+3  1146 1147 589
+3  1147 402 587
+3  1087 1088 95
+3  1088 655 95
+3  655 662 95
+3  1820 601 1819
+3  2291 2281 2365
+3  2286 2291 2365
+3  1527 982 1594
+3  962 1124 1101
+3  1203 1204 616
+3  1093 841 1075
+3  1091 599 1090
+3  599 1089 1090
+3  2469 1090 2470
+3  1103 2461 1122
+3  2458 2461 1103
+3  399 2472 2473
+3  2472 399 1151
+3  2060 2061 26
+3  794 2237 2232
+3  599 1081 1089
+3  603 721 800
+3  721 720 800
+3  2117 2114 2115
+3  2466 2468 2467
+3  689 2466 2467
+3  2468 1090 2469
+3  2467 2468 2469
+3  626 2470 1089
+3  688 1092 689
+3  1820 689 1821
+3  1099 1079 599
+3  1126 332 338
+3  332 25 338
+3  706 686 1504
+3  683 686 706
+3  702 1094 1095
+3  962 1100 1092
+3  1078 1079 1098
+3  1079 1078 597
+3  2566 1747 342
+3  1074 1098 594
+3  1074 1078 1098
+3  594 1099 1102
+3  2562 2563 711
+3  841 1124 1075
+3  1098 1079 1099
+3  1460 1630 1461
+3  1760 329 1761
+3  249 2002 2763
+3  254 249 2763
+3  1628 1629 642
+3  1628 720 1629
+3  925 1104 173
+3  1823 1824 688
+3  1125 1124 962
+3  677 1633 1634
+3  1183 815 814
+3  1503 2282 2287
+3  2288 1503 2287
+3  2717 2718 1432
+3  856 542 1104
+3  858 856 1104
+3  858 857 856
+3  858 1104 925
+3  858 925 859
+3  594 2459 2460
+3  1102 1099 690
+3  2578 1026 2576
+3  1671 1039 870
+3  1107 1106 5
+3  1108 1107 5
+3  2277 2505 2501
+3  2277 2504 2505
+3  2505 1061 2501
+3  1672 1038 1699
+3  2367 370 1475
+3  2014 936 2032
+3  521 1791 1785
+3  1814 1054 866
+3  1815 1814 866
+3  1608 1607 470
+3  1607 1608 2687
+3  1154 2687 1608
+3  1108 867 1109
+3  1107 1108 1109
+3  1030 2514 2515
+3  1281 2262 1696
+3  2262 1281 2263
+3  1825 8 107
+3  1956 1955 1117
+3  2185 2184 288
+3  1003 2184 2185
+3  2224 2225 2189
+3  2226 2224 2189
+3  88 2565 2566
+3  185 448 1887
+3  1174 780 406
+3  286 232 285
+3  1834 1753 1746
+3  116 2021 2023
+3  2021 2013 2023
+3  1954 1963 1112
+3  1833 1832 1830
+3  620 624 618
+3  2760 1004 2188
+3  799 1524 2535
+3  2210 2200 2209
+3  1116 1119 291
+3  2416 1190 1189
+3  1521 2230 1525
+3  8 102 107
+3  1118 1953 1955
+3  1954 1953 1118
+3  1007 2196 2195
+3  1116 475 1613
+3  1113 1956 1957
+3  1952 1951 1120
+3  1950 290 1111
+3  882 2007 765
+3  1955 1964 1118
+3  1445 1444 1966
+3  1008 2202 473
+3  896 2699 2700
+3  683 707 800
+3  2242 2241 2243
+3  675 1687 681
+3  639 675 681
+3  1525 1529 1530
+3  2546 1626 3
+3  2461 841 1122
+3  912 913 1127
+3  912 1127 410
+3  953 410 1127
+3  810 827 174
+3  827 2476 174
+3  2481 2480 783
+3  2481 1127 2480
+3  845 846 2482
+3  2352 2353 2357
+3  1229 182 1228
+3  182 1230 1228
+3  1069 641 722
+3  641 1133 722
+3  2320 2319 691
+3  722 1132 1131
+3  1132 722 1133
+3  361 383 1167
+3  1902 391 1901
+3  431 147 432
+3  1229 181 423
+3  2316 2318 714
+3  2318 2316 1019
+3  1131 659 1019
+3  1131 1132 659
+3  2716 459 192
+3  1136 2 354
+3  461 2662 191
+3  536 2725 2726
+3  484 485 1462
+3  484 389 485
+3  973 2452 2450
+3  2452 921 2450
+3  2141 1891 355
+3  1440 1701 437
+3  1936 1935 13
+3  1409 2520 2519
+3  355 494 1453
+3  346 1760 1761
+3  329 1758 340
+3  1847 193 1845
+3  1846 1847 1845
+3  1856 1853 1689
+3  1247 1266 1860
+3  1395 1126 338
+3  153 366 943
+3  366 1267 943
+3  1798 525 1774
+3  2696 2690 2695
+3  363 2696 2695
+3  898 2695 301
+3  94 805 804
+3  1194 111 1142
+3  633 1805 1804
+3  1737 997 1736
+3  1517 1208 1518
+3  523 1517 1518
+3  715 2321 1620
+3  582 2474 2471
+3  582 2473 2474
+3  111 304 1142
+3  303 304 2692
+3  111 1194 905
+3  1498 2135 142
+3  2136 357 2137
+3  304 306 1143
+3  152 306 304
+3  408 810 811
+3  2476 2478 2475
+3  913 2476 2475
+3  593 1148 1149
+3  592 1149 1148
+3  808 408 2241
+3  1148 593 1086
+3  594 2460 1074
+3  2459 1103 2460
+3  1558 2231 1521
+3  810 408 809
+3  809 408 808
+3  2275 2270 2272
+3  886 885 765
+3  1149 592 812
+3  592 811 812
+3  2145 385 384
+3  142 2145 384
+3  1197 610 1198
+3  2517 1776 523
+3  2724 2723 257
+3  56 2267 1709
+3  471 2183 1612
+3  227 2112 2111
+3  1608 1609 1610
+3  470 1609 1608
+3  314 313 233
+3  313 312 233
+3  1310 1309 231
+3  276 1310 1297
+3  1292 2413 2415
+3  48 372 313
+3  471 1610 1609
+3  2387 2388 161
+3  2391 2387 161
+3  225 1603 1155
+3  321 2066 2064
+3  1163 2066 321
+3  2261 472 2260
+3  1606 1154 1155
+3  1603 1602 1155
+3  1083 1697 1077
+3  2263 1281 1287
+3  1281 228 1287
+3  1611 225 1155
+3  1310 231 1297
+3  1306 1299 2112
+3  1277 1288 1289
+3  1288 1294 1289
+3  1694 1695 1279
+3  1695 1694 1281
+3  643 661 1804
+3  1382 661 643
+3  1279 1275 1656
+3  1278 1654 1655
+3  1156 1660 1292
+3  1291 1652 1653
+3  1291 1277 1289
+3  228 1288 1287
+3  228 1293 1288
+3  1970 1275 1273
+3  2347 2348 2346
+3  2291 2432 2431
+3  2432 2291 2286
+3  2013 2366 2023
+3  366 153 312
+3  2372 366 312
+3  370 2368 2369
+3  472 2261 1119
+3  1951 2297 2218
+3  2217 2297 1950
+3  2297 2217 2218
+3  1365 1366 775
+3  1166 1167 360
+3  361 1167 1166
+3  2206 2207 1009
+3  277 6 279
+3  277 1168 6
+3  444 1459 2743
+3  268 97 274
+3  821 823 822
+3  1458 1460 2743
+3  1460 1458 1457
+3  1005 2227 2193
+3  2159 1057 2158
+3  1919 2258 2257
+3  2169 1046 2765
+3  720 721 1629
+3  657 1171 1632
+3  1627 2547 1628
+3  2466 2465 1091
+3  2466 1092 2465
+3  1523 1594 1532
+3  426 2485 1182
+3  617 426 1182
+3  1196 2356 1199
+3  2356 1196 1197
+3  1181 0 618
+3  0 620 618
+3  1180 619 1175
+3  741 192 457
+3  1250 1254 1244
+3  1176 774 1177
+3  773 915 1181
+3  1636 1405 1635
+3  619 1180 1181
+3  462 192 741
+3  646 1911 645
+3  1911 646 1912
+3  771 1367 888
+3  629 1387 1910
+3  1926 629 1910
+3  1371 775 406
+3  1179 406 775
+3  1369 1179 775
+3  1440 2592 1701
+3  2590 1437 1436
+3  712 1093 1075
+3  1176 1175 774
+3  2484 619 1182
+3  2485 2484 1182
+3  2486 2487 780
+3  779 780 2487
+3  780 779 406
+3  98 272 284
+3  272 2399 284
+3  424 1187 655
+3  38 2084 1334
+3  494 493 1450
+3  1192 654 1191
+3  799 2535 1522
+3  812 1185 424
+3  1149 812 424
+3  1808 663 645
+3  812 1186 1185
+3  812 174 1186
+3  811 174 812
+3  1404 1428 2521
+3  1666 1668 861
+3  1189 654 1188
+3  2417 1189 2419
+3  884 2423 648
+3  647 1913 2397
+3  655 1187 1192
+3  2426 890 2416
+3  654 1189 1190
+3  270 2402 2401
+3  1193 906 900
+3  271 2398 97
+3  2417 2416 1189
+3  2426 2416 2417
+3  1191 2397 607
+3  654 1190 1191
+3  305 906 1193
+3  906 305 754
+3  654 1192 1187
+3  1936 108 1939
+3  1192 662 655
+3  607 662 1192
+3  662 607 663
+3  2446 1027 2448
+3  1570 2587 1587
+3  2587 1563 1587
+3  647 1914 1913
+3  1142 1143 305
+3  1193 1142 305
+3  2595 2639 190
+3  1473 2639 2595
+3  322 905 1194
+3  1194 1142 1193
+3  1196 394 1197
+3  915 0 1181
+3  2676 486 2575
+3  615 556 400
+3  1206 1205 556
+3  529 1206 556
+3  1205 400 556
+3  2211 350 2213
+3  2211 1961 350
+3  577 2471 2472
+3  576 577 2472
+3  805 397 804
+3  1195 394 1196
+3  2486 2485 426
+3  913 623 2477
+3  2636 577 94
+3  1631 719 720
+3  719 1631 2539
+3  1195 1196 613
+3  1196 1199 613
+3  616 2350 2349
+3  616 1150 2350
+3  528 2740 1511
+3  1209 1224 1210
+3  1224 2358 1210
+3  2242 407 2244
+3  83 1711 1302
+3  1300 83 1302
+3  1727 1726 579
+3  1645 579 2011
+3  1516 557 1509
+3  1221 1222 576
+3  1222 94 576
+3  613 1199 1202
+3  170 1716 263
+3  1355 528 1511
+3  1210 400 1209
+3  2767 2768 824
+3  2221 2196 2756
+3  2196 2221 2220
+3  570 1217 1219
+3  1217 570 520
+3  2739 1207 529
+3  1219 1693 1220
+3  1219 524 1693
+3  207 910 154
+3  1212 1213 574
+3  2358 1224 2357
+3  1204 400 1205
+3  1204 1209 400
+3  613 2349 2350
+3  2351 1203 2347
+3  1203 2348 2347
+3  399 1727 583
+3  501 1788 1787
+3  1971 1973 1972
+3  396 575 1130
+3  575 1225 1130
+3  2352 2357 1224
+3  1639 1726 582
+3  520 1218 1217
+3  497 518 1792
+3  2348 1202 2346
+3  242 1217 1218
+3  1208 2739 2740
+3  2739 529 2740
+3  277 2742 1168
+3  519 1785 1786
+3  1772 522 1771
+3  1218 825 242
+3  825 1218 500
+3  2023 2366 935
+3  242 1216 1217
+3  782 1327 1330
+3  56 1788 1789
+3  2631 2633 2628
+3  2631 2634 2633
+3  233 312 984
+3  1786 1785 1791
+3  1793 1784 1794
+3  570 1220 1776
+3  1223 616 573
+3  616 1205 573
+3  1776 2517 1783
+3  574 1221 1214
+3  1199 2356 2354
+3  1214 1223 573
+3  1223 1214 1221
+3  1211 1214 573
+3  419 624 625
+3  1515 419 625
+3  1150 1195 2350
+3  2455 1685 1680
+3  2286 2362 1172
+3  1695 1696 375
+3  1219 1220 570
+3  2359 615 1210
+3  2358 2359 1210
+3  1509 557 1512
+3  1236 725 1237
+3  949 946 1226
+3  183 946 949
+3  36 1232 998
+3  997 36 998
+3  874 1037 1043
+3  1226 1227 949
+3  1226 181 1227
+3  1123 702 1943
+3  740 739 1240
+3  1836 740 1240
+3  354 2 1891
+3  1229 1227 181
+3  1229 1228 1227
+3  1909 769 1358
+3  772 1909 1358
+3  923 1029 1028
+3  2178 1044 1674
+3  1232 36 950
+3  859 925 218
+3  925 1997 218
+3  924 217 1996
+3  1817 949 1230
+3  945 184 947
+3  2044 2048 2049
+3  2048 2044 2026
+3  2734 537 238
+3  1836 2496 2497
+3  1236 1020 735
+3  18 2642 2644
+3  1844 1839 1843
+3  1839 1844 1840
+3  1235 725 1236
+3  1243 1846 739
+3  1251 193 1847
+3  1366 1365 888
+3  1846 1250 1847
+3  1022 1842 1023
+3  458 1838 1242
+3  1842 1242 1841
+3  739 457 1243
+3  737 725 736
+3  2383 2386 2385
+3  2382 2383 2385
+3  1845 193 1840
+3  1255 193 1251
+3  184 535 947
+3  947 535 49
+3  535 1888 49
+3  1237 1239 1020
+3  1839 1838 1837
+3  1243 1250 1846
+3  192 456 457
+3  458 1242 1842
+3  1251 1252 1024
+3  339 2052 2051
+3  110 2065 2066
+3  1616 1617 130
+3  69 1616 130
+3  448 90 335
+3  1842 1841 1023
+3  2622 1418 2621
+3  1268 1270 367
+3  909 2050 2051
+3  2050 909 127
+3  1255 1841 193
+3  1247 1869 1868
+3  456 1254 457
+3  2552 60 2551
+3  1253 1254 456
+3  456 459 1249
+3  1249 439 1247
+3  1835 2496 1239
+3  2496 1835 2497
+3  1243 457 1254
+3  1496 1494 336
+3  2418 2419 763
+3  1277 1291 1653
+3  1909 1908 895
+3  1497 187 353
+3  2574 2230 2231
+3  1864 1865 1264
+3  1864 1265 1865
+3  440 2524 1412
+3  1254 1851 1244
+3  2308 2307 430
+3  1691 1692 1213
+3  1692 1215 1213
+3  1691 1690 1519
+3  450 1022 1258
+3  1909 1178 1908
+3  1263 1855 1262
+3  1024 1255 1251
+3  454 453 1259
+3  90 448 447
+3  448 186 447
+3  1256 1255 1024
+3  1255 1256 1023
+3  186 449 447
+3  1257 1260 54
+3  1023 1258 1022
+3  1023 1256 1258
+3  1256 1257 1258
+3  1258 1257 1259
+3  1245 1260 1252
+3  1488 2383 1285
+3  2383 1488 369
+3  1253 456 1862
+3  1861 1246 1862
+3  1367 768 1366
+3  1907 768 1367
+3  2764 1458 2743
+3  1459 2764 2743
+3  25 2052 339
+3  1263 1262 1866
+3  1519 1690 244
+3  1690 1212 244
+3  1457 386 1460
+3  1298 2110 2111
+3  1403 2327 2332
+3  2333 1403 2332
+3  2214 2219 2534
+3  2229 2219 2214
+3  2375 2380 2376
+3  140 74 908
+3  74 140 2054
+3  338 25 339
+3  520 570 571
+3  2598 250 2600
+3  1699 1038 1600
+3  1481 1479 369
+3  56 1789 2267
+3  1267 1270 1268
+3  1270 1267 366
+3  374 1267 1268
+3  433 1905 431
+3  1004 2186 2185
+3  351 1489 34
+3  1272 1801 1802
+3  2379 230 2378
+3  230 2377 2378
+3  1282 2394 53
+3  371 2683 2684
+3  1296 2107 2108
+3  1658 1293 228
+3  1314 98 285
+3  375 1282 1275
+3  1279 375 1275
+3  469 468 133
+3  1017 469 309
+3  1018 1017 309
+3  2393 1017 1018
+3  2684 2683 53
+3  2733 2732 2734
+3  551 2734 238
+3  555 537 2727
+3  2107 1296 1295
+3  1158 2107 1295
+3  1159 1291 1290
+3  1291 1289 1290
+3  1273 1282 53
+3  1269 1273 53
+3  1292 1159 2413
+3  312 153 311
+3  2125 17 1715
+3  2119 2120 1304
+3  2401 271 296
+3  1288 1277 1287
+3  2122 2125 2124
+3  375 1279 1695
+3  1656 1657 1279
+3  1656 1278 1657
+3  373 1969 1970
+3  424 1185 1187
+3  716 1623 642
+3  1131 716 642
+3  288 2184 2181
+3  2184 2182 2181
+3  1360 778 1359
+3  369 2384 2383
+3  2028 35 994
+3  370 1300 1319
+3  158 2100 2101
+3  227 2118 1306
+3  828 489 2682
+3  52 160 1478
+3  1305 52 1478
+3  1036 2429 1666
+3  1270 2378 367
+3  2378 1270 2379
+3  2730 2735 533
+3  1271 2375 2376
+3  1802 1271 1803
+3  1271 2376 1803
+3  1152 1307 1309
+3  1307 231 1309
+3  315 314 233
+3  231 1307 1295
+3  2111 2114 227
+3  1802 1803 1272
+3  2560 166 2549
+3  1158 2104 2106
+3  1158 2103 2104
+3  1153 1290 1289
+3  984 312 311
+3  1710 1709 2265
+3  1309 1311 470
+3  1153 1294 1295
+3  289 1336 2121
+3  2384 2386 2383
+3  1336 297 2121
+3  1300 229 1299
+3  83 1300 1299
+3  1607 1309 470
+3  48 1297 1313
+3  2272 1325 2275
+3  1325 2272 2273
+3  1422 1423 1399
+3  1114 1115 291
+3  1603 2301 1602
+3  1605 1308 1152
+3  1012 2211 2210
+3  371 2384 2391
+3  1116 1764 475
+3  1306 2112 227
+3  1280 1293 1658
+3  1603 225 1317
+3  1156 1603 1317
+3  475 1764 1763
+3  162 1476 1475
+3  2685 972 1816
+3  978 972 2685
+3  2114 2111 1301
+3  1315 1314 276
+3  1315 1297 275
+3  275 1297 48
+3  1315 276 1297
+3  2076 91 2077
+3  91 2076 2073
+3  2221 2760 2188
+3  1319 1302 1483
+3  369 1479 2387
+3  2384 369 2387
+3  160 52 159
+3  469 753 309
+3  1989 1991 322
+3  543 555 30
+3  555 2727 30
+3  189 1881 2437
+3  2396 1191 1190
+3  1009 2203 2206
+3  2106 2107 1158
+3  2107 2106 1298
+3  1290 2410 2413
+3  367 1802 1801
+3  1153 1295 1308
+3  1308 1295 1307
+3  1308 1307 1152
+3  47 280 283
+3  372 48 1313
+3  230 2370 1380
+3  1610 1154 1608
+3  1311 1309 1310
+3  235 314 315
+3  283 273 235
+3  2110 1301 2111
+3  98 1314 1315
+3  1311 276 1312
+3  276 1311 1310
+3  1660 1280 1659
+3  1312 1314 285
+3  1314 1312 276
+3  1301 2116 2115
+3  2115 2123 1303
+3  2123 1763 1303
+3  1010 2208 2206
+3  2410 2415 2413
+3  2651 2650 2142
+3  2412 2411 1153
+3  966 1155 1602
+3  1304 2120 2122
+3  1602 1157 1601
+3  2301 1292 2415
+3  1316 2409 2411
+3  2756 2196 2757
+3  647 2397 2396
+3  1357 777 1356
+3  1357 767 777
+3  911 778 623
+3  912 911 623
+3  611 911 914
+3  587 1650 1651
+3  805 575 396
+3  2478 783 2475
+3  952 2479 815
+3  405 1320 1321
+3  782 781 405
+3  2261 1013 1119
+3  1360 623 778
+3  1116 472 1119
+3  1322 1321 609
+3  1042 1129 2498
+3  1042 2769 1129
+3  2372 2379 366
+3  2379 1270 366
+3  2084 919 213
+3  2012 2673 1725
+3  1323 1322 609
+3  782 2299 2300
+3  1064 1600 871
+3  2274 1697 588
+3  2505 2506 1061
+3  1324 587 2270
+3  2400 284 2399
+3  2402 284 2400
+3  2074 977 2073
+3  977 2074 203
+3  2270 2269 1324
+3  404 2417 2418
+3  2417 2419 2418
+3  2563 2562 842
+3  2396 2397 1191
+3  1819 1822 688
+3  1330 1327 1328
+3  761 1330 1328
+3  1801 2264 1286
+3  2373 1475 1476
+3  918 1331 919
+3  1331 918 216
+3  1732 1730 1731
+3  2215 1950 1111
+3  77 1126 1395
+3  1757 1760 331
+3  1760 1139 331
+3  324 323 1744
+3  296 1336 270
+3  297 1336 296
+3  1038 871 1600
+3  1538 1534 1539
+3  728 1338 1337
+3  1340 1338 728
+3  1338 1340 1339
+3  1339 564 1338
+3  1339 569 564
+3  569 1339 565
+3  1346 1344 726
+3  564 1351 1350
+3  1351 564 1352
+3  451 1339 1340
+3  1339 451 565
+3  2168 2169 2765
+3  1349 1348 727
+3  450 1346 726
+3  955 957 411
+3  508 2267 2268
+3  453 1346 450
+3  1341 1347 727
+3  449 186 713
+3  1345 449 713
+3  1347 751 727
+3  451 1342 1343
+3  1349 727 480
+3  1343 713 566
+3  1344 713 1343
+3  713 1344 1345
+3  746 747 464
+3  463 2660 748
+3  2717 1432 467
+3  1344 1343 1342
+3  1347 728 751
+3  2660 178 748
+3  2659 178 2660
+3  1340 728 1347
+3  1340 1347 1341
+3  386 1345 453
+3  1345 1344 1346
+3  726 1342 1348
+3  726 1348 1349
+3  726 1349 450
+3  741 464 462
+3  741 744 464
+3  450 1349 1022
+3  1350 1351 729
+3  452 1353 1354
+3  1353 527 1354
+3  781 1360 1359
+3  411 2705 784
+3  569 1352 564
+3  1006 2201 2198
+3  497 1792 850
+3  887 888 403
+3  419 1516 615
+3  522 1772 526
+3  569 1353 1352
+3  452 1354 560
+3  561 452 560
+3  463 747 563
+3  776 406 779
+3  2443 1423 1422
+3  1379 885 886
+3  1362 1363 767
+3  1394 1910 644
+3  1910 1387 644
+3  1361 766 1362
+3  1576 1994 516
+3  590 1323 2271
+3  1357 1358 767
+3  1358 1361 767
+3  1407 1420 1400
+3  911 611 778
+3  611 772 1358
+3  1927 1922 758
+3  2331 1406 1402
+3  767 1361 1362
+3  888 1365 1364
+3  892 1363 1362
+3  1363 892 403
+3  1364 1363 403
+3  888 1364 403
+3  1177 772 1176
+3  766 892 1362
+3  774 1369 1370
+3  1177 774 1370
+3  1370 1368 768
+3  1178 1370 768
+3  1178 1177 1370
+3  1366 1368 775
+3  772 1177 1178
+3  2174 879 2173
+3  2174 2175 879
+3  2282 1161 2283
+3  2279 2282 1503
+3  1369 775 1368
+3  1370 1369 1368
+3  781 1359 405
+3  767 1363 1373
+3  709 708 2629
+3  1650 587 1324
+3  1646 608 1643
+3  1733 323 324
+3  1363 1364 1372
+3  2299 1322 2300
+3  767 1373 777
+3  1373 776 777
+3  1326 1697 2274
+3  1373 1363 1372
+3  1371 776 1372
+3  776 1373 1372
+3  1360 2477 623
+3  770 600 1376
+3  1374 770 1376
+3  1378 764 1379
+3  764 885 1379
+3  600 1377 1376
+3  1376 1377 766
+3  1234 724 1233
+3  1377 1378 766
+3  696 1186 695
+3  600 891 1377
+3  1028 1668 218
+3  554 217 924
+3  1378 1379 892
+3  766 1378 892
+3  1993 886 887
+3  1381 628 693
+3  670 1381 693
+3  1381 670 661
+3  1914 630 1915
+3  630 1916 1915
+3  1428 1417 1429
+3  1941 293 294
+3  1382 628 1381
+3  661 1382 1381
+3  2508 2509 1041
+3  2509 2512 1041
+3  980 219 979
+3  1809 219 980
+3  664 1388 1071
+3  651 1387 1390
+3  645 1384 602
+3  1072 1765 1115
+3  629 1390 1387
+3  629 1933 1390
+3  1870 1871 1265
+3  1874 1871 1870
+3  643 1804 1805
+3  2524 1410 2525
+3  644 1386 1385
+3  644 1387 1386
+3  628 1385 664
+3  644 1385 1383
+3  1383 1385 1382
+3  1385 628 1382
+3  1383 1382 643
+3  2523 1411 2333
+3  651 1388 1386
+3  1387 651 1386
+3  1934 1931 1932
+3  1931 650 1932
+3  2654 2653 596
+3  2654 2656 2653
+3  1615 1616 69
+3  1761 33 346
+3  644 1383 1384
+3  1929 1928 1925
+3  1391 1929 652
+3  598 1391 652
+3  598 1389 1391
+3  1388 1389 627
+3  1071 1388 627
+3  601 1818 1819
+3  1393 788 631
+3  1393 601 788
+3  290 1950 2298
+3  1388 664 1386
+3  1910 1394 1926
+3  1388 651 1389
+3  1407 2329 2330
+3  629 1926 1912
+3  1926 1911 1912
+3  638 665 666
+3  1389 651 1391
+3  440 1402 1406
+3  1666 862 1667
+3  862 1666 2429
+3  2672 700 2671
+3  92 1619 981
+3  1392 627 1389
+3  1394 644 1384
+3  1923 762 881
+3  1669 1672 1699
+3  200 2641 199
+3  126 261 1395
+3  260 1754 1752
+3  2123 475 1763
+3  475 2123 2116
+3  1263 1866 1865
+3  1496 353 75
+3  1497 353 1496
+3  1916 1922 1927
+3  1432 1431 1426
+3  1476 2720 1285
+3  2720 1476 1477
+3  2141 356 1891
+3  1441 1442 1134
+3  909 139 127
+3  838 837 1473
+3  306 152 307
+3  25 1002 2053
+3  189 1878 1881
+3  2328 2329 1407
+3  1436 441 2623
+3  2130 2131 356
+3  1411 1403 2333
+3  383 384 44
+3  1040 2513 1105
+3  1617 1616 127
+3  2449 1421 2528
+3  1410 2521 1428
+3  2521 1410 2520
+3  1811 862 1813
+3  1411 1406 1403
+3  1918 2259 1920
+3  630 1918 1920
+3  920 425 2450
+3  1439 2331 1402
+3  2331 1439 2444
+3  10 1413 2525
+3  1413 1412 2525
+3  1425 1424 460
+3  381 74 237
+3  1868 1869 1248
+3  2327 1408 2522
+3  1414 1413 441
+3  1418 1436 2623
+3  2701 1672 869
+3  1635 1415 1636
+3  2399 2398 271
+3  1409 2521 2520
+3  1416 1417 1404
+3  2314 2308 2311
+3  2330 1401 1420
+3  147 430 2307
+3  147 431 430
+3  1404 1417 1428
+3  353 1136 352
+3  1165 361 1166
+3  2332 2327 1409
+3  237 384 383
+3  2331 1401 2330
+3  188 442 1414
+3  349 74 376
+3  753 133 752
+3  133 1883 752
+3  1424 438 1405
+3  438 1424 1425
+3  2055 1002 143
+3  2053 1002 2055
+3  1879 2441 442
+3  579 583 1727
+3  2131 2130 2136
+3  1636 1415 1429
+3  1437 1441 1134
+3  1401 2443 1422
+3  439 1426 1427
+3  1426 1425 1427
+3  2260 2191 1013
+3  2443 2444 2442
+3  1807 1808 602
+3  1431 438 1426
+3  1431 1430 438
+3  2717 437 2718
+3  2714 2713 1433
+3  364 365 1883
+3  2525 2529 10
+3  2525 1410 2529
+3  1878 1880 1396
+3  838 2595 1435
+3  1879 189 2439
+3  460 1424 1421
+3  1881 1877 1398
+3  1877 1881 1878
+3  1434 2713 2714
+3  356 2140 2130
+3  1880 443 1396
+3  1397 443 1880
+3  442 1025 1397
+3  467 1432 1426
+3  1399 1423 1875
+3  1867 1870 1265
+3  1870 1867 1868
+3  1427 1425 460
+3  1248 1876 1875
+3  444 1461 436
+3  1426 438 1425
+3  2524 2525 1412
+3  2530 1947 1946
+3  1947 2530 703
+3  1807 633 626
+3  2133 2132 144
+3  2404 286 2403
+3  108 1936 1937
+3  1419 1701 1700
+3  600 1921 891
+3  1700 1701 1135
+3  1417 1636 1429
+3  1406 2331 2330
+3  2620 2623 1413
+3  1913 1914 1915
+3  714 2290 2284
+3  2290 714 2318
+3  1703 2771 1702
+3  1405 1636 1417
+3  1416 1405 1417
+3  1919 1918 649
+3  2259 1918 1919
+3  2716 467 459
+3  66 351 197
+3  351 66 1489
+3  132 73 2533
+3  1438 435 188
+3  1459 435 1471
+3  441 1437 1438
+3  2652 44 385
+3  2444 1439 2442
+3  265 266 1502
+3  266 267 1502
+3  1500 2302 2305
+3  2440 2439 1402
+3  1437 441 1436
+3  1447 491 1451
+3  188 1414 1438
+3  1134 435 1438
+3  2593 1440 2594
+3  2201 1008 2198
+3  2651 2136 358
+3  2652 2651 358
+3  2492 2491 434
+3  1977 1596 1597
+3  2637 2638 1442
+3  434 1474 1470
+3  1450 493 1448
+3  2 493 494
+3  1445 491 1447
+3  492 1445 1447
+3  2595 838 1473
+3  2639 2638 2637
+3  2639 1443 2638
+3  413 491 1446
+3  413 511 491
+3  1446 836 413
+3  413 836 835
+3  414 836 1446
+3  491 1445 1446
+3  1447 1448 492
+3  1448 1449 492
+3  445 1455 1137
+3  485 389 486
+3  926 1680 1685
+3  388 1447 1451
+3  2598 2600 253
+3  1472 2592 2593
+3  190 1472 2593
+3  493 1449 1448
+3  485 486 1902
+3  1448 388 1450
+3  1454 1455 1456
+3  1134 2492 1471
+3  1451 511 484
+3  511 1451 491
+3  511 389 484
+3  1447 388 1448
+3  482 1467 481
+3  1966 1444 1967
+3  1451 484 1452
+3  1454 1449 493
+3  388 1453 1450
+3  388 1452 1453
+3  2485 2486 780
+3  1456 387 1454
+3  1455 2 1137
+3  551 79 2734
+3  552 79 551
+3  1906 1367 771
+3  1456 1455 445
+3  1902 486 2483
+3  780 1174 2484
+3  1458 387 1456
+3  1457 1458 1456
+3  1459 444 435
+3  1452 1462 483
+3  1452 484 1462
+3  1457 445 446
+3  521 1785 1773
+3  1324 2269 2271
+3  2269 590 2271
+3  1471 1470 1459
+3  1134 1471 435
+3  1470 1471 434
+3  2766 826 822
+3  2744 1784 1795
+3  939 2746 2748
+3  1794 1784 2744
+3  1452 483 1453
+3  1462 1463 483
+3  1469 355 1468
+3  482 1469 1468
+3  485 1463 1462
+3  619 2583 1175
+3  1174 2583 619
+3  830 829 2236
+3  2680 829 830
+3  392 2580 2238
+3  1466 1467 482
+3  942 1984 2245
+3  2255 1445 492
+3  1561 2238 794
+3  1465 431 432
+3  433 431 1465
+3  1466 432 1467
+3  60 2552 2005
+3  2552 246 2005
+3  1465 1464 148
+3  1455 493 2
+3  255 1892 60
+3  415 837 838
+3  355 1453 483
+3  1468 355 483
+3  482 1468 1464
+3  2133 2142 2134
+3  476 834 816
+3  476 413 834
+3  476 510 413
+3  1284 2374 2367
+3  2168 2765 2167
+3  2765 1046 2167
+3  2141 481 2140
+3  1469 481 2141
+3  1664 889 764
+3  2689 301 2694
+3  493 1455 1454
+3  2638 2492 1442
+3  2492 2638 2491
+3  2491 2638 1443
+3  836 149 835
+3  414 837 836
+3  414 1473 837
+3  837 149 836
+3  1712 171 1713
+3  2398 1168 97
+3  1168 2398 272
+3  1305 1485 52
+3  2265 2266 2339
+3  1476 162 1477
+3  66 197 1684
+3  2368 1380 2370
+3  370 2367 2368
+3  1302 1484 1483
+3  1319 162 1475
+3  1475 370 1319
+3  1902 2483 391
+3  2395 1017 2393
+3  19 1898 1479
+3  2389 161 2388
+3  1478 1479 1481
+3  1478 160 1479
+3  2723 531 257
+3  2750 1797 56
+3  930 34 175
+3  34 1489 175
+3  2369 2370 372
+3  1482 1483 1484
+3  1482 162 1483
+3  1716 1714 155
+3  1717 1716 155
+3  1305 1484 1485
+3  1484 172 1485
+3  2730 2729 532
+3  2303 1501 146
+3  2303 2302 1501
+3  147 2307 2306
+3  1566 1544 512
+3  1501 1500 113
+3  220 2080 2088
+3  2726 2725 538
+3  2392 2395 2393
+3  1477 1480 1488
+3  172 1486 1485
+3  1486 172 1487
+3  1017 2389 469
+3  161 2389 1017
+3  1712 1487 172
+3  1487 1712 1713
+3  1480 1481 1488
+3  2727 538 30
+3  538 539 30
+3  2727 2726 538
+3  536 2724 2725
+3  2724 257 2725
+3  1493 1492 334
+3  1858 1246 1859
+3  1025 54 1263
+3  1493 336 1494
+3  143 1493 1494
+3  2306 2309 2305
+3  2309 1500 2305
+3  26 2058 2059
+3  140 908 909
+3  1879 1878 189
+3  430 23 2308
+3  1498 1495 75
+3  90 1497 336
+3  234 151 852
+3  447 187 1497
+3  90 447 1497
+3  1025 455 54
+3  1495 1498 2056
+3  1498 141 2056
+3  2136 2130 357
+3  2308 2314 428
+3  2138 114 362
+3  771 888 887
+3  2294 2295 865
+3  2294 880 2295
+3  2322 2323 1622
+3  2686 865 1109
+3  463 563 562
+3  1506 1513 558
+3  1513 1506 559
+3  466 561 1505
+3  559 1505 1507
+3  1355 1511 179
+3  523 1220 1517
+3  2129 2148 114
+3  2704 2703 2658
+3  2705 2704 784
+3  2657 1506 2658
+3  1506 784 2658
+3  1440 2713 1434
+3  2594 1440 1434
+3  2660 466 2659
+3  556 1510 529
+3  1509 1510 556
+3  1509 179 1510
+3  1508 1512 559
+3  1509 1512 1508
+3  2703 466 2658
+3  2703 2659 466
+3  1726 2009 579
+3  579 2009 2010
+3  1222 574 575
+3  2707 418 2708
+3  2705 785 2704
+3  557 1514 1512
+3  2435 2434 847
+3  1786 501 1787
+3  1510 1511 529
+3  179 1511 1510
+3  530 2721 2723
+3  2721 531 2723
+3  536 530 2724
+3  730 749 731
+3  730 563 749
+3  625 1513 1514
+3  1514 1513 1512
+3  2129 2147 2148
+3  452 562 1351
+3  1207 2741 1206
+3  2251 420 1553
+3  295 2058 2057
+3  806 2243 807
+3  393 806 807
+3  269 268 2149
+3  486 389 2575
+3  389 509 2575
+3  1561 488 2238
+3  408 1145 2240
+3  408 1144 1145
+3  144 2132 2131
+3  787 1821 2653
+3  1445 2255 1444
+3  1554 1559 421
+3  792 1520 1522
+3  2708 1435 191
+3  1545 1542 55
+3  1542 1541 55
+3  2222 2756 2761
+3  1546 1541 1542
+3  1536 1541 1546
+3  2231 2230 1521
+3  2691 905 2689
+3  2515 1031 1032
+3  2325 1627 723
+3  1528 792 1522
+3  583 1645 1644
+3  1527 1524 422
+3  303 152 304
+3  2690 2694 2695
+3  2697 301 2688
+3  1164 1121 288
+3  2677 1121 1164
+3  1532 982 1533
+3  1530 1532 1533
+3  434 2491 1474
+3  2057 2058 294
+3  299 298 295
+3  791 1525 1531
+3  2541 1631 2542
+3  1555 1570 1588
+3  1572 1555 1588
+3  1999 253 2600
+3  421 1551 1550
+3  2585 2584 1568
+3  2584 2585 1567
+3  789 1536 1569
+3  704 1026 1945
+3  982 983 1534
+3  2059 2058 295
+3  2541 2540 1631
+3  2064 2057 321
+3  1531 1539 1535
+3  1539 1531 1538
+3  2057 2064 295
+3  1937 1942 108
+3  1406 2329 1403
+3  2329 2328 1403
+3  282 281 47
+3  1723 215 87
+3  1546 1548 1569
+3  1551 789 1549
+3  2670 1948 2669
+3  1562 791 1535
+3  1587 1589 1571
+3  1543 1544 513
+3  1552 1557 790
+3  1097 578 1947
+3  578 1097 1725
+3  1547 1548 1564
+3  1548 1547 1549
+3  1560 793 1559
+3  1622 2324 1623
+3  2324 723 1623
+3  498 1 1556
+3  1546 1537 1548
+3  677 678 637
+3  677 1634 678
+3  513 1537 1543
+3  1942 12 293
+3  293 12 282
+3  1540 1539 55
+3  1543 1542 1545
+3  1543 1537 1542
+3  1545 55 1534
+3  1539 1534 55
+3  798 420 797
+3  422 1544 983
+3  512 1553 420
+3  1557 1558 790
+3  1937 1938 12
+3  421 1552 1551
+3  131 132 134
+3  1544 1543 983
+3  1054 880 2294
+3  1534 983 1545
+3  1540 1535 1539
+3  1536 1535 1540
+3  843 842 2588
+3  1565 1564 513
+3  1537 1546 1542
+3  512 420 514
+3  1567 1565 1563
+3  1565 1567 1564
+3  1591 2610 16
+3  1556 1 1560
+3  498 1556 786
+3  1946 1948 701
+3  2323 2324 1622
+3  321 2063 109
+3  1552 421 1557
+3  1579 1580 516
+3  1569 1536 1546
+3  1162 23 78
+3  2587 2584 1563
+3  2587 1568 2584
+3  1551 1552 789
+3  1171 640 1632
+3  422 1553 1544
+3  1558 1521 790
+3  2254 795 2234
+3  1120 1951 2218
+3  512 1544 1553
+3  1549 789 1569
+3  2609 16 2610
+3  2281 2364 2365
+3  234 552 78
+3  1121 961 2187
+3  1554 1550 1555
+3  2460 2577 843
+3  499 2614 1578
+3  2614 499 1768
+3  494 1450 1453
+3  482 1464 1466
+3  794 2232 1558
+3  796 2252 2251
+3  793 794 1558
+3  504 2601 2602
+3  2613 504 2602
+3  488 2239 2238
+3  1905 151 431
+3  1 1561 1560
+3  1 851 1561
+3  1561 851 488
+3  2063 2057 294
+3  1552 1562 789
+3  790 1521 1562
+3  506 1591 1592
+3  1550 1551 2586
+3  2586 2585 1568
+3  1570 1568 2587
+3  1550 1568 1555
+3  1555 1574 786
+3  1569 1548 1549
+3  2603 2613 2602
+3  1574 2606 786
+3  787 788 1821
+3  1554 421 1550
+3  1565 1566 507
+3  1566 1565 513
+3  1544 1566 513
+3  843 2588 1074
+3  2586 1568 1550
+3  982 1532 1594
+3  281 12 1938
+3  1567 1563 2584
+3  1570 1555 1568
+3  2491 1444 1474
+3  1555 1572 1574
+3  976 211 975
+3  2604 1575 2605
+3  507 1591 1590
+3  46 976 203
+3  516 505 515
+3  1573 2618 2608
+3  1582 1581 499
+3  684 2289 2288
+3  951 1232 1231
+3  1575 515 2605
+3  1790 505 1789
+3  2319 2317 691
+3  2287 2282 2283
+3  1779 2753 1800
+3  572 1779 1800
+3  206 205 59
+3  1577 2612 2614
+3  1768 1577 2614
+3  2611 2601 504
+3  69 129 125
+3  1583 1582 938
+3  2742 277 64
+3  1580 505 516
+3  218 1995 923
+3  1581 1768 499
+3  1708 2341 1707
+3  2659 785 2661
+3  1772 240 526
+3  240 527 526
+3  2170 1045 2166
+3  1064 871 2505
+3  238 555 543
+3  1230 949 1228
+3  876 2158 2157
+3  1809 1810 222
+3  862 1809 222
+3  979 972 2097
+3  972 2098 2097
+3  972 978 2098
+3  1817 1739 183
+3  2287 2283 2284
+3  1050 2172 1051
+3  2164 2160 2167
+3  2160 1048 2167
+3  848 1900 1899
+3  2357 2353 614
+3  2342 2102 51
+3  2342 156 2102
+3  183 949 1817
+3  578 1948 1947
+3  1620 714 3
+3  446 2326 1457
+3  2671 2674 580
+3  1709 2747 56
+3  1121 2677 1120
+3  517 1580 1579
+3  1585 2772 2771
+3  1300 1302 1319
+3  1707 1583 572
+3  1789 505 2268
+3  1781 1583 938
+3  543 545 212
+3  1200 1224 1209
+3  1563 1565 1589
+3  507 1566 1592
+3  1590 1589 507
+3  1571 1589 1590
+3  786 1554 1555
+3  507 1592 1591
+3  1554 786 1556
+3  604 788 787
+3  1554 1556 1559
+3  1556 1560 1559
+3  1588 1571 1593
+3  1590 1593 1571
+3  1590 16 1593
+3  2217 2216 1949
+3  1527 1594 1526
+3  1594 1523 1526
+3  2400 2401 2402
+3  2324 2323 2325
+3  1132 639 2150
+3  1978 1598 940
+3  1598 1979 940
+3  1980 1979 1598
+3  926 1683 1681
+3  1838 458 1020
+3  1156 1292 2301
+3  940 1976 1975
+3  1598 1599 27
+3  869 1669 1670
+3  2597 1595 2599
+3  2643 31 1682
+3  1828 1825 1826
+3  1829 28 1830
+3  1113 1957 1958
+3  1963 1954 1118
+3  2559 42 1831
+3  1599 1598 1597
+3  101 2550 2549
+3  2264 1287 1286
+3  1801 1272 2264
+3  1277 1286 1287
+3  1294 1158 1295
+3  1611 1155 1154
+3  2194 2227 2226
+3  206 58 205
+3  58 2490 205
+3  966 1605 1606
+3  966 1606 1155
+3  2087 2086 968
+3  2086 974 968
+3  2084 2083 919
+3  311 153 307
+3  958 471 1609
+3  232 958 1609
+3  1013 2207 1119
+3  1604 966 1601
+3  966 1602 1601
+3  2227 1005 2192
+3  1607 1152 1309
+3  1152 1607 1605
+3  1606 1605 1607
+3  143 1002 1491
+3  1609 1312 232
+3  470 1312 1609
+3  1311 1312 470
+3  1013 2261 2260
+3  1617 127 139
+3  2008 883 2007
+3  503 2610 506
+3  1614 1616 1615
+3  125 1615 69
+3  1615 125 1618
+3  1764 1072 1763
+3  2774 136 379
+3  694 2008 882
+3  2257 2008 694
+3  2015 2033 2031
+3  73 1617 139
+3  73 132 131
+3  2214 2534 2216
+3  2534 1949 2216
+3  130 73 131
+3  130 1617 73
+3  129 69 130
+3  71 129 130
+3  2002 248 2003
+3  2669 1948 578
+3  2668 2669 578
+3  126 1614 1615
+3  217 554 916
+3  318 321 109
+3  141 1499 2054
+3  2534 2219 2228
+3  961 2534 2228
+3  2284 2283 3
+3  689 1092 2466
+3  2465 690 1091
+3  1634 1633 640
+3  677 676 1633
+3  676 677 636
+3  1632 640 1633
+3  472 1612 2260
+3  2544 2545 2546
+3  2363 2544 2546
+3  723 1628 642
+3  721 642 1629
+3  1630 386 453
+3  1624 1623 716
+3  1102 2463 2462
+3  2458 1102 2462
+3  1173 2543 2363
+3  962 1101 1100
+3  1101 2462 1100
+3  963 68 254
+3  722 1131 642
+3  1623 723 642
+3  1103 2459 2458
+3  2459 1102 2458
+3  1461 1630 454
+3  2579 830 2236
+3  2067 299 2065
+3  2070 2068 110
+3  2068 2070 303
+3  2067 2065 110
+3  2691 300 2692
+3  2446 45 2445
+3  152 2070 320
+3  320 2070 110
+3  111 2691 2692
+3  2362 2363 1172
+3  678 1634 640
+3  1621 1622 1623
+3  2183 2260 1612
+3  2257 2259 1919
+3  2257 694 2259
+3  438 1635 1405
+3  2667 2671 700
+3  1920 2259 694
+3  1413 1414 1412
+3  1635 438 1430
+3  1435 1434 2714
+3  1324 1323 401
+3  1650 1324 401
+3  1145 393 807
+3  711 1641 710
+3  1291 1159 1652
+3  502 938 499
+3  401 1646 1648
+3  1649 401 1648
+3  1647 608 1646
+3  2559 62 167
+3  2634 802 2626
+3  1647 1646 1323
+3  1648 1646 1643
+3  608 612 583
+3  1644 608 583
+3  1642 1643 584
+3  585 2634 2626
+3  2627 2629 2632
+3  585 2627 2632
+3  586 1649 2626
+3  802 586 2626
+3  1647 394 612
+3  612 608 1647
+3  394 1647 609
+3  2489 1036 1035
+3  613 2348 2349
+3  609 1647 1323
+3  401 1649 1650
+3  1654 1656 1276
+3  1649 586 1651
+3  1649 1651 1650
+3  2564 1735 1742
+3  2408 1011 2407
+3  1277 1653 1286
+3  1275 1276 1656
+3  1159 1654 1652
+3  1652 1654 1276
+3  157 479 120
+3  159 157 120
+3  479 121 120
+3  2101 82 158
+3  1293 1294 1288
+3  1660 1659 1278
+3  1899 496 1770
+3  1654 1278 1656
+3  1655 1660 1278
+3  1970 1969 1275
+3  1329 588 760
+3  2100 159 2101
+3  159 52 2101
+3  1486 82 2101
+3  2117 227 2114
+3  1294 2103 1158
+3  1662 1454 387
+3  1474 1661 1470
+3  2124 1304 2122
+3  1660 1655 1292
+3  1470 1662 387
+3  1663 1378 1377
+3  1378 1663 764
+3  2425 2422 759
+3  2425 757 2422
+3  1664 891 1665
+3  891 1664 1663
+3  1664 764 1663
+3  2421 2417 404
+3  889 884 764
+3  2417 2421 2426
+3  1667 1668 1666
+3  1668 1667 859
+3  1814 1812 863
+3  1666 861 1036
+3  2686 1109 867
+3  1029 861 1028
+3  1668 859 218
+3  1371 1365 775
+3  861 1668 1028
+3  1677 2175 2174
+3  2176 1677 2174
+3  1670 1669 1039
+3  1044 1675 1674
+3  1675 1043 1674
+3  958 232 2179
+3  46 550 976
+3  1675 1050 1034
+3  1845 1241 1846
+3  2162 1671 870
+3  872 1049 1056
+3  1671 1670 1039
+3  2295 1059 2296
+3  1059 2295 876
+3  1095 2502 702
+3  1722 1721 154
+3  874 1675 1034
+3  2174 2173 1044
+3  2176 2174 1044
+3  1122 1123 1943
+3  1037 874 1032
+3  1045 1670 1671
+3  874 1043 1675
+3  1834 262 1753
+3  1670 1676 869
+3  1670 1045 1676
+3  2335 946 947
+3  946 945 947
+3  201 4 1679
+3  1673 1678 1674
+3  2018 116 2023
+3  519 1774 2702
+3  662 663 606
+3  2664 692 2666
+3  1890 90 334
+3  1687 660 681
+3  1729 197 323
+3  2568 2567 343
+3  387 1458 2764
+3  2643 1682 9
+3  200 199 201
+3  2343 200 201
+3  119 118 122
+3  250 941 85
+3  250 1976 941
+3  1999 1998 253
+3  681 660 682
+3  2647 195 2648
+3  2008 2007 882
+3  175 1683 926
+3  927 175 926
+3  1686 1684 1682
+3  32 2035 2034
+3  32 2036 2035
+3  1614 1698 2050
+3  1595 2598 2599
+3  1683 1686 1681
+3  1866 1262 1873
+3  1262 1858 1873
+3  2017 2023 935
+3  1848 1250 1244
+3  1400 2527 1408
+3  1923 881 648
+3  679 632 1688
+3  679 667 632
+3  679 1688 1687
+3  675 679 1687
+3  603 1068 1069
+3  1688 632 707
+3  1852 1252 1850
+3  1882 2442 2437
+3  1861 1860 1246
+3  1693 1519 1517
+3  1519 1208 1517
+3  1220 1693 1517
+3  524 1691 1693
+3  1772 1771 521
+3  938 1782 1781
+3  1696 1318 375
+3  1657 1694 1279
+3  1657 228 1694
+3  1658 228 1657
+3  1281 1694 228
+3  1696 1695 1281
+3  1059 876 2156
+3  1326 1085 1697
+3  1096 703 1095
+3  1698 338 339
+3  1698 1395 338
+3  126 1395 1698
+3  126 1698 1614
+3  1706 1705 508
+3  1300 370 2369
+3  1436 1700 1135
+3  1700 1436 1418
+3  1582 1702 1586
+3  1581 1582 1586
+3  2007 893 765
+3  883 893 2007
+3  1418 2623 2620
+3  952 815 1183
+3  1817 950 1739
+3  2269 2275 1325
+3  2275 2269 2270
+3  1679 202 2455
+3  1790 1788 501
+3  1583 1703 1702
+3  379 378 2774
+3  2738 244 2737
+3  1711 172 1484
+3  1302 1711 1484
+3  826 2127 822
+3  2490 7 205
+3  2733 2734 79
+3  2474 2473 2472
+3  2471 2474 2472
+3  833 2575 490
+3  833 2676 2575
+3  1214 1211 1212
+3  849 2736 852
+3  2729 531 532
+3  2341 1584 1704
+3  1707 2341 1704
+3  2266 1584 2339
+3  2341 2340 1584
+3  2280 2430 2293
+3  156 2100 2102
+3  2100 158 2102
+3  171 83 224
+3  171 1711 83
+3  243 822 2127
+3  821 822 243
+3  172 1711 1712
+3  82 1486 1487
+3  1732 997 996
+3  1721 1719 154
+3  1713 171 1714
+3  1719 64 154
+3  171 1712 1711
+3  1476 1285 2381
+3  701 2670 708
+3  82 170 169
+3  272 98 273
+3  17 269 1715
+3  823 821 478
+3  155 1720 1717
+3  2448 1027 2075
+3  2447 2446 2448
+3  1717 263 1716
+3  263 1717 1718
+3  1719 1718 264
+3  1719 24 1718
+3  64 1719 264
+3  24 263 1718
+3  2075 7 2448
+3  2447 2448 7
+3  209 2083 2084
+3  38 209 2084
+3  266 1721 1723
+3  1721 266 24
+3  266 1723 87
+3  2645 2644 9
+3  1722 214 215
+3  1721 1722 1723
+3  1758 1759 340
+3  1714 1724 1713
+3  1724 1714 1716
+3  1716 170 1724
+3  1097 698 1725
+3  2675 1637 1638
+3  1637 1639 582
+3  576 1151 1223
+3  576 2472 1151
+3  994 2036 2042
+3  2010 1639 699
+3  2010 2009 1639
+3  1639 1640 699
+3  1578 2603 503
+3  2170 1677 1045
+3  2172 2171 1047
+3  2038 2024 2037
+3  261 259 1752
+3  820 243 844
+3  1729 1730 37
+3  323 1730 1729
+3  325 1333 328
+3  1333 325 1734
+3  1736 1731 325
+3  1733 1731 323
+3  1744 323 99
+3  2230 2574 1520
+3  792 2230 1520
+3  2564 2565 2573
+3  2565 2564 345
+3  325 1738 1737
+3  1743 326 340
+3  93 917 918
+3  260 341 1754
+3  1743 324 1744
+3  36 1738 1739
+3  1730 1732 37
+3  37 1732 996
+3  38 344 215
+3  325 328 1738
+3  1733 324 1735
+3  1126 337 332
+3  2573 2565 2569
+3  1741 1331 216
+3  1041 2512 875
+3  213 1333 1334
+3  36 997 1737
+3  340 1745 1742
+3  345 1745 1746
+3  1741 1332 1331
+3  2746 2750 2748
+3  2750 2746 1780
+3  1737 1738 36
+3  1070 1393 631
+3  1392 692 1818
+3  1891 2 494
+3  1889 335 1890
+3  355 1891 494
+3  36 1739 950
+3  1741 328 1332
+3  1072 226 1763
+3  1739 1738 328
+3  1740 1739 328
+3  1741 1740 328
+3  916 216 918
+3  216 1740 1741
+3  2142 144 2651
+3  1742 1743 340
+3  326 1743 1744
+3  99 326 1744
+3  327 326 99
+3  1754 337 77
+3  1734 325 2570
+3  2569 88 2571
+3  2570 2569 2571
+3  1750 343 2567
+3  342 1750 2567
+3  460 1421 1399
+3  1751 259 65
+3  260 1749 1748
+3  342 1748 1749
+3  1749 65 1750
+3  1750 342 1749
+3  2431 2430 2291
+3  327 99 351
+3  77 261 1752
+3  260 1752 1751
+3  341 1753 1754
+3  1753 1755 1754
+3  347 330 346
+3  1758 329 1757
+3  331 1141 1490
+3  1756 331 1490
+3  1141 89 1490
+3  1491 1490 89
+3  1492 1491 89
+3  1141 1140 89
+3  1753 262 1755
+3  948 330 181
+3  948 1140 330
+3  1850 1849 1244
+3  1850 1252 1849
+3  2020 2030 2029
+3  2030 2019 2029
+3  2018 2023 2017
+3  262 1759 1758
+3  262 1758 1757
+3  948 333 1140
+3  33 1762 327
+3  1762 326 327
+3  1762 33 1761
+3  329 1762 1761
+3  1613 475 2116
+3  1072 1073 226
+3  472 1116 1613
+3  40 103 104
+3  1115 292 1072
+3  4 931 1766
+3  1765 1072 1764
+3  1612 472 1613
+3  4 1766 202
+3  927 1685 202
+3  1766 927 202
+3  129 70 125
+3  931 4 932
+3  929 1767 928
+3  121 70 129
+3  1777 571 570
+3  549 92 967
+3  517 1769 1581
+3  1586 517 1581
+3  1769 517 1579
+3  1776 1777 570
+3  2658 784 2704
+3  2662 178 2707
+3  242 2768 1130
+3  496 521 1770
+3  1327 590 2269
+3  1353 569 527
+3  1773 1772 521
+3  1772 1773 240
+3  163 1894 19
+3  1774 525 1775
+3  1710 2747 1709
+3  530 239 526
+3  1786 518 501
+3  1208 1519 2738
+3  571 1777 1778
+3  2751 1710 2336
+3  1787 1788 1798
+3  2752 2749 2751
+3  1800 2752 2751
+3  571 2754 2755
+3  610 394 609
+3  502 500 1218
+3  938 502 1782
+3  1327 2269 1325
+3  2657 2658 466
+3  1776 1783 1777
+3  1505 2657 466
+3  1518 1208 2740
+3  1775 2517 523
+3  523 1799 1775
+3  382 383 361
+3  1211 2737 244
+3  1774 1775 1799
+3  1786 1787 519
+3  1789 2268 2267
+3  2336 2337 1708
+3  505 1790 498
+3  1580 2268 505
+3  501 498 1790
+3  502 520 1782
+3  521 496 1791
+3  1705 1706 2773
+3  1791 1792 518
+3  1791 496 1792
+3  2333 2520 2523
+3  2517 1775 2516
+3  2517 2516 1783
+3  2519 2332 1409
+3  2738 1519 244
+3  313 2371 2372
+3  2266 1709 2267
+3  1780 1795 1796
+3  2523 1410 2524
+3  1793 1794 1778
+3  1777 1793 1778
+3  1798 1774 519
+3  1787 1798 519
+3  1795 1784 1796
+3  56 1797 1788
+3  526 239 522
+3  525 1797 1796
+3  1796 1797 1780
+3  2752 2753 939
+3  1788 1797 1798
+3  525 1798 1797
+3  313 372 2371
+3  2262 2263 1272
+3  1272 1803 2262
+3  1274 1269 367
+3  372 2370 2371
+3  661 605 1804
+3  661 685 605
+3  2419 1188 763
+3  2611 1576 2601
+3  1804 605 633
+3  605 634 633
+3  633 1806 1805
+3  643 1805 1806
+3  643 1806 602
+3  1384 643 602
+3  688 962 1092
+3  1824 962 688
+3  1383 643 1384
+3  606 1808 1807
+3  1125 1824 656
+3  1809 862 1811
+3  708 2672 2630
+3  652 759 2665
+3  652 1925 757
+3  699 1640 2667
+3  980 1619 1810
+3  1810 1619 546
+3  104 2342 51
+3  218 923 1028
+3  219 1811 1812
+3  1816 972 979
+3  1054 1055 1053
+3  2327 2328 1407
+3  474 2182 2190
+3  1230 950 1817
+3  2289 1503 2288
+3  1508 559 1507
+3  1082 1823 687
+3  42 8 1825
+3  1080 95 1081
+3  689 1820 688
+3  1819 688 1820
+3  94 577 576
+3  634 605 697
+3  2656 634 697
+3  634 2656 2654
+3  664 1385 1386
+3  601 1821 788
+3  962 1824 1125
+3  1955 1952 1117
+3  1827 80 1599
+3  1979 941 1976
+3  80 57 40
+3  1827 57 80
+3  1831 1829 1832
+3  1831 62 2559
+3  1745 1834 1746
+3  1826 1827 28
+3  1827 1599 28
+3  292 1115 1957
+3  1827 1826 57
+3  1832 62 1831
+3  42 1828 1829
+3  1828 28 1829
+3  1828 1826 28
+3  1415 2621 1429
+3  2622 2621 1415
+3  1759 1745 340
+3  1745 1759 1834
+3  1888 1889 49
+3  1595 1975 1976
+3  57 107 40
+3  42 1829 1831
+3  100 2561 2560
+3  8 2561 100
+3  2550 100 2560
+3  102 29 103
+3  1832 1833 43
+3  78 203 1027
+3  2335 49 948
+3  2334 2335 948
+3  946 2335 2334
+3  1838 1020 1239
+3  1837 1838 1239
+3  535 1887 1888
+3  738 457 739
+3  1839 1840 1242
+3  458 1842 1022
+3  1844 1241 1845
+3  1844 1843 1241
+3  1841 1255 1023
+3  1229 423 985
+3  2494 1240 2493
+3  1240 1843 2493
+3  1836 1240 2494
+3  2494 2493 1837
+3  1849 1848 1244
+3  1847 1848 1251
+3  1245 1855 1260
+3  1247 1427 1869
+3  1844 1845 1840
+3  1843 1240 739
+3  1241 1843 739
+3  739 1846 1241
+3  1245 1252 1852
+3  78 552 203
+3  2020 2022 932
+3  347 423 181
+3  2139 2146 362
+3  2146 2138 362
+3  1849 1252 1251
+3  1851 1850 1244
+3  1851 1689 1850
+3  1254 1253 1851
+3  1253 1689 1851
+3  1847 1250 1848
+3  1260 1855 54
+3  1245 1853 1854
+3  1853 1245 1852
+3  1855 1245 1854
+3  1869 1876 1248
+3  333 334 1492
+3  1139 1140 1141
+3  330 1140 1139
+3  1253 1862 1261
+3  1862 1246 1261
+3  1857 1854 1853
+3  1856 1857 1853
+3  1852 1689 1853
+3  1261 1857 1856
+3  1246 1858 1261
+3  1858 1857 1261
+3  1266 1247 1868
+3  1253 1261 1856
+3  1864 1264 1863
+3  769 1909 895
+3  1858 1859 1873
+3  25 2053 2052
+3  1859 1246 1860
+3  1249 1862 456
+3  1249 1861 1862
+3  895 1907 1906
+3  436 1461 454
+3  1266 1868 1867
+3  1863 1859 1860
+3  1266 1863 1860
+3  1864 1863 1867
+3  1266 1867 1863
+3  1866 1264 1865
+3  1859 1863 1264
+3  349 139 908
+3  1868 1248 1870
+3  2051 2052 140
+3  1872 1396 443
+3  1872 1865 1265
+3  1865 1872 443
+3  1872 1265 1871
+3  1871 1396 1872
+3  1264 1866 1873
+3  1873 1859 1264
+3  1858 1262 1857
+3  1875 1876 1399
+3  1248 1874 1870
+3  1876 460 1399
+3  460 1876 1869
+3  2440 2441 1879
+3  1874 1877 1871
+3  1398 1877 1874
+3  1877 1396 1871
+3  318 1163 321
+3  1495 2055 143
+3  1398 1874 1875
+3  1874 1248 1875
+3  1879 2439 2440
+3  1875 1423 1882
+3  1398 1875 1882
+3  1879 1397 1880
+3  1878 1879 1880
+3  1869 1427 460
+3  442 1397 1879
+3  1878 1396 1877
+3  45 2447 1935
+3  2438 189 2437
+3  1881 1398 1882
+3  1883 365 752
+3  133 1884 1883
+3  1884 364 1883
+3  134 364 1884
+3  198 2043 2049
+3  1698 339 2050
+3  2498 2499 1042
+3  1128 877 2498
+3  877 2499 2498
+3  960 2677 1164
+3  2641 2640 1886
+3  1885 194 35
+3  2027 198 2048
+3  2040 2024 2039
+3  214 1722 910
+3  539 2730 533
+3  857 223 856
+3  1887 448 335
+3  948 49 1889
+3  333 948 1889
+3  335 1889 1888
+3  1890 333 1889
+3  334 333 1890
+3  335 90 1890
+3  1895 1897 19
+3  101 2549 2548
+3  532 2735 2730
+3  1006 2204 2203
+3  164 71 165
+3  121 71 164
+3  119 70 121
+3  121 129 71
+3  479 20 119
+3  134 1884 1896
+3  1884 468 1896
+3  468 1884 133
+3  19 1894 1895
+3  1423 2442 1882
+3  1423 2443 2442
+3  177 531 2721
+3  531 177 567
+3  160 19 1479
+3  134 132 364
+3  1488 1481 369
+3  1770 848 1899
+3  850 495 851
+3  1903 241 849
+3  241 1903 848
+3  1901 848 1903
+3  1900 1901 391
+3  1902 1904 433
+3  1902 1901 1904
+3  86 1985 2245
+3  86 1987 1985
+3  148 485 1902
+3  497 850 851
+3  495 488 851
+3  2648 2646 2647
+3  1903 1904 1901
+3  1904 1905 433
+3  793 1561 794
+3  849 1905 1904
+3  849 151 1905
+3  849 852 151
+3  1137 187 445
+3  883 894 893
+3  893 894 771
+3  894 1906 771
+3  769 895 1375
+3  894 895 1906
+3  1375 895 894
+3  1178 768 1908
+3  1908 768 1907
+3  1249 1247 1861
+3  1909 772 1178
+3  1488 2720 1477
+3  191 2716 461
+3  1929 650 1928
+3  1933 650 1929
+3  1394 1384 645
+3  2424 763 591
+3  2341 1708 2340
+3  649 630 1914
+3  630 649 1918
+3  758 1925 1928
+3  671 666 667
+3  2260 474 2191
+3  649 891 1921
+3  1924 1923 648
+3  630 1920 1917
+3  1921 1919 649
+3  474 2190 2191
+3  2262 1318 1696
+3  764 884 885
+3  884 881 882
+3  885 884 882
+3  762 1917 1920
+3  934 2018 2017
+3  1026 2577 2576
+3  1916 1917 1922
+3  1917 762 1922
+3  630 1917 1916
+3  814 815 409
+3  1188 1187 696
+3  1923 1922 762
+3  2019 116 2018
+3  1700 2622 1419
+3  1923 1924 758
+3  1913 1934 1912
+3  1916 1927 1930
+3  1927 1928 1930
+3  1931 1930 650
+3  1930 1928 650
+3  1390 1933 1391
+3  1916 1930 1931
+3  1915 1916 1931
+3  1407 1400 1408
+3  2528 2527 1400
+3  1933 1929 1391
+3  595 665 1071
+3  1931 1934 1915
+3  1932 1933 629
+3  1932 629 1912
+3  1913 1912 646
+3  112 45 1935
+3  1932 650 1933
+3  2621 1418 2620
+3  1913 1915 1934
+3  1936 13 1937
+3  1937 13 1938
+3  1967 2491 1443
+3  1473 1967 1443
+3  108 1941 1940
+3  1942 293 1941
+3  108 1942 1941
+3  1940 1939 108
+3  2247 2246 1985
+3  2278 1064 2503
+3  1527 983 982
+3  1944 1943 702
+3  1944 1945 1943
+3  2670 701 1948
+3  2503 2276 2278
+3  1335 289 959
+3  959 289 960
+3  2533 139 2532
+3  2222 2761 2225
+3  1625 2325 2323
+3  703 1097 1947
+3  1946 1947 1948
+3  2576 2577 1122
+3  1062 2511 1128
+3  1944 1946 1945
+3  1946 704 1945
+3  1172 2363 2546
+3  635 678 640
+3  701 704 1946
+3  449 2326 446
+3  1955 1953 1952
+3  1164 959 960
+3  1120 1949 1121
+3  1949 961 1121
+3  1117 1952 1120
+3  1115 1114 1958
+3  292 1956 1117
+3  2186 1004 2223
+3  2197 1005 2193
+3  1012 2199 1011
+3  1113 1964 1956
+3  2221 2188 2220
+3  1111 290 1954
+3  1953 1954 290
+3  2144 359 358
+3  359 2652 358
+3  1114 350 1959
+3  1111 1954 1112
+3  1962 1965 1960
+3  1965 1962 1959
+3  1956 292 1957
+3  2226 2227 2678
+3  1964 1113 1962
+3  1964 1962 1963
+3  1956 1964 1955
+3  1965 1959 350
+3  42 15 8
+3  1959 1113 1958
+3  1112 1960 1961
+3  2561 15 2558
+3  1960 350 1961
+3  166 2558 2557
+3  2561 2558 166
+3  8 15 2561
+3  1963 1118 1964
+3  388 1451 1452
+3  414 1446 1966
+3  1445 1966 1446
+3  1652 1276 1968
+3  1653 1652 1968
+3  1201 1199 2353
+3  1201 1202 1199
+3  1969 1276 1275
+3  373 1653 1969
+3  1653 373 1286
+3  1653 1968 1969
+3  1968 1276 1969
+3  2557 2554 2553
+3  1286 373 1274
+3  348 63 2556
+3  1971 348 2556
+3  31 200 2343
+3  2457 31 2343
+3  2596 1595 2597
+3  62 1832 43
+3  681 2150 639
+3  1977 940 1975
+3  1977 1974 1596
+3  1977 1975 1974
+3  1596 43 1833
+3  2640 2641 18
+3  1596 1974 1973
+3  1975 1595 1974
+3  1596 1973 43
+3  1980 941 1979
+3  1982 1981 942
+3  1981 1982 1980
+3  1978 940 1977
+3  1597 1978 1977
+3  1833 1597 1596
+3  1830 1597 1833
+3  2247 85 2246
+3  942 2246 1982
+3  2245 2246 942
+3  2648 195 995
+3  18 1885 2640
+3  1885 18 2645
+3  169 1986 1984
+3  1983 169 1984
+3  420 2251 2252
+3  685 680 605
+3  1991 904 322
+3  1194 1193 322
+3  1193 1989 322
+3  80 81 1981
+3  80 1981 27
+3  1981 1980 27
+3  1982 941 1980
+3  2247 128 965
+3  85 941 1982
+3  1988 65 258
+3  2697 2698 301
+3  897 2698 2697
+3  1988 258 128
+3  1981 1983 942
+3  1981 81 1983
+3  965 85 2247
+3  1984 942 1983
+3  2233 795 2254
+3  169 81 168
+3  81 169 1983
+3  265 1502 86
+3  1502 1987 86
+3  169 170 1986
+3  1988 128 1987
+3  197 1728 1684
+3  1750 65 1988
+3  267 1750 1988
+3  87 1750 267
+3  267 1988 1987
+3  796 2250 2249
+3  799 2249 2250
+3  1992 1379 886
+3  588 1329 2273
+3  2270 587 1326
+3  1573 2608 2609
+3  1328 1327 1325
+3  1992 892 1379
+3  2611 2615 1994
+3  887 403 1993
+3  892 1992 403
+3  1992 1993 403
+3  2298 2619 290
+3  2619 2298 1951
+3  218 1997 1995
+3  1106 2453 5
+3  1996 1997 924
+3  1766 1767 927
+3  2454 1215 1692
+3  2454 1216 1225
+3  1215 2454 1225
+3  524 2454 1692
+3  2454 524 1216
+3  963 251 252
+3  964 907 963
+3  348 1998 2000
+3  247 348 2001
+3  63 348 247
+3  348 2000 2001
+3  2001 2000 41
+3  2001 2003 247
+3  1619 92 547
+3  2006 517 1586
+3  2005 2004 60
+3  247 2004 2005
+3  2004 248 60
+3  122 70 119
+3  248 2004 2003
+3  517 2006 1706
+3  2772 1585 2773
+3  2033 2017 935
+3  1367 1366 888
+3  2528 1416 2527
+3  2019 2018 934
+3  243 820 821
+3  1807 1806 633
+3  2345 2346 1202
+3  2631 2628 1638
+3  2016 2033 2034
+3  877 2500 2499
+3  923 2453 1106
+3  1040 2508 2506
+3  196 2027 2048
+3  1885 2027 196
+3  2048 198 2049
+3  2026 115 196
+3  35 2027 1885
+3  2044 2045 2026
+3  2045 2044 2025
+3  199 2047 201
+3  2016 934 2017
+3  2034 2033 2015
+3  2043 2038 2025
+3  2043 2039 2038
+3  96 987 986
+3  96 992 988
+3  1081 662 606
+3  933 2019 934
+3  2021 932 937
+3  2020 116 2030
+3  116 2019 2030
+3  2041 198 2028
+3  1679 2343 201
+3  1679 2457 2343
+3  2040 2039 2041
+3  61 993 994
+3  2042 2036 32
+3  202 1685 2455
+3  1699 2278 1063
+3  1998 1971 1972
+3  2044 2049 2043
+3  2041 2039 198
+3  1042 2500 1041
+3  2048 2026 196
+3  2029 2022 2020
+3  2022 2029 933
+3  2029 2019 933
+3  1503 1504 686
+3  2279 1503 686
+3  933 928 2022
+3  2014 2031 935
+3  2031 2033 935
+3  2035 2016 2034
+3  2035 988 2016
+3  2513 2507 1031
+3  2161 2160 2164
+3  2013 936 2014
+3  988 992 2016
+3  2511 877 1128
+3  1110 2296 875
+3  2512 1110 875
+3  2040 2042 32
+3  2015 2032 2024
+3  2032 2037 2024
+3  2032 936 2037
+3  2502 1944 702
+3  1946 1944 2530
+3  2451 920 2450
+3  994 993 2036
+3  2015 2040 32
+3  32 2034 2015
+3  2040 2015 2024
+3  993 61 1001
+3  2037 936 2025
+3  2046 115 2045
+3  198 2039 2043
+3  868 1110 2512
+3  2042 2040 2041
+3  936 2045 2025
+3  1039 1063 1062
+3  1039 1062 870
+3  1669 1063 1039
+3  1601 1157 2409
+3  1045 1671 2166
+3  4 201 2047
+3  937 2047 2046
+3  2046 2047 115
+3  287 2758 2759
+3  878 2161 2164
+3  199 115 2047
+3  932 2047 937
+3  4 2047 932
+3  2309 2313 1500
+3  428 2313 2309
+3  2310 1500 2313
+3  26 1940 2058
+3  1940 294 2058
+3  2308 23 2311
+3  791 1531 1535
+3  113 2060 298
+3  2060 26 2059
+3  26 2061 1940
+3  1940 2061 1939
+3  2061 112 1939
+3  2062 112 2061
+3  427 112 2062
+3  112 427 45
+3  113 1500 2310
+3  2310 2062 113
+3  307 152 320
+3  2069 22 299
+3  299 295 2065
+3  2068 2069 2067
+3  2069 299 2067
+3  2068 2067 110
+3  343 215 344
+3  975 91 977
+3  2085 91 975
+3  205 204 2071
+3  479 157 20
+3  59 208 214
+3  208 59 2071
+3  22 1501 298
+3  1501 113 298
+3  152 303 2070
+3  2082 210 2078
+3  210 2082 2086
+3  2083 2079 919
+3  203 2074 1027
+3  204 2074 2073
+3  2075 2074 204
+3  1027 2074 2075
+3  2446 2445 1162
+3  2071 59 205
+3  2075 204 205
+3  7 2075 205
+3  24 1719 1721
+3  1422 1421 2449
+3  1421 1422 1399
+3  2080 2079 210
+3  214 2081 38
+3  549 211 548
+3  2081 209 38
+3  220 922 2080
+3  2078 2079 2083
+3  208 2077 2081
+3  2076 2077 208
+3  220 2091 970
+3  2078 2083 209
+3  2077 2078 209
+3  203 976 977
+3  157 156 20
+3  156 157 2100
+3  204 2072 2071
+3  2085 2082 91
+3  1317 1280 1156
+3  210 2086 2087
+3  2087 2088 2080
+3  2088 2087 968
+3  2087 2080 210
+3  549 967 2089
+3  968 549 2089
+3  968 2089 2088
+3  969 2090 967
+3  969 2091 2090
+3  2091 220 2090
+3  970 2093 971
+3  2072 208 2071
+3  206 59 207
+3  2095 92 981
+3  2097 221 979
+3  967 92 2095
+3  969 2094 2092
+3  11 58 206
+3  2095 981 221
+3  980 981 1619
+3  221 2094 2095
+3  969 2095 2094
+3  2686 2294 865
+3  2085 2099 2082
+3  2086 2099 974
+3  2082 2099 2086
+3  1317 225 2105
+3  1293 2103 1294
+3  2102 158 51
+3  2103 1293 1280
+3  2104 2103 1280
+3  1280 1317 2104
+3  1317 2105 2104
+3  1296 231 1295
+3  1297 2109 1313
+3  2108 229 2109
+3  1296 2108 2109
+3  2113 2112 1299
+3  2124 2125 224
+3  2110 1298 2106
+3  2105 2110 2106
+3  2108 2113 229
+3  1298 2112 2113
+3  2111 2112 1298
+3  1297 1296 2109
+3  231 1296 1297
+3  2106 2104 2105
+3  2105 1301 2110
+3  2123 2115 2116
+3  232 1312 285
+3  1298 2113 2108
+3  2124 2118 1304
+3  2118 2124 1306
+3  84 51 168
+3  227 2117 2118
+3  1301 2115 2114
+3  226 1073 2121
+3  1303 2119 2117
+3  2119 2118 2117
+3  2119 1304 2118
+3  1156 1280 1660
+3  1303 226 2119
+3  1303 1763 226
+3  2105 225 2116
+3  2105 2116 1301
+3  225 1613 2116
+3  1073 289 2121
+3  1336 289 1335
+3  2117 2115 1303
+3  297 2120 2121
+3  297 2122 2120
+3  2126 1715 171
+3  224 1306 2124
+3  82 169 168
+3  2125 1715 2126
+3  1714 171 1715
+3  171 224 2126
+3  2682 489 2681
+3  828 2682 826
+3  395 828 826
+3  826 2682 2127
+3  487 845 833
+3  1467 2148 481
+3  2148 2147 481
+3  2143 2139 362
+3  2135 2134 142
+3  787 2653 2656
+3  2502 2530 1944
+3  2502 703 2530
+3  302 359 896
+3  2700 302 896
+3  2537 1173 2362
+3  1891 2132 354
+3  1891 356 2132
+3  352 2134 2135
+3  354 2132 2133
+3  1508 1507 179
+3  2650 2651 385
+3  2133 2134 352
+3  2524 1411 2523
+3  2320 691 2315
+3  357 2129 2138
+3  1499 1498 142
+3  147 2306 429
+3  353 352 2135
+3  357 2138 2137
+3  2129 114 2138
+3  898 145 363
+3  2147 2130 2140
+3  155 2149 1720
+3  2149 268 1720
+3  300 2691 2693
+3  2149 155 1715
+3  2142 2133 144
+3  385 44 384
+3  2688 904 2697
+3  411 621 955
+3  2143 363 2139
+3  363 145 2139
+3  114 801 362
+3  242 825 2768
+3  2143 2304 146
+3  626 606 1807
+3  2689 905 2688
+3  301 2689 2688
+3  1528 1529 792
+3  2140 481 2147
+3  1715 269 2149
+3  268 269 97
+3  1132 2150 2151
+3  2150 681 2151
+3  659 1132 2151
+3  2152 1504 684
+3  2153 2152 684
+3  691 2317 2316
+3  2151 681 682
+3  2152 2151 682
+3  2283 2285 3
+3  1161 2285 2283
+3  714 2284 3
+3  659 2152 2153
+3  659 2151 2152
+3  1621 1623 1624
+3  1019 716 1131
+3  985 989 2154
+3  96 2154 989
+3  182 985 986
+3  2157 2156 876
+3  1060 870 1129
+3  293 109 2063
+3  1058 870 2155
+3  867 2685 2686
+3  2215 2216 2217
+3  287 1011 2194
+3  1059 2156 2155
+3  878 1671 2162
+3  2158 876 1049
+3  2166 1671 878
+3  872 1047 1048
+3  2160 1057 2159
+3  1110 865 2296
+3  2162 870 1058
+3  2169 2171 879
+3  2168 1048 1047
+3  2175 2169 879
+3  2175 2170 2169
+3  2176 1676 1677
+3  2170 1046 2169
+3  2279 2293 2282
+3  2293 1161 2282
+3  1044 2173 1675
+3  2173 1050 1675
+3  1201 2345 1202
+3  1204 1203 2351
+3  1201 2344 2345
+3  1318 2262 1803
+3  1678 1676 2178
+3  1676 1678 869
+3  2179 286 959
+3  286 1335 959
+3  2181 958 2180
+3  2180 958 2179
+3  959 2180 2179
+3  2222 1004 2760
+3  1898 19 1897
+3  2220 2188 2228
+3  2759 1961 287
+3  2266 2267 508
+3  2181 2182 958
+3  958 2183 471
+3  958 2182 2183
+3  2182 474 2183
+3  2698 898 301
+3  1420 2449 1400
+3  2449 2528 1400
+3  894 883 770
+3  2186 1005 1003
+3  2185 2186 1003
+3  2202 1009 2205
+3  966 1604 1605
+3  1008 1005 2197
+3  2188 1004 2187
+3  2184 2190 2182
+3  711 1094 2562
+3  711 1095 1094
+3  2181 2180 288
+3  2187 2185 288
+3  2195 2220 2229
+3  2198 1008 2197
+3  2230 792 1525
+3  2407 2406 2198
+3  1112 2759 1007
+3  2579 831 2679
+3  830 2579 2679
+3  1009 1013 2205
+3  2208 2207 2206
+3  2211 2213 2212
+3  2213 1114 2208
+3  2550 2560 2549
+3  2200 2212 2204
+3  2200 2211 2212
+3  2548 2549 245
+3  1316 2414 1604
+3  2412 1153 1308
+3  2549 166 245
+3  291 1119 2207
+3  2416 2427 1190
+3  1013 1009 2207
+3  291 2208 1114
+3  2208 291 2207
+3  1655 1159 1292
+3  2002 2003 2001
+3  1654 1159 1655
+3  2210 2209 2199
+3  1012 2210 2199
+3  1010 2204 2212
+3  2213 1010 2212
+3  8 100 102
+3  1012 1961 2211
+3  2298 2297 1951
+3  2298 1950 2297
+3  2215 2217 1950
+3  843 2577 1026
+3  296 270 2401
+3  1111 2195 2214
+3  2194 2193 2227
+3  1111 2214 2215
+3  2183 474 2260
+3  353 2135 75
+3  2757 2758 2189
+3  1111 1112 1007
+3  2195 1111 1007
+3  2214 2216 2215
+3  1336 1335 270
+3  2218 2217 1949
+3  1830 1832 1829
+3  2641 1886 199
+3  2232 2231 1558
+3  2222 2225 2223
+3  97 269 271
+3  18 2641 2642
+3  1335 286 2404
+3  961 2228 2188
+3  1520 799 1522
+3  2583 1179 774
+3  2232 2233 150
+3  795 2582 2234
+3  2581 2582 795
+3  2253 2254 2234
+3  1465 432 1466
+3  1465 1466 1464
+3  2243 806 407
+3  2583 1174 1179
+3  2679 2680 830
+3  148 1902 433
+3  2237 795 2233
+3  2233 2254 150
+3  2234 2235 796
+3  2249 2234 796
+3  1175 2583 774
+3  393 1145 589
+3  2467 2469 596
+3  2579 2581 2580
+3  2579 2236 2581
+3  1473 414 1967
+3  148 433 1465
+3  2581 795 2580
+3  392 2239 487
+3  828 829 489
+3  2239 488 495
+3  487 2239 495
+3  846 487 495
+3  2241 2242 808
+3  817 808 2242
+3  2243 407 2242
+3  2240 807 2241
+3  2241 408 2240
+3  2242 2244 817
+3  807 2240 1145
+3  797 2252 2235
+3  266 87 267
+3  2245 1984 86
+3  1524 799 2250
+3  2648 995 35
+3  194 2648 35
+3  2245 1985 2246
+3  1520 2248 799
+3  118 119 20
+3  2254 2253 150
+3  150 2231 2232
+3  1821 2467 596
+3  2579 392 831
+3  2576 1943 2578
+3  2237 2233 2232
+3  797 420 2252
+3  2249 2253 2234
+3  2255 1474 1444
+3  492 1474 2255
+3  2220 2228 2256
+3  2229 2220 2256
+3  2219 2256 2228
+3  2229 2256 2219
+3  2155 2156 1058
+3  2157 1058 2156
+3  272 6 1168
+3  285 98 2403
+3  2402 2404 2403
+3  2404 2402 270
+3  2198 2406 2405
+3  1006 2198 2405
+3  2535 1526 1522
+3  284 2402 2403
+3  2163 1058 2157
+3  1272 2263 2264
+3  2263 1287 2264
+3  572 1583 1781
+3  2340 2339 1584
+3  2340 2338 2339
+3  1708 2338 2340
+3  1584 2266 1705
+3  2266 508 1705
+3  533 2735 535
+3  2735 1887 535
+3  1993 1992 886
+3  2270 1326 2272
+3  405 2299 782
+3  704 2563 842
+3  2273 1328 1325
+3  1329 1328 2273
+3  761 1328 1329
+3  1063 2276 1062
+3  2507 1040 2506
+3  2507 2513 1040
+3  2500 877 2501
+3  2277 877 2511
+3  2510 2277 2511
+3  2503 1064 2504
+3  2365 718 2286
+3  2278 1600 1064
+3  2278 1699 1600
+3  718 2362 2286
+3  2366 2014 935
+3  2322 715 2323
+3  2284 2288 2287
+3  684 2290 2153
+3  1403 2328 2327
+3  2290 2288 2284
+3  382 361 1165
+3  1504 2289 684
+3  719 1160 686
+3  684 2288 2290
+3  1504 1503 2289
+3  1154 1606 2687
+3  1606 1607 2687
+3  2319 1621 1624
+3  2315 1620 2320
+3  2280 1160 2292
+3  2279 1160 2280
+3  686 1160 2279
+3  2279 2280 2293
+3  2011 579 2010
+3  2012 2011 2010
+3  2671 1640 2674
+3  1725 698 2012
+3  698 2011 2012
+3  698 1097 1096
+3  500 514 825
+3  2291 2292 2281
+3  2292 1160 2281
+3  2430 2431 2293
+3  2431 1161 2293
+3  2300 1327 782
+3  2300 590 1327
+3  2411 2410 1290
+3  2411 2409 2410
+3  1290 2413 1159
+3  1603 1156 2301
+3  1521 1525 791
+3  801 2303 2304
+3  801 429 2303
+3  146 2304 2303
+3  2304 2143 362
+3  801 2304 362
+3  2434 496 2436
+3  2062 2310 427
+3  2314 2310 2313
+3  2314 427 2310
+3  1500 1501 2302
+3  2311 23 2312
+3  427 2311 2312
+3  429 2306 2305
+3  428 2307 2308
+3  428 2314 2313
+3  2311 427 2314
+3  1496 75 1495
+3  45 427 2312
+3  2446 2447 45
+3  1027 2446 1162
+3  2449 1420 1422
+3  1420 1401 1422
+3  2529 1410 1428
+3  870 1062 1128
+3  2317 716 1019
+3  1624 716 2317
+3  2322 1622 1621
+3  2318 2153 2290
+3  2319 1624 2317
+3  1410 2523 2520
+3  1578 2613 2603
+3  513 1564 1537
+3  1173 2363 2362
+3  386 1457 2326
+3  2002 2001 41
+3  2763 2002 41
+3  2366 2013 2014
+3  2541 2542 717
+3  2544 2541 717
+3  2434 1792 496
+3  850 1792 2434
+3  2303 429 2433
+3  626 633 2655
+3  2324 2325 723
+3  1625 1627 2325
+3  2327 1407 1408
+3  386 449 1345
+3  386 2326 449
+3  2329 1406 2330
+3  2445 2312 1162
+3  2312 2445 45
+3  136 382 379
+3  136 381 382
+3  1836 2494 2495
+3  2496 1836 2495
+3  1226 946 2334
+3  2496 2495 1239
+3  136 376 381
+3  572 2336 1708
+3  2335 947 49
+3  506 1592 514
+3  514 500 506
+3  2195 2196 2220
+3  2344 1200 2345
+3  572 1800 2336
+3  287 2194 2758
+3  531 2729 257
+3  2750 56 2747
+3  2337 2338 1708
+3  2337 2265 2338
+3  2725 2728 538
+3  447 446 187
+3  1667 862 222
+3  156 2342 105
+3  51 84 104
+3  2607 2605 1572
+3  2342 104 105
+3  2462 2463 1100
+3  1971 43 1973
+3  2641 200 2642
+3  2752 939 2749
+3  1795 2746 2745
+3  2745 2744 1795
+3  2344 1224 1200
+3  2344 2352 1224
+3  1209 2347 1200
+3  2347 2346 1200
+3  615 400 1210
+3  2347 1209 2351
+3  2745 2746 939
+3  1778 2745 939
+3  2545 1625 1626
+3  2352 1201 2353
+3  617 615 2359
+3  617 419 615
+3  2356 2355 2354
+3  426 2355 2356
+3  2354 2353 1199
+3  777 779 2488
+3  779 777 776
+3  426 617 2355
+3  1197 426 2356
+3  426 1197 1198
+3  2355 617 2359
+3  2359 2358 614
+3  2357 614 2358
+3  2355 2359 614
+3  2354 2355 614
+3  718 2364 2536
+3  2537 718 2536
+3  2360 2539 2538
+3  2360 719 2539
+3  2360 2538 2537
+3  2536 2360 2537
+3  718 2537 2362
+3  717 2545 2544
+3  719 2360 2361
+3  719 2361 1160
+3  2364 2281 1160
+3  2361 2364 1160
+3  1313 2369 372
+3  2109 229 1313
+3  229 2369 1313
+3  1300 2369 229
+3  315 316 236
+3  2370 230 2371
+3  2367 1475 2373
+3  312 313 2372
+3  1284 2367 2373
+3  315 233 316
+3  2381 2373 1476
+3  2382 1285 2383
+3  2374 2375 1380
+3  1284 2375 2374
+3  2378 1802 367
+3  2378 1271 1802
+3  2381 2380 1284
+3  1284 2373 2381
+3  1803 2376 1318
+3  2379 2372 230
+3  2371 230 2372
+3  2375 1284 2380
+3  310 1267 374
+3  2392 310 374
+3  2389 468 469
+3  468 2389 1898
+3  2378 2377 1271
+3  2380 368 2376
+3  1479 2388 2387
+3  2380 2382 368
+3  2381 2382 2380
+3  1283 2385 2386
+3  162 1480 1477
+3  2388 1898 2389
+3  2381 1285 2382
+3  2382 2385 368
+3  2384 2387 2391
+3  19 160 163
+3  163 160 159
+3  1478 1481 1480
+3  2392 2391 2395
+3  310 2392 2393
+3  368 2390 1318
+3  2376 368 1318
+3  2390 368 2385
+3  2385 1283 2390
+3  1480 162 1482
+3  1478 1480 1482
+3  1483 162 1319
+3  1283 2684 53
+3  2397 646 607
+3  646 2397 1913
+3  375 2390 2394
+3  375 1318 2390
+3  371 2391 2392
+3  310 2393 1018
+3  375 2394 1282
+3  2391 161 2395
+3  79 2736 2733
+3  1925 652 1929
+3  1665 890 2426
+3  2399 272 2398
+3  98 284 2403
+3  271 2401 2400
+3  2405 2199 2209
+3  1006 2405 2209
+3  2403 286 285
+3  1011 2406 2407
+3  2406 1011 2199
+3  2406 2199 2405
+3  2193 1011 2408
+3  2408 2197 2193
+3  1153 2411 1290
+3  1308 1605 2414
+3  1308 2414 2412
+3  2412 2414 1316
+3  1605 1604 2414
+3  1009 2202 2203
+3  1316 1601 2409
+3  1604 1601 1316
+3  1157 2410 2409
+3  2411 2412 1316
+3  1157 2415 2410
+3  2415 1157 2301
+3  2427 2396 1190
+3  2426 2421 1665
+3  2421 889 1665
+3  2422 404 759
+3  2424 591 1330
+3  1665 889 1664
+3  761 2420 2424
+3  2420 2418 2424
+3  761 1329 2420
+3  652 2425 759
+3  652 757 2425
+3  1189 1188 2419
+3  2418 2420 404
+3  760 759 2420
+3  759 404 2420
+3  760 588 1083
+3  1329 760 2420
+3  2424 1330 761
+3  1914 2428 649
+3  2423 884 889
+3  2418 763 2424
+3  890 2427 2416
+3  2427 2428 2396
+3  2427 890 2428
+3  2428 647 2396
+3  1914 647 2428
+3  1924 2423 757
+3  890 649 2428
+3  648 2423 1924
+3  862 2429 1813
+3  2291 2430 2292
+3  429 2305 2433
+3  2433 2302 2303
+3  2439 1439 1402
+3  2435 847 495
+3  847 846 495
+3  850 2435 495
+3  2482 2483 845
+3  486 845 2483
+3  2436 1900 847
+3  2434 2436 847
+3  1130 2767 396
+3  2767 2766 396
+3  850 2434 2435
+3  1899 1900 2436
+3  496 1899 2436
+3  391 2483 2482
+3  440 2440 1402
+3  440 2441 2440
+3  440 1414 2441
+3  189 2438 2439
+3  2438 1439 2439
+3  1414 442 2441
+3  1401 2444 2443
+3  1439 2438 2442
+3  2438 2437 2442
+3  2447 7 13
+3  1935 2447 13
+3  2566 342 2567
+3  425 973 2450
+3  996 61 995
+3  921 2451 2450
+3  2453 923 1995
+3  2453 1995 921
+3  2452 2453 921
+3  2452 5 2453
+3  973 5 2452
+3  978 5 973
+3  1103 1122 2577
+3  1972 253 1998
+3  2456 2457 1679
+3  2456 31 2457
+3  2455 2456 1679
+3  250 85 252
+3  907 68 963
+3  1026 2578 1945
+3  843 1074 2460
+3  2542 720 1628
+3  2461 1101 841
+3  1101 1124 841
+3  1100 2463 2464
+3  690 2464 2463
+3  2458 1101 2461
+3  2458 2462 1101
+3  2464 1092 1100
+3  2470 2654 596
+3  2469 2470 596
+3  626 2655 2470
+3  690 2465 2464
+3  2465 1092 2464
+3  1821 689 2467
+3  1222 575 94
+3  2470 1090 1089
+3  799 2248 2249
+3  2361 2536 2364
+3  2471 1637 582
+3  1127 913 2475
+3  952 1183 416
+3  2476 913 2477
+3  1057 2161 2163
+3  952 416 953
+3  2481 952 953
+3  520 502 1218
+3  1637 1640 1639
+3  577 1637 2471
+3  1638 1637 577
+3  1727 2473 582
+3  2477 174 2476
+3  810 409 827
+3  2540 2762 1173
+3  2538 2540 1173
+3  1173 2762 2543
+3  2762 2541 2543
+3  1781 1779 572
+3  952 2481 783
+3  809 409 810
+3  815 2479 409
+3  1360 695 2477
+3  695 1360 781
+3  2478 2476 827
+3  2479 2478 827
+3  2478 2479 783
+3  2767 824 2766
+3  2480 1127 2475
+3  847 391 2482
+3  1900 391 847
+3  2482 846 847
+3  1320 2488 610
+3  2710 418 2709
+3  957 2710 2709
+3  785 2712 2706
+3  2712 2709 2706
+3  2709 2712 2705
+3  610 2488 1198
+3  1813 2429 2489
+3  1035 863 2489
+3  1198 2487 2486
+3  863 1813 2489
+3  1813 863 1812
+3  2488 2487 1198
+3  2488 779 2487
+3  2429 1036 2489
+3  2609 2608 16
+3  7 2490 13
+3  13 2490 1938
+3  2493 1839 1837
+3  1974 1595 2596
+3  252 85 965
+3  2596 1973 1974
+3  2493 1843 1839
+3  2495 2494 1837
+3  1237 1835 1239
+3  948 181 1226
+3  2495 1837 1239
+3  2497 1238 740
+3  2646 2645 9
+3  2497 740 1836
+3  1128 2498 1129
+3  2770 2296 1059
+3  2769 2770 1060
+3  877 2277 2501
+3  2504 1064 2505
+3  868 2512 2509
+3  2499 2500 1042
+3  1105 864 1106
+3  2506 2505 871
+3  2507 871 1037
+3  2506 871 2507
+3  1031 2507 1037
+3  1105 2509 1040
+3  2509 1105 868
+3  2276 1063 2278
+3  2510 2276 2503
+3  2504 2510 2503
+3  2504 2277 2510
+3  875 1042 1041
+3  2276 2510 1062
+3  864 2513 2514
+3  2513 864 1105
+3  864 2514 1030
+3  1775 1893 2516
+3  2738 2739 1208
+3  1778 2753 2754
+3  2518 1893 1784
+3  2518 2516 1893
+3  1793 2518 1784
+3  1793 1783 2518
+3  1783 2516 2518
+3  1167 383 44
+3  1990 302 899
+3  901 302 1990
+3  360 302 901
+3  1409 2522 2521
+3  2521 2522 1404
+3  1989 1990 899
+3  2327 2522 1409
+3  440 1411 2524
+3  1424 1405 2528
+3  2528 1405 1416
+3  1421 1424 2528
+3  2522 1408 2526
+3  2522 2526 1404
+3  2526 1416 1404
+3  2526 2527 1416
+3  2526 1408 2527
+3  1678 2701 869
+3  376 377 349
+3  377 2531 349
+3  165 131 134
+3  377 135 2531
+3  135 2532 2531
+3  2532 349 2531
+3  139 349 2532
+3  73 139 2533
+3  74 349 908
+3  270 1335 2404
+3  2552 2551 245
+3  2360 2536 2361
+3  720 2542 1631
+3  251 254 2763
+3  841 1123 1122
+3  251 2763 41
+3  41 1999 251
+3  1631 2540 2539
+3  2540 2538 2539
+3  963 254 251
+3  2541 2544 2543
+3  2543 2544 2363
+3  2547 1627 717
+3  723 1627 1628
+3  1627 1625 2545
+3  1172 2546 3
+3  719 683 720
+3  2542 2547 717
+3  1892 101 2548
+3  29 100 2550
+3  100 29 102
+3  345 2564 1745
+3  256 2550 101
+3  256 29 2550
+3  60 1892 2551
+3  245 2553 2552
+3  2553 246 2552
+3  245 2551 2548
+3  2551 1892 2548
+3  167 2557 2558
+3  167 2554 2557
+3  67 255 248
+3  2556 63 2555
+3  2554 246 2553
+3  167 62 2555
+3  167 2555 2554
+3  2555 63 2554
+3  255 256 101
+3  67 256 255
+3  2557 2553 245
+3  166 2557 245
+3  2556 62 43
+3  1971 2556 43
+3  1680 31 2456
+3  62 2556 2555
+3  2558 2559 167
+3  2559 2558 15
+3  15 42 2559
+3  2562 1094 712
+3  842 2562 712
+3  2574 2231 150
+3  587 1651 1147
+3  2563 704 581
+3  2565 345 1747
+3  1735 324 1742
+3  2570 2571 1734
+3  2568 2571 88
+3  2571 2568 2572
+3  1735 2564 2573
+3  341 1747 1746
+3  88 2567 2568
+3  343 344 2568
+3  2569 1735 2573
+3  2569 2570 1735
+3  1734 2571 2572
+3  1333 213 1332
+3  432 147 801
+3  1731 1733 325
+3  1333 1734 2572
+3  2572 1334 1333
+3  1334 2572 344
+3  344 2572 2568
+3  432 801 114
+3  432 114 1467
+3  1467 114 2148
+3  2575 509 490
+3  1464 1463 148
+3  2235 2582 2236
+3  1103 2577 2460
+3  243 2682 2681
+3  844 243 2681
+3  2582 2235 2234
+3  2236 797 2235
+3  2237 2580 795
+3  2244 407 818
+3  1942 1937 12
+3  1549 1547 2586
+3  2586 1547 2585
+3  2585 1547 1567
+3  1547 1564 1567
+3  2586 1551 1549
+3  2588 842 712
+3  2590 1436 1135
+3  2590 2591 1441
+3  1472 1441 2591
+3  1472 1442 1441
+3  2593 2592 1440
+3  2590 1135 2591
+3  1701 2592 1135
+3  2591 1135 2592
+3  1437 2590 1441
+3  1472 2591 2592
+3  308 306 307
+3  2595 1434 1435
+3  2492 434 1471
+3  2595 2594 1434
+3  2600 251 1999
+3  2597 1973 2596
+3  2598 1595 1976
+3  251 250 252
+3  2600 250 251
+3  2598 1976 250
+3  1972 2599 253
+3  2599 2598 253
+3  550 548 976
+3  2617 1588 1593
+3  1591 506 2610
+3  2602 2604 1573
+3  974 549 968
+3  1572 2605 1574
+3  2602 1573 2603
+3  2612 2613 2614
+3  2613 1578 2614
+3  79 552 234
+3  852 79 234
+3  2605 515 2606
+3  2609 2603 1573
+3  2609 2610 2603
+3  515 498 2606
+3  2603 2610 503
+3  2616 1593 2608
+3  2612 2611 504
+3  2612 2615 2611
+3  1952 2619 1951
+3  2607 1572 2616
+3  1576 515 1575
+3  1576 1575 2601
+3  2613 2612 504
+3  2612 1577 2615
+3  1577 1994 2615
+3  1577 1579 1994
+3  1769 1579 1577
+3  1573 2607 2618
+3  2619 1953 290
+3  2616 2617 1593
+3  1572 2617 2616
+3  2617 1572 1588
+3  2620 1413 10
+3  2621 2620 10
+3  1429 2621 10
+3  10 2529 1429
+3  2529 1428 1429
+3  441 1413 2623
+3  585 2626 2625
+3  1642 1641 2624
+3  1648 1642 2624
+3  585 2625 2627
+3  608 1644 1643
+3  2625 1649 1648
+3  708 709 581
+3  398 2634 2631
+3  2635 2631 1638
+3  2629 2627 709
+3  2630 2628 2629
+3  2627 2624 709
+3  2627 2625 2624
+3  1648 2624 2625
+3  2629 708 2630
+3  2635 1638 577
+3  2628 2630 580
+3  398 803 802
+3  2633 2634 585
+3  2633 585 2632
+3  398 2631 2635
+3  2632 2629 2628
+3  2633 2632 2628
+3  398 802 2634
+3  1151 399 1150
+3  1195 1150 612
+3  2636 94 804
+3  845 486 2676
+3  2639 1473 1443
+3  215 1723 1722
+3  2643 2644 2642
+3  2145 2650 385
+3  200 31 2643
+3  200 2643 2642
+3  2645 18 2644
+3  1886 2649 196
+3  2647 2646 9
+3  1684 1728 2647
+3  1728 195 2647
+3  2645 194 1885
+3  1684 2647 9
+3  995 994 35
+3  2646 2648 194
+3  2645 2646 194
+3  1728 1729 195
+3  2649 1885 196
+3  2136 2137 358
+3  2651 2652 385
+3  144 2136 2651
+3  359 44 2652
+3  360 44 359
+3  604 787 635
+3  2655 2654 2470
+3  2655 634 2654
+3  2655 633 634
+3  1526 1528 1522
+3  785 2703 2704
+3  785 2659 2703
+3  559 2657 1505
+3  1506 2657 559
+3  2707 2706 418
+3  1433 467 2715
+3  2706 2661 785
+3  466 2660 561
+3  561 2660 562
+3  558 784 1506
+3  1835 1237 2663
+3  2663 1237 725
+3  725 1238 2663
+3  1835 1238 2497
+3  1835 2663 1238
+3  2664 687 1822
+3  687 1823 1822
+3  692 1819 1818
+3  692 1392 2666
+3  1389 598 2666
+3  2630 2672 580
+3  1191 607 1192
+3  653 2665 759
+3  512 1592 1566
+3  797 829 798
+3  1592 512 514
+3  397 803 804
+3  2012 2010 699
+3  2668 578 2673
+3  1640 2671 2667
+3  2012 699 2673
+3  578 1725 2673
+3  2760 2756 2222
+3  1638 2628 2675
+3  2675 2628 580
+3  580 2674 2675
+3  398 2636 804
+3  398 2635 2636
+3  1637 2675 2674
+3  2674 1640 1637
+3  1645 583 579
+3  1120 2677 1117
+3  2756 2760 2221
+3  1957 1115 1958
+3  1117 2677 960
+3  2682 243 2127
+3  832 2680 2679
+3  2392 374 2683
+3  371 2392 2683
+3  53 2683 1269
+3  367 1269 1268
+3  1269 374 1268
+3  374 1269 2683
+3  867 978 2685
+3  1054 2686 866
+3  1054 2294 2686
+3  905 904 2688
+3  905 322 904
+3  146 2696 363
+3  2691 111 905
+3  2690 2693 2694
+3  896 145 2699
+3  2690 300 2693
+3  2690 22 300
+3  363 2695 898
+3  2690 2696 22
+3  2692 304 111
+3  2699 898 2698
+3  2698 897 2699
+3  1501 22 2696
+3  359 145 896
+3  2700 2699 897
+3  899 2700 897
+3  302 2700 899
+3  2702 1774 1773
+3  1785 2702 1773
+3  2702 1785 519
+3  435 444 436
+3  2713 1440 437
+3  521 1771 1770
+3  527 1355 1354
+3  2715 467 2716
+3  2710 839 2711
+3  1800 2751 2336
+3  957 2705 411
+3  2707 2661 2706
+3  417 957 955
+3  2705 2712 785
+3  2708 415 1435
+3  415 838 1435
+3  1795 1780 2746
+3  2711 2708 418
+3  2710 2711 418
+3  492 1449 1661
+3  839 415 2711
+3  191 1435 2715
+3  2715 1435 2714
+3  1431 2719 1430
+3  1430 2719 1419
+3  1431 2718 2719
+3  2718 1431 1432
+3  2718 437 2719
+3  1701 1419 2719
+3  753 755 309
+3  14 306 308
+3  1017 2395 161
+3  308 307 153
+3  539 2729 2730
+3  2732 2731 537
+3  239 2731 2732
+3  2735 185 1887
+3  532 185 2735
+3  2733 522 239
+3  2733 239 2732
+3  97 2742 274
+3  2742 97 1168
+3  2741 573 1206
+3  1207 2737 2741
+3  939 2748 2749
+3  2753 1778 939
+3  2749 2747 1710
+3  1782 571 2755
+3  1779 2755 2754
+3  2541 2762 2540
+3  2196 2758 2757
+3  1007 2758 2196
+3  2758 1007 2759
+3  2759 1112 1961
+3  2225 2761 2757
+3  2756 2757 2761
+3  387 2764 1470
+3  2766 824 826
+3  825 824 2768
+3  1703 1585 2771
+3  1042 875 2769
+
diff -pruN 5.0.3-1/examples/Surface_mesh_topology/data/elephant-with-holes.off 5.1-1/examples/Surface_mesh_topology/data/elephant-with-holes.off
--- 5.0.3-1/examples/Surface_mesh_topology/data/elephant-with-holes.off	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_topology/data/elephant-with-holes.off	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,7265 @@
+OFF
+2798 4463 0
+
+0.262933 0.102269 0.138247
+0.0843142 0.0418575 -0.0419302
+0.0676609 -0.0308717 0.133371
+0.202895 0.468475 0.0802072
+0.113075 -0.465378 -0.0546734
+-0.146525 -0.22403 -0.169286
+-0.0249865 -0.129931 -0.13238
+-0.160965 -0.480027 -0.0707824
+0.0794063 -0.415277 -0.0839096
+0.0469116 -0.050008 0.252355
+-0.0998372 -0.193745 -0.16268
+-0.111131 -0.104825 -0.14166
+-0.061459 -0.0977343 -0.133353
+-0.247783 -0.131539 0.0722694
+-0.145972 -0.486627 -0.0633275
+0.0916759 0.0333604 -0.133002
+-0.184954 -0.325468 -0.133853
+0.0847778 -0.432659 -0.0978527
+-0.210776 -0.299151 0.0751516
+-0.151539 -0.388715 0.01282
+-0.154564 -0.438244 -0.0249412
+-0.123411 -0.0182656 0.0135824
+0.00773278 -0.053391 -0.0428874
+-0.0774524 -0.292395 -0.106335
+0.0201594 -0.263586 0.119859
+-0.105646 -0.0365155 -0.0770751
+-0.113391 -0.41896 -0.0906771
+-0.128213 -0.46259 -0.0994907
+-0.160526 -0.468057 -0.0393445
+0.197609 -0.127492 -0.0574767
+0.0757306 -0.449539 -0.0614557
+0.147148 -0.412461 -0.0801639
+0.103782 -0.333007 0.0288926
+0.108917 -0.401981 -0.00841926
+0.11392 -0.408931 -0.0973654
+0.129733 -0.31331 -0.0672325
+0.0904388 -0.356531 -0.0803906
+0.0359798 -0.255445 -0.113562
+-0.22312 -0.107007 -0.0611904
+-0.163408 -0.433458 -0.0704785
+-0.0915185 -0.473885 -0.0165533
+-0.141417 -0.474418 -0.0834065
+-0.0944539 -0.479454 -0.0663683
+-0.085985 -0.101282 0.158093
+-0.0225438 -0.0889567 -0.0848587
+0.092707 -0.109496 -0.114561
+-0.159213 -0.159272 -0.138793
+-0.24541 -0.230248 -0.110578
+0.183744 -0.231931 0.0176086
+0.294273 -0.0955008 0.116328
+-0.16477 -0.385872 -0.0413331
+-0.185303 -0.313456 -0.00374653
+-0.297084 -0.262835 -0.00415653
+0.167756 -0.128776 0.197921
+-0.0273811 0.0608812 -0.0989637
+0.208303 0.00452459 -0.0746544
+-0.150831 -0.448909 -0.0901866
+-0.0720206 -0.147118 -0.154796
+-0.0197986 -0.210658 -0.154154
+-0.130593 -0.498558 -0.021161
+0.129662 -0.373473 -0.0794411
+-0.120692 -0.481168 -0.0685411
+-0.108692 -0.494274 -0.0429275
+-0.0970902 -0.252938 -0.14843
+-0.0098054 -0.335319 -0.0147292
+0.0769246 -0.410346 -0.0391239
+-0.132409 -0.464154 -0.0197707
+-0.102806 -0.4217 0.00855496
+-0.0760332 -0.34128 0.100617
+-0.104435 -0.377391 0.0553722
+-0.14788 -0.33188 0.0922673
+-0.151409 -0.218926 0.165812
+-0.111518 -0.286925 0.148566
+-0.0458796 -0.210094 0.18627
+0.0439919 -0.147905 0.137964
+-0.0358575 -0.361749 0.0457868
+0.0284346 -0.311464 0.0537885
+0.0409609 -0.0869988 -0.085353
+0.118462 -0.0465078 -0.0483438
+-0.141108 -0.422593 -0.0846889
+-0.129145 -0.379198 -0.0712284
+-0.149426 -0.329507 -0.0473837
+-0.212723 -0.300322 -0.0706413
+-0.156151 -0.403775 -0.0651516
+-0.077704 -0.400019 -0.0437424
+-0.0783601 -0.342543 -0.0687412
+-0.0202922 -0.309739 -0.0758353
+0.0501325 -0.290557 -0.0430839
+0.0954448 -0.256567 0.0733017
+0.144565 -0.158299 0.0913557
+0.0562207 -0.179498 -0.140905
+0.16863 -0.175124 -0.176771
+0.121523 -0.241503 -0.138152
+0.196749 0.136407 0.00365942
+0.14057 0.277481 0.154966
+0.156168 -0.385879 -0.0324162
+-0.146564 -0.288069 -0.168907
+-0.212533 -0.256019 -0.170893
+0.0775241 -0.353084 -0.0221894
+-0.161054 -0.48311 -0.0516067
+-0.151386 -0.488726 -0.0297486
+-0.167494 -0.44542 -0.0440266
+-0.166175 -0.416312 -0.0405929
+-0.155665 -0.409399 -0.0112037
+-0.130777 -0.428881 -0.00549876
+-0.162855 -0.460869 -0.0783452
+-0.0949104 -0.0628065 -0.118829
+-0.171364 -0.0681205 -0.104606
+-0.189411 -0.0391257 -0.0301772
+-0.201329 -0.0450332 0.0672375
+-0.0621659 -0.0595357 -0.0819357
+-0.0724451 -0.0386843 -0.0212262
+-0.0317016 -0.00827391 0.0812953
+0.125617 -0.445138 -0.086335
+0.16914 -0.45638 -0.0514974
+-0.188699 -0.102786 0.151505
+-0.127982 -0.406882 0.0143939
+-0.130706 -0.384 0.0373578
+-0.17253 -0.34721 0.0442322
+-0.136499 -0.357829 0.0655808
+-0.101729 -0.398953 0.0314689
+-0.0683606 -0.382507 0.0422398
+-0.0619729 -0.361557 0.0725343
+-0.0129886 -0.329893 0.099078
+-0.0485077 -0.306718 0.14972
+-0.0586671 -0.359537 -0.0297503
+-0.105132 -0.354471 0.0834642
+-0.111953 -0.324339 0.110194
+-0.146952 -0.298663 0.122985
+-0.147935 -0.259899 0.146585
+-0.224685 -0.230598 0.12386
+-0.194785 -0.274899 0.125615
+-0.1144 -0.250007 0.185149
+-0.10665 -0.185071 0.204627
+-0.155805 -0.154327 0.174517
+-0.215755 -0.165279 0.142947
+-0.0743299 -0.277016 0.17853
+-0.0218198 -0.257755 0.16445
+0.000800113 -0.199671 0.138749
+-0.0254547 -0.133829 0.140633
+0.0635315 -0.20852 0.111271
+-0.0256804 -0.0569419 0.140514
+-0.0948894 -0.0414189 0.110395
+-0.0705488 -0.0374163 0.0415111
+-0.00760713 -0.0195458 0.0164934
+0.0499878 0.0289775 0.0673286
+0.140466 0.0718009 0.144428
+-0.00188983 0.0792593 -0.003093
+0.0559872 -0.0140882 -0.011506
+-0.227407 -0.0888195 0.0117762
+-0.273528 -0.167121 -0.0102922
+-0.0402782 -0.263586 -0.140793
+-0.137564 -0.296782 -0.109847
+-0.166109 -0.381665 -0.0120175
+-0.16983 -0.360755 0.0137933
+-0.16456 -0.354211 -0.0308766
+-0.185156 -0.334386 0.0190529
+-0.207978 -0.298552 0.0310885
+-0.252507 -0.243462 0.0511336
+-0.223388 -0.268692 -0.019703
+-0.193938 -0.322649 0.048456
+-0.175095 -0.332532 0.0736524
+-0.176111 -0.310734 0.103586
+-0.147337 -0.494175 -0.0467278
+-0.129359 -0.490613 -0.0537016
+-0.148829 -0.364106 -0.0543963
+-0.120087 -0.343197 -0.0662243
+-0.130345 -0.305692 -0.0722136
+-0.171529 -0.299424 -0.0833938
+-0.181343 -0.292949 -0.0416997
+0.207831 -0.217496 -0.0966998
+0.180453 0.174523 0.151623
+0.0997558 -0.44202 -0.0211251
+0.141829 -0.427452 -0.0246256
+0.231902 -0.0518172 0.0262484
+0.222165 -0.00765217 0.131632
+0.257605 0.0468554 0.0496674
+0.134457 -0.362329 0.00455646
+0.162364 -0.298449 0.0119315
+0.167632 -0.334853 -0.0258239
+0.166781 -0.258058 -0.0451734
+0.196618 -0.203737 -0.0367216
+0.204966 -0.148702 0.029823
+0.188775 -0.0899398 0.0842549
+0.118735 -0.0897053 0.120666
+0.108518 -0.0623755 0.19827
+0.0943505 -0.134977 0.262277
+0.0987744 0.0186293 0.19077
+0.164162 -0.016078 0.17849
+0.217401 -0.0732307 0.174029
+0.244384 -0.159022 0.158735
+0.0938974 -0.414078 -0.0986619
+0.0882094 -0.386204 -0.0874415
+0.110237 -0.433985 -0.106491
+0.0752909 -0.377302 -0.0545033
+0.136893 -0.424401 -0.103861
+0.101623 -0.446793 -0.0879738
+0.0803589 -0.447974 -0.0828166
+0.0955794 -0.458697 -0.0690652
+0.0852254 -0.472952 -0.043714
+0.0423481 -0.131534 -0.117698
+0.0102534 -0.163639 -0.135844
+-0.0294235 -0.16777 -0.154302
+-0.0581903 -0.19678 -0.162941
+-0.0495263 -0.23206 -0.157295
+0.0213258 -0.204668 -0.142847
+0.0591534 -0.221375 -0.128414
+0.094939 -0.204956 -0.158559
+0.102368 -0.149902 -0.152017
+0.161838 -0.127189 -0.119413
+0.0883157 -0.255475 -0.0965547
+0.00337956 -0.243572 -0.138992
+-0.00441584 -0.28038 -0.109802
+0.139454 -0.244447 -0.0884262
+0.17799 -0.253819 -0.13106
+0.239512 -0.249475 -0.139746
+0.239769 -0.199663 -0.201225
+0.139899 -0.220537 -0.184787
+0.188104 -0.214287 -0.208834
+0.241446 -0.186766 -0.139849
+0.209446 -0.161297 -0.101029
+-0.199048 -0.317265 -0.100601
+-0.293252 -0.31357 -0.122798
+-0.243299 -0.355691 -0.130502
+-0.247594 -0.319033 -0.0988683
+-0.291668 -0.287409 -0.0637038
+-0.248832 -0.272185 -0.0865389
+-0.259174 -0.220646 -0.0469428
+-0.282126 -0.263379 -0.124759
+-0.269731 -0.294935 -0.18724
+-0.247297 -0.165961 -0.0866554
+0.0601914 -0.0464014 -0.0549033
+-0.211085 -0.18579 -0.128927
+-0.200721 -0.132547 -0.108259
+-0.0634953 -0.15489 0.179525
+0.153912 -0.0887253 -0.0733484
+0.185145 -0.0471144 -0.0260532
+0.211443 0.0144377 0.0032337
+0.139575 0.0029693 0.000737671
+0.166751 0.0710484 -0.0323169
+0.107842 0.107181 0.0410238
+0.241648 0.0800111 -0.0150955
+-0.140262 -0.4992 -0.033581
+-0.123763 -0.497978 -0.0364981
+-0.108133 -0.495253 -0.0202341
+-0.1219 -0.481038 -0.0158085
+-0.110456 -0.461331 -0.014079
+-0.087635 -0.441699 -0.0445804
+-0.0916102 -0.451765 -0.0239283
+-0.0811529 -0.421697 -0.020421
+-0.0723115 -0.470105 -0.0423155
+-0.102577 -0.44033 -0.0062072
+-0.139118 -0.480546 -0.0199347
+-0.149266 -0.466768 -0.0254061
+0.222317 -0.0925366 -0.0161067
+-0.0417433 -0.361447 0.00632817
+-0.00965295 -0.347119 0.0218202
+0.0216724 -0.318231 0.010836
+-0.00474667 -0.341486 0.0613973
+0.0698488 -0.285166 0.0205835
+-0.108371 -0.29389 -0.0941984
+-0.11094 -0.279255 -0.127432
+-0.0852691 -0.31423 -0.083395
+-0.0516331 -0.309713 -0.0908278
+-0.0463843 -0.329032 -0.0615785
+-0.13427 -0.295211 -0.140976
+-0.16029 -0.312528 -0.151535
+-0.208046 -0.313472 -0.188061
+-0.177638 -0.29982 -0.178676
+-0.173332 -0.259819 -0.179118
+-0.186949 -0.225139 -0.161631
+-0.121077 -0.271737 -0.158365
+-0.222098 -0.229029 -0.143015
+-0.254895 -0.254361 -0.158387
+-0.119307 -0.241249 -0.168619
+-0.0940058 -0.224454 -0.161034
+-0.119452 -0.213541 -0.164903
+-0.133444 -0.181194 -0.153714
+-0.123212 -0.142937 -0.146434
+-0.154852 -0.114002 -0.127562
+-0.16919 -0.193253 -0.151916
+-0.202578 -0.281459 -0.18846
+-0.238459 -0.276537 -0.185897
+-0.240282 -0.308392 -0.197824
+-0.31592 -0.411023 -0.222975
+-0.28556 -0.354091 -0.210885
+-0.235315 -0.349789 -0.176342
+-0.247249 -0.413591 -0.205119
+-0.313616 -0.39085 -0.145549
+-0.265064 -0.38952 -0.162643
+-0.137308 -0.0765217 -0.126585
+-0.138269 -0.043252 -0.101811
+-0.149787 -0.024537 -0.0569204
+-0.190238 -0.325155 -0.164883
+-0.214126 -0.342005 -0.145019
+-0.11169 -0.0249744 -0.0351987
+-0.154494 -0.0206052 -0.0150451
+-0.170024 -0.0235376 0.033341
+-0.149964 -0.0212297 0.0915499
+-0.146292 -0.0570396 0.143171
+-0.204533 -0.050976 0.0222898
+-0.222839 -0.0748301 0.0497298
+-0.218112 -0.0836084 0.106186
+-0.241131 -0.10804 0.0423425
+-0.253857 -0.126023 0.000914005
+-0.268045 -0.155579 0.0359862
+-0.254096 -0.191499 0.0837602
+-0.284127 -0.206805 0.0257642
+-0.255388 -0.139415 -0.0455141
+-0.266486 -0.179542 -0.0496151
+-0.255448 -0.200689 -0.0776309
+-0.238672 -0.192995 -0.107883
+-0.225097 -0.163824 -0.109937
+-0.22995 -0.135947 -0.0896828
+-0.209528 -0.114818 -0.0862766
+-0.197715 -0.0771741 -0.074156
+-0.18152 -0.102481 -0.104911
+-0.212552 -0.0754645 -0.0336012
+-0.179069 -0.0426745 -0.0757348
+-0.191113 -0.0400122 0.118805
+0.0761558 -0.344791 -0.0536197
+0.0734921 -0.316433 -0.0291825
+0.0865869 -0.298841 -0.0691012
+0.0805876 -0.328744 0.0029047
+0.0952029 -0.361348 0.00820626
+0.11906 -0.273469 -0.0673767
+0.0841053 -0.310717 0.0264162
+0.125408 -0.292597 0.041066
+0.0878905 -0.285295 0.052224
+0.0491288 -0.272854 0.0806291
+0.143869 -0.242882 0.0528378
+0.117789 -0.207199 0.0870858
+0.169353 -0.193132 0.0552639
+0.0969256 -0.166289 0.115505
+0.054006 -0.288058 0.0505697
+0.0199581 -0.301066 0.0971587
+-0.00766116 -0.296894 0.132807
+0.0766599 -0.302889 0.00259846
+0.0459981 -0.299327 0.00649094
+0.0275534 -0.307808 -0.0222832
+0.0149271 -0.300169 -0.0594622
+0.0407712 -0.276916 -0.0798141
+0.0598138 -0.291021 -0.0166231
+0.105499 -0.310063 0.042044
+0.129965 -0.318676 0.0271739
+-0.0843085 -0.494264 -0.0321529
+-0.0752323 -0.238787 0.196776
+-0.295928 -0.43397 -0.176216
+0.0851801 -0.382062 -0.0130588
+0.0187394 -0.0952484 0.145146
+0.0700063 -0.10227 0.135872
+0.0221841 -0.0461712 0.144279
+0.0234739 0.0145751 0.123876
+-0.00955997 -0.0145334 0.135796
+-0.0455413 -0.0274983 0.116817
+-0.0630042 -0.0646849 0.123712
+-0.0996182 -0.0685448 0.139479
+-0.129134 -0.0937854 0.159793
+-0.112763 -0.133107 0.183753
+-0.0586283 -0.0384282 0.0801443
+-0.0976008 -0.0306336 0.0712047
+-0.187313 -0.236737 0.146886
+-0.186919 -0.194456 0.158247
+-0.276732 -0.200888 -0.0224537
+-0.291326 -0.23573 -0.0313163
+-0.262172 -0.26119 -0.0228289
+-0.244304 -0.258186 0.0138536
+-0.238049 -0.253808 -0.053252
+-0.278468 -0.245353 0.0237178
+-0.250374 -0.233381 -0.0762153
+-0.317786 -0.266287 -0.0397057
+-0.29694 -0.227134 0.00135872
+-0.28761 -0.282597 -0.0302299
+-0.0768305 -0.203891 0.202176
+-0.107975 -0.220055 0.202264
+-0.134773 -0.20066 0.190676
+-0.13214 -0.167949 0.192848
+-0.157713 -0.187173 0.17141
+-0.0792541 -0.17391 0.197354
+-0.103166 -0.157466 0.196315
+-0.0861693 -0.139966 0.183699
+-0.0642112 -0.126048 0.16286
+-0.0525585 -0.0978366 0.139755
+0.173523 -0.0454835 0.124705
+0.124762 -0.0100876 0.132612
+0.0801162 0.0231847 0.117816
+0.0903158 0.0691509 0.0824377
+0.111706 0.138719 0.113497
+0.109897 0.0476799 0.0291314
+0.0568194 0.0789592 0.018431
+0.166721 0.186565 0.0672199
+0.252858 0.160254 0.0700128
+0.103948 0.0891733 -0.0142249
+0.151331 0.103958 0.00831307
+0.156258 0.148984 0.0332697
+0.195526 0.176469 0.0301312
+0.246249 0.159404 0.0147221
+0.220496 0.208803 0.0718273
+0.284857 0.213191 0.163319
+0.220004 0.262975 0.168971
+0.243752 0.187223 0.124992
+0.303317 0.156118 0.132187
+0.124557 0.160014 0.070217
+0.145476 0.178627 0.113721
+0.143822 0.145597 0.146983
+0.199699 0.112576 0.148784
+0.221259 0.0552492 0.134196
+0.124553 0.109697 0.139987
+0.100062 0.0751807 0.125633
+0.107686 0.0453443 0.15701
+0.152453 0.0251604 0.154394
+0.160741 0.1045 0.158385
+0.183054 0.0708926 0.1447
+0.188656 0.0314673 0.13741
+0.044874 0.0868553 -0.0397086
+0.0292084 0.0351428 -0.0773123
+-0.00652383 0.0868322 -0.0563984
+0.153395 -0.330946 0.00521793
+0.165687 0.227811 0.159326
+0.176713 -0.253714 -0.180764
+0.276141 0.126578 0.0974557
+-0.0329659 -0.0648403 -0.0508016
+-0.022748 -0.0460692 -0.0136176
+-0.0397109 -0.0394184 0.0195973
+0.00993129 -0.0278328 -0.0155697
+0.0270531 -0.00832198 0.0106523
+0.0103826 0.00500549 0.0538795
+0.0666135 0.0125544 0.0261568
+0.103369 0.00889595 0.155654
+0.11659 -0.0298082 0.170544
+0.153029 -0.0696813 0.168481
+0.113461 -0.0186162 0.216944
+0.10035 -0.0580327 0.251516
+0.150235 -0.084971 0.233939
+0.0675669 -0.0946043 0.253879
+0.0719166 -0.0391569 0.214813
+0.10687 -0.103697 0.22877
+0.136543 -0.144876 0.235107
+0.147147 -0.0351617 0.149371
+0.1197 -0.0491892 0.122959
+0.148505 -0.0696344 0.112799
+0.152678 -0.114026 0.101195
+0.181641 -0.136376 0.0700095
+0.177794 -0.0667748 0.102185
+0.219377 -0.0856118 0.124697
+0.231869 -0.0572983 0.0801841
+0.250938 -0.00881912 0.0578761
+0.198815 -0.0628459 0.124187
+0.177363 -0.0676828 0.147605
+0.177944 -0.101552 0.169756
+0.213844 -0.114113 0.199683
+0.250347 -0.109904 0.165803
+0.259893 -0.130195 0.122038
+0.180633 -0.0723966 0.203599
+0.114076 -0.0955963 0.272717
+0.201886 -0.0333539 0.161782
+0.240288 -0.0485439 0.141813
+0.261312 -0.0227503 0.104643
+0.273982 -0.0593402 0.119899
+0.280362 -0.074792 0.0701015
+0.238595 0.0177583 0.0951404
+0.14643 -0.0478943 0.212489
+-0.227331 -0.265687 0.0979085
+-0.244646 -0.230401 0.0887962
+-0.293688 -0.268968 -0.164669
+-0.297979 -0.308788 -0.162348
+-0.312868 -0.346598 -0.144794
+-0.344082 -0.367401 -0.192127
+-0.317302 -0.337357 -0.184462
+-0.280828 -0.350074 -0.123607
+0.105148 0.105807 0.112878
+0.112708 0.122082 0.0774818
+0.133565 0.128779 0.0503619
+-0.153371 -0.370618 0.0324641
+0.254239 -0.0926281 0.0998171
+-0.0128437 0.0136567 0.10826
+0.0175667 0.0217155 0.0871281
+0.0477136 0.0340255 0.104217
+0.0750182 0.0489044 0.100742
+0.0776037 0.0433948 0.0700673
+0.0973389 0.0603657 0.0539823
+0.0861821 0.0686274 0.0252903
+0.0735784 0.0589072 -0.0094551
+0.0811061 0.0911963 0.0569078
+0.0940457 0.0476479 0.121838
+0.105428 0.0231317 0.131003
+0.0916425 -0.00738665 0.126455
+0.0604145 5.34629e-05 0.128347
+0.10359 0.0595398 0.00049955
+0.144344 0.0457444 0.00327488
+0.122523 0.0419513 -0.0300499
+0.11811 0.0162342 -0.0830375
+0.170091 0.0155571 -0.145681
+0.138389 0.0626357 -0.0743287
+0.165069 0.0235027 -0.0532363
+0.177768 0.0409007 -0.100742
+0.136707 0.0380317 -0.122381
+0.124141 0.00775387 -0.157586
+0.154959 -0.000810753 -0.105169
+0.105591 0.0562623 -0.100272
+0.0594242 0.0548004 -0.106957
+0.201162 -0.0168583 -0.120783
+0.0976411 0.0972697 0.0801317
+0.0943337 0.0848541 0.102457
+0.0890479 0.0650811 0.109825
+0.0389773 0.0745551 -0.0779593
+0.0138551 0.0593589 -0.108474
+0.0773146 0.0765993 -0.0629692
+0.118409 0.00125651 -0.11931
+0.14578 -0.0101392 -0.137264
+0.178622 -0.0192175 -0.148664
+0.15461 0.0388676 -0.0266866
+0.193622 0.0322602 -0.0272748
+0.1942 0.050466 -0.0642106
+0.173634 0.0303747 -0.00069076
+0.179101 -0.0110744 -0.00523936
+0.230507 0.0422098 -0.0173425
+0.202418 0.070871 -0.0135348
+0.22302 0.0184239 -0.0441485
+0.215157 -0.0248187 0.00288885
+0.233855 -0.00344678 0.0259658
+0.238719 0.033163 0.0205233
+0.25481 0.0672767 0.0179763
+0.222129 -0.0581209 -0.00731524
+0.227145 -0.0819987 0.0145053
+0.219253 -0.118982 0.0109807
+0.211654 -0.158762 -0.0192499
+0.210611 -0.101269 0.0490966
+0.200451 -0.190104 0.0116714
+0.203938 -0.0712158 -0.0282028
+0.173774 -0.0735362 -0.0475631
+0.2027 -0.10102 -0.0400541
+0.212795 -0.129107 -0.0298945
+0.206091 -0.151217 -0.0455168
+0.202467 -0.150775 -0.0745887
+0.209378 -0.187531 -0.0743719
+0.181149 -0.126003 -0.0892617
+0.205972 -0.177843 -0.0383337
+0.188301 -0.142819 -0.114464
+0.208779 -0.164491 -0.140437
+0.171227 -0.151151 -0.147387
+0.136617 -0.138704 -0.143839
+0.131975 -0.166643 -0.169617
+0.128263 -0.113939 -0.11894
+0.116875 -0.0858965 -0.091059
+0.0786764 -0.0829677 -0.090746
+0.187859 -0.214057 -0.0686071
+0.175679 -0.237132 -0.0957975
+0.178163 -0.10588 -0.0665832
+0.269129 0.0763722 0.0451229
+0.27889 0.058723 0.0679827
+0.267652 0.0453551 0.105512
+0.2613 0.0363273 0.0775663
+0.250278 0.0199214 0.0566543
+0.250269 0.00638094 0.0769677
+0.262967 -0.0160504 0.0796628
+0.285344 -0.0433845 0.0872574
+0.24971 -0.0386027 0.0640826
+0.229012 -0.0528564 0.0533874
+0.210073 -0.0746435 0.0657137
+0.221889 -0.0775407 0.0410909
+0.302572 -0.0710887 0.0949218
+0.236871 -0.0290995 0.0413907
+0.21774 0.0486965 -0.0445342
+0.224846 0.0339602 -0.0740774
+0.244042 0.0044793 -0.10874
+0.254336 0.102378 0.0100739
+0.220262 0.108437 -0.00385435
+0.184431 0.103867 -0.0063665
+0.227766 0.138272 0.00117268
+0.21603 0.168687 0.00560537
+0.214962 0.252952 4.36931e-05
+0.24674 0.211475 0.00674743
+0.208103 0.206716 0.00898043
+0.198252 0.239152 0.0425261
+0.241235 0.183275 -0.00372162
+0.24846 0.187657 0.0253371
+0.236173 0.217937 0.0406376
+0.208251 0.202717 0.0430183
+0.190765 0.204953 0.0676283
+0.199822 0.228771 0.0907818
+0.212055 0.262964 0.121847
+0.167888 0.20936 0.0941323
+0.230768 0.216285 0.106419
+0.226441 0.220307 0.1523
+0.160904 0.217518 0.126833
+0.151535 0.251202 0.133586
+0.158786 0.306935 0.0984538
+0.193903 0.396086 0.195106
+0.119767 0.364922 0.154966
+0.158434 0.284691 0.125614
+0.188265 0.327264 0.175224
+0.139712 0.323472 0.144742
+0.247398 0.226772 0.208026
+0.162628 0.357991 0.158091
+0.132463 0.334186 0.215838
+0.184892 0.418747 0.128073
+0.148158 0.405567 0.1589
+0.116952 0.392105 0.208798
+0.128904 0.307582 0.18276
+0.173163 0.280669 0.197839
+0.237808 0.190239 0.0542196
+0.242415 0.187695 0.0885314
+0.257589 0.159811 0.105872
+0.25799 0.149475 0.1552
+0.249199 0.167605 0.0439231
+0.270589 0.141138 0.0478035
+0.291007 0.120776 0.0668204
+0.27986 0.0943457 0.0648346
+0.262028 0.128688 0.0220064
+0.283888 0.104244 0.08796
+0.278117 0.09277 0.114653
+0.296211 0.119268 0.134225
+0.25745 0.0686993 0.129638
+0.231499 0.0874258 0.144645
+0.23624 0.121986 0.150481
+0.21822 0.158716 0.152133
+0.114064 0.342002 0.184102
+0.184346 0.361594 0.187357
+0.161147 0.378013 0.229421
+0.186402 0.32651 0.232026
+0.224199 0.278314 0.231623
+0.178684 0.387524 0.165454
+0.207571 0.416517 0.162223
+0.115261 0.360345 0.210089
+0.111327 0.375408 0.183082
+0.123148 0.405595 0.17979
+0.161317 0.441861 0.200022
+0.134075 0.42215 0.208124
+0.166581 0.413272 0.221996
+0.184888 0.467555 0.155941
+0.139501 0.435526 0.173911
+0.158717 0.435906 0.144337
+0.167498 0.45215 0.103084
+0.137763 0.359978 0.22921
+0.165611 0.348074 0.227085
+0.15984 0.318857 0.217125
+0.185554 0.304169 0.211919
+0.205103 0.278784 0.204981
+0.244841 0.279219 0.196675
+0.226891 0.25345 0.209087
+0.216714 0.31142 0.225032
+0.18164 0.33693 0.20427
+0.203702 0.303717 0.18726
+0.193627 0.296232 0.146589
+0.191637 0.257284 0.18429
+0.157803 0.257523 0.177293
+0.175634 0.311565 0.125155
+0.161107 0.457828 0.173386
+0.194169 0.447882 0.185268
+0.194408 0.477711 0.118315
+0.219599 0.449898 0.13857
+0.135294 0.387364 0.228733
+0.145147 0.282789 0.187345
+0.143956 0.303159 0.203618
+0.177446 0.366384 0.212315
+0.179329 0.391491 0.217312
+0.18672 0.412483 0.210145
+0.202606 0.421792 0.189409
+0.174085 0.42914 0.210627
+0.153641 0.426813 0.21312
+0.144424 0.408706 0.223131
+0.190464 0.428693 0.201747
+0.178837 0.441581 0.19921
+0.169704 0.452082 0.187978
+0.180705 0.459908 0.173619
+0.202088 0.458198 0.166952
+0.151672 0.449754 0.186725
+0.142475 0.436806 0.198199
+0.127991 0.425256 0.188222
+0.210178 0.437176 0.172385
+0.119636 0.410278 0.199562
+0.206557 0.46857 0.145869
+0.215386 0.468035 0.123827
+0.207969 0.436894 0.106764
+0.212477 0.477963 0.10116
+0.128023 0.4054 0.215288
+0.223073 0.454408 0.0921651
+0.184017 0.321535 0.149004
+0.164058 0.339436 0.133054
+0.141763 0.356765 0.138374
+0.138767 0.331395 0.111494
+0.18387 0.5 0.0894472
+0.178253 0.341112 0.15974
+0.158692 0.397227 0.229447
+0.261987 0.258572 0.22454
+0.209557 0.191204 0.156985
+0.19703 0.220129 0.168363
+0.114158 0.395299 0.191384
+0.237163 0.239354 0.0176536
+0.22873 0.228755 -0.0079498
+0.204237 0.229854 -0.000985107
+0.191898 0.248869 0.0164149
+0.195144 0.299533 0.0501168
+0.211408 0.277777 0.0261684
+0.181525 0.269092 0.042102
+0.232338 -0.0296485 0.018274
+0.220704 0.448894 0.117889
+0.211836 0.430361 0.128709
+0.200007 0.22745 0.0221073
+0.206622 0.221816 0.0395427
+0.223655 0.239447 0.0484776
+0.206846 0.263872 0.061817
+0.192412 0.289403 0.0880222
+0.200823 -0.0669613 0.089238
+0.1998 0.487191 0.0879627
+0.178418 0.478766 0.0726805
+0.171907 0.478 0.098484
+0.179607 0.4432 0.070784
+0.215421 0.44036 0.0572748
+0.206111 0.438517 0.081171
+0.184196 0.432818 0.0953164
+0.172787 0.433405 0.115219
+0.16681 0.454422 0.128591
+0.158037 0.463512 0.080819
+0.300027 -0.106441 0.0798503
+0.301565 -0.138608 0.110073
+0.223785 -0.0676095 0.104371
+0.244386 -0.0720528 0.0937201
+0.25624 -0.0594904 0.0758547
+0.271472 -0.0437947 0.0690266
+0.283677 -0.0568988 0.0744319
+0.294284 -0.0698767 0.0795338
+0.293906 -0.0887216 0.0736412
+0.275743 -0.101071 0.0877101
+0.309738 -0.0894098 0.0874741
+0.287014 -0.123527 0.0945964
+0.311125 -0.118131 0.0991123
+0.298899 -0.117613 0.118193
+0.276193 -0.109289 0.135451
+0.270493 -0.137621 0.153632
+0.286312 -0.136798 0.132025
+0.257826 -0.0797919 0.144654
+0.303792 -0.0864306 0.104131
+0.28817 -0.0747287 0.113826
+0.276632 -0.0878111 0.128274
+0.30695 -0.103289 0.107245
+0.287682 -0.0559229 0.10414
+0.272738 -0.0407993 0.108633
+0.253435 -0.0365139 0.124117
+0.295415 -0.0573592 0.0898843
+0.270105 -0.0629299 0.0693418
+0.263375 -0.0783235 0.0820667
+-0.216962 -0.20051 0.140681
+-0.236047 -0.180496 0.115013
+-0.230292 -0.136771 0.112494
+-0.247625 -0.159292 0.0914591
+-0.209896 -0.129892 0.141237
+0.222682 0.288177 0.190351
+0.233778 0.296853 0.212089
+0.209365 0.287061 0.167624
+0.208618 0.271128 0.146657
+0.223776 0.247151 0.14266
+0.250594 0.280585 0.221005
+0.215525 0.238928 0.164607
+0.248724 0.243405 0.176599
+0.282927 0.23674 0.194658
+0.253819 0.208937 0.178714
+0.265424 0.184312 0.155732
+0.308614 0.169998 0.183237
+0.273365 0.179729 0.192265
+0.265912 0.204786 0.204674
+0.300509 0.203464 0.194384
+0.281969 0.151677 0.177526
+0.279246 0.127373 0.158699
+0.31079 0.143896 0.162421
+0.30954 0.171009 0.155179
+0.288086 0.1804 0.137744
+0.264265 0.175268 0.132545
+0.241184 0.168571 0.143536
+0.282052 0.162312 0.123541
+0.290218 0.138764 0.11928
+0.232003 0.191432 0.146429
+0.237199 0.211128 0.131059
+0.175486 0.13591 0.157401
+0.244193 0.0453066 0.121153
+0.216838 0.0295154 0.115567
+0.0778181 0.0182774 -0.0959304
+0.132697 0.385267 0.165833
+0.155812 0.38306 0.160495
+-0.00373338 0.0386319 -0.0871428
+0.0052284 0.0508015 -0.0521262
+-0.0272532 0.0521944 -0.0650671
+-0.0417118 0.0760468 -0.0274796
+0.0432101 0.0478592 -0.0430105
+0.0360437 0.064037 -0.0095129
+0.0264403 0.0878588 0.0105855
+0.0200841 0.0963175 -0.0204482
+0.0508265 0.0939603 -0.0091335
+0.0753367 0.087282 -0.0290458
+-0.0114666 0.0989277 -0.0268583
+0.189464 0.426182 0.111762
+-0.04038 -0.0265907 0.0536548
+0.188037 0.19051 0.048384
+0.170897 0.170857 0.0404072
+0.180803 0.154042 0.0195245
+0.168583 0.128396 0.0100026
+0.150344 0.161847 0.0580756
+0.146195 0.173828 0.0846654
+0.123104 0.163389 0.100752
+0.131952 0.158423 0.126057
+0.154039 0.169296 0.137953
+0.163282 0.191526 0.127822
+0.170691 0.206066 0.147249
+0.123979 0.136658 0.135
+0.136161 0.125537 0.148878
+0.153818 0.131557 0.161379
+0.111897 0.12133 0.126074
+0.111889 0.144276 0.0890707
+0.11658 0.140768 0.0690434
+0.119959 0.124948 0.0613596
+0.107779 0.107117 0.0626571
+0.122618 0.115267 0.0466942
+0.127454 0.104238 0.0219653
+0.136258 0.119892 0.0320023
+0.129073 0.0915077 -0.00265103
+0.130797 0.0780035 -0.0369633
+0.10768 0.094992 0.00979378
+0.163926 0.154671 0.152149
+0.0894836 0.0909923 0.00058556
+0.0689505 0.0963924 0.00171312
+0.0612997 0.100634 0.0224348
+0.0675451 0.0846698 0.038694
+0.0795109 0.103357 0.0384133
+0.0848094 0.0754581 0.0444653
+0.110567 0.10366 0.130086
+0.12281 0.0864143 0.139975
+0.117855 0.062854 0.143513
+0.13666 0.0472165 0.155281
+0.128164 0.0235742 0.176647
+0.163067 0.0498951 0.143567
+0.143932 0.0949004 0.145284
+0.179917 0.317727 0.0742859
+0.183725 0.275085 0.0676723
+0.16838 0.29297 0.0787056
+0.0930951 0.102542 0.05002
+0.100339 0.0681106 0.0373411
+0.102886 0.0622715 0.0197467
+0.121511 0.0540863 0.0117598
+0.124719 0.0242285 0.0166428
+0.0967861 -0.00310471 -0.0020113
+0.12138 0.0519179 -0.0102922
+0.0990646 0.0492208 -0.022422
+0.0873807 -0.0275369 -0.03209
+0.200694 -0.191636 -0.0546067
+0.206298 -0.170055 -0.0598788
+0.209964 -0.168421 -0.0791806
+0.221182 -0.183261 -0.0963771
+0.222775 -0.172837 -0.120159
+0.235715 -0.195921 -0.115182
+0.253933 -0.218526 -0.134037
+0.311213 -0.253911 -0.191866
+0.279294 -0.244732 -0.16099
+0.266185 -0.201338 -0.169529
+0.285572 -0.216415 -0.213382
+0.273765 -0.285731 -0.187819
+0.259679 -0.265381 -0.248632
+0.24894 -0.227823 -0.231771
+0.232153 -0.25966 -0.225227
+0.254118 -0.290735 -0.220386
+0.336364 -0.328047 -0.241676
+0.281317 -0.319577 -0.26697
+0.295033 -0.317038 -0.218433
+0.327766 -0.263669 -0.27537
+0.320681 -0.238904 -0.235706
+0.333487 -0.28367 -0.222752
+0.25789 -0.299076 -0.25318
+0.280382 -0.278404 -0.287734
+0.262726 -0.334272 -0.234674
+0.315714 -0.303377 -0.28762
+0.358898 -0.298323 -0.270079
+0.292961 -0.250812 -0.263064
+0.260427 0.269097 0.206442
+0.273912 0.251948 0.207483
+0.274266 0.226866 0.218876
+0.254508 0.262332 0.186312
+0.268737 0.247011 0.182676
+0.278883 0.230014 0.174886
+0.292676 0.216891 0.181537
+0.301666 0.196568 0.170643
+0.235991 0.25839 0.179999
+0.216996 0.262302 0.191107
+0.233602 0.240223 0.192553
+0.26623 0.217767 0.166603
+0.291208 0.219735 0.206357
+0.285626 0.200003 0.208179
+0.295054 0.181857 0.198439
+-0.119559 -0.0454446 0.130205
+-0.148541 -0.0288065 0.124554
+-0.122421 -0.0280036 0.104512
+-0.169628 -0.0428483 0.136658
+-0.192691 -0.0700149 0.138018
+-0.165949 -0.0805689 0.151606
+-0.157867 -0.111652 0.162619
+-0.182289 -0.134815 0.160033
+-0.171616 -0.0265274 0.119564
+-0.182821 -0.0294707 0.089096
+-0.207158 -0.0941133 0.130893
+-0.0813687 -0.408143 0.0168626
+-0.0493082 -0.255289 0.183439
+-0.0417823 -0.281988 0.16825
+-0.0232624 -0.242141 -0.150317
+0.237084 0.148575 0.150278
+0.21825 0.135883 0.152701
+0.196547 0.147262 0.152063
+0.248839 0.134889 0.149793
+0.25417 0.116897 0.146677
+0.154738 -0.248351 -0.113516
+0.149894 -0.252291 -0.14374
+0.127807 -0.247316 -0.112579
+0.100037 -0.239188 -0.118127
+0.171952 -0.258325 -0.155783
+0.206243 -0.267544 -0.164319
+0.152779 -0.244265 -0.170212
+0.238869 -0.271194 -0.164355
+0.19948 -0.240785 -0.114164
+0.228533 -0.228656 -0.117975
+0.0806348 -0.448964 -0.0364622
+0.092817 -0.46403 -0.0236687
+0.131465 -0.464547 -0.0186627
+0.111576 -0.45856 -0.0162136
+0.12236 -0.437795 -0.0167687
+0.116113 -0.473014 -0.0333379
+0.141834 -0.466374 -0.0462667
+0.15629 -0.45187 -0.0265272
+0.162053 -0.430562 -0.0436087
+0.170805 -0.433786 -0.074571
+0.150694 -0.440322 -0.089161
+0.142403 -0.453672 -0.0687084
+0.20922 0.0225383 -0.118012
+0.245897 0.00269769 -0.0710137
+-0.0868896 -0.445579 -0.0827631
+-0.0899978 -0.418668 -0.0662628
+-0.0919895 -0.382051 -0.0731611
+-0.286076 -0.18977 0.00251657
+0.166397 -0.235956 -0.0665238
+0.18289 -0.231659 -0.0402536
+0.183601 -0.256036 -0.0114407
+0.19304 -0.222737 -0.0114233
+0.168396 -0.264129 0.0198747
+0.175145 -0.292863 -0.0261367
+0.159612 -0.311932 -0.0502102
+0.151795 -0.349231 -0.058414
+0.168467 0.120276 0.165442
+0.179322 0.109989 0.151163
+0.191745 0.091503 0.1476
+0.207409 0.0731218 0.143583
+0.170472 0.088013 0.148441
+0.198308 0.0526608 0.137187
+-0.288444 -0.322548 -0.196751
+-0.258254 -0.336596 -0.201797
+-0.260706 -0.370334 -0.191889
+-0.262012 -0.38355 -0.234182
+0.169409 0.331718 0.106522
+-0.0883279 -0.427369 -0.00320489
+-0.0757242 -0.410706 -0.00350047
+-0.0694098 -0.396348 -0.0215868
+-0.339105 -0.28249 -0.133907
+0.14338 -0.190029 -0.185968
+0.113197 -0.189729 -0.178573
+0.187542 -0.24244 -0.20457
+0.214342 -0.231174 -0.221761
+0.200695 -0.263551 -0.191549
+0.0888974 -0.174918 -0.165344
+0.0728578 -0.155488 -0.148655
+0.0857975 -0.13271 -0.133069
+0.0496654 -0.153477 -0.133288
+0.208417 -0.252602 -0.211864
+0.214499 -0.20684 -0.209109
+0.212326 -0.182246 -0.176825
+0.196622 -0.193456 -0.194925
+-0.0366034 0.0848157 -0.0747335
+-0.0106036 0.0767347 -0.0825468
+-0.248014 -0.143811 -0.0711582
+0.156176 -0.353723 -0.00967102
+0.161881 -0.35946 -0.0354879
+0.154192 -0.374021 -0.054565
+0.153835 -0.401954 -0.0551512
+0.147106 -0.376782 -0.0111704
+0.141013 -0.401853 -0.0175381
+0.127378 -0.38782 -0.00428773
+0.152558 -0.410563 -0.0345712
+0.144573 -0.387551 -0.0699167
+0.129797 -0.395951 -0.0860393
+0.110844 -0.383365 -0.0877166
+0.111358 -0.362136 -0.0828181
+0.10863 -0.332992 -0.0757964
+0.131091 -0.348484 -0.0736181
+0.114528 -0.372564 0.00601769
+0.116893 -0.350867 0.0177725
+0.143657 -0.369483 -0.0686154
+0.0433039 -0.239647 0.0998892
+-0.318832 -0.357055 -0.211401
+-0.299837 -0.377374 -0.238049
+-0.340344 -0.383626 -0.224893
+-0.356366 -0.419986 -0.188103
+-0.285529 -0.404192 -0.228972
+-0.356375 -0.393121 -0.201407
+-0.349321 -0.392013 -0.165399
+-0.328811 -0.42272 -0.163607
+-0.345548 -0.417553 -0.216974
+-0.322795 -0.427865 -0.195551
+-0.33518 -0.363207 -0.161311
+-0.0812327 -0.396788 0.0342935
+-0.065289 -0.38943 0.0224745
+-0.0508718 -0.371639 0.0298172
+-0.260668 -0.216401 0.0653687
+-0.2704 -0.185201 0.0538295
+0.187032 0.486356 0.0996338
+0.279593 -0.136382 0.110973
+0.26837 -0.117918 0.105466
+0.248285 -0.109463 0.116456
+0.232397 -0.125931 0.141691
+0.137574 -0.108705 0.207737
+0.169921 0.29167 0.0522744
+0.00992085 -0.113945 -0.0964453
+0.258261 -0.257702 -0.154872
+0.275321 -0.267702 -0.17038
+0.295633 -0.272896 -0.184932
+0.295815 -0.294739 -0.20199
+0.314713 -0.27743 -0.201437
+0.327332 -0.256197 -0.214678
+0.340385 -0.261017 -0.241446
+0.313356 -0.232789 -0.209684
+0.296344 -0.226073 -0.182212
+0.31592 -0.301772 -0.216769
+0.318635 -0.326842 -0.222796
+0.307178 -0.325937 -0.251416
+0.274745 -0.303144 -0.21295
+0.263287 -0.308346 -0.232527
+0.255382 -0.319889 -0.248754
+0.332002 -0.310475 -0.229091
+0.353636 -0.307458 -0.245121
+0.335859 -0.313851 -0.265914
+0.340499 -0.300896 -0.286697
+0.344989 -0.280039 -0.276203
+0.327449 -0.28026 -0.295194
+0.304942 -0.268737 -0.28762
+0.34983 -0.283684 -0.251927
+0.343136 -0.265469 -0.261534
+0.326713 -0.248963 -0.256716
+0.307917 -0.24034 -0.252462
+0.279159 -0.231625 -0.241522
+0.300353 -0.229376 -0.234338
+0.311358 -0.251222 -0.26759
+0.299024 -0.289572 -0.301481
+0.279907 -0.305571 -0.290358
+0.263002 -0.29286 -0.276914
+0.260616 -0.313748 -0.268771
+0.275922 -0.322371 -0.224156
+0.284975 -0.33259 -0.244221
+0.303023 -0.336946 -0.233483
+0.289605 -0.333838 -0.221905
+0.166435 0.39886 0.154509
+0.189152 0.404825 0.153461
+0.197133 0.400723 0.1737
+0.17424 0.413608 0.143911
+0.169142 0.426834 0.132439
+0.188725 0.386035 0.180698
+0.186219 0.378081 0.198788
+-0.267025 -0.372115 -0.138391
+-0.248022 -0.367522 -0.158602
+0.174505 0.292586 0.0990952
+0.186251 0.309263 0.0928235
+0.181697 0.30513 0.109614
+0.189499 0.279776 0.119317
+0.172647 0.294457 0.120787
+0.158057 0.303956 0.130265
+0.143476 0.291331 0.139139
+0.131312 0.30031 0.159002
+0.186095 0.298863 0.131041
+0.198975 0.28677 0.132576
+0.172236 0.277192 0.117866
+0.184313 0.260452 0.109718
+0.145983 0.269471 0.137935
+0.151234 0.256355 0.155644
+0.124293 0.323596 0.164277
+0.126678 0.342635 0.150186
+0.129524 0.341544 0.128525
+0.146576 0.349856 0.118779
+0.192269 0.30374 0.0759625
+0.202728 0.285813 0.0690781
+0.210479 0.281567 0.0483558
+0.222196 0.26209 0.0407369
+0.224073 0.261141 0.0186008
+0.162756 0.306191 0.114529
+0.149682 0.318744 0.121136
+0.152677 0.341974 0.100993
+0.164071 0.331208 0.0841361
+0.146523 0.323278 0.093798
+0.15695 0.312461 0.0714443
+0.22266 -0.203518 -0.100877
+0.262076 -0.292278 -0.202356
+0.249754 -0.28212 -0.183581
+0.242451 -0.284733 -0.203885
+0.229725 -0.273053 -0.211906
+0.243386 -0.275428 -0.225419
+0.255999 -0.283472 -0.239546
+-0.261873 -0.405224 -0.222848
+-0.280772 -0.420055 -0.201182
+-0.274389 -0.414773 -0.173401
+-0.298626 -0.411672 -0.158377
+-0.28738 -0.389898 -0.148508
+-0.300008 -0.369107 -0.135836
+-0.257381 -0.39915 -0.185758
+-0.257531 -0.421709 -0.186727
+-0.319983 -0.369357 -0.146016
+-0.25404 -0.39271 -0.206532
+-0.269186 -0.375948 -0.213104
+0.171129 0.312683 0.0555484
+0.186538 0.309469 0.0612947
+0.17697 0.322584 0.0897939
+0.180094 0.317456 0.102692
+0.0389455 -0.294719 0.0707663
+0.19085 0.129482 0.148059
+0.26893 -0.330546 -0.250405
+0.261495 -0.324416 -0.260179
+0.163955 0.0845671 -0.00775852
+0.172992 0.467003 0.114773
+0.17962 0.47069 0.135115
+0.167392 0.460661 0.148013
+0.0927702 -0.0102964 0.178794
+0.0791092 -0.00358862 0.211868
+0.0484002 -0.0727004 0.143042
+0.0857054 -0.0664246 0.132462
+0.170606 0.462905 0.162683
+0.107346 -0.291576 0.0507084
+0.123054 -0.26548 0.0555752
+0.1033 -0.273351 0.0618915
+-0.217347 -0.0684085 0.0793768
+-0.232534 -0.1003 0.0785864
+0.160705 0.203815 0.112095
+0.157448 0.189802 0.0951937
+0.163855 0.222961 0.107992
+0.178039 0.226994 0.0939715
+0.157779 0.236558 0.121407
+0.156862 0.233096 0.141593
+0.254115 0.147478 0.0328869
+0.246739 0.139758 0.0163119
+-0.313249 -0.26088 -0.138114
+-0.319034 -0.272336 -0.0954566
+-0.312031 -0.286413 -0.151154
+-0.318615 -0.301412 -0.127758
+-0.342054 -0.297563 -0.104135
+-0.285707 -0.289103 -0.098473
+0.224391 0.444149 0.0748945
+0.224127 0.466497 0.0733316
+0.00933378 -0.0890982 -0.073455
+-0.196836 -0.0544369 -0.0547609
+-0.268852 -0.35939 -0.204575
+-0.134821 -0.144762 0.184037
+-0.134018 -0.119846 0.172991
+-0.108849 -0.110436 0.169417
+-0.142893 -0.258813 -0.176858
+0.163435 0.422628 0.144619
+0.149105 0.425301 0.157986
+0.151653 0.445126 0.160854
+0.205634 0.453218 0.0682861
+0.196116 0.437078 0.0613117
+0.305429 0.136514 0.131635
+0.304223 0.128705 0.150462
+0.294739 0.1352 0.16485
+0.29983 0.148475 0.176204
+0.293633 0.16134 0.186506
+0.312816 0.146868 0.144611
+0.290574 0.119539 0.149687
+0.280449 0.10933 0.137413
+0.285959 0.112117 0.117297
+0.154398 0.116961 0.162863
+0.147042 0.108372 0.152274
+0.179394 0.214392 0.162515
+0.185651 0.194518 0.157404
+0.180628 0.232354 0.173507
+0.198969 0.238329 0.174545
+0.207694 0.253512 0.177641
+0.203869 0.264161 0.186147
+0.189501 0.273156 0.193533
+0.17312 0.263127 0.188581
+-0.206119 -0.0619918 0.116346
+-0.201545 -0.0465532 0.095691
+0.256266 0.152102 0.0503785
+0.264034 0.143148 0.0672501
+0.26342 0.152022 0.0867622
+0.269601 0.144159 0.103885
+0.281713 0.137095 0.0629023
+0.296165 0.127981 0.0376256
+0.300117 0.13462 0.0559043
+0.287642 0.140563 0.0456708
+0.278661 0.132147 0.0309413
+0.271646 0.118273 0.028754
+0.262921 0.1025 0.0261376
+0.261118 0.0855637 0.0169037
+0.25422 0.0767023 0.000233527
+0.237362 0.0575218 0.00198963
+0.283388 0.119704 0.0383643
+0.282941 0.112062 0.0540402
+0.249937 0.0937398 -0.00575339
+0.231931 0.0954578 -0.0107377
+0.210437 0.0914748 -0.0140972
+0.239252 0.109782 0.000233887
+0.191402 0.0870306 -0.0134669
+0.184376 0.0704441 -0.0225342
+0.184719 0.0606504 -0.044543
+0.166145 0.0578967 -0.0668264
+0.200251 0.061117 -0.0313226
+0.217304 0.057967 -0.0199728
+0.227686 0.12158 0.00370932
+0.20937 0.123042 0.000300618
+0.244333 0.124524 0.00810813
+0.295429 0.120597 0.0522453
+0.178132 0.0791963 -0.0112798
+0.177197 -0.279627 -0.000252101
+0.173639 -0.299787 -0.00606886
+0.172196 -0.313975 -0.0223573
+0.166117 -0.326159 -0.00858114
+0.168079 -0.321719 -0.0379096
+0.162521 -0.339731 -0.0445028
+0.151373 -0.32895 -0.0576036
+0.312369 -0.105021 0.0899766
+0.306 -0.11652 0.0866674
+0.301418 -0.129182 0.0942967
+0.290875 -0.135236 0.101483
+0.289966 -0.143698 0.1109
+0.295915 -0.13953 0.122573
+0.279108 -0.148197 0.122497
+0.27841 -0.152315 0.142258
+0.265867 -0.157983 0.158785
+0.256687 -0.14661 0.137703
+0.25422 -0.140442 0.172759
+0.232237 -0.161116 0.192798
+0.192807 -0.160413 0.202508
+0.178601 -0.140578 0.237092
+0.154384 -0.117167 0.24831
+0.13004 -0.134226 0.269277
+0.17845 -0.105479 0.223698
+0.247464 -0.158843 0.179106
+0.217821 -0.141328 0.207646
+0.23745 -0.141109 0.186416
+0.229503 -0.142546 0.156329
+0.214657 -0.125289 0.155947
+0.193847 -0.12199 0.171057
+0.217246 -0.105649 0.140104
+0.198069 -0.0856194 0.142621
+0.193693 -0.14343 0.18682
+0.198045 -0.145975 0.222277
+0.178989 -0.164931 0.225321
+0.158831 -0.146831 0.218292
+0.158889 -0.159611 0.244735
+0.138513 -0.154491 0.257842
+0.156487 -0.13924 0.256434
+-0.290841 -0.208347 -0.00770324
+-0.29533 -0.224009 -0.017137
+-0.30561 -0.24462 -0.0155011
+-0.278365 -0.220452 -0.0277869
+-0.267819 -0.241114 -0.0357093
+-0.286737 -0.259306 -0.0462551
+-0.310093 -0.264404 -0.0203987
+-0.307167 -0.250649 -0.0341839
+-0.307612 -0.281242 -0.0310235
+-0.321036 -0.286281 -0.0471544
+-0.310491 -0.270102 -0.0639157
+-0.297683 -0.291436 -0.0450671
+-0.285459 -0.27779 -0.0488974
+-0.296472 -0.275126 -0.0168658
+-0.276222 -0.270671 -0.00788631
+-0.246454 -0.248267 -0.0329014
+-0.244456 -0.252303 -0.00937036
+-0.304286 -0.259682 -0.0492577
+-0.294933 -0.27203 -0.0604957
+-0.300998 -0.278796 -0.0743602
+-0.316674 -0.274623 -0.0789273
+-0.337714 -0.280945 -0.0853892
+-0.325825 -0.275251 -0.0655092
+-0.332977 -0.302425 -0.0833069
+-0.298024 -0.281309 -0.0905074
+-0.293634 -0.272694 -0.109468
+-0.270428 -0.274072 -0.107657
+-0.256077 -0.253991 -0.120812
+-0.261078 -0.296954 -0.0999473
+-0.2381 -0.297373 -0.0849839
+-0.232752 -0.278906 -0.0645746
+-0.272041 -0.319246 -0.10842
+-0.21032 -0.284441 -0.0448701
+-0.256436 -0.341131 -0.11181
+-0.225098 -0.336223 -0.113241
+-0.200175 -0.288996 -0.0158483
+-0.223009 -0.317757 -0.0905879
+-0.301096 -0.262202 -0.123506
+-0.32405 -0.263071 -0.116908
+-0.292477 -0.257052 -0.14263
+-0.270149 -0.255918 -0.142108
+-0.275486 -0.258003 -0.162669
+-0.259523 -0.2674 -0.178244
+-0.251069 -0.250018 -0.0970302
+-0.24056 -0.260706 -0.172311
+-0.232499 -0.247239 -0.154361
+-0.34182 -0.274562 -0.107339
+-0.275844 -0.269552 -0.0355918
+-0.227344 -0.270897 -0.045083
+0.25093 0.170443 0.120028
+0.248384 0.184009 0.106523
+0.235777 0.198062 0.103222
+0.23641 0.211305 0.086191
+0.221152 0.226047 0.0898201
+0.223243 0.24123 0.111495
+0.203883 0.250541 0.105061
+0.226958 0.225862 0.123117
+0.219737 0.240254 0.12789
+0.214138 0.253162 0.132941
+0.232275 0.224561 0.138088
+0.109391 -0.247931 -0.0995264
+0.104952 -0.262437 -0.0820427
+0.080763 -0.274056 -0.0783487
+0.0604155 -0.262392 -0.0950373
+-0.227936 -0.329497 -0.190147
+-0.214125 -0.337031 -0.170051
+0.263325 -0.0528405 0.0685009
+0.251926 -0.0484757 0.0715634
+0.237277 -0.0480579 0.0658708
+0.244491 -0.055646 0.0790843
+0.239659 -0.0616442 0.0888399
+0.226162 -0.0616059 0.0920824
+0.21522 -0.0631162 0.0781895
+0.212545 -0.0629682 0.0966232
+0.196781 -0.0599901 0.105959
+0.210127 -0.0657511 0.11188
+0.250495 -0.0660959 0.0846797
+0.234672 -0.0671026 0.0978575
+0.237261 -0.0846396 0.108485
+0.25989 -0.0423809 0.0666642
+0.259987 -0.0257531 0.0664908
+0.247883 -0.0247631 0.0554477
+0.242689 -0.0135836 0.0440572
+0.246291 0.00496067 0.0457059
+0.245187 0.0234669 0.0372413
+0.250061 0.17416 0.133813
+0.259114 0.167602 0.148828
+0.26511 0.166925 0.171546
+0.238618 0.181672 0.138204
+0.229669 0.176644 0.149535
+0.262049 0.187134 0.176237
+0.261795 0.200868 0.165371
+0.276513 0.199318 0.153854
+0.292677 0.199982 0.157209
+0.302715 0.186361 0.157188
+0.308755 0.181779 0.171774
+0.305926 0.190608 0.185482
+0.312423 0.167782 0.169221
+0.31381 0.157653 0.158921
+0.310783 0.154703 0.171639
+0.301108 0.177938 0.144841
+0.293264 0.189309 0.147256
+0.279322 0.188458 0.145304
+0.262483 0.193127 0.192418
+0.274289 0.192768 0.20161
+0.254841 0.206509 0.193978
+0.245154 0.222549 0.189318
+0.253012 0.22552 0.174345
+0.265429 0.23121 0.17081
+-0.253206 -0.236908 -0.0443888
+0.149428 0.386275 0.232451
+0.147867 0.371017 0.232576
+0.152926 0.356705 0.229375
+0.149032 0.338169 0.225131
+0.166114 0.363727 0.223417
+0.172256 0.353426 0.215568
+0.179541 0.338962 0.221969
+0.181625 0.354174 0.201921
+0.186426 0.343494 0.187951
+0.192075 0.328517 0.215563
+0.192453 0.320313 0.195502
+0.17815 0.351274 0.174302
+0.17655 0.370022 0.172318
+0.165276 0.33179 0.225541
+0.00939661 -0.323412 0.0804555
+0.117561 -0.144032 0.250595
+0.120353 -0.125008 0.233582
+0.111251 -0.145731 0.271784
+0.11093 -0.120815 0.285658
+0.0754192 -0.0921393 0.301481
+0.0769558 -0.124739 0.290267
+0.0748528 -0.117401 0.265004
+0.0426218 -0.103655 0.282975
+0.0836935 -0.0669285 0.279976
+0.0588543 -0.11803 0.279938
+0.0586609 -0.105277 0.297458
+0.0381914 -0.083198 0.286361
+0.0371995 -0.0680895 0.266535
+0.047335 -0.0888232 0.265922
+0.05563 -0.0732275 0.247968
+0.0616189 -0.0543441 0.232231
+0.0811821 -0.0784737 0.231059
+0.0764784 -0.0353683 0.254905
+0.0661415 -0.0548095 0.278244
+0.0608765 -0.028919 0.235586
+0.087271 -0.018263 0.234864
+0.0756435 -0.10953 0.298971
+0.0951645 -0.0988538 0.288259
+0.0930654 -0.116235 0.295574
+0.0947803 -0.133863 0.284699
+0.0966033 -0.0830131 0.273711
+0.113798 -0.076403 0.258582
+0.127442 -0.0600385 0.234949
+0.132794 -0.0953943 0.252522
+0.0489111 -0.0556973 0.272686
+0.0606107 -0.0431898 0.263976
+0.0971312 -0.036293 0.245012
+0.110687 -0.0488392 0.23675
+0.126176 -0.0414728 0.221378
+0.136057 -0.0243951 0.201716
+0.122078 0.00127519 0.196707
+0.142291 0.00027007 0.178375
+0.0664164 -0.0191578 0.217877
+0.0802696 -0.0234954 0.199319
+0.0910082 -0.0454502 0.20083
+0.0798776 -0.132726 0.273919
+0.102454 0.00166634 0.210465
+0.0802091 -0.00534582 0.192641
+0.0863635 0.00814252 0.179895
+0.0946306 0.0282399 0.168655
+0.0981336 0.029523 0.148764
+0.101777 0.0395859 0.135111
+0.102082 0.0581642 0.134881
+0.0956437 0.0321468 0.122829
+0.0938658 0.0161985 0.122282
+0.107901 0.00369122 0.128399
+0.078491 0.00576055 0.124425
+0.0841199 0.0382034 0.11313
+0.0659972 0.0331947 0.109714
+0.0520056 0.0196069 0.12098
+0.108345 -0.0142416 0.126349
+0.0915975 -0.0313276 0.129913
+0.122833 -0.0287943 0.124051
+0.146418 -0.0426624 0.119895
+0.142658 -0.0228961 0.131632
+0.132595 -0.0194368 0.150249
+0.160419 -0.0327892 0.133975
+0.163125 -0.050543 0.147173
+0.060233 0.0422739 0.0970401
+0.0469107 0.0353785 0.0851224
+0.0330985 0.0252247 0.0760721
+0.0344131 0.0145341 0.0525338
+0.0185411 0.0141735 0.0691085
+-0.00651953 0.00766464 0.081092
+0.0314926 0.0314677 0.0951056
+0.0155266 0.0258059 0.107793
+0.11957 0.0050674 0.144577
+0.11348 -0.010764 0.15919
+0.0864989 0.00930931 0.196192
+0.112679 0.0343949 0.172228
+0.110056 0.0172326 0.144505
+-0.231861 -0.256194 -0.0373807
+-0.233847 -0.249852 -0.0220477
+-0.230235 -0.258784 -0.00973726
+-0.217151 -0.280061 0.00814621
+-0.228026 -0.273051 0.0389012
+-0.222798 -0.271648 -0.00409819
+-0.229682 -0.268052 0.0118625
+-0.212876 -0.279385 -0.0140325
+-0.214439 -0.277583 -0.0303969
+-0.199886 -0.285356 -0.0320197
+-0.187179 -0.295224 -0.0255156
+-0.167219 -0.315028 -0.0304058
+-0.156845 -0.302235 -0.0544314
+-0.236758 -0.254924 0.00227314
+0.0884351 -0.404348 -0.0195924
+0.0722252 -0.268984 0.0670772
+0.070147 -0.245252 0.0862941
+0.0966471 -0.229612 0.0882203
+0.0926053 -0.203806 0.103764
+0.0777673 -0.186021 0.118999
+0.0417862 -0.182948 0.126604
+0.0709209 -0.158179 0.128709
+0.095465 -0.128791 0.124296
+0.0102764 -0.16375 0.133447
+-0.0233099 -0.171526 0.153541
+-0.0499934 -0.0465621 -0.00582837
+-0.0749973 -0.0355191 0.0101971
+-0.0645299 -0.324704 -0.077104
+0.221779 0.470802 0.0892895
+0.219761 0.464582 0.1032
+0.246597 0.0246927 0.0795416
+0.253586 0.0320272 0.0977802
+0.252394 0.0305168 0.0670227
+0.261064 0.0424817 0.0637394
+0.267205 0.0556508 0.058182
+0.262106 0.0615289 0.0436471
+0.250683 0.0500973 0.0298185
+0.272972 0.0454164 0.0731281
+0.274554 0.0385023 0.090004
+0.285852 0.0505507 0.0780368
+0.287544 0.066644 0.0791877
+0.280213 0.0764884 0.0661465
+0.226162 0.0537579 -0.00800313
+0.231726 0.0372941 0.00212281
+0.226833 0.070796 -0.0125112
+-0.0228881 0.0822405 -0.0141151
+-0.0164496 0.062439 -0.0353807
+-0.0311204 0.09353 -0.0291259
+-0.0468617 0.0890411 -0.0503417
+-0.0192694 0.0958145 -0.0445491
+-0.0451838 0.0631149 -0.0497516
+-0.0349277 0.0970377 -0.0439657
+-0.0281121 0.0908295 -0.0597278
+-0.0422149 0.0875756 -0.0360098
+-0.0493018 0.0748281 -0.0407497
+-0.0508974 0.0756362 -0.0555946
+-0.045787 0.0616401 -0.0714534
+-0.0463878 0.0860458 -0.0643425
+-0.0469 0.0754389 -0.0734442
+-0.0361876 0.0721397 -0.0877453
+-0.0344753 0.0506229 -0.0835042
+-0.0209274 0.0429352 -0.0951495
+-0.00835098 0.0548898 -0.109767
+-0.0439408 0.0675081 -0.0809412
+-0.0384794 0.0604865 -0.0888934
+-0.0325578 0.0529818 -0.0938199
+-0.0250287 0.0507859 -0.100857
+-0.0171358 0.0586173 -0.103485
+-0.00108771 0.0669554 -0.09638
+0.0141047 0.0764389 -0.0789872
+-0.0166767 0.0687863 -0.0931008
+-0.0154196 0.0443974 -0.106822
+0.00804033 0.0334573 -0.118139
+-0.00353356 0.0420783 -0.115203
+0.00267945 0.0333406 -0.10199
+0.0284162 0.0260832 -0.102463
+0.0140719 0.0342408 -0.0876383
+0.01206 0.042083 -0.0690778
+0.0262722 0.0834922 -0.057317
+0.052314 0.0257381 -0.0888721
+0.0550064 0.0153199 -0.121681
+0.0742668 0.0307726 -0.0684202
+0.0254542 0.0462258 -0.0537161
+0.0192751 0.0579365 -0.0299961
+0.043388 0.0389601 -0.0621497
+0.0632571 0.0407552 -0.0511604
+0.0628168 0.0511918 -0.0295138
+-0.00769957 0.0468719 -0.0674097
+0.0356439 0.036944 -0.129564
+0.00985644 0.0486694 -0.11871
+0.0293481 0.0500299 -0.116265
+0.034304 0.0639803 -0.0982281
+0.0203799 0.0394103 -0.126099
+0.0274107 0.0238815 -0.120304
+-0.00989932 0.038173 -0.0977698
+0.0409915 0.0204434 -0.130582
+0.0603476 0.0361414 -0.135064
+0.0745924 0.0111822 -0.144256
+0.0995383 0.0164619 -0.152082
+0.079494 0.0098491 -0.119596
+0.109969 -0.00178903 -0.14121
+0.128369 -0.00667529 -0.145234
+0.147176 -0.000628591 -0.159936
+0.143478 0.0207956 -0.143584
+0.15945 -0.0140125 -0.151421
+0.167046 -0.013357 -0.130058
+0.175024 -0.000463673 -0.161193
+0.200548 0.00730904 -0.144289
+0.228188 0.00606999 -0.13371
+0.232374 -0.0225206 -0.12062
+0.217352 -0.0205681 -0.149989
+0.196446 -0.0094088 -0.159916
+0.0468892 0.0318746 -0.136603
+0.0588847 0.0194871 -0.141629
+0.0497176 0.0425705 -0.12546
+0.067316 0.0466181 -0.121528
+0.0829023 0.0554408 -0.10632
+0.0702884 0.0671402 -0.0864572
+0.0736048 0.0280122 -0.141966
+-0.0393368 0.0927115 -0.058201
+-0.074253 -0.462729 -0.0674302
+-0.0882268 -0.46748 -0.0836924
+-0.10265 -0.454789 -0.102517
+-0.100302 -0.434985 -0.0953132
+-0.123508 -0.439573 -0.0964816
+0.30376 -0.329009 -0.220778
+-0.34576 -0.287787 -0.1184
+-0.333412 -0.299886 -0.118936
+-0.318278 -0.306436 -0.109371
+-0.342798 -0.273541 -0.122349
+-0.330946 -0.265681 -0.13448
+-0.32759 -0.278136 -0.147142
+-0.313986 -0.269795 -0.152301
+-0.304937 -0.277784 -0.160852
+-0.289134 -0.290053 -0.171285
+-0.300398 -0.29313 -0.159904
+-0.305136 -0.304081 -0.143109
+-0.303823 -0.326091 -0.148763
+-0.297334 -0.338307 -0.131821
+-0.0297344 -0.320795 0.126249
+-0.0440833 -0.337561 0.102751
+-0.0635963 -0.3222 0.124646
+-0.0820192 -0.303394 0.145977
+-0.0280108 -0.346215 0.0792887
+0.190023 -0.166658 -0.160984
+0.187606 0.489479 0.0788413
+0.169798 0.492699 0.0805537
+0.160239 0.479513 0.0724411
+0.160997 0.477019 0.0876359
+0.167909 0.48647 0.0907839
+0.168839 0.465921 0.0685027
+0.185848 0.461592 0.0756332
+0.163713 0.449815 0.0719867
+0.171034 0.442192 0.0868531
+0.172622 0.439121 0.100638
+0.176648 -0.0540189 0.138439
+0.191763 0.435033 0.0761478
+0.150024 0.450095 0.173316
+0.142684 0.444609 0.180888
+0.134925 0.435737 0.185852
+0.0901553 -0.0455623 0.257862
+0.0764097 -0.0467149 0.266788
+0.218445 0.190081 -0.00453558
+0.212983 0.187132 0.0158667
+0.234912 0.203072 -0.0090602
+0.217724 0.208098 -0.00630956
+0.209095 0.231498 0.0538229
+0.223398 0.222537 0.0510417
+0.229495 0.205818 0.051858
+0.241614 0.200114 0.0387668
+0.245196 0.212463 0.0252865
+0.232616 0.199508 0.066443
+0.245867 0.187445 0.0704899
+0.218046 0.211438 0.0580939
+0.206013 0.207163 0.0667587
+0.208167 0.217137 0.0808695
+0.189985 0.216171 0.0819923
+-0.328288 -0.282754 -0.0562593
+-0.321215 -0.270892 -0.0527491
+-0.309665 -0.294095 -0.0515043
+-0.298437 -0.298535 -0.0585809
+-0.293755 -0.297175 -0.0685565
+-0.301865 -0.303693 -0.0670723
+0.113855 0.0103494 0.135894
+0.117356 -0.000872108 0.134011
+0.24192 0.231796 0.179613
+0.236568 0.24336 0.180529
+0.226028 0.250706 0.186633
+0.280884 -0.220307 -0.163899
+0.26654 -0.211351 -0.150233
+0.265056 -0.233891 -0.14678
+0.318482 -0.331697 -0.239506
+0.324395 -0.322699 -0.253479
+0.31194 -0.314679 -0.269415
+0.329739 -0.334727 -0.229668
+0.336703 -0.323152 -0.229076
+0.344428 -0.313085 -0.23571
+0.342498 -0.293727 -0.236916
+0.350543 -0.322922 -0.2465
+0.353252 -0.314 -0.261759
+0.347047 -0.326986 -0.235121
+0.0827363 -0.465408 -0.0592863
+0.0757733 -0.45735 -0.0483001
+0.0758332 -0.442944 -0.0486429
+0.0763754 -0.427162 -0.064874
+0.0810297 -0.429275 -0.0354104
+0.0760881 -0.402196 -0.0628009
+0.0803331 -0.462498 -0.0366385
+0.075746 -0.427719 -0.0496393
+0.213629 0.454927 0.154222
+0.216172 0.433851 0.152191
+0.213875 -0.163106 0.208849
+0.228479 0.0837143 -0.0171959
+0.214862 0.0777937 -0.0178642
+0.203914 0.0820668 -0.0169156
+0.213551 0.066976 -0.0133462
+-0.291696 -0.291685 -0.053913
+-0.288445 -0.286967 -0.0453936
+-0.283457 -0.280518 -0.0392925
+0.198493 0.267303 0.113918
+-0.00816198 -0.315253 0.11742
+0.312224 -0.337685 -0.224702
+0.0776115 -0.0114015 0.225043
+0.094616 -0.00802053 0.222738
+0.212523 -0.00317223 -0.153514
+0.228944 -0.0069181 -0.146501
+0.230166 -0.0223035 -0.138067
+0.212902 -0.0238532 -0.132712
+0.194522 -0.0219296 -0.138137
+0.240306 -0.00819778 -0.130311
+0.221562 -0.0094445 -0.0973672
+0.242491 -0.00970392 -0.0927411
+-0.195428 -0.293105 -0.0588105
+-0.174639 -0.293714 -0.0615636
+-0.159198 -0.295096 -0.0708856
+-0.150745 -0.297027 -0.0916319
+-0.165094 -0.309633 -0.116063
+-0.129532 -0.293794 -0.0901166
+-0.12154 -0.29002 -0.105668
+-0.103655 -0.284754 -0.110478
+-0.083304 -0.273535 -0.126518
+-0.126229 -0.291149 -0.124142
+-0.0551206 -0.282297 -0.123125
+-0.0279253 -0.280704 -0.123568
+-0.0254074 -0.296918 -0.100527
+-0.143636 -0.296672 -0.0765836
+0.228772 0.249576 0.00807469
+0.246793 0.197559 -0.000456927
+0.249351 0.185655 0.00965958
+0.077471 -0.38535 -0.0716786
+0.0780138 -0.366405 -0.0698751
+0.0796117 -0.349597 -0.0694382
+0.0817673 -0.324046 -0.067713
+0.0920794 -0.339278 -0.0766687
+0.0745961 -0.308674 -0.0513846
+0.0686559 -0.285979 -0.0584538
+0.0701691 -0.296007 -0.0346939
+0.0958416 -0.324563 -0.0739969
+0.111686 -0.315208 -0.0712996
+0.118212 -0.295045 -0.0671537
+0.145789 -0.285221 -0.0572786
+0.142444 -0.258951 -0.0651148
+0.125807 -0.256448 -0.0777825
+0.0725557 -0.302564 -0.0188272
+0.0757738 -0.318738 -0.0113816
+0.0755625 -0.334153 -0.022547
+0.0716925 -0.292684 -0.00740334
+0.0574002 -0.292216 -0.00131701
+0.0442788 -0.29974 -0.0118635
+0.0322898 -0.309073 -0.00126775
+0.0158934 -0.319374 -0.00927168
+0.00270388 -0.318254 -0.036966
+0.00321031 -0.33353 0.006095
+0.0101246 -0.330737 0.0331595
+0.0571801 -0.291727 0.0142738
+0.0411549 -0.302632 0.0323113
+0.0568056 -0.290026 0.0325413
+0.0726196 -0.279494 0.0399911
+0.0843075 -0.293947 0.0355701
+0.0802412 -0.296732 0.0208207
+0.0757523 -0.290551 0.00911073
+0.0904109 -0.307387 0.0401159
+0.0937218 -0.321431 0.0333001
+0.0876185 -0.332554 0.0196986
+-0.261918 -0.356193 -0.123562
+-0.274572 -0.363058 -0.128675
+-0.283885 -0.374671 -0.137095
+0.0993425 -0.475253 -0.0345906
+0.105878 -0.470302 -0.022596
+0.161209 0.00758193 -0.157082
+0.162861 -0.00496129 -0.160125
+0.14622 0.0234899 0.00861402
+0.16407 0.00688456 0.00334424
+0.187476 0.0108016 -0.000104135
+0.198618 0.0267239 -0.00776275
+0.213159 0.032032 -0.0211609
+0.229764 0.0271876 -0.0288068
+0.229938 0.0432028 -0.0342052
+0.231973 0.0361819 -0.05417
+0.241585 0.0230077 -0.0674939
+0.235826 0.0200922 -0.0913851
+0.229532 0.00831192 -0.0604593
+0.228417 0.0182383 -0.11087
+0.214387 0.0313739 -0.095916
+0.23826 0.0319925 -0.0407356
+0.240603 0.0153722 -0.0478262
+0.185434 0.0390225 -0.0120783
+0.174046 0.035224 -0.0278333
+0.183039 0.0254835 -0.0435084
+0.185088 0.013208 -0.0664572
+0.184517 -0.00115634 -0.0944301
+0.15883 0.0112792 -0.0789407
+0.166768 0.0454452 -0.0125857
+0.149871 0.0473393 -0.0127261
+0.24097 0.0273212 -0.0523021
+0.244689 0.0183228 -0.0578172
+0.239121 0.00955592 -0.0564169
+0.23067 0.0133196 -0.0511149
+0.217815 0.013095 -0.0555127
+0.202953 0.0215885 -0.0445658
+0.223255 0.0309758 -0.00933388
+0.246338 0.00680161 -0.0920968
+-0.29666 -0.251794 -0.0420083
+-0.282377 -0.246563 -0.0382484
+-0.274248 -0.257163 -0.0411355
+0.121069 0.3764 0.220244
+0.124178 0.361607 0.222425
+0.12316 0.347289 0.21517
+0.120818 0.335014 0.200629
+0.131773 0.318764 0.203442
+0.226426 -0.17846 -0.157236
+0.25469 -0.197929 -0.186668
+0.262527 -0.207499 -0.20859
+0.275709 -0.206748 -0.189755
+0.268196 -0.218714 -0.226562
+0.252109 -0.214264 -0.221034
+0.231966 -0.219863 -0.222417
+0.16439 -0.290409 -0.045741
+0.174232 0.353361 0.161991
+0.167416 0.346038 0.151615
+0.153417 0.351454 0.147309
+0.141711 0.365653 0.160008
+0.174784 0.33351 0.146976
+0.172695 0.32064 0.137876
+0.171538 0.327653 0.122668
+-0.153467 -0.465689 -0.0917447
+-0.142244 -0.458217 -0.0978282
+-0.137315 -0.444624 -0.0945322
+-0.142546 -0.468457 -0.0929801
+-0.130113 -0.470071 -0.0887414
+-0.111868 -0.466578 -0.0912306
+-0.129822 -0.476932 -0.0773011
+-0.116192 -0.47396 -0.0799073
+-0.102072 -0.471199 -0.0816089
+0.0648327 -0.288276 0.006469
+0.290523 -0.14969 0.120852
+0.286341 -0.149084 0.132395
+0.276561 -0.154417 0.131433
+0.266874 -0.144863 0.128327
+0.266123 -0.155511 0.139202
+0.254682 -0.156501 0.14847
+0.243072 -0.144724 0.147014
+0.246658 -0.132891 0.13506
+0.271484 -0.158645 0.148826
+0.262513 -0.161728 0.150556
+0.255383 -0.163883 0.160093
+0.258182 -0.154155 0.170424
+0.247063 -0.165211 0.168804
+0.219971 -0.167303 0.19665
+0.224247 -0.15536 0.203023
+0.206877 -0.166582 0.198203
+0.201256 -0.167208 0.210293
+0.187949 -0.165778 0.214542
+0.175727 -0.151043 0.207983
+0.205839 -0.156778 0.218275
+0.191606 -0.158183 0.226132
+0.180969 -0.154585 0.235065
+0.170507 -0.150398 0.244702
+0.167678 -0.135398 0.246604
+0.174695 -0.121433 0.234619
+0.192199 -0.123695 0.219574
+0.161002 -0.150315 0.252813
+0.149901 -0.156196 0.25282
+0.145756 -0.15618 0.238445
+0.158231 -0.157867 0.22927
+0.148019 -0.147104 0.261177
+0.14318 -0.132144 0.260892
+0.136414 -0.145181 0.266602
+0.124033 -0.152215 0.261533
+0.128756 -0.151616 0.248741
+0.169715 -0.162656 0.235355
+0.125332 -0.145025 0.270918
+0.118124 -0.13431 0.277651
+0.122928 -0.121301 0.271952
+0.113017 -0.148351 0.260498
+0.105868 -0.137752 0.254904
+0.0970603 -0.116879 0.246491
+0.113377 -0.13188 0.244126
+0.102989 -0.14407 0.26402
+0.09993 -0.141861 0.276203
+-0.208269 -0.22401 0.141633
+-0.20692 -0.250284 0.132437
+0.0965268 -0.427234 -0.105996
+0.0991486 -0.439673 -0.0998914
+0.19126 -0.17619 0.0390751
+0.184715 -0.204433 0.0324441
+0.164304 -0.221583 0.0465757
+0.1444 -0.208807 0.0686986
+0.0226172 -0.0147867 0.137076
+-0.142255 -0.49298 -0.0216335
+0.233691 0.0208001 -0.0385404
+-0.192672 -0.315988 0.0839294
+-0.202286 -0.295403 0.104035
+-0.212462 -0.276423 0.111373
+-0.218066 -0.285356 0.0914372
+-0.230517 -0.270625 0.068595
+0.136055 0.0355608 0.0131192
+0.121886 0.0399552 0.0198872
+0.106186 0.0275039 0.0255554
+0.0928691 0.0373146 0.0446539
+0.107832 0.0113031 0.0106037
+0.0853288 0.00897116 0.0135994
+0.0648776 -0.00152148 0.00684991
+0.301901 0.1917 0.196837
+0.304498 0.181718 0.192274
+0.29757 0.171529 0.192207
+0.283611 0.170157 0.190984
+0.17589 0.332048 0.230978
+0.176274 0.318104 0.221305
+0.189786 0.308195 0.22975
+0.199452 0.291101 0.221444
+0.210647 0.278153 0.221165
+0.212403 0.290739 0.232707
+0.229872 0.295702 0.229994
+0.238851 0.283146 0.23072
+0.237981 0.260514 0.227763
+0.248013 0.243086 0.221386
+0.245976 0.271423 0.230129
+0.237201 0.240055 0.208931
+0.239318 0.292509 0.222434
+0.244312 0.28734 0.210021
+0.234086 0.289095 0.199595
+0.221914 0.298769 0.201947
+0.1778 0.322602 0.229777
+0.232474 0.299285 0.221376
+0.223334 0.304746 0.213608
+0.207337 0.311267 0.204764
+0.224642 0.304397 0.226583
+0.214619 0.302742 0.233384
+0.203769 0.313465 0.232857
+0.204968 0.319702 0.218324
+0.201735 0.298965 0.232154
+-0.0492057 -0.0812756 -0.10551
+-0.0710597 -0.075701 -0.12067
+-0.0879497 -0.0905516 -0.13677
+-0.0843511 -0.119489 -0.147588
+-0.0762899 -0.059326 -0.102542
+-0.0985707 -0.0477827 -0.098696
+-0.118275 -0.0536394 -0.113913
+-0.11441 -0.0717159 -0.128365
+0.179503 0.303256 0.0436704
+0.192013 0.290229 0.0290644
+0.17689 0.287515 0.0370015
+0.192038 0.271729 0.0156116
+0.209167 0.267708 0.00969983
+0.200133 0.255704 0.00478026
+-0.247504 -0.392344 -0.224787
+-0.247477 -0.405986 -0.218212
+-0.243099 -0.400741 -0.206484
+-0.246703 -0.405341 -0.193269
+-0.249065 -0.417066 -0.193806
+-0.263856 -0.417281 -0.202583
+-0.253854 -0.410374 -0.185389
+-0.264275 -0.407223 -0.177693
+-0.272044 -0.40206 -0.166216
+-0.284666 -0.409799 -0.163624
+-0.28689 -0.423019 -0.170791
+-0.283285 -0.428723 -0.186577
+-0.301536 -0.424549 -0.197331
+-0.276684 -0.426319 -0.176604
+-0.270204 -0.426002 -0.18843
+-0.267033 -0.421163 -0.178871
+-0.28572 -0.431162 -0.176917
+0.0984334 0.0428629 0.147655
+0.0984512 0.0370662 0.158757
+-0.324734 -0.278381 -0.0492733
+-0.319049 -0.278035 -0.0388895
+-0.314693 -0.271902 -0.0298211
+-0.0814975 -0.486596 -0.0514846
+-0.0716057 -0.47534 -0.0576231
+-0.0807984 -0.475501 -0.0687174
+-0.0782273 -0.46782 -0.0773567
+-0.0794515 -0.454639 -0.076562
+-0.0865219 -0.442019 -0.0638219
+-0.0826697 -0.456554 -0.0896798
+-0.0913604 -0.446861 -0.0969099
+-0.0924184 -0.432044 -0.0753911
+-0.098273 -0.416607 -0.0817744
+-0.111942 -0.395187 -0.0818225
+-0.0923505 -0.401008 -0.0727607
+-0.110463 -0.376831 -0.0757945
+-0.0955645 -0.358515 -0.0727168
+-0.0748788 -0.367775 -0.053732
+-0.099584 -0.336657 -0.0718408
+-0.0617229 -0.346951 -0.0508932
+-0.0372908 -0.340191 -0.0358611
+-0.187209 -0.0514446 0.134371
+-0.172275 -0.062062 0.144345
+-0.181772 -0.0387772 0.12984
+0.27325 0.225257 0.167666
+0.283037 0.221467 0.169903
+0.141164 -0.00931766 -0.150578
+0.223335 -0.262546 -0.149684
+0.199958 -0.258684 -0.141852
+0.218212 -0.249649 -0.131577
+-0.0730224 -0.485274 -0.0383372
+-0.0765985 -0.473674 -0.025717
+-0.0833487 -0.485599 -0.0202509
+-0.0969784 -0.487241 -0.0157338
+-0.106328 -0.475595 -0.0156679
+-0.107519 -0.486132 -0.0147883
+-0.117398 -0.491224 -0.0155421
+-0.11866 -0.497871 -0.0255495
+0.192205 -0.0198585 -0.151725
+0.281772 0.238327 0.210562
+0.269826 0.243309 0.222088
+0.243244 0.207816 -0.00325363
+0.239729 0.221932 -0.000908316
+0.243256 0.225044 0.0142927
+0.235434 0.23543 0.00346349
+0.158954 -0.447934 -0.074517
+0.164 -0.440163 -0.0849532
+0.160645 -0.422992 -0.0839209
+0.160099 -0.41995 -0.0633914
+0.170063 -0.432118 -0.0580684
+0.170775 -0.442201 -0.0470561
+0.167356 -0.452883 -0.036848
+0.157537 -0.464023 -0.0432353
+0.154265 -0.458207 -0.0582632
+0.146921 -0.466798 -0.0291891
+0.17099 -0.445942 -0.0649823
+0.152813 -0.426396 -0.0985179
+0.142833 -0.435406 -0.0989063
+0.126573 -0.436557 -0.100225
+0.120949 -0.42189 -0.103936
+0.130685 -0.411647 -0.0938972
+0.160351 -0.460975 -0.0324222
+0.164884 -0.460693 -0.0398836
+0.167083 -0.430709 -0.0834012
+0.161089 -0.432508 -0.0931652
+0.165433 -0.424956 -0.0749657
+0.155909 -0.417913 -0.0743018
+0.149825 -0.407366 -0.067413
+0.142015 -0.399818 -0.0764637
+0.153043 -0.435416 -0.0966096
+0.148842 -0.431897 -0.101759
+0.145045 -0.424265 -0.10134
+0.147732 -0.418014 -0.0911084
+0.138977 -0.417451 -0.0965307
+0.139635 -0.411036 -0.0878314
+0.14174 -0.430413 -0.104765
+0.134784 -0.434536 -0.101822
+0.135086 -0.440748 -0.0918112
+0.13766 -0.447456 -0.0800542
+0.122938 -0.455334 -0.0698495
+0.127914 -0.428797 -0.106691
+0.135671 -0.430083 -0.106069
+-0.0268041 -0.304928 0.142542
+-0.0182344 -0.281255 0.151927
+0.000262015 -0.258244 0.142866
+0.0151742 -0.229959 0.128466
+-0.00970849 -0.227931 0.154073
+0.0337917 -0.209923 0.115845
+0.0199495 -0.192987 0.125832
+-0.154293 -0.0335699 -0.082135
+-0.129631 -0.0324487 -0.0813475
+-0.120097 -0.027431 -0.0586998
+-0.0956922 -0.0340394 -0.0533561
+-0.0814148 -0.0448428 -0.0722969
+-0.0594432 -0.0515596 -0.0534184
+-0.160793 -0.0482086 -0.0989707
+-0.166155 -0.0307425 -0.0663998
+-0.169924 -0.0270352 -0.0414151
+-0.183311 -0.0375758 -0.0551581
+-0.174206 -0.0274939 -0.0203147
+-0.192353 -0.0397338 -0.00141151
+-0.170447 -0.0249801 0.0063437
+-0.211587 -0.0636911 -0.00501259
+-0.0018753 -0.187141 -0.149775
+0.0183103 -0.182965 -0.142326
+0.0322217 -0.167313 -0.134641
+0.0236675 -0.146992 -0.123167
+-0.00232452 -0.142332 -0.126149
+0.0375806 -0.18533 -0.140019
+0.0530379 -0.201545 -0.137283
+0.0772348 -0.20845 -0.140694
+0.0988175 -0.224384 -0.140468
+0.119251 -0.222512 -0.162731
+0.03788 -0.218276 -0.135529
+0.0772845 -0.19139 -0.155355
+0.080882 -0.225907 -0.127445
+0.0653619 -0.242778 -0.113036
+0.0731805 -0.173068 -0.155239
+0.0897045 -0.191329 -0.166141
+0.102707 -0.199007 -0.171074
+0.119058 -0.208637 -0.176942
+0.13998 -0.205302 -0.190755
+0.175364 -0.230409 -0.20401
+0.173584 -0.215336 -0.204853
+0.178202 -0.198362 -0.197173
+0.188397 -0.22907 -0.211289
+0.202734 -0.22056 -0.215065
+0.199745 -0.239634 -0.214151
+0.0815106 -0.18364 -0.162988
+-0.172104 -0.359269 -0.00938238
+-0.172319 -0.335226 -0.0164663
+-0.16873 -0.368903 -0.0231312
+-0.280983 -0.308012 -0.105167
+-0.278654 -0.297571 -0.101297
+-0.274336 -0.285615 -0.103446
+-0.260134 -0.28158 -0.0989442
+-0.257005 -0.265144 -0.10215
+-0.26942 -0.305285 -0.10276
+-0.257003 -0.309674 -0.100476
+-0.244993 -0.306014 -0.0938734
+-0.249351 -0.292113 -0.0926885
+-0.25954 -0.322424 -0.104282
+-0.267093 -0.332079 -0.111051
+-0.283312 -0.328944 -0.119574
+-0.249017 -0.331591 -0.10481
+-0.232981 -0.32784 -0.100164
+-0.240291 -0.342062 -0.113719
+-0.229688 -0.345883 -0.126712
+-0.23058 -0.352629 -0.145077
+-0.21352 -0.337371 -0.127344
+-0.269191 -0.344874 -0.117105
+-0.208623 -0.327937 -0.112241
+-0.191793 -0.321843 -0.117022
+-0.180909 -0.311277 -0.104708
+0.11012 0.10505 0.0238496
+0.213679 0.221732 0.163906
+-0.0357839 -0.0025294 0.108473
+-0.0312254 -0.0135193 0.128152
+-0.0238807 -0.033229 0.139313
+-0.00300831 -0.046529 0.144036
+-0.00364169 -0.0760125 0.145155
+-0.0103288 -0.10643 0.141831
+0.015326 -0.129347 0.142131
+-0.041062 -0.0443202 0.130625
+-0.0555252 -0.0465254 0.114753
+-0.0556686 -0.0325657 0.0996413
+-0.0768736 -0.0422105 0.0949058
+-0.0167984 0.000564353 0.123722
+0.00524698 0.0020139 0.129964
+-0.0281137 -0.0861213 0.139333
+-0.0785841 -0.0379469 0.0747431
+-0.0762529 -0.0505618 0.114297
+-0.032521 -0.108383 0.136839
+-0.0633754 -0.0458183 0.101476
+-0.0250298 0.00663901 0.112981
+-0.0219675 0.00393164 0.0935556
+-0.147404 -0.304789 -0.127071
+0.192111 0.473304 0.143665
+0.202701 0.475169 0.131787
+0.206558 0.475874 0.120017
+0.202492 0.480449 0.108775
+0.157654 -0.366957 -0.0205798
+0.26661 -0.307414 -0.281977
+0.270077 -0.295571 -0.288815
+0.283263 -0.291236 -0.297392
+0.290598 -0.279448 -0.297082
+0.315035 -0.2885 -0.301158
+0.307588 -0.298676 -0.297006
+0.297613 -0.307939 -0.283621
+0.293787 -0.301201 -0.295424
+0.318389 -0.297707 -0.296483
+0.328066 -0.301032 -0.289042
+0.324582 -0.309109 -0.276338
+0.33579 -0.291676 -0.2964
+0.346867 -0.288071 -0.287976
+0.353956 -0.299169 -0.28317
+0.345495 -0.307423 -0.274703
+0.352866 -0.288693 -0.277818
+0.351312 -0.284395 -0.265224
+0.355354 -0.294774 -0.257468
+0.360217 -0.304818 -0.260578
+0.35682 -0.308388 -0.269609
+0.359106 -0.312264 -0.252967
+0.356474 -0.316127 -0.245135
+0.351445 -0.319298 -0.237976
+-0.26647 -0.314705 -0.199507
+-0.27426 -0.329739 -0.204261
+-0.290128 -0.337563 -0.206145
+-0.303723 -0.332243 -0.195363
+-0.303858 -0.323407 -0.176279
+-0.302757 -0.349394 -0.210447
+-0.304665 -0.364526 -0.223447
+-0.321319 -0.375627 -0.232938
+-0.285437 -0.371465 -0.224735
+-0.28011 -0.37957 -0.242654
+-0.314609 -0.397575 -0.24523
+-0.31498 -0.344154 -0.199865
+-0.330892 -0.351901 -0.189469
+-0.332902 -0.388937 -0.243068
+-0.348245 -0.402805 -0.233911
+-0.328232 -0.404959 -0.235266
+-0.273541 -0.396265 -0.240028
+-0.293049 -0.39498 -0.245656
+-0.355214 -0.402159 -0.217135
+-0.360108 -0.415844 -0.205721
+-0.346381 -0.42668 -0.201491
+-0.340374 -0.436455 -0.180032
+-0.360217 -0.406031 -0.193698
+-0.356506 -0.391255 -0.182727
+-0.356512 -0.407116 -0.177427
+-0.347384 -0.421732 -0.172779
+-0.348669 -0.37532 -0.17511
+-0.341266 -0.408591 -0.160307
+-0.332592 -0.391247 -0.153464
+-0.323849 -0.407723 -0.153687
+-0.349384 -0.43169 -0.189135
+-0.335815 -0.43119 -0.192046
+-0.324454 -0.435806 -0.181733
+-0.331465 -0.433295 -0.170921
+-0.314528 -0.431132 -0.168412
+-0.260177 -0.397767 -0.235
+-0.252953 -0.401301 -0.227678
+-0.24372 -0.401046 -0.225417
+-0.258003 -0.38952 -0.247437
+-0.272162 -0.387844 -0.252537
+-0.287147 -0.384539 -0.255755
+-0.315505 -0.382766 -0.24459
+-0.323099 -0.390444 -0.249836
+-0.312399 -0.390336 -0.253745
+-0.328503 -0.399215 -0.245058
+-0.340635 -0.398589 -0.24271
+-0.266346 -0.382624 -0.244488
+-0.267321 -0.391914 -0.245578
+-0.0266812 0.0695328 -0.0242573
+-0.00773299 0.0681739 -0.0184911
+0.0122858 0.0669077 -0.0123781
+0.0150035 0.0767227 0.00239352
+0.0141467 0.0954062 -0.00215996
+0.0325338 0.098411 -0.00617133
+0.0450676 0.0983028 0.010086
+0.0314473 0.0730983 0.00401189
+0.0505593 0.0686309 0.00292132
+0.0698817 0.067505 0.00832925
+0.145383 0.180744 0.0984363
+0.132189 0.17376 0.0925198
+0.121241 0.164951 0.0850023
+0.133425 0.169934 0.0774819
+0.11505 0.153676 0.07879
+-0.083942 -0.368893 -0.0674225
+-0.0809876 -0.385027 -0.0581697
+-0.0723248 -0.382058 -0.0416794
+-0.00904893 0.0914446 -0.0120745
+0.00504523 0.0987359 -0.0150796
+0.0060609 0.0932522 -0.034057
+0.0248461 0.0873188 -0.0377031
+0.0363994 0.089055 -0.023789
+0.00213821 0.0914225 -0.00482177
+0.0092558 0.0863088 0.00212053
+0.106375 0.0274106 0.14138
+-0.263884 -0.385451 -0.252252
+0.258755 0.24689 0.225661
+0.259085 0.23306 0.219814
+0.249971 0.25591 0.228242
+-0.322841 -0.345115 -0.164492
+-0.323706 -0.355326 -0.152393
+-0.279169 -0.265328 -0.0439542
+-0.285416 -0.267917 -0.0516616
+-0.294745 -0.263175 -0.0517725
+0.219738 -0.0165453 -0.112039
+0.204608 -0.00981825 -0.104246
+0.187184 -0.00954937 -0.110855
+0.228145 0.230452 0.102316
+0.214447 0.238029 0.0983297
+0.23229 0.220992 0.0943503
+0.215398 0.247623 0.105271
+0.217938 0.25177 0.117669
+0.210276 0.258003 0.111405
+0.220949 0.241286 0.103103
+0.290344 -0.337843 -0.233955
+0.276226 -0.337233 -0.22831
+0.296573 -0.339782 -0.226215
+0.227663 0.461812 0.0838481
+0.234265 0.455281 0.0718225
+0.229698 0.445794 0.0603386
+0.225781 0.470182 0.0813133
+0.214396 0.4698 0.0788369
+0.208406 0.478123 0.0862021
+0.214031 0.460896 0.0711899
+0.217085 0.451741 0.0628259
+0.219354 0.474333 0.0827819
+0.21619 0.47758 0.0903862
+0.217994 0.472178 0.097233
+0.209525 0.481852 0.0939712
+0.227208 0.456115 0.0633709
+0.234329 0.451682 0.0642008
+0.23166 0.462658 0.0759848
+0.273713 -0.249314 -0.252993
+0.274317 -0.268417 -0.267071
+0.263304 -0.282613 -0.260934
+-0.241182 -0.408607 -0.207233
+0.243855 0.194683 0.113624
+0.235959 0.206195 0.114609
+-0.329827 -0.30598 -0.098469
+-0.057071 -0.0425853 0.0117122
+-0.0551192 -0.0420888 0.0309046
+-0.0534835 -0.0402863 0.0500454
+-0.0435993 -0.0469165 0.00748095
+-0.0255629 -0.0374196 0.00481763
+-0.00817324 -0.0328811 -0.0061182
+-0.00350439 -0.0437548 -0.0255784
+-0.0349503 -0.0490115 -0.00492533
+-0.0419875 -0.0536475 -0.0293419
+-0.0139014 -0.0607747 -0.0378374
+-0.0105205 -0.0780801 -0.0585195
+-0.0335249 -0.0540562 -0.0180409
+-0.0278411 -0.0595083 -0.0313207
+0.193253 0.49569 0.0858481
+0.189805 0.494474 0.0949255
+0.179559 0.491992 0.0947812
+0.19709 0.487808 0.0978913
+0.174576 0.497081 0.0879734
+0.179584 0.496616 0.0784813
+0.175502 0.48892 0.0725035
+0.169475 0.481635 0.069206
+0.164994 0.474033 0.0669237
+0.155945 0.469227 0.0714265
+0.159639 0.459314 0.0671634
+0.160451 -0.0544036 0.113535
+0.0454737 -0.0938547 0.293886
+0.0481458 -0.104952 0.292926
+0.0550885 -0.114349 0.29062
+0.0654216 -0.120788 0.289843
+0.0691263 -0.126298 0.278339
+0.0372417 -0.0943784 0.286279
+0.0377118 -0.088702 0.274623
+0.179652 -0.262284 0.0054219
+0.186087 -0.244472 0.00347757
+0.246807 -0.00615903 -0.103541
+0.232593 -0.0188607 -0.112191
+0.240348 -0.0194541 -0.117278
+0.238974 -0.018297 -0.127651
+-0.164632 -0.397326 -0.024693
+0.0813645 -0.458079 -0.0716845
+0.302397 0.127887 0.0470846
+0.298473 0.138148 0.0455108
+0.292313 0.136477 0.0386335
+0.288054 0.130079 0.0326142
+0.28271 0.137427 0.0379629
+0.271939 0.136706 0.038053
+0.261008 0.14116 0.0404912
+0.28194 0.12422 0.0312108
+0.300876 0.126279 0.0551937
+0.295306 0.129324 0.0632115
+0.286846 0.130603 0.0706512
+0.282604 0.11846 0.0804122
+0.273418 0.134767 0.0747422
+0.296205 0.121995 0.0600976
+0.289869 0.116278 0.0576534
+0.283444 0.108666 0.0679441
+0.208186 0.435058 0.0678801
+0.218992 0.437774 0.0675136
+0.20517 0.444041 0.0615
+0.195457 0.447135 0.0675955
+0.224157 0.438328 0.0597838
+0.221367 0.446069 0.0584788
+0.168291 -0.443724 -0.076199
+-0.239169 -0.248023 -0.0426243
+-0.246509 -0.244281 -0.0523825
+-0.245933 -0.250709 -0.0701381
+-0.252213 -0.23035 -0.0598084
+-0.256922 -0.212794 -0.063622
+-0.26443 -0.20039 -0.048456
+-0.236601 -0.248665 -0.0331221
+-0.248238 -0.244973 -0.04299
+-0.257005 -0.243536 -0.0363368
+-0.264619 -0.253098 -0.0324677
+-0.260909 -0.234911 -0.0387305
+-0.270856 -0.228969 -0.0335016
+-0.268106 -0.214873 -0.0367573
+-0.255525 -0.250409 -0.0291707
+-0.246354 -0.252152 -0.0213798
+-0.25421 -0.258055 -0.0140562
+-0.253371 -0.258976 0.00135493
+-0.263391 -0.256528 0.0180325
+-0.264412 -0.265611 -0.0106557
+-0.268835 -0.263918 0.00476582
+-0.24972 -0.252323 0.0327963
+-0.239732 -0.259608 0.0493553
+-0.242639 -0.251027 0.0706418
+-0.27192 -0.270836 -0.02103
+-0.264888 -0.241199 0.0366373
+-0.279792 -0.22631 0.033251
+-0.274206 -0.207933 0.0474852
+-0.283361 -0.276288 -0.0174295
+-0.267659 -0.226048 0.0482271
+0.202151 0.274483 0.19338
+0.194908 0.283204 0.198963
+-0.157532 -0.273615 -0.179435
+-0.176899 -0.279729 -0.184503
+-0.188947 -0.290942 -0.187096
+-0.192598 -0.3074 -0.18306
+-0.203551 -0.297799 -0.190929
+-0.222085 -0.288246 -0.191352
+-0.217908 -0.303036 -0.194268
+-0.355074 -0.426501 -0.195973
+-0.354866 -0.423993 -0.205337
+-0.355569 -0.419108 -0.214528
+-0.353763 -0.412061 -0.224887
+-0.346029 -0.286085 -0.1062
+-0.341227 -0.288967 -0.0947058
+-0.336277 -0.278132 -0.0971269
+-0.328988 -0.269127 -0.105169
+-0.340297 -0.292656 -0.0831052
+-0.335578 -0.266756 -0.115188
+-0.339311 -0.299368 -0.0917431
+0.212569 0.261061 0.18216
+0.216886 0.255854 0.175009
+0.219484 0.251322 0.162982
+0.212289 0.247584 0.170006
+0.218513 0.26322 0.154269
+0.225667 0.261532 0.178216
+0.218436 0.276817 0.178562
+0.234042 0.273996 0.185236
+0.223426 0.242038 0.154678
+0.21181 0.288948 0.181391
+0.220498 0.257755 0.18435
+0.211254 0.266563 0.187703
+0.211739 0.26954 0.199826
+0.233056 0.457965 0.0658843
+0.227063 0.46182 0.0682472
+0.220168 0.458063 0.0666597
+-0.0481392 -0.0447802 0.0166181
+0.131516 0.0530135 0.000672445
+0.12038 0.0567042 0.000376152
+0.134766 0.046581 0.0097546
+0.0956782 -0.141364 0.26837
+0.0877085 -0.13595 0.269242
+0.0854698 -0.124959 0.261926
+0.0839071 -0.111836 0.253439
+0.0904091 -0.099649 0.239147
+0.0872053 -0.136949 0.279095
+0.085161 -0.130401 0.287023
+-0.00884361 -0.0890856 -0.0728998
+-0.00654069 -0.102279 -0.0895079
+-0.0290648 -0.104665 -0.111248
+-0.0100257 -0.116287 -0.107029
+0.0875054 -0.104584 0.297054
+0.191386 -0.264049 -0.175252
+0.190045 -0.262732 -0.156889
+0.204589 -0.269071 -0.178679
+0.222111 -0.273266 -0.172895
+0.189235 0.0753918 -0.0129238
+0.0782752 -0.467624 -0.0498343
+0.0759673 -0.46177 -0.0555898
+0.0772195 -0.460605 -0.0642783
+0.153175 0.313215 0.0881136
+0.161272 0.302381 0.0857396
+0.163146 0.324848 0.0718597
+0.151557 0.334415 0.0880604
+0.142886 0.334889 0.0972586
+0.140662 0.34411 0.107021
+0.133598 0.347717 0.117582
+0.131314 0.355467 0.129074
+0.127988 0.361743 0.142546
+0.123763 0.348981 0.138438
+0.119822 0.353337 0.149718
+0.116288 0.351928 0.168204
+0.226419 0.174912 -0.00671405
+0.232006 0.15884 0.00199041
+0.243933 0.169091 0.00253819
+0.237571 0.172786 -0.00612936
+0.183717 0.142245 0.147062
+0.183533 0.157982 0.153224
+0.200684 0.169917 0.154336
+0.175927 0.14817 0.154877
+0.164064 0.143191 0.159694
+0.181391 0.132744 0.149775
+0.177453 0.123521 0.157312
+0.108424 0.0602759 0.0311611
+0.101691 0.0511896 0.042029
+0.297187 0.12858 0.124591
+0.286869 0.125817 0.115842
+0.279264 0.13651 0.11071
+0.277464 0.150893 0.114839
+0.267972 0.163015 0.116989
+0.29066 -0.215317 -0.195858
+-0.0439761 -0.143405 -0.149346
+0.0959309 0.0199379 0.158053
+0.0941358 0.00844127 0.16726
+0.273991 -0.158318 0.138404
+0.280108 -0.155995 0.136908
+0.28311 -0.154668 0.131232
+0.287165 -0.152142 0.126309
+0.291082 -0.145525 0.127381
+0.258354 -0.329753 -0.2515
+0.256649 -0.327468 -0.240856
+0.265642 -0.321399 -0.233195
+0.269005 -0.32965 -0.227653
+0.204877 0.287044 0.0357487
+0.289139 -0.339472 -0.226774
+0.282728 -0.335989 -0.224475
+0.283065 -0.327384 -0.221193
+0.28398 -0.316486 -0.219293
+0.289461 -0.305296 -0.210924
+0.2738 -0.310998 -0.221733
+0.2646 -0.301502 -0.221281
+0.282981 -0.339471 -0.231684
+0.275544 -0.336339 -0.239462
+0.259131 -0.2954 -0.232139
+0.281853 -0.296955 -0.202405
+0.287258 -0.287693 -0.192682
+0.301236 -0.282638 -0.194913
+0.240774 0.0243245 -0.0402136
+0.034599 -0.0884904 0.28182
+0.0345637 -0.0787252 0.277243
+0.0395236 -0.0802562 0.266836
+0.0486856 -0.0788086 0.257578
+0.044992 -0.0647291 0.254151
+0.0829644 -0.0808872 0.290453
+0.0435251 -0.0559756 0.262078
+0.20354 0.276522 0.016541
+-0.0980428 -0.240155 0.197738
+-0.0924965 -0.26196 0.186819
+-0.109853 -0.270124 0.168775
+-0.253582 -0.386742 -0.238773
+-0.0267016 0.0982672 -0.0374627
+0.214024 0.433945 0.0622105
+0.204736 0.432758 0.058829
+0.201109 0.433103 0.0655996
+0.201809 0.436011 0.073894
+0.193477 0.433855 0.0682907
+0.185218 0.436354 0.0703184
+0.191166 0.440882 0.0659291
+0.187348 0.447071 0.070626
+0.179215 0.453739 0.0726364
+0.193028 0.454826 0.0736221
+0.173421 0.440644 0.0776765
+-0.147031 -0.496444 -0.028386
+-0.151597 -0.495421 -0.0374969
+-0.157627 -0.484775 -0.0397619
+-0.140246 -0.499465 -0.0256815
+-0.132401 -0.5 -0.0296698
+-0.132703 -0.497498 -0.0384881
+-0.126331 -0.494492 -0.0452588
+-0.11646 -0.490117 -0.0524448
+-0.101295 -0.487303 -0.0547567
+-0.136101 -0.494007 -0.0471871
+-0.13938 -0.490039 -0.0561432
+-0.133381 -0.483866 -0.0661423
+-0.15577 -0.492035 -0.0446482
+-0.153261 -0.490282 -0.0555144
+0.197755 0.272342 0.0690149
+0.190382 0.263313 0.0560052
+0.0686632 -0.292912 -0.0237205
+0.056243 -0.29127 -0.0303266
+0.0398126 -0.298058 -0.030698
+0.0315681 -0.295788 -0.0489563
+0.043708 -0.285681 -0.061727
+0.0621136 -0.289132 -0.040881
+0.0722108 -0.295077 -0.0484755
+0.0577034 -0.286988 -0.0524253
+0.0569935 -0.281989 -0.0659264
+0.064236 -0.290328 -0.0328163
+-0.0127838 0.0757233 -0.00921143
+0.0935192 0.0772038 0.0642915
+0.169714 0.302879 0.0497006
+0.163659 0.300165 0.0640716
+0.172929 0.295611 0.0434924
+0.049865 0.0913802 0.0221499
+0.0418831 0.0808731 0.013212
+0.0368549 0.0913191 0.0145569
+0.0319565 0.0968433 0.00544037
+0.310435 0.13526 0.142507
+0.026474 0.0305763 -0.129196
+0.017822 0.0292426 -0.122273
+0.0163904 0.0280919 -0.108702
+0.0350495 0.0278097 -0.132911
+0.178361 0.286185 0.0866978
+0.184473 0.288229 0.0959947
+0.0746028 -0.0119842 0.202652
+0.0770488 -0.00198153 0.201878
+0.0861829 0.00359659 0.206228
+0.0964676 0.00912576 0.20305
+0.110414 0.00821695 0.200752
+0.119478 0.0159283 0.18886
+-0.0749585 -0.470198 -0.0703816
+-0.0722579 -0.469101 -0.0627931
+-0.0775597 -0.45771 -0.0519849
+-0.0725204 -0.466379 -0.0530837
+-0.0822617 -0.458336 -0.0360309
+0.11796 -0.00196684 -0.151498
+0.110489 0.008862 -0.155734
+0.119387 0.0273131 -0.141295
+0.100036 0.00317869 -0.149099
+0.0946498 0.00360487 -0.130289
+0.0996271 0.00897841 -0.108462
+0.0876406 0.00969553 -0.149119
+0.0889673 0.0239205 -0.144401
+0.103773 0.0275171 -0.140968
+0.112143 0.0407687 -0.122665
+0.128275 -0.00210722 -0.155193
+0.136944 0.00690927 -0.158099
+0.1315 0.01712 -0.150046
+0.148823 0.0111196 -0.154092
+0.137878 -0.00286061 -0.157253
+0.0812501 0.0180999 -0.148671
+0.0723702 0.0199576 -0.146504
+0.0894465 0.0177562 -0.14994
+-0.244247 -0.411744 -0.197734
+0.0532101 -0.0425986 0.239458
+0.0614299 -0.034607 0.250277
+0.0718515 -0.0264935 0.244569
+0.0612036 -0.0408317 0.227838
+0.211416 0.21985 0.0506815
+0.209629 0.209187 0.0516229
+0.197715 0.200596 0.0531448
+0.210711 0.212673 0.041983
+0.209533 0.205981 0.0261421
+0.207685 0.214484 0.0320655
+0.204793 0.215907 0.019768
+0.207322 0.190012 0.0316877
+0.210424 0.206724 0.0353988
+0.209086 0.197885 0.0355421
+0.19948 0.191764 0.0420701
+0.206287 0.1798 0.0239909
+0.199532 0.162119 0.0159658
+0.0889829 0.0153312 0.187549
+0.0895058 0.0194699 0.175832
+0.0997882 0.0258376 0.180178
+0.0912633 -0.43583 -0.104962
+0.0893849 -0.44209 -0.0966632
+0.0818551 -0.438899 -0.0891003
+0.0775492 -0.435915 -0.0759439
+0.0805228 -0.426773 -0.087989
+0.0848008 -0.421093 -0.0971385
+0.0844397 -0.410175 -0.0930928
+0.0796444 -0.399052 -0.082153
+0.096306 -0.399151 -0.0928503
+0.0992866 -0.434977 -0.10667
+-0.0453136 -0.074362 0.132403
+-0.0642973 -0.0816285 0.135216
+0.128958 0.371237 0.164377
+0.114324 0.368213 0.169385
+0.110394 0.358489 0.182698
+0.119359 0.382173 0.172221
+0.244566 0.0274799 0.091132
+0.236517 0.0321023 0.108593
+0.228129 0.0154777 0.110489
+0.243652 -0.0044318 0.106704
+0.215089 0.0116198 0.126655
+0.193164 -0.00203925 0.149761
+0.295901 -0.145148 0.11649
+0.181002 0.326878 0.162405
+0.192626 0.308402 0.164847
+0.180295 0.335983 0.171005
+0.215019 0.222685 -0.0085752
+0.216226 0.238327 -0.00679645
+0.204842 0.243639 -0.00108745
+0.197086 0.237151 0.00943393
+0.207737 0.21475 -0.00125832
+0.200663 0.225332 0.00994825
+0.227239 0.239308 -0.00361834
+0.210369 0.205349 -0.000451507
+0.212421 0.195338 0.00658041
+0.0965367 0.0723068 0.0490018
+-0.263237 -0.382248 -0.20077
+-0.334935 -0.395374 -0.246716
+0.0666968 0.0983382 0.0352739
+0.0735768 0.104043 0.0256427
+0.0854137 0.105714 0.0280147
+0.0966684 0.103408 0.0197312
+-0.293219 -0.246559 0.00868918
+-0.284829 -0.260543 0.00656712
+0.231694 -0.239358 -0.229883
+0.249173 -0.248287 -0.23972
+-0.313753 -0.278534 -0.156686
+-0.167921 -0.0222432 0.100354
+-0.166557 -0.0217484 0.0804222
+-0.137191 -0.0189498 0.0544508
+-0.193717 -0.0386652 0.0428719
+-0.162262 -0.0205653 0.0563587
+-0.148465 -0.0201145 0.071316
+-0.122512 -0.0229086 0.0762312
+-0.112808 -0.0225311 0.0535636
+-0.15755 -0.0225071 0.112728
+-0.13968 -0.0242143 0.109228
+-0.128528 -0.0323847 0.121144
+-0.137944 -0.0426964 0.133983
+0.338353 -0.331842 -0.233
+0.197716 0.0369013 -0.0158252
+0.225598 0.0269623 0.107608
+0.227611 0.0352699 0.116368
+0.216481 0.0416846 0.126455
+0.20366 0.0291575 0.127785
+0.197706 0.0139181 0.138946
+0.177004 0.0154396 0.15063
+0.196436 0.0406475 0.130552
+0.183582 0.0465791 0.137895
+0.171962 0.0340634 0.144092
+0.209838 0.0350513 0.1214
+0.122018 -0.0147207 0.202688
+0.134625 -0.010294 0.190699
+0.150345 -0.0204184 0.190322
+0.171467 -0.0443352 0.191435
+0.124137 -0.0275222 0.21069
+0.115004 -0.0324307 0.220668
+0.103488 -0.0235525 0.230568
+-0.234966 -0.250492 -0.00657503
+0.230136 -0.0629922 0.010065
+0.22781 -0.0439028 0.0052695
+0.226758 -0.0758146 -0.00318988
+0.218119 -0.0755566 -0.0185995
+0.210925 -0.0881128 -0.0299059
+0.195391 -0.0835698 -0.0397083
+0.187049 -0.0970928 -0.0513544
+0.215909 -0.10623 -0.0275177
+0.221663 -0.111792 -0.00835326
+0.21689 -0.135734 -0.006698
+0.187289 -0.0668273 -0.0361071
+0.170089 -0.0549841 -0.0384156
+0.158135 -0.0292105 -0.0224101
+0.144698 -0.0631652 -0.0561794
+0.210349 -0.156804 0.00619425
+0.122465 -0.0210506 -0.020971
+0.25102 0.0827579 -0.00901225
+0.246076 0.0717907 -0.00732438
+0.248146 0.0653429 0.00439784
+0.245544 0.0538898 0.0151974
+0.255748 0.0868229 -0.000826293
+-0.125725 -0.258433 -0.170214
+0.151089 -0.0268375 0.140451
+0.244155 0.0131187 -0.0533056
+0.246127 0.0105937 -0.0612737
+0.239403 0.00492409 -0.0645919
+0.228547 -0.00190445 -0.0800819
+0.236042 0.000460551 -0.073323
+0.244455 -0.00400961 -0.0816292
+0.225173 0.00380285 -0.0703884
+0.247885 -0.00234363 -0.0902777
+0.247838 0.00379159 -0.0839495
+0.243353 0.0148187 -0.0838177
+0.236031 0.0244971 -0.0791546
+0.225616 0.0291453 -0.0875646
+-0.29518 -0.390079 -0.254006
+-0.303646 -0.394416 -0.248506
+-0.302477 -0.403327 -0.233435
+-0.298023 -0.412267 -0.217588
+-0.29425 -0.38151 -0.250683
+-0.302074 -0.390767 -0.254186
+0.191342 0.435898 0.0636941
+-0.0964353 -0.460913 -0.0144457
+0.137532 -0.0116873 0.139128
+0.343547 -0.294973 -0.292923
+0.137424 0.0947211 0.0118894
+0.147115 0.0902563 -0.00271409
+0.147874 0.0802052 -0.0226835
+0.255027 -0.310713 -0.257667
+0.257498 -0.300887 -0.266482
+0.220835 -0.0105204 -0.15317
+0.210403 -0.014095 -0.156518
+0.204562 -0.0228168 -0.148776
+-0.117774 -0.20207 0.202016
+0.181641 -0.136376 0.0700095
+-0.0985707 -0.0477827 -0.098696
+0.286002 0.0789154 0.0896861
+0.309738 -0.0894098 0.0874741
+0.220704 0.448894 0.117889
+0.047335 -0.0888232 0.265922
+0.172006 0.242405 0.105809
+0.283677 -0.0568988 0.0744319
+0.0958416 -0.324563 -0.0739969
+0.00993129 -0.0278328 -0.0155697
+0.137574 -0.108705 0.207737
+0.262967 -0.0160504 0.0796628
+0.222165 -0.00765217 0.131632
+0.215089 0.0116198 0.126655
+0.344989 -0.280039 -0.276203
+-0.311345 -0.301149 -0.0633156
+-0.298437 -0.298535 -0.0585809
+-0.254096 -0.191499 0.0837602
+0.0761558 -0.344791 -0.0536197
+0.218046 0.211438 0.0580939
+0.0876406 0.00969553 -0.149119
+0.23393 -0.0149701 -0.10397
+0.209378 -0.187531 -0.0743719
+0.216481 0.0416846 0.126455
+0.143956 0.303159 0.203618
+0.19948 -0.240785 -0.114164
+0.286002 0.0789154 0.0896861
+0.217938 0.25177 0.117669
+0.144424 0.408706 0.223131
+0.234086 0.289095 0.199595
+0.260427 0.269097 0.206442
+0.26654 -0.211351 -0.150233
+0.119959 0.124948 0.0613596
+0.127454 0.104238 0.0219653
+-0.267819 -0.241114 -0.0357093
+0.163143 -0.233988 -0.192556
+0.161209 0.00758193 -0.157082
+0.200684 0.169917 0.154336
+0.265642 -0.321399 -0.233195
+-0.291326 -0.23573 -0.0313163
+0.355354 -0.294774 -0.257468
+0.24497 -0.0122967 -0.113875
+0.304942 -0.268737 -0.28762
+-0.345548 -0.417553 -0.216974
+0.161752 -0.208101 -0.1989
+0.33579 -0.291676 -0.2964
+-0.304665 -0.364526 -0.223447
+-0.0814148 -0.0448428 -0.0722969
+0.221779 0.470802 0.0892895
+0.156487 -0.13924 0.256434
+0.015326 -0.129347 0.142131
+0.170691 0.206066 0.147249
+-0.293564 -0.290291 -0.0772666
+0.290218 0.138764 0.11928
+0.178601 -0.140578 0.237092
+-0.227331 -0.265687 0.0979085
+-0.332554 -0.290038 -0.0650158
+0.229872 0.295702 0.229994
+0.112679 0.0343949 0.172228
+0.0422003 -0.0728232 0.283477
+0.266185 -0.201338 -0.169529
+0.0606107 -0.0431898 0.263976
+0.0689625 -0.0703705 0.29266
+0.160239 0.479513 0.0724411
+0.253586 0.0320272 0.0977802
+0.0843142 0.0418575 -0.0419302
+-0.31358 -0.30952 -0.0911544
+0.20517 0.444041 0.0615
+-0.312242 -0.307406 -0.0760424
+0.209982 -0.159856 0.187271
+0.211408 0.277777 0.0261684
+0.0315681 -0.295788 -0.0489563
+0.168396 -0.264129 0.0198747
+0.280108 -0.155995 0.136908
+0.0557269 -0.0839063 0.29826
+-0.280983 -0.308012 -0.105167
+0.288054 0.130079 0.0326142
+0.282604 0.11846 0.0804122
+-0.29694 -0.227134 0.00135872
+0.211416 0.21985 0.0506815
+3  568 1200 1210
+3  893 134 1151
+3  2085 16 2088
+3  1312 1311 325
+3  1019 1025 1020
+3  1220 1219 727
+3  975 1216 179
+3  525 527 182
+3  527 2718 182
+3  1564 1546 1563
+3  665 666 630
+3  1117 1210 239
+3  276 5 277
+3  5 280 277
+3  614 945 946
+3  232 270 272
+3  2510 1719 2511
+3  2184 2183 2182
+3  10 203 275
+3  913 92 2048
+3  523 2666 2667
+3  2107 2109 2102
+3  1353 1355 757
+3  1340 1353 757
+3  981 95 979
+3  1654 1655 30
+3  1659 1655 1654
+3  2032 2033 292
+3  738 739 458
+3  992 2153 2152
+3  2146 992 2152
+3  211 37 212
+3  2164 2167 467
+3  2668 254 2671
+3  213 547 935
+3  1433 1432 439
+3  975 2117 95
+3  452 1011 1010
+3  1758 1763 511
+3  1562 2716 1974
+3  51 1461 1462
+3  2066 51 1462
+3  2323 2329 1359
+3  2022 1475 73
+3  326 323 1734
+3  924 925 173
+3  520 562 696
+3  1972 244 1971
+3  1123 351 349
+3  1864 1863 131
+3  79 26 1575
+3  1465 1656 172
+3  1920 1921 2564
+3  20 103 102
+3  291 2719 1909
+3  1727 1728 334
+3  687 1172 1170
+3  1171 687 1170
+3  1465 65 1656
+3  67 898 120
+3  257 1720 338
+3  1319 1320 445
+3  797 394 472
+3  1316 741 719
+3  1492 2720 1491
+3  276 10 275
+3  274 276 275
+3  255 256 75
+3  677 674 710
+3  1217 941 1215
+3  1573 1574 1946
+3  90 2041 966
+3  715 723 2721
+3  84 1950 2205
+3  204 151 63
+3  1968 1966 1967
+3  75 258 1594
+3  256 258 75
+3  673 2722 1480
+3  293 16 294
+3  16 2085 294
+3  547 213 907
+3  378 380 379
+3  268 266 293
+3  266 16 293
+3  1070 647 1794
+3  113 196 193
+3  1496 2519 2207
+3  209 543 235
+3  1948 1574 26
+3  361 71 362
+3  665 648 664
+3  81 165 155
+3  1120 632 1156
+3  434 1384 2723
+3  795 794 395
+3  156 2065 154
+3  1475 138 1474
+3  2107 2100 2109
+3  355 2100 2107
+3  560 527 524
+3  63 261 271
+3  134 362 377
+3  376 134 377
+3  115 894 893
+3  374 373 133
+3  82 1279 1286
+3  1790 683 1793
+3  1618 577 701
+3  496 493 495
+3  1824 1246 53
+3  1679 1556 510
+3  1154 1057 1056
+3  41 1796 1799
+3  80 79 83
+3  19 103 116
+3  19 153 103
+3  1004 255 1005
+3  256 255 64
+3  640 881 1638
+3  1989 114 1988
+3  1133 2724 1134
+3  123 1591 1594
+3  161 1862 160
+3  2717 375 374
+3  1768 1861 518
+3  503 504 408
+3  1130 745 302
+3  857 1787 1786
+3  2629 1787 857
+3  1646 1026 1645
+3  155 165 50
+3  741 2725 459
+3  720 2725 741
+3  985 987 60
+3  277 278 10
+3  910 92 909
+3  583 2257 1301
+3  1285 1454 1458
+3  67 120 116
+3  235 530 548
+3  135 362 894
+3  1512 1538 1511
+3  261 1692 265
+3  2047 2048 92
+3  5 270 280
+3  2499 62 243
+3  233 314 316
+3  39 83 79
+3  83 39 102
+3  769 1336 1335
+3  1338 769 1335
+3  167 260 262
+3  935 180 1712
+3  935 936 180
+3  1862 161 162
+3  62 1973 243
+3  311 232 272
+3  429 430 185
+3  1718 1719 342
+3  1146 2385 2386
+3  2411 587 1089
+3  1460 1285 1458
+3  2 1123 1124
+3  1050 2462 2463
+3  2726 986 1704
+3  261 265 271
+3  433 1403 1232
+3  2665 2664 523
+3  116 103 104
+3  161 118 119
+3  70 128 162
+3  62 244 1973
+3  1862 162 1863
+3  2022 2020 2021
+3  251 104 246
+3  251 67 104
+3  978 177 980
+3  260 23 262
+3  1989 1988 923
+3  1696 167 1463
+3  121 1005 75
+3  274 275 63
+3  425 148 2727
+3  2184 2700 2151
+3  809 402 472
+3  1475 234 73
+3  1962 1552 2556
+3  2640 895 888
+3  895 890 888
+3  374 375 132
+3  129 71 361
+3  279 107 290
+3  107 279 316
+3  221 82 1286
+3  73 378 373
+3  2022 2021 138
+3  2185 2622 2183
+3  1121 1414 1413
+3  407 805 804
+3  123 1594 258
+3  890 1959 1957
+3  97 1295 272
+3  919 1739 922
+3  141 2094 2099
+3  22 231 77
+3  334 1466 329
+3  1466 1467 329
+3  1327 1320 718
+3  1567 15 1566
+3  540 91 542
+3  2156 2185 2151
+3  2195 2527 2526
+3  375 71 132
+3  1768 1756 1861
+3  275 203 204
+3  167 81 1463
+3  262 23 263
+3  305 150 934
+3  307 305 934
+3  562 558 174
+3  264 1955 1478
+3  233 313 314
+3  1863 162 131
+3  1866 2343 1455
+3  1440 1444 1439
+3  747 894 115
+3  135 894 747
+3  942 1216 975
+3  976 942 975
+3  439 184 440
+3  1694 151 901
+3  83 165 80
+3  143 360 2106
+3  1302 1303 2229
+3  67 116 104
+3  1031 1026 1646
+3  1216 1217 1215
+3  1316 720 741
+3  913 908 92
+3  816 392 815
+3  503 386 504
+3  754 1173 2091
+3  553 1334 176
+3  553 1333 1334
+3  890 895 1959
+3  630 1120 1125
+3  665 630 1125
+3  1235 1228 1237
+3  2658 2657 1409
+3  495 492 496
+3  1663 1198 1197
+3  1321 445 1320
+3  1376 2728 437
+3  920 919 921
+3  1824 1823 1245
+3  1823 1826 1245
+3  2676 2679 238
+3  990 976 982
+3  1446 2707 1435
+3  261 1690 1689
+3  343 327 1126
+3  1124 1123 350
+3  2013 113 1994
+3  140 1470 1471
+3  1463 1685 1696
+3  208 963 964
+3  316 315 107
+3  316 314 315
+3  2049 2045 206
+3  204 58 901
+3  1742 238 839
+3  539 848 1781
+3  86 340 1722
+3  614 906 615
+3  390 794 797
+3  966 965 964
+3  1984 981 925
+3  981 173 925
+3  1766 2687 2688
+3  275 204 63
+3  517 1665 1664
+3  2715 1675 2714
+3  1675 2715 1562
+3  187 2538 2539
+3  793 390 579
+3  1749 1755 1765
+3  470 807 387
+3  509 508 1552
+3  318 2025 2031
+3  470 471 502
+3  1165 764 1166
+3  1212 1213 940
+3  1860 252 100
+3  1887 593 2217
+3  121 1003 1004
+3  270 97 272
+3  222 1588 1589
+3  47 310 311
+3  532 2671 2672
+3  2671 532 531
+3  785 2197 2196
+3  304 38 308
+3  38 304 317
+3  1559 1675 1558
+3  405 945 614
+3  453 433 1234
+3  522 1487 2683
+3  1242 444 1241
+3  1650 2132 2137
+3  502 386 503
+3  1677 1676 1680
+3  1676 1559 1680
+3  1620 1621 577
+3  457 2729 2604
+3  172 920 921
+3  377 71 375
+3  362 71 377
+3  2478 132 2479
+3  125 1953 1955
+3  1108 2149 285
+3  132 129 2479
+3  75 1005 255
+3  115 893 892
+3  129 132 71
+3  899 136 900
+3  122 75 1594
+3  75 122 121
+3  728 727 736
+3  29 534 536
+3  29 533 534
+3  392 817 815
+3  257 1723 1721
+3  338 1718 1725
+3  1128 328 1126
+3  1728 1466 334
+3  2612 2611 691
+3  1723 1724 256
+3  227 1359 2332
+3  2100 2101 2109
+3  581 1670 1071
+3  1057 1154 632
+3  10 276 277
+3  910 1310 210
+3  1602 706 3
+3  2329 2323 2322
+3  2730 2731 2650
+3  317 315 38
+3  315 314 38
+3  5 269 270
+3  2134 2732 1039
+3  1046 2125 2124
+3  234 1475 139
+3  1771 516 1494
+3  231 22 2727
+3  22 1147 2269
+3  2154 2622 2186
+3  2625 2624 823
+3  233 316 279
+3  808 387 799
+3  502 503 470
+3  1634 1450 485
+3  2733 1633 2734
+3  188 2658 2659
+3  1125 1120 1156
+3  826 831 407
+3  831 1169 407
+3  135 747 745
+3  21 2037 297
+3  2735 746 1007
+3  1743 2676 238
+3  2676 1743 515
+3  376 377 375
+3  1721 1723 64
+3  1701 1700 194
+3  297 2037 2036
+3  894 134 893
+3  24 329 991
+3  1147 77 1015
+3  144 423 792
+3  357 356 299
+3  745 1130 13
+3  2649 2652 413
+3  1198 568 567
+3  1200 568 1198
+3  2319 709 2320
+3  528 181 537
+3  603 1182 391
+3  830 146 828
+3  947 146 830
+3  665 1125 648
+3  337 323 326
+3  1406 1392 1611
+3  2608 1793 683
+3  935 546 936
+3  528 537 526
+3  1712 213 935
+3  331 89 333
+3  2427 774 2432
+3  918 1739 920
+3  1250 307 934
+3  2431 943 774
+3  1131 585 1133
+3  920 172 918
+3  2736 1703 1702
+3  2238 2250 2384
+3  1782 1784 1783
+3  219 848 849
+3  343 1126 1732
+3  800 799 387
+3  289 1061 1060
+3  1100 1927 1930
+3  1737 1736 1103
+3  502 811 835
+3  412 830 2653
+3  1439 1444 477
+3  2677 2675 530
+3  561 733 725
+3  2028 2027 295
+3  184 1124 350
+3  1414 1391 435
+3  463 130 744
+3  892 893 357
+3  893 1151 357
+3  585 1131 802
+3  148 231 2727
+3  2278 706 2279
+3  2737 1619 1618
+3  491 777 2551
+3  1477 2639 143
+3  2642 142 889
+3  370 1938 1256
+3  1214 974 179
+3  381 234 139
+3  1256 1258 1263
+3  891 1957 1178
+3  378 234 380
+3  830 828 410
+3  2662 431 1674
+3  780 1538 1512
+3  2549 2738 1549
+3  811 502 471
+3  1169 831 411
+3  2660 1409 2656
+3  431 2660 2656
+3  1201 1204 1202
+3  391 1180 605
+3  235 543 544
+3  1191 522 1192
+3  1448 187 2582
+3  1750 1766 2688
+3  1070 1066 647
+3  1681 2224 2739
+3  2091 1173 687
+3  816 507 789
+3  290 2031 291
+3  212 340 86
+3  844 845 537
+3  845 844 2740
+3  2712 1048 1116
+3  209 540 541
+3  119 473 117
+3  775 613 406
+3  1299 603 1300
+3  1494 2683 521
+3  405 944 945
+3  948 2652 2741
+3  1635 1446 1634
+3  884 762 878
+3  1394 2567 1673
+3  1327 1321 1320
+3  520 521 1334
+3  449 430 448
+3  1677 1678 1561
+3  1676 1677 1561
+3  1868 839 1869
+3  397 1302 1622
+3  416 2209 1529
+3  2209 416 1500
+3  659 657 662
+3  1445 476 475
+3  1425 1635 1634
+3  615 903 405
+3  835 2625 823
+3  457 739 738
+3  461 1409 2660
+3  726 1010 724
+3  1201 517 1204
+3  831 947 411
+3  403 802 1131
+3  443 440 441
+3  2674 529 2669
+3  569 1208 1206
+3  455 2659 189
+3  591 2609 2608
+3  1131 1133 582
+3  618 1371 1372
+3  562 174 696
+3  1369 1370 642
+3  1771 1494 237
+3  1076 599 617
+3  456 455 189
+3  189 453 450
+3  664 667 627
+3  1011 1328 474
+3  2665 174 2664
+3  456 175 455
+3  604 1337 1336
+3  1606 1436 383
+3  2670 530 2669
+3  739 175 456
+3  474 724 1010
+3  972 1519 1521
+3  524 527 525
+3  769 604 1336
+3  2738 2549 2550
+3  560 558 559
+3  739 456 458
+3  407 807 825
+3  734 737 458
+3  599 2742 1780
+3  1742 1743 238
+3  825 826 407
+3  414 789 507
+3  1430 1635 1425
+3  2263 422 2266
+3  471 470 387
+3  1502 1499 1504
+3  1538 1497 782
+3  544 45 545
+3  490 494 511
+3  781 1538 1528
+3  1197 567 1199
+3  1564 1563 1547
+3  2007 1992 2006
+3  1698 1699 575
+3  543 45 544
+3  271 2685 63
+3  1751 1753 2698
+3  1753 1754 2698
+3  496 499 493
+3  1539 1565 1563
+3  498 491 508
+3  798 2202 2200
+3  1565 1541 500
+3  1708 322 1709
+3  1506 1507 1501
+3  863 1040 1039
+3  1772 2696 2695
+3  596 1057 712
+3  2609 644 678
+3  1489 552 1488
+3  2425 1699 396
+3  2691 2693 2690
+3  237 1745 1771
+3  1745 1746 1771
+3  744 743 135
+3  1058 618 1372
+3  1059 618 1058
+3  735 734 458
+3  794 394 797
+3  1009 452 1010
+3  845 533 537
+3  533 526 537
+3  802 801 171
+3  1316 1317 1329
+3  236 523 529
+3  239 1210 1201
+3  49 728 736
+3  533 532 526
+3  532 533 29
+3  892 1958 891
+3  706 1596 3
+3  209 541 543
+3  538 536 534
+3  220 538 534
+3  2409 2408 1653
+3  538 539 540
+3  220 539 538
+3  209 538 540
+3  936 937 180
+3  542 541 540
+3  2691 2690 2692
+3  214 908 911
+3  170 2743 916
+3  846 535 847
+3  45 200 545
+3  1069 94 653
+3  146 947 831
+3  1481 553 1483
+3  733 49 736
+3  176 1485 1484
+3  1483 553 176
+3  2519 147 2207
+3  656 629 684
+3  564 1750 2697
+3  718 742 474
+3  876 873 756
+3  174 558 560
+3  2724 1305 580
+3  549 1492 1485
+3  1663 1664 1198
+3  716 1220 727
+3  1353 760 1354
+3  1340 760 1353
+3  1322 559 558
+3  445 1322 558
+3  698 1054 596
+3  698 623 1054
+3  1064 647 1066
+3  577 1619 1620
+3  591 643 2609
+3  2581 2580 570
+3  764 604 905
+3  604 764 763
+3  905 615 906
+3  671 688 598
+3  654 637 636
+3  723 715 724
+3  611 610 1167
+3  610 611 2744
+3  835 483 502
+3  688 671 626
+3  1058 588 1059
+3  1796 105 56
+3  805 404 804
+3  627 663 664
+3  891 115 892
+3  1191 566 1190
+3  649 670 666
+3  2597 1792 589
+3  405 614 615
+3  2681 2680 1192
+3  640 1638 882
+3  613 0 614
+3  671 669 626
+3  665 649 666
+3  591 2608 2610
+3  1779 595 1778
+3  689 701 577
+3  701 689 1083
+3  392 789 819
+3  1344 1351 1343
+3  1350 1351 1344
+3  404 2431 818
+3  1300 603 602
+3  1138 609 1208
+3  2139 2138 1032
+3  0 906 614
+3  2608 2609 678
+3  2616 2615 573
+3  2680 1196 2684
+3  1325 447 1324
+3  2306 1194 1189
+3  836 2620 824
+3  2280 706 1601
+3  1138 1137 609
+3  2307 2299 1186
+3  2745 2232 581
+3  608 1492 549
+3  550 1492 1491
+3  735 49 734
+3  1132 1131 582
+3  1166 1167 612
+3  1167 1166 611
+3  2491 2490 1602
+3  613 614 406
+3  1353 1354 761
+3  400 1335 1299
+3  1338 1335 400
+3  142 2107 2102
+3  1332 446 1331
+3  1445 2104 352
+3  134 376 1150
+3  775 551 613
+3  2746 684 629
+3  659 662 663
+3  2696 1751 2697
+3  1057 713 712
+3  645 1172 1173
+3  628 669 671
+3  1660 1661 651
+3  804 387 807
+3  662 658 649
+3  658 670 649
+3  2305 1180 606
+3  656 588 657
+3  629 657 659
+3  702 1083 1082
+3  760 1340 1337
+3  749 2747 1893
+3  1957 891 1958
+3  1168 1169 411
+3  1396 1385 2287
+3  628 671 676
+3  663 662 649
+3  660 2746 629
+3  627 660 659
+3  660 629 659
+3  1887 1889 593
+3  807 407 804
+3  684 619 656
+3  1070 1065 1066
+3  627 659 663
+3  657 588 658
+3  1617 1618 701
+3  875 639 2748
+3  1661 698 651
+3  628 676 661
+3  142 356 2107
+3  1876 1875 886
+3  697 651 698
+3  1830 1232 1829
+3  712 713 633
+3  599 1076 1069
+3  1331 446 1330
+3  2115 673 1480
+3  848 539 220
+3  708 1603 2490
+3  1752 1769 2693
+3  738 737 556
+3  1120 630 1119
+3  1524 1513 1540
+3  956 955 249
+3  483 835 823
+3  1160 1161 1165
+3  1533 1535 784
+3  1698 1615 1977
+3  981 979 173
+3  850 219 849
+3  2749 219 850
+3  723 459 722
+3  459 2725 722
+3  354 2093 2092
+3  459 723 724
+3  1345 759 1347
+3  737 740 556
+3  722 2721 723
+3  734 561 737
+3  561 734 733
+3  1555 509 1962
+3  764 1161 763
+3  947 830 412
+3  49 733 734
+3  1218 727 1219
+3  742 724 474
+3  894 362 134
+3  459 724 742
+3  742 719 741
+3  459 742 741
+3  743 362 135
+3  1890 1891 749
+3  1368 2610 1371
+3  568 796 93
+3  872 753 685
+3  1418 2444 1121
+3  878 877 756
+3  401 767 1350
+3  945 412 946
+3  583 1301 1302
+3  134 1150 1151
+3  764 1165 1161
+3  615 905 902
+3  433 461 1402
+3  773 584 772
+3  756 884 878
+3  408 826 825
+3  796 814 394
+3  1335 768 1299
+3  1009 1221 1222
+3  784 1534 1533
+3  2630 2629 857
+3  2274 1008 2273
+3  1533 415 1535
+3  283 2141 229
+3  1649 2138 2139
+3  351 2096 349
+3  390 793 794
+3  402 797 472
+3  945 947 412
+3  768 2440 1299
+3  1197 1196 241
+3  804 800 387
+3  1169 1168 805
+3  826 146 831
+3  1114 2433 944
+3  2750 812 811
+3  809 472 810
+3  796 393 814
+3  2750 811 471
+3  1196 566 2684
+3  392 816 789
+3  2195 820 788
+3  789 820 819
+3  947 944 411
+3  944 947 945
+3  1169 805 407
+3  1911 2521 1109
+3  481 837 836
+3  15 2553 1569
+3  1562 2715 2716
+3  847 220 846
+3  2156 2151 2701
+3  236 529 2674
+3  2253 2254 856
+3  2132 2128 1034
+3  1125 1156 648
+3  864 1043 1023
+3  2014 2013 927
+3  1152 43 357
+3  866 2254 2713
+3  1039 1040 1022
+3  905 906 764
+3  303 13 1130
+3  2638 2637 298
+3  2308 2309 607
+3  685 753 1888
+3  1874 1875 1346
+3  1129 301 1130
+3  897 115 891
+3  747 115 897
+3  922 1739 1738
+3  745 747 897
+3  211 901 58
+3  1116 861 1115
+3  76 334 1113
+3  908 214 907
+3  908 907 909
+3  990 987 942
+3  1371 1789 1372
+3  1711 1712 180
+3  1063 1111 1112
+3  955 954 249
+3  1470 1472 1471
+3  2651 2606 2650
+3  1849 1850 2391
+3  393 2709 2751
+3  814 393 2751
+3  2654 410 2651
+3  412 2653 948
+3  997 467 994
+3  286 1061 951
+3  70 127 128
+3  2018 336 2019
+3  1601 706 1602
+3  2470 2290 2469
+3  2159 2362 2160
+3  2622 2154 2183
+3  2387 2262 2259
+3  121 1004 1005
+3  245 252 59
+3  467 992 994
+3  2153 992 467
+3  2004 982 2003
+3  1192 2682 2681
+3  1227 1809 1810
+3  917 1654 1653
+3  1096 856 1097
+3  859 1096 1097
+3  1025 1026 1031
+3  2711 2710 1117
+3  2070 1278 2071
+3  2332 2330 2752
+3  1359 2330 2332
+3  1274 1140 1270
+3  984 60 983
+3  1236 1825 1662
+3  2076 1279 226
+3  2378 880 2376
+3  1257 1256 1252
+3  370 1256 1257
+3  1586 1587 1141
+3  1586 465 1587
+3  2083 223 2084
+3  2481 1500 1502
+3  1354 885 761
+3  2040 201 2041
+3  2546 2549 2547
+3  2546 1551 2549
+3  970 969 968
+3  970 968 218
+3  913 419 911
+3  419 913 2753
+3  2753 913 217
+3  56 1797 1796
+3  2130 2133 2131
+3  205 2049 211
+3  761 2217 593
+3  2217 761 874
+3  2053 964 963
+3  542 963 208
+3  332 1856 1855
+3  1313 2052 210
+3  965 541 208
+3  2047 2046 207
+3  1727 334 1726
+3  1311 210 1310
+3  860 1645 1642
+3  278 57 10
+3  57 278 1906
+3  1786 1787 216
+3  1787 968 216
+3  1783 1786 216
+3  987 1217 942
+3  1965 1964 1963
+3  2061 2059 218
+3  2063 967 960
+3  967 962 960
+3  2058 2061 960
+3  2061 2063 960
+3  2063 2061 2062
+3  1595 540 539
+3  1964 912 1963
+3  2061 218 2062
+3  1792 594 1791
+3  1741 1553 2754
+3  1557 1741 2754
+3  916 2743 1965
+3  973 313 230
+3  300 2635 2036
+3  2092 2093 2110
+3  1505 1501 783
+3  1506 1505 1499
+3  2671 254 2672
+3  2617 690 2612
+3  690 2611 2612
+3  1517 1516 54
+3  1512 1516 1517
+3  1509 1507 1506
+3  345 1939 1966
+3  1499 1508 1506
+3  2209 2210 1529
+3  2210 2209 787
+3  667 648 1608
+3  1503 416 972
+3  2207 147 2212
+3  2208 2207 2212
+3  2636 2633 2637
+3  21 295 296
+3  2600 688 626
+3  778 2600 626
+3  2544 1940 2542
+3  1940 1941 2542
+3  313 38 314
+3  313 973 38
+3  973 308 38
+3  1372 1789 594
+3  36 192 1701
+3  978 979 95
+3  1657 8 1655
+3  987 990 60
+3  65 1659 1656
+3  979 921 173
+3  921 924 173
+3  295 2027 292
+3  438 1436 1437
+3  438 2686 1436
+3  192 985 984
+3  1650 1649 1033
+3  974 177 978
+3  177 974 417
+3  686 1171 2755
+3  1672 1051 1642
+3  921 919 924
+3  1018 1020 851
+3  1018 2467 1020
+3  1854 2585 2594
+3  1146 2386 2243
+3  1469 331 333
+3  37 1313 341
+3  1836 1248 1833
+3  1652 4 199
+3  2018 124 1590
+3  124 1592 1590
+3  913 911 908
+3  192 36 985
+3  1703 2726 1704
+3  1729 326 1732
+3  348 33 988
+3  324 348 988
+3  765 1164 1348
+3  1467 991 329
+3  2510 87 2514
+3  985 986 987
+3  988 980 177
+3  989 988 177
+3  989 324 988
+3  2103 353 2104
+3  1715 1714 321
+3  989 177 417
+3  976 990 942
+3  1017 855 1018
+3  951 289 1104
+3  1061 289 951
+3  1927 1926 1101
+3  1970 246 245
+3  246 66 245
+3  2163 995 2165
+3  2162 2172 2173
+3  1907 1903 1904
+3  1907 110 1903
+3  1509 971 1510
+3  1514 1509 1510
+3  69 121 122
+3  1003 121 69
+3  1996 983 2010
+3  2009 1996 2010
+3  2154 994 2148
+3  1930 1928 1931
+3  1933 347 1928
+3  2149 952 2150
+3  40 1968 1967
+3  120 898 1003
+3  69 120 1003
+3  347 1101 2175
+3  2171 2162 2166
+3  2171 2166 995
+3  2612 691 2613
+3  138 2021 2024
+3  999 2175 2170
+3  2258 1143 1578
+3  483 823 2623
+3  999 2168 2166
+3  1002 2153 467
+3  2167 1002 467
+3  2506 163 2505
+3  2163 2164 997
+3  2165 2164 2163
+3  2324 2323 2325
+3  1264 2349 2345
+3  974 2117 975
+3  2044 2040 2041
+3  2044 205 2040
+3  305 307 1007
+3  746 305 1007
+3  305 746 13
+3  2300 1185 2301
+3  728 731 1223
+3  1009 1010 726
+3  729 728 49
+3  49 735 729
+3  1316 1329 720
+3  474 1010 1011
+3  729 735 732
+3  451 729 732
+3  1811 1812 1227
+3  1666 1664 1495
+3  2728 436 185
+3  2023 2024 2021
+3  2024 2023 1471
+3  333 1473 1472
+3  2644 1651 1646
+3  1645 2644 1646
+3  430 449 2728
+3  185 430 2728
+3  453 461 433
+3  693 1110 1080
+3  1014 833 834
+3  1080 1110 832
+3  1913 2523 1911
+3  2628 2339 368
+3  2339 2628 2341
+3  1502 1500 1503
+3  2211 787 2194
+3  1064 1112 647
+3  682 2273 2272
+3  695 833 1014
+3  339 1719 1720
+3  1719 338 1720
+3  2033 2032 2034
+3  2047 92 910
+3  2046 2045 2050
+3  2045 90 2050
+3  1216 1215 179
+3  1651 2140 1647
+3  1649 2139 2140
+3  2139 1032 2140
+3  2756 2460 1029
+3  374 133 2717
+3  865 1021 1020
+3  1045 2125 2120
+3  1221 726 1220
+3  1786 1783 1785
+3  492 1554 496
+3  2272 2273 705
+3  1596 2272 705
+3  379 1150 376
+3  1021 864 1023
+3  1021 1022 864
+3  1223 716 728
+3  852 1018 851
+3  1024 852 851
+3  1023 1024 851
+3  1906 2442 57
+3  852 1017 1018
+3  1773 1267 1257
+3  2757 1773 1257
+3  928 1981 1989
+3  1646 1647 1031
+3  1392 1406 2567
+3  1406 1395 2567
+3  2123 1045 2122
+3  1025 862 1026
+3  27 1575 1573
+3  1801 27 1573
+3  2123 2124 2125
+3  2758 2134 1038
+3  960 419 2753
+3  649 665 664
+3  779 597 1053
+3  622 779 1053
+3  650 2116 1008
+3  2116 2275 1008
+3  419 960 962
+3  859 1092 1094
+3  1096 859 1094
+3  34 1996 1995
+3  1996 195 1995
+3  1982 2000 1999
+3  2000 1983 1999
+3  1035 2133 2130
+3  1034 2128 2127
+3  1805 1731 259
+3  864 1041 1043
+3  1687 152 1686
+3  2461 2460 1028
+3  2674 2669 530
+3  1678 1677 1560
+3  609 2304 1188
+3  235 536 209
+3  2732 863 1039
+3  1038 1039 1022
+3  2759 1680 565
+3  1654 30 1653
+3  2465 1019 2466
+3  1019 2467 2466
+3  2460 2756 1049
+3  1021 865 1022
+3  1022 1040 864
+3  1040 1041 864
+3  1043 854 1023
+3  1020 1021 851
+3  1021 1023 851
+3  1042 854 1043
+3  2760 2253 871
+3  2253 2760 867
+3  1935 1420 2443
+3  1044 1040 863
+3  1679 501 2226
+3  1556 1679 2226
+3  1785 854 1042
+3  1760 1762 1761
+3  1984 1985 2001
+3  2156 2761 2155
+3  2060 2762 958
+3  2763 1034 2127
+3  2124 2123 869
+3  2589 2588 8
+3  1976 2216 2217
+3  874 1976 2217
+3  993 2764 2149
+3  664 663 649
+3  931 1947 1944
+3  196 1854 193
+3  2712 866 2713
+3  1093 855 1017
+3  914 1093 1017
+3  2458 2460 1049
+3  631 1156 1155
+3  1156 632 1155
+3  2183 2154 2182
+3  2118 861 1048
+3  237 521 520
+3  1494 521 237
+3  2003 982 977
+3  2064 2053 963
+3  2509 1714 1717
+3  1981 2014 927
+3  928 2014 1981
+3  1657 1659 65
+3  622 1053 1054
+3  1055 622 1054
+3  658 1055 623
+3  1055 1054 623
+3  588 1055 658
+3  1078 2420 2418
+3  1054 1056 596
+3  1054 1053 1056
+3  1155 626 631
+3  1902 1900 1880
+3  1896 1895 749
+3  1055 588 1058
+3  1056 1053 1154
+3  465 1588 1587
+3  1588 222 1587
+3  1055 1058 622
+3  2354 268 2355
+3  656 655 1059
+3  655 656 619
+3  2088 2089 221
+3  703 1063 2534
+3  2533 703 2534
+3  605 396 576
+3  396 605 1137
+3  1064 1066 1062
+3  664 648 667
+3  1102 1737 1103
+3  288 1102 1103
+3  644 1071 1070
+3  678 644 1070
+3  647 1795 1794
+3  1065 1070 1071
+3  1063 1112 1064
+3  1081 1080 703
+3  702 701 1083
+3  1081 693 1080
+3  577 1979 689
+3  1979 577 1621
+3  1074 1068 590
+3  1068 1074 94
+3  586 1075 1074
+3  1066 1065 1072
+3  1879 1373 636
+3  1879 1894 1373
+3  681 2415 1089
+3  1064 1062 2534
+3  1793 1794 679
+3  170 1091 535
+3  1078 1077 2420
+3  1086 592 1078
+3  681 1086 1078
+3  1505 1504 1499
+3  587 1085 1086
+3  1065 1073 1072
+3  590 1068 1067
+3  1931 1928 1099
+3  2378 2377 748
+3  2377 2378 2376
+3  399 2377 2376
+3  751 644 750
+3  751 1071 644
+3  47 311 272
+3  2534 1063 1064
+3  2533 2534 1062
+3  1133 1135 2724
+3  1133 585 1135
+3  702 2508 574
+3  584 773 1309
+3  1133 1134 582
+3  1134 2724 580
+3  1074 1075 94
+3  1075 646 94
+3  646 653 94
+3  1791 594 1790
+3  2249 2239 2320
+3  2244 2249 2320
+3  1503 971 1570
+3  953 1111 1088
+3  1188 1189 609
+3  1080 832 1063
+3  2421 1077 2422
+3  396 2424 2425
+3  2424 396 1138
+3  2028 2765 25
+3  785 2196 2191
+3  596 712 791
+3  712 711 791
+3  2080 2077 2078
+3  2418 2420 2419
+3  680 2418 2419
+3  2420 1077 2421
+3  2419 2420 2421
+3  617 2422 1076
+3  1086 1067 592
+3  1113 329 335
+3  329 24 335
+3  2722 677 1480
+3  674 677 2722
+3  953 1087 1079
+3  1066 1067 1085
+3  1067 1066 590
+3  1062 1085 587
+3  1062 1066 1085
+3  587 1086 1089
+3  2507 2508 702
+3  832 1111 1063
+3  1085 1067 1086
+3  1436 1606 1437
+3  1732 326 1733
+3  246 1970 2706
+3  251 246 2706
+3  916 1091 170
+3  1112 1111 953
+3  668 1609 1610
+3  1168 806 805
+3  2766 2240 2245
+3  2246 2766 2245
+3  2660 2661 1408
+3  847 535 1091
+3  849 847 1091
+3  849 848 847
+3  849 1091 916
+3  849 916 850
+3  587 2411 2412
+3  1089 1086 681
+3  2523 1014 2521
+3  1644 1027 861
+3  2235 2456 2457
+3  1645 1026 1672
+3  2322 367 1451
+3  1982 927 2000
+3  514 1763 1757
+3  1785 1042 857
+3  1786 1785 857
+3  1583 1584 2631
+3  1141 2631 1584
+3  1095 858 1096
+3  1094 1095 1096
+3  1018 2466 2467
+3  1262 2221 1669
+3  2221 1262 2222
+3  1796 7 105
+3  1924 1923 1104
+3  992 2146 2147
+3  2183 2184 2151
+3  2185 2183 2151
+3  87 2510 2511
+3  182 2718 1855
+3  283 229 282
+3  1805 1725 1718
+3  114 1989 1991
+3  1989 1981 1991
+3  1922 1931 1099
+3  1804 1803 1801
+3  2703 993 2150
+3  790 1500 2481
+3  2172 2162 2171
+3  1103 1106 288
+3  2371 1175 1174
+3  1497 2189 1501
+3  1105 1921 1923
+3  1922 1921 1105
+3  996 2158 2157
+3  1103 469 1589
+3  1100 1924 1925
+3  1918 287 1098
+3  873 1975 756
+3  1923 1932 1105
+3  1421 1420 1934
+3  997 2164 467
+3  887 2642 2643
+3  2201 2200 2202
+3  666 1660 672
+3  630 666 672
+3  1501 1505 1506
+3  2491 1602 3
+3  2413 832 1109
+3  903 904 1114
+3  903 1114 405
+3  944 405 1114
+3  801 818 171
+3  818 2428 171
+3  2433 2432 774
+3  2433 1114 2432
+3  836 837 2434
+3  2307 2308 2312
+3  1214 179 1213
+3  179 1215 1213
+3  1057 632 713
+3  632 1120 713
+3  2277 2276 682
+3  713 1119 1118
+3  1119 713 1120
+3  358 380 1152
+3  1870 388 1869
+3  425 144 426
+3  2273 2275 705
+3  2275 2273 1008
+3  1118 650 1008
+3  1118 1119 650
+3  2659 453 189
+3  1123 2 351
+3  529 2668 2669
+3  478 479 1438
+3  478 386 479
+3  2405 912 2403
+3  2104 1859 352
+3  1416 1674 431
+3  1904 1903 12
+3  1386 2470 2469
+3  352 487 1429
+3  343 1732 1733
+3  326 1730 337
+3  1817 1818 1816
+3  1825 1822 1662
+3  1232 2767 1829
+3  1374 1113 335
+3  150 363 934
+3  363 1250 934
+3  1770 518 1746
+3  2639 2634 2638
+3  360 2639 2638
+3  889 2638 298
+3  93 796 795
+3  1179 109 1129
+3  1493 1193 1494
+3  706 2278 1596
+3  575 2426 2423
+3  575 2425 2426
+3  109 301 1129
+3  300 301 2635
+3  109 1179 896
+3  1474 2768 139
+3  2099 354 2100
+3  301 303 1130
+3  149 303 301
+3  403 801 802
+3  2428 2430 2427
+3  904 2428 2427
+3  585 1136 1135
+3  799 403 2200
+3  587 2412 1062
+3  2411 1090 2412
+3  1534 2190 1497
+3  800 403 799
+3  2233 2228 2230
+3  877 876 756
+3  1136 585 2769
+3  585 802 2769
+3  2108 382 381
+3  1182 603 1183
+3  2667 2666 254
+3  55 2225 1681
+3  465 2145 1588
+3  224 2075 2074
+3  1584 1585 1586
+3  464 1585 1584
+3  311 310 230
+3  310 309 230
+3  1290 1289 228
+3  273 1290 1277
+3  1273 2368 2370
+3  47 369 310
+3  465 1586 1585
+3  2342 2343 158
+3  2346 2342 158
+3  318 2034 2032
+3  1148 2034 318
+3  2220 466 2219
+3  1582 1141 1142
+3  1579 1578 1142
+3  1071 1670 1065
+3  2222 1262 1268
+3  1587 222 1142
+3  1290 228 1277
+3  1286 1279 2075
+3  1260 1269 1270
+3  1269 1274 1270
+3  1667 1668 1261
+3  1361 652 634
+3  1261 1258 1630
+3  1272 1260 1270
+3  225 1269 1268
+3  225 2770 1269
+3  1938 1258 1256
+3  2249 2386 2385
+3  2386 2249 2244
+3  1981 2321 1991
+3  363 150 309
+3  2327 363 309
+3  367 2323 2324
+3  466 2220 1106
+3  1344 1345 766
+3  1151 1152 357
+3  358 1152 1151
+3  274 5 276
+3  274 1153 5
+3  265 96 271
+3  1434 1436 2686
+3  1436 1434 1433
+3  994 2186 2155
+3  1887 2217 2216
+3  2131 1034 2708
+3  648 1156 1608
+3  2418 2417 1078
+3  2418 1079 2417
+3  1499 1570 1508
+3  420 2437 1167
+3  610 420 1167
+3  1181 2311 1184
+3  1166 0 611
+3  1165 612 1160
+3  732 189 451
+3  1161 765 1162
+3  764 906 1166
+3  1612 1383 1611
+3  612 1165 1166
+3  456 189 732
+3  637 1879 636
+3  1879 637 1880
+3  762 1346 879
+3  620 1366 1878
+3  1894 620 1878
+3  1416 2537 1674
+3  2535 1413 1412
+3  703 1080 1063
+3  1161 1160 765
+3  2436 612 1167
+3  2437 2436 1167
+3  770 2771 2439
+3  2771 770 401
+3  269 2354 281
+3  418 1172 646
+3  37 2052 1313
+3  487 486 1426
+3  1177 645 1176
+3  790 2481 1498
+3  803 1170 418
+3  1639 1641 852
+3  1174 645 1173
+3  2372 1174 2374
+3  875 2378 639
+3  638 1881 2352
+3  646 1172 1177
+3  2381 881 2371
+3  645 1174 1175
+3  267 2357 2356
+3  1178 897 891
+3  268 2353 96
+3  2372 2371 1174
+3  2381 2371 2372
+3  645 1175 1176
+3  302 897 1178
+3  897 302 745
+3  645 1177 1172
+3  1904 106 1907
+3  1177 653 646
+3  600 653 1177
+3  653 600 2742
+3  2399 1015 2401
+3  1546 2532 1563
+3  2532 1539 1563
+3  638 1882 1881
+3  1129 1130 302
+3  1178 1129 302
+3  1179 1129 1178
+3  1181 391 1182
+3  906 0 1166
+3  2620 480 2520
+3  1191 1190 549
+3  522 1191 549
+3  2173 347 2175
+3  2173 1929 347
+3  570 2423 2424
+3  569 570 2424
+3  796 394 795
+3  1180 391 1181
+3  2438 2437 420
+3  904 616 2429
+3  2581 570 93
+3  1180 1181 606
+3  1181 1184 606
+3  609 2305 2304
+3  609 1137 2305
+3  521 2683 1487
+3  1209 2313 1195
+3  2201 402 2203
+3  82 1683 1282
+3  1280 82 1282
+3  1699 1698 572
+3  1621 572 1979
+3  1492 550 1485
+3  1207 93 569
+3  606 1184 1187
+3  167 1688 260
+3  1334 521 1487
+3  2710 2711 815
+3  2181 2158 2699
+3  2158 2181 2180
+3  563 1202 1204
+3  1202 563 513
+3  2682 1192 522
+3  1204 1666 1205
+3  1204 517 1666
+3  204 901 151
+3  1197 1198 567
+3  2313 1209 2312
+3  606 2304 2305
+3  396 1699 576
+3  494 1760 1759
+3  1939 1941 1940
+3  393 568 1117
+3  568 1210 1117
+3  2307 2312 1209
+3  1615 1698 575
+3  513 1203 1202
+3  490 511 1764
+3  239 1202 1203
+3  1193 2682 2683
+3  2682 522 2683
+3  274 2685 1153
+3  512 1757 1758
+3  1744 515 1743
+3  1203 816 239
+3  816 1203 493
+3  239 1201 1202
+3  773 1306 1309
+3  55 1760 1761
+3  2576 2578 2573
+3  2576 2579 2578
+3  230 309 973
+3  1758 1757 1763
+3  1765 1756 1766
+3  563 1205 1748
+3  1208 609 566
+3  609 1190 566
+3  1184 2311 2309
+3  1199 1208 566
+3  1208 1199 1206
+3  1196 1199 566
+3  1137 1180 2305
+3  2408 1658 1653
+3  1668 1669 372
+3  1204 1205 563
+3  2313 2314 1195
+3  1485 550 1488
+3  1221 716 1222
+3  940 937 1211
+3  180 937 940
+3  35 1217 987
+3  865 1025 1031
+3  1110 693 1911
+3  351 2 1859
+3  1214 1213 1212
+3  914 1017 1016
+3  2140 1032 1647
+3  1217 35 941
+3  916 1965 215
+3  1788 940 1215
+3  936 181 938
+3  2012 2016 2017
+3  2016 2012 1994
+3  2677 530 235
+3  1807 2448 2449
+3  1221 1009 726
+3  17 2587 2589
+3  1815 1810 1814
+3  1810 1815 1811
+3  1220 716 1221
+3  1228 1817 730
+3  1345 1344 879
+3  1817 1235 1818
+3  1011 1813 1012
+3  452 1809 1227
+3  1813 1227 1812
+3  730 451 1228
+3  728 716 727
+3  2338 2341 2340
+3  2337 2338 2340
+3  1816 190 1811
+3  181 528 938
+3  938 528 48
+3  528 1856 48
+3  1222 1224 1009
+3  1810 1809 1808
+3  1228 1235 1817
+3  189 450 451
+3  452 1227 1813
+3  108 2033 2034
+3  1592 1593 127
+3  68 1592 127
+3  2718 89 332
+3  1813 1812 1012
+3  2567 1394 2566
+3  900 2018 2019
+3  2018 900 124
+3  450 1237 451
+3  2497 59 2496
+3  1236 1237 450
+3  450 453 1234
+3  1234 433 1232
+3  1806 2448 1224
+3  2448 1806 2449
+3  1228 451 1237
+3  1472 1470 333
+3  2373 2374 754
+3  1877 1876 886
+3  1473 184 350
+3  2519 2189 2190
+3  1833 1834 1247
+3  1833 1248 1834
+3  1237 1820 1229
+3  2265 2264 424
+3  1664 1665 1198
+3  1665 1200 1198
+3  1664 1663 1495
+3  444 1011 1241
+3  1877 1163 1876
+3  1246 1824 1245
+3  442 183 441
+3  1238 1239 1012
+3  183 443 441
+3  1012 1241 1011
+3  1012 1239 1241
+3  1241 1240 1242
+3  1464 2338 1266
+3  2338 1464 366
+3  1236 450 1831
+3  1830 2772 1831
+3  1346 759 1345
+3  1875 759 1346
+3  2707 1434 2686
+3  1435 2707 2686
+3  24 2020 336
+3  1246 1245 1835
+3  1495 1663 241
+3  1663 1197 241
+3  1433 383 1436
+3  1278 2073 2074
+3  137 73 899
+3  73 137 2022
+3  335 24 336
+3  1672 1026 1576
+3  1457 1455 366
+3  55 1761 2225
+3  427 1873 425
+3  348 1465 33
+3  1255 1773 1774
+3  2334 227 2333
+3  227 2332 2333
+3  1263 2349 52
+3  368 2627 2628
+3  1276 2070 2071
+3  1632 2770 225
+3  1294 97 282
+3  372 1263 1258
+3  1261 372 1258
+3  463 2773 130
+3  1007 1006 2735
+3  2348 1006 1007
+3  2676 2675 2677
+3  544 2677 235
+3  548 530 2670
+3  2070 1276 1275
+3  1144 2070 1275
+3  2774 1272 1271
+3  1272 1270 1271
+3  1256 1263 52
+3  1252 1256 52
+3  1273 2774 2368
+3  309 150 308
+3  2088 16 1687
+3  2356 268 293
+3  1269 1260 1268
+3  2085 2088 2087
+3  372 1261 1668
+3  1630 2734 1261
+3  1630 2733 2734
+3  370 1937 1938
+3  418 1170 1172
+3  707 1599 633
+3  1118 707 633
+3  1339 769 1338
+3  366 2339 2338
+3  1996 34 983
+3  367 1280 1298
+3  155 2065 2066
+3  224 2081 1286
+3  51 157 1454
+3  1285 51 1454
+3  1253 2333 364
+3  2333 1253 2334
+3  2673 2678 526
+3  1254 2331 1775
+3  1139 1287 1289
+3  1287 228 1289
+3  312 311 230
+3  228 1287 1275
+3  2074 2077 224
+3  1774 1775 1255
+3  2505 163 2494
+3  1140 1271 1270
+3  973 309 308
+3  1682 1681 2739
+3  1140 1274 1275
+3  286 1315 2084
+3  2339 2341 2338
+3  1315 294 2084
+3  1280 226 1279
+3  82 1280 1279
+3  47 1277 1293
+3  2230 1304 2233
+3  1304 2230 2231
+3  1398 1399 1378
+3  1101 1102 288
+3  1579 2258 1578
+3  1581 1288 1139
+3  1001 2173 2172
+3  368 2339 2346
+3  1103 1736 469
+3  1286 2075 224
+3  469 1736 1735
+3  159 1452 1451
+3  2629 961 1787
+3  2077 2074 1281
+3  1295 1294 273
+3  1295 1277 272
+3  272 1277 47
+3  1295 273 1277
+3  2044 90 2045
+3  90 2044 2041
+3  2181 2703 2150
+3  1298 1282 1459
+3  366 1455 2342
+3  2339 366 2342
+3  157 51 156
+3  463 744 306
+3  1957 1959 319
+3  536 548 29
+3  548 2670 29
+3  186 1849 2391
+3  2351 1176 1175
+3  2069 2070 1144
+3  2070 2069 1278
+3  1271 2365 2368
+3  2757 1774 1773
+3  1140 1275 1288
+3  1288 1275 1287
+3  1288 1287 1139
+3  46 277 280
+3  369 47 1293
+3  227 2325 1359
+3  1586 1141 1584
+3  1291 1289 1290
+3  232 311 312
+3  280 270 232
+3  2073 1281 2074
+3  97 1294 1295
+3  1291 273 1292
+3  273 1291 1290
+3  1292 1294 282
+3  1294 1292 273
+3  2078 2086 1283
+3  2086 1735 1283
+3  999 2170 2168
+3  2365 2370 2368
+3  2367 2366 1140
+3  957 1142 1578
+3  1578 1143 1577
+3  2258 1273 2370
+3  2699 2158 2700
+3  638 2352 2351
+3  1336 768 1335
+3  604 902 905
+3  580 1626 1627
+3  796 568 393
+3  2430 774 2427
+3  943 2431 806
+3  773 772 400
+3  2220 1002 1106
+3  1339 616 769
+3  1103 466 1106
+3  1301 1300 602
+3  1030 1116 2450
+3  1030 2712 1116
+3  2327 2334 363
+3  2334 1253 363
+3  2052 910 210
+3  1980 2617 1697
+3  1302 1301 602
+3  773 2256 2257
+3  2232 1670 581
+3  1303 580 2228
+3  2355 281 2354
+3  2357 281 2355
+3  2042 966 2041
+3  966 2042 200
+3  2228 2227 1303
+3  399 2372 2373
+3  2372 2374 2373
+3  2508 2507 833
+3  2351 2352 1176
+3  1790 1793 679
+3  752 1309 1307
+3  1773 2223 1267
+3  2328 1451 1452
+3  909 1310 910
+3  1704 1702 1703
+3  2177 1918 1098
+3  76 1113 1374
+3  1729 1732 328
+3  1732 1126 328
+3  321 320 1716
+3  293 1315 267
+3  294 1315 293
+3  1026 862 1576
+3  719 1317 1316
+3  1319 1317 719
+3  1317 1319 1318
+3  1318 557 1317
+3  1318 562 557
+3  562 1318 558
+3  445 1318 1319
+3  1318 445 558
+3  2130 2131 2708
+3  1328 1327 718
+3  444 1325 717
+3  946 948 406
+3  501 2225 2226
+3  447 1325 444
+3  1320 1326 718
+3  443 183 704
+3  1324 443 704
+3  1326 742 718
+3  445 1321 1322
+3  1328 718 474
+3  1322 704 559
+3  1323 704 1322
+3  704 1323 1324
+3  737 738 458
+3  457 2604 739
+3  2660 1408 461
+3  1326 719 742
+3  1319 719 1326
+3  1319 1326 1320
+3  383 1324 447
+3  1324 1323 1325
+3  717 1327 1328
+3  717 1328 444
+3  732 458 456
+3  732 735 458
+3  444 1328 1011
+3  446 1332 1333
+3  1332 520 1333
+3  406 2741 775
+3  562 1331 557
+3  995 2163 2160
+3  490 1764 841
+3  878 879 398
+3  2720 1492 608
+3  515 1744 519
+3  562 1332 1331
+3  457 738 556
+3  767 401 770
+3  2397 1399 1398
+3  1358 876 877
+3  1341 1342 758
+3  1373 1878 635
+3  1878 1366 635
+3  1340 757 1341
+3  1552 1962 509
+3  583 1302 2229
+3  1336 1337 758
+3  1337 1340 758
+3  1385 1396 1379
+3  902 604 769
+3  1895 1890 749
+3  2288 1384 1381
+3  758 1340 1341
+3  879 1344 1343
+3  883 1342 1341
+3  1342 883 398
+3  1343 1342 398
+3  879 1343 398
+3  1162 763 1161
+3  757 883 1341
+3  765 1348 1349
+3  1162 765 1349
+3  1349 1347 759
+3  1163 1349 759
+3  1163 1162 1349
+3  1345 1347 766
+3  763 1162 1163
+3  2136 870 2135
+3  2136 2137 870
+3  2240 1146 2241
+3  1348 766 1347
+3  1349 1348 1347
+3  772 1338 400
+3  700 699 2574
+3  1626 580 1303
+3  1705 320 321
+3  1342 1343 1351
+3  2256 1301 2257
+3  1352 1342 1351
+3  1350 767 1351
+3  767 1352 1351
+3  1339 2429 616
+3  761 593 1355
+3  1353 761 1355
+3  1357 755 1358
+3  755 876 1358
+3  593 1356 1355
+3  1355 1356 757
+3  1356 1357 757
+3  687 1171 686
+3  547 214 915
+3  1357 1358 883
+3  757 1357 883
+3  1961 877 878
+3  1360 619 684
+3  2746 1360 684
+3  1360 2746 652
+3  1882 621 1883
+3  621 2775 1883
+3  1909 290 291
+3  1361 619 1360
+3  652 1361 1360
+3  2460 2461 1029
+3  2461 2464 1029
+3  969 216 968
+3  655 1367 1059
+3  642 1366 1369
+3  1060 1737 1102
+3  620 1369 1366
+3  620 1901 1369
+3  1838 1839 1248
+3  2472 1387 2473
+3  635 1365 1364
+3  635 1366 1365
+3  619 1364 655
+3  635 1364 1362
+3  1362 1364 1361
+3  1364 619 1361
+3  1362 1361 634
+3  2471 1388 2290
+3  642 1367 1365
+3  1366 642 1365
+3  1902 1899 1900
+3  1899 641 1900
+3  2598 2597 589
+3  2598 2600 2597
+3  1733 32 343
+3  635 1362 1363
+3  1897 1896 1893
+3  1370 1897 643
+3  591 1370 643
+3  591 1368 1370
+3  1367 1368 618
+3  1059 1367 618
+3  594 1789 1790
+3  1372 779 622
+3  1372 594 779
+3  287 1918 2255
+3  1367 655 1365
+3  1878 1373 1894
+3  1367 642 1368
+3  1385 2286 2287
+3  620 1894 1880
+3  1894 1879 1880
+3  629 656 657
+3  1368 642 1370
+3  434 1381 1384
+3  91 1595 970
+3  1371 618 1368
+3  1373 635 1363
+3  1891 753 872
+3  1642 1645 1672
+3  197 2586 196
+3  123 258 1374
+3  257 1726 1724
+3  2086 469 1735
+3  469 2086 2079
+3  1473 350 1472
+3  1884 1890 1895
+3  1408 1407 1402
+3  2104 353 1859
+3  1417 1418 1121
+3  900 136 124
+3  829 828 2776
+3  303 149 304
+3  24 991 2021
+3  186 1846 1849
+3  2285 2286 1385
+3  1412 435 2568
+3  380 381 43
+3  1028 2465 1092
+3  1593 1592 124
+3  2402 1397 2474
+3  1387 2777 1404
+3  2777 1387 2470
+3  1782 2778 1784
+3  2723 1384 1382
+3  1886 2218 1888
+3  911 419 2403
+3  1415 2288 1381
+3  9 1390 2473
+3  1390 1389 2473
+3  378 73 234
+3  1391 1390 435
+3  1394 1412 2568
+3  2644 1645 860
+3  1611 1392 1612
+3  2354 2353 268
+3  1386 2777 2470
+3  2287 1380 1396
+3  144 425 2727
+3  350 1123 349
+3  1150 358 1151
+3  234 381 380
+3  185 436 1391
+3  346 73 373
+3  744 130 743
+3  130 1851 743
+3  1400 432 1383
+3  432 1400 1401
+3  2023 991 140
+3  2021 991 2023
+3  1847 2395 436
+3  572 576 1699
+3  1612 1392 2779
+3  1413 1417 1121
+3  1380 2397 1398
+3  433 1402 1403
+3  2219 2153 1002
+3  1407 432 1402
+3  1407 1406 432
+3  2660 431 2661
+3  2657 2656 1409
+3  361 362 1851
+3  2473 2475 9
+3  2473 1387 2475
+3  1846 1848 1375
+3  829 2540 1411
+3  1847 186 2393
+3  454 1400 1397
+3  1849 1845 1377
+3  1845 1849 1846
+3  1410 2656 2657
+3  1848 437 1375
+3  1376 437 1848
+3  436 2728 1376
+3  461 1408 1402
+3  1378 1399 1843
+3  1836 1838 1248
+3  1838 1836 1837
+3  1233 1844 1843
+3  438 1437 430
+3  1402 432 1401
+3  2476 1915 1914
+3  1779 624 617
+3  2096 2095 141
+3  2359 283 2358
+3  106 1904 1905
+3  1395 1674 1673
+3  593 1889 882
+3  1673 1674 1122
+3  1393 1612 2779
+3  1384 2288 2287
+3  2565 2568 1390
+3  1881 1882 1883
+3  705 2248 2242
+3  2248 705 2275
+3  1676 2714 1675
+3  1383 1612 1393
+3  2780 1383 1393
+3  1887 1886 640
+3  2218 1886 1887
+3  2659 461 453
+3  65 348 194
+3  348 65 1465
+3  129 72 2479
+3  1414 429 185
+3  435 1413 1414
+3  2596 43 382
+3  262 263 1478
+3  263 264 1478
+3  1476 2259 2262
+3  2394 2393 1381
+3  1413 435 1412
+3  1423 484 1427
+3  185 1391 1414
+3  1121 429 1414
+3  2538 1416 2539
+3  2163 997 2160
+3  2595 2099 355
+3  2596 2595 355
+3  2444 2443 428
+3  1945 1572 1573
+3  428 1450 1446
+3  1426 486 1424
+3  2 486 487
+3  1421 484 1423
+3  485 1421 1423
+3  2540 829 2776
+3  2584 1419 2583
+3  408 484 1422
+3  408 504 484
+3  1422 827 408
+3  408 827 826
+3  409 827 1422
+3  484 1421 1422
+3  1423 1424 485
+3  1424 1425 485
+3  439 1431 1124
+3  479 386 480
+3  917 1653 1658
+3  385 1423 1427
+3  1448 2537 2538
+3  187 1448 2538
+3  486 1425 1424
+3  479 480 1870
+3  1424 385 1426
+3  1430 1431 1432
+3  1121 2444 1447
+3  1427 504 478
+3  504 1427 484
+3  504 386 478
+3  1423 385 1424
+3  476 1443 475
+3  1934 1420 1935
+3  1427 478 1428
+3  1430 1425 486
+3  385 1429 1426
+3  385 1428 1429
+3  2437 2438 771
+3  1432 384 1430
+3  1431 2 1124
+3  544 78 2677
+3  545 78 544
+3  1874 1346 762
+3  1432 1431 439
+3  1870 480 2435
+3  1434 384 1432
+3  1433 1434 1432
+3  1428 1438 477
+3  1428 478 1438
+3  1433 439 440
+3  514 1757 1745
+3  1303 2227 2229
+3  2227 583 2229
+3  1447 1446 1435
+3  1121 1447 429
+3  1446 1447 428
+3  2709 817 2751
+3  930 2689 2691
+3  1428 477 1429
+3  1438 1439 477
+3  1445 352 1444
+3  476 1445 1444
+3  612 2528 1160
+3  1159 2528 612
+3  821 820 2195
+3  1442 1443 476
+3  933 1952 2204
+3  2214 1421 485
+3  1537 2197 785
+3  1441 425 426
+3  427 425 1441
+3  1442 426 1443
+3  59 2497 1973
+3  2497 243 1973
+3  1441 1440 145
+3  1431 486 2
+3  252 1860 59
+3  410 828 829
+3  352 1429 477
+3  1444 352 477
+3  476 1444 1440
+3  2096 2105 2097
+3  470 825 807
+3  470 408 825
+3  470 503 408
+3  1265 2329 2322
+3  2708 1034 2763
+3  2104 475 2103
+3  1445 475 2104
+3  1637 880 755
+3  2633 298 2637
+3  486 1431 1430
+3  2444 2583 2443
+3  2443 2583 1419
+3  827 146 826
+3  409 828 827
+3  828 146 827
+3  1684 168 1685
+3  2353 1153 96
+3  1153 2353 269
+3  1285 1461 51
+3  2739 2224 2294
+3  1452 159 1453
+3  65 194 1657
+3  2323 1359 2325
+3  367 2322 2323
+3  1282 1460 1459
+3  1298 159 1451
+3  1451 367 1298
+3  1870 2435 388
+3  2350 1006 2348
+3  18 1866 1455
+3  2344 158 2343
+3  1454 1455 1457
+3  1454 157 1455
+3  2666 524 254
+3  2693 1769 55
+3  921 33 172
+3  33 1465 172
+3  2324 2325 369
+3  1458 1459 1460
+3  1458 159 1459
+3  1688 1686 152
+3  1285 1460 1461
+3  1460 169 1461
+3  2673 2672 525
+3  2260 1477 143
+3  2260 2259 1477
+3  1542 1520 505
+3  1477 1476 111
+3  217 2048 2056
+3  2669 2668 531
+3  1453 1456 1464
+3  169 1462 1461
+3  1462 169 1463
+3  1684 1463 169
+3  1463 1684 1685
+3  1456 1457 1464
+3  2670 531 29
+3  531 532 29
+3  2670 2669 531
+3  529 2667 2668
+3  2667 254 2668
+3  1469 1468 331
+3  1827 1231 1828
+3  1013 53 1246
+3  1469 333 1470
+3  140 1469 1470
+3  2263 2266 2262
+3  2266 1476 2262
+3  137 899 900
+3  1847 1846 186
+3  1474 1471 74
+3  231 148 843
+3  1471 1474 2024
+3  1474 138 2024
+3  2099 2093 354
+3  2265 2271 422
+3  2101 112 359
+3  762 879 878
+3  2252 2253 856
+3  2252 871 2253
+3  2279 2280 2781
+3  2630 856 1096
+3  1489 2782 552
+3  460 554 1481
+3  552 1481 1483
+3  1334 1487 176
+3  2601 2782 2602
+3  1416 2656 1410
+3  2539 1416 1410
+3  549 1486 522
+3  1485 1486 549
+3  1485 176 1486
+3  1484 1488 552
+3  1485 1488 1484
+3  2646 460 2602
+3  2646 2603 460
+3  1698 1977 572
+3  572 1977 1978
+3  2650 413 2651
+3  2648 776 2647
+3  550 1490 1488
+3  2389 2388 838
+3  1758 494 1759
+3  1486 1487 522
+3  176 1487 1486
+3  523 2664 2666
+3  2664 524 2666
+3  529 523 2667
+3  721 556 740
+3  1490 1489 1488
+3  446 555 1330
+3  1192 2684 1191
+3  2210 414 1529
+3  797 2202 798
+3  390 797 798
+3  480 386 2520
+3  386 502 2520
+3  1537 482 2197
+3  403 1132 2199
+3  403 1131 1132
+3  141 2095 2094
+3  778 1792 2597
+3  1421 2214 1420
+3  1530 1535 415
+3  783 1496 1498
+3  2651 1411 188
+3  1521 1518 54
+3  1518 1517 54
+3  1522 1517 1518
+3  1512 1517 1522
+3  2190 2189 1497
+3  2467 1019 1020
+3  2282 1603 714
+3  1504 783 1498
+3  576 1621 1620
+3  1503 1500 416
+3  300 149 301
+3  2634 2637 2638
+3  2640 298 2632
+3  1149 1108 285
+3  2621 1108 1149
+3  1508 971 1509
+3  1506 1508 1509
+3  428 2443 1450
+3  296 295 292
+3  782 1501 1507
+3  415 1527 1526
+3  2530 2529 1544
+3  2529 2530 1543
+3  780 1512 1545
+3  695 1014 1913
+3  971 972 1510
+3  2027 2026 292
+3  2032 2025 318
+3  2025 2032 292
+3  1384 2286 1382
+3  2286 2285 1382
+3  1695 212 86
+3  1522 1524 1545
+3  2614 1916 2613
+3  1538 782 1511
+3  1563 1565 1547
+3  1519 1520 506
+3  1528 1533 781
+3  571 1084 1697
+3  1523 1524 1540
+3  1524 1523 1525
+3  1598 2281 1599
+3  2281 714 1599
+3  1522 1513 1524
+3  668 1610 669
+3  506 1513 1519
+3  1910 11 290
+3  290 11 279
+3  1519 1518 1521
+3  1519 1513 1518
+3  1521 54 1510
+3  1515 1510 54
+3  789 414 788
+3  416 1520 972
+3  505 1529 414
+3  1533 1534 781
+3  1905 1906 11
+3  128 129 131
+3  1520 1519 972
+3  1042 871 2252
+3  1510 972 1521
+3  834 833 2533
+3  1541 1540 506
+3  1513 1522 1518
+3  505 414 507
+3  1543 1541 1539
+3  1541 1543 1540
+3  1567 2555 15
+3  1532 2783 1536
+3  491 1532 777
+3  1914 1916 692
+3  318 2031 107
+3  1528 415 1533
+3  1555 1556 509
+3  1545 1512 1522
+3  1147 22 77
+3  2532 2529 1539
+3  2532 1544 2529
+3  416 1529 1520
+3  1534 1497 781
+3  2213 786 2193
+3  505 1520 1529
+3  1525 780 1545
+3  2554 15 2555
+3  2239 2319 2320
+3  231 545 77
+3  1108 952 2149
+3  1530 1526 1531
+3  2412 2522 834
+3  492 2559 1554
+3  2559 492 1740
+3  487 1426 1429
+3  476 1440 1442
+3  785 2191 1534
+3  787 2211 2210
+3  784 785 1534
+3  497 2546 2547
+3  2558 497 2547
+3  482 2198 2197
+3  1873 148 425
+3  1 842 1537
+3  1537 842 482
+3  781 1497 1538
+3  499 1567 1568
+3  1526 1527 2531
+3  2531 2530 1544
+3  1546 1544 2532
+3  1526 1544 1531
+3  1531 1550 777
+3  1545 1524 1525
+3  2548 2558 2547
+3  1550 2551 777
+3  778 779 1792
+3  1530 415 1526
+3  1541 1542 500
+3  1542 1541 506
+3  1520 1542 506
+3  834 2533 1062
+3  2531 1544 1526
+3  971 1508 1570
+3  278 11 1906
+3  1543 1539 2529
+3  1546 1531 1544
+3  2443 1420 1450
+3  965 208 964
+3  2549 1551 2550
+3  500 1567 1566
+3  45 965 200
+3  509 498 508
+3  942 1217 1216
+3  1551 508 2550
+3  1762 498 1761
+3  2276 2274 682
+3  2245 2240 2241
+3  1751 2696 1772
+3  565 1751 1772
+3  2556 2546 497
+3  1559 1558 929
+3  2685 274 63
+3  1556 498 509
+3  215 1963 914
+3  2759 2296 1680
+3  2603 776 2605
+3  1744 237 519
+3  237 520 519
+3  2132 1033 2128
+3  235 548 536
+3  1215 940 1213
+3  867 2121 2120
+3  968 961 2062
+3  961 2063 2062
+3  961 967 2063
+3  1788 1711 180
+3  2245 2241 2242
+3  1038 2134 1039
+3  2126 2122 2129
+3  839 1868 1867
+3  2312 2308 607
+3  2297 2067 50
+3  2297 153 2067
+3  180 940 1788
+3  571 1916 1915
+3  1596 705 3
+3  440 2283 1433
+3  2615 2618 573
+3  1681 2690 55
+3  1108 2621 1107
+3  510 1556 1555
+3  1561 2715 2714
+3  1280 1282 1298
+3  1680 1559 565
+3  1761 498 2226
+3  1753 1559 929
+3  536 538 209
+3  1539 1541 1565
+3  500 1542 1568
+3  1566 1565 500
+3  1547 1565 1566
+3  777 1530 1531
+3  500 1568 1567
+3  1530 777 1532
+3  597 779 778
+3  1530 1532 1535
+3  1532 1536 1535
+3  1564 1547 1569
+3  1566 1569 1547
+3  1566 15 1569
+3  1503 1570 1502
+3  1570 1499 1502
+3  2355 2356 2357
+3  1119 630 2113
+3  1946 1574 931
+3  1574 1947 931
+3  1948 1947 1574
+3  917 1656 1654
+3  1809 452 1009
+3  2784 1273 2258
+3  931 1944 1943
+3  1574 1575 26
+3  860 1642 1643
+3  2542 1571 2544
+3  2588 30 1655
+3  1799 1796 1797
+3  1800 27 1801
+3  1100 1925 1926
+3  1931 1922 1105
+3  2504 41 1802
+3  1575 1574 1573
+3  100 2495 2494
+3  2223 1268 1267
+3  1773 1255 2223
+3  1260 1267 1268
+3  1587 1142 1141
+3  2156 2186 2185
+3  57 2442 202
+3  957 1581 1582
+3  957 1582 1142
+3  2054 963 959
+3  308 150 304
+3  949 465 1585
+3  229 949 1585
+3  1580 957 1577
+3  957 1578 1577
+3  2186 994 2154
+3  1139 1583 1581
+3  1582 1581 1583
+3  1585 1292 229
+3  464 1292 1585
+3  1291 1292 464
+3  1002 2220 2219
+3  1593 124 136
+3  1976 874 1975
+3  496 2555 499
+3  1590 1592 1591
+3  1736 1060 1735
+3  2717 133 376
+3  1983 2001 1999
+3  72 1593 136
+3  72 129 128
+3  127 72 128
+3  127 1593 72
+3  126 68 127
+3  70 126 127
+3  1970 245 1971
+3  2613 1916 571
+3  2612 2613 571
+3  123 1590 1591
+3  214 547 907
+3  315 318 107
+3  138 1475 2022
+3  2480 2179 2187
+3  2242 2241 3
+3  680 1079 2418
+3  2417 681 1078
+3  668 667 1609
+3  667 668 627
+3  466 1588 2219
+3  2489 2490 2491
+3  2318 2489 2491
+3  714 1604 633
+3  712 633 1605
+3  1606 383 447
+3  1158 2488 2318
+3  953 1088 1087
+3  1088 2414 1087
+3  954 67 251
+3  713 1118 633
+3  1599 714 633
+3  1437 1606 448
+3  2524 821 2195
+3  2035 296 2033
+3  2038 2036 108
+3  2036 2038 300
+3  2035 2033 108
+3  2399 44 2398
+3  149 2038 317
+3  2785 2318 1157
+3  669 1610 631
+3  2145 2219 1588
+3  2216 2218 1887
+3  2216 685 2218
+3  432 1611 1383
+3  2611 2615 691
+3  1888 2218 685
+3  1390 1391 1389
+3  1611 432 1406
+3  1411 1410 2657
+3  1303 1302 397
+3  1626 1303 397
+3  1132 390 798
+3  702 1617 701
+3  495 929 492
+3  397 1622 1624
+3  1625 397 1624
+3  2504 61 164
+3  2579 793 2571
+3  1623 1622 1302
+3  601 605 576
+3  1620 601 576
+3  1618 1619 577
+3  578 2579 2571
+3  2572 2574 2577
+3  578 2572 2577
+3  793 579 2571
+3  1623 391 605
+3  605 601 1623
+3  391 1623 602
+3  606 2303 2304
+3  602 1623 1302
+3  397 1625 1626
+3  1625 1627 1626
+3  2509 1707 1714
+3  1260 1629 1267
+3  473 119 118
+3  2066 81 155
+3  2770 1274 1269
+3  2786 1633 2733
+3  1867 489 1742
+3  1938 1937 1258
+3  1308 581 751
+3  2065 156 2066
+3  156 51 2066
+3  1462 81 2066
+3  2080 224 2077
+3  1635 1430 384
+3  1450 1634 1446
+3  2087 1284 2085
+3  1446 1635 384
+3  1357 1636 755
+3  2380 2377 750
+3  2380 748 2377
+3  1637 882 1638
+3  1637 755 1636
+3  2376 2372 399
+3  880 875 755
+3  2372 2376 2381
+3  1640 1641 1639
+3  1785 1783 854
+3  1639 852 1024
+3  2630 1096 858
+3  1017 852 1016
+3  1350 1344 766
+3  852 1641 1016
+3  1650 2137 2136
+3  2138 1650 2136
+3  1643 1642 1027
+3  1648 1031 1647
+3  949 229 2141
+3  45 543 965
+3  1648 1038 1022
+3  1816 1226 1817
+3  2124 1644 861
+3  863 1037 1044
+3  1644 1643 1027
+3  2253 1047 2254
+3  1047 2253 867
+3  1694 1693 151
+3  865 1648 1022
+3  2138 2136 1032
+3  1109 1110 1911
+3  1025 865 1020
+3  1033 1643 1644
+3  865 1031 1648
+3  1805 259 1725
+3  1643 1649 860
+3  1643 1033 1649
+3  2292 937 938
+3  937 936 938
+3  198 4 1652
+3  1646 1651 1647
+3  1986 114 1991
+3  512 1746 2645
+3  653 2742 599
+3  2608 683 2610
+3  1858 89 331
+3  384 1434 2707
+3  2588 1655 8
+3  197 196 198
+3  2298 197 198
+3  117 116 120
+3  247 932 84
+3  247 1944 932
+3  1967 1966 250
+3  2592 192 2593
+3  172 1656 917
+3  918 172 917
+3  1659 1657 1655
+3  31 2003 2002
+3  31 2004 2003
+3  1590 1671 2018
+3  1571 2543 2544
+3  1656 1659 1654
+3  1835 1245 1841
+3  1245 1827 1841
+3  1985 1991 926
+3  670 623 1661
+3  670 658 623
+3  670 1661 1660
+3  666 670 1660
+3  596 1056 1057
+3  1661 623 698
+3  1821 2787 1819
+3  1850 2396 2391
+3  1830 1829 2772
+3  1666 1495 1493
+3  1495 1193 1493
+3  517 1664 1666
+3  1744 1743 514
+3  929 1754 1753
+3  1669 1297 372
+3  2734 1667 1261
+3  1632 225 1631
+3  1669 1668 1262
+3  1047 867 2119
+3  1083 2788 1082
+3  1671 335 336
+3  1671 1374 335
+3  123 1374 1671
+3  123 1671 1590
+3  1280 367 2324
+3  1412 1673 1122
+3  1673 1412 1394
+3  1558 1675 1562
+3  1975 884 756
+3  874 884 1975
+3  1394 2568 2565
+3  943 806 1168
+3  1788 941 1711
+3  2227 2233 1304
+3  2233 2227 2228
+3  1652 199 2408
+3  1762 1760 494
+3  1559 1676 1675
+3  376 375 2717
+3  2681 241 2680
+3  1683 169 1460
+3  1282 1683 1460
+3  817 2090 2751
+3  2442 6 202
+3  2676 2677 78
+3  2426 2425 2424
+3  2423 2426 2424
+3  824 2620 2520
+3  1199 1196 1197
+3  2672 524 525
+3  2296 1560 1677
+3  1680 2296 1677
+3  2224 1560 2294
+3  2296 2295 1560
+3  2238 2384 2251
+3  153 2065 2067
+3  2065 155 2067
+3  168 82 221
+3  168 1683 82
+3  812 813 240
+3  169 1683 1684
+3  81 1462 1463
+3  1704 986 985
+3  1693 1691 151
+3  1685 168 1686
+3  1691 63 151
+3  168 1684 1683
+3  81 167 166
+3  269 97 270
+3  16 266 1687
+3  2401 1015 2043
+3  2400 2399 2401
+3  1689 260 1688
+3  260 1689 1690
+3  1691 1690 261
+3  1691 23 1690
+3  63 1691 261
+3  23 260 1690
+3  2043 6 2401
+3  2400 2401 6
+3  263 1693 1695
+3  1693 263 23
+3  263 1695 86
+3  2590 2589 8
+3  1694 211 212
+3  1693 1694 1695
+3  1730 1731 337
+3  1686 1696 1685
+3  1696 1686 1688
+3  1688 167 1696
+3  1084 689 1697
+3  2619 1613 1614
+3  1613 1615 575
+3  569 1138 1208
+3  569 2424 1138
+3  983 2004 2010
+3  1978 1615 690
+3  1978 1977 1615
+3  1615 1616 690
+3  1554 2548 496
+3  2132 1650 1033
+3  2006 1992 2005
+3  811 240 835
+3  1701 1702 36
+3  2736 1702 1701
+3  322 1312 325
+3  1312 322 1706
+3  2189 2519 1496
+3  783 2189 1496
+3  2509 2510 2518
+3  2510 2509 342
+3  322 1710 1709
+3  1715 323 337
+3  92 908 909
+3  257 338 1726
+3  1715 321 1716
+3  35 1710 1711
+3  1702 1704 36
+3  36 1704 985
+3  37 341 212
+3  322 325 1710
+3  1705 321 1707
+3  1113 334 329
+3  2518 2510 2514
+3  1029 2464 866
+3  210 1312 1313
+3  337 1717 1714
+3  342 1717 1718
+3  2689 2693 2691
+3  2693 2689 1752
+3  1058 1372 622
+3  1371 683 1789
+3  1859 2 487
+3  1857 332 1858
+3  352 1859 487
+3  35 1711 941
+3  1713 325 1311
+3  1060 223 1735
+3  1711 1710 325
+3  1712 1711 325
+3  1713 1712 325
+3  907 213 909
+3  2105 141 2595
+3  1714 1715 337
+3  323 1715 1716
+3  98 323 1716
+3  324 323 98
+3  1726 334 76
+3  1706 322 2515
+3  2514 87 2516
+3  1722 340 2789
+3  454 1397 1378
+3  1723 256 64
+3  257 1721 1720
+3  339 1720 1721
+3  1721 64 1722
+3  1722 339 1721
+3  2385 2384 2249
+3  324 98 348
+3  257 1724 1723
+3  338 1725 1726
+3  1725 1727 1726
+3  344 327 343
+3  1730 326 1729
+3  328 1128 1466
+3  1728 328 1466
+3  1128 88 1466
+3  1467 1466 88
+3  1725 259 1727
+3  2790 327 178
+3  2790 1127 327
+3  1988 1998 1997
+3  1998 1987 1997
+3  1986 1991 1985
+3  259 1731 1730
+3  32 1734 324
+3  1734 323 324
+3  1734 32 1733
+3  326 1734 1733
+3  1589 469 2079
+3  1060 1061 223
+3  466 1103 1589
+3  39 101 102
+3  1102 289 1060
+3  4 922 1738
+3  1737 1060 1736
+3  1588 466 1589
+3  4 1738 199
+3  126 69 122
+3  920 1739 919
+3  119 69 126
+3  1749 564 563
+3  510 1741 1557
+3  1562 510 1557
+3  1741 510 1555
+3  1748 1749 563
+3  2606 2730 2650
+3  239 2711 1117
+3  489 514 1742
+3  1306 583 2227
+3  1332 562 520
+3  1745 1744 514
+3  1744 1745 237
+3  160 1862 18
+3  1746 518 1747
+3  1682 2690 1681
+3  523 236 519
+3  1758 511 494
+3  1193 1495 2681
+3  564 1749 1750
+3  1759 1760 1770
+3  1772 2695 2694
+3  564 2697 2698
+3  603 391 602
+3  495 493 1203
+3  929 495 1754
+3  2601 2602 460
+3  1748 1755 1749
+3  1481 2601 460
+3  1494 1193 2683
+3  516 1771 1747
+3  379 380 358
+3  1196 2680 241
+3  1746 1747 1771
+3  1758 1759 512
+3  1761 2226 2225
+3  498 1762 491
+3  1556 2226 498
+3  494 491 1762
+3  514 489 1763
+3  1763 1764 511
+3  1763 489 1764
+3  2290 2470 2471
+3  2469 2289 1386
+3  2681 1495 241
+3  310 2326 2327
+3  2224 1681 2225
+3  2471 1387 2472
+3  1765 1766 1750
+3  1749 1765 1750
+3  1770 1746 512
+3  1759 1770 512
+3  55 1769 1760
+3  519 236 515
+3  518 1769 1768
+3  2695 2696 930
+3  1760 1769 1770
+3  518 1770 1769
+3  310 369 2326
+3  2221 2222 1255
+3  1255 1775 2221
+3  1257 1252 2757
+3  369 2325 2326
+3  2374 1173 754
+3  2556 1552 2546
+3  1776 598 624
+3  598 625 624
+3  624 1778 1777
+3  634 1778 595
+3  1363 634 595
+3  1362 634 1363
+3  599 1780 1779
+3  1112 1795 647
+3  699 2616 2575
+3  643 750 2609
+3  690 1616 2611
+3  969 1595 1781
+3  1781 1595 539
+3  102 2297 50
+3  215 914 1016
+3  1787 961 968
+3  1042 1043 1041
+3  2284 2285 1385
+3  468 2144 2152
+3  1215 941 1788
+3  2247 2766 2246
+3  1484 552 1483
+3  1070 1794 678
+3  41 7 1796
+3  93 570 569
+3  625 598 688
+3  2600 625 688
+3  625 2600 2598
+3  655 1364 1365
+3  594 1792 779
+3  1923 1920 1104
+3  1798 79 1575
+3  1947 932 1944
+3  79 56 39
+3  1798 56 79
+3  1802 1800 1803
+3  1802 61 2504
+3  1717 1805 1718
+3  1797 1798 27
+3  1798 1575 27
+3  289 1102 1925
+3  1798 1797 56
+3  1803 61 1802
+3  41 1799 1800
+3  1799 27 1800
+3  1799 1797 27
+3  1392 2566 2779
+3  2567 2566 1392
+3  1731 1717 337
+3  1717 1731 1805
+3  1856 1857 48
+3  1571 1943 1944
+3  41 1800 1802
+3  99 2506 2505
+3  7 2506 99
+3  2495 99 2505
+3  1803 1804 42
+3  77 200 1015
+3  937 2292 2291
+3  1809 1009 1224
+3  528 1855 1856
+3  1810 1811 1227
+3  452 1813 1011
+3  1815 1226 1816
+3  1815 1814 1226
+3  1214 417 974
+3  2791 1225 2445
+3  1225 1814 2445
+3  2791 2445 1808
+3  1230 1824 1243
+3  1815 1816 1811
+3  1814 1225 730
+3  1226 1814 730
+3  730 1817 1226
+3  1230 2787 1821
+3  77 545 200
+3  2102 2109 359
+3  2109 2101 359
+3  1237 1236 1820
+3  1243 1824 53
+3  1230 1822 1823
+3  1822 1230 1821
+3  1824 1230 1823
+3  330 331 1468
+3  1126 1127 1128
+3  327 1127 1126
+3  1236 1831 1244
+3  1831 2772 1244
+3  1826 1823 1822
+3  1825 1826 1822
+3  1821 1662 1822
+3  1236 1244 1825
+3  1833 1247 1832
+3  760 1877 886
+3  1827 1828 1841
+3  24 2021 2020
+3  1234 1831 450
+3  1234 1830 1831
+3  430 1437 448
+3  1249 1837 1836
+3  1833 1832 1836
+3  1249 1836 1832
+3  1835 1247 1834
+3  346 136 899
+3  1837 1233 1838
+3  1840 1375 437
+3  1840 1834 1248
+3  1840 1248 1839
+3  1839 1375 1840
+3  1247 1835 1841
+3  1841 1828 1247
+3  1843 1844 1378
+3  1233 1842 1838
+3  2394 2395 1847
+3  315 1148 318
+3  1471 2023 140
+3  1377 1842 1843
+3  1842 1233 1843
+3  1847 2393 2394
+3  1843 1399 1850
+3  1377 1843 1850
+3  1847 1376 1848
+3  1846 1847 1848
+3  436 1376 1847
+3  1846 1375 1845
+3  44 2400 1903
+3  2392 186 2391
+3  1849 1377 1850
+3  1851 362 743
+3  130 1852 1851
+3  1852 361 1851
+3  131 361 1852
+3  195 2011 2017
+3  1671 336 2018
+3  951 2621 1149
+3  2586 2585 1854
+3  1853 191 34
+3  1995 195 2016
+3  2008 1992 2007
+3  211 1694 901
+3  532 2673 526
+3  848 220 847
+3  1855 2718 332
+3  330 939 1857
+3  332 1857 1856
+3  1858 330 1857
+3  331 330 1858
+3  332 89 1858
+3  1863 1865 18
+3  100 2494 2493
+3  525 2678 2673
+3  995 2166 2165
+3  161 70 162
+3  119 70 161
+3  117 69 119
+3  119 126 70
+3  131 1852 1864
+3  2773 1852 130
+3  18 1862 1863
+3  1399 2396 1850
+3  174 524 2664
+3  524 174 560
+3  157 18 1455
+3  131 129 361
+3  1464 1457 366
+3  1742 839 1867
+3  841 488 842
+3  238 1871 839
+3  1869 839 1871
+3  1868 1869 388
+3  1870 1872 427
+3  1870 1869 1872
+3  85 1953 2204
+3  85 1955 1953
+3  490 841 842
+3  488 482 842
+3  2593 2591 2592
+3  1872 1873 427
+3  840 148 1873
+3  1124 184 439
+3  874 885 884
+3  884 885 762
+3  760 886 1354
+3  1163 759 1876
+3  1876 759 1875
+3  1234 1232 1830
+3  1464 2663 1453
+3  1897 641 1896
+3  1901 641 1897
+3  1373 1363 636
+3  2379 754 584
+3  2296 2759 2295
+3  640 621 1882
+3  621 640 1886
+3  749 1893 1896
+3  662 657 658
+3  2219 468 2153
+3  640 882 1889
+3  1889 1887 640
+3  468 2152 2153
+3  2221 1297 1669
+3  755 875 876
+3  875 2748 873
+3  876 875 873
+3  925 1986 1985
+3  1014 2522 2521
+3  1884 1885 1890
+3  1885 753 1890
+3  805 806 404
+3  1173 1172 687
+3  1891 1890 753
+3  1987 114 1986
+3  1673 2567 1395
+3  1881 1902 1880
+3  1895 1896 1898
+3  1898 1896 641
+3  1369 1901 1370
+3  1385 1379 2792
+3  1901 1897 1370
+3  588 656 1059
+3  1899 1902 1883
+3  1900 1901 620
+3  1900 620 1880
+3  1881 1880 637
+3  110 44 1903
+3  1900 641 1901
+3  1881 1883 1902
+3  1904 12 1905
+3  1905 12 1906
+3  1935 2443 1419
+3  1908 1907 106
+3  1503 972 971
+3  1912 1911 693
+3  1912 1913 1911
+3  2614 692 1916
+3  2455 2234 2236
+3  2479 136 2478
+3  1914 1915 1916
+3  2521 2522 1109
+3  1050 2463 1115
+3  1912 1914 1913
+3  1914 695 1913
+3  1157 2318 2491
+3  626 669 631
+3  692 695 1914
+3  443 2283 440
+3  1923 1921 1920
+3  1107 1917 1108
+3  1104 1920 1107
+3  1102 1101 1926
+3  289 1924 1104
+3  2159 994 2155
+3  1001 2161 1000
+3  1100 1932 1924
+3  2181 2150 2180
+3  1098 287 1922
+3  1921 1922 287
+3  2107 356 355
+3  356 2596 355
+3  1101 347 1927
+3  1098 1922 1099
+3  1930 1933 1928
+3  1933 1930 1927
+3  1924 289 1925
+3  2185 2186 2622
+3  1932 1100 1930
+3  1932 1930 1931
+3  1924 1932 1923
+3  1933 1927 347
+3  41 14 7
+3  1927 1100 1926
+3  1099 1928 1929
+3  2506 14 2503
+3  1928 347 1929
+3  163 2503 2502
+3  2506 2503 163
+3  7 14 2506
+3  1931 1105 1932
+3  385 1427 1428
+3  409 1422 1934
+3  1421 1934 1422
+3  1628 1259 1936
+3  1186 1184 2308
+3  1186 1187 1184
+3  1937 1259 1258
+3  370 1629 1937
+3  1629 370 1267
+3  1629 1936 1937
+3  1936 1259 1937
+3  2502 2499 2498
+3  1267 370 1257
+3  345 62 2501
+3  1939 345 2501
+3  30 197 2298
+3  2410 30 2298
+3  2541 1571 2542
+3  61 1803 42
+3  672 2113 630
+3  1945 931 1943
+3  1945 1942 1572
+3  1945 1943 1942
+3  1572 42 1804
+3  2585 2586 17
+3  1572 1942 1941
+3  1943 1571 1942
+3  1572 1941 42
+3  1948 932 1947
+3  1950 1949 933
+3  1949 1950 1948
+3  1946 931 1945
+3  1573 1946 1945
+3  1804 1573 1572
+3  1801 1573 1804
+3  933 2205 1950
+3  2204 2205 933
+3  2593 192 984
+3  17 1853 2585
+3  1853 17 2590
+3  166 1954 1952
+3  1951 166 1952
+3  414 2210 2211
+3  1179 1178 319
+3  1178 1957 319
+3  79 1949 26
+3  1949 1948 26
+3  1950 932 1948
+3  84 932 1950
+3  1956 64 255
+3  2640 2641 298
+3  888 2641 2640
+3  1956 255 125
+3  956 84 2206
+3  1952 933 1951
+3  2192 786 2213
+3  166 80 165
+3  1478 1955 85
+3  166 167 1954
+3  1956 125 1955
+3  194 1700 1657
+3  1722 64 1956
+3  264 1722 1956
+3  86 1722 264
+3  264 1956 1955
+3  787 2209 2208
+3  790 2208 2209
+3  1960 1358 877
+3  581 1308 2745
+3  2228 580 1305
+3  1549 2553 2554
+3  1960 883 1358
+3  2556 2560 1962
+3  878 398 1961
+3  883 1960 398
+3  1960 1961 398
+3  2255 2564 287
+3  215 1965 1963
+3  1964 1965 2743
+3  1738 1739 918
+3  2407 1200 1665
+3  2407 1201 1210
+3  1200 2407 1210
+3  517 2407 1665
+3  2407 517 1201
+3  954 248 249
+3  955 898 954
+3  345 1966 1968
+3  244 345 1969
+3  62 345 244
+3  345 1968 1969
+3  1969 1968 40
+3  1969 1971 244
+3  1595 91 540
+3  1974 510 1562
+3  1973 1972 59
+3  244 1972 1973
+3  1972 245 59
+3  120 69 117
+3  245 1972 1971
+3  510 1974 1679
+3  2715 1561 2716
+3  2001 1985 926
+3  1346 1345 879
+3  1987 1986 925
+3  240 811 812
+3  1779 1778 624
+3  2576 2573 1614
+3  1984 2001 2002
+3  914 2406 1093
+3  1028 2460 2458
+3  193 1995 2016
+3  2016 195 2017
+3  1994 113 193
+3  34 1995 1853
+3  2012 2013 1994
+3  2013 2012 1993
+3  196 2015 198
+3  1984 925 1985
+3  2002 2001 1983
+3  1069 653 599
+3  924 1987 925
+3  1989 923 928
+3  1988 114 1998
+3  114 1987 1998
+3  2009 195 1996
+3  1652 2298 198
+3  1652 2410 2298
+3  2008 2007 2009
+3  60 982 983
+3  2010 2004 31
+3  1672 2236 1051
+3  1966 1939 1940
+3  2012 2017 2011
+3  2009 2007 195
+3  1030 2756 1029
+3  2016 1994 193
+3  1997 1990 1988
+3  1990 1997 924
+3  1997 1987 924
+3  1479 1480 677
+3  2237 1479 677
+3  924 919 1990
+3  1999 2001 926
+3  2003 1984 2002
+3  2003 977 1984
+3  2465 2459 1019
+3  2123 2122 2126
+3  1981 927 1982
+3  977 981 1984
+3  2463 868 1115
+3  1097 2254 866
+3  2464 1097 866
+3  2008 2010 31
+3  1983 2000 1992
+3  2000 2005 1992
+3  2000 927 2005
+3  1914 1912 2476
+3  2404 911 2403
+3  983 982 2004
+3  1983 2008 31
+3  31 2002 1983
+3  2008 1983 1992
+3  982 60 990
+3  2005 927 1993
+3  2014 113 2013
+3  195 2007 2011
+3  859 1097 2464
+3  2010 2008 2009
+3  927 2013 1993
+3  1027 1051 1050
+3  1027 1050 861
+3  1642 1051 1027
+3  1577 1143 2364
+3  1033 1644 2128
+3  4 198 2015
+3  928 2015 2014
+3  2014 2015 113
+3  284 2701 2702
+3  196 113 2015
+3  923 2015 928
+3  4 2015 923
+3  2266 2270 1476
+3  422 2270 2266
+3  2267 1476 2270
+3  782 1507 1511
+3  2028 25 2027
+3  1908 2029 1907
+3  2029 110 1907
+3  2030 110 2029
+3  421 110 2030
+3  110 421 44
+3  111 1476 2267
+3  304 149 317
+3  2037 21 296
+3  296 292 2033
+3  2036 2037 2035
+3  2037 296 2035
+3  2036 2035 108
+3  964 90 966
+3  2053 90 964
+3  202 201 2039
+3  58 205 211
+3  205 58 2039
+3  21 1477 295
+3  1477 111 295
+3  149 300 2038
+3  2050 207 2046
+3  207 2050 2054
+3  2051 2047 910
+3  200 2042 1015
+3  201 2042 2041
+3  2043 2042 201
+3  1015 2042 2043
+3  2399 2398 1147
+3  2043 201 202
+3  6 2043 202
+3  23 1691 1693
+3  1398 1397 2402
+3  1397 1398 1378
+3  2048 2047 207
+3  211 2049 37
+3  542 208 541
+3  2049 206 37
+3  217 913 2048
+3  2046 2047 2051
+3  205 2045 2049
+3  2044 2045 205
+3  2045 2046 206
+3  200 965 966
+3  153 154 2065
+3  201 2040 2039
+3  2053 2050 90
+3  207 2054 2055
+3  2055 2048 207
+3  2762 2057 958
+3  2753 2058 960
+3  2040 205 2039
+3  2060 91 970
+3  2062 218 968
+3  958 91 2060
+3  10 57 203
+3  2060 970 218
+3  969 970 1595
+3  218 2059 2060
+3  2762 2060 2059
+3  2630 2252 856
+3  2053 2064 2050
+3  2054 2064 963
+3  2050 2064 2054
+3  2067 155 50
+3  1276 228 1275
+3  1277 2072 1293
+3  2071 226 2072
+3  1276 2071 2072
+3  2076 2075 1279
+3  2087 2088 221
+3  2073 1278 2069
+3  2071 2076 226
+3  1278 2075 2076
+3  2074 2075 1278
+3  1277 1276 2072
+3  228 1276 1277
+3  2793 1281 2073
+3  229 1292 282
+3  1278 2076 2071
+3  2087 2081 1284
+3  2081 2087 1286
+3  83 50 165
+3  224 2080 2081
+3  223 1061 2084
+3  1283 2082 2080
+3  2082 2081 2080
+3  1283 223 2082
+3  1283 1735 223
+3  2068 222 2079
+3  222 1589 2079
+3  1061 286 2084
+3  2080 2078 1283
+3  294 2083 2084
+3  294 2085 2083
+3  2089 1687 168
+3  221 1286 2087
+3  81 166 165
+3  2088 1687 2089
+3  1686 168 1687
+3  168 221 2089
+3  819 2626 817
+3  392 819 817
+3  481 836 824
+3  1443 2111 475
+3  2111 2110 475
+3  2106 2102 359
+3  778 2597 2600
+3  2454 2476 1912
+3  2454 694 2476
+3  299 356 887
+3  1859 2095 351
+3  1859 353 2095
+3  349 2097 2098
+3  351 2095 2096
+3  1484 1483 176
+3  2096 2097 349
+3  2472 1388 2471
+3  2277 682 2272
+3  354 2092 2101
+3  1475 1474 139
+3  354 2101 2100
+3  889 142 360
+3  2110 2093 2103
+3  2112 152 1687
+3  2105 2096 141
+3  382 43 381
+3  2632 895 2640
+3  406 614 946
+3  2106 360 2102
+3  360 142 2102
+3  112 792 359
+3  239 816 2711
+3  2106 2261 143
+3  617 599 1779
+3  298 2633 2632
+3  1504 1505 783
+3  2103 475 2110
+3  1119 2113 2114
+3  2113 672 2114
+3  650 1119 2114
+3  2116 2115 675
+3  682 2274 2273
+3  2115 2114 673
+3  2241 2243 3
+3  1146 2243 2241
+3  705 2242 3
+3  650 2115 2116
+3  650 2114 2115
+3  1008 707 1118
+3  974 978 2117
+3  95 2117 978
+3  179 974 975
+3  2120 2119 867
+3  1048 861 1116
+3  290 107 2031
+3  1046 861 2118
+3  284 2761 2156
+3  869 1644 2124
+3  2121 867 2760
+3  2128 1644 869
+3  863 2732 1036
+3  1097 856 2254
+3  2124 861 1046
+3  2137 2131 870
+3  2137 2132 2131
+3  2138 1649 1650
+3  2132 1034 2131
+3  1186 2300 1187
+3  1189 1188 2306
+3  1186 2299 2300
+3  1297 2221 1775
+3  1651 1649 2140
+3  1649 1651 860
+3  2141 283 950
+3  2143 949 2142
+3  2142 949 2141
+3  1866 18 1865
+3  2180 2150 2187
+3  2224 2225 501
+3  2641 889 298
+3  1396 2402 1379
+3  885 874 761
+3  2148 994 992
+3  2147 2148 992
+3  2164 998 2167
+3  957 1580 1581
+3  997 994 2159
+3  2150 993 2149
+3  2146 2152 2144
+3  702 1081 2507
+3  702 1082 1081
+3  2149 2764 285
+3  2157 2180 2188
+3  2160 997 2159
+3  2189 783 1501
+3  2362 2361 2160
+3  1099 2702 996
+3  2524 822 2623
+3  821 2524 2623
+3  2170 2169 2168
+3  2173 2175 2174
+3  2175 1101 2170
+3  2495 2505 2494
+3  2162 2174 2166
+3  2162 2173 2174
+3  2493 2494 242
+3  1296 2369 1580
+3  2367 1140 1288
+3  2494 163 242
+3  288 1106 2169
+3  2371 2382 1175
+3  288 2170 1101
+3  2170 288 2169
+3  1970 1971 1969
+3  2172 2171 2161
+3  1001 2172 2161
+3  999 2166 2174
+3  2175 999 2174
+3  2177 2178 1918
+3  834 2522 1014
+3  293 267 2356
+3  1098 2157 2176
+3  2156 2155 2186
+3  2145 468 2219
+3  2700 2701 2151
+3  1098 1099 996
+3  2157 1098 996
+3  1315 1314 267
+3  1801 1803 1800
+3  2586 1854 196
+3  2191 2190 1534
+3  96 266 268
+3  17 2586 2587
+3  1314 283 2359
+3  952 2187 2150
+3  1496 790 1498
+3  2528 1164 765
+3  2191 2192 147
+3  786 2527 2193
+3  2526 2527 786
+3  2212 2213 2193
+3  1441 426 1442
+3  1441 1442 1440
+3  2202 797 402
+3  2528 1159 1164
+3  2623 2624 821
+3  145 1870 427
+3  2196 786 2192
+3  2192 2213 147
+3  2193 2194 787
+3  2208 2193 787
+3  1160 2528 765
+3  390 1132 582
+3  2419 2421 589
+3  2524 2526 2525
+3  2524 2195 2526
+3  145 427 1441
+3  2526 786 2525
+3  2198 482 488
+3  837 481 488
+3  2200 2201 799
+3  808 799 2201
+3  2202 402 2201
+3  2199 798 2200
+3  2200 403 2199
+3  2201 2203 808
+3  798 2199 1132
+3  788 2211 2194
+3  263 86 264
+3  2204 1952 85
+3  1500 790 2209
+3  2593 984 34
+3  191 2593 34
+3  1496 2207 790
+3  116 117 19
+3  2213 2212 147
+3  147 2190 2191
+3  1792 2419 589
+3  2524 389 822
+3  2521 1911 2523
+3  2196 2192 2191
+3  788 414 2211
+3  2208 2212 2193
+3  2214 1450 1420
+3  485 1450 2214
+3  2180 2187 2215
+3  2188 2180 2215
+3  2179 2215 2187
+3  2118 2119 1046
+3  2120 1046 2119
+3  269 5 1153
+3  2357 2359 2358
+3  2359 2357 267
+3  2160 2361 2360
+3  995 2160 2360
+3  2481 1502 1498
+3  2125 1046 2120
+3  1255 2222 2223
+3  2222 1268 2223
+3  565 1559 1753
+3  1560 2224 1678
+3  2224 501 1678
+3  526 2678 528
+3  2678 1855 528
+3  1961 1960 877
+3  695 2508 833
+3  1308 1307 2745
+3  752 1307 1308
+3  1051 2234 1050
+3  2459 1028 2458
+3  2459 2465 1028
+3  2452 868 2453
+3  2235 868 2463
+3  2462 2235 2463
+3  2320 709 2244
+3  2236 1576 1052
+3  2236 1672 1576
+3  709 2317 2244
+3  2279 706 2280
+3  2242 2246 2245
+3  675 2248 2116
+3  1382 2285 2284
+3  2248 2246 2242
+3  379 358 1150
+3  710 1145 677
+3  675 2246 2248
+3  1141 1582 2631
+3  1582 1583 2631
+3  2272 1596 2277
+3  2238 1145 2250
+3  2237 1145 2238
+3  677 1145 2237
+3  1979 572 1978
+3  1980 1979 1978
+3  2615 1616 2618
+3  1697 689 1980
+3  689 1979 1980
+3  689 1084 1083
+3  493 507 816
+3  2249 2250 2239
+3  2250 1145 2239
+3  2384 2385 2251
+3  2385 1146 2251
+3  2257 1306 773
+3  2257 583 1306
+3  1271 2368 2774
+3  1579 2784 2258
+3  1497 1501 782
+3  792 2260 2261
+3  792 423 2260
+3  143 2261 2260
+3  2261 2106 359
+3  792 2261 359
+3  2388 489 2390
+3  2030 2267 421
+3  2271 2267 2270
+3  2271 421 2267
+3  1476 1477 2259
+3  421 2268 2269
+3  423 2263 2262
+3  422 2264 2265
+3  422 2271 2270
+3  2268 421 2271
+3  1472 74 1471
+3  44 421 2269
+3  2399 2400 44
+3  1015 2399 1147
+3  2402 1396 1398
+3  1396 1380 1398
+3  2475 1387 1404
+3  861 1050 1115
+3  2274 707 1008
+3  1600 707 2274
+3  2279 2781 1597
+3  2275 2116 2248
+3  2276 1600 2274
+3  1387 2471 2470
+3  1554 2558 2548
+3  506 1540 1513
+3  383 1433 2283
+3  1970 1969 40
+3  2706 1970 40
+3  2489 2487 708
+3  2388 1764 489
+3  841 1764 2388
+3  2260 423 2387
+3  617 624 2599
+3  1601 1603 2282
+3  2284 1385 2792
+3  383 443 1324
+3  383 2283 443
+3  2286 1384 2287
+3  2398 2269 1147
+3  2269 2398 44
+3  133 379 376
+3  133 378 379
+3  1807 2446 2447
+3  2448 1807 2447
+3  133 373 378
+3  565 2293 2759
+3  499 1568 507
+3  507 493 499
+3  2157 2158 2180
+3  2299 1185 2300
+3  565 1772 2293
+3  284 2156 2701
+3  524 2672 254
+3  2693 55 2690
+3  2668 2671 531
+3  441 440 184
+3  2749 853 219
+3  153 2297 103
+3  50 83 102
+3  2297 102 103
+3  2414 2415 1087
+3  1939 42 1941
+3  2586 197 2587
+3  2695 930 2692
+3  1767 2689 2688
+3  2299 2307 1209
+3  2794 2301 1185
+3  2302 1194 2306
+3  2688 2689 930
+3  1750 2688 930
+3  2490 1601 1602
+3  2307 1186 2308
+3  2311 2795 2309
+3  2309 2308 1184
+3  768 770 2440
+3  770 768 767
+3  420 610 2310
+3  2314 2313 607
+3  2312 607 2313
+3  2795 2314 607
+3  2309 2795 607
+3  709 2319 2482
+3  2483 709 2482
+3  2315 2485 2484
+3  2315 710 2485
+3  2315 2484 2483
+3  2482 2315 2483
+3  708 2490 2489
+3  710 2315 2316
+3  710 2316 1145
+3  2319 2239 1145
+3  2316 2319 1145
+3  1293 2324 369
+3  2072 226 1293
+3  226 2324 1293
+3  1280 2324 226
+3  312 313 233
+3  2325 227 2326
+3  2322 1451 2328
+3  309 310 2327
+3  1265 2322 2328
+3  312 230 313
+3  2336 2328 1452
+3  2337 1266 2338
+3  2329 2330 1359
+3  1265 2330 2329
+3  2336 2335 1265
+3  1265 2328 2336
+3  1775 2331 1297
+3  2334 2327 227
+3  2326 227 2327
+3  307 1250 2796
+3  462 2344 1866
+3  1455 2343 2342
+3  2335 2337 365
+3  2336 2337 2335
+3  1264 2340 2341
+3  159 1456 1453
+3  2343 1866 2344
+3  2336 1266 2337
+3  2337 2340 365
+3  2339 2342 2346
+3  18 157 160
+3  1454 1457 1456
+3  2347 2346 2350
+3  365 2345 1297
+3  2331 365 1297
+3  2345 365 2340
+3  2340 1264 2345
+3  1456 159 1458
+3  1454 1456 1458
+3  1459 159 1298
+3  372 2345 2349
+3  372 1297 2345
+3  368 2346 2347
+3  372 2349 1263
+3  2346 158 2350
+3  78 2679 2676
+3  1638 881 2381
+3  2354 269 2353
+3  268 2356 2355
+3  2360 2161 2171
+3  995 2360 2171
+3  2358 283 282
+3  2361 1000 2161
+3  2361 2161 2360
+3  2363 2159 2155
+3  1140 2366 1271
+3  1288 1581 2369
+3  1288 2369 2367
+3  2367 2369 1296
+3  1581 1580 2369
+3  1296 1577 2364
+3  1580 1577 1296
+3  2366 2367 1296
+3  1143 2370 2365
+3  2370 1143 2258
+3  2382 2351 1175
+3  2381 2376 1638
+3  2376 880 1638
+3  2377 399 750
+3  2379 584 1309
+3  1638 880 1637
+3  752 2375 2379
+3  2375 2373 2379
+3  752 1308 2375
+3  643 2380 750
+3  1174 1173 2374
+3  2373 2375 399
+3  751 750 2375
+3  750 399 2375
+3  751 581 1071
+3  1308 751 2375
+3  2379 1309 752
+3  1882 2383 640
+3  2378 875 880
+3  2373 754 2379
+3  881 2382 2371
+3  2382 2383 2351
+3  2382 881 2383
+3  2383 638 2351
+3  1882 638 2383
+3  1892 2378 748
+3  881 640 2383
+3  2249 2384 2250
+3  423 2262 2387
+3  2387 2259 2260
+3  2393 1415 1381
+3  2389 838 488
+3  838 837 488
+3  841 2389 488
+3  2434 2435 836
+3  480 836 2435
+3  2390 1868 838
+3  2388 2390 838
+3  1117 2710 393
+3  2710 2709 393
+3  841 2388 2389
+3  1867 1868 2390
+3  489 1867 2390
+3  388 2435 2434
+3  434 2394 1381
+3  434 2395 2394
+3  434 1391 2395
+3  186 2392 2393
+3  1391 436 2395
+3  2400 6 12
+3  1903 2400 12
+3  419 962 2403
+3  985 60 984
+3  912 2404 2403
+3  2406 914 1963
+3  2406 1963 912
+3  2405 2406 912
+3  1090 1109 2522
+3  1940 250 1966
+3  2409 2410 1652
+3  2409 30 2410
+3  2408 2409 1652
+3  247 84 249
+3  898 67 954
+3  1014 2523 1913
+3  834 1062 2412
+3  2413 1088 832
+3  1088 1111 832
+3  1087 2415 2416
+3  681 2416 2415
+3  2416 1079 1087
+3  2422 2598 589
+3  2421 2422 589
+3  617 2599 2422
+3  681 2417 2416
+3  2417 1079 2416
+3  1792 680 2419
+3  1207 568 93
+3  790 2207 2208
+3  2316 2482 2319
+3  2423 1613 575
+3  1114 904 2427
+3  943 1168 411
+3  2428 904 2429
+3  1045 2123 2125
+3  943 411 944
+3  2433 943 944
+3  513 495 1203
+3  1613 1616 1615
+3  570 1613 2423
+3  1614 1613 570
+3  1699 2425 575
+3  2429 171 2428
+3  2486 2705 1158
+3  2484 2486 1158
+3  1158 2705 2488
+3  2705 2487 2488
+3  1753 1751 565
+3  943 2433 774
+3  806 2431 404
+3  2430 2428 818
+3  2431 2430 818
+3  2430 2431 774
+3  2710 815 2709
+3  2432 1114 2427
+3  838 388 2434
+3  1868 388 838
+3  2434 837 838
+3  1299 2440 603
+3  2653 413 2652
+3  948 2653 2652
+3  2652 2655 2741
+3  603 2440 1183
+3  1023 854 2441
+3  854 1784 2441
+3  1784 854 1783
+3  2440 770 2439
+3  2554 2553 15
+3  6 2442 12
+3  12 2442 1906
+3  2445 1810 1808
+3  1942 1571 2541
+3  249 84 956
+3  2541 1941 1942
+3  2445 1814 1810
+3  1222 1806 1224
+3  2449 1223 731
+3  2591 2590 8
+3  1115 2450 1116
+3  2712 2713 1048
+3  868 2235 2453
+3  859 2464 2461
+3  2451 2756 1030
+3  1092 855 1093
+3  2459 862 1025
+3  2458 862 2459
+3  1019 2459 1025
+3  1092 2461 1028
+3  2461 1092 859
+3  2234 1051 2236
+3  2462 2234 2455
+3  2456 2235 2462
+3  866 1030 1029
+3  2234 2462 1050
+3  855 2465 2466
+3  2465 855 1092
+3  855 2466 1018
+3  2681 2682 1193
+3  1750 2696 2697
+3  2468 1861 1756
+3  1765 2468 1756
+3  1152 380 43
+3  892 299 1958
+3  357 299 892
+3  1957 1958 890
+3  1400 1383 2474
+3  2474 1383 2780
+3  1397 1400 2474
+3  1651 2644 860
+3  373 374 346
+3  374 2477 346
+3  162 128 131
+3  374 132 2477
+3  132 2478 2477
+3  2478 346 2477
+3  136 346 2478
+3  72 136 2479
+3  73 346 899
+3  267 1314 2359
+3  2497 2496 242
+3  2315 2482 2316
+3  248 251 2706
+3  832 1110 1109
+3  248 2706 40
+3  40 1967 248
+3  1607 2486 2485
+3  2486 2484 2485
+3  954 251 248
+3  2487 2489 2488
+3  2488 2489 2318
+3  2492 1603 708
+3  714 1603 1604
+3  1603 1601 2490
+3  1157 2491 3
+3  1860 100 2493
+3  342 2509 1717
+3  253 2495 100
+3  253 28 2495
+3  59 1860 2496
+3  242 2498 2497
+3  2498 243 2497
+3  242 2496 2493
+3  2496 1860 2493
+3  164 2502 2503
+3  164 2499 2502
+3  66 252 245
+3  2501 62 2500
+3  2499 243 2498
+3  164 61 2500
+3  164 2500 2499
+3  2500 62 2499
+3  252 253 100
+3  66 253 252
+3  2502 2498 242
+3  163 2502 242
+3  2501 61 42
+3  1939 2501 42
+3  1653 30 2409
+3  61 2501 2500
+3  2503 2504 164
+3  2504 2503 14
+3  14 41 2504
+3  2507 1081 703
+3  833 2507 703
+3  2519 2190 147
+3  580 1627 1134
+3  2508 695 574
+3  2510 342 1719
+3  1707 321 1714
+3  2515 2516 1706
+3  2513 2516 87
+3  2516 2513 2517
+3  1707 2509 2518
+3  338 1719 1718
+3  87 2512 2513
+3  1706 2516 2517
+3  1312 210 1311
+3  426 144 792
+3  1312 1706 2517
+3  2517 1313 1312
+3  1313 2517 341
+3  426 792 112
+3  426 112 1443
+3  2520 502 483
+3  2194 2527 2195
+3  1090 2522 2412
+3  835 240 2625
+3  2527 2194 2193
+3  2195 788 2194
+3  2196 2525 786
+3  2203 402 809
+3  1910 1905 11
+3  1525 1523 2531
+3  2531 1523 2530
+3  2530 1523 1543
+3  1523 1540 1543
+3  2531 1527 1525
+3  2533 833 703
+3  2535 1412 1122
+3  2538 2537 1416
+3  2535 1122 2536
+3  1674 2537 1122
+3  2536 1122 2537
+3  1413 2535 1417
+3  305 303 304
+3  2540 1410 1411
+3  2444 428 1447
+3  2540 2539 1410
+3  2545 248 1967
+3  2542 1941 2541
+3  2543 1571 1944
+3  248 247 249
+3  2545 247 248
+3  2543 1944 247
+3  1940 2544 250
+3  543 541 965
+3  2562 1564 1569
+3  1567 499 2555
+3  2547 2549 1549
+3  963 542 959
+3  2547 1549 2548
+3  2558 1554 2559
+3  78 545 231
+3  843 78 231
+3  2550 508 2551
+3  2554 2548 1549
+3  2554 2555 2548
+3  508 491 2551
+3  2548 2555 496
+3  2557 2556 497
+3  2557 2560 2556
+3  1920 2564 1919
+3  2552 1548 2561
+3  1552 508 1551
+3  1552 1551 2546
+3  2558 2557 497
+3  1553 1962 2560
+3  1553 1555 1962
+3  1741 1555 1553
+3  1549 2738 2563
+3  2564 1921 287
+3  2561 2562 1569
+3  1548 2562 2561
+3  9 2475 1405
+3  2475 1404 1405
+3  435 1390 2568
+3  1618 1617 2797
+3  578 2570 2572
+3  395 2579 2576
+3  2580 2576 1614
+3  2574 2572 700
+3  2575 2573 2574
+3  2572 2569 700
+3  2574 699 2575
+3  2580 1614 570
+3  395 794 793
+3  2578 2579 578
+3  2578 578 2577
+3  395 2576 2580
+3  2577 2574 2573
+3  2578 2577 2573
+3  395 793 2579
+3  1138 396 1137
+3  1180 1137 605
+3  2581 93 795
+3  836 480 2620
+3  2584 1449 1419
+3  212 1695 1694
+3  2588 2589 2587
+3  197 30 2588
+3  197 2588 2587
+3  2590 17 2589
+3  2592 2591 8
+3  1657 1700 2592
+3  1700 192 2592
+3  2590 191 1853
+3  1657 2592 8
+3  984 983 34
+3  2591 2593 191
+3  2590 2591 191
+3  1700 1701 192
+3  2099 2100 355
+3  141 2099 2595
+3  356 43 2596
+3  357 43 356
+3  597 778 626
+3  2599 2598 2422
+3  2599 625 2598
+3  2599 624 625
+3  1502 1504 1498
+3  776 2646 2647
+3  776 2603 2646
+3  552 2601 1481
+3  2782 2601 552
+3  2650 2649 413
+3  1409 461 2658
+3  460 2604 554
+3  554 2604 2729
+3  551 775 1482
+3  1806 1222 2607
+3  2607 1222 716
+3  716 1223 2607
+3  1806 1223 2449
+3  1806 2607 1223
+3  2608 678 1793
+3  678 1794 1793
+3  683 1790 1789
+3  683 1371 2610
+3  1368 591 2610
+3  2575 2616 573
+3  1176 600 1177
+3  644 2609 750
+3  505 1568 1542
+3  788 820 789
+3  1568 505 507
+3  394 794 795
+3  1980 1978 690
+3  2612 571 2617
+3  1616 2615 2611
+3  1980 690 2617
+3  571 1697 2617
+3  395 2581 795
+3  395 2580 2581
+3  1613 2619 2618
+3  2618 1616 1613
+3  1621 576 572
+3  1107 2621 1104
+3  1925 1102 1926
+3  1104 2621 951
+3  823 2624 2623
+3  2347 371 2627
+3  368 2347 2627
+3  52 2627 1252
+3  2757 1252 1251
+3  1252 371 1251
+3  371 1252 2627
+3  1042 2630 857
+3  1042 2252 2630
+3  143 2639 360
+3  2634 2636 2637
+3  887 142 2642
+3  2634 297 2636
+3  2634 21 297
+3  360 2638 889
+3  2634 2639 21
+3  2635 301 109
+3  2642 889 2641
+3  2641 888 2642
+3  1477 21 2639
+3  356 142 887
+3  2643 2642 888
+3  2645 1746 1745
+3  1757 2645 1745
+3  2645 1757 512
+3  2656 1416 431
+3  514 1743 1742
+3  520 1334 1333
+3  2658 461 2659
+3  2653 830 2654
+3  1772 2694 2293
+3  948 2741 406
+3  2650 2731 2649
+3  412 948 946
+3  2651 410 1411
+3  410 829 1411
+3  2654 2651 413
+3  2653 2654 413
+3  485 1425 1634
+3  830 410 2654
+3  188 1411 2658
+3  2658 1411 2657
+3  1406 2662 1395
+3  1674 1395 2662
+3  13 303 305
+3  1006 2350 158
+3  305 304 150
+3  532 2672 2673
+3  2675 2674 530
+3  236 2674 2675
+3  2678 182 1855
+3  525 182 2678
+3  2676 515 236
+3  2676 236 2675
+3  96 2685 271
+3  2685 96 1153
+3  2684 566 1191
+3  1192 2680 2684
+3  930 2691 2692
+3  2696 1750 930
+3  1751 2698 2697
+3  2487 2705 2486
+3  2158 2701 2700
+3  996 2701 2158
+3  2701 996 2702
+3  2702 1099 1929
+3  2699 2700 2704
+3  384 2707 1446
+3  2709 815 817
+3  816 815 2711
+3  1676 1561 2714
+3  1030 866 2712
+
diff -pruN 5.0.3-1/examples/Surface_mesh_topology/data/torus_quad.off 5.1-1/examples/Surface_mesh_topology/data/torus_quad.off
--- 5.0.3-1/examples/Surface_mesh_topology/data/torus_quad.off	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_topology/data/torus_quad.off	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,52 @@
+OFF
+25 25 50
+0.107677 -1.54743e-018 -0.331395
+-0.281902 -1.54743e-018 -0.204814
+-0.281902 -1.54743e-018 0.204814
+0.107676 -1.54743e-018 0.331395
+0.34845 -1.54743e-018 8.92032e-008
+0.18208 0.331395 -0.560383
+-0.476691 0.331395 -0.346336
+-0.476691 0.331395 0.346336
+0.18208 0.331395 0.560384
+0.589223 0.331395 8.92032e-008
+0.302466 0.204814 -0.930895
+-0.791866 0.204814 -0.575325
+-0.791866 0.204814 0.575324
+0.302466 0.204814 0.930895
+0.978801 0.204814 8.92032e-008
+0.302466 -0.204814 -0.930895
+-0.791866 -0.204814 -0.575325
+-0.791866 -0.204814 0.575324
+0.302466 -0.204814 0.930895
+0.978801 -0.204814 8.92032e-008
+0.18208 -0.331395 -0.560383
+-0.476691 -0.331395 -0.346336
+-0.476691 -0.331395 0.346336
+0.18208 -0.331395 0.560384
+0.589222 -0.331395 8.92032e-008
+4   5 6 1 0 
+4   6 7 2 1 
+4   7 8 3 2 
+4   8 9 4 3 
+4   9 5 0 4 
+4   10 11 6 5 
+4   11 12 7 6 
+4   12 13 8 7 
+4   13 14 9 8 
+4   14 10 5 9 
+4   15 16 11 10 
+4   16 17 12 11 
+4   17 18 13 12 
+4   18 19 14 13 
+4   19 15 10 14 
+4   20 21 16 15 
+4   21 22 17 16 
+4   22 23 18 17 
+4   23 24 19 18 
+4   24 20 15 19 
+4   0 1 21 20 
+4   1 2 22 21 
+4   2 3 23 22 
+4   3 4 24 23 
+4   4 0 20 24 
diff -pruN 5.0.3-1/examples/Surface_mesh_topology/draw_facewidth.h 5.1-1/examples/Surface_mesh_topology/draw_facewidth.h
--- 5.0.3-1/examples/Surface_mesh_topology/draw_facewidth.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_topology/draw_facewidth.h	2020-09-08 12:15:45.000000000 +0000
@@ -0,0 +1,88 @@
+#ifndef DRAW_FACEWIDTH_H
+#define DRAW_FACEWIDTH_H
+
+#include <iostream>
+
+#ifdef CGAL_USE_BASIC_VIEWER
+
+#include <CGAL/draw_linear_cell_complex.h>
+
+template<typename ALCC>
+struct Facewidth_draw_functor : public CGAL::DefaultDrawingFunctorLCC
+{
+  Facewidth_draw_functor(typename ALCC::size_type amark1, typename ALCC::size_type amark2) :
+    m_vertex_mark(amark1), m_face_mark(amark2)
+  {}
+
+  template<typename LCC>
+  bool colored_vertex(const LCC& alcc, typename LCC::Dart_const_handle dh) const
+  { return alcc.is_marked(dh, m_vertex_mark); }
+
+  template<typename LCC>
+  CGAL::Color vertex_color(const LCC& /* alcc */,
+                           typename LCC::Dart_const_handle /* dh */) const
+  { return CGAL::Color(0, 255, 0); }
+
+  template<typename LCC>
+  bool colored_edge(const LCC& /*alcc*/, typename LCC::Dart_const_handle /*dh*/) const
+  { return false; }
+
+  template<typename LCC>
+  CGAL::Color edge_color(const LCC& /* alcc*/,
+                         typename LCC::Dart_const_handle /* dh */) const
+  { return CGAL::Color(0, 0, 255); }
+
+  template<typename LCC>
+  bool colored_face(const LCC& /* alcc */,
+                    typename LCC::Dart_const_handle /* dh */) const
+  {return true;}
+
+  template<typename LCC>
+  CGAL::Color face_color(const LCC& alcc, typename LCC::Dart_const_handle dh) const
+  { return alcc.is_marked(dh, m_face_mark)?CGAL::Color(255, 0, 0)
+                                          :CGAL::Color(211, 211, 211); }
+
+  template<typename LCC>
+  bool colored_volume(const LCC& /* alcc */,
+                      typename LCC::Dart_const_handle /* dh */) const
+  { return false; }
+
+  typename ALCC::size_type m_vertex_mark, m_face_mark;
+};
+
+template<typename LCC>
+void draw_facewidth(const LCC& lcc,
+                    const std::vector<typename LCC::Dart_const_handle>& cycle)
+{
+  int nbv=0, nbf=0;
+  typename LCC::size_type vertex_mark = lcc.get_new_mark();
+  typename LCC::size_type face_mark = lcc.get_new_mark();
+  for (std::size_t i=0; i<cycle.size(); ++i)
+  {
+    // Color the vertex
+    if (!lcc.is_marked(cycle[i], vertex_mark))
+    { lcc.template mark_cell<0>(cycle[i], vertex_mark); ++nbv; }
+    // Color the face
+    if (!lcc.is_marked(cycle[i], face_mark))
+    { lcc.template mark_cell<2>(cycle[i], face_mark); ++nbf; }
+  }
+
+  Facewidth_draw_functor<LCC> df(vertex_mark, face_mark);
+  CGAL::draw(lcc, "Face width", false, df);
+
+  lcc.free_mark(vertex_mark);
+  lcc.free_mark(face_mark);
+}
+
+#else  // CGAL_USE_BASIC_VIEWER
+
+template<typename LCC>
+void draw_facewidth(const LCC&,
+                    const std::vector<typename LCC::Dart_const_handle>&)
+{
+  std::cerr<<"Impossible to draw, CGAL_USE_BASIC_VIEWER is not defined."<<std::endl;
+}
+
+#endif // CGAL_USE_BASIC_VIEWER
+
+#endif // DRAW_FACEWIDTH_H
diff -pruN 5.0.3-1/examples/Surface_mesh_topology/edgewidth_lcc.cpp 5.1-1/examples/Surface_mesh_topology/edgewidth_lcc.cpp
--- 5.0.3-1/examples/Surface_mesh_topology/edgewidth_lcc.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_topology/edgewidth_lcc.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,54 @@
+#include <CGAL/Linear_cell_complex_for_combinatorial_map.h>
+#include <CGAL/Linear_cell_complex_constructors.h>
+#include <fstream>
+#include <CGAL/Curves_on_surface_topology.h>
+#include <CGAL/Path_on_surface.h>
+#include <CGAL/squared_distance_3.h>
+#include <CGAL/draw_face_graph_with_paths.h>
+
+using LCC_3          =CGAL::Linear_cell_complex_for_combinatorial_map<2, 3>;
+using Path_on_surface=CGAL::Surface_mesh_topology::Path_on_surface<LCC_3>;
+
+double cycle_length(const LCC_3& lcc, const Path_on_surface& cycle)
+{ // Compute the length of the given cycle.
+  double res=0;
+  for (std::size_t i=0; i<cycle.length(); ++i)
+  { res+=std::sqrt
+      (CGAL::squared_distance(lcc.point(cycle[i]),
+                              lcc.point(lcc.other_extremity(cycle[i])))); }
+  return res;
+}
+
+void display_cycle_info(const LCC_3& lcc, const Path_on_surface& cycle)
+{ // Display information about the given cycle.
+  if (cycle.is_empty()) { std::cout<<"Empty."<<std::endl; return; }
+  std::cout<<"Root: "<<lcc.point(cycle[0])<<"; "
+           <<"Number of edges: "<<cycle.length()<<"; "
+           <<"Length: "<<cycle_length(lcc, cycle)<<std::endl;
+}
+
+int main(int argc, char* argv[])
+{
+  std::string filename(argc==1?"data/3torus.off":argv[1]);
+  bool draw=(argc<3?false:(std::string(argv[2])=="-draw"));
+  LCC_3 lcc;
+  if (!CGAL::load_off(lcc, filename.c_str())) // Load the off file.
+  {
+    std::cout<<"Cannot read file '"<<filename<<"'. Exiting program"<<std::endl;
+    return EXIT_FAILURE;
+  }
+ std::cout<<"File '"<<filename<<"' loaded. Finding edge-width of the mesh..."<<std::endl;
+
+  CGAL::Surface_mesh_topology::Curves_on_surface_topology<LCC_3> cst(lcc, true);
+
+  Path_on_surface cycle1=cst.compute_edge_width(true);
+
+  CGAL::Surface_mesh_topology::Euclidean_length_weight_functor<LCC_3> wf(lcc);
+  Path_on_surface cycle2=cst.compute_shortest_non_contractible_cycle(wf, true);
+
+  std::cout<<"Cycle 1 (pink): "; display_cycle_info(lcc, cycle1);
+  std::cout<<"Cycle 2 (green): "; display_cycle_info(lcc, cycle2);
+  if (draw) { CGAL::draw(lcc, {cycle1, cycle2}); }
+
+  return EXIT_SUCCESS;
+}
diff -pruN 5.0.3-1/examples/Surface_mesh_topology/edgewidth_surface_mesh.cpp 5.1-1/examples/Surface_mesh_topology/edgewidth_surface_mesh.cpp
--- 5.0.3-1/examples/Surface_mesh_topology/edgewidth_surface_mesh.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_topology/edgewidth_surface_mesh.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,56 @@
+#include <CGAL/Simple_cartesian.h>
+#include <CGAL/Surface_mesh.h>
+#include <fstream>
+#include <CGAL/Curves_on_surface_topology.h>
+#include <CGAL/Path_on_surface.h>
+#include <CGAL/squared_distance_3.h>
+#include <CGAL/draw_face_graph_with_paths.h>
+
+using Mesh           =CGAL::Surface_mesh<CGAL::Simple_cartesian<double>::Point_3>;
+using Path_on_surface=CGAL::Surface_mesh_topology::Path_on_surface<Mesh>;
+
+double cycle_length(const Mesh& mesh, const Path_on_surface& cycle)
+{ // Compute the length of the given cycle.
+  double res=0;
+  for (std::size_t i=0; i<cycle.length(); ++i)
+  { res+=std::sqrt
+        (CGAL::squared_distance(mesh.point(mesh.vertex(mesh.edge(cycle[i]), 0)),
+                                mesh.point(mesh.vertex(mesh.edge(cycle[i]), 1)))); }
+  return res;
+}
+
+void display_cycle_info(const Mesh& mesh, const Path_on_surface& cycle)
+{ // Display information about the given cycle.
+  if (cycle.is_empty()) { std::cout<<"Empty."<<std::endl; return; }
+  std::cout<<"Root: "<<mesh.point(mesh.vertex(mesh.edge(cycle[0]), 0))<<"; "
+           <<"Number of edges: "<<cycle.length()<<"; "
+           <<"Length: "<<cycle_length(mesh, cycle)<<std::endl;
+}
+
+int main(int argc, char* argv[])
+{
+  std::string filename(argc==1?"data/3torus.off":argv[1]);
+  bool draw=(argc<3?false:(std::string(argv[2])=="-draw"));
+  std::ifstream inp(filename);
+  if (inp.fail())
+  {
+    std::cout<<"Cannot read file '"<<filename<<"'. Exiting program"<<std::endl;
+    return EXIT_FAILURE;
+  }
+  Mesh sm;
+  inp>>sm;
+  std::cout<<"File '"<<filename<<"' loaded. Finding edge-width of the mesh..."<<std::endl;
+
+  CGAL::Surface_mesh_topology::Curves_on_surface_topology<Mesh> cst(sm, true);
+
+  Path_on_surface cycle1=cst.compute_edge_width(true);
+
+  CGAL::Surface_mesh_topology::Euclidean_length_weight_functor<Mesh> wf(sm);
+  Path_on_surface cycle2=cst.compute_shortest_non_contractible_cycle(wf, true);
+
+  std::cout<<"Cycle 1 (pink): "; display_cycle_info(sm, cycle1);
+  std::cout<<"Cycle 2 (green): "; display_cycle_info(sm, cycle2);
+  if (draw) { CGAL::draw(sm, {cycle1, cycle2}); }
+
+  return EXIT_SUCCESS;
+}
diff -pruN 5.0.3-1/examples/Surface_mesh_topology/facewidth.cpp 5.1-1/examples/Surface_mesh_topology/facewidth.cpp
--- 5.0.3-1/examples/Surface_mesh_topology/facewidth.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_topology/facewidth.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,42 @@
+#include <CGAL/Linear_cell_complex_for_combinatorial_map.h>
+#include <CGAL/Linear_cell_complex_constructors.h>
+#include <iostream>
+#include <fstream>
+#include <cstdlib>
+#include <CGAL/Curves_on_surface_topology.h>
+#include "draw_facewidth.h"
+
+using LCC_3            =CGAL::Linear_cell_complex_for_combinatorial_map<2, 3>;
+using CST              =CGAL::Surface_mesh_topology::Curves_on_surface_topology<LCC_3>;
+using Dart_const_handle=LCC_3::Dart_const_handle;
+
+int main(int argc, char* argv[])
+{
+  std::cout<<"Program facewidth_on_unweighted_map started."<<std::endl;
+  std::string filename(argc==1?"data/double-torus.off":argv[1]);
+  bool draw=(argc<3?false:std::string(argv[2])=="-draw");
+
+  std::ifstream inp(filename);
+  if (inp.fail())
+  {
+    std::cout<<"Cannot read file '"<<filename<<"'. Exiting program"<<std::endl;
+    return EXIT_FAILURE;
+  }
+  LCC_3 lcc;
+  CGAL::load_off(lcc, inp);
+  std::cout<<"File '"<<filename<<"' loaded. Finding the facewidth..."<<std::endl;
+
+  CST cst(lcc, true);
+  std::vector<Dart_const_handle> cycle=cst.compute_face_width(true);
+
+  if (cycle.size()==0)
+  { std::cout<<"  Cannot find such cycle."<<std::endl; }
+  else
+  {
+    std::cout<<"  Number of faces: "<<cycle.size()<<std::endl;
+
+    if (draw) { draw_facewidth(lcc, cycle); }
+  }
+
+  return EXIT_SUCCESS;
+}
diff -pruN 5.0.3-1/examples/Surface_mesh_topology/map_2_constructor.cpp 5.1-1/examples/Surface_mesh_topology/map_2_constructor.cpp
--- 5.0.3-1/examples/Surface_mesh_topology/map_2_constructor.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_topology/map_2_constructor.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,34 @@
+#include <CGAL/Polygonal_schema.h>
+#include <CGAL/Path_on_surface.h>
+#include <iostream>
+#include <cstdlib>
+
+using namespace CGAL::Surface_mesh_topology;
+
+typedef Polygonal_schema_with_combinatorial_map<> CMap;
+typedef Polygonal_schema_with_generalized_map<>   GMap;
+
+template<typename Map>
+void construct_map_from_edges()
+{
+  Map cm;
+
+  cm.add_facet("a b c -a -b");
+  cm.add_facet("-c d e -d -e");
+
+  Path_on_surface<Map> p(cm); p.push_back_by_label("a b d e");
+
+  std::cout<<"Map valid="<<cm.is_valid()<<std::flush;
+  cm.display_characteristics(std::cout);
+  std::cout<<"; path lenght="<<p.length()
+     <<", isclosed? "<<(p.is_closed()?"true":"false")<<std::endl;
+}
+
+int main()
+{
+  construct_map_from_edges<CMap>();
+  construct_map_from_edges<GMap>();
+
+  return EXIT_SUCCESS;
+}
+
diff -pruN 5.0.3-1/examples/Surface_mesh_topology/open_path_homotopy.cpp 5.1-1/examples/Surface_mesh_topology/open_path_homotopy.cpp
--- 5.0.3-1/examples/Surface_mesh_topology/open_path_homotopy.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_topology/open_path_homotopy.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,70 @@
+#include <CGAL/Surface_mesh.h>
+#include <CGAL/Curves_on_surface_topology.h>
+#include <CGAL/Path_on_surface.h>
+#include <CGAL/draw_face_graph_with_paths.h>
+
+typedef CGAL::Exact_predicates_inexact_constructions_kernel Kernel;
+typedef Kernel::Point_3                                     Point_3;
+typedef CGAL::Surface_mesh<Point_3>                         SM;
+
+using namespace CGAL::Surface_mesh_topology;
+
+///////////////////////////////////////////////////////////////////////////////
+void create_path_1(Path_on_surface<SM>& p)
+{
+  p.push_back_by_index(88); // Its starting dart
+  for (int i=0; i<3; ++i)
+  { p.extend_positive_turn(2); } // Extend the path
+}
+
+///////////////////////////////////////////////////////////////////////////////
+void create_path_2(Path_on_surface<SM>& p)
+{
+  p.push_back_by_index(300);  // Its starting dart
+  for (int i=0; i<3; ++i)
+  { p.extend_negative_turn(2); } // Extend the path
+}
+
+///////////////////////////////////////////////////////////////////////////////
+void create_path_3(Path_on_surface<SM>& p)
+{
+  p.push_back_by_index(87); // Its starting dart
+  p.extend_positive_turn(1); // Extend the path
+  for (int i=0; i<3; ++i)
+  { p.extend_positive_turn(2); }
+  p.extend_positive_turn(1);
+}
+
+///////////////////////////////////////////////////////////////////////////////
+int main(int argc, char* argv[])
+{
+  bool draw=(argc>1?std::string(argv[1])=="-draw":false);
+  std::ifstream in("data/double-torus.off");
+  if (!in.is_open())
+  {
+    std::cout<<"ERROR reading file data/double-torus.off"<<std::endl;
+    exit(EXIT_FAILURE);
+  }
+  SM sm;
+  in>>sm;
+
+  Curves_on_surface_topology<SM> cst(sm);
+  Path_on_surface<SM> p1(sm), p2(sm), p3(sm);
+  create_path_1(p1);
+  create_path_2(p2);
+  create_path_3(p3);
+
+  bool res1=cst.are_base_point_homotopic(p1, p2);
+  std::cout<<"Path p1 (pink) "<<(res1?"IS":"IS NOT")
+           <<" base point homotopic with path p2 (green)."<<std::endl;
+
+  bool res2=cst.are_base_point_homotopic(p2, p3);
+  std::cout<<"Path p2 (green) "<<(res2?"IS":"IS NOT")
+           <<" base point homotopic with path p3 (orange)."<<std::endl;
+
+  if (draw)
+  { CGAL::draw(sm, {p1, p2, p3}); }
+
+  return EXIT_SUCCESS;
+}
+///////////////////////////////////////////////////////////////////////////////
diff -pruN 5.0.3-1/examples/Surface_mesh_topology/path_homotopy.cpp 5.1-1/examples/Surface_mesh_topology/path_homotopy.cpp
--- 5.0.3-1/examples/Surface_mesh_topology/path_homotopy.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_topology/path_homotopy.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,199 @@
+#include <CGAL/Linear_cell_complex_for_combinatorial_map.h>
+#include <CGAL/Linear_cell_complex_constructors.h>
+#include <CGAL/Curves_on_surface_topology.h>
+#include <CGAL/Path_on_surface.h>
+
+/* If you want to use a viewer, you can use qglviewer. */
+#ifdef CGAL_USE_BASIC_VIEWER
+#include <CGAL/draw_face_graph_with_paths.h>
+#endif
+
+typedef CGAL::Linear_cell_complex_for_combinatorial_map<2,3> LCC_3_cmap;
+using namespace CGAL::Surface_mesh_topology;
+
+///////////////////////////////////////////////////////////////////////////////
+[[ noreturn ]] void usage(int /*argc*/, char** argv)
+{
+  std::cout<<"usage: "<<argv[0]<<" file [-draw] [-L l1 l2] [-D d1 d2] "
+           <<" [-N n] [-seed S] [-time]"<<std::endl
+           <<"   Load the given off file, compute one random path, deform it "
+           <<"into a second path and test that the two paths are homotopic."
+           <<std::endl
+           <<"   -draw: draw mesh and the two paths." <<std::endl
+           <<"   -L l1 l2: create a path of length >= l: a random number between [l1, l2] (by default [10, 100])."<<std::endl
+           <<"   -D d1 d2: use d deformations to generate the second path: d is a random number between [d1, d2] (by default [10, 100]."<<std::endl
+           <<"   -N n: do n tests of homotopy (using 2*n random paths) (by default 1)."<<std::endl
+           <<"   -seed S: uses S as seed of random generator. Otherwise use a different seed at each run (based on time)."<<std::endl
+           <<"   -time: display computation times."<<std::endl
+           <<std::endl;
+  exit(EXIT_FAILURE);
+}
+///////////////////////////////////////////////////////////////////////////////
+[[ noreturn ]] void error_command_line(int argc, char** argv, const char* msg)
+{
+  std::cout<<"[ERROR] "<<msg<<std::endl;
+  usage(argc, argv);
+}
+///////////////////////////////////////////////////////////////////////////////
+void process_command_line(int argc, char** argv,
+                          std::string& file,
+                          bool& draw,
+                          int& l1,
+                          int& l2,
+                          int& d1,
+                          int& d2,
+                          unsigned int& N,
+                          CGAL::Random& random,
+                          bool& time)
+{
+  std::string arg;
+  bool usage_required=false;
+  for (int i=1; i<argc; ++i)
+  {
+    arg=argv[i];
+    if (arg=="-draw")
+    { draw=true; }
+    else if (arg=="-D")
+    {
+     if (i+2>=argc)
+     { error_command_line(argc, argv, "not enough number after -D option."); }
+     d1=std::stoi(std::string(argv[++i]));
+     d2=std::stoi(std::string(argv[++i]));
+    }
+    else if (arg=="-L")
+    {
+     if (i+2>=argc)
+     { error_command_line(argc, argv, "not enough number after -L option."); }
+     l1=std::stoi(std::string(argv[++i]));
+     l2=std::stoi(std::string(argv[++i]));
+    }
+    else if (arg=="-N")
+    {
+      if (i==argc-1)
+      { error_command_line(argc, argv, "no number after -nbtests option."); }
+      N=static_cast<unsigned int>(std::stoi(std::string(argv[++i])));
+    }
+    else if (arg=="-seed")
+    {
+      if (i==argc-1)
+      { error_command_line(argc, argv, "no number after -seed option."); }
+      random=CGAL::Random(static_cast<unsigned int>
+                          (std::stoi(std::string(argv[++i]))));
+      // initialize the random generator with the given seed
+    }
+    else if (arg=="-time")
+    { time=true; }
+    else if (arg=="-h" || arg=="--help" || arg=="-?")
+    { usage_required=true; }
+    else if (arg[0]=='-')
+    { std::cout<<"[WARNING] Unknown option "<<arg<<", ignored."<<std::endl; }
+    else
+    { file=arg; }
+  }
+
+  if (N==0) { N=1; }
+  if (l2<l1) l2=l1;
+  if (d2<d1) d2=d1;
+
+  if (usage_required) {usage(argc, argv); }
+}
+///////////////////////////////////////////////////////////////////////////////
+int main(int argc, char** argv)
+{
+  std::string file="data/elephant.off";
+  bool draw=false;
+  int l1=10, l2=100;
+  int d1=10, d2=100;
+  unsigned int N=1;
+  CGAL::Random random; // Used when user do not provide its own seed.
+  bool time=false;
+
+  process_command_line(argc, argv, file, draw, l1, l2, d1, d2, N, random, time);
+
+  LCC_3_cmap lcc;
+  if (!CGAL::load_off(lcc, file.c_str()))
+  {
+    std::cout<<"PROBLEM reading file "<<file<<std::endl;
+    exit(EXIT_FAILURE);
+  }
+
+  std::cout<<"Initial map: ";
+  lcc.display_characteristics(std::cout) << ", valid="
+                                         << lcc.is_valid() << std::endl;
+
+  Curves_on_surface_topology<LCC_3_cmap> cst(lcc, time);
+
+  cst.compute_minimal_quadrangulation(time);
+
+  std::cout<<"Reduced map: ";
+  cst.get_minimal_quadrangulation().display_characteristics(std::cout)
+      <<", valid="<< cst.get_minimal_quadrangulation().is_valid()<<std::endl;
+
+  unsigned int nbcontractible=0;
+  std::vector<std::size_t> errors_seeds;
+  unsigned int length, defo;
+
+  for (unsigned int i=0; i<N; ++i)
+  {
+    if (i!=0)
+    {
+      random=CGAL::Random(static_cast<unsigned int>
+                          (random.get_int(0, (std::numeric_limits<int>::max)())));
+    }
+    std::cout<<"Random seed: "<<random.get_seed()<<": ";
+
+    length=static_cast<unsigned int>(random.get_int(l1, l2+1));
+    defo=static_cast<unsigned int>(random.get_int(d1, d2+1));
+
+    std::vector<Path_on_surface<LCC_3_cmap> > paths;
+    std::vector<Path_on_surface<LCC_3_cmap> > transformed_paths;
+
+    Path_on_surface<LCC_3_cmap> path1(lcc);
+    path1.generate_random_closed_path(length, random);
+
+    std::cout<<"Path1 size: "<<path1.length()<<" (from "<<length<<" darts); ";
+    paths.push_back(path1);
+
+    Path_on_surface<LCC_3_cmap> path2(path1);
+    path2.update_path_randomly(defo, random);
+    std::cout<<"Path2 size: "<<path2.length()<<" (from "<<defo<<" deformations).";
+    paths.push_back(path2);
+    std::cout<<std::endl;
+
+    if (cst.is_contractible(path1, time))
+    { ++nbcontractible; }
+
+    bool res=cst.are_freely_homotopic(path1, path2, time);
+    if (!res)
+    { errors_seeds.push_back(random.get_seed()); }
+
+#ifdef CGAL_USE_BASIC_VIEWER
+    if (draw)
+    { CGAL::draw(lcc, paths); }
+#endif
+  }
+
+  if (errors_seeds.empty())
+  {
+    if (N==1) { std::cout<<"Test OK: both paths are homotopic."<<std::endl; }
+    else
+    { std::cout<<"All the "<<N
+               <<" tests OK: each pair of paths were homotopic."<<std::endl; }
+  }
+  else
+  {
+    std::cout<<"ERRORS for "<<errors_seeds.size()<<" tests among "<<N
+            <<" (i.e. "<<static_cast<double>(errors_seeds.size()*100)/
+              static_cast<double>(N)<<"%)."<<std::endl;
+    std::cout<<"Errors for seeds: ";
+    for (std::size_t i=0; i<errors_seeds.size(); ++i)
+    { std::cout<<errors_seeds[i]<<"  "; }
+    std::cout<<std::endl;
+  }
+
+  std::cout<<"Number of contractible paths: "<<nbcontractible<<" among "<<N
+           <<" (i.e. "<<static_cast<double>(nbcontractible*100)/
+             static_cast<double>(N)<<"%)."<<std::endl;
+
+  return EXIT_SUCCESS;
+}
diff -pruN 5.0.3-1/examples/Surface_mesh_topology/path_homotopy_double_torus.cpp 5.1-1/examples/Surface_mesh_topology/path_homotopy_double_torus.cpp
--- 5.0.3-1/examples/Surface_mesh_topology/path_homotopy_double_torus.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_topology/path_homotopy_double_torus.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,65 @@
+#include <CGAL/Linear_cell_complex_for_combinatorial_map.h>
+#include <CGAL/Linear_cell_complex_constructors.h>
+#include <CGAL/Curves_on_surface_topology.h>
+#include <CGAL/Path_on_surface.h>
+#include <CGAL/draw_face_graph_with_paths.h>
+
+typedef CGAL::Linear_cell_complex_for_combinatorial_map<2,3> LCC_3_cmap;
+using namespace CGAL::Surface_mesh_topology;
+
+///////////////////////////////////////////////////////////////////////////////
+void create_path_1(Path_on_surface<LCC_3_cmap>& p)
+{
+  p.push_back_by_index(14); // Its starting dart
+  for (int i=0; i<7; ++i)
+  { p.extend_positive_turn(2); } // Extend the path
+}
+
+///////////////////////////////////////////////////////////////////////////////
+void create_path_2(Path_on_surface<LCC_3_cmap>& p)
+{ p.push_back_by_index({202, 206, 335, 317, 322, 69, 62, 414}); }
+
+///////////////////////////////////////////////////////////////////////////////
+void create_path_3(Path_on_surface<LCC_3_cmap>& p)
+{
+  p.push_back_by_index(470); // Its starting dart
+  for (int i=0; i<13; ++i)
+  { p.extend_positive_turn(2); } // Extend the path
+}
+
+///////////////////////////////////////////////////////////////////////////////
+int main(int argc, char** argv)
+{
+  bool draw=(argc>1?std::string(argv[1])=="-draw":false);
+  LCC_3_cmap lcc;
+  if (!CGAL::load_off(lcc, "data/double-torus.off"))
+  {
+    std::cout<<"ERROR reading file data/double-torus.off"<<std::endl;
+    exit(EXIT_FAILURE);
+  }
+
+  Curves_on_surface_topology<LCC_3_cmap> cst(lcc);
+  Path_on_surface<LCC_3_cmap> p1(lcc), p2(lcc), p3(lcc);
+  create_path_1(p1);
+  create_path_2(p2);
+  create_path_3(p3);
+
+  bool res1=cst.is_contractible(p1);
+  std::cout<<"Path p1 (pink) "<<(res1?"IS":"IS NOT")
+           <<" contractible."<<std::endl;
+
+  bool res2=cst.are_freely_homotopic(p1, p2);
+  std::cout<<"Path p1 (pink) "<<(res2?"IS":"IS NOT")
+           <<" homotopic with path p2 (green)."<<std::endl;
+
+  bool res3=cst.are_freely_homotopic(p1, p3);
+  std::cout<<"Path p1 (pink) "<<(res3?"IS":"IS NOT")
+           <<" homotopic with path p3 (orange)."<<std::endl;
+
+  if (draw)
+  { CGAL::draw(lcc, {p1, p2, p3}); }
+
+  return EXIT_SUCCESS;
+}
+///////////////////////////////////////////////////////////////////////////////
+
diff -pruN 5.0.3-1/examples/Surface_mesh_topology/path_homotopy_torus.cpp 5.1-1/examples/Surface_mesh_topology/path_homotopy_torus.cpp
--- 5.0.3-1/examples/Surface_mesh_topology/path_homotopy_torus.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_topology/path_homotopy_torus.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,56 @@
+#include <CGAL/Linear_cell_complex_for_combinatorial_map.h>
+#include <CGAL/Linear_cell_complex_constructors.h>
+#include <CGAL/Curves_on_surface_topology.h>
+#include <CGAL/Path_on_surface.h>
+#include <CGAL/draw_face_graph_with_paths.h>
+
+typedef CGAL::Linear_cell_complex_for_combinatorial_map<2,3> LCC_3_cmap;
+using namespace CGAL::Surface_mesh_topology;
+
+///////////////////////////////////////////////////////////////////////////////
+void create_path_1(Path_on_surface<LCC_3_cmap>& p)
+{
+  p.push_back_by_index(0); // Its starting dart
+  for (int i=0; i<4; ++i)
+  { p.extend_positive_turn(2); } // Extend the path
+}
+
+///////////////////////////////////////////////////////////////////////////////
+void create_path_2(Path_on_surface<LCC_3_cmap>& p)
+{
+  p.push_back_by_index(1); // Its starting dart
+  for (int i=0; i<4; ++i)
+  { p.extend_positive_turn(2); } // Extend the path
+}
+
+///////////////////////////////////////////////////////////////////////////////
+int main(int argc, char** argv)
+{
+  bool draw=(argc>1?std::string(argv[1])=="-draw":false);
+  LCC_3_cmap lcc;
+  if (!CGAL::load_off(lcc, "data/torus_quad.off"))
+  {
+    std::cout<<"ERROR reading file data/torus_quad.off"<<std::endl;
+    exit(EXIT_FAILURE);
+  }
+
+  Curves_on_surface_topology<LCC_3_cmap> cst(lcc);
+  Path_on_surface<LCC_3_cmap> p1(lcc), p2(lcc);
+  create_path_1(p1);
+  create_path_2(p2);
+
+  bool res1=cst.is_contractible(p1);
+  std::cout<<"Path p1 (pink) "<<(res1?"IS":"IS NOT")
+           <<" contractible."<<std::endl;
+
+  bool res2=cst.are_freely_homotopic(p1, p2);
+  std::cout<<"Path p1 (pink) "<<(res2?"IS":"IS NOT")
+           <<" homotopic with path p2 (green)."<<std::endl;
+
+  if (draw)
+  { CGAL::draw(lcc, {p1, p2}); }
+
+  return EXIT_SUCCESS;
+}
+///////////////////////////////////////////////////////////////////////////////
+
diff -pruN 5.0.3-1/examples/Surface_mesh_topology/path_homotopy_with_sm_and_polyhedron.cpp 5.1-1/examples/Surface_mesh_topology/path_homotopy_with_sm_and_polyhedron.cpp
--- 5.0.3-1/examples/Surface_mesh_topology/path_homotopy_with_sm_and_polyhedron.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_topology/path_homotopy_with_sm_and_polyhedron.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,81 @@
+#include <CGAL/Linear_cell_complex_for_combinatorial_map.h>
+#include <CGAL/Linear_cell_complex_constructors.h>
+#include <CGAL/Polyhedron_3.h>
+#include <CGAL/Surface_mesh.h>
+#include <CGAL/boost/graph/io.h>
+#include <CGAL/Curves_on_surface_topology.h>
+#include <CGAL/Path_on_surface.h>
+#include <CGAL/Face_graph_wrapper.h>
+#include <CGAL/draw_face_graph_with_paths.h>
+
+typedef CGAL::Linear_cell_complex_for_combinatorial_map<2,3> LCC_3_cmap;
+typedef CGAL::Exact_predicates_inexact_constructions_kernel Kernel;
+typedef CGAL::Polyhedron_3<Kernel>                          Polyhedron;
+typedef Kernel::Point_3                                     Point_3;
+typedef CGAL::Surface_mesh<Point_3>                         SM;
+using namespace CGAL::Surface_mesh_topology;
+
+static unsigned int seed; // Use the same seed for all the tests
+
+///////////////////////////////////////////////////////////////////////////////
+template<class FaceGraph>
+void test(const FaceGraph& mesh, bool draw, const char* title)
+{
+  CGAL::Random random(seed);
+  Curves_on_surface_topology<FaceGraph> cst(mesh);
+
+  Path_on_surface<FaceGraph> p1(mesh); // A first path
+  p1.generate_random_closed_path(10, random);
+
+  Path_on_surface<FaceGraph> p2(mesh); // A second path
+  p2.generate_random_closed_path(10, random);
+
+  bool res1=cst.is_contractible(p1, true);
+  std::cout<<"Path p1 "<<(res1?"IS":"IS NOT")<<" contractible."<<std::endl;
+
+  bool res2=cst.are_freely_homotopic(p1, p2, true);
+  std::cout<<"Path p1 "<<(res2?"IS":"IS NOT")<<" homotopic with path p2."<<std::endl;
+
+  if (draw)
+  { CGAL::draw(mesh, {p1, p2}, title); }
+}
+///////////////////////////////////////////////////////////////////////////////
+int main(int argc, char** argv)
+{
+  std::string file=(argc==1?"data/elephant.off":argv[1]);
+  bool draw=(argc>2?std::string(argv[2])=="-draw":false);
+  seed=static_cast<unsigned int>(CGAL::get_default_random().get_int(0,INT_MAX));
+
+  {
+    LCC_3_cmap lcc;
+    if (!CGAL::load_off(lcc, file.c_str()))
+    {
+      std::cout<<"ERROR reading file "<<file<<" for linear cell complex."<<std::endl;
+      exit(EXIT_FAILURE);
+    }
+    test(lcc, draw, "Linear cell complex for combinatorial map");
+  }
+
+  {
+    Polyhedron p;
+    if (!CGAL::read_off(file, p))
+    {
+      std::cout<<"ERROR reading file "<<file<<" for polyhedron."<<std::endl;
+      exit(EXIT_FAILURE);
+    }
+    test(p, draw, "Polyhedron");
+  }
+
+  {
+    SM sm;
+    if (!CGAL::read_off(file, sm))
+    {
+      std::cout<<"ERROR reading file "<<file<<" for surface mesh."<<std::endl;
+      exit(EXIT_FAILURE);
+    }
+    test(sm, draw, "Surface mesh");
+  }
+
+  return EXIT_SUCCESS;
+}
+///////////////////////////////////////////////////////////////////////////////
diff -pruN 5.0.3-1/examples/Surface_mesh_topology/path_homotopy_with_symbols_2.cpp 5.1-1/examples/Surface_mesh_topology/path_homotopy_with_symbols_2.cpp
--- 5.0.3-1/examples/Surface_mesh_topology/path_homotopy_with_symbols_2.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_topology/path_homotopy_with_symbols_2.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,37 @@
+#include <CGAL/Polygonal_schema.h>
+#include <CGAL/Path_on_surface.h>
+#include <CGAL/Curves_on_surface_topology.h>
+#include <iostream>
+#include <cstdlib>
+
+typedef CGAL::Surface_mesh_topology::Polygonal_schema_with_combinatorial_map<> PS;
+
+int main()
+{
+  PS ps;
+
+  ps.add_facet("a b -a c"); // First facet, giving directly its sequence of edges
+  ps.add_facet("d -c e -b"); // Second facet
+
+  ps.init_facet(); // Third facet
+  ps.add_edges_to_facet("f"); // Here, each edge is added one at a time
+  ps.add_edges_to_facet("-d");
+  ps.add_edges_to_facet("-f");
+  ps.add_edges_to_facet("-e");
+  ps.finish_facet();
+
+  ps.perforate_facet("f");
+
+  std::cout<<"Number of cells of the combinatorial maps: ";
+  ps.display_characteristics(std::cout)<<std::endl;
+
+  CGAL::Surface_mesh_topology::Path_on_surface<PS> p(ps);
+  p.push_back_by_label("a b -a e -b d");
+
+  CGAL::Surface_mesh_topology::Curves_on_surface_topology<PS> cst(ps);
+  bool res=cst.is_contractible(p);
+  std::cout<<"Path "<<(res?"IS":"IS NOT")<<" contractible."<<std::endl;
+
+  return EXIT_SUCCESS;
+}
+
diff -pruN 5.0.3-1/examples/Surface_mesh_topology/path_homotopy_with_symbols.cpp 5.1-1/examples/Surface_mesh_topology/path_homotopy_with_symbols.cpp
--- 5.0.3-1/examples/Surface_mesh_topology/path_homotopy_with_symbols.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_topology/path_homotopy_with_symbols.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,30 @@
+#include <CGAL/Polygonal_schema.h>
+#include <CGAL/Path_on_surface.h>
+#include <CGAL/Curves_on_surface_topology.h>
+#include <iostream>
+#include <cstdlib>
+
+using namespace CGAL::Surface_mesh_topology;
+typedef Polygonal_schema_with_combinatorial_map<> PS;
+
+int main()
+{
+  PS ps;
+  ps.add_facet("a b -a -b c d -c -d");
+
+  Path_on_surface<PS> p1(ps); p1.push_back_by_label("a");
+  Path_on_surface<PS> p2(ps); p2.push_back_by_label("b c a -c -b");
+
+  Curves_on_surface_topology<PS> cst(ps);
+
+  bool res1=cst.are_freely_homotopic(p1, p2);
+  std::cout<<"Paths p1 and p2 "<<(res1?"ARE":"ARE NOT")
+           <<" freely homotopic."<<std::endl;
+
+  bool res2=cst.are_base_point_homotopic(p1, p2);
+  std::cout<<"Paths p1 and p2 "<<(res2?"ARE":"ARE NOT")
+           <<" base point homotopic."<<std::endl;
+
+  return EXIT_SUCCESS;
+}
+
diff -pruN 5.0.3-1/examples/Surface_mesh_topology/shortest_noncontractible_cycle_2.cpp 5.1-1/examples/Surface_mesh_topology/shortest_noncontractible_cycle_2.cpp
--- 5.0.3-1/examples/Surface_mesh_topology/shortest_noncontractible_cycle_2.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_topology/shortest_noncontractible_cycle_2.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,123 @@
+#include <CGAL/Linear_cell_complex_for_generalized_map.h>
+#include <CGAL/Linear_cell_complex_constructors.h>
+#include <iostream>
+#include <fstream>
+#include <cstdlib>
+#include <CGAL/Curves_on_surface_topology.h>
+#include <CGAL/Path_on_surface.h>
+#include <CGAL/draw_face_graph_with_paths.h>
+#include <CGAL/Random.h>
+
+///////////////////////////////////////////////////////////////////////////////
+using LCC_3            =CGAL::Linear_cell_complex_for_generalized_map<2, 3>;
+using Dart_const_handle=LCC_3::Dart_const_handle;
+using Path_on_surface  =CGAL::Surface_mesh_topology::Path_on_surface<LCC_3>;
+using CST              =CGAL::Surface_mesh_topology::Curves_on_surface_topology<LCC_3>;
+///////////////////////////////////////////////////////////////////////////////
+[[ noreturn ]] void usage(int /*argc*/, char* argv[])
+{
+  std::cout<<"Usage "<<argv[0]<<" [-draw] [-dist] [-time] [-seed S] file.off"<<std::endl
+           <<"  Compute the shortest non contractible cycle passing through a vertex on the given mesh."
+           <<"  Vertex is choosen randomly."<<std::endl
+           <<"   -draw: draw the mesh and the cycle."<<std::endl
+           <<"   -dist: use Euclidean distance (instead of bfs by default)."<<std::endl
+           <<"   -time: show computation times."<<std::endl
+           <<"   -seed S: uses S as seed of random generator. Otherwise use a different seed at each run (based on time)."<<std::endl
+           <<std::endl;
+  exit(EXIT_FAILURE);
+}
+///////////////////////////////////////////////////////////////////////////////
+[[ noreturn ]] void error_command_line(int argc, char** argv, const char* msg)
+{
+  std::cout<<"[ERROR] "<<msg<<std::endl;
+  usage(argc, argv);
+}
+///////////////////////////////////////////////////////////////////////////////
+void process_command_line(int argc, char** argv,
+                          std::string& file,
+                          bool& draw,
+                          bool& dist,
+                          bool& time,
+                          CGAL::Random& random)
+{
+  std::string arg;
+  bool usage_required=false;
+  for (int i=1; i<argc; ++i)
+  {
+    arg=argv[i];
+    if (arg=="-draw")
+    { draw=true; }
+    else if (arg=="-dist")
+    { dist=true; }
+    else if (arg=="-time")
+    { time=true; }
+    else if (arg=="-seed")
+    {
+      if (i==argc-1)
+      { error_command_line(argc, argv, "no number after -seed option."); }
+      random=CGAL::Random(static_cast<unsigned int>
+                          (std::stoi(std::string(argv[++i]))));
+      // initialize the random generator with the given seed
+    }
+    else if (arg=="-h" || arg=="--help" || arg=="-?")
+    { usage_required=true; }
+    else if (arg[0]=='-')
+    { std::cout<<"[WARNING] Unknown option "<<arg<<", ignored."<<std::endl; }
+    else
+    { file=arg; }
+  }
+
+  if (usage_required) {usage(argc, argv); }
+}
+///////////////////////////////////////////////////////////////////////////////
+int main(int argc, char* argv[])
+{
+  std::cout<<"Program shortest_noncontractible_cycle_through_a_vertex started."
+           <<std::endl;
+  std::string filename("data/3torus.off");
+  bool draw=false;
+  bool dist=false;
+  bool time=false;
+  CGAL::Random random; // Used when user do not provide its own seed.
+  process_command_line(argc, argv, filename, draw, dist, time, random);
+
+  std::ifstream inp(filename);
+  if (inp.fail())
+  {
+    std::cout<<"[ERROR] Cannot read file '"<<filename<<"'. Exiting program"<<std::endl;
+    usage(argc, argv);
+  }
+
+  LCC_3 lcc;
+  CGAL::load_off(lcc, inp);
+  std::cout<<"File '"<<filename<<"' loaded. Running the main program (seed="
+           <<random.get_seed()<<")"<<std::endl;
+
+  CGAL::Surface_mesh_topology::Euclidean_length_weight_functor<LCC_3> wf(lcc);
+  CST            cst(lcc, time);
+
+  /// Change the value of `root` to test the algorithm at another vertex
+  Dart_const_handle root=lcc.dart_handle(random.get_int(0,  static_cast<int>(lcc.number_of_darts())));
+  std::cout<<"Finding the shortest noncontractible cycle..."<<std::endl;
+  Path_on_surface cycle(lcc);
+  if (dist)
+  { cycle=cst.compute_shortest_non_contractible_cycle_with_base_point(root, wf, time); }
+  else
+  { cycle=cst.compute_shortest_non_contractible_cycle_with_base_point(root, time); }
+
+  if (cycle.length()==0)
+  { std::cout<<"  Cannot find such cycle. Stop."<<std::endl; }
+  else
+  {
+    double cycle_length=0;
+    for (std::size_t i=0; i<cycle.length(); ++i)
+    { cycle_length+=wf(cycle[i]); }
+
+    std::cout<<"  Number of edges in cycle: "<<cycle.length()<<std::endl;
+    std::cout<<"  Cycle length: "<<cycle_length<<std::endl;
+    std::cout<<"  Root: "<<lcc.point(root)<<std::endl;
+    if (draw) { CGAL::draw(lcc, {cycle}); }
+  }
+
+  return EXIT_SUCCESS;
+}
diff -pruN 5.0.3-1/examples/Surface_mesh_topology/shortest_noncontractible_cycle.cpp 5.1-1/examples/Surface_mesh_topology/shortest_noncontractible_cycle.cpp
--- 5.0.3-1/examples/Surface_mesh_topology/shortest_noncontractible_cycle.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_topology/shortest_noncontractible_cycle.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,60 @@
+#include <CGAL/Linear_cell_complex_for_combinatorial_map.h>
+#include <CGAL/Linear_cell_complex_constructors.h>
+#include <CGAL/Curves_on_surface_topology.h>
+#include <CGAL/Path_on_surface.h>
+#include <CGAL/draw_face_graph_with_paths.h>
+#include <CGAL/Random.h>
+#include <iostream>
+#include <cstdlib>
+
+using LCC_3          =CGAL::Linear_cell_complex_for_combinatorial_map<2, 3>;
+using Path_on_surface=CGAL::Surface_mesh_topology::Path_on_surface<LCC_3>;
+
+double cycle_length(const LCC_3& lcc, const Path_on_surface& cycle)
+{ // Compute the length of the given cycle.
+  double res=0;
+  for (std::size_t i=0; i<cycle.length(); ++i)
+  { res+=std::sqrt
+        (CGAL::squared_distance(lcc.point(cycle[i]),
+                                lcc.point(lcc.other_extremity(cycle[i])))); }
+  return res;
+}
+
+void display_cycle_info(const LCC_3& lcc, const Path_on_surface& cycle)
+{ // Display information about the given cycle.
+  if (cycle.is_empty()) { std::cout<<"Empty."<<std::endl; return; }
+  std::cout<<"Root: "<<lcc.point(cycle[0])<<"; "
+           <<"Number of edges: "<<cycle.length()<<"; "
+           <<"Length: "<<cycle_length(lcc, cycle)<<std::endl;
+}
+
+int main(int argc, char* argv[])
+{
+  std::string filename(argc==1?"data/3torus.off":argv[1]);
+  bool draw=(argc<3?false:std::string(argv[2])=="-draw");
+
+  LCC_3 lcc;
+  if (!CGAL::load_off(lcc, filename.c_str())) // Load the off file.
+  {
+    std::cout<<"Cannot read file '"<<filename<<"'. Exiting program"<<std::endl;
+    return EXIT_FAILURE;
+  }
+  std::cout<<"File '"<<filename<<"' loaded. Finding shortest non contractible cycle..."<<std::endl;
+
+  CGAL::Surface_mesh_topology::Curves_on_surface_topology<LCC_3> cst(lcc);
+  LCC_3::Dart_const_handle root=lcc.dart_handle
+    (CGAL::get_default_random().get_int(0, static_cast<int>(lcc.number_of_darts()))); // One dart of the mesh
+
+  Path_on_surface cycle1=
+      cst.compute_shortest_non_contractible_cycle_with_base_point(root);
+
+  CGAL::Surface_mesh_topology::Euclidean_length_weight_functor<LCC_3> wf(lcc);
+  Path_on_surface cycle2=
+      cst.compute_shortest_non_contractible_cycle_with_base_point(root, wf);
+
+  std::cout<<"Cycle 1 (pink): "; display_cycle_info(lcc, cycle1);
+  std::cout<<"Cycle 2 (green): "; display_cycle_info(lcc, cycle2);
+  if (draw) { CGAL::draw(lcc, {cycle1, cycle2}); }
+
+  return EXIT_SUCCESS;
+}
diff -pruN 5.0.3-1/examples/Surface_mesh_topology/unsew_edgewidth_repeatedly.cpp 5.1-1/examples/Surface_mesh_topology/unsew_edgewidth_repeatedly.cpp
--- 5.0.3-1/examples/Surface_mesh_topology/unsew_edgewidth_repeatedly.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Surface_mesh_topology/unsew_edgewidth_repeatedly.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,170 @@
+#include <CGAL/Linear_cell_complex_for_generalized_map.h>
+#include <CGAL/Linear_cell_complex_constructors.h>
+#include <CGAL/Curves_on_surface_topology.h>
+#include <CGAL/Path_on_surface.h>
+#include <CGAL/IO/Color.h>
+#include <CGAL/draw_linear_cell_complex.h>
+#include <CGAL/squared_distance_3.h>
+#include <iostream>
+#include <fstream>
+#include <cstdlib>
+#include <unordered_set>
+
+using LCC_3            =CGAL::Linear_cell_complex_for_generalized_map<2, 3>;
+using Dart_handle      =LCC_3::Dart_handle;
+using Dart_const_handle=LCC_3::Dart_const_handle;
+using Dart_container   =std::vector<Dart_handle>;
+using Point            =LCC_3::Point;
+using Path_on_surface  =CGAL::Surface_mesh_topology::Path_on_surface<LCC_3>;
+using CST              =CGAL::Surface_mesh_topology::Curves_on_surface_topology<LCC_3>;
+
+struct Weight_functor
+{
+  Weight_functor(const LCC_3& lcc) : m_lcc(lcc) {}
+  using Weight_t=double;
+  Weight_t operator()(Dart_const_handle dh) const
+  {
+    const Point& x=m_lcc.point(dh);
+    const Point& y=m_lcc.point(m_lcc.template alpha<0>(dh));
+    return CGAL::sqrt(CGAL::squared_distance(x, y));
+  }
+private:
+  const LCC_3& m_lcc;
+};
+
+#ifdef CGAL_USE_BASIC_VIEWER
+
+struct Draw_functor : public CGAL::DefaultDrawingFunctorLCC
+{
+  Draw_functor(LCC_3::size_type am1, LCC_3::size_type am2) : is_root(am1),
+                                                             belong_to_cycle(am2)
+  {}
+
+  template<typename LCC>
+  bool colored_vertex(const LCC& alcc, typename LCC::Dart_const_handle dh) const
+  { return alcc.is_marked(dh, is_root); }
+
+  template<typename LCC>
+  CGAL::Color vertex_color(const LCC& /* alcc */,
+                           typename LCC::Dart_const_handle /* dh */) const
+  { return CGAL::Color(0,255,0); }
+
+  template<typename LCC>
+  bool colored_edge(const LCC& alcc, typename LCC::Dart_const_handle dh) const
+  { return alcc.is_marked(dh, belong_to_cycle); }
+
+  template<typename LCC>
+  CGAL::Color edge_color(const LCC& /* alcc*/,
+                         typename LCC::Dart_const_handle /* dh */) const
+  { return CGAL::Color(0, 0, 255); }
+
+  template<typename LCC>
+  bool colored_face(const LCC& /* alcc */,
+                    typename LCC::Dart_const_handle /* dh */) const {return true;}
+
+  template<typename LCC>
+  CGAL::Color face_color(const LCC& /* alcc */,
+                         typename LCC::Dart_const_handle /* dh */) const
+  {return CGAL::Color(211, 211, 211);}
+
+  template<typename LCC>
+  bool colored_volume(const LCC& /* alcc */,
+                      typename LCC::Dart_const_handle /* dh */) const { return false; }
+
+  LCC_3::size_type is_root;
+  LCC_3::size_type belong_to_cycle;
+};
+
+#endif // CGAL_USE_BASIC_VIEWER
+
+int main(int argc, char* argv[])
+{
+  std::cout<<"Program unsew_edgewidth_repeatedly started."<<std::endl;
+  std::string filename(argc==1?"data/double-torus.off":argv[1]);
+
+#ifdef CGAL_USE_BASIC_VIEWER
+  bool draw=(argc<3?false:std::string(argv[2])=="-draw");
+#endif // CGAL_USE_BASIC_VIEWER
+
+  std::ifstream inp(filename);
+  if (inp.fail())
+  {
+    std::cout<<"Cannot read file '"<<filename<<"'. Exiting program"<<std::endl;
+    return EXIT_FAILURE;
+  }
+
+  LCC_3 lccoriginal, lcccopy;
+  CGAL::load_off(lccoriginal, inp);
+  std::cout<<"File '"<<filename<<"' loaded. Running the main program..."<<std::endl;
+
+  boost::unordered_map<Dart_handle, Dart_handle> origin_to_copy;
+  lcccopy.copy(lccoriginal, &origin_to_copy, NULL);
+
+  LCC_3::size_type is_root=lccoriginal.get_new_mark();
+  LCC_3::size_type belong_to_cycle=lccoriginal.get_new_mark();
+
+  int loop=1;
+  bool cycle_exist=true;
+  Path_on_surface cycle(lcccopy);
+  Weight_functor wf(lcccopy);
+  do
+  {
+    std::cout<<"Finding #"<<loop++<<" edge-width:"<<std::endl;
+    {
+      CST cst(lcccopy);
+      cycle=cst.compute_shortest_non_contractible_cycle(wf);
+    }
+    if (cycle.length()==0)
+    { std::cout << "  Cannot find edge-width. Stop.\n"; cycle_exist=false; }
+    else
+    {
+      LCC_3::size_type is_root_copy=lcccopy.get_new_mark();
+      LCC_3::size_type belong_to_cycle_copy=lcccopy.get_new_mark();
+
+      lcccopy.mark_cell<0>(cycle[0], is_root_copy);
+      double cycle_length=0;
+      for (std::size_t i=0; i<cycle.length(); ++i)
+      {
+        cycle_length+=wf(cycle[i]);
+        if (!lcccopy.is_marked(cycle[i], belong_to_cycle_copy))
+        { lcccopy.mark_cell<1>(cycle[i], belong_to_cycle_copy); }
+      }
+
+      for (auto dh=lccoriginal.darts().begin(), dhend=lccoriginal.darts().end();
+           dh!=dhend; ++dh)
+      {
+        if (lcccopy.is_marked(origin_to_copy[dh], is_root_copy) &&
+            !lccoriginal.is_marked(dh, is_root))
+        { lccoriginal.mark(dh, is_root); }
+        if (lcccopy.is_marked(origin_to_copy[dh], belong_to_cycle_copy) &&
+            !lccoriginal.is_marked(dh, belong_to_cycle))
+        { lccoriginal.mark(dh, belong_to_cycle); }
+        if (lcccopy.is_marked(origin_to_copy[dh], belong_to_cycle_copy) &&
+            !lcccopy.is_free<2>(origin_to_copy[dh]))
+        { lcccopy.unsew<2>(origin_to_copy[dh]); }
+      }
+      lcccopy.close<2>();
+
+      lcccopy.free_mark(belong_to_cycle_copy);
+      lcccopy.free_mark(is_root_copy);
+
+      std::cout<<"  Number of edges in cycle: "<<cycle.length()<<std::endl;
+      std::cout<<"  Cycle length: "<<cycle_length<<std::endl;
+      std::cout<<"  Root: "<<lcccopy.point(cycle[0])<<std::endl;
+    }
+  }
+  while(cycle_exist);
+
+#ifdef CGAL_USE_BASIC_VIEWER
+  if (draw)
+  {
+    Draw_functor df(is_root, belong_to_cycle);
+    CGAL::draw(lccoriginal, "Unsew edge width repeatdly", false, df);
+  }
+#endif // CGAL_USE_BASIC_VIEWER
+
+  lccoriginal.free_mark(belong_to_cycle);
+  lccoriginal.free_mark(is_root);
+
+  return EXIT_SUCCESS;
+}
diff -pruN 5.0.3-1/examples/Tetrahedral_remeshing/CMakeLists.txt 5.1-1/examples/Tetrahedral_remeshing/CMakeLists.txt
--- 5.0.3-1/examples/Tetrahedral_remeshing/CMakeLists.txt	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Tetrahedral_remeshing/CMakeLists.txt	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,34 @@
+# Created by the script cgal_create_CMakeLists
+# This is the CMake script for compiling a set of CGAL applications.
+
+cmake_minimum_required(VERSION 3.1...3.14)
+
+project( Tetrahedral_remeshing_Examples )
+
+# CGAL and its components
+find_package( CGAL REQUIRED )
+if ( NOT CGAL_FOUND )
+  message(STATUS "This project requires the CGAL library, and will not be compiled.")
+  return()
+endif()
+
+# Boost and its components
+find_package( Boost REQUIRED )
+if ( NOT Boost_FOUND )
+  message(STATUS "This project requires the Boost library, and will not be compiled.")
+  return()
+endif()
+
+# Use Eigen for Mesh_3
+find_package(Eigen3 3.1.0 REQUIRED) #(3.1.0 or greater)
+include(CGAL_Eigen_support)
+
+# Creating entries for all C++ files with "main" routine
+# ##########################################################
+create_single_source_cgal_program( "tetrahedral_remeshing_example.cpp" )
+create_single_source_cgal_program( "tetrahedral_remeshing_with_features.cpp")
+create_single_source_cgal_program( "tetrahedral_remeshing_of_one_subdomain.cpp")
+create_single_source_cgal_program( "tetrahedral_remeshing_from_mesh.cpp")
+
+create_single_source_cgal_program( "mesh_and_remesh_polyhedral_domain_with_features.cpp" )
+target_link_libraries(mesh_and_remesh_polyhedral_domain_with_features PUBLIC CGAL::Eigen_support)
diff -pruN 5.0.3-1/examples/Tetrahedral_remeshing/data/fandisk.off 5.1-1/examples/Tetrahedral_remeshing/data/fandisk.off
--- 5.0.3-1/examples/Tetrahedral_remeshing/data/fandisk.off	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Tetrahedral_remeshing/data/fandisk.off	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,19425 @@
+OFF
+6475 12946 0
+
+0.1696 0.04095 -0.0471
+0.1809 0.03455 -0.046
+0.1628 0.03455 -0.046
+0.1539 0.04355 -0.0475
+0.1651 0.05375 -0.0494
+0.1811 0.05105 -0.0489
+0.1952 0.04725 -0.0482
+0.1949 0.05065 -0.0275
+0.1813 0.03875 -0.0237
+0.1666 0.02125 -0.0389
+0.1585 0.02595 -0.0412
+0.1446 0.03455 -0.046
+0.1367 0.04495 -0.0478
+0.148 0.05595 -0.0497
+0.1604 0.06725 -0.0517
+0.1778 0.06475 -0.0513
+0.1944 0.06205 -0.0508
+0.2101 0.05925 -0.0503
+0.2105 0.06305 -0.0313
+0.2093 0.06565 -0.011
+0.1945 0.05355 -0.0089
+0.1814 0.04205 -0.0057
+0.1668 0.02495 -0.0191
+0.1527 0.00675 -0.033
+0.142 0.02155 -0.039
+0.1264 0.03455 -0.046
+0.119 0.04605 -0.048
+0.1304 0.05785 -0.0501
+0.1427 0.06965 -0.0522
+0.1557 0.08145 -0.0542
+0.1737 0.07865 -0.0537
+0.1915 0.07585 -0.0532
+0.2089 0.07305 -0.0528
+0.2255 0.07055 -0.0523
+0.2256 0.07405 -0.0334
+0.2248 0.07695 -0.013
+0.2247 0.08035 0.006
+0.2093 0.06895 0.008
+0.1945 0.05685 0.0101
+0.1802 0.04415 0.0124
+0.1673 0.02835 -0.003
+0.1528 0.01005 -0.0147
+0.1396 -0.00875 -0.0287
+0.1156 -0.00185 -0.0304
+0.1199 0.02535 -0.0409
+0.1082 0.03455 -0.046
+0.1011 0.04715 -0.0482
+0.1126 0.05975 -0.0504
+0.1248 0.07225 -0.0526
+0.1376 0.08435 -0.0547
+0.1509 0.09615 -0.0568
+0.1693 0.09285 -0.0562
+0.1878 0.08965 -0.0557
+0.2061 0.08665 -0.0551
+0.2241 0.08375 -0.0546
+0.2415 0.08125 -0.0542
+0.2415 0.08455 -0.0353
+0.2407 0.08765 -0.0149
+0.2407 0.09095 0.0041
+0.2358 0.06555 0.0086
+0.221 0.05475 0.0105
+0.2068 0.04315 0.0125
+0.1932 0.03095 0.0147
+0.1801 0.01815 0.017
+0.1665 0.03065 0.0147
+0.1534 0.01375 0.0015
+0.141 -0.00105 0.0045
+0.1398 -0.00535 -0.0115
+0.1274 -0.02215 -0.0092
+0.127 -0.02565 -0.0261
+0.1081 -0.02565 -0.0261
+0.0866 -0.00565 -0.0294
+0.1038 0.01495 -0.036
+0.11 0.02835 -0.0424
+0.0965 0.02515 -0.0407
+0.09 0.03455 -0.046
+0.083 0.04855 -0.0484
+0.0947 0.06195 -0.0508
+0.1068 0.07505 -0.0531
+0.1194 0.08765 -0.0553
+0.1325 0.09975 -0.0575
+0.146 0.11145 -0.0595
+0.1647 0.10745 -0.0588
+0.1835 0.10375 -0.0582
+0.2024 0.10035 -0.0575
+0.2213 0.09695 -0.057
+0.2398 0.09375 -0.0564
+0.258 0.09105 -0.0559
+0.258 0.09435 -0.037
+0.2572 0.09755 -0.0167
+0.2572 0.10085 0.0024
+0.251 0.07575 0.0068
+0.2467 0.05075 0.0112
+0.2327 0.04045 0.013
+0.2191 0.02945 0.015
+0.2061 0.01785 0.017
+0.1937 0.00565 0.0192
+0.1819 -0.00715 0.0214
+0.1677 0.00475 0.0193
+0.1535 0.01655 0.0172
+0.1412 0.00185 0.0198
+0.1296 -0.01335 0.0225
+0.1283 -0.01805 0.0061
+0.1176 -0.03395 0.0082
+0.1168 -0.03795 -0.0072
+0.1166 -0.04135 -0.0245
+0.0986 -0.04185 -0.0245
+0.0891 -0.02565 -0.0261
+0.0702 -0.02565 -0.0261
+0.0693 -0.00485 -0.0296
+0.0819 0.00935 -0.0339
+0.0793 0.02165 -0.039
+0.0719 0.03455 -0.046
+0.0827 0.04705 -0.0544
+0.0867 0.05215 -0.053
+0.0962 0.06235 -0.0584
+0.1059 0.07215 -0.0641
+0.1173 0.08265 -0.0717
+0.1331 0.09575 -0.0834
+0.1485 0.10995 -0.0798
+0.16 0.12255 -0.0615
+0.179 0.11825 -0.0607
+0.1982 0.11425 -0.06
+0.2176 0.11045 -0.0593
+0.237 0.10665 -0.0587
+0.2561 0.10315 -0.0581
+0.2749 0.10015 -0.0575
+0.2749 0.10345 -0.0386
+0.2741 0.10665 -0.0183
+0.2741 0.10995 0.0008
+0.2668 0.08515 0.0051
+0.2612 0.06045 0.0095
+0.2575 0.03585 0.0138
+0.2442 0.02605 0.0156
+0.2313 0.01565 0.0174
+0.219 0.00455 0.0193
+0.2072 -0.00695 0.0214
+0.196 -0.01915 0.0235
+0.1853 -0.03175 0.0258
+0.1707 -0.02045 0.0238
+0.156 -0.00925 0.0218
+0.1449 -0.02385 0.0244
+0.1345 -0.03885 0.027
+0.1188 -0.02915 0.0253
+0.1087 -0.04545 0.0282
+0.1078 -0.05015 0.0106
+0.1071 -0.05415 -0.0052
+0.1069 -0.05755 -0.0228
+0.0896 -0.05825 -0.0227
+0.0803 -0.04205 -0.0244
+0.0619 -0.04225 -0.0244
+0.0512 -0.02565 -0.0261
+0.0516 -0.00295 -0.0301
+0.0695 0.00915 -0.0339
+0.0703 0.01785 -0.0373
+0.0612 0.02365 -0.04
+0.058 0.04425 -0.0524
+0.0678 0.05475 -0.0607
+0.0785 0.06495 -0.0706
+0.0935 0.05825 -0.0639
+0.1042 0.06845 -0.0744
+0.1149 0.07755 -0.0861
+0.1252 0.08555 -0.0988
+0.1426 0.10095 -0.1024
+0.1505 0.10905 -0.095
+0.1619 0.12065 -0.0807
+0.1743 0.13325 -0.0633
+0.1937 0.12855 -0.0625
+0.2133 0.12415 -0.0617
+0.2332 0.11995 -0.061
+0.2531 0.11575 -0.0603
+0.2728 0.11175 -0.0596
+0.2921 0.10835 -0.059
+0.2921 0.11165 -0.0401
+0.2915 0.11495 -0.0198
+0.2915 0.11835 -0.0007
+0.283 0.09385 0.0036
+0.2762 0.06935 0.0079
+0.2713 0.04505 0.0122
+0.2812 0.02955 0.0149
+0.2682 0.02085 0.0165
+0.2555 0.01155 0.0181
+0.2434 0.00165 0.0199
+0.2317 -0.00875 0.0217
+0.2205 -0.01975 0.0236
+0.2099 -0.03125 0.0257
+0.1998 -0.04325 0.0278
+0.1753 -0.04485 0.0281
+0.1602 -0.03425 0.0262
+0.1503 -0.04865 0.0287
+0.1411 -0.06335 0.0313
+0.1249 -0.05435 0.0297
+0.1161 -0.07035 0.0326
+0.0994 -0.06215 0.0311
+0.0986 -0.06685 0.0132
+0.0975 -0.07185 -0.0021
+0.098 -0.07425 -0.021
+0.0811 -0.07505 -0.0209
+0.0719 -0.05865 -0.0227
+0.0539 -0.05885 -0.0227
+0.0433 -0.04225 -0.0244
+0.0323 -0.02565 -0.0261
+0.028 0.00185 -0.0315
+0.0368 0.01995 -0.0383
+0.0553 0.01275 -0.0352
+0.0467 0.03255 -0.0448
+0.0425 0.05105 -0.0576
+0.0546 0.06165 -0.0671
+0.0657 0.07095 -0.0773
+0.0755 0.07775 -0.0863
+0.089 0.07405 -0.0813
+0.097 0.08095 -0.0911
+0.1062 0.08845 -0.1043
+0.1183 0.09685 -0.1248
+0.135 0.09235 -0.1126
+0.1521 0.10645 -0.117
+0.1623 0.11735 -0.102
+0.1768 0.12855 -0.1002
+0.1743 0.13015 -0.0811
+0.189 0.14335 -0.0651
+0.2088 0.13825 -0.0642
+0.2288 0.13345 -0.0634
+0.2492 0.12875 -0.0626
+0.2696 0.12415 -0.0617
+0.2899 0.11975 -0.061
+0.3098 0.11575 -0.0603
+0.3098 0.11905 -0.0413
+0.3093 0.12245 -0.0211
+0.3093 0.12585 -0.002
+0.2995 0.10175 0.0022
+0.2916 0.07765 0.0064
+0.2855 0.05355 0.0107
+0.2947 0.03765 0.0135
+0.3037 0.02165 0.0164
+0.291 0.01395 0.0177
+0.2786 0.00575 0.0191
+0.2667 -0.00305 0.0207
+0.2552 -0.01235 0.0223
+0.2442 -0.02225 0.0241
+0.2337 -0.03265 0.0259
+0.2237 -0.04345 0.0278
+0.2142 -0.05485 0.0298
+0.1903 -0.05565 0.03
+0.1659 -0.05855 0.0305
+0.1572 -0.07245 0.0329
+0.1492 -0.08695 0.0355
+0.1327 -0.07855 0.034
+0.125 -0.09405 0.0368
+0.108 -0.08665 0.0354
+0.091 -0.07925 0.0341
+0.0902 -0.08415 0.0158
+0.0894 -0.08885 -0.0002
+0.0898 -0.09125 -0.019
+0.0734 -0.09205 -0.0189
+0.0639 -0.07545 -0.0209
+0.0463 -0.07575 -0.0208
+0.0357 -0.05895 -0.0227
+0.0248 -0.04235 -0.0244
+0.0133 -0.02565 -0.0261
+0.0106 -0.00485 -0.0296
+0.0134 0.01495 -0.0361
+0.022 0.02755 -0.042
+0.032 0.04025 -0.0496
+0.0255 0.05775 -0.0634
+0.0384 0.07145 -0.078
+0.0571 0.07685 -0.0851
+0.0677 0.08045 -0.0902
+0.0801 0.08355 -0.0952
+0.085 0.09005 -0.1075
+0.0902 0.09695 -0.1252
+0.1131 0.10145 -0.1436
+0.1324 0.09915 -0.1327
+0.144 0.09775 -0.1275
+0.1607 0.11045 -0.1334
+0.1717 0.12105 -0.1214
+0.1798 0.12855 -0.1127
+0.1891 0.13715 -0.1005
+0.1891 0.14025 -0.0828
+0.2042 0.15295 -0.0668
+0.2242 0.14735 -0.0659
+0.2447 0.14215 -0.0649
+0.2655 0.13695 -0.064
+0.2865 0.13185 -0.0631
+0.3074 0.12685 -0.0622
+0.3278 0.12235 -0.0614
+0.3278 0.12565 -0.0425
+0.3274 0.12915 -0.0223
+0.3274 0.13245 -0.0032
+0.3164 0.10895 0.0009
+0.3074 0.08515 0.0051
+0.3001 0.06135 0.0093
+0.3085 0.04505 0.0122
+0.3168 0.02865 0.0151
+0.3055 0.02605 0.0355
+0.2928 0.01855 0.0368
+0.2805 0.01045 0.0383
+0.2687 0.00185 0.0398
+0.2575 -0.00705 0.0414
+0.2476 -0.01575 0.0429
+0.2404 -0.02245 0.0441
+0.2312 -0.03175 0.0457
+0.2217 -0.04225 0.0476
+0.2126 -0.05345 0.0496
+0.2053 -0.06655 0.0319
+0.1815 -0.06855 0.0322
+0.1733 -0.08175 0.0346
+0.1657 -0.09545 0.037
+0.1419 -0.10165 0.0381
+0.1353 -0.11685 0.0408
+0.118 -0.11005 0.0396
+0.1007 -0.10345 0.0384
+0.0834 -0.09675 0.0372
+0.0834 -0.10015 0.0182
+0.0828 -0.10455 0.0013
+0.0825 -0.10865 -0.0169
+0.0663 -0.10945 -0.0168
+0.0565 -0.09255 -0.0189
+0.0392 -0.09285 -0.0188
+0.0284 -0.07585 -0.0208
+0.0174 -0.05905 -0.0227
+0.0061 -0.04235 -0.0244
+-0.0056 -0.02565 -0.0261
+-0.0056 -0.00605 -0.0293
+0.0059 0.00555 -0.0326
+-0.0025 0.02095 -0.0387
+0.0064 0.03465 -0.046
+0.0162 0.04645 -0.054
+0.0075 0.06325 -0.0688
+0.0125 0.07285 -0.0797
+0.0373 0.08155 -0.092
+0.0543 0.08355 -0.0952
+0.0682 0.08465 -0.0971
+0.0701 0.09075 -0.109
+0.0726 0.09715 -0.1256
+0.0743 0.10145 -0.1436
+0.0937 0.10145 -0.1436
+0.0921 0.10525 -0.1646
+0.1117 0.10635 -0.1713
+0.1324 0.10145 -0.1436
+0.1518 0.10145 -0.1436
+0.1704 0.11485 -0.1511
+0.1867 0.12935 -0.1351
+0.1899 0.13485 -0.1166
+0.2042 0.14665 -0.1022
+0.2042 0.14985 -0.0845
+0.2196 0.16195 -0.0684
+0.24 0.15585 -0.0674
+0.2609 0.15015 -0.0663
+0.2822 0.14455 -0.0653
+0.3037 0.13875 -0.0643
+0.3251 0.13315 -0.0633
+0.3462 0.12815 -0.0625
+0.3461 0.13145 -0.0435
+0.3458 0.13495 -0.0233
+0.3458 0.13825 -0.0042
+0.3337 0.11525 -0.0002
+0.3235 0.09195 0.0039
+0.3151 0.06855 0.0081
+0.3227 0.05185 0.011
+0.3302 0.03505 0.014
+0.3185 0.03295 0.0343
+0.3069 0.03025 0.0548
+0.2943 0.02285 0.0561
+0.282 0.01495 0.0575
+0.2701 0.00635 0.059
+0.2588 -0.00255 0.0606
+0.2483 -0.01165 0.0621
+0.239 -0.02045 0.0637
+0.2295 -0.03025 0.0654
+0.2201 -0.04085 0.0673
+0.211 -0.05205 0.0693
+0.2038 -0.06515 0.0516
+0.1969 -0.07875 0.0341
+0.1891 -0.09125 0.0362
+0.182 -0.10415 0.0385
+0.1587 -0.10945 0.0395
+0.1525 -0.12375 0.042
+0.1294 -0.13225 0.0435
+0.1119 -0.12625 0.0424
+0.0943 -0.12045 0.0414
+0.0766 -0.11465 0.0404
+0.0766 -0.11795 0.0213
+0.0762 -0.12235 0.0033
+0.0759 -0.12645 -0.0146
+0.0601 -0.12705 -0.0145
+0.0497 -0.10985 -0.0167
+0.0327 -0.11015 -0.0167
+0.0217 -0.09305 -0.0188
+0.0105 -0.07595 -0.0208
+-0.0009 -0.05915 -0.0226
+-0.0125 -0.04235 -0.0244
+-0.0245 -0.02565 -0.0261
+-0.0245 -0.00645 -0.0292
+-0.0233 0.00845 -0.0336
+-0.0058 0.00675 -0.033
+-0.0179 0.02615 -0.0413
+-0.0092 0.04275 -0.0513
+0.0004 0.05475 -0.0607
+-0.0028 0.06575 -0.0714
+-0.0047 0.07275 -0.0796
+-0.0052 0.08145 -0.0919
+0.0148 0.08175 -0.0924
+0.0355 0.08975 -0.1069
+0.0549 0.09095 -0.1095
+0.0549 0.09685 -0.1248
+0.0549 0.10145 -0.1436
+0.0565 0.10505 -0.1639
+0.0759 0.10515 -0.1644
+0.0893 0.10735 -0.1766
+0.0987 0.10885 -0.1853
+0.1293 0.11045 -0.1945
+0.1404 0.10535 -0.1653
+0.1594 0.10455 -0.1611
+0.1801 0.11875 -0.1693
+0.1957 0.13095 -0.1607
+0.2083 0.14315 -0.1369
+0.2059 0.14485 -0.1186
+0.2197 0.15575 -0.1038
+0.2197 0.15885 -0.0861
+0.2354 0.17045 -0.0699
+0.2561 0.16385 -0.0688
+0.2774 0.15755 -0.0676
+0.2992 0.15135 -0.0666
+0.3212 0.14495 -0.0654
+0.3431 0.13875 -0.0643
+0.3648 0.13305 -0.0633
+0.3647 0.13635 -0.0444
+0.3645 0.13985 -0.0242
+0.3645 0.14325 -0.0051
+0.3512 0.12075 -0.0011
+0.3399 0.09795 0.0029
+0.3304 0.07495 0.0069
+0.3372 0.05795 0.0099
+0.3439 0.04085 0.013
+0.3318 0.03925 0.0332
+0.3198 0.03705 0.0536
+0.3077 0.03405 0.074
+0.2951 0.02685 0.0753
+0.2829 0.01895 0.0767
+0.271 0.01045 0.0782
+0.2596 0.00145 0.0798
+0.2487 -0.00785 0.0814
+0.2385 -0.01755 0.0831
+0.2287 -0.02765 0.0849
+0.2192 -0.03845 0.0868
+0.2102 -0.04975 0.0888
+0.2024 -0.06365 0.0713
+0.1957 -0.07715 0.0537
+0.1881 -0.08965 0.0559
+0.1811 -0.10245 0.0582
+0.1755 -0.11735 0.0409
+0.1696 -0.13085 0.0432
+0.147 -0.13835 0.0445
+0.1243 -0.14785 0.0462
+0.1065 -0.14275 0.0453
+0.0886 -0.13775 0.0444
+0.0707 -0.13275 0.0436
+0.0707 -0.13605 0.0245
+0.07 -0.14185 0.0066
+0.0702 -0.14445 -0.0122
+0.0546 -0.14505 -0.0121
+0.0437 -0.12755 -0.0145
+0.0269 -0.12775 -0.0144
+0.0155 -0.11035 -0.0167
+0.004 -0.09315 -0.0188
+-0.0075 -0.07605 -0.0208
+-0.0192 -0.05915 -0.0226
+-0.0312 -0.04235 -0.0244
+-0.0435 -0.02565 -0.0261
+-0.0435 -0.00665 -0.0292
+-0.0439 0.00985 -0.0341
+-0.0403 0.03355 -0.0454
+-0.0258 0.04765 -0.0549
+-0.0115 0.06035 -0.0659
+-0.0231 0.06825 -0.0742
+-0.0233 0.08105 -0.0912
+-0.0227 0.09005 -0.1076
+-0.0033 0.08995 -0.1072
+0.0161 0.08985 -0.107
+0.0356 0.09685 -0.1248
+0.0356 0.10145 -0.1436
+0.0341 0.10485 -0.1625
+0.0527 0.10925 -0.1877
+0.078 0.10795 -0.1803
+0.0868 0.11055 -0.1949
+0.0958 0.11325 -0.2099
+0.1112 0.11195 -0.2031
+0.1226 0.11505 -0.2203
+0.1403 0.11335 -0.2109
+0.1498 0.10895 -0.1857
+0.167 0.10755 -0.178
+0.1879 0.12125 -0.1859
+0.202 0.13205 -0.1774
+0.2117 0.14175 -0.1564
+0.2221 0.15125 -0.1374
+0.221 0.15355 -0.1207
+0.2354 0.16415 -0.1053
+0.2354 0.16725 -0.0876
+0.2515 0.17825 -0.0713
+0.2725 0.17115 -0.07
+0.2941 0.16435 -0.0688
+0.3164 0.15755 -0.0676
+0.3388 0.15055 -0.0664
+0.3612 0.14365 -0.0652
+0.3836 0.13705 -0.064
+0.3836 0.14035 -0.0451
+0.3834 0.14395 -0.0249
+0.3834 0.14725 -0.0058
+0.369 0.12545 -0.002
+0.3565 0.10325 0.002
+0.346 0.08065 0.0059
+0.3519 0.06335 0.009
+0.3578 0.04595 0.012
+0.3454 0.04485 0.0322
+0.333 0.04315 0.0525
+0.3206 0.04075 0.0729
+0.308 0.03765 0.0934
+0.2954 0.03045 0.0947
+0.2832 0.02255 0.096
+0.2714 0.01415 0.0975
+0.2599 0.00515 0.0991
+0.2489 -0.00425 0.1008
+0.2385 -0.01415 0.1025
+0.2285 -0.02445 0.1043
+0.219 -0.03535 0.1063
+0.2099 -0.04665 0.1082
+0.2017 -0.06145 0.0909
+0.1944 -0.07575 0.0734
+0.187 -0.08815 0.0756
+0.1802 -0.10085 0.0779
+0.1748 -0.11555 0.0605
+0.1691 -0.12885 0.0629
+0.1644 -0.14465 0.0457
+0.1421 -0.15315 0.0472
+0.12 -0.16383 0.0491
+0.1019 -0.15951 0.0483
+0.0838 -0.15525 0.0475
+0.0657 -0.15115 0.0468
+0.0657 -0.15445 0.0277
+0.0652 -0.15992 0.0091
+0.0654 -0.16264 -0.0097
+0.0498 -0.16313 -0.0096
+0.0383 -0.14535 -0.0121
+0.0217 -0.14565 -0.0121
+0.0099 -0.12795 -0.0144
+-0.0019 -0.11045 -0.0167
+-0.0137 -0.09325 -0.0188
+-0.0256 -0.07615 -0.0208
+-0.0376 -0.05915 -0.0226
+-0.0498 -0.04235 -0.0244
+-0.0624 -0.02565 -0.0261
+-0.0624 -0.00685 -0.0292
+-0.0637 0.00985 -0.0341
+-0.0606 0.03115 -0.044
+-0.047 0.04285 -0.0514
+-0.04 0.05155 -0.058
+-0.048 0.07045 -0.0767
+-0.0447 0.08185 -0.0925
+-0.0422 0.09035 -0.1082
+-0.0226 0.09695 -0.125
+-0.0032 0.09695 -0.1249
+0.0162 0.09685 -0.1248
+0.0162 0.10145 -0.1436
+0.015 0.10485 -0.1629
+0.0293 0.10845 -0.1832
+0.0374 0.11245 -0.2054
+0.067 0.11225 -0.2045
+0.0771 0.11525 -0.2215
+0.105 0.11685 -0.2308
+0.1312 0.11815 -0.2381
+0.1487 0.11645 -0.2283
+0.1661 0.11495 -0.2196
+0.1578 0.11195 -0.2027
+0.1749 0.11055 -0.1947
+0.196 0.12385 -0.2023
+0.2098 0.13395 -0.1937
+0.2169 0.14205 -0.1723
+0.2252 0.14995 -0.1542
+0.2369 0.15905 -0.1387
+0.2355 0.16105 -0.123
+0.2515 0.16895 -0.1243
+0.2515 0.17205 -0.1067
+0.2515 0.17515 -0.089
+0.2679 0.18555 -0.0726
+0.289 0.17795 -0.0713
+0.3108 0.17065 -0.07
+0.3332 0.16345 -0.0687
+0.3559 0.15585 -0.0673
+0.3791 0.14795 -0.0659
+0.4025 0.14015 -0.0646
+0.4025 0.14355 -0.0456
+0.4025 0.14705 -0.0255
+0.4025 0.15045 -0.0064
+0.387 0.12935 -0.0026
+0.3735 0.10765 0.0012
+0.3778 0.08985 0.0043
+0.3618 0.08565 0.0051
+0.3669 0.06805 0.0082
+0.372 0.05045 0.0113
+0.3592 0.04985 0.0313
+0.3465 0.04875 0.0515
+0.3337 0.04685 0.0718
+0.3208 0.04435 0.0922
+0.3208 0.04775 0.1116
+0.308 0.04105 0.1128
+0.2954 0.03385 0.114
+0.2832 0.02595 0.1154
+0.2714 0.01765 0.1169
+0.26 0.00865 0.1185
+0.249 -0.00075 0.1201
+0.2385 -0.01065 0.1219
+0.2285 -0.02105 0.1237
+0.219 -0.03185 0.1256
+0.21 -0.04315 0.1276
+0.2015 -0.05485 0.1296
+0.2014 -0.05835 0.1103
+0.1937 -0.07345 0.093
+0.1864 -0.08585 0.0952
+0.1797 -0.09845 0.0974
+0.174 -0.11385 0.0802
+0.1684 -0.12715 0.0825
+0.164 -0.14255 0.0653
+0.1599 -0.15858 0.0481
+0.1381 -0.16822 0.0498
+0.1165 -0.17977 0.0519
+0.0982 -0.17626 0.0512
+0.0799 -0.17292 0.0507
+0.0616 -0.1697 0.0501
+0.0616 -0.17304 0.031
+0.0613 -0.17819 0.0118
+0.0614 -0.18101 -0.007
+0.0458 -0.18142 -0.0069
+0.0336 -0.16343 -0.0095
+0.017 -0.16364 -0.0095
+0.0048 -0.14575 -0.012
+-0.0074 -0.12805 -0.0144
+-0.0195 -0.11055 -0.0167
+-0.0315 -0.09325 -0.0188
+-0.0437 -0.07615 -0.0208
+-0.056 -0.05925 -0.0226
+-0.0685 -0.04245 -0.0244
+-0.0814 -0.02565 -0.0261
+-0.0814 -0.00685 -0.0291
+-0.0817 0.01025 -0.0343
+-0.0808 0.03045 -0.0436
+-0.08 0.04295 -0.0515
+-0.0591 0.04305 -0.0515
+-0.0561 0.05465 -0.0606
+-0.0701 0.07305 -0.08
+-0.0645 0.08305 -0.0945
+-0.0609 0.09075 -0.109
+-0.0419 0.09695 -0.1252
+-0.0226 0.10145 -0.1436
+-0.0032 0.10145 -0.1436
+-0.0049 0.10485 -0.1629
+0.0136 0.10815 -0.1813
+0.0241 0.11125 -0.1986
+0.0204 0.11295 -0.2085
+0.0267 0.11425 -0.216
+0.0492 0.11455 -0.2176
+0.059 0.11745 -0.234
+0.0873 0.11885 -0.2418
+0.1137 0.12005 -0.249
+0.1401 0.12135 -0.2559
+0.1574 0.11945 -0.2456
+0.1747 0.11785 -0.2364
+0.1916 0.11635 -0.2279
+0.1831 0.11345 -0.2114
+0.2044 0.12635 -0.2185
+0.218 0.13605 -0.2098
+0.2234 0.14335 -0.186
+0.2293 0.14975 -0.168
+0.2382 0.15725 -0.1528
+0.2511 0.16385 -0.1516
+0.2511 0.16605 -0.1394
+0.268 0.17395 -0.1387
+0.2679 0.17625 -0.1255
+0.2679 0.17935 -0.1079
+0.2679 0.18245 -0.0903
+0.2845 0.19225 -0.0738
+0.3054 0.18435 -0.0724
+0.3269 0.17685 -0.071
+0.3491 0.16925 -0.0697
+0.3723 0.16105 -0.0683
+0.3965 0.15195 -0.0667
+0.4216 0.14245 -0.065
+0.4216 0.14575 -0.0461
+0.4216 0.14935 -0.0259
+0.4216 0.15265 -0.0068
+0.4051 0.13235 -0.0032
+0.3906 0.11135 0.0005
+0.3941 0.09335 0.0037
+0.3976 0.07535 0.0069
+0.3822 0.07205 0.0074
+0.3865 0.05425 0.0106
+0.3733 0.05425 0.0306
+0.3603 0.05365 0.0506
+0.3472 0.05235 0.0708
+0.3339 0.05035 0.0911
+0.3339 0.05375 0.1105
+0.3338 0.05715 0.1299
+0.3207 0.05115 0.1309
+0.3079 0.04445 0.1321
+0.2954 0.03725 0.1334
+0.2832 0.02935 0.1348
+0.2714 0.02105 0.1362
+0.26 0.01205 0.1378
+0.2491 0.00265 0.1395
+0.2386 -0.00715 0.1412
+0.2286 -0.01755 0.1431
+0.2191 -0.02835 0.145
+0.2101 -0.03965 0.147
+0.2016 -0.05125 0.149
+0.1937 -0.06335 0.1511
+0.1936 -0.06685 0.1318
+0.1935 -0.07035 0.1124
+0.1862 -0.08275 0.1146
+0.1795 -0.09535 0.1168
+0.1735 -0.11145 0.0997
+0.168 -0.12475 0.102
+0.1634 -0.14075 0.0849
+0.1596 -0.15635 0.0677
+0.1561 -0.17274 0.0506
+0.1347 -0.18339 0.0525
+0.1137 -0.19622 0.0548
+0.0952 -0.19358 0.0543
+0.0768 -0.19097 0.0538
+0.0584 -0.18841 0.0534
+0.0584 -0.19175 0.0344
+0.0582 -0.19659 0.0147
+0.0583 -0.19953 -0.0041
+0.0424 -0.19985 -0.0041
+0.0295 -0.18168 -0.0069
+0.0129 -0.18185 -0.0068
+0.0002 -0.1638 -0.0095
+-0.0123 -0.14585 -0.012
+-0.0247 -0.12815 -0.0144
+-0.037 -0.11065 -0.0167
+-0.0494 -0.09335 -0.0188
+-0.0618 -0.07625 -0.0208
+-0.0743 -0.05925 -0.0226
+-0.0871 -0.04245 -0.0244
+-0.1003 -0.02565 -0.0261
+-0.1003 -0.00695 -0.0291
+-0.1004 0.01055 -0.0344
+-0.1001 0.02845 -0.0425
+-0.1004 0.04265 -0.0512
+-0.0794 0.05635 -0.0621
+-0.0893 0.07535 -0.083
+-0.0841 0.08415 -0.0962
+-0.0813 0.09125 -0.1101
+-0.0613 0.09715 -0.1255
+-0.0419 0.10145 -0.1436
+-0.0235 0.10535 -0.1656
+-0.0062 0.10815 -0.1813
+0.0113 0.11075 -0.1959
+0.0109 0.11335 -0.2107
+0.017 0.11565 -0.2238
+0.0357 0.11675 -0.2299
+0.0442 0.11965 -0.2465
+0.07 0.12085 -0.2535
+0.0964 0.12215 -0.2607
+0.1229 0.12335 -0.2673
+0.1495 0.12445 -0.2736
+0.1665 0.12255 -0.2627
+0.1837 0.12085 -0.2531
+0.2004 0.11925 -0.2442
+0.2131 0.12885 -0.2345
+0.2265 0.13815 -0.2256
+0.2315 0.14505 -0.2018
+0.2349 0.15065 -0.1806
+0.2413 0.15675 -0.1645
+0.2493 0.16125 -0.1617
+0.2571 0.16375 -0.1688
+0.2683 0.17135 -0.1542
+0.2835 0.17905 -0.1461
+0.2834 0.18075 -0.1367
+0.2845 0.18315 -0.1249
+0.2846 0.18595 -0.1091
+0.2846 0.18915 -0.0915
+0.3014 0.19835 -0.0748
+0.3214 0.19045 -0.0734
+0.3421 0.18305 -0.0721
+0.364 0.17545 -0.0708
+0.3874 0.16675 -0.0693
+0.4129 0.15625 -0.0674
+0.4408 0.14375 -0.0652
+0.4407 0.14715 -0.0462
+0.4409 0.15065 -0.0263
+0.4409 0.15405 -0.007
+0.4234 0.13445 -0.0036
+0.4078 0.11415 0
+0.4105 0.09605 0.0032
+0.4131 0.07795 0.0064
+0.4157 0.05985 0.0096
+0.401 0.05735 0.01
+0.3876 0.05795 0.0299
+0.3743 0.05795 0.0499
+0.3609 0.05725 0.07
+0.3473 0.05585 0.0902
+0.3473 0.05925 0.1096
+0.3472 0.06265 0.1289
+0.3471 0.06605 0.1483
+0.3338 0.06055 0.1493
+0.3207 0.05445 0.1503
+0.3078 0.04785 0.1515
+0.2953 0.04055 0.1528
+0.2832 0.03275 0.1542
+0.2714 0.02445 0.1556
+0.26 0.01555 0.1572
+0.2491 0.00615 0.1588
+0.2387 -0.00375 0.1606
+0.2287 -0.01405 0.1624
+0.2191 -0.02485 0.1643
+0.2102 -0.03605 0.1663
+0.2017 -0.04775 0.1683
+0.1938 -0.05975 0.1705
+0.1865 -0.07205 0.1726
+0.1864 -0.07565 0.1533
+0.1863 -0.07925 0.134
+0.1796 -0.09185 0.1362
+0.1735 -0.10475 0.1385
+0.1734 -0.10835 0.1191
+0.1679 -0.12155 0.1215
+0.1631 -0.13825 0.1044
+0.1591 -0.15445 0.0873
+0.1558 -0.17034 0.0702
+0.1529 -0.18707 0.0532
+0.132 -0.19888 0.0552
+0.1117 -0.21255 0.0576
+0.0932 -0.21078 0.0573
+0.0747 -0.20903 0.057
+0.0562 -0.20729 0.0567
+0.0562 -0.21061 0.0377
+0.0561 -0.21507 0.0178
+0.0561 -0.21815 -0.0012
+0.0397 -0.21839 -0.0011
+0.026 -0.20005 -0.0041
+0.0091 -0.20019 -0.004
+-0.0041 -0.18197 -0.0068
+-0.0169 -0.1639 -0.0095
+-0.0295 -0.14595 -0.012
+-0.0421 -0.12825 -0.0144
+-0.0547 -0.11065 -0.0166
+-0.0672 -0.09335 -0.0188
+-0.0799 -0.07625 -0.0208
+-0.0927 -0.05925 -0.0226
+-0.1058 -0.04245 -0.0244
+-0.1193 -0.02565 -0.0261
+-0.1193 -0.00695 -0.0291
+-0.1193 0.01085 -0.0345
+-0.1191 0.02845 -0.0425
+-0.1181 0.04235 -0.051
+-0.1171 0.05215 -0.0584
+-0.1027 0.05785 -0.0635
+-0.1173 0.06825 -0.0742
+-0.1069 0.07905 -0.0881
+-0.1017 0.08585 -0.0993
+-0.0986 0.09145 -0.1106
+-0.0807 0.09725 -0.1261
+-0.0613 0.10145 -0.1436
+-0.0641 0.10485 -0.1625
+-0.047 0.10605 -0.1692
+-0.0383 0.10885 -0.1854
+-0.0245 0.10845 -0.183
+-0.0076 0.11045 -0.1946
+-0.0093 0.11425 -0.2158
+0.0028 0.11775 -0.236
+0.0253 0.11835 -0.2389
+0.0339 0.12105 -0.2547
+0.039 0.12285 -0.2646
+0.0515 0.12295 -0.2652
+0.0692 0.12625 -0.2838
+0.0823 0.12425 -0.2727
+0.106 0.12545 -0.2792
+0.1326 0.12655 -0.2855
+0.1593 0.12755 -0.2912
+0.176 0.12555 -0.2798
+0.193 0.12375 -0.2696
+0.2095 0.12205 -0.2603
+0.2222 0.13135 -0.2504
+0.2355 0.14025 -0.2411
+0.2399 0.14675 -0.2173
+0.2442 0.15275 -0.1953
+0.2466 0.15755 -0.1752
+0.2554 0.15945 -0.1886
+0.2672 0.16625 -0.1802
+0.282 0.17495 -0.1661
+0.2933 0.18175 -0.152
+0.2967 0.18515 -0.1401
+0.299 0.18845 -0.1258
+0.3014 0.19205 -0.1101
+0.3014 0.19515 -0.0925
+0.3186 0.20065 -0.0935
+0.3186 0.20375 -0.0758
+0.337 0.19625 -0.0745
+0.3564 0.18945 -0.0733
+0.3773 0.18235 -0.072
+0.4003 0.17395 -0.0705
+0.4268 0.16245 -0.0685
+0.4603 0.14425 -0.0653
+0.4603 0.14765 -0.0459
+0.4603 0.15105 -0.0265
+0.4603 0.15445 -0.0071
+0.4418 0.13575 -0.0038
+0.4252 0.11625 -0.0003
+0.427 0.09805 0.0029
+0.4288 0.07975 0.0061
+0.4305 0.06155 0.0093
+0.4166 0.06335 0.029
+0.4021 0.06095 0.0294
+0.3886 0.06155 0.0492
+0.3749 0.06155 0.0692
+0.3611 0.06075 0.0893
+0.361 0.06415 0.1087
+0.3609 0.06745 0.128
+0.3608 0.07085 0.1474
+0.3608 0.07425 0.1668
+0.3472 0.06945 0.1677
+0.3338 0.06395 0.1686
+0.3207 0.05795 0.1697
+0.3079 0.05125 0.1708
+0.2954 0.04405 0.1721
+0.2833 0.03625 0.1735
+0.2716 0.02795 0.175
+0.2602 0.01915 0.1765
+0.2493 0.00975 0.1782
+0.2389 -5e-005 0.1799
+0.2289 -0.01035 0.1817
+0.2194 -0.02115 0.1836
+0.2104 -0.03235 0.1856
+0.202 -0.04385 0.1876
+0.1941 -0.05585 0.1897
+0.1868 -0.06815 0.1919
+0.18 -0.08075 0.1941
+0.1797 -0.08465 0.1749
+0.1796 -0.08825 0.1555
+0.1735 -0.10115 0.1578
+0.1681 -0.11435 0.1601
+0.168 -0.11795 0.1408
+0.1631 -0.13145 0.1432
+0.163 -0.13505 0.1238
+0.1589 -0.15185 0.1069
+0.1555 -0.16831 0.0898
+0.1527 -0.18446 0.0726
+0.1504 -0.20156 0.0557
+0.1302 -0.21434 0.058
+0.1105 -0.22895 0.0605
+0.0919 -0.22805 0.0604
+0.0733 -0.22715 0.0602
+0.0548 -0.22622 0.06
+0.0548 -0.22958 0.0409
+0.0548 -0.23361 0.0211
+0.0548 -0.23686 0.002
+0.0376 -0.23696 0.002
+0.0228 -0.21852 -0.0011
+0.0056 -0.21861 -0.0011
+-0.008 -0.20028 -0.004
+-0.0212 -0.18206 -0.0068
+-0.034 -0.16397 -0.0095
+-0.0468 -0.14605 -0.012
+-0.0595 -0.12825 -0.0144
+-0.0723 -0.11075 -0.0166
+-0.0851 -0.09345 -0.0187
+-0.098 -0.07625 -0.0208
+-0.1111 -0.05935 -0.0226
+-0.1244 -0.04245 -0.0244
+-0.1382 -0.02565 -0.0261
+-0.1382 -0.00695 -0.0291
+-0.1382 0.01095 -0.0345
+-0.1381 0.02845 -0.0425
+-0.1348 0.04635 -0.0539
+-0.1272 0.05785 -0.0635
+-0.1414 0.06525 -0.0709
+-0.1289 0.07535 -0.0829
+-0.1198 0.08285 -0.0941
+-0.1158 0.08775 -0.1028
+-0.1148 0.09215 -0.1122
+-0.0989 0.09745 -0.1266
+-0.0807 0.10145 -0.1436
+-0.0799 0.10475 -0.162
+-0.0666 0.10675 -0.1735
+-0.0587 0.10805 -0.1806
+-0.0487 0.10995 -0.1916
+-0.0349 0.11025 -0.1934
+-0.0258 0.11055 -0.1947
+-0.0297 0.11275 -0.2073
+-0.0256 0.11765 -0.2353
+-0.0122 0.12015 -0.2494
+0.0123 0.12045 -0.2513
+0.0238 0.12265 -0.2635
+0.033 0.12425 -0.2728
+0.0427 0.12585 -0.2814
+0.0588 0.12855 -0.2968
+0.0811 0.12875 -0.2982
+0.0921 0.12725 -0.2896
+0.0988 0.12955 -0.3025
+0.1164 0.12815 -0.2949
+0.1263 0.13105 -0.311
+0.1427 0.12965 -0.3034
+0.1695 0.13065 -0.3087
+0.186 0.12845 -0.2967
+0.2026 0.12655 -0.2859
+0.219 0.12495 -0.2763
+0.2317 0.13385 -0.266
+0.2449 0.14235 -0.2565
+0.2488 0.14855 -0.2325
+0.2538 0.15475 -0.2111
+0.2672 0.16205 -0.2041
+0.279 0.16875 -0.1943
+0.2956 0.17815 -0.1774
+0.3069 0.18545 -0.1582
+0.3115 0.18975 -0.1423
+0.3141 0.19335 -0.1269
+0.3168 0.19705 -0.1106
+0.3356 0.20235 -0.1119
+0.3358 0.20545 -0.0943
+0.3358 0.20855 -0.0766
+0.3517 0.20205 -0.0755
+0.369 0.19635 -0.0745
+0.3882 0.19065 -0.0735
+0.4097 0.18395 -0.0723
+0.4336 0.17535 -0.0708
+0.4603 0.16435 -0.0688
+0.4603 0.16705 -0.0493
+0.4603 0.17005 -0.0299
+0.4603 0.17285 -0.0138
+0.4603 0.17635 0.0011
+0.4603 0.15885 0.0093
+0.4603 0.13615 -0.0039
+0.4427 0.11745 -0.0006
+0.4436 0.09915 0.0027
+0.4445 0.08085 0.0059
+0.4454 0.06265 0.0091
+0.4311 0.06505 0.0286
+0.4172 0.06685 0.0483
+0.4029 0.06455 0.0487
+0.389 0.06515 0.0686
+0.375 0.06495 0.0886
+0.3749 0.06835 0.1079
+0.3748 0.07175 0.1273
+0.3747 0.07515 0.1467
+0.3746 0.07855 0.1661
+0.3692 0.09585 0.163
+0.3545 0.09135 0.1638
+0.3402 0.08625 0.1647
+0.326 0.08045 0.1657
+0.3121 0.07395 0.1669
+0.2985 0.06695 0.1681
+0.2853 0.05925 0.1694
+0.2725 0.05095 0.1709
+0.2601 0.04215 0.1725
+0.2481 0.03275 0.1741
+0.2365 0.02285 0.1759
+0.2255 0.01235 0.1777
+0.2149 0.00145 0.1796
+0.2049 -0.00995 0.1817
+0.1954 -0.02185 0.1837
+0.1865 -0.03405 0.1859
+0.1781 -0.04675 0.1881
+0.1704 -0.05975 0.1904
+0.1633 -0.07305 0.1928
+0.1739 -0.09365 0.1964
+0.1736 -0.09755 0.1771
+0.1681 -0.11075 0.1795
+0.1632 -0.12425 0.1818
+0.1632 -0.12785 0.1625
+0.1589 -0.14145 0.1649
+0.1588 -0.14515 0.1456
+0.1588 -0.14865 0.1262
+0.1553 -0.1657 0.1093
+0.1525 -0.18226 0.0922
+0.1504 -0.19863 0.0752
+0.1487 -0.21612 0.0583
+0.129 -0.22985 0.0607
+0.1101 -0.24528 0.0634
+0.0915 -0.24528 0.0634
+0.0729 -0.24528 0.0634
+0.0543 -0.24528 0.0634
+0.0543 -0.248714 0.044
+0.0543 -0.252132 0.0246
+0.0543 -0.25555 0.0052
+0.0359 -0.25555 0.0052
+0.0201 -0.23703 0.002
+0.0023 -0.23707 0.002
+-0.0119 -0.21867 -0.0011
+-0.0253 -0.20034 -0.004
+-0.0384 -0.18212 -0.0068
+-0.0513 -0.16403 -0.0095
+-0.0641 -0.14605 -0.012
+-0.077 -0.12835 -0.0144
+-0.0899 -0.11075 -0.0166
+-0.103 -0.09345 -0.0187
+-0.1161 -0.07635 -0.0208
+-0.1295 -0.05935 -0.0226
+-0.1431 -0.04245 -0.0244
+-0.1572 -0.02565 -0.0261
+-0.1572 -0.00695 -0.0291
+-0.1572 0.01105 -0.0346
+-0.1571 0.02845 -0.0425
+-0.1561 0.05125 -0.0578
+-0.1696 0.06615 -0.0719
+-0.1534 0.07355 -0.0806
+-0.1397 0.08155 -0.092
+-0.1294 0.08625 -0.1
+-0.1238 0.08855 -0.1045
+-0.1263 0.09175 -0.1111
+-0.1164 0.09755 -0.1271
+-0.1001 0.10145 -0.1436
+-0.1042 0.10525 -0.165
+-0.0913 0.10845 -0.1828
+-0.0761 0.10735 -0.1765
+-0.0697 0.10915 -0.1871
+-0.0602 0.11125 -0.1985
+-0.0493 0.11325 -0.2102
+-0.0407 0.11155 -0.2004
+-0.0398 0.11505 -0.2202
+-0.0504 0.11725 -0.2328
+-0.0372 0.11985 -0.2474
+-0.0255 0.12215 -0.2609
+-0.0007 0.12245 -0.2622
+0.0112 0.12465 -0.2747
+0.0233 0.12675 -0.2868
+0.0336 0.12835 -0.2961
+0.0486 0.13085 -0.31
+0.0717 0.13055 -0.3084
+0.0878 0.13005 -0.3056
+0.0841 0.13185 -0.3158
+0.1017 0.13265 -0.3203
+0.1205 0.13625 -0.3408
+0.1388 0.13435 -0.3296
+0.1535 0.13275 -0.321
+0.1803 0.13365 -0.3259
+0.1964 0.13145 -0.3134
+0.2127 0.12945 -0.3021
+0.2288 0.12765 -0.2921
+0.2414 0.13625 -0.2813
+0.2545 0.14435 -0.2715
+0.2581 0.15025 -0.2476
+0.2626 0.15625 -0.2255
+0.2763 0.16355 -0.2179
+0.2919 0.17135 -0.2082
+0.3154 0.18225 -0.1924
+0.326 0.19045 -0.1639
+0.3313 0.19525 -0.1455
+0.3337 0.19895 -0.1283
+0.3523 0.20325 -0.13
+0.3529 0.20655 -0.1126
+0.3532 0.20965 -0.095
+0.3533 0.21275 -0.0774
+0.3646 0.20735 -0.0764
+0.3792 0.20375 -0.0758
+0.3966 0.20015 -0.0752
+0.4162 0.19595 -0.0744
+0.4376 0.19075 -0.0735
+0.4603 0.18435 -0.0724
+0.4603 0.18605 -0.0507
+0.4603 0.18605 -0.0312
+0.4603 0.18405 -0.0178
+0.4603 0.19475 -0.0091
+0.4603 0.20045 0.0156
+0.4603 0.18125 0.02
+0.4603 0.16265 0.0267
+0.4603 0.13945 0.0146
+0.4603 0.11785 -0.0006
+0.4603 0.09955 0.0026
+0.4603 0.08125 0.0058
+0.4603 0.06295 0.009
+0.4457 0.06605 0.0285
+0.4316 0.06845 0.048
+0.4176 0.07035 0.0677
+0.4033 0.06805 0.0681
+0.3891 0.06855 0.0879
+0.389 0.07195 0.1073
+0.3889 0.07535 0.1267
+0.3887 0.07865 0.1461
+0.3885 0.08205 0.1654
+0.3841 0.09965 0.1623
+0.3796 0.11735 0.1592
+0.364 0.11325 0.1599
+0.3485 0.10855 0.1607
+0.3333 0.10305 0.1617
+0.3182 0.09695 0.1628
+0.3036 0.09005 0.164
+0.2893 0.08265 0.1653
+0.2753 0.07445 0.1668
+0.2618 0.06575 0.1683
+0.2487 0.05645 0.1699
+0.236 0.04645 0.1717
+0.2238 0.03595 0.1735
+0.2122 0.02495 0.1755
+0.2011 0.01345 0.1775
+0.1905 0.00135 0.1797
+0.1805 -0.01115 0.1819
+0.1711 -0.02415 0.1841
+0.1623 -0.03745 0.1865
+0.1541 -0.05125 0.1889
+0.1466 -0.06525 0.1914
+0.1568 -0.08675 0.1952
+0.1683 -0.10675 0.1987
+0.1634 -0.12015 0.2011
+0.1592 -0.13375 0.2035
+0.159 -0.13785 0.1842
+0.1554 -0.15165 0.1867
+0.1553 -0.15525 0.1674
+0.1553 -0.15893 0.148
+0.1552 -0.16247 0.1287
+0.1524 -0.17956 0.1117
+0.1502 -0.19624 0.0947
+0.1487 -0.2129 0.0777
+0.1476 -0.23077 0.0608
+0.1287 -0.24528 0.0634
+0.1101 -0.242 0.082
+0.0915 -0.24203 0.0819
+0.0729 -0.24202 0.082
+0.0543 -0.24197 0.0822
+0.035 -0.24528 0.0634
+0.0353 -0.248714 0.044
+0.0359 -0.2523 0.0236
+0.0176 -0.252361 0.0233
+0.0176 -0.25555 0.0052
+-0.0008 -0.25555 0.0052
+-0.0156 -0.2371 0.002
+-0.0294 -0.21871 -0.0011
+-0.0427 -0.2004 -0.004
+-0.0557 -0.18217 -0.0068
+-0.0685 -0.16407 -0.0095
+-0.0814 -0.14615 -0.012
+-0.0944 -0.12835 -0.0144
+-0.1076 -0.11085 -0.0166
+-0.1208 -0.09345 -0.0187
+-0.1342 -0.07635 -0.0208
+-0.1478 -0.05935 -0.0226
+-0.1618 -0.04245 -0.0244
+-0.1761 -0.02565 -0.0261
+-0.1761 -0.00695 -0.0291
+-0.1761 0.01105 -0.0346
+-0.1761 0.02845 -0.0425
+-0.18 0.04425 -0.0524
+-0.1878 0.05815 -0.0638
+-0.1965 0.06915 -0.0753
+-0.1807 0.07605 -0.0839
+-0.165 0.08245 -0.0934
+-0.149 0.08705 -0.1014
+-0.1368 0.09005 -0.1074
+-0.1317 0.09675 -0.1244
+-0.1195 0.10145 -0.1436
+-0.1247 0.10365 -0.156
+-0.1204 0.10765 -0.1784
+-0.1087 0.10985 -0.1909
+-0.0974 0.11215 -0.2038
+-0.083 0.11045 -0.1945
+-0.0725 0.11265 -0.2068
+-0.0607 0.11505 -0.2202
+-0.0742 0.11695 -0.2313
+-0.0626 0.11935 -0.2451
+-0.051 0.12185 -0.2588
+-0.0392 0.12425 -0.2725
+-0.0271 0.12665 -0.286
+-0.0137 0.12445 -0.274
+-0.0015 0.12675 -0.2868
+0.011 0.12895 -0.2992
+0.0238 0.13105 -0.3113
+0.037 0.13315 -0.323
+0.0505 0.13515 -0.3344
+0.0618 0.13285 -0.3211
+0.0751 0.13465 -0.3318
+0.0898 0.13645 -0.3417
+0.106 0.13865 -0.3542
+0.1201 0.14045 -0.3643
+0.1308 0.13865 -0.354
+0.1468 0.13705 -0.3452
+0.1649 0.13545 -0.3361
+0.1769 0.13815 -0.3512
+0.1917 0.13665 -0.3428
+0.2072 0.13435 -0.3299
+0.2233 0.13225 -0.318
+0.2391 0.13045 -0.3076
+0.2516 0.13855 -0.2964
+0.2645 0.14635 -0.2862
+0.2677 0.15195 -0.2623
+0.2718 0.15775 -0.2399
+0.2861 0.16475 -0.2332
+0.3021 0.17215 -0.2245
+0.3215 0.17985 -0.2163
+0.3443 0.19045 -0.192
+0.3479 0.19555 -0.1679
+0.3506 0.19975 -0.1481
+0.3672 0.20285 -0.1505
+0.3695 0.20675 -0.131
+0.3703 0.21005 -0.1132
+0.3707 0.21325 -0.0957
+0.3708 0.21635 -0.078
+0.374 0.21165 -0.0772
+0.3861 0.21125 -0.0771
+0.4025 0.21055 -0.077
+0.4208 0.20925 -0.0768
+0.4403 0.20715 -0.0764
+0.4603 0.20445 -0.0759
+0.4603 0.20295 -0.0523
+0.4603 0.20065 -0.0309
+0.4603 0.21945 -0.0089
+0.4603 0.21865 0.013
+0.4603 0.21825 0.0303
+0.4603 0.20205 0.0327
+0.4603 0.18555 0.0343
+0.4603 0.16895 0.0429
+0.4603 0.14275 0.0331
+0.4603 0.12115 0.0179
+0.4603 0.10285 0.0213
+0.4603 0.08465 0.0248
+0.4603 0.06635 0.0284
+0.4459 0.06945 0.0478
+0.4318 0.07195 0.0674
+0.4318 0.07535 0.0868
+0.4176 0.07375 0.087
+0.4033 0.07145 0.0874
+0.4032 0.07485 0.1068
+0.403 0.07825 0.1262
+0.4029 0.08165 0.1455
+0.4027 0.08505 0.1649
+0.399 0.10285 0.1618
+0.3955 0.12065 0.1586
+0.3921 0.13845 0.1555
+0.3754 0.13495 0.1561
+0.3588 0.13075 0.1568
+0.3425 0.12575 0.1577
+0.3265 0.11995 0.1587
+0.3107 0.11345 0.1599
+0.2953 0.10625 0.1611
+0.2802 0.09845 0.1625
+0.2655 0.08985 0.164
+0.2513 0.08065 0.1657
+0.2375 0.07085 0.1674
+0.2241 0.06035 0.1693
+0.2113 0.04925 0.1712
+0.199 0.03765 0.1733
+0.1873 0.02545 0.1754
+0.1762 0.01285 0.1776
+0.1656 -0.00045 0.18
+0.1557 -0.01405 0.1824
+0.1465 -0.02805 0.1849
+0.1379 -0.04255 0.1874
+0.13 -0.05745 0.19
+0.1398 -0.07975 0.1939
+0.151 -0.10065 0.1976
+0.1458 -0.11475 0.2001
+0.1413 -0.12915 0.2027
+0.1555 -0.14765 0.2059
+0.1526 -0.16157 0.2084
+0.1524 -0.16558 0.1891
+0.1524 -0.16918 0.1698
+0.1524 -0.1728 0.1504
+0.1523 -0.17633 0.1311
+0.1501 -0.19345 0.1142
+0.1486 -0.21023 0.0972
+0.1476 -0.22728 0.0802
+0.1473 -0.24188 0.0828
+0.1473 -0.24528 0.0634
+0.1283 -0.24188 0.0828
+0.128 -0.23846 0.1021
+0.1092 -0.23849 0.1019
+0.0915 -0.23869 0.1008
+0.0731 -0.23867 0.1009
+0.0546 -0.23857 0.1015
+0.0349 -0.24191 0.0826
+0.0153 -0.24198 0.0822
+0.0164 -0.245861 0.0601
+0.0174 -0.24908 0.0419
+-0.0008 -0.249172 0.0414
+-0.0008 -0.252361 0.0233
+-0.0192 -0.252361 0.0233
+-0.0192 -0.25555 0.0052
+-0.0335 -0.23713 0.002
+-0.0471 -0.21875 -0.0011
+-0.0602 -0.20043 -0.004
+-0.073 -0.18222 -0.0068
+-0.0858 -0.1641 -0.0095
+-0.0988 -0.14615 -0.012
+-0.1119 -0.12845 -0.0144
+-0.1252 -0.11085 -0.0166
+-0.1387 -0.09355 -0.0187
+-0.1523 -0.07635 -0.0208
+-0.1662 -0.05935 -0.0226
+-0.1804 -0.04245 -0.0244
+-0.1951 -0.02565 -0.0261
+-0.1951 -0.00695 -0.0291
+-0.1951 0.01115 -0.0346
+-0.1944 0.02735 -0.0419
+-0.1961 0.04025 -0.0496
+-0.2025 0.05045 -0.0571
+-0.2125 0.06185 -0.0674
+-0.2073 0.07875 -0.0877
+-0.1918 0.08455 -0.0969
+-0.1765 0.08975 -0.1069
+-0.1608 0.09325 -0.1147
+-0.1465 0.09565 -0.1212
+-0.1388 0.10145 -0.1436
+-0.1376 0.10275 -0.1509
+-0.1326 0.10555 -0.1664
+-0.1347 0.12425 -0.1652
+-0.1213 0.12685 -0.1784
+-0.1089 0.12875 -0.1915
+-0.0974 0.13005 -0.2046
+-0.0858 0.11455 -0.2176
+-0.0858 0.13215 -0.2183
+-0.0742 0.13425 -0.232
+-0.0626 0.13635 -0.2457
+-0.051 0.13855 -0.2594
+-0.0393 0.14055 -0.273
+-0.0272 0.14265 -0.2865
+-0.0145 0.12895 -0.2991
+-0.0017 0.13115 -0.3119
+0.0115 0.13335 -0.3243
+0.0251 0.13545 -0.3364
+0.039 0.13755 -0.3481
+0.0643 0.13705 -0.3454
+0.0784 0.13895 -0.356
+0.0942 0.14095 -0.3674
+0.1089 0.14265 -0.3772
+0.1224 0.14415 -0.3856
+0.1311 0.14205 -0.3734
+0.1369 0.14035 -0.3641
+0.1534 0.14035 -0.3641
+0.1749 0.14285 -0.3783
+0.1896 0.14075 -0.3664
+0.2039 0.13915 -0.3573
+0.2186 0.13725 -0.3462
+0.2342 0.13505 -0.3337
+0.2498 0.13305 -0.3227
+0.2621 0.14095 -0.3112
+0.2749 0.14835 -0.3006
+0.2776 0.15365 -0.2768
+0.2813 0.15915 -0.2542
+0.2955 0.16585 -0.2469
+0.3091 0.17175 -0.2405
+0.3237 0.17715 -0.2359
+0.3432 0.18285 -0.2333
+0.3441 0.18655 -0.2136
+0.3634 0.19135 -0.2115
+0.3637 0.19485 -0.1921
+0.3643 0.19835 -0.1727
+0.3838 0.20525 -0.154
+0.3869 0.20955 -0.1323
+0.3878 0.21295 -0.114
+0.3883 0.21615 -0.0963
+0.3886 0.21925 -0.0785
+0.4064 0.22155 -0.0789
+0.4243 0.22325 -0.0792
+0.4422 0.22415 -0.0794
+0.4603 0.22455 -0.0794
+0.4603 0.22105 -0.0538
+0.4603 0.22075 -0.0307
+0.4603 0.23675 -0.0114
+0.4603 0.23755 0.0093
+0.4603 0.23715 0.028
+0.4603 0.23675 0.045
+0.4603 0.21785 0.045
+0.4603 0.20225 0.046
+0.4603 0.18955 0.0453
+0.4603 0.17945 0.0596
+0.4603 0.15035 0.0712
+0.4603 0.14635 0.0519
+0.4603 0.12455 0.0366
+0.4603 0.10655 0.0404
+0.4603 0.08895 0.0444
+0.4603 0.06975 0.0478
+0.446 0.07285 0.0672
+0.446 0.07625 0.0866
+0.4459 0.07975 0.1059
+0.4317 0.07875 0.1061
+0.4174 0.07715 0.1064
+0.4173 0.08055 0.1258
+0.4171 0.08395 0.1451
+0.417 0.08735 0.1645
+0.4144 0.10525 0.1613
+0.4118 0.12325 0.1582
+0.4092 0.14125 0.155
+0.3888 0.15635 0.1523
+0.3712 0.15265 0.153
+0.3538 0.14815 0.1538
+0.3367 0.14295 0.1547
+0.3199 0.13695 0.1558
+0.3033 0.13015 0.157
+0.2871 0.12255 0.1583
+0.2712 0.11425 0.1597
+0.2558 0.10535 0.1613
+0.2409 0.09565 0.163
+0.2263 0.08535 0.1649
+0.2123 0.07435 0.1668
+0.1989 0.06265 0.1688
+0.186 0.05045 0.171
+0.1736 0.03775 0.1732
+0.1619 0.02435 0.1756
+0.1509 0.01055 0.1781
+0.1405 -0.00385 0.1806
+0.1307 -0.01865 0.1832
+0.1218 -0.03385 0.1859
+0.1135 -0.04945 0.1886
+0.1228 -0.07265 0.1927
+0.1337 -0.09435 0.1965
+0.1282 -0.10935 0.1992
+0.1235 -0.12455 0.2018
+0.1375 -0.14375 0.2052
+0.1344 -0.15855 0.2079
+0.1502 -0.17561 0.2109
+0.1502 -0.17958 0.1916
+0.1501 -0.18313 0.1723
+0.1501 -0.1867 0.1529
+0.1501 -0.1902 0.1336
+0.1485 -0.2073 0.1166
+0.1476 -0.22428 0.0996
+0.1473 -0.23846 0.1021
+0.1473 -0.23504 0.1215
+0.1278 -0.23504 0.1215
+0.1086 -0.23506 0.1214
+0.0901 -0.23513 0.121
+0.0735 -0.23529 0.1201
+0.0551 -0.23521 0.1206
+0.0352 -0.23523 0.1204
+0.0349 -0.23851 0.1018
+0.0146 -0.23841 0.1024
+-0.0037 -0.23916 0.0981
+-0.0014 -0.24264 0.0784
+-0.0011 -0.245983 0.0595
+-0.0192 -0.249172 0.0414
+-0.0376 -0.249172 0.0414
+-0.0376 -0.252361 0.0233
+-0.0376 -0.25555 0.0052
+-0.0515 -0.23715 0.002
+-0.0648 -0.21878 -0.001
+-0.0777 -0.20046 -0.004
+-0.0904 -0.18225 -0.0068
+-0.1031 -0.16415 -0.0095
+-0.1161 -0.14625 -0.012
+-0.1294 -0.12845 -0.0144
+-0.1429 -0.11085 -0.0166
+-0.1566 -0.09355 -0.0187
+-0.1705 -0.07635 -0.0207
+-0.1846 -0.05935 -0.0226
+-0.1991 -0.04255 -0.0244
+-0.214 -0.02565 -0.0261
+-0.214 -0.00695 -0.0291
+-0.214 0.01115 -0.0346
+-0.2115 0.02715 -0.0418
+-0.2083 0.03735 -0.0477
+-0.2166 0.04435 -0.0525
+-0.2276 0.05535 -0.0613
+-0.223 0.07235 -0.0791
+-0.2332 0.08155 -0.092
+-0.2178 0.08685 -0.1011
+-0.2026 0.09155 -0.1108
+-0.1875 0.09565 -0.1211
+-0.1727 0.09895 -0.132
+-0.1582 0.10145 -0.1436
+-0.1452 0.10345 -0.1547
+-0.1473 0.12155 -0.1538
+-0.1485 0.14115 -0.1535
+-0.1354 0.14405 -0.1653
+-0.1217 0.14675 -0.1788
+-0.1093 0.14845 -0.1918
+-0.0974 0.14795 -0.2053
+-0.0858 0.14975 -0.219
+-0.0742 0.15155 -0.2326
+-0.0626 0.15345 -0.2463
+-0.051 0.15525 -0.26
+-0.0392 0.15695 -0.2737
+-0.027 0.15875 -0.2872
+-0.0146 0.14465 -0.2996
+-0.0016 0.14665 -0.3125
+0.0118 0.14855 -0.3251
+0.0255 0.15045 -0.3373
+0.0396 0.15225 -0.3491
+0.0531 0.13955 -0.3594
+0.0676 0.14145 -0.3703
+0.0824 0.14335 -0.3808
+0.0975 0.14515 -0.391
+0.1129 0.14685 -0.4007
+0.1285 0.14845 -0.41
+0.1377 0.14575 -0.3946
+0.1465 0.14315 -0.3797
+0.1639 0.14505 -0.3904
+0.1772 0.14645 -0.3984
+0.1859 0.14475 -0.3888
+0.1979 0.14335 -0.3811
+0.2151 0.14205 -0.3735
+0.2306 0.14005 -0.3622
+0.2457 0.13775 -0.3491
+0.2609 0.13575 -0.3376
+0.2731 0.14315 -0.3256
+0.2858 0.15025 -0.3148
+0.288 0.15525 -0.2909
+0.2912 0.16055 -0.2682
+0.3052 0.16685 -0.2607
+0.3161 0.17135 -0.2552
+0.3274 0.17525 -0.2525
+0.3422 0.17935 -0.2519
+0.3624 0.18425 -0.2508
+0.3629 0.18785 -0.231
+0.3828 0.19155 -0.2307
+0.383 0.19495 -0.2116
+0.3831 0.19835 -0.1926
+0.3832 0.20175 -0.1734
+0.4024 0.20435 -0.1741
+0.4027 0.20775 -0.1549
+0.4043 0.21155 -0.1341
+0.4054 0.21515 -0.1149
+0.406 0.21835 -0.0969
+0.4237 0.21995 -0.0976
+0.4412 0.22075 -0.0984
+0.4603 0.22115 -0.0985
+0.4603 0.24085 -0.083
+0.4603 0.23945 -0.0729
+0.4603 0.23725 -0.0507
+0.4603 0.23675 -0.0302
+0.4603 0.25555 -0.0114
+0.4603 0.25555 0.0074
+0.4603 0.25555 0.0262
+0.4603 0.25555 0.045
+0.4603 0.25555 0.0638
+0.4603 0.23675 0.0638
+0.4603 0.21785 0.0638
+0.4603 0.19995 0.0642
+0.4603 0.19815 0.0871
+0.4603 0.17695 0.0908
+0.4603 0.15325 0.0952
+0.4603 0.13085 0.0767
+0.4603 0.12875 0.0559
+0.4603 0.11125 0.0603
+0.4603 0.09255 0.0637
+0.4603 0.07315 0.0671
+0.4603 0.07665 0.0865
+0.4603 0.08005 0.1059
+0.4603 0.08345 0.1252
+0.4459 0.08315 0.1253
+0.4316 0.08215 0.1255
+0.4315 0.08555 0.1448
+0.4314 0.08895 0.1642
+0.4296 0.10705 0.161
+0.4279 0.12515 0.1578
+0.4262 0.14315 0.1546
+0.4067 0.15925 0.1518
+0.3858 0.17425 0.1491
+0.3675 0.17045 0.1498
+0.3494 0.16585 0.1506
+0.3315 0.16035 0.1516
+0.3139 0.15415 0.1527
+0.2966 0.14705 0.154
+0.2797 0.13925 0.1553
+0.2631 0.13065 0.1569
+0.247 0.12125 0.1585
+0.2313 0.11125 0.1603
+0.2161 0.10045 0.1622
+0.2014 0.08905 0.1642
+0.1873 0.07695 0.1663
+0.1738 0.06415 0.1686
+0.1608 0.05085 0.1709
+0.1485 0.03695 0.1734
+0.1369 0.02255 0.1759
+0.126 0.00755 0.1786
+0.1158 -0.00795 0.1813
+0.1063 -0.02375 0.1841
+0.0976 -0.04015 0.187
+0.0896 -0.05675 0.1899
+0.1059 -0.06545 0.1914
+0.1164 -0.08805 0.1954
+0.1107 -0.10385 0.1982
+0.1057 -0.11985 0.201
+0.1195 -0.13995 0.2046
+0.1162 -0.15545 0.2073
+0.1319 -0.17336 0.2105
+0.1486 -0.18975 0.2134
+0.1486 -0.1936 0.1941
+0.1485 -0.19708 0.1747
+0.1485 -0.20059 0.1554
+0.1485 -0.20404 0.136
+0.1476 -0.22114 0.119
+0.1476 -0.21783 0.1384
+0.1473 -0.23162 0.1409
+0.1277 -0.23162 0.1409
+0.1083 -0.23164 0.1408
+0.0896 -0.23165 0.1407
+0.0721 -0.23127 0.1429
+0.0544 -0.23101 0.1444
+0.0353 -0.23143 0.142
+0.0132 -0.23555 0.1186
+-0.0049 -0.23617 0.1151
+-0.0216 -0.23632 0.1142
+-0.0196 -0.23959 0.0957
+-0.0192 -0.24278 0.0776
+-0.0192 -0.245983 0.0595
+-0.0376 -0.245983 0.0595
+-0.056 -0.245983 0.0595
+-0.056 -0.249172 0.0414
+-0.056 -0.252361 0.0233
+-0.056 -0.25555 0.0052
+-0.0695 -0.23716 0.002
+-0.0825 -0.21881 -0.001
+-0.0952 -0.2005 -0.004
+-0.1077 -0.18228 -0.0068
+-0.1205 -0.16418 -0.0095
+-0.1335 -0.14625 -0.012
+-0.1469 -0.12845 -0.0144
+-0.1605 -0.11095 -0.0166
+-0.1744 -0.09355 -0.0187
+-0.1886 -0.07645 -0.0207
+-0.203 -0.05945 -0.0226
+-0.2177 -0.04255 -0.0244
+-0.233 -0.02565 -0.0261
+-0.233 -0.00695 -0.0291
+-0.233 0.01115 -0.0346
+-0.2302 0.03265 -0.0448
+-0.2448 0.04825 -0.0554
+-0.2388 0.06565 -0.0713
+-0.2488 0.07585 -0.0836
+-0.2547 0.09365 -0.0817
+-0.2386 0.09955 -0.0902
+-0.2226 0.10495 -0.0993
+-0.2067 0.10975 -0.1091
+-0.1909 0.11375 -0.1196
+-0.1754 0.11685 -0.1309
+-0.1603 0.11905 -0.1427
+-0.1628 0.13745 -0.1415
+-0.1639 0.15985 -0.1414
+-0.1496 0.16245 -0.1533
+-0.1353 0.16455 -0.1662
+-0.1217 0.16655 -0.1794
+-0.1094 0.16625 -0.1923
+-0.0974 0.16585 -0.2059
+-0.0858 0.16735 -0.2195
+-0.0742 0.16885 -0.2332
+-0.0626 0.17045 -0.2468
+-0.051 0.17195 -0.2605
+-0.0391 0.17335 -0.2743
+-0.0268 0.17485 -0.2879
+-0.0143 0.16045 -0.3005
+-0.0012 0.16215 -0.3134
+0.0123 0.16375 -0.3261
+0.0263 0.16545 -0.3384
+0.0405 0.16695 -0.3503
+0.054 0.15395 -0.3605
+0.0688 0.15565 -0.3716
+0.0838 0.15725 -0.3822
+0.099 0.15885 -0.3923
+0.1145 0.16035 -0.4021
+0.1302 0.16175 -0.4113
+0.1444 0.15005 -0.4188
+0.153 0.14725 -0.4033
+0.1687 0.14875 -0.4115
+0.1893 0.14775 -0.4056
+0.1927 0.14615 -0.397
+0.2031 0.14595 -0.3957
+0.2251 0.14565 -0.3938
+0.2434 0.14275 -0.3776
+0.2578 0.14045 -0.3642
+0.2725 0.13825 -0.352
+0.2845 0.14535 -0.3397
+0.297 0.15195 -0.3285
+0.2987 0.15675 -0.3047
+0.3014 0.16175 -0.2819
+0.3129 0.16745 -0.272
+0.3192 0.17065 -0.2649
+0.3281 0.17285 -0.2672
+0.3424 0.17625 -0.2702
+0.3619 0.18055 -0.2709
+0.3823 0.18465 -0.2694
+0.3826 0.18815 -0.25
+0.4022 0.19085 -0.2502
+0.4023 0.19425 -0.2312
+0.4023 0.19755 -0.2122
+0.4024 0.20095 -0.1931
+0.4216 0.20285 -0.1934
+0.4216 0.20625 -0.1744
+0.4216 0.20955 -0.1554
+0.4219 0.21295 -0.1363
+0.423 0.21655 -0.1161
+0.4409 0.21745 -0.1175
+0.4603 0.21775 -0.1175
+0.4603 0.25555 -0.1054
+0.4603 0.25555 -0.0866
+0.4603 0.25555 -0.0678
+0.4603 0.25555 -0.049
+0.4603 0.25555 -0.0302
+0.4415 0.25555 -0.0114
+0.4415 0.25555 0.0074
+0.4415 0.25555 0.0262
+0.4415 0.25555 0.045
+0.4415 0.25555 0.0638
+0.4415 0.25555 0.0826
+0.4603 0.25555 0.0826
+0.4603 0.23675 0.0826
+0.4603 0.21695 0.0849
+0.4603 0.21585 0.1049
+0.4603 0.19775 0.1088
+0.4603 0.17645 0.1129
+0.4603 0.15665 0.1152
+0.4603 0.13515 0.099
+0.4603 0.11465 0.0798
+0.4603 0.09595 0.0831
+0.4603 0.09935 0.1025
+0.4603 0.10245 0.1239
+0.4603 0.08685 0.1446
+0.4458 0.08655 0.1447
+0.4458 0.08995 0.164
+0.4449 0.10805 0.1608
+0.4441 0.12625 0.1576
+0.4432 0.14435 0.1544
+0.4245 0.16125 0.1515
+0.4043 0.17725 0.1486
+0.4043 0.18035 0.1665
+0.3858 0.17745 0.167
+0.3675 0.17365 0.1677
+0.3494 0.16895 0.1685
+0.3315 0.16355 0.1694
+0.3139 0.15725 0.1706
+0.2966 0.15025 0.1718
+0.2797 0.14235 0.1732
+0.2631 0.13375 0.1747
+0.247 0.12445 0.1763
+0.2313 0.11435 0.1781
+0.2161 0.10365 0.18
+0.2014 0.09215 0.182
+0.1873 0.08005 0.1842
+0.1738 0.06735 0.1864
+0.1608 0.05395 0.1888
+0.1485 0.04005 0.1912
+0.1369 0.02565 0.1938
+0.126 0.01065 0.1964
+0.1158 -0.00475 0.1991
+0.1063 -0.02065 0.2019
+0.0976 -0.03695 0.2048
+0.0896 -0.05365 0.2077
+0.0824 -0.07385 0.1929
+0.0991 -0.08165 0.1943
+0.0931 -0.09825 0.1972
+0.0879 -0.11505 0.2002
+0.1015 -0.13605 0.2039
+0.098 -0.15245 0.2068
+0.1136 -0.17115 0.2101
+0.1302 -0.18835 0.2131
+0.1476 -0.20397 0.2159
+0.1476 -0.20762 0.1965
+0.1476 -0.21098 0.1772
+0.1476 -0.21441 0.1578
+0.1473 -0.22821 0.1602
+0.1277 -0.22821 0.1602
+0.1085 -0.22819 0.1603
+0.091 -0.22793 0.1618
+0.0731 -0.22775 0.1628
+0.0549 -0.22766 0.1633
+0.0365 -0.22763 0.1636
+0.0108 -0.22776 0.1628
+0.0055 -0.23111 0.1438
+-0.0102 -0.23255 0.1356
+-0.0249 -0.23353 0.1301
+-0.0376 -0.23643 0.1137
+-0.0376 -0.23959 0.0957
+-0.0376 -0.24278 0.0776
+-0.056 -0.24278 0.0776
+-0.0743 -0.24278 0.0776
+-0.0743 -0.245983 0.0595
+-0.0743 -0.249172 0.0414
+-0.0743 -0.252361 0.0233
+-0.0743 -0.25555 0.0052
+-0.0876 -0.23718 0.002
+-0.1003 -0.21882 -0.001
+-0.1127 -0.20051 -0.004
+-0.1251 -0.18229 -0.0068
+-0.1378 -0.16421 -0.0094
+-0.1508 -0.14625 -0.012
+-0.1644 -0.12855 -0.0144
+-0.1782 -0.11095 -0.0166
+-0.1923 -0.09365 -0.0187
+-0.2067 -0.07645 -0.0207
+-0.2213 -0.05945 -0.0226
+-0.2364 -0.04255 -0.0244
+-0.2519 -0.02565 -0.0261
+-0.2519 -0.00695 -0.0291
+-0.2538 0.01075 -0.0345
+-0.2554 0.02635 -0.0414
+-0.2628 0.03955 -0.0491
+-0.2549 0.05845 -0.0641
+-0.2645 0.06965 -0.0758
+-0.2708 0.08725 -0.0741
+-0.2773 0.10545 -0.0723
+-0.2607 0.11205 -0.0798
+-0.2441 0.11815 -0.0882
+-0.2274 0.12375 -0.0974
+-0.2108 0.12865 -0.1073
+-0.1944 0.13435 -0.1181
+-0.1778 0.13525 -0.1299
+-0.1778 0.15755 -0.1306
+-0.1777 0.18185 -0.1314
+-0.1633 0.18255 -0.1425
+-0.1485 0.18405 -0.1548
+-0.1349 0.18415 -0.167
+-0.1219 0.18425 -0.1797
+-0.1094 0.18415 -0.1927
+-0.0974 0.18385 -0.2063
+-0.0858 0.18505 -0.22
+-0.0742 0.18625 -0.2336
+-0.0626 0.18745 -0.2473
+-0.051 0.18865 -0.2609
+-0.039 0.18975 -0.2748
+-0.0266 0.19095 -0.2885
+-0.014 0.17625 -0.3012
+-0.0007 0.17765 -0.3142
+0.0129 0.17905 -0.327
+0.027 0.18035 -0.3394
+0.0414 0.18165 -0.3514
+0.0552 0.16845 -0.3618
+0.0701 0.16985 -0.3729
+0.0852 0.17125 -0.3836
+0.1006 0.17255 -0.3937
+0.1161 0.17385 -0.4034
+0.1319 0.17505 -0.4127
+0.1461 0.16315 -0.4201
+0.1605 0.15155 -0.4273
+0.1769 0.15295 -0.4353
+0.1847 0.15015 -0.4192
+0.2011 0.14845 -0.4099
+0.2212 0.14965 -0.4168
+0.2477 0.14795 -0.4071
+0.2597 0.14535 -0.3921
+0.2705 0.14305 -0.379
+0.2847 0.14075 -0.3661
+0.2963 0.14735 -0.3532
+0.3085 0.15365 -0.3418
+0.3097 0.15825 -0.3181
+0.3105 0.16285 -0.2937
+0.3223 0.16885 -0.2806
+0.3389 0.17135 -0.2924
+0.3618 0.17695 -0.2914
+0.3821 0.18115 -0.2891
+0.402 0.18405 -0.2886
+0.4021 0.18745 -0.2693
+0.4216 0.18945 -0.2696
+0.4216 0.19275 -0.2506
+0.4216 0.19615 -0.2315
+0.4216 0.19945 -0.2125
+0.4409 0.20395 -0.1936
+0.4409 0.20735 -0.1746
+0.4409 0.21075 -0.1556
+0.4409 0.21405 -0.1365
+0.4603 0.21445 -0.1366
+0.4603 0.23995 -0.144
+0.4603 0.25555 -0.1241
+0.4415 0.25555 -0.1054
+0.4415 0.25555 -0.0866
+0.4415 0.25555 -0.0678
+0.4415 0.25555 -0.049
+0.4415 0.25555 -0.0302
+0.4227 0.25555 -0.0114
+0.4227 0.25555 0.0074
+0.4227 0.25555 0.0262
+0.4227 0.25555 0.045
+0.4227 0.25555 0.0638
+0.4227 0.25555 0.0826
+0.4227 0.25555 0.1014
+0.4415 0.25555 0.1014
+0.4603 0.25555 0.1014
+0.4603 0.23675 0.1014
+0.4603 0.23675 0.1202
+0.4603 0.21675 0.1234
+0.4603 0.19765 0.1271
+0.4603 0.17785 0.13
+0.4603 0.15975 0.1331
+0.4603 0.14155 0.1363
+0.4603 0.13845 0.1183
+0.4603 0.11725 0.1019
+0.4603 0.12035 0.1214
+0.4603 0.10515 0.1424
+0.4603 0.09025 0.164
+0.4603 0.10845 0.1608
+0.4603 0.12655 0.1576
+0.4603 0.14475 0.1544
+0.4423 0.16255 0.1512
+0.4229 0.17935 0.1483
+0.4229 0.18255 0.1661
+0.4229 0.18565 0.1839
+0.4043 0.18355 0.1843
+0.3858 0.18055 0.1848
+0.3675 0.17675 0.1855
+0.3494 0.17215 0.1863
+0.3315 0.16665 0.1873
+0.3139 0.16045 0.1884
+0.2966 0.15335 0.1896
+0.2797 0.14555 0.191
+0.2631 0.13695 0.1925
+0.247 0.12755 0.1942
+0.2313 0.11755 0.1959
+0.2161 0.10675 0.1979
+0.2014 0.09535 0.1999
+0.1873 0.08325 0.202
+0.1738 0.07045 0.2043
+0.1608 0.05715 0.2066
+0.1485 0.04325 0.209
+0.1369 0.02875 0.2116
+0.126 0.01385 0.2142
+0.1158 -0.00165 0.217
+0.1063 -0.01755 0.2198
+0.0976 -0.03385 0.2226
+0.0896 -0.05045 0.2256
+0.0824 -0.07065 0.2108
+0.0761 -0.09115 0.196
+0.0705 -0.10875 0.1991
+0.0658 -0.12665 0.2022
+0.0835 -0.13215 0.2032
+0.0798 -0.14935 0.2062
+0.0953 -0.16894 0.2097
+0.1118 -0.18698 0.2129
+0.1292 -0.20341 0.2158
+0.1473 -0.21797 0.2183
+0.1473 -0.22139 0.199
+0.1473 -0.2248 0.1796
+0.1279 -0.2248 0.1796
+0.1101 -0.22462 0.1806
+0.0918 -0.22451 0.1812
+0.0734 -0.22445 0.1816
+0.055 -0.22441 0.1818
+0.0365 -0.22441 0.1819
+0.0147 -0.22441 0.1818
+-0.0005 -0.22438 0.182
+-0.0029 -0.22776 0.1628
+-0.0043 -0.22972 0.1517
+-0.0137 -0.23036 0.1481
+-0.0268 -0.23117 0.1435
+-0.0392 -0.23399 0.1275
+-0.056 -0.23645 0.1135
+-0.056 -0.23959 0.0957
+-0.0743 -0.2396 0.0957
+-0.0927 -0.2396 0.0956
+-0.0927 -0.24278 0.0776
+-0.0927 -0.245983 0.0595
+-0.0927 -0.249172 0.0414
+-0.0927 -0.252361 0.0233
+-0.0927 -0.25555 0.0052
+-0.1057 -0.23718 0.002
+-0.1181 -0.21884 -0.001
+-0.1302 -0.20054 -0.004
+-0.1425 -0.18232 -0.0068
+-0.1551 -0.16424 -0.0094
+-0.1682 -0.14635 -0.012
+-0.1818 -0.12855 -0.0144
+-0.1958 -0.11105 -0.0166
+-0.2102 -0.09365 -0.0187
+-0.2248 -0.07645 -0.0207
+-0.2397 -0.05945 -0.0226
+-0.2551 -0.04255 -0.0244
+-0.2708 -0.02565 -0.0261
+-0.2708 -0.00695 -0.0291
+-0.2695 0.01075 -0.0345
+-0.2717 0.02205 -0.0392
+-0.2784 0.03305 -0.0451
+-0.271 0.05105 -0.0576
+-0.2803 0.06305 -0.0686
+-0.287 0.08055 -0.0669
+-0.2939 0.09845 -0.0653
+-0.3011 0.11695 -0.0637
+-0.2841 0.12425 -0.0704
+-0.2668 0.13145 -0.078
+-0.2493 0.13795 -0.0865
+-0.2307 0.14465 -0.0964
+-0.2148 0.15245 -0.1058
+-0.1991 0.16205 -0.116
+-0.1942 0.18245 -0.1198
+-0.1911 0.20185 -0.1223
+-0.1765 0.20185 -0.1327
+-0.1624 0.20195 -0.1436
+-0.1487 0.20195 -0.155
+-0.1354 0.20185 -0.1669
+-0.1222 0.20185 -0.1797
+-0.1096 0.20195 -0.1929
+-0.0974 0.20175 -0.2067
+-0.0858 0.20265 -0.2203
+-0.0742 0.20355 -0.2339
+-0.0626 0.20445 -0.2476
+-0.051 0.20535 -0.2612
+-0.0389 0.20625 -0.2752
+-0.0264 0.20715 -0.2889
+-0.0137 0.19215 -0.3018
+-0.0003 0.19325 -0.315
+0.0135 0.19435 -0.3278
+0.0277 0.19535 -0.3402
+0.0422 0.19645 -0.3523
+0.0562 0.18295 -0.363
+0.0713 0.18405 -0.3742
+0.0866 0.18525 -0.3848
+0.1021 0.18635 -0.395
+0.1177 0.18745 -0.4047
+0.1335 0.18845 -0.4138
+0.1478 0.17625 -0.4214
+0.1623 0.16445 -0.4285
+0.1787 0.16565 -0.4365
+0.1934 0.15425 -0.4428
+0.1979 0.15115 -0.425
+0.2171 0.15265 -0.4335
+0.2399 0.15095 -0.4238
+0.2643 0.14995 -0.4185
+0.2733 0.14795 -0.4068
+0.2839 0.14555 -0.3931
+0.2974 0.14315 -0.3797
+0.3086 0.14935 -0.3663
+0.3206 0.15525 -0.3545
+0.3212 0.15955 -0.3311
+0.323 0.16405 -0.3084
+0.348 0.16945 -0.316
+0.3645 0.17375 -0.3126
+0.3829 0.17745 -0.3107
+0.4019 0.18065 -0.308
+0.4216 0.18275 -0.3077
+0.4216 0.18605 -0.2886
+0.4409 0.18715 -0.2889
+0.4409 0.19055 -0.2698
+0.4409 0.19395 -0.2508
+0.4409 0.19725 -0.2317
+0.4409 0.20065 -0.2127
+0.4603 0.20435 -0.1937
+0.4603 0.20775 -0.1747
+0.4603 0.21105 -0.1556
+0.4603 0.22755 -0.1483
+0.4603 0.23915 -0.1606
+0.4603 0.25555 -0.1617
+0.4603 0.25555 -0.1429
+0.4415 0.25555 -0.1241
+0.4227 0.25555 -0.1054
+0.4227 0.25555 -0.0866
+0.4227 0.25555 -0.0678
+0.4227 0.25555 -0.049
+0.4227 0.25555 -0.0302
+0.4039 0.25555 -0.0114
+0.4039 0.25555 0.0074
+0.4039 0.25555 0.0262
+0.4039 0.25555 0.045
+0.4039 0.25555 0.0638
+0.4039 0.25555 0.0826
+0.4039 0.25555 0.1014
+0.4039 0.25555 0.1202
+0.4227 0.25555 0.1202
+0.4415 0.25555 0.1202
+0.4603 0.25555 0.1202
+0.4603 0.25555 0.139
+0.4603 0.23675 0.139
+0.4603 0.21785 0.142
+0.4603 0.19895 0.1448
+0.4603 0.18105 0.148
+0.4603 0.16285 0.1512
+0.4416 0.18065 0.148
+0.4416 0.18375 0.1659
+0.4416 0.18695 0.1837
+0.4416 0.19005 0.2015
+0.4229 0.18875 0.2018
+0.4043 0.18665 0.2021
+0.3858 0.18365 0.2027
+0.3675 0.17985 0.2033
+0.3494 0.17525 0.2042
+0.3315 0.16985 0.2051
+0.3139 0.16355 0.2062
+0.2966 0.15655 0.2075
+0.2797 0.14865 0.2089
+0.2631 0.14005 0.2104
+0.247 0.13075 0.212
+0.2313 0.12065 0.2138
+0.2161 0.10995 0.2157
+0.2014 0.09845 0.2177
+0.1873 0.08635 0.2198
+0.1738 0.07365 0.2221
+0.1608 0.06025 0.2244
+0.1485 0.04635 0.2269
+0.1369 0.03195 0.2294
+0.126 0.01695 0.2321
+0.1158 0.00155 0.2348
+0.1063 -0.01435 0.2376
+0.0976 -0.03065 0.2405
+0.0896 -0.04735 0.2434
+0.0824 -0.06755 0.2286
+0.0761 -0.08805 0.2138
+0.0705 -0.10565 0.2169
+0.0658 -0.12345 0.22
+0.062 -0.14465 0.2054
+0.059 -0.16284 0.2086
+0.077 -0.16673 0.2093
+0.0934 -0.1856 0.2126
+0.1107 -0.20286 0.2157
+0.1288 -0.21797 0.2183
+0.1288 -0.22131 0.1994
+0.1104 -0.22125 0.1997
+0.092 -0.22122 0.1999
+0.0735 -0.2212 0.2
+0.0551 -0.22119 0.2001
+0.0366 -0.22119 0.2001
+0.0181 -0.22117 0.2002
+-0.0005 -0.22116 0.2003
+-0.0191 -0.22438 0.182
+-0.0161 -0.22795 0.1617
+-0.0316 -0.22828 0.1598
+-0.0428 -0.23139 0.1422
+-0.0557 -0.23405 0.1271
+-0.0736 -0.23437 0.1253
+-0.0744 -0.2368 0.1116
+-0.0926 -0.23706 0.1101
+-0.1117 -0.23729 0.1088
+-0.1111 -0.2396 0.0956
+-0.1111 -0.24278 0.0776
+-0.1111 -0.245983 0.0595
+-0.1111 -0.249172 0.0414
+-0.1111 -0.252361 0.0233
+-0.1111 -0.25555 0.0052
+-0.1237 -0.23719 0.002
+-0.1358 -0.21885 -0.001
+-0.1477 -0.20056 -0.004
+-0.1599 -0.18235 -0.0068
+-0.1724 -0.16427 -0.0094
+-0.1856 -0.14635 -0.0119
+-0.1993 -0.12855 -0.0144
+-0.2135 -0.11105 -0.0166
+-0.228 -0.09365 -0.0187
+-0.2429 -0.07655 -0.0207
+-0.2581 -0.05945 -0.0226
+-0.2737 -0.04255 -0.0244
+-0.2898 -0.02565 -0.0261
+-0.2869 -0.00705 -0.0291
+-0.2845 0.00935 -0.0339
+-0.2819 0.01915 -0.0379
+-0.2904 0.02615 -0.0413
+-0.2872 0.04335 -0.0517
+-0.2963 0.05615 -0.0619
+-0.3034 0.07345 -0.0605
+-0.3107 0.09115 -0.059
+-0.3165 0.10875 -0.0581
+-0.3242 0.12635 -0.0565
+-0.307 0.13775 -0.0625
+-0.2916 0.14755 -0.0684
+-0.2745 0.15875 -0.0756
+-0.252 0.15885 -0.0859
+-0.2338 0.16115 -0.0953
+-0.2177 0.16705 -0.1046
+-0.2074 0.17045 -0.1109
+-0.2068 0.18275 -0.1116
+-0.206 0.20185 -0.1125
+-0.2062 0.21975 -0.1127
+-0.1912 0.21975 -0.1225
+-0.1767 0.21975 -0.1329
+-0.1626 0.21975 -0.1437
+-0.1488 0.21975 -0.1552
+-0.1355 0.21975 -0.1671
+-0.1226 0.21975 -0.1795
+-0.1099 0.21985 -0.1928
+-0.0974 0.21965 -0.2069
+-0.0858 0.22025 -0.2205
+-0.0742 0.22085 -0.2342
+-0.0626 0.22145 -0.2478
+-0.051 0.22205 -0.2614
+-0.0388 0.22265 -0.2755
+-0.0263 0.22325 -0.2893
+-0.0134 0.20795 -0.3024
+0 0.20875 -0.3156
+0.0139 0.20965 -0.3284
+0.0281 0.21035 -0.3409
+0.0428 0.21115 -0.353
+0.0571 0.19735 -0.364
+0.0724 0.19835 -0.3752
+0.0878 0.19925 -0.3859
+0.1033 0.20015 -0.3961
+0.119 0.20105 -0.4057
+0.1349 0.20185 -0.4149
+0.1495 0.18945 -0.4226
+0.164 0.17735 -0.4297
+0.1804 0.17845 -0.4376
+0.1953 0.16685 -0.444
+0.2102 0.15555 -0.4499
+0.2272 0.15665 -0.4565
+0.2336 0.15375 -0.4399
+0.2503 0.15485 -0.4459
+0.2579 0.15205 -0.4303
+0.2736 0.15125 -0.4258
+0.2858 0.15065 -0.4224
+0.2981 0.14785 -0.4067
+0.3108 0.14545 -0.3927
+0.3215 0.15125 -0.3788
+0.3331 0.15675 -0.3666
+0.3319 0.15995 -0.3463
+0.342 0.16385 -0.3395
+0.3338 0.16445 -0.3238
+0.3544 0.16755 -0.3356
+0.3695 0.17125 -0.3325
+0.3846 0.17425 -0.3303
+0.4021 0.17725 -0.3275
+0.4216 0.17935 -0.3269
+0.4409 0.18045 -0.3269
+0.4409 0.18385 -0.3079
+0.4603 0.18755 -0.2889
+0.4603 0.19095 -0.2699
+0.4603 0.19425 -0.2509
+0.4603 0.19765 -0.2318
+0.4603 0.20105 -0.2127
+0.4603 0.22235 -0.1969
+0.4603 0.22305 -0.1777
+0.4603 0.22505 -0.1591
+0.4603 0.23805 -0.1794
+0.4603 0.25555 -0.1805
+0.4415 0.25555 -0.1617
+0.4415 0.25555 -0.1429
+0.4227 0.25555 -0.1241
+0.4039 0.25555 -0.1054
+0.4039 0.25555 -0.0866
+0.4039 0.25555 -0.0678
+0.4039 0.25555 -0.049
+0.4039 0.25555 -0.0302
+0.3851 0.25555 -0.0114
+0.3851 0.25555 0.0074
+0.3851 0.25555 0.0262
+0.3851 0.25555 0.045
+0.3851 0.25555 0.0638
+0.3851 0.25555 0.0826
+0.3851 0.25555 0.1014
+0.3851 0.25555 0.1202
+0.4039 0.25555 0.139
+0.4227 0.25555 0.139
+0.4415 0.25555 0.139
+0.4415 0.25555 0.1578
+0.4603 0.25555 0.1578
+0.4603 0.23675 0.1578
+0.4603 0.21925 0.1585
+0.4603 0.20215 0.1626
+0.4603 0.18415 0.1658
+0.4603 0.18735 0.1836
+0.4603 0.19055 0.2015
+0.4603 0.19365 0.2193
+0.4416 0.19325 0.2194
+0.4229 0.19195 0.2196
+0.4043 0.18985 0.22
+0.3858 0.18685 0.2205
+0.3675 0.18305 0.2212
+0.3494 0.17845 0.222
+0.3315 0.17295 0.223
+0.3139 0.16665 0.2241
+0.2966 0.15965 0.2253
+0.2797 0.15185 0.2267
+0.2631 0.14325 0.2282
+0.247 0.13385 0.2299
+0.2313 0.12385 0.2316
+0.2161 0.11305 0.2335
+0.2014 0.10165 0.2355
+0.1873 0.08955 0.2377
+0.1738 0.07675 0.2399
+0.1608 0.06345 0.2423
+0.1485 0.04955 0.2447
+0.1369 0.03505 0.2473
+0.126 0.02015 0.2499
+0.1158 0.00465 0.2526
+0.1063 -0.01125 0.2554
+0.0976 -0.02755 0.2583
+0.0896 -0.04425 0.2613
+0.0824 -0.06435 0.2464
+0.0761 -0.08485 0.2316
+0.0705 -0.10245 0.2348
+0.0658 -0.12035 0.2379
+0.062 -0.14155 0.2232
+0.059 -0.15969 0.2264
+0.0568 -0.18115 0.2118
+0.075 -0.18424 0.2124
+0.0923 -0.20234 0.2156
+0.1104 -0.21797 0.2183
+0.092 -0.21797 0.2183
+0.0735 -0.21797 0.2183
+0.0551 -0.21797 0.2183
+0.0367 -0.21797 0.2183
+0.0182 -0.21795 0.2184
+-0.0004 -0.21792 0.2186
+-0.0191 -0.22114 0.2003
+-0.0397 -0.22113 0.2004
+-0.037 -0.22486 0.1792
+-0.0467 -0.22869 0.1575
+-0.0607 -0.22956 0.1526
+-0.0578 -0.23185 0.1396
+-0.073 -0.23226 0.1373
+-0.0885 -0.2319 0.1393
+-0.0914 -0.23455 0.1242
+-0.1143 -0.2349 0.1223
+-0.1286 -0.23603 0.1159
+-0.1291 -0.23791 0.1052
+-0.1294 -0.24018 0.0924
+-0.1295 -0.24278 0.0776
+-0.1295 -0.245983 0.0595
+-0.1295 -0.249172 0.0414
+-0.1295 -0.252361 0.0233
+-0.1295 -0.25555 0.0052
+-0.1418 -0.23719 0.002
+-0.1536 -0.21887 -0.001
+-0.1653 -0.20057 -0.004
+-0.1772 -0.18238 -0.0068
+-0.1897 -0.1643 -0.0094
+-0.2029 -0.14635 -0.0119
+-0.2168 -0.12865 -0.0144
+-0.2311 -0.11105 -0.0166
+-0.2459 -0.09375 -0.0187
+-0.261 -0.07655 -0.0207
+-0.2765 -0.05955 -0.0226
+-0.2924 -0.04255 -0.0244
+-0.3087 -0.02565 -0.0261
+-0.3064 -0.00775 -0.0289
+-0.3031 0.01405 -0.0357
+-0.3013 0.03725 -0.0476
+-0.3123 0.04895 -0.0559
+-0.3197 0.06605 -0.0547
+-0.3274 0.08355 -0.0534
+-0.3331 0.10085 -0.0528
+-0.3398 0.11585 -0.0517
+-0.3486 0.12705 -0.05
+-0.3353 0.14665 -0.0542
+-0.3121 0.15645 -0.0615
+-0.2971 0.16235 -0.0669
+-0.2855 0.16685 -0.0714
+-0.2708 0.18295 -0.0779
+-0.253 0.18415 -0.0861
+-0.237 0.18405 -0.0942
+-0.2199 0.18295 -0.1037
+-0.2214 0.20185 -0.1032
+-0.2215 0.21975 -0.1034
+-0.2062 0.23765 -0.1128
+-0.1913 0.23765 -0.1226
+-0.1768 0.23765 -0.133
+-0.1626 0.23765 -0.1438
+-0.1489 0.23765 -0.1553
+-0.1356 0.23765 -0.1672
+-0.1227 0.23765 -0.1797
+-0.1103 0.23755 -0.1926
+-0.0974 0.23755 -0.2071
+-0.0858 0.23785 -0.2207
+-0.0742 0.23815 -0.2343
+-0.0626 0.23845 -0.248
+-0.051 0.23875 -0.2616
+-0.0388 0.23905 -0.2757
+-0.0261 0.23935 -0.2895
+-0.0132 0.22375 -0.3028
+0.0002 0.22435 -0.316
+0.0141 0.22485 -0.3288
+0.0284 0.22545 -0.3413
+0.043 0.22595 -0.3533
+0.0578 0.21185 -0.3647
+0.073 0.21265 -0.3759
+0.0885 0.21325 -0.3866
+0.1041 0.21395 -0.3968
+0.1199 0.21465 -0.4064
+0.1358 0.21525 -0.4156
+0.1509 0.20265 -0.4235
+0.1657 0.19035 -0.4308
+0.182 0.19125 -0.4386
+0.197 0.17945 -0.445
+0.2121 0.16795 -0.451
+0.2291 0.16895 -0.4575
+0.2444 0.15775 -0.4627
+0.2672 0.15575 -0.4515
+0.271 0.15305 -0.436
+0.2851 0.15365 -0.4394
+0.305 0.15195 -0.4299
+0.3122 0.14985 -0.4179
+0.3247 0.14755 -0.4049
+0.3349 0.15305 -0.3905
+0.3455 0.15835 -0.3759
+0.341 0.16075 -0.356
+0.3463 0.16335 -0.3487
+0.3561 0.16535 -0.35
+0.371 0.16815 -0.3514
+0.3872 0.17085 -0.3522
+0.4038 0.17365 -0.3488
+0.4218 0.17595 -0.3462
+0.441 0.17715 -0.346
+0.4603 0.18085 -0.327
+0.4603 0.18425 -0.308
+0.4603 0.21065 -0.3029
+0.4603 0.21385 -0.2739
+0.4603 0.21555 -0.2549
+0.4603 0.21635 -0.2351
+0.4603 0.21945 -0.216
+0.4603 0.23675 -0.1993
+0.4603 0.25555 -0.1993
+0.4415 0.25555 -0.1805
+0.4227 0.25555 -0.1617
+0.4227 0.25555 -0.1429
+0.4039 0.25555 -0.1241
+0.3851 0.25555 -0.1054
+0.3851 0.25555 -0.0866
+0.3851 0.25555 -0.0678
+0.3851 0.25555 -0.049
+0.3851 0.25555 -0.0302
+0.3663 0.25555 -0.0114
+0.3663 0.25555 0.0074
+0.3663 0.25555 0.0262
+0.3663 0.25555 0.045
+0.3663 0.25555 0.0638
+0.3663 0.25555 0.0826
+0.3663 0.25555 0.1014
+0.3663 0.25555 0.1201
+0.385 0.25555 0.1389
+0.4038 0.25555 0.1577
+0.4227 0.25555 0.1578
+0.4415 0.25555 0.1765
+0.4603 0.25555 0.1765
+0.4603 0.23675 0.1765
+0.4603 0.22205 0.1778
+0.4603 0.20535 0.1805
+0.4603 0.20855 0.1983
+0.4603 0.20985 0.2166
+0.4603 0.19675 0.2371
+0.4416 0.19635 0.2372
+0.4229 0.19505 0.2375
+0.4043 0.19295 0.2378
+0.3858 0.18995 0.2384
+0.3675 0.18615 0.239
+0.3494 0.18155 0.2398
+0.3315 0.17605 0.2408
+0.3139 0.16985 0.2419
+0.2966 0.16275 0.2431
+0.2797 0.15495 0.2445
+0.2631 0.14635 0.246
+0.247 0.13705 0.2477
+0.2313 0.12695 0.2495
+0.2161 0.11615 0.2514
+0.2014 0.10475 0.2534
+0.1873 0.09265 0.2555
+0.1738 0.07995 0.2578
+0.1608 0.06655 0.2601
+0.1485 0.05265 0.2626
+0.1369 0.03825 0.2651
+0.126 0.02325 0.2678
+0.1158 0.00785 0.2705
+0.1063 -0.00805 0.2733
+0.0976 -0.02435 0.2762
+0.0896 -0.04105 0.2791
+0.0824 -0.06125 0.2642
+0.0761 -0.08175 0.2495
+0.0705 -0.09935 0.2526
+0.0658 -0.11715 0.2557
+0.062 -0.13835 0.2411
+0.059 -0.15655 0.2443
+0.0568 -0.178 0.2297
+0.0555 -0.19953 0.2151
+0.0739 -0.20184 0.2155
+0.0555 -0.19639 0.2329
+0.0551 -0.21482 0.2362
+0.0368 -0.21482 0.2362
+0.0184 -0.21481 0.2363
+0 -0.21476 0.2365
+-0.0185 -0.2147 0.2368
+-0.019 -0.21788 0.2189
+-0.0388 -0.21781 0.2192
+-0.0553 -0.21731 0.222
+-0.056 -0.22161 0.1977
+-0.0514 -0.2257 0.1745
+-0.0684 -0.22712 0.1664
+-0.0722 -0.23049 0.1473
+-0.0815 -0.22937 0.1537
+-0.1031 -0.22752 0.1642
+-0.1191 -0.23179 0.14
+-0.14 -0.23475 0.1232
+-0.1465 -0.23751 0.1075
+-0.1482 -0.24025 0.092
+-0.1479 -0.24278 0.0776
+-0.1479 -0.245983 0.0595
+-0.1479 -0.249172 0.0414
+-0.1479 -0.252361 0.0233
+-0.1479 -0.25555 0.0052
+-0.1598 -0.23721 0.002
+-0.1714 -0.21888 -0.001
+-0.1828 -0.2006 -0.004
+-0.1946 -0.1824 -0.0068
+-0.2071 -0.16433 -0.0094
+-0.2203 -0.14645 -0.0119
+-0.2343 -0.12865 -0.0143
+-0.2488 -0.11115 -0.0166
+-0.2638 -0.09375 -0.0187
+-0.2791 -0.07655 -0.0207
+-0.2949 -0.05955 -0.0226
+-0.311 -0.04255 -0.0244
+-0.3277 -0.02565 -0.0261
+-0.3268 -0.00845 -0.0288
+-0.3285 0.00735 -0.0332
+-0.3183 0.02935 -0.0429
+-0.328 0.04165 -0.0505
+-0.3356 0.05845 -0.0496
+-0.3434 0.07565 -0.0486
+-0.3488 0.09405 -0.0483
+-0.3526 0.10865 -0.0482
+-0.3554 0.11745 -0.048
+-0.36061 0.13385 -0.0475
+-0.3554 0.15435 -0.0495
+-0.3348 0.17545 -0.0552
+-0.317 0.17895 -0.0606
+-0.3007 0.18075 -0.0661
+-0.2857 0.18215 -0.0717
+-0.2857 0.20205 -0.0722
+-0.2693 0.20205 -0.079
+-0.2531 0.20195 -0.0864
+-0.2371 0.20185 -0.0946
+-0.2371 0.21975 -0.0948
+-0.2372 0.23765 -0.0949
+-0.2216 0.23765 -0.1036
+-0.2216 0.25555 -0.1036
+-0.2062 0.25555 -0.1129
+-0.1913 0.25555 -0.1227
+-0.1768 0.25555 -0.133
+-0.1626 0.25555 -0.1439
+-0.1489 0.25555 -0.1553
+-0.1356 0.25555 -0.1672
+-0.1227 0.25555 -0.1797
+-0.1103 0.25555 -0.1926
+-0.0974 0.25555 -0.2071
+-0.0858 0.25555 -0.2208
+-0.0742 0.25555 -0.2344
+-0.0626 0.25555 -0.248
+-0.051 0.25555 -0.2616
+-0.0387 0.25555 -0.2758
+-0.026 0.25555 -0.2897
+-0.0131 0.23965 -0.303
+0.0004 0.23995 -0.3162
+0.0142 0.24025 -0.329
+0.0284 0.24045 -0.3414
+0.043 0.24075 -0.3534
+0.058 0.22645 -0.365
+0.0732 0.22695 -0.3762
+0.0887 0.22735 -0.3869
+0.1044 0.22785 -0.3971
+0.1202 0.22825 -0.4068
+0.1363 0.22865 -0.416
+0.1519 0.21585 -0.4243
+0.1671 0.20335 -0.4318
+0.1835 0.20405 -0.4395
+0.1986 0.19215 -0.446
+0.2138 0.18045 -0.452
+0.2307 0.18125 -0.4585
+0.2462 0.16995 -0.4637
+0.2617 0.15875 -0.4684
+0.2824 0.15665 -0.4563
+0.302 0.15435 -0.4432
+0.3165 0.15285 -0.4349
+0.3265 0.15165 -0.4277
+0.3393 0.14965 -0.4164
+0.3489 0.15465 -0.4015
+0.3594 0.15965 -0.3865
+0.3684 0.16445 -0.3698
+0.3528 0.16245 -0.3624
+0.3877 0.16635 -0.3779
+0.4082 0.17055 -0.3693
+0.4235 0.17235 -0.3672
+0.441 0.17375 -0.365
+0.4603 0.17755 -0.3461
+0.4603 0.19615 -0.3321
+0.4603 0.20085 -0.3178
+0.4603 0.21925 -0.3197
+0.4603 0.23645 -0.2965
+0.4603 0.23675 -0.2745
+0.4603 0.23675 -0.2557
+0.4603 0.23675 -0.2369
+0.4603 0.23675 -0.2181
+0.4603 0.25555 -0.2181
+0.4415 0.25555 -0.1993
+0.4227 0.25555 -0.1805
+0.4039 0.25555 -0.1617
+0.4039 0.25555 -0.1429
+0.3851 0.25555 -0.1241
+0.3663 0.25555 -0.1054
+0.3663 0.25555 -0.0866
+0.3663 0.25555 -0.0678
+0.3663 0.25555 -0.049
+0.3663 0.25555 -0.0302
+0.3475 0.25555 -0.0114
+0.3475 0.25555 0.0074
+0.3475 0.25555 0.0262
+0.3475 0.25555 0.045
+0.3475 0.25555 0.0638
+0.3475 0.25555 0.0826
+0.3475 0.25555 0.1013
+0.3474 0.25555 0.1201
+0.3662 0.25555 0.1389
+0.385 0.25555 0.1577
+0.4038 0.25555 0.1765
+0.4226 0.25555 0.1765
+0.4415 0.25555 0.1953
+0.4603 0.25555 0.1953
+0.4603 0.23845 0.1951
+0.4603 0.22365 0.1961
+0.4603 0.22385 0.2146
+0.4603 0.22585 0.2326
+0.4603 0.21285 0.2345
+0.4603 0.19995 0.255
+0.4416 0.19955 0.2551
+0.4229 0.19825 0.2553
+0.4043 0.19615 0.2557
+0.3858 0.19315 0.2562
+0.3675 0.18935 0.2569
+0.3494 0.18475 0.2577
+0.3315 0.17925 0.2586
+0.3139 0.17295 0.2597
+0.2966 0.16595 0.261
+0.2797 0.15815 0.2624
+0.2631 0.14955 0.2639
+0.247 0.14015 0.2655
+0.2313 0.13015 0.2673
+0.2161 0.11935 0.2692
+0.2014 0.10785 0.2712
+0.1873 0.09575 0.2733
+0.1738 0.08305 0.2756
+0.1608 0.06975 0.2779
+0.1485 0.05585 0.2804
+0.1369 0.04135 0.283
+0.126 0.02645 0.2856
+0.1158 0.01095 0.2883
+0.1063 -0.00495 0.2911
+0.0976 -0.02125 0.294
+0.0896 -0.03795 0.2969
+0.0824 -0.05805 0.2821
+0.0761 -0.07855 0.2673
+0.0705 -0.09615 0.2704
+0.0658 -0.11405 0.2736
+0.062 -0.13525 0.2589
+0.059 -0.15345 0.2621
+0.0568 -0.17486 0.2475
+0.0555 -0.19325 0.2507
+0.0551 -0.21168 0.254
+0.0369 -0.21168 0.254
+0.0186 -0.21167 0.254
+0.0004 -0.21165 0.2542
+-0.0161 -0.2116 0.2545
+-0.0298 -0.21159 0.2545
+-0.0342 -0.2144 0.2386
+-0.0474 -0.21363 0.2429
+-0.0797 -0.21641 0.2272
+-0.0829 -0.22282 0.1908
+-0.1206 -0.21981 0.2079
+-0.1348 -0.22511 0.1778
+-0.1503 -0.22956 0.1526
+-0.1632 -0.23318 0.132
+-0.1774 -0.23773 0.1063
+-0.1712 -0.24093 0.0881
+-0.1678 -0.24336 0.0744
+-0.1662 -0.245983 0.0595
+-0.1662 -0.249172 0.0414
+-0.1662 -0.252361 0.0233
+-0.1662 -0.25555 0.0052
+-0.1779 -0.23721 0.002
+-0.1891 -0.2189 -0.001
+-0.2003 -0.20062 -0.004
+-0.212 -0.18243 -0.0068
+-0.2244 -0.16436 -0.0094
+-0.2377 -0.14645 -0.0119
+-0.2517 -0.12875 -0.0143
+-0.2664 -0.11115 -0.0166
+-0.2816 -0.09375 -0.0187
+-0.2973 -0.07655 -0.0207
+-0.3132 -0.05955 -0.0226
+-0.3297 -0.04265 -0.0244
+-0.3466 -0.02565 -0.0261
+-0.3449 -0.00805 -0.0289
+-0.3415 0.00435 -0.0322
+-0.3352 0.02075 -0.0386
+-0.3442 0.03385 -0.0456
+-0.3545 0.04945 -0.0442
+-0.36859 0.06795 -0.0425
+-0.36566 0.09105 -0.0444
+-0.36476 0.10745 -0.0455
+-0.36452 0.11955 -0.0461
+-0.37624 0.12115 -0.0439
+-0.37393 0.13845 -0.0451
+-0.3701 0.15865 -0.0465
+-0.3523 0.17795 -0.0509
+-0.3361 0.20005 -0.0555
+-0.3194 0.20195 -0.0603
+-0.3023 0.20195 -0.066
+-0.3024 0.21975 -0.0662
+-0.2858 0.21985 -0.0724
+-0.2694 0.21985 -0.0792
+-0.2531 0.21985 -0.0867
+-0.2532 0.23765 -0.0868
+-0.2372 0.25555 -0.0949
+-0.2272 0.25555 -0.0797
+-0.211 0.25555 -0.0887
+-0.1952 0.25555 -0.0984
+-0.1797 0.25555 -0.1086
+-0.1646 0.25555 -0.1194
+-0.15 0.25555 -0.1308
+-0.1358 0.25555 -0.1427
+-0.122 0.25555 -0.1551
+-0.1087 0.25555 -0.1681
+-0.096 0.25555 -0.1816
+-0.0838 0.25555 -0.1954
+-0.0721 0.25555 -0.2091
+-0.0604 0.25555 -0.2226
+-0.0487 0.25555 -0.2361
+-0.0369 0.25555 -0.2495
+-0.0248 0.25555 -0.2635
+-0.0125 0.25555 -0.277
+0.0002 0.25555 -0.2901
+-0.013 0.25555 -0.3032
+0.0005 0.25555 -0.3164
+0.0143 0.25555 -0.3291
+0.0284 0.25555 -0.3414
+0.0429 0.25555 -0.3534
+0.0577 0.25555 -0.3649
+0.0579 0.24095 -0.365
+0.0731 0.24125 -0.3762
+0.0886 0.24145 -0.3869
+0.1043 0.24165 -0.3971
+0.1202 0.24185 -0.4068
+0.1364 0.24205 -0.4161
+0.1525 0.22905 -0.4247
+0.1682 0.21635 -0.4325
+0.1847 0.21695 -0.4402
+0.2001 0.20475 -0.4468
+0.2154 0.19285 -0.4529
+0.2324 0.19365 -0.4593
+0.2479 0.18205 -0.4645
+0.2636 0.17085 -0.4693
+0.2792 0.15965 -0.4736
+0.2996 0.15745 -0.4608
+0.3178 0.15495 -0.4467
+0.3381 0.15445 -0.4438
+0.3547 0.15145 -0.427
+0.3634 0.15605 -0.4116
+0.3752 0.16085 -0.3972
+0.3982 0.16455 -0.3968
+0.4116 0.16735 -0.3898
+0.4255 0.16905 -0.3869
+0.441 0.17045 -0.3841
+0.4603 0.17415 -0.3651
+0.4603 0.19285 -0.3498
+0.4603 0.20825 -0.3342
+0.4603 0.20915 -0.3238
+0.4603 0.22075 -0.3334
+0.4603 0.23675 -0.3309
+0.4603 0.23655 -0.3158
+0.4603 0.25555 -0.3121
+0.4603 0.25555 -0.2933
+0.4603 0.25555 -0.2745
+0.4603 0.25555 -0.2557
+0.4603 0.25555 -0.2369
+0.4415 0.25555 -0.2181
+0.4227 0.25555 -0.1993
+0.4039 0.25555 -0.1805
+0.3851 0.25555 -0.1617
+0.3851 0.25555 -0.1429
+0.3663 0.25555 -0.1241
+0.3475 0.25555 -0.1054
+0.3475 0.25555 -0.0866
+0.3475 0.25555 -0.0678
+0.3475 0.25555 -0.049
+0.3475 0.25555 -0.0302
+0.3287 0.25555 -0.0114
+0.3287 0.25555 0.0074
+0.3287 0.25555 0.0262
+0.3287 0.25555 0.045
+0.3287 0.25555 0.0638
+0.3287 0.25555 0.0825
+0.3286 0.25555 0.1013
+0.3284 0.25555 0.1199
+0.3471 0.25555 0.1387
+0.366 0.25555 0.1576
+0.3848 0.25555 0.1764
+0.4037 0.25555 0.1953
+0.4226 0.25555 0.1953
+0.4414 0.25555 0.2141
+0.4603 0.25555 0.2141
+0.4603 0.23925 0.2143
+0.4603 0.23925 0.2324
+0.4603 0.22835 0.2498
+0.4603 0.21665 0.2519
+0.4603 0.21975 0.2658
+0.4603 0.20305 0.2728
+0.4416 0.20265 0.2729
+0.4229 0.20135 0.2731
+0.4043 0.19925 0.2735
+0.3858 0.19625 0.274
+0.3675 0.19245 0.2747
+0.3494 0.18785 0.2755
+0.3315 0.18235 0.2765
+0.3139 0.17615 0.2776
+0.2966 0.16905 0.2788
+0.2797 0.16125 0.2802
+0.2631 0.15265 0.2817
+0.247 0.14335 0.2834
+0.2313 0.13325 0.2852
+0.2161 0.12245 0.287
+0.2014 0.11105 0.2891
+0.1873 0.09895 0.2912
+0.1738 0.08625 0.2934
+0.1608 0.07285 0.2958
+0.1485 0.05895 0.2982
+0.1369 0.04455 0.3008
+0.126 0.02955 0.3034
+0.1158 0.01415 0.3061
+0.1063 -0.00175 0.3089
+0.0976 -0.01805 0.3118
+0.0896 -0.03475 0.3148
+0.0824 -0.05495 0.2999
+0.0761 -0.07545 0.2852
+0.0705 -0.09305 0.2883
+0.0658 -0.11085 0.2914
+0.062 -0.13205 0.2767
+0.059 -0.15025 0.28
+0.0568 -0.17172 0.2653
+0.0555 -0.1901 0.2686
+0.0551 -0.20854 0.2718
+0.0369 -0.20854 0.2718
+0.0187 -0.20854 0.2719
+0.0006 -0.20851 0.272
+-0.0176 -0.20849 0.2721
+-0.032 -0.20887 0.2699
+-0.0397 -0.21151 0.2549
+-0.0636 -0.21095 0.2581
+-0.1036 -0.21038 0.2614
+-0.1112 -0.21514 0.2344
+-0.1396 -0.21412 0.2401
+-0.1487 -0.21846 0.2156
+-0.1609 -0.22288 0.1905
+-0.1756 -0.22685 0.168
+-0.1907 -0.23051 0.1472
+-0.2158 -0.23576 0.1175
+-0.1963 -0.24108 0.0873
+-0.1844 -0.24218 0.081
+-0.1843 -0.244 0.0707
+-0.1847 -0.246318 0.0575
+-0.1846 -0.249172 0.0414
+-0.1846 -0.252361 0.0233
+-0.1846 -0.25555 0.0052
+-0.196 -0.23722 0.002
+-0.2069 -0.2189 -0.001
+-0.2179 -0.20063 -0.004
+-0.2294 -0.18246 -0.0067
+-0.2417 -0.16439 -0.0094
+-0.255 -0.14645 -0.0119
+-0.2692 -0.12875 -0.0143
+-0.2841 -0.11115 -0.0166
+-0.2995 -0.09385 -0.0187
+-0.3154 -0.07665 -0.0207
+-0.3316 -0.05965 -0.0226
+-0.3483 -0.04265 -0.0244
+-0.36557 -0.02565 -0.0261
+-0.36434 -0.00175 -0.0304
+-0.3505 0.01285 -0.0353
+-0.36046 0.02595 -0.0412
+-0.36955 0.04115 -0.0404
+-0.37964 0.05525 -0.0396
+-0.38359 0.07245 -0.04
+-0.38091 0.08965 -0.0415
+-0.37815 0.10625 -0.0429
+-0.38866 0.12355 -0.042
+-0.38843 0.14065 -0.0427
+-0.38639 0.16115 -0.0438
+-0.36815 0.18005 -0.0475
+-0.3533 0.20175 -0.0512
+-0.3366 0.21975 -0.0556
+-0.3194 0.21975 -0.0606
+-0.3025 0.23765 -0.0663
+-0.2859 0.23765 -0.0725
+-0.2694 0.23765 -0.0794
+-0.2532 0.25555 -0.0869
+-0.2438 0.25555 -0.0712
+-0.2183 0.25555 -0.063
+-0.2015 0.25555 -0.0724
+-0.185 0.25555 -0.0825
+-0.169 0.25555 -0.0931
+-0.1534 0.25555 -0.1043
+-0.1382 0.25555 -0.1161
+-0.1234 0.25555 -0.1285
+-0.1092 0.25555 -0.1416
+-0.0956 0.25555 -0.1551
+-0.0826 0.25555 -0.1693
+-0.0703 0.25555 -0.1836
+-0.0585 0.25555 -0.1974
+-0.0468 0.25555 -0.2109
+-0.0351 0.25555 -0.2243
+-0.0233 0.25555 -0.2377
+-0.0114 0.25555 -0.2515
+0.0006 0.25555 -0.2646
+0.0129 0.25555 -0.2774
+0.0255 0.25555 -0.2898
+0.0133 0.25555 -0.3029
+0.0267 0.25555 -0.3154
+0.0404 0.25555 -0.3273
+0.0545 0.25555 -0.3389
+0.0689 0.25555 -0.3501
+0.0727 0.25555 -0.376
+0.0883 0.25555 -0.3867
+0.104 0.25555 -0.397
+0.12 0.25555 -0.4067
+0.1363 0.25555 -0.4161
+0.1528 0.24225 -0.4249
+0.1689 0.22945 -0.433
+0.1856 0.22975 -0.4407
+0.2014 0.21745 -0.4475
+0.2169 0.20535 -0.4537
+0.2339 0.20595 -0.4601
+0.2495 0.19435 -0.4653
+0.2651 0.18285 -0.4701
+0.281 0.17165 -0.4745
+0.2968 0.16055 -0.4783
+0.3169 0.15815 -0.4648
+0.336 0.15885 -0.4689
+0.3611 0.15675 -0.4571
+0.3708 0.15315 -0.4365
+0.3803 0.15755 -0.4209
+0.3893 0.16135 -0.4079
+0.4059 0.16395 -0.4051
+0.4153 0.16505 -0.4047
+0.428 0.16605 -0.4048
+0.4429 0.16705 -0.4034
+0.4603 0.17075 -0.3841
+0.4603 0.19055 -0.3681
+0.4603 0.20595 -0.3518
+0.4603 0.21985 -0.3517
+0.4603 0.23675 -0.3497
+0.4603 0.25555 -0.3309
+0.4415 0.25555 -0.3121
+0.4415 0.25555 -0.2933
+0.4415 0.25555 -0.2745
+0.4415 0.25555 -0.2557
+0.4415 0.25555 -0.2369
+0.4227 0.25555 -0.2181
+0.4039 0.25555 -0.1993
+0.3851 0.25555 -0.1805
+0.3663 0.25555 -0.1617
+0.3663 0.25555 -0.1429
+0.3475 0.25555 -0.1241
+0.3287 0.25555 -0.1054
+0.3287 0.25555 -0.0866
+0.3287 0.25555 -0.0678
+0.3287 0.25555 -0.049
+0.3287 0.25555 -0.0302
+0.3099 0.25555 -0.0114
+0.31 0.25555 0.0074
+0.31 0.25555 0.0262
+0.31 0.25555 0.045
+0.31 0.25555 0.0638
+0.3098 0.25555 0.0798
+0.3099 0.25555 0.1011
+0.3091 0.25555 0.1189
+0.3278 0.25555 0.1384
+0.3466 0.25555 0.1573
+0.3656 0.25555 0.1762
+0.3846 0.25555 0.1951
+0.4035 0.25555 0.214
+0.4226 0.25555 0.2141
+0.4414 0.25555 0.2329
+0.4603 0.25555 0.2329
+0.4603 0.24085 0.2506
+0.4603 0.22895 0.2596
+0.4603 0.23915 0.2646
+0.4603 0.23095 0.2812
+0.4603 0.20625 0.2907
+0.4416 0.20575 0.2907
+0.4229 0.20455 0.291
+0.4043 0.20245 0.2913
+0.3858 0.19945 0.2919
+0.3675 0.19565 0.2925
+0.3494 0.19095 0.2933
+0.3315 0.18555 0.2943
+0.3139 0.17925 0.2954
+0.2966 0.17225 0.2967
+0.2797 0.16435 0.298
+0.2631 0.15575 0.2996
+0.247 0.14645 0.3012
+0.2313 0.13635 0.303
+0.2161 0.12565 0.3049
+0.2014 0.11415 0.3069
+0.1873 0.10205 0.309
+0.1738 0.08935 0.3113
+0.1608 0.07605 0.3136
+0.1485 0.06215 0.3161
+0.1369 0.04765 0.3186
+0.126 0.03265 0.3213
+0.1158 0.01725 0.324
+0.1063 0.00135 0.3268
+0.0976 -0.01495 0.3297
+0.0896 -0.03165 0.3326
+0.0824 -0.05185 0.3178
+0.0761 -0.07225 0.303
+0.0705 -0.08985 0.3061
+0.0658 -0.10775 0.3092
+0.062 -0.12895 0.2946
+0.059 -0.14715 0.2978
+0.0568 -0.16857 0.2832
+0.0555 -0.18696 0.2864
+0.0551 -0.20539 0.2897
+0.0369 -0.20539 0.2897
+0.0188 -0.20538 0.2897
+0.0007 -0.20538 0.2897
+-0.0174 -0.20536 0.2898
+-0.0341 -0.20558 0.2886
+-0.047 -0.20594 0.2866
+-0.0453 -0.20947 0.2666
+-0.0652 -0.20596 0.2865
+-0.1007 -0.20561 0.2884
+-0.1282 -0.20541 0.2896
+-0.1328 -0.20974 0.265
+-0.1579 -0.20927 0.2677
+-0.165 -0.21327 0.2449
+-0.174 -0.21708 0.2234
+-0.1849 -0.22091 0.2016
+-0.1987 -0.22447 0.1815
+-0.214 -0.22775 0.1629
+-0.2339 -0.2313 0.1427
+-0.2438 -0.23733 0.1085
+-0.2178 -0.24029 0.0917
+-0.1996 -0.24348 0.0736
+-0.2017 -0.246211 0.0582
+-0.203 -0.249172 0.0414
+-0.203 -0.252361 0.0233
+-0.203 -0.25555 0.0052
+-0.214 -0.23722 0.002
+-0.2247 -0.21891 -0.001
+-0.2354 -0.20066 -0.004
+-0.2467 -0.18247 -0.0067
+-0.259 -0.16442 -0.0094
+-0.2724 -0.14655 -0.0119
+-0.2867 -0.12875 -0.0143
+-0.3018 -0.11125 -0.0166
+-0.3174 -0.09385 -0.0187
+-0.3335 -0.07675 -0.0207
+-0.35 -0.05975 -0.0226
+-0.36699 -0.04285 -0.0244
+-0.38452 -0.02565 -0.0261
+-0.38874 -0.00785 -0.0289
+-0.37673 0.01785 -0.0373
+-0.38507 0.03445 -0.0372
+-0.3908 0.04895 -0.0374
+-0.38686 0.06305 -0.0389
+-0.39461 0.07285 -0.0384
+-0.39344 0.08885 -0.0395
+-0.39139 0.10565 -0.0407
+-0.4054 0.12365 -0.0397
+-0.40477 0.14255 -0.0406
+-0.40444 0.16145 -0.0413
+-0.40427 0.18025 -0.0419
+-0.38591 0.18025 -0.0444
+-0.36988 0.20115 -0.0477
+-0.3539 0.21965 -0.0513
+-0.354 0.23765 -0.0515
+-0.3367 0.23765 -0.0558
+-0.3195 0.23765 -0.0607
+-0.3195 0.25555 -0.0608
+-0.3025 0.25555 -0.0664
+-0.2859 0.25555 -0.0726
+-0.2694 0.25555 -0.0794
+-0.2606 0.25555 -0.0634
+-0.253 0.25555 -0.0461
+-0.2355 0.25555 -0.0542
+-0.2091 0.25555 -0.0457
+-0.1916 0.25555 -0.0555
+-0.1745 0.25555 -0.0659
+-0.1579 0.25555 -0.077
+-0.1417 0.25555 -0.0886
+-0.1259 0.25555 -0.101
+-0.1107 0.25555 -0.114
+-0.096 0.25555 -0.1277
+-0.0821 0.25555 -0.1422
+-0.069 0.25555 -0.1572
+-0.0567 0.25555 -0.172
+-0.0452 0.25555 -0.1857
+-0.0336 0.25555 -0.1991
+-0.022 0.25555 -0.2124
+-0.0105 0.25555 -0.2258
+0.0009 0.25555 -0.2392
+0.0125 0.25555 -0.2519
+0.0244 0.25555 -0.2644
+0.0367 0.25555 -0.2765
+0.0493 0.25555 -0.2882
+0.0385 0.25555 -0.3019
+0.0519 0.25555 -0.3135
+0.0656 0.25555 -0.3248
+0.0796 0.25555 -0.3357
+0.094 0.25555 -0.3461
+0.0835 0.25555 -0.3608
+0.0986 0.25555 -0.3713
+0.1139 0.25555 -0.3813
+0.1295 0.25555 -0.3907
+0.1453 0.25555 -0.3998
+0.1614 0.25555 -0.4085
+0.1529 0.25555 -0.425
+0.1694 0.24245 -0.4332
+0.1862 0.24265 -0.4411
+0.2024 0.23015 -0.4481
+0.2183 0.21795 -0.4544
+0.2353 0.21835 -0.4607
+0.251 0.20655 -0.466
+0.2667 0.19495 -0.4708
+0.2826 0.18355 -0.4751
+0.2987 0.17235 -0.4791
+0.3146 0.16125 -0.4826
+0.3324 0.16195 -0.4863
+0.3543 0.15975 -0.4738
+0.3677 0.16085 -0.4799
+0.3811 0.15935 -0.4716
+0.3876 0.15455 -0.4446
+0.3943 0.15845 -0.4283
+0.4015 0.16185 -0.4145
+0.4139 0.16265 -0.4177
+0.4278 0.16325 -0.4204
+0.443 0.16375 -0.4221
+0.4603 0.16405 -0.4222
+0.4603 0.16745 -0.4032
+0.4603 0.18985 -0.3875
+0.4603 0.20405 -0.3697
+0.4603 0.21885 -0.3688
+0.4603 0.23675 -0.3685
+0.4603 0.25555 -0.3497
+0.4415 0.25555 -0.3309
+0.4227 0.25555 -0.3121
+0.4227 0.25555 -0.2933
+0.4227 0.25555 -0.2745
+0.4227 0.25555 -0.2557
+0.4227 0.25555 -0.2369
+0.4039 0.25555 -0.2181
+0.3851 0.25555 -0.1993
+0.3663 0.25555 -0.1805
+0.3475 0.25555 -0.1617
+0.3475 0.25555 -0.1429
+0.3287 0.25555 -0.1241
+0.3099 0.25555 -0.1054
+0.3099 0.25555 -0.0866
+0.3099 0.25555 -0.0678
+0.3099 0.25555 -0.049
+0.3099 0.25555 -0.0302
+0.2912 0.25555 -0.0114
+0.2912 0.25555 0.0074
+0.2913 0.25555 0.0262
+0.2914 0.25555 0.045
+0.2911 0.25555 0.0616
+0.2902 0.25555 0.0766
+0.2885 0.25555 0.0988
+0.2849 0.25555 0.1177
+0.3059 0.25555 0.1375
+0.3263 0.25555 0.1561
+0.3456 0.25555 0.1756
+0.3648 0.25555 0.1947
+0.384 0.25555 0.2138
+0.4032 0.25555 0.2328
+0.4225 0.25555 0.2329
+0.4414 0.25555 0.2517
+0.4603 0.25555 0.2517
+0.4603 0.25555 0.2705
+0.4603 0.25555 0.2893
+0.4603 0.23245 0.3082
+0.4603 0.20935 0.3085
+0.4417 0.20895 0.3086
+0.4231 0.20765 0.3088
+0.4045 0.20555 0.3092
+0.386 0.20265 0.3097
+0.3677 0.19885 0.3104
+0.3496 0.19415 0.3112
+0.3317 0.18875 0.3121
+0.314 0.18245 0.3132
+0.2967 0.17545 0.3145
+0.2797 0.16755 0.3159
+0.2632 0.15895 0.3174
+0.247 0.14965 0.319
+0.2314 0.13955 0.3208
+0.2162 0.12885 0.3227
+0.2015 0.11735 0.3247
+0.1874 0.10525 0.3268
+0.1738 0.09255 0.3291
+0.1609 0.07925 0.3314
+0.1486 0.06535 0.3339
+0.137 0.05085 0.3364
+0.126 0.03595 0.3391
+0.1158 0.02045 0.3418
+0.1063 0.00455 0.3446
+0.0976 -0.01175 0.3475
+0.0897 -0.02845 0.3504
+0.0825 -0.04545 0.3534
+0.0824 -0.04865 0.3356
+0.0761 -0.06915 0.3208
+0.0705 -0.08675 0.3239
+0.0658 -0.10455 0.3271
+0.062 -0.12575 0.3124
+0.059 -0.14395 0.3156
+0.0568 -0.16543 0.301
+0.0555 -0.18382 0.3043
+0.0551 -0.20225 0.3075
+0.0369 -0.20225 0.3075
+0.0188 -0.20224 0.3075
+0.0007 -0.20224 0.3076
+-0.0176 -0.2026 0.3055
+-0.0346 -0.2032 0.3021
+-0.045 -0.20378 0.2989
+-0.0571 -0.20196 0.3092
+-0.0961 -0.20127 0.313
+-0.1245 -0.2014 0.3123
+-0.149 -0.20127 0.313
+-0.1525 -0.20527 0.2903
+-0.1722 -0.20507 0.2915
+-0.1772 -0.20883 0.2702
+-0.1843 -0.21249 0.2494
+-0.1935 -0.21603 0.2293
+-0.2047 -0.21942 0.2101
+-0.2178 -0.22259 0.1921
+-0.2327 -0.22552 0.1755
+-0.2499 -0.22822 0.1601
+-0.2539 -0.23382 0.1284
+-0.2602 -0.23896 0.0992
+-0.2509 -0.24095 0.088
+-0.2385 -0.24052 0.0904
+-0.2196 -0.24311 0.0757
+-0.2214 -0.245983 0.0595
+-0.2214 -0.249172 0.0414
+-0.2214 -0.252361 0.0233
+-0.2214 -0.25555 0.0052
+-0.2321 -0.23724 0.002
+-0.2424 -0.21893 -0.001
+-0.2529 -0.20068 -0.004
+-0.2641 -0.18251 -0.0067
+-0.2764 -0.16445 -0.0094
+-0.2897 -0.14655 -0.0119
+-0.3042 -0.12885 -0.0143
+-0.3194 -0.11135 -0.0166
+-0.3353 -0.09405 -0.0187
+-0.3516 -0.07695 -0.0207
+-0.36838 -0.06005 -0.0225
+-0.38565 -0.04305 -0.0243
+-0.40346 -0.02565 -0.0261
+-0.40535 -0.01225 -0.028
+-0.40946 0.00105 -0.0312
+-0.39308 0.00955 -0.034
+-0.39942 0.02615 -0.0345
+-0.40377 0.04165 -0.0351
+-0.40746 0.05735 -0.0358
+-0.39455 0.06095 -0.0377
+-0.40892 0.07105 -0.0365
+-0.40775 0.08725 -0.0376
+-0.4065 0.10465 -0.0387
+-0.42325 0.12405 -0.038
+-0.42304 0.14285 -0.0388
+-0.42292 0.16155 -0.0395
+-0.42285 0.18035 -0.04
+-0.40456 0.19925 -0.0422
+-0.38622 0.20015 -0.0448
+-0.37134 0.21955 -0.0477
+-0.37142 0.23755 -0.0478
+-0.37142 0.25555 -0.0479
+-0.354 0.25555 -0.0515
+-0.3367 0.25555 -0.0558
+-0.3305 0.25555 -0.0387
+-0.3127 0.25555 -0.0439
+-0.2951 0.25555 -0.0497
+-0.2777 0.25555 -0.0562
+-0.2708 0.25555 -0.0387
+-0.2451 0.25555 -0.0281
+-0.2269 0.25555 -0.0366
+-0.1995 0.25555 -0.0278
+-0.1813 0.25555 -0.0379
+-0.1636 0.25555 -0.0488
+-0.1463 0.25555 -0.0602
+-0.1295 0.25555 -0.0724
+-0.1131 0.25555 -0.0853
+-0.0973 0.25555 -0.0991
+-0.0823 0.25555 -0.1138
+-0.0682 0.25555 -0.1294
+-0.0552 0.25555 -0.1455
+-0.0436 0.25555 -0.1603
+-0.0323 0.25555 -0.1737
+-0.021 0.25555 -0.187
+-0.0097 0.25555 -0.2002
+0.0014 0.25555 -0.2134
+0.0123 0.25555 -0.2263
+0.0235 0.25555 -0.2387
+0.0351 0.25555 -0.2509
+0.047 0.25555 -0.2627
+0.0592 0.25555 -0.2742
+0.0719 0.25555 -0.2852
+0.0623 0.25555 -0.2995
+0.0756 0.25555 -0.3105
+0.0893 0.25555 -0.3211
+0.1032 0.25555 -0.3314
+0.1174 0.25555 -0.3413
+0.1086 0.25555 -0.3563
+0.1234 0.25555 -0.366
+0.1386 0.25555 -0.3752
+0.1541 0.25555 -0.384
+0.1697 0.25555 -0.3925
+0.1856 0.25555 -0.4005
+0.1778 0.25555 -0.4167
+0.1697 0.25555 -0.4334
+0.1868 0.25555 -0.4414
+0.2032 0.24285 -0.4485
+0.2194 0.23045 -0.4549
+0.2366 0.23075 -0.4613
+0.2525 0.21875 -0.4666
+0.2683 0.20705 -0.4714
+0.2841 0.19555 -0.4758
+0.3001 0.18415 -0.4798
+0.3164 0.17295 -0.4833
+0.3342 0.17355 -0.487
+0.3504 0.16255 -0.4896
+0.3685 0.16305 -0.4924
+0.3866 0.16345 -0.4947
+0.4048 0.15995 -0.4753
+0.405 0.15575 -0.4513
+0.4106 0.15935 -0.4346
+0.4258 0.16005 -0.4377
+0.4413 0.16035 -0.4412
+0.4603 0.16075 -0.4413
+0.4603 0.18315 -0.4256
+0.4603 0.18645 -0.4066
+0.4603 0.20335 -0.4069
+0.4603 0.20515 -0.388
+0.4603 0.21975 -0.3874
+0.4603 0.23675 -0.3872
+0.4603 0.25555 -0.3685
+0.4415 0.25555 -0.3497
+0.4227 0.25555 -0.3309
+0.4039 0.25555 -0.3121
+0.4039 0.25555 -0.2933
+0.4039 0.25555 -0.2745
+0.4039 0.25555 -0.2557
+0.4039 0.25555 -0.2369
+0.3851 0.25555 -0.2181
+0.3663 0.25555 -0.1993
+0.3475 0.25555 -0.1805
+0.3287 0.25555 -0.1617
+0.3287 0.25555 -0.1429
+0.3099 0.25555 -0.1241
+0.2911 0.25555 -0.1054
+0.2911 0.25555 -0.0866
+0.2911 0.25555 -0.0678
+0.2911 0.25555 -0.049
+0.2912 0.25555 -0.0302
+0.2724 0.25555 -0.0113
+0.2725 0.25555 0.0075
+0.2725 0.25555 0.0263
+0.2746 0.25555 0.0457
+0.2741 0.25555 0.0596
+0.2718 0.25555 0.0719
+0.2669 0.25555 0.0931
+0.2614 0.25555 0.1174
+0.2823 0.25555 0.1371
+0.3025 0.25555 0.1561
+0.3237 0.25555 0.1757
+0.3436 0.25555 0.1934
+0.3634 0.25555 0.2129
+0.3831 0.25555 0.2323
+0.4027 0.25555 0.2514
+0.4223 0.25555 0.2517
+0.4413 0.25555 0.2705
+0.4412 0.25555 0.2893
+0.4603 0.25555 0.3081
+0.4411 0.25555 0.3082
+0.4424 0.23125 0.3083
+0.4221 0.23065 0.3085
+0.3942 0.22915 0.3091
+0.3785 0.21785 0.3096
+0.3637 0.21405 0.3102
+0.347 0.20905 0.311
+0.3292 0.20425 0.3119
+0.3108 0.19845 0.313
+0.2926 0.19185 0.3143
+0.2713 0.18475 0.3159
+0.254 0.17575 0.3175
+0.2402 0.16695 0.319
+0.2226 0.15505 0.321
+0.2067 0.14405 0.3229
+0.1926 0.13285 0.3248
+0.1747 0.11955 0.3274
+0.1605 0.10625 0.3298
+0.147 0.09225 0.3322
+0.1341 0.07775 0.3348
+0.1219 0.06265 0.3375
+0.1105 0.04695 0.3402
+0.0998 0.03075 0.3431
+0.0899 0.01415 0.346
+0.0809 -0.00155 0.3489
+0.0708 -0.01655 0.3519
+0.0625 -0.03405 0.3551
+0.0761 -0.06285 0.3565
+0.0761 -0.06605 0.3387
+0.0705 -0.08365 0.3418
+0.0658 -0.10145 0.3449
+0.062 -0.12265 0.3303
+0.059 -0.14085 0.3335
+0.0568 -0.16227 0.3189
+0.0555 -0.18067 0.3221
+0.0551 -0.19909 0.3253
+0.037 -0.19909 0.3253
+0.0188 -0.19909 0.3254
+-0.0003 -0.19914 0.3251
+-0.0188 -0.19967 0.3221
+-0.0379 -0.2005 0.3174
+-0.0675 -0.19575 0.3444
+-0.0983 -0.19625 0.3415
+-0.1239 -0.19692 0.3377
+-0.1477 -0.19737 0.3352
+-0.1683 -0.1975 0.3344
+-0.1694 -0.20127 0.313
+-0.1884 -0.20118 0.3135
+-0.1914 -0.20472 0.2934
+-0.1977 -0.20791 0.2754
+-0.2045 -0.21125 0.2564
+-0.213 -0.21447 0.2382
+-0.2234 -0.21752 0.2208
+-0.2355 -0.22039 0.2046
+-0.2491 -0.22303 0.1896
+-0.2643 -0.22544 0.1759
+-0.2677 -0.23059 0.1468
+-0.2734 -0.23574 0.1176
+-0.2934 -0.23747 0.1078
+-0.2754 -0.24169 0.0838
+-0.255 -0.24311 0.0758
+-0.2386 -0.24314 0.0755
+-0.2397 -0.245983 0.0595
+-0.2397 -0.249172 0.0414
+-0.2397 -0.252361 0.0233
+-0.2397 -0.25555 0.0052
+-0.2502 -0.23724 0.002
+-0.2602 -0.21894 -0.001
+-0.2705 -0.20069 -0.0039
+-0.2815 -0.18254 -0.0067
+-0.2937 -0.16448 -0.0094
+-0.3071 -0.14665 -0.0119
+-0.3216 -0.12895 -0.0143
+-0.3371 -0.11145 -0.0165
+-0.3531 -0.09435 -0.0187
+-0.36971 -0.07745 -0.0206
+-0.38677 -0.06055 -0.0225
+-0.40431 -0.04345 -0.0243
+-0.42241 -0.02565 -0.0261
+-0.42231 -0.01615 -0.0273
+-0.42591 -0.00745 -0.029
+-0.43116 0.00555 -0.0298
+-0.41831 0.01525 -0.0315
+-0.4269 0.02865 -0.0319
+-0.42604 0.05385 -0.0338
+-0.42607 0.06935 -0.0349
+-0.42437 0.08635 -0.036
+-0.42366 0.10515 -0.0371
+-0.44165 0.12415 -0.0369
+-0.44159 0.14285 -0.0377
+-0.44156 0.16165 -0.0384
+-0.44155 0.18045 -0.0389
+-0.44153 0.19915 -0.0393
+-0.42282 0.19915 -0.0405
+-0.40619 0.21885 -0.0424
+-0.38893 0.21935 -0.0447
+-0.38904 0.23755 -0.0448
+-0.38904 0.25555 -0.0449
+-0.38486 0.25555 -0.0271
+-0.36659 0.25555 -0.0303
+-0.3485 0.25555 -0.0341
+-0.3256 0.25555 -0.0205
+-0.3071 0.25555 -0.0258
+-0.2888 0.25555 -0.0319
+-0.2673 0.25555 -0.0193
+-0.2406 0.25555 -0.0091
+-0.218 0.25555 -0.0183
+-0.1921 0.25555 -0.0135
+-0.1727 0.25555 -0.0205
+-0.1522 0.25555 -0.031
+-0.1343 0.25555 -0.0428
+-0.1167 0.25555 -0.0554
+-0.0996 0.25555 -0.0692
+-0.0833 0.25555 -0.084
+-0.0679 0.25555 -0.1
+-0.0539 0.25555 -0.1172
+-0.0417 0.25555 -0.1343
+-0.031 0.25555 -0.1483
+-0.0201 0.25555 -0.1615
+-0.0091 0.25555 -0.1745
+0.0018 0.25555 -0.1875
+0.0124 0.25555 -0.2004
+0.023 0.25555 -0.213
+0.0338 0.25555 -0.2251
+0.045 0.25555 -0.237
+0.0565 0.25555 -0.2485
+0.0683 0.25555 -0.2597
+0.0805 0.25555 -0.2705
+0.093 0.25555 -0.281
+0.0848 0.25555 -0.296
+0.098 0.25555 -0.3063
+0.1115 0.25555 -0.3163
+0.1253 0.25555 -0.326
+0.1393 0.25555 -0.3352
+0.1319 0.25555 -0.3507
+0.1466 0.25555 -0.3597
+0.1616 0.25555 -0.3683
+0.1768 0.25555 -0.3766
+0.1923 0.25555 -0.3844
+0.2079 0.25555 -0.3917
+0.2017 0.25555 -0.408
+0.1944 0.25555 -0.4245
+0.2112 0.25555 -0.4317
+0.2041 0.25555 -0.4489
+0.2205 0.24295 -0.4554
+0.2378 0.24315 -0.4618
+0.254 0.23095 -0.4672
+0.2699 0.21915 -0.4721
+0.2857 0.20755 -0.4764
+0.3016 0.19605 -0.4803
+0.3177 0.18475 -0.4839
+0.3354 0.18525 -0.4875
+0.352 0.17405 -0.4902
+0.37 0.17455 -0.4929
+0.388 0.17515 -0.4951
+0.4048 0.16375 -0.4965
+0.4233 0.16055 -0.4782
+0.4231 0.15665 -0.4562
+0.4415 0.15715 -0.4593
+0.4603 0.15735 -0.4603
+0.4603 0.18135 -0.4603
+0.4603 0.18225 -0.4434
+0.4603 0.20085 -0.4446
+0.4603 0.20225 -0.4252
+0.4603 0.21825 -0.4249
+0.4603 0.21805 -0.4061
+0.4603 0.23675 -0.406
+0.4603 0.25555 -0.3872
+0.4415 0.25555 -0.3685
+0.4227 0.25555 -0.3497
+0.4039 0.25555 -0.3309
+0.3851 0.25555 -0.3121
+0.3851 0.25555 -0.2933
+0.3851 0.25555 -0.2745
+0.3851 0.25555 -0.2557
+0.3851 0.25555 -0.2369
+0.3663 0.25555 -0.2181
+0.3475 0.25555 -0.1993
+0.3287 0.25555 -0.1805
+0.3099 0.25555 -0.1617
+0.3099 0.25555 -0.1458
+0.2911 0.25555 -0.1241
+0.2723 0.25555 -0.1054
+0.2723 0.25555 -0.0866
+0.2723 0.25555 -0.0678
+0.2723 0.25555 -0.049
+0.2724 0.25555 -0.0302
+0.2543 0.25555 -0.0113
+0.2554 0.25555 0.0094
+0.2572 0.25555 0.0296
+0.2611 0.25555 0.0476
+0.2643 0.25555 0.0579
+0.2574 0.25555 0.0655
+0.2417 0.25555 0.0826
+0.2365 0.25555 0.1163
+0.2592 0.25555 0.1375
+0.2796 0.25555 0.1562
+0.301 0.25555 0.1757
+0.3221 0.25555 0.1949
+0.3411 0.25555 0.2132
+0.3617 0.25555 0.2322
+0.382 0.25555 0.2512
+0.402 0.25555 0.2703
+0.422 0.25555 0.2704
+0.4217 0.25555 0.2892
+0.422 0.25555 0.3084
+0.4028 0.25555 0.3087
+0.3837 0.25555 0.3092
+0.3773 0.23765 0.3095
+0.3722 0.22715 0.3098
+0.3622 0.22625 0.3101
+0.3451 0.22395 0.3108
+0.3271 0.22075 0.3117
+0.3076 0.21775 0.3129
+0.289 0.21345 0.3141
+0.2614 0.20715 0.3162
+0.2462 0.19215 0.3177
+0.2337 0.18445 0.319
+0.2133 0.17135 0.3212
+0.1969 0.15965 0.3233
+0.1842 0.15015 0.325
+0.1651 0.13525 0.3277
+0.1466 0.12055 0.3307
+0.1324 0.10595 0.3332
+0.119 0.09065 0.3359
+0.1063 0.07485 0.3387
+0.0944 0.05845 0.3416
+0.0833 0.04145 0.3446
+0.0731 0.02395 0.3476
+0.0648 0.01265 0.3499
+0.049 0.00035 0.3536
+0.0428 -0.02265 0.357
+0.0379 -0.03575 0.3593
+0.0485 -0.05145 0.3594
+0.049 -0.07655 0.3625
+0.0705 -0.08045 0.3596
+0.0658 -0.09835 0.3627
+0.062 -0.11635 0.3659
+0.062 -0.11945 0.3481
+0.059 -0.13765 0.3513
+0.0568 -0.15913 0.3367
+0.0555 -0.17753 0.3399
+0.0551 -0.19595 0.3432
+0.037 -0.19595 0.3432
+0.0182 -0.1956 0.3452
+-0.0009 -0.19535 0.3466
+-0.02 -0.19512 0.3479
+-0.0417 -0.19538 0.3464
+-0.0409 -0.19035 0.375
+-0.0683 -0.18913 0.3819
+-0.1001 -0.19132 0.3694
+-0.1248 -0.19229 0.364
+-0.1484 -0.19314 0.3591
+-0.1686 -0.19374 0.3557
+-0.1874 -0.1941 0.3537
+-0.1871 -0.19769 0.3333
+-0.2079 -0.20085 0.3154
+-0.2106 -0.20407 0.2972
+-0.2152 -0.20721 0.2794
+-0.2215 -0.21026 0.262
+-0.2294 -0.21319 0.2454
+-0.239 -0.21598 0.2296
+-0.2502 -0.21859 0.2148
+-0.2627 -0.22099 0.2012
+-0.2765 -0.22317 0.1888
+-0.2805 -0.22757 0.1639
+-0.2865 -0.23263 0.1351
+-0.3063 -0.23439 0.1252
+-0.3138 -0.2391 0.0984
+-0.3057 -0.24269 0.0782
+-0.2765 -0.245983 0.0595
+-0.2581 -0.245983 0.0595
+-0.2581 -0.249172 0.0414
+-0.2581 -0.252361 0.0233
+-0.2581 -0.25555 0.0052
+-0.2682 -0.23725 0.002
+-0.278 -0.21896 -0.001
+-0.288 -0.20072 -0.0039
+-0.2989 -0.18257 -0.0067
+-0.311 -0.16453 -0.0094
+-0.3245 -0.14665 -0.0119
+-0.3391 -0.12915 -0.0143
+-0.3547 -0.11185 -0.0165
+-0.371 -0.09485 -0.0186
+-0.38782 -0.07805 -0.0205
+-0.40514 -0.06115 -0.0224
+-0.42298 -0.04385 -0.0242
+-0.44135 -0.02565 -0.0261
+-0.43531 -0.02015 -0.0267
+-0.44232 -0.01605 -0.0273
+-0.44475 0.00055 -0.0287
+-0.43573 0.01385 -0.0302
+-0.44326 0.03115 -0.0313
+-0.44261 0.05135 -0.0328
+-0.4422 0.06785 -0.0339
+-0.44191 0.08665 -0.035
+-0.44174 0.10535 -0.036
+-0.4603 0.12415 -0.0366
+-0.4603 0.14285 -0.0374
+-0.4603 0.16165 -0.038
+-0.4603 0.18045 -0.0386
+-0.4603 0.19915 -0.039
+-0.4603 0.21795 -0.0393
+-0.44153 0.21795 -0.0397
+-0.42314 0.21805 -0.0407
+-0.40676 0.23735 -0.0425
+-0.40676 0.25555 -0.0425
+-0.40325 0.25555 -0.0247
+-0.38199 0.25555 -0.0085
+-0.36304 0.25555 -0.0118
+-0.3443 0.25555 -0.0157
+-0.3204 0.25555 -0.0015
+-0.3012 0.25555 -0.0071
+-0.2823 0.25555 -0.0134
+-0.264 0.25555 -0.0019
+-0.2617 0.25555 0.0135
+-0.2351 0.25555 0.0145
+-0.2092 0.25555 -0.0013
+-0.1854 0.25555 -0.0005
+-0.1654 0.25555 0.0008
+-0.1384 0.25555 -0.0138
+-0.1191 0.25555 -0.027
+-0.1012 0.25555 -0.041
+-0.0839 0.25555 -0.0559
+-0.0673 0.25555 -0.0699
+-0.0525 0.25555 -0.0875
+-0.0397 0.25555 -0.1061
+-0.0289 0.25555 -0.1208
+-0.0183 0.25555 -0.1344
+-0.0082 0.25555 -0.1472
+0.0021 0.25555 -0.1601
+0.0126 0.25555 -0.1743
+0.0228 0.25555 -0.1868
+0.0329 0.25555 -0.1991
+0.0433 0.25555 -0.211
+0.0541 0.25555 -0.2226
+0.0652 0.25555 -0.2338
+0.0766 0.25555 -0.2448
+0.0884 0.25555 -0.2554
+0.1005 0.25555 -0.2657
+0.1129 0.25555 -0.2756
+0.1059 0.25555 -0.2911
+0.1189 0.25555 -0.3009
+0.1322 0.25555 -0.3104
+0.1458 0.25555 -0.3194
+0.1597 0.25555 -0.328
+0.1536 0.25555 -0.344
+0.1682 0.25555 -0.3524
+0.1829 0.25555 -0.3605
+0.1978 0.25555 -0.3681
+0.213 0.25555 -0.3753
+0.2283 0.25555 -0.3821
+0.2237 0.25555 -0.3986
+0.218 0.25555 -0.4151
+0.2345 0.25555 -0.4216
+0.2281 0.25555 -0.4385
+0.2215 0.25555 -0.4558
+0.2391 0.25555 -0.4623
+0.2554 0.24325 -0.4678
+0.2715 0.23125 -0.4727
+0.2873 0.21945 -0.477
+0.3031 0.20785 -0.4809
+0.3191 0.19645 -0.4844
+0.3367 0.19685 -0.4879
+0.3532 0.18565 -0.4906
+0.3709 0.18615 -0.4933
+0.3888 0.18705 -0.4955
+0.406 0.17615 -0.4969
+0.423 0.16395 -0.4977
+0.4418 0.16085 -0.4805
+0.4603 0.16075 -0.4795
+0.4603 0.18285 -0.4793
+0.4603 0.20025 -0.4614
+0.4603 0.21885 -0.4625
+0.4603 0.21845 -0.4437
+0.4603 0.23675 -0.4248
+0.4603 0.25555 -0.406
+0.4415 0.25555 -0.3872
+0.4227 0.25555 -0.3685
+0.4039 0.25555 -0.3497
+0.3851 0.25555 -0.3309
+0.3663 0.25555 -0.3121
+0.3667 0.25555 -0.2951
+0.3663 0.25555 -0.2745
+0.3663 0.25555 -0.2557
+0.3663 0.25555 -0.2369
+0.3475 0.25555 -0.2181
+0.3287 0.25555 -0.1993
+0.3099 0.25555 -0.1805
+0.2914 0.25555 -0.165
+0.2917 0.25555 -0.1488
+0.2735 0.25555 -0.1274
+0.2536 0.25555 -0.1054
+0.2536 0.25555 -0.0866
+0.2537 0.25555 -0.0677
+0.2538 0.25555 -0.0489
+0.254 0.25555 -0.0301
+0.2338 0.25555 -0.0298
+0.2342 0.25555 -0.0087
+0.2379 0.25555 0.0127
+0.2413 0.25555 0.0331
+0.248 0.25555 0.0517
+0.2285 0.25555 0.0586
+0.2072 0.25555 0.0907
+0.2123 0.25555 0.1157
+0.2151 0.25555 0.1392
+0.2355 0.25555 0.138
+0.2572 0.25555 0.1573
+0.2788 0.25555 0.1763
+0.3001 0.25555 0.1952
+0.3212 0.25555 0.214
+0.3222 0.25555 0.2338
+0.3419 0.25555 0.2329
+0.3428 0.25555 0.2525
+0.3624 0.25555 0.2518
+0.3631 0.25555 0.2713
+0.3825 0.25555 0.2706
+0.3831 0.25555 0.29
+0.4024 0.25555 0.2895
+0.3638 0.25555 0.2906
+0.3645 0.25555 0.3099
+0.3629 0.23905 0.31
+0.3445 0.23785 0.3107
+0.3262 0.23625 0.3116
+0.3071 0.23625 0.3127
+0.2879 0.23635 0.3139
+0.2668 0.23565 0.3154
+0.2439 0.21495 0.3174
+0.2391 0.20185 0.3181
+0.2287 0.20025 0.319
+0.2028 0.19295 0.3217
+0.1883 0.17765 0.3236
+0.1753 0.16985 0.3252
+0.1509 0.16045 0.3284
+0.1321 0.13705 0.3316
+0.1174 0.12005 0.3344
+0.1035 0.10395 0.3372
+0.0904 0.08495 0.3403
+0.0786 0.06945 0.3431
+0.0684 0.05045 0.3461
+0.0602 0.03355 0.3487
+0.056 0.02325 0.3503
+0.0418 0.02205 0.3527
+0.0228 0.00135 0.3579
+0.0235 -0.01855 0.3598
+0.024 -0.03625 0.3617
+0.0246 -0.05395 0.3637
+0.0237 -0.07355 0.3663
+0.0457 -0.09495 0.3655
+0.0436 -0.11245 0.3683
+0.0406 -0.13145 0.3717
+0.059 -0.13455 0.3691
+0.0568 -0.15285 0.3724
+0.0568 -0.15598 0.3545
+0.0555 -0.17437 0.3578
+0.0551 -0.1928 0.361
+0.0375 -0.19248 0.3629
+0.0185 -0.19203 0.3655
+-0.0003 -0.19146 0.3686
+-0.0188 -0.19088 0.3719
+-0.0184 -0.18754 0.3909
+-0.0387 -0.18699 0.394
+-0.0579 -0.186 0.3996
+-0.0807 -0.18618 0.3985
+-0.1016 -0.18739 0.3918
+-0.1256 -0.18843 0.3858
+-0.1494 -0.18945 0.3801
+-0.1695 -0.19023 0.3757
+-0.1879 -0.19061 0.3735
+-0.207 -0.1941 0.3537
+-0.207 -0.19761 0.3338
+-0.2079 -0.18119 0.312
+-0.2106 -0.18428 0.2938
+-0.2152 -0.1873 0.2759
+-0.2215 -0.19023 0.2585
+-0.2295 -0.19303 0.2418
+-0.2391 -0.1957 0.226
+-0.2503 -0.19819 0.2112
+-0.2628 -0.2005 0.1975
+-0.2767 -0.20259 0.1851
+-0.2914 -0.22511 0.1779
+-0.2979 -0.22941 0.1534
+-0.3163 -0.23097 0.1446
+-0.3258 -0.23593 0.1165
+-0.3344 -0.24058 0.0901
+-0.3266 -0.24359 0.073
+-0.3224 -0.246471 0.0567
+-0.3008 -0.246196 0.0583
+-0.2979 -0.249248 0.041
+-0.2765 -0.249172 0.0414
+-0.2765 -0.252361 0.0233
+-0.2765 -0.25555 0.0052
+-0.2863 -0.23725 0.002
+-0.2957 -0.21897 -0.001
+-0.3055 -0.20074 -0.0039
+-0.3162 -0.1826 -0.0067
+-0.3283 -0.16462 -0.0094
+-0.3418 -0.14695 -0.0119
+-0.3566 -0.12955 -0.0142
+-0.37238 -0.11255 -0.0164
+-0.38887 -0.09575 -0.0185
+-0.40595 -0.07905 -0.0204
+-0.42353 -0.06205 -0.0223
+-0.44164 -0.04435 -0.0242
+-0.4603 -0.02565 -0.0261
+-0.4603 -0.00695 -0.0278
+-0.4603 0.01175 -0.0294
+-0.44617 0.01355 -0.0298
+-0.4603 0.03045 -0.0309
+-0.4603 0.04915 -0.0323
+-0.4603 0.06785 -0.0335
+-0.4603 0.08665 -0.0347
+-0.4603 0.10535 -0.0357
+-0.4603 0.10635 -0.017
+-0.4603 0.12495 -0.0178
+-0.4603 0.14365 -0.0186
+-0.4603 0.16225 -0.0192
+-0.4603 0.18095 -0.0196
+-0.4603 0.19955 -0.0199
+-0.4603 0.21825 -0.0199
+-0.4603 0.23675 -0.0395
+-0.44173 0.23675 -0.0398
+-0.42453 0.23715 -0.0408
+-0.42456 0.25555 -0.0409
+-0.42169 0.25555 -0.0228
+-0.40106 0.25555 -0.0059
+-0.39826 0.25555 0.0144
+-0.3763 0.25555 0.0107
+-0.3593 0.25555 0.0074
+-0.3425 0.25555 0.0037
+-0.3415 0.25555 0.0219
+-0.3128 0.25555 0.0232
+-0.2912 0.25555 0.0095
+-0.2777 0.25555 0.0024
+-0.2733 0.25555 0.0124
+-0.267 0.25555 0.0296
+-0.2463 0.25555 0.0403
+-0.2093 0.25555 0.0315
+-0.1948 0.25555 0.0133
+-0.1749 0.25555 0.0258
+-0.1367 0.25555 0.0224
+-0.1207 0.25555 0.0064
+-0.1038 0.25555 -0.0102
+-0.0853 0.25555 -0.025
+-0.0678 0.25555 -0.0412
+-0.0507 0.25555 -0.0572
+-0.0392 0.25555 -0.077
+-0.0263 0.25555 -0.0955
+-0.0165 0.25555 -0.1088
+-0.007 0.25555 -0.1217
+0.0027 0.25555 -0.1341
+0.0128 0.25555 -0.1465
+0.0239 0.25555 -0.1609
+0.0338 0.25555 -0.1732
+0.0437 0.25555 -0.1847
+0.0539 0.25555 -0.1961
+0.0644 0.25555 -0.2074
+0.0751 0.25555 -0.2183
+0.0861 0.25555 -0.229
+0.0975 0.25555 -0.2394
+0.1092 0.25555 -0.2494
+0.121 0.25555 -0.2593
+0.133 0.25555 -0.2683
+0.1255 0.25555 -0.2852
+0.1384 0.25555 -0.2944
+0.1515 0.25555 -0.3033
+0.1665 0.25555 -0.3119
+0.18 0.25555 -0.3195
+0.1738 0.25555 -0.3363
+0.188 0.25555 -0.3442
+0.2024 0.25555 -0.3517
+0.217 0.25555 -0.3587
+0.2328 0.25555 -0.364
+0.2476 0.25555 -0.37
+0.2438 0.25555 -0.3884
+0.2397 0.25555 -0.405
+0.2559 0.25555 -0.4111
+0.2512 0.25555 -0.4278
+0.2453 0.25555 -0.4448
+0.2627 0.25555 -0.4506
+0.257 0.25555 -0.4683
+0.2732 0.24335 -0.4732
+0.2891 0.23145 -0.4776
+0.3048 0.21975 -0.4815
+0.3206 0.20825 -0.4849
+0.3382 0.20855 -0.4884
+0.3543 0.19725 -0.491
+0.372 0.19775 -0.4937
+0.3897 0.19905 -0.4958
+0.4066 0.18885 -0.4972
+0.4241 0.17785 -0.4981
+0.4413 0.16405 -0.4985
+0.4603 0.16415 -0.4988
+0.4603 0.18235 -0.4992
+0.4603 0.20105 -0.4802
+0.4603 0.21915 -0.4809
+0.4603 0.23695 -0.4624
+0.4603 0.23685 -0.4436
+0.4603 0.25555 -0.4248
+0.4415 0.25555 -0.406
+0.4227 0.25555 -0.3872
+0.4039 0.25555 -0.3685
+0.3851 0.25555 -0.3497
+0.3663 0.25555 -0.3309
+0.3494 0.25555 -0.3137
+0.3483 0.25555 -0.2973
+0.3474 0.25555 -0.2771
+0.3475 0.25555 -0.2557
+0.3475 0.25555 -0.2369
+0.3287 0.25555 -0.2181
+0.3099 0.25555 -0.1993
+0.2911 0.25555 -0.1805
+0.2779 0.25555 -0.1673
+0.2761 0.25555 -0.1524
+0.2552 0.25555 -0.1302
+0.2363 0.25555 -0.1282
+0.2356 0.25555 -0.1079
+0.2363 0.25555 -0.0893
+0.2358 0.25555 -0.0704
+0.2361 0.25555 -0.0513
+0.2183 0.25555 -0.0536
+0.2137 0.25555 -0.033
+0.2044 0.25555 -0.0079
+0.2132 0.25555 0.0204
+0.2198 0.25555 0.0385
+0.2004 0.25555 0.0669
+0.1839 0.25555 0.0937
+0.1895 0.25555 0.1175
+0.1947 0.25555 0.1403
+0.1962 0.25555 0.1614
+0.2166 0.25555 0.16
+0.2369 0.25555 0.1586
+0.2385 0.25555 0.1791
+0.2586 0.25555 0.1777
+0.2601 0.25555 0.1978
+0.2801 0.25555 0.1964
+0.2814 0.25555 0.2164
+0.3013 0.25555 0.2151
+0.3025 0.25555 0.2349
+0.3232 0.25555 0.2535
+0.3437 0.25555 0.272
+0.3445 0.25555 0.2914
+0.3454 0.25555 0.3106
+0.3262 0.25555 0.3116
+0.3071 0.25555 0.3126
+0.2879 0.25555 0.3138
+0.2688 0.25555 0.3152
+0.2497 0.25555 0.3166
+0.2476 0.23595 0.3169
+0.2306 0.23635 0.3183
+0.2306 0.21745 0.3186
+0.2081 0.21715 0.3206
+0.1889 0.21725 0.3226
+0.1854 0.19905 0.3233
+0.1809 0.18735 0.3241
+0.171 0.18605 0.3252
+0.1528 0.18415 0.3274
+0.1276 0.16735 0.331
+0.1165 0.15135 0.3331
+0.1006 0.13485 0.336
+0.0862 0.11345 0.3391
+0.0742 0.09395 0.342
+0.0655 0.07875 0.3444
+0.049 0.06405 0.3481
+0.0421 0.03825 0.3512
+0.0228 0.02225 0.3559
+0.0026 0.00455 0.3611
+0.0026 -0.01465 0.3631
+0.0026 -0.03385 0.3652
+0.0026 -0.05305 0.3675
+0.0026 -0.07225 0.3699
+0.0217 -0.09185 0.3691
+0.0217 -0.11105 0.3718
+0.0217 -0.13025 0.3747
+0.0385 -0.15065 0.375
+0.0555 -0.17123 0.3756
+0.0551 -0.18966 0.3789
+0.0376 -0.18917 0.3817
+0.0201 -0.18865 0.3845
+0.0026 -0.18812 0.3876
+0.0026 -0.18428 0.40939
+-0.018 -0.18428 0.40941
+-0.0368 -0.184 0.41095
+-0.0551 -0.18357 0.41337
+-0.0694 -0.18504 0.40512
+-0.0849 -0.18365 0.41301
+-0.1035 -0.18386 0.41176
+-0.1267 -0.18482 0.40632
+-0.1504 -0.18572 0.40123
+-0.1712 -0.18643 0.3972
+-0.1887 -0.1871 0.3934
+-0.207 -0.19059 0.3736
+-0.207 -0.17149 0.3703
+-0.207 -0.17477 0.3503
+-0.207 -0.17805 0.3304
+-0.2079 -0.16152 0.3088
+-0.2106 -0.16445 0.2905
+-0.2152 -0.16734 0.2726
+-0.2215 -0.17013 0.2552
+-0.2295 -0.17282 0.2385
+-0.2391 -0.17536 0.2226
+-0.2503 -0.17774 0.2078
+-0.2628 -0.17996 0.1941
+-0.2767 -0.18194 0.1817
+-0.2917 -0.18371 0.1707
+-0.2917 -0.20443 0.1741
+-0.3075 -0.22677 0.1684
+-0.3244 -0.22818 0.1605
+-0.3351 -0.23222 0.1375
+-0.3458 -0.23742 0.108
+-0.3492 -0.24226 0.0806
+-0.3412 -0.2445 0.0679
+-0.3386 -0.2467 0.0554
+-0.3361 -0.249202 0.0412
+-0.3187 -0.249248 0.0409
+-0.315 -0.252437 0.0229
+-0.2949 -0.252361 0.0233
+-0.2949 -0.25555 0.0052
+-0.3044 -0.23727 0.002
+-0.3135 -0.21899 -0.001
+-0.323 -0.20077 -0.0039
+-0.3336 -0.18267 -0.0067
+-0.3456 -0.16482 -0.0093
+-0.3591 -0.14735 -0.0118
+-0.37403 -0.13025 -0.0141
+-0.39 -0.11355 -0.0163
+-0.40674 -0.09705 -0.0183
+-0.42406 -0.08035 -0.0203
+-0.44191 -0.06305 -0.0222
+-0.4603 -0.04495 -0.0241
+-0.4603 -0.02385 -0.0074
+-0.4603 -0.00535 -0.0091
+-0.4603 0.01325 -0.0107
+-0.4603 0.03185 -0.0122
+-0.4603 0.05045 -0.0136
+-0.4603 0.06905 -0.0148
+-0.4603 0.08775 -0.016
+-0.4603 0.08875 0.0026
+-0.4603 0.10725 0.0017
+-0.4603 0.12575 0.0008
+-0.4603 0.14425 0.0001
+-0.4603 0.16275 -0.0004
+-0.4603 0.18125 -0.0007
+-0.4603 0.19955 -0.0005
+-0.4603 0.21755 0.0003
+-0.4603 0.23535 0.0017
+-0.4603 0.23685 -0.0197
+-0.4603 0.25555 -0.0395
+-0.44242 0.25555 -0.0399
+-0.4402 0.25555 -0.0212
+-0.41986 0.25555 -0.0032
+-0.41959 0.25555 0.018
+-0.39725 0.25555 0.0381
+-0.36826 0.25555 0.0311
+-0.356 0.25555 0.019
+-0.3465 0.25555 0.042
+-0.3255 0.25555 0.0523
+-0.2921 0.25555 0.0383
+-0.2802 0.25555 0.0217
+-0.2755 0.25555 0.0496
+-0.2563 0.25555 0.0639
+-0.2226 0.25555 0.0541
+-0.1851 0.25555 0.0476
+-0.1476 0.25555 0.0432
+-0.1145 0.25555 0.0372
+-0.1047 0.25555 0.0192
+-0.0904 0.25555 0.0016
+-0.0718 0.25555 -0.0126
+-0.0526 0.25555 -0.029
+-0.0362 0.25555 -0.0464
+-0.0245 0.25555 -0.0663
+-0.0134 0.25555 -0.0844
+-0.0048 0.25555 -0.0968
+0.0039 0.25555 -0.1091
+0.0126 0.25555 -0.1218
+0.0226 0.25555 -0.134
+0.0342 0.25555 -0.1484
+0.0438 0.25555 -0.1606
+0.0532 0.25555 -0.1725
+0.0629 0.25555 -0.1837
+0.0729 0.25555 -0.1948
+0.0832 0.25555 -0.2056
+0.0938 0.25555 -0.2161
+0.1049 0.25555 -0.2262
+0.1162 0.25555 -0.2361
+0.1278 0.25555 -0.2456
+0.1395 0.25555 -0.2538
+0.1513 0.25555 -0.262
+0.1452 0.25555 -0.277
+0.1585 0.25555 -0.2858
+0.1729 0.25555 -0.2952
+0.186 0.25555 -0.3031
+0.1992 0.25555 -0.3107
+0.1938 0.25555 -0.327
+0.2077 0.25555 -0.3342
+0.2219 0.25555 -0.3408
+0.2362 0.25555 -0.3472
+0.2511 0.25555 -0.3535
+0.2657 0.25555 -0.3591
+0.2628 0.25555 -0.3757
+0.2594 0.25555 -0.3943
+0.275 0.25555 -0.3998
+0.2721 0.25555 -0.4167
+0.268 0.25555 -0.4335
+0.285 0.25555 -0.4387
+0.2801 0.25555 -0.4559
+0.275 0.25555 -0.4738
+0.291 0.24355 -0.4781
+0.3068 0.23165 -0.482
+0.3224 0.22005 -0.4854
+0.3401 0.22035 -0.4889
+0.3558 0.20895 -0.4915
+0.3734 0.20945 -0.494
+0.3908 0.21085 -0.4961
+0.4073 0.20155 -0.4975
+0.4244 0.19185 -0.4984
+0.4421 0.18005 -0.4989
+0.4423 0.19595 -0.4992
+0.4603 0.20065 -0.4996
+0.4603 0.21895 -0.4998
+0.4603 0.23705 -0.4812
+0.4603 0.25555 -0.4624
+0.4603 0.25555 -0.4436
+0.4415 0.25555 -0.4248
+0.4227 0.25555 -0.406
+0.4039 0.25555 -0.3872
+0.3851 0.25555 -0.3685
+0.3663 0.25555 -0.3497
+0.3505 0.25555 -0.3337
+0.3347 0.25555 -0.3323
+0.333 0.25555 -0.3163
+0.3289 0.25555 -0.3019
+0.3192 0.25555 -0.2823
+0.3247 0.25555 -0.2545
+0.3272 0.25555 -0.2355
+0.3068 0.25555 -0.2185
+0.2911 0.25555 -0.1993
+0.2775 0.25555 -0.1808
+0.2682 0.25555 -0.1694
+0.2587 0.25555 -0.1587
+0.2349 0.25555 -0.1518
+0.217 0.25555 -0.1472
+0.2193 0.25555 -0.126
+0.2186 0.25555 -0.1072
+0.2186 0.25555 -0.089
+0.2185 0.25555 -0.0713
+0.1963 0.25555 -0.0695
+0.1941 0.25555 -0.0535
+0.1902 0.25555 -0.038
+0.1827 0.25555 -0.022
+0.1813 0.25555 0.0082
+0.188 0.25555 0.0257
+0.1937 0.25555 0.044
+0.1783 0.25555 0.07
+0.1598 0.25555 0.0954
+0.165 0.25555 0.1193
+0.1715 0.25555 0.143
+0.1757 0.25555 0.163
+0.1778 0.25555 0.1844
+0.1981 0.25555 0.1825
+0.2183 0.25555 0.1808
+0.2201 0.25555 0.201
+0.2401 0.25555 0.1993
+0.2417 0.25555 0.2193
+0.2616 0.25555 0.2178
+0.2828 0.25555 0.2362
+0.3036 0.25555 0.2546
+0.3242 0.25555 0.273
+0.3252 0.25555 0.2923
+0.3059 0.25555 0.2934
+0.2866 0.25555 0.2946
+0.2674 0.25555 0.296
+0.2481 0.25555 0.2975
+0.2289 0.25555 0.2991
+0.2306 0.25555 0.3183
+0.2115 0.25555 0.32
+0.2115 0.23635 0.3201
+0.1924 0.23635 0.322
+0.1733 0.23635 0.3241
+0.1733 0.21715 0.3243
+0.1725 0.20075 0.3247
+0.1532 0.19995 0.3269
+0.133 0.19915 0.3294
+0.131 0.18395 0.33
+0.1172 0.17255 0.3322
+0.1052 0.16535 0.3341
+0.0808 0.15265 0.338
+0.06 0.12535 0.3425
+0.0575 0.10025 0.3442
+0.0553 0.08795 0.3454
+0.0418 0.08395 0.3478
+0.0217 0.06215 0.3527
+0.0218 0.04085 0.3544
+0.0026 0.02385 0.3593
+0.0026 0.02475 0.3814
+0.0026 0.00565 0.3833
+0.0026 -0.01345 0.3852
+0.0026 -0.03235 0.3873
+0.0026 -0.05145 0.3896
+0.0026 -0.07045 0.392
+0.0026 -0.09145 0.3724
+0.0026 -0.11055 0.3751
+0.0026 -0.12975 0.378
+0.0214 -0.14935 0.3777
+0.0376 -0.17066 0.3784
+0.0201 -0.16947 0.3812
+0.0026 -0.16789 0.3841
+0.0026 -0.16537 0.40613
+0.0026 -0.18043 0.43123
+-0.0186 -0.18043 0.43123
+-0.0385 -0.18051 0.4308
+-0.0545 -0.18043 0.4312
+-0.0697 -0.18322 0.41536
+-0.0872 -0.18008 0.43323
+-0.1043 -0.18034 0.43178
+-0.1308 -0.1795 0.43646
+-0.1561 -0.18151 0.42506
+-0.1742 -0.18252 0.41939
+-0.1905 -0.18315 0.41583
+-0.207 -0.18359 0.4133
+-0.207 -0.1871 0.3934
+-0.207 -0.16821 0.3902
+-0.207 -0.14915 0.387
+-0.207 -0.15225 0.3671
+-0.207 -0.15535 0.3471
+-0.207 -0.1585 0.3272
+-0.2079 -0.14185 0.3057
+-0.2106 -0.14465 0.2874
+-0.2152 -0.14735 0.2695
+-0.2215 -0.15005 0.252
+-0.2295 -0.15255 0.2353
+-0.2391 -0.15505 0.2194
+-0.2503 -0.15728 0.2046
+-0.2628 -0.15937 0.1909
+-0.2767 -0.16128 0.1785
+-0.2917 -0.16296 0.1674
+-0.3078 -0.16442 0.1579
+-0.3078 -0.18524 0.1612
+-0.3078 -0.20603 0.1646
+-0.3247 -0.20736 0.1567
+-0.3419 -0.22927 0.1542
+-0.3549 -0.2331 0.1325
+-0.37509 -0.23399 0.1275
+-0.36817 -0.23913 0.0983
+-0.3792 -0.24292 0.0768
+-0.3591 -0.2443 0.069
+-0.3502 -0.2452 0.0639
+-0.3503 -0.246852 0.0545
+-0.35 -0.249141 0.0416
+-0.35 -0.252346 0.0234
+-0.3333 -0.252483 0.0226
+-0.3317 -0.25555 0.0052
+-0.3133 -0.25555 0.0052
+-0.3224 -0.23727 0.002
+-0.3313 -0.219 -0.001
+-0.3405 -0.20082 -0.0039
+-0.3508 -0.18283 -0.0067
+-0.36275 -0.1652 -0.0093
+-0.37633 -0.14805 -0.0117
+-0.39141 -0.13145 -0.014
+-0.40761 -0.11515 -0.0161
+-0.42459 -0.09875 -0.0181
+-0.44217 -0.08185 -0.0201
+-0.4603 -0.06415 -0.0221
+-0.4603 -0.04295 -0.0053
+-0.4603 -0.02195 0.0111
+-0.4603 -0.00365 0.0094
+-0.4603 0.01485 0.0078
+-0.4603 0.03325 0.0063
+-0.4603 0.05175 0.005
+-0.4603 0.07025 0.0037
+-0.4603 0.08835 0.0235
+-0.4603 0.10675 0.0225
+-0.4603 0.12505 0.0216
+-0.4603 0.14315 0.021
+-0.4603 0.16125 0.0209
+-0.4603 0.17945 0.0216
+-0.4603 0.19825 0.0239
+-0.4603 0.21725 0.024
+-0.4603 0.23515 0.0223
+-0.4603 0.25555 0.0218
+-0.4603 0.25555 0.0013
+-0.4603 0.25555 -0.0191
+-0.44043 0.25555 -0.0008
+-0.43958 0.25555 0.0218
+-0.41814 0.25555 0.0421
+-0.39644 0.25555 0.0619
+-0.37358 0.25555 0.0619
+-0.3525 0.25555 0.0668
+-0.3321 0.25555 0.074
+-0.3027 0.25555 0.0598
+-0.2851 0.25555 0.0719
+-0.2658 0.25555 0.0857
+-0.2365 0.25555 0.0802
+-0.1999 0.25555 0.0708
+-0.1584 0.25555 0.0657
+-0.1221 0.25555 0.0546
+-0.1022 0.25555 0.0587
+-0.0983 0.25555 0.0449
+-0.0909 0.25555 0.0279
+-0.0786 0.25555 0.0122
+-0.0593 0.25555 0
+-0.0392 0.25555 -0.0172
+-0.0227 0.25555 -0.0356
+-0.0107 0.25555 -0.0562
+-0.0007 0.25555 -0.0737
+0.0068 0.25555 -0.085
+0.0148 0.25555 -0.0963
+0.0234 0.25555 -0.1084
+0.0335 0.25555 -0.1208
+0.0455 0.25555 -0.1361
+0.0548 0.25555 -0.1481
+0.0637 0.25555 -0.1598
+0.073 0.25555 -0.1707
+0.0826 0.25555 -0.1816
+0.0926 0.25555 -0.1921
+0.1028 0.25555 -0.2022
+0.1134 0.25555 -0.2121
+0.1243 0.25555 -0.2217
+0.1356 0.25555 -0.2311
+0.1463 0.25555 -0.2384
+0.1579 0.25555 -0.2459
+0.1719 0.25555 -0.2555
+0.1644 0.25555 -0.2716
+0.1784 0.25555 -0.2818
+0.1911 0.25555 -0.2895
+0.2039 0.25555 -0.2969
+0.2169 0.25555 -0.304
+0.2126 0.25555 -0.3178
+0.2262 0.25555 -0.3246
+0.2396 0.25555 -0.3312
+0.2543 0.25555 -0.3377
+0.2685 0.25555 -0.3441
+0.2811 0.25555 -0.3488
+0.2794 0.25555 -0.3639
+0.2774 0.25555 -0.3808
+0.2925 0.25555 -0.3857
+0.2908 0.25555 -0.4048
+0.2885 0.25555 -0.4218
+0.305 0.25555 -0.4264
+0.302 0.25555 -0.4435
+0.2978 0.25555 -0.4608
+0.2931 0.25555 -0.4787
+0.309 0.24365 -0.4826
+0.3246 0.23185 -0.486
+0.3424 0.23205 -0.4894
+0.3577 0.22055 -0.4919
+0.3753 0.22115 -0.4944
+0.3924 0.22235 -0.4964
+0.4078 0.21365 -0.4977
+0.4246 0.20575 -0.4987
+0.4421 0.21175 -0.4995
+0.4413 0.22655 -0.4996
+0.4603 0.23725 -0.4999
+0.4603 0.25555 -0.4812
+0.4415 0.25555 -0.4624
+0.4415 0.25555 -0.4436
+0.4227 0.25555 -0.4248
+0.4039 0.25555 -0.406
+0.3851 0.25555 -0.3872
+0.3675 0.25555 -0.3844
+0.3663 0.25555 -0.3685
+0.3526 0.25555 -0.3662
+0.3518 0.25555 -0.3504
+0.3355 0.25555 -0.3471
+0.3209 0.25555 -0.3442
+0.3203 0.25555 -0.3309
+0.3189 0.25555 -0.3187
+0.3154 0.25555 -0.3079
+0.3055 0.25555 -0.2979
+0.2875 0.25555 -0.2713
+0.2997 0.25555 -0.2495
+0.304 0.25555 -0.2338
+0.2895 0.25555 -0.2191
+0.276 0.25555 -0.1988
+0.2614 0.25555 -0.1949
+0.2647 0.25555 -0.1796
+0.251 0.25555 -0.1732
+0.233 0.25555 -0.1682
+0.2137 0.25555 -0.1645
+0.1994 0.25555 -0.1426
+0.1999 0.25555 -0.1238
+0.199 0.25555 -0.1048
+0.1978 0.25555 -0.0867
+0.1831 0.25555 -0.0852
+0.1815 0.25555 -0.0684
+0.1789 0.25555 -0.0535
+0.1761 0.25555 -0.0426
+0.17 0.25555 -0.0321
+0.1557 0.25555 -0.0187
+0.1698 0.25555 -0.0046
+0.1642 0.25555 0.0176
+0.1682 0.25555 0.0307
+0.1728 0.25555 0.0475
+0.1548 0.25555 0.0718
+0.1364 0.25555 0.1031
+0.1437 0.25555 0.1229
+0.1498 0.25555 0.1465
+0.1545 0.25555 0.1649
+0.1574 0.25555 0.1866
+0.1599 0.25555 0.2072
+0.18 0.25555 0.2049
+0.2001 0.25555 0.2029
+0.202 0.25555 0.2229
+0.2219 0.25555 0.221
+0.2433 0.25555 0.2391
+0.2631 0.25555 0.2375
+0.2841 0.25555 0.2558
+0.3048 0.25555 0.274
+0.2854 0.25555 0.2753
+0.2659 0.25555 0.2766
+0.2465 0.25555 0.2781
+0.2271 0.25555 0.2798
+0.2078 0.25555 0.2816
+0.2096 0.25555 0.3009
+0.1904 0.25555 0.3028
+0.1924 0.25555 0.3219
+0.1733 0.25555 0.324
+0.1543 0.25555 0.3262
+0.1543 0.23635 0.3263
+0.1543 0.21715 0.3265
+0.1353 0.21715 0.3288
+0.1162 0.21715 0.3313
+0.1163 0.19815 0.3316
+0.1164 0.18415 0.332
+0.1003 0.18155 0.3342
+0.0784 0.17935 0.3375
+0.0553 0.15745 0.3418
+0.0494 0.14175 0.3434
+0.0415 0.12195 0.3456
+0.0408 0.10045 0.3469
+0.0217 0.10075 0.3501
+0.0217 0.08145 0.3513
+0.0026 0.06235 0.3561
+0.0026 0.04305 0.3576
+0.0026 0.04395 0.3798
+0.0026 0.02695 0.4035
+0.0026 0.00795 0.40532
+0.0026 -0.01095 0.40726
+0.0026 -0.02975 0.40935
+0.0026 -0.04865 0.41159
+0.0026 -0.06755 0.41395
+0.0026 -0.08645 0.41649
+0.0026 -0.08955 0.3945
+0.0026 -0.10845 0.3972
+0.0026 -0.12745 0.40005
+0.0026 -0.14885 0.381
+0.0026 -0.14645 0.40303
+0.0026 -0.16168 0.42798
+0.0026 -0.17658 0.45306
+-0.0191 -0.17719 0.44955
+-0.0391 -0.17712 0.45001
+-0.0539 -0.17707 0.45027
+-0.0695 -0.18014 0.43284
+-0.0857 -0.177 0.45073
+-0.1033 -0.17738 0.44856
+-0.1222 -0.1766 0.45297
+-0.1344 -0.17581 0.45746
+-0.1445 -0.17694 0.45105
+-0.1615 -0.17828 0.44345
+-0.177 -0.17913 0.43858
+-0.1918 -0.1797 0.43536
+-0.207 -0.18008 0.43317
+-0.207 -0.16165 0.43001
+-0.207 -0.16493 0.41008
+-0.207 -0.14595 0.40697
+-0.207 -0.12705 0.40399
+-0.207 -0.13005 0.384
+-0.207 -0.13295 0.3641
+-0.207 -0.13595 0.3441
+-0.207 -0.13895 0.3242
+-0.2079 -0.12205 0.3027
+-0.2106 -0.12475 0.2844
+-0.2152 -0.12735 0.2665
+-0.2215 -0.12985 0.249
+-0.2295 -0.13235 0.2323
+-0.2391 -0.13465 0.2164
+-0.2503 -0.13675 0.2015
+-0.2628 -0.13875 0.1878
+-0.2767 -0.14055 0.1754
+-0.2917 -0.14215 0.1643
+-0.3078 -0.14355 0.1548
+-0.3247 -0.14475 0.1469
+-0.3247 -0.16563 0.15
+-0.3247 -0.1865 0.1533
+-0.3423 -0.18751 0.1471
+-0.3423 -0.20841 0.1505
+-0.36 -0.23007 0.1497
+-0.37848 -0.23056 0.147
+-0.39716 -0.23416 0.1266
+-0.39757 -0.23823 0.1034
+-0.3998 -0.2418 0.0832
+-0.4029 -0.24543 0.0625
+-0.38387 -0.246059 0.059
+-0.36327 -0.246608 0.0559
+-0.36582 -0.249156 0.0415
+-0.36841 -0.25233 0.0234
+-0.36841 -0.25555 0.0052
+-0.35 -0.25555 0.0052
+-0.3405 -0.23727 0.002
+-0.349 -0.21902 -0.001
+-0.3579 -0.20089 -0.0039
+-0.36791 -0.1831 -0.0066
+-0.3797 -0.16589 -0.0092
+-0.39339 -0.14935 -0.0115
+-0.40871 -0.13315 -0.0137
+-0.42519 -0.11705 -0.0158
+-0.44245 -0.10055 -0.0179
+-0.4603 -0.08335 -0.0199
+-0.4603 -0.06205 -0.0031
+-0.4603 -0.04095 0.0134
+-0.4603 -0.02185 0.0318
+-0.4603 -0.00355 0.0302
+-0.4603 0.01485 0.0286
+-0.4603 0.03325 0.0271
+-0.4603 0.05155 0.0258
+-0.4603 0.06995 0.0246
+-0.4603 0.08905 0.0487
+-0.4603 0.10745 0.0478
+-0.4603 0.12575 0.0471
+-0.4603 0.14415 0.0464
+-0.4603 0.16275 0.0463
+-0.4603 0.18115 0.0462
+-0.4603 0.19915 0.0473
+-0.4603 0.21745 0.046
+-0.4603 0.23545 0.043
+-0.4603 0.25555 0.0422
+-0.43969 0.25555 0.0422
+-0.41863 0.25555 0.0623
+-0.39716 0.25555 0.0827
+-0.377 0.25555 0.0843
+-0.3566 0.25555 0.0876
+-0.3366 0.25555 0.0927
+-0.3109 0.25555 0.0825
+-0.2931 0.25555 0.092
+-0.275 0.25555 0.1043
+-0.2473 0.25555 0.1004
+-0.2168 0.25555 0.0968
+-0.1782 0.25555 0.0905
+-0.1579 0.25555 0.1131
+-0.1281 0.25555 0.0828
+-0.1024 0.25555 0.0786
+-0.0858 0.25555 0.074
+-0.0877 0.25555 0.0601
+-0.0849 0.25555 0.0492
+-0.0786 0.25555 0.0352
+-0.0695 0.25555 0.0234
+-0.0478 0.25555 0.0172
+-0.024 0.25555 -0.0045
+-0.0083 0.25555 -0.0246
+0.004 0.25555 -0.0477
+0.0103 0.25555 -0.0647
+0.0174 0.25555 -0.0736
+0.0253 0.25555 -0.0836
+0.0344 0.25555 -0.0952
+0.0446 0.25555 -0.1078
+0.0563 0.25555 -0.1235
+0.0654 0.25555 -0.1352
+0.0745 0.25555 -0.1457
+0.0839 0.25555 -0.1566
+0.0933 0.25555 -0.1675
+0.1028 0.25555 -0.1783
+0.1114 0.25555 -0.1878
+0.1218 0.25555 -0.1982
+0.1318 0.25555 -0.2068
+0.1431 0.25555 -0.2157
+0.1531 0.25555 -0.2233
+0.1643 0.25555 -0.2313
+0.1807 0.25555 -0.2427
+0.1938 0.25555 -0.2512
+0.186 0.25555 -0.266
+0.198 0.25555 -0.2733
+0.2103 0.25555 -0.2811
+0.222 0.25555 -0.2886
+0.236 0.25555 -0.2969
+0.2301 0.25555 -0.3106
+0.2448 0.25555 -0.3183
+0.2593 0.25555 -0.3255
+0.2718 0.25555 -0.3317
+0.2831 0.25555 -0.3355
+0.2963 0.25555 -0.339
+0.295 0.25555 -0.3533
+0.294 0.25555 -0.3686
+0.3081 0.25555 -0.3727
+0.3075 0.25555 -0.3902
+0.3067 0.25555 -0.4094
+0.3225 0.25555 -0.4136
+0.3216 0.25555 -0.4306
+0.3192 0.25555 -0.4477
+0.3155 0.25555 -0.4651
+0.3114 0.25555 -0.4832
+0.327 0.24365 -0.4865
+0.3451 0.24375 -0.49
+0.3602 0.23225 -0.4924
+0.3779 0.23255 -0.4948
+0.3948 0.23325 -0.4967
+0.4084 0.22475 -0.4978
+0.4243 0.21865 -0.4988
+0.423 0.22935 -0.4989
+0.4384 0.23875 -0.4996
+0.4603 0.25555 -0.5
+0.4415 0.25555 -0.4997
+0.4415 0.25555 -0.4812
+0.4237 0.25555 -0.4805
+0.4229 0.25555 -0.4624
+0.4227 0.25555 -0.4436
+0.4042 0.25555 -0.4437
+0.4039 0.25555 -0.4248
+0.3856 0.25555 -0.425
+0.3851 0.25555 -0.406
+0.3687 0.25555 -0.4033
+0.3536 0.25555 -0.4007
+0.3529 0.25555 -0.3821
+0.337 0.25555 -0.3794
+0.3363 0.25555 -0.3632
+0.3216 0.25555 -0.3596
+0.3083 0.25555 -0.3566
+0.3086 0.25555 -0.3417
+0.3089 0.25555 -0.3299
+0.3084 0.25555 -0.3199
+0.308 0.25555 -0.3125
+0.2999 0.25555 -0.3089
+0.2869 0.25555 -0.3061
+0.2872 0.25555 -0.2893
+0.2765 0.25555 -0.2758
+0.2761 0.25555 -0.2592
+0.2849 0.25555 -0.2444
+0.2884 0.25555 -0.2324
+0.2793 0.25555 -0.2318
+0.2736 0.25555 -0.2205
+0.2562 0.25555 -0.2106
+0.2404 0.25555 -0.2033
+0.2456 0.25555 -0.1878
+0.2291 0.25555 -0.1827
+0.2104 0.25555 -0.1792
+0.1945 0.25555 -0.1633
+0.1816 0.25555 -0.1404
+0.1837 0.25555 -0.1216
+0.1838 0.25555 -0.103
+0.1688 0.25555 -0.1008
+0.1704 0.25555 -0.0837
+0.1697 0.25555 -0.0674
+0.1687 0.25555 -0.0535
+0.1691 0.25555 -0.0462
+0.1618 0.25555 -0.0414
+0.1464 0.25555 -0.0335
+0.1424 0.25555 -0.0087
+0.1563 0.25555 0.0061
+0.1552 0.25555 0.0223
+0.1543 0.25555 0.0327
+0.1541 0.25555 0.0501
+0.1331 0.25555 0.0472
+0.1254 0.25555 0.0747
+0.0977 0.25555 0.0978
+0.1105 0.25555 0.1163
+0.1199 0.25555 0.1331
+0.1256 0.25555 0.1507
+0.1308 0.25555 0.1694
+0.1366 0.25555 0.1894
+0.1397 0.25555 0.21
+0.1624 0.25555 0.2272
+0.1822 0.25555 0.2249
+0.204 0.25555 0.2426
+0.2237 0.25555 0.2408
+0.2449 0.25555 0.2587
+0.2645 0.25555 0.2572
+0.2254 0.25555 0.2604
+0.2059 0.25555 0.2622
+0.1864 0.25555 0.2641
+0.1884 0.25555 0.2836
+0.169 0.25555 0.2856
+0.1712 0.25555 0.3049
+0.152 0.25555 0.3071
+0.1328 0.25555 0.3095
+0.1352 0.25555 0.3285
+0.1352 0.23635 0.3286
+0.1162 0.23635 0.3311
+0.0972 0.23635 0.3337
+0.0972 0.21715 0.3339
+0.0973 0.19805 0.3342
+0.0783 0.19805 0.337
+0.0594 0.19805 0.3399
+0.0596 0.17895 0.3403
+0.0408 0.17795 0.3434
+0.0408 0.15855 0.3441
+0.0382 0.14065 0.3452
+0.0217 0.13935 0.3481
+0.0217 0.12005 0.349
+0.0026 0.10095 0.3534
+0.0026 0.08165 0.3547
+0.0026 0.08215 0.3769
+0.0026 0.06305 0.3782
+0.0026 0.04595 0.40184
+0.0026 0.02915 0.42557
+0.0026 0.01035 0.42735
+0.0026 -0.00845 0.42927
+0.0026 -0.02725 0.43135
+0.0026 -0.04595 0.43356
+0.0026 -0.06465 0.43593
+0.0026 -0.08335 0.43843
+0.0026 -0.10205 0.44108
+0.0026 -0.10525 0.41913
+0.0026 -0.12405 0.42195
+0.0026 -0.14285 0.4249
+0.0026 -0.15798 0.44983
+-0.001 -0.17341 0.47099
+-0.0204 -0.17449 0.46489
+-0.0368 -0.17417 0.4667
+-0.0513 -0.17417 0.46672
+-0.0677 -0.17417 0.4667
+-0.0693 -0.17671 0.45238
+-0.0845 -0.17431 0.46594
+-0.1022 -0.17442 0.46535
+-0.1203 -0.17422 0.4664
+-0.1357 -0.17408 0.46718
+-0.1489 -0.17478 0.46327
+-0.1659 -0.17552 0.45911
+-0.1799 -0.17626 0.45483
+-0.1927 -0.17665 0.45265
+-0.207 -0.17658 0.45306
+-0.207 -0.15797 0.44983
+-0.207 -0.14285 0.42689
+-0.207 -0.12405 0.42393
+-0.207 -0.10525 0.42114
+-0.207 -0.10805 0.40115
+-0.207 -0.11085 0.3812
+-0.207 -0.11375 0.3612
+-0.207 -0.11655 0.3412
+-0.207 -0.11935 0.3212
+-0.2079 -0.10235 0.3
+-0.2106 -0.10485 0.2816
+-0.2152 -0.10735 0.2636
+-0.2215 -0.10975 0.2462
+-0.2295 -0.11205 0.2294
+-0.2391 -0.11425 0.2135
+-0.2503 -0.11625 0.1986
+-0.2628 -0.11815 0.1849
+-0.2767 -0.11985 0.1725
+-0.2917 -0.12135 0.1614
+-0.3078 -0.12275 0.1519
+-0.3247 -0.12375 0.1439
+-0.3423 -0.12465 0.1377
+-0.3423 -0.14565 0.1407
+-0.3423 -0.16657 0.1438
+-0.36046 -0.16726 0.1393
+-0.36046 -0.18823 0.1426
+-0.36046 -0.20916 0.146
+-0.37897 -0.20962 0.1433
+-0.39766 -0.23072 0.146
+-0.41854 -0.23072 0.146
+-0.41854 -0.23426 0.1259
+-0.41878 -0.23783 0.1057
+-0.41905 -0.24137 0.0856
+-0.41989 -0.24496 0.0653
+-0.40489 -0.248943 0.0426
+-0.38664 -0.24905 0.0421
+-0.38678 -0.252315 0.0235
+-0.38678 -0.25555 0.0052
+-0.37636 -0.23725 0.002
+-0.3585 -0.23727 0.002
+-0.36649 -0.21902 -0.001
+-0.3749 -0.201 -0.0039
+-0.38458 -0.18357 -0.0066
+-0.39633 -0.167 -0.009
+-0.41027 -0.15115 -0.0113
+-0.42595 -0.13545 -0.0134
+-0.44275 -0.11935 -0.0156
+-0.4603 -0.10255 -0.0176
+-0.4603 -0.08115 -0.0009
+-0.4603 -0.05995 0.0157
+-0.4603 -0.04005 0.0341
+-0.4603 -0.02105 0.0572
+-0.4603 -0.00295 0.0556
+-0.4603 0.01525 0.054
+-0.4603 0.03365 0.0525
+-0.4603 0.05205 0.0511
+-0.4603 0.07065 0.0499
+-0.4603 0.07235 0.073
+-0.4603 0.09065 0.0718
+-0.4603 0.10895 0.0708
+-0.4603 0.12725 0.07
+-0.4603 0.14555 0.0692
+-0.4603 0.16395 0.0685
+-0.4603 0.18225 0.068
+-0.4603 0.20055 0.0676
+-0.4603 0.21865 0.0667
+-0.4603 0.23615 0.0639
+-0.4603 0.25555 0.0626
+-0.43969 0.25555 0.0626
+-0.41854 0.25555 0.0828
+-0.39766 0.25555 0.1035
+-0.37897 0.25555 0.1044
+-0.36046 0.25555 0.1072
+-0.3423 0.25555 0.1117
+-0.3171 0.25555 0.0998
+-0.3004 0.25555 0.1094
+-0.2828 0.25555 0.1197
+-0.2575 0.25555 0.1175
+-0.23 0.25555 0.1166
+-0.1976 0.25555 0.1148
+-0.1796 0.25555 0.1353
+-0.1394 0.25555 0.1378
+-0.1133 0.25555 0.1188
+-0.0918 0.25555 0.1052
+-0.0748 0.25555 0.0946
+-0.0608 0.25555 0.0828
+-0.0739 0.25555 0.068
+-0.0787 0.25555 0.0601
+-0.0746 0.25555 0.0529
+-0.0685 0.25555 0.0415
+-0.0638 0.25555 0.034
+-0.0516 0.25555 0.0349
+-0.0299 0.25555 0.0448
+-0.0184 0.25555 0.0289
+-0.0041 0.25555 0.0103
+0.0078 0.25555 -0.0118
+0.0222 0.25555 -0.0431
+0.02 0.25555 -0.0597
+0.027 0.25555 -0.0648
+0.035 0.25555 -0.0721
+0.0445 0.25555 -0.0842
+0.0545 0.25555 -0.0972
+0.0658 0.25555 -0.1112
+0.0748 0.25555 -0.1224
+0.084 0.25555 -0.1336
+0.0934 0.25555 -0.1447
+0.1026 0.25555 -0.1554
+0.1119 0.25555 -0.1663
+0.1195 0.25555 -0.1757
+0.1288 0.25555 -0.1854
+0.1381 0.25555 -0.1933
+0.1498 0.25555 -0.2016
+0.1596 0.25555 -0.2086
+0.1743 0.25555 -0.2178
+0.192 0.25555 -0.2304
+0.2043 0.25555 -0.2394
+0.2053 0.25555 -0.2586
+0.2171 0.25555 -0.2656
+0.2286 0.25555 -0.2737
+0.2429 0.25555 -0.2835
+0.2573 0.25555 -0.2929
+0.2499 0.25555 -0.3054
+0.266 0.25555 -0.3149
+0.2761 0.25555 -0.3221
+0.2848 0.25555 -0.3251
+0.2972 0.25555 -0.3283
+0.298 0.25555 -0.3185
+0.2864 0.25555 -0.3158
+0.2799 0.25555 -0.3152
+0.2753 0.25555 -0.305
+0.2754 0.25555 -0.2848
+0.2631 0.25555 -0.2793
+0.2692 0.25555 -0.2695
+0.2584 0.25555 -0.2601
+0.2671 0.25555 -0.2502
+0.274 0.25555 -0.2395
+0.2624 0.25555 -0.2322
+0.2476 0.25555 -0.2238
+0.2339 0.25555 -0.2176
+0.2224 0.25555 -0.2143
+0.2243 0.25555 -0.1976
+0.2084 0.25555 -0.1944
+0.1923 0.25555 -0.1767
+0.1811 0.25555 -0.1751
+0.1762 0.25555 -0.1639
+0.1583 0.25555 -0.1548
+0.1625 0.25555 -0.1393
+0.1669 0.25555 -0.1194
+0.1519 0.25555 -0.0982
+0.1566 0.25555 -0.0818
+0.157 0.25555 -0.0659
+0.1584 0.25555 -0.0522
+0.1427 0.25555 -0.0488
+0.1268 0.25555 -0.0465
+0.1319 0.25555 -0.0274
+0.1286 0.25555 0.0025
+0.1463 0.25555 0.0161
+0.1398 0.25555 0.0298
+0.1172 0.25555 0.0188
+0.1067 0.25555 0.0371
+0.0953 0.25555 0.0568
+0.0788 0.25555 0.0744
+0.0632 0.25555 0.0938
+0.0815 0.25555 0.1155
+0.0934 0.25555 0.13
+0.1016 0.25555 0.1423
+0.1062 0.25555 0.1562
+0.1103 0.25555 0.1725
+0.1153 0.25555 0.1934
+0.1194 0.25555 0.2135
+0.1425 0.25555 0.2297
+0.1646 0.25555 0.2468
+0.1843 0.25555 0.2446
+0.1669 0.25555 0.2663
+0.1474 0.25555 0.2686
+0.1497 0.25555 0.2879
+0.1304 0.25555 0.2903
+0.1111 0.25555 0.2928
+0.1137 0.25555 0.312
+0.1162 0.25555 0.331
+0.0972 0.25555 0.3336
+0.0783 0.23635 0.3364
+0.0783 0.21715 0.3367
+0.0593 0.21715 0.3395
+0.0407 0.19795 0.3429
+0.0217 0.19745 0.3461
+0.0217 0.17805 0.3466
+0.0217 0.15875 0.3472
+0.0026 0.13955 0.3514
+0.0026 0.12025 0.3523
+0.0026 0.12045 0.3745
+0.0026 0.10125 0.3756
+0.0026 0.08385 0.399
+0.0026 0.06485 0.40033
+0.0026 0.04795 0.42392
+-0.0011 0.03095 0.44408
+-0.0011 0.01225 0.44585
+-0.0011 -0.00635 0.44777
+-0.0011 -0.02505 0.44983
+-0.0011 -0.04365 0.45203
+-0.0011 -0.06225 0.45438
+-0.0011 -0.08075 0.45685
+-0.0011 -0.09935 0.45949
+-0.0011 -0.11785 0.46225
+0.0026 -0.12075 0.44385
+0.0026 -0.13935 0.44678
+-0.0011 -0.15485 0.4682
+-0.0113 -0.17074 0.48622
+-0.0231 -0.17257 0.47577
+-0.0346 -0.17192 0.47952
+-0.0484 -0.1713 0.48297
+-0.0655 -0.17112 0.48398
+-0.0837 -0.17114 0.48389
+-0.103 -0.17126 0.48322
+-0.1212 -0.17138 0.48256
+-0.1382 -0.17159 0.48134
+-0.1538 -0.1719 0.47961
+-0.1699 -0.1725 0.47621
+-0.183 -0.17352 0.47037
+-0.1926 -0.17446 0.46507
+-0.2034 -0.17341 0.47104
+-0.2034 -0.15495 0.46779
+-0.2034 -0.13645 0.46475
+-0.207 -0.13935 0.44676
+-0.207 -0.12065 0.44385
+-0.207 -0.10205 0.44107
+-0.207 -0.08635 0.41847
+-0.207 -0.08905 0.3985
+-0.207 -0.09175 0.3785
+-0.207 -0.09435 0.3585
+-0.207 -0.09705 0.3385
+-0.207 -0.09975 0.3185
+-0.2079 -0.08255 0.2973
+-0.2106 -0.08495 0.2789
+-0.2152 -0.08725 0.261
+-0.2215 -0.08955 0.2435
+-0.2295 -0.09175 0.2267
+-0.2391 -0.09385 0.2108
+-0.2503 -0.09575 0.1959
+-0.2628 -0.09755 0.1821
+-0.2767 -0.09915 0.1697
+-0.2917 -0.10055 0.1586
+-0.3078 -0.10185 0.1491
+-0.3247 -0.10285 0.1412
+-0.3423 -0.10365 0.1349
+-0.36046 -0.10425 0.1304
+-0.36046 -0.12525 0.1332
+-0.36046 -0.14625 0.1362
+-0.37897 -0.14665 0.1334
+-0.37897 -0.16767 0.1366
+-0.37897 -0.18867 0.1399
+-0.39766 -0.18881 0.139
+-0.39766 -0.20977 0.1424
+-0.4151 -0.20828 0.1422
+-0.42743 -0.21163 0.1427
+-0.43943 -0.23072 0.146
+-0.43959 -0.23426 0.1259
+-0.43972 -0.23782 0.1058
+-0.43978 -0.24136 0.0857
+-0.43982 -0.2449 0.0655
+-0.42278 -0.248744 0.0438
+-0.42354 -0.252224 0.024
+-0.40517 -0.252285 0.0237
+-0.40517 -0.25555 0.0052
+-0.39403 -0.23715 0.002
+-0.3837 -0.21897 -0.001
+-0.39133 -0.20114 -0.0039
+-0.40064 -0.18437 -0.0064
+-0.41257 -0.16876 -0.0088
+-0.427 -0.15355 -0.0109
+-0.44315 -0.13805 -0.0131
+-0.4603 -0.12175 -0.0153
+-0.4603 -0.10025 0.0014
+-0.4603 -0.07895 0.018
+-0.4603 -0.05855 0.0366
+-0.4603 -0.03915 0.0591
+-0.4603 -0.01915 0.0804
+-0.4603 -0.00085 0.0786
+-0.4603 0.01745 0.0771
+-0.4603 0.03575 0.0756
+-0.4603 0.05405 0.0742
+-0.4603 0.07345 0.0912
+-0.4603 0.09165 0.0901
+-0.4603 0.10985 0.0891
+-0.4603 0.12805 0.0882
+-0.4603 0.14625 0.0875
+-0.4603 0.16445 0.0868
+-0.4603 0.18275 0.0863
+-0.4603 0.20095 0.0859
+-0.4603 0.21915 0.0856
+-0.4603 0.23735 0.0849
+-0.4603 0.25555 0.0831
+-0.43943 0.25555 0.0829
+-0.41854 0.25555 0.1035
+-0.41507 0.23345 0.1036
+-0.39766 0.23425 0.1036
+-0.37897 0.23425 0.1045
+-0.36046 0.23425 0.1072
+-0.3423 0.23435 0.1118
+-0.3247 0.23435 0.1181
+-0.3246 0.25555 0.118
+-0.3077 0.25555 0.126
+-0.2917 0.25555 0.1357
+-0.2663 0.25555 0.1318
+-0.2412 0.25555 0.1322
+-0.2129 0.25555 0.1333
+-0.1968 0.25555 0.1517
+-0.1636 0.25555 0.157
+-0.1275 0.25555 0.1626
+-0.1012 0.25555 0.1434
+-0.0798 0.25555 0.1278
+-0.0627 0.25555 0.1141
+-0.0448 0.25555 0.0984
+-0.0339 0.25555 0.0839
+-0.0496 0.25555 0.071
+-0.065 0.25555 0.0598
+-0.0569 0.25555 0.0483
+-0.0396 0.25555 0.0587
+-0.0237 0.25555 0.0716
+-0.0111 0.25555 0.0545
+-0.0004 0.25555 0.0394
+0.0123 0.25555 0.0217
+0.0243 0.25555 0.0032
+0.04 0.25555 -0.0217
+0.0505 0.25555 -0.0367
+0.0357 0.25555 -0.0517
+0.0465 0.25555 -0.0612
+0.0552 0.25555 -0.0725
+0.0651 0.25555 -0.0862
+0.0753 0.25555 -0.0996
+0.0838 0.25555 -0.1101
+0.0938 0.25555 -0.1215
+0.1032 0.25555 -0.1322
+0.1121 0.25555 -0.143
+0.1214 0.25555 -0.1543
+0.1289 0.25555 -0.1632
+0.1371 0.25555 -0.1709
+0.1459 0.25555 -0.1787
+0.157 0.25555 -0.188
+0.1666 0.25555 -0.1952
+0.1841 0.25555 -0.205
+0.2071 0.25555 -0.2159
+0.2149 0.25555 -0.229
+0.2139 0.25555 -0.246
+0.2248 0.25555 -0.2524
+0.2366 0.25555 -0.2604
+0.2502 0.25555 -0.2702
+0.2456 0.25555 -0.2504
+0.2538 0.25555 -0.2416
+0.2406 0.25555 -0.2334
+0.229 0.25555 -0.2273
+0.2224 0.25555 -0.2239
+0.223 0.25555 -0.2349
+0.234 0.25555 -0.2417
+0.1888 0.25555 -0.19
+0.1752 0.25555 -0.1835
+0.1644 0.25555 -0.1753
+0.1519 0.25555 -0.1665
+0.1475 0.25555 -0.151
+0.1488 0.25555 -0.1387
+0.1451 0.25555 -0.1189
+0.1339 0.25555 -0.1096
+0.1359 0.25555 -0.0951
+0.1379 0.25555 -0.079
+0.1394 0.25555 -0.0632
+0.1233 0.25555 -0.0617
+0.1098 0.25555 -0.0448
+0.1111 0.25555 -0.0203
+0.0983 0.25555 0.003
+0.0874 0.25555 0.022
+0.0748 0.25555 0.0429
+0.0627 0.25555 0.0582
+0.0478 0.25555 0.0769
+0.0343 0.25555 0.0917
+0.0495 0.25555 0.1099
+0.0684 0.25555 0.1318
+0.0817 0.25555 0.1417
+0.0913 0.25555 0.1474
+0.0914 0.25555 0.1587
+0.0927 0.25555 0.1765
+0.0958 0.25555 0.1985
+0.0988 0.25555 0.2183
+0.1226 0.25555 0.2327
+0.145 0.25555 0.2492
+0.1254 0.25555 0.2518
+0.1279 0.25555 0.271
+0.1085 0.25555 0.2737
+0.089 0.25555 0.2767
+0.0918 0.25555 0.2956
+0.0945 0.25555 0.3146
+0.0754 0.25555 0.3174
+0.0783 0.25555 0.3364
+0.0593 0.23635 0.3393
+0.0404 0.23635 0.3423
+0.0405 0.21705 0.3426
+0.0216 0.21675 0.3457
+0.0026 0.19755 0.3494
+0.0026 0.17815 0.3499
+0.0026 0.15885 0.3506
+0.0026 0.15875 0.3728
+0.0026 0.13955 0.3736
+0.0026 0.12175 0.3967
+0.0026 0.10285 0.3977
+0.0026 0.08555 0.42107
+0.0026 0.06675 0.42241
+-0.0011 0.04965 0.44243
+-0.0115 0.03235 0.45929
+-0.0115 0.01375 0.46104
+-0.0115 -0.00475 0.46295
+-0.0115 -0.02325 0.465
+-0.0115 -0.04175 0.46718
+-0.0115 -0.06025 0.46951
+-0.0115 -0.07865 0.47197
+-0.0115 -0.09715 0.47458
+-0.0115 -0.11555 0.47731
+-0.0115 -0.13395 0.48021
+-0.0011 -0.13635 0.46515
+-0.0115 -0.15235 0.48322
+-0.0267 -0.16894 0.496414
+-0.045 -0.1683 0.5
+-0.0641 -0.1683 0.5
+-0.0831 -0.1683 0.5
+-0.1022 -0.1683 0.5
+-0.1212 -0.1683 0.5
+-0.1403 -0.1683 0.5
+-0.1593 -0.1683 0.5
+-0.1775 -0.16894 0.496414
+-0.193 -0.17073 0.4863
+-0.1931 -0.15235 0.48302
+-0.1931 -0.13395 0.47998
+-0.1931 -0.11555 0.4771
+-0.2034 -0.11795 0.46185
+-0.2034 -0.09935 0.45908
+-0.2034 -0.08085 0.45645
+-0.207 -0.08335 0.43843
+-0.207 -0.06745 0.41595
+-0.207 -0.06995 0.3959
+-0.207 -0.07255 0.3759
+-0.207 -0.07505 0.3559
+-0.207 -0.07765 0.3359
+-0.207 -0.08015 0.3158
+-0.2079 -0.06275 0.2948
+-0.2106 -0.06505 0.2765
+-0.2152 -0.06725 0.2584
+-0.2215 -0.06935 0.2409
+-0.2295 -0.07145 0.2241
+-0.2391 -0.07335 0.2082
+-0.2503 -0.07515 0.1933
+-0.2628 -0.07685 0.1795
+-0.2767 -0.07835 0.1671
+-0.2917 -0.07975 0.156
+-0.3078 -0.08085 0.1464
+-0.3247 -0.08185 0.1385
+-0.3423 -0.08265 0.1322
+-0.36046 -0.08315 0.1277
+-0.37897 -0.08355 0.125
+-0.37897 -0.10455 0.1277
+-0.37897 -0.12565 0.1305
+-0.39766 -0.12575 0.1296
+-0.39766 -0.14675 0.1325
+-0.39766 -0.16783 0.1357
+-0.41063 -0.17822 0.1373
+-0.41164 -0.19529 0.14
+-0.42371 -0.19621 0.1402
+-0.43286 -0.19564 0.1401
+-0.44324 -0.2125 0.1429
+-0.4603 -0.23072 0.146
+-0.4603 -0.23426 0.1259
+-0.4603 -0.23782 0.1058
+-0.4603 -0.24136 0.0857
+-0.4603 -0.2449 0.0655
+-0.4403 -0.248455 0.0454
+-0.44191 -0.252132 0.0245
+-0.44193 -0.25555 0.0052
+-0.42354 -0.25555 0.0052
+-0.41115 -0.23683 0.002
+-0.40018 -0.21876 -0.0011
+-0.40676 -0.20135 -0.0038
+-0.41593 -0.18577 -0.0062
+-0.42852 -0.17137 -0.0084
+-0.44368 -0.15666 -0.0105
+-0.4603 -0.14095 -0.0127
+-0.4603 -0.11925 0.0038
+-0.4603 -0.09785 0.0204
+-0.4603 -0.07715 0.0392
+-0.4603 -0.05735 0.0612
+-0.4603 -0.03735 0.0822
+-0.4603 -0.01735 0.0986
+-0.4603 0.00085 0.0969
+-0.4603 0.01895 0.0953
+-0.4603 0.03715 0.0938
+-0.4603 0.05535 0.0924
+-0.4603 0.07465 0.1093
+-0.4603 0.09275 0.1082
+-0.4603 0.11085 0.1072
+-0.4603 0.12895 0.1063
+-0.4603 0.14695 0.1056
+-0.4603 0.16505 0.105
+-0.4603 0.18315 0.1044
+-0.4603 0.20125 0.104
+-0.4603 0.21935 0.1037
+-0.4603 0.23745 0.1035
+-0.4603 0.25555 0.1035
+-0.43943 0.25555 0.1035
+-0.42739 0.23675 0.1035
+-0.42363 0.22145 0.1037
+-0.41161 0.22015 0.1037
+-0.39766 0.21305 0.1038
+-0.37897 0.21305 0.1047
+-0.36046 0.21305 0.1075
+-0.3423 0.21315 0.112
+-0.3247 0.21325 0.1183
+-0.3078 0.21335 0.1263
+-0.3078 0.23445 0.1261
+-0.2917 0.23455 0.1357
+-0.2767 0.23455 0.1468
+-0.2766 0.25555 0.1468
+-0.251 0.25555 0.1455
+-0.2261 0.25555 0.1483
+-0.2123 0.25555 0.1659
+-0.1828 0.25555 0.1716
+-0.1504 0.25555 0.1785
+-0.117 0.25555 0.1862
+-0.0887 0.25555 0.1669
+-0.0673 0.25555 0.1505
+-0.0495 0.25555 0.1349
+-0.03 0.25555 0.1153
+-0.0166 0.25555 0.0982
+-0.008 0.25555 0.0845
+0.005 0.25555 0.0661
+0.0165 0.25555 0.0514
+0.0288 0.25555 0.0341
+0.0405 0.25555 0.0181
+0.0545 0.25555 -0.0063
+0.0651 0.25555 -0.0252
+0.0727 0.25555 -0.0376
+0.0588 0.25555 -0.0484
+0.0675 0.25555 -0.0603
+0.0772 0.25555 -0.0734
+0.0858 0.25555 -0.0865
+0.0933 0.25555 -0.0958
+0.1035 0.25555 -0.1097
+0.1125 0.25555 -0.1225
+0.1207 0.25555 -0.1344
+0.1292 0.25555 -0.1464
+0.1356 0.25555 -0.1544
+0.143 0.25555 -0.16
+0.1403 0.25555 -0.1493
+0.1379 0.25555 -0.1414
+0.1318 0.25555 -0.1275
+0.1234 0.25555 -0.1138
+0.116 0.25555 -0.0947
+0.1203 0.25555 -0.0754
+0.1073 0.25555 -0.0713
+0.1087 0.25555 -0.0604
+0.0976 0.25555 -0.0609
+0.0935 0.25555 -0.0468
+0.0878 0.25555 -0.0299
+0.08 0.25555 -0.013
+0.0689 0.25555 0.0074
+0.0565 0.25555 0.03
+0.0452 0.25555 0.046
+0.033 0.25555 0.0637
+0.0211 0.25555 0.0781
+0.0069 0.25555 0.0953
+0.0216 0.25555 0.106
+0.0348 0.25555 0.1257
+0.0549 0.25555 0.1526
+0.0767 0.25555 0.1572
+0.0766 0.25555 0.1803
+0.0778 0.25555 0.2029
+0.079 0.25555 0.2241
+0.1026 0.25555 0.2364
+0.1057 0.25555 0.2548
+0.0861 0.25555 0.2584
+0.0665 0.25555 0.2627
+0.0696 0.25555 0.2801
+0.0726 0.25555 0.2985
+0.0534 0.25555 0.3017
+0.0564 0.25555 0.3203
+0.0593 0.25555 0.3392
+0.0404 0.25555 0.3423
+0.0215 0.25555 0.3455
+0.0215 0.23635 0.3455
+0.0026 0.23615 0.3488
+0.0026 0.21685 0.349
+0.0026 0.21635 0.3712
+0.0026 0.19715 0.3716
+0.0026 0.17795 0.3721
+0.0026 0.15985 0.3949
+0.0026 0.14075 0.3957
+0.0026 0.12315 0.41875
+0.0026 0.10435 0.41984
+-0.0011 0.08695 0.43959
+-0.0011 0.06825 0.44095
+-0.0115 0.05085 0.45767
+-0.0269 0.03325 0.46945
+-0.0269 0.01475 0.47121
+-0.0269 -0.00365 0.4731
+-0.0269 -0.02215 0.47514
+-0.0269 -0.04055 0.47731
+-0.0269 -0.05895 0.47963
+-0.0269 -0.07735 0.48207
+-0.0269 -0.09565 0.48466
+-0.0269 -0.11405 0.48741
+-0.0269 -0.13235 0.49028
+-0.0269 -0.15065 0.493286
+-0.045 -0.15005 0.496857
+-0.0641 -0.15005 0.496857
+-0.0831 -0.15005 0.496857
+-0.1022 -0.15005 0.496857
+-0.1212 -0.15005 0.496857
+-0.1403 -0.15005 0.496857
+-0.1593 -0.15005 0.496857
+-0.1777 -0.15065 0.493194
+-0.1777 -0.13235 0.490173
+-0.1777 -0.11405 0.4873
+-0.1777 -0.09565 0.48457
+-0.1931 -0.09715 0.47435
+-0.1931 -0.07875 0.47174
+-0.1931 -0.06025 0.46928
+-0.2034 -0.06225 0.45396
+-0.207 -0.06455 0.43591
+-0.207 -0.04855 0.41357
+-0.207 -0.05095 0.3935
+-0.207 -0.05335 0.3735
+-0.207 -0.05575 0.3535
+-0.207 -0.05815 0.3334
+-0.207 -0.06055 0.3134
+-0.2079 -0.04295 0.2925
+-0.2106 -0.04505 0.2741
+-0.2152 -0.04715 0.2561
+-0.2215 -0.04915 0.2386
+-0.2295 -0.05105 0.2218
+-0.2391 -0.05285 0.2058
+-0.2503 -0.05465 0.1909
+-0.2628 -0.05615 0.1771
+-0.2767 -0.05765 0.1646
+-0.2917 -0.05885 0.1535
+-0.3078 -0.05995 0.144
+-0.3247 -0.06085 0.136
+-0.3423 -0.06155 0.1298
+-0.36046 -0.06215 0.1252
+-0.37897 -0.06245 0.1225
+-0.39766 -0.06255 0.1216
+-0.39766 -0.08365 0.1241
+-0.39766 -0.10465 0.1267
+-0.4102 -0.11995 0.1288
+-0.41024 -0.13985 0.1315
+-0.41033 -0.15942 0.1344
+-0.4229 -0.16967 0.136
+-0.42317 -0.18519 0.1384
+-0.43451 -0.18052 0.1376
+-0.44585 -0.19519 0.14
+-0.4603 -0.2129 0.1429
+-0.4603 -0.21615 0.1237
+-0.4603 -0.21815 0.1022
+-0.4603 -0.22135 0.0819
+-0.4603 -0.22512 0.0617
+-0.4603 -0.248455 0.0454
+-0.4603 -0.251995 0.0253
+-0.4603 -0.25555 0.0052
+-0.44008 -0.23288 0.0013
+-0.42705 -0.23587 0.0018
+-0.41529 -0.21818 -0.0012
+-0.42069 -0.20182 -0.0038
+-0.43056 -0.18829 -0.0059
+-0.4444 -0.17497 -0.0079
+-0.4603 -0.16007 -0.01
+-0.4603 -0.13835 0.0064
+-0.4603 -0.11685 0.0228
+-0.4603 -0.09565 0.0417
+-0.4603 -0.07555 0.0635
+-0.4603 -0.05565 0.0842
+-0.4603 -0.03545 0.1004
+-0.4603 -0.01555 0.1166
+-0.4603 0.00245 0.115
+-0.4603 0.02045 0.1134
+-0.4603 0.03855 0.1119
+-0.4603 0.05665 0.1106
+-0.44777 0.05715 0.1105
+-0.44777 0.07545 0.1093
+-0.44777 0.09385 0.1082
+-0.44777 0.11215 0.1072
+-0.44777 0.13055 0.1063
+-0.44776 0.14885 0.1055
+-0.44767 0.16725 0.1049
+-0.44745 0.18525 0.1044
+-0.44696 0.20285 0.104
+-0.44579 0.22005 0.1037
+-0.44323 0.23735 0.1035
+-0.43279 0.22085 0.1037
+-0.4231 0.21025 0.1038
+-0.41059 0.20265 0.104
+-0.39766 0.19175 0.1042
+-0.37897 0.19175 0.1051
+-0.36046 0.19185 0.1079
+-0.3423 0.19195 0.1124
+-0.3247 0.19215 0.1187
+-0.3078 0.19225 0.1267
+-0.2917 0.19255 0.1363
+-0.2917 0.21345 0.1359
+-0.2767 0.21365 0.1471
+-0.2628 0.21385 0.1597
+-0.2628 0.23465 0.1594
+-0.2628 0.25555 0.1594
+-0.2371 0.25555 0.1605
+-0.2248 0.25555 0.177
+-0.1998 0.25555 0.1852
+-0.1711 0.25555 0.1923
+-0.1404 0.25555 0.2016
+-0.1064 0.25555 0.2139
+-0.0754 0.25555 0.1959
+-0.0511 0.25555 0.1784
+-0.0331 0.25555 0.1617
+-0.0103 0.25555 0.136
+-0.0021 0.25555 0.1085
+0.0119 0.25555 0.1141
+0.0158 0.25555 0.1349
+0.0265 0.25555 0.1665
+0.0567 0.25555 0.1863
+0.0591 0.25555 0.2072
+0.0621 0.25555 0.2284
+0.0827 0.25555 0.2412
+0.0654 0.25555 0.2458
+0.0478 0.25555 0.2495
+0.0488 0.25555 0.2669
+0.0503 0.25555 0.2838
+0.0324 0.25555 0.2874
+0.0343 0.25555 0.3051
+0.0373 0.25555 0.3235
+0.0183 0.25555 0.3266
+0.0004 0.25555 0.3292
+0.0026 0.25555 0.3488
+0.0026 0.25555 0.3709
+0.0026 0.23545 0.371
+0.0026 0.21685 0.3934
+0.0026 0.19785 0.3937
+0.0026 0.17885 0.3943
+0.0026 0.16085 0.41704
+0.0026 0.14205 0.41785
+-0.0011 0.12435 0.43733
+-0.0011 0.10565 0.4384
+-0.0115 0.08805 0.45485
+-0.0115 0.06945 0.45618
+-0.0269 0.05175 0.46783
+-0.045 0.03355 0.47302
+-0.045 0.01515 0.47478
+-0.045 -0.00325 0.47668
+-0.045 -0.02175 0.4787
+-0.045 -0.04005 0.48088
+-0.045 -0.05845 0.4832
+-0.045 -0.07685 0.48564
+-0.045 -0.09515 0.48824
+-0.045 -0.11345 0.490967
+-0.045 -0.13175 0.493835
+-0.0641 -0.13175 0.493835
+-0.0831 -0.13175 0.493835
+-0.1022 -0.13175 0.493835
+-0.1212 -0.13175 0.493835
+-0.1403 -0.13175 0.493835
+-0.1593 -0.13175 0.493835
+-0.1593 -0.11345 0.490967
+-0.1593 -0.09515 0.48824
+-0.1593 -0.07685 0.48564
+-0.1777 -0.07735 0.48198
+-0.1777 -0.05895 0.47952
+-0.1777 -0.04055 0.47719
+-0.1931 -0.04175 0.46695
+-0.2034 -0.04365 0.45164
+-0.207 -0.04585 0.43356
+-0.207 -0.02965 0.41131
+-0.207 -0.03185 0.3913
+-0.207 -0.03415 0.3712
+-0.207 -0.03635 0.3512
+-0.207 -0.03865 0.3311
+-0.207 -0.04085 0.311
+-0.2079 -0.02315 0.2903
+-0.2106 -0.02515 0.2719
+-0.2152 -0.02705 0.2538
+-0.2215 -0.02895 0.2363
+-0.2295 -0.03075 0.2195
+-0.2391 -0.03245 0.2035
+-0.2503 -0.03405 0.1886
+-0.2628 -0.03545 0.1748
+-0.2767 -0.03685 0.1623
+-0.2917 -0.03795 0.1512
+-0.3078 -0.03905 0.1417
+-0.3247 -0.03985 0.1337
+-0.3423 -0.04055 0.1274
+-0.36046 -0.04105 0.1229
+-0.37897 -0.04125 0.1202
+-0.39766 -0.04145 0.1193
+-0.41019 -0.05955 0.1213
+-0.41019 -0.07975 0.1236
+-0.41019 -0.09995 0.1261
+-0.42272 -0.11465 0.1281
+-0.42273 -0.13355 0.1307
+-0.42278 -0.15215 0.1333
+-0.43502 -0.16415 0.1351
+-0.44704 -0.17823 0.1373
+-0.4603 -0.19506 0.14
+-0.4603 -0.19812 0.1213
+-0.4603 -0.20034 0.1011
+-0.4603 -0.20335 0.0795
+-0.4603 -0.20623 0.0572
+-0.4603 -0.22908 0.0415
+-0.4603 -0.23323 0.0211
+-0.4603 -0.23649 0.0019
+-0.44539 -0.22326 -0.0003
+-0.42772 -0.21684 -0.0014
+-0.43302 -0.20344 -0.0035
+-0.44527 -0.1927 -0.0052
+-0.4603 -0.17919 -0.0072
+-0.4603 -0.15734 0.009
+-0.4603 -0.13575 0.0254
+-0.4603 -0.13255 0.0465
+-0.4603 -0.11415 0.0441
+-0.4603 -0.09385 0.0658
+-0.4603 -0.07385 0.0862
+-0.4603 -0.05355 0.1023
+-0.4603 -0.03365 0.1185
+-0.44777 -0.01615 0.1167
+-0.44777 0.00215 0.115
+-0.44777 0.02045 0.1134
+-0.44777 0.03875 0.1119
+-0.43524 0.03935 0.1118
+-0.43524 0.05815 0.1105
+-0.43524 0.07695 0.1092
+-0.43524 0.09575 0.108
+-0.43524 0.11455 0.107
+-0.43524 0.13345 0.1062
+-0.43521 0.15225 0.1054
+-0.43512 0.17075 0.1048
+-0.43492 0.18875 0.1043
+-0.43441 0.20555 0.1039
+-0.42281 0.19435 0.1041
+-0.4103 0.18345 0.1044
+-0.39766 0.17055 0.1048
+-0.37897 0.17055 0.1057
+-0.36046 0.17065 0.1085
+-0.3423 0.17075 0.113
+-0.3247 0.17095 0.1193
+-0.3078 0.17125 0.1273
+-0.2917 0.17155 0.1369
+-0.2767 0.17185 0.148
+-0.2767 0.19275 0.1475
+-0.2628 0.19305 0.16
+-0.2503 0.19335 0.1739
+-0.2503 0.21405 0.1735
+-0.2503 0.23475 0.1733
+-0.2502 0.25555 0.1733
+-0.2391 0.25555 0.1883
+-0.2141 0.25555 0.1945
+-0.1899 0.25555 0.2048
+-0.1611 0.25555 0.2136
+-0.1301 0.25555 0.2269
+-0.0948 0.25555 0.247
+-0.0539 0.25555 0.2344
+-0.0311 0.25555 0.2083
+-0.0139 0.25555 0.1926
+0.005 0.25555 0.1771
+0.0348 0.25555 0.1944
+0.04 0.25555 0.2124
+0.0453 0.25555 0.2315
+0.0313 0.25555 0.2507
+0.0321 0.25555 0.2699
+0.0143 0.25555 0.2695
+0.0153 0.25555 0.2896
+0.0161 0.25555 0.3081
+-0.0012 0.25555 0.3095
+-0.0178 0.25555 0.3125
+-0.0186 0.25555 0.3296
+-0.0173 0.25555 0.3499
+-0.016 0.25555 0.3708
+-0.015 0.25555 0.3924
+0.0026 0.25555 0.3931
+0.0026 0.23595 0.3932
+0.0026 0.21745 0.41553
+0.0026 0.19865 0.41589
+0.0026 0.17975 0.4164
+-0.0011 0.16185 0.43561
+-0.0011 0.14315 0.43639
+-0.0115 0.12515 0.45261
+-0.0115 0.10665 0.45364
+-0.0269 0.08875 0.46503
+-0.0269 0.07025 0.46637
+-0.045 0.05205 0.47142
+-0.0641 0.03355 0.47302
+-0.0641 0.01515 0.47478
+-0.0641 -0.00325 0.47668
+-0.0641 -0.02175 0.4787
+-0.0641 -0.04005 0.48088
+-0.0641 -0.05845 0.4832
+-0.0641 -0.07685 0.48564
+-0.0641 -0.09515 0.48824
+-0.0641 -0.11345 0.490967
+-0.0831 -0.11345 0.490967
+-0.1022 -0.11345 0.490967
+-0.1212 -0.11345 0.490967
+-0.1403 -0.11345 0.490967
+-0.1403 -0.09515 0.48824
+-0.1403 -0.07685 0.48564
+-0.1403 -0.05845 0.4832
+-0.1593 -0.05845 0.4832
+-0.1593 -0.04005 0.48088
+-0.1593 -0.02175 0.4787
+-0.1777 -0.02215 0.47502
+-0.1931 -0.02325 0.46477
+-0.2034 -0.02505 0.44943
+-0.207 -0.02715 0.43133
+-0.207 -0.01075 0.40924
+-0.207 -0.01285 0.3892
+-0.207 -0.01485 0.3691
+-0.207 -0.01695 0.349
+-0.207 -0.01905 0.3289
+-0.207 -0.02115 0.3089
+-0.2079 -0.00335 0.2883
+-0.2106 -0.00515 0.2698
+-0.2152 -0.00695 0.2518
+-0.2215 -0.00865 0.2342
+-0.2295 -0.01035 0.2174
+-0.2391 -0.01195 0.2014
+-0.2503 -0.01345 0.1865
+-0.2628 -0.01475 0.1727
+-0.2767 -0.01595 0.1602
+-0.2917 -0.01705 0.1491
+-0.3078 -0.01805 0.1395
+-0.3247 -0.01885 0.1315
+-0.3423 -0.01945 0.1253
+-0.36046 -0.01995 0.1208
+-0.37897 -0.02015 0.118
+-0.39766 -0.02025 0.1171
+-0.41019 -0.03935 0.1191
+-0.42272 -0.05675 0.121
+-0.42272 -0.07615 0.1232
+-0.42272 -0.09545 0.1256
+-0.43523 -0.11035 0.1275
+-0.43521 -0.12875 0.13
+-0.43517 -0.14675 0.1325
+-0.4475 -0.16097 0.1347
+-0.4603 -0.17719 0.1371
+-0.4603 -0.18008 0.1187
+-0.4603 -0.18247 0.0996
+-0.4603 -0.1842 0.0782
+-0.4603 -0.18786 0.0543
+-0.4603 -0.19238 0.0336
+-0.4603 -0.21105 0.037
+-0.4603 -0.2143 0.0178
+-0.4603 -0.2174 -0.0013
+-0.446 -0.20921 -0.0026
+-0.43532 -0.2145 -0.0017
+-0.4603 -0.19831 -0.0043
+-0.4603 -0.17636 0.0118
+-0.4603 -0.15465 0.028
+-0.4603 -0.15095 0.0489
+-0.4603 -0.13015 0.0704
+-0.4603 -0.11205 0.068
+-0.4603 -0.09205 0.0884
+-0.4603 -0.08985 0.1065
+-0.4603 -0.07175 0.1044
+-0.4603 -0.06965 0.1224
+-0.4603 -0.05165 0.1204
+-0.44777 -0.03445 0.1185
+-0.43524 -0.01695 0.1168
+-0.43524 0.00185 0.115
+-0.43524 0.02055 0.1134
+-0.42272 0.02095 0.1133
+-0.42272 0.04035 0.1118
+-0.42272 0.05985 0.1103
+-0.42272 0.07925 0.109
+-0.42272 0.09875 0.1079
+-0.42272 0.11825 0.1069
+-0.42272 0.13775 0.106
+-0.42272 0.15715 0.1052
+-0.42272 0.17625 0.1046
+-0.41022 0.16355 0.105
+-0.39766 0.14935 0.1055
+-0.37897 0.14935 0.1064
+-0.36046 0.14945 0.1092
+-0.3423 0.14965 0.1137
+-0.3247 0.14985 0.12
+-0.3078 0.15015 0.128
+-0.2917 0.15055 0.1376
+-0.2767 0.15095 0.1488
+-0.2628 0.15145 0.1613
+-0.2628 0.17225 0.1606
+-0.2503 0.17265 0.1745
+-0.2391 0.17315 0.1895
+-0.2391 0.19375 0.1889
+-0.2391 0.21435 0.1885
+-0.2391 0.23495 0.1883
+-0.2295 0.23505 0.2044
+-0.2295 0.25555 0.2044
+-0.2052 0.25555 0.2129
+-0.1815 0.25555 0.225
+-0.1524 0.25555 0.2367
+-0.1212 0.25555 0.2556
+-0.0901 0.25555 0.2776
+-0.0578 0.25555 0.2731
+-0.0269 0.25555 0.245
+-0.0149 0.25555 0.227
+-0.0012 0.25555 0.2154
+0.0158 0.25555 0.2044
+0.0218 0.25555 0.22
+0.0272 0.25555 0.2342
+0.0109 0.25555 0.2405
+0.0131 0.25555 0.253
+0.001 0.25555 0.2533
+-0.0001 0.25555 0.2704
+-0.0018 0.25555 0.2898
+-0.0167 0.25555 0.2928
+-0.0351 0.25555 0.2975
+-0.0366 0.25555 0.3162
+-0.0376 0.25555 0.332
+-0.038 0.25555 0.3512
+-0.0365 0.25555 0.3722
+-0.0352 0.25555 0.3918
+-0.0335 0.25555 0.41113
+-0.0149 0.25555 0.41307
+0.0026 0.25555 0.41525
+0.0026 0.23635 0.41533
+-0.0011 0.21805 0.43413
+-0.0011 0.19925 0.43446
+-0.0011 0.18055 0.43497
+-0.0115 0.16245 0.4509
+-0.0115 0.14375 0.45169
+-0.0269 0.12575 0.46278
+-0.0269 0.10725 0.46384
+-0.045 0.08895 0.4686
+-0.045 0.07045 0.46992
+-0.0641 0.05205 0.47142
+-0.0831 0.03355 0.47302
+-0.0831 0.01515 0.47478
+-0.0831 -0.00325 0.47668
+-0.0831 -0.02175 0.4787
+-0.0831 -0.04005 0.48088
+-0.0831 -0.05845 0.4832
+-0.0831 -0.07685 0.48564
+-0.0831 -0.09515 0.48824
+-0.1022 -0.09515 0.48824
+-0.1212 -0.09515 0.48824
+-0.1212 -0.07685 0.48564
+-0.1212 -0.05845 0.4832
+-0.1212 -0.04005 0.48088
+-0.1403 -0.04005 0.48088
+-0.1403 -0.02175 0.4787
+-0.1403 -0.00325 0.47668
+-0.1593 -0.00325 0.47668
+-0.1777 -0.00365 0.47298
+-0.1931 -0.00475 0.46272
+-0.2034 -0.00645 0.44739
+-0.207 -0.00835 0.42927
+-0.207 0.00825 0.40727
+-0.207 0.00635 0.3872
+-0.207 0.00435 0.3671
+-0.207 0.00245 0.347
+-0.207 0.00045 0.3269
+-0.207 -0.00145 0.3069
+-0.2079 0.01655 0.2864
+-0.2106 0.01485 0.2679
+-0.2152 0.01315 0.2499
+-0.2215 0.01155 0.2323
+-0.2295 0.01005 0.2155
+-0.2391 0.00855 0.1995
+-0.2503 0.00725 0.1845
+-0.2628 0.00595 0.1707
+-0.2767 0.00485 0.1582
+-0.2917 0.00385 0.1471
+-0.3078 0.00295 0.1375
+-0.3247 0.00225 0.1295
+-0.3423 0.00165 0.1233
+-0.36046 0.00125 0.1187
+-0.37897 0.00095 0.116
+-0.39766 0.00085 0.1151
+-0.41019 -0.01905 0.117
+-0.42272 -0.03735 0.1188
+-0.43524 -0.05445 0.1207
+-0.43524 -0.07315 0.1228
+-0.43524 -0.09175 0.1251
+-0.44776 -0.10735 0.1271
+-0.44774 -0.12545 0.1295
+-0.44768 -0.14335 0.1321
+-0.4603 -0.15931 0.1344
+-0.4603 -0.16204 0.1162
+-0.4603 -0.16456 0.0976
+-0.4603 -0.16584 0.0758
+-0.4603 -0.16944 0.0513
+-0.4603 -0.17352 0.0308
+-0.4603 -0.19534 0.0147
+-0.4603 -0.14805 0.0729
+-0.4603 -0.12835 0.093
+-0.4603 -0.11025 0.0907
+-0.4603 -0.10785 0.1088
+-0.4603 -0.10555 0.1269
+-0.4603 -0.08755 0.1246
+-0.44777 -0.07095 0.1226
+-0.44777 -0.05275 0.1205
+-0.43524 -0.03565 0.1187
+-0.42272 -0.01795 0.1169
+-0.42272 0.00145 0.115
+-0.41019 0.00115 0.1151
+-0.41019 0.02145 0.1133
+-0.41019 0.04175 0.1117
+-0.41019 0.06205 0.1102
+-0.41019 0.08235 0.1089
+-0.41019 0.10265 0.1077
+-0.41019 0.12295 0.1066
+-0.41019 0.14325 0.1057
+-0.39766 0.12805 0.1064
+-0.37897 0.12815 0.1073
+-0.36046 0.12825 0.1101
+-0.3423 0.12845 0.1146
+-0.3247 0.12875 0.1209
+-0.3078 0.12905 0.1289
+-0.2917 0.12955 0.1385
+-0.2767 0.13005 0.1496
+-0.2628 0.13065 0.1622
+-0.2503 0.13125 0.176
+-0.2503 0.15195 0.1752
+-0.2391 0.15255 0.1902
+-0.2295 0.15315 0.2062
+-0.2295 0.17355 0.2055
+-0.2295 0.19405 0.205
+-0.2295 0.21455 0.2046
+-0.2215 0.21485 0.2215
+-0.2215 0.23515 0.2213
+-0.2215 0.25555 0.2213
+-0.1982 0.25555 0.2322
+-0.1742 0.25555 0.2459
+-0.145 0.25555 0.263
+-0.1174 0.25555 0.2819
+-0.1167 0.25555 0.309
+-0.0806 0.25555 0.312
+-0.057 0.25555 0.3035
+-0.0558 0.25555 0.3202
+-0.056 0.25555 0.3337
+-0.055 0.25555 0.354
+-0.0519 0.25555 0.3743
+-0.0501 0.25555 0.3916
+-0.0487 0.25555 0.40956
+-0.0472 0.25555 0.4261
+-0.0327 0.25555 0.42612
+-0.0183 0.25555 0.42735
+-0.0011 0.25555 0.43384
+-0.0011 0.23675 0.4339
+-0.0115 0.21825 0.44943
+-0.0115 0.19965 0.44978
+-0.0115 0.18105 0.45027
+-0.0269 0.16275 0.46109
+-0.0269 0.14425 0.46187
+-0.045 0.12595 0.46637
+-0.045 0.10745 0.46741
+-0.0641 0.08895 0.4686
+-0.0641 0.07045 0.46992
+-0.0831 0.05205 0.47142
+-0.1022 0.03355 0.47302
+-0.1022 0.01515 0.47478
+-0.1022 -0.00325 0.47668
+-0.1022 -0.02175 0.4787
+-0.1022 -0.04005 0.48088
+-0.1022 -0.05845 0.4832
+-0.1022 -0.07685 0.48564
+-0.1212 -0.02175 0.4787
+-0.1212 -0.00325 0.47668
+-0.1212 0.01515 0.47478
+-0.1403 0.01515 0.47478
+-0.1593 0.01515 0.47478
+-0.1777 0.01475 0.4711
+-0.1931 0.01375 0.46081
+-0.2034 0.01225 0.44546
+-0.207 0.01045 0.42734
+-0.207 0.02725 0.40549
+-0.207 0.02545 0.3854
+-0.207 0.02365 0.3653
+-0.207 0.02185 0.3452
+-0.207 0.02005 0.3251
+-0.207 0.01825 0.305
+-0.2079 0.03645 0.2846
+-0.2106 0.03485 0.2662
+-0.2152 0.03335 0.2481
+-0.2215 0.03185 0.2305
+-0.2295 0.03045 0.2137
+-0.2391 0.02915 0.1977
+-0.2503 0.02785 0.1827
+-0.2628 0.02675 0.1689
+-0.2767 0.02565 0.1564
+-0.2917 0.02475 0.1453
+-0.3078 0.02395 0.1357
+-0.3247 0.02325 0.1277
+-0.3423 0.02275 0.1214
+-0.36046 0.02235 0.1169
+-0.37897 0.02215 0.1141
+-0.39766 0.02205 0.1132
+-0.39766 0.04325 0.1116
+-0.39766 0.06445 0.11
+-0.39766 0.08565 0.1086
+-0.39766 0.10685 0.1074
+-0.37897 0.10685 0.1083
+-0.36046 0.10705 0.1111
+-0.3423 0.10725 0.1156
+-0.3247 0.10765 0.1219
+-0.3078 0.10805 0.1299
+-0.2917 0.10855 0.1395
+-0.2767 0.10915 0.1507
+-0.2628 0.10985 0.1632
+-0.2503 0.11055 0.1771
+-0.2391 0.11135 0.1921
+-0.2391 0.13195 0.1911
+-0.2295 0.13265 0.2071
+-0.2215 0.13345 0.224
+-0.2215 0.15375 0.2231
+-0.2215 0.17405 0.2224
+-0.2215 0.19445 0.2219
+-0.2152 0.19485 0.2395
+-0.2152 0.21505 0.2391
+-0.2152 0.23525 0.2389
+-0.2151 0.25555 0.239
+-0.1929 0.25555 0.2519
+-0.1684 0.25555 0.2689
+-0.1412 0.25555 0.2865
+-0.1413 0.25555 0.3106
+-0.1201 0.25555 0.3336
+-0.0895 0.25555 0.335
+-0.0676 0.25555 0.3232
+-0.07 0.25555 0.3355
+-0.0705 0.25555 0.357
+-0.0683 0.25555 0.3759
+-0.0663 0.25555 0.3924
+-0.0649 0.25555 0.40825
+-0.0645 0.25555 0.42612
+-0.0641 0.25555 0.44463
+-0.0465 0.25555 0.4443
+-0.0327 0.25555 0.44203
+-0.0215 0.25555 0.43901
+-0.0115 0.25555 0.44917
+-0.0115 0.23685 0.44922
+-0.0269 0.21845 0.45961
+-0.0269 0.19985 0.45996
+-0.0269 0.18135 0.46045
+-0.045 0.16295 0.46468
+-0.045 0.14445 0.46545
+-0.0641 0.12595 0.46637
+-0.0641 0.10745 0.46741
+-0.0831 0.08895 0.4686
+-0.0831 0.07045 0.46992
+-0.1022 0.05205 0.47142
+-0.1212 0.03355 0.47302
+-0.1403 0.03355 0.47302
+-0.1593 0.03355 0.47302
+-0.1777 0.03325 0.46934
+-0.1931 0.03225 0.45906
+-0.2034 0.03085 0.44369
+-0.207 0.02925 0.42557
+-0.207 0.04625 0.40381
+-0.207 0.04455 0.3837
+-0.207 0.04295 0.3636
+-0.207 0.04125 0.3435
+-0.207 0.03965 0.3234
+-0.207 0.03795 0.3032
+-0.2079 0.05625 0.283
+-0.2106 0.05485 0.2646
+-0.2152 0.05345 0.2465
+-0.2215 0.05215 0.2289
+-0.2295 0.05085 0.212
+-0.2391 0.04965 0.196
+-0.2503 0.04855 0.1811
+-0.2628 0.04745 0.1672
+-0.2767 0.04655 0.1547
+-0.2917 0.04565 0.1436
+-0.3078 0.04495 0.134
+-0.3247 0.04435 0.126
+-0.3423 0.04385 0.1197
+-0.36046 0.04355 0.1152
+-0.37897 0.04325 0.1125
+-0.37897 0.06445 0.1109
+-0.37897 0.08565 0.1096
+-0.36046 0.08585 0.1123
+-0.3423 0.08615 0.1168
+-0.3247 0.08655 0.1231
+-0.3078 0.08705 0.1311
+-0.2917 0.08755 0.1407
+-0.2767 0.08825 0.1518
+-0.2628 0.08905 0.1644
+-0.2503 0.08985 0.1782
+-0.2391 0.09075 0.1932
+-0.2295 0.09175 0.2093
+-0.2295 0.11215 0.2081
+-0.2215 0.11305 0.225
+-0.2152 0.11405 0.2426
+-0.2152 0.13425 0.2416
+-0.2152 0.15445 0.2407
+-0.2152 0.17465 0.24
+-0.2106 0.17515 0.2582
+-0.2106 0.19525 0.2577
+-0.2106 0.21535 0.2573
+-0.2106 0.23545 0.257
+-0.2106 0.25555 0.2571
+-0.1883 0.25555 0.2737
+-0.1666 0.25555 0.2906
+-0.1655 0.25555 0.3141
+-0.1424 0.25555 0.3334
+-0.1234 0.25555 0.3583
+-0.0987 0.25555 0.365
+-0.0898 0.25555 0.3823
+-0.0852 0.25555 0.3959
+-0.0834 0.25555 0.40962
+-0.0837 0.25555 0.42702
+-0.0831 0.25555 0.44517
+-0.0641 0.25555 0.46292
+-0.045 0.25555 0.46292
+-0.0269 0.25555 0.45933
+-0.0269 0.23695 0.45941
+-0.045 0.21845 0.4632
+-0.045 0.19995 0.46355
+-0.045 0.18145 0.46403
+-0.0641 0.16295 0.46468
+-0.0641 0.14445 0.46545
+-0.0831 0.12595 0.46637
+-0.0831 0.10745 0.46741
+-0.1022 0.08895 0.4686
+-0.1022 0.07045 0.46992
+-0.1212 0.05205 0.47142
+-0.1403 0.05205 0.47142
+-0.1593 0.05205 0.47142
+-0.1777 0.05175 0.46771
+-0.1931 0.05085 0.45743
+-0.2034 0.04955 0.44205
+-0.207 0.04805 0.4239
+-0.207 0.06525 0.4023
+-0.207 0.06375 0.3822
+-0.207 0.06225 0.362
+-0.207 0.06075 0.3419
+-0.207 0.05925 0.3218
+-0.207 0.05775 0.3017
+-0.2079 0.07615 0.2816
+-0.2106 0.07485 0.2631
+-0.2152 0.07365 0.245
+-0.2215 0.07245 0.2274
+-0.2295 0.07125 0.2106
+-0.2391 0.07025 0.1946
+-0.2503 0.06915 0.1796
+-0.2628 0.06825 0.1657
+-0.2767 0.06735 0.1532
+-0.2917 0.06665 0.1421
+-0.3078 0.06595 0.1325
+-0.3247 0.06545 0.1245
+-0.3423 0.06495 0.1182
+-0.36046 0.06465 0.1137
+-0.2215 0.09275 0.2261
+-0.2152 0.09385 0.2437
+-0.2106 0.09495 0.2619
+-0.2106 0.11505 0.2607
+-0.2106 0.13505 0.2597
+-0.2106 0.15515 0.2589
+-0.2079 0.15585 0.2774
+-0.2079 0.17575 0.2767
+-0.2079 0.19565 0.2762
+-0.2079 0.21565 0.2758
+-0.2079 0.23555 0.2756
+-0.2079 0.25555 0.2756
+-0.1874 0.25555 0.2937
+-0.187 0.25555 0.3143
+-0.1658 0.25555 0.3344
+-0.1438 0.25555 0.355
+-0.1449 0.25555 0.375
+-0.1289 0.25555 0.3784
+-0.1137 0.25555 0.384
+-0.1018 0.25555 0.39
+-0.1013 0.25555 0.40031
+-0.1014 0.25555 0.41275
+-0.1019 0.25555 0.42763
+-0.1012 0.25555 0.44583
+-0.0831 0.25555 0.46292
+-0.0831 0.23695 0.463
+-0.0641 0.23695 0.463
+-0.045 0.23695 0.463
+-0.0641 0.21845 0.4632
+-0.0641 0.19995 0.46355
+-0.0641 0.18145 0.46403
+-0.0831 0.16295 0.46468
+-0.0831 0.14445 0.46545
+-0.1022 0.12595 0.46637
+-0.1022 0.10745 0.46741
+-0.1212 0.08895 0.4686
+-0.1212 0.07045 0.46992
+-0.1403 0.07045 0.46992
+-0.1593 0.07045 0.46992
+-0.1777 0.07015 0.46625
+-0.1931 0.06945 0.45595
+-0.2034 0.06825 0.44057
+-0.207 0.06685 0.42241
+-0.207 0.08425 0.40092
+-0.207 0.08285 0.3808
+-0.207 0.08155 0.3607
+-0.207 0.08015 0.3405
+-0.207 0.07885 0.3204
+-0.207 0.07745 0.3002
+-0.2079 0.09605 0.2803
+-0.2079 0.11595 0.2792
+-0.2079 0.13585 0.2782
+-0.207 0.13675 0.2969
+-0.207 0.15655 0.296
+-0.207 0.17635 0.2954
+-0.207 0.19615 0.2948
+-0.207 0.21595 0.2945
+-0.207 0.23575 0.2942
+-0.207 0.25555 0.2942
+-0.207 0.25555 0.3143
+-0.207 0.25555 0.3345
+-0.1869 0.25555 0.3345
+-0.1662 0.25555 0.3547
+-0.1664 0.25555 0.3749
+-0.148 0.25555 0.3958
+-0.1328 0.25555 0.3975
+-0.1164 0.25555 0.3987
+-0.1179 0.25555 0.4137
+-0.119 0.25555 0.42927
+-0.1204 0.25555 0.44653
+-0.1022 0.25555 0.46292
+-0.1022 0.23695 0.463
+-0.1022 0.21845 0.4632
+-0.0831 0.21845 0.4632
+-0.0831 0.19995 0.46355
+-0.0831 0.18145 0.46403
+-0.1022 0.16295 0.46468
+-0.1022 0.14445 0.46545
+-0.1212 0.12595 0.46637
+-0.1212 0.10745 0.46741
+-0.1403 0.08895 0.4686
+-0.1593 0.08895 0.4686
+-0.1777 0.08875 0.46489
+-0.1931 0.08795 0.4546
+-0.2034 0.08695 0.43921
+-0.207 0.08575 0.42103
+-0.207 0.10325 0.3997
+-0.207 0.10205 0.3795
+-0.207 0.10085 0.3594
+-0.207 0.09965 0.3393
+-0.207 0.09845 0.3191
+-0.207 0.09725 0.299
+-0.207 0.11695 0.2978
+-0.207 0.13765 0.317
+-0.207 0.15725 0.3162
+-0.207 0.17695 0.3155
+-0.207 0.19655 0.315
+-0.207 0.21625 0.3146
+-0.207 0.23585 0.3144
+-0.207 0.23605 0.3346
+-0.207 0.23615 0.3548
+-0.207 0.25555 0.3547
+-0.1869 0.25555 0.3547
+-0.1868 0.25555 0.3749
+-0.1665 0.25555 0.395
+-0.1502 0.25555 0.41331
+-0.1348 0.25555 0.41456
+-0.1372 0.25555 0.42998
+-0.139 0.25555 0.44658
+-0.1212 0.25555 0.46292
+-0.1212 0.23695 0.463
+-0.1212 0.21845 0.4632
+-0.1212 0.19995 0.46355
+-0.1022 0.19995 0.46355
+-0.1022 0.18145 0.46403
+-0.1212 0.16295 0.46468
+-0.1212 0.14445 0.46545
+-0.1403 0.12595 0.46637
+-0.1403 0.10745 0.46741
+-0.1593 0.10745 0.46741
+-0.1777 0.10725 0.46371
+-0.1931 0.10655 0.4534
+-0.2034 0.10565 0.43802
+-0.207 0.10455 0.41981
+-0.207 0.12225 0.3986
+-0.207 0.12115 0.3784
+-0.207 0.12015 0.3583
+-0.207 0.11915 0.3381
+-0.207 0.11805 0.318
+-0.207 0.13855 0.3372
+-0.207 0.15805 0.3364
+-0.207 0.17755 0.3357
+-0.207 0.19705 0.3352
+-0.207 0.21655 0.3348
+-0.207 0.21685 0.355
+-0.207 0.23635 0.375
+-0.207 0.25555 0.3749
+-0.207 0.25555 0.3951
+-0.1868 0.25555 0.3951
+-0.1673 0.25555 0.41261
+-0.1516 0.25555 0.42927
+-0.1557 0.25555 0.44553
+-0.1403 0.25555 0.46292
+-0.1403 0.23695 0.463
+-0.1403 0.21845 0.4632
+-0.1403 0.19995 0.46355
+-0.1403 0.18145 0.46403
+-0.1212 0.18145 0.46403
+-0.1403 0.16295 0.46468
+-0.1403 0.14445 0.46545
+-0.1593 0.12595 0.46637
+-0.1777 0.12575 0.46265
+-0.1931 0.12515 0.45235
+-0.2034 0.12435 0.43695
+-0.207 0.12335 0.41875
+-0.207 0.14125 0.3977
+-0.207 0.14035 0.3775
+-0.207 0.13945 0.3573
+-0.207 0.15885 0.3565
+-0.207 0.17815 0.3559
+-0.207 0.19745 0.3554
+-0.207 0.21715 0.3752
+-0.207 0.23645 0.3951
+-0.207 0.23665 0.41533
+-0.207 0.25555 0.41525
+-0.1868 0.25555 0.41327
+-0.1847 0.25555 0.4272
+-0.1675 0.25555 0.42856
+-0.1699 0.25555 0.44197
+-0.1593 0.25555 0.46292
+-0.1593 0.23695 0.463
+-0.1593 0.21845 0.4632
+-0.1593 0.19995 0.46355
+-0.1593 0.18145 0.46403
+-0.1593 0.16295 0.46468
+-0.1593 0.14445 0.46545
+-0.1777 0.14425 0.46173
+-0.1931 0.14375 0.45142
+-0.2034 0.14305 0.43603
+-0.207 0.14225 0.41781
+-0.207 0.16035 0.3969
+-0.207 0.15955 0.3767
+-0.207 0.17875 0.376
+-0.207 0.19795 0.3755
+-0.207 0.21745 0.3954
+-0.207 0.21775 0.41553
+-0.2034 0.23675 0.43355
+-0.2034 0.25555 0.43346
+-0.1818 0.25555 0.43826
+-0.1777 0.25555 0.45921
+-0.1778 0.23695 0.45927
+-0.1778 0.21845 0.45949
+-0.1778 0.19985 0.45984
+-0.1777 0.18135 0.46034
+-0.1777 0.16275 0.46097
+-0.1931 0.16235 0.45065
+-0.2034 0.16185 0.43524
+-0.207 0.16115 0.41704
+-0.207 0.17935 0.3962
+-0.207 0.19835 0.3957
+-0.207 0.19885 0.41589
+-0.2034 0.21805 0.43376
+-0.1931 0.23685 0.44896
+-0.1931 0.25555 0.4489
+-0.1931 0.21825 0.44917
+-0.1931 0.19965 0.44951
+-0.1931 0.18105 0.45001
+-0.2034 0.18055 0.4346
+-0.207 0.17995 0.4164
+-0.2034 0.19925 0.4341
+-0.44777 -0.08915 0.1248
+-0.4603 -0.12345 0.1293
+-0.4603 -0.14145 0.1318
+-0.4603 -0.14405 0.1137
+-0.4603 -0.14655 0.0953
+-0.4603 -0.12595 0.1112
+-0.0327 0.25555 0.2716
+-0.015 0.25555 0.271
+-0.0115 0.25555 0.2503
+-0.0046 0.25555 0.2376
+0.0061 0.25555 0.2295
+0.002 0.25555 0.2437
+0.0806 0.25555 -0.0512
+0.0882 0.25555 -0.0649
+0.0946 0.25555 -0.0743
+0.102 0.25555 -0.0816
+0.0997 0.25555 -0.0689
+0.322 0.25555 -0.376
+0.3222 0.25555 -0.394
+0.3378 0.25555 -0.3977
+0.3384 0.25555 -0.4173
+0.3382 0.25555 -0.4344
+0.3365 0.25555 -0.4515
+0.3333 0.25555 -0.4689
+0.3297 0.25555 -0.4871
+0.3482 0.25555 -0.4905
+0.3633 0.24385 -0.4929
+0.3813 0.24405 -0.4953
+0.3988 0.24425 -0.4972
+0.4099 0.23445 -0.498
+0.4207 0.23565 -0.4988
+0.4289 0.24355 -0.4992
+0.4227 0.25555 -0.4989
+0.404 0.25555 -0.4976
+0.4055 0.25555 -0.4792
+0.4058 0.25555 -0.4615
+0.3868 0.25555 -0.4597
+0.3866 0.25555 -0.4415
+0.3697 0.25555 -0.44
+0.3699 0.25555 -0.4235
+0.3542 0.25555 -0.4207
+0.3539 0.25555 -0.437
+0.3538 0.25555 -0.4548
+0.3513 0.25555 -0.4722
+0.3693 0.25555 -0.4751
+0.3667 0.25555 -0.4934
+0.3854 0.25555 -0.4958
+0.3874 0.25555 -0.4774
+0.3694 0.25555 -0.4574
+0.4151 0.24435 -0.4985
+0.4603 0.12335 0.1394
+3  0 1 2
+3  0 2 3
+3  0 3 4
+3  0 4 5
+3  0 5 1
+3  1 5 6
+3  1 6 7
+3  1 7 8
+3  1 8 9
+3  1 9 2
+3  2 9 10
+3  2 10 11
+3  2 11 3
+3  3 11 12
+3  3 12 13
+3  3 13 4
+3  4 13 14
+3  4 14 15
+3  4 15 5
+3  5 15 16
+3  5 16 6
+3  6 16 17
+3  6 17 18
+3  6 18 7
+3  7 18 19
+3  7 19 20
+3  7 20 8
+3  8 20 21
+3  8 21 22
+3  8 22 9
+3  9 22 23
+3  9 23 10
+3  10 23 24
+3  10 24 11
+3  11 24 25
+3  11 25 12
+3  12 25 26
+3  12 26 27
+3  12 27 13
+3  13 27 28
+3  13 28 14
+3  14 28 29
+3  14 29 30
+3  14 30 15
+3  15 30 31
+3  15 31 16
+3  16 31 32
+3  16 32 17
+3  17 32 33
+3  17 33 18
+3  18 33 34
+3  18 34 35
+3  18 35 19
+3  19 35 36
+3  19 36 37
+3  19 37 20
+3  20 37 38
+3  20 38 21
+3  21 38 39
+3  21 39 40
+3  21 40 22
+3  22 40 41
+3  22 41 23
+3  23 41 42
+3  23 42 24
+3  24 42 43
+3  24 43 44
+3  24 44 25
+3  25 44 45
+3  25 45 26
+3  26 45 46
+3  26 46 47
+3  26 47 27
+3  27 47 48
+3  27 48 28
+3  28 48 49
+3  28 49 29
+3  29 49 50
+3  29 50 51
+3  29 51 30
+3  30 51 52
+3  30 52 31
+3  31 52 53
+3  31 53 32
+3  32 53 54
+3  32 54 33
+3  33 54 55
+3  33 55 34
+3  34 55 56
+3  34 56 57
+3  34 57 35
+3  35 57 58
+3  35 58 36
+3  36 58 59
+3  36 59 60
+3  36 60 37
+3  37 60 61
+3  37 61 38
+3  38 61 62
+3  38 62 39
+3  39 62 63
+3  39 63 64
+3  39 64 40
+3  40 64 65
+3  40 65 41
+3  41 65 66
+3  41 66 67
+3  41 67 42
+3  42 67 68
+3  42 68 69
+3  42 69 43
+3  43 69 70
+3  43 70 71
+3  43 71 72
+3  43 72 44
+3  44 72 73
+3  44 73 45
+3  45 73 74
+3  45 74 75
+3  45 75 46
+3  46 75 76
+3  46 76 77
+3  46 77 47
+3  47 77 78
+3  47 78 48
+3  48 78 79
+3  48 79 49
+3  49 79 80
+3  49 80 50
+3  50 80 81
+3  50 81 82
+3  50 82 51
+3  51 82 83
+3  51 83 52
+3  52 83 84
+3  52 84 53
+3  53 84 85
+3  53 85 54
+3  54 85 86
+3  54 86 55
+3  55 86 87
+3  55 87 56
+3  56 87 88
+3  56 88 89
+3  56 89 57
+3  57 89 90
+3  57 90 58
+3  58 90 91
+3  58 91 59
+3  59 91 92
+3  59 92 93
+3  59 93 60
+3  60 93 94
+3  60 94 61
+3  61 94 95
+3  61 95 62
+3  62 95 96
+3  62 96 63
+3  63 96 97
+3  63 97 98
+3  63 98 64
+3  64 98 99
+3  64 99 65
+3  65 99 100
+3  65 100 66
+3  66 100 101
+3  66 101 102
+3  66 102 67
+3  67 102 68
+3  68 102 103
+3  68 103 104
+3  68 104 69
+3  69 104 105
+3  69 105 70
+3  70 105 106
+3  70 106 107
+3  70 107 71
+3  71 107 108
+3  71 108 109
+3  71 109 110
+3  71 110 72
+3  72 110 74
+3  74 110 111
+3  74 111 75
+3  75 111 112
+3  75 112 76
+3  76 112 113
+3  76 113 114
+3  76 114 77
+3  77 114 115
+3  77 115 78
+3  78 115 116
+3  78 116 79
+3  79 116 117
+3  79 117 80
+3  80 117 118
+3  80 118 119
+3  80 119 81
+3  81 119 120
+3  81 120 82
+3  82 120 121
+3  82 121 83
+3  83 121 122
+3  83 122 84
+3  84 122 123
+3  84 123 85
+3  85 123 124
+3  85 124 86
+3  86 124 125
+3  86 125 87
+3  87 125 126
+3  87 126 127
+3  87 127 88
+3  88 127 128
+3  88 128 89
+3  89 128 129
+3  89 129 90
+3  90 129 130
+3  90 130 91
+3  91 130 131
+3  91 131 92
+3  92 131 132
+3  92 132 133
+3  92 133 93
+3  93 133 134
+3  93 134 94
+3  94 134 135
+3  94 135 95
+3  95 135 136
+3  95 136 96
+3  96 136 137
+3  96 137 97
+3  97 137 138
+3  97 138 139
+3  97 139 98
+3  98 139 140
+3  98 140 99
+3  99 140 100
+3  100 140 141
+3  100 141 101
+3  101 141 142
+3  101 142 143
+3  101 143 102
+3  102 143 103
+3  103 143 144
+3  103 144 145
+3  103 145 104
+3  104 145 146
+3  104 146 105
+3  105 146 147
+3  105 147 106
+3  106 147 148
+3  106 148 149
+3  106 149 107
+3  107 149 108
+3  108 149 150
+3  108 150 151
+3  108 151 109
+3  109 151 152
+3  109 152 153
+3  109 153 110
+3  110 153 154
+3  110 154 111
+3  111 154 155
+3  111 155 112
+3  112 155 156
+3  112 156 113
+3  113 156 157
+3  113 157 158
+3  113 158 159
+3  113 159 114
+3  114 159 115
+3  115 159 116
+3  116 159 160
+3  116 160 117
+3  117 160 161
+3  117 161 118
+3  118 161 162
+3  118 162 163
+3  118 163 164
+3  118 164 119
+3  119 164 165
+3  119 165 120
+3  120 165 166
+3  120 166 121
+3  121 166 167
+3  121 167 122
+3  122 167 168
+3  122 168 123
+3  123 168 169
+3  123 169 124
+3  124 169 170
+3  124 170 125
+3  125 170 171
+3  125 171 126
+3  126 171 172
+3  126 172 173
+3  126 173 127
+3  127 173 174
+3  127 174 128
+3  128 174 175
+3  128 175 129
+3  129 175 176
+3  129 176 130
+3  130 176 177
+3  130 177 131
+3  131 177 178
+3  131 178 132
+3  132 178 179
+3  132 179 180
+3  132 180 133
+3  133 180 181
+3  133 181 134
+3  134 181 182
+3  134 182 135
+3  135 182 183
+3  135 183 136
+3  136 183 184
+3  136 184 137
+3  137 184 185
+3  137 185 138
+3  138 185 186
+3  138 186 187
+3  138 187 139
+3  139 187 188
+3  139 188 140
+3  140 188 141
+3  141 188 189
+3  141 189 142
+3  142 189 190
+3  142 190 191
+3  142 191 143
+3  143 191 144
+3  144 191 192
+3  144 192 193
+3  144 193 145
+3  145 193 194
+3  145 194 146
+3  146 194 195
+3  146 195 147
+3  147 195 196
+3  147 196 148
+3  148 196 197
+3  148 197 198
+3  148 198 149
+3  149 198 150
+3  150 198 199
+3  150 199 200
+3  150 200 151
+3  151 200 201
+3  151 201 152
+3  152 201 202
+3  152 202 203
+3  152 203 204
+3  152 204 153
+3  153 204 154
+3  154 204 155
+3  155 204 205
+3  155 205 156
+3  156 205 206
+3  156 206 207
+3  156 207 157
+3  157 207 208
+3  157 208 158
+3  158 208 209
+3  158 209 210
+3  158 210 159
+3  159 210 160
+3  160 210 211
+3  160 211 161
+3  161 211 212
+3  161 212 162
+3  162 212 213
+3  162 213 214
+3  162 214 163
+3  163 214 215
+3  163 215 164
+3  164 215 216
+3  164 216 165
+3  165 216 217
+3  165 217 218
+3  165 218 166
+3  166 218 219
+3  166 219 167
+3  167 219 220
+3  167 220 168
+3  168 220 221
+3  168 221 169
+3  169 221 222
+3  169 222 170
+3  170 222 223
+3  170 223 171
+3  171 223 224
+3  171 224 172
+3  172 224 225
+3  172 225 173
+3  173 225 226
+3  173 226 227
+3  173 227 174
+3  174 227 228
+3  174 228 175
+3  175 228 229
+3  175 229 176
+3  176 229 230
+3  176 230 177
+3  177 230 231
+3  177 231 178
+3  178 231 232
+3  178 232 179
+3  179 232 233
+3  179 233 234
+3  179 234 180
+3  180 234 235
+3  180 235 181
+3  181 235 236
+3  181 236 182
+3  182 236 237
+3  182 237 183
+3  183 237 238
+3  183 238 184
+3  184 238 239
+3  184 239 185
+3  185 239 240
+3  185 240 186
+3  186 240 241
+3  186 241 242
+3  186 242 187
+3  187 242 243
+3  187 243 188
+3  188 243 189
+3  189 243 244
+3  189 244 190
+3  190 244 245
+3  190 245 246
+3  190 246 191
+3  191 246 192
+3  192 246 247
+3  192 247 248
+3  192 248 193
+3  193 248 249
+3  193 249 194
+3  194 249 250
+3  194 250 195
+3  195 250 251
+3  195 251 252
+3  195 252 196
+3  196 252 197
+3  197 252 253
+3  197 253 254
+3  197 254 198
+3  198 254 199
+3  199 254 255
+3  199 255 256
+3  199 256 200
+3  200 256 257
+3  200 257 201
+3  201 257 258
+3  201 258 259
+3  201 259 202
+3  202 259 260
+3  202 260 203
+3  203 260 261
+3  203 261 262
+3  203 262 205
+3  205 262 206
+3  206 262 263
+3  206 263 264
+3  206 264 207
+3  207 264 265
+3  207 265 208
+3  208 265 266
+3  208 266 209
+3  209 266 267
+3  209 267 210
+3  210 267 211
+3  211 267 268
+3  211 268 212
+3  212 268 269
+3  212 269 213
+3  213 269 270
+3  213 270 271
+3  213 271 214
+3  214 271 272
+3  214 272 215
+3  215 272 273
+3  215 273 274
+3  215 274 216
+3  216 274 217
+3  217 274 275
+3  217 275 276
+3  217 276 277
+3  217 277 218
+3  218 277 219
+3  219 277 278
+3  219 278 220
+3  220 278 279
+3  220 279 221
+3  221 279 280
+3  221 280 222
+3  222 280 281
+3  222 281 223
+3  223 281 282
+3  223 282 224
+3  224 282 283
+3  224 283 225
+3  225 283 284
+3  225 284 226
+3  226 284 285
+3  226 285 286
+3  226 286 227
+3  227 286 287
+3  227 287 228
+3  228 287 288
+3  228 288 229
+3  229 288 289
+3  229 289 230
+3  230 289 290
+3  230 290 231
+3  231 290 291
+3  231 291 232
+3  232 291 292
+3  232 292 233
+3  233 292 293
+3  233 293 294
+3  233 294 234
+3  234 294 295
+3  234 295 235
+3  235 295 296
+3  235 296 236
+3  236 296 297
+3  236 297 237
+3  237 297 298
+3  237 298 238
+3  238 298 299
+3  238 299 239
+3  239 299 300
+3  239 300 240
+3  240 300 301
+3  240 301 241
+3  241 301 302
+3  241 302 303
+3  241 303 242
+3  242 303 304
+3  242 304 243
+3  243 304 244
+3  244 304 305
+3  244 305 245
+3  245 305 306
+3  245 306 307
+3  245 307 246
+3  246 307 247
+3  247 307 308
+3  247 308 309
+3  247 309 248
+3  248 309 310
+3  248 310 249
+3  249 310 311
+3  249 311 250
+3  250 311 312
+3  250 312 251
+3  251 312 313
+3  251 313 252
+3  252 313 314
+3  252 314 253
+3  253 314 315
+3  253 315 316
+3  253 316 254
+3  254 316 255
+3  255 316 317
+3  255 317 318
+3  255 318 256
+3  256 318 319
+3  256 319 257
+3  257 319 320
+3  257 320 258
+3  258 320 321
+3  258 321 259
+3  259 321 322
+3  259 322 323
+3  259 323 260
+3  260 323 324
+3  260 324 325
+3  260 325 261
+3  261 325 326
+3  261 326 262
+3  262 326 263
+3  263 326 327
+3  263 327 328
+3  263 328 264
+3  264 328 329
+3  264 329 265
+3  265 329 330
+3  265 330 266
+3  266 330 331
+3  266 331 267
+3  267 331 332
+3  267 332 268
+3  268 332 333
+3  268 333 269
+3  269 333 334
+3  269 334 335
+3  269 335 270
+3  270 335 336
+3  270 336 337
+3  270 337 338
+3  270 338 271
+3  271 338 272
+3  272 338 339
+3  272 339 273
+3  273 339 340
+3  273 340 274
+3  274 340 341
+3  274 341 342
+3  274 342 275
+3  275 342 276
+3  276 342 343
+3  276 343 344
+3  276 344 277
+3  277 344 278
+3  278 344 345
+3  278 345 279
+3  279 345 346
+3  279 346 280
+3  280 346 347
+3  280 347 281
+3  281 347 348
+3  281 348 282
+3  282 348 349
+3  282 349 283
+3  283 349 350
+3  283 350 284
+3  284 350 351
+3  284 351 352
+3  284 352 285
+3  285 352 353
+3  285 353 286
+3  286 353 354
+3  286 354 287
+3  287 354 355
+3  287 355 288
+3  288 355 356
+3  288 356 289
+3  289 356 357
+3  289 357 290
+3  290 357 358
+3  290 358 291
+3  291 358 359
+3  291 359 292
+3  292 359 360
+3  292 360 293
+3  293 360 361
+3  293 361 362
+3  293 362 294
+3  294 362 363
+3  294 363 295
+3  295 363 364
+3  295 364 296
+3  296 364 365
+3  296 365 297
+3  297 365 366
+3  297 366 298
+3  298 366 299
+3  299 366 367
+3  299 367 300
+3  300 367 368
+3  300 368 301
+3  301 368 369
+3  301 369 302
+3  302 369 370
+3  302 370 371
+3  302 371 303
+3  303 371 372
+3  303 372 304
+3  304 372 305
+3  305 372 373
+3  305 373 306
+3  306 373 374
+3  306 374 375
+3  306 375 307
+3  307 375 308
+3  308 375 376
+3  308 376 377
+3  308 377 309
+3  309 377 378
+3  309 378 310
+3  310 378 379
+3  310 379 311
+3  311 379 380
+3  311 380 381
+3  311 381 312
+3  312 381 313
+3  313 381 382
+3  313 382 314
+3  314 382 383
+3  314 383 315
+3  315 383 384
+3  315 384 385
+3  315 385 316
+3  316 385 317
+3  317 385 386
+3  317 386 387
+3  317 387 318
+3  318 387 388
+3  318 388 319
+3  319 388 389
+3  319 389 320
+3  320 389 390
+3  320 390 321
+3  321 390 391
+3  321 391 392
+3  321 392 322
+3  322 392 393
+3  322 393 394
+3  322 394 323
+3  323 394 324
+3  324 394 395
+3  324 395 396
+3  324 396 325
+3  325 396 397
+3  325 397 326
+3  326 397 327
+3  327 397 398
+3  327 398 328
+3  328 398 399
+3  328 399 400
+3  328 400 401
+3  328 401 329
+3  329 401 402
+3  329 402 330
+3  330 402 403
+3  330 403 331
+3  331 403 332
+3  332 403 404
+3  332 404 333
+3  333 404 405
+3  333 405 334
+3  334 405 406
+3  334 406 407
+3  334 407 335
+3  335 407 336
+3  336 407 408
+3  336 408 409
+3  336 409 337
+3  337 409 410
+3  337 410 411
+3  337 411 338
+3  338 411 339
+3  339 411 412
+3  339 412 340
+3  340 412 413
+3  340 413 414
+3  340 414 341
+3  341 414 415
+3  341 415 342
+3  342 415 416
+3  342 416 343
+3  343 416 417
+3  343 417 344
+3  344 417 418
+3  344 418 345
+3  345 418 419
+3  345 419 346
+3  346 419 420
+3  346 420 347
+3  347 420 421
+3  347 421 348
+3  348 421 422
+3  348 422 349
+3  349 422 423
+3  349 423 350
+3  350 423 424
+3  350 424 351
+3  351 424 425
+3  351 425 426
+3  351 426 352
+3  352 426 427
+3  352 427 353
+3  353 427 428
+3  353 428 354
+3  354 428 429
+3  354 429 355
+3  355 429 430
+3  355 430 356
+3  356 430 431
+3  356 431 357
+3  357 431 432
+3  357 432 358
+3  358 432 433
+3  358 433 359
+3  359 433 434
+3  359 434 360
+3  360 434 435
+3  360 435 361
+3  361 435 436
+3  361 436 437
+3  361 437 362
+3  362 437 438
+3  362 438 363
+3  363 438 439
+3  363 439 364
+3  364 439 440
+3  364 440 365
+3  365 440 441
+3  365 441 366
+3  366 441 367
+3  367 441 442
+3  367 442 368
+3  368 442 443
+3  368 443 369
+3  369 443 444
+3  369 444 370
+3  370 444 445
+3  370 445 446
+3  370 446 371
+3  371 446 447
+3  371 447 372
+3  372 447 373
+3  373 447 448
+3  373 448 374
+3  374 448 449
+3  374 449 450
+3  374 450 375
+3  375 450 376
+3  376 450 451
+3  376 451 452
+3  376 452 377
+3  377 452 453
+3  377 453 378
+3  378 453 454
+3  378 454 379
+3  379 454 455
+3  379 455 380
+3  380 455 456
+3  380 456 457
+3  380 457 381
+3  381 457 382
+3  382 457 458
+3  382 458 383
+3  383 458 459
+3  383 459 384
+3  384 459 460
+3  384 460 461
+3  384 461 385
+3  385 461 386
+3  386 461 462
+3  386 462 463
+3  386 463 387
+3  387 463 464
+3  387 464 388
+3  388 464 465
+3  388 465 389
+3  389 465 466
+3  389 466 390
+3  390 466 467
+3  390 467 391
+3  391 467 468
+3  391 468 469
+3  391 469 392
+3  392 469 393
+3  393 469 470
+3  393 470 395
+3  395 470 471
+3  395 471 472
+3  395 472 396
+3  396 472 473
+3  396 473 397
+3  397 473 398
+3  398 473 399
+3  399 473 474
+3  399 474 475
+3  399 475 400
+3  400 475 476
+3  400 476 477
+3  400 477 401
+3  401 477 478
+3  401 478 402
+3  402 478 479
+3  402 479 403
+3  403 479 404
+3  404 479 405
+3  405 479 480
+3  405 480 481
+3  405 481 406
+3  406 481 482
+3  406 482 483
+3  406 483 407
+3  407 483 408
+3  408 483 484
+3  408 484 409
+3  409 484 485
+3  409 485 486
+3  409 486 410
+3  410 486 487
+3  410 487 488
+3  410 488 489
+3  410 489 411
+3  411 489 412
+3  412 489 490
+3  412 490 413
+3  413 490 491
+3  413 491 492
+3  413 492 414
+3  414 492 493
+3  414 493 415
+3  415 493 494
+3  415 494 495
+3  415 495 416
+3  416 495 417
+3  417 495 496
+3  417 496 497
+3  417 497 418
+3  418 497 419
+3  419 497 498
+3  419 498 420
+3  420 498 499
+3  420 499 421
+3  421 499 500
+3  421 500 422
+3  422 500 501
+3  422 501 423
+3  423 501 502
+3  423 502 424
+3  424 502 503
+3  424 503 425
+3  425 503 504
+3  425 504 505
+3  425 505 426
+3  426 505 506
+3  426 506 427
+3  427 506 507
+3  427 507 428
+3  428 507 508
+3  428 508 429
+3  429 508 509
+3  429 509 430
+3  430 509 510
+3  430 510 431
+3  431 510 511
+3  431 511 432
+3  432 511 512
+3  432 512 433
+3  433 512 513
+3  433 513 434
+3  434 513 514
+3  434 514 435
+3  435 514 515
+3  435 515 436
+3  436 515 516
+3  436 516 517
+3  436 517 437
+3  437 517 518
+3  437 518 438
+3  438 518 519
+3  438 519 439
+3  439 519 520
+3  439 520 440
+3  440 520 521
+3  440 521 441
+3  441 521 522
+3  441 522 442
+3  442 522 443
+3  443 522 523
+3  443 523 444
+3  444 523 524
+3  444 524 445
+3  445 524 525
+3  445 525 526
+3  445 526 446
+3  446 526 527
+3  446 527 447
+3  447 527 448
+3  448 527 528
+3  448 528 449
+3  449 528 529
+3  449 529 530
+3  449 530 450
+3  450 530 451
+3  451 530 531
+3  451 531 532
+3  451 532 452
+3  452 532 533
+3  452 533 453
+3  453 533 534
+3  453 534 454
+3  454 534 535
+3  454 535 455
+3  455 535 536
+3  455 536 456
+3  456 536 537
+3  456 537 538
+3  456 538 457
+3  457 538 458
+3  458 538 539
+3  458 539 540
+3  458 540 459
+3  459 540 460
+3  460 540 541
+3  460 541 542
+3  460 542 461
+3  461 542 462
+3  462 542 543
+3  462 543 544
+3  462 544 463
+3  463 544 545
+3  463 545 464
+3  464 545 546
+3  464 546 465
+3  465 546 547
+3  465 547 466
+3  466 547 548
+3  466 548 467
+3  467 548 549
+3  467 549 468
+3  468 549 550
+3  468 550 551
+3  468 551 469
+3  469 551 470
+3  470 551 552
+3  470 552 553
+3  470 553 471
+3  471 553 554
+3  471 554 472
+3  472 554 555
+3  472 555 474
+3  474 555 556
+3  474 556 475
+3  475 556 557
+3  475 557 558
+3  475 558 476
+3  476 558 559
+3  476 559 560
+3  476 560 477
+3  477 560 561
+3  477 561 478
+3  478 561 479
+3  479 561 480
+3  480 561 562
+3  480 562 481
+3  481 562 563
+3  481 563 564
+3  481 564 482
+3  482 564 565
+3  482 565 566
+3  482 566 483
+3  483 566 484
+3  484 566 567
+3  484 567 485
+3  485 567 568
+3  485 568 486
+3  486 568 487
+3  487 568 569
+3  487 569 570
+3  487 570 488
+3  488 570 571
+3  488 571 572
+3  488 572 489
+3  489 572 573
+3  489 573 490
+3  490 573 491
+3  491 573 574
+3  491 574 575
+3  491 575 492
+3  492 575 576
+3  492 576 493
+3  493 576 577
+3  493 577 494
+3  494 577 578
+3  494 578 579
+3  494 579 495
+3  495 579 496
+3  496 579 580
+3  496 580 581
+3  496 581 497
+3  497 581 582
+3  497 582 498
+3  498 582 583
+3  498 583 499
+3  499 583 584
+3  499 584 500
+3  500 584 585
+3  500 585 501
+3  501 585 586
+3  501 586 502
+3  502 586 587
+3  502 587 503
+3  503 587 588
+3  503 588 504
+3  504 588 589
+3  504 589 590
+3  504 590 505
+3  505 590 591
+3  505 591 506
+3  506 591 592
+3  506 592 507
+3  507 592 593
+3  507 593 508
+3  508 593 594
+3  508 594 509
+3  509 594 595
+3  509 595 596
+3  509 596 510
+3  510 596 597
+3  510 597 511
+3  511 597 598
+3  511 598 512
+3  512 598 599
+3  512 599 513
+3  513 599 600
+3  513 600 514
+3  514 600 601
+3  514 601 515
+3  515 601 602
+3  515 602 516
+3  516 602 603
+3  516 603 604
+3  516 604 605
+3  516 605 517
+3  517 605 606
+3  517 606 518
+3  518 606 607
+3  518 607 519
+3  519 607 608
+3  519 608 520
+3  520 608 609
+3  520 609 521
+3  521 609 610
+3  521 610 522
+3  522 610 611
+3  522 611 523
+3  523 611 612
+3  523 612 524
+3  524 612 613
+3  524 613 525
+3  525 613 614
+3  525 614 615
+3  525 615 526
+3  526 615 616
+3  526 616 527
+3  527 616 528
+3  528 616 617
+3  528 617 529
+3  529 617 618
+3  529 618 619
+3  529 619 530
+3  530 619 531
+3  531 619 620
+3  531 620 621
+3  531 621 532
+3  532 621 622
+3  532 622 533
+3  533 622 623
+3  533 623 534
+3  534 623 624
+3  534 624 625
+3  534 625 535
+3  535 625 536
+3  536 625 626
+3  536 626 537
+3  537 626 627
+3  537 627 628
+3  537 628 538
+3  538 628 539
+3  539 628 629
+3  539 629 630
+3  539 630 540
+3  540 630 541
+3  541 630 631
+3  541 631 632
+3  541 632 542
+3  542 632 543
+3  543 632 633
+3  543 633 634
+3  543 634 544
+3  544 634 635
+3  544 635 545
+3  545 635 636
+3  545 636 546
+3  546 636 637
+3  546 637 547
+3  547 637 638
+3  547 638 548
+3  548 638 639
+3  548 639 549
+3  549 639 640
+3  549 640 550
+3  550 640 641
+3  550 641 642
+3  550 642 551
+3  551 642 552
+3  552 642 643
+3  552 643 644
+3  552 644 553
+3  553 644 645
+3  553 645 646
+3  553 646 554
+3  554 646 647
+3  554 647 555
+3  555 647 556
+3  556 647 648
+3  556 648 649
+3  556 649 557
+3  557 649 650
+3  557 650 558
+3  558 650 651
+3  558 651 559
+3  559 651 652
+3  559 652 653
+3  559 653 560
+3  560 653 562
+3  562 653 563
+3  563 653 654
+3  563 654 655
+3  563 655 564
+3  564 655 656
+3  564 656 565
+3  565 656 657
+3  565 657 658
+3  565 658 659
+3  565 659 566
+3  566 659 567
+3  567 659 660
+3  567 660 661
+3  567 661 568
+3  568 661 662
+3  568 662 569
+3  569 662 663
+3  569 663 664
+3  569 664 570
+3  570 664 665
+3  570 665 571
+3  571 665 666
+3  571 666 667
+3  571 667 572
+3  572 667 573
+3  573 667 574
+3  574 667 668
+3  574 668 669
+3  574 669 575
+3  575 669 670
+3  575 670 576
+3  576 670 671
+3  576 671 577
+3  577 671 672
+3  577 672 578
+3  578 672 673
+3  578 673 674
+3  578 674 580
+3  580 674 675
+3  580 675 676
+3  580 676 581
+3  581 676 677
+3  581 677 582
+3  582 677 678
+3  582 678 583
+3  583 678 679
+3  583 679 584
+3  584 679 680
+3  584 680 585
+3  585 680 681
+3  585 681 586
+3  586 681 682
+3  586 682 587
+3  587 682 683
+3  587 683 588
+3  588 683 684
+3  588 684 589
+3  589 684 685
+3  589 685 686
+3  589 686 590
+3  590 686 687
+3  590 687 591
+3  591 687 688
+3  591 688 592
+3  592 688 689
+3  592 689 593
+3  593 689 690
+3  593 690 594
+3  594 690 595
+3  595 690 691
+3  595 691 692
+3  595 692 693
+3  595 693 596
+3  596 693 597
+3  597 693 694
+3  597 694 598
+3  598 694 695
+3  598 695 599
+3  599 695 696
+3  599 696 600
+3  600 696 697
+3  600 697 601
+3  601 697 698
+3  601 698 602
+3  602 698 699
+3  602 699 603
+3  603 699 700
+3  603 700 701
+3  603 701 604
+3  604 701 702
+3  604 702 605
+3  605 702 703
+3  605 703 606
+3  606 703 704
+3  606 704 607
+3  607 704 705
+3  607 705 706
+3  607 706 608
+3  608 706 707
+3  608 707 609
+3  609 707 708
+3  609 708 610
+3  610 708 709
+3  610 709 611
+3  611 709 710
+3  611 710 612
+3  612 710 711
+3  612 711 613
+3  613 711 712
+3  613 712 614
+3  614 712 713
+3  614 713 714
+3  614 714 615
+3  615 714 715
+3  615 715 616
+3  616 715 617
+3  617 715 716
+3  617 716 618
+3  618 716 717
+3  618 717 718
+3  618 718 619
+3  619 718 620
+3  620 718 719
+3  620 719 720
+3  620 720 621
+3  621 720 721
+3  621 721 622
+3  622 721 722
+3  622 722 623
+3  623 722 723
+3  623 723 624
+3  624 723 724
+3  624 724 725
+3  624 725 625
+3  625 725 626
+3  626 725 726
+3  626 726 627
+3  627 726 727
+3  627 727 728
+3  627 728 628
+3  628 728 629
+3  629 728 729
+3  629 729 730
+3  629 730 630
+3  630 730 631
+3  631 730 731
+3  631 731 732
+3  631 732 632
+3  632 732 633
+3  633 732 733
+3  633 733 734
+3  633 734 634
+3  634 734 735
+3  634 735 635
+3  635 735 736
+3  635 736 636
+3  636 736 737
+3  636 737 637
+3  637 737 738
+3  637 738 638
+3  638 738 739
+3  638 739 639
+3  639 739 740
+3  639 740 640
+3  640 740 741
+3  640 741 641
+3  641 741 742
+3  641 742 743
+3  641 743 642
+3  642 743 643
+3  643 743 744
+3  643 744 745
+3  643 745 644
+3  644 745 746
+3  644 746 645
+3  645 746 747
+3  645 747 646
+3  646 747 647
+3  647 747 648
+3  648 747 748
+3  648 748 749
+3  648 749 649
+3  649 749 750
+3  649 750 650
+3  650 750 751
+3  650 751 651
+3  651 751 752
+3  651 752 652
+3  652 752 753
+3  652 753 654
+3  654 753 754
+3  654 754 655
+3  655 754 755
+3  655 755 656
+3  656 755 657
+3  657 755 756
+3  657 756 757
+3  657 757 658
+3  658 757 758
+3  658 758 659
+3  659 758 660
+3  660 758 759
+3  660 759 760
+3  660 760 661
+3  661 760 761
+3  661 761 662
+3  662 761 762
+3  662 762 663
+3  663 762 763
+3  663 763 764
+3  663 764 664
+3  664 764 765
+3  664 765 665
+3  665 765 766
+3  665 766 666
+3  666 766 767
+3  666 767 668
+3  668 767 768
+3  668 768 669
+3  669 768 769
+3  669 769 670
+3  670 769 770
+3  670 770 671
+3  671 770 771
+3  671 771 672
+3  672 771 772
+3  672 772 673
+3  673 772 773
+3  673 773 774
+3  673 774 674
+3  674 774 675
+3  675 774 775
+3  675 775 776
+3  675 776 676
+3  676 776 777
+3  676 777 677
+3  677 777 778
+3  677 778 678
+3  678 778 779
+3  678 779 679
+3  679 779 780
+3  679 780 680
+3  680 780 781
+3  680 781 681
+3  681 781 782
+3  681 782 682
+3  682 782 783
+3  682 783 683
+3  683 783 784
+3  683 784 684
+3  684 784 785
+3  684 785 685
+3  685 785 786
+3  685 786 686
+3  686 786 787
+3  686 787 687
+3  687 787 788
+3  687 788 789
+3  687 789 688
+3  688 789 790
+3  688 790 689
+3  689 790 791
+3  689 791 690
+3  690 791 691
+3  691 791 792
+3  691 792 793
+3  691 793 692
+3  692 793 794
+3  692 794 795
+3  692 795 693
+3  693 795 694
+3  694 795 796
+3  694 796 695
+3  695 796 797
+3  695 797 696
+3  696 797 798
+3  696 798 697
+3  697 798 799
+3  697 799 698
+3  698 799 800
+3  698 800 699
+3  699 800 801
+3  699 801 700
+3  700 801 802
+3  700 802 803
+3  700 803 701
+3  701 803 804
+3  701 804 702
+3  702 804 805
+3  702 805 703
+3  703 805 806
+3  703 806 704
+3  704 806 807
+3  704 807 705
+3  705 807 808
+3  705 808 809
+3  705 809 706
+3  706 809 810
+3  706 810 707
+3  707 810 811
+3  707 811 708
+3  708 811 812
+3  708 812 709
+3  709 812 813
+3  709 813 710
+3  710 813 814
+3  710 814 711
+3  711 814 815
+3  711 815 712
+3  712 815 816
+3  712 816 713
+3  713 816 817
+3  713 817 818
+3  713 818 714
+3  714 818 819
+3  714 819 715
+3  715 819 716
+3  716 819 820
+3  716 820 717
+3  717 820 821
+3  717 821 822
+3  717 822 718
+3  718 822 719
+3  719 822 823
+3  719 823 824
+3  719 824 720
+3  720 824 825
+3  720 825 721
+3  721 825 826
+3  721 826 722
+3  722 826 827
+3  722 827 723
+3  723 827 828
+3  723 828 724
+3  724 828 829
+3  724 829 830
+3  724 830 725
+3  725 830 831
+3  725 831 726
+3  726 831 727
+3  727 831 832
+3  727 832 833
+3  727 833 728
+3  728 833 729
+3  729 833 834
+3  729 834 835
+3  729 835 730
+3  730 835 731
+3  731 835 836
+3  731 836 837
+3  731 837 732
+3  732 837 733
+3  733 837 838
+3  733 838 839
+3  733 839 734
+3  734 839 840
+3  734 840 735
+3  735 840 841
+3  735 841 736
+3  736 841 842
+3  736 842 737
+3  737 842 843
+3  737 843 738
+3  738 843 844
+3  738 844 739
+3  739 844 845
+3  739 845 740
+3  740 845 846
+3  740 846 741
+3  741 846 847
+3  741 847 742
+3  742 847 848
+3  742 848 849
+3  742 849 743
+3  743 849 744
+3  744 849 850
+3  744 850 851
+3  744 851 745
+3  745 851 852
+3  745 852 746
+3  746 852 853
+3  746 853 854
+3  746 854 747
+3  747 854 748
+3  748 854 855
+3  748 855 856
+3  748 856 857
+3  748 857 749
+3  749 857 858
+3  749 858 750
+3  750 858 859
+3  750 859 751
+3  751 859 860
+3  751 860 752
+3  752 860 861
+3  752 861 862
+3  752 862 753
+3  753 862 863
+3  753 863 864
+3  753 864 754
+3  754 864 865
+3  754 865 755
+3  755 865 756
+3  756 865 866
+3  756 866 867
+3  756 867 757
+3  757 867 868
+3  757 868 758
+3  758 868 759
+3  759 868 869
+3  759 869 870
+3  759 870 871
+3  759 871 760
+3  760 871 872
+3  760 872 873
+3  760 873 761
+3  761 873 874
+3  761 874 762
+3  762 874 875
+3  762 875 763
+3  763 875 876
+3  763 876 877
+3  763 877 764
+3  764 877 878
+3  764 878 765
+3  765 878 879
+3  765 879 766
+3  766 879 880
+3  766 880 767
+3  767 880 881
+3  767 881 768
+3  768 881 882
+3  768 882 769
+3  769 882 883
+3  769 883 770
+3  770 883 884
+3  770 884 771
+3  771 884 772
+3  772 884 773
+3  773 884 885
+3  773 885 886
+3  773 886 774
+3  774 886 887
+3  774 887 775
+3  775 887 888
+3  775 888 889
+3  775 889 776
+3  776 889 777
+3  777 889 890
+3  777 890 778
+3  778 890 891
+3  778 891 779
+3  779 891 892
+3  779 892 780
+3  780 892 893
+3  780 893 894
+3  780 894 781
+3  781 894 895
+3  781 895 782
+3  782 895 896
+3  782 896 783
+3  783 896 897
+3  783 897 784
+3  784 897 898
+3  784 898 785
+3  785 898 899
+3  785 899 786
+3  786 899 900
+3  786 900 787
+3  787 900 901
+3  787 901 788
+3  788 901 902
+3  788 902 903
+3  788 903 789
+3  789 903 904
+3  789 904 790
+3  790 904 905
+3  790 905 791
+3  791 905 792
+3  792 905 906
+3  792 906 907
+3  792 907 793
+3  793 907 908
+3  793 908 794
+3  794 908 909
+3  794 909 910
+3  794 910 795
+3  795 910 796
+3  796 910 911
+3  796 911 797
+3  797 911 912
+3  797 912 798
+3  798 912 913
+3  798 913 799
+3  799 913 914
+3  799 914 800
+3  800 914 915
+3  800 915 801
+3  801 915 916
+3  801 916 802
+3  802 916 917
+3  802 917 918
+3  802 918 919
+3  802 919 803
+3  803 919 920
+3  803 920 804
+3  804 920 921
+3  804 921 805
+3  805 921 922
+3  805 922 806
+3  806 922 923
+3  806 923 807
+3  807 923 924
+3  807 924 808
+3  808 924 925
+3  808 925 809
+3  809 925 926
+3  809 926 810
+3  810 926 927
+3  810 927 811
+3  811 927 928
+3  811 928 812
+3  812 928 929
+3  812 929 813
+3  813 929 930
+3  813 930 814
+3  814 930 931
+3  814 931 815
+3  815 931 932
+3  815 932 816
+3  816 932 933
+3  816 933 817
+3  817 933 934
+3  817 934 935
+3  817 935 818
+3  818 935 936
+3  818 936 819
+3  819 936 820
+3  820 936 937
+3  820 937 821
+3  821 937 938
+3  821 938 939
+3  821 939 822
+3  822 939 823
+3  823 939 940
+3  823 940 941
+3  823 941 824
+3  824 941 942
+3  824 942 825
+3  825 942 943
+3  825 943 826
+3  826 943 944
+3  826 944 827
+3  827 944 945
+3  827 945 828
+3  828 945 946
+3  828 946 829
+3  829 946 947
+3  829 947 948
+3  829 948 830
+3  830 948 949
+3  830 949 831
+3  831 949 950
+3  831 950 832
+3  832 950 833
+3  833 950 951
+3  833 951 834
+3  834 951 952
+3  834 952 953
+3  834 953 835
+3  835 953 836
+3  836 953 954
+3  836 954 955
+3  836 955 837
+3  837 955 838
+3  838 955 956
+3  838 956 957
+3  838 957 839
+3  839 957 958
+3  839 958 840
+3  840 958 959
+3  840 959 841
+3  841 959 960
+3  841 960 842
+3  842 960 961
+3  842 961 843
+3  843 961 962
+3  843 962 844
+3  844 962 963
+3  844 963 845
+3  845 963 964
+3  845 964 846
+3  846 964 965
+3  846 965 847
+3  847 965 966
+3  847 966 848
+3  848 966 967
+3  848 967 968
+3  848 968 849
+3  849 968 850
+3  850 968 969
+3  850 969 970
+3  850 970 851
+3  851 970 852
+3  852 970 971
+3  852 971 853
+3  853 971 972
+3  853 972 855
+3  855 972 973
+3  855 973 974
+3  855 974 856
+3  856 974 975
+3  856 975 857
+3  857 975 976
+3  857 976 858
+3  858 976 977
+3  858 977 978
+3  858 978 859
+3  859 978 979
+3  859 979 860
+3  860 979 861
+3  861 979 980
+3  861 980 981
+3  861 981 982
+3  861 982 862
+3  862 982 863
+3  863 982 983
+3  863 983 984
+3  863 984 864
+3  864 984 985
+3  864 985 865
+3  865 985 986
+3  865 986 866
+3  866 986 987
+3  866 987 867
+3  867 987 988
+3  867 988 989
+3  867 989 868
+3  868 989 869
+3  869 989 990
+3  869 990 870
+3  870 990 991
+3  870 991 992
+3  870 992 871
+3  871 992 872
+3  872 992 993
+3  872 993 994
+3  872 994 995
+3  872 995 873
+3  873 995 874
+3  874 995 996
+3  874 996 997
+3  874 997 875
+3  875 997 998
+3  875 998 999
+3  875 999 876
+3  876 999 1000
+3  876 1000 1001
+3  876 1001 877
+3  877 1001 1002
+3  877 1002 878
+3  878 1002 1003
+3  878 1003 879
+3  879 1003 1004
+3  879 1004 880
+3  880 1004 1005
+3  880 1005 881
+3  881 1005 1006
+3  881 1006 882
+3  882 1006 1007
+3  882 1007 883
+3  883 1007 885
+3  885 1007 1008
+3  885 1008 886
+3  886 1008 1009
+3  886 1009 887
+3  887 1009 1010
+3  887 1010 888
+3  888 1010 1011
+3  888 1011 889
+3  889 1011 1012
+3  889 1012 890
+3  890 1012 1013
+3  890 1013 891
+3  891 1013 1014
+3  891 1014 892
+3  892 1014 893
+3  893 1014 1015
+3  893 1015 1016
+3  893 1016 894
+3  894 1016 1017
+3  894 1017 895
+3  895 1017 1018
+3  895 1018 896
+3  896 1018 1019
+3  896 1019 897
+3  897 1019 1020
+3  897 1020 898
+3  898 1020 1021
+3  898 1021 899
+3  899 1021 1022
+3  899 1022 1023
+3  899 1023 900
+3  900 1023 1024
+3  900 1024 901
+3  901 1024 1025
+3  901 1025 902
+3  902 1025 1026
+3  902 1026 903
+3  903 1026 1027
+3  903 1027 1028
+3  903 1028 1029
+3  903 1029 904
+3  904 1029 1030
+3  904 1030 905
+3  905 1030 906
+3  906 1030 1031
+3  906 1031 1032
+3  906 1032 907
+3  907 1032 1033
+3  907 1033 908
+3  908 1033 1034
+3  908 1034 909
+3  909 1034 1035
+3  909 1035 1036
+3  909 1036 910
+3  910 1036 911
+3  911 1036 1037
+3  911 1037 912
+3  912 1037 1038
+3  912 1038 913
+3  913 1038 1039
+3  913 1039 914
+3  914 1039 1040
+3  914 1040 915
+3  915 1040 1041
+3  915 1041 916
+3  916 1041 1042
+3  916 1042 917
+3  917 1042 1043
+3  917 1043 1044
+3  917 1044 918
+3  918 1044 1045
+3  918 1045 919
+3  919 1045 1046
+3  919 1046 920
+3  920 1046 1047
+3  920 1047 921
+3  921 1047 1048
+3  921 1048 922
+3  922 1048 1049
+3  922 1049 923
+3  923 1049 1050
+3  923 1050 924
+3  924 1050 1051
+3  924 1051 925
+3  925 1051 1052
+3  925 1052 926
+3  926 1052 1053
+3  926 1053 927
+3  927 1053 1054
+3  927 1054 928
+3  928 1054 1055
+3  928 1055 929
+3  929 1055 1056
+3  929 1056 930
+3  930 1056 1057
+3  930 1057 931
+3  931 1057 1058
+3  931 1058 932
+3  932 1058 1059
+3  932 1059 933
+3  933 1059 1060
+3  933 1060 934
+3  934 1060 1061
+3  934 1061 1062
+3  934 1062 935
+3  935 1062 1063
+3  935 1063 936
+3  936 1063 937
+3  937 1063 1064
+3  937 1064 938
+3  938 1064 1065
+3  938 1065 1066
+3  938 1066 939
+3  939 1066 940
+3  940 1066 1067
+3  940 1067 1068
+3  940 1068 941
+3  941 1068 1069
+3  941 1069 942
+3  942 1069 1070
+3  942 1070 943
+3  943 1070 1071
+3  943 1071 944
+3  944 1071 1072
+3  944 1072 945
+3  945 1072 1073
+3  945 1073 946
+3  946 1073 1074
+3  946 1074 947
+3  947 1074 1075
+3  947 1075 1076
+3  947 1076 948
+3  948 1076 1077
+3  948 1077 949
+3  949 1077 1078
+3  949 1078 950
+3  950 1078 951
+3  951 1078 1079
+3  951 1079 952
+3  952 1079 1080
+3  952 1080 1081
+3  952 1081 953
+3  953 1081 954
+3  954 1081 1082
+3  954 1082 1083
+3  954 1083 955
+3  955 1083 956
+3  956 1083 1084
+3  956 1084 1085
+3  956 1085 957
+3  957 1085 1086
+3  957 1086 958
+3  958 1086 1087
+3  958 1087 959
+3  959 1087 1088
+3  959 1088 960
+3  960 1088 1089
+3  960 1089 961
+3  961 1089 1090
+3  961 1090 962
+3  962 1090 1091
+3  962 1091 963
+3  963 1091 1092
+3  963 1092 964
+3  964 1092 1093
+3  964 1093 965
+3  965 1093 1094
+3  965 1094 966
+3  966 1094 1095
+3  966 1095 967
+3  967 1095 1096
+3  967 1096 1097
+3  967 1097 968
+3  968 1097 969
+3  969 1097 1098
+3  969 1098 1099
+3  969 1099 970
+3  970 1099 971
+3  971 1099 1100
+3  971 1100 973
+3  973 1100 1101
+3  973 1101 1102
+3  973 1102 974
+3  974 1102 1103
+3  974 1103 975
+3  975 1103 1104
+3  975 1104 1105
+3  975 1105 976
+3  976 1105 977
+3  977 1105 1106
+3  977 1106 1107
+3  977 1107 978
+3  978 1107 1108
+3  978 1108 979
+3  979 1108 980
+3  980 1108 1109
+3  980 1109 1110
+3  980 1110 1111
+3  980 1111 981
+3  981 1111 1112
+3  981 1112 982
+3  982 1112 1113
+3  982 1113 983
+3  983 1113 1114
+3  983 1114 1115
+3  983 1115 984
+3  984 1115 986
+3  986 1115 1114
+3  986 1114 1116
+3  986 1116 987
+3  987 1116 1117
+3  987 1117 1118
+3  987 1118 988
+3  988 1118 1119
+3  988 1119 1120
+3  988 1120 989
+3  989 1120 1121
+3  989 1121 990
+3  990 1121 991
+3  991 1121 1122
+3  991 1122 992
+3  992 1122 1123
+3  992 1123 993
+3  993 1123 1124
+3  993 1124 1125
+3  993 1125 994
+3  994 1125 1126
+3  994 1126 995
+3  995 1126 996
+3  996 1126 1127
+3  996 1127 1128
+3  996 1128 998
+3  998 1128 1129
+3  998 1129 1130
+3  998 1130 999
+3  999 1130 1131
+3  999 1131 1000
+3  1000 1131 1132
+3  1000 1132 1133
+3  1000 1133 1001
+3  1001 1133 1134
+3  1001 1134 1002
+3  1002 1134 1135
+3  1002 1135 1003
+3  1003 1135 1136
+3  1003 1136 1004
+3  1004 1136 1137
+3  1004 1137 1005
+3  1005 1137 1138
+3  1005 1138 1006
+3  1006 1138 1139
+3  1006 1139 1007
+3  1007 1139 1008
+3  1008 1139 1140
+3  1008 1140 1009
+3  1009 1140 1141
+3  1009 1141 1010
+3  1010 1141 1142
+3  1010 1142 1011
+3  1011 1142 1143
+3  1011 1143 1012
+3  1012 1143 1144
+3  1012 1144 1013
+3  1013 1144 1145
+3  1013 1145 1014
+3  1014 1145 1015
+3  1015 1145 1146
+3  1015 1146 1147
+3  1015 1147 1016
+3  1016 1147 1148
+3  1016 1148 1017
+3  1017 1148 1149
+3  1017 1149 1018
+3  1018 1149 1150
+3  1018 1150 1019
+3  1019 1150 1151
+3  1019 1151 1020
+3  1020 1151 1152
+3  1020 1152 1021
+3  1021 1152 1153
+3  1021 1153 1022
+3  1022 1153 1154
+3  1022 1154 1155
+3  1022 1155 1023
+3  1023 1155 1156
+3  1023 1156 1024
+3  1024 1156 1025
+3  1025 1156 1157
+3  1025 1157 1158
+3  1025 1158 1026
+3  1026 1158 1027
+3  1027 1158 1159
+3  1027 1159 1160
+3  1027 1160 1161
+3  1027 1161 1028
+3  1028 1161 1162
+3  1028 1162 1163
+3  1028 1163 1029
+3  1029 1163 1164
+3  1029 1164 1030
+3  1030 1164 1031
+3  1031 1164 1165
+3  1031 1165 1032
+3  1032 1165 1166
+3  1032 1166 1167
+3  1032 1167 1033
+3  1033 1167 1168
+3  1033 1168 1034
+3  1034 1168 1169
+3  1034 1169 1035
+3  1035 1169 1170
+3  1035 1170 1171
+3  1035 1171 1036
+3  1036 1171 1037
+3  1037 1171 1172
+3  1037 1172 1038
+3  1038 1172 1173
+3  1038 1173 1039
+3  1039 1173 1174
+3  1039 1174 1040
+3  1040 1174 1175
+3  1040 1175 1041
+3  1041 1175 1176
+3  1041 1176 1042
+3  1042 1176 1177
+3  1042 1177 1043
+3  1043 1177 1178
+3  1043 1178 1179
+3  1043 1179 1044
+3  1044 1179 1180
+3  1044 1180 1045
+3  1045 1180 1181
+3  1045 1181 1046
+3  1046 1181 1182
+3  1046 1182 1047
+3  1047 1182 1183
+3  1047 1183 1048
+3  1048 1183 1184
+3  1048 1184 1049
+3  1049 1184 1185
+3  1049 1185 1050
+3  1050 1185 1186
+3  1050 1186 1051
+3  1051 1186 1187
+3  1051 1187 1052
+3  1052 1187 1188
+3  1052 1188 1053
+3  1053 1188 1189
+3  1053 1189 1054
+3  1054 1189 1190
+3  1054 1190 1055
+3  1055 1190 1191
+3  1055 1191 1056
+3  1056 1191 1192
+3  1056 1192 1057
+3  1057 1192 1193
+3  1057 1193 1058
+3  1058 1193 1194
+3  1058 1194 1059
+3  1059 1194 1195
+3  1059 1195 1060
+3  1060 1195 1196
+3  1060 1196 1061
+3  1061 1196 1197
+3  1061 1197 1198
+3  1061 1198 1062
+3  1062 1198 1199
+3  1062 1199 1063
+3  1063 1199 1064
+3  1064 1199 1200
+3  1064 1200 1065
+3  1065 1200 1201
+3  1065 1201 1202
+3  1065 1202 1066
+3  1066 1202 1067
+3  1067 1202 1203
+3  1067 1203 1204
+3  1067 1204 1068
+3  1068 1204 1205
+3  1068 1205 1069
+3  1069 1205 1206
+3  1069 1206 1070
+3  1070 1206 1207
+3  1070 1207 1071
+3  1071 1207 1208
+3  1071 1208 1072
+3  1072 1208 1209
+3  1072 1209 1073
+3  1073 1209 1210
+3  1073 1210 1074
+3  1074 1210 1211
+3  1074 1211 1075
+3  1075 1211 1212
+3  1075 1212 1213
+3  1075 1213 1076
+3  1076 1213 1077
+3  1077 1213 1214
+3  1077 1214 1078
+3  1078 1214 1215
+3  1078 1215 1216
+3  1078 1216 1217
+3  1078 1217 1079
+3  1079 1217 1080
+3  1080 1217 1218
+3  1080 1218 1081
+3  1081 1218 1082
+3  1082 1218 1219
+3  1082 1219 1220
+3  1082 1220 1083
+3  1083 1220 1084
+3  1084 1220 1221
+3  1084 1221 1222
+3  1084 1222 1085
+3  1085 1222 1223
+3  1085 1223 1086
+3  1086 1223 1224
+3  1086 1224 1087
+3  1087 1224 1225
+3  1087 1225 1088
+3  1088 1225 1226
+3  1088 1226 1089
+3  1089 1226 1227
+3  1089 1227 1090
+3  1090 1227 1228
+3  1090 1228 1091
+3  1091 1228 1229
+3  1091 1229 1092
+3  1092 1229 1230
+3  1092 1230 1093
+3  1093 1230 1231
+3  1093 1231 1094
+3  1094 1231 1232
+3  1094 1232 1095
+3  1095 1232 1233
+3  1095 1233 1096
+3  1096 1233 1234
+3  1096 1234 1235
+3  1096 1235 1097
+3  1097 1235 1098
+3  1098 1235 1236
+3  1098 1236 1237
+3  1098 1237 1099
+3  1099 1237 1238
+3  1099 1238 1100
+3  1100 1238 1101
+3  1101 1238 1239
+3  1101 1239 1240
+3  1101 1240 1241
+3  1101 1241 1242
+3  1101 1242 1102
+3  1102 1242 1243
+3  1102 1243 1103
+3  1103 1243 1244
+3  1103 1244 1104
+3  1104 1244 1106
+3  1106 1244 1245
+3  1106 1245 1107
+3  1107 1245 1246
+3  1107 1246 1108
+3  1108 1246 1109
+3  1109 1246 1247
+3  1109 1247 1248
+3  1109 1248 1249
+3  1109 1249 1110
+3  1110 1249 1250
+3  1110 1250 1251
+3  1110 1251 1111
+3  1111 1251 1112
+3  1112 1251 1252
+3  1112 1252 1113
+3  1113 1252 1253
+3  1113 1253 1114
+3  1114 1253 1116
+3  1116 1253 1117
+3  1117 1253 1254
+3  1117 1254 1255
+3  1117 1255 1118
+3  1118 1255 1256
+3  1118 1256 1119
+3  1119 1256 1257
+3  1119 1257 1258
+3  1119 1258 1259
+3  1119 1259 1120
+3  1120 1259 1121
+3  1121 1259 1260
+3  1121 1260 1122
+3  1122 1260 1261
+3  1122 1261 1123
+3  1123 1261 1262
+3  1123 1262 1124
+3  1124 1262 1263
+3  1124 1263 1264
+3  1124 1264 1265
+3  1124 1265 1125
+3  1125 1265 1127
+3  1127 1265 1266
+3  1127 1266 1267
+3  1127 1267 1128
+3  1128 1267 1129
+3  1129 1267 1268
+3  1129 1268 1269
+3  1129 1269 1270
+3  1129 1270 1130
+3  1130 1270 1271
+3  1130 1271 1131
+3  1131 1271 1272
+3  1131 1272 1132
+3  1132 1272 1273
+3  1132 1273 1274
+3  1132 1274 1275
+3  1132 1275 1133
+3  1133 1275 1276
+3  1133 1276 1134
+3  1134 1276 1277
+3  1134 1277 1135
+3  1135 1277 1278
+3  1135 1278 1136
+3  1136 1278 1279
+3  1136 1279 1137
+3  1137 1279 1280
+3  1137 1280 1138
+3  1138 1280 1281
+3  1138 1281 1139
+3  1139 1281 1140
+3  1140 1281 1282
+3  1140 1282 1141
+3  1141 1282 1283
+3  1141 1283 1284
+3  1141 1284 1142
+3  1142 1284 1285
+3  1142 1285 1143
+3  1143 1285 1286
+3  1143 1286 1144
+3  1144 1286 1287
+3  1144 1287 1145
+3  1145 1287 1146
+3  1146 1287 1288
+3  1146 1288 1289
+3  1146 1289 1147
+3  1147 1289 1290
+3  1147 1290 1148
+3  1148 1290 1291
+3  1148 1291 1149
+3  1149 1291 1292
+3  1149 1292 1150
+3  1150 1292 1293
+3  1150 1293 1151
+3  1151 1293 1294
+3  1151 1294 1152
+3  1152 1294 1295
+3  1152 1295 1153
+3  1153 1295 1296
+3  1153 1296 1154
+3  1154 1296 1297
+3  1154 1297 1298
+3  1154 1298 1155
+3  1155 1298 1299
+3  1155 1299 1156
+3  1156 1299 1300
+3  1156 1300 1157
+3  1157 1300 1158
+3  1158 1300 1159
+3  1159 1300 1301
+3  1159 1301 1160
+3  1160 1301 1302
+3  1160 1302 1303
+3  1160 1303 1304
+3  1160 1304 1305
+3  1160 1305 1161
+3  1161 1305 1162
+3  1162 1305 1306
+3  1162 1306 1307
+3  1162 1307 1163
+3  1163 1307 1308
+3  1163 1308 1164
+3  1164 1308 1165
+3  1165 1308 1309
+3  1165 1309 1166
+3  1166 1309 1310
+3  1166 1310 1167
+3  1167 1310 1311
+3  1167 1311 1168
+3  1168 1311 1312
+3  1168 1312 1169
+3  1169 1312 1313
+3  1169 1313 1170
+3  1170 1313 1314
+3  1170 1314 1315
+3  1170 1315 1316
+3  1170 1316 1171
+3  1171 1316 1172
+3  1172 1316 1317
+3  1172 1317 1173
+3  1173 1317 1318
+3  1173 1318 1174
+3  1174 1318 1319
+3  1174 1319 1175
+3  1175 1319 1320
+3  1175 1320 1176
+3  1176 1320 1321
+3  1176 1321 1177
+3  1177 1321 1322
+3  1177 1322 1178
+3  1178 1322 1323
+3  1178 1323 1324
+3  1178 1324 1179
+3  1179 1324 1325
+3  1179 1325 1180
+3  1180 1325 1326
+3  1180 1326 1181
+3  1181 1326 1327
+3  1181 1327 1182
+3  1182 1327 1328
+3  1182 1328 1183
+3  1183 1328 1329
+3  1183 1329 1184
+3  1184 1329 1330
+3  1184 1330 1185
+3  1185 1330 1331
+3  1185 1331 1186
+3  1186 1331 1332
+3  1186 1332 1187
+3  1187 1332 1333
+3  1187 1333 1188
+3  1188 1333 1334
+3  1188 1334 1189
+3  1189 1334 1335
+3  1189 1335 1190
+3  1190 1335 1336
+3  1190 1336 1191
+3  1191 1336 1337
+3  1191 1337 1192
+3  1192 1337 1338
+3  1192 1338 1193
+3  1193 1338 1339
+3  1193 1339 1194
+3  1194 1339 1340
+3  1194 1340 1195
+3  1195 1340 1341
+3  1195 1341 1196
+3  1196 1341 1342
+3  1196 1342 1197
+3  1197 1342 1343
+3  1197 1343 1344
+3  1197 1344 1198
+3  1198 1344 1345
+3  1198 1345 1199
+3  1199 1345 1200
+3  1200 1345 1346
+3  1200 1346 1201
+3  1201 1346 1347
+3  1201 1347 1348
+3  1201 1348 1202
+3  1202 1348 1203
+3  1203 1348 1349
+3  1203 1349 1350
+3  1203 1350 1204
+3  1204 1350 1351
+3  1204 1351 1205
+3  1205 1351 1352
+3  1205 1352 1206
+3  1206 1352 1353
+3  1206 1353 1207
+3  1207 1353 1354
+3  1207 1354 1208
+3  1208 1354 1355
+3  1208 1355 1209
+3  1209 1355 1356
+3  1209 1356 1210
+3  1210 1356 1357
+3  1210 1357 1358
+3  1210 1358 1211
+3  1211 1358 1357
+3  1211 1357 1359
+3  1211 1359 1212
+3  1212 1359 1360
+3  1212 1360 1361
+3  1212 1361 1362
+3  1212 1362 1213
+3  1213 1362 1214
+3  1214 1362 1363
+3  1214 1363 1215
+3  1215 1363 1364
+3  1215 1364 1365
+3  1215 1365 1216
+3  1216 1365 1366
+3  1216 1366 1367
+3  1216 1367 1217
+3  1217 1367 1368
+3  1217 1368 1218
+3  1218 1368 1219
+3  1219 1368 1369
+3  1219 1369 1370
+3  1219 1370 1220
+3  1220 1370 1221
+3  1221 1370 1371
+3  1221 1371 1372
+3  1221 1372 1222
+3  1222 1372 1373
+3  1222 1373 1223
+3  1223 1373 1374
+3  1223 1374 1224
+3  1224 1374 1375
+3  1224 1375 1225
+3  1225 1375 1376
+3  1225 1376 1226
+3  1226 1376 1377
+3  1226 1377 1227
+3  1227 1377 1378
+3  1227 1378 1228
+3  1228 1378 1379
+3  1228 1379 1229
+3  1229 1379 1380
+3  1229 1380 1230
+3  1230 1380 1381
+3  1230 1381 1231
+3  1231 1381 1382
+3  1231 1382 1232
+3  1232 1382 1383
+3  1232 1383 1233
+3  1233 1383 1384
+3  1233 1384 1234
+3  1234 1384 1385
+3  1234 1385 1235
+3  1235 1385 1386
+3  1235 1386 1236
+3  1236 1386 1387
+3  1236 1387 1388
+3  1236 1388 1237
+3  1237 1388 1389
+3  1237 1389 1238
+3  1238 1389 1390
+3  1238 1390 1239
+3  1239 1390 1391
+3  1239 1391 1240
+3  1240 1391 1392
+3  1240 1392 1393
+3  1240 1393 1241
+3  1241 1393 1394
+3  1241 1394 1242
+3  1242 1394 1395
+3  1242 1395 1243
+3  1243 1395 1396
+3  1243 1396 1244
+3  1244 1396 1245
+3  1245 1396 1397
+3  1245 1397 1246
+3  1246 1397 1247
+3  1247 1397 1398
+3  1247 1398 1399
+3  1247 1399 1248
+3  1248 1399 1400
+3  1248 1400 1401
+3  1248 1401 1249
+3  1249 1401 1402
+3  1249 1402 1250
+3  1250 1402 1403
+3  1250 1403 1404
+3  1250 1404 1251
+3  1251 1404 1252
+3  1252 1404 1254
+3  1254 1404 1405
+3  1254 1405 1406
+3  1254 1406 1255
+3  1255 1406 1407
+3  1255 1407 1256
+3  1256 1407 1408
+3  1256 1408 1257
+3  1257 1408 1409
+3  1257 1409 1258
+3  1258 1409 1410
+3  1258 1410 1411
+3  1258 1411 1260
+3  1260 1411 1412
+3  1260 1412 1261
+3  1261 1412 1262
+3  1262 1412 1413
+3  1262 1413 1263
+3  1263 1413 1414
+3  1263 1414 1264
+3  1264 1414 1415
+3  1264 1415 1416
+3  1264 1416 1266
+3  1266 1416 1417
+3  1266 1417 1267
+3  1267 1417 1418
+3  1267 1418 1268
+3  1268 1418 1419
+3  1268 1419 1269
+3  1269 1419 1420
+3  1269 1420 1421
+3  1269 1421 1422
+3  1269 1422 1270
+3  1270 1422 1271
+3  1271 1422 1423
+3  1271 1423 1272
+3  1272 1423 1273
+3  1273 1423 1424
+3  1273 1424 1425
+3  1273 1425 1274
+3  1274 1425 1426
+3  1274 1426 1427
+3  1274 1427 1275
+3  1275 1427 1428
+3  1275 1428 1276
+3  1276 1428 1429
+3  1276 1429 1277
+3  1277 1429 1430
+3  1277 1430 1278
+3  1278 1430 1431
+3  1278 1431 1279
+3  1279 1431 1432
+3  1279 1432 1280
+3  1280 1432 1433
+3  1280 1433 1281
+3  1281 1433 1282
+3  1282 1433 1434
+3  1282 1434 1283
+3  1283 1434 1435
+3  1283 1435 1436
+3  1283 1436 1284
+3  1284 1436 1437
+3  1284 1437 1438
+3  1284 1438 1285
+3  1285 1438 1439
+3  1285 1439 1440
+3  1285 1440 1441
+3  1285 1441 1286
+3  1286 1441 1288
+3  1288 1441 1442
+3  1288 1442 1289
+3  1289 1442 1443
+3  1289 1443 1290
+3  1290 1443 1444
+3  1290 1444 1291
+3  1291 1444 1445
+3  1291 1445 1292
+3  1292 1445 1446
+3  1292 1446 1294
+3  1294 1446 1295
+3  1295 1446 1447
+3  1295 1447 1296
+3  1296 1447 1448
+3  1296 1448 1297
+3  1297 1448 1449
+3  1297 1449 1450
+3  1297 1450 1298
+3  1298 1450 1451
+3  1298 1451 1299
+3  1299 1451 1452
+3  1299 1452 1300
+3  1300 1452 1301
+3  1301 1452 1453
+3  1301 1453 1454
+3  1301 1454 1302
+3  1302 1454 1455
+3  1302 1455 1303
+3  1303 1455 1456
+3  1303 1456 1457
+3  1303 1457 1304
+3  1304 1457 1458
+3  1304 1458 1459
+3  1304 1459 1305
+3  1305 1459 1306
+3  1306 1459 1460
+3  1306 1460 1461
+3  1306 1461 1462
+3  1306 1462 1307
+3  1307 1462 1463
+3  1307 1463 1308
+3  1308 1463 1309
+3  1309 1463 1464
+3  1309 1464 1310
+3  1310 1464 1465
+3  1310 1465 1466
+3  1310 1466 1311
+3  1311 1466 1312
+3  1312 1466 1467
+3  1312 1467 1313
+3  1313 1467 1468
+3  1313 1468 1314
+3  1314 1468 1469
+3  1314 1469 1470
+3  1314 1470 1315
+3  1315 1470 1471
+3  1315 1471 1316
+3  1316 1471 1317
+3  1317 1471 1472
+3  1317 1472 1318
+3  1318 1472 1473
+3  1318 1473 1319
+3  1319 1473 1474
+3  1319 1474 1320
+3  1320 1474 1475
+3  1320 1475 1321
+3  1321 1475 1476
+3  1321 1476 1322
+3  1322 1476 1477
+3  1322 1477 1323
+3  1323 1477 1478
+3  1323 1478 1479
+3  1323 1479 1324
+3  1324 1479 1480
+3  1324 1480 1325
+3  1325 1480 1481
+3  1325 1481 1326
+3  1326 1481 1482
+3  1326 1482 1327
+3  1327 1482 1483
+3  1327 1483 1328
+3  1328 1483 1484
+3  1328 1484 1329
+3  1329 1484 1485
+3  1329 1485 1330
+3  1330 1485 1486
+3  1330 1486 1331
+3  1331 1486 1487
+3  1331 1487 1332
+3  1332 1487 1488
+3  1332 1488 1333
+3  1333 1488 1489
+3  1333 1489 1334
+3  1334 1489 1490
+3  1334 1490 1335
+3  1335 1490 1491
+3  1335 1491 1336
+3  1336 1491 1492
+3  1336 1492 1337
+3  1337 1492 1493
+3  1337 1493 1338
+3  1338 1493 1494
+3  1338 1494 1339
+3  1339 1494 1495
+3  1339 1495 1340
+3  1340 1495 1341
+3  1341 1495 1496
+3  1341 1496 1342
+3  1342 1496 1497
+3  1342 1497 1343
+3  1343 1497 1498
+3  1343 1498 1499
+3  1343 1499 1344
+3  1344 1499 1500
+3  1344 1500 1345
+3  1345 1500 1346
+3  1346 1500 1501
+3  1346 1501 1347
+3  1347 1501 1502
+3  1347 1502 1503
+3  1347 1503 1348
+3  1348 1503 1349
+3  1349 1503 1504
+3  1349 1504 1505
+3  1349 1505 1350
+3  1350 1505 1506
+3  1350 1506 1351
+3  1351 1506 1507
+3  1351 1507 1352
+3  1352 1507 1508
+3  1352 1508 1353
+3  1353 1508 1509
+3  1353 1509 1354
+3  1354 1509 1510
+3  1354 1510 1355
+3  1355 1510 1511
+3  1355 1511 1356
+3  1356 1511 1357
+3  1357 1511 1512
+3  1357 1512 1359
+3  1359 1512 1360
+3  1360 1512 1513
+3  1360 1513 1514
+3  1360 1514 1361
+3  1361 1514 1515
+3  1361 1515 1362
+3  1362 1515 1516
+3  1362 1516 1363
+3  1363 1516 1517
+3  1363 1517 1364
+3  1364 1517 1518
+3  1364 1518 1519
+3  1364 1519 1520
+3  1364 1520 1365
+3  1365 1520 1366
+3  1366 1520 1521
+3  1366 1521 1522
+3  1366 1522 1523
+3  1366 1523 1367
+3  1367 1523 1524
+3  1367 1524 1369
+3  1369 1524 1525
+3  1369 1525 1370
+3  1370 1525 1371
+3  1371 1525 1526
+3  1371 1526 1527
+3  1371 1527 1372
+3  1372 1527 1528
+3  1372 1528 1373
+3  1373 1528 1529
+3  1373 1529 1374
+3  1374 1529 1530
+3  1374 1530 1375
+3  1375 1530 1531
+3  1375 1531 1376
+3  1376 1531 1532
+3  1376 1532 1377
+3  1377 1532 1533
+3  1377 1533 1378
+3  1378 1533 1534
+3  1378 1534 1379
+3  1379 1534 1535
+3  1379 1535 1380
+3  1380 1535 1536
+3  1380 1536 1381
+3  1381 1536 1537
+3  1381 1537 1382
+3  1382 1537 1538
+3  1382 1538 1383
+3  1383 1538 1539
+3  1383 1539 1384
+3  1384 1539 1540
+3  1384 1540 1385
+3  1385 1540 1541
+3  1385 1541 1386
+3  1386 1541 1542
+3  1386 1542 1387
+3  1387 1542 1543
+3  1387 1543 1544
+3  1387 1544 1388
+3  1388 1544 1545
+3  1388 1545 1389
+3  1389 1545 1390
+3  1390 1545 1546
+3  1390 1546 1391
+3  1391 1546 1547
+3  1391 1547 1548
+3  1391 1548 1392
+3  1392 1548 1549
+3  1392 1549 1550
+3  1392 1550 1393
+3  1393 1550 1551
+3  1393 1551 1394
+3  1394 1551 1552
+3  1394 1552 1553
+3  1394 1553 1395
+3  1395 1553 1396
+3  1396 1553 1554
+3  1396 1554 1397
+3  1397 1554 1555
+3  1397 1555 1398
+3  1398 1555 1399
+3  1399 1555 1556
+3  1399 1556 1400
+3  1400 1556 1557
+3  1400 1557 1558
+3  1400 1558 1401
+3  1401 1558 1559
+3  1401 1559 1402
+3  1402 1559 1560
+3  1402 1560 1403
+3  1403 1560 1561
+3  1403 1561 1405
+3  1405 1561 1562
+3  1405 1562 1406
+3  1406 1562 1563
+3  1406 1563 1407
+3  1407 1563 1564
+3  1407 1564 1408
+3  1408 1564 1565
+3  1408 1565 1409
+3  1409 1565 1566
+3  1409 1566 1410
+3  1410 1566 1567
+3  1410 1567 1568
+3  1410 1568 1411
+3  1411 1568 1412
+3  1412 1568 1569
+3  1412 1569 1413
+3  1413 1569 1570
+3  1413 1570 1414
+3  1414 1570 1571
+3  1414 1571 1415
+3  1415 1571 1572
+3  1415 1572 1573
+3  1415 1573 1416
+3  1416 1573 1574
+3  1416 1574 1417
+3  1417 1574 1575
+3  1417 1575 1418
+3  1418 1575 1576
+3  1418 1576 1419
+3  1419 1576 1577
+3  1419 1577 1420
+3  1420 1577 1578
+3  1420 1578 1579
+3  1420 1579 1421
+3  1421 1579 1580
+3  1421 1580 1422
+3  1422 1580 1423
+3  1423 1580 1424
+3  1424 1580 1581
+3  1424 1581 1582
+3  1424 1582 1583
+3  1424 1583 1425
+3  1425 1583 1584
+3  1425 1584 1426
+3  1426 1584 1585
+3  1426 1585 1586
+3  1426 1586 1427
+3  1427 1586 1587
+3  1427 1587 1428
+3  1428 1587 1588
+3  1428 1588 1429
+3  1429 1588 1589
+3  1429 1589 1430
+3  1430 1589 1431
+3  1431 1589 1590
+3  1431 1590 1591
+3  1431 1591 1432
+3  1432 1591 1592
+3  1432 1592 1433
+3  1433 1592 1434
+3  1434 1592 1593
+3  1434 1593 1435
+3  1435 1593 1594
+3  1435 1594 1436
+3  1436 1594 1595
+3  1436 1595 1596
+3  1436 1596 1437
+3  1437 1596 1597
+3  1437 1597 1598
+3  1437 1598 1438
+3  1438 1598 1439
+3  1439 1598 1599
+3  1439 1599 1600
+3  1439 1600 1440
+3  1440 1600 1601
+3  1440 1601 1441
+3  1441 1601 1602
+3  1441 1602 1442
+3  1442 1602 1603
+3  1442 1603 1604
+3  1442 1604 1443
+3  1443 1604 1605
+3  1443 1605 1606
+3  1443 1606 1444
+3  1444 1606 1445
+3  1445 1606 1607
+3  1445 1607 1446
+3  1446 1607 1447
+3  1447 1607 1608
+3  1447 1608 1448
+3  1448 1608 1609
+3  1448 1609 1449
+3  1449 1609 1610
+3  1449 1610 1450
+3  1450 1610 1611
+3  1450 1611 1612
+3  1450 1612 1451
+3  1451 1612 1613
+3  1451 1613 1452
+3  1452 1613 1614
+3  1452 1614 1453
+3  1453 1614 1615
+3  1453 1615 1616
+3  1453 1616 1454
+3  1454 1616 1617
+3  1454 1617 1455
+3  1455 1617 1618
+3  1455 1618 1456
+3  1456 1618 1619
+3  1456 1619 1620
+3  1456 1620 1621
+3  1456 1621 1457
+3  1457 1621 1458
+3  1458 1621 1622
+3  1458 1622 1459
+3  1459 1622 1460
+3  1460 1622 1623
+3  1460 1623 1624
+3  1460 1624 1461
+3  1461 1624 1625
+3  1461 1625 1626
+3  1461 1626 1627
+3  1461 1627 1462
+3  1462 1627 1463
+3  1463 1627 1464
+3  1464 1627 1628
+3  1464 1628 1629
+3  1464 1629 1465
+3  1465 1629 1630
+3  1465 1630 1466
+3  1466 1630 1467
+3  1467 1630 1631
+3  1467 1631 1468
+3  1468 1631 1632
+3  1468 1632 1469
+3  1469 1632 1633
+3  1469 1633 1634
+3  1469 1634 1470
+3  1470 1634 1635
+3  1470 1635 1471
+3  1471 1635 1472
+3  1472 1635 1636
+3  1472 1636 1473
+3  1473 1636 1637
+3  1473 1637 1474
+3  1474 1637 1638
+3  1474 1638 1475
+3  1475 1638 1639
+3  1475 1639 1476
+3  1476 1639 1640
+3  1476 1640 1477
+3  1477 1640 1641
+3  1477 1641 1478
+3  1478 1641 1642
+3  1478 1642 1643
+3  1478 1643 1479
+3  1479 1643 1644
+3  1479 1644 1480
+3  1480 1644 1645
+3  1480 1645 1481
+3  1481 1645 1646
+3  1481 1646 1482
+3  1482 1646 1647
+3  1482 1647 1483
+3  1483 1647 1648
+3  1483 1648 1484
+3  1484 1648 1649
+3  1484 1649 1485
+3  1485 1649 1650
+3  1485 1650 1486
+3  1486 1650 1651
+3  1486 1651 1487
+3  1487 1651 1652
+3  1487 1652 1488
+3  1488 1652 1653
+3  1488 1653 1489
+3  1489 1653 1654
+3  1489 1654 1490
+3  1490 1654 1655
+3  1490 1655 1491
+3  1491 1655 1656
+3  1491 1656 1492
+3  1492 1656 1657
+3  1492 1657 1493
+3  1493 1657 1658
+3  1493 1658 1494
+3  1494 1658 1659
+3  1494 1659 1495
+3  1495 1659 1660
+3  1495 1660 1496
+3  1496 1660 1661
+3  1496 1661 1497
+3  1497 1661 1662
+3  1497 1662 1498
+3  1498 1662 1663
+3  1498 1663 1664
+3  1498 1664 1499
+3  1499 1664 1665
+3  1499 1665 1500
+3  1500 1665 1501
+3  1501 1665 1666
+3  1501 1666 1502
+3  1502 1666 1667
+3  1502 1667 1668
+3  1502 1668 1503
+3  1503 1668 1504
+3  1504 1668 1669
+3  1504 1669 1670
+3  1504 1670 1505
+3  1505 1670 1671
+3  1505 1671 1506
+3  1506 1671 1672
+3  1506 1672 1507
+3  1507 1672 1673
+3  1507 1673 1508
+3  1508 1673 1674
+3  1508 1674 1509
+3  1509 1674 1675
+3  1509 1675 1510
+3  1510 1675 1676
+3  1510 1676 1511
+3  1511 1676 1512
+3  1512 1676 1513
+3  1513 1676 1677
+3  1513 1677 1678
+3  1513 1678 1514
+3  1514 1678 1679
+3  1514 1679 1515
+3  1515 1679 1680
+3  1515 1680 1516
+3  1516 1680 1681
+3  1516 1681 1517
+3  1517 1681 1682
+3  1517 1682 1518
+3  1518 1682 1683
+3  1518 1683 1684
+3  1518 1684 1519
+3  1519 1684 1685
+3  1519 1685 1521
+3  1521 1685 1686
+3  1521 1686 1522
+3  1522 1686 1687
+3  1522 1687 1688
+3  1522 1688 1523
+3  1523 1688 1689
+3  1523 1689 1524
+3  1524 1689 1690
+3  1524 1690 1525
+3  1525 1690 1691
+3  1525 1691 1526
+3  1526 1691 1692
+3  1526 1692 1693
+3  1526 1693 1527
+3  1527 1693 1694
+3  1527 1694 1528
+3  1528 1694 1695
+3  1528 1695 1529
+3  1529 1695 1696
+3  1529 1696 1530
+3  1530 1696 1697
+3  1530 1697 1531
+3  1531 1697 1698
+3  1531 1698 1532
+3  1532 1698 1699
+3  1532 1699 1533
+3  1533 1699 1700
+3  1533 1700 1534
+3  1534 1700 1701
+3  1534 1701 1535
+3  1535 1701 1702
+3  1535 1702 1536
+3  1536 1702 1703
+3  1536 1703 1537
+3  1537 1703 1704
+3  1537 1704 1538
+3  1538 1704 1705
+3  1538 1705 1539
+3  1539 1705 1706
+3  1539 1706 1540
+3  1540 1706 1707
+3  1540 1707 1541
+3  1541 1707 1708
+3  1541 1708 1542
+3  1542 1708 1709
+3  1542 1709 1543
+3  1543 1709 1710
+3  1543 1710 1544
+3  1544 1710 1711
+3  1544 1711 1546
+3  1546 1711 1547
+3  1547 1711 1712
+3  1547 1712 1713
+3  1547 1713 1548
+3  1548 1713 1714
+3  1548 1714 1549
+3  1549 1714 1715
+3  1549 1715 1716
+3  1549 1716 1550
+3  1550 1716 1717
+3  1550 1717 1718
+3  1550 1718 1551
+3  1551 1718 1719
+3  1551 1719 1552
+3  1552 1719 1720
+3  1552 1720 1553
+3  1553 1720 1721
+3  1553 1721 1554
+3  1554 1721 1555
+3  1555 1721 1556
+3  1556 1721 1722
+3  1556 1722 1557
+3  1557 1722 1723
+3  1557 1723 1724
+3  1557 1724 1558
+3  1558 1724 1725
+3  1558 1725 1559
+3  1559 1725 1726
+3  1559 1726 1560
+3  1560 1726 1727
+3  1560 1727 1728
+3  1560 1728 1561
+3  1561 1728 1562
+3  1562 1728 1729
+3  1562 1729 1563
+3  1563 1729 1730
+3  1563 1730 1564
+3  1564 1730 1731
+3  1564 1731 1565
+3  1565 1731 1732
+3  1565 1732 1566
+3  1566 1732 1733
+3  1566 1733 1567
+3  1567 1733 1734
+3  1567 1734 1735
+3  1567 1735 1568
+3  1568 1735 1569
+3  1569 1735 1736
+3  1569 1736 1570
+3  1570 1736 1737
+3  1570 1737 1571
+3  1571 1737 1738
+3  1571 1738 1572
+3  1572 1738 1739
+3  1572 1739 1740
+3  1572 1740 1573
+3  1573 1740 1574
+3  1574 1740 1741
+3  1574 1741 1575
+3  1575 1741 1742
+3  1575 1742 1576
+3  1576 1742 1743
+3  1576 1743 1577
+3  1577 1743 1744
+3  1577 1744 1578
+3  1578 1744 1745
+3  1578 1745 1746
+3  1578 1746 1747
+3  1578 1747 1579
+3  1579 1747 1580
+3  1580 1747 1581
+3  1581 1747 1748
+3  1581 1748 1582
+3  1582 1748 1749
+3  1582 1749 1750
+3  1582 1750 1583
+3  1583 1750 1584
+3  1584 1750 1751
+3  1584 1751 1585
+3  1585 1751 1752
+3  1585 1752 1753
+3  1585 1753 1586
+3  1586 1753 1754
+3  1586 1754 1587
+3  1587 1754 1755
+3  1587 1755 1588
+3  1588 1755 1756
+3  1588 1756 1589
+3  1589 1756 1590
+3  1590 1756 1757
+3  1590 1757 1758
+3  1590 1758 1591
+3  1591 1758 1759
+3  1591 1759 1592
+3  1592 1759 1593
+3  1593 1759 1760
+3  1593 1760 1761
+3  1593 1761 1594
+3  1594 1761 1595
+3  1595 1761 1762
+3  1595 1762 1596
+3  1596 1762 1763
+3  1596 1763 1764
+3  1596 1764 1597
+3  1597 1764 1765
+3  1597 1765 1766
+3  1597 1766 1598
+3  1598 1766 1599
+3  1599 1766 1767
+3  1599 1767 1768
+3  1599 1768 1600
+3  1600 1768 1769
+3  1600 1769 1601
+3  1601 1769 1770
+3  1601 1770 1603
+3  1603 1770 1771
+3  1603 1771 1772
+3  1603 1772 1604
+3  1604 1772 1773
+3  1604 1773 1774
+3  1604 1774 1605
+3  1605 1774 1775
+3  1605 1775 1606
+3  1606 1775 1608
+3  1608 1775 1609
+3  1609 1775 1776
+3  1609 1776 1777
+3  1609 1777 1610
+3  1610 1777 1778
+3  1610 1778 1611
+3  1611 1778 1779
+3  1611 1779 1780
+3  1611 1780 1612
+3  1612 1780 1613
+3  1613 1780 1781
+3  1613 1781 1614
+3  1614 1781 1782
+3  1614 1782 1615
+3  1615 1782 1783
+3  1615 1783 1784
+3  1615 1784 1616
+3  1616 1784 1785
+3  1616 1785 1617
+3  1617 1785 1786
+3  1617 1786 1618
+3  1618 1786 1787
+3  1618 1787 1619
+3  1619 1787 1788
+3  1619 1788 1789
+3  1619 1789 1620
+3  1620 1789 1790
+3  1620 1790 1621
+3  1621 1790 1791
+3  1621 1791 1622
+3  1622 1791 1623
+3  1623 1791 1792
+3  1623 1792 1793
+3  1623 1793 1624
+3  1624 1793 1794
+3  1624 1794 1625
+3  1625 1794 1795
+3  1625 1795 1796
+3  1625 1796 1626
+3  1626 1796 1797
+3  1626 1797 1628
+3  1628 1797 1629
+3  1629 1797 1798
+3  1629 1798 1630
+3  1630 1798 1631
+3  1631 1798 1799
+3  1631 1799 1632
+3  1632 1799 1633
+3  1633 1799 1800
+3  1633 1800 1801
+3  1633 1801 1634
+3  1634 1801 1802
+3  1634 1802 1635
+3  1635 1802 1636
+3  1636 1802 1803
+3  1636 1803 1637
+3  1637 1803 1804
+3  1637 1804 1638
+3  1638 1804 1805
+3  1638 1805 1639
+3  1639 1805 1806
+3  1639 1806 1640
+3  1640 1806 1807
+3  1640 1807 1641
+3  1641 1807 1808
+3  1641 1808 1642
+3  1642 1808 1809
+3  1642 1809 1810
+3  1642 1810 1643
+3  1643 1810 1811
+3  1643 1811 1644
+3  1644 1811 1812
+3  1644 1812 1645
+3  1645 1812 1813
+3  1645 1813 1646
+3  1646 1813 1814
+3  1646 1814 1647
+3  1647 1814 1815
+3  1647 1815 1648
+3  1648 1815 1816
+3  1648 1816 1649
+3  1649 1816 1817
+3  1649 1817 1650
+3  1650 1817 1818
+3  1650 1818 1651
+3  1651 1818 1819
+3  1651 1819 1652
+3  1652 1819 1820
+3  1652 1820 1653
+3  1653 1820 1821
+3  1653 1821 1654
+3  1654 1821 1822
+3  1654 1822 1655
+3  1655 1822 1823
+3  1655 1823 1656
+3  1656 1823 1824
+3  1656 1824 1657
+3  1657 1824 1825
+3  1657 1825 1658
+3  1658 1825 1826
+3  1658 1826 1659
+3  1659 1826 1827
+3  1659 1827 1660
+3  1660 1827 1828
+3  1660 1828 1661
+3  1661 1828 1829
+3  1661 1829 1662
+3  1662 1829 1830
+3  1662 1830 1663
+3  1663 1830 1831
+3  1663 1831 1832
+3  1663 1832 1664
+3  1664 1832 1833
+3  1664 1833 1665
+3  1665 1833 1666
+3  1666 1833 1834
+3  1666 1834 1667
+3  1667 1834 1835
+3  1667 1835 1836
+3  1667 1836 1668
+3  1668 1836 1669
+3  1669 1836 1837
+3  1669 1837 1838
+3  1669 1838 1670
+3  1670 1838 1839
+3  1670 1839 1671
+3  1671 1839 1840
+3  1671 1840 1672
+3  1672 1840 1841
+3  1672 1841 1673
+3  1673 1841 1842
+3  1673 1842 1674
+3  1674 1842 1843
+3  1674 1843 1675
+3  1675 1843 1677
+3  1677 1843 1844
+3  1677 1844 1678
+3  1678 1844 1679
+3  1679 1844 1845
+3  1679 1845 1846
+3  1679 1846 1680
+3  1680 1846 1681
+3  1681 1846 1847
+3  1681 1847 1682
+3  1682 1847 1848
+3  1682 1848 1683
+3  1683 1848 1849
+3  1683 1849 1850
+3  1683 1850 1684
+3  1684 1850 1851
+3  1684 1851 1852
+3  1684 1852 1685
+3  1685 1852 1853
+3  1685 1853 1686
+3  1686 1853 1687
+3  1687 1853 1854
+3  1687 1854 1855
+3  1687 1855 1856
+3  1687 1856 1688
+3  1688 1856 1857
+3  1688 1857 1689
+3  1689 1857 1690
+3  1690 1857 1691
+3  1691 1857 1858
+3  1691 1858 1692
+3  1692 1858 1859
+3  1692 1859 1860
+3  1692 1860 1693
+3  1693 1860 1861
+3  1693 1861 1694
+3  1694 1861 1862
+3  1694 1862 1695
+3  1695 1862 1863
+3  1695 1863 1696
+3  1696 1863 1864
+3  1696 1864 1697
+3  1697 1864 1865
+3  1697 1865 1698
+3  1698 1865 1866
+3  1698 1866 1699
+3  1699 1866 1867
+3  1699 1867 1700
+3  1700 1867 1868
+3  1700 1868 1701
+3  1701 1868 1869
+3  1701 1869 1702
+3  1702 1869 1870
+3  1702 1870 1703
+3  1703 1870 1871
+3  1703 1871 1704
+3  1704 1871 1872
+3  1704 1872 1705
+3  1705 1872 1873
+3  1705 1873 1706
+3  1706 1873 1874
+3  1706 1874 1707
+3  1707 1874 1875
+3  1707 1875 1708
+3  1708 1875 1876
+3  1708 1876 1709
+3  1709 1876 1877
+3  1709 1877 1710
+3  1710 1877 1878
+3  1710 1878 1879
+3  1710 1879 1711
+3  1711 1879 1712
+3  1712 1879 1880
+3  1712 1880 1881
+3  1712 1881 1713
+3  1713 1881 1882
+3  1713 1882 1714
+3  1714 1882 1883
+3  1714 1883 1715
+3  1715 1883 1884
+3  1715 1884 1885
+3  1715 1885 1716
+3  1716 1885 1886
+3  1716 1886 1717
+3  1717 1886 1887
+3  1717 1887 1888
+3  1717 1888 1718
+3  1718 1888 1719
+3  1719 1888 1889
+3  1719 1889 1890
+3  1719 1890 1720
+3  1720 1890 1722
+3  1722 1890 1891
+3  1722 1891 1723
+3  1723 1891 1892
+3  1723 1892 1893
+3  1723 1893 1724
+3  1724 1893 1894
+3  1724 1894 1725
+3  1725 1894 1895
+3  1725 1895 1726
+3  1726 1895 1896
+3  1726 1896 1897
+3  1726 1897 1727
+3  1727 1897 1898
+3  1727 1898 1728
+3  1728 1898 1729
+3  1729 1898 1899
+3  1729 1899 1730
+3  1730 1899 1900
+3  1730 1900 1731
+3  1731 1900 1901
+3  1731 1901 1732
+3  1732 1901 1902
+3  1732 1902 1733
+3  1733 1902 1903
+3  1733 1903 1734
+3  1734 1903 1904
+3  1734 1904 1905
+3  1734 1905 1735
+3  1735 1905 1736
+3  1736 1905 1906
+3  1736 1906 1737
+3  1737 1906 1907
+3  1737 1907 1738
+3  1738 1907 1908
+3  1738 1908 1739
+3  1739 1908 1909
+3  1739 1909 1910
+3  1739 1910 1740
+3  1740 1910 1741
+3  1741 1910 1911
+3  1741 1911 1742
+3  1742 1911 1912
+3  1742 1912 1743
+3  1743 1912 1913
+3  1743 1913 1744
+3  1744 1913 1914
+3  1744 1914 1745
+3  1745 1914 1915
+3  1745 1915 1916
+3  1745 1916 1746
+3  1746 1916 1917
+3  1746 1917 1747
+3  1747 1917 1748
+3  1748 1917 1918
+3  1748 1918 1919
+3  1748 1919 1749
+3  1749 1919 1920
+3  1749 1920 1750
+3  1750 1920 1751
+3  1751 1920 1921
+3  1751 1921 1752
+3  1752 1921 1922
+3  1752 1922 1753
+3  1753 1922 1923
+3  1753 1923 1924
+3  1753 1924 1754
+3  1754 1924 1925
+3  1754 1925 1755
+3  1755 1925 1926
+3  1755 1926 1756
+3  1756 1926 1757
+3  1757 1926 1927
+3  1757 1927 1928
+3  1757 1928 1758
+3  1758 1928 1929
+3  1758 1929 1759
+3  1759 1929 1930
+3  1759 1930 1760
+3  1760 1930 1761
+3  1761 1930 1762
+3  1762 1930 1763
+3  1763 1930 1931
+3  1763 1931 1932
+3  1763 1932 1764
+3  1764 1932 1933
+3  1764 1933 1765
+3  1765 1933 1934
+3  1765 1934 1935
+3  1765 1935 1766
+3  1766 1935 1767
+3  1767 1935 1936
+3  1767 1936 1937
+3  1767 1937 1768
+3  1768 1937 1938
+3  1768 1938 1769
+3  1769 1938 1939
+3  1769 1939 1770
+3  1770 1939 1771
+3  1771 1939 1940
+3  1771 1940 1941
+3  1771 1941 1772
+3  1772 1941 1942
+3  1772 1942 1773
+3  1773 1942 1774
+3  1774 1942 1943
+3  1774 1943 1776
+3  1776 1943 1777
+3  1777 1943 1944
+3  1777 1944 1945
+3  1777 1945 1946
+3  1777 1946 1778
+3  1778 1946 1947
+3  1778 1947 1948
+3  1778 1948 1779
+3  1779 1948 1949
+3  1779 1949 1780
+3  1780 1949 1950
+3  1780 1950 1781
+3  1781 1950 1951
+3  1781 1951 1782
+3  1782 1951 1783
+3  1783 1951 1952
+3  1783 1952 1953
+3  1783 1953 1784
+3  1784 1953 1954
+3  1784 1954 1785
+3  1785 1954 1955
+3  1785 1955 1786
+3  1786 1955 1956
+3  1786 1956 1787
+3  1787 1956 1957
+3  1787 1957 1788
+3  1788 1957 1958
+3  1788 1958 1959
+3  1788 1959 1789
+3  1789 1959 1960
+3  1789 1960 1790
+3  1790 1960 1961
+3  1790 1961 1791
+3  1791 1961 1792
+3  1792 1961 1962
+3  1792 1962 1963
+3  1792 1963 1793
+3  1793 1963 1964
+3  1793 1964 1794
+3  1794 1964 1965
+3  1794 1965 1795
+3  1795 1965 1966
+3  1795 1966 1967
+3  1795 1967 1968
+3  1795 1968 1796
+3  1796 1968 1969
+3  1796 1969 1797
+3  1797 1969 1799
+3  1799 1969 1800
+3  1800 1969 1970
+3  1800 1970 1971
+3  1800 1971 1801
+3  1801 1971 1972
+3  1801 1972 1802
+3  1802 1972 1803
+3  1803 1972 1973
+3  1803 1973 1804
+3  1804 1973 1974
+3  1804 1974 1805
+3  1805 1974 1975
+3  1805 1975 1806
+3  1806 1975 1976
+3  1806 1976 1807
+3  1807 1976 1977
+3  1807 1977 1808
+3  1808 1977 1978
+3  1808 1978 1809
+3  1809 1978 1979
+3  1809 1979 1980
+3  1809 1980 1810
+3  1810 1980 1981
+3  1810 1981 1811
+3  1811 1981 1982
+3  1811 1982 1812
+3  1812 1982 1983
+3  1812 1983 1813
+3  1813 1983 1984
+3  1813 1984 1814
+3  1814 1984 1985
+3  1814 1985 1815
+3  1815 1985 1986
+3  1815 1986 1816
+3  1816 1986 1987
+3  1816 1987 1817
+3  1817 1987 1988
+3  1817 1988 1818
+3  1818 1988 1989
+3  1818 1989 1819
+3  1819 1989 1990
+3  1819 1990 1820
+3  1820 1990 1991
+3  1820 1991 1821
+3  1821 1991 1992
+3  1821 1992 1822
+3  1822 1992 1993
+3  1822 1993 1823
+3  1823 1993 1994
+3  1823 1994 1824
+3  1824 1994 1995
+3  1824 1995 1825
+3  1825 1995 1996
+3  1825 1996 1826
+3  1826 1996 1997
+3  1826 1997 1827
+3  1827 1997 1998
+3  1827 1998 1828
+3  1828 1998 1999
+3  1828 1999 1829
+3  1829 1999 2000
+3  1829 2000 1830
+3  1830 2000 2001
+3  1830 2001 1831
+3  1831 2001 2002
+3  1831 2002 2003
+3  1831 2003 1832
+3  1832 2003 2004
+3  1832 2004 1833
+3  1833 2004 1834
+3  1834 2004 2005
+3  1834 2005 1835
+3  1835 2005 2006
+3  1835 2006 2007
+3  1835 2007 1836
+3  1836 2007 1837
+3  1837 2007 2008
+3  1837 2008 2009
+3  1837 2009 1838
+3  1838 2009 2010
+3  1838 2010 1839
+3  1839 2010 2011
+3  1839 2011 1840
+3  1840 2011 2012
+3  1840 2012 1841
+3  1841 2012 2013
+3  1841 2013 2014
+3  1841 2014 1842
+3  1842 2014 1843
+3  1843 2014 1844
+3  1844 2014 2015
+3  1844 2015 1845
+3  1845 2015 2016
+3  1845 2016 1846
+3  1846 2016 1847
+3  1847 2016 2017
+3  1847 2017 1848
+3  1848 2017 2018
+3  1848 2018 1849
+3  1849 2018 2019
+3  1849 2019 1850
+3  1850 2019 2020
+3  1850 2020 2021
+3  1850 2021 1851
+3  1851 2021 2022
+3  1851 2022 2023
+3  1851 2023 2024
+3  1851 2024 1852
+3  1852 2024 1853
+3  1853 2024 2025
+3  1853 2025 2026
+3  1853 2026 1854
+3  1854 2026 2027
+3  1854 2027 1855
+3  1855 2027 2028
+3  1855 2028 1856
+3  1856 2028 2029
+3  1856 2029 1857
+3  1857 2029 1858
+3  1858 2029 2030
+3  1858 2030 1859
+3  1859 2030 2031
+3  1859 2031 2032
+3  1859 2032 1860
+3  1860 2032 2033
+3  1860 2033 1861
+3  1861 2033 2034
+3  1861 2034 1862
+3  1862 2034 2035
+3  1862 2035 1863
+3  1863 2035 2036
+3  1863 2036 1864
+3  1864 2036 2037
+3  1864 2037 1865
+3  1865 2037 2038
+3  1865 2038 1866
+3  1866 2038 2039
+3  1866 2039 1867
+3  1867 2039 2040
+3  1867 2040 1868
+3  1868 2040 2041
+3  1868 2041 1869
+3  1869 2041 2042
+3  1869 2042 1870
+3  1870 2042 2043
+3  1870 2043 1871
+3  1871 2043 2044
+3  1871 2044 1872
+3  1872 2044 2045
+3  1872 2045 1873
+3  1873 2045 2046
+3  1873 2046 1874
+3  1874 2046 2047
+3  1874 2047 1875
+3  1875 2047 2048
+3  1875 2048 1876
+3  1876 2048 2049
+3  1876 2049 1877
+3  1877 2049 2050
+3  1877 2050 1878
+3  1878 2050 2051
+3  1878 2051 2052
+3  1878 2052 1879
+3  1879 2052 1880
+3  1880 2052 2053
+3  1880 2053 2054
+3  1880 2054 1881
+3  1881 2054 2055
+3  1881 2055 1882
+3  1882 2055 2056
+3  1882 2056 1883
+3  1883 2056 2057
+3  1883 2057 1884
+3  1884 2057 2058
+3  1884 2058 2059
+3  1884 2059 1885
+3  1885 2059 2060
+3  1885 2060 1886
+3  1886 2060 2061
+3  1886 2061 1887
+3  1887 2061 2062
+3  1887 2062 1888
+3  1888 2062 2063
+3  1888 2063 1889
+3  1889 2063 2064
+3  1889 2064 1891
+3  1891 2064 2065
+3  1891 2065 1892
+3  1892 2065 2066
+3  1892 2066 2067
+3  1892 2067 1893
+3  1893 2067 2068
+3  1893 2068 1894
+3  1894 2068 2069
+3  1894 2069 2070
+3  1894 2070 1895
+3  1895 2070 2071
+3  1895 2071 1896
+3  1896 2071 2072
+3  1896 2072 1897
+3  1897 2072 2073
+3  1897 2073 1898
+3  1898 2073 1899
+3  1899 2073 2074
+3  1899 2074 1900
+3  1900 2074 2075
+3  1900 2075 1901
+3  1901 2075 2076
+3  1901 2076 1902
+3  1902 2076 2077
+3  1902 2077 1903
+3  1903 2077 2078
+3  1903 2078 1904
+3  1904 2078 2079
+3  1904 2079 2080
+3  1904 2080 1905
+3  1905 2080 1906
+3  1906 2080 2081
+3  1906 2081 1907
+3  1907 2081 2082
+3  1907 2082 1908
+3  1908 2082 2083
+3  1908 2083 1909
+3  1909 2083 2084
+3  1909 2084 2085
+3  1909 2085 1910
+3  1910 2085 1911
+3  1911 2085 2086
+3  1911 2086 1912
+3  1912 2086 2087
+3  1912 2087 1913
+3  1913 2087 2088
+3  1913 2088 1914
+3  1914 2088 2089
+3  1914 2089 1915
+3  1915 2089 2090
+3  1915 2090 2091
+3  1915 2091 1916
+3  1916 2091 2092
+3  1916 2092 1917
+3  1917 2092 1918
+3  1918 2092 2093
+3  1918 2093 2094
+3  1918 2094 2095
+3  1918 2095 1919
+3  1919 2095 1920
+3  1920 2095 1921
+3  1921 2095 2096
+3  1921 2096 2097
+3  1921 2097 1922
+3  1922 2097 2098
+3  1922 2098 2099
+3  1922 2099 1923
+3  1923 2099 2100
+3  1923 2100 1924
+3  1924 2100 2101
+3  1924 2101 1925
+3  1925 2101 2102
+3  1925 2102 1926
+3  1926 2102 1927
+3  1927 2102 2103
+3  1927 2103 2104
+3  1927 2104 1928
+3  1928 2104 2105
+3  1928 2105 1929
+3  1929 2105 1930
+3  1930 2105 1931
+3  1931 2105 2106
+3  1931 2106 1932
+3  1932 2106 2107
+3  1932 2107 2108
+3  1932 2108 1933
+3  1933 2108 2109
+3  1933 2109 1934
+3  1934 2109 2110
+3  1934 2110 2111
+3  1934 2111 1935
+3  1935 2111 1936
+3  1936 2111 2112
+3  1936 2112 2113
+3  1936 2113 1937
+3  1937 2113 2114
+3  1937 2114 1938
+3  1938 2114 2115
+3  1938 2115 2116
+3  1938 2116 1939
+3  1939 2116 1940
+3  1940 2116 2117
+3  1940 2117 2118
+3  1940 2118 1941
+3  1941 2118 2119
+3  1941 2119 1942
+3  1942 2119 1944
+3  1944 2119 2120
+3  1944 2120 1945
+3  1945 2120 2121
+3  1945 2121 2122
+3  1945 2122 2123
+3  1945 2123 1946
+3  1946 2123 2124
+3  1946 2124 1947
+3  1947 2124 2125
+3  1947 2125 2126
+3  1947 2126 1948
+3  1948 2126 2127
+3  1948 2127 1949
+3  1949 2127 2128
+3  1949 2128 1950
+3  1950 2128 2129
+3  1950 2129 1951
+3  1951 2129 1952
+3  1952 2129 2130
+3  1952 2130 2131
+3  1952 2131 1953
+3  1953 2131 2132
+3  1953 2132 1954
+3  1954 2132 2133
+3  1954 2133 1955
+3  1955 2133 2134
+3  1955 2134 1956
+3  1956 2134 2135
+3  1956 2135 1957
+3  1957 2135 2136
+3  1957 2136 1958
+3  1958 2136 2137
+3  1958 2137 2138
+3  1958 2138 1959
+3  1959 2138 2139
+3  1959 2139 1960
+3  1960 2139 2140
+3  1960 2140 1961
+3  1961 2140 1962
+3  1962 2140 2141
+3  1962 2141 2142
+3  1962 2142 1963
+3  1963 2142 2143
+3  1963 2143 1964
+3  1964 2143 2144
+3  1964 2144 1965
+3  1965 2144 2145
+3  1965 2145 2146
+3  1965 2146 1966
+3  1966 2146 1975
+3  1975 2146 1976
+3  1976 2146 2147
+3  1976 2147 1977
+3  1977 2147 2148
+3  1977 2148 1978
+3  1978 2148 2149
+3  1978 2149 1979
+3  1979 2149 2150
+3  1979 2150 2151
+3  1979 2151 1980
+3  1980 2151 2152
+3  1980 2152 1981
+3  1981 2152 2153
+3  1981 2153 1982
+3  1982 2153 2154
+3  1982 2154 1983
+3  1983 2154 2155
+3  1983 2155 1984
+3  1984 2155 2156
+3  1984 2156 1985
+3  1985 2156 2157
+3  1985 2157 1986
+3  1986 2157 2158
+3  1986 2158 1987
+3  1987 2158 2159
+3  1987 2159 1988
+3  1988 2159 2160
+3  1988 2160 1989
+3  1989 2160 2161
+3  1989 2161 1990
+3  1990 2161 2162
+3  1990 2162 1991
+3  1991 2162 2163
+3  1991 2163 1992
+3  1992 2163 2164
+3  1992 2164 1993
+3  1993 2164 2165
+3  1993 2165 1994
+3  1994 2165 2166
+3  1994 2166 1995
+3  1995 2166 2167
+3  1995 2167 1996
+3  1996 2167 2168
+3  1996 2168 1997
+3  1997 2168 2169
+3  1997 2169 1998
+3  1998 2169 2170
+3  1998 2170 1999
+3  1999 2170 2171
+3  1999 2171 2000
+3  2000 2171 2172
+3  2000 2172 2001
+3  2001 2172 2173
+3  2001 2173 2002
+3  2002 2173 2174
+3  2002 2174 2175
+3  2002 2175 2003
+3  2003 2175 2176
+3  2003 2176 2004
+3  2004 2176 2005
+3  2005 2176 2177
+3  2005 2177 2006
+3  2006 2177 2178
+3  2006 2178 2179
+3  2006 2179 2007
+3  2007 2179 2008
+3  2008 2179 2180
+3  2008 2180 2181
+3  2008 2181 2009
+3  2009 2181 2182
+3  2009 2182 2010
+3  2010 2182 2183
+3  2010 2183 2011
+3  2011 2183 2184
+3  2011 2184 2012
+3  2012 2184 2185
+3  2012 2185 2013
+3  2013 2185 2014
+3  2014 2185 2015
+3  2015 2185 2016
+3  2016 2185 2186
+3  2016 2186 2017
+3  2017 2186 2187
+3  2017 2187 2018
+3  2018 2187 2188
+3  2018 2188 2019
+3  2019 2188 2189
+3  2019 2189 2020
+3  2020 2189 2190
+3  2020 2190 2191
+3  2020 2191 2021
+3  2021 2191 2192
+3  2021 2192 2022
+3  2022 2192 2193
+3  2022 2193 2023
+3  2023 2193 2194
+3  2023 2194 2024
+3  2024 2194 2025
+3  2025 2194 2026
+3  2026 2194 2195
+3  2026 2195 2196
+3  2026 2196 2027
+3  2027 2196 2197
+3  2027 2197 2028
+3  2028 2197 2198
+3  2028 2198 2199
+3  2028 2199 2029
+3  2029 2199 2030
+3  2030 2199 2200
+3  2030 2200 2031
+3  2031 2200 2201
+3  2031 2201 2202
+3  2031 2202 2032
+3  2032 2202 2203
+3  2032 2203 2033
+3  2033 2203 2204
+3  2033 2204 2034
+3  2034 2204 2205
+3  2034 2205 2035
+3  2035 2205 2206
+3  2035 2206 2036
+3  2036 2206 2207
+3  2036 2207 2037
+3  2037 2207 2208
+3  2037 2208 2038
+3  2038 2208 2209
+3  2038 2209 2039
+3  2039 2209 2210
+3  2039 2210 2040
+3  2040 2210 2211
+3  2040 2211 2041
+3  2041 2211 2212
+3  2041 2212 2042
+3  2042 2212 2213
+3  2042 2213 2043
+3  2043 2213 2214
+3  2043 2214 2044
+3  2044 2214 2215
+3  2044 2215 2045
+3  2045 2215 2216
+3  2045 2216 2046
+3  2046 2216 2217
+3  2046 2217 2047
+3  2047 2217 2218
+3  2047 2218 2048
+3  2048 2218 2219
+3  2048 2219 2049
+3  2049 2219 2220
+3  2049 2220 2221
+3  2049 2221 2050
+3  2050 2221 2222
+3  2050 2222 2051
+3  2051 2222 2223
+3  2051 2223 2052
+3  2052 2223 2053
+3  2053 2223 2224
+3  2053 2224 2225
+3  2053 2225 2054
+3  2054 2225 2226
+3  2054 2226 2055
+3  2055 2226 2227
+3  2055 2227 2056
+3  2056 2227 2228
+3  2056 2228 2057
+3  2057 2228 2229
+3  2057 2229 2058
+3  2058 2229 2230
+3  2058 2230 2231
+3  2058 2231 2059
+3  2059 2231 2232
+3  2059 2232 2060
+3  2060 2232 2233
+3  2060 2233 2234
+3  2060 2234 2061
+3  2061 2234 2062
+3  2062 2234 2235
+3  2062 2235 2063
+3  2063 2235 2236
+3  2063 2236 2237
+3  2063 2237 2064
+3  2064 2237 2065
+3  2065 2237 2238
+3  2065 2238 2239
+3  2065 2239 2066
+3  2066 2239 2240
+3  2066 2240 2241
+3  2066 2241 2067
+3  2067 2241 2242
+3  2067 2242 2068
+3  2068 2242 2243
+3  2068 2243 2069
+3  2069 2243 2244
+3  2069 2244 2245
+3  2069 2245 2070
+3  2070 2245 2246
+3  2070 2246 2071
+3  2071 2246 2247
+3  2071 2247 2072
+3  2072 2247 2248
+3  2072 2248 2073
+3  2073 2248 2074
+3  2074 2248 2249
+3  2074 2249 2075
+3  2075 2249 2250
+3  2075 2250 2076
+3  2076 2250 2251
+3  2076 2251 2077
+3  2077 2251 2252
+3  2077 2252 2078
+3  2078 2252 2253
+3  2078 2253 2079
+3  2079 2253 2254
+3  2079 2254 2255
+3  2079 2255 2080
+3  2080 2255 2081
+3  2081 2255 2256
+3  2081 2256 2082
+3  2082 2256 2257
+3  2082 2257 2083
+3  2083 2257 2258
+3  2083 2258 2084
+3  2084 2258 2259
+3  2084 2259 2260
+3  2084 2260 2085
+3  2085 2260 2086
+3  2086 2260 2261
+3  2086 2261 2087
+3  2087 2261 2262
+3  2087 2262 2088
+3  2088 2262 2263
+3  2088 2263 2089
+3  2089 2263 2264
+3  2089 2264 2090
+3  2090 2264 2265
+3  2090 2265 2266
+3  2090 2266 2091
+3  2091 2266 2267
+3  2091 2267 2092
+3  2092 2267 2093
+3  2093 2267 2268
+3  2093 2268 2269
+3  2093 2269 2094
+3  2094 2269 2270
+3  2094 2270 2096
+3  2096 2270 2271
+3  2096 2271 2272
+3  2096 2272 2097
+3  2097 2272 2273
+3  2097 2273 2274
+3  2097 2274 2098
+3  2098 2274 2275
+3  2098 2275 2099
+3  2099 2275 2276
+3  2099 2276 2277
+3  2099 2277 2100
+3  2100 2277 2278
+3  2100 2278 2101
+3  2101 2278 2279
+3  2101 2279 2102
+3  2102 2279 2103
+3  2103 2279 2280
+3  2103 2280 2281
+3  2103 2281 2104
+3  2104 2281 2282
+3  2104 2282 2283
+3  2104 2283 2105
+3  2105 2283 2106
+3  2106 2283 2284
+3  2106 2284 2107
+3  2107 2284 2285
+3  2107 2285 2108
+3  2108 2285 2286
+3  2108 2286 2287
+3  2108 2287 2109
+3  2109 2287 2288
+3  2109 2288 2110
+3  2110 2288 2289
+3  2110 2289 2290
+3  2110 2290 2111
+3  2111 2290 2112
+3  2112 2290 2291
+3  2112 2291 2292
+3  2112 2292 2113
+3  2113 2292 2293
+3  2113 2293 2114
+3  2114 2293 2294
+3  2114 2294 2115
+3  2115 2294 2295
+3  2115 2295 2116
+3  2116 2295 2117
+3  2117 2295 2296
+3  2117 2296 2297
+3  2117 2297 2118
+3  2118 2297 2298
+3  2118 2298 2119
+3  2119 2298 2120
+3  2120 2298 2121
+3  2121 2298 2297
+3  2121 2297 2299
+3  2121 2299 2122
+3  2122 2299 2300
+3  2122 2300 2301
+3  2122 2301 2302
+3  2122 2302 2123
+3  2123 2302 2124
+3  2124 2302 2303
+3  2124 2303 2125
+3  2125 2303 2304
+3  2125 2304 2305
+3  2125 2305 2126
+3  2126 2305 2306
+3  2126 2306 2127
+3  2127 2306 2307
+3  2127 2307 2128
+3  2128 2307 2308
+3  2128 2308 2129
+3  2129 2308 2130
+3  2130 2308 2309
+3  2130 2309 2310
+3  2130 2310 2131
+3  2131 2310 2311
+3  2131 2311 2132
+3  2132 2311 2312
+3  2132 2312 2133
+3  2133 2312 2313
+3  2133 2313 2134
+3  2134 2313 2314
+3  2134 2314 2135
+3  2135 2314 2315
+3  2135 2315 2136
+3  2136 2315 2137
+3  2137 2315 2316
+3  2137 2316 2317
+3  2137 2317 2318
+3  2137 2318 2138
+3  2138 2318 2139
+3  2139 2318 2319
+3  2139 2319 2140
+3  2140 2319 2141
+3  2141 2319 2320
+3  2141 2320 2321
+3  2141 2321 2142
+3  2142 2321 2322
+3  2142 2322 2143
+3  2143 2322 2323
+3  2143 2323 2144
+3  2144 2323 2324
+3  2144 2324 2325
+3  2144 2325 2145
+3  2145 2325 2147
+3  2147 2325 2148
+3  2148 2325 2326
+3  2148 2326 2149
+3  2149 2326 2327
+3  2149 2327 2150
+3  2150 2327 2328
+3  2150 2328 2329
+3  2150 2329 2151
+3  2151 2329 2330
+3  2151 2330 2152
+3  2152 2330 2331
+3  2152 2331 2153
+3  2153 2331 2332
+3  2153 2332 2154
+3  2154 2332 2333
+3  2154 2333 2155
+3  2155 2333 2334
+3  2155 2334 2156
+3  2156 2334 2335
+3  2156 2335 2157
+3  2157 2335 2336
+3  2157 2336 2158
+3  2158 2336 2337
+3  2158 2337 2159
+3  2159 2337 2338
+3  2159 2338 2160
+3  2160 2338 2339
+3  2160 2339 2161
+3  2161 2339 2340
+3  2161 2340 2162
+3  2162 2340 2341
+3  2162 2341 2163
+3  2163 2341 2342
+3  2163 2342 2164
+3  2164 2342 2343
+3  2164 2343 2165
+3  2165 2343 2344
+3  2165 2344 2166
+3  2166 2344 2345
+3  2166 2345 2167
+3  2167 2345 2346
+3  2167 2346 2168
+3  2168 2346 2347
+3  2168 2347 2169
+3  2169 2347 2348
+3  2169 2348 2170
+3  2170 2348 2349
+3  2170 2349 2171
+3  2171 2349 2350
+3  2171 2350 2172
+3  2172 2350 2351
+3  2172 2351 2173
+3  2173 2351 2352
+3  2173 2352 2174
+3  2174 2352 2353
+3  2174 2353 2354
+3  2174 2354 2175
+3  2175 2354 2355
+3  2175 2355 2176
+3  2176 2355 2177
+3  2177 2355 2356
+3  2177 2356 2178
+3  2178 2356 2357
+3  2178 2357 2358
+3  2178 2358 2179
+3  2179 2358 2180
+3  2180 2358 2359
+3  2180 2359 2360
+3  2180 2360 2181
+3  2181 2360 2361
+3  2181 2361 2182
+3  2182 2361 2362
+3  2182 2362 2183
+3  2183 2362 2363
+3  2183 2363 2184
+3  2184 2363 2186
+3  2186 2363 2187
+3  2187 2363 2364
+3  2187 2364 2188
+3  2188 2364 2365
+3  2188 2365 2189
+3  2189 2365 2366
+3  2189 2366 2367
+3  2189 2367 2190
+3  2190 2367 2368
+3  2190 2368 2191
+3  2191 2368 2192
+3  2192 2368 2369
+3  2192 2369 2370
+3  2192 2370 2193
+3  2193 2370 2371
+3  2193 2371 2372
+3  2193 2372 2195
+3  2195 2372 2373
+3  2195 2373 2196
+3  2196 2373 2374
+3  2196 2374 2375
+3  2196 2375 2197
+3  2197 2375 2376
+3  2197 2376 2198
+3  2198 2376 2377
+3  2198 2377 2378
+3  2198 2378 2199
+3  2199 2378 2200
+3  2200 2378 2379
+3  2200 2379 2201
+3  2201 2379 2380
+3  2201 2380 2381
+3  2201 2381 2202
+3  2202 2381 2382
+3  2202 2382 2203
+3  2203 2382 2383
+3  2203 2383 2204
+3  2204 2383 2384
+3  2204 2384 2205
+3  2205 2384 2385
+3  2205 2385 2206
+3  2206 2385 2386
+3  2206 2386 2207
+3  2207 2386 2387
+3  2207 2387 2208
+3  2208 2387 2388
+3  2208 2388 2209
+3  2209 2388 2389
+3  2209 2389 2210
+3  2210 2389 2390
+3  2210 2390 2211
+3  2211 2390 2391
+3  2211 2391 2212
+3  2212 2391 2392
+3  2212 2392 2213
+3  2213 2392 2393
+3  2213 2393 2214
+3  2214 2393 2394
+3  2214 2394 2215
+3  2215 2394 2395
+3  2215 2395 2216
+3  2216 2395 2396
+3  2216 2396 2217
+3  2217 2396 2397
+3  2217 2397 2218
+3  2218 2397 2398
+3  2218 2398 2219
+3  2219 2398 2399
+3  2219 2399 2220
+3  2220 2399 2400
+3  2220 2400 2401
+3  2220 2401 2221
+3  2221 2401 2402
+3  2221 2402 2222
+3  2222 2402 2224
+3  2224 2402 2403
+3  2224 2403 2225
+3  2225 2403 2226
+3  2226 2403 2404
+3  2226 2404 2405
+3  2226 2405 2227
+3  2227 2405 2406
+3  2227 2406 2228
+3  2228 2406 2407
+3  2228 2407 2229
+3  2229 2407 2408
+3  2229 2408 2230
+3  2230 2408 2409
+3  2230 2409 2410
+3  2230 2410 2231
+3  2231 2410 2411
+3  2231 2411 2232
+3  2232 2411 2412
+3  2232 2412 2413
+3  2232 2413 2233
+3  2233 2413 2414
+3  2233 2414 2234
+3  2234 2414 2415
+3  2234 2415 2416
+3  2234 2416 2235
+3  2235 2416 2417
+3  2235 2417 2236
+3  2236 2417 2237
+3  2237 2417 2238
+3  2238 2417 2239
+3  2239 2417 2418
+3  2239 2418 2240
+3  2240 2418 2419
+3  2240 2419 2420
+3  2240 2420 2241
+3  2241 2420 2421
+3  2241 2421 2242
+3  2242 2421 2422
+3  2242 2422 2243
+3  2243 2422 2423
+3  2243 2423 2244
+3  2244 2423 2424
+3  2244 2424 2245
+3  2245 2424 2425
+3  2245 2425 2246
+3  2246 2425 2426
+3  2246 2426 2427
+3  2246 2427 2247
+3  2247 2427 2428
+3  2247 2428 2248
+3  2248 2428 2249
+3  2249 2428 2429
+3  2249 2429 2250
+3  2250 2429 2430
+3  2250 2430 2251
+3  2251 2430 2431
+3  2251 2431 2252
+3  2252 2431 2432
+3  2252 2432 2253
+3  2253 2432 2433
+3  2253 2433 2254
+3  2254 2433 2434
+3  2254 2434 2435
+3  2254 2435 2255
+3  2255 2435 2256
+3  2256 2435 2436
+3  2256 2436 2257
+3  2257 2436 2437
+3  2257 2437 2258
+3  2258 2437 2438
+3  2258 2438 2259
+3  2259 2438 2439
+3  2259 2439 2440
+3  2259 2440 2260
+3  2260 2440 2261
+3  2261 2440 2441
+3  2261 2441 2262
+3  2262 2441 2442
+3  2262 2442 2263
+3  2263 2442 2443
+3  2263 2443 2264
+3  2264 2443 2444
+3  2264 2444 2265
+3  2265 2444 2445
+3  2265 2445 2446
+3  2265 2446 2266
+3  2266 2446 2447
+3  2266 2447 2267
+3  2267 2447 2268
+3  2268 2447 2448
+3  2268 2448 2449
+3  2268 2449 2269
+3  2269 2449 2450
+3  2269 2450 2270
+3  2270 2450 2271
+3  2271 2450 2451
+3  2271 2451 2452
+3  2271 2452 2272
+3  2272 2452 2273
+3  2273 2452 2453
+3  2273 2453 2454
+3  2273 2454 2274
+3  2274 2454 2275
+3  2275 2454 2455
+3  2275 2455 2276
+3  2276 2455 2456
+3  2276 2456 2277
+3  2277 2456 2457
+3  2277 2457 2278
+3  2278 2457 2458
+3  2278 2458 2459
+3  2278 2459 2279
+3  2279 2459 2280
+3  2280 2459 2460
+3  2280 2460 2461
+3  2280 2461 2281
+3  2281 2461 2462
+3  2281 2462 2282
+3  2282 2462 2284
+3  2284 2462 2463
+3  2284 2463 2285
+3  2285 2463 2464
+3  2285 2464 2286
+3  2286 2464 2465
+3  2286 2465 2466
+3  2286 2466 2287
+3  2287 2466 2467
+3  2287 2467 2288
+3  2288 2467 2468
+3  2288 2468 2289
+3  2289 2468 2469
+3  2289 2469 2470
+3  2289 2470 2290
+3  2290 2470 2291
+3  2291 2470 2471
+3  2291 2471 2472
+3  2291 2472 2292
+3  2292 2472 2473
+3  2292 2473 2293
+3  2293 2473 2474
+3  2293 2474 2294
+3  2294 2474 2475
+3  2294 2475 2295
+3  2295 2475 2296
+3  2296 2475 2476
+3  2296 2476 2299
+3  2299 2476 2477
+3  2299 2477 2300
+3  2300 2477 2478
+3  2300 2478 2301
+3  2301 2478 2479
+3  2301 2479 2480
+3  2301 2480 2302
+3  2302 2480 2303
+3  2303 2480 2481
+3  2303 2481 2304
+3  2304 2481 2482
+3  2304 2482 2483
+3  2304 2483 2305
+3  2305 2483 2484
+3  2305 2484 2306
+3  2306 2484 2485
+3  2306 2485 2307
+3  2307 2485 2486
+3  2307 2486 2308
+3  2308 2486 2309
+3  2309 2486 2487
+3  2309 2487 2488
+3  2309 2488 2310
+3  2310 2488 2489
+3  2310 2489 2311
+3  2311 2489 2490
+3  2311 2490 2312
+3  2312 2490 2491
+3  2312 2491 2313
+3  2313 2491 2492
+3  2313 2492 2314
+3  2314 2492 2315
+3  2315 2492 2493
+3  2315 2493 2316
+3  2316 2493 2494
+3  2316 2494 2495
+3  2316 2495 2317
+3  2317 2495 2496
+3  2317 2496 2497
+3  2317 2497 2318
+3  2318 2497 2320
+3  2320 2497 2498
+3  2320 2498 2321
+3  2321 2498 2499
+3  2321 2499 2322
+3  2322 2499 2500
+3  2322 2500 2501
+3  2322 2501 2323
+3  2323 2501 2324
+3  2324 2501 2502
+3  2324 2502 2326
+3  2326 2502 2327
+3  2327 2502 2503
+3  2327 2503 2504
+3  2327 2504 2328
+3  2328 2504 2505
+3  2328 2505 2329
+3  2329 2505 2506
+3  2329 2506 2330
+3  2330 2506 2507
+3  2330 2507 2331
+3  2331 2507 2508
+3  2331 2508 2332
+3  2332 2508 2509
+3  2332 2509 2333
+3  2333 2509 2510
+3  2333 2510 2334
+3  2334 2510 2511
+3  2334 2511 2335
+3  2335 2511 2512
+3  2335 2512 2336
+3  2336 2512 2513
+3  2336 2513 2337
+3  2337 2513 2514
+3  2337 2514 2338
+3  2338 2514 2515
+3  2338 2515 2339
+3  2339 2515 2516
+3  2339 2516 2340
+3  2340 2516 2517
+3  2340 2517 2341
+3  2341 2517 2518
+3  2341 2518 2342
+3  2342 2518 2519
+3  2342 2519 2343
+3  2343 2519 2520
+3  2343 2520 2344
+3  2344 2520 2521
+3  2344 2521 2345
+3  2345 2521 2522
+3  2345 2522 2346
+3  2346 2522 2523
+3  2346 2523 2347
+3  2347 2523 2524
+3  2347 2524 2348
+3  2348 2524 2525
+3  2348 2525 2349
+3  2349 2525 2526
+3  2349 2526 2350
+3  2350 2526 2527
+3  2350 2527 2351
+3  2351 2527 2528
+3  2351 2528 2352
+3  2352 2528 2529
+3  2352 2529 2353
+3  2353 2529 2530
+3  2353 2530 2531
+3  2353 2531 2354
+3  2354 2531 2532
+3  2354 2532 2355
+3  2355 2532 2356
+3  2356 2532 2533
+3  2356 2533 2357
+3  2357 2533 2534
+3  2357 2534 2535
+3  2357 2535 2358
+3  2358 2535 2359
+3  2359 2535 2536
+3  2359 2536 2537
+3  2359 2537 2360
+3  2360 2537 2538
+3  2360 2538 2361
+3  2361 2538 2539
+3  2361 2539 2362
+3  2362 2539 2364
+3  2364 2539 2365
+3  2365 2539 2366
+3  2366 2539 2538
+3  2366 2538 2540
+3  2366 2540 2541
+3  2366 2541 2542
+3  2366 2542 2367
+3  2367 2542 2543
+3  2367 2543 2368
+3  2368 2543 2544
+3  2368 2544 2369
+3  2369 2544 2545
+3  2369 2545 2546
+3  2369 2546 2370
+3  2370 2546 2547
+3  2370 2547 2371
+3  2371 2547 2548
+3  2371 2548 2549
+3  2371 2549 2372
+3  2372 2549 2550
+3  2372 2550 2373
+3  2373 2550 2551
+3  2373 2551 2374
+3  2374 2551 2552
+3  2374 2552 2375
+3  2375 2552 2376
+3  2376 2552 2377
+3  2377 2552 2553
+3  2377 2553 2554
+3  2377 2554 2555
+3  2377 2555 2379
+3  2379 2555 2556
+3  2379 2556 2380
+3  2380 2556 2557
+3  2380 2557 2381
+3  2381 2557 2382
+3  2382 2557 2558
+3  2382 2558 2383
+3  2383 2558 2559
+3  2383 2559 2384
+3  2384 2559 2560
+3  2384 2560 2385
+3  2385 2560 2561
+3  2385 2561 2386
+3  2386 2561 2562
+3  2386 2562 2387
+3  2387 2562 2563
+3  2387 2563 2388
+3  2388 2563 2564
+3  2388 2564 2389
+3  2389 2564 2565
+3  2389 2565 2390
+3  2390 2565 2566
+3  2390 2566 2391
+3  2391 2566 2567
+3  2391 2567 2392
+3  2392 2567 2568
+3  2392 2568 2393
+3  2393 2568 2569
+3  2393 2569 2394
+3  2394 2569 2570
+3  2394 2570 2395
+3  2395 2570 2571
+3  2395 2571 2396
+3  2396 2571 2572
+3  2396 2572 2397
+3  2397 2572 2573
+3  2397 2573 2398
+3  2398 2573 2574
+3  2398 2574 2399
+3  2399 2574 2575
+3  2399 2575 2400
+3  2400 2575 2576
+3  2400 2576 2577
+3  2400 2577 2401
+3  2401 2577 2578
+3  2401 2578 2402
+3  2402 2578 2579
+3  2402 2579 2403
+3  2403 2579 2404
+3  2404 2579 2580
+3  2404 2580 2581
+3  2404 2581 2405
+3  2405 2581 2582
+3  2405 2582 2406
+3  2406 2582 2583
+3  2406 2583 2407
+3  2407 2583 2584
+3  2407 2584 2408
+3  2408 2584 2585
+3  2408 2585 2409
+3  2409 2585 2586
+3  2409 2586 2587
+3  2409 2587 2410
+3  2410 2587 2588
+3  2410 2588 2411
+3  2411 2588 2589
+3  2411 2589 2412
+3  2412 2589 2590
+3  2412 2590 2413
+3  2413 2590 2591
+3  2413 2591 2414
+3  2414 2591 2592
+3  2414 2592 2593
+3  2414 2593 2415
+3  2415 2593 2594
+3  2415 2594 2595
+3  2415 2595 2416
+3  2416 2595 2418
+3  2418 2595 2596
+3  2418 2596 2419
+3  2419 2596 2597
+3  2419 2597 2598
+3  2419 2598 2420
+3  2420 2598 2599
+3  2420 2599 2600
+3  2420 2600 2421
+3  2421 2600 2601
+3  2421 2601 2422
+3  2422 2601 2602
+3  2422 2602 2423
+3  2423 2602 2603
+3  2423 2603 2424
+3  2424 2603 2604
+3  2424 2604 2425
+3  2425 2604 2605
+3  2425 2605 2606
+3  2425 2606 2426
+3  2426 2606 2607
+3  2426 2607 2427
+3  2427 2607 2428
+3  2428 2607 2608
+3  2428 2608 2429
+3  2429 2608 2609
+3  2429 2609 2430
+3  2430 2609 2610
+3  2430 2610 2431
+3  2431 2610 2611
+3  2431 2611 2432
+3  2432 2611 2612
+3  2432 2612 2433
+3  2433 2612 2613
+3  2433 2613 2434
+3  2434 2613 2614
+3  2434 2614 2615
+3  2434 2615 2435
+3  2435 2615 2436
+3  2436 2615 2616
+3  2436 2616 2437
+3  2437 2616 2617
+3  2437 2617 2438
+3  2438 2617 2618
+3  2438 2618 2439
+3  2439 2618 2619
+3  2439 2619 2620
+3  2439 2620 2440
+3  2440 2620 2441
+3  2441 2620 2621
+3  2441 2621 2442
+3  2442 2621 2622
+3  2442 2622 2443
+3  2443 2622 2623
+3  2443 2623 2444
+3  2444 2623 2624
+3  2444 2624 2445
+3  2445 2624 2625
+3  2445 2625 2626
+3  2445 2626 2446
+3  2446 2626 2627
+3  2446 2627 2447
+3  2447 2627 2448
+3  2448 2627 2628
+3  2448 2628 2629
+3  2448 2629 2449
+3  2449 2629 2630
+3  2449 2630 2450
+3  2450 2630 2451
+3  2451 2630 2631
+3  2451 2631 2632
+3  2451 2632 2452
+3  2452 2632 2633
+3  2452 2633 2453
+3  2453 2633 2634
+3  2453 2634 2455
+3  2455 2634 2635
+3  2455 2635 2456
+3  2456 2635 2636
+3  2456 2636 2457
+3  2457 2636 2637
+3  2457 2637 2458
+3  2458 2637 2638
+3  2458 2638 2639
+3  2458 2639 2459
+3  2459 2639 2640
+3  2459 2640 2460
+3  2460 2640 2641
+3  2460 2641 2642
+3  2460 2642 2461
+3  2461 2642 2462
+3  2462 2642 2463
+3  2463 2642 2464
+3  2464 2642 2641
+3  2464 2641 2465
+3  2465 2641 2643
+3  2465 2643 2644
+3  2465 2644 2466
+3  2466 2644 2645
+3  2466 2645 2467
+3  2467 2645 2646
+3  2467 2646 2468
+3  2468 2646 2647
+3  2468 2647 2469
+3  2469 2647 2648
+3  2469 2648 2649
+3  2469 2649 2470
+3  2470 2649 2471
+3  2471 2649 2650
+3  2471 2650 2651
+3  2471 2651 2472
+3  2472 2651 2652
+3  2472 2652 2473
+3  2473 2652 2653
+3  2473 2653 2654
+3  2473 2654 2474
+3  2474 2654 2655
+3  2474 2655 2475
+3  2475 2655 2476
+3  2476 2655 2477
+3  2477 2655 2656
+3  2477 2656 2657
+3  2477 2657 2478
+3  2478 2657 2658
+3  2478 2658 2479
+3  2479 2658 2659
+3  2479 2659 2660
+3  2479 2660 2480
+3  2480 2660 2481
+3  2481 2660 2661
+3  2481 2661 2482
+3  2482 2661 2662
+3  2482 2662 2663
+3  2482 2663 2483
+3  2483 2663 2664
+3  2483 2664 2484
+3  2484 2664 2665
+3  2484 2665 2485
+3  2485 2665 2666
+3  2485 2666 2486
+3  2486 2666 2487
+3  2487 2666 2667
+3  2487 2667 2668
+3  2487 2668 2488
+3  2488 2668 2669
+3  2488 2669 2489
+3  2489 2669 2670
+3  2489 2670 2490
+3  2490 2670 2671
+3  2490 2671 2491
+3  2491 2671 2672
+3  2491 2672 2492
+3  2492 2672 2493
+3  2493 2672 2673
+3  2493 2673 2494
+3  2494 2673 2674
+3  2494 2674 2675
+3  2494 2675 2495
+3  2495 2675 2676
+3  2495 2676 2496
+3  2496 2676 2677
+3  2496 2677 2678
+3  2496 2678 2497
+3  2497 2678 2498
+3  2498 2678 2679
+3  2498 2679 2680
+3  2498 2680 2499
+3  2499 2680 2681
+3  2499 2681 2500
+3  2500 2681 2682
+3  2500 2682 2501
+3  2501 2682 2502
+3  2502 2682 2503
+3  2503 2682 2683
+3  2503 2683 2504
+3  2504 2683 2684
+3  2504 2684 2685
+3  2504 2685 2505
+3  2505 2685 2686
+3  2505 2686 2506
+3  2506 2686 2687
+3  2506 2687 2507
+3  2507 2687 2688
+3  2507 2688 2508
+3  2508 2688 2689
+3  2508 2689 2509
+3  2509 2689 2690
+3  2509 2690 2510
+3  2510 2690 2691
+3  2510 2691 2511
+3  2511 2691 2692
+3  2511 2692 2512
+3  2512 2692 2693
+3  2512 2693 2513
+3  2513 2693 2694
+3  2513 2694 2514
+3  2514 2694 2695
+3  2514 2695 2515
+3  2515 2695 2696
+3  2515 2696 2516
+3  2516 2696 2697
+3  2516 2697 2517
+3  2517 2697 2698
+3  2517 2698 2518
+3  2518 2698 2699
+3  2518 2699 2519
+3  2519 2699 2700
+3  2519 2700 2520
+3  2520 2700 2701
+3  2520 2701 2521
+3  2521 2701 2702
+3  2521 2702 2522
+3  2522 2702 2703
+3  2522 2703 2523
+3  2523 2703 2704
+3  2523 2704 2524
+3  2524 2704 2705
+3  2524 2705 2525
+3  2525 2705 2706
+3  2525 2706 2526
+3  2526 2706 2707
+3  2526 2707 2527
+3  2527 2707 2708
+3  2527 2708 2528
+3  2528 2708 2709
+3  2528 2709 2529
+3  2529 2709 2710
+3  2529 2710 2530
+3  2530 2710 2711
+3  2530 2711 2712
+3  2530 2712 2531
+3  2531 2712 2713
+3  2531 2713 2532
+3  2532 2713 2533
+3  2533 2713 2714
+3  2533 2714 2534
+3  2534 2714 2715
+3  2534 2715 2716
+3  2534 2716 2535
+3  2535 2716 2536
+3  2536 2716 2717
+3  2536 2717 2718
+3  2536 2718 2537
+3  2537 2718 2540
+3  2540 2718 2719
+3  2540 2719 2541
+3  2541 2719 2720
+3  2541 2720 2721
+3  2541 2721 2542
+3  2542 2721 2722
+3  2542 2722 2543
+3  2543 2722 2723
+3  2543 2723 2544
+3  2544 2723 2724
+3  2544 2724 2545
+3  2545 2724 2725
+3  2545 2725 2726
+3  2545 2726 2546
+3  2546 2726 2547
+3  2547 2726 2727
+3  2547 2727 2548
+3  2548 2727 2728
+3  2548 2728 2549
+3  2549 2728 2729
+3  2549 2729 2551
+3  2551 2729 2554
+3  2554 2729 2730
+3  2554 2730 2731
+3  2554 2731 2555
+3  2555 2731 2732
+3  2555 2732 2556
+3  2556 2732 2733
+3  2556 2733 2557
+3  2557 2733 2734
+3  2557 2734 2735
+3  2557 2735 2558
+3  2558 2735 2559
+3  2559 2735 2736
+3  2559 2736 2560
+3  2560 2736 2737
+3  2560 2737 2561
+3  2561 2737 2738
+3  2561 2738 2562
+3  2562 2738 2739
+3  2562 2739 2563
+3  2563 2739 2740
+3  2563 2740 2564
+3  2564 2740 2741
+3  2564 2741 2565
+3  2565 2741 2742
+3  2565 2742 2566
+3  2566 2742 2743
+3  2566 2743 2567
+3  2567 2743 2744
+3  2567 2744 2568
+3  2568 2744 2745
+3  2568 2745 2569
+3  2569 2745 2746
+3  2569 2746 2570
+3  2570 2746 2747
+3  2570 2747 2571
+3  2571 2747 2748
+3  2571 2748 2572
+3  2572 2748 2749
+3  2572 2749 2573
+3  2573 2749 2750
+3  2573 2750 2574
+3  2574 2750 2751
+3  2574 2751 2575
+3  2575 2751 2752
+3  2575 2752 2576
+3  2576 2752 2753
+3  2576 2753 2754
+3  2576 2754 2577
+3  2577 2754 2755
+3  2577 2755 2578
+3  2578 2755 2756
+3  2578 2756 2579
+3  2579 2756 2580
+3  2580 2756 2757
+3  2580 2757 2581
+3  2581 2757 2758
+3  2581 2758 2582
+3  2582 2758 2759
+3  2582 2759 2760
+3  2582 2760 2583
+3  2583 2760 2761
+3  2583 2761 2584
+3  2584 2761 2585
+3  2585 2761 2762
+3  2585 2762 2586
+3  2586 2762 2763
+3  2586 2763 2764
+3  2586 2764 2587
+3  2587 2764 2765
+3  2587 2765 2766
+3  2587 2766 2588
+3  2588 2766 2767
+3  2588 2767 2589
+3  2589 2767 2768
+3  2589 2768 2769
+3  2589 2769 2590
+3  2590 2769 2591
+3  2591 2769 2592
+3  2592 2769 2770
+3  2592 2770 2771
+3  2592 2771 2772
+3  2592 2772 2593
+3  2593 2772 2773
+3  2593 2773 2594
+3  2594 2773 2596
+3  2596 2773 2597
+3  2597 2773 2774
+3  2597 2774 2775
+3  2597 2775 2598
+3  2598 2775 2599
+3  2599 2775 2776
+3  2599 2776 2777
+3  2599 2777 2600
+3  2600 2777 2778
+3  2600 2778 2601
+3  2601 2778 2779
+3  2601 2779 2602
+3  2602 2779 2780
+3  2602 2780 2603
+3  2603 2780 2781
+3  2603 2781 2604
+3  2604 2781 2782
+3  2604 2782 2605
+3  2605 2782 2783
+3  2605 2783 2606
+3  2606 2783 2784
+3  2606 2784 2607
+3  2607 2784 2785
+3  2607 2785 2608
+3  2608 2785 2786
+3  2608 2786 2787
+3  2608 2787 2609
+3  2609 2787 2610
+3  2610 2787 2788
+3  2610 2788 2611
+3  2611 2788 2789
+3  2611 2789 2612
+3  2612 2789 2790
+3  2612 2790 2791
+3  2612 2791 2613
+3  2613 2791 2614
+3  2614 2791 2792
+3  2614 2792 2793
+3  2614 2793 2794
+3  2614 2794 2615
+3  2615 2794 2616
+3  2616 2794 2795
+3  2616 2795 2617
+3  2617 2795 2796
+3  2617 2796 2618
+3  2618 2796 2797
+3  2618 2797 2619
+3  2619 2797 2798
+3  2619 2798 2799
+3  2619 2799 2800
+3  2619 2800 2620
+3  2620 2800 2621
+3  2621 2800 2801
+3  2621 2801 2622
+3  2622 2801 2802
+3  2622 2802 2623
+3  2623 2802 2803
+3  2623 2803 2624
+3  2624 2803 2804
+3  2624 2804 2625
+3  2625 2804 2805
+3  2625 2805 2806
+3  2625 2806 2626
+3  2626 2806 2807
+3  2626 2807 2627
+3  2627 2807 2628
+3  2628 2807 2808
+3  2628 2808 2809
+3  2628 2809 2629
+3  2629 2809 2810
+3  2629 2810 2630
+3  2630 2810 2631
+3  2631 2810 2811
+3  2631 2811 2812
+3  2631 2812 2632
+3  2632 2812 2813
+3  2632 2813 2633
+3  2633 2813 2814
+3  2633 2814 2634
+3  2634 2814 2815
+3  2634 2815 2635
+3  2635 2815 2816
+3  2635 2816 2636
+3  2636 2816 2637
+3  2637 2816 2817
+3  2637 2817 2638
+3  2638 2817 2818
+3  2638 2818 2819
+3  2638 2819 2639
+3  2639 2819 2640
+3  2640 2819 2820
+3  2640 2820 2641
+3  2641 2820 2643
+3  2643 2820 2821
+3  2643 2821 2822
+3  2643 2822 2644
+3  2644 2822 2823
+3  2644 2823 2645
+3  2645 2823 2824
+3  2645 2824 2646
+3  2646 2824 2825
+3  2646 2825 2647
+3  2647 2825 2826
+3  2647 2826 2648
+3  2648 2826 2827
+3  2648 2827 2828
+3  2648 2828 2649
+3  2649 2828 2650
+3  2650 2828 2829
+3  2650 2829 2830
+3  2650 2830 2831
+3  2650 2831 2651
+3  2651 2831 2832
+3  2651 2832 2833
+3  2651 2833 2652
+3  2652 2833 2834
+3  2652 2834 2835
+3  2652 2835 2653
+3  2653 2835 2836
+3  2653 2836 2654
+3  2654 2836 2656
+3  2656 2836 2837
+3  2656 2837 2657
+3  2657 2837 2838
+3  2657 2838 2658
+3  2658 2838 2839
+3  2658 2839 2659
+3  2659 2839 2840
+3  2659 2840 2841
+3  2659 2841 2660
+3  2660 2841 2661
+3  2661 2841 2842
+3  2661 2842 2662
+3  2662 2842 2843
+3  2662 2843 2844
+3  2662 2844 2663
+3  2663 2844 2845
+3  2663 2845 2664
+3  2664 2845 2846
+3  2664 2846 2665
+3  2665 2846 2847
+3  2665 2847 2666
+3  2666 2847 2667
+3  2667 2847 2848
+3  2667 2848 2849
+3  2667 2849 2668
+3  2668 2849 2850
+3  2668 2850 2669
+3  2669 2850 2851
+3  2669 2851 2670
+3  2670 2851 2852
+3  2670 2852 2671
+3  2671 2852 2672
+3  2672 2852 2853
+3  2672 2853 2673
+3  2673 2853 2854
+3  2673 2854 2674
+3  2674 2854 2855
+3  2674 2855 2856
+3  2674 2856 2675
+3  2675 2856 2857
+3  2675 2857 2676
+3  2676 2857 2858
+3  2676 2858 2677
+3  2677 2858 2859
+3  2677 2859 2860
+3  2677 2860 2678
+3  2678 2860 2679
+3  2679 2860 2861
+3  2679 2861 2862
+3  2679 2862 2680
+3  2680 2862 2863
+3  2680 2863 2681
+3  2681 2863 2682
+3  2682 2863 2683
+3  2683 2863 2684
+3  2684 2863 2864
+3  2684 2864 2865
+3  2684 2865 2866
+3  2684 2866 2685
+3  2685 2866 2686
+3  2686 2866 2867
+3  2686 2867 2868
+3  2686 2868 2687
+3  2687 2868 2869
+3  2687 2869 2688
+3  2688 2869 2870
+3  2688 2870 2689
+3  2689 2870 2871
+3  2689 2871 2690
+3  2690 2871 2872
+3  2690 2872 2691
+3  2691 2872 2873
+3  2691 2873 2692
+3  2692 2873 2874
+3  2692 2874 2693
+3  2693 2874 2875
+3  2693 2875 2694
+3  2694 2875 2876
+3  2694 2876 2695
+3  2695 2876 2877
+3  2695 2877 2696
+3  2696 2877 2878
+3  2696 2878 2697
+3  2697 2878 2879
+3  2697 2879 2698
+3  2698 2879 2880
+3  2698 2880 2699
+3  2699 2880 2881
+3  2699 2881 2700
+3  2700 2881 2882
+3  2700 2882 2701
+3  2701 2882 2883
+3  2701 2883 2702
+3  2702 2883 2884
+3  2702 2884 2703
+3  2703 2884 2885
+3  2703 2885 2704
+3  2704 2885 2886
+3  2704 2886 2705
+3  2705 2886 2887
+3  2705 2887 2706
+3  2706 2887 2888
+3  2706 2888 2707
+3  2707 2888 2889
+3  2707 2889 2708
+3  2708 2889 2890
+3  2708 2890 2709
+3  2709 2890 2891
+3  2709 2891 2710
+3  2710 2891 2892
+3  2710 2892 2711
+3  2711 2892 2893
+3  2711 2893 2894
+3  2711 2894 2712
+3  2712 2894 2895
+3  2712 2895 2713
+3  2713 2895 2714
+3  2714 2895 2896
+3  2714 2896 2715
+3  2715 2896 2897
+3  2715 2897 2898
+3  2715 2898 2716
+3  2716 2898 2717
+3  2717 2898 2899
+3  2717 2899 2900
+3  2717 2900 2718
+3  2718 2900 2719
+3  2719 2900 2901
+3  2719 2901 2720
+3  2720 2901 2902
+3  2720 2902 2903
+3  2720 2903 2721
+3  2721 2903 2904
+3  2721 2904 2722
+3  2722 2904 2905
+3  2722 2905 2723
+3  2723 2905 2724
+3  2724 2905 2906
+3  2724 2906 2725
+3  2725 2906 2907
+3  2725 2907 2908
+3  2725 2908 2726
+3  2726 2908 2727
+3  2727 2908 2909
+3  2727 2909 2728
+3  2728 2909 2910
+3  2728 2910 2911
+3  2728 2911 2730
+3  2730 2911 2912
+3  2730 2912 2913
+3  2730 2913 2731
+3  2731 2913 2914
+3  2731 2914 2732
+3  2732 2914 2915
+3  2732 2915 2733
+3  2733 2915 2916
+3  2733 2916 2734
+3  2734 2916 2917
+3  2734 2917 2918
+3  2734 2918 2735
+3  2735 2918 2919
+3  2735 2919 2736
+3  2736 2919 2920
+3  2736 2920 2737
+3  2737 2920 2921
+3  2737 2921 2738
+3  2738 2921 2922
+3  2738 2922 2739
+3  2739 2922 2923
+3  2739 2923 2740
+3  2740 2923 2924
+3  2740 2924 2741
+3  2741 2924 2925
+3  2741 2925 2742
+3  2742 2925 2926
+3  2742 2926 2743
+3  2743 2926 2927
+3  2743 2927 2744
+3  2744 2927 2928
+3  2744 2928 2745
+3  2745 2928 2929
+3  2745 2929 2746
+3  2746 2929 2930
+3  2746 2930 2747
+3  2747 2930 2931
+3  2747 2931 2748
+3  2748 2931 2932
+3  2748 2932 2749
+3  2749 2932 2933
+3  2749 2933 2750
+3  2750 2933 2934
+3  2750 2934 2751
+3  2751 2934 2935
+3  2751 2935 2752
+3  2752 2935 2936
+3  2752 2936 2753
+3  2753 2936 2937
+3  2753 2937 2754
+3  2754 2937 2938
+3  2754 2938 2939
+3  2754 2939 2755
+3  2755 2939 2756
+3  2756 2939 2757
+3  2757 2939 2940
+3  2757 2940 2758
+3  2758 2940 2941
+3  2758 2941 2942
+3  2758 2942 2759
+3  2759 2942 2943
+3  2759 2943 2944
+3  2759 2944 2760
+3  2760 2944 2945
+3  2760 2945 2761
+3  2761 2945 2763
+3  2763 2945 2946
+3  2763 2946 2764
+3  2764 2946 2947
+3  2764 2947 2948
+3  2764 2948 2765
+3  2765 2948 2949
+3  2765 2949 2766
+3  2766 2949 2950
+3  2766 2950 2767
+3  2767 2950 2951
+3  2767 2951 2768
+3  2768 2951 2952
+3  2768 2952 2770
+3  2770 2952 2953
+3  2770 2953 2771
+3  2771 2953 2954
+3  2771 2954 2955
+3  2771 2955 2772
+3  2772 2955 2774
+3  2774 2955 2956
+3  2774 2956 2775
+3  2775 2956 2957
+3  2775 2957 2776
+3  2776 2957 2958
+3  2776 2958 2777
+3  2777 2958 2959
+3  2777 2959 2960
+3  2777 2960 2778
+3  2778 2960 2961
+3  2778 2961 2779
+3  2779 2961 2780
+3  2780 2961 2962
+3  2780 2962 2963
+3  2780 2963 2781
+3  2781 2963 2964
+3  2781 2964 2782
+3  2782 2964 2965
+3  2782 2965 2783
+3  2783 2965 2966
+3  2783 2966 2784
+3  2784 2966 2967
+3  2784 2967 2785
+3  2785 2967 2968
+3  2785 2968 2786
+3  2786 2968 2969
+3  2786 2969 2787
+3  2787 2969 2788
+3  2788 2969 2970
+3  2788 2970 2789
+3  2789 2970 2971
+3  2789 2971 2790
+3  2790 2971 2972
+3  2790 2972 2973
+3  2790 2973 2791
+3  2791 2973 2974
+3  2791 2974 2792
+3  2792 2974 2975
+3  2792 2975 2793
+3  2793 2975 2976
+3  2793 2976 2977
+3  2793 2977 2794
+3  2794 2977 2795
+3  2795 2977 2978
+3  2795 2978 2796
+3  2796 2978 2797
+3  2797 2978 2979
+3  2797 2979 2980
+3  2797 2980 2798
+3  2798 2980 2799
+3  2799 2980 2981
+3  2799 2981 2982
+3  2799 2982 2800
+3  2800 2982 2801
+3  2801 2982 2983
+3  2801 2983 2802
+3  2802 2983 2984
+3  2802 2984 2803
+3  2803 2984 2985
+3  2803 2985 2804
+3  2804 2985 2986
+3  2804 2986 2805
+3  2805 2986 2987
+3  2805 2987 2806
+3  2806 2987 2988
+3  2806 2988 2807
+3  2807 2988 2808
+3  2808 2988 2989
+3  2808 2989 2990
+3  2808 2990 2809
+3  2809 2990 2991
+3  2809 2991 2810
+3  2810 2991 2811
+3  2811 2991 2992
+3  2811 2992 2993
+3  2811 2993 2812
+3  2812 2993 2994
+3  2812 2994 2813
+3  2813 2994 2995
+3  2813 2995 2814
+3  2814 2995 2996
+3  2814 2996 2815
+3  2815 2996 2997
+3  2815 2997 2816
+3  2816 2997 2998
+3  2816 2998 2817
+3  2817 2998 2999
+3  2817 2999 2818
+3  2818 2999 3000
+3  2818 3000 2819
+3  2819 3000 3001
+3  2819 3001 2820
+3  2820 3001 3002
+3  2820 3002 2821
+3  2821 3002 3003
+3  2821 3003 2822
+3  2822 3003 3004
+3  2822 3004 2823
+3  2823 3004 3005
+3  2823 3005 3006
+3  2823 3006 2824
+3  2824 3006 3007
+3  2824 3007 2825
+3  2825 3007 3008
+3  2825 3008 2826
+3  2826 3008 3009
+3  2826 3009 2827
+3  2827 3009 3010
+3  2827 3010 2829
+3  2829 3010 3011
+3  2829 3011 2830
+3  2830 3011 3012
+3  2830 3012 2831
+3  2831 3012 2832
+3  2832 3012 3013
+3  2832 3013 3014
+3  2832 3014 2833
+3  2833 3014 3015
+3  2833 3015 2834
+3  2834 3015 3016
+3  2834 3016 2835
+3  2835 3016 3017
+3  2835 3017 2836
+3  2836 3017 2837
+3  2837 3017 3018
+3  2837 3018 2838
+3  2838 3018 3019
+3  2838 3019 2839
+3  2839 3019 3020
+3  2839 3020 2840
+3  2840 3020 3021
+3  2840 3021 3022
+3  2840 3022 2841
+3  2841 3022 2842
+3  2842 3022 3023
+3  2842 3023 2843
+3  2843 3023 3024
+3  2843 3024 3025
+3  2843 3025 2844
+3  2844 3025 3026
+3  2844 3026 2845
+3  2845 3026 3027
+3  2845 3027 2846
+3  2846 3027 3028
+3  2846 3028 2847
+3  2847 3028 2848
+3  2848 3028 3029
+3  2848 3029 3030
+3  2848 3030 2849
+3  2849 3030 3031
+3  2849 3031 2850
+3  2850 3031 3032
+3  2850 3032 2851
+3  2851 3032 3033
+3  2851 3033 2852
+3  2852 3033 3034
+3  2852 3034 2853
+3  2853 3034 3035
+3  2853 3035 2854
+3  2854 3035 3036
+3  2854 3036 2855
+3  2855 3036 3037
+3  2855 3037 2856
+3  2856 3037 3038
+3  2856 3038 2857
+3  2857 3038 3039
+3  2857 3039 2858
+3  2858 3039 3040
+3  2858 3040 2859
+3  2859 3040 3041
+3  2859 3041 3042
+3  2859 3042 2860
+3  2860 3042 2861
+3  2861 3042 3043
+3  2861 3043 3044
+3  2861 3044 2862
+3  2862 3044 2864
+3  2864 3044 3045
+3  2864 3045 2865
+3  2865 3045 3046
+3  2865 3046 2866
+3  2866 3046 2867
+3  2867 3046 3047
+3  2867 3047 3048
+3  2867 3048 3049
+3  2867 3049 2868
+3  2868 3049 2869
+3  2869 3049 3050
+3  2869 3050 2870
+3  2870 3050 3051
+3  2870 3051 2871
+3  2871 3051 3052
+3  2871 3052 2872
+3  2872 3052 3053
+3  2872 3053 2873
+3  2873 3053 3054
+3  2873 3054 2874
+3  2874 3054 3055
+3  2874 3055 2875
+3  2875 3055 3056
+3  2875 3056 2876
+3  2876 3056 3057
+3  2876 3057 2877
+3  2877 3057 3058
+3  2877 3058 2878
+3  2878 3058 3059
+3  2878 3059 2879
+3  2879 3059 3060
+3  2879 3060 2880
+3  2880 3060 3061
+3  2880 3061 2881
+3  2881 3061 3062
+3  2881 3062 2882
+3  2882 3062 3063
+3  2882 3063 2883
+3  2883 3063 3064
+3  2883 3064 2884
+3  2884 3064 3065
+3  2884 3065 2885
+3  2885 3065 3066
+3  2885 3066 2886
+3  2886 3066 3067
+3  2886 3067 2887
+3  2887 3067 3068
+3  2887 3068 2888
+3  2888 3068 3069
+3  2888 3069 2889
+3  2889 3069 3070
+3  2889 3070 2890
+3  2890 3070 3071
+3  2890 3071 2891
+3  2891 3071 3072
+3  2891 3072 2892
+3  2892 3072 3073
+3  2892 3073 2893
+3  2893 3073 3074
+3  2893 3074 3075
+3  2893 3075 2894
+3  2894 3075 3076
+3  2894 3076 2895
+3  2895 3076 2896
+3  2896 3076 3077
+3  2896 3077 2897
+3  2897 3077 3078
+3  2897 3078 3079
+3  2897 3079 2898
+3  2898 3079 2899
+3  2899 3079 3080
+3  2899 3080 3081
+3  2899 3081 2900
+3  2900 3081 2901
+3  2901 3081 3082
+3  2901 3082 2902
+3  2902 3082 3083
+3  2902 3083 3084
+3  2902 3084 2903
+3  2903 3084 3085
+3  2903 3085 2904
+3  2904 3085 3086
+3  2904 3086 2905
+3  2905 3086 3087
+3  2905 3087 2906
+3  2906 3087 3088
+3  2906 3088 2907
+3  2907 3088 3089
+3  2907 3089 3090
+3  2907 3090 2908
+3  2908 3090 2909
+3  2909 3090 3091
+3  2909 3091 2910
+3  2910 3091 3092
+3  2910 3092 3093
+3  2910 3093 3094
+3  2910 3094 2911
+3  2911 3094 2912
+3  2912 3094 3095
+3  2912 3095 3096
+3  2912 3096 2913
+3  2913 3096 3097
+3  2913 3097 2914
+3  2914 3097 3098
+3  2914 3098 2915
+3  2915 3098 3099
+3  2915 3099 2916
+3  2916 3099 3100
+3  2916 3100 3101
+3  2916 3101 2917
+3  2917 3101 3102
+3  2917 3102 3103
+3  2917 3103 2918
+3  2918 3103 3104
+3  2918 3104 2919
+3  2919 3104 2920
+3  2920 3104 3105
+3  2920 3105 2921
+3  2921 3105 2922
+3  2922 3105 3106
+3  2922 3106 2923
+3  2923 3106 3107
+3  2923 3107 2924
+3  2924 3107 3108
+3  2924 3108 2925
+3  2925 3108 3109
+3  2925 3109 2926
+3  2926 3109 3110
+3  2926 3110 2927
+3  2927 3110 3111
+3  2927 3111 2928
+3  2928 3111 3112
+3  2928 3112 2929
+3  2929 3112 3113
+3  2929 3113 2930
+3  2930 3113 3114
+3  2930 3114 2931
+3  2931 3114 3115
+3  2931 3115 2932
+3  2932 3115 3116
+3  2932 3116 2933
+3  2933 3116 3117
+3  2933 3117 2934
+3  2934 3117 3118
+3  2934 3118 2935
+3  2935 3118 3119
+3  2935 3119 2936
+3  2936 3119 3120
+3  2936 3120 2937
+3  2937 3120 3121
+3  2937 3121 3122
+3  2937 3122 2938
+3  2938 3122 3123
+3  2938 3123 2939
+3  2939 3123 2940
+3  2940 3123 2941
+3  2941 3123 3124
+3  2941 3124 2942
+3  2942 3124 3125
+3  2942 3125 3126
+3  2942 3126 2943
+3  2943 3126 3127
+3  2943 3127 3128
+3  2943 3128 2944
+3  2944 3128 3129
+3  2944 3129 2945
+3  2945 3129 2946
+3  2946 3129 3130
+3  2946 3130 2947
+3  2947 3130 3131
+3  2947 3131 3132
+3  2947 3132 2948
+3  2948 3132 3133
+3  2948 3133 3134
+3  2948 3134 2949
+3  2949 3134 3135
+3  2949 3135 2950
+3  2950 3135 3136
+3  2950 3136 2951
+3  2951 3136 3137
+3  2951 3137 3138
+3  2951 3138 2952
+3  2952 3138 3139
+3  2952 3139 2953
+3  2953 3139 3140
+3  2953 3140 3141
+3  2953 3141 2954
+3  2954 3141 3142
+3  2954 3142 3143
+3  2954 3143 2955
+3  2955 3143 2956
+3  2956 3143 3144
+3  2956 3144 2957
+3  2957 3144 3145
+3  2957 3145 3146
+3  2957 3146 2958
+3  2958 3146 3147
+3  2958 3147 2959
+3  2959 3147 3148
+3  2959 3148 3149
+3  2959 3149 2960
+3  2960 3149 2961
+3  2961 3149 3150
+3  2961 3150 2962
+3  2962 3150 3151
+3  2962 3151 3152
+3  2962 3152 2963
+3  2963 3152 3153
+3  2963 3153 2964
+3  2964 3153 3154
+3  2964 3154 2965
+3  2965 3154 3155
+3  2965 3155 2966
+3  2966 3155 3156
+3  2966 3156 2967
+3  2967 3156 3157
+3  2967 3157 2968
+3  2968 3157 3158
+3  2968 3158 2969
+3  2969 3158 3159
+3  2969 3159 2970
+3  2970 3159 3160
+3  2970 3160 2971
+3  2971 3160 3161
+3  2971 3161 2972
+3  2972 3161 3162
+3  2972 3162 2973
+3  2973 3162 3163
+3  2973 3163 2974
+3  2974 3163 3164
+3  2974 3164 2975
+3  2975 3164 3165
+3  2975 3165 2976
+3  2976 3165 3166
+3  2976 3166 3167
+3  2976 3167 2977
+3  2977 3167 2978
+3  2978 3167 3168
+3  2978 3168 2979
+3  2979 3168 3169
+3  2979 3169 2980
+3  2980 3169 3170
+3  2980 3170 2981
+3  2981 3170 3171
+3  2981 3171 3172
+3  2981 3172 2982
+3  2982 3172 3173
+3  2982 3173 2983
+3  2983 3173 3174
+3  2983 3174 2984
+3  2984 3174 2985
+3  2985 3174 3175
+3  2985 3175 2986
+3  2986 3175 3176
+3  2986 3176 3177
+3  2986 3177 3178
+3  2986 3178 2987
+3  2987 3178 3179
+3  2987 3179 2988
+3  2988 3179 2989
+3  2989 3179 3180
+3  2989 3180 3181
+3  2989 3181 2990
+3  2990 3181 3182
+3  2990 3182 2991
+3  2991 3182 2992
+3  2992 3182 3183
+3  2992 3183 3184
+3  2992 3184 2993
+3  2993 3184 3185
+3  2993 3185 2994
+3  2994 3185 3186
+3  2994 3186 2995
+3  2995 3186 3187
+3  2995 3187 2996
+3  2996 3187 3188
+3  2996 3188 2997
+3  2997 3188 2998
+3  2998 3188 3189
+3  2998 3189 3190
+3  2998 3190 2999
+3  2999 3190 3191
+3  2999 3191 3192
+3  2999 3192 3193
+3  2999 3193 3000
+3  3000 3193 3001
+3  3001 3193 3194
+3  3001 3194 3002
+3  3002 3194 3195
+3  3002 3195 3003
+3  3003 3195 3196
+3  3003 3196 3004
+3  3004 3196 3005
+3  3005 3196 3197
+3  3005 3197 3198
+3  3005 3198 3006
+3  3006 3198 3199
+3  3006 3199 3200
+3  3006 3200 3007
+3  3007 3200 3201
+3  3007 3201 3008
+3  3008 3201 3202
+3  3008 3202 3009
+3  3009 3202 3203
+3  3009 3203 3010
+3  3010 3203 3204
+3  3010 3204 3011
+3  3011 3204 3205
+3  3011 3205 3012
+3  3012 3205 3206
+3  3012 3206 3013
+3  3013 3206 3207
+3  3013 3207 3208
+3  3013 3208 3014
+3  3014 3208 3209
+3  3014 3209 3015
+3  3015 3209 3210
+3  3015 3210 3016
+3  3016 3210 3211
+3  3016 3211 3017
+3  3017 3211 3018
+3  3018 3211 3212
+3  3018 3212 3019
+3  3019 3212 3213
+3  3019 3213 3020
+3  3020 3213 3214
+3  3020 3214 3021
+3  3021 3214 3215
+3  3021 3215 3216
+3  3021 3216 3022
+3  3022 3216 3023
+3  3023 3216 3217
+3  3023 3217 3024
+3  3024 3217 3218
+3  3024 3218 3219
+3  3024 3219 3025
+3  3025 3219 3220
+3  3025 3220 3026
+3  3026 3220 3221
+3  3026 3221 3027
+3  3027 3221 3222
+3  3027 3222 3028
+3  3028 3222 3029
+3  3029 3222 3223
+3  3029 3223 3224
+3  3029 3224 3030
+3  3030 3224 3225
+3  3030 3225 3031
+3  3031 3225 3226
+3  3031 3226 3032
+3  3032 3226 3227
+3  3032 3227 3033
+3  3033 3227 3228
+3  3033 3228 3034
+3  3034 3228 3229
+3  3034 3229 3035
+3  3035 3229 3036
+3  3036 3229 3230
+3  3036 3230 3231
+3  3036 3231 3037
+3  3037 3231 3232
+3  3037 3232 3038
+3  3038 3232 3233
+3  3038 3233 3039
+3  3039 3233 3234
+3  3039 3234 3040
+3  3040 3234 3235
+3  3040 3235 3041
+3  3041 3235 3236
+3  3041 3236 3237
+3  3041 3237 3042
+3  3042 3237 3043
+3  3043 3237 3238
+3  3043 3238 3239
+3  3043 3239 3044
+3  3044 3239 3045
+3  3045 3239 3047
+3  3047 3239 3240
+3  3047 3240 3048
+3  3048 3240 3241
+3  3048 3241 3242
+3  3048 3242 3049
+3  3049 3242 3243
+3  3049 3243 3244
+3  3049 3244 3050
+3  3050 3244 3245
+3  3050 3245 3051
+3  3051 3245 3246
+3  3051 3246 3052
+3  3052 3246 3247
+3  3052 3247 3053
+3  3053 3247 3248
+3  3053 3248 3054
+3  3054 3248 3249
+3  3054 3249 3055
+3  3055 3249 3250
+3  3055 3250 3056
+3  3056 3250 3251
+3  3056 3251 3057
+3  3057 3251 3252
+3  3057 3252 3058
+3  3058 3252 3253
+3  3058 3253 3059
+3  3059 3253 3254
+3  3059 3254 3060
+3  3060 3254 3255
+3  3060 3255 3061
+3  3061 3255 3256
+3  3061 3256 3062
+3  3062 3256 3257
+3  3062 3257 3063
+3  3063 3257 3258
+3  3063 3258 3064
+3  3064 3258 3259
+3  3064 3259 3065
+3  3065 3259 3260
+3  3065 3260 3066
+3  3066 3260 3261
+3  3066 3261 3067
+3  3067 3261 3262
+3  3067 3262 3068
+3  3068 3262 3263
+3  3068 3263 3069
+3  3069 3263 3264
+3  3069 3264 3070
+3  3070 3264 3265
+3  3070 3265 3071
+3  3071 3265 3266
+3  3071 3266 3072
+3  3072 3266 3267
+3  3072 3267 3073
+3  3073 3267 3268
+3  3073 3268 3074
+3  3074 3268 3269
+3  3074 3269 3270
+3  3074 3270 3075
+3  3075 3270 3271
+3  3075 3271 3076
+3  3076 3271 3077
+3  3077 3271 3272
+3  3077 3272 3078
+3  3078 3272 3273
+3  3078 3273 3274
+3  3078 3274 3079
+3  3079 3274 3080
+3  3080 3274 3275
+3  3080 3275 3276
+3  3080 3276 3081
+3  3081 3276 3082
+3  3082 3276 3277
+3  3082 3277 3083
+3  3083 3277 3278
+3  3083 3278 3279
+3  3083 3279 3084
+3  3084 3279 3280
+3  3084 3280 3085
+3  3085 3280 3281
+3  3085 3281 3086
+3  3086 3281 3282
+3  3086 3282 3087
+3  3087 3282 3283
+3  3087 3283 3088
+3  3088 3283 3284
+3  3088 3284 3089
+3  3089 3284 3091
+3  3091 3284 3285
+3  3091 3285 3286
+3  3091 3286 3092
+3  3092 3286 3287
+3  3092 3287 3093
+3  3093 3287 3288
+3  3093 3288 3289
+3  3093 3289 3094
+3  3094 3289 3095
+3  3095 3289 3290
+3  3095 3290 3291
+3  3095 3291 3096
+3  3096 3291 3292
+3  3096 3292 3097
+3  3097 3292 3293
+3  3097 3293 3098
+3  3098 3293 3294
+3  3098 3294 3099
+3  3099 3294 3295
+3  3099 3295 3100
+3  3100 3295 3296
+3  3100 3296 3101
+3  3101 3296 3297
+3  3101 3297 3298
+3  3101 3298 3102
+3  3102 3298 3299
+3  3102 3299 3300
+3  3102 3300 3301
+3  3102 3301 3103
+3  3103 3301 3302
+3  3103 3302 3104
+3  3104 3302 3105
+3  3105 3302 3303
+3  3105 3303 3106
+3  3106 3303 3304
+3  3106 3304 3107
+3  3107 3304 3305
+3  3107 3305 3108
+3  3108 3305 3306
+3  3108 3306 3109
+3  3109 3306 3307
+3  3109 3307 3110
+3  3110 3307 3308
+3  3110 3308 3111
+3  3111 3308 3309
+3  3111 3309 3112
+3  3112 3309 3310
+3  3112 3310 3113
+3  3113 3310 3311
+3  3113 3311 3114
+3  3114 3311 3312
+3  3114 3312 3115
+3  3115 3312 3313
+3  3115 3313 3116
+3  3116 3313 3314
+3  3116 3314 3117
+3  3117 3314 3315
+3  3117 3315 3118
+3  3118 3315 3316
+3  3118 3316 3119
+3  3119 3316 3317
+3  3119 3317 3120
+3  3120 3317 3318
+3  3120 3318 3121
+3  3121 3318 3319
+3  3121 3319 3122
+3  3122 3319 3320
+3  3122 3320 3321
+3  3122 3321 3322
+3  3122 3322 3123
+3  3123 3322 3323
+3  3123 3323 3124
+3  3124 3323 3324
+3  3124 3324 3125
+3  3125 3324 3325
+3  3125 3325 3326
+3  3125 3326 3126
+3  3126 3326 3127
+3  3127 3326 3327
+3  3127 3327 3328
+3  3127 3328 3128
+3  3128 3328 3329
+3  3128 3329 3129
+3  3129 3329 3130
+3  3130 3329 3330
+3  3130 3330 3331
+3  3130 3331 3131
+3  3131 3331 3332
+3  3131 3332 3132
+3  3132 3332 3333
+3  3132 3333 3133
+3  3133 3333 3334
+3  3133 3334 3134
+3  3134 3334 3335
+3  3134 3335 3135
+3  3135 3335 3336
+3  3135 3336 3136
+3  3136 3336 3337
+3  3136 3337 3137
+3  3137 3337 3338
+3  3137 3338 3339
+3  3137 3339 3138
+3  3138 3339 3340
+3  3138 3340 3139
+3  3139 3340 3140
+3  3140 3340 3341
+3  3140 3341 3342
+3  3140 3342 3141
+3  3141 3342 3343
+3  3141 3343 3142
+3  3142 3343 3344
+3  3142 3344 3143
+3  3143 3344 3144
+3  3144 3344 3345
+3  3144 3345 3145
+3  3145 3345 3346
+3  3145 3346 3146
+3  3146 3346 3347
+3  3146 3347 3147
+3  3147 3347 3348
+3  3147 3348 3148
+3  3148 3348 3349
+3  3148 3349 3350
+3  3148 3350 3149
+3  3149 3350 3150
+3  3150 3350 3351
+3  3150 3351 3151
+3  3151 3351 3352
+3  3151 3352 3353
+3  3151 3353 3152
+3  3152 3353 3354
+3  3152 3354 3153
+3  3153 3354 3355
+3  3153 3355 3154
+3  3154 3355 3356
+3  3154 3356 3155
+3  3155 3356 3357
+3  3155 3357 3156
+3  3156 3357 3358
+3  3156 3358 3157
+3  3157 3358 3359
+3  3157 3359 3158
+3  3158 3359 3360
+3  3158 3360 3159
+3  3159 3360 3361
+3  3159 3361 3160
+3  3160 3361 3362
+3  3160 3362 3161
+3  3161 3362 3363
+3  3161 3363 3162
+3  3162 3363 3364
+3  3162 3364 3163
+3  3163 3364 3365
+3  3163 3365 3164
+3  3164 3365 3366
+3  3164 3366 3165
+3  3165 3366 3367
+3  3165 3367 3166
+3  3166 3367 3368
+3  3166 3368 3369
+3  3166 3369 3167
+3  3167 3369 3168
+3  3168 3369 3370
+3  3168 3370 3169
+3  3169 3370 3371
+3  3169 3371 3170
+3  3170 3371 3372
+3  3170 3372 3171
+3  3171 3372 3373
+3  3171 3373 3374
+3  3171 3374 3172
+3  3172 3374 3173
+3  3173 3374 3375
+3  3173 3375 3174
+3  3174 3375 3376
+3  3174 3376 3175
+3  3175 3376 3377
+3  3175 3377 3176
+3  3176 3377 3378
+3  3176 3378 3177
+3  3177 3378 3379
+3  3177 3379 3380
+3  3177 3380 3178
+3  3178 3380 3381
+3  3178 3381 3179
+3  3179 3381 3180
+3  3180 3381 3382
+3  3180 3382 3383
+3  3180 3383 3181
+3  3181 3383 3384
+3  3181 3384 3182
+3  3182 3384 3183
+3  3183 3384 3385
+3  3183 3385 3386
+3  3183 3386 3184
+3  3184 3386 3387
+3  3184 3387 3185
+3  3185 3387 3388
+3  3185 3388 3186
+3  3186 3388 3389
+3  3186 3389 3187
+3  3187 3389 3390
+3  3187 3390 3188
+3  3188 3390 3189
+3  3189 3390 3391
+3  3189 3391 3392
+3  3189 3392 3190
+3  3190 3392 3191
+3  3191 3392 3393
+3  3191 3393 3394
+3  3191 3394 3192
+3  3192 3394 3395
+3  3192 3395 3396
+3  3192 3396 3193
+3  3193 3396 3194
+3  3194 3396 3397
+3  3194 3397 3195
+3  3195 3397 3196
+3  3196 3397 3197
+3  3197 3397 3398
+3  3197 3398 3198
+3  3198 3398 3399
+3  3198 3399 3400
+3  3198 3400 3199
+3  3199 3400 3401
+3  3199 3401 3402
+3  3199 3402 3200
+3  3200 3402 3201
+3  3201 3402 3403
+3  3201 3403 3404
+3  3201 3404 3202
+3  3202 3404 3203
+3  3203 3404 3405
+3  3203 3405 3204
+3  3204 3405 3406
+3  3204 3406 3407
+3  3204 3407 3205
+3  3205 3407 3408
+3  3205 3408 3206
+3  3206 3408 3409
+3  3206 3409 3207
+3  3207 3409 3410
+3  3207 3410 3411
+3  3207 3411 3208
+3  3208 3411 3412
+3  3208 3412 3209
+3  3209 3412 3413
+3  3209 3413 3210
+3  3210 3413 3414
+3  3210 3414 3211
+3  3211 3414 3212
+3  3212 3414 3415
+3  3212 3415 3213
+3  3213 3415 3416
+3  3213 3416 3214
+3  3214 3416 3417
+3  3214 3417 3215
+3  3215 3417 3418
+3  3215 3418 3419
+3  3215 3419 3216
+3  3216 3419 3217
+3  3217 3419 3420
+3  3217 3420 3218
+3  3218 3420 3421
+3  3218 3421 3422
+3  3218 3422 3219
+3  3219 3422 3423
+3  3219 3423 3220
+3  3220 3423 3424
+3  3220 3424 3221
+3  3221 3424 3425
+3  3221 3425 3222
+3  3222 3425 3223
+3  3223 3425 3426
+3  3223 3426 3427
+3  3223 3427 3224
+3  3224 3427 3428
+3  3224 3428 3225
+3  3225 3428 3429
+3  3225 3429 3226
+3  3226 3429 3430
+3  3226 3430 3227
+3  3227 3430 3431
+3  3227 3431 3228
+3  3228 3431 3432
+3  3228 3432 3229
+3  3229 3432 3230
+3  3230 3432 3433
+3  3230 3433 3434
+3  3230 3434 3231
+3  3231 3434 3435
+3  3231 3435 3232
+3  3232 3435 3436
+3  3232 3436 3233
+3  3233 3436 3437
+3  3233 3437 3234
+3  3234 3437 3438
+3  3234 3438 3235
+3  3235 3438 3439
+3  3235 3439 3236
+3  3236 3439 3440
+3  3236 3440 3441
+3  3236 3441 3237
+3  3237 3441 3238
+3  3238 3441 3442
+3  3238 3442 3240
+3  3240 3442 3241
+3  3241 3442 3443
+3  3241 3443 3444
+3  3241 3444 3242
+3  3242 3444 3445
+3  3242 3445 3446
+3  3242 3446 3243
+3  3243 3446 3244
+3  3244 3446 3447
+3  3244 3447 3245
+3  3245 3447 3246
+3  3246 3447 3448
+3  3246 3448 3247
+3  3247 3448 3449
+3  3247 3449 3248
+3  3248 3449 3450
+3  3248 3450 3451
+3  3248 3451 3249
+3  3249 3451 3452
+3  3249 3452 3250
+3  3250 3452 3453
+3  3250 3453 3251
+3  3251 3453 3454
+3  3251 3454 3252
+3  3252 3454 3455
+3  3252 3455 3253
+3  3253 3455 3254
+3  3254 3455 3456
+3  3254 3456 3457
+3  3254 3457 3255
+3  3255 3457 3458
+3  3255 3458 3256
+3  3256 3458 3459
+3  3256 3459 3257
+3  3257 3459 3460
+3  3257 3460 3258
+3  3258 3460 3461
+3  3258 3461 3259
+3  3259 3461 3462
+3  3259 3462 3260
+3  3260 3462 3463
+3  3260 3463 3261
+3  3261 3463 3464
+3  3261 3464 3262
+3  3262 3464 3465
+3  3262 3465 3263
+3  3263 3465 3466
+3  3263 3466 3264
+3  3264 3466 3467
+3  3264 3467 3265
+3  3265 3467 3266
+3  3266 3467 3468
+3  3266 3468 3469
+3  3266 3469 3267
+3  3267 3469 3470
+3  3267 3470 3268
+3  3268 3470 3471
+3  3268 3471 3269
+3  3269 3471 3472
+3  3269 3472 3270
+3  3270 3472 3473
+3  3270 3473 3271
+3  3271 3473 3272
+3  3272 3473 3474
+3  3272 3474 3273
+3  3273 3474 3475
+3  3273 3475 3476
+3  3273 3476 3274
+3  3274 3476 3275
+3  3275 3476 3477
+3  3275 3477 3478
+3  3275 3478 3276
+3  3276 3478 3277
+3  3277 3478 3479
+3  3277 3479 3278
+3  3278 3479 3480
+3  3278 3480 3481
+3  3278 3481 3279
+3  3279 3481 3482
+3  3279 3482 3280
+3  3280 3482 3281
+3  3281 3482 3483
+3  3281 3483 3484
+3  3281 3484 3282
+3  3282 3484 3485
+3  3282 3485 3283
+3  3283 3485 3284
+3  3284 3485 3285
+3  3285 3485 3486
+3  3285 3486 3286
+3  3286 3486 3487
+3  3286 3487 3488
+3  3286 3488 3287
+3  3287 3488 3489
+3  3287 3489 3288
+3  3288 3489 3490
+3  3288 3490 3491
+3  3288 3491 3289
+3  3289 3491 3290
+3  3290 3491 3492
+3  3290 3492 3493
+3  3290 3493 3291
+3  3291 3493 3494
+3  3291 3494 3292
+3  3292 3494 3495
+3  3292 3495 3293
+3  3293 3495 3496
+3  3293 3496 3294
+3  3294 3496 3497
+3  3294 3497 3295
+3  3295 3497 3498
+3  3295 3498 3296
+3  3296 3498 3499
+3  3296 3499 3297
+3  3297 3499 3500
+3  3297 3500 3501
+3  3297 3501 3298
+3  3298 3501 3502
+3  3298 3502 3299
+3  3299 3502 3503
+3  3299 3503 3504
+3  3299 3504 3505
+3  3299 3505 3300
+3  3300 3505 3506
+3  3300 3506 3301
+3  3301 3506 3302
+3  3302 3506 3303
+3  3303 3506 3507
+3  3303 3507 3304
+3  3304 3507 3508
+3  3304 3508 3305
+3  3305 3508 3509
+3  3305 3509 3306
+3  3306 3509 3510
+3  3306 3510 3307
+3  3307 3510 3511
+3  3307 3511 3308
+3  3308 3511 3512
+3  3308 3512 3309
+3  3309 3512 3513
+3  3309 3513 3310
+3  3310 3513 3514
+3  3310 3514 3311
+3  3311 3514 3515
+3  3311 3515 3312
+3  3312 3515 3516
+3  3312 3516 3313
+3  3313 3516 3517
+3  3313 3517 3314
+3  3314 3517 3518
+3  3314 3518 3315
+3  3315 3518 3519
+3  3315 3519 3316
+3  3316 3519 3520
+3  3316 3520 3317
+3  3317 3520 3521
+3  3317 3521 3318
+3  3318 3521 3522
+3  3318 3522 3319
+3  3319 3522 3523
+3  3319 3523 3524
+3  3319 3524 3320
+3  3320 3524 3525
+3  3320 3525 3321
+3  3321 3525 3526
+3  3321 3526 3527
+3  3321 3527 3322
+3  3322 3527 3323
+3  3323 3527 3528
+3  3323 3528 3324
+3  3324 3528 3529
+3  3324 3529 3325
+3  3325 3529 3530
+3  3325 3530 3327
+3  3327 3530 3531
+3  3327 3531 3328
+3  3328 3531 3532
+3  3328 3532 3329
+3  3329 3532 3330
+3  3330 3532 3533
+3  3330 3533 3534
+3  3330 3534 3331
+3  3331 3534 3535
+3  3331 3535 3332
+3  3332 3535 3536
+3  3332 3536 3333
+3  3333 3536 3537
+3  3333 3537 3538
+3  3333 3538 3334
+3  3334 3538 3539
+3  3334 3539 3540
+3  3334 3540 3335
+3  3335 3540 3336
+3  3336 3540 3541
+3  3336 3541 3337
+3  3337 3541 3542
+3  3337 3542 3338
+3  3338 3542 3543
+3  3338 3543 3544
+3  3338 3544 3339
+3  3339 3544 3545
+3  3339 3545 3340
+3  3340 3545 3341
+3  3341 3545 3546
+3  3341 3546 3342
+3  3342 3546 3547
+3  3342 3547 3343
+3  3343 3547 3548
+3  3343 3548 3344
+3  3344 3548 3345
+3  3345 3548 3549
+3  3345 3549 3346
+3  3346 3549 3550
+3  3346 3550 3551
+3  3346 3551 3347
+3  3347 3551 3348
+3  3348 3551 3552
+3  3348 3552 3349
+3  3349 3552 3553
+3  3349 3553 3350
+3  3350 3553 3554
+3  3350 3554 3351
+3  3351 3554 3555
+3  3351 3555 3352
+3  3352 3555 3556
+3  3352 3556 3557
+3  3352 3557 3353
+3  3353 3557 3558
+3  3353 3558 3354
+3  3354 3558 3559
+3  3354 3559 3355
+3  3355 3559 3560
+3  3355 3560 3356
+3  3356 3560 3561
+3  3356 3561 3357
+3  3357 3561 3562
+3  3357 3562 3358
+3  3358 3562 3563
+3  3358 3563 3359
+3  3359 3563 3564
+3  3359 3564 3360
+3  3360 3564 3565
+3  3360 3565 3361
+3  3361 3565 3566
+3  3361 3566 3362
+3  3362 3566 3567
+3  3362 3567 3363
+3  3363 3567 3568
+3  3363 3568 3364
+3  3364 3568 3569
+3  3364 3569 3365
+3  3365 3569 3570
+3  3365 3570 3366
+3  3366 3570 3571
+3  3366 3571 3367
+3  3367 3571 3572
+3  3367 3572 3368
+3  3368 3572 3573
+3  3368 3573 3574
+3  3368 3574 3369
+3  3369 3574 3370
+3  3370 3574 3575
+3  3370 3575 3371
+3  3371 3575 3576
+3  3371 3576 3372
+3  3372 3576 3577
+3  3372 3577 3373
+3  3373 3577 3578
+3  3373 3578 3579
+3  3373 3579 3374
+3  3374 3579 3375
+3  3375 3579 3580
+3  3375 3580 3376
+3  3376 3580 3581
+3  3376 3581 3377
+3  3377 3581 3582
+3  3377 3582 3378
+3  3378 3582 3583
+3  3378 3583 3379
+3  3379 3583 3584
+3  3379 3584 3585
+3  3379 3585 3380
+3  3380 3585 3586
+3  3380 3586 3381
+3  3381 3586 3382
+3  3382 3586 3587
+3  3382 3587 3588
+3  3382 3588 3383
+3  3383 3588 3589
+3  3383 3589 3384
+3  3384 3589 3385
+3  3385 3589 3590
+3  3385 3590 3591
+3  3385 3591 3386
+3  3386 3591 3592
+3  3386 3592 3387
+3  3387 3592 3593
+3  3387 3593 3388
+3  3388 3593 3594
+3  3388 3594 3389
+3  3389 3594 3595
+3  3389 3595 3390
+3  3390 3595 3391
+3  3391 3595 3596
+3  3391 3596 3597
+3  3391 3597 3392
+3  3392 3597 3393
+3  3393 3597 3598
+3  3393 3598 3394
+3  3394 3598 3599
+3  3394 3599 3600
+3  3394 3600 3395
+3  3395 3600 3601
+3  3395 3601 3602
+3  3395 3602 3396
+3  3396 3602 3397
+3  3397 3602 3398
+3  3398 3602 3399
+3  3399 3602 3603
+3  3399 3603 3400
+3  3400 3603 3604
+3  3400 3604 3605
+3  3400 3605 3606
+3  3400 3606 3401
+3  3401 3606 3607
+3  3401 3607 3608
+3  3401 3608 3402
+3  3402 3608 3403
+3  3403 3608 3609
+3  3403 3609 3610
+3  3403 3610 3404
+3  3404 3610 3405
+3  3405 3610 3611
+3  3405 3611 3406
+3  3406 3611 3612
+3  3406 3612 3407
+3  3407 3612 3613
+3  3407 3613 3408
+3  3408 3613 3614
+3  3408 3614 3409
+3  3409 3614 3615
+3  3409 3615 3410
+3  3410 3615 3616
+3  3410 3616 3617
+3  3410 3617 3411
+3  3411 3617 3618
+3  3411 3618 3412
+3  3412 3618 3619
+3  3412 3619 3413
+3  3413 3619 3620
+3  3413 3620 3414
+3  3414 3620 3415
+3  3415 3620 3621
+3  3415 3621 3416
+3  3416 3621 3622
+3  3416 3622 3417
+3  3417 3622 3623
+3  3417 3623 3418
+3  3418 3623 3624
+3  3418 3624 3625
+3  3418 3625 3419
+3  3419 3625 3420
+3  3420 3625 3626
+3  3420 3626 3421
+3  3421 3626 3627
+3  3421 3627 3628
+3  3421 3628 3422
+3  3422 3628 3629
+3  3422 3629 3423
+3  3423 3629 3630
+3  3423 3630 3424
+3  3424 3630 3631
+3  3424 3631 3425
+3  3425 3631 3426
+3  3426 3631 3632
+3  3426 3632 3427
+3  3427 3632 3633
+3  3427 3633 3428
+3  3428 3633 3634
+3  3428 3634 3429
+3  3429 3634 3635
+3  3429 3635 3636
+3  3429 3636 3430
+3  3430 3636 3431
+3  3431 3636 3637
+3  3431 3637 3432
+3  3432 3637 3638
+3  3432 3638 3639
+3  3432 3639 3433
+3  3433 3639 3640
+3  3433 3640 3434
+3  3434 3640 3641
+3  3434 3641 3435
+3  3435 3641 3642
+3  3435 3642 3436
+3  3436 3642 3643
+3  3436 3643 3437
+3  3437 3643 3644
+3  3437 3644 3438
+3  3438 3644 3645
+3  3438 3645 3439
+3  3439 3645 3646
+3  3439 3646 3440
+3  3440 3646 3647
+3  3440 3647 3648
+3  3440 3648 3441
+3  3441 3648 3442
+3  3442 3648 3443
+3  3443 3648 3649
+3  3443 3649 3650
+3  3443 3650 3445
+3  3445 3650 3447
+3  3447 3650 3651
+3  3447 3651 3448
+3  3448 3651 3652
+3  3448 3652 3653
+3  3448 3653 3449
+3  3449 3653 3654
+3  3449 3654 3450
+3  3450 3654 3655
+3  3450 3655 3656
+3  3450 3656 3451
+3  3451 3656 3657
+3  3451 3657 3452
+3  3452 3657 3658
+3  3452 3658 3453
+3  3453 3658 3659
+3  3453 3659 3454
+3  3454 3659 3455
+3  3455 3659 3660
+3  3455 3660 3661
+3  3455 3661 3456
+3  3456 3661 3662
+3  3456 3662 3457
+3  3457 3662 3663
+3  3457 3663 3458
+3  3458 3663 3664
+3  3458 3664 3459
+3  3459 3664 3665
+3  3459 3665 3460
+3  3460 3665 3666
+3  3460 3666 3461
+3  3461 3666 3667
+3  3461 3667 3462
+3  3462 3667 3668
+3  3462 3668 3463
+3  3463 3668 3669
+3  3463 3669 3464
+3  3464 3669 3670
+3  3464 3670 3465
+3  3465 3670 3671
+3  3465 3671 3466
+3  3466 3671 3672
+3  3466 3672 3467
+3  3467 3672 3673
+3  3467 3673 3468
+3  3468 3673 3674
+3  3468 3674 3469
+3  3469 3674 3470
+3  3470 3674 3675
+3  3470 3675 3676
+3  3470 3676 3471
+3  3471 3676 3677
+3  3471 3677 3678
+3  3471 3678 3472
+3  3472 3678 3679
+3  3472 3679 3680
+3  3472 3680 3473
+3  3473 3680 3474
+3  3474 3680 3681
+3  3474 3681 3475
+3  3475 3681 3682
+3  3475 3682 3683
+3  3475 3683 3476
+3  3476 3683 3477
+3  3477 3683 3684
+3  3477 3684 3685
+3  3477 3685 3478
+3  3478 3685 3479
+3  3479 3685 3686
+3  3479 3686 3480
+3  3480 3686 3687
+3  3480 3687 3688
+3  3480 3688 3481
+3  3481 3688 3482
+3  3482 3688 3689
+3  3482 3689 3483
+3  3483 3689 3690
+3  3483 3690 3691
+3  3483 3691 3484
+3  3484 3691 3692
+3  3484 3692 3485
+3  3485 3692 3486
+3  3486 3692 3693
+3  3486 3693 3694
+3  3486 3694 3695
+3  3486 3695 3487
+3  3487 3695 3696
+3  3487 3696 3488
+3  3488 3696 3697
+3  3488 3697 3489
+3  3489 3697 3698
+3  3489 3698 3490
+3  3490 3698 3699
+3  3490 3699 3700
+3  3490 3700 3491
+3  3491 3700 3492
+3  3492 3700 3701
+3  3492 3701 3493
+3  3493 3701 3702
+3  3493 3702 3494
+3  3494 3702 3703
+3  3494 3703 3495
+3  3495 3703 3704
+3  3495 3704 3496
+3  3496 3704 3705
+3  3496 3705 3497
+3  3497 3705 3706
+3  3497 3706 3498
+3  3498 3706 3707
+3  3498 3707 3499
+3  3499 3707 3708
+3  3499 3708 3500
+3  3500 3708 3709
+3  3500 3709 3710
+3  3500 3710 3501
+3  3501 3710 3711
+3  3501 3711 3502
+3  3502 3711 3503
+3  3503 3711 3712
+3  3503 3712 3713
+3  3503 3713 3714
+3  3503 3714 3504
+3  3504 3714 3715
+3  3504 3715 3505
+3  3505 3715 3716
+3  3505 3716 3507
+3  3507 3716 3508
+3  3508 3716 3717
+3  3508 3717 3509
+3  3509 3717 3718
+3  3509 3718 3510
+3  3510 3718 3719
+3  3510 3719 3511
+3  3511 3719 3720
+3  3511 3720 3512
+3  3512 3720 3721
+3  3512 3721 3513
+3  3513 3721 3722
+3  3513 3722 3514
+3  3514 3722 3723
+3  3514 3723 3515
+3  3515 3723 3724
+3  3515 3724 3516
+3  3516 3724 3725
+3  3516 3725 3517
+3  3517 3725 3726
+3  3517 3726 3518
+3  3518 3726 3727
+3  3518 3727 3519
+3  3519 3727 3728
+3  3519 3728 3520
+3  3520 3728 3729
+3  3520 3729 3521
+3  3521 3729 3730
+3  3521 3730 3522
+3  3522 3730 3731
+3  3522 3731 3523
+3  3523 3731 3732
+3  3523 3732 3733
+3  3523 3733 3524
+3  3524 3733 3525
+3  3525 3733 3734
+3  3525 3734 3735
+3  3525 3735 3526
+3  3526 3735 3736
+3  3526 3736 3527
+3  3527 3736 3528
+3  3528 3736 3737
+3  3528 3737 3738
+3  3528 3738 3529
+3  3529 3738 3739
+3  3529 3739 3530
+3  3530 3739 3740
+3  3530 3740 3531
+3  3531 3740 3741
+3  3531 3741 3532
+3  3532 3741 3533
+3  3533 3741 3742
+3  3533 3742 3743
+3  3533 3743 3534
+3  3534 3743 3744
+3  3534 3744 3535
+3  3535 3744 3745
+3  3535 3745 3536
+3  3536 3745 3746
+3  3536 3746 3537
+3  3537 3746 3747
+3  3537 3747 3748
+3  3537 3748 3749
+3  3537 3749 3538
+3  3538 3749 3539
+3  3539 3749 3750
+3  3539 3750 3541
+3  3541 3750 3751
+3  3541 3751 3542
+3  3542 3751 3752
+3  3542 3752 3543
+3  3543 3752 3753
+3  3543 3753 3754
+3  3543 3754 3544
+3  3544 3754 3545
+3  3545 3754 3755
+3  3545 3755 3546
+3  3546 3755 3756
+3  3546 3756 3547
+3  3547 3756 3757
+3  3547 3757 3548
+3  3548 3757 3758
+3  3548 3758 3549
+3  3549 3758 3759
+3  3549 3759 3550
+3  3550 3759 3760
+3  3550 3760 3761
+3  3550 3761 3762
+3  3550 3762 3551
+3  3551 3762 3552
+3  3552 3762 3763
+3  3552 3763 3553
+3  3553 3763 3764
+3  3553 3764 3554
+3  3554 3764 3765
+3  3554 3765 3555
+3  3555 3765 3766
+3  3555 3766 3556
+3  3556 3766 3767
+3  3556 3767 3557
+3  3557 3767 3768
+3  3557 3768 3558
+3  3558 3768 3769
+3  3558 3769 3559
+3  3559 3769 3770
+3  3559 3770 3771
+3  3559 3771 3560
+3  3560 3771 3772
+3  3560 3772 3561
+3  3561 3772 3773
+3  3561 3773 3562
+3  3562 3773 3774
+3  3562 3774 3563
+3  3563 3774 3775
+3  3563 3775 3564
+3  3564 3775 3776
+3  3564 3776 3565
+3  3565 3776 3777
+3  3565 3777 3566
+3  3566 3777 3778
+3  3566 3778 3567
+3  3567 3778 3779
+3  3567 3779 3568
+3  3568 3779 3780
+3  3568 3780 3569
+3  3569 3780 3781
+3  3569 3781 3570
+3  3570 3781 3782
+3  3570 3782 3571
+3  3571 3782 3783
+3  3571 3783 3572
+3  3572 3783 3784
+3  3572 3784 3573
+3  3573 3784 3785
+3  3573 3785 3786
+3  3573 3786 3574
+3  3574 3786 3575
+3  3575 3786 3787
+3  3575 3787 3576
+3  3576 3787 3788
+3  3576 3788 3577
+3  3577 3788 3789
+3  3577 3789 3578
+3  3578 3789 3790
+3  3578 3790 3791
+3  3578 3791 3579
+3  3579 3791 3580
+3  3580 3791 3792
+3  3580 3792 3581
+3  3581 3792 3793
+3  3581 3793 3582
+3  3582 3793 3794
+3  3582 3794 3583
+3  3583 3794 3795
+3  3583 3795 3584
+3  3584 3795 3796
+3  3584 3796 3797
+3  3584 3797 3585
+3  3585 3797 3798
+3  3585 3798 3586
+3  3586 3798 3587
+3  3587 3798 3799
+3  3587 3799 3800
+3  3587 3800 3801
+3  3587 3801 3588
+3  3588 3801 3589
+3  3589 3801 3590
+3  3590 3801 3802
+3  3590 3802 3803
+3  3590 3803 3591
+3  3591 3803 3804
+3  3591 3804 3592
+3  3592 3804 3805
+3  3592 3805 3593
+3  3593 3805 3806
+3  3593 3806 3594
+3  3594 3806 3807
+3  3594 3807 3595
+3  3595 3807 3596
+3  3596 3807 3808
+3  3596 3808 3809
+3  3596 3809 3597
+3  3597 3809 3598
+3  3598 3809 3810
+3  3598 3810 3599
+3  3599 3810 3811
+3  3599 3811 3812
+3  3599 3812 3600
+3  3600 3812 3813
+3  3600 3813 3601
+3  3601 3813 3814
+3  3601 3814 3603
+3  3603 3814 3604
+3  3604 3814 3815
+3  3604 3815 3605
+3  3605 3815 3816
+3  3605 3816 3817
+3  3605 3817 3607
+3  3607 3817 3818
+3  3607 3818 3819
+3  3607 3819 3608
+3  3608 3819 3609
+3  3609 3819 3820
+3  3609 3820 3611
+3  3611 3820 3821
+3  3611 3821 3612
+3  3612 3821 3822
+3  3612 3822 3613
+3  3613 3822 3823
+3  3613 3823 3614
+3  3614 3823 3824
+3  3614 3824 3615
+3  3615 3824 3825
+3  3615 3825 3616
+3  3616 3825 3826
+3  3616 3826 3827
+3  3616 3827 3617
+3  3617 3827 3828
+3  3617 3828 3618
+3  3618 3828 3829
+3  3618 3829 3619
+3  3619 3829 3830
+3  3619 3830 3620
+3  3620 3830 3621
+3  3621 3830 3831
+3  3621 3831 3622
+3  3622 3831 3832
+3  3622 3832 3623
+3  3623 3832 3833
+3  3623 3833 3624
+3  3624 3833 3834
+3  3624 3834 3835
+3  3624 3835 3625
+3  3625 3835 3626
+3  3626 3835 3836
+3  3626 3836 3627
+3  3627 3836 3837
+3  3627 3837 3838
+3  3627 3838 3628
+3  3628 3838 3839
+3  3628 3839 3629
+3  3629 3839 3840
+3  3629 3840 3630
+3  3630 3840 3841
+3  3630 3841 3631
+3  3631 3841 3632
+3  3632 3841 3842
+3  3632 3842 3843
+3  3632 3843 3633
+3  3633 3843 3844
+3  3633 3844 3634
+3  3634 3844 3845
+3  3634 3845 3846
+3  3634 3846 3635
+3  3635 3846 3636
+3  3636 3846 3637
+3  3637 3846 3847
+3  3637 3847 3638
+3  3638 3847 3848
+3  3638 3848 3639
+3  3639 3848 3849
+3  3639 3849 3850
+3  3639 3850 3851
+3  3639 3851 3640
+3  3640 3851 3852
+3  3640 3852 3641
+3  3641 3852 3853
+3  3641 3853 3642
+3  3642 3853 3854
+3  3642 3854 3643
+3  3643 3854 3855
+3  3643 3855 3644
+3  3644 3855 3856
+3  3644 3856 3857
+3  3644 3857 3645
+3  3645 3857 3858
+3  3645 3858 3859
+3  3645 3859 3646
+3  3646 3859 3860
+3  3646 3860 3861
+3  3646 3861 3647
+3  3647 3861 3862
+3  3647 3862 3863
+3  3647 3863 3649
+3  3649 3863 3651
+3  3651 3863 3862
+3  3651 3862 3652
+3  3652 3862 3864
+3  3652 3864 3865
+3  3652 3865 3653
+3  3653 3865 3866
+3  3653 3866 3654
+3  3654 3866 3655
+3  3655 3866 3867
+3  3655 3867 3656
+3  3656 3867 3868
+3  3656 3868 3657
+3  3657 3868 3869
+3  3657 3869 3658
+3  3658 3869 3870
+3  3658 3870 3659
+3  3659 3870 3871
+3  3659 3871 3660
+3  3660 3871 3872
+3  3660 3872 3661
+3  3661 3872 3873
+3  3661 3873 3662
+3  3662 3873 3874
+3  3662 3874 3875
+3  3662 3875 3663
+3  3663 3875 3876
+3  3663 3876 3664
+3  3664 3876 3877
+3  3664 3877 3665
+3  3665 3877 3878
+3  3665 3878 3666
+3  3666 3878 3879
+3  3666 3879 3667
+3  3667 3879 3880
+3  3667 3880 3668
+3  3668 3880 3881
+3  3668 3881 3669
+3  3669 3881 3670
+3  3670 3881 3882
+3  3670 3882 3671
+3  3671 3882 3883
+3  3671 3883 3884
+3  3671 3884 3672
+3  3672 3884 3885
+3  3672 3885 3673
+3  3673 3885 3886
+3  3673 3886 3674
+3  3674 3886 3675
+3  3675 3886 3887
+3  3675 3887 3888
+3  3675 3888 3676
+3  3676 3888 3889
+3  3676 3889 3677
+3  3677 3889 3890
+3  3677 3890 3891
+3  3677 3891 3678
+3  3678 3891 3679
+3  3679 3891 3892
+3  3679 3892 3893
+3  3679 3893 3681
+3  3681 3893 3682
+3  3682 3893 3894
+3  3682 3894 3895
+3  3682 3895 3896
+3  3682 3896 3683
+3  3683 3896 3684
+3  3684 3896 3897
+3  3684 3897 3898
+3  3684 3898 3685
+3  3685 3898 3686
+3  3686 3898 3899
+3  3686 3899 3687
+3  3687 3899 3900
+3  3687 3900 3688
+3  3688 3900 3901
+3  3688 3901 3689
+3  3689 3901 3902
+3  3689 3902 3690
+3  3690 3902 3903
+3  3690 3903 3691
+3  3691 3903 3904
+3  3691 3904 3693
+3  3693 3904 3905
+3  3693 3905 3906
+3  3693 3906 3907
+3  3693 3907 3694
+3  3694 3907 3908
+3  3694 3908 3909
+3  3694 3909 3695
+3  3695 3909 3910
+3  3695 3910 3696
+3  3696 3910 3911
+3  3696 3911 3697
+3  3697 3911 3912
+3  3697 3912 3698
+3  3698 3912 3913
+3  3698 3913 3699
+3  3699 3913 3914
+3  3699 3914 3915
+3  3699 3915 3700
+3  3700 3915 3701
+3  3701 3915 3916
+3  3701 3916 3702
+3  3702 3916 3917
+3  3702 3917 3703
+3  3703 3917 3918
+3  3703 3918 3704
+3  3704 3918 3919
+3  3704 3919 3705
+3  3705 3919 3920
+3  3705 3920 3706
+3  3706 3920 3921
+3  3706 3921 3707
+3  3707 3921 3922
+3  3707 3922 3708
+3  3708 3922 3923
+3  3708 3923 3709
+3  3709 3923 3924
+3  3709 3924 3925
+3  3709 3925 3710
+3  3710 3925 3926
+3  3710 3926 3711
+3  3711 3926 3712
+3  3712 3926 3927
+3  3712 3927 3928
+3  3712 3928 3713
+3  3713 3928 3929
+3  3713 3929 3930
+3  3713 3930 3714
+3  3714 3930 3931
+3  3714 3931 3932
+3  3714 3932 3715
+3  3715 3932 3933
+3  3715 3933 3934
+3  3715 3934 3717
+3  3717 3934 3718
+3  3718 3934 3935
+3  3718 3935 3719
+3  3719 3935 3936
+3  3719 3936 3720
+3  3720 3936 3937
+3  3720 3937 3721
+3  3721 3937 3938
+3  3721 3938 3722
+3  3722 3938 3939
+3  3722 3939 3723
+3  3723 3939 3940
+3  3723 3940 3724
+3  3724 3940 3941
+3  3724 3941 3725
+3  3725 3941 3942
+3  3725 3942 3726
+3  3726 3942 3943
+3  3726 3943 3727
+3  3727 3943 3944
+3  3727 3944 3728
+3  3728 3944 3945
+3  3728 3945 3729
+3  3729 3945 3946
+3  3729 3946 3730
+3  3730 3946 3947
+3  3730 3947 3731
+3  3731 3947 3948
+3  3731 3948 3732
+3  3732 3948 3949
+3  3732 3949 3734
+3  3734 3949 3950
+3  3734 3950 3735
+3  3735 3950 3951
+3  3735 3951 3952
+3  3735 3952 3736
+3  3736 3952 3737
+3  3737 3952 3953
+3  3737 3953 3954
+3  3737 3954 3738
+3  3738 3954 3955
+3  3738 3955 3739
+3  3739 3955 3956
+3  3739 3956 3740
+3  3740 3956 3957
+3  3740 3957 3741
+3  3741 3957 3742
+3  3742 3957 3958
+3  3742 3958 3959
+3  3742 3959 3743
+3  3743 3959 3960
+3  3743 3960 3744
+3  3744 3960 3961
+3  3744 3961 3745
+3  3745 3961 3962
+3  3745 3962 3746
+3  3746 3962 3963
+3  3746 3963 3747
+3  3747 3963 3964
+3  3747 3964 3965
+3  3747 3965 3966
+3  3747 3966 3748
+3  3748 3966 3967
+3  3748 3967 3749
+3  3749 3967 3750
+3  3750 3967 3968
+3  3750 3968 3751
+3  3751 3968 3969
+3  3751 3969 3752
+3  3752 3969 3970
+3  3752 3970 3753
+3  3753 3970 3971
+3  3753 3971 3972
+3  3753 3972 3754
+3  3754 3972 3973
+3  3754 3973 3974
+3  3754 3974 3755
+3  3755 3974 3756
+3  3756 3974 3975
+3  3756 3975 3976
+3  3756 3976 3977
+3  3756 3977 3757
+3  3757 3977 3758
+3  3758 3977 3978
+3  3758 3978 3759
+3  3759 3978 3979
+3  3759 3979 3760
+3  3760 3979 3980
+3  3760 3980 3981
+3  3760 3981 3761
+3  3761 3981 3982
+3  3761 3982 3762
+3  3762 3982 3983
+3  3762 3983 3763
+3  3763 3983 3984
+3  3763 3984 3764
+3  3764 3984 3985
+3  3764 3985 3765
+3  3765 3985 3986
+3  3765 3986 3766
+3  3766 3986 3987
+3  3766 3987 3767
+3  3767 3987 3988
+3  3767 3988 3768
+3  3768 3988 3989
+3  3768 3989 3769
+3  3769 3989 3990
+3  3769 3990 3991
+3  3769 3991 3770
+3  3770 3991 3992
+3  3770 3992 3771
+3  3771 3992 3993
+3  3771 3993 3772
+3  3772 3993 3994
+3  3772 3994 3773
+3  3773 3994 3995
+3  3773 3995 3774
+3  3774 3995 3996
+3  3774 3996 3775
+3  3775 3996 3997
+3  3775 3997 3776
+3  3776 3997 3998
+3  3776 3998 3777
+3  3777 3998 3999
+3  3777 3999 3778
+3  3778 3999 4000
+3  3778 4000 3779
+3  3779 4000 4001
+3  3779 4001 3780
+3  3780 4001 4002
+3  3780 4002 3781
+3  3781 4002 4003
+3  3781 4003 3782
+3  3782 4003 4004
+3  3782 4004 3783
+3  3783 4004 4005
+3  3783 4005 3784
+3  3784 4005 4006
+3  3784 4006 3785
+3  3785 4006 4007
+3  3785 4007 4008
+3  3785 4008 3786
+3  3786 4008 3787
+3  3787 4008 4009
+3  3787 4009 3788
+3  3788 4009 4010
+3  3788 4010 3789
+3  3789 4010 4011
+3  3789 4011 3790
+3  3790 4011 4012
+3  3790 4012 4013
+3  3790 4013 3791
+3  3791 4013 3792
+3  3792 4013 4014
+3  3792 4014 3793
+3  3793 4014 4015
+3  3793 4015 3794
+3  3794 4015 4016
+3  3794 4016 3795
+3  3795 4016 4017
+3  3795 4017 3796
+3  3796 4017 4018
+3  3796 4018 4019
+3  3796 4019 3797
+3  3797 4019 4020
+3  3797 4020 3798
+3  3798 4020 3799
+3  3799 4020 4021
+3  3799 4021 4022
+3  3799 4022 3800
+3  3800 4022 4023
+3  3800 4023 3802
+3  3802 4023 4024
+3  3802 4024 4025
+3  3802 4025 3803
+3  3803 4025 4026
+3  3803 4026 3804
+3  3804 4026 4027
+3  3804 4027 3805
+3  3805 4027 4028
+3  3805 4028 3806
+3  3806 4028 4029
+3  3806 4029 3807
+3  3807 4029 3808
+3  3808 4029 4030
+3  3808 4030 4031
+3  3808 4031 3809
+3  3809 4031 3810
+3  3810 4031 4032
+3  3810 4032 3811
+3  3811 4032 4033
+3  3811 4033 4034
+3  3811 4034 3812
+3  3812 4034 4035
+3  3812 4035 3813
+3  3813 4035 4036
+3  3813 4036 3814
+3  3814 4036 4037
+3  3814 4037 3815
+3  3815 4037 3816
+3  3816 4037 4038
+3  3816 4038 4039
+3  3816 4039 3817
+3  3817 4039 3818
+3  3818 4039 4040
+3  3818 4040 4041
+3  3818 4041 4042
+3  3818 4042 3819
+3  3819 4042 3820
+3  3820 4042 4043
+3  3820 4043 3821
+3  3821 4043 4044
+3  3821 4044 3822
+3  3822 4044 4045
+3  3822 4045 3823
+3  3823 4045 4046
+3  3823 4046 3824
+3  3824 4046 4047
+3  3824 4047 3825
+3  3825 4047 4048
+3  3825 4048 3826
+3  3826 4048 4049
+3  3826 4049 4050
+3  3826 4050 3827
+3  3827 4050 4051
+3  3827 4051 3828
+3  3828 4051 4052
+3  3828 4052 3829
+3  3829 4052 4053
+3  3829 4053 3830
+3  3830 4053 3831
+3  3831 4053 4054
+3  3831 4054 3832
+3  3832 4054 4055
+3  3832 4055 3833
+3  3833 4055 4056
+3  3833 4056 3834
+3  3834 4056 4057
+3  3834 4057 4058
+3  3834 4058 3835
+3  3835 4058 3836
+3  3836 4058 4059
+3  3836 4059 3837
+3  3837 4059 4060
+3  3837 4060 4061
+3  3837 4061 4062
+3  3837 4062 3838
+3  3838 4062 4063
+3  3838 4063 3839
+3  3839 4063 4064
+3  3839 4064 3840
+3  3840 4064 3842
+3  3842 4064 4065
+3  3842 4065 4066
+3  3842 4066 3843
+3  3843 4066 4067
+3  3843 4067 4068
+3  3843 4068 3844
+3  3844 4068 3845
+3  3845 4068 4069
+3  3845 4069 3847
+3  3847 4069 4070
+3  3847 4070 3848
+3  3848 4070 4071
+3  3848 4071 4072
+3  3848 4072 3849
+3  3849 4072 4073
+3  3849 4073 3850
+3  3850 4073 4074
+3  3850 4074 4075
+3  3850 4075 3851
+3  3851 4075 4076
+3  3851 4076 3852
+3  3852 4076 4077
+3  3852 4077 4078
+3  3852 4078 3853
+3  3853 4078 4079
+3  3853 4079 4080
+3  3853 4080 3854
+3  3854 4080 4081
+3  3854 4081 4082
+3  3854 4082 3855
+3  3855 4082 3856
+3  3856 4082 4083
+3  3856 4083 4084
+3  3856 4084 3858
+3  3858 4084 4085
+3  3858 4085 3860
+3  3860 4085 3864
+3  3864 4085 4086
+3  3864 4086 3865
+3  3865 4086 4087
+3  3865 4087 3866
+3  3866 4087 3867
+3  3867 4087 4088
+3  3867 4088 3868
+3  3868 4088 3869
+3  3869 4088 4089
+3  3869 4089 3870
+3  3870 4089 4090
+3  3870 4090 4091
+3  3870 4091 3871
+3  3871 4091 4092
+3  3871 4092 4093
+3  3871 4093 3872
+3  3872 4093 4094
+3  3872 4094 4095
+3  3872 4095 3873
+3  3873 4095 3874
+3  3874 4095 4096
+3  3874 4096 3875
+3  3875 4096 4097
+3  3875 4097 4098
+3  3875 4098 3876
+3  3876 4098 4099
+3  3876 4099 3877
+3  3877 4099 4100
+3  3877 4100 4101
+3  3877 4101 3878
+3  3878 4101 4102
+3  3878 4102 3879
+3  3879 4102 4103
+3  3879 4103 3880
+3  3880 4103 4104
+3  3880 4104 3881
+3  3881 4104 4105
+3  3881 4105 3882
+3  3882 4105 4106
+3  3882 4106 3883
+3  3883 4106 4107
+3  3883 4107 3884
+3  3884 4107 4108
+3  3884 4108 4109
+3  3884 4109 3885
+3  3885 4109 3886
+3  3886 4109 3887
+3  3887 4109 4110
+3  3887 4110 3888
+3  3888 4110 4111
+3  3888 4111 4112
+3  3888 4112 3889
+3  3889 4112 4113
+3  3889 4113 3890
+3  3890 4113 4114
+3  3890 4114 3891
+3  3891 4114 4115
+3  3891 4115 3892
+3  3892 4115 4116
+3  3892 4116 3893
+3  3893 4116 4117
+3  3893 4117 3894
+3  3894 4117 3895
+3  3895 4117 4118
+3  3895 4118 4119
+3  3895 4119 3896
+3  3896 4119 3897
+3  3897 4119 4120
+3  3897 4120 3898
+3  3898 4120 3899
+3  3899 4120 3900
+3  3900 4120 4121
+3  3900 4121 3901
+3  3901 4121 4122
+3  3901 4122 3902
+3  3902 4122 4123
+3  3902 4123 3903
+3  3903 4123 4124
+3  3903 4124 3904
+3  3904 4124 3905
+3  3905 4124 4125
+3  3905 4125 4126
+3  3905 4126 3906
+3  3906 4126 4127
+3  3906 4127 3907
+3  3907 4127 4128
+3  3907 4128 4129
+3  3907 4129 3908
+3  3908 4129 4130
+3  3908 4130 4131
+3  3908 4131 3909
+3  3909 4131 4132
+3  3909 4132 3910
+3  3910 4132 4133
+3  3910 4133 3911
+3  3911 4133 4134
+3  3911 4134 3912
+3  3912 4134 4135
+3  3912 4135 3913
+3  3913 4135 4136
+3  3913 4136 3914
+3  3914 4136 4137
+3  3914 4137 4138
+3  3914 4138 3915
+3  3915 4138 4139
+3  3915 4139 3916
+3  3916 4139 4140
+3  3916 4140 4141
+3  3916 4141 3917
+3  3917 4141 4142
+3  3917 4142 3918
+3  3918 4142 4143
+3  3918 4143 3919
+3  3919 4143 4144
+3  3919 4144 3920
+3  3920 4144 4145
+3  3920 4145 3921
+3  3921 4145 4146
+3  3921 4146 3922
+3  3922 4146 4147
+3  3922 4147 3923
+3  3923 4147 4148
+3  3923 4148 3924
+3  3924 4148 4149
+3  3924 4149 4150
+3  3924 4150 3925
+3  3925 4150 4151
+3  3925 4151 3926
+3  3926 4151 3927
+3  3927 4151 4152
+3  3927 4152 4153
+3  3927 4153 3928
+3  3928 4153 4154
+3  3928 4154 3929
+3  3929 4154 4155
+3  3929 4155 4156
+3  3929 4156 3930
+3  3930 4156 4157
+3  3930 4157 3931
+3  3931 4157 4158
+3  3931 4158 4159
+3  3931 4159 3932
+3  3932 4159 3933
+3  3933 4159 4160
+3  3933 4160 4161
+3  3933 4161 3934
+3  3934 4161 3935
+3  3935 4161 4162
+3  3935 4162 3936
+3  3936 4162 3937
+3  3937 4162 4163
+3  3937 4163 3938
+3  3938 4163 4164
+3  3938 4164 3939
+3  3939 4164 4165
+3  3939 4165 3940
+3  3940 4165 4166
+3  3940 4166 3941
+3  3941 4166 4167
+3  3941 4167 3942
+3  3942 4167 4168
+3  3942 4168 3943
+3  3943 4168 4169
+3  3943 4169 3944
+3  3944 4169 4170
+3  3944 4170 3945
+3  3945 4170 4171
+3  3945 4171 3946
+3  3946 4171 4172
+3  3946 4172 3947
+3  3947 4172 4173
+3  3947 4173 3948
+3  3948 4173 4174
+3  3948 4174 3949
+3  3949 4174 4175
+3  3949 4175 4176
+3  3949 4176 3950
+3  3950 4176 4177
+3  3950 4177 3951
+3  3951 4177 4178
+3  3951 4178 3953
+3  3953 4178 4179
+3  3953 4179 3954
+3  3954 4179 4180
+3  3954 4180 3955
+3  3955 4180 3956
+3  3956 4180 4181
+3  3956 4181 3957
+3  3957 4181 3958
+3  3958 4181 4182
+3  3958 4182 4183
+3  3958 4183 3959
+3  3959 4183 4184
+3  3959 4184 3960
+3  3960 4184 4185
+3  3960 4185 3961
+3  3961 4185 4186
+3  3961 4186 3962
+3  3962 4186 4187
+3  3962 4187 3963
+3  3963 4187 4188
+3  3963 4188 3964
+3  3964 4188 4189
+3  3964 4189 4190
+3  3964 4190 4191
+3  3964 4191 3965
+3  3965 4191 4192
+3  3965 4192 4193
+3  3965 4193 3966
+3  3966 4193 3968
+3  3968 4193 4194
+3  3968 4194 3969
+3  3969 4194 4195
+3  3969 4195 3970
+3  3970 4195 3971
+3  3971 4195 4196
+3  3971 4196 4197
+3  3971 4197 4198
+3  3971 4198 3972
+3  3972 4198 4199
+3  3972 4199 3973
+3  3973 4199 3974
+3  3974 4199 3975
+3  3975 4199 4200
+3  3975 4200 4201
+3  3975 4201 3976
+3  3976 4201 4202
+3  3976 4202 3977
+3  3977 4202 4203
+3  3977 4203 3979
+3  3979 4203 3980
+3  3980 4203 4202
+3  3980 4202 4204
+3  3980 4204 3981
+3  3981 4204 4205
+3  3981 4205 4206
+3  3981 4206 3982
+3  3982 4206 4207
+3  3982 4207 3984
+3  3984 4207 4208
+3  3984 4208 3985
+3  3985 4208 4209
+3  3985 4209 3986
+3  3986 4209 4210
+3  3986 4210 3987
+3  3987 4210 4211
+3  3987 4211 3988
+3  3988 4211 4212
+3  3988 4212 3989
+3  3989 4212 4213
+3  3989 4213 3990
+3  3990 4213 4214
+3  3990 4214 4215
+3  3990 4215 3991
+3  3991 4215 4216
+3  3991 4216 3992
+3  3992 4216 4217
+3  3992 4217 3993
+3  3993 4217 4218
+3  3993 4218 3994
+3  3994 4218 4219
+3  3994 4219 3995
+3  3995 4219 4220
+3  3995 4220 3996
+3  3996 4220 4221
+3  3996 4221 3997
+3  3997 4221 4222
+3  3997 4222 3998
+3  3998 4222 4223
+3  3998 4223 3999
+3  3999 4223 4224
+3  3999 4224 4000
+3  4000 4224 4225
+3  4000 4225 4001
+3  4001 4225 4226
+3  4001 4226 4002
+3  4002 4226 4227
+3  4002 4227 4003
+3  4003 4227 4228
+3  4003 4228 4004
+3  4004 4228 4229
+3  4004 4229 4005
+3  4005 4229 4230
+3  4005 4230 4006
+3  4006 4230 4231
+3  4006 4231 4007
+3  4007 4231 4232
+3  4007 4232 4233
+3  4007 4233 4008
+3  4008 4233 4009
+3  4009 4233 4234
+3  4009 4234 4010
+3  4010 4234 4235
+3  4010 4235 4011
+3  4011 4235 4236
+3  4011 4236 4012
+3  4012 4236 4237
+3  4012 4237 4238
+3  4012 4238 4013
+3  4013 4238 4014
+3  4014 4238 4239
+3  4014 4239 4015
+3  4015 4239 4240
+3  4015 4240 4016
+3  4016 4240 4241
+3  4016 4241 4017
+3  4017 4241 4242
+3  4017 4242 4018
+3  4018 4242 4243
+3  4018 4243 4244
+3  4018 4244 4019
+3  4019 4244 4245
+3  4019 4245 4020
+3  4020 4245 4021
+3  4021 4245 4246
+3  4021 4246 4247
+3  4021 4247 4022
+3  4022 4247 4248
+3  4022 4248 4023
+3  4023 4248 4024
+3  4024 4248 4249
+3  4024 4249 4250
+3  4024 4250 4025
+3  4025 4250 4251
+3  4025 4251 4026
+3  4026 4251 4252
+3  4026 4252 4027
+3  4027 4252 4253
+3  4027 4253 4028
+3  4028 4253 4254
+3  4028 4254 4029
+3  4029 4254 4030
+3  4030 4254 4255
+3  4030 4255 4256
+3  4030 4256 4031
+3  4031 4256 4032
+3  4032 4256 4257
+3  4032 4257 4033
+3  4033 4257 4258
+3  4033 4258 4259
+3  4033 4259 4034
+3  4034 4259 4260
+3  4034 4260 4035
+3  4035 4260 4261
+3  4035 4261 4036
+3  4036 4261 4037
+3  4037 4261 4038
+3  4038 4261 4262
+3  4038 4262 4263
+3  4038 4263 4039
+3  4039 4263 4040
+3  4040 4263 4264
+3  4040 4264 4265
+3  4040 4265 4041
+3  4041 4265 4266
+3  4041 4266 4267
+3  4041 4267 4042
+3  4042 4267 4043
+3  4043 4267 4268
+3  4043 4268 4044
+3  4044 4268 4269
+3  4044 4269 4045
+3  4045 4269 4270
+3  4045 4270 4046
+3  4046 4270 4271
+3  4046 4271 4047
+3  4047 4271 4272
+3  4047 4272 4048
+3  4048 4272 4273
+3  4048 4273 4049
+3  4049 4273 4274
+3  4049 4274 4275
+3  4049 4275 4276
+3  4049 4276 4050
+3  4050 4276 4051
+3  4051 4276 4277
+3  4051 4277 4278
+3  4051 4278 4052
+3  4052 4278 4279
+3  4052 4279 4053
+3  4053 4279 4054
+3  4054 4279 4280
+3  4054 4280 4055
+3  4055 4280 4281
+3  4055 4281 4056
+3  4056 4281 4282
+3  4056 4282 4057
+3  4057 4282 4283
+3  4057 4283 4058
+3  4058 4283 4284
+3  4058 4284 4059
+3  4059 4284 4285
+3  4059 4285 4060
+3  4060 4285 4286
+3  4060 4286 4287
+3  4060 4287 4061
+3  4061 4287 4288
+3  4061 4288 4062
+3  4062 4288 4289
+3  4062 4289 4290
+3  4062 4290 4063
+3  4063 4290 4065
+3  4065 4290 4291
+3  4065 4291 4292
+3  4065 4292 4066
+3  4066 4292 4293
+3  4066 4293 4294
+3  4066 4294 4067
+3  4067 4294 4295
+3  4067 4295 4068
+3  4068 4295 4296
+3  4068 4296 4297
+3  4068 4297 4069
+3  4069 4297 4070
+3  4070 4297 4298
+3  4070 4298 4071
+3  4071 4298 4299
+3  4071 4299 4300
+3  4071 4300 4072
+3  4072 4300 4301
+3  4072 4301 4073
+3  4073 4301 4302
+3  4073 4302 4074
+3  4074 4302 4303
+3  4074 4303 4304
+3  4074 4304 4075
+3  4075 4304 4305
+3  4075 4305 4076
+3  4076 4305 4077
+3  4077 4305 4306
+3  4077 4306 4307
+3  4077 4307 4078
+3  4078 4307 4079
+3  4079 4307 4308
+3  4079 4308 4309
+3  4079 4309 4080
+3  4080 4309 4081
+3  4081 4309 4310
+3  4081 4310 4083
+3  4083 4310 4311
+3  4083 4311 4084
+3  4084 4311 4312
+3  4084 4312 4085
+3  4085 4312 4086
+3  4086 4312 4313
+3  4086 4313 4087
+3  4087 4313 4088
+3  4088 4313 4314
+3  4088 4314 4089
+3  4089 4314 4315
+3  4089 4315 4090
+3  4090 4315 4316
+3  4090 4316 4091
+3  4091 4316 4317
+3  4091 4317 4092
+3  4092 4317 4318
+3  4092 4318 4319
+3  4092 4319 4093
+3  4093 4319 4094
+3  4094 4319 4320
+3  4094 4320 4321
+3  4094 4321 4095
+3  4095 4321 4096
+3  4096 4321 4322
+3  4096 4322 4097
+3  4097 4322 4323
+3  4097 4323 4324
+3  4097 4324 4098
+3  4098 4324 4325
+3  4098 4325 4099
+3  4099 4325 4100
+3  4100 4325 4326
+3  4100 4326 4101
+3  4101 4326 4327
+3  4101 4327 4328
+3  4101 4328 4102
+3  4102 4328 4329
+3  4102 4329 4103
+3  4103 4329 4330
+3  4103 4330 4104
+3  4104 4330 4331
+3  4104 4331 4105
+3  4105 4331 4332
+3  4105 4332 4333
+3  4105 4333 4106
+3  4106 4333 4334
+3  4106 4334 4107
+3  4107 4334 4335
+3  4107 4335 4108
+3  4108 4335 4336
+3  4108 4336 4109
+3  4109 4336 4337
+3  4109 4337 4110
+3  4110 4337 4338
+3  4110 4338 4111
+3  4111 4338 4339
+3  4111 4339 4340
+3  4111 4340 4112
+3  4112 4340 4341
+3  4112 4341 4113
+3  4113 4341 4342
+3  4113 4342 4114
+3  4114 4342 4343
+3  4114 4343 4115
+3  4115 4343 4344
+3  4115 4344 4345
+3  4115 4345 4116
+3  4116 4345 4346
+3  4116 4346 4117
+3  4117 4346 4347
+3  4117 4347 4118
+3  4118 4347 4348
+3  4118 4348 4119
+3  4119 4348 4349
+3  4119 4349 4120
+3  4120 4349 4121
+3  4121 4349 4122
+3  4122 4349 4123
+3  4123 4349 4350
+3  4123 4350 4124
+3  4124 4350 4351
+3  4124 4351 4352
+3  4124 4352 4125
+3  4125 4352 4353
+3  4125 4353 4126
+3  4126 4353 4354
+3  4126 4354 4127
+3  4127 4354 4355
+3  4127 4355 4128
+3  4128 4355 4356
+3  4128 4356 4357
+3  4128 4357 4129
+3  4129 4357 4130
+3  4130 4357 4358
+3  4130 4358 4359
+3  4130 4359 4131
+3  4131 4359 4132
+3  4132 4359 4360
+3  4132 4360 4361
+3  4132 4361 4133
+3  4133 4361 4362
+3  4133 4362 4134
+3  4134 4362 4363
+3  4134 4363 4135
+3  4135 4363 4364
+3  4135 4364 4365
+3  4135 4365 4136
+3  4136 4365 4366
+3  4136 4366 4137
+3  4137 4366 4367
+3  4137 4367 4368
+3  4137 4368 4138
+3  4138 4368 4369
+3  4138 4369 4139
+3  4139 4369 4370
+3  4139 4370 4140
+3  4140 4370 4371
+3  4140 4371 4372
+3  4140 4372 4141
+3  4141 4372 4373
+3  4141 4373 4142
+3  4142 4373 4374
+3  4142 4374 4143
+3  4143 4374 4375
+3  4143 4375 4144
+3  4144 4375 4376
+3  4144 4376 4145
+3  4145 4376 4377
+3  4145 4377 4146
+3  4146 4377 4378
+3  4146 4378 4147
+3  4147 4378 4379
+3  4147 4379 4148
+3  4148 4379 4380
+3  4148 4380 4149
+3  4149 4380 4381
+3  4149 4381 4382
+3  4149 4382 4150
+3  4150 4382 4383
+3  4150 4383 4151
+3  4151 4383 4152
+3  4152 4383 4384
+3  4152 4384 4385
+3  4152 4385 4153
+3  4153 4385 4386
+3  4153 4386 4154
+3  4154 4386 4387
+3  4154 4387 4388
+3  4154 4388 4155
+3  4155 4388 4389
+3  4155 4389 4390
+3  4155 4390 4391
+3  4155 4391 4156
+3  4156 4391 4157
+3  4157 4391 4392
+3  4157 4392 4393
+3  4157 4393 4158
+3  4158 4393 4394
+3  4158 4394 4395
+3  4158 4395 4159
+3  4159 4395 4160
+3  4160 4395 4396
+3  4160 4396 4397
+3  4160 4397 4161
+3  4161 4397 4162
+3  4162 4397 4163
+3  4163 4397 4398
+3  4163 4398 4164
+3  4164 4398 4399
+3  4164 4399 4165
+3  4165 4399 4400
+3  4165 4400 4166
+3  4166 4400 4401
+3  4166 4401 4167
+3  4167 4401 4402
+3  4167 4402 4168
+3  4168 4402 4403
+3  4168 4403 4169
+3  4169 4403 4404
+3  4169 4404 4170
+3  4170 4404 4405
+3  4170 4405 4171
+3  4171 4405 4406
+3  4171 4406 4172
+3  4172 4406 4407
+3  4172 4407 4173
+3  4173 4407 4408
+3  4173 4408 4174
+3  4174 4408 4409
+3  4174 4409 4175
+3  4175 4409 4410
+3  4175 4410 4176
+3  4176 4410 4411
+3  4176 4411 4412
+3  4176 4412 4177
+3  4177 4412 4413
+3  4177 4413 4178
+3  4178 4413 4414
+3  4178 4414 4179
+3  4179 4414 4415
+3  4179 4415 4180
+3  4180 4415 4181
+3  4181 4415 4182
+3  4182 4415 4416
+3  4182 4416 4417
+3  4182 4417 4183
+3  4183 4417 4418
+3  4183 4418 4184
+3  4184 4418 4419
+3  4184 4419 4185
+3  4185 4419 4420
+3  4185 4420 4186
+3  4186 4420 4421
+3  4186 4421 4187
+3  4187 4421 4422
+3  4187 4422 4188
+3  4188 4422 4423
+3  4188 4423 4189
+3  4189 4423 4424
+3  4189 4424 4190
+3  4190 4424 4425
+3  4190 4425 4426
+3  4190 4426 4191
+3  4191 4426 4427
+3  4191 4427 4192
+3  4192 4427 4194
+3  4194 4427 4428
+3  4194 4428 4195
+3  4195 4428 4196
+3  4196 4428 4429
+3  4196 4429 4430
+3  4196 4430 4197
+3  4197 4430 4431
+3  4197 4431 4432
+3  4197 4432 4198
+3  4198 4432 4200
+3  4200 4432 4433
+3  4200 4433 4201
+3  4201 4433 4434
+3  4201 4434 4435
+3  4201 4435 4202
+3  4202 4435 4204
+3  4204 4435 4436
+3  4204 4436 4205
+3  4205 4436 4437
+3  4205 4437 4438
+3  4205 4438 4206
+3  4206 4438 4439
+3  4206 4439 4207
+3  4207 4439 4440
+3  4207 4440 4208
+3  4208 4440 4441
+3  4208 4441 4209
+3  4209 4441 4442
+3  4209 4442 4443
+3  4209 4443 4444
+3  4209 4444 4210
+3  4210 4444 4211
+3  4211 4444 4445
+3  4211 4445 4212
+3  4212 4445 4446
+3  4212 4446 4213
+3  4213 4446 4447
+3  4213 4447 4214
+3  4214 4447 4448
+3  4214 4448 4449
+3  4214 4449 4215
+3  4215 4449 4450
+3  4215 4450 4216
+3  4216 4450 4451
+3  4216 4451 4217
+3  4217 4451 4452
+3  4217 4452 4218
+3  4218 4452 4453
+3  4218 4453 4219
+3  4219 4453 4454
+3  4219 4454 4220
+3  4220 4454 4455
+3  4220 4455 4221
+3  4221 4455 4456
+3  4221 4456 4222
+3  4222 4456 4457
+3  4222 4457 4223
+3  4223 4457 4458
+3  4223 4458 4224
+3  4224 4458 4459
+3  4224 4459 4225
+3  4225 4459 4460
+3  4225 4460 4226
+3  4226 4460 4461
+3  4226 4461 4227
+3  4227 4461 4462
+3  4227 4462 4228
+3  4228 4462 4463
+3  4228 4463 4229
+3  4229 4463 4464
+3  4229 4464 4230
+3  4230 4464 4465
+3  4230 4465 4231
+3  4231 4465 4466
+3  4231 4466 4232
+3  4232 4466 4467
+3  4232 4467 4468
+3  4232 4468 4233
+3  4233 4468 4234
+3  4234 4468 4469
+3  4234 4469 4235
+3  4235 4469 4470
+3  4235 4470 4236
+3  4236 4470 4471
+3  4236 4471 4237
+3  4237 4471 4472
+3  4237 4472 4473
+3  4237 4473 4238
+3  4238 4473 4239
+3  4239 4473 4474
+3  4239 4474 4240
+3  4240 4474 4475
+3  4240 4475 4241
+3  4241 4475 4476
+3  4241 4476 4242
+3  4242 4476 4477
+3  4242 4477 4243
+3  4243 4477 4478
+3  4243 4478 4479
+3  4243 4479 4244
+3  4244 4479 4480
+3  4244 4480 4245
+3  4245 4480 4246
+3  4246 4480 4481
+3  4246 4481 4482
+3  4246 4482 4247
+3  4247 4482 4483
+3  4247 4483 4248
+3  4248 4483 4249
+3  4249 4483 4484
+3  4249 4484 4485
+3  4249 4485 4250
+3  4250 4485 4486
+3  4250 4486 4251
+3  4251 4486 4487
+3  4251 4487 4252
+3  4252 4487 4488
+3  4252 4488 4253
+3  4253 4488 4489
+3  4253 4489 4254
+3  4254 4489 4255
+3  4255 4489 4490
+3  4255 4490 4491
+3  4255 4491 4256
+3  4256 4491 4257
+3  4257 4491 4492
+3  4257 4492 4258
+3  4258 4492 4493
+3  4258 4493 4494
+3  4258 4494 4259
+3  4259 4494 4495
+3  4259 4495 4260
+3  4260 4495 4262
+3  4262 4495 4496
+3  4262 4496 4263
+3  4263 4496 4264
+3  4264 4496 4497
+3  4264 4497 4498
+3  4264 4498 4265
+3  4265 4498 4499
+3  4265 4499 4266
+3  4266 4499 4500
+3  4266 4500 4501
+3  4266 4501 4267
+3  4267 4501 4268
+3  4268 4501 4502
+3  4268 4502 4269
+3  4269 4502 4503
+3  4269 4503 4270
+3  4270 4503 4504
+3  4270 4504 4271
+3  4271 4504 4505
+3  4271 4505 4506
+3  4271 4506 4272
+3  4272 4506 4507
+3  4272 4507 4508
+3  4272 4508 4273
+3  4273 4508 4509
+3  4273 4509 4274
+3  4274 4509 4510
+3  4274 4510 4511
+3  4274 4511 4275
+3  4275 4511 4512
+3  4275 4512 4513
+3  4275 4513 4276
+3  4276 4513 4514
+3  4276 4514 4277
+3  4277 4514 4515
+3  4277 4515 4516
+3  4277 4516 4278
+3  4278 4516 4517
+3  4278 4517 4279
+3  4279 4517 4280
+3  4280 4517 4518
+3  4280 4518 4281
+3  4281 4518 4519
+3  4281 4519 4282
+3  4282 4519 4520
+3  4282 4520 4521
+3  4282 4521 4283
+3  4283 4521 4522
+3  4283 4522 4284
+3  4284 4522 4285
+3  4285 4522 4523
+3  4285 4523 4524
+3  4285 4524 4286
+3  4286 4524 4525
+3  4286 4525 4287
+3  4287 4525 4526
+3  4287 4526 4288
+3  4288 4526 4527
+3  4288 4527 4289
+3  4289 4527 4528
+3  4289 4528 4290
+3  4290 4528 4291
+3  4291 4528 4529
+3  4291 4529 4530
+3  4291 4530 4292
+3  4292 4530 4531
+3  4292 4531 4293
+3  4293 4531 4532
+3  4293 4532 4533
+3  4293 4533 4294
+3  4294 4533 4534
+3  4294 4534 4535
+3  4294 4535 4295
+3  4295 4535 4536
+3  4295 4536 4296
+3  4296 4536 4537
+3  4296 4537 4538
+3  4296 4538 4297
+3  4297 4538 4298
+3  4298 4538 4539
+3  4298 4539 4299
+3  4299 4539 4540
+3  4299 4540 4541
+3  4299 4541 4300
+3  4300 4541 4542
+3  4300 4542 4301
+3  4301 4542 4543
+3  4301 4543 4302
+3  4302 4543 4544
+3  4302 4544 4303
+3  4303 4544 4545
+3  4303 4545 4546
+3  4303 4546 4304
+3  4304 4546 4547
+3  4304 4547 4305
+3  4305 4547 4306
+3  4306 4547 4548
+3  4306 4548 4549
+3  4306 4549 4307
+3  4307 4549 4308
+3  4308 4549 4550
+3  4308 4550 4551
+3  4308 4551 4309
+3  4309 4551 4310
+3  4310 4551 4552
+3  4310 4552 4311
+3  4311 4552 4553
+3  4311 4553 4312
+3  4312 4553 4313
+3  4313 4553 4314
+3  4314 4553 4554
+3  4314 4554 4315
+3  4315 4554 4555
+3  4315 4555 4316
+3  4316 4555 4556
+3  4316 4556 4317
+3  4317 4556 4557
+3  4317 4557 4318
+3  4318 4557 4558
+3  4318 4558 4559
+3  4318 4559 4319
+3  4319 4559 4320
+3  4320 4559 4560
+3  4320 4560 4561
+3  4320 4561 4321
+3  4321 4561 4322
+3  4322 4561 4562
+3  4322 4562 4323
+3  4323 4562 4563
+3  4323 4563 4564
+3  4323 4564 4324
+3  4324 4564 4565
+3  4324 4565 4325
+3  4325 4565 4326
+3  4326 4565 4566
+3  4326 4566 4327
+3  4327 4566 4567
+3  4327 4567 4568
+3  4327 4568 4328
+3  4328 4568 4569
+3  4328 4569 4329
+3  4329 4569 4570
+3  4329 4570 4330
+3  4330 4570 4571
+3  4330 4571 4331
+3  4331 4571 4572
+3  4331 4572 4332
+3  4332 4572 4573
+3  4332 4573 4574
+3  4332 4574 4333
+3  4333 4574 4575
+3  4333 4575 4334
+3  4334 4575 4335
+3  4335 4575 4576
+3  4335 4576 4577
+3  4335 4577 4336
+3  4336 4577 4578
+3  4336 4578 4579
+3  4336 4579 4337
+3  4337 4579 4338
+3  4338 4579 4580
+3  4338 4580 4339
+3  4339 4580 4581
+3  4339 4581 4582
+3  4339 4582 4340
+3  4340 4582 4583
+3  4340 4583 4341
+3  4341 4583 4584
+3  4341 4584 4342
+3  4342 4584 4585
+3  4342 4585 4343
+3  4343 4585 4586
+3  4343 4586 4344
+3  4344 4586 4587
+3  4344 4587 4588
+3  4344 4588 4345
+3  4345 4588 4346
+3  4346 4588 4589
+3  4346 4589 4347
+3  4347 4589 4590
+3  4347 4590 4591
+3  4347 4591 4348
+3  4348 4591 4350
+3  4350 4591 4351
+3  4351 4591 4592
+3  4351 4592 4352
+3  4352 4592 4593
+3  4352 4593 4353
+3  4353 4593 4594
+3  4353 4594 4595
+3  4353 4595 4354
+3  4354 4595 4355
+3  4355 4595 4596
+3  4355 4596 4356
+3  4356 4596 4597
+3  4356 4597 4598
+3  4356 4598 4357
+3  4357 4598 4358
+3  4358 4598 4599
+3  4358 4599 4600
+3  4358 4600 4359
+3  4359 4600 4601
+3  4359 4601 4360
+3  4360 4601 4602
+3  4360 4602 4603
+3  4360 4603 4361
+3  4361 4603 4604
+3  4361 4604 4605
+3  4361 4605 4362
+3  4362 4605 4606
+3  4362 4606 4363
+3  4363 4606 4607
+3  4363 4607 4364
+3  4364 4607 4608
+3  4364 4608 4609
+3  4364 4609 4365
+3  4365 4609 4366
+3  4366 4609 4610
+3  4366 4610 4367
+3  4367 4610 4611
+3  4367 4611 4612
+3  4367 4612 4368
+3  4368 4612 4613
+3  4368 4613 4369
+3  4369 4613 4614
+3  4369 4614 4370
+3  4370 4614 4615
+3  4370 4615 4371
+3  4371 4615 4616
+3  4371 4616 4617
+3  4371 4617 4372
+3  4372 4617 4618
+3  4372 4618 4373
+3  4373 4618 4619
+3  4373 4619 4374
+3  4374 4619 4620
+3  4374 4620 4375
+3  4375 4620 4621
+3  4375 4621 4376
+3  4376 4621 4622
+3  4376 4622 4377
+3  4377 4622 4623
+3  4377 4623 4378
+3  4378 4623 4624
+3  4378 4624 4379
+3  4379 4624 4625
+3  4379 4625 4380
+3  4380 4625 4626
+3  4380 4626 4381
+3  4381 4626 4627
+3  4381 4627 4628
+3  4381 4628 4382
+3  4382 4628 4629
+3  4382 4629 4383
+3  4383 4629 4384
+3  4384 4629 4630
+3  4384 4630 4631
+3  4384 4631 4385
+3  4385 4631 4632
+3  4385 4632 4386
+3  4386 4632 4387
+3  4387 4632 4633
+3  4387 4633 4634
+3  4387 4634 4635
+3  4387 4635 4388
+3  4388 4635 4389
+3  4389 4635 4636
+3  4389 4636 4637
+3  4389 4637 4638
+3  4389 4638 4639
+3  4389 4639 4390
+3  4390 4639 4391
+3  4391 4639 4392
+3  4392 4639 4393
+3  4393 4639 4640
+3  4393 4640 4394
+3  4394 4640 4641
+3  4394 4641 4642
+3  4394 4642 4643
+3  4394 4643 4395
+3  4395 4643 4396
+3  4396 4643 4644
+3  4396 4644 4398
+3  4398 4644 4399
+3  4399 4644 4645
+3  4399 4645 4400
+3  4400 4645 4646
+3  4400 4646 4401
+3  4401 4646 4647
+3  4401 4647 4402
+3  4402 4647 4648
+3  4402 4648 4403
+3  4403 4648 4649
+3  4403 4649 4404
+3  4404 4649 4650
+3  4404 4650 4405
+3  4405 4650 4651
+3  4405 4651 4406
+3  4406 4651 4652
+3  4406 4652 4407
+3  4407 4652 4653
+3  4407 4653 4408
+3  4408 4653 4654
+3  4408 4654 4409
+3  4409 4654 4655
+3  4409 4655 4410
+3  4410 4655 4656
+3  4410 4656 4657
+3  4410 4657 4411
+3  4411 4657 4658
+3  4411 4658 4412
+3  4412 4658 4659
+3  4412 4659 4413
+3  4413 4659 4660
+3  4413 4660 4414
+3  4414 4660 4661
+3  4414 4661 4415
+3  4415 4661 4416
+3  4416 4661 4662
+3  4416 4662 4663
+3  4416 4663 4417
+3  4417 4663 4664
+3  4417 4664 4418
+3  4418 4664 4665
+3  4418 4665 4419
+3  4419 4665 4420
+3  4420 4665 4666
+3  4420 4666 4421
+3  4421 4666 4667
+3  4421 4667 4422
+3  4422 4667 4668
+3  4422 4668 4669
+3  4422 4669 4423
+3  4423 4669 4670
+3  4423 4670 4424
+3  4424 4670 4671
+3  4424 4671 4425
+3  4425 4671 4672
+3  4425 4672 4429
+3  4429 4672 4430
+3  4430 4672 4673
+3  4430 4673 4431
+3  4431 4673 4674
+3  4431 4674 4675
+3  4431 4675 4432
+3  4432 4675 4433
+3  4433 4675 4676
+3  4433 4676 4434
+3  4434 4676 4677
+3  4434 4677 4678
+3  4434 4678 4435
+3  4435 4678 4436
+3  4436 4678 4679
+3  4436 4679 4437
+3  4437 4679 4680
+3  4437 4680 4681
+3  4437 4681 4438
+3  4438 4681 4682
+3  4438 4682 4439
+3  4439 4682 4683
+3  4439 4683 4440
+3  4440 4683 4684
+3  4440 4684 4685
+3  4440 4685 4441
+3  4441 4685 4686
+3  4441 4686 4442
+3  4442 4686 4687
+3  4442 4687 4688
+3  4442 4688 4443
+3  4443 4688 4689
+3  4443 4689 4690
+3  4443 4690 4444
+3  4444 4690 4691
+3  4444 4691 4445
+3  4445 4691 4446
+3  4446 4691 4692
+3  4446 4692 4447
+3  4447 4692 4693
+3  4447 4693 4448
+3  4448 4693 4694
+3  4448 4694 4695
+3  4448 4695 4449
+3  4449 4695 4696
+3  4449 4696 4450
+3  4450 4696 4697
+3  4450 4697 4451
+3  4451 4697 4698
+3  4451 4698 4452
+3  4452 4698 4699
+3  4452 4699 4453
+3  4453 4699 4700
+3  4453 4700 4454
+3  4454 4700 4701
+3  4454 4701 4455
+3  4455 4701 4702
+3  4455 4702 4456
+3  4456 4702 4703
+3  4456 4703 4457
+3  4457 4703 4704
+3  4457 4704 4458
+3  4458 4704 4705
+3  4458 4705 4459
+3  4459 4705 4706
+3  4459 4706 4460
+3  4460 4706 4707
+3  4460 4707 4461
+3  4461 4707 4708
+3  4461 4708 4462
+3  4462 4708 4709
+3  4462 4709 4463
+3  4463 4709 4710
+3  4463 4710 4464
+3  4464 4710 4711
+3  4464 4711 4465
+3  4465 4711 4712
+3  4465 4712 4466
+3  4466 4712 4713
+3  4466 4713 4467
+3  4467 4713 4714
+3  4467 4714 4715
+3  4467 4715 4468
+3  4468 4715 4469
+3  4469 4715 4716
+3  4469 4716 4470
+3  4470 4716 4717
+3  4470 4717 4471
+3  4471 4717 4718
+3  4471 4718 4472
+3  4472 4718 4719
+3  4472 4719 4720
+3  4472 4720 4473
+3  4473 4720 4474
+3  4474 4720 4721
+3  4474 4721 4475
+3  4475 4721 4722
+3  4475 4722 4476
+3  4476 4722 4723
+3  4476 4723 4477
+3  4477 4723 4724
+3  4477 4724 4478
+3  4478 4724 4725
+3  4478 4725 4726
+3  4478 4726 4479
+3  4479 4726 4727
+3  4479 4727 4480
+3  4480 4727 4481
+3  4481 4727 4728
+3  4481 4728 4729
+3  4481 4729 4482
+3  4482 4729 4730
+3  4482 4730 4483
+3  4483 4730 4484
+3  4484 4730 4731
+3  4484 4731 4732
+3  4484 4732 4485
+3  4485 4732 4733
+3  4485 4733 4486
+3  4486 4733 4734
+3  4486 4734 4487
+3  4487 4734 4735
+3  4487 4735 4488
+3  4488 4735 4736
+3  4488 4736 4489
+3  4489 4736 4490
+3  4490 4736 4737
+3  4490 4737 4738
+3  4490 4738 4491
+3  4491 4738 4492
+3  4492 4738 4739
+3  4492 4739 4493
+3  4493 4739 4740
+3  4493 4740 4741
+3  4493 4741 4494
+3  4494 4741 4742
+3  4494 4742 4495
+3  4495 4742 4496
+3  4496 4742 4497
+3  4497 4742 4743
+3  4497 4743 4744
+3  4497 4744 4498
+3  4498 4744 4745
+3  4498 4745 4499
+3  4499 4745 4746
+3  4499 4746 4747
+3  4499 4747 4500
+3  4500 4747 4748
+3  4500 4748 4749
+3  4500 4749 4501
+3  4501 4749 4750
+3  4501 4750 4502
+3  4502 4750 4751
+3  4502 4751 4752
+3  4502 4752 4503
+3  4503 4752 4753
+3  4503 4753 4754
+3  4503 4754 4504
+3  4504 4754 4755
+3  4504 4755 4505
+3  4505 4755 4756
+3  4505 4756 4757
+3  4505 4757 4506
+3  4506 4757 4507
+3  4507 4757 4758
+3  4507 4758 4759
+3  4507 4759 4508
+3  4508 4759 4509
+3  4509 4759 4760
+3  4509 4760 4510
+3  4510 4760 4761
+3  4510 4761 4762
+3  4510 4762 4511
+3  4511 4762 4763
+3  4511 4763 4512
+3  4512 4763 4764
+3  4512 4764 4765
+3  4512 4765 4513
+3  4513 4765 4514
+3  4514 4765 4766
+3  4514 4766 4767
+3  4514 4767 4768
+3  4514 4768 4515
+3  4515 4768 4769
+3  4515 4769 4770
+3  4515 4770 4516
+3  4516 4770 4771
+3  4516 4771 4772
+3  4516 4772 4517
+3  4517 4772 4518
+3  4518 4772 4773
+3  4518 4773 4774
+3  4518 4774 4519
+3  4519 4774 4775
+3  4519 4775 4520
+3  4520 4775 4776
+3  4520 4776 4777
+3  4520 4777 4521
+3  4521 4777 4522
+3  4522 4777 4523
+3  4523 4777 4778
+3  4523 4778 4524
+3  4524 4778 4779
+3  4524 4779 4780
+3  4524 4780 4525
+3  4525 4780 4781
+3  4525 4781 4526
+3  4526 4781 4782
+3  4526 4782 4527
+3  4527 4782 4783
+3  4527 4783 4528
+3  4528 4783 4529
+3  4529 4783 4784
+3  4529 4784 4785
+3  4529 4785 4530
+3  4530 4785 4786
+3  4530 4786 4531
+3  4531 4786 4787
+3  4531 4787 4788
+3  4531 4788 4532
+3  4532 4788 4533
+3  4533 4788 4789
+3  4533 4789 4534
+3  4534 4789 4790
+3  4534 4790 4791
+3  4534 4791 4792
+3  4534 4792 4535
+3  4535 4792 4536
+3  4536 4792 4793
+3  4536 4793 4537
+3  4537 4793 4794
+3  4537 4794 4538
+3  4538 4794 4795
+3  4538 4795 4539
+3  4539 4795 4796
+3  4539 4796 4797
+3  4539 4797 4540
+3  4540 4797 4798
+3  4540 4798 4799
+3  4540 4799 4541
+3  4541 4799 4800
+3  4541 4800 4542
+3  4542 4800 4801
+3  4542 4801 4802
+3  4542 4802 4543
+3  4543 4802 4803
+3  4543 4803 4544
+3  4544 4803 4804
+3  4544 4804 4545
+3  4545 4804 4805
+3  4545 4805 4546
+3  4546 4805 4806
+3  4546 4806 4547
+3  4547 4806 4548
+3  4548 4806 4807
+3  4548 4807 4808
+3  4548 4808 4549
+3  4549 4808 4550
+3  4550 4808 4809
+3  4550 4809 4810
+3  4550 4810 4551
+3  4551 4810 4552
+3  4552 4810 4554
+3  4554 4810 4555
+3  4555 4810 4809
+3  4555 4809 4556
+3  4556 4809 4811
+3  4556 4811 4557
+3  4557 4811 4812
+3  4557 4812 4558
+3  4558 4812 4813
+3  4558 4813 4814
+3  4558 4814 4559
+3  4559 4814 4560
+3  4560 4814 4815
+3  4560 4815 4816
+3  4560 4816 4561
+3  4561 4816 4562
+3  4562 4816 4817
+3  4562 4817 4563
+3  4563 4817 4818
+3  4563 4818 4819
+3  4563 4819 4564
+3  4564 4819 4820
+3  4564 4820 4565
+3  4565 4820 4566
+3  4566 4820 4821
+3  4566 4821 4567
+3  4567 4821 4822
+3  4567 4822 4823
+3  4567 4823 4568
+3  4568 4823 4824
+3  4568 4824 4570
+3  4570 4824 4571
+3  4571 4824 4825
+3  4571 4825 4826
+3  4571 4826 4827
+3  4571 4827 4572
+3  4572 4827 4828
+3  4572 4828 4829
+3  4572 4829 4573
+3  4573 4829 4830
+3  4573 4830 4574
+3  4574 4830 4831
+3  4574 4831 4832
+3  4574 4832 4575
+3  4575 4832 4576
+3  4576 4832 4833
+3  4576 4833 4577
+3  4577 4833 4834
+3  4577 4834 4578
+3  4578 4834 4835
+3  4578 4835 4836
+3  4578 4836 4579
+3  4579 4836 4580
+3  4580 4836 4837
+3  4580 4837 4581
+3  4581 4837 4838
+3  4581 4838 4839
+3  4581 4839 4582
+3  4582 4839 4840
+3  4582 4840 4583
+3  4583 4840 4841
+3  4583 4841 4584
+3  4584 4841 4842
+3  4584 4842 4585
+3  4585 4842 4843
+3  4585 4843 4586
+3  4586 4843 4844
+3  4586 4844 4587
+3  4587 4844 4845
+3  4587 4845 4846
+3  4587 4846 4588
+3  4588 4846 4589
+3  4589 4846 4847
+3  4589 4847 4590
+3  4590 4847 4848
+3  4590 4848 4592
+3  4592 4848 4593
+3  4593 4848 4849
+3  4593 4849 4594
+3  4594 4849 4850
+3  4594 4850 4851
+3  4594 4851 4595
+3  4595 4851 4596
+3  4596 4851 4852
+3  4596 4852 4853
+3  4596 4853 4597
+3  4597 4853 4854
+3  4597 4854 4855
+3  4597 4855 4598
+3  4598 4855 4599
+3  4599 4855 4856
+3  4599 4856 4857
+3  4599 4857 4600
+3  4600 4857 4601
+3  4601 4857 4858
+3  4601 4858 4602
+3  4602 4858 4859
+3  4602 4859 4860
+3  4602 4860 4603
+3  4603 4860 4861
+3  4603 4861 4604
+3  4604 4861 4862
+3  4604 4862 4605
+3  4605 4862 4863
+3  4605 4863 4606
+3  4606 4863 4864
+3  4606 4864 4607
+3  4607 4864 4608
+3  4608 4864 4865
+3  4608 4865 4866
+3  4608 4866 4609
+3  4609 4866 4610
+3  4610 4866 4867
+3  4610 4867 4611
+3  4611 4867 4868
+3  4611 4868 4869
+3  4611 4869 4612
+3  4612 4869 4870
+3  4612 4870 4613
+3  4613 4870 4871
+3  4613 4871 4614
+3  4614 4871 4872
+3  4614 4872 4615
+3  4615 4872 4873
+3  4615 4873 4616
+3  4616 4873 4874
+3  4616 4874 4875
+3  4616 4875 4617
+3  4617 4875 4876
+3  4617 4876 4618
+3  4618 4876 4877
+3  4618 4877 4619
+3  4619 4877 4878
+3  4619 4878 4620
+3  4620 4878 4879
+3  4620 4879 4621
+3  4621 4879 4880
+3  4621 4880 4622
+3  4622 4880 4881
+3  4622 4881 4623
+3  4623 4881 4882
+3  4623 4882 4624
+3  4624 4882 4883
+3  4624 4883 4625
+3  4625 4883 4884
+3  4625 4884 4626
+3  4626 4884 4885
+3  4626 4885 4627
+3  4627 4885 4886
+3  4627 4886 4887
+3  4627 4887 4628
+3  4628 4887 4888
+3  4628 4888 4629
+3  4629 4888 4630
+3  4630 4888 4889
+3  4630 4889 4890
+3  4630 4890 4631
+3  4631 4890 4891
+3  4631 4891 4632
+3  4632 4891 4633
+3  4633 4891 4892
+3  4633 4892 4893
+3  4633 4893 4634
+3  4634 4893 4894
+3  4634 4894 4895
+3  4634 4895 4896
+3  4634 4896 4635
+3  4635 4896 4897
+3  4635 4897 4636
+3  4636 4897 4898
+3  4636 4898 4637
+3  4637 4898 4899
+3  4637 4899 4900
+3  4637 4900 4638
+3  4638 4900 4640
+3  4640 4900 4641
+3  4641 4900 4901
+3  4641 4901 4902
+3  4641 4902 4642
+3  4642 4902 4903
+3  4642 4903 4904
+3  4642 4904 4643
+3  4643 4904 4644
+3  4644 4904 4645
+3  4645 4904 4905
+3  4645 4905 4646
+3  4646 4905 4906
+3  4646 4906 4647
+3  4647 4906 4907
+3  4647 4907 4648
+3  4648 4907 4908
+3  4648 4908 4649
+3  4649 4908 4909
+3  4649 4909 4650
+3  4650 4909 4910
+3  4650 4910 4651
+3  4651 4910 4911
+3  4651 4911 4652
+3  4652 4911 4912
+3  4652 4912 4653
+3  4653 4912 4913
+3  4653 4913 4654
+3  4654 4913 4914
+3  4654 4914 4655
+3  4655 4914 4915
+3  4655 4915 4656
+3  4656 4915 4916
+3  4656 4916 4917
+3  4656 4917 4657
+3  4657 4917 4918
+3  4657 4918 4658
+3  4658 4918 4919
+3  4658 4919 4659
+3  4659 4919 4920
+3  4659 4920 4660
+3  4660 4920 4921
+3  4660 4921 4661
+3  4661 4921 4662
+3  4662 4921 4922
+3  4662 4922 4923
+3  4662 4923 4663
+3  4663 4923 4924
+3  4663 4924 4664
+3  4664 4924 4925
+3  4664 4925 4665
+3  4665 4925 4926
+3  4665 4926 4666
+3  4666 4926 4927
+3  4666 4927 4667
+3  4667 4927 4928
+3  4667 4928 4668
+3  4668 4928 4929
+3  4668 4929 4669
+3  4669 4929 4930
+3  4669 4930 4931
+3  4669 4931 4670
+3  4670 4931 4932
+3  4670 4932 4671
+3  4671 4932 4672
+3  4672 4932 4933
+3  4672 4933 4673
+3  4673 4933 4934
+3  4673 4934 4674
+3  4674 4934 4935
+3  4674 4935 4675
+3  4675 4935 4936
+3  4675 4936 4676
+3  4676 4936 4937
+3  4676 4937 4677
+3  4677 4937 4938
+3  4677 4938 4939
+3  4677 4939 4678
+3  4678 4939 4679
+3  4679 4939 4940
+3  4679 4940 4680
+3  4680 4940 4941
+3  4680 4941 4942
+3  4680 4942 4681
+3  4681 4942 4943
+3  4681 4943 4682
+3  4682 4943 4944
+3  4682 4944 4683
+3  4683 4944 4684
+3  4684 4944 4945
+3  4684 4945 4946
+3  4684 4946 4947
+3  4684 4947 4685
+3  4685 4947 4686
+3  4686 4947 4948
+3  4686 4948 4687
+3  4687 4948 4949
+3  4687 4949 4950
+3  4687 4950 4951
+3  4687 4951 4952
+3  4687 4952 4688
+3  4688 4952 4689
+3  4689 4952 4953
+3  4689 4953 4954
+3  4689 4954 4690
+3  4690 4954 4955
+3  4690 4955 4691
+3  4691 4955 4956
+3  4691 4956 4692
+3  4692 4956 4957
+3  4692 4957 4958
+3  4692 4958 4693
+3  4693 4958 4959
+3  4693 4959 4960
+3  4693 4960 4694
+3  4694 4960 4961
+3  4694 4961 4695
+3  4695 4961 4962
+3  4695 4962 4696
+3  4696 4962 4697
+3  4697 4962 4963
+3  4697 4963 4964
+3  4697 4964 4698
+3  4698 4964 4965
+3  4698 4965 4699
+3  4699 4965 4966
+3  4699 4966 4700
+3  4700 4966 4967
+3  4700 4967 4701
+3  4701 4967 4968
+3  4701 4968 4702
+3  4702 4968 4969
+3  4702 4969 4703
+3  4703 4969 4970
+3  4703 4970 4704
+3  4704 4970 4971
+3  4704 4971 4705
+3  4705 4971 4972
+3  4705 4972 4706
+3  4706 4972 4973
+3  4706 4973 4707
+3  4707 4973 4974
+3  4707 4974 4708
+3  4708 4974 4975
+3  4708 4975 4709
+3  4709 4975 4976
+3  4709 4976 4710
+3  4710 4976 4977
+3  4710 4977 4711
+3  4711 4977 4978
+3  4711 4978 4712
+3  4712 4978 4713
+3  4713 4978 4979
+3  4713 4979 4714
+3  4714 4979 4980
+3  4714 4980 4981
+3  4714 4981 4715
+3  4715 4981 4716
+3  4716 4981 4982
+3  4716 4982 4717
+3  4717 4982 4983
+3  4717 4983 4718
+3  4718 4983 4984
+3  4718 4984 4719
+3  4719 4984 4985
+3  4719 4985 4986
+3  4719 4986 4720
+3  4720 4986 4721
+3  4721 4986 4987
+3  4721 4987 4722
+3  4722 4987 4988
+3  4722 4988 4723
+3  4723 4988 4989
+3  4723 4989 4724
+3  4724 4989 4990
+3  4724 4990 4725
+3  4725 4990 4763
+3  4763 4990 4764
+3  4764 4990 4991
+3  4764 4991 4765
+3  4765 4991 4766
+3  4766 4991 4992
+3  4766 4992 4767
+3  4767 4992 4993
+3  4767 4993 4994
+3  4767 4994 4768
+3  4768 4994 4995
+3  4768 4995 4769
+3  4769 4995 4996
+3  4769 4996 4997
+3  4769 4997 4770
+3  4770 4997 4998
+3  4770 4998 4999
+3  4770 4999 4771
+3  4771 4999 5000
+3  4771 5000 4773
+3  4773 5000 5001
+3  4773 5001 4774
+3  4774 5001 4775
+3  4775 5001 5002
+3  4775 5002 4776
+3  4776 5002 5003
+3  4776 5003 5004
+3  4776 5004 5005
+3  4776 5005 4777
+3  4777 5005 4778
+3  4778 5005 4779
+3  4779 5005 5006
+3  4779 5006 5007
+3  4779 5007 4780
+3  4780 5007 5008
+3  4780 5008 5009
+3  4780 5009 4781
+3  4781 5009 5010
+3  4781 5010 5011
+3  4781 5011 5012
+3  4781 5012 4782
+3  4782 5012 4783
+3  4783 5012 4784
+3  4784 5012 5013
+3  4784 5013 5014
+3  4784 5014 4785
+3  4785 5014 4786
+3  4786 5014 5015
+3  4786 5015 4787
+3  4787 5015 5016
+3  4787 5016 4788
+3  4788 5016 4789
+3  4789 5016 5017
+3  4789 5017 4790
+3  4790 5017 5018
+3  4790 5018 5019
+3  4790 5019 4791
+3  4791 5019 5020
+3  4791 5020 5021
+3  4791 5021 4792
+3  4792 5021 4793
+3  4793 5021 5022
+3  4793 5022 4794
+3  4794 5022 4795
+3  4795 5022 4796
+3  4796 5022 5023
+3  4796 5023 5024
+3  4796 5024 5025
+3  4796 5025 4797
+3  4797 5025 4798
+3  4798 5025 5026
+3  4798 5026 5027
+3  4798 5027 5028
+3  4798 5028 4799
+3  4799 5028 5029
+3  4799 5029 4800
+3  4800 5029 5030
+3  4800 5030 4801
+3  4801 5030 5031
+3  4801 5031 5032
+3  4801 5032 4802
+3  4802 5032 5033
+3  4802 5033 4803
+3  4803 5033 4804
+3  4804 5033 5034
+3  4804 5034 5035
+3  4804 5035 4805
+3  4805 5035 5036
+3  4805 5036 5037
+3  4805 5037 4806
+3  4806 5037 4807
+3  4807 5037 4812
+3  4812 5037 4813
+3  4813 5037 5036
+3  4813 5036 5038
+3  4813 5038 4814
+3  4814 5038 4815
+3  4815 5038 5039
+3  4815 5039 5040
+3  4815 5040 4816
+3  4816 5040 4817
+3  4817 5040 5041
+3  4817 5041 4818
+3  4818 5041 5042
+3  4818 5042 5043
+3  4818 5043 4819
+3  4819 5043 5044
+3  4819 5044 4820
+3  4820 5044 4821
+3  4821 5044 4822
+3  4822 5044 5045
+3  4822 5045 5046
+3  4822 5046 4823
+3  4823 5046 5047
+3  4823 5047 4824
+3  4824 5047 4825
+3  4825 5047 4826
+3  4826 5047 5048
+3  4826 5048 5049
+3  4826 5049 4827
+3  4827 5049 4828
+3  4828 5049 5050
+3  4828 5050 5051
+3  4828 5051 4829
+3  4829 5051 5052
+3  4829 5052 4830
+3  4830 5052 4831
+3  4831 5052 5053
+3  4831 5053 5054
+3  4831 5054 4832
+3  4832 5054 4833
+3  4833 5054 5055
+3  4833 5055 5056
+3  4833 5056 4834
+3  4834 5056 4835
+3  4835 5056 5057
+3  4835 5057 5058
+3  4835 5058 4836
+3  4836 5058 4837
+3  4837 5058 5059
+3  4837 5059 4838
+3  4838 5059 5060
+3  4838 5060 5061
+3  4838 5061 4839
+3  4839 5061 5062
+3  4839 5062 4840
+3  4840 5062 5063
+3  4840 5063 4841
+3  4841 5063 5064
+3  4841 5064 4842
+3  4842 5064 5065
+3  4842 5065 4843
+3  4843 5065 5066
+3  4843 5066 4844
+3  4844 5066 5067
+3  4844 5067 4845
+3  4845 5067 5068
+3  4845 5068 5069
+3  4845 5069 4846
+3  4846 5069 4847
+3  4847 5069 5070
+3  4847 5070 4848
+3  4848 5070 4849
+3  4849 5070 5071
+3  4849 5071 4850
+3  4850 5071 5072
+3  4850 5072 5073
+3  4850 5073 4851
+3  4851 5073 4852
+3  4852 5073 5074
+3  4852 5074 5075
+3  4852 5075 4853
+3  4853 5075 5076
+3  4853 5076 4854
+3  4854 5076 5077
+3  4854 5077 4856
+3  4856 5077 5078
+3  4856 5078 4857
+3  4857 5078 4858
+3  4858 5078 5079
+3  4858 5079 4859
+3  4859 5079 5080
+3  4859 5080 4860
+3  4860 5080 5081
+3  4860 5081 4861
+3  4861 5081 5082
+3  4861 5082 5083
+3  4861 5083 4862
+3  4862 5083 5084
+3  4862 5084 4863
+3  4863 5084 4864
+3  4864 5084 5085
+3  4864 5085 5086
+3  4864 5086 4865
+3  4865 5086 5087
+3  4865 5087 5088
+3  4865 5088 4866
+3  4866 5088 4867
+3  4867 5088 5089
+3  4867 5089 4868
+3  4868 5089 5090
+3  4868 5090 5091
+3  4868 5091 4869
+3  4869 5091 5092
+3  4869 5092 4870
+3  4870 5092 5093
+3  4870 5093 4871
+3  4871 5093 5094
+3  4871 5094 4872
+3  4872 5094 5095
+3  4872 5095 4873
+3  4873 5095 5096
+3  4873 5096 4874
+3  4874 5096 5097
+3  4874 5097 5098
+3  4874 5098 4875
+3  4875 5098 5099
+3  4875 5099 4876
+3  4876 5099 5100
+3  4876 5100 4877
+3  4877 5100 5101
+3  4877 5101 4878
+3  4878 5101 5102
+3  4878 5102 4879
+3  4879 5102 5103
+3  4879 5103 4880
+3  4880 5103 5104
+3  4880 5104 4881
+3  4881 5104 5105
+3  4881 5105 4882
+3  4882 5105 5106
+3  4882 5106 4883
+3  4883 5106 5107
+3  4883 5107 4884
+3  4884 5107 5108
+3  4884 5108 4885
+3  4885 5108 5109
+3  4885 5109 4886
+3  4886 5109 5110
+3  4886 5110 5111
+3  4886 5111 4887
+3  4887 5111 5112
+3  4887 5112 4888
+3  4888 5112 4889
+3  4889 5112 5113
+3  4889 5113 5114
+3  4889 5114 4890
+3  4890 5114 5115
+3  4890 5115 4891
+3  4891 5115 4892
+3  4892 5115 5116
+3  4892 5116 5117
+3  4892 5117 4893
+3  4893 5117 5118
+3  4893 5118 4894
+3  4894 5118 5119
+3  4894 5119 5120
+3  4894 5120 4895
+3  4895 5120 5121
+3  4895 5121 4896
+3  4896 5121 5122
+3  4896 5122 4897
+3  4897 5122 5123
+3  4897 5123 4898
+3  4898 5123 5124
+3  4898 5124 5125
+3  4898 5125 4899
+3  4899 5125 5126
+3  4899 5126 5127
+3  4899 5127 4900
+3  4900 5127 4901
+3  4901 5127 5128
+3  4901 5128 4902
+3  4902 5128 5129
+3  4902 5129 4903
+3  4903 5129 5130
+3  4903 5130 4905
+3  4905 5130 4906
+3  4906 5130 5131
+3  4906 5131 4907
+3  4907 5131 5132
+3  4907 5132 4908
+3  4908 5132 5133
+3  4908 5133 4909
+3  4909 5133 5134
+3  4909 5134 4910
+3  4910 5134 5135
+3  4910 5135 4911
+3  4911 5135 5136
+3  4911 5136 4912
+3  4912 5136 5137
+3  4912 5137 4913
+3  4913 5137 5138
+3  4913 5138 4914
+3  4914 5138 5139
+3  4914 5139 4915
+3  4915 5139 5140
+3  4915 5140 4916
+3  4916 5140 5141
+3  4916 5141 5142
+3  4916 5142 4917
+3  4917 5142 4918
+3  4918 5142 5143
+3  4918 5143 4919
+3  4919 5143 5144
+3  4919 5144 4920
+3  4920 5144 5145
+3  4920 5145 4921
+3  4921 5145 4922
+3  4922 5145 5146
+3  4922 5146 5147
+3  4922 5147 4923
+3  4923 5147 5148
+3  4923 5148 4924
+3  4924 5148 5149
+3  4924 5149 4925
+3  4925 5149 5150
+3  4925 5150 4926
+3  4926 5150 5151
+3  4926 5151 4927
+3  4927 5151 5152
+3  4927 5152 4928
+3  4928 5152 5153
+3  4928 5153 4929
+3  4929 5153 5154
+3  4929 5154 4930
+3  4930 5154 5155
+3  4930 5155 4931
+3  4931 5155 5156
+3  4931 5156 4932
+3  4932 5156 4933
+3  4933 5156 5157
+3  4933 5157 4934
+3  4934 5157 5158
+3  4934 5158 4935
+3  4935 5158 5159
+3  4935 5159 5160
+3  4935 5160 5161
+3  4935 5161 4936
+3  4936 5161 5162
+3  4936 5162 4937
+3  4937 5162 5163
+3  4937 5163 4938
+3  4938 5163 5164
+3  4938 5164 5165
+3  4938 5165 4939
+3  4939 5165 4940
+3  4940 5165 5166
+3  4940 5166 4941
+3  4941 5166 5167
+3  4941 5167 5168
+3  4941 5168 4942
+3  4942 5168 5169
+3  4942 5169 4943
+3  4943 5169 5170
+3  4943 5170 4944
+3  4944 5170 4945
+3  4945 5170 5171
+3  4945 5171 5172
+3  4945 5172 4946
+3  4946 5172 5173
+3  4946 5173 5174
+3  4946 5174 4947
+3  4947 5174 5175
+3  4947 5175 4948
+3  4948 5175 5176
+3  4948 5176 4949
+3  4949 5176 5177
+3  4949 5177 4950
+3  4950 5177 5178
+3  4950 5178 5179
+3  4950 5179 5180
+3  4950 5180 4951
+3  4951 5180 4952
+3  4952 5180 4953
+3  4953 5180 5181
+3  4953 5181 4954
+3  4954 5181 4955
+3  4955 5181 4956
+3  4956 5181 5182
+3  4956 5182 4957
+3  4957 5182 5183
+3  4957 5183 5184
+3  4957 5184 4958
+3  4958 5184 5185
+3  4958 5185 4959
+3  4959 5185 5186
+3  4959 5186 5187
+3  4959 5187 4960
+3  4960 5187 5188
+3  4960 5188 4961
+3  4961 5188 5189
+3  4961 5189 5190
+3  4961 5190 4962
+3  4962 5190 4963
+3  4963 5190 5191
+3  4963 5191 4964
+3  4964 5191 5192
+3  4964 5192 4965
+3  4965 5192 5193
+3  4965 5193 4966
+3  4966 5193 5194
+3  4966 5194 4967
+3  4967 5194 5195
+3  4967 5195 4968
+3  4968 5195 5196
+3  4968 5196 4969
+3  4969 5196 5197
+3  4969 5197 4970
+3  4970 5197 5198
+3  4970 5198 4971
+3  4971 5198 5199
+3  4971 5199 4972
+3  4972 5199 5200
+3  4972 5200 4973
+3  4973 5200 5201
+3  4973 5201 4974
+3  4974 5201 5202
+3  4974 5202 4975
+3  4975 5202 5203
+3  4975 5203 4976
+3  4976 5203 5204
+3  4976 5204 4977
+3  4977 5204 4978
+3  4978 5204 5205
+3  4978 5205 4979
+3  4979 5205 5206
+3  4979 5206 5207
+3  4979 5207 4980
+3  4980 5207 5208
+3  4980 5208 4981
+3  4981 5208 4982
+3  4982 5208 5209
+3  4982 5209 5210
+3  4982 5210 4983
+3  4983 5210 5211
+3  4983 5211 4984
+3  4984 5211 4996
+3  4996 5211 4997
+3  4997 5211 4998
+3  4998 5211 5212
+3  4998 5212 5213
+3  4998 5213 4999
+3  4999 5213 5001
+3  5001 5213 5002
+3  5002 5213 5214
+3  5002 5214 5003
+3  5003 5214 5215
+3  5003 5215 5216
+3  5003 5216 5004
+3  5004 5216 5207
+3  5207 5216 5217
+3  5207 5217 5208
+3  5208 5217 5209
+3  5209 5217 5218
+3  5209 5218 5210
+3  5210 5218 5212
+3  5212 5218 5214
+3  5214 5218 5215
+3  5215 5218 5217
+3  5215 5217 5216
+3  5212 5214 5213
+3  5210 5212 5211
+3  5004 5207 5206
+3  5004 5206 5006
+3  5006 5206 5205
+3  5006 5205 5219
+3  5006 5219 5007
+3  5007 5219 5008
+3  5008 5219 5220
+3  5008 5220 5221
+3  5008 5221 5009
+3  5009 5221 5010
+3  5010 5221 5222
+3  5010 5222 5223
+3  5010 5223 5224
+3  5010 5224 5011
+3  5011 5224 5012
+3  5012 5224 5225
+3  5012 5225 5013
+3  5013 5225 5226
+3  5013 5226 5227
+3  5013 5227 5228
+3  5013 5228 5014
+3  5014 5228 5015
+3  5015 5228 5229
+3  5015 5229 5016
+3  5016 5229 5017
+3  5017 5229 5018
+3  5018 5229 5230
+3  5018 5230 5231
+3  5018 5231 5019
+3  5019 5231 5232
+3  5019 5232 5020
+3  5020 5232 5233
+3  5020 5233 5023
+3  5023 5233 5234
+3  5023 5234 5024
+3  5024 5234 5235
+3  5024 5235 5025
+3  5025 5235 5026
+3  5026 5235 5236
+3  5026 5236 5237
+3  5026 5237 5027
+3  5027 5237 5238
+3  5027 5238 5239
+3  5027 5239 5028
+3  5028 5239 5240
+3  5028 5240 5029
+3  5029 5240 5241
+3  5029 5241 5242
+3  5029 5242 5030
+3  5030 5242 5031
+3  5031 5242 5243
+3  5031 5243 5032
+3  5032 5243 5244
+3  5032 5244 5033
+3  5033 5244 5245
+3  5033 5245 5034
+3  5034 5245 5246
+3  5034 5246 5247
+3  5034 5247 5035
+3  5035 5247 5248
+3  5035 5248 5036
+3  5036 5248 5038
+3  5038 5248 5039
+3  5039 5248 5249
+3  5039 5249 5250
+3  5039 5250 5040
+3  5040 5250 5041
+3  5041 5250 5251
+3  5041 5251 5042
+3  5042 5251 5252
+3  5042 5252 5253
+3  5042 5253 5043
+3  5043 5253 5254
+3  5043 5254 5044
+3  5044 5254 5045
+3  5045 5254 5255
+3  5045 5255 5256
+3  5045 5256 5046
+3  5046 5256 5257
+3  5046 5257 5048
+3  5048 5257 5258
+3  5048 5258 5259
+3  5048 5259 5049
+3  5049 5259 5260
+3  5049 5260 5050
+3  5050 5260 5261
+3  5050 5261 5262
+3  5050 5262 5051
+3  5051 5262 5263
+3  5051 5263 5052
+3  5052 5263 5053
+3  5053 5263 5264
+3  5053 5264 5265
+3  5053 5265 5054
+3  5054 5265 5055
+3  5055 5265 5266
+3  5055 5266 5267
+3  5055 5267 5056
+3  5056 5267 5057
+3  5057 5267 5268
+3  5057 5268 5269
+3  5057 5269 5058
+3  5058 5269 5059
+3  5059 5269 5270
+3  5059 5270 5060
+3  5060 5270 5271
+3  5060 5271 5272
+3  5060 5272 5061
+3  5061 5272 5273
+3  5061 5273 5062
+3  5062 5273 5274
+3  5062 5274 5063
+3  5063 5274 5275
+3  5063 5275 5064
+3  5064 5275 5276
+3  5064 5276 5065
+3  5065 5276 5277
+3  5065 5277 5066
+3  5066 5277 5278
+3  5066 5278 5067
+3  5067 5278 5279
+3  5067 5279 5068
+3  5068 5279 5280
+3  5068 5280 5281
+3  5068 5281 5069
+3  5069 5281 5070
+3  5070 5281 5071
+3  5071 5281 5282
+3  5071 5282 5072
+3  5072 5282 5283
+3  5072 5283 5073
+3  5073 5283 5074
+3  5074 5283 5284
+3  5074 5284 5075
+3  5075 5284 5285
+3  5075 5285 5076
+3  5076 5285 5286
+3  5076 5286 5077
+3  5077 5286 5287
+3  5077 5287 5078
+3  5078 5287 5288
+3  5078 5288 5079
+3  5079 5288 5080
+3  5080 5288 5289
+3  5080 5289 5081
+3  5081 5289 5290
+3  5081 5290 5082
+3  5082 5290 5291
+3  5082 5291 5292
+3  5082 5292 5083
+3  5083 5292 5085
+3  5085 5292 5293
+3  5085 5293 5086
+3  5086 5293 5294
+3  5086 5294 5087
+3  5087 5294 5295
+3  5087 5295 5296
+3  5087 5296 5088
+3  5088 5296 5089
+3  5089 5296 5297
+3  5089 5297 5090
+3  5090 5297 5298
+3  5090 5298 5299
+3  5090 5299 5091
+3  5091 5299 5300
+3  5091 5300 5092
+3  5092 5300 5301
+3  5092 5301 5093
+3  5093 5301 5302
+3  5093 5302 5094
+3  5094 5302 5303
+3  5094 5303 5095
+3  5095 5303 5304
+3  5095 5304 5096
+3  5096 5304 5305
+3  5096 5305 5097
+3  5097 5305 5306
+3  5097 5306 5307
+3  5097 5307 5098
+3  5098 5307 5308
+3  5098 5308 5099
+3  5099 5308 5309
+3  5099 5309 5100
+3  5100 5309 5310
+3  5100 5310 5101
+3  5101 5310 5311
+3  5101 5311 5102
+3  5102 5311 5312
+3  5102 5312 5103
+3  5103 5312 5313
+3  5103 5313 5104
+3  5104 5313 5314
+3  5104 5314 5105
+3  5105 5314 5315
+3  5105 5315 5106
+3  5106 5315 5316
+3  5106 5316 5107
+3  5107 5316 5317
+3  5107 5317 5108
+3  5108 5317 5318
+3  5108 5318 5109
+3  5109 5318 5319
+3  5109 5319 5110
+3  5110 5319 5320
+3  5110 5320 5321
+3  5110 5321 5111
+3  5111 5321 5322
+3  5111 5322 5112
+3  5112 5322 5113
+3  5113 5322 5323
+3  5113 5323 5324
+3  5113 5324 5114
+3  5114 5324 5325
+3  5114 5325 5115
+3  5115 5325 5116
+3  5116 5325 5326
+3  5116 5326 5327
+3  5116 5327 5117
+3  5117 5327 5118
+3  5118 5327 5328
+3  5118 5328 5119
+3  5119 5328 5329
+3  5119 5329 5330
+3  5119 5330 5120
+3  5120 5330 5331
+3  5120 5331 5121
+3  5121 5331 5332
+3  5121 5332 5122
+3  5122 5332 5333
+3  5122 5333 5123
+3  5123 5333 5334
+3  5123 5334 5124
+3  5124 5334 5335
+3  5124 5335 5336
+3  5124 5336 5125
+3  5125 5336 5337
+3  5125 5337 5126
+3  5126 5337 5338
+3  5126 5338 5339
+3  5126 5339 5127
+3  5127 5339 5128
+3  5128 5339 5340
+3  5128 5340 5129
+3  5129 5340 5341
+3  5129 5341 5130
+3  5130 5341 5131
+3  5131 5341 5342
+3  5131 5342 5132
+3  5132 5342 5343
+3  5132 5343 5133
+3  5133 5343 5344
+3  5133 5344 5134
+3  5134 5344 5345
+3  5134 5345 5135
+3  5135 5345 5346
+3  5135 5346 5136
+3  5136 5346 5347
+3  5136 5347 5137
+3  5137 5347 5348
+3  5137 5348 5138
+3  5138 5348 5349
+3  5138 5349 5139
+3  5139 5349 5350
+3  5139 5350 5140
+3  5140 5350 5351
+3  5140 5351 5141
+3  5141 5351 5352
+3  5141 5352 5353
+3  5141 5353 5142
+3  5142 5353 5354
+3  5142 5354 5143
+3  5143 5354 5355
+3  5143 5355 5144
+3  5144 5355 5356
+3  5144 5356 5145
+3  5145 5356 5146
+3  5146 5356 5357
+3  5146 5357 5358
+3  5146 5358 5147
+3  5147 5358 5359
+3  5147 5359 5148
+3  5148 5359 5360
+3  5148 5360 5149
+3  5149 5360 5361
+3  5149 5361 5150
+3  5150 5361 5362
+3  5150 5362 5151
+3  5151 5362 5363
+3  5151 5363 5152
+3  5152 5363 5364
+3  5152 5364 5153
+3  5153 5364 5154
+3  5154 5364 5365
+3  5154 5365 5366
+3  5154 5366 5155
+3  5155 5366 5367
+3  5155 5367 5156
+3  5156 5367 5368
+3  5156 5368 5157
+3  5157 5368 5158
+3  5158 5368 5369
+3  5158 5369 5159
+3  5159 5369 5370
+3  5159 5370 5371
+3  5159 5371 5160
+3  5160 5371 5372
+3  5160 5372 5373
+3  5160 5373 5161
+3  5161 5373 5374
+3  5161 5374 5162
+3  5162 5374 5375
+3  5162 5375 5163
+3  5163 5375 5376
+3  5163 5376 5164
+3  5164 5376 5377
+3  5164 5377 5378
+3  5164 5378 5165
+3  5165 5378 5166
+3  5166 5378 5379
+3  5166 5379 5167
+3  5167 5379 5380
+3  5167 5380 5381
+3  5167 5381 5168
+3  5168 5381 5382
+3  5168 5382 5169
+3  5169 5382 5383
+3  5169 5383 5170
+3  5170 5383 5171
+3  5171 5383 5384
+3  5171 5384 5385
+3  5171 5385 5172
+3  5172 5385 5386
+3  5172 5386 5173
+3  5173 5386 5387
+3  5173 5387 5388
+3  5173 5388 5174
+3  5174 5388 5389
+3  5174 5389 5175
+3  5175 5389 5390
+3  5175 5390 5176
+3  5176 5390 5391
+3  5176 5391 5177
+3  5177 5391 5392
+3  5177 5392 5178
+3  5178 5392 5393
+3  5178 5393 5183
+3  5183 5393 5394
+3  5183 5394 5184
+3  5184 5394 5185
+3  5185 5394 5395
+3  5185 5395 5396
+3  5185 5396 5186
+3  5186 5396 5397
+3  5186 5397 5187
+3  5187 5397 5398
+3  5187 5398 5188
+3  5188 5398 5399
+3  5188 5399 5189
+3  5189 5399 5400
+3  5189 5400 5401
+3  5189 5401 5190
+3  5190 5401 5191
+3  5191 5401 5402
+3  5191 5402 5192
+3  5192 5402 5403
+3  5192 5403 5193
+3  5193 5403 5404
+3  5193 5404 5194
+3  5194 5404 5405
+3  5194 5405 5195
+3  5195 5405 5406
+3  5195 5406 5196
+3  5196 5406 5407
+3  5196 5407 5197
+3  5197 5407 5408
+3  5197 5408 5198
+3  5198 5408 5409
+3  5198 5409 5199
+3  5199 5409 5410
+3  5199 5410 5200
+3  5200 5410 5411
+3  5200 5411 5201
+3  5201 5411 5222
+3  5222 5411 5223
+3  5223 5411 5412
+3  5223 5412 5224
+3  5224 5412 5413
+3  5224 5413 5414
+3  5224 5414 5225
+3  5225 5414 5226
+3  5226 5414 5415
+3  5226 5415 5227
+3  5227 5415 5416
+3  5227 5416 5417
+3  5227 5417 5228
+3  5228 5417 5230
+3  5230 5417 5418
+3  5230 5418 5419
+3  5230 5419 5231
+3  5231 5419 5420
+3  5231 5420 5421
+3  5231 5421 5422
+3  5231 5422 5232
+3  5232 5422 5423
+3  5232 5423 5233
+3  5233 5423 5424
+3  5233 5424 5234
+3  5234 5424 5425
+3  5234 5425 5235
+3  5235 5425 5236
+3  5236 5425 5426
+3  5236 5426 5427
+3  5236 5427 5237
+3  5237 5427 5428
+3  5237 5428 5238
+3  5238 5428 5429
+3  5238 5429 5430
+3  5238 5430 5239
+3  5239 5430 5431
+3  5239 5431 5240
+3  5240 5431 5432
+3  5240 5432 5433
+3  5240 5433 5241
+3  5241 5433 5242
+3  5242 5433 5243
+3  5243 5433 5244
+3  5244 5433 5434
+3  5244 5434 5245
+3  5245 5434 5435
+3  5245 5435 5246
+3  5246 5435 5436
+3  5246 5436 5437
+3  5246 5437 5247
+3  5247 5437 5249
+3  5249 5437 5438
+3  5249 5438 5250
+3  5250 5438 5251
+3  5251 5438 5439
+3  5251 5439 5252
+3  5252 5439 5440
+3  5252 5440 5441
+3  5252 5441 5253
+3  5253 5441 5442
+3  5253 5442 5254
+3  5254 5442 5255
+3  5255 5442 5443
+3  5255 5443 5444
+3  5255 5444 5256
+3  5256 5444 5445
+3  5256 5445 5257
+3  5257 5445 5258
+3  5258 5445 5446
+3  5258 5446 5447
+3  5258 5447 5448
+3  5258 5448 5259
+3  5259 5448 5260
+3  5260 5448 5449
+3  5260 5449 5450
+3  5260 5450 5261
+3  5261 5450 5451
+3  5261 5451 5452
+3  5261 5452 5262
+3  5262 5452 5453
+3  5262 5453 5263
+3  5263 5453 5264
+3  5264 5453 5454
+3  5264 5454 5455
+3  5264 5455 5265
+3  5265 5455 5266
+3  5266 5455 5456
+3  5266 5456 5457
+3  5266 5457 5267
+3  5267 5457 5268
+3  5268 5457 5458
+3  5268 5458 5459
+3  5268 5459 5269
+3  5269 5459 5270
+3  5270 5459 5460
+3  5270 5460 5271
+3  5271 5460 5461
+3  5271 5461 5462
+3  5271 5462 5272
+3  5272 5462 5463
+3  5272 5463 5273
+3  5273 5463 5464
+3  5273 5464 5274
+3  5274 5464 5465
+3  5274 5465 5275
+3  5275 5465 5466
+3  5275 5466 5276
+3  5276 5466 5467
+3  5276 5467 5277
+3  5277 5467 5468
+3  5277 5468 5278
+3  5278 5468 5469
+3  5278 5469 5279
+3  5279 5469 5470
+3  5279 5470 5280
+3  5280 5470 5471
+3  5280 5471 5282
+3  5282 5471 5283
+3  5283 5471 5284
+3  5284 5471 5472
+3  5284 5472 5285
+3  5285 5472 5473
+3  5285 5473 5286
+3  5286 5473 5474
+3  5286 5474 5287
+3  5287 5474 5475
+3  5287 5475 5288
+3  5288 5475 5476
+3  5288 5476 5289
+3  5289 5476 5477
+3  5289 5477 5290
+3  5290 5477 5478
+3  5290 5478 5291
+3  5291 5478 5479
+3  5291 5479 5292
+3  5292 5479 5293
+3  5293 5479 5480
+3  5293 5480 5294
+3  5294 5480 5481
+3  5294 5481 5295
+3  5295 5481 5482
+3  5295 5482 5483
+3  5295 5483 5296
+3  5296 5483 5297
+3  5297 5483 5484
+3  5297 5484 5298
+3  5298 5484 5485
+3  5298 5485 5486
+3  5298 5486 5299
+3  5299 5486 5487
+3  5299 5487 5300
+3  5300 5487 5488
+3  5300 5488 5301
+3  5301 5488 5489
+3  5301 5489 5302
+3  5302 5489 5490
+3  5302 5490 5303
+3  5303 5490 5491
+3  5303 5491 5304
+3  5304 5491 5492
+3  5304 5492 5305
+3  5305 5492 5493
+3  5305 5493 5306
+3  5306 5493 5494
+3  5306 5494 5495
+3  5306 5495 5307
+3  5307 5495 5496
+3  5307 5496 5308
+3  5308 5496 5497
+3  5308 5497 5309
+3  5309 5497 5498
+3  5309 5498 5310
+3  5310 5498 5499
+3  5310 5499 5311
+3  5311 5499 5500
+3  5311 5500 5312
+3  5312 5500 5501
+3  5312 5501 5313
+3  5313 5501 5502
+3  5313 5502 5314
+3  5314 5502 5503
+3  5314 5503 5315
+3  5315 5503 5504
+3  5315 5504 5316
+3  5316 5504 5505
+3  5316 5505 5317
+3  5317 5505 5506
+3  5317 5506 5318
+3  5318 5506 5507
+3  5318 5507 5319
+3  5319 5507 5508
+3  5319 5508 5320
+3  5320 5508 5509
+3  5320 5509 5510
+3  5320 5510 5321
+3  5321 5510 5511
+3  5321 5511 5322
+3  5322 5511 5323
+3  5323 5511 5512
+3  5323 5512 5513
+3  5323 5513 5324
+3  5324 5513 5514
+3  5324 5514 5325
+3  5325 5514 5326
+3  5326 5514 5515
+3  5326 5515 5516
+3  5326 5516 5327
+3  5327 5516 5328
+3  5328 5516 5329
+3  5329 5516 5517
+3  5329 5517 5518
+3  5329 5518 5330
+3  5330 5518 5519
+3  5330 5519 5331
+3  5331 5519 5520
+3  5331 5520 5332
+3  5332 5520 5333
+3  5333 5520 5521
+3  5333 5521 5334
+3  5334 5521 5522
+3  5334 5522 5335
+3  5335 5522 5523
+3  5335 5523 5524
+3  5335 5524 5336
+3  5336 5524 5337
+3  5337 5524 5525
+3  5337 5525 5526
+3  5337 5526 5338
+3  5338 5526 5527
+3  5338 5527 5528
+3  5338 5528 5339
+3  5339 5528 5340
+3  5340 5528 5529
+3  5340 5529 5341
+3  5341 5529 5342
+3  5342 5529 5530
+3  5342 5530 5343
+3  5343 5530 5531
+3  5343 5531 5344
+3  5344 5531 5532
+3  5344 5532 5345
+3  5345 5532 5533
+3  5345 5533 5346
+3  5346 5533 5534
+3  5346 5534 5347
+3  5347 5534 5535
+3  5347 5535 5348
+3  5348 5535 5536
+3  5348 5536 5349
+3  5349 5536 5537
+3  5349 5537 5350
+3  5350 5537 5538
+3  5350 5538 5351
+3  5351 5538 5539
+3  5351 5539 5352
+3  5352 5539 5540
+3  5352 5540 5541
+3  5352 5541 5353
+3  5353 5541 5542
+3  5353 5542 5354
+3  5354 5542 5543
+3  5354 5543 5355
+3  5355 5543 5544
+3  5355 5544 5356
+3  5356 5544 5357
+3  5357 5544 5545
+3  5357 5545 5546
+3  5357 5546 5358
+3  5358 5546 5547
+3  5358 5547 5359
+3  5359 5547 5548
+3  5359 5548 5360
+3  5360 5548 5549
+3  5360 5549 5361
+3  5361 5549 5550
+3  5361 5550 5362
+3  5362 5550 5551
+3  5362 5551 5363
+3  5363 5551 5552
+3  5363 5552 5364
+3  5364 5552 5553
+3  5364 5553 5365
+3  5365 5553 5554
+3  5365 5554 5366
+3  5366 5554 5555
+3  5366 5555 5367
+3  5367 5555 5368
+3  5368 5555 5369
+3  5369 5555 5556
+3  5369 5556 5370
+3  5370 5556 5557
+3  5370 5557 5371
+3  5371 5557 5558
+3  5371 5558 5372
+3  5372 5558 5559
+3  5372 5559 5560
+3  5372 5560 5373
+3  5373 5560 5561
+3  5373 5561 5374
+3  5374 5561 5562
+3  5374 5562 5375
+3  5375 5562 5563
+3  5375 5563 5376
+3  5376 5563 5564
+3  5376 5564 5377
+3  5377 5564 5565
+3  5377 5565 5566
+3  5377 5566 5378
+3  5378 5566 5379
+3  5379 5566 5567
+3  5379 5567 5380
+3  5380 5567 5568
+3  5380 5568 5569
+3  5380 5569 5381
+3  5381 5569 5570
+3  5381 5570 5382
+3  5382 5570 5571
+3  5382 5571 5383
+3  5383 5571 5384
+3  5384 5571 5572
+3  5384 5572 5573
+3  5384 5573 5385
+3  5385 5573 5574
+3  5385 5574 5386
+3  5386 5574 5575
+3  5386 5575 5387
+3  5387 5575 5576
+3  5387 5576 5577
+3  5387 5577 5388
+3  5388 5577 5578
+3  5388 5578 5389
+3  5389 5578 5579
+3  5389 5579 5390
+3  5390 5579 5580
+3  5390 5580 5391
+3  5391 5580 5581
+3  5391 5581 5392
+3  5392 5581 5429
+3  5429 5581 5582
+3  5429 5582 5430
+3  5430 5582 5431
+3  5431 5582 5583
+3  5431 5583 5584
+3  5431 5584 5432
+3  5432 5584 5585
+3  5432 5585 5434
+3  5434 5585 5435
+3  5435 5585 5586
+3  5435 5586 5436
+3  5436 5586 5587
+3  5436 5587 5588
+3  5436 5588 5437
+3  5437 5588 5438
+3  5438 5588 5439
+3  5439 5588 5589
+3  5439 5589 5440
+3  5440 5589 5590
+3  5440 5590 5591
+3  5440 5591 5441
+3  5441 5591 5592
+3  5441 5592 5442
+3  5442 5592 5443
+3  5443 5592 5593
+3  5443 5593 5594
+3  5443 5594 5444
+3  5444 5594 5595
+3  5444 5595 5445
+3  5445 5595 5446
+3  5446 5595 5447
+3  5447 5595 5596
+3  5447 5596 5597
+3  5447 5597 5598
+3  5447 5598 5448
+3  5448 5598 5449
+3  5449 5598 5599
+3  5449 5599 5600
+3  5449 5600 5450
+3  5450 5600 5451
+3  5451 5600 5601
+3  5451 5601 5602
+3  5451 5602 5452
+3  5452 5602 5603
+3  5452 5603 5453
+3  5453 5603 5454
+3  5454 5603 5604
+3  5454 5604 5605
+3  5454 5605 5455
+3  5455 5605 5456
+3  5456 5605 5606
+3  5456 5606 5607
+3  5456 5607 5457
+3  5457 5607 5458
+3  5458 5607 5608
+3  5458 5608 5609
+3  5458 5609 5459
+3  5459 5609 5460
+3  5460 5609 5610
+3  5460 5610 5461
+3  5461 5610 5611
+3  5461 5611 5612
+3  5461 5612 5462
+3  5462 5612 5613
+3  5462 5613 5463
+3  5463 5613 5614
+3  5463 5614 5464
+3  5464 5614 5615
+3  5464 5615 5465
+3  5465 5615 5616
+3  5465 5616 5466
+3  5466 5616 5617
+3  5466 5617 5467
+3  5467 5617 5618
+3  5467 5618 5468
+3  5468 5618 5619
+3  5468 5619 5469
+3  5469 5619 5620
+3  5469 5620 5470
+3  5470 5620 5472
+3  5472 5620 5473
+3  5473 5620 5621
+3  5473 5621 5474
+3  5474 5621 5622
+3  5474 5622 5475
+3  5475 5622 5623
+3  5475 5623 5476
+3  5476 5623 5624
+3  5476 5624 5477
+3  5477 5624 5625
+3  5477 5625 5478
+3  5478 5625 5626
+3  5478 5626 5479
+3  5479 5626 5480
+3  5480 5626 5627
+3  5480 5627 5481
+3  5481 5627 5628
+3  5481 5628 5482
+3  5482 5628 5629
+3  5482 5629 5630
+3  5482 5630 5483
+3  5483 5630 5484
+3  5484 5630 5631
+3  5484 5631 5485
+3  5485 5631 5632
+3  5485 5632 5633
+3  5485 5633 5486
+3  5486 5633 5634
+3  5486 5634 5487
+3  5487 5634 5635
+3  5487 5635 5488
+3  5488 5635 5636
+3  5488 5636 5489
+3  5489 5636 5637
+3  5489 5637 5490
+3  5490 5637 5638
+3  5490 5638 5491
+3  5491 5638 5639
+3  5491 5639 5492
+3  5492 5639 5640
+3  5492 5640 5493
+3  5493 5640 5641
+3  5493 5641 5494
+3  5494 5641 5642
+3  5494 5642 5643
+3  5494 5643 5495
+3  5495 5643 5644
+3  5495 5644 5496
+3  5496 5644 5645
+3  5496 5645 5497
+3  5497 5645 5646
+3  5497 5646 5498
+3  5498 5646 5647
+3  5498 5647 5499
+3  5499 5647 5648
+3  5499 5648 5500
+3  5500 5648 5649
+3  5500 5649 5501
+3  5501 5649 5650
+3  5501 5650 5502
+3  5502 5650 5651
+3  5502 5651 5503
+3  5503 5651 5652
+3  5503 5652 5504
+3  5504 5652 5653
+3  5504 5653 5505
+3  5505 5653 5654
+3  5505 5654 5506
+3  5506 5654 5655
+3  5506 5655 5507
+3  5507 5655 5656
+3  5507 5656 5508
+3  5508 5656 5657
+3  5508 5657 5509
+3  5509 5657 5658
+3  5509 5658 5659
+3  5509 5659 5510
+3  5510 5659 5660
+3  5510 5660 5511
+3  5511 5660 5512
+3  5512 5660 5661
+3  5512 5661 5662
+3  5512 5662 5513
+3  5513 5662 5663
+3  5513 5663 5514
+3  5514 5663 5515
+3  5515 5663 5664
+3  5515 5664 5517
+3  5517 5664 5665
+3  5517 5665 5518
+3  5518 5665 5666
+3  5518 5666 5519
+3  5519 5666 5667
+3  5519 5667 5520
+3  5520 5667 5668
+3  5520 5668 5521
+3  5521 5668 5669
+3  5521 5669 5522
+3  5522 5669 5523
+3  5523 5669 5670
+3  5523 5670 5671
+3  5523 5671 5524
+3  5524 5671 5525
+3  5525 5671 5672
+3  5525 5672 5526
+3  5526 5672 5673
+3  5526 5673 5527
+3  5527 5673 5674
+3  5527 5674 5675
+3  5527 5675 5528
+3  5528 5675 5529
+3  5529 5675 5530
+3  5530 5675 5676
+3  5530 5676 5531
+3  5531 5676 5677
+3  5531 5677 5532
+3  5532 5677 5678
+3  5532 5678 5533
+3  5533 5678 5679
+3  5533 5679 5534
+3  5534 5679 5680
+3  5534 5680 5535
+3  5535 5680 5681
+3  5535 5681 5682
+3  5535 5682 5536
+3  5536 5682 5683
+3  5536 5683 5537
+3  5537 5683 5684
+3  5537 5684 5538
+3  5538 5684 5685
+3  5538 5685 5539
+3  5539 5685 5686
+3  5539 5686 5540
+3  5540 5686 5687
+3  5540 5687 5688
+3  5540 5688 5541
+3  5541 5688 5689
+3  5541 5689 5542
+3  5542 5689 5543
+3  5543 5689 5690
+3  5543 5690 5544
+3  5544 5690 5545
+3  5545 5690 5691
+3  5545 5691 5692
+3  5545 5692 5546
+3  5546 5692 5693
+3  5546 5693 5547
+3  5547 5693 5694
+3  5547 5694 5548
+3  5548 5694 5695
+3  5548 5695 5549
+3  5549 5695 5696
+3  5549 5696 5550
+3  5550 5696 5697
+3  5550 5697 5551
+3  5551 5697 5698
+3  5551 5698 5552
+3  5552 5698 5699
+3  5552 5699 5553
+3  5553 5699 5700
+3  5553 5700 5554
+3  5554 5700 5556
+3  5556 5700 5557
+3  5557 5700 5701
+3  5557 5701 5558
+3  5558 5701 5702
+3  5558 5702 5559
+3  5559 5702 5703
+3  5559 5703 5704
+3  5559 5704 5560
+3  5560 5704 5705
+3  5560 5705 5561
+3  5561 5705 5706
+3  5561 5706 5562
+3  5562 5706 5707
+3  5562 5707 5563
+3  5563 5707 5708
+3  5563 5708 5564
+3  5564 5708 5709
+3  5564 5709 5565
+3  5565 5709 5710
+3  5565 5710 5711
+3  5565 5711 5566
+3  5566 5711 5567
+3  5567 5711 5712
+3  5567 5712 5568
+3  5568 5712 5713
+3  5568 5713 5714
+3  5568 5714 5569
+3  5569 5714 5715
+3  5569 5715 5570
+3  5570 5715 5716
+3  5570 5716 5571
+3  5571 5716 5572
+3  5572 5716 5717
+3  5572 5717 5718
+3  5572 5718 5573
+3  5573 5718 5719
+3  5573 5719 5574
+3  5574 5719 5720
+3  5574 5720 5575
+3  5575 5720 5721
+3  5575 5721 5576
+3  5576 5721 5722
+3  5576 5722 5577
+3  5577 5722 5723
+3  5577 5723 5724
+3  5577 5724 5578
+3  5578 5724 5725
+3  5578 5725 5579
+3  5579 5725 5726
+3  5579 5726 5580
+3  5580 5726 5583
+3  5583 5726 5584
+3  5584 5726 5727
+3  5584 5727 5585
+3  5585 5727 5586
+3  5586 5727 5728
+3  5586 5728 5587
+3  5587 5728 5729
+3  5587 5729 5589
+3  5589 5729 5590
+3  5590 5729 5730
+3  5590 5730 5591
+3  5591 5730 5731
+3  5591 5731 5592
+3  5592 5731 5593
+3  5593 5731 5732
+3  5593 5732 5733
+3  5593 5733 5594
+3  5594 5733 5734
+3  5594 5734 5595
+3  5595 5734 5596
+3  5596 5734 5735
+3  5596 5735 5597
+3  5597 5735 5736
+3  5597 5736 5737
+3  5597 5737 5738
+3  5597 5738 5598
+3  5598 5738 5739
+3  5598 5739 5599
+3  5599 5739 5740
+3  5599 5740 5741
+3  5599 5741 5742
+3  5599 5742 5600
+3  5600 5742 5601
+3  5601 5742 5743
+3  5601 5743 5744
+3  5601 5744 5602
+3  5602 5744 5745
+3  5602 5745 5603
+3  5603 5745 5604
+3  5604 5745 5746
+3  5604 5746 5747
+3  5604 5747 5605
+3  5605 5747 5606
+3  5606 5747 5748
+3  5606 5748 5749
+3  5606 5749 5607
+3  5607 5749 5608
+3  5608 5749 5750
+3  5608 5750 5751
+3  5608 5751 5609
+3  5609 5751 5610
+3  5610 5751 5752
+3  5610 5752 5611
+3  5611 5752 5753
+3  5611 5753 5754
+3  5611 5754 5612
+3  5612 5754 5755
+3  5612 5755 5613
+3  5613 5755 5756
+3  5613 5756 5614
+3  5614 5756 5757
+3  5614 5757 5615
+3  5615 5757 5758
+3  5615 5758 5616
+3  5616 5758 5759
+3  5616 5759 5617
+3  5617 5759 5760
+3  5617 5760 5618
+3  5618 5760 5761
+3  5618 5761 5619
+3  5619 5761 5621
+3  5621 5761 5622
+3  5622 5761 5762
+3  5622 5762 5623
+3  5623 5762 5763
+3  5623 5763 5624
+3  5624 5763 5764
+3  5624 5764 5625
+3  5625 5764 5765
+3  5625 5765 5626
+3  5626 5765 5627
+3  5627 5765 5766
+3  5627 5766 5628
+3  5628 5766 5767
+3  5628 5767 5629
+3  5629 5767 5768
+3  5629 5768 5769
+3  5629 5769 5630
+3  5630 5769 5631
+3  5631 5769 5770
+3  5631 5770 5632
+3  5632 5770 5771
+3  5632 5771 5772
+3  5632 5772 5633
+3  5633 5772 5773
+3  5633 5773 5634
+3  5634 5773 5774
+3  5634 5774 5635
+3  5635 5774 5775
+3  5635 5775 5636
+3  5636 5775 5776
+3  5636 5776 5637
+3  5637 5776 5777
+3  5637 5777 5638
+3  5638 5777 5778
+3  5638 5778 5639
+3  5639 5778 5779
+3  5639 5779 5640
+3  5640 5779 5780
+3  5640 5780 5641
+3  5641 5780 5781
+3  5641 5781 5642
+3  5642 5781 5782
+3  5642 5782 5783
+3  5642 5783 5643
+3  5643 5783 5784
+3  5643 5784 5644
+3  5644 5784 5785
+3  5644 5785 5645
+3  5645 5785 5786
+3  5645 5786 5646
+3  5646 5786 5787
+3  5646 5787 5647
+3  5647 5787 5788
+3  5647 5788 5648
+3  5648 5788 5789
+3  5648 5789 5649
+3  5649 5789 5790
+3  5649 5790 5650
+3  5650 5790 5791
+3  5650 5791 5651
+3  5651 5791 5792
+3  5651 5792 5652
+3  5652 5792 5793
+3  5652 5793 5653
+3  5653 5793 5794
+3  5653 5794 5654
+3  5654 5794 5795
+3  5654 5795 5655
+3  5655 5795 5796
+3  5655 5796 5656
+3  5656 5796 5797
+3  5656 5797 5657
+3  5657 5797 5798
+3  5657 5798 5658
+3  5658 5798 5799
+3  5658 5799 5800
+3  5658 5800 5659
+3  5659 5800 5801
+3  5659 5801 5660
+3  5660 5801 5661
+3  5661 5801 5802
+3  5661 5802 5803
+3  5661 5803 5662
+3  5662 5803 5804
+3  5662 5804 5663
+3  5663 5804 5664
+3  5664 5804 5805
+3  5664 5805 5665
+3  5665 5805 5806
+3  5665 5806 5666
+3  5666 5806 5807
+3  5666 5807 5667
+3  5667 5807 5808
+3  5667 5808 5668
+3  5668 5808 5809
+3  5668 5809 5669
+3  5669 5809 5810
+3  5669 5810 5670
+3  5670 5810 5811
+3  5670 5811 5812
+3  5670 5812 5671
+3  5671 5812 5672
+3  5672 5812 5813
+3  5672 5813 5673
+3  5673 5813 5814
+3  5673 5814 5674
+3  5674 5814 5815
+3  5674 5815 5816
+3  5674 5816 5675
+3  5675 5816 5676
+3  5676 5816 5815
+3  5676 5815 5677
+3  5677 5815 5817
+3  5677 5817 5678
+3  5678 5817 5818
+3  5678 5818 5679
+3  5679 5818 5819
+3  5679 5819 5680
+3  5680 5819 5820
+3  5680 5820 5681
+3  5681 5820 5821
+3  5681 5821 5822
+3  5681 5822 5682
+3  5682 5822 5683
+3  5683 5822 5823
+3  5683 5823 5684
+3  5684 5823 5824
+3  5684 5824 5825
+3  5684 5825 5685
+3  5685 5825 5826
+3  5685 5826 5827
+3  5685 5827 5686
+3  5686 5827 5828
+3  5686 5828 5687
+3  5687 5828 5829
+3  5687 5829 5830
+3  5687 5830 5688
+3  5688 5830 5831
+3  5688 5831 5689
+3  5689 5831 5690
+3  5690 5831 5691
+3  5691 5831 5832
+3  5691 5832 5833
+3  5691 5833 5692
+3  5692 5833 5834
+3  5692 5834 5693
+3  5693 5834 5835
+3  5693 5835 5694
+3  5694 5835 5836
+3  5694 5836 5695
+3  5695 5836 5837
+3  5695 5837 5696
+3  5696 5837 5838
+3  5696 5838 5697
+3  5697 5838 5839
+3  5697 5839 5698
+3  5698 5839 5840
+3  5698 5840 5699
+3  5699 5840 5701
+3  5701 5840 5702
+3  5702 5840 5841
+3  5702 5841 5703
+3  5703 5841 5842
+3  5703 5842 5843
+3  5703 5843 5704
+3  5704 5843 5844
+3  5704 5844 5705
+3  5705 5844 5845
+3  5705 5845 5706
+3  5706 5845 5846
+3  5706 5846 5707
+3  5707 5846 5847
+3  5707 5847 5708
+3  5708 5847 5848
+3  5708 5848 5709
+3  5709 5848 5849
+3  5709 5849 5710
+3  5710 5849 5850
+3  5710 5850 5851
+3  5710 5851 5711
+3  5711 5851 5712
+3  5712 5851 5852
+3  5712 5852 5713
+3  5713 5852 5853
+3  5713 5853 5854
+3  5713 5854 5714
+3  5714 5854 5855
+3  5714 5855 5715
+3  5715 5855 5856
+3  5715 5856 5716
+3  5716 5856 5717
+3  5717 5856 5857
+3  5717 5857 5858
+3  5717 5858 5718
+3  5718 5858 5859
+3  5718 5859 5719
+3  5719 5859 5860
+3  5719 5860 5720
+3  5720 5860 5861
+3  5720 5861 5721
+3  5721 5861 5862
+3  5721 5862 5722
+3  5722 5862 5863
+3  5722 5863 5864
+3  5722 5864 5723
+3  5723 5864 5865
+3  5723 5865 5724
+3  5724 5865 5866
+3  5724 5866 5867
+3  5724 5867 5725
+3  5725 5867 5868
+3  5725 5868 5726
+3  5726 5868 5727
+3  5727 5868 5728
+3  5728 5868 5869
+3  5728 5869 5870
+3  5728 5870 5729
+3  5729 5870 5730
+3  5730 5870 5871
+3  5730 5871 5872
+3  5730 5872 5732
+3  5732 5872 5873
+3  5732 5873 5874
+3  5732 5874 5733
+3  5733 5874 5875
+3  5733 5875 5734
+3  5734 5875 5735
+3  5735 5875 5876
+3  5735 5876 5736
+3  5736 5876 5877
+3  5736 5877 5878
+3  5736 5878 5737
+3  5737 5878 5879
+3  5737 5879 5738
+3  5738 5879 5880
+3  5738 5880 5881
+3  5738 5881 5739
+3  5739 5881 5882
+3  5739 5882 5740
+3  5740 5882 5883
+3  5740 5883 5884
+3  5740 5884 5741
+3  5741 5884 5885
+3  5741 5885 5886
+3  5741 5886 5742
+3  5742 5886 5743
+3  5743 5886 5887
+3  5743 5887 5888
+3  5743 5888 5744
+3  5744 5888 5889
+3  5744 5889 5745
+3  5745 5889 5746
+3  5746 5889 5890
+3  5746 5890 5891
+3  5746 5891 5747
+3  5747 5891 5748
+3  5748 5891 5892
+3  5748 5892 5893
+3  5748 5893 5749
+3  5749 5893 5750
+3  5750 5893 5894
+3  5750 5894 5895
+3  5750 5895 5751
+3  5751 5895 5752
+3  5752 5895 5896
+3  5752 5896 5753
+3  5753 5896 5897
+3  5753 5897 5898
+3  5753 5898 5754
+3  5754 5898 5899
+3  5754 5899 5755
+3  5755 5899 5900
+3  5755 5900 5756
+3  5756 5900 5901
+3  5756 5901 5757
+3  5757 5901 5902
+3  5757 5902 5758
+3  5758 5902 5903
+3  5758 5903 5759
+3  5759 5903 5904
+3  5759 5904 5760
+3  5760 5904 5762
+3  5762 5904 5763
+3  5763 5904 5905
+3  5763 5905 5764
+3  5764 5905 5906
+3  5764 5906 5765
+3  5765 5906 5766
+3  5766 5906 5907
+3  5766 5907 5767
+3  5767 5907 5908
+3  5767 5908 5768
+3  5768 5908 5909
+3  5768 5909 5910
+3  5768 5910 5769
+3  5769 5910 5770
+3  5770 5910 5911
+3  5770 5911 5771
+3  5771 5911 5912
+3  5771 5912 5913
+3  5771 5913 5772
+3  5772 5913 5914
+3  5772 5914 5773
+3  5773 5914 5915
+3  5773 5915 5774
+3  5774 5915 5916
+3  5774 5916 5775
+3  5775 5916 5917
+3  5775 5917 5776
+3  5776 5917 5918
+3  5776 5918 5777
+3  5777 5918 5919
+3  5777 5919 5778
+3  5778 5919 5920
+3  5778 5920 5779
+3  5779 5920 5921
+3  5779 5921 5780
+3  5780 5921 5922
+3  5780 5922 5781
+3  5781 5922 5923
+3  5781 5923 5782
+3  5782 5923 5924
+3  5782 5924 5925
+3  5782 5925 5783
+3  5783 5925 5926
+3  5783 5926 5784
+3  5784 5926 5927
+3  5784 5927 5785
+3  5785 5927 5928
+3  5785 5928 5786
+3  5786 5928 5929
+3  5786 5929 5787
+3  5787 5929 5930
+3  5787 5930 5788
+3  5788 5930 5931
+3  5788 5931 5789
+3  5789 5931 5932
+3  5789 5932 5790
+3  5790 5932 5933
+3  5790 5933 5791
+3  5791 5933 5934
+3  5791 5934 5792
+3  5792 5934 5935
+3  5792 5935 5793
+3  5793 5935 5936
+3  5793 5936 5794
+3  5794 5936 5937
+3  5794 5937 5795
+3  5795 5937 5938
+3  5795 5938 5796
+3  5796 5938 5939
+3  5796 5939 5797
+3  5797 5939 5940
+3  5797 5940 5798
+3  5798 5940 5941
+3  5798 5941 5799
+3  5799 5941 5942
+3  5799 5942 5943
+3  5799 5943 5800
+3  5800 5943 5944
+3  5800 5944 5801
+3  5801 5944 5802
+3  5802 5944 5945
+3  5802 5945 5946
+3  5802 5946 5803
+3  5803 5946 5947
+3  5803 5947 5804
+3  5804 5947 5805
+3  5805 5947 5948
+3  5805 5948 5806
+3  5806 5948 5949
+3  5806 5949 5807
+3  5807 5949 5950
+3  5807 5950 5808
+3  5808 5950 5809
+3  5809 5950 5951
+3  5809 5951 5952
+3  5809 5952 5810
+3  5810 5952 5953
+3  5810 5953 5811
+3  5811 5953 5954
+3  5811 5954 5813
+3  5813 5954 5814
+3  5814 5954 5817
+3  5817 5954 5818
+3  5818 5954 5953
+3  5818 5953 5819
+3  5819 5953 5952
+3  5819 5952 5820
+3  5820 5952 5955
+3  5820 5955 5821
+3  5821 5955 5956
+3  5821 5956 5957
+3  5821 5957 5822
+3  5822 5957 5823
+3  5823 5957 5958
+3  5823 5958 5824
+3  5824 5958 5959
+3  5824 5959 5960
+3  5824 5960 5825
+3  5825 5960 5826
+3  5826 5960 5961
+3  5826 5961 5962
+3  5826 5962 5827
+3  5827 5962 5828
+3  5828 5962 5963
+3  5828 5963 5829
+3  5829 5963 5964
+3  5829 5964 5965
+3  5829 5965 5830
+3  5830 5965 5832
+3  5832 5965 5966
+3  5832 5966 5967
+3  5832 5967 5833
+3  5833 5967 5968
+3  5833 5968 5834
+3  5834 5968 5969
+3  5834 5969 5835
+3  5835 5969 5970
+3  5835 5970 5836
+3  5836 5970 5971
+3  5836 5971 5837
+3  5837 5971 5972
+3  5837 5972 5838
+3  5838 5972 5973
+3  5838 5973 5839
+3  5839 5973 5841
+3  5841 5973 5842
+3  5842 5973 5974
+3  5842 5974 5975
+3  5842 5975 5843
+3  5843 5975 5976
+3  5843 5976 5844
+3  5844 5976 5977
+3  5844 5977 5845
+3  5845 5977 5978
+3  5845 5978 5846
+3  5846 5978 5979
+3  5846 5979 5847
+3  5847 5979 5980
+3  5847 5980 5848
+3  5848 5980 5981
+3  5848 5981 5849
+3  5849 5981 5982
+3  5849 5982 5850
+3  5850 5982 5983
+3  5850 5983 5984
+3  5850 5984 5851
+3  5851 5984 5852
+3  5852 5984 5985
+3  5852 5985 5853
+3  5853 5985 5986
+3  5853 5986 5987
+3  5853 5987 5854
+3  5854 5987 5988
+3  5854 5988 5855
+3  5855 5988 5989
+3  5855 5989 5856
+3  5856 5989 5857
+3  5857 5989 5990
+3  5857 5990 5991
+3  5857 5991 5858
+3  5858 5991 5992
+3  5858 5992 5859
+3  5859 5992 5993
+3  5859 5993 5860
+3  5860 5993 5994
+3  5860 5994 5861
+3  5861 5994 5995
+3  5861 5995 5862
+3  5862 5995 5996
+3  5862 5996 5863
+3  5863 5996 5997
+3  5863 5997 5998
+3  5863 5998 5999
+3  5863 5999 5864
+3  5864 5999 5877
+3  5877 5999 5878
+3  5878 5999 6000
+3  5878 6000 5879
+3  5879 6000 6001
+3  5879 6001 5880
+3  5880 6001 6002
+3  5880 6002 5881
+3  5881 6002 6003
+3  5881 6003 6004
+3  5881 6004 5882
+3  5882 6004 6005
+3  5882 6005 5883
+3  5883 6005 6006
+3  5883 6006 6007
+3  5883 6007 5884
+3  5884 6007 6008
+3  5884 6008 5885
+3  5885 6008 6009
+3  5885 6009 6010
+3  5885 6010 5886
+3  5886 6010 5887
+3  5887 6010 6011
+3  5887 6011 6012
+3  5887 6012 5888
+3  5888 6012 6013
+3  5888 6013 5889
+3  5889 6013 5890
+3  5890 6013 6014
+3  5890 6014 6015
+3  5890 6015 5891
+3  5891 6015 5892
+3  5892 6015 6016
+3  5892 6016 6017
+3  5892 6017 5893
+3  5893 6017 5894
+3  5894 6017 6018
+3  5894 6018 6019
+3  5894 6019 5895
+3  5895 6019 5896
+3  5896 6019 6020
+3  5896 6020 5897
+3  5897 6020 6021
+3  5897 6021 6022
+3  5897 6022 5898
+3  5898 6022 6023
+3  5898 6023 5899
+3  5899 6023 6024
+3  5899 6024 5900
+3  5900 6024 6025
+3  5900 6025 5901
+3  5901 6025 6026
+3  5901 6026 5902
+3  5902 6026 6027
+3  5902 6027 5903
+3  5903 6027 5905
+3  5905 6027 5906
+3  5906 6027 5907
+3  5907 6027 6026
+3  5907 6026 5908
+3  5908 6026 6025
+3  5908 6025 5909
+3  5909 6025 6024
+3  5909 6024 6028
+3  5909 6028 5910
+3  5910 6028 5911
+3  5911 6028 6029
+3  5911 6029 5912
+3  5912 6029 6030
+3  5912 6030 6031
+3  5912 6031 5913
+3  5913 6031 6032
+3  5913 6032 5914
+3  5914 6032 6033
+3  5914 6033 5915
+3  5915 6033 6034
+3  5915 6034 5916
+3  5916 6034 6035
+3  5916 6035 5917
+3  5917 6035 6036
+3  5917 6036 5918
+3  5918 6036 6037
+3  5918 6037 5919
+3  5919 6037 6038
+3  5919 6038 5920
+3  5920 6038 6039
+3  5920 6039 5921
+3  5921 6039 6040
+3  5921 6040 5922
+3  5922 6040 6041
+3  5922 6041 5923
+3  5923 6041 6042
+3  5923 6042 5924
+3  5924 6042 6043
+3  5924 6043 6044
+3  5924 6044 5925
+3  5925 6044 6045
+3  5925 6045 5926
+3  5926 6045 6046
+3  5926 6046 5927
+3  5927 6046 6047
+3  5927 6047 5928
+3  5928 6047 6048
+3  5928 6048 5929
+3  5929 6048 6049
+3  5929 6049 5930
+3  5930 6049 6050
+3  5930 6050 5931
+3  5931 6050 6051
+3  5931 6051 5932
+3  5932 6051 6052
+3  5932 6052 5933
+3  5933 6052 6053
+3  5933 6053 5934
+3  5934 6053 6054
+3  5934 6054 5935
+3  5935 6054 6055
+3  5935 6055 5936
+3  5936 6055 6056
+3  5936 6056 5937
+3  5937 6056 6057
+3  5937 6057 5938
+3  5938 6057 6058
+3  5938 6058 5939
+3  5939 6058 5967
+3  5967 6058 5968
+3  5968 6058 6059
+3  5968 6059 5969
+3  5969 6059 6060
+3  5969 6060 5970
+3  5970 6060 6061
+3  5970 6061 5971
+3  5971 6061 6062
+3  5971 6062 5972
+3  5972 6062 5974
+3  5974 6062 6063
+3  5974 6063 5975
+3  5975 6063 6064
+3  5975 6064 5976
+3  5976 6064 6065
+3  5976 6065 5977
+3  5977 6065 6066
+3  5977 6066 5978
+3  5978 6066 6067
+3  5978 6067 5979
+3  5979 6067 6068
+3  5979 6068 5980
+3  5980 6068 6069
+3  5980 6069 5981
+3  5981 6069 6070
+3  5981 6070 5982
+3  5982 6070 6071
+3  5982 6071 5983
+3  5983 6071 6072
+3  5983 6072 6073
+3  5983 6073 5984
+3  5984 6073 5985
+3  5985 6073 6074
+3  5985 6074 5986
+3  5986 6074 6075
+3  5986 6075 6076
+3  5986 6076 5987
+3  5987 6076 6077
+3  5987 6077 5988
+3  5988 6077 6078
+3  5988 6078 5989
+3  5989 6078 5990
+3  5990 6078 6079
+3  5990 6079 6080
+3  5990 6080 5991
+3  5991 6080 6081
+3  5991 6081 5992
+3  5992 6081 6082
+3  5992 6082 5993
+3  5993 6082 6083
+3  5993 6083 5994
+3  5994 6083 6084
+3  5994 6084 5995
+3  5995 6084 6085
+3  5995 6085 5996
+3  5996 6085 5997
+3  5997 6085 6086
+3  5997 6086 6087
+3  5997 6087 6088
+3  5997 6088 5998
+3  5998 6088 6089
+3  5998 6089 5999
+3  5999 6089 6000
+3  6000 6089 6001
+3  6001 6089 6090
+3  6001 6090 6002
+3  6002 6090 6091
+3  6002 6091 6003
+3  6003 6091 6092
+3  6003 6092 6093
+3  6003 6093 6004
+3  6004 6093 6094
+3  6004 6094 6005
+3  6005 6094 6095
+3  6005 6095 6006
+3  6006 6095 6096
+3  6006 6096 6097
+3  6006 6097 6098
+3  6006 6098 6007
+3  6007 6098 6099
+3  6007 6099 6008
+3  6008 6099 6009
+3  6009 6099 6100
+3  6009 6100 6101
+3  6009 6101 6010
+3  6010 6101 6011
+3  6011 6101 6102
+3  6011 6102 6103
+3  6011 6103 6012
+3  6012 6103 6104
+3  6012 6104 6013
+3  6013 6104 6014
+3  6014 6104 6105
+3  6014 6105 6106
+3  6014 6106 6015
+3  6015 6106 6016
+3  6016 6106 6107
+3  6016 6107 6108
+3  6016 6108 6017
+3  6017 6108 6018
+3  6018 6108 6109
+3  6018 6109 6110
+3  6018 6110 6019
+3  6019 6110 6020
+3  6020 6110 6111
+3  6020 6111 6021
+3  6021 6111 6112
+3  6021 6112 6030
+3  6030 6112 6031
+3  6031 6112 6113
+3  6031 6113 6032
+3  6032 6113 6114
+3  6032 6114 6033
+3  6033 6114 6115
+3  6033 6115 6034
+3  6034 6115 6116
+3  6034 6116 6035
+3  6035 6116 6117
+3  6035 6117 6036
+3  6036 6117 6118
+3  6036 6118 6037
+3  6037 6118 6119
+3  6037 6119 6038
+3  6038 6119 6120
+3  6038 6120 6039
+3  6039 6120 6121
+3  6039 6121 6040
+3  6040 6121 6122
+3  6040 6122 6041
+3  6041 6122 6123
+3  6041 6123 6042
+3  6042 6123 6124
+3  6042 6124 6043
+3  6043 6124 6125
+3  6043 6125 6126
+3  6043 6126 6044
+3  6044 6126 6127
+3  6044 6127 6045
+3  6045 6127 6128
+3  6045 6128 6046
+3  6046 6128 6129
+3  6046 6129 6047
+3  6047 6129 6130
+3  6047 6130 6048
+3  6048 6130 6131
+3  6048 6131 6049
+3  6049 6131 6132
+3  6049 6132 6050
+3  6050 6132 6133
+3  6050 6133 6051
+3  6051 6133 6134
+3  6051 6134 6052
+3  6052 6134 6135
+3  6052 6135 6053
+3  6053 6135 6136
+3  6053 6136 6054
+3  6054 6136 6137
+3  6054 6137 6055
+3  6055 6137 6138
+3  6055 6138 6056
+3  6056 6138 6139
+3  6056 6139 6057
+3  6057 6139 6059
+3  6059 6139 6060
+3  6060 6139 6140
+3  6060 6140 6061
+3  6061 6140 6141
+3  6061 6141 6062
+3  6062 6141 6063
+3  6063 6141 6142
+3  6063 6142 6064
+3  6064 6142 6143
+3  6064 6143 6065
+3  6065 6143 6144
+3  6065 6144 6066
+3  6066 6144 6145
+3  6066 6145 6067
+3  6067 6145 6146
+3  6067 6146 6068
+3  6068 6146 6147
+3  6068 6147 6069
+3  6069 6147 6148
+3  6069 6148 6070
+3  6070 6148 6149
+3  6070 6149 6071
+3  6071 6149 6150
+3  6071 6150 6072
+3  6072 6150 6151
+3  6072 6151 6152
+3  6072 6152 6073
+3  6073 6152 6074
+3  6074 6152 6153
+3  6074 6153 6075
+3  6075 6153 6154
+3  6075 6154 6155
+3  6075 6155 6076
+3  6076 6155 6156
+3  6076 6156 6077
+3  6077 6156 6157
+3  6077 6157 6078
+3  6078 6157 6079
+3  6079 6157 6158
+3  6079 6158 6159
+3  6079 6159 6080
+3  6080 6159 6160
+3  6080 6160 6081
+3  6081 6160 6161
+3  6081 6161 6082
+3  6082 6161 6162
+3  6082 6162 6083
+3  6083 6162 6163
+3  6083 6163 6084
+3  6084 6163 6164
+3  6084 6164 6085
+3  6085 6164 6086
+3  6086 6164 6165
+3  6086 6165 6166
+3  6086 6166 6087
+3  6087 6166 6167
+3  6087 6167 6168
+3  6087 6168 6088
+3  6088 6168 6091
+3  6091 6168 6169
+3  6091 6169 6092
+3  6092 6169 6093
+3  6093 6169 6170
+3  6093 6170 6094
+3  6094 6170 6171
+3  6094 6171 6095
+3  6095 6171 6172
+3  6095 6172 6096
+3  6096 6172 6173
+3  6096 6173 6174
+3  6096 6174 6097
+3  6097 6174 6175
+3  6097 6175 6098
+3  6098 6175 6176
+3  6098 6176 6099
+3  6099 6176 6100
+3  6100 6176 6177
+3  6100 6177 6101
+3  6101 6177 6102
+3  6102 6177 6178
+3  6102 6178 6179
+3  6102 6179 6103
+3  6103 6179 6180
+3  6103 6180 6104
+3  6104 6180 6105
+3  6105 6180 6181
+3  6105 6181 6182
+3  6105 6182 6106
+3  6106 6182 6107
+3  6107 6182 6183
+3  6107 6183 6184
+3  6107 6184 6108
+3  6108 6184 6109
+3  6109 6184 6185
+3  6109 6185 6186
+3  6109 6186 6110
+3  6110 6186 6111
+3  6111 6186 6187
+3  6111 6187 6112
+3  6112 6187 6113
+3  6113 6187 6188
+3  6113 6188 6114
+3  6114 6188 6189
+3  6114 6189 6115
+3  6115 6189 6190
+3  6115 6190 6116
+3  6116 6190 6191
+3  6116 6191 6117
+3  6117 6191 6192
+3  6117 6192 6118
+3  6118 6192 6193
+3  6118 6193 6119
+3  6119 6193 6194
+3  6119 6194 6120
+3  6120 6194 6195
+3  6120 6195 6121
+3  6121 6195 6196
+3  6121 6196 6122
+3  6122 6196 6197
+3  6122 6197 6123
+3  6123 6197 6198
+3  6123 6198 6124
+3  6124 6198 6199
+3  6124 6199 6125
+3  6125 6199 6200
+3  6125 6200 6201
+3  6125 6201 6126
+3  6126 6201 6202
+3  6126 6202 6127
+3  6127 6202 6203
+3  6127 6203 6128
+3  6128 6203 6204
+3  6128 6204 6129
+3  6129 6204 6205
+3  6129 6205 6130
+3  6130 6205 6206
+3  6130 6206 6131
+3  6131 6206 6207
+3  6131 6207 6132
+3  6132 6207 6208
+3  6132 6208 6133
+3  6133 6208 6209
+3  6133 6209 6134
+3  6134 6209 6210
+3  6134 6210 6135
+3  6135 6210 6211
+3  6135 6211 6136
+3  6136 6211 6212
+3  6136 6212 6137
+3  6137 6212 6213
+3  6137 6213 6138
+3  6138 6213 6140
+3  6140 6213 6141
+3  6141 6213 6142
+3  6142 6213 6212
+3  6142 6212 6143
+3  6143 6212 6211
+3  6143 6211 6144
+3  6144 6211 6210
+3  6144 6210 6145
+3  6145 6210 6209
+3  6145 6209 6146
+3  6146 6209 6208
+3  6146 6208 6147
+3  6147 6208 6207
+3  6147 6207 6148
+3  6148 6207 6206
+3  6148 6206 6149
+3  6149 6206 6205
+3  6149 6205 6150
+3  6150 6205 6204
+3  6150 6204 6151
+3  6151 6204 6203
+3  6151 6203 6214
+3  6151 6214 6152
+3  6152 6214 6153
+3  6153 6214 6215
+3  6153 6215 6154
+3  6154 6215 6216
+3  6154 6216 6217
+3  6154 6217 6155
+3  6155 6217 6218
+3  6155 6218 6156
+3  6156 6218 6219
+3  6156 6219 6157
+3  6157 6219 6158
+3  6158 6219 6220
+3  6158 6220 6221
+3  6158 6221 6159
+3  6159 6221 6222
+3  6159 6222 6160
+3  6160 6222 6223
+3  6160 6223 6161
+3  6161 6223 6224
+3  6161 6224 6162
+3  6162 6224 6225
+3  6162 6225 6163
+3  6163 6225 6226
+3  6163 6226 6164
+3  6164 6226 6165
+3  6165 6226 6227
+3  6165 6227 6228
+3  6165 6228 6166
+3  6166 6228 6229
+3  6166 6229 6167
+3  6167 6229 6230
+3  6167 6230 6231
+3  6167 6231 6232
+3  6167 6232 6168
+3  6168 6232 6169
+3  6169 6232 6233
+3  6169 6233 6170
+3  6170 6233 6234
+3  6170 6234 6171
+3  6171 6234 6235
+3  6171 6235 6172
+3  6172 6235 6236
+3  6172 6236 6237
+3  6172 6237 6173
+3  6173 6237 6238
+3  6173 6238 6174
+3  6174 6238 6239
+3  6174 6239 6240
+3  6174 6240 6175
+3  6175 6240 6241
+3  6175 6241 6176
+3  6176 6241 6177
+3  6177 6241 6178
+3  6178 6241 6242
+3  6178 6242 6243
+3  6178 6243 6179
+3  6179 6243 6244
+3  6179 6244 6180
+3  6180 6244 6181
+3  6181 6244 6245
+3  6181 6245 6246
+3  6181 6246 6182
+3  6182 6246 6183
+3  6183 6246 6247
+3  6183 6247 6248
+3  6183 6248 6184
+3  6184 6248 6185
+3  6185 6248 6249
+3  6185 6249 6250
+3  6185 6250 6186
+3  6186 6250 6187
+3  6187 6250 6188
+3  6188 6250 6251
+3  6188 6251 6189
+3  6189 6251 6252
+3  6189 6252 6190
+3  6190 6252 6253
+3  6190 6253 6191
+3  6191 6253 6254
+3  6191 6254 6192
+3  6192 6254 6255
+3  6192 6255 6193
+3  6193 6255 6256
+3  6193 6256 6194
+3  6194 6256 6257
+3  6194 6257 6195
+3  6195 6257 6258
+3  6195 6258 6196
+3  6196 6258 6259
+3  6196 6259 6197
+3  6197 6259 6260
+3  6197 6260 6198
+3  6198 6260 6261
+3  6198 6261 6199
+3  6199 6261 6262
+3  6199 6262 6200
+3  6200 6262 6263
+3  6200 6263 6216
+3  6216 6263 6217
+3  6217 6263 6264
+3  6217 6264 6218
+3  6218 6264 6265
+3  6218 6265 6219
+3  6219 6265 6220
+3  6220 6265 6266
+3  6220 6266 6267
+3  6220 6267 6221
+3  6221 6267 6268
+3  6221 6268 6222
+3  6222 6268 6269
+3  6222 6269 6223
+3  6223 6269 6270
+3  6223 6270 6224
+3  6224 6270 6271
+3  6224 6271 6225
+3  6225 6271 6272
+3  6225 6272 6226
+3  6226 6272 6227
+3  6227 6272 6273
+3  6227 6273 6274
+3  6227 6274 6275
+3  6227 6275 6228
+3  6228 6275 6276
+3  6228 6276 6229
+3  6229 6276 6230
+3  6230 6276 6277
+3  6230 6277 6278
+3  6230 6278 6279
+3  6230 6279 6231
+3  6231 6279 6232
+3  6232 6279 6280
+3  6232 6280 6233
+3  6233 6280 6234
+3  6234 6280 6235
+3  6235 6280 6281
+3  6235 6281 6236
+3  6236 6281 6282
+3  6236 6282 6237
+3  6237 6282 6283
+3  6237 6283 6284
+3  6237 6284 6238
+3  6238 6284 6285
+3  6238 6285 6239
+3  6239 6285 6286
+3  6239 6286 6287
+3  6239 6287 6240
+3  6240 6287 6242
+3  6242 6287 6288
+3  6242 6288 6243
+3  6243 6288 6289
+3  6243 6289 6244
+3  6244 6289 6245
+3  6245 6289 6290
+3  6245 6290 6291
+3  6245 6291 6246
+3  6246 6291 6247
+3  6247 6291 6292
+3  6247 6292 6293
+3  6247 6293 6248
+3  6248 6293 6249
+3  6249 6293 6294
+3  6249 6294 6251
+3  6251 6294 6252
+3  6252 6294 6295
+3  6252 6295 6253
+3  6253 6295 6296
+3  6253 6296 6254
+3  6254 6296 6297
+3  6254 6297 6255
+3  6255 6297 6298
+3  6255 6298 6256
+3  6256 6298 6299
+3  6256 6299 6257
+3  6257 6299 6300
+3  6257 6300 6258
+3  6258 6300 6301
+3  6258 6301 6259
+3  6259 6301 6302
+3  6259 6302 6260
+3  6260 6302 6303
+3  6260 6303 6261
+3  6261 6303 6304
+3  6261 6304 6262
+3  6262 6304 6305
+3  6262 6305 6263
+3  6263 6305 6264
+3  6264 6305 6306
+3  6264 6306 6265
+3  6265 6306 6266
+3  6266 6306 6307
+3  6266 6307 6308
+3  6266 6308 6267
+3  6267 6308 6309
+3  6267 6309 6268
+3  6268 6309 6310
+3  6268 6310 6269
+3  6269 6310 6311
+3  6269 6311 6270
+3  6270 6311 6312
+3  6270 6312 6271
+3  6271 6312 6272
+3  6272 6312 6273
+3  6273 6312 6313
+3  6273 6313 6274
+3  6274 6313 6314
+3  6274 6314 6315
+3  6274 6315 6275
+3  6275 6315 6316
+3  6275 6316 6276
+3  6276 6316 6277
+3  6277 6316 6317
+3  6277 6317 6318
+3  6277 6318 6278
+3  6278 6318 6319
+3  6278 6319 6320
+3  6278 6320 6279
+3  6279 6320 6281
+3  6281 6320 6282
+3  6282 6320 6321
+3  6282 6321 6283
+3  6283 6321 6322
+3  6283 6322 6323
+3  6283 6323 6284
+3  6284 6323 6324
+3  6284 6324 6285
+3  6285 6324 6325
+3  6285 6325 6286
+3  6286 6325 6326
+3  6286 6326 6327
+3  6286 6327 6287
+3  6287 6327 6288
+3  6288 6327 6328
+3  6288 6328 6289
+3  6289 6328 6290
+3  6290 6328 6329
+3  6290 6329 6330
+3  6290 6330 6291
+3  6291 6330 6292
+3  6292 6330 6331
+3  6292 6331 6332
+3  6292 6332 6293
+3  6293 6332 6294
+3  6294 6332 6295
+3  6295 6332 6333
+3  6295 6333 6296
+3  6296 6333 6334
+3  6296 6334 6297
+3  6297 6334 6335
+3  6297 6335 6298
+3  6298 6335 6336
+3  6298 6336 6299
+3  6299 6336 6337
+3  6299 6337 6300
+3  6300 6337 6338
+3  6300 6338 6301
+3  6301 6338 6339
+3  6301 6339 6302
+3  6302 6339 6340
+3  6302 6340 6303
+3  6303 6340 6341
+3  6303 6341 6304
+3  6304 6341 6342
+3  6304 6342 6305
+3  6305 6342 6306
+3  6306 6342 6307
+3  6307 6342 6343
+3  6307 6343 6344
+3  6307 6344 6308
+3  6308 6344 6345
+3  6308 6345 6309
+3  6309 6345 6346
+3  6309 6346 6310
+3  6310 6346 6347
+3  6310 6347 6311
+3  6311 6347 6313
+3  6313 6347 6314
+3  6314 6347 6348
+3  6314 6348 6349
+3  6314 6349 6315
+3  6315 6349 6350
+3  6315 6350 6316
+3  6316 6350 6317
+3  6317 6350 6351
+3  6317 6351 6352
+3  6317 6352 6318
+3  6318 6352 6353
+3  6318 6353 6319
+3  6319 6353 6354
+3  6319 6354 6321
+3  6321 6354 6322
+3  6322 6354 6355
+3  6322 6355 6356
+3  6322 6356 6323
+3  6323 6356 6357
+3  6323 6357 6324
+3  6324 6357 6358
+3  6324 6358 6325
+3  6325 6358 6359
+3  6325 6359 6326
+3  6326 6359 6360
+3  6326 6360 6361
+3  6326 6361 6327
+3  6327 6361 6328
+3  6328 6361 6329
+3  6329 6361 6362
+3  6329 6362 6363
+3  6329 6363 6330
+3  6330 6363 6331
+3  6331 6363 6364
+3  6331 6364 6333
+3  6333 6364 6334
+3  6334 6364 6365
+3  6334 6365 6335
+3  6335 6365 6366
+3  6335 6366 6336
+3  6336 6366 6367
+3  6336 6367 6337
+3  6337 6367 6368
+3  6337 6368 6338
+3  6338 6368 6369
+3  6338 6369 6339
+3  6339 6369 6370
+3  6339 6370 6340
+3  6340 6370 6371
+3  6340 6371 6341
+3  6341 6371 6343
+3  6343 6371 6372
+3  6343 6372 6344
+3  6344 6372 6373
+3  6344 6373 6345
+3  6345 6373 6374
+3  6345 6374 6346
+3  6346 6374 6348
+3  6348 6374 6375
+3  6348 6375 6349
+3  6349 6375 6376
+3  6349 6376 6350
+3  6350 6376 6351
+3  6351 6376 6377
+3  6351 6377 6378
+3  6351 6378 6379
+3  6351 6379 6352
+3  6352 6379 6353
+3  6353 6379 6380
+3  6353 6380 6381
+3  6353 6381 6354
+3  6354 6381 6355
+3  6355 6381 6382
+3  6355 6382 6383
+3  6355 6383 6356
+3  6356 6383 6384
+3  6356 6384 6357
+3  6357 6384 6385
+3  6357 6385 6358
+3  6358 6385 6386
+3  6358 6386 6359
+3  6359 6386 6387
+3  6359 6387 6360
+3  6360 6387 6388
+3  6360 6388 6362
+3  6362 6388 6389
+3  6362 6389 6363
+3  6363 6389 6364
+3  6364 6389 6365
+3  6365 6389 6390
+3  6365 6390 6366
+3  6366 6390 6391
+3  6366 6391 6367
+3  6367 6391 6392
+3  6367 6392 6368
+3  6368 6392 6393
+3  6368 6393 6369
+3  6369 6393 6394
+3  6369 6394 6370
+3  6370 6394 6395
+3  6370 6395 6371
+3  6371 6395 6372
+3  6372 6395 6396
+3  6372 6396 6373
+3  6373 6396 6397
+3  6373 6397 6374
+3  6374 6397 6375
+3  6375 6397 6398
+3  6375 6398 6376
+3  6376 6398 6399
+3  6376 6399 6377
+3  6377 6399 6400
+3  6377 6400 6401
+3  6377 6401 6378
+3  6378 6401 6380
+3  6380 6401 6402
+3  6380 6402 6381
+3  6381 6402 6382
+3  6382 6402 6403
+3  6382 6403 6383
+3  6383 6403 6384
+3  6384 6403 6404
+3  6384 6404 6405
+3  6384 6405 6385
+3  6385 6405 6406
+3  6385 6406 6386
+3  6386 6406 6407
+3  6386 6407 6387
+3  6387 6407 6408
+3  6387 6408 6388
+3  6388 6408 6390
+3  6390 6408 6391
+3  6391 6408 6409
+3  6391 6409 6392
+3  6392 6409 6410
+3  6392 6410 6393
+3  6393 6410 6411
+3  6393 6411 6394
+3  6394 6411 6412
+3  6394 6412 6395
+3  6395 6412 6396
+3  6396 6412 6413
+3  6396 6413 6397
+3  6397 6413 6398
+3  6398 6413 6414
+3  6398 6414 6399
+3  6399 6414 6415
+3  6399 6415 6400
+3  6400 6415 6416
+3  6400 6416 6401
+3  6401 6416 6417
+3  6401 6417 6402
+3  6402 6417 6403
+3  6403 6417 6404
+3  6404 6417 6416
+3  6404 6416 6418
+3  6404 6418 6405
+3  6405 6418 6419
+3  6405 6419 6406
+3  6406 6419 6420
+3  6406 6420 6407
+3  6407 6420 6409
+3  6409 6420 6410
+3  6410 6420 6421
+3  6410 6421 6411
+3  6411 6421 6422
+3  6411 6422 6412
+3  6412 6422 6413
+3  6413 6422 6414
+3  6414 6422 6423
+3  6414 6423 6415
+3  6415 6423 6418
+3  6418 6423 6419
+3  6419 6423 6421
+3  6421 6423 6422
+3  6419 6421 6420
+3  6415 6418 6416
+3  6407 6409 6408
+3  6388 6390 6389
+3  6378 6380 6379
+3  6360 6362 6361
+3  6346 6348 6347
+3  6341 6343 6342
+3  6331 6333 6332
+3  6319 6321 6320
+3  6311 6313 6312
+3  6279 6281 6280
+3  6249 6251 6250
+3  6240 6242 6241
+3  6200 6216 6201
+3  6201 6216 6215
+3  6201 6215 6202
+3  6202 6215 6214
+3  6202 6214 6203
+3  6138 6140 6139
+3  6088 6091 6090
+3  6088 6090 6089
+3  6057 6059 6058
+3  6021 6030 6022
+3  6022 6030 6029
+3  6022 6029 6023
+3  6023 6029 6028
+3  6023 6028 6024
+3  5972 5974 5973
+3  5939 5967 5966
+3  5939 5966 5940
+3  5940 5966 5964
+3  5964 5966 5965
+3  5940 5964 5941
+3  5941 5964 5963
+3  5941 5963 5942
+3  5942 5963 5962
+3  5942 5962 5961
+3  5942 5961 5943
+3  5943 5961 6424
+3  5943 6424 5944
+3  5944 6424 5945
+3  5945 6424 5959
+3  5959 6424 5960
+3  5960 6424 5961
+3  5945 5959 6425
+3  5945 6425 5946
+3  5946 6425 6426
+3  5946 6426 5947
+3  5947 6426 5948
+3  5948 6426 6427
+3  5948 6427 5949
+3  5949 6427 6428
+3  5949 6428 5950
+3  5950 6428 5951
+3  5951 6428 5955
+3  5955 6428 5956
+3  5956 6428 6427
+3  5956 6427 6429
+3  5956 6429 5957
+3  5957 6429 5958
+3  5958 6429 6425
+3  6425 6429 6426
+3  6426 6429 6427
+3  5958 6425 5959
+3  5951 5955 5952
+3  5903 5905 5904
+3  5864 5877 6430
+3  5864 6430 5865
+3  5865 6430 6431
+3  5865 6431 6432
+3  5865 6432 6433
+3  5865 6433 5866
+3  5866 6433 6434
+3  5866 6434 5867
+3  5867 6434 5869
+3  5869 6434 5870
+3  5870 6434 5871
+3  5871 6434 6435
+3  5871 6435 5872
+3  5872 6435 5873
+3  5873 6435 6432
+3  6432 6435 6433
+3  6433 6435 6434
+3  5873 6432 5874
+3  5874 6432 6431
+3  5874 6431 5875
+3  5875 6431 5876
+3  5876 6431 6430
+3  5876 6430 5877
+3  5867 5869 5868
+3  5839 5841 5840
+3  5830 5832 5831
+3  5814 5817 5815
+3  5811 5813 5812
+3  5760 5762 5761
+3  5730 5732 5731
+3  5699 5701 5700
+3  5619 5621 5620
+3  5587 5589 5588
+3  5580 5583 5582
+3  5580 5582 5581
+3  5554 5556 5555
+3  5515 5517 5516
+3  5470 5472 5471
+3  5432 5434 5433
+3  5392 5429 5393
+3  5393 5429 5394
+3  5394 5429 5428
+3  5394 5428 5395
+3  5395 5428 5427
+3  5395 5427 5426
+3  5395 5426 5396
+3  5396 5426 5425
+3  5396 5425 5397
+3  5397 5425 5424
+3  5397 5424 5398
+3  5398 5424 5423
+3  5398 5423 5399
+3  5399 5423 5422
+3  5399 5422 5400
+3  5400 5422 6436
+3  5400 6436 5401
+3  5401 6436 5402
+3  5402 6436 6437
+3  5402 6437 5403
+3  5403 6437 6438
+3  5403 6438 5404
+3  5404 6438 6439
+3  5404 6439 5405
+3  5405 6439 5416
+3  5416 6439 5417
+3  5417 6439 5418
+3  5418 6439 6440
+3  5418 6440 5419
+3  5419 6440 5420
+3  5420 6440 6437
+3  6437 6440 6438
+3  6438 6440 6439
+3  5420 6437 5421
+3  5421 6437 6436
+3  5421 6436 5422
+3  5405 5416 5406
+3  5406 5416 5415
+3  5406 5415 5407
+3  5407 5415 5414
+3  5407 5414 5408
+3  5408 5414 5413
+3  5408 5413 5409
+3  5409 5413 5412
+3  5409 5412 5410
+3  5410 5412 5411
+3  5280 5282 5281
+3  5247 5249 5248
+3  5228 5230 5229
+3  5201 5222 5202
+3  5202 5222 5221
+3  5202 5221 5203
+3  5203 5221 5220
+3  5203 5220 5204
+3  5204 5220 5219
+3  5204 5219 5205
+3  5178 5183 5182
+3  5178 5182 5179
+3  5179 5182 5181
+3  5179 5181 5180
+3  5083 5085 5084
+3  5046 5048 5047
+3  5020 5023 5021
+3  5021 5023 5022
+3  5004 5006 5005
+3  4999 5001 5000
+3  4984 4996 4985
+3  4985 4996 4995
+3  4985 4995 4994
+3  4985 4994 4987
+3  4987 4994 4993
+3  4987 4993 4988
+3  4988 4993 4989
+3  4989 4993 4992
+3  4989 4992 4991
+3  4989 4991 4990
+3  4985 4987 4986
+3  4903 4905 4904
+3  4854 4856 4855
+3  4807 4812 4811
+3  4807 4811 4808
+3  4808 4811 4809
+3  4771 4773 4772
+3  4725 4763 4762
+3  4725 4762 4726
+3  4726 4762 4761
+3  4726 4761 4727
+3  4727 4761 4728
+3  4728 4761 4760
+3  4728 4760 6441
+3  4728 6441 4729
+3  4729 6441 6442
+3  4729 6442 4730
+3  4730 6442 4731
+3  4731 6442 6443
+3  4731 6443 6444
+3  4731 6444 4732
+3  4732 6444 6445
+3  4732 6445 4733
+3  4733 6445 6446
+3  4733 6446 4734
+3  4734 6446 6447
+3  4734 6447 6448
+3  4734 6448 4735
+3  4735 6448 4736
+3  4736 6448 4737
+3  4737 6448 6449
+3  4737 6449 6450
+3  4737 6450 4738
+3  4738 6450 4739
+3  4739 6450 6451
+3  4739 6451 4740
+3  4740 6451 6452
+3  4740 6452 6453
+3  4740 6453 4741
+3  4741 6453 4743
+3  4743 6453 6454
+3  4743 6454 6455
+3  4743 6455 4744
+3  4744 6455 4746
+3  4746 6455 6456
+3  4746 6456 4747
+3  4747 6456 4748
+3  4748 6456 6457
+3  4748 6457 6458
+3  4748 6458 4749
+3  4749 6458 6459
+3  4749 6459 4750
+3  4750 6459 4751
+3  4751 6459 6460
+3  4751 6460 6461
+3  4751 6461 4752
+3  4752 6461 4753
+3  4753 6461 6462
+3  4753 6462 6463
+3  4753 6463 4754
+3  4754 6463 4755
+3  4755 6463 6464
+3  4755 6464 4756
+3  4756 6464 6444
+3  6444 6464 6445
+3  6445 6464 6465
+3  6445 6465 6446
+3  6446 6465 6466
+3  6446 6466 6447
+3  6447 6466 6467
+3  6447 6467 6448
+3  6448 6467 6449
+3  6449 6467 6468
+3  6449 6468 6469
+3  6449 6469 6450
+3  6450 6469 6451
+3  6451 6469 6470
+3  6451 6470 6452
+3  6452 6470 6457
+3  6457 6470 6458
+3  6458 6470 6471
+3  6458 6471 6459
+3  6459 6471 6460
+3  6460 6471 6468
+3  6468 6471 6469
+3  6469 6471 6470
+3  6460 6468 6472
+3  6460 6472 6461
+3  6461 6472 6462
+3  6462 6472 6466
+3  6466 6472 6467
+3  6467 6472 6468
+3  6462 6466 6465
+3  6462 6465 6463
+3  6463 6465 6464
+3  6452 6457 6473
+3  6452 6473 6453
+3  6453 6473 6454
+3  6454 6473 6455
+3  6455 6473 6456
+3  6456 6473 6457
+3  4756 6444 6443
+3  4756 6443 4757
+3  4757 6443 4758
+3  4758 6443 6442
+3  4758 6442 6441
+3  4758 6441 4759
+3  4759 6441 4760
+3  4744 4746 4745
+3  4741 4743 4742
+3  4638 4640 4639
+3  4590 4592 4591
+3  4568 4570 4569
+3  4552 4554 4553
+3  4425 4429 4426
+3  4426 4429 4428
+3  4426 4428 4427
+3  4396 4398 4397
+3  4348 4350 4349
+3  4260 4262 4261
+3  4198 4200 4199
+3  4192 4194 4193
+3  4081 4083 4082
+3  4063 4065 4064
+3  3982 3984 3983
+3  3977 3979 3978
+3  3966 3968 3967
+3  3951 3953 3952
+3  3860 3864 3862
+3  3860 3862 3861
+3  3858 3860 3859
+3  3856 3858 3857
+3  3845 3847 3846
+3  3840 3842 3841
+3  3800 3802 3801
+3  3732 3734 3733
+3  3715 3717 3716
+3  3691 3693 3692
+3  3679 3681 3680
+3  3649 3651 3650
+3  3647 3649 3648
+3  3609 3611 3610
+3  3605 3607 3606
+3  3601 3603 3602
+3  3539 3541 3540
+3  3505 3507 3506
+3  3445 3447 3446
+3  3443 3445 3444
+3  3325 3327 3326
+3  3238 3240 3239
+3  3089 3091 3090
+3  3045 3047 3046
+3  2862 2864 2863
+3  2827 2829 2828
+3  2772 2774 2773
+3  2768 2770 2769
+3  2761 2763 2762
+3  2728 2730 2729
+3  2654 2656 2655
+3  2594 2596 2595
+3  2551 2554 2553
+3  2551 2553 2552
+3  2549 2551 2550
+3  2537 2540 2538
+3  2453 2455 2454
+3  2416 2418 2417
+3  2377 2379 2378
+3  2362 2364 2363
+3  2324 2326 2325
+3  2318 2320 2319
+3  2296 2299 2297
+3  2282 2284 2283
+3  2222 2224 2223
+3  2193 2195 2194
+3  2184 2186 2185
+3  2145 2147 2146
+3  2094 2096 2095
+3  1966 1975 1967
+3  1967 1975 1974
+3  1967 1974 6474
+3  1967 6474 1970
+3  1970 6474 1971
+3  1971 6474 1973
+3  1973 6474 1974
+3  1971 1973 1972
+3  1967 1970 1968
+3  1968 1970 1969
+3  1942 1944 1943
+3  1889 1891 1890
+3  1797 1799 1798
+3  1774 1776 1775
+3  1720 1722 1721
+3  1675 1677 1676
+3  1626 1628 1627
+3  1606 1608 1607
+3  1601 1603 1602
+3  1544 1546 1545
+3  1519 1521 1520
+3  1403 1405 1404
+3  1367 1369 1368
+3  1292 1294 1293
+3  1286 1288 1287
+3  1264 1266 1265
+3  1258 1260 1259
+3  1252 1254 1253
+3  1125 1127 1126
+3  1104 1106 1105
+3  996 998 997
+3  984 986 985
+3  971 973 972
+3  883 885 884
+3  853 855 854
+3  666 668 667
+3  652 654 653
+3  578 580 579
+3  560 562 561
+3  472 474 473
+3  393 395 394
+3  203 205 204
+3  72 74 73
+
diff -pruN 5.0.3-1/examples/Tetrahedral_remeshing/data/sphere.mesh 5.1-1/examples/Tetrahedral_remeshing/data/sphere.mesh
--- 5.0.3-1/examples/Tetrahedral_remeshing/data/sphere.mesh	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Tetrahedral_remeshing/data/sphere.mesh	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,5046 @@
+MeshVersionFormatted 1
+Dimension 3
+Vertices
+757
+-0.00017958008027845786 -0.29158438056560654 -0.3988195263311396 2
+0.33746273152274708 -0.35726277852053662 0.076905488424117915 2
+0.10190320305717479 -0.44537090218944148 0.19023485796529088 2
+-0.22291708884561526 0.24110089782764016 0.3698544290906931 2
+-0.015169005603949674 -0.4920502072817815 -0.053433473242467985 2
+-0.26605755950148613 0.34871795230918606 0.23040729539756138 2
+-0.030951975074221216 0.48125453898639275 -0.1043111632434163 2
+0.059997821745160003 -0.35306497158682226 0.34146470435235443 2
+-0.47842728941771906 -0.11383138831461681 -0.085210684090395519 2
+0.36358525568361316 0.21273180556147436 0.25617337548594726 2
+0.36725408516443148 0.13052845625068488 -0.30523087317317177 2
+-0.22325756200134647 0.14972356892330102 -0.41730303115077449 2
+-0.33402647635257132 -0.1744766427005035 0.31869283771003265 2
+0.093487696703693335 0.02119707361997069 0.48380505033599003 2
+-0.41062731925574825 0.25760949214328016 -0.10699221840973314 2
+0.49505762706510581 -0.028269043401154839 0.005588588618755724 2
+0.34186587730419771 -0.23615312879880351 -0.27171510437341651 2
+0.30887568698642764 0.38011138416438828 -0.055425037144192238 2
+0.081954497410480692 0.43221656346370257 0.23174479064232939 2
+0.080762763732273479 0.30645943468988468 -0.37895869522206083 2
+0.35704674384671609 -0.12142305171818887 0.32188881275214021 2
+-0.31466635484011551 -0.3761124021911264 0.049178927462616873 2
+-0.29949621897948708 -0.16366746550400973 -0.3559360399848972 2
+0.12477546186735991 -0.0009654011072460668 -0.48017447662099233 2
+-0.45489993093049341 0.089486639415368968 0.16934989437353176 2
+-0.21949385855586256 -0.38887603838234641 -0.21249580489705544 2
+-0.17554447746608282 -0.008967119829543662 0.46417052578394857 2
+-0.21596925005035716 0.36305807543520158 -0.25659442404768845 2
+0.21818581779788443 -0.42407361285115364 -0.14318551682223507 2
+0.029500307224903276 0.26282332358153948 0.4193318651156448 2
+-0.41140983700838274 0.055777199004947785 -0.26777152182208963 2
+-0.18318546651083045 -0.37360295143388589 0.26762502445563596 2
+-0.25403346126706278 0.42468917850563082 -0.0072330644157137559 2
+-0.11305983472243486 -0.060902778843192146 -0.47777514755888634 2
+0.44919653003104693 0.20594839867448833 0.031940926454485326 2
+-0.11066913845628298 -0.2262153965428356 0.42803075198822749 2
+-0.43545677914012793 -0.19909033271144083 0.11894257742861167 2
+0.1695993626242438 -0.17935343970708448 0.42607203736875154 2
+0.44458432799815872 -0.051677795976616092 -0.20786699465654979 2
+0.43234224599164228 -0.22486127018578264 -0.071274965547427127 2
+0.26639842732560048 0.32085642363039735 -0.26255407845880468 2
+0.26294920120495924 -0.31175710164257442 0.27584799753054995 2
+0.28532208912927193 0.068229649454114999 0.39636186284206176 2
+-0.34227612327536827 0.063789233931985129 0.35025252357992426 2
+0.13671557462523365 0.47582166858216723 0.029136645926506381 2
+0.30614427994747062 -0.058158535566455949 -0.38363836413890928 2
+0.27804667170978731 0.37888988049526307 0.15488894457208352 2
+-0.079983560189783809 0.47755425170528942 0.10219500715280494 2
+0.44814787431466785 0.027095935407346859 0.20340275687843323 2
+0.43643009646155334 -0.18171607702779971 0.13679244425007148 2
+-0.021047213889092184 0.14820686384301263 -0.46961700869255452 2
+0.03619253272475921 -0.42721740050619039 -0.24476932875897767 2
+-0.37955898467105353 -0.29024182965842682 -0.12706737614326807 2
+-0.40060756364504235 0.27265441281304659 0.096625520813105659 2
+-0.4854705637493073 0.083104466088603279 -0.032289989270033626 2
+0.18733664061540145 -0.21944745392631665 -0.39902475350724975 2
+0.20220985983165063 0.30550901002185649 0.32856225983545018 2
+-0.14260311796152336 -0.46277003604768613 0.095417085459266915 2
+-0.088536490739146129 0.37455894762711039 0.3096149430548682 2
+0.2094668074720448 0.16368187491205238 -0.41638280720002807 2
+0.15113456598618952 0.44235088220955404 -0.16250801345043558 2
+0.40486835137050703 0.24289072279009177 -0.15252766171211285 2
+-0.036177835783256007 0.4078011138562273 -0.27535546612298556 2
+-0.34296523108150667 0.2286131340909362 -0.27799669728182952 2
+-0.10719098758575357 0.28623761047718843 -0.38960924485531329 2
+0.19147451046242708 -0.45501523442130337 0.033189878304607082 2
+-0.17451656340254462 -0.2922859658076577 -0.35675496184109445 2
+-0.054032309386595652 0.11876440286815396 0.47910599326324754 2
+-0.36854633000359827 0.21078255760783893 0.26015365494222453 2
+-0.32895930270343821 -0.30492173388755761 0.20577000320862165 2
+0.47233660066064437 0.089962065069455227 -0.11598350628513786 2
+0.18279236780855426 -0.35559836146202833 -0.29576156364432438 2
+-0.30653064961140419 0.0030393921083115001 -0.39158959558808887 2
+-0.43175645289087117 -0.06856641898074925 0.2281241768759929 2
+-0.41664292297822403 -0.1125349496504639 -0.24369438759577611 2
+0.034215630841900949 -0.14316065841453818 -0.47150453356804722 2
+0.01741918812908615 -0.12866441808574386 0.47673458515849332 2
+-0.4855430365961404 -0.047209142857281922 0.071742419244237965 2
+-0.17664526043497175 -0.45607460135991063 -0.066898790386732021 2
+0.15896257150373266 0.16406638468588108 0.43749429982969956 2
+0.3413309978648793 -0.35180180232942027 -0.079355360895298654 2
+-0.050993390661918936 -0.44023303185605778 0.21930692567953947 2
+-0.1744764647710777 0.44331712473052226 -0.13489506049135713 2
+-0.3060405881569907 0.36310552047656275 -0.13515313949756735 2
+-0.31223156562504195 -0.28062263090874601 -0.26163375513939824 2
+-0.22970278795957924 -0.13870241858074583 0.41755645512950001 2
+0.0024393018078136446 -0.48492697824363451 0.091171312301450952 2
+-0.10287286315024358 -0.19966181605110775 -0.44059088195446339 2
+-0.096592995767797049 -0.39151818968827484 -0.28742421301965626 2
+-0.22843344125151183 -0.26611354901572115 0.35137427683212796 2
+0.38322648327275688 0.28370503187708462 0.13296338549842965 2
+0.23663146930518839 -0.40226297814535272 0.16806272118371773 2
+0.26057820529124903 -0.070791204074534875 0.41330127504314429 2
+-0.21136233974279531 0.43042319754635217 0.12802710612079471 2
+-0.47476981336899093 0.0097871739157582519 -0.15138890370937802 2
+-0.23333367583594972 0.11222027080473665 0.42394265074000254 2
+-0.44085548332912106 -0.22979962226592909 -0.017147875405908862 2
+-0.078239229460793286 -0.33884832180985031 0.35255427151172458 2
+-0.083108225197500205 -0.45674438476313062 -0.16738183558114764 2
+-0.29408537723534212 -0.38852555688047163 -0.085435314425446315 2
+0.48133564768818482 0.083381485363606622 0.083302443947374058 2
+0.14428498543762064 0.37758810641209212 -0.28194428393217741 2
+0.3671246210538302 -0.23121760625314156 0.24258442560276988 2
+0.021926538865993082 -0.25448244851283763 0.42640784569274537 2
+-0.44247951547367903 0.14036749361034515 -0.16673429414803598 2
+-0.23376468875393869 0.26504877262009335 -0.34821691711422748 2
+-0.35704052636532213 0.33794495130243385 -0.012949825968558894 2
+0.37906928942131157 0.011439036263529231 0.31801237398069648 2
+0.27641139205562737 0.19692195103917809 0.3576249551891042 2
+0.092299584758946646 -0.46848280891657568 -0.13014498287023957 2
+-0.10077498621597741 0.23174182723272241 0.42421701501864795 2
+-0.0066485172726347321 0.017976077671047648 -0.49685469073126953 2
+-0.12973470064512721 0.071108805185515739 -0.47051085686132765 2
+-0.045859962509014612 -0.013040570986989755 0.49188107961940719 2
+0.079555876774644596 0.35323065472379733 0.33735822117185454 2
+0.30027051670008725 -0.34103946911028737 -0.20394431470188995 2
+-0.24140984558607992 -0.40111697443685596 0.1541797787531235 2
+-0.12786542956347918 0.47578064354031468 -0.018312676486796277 2
+-0.33073847277519897 0.12310202367120916 -0.35116285886815279 2
+0.19933706144873503 -0.098980558450449771 -0.44274269538406724 2
+0.47513113190649536 -0.10584286330488513 -0.095171214369974694 2
+-0.46194239384718461 0.16413779819561203 0.064816441432120703 2
+0.29292266379811227 0.069478725123429091 -0.39043692971152011 2
+0.4200208818716501 -0.17627342566505305 -0.18940858505931396 2
+0.27141459182530936 -0.21104712412748511 0.35480982777696679 2
+0.16960402134421976 -0.28958411644868443 0.36076483378978291 2
+0.38912037422664431 0.0043887969041679778 -0.3087738216613114 2
+0.011497871488576249 0.49686191298521987 0.015154654283299826 2
+-0.31912896462267604 -0.059707423797282617 0.37238981914980496 2
+-0.22834353192806978 -0.089488394878762173 -0.431392095895892 2
+0.10244600457475248 0.12264851193906079 -0.46898522519895247 2
+-0.31732877963655171 0.36699254715931046 0.1030480292676519 2
+0.43307512121460301 0.17115163575065317 0.15968727323737264 2
+0.20016935386821505 0.27437903049128654 -0.35830063594519002 2
+-0.041789517959002119 0.44443177473295281 0.21678051623622563 2
+0.17151835385992589 0.44389799847661182 0.14506941487236816 2
+0.41566132168029968 -0.26488526360467146 0.045899390071247409 2
+0.37932345247717253 -0.11894828342860483 -0.29251072257759048 2
+0.43222030269315437 -0.094575414878469932 0.22599275674135835 2
+0.46904216524592113 -0.14992640959363651 0.021012576355346015 2
+0.3952129455499247 0.29237028550029098 -0.039039028322742399 2
+0.24880421219055254 0.42474901485888011 0.043448685018998956 2
+0.16091128374421571 -0.38054996657036488 0.27661235401976281 2
+-0.11188857008982331 -0.11203266446059854 0.46872664816830067 2
+0.47601719442122925 -0.055826239046367056 0.12100320850727279 2
+0.036473226508413531 0.48000758276333438 0.13140181756020897 2
+0.2575687962417475 0.38744266403728805 -0.1633963398793602 2
+0.2924747065742927 0.308550488374844 0.25036594764074926 2
+0.28955232447365836 -0.17346831825824974 -0.35852620068112112 2
+0.35057344068321522 0.23769120150981893 -0.25765117551066152 2
+0.038207711976355391 0.45161905671520203 -0.19507142457444865 2
+0.071294318064950041 -0.35796429223310122 -0.33311113633367473 2
+0.43261149596338622 0.074600055933304682 -0.22536433518471577 2
+0.19968372557432726 0.38413167365058842 0.24178225128604081 2
+0.082402113370849905 0.48067521037188088 -0.076219372780585271 2
+-0.42710351472239827 -0.18888456582417157 -0.15710660253181968 2
+0.082794585915315716 -0.48573281898067155 0.0078763594179726056 2
+0.34003460150602732 0.35462961771583879 0.052771194783181721 2
+-0.39194251064962948 -0.29224682205434288 0.066620318060341605 2
+0.21146250325321714 0.44101239252657731 -0.064342207106371507 2
+-0.41615443950919362 0.039237897917599171 0.26542474410989447 2
+-0.38719949065611015 -0.20757979990652833 0.22262834471167253 2
+-0.16531972398455036 0.40347241431094483 0.22951939058533671 2
+0.15690430646234238 -0.071378304166784812 0.46162512212067885 2
+0.35564617300046225 0.12242924091988611 0.32468958401881665 2
+-0.0060320878049794618 0.25341782719601502 -0.42951650890993065 2
+0.32833843158390147 -0.31993134286566782 0.18357902655304531 2
+0.058718163718817197 0.12869283620093092 0.47645626595297297 2
+0.28928152578976907 0.20184091716765834 -0.34578827410991531 2
+0.37948009693020479 -0.27624437622995857 -0.15659643437506138 2
+-0.30776469856566913 0.17092044148401719 0.34618439766473219 2
+-0.19171807981037964 0.33160932280601768 0.31212856210359446 2
+0.25245356334943464 -0.27407538282341692 -0.32669723598210432 2
+-0.12065350122228868 0.18725095194360497 -0.43877633407113148 2
+-0.37745787185773871 -0.0382140383301856 -0.31653800141977029 2
+0.19806397299681111 0.02999004881521776 0.45116075228027064 2
+-0.20568413631753474 -0.19724937765481759 -0.40208796797455137 2
+-0.22649260170964641 -0.43820598329127824 0.029158470237451224 2
+0.26096043287752402 -0.41666063080901283 -0.042482612613948825 2
+-0.42206584927238949 0.1930123558717774 0.16731138331459694 2
+-0.34913096406689792 0.29154108584899463 0.19029074976753513 2
+0.083482542589013012 -0.22935515302358866 -0.42817910407127691 2
+0.14066296993008021 -0.4182454383267602 -0.21834552190506698 2
+-0.019478350744578408 0.34101305701760021 -0.3585398694486942 2
+-0.43280484789178914 0.24358274269801217 -0.0022532636193535009 2
+-0.12477135431295031 0.36024492171819705 -0.31355846705091628 2
+0.45040719996150702 0.1861340898361179 -0.073577552235943366 2
+0.1346615486145373 0.26100651447537548 0.39883943153366519 2
+0.11972325147581214 0.22046309413202625 -0.42926431989465519 2
+0.22358187991859324 0.0050357451443411479 -0.43997309187748174 2
+-0.49347606480783912 -0.023059296478814308 -0.031645682797479362 2
+-0.48381106627964754 0.059240694033084892 0.071452086281896962 2
+-0.084385524716018762 0.45031338330530002 -0.19076516385437436 2
+0.3417736296918108 0.32719202506290757 -0.1410758629516356 2
+-0.36843273055519271 -0.3299393961900482 -0.029516851245006446 2
+-0.30678393175041857 0.31567131428080369 -0.22980892622700838 2
+-0.21503122188279383 0.01476232120403273 -0.44334634529176886 2
+-0.47362727421985362 -0.13617417912168595 0.017268332209693128 2
+-0.28942563338168964 0.26966264140311347 0.29454935861001885 2
+-0.10357320476016603 -0.48541613363165004 0.0014314713936747154 2
+-0.30619487146561236 -0.34275586595504343 -0.17858497629969106 2
+-0.26448962933874209 0.014398478534287336 0.4181714868137163 2
+-0.35906984920527363 -0.1906562517760223 -0.27691736545448375 2
+0.037568593306109448 -0.41524091088866422 0.26360868693250672 2
+-0.052051300777545648 0.3093965001589879 0.37924966574493602 2
+0.48113849982037626 0.107746899060504 -0.015838893941665894 2
+0.41638357000568371 0.11530672961586626 0.24322629330302123 2
+-0.45690946732483273 0.17272070723147723 -0.071277948485328785 2
+-0.1477355042464335 -0.42971918080443861 0.19106483854020784 2
+0.16751071157708736 -0.45977476800289391 -0.064198711623625643 2
+0.47961429128881028 -0.0097851515742015203 -0.12372263022700566 2
+-0.14598560906461117 0.085639494944031613 0.46255018063316672 2
+-0.47143984229194141 -0.0076156197959385248 0.1611981163195744 2
+0.0053565411416260453 0.39866517583218986 0.2905565413733302 2
+0.40942048554799781 0.16977242091374942 -0.2179739168280094 2
+-0.083206338889110512 -0.3281379252895808 -0.36122561072450982 2
+0.11969783099324528 -0.47225074378320681 0.097672505668078863 2
+0.14292055894772571 -0.29980614857725663 -0.36544303656835508 2
+-0.39655583689562229 0.15108355572292786 -0.25229011591254025 2
+-0.26185294212339283 -0.25222346318848121 -0.34014502099352856 2
+-0.45979899142386105 -0.10795554987359728 0.14349074920444599 2
+-0.30365720592299195 -0.26233229751469905 0.28962558083675682 2
+-0.23195364931286058 -0.32859076606665827 -0.28749347484194809 2
+0.051224430272876202 0.38202009786492297 -0.30719009190783275 2
+-0.39207220767833917 0.22819369686113644 -0.19600924853509294 2
+-0.37773388282905052 -0.09086954027691431 0.30368729032030672 2
+0.39467792270053165 -0.26759477350524591 0.13904345645079874 2
+-0.41191482597905649 0.12980389870405051 0.2420918813103799 2
+0.26047083115505809 -0.41041354526291379 0.077956591385467233 2
+0.11177459390251984 -0.092412636589793901 -0.47242915830433518 2
+-0.43591522475224886 -0.023732045318672498 -0.22800657623942089 2
+-0.022261713829573419 -0.072318172300016692 -0.48789704836361258 2
+0.0081395223289316282 -0.46592641407744073 -0.1665013453294038 2
+0.081093397331894718 -0.18798589403991434 0.44810321338320824 2
+-0.37558558819514476 -0.24574358317084224 -0.20636842014191564 2
+-0.096338626192954552 -0.39466883610146292 0.28334605052085349 2
+-0.17708873486102419 0.18311451377670129 0.42210576825870783 2
+-0.013860117340804824 -0.21508084368989178 -0.44509421826479573 2
+0.18239848147210697 0.084697118181986275 -0.45087205027424987 2
+-0.17078960076831395 -0.43897569675787684 -0.15534122833161501 2
+-0.18763982805866186 0.45459829368467236 0.045035969439469983 2
+-0.016291632660891162 -0.37370752214727754 -0.3228929421921421 2
+-0.45558439255106498 -0.086590584353522265 -0.16737299007582934 2
+-0.022718764127191621 0.19781133358246469 0.45152398259708748 2
+0.31848408868533651 -0.01256202942642962 0.378759664305091 2
+-0.13927052507890036 0.29328859855178979 0.37285965701252483 2
+-0.3884619298661216 -0.26700045674280587 0.1517506844967999 2
+0.01152504496571128 0.054784659426273849 0.49077140770710381 2
+0.39344564527600812 -0.29567500239123962 -0.034390428629941142 2
+-0.26081803292429601 -0.33226122531683966 0.25565533811438645 2
+0.22251341765848939 -0.38054017239093424 -0.22041067021106323 2
+-0.31713717236303129 -0.35857571029063873 0.1361112009692359 2
+-0.23659629671011739 0.39734206321507148 -0.17778362562216143 2
+-0.14584093702298123 -0.28664018419369452 0.37434403279067496 2
+0.40321686626675135 0.27922069737849553 0.047493421869457403 2
+-0.14134661646917412 0.40629626664699892 -0.24081507758184034 2
+0.035859571147540806 -0.044279065426086374 0.49067862158384645 2
+-0.32037001354706696 -0.079773173240901274 -0.36780943399441413 2
+-0.29237358412217507 0.20085567941458216 -0.34319618038352179 2
+-0.24601576240771028 0.41634162313809264 -0.093434007278415776 2
+0.23816018360973706 0.13983707246510318 0.4109407503558189 2
+-0.068273343569342407 -0.47092136149196839 0.13980209463711818 2
+0.018801598797916399 -0.46299859053497849 0.17334896426697666 2
+-0.03416076831939252 -0.19390981014461106 0.45259077825678906 2
+-0.1547331916875945 -0.13107618004040195 -0.44933330796862875 2
+-0.094438908877836236 -0.47640015231839816 -0.084018910988347975 2
+-0.35724359501005171 0.29868741054458459 -0.16074154128435605 2
+-0.13364638908971294 0.44598349918207303 0.16167096559856231 2
+-0.27408682777806986 0.082143784286755478 -0.40134550473258368 2
+0.087697484975903461 0.42291710705325825 -0.24992582746355946 2
+0.21454829343853296 0.38156282591278823 -0.23560767118701434 2
+0.12550167434991091 -0.24619716991715462 0.41321139943412383 2
+-0.15104938876688062 0.0014584089104602367 -0.47597596324174679 2
+0.33514719842099605 -0.11676342614616753 -0.35080058024630445 2
+0.34140929106701184 -0.17432465269026692 -0.31177814142542615 2
+-0.47990278250569951 -0.11339875697248844 0.079887112571497526 2
+-0.30962512834722744 0.38356950160675773 -0.058101849310527684 2
+0.03554689615386189 0.48034552633383665 -0.12991041112850576 2
+-0.02709267922225124 0.36247438890288292 0.34097725028864367 2
+0.20848291996620624 -0.11757687601054088 0.43634411921676908 2
+-0.42882400034908141 -0.14350399053844937 0.20590057102758208 2
+-0.090560564339627767 0.13105873777682209 -0.47143063834755128 2
+-0.11090040697797957 0.47505096341799569 -0.097021220812907455 2
+-0.35545535296641972 0.33196185443364101 -0.087854040926861526 2
+0.47105024097298787 -0.12774577332923742 0.094515479810394459 2
+-0.15460594550149334 0.13044361732060336 -0.45085209386903086 2
+-0.074848554765210656 -0.13177409721165256 -0.47382282861169434 2
+0.15101292400278854 -0.0080031878833149023 0.47521043462103896 2
+-0.35736564422440908 -0.12769543239265371 -0.3207034667651314 2
+0.094114109904528578 -0.11570906014409471 0.47388311860317556 2
+0.28535641736280803 0.0030412638087362431 -0.40863291386221512 2
+-0.039990091122710111 -0.48199468786047772 -0.12270610879185923 2
+-0.18853463770583434 -0.43952710870582812 0.1385927951163417 2
+0.13625350025545474 -0.25384962961629476 -0.40660051121768037 2
+0.076403828318303169 -0.29026739247135586 -0.39079336597955455 2
+0.13728432455485556 -0.19414197011251358 -0.43056334484244385 2
+0.42298765123118853 -0.11727360742662864 -0.23329534385311404 2
+0.30420797260019533 0.37562352479984207 -0.11993652420543416 2
+-0.3502334626941026 -0.25676834588920638 0.24429292962306959 2
+0.38807058492308766 -0.17506321357061666 -0.2516437757750114 2
+-0.4334710607829404 0.10192783216805726 -0.22248770476455654 2
+0.40914491750034232 -0.057003079574881294 -0.27052242425340334 2
+0.0091819319799406746 -0.49666255021107453 0.01737846001306019 2
+3.014162351272498e-05 -0.00022412216118153361 0.00023687663445681739 1
+0.052801989430259211 0.23970717573936154 0.056895539019464536 1
+0.14450234786464269 -0.19474901447604212 -0.069052969614780621 1
+-0.17916427332813784 0.025715782501462409 -0.17491934196076869 1
+-0.094315748745573691 -0.1049186313862833 0.20842808898403417 1
+0.2028455816993574 -0.032721226494168885 0.14467222002492844 1
+-0.11961709481342205 -0.20932670315450266 -0.068932502351132449 1
+0.18154163452546179 0.031730055037039656 -0.16932814806384502 1
+-0.19142561708366748 0.1138767024163574 0.11305167457083386 1
+-0.001473790868043362 0.18871183238848396 -0.17245421855316698 1
+0.028830209928174921 0.094933596521697028 0.24343604928426632 1
+0.00445678131695941 -0.10545364547208048 -0.23862990061537631 1
+0.083937504337175992 -0.22041685542625192 0.14318288917914965 1
+-0.26084484777752204 -0.084924861365920409 0.046567197308790775 1
+0.24072617665983023 0.13728163083605013 0.012659213366990968 1
+-0.18328211734473712 0.20703285866735111 -0.07286245033855264 1
+0.29050347659524578 -0.063745480233620366 -0.031980568377714735 1
+0.081285632368155863 -0.085380926800716406 0.28373794642767847 1
+-0.096583387241615679 -0.26654065258928961 0.10998932892936106 1
+-0.04719957583257596 0.062530421018024848 -0.30281668858126498 1
+0.19645413834244752 0.15931720652006937 0.19260664679725728 1
+-0.076079067615941798 0.23616026396562506 0.18857037620037154 1
+-0.30229839132862157 0.069289308586842888 -0.045123575222269438 1
+0.1746994390040173 0.22574465843704122 -0.13452130698459194 1
+0.018790113036324302 -0.31664520595790835 -0.020976598390514467 1
+-0.14425146477183853 0.054707135099594573 0.27736998815826319 1
+-0.17029299303790246 -0.14528482679334123 -0.22878116705187657 1
+0.24566480433886223 -0.19641479873476103 0.075925501417087884 1
+0.17748833700630379 -0.13286378257956177 -0.22983460624370367 1
+-0.047773565441192503 0.31415600317739373 -0.062623118520660589 1
+-0.0068340816863229417 -0.26949598878753778 -0.18529913609387003 1
+-0.29131727265500779 -0.091219267782148672 -0.12125383672688822 1
+0.10750821452013629 0.1282916389194351 -0.2864660752012067 1
+-0.27344320558630231 -0.014901787152823198 0.19765697923321177 1
+-0.15459676992893528 0.17817758116580687 -0.23261792930941644 1
+-0.18299147861248341 0.2794439735685183 0.073269565509758133 1
+-0.22576517276420693 -0.18779410440076966 0.16560020829646344 1
+0.0087367713580315202 -0.16014075378076173 0.014482767898634712 1
+0.02166937950933422 0.030821587360851471 -0.15569200149497292 1
+0.10416721025013662 0.090115232451198596 0.081606705957980669 1
+-0.067724231557979284 0.14324655796362867 0.016712147417071811 1
+-0.0068808023839113941 -0.22059458984402644 0.27372687504858612 1
+0.11043443124615403 -0.025263791341274933 -0.32449021017160729 1
+-0.15634747002974614 0.014235725496882193 -0.018800570528364813 1
+0.33273455404191438 0.032392020270790335 0.08489051290760373 1
+0.077557837652741934 0.26039794514439762 0.21064194760877653 1
+-0.24932450725818317 -0.23792265115648828 0.014355512766715366 1
+0.16779116883852999 0.044701666713052791 0.29548740805949181 1
+0.31575259435477893 0.09399522491339185 -0.11786154416048142 1
+0.051109009578189674 -0.066547638567894724 0.13185951628960896 1
+0.19280950303903735 0.28465046570209951 0.0077455433522710115 1
+0.20543592423973955 -0.16649638529616964 0.2222244118011279 1
+0.14147121840903676 -0.060593393920313841 0.0063129913528785359 1
+-0.066668144025147216 0.030475814304770879 0.1427894834947423 1
+-0.053342746902161536 -0.047652654695166191 0.34386322870563746 1
+0.08280059069911061 0.13129665268237911 -0.061652604676741274 1
+-0.080890803084922636 -0.071920152617245853 -0.1163127959139466 1
+0.15321807762093834 -0.31048652461961324 0.04527565972171714 1
+0.27730652027962055 -0.17358936258398658 -0.12934685199851487 1
+-0.29575415255273296 0.19811115028253887 0.02544424014218119 1
+-0.11594462289123944 -0.1003298772972976 0.063416305732725525 1
+-0.16453093569200922 -0.02467014850318244 -0.31165181888248822 1
+-0.32992845318152686 0.073441695008099939 0.097303171396798366 1
+-0.050590078854345299 0.33650480258096455 0.087263062342506803 1
+-0.27909076718019299 0.14205013356346963 -0.1681282101913823 1
+0.093643905047351469 0.33473719934111179 -0.084850635899742316 1
+0.1365239106386455 -0.26572413696193398 -0.19369272959981307 1
+0.30579586459250829 -0.039832176647313075 -0.17256885252520576 1
+-0.12107511502677282 -0.33847608452377875 -0.0099517616438331291 1
+0.28807479052278323 0.053966727407861501 0.21781336000382234 1
+-0.24325839346969885 -0.22340575839088872 -0.13616182431928803 1
+-0.19605855379734358 0.18006794204638638 0.2378006126850809 1
+0.24262572115639683 0.14439754208748706 -0.22653499049722928 1
+-0.21374383143570785 -0.1099605652536701 0.28141428517341038 1
+0.055838347132006666 -0.09165768297793575 -0.098981075271938807 1
+0.32536668354276843 -0.097853682623521454 0.1334779603578797 1
+-0.069607517133508434 -0.20161380987089875 -0.30588629293096281 1
+-0.037992256595614907 0.20068997990658471 -0.30550684354217372 1
+-0.069883019615067571 0.16656432158083861 0.30707360056768007 1
+0.020221814841485362 -0.33876987985287055 0.11463516103966132 1
+0.095112459804220423 0.25670232489504585 -0.24126307771534589 1
+-0.090351576153363491 0.29057069730420337 -0.1899468271782144 1
+-0.085771491224872862 0.10131329866839714 -0.11133497642330889 1
+-0.1356134596047664 -0.24931697767483774 0.24502582019785868 1
+-0.14014341150326934 -0.28056116803993503 -0.20137551252303879 1
+-0.043055607684475763 -0.060656661150435005 -0.35585010276686074 1
+-0.27898679140235272 0.035739763062450192 -0.26373725595802683 1
+0.12196930650670011 0.16964660742462959 0.30264007452684843 1
+0.081466052102798017 -0.19851378729903479 -0.29451536740227535 1
+0.18864200228449263 0.28147737267882583 0.14062997857195028 1
+0.23548246919943949 -0.0029127207663287996 -0.28566493689687922 1
+0.051510065226324356 0.032472449480436719 0.3583650570682278 1
+0.084047659205297276 0.36619489627271401 0.082345773170841685 1
+0.22057270245230437 -0.29979009662294809 -0.096890920350708759 1
+0.30651207484667292 0.17635977393943134 0.12082800154917915 1
+0.2935085375499632 0.21769740288806988 -0.077671326675799554 1
+-0.36431048669079091 -0.04414505302889548 -0.022970448508646157 1
+0.2490734023486208 -0.055997172011154006 0.27682372524595567 1
+-0.0027720154490750992 0.14722434378837171 0.12896792856686445 1
+0.38427923619976989 0.024627473125503771 -0.033608726736941097 1
+0.19618479811608763 -0.28022204855520977 0.16273309474366399 1
+-0.29217712036934773 0.18281679758375746 0.1528123312776547 1
+0.18941981895892393 -0.082062422506531035 -0.11068868674041099 1
+-0.34203047890473742 0.019961039355569699 -0.15491427763962118 1
+-0.17130406611996379 0.32870589590726007 -0.042170692256760595 1
+-0.1859801112964658 -0.10335399169545155 -0.053130397606924826 1
+0.20925354190654583 0.031039983537490969 -0.046713072711011647 1
+0.34316257935580979 -0.17305233972043019 0.00033628785594420896 1
+0.10585438212623531 -0.27068000967665412 0.25522470003309389 1
+0.10570601475333902 -0.34728880042335131 -0.10475567263392008 1
+-0.10737487496542224 -0.15785160692915554 0.32429126121625373 1
+-0.28942329711923143 0.098899368336733351 0.24387548669130543 1
+-0.18600538017573232 -0.0097165410864480765 0.11024835660358068 1
+-0.33700909228305376 -0.16440212684772157 -0.032615861308760478 1
+-0.35754796563725394 -0.045926152248843527 0.11272604512623377 1
+0.040439522622158219 0.052995831021695201 -0.38868361297404813 1
+-0.1627994660789196 0.097276470891627137 -0.3244017063735235 1
+-0.27503765461948215 -0.084196368023144799 -0.24192559305495548 1
+-0.026027040635067243 -0.16944540002448633 0.13087314993656254 1
+-0.20907905584250255 0.26967368522422452 -0.1738677949369267 1
+-0.054707311001315637 -0.35288645455937306 -0.11083771311146649 1
+-0.0079051269417109576 -0.019815464014738576 0.23551126727341776 1
+0.022414870924025512 0.33271787709186629 -0.1821910464244727 1
+0.35850841308377668 0.1399296416829249 -0.011329108742475343 1
+-0.32012272906753891 -0.17921166014651391 0.091689120891368547 1
+-0.081159267539283056 -0.02258826290625289 -0.22555567818167405 1
+0.10708411622202758 0.017795180136314259 0.19583537139940682 1
+-0.19308381251986259 -0.32740631883353943 -0.10485224870297609 1
+-0.069656435928420329 0.066957246422228889 0.37276360757684202 1
+0.15016047438977465 -0.13052573208833282 0.1021457130615888 1
+-0.21237097053697102 -0.29348949150614301 0.1127809144233278 1
+-0.042776646964407375 -0.31581483089999418 0.21259701644815598 1
+0.28174334907322024 -0.28288909613825791 0.0032659077871620146 1
+0.10038569069270485 -0.048618048372519103 -0.19958184701886453 1
+0.029291712945843311 -0.20652559834069623 -0.092156251936651362 1
+0.22805496052133986 0.077988805449757809 0.11357146320614143 1
+-0.29822637932689999 0.22674577075059674 -0.088682957886754801 1
+0.38538230783394484 -0.06920137199531573 0.036820948221615454 1
+0.012587257164439321 -0.14285071010234318 0.3586003143792359 1
+-0.069234065681228973 -0.17125441163822883 -0.17647764920933623 1
+0.15789088244738314 0.19268689578488629 0.072338732863710217 1
+-0.20601399684411509 0.11784763250974534 -0.0020719494738760252 1
+-0.01215928598363232 0.33268402163839006 0.19670344264770664 1
+0.096144947117105473 0.016204821808735136 -0.067923735806303481 1
+-0.0054554378407597054 0.20634708000879021 -0.05796204423258542 1
+0.12039083155079047 0.13017578933584079 -0.17129088773129522 1
+0.012747854871683756 0.24767855708034087 0.3057576437926483 1
+-0.30789349699463731 -0.12375268876932422 0.21577078494474838 1
+-0.16720437603028054 -0.03083908758531357 0.35123858918745399 1
+-0.18128081866335391 0.2830487056717238 0.1883784304214256 1
+-0.10436502357698049 0.12509187236227029 0.19585659946239731 1
+0.31901916410525533 0.058808598066358339 -0.22972424345691189 1
+0.027769095322753831 0.14464675957004966 0.36323507776810504 1
+0.13960116785223819 -0.17898415571113385 0.31534230524128104 1
+0.023568426965540581 0.38692355277584711 -0.011858440788303823 1
+0.23985837710368524 -0.22766093610137156 -0.22213533463966978 1
+0.15778991217941379 -0.12673816359074469 -0.34115248458186781 1
+0.29625857514519732 0.24566730561158784 0.031953158536357829 1
+0.080656448812452167 -0.16841296531297367 -0.18478003237880164 1
+0.16496489167827175 -0.052783274797780409 0.35116768097441575 1
+-0.036474157223670325 -0.39535327705576773 0.036384073815337016 1
+0.36371164785207277 -0.10708387179365877 -0.10454864078898034 1
+-0.087048809320665527 0.25316102697845128 0.022369237421085028 1
+0.20897227862757828 0.099018777264479663 -0.3229107295199839 1
+0.043871234042261342 0.16303775916229074 -0.37115051312375918 1
+0.10637190794209886 -0.20995542498124597 0.034538474900766614 1
+0.20987672192604165 0.12455653007105925 -0.10616667532416831 1
+0.050145833646477067 -0.11927746362871103 -0.36483038374168059 1
+-0.2569589135759463 0.011876919793466743 0.30339559661531923 1
+-0.14553415621357807 -0.12941862681585642 -0.33965282644394962 1
+-0.38045926172758571 0.13381294154801746 -0.0024935828596186829 1
+-0.14786565736419174 -0.20218619506824634 0.037078655502300446 1
+0.27185960596773107 0.24683992968871024 -0.18144502724386516 1
+0.10624225261503392 0.23598317618805897 -0.038815416384184613 1
+0.20207022106946587 0.32360756219858922 -0.094044676976052274 1
+0.27731456915564023 -0.10000774304333798 -0.2589370471696687 1
+0.23834487210314695 -0.15931874140263813 -0.026333756732376437 1
+0.050413814320079887 0.039243068358821825 -0.2604303190974549 1
+0.24065670405888243 0.12440863007856787 0.28530892422349785 1
+0.24131817164355052 -0.089363242825941602 0.060911262413372344 1
+-0.2364719245809476 -0.004270508769169068 -0.088600013049095899 1
+0.28515263160773274 -0.22023725186962631 0.1733951527718868 1
+0.34009321243624258 -0.039735948062104037 0.22243245385186622 1
+0.08984024346625695 0.15791205654876458 0.18182196119615981 1
+-0.24638507448744487 0.13757586356258628 -0.27002689226946663 1
+-0.16857571095821844 -0.02812365115043744 0.21418353376682697 1
+-0.1731917451426222 0.2364117398325114 -0.39729462373902957 2
+0.12336966796224377 -0.35454250264062137 0.13803251821307455 1
+-0.19775114308164371 0.10713396961444183 -0.10814111302044851 1
+0.19857233363919352 0.23634253201295671 -0.25847400182932251 1
+-0.11478742160567977 0.18739418423886112 0.10180484691975492 1
+-0.053611188785143718 -0.24671323997391675 0.014373748327959429 1
+0.006763757672023106 -0.27457303434908686 -0.29097458896782502 1
+0.13236190385685356 0.33815344006019898 -0.183870581840652 1
+-0.10242343439746388 0.087166834030478577 -0.21537745486588183 1
+-0.2583766411997464 0.021032172749625697 0.03875788400968589 1
+0.076945048663832172 -0.38384722380040015 0.037063846680694387 1
+-0.26113960503705214 0.29588945218951068 0.0035529301492612936 1
+0.041284901195559109 -0.15059551548257941 0.21046777203360834 1
+0.17819004744153116 0.25123110214993383 0.24124469073292951 1
+-0.21817168097492304 0.11711712038934805 0.3195954816265269 1
+-0.23396064679188577 -0.33983934322570641 -0.008348234341521682 1
+-0.096390153856920888 -0.37138062730386218 0.11998934286889619 1
+-0.089740354549113041 0.21183842613184775 -0.1204125200007857 1
+-0.1315348553333206 0.25354532785257095 0.27599901609419952 1
+-0.47309049000944725 0.083917611539784609 -0.11427548002999455 2
+-0.10232075630902869 0.021751408944293536 -0.3821508088222666 1
+0.14926503561707283 0.37861796383415519 -0.0077371296117765553 1
+-0.19306479566447415 -0.075852084128277375 -0.1538589796531375 1
+-0.26369421911518526 -0.19216976823655502 -0.23391148886092913 1
+0.12020736222706946 0.35044000592900432 0.17911686727626699 1
+0.3048115684889941 -0.13717734716591962 0.23313392161126517 1
+-0.1230622188165649 0.25876006525325701 -0.2904721485393742 1
+-0.093393508717123319 0.38020496232867673 0.0033071108055348006 1
+-0.36027962609956549 0.1449760584656708 -0.10362940598672385 1
+0.38469283098921392 0.02319611420450559 -0.14916372426046942 1
+0.13749184053910829 -0.092794069634492327 0.19742968657155993 1
+0.074797661013089226 0.25012545596594921 -0.1404934926198349 1
+-0.054477938391975539 -0.085628442065833837 -0.017799871366431477 1
+0.024806284611034665 0.098272052066470883 0.016645819433334985 1
+-0.022061313749070353 0.2946354401931443 -0.26676223580777092 1
+0.024946442360908885 0.0263042580677295 0.096367363640021172 1
+-0.12866014902766865 -0.17232468112762231 0.13566497641610362 1
+-0.28146278773699562 0.27081314138547607 0.10101636941299799 1
+0.36011527425244588 0.097685548130398842 0.15920693658319554 1
+0.066187236726057849 -0.34026801549245778 -0.19922030279276431 1
+0.07022159834384252 -0.063623002441841325 0.38885282067455429 1
+-0.11758927070462714 0.065467038838721014 0.06103679065767291 1
+-0.10199424288607814 0.37130749604407531 -0.12804351163479197 1
+-0.045794287223631047 -0.4324557833521423 -0.23687521773595907 2
+0.13954101029359572 0.087691825752021196 0.46423620151461575 2
+-0.39476624440822194 0.038136393518752726 0.027823555675558567 1
+0.042775402298076214 -0.40900634810064485 -0.054889459858675979 1
+-0.056154878290525095 0.1311062337485458 -0.37645110033906548 1
+0.32857963727660389 0.32088575127821412 0.1792588089484089 2
+0.27611524808374921 0.2192909383500975 0.20670216949263528 1
+-0.30793708451359386 -0.25493837901738242 -0.065728746169034485 1
+0.17340100444749926 0.025669119720247442 0.047186460674716942 1
+-0.36289506615087352 0.027981941147756248 0.18079850569588596 1
+-0.20657734755208726 0.063754151458302072 0.19893688931836873 1
+-0.34094180928632267 -0.14757967407118869 -0.18786326339274695 1
+-0.23553097831014172 -0.090399560703751214 0.14351157136500015 1
+-0.10738296764579104 0.34812174360645026 0.16914638577466848 1
+0.12295950646474089 0.21757928211552441 -0.32918054547200276 1
+0.012981378463993831 0.13186935449453266 -0.25361598352399589 1
+-0.18282846177641854 -0.23173302295590589 -0.2775513473831005 1
+0.0052481079377969496 0.19020502243444401 0.22426672589204605 1
+-0.39541077193317081 0.037156530734504901 -0.072147206050796608 1
+0.34431029335123825 -0.18967359339033504 0.098873723612798434 1
+-0.22440434312429891 -0.20757435375880903 0.26343757447756794 1
+0.17560792936664665 -0.36448846737671531 -0.035620747830156084 1
+-0.048532728179369017 -0.066526068312505363 0.12862283474273536 1
+-0.38032248306506256 -0.11517674900610347 0.044923628284516896 1
+0.14986287173910431 0.041731457075121423 -0.26282606340174863 1
+0.29507169567874758 0.055981517478235029 -0.0037521482767681157 1
+0.3099538909046799 -0.25775877376865519 -0.088388609139942564 1
+0.18306476248019357 0.36453429163376155 0.086802054568749953 1
+-0.070403421397669072 0.013740322100972452 -0.067869860279548375 1
+0.1791412793429773 -0.21367260721022319 -0.29899955568168235 1
+-0.36614232797971086 0.16937465922964734 0.088622907414423815 1
+-0.15448089410822588 -0.32615429828064552 0.18586659724105453 1
+0.052324108742054357 -0.038351248529848364 -0.48856124210133045 2
+0.27339126238480399 -0.28705305917821577 0.10218477312054483 1
+0.12554562290815949 0.096164566717367839 0.36751961870386074 1
+-0.021181431807494128 -0.10907330839819857 0.27324381903867717 1
+0.052550146099273894 -0.33695348472230824 0.20692692556226724 1
+-0.28494321772920805 0.27677228364487483 -0.29224974361019257 2
+0.10772611365430303 -0.27870879909125396 -0.035165389045932201 1
+-0.14856308449960456 0.37033969772799641 0.083357807780090915 1
+0.025831712627652915 -0.2515063025023635 0.071048621658933692 1
+0.073182083123299962 -0.29466437179154648 0.38780511575873033 2
+0.0080333500708874642 0.25411677277280265 0.14242338431531035 1
+-0.0017606887646237808 0.38831694465959848 -0.10592558893991103 1
+0.26398114212563917 -0.14381701618748871 0.39063091979078446 2
+-0.38326706137033695 -0.11875831278201833 -0.10508308432111768 1
+0.46034682268860305 0.15676767565379729 0.089581324608377158 2
+0.13578306781534899 0.056437373500978286 -0.37048711030225778 1
+-0.052966710537569503 -0.27929443321811831 0.40328721189090194 2
+0.192560991004216 0.063008277917616262 0.20350548147513242 1
+0.050258612180519122 -0.075186495631741901 0.03540413824808368 1
+0.10777834999410811 0.26611091895884736 0.30598558072669596 1
+-0.066278520399229893 -0.34290151563057464 -0.2064101834937612 1
+0.19685886776352077 0.2027080197434461 -0.043322649523488271 1
+0.0015847901613452037 0.080830144016209665 -0.075502012245031377 1
+0.26368489950477669 0.033773895863873762 0.30891554526250792 1
+-0.23299918759270005 -0.090432163829179152 -0.32846528987419321 1
+-0.088108143022930407 -0.11027631435176584 -0.26475408611063156 1
+0.38973726098737016 -0.019243022032469914 0.14264304566481822 1
+-0.01289473966910289 0.48606046903305394 0.08085761591040605 2
+0.33463968548109069 0.17827648066868423 -0.15894998183293146 1
+0.15134385076449164 0.10581068340528867 -5.9750900477974211e-05 1
+-0.17405379599508039 -0.36653139572584437 0.064489678375509452 1
+-0.048854911632127768 0.053711968379438257 0.2805493974584623 1
+0.21655805777292733 -0.049594875144037723 -0.20470694456379607 1
+-0.20231826270209063 0.20805349381289187 0.13341602539504827 1
+0.092366405386281519 -0.1059266853010145 -0.27518724745635098 1
+-0.16707682968160315 0.20164704292262747 0.020780910447402774 1
+0.19524860317316997 -0.24489588806719859 0.27517702750751011 1
+-0.3039111084610352 -0.27281812486539642 0.097065124694635241 1
+0.19626955570096213 -0.24183875140106834 -0.0045910103775962108 1
+0.19582217433086202 -0.042336659338486862 -0.36245467861757791 1
+0.1868752064317967 -0.18873558420146364 -0.15372957291117345 1
+-0.045633180489322724 -0.15147108696986303 -0.38255403163522023 1
+0.48770423858475598 0.0061209045436740276 0.075154425503095423 2
+0.47672175612571999 0.0094806422807550209 0.14315724795141693 2
+0.033436062702507333 0.29305660163602409 -0.018840324986662348 1
+0.26197014702976135 0.31243594682877091 0.089519958804874414 1
+-0.043673796350527663 -0.26494914974355055 -0.077875466162947674 1
+-0.16399744816910741 0.061074557863654004 0.3730679880969614 1
+-0.16874774479793608 -0.16542383351305237 -0.13655358726981978 1
+0.10558920218023279 -0.2883522675688806 -0.27973042458712222 1
+0.28759948121919943 0.0075922568669945789 -0.093321899493478105 1
+-0.29486701362731427 0.23302861539801364 -0.18637025878136035 1
+-0.3754764128643342 -0.0084805563105020514 0.32194621216299329 2
+-0.24651850149330784 -0.17519194055264725 -0.055562192344838009 1
+-0.22751710255401345 -0.16845076940402859 0.073697384148184594 1
+-0.24536469437509145 0.30414389962752025 -0.094414445157329593 1
+-0.23350857302408798 -0.28870963842640884 -0.20268635756650838 1
+0.33355094667626339 -0.12994863842172816 -0.19024905887558033 1
+-0.046583807889072994 -0.15241831865196193 -0.082961453490844869 1
+-0.074611879123396452 -0.017134658095282806 0.05376761501253928 1
+-0.26578688709696846 0.38360878439230706 0.159993866467324 2
+-0.06913794883377869 -0.19458616531484274 0.20951642194864953 1
+0.089498950171411212 -0.13494631779164529 -0.023959860311300797 1
+-0.18558142756911827 0.18711832187159916 -0.31964230522062342 1
+0.23634040692314512 -0.11916228796852599 0.14864364771349806 1
+-0.088210793990276731 -0.2879883344945246 -0.27781252700893333 1
+-0.26086854777577478 -0.017830342079347795 -0.1788234347614589 1
+-0.33787933076748999 0.089161001111497051 -0.21629255429682587 1
+-0.082238342689786859 -0.24671853062405819 0.32044664884209845 1
+-0.0054062426620777558 -0.32672003076719502 0.36865108381055367 2
+-0.0056290036019684894 -0.38106105636912002 0.31574290758864998 2
+0.22220628949086949 -0.13840275692162152 0.30851771391391392 1
+0.059753767986784327 -0.14859920887922184 0.090513805315648982 1
+-0.2667268431609312 0.25334447042736447 0.20660573212278818 1
+0.38480762928955115 0.10723688934594511 0.070832461234396973 1
+-0.15677184091914464 0.18482085016455907 0.33213232709257567 1
+-0.00064705198299440381 -0.25340022477919039 0.15936372413060598 1
+-0.11020620758406793 0.022836216030780593 0.48067896906982044 2
+-0.0032908640424136838 -0.19714476189659899 -0.242153776325925 1
+0.0093998308755075255 0.39201559930573759 0.12966967049431224 1
+-0.35410381558087078 -0.057697060927998811 -0.20288559066945844 1
+0.059673730534753888 -0.21897248874357986 0.33725101689325682 1
+-0.17838054976210899 -0.27075414553594135 -0.033985269742722632 1
+0.27111033575769367 0.29800247292168042 -0.039183813111945853 1
+0.037693459566387935 0.25035362521103288 -0.32050439627600708 1
+-0.26377566706920014 0.40474764330518331 0.097782864651467877 2
+-0.0022521275033995578 -0.057420856046653486 -0.16094826021995451 1
+-0.26724669473642121 -0.19316209606505375 0.36593332465911249 2
+0.085499007306107461 0.023008571003886057 0.022531195875965832 1
+0.098844449635862128 0.28069945194991636 0.12422379247805743 1
+0.31655333775702299 0.14012618136656779 0.22720581707510407 1
+-0.36004573663529021 0.12360063255151099 0.16730063077305124 1
+0.019555949360480052 -0.21128114961576963 -0.36029218479847658 1
+0.024318355975380859 -0.039456773514895682 -0.2982578166862066 1
+-0.053302970875965923 0.29993165832454888 0.27107195243223825 1
+0.11074740663859059 -0.0023209238812337309 0.10707812100084599 1
+-0.23369222637748979 0.026076224351167233 -0.34209863262360352 1
+-0.093658627887151527 -0.024119861366794414 0.26572354640586565 1
+0.033069219405781114 -0.037001826242540892 -0.39966811891681564 1
+0.062480776161146667 0.3415433860776525 0.24881093986055752 1
+-0.19006710051136508 -0.21182820767376392 0.40334880581212612 2
+-0.15117658436565384 -0.057729665999662422 -0.39529041860225311 1
+0.21096164409808271 0.2411647762236705 0.3779204346949695 2
+0.26151083986578794 0.26282880113491397 0.32980393996530022 2
+0.29009947850455003 -0.0095043136026206898 0.15308083393387206 1
+-0.3098285643626284 -0.22121496989046627 0.17135381975820951 1
+-0.0030745485663673938 -0.015807326101892019 0.41212024611049008 1
+-0.14172327018728048 0.28556847263614998 -0.11581586672608517 1
+-0.22499271278891853 -0.27147465929726566 0.199387037696865 1
+0.33344952423229235 -0.19635668104034432 0.30633731184840085 2
+-0.021396171792294139 0.26879464592372543 -0.13600674659771203 1
+0.41451092319956245 -0.059018811645844285 -0.047794195950891169 1
+0.11862227866704492 0.10064040959043494 0.24498147812342444 1
+0.086230367737865965 0.16375703761964155 0.022170312114837873 1
+0.17954953571185736 -0.02124783317111862 0.23179488813310006 1
+-0.33908504237729525 -0.23921603457248902 0.017026141102442993 1
+-0.1049325276446455 -0.045180241355882306 0.48040953182320123 2
+0.16385903817559577 -0.22218965562641499 0.10237333185805998 1
+-0.29305831491827128 -0.068099933139374097 0.28458839114693735 1
+-0.20092482262250835 0.10964844962442945 -0.19791334687654147 1
+0.068942202224407514 0.0029955069730034645 0.27555875309342848 1
+-0.16236711227235467 -0.15510138526654288 0.43923281716767948 2
+0.41976828002296784 -0.22321200687821596 -0.13898651814964413 2
+-0.26376459679853714 0.14801102106046626 -0.063514948655925496 1
+-0.043707734836280182 -0.087160102933500516 0.48560279250516092 2
+-0.15802654231952018 -0.16653664499584497 0.21988342840539676 1
+0.024331064489864926 -0.31125973494592851 -0.11271876604229143 1
+-0.11559571688941267 0.15387999526142576 0.45400156595678642 2
+-0.07167740472751119 0.20992098201559808 -0.22335335036418605 1
+-0.1491917475325466 -0.016357705696806901 -0.10224485743117828 1
+-0.2306497953749824 0.22228666588434001 -0.24723041022010697 1
+-0.13144693740742314 0.14002434622071488 -0.04541299504574707 1
+-0.28141787556258735 0.18558837696116509 0.26243372199734821 1
+0.13054785106683886 0.14479124561273679 -0.37983066967241486 1
+0.25769984113159344 -0.0023623980042580817 0.052061446411661758 1
+-0.26494850649782675 0.12426101634281547 0.064172013322501803 1
+0.12005227946995625 -0.1908756192412181 0.2188096871298075 1
+-0.27887316288322439 -0.06352253435650923 -0.037757109208955322 1
+-0.065960876300959509 -0.12031833421810569 0.39323461699135742 1
+-0.48122871014891949 -0.042584193588204991 -0.10092600199458213 2
+0.36491740070743223 0.1957270440341696 0.057139964805739601 1
+-0.024962087819082851 0.10339693677256825 -0.17566633837899109 1
+0.44619912765057934 0.12464786437614742 -0.17434519797785047 2
+-0.36430451916850232 -0.12690065910547679 0.14770022129768121 1
+0.054790863636619912 0.079625420499855021 0.16029551450519436 1
+-0.02776712276609326 0.20451529123961837 0.066410880598844022 1
+-0.19092551559039672 0.042667425444435254 -0.26085762257038858 1
+0.10833714071521103 0.30163084998123035 0.0271809783016701 1
+0.20541729434062569 -0.34740715594795879 0.10618042678623019 1
+0.2373618104996971 0.22434971528213393 0.094176968625838686 1
+0.32334197254079833 -0.024750104564432576 -0.25770646780151452 1
+0.17342445445381852 -0.33853545638595112 -0.16047221364995759 1
+-0.040852174876646268 0.080635219182546541 -0.48576716949190768 2
+-0.066215788508568144 0.027311811889385303 -0.15467671760363408 1
+-0.10575010161414924 0.2738638707643401 0.11489477718290128 1
+-0.25612470755483285 0.039226722898682218 0.13060064002721536 1
+0.39443774579235996 0.12024045120312925 -0.089008472404216388 1
+0.018961428251670451 -0.30418817632757017 0.28114984423349887 1
+-0.1481399319006777 -0.081857618651120068 0.14299434933456259 1
+-0.16588969726651318 0.34558034965027851 -0.18240870654409286 1
+0.23118384919793089 0.21683538121308274 -0.37893771963620582 2
+-0.26587874704060482 0.06032356944427035 -0.13978542303404215 1
+-0.39731996266264513 0.28851976009431118 -0.044378308977794788 2
+-0.18099553473653571 -0.059027555211072211 0.022079614876154771 1
+-0.096793182622928298 0.13922916786611267 -0.28536628352287036 1
+-0.42524424938196426 -0.23557187749304304 -0.093936862140836422 2
+-0.34981351324110294 -0.1984420378206895 -0.11746291563394778 1
+0.083518890584546424 0.20195189593146817 0.44228024589446685 2
+-0.35934001074403754 0.04370741166802089 -0.33503702749383063 2
+0.38112499666258443 -0.23380860756053309 -0.21304072467703439 2
+0.011336598332309364 -0.018148889418823255 -0.084319480306515843 1
+-0.11968030805226781 -0.28209895052106987 -0.11675099546023027 1
+-0.20458476828247013 0.18494194933765132 -0.15434296098837028 1
+0.21573732351322028 0.31334177075713721 -0.18160115611199051 1
+-0.072809298939057521 -0.40323032259712421 -0.042252898590777778 1
+0.16455498969273469 -0.14747803469444665 0.018095161482380974 1
+0.12845642537231569 0.013435188001823466 0.39404504012287489 1
+0.093422144597508638 -0.25888403351035577 -0.11855113815612503 1
+-0.19890103708578824 -0.043197668884905158 -0.23409956778195812 1
+-0.064668727888912508 0.48885627077083871 0.023828543054009343 2
+0.089472047390780046 0.4591033498031909 -0.15731063262053824 2
+-0.054289356976667952 0.36862767534947127 -0.20027274759484467 1
+-0.067122760381802288 -0.17129596311335374 0.0546918314881501 1
+-0.035150278326154444 0.085603549601096771 0.077659853514379512 1
+0.46415485128801681 -0.16255780555360716 -0.050651028921335488 2
+0.099192903888102618 0.048407584042190067 -0.18965444786829586 1
+-0.13000412853418764 -0.36299846990813228 -0.15178608549358752 1
+0.17459221562564242 -0.06392336467034504 -0.28165033671196693 1
+0.41129153200093671 -0.17107995715116933 0.21247928509905528 2
+0.41493340733122097 0.046065397327095713 0.26516695791897971 2
+0.31510700428231037 0.1330607061829582 -0.3584498307402415 2
+-0.39744303830165573 -0.035047204920095415 -0.11884051728647897 1
+-0.037935830376490232 0.25699347244871928 0.42226421765882671 2
+Triangles
+694
+166 189 20 2
+138 300 297 2
+674 103 753 2
+264 104 234 2
+59 172 163 2
+151 278 7 2
+7 193 151 2
+122 55 192 2
+152 295 1 2
+268 241 48 2
+241 268 94 2
+70 222 250 2
+222 70 299 2
+231 95 301 2
+145 607 608 2
+94 650 625 2
+263 217 87 2
+131 60 189 2
+745 270 151 2
+42 674 125 2
+242 52 152 2
+234 264 77 2
+201 26 223 2
+71 211 206 2
+206 211 16 2
+533 233 52 2
+21 577 674 2
+225 219 105 2
+75 156 203 2
+156 235 203 2
+96 202 44 2
+29 116 251 2
+745 278 151 2
+131 239 60 2
+178 58 200 2
+10 91 133 2
+233 533 99 2
+99 292 233 2
+58 209 293 2
+249 137 40 2
+169 11 150 2
+120 230 24 2
+88 177 67 2
+269 73 197 2
+189 166 51 2
+51 131 189 2
+110 183 52 2
+164 176 93 2
+288 176 164 2
+128 155 45 2
+727 185 107 2
+160 298 18 2
+108 21 245 2
+17 116 173 2
+150 11 215 2
+54 180 122 2
+61 147 160 2
+157 110 5 2
+126 143 8 2
+82 635 236 2
+160 147 298 2
+135 146 19 2
+55 122 208 2
+233 110 52 2
+12 286 174 2
+247 299 70 2
+34 265 287 2
+255 141 35 2
+146 136 19 2
+220 85 203 2
+67 89 216 2
+217 92 66 2
+118 241 33 2
+109 165 10 2
+217 66 157 2
+117 209 32 2
+150 62 194 2
+251 72 173 2
+207 133 10 2
+534 261 176 2
+104 264 581 2
+37 97 159 2
+19 154 115 2
+262 200 58 2
+96 27 202 2
+252 22 117 2
+161 617 74 2
+303 157 5 2
+19 214 115 2
+237 4 246 2
+193 63 151 2
+175 75 289 2
+296 294 56 2
+165 754 108 2
+2 92 229 2
+37 162 281 2
+212 692 68 2
+267 15 225 2
+155 160 45 2
+146 592 48 2
+127 291 46 2
+248 168 14 2
+32 236 98 2
+110 29 183 2
+134 169 41 2
+117 178 22 2
+170 734 687 2
+91 538 10 2
+78 191 198 2
+169 150 41 2
+180 69 228 2
+187 707 71 2
+92 143 42 2
+146 45 136 2
+63 224 270 2
+144 689 264 2
+222 13 652 2
+141 62 187 2
+278 155 7 2
+7 155 128 2
+161 213 25 2
+128 45 146 2
+160 45 142 2
+81 249 170 2
+119 269 12 2
+155 61 160 2
+225 15 105 2
+23 289 203 2
+203 220 23 2
+72 173 218 2
+215 62 707 2
+263 82 204 2
+689 257 114 2
+184 166 20 2
+251 29 183 2
+181 69 180 2
+24 120 190 2
+100 240 26 2
+126 42 143 2
+216 67 88 2
+197 73 130 2
+240 266 99 2
+266 292 99 2
+96 44 171 2
+50 137 227 2
+4 172 246 2
+138 297 302 2
+41 150 194 2
+152 218 295 2
+215 11 153 2
+38 125 126 2
+126 272 38 2
+236 209 32 2
+102 20 134 2
+115 188 30 2
+109 165 43 2
+43 261 109 2
+191 9 198 2
+581 98 634 2
+192 213 78 2
+263 262 82 2
+83 118 283 2
+163 268 135 2
+107 284 277 2
+154 47 148 2
+136 45 142 2
+12 269 197 2
+4 96 171 2
+167 92 42 2
+120 56 296 2
+172 6 163 2
+84 284 267 2
+261 667 80 2
+194 62 141 2
+56 120 149 2
+91 255 35 2
+80 261 534 2
+251 72 183 2
+10 668 109 2
+159 22 195 2
+203 235 85 2
+652 222 90 2
+258 23 130 2
+276 221 78 2
+196 267 225 2
+210 110 157 2
+86 665 686 2
+165 108 43 2
+108 245 43 2
+608 101 49 2
+201 223 85 2
+141 187 35 2
+41 147 194 2
+201 100 53 2
+85 223 220 2
+33 277 260 2
+176 245 93 2
+166 174 65 2
+61 270 745 2
+50 285 139 2
+275 17 149 2
+54 132 181 2
+686 36 665 2
+142 160 18 2
+92 3 143 2
+186 106 28 2
+72 218 152 2
+189 725 134 2
+681 642 114 2
+82 262 209 2
+148 47 538 2
+100 22 178 2
+227 2 167 2
+68 244 168 2
+121 297 124 2
+161 74 213 2
+28 570 106 2
+163 94 268 2
+224 102 270 2
+211 39 121 2
+253 28 256 2
+260 253 83 2
+172 59 246 2
+59 163 135 2
+755 60 169 2
+9 156 243 2
+30 279 115 2
+31 301 231 2
+238 88 1 2
+217 157 87 2
+87 157 303 2
+214 135 19 2
+281 221 74 2
+221 213 74 2
+287 34 232 2
+98 634 635 2
+29 81 116 2
+84 267 196 2
+130 23 177 2
+291 123 127 2
+34 130 265 2
+111 205 246 2
+100 178 79 2
+79 200 266 2
+92 3 217 2
+100 79 240 2
+282 51 174 2
+174 51 166 2
+112 232 34 2
+11 127 123 2
+185 54 122 2
+27 642 681 2
+81 170 116 2
+54 181 180 2
+41 271 102 2
+102 134 41 2
+63 186 184 2
+122 180 25 2
+114 248 257 2
+35 187 206 2
+252 159 247 2
+119 259 12 2
+83 256 193 2
+261 43 176 2
+285 16 140 2
+158 18 141 2
+142 18 158 2
+57 154 115 2
+105 15 208 2
+179 2 81 2
+257 288 164 2
+164 290 257 2
+178 200 79 2
+127 302 153 2
+302 39 153 2
+66 210 157 2
+182 296 294 2
+107 132 54 2
+176 43 245 2
+175 31 733 2
+139 21 108 2
+169 11 755 2
+44 129 617 2
+82 209 236 2
+245 21 93 2
+47 142 158 2
+179 229 66 2
+189 60 725 2
+125 42 126 2
+254 581 98 2
+12 197 286 2
+69 171 228 2
+171 44 228 2
+89 242 216 2
+76 230 296 2
+131 112 24 2
+490 106 12 2
+180 25 228 2
+121 39 297 2
+53 195 100 2
+136 47 154 2
+254 36 665 2
+87 200 262 2
+200 87 303 2
+73 175 733 2
+136 142 47 2
+202 44 129 2
+115 57 188 2
+123 755 11 2
+147 271 61 2
+57 667 668 2
+185 122 208 2
+234 290 77 2
+20 189 134 2
+48 146 135 2
+175 31 231 2
+132 650 33 2
+275 300 138 2
+138 274 275 2
+279 214 115 2
+96 4 237 2
+170 687 40 2
+195 22 100 2
+193 283 7 2
+83 253 256 2
+193 256 63 2
+210 179 66 2
+199 171 69 2
+149 56 173 2
+50 137 140 2
+247 162 299 2
+224 102 20 2
+97 9 198 2
+19 136 154 2
+153 127 11 2
+687 121 124 2
+298 194 147 2
+303 5 200 2
+15 185 208 2
+227 167 103 2
+53 201 235 2
+266 79 240 2
+202 86 129 2
+55 192 191 2
+159 37 247 2
+149 173 17 2
+105 509 208 2
+78 198 276 2
+192 25 213 2
+24 565 112 2
+186 65 184 2
+186 106 65 2
+181 199 69 2
+181 6 199 2
+196 225 64 2
+744 241 118 2
+150 215 62 2
+103 167 42 2
+224 184 63 2
+129 13 226 2
+2 92 167 2
+88 216 1 2
+243 156 75 2
+215 153 707 2
+237 246 111 2
+221 78 213 2
+119 31 733 2
+490 106 65 2
+116 734 170 2
+277 284 84 2
+182 76 296 2
+107 54 185 2
+177 23 220 2
+177 220 67 2
+290 164 38 2
+164 280 38 2
+159 195 97 2
+238 287 88 2
+217 3 263 2
+81 2 249 2
+140 121 16 2
+179 81 29 2
+22 159 252 2
+10 207 165 2
+725 169 134 2
+30 205 279 2
+205 30 757 2
+184 65 166 2
+300 275 17 2
+131 24 239 2
+253 84 196 2
+71 153 211 2
+93 280 164 2
+297 39 302 2
+117 252 250 2
+28 253 196 2
+299 13 222 2
+108 139 754 2
+210 29 110 2
+252 247 70 2
+27 86 202 2
+509 95 704 2
+231 95 243 2
+52 183 152 2
+183 72 152 2
+64 219 119 2
+231 243 75 2
+235 201 85 2
+143 204 8 2
+143 3 204 2
+210 179 29 2
+32 250 117 2
+26 240 89 2
+175 231 75 2
+286 113 197 2
+248 168 68 2
+100 26 201 2
+192 78 191 2
+122 25 192 2
+692 68 244 2
+211 121 16 2
+128 744 7 2
+211 153 39 2
+262 209 58 2
+283 118 7 2
+4 171 199 2
+4 199 172 2
+187 71 206 2
+178 117 293 2
+199 6 172 2
+47 158 538 2
+270 102 61 2
+102 271 61 2
+581 36 264 2
+42 103 674 2
+749 121 687 2
+15 185 727 2
+162 13 299 2
+650 625 132 2
+113 282 717 2
+120 230 296 2
+106 259 12 2
+203 75 289 2
+133 91 579 2
+254 665 90 2
+252 70 250 2
+119 733 269 2
+733 73 269 2
+3 263 204 2
+717 273 113 2
+273 717 112 2
+99 240 89 2
+168 80 534 2
+137 2 227 2
+57 148 154 2
+263 87 262 2
+650 94 241 2
+258 289 175 2
+76 238 182 2
+238 1 182 2
+27 96 212 2
+138 302 127 2
+127 274 138 2
+260 83 118 2
+118 33 260 2
+220 223 67 2
+223 89 67 2
+26 89 223 2
+153 707 71 2
+184 20 224 2
+107 132 33 2
+509 208 55 2
+257 248 14 2
+64 225 219 2
+717 282 51 2
+44 161 228 2
+161 25 228 2
+141 158 255 2
+509 704 191 2
+92 229 66 2
+77 689 257 2
+229 2 179 2
+239 190 123 2
+190 291 123 2
+21 93 577 2
+63 270 151 2
+226 281 74 2
+91 158 255 2
+105 301 219 2
+73 175 258 2
+239 24 190 2
+60 239 123 2
+242 152 1 2
+216 242 1 2
+73 258 130 2
+249 2 137 2
+250 90 32 2
+222 90 250 2
+574 126 8 2
+272 126 574 2
+251 116 173 2
+689 77 264 2
+254 98 32 2
+90 254 32 2
+40 249 170 2
+293 209 117 2
+273 130 197 2
+256 28 186 2
+256 186 63 2
+617 129 226 2
+260 84 253 2
+148 10 668 2
+285 16 145 2
+104 234 272 2
+744 592 128 2
+259 64 119 2
+135 268 48 2
+144 264 36 2
+277 260 84 2
+196 64 570 2
+88 265 177 2
+265 130 177 2
+293 58 178 2
+674 753 21 2
+76 232 287 2
+76 287 238 2
+198 37 276 2
+273 130 34 2
+273 34 112 2
+127 46 274 2
+120 149 46 2
+149 274 46 2
+301 31 219 2
+219 31 119 2
+538 148 10 2
+274 149 275 2
+37 97 198 2
+139 145 285 2
+107 277 33 2
+214 59 135 2
+59 214 279 2
+246 59 205 2
+205 59 279 2
+258 289 23 2
+281 37 221 2
+37 276 221 2
+37 162 247 2
+124 297 300 2
+13 162 226 2
+226 162 281 2
+283 83 193 2
+284 15 267 2
+50 285 140 2
+286 282 174 2
+286 113 282 2
+197 273 113 2
+287 265 88 2
+257 14 288 2
+234 38 272 2
+290 38 234 2
+77 257 290 2
+46 291 190 2
+190 120 46 2
+200 5 266 2
+5 292 266 2
+292 5 233 2
+233 5 110 2
+56 294 218 2
+218 173 56 2
+744 48 592 2
+298 194 18 2
+18 194 141 2
+570 196 28 2
+147 271 41 2
+295 218 294 2
+294 182 295 2
+295 182 1 2
+93 577 280 2
+12 174 490 2
+634 581 104 2
+607 101 608 2
+86 129 652 2
+732 188 80 2
+717 112 131 2
+82 635 204 2
+95 704 243 2
+704 9 243 2
+104 272 574 2
+230 565 76 2
+125 577 674 2
+668 667 109 2
+174 490 65 2
+21 139 753 2
+288 534 176 2
+8 634 574 2
+634 104 574 2
+248 642 114 2
+144 27 686 2
+107 727 284 2
+30 757 244 2
+33 650 241 2
+49 608 139 2
+133 207 49 2
+509 105 301 2
+533 52 242 2
+97 9 730 2
+635 634 8 2
+116 17 734 2
+236 635 98 2
+592 128 146 2
+608 145 139 2
+89 533 242 2
+686 144 36 2
+48 744 241 2
+80 732 168 2
+123 60 755 2
+191 509 55 2
+204 635 8 2
+730 156 53 2
+565 232 76 2
+99 533 89 2
+132 625 181 2
+625 6 181 2
+744 118 7 2
+111 205 757 2
+565 232 112 2
+625 163 94 2
+95 509 301 2
+137 40 140 2
+74 226 617 2
+652 665 90 2
+86 665 652 2
+35 206 579 2
+579 91 35 2
+30 732 244 2
+244 732 168 2
+44 617 161 2
+248 68 642 2
+106 570 259 2
+570 64 259 2
+704 191 9 2
+254 36 581 2
+288 534 14 2
+16 607 145 2
+96 237 212 2
+237 692 212 2
+206 579 101 2
+577 125 38 2
+38 280 577 2
+163 6 625 2
+749 40 140 2
+206 101 607 2
+206 607 16 2
+667 188 80 2
+129 652 13 2
+27 681 144 2
+757 111 244 2
+53 156 235 2
+57 188 667 2
+538 158 91 2
+707 62 187 2
+207 49 754 2
+121 749 140 2
+101 579 133 2
+101 133 49 2
+169 60 725 2
+686 27 86 2
+14 534 168 2
+732 30 188 2
+730 9 156 2
+24 565 230 2
+692 111 244 2
+50 139 753 2
+745 278 155 2
+753 227 103 2
+261 667 109 2
+687 749 40 2
+284 727 15 2
+212 68 642 2
+27 642 212 2
+668 148 57 2
+681 689 144 2
+689 114 681 2
+111 237 692 2
+51 717 131 2
+195 730 97 2
+53 195 730 2
+687 734 124 2
+734 124 300 2
+734 300 17 2
+61 745 155 2
+753 227 50 2
+165 207 754 2
+49 139 754 2
+Tetrahedra
+3586
+669 309 483 629 1
+510 420 537 286 1
+475 372 495 322 1
+103 753 515 674 1
+264 234 104 442 1
+531 357 748 454 1
+314 382 454 596 1
+122 192 55 474 1
+328 495 611 372 1
+425 357 314 709 1
+594 359 586 470 1
+145 591 139 49 1
+407 231 95 301 1
+591 145 607 608 1
+94 625 650 572 1
+497 745 270 151 1
+674 125 601 42 1
+335 702 756 400 1
+594 653 541 343 1
+393 514 654 349 1
+440 688 518 368 1
+486 401 21 108 1
+37 428 708 670 1
+477 586 354 444 1
+352 455 615 311 1
+405 638 697 375 1
+586 354 461 648 1
+624 531 347 416 1
+680 428 602 350 1
+21 674 577 636 1
+219 105 518 225 1
+406 410 311 447 1
+346 580 481 557 1
+688 726 518 368 1
+347 492 484 445 1
+646 502 701 345 1
+723 364 526 619 1
+375 4 508 199 1
+397 459 362 559 1
+232 663 112 510 1
+58 293 209 506 1
+610 91 538 398 1
+137 436 411 249 1
+583 555 341 637 1
+642 432 671 358 1
+230 604 120 24 1
+409 623 310 613 1
+376 467 394 557 1
+428 670 247 602 1
+386 307 694 492 1
+717 51 537 419 1
+486 754 373 108 1
+727 107 185 363 1
+308 662 377 414 1
+433 682 316 637 1
+728 545 416 317 1
+17 173 116 459 1
+337 617 472 542 1
+596 329 454 357 1
+305 678 710 448 1
+736 431 647 374 1
+519 455 371 352 1
+635 435 82 236 1
+122 474 55 208 1
+143 412 569 8 1
+503 324 539 482 1
+734 362 459 170 1
+583 660 653 525 1
+523 344 448 587 1
+580 336 698 419 1
+85 513 220 203 1
+202 129 472 452 1
+391 324 503 482 1
+109 10 165 655 1
+209 32 564 117 1
+447 356 410 541 1
+459 251 72 173 1
+133 655 207 10 1
+264 442 104 581 1
+438 378 341 627 1
+97 680 37 159 1
+347 694 484 492 1
+560 396 45 511 1
+680 556 428 417 1
+491 263 3 383 1
+683 617 472 337 1
+495 464 383 322 1
+317 728 545 619 1
+434 252 22 117 1
+261 351 588 43 1
+578 335 756 400 1
+664 19 214 115 1
+25 656 366 563 1
+330 473 380 549 1
+475 522 747 310 1
+472 337 415 543 1
+624 357 531 416 1
+427 558 352 403 1
+429 365 389 590 1
+454 375 543 329 1
+2 229 92 566 1
+544 203 513 235 1
+584 450 456 30 1
+37 281 162 708 1
+384 224 524 426 1
+678 305 477 448 1
+709 677 314 430 1
+471 346 663 658 1
+644 146 592 48 1
+387 32 236 98 1
+636 321 520 457 1
+447 359 470 449 1
+514 19 136 146 1
+734 362 170 687 1
+651 437 481 342 1
+539 91 538 10 1
+191 198 400 78 1
+45 511 396 458 1
+9 704 400 756 1
+478 477 354 369 1
+41 194 738 476 1
+587 448 313 386 1
+726 390 632 684 1
+755 467 376 169 1
+528 591 608 49 1
+712 477 609 369 1
+160 142 45 511 1
+425 525 709 430 1
+245 108 21 401 1
+135 644 446 546 1
+658 481 346 419 1
+440 267 616 225 1
+705 639 35 579 1
+451 299 553 222 1
+582 487 324 343 1
+627 356 378 583 1
+427 399 352 318 1
+215 707 62 593 1
+637 627 469 341 1
+689 114 257 671 1
+489 543 416 337 1
+548 381 693 313 1
+251 183 29 716 1
+347 531 561 696 1
+560 610 393 354 1
+363 600 688 319 1
+516 106 695 28 1
+344 531 748 494 1
+568 703 414 442 1
+448 305 477 609 1
+459 149 562 479 1
+747 623 495 310 1
+551 191 535 55 1
+542 451 337 74 1
+541 660 343 653 1
+67 380 216 88 1
+45 396 128 458 1
+602 670 247 70 1
+396 514 136 146 1
+511 648 160 18 1
+382 456 757 244 1
+353 520 430 321 1
+230 460 604 346 1
+669 679 373 582 1
+378 438 462 306 1
+441 140 676 16 1
+313 384 524 426 1
+472 683 337 489 1
+380 630 549 67 1
+66 217 500 157 1
+695 628 338 488 1
+542 418 213 535 1
+527 363 501 339 1
+131 419 468 51 1
+537 729 628 420 1
+539 668 482 109 1
+9 400 191 198 1
+98 633 581 634 1
+659 279 205 450 1
+680 428 37 159 1
+530 234 442 457 1
+513 330 549 589 1
+441 411 379 483 1
+708 281 162 451 1
+434 595 117 22 1
+5 110 536 157 1
+731 540 730 53 1
+550 575 446 349 1
+83 283 118 408 1
+39 371 465 519 1
+713 361 554 66 1
+485 566 167 404 1
+545 723 526 619 1
+56 460 120 296 1
+726 492 688 326 1
+696 507 319 466 1
+376 336 467 557 1
+510 112 273 113 1
+261 80 667 391 1
+471 658 663 389 1
+365 666 589 661 1
+56 149 120 460 1
+526 475 322 434 1
+657 606 643 471 1
+594 318 586 444 1
+668 539 10 109 1
+703 358 671 442 1
+618 374 540 731 1
+550 349 450 391 1
+470 359 327 449 1
+614 334 496 529 1
+752 604 460 346 1
+652 90 222 553 1
+418 276 221 78 1
+453 598 527 339 1
+536 554 413 500 1
+323 510 419 389 1
+239 60 131 698 1
+436 2 566 229 1
+718 429 694 307 1
+86 686 665 414 1
+440 319 363 688 1
+638 405 527 598 1
+382 432 596 456 1
+653 304 523 587 1
+528 608 101 49 1
+415 504 171 44 1
+41 194 147 738 1
+500 554 361 66 1
+463 321 636 457 1
+386 313 507 448 1
+356 740 433 583 1
+540 201 100 53 1
+412 646 345 722 1
+384 449 548 336 1
+174 381 166 65 1
+701 520 316 502 1
+523 402 710 748 1
+497 61 270 745 1
+379 50 285 139 1
+519 71 707 721 1
+334 643 630 443 1
+314 402 357 454 1
+707 455 519 352 1
+431 26 240 100 1
+464 262 506 58 1
+17 149 459 275 1
+545 720 317 418 1
+397 116 29 716 1
+666 34 273 510 1
+495 573 328 341 1
+516 186 106 28 1
+136 560 45 142 1
+437 599 462 315 1
+530 321 358 685 1
+467 725 547 493 1
+720 416 317 499 1
+518 551 407 326 1
+359 523 448 587 1
+210 554 29 413 1
+456 432 244 382 1
+313 649 381 524 1
+313 359 521 448 1
+501 727 440 363 1
+498 706 693 386 1
+409 522 360 694 1
+496 1 216 380 1
+675 369 576 333 1
+45 511 458 155 1
+576 369 458 609 1
+261 567 482 351 1
+548 384 649 313 1
+755 169 60 467 1
+321 568 358 425 1
+461 91 610 398 1
+716 183 72 251 1
+346 481 437 557 1
+412 641 569 722 1
+539 503 482 668 1
+339 572 453 719 1
+447 523 359 587 1
+747 573 495 341 1
+287 232 34 389 1
+334 736 443 388 1
+446 325 659 546 1
+378 437 447 406 1
+518 726 632 368 1
+627 306 740 469 1
+400 326 535 499 1
+453 94 625 546 1
+481 342 429 651 1
+685 596 395 314 1
+640 246 382 508 1
+485 753 515 103 1
+123 715 291 127 1
+672 737 423 507 1
+518 326 407 726 1
+210 536 110 157 1
+644 367 446 546 1
+658 481 323 429 1
+632 518 368 225 1
+500 571 328 361 1
+575 402 305 487 1
+501 440 284 620 1
+111 246 205 382 1
+324 439 373 582 1
+187 141 62 399 1
+342 735 718 651 1
+569 316 383 641 1
+380 177 88 67 1
+92 217 3 491 1
+317 416 728 499 1
+381 649 184 524 1
+419 510 112 663 1
+232 510 112 34 1
+644 592 458 367 1
+455 11 127 123 1
+701 502 412 345 1
+426 497 270 151 1
+434 595 506 564 1
+710 494 719 466 1
+39 465 121 211 1
+374 621 431 201 1
+526 364 723 555 1
+259 12 269 119 1
+401 679 309 629 1
+148 503 539 668 1
+484 512 409 335 1
+154 584 57 115 1
+647 374 540 618 1
+314 677 391 456 1
+5 464 536 739 1
+296 392 182 294 1
+31 390 175 733 1
+619 475 526 434 1
+639 607 101 591 1
+11 755 376 169 1
+669 309 629 401 1
+658 389 590 429 1
+544 374 335 731 1
+620 423 724 672 1
+675 426 524 385 1
+399 318 427 461 1
+630 216 89 67 1
+313 524 381 693 1
+683 617 337 542 1
+694 512 360 409 1
+31 632 390 733 1
+394 455 715 123 1
+229 554 179 66 1
+451 299 670 553 1
+483 379 699 669 1
+590 473 380 330 1
+189 468 166 51 1
+626 568 414 345 1
+604 190 120 24 1
+698 547 189 468 1
+230 471 76 296 1
+398 427 705 639 1
+497 738 271 147 1
+706 342 718 481 1
+428 708 670 451 1
+484 407 326 726 1
+517 458 367 333 1
+618 475 647 310 1
+626 308 422 526 1
+25 228 656 180 1
+540 731 374 53 1
+425 596 314 357 1
+66 500 713 361 1
+440 616 267 620 1
+633 254 36 665 1
+536 233 292 424 1
+399 141 62 194 1
+482 324 373 582 1
+580 336 481 557 1
+309 699 669 483 1
+505 100 195 540 1
+418 78 213 535 1
+440 363 185 474 1
+175 390 73 733 1
+393 712 560 354 1
+586 648 461 399 1
+426 151 746 576 1
+249 559 81 436 1
+465 622 297 124 1
+318 639 427 398 1
+202 129 44 472 1
+495 438 623 341 1
+685 463 321 679 1
+755 11 455 123 1
+634 345 646 722 1
+497 147 271 61 1
+667 503 57 668 1
+102 738 271 497 1
+290 77 442 234 1
+437 447 311 750 1
+557 750 437 311 1
+310 522 623 409 1
+150 593 476 376 1
+486 669 629 401 1
+437 447 750 342 1
+159 247 602 428 1
+650 501 132 33 1
+748 357 402 454 1
+307 429 512 743 1
+408 532 724 83 1
+143 92 3 491 1
+398 539 655 10 1
+387 690 626 526 1
+537 381 166 174 1
+12 420 286 628 1
+687 559 170 40 1
+380 630 388 549 1
+513 23 220 203 1
+507 338 423 385 1
+86 414 377 452 1
+481 750 706 342 1
+532 193 283 7 1
+552 331 379 483 1
+206 639 35 427 1
+672 319 507 466 1
+554 210 179 66 1
+74 213 542 418 1
+597 479 362 332 1
+643 380 657 496 1
+431 79 505 100 1
+548 649 381 313 1
+325 494 719 710 1
+413 183 29 110 1
+537 729 420 323 1
+165 655 373 482 1
+16 140 285 441 1
+552 50 137 140 1
+613 623 310 443 1
+504 96 171 44 1
+501 107 440 727 1
+386 561 347 694 1
+102 384 224 20 1
+597 455 371 394 1
+517 744 592 458 1
+176 463 588 93 1
+684 737 695 488 1
+78 556 535 418 1
+369 160 61 511 1
+153 11 127 455 1
+366 700 499 326 1
+390 661 73 733 1
+587 344 448 386 1
+194 738 298 147 1
+18 158 142 648 1
+498 429 718 307 1
+137 436 552 411 1
+714 610 393 539 1
+217 263 500 87 1
+79 751 266 240 1
+657 238 380 606 1
+610 538 393 539 1
+308 414 377 690 1
+382 246 205 508 1
+352 519 721 403 1
+333 532 517 576 1
+540 100 195 53 1
+674 125 636 601 1
+644 664 446 514 1
+444 712 654 393 1
+457 502 701 646 1
+747 526 475 322 1
+546 453 508 163 1
+611 443 623 438 1
+486 528 591 373 1
+509 208 518 105 1
+442 321 457 646 1
+198 276 556 78 1
+356 483 699 541 1
+337 418 542 720 1
+736 374 388 431 1
+565 419 24 112 1
+588 463 401 93 1
+638 508 199 375 1
+359 447 653 523 1
+37 708 162 670 1
+679 520 309 629 1
+681 358 671 703 1
+359 586 678 594 1
+311 371 615 597 1
+524 224 184 63 1
+440 727 185 363 1
+536 303 464 5 1
+137 40 249 411 1
+328 464 383 495 1
+516 693 524 385 1
+406 356 740 480 1
+715 302 622 138 1
+735 304 561 522 1
+688 600 445 319 1
+514 349 503 664 1
+265 666 130 473 1
+21 674 636 515 1
+347 484 499 445 1
+435 641 626 345 1
+487 550 575 402 1
+490 65 106 516 1
+327 376 493 476 1
+261 482 109 43 1
+314 395 567 456 1
+734 459 116 170 1
+648 461 610 354 1
+399 586 478 327 1
+454 494 312 598 1
+182 296 76 471 1
+462 332 437 406 1
+112 510 717 113 1
+640 4 375 697 1
+217 263 3 491 1
+614 392 370 562 1
+359 653 594 523 1
+447 437 311 406 1
+702 499 400 317 1
+417 556 702 400 1
+676 140 121 16 1
+405 54 180 181 1
+546 508 659 163 1
+548 468 381 649 1
+98 254 581 633 1
+682 566 404 713 1
+376 11 455 755 1
+473 130 589 666 1
+478 147 497 61 1
+434 564 673 117 1
+643 380 630 443 1
+405 563 527 363 1
+376 311 352 470 1
+374 621 201 235 1
+375 405 638 598 1
+678 523 359 594 1
+578 335 417 731 1
+575 710 305 402 1
+623 651 360 378 1
+246 4 172 508 1
+39 519 302 715 1
+397 370 605 459 1
+471 663 76 606 1
+382 329 596 432 1
+502 316 701 641 1
+139 754 486 108 1
+551 509 95 704 1
+489 545 723 690 1
+747 341 495 623 1
+732 168 456 567 1
+507 693 675 313 1
+469 573 361 316 1
+564 32 250 117 1
+521 359 477 448 1
+321 502 568 425 1
+485 355 404 601 1
+449 706 587 359 1
+501 527 132 107 1
+422 573 747 341 1
+724 385 516 186 1
+286 197 113 420 1
+549 67 220 177 1
+527 650 625 339 1
+168 68 456 248 1
+211 16 206 403 1
+552 50 285 379 1
+658 389 471 315 1
+688 445 700 326 1
+486 754 528 373 1
+68 432 692 244 1
+388 736 443 613 1
+529 242 152 52 1
+631 743 307 512 1
+542 213 161 25 1
+45 146 128 396 1
+523 561 344 587 1
+494 748 402 454 1
+81 397 436 559 1
+369 478 497 61 1
+176 351 43 588 1
+664 279 450 115 1
+493 41 738 476 1
+256 63 193 746 1
+400 535 418 499 1
+600 696 344 445 1
+137 50 552 227 1
+494 466 710 344 1
+41 134 493 102 1
+12 488 269 420 1
+353 583 555 525 1
+15 518 185 208 1
+370 529 614 716 1
+209 58 506 262 1
+457 701 412 646 1
+141 187 427 399 1
+517 283 118 7 1
+409 364 475 522 1
+536 413 691 328 1
+634 633 722 98 1
+586 359 327 470 1
+418 451 542 74 1
+636 457 125 38 1
+103 674 485 42 1
+15 727 185 518 1
+479 362 332 459 1
+604 467 190 580 1
+102 493 497 384 1
+539 655 10 109 1
+138 300 297 622 1
+385 693 338 516 1
+406 597 362 332 1
+318 558 427 639 1
+631 645 407 390 1
+397 370 742 605 1
+113 717 282 537 1
+333 408 532 672 1
+507 696 319 737 1
+75 544 203 289 1
+440 225 616 518 1
+210 413 110 536 1
+398 91 539 10 1
+418 213 221 74 1
+414 254 665 90 1
+685 395 677 314 1
+483 356 433 309 1
+366 535 474 326 1
+273 112 717 113 1
+500 464 328 536 1
+99 89 240 751 1
+363 54 527 107 1
+662 452 377 414 1
+623 522 360 409 1
+731 235 374 53 1
+503 57 148 154 1
+602 505 434 22 1
+576 609 458 333 1
+393 538 47 148 1
+47 393 148 154 1
+94 241 572 650 1
+418 556 428 708 1
+289 175 421 258 1
+601 126 412 143 1
+210 413 29 110 1
+245 176 43 93 1
+314 567 677 456 1
+462 643 370 334 1
+643 462 443 438 1
+513 621 374 235 1
+174 381 490 628 1
+552 436 566 331 1
+687 465 559 40 1
+74 213 161 542 1
+179 554 29 210 1
+350 647 540 618 1
+237 692 612 640 1
+576 283 517 7 1
+352 376 455 311 1
+333 517 466 367 1
+648 461 158 610 1
+621 85 201 235 1
+416 357 531 312 1
+153 71 707 519 1
+370 334 614 529 1
+501 107 284 440 1
+549 330 513 374 1
+501 107 132 33 1
+260 408 118 33 1
+479 715 622 138 1
+717 51 282 537 1
+429 743 590 330 1
+186 524 184 63 1
+555 502 568 308 1
+230 460 120 604 1
+81 116 397 559 1
+548 313 706 449 1
+727 440 185 518 1
+421 175 390 258 1
+684 368 737 488 1
+640 697 375 504 1
+572 408 339 501 1
+103 485 167 42 1
+614 152 72 183 1
+461 141 158 255 1
+551 509 704 191 1
+613 360 623 443 1
+625 339 650 572 1
+380 1 216 88 1
+66 217 491 500 1
+202 504 472 44 1
+536 210 413 554 1
+39 124 297 121 1
+146 514 644 396 1
+573 383 491 316 1
+515 636 355 674 1
+489 723 308 690 1
+21 577 93 401 1
+492 445 688 326 1
+600 445 531 312 1
+168 456 395 248 1
+421 743 512 330 1
+15 225 518 105 1
+396 514 644 654 1
+521 609 477 369 1
+361 573 328 500 1
+751 26 240 431 1
+288 741 176 164 1
+439 558 348 699 1
+331 411 436 480 1
+230 296 120 460 1
+478 369 511 160 1
+569 641 383 435 1
+151 745 576 426 1
+296 56 392 294 1
+185 563 122 474 1
+328 571 341 573 1
+485 682 404 355 1
+439 318 348 558 1
+541 439 318 594 1
+476 470 399 327 1
+157 217 500 87 1
+380 216 630 67 1
+581 633 36 264 1
+462 370 605 742 1
+110 52 529 183 1
+501 363 440 319 1
+200 262 87 464 1
+249 2 436 81 1
+508 4 375 640 1
+99 751 585 533 1
+382 456 450 757 1
+487 550 349 575 1
+626 422 641 322 1
+313 426 524 675 1
+655 528 373 439 1
+575 446 367 719 1
+143 126 412 8 1
+663 658 419 389 1
+45 458 128 155 1
+45 396 136 146 1
+645 175 390 421 1
+485 552 629 379 1
+442 321 358 530 1
+662 596 425 357 1
+460 332 392 562 1
+143 92 404 42 1
+493 169 134 41 1
+590 360 443 651 1
+619 364 526 475 1
+527 54 132 107 1
+519 211 71 403 1
+450 30 205 757 1
+716 614 72 183 1
+564 387 435 322 1
+188 115 57 584 1
+684 498 386 492 1
+511 160 61 155 1
+569 641 435 722 1
+23 513 549 589 1
+81 116 559 170 1
+246 4 640 237 1
+321 685 430 425 1
+185 563 474 363 1
+652 13 377 553 1
+167 404 92 42 1
+633 442 646 345 1
+695 106 570 28 1
+102 493 738 497 1
+494 600 531 312 1
+536 691 424 328 1
+31 390 645 175 1
+318 399 352 470 1
+352 615 519 403 1
+31 632 645 390 1
+530 463 164 38 1
+359 678 477 448 1
+682 331 629 485 1
+573 316 469 637 1
+40 170 249 559 1
+318 439 444 594 1
+649 20 166 184 1
+316 569 491 412 1
+194 478 738 476 1
+527 501 132 650 1
+397 605 306 362 1
+741 567 176 351 1
+430 685 677 314 1
+564 293 209 117 1
+603 740 480 306 1
+407 509 95 551 1
+130 666 273 197 1
+411 749 676 140 1
+324 503 539 393 1
+397 459 605 362 1
+542 617 472 415 1
+333 517 458 576 1
+409 512 613 335 1
+694 522 561 347 1
+23 177 130 589 1
+16 441 285 145 1
+729 693 498 706 1
+148 539 10 668 1
+526 555 308 422 1
+627 583 378 341 1
+369 478 61 160 1
+744 128 592 458 1
+413 691 742 529 1
+489 555 308 723 1
+560 511 45 142 1
+336 548 468 419 1
+206 71 403 721 1
+511 369 458 155 1
+456 432 596 395 1
+350 428 602 619 1
+516 65 106 186 1
+245 588 401 93 1
+481 323 548 419 1
+589 473 549 177 1
+81 397 29 179 1
+308 414 690 626 1
+133 91 398 10 1
+410 356 406 320 1
+277 84 260 620 1
+102 61 270 497 1
+292 233 99 424 1
+113 510 717 537 1
+659 508 59 163 1
+536 210 554 500 1
+461 610 714 398 1
+532 283 576 7 1
+584 115 450 30 1
+382 659 205 450 1
+636 463 457 38 1
+440 688 363 474 1
+180 69 228 656 1
+153 39 211 519 1
+372 505 595 647 1
+534 168 567 395 1
+452 703 414 358 1
+483 309 433 629 1
+636 355 601 457 1
+374 201 540 53 1
+382 596 314 456 1
+753 21 515 674 1
+544 335 513 421 1
+401 629 515 355 1
+198 276 37 556 1
+197 666 510 661 1
+725 547 189 698 1
+461 158 91 255 1
+102 497 271 61 1
+336 481 548 419 1
+672 319 737 507 1
+666 365 510 661 1
+469 438 341 627 1
+558 320 615 403 1
+539 538 148 10 1
+39 715 302 371 1
+372 611 647 310 1
+424 736 585 751 1
+122 535 474 563 1
+165 373 754 108 1
+746 426 524 63 1
+5 200 464 739 1
+385 426 524 746 1
+523 304 561 587 1
+141 705 35 255 1
+9 198 578 400 1
+578 198 556 400 1
+658 323 481 419 1
+587 718 706 342 1
+498 706 386 718 1
+498 718 386 307 1
+745 497 426 151 1
+702 728 499 317 1
+153 715 302 519 1
+314 550 402 454 1
+112 24 131 419 1
+484 728 499 702 1
+149 562 173 459 1
+466 448 710 344 1
+340 602 434 670 1
+617 161 542 74 1
+530 685 358 395 1
+487 349 550 391 1
+499 531 347 445 1
+552 436 331 411 1
+535 78 213 192 1
+405 543 312 375 1
+388 89 585 751 1
+474 363 700 326 1
+506 434 322 595 1
+566 404 92 167 1
+555 522 747 364 1
+395 168 248 14 1
+672 385 423 724 1
+365 323 389 510 1
+744 592 48 517 1
+249 170 81 559 1
+447 587 359 342 1
+695 570 196 28 1
+113 510 537 286 1
+391 584 349 450 1
+232 389 663 510 1
+648 610 142 354 1
+378 462 406 306 1
+606 238 287 76 1
+396 644 458 367 1
+314 550 454 382 1
+376 449 470 327 1
+693 706 507 386 1
+469 438 627 306 1
+375 4 199 697 1
+284 620 267 84 1
+702 556 317 400 1
+33 277 408 501 1
+495 310 611 372 1
+472 377 683 489 1
+493 725 547 134 1
+25 542 366 656 1
+311 455 597 394 1
+590 365 389 473 1
+378 462 437 406 1
+362 559 480 397 1
+591 607 101 608 1
+266 79 200 739 1
+489 329 662 357 1
+602 247 252 70 1
+590 365 473 330 1
+550 659 382 450 1
+391 677 487 324 1
+60 123 467 239 1
+573 316 422 641 1
+542 161 415 228 1
+415 161 44 228 1
+70 252 673 250 1
+354 477 712 369 1
+394 467 604 580 1
+90 250 387 32 1
+90 387 254 32 1
+118 260 83 408 1
+465 676 121 211 1
+594 439 444 343 1
+165 10 207 655 1
+684 695 338 488 1
+203 513 289 421 1
+157 210 536 500 1
+336 467 547 493 1
+318 558 352 427 1
+13 377 553 451 1
+487 677 314 709 1
+497 327 478 369 1
+500 210 554 66 1
+87 262 263 383 1
+618 475 350 647 1
+261 588 482 43 1
+626 641 435 322 1
+459 173 116 251 1
+165 482 588 43 1
+179 397 29 554 1
+619 428 545 317 1
+680 505 195 540 1
+527 501 650 339 1
+706 313 587 359 1
+280 164 463 38 1
+600 445 344 531 1
+335 421 512 513 1
+713 500 491 361 1
+563 700 363 405 1
+674 601 355 485 1
+581 442 646 633 1
+214 279 664 115 1
+258 589 73 661 1
+129 652 377 86 1
+591 669 379 486 1
+739 372 328 424 1
+420 488 338 628 1
+131 468 189 51 1
+77 264 442 234 1
+465 124 121 687 1
+596 358 395 432 1
+702 326 400 499 1
+633 665 36 414 1
+527 132 625 650 1
+686 144 703 452 1
+42 601 404 143 1
+33 408 572 501 1
+530 457 463 38 1
+583 522 341 555 1
+368 492 688 726 1
+377 690 553 340 1
+591 379 139 486 1
+349 503 393 514 1
+168 68 244 456 1
+310 647 618 374 1
+474 700 363 563 1
+369 155 745 576 1
+655 324 373 482 1
+142 354 511 648 1
+437 599 332 462 1
+717 419 537 510 1
+438 462 623 378 1
+615 455 371 311 1
+378 735 623 522 1
+1 182 657 295 1
+532 283 517 576 1
+11 593 150 376 1
+648 158 142 610 1
+79 431 751 240 1
+411 331 552 483 1
+464 262 87 383 1
+521 477 327 369 1
+582 351 677 430 1
+356 627 740 583 1
+443 549 388 330 1
+349 514 446 664 1
+401 520 679 355 1
+185 54 122 563 1
+485 404 167 42 1
+580 419 663 346 1
+638 69 199 181 1
+102 493 271 738 1
+719 572 453 546 1
+684 737 338 695 1
+667 482 503 668 1
+487 582 677 709 1
+486 629 515 401 1
+176 164 463 93 1
+135 546 268 644 1
+382 508 375 640 1
+530 671 395 358 1
+561 522 304 624 1
+662 596 358 425 1
+378 583 522 341 1
+500 464 383 328 1
+629 433 331 682 1
+425 709 314 430 1
+594 586 678 444 1
+5 739 292 266 1
+345 633 722 634 1
+382 375 454 329 1
+630 380 388 443 1
+578 156 243 544 1
+227 566 2 167 1
+381 184 166 65 1
+142 511 160 18 1
+456 68 244 432 1
+404 713 92 491 1
+512 335 631 421 1
+403 320 441 558 1
+410 594 470 447 1
+667 482 668 109 1
+537 381 729 323 1
+545 526 340 619 1
+540 431 647 505 1
+320 362 465 480 1
+658 346 481 437 1
+174 65 490 381 1
+495 372 464 322 1
+576 517 458 7 1
+688 326 518 726 1
+557 467 580 336 1
+723 340 690 545 1
+378 522 623 341 1
+473 177 130 265 1
+466 517 572 367 1
+426 151 270 63 1
+136 19 514 154 1
+512 360 590 429 1
+572 241 408 33 1
+740 637 433 583 1
+658 437 481 651 1
+396 592 128 458 1
+306 627 406 378 1
+290 530 164 38 1
+471 663 606 389 1
+557 376 336 449 1
+449 706 342 587 1
+642 114 432 248 1
+611 310 443 736 1
+593 399 352 721 1
+109 482 165 43 1
+176 588 43 93 1
+483 356 480 740 1
+479 604 460 752 1
+328 372 464 495 1
+449 587 342 359 1
+712 305 396 609 1
+383 641 322 435 1
+187 399 593 721 1
+198 556 400 78 1
+495 611 310 623 1
+554 716 29 413 1
+406 356 480 320 1
+594 359 470 447 1
+517 592 48 367 1
+543 312 357 416 1
+398 318 348 439 1
+666 510 273 197 1
+64 616 196 225 1
+463 530 321 457 1
+681 358 703 452 1
+391 567 677 351 1
+89 388 223 26 1
+202 96 612 504 1
+550 325 454 382 1
+409 522 694 347 1
+428 556 37 708 1
+290 77 530 442 1
+539 324 655 482 1
+614 370 72 562 1
+539 482 655 109 1
+408 532 517 333 1
+89 67 223 630 1
+402 325 550 575 1
+13 299 162 451 1
+506 564 435 322 1
+560 514 393 154 1
+435 209 82 236 1
+591 348 379 669 1
+432 671 358 395 1
+254 32 387 98 1
+136 396 560 514 1
+550 325 402 454 1
+438 742 462 306 1
+563 405 656 366 1
+123 394 291 715 1
+644 367 546 48 1
+356 309 483 541 1
+260 408 724 83 1
+27 452 144 686 1
+606 88 287 238 1
+256 746 186 63 1
+516 184 65 186 1
+386 344 696 561 1
+154 19 664 115 1
+194 478 298 738 1
+511 354 478 648 1
+185 54 363 107 1
+403 558 441 348 1
+459 562 332 479 1
+490 65 516 381 1
+512 360 330 590 1
+652 377 665 553 1
+757 456 30 244 1
+636 520 355 457 1
+321 442 358 568 1
+544 421 203 289 1
+502 641 412 345 1
+183 529 152 52 1
+326 551 407 400 1
+498 729 338 693 1
+91 705 579 133 1
+13 652 222 553 1
+694 307 484 492 1
+489 543 357 416 1
+615 371 406 597 1
+454 598 312 375 1
+650 241 572 33 1
+409 522 475 310 1
+59 659 446 214 1
+396 644 367 575 1
+644 446 367 575 1
+573 328 571 361 1
+133 49 207 528 1
+105 407 509 301 1
+548 323 468 419 1
+666 130 661 197 1
+206 71 721 187 1
+670 299 247 70 1
+384 493 547 134 1
+374 235 201 53 1
+9 578 97 730 1
+218 562 72 173 1
+519 615 676 403 1
+257 671 14 530 1
+352 707 593 721 1
+12 197 286 420 1
+674 601 485 42 1
+180 69 405 181 1
+412 569 491 143 1
+224 426 270 63 1
+633 345 646 634 1
+738 41 271 147 1
+739 536 292 424 1
+17 459 116 734 1
+640 4 504 237 1
+505 79 178 100 1
+665 377 414 553 1
+560 654 393 514 1
+535 366 418 499 1
+180 69 656 405 1
+451 418 337 545 1
+314 550 487 402 1
+611 443 310 623 1
+413 529 536 691 1
+546 659 59 163 1
+206 403 639 427 1
+698 467 725 547 1
+398 324 655 539 1
+438 378 627 306 1
+673 252 117 250 1
+425 357 709 525 1
+384 493 336 547 1
+681 703 144 452 1
+635 435 236 98 1
+125 457 601 126 1
+624 531 304 561 1
+555 502 425 568 1
+645 231 407 31 1
+118 241 517 744 1
+647 505 434 350 1
+426 224 524 63 1
+5 200 739 266 1
+409 475 618 310 1
+201 26 431 100 1
+490 381 516 628 1
+219 632 368 225 1
+462 392 332 370 1
+408 501 620 319 1
+5 536 292 739 1
+654 396 305 575 1
+603 469 740 306 1
+597 622 362 479 1
+661 365 510 420 1
+78 213 221 418 1
+442 502 646 345 1
+327 477 586 478 1
+745 369 576 426 1
+535 326 366 499 1
+409 360 623 613 1
+258 390 73 175 1
+403 639 427 558 1
+187 206 35 427 1
+664 349 503 584 1
+478 18 298 160 1
+320 441 676 403 1
+86 377 414 665 1
+11 376 150 169 1
+366 720 656 542 1
+295 1 496 657 1
+720 415 656 542 1
+744 241 517 48 1
+480 362 465 411 1
+80 168 732 567 1
+456 68 432 248 1
+406 356 447 378 1
+370 459 72 562 1
+338 628 695 516 1
+701 316 412 641 1
+334 643 443 438 1
+135 546 163 268 1
+25 161 542 228 1
+695 338 423 737 1
+585 630 89 388 1
+345 414 633 442 1
+338 488 420 684 1
+123 755 60 467 1
+696 386 507 448 1
+406 356 410 447 1
+226 13 162 451 1
+558 615 352 403 1
+675 369 333 609 1
+136 396 45 560 1
+331 566 485 404 1
+691 611 424 328 1
+421 743 631 512 1
+139 486 379 515 1
+345 633 435 722 1
+649 547 189 20 1
+447 304 653 587 1
+512 421 330 513 1
+84 620 253 260 1
+362 406 480 320 1
+522 623 747 310 1
+449 313 706 359 1
+462 605 406 306 1
+500 383 491 361 1
+563 535 474 366 1
+185 122 208 474 1
+677 582 487 324 1
+141 399 427 461 1
+149 479 460 562 1
+334 611 443 736 1
+527 363 339 598 1
+320 411 676 441 1
+754 655 207 373 1
+663 346 419 658 1
+411 749 465 676 1
+379 515 629 485 1
+411 140 676 441 1
+160 511 45 155 1
+513 85 621 235 1
+307 484 631 512 1
+646 701 412 345 1
+245 401 21 93 1
+465 749 121 676 1
+519 676 465 211 1
+520 637 316 502 1
+417 350 428 680 1
+519 403 676 211 1
+684 390 711 307 1
+601 412 404 143 1
+513 23 289 421 1
+498 507 338 386 1
+123 239 190 467 1
+12 259 488 628 1
+328 495 438 611 1
+631 407 484 726 1
+497 369 426 521 1
+432 671 395 248 1
+245 43 588 93 1
+496 216 630 380 1
+202 96 504 44 1
+504 96 4 171 1
+156 731 730 53 1
+437 597 311 406 1
+425 662 357 555 1
+151 745 278 576 1
+383 573 322 641 1
+34 112 273 510 1
+420 729 498 323 1
+741 351 588 463 1
+743 365 590 330 1
+66 500 491 713 1
+434 252 117 673 1
+686 703 36 414 1
+589 743 421 330 1
+411 552 379 483 1
+388 330 549 374 1
+618 409 335 702 1
+573 571 341 469 1
+197 420 510 113 1
+154 664 503 584 1
+366 418 542 535 1
+375 638 697 199 1
+556 276 418 78 1
+367 517 572 48 1
+584 115 664 450 1
+227 485 167 103 1
+537 381 628 729 1
+512 590 330 429 1
+475 647 434 350 1
+135 546 59 163 1
+139 486 21 108 1
+469 573 571 361 1
+499 484 326 445 1
+462 332 406 605 1
+554 397 716 413 1
+678 402 710 523 1
+415 228 44 171 1
+661 269 73 733 1
+78 400 535 556 1
+683 542 337 74 1
+526 723 308 555 1
+459 479 362 622 1
+496 242 152 529 1
+353 555 583 637 1
+81 116 29 397 1
+477 359 327 586 1
+560 396 654 514 1
+406 371 320 362 1
+563 54 527 363 1
+258 130 73 589 1
+75 243 156 544 1
+450 456 30 757 1
+493 134 384 102 1
+633 414 36 703 1
+488 695 616 368 1
+641 626 502 422 1
+397 436 179 81 1
+458 128 155 7 1
+277 107 284 501 1
+217 383 491 500 1
+392 332 370 562 1
+548 693 729 706 1
+539 538 393 148 1
+436 2 179 81 1
+493 376 150 476 1
+640 432 612 382 1
+154 19 514 664 1
+320 480 465 411 1
+188 115 584 30 1
+384 521 313 449 1
+420 729 338 498 1
+99 89 751 533 1
+423 385 338 516 1
+411 40 465 749 1
+585 242 496 529 1
+336 649 548 384 1
+698 468 189 131 1
+488 390 711 684 1
+375 405 598 312 1
+443 380 549 330 1
+63 151 193 746 1
+277 620 260 408 1
+492 484 445 326 1
+640 382 692 432 1
+425 358 395 596 1
+99 585 751 424 1
+559 436 603 397 1
+462 370 332 605 1
+373 669 486 401 1
+125 636 601 457 1
+596 382 454 329 1
+591 486 139 49 1
+79 431 240 100 1
+75 231 243 645 1
+42 126 601 143 1
+347 484 694 409 1
+143 569 204 8 1
+400 191 535 551 1
+721 519 71 403 1
+480 603 306 397 1
+17 149 173 459 1
+536 110 233 413 1
+393 503 148 154 1
+523 748 343 402 1
+464 303 87 200 1
+110 529 413 183 1
+410 615 352 558 1
+392 295 294 218 1
+466 448 507 333 1
+182 295 294 392 1
+520 353 502 321 1
+1 238 88 380 1
+440 616 368 518 1
+125 601 42 126 1
+731 374 335 618 1
+408 241 118 33 1
+692 212 432 68 1
+594 410 318 541 1
+214 659 446 664 1
+725 134 189 547 1
+158 91 610 461 1
+488 711 420 684 1
+405 69 638 181 1
+203 23 289 513 1
+677 567 391 456 1
+311 376 455 394 1
+514 503 393 154 1
+614 295 152 496 1
+245 43 108 588 1
+75 544 156 203 1
+557 481 437 750 1
+370 251 72 459 1
+334 462 643 438 1
+371 362 465 320 1
+372 736 431 647 1
+47 136 560 154 1
+724 256 253 83 1
+605 332 362 459 1
+468 649 166 381 1
+311 376 557 449 1
+670 602 673 70 1
+456 432 395 248 1
+546 325 508 453 1
+206 721 403 427 1
+189 20 166 649 1
+384 649 524 224 1
+583 624 522 555 1
+323 729 498 548 1
+207 528 754 373 1
+318 398 461 714 1
+206 721 427 187 1
+406 627 306 740 1
+589 177 130 473 1
+5 303 464 200 1
+223 621 85 201 1
+530 442 321 457 1
+23 589 549 177 1
+508 4 172 199 1
+550 325 446 575 1
+383 569 3 491 1
+391 677 314 487 1
+498 684 420 711 1
+575 654 446 349 1
+578 335 702 417 1
+403 211 71 206 1
+565 232 76 663 1
+232 663 565 112 1
+614 392 496 643 1
+306 603 571 397 1
+499 312 531 445 1
+15 518 208 105 1
+625 94 163 546 1
+531 357 454 312 1
+334 585 496 529 1
+520 433 316 637 1
+150 215 62 593 1
+415 472 504 44 1
+670 708 162 451 1
+290 234 530 38 1
+328 438 495 341 1
+643 657 392 496 1
+209 564 435 506 1
+673 564 250 117 1
+486 629 379 515 1
+446 659 59 546 1
+426 521 675 313 1
+605 406 306 480 1
+246 4 508 640 1
+391 503 324 487 1
+393 487 324 503 1
+277 408 501 620 1
+60 725 189 698 1
+272 104 646 574 1
+137 2 436 249 1
+526 340 723 545 1
+416 545 723 489 1
+407 231 301 31 1
+113 420 510 286 1
+452 612 432 329 1
+537 174 282 286 1
+316 641 502 422 1
+33 277 260 408 1
+693 729 338 516 1
+604 580 346 557 1
+401 520 355 636 1
+353 660 430 520 1
+509 407 95 301 1
+471 296 460 392 1
+230 460 471 296 1
+343 660 709 525 1
+327 738 497 478 1
+219 616 64 225 1
+40 411 137 140 1
+77 671 530 442 1
+451 377 553 340 1
+97 556 37 680 1
+627 637 740 583 1
+226 617 683 74 1
+13 553 222 451 1
+413 691 571 742 1
+408 600 466 339 1
+610 560 142 354 1
+147 478 298 160 1
+518 440 185 474 1
+187 399 62 593 1
+280 93 577 463 1
+350 505 434 602 1
+547 20 649 384 1
+489 357 308 555 1
+383 573 328 495 1
+555 422 502 308 1
+329 504 375 543 1
+657 471 643 392 1
+228 69 171 415 1
+543 504 375 415 1
+217 263 491 383 1
+644 514 446 654 1
+688 326 363 474 1
+350 602 434 619 1
+340 428 545 619 1
+340 619 526 434 1
+619 428 602 340 1
+707 721 62 593 1
+721 187 62 593 1
+153 519 211 71 1
+618 731 540 417 1
+425 353 430 321 1
+384 521 426 313 1
+654 575 305 487 1
+434 602 252 673 1
+543 312 416 720 1
+562 459 72 173 1
+384 493 497 738 1
+589 330 549 473 1
+665 686 36 414 1
+389 429 323 365 1
+400 499 418 317 1
+638 625 527 181 1
+559 603 480 397 1
+308 489 662 357 1
+393 654 560 712 1
+638 625 453 527 1
+157 303 536 5 1
+452 489 472 329 1
+663 232 76 606 1
+696 531 561 344 1
+662 329 596 357 1
+12 197 420 269 1
+369 511 61 155 1
+646 234 457 442 1
+646 104 234 442 1
+77 703 671 442 1
+684 307 498 492 1
+472 129 44 617 1
+565 663 419 112 1
+425 596 395 685 1
+386 696 347 561 1
+720 418 542 366 1
+452 377 472 489 1
+394 604 346 557 1
+604 580 24 346 1
+423 196 253 28 1
+84 196 253 423 1
+621 513 549 85 1
+223 621 549 85 1
+431 621 26 201 1
+374 431 647 540 1
+46 715 274 127 1
+292 424 99 266 1
+406 356 627 740 1
+629 433 355 520 1
+309 433 629 520 1
+540 680 350 417 1
+530 234 457 38 1
+290 234 442 530 1
+629 515 355 485 1
+685 358 395 425 1
+321 358 685 425 1
+353 555 637 422 1
+351 463 685 679 1
+583 624 555 525 1
+615 320 676 403 1
+654 514 446 349 1
+583 627 637 341 1
+425 685 430 314 1
+528 639 398 133 1
+498 307 386 492 1
+693 675 524 385 1
+559 116 459 170 1
+141 187 35 427 1
+483 433 331 629 1
+682 469 316 637 1
+553 690 673 340 1
+545 340 690 377 1
+37 159 428 247 1
+331 436 566 603 1
+671 358 530 442 1
+553 387 673 690 1
+525 709 343 402 1
+736 388 585 751 1
+376 557 394 311 1
+751 89 240 26 1
+620 423 616 84 1
+25 535 542 192 1
+68 642 432 248 1
+546 94 163 268 1
+493 41 271 738 1
+504 96 612 237 1
+507 385 675 693 1
+586 399 470 327 1
+752 394 346 557 1
+483 356 740 433 1
+516 381 524 693 1
+313 675 524 693 1
+372 79 178 505 1
+372 79 505 431 1
+22 100 195 505 1
+406 371 362 597 1
+597 394 557 311 1
+311 455 371 597 1
+6 172 453 163 1
+22 505 178 100 1
+573 422 322 641 1
+391 503 349 584 1
+638 69 697 199 1
+415 504 375 697 1
+493 476 738 327 1
+697 4 199 171 1
+27 612 642 452 1
+554 436 179 397 1
+671 395 248 14 1
+603 436 554 397 1
+571 397 603 554 1
+741 395 567 351 1
+337 451 683 74 1
+567 741 534 395 1
+683 377 451 337 1
+559 436 480 603 1
+603 436 566 361 1
+480 331 603 436 1
+324 398 714 539 1
+697 504 4 171 1
+379 483 331 629 1
+437 481 342 750 1
+694 484 512 409 1
+229 2 179 436 1
+567 168 456 395 1
+23 549 220 177 1
+472 612 452 329 1
+612 432 329 382 1
+334 496 630 643 1
+432 382 692 244 1
+433 583 637 353 1
+403 441 676 16 1
+367 644 592 48 1
+133 398 528 655 1
+280 164 93 463 1
+390 589 421 258 1
+222 299 553 670 1
+370 716 72 251 1
+15 267 440 225 1
+617 161 44 415 1
+15 440 267 284 1
+621 549 513 374 1
+542 161 617 415 1
+437 752 346 557 1
+355 682 629 485 1
+310 372 495 647 1
+540 647 350 505 1
+484 347 499 728 1
+728 416 347 499 1
+728 624 347 416 1
+709 660 430 525 1
+647 372 495 475 1
+621 223 26 201 1
+516 729 338 628 1
+284 440 267 620 1
+647 372 505 431 1
+690 414 553 387 1
+690 387 673 340 1
+705 427 35 639 1
+680 556 37 428 1
+662 308 357 555 1
+569 204 3 143 1
+491 569 3 143 1
+494 531 454 312 1
+485 601 404 42 1
+444 439 324 343 1
+506 595 322 372 1
+715 138 274 127 1
+479 138 274 715 1
+478 147 61 160 1
+578 417 97 730 1
+637 422 502 353 1
+515 636 401 355 1
+405 697 415 375 1
+550 325 382 659 1
+701 412 316 404 1
+86 377 665 652 1
+288 14 534 741 1
+660 583 433 353 1
+135 214 59 446 1
+726 390 407 632 1
+683 617 542 74 1
+162 281 226 451 1
+451 281 226 74 1
+13 299 451 222 1
+446 19 135 214 1
+165 655 754 373 1
+553 90 222 250 1
+645 632 407 390 1
+641 345 502 626 1
+237 612 504 640 1
+529 152 614 183 1
+471 392 599 315 1
+466 600 344 494 1
+348 441 607 591 1
+339 600 466 494 1
+599 346 658 437 1
+441 607 591 145 1
+598 600 494 312 1
+441 16 607 145 1
+486 49 528 754 1
+140 50 285 552 1
+583 353 660 525 1
+205 279 30 450 1
+450 279 30 115 1
+15 440 727 518 1
+646 321 457 502 1
+652 553 665 90 1
+665 553 414 90 1
+660 353 433 520 1
+309 660 433 520 1
+639 705 398 133 1
+568 308 662 425 1
+512 360 613 330 1
+614 643 496 334 1
+613 360 443 330 1
+388 443 330 613 1
+27 202 452 86 1
+202 129 452 86 1
+452 129 377 86 1
+640 612 329 382 1
+314 487 550 391 1
+628 106 695 516 1
+314 550 456 391 1
+451 670 340 553 1
+639 591 101 608 1
+206 403 101 639 1
+651 378 437 342 1
+528 101 639 133 1
+206 101 579 639 1
+679 520 629 355 1
+11 455 153 215 1
+422 637 573 341 1
+417 618 335 702 1
+31 632 733 119 1
+733 632 390 119 1
+484 335 631 512 1
+639 579 705 133 1
+398 705 91 133 1
+84 616 196 423 1
+437 332 597 406 1
+712 609 396 458 1
+440 423 616 620 1
+642 432 114 671 1
+125 38 457 126 1
+272 457 38 126 1
+272 234 38 457 1
+465 687 121 749 1
+406 740 306 480 1
+200 262 464 58 1
+731 618 335 417 1
+386 694 347 492 1
+663 232 606 389 1
+483 480 331 740 1
+420 174 537 286 1
+573 637 469 341 1
+739 424 292 266 1
+614 496 152 529 1
+47 560 393 154 1
+47 136 142 560 1
+378 735 342 651 1
+389 323 658 419 1
+389 658 323 429 1
+394 580 604 557 1
+315 599 658 437 1
+122 25 563 180 1
+185 54 563 363 1
+325 710 719 575 1
+715 46 291 127 1
+380 88 606 238 1
+511 478 160 648 1
+277 620 284 84 1
+405 415 543 375 1
+1 657 380 496 1
+563 25 192 366 1
+657 182 392 295 1
+54 180 122 563 1
+295 657 496 392 1
+12 488 420 628 1
+232 606 287 76 1
+362 480 306 397 1
+190 580 24 604 1
+502 701 412 641 1
+539 503 148 393 1
+523 304 344 561 1
+601 457 701 412 1
+489 308 662 377 1
+633 646 581 634 1
+255 35 91 705 1
+23 130 258 589 1
+589 23 289 258 1
+421 589 289 258 1
+555 568 425 308 1
+453 625 163 546 1
+416 531 347 499 1
+462 742 605 306 1
+321 502 442 568 1
+149 56 173 562 1
+39 465 297 124 1
+39 124 121 465 1
+438 462 443 623 1
+560 511 142 354 1
+741 685 530 395 1
+518 474 185 208 1
+601 126 457 412 1
+669 309 679 582 1
+376 455 394 467 1
+467 455 394 123 1
+467 394 291 123 1
+190 467 291 123 1
+605 332 406 362 1
+704 756 95 551 1
+9 756 400 578 1
+756 551 191 400 1
+704 551 191 756 1
+756 407 95 551 1
+439 582 324 343 1
+737 696 319 492 1
+595 505 434 647 1
+452 129 683 377 1
+452 683 472 377 1
+129 683 377 652 1
+683 377 337 489 1
+531 696 347 445 1
+410 470 311 447 1
+447 470 311 449 1
+376 311 470 449 1
+135 644 268 48 1
+403 441 607 348 1
+481 651 429 658 1
+182 296 471 392 1
+145 139 608 49 1
+403 16 607 441 1
+450 659 446 550 1
+227 485 566 167 1
+227 552 566 485 1
+552 566 485 331 1
+2 566 92 167 1
+230 346 471 460 1
+714 444 439 324 1
+456 596 314 395 1
+550 450 456 391 1
+380 473 88 177 1
+473 265 88 177 1
+206 607 101 403 1
+206 16 607 403 1
+188 80 391 667 1
+313 384 649 524 1
+521 448 477 609 1
+486 49 591 528 1
+591 145 608 49 1
+382 375 329 640 1
+674 125 577 636 1
+616 225 368 518 1
+531 344 696 445 1
+696 492 445 319 1
+708 556 37 221 1
+197 510 273 113 1
+615 371 320 406 1
+363 527 501 107 1
+494 402 325 454 1
+720 542 337 415 1
+458 155 576 7 1
+720 415 543 405 1
+720 543 312 405 1
+747 364 522 475 1
+109 655 165 482 1
+599 392 332 462 1
+150 476 194 41 1
+150 476 62 194 1
+476 399 62 194 1
+495 438 611 623 1
+318 594 586 470 1
+460 392 332 599 1
+376 470 476 327 1
+588 582 679 351 1
+92 713 229 66 1
+371 302 622 715 1
+430 520 309 679 1
+587 561 344 386 1
+712 369 609 458 1
+327 477 478 369 1
+511 369 712 458 1
+511 712 396 458 1
+634 722 635 98 1
+476 399 478 327 1
+649 20 184 224 1
+536 739 328 424 1
+384 20 649 224 1
+381 649 166 184 1
+336 468 548 649 1
+378 623 438 341 1
+17 275 459 479 1
+459 149 479 275 1
+149 275 274 479 1
+479 275 274 138 1
+716 413 370 529 1
+183 413 716 529 1
+716 183 29 413 1
+370 459 397 716 1
+554 397 29 716 1
+370 614 72 716 1
+459 116 397 716 1
+716 529 614 183 1
+605 332 459 370 1
+512 429 330 743 1
+46 149 274 479 1
+549 177 380 67 1
+549 513 220 85 1
+654 487 305 444 1
+388 613 330 374 1
+512 513 330 613 1
+331 483 411 480 1
+483 320 411 480 1
+320 356 480 483 1
+176 741 588 463 1
+700 445 499 326 1
+188 732 391 80 1
+441 411 483 320 1
+616 695 196 423 1
+550 450 382 456 1
+314 550 382 456 1
+261 351 43 567 1
+567 351 43 176 1
+567 391 482 351 1
+77 671 257 530 1
+653 525 343 523 1
+711 661 743 390 1
+223 630 549 388 1
+430 679 309 582 1
+387 90 553 250 1
+661 197 73 269 1
+257 671 248 14 1
+103 674 515 485 1
+500 554 571 361 1
+678 402 305 710 1
+674 636 355 601 1
+517 458 592 367 1
+631 421 645 390 1
+736 310 443 613 1
+6 625 638 181 1
+12 628 286 174 1
+324 487 444 343 1
+393 444 324 487 1
+356 583 447 378 1
+474 55 551 535 1
+12 259 269 488 1
+269 488 259 119 1
+488 119 64 259 1
+219 119 64 488 1
+581 254 36 633 1
+568 662 358 425 1
+419 510 663 389 1
+706 313 507 386 1
+702 326 407 400 1
+544 513 203 421 1
+156 203 544 235 1
+75 175 231 645 1
+324 714 393 539 1
+33 277 501 107 1
+444 714 393 324 1
+444 714 354 393 1
+683 13 451 377 1
+316 491 404 412 1
+404 412 491 143 1
+143 92 491 404 1
+398 714 439 324 1
+398 538 714 539 1
+493 476 150 41 1
+169 493 150 41 1
+169 376 150 493 1
+376 169 467 493 1
+722 641 435 345 1
+384 449 313 548 1
+376 467 336 493 1
+449 376 336 493 1
+449 493 336 384 1
+327 493 449 384 1
+327 376 449 493 1
+444 354 712 393 1
+102 41 271 493 1
+576 155 278 7 1
+151 576 278 7 1
+576 151 193 7 1
+637 353 502 520 1
+111 382 757 244 1
+612 96 212 237 1
+202 504 612 472 1
+202 612 452 472 1
+472 504 612 329 1
+494 710 402 748 1
+304 748 523 344 1
+561 531 748 344 1
+561 531 304 748 1
+574 412 126 8 1
+644 546 268 48 1
+295 1 152 496 1
+496 1 152 242 1
+242 1 216 496 1
+318 399 470 586 1
+322 595 434 475 1
+318 586 461 399 1
+524 649 184 224 1
+410 615 320 406 1
+555 624 522 364 1
+583 304 522 624 1
+497 384 426 224 1
+497 426 270 224 1
+102 497 270 224 1
+102 384 497 224 1
+739 79 200 372 1
+464 200 372 739 1
+536 464 328 739 1
+464 372 328 739 1
+581 442 104 646 1
+403 721 352 427 1
+601 412 701 404 1
+626 345 502 568 1
+686 703 414 452 1
+500 263 383 87 1
+500 464 87 383 1
+500 303 87 464 1
+323 718 498 429 1
+419 537 468 51 1
+113 537 282 286 1
+51 537 166 174 1
+157 210 500 66 1
+157 303 87 500 1
+217 263 383 500 1
+409 728 522 347 1
+542 415 656 228 1
+398 348 528 439 1
+25 656 563 180 1
+433 469 682 637 1
+180 405 656 563 1
+322 573 383 495 1
+156 235 731 53 1
+638 405 181 527 1
+521 327 497 369 1
+610 538 47 393 1
+449 521 313 359 1
+353 502 321 425 1
+393 487 503 349 1
+391 349 503 487 1
+737 319 688 492 1
+233 585 99 424 1
+479 394 604 752 1
+529 585 233 424 1
+139 21 486 515 1
+401 486 21 515 1
+39 519 465 211 1
+237 96 4 504 1
+335 513 512 613 1
+335 513 613 374 1
+640 375 329 504 1
+613 513 330 374 1
+409 360 613 512 1
+372 464 322 506 1
+383 506 322 464 1
+383 262 506 464 1
+383 262 263 506 1
+506 262 263 82 1
+383 506 263 82 1
+435 506 383 82 1
+209 506 435 82 1
+209 262 506 82 1
+664 659 446 450 1
+154 514 503 664 1
+664 19 446 214 1
+435 506 322 383 1
+384 738 497 327 1
+673 602 252 70 1
+356 660 541 653 1
+660 525 343 653 1
+660 309 582 430 1
+726 684 632 368 1
+453 508 375 325 1
+508 375 325 454 1
+382 508 325 454 1
+382 375 508 454 1
+204 435 82 635 1
+508 172 163 453 1
+59 172 163 508 1
+518 407 632 726 1
+660 430 582 343 1
+59 246 172 508 1
+420 365 510 323 1
+425 308 662 555 1
+584 503 57 667 1
+188 584 57 667 1
+261 391 667 482 1
+391 482 503 667 1
+700 600 312 445 1
+232 389 510 34 1
+478 369 354 511 1
+715 455 127 123 1
+411 40 749 140 1
+640 4 697 504 1
+338 695 423 516 1
+738 478 298 147 1
+478 738 497 147 1
+423 695 196 28 1
+516 695 423 28 1
+219 632 64 368 1
+632 488 64 368 1
+219 518 632 225 1
+105 518 632 219 1
+340 428 451 545 1
+597 371 622 479 1
+203 85 513 235 1
+337 545 416 489 1
+707 187 62 721 1
+532 256 724 83 1
+721 71 707 187 1
+622 302 297 138 1
+622 300 297 124 1
+522 341 747 623 1
+532 576 193 7 1
+560 610 47 393 1
+223 67 549 630 1
+328 691 571 413 1
+587 386 313 706 1
+425 525 430 353 1
+25 542 213 192 1
+704 756 191 400 1
+150 593 62 476 1
+11 215 150 593 1
+399 476 62 593 1
+118 241 408 517 1
+453 638 527 598 1
+561 748 304 344 1
+547 134 189 20 1
+466 408 339 517 1
+333 408 466 517 1
+522 728 624 347 1
+15 225 440 518 1
+472 617 44 415 1
+89 630 223 388 1
+77 689 257 671 1
+442 633 581 264 1
+459 116 716 251 1
+716 116 29 251 1
+370 459 716 251 1
+39 153 302 519 1
+637 555 341 422 1
+730 731 417 540 1
+195 159 680 97 1
+258 661 73 390 1
+258 589 661 390 1
+6 199 181 638 1
+521 327 477 359 1
+449 327 521 359 1
+384 327 521 449 1
+384 327 497 521 1
+497 521 426 384 1
+562 479 460 332 1
+422 316 573 637 1
+548 381 729 693 1
+556 400 535 418 1
+562 332 370 459 1
+494 531 748 454 1
+672 507 423 385 1
+556 276 37 221 1
+627 356 406 378 1
+56 562 218 173 1
+648 478 160 18 1
+322 372 595 475 1
+373 582 679 588 1
+401 679 629 355 1
+186 385 516 524 1
+186 516 184 524 1
+524 184 65 516 1
+381 524 65 516 1
+381 184 65 524 1
+572 339 650 501 1
+678 523 710 448 1
+325 494 710 402 1
+676 749 121 140 1
+542 535 213 192 1
+556 418 317 400 1
+25 366 542 535 1
+31 645 231 175 1
+361 573 491 316 1
+473 365 743 330 1
+181 54 132 527 1
+308 626 502 568 1
+639 101 579 133 1
+403 607 101 639 1
+181 54 527 405 1
+308 568 414 626 1
+390 743 421 589 1
+489 545 690 377 1
+362 605 306 480 1
+133 528 101 49 1
+396 146 128 592 1
+644 19 664 514 1
+567 395 314 677 1
+110 529 233 413 1
+468 537 166 51 1
+6 625 453 638 1
+568 442 414 345 1
+692 612 212 237 1
+642 612 432 452 1
+110 52 233 529 1
+86 686 414 452 1
+384 493 738 327 1
+408 319 672 466 1
+467 169 60 725 1
+290 77 257 530 1
+290 530 257 164 1
+288 257 530 164 1
+397 742 306 605 1
+691 334 438 742 1
+27 452 686 86 1
+413 742 370 529 1
+672 507 333 466 1
+288 257 14 530 1
+408 672 333 466 1
+416 312 531 499 1
+625 94 453 572 1
+418 276 556 221 1
+429 360 590 651 1
+339 625 453 572 1
+625 132 527 181 1
+698 467 60 725 1
+83 283 408 532 1
+408 283 118 532 1
+532 283 118 517 1
+408 532 118 517 1
+83 193 283 532 1
+83 256 193 532 1
+223 388 549 621 1
+316 422 502 637 1
+555 522 341 747 1
+586 477 354 478 1
+534 168 395 14 1
+568 345 502 442 1
+25 366 535 192 1
+122 192 474 535 1
+474 192 55 535 1
+535 191 192 55 1
+227 552 485 103 1
+411 552 137 140 1
+678 359 523 448 1
+78 191 535 400 1
+78 191 192 535 1
+454 312 357 543 1
+454 375 312 543 1
+413 529 233 536 1
+5 110 233 536 1
+5 233 292 536 1
+157 303 500 536 1
+500 303 464 536 1
+510 323 537 420 1
+473 177 380 549 1
+468 381 166 537 1
+419 323 468 537 1
+419 537 510 323 1
+323 706 718 429 1
+706 718 429 481 1
+718 342 429 481 1
+718 694 386 307 1
+537 51 282 174 1
+486 669 379 629 1
+379 669 483 629 1
+398 91 538 539 1
+25 542 656 228 1
+228 69 415 656 1
+377 340 451 545 1
+201 431 374 540 1
+399 648 478 586 1
+648 141 461 399 1
+540 431 505 100 1
+201 431 540 100 1
+595 293 506 564 1
+595 293 564 117 1
+397 116 459 559 1
+559 459 362 170 1
+515 674 355 485 1
+589 130 73 661 1
+661 390 488 119 1
+595 178 117 22 1
+505 178 595 22 1
+434 595 564 117 1
+661 130 73 197 1
+418 451 337 542 1
+80 732 391 567 1
+442 414 633 703 1
+618 613 374 335 1
+337 542 472 415 1
+519 721 707 352 1
+380 606 643 657 1
+472 504 543 415 1
+472 504 329 543 1
+472 489 543 329 1
+329 543 357 489 1
+329 454 357 543 1
+472 489 337 543 1
+732 584 456 30 1
+456 168 732 244 1
+456 732 30 244 1
+111 246 640 237 1
+111 246 382 640 1
+619 602 434 340 1
+335 513 374 544 1
+260 724 253 83 1
+691 334 424 611 1
+377 545 451 337 1
+377 489 545 337 1
+356 660 653 583 1
+660 309 430 520 1
+135 446 59 546 1
+748 525 343 402 1
+102 134 384 547 1
+102 547 384 20 1
+102 134 547 20 1
+572 501 650 33 1
+468 548 381 537 1
+548 323 381 537 1
+468 323 548 537 1
+9 578 730 156 1
+223 549 220 85 1
+223 67 220 549 1
+23 513 220 549 1
+590 360 330 443 1
+590 380 443 330 1
+443 380 388 549 1
+746 151 193 576 1
+450 550 446 349 1
+660 353 430 525 1
+474 535 551 326 1
+326 535 551 400 1
+518 474 551 326 1
+551 208 474 518 1
+509 208 551 518 1
+105 509 551 518 1
+105 551 407 518 1
+105 509 407 551 1
+55 208 551 509 1
+551 55 509 191 1
+55 208 474 551 1
+441 552 285 379 1
+411 552 441 379 1
+140 411 552 441 1
+140 552 285 441 1
+207 528 49 754 1
+485 552 379 515 1
+571 742 397 413 1
+691 334 742 529 1
+594 653 343 523 1
+447 653 523 587 1
+229 436 179 554 1
+398 538 610 714 1
+230 346 604 24 1
+387 626 345 435 1
+337 720 545 418 1
+572 94 453 546 1
+28 256 724 186 1
+724 256 385 186 1
+425 357 525 555 1
+425 555 525 353 1
+353 502 425 555 1
+353 422 502 555 1
+584 450 391 456 1
+523 748 710 344 1
+526 364 555 422 1
+494 710 748 344 1
+206 579 35 639 1
+390 661 733 119 1
+661 269 733 119 1
+97 198 556 417 1
+690 414 387 626 1
+417 428 317 556 1
+428 418 317 556 1
+690 414 377 553 1
+97 198 37 556 1
+563 366 700 405 1
+474 366 700 563 1
+187 399 721 427 1
+721 399 352 427 1
+688 700 363 326 1
+616 267 196 225 1
+352 558 470 410 1
+558 318 470 410 1
+541 318 558 410 1
+474 700 366 326 1
+352 318 470 558 1
+249 411 559 436 1
+436 411 559 480 1
+559 411 362 480 1
+411 362 465 559 1
+440 518 688 474 1
+518 326 688 474 1
+411 559 465 40 1
+411 40 249 559 1
+560 136 514 154 1
+328 611 424 372 1
+142 648 511 18 1
+616 695 423 737 1
+623 443 360 651 1
+597 752 394 479 1
+666 130 589 661 1
+165 588 108 43 1
+149 56 562 460 1
+56 562 460 296 1
+562 392 460 296 1
+56 392 562 296 1
+56 392 294 562 1
+562 392 294 218 1
+56 562 294 218 1
+311 615 406 597 1
+410 320 615 558 1
+563 192 535 366 1
+122 192 535 563 1
+122 25 192 563 1
+563 54 180 405 1
+405 54 527 563 1
+209 32 236 564 1
+564 32 236 387 1
+435 564 236 387 1
+209 564 236 435 1
+506 293 209 564 1
+596 454 314 357 1
+506 564 322 434 1
+683 451 226 74 1
+566 137 2 436 1
+552 137 566 436 1
+227 137 566 552 1
+227 137 2 566 1
+261 567 43 176 1
+261 534 567 176 1
+742 462 370 334 1
+438 462 742 334 1
+571 438 306 742 1
+742 334 370 529 1
+261 80 567 534 1
+80 168 567 534 1
+261 80 391 567 1
+261 391 482 567 1
+642 432 358 452 1
+435 383 569 82 1
+569 383 263 82 1
+204 569 263 82 1
+204 435 569 82 1
+204 569 3 263 1
+569 383 3 263 1
+316 383 491 569 1
+485 552 331 629 1
+629 552 331 379 1
+571 397 554 413 1
+500 413 571 554 1
+536 413 571 500 1
+536 571 328 500 1
+536 413 328 571 1
+692 382 111 244 1
+589 743 661 390 1
+571 438 469 306 1
+571 438 341 469 1
+340 670 428 602 1
+328 438 341 571 1
+517 241 572 48 1
+572 241 268 48 1
+546 572 268 48 1
+546 367 572 48 1
+572 94 546 268 1
+241 94 572 268 1
+408 241 572 517 1
+408 572 339 517 1
+466 517 339 572 1
+573 383 328 500 1
+361 383 573 500 1
+361 383 491 573 1
+680 505 602 22 1
+498 684 711 307 1
+498 711 429 307 1
+396 575 367 305 1
+133 398 655 10 1
+398 528 655 439 1
+31 632 407 645 1
+757 382 205 450 1
+532 256 193 746 1
+746 576 193 532 1
+63 151 746 426 1
+369 155 576 458 1
+186 746 524 63 1
+433 353 637 520 1
+578 556 417 400 1
+568 358 414 703 1
+581 646 104 634 1
+578 198 417 556 1
+578 198 97 417 1
+730 578 417 731 1
+156 731 578 730 1
+9 156 243 578 1
+91 35 579 705 1
+9 198 97 578 1
+541 410 558 699 1
+356 699 410 541 1
+669 439 348 699 1
+558 699 441 348 1
+580 131 24 419 1
+239 131 24 580 1
+190 239 24 580 1
+467 239 190 580 1
+362 406 605 480 1
+394 467 580 557 1
+699 379 348 669 1
+441 699 379 348 1
+309 541 699 483 1
+336 419 580 481 1
+580 419 346 481 1
+710 523 344 448 1
+678 586 477 444 1
+678 444 477 305 1
+390 711 307 631 1
+118 744 517 7 1
+97 556 680 417 1
+494 339 719 466 1
+453 527 625 339 1
+346 460 752 599 1
+239 698 131 580 1
+698 336 468 419 1
+560 712 511 354 1
+349 575 654 487 1
+600 319 696 445 1
+375 638 453 598 1
+453 508 638 375 1
+12 628 174 490 1
+628 420 286 174 1
+154 503 57 584 1
+139 379 50 753 1
+139 379 753 515 1
+368 737 688 492 1
+437 597 752 557 1
+597 332 752 479 1
+597 394 752 557 1
+425 596 685 314 1
+501 107 727 363 1
+533 242 89 585 1
+332 597 752 437 1
+233 533 99 585 1
+585 52 233 533 1
+533 242 585 52 1
+585 242 529 52 1
+529 52 233 585 1
+479 752 460 332 1
+359 448 313 587 1
+691 438 571 742 1
+588 351 401 463 1
+159 428 602 680 1
+482 582 588 351 1
+261 351 482 588 1
+482 582 373 588 1
+22 680 159 602 1
+486 373 401 588 1
+486 588 401 108 1
+245 108 401 588 1
+486 373 588 108 1
+165 373 108 588 1
+165 482 373 588 1
+513 589 421 330 1
+23 589 421 513 1
+421 23 289 589 1
+517 744 458 7 1
+441 591 285 145 1
+145 591 285 139 1
+591 379 285 139 1
+441 379 285 591 1
+441 348 379 591 1
+745 155 278 576 1
+376 593 476 470 1
+476 593 399 470 1
+593 352 399 470 1
+376 352 593 470 1
+455 593 376 352 1
+37 281 708 221 1
+418 556 708 221 1
+221 281 708 74 1
+11 215 593 455 1
+11 593 376 455 1
+221 708 418 74 1
+418 708 451 74 1
+708 281 451 74 1
+412 641 722 345 1
+722 435 635 98 1
+722 633 435 98 1
+644 654 446 575 1
+541 439 594 343 1
+396 654 644 575 1
+566 713 92 404 1
+401 520 636 679 1
+318 594 470 410 1
+351 430 582 679 1
+505 595 434 22 1
+372 178 595 505 1
+372 79 595 178 1
+595 79 200 178 1
+58 595 200 178 1
+58 293 595 178 1
+595 293 117 178 1
+58 293 506 595 1
+595 464 506 58 1
+200 464 595 58 1
+200 464 372 595 1
+372 464 506 595 1
+372 79 200 595 1
+437 597 557 311 1
+485 552 753 103 1
+494 600 344 531 1
+405 363 527 598 1
+446 325 546 719 1
+453 325 375 598 1
+454 325 598 375 1
+454 325 494 598 1
+720 312 416 499 1
+139 21 515 753 1
+346 471 460 599 1
+471 392 460 599 1
+659 325 446 550 1
+405 69 697 638 1
+415 504 697 171 1
+415 69 171 697 1
+408 319 466 600 1
+600 408 319 501 1
+600 501 319 363 1
+339 501 600 363 1
+363 339 598 600 1
+339 600 494 598 1
+339 408 600 501 1
+697 69 171 199 1
+333 672 675 507 1
+675 672 385 507 1
+448 507 333 675 1
+602 252 22 434 1
+159 252 22 602 1
+27 642 681 452 1
+642 671 114 681 1
+642 671 681 358 1
+642 358 681 452 1
+27 681 144 452 1
+159 247 252 602 1
+469 571 603 361 1
+571 603 361 554 1
+361 436 554 603 1
+469 571 306 603 1
+149 604 120 460 1
+604 149 479 460 1
+227 753 552 103 1
+485 552 515 753 1
+46 604 479 394 1
+604 46 291 394 1
+604 394 291 467 1
+190 604 291 467 1
+190 46 291 604 1
+604 46 120 190 1
+149 46 120 604 1
+46 149 479 604 1
+146 644 135 48 1
+606 389 315 471 1
+154 664 584 115 1
+232 389 287 606 1
+188 584 667 391 1
+261 667 109 482 1
+88 606 287 265 1
+88 473 606 265 1
+584 503 667 391 1
+590 389 606 473 1
+590 606 380 473 1
+88 380 606 473 1
+315 606 380 590 1
+315 389 606 590 1
+504 612 329 640 1
+458 609 367 333 1
+367 333 609 466 1
+466 333 609 448 1
+211 676 121 16 1
+211 403 676 16 1
+576 369 609 333 1
+744 128 458 7 1
+367 609 305 466 1
+396 305 367 609 1
+396 609 367 458 1
+404 713 491 361 1
+682 713 404 361 1
+229 713 554 66 1
+682 566 713 361 1
+158 91 538 610 1
+158 538 47 610 1
+158 47 142 610 1
+610 47 142 560 1
+336 547 649 384 1
+684 492 737 368 1
+737 368 695 488 1
+692 432 612 640 1
+465 687 749 40 1
+334 443 611 438 1
+27 612 452 202 1
+27 96 612 202 1
+27 96 212 612 1
+643 496 630 380 1
+525 624 555 357 1
+737 338 423 507 1
+218 614 72 562 1
+218 392 614 562 1
+218 295 614 392 1
+614 295 496 392 1
+218 295 152 614 1
+218 152 72 614 1
+352 615 470 311 1
+615 410 470 311 1
+311 615 410 406 1
+352 410 470 615 1
+352 455 371 615 1
+352 371 519 615 1
+371 615 465 519 1
+371 320 465 615 1
+219 368 616 225 1
+219 368 64 616 1
+64 488 616 368 1
+696 492 347 445 1
+64 570 196 616 1
+84 267 196 616 1
+388 549 621 374 1
+374 388 431 621 1
+350 618 540 417 1
+620 616 267 84 1
+620 501 440 319 1
+732 456 391 567 1
+648 354 478 586 1
+310 613 374 618 1
+310 613 618 409 1
+409 613 618 335 1
+619 350 475 434 1
+619 475 350 618 1
+350 618 417 619 1
+619 618 417 317 1
+417 317 428 619 1
+417 619 428 350 1
+317 619 618 409 1
+619 475 618 409 1
+548 729 498 706 1
+693 381 729 516 1
+277 501 284 620 1
+673 299 670 70 1
+222 299 673 70 1
+250 222 673 70 1
+467 169 725 493 1
+84 423 253 620 1
+388 223 26 621 1
+735 360 623 522 1
+17 275 622 300 1
+138 300 622 275 1
+138 622 479 275 1
+17 275 479 622 1
+17 479 459 622 1
+651 735 718 360 1
+735 561 360 522 1
+362 622 371 465 1
+39 371 622 465 1
+39 622 297 465 1
+39 302 297 622 1
+39 302 622 371 1
+597 371 362 622 1
+583 304 624 525 1
+561 522 624 347 1
+347 531 624 561 1
+28 724 516 186 1
+6 453 625 163 1
+626 526 422 322 1
+578 702 400 417 1
+387 626 435 322 1
+387 526 626 322 1
+629 433 682 355 1
+422 626 502 308 1
+741 530 463 164 1
+671 395 741 530 1
+288 530 741 164 1
+741 463 176 164 1
+176 351 588 741 1
+732 584 391 456 1
+730 680 417 97 1
+540 680 417 730 1
+174 537 628 420 1
+174 381 628 537 1
+600 696 319 466 1
+344 696 600 466 1
+696 448 507 466 1
+490 516 106 628 1
+12 628 490 106 1
+12 259 628 106 1
+344 386 696 448 1
+344 448 696 466 1
+334 630 388 443 1
+334 630 585 388 1
+334 496 585 630 1
+242 630 585 496 1
+242 216 630 496 1
+242 216 89 630 1
+242 630 89 585 1
+484 335 407 631 1
+105 632 301 219 1
+632 31 301 219 1
+31 219 632 119 1
+632 219 488 119 1
+390 632 488 119 1
+634 646 104 574 1
+407 31 301 632 1
+105 407 301 632 1
+105 407 632 518 1
+219 488 64 632 1
+435 633 236 98 1
+633 387 236 98 1
+254 387 633 98 1
+90 633 254 387 1
+90 553 633 387 1
+633 553 414 387 1
+626 414 387 633 1
+626 633 387 345 1
+345 387 435 633 1
+435 387 236 633 1
+626 414 633 345 1
+90 553 414 633 1
+90 414 254 633 1
+310 647 495 475 1
+280 636 577 38 1
+577 636 125 38 1
+280 463 636 38 1
+628 729 338 420 1
+723 624 555 364 1
+416 624 723 364 1
+463 93 636 401 1
+636 93 577 401 1
+21 577 401 636 1
+21 636 401 515 1
+475 747 495 310 1
+675 609 333 448 1
+333 675 532 576 1
+747 364 475 526 1
+463 93 577 636 1
+280 463 577 636 1
+463 685 530 741 1
+131 698 468 419 1
+131 580 698 419 1
+60 698 189 131 1
+638 172 199 508 1
+453 172 638 508 1
+6 172 638 453 1
+6 172 199 638 1
+528 639 101 608 1
+528 591 639 608 1
+528 348 639 591 1
+348 607 639 591 1
+403 607 639 348 1
+403 348 639 558 1
+318 348 558 639 1
+318 348 639 398 1
+348 639 398 528 1
+383 316 573 641 1
+412 316 569 641 1
+335 409 484 702 1
+68 642 212 432 1
+642 212 432 612 1
+27 212 642 612 1
+370 643 614 334 1
+370 392 614 643 1
+462 392 370 643 1
+599 392 462 643 1
+599 643 462 315 1
+643 315 443 462 1
+315 643 443 590 1
+643 380 443 590 1
+315 380 643 590 1
+380 606 315 643 1
+643 606 315 471 1
+643 471 315 392 1
+599 392 643 315 1
+315 658 590 429 1
+658 651 429 315 1
+644 19 135 446 1
+146 19 644 514 1
+146 19 135 644 1
+396 146 592 644 1
+396 592 458 644 1
+75 645 243 544 1
+544 645 243 578 1
+111 382 205 757 1
+544 645 578 335 1
+544 421 645 335 1
+631 335 645 421 1
+631 335 407 645 1
+645 421 544 289 1
+75 645 544 289 1
+75 175 645 289 1
+645 175 421 289 1
+442 321 646 502 1
+646 412 457 126 1
+272 646 457 126 1
+272 574 646 126 1
+574 412 646 126 1
+272 234 457 646 1
+272 104 234 646 1
+475 595 434 647 1
+475 372 595 647 1
+648 141 158 461 1
+18 141 158 648 1
+648 141 194 18 1
+194 18 298 648 1
+648 18 298 478 1
+194 648 298 478 1
+194 648 478 476 1
+648 399 478 476 1
+194 399 648 476 1
+399 141 194 648 1
+336 468 649 547 1
+468 547 189 649 1
+189 649 166 468 1
+736 585 388 334 1
+424 585 736 334 1
+623 462 651 378 1
+651 462 437 378 1
+315 462 437 651 1
+659 325 508 546 1
+382 508 659 325 1
+429 651 590 315 1
+590 651 443 315 1
+462 315 443 651 1
+623 462 443 651 1
+653 304 447 583 1
+356 653 447 583 1
+354 712 511 369 1
+269 661 420 488 1
+350 505 602 680 1
+269 661 488 119 1
+447 356 541 653 1
+653 447 410 541 1
+594 653 410 541 1
+594 447 410 653 1
+359 447 594 653 1
+583 525 653 304 1
+304 525 653 523 1
+713 217 491 66 1
+92 217 491 713 1
+92 217 713 66 1
+393 444 487 654 1
+393 654 487 349 1
+655 439 373 324 1
+398 439 655 324 1
+528 373 207 655 1
+467 755 455 123 1
+133 528 207 655 1
+1 238 380 657 1
+1 238 657 182 1
+238 76 657 182 1
+657 76 471 182 1
+657 471 392 182 1
+471 657 606 76 1
+657 238 606 76 1
+599 658 471 315 1
+346 658 471 599 1
+315 389 590 658 1
+315 437 658 651 1
+59 279 659 214 1
+59 279 205 659 1
+59 246 659 205 1
+659 246 508 205 1
+382 508 205 659 1
+59 246 508 659 1
+439 660 582 343 1
+541 660 439 343 1
+309 660 439 541 1
+309 660 541 356 1
+356 660 433 309 1
+356 583 433 660 1
+439 309 582 660 1
+197 661 510 420 1
+420 197 661 269 1
+439 669 373 582 1
+662 358 452 414 1
+662 358 414 568 1
+308 662 414 568 1
+452 432 358 662 1
+358 662 432 596 1
+662 329 432 596 1
+452 432 662 329 1
+329 489 662 452 1
+489 662 452 377 1
+580 663 24 346 1
+230 346 24 663 1
+230 663 24 565 1
+565 663 24 419 1
+580 419 24 663 1
+230 565 76 663 1
+230 663 76 471 1
+230 346 663 471 1
+446 450 349 664 1
+584 664 349 450 1
+664 279 659 450 1
+214 279 659 664 1
+644 19 446 664 1
+633 254 665 414 1
+389 510 34 666 1
+287 389 34 666 1
+287 666 34 265 1
+265 34 130 666 1
+130 34 273 666 1
+287 606 666 265 1
+473 666 606 265 1
+473 389 606 666 1
+287 389 666 606 1
+389 365 666 473 1
+473 666 589 365 1
+389 365 510 666 1
+148 57 503 668 1
+439 309 669 582 1
+405 363 598 700 1
+363 598 700 600 1
+700 312 499 445 1
+700 598 312 600 1
+528 348 669 439 1
+528 669 373 439 1
+591 528 669 373 1
+486 591 669 373 1
+591 528 348 669 1
+451 299 162 670 1
+162 299 247 670 1
+37 670 162 247 1
+37 428 670 247 1
+709 487 343 402 1
+709 314 487 402 1
+357 314 709 402 1
+428 451 670 340 1
+432 114 671 248 1
+257 114 248 671 1
+489 690 308 377 1
+672 319 620 423 1
+493 169 725 134 1
+408 319 620 672 1
+564 322 434 673 1
+673 322 434 526 1
+673 526 434 340 1
+673 340 434 670 1
+670 602 434 673 1
+673 553 340 670 1
+673 222 553 670 1
+222 299 670 673 1
+250 222 553 673 1
+250 387 673 553 1
+473 743 589 330 1
+387 526 673 340 1
+387 322 673 526 1
+387 322 564 673 1
+387 32 673 564 1
+673 32 250 564 1
+387 32 250 673 1
+521 448 675 313 1
+448 507 675 313 1
+521 609 675 448 1
+521 369 675 609 1
+426 369 675 521 1
+426 369 576 675 1
+675 532 385 672 1
+333 532 675 672 1
+750 359 449 342 1
+750 706 342 449 1
+320 411 465 676 1
+615 320 465 676 1
+519 615 465 676 1
+447 359 750 342 1
+677 391 482 324 1
+582 677 482 324 1
+351 677 482 582 1
+351 391 482 677 1
+343 402 678 523 1
+523 343 594 678 1
+678 343 594 444 1
+444 487 678 343 1
+487 402 678 343 1
+487 402 305 678 1
+444 487 305 678 1
+359 586 477 678 1
+679 321 520 636 1
+463 321 679 636 1
+401 679 636 463 1
+351 463 679 401 1
+588 679 401 351 1
+373 679 401 588 1
+373 679 669 401 1
+679 309 669 401 1
+350 505 680 540 1
+22 505 195 680 1
+22 195 159 680 1
+469 316 361 682 1
+361 491 682 316 1
+682 491 404 316 1
+361 491 404 682 1
+714 538 610 539 1
+682 603 566 361 1
+361 469 682 603 1
+741 14 534 395 1
+331 603 566 682 1
+331 566 404 682 1
+485 331 404 682 1
+683 13 377 652 1
+129 13 683 652 1
+129 13 226 683 1
+226 13 451 683 1
+683 129 617 226 1
+472 129 617 683 1
+452 129 472 683 1
+727 107 440 284 1
+15 727 440 284 1
+390 632 684 488 1
+684 632 368 488 1
+338 498 386 684 1
+338 684 420 498 1
+679 321 685 520 1
+685 321 430 520 1
+679 685 430 520 1
+351 685 430 679 1
+430 685 351 677 1
+567 685 677 351 1
+567 395 685 351 1
+567 395 677 685 1
+463 321 530 685 1
+583 304 447 735 1
+447 304 587 735 1
+587 561 718 735 1
+587 304 561 735 1
+735 304 522 583 1
+378 735 522 583 1
+687 622 465 124 1
+362 622 465 687 1
+687 362 559 465 1
+687 362 170 559 1
+735 561 718 360 1
+492 319 688 445 1
+327 476 738 478 1
+363 600 445 688 1
+9 704 191 400 1
+114 671 689 681 1
+690 308 626 526 1
+28 724 423 516 1
+724 28 253 256 1
+724 385 423 516 1
+340 690 387 526 1
+536 529 233 691 1
+691 529 233 424 1
+536 691 233 424 1
+529 585 424 691 1
+691 585 424 334 1
+529 585 691 334 1
+611 334 438 691 1
+328 611 438 691 1
+328 438 571 691 1
+640 382 111 692 1
+237 640 111 692 1
+432 612 212 692 1
+455 593 707 215 1
+153 455 519 707 1
+455 707 153 215 1
+498 693 507 386 1
+498 693 338 507 1
+507 338 385 693 1
+360 522 561 694 1
+325 719 453 546 1
+694 360 512 429 1
+307 694 512 429 1
+572 546 719 367 1
+446 719 546 367 1
+719 710 367 575 1
+466 572 339 719 1
+484 307 694 512 1
+616 570 196 695 1
+64 695 570 616 1
+64 488 695 616 1
+64 695 488 259 1
+695 628 488 259 1
+695 106 628 259 1
+570 106 695 259 1
+64 570 695 259 1
+507 386 696 492 1
+386 492 347 696 1
+656 697 415 405 1
+656 69 697 405 1
+656 69 415 697 1
+698 336 547 468 1
+336 467 698 547 1
+336 467 580 698 1
+698 467 580 239 1
+60 467 698 239 1
+699 410 558 320 1
+558 320 441 699 1
+441 320 483 699 1
+483 356 699 320 1
+699 356 410 320 1
+441 483 379 699 1
+699 318 558 541 1
+439 318 699 541 1
+309 439 699 541 1
+309 439 669 699 1
+439 318 558 699 1
+405 700 598 312 1
+363 600 700 445 1
+688 445 363 700 1
+682 316 404 701 1
+682 701 404 355 1
+601 701 355 404 1
+601 457 355 701 1
+520 701 355 457 1
+321 701 520 457 1
+321 502 701 457 1
+321 502 520 701 1
+355 433 701 520 1
+701 433 316 520 1
+433 316 682 701 1
+433 701 682 355 1
+516 381 729 628 1
+618 409 702 317 1
+417 618 702 317 1
+417 556 317 702 1
+484 326 702 499 1
+484 326 407 702 1
+335 484 407 702 1
+578 335 400 702 1
+442 264 77 703 1
+703 264 77 689 1
+77 689 671 703 1
+681 703 671 689 1
+681 703 689 144 1
+703 264 689 144 1
+703 144 36 264 1
+633 703 36 264 1
+442 703 633 264 1
+686 144 36 703 1
+568 358 703 442 1
+141 427 35 705 1
+141 461 427 705 1
+461 318 427 705 1
+705 318 427 398 1
+461 318 705 398 1
+398 461 91 705 1
+705 461 91 255 1
+141 461 705 255 1
+323 706 429 481 1
+575 446 719 325 1
+751 89 585 533 1
+481 548 323 706 1
+706 548 323 498 1
+548 693 706 313 1
+693 313 507 706 1
+455 352 707 593 1
+428 418 708 317 1
+708 418 545 317 1
+428 708 545 317 1
+428 708 451 545 1
+708 418 451 545 1
+709 582 677 430 1
+430 709 582 343 1
+709 487 582 343 1
+343 660 430 709 1
+325 710 575 402 1
+720 499 317 418 1
+575 367 305 710 1
+710 367 305 466 1
+466 609 305 710 1
+710 609 305 448 1
+466 609 710 448 1
+337 543 416 720 1
+720 337 543 415 1
+420 711 323 498 1
+498 711 323 429 1
+711 365 323 429 1
+420 365 323 711 1
+420 661 365 711 1
+661 711 420 488 1
+661 390 711 488 1
+654 396 560 712 1
+560 396 511 712 1
+712 396 305 654 1
+444 712 305 654 1
+477 444 712 305 1
+712 305 609 477 1
+477 444 354 712 1
+361 436 566 713 1
+713 436 566 229 1
+566 229 92 713 1
+229 436 554 713 1
+713 436 554 361 1
+714 610 354 393 1
+714 461 354 610 1
+586 461 354 714 1
+586 714 354 444 1
+318 714 586 444 1
+714 318 439 444 1
+398 318 439 714 1
+318 461 586 714 1
+394 46 291 715 1
+46 394 479 715 1
+715 597 394 479 1
+371 597 715 479 1
+371 715 622 479 1
+371 597 394 715 1
+371 455 715 394 1
+519 455 715 371 1
+39 519 715 371 1
+153 715 519 455 1
+153 127 715 455 1
+153 127 302 715 1
+138 127 715 302 1
+46 479 274 715 1
+717 419 131 51 1
+112 419 131 717 1
+112 419 717 510 1
+587 386 706 718 1
+587 561 386 718 1
+718 386 694 561 1
+718 561 694 360 1
+429 718 694 360 1
+651 718 429 360 1
+310 736 647 374 1
+736 310 613 374 1
+342 718 429 651 1
+388 736 613 374 1
+323 706 498 718 1
+453 598 339 719 1
+719 598 339 494 1
+719 325 598 494 1
+453 325 598 719 1
+466 572 719 367 1
+710 719 367 466 1
+312 700 720 405 1
+720 700 366 405 1
+366 720 405 656 1
+720 415 405 656 1
+366 720 499 700 1
+720 312 499 700 1
+720 499 418 366 1
+337 416 545 720 1
+545 416 317 720 1
+569 435 204 722 1
+722 435 204 635 1
+722 635 204 8 1
+569 722 204 8 1
+412 722 569 8 1
+412 574 722 8 1
+634 722 574 8 1
+722 634 635 8 1
+634 646 574 722 1
+412 574 646 722 1
+526 690 308 723 1
+526 340 690 723 1
+489 357 555 723 1
+723 357 555 624 1
+416 357 723 624 1
+489 357 723 416 1
+555 747 341 422 1
+672 385 724 532 1
+408 672 724 532 1
+408 672 620 724 1
+620 408 724 260 1
+620 724 253 260 1
+620 423 253 724 1
+423 28 253 724 1
+726 484 492 326 1
+726 307 492 484 1
+631 307 726 484 1
+307 390 631 726 1
+631 390 407 726 1
+307 390 726 684 1
+684 307 492 726 1
+368 684 492 726 1
+545 728 723 619 1
+728 364 723 619 1
+619 364 475 728 1
+728 364 475 409 1
+619 728 475 409 1
+317 728 619 409 1
+409 728 702 317 1
+409 728 484 702 1
+409 347 484 728 1
+409 364 522 728 1
+522 364 624 728 1
+364 624 728 416 1
+364 723 416 728 1
+723 545 416 728 1
+323 381 729 548 1
+195 680 730 97 1
+195 680 540 730 1
+195 540 53 730 1
+188 584 732 30 1
+188 584 391 732 1
+578 335 731 544 1
+156 544 578 731 1
+156 235 544 731 1
+544 513 731 235 1
+731 513 374 235 1
+544 513 374 731 1
+622 362 734 687 1
+622 687 734 124 1
+17 622 734 300 1
+300 622 734 124 1
+17 459 734 622 1
+734 459 362 622 1
+587 735 718 342 1
+447 735 587 342 1
+437 735 447 342 1
+437 378 735 342 1
+437 378 447 735 1
+583 735 447 378 1
+378 360 623 735 1
+378 735 651 360 1
+424 334 736 611 1
+372 424 736 611 1
+372 611 736 647 1
+647 611 736 310 1
+440 616 423 737 1
+620 737 440 423 1
+620 319 737 423 1
+672 319 423 737 1
+620 319 440 737 1
+737 319 440 688 1
+368 737 440 688 1
+440 616 737 368 1
+616 695 737 368 1
+338 684 386 737 1
+737 684 386 492 1
+737 386 507 492 1
+507 696 737 492 1
+338 386 507 737 1
+79 372 739 431 1
+752 394 604 346 1
+552 753 379 515 1
+599 752 346 437 1
+332 752 599 437 1
+752 460 332 599 1
+603 740 331 480 1
+682 740 331 603 1
+682 469 740 603 1
+469 682 740 433 1
+740 682 331 433 1
+483 740 331 433 1
+740 469 433 637 1
+627 469 740 637 1
+671 741 14 530 1
+288 14 741 530 1
+671 395 14 741 1
+395 685 351 741 1
+741 685 351 463 1
+288 534 176 741 1
+741 534 176 567 1
+742 397 413 716 1
+370 742 413 716 1
+370 397 742 716 1
+306 742 397 571 1
+390 711 631 743 1
+631 711 307 743 1
+711 743 429 307 1
+711 365 429 743 1
+429 365 590 743 1
+711 661 365 743 1
+589 365 661 743 1
+473 365 589 743 1
+390 743 631 421 1
+369 61 497 745 1
+369 497 426 745 1
+369 61 745 155 1
+186 385 524 746 1
+256 385 186 746 1
+385 724 746 256 1
+746 724 532 256 1
+385 724 532 746 1
+385 746 532 675 1
+675 746 532 576 1
+675 426 746 576 1
+385 426 746 675 1
+747 475 495 322 1
+322 747 573 495 1
+322 422 573 747 1
+422 526 747 322 1
+422 364 747 526 1
+555 364 747 422 1
+304 624 748 531 1
+624 357 748 531 1
+624 357 525 748 1
+357 709 525 748 1
+748 709 525 402 1
+357 709 748 402 1
+304 525 748 624 1
+304 525 523 748 1
+523 525 343 748 1
+447 449 311 750 1
+750 557 449 311 1
+750 336 449 557 1
+481 336 750 557 1
+336 548 481 750 1
+481 548 706 750 1
+548 706 750 449 1
+336 548 750 449 1
+447 359 449 750 1
+751 99 266 240 1
+424 99 266 751 1
+739 424 266 751 1
+79 739 266 751 1
+79 431 739 751 1
+739 372 751 431 1
+372 751 431 736 1
+736 388 751 431 1
+431 388 751 621 1
+751 388 26 621 1
+431 751 26 621 1
+388 89 751 26 1
+372 424 751 736 1
+739 372 424 751 1
+552 50 379 753 1
+227 50 552 753 1
+165 655 207 754 1
+139 49 486 754 1
+376 755 455 467 1
+756 702 407 400 1
+756 407 551 400 1
+335 702 407 756 1
+645 407 756 335 1
+645 756 578 335 1
+645 756 243 578 1
+9 756 578 243 1
+9 704 756 243 1
+243 756 95 704 1
+231 756 95 243 1
+231 645 756 243 1
+231 407 756 645 1
+231 407 95 756 1
+End
diff -pruN 5.0.3-1/examples/Tetrahedral_remeshing/mesh_and_remesh_polyhedral_domain_with_features.cpp 5.1-1/examples/Tetrahedral_remeshing/mesh_and_remesh_polyhedral_domain_with_features.cpp
--- 5.0.3-1/examples/Tetrahedral_remeshing/mesh_and_remesh_polyhedral_domain_with_features.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Tetrahedral_remeshing/mesh_and_remesh_polyhedral_domain_with_features.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,75 @@
+#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
+
+#include <CGAL/Mesh_triangulation_3.h>
+#include <CGAL/Mesh_complex_3_in_triangulation_3.h>
+#include <CGAL/Mesh_criteria_3.h>
+
+#include <CGAL/Polyhedral_mesh_domain_with_features_3.h>
+#include <CGAL/make_mesh_3.h>
+
+#include <CGAL/tetrahedral_remeshing.h>
+
+// Domain
+typedef CGAL::Exact_predicates_inexact_constructions_kernel K;
+typedef CGAL::Mesh_polyhedron_3<K>::type Polyhedron;
+typedef CGAL::Polyhedral_mesh_domain_with_features_3<K> Mesh_domain;
+
+// Triangulation for Meshing
+typedef CGAL::Mesh_triangulation_3<Mesh_domain, CGAL::Default, CGAL::Default>::type Tr;
+typedef CGAL::Mesh_complex_3_in_triangulation_3<
+  Tr, Mesh_domain::Corner_index, Mesh_domain::Curve_index> C3t3;
+
+// Criteria
+typedef CGAL::Mesh_criteria_3<Tr> Mesh_criteria;
+
+// Triangulation for Remeshing
+typedef CGAL::Triangulation_3<typename Tr::Geom_traits,
+  typename Tr::Triangulation_data_structure> Triangulation_3;
+
+// To avoid verbose function and named parameters call
+using namespace CGAL::parameters;
+
+int main(int argc, char* argv[])
+{
+  const char* fname = (argc > 1) ? argv[1] : "data/fandisk.off";
+  std::ifstream input(fname);
+  Polyhedron polyhedron;
+  input >> polyhedron;
+  if (input.fail()) {
+    std::cerr << "Error: Cannot read file " << fname << std::endl;
+    return EXIT_FAILURE;
+  }
+
+  if (!CGAL::is_triangle_mesh(polyhedron)) {
+    std::cerr << "Input geometry is not triangulated." << std::endl;
+    return EXIT_FAILURE;
+  }
+
+  // Create domain
+  Mesh_domain domain(polyhedron);
+
+  // Get sharp features
+  domain.detect_features();
+
+  // Mesh criteria
+  Mesh_criteria criteria(edge_size = 0.025,
+    facet_angle = 25, facet_size = 0.05, facet_distance = 0.005,
+    cell_radius_edge_ratio = 3, cell_size = 0.05);
+
+  // Mesh generation
+  C3t3 c3t3 = CGAL::make_mesh_3<C3t3>(domain, criteria);
+
+  Triangulation_3 tr = CGAL::convert_to_triangulation_3(std::move(c3t3));
+  //note we use the move semantic, with std::move(c3t3),
+  //  to avoid a copy of the triangulation by the function
+  //  `CGAL::convert_to_triangulation_3()`
+  //  After the call to this function, c3t3 is an empty and valid C3t3.
+  //It is possible to use :  CGAL::convert_to_triangulation_3(c3t3),
+  //  Then the triangulation is copied and duplicated, and c3t3 remains as is.
+
+  const double target_edge_length = 0.1;//coarsen the mesh
+  CGAL::tetrahedral_isotropic_remeshing(tr, target_edge_length,
+    CGAL::parameters::number_of_iterations(3));
+
+  return EXIT_SUCCESS;
+}
diff -pruN 5.0.3-1/examples/Tetrahedral_remeshing/tetrahedral_remeshing_example.cpp 5.1-1/examples/Tetrahedral_remeshing/tetrahedral_remeshing_example.cpp
--- 5.0.3-1/examples/Tetrahedral_remeshing/tetrahedral_remeshing_example.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Tetrahedral_remeshing/tetrahedral_remeshing_example.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,27 @@
+#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
+
+#include <CGAL/Tetrahedral_remeshing/Remeshing_triangulation_3.h>
+#include <CGAL/tetrahedral_remeshing.h>
+
+#include "tetrahedral_remeshing_generate_input.h"
+
+#include <iostream>
+#include <fstream>
+#include <string>
+
+typedef CGAL::Exact_predicates_inexact_constructions_kernel K;
+
+typedef CGAL::Tetrahedral_remeshing::Remeshing_triangulation_3<K> Remeshing_triangulation;
+
+int main(int argc, char* argv[])
+{
+  const double target_edge_length = (argc > 1) ? atof(argv[1]) : 0.1;
+  const std::size_t nbv = (argc > 2) ? atoi(argv[2]) : 1000;
+
+  Remeshing_triangulation tr;
+  CGAL::Tetrahedral_remeshing::generate_input_one_subdomain(nbv, tr);
+
+  CGAL::tetrahedral_isotropic_remeshing(tr, target_edge_length);
+
+  return EXIT_SUCCESS;
+}
diff -pruN 5.0.3-1/examples/Tetrahedral_remeshing/tetrahedral_remeshing_from_mesh.cpp 5.1-1/examples/Tetrahedral_remeshing/tetrahedral_remeshing_from_mesh.cpp
--- 5.0.3-1/examples/Tetrahedral_remeshing/tetrahedral_remeshing_from_mesh.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Tetrahedral_remeshing/tetrahedral_remeshing_from_mesh.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,30 @@
+#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
+
+#include <CGAL/Tetrahedral_remeshing/Remeshing_triangulation_3.h>
+#include <CGAL/tetrahedral_remeshing.h>
+
+#include <CGAL/IO/File_medit.h>
+
+#include <fstream>
+
+typedef CGAL::Exact_predicates_inexact_constructions_kernel K;
+
+typedef CGAL::Tetrahedral_remeshing::Remeshing_triangulation_3<K> Remeshing_triangulation;
+
+int main(int argc, char* argv[])
+{
+  const char* filename = (argc > 1) ? argv[1] : "data/sphere.mesh";
+  const double target_edge_length = (argc > 2) ? atof(argv[2]) : 0.1;
+
+  Remeshing_triangulation tr;
+
+  std::ifstream is(filename, std::ios_base::in);
+  CGAL::read_MEDIT(is, tr);
+
+  CGAL::tetrahedral_isotropic_remeshing(tr, target_edge_length);
+
+  std::ofstream os("after_remeshing.mesh");
+  CGAL::write_MEDIT(os, tr);
+
+  return EXIT_SUCCESS;
+}
diff -pruN 5.0.3-1/examples/Tetrahedral_remeshing/tetrahedral_remeshing_generate_input.h 5.1-1/examples/Tetrahedral_remeshing/tetrahedral_remeshing_generate_input.h
--- 5.0.3-1/examples/Tetrahedral_remeshing/tetrahedral_remeshing_generate_input.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Tetrahedral_remeshing/tetrahedral_remeshing_generate_input.h	2020-09-08 12:15:48.000000000 +0000
@@ -0,0 +1,191 @@
+// Copyright (c) 2020 GeometryFactory (France) and Telecom Paris (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Tetrahedral_remeshing/examples/Tetrahedral_remeshing/tetrahedral_remeshing_generate_input.h $
+// $Id: tetrahedral_remeshing_generate_input.h 25b0dc5 2020-05-27T19:04:46+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Jane Tournois, Noura Faraj
+
+#include <CGAL/Random.h>
+
+#include <boost/unordered_set.hpp>
+#include <utility>
+
+namespace CGAL
+{
+namespace Tetrahedral_remeshing
+{
+  template<typename Tr>
+  void generate_input_two_subdomains(const std::size_t& nbv, Tr& tr)
+  {
+    CGAL::Random rng;
+
+    typedef typename Tr::Point Point;
+    typedef typename Tr::Cell_handle Cell_handle;
+
+    while (tr.number_of_vertices() < nbv)
+      tr.insert(Point(rng.get_double(-1., 1.), rng.get_double(-1., 1.), rng.get_double(-1., 1.)));
+
+    const typename Tr::Geom_traits::Plane_3
+      plane(Point(0, 0, 0), Point(0, 1, 0), Point(0, 0, 1));
+
+    for (Cell_handle c : tr.finite_cell_handles())
+    {
+      if (plane.has_on_positive_side(
+        CGAL::centroid(c->vertex(0)->point(), c->vertex(1)->point(),
+                       c->vertex(2)->point(), c->vertex(3)->point())))
+        c->set_subdomain_index(1);
+      else
+        c->set_subdomain_index(2);
+    }
+    CGAL_assertion(tr.is_valid(true));
+  }
+
+
+  template<typename Tr>
+  void generate_input_one_subdomain(const std::size_t nbv, Tr& tr)
+  {
+    CGAL::Random rng;
+
+    typedef typename Tr::Point Point;
+    std::vector<Point> pts;
+    while (pts.size() < nbv)
+    {
+      const double x = rng.get_double(-1., 1.);
+      const double y = rng.get_double(-1., 1.);
+      const double z = rng.get_double(-1., 1.);
+
+      pts.push_back(Point(x, y, z));
+    }
+    tr.insert(pts.begin(), pts.end());
+
+    for (typename Tr::Cell_handle c : tr.finite_cell_handles())
+      c->set_subdomain_index(1);
+
+    CGAL_assertion(tr.is_valid(true));
+  }
+
+  template<typename Tr>
+  void add_edge(typename Tr::Vertex_handle v1,
+    typename Tr::Vertex_handle v2,
+    const Tr& tr,
+    boost::unordered_set<std::pair<typename Tr::Vertex_handle,
+                                   typename Tr::Vertex_handle> >& constraints)
+  {
+    typename Tr::Cell_handle c;
+    int i, j;
+    if (tr.is_edge(v1, v2, c, i, j))
+      constraints.insert(std::make_pair(v1, v2));
+  }
+
+  template<typename Tr>
+  void make_constraints_from_cube_edges(
+    Tr& tr,
+    boost::unordered_set<std::pair<typename Tr::Vertex_handle,
+                                   typename Tr::Vertex_handle> >& constraints)
+  {
+    typedef typename Tr::Point Point;
+    typedef typename Tr::Vertex_handle Vertex_handle;
+    typedef typename Tr::Cell_handle Cell_handle;
+
+    const Point p0(-2., -2., -2.);
+    const Point p1(-2., -2., -2.);
+    const Point p2(2., -2., -2.);
+    const Point p3(2., -2., 2.);
+    const Point p4(-2., 2., -2.);
+    const Point p5(-2., 2., 2.);
+    const Point p6(2., 2., -2.);
+    const Point p7(2., 2., 2.);
+
+    typename Tr::Locate_type lt;
+    int li, lj;
+    Cell_handle c = tr.locate(p0, lt, li, lj);
+    Vertex_handle v0 = c->vertex(li);
+    c = tr.locate(p1, lt, li, lj);
+    Vertex_handle v1 = c->vertex(li);
+
+    c = tr.locate(p2, lt, li, lj);
+    Vertex_handle v2 = c->vertex(li);
+    c = tr.locate(p3, lt, li, lj);
+    Vertex_handle v3 = c->vertex(li);
+
+    c = tr.locate(p4, lt, li, lj);
+    Vertex_handle v4 = c->vertex(li);
+    c = tr.locate(p5, lt, li, lj);
+    Vertex_handle v5 = c->vertex(li);
+
+    c = tr.locate(p6, lt, li, lj);
+    Vertex_handle v6 = c->vertex(li);
+    c = tr.locate(p7, lt, li, lj);
+    Vertex_handle v7 = c->vertex(li);
+
+    // constrain cube edges
+    add_edge(v0, v1, tr, constraints);
+    add_edge(v1, v2, tr, constraints);
+    add_edge(v2, v3, tr, constraints);
+    add_edge(v3, v0, tr, constraints);
+
+    add_edge(v4, v5, tr, constraints);
+    add_edge(v5, v6, tr, constraints);
+    add_edge(v6, v7, tr, constraints);
+    add_edge(v7, v4, tr, constraints);
+
+    add_edge(v0, v4, tr, constraints);
+    add_edge(v1, v5, tr, constraints);
+    add_edge(v2, v6, tr, constraints);
+    add_edge(v3, v7, tr, constraints);
+  }
+
+  template<typename Tr>
+  void generate_input_cube(const std::size_t& n,
+    Tr& tr,
+    boost::unordered_set<std::pair<typename Tr::Vertex_handle,
+                                   typename Tr::Vertex_handle> >& constraints)
+  {
+    typedef typename Tr::Vertex_handle Vertex_handle;
+    typedef typename Tr::Point Point;
+    CGAL::Random rng;
+
+    // points in a sphere
+    std::vector<Point> pts;
+    while (pts.size() < n)
+      pts.push_back(Point(rng.get_double(-1., 1.), rng.get_double(-1., 1.), rng.get_double(-1., 1.)));
+    tr.insert(pts.begin(), pts.end());
+
+    // vertices of a larger cube
+    Vertex_handle v0 = tr.insert(Point(-2., -2., -2.));
+    Vertex_handle v1 = tr.insert(Point(-2., -2., 2.));
+
+    Vertex_handle v2 = tr.insert(Point(2., -2., -2.));
+    Vertex_handle v3 = tr.insert(Point(2., -2., 2.));
+
+    Vertex_handle v4 = tr.insert(Point(-2., 2., -2.));
+    Vertex_handle v5 = tr.insert(Point(-2., 2., 2.));
+
+    Vertex_handle v6 = tr.insert(Point(2., 2., -2.));
+    Vertex_handle v7 = tr.insert(Point(2., 2., 2.));
+
+    // constrain cube edges
+    add_edge(v0, v1, tr, constraints);
+    add_edge(v1, v2, tr, constraints);
+    add_edge(v2, v3, tr, constraints);
+    add_edge(v3, v0, tr, constraints);
+
+    add_edge(v4, v5, tr, constraints);
+    add_edge(v5, v6, tr, constraints);
+    add_edge(v6, v7, tr, constraints);
+    add_edge(v7, v4, tr, constraints);
+
+    add_edge(v0, v4, tr, constraints);
+    add_edge(v1, v5, tr, constraints);
+    add_edge(v2, v6, tr, constraints);
+    add_edge(v3, v7, tr, constraints);
+
+    CGAL_assertion(tr.is_valid(true));
+  }
+}
+}
\ No newline at end of file
diff -pruN 5.0.3-1/examples/Tetrahedral_remeshing/tetrahedral_remeshing_of_one_subdomain.cpp 5.1-1/examples/Tetrahedral_remeshing/tetrahedral_remeshing_of_one_subdomain.cpp
--- 5.0.3-1/examples/Tetrahedral_remeshing/tetrahedral_remeshing_of_one_subdomain.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Tetrahedral_remeshing/tetrahedral_remeshing_of_one_subdomain.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,42 @@
+#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
+
+#include <CGAL/Tetrahedral_remeshing/Remeshing_triangulation_3.h>
+#include <CGAL/tetrahedral_remeshing.h>
+
+#include "tetrahedral_remeshing_generate_input.h"
+
+typedef CGAL::Exact_predicates_inexact_constructions_kernel K;
+
+typedef CGAL::Tetrahedral_remeshing::Remeshing_triangulation_3<K> Remeshing_triangulation;
+
+
+struct Cells_of_subdomain
+{
+private:
+  const int m_subdomain;
+
+public:
+  Cells_of_subdomain(const int& subdomain)
+    : m_subdomain(subdomain)
+  {}
+
+  bool operator()(Remeshing_triangulation::Cell_handle c) const
+  {
+    return m_subdomain == c->subdomain_index();
+  }
+};
+
+int main(int argc, char* argv[])
+{
+  const double target_edge_length = (argc > 1) ? atof(argv[1]) : 0.1;
+  const std::size_t nbv = (argc > 2) ? atoi(argv[2]) : 1000;
+
+  Remeshing_triangulation tr;
+  CGAL::Tetrahedral_remeshing::generate_input_two_subdomains(nbv, tr);
+
+  CGAL::tetrahedral_isotropic_remeshing(tr, target_edge_length,
+      CGAL::parameters::cell_selector(Cells_of_subdomain(2)));
+
+  return EXIT_SUCCESS;
+}
+
diff -pruN 5.0.3-1/examples/Tetrahedral_remeshing/tetrahedral_remeshing_with_features.cpp 5.1-1/examples/Tetrahedral_remeshing/tetrahedral_remeshing_with_features.cpp
--- 5.0.3-1/examples/Tetrahedral_remeshing/tetrahedral_remeshing_with_features.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Tetrahedral_remeshing/tetrahedral_remeshing_with_features.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,92 @@
+#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
+
+#include <CGAL/Tetrahedral_remeshing/Remeshing_triangulation_3.h>
+#include <CGAL/tetrahedral_remeshing.h>
+
+#include <CGAL/property_map.h>
+
+#include <boost/unordered_set.hpp>
+
+#include <iostream>
+#include <utility>
+
+#include "tetrahedral_remeshing_generate_input.h"
+
+typedef CGAL::Exact_predicates_inexact_constructions_kernel K;
+
+typedef CGAL::Tetrahedral_remeshing::Remeshing_triangulation_3<K> Remeshing_triangulation;
+
+typedef Remeshing_triangulation::Point         Point;
+typedef Remeshing_triangulation::Vertex_handle Vertex_handle;
+typedef Remeshing_triangulation::Cell_handle   Cell_handle;
+typedef Remeshing_triangulation::Edge          Edge;
+
+class Constrained_edges_property_map
+{
+public:
+  typedef bool                               value_type;
+  typedef bool                               reference;
+  typedef std::pair<Vertex_handle, Vertex_handle> key_type;
+  typedef boost::read_write_property_map_tag category;
+
+private:
+  boost::unordered_set<key_type>* m_set_ptr;
+
+public:
+  Constrained_edges_property_map()
+    : m_set_ptr(NULL)
+  {}
+  Constrained_edges_property_map(boost::unordered_set<key_type>* set_)
+    : m_set_ptr(set_)
+  {}
+
+public:
+  friend void put(Constrained_edges_property_map& map,
+                  const key_type& k,
+                  const bool b)
+  {
+    CGAL_assertion(map.m_set_ptr != NULL);
+    CGAL_assertion(k.first < k.second);
+    if (b)  map.m_set_ptr->insert(k);
+    else    map.m_set_ptr->erase(k);
+  }
+
+  friend value_type get(const Constrained_edges_property_map& map,
+                              const key_type& k)
+  {
+    CGAL_assertion(map.m_set_ptr != NULL);
+    CGAL_assertion(k.first < k.second);
+    return (map.m_set_ptr->count(k) > 0);
+  }
+};
+
+void set_subdomain(Remeshing_triangulation& tr, const int index)
+{
+  for (Remeshing_triangulation::Finite_cells_iterator cit = tr.finite_cells_begin();
+       cit != tr.finite_cells_end(); ++cit)
+  {
+    cit->set_subdomain_index(index);
+  }
+}
+
+int main(int argc, char* argv[])
+{
+  const double target_edge_length = (argc > 1) ? atof(argv[1]) : 0.02;
+  const int nb_iter = (argc > 2) ? atoi(argv[2]) : 1;
+  const int nbv = (argc > 3) ? atoi(argv[3]) : 500;
+
+  Remeshing_triangulation t3;
+  boost::unordered_set<std::pair<Vertex_handle, Vertex_handle> > constraints;
+
+  CGAL::Tetrahedral_remeshing::generate_input_cube(nbv, t3, constraints);
+  make_constraints_from_cube_edges(t3, constraints);
+  CGAL_assertion(t3.is_valid());
+
+  CGAL::tetrahedral_isotropic_remeshing(t3, target_edge_length,
+    CGAL::parameters::edge_is_constrained_map(
+      Constrained_edges_property_map(&constraints))
+    .number_of_iterations(nb_iter));
+
+  return EXIT_SUCCESS;
+}
+
diff -pruN 5.0.3-1/examples/Triangulation/CMakeLists.txt 5.1-1/examples/Triangulation/CMakeLists.txt
--- 5.0.3-1/examples/Triangulation/CMakeLists.txt	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Triangulation/CMakeLists.txt	2020-09-08 12:12:21.000000000 +0000
@@ -17,9 +17,8 @@ find_package(CGAL QUIET)
 if ( CGAL_FOUND )
 
   find_package(Eigen3 3.1.0)
-  if (EIGEN3_FOUND)
-    include( ${EIGEN3_USE_FILE} )
-
+  include(CGAL_Eigen_support)
+  if (TARGET CGAL::Eigen_support)
     create_single_source_cgal_program( "barycentric_subdivision.cpp" )
     create_single_source_cgal_program( "delaunay_triangulation.cpp" )
     create_single_source_cgal_program( "convex_hull.cpp" )
@@ -28,6 +27,16 @@ if ( CGAL_FOUND )
     create_single_source_cgal_program( "triangulation_data_structure_dynamic.cpp" )
     create_single_source_cgal_program( "triangulation_data_structure_static.cpp" )
 
+    foreach(target
+        barycentric_subdivision
+        delaunay_triangulation
+        convex_hull
+        regular_triangulation
+        triangulation
+        triangulation_data_structure_dynamic
+        triangulation_data_structure_static)
+      target_link_libraries(${target} PUBLIC CGAL::Eigen_support)
+    endforeach()
   else()
     message(STATUS "NOTICE: Some of the executables in this directory need Eigen 3.1 (or greater) and will not be compiled.")
   endif()
@@ -37,4 +46,3 @@ else()
     message(STATUS "This program requires the CGAL library, and will not be compiled.")
 
 endif()
-
diff -pruN 5.0.3-1/examples/Triangulation_2/draw_triangulation_2.cpp 5.1-1/examples/Triangulation_2/draw_triangulation_2.cpp
--- 5.0.3-1/examples/Triangulation_2/draw_triangulation_2.cpp	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Triangulation_2/draw_triangulation_2.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -7,8 +7,8 @@ typedef CGAL::Exact_predicates_inexact_c
 typedef CGAL::Triangulation_2<K>                            Triangulation;
 typedef Triangulation::Point                                Point;
 
-int main() {
-  std::ifstream in("data/triangulation_prog1.cin");
+int main(int argc, char* argv[]) {
+  std::ifstream in((argc>1)?argv[1]:"data/triangulation_prog1.cin");
   std::istream_iterator<Point> begin(in);
   std::istream_iterator<Point> end;
 
diff -pruN 5.0.3-1/examples/Triangulation_2/low_dimensional.cpp 5.1-1/examples/Triangulation_2/low_dimensional.cpp
--- 5.0.3-1/examples/Triangulation_2/low_dimensional.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Triangulation_2/low_dimensional.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,74 @@
+#include <iostream>
+
+#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
+#include <CGAL/Triangulation_2.h>
+
+typedef CGAL::Exact_predicates_inexact_constructions_kernel K;
+typedef CGAL::Triangulation_2<K>          Triangulation;
+typedef Triangulation::Vertex_handle      Vertex_handle;
+typedef Triangulation::Face_handle        Face_handle;
+typedef Triangulation::All_faces_iterator All_faces_iterator;
+typedef Triangulation::All_edges_iterator All_edges_iterator;
+typedef Triangulation::Point              Point;
+
+int main() {
+
+  Point p(0,0), q(1,0);
+
+  Triangulation t;
+
+  Vertex_handle inf = t.infinite_vertex();
+  Face_handle fh = inf->face();
+  assert(fh->vertex(0) == inf);
+  assert(fh->vertex(1) == Vertex_handle());
+  assert(fh->vertex(2) == Vertex_handle());
+
+  assert(t.all_faces_begin() == t.all_faces_end());
+  assert(t.all_edges_begin() == t.all_edges_end());
+
+  t.insert(p);
+  Vertex_handle pvh = t.finite_vertices_begin();
+  Face_handle pfh = pvh->face();
+  assert(pfh->neighbor(0) == fh);
+
+  t.insert(q);
+
+  assert(t.infinite_vertex()->face() == fh);
+
+  assert( (fh->vertex(0) == inf) || (fh->vertex(1) == inf) );
+
+  std::cout << "After the insertion of the second point" <<std::endl;
+  std::cout << "|V| = " << t.number_of_vertices() << std::endl;
+  std::cout << "|F| = " << t.number_of_faces() << std::endl;
+
+
+  // Even now we have not really faces
+  assert(t.all_faces_begin() == t.all_faces_end());
+
+  for (Triangulation::All_edges_iterator it = t.all_edges_begin();
+      it != t.all_edges_end();  ++it) {
+    Face_handle fh = it->first;
+    Vertex_handle v0 = fh->vertex(0);
+    Vertex_handle v1 = fh->vertex(1);
+    std::cout << "Edge: ";
+    if (v0 == inf) {std::cout << "inf -- ";}else{ std::cout<< v0->point() << " -- ";}
+    if (v1 == inf) {std::cout << "inf\n";}else{ std::cout<< v1->point() << std::endl;}
+  }
+
+  std::cout << "Edge traversal by hand" << std::endl;
+  Face_handle done = fh;
+  do {
+      assert(fh->vertex(2) == Vertex_handle());
+      assert(fh->neighbor(2) == Face_handle());
+      Vertex_handle v0 = fh->vertex(0);
+      Vertex_handle v1 = fh->vertex(1);
+      std::cout << "Edge: ";
+      if (v0 == inf) {std::cout << "inf -- ";}else{ std::cout<< v0->point() << " -- ";}
+      if (v1 == inf) {std::cout << "inf\n";}else{ std::cout<< v1->point() << std::endl;}
+      fh = fh->neighbor(0);
+  } while (fh != done);
+
+  std::cout << std::endl;
+
+  return 0;
+}
diff -pruN 5.0.3-1/examples/Triangulation_2/segment_soup_to_polylines.cpp 5.1-1/examples/Triangulation_2/segment_soup_to_polylines.cpp
--- 5.0.3-1/examples/Triangulation_2/segment_soup_to_polylines.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Triangulation_2/segment_soup_to_polylines.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,51 @@
+#include <CGAL/Exact_predicates_exact_constructions_kernel.h>
+#include <CGAL/Constrained_Delaunay_triangulation_2.h>
+#include <CGAL/Constrained_triangulation_plus_2.h>
+
+typedef CGAL::Exact_predicates_exact_constructions_kernel                 K;
+typedef CGAL::Constrained_Delaunay_triangulation_2<K>                     CDT;
+typedef CGAL::Constrained_triangulation_plus_2<CDT>                       CDTP;
+
+typedef CDTP::Point                                                       Point;
+typedef CDTP::Constraint_id                                               Cid;
+typedef CDTP::Vertex_handle                                               Vertex_handle;
+
+int main()
+{
+  CDTP cdtp;
+
+  // First polyline
+  cdtp.insert_constraint(Point(0,0), Point(1,1));
+  cdtp.insert_constraint(Point(1,1), Point(2,2));
+  cdtp.insert_constraint(Point(2,2), Point(3,3));
+
+  // Second polyline that cuts the first one in 2
+  cdtp.insert_constraint(Point(1,1), Point(1,2));
+  cdtp.insert_constraint(Point(1,2), Point(1,3));
+
+  // Third polyline
+  cdtp.insert_constraint(Point(10,10), Point(20,20));
+  cdtp.insert_constraint(Point(20,20), Point(30,30));
+
+  // Fourth polyline
+  cdtp.insert_constraint(Point(100,100), Point(200,200));
+
+  // Segment soup of 8 segments as input
+  std::cout << "Input CDT+ has " << cdtp.number_of_constraints() << " constraints/subconstraints" << std::endl;
+
+  std::cout << "Splitting subconstraints graph into constraints" << std::endl;
+  cdtp.split_subconstraint_graph_into_constraints();
+
+  // 5 polylines as output
+  std::cout << "Output CDT+ has " << cdtp.number_of_constraints() << " constraints:" << std::endl;
+
+  for (CDTP::Constraint_id cid : cdtp.constraints())
+  {
+    std::cout << " *";
+    for (CDTP::Vertex_handle vh : cdtp.vertices_in_constraint(cid))
+      std::cout << " (" << vh->point() << ")";
+    std::cout << std::endl;
+  }
+
+  return EXIT_SUCCESS;
+}
diff -pruN 5.0.3-1/examples/Triangulation_3/CMakeLists.txt 5.1-1/examples/Triangulation_3/CMakeLists.txt
--- 5.0.3-1/examples/Triangulation_3/CMakeLists.txt	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Triangulation_3/CMakeLists.txt	2020-09-08 12:12:21.000000000 +0000
@@ -39,16 +39,15 @@ if ( CGAL_FOUND )
   endif()
 
   find_package( TBB QUIET )
+  include(CGAL_TBB_support)
 
-  if( TBB_FOUND )
-    include( CGAL_target_use_TBB )
-
+  if( TARGET CGAL::TBB_support )
     create_single_source_cgal_program( "parallel_insertion_and_removal_in_regular_3.cpp" )
     create_single_source_cgal_program( "parallel_insertion_in_delaunay_3.cpp" )
     create_single_source_cgal_program( "sequential_parallel.cpp" )
-    CGAL_target_use_TBB( parallel_insertion_and_removal_in_regular_3 )
-    CGAL_target_use_TBB( parallel_insertion_in_delaunay_3 )
-    CGAL_target_use_TBB( sequential_parallel )
+    target_link_libraries( parallel_insertion_and_removal_in_regular_3 PUBLIC CGAL::TBB_support)
+    target_link_libraries( parallel_insertion_in_delaunay_3 PUBLIC CGAL::TBB_support )
+    target_link_libraries( sequential_parallel  PUBLIC CGAL::TBB_support)
   else()
     message(STATUS "NOTICE: a few examples require TBB and will not be compiled.")
   endif()
@@ -58,4 +57,3 @@ else()
     message(STATUS "This program requires the CGAL library, and will not be compiled.")
 
 endif()
-
diff -pruN 5.0.3-1/examples/Voronoi_diagram_2/CMakeLists.txt 5.1-1/examples/Voronoi_diagram_2/CMakeLists.txt
--- 5.0.3-1/examples/Voronoi_diagram_2/CMakeLists.txt	2020-08-01 19:00:34.000000000 +0000
+++ 5.1-1/examples/Voronoi_diagram_2/CMakeLists.txt	2020-09-08 12:12:21.000000000 +0000
@@ -6,7 +6,11 @@ cmake_minimum_required(VERSION 3.1...3.1
 project( Voronoi_diagram_2_Examples )
 
 
-find_package(CGAL QUIET)
+find_package(CGAL COMPONENTS Qt5)
+
+if(CGAL_Qt5_FOUND)
+  add_definitions(-DCGAL_USE_BASIC_VIEWER -DQT_NO_KEYWORDS)
+endif()
 
 if ( CGAL_FOUND )
 
@@ -16,6 +20,11 @@ if ( CGAL_FOUND )
     create_single_source_cgal_program( "${cppfile}" )
   endforeach()
 
+  if(CGAL_Qt5_FOUND )
+    target_link_libraries(draw_voronoi_diagram_2 PUBLIC CGAL::CGAL_Qt5)
+  endif()
+
+
 else()
 
     message(STATUS "This program requires the CGAL library, and will not be compiled.")
diff -pruN 5.0.3-1/examples/Voronoi_diagram_2/data/data4.dt.cin 5.1-1/examples/Voronoi_diagram_2/data/data4.dt.cin
--- 5.0.3-1/examples/Voronoi_diagram_2/data/data4.dt.cin	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Voronoi_diagram_2/data/data4.dt.cin	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,10 @@
+0 0
+100 0
+100 100
+0 100
+200 0
+300 0
+350 0
+150 150
+100 150
+300 325
diff -pruN 5.0.3-1/examples/Voronoi_diagram_2/draw_voronoi_diagram_2.cpp 5.1-1/examples/Voronoi_diagram_2/draw_voronoi_diagram_2.cpp
--- 5.0.3-1/examples/Voronoi_diagram_2/draw_voronoi_diagram_2.cpp	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/examples/Voronoi_diagram_2/draw_voronoi_diagram_2.cpp	2020-09-08 12:12:21.000000000 +0000
@@ -0,0 +1,37 @@
+// standard includes
+#include <fstream>
+
+// includes for drawing the Voronoi Diagram
+#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
+#include <CGAL/Voronoi_diagram_2.h>
+#include <CGAL/Delaunay_triangulation_2.h>
+#include <CGAL/Delaunay_triangulation_adaptation_traits_2.h>
+#include <CGAL/Delaunay_triangulation_adaptation_policies_2.h>
+#include <CGAL/draw_voronoi_diagram_2.h>
+
+// typedefs for defining the adaptor
+typedef CGAL::Exact_predicates_inexact_constructions_kernel                  K;
+typedef CGAL::Delaunay_triangulation_2<K>                                    DT;
+typedef CGAL::Delaunay_triangulation_adaptation_traits_2<DT>                 AT;
+typedef CGAL::Delaunay_triangulation_caching_degeneracy_removal_policy_2<DT> AP;
+typedef CGAL::Voronoi_diagram_2<DT,AT,AP>                                    VD;
+
+// typedef for the result type of the point location
+typedef AT::Site_2                    Site_2;
+
+int main(int argc, char* argv[])
+{
+  VD vd;
+  std::ifstream ifs((argc>1)?argv[1]:"data/data4.dt.cin");
+  assert(ifs);
+
+  Site_2 t;
+  while ( ifs >> t ) { vd.insert(t); }
+  ifs.close();
+
+  assert( vd.is_valid() );
+
+  CGAL::draw(vd);
+
+  return EXIT_SUCCESS;
+}
diff -pruN 5.0.3-1/include/CGAL/AABB_face_graph_triangle_primitive.h 5.1-1/include/CGAL/AABB_face_graph_triangle_primitive.h
--- 5.0.3-1/include/CGAL/AABB_face_graph_triangle_primitive.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/AABB_face_graph_triangle_primitive.h	2020-09-08 12:14:29.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/AABB_tree/include/CGAL/AABB_face_graph_triangle_primitive.h $
-// $Id: AABB_face_graph_triangle_primitive.h ff82af6 2020-07-08T12:57:05+02:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/AABB_tree/include/CGAL/AABB_face_graph_triangle_primitive.h $
+// $Id: AABB_face_graph_triangle_primitive.h 18f4e44 2020-07-07T11:44:40+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -38,7 +38,7 @@ namespace CGAL {
  *\tparam FaceGraph is a model of the face graph concept.
  *\tparam VertexPointPMap  is a property map with `boost::graph_traits<FaceGraph>::%vertex_descriptor`
  *   as key type and a \cgal Kernel `Point_3` as value type.
- *                         The default is `typename boost::property_map< FaceGraph,vertex_point_t>::%type`.
+ *                         The default is `typename boost::property_map< FaceGraph,vertex_point_t>::%const_type`.
  *\tparam OneFaceGraphPerTree is either `CGAL::Tag_true` or `CGAL::Tag_false`.
  * In the former case, we guarantee that all the primitives will be from a
  * common `FaceGraph` and some data will be factorized so that the size of
@@ -130,7 +130,7 @@ public:
   // constructors
   /*!
     \tparam Iterator an input iterator with `Id` as value type.
-    Constructs a primitive.
+    constructs a primitive.
     If `VertexPointPMap` is the default of the class, an additional constructor
     is available with `vppm` set to `get(vertex_point, graph)`.
   */
@@ -142,7 +142,7 @@ public:
   {}
 
   /*!
-    Constructs a primitive.
+    constructs a primitive.
     If `VertexPointPMap` is the default of the class, an additional constructor
     is available with `vppm` set to `get(vertex_point, graph)`.
   */
diff -pruN 5.0.3-1/include/CGAL/AABB_halfedge_graph_segment_primitive.h 5.1-1/include/CGAL/AABB_halfedge_graph_segment_primitive.h
--- 5.0.3-1/include/CGAL/AABB_halfedge_graph_segment_primitive.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/AABB_halfedge_graph_segment_primitive.h	2020-09-08 12:14:29.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/AABB_tree/include/CGAL/AABB_halfedge_graph_segment_primitive.h $
-// $Id: AABB_halfedge_graph_segment_primitive.h ff82af6 2020-07-08T12:57:05+02:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/AABB_tree/include/CGAL/AABB_halfedge_graph_segment_primitive.h $
+// $Id: AABB_halfedge_graph_segment_primitive.h 62c15ff 2020-07-08T12:52:12+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -49,7 +49,7 @@ namespace CGAL {
  * \tparam HalfedgeGraph is a model of the halfedge graph concept.
  *   as key type and a \cgal Kernel `Point_3` as value type.
  * \tparam VertexPointPMap is a property map with `boost::graph_traits<HalfedgeGraph>::%vertex_descriptor`.
- *                         The default is `typename boost::property_map< HalfedgeGraph,vertex_point_t>::%type`.
+ *                         The default is `typename boost::property_map< HalfedgeGraph,vertex_point_t>::%const_type`.
  * \tparam OneHalfedgeGraphPerTree is either `CGAL::Tag_true` or `CGAL::Tag_false`.
  * In the former case, we guarantee that all the primitives will be from a
  * common `HalfedgeGraph` and some data will be factorized so that the size of
@@ -77,17 +77,17 @@ class AABB_halfedge_graph_segment_primit
                               HalfedgeGraph,
                               typename Default::Get<VertexPointPMap,
                                                     typename boost::property_map< HalfedgeGraph,
-                                                                                  vertex_point_t>::type >::type >,
+                                                                                  vertex_point_t>::const_type >::type >,
                             Source_point_from_edge_descriptor_map<
                               HalfedgeGraph,
                               typename Default::Get<VertexPointPMap,
                                                     typename boost::property_map< HalfedgeGraph,
-                                                                                  vertex_point_t>::type >::type >,
+                                                                                  vertex_point_t>::const_type >::type >,
                             OneHalfedgeGraphPerTree,
                             CacheDatum >
 #endif
 {
-  typedef typename Default::Get<VertexPointPMap,typename boost::property_map< HalfedgeGraph,vertex_point_t>::type >::type  VertexPointPMap_;
+  typedef typename Default::Get<VertexPointPMap,typename boost::property_map< HalfedgeGraph,vertex_point_t>::const_type >::type  VertexPointPMap_;
   typedef typename boost::graph_traits<HalfedgeGraph>::edge_descriptor ED;
   typedef typename boost::mpl::if_<OneHalfedgeGraphPerTree, ED, std::pair<ED, const HalfedgeGraph*> >::type Id_;
 
@@ -118,7 +118,7 @@ public:
   /*!
   The point type.
   */
-  typedef boost::property_traits< boost::property_map< HalfedgeGraph, vertex_point_t>::type >::value_type Point;
+  typedef boost::property_traits< boost::property_map< HalfedgeGraph, vertex_point_t>::const_type >::value_type Point;
   /*!
   Geometric data type.
   */
@@ -141,7 +141,7 @@ public:
   typedef typename boost::graph_traits<HalfedgeGraph>::edge_descriptor edge_descriptor;
 
   /*!
-  Constructs a primitive.
+  constructs a primitive.
   \tparam Iterator is an input iterator with `Id` as value type.
   This \ref AABB_tree/AABB_halfedge_graph_edge_example.cpp "example" gives a way to call this constructor
   using the insert-by-range method of the class `AABB_tree<Traits>`.
@@ -156,7 +156,7 @@ public:
   {}
 
   /*!
-  Constructs a primitive.
+  constructs a primitive.
   If `VertexPointPMap` is the default of the class, an additional constructor
   is available with `vppm` set to `boost::get(vertex_point, graph)`.
   */
diff -pruN 5.0.3-1/include/CGAL/AABB_polyhedral_oracle.h 5.1-1/include/CGAL/AABB_polyhedral_oracle.h
--- 5.0.3-1/include/CGAL/AABB_polyhedral_oracle.h	2020-08-01 19:02:56.000000000 +0000
+++ 5.1-1/include/CGAL/AABB_polyhedral_oracle.h	2020-09-08 12:15:47.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/AABB_polyhedral_oracle.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/AABB_polyhedral_oracle.h $
 // $Id: AABB_polyhedral_oracle.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/AABB_polyhedron_segment_primitive.h 5.1-1/include/CGAL/AABB_polyhedron_segment_primitive.h
--- 5.0.3-1/include/CGAL/AABB_polyhedron_segment_primitive.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/AABB_polyhedron_segment_primitive.h	2020-09-08 12:14:29.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/AABB_tree/include/CGAL/AABB_polyhedron_segment_primitive.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/AABB_tree/include/CGAL/AABB_polyhedron_segment_primitive.h $
 // $Id: AABB_polyhedron_segment_primitive.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/AABB_polyhedron_triangle_primitive.h 5.1-1/include/CGAL/AABB_polyhedron_triangle_primitive.h
--- 5.0.3-1/include/CGAL/AABB_polyhedron_triangle_primitive.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/AABB_polyhedron_triangle_primitive.h	2020-09-08 12:14:29.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/AABB_tree/include/CGAL/AABB_polyhedron_triangle_primitive.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/AABB_tree/include/CGAL/AABB_polyhedron_triangle_primitive.h $
 // $Id: AABB_polyhedron_triangle_primitive.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/AABB_primitive.h 5.1-1/include/CGAL/AABB_primitive.h
--- 5.0.3-1/include/CGAL/AABB_primitive.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/AABB_primitive.h	2020-09-08 12:14:29.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/AABB_tree/include/CGAL/AABB_primitive.h $
-// $Id: AABB_primitive.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/AABB_tree/include/CGAL/AABB_primitive.h $
+// $Id: AABB_primitive.h 5ec5ad6 2020-04-22T16:23:57+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -112,14 +112,14 @@ struct AABB_primitive
   /// @}
 
   /*!
-  Constructs a primitive and initializes the property maps.
+  constructs a primitive and initializes the property maps.
   */
   AABB_primitive(Id id,
                  ObjectPropertyMap o_pmap=ObjectPropertyMap(),
                  PointPropertyMap p_pmap=PointPropertyMap());
 
   /*!
-  Constructs a primitive from an iterator with `Id` as value type
+  constructs a primitive from an iterator with `Id` as value type
   and initializes the property maps.
   */
   template <class Iterator>
diff -pruN 5.0.3-1/include/CGAL/AABB_segment_primitive.h 5.1-1/include/CGAL/AABB_segment_primitive.h
--- 5.0.3-1/include/CGAL/AABB_segment_primitive.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/AABB_segment_primitive.h	2020-09-08 12:14:29.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/AABB_tree/include/CGAL/AABB_segment_primitive.h $
-// $Id: AABB_segment_primitive.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/AABB_tree/include/CGAL/AABB_segment_primitive.h $
+// $Id: AABB_segment_primitive.h 5ec5ad6 2020-04-22T16:23:57+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -88,7 +88,7 @@ class AABB_segment_primitive
                           Tag_false,
                           CacheDatum > Base;
 public:
-  ///Constructor from an iterator
+  ///constructor from an iterator
   AABB_segment_primitive(Iterator it) : Base(it){}
 };
 
diff -pruN 5.0.3-1/include/CGAL/AABB_traits.h 5.1-1/include/CGAL/AABB_traits.h
--- 5.0.3-1/include/CGAL/AABB_traits.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/AABB_traits.h	2020-09-08 12:14:29.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/AABB_tree/include/CGAL/AABB_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/AABB_tree/include/CGAL/AABB_traits.h $
 // $Id: AABB_traits.h 2e8a59d 2020-07-21T15:25:54+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/AABB_tree.h 5.1-1/include/CGAL/AABB_tree.h
--- 5.0.3-1/include/CGAL/AABB_tree.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/AABB_tree.h	2020-09-08 12:14:29.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/AABB_tree/include/CGAL/AABB_tree.h $
-// $Id: AABB_tree.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/AABB_tree/include/CGAL/AABB_tree.h $
+// $Id: AABB_tree.h 1379710 2020-04-29T17:02:48+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -39,7 +39,7 @@ namespace CGAL {
 /// @{
 
   /**
-   * Class AABB_tree is a static data structure for efficient
+   * Static data structure for efficient
    * intersection and distance computations in 3D. It builds a
    * hierarchy of axis-aligned bounding boxes (an AABB tree) from a set
    * of 3D geometric objects, and can receive intersection and distance
@@ -61,6 +61,8 @@ namespace CGAL {
     // type of the primitives container
     typedef std::vector<typename AABBTraits::Primitive> Primitives;
 
+    typedef internal::Primitive_helper<AABBTraits> Helper;
+
   public:
     typedef AABBTraits AABB_traits;
 
@@ -107,7 +109,7 @@ namespace CGAL {
     /// \name Creation
     ///@{
 
-    /// Constructs an empty tree, and initializes the internally stored traits
+    /// constructs an empty tree, and initializes the internally stored traits
     /// class using `traits`.
     AABB_tree(const AABBTraits& traits = AABBTraits());
 
@@ -116,39 +118,45 @@ namespace CGAL {
      * @param first iterator over first primitive to insert
      * @param beyond past-the-end iterator
      *
-     * It is equivalent to constructing an empty tree and calling `insert(first,last,t...)`.
+     * constructs an empty tree followed by a call to `insert(first,last,t...)`.
      * The tree stays empty if the memory allocation is not successful.
      */
     template<typename InputIterator,typename ... T>
     AABB_tree(InputIterator first, InputIterator beyond,T&& ...);
 
-    /// After one or more calls to `insert()` the internal data
-    /// structure of the tree must be reconstructed. This procedure
-    /// has a complexity of \f$O(n log(n))\f$, where \f$n\f$ is the number of
-    /// primitives of the tree.  This procedure is called implicitly
-    /// at the first call to a query member function. You can call
-    /// `build()` explicitly to ensure that the next call to
-    /// query functions will not trigger the reconstruction of the
-    /// data structure.
-    /// A call to `AABBTraits::set_shared_data(t...)`
-    /// is made using the internally stored traits.
+    /// triggers the (re)construction of the internal tree structure.
+    /// The internal tree structure is automatically invalidated by the insertion of any primitives
+    /// after one or more calls to `insert()`.
+    /// This procedure is called implicitly at the first call to a query member function.
+    /// An explicit call to `build()` must be made to ensure that the next call to
+    /// a query function will not trigger the construction of the data structure.
+    /// A call to `AABBTraits::set_shared_data(t...)` is made using the internally stored traits.
+    /// This procedure has a complexity of \f$O(n log(n))\f$, where \f$n\f$ is the number of
+    /// primitives of the tree.
     template<typename ... T>
     void build(T&& ...);
 #ifndef DOXYGEN_RUNNING
     void build();
+
+    /// triggers the (re)construction of the tree similarly to a call to `build()`
+    /// but the traits functors `Compute_bbox` and `Split_primitives` are ignored
+    /// and `compute_bbox` and `split_primitives` are used instead.
+    template <class ComputeBbox, class SplitPrimitives>
+    void custom_build(const ComputeBbox& compute_bbox,
+                      const SplitPrimitives& split_primitives);
 #endif
     ///@}
 
     /// \name Operations
     ///@{
 
-    /// Equivalent to calling `clear()` and then `insert(first,last,t...)`.
+    /// is equivalent to calling `clear()`, `insert(first,last,t...)`, and `build()`
     template<typename ConstPrimitiveIterator,typename ... T>
     void rebuild(ConstPrimitiveIterator first, ConstPrimitiveIterator beyond,T&& ...);
 
 
-    /// Add a sequence of primitives to the set of primitives of the AABB tree.
-    /// `%InputIterator` is any iterator and the parameter pack `T` are any types
+    /// adds a sequence of primitives to the set of primitives of the AABB tree.
+    /// `%InputIterator` is any iterator and the parameter pack `T` contains any types
     /// such that `Primitive` has a constructor with the following signature:
     /// `Primitive(%InputIterator, T...)`. If `Primitive` is a model of the concept
     /// `AABBPrimitiveWithSharedData`, a call to `AABBTraits::set_shared_data(t...)`
@@ -156,30 +164,31 @@ namespace CGAL {
     template<typename InputIterator,typename ... T>
     void insert(InputIterator first, InputIterator beyond,T&& ...);
 
-    /// Adds a primitive to the set of primitives of the tree.
+    /// adds a primitive to the set of primitives of the tree.
     inline void insert(const Primitive& p);
 
-    /// Clears and destroys the tree.
+    /// clears and destroys the tree.
     ~AABB_tree()
     {
       clear();
     }
-    /// Returns a const reference to the internally stored traits class.
+    /// returns a const reference to the internally stored traits class.
     const AABBTraits& traits() const{
       return m_traits;
     }
 
-    /// Clears the tree.
+    /// clears the tree and the search tree if it was constructed,
+    /// and switches on the usage of the search tree to find the hint for the distance queries
     void clear()
     {
       // clear AABB tree
       clear_nodes();
       m_primitives.clear();
       clear_search_tree();
-      m_default_search_tree_constructed = false;
+      m_use_default_search_tree = true;
     }
 
-    /// Returns the axis-aligned bounding box of the whole tree.
+    /// returns the axis-aligned bounding box of the whole tree.
     /// \pre `!empty()`
     const Bounding_box bbox() const {
       CGAL_precondition(!empty());
@@ -190,10 +199,10 @@ namespace CGAL {
                       m_primitives.end());
     }
 
-    /// Returns the number of primitives in the tree.
+    /// returns the number of primitives in the tree.
     size_type size() const { return m_primitives.size(); }
 
-    /// Returns \c true, iff the tree contains no primitive.
+    /// returns \c true, iff the tree contains no primitive.
     bool empty() const { return m_primitives.empty(); }
     ///@}
 
@@ -209,47 +218,46 @@ namespace CGAL {
       set_primitive_data_impl(CGAL::Boolean_tag<internal::Has_nested_type_Shared_data<Primitive>::value>(),std::forward<T>(t)...);
     }
 
-    bool build_kd_tree() const;
+    bool build_kd_tree();
     template<typename ConstPointIterator>
-    bool build_kd_tree(ConstPointIterator first, ConstPointIterator beyond) const;
+    bool build_kd_tree(ConstPointIterator first, ConstPointIterator beyond);
 public:
 
     /// \name Intersection Tests
     ///@{
 
-    /// Returns `true`, iff the query intersects at least one of
-    /// the input primitives. \tparam Query must be a type for
-    /// which `do_intersect` predicates are
-    /// defined in the traits class `AABBTraits`.
+    /// returns `true`, iff the query intersects at least one of
+    /// the input primitives.
+    /// \tparam Query must be a type for which `Do_intersect` operators are
+    ///               defined in the traits class `AABBTraits`.
     template<typename Query>
     bool do_intersect(const Query& query) const;
 
-    /// Returns the number of primitives intersected by the
-    /// query. \tparam Query must be a type for which
-    /// `do_intersect` predicates are defined
-    /// in the traits class `AABBTraits`.
+    /// returns the number of primitives intersected by the
+    /// query.
+    /// \tparam Query must be a type for which `Do_intersect` operators are
+    ///               defined in the traits class `AABBTraits`.
     template<typename Query>
     size_type number_of_intersected_primitives(const Query& query) const;
 
-    /// Outputs to the iterator the list of all intersected primitives
-    /// ids. This function does not compute the intersection points
+    /// puts in `out` the ids of all intersected primitives.
+    /// This function does not compute the intersection points
     /// and is hence faster than the function `all_intersections()`
-    /// function below. \tparam Query must be a type for which
-    /// `do_intersect` predicates are defined
-    /// in the traits class `AABBTraits`.
+    /// function below.
+    /// \tparam Query must be a type for which `Do_intersect` operators are
+    ///               defined in the traits class `AABBTraits`.
     template<typename Query, typename OutputIterator>
     OutputIterator all_intersected_primitives(const Query& query, OutputIterator out) const;
 
 
-    /// Returns the intersected primitive id that is encountered first
+    /// returns the id of the intersected primitive that is encountered first
     /// in the tree traversal, iff
     /// the query intersects at least one of the input primitives. No
     /// particular order is guaranteed over the tree traversal, such
     /// that, e.g, the primitive returned is not necessarily the
-    /// closest from the source point of a ray query. \tparam Query
-    /// must be a type for which
-    /// `do_intersect` predicates are defined
-    /// in the traits class `AABBTraits`.
+    /// closest from the source point of a ray query.
+    /// \tparam Query must be a type for which `Do_intersect` operators are
+    ///               defined in the traits class `AABBTraits`.
     template <typename Query>
     boost::optional<Primitive_id> any_intersected_primitive(const Query& query) const;
     ///@}
@@ -257,30 +265,29 @@ public:
     /// \name Intersections
     ///@{
 
-    /// Outputs the list of all intersections, as objects of
+    /// puts in `out` all intersections, as objects of
     /// `Intersection_and_primitive_id<Query>::%Type`,
     /// between the query and the input data to
-    /// the iterator. `do_intersect()`
-    /// predicates and intersections must be defined for `Query`
-    /// in the `AABBTraits` class.
+    /// the iterator.
+    /// \tparam Query must be a type for which `Do_intersect` and `Intersection` operators are
+    ///               defined in the traits class `AABBTraits`.
     template<typename Query, typename OutputIterator>
     OutputIterator all_intersections(const Query& query, OutputIterator out) const;
 
 
-    /// Returns the intersection that is encountered first
+    /// returns if any the intersection that is encountered first
     /// in the tree traversal. No particular
     /// order is guaranteed over the tree traversal, e.g, the
-    /// primitive returned is not necessarily the closest from the
-    /// source point of a ray query. Type `Query` must be a type
-    /// for which `do_intersect` predicates
-    /// and intersections are defined in the traits class AABBTraits.
+    /// primitive returned is not necessarily the closest from the query.
+    /// \tparam Query must be a type for which `Do_intersect` and `Intersection` operators are
+    ///               defined in the traits class `AABBTraits`.
     template <typename Query>
     boost::optional< typename Intersection_and_primitive_id<Query>::Type >
     any_intersection(const Query& query) const;
 
 
 
-    /// Returns the intersection and  primitive id closest to the source point of the ray
+    /// returns the intersection and  primitive id closest to the source point of the ray
     /// query.
     /// \tparam Ray must be the same as `AABBTraits::Ray_3` and
     /// `do_intersect` predicates and intersections for it must be
@@ -311,7 +318,7 @@ public:
     }
     /// \endcond
 
-    /// Returns the primitive id closest to the source point of the ray
+    /// returns the primitive id closest to the source point of the ray
     /// query.
     /// \tparam Ray must be the same as `AABBTraits::Ray_3` and
     /// `do_intersect` predicates and intersections for it must be
@@ -340,31 +347,22 @@ public:
     /// \name Distance Queries
     ///@{
 
-    /// Returns the minimum squared distance between the query point
-    /// and all input primitives. Method
-    /// `accelerate_distance_queries()` should be called before the
-    /// first distance query, so that an internal secondary search
-    /// structure is build, for improving performance.
+    /// returns the minimum squared distance between the query point
+    /// and all input primitives.
     /// \pre `!empty()`
     FT squared_distance(const Point& query) const;
 
-    /// Returns the point in the union of all input primitives which
+    /// returns the point in the union of all input primitives which
     /// is closest to the query. In case there are several closest
     /// points, one arbitrarily chosen closest point is
-    /// returned. Method `accelerate_distance_queries()` should be
-    /// called before the first distance query, so that an internal
-    /// secondary search structure is build, for improving
-    /// performance.
+    /// returned.
     /// \pre `!empty()`
     Point closest_point(const Point& query) const;
 
 
-    /// Returns a `Point_and_primitive_id` which realizes the
+    /// returns a `Point_and_primitive_id` which realizes the
     /// smallest distance between the query point and all input
-    /// primitives. Method `accelerate_distance_queries()` should be
-    /// called before the first distance query, so that an internal
-    /// secondary search structure is build, for improving
-    /// performance.
+    /// primitives.
     /// \pre `!empty()`
     Point_and_primitive_id closest_point_and_primitive(const Point& query) const;
 
@@ -404,19 +402,18 @@ public:
     /// one may want to provide a much better hint than a vertex of
     /// the triangle soup could be. It could be, for example, the
     /// barycenter of one of the triangles. But, except with the use
-    /// of an exact constructions kernel, one cannot easily construct
+    /// of a kernel with exact constructions, one cannot easily construct
     /// points other than the vertices, that lie exactly on a triangle
     /// soup. Hence, providing a good hint sometimes means not being
     /// able to provide it exactly on the primitives. In rare
     /// occasions, this hint can be returned as the closest point.
     /// In order to accelerate distance queries significantly, the
     /// AABB tree builds an internal KD-tree containing a set of
-    /// potential hints, when the method
-    /// `accelerate_distance_queries()` is called. This KD-tree
-    /// provides very good hints that allow the algorithms to run much
-    /// faster than with a default hint (such as the
-    /// `reference_point` of the first primitive). The set of
-    /// potential hints is a sampling of the union of the primitives,
+    /// potential hints. This KD-tree provides very good hints
+    /// that allow the algorithms to run much faster than
+    /// when `do_not_accelerate_distance_queries()` that makes the
+    /// hint to always be the  `reference_point` of the first primitive.
+    /// The set of potential hints is a sampling of the union of the primitives,
     /// which is obtained, by default, by calling the method
     /// `reference_point` of each of the primitives. However, such
     /// a sampling with one point per primitive may not be the most
@@ -424,43 +421,46 @@ public:
     /// inserting more than one sample on them. Conversely, a sparser
     /// sampling with less than one point per input primitive is
     /// relevant in some cases.
+    /// The internal KD-tree is always used if no call to `do_not_accelerate_distance_queries()`
+    /// was made since object creation or the last call to `clear()`. It will be built by
+    /// the first distance query or by a call to `accelerate_distance_queries()`.
     ///@{
 
-    /// Constructs internal search tree from
+    /// constructs the internal search tree from
     /// a point set taken on the internal primitives
     /// returns `true` iff successful memory allocation
-    bool accelerate_distance_queries() const;
+    bool accelerate_distance_queries();
+    /// turns off the usage of the internal search tree and clears it if it was already constructed.
+    void do_not_accelerate_distance_queries();
 
-    /// Constructs an internal KD-tree containing the specified point
+    /// constructs an internal KD-tree containing the specified point
     /// set, to be used as the set of potential hints for accelerating
-    /// the distance queries.
+    /// the distance queries. Note that the search tree built in
+    /// this function will not be invalidated by the insertion of a new
+    /// primitive, and an explicit call to `accelerate_distance_queries()`
+    /// is needed to update the search tree.
     /// \tparam ConstPointIterator is an iterator with
     /// value type `Point_and_primitive_id`.
     template<typename ConstPointIterator>
-    bool accelerate_distance_queries(ConstPointIterator first, ConstPointIterator beyond) const
+    bool accelerate_distance_queries(ConstPointIterator first, ConstPointIterator beyond)
     {
-      #ifdef CGAL_HAS_THREADS
-      //this ensures that this is done once at a time
-      CGAL_SCOPED_LOCK(kd_tree_mutex);
-      #endif
-      clear_search_tree();
-      m_default_search_tree_constructed = false; // not a default kd-tree
+      m_use_default_search_tree = false;
       return build_kd_tree(first,beyond);
     }
 
-    /// Returns the minimum squared distance between the query point
+    /// returns the minimum squared distance between the query point
     /// and all input primitives. The internal KD-tree is not used.
     /// \pre `!empty()`
     FT squared_distance(const Point& query, const Point& hint) const;
 
-    /// Returns the point in the union of all input primitives which
+    /// returns the point in the union of all input primitives which
     /// is closest to the query. In case there are several closest
     /// points, one arbitrarily chosen closest point is returned. The
     /// internal KD-tree is not used.
     /// \pre `!empty()`
     Point closest_point(const Point& query, const Point& hint) const;
 
-    /// Returns a `Point_and_primitive_id` which realizes the
+    /// returns a `Point_and_primitive_id` which realizes the
     /// smallest distance between the query point and all input
     /// primitives. The internal KD-tree is not used.
     /// \pre `!empty()`
@@ -482,15 +482,23 @@ public:
     }
 
     // clears internal KD tree
-    void clear_search_tree() const
+    void clear_search_tree()
     {
+#ifdef CGAL_HAS_THREADS
+      if ( m_atomic_search_tree_constructed.load(std::memory_order_relaxed) )
+#else
       if ( m_search_tree_constructed )
+#endif
       {
         CGAL_assertion( m_p_search_tree!=nullptr );
         delete m_p_search_tree;
         m_p_search_tree = nullptr;
+#ifdef CGAL_HAS_THREADS
+        m_atomic_search_tree_constructed.store(false, std::memory_order_relaxed);
+#else
         m_search_tree_constructed = false;
-                        }
+#endif
+      }
     }
 
   public:
@@ -521,31 +529,43 @@ public:
     {
       CGAL_assertion(!empty());
       return Point_and_primitive_id(
-        internal::Primitive_helper<AABB_traits>::get_reference_point(m_primitives[0],m_traits), m_primitives[0].id()
+        Helper::get_reference_point(m_primitives[0],m_traits), m_primitives[0].id()
       );
     }
 
   public:
     Point_and_primitive_id best_hint(const Point& query) const
     {
+#ifdef CGAL_HAS_THREADS
+      bool m_search_tree_constructed = m_atomic_search_tree_constructed.load(std::memory_order_acquire);
+#endif
+
+      // lazily build the search tree in case the default should be used
+      if (m_use_default_search_tree && !m_search_tree_constructed)
+      {
+#ifdef CGAL_HAS_THREADS
+        CGAL_SCOPED_LOCK(build_mutex);
+        m_search_tree_constructed = m_atomic_search_tree_constructed.load(std::memory_order_relaxed);
+        if (!m_search_tree_constructed)
+#endif
+        m_search_tree_constructed = const_cast<AABB_tree*>(this)->build_kd_tree();
+      }
+
       if(m_search_tree_constructed)
-                        {
         return m_p_search_tree->closest_point(query);
-                        }
       else
         return this->any_reference_point_and_id();
     }
 
     //! Returns the datum (geometric object) represented `p`.
 #ifndef DOXYGEN_RUNNING
-    typename internal::Primitive_helper<AABBTraits>::Datum_type
+    typename Helper::Datum_type
 #else
     typename AABBTraits::Primitive::Datum_reference
 #endif
     datum(Primitive& p)const
     {
-      return internal::Primitive_helper<AABBTraits>::
-          get_datum(p, this->traits());
+      return Helper::get_datum(p, this->traits());
     }
 
   private:
@@ -554,21 +574,25 @@ public:
     // set of input primitives
     Primitives m_primitives;
     // single root node
-    Node* m_p_root_node;
+    Node* m_p_root_node = nullptr;
     #ifdef CGAL_HAS_THREADS
-    mutable CGAL_MUTEX internal_tree_mutex;//mutex used to protect const calls inducing build()
-    mutable CGAL_MUTEX kd_tree_mutex;//mutex used to protect calls to accelerate_distance_queries
+    mutable CGAL_MUTEX build_mutex; // mutex used to protect const calls inducing build() and build_kd_tree()
     #endif
 
     const Node* root_node() const {
       CGAL_assertion(size() > 1);
+
+#ifdef CGAL_HAS_THREADS
+      bool m_need_build = m_atomic_need_build.load(std::memory_order_acquire);
+#endif
       if(m_need_build){
-        #ifdef CGAL_HAS_THREADS
+#ifdef CGAL_HAS_THREADS
         //this ensures that build() will be called once
-        CGAL_SCOPED_LOCK(internal_tree_mutex);
+        CGAL_SCOPED_LOCK(build_mutex);
+        m_need_build = m_atomic_need_build.load(std::memory_order_relaxed);
         if(m_need_build)
-        #endif
-          const_cast< AABB_tree<AABBTraits>* >(this)->build();
+#endif
+        const_cast< AABB_tree<AABBTraits>* >(this)->build();
       }
       return m_p_root_node;
     }
@@ -579,10 +603,15 @@ public:
     }
 
     // search KD-tree
-    mutable const Search_tree* m_p_search_tree;
-    mutable bool m_search_tree_constructed;
-    mutable bool m_default_search_tree_constructed; // indicates whether the internal kd-tree should be built
-    bool m_need_build;
+    const Search_tree* m_p_search_tree = nullptr;
+    bool m_use_default_search_tree = true; // indicates whether the internal kd-tree should be built
+#ifdef CGAL_HAS_THREADS
+    std::atomic<bool> m_atomic_need_build;
+    std::atomic<bool> m_atomic_search_tree_constructed;
+#else
+    bool m_need_build = false;
+    mutable bool m_search_tree_constructed = false;
+#endif
 
   private:
     // Disabled copy constructor & assignment operator
@@ -597,12 +626,10 @@ public:
   template<typename Tr>
   AABB_tree<Tr>::AABB_tree(const Tr& traits)
     : m_traits(traits)
-    , m_primitives()
-    , m_p_root_node(nullptr)
-    , m_p_search_tree(nullptr)
-    , m_search_tree_constructed(false)
-    , m_default_search_tree_constructed(false)
-    , m_need_build(false)
+#ifdef CGAL_HAS_THREADS
+    , m_atomic_need_build(false)
+    , m_atomic_search_tree_constructed(false)
+#endif
   {}
 
    template<typename Tr>
@@ -610,13 +637,10 @@ public:
   AABB_tree<Tr>::AABB_tree(ConstPrimitiveIterator first,
                            ConstPrimitiveIterator beyond,
                            T&& ... t)
-    : m_traits()
-    , m_primitives()
-    , m_p_root_node(nullptr)
-    , m_p_search_tree(nullptr)
-    , m_search_tree_constructed(false)
-    , m_default_search_tree_constructed(false)
-    , m_need_build(false)
+#ifdef CGAL_HAS_THREADS
+    : m_atomic_need_build(false)
+    , m_atomic_search_tree_constructed(false)
+#endif
   {
     // Insert each primitive into tree
     insert(first, beyond,std::forward<T>(t)...);
@@ -628,13 +652,19 @@ public:
                              ConstPrimitiveIterator beyond,
                              T&& ... t)
   {
+    if (m_use_default_search_tree && first!=beyond)
+      clear_search_tree();
     set_shared_data(std::forward<T>(t)...);
     while(first != beyond)
     {
       m_primitives.push_back(Primitive(first,std::forward<T>(t)...));
       ++first;
     }
+#ifdef CGAL_HAS_THREADS
+    m_atomic_need_build.store(true, std::memory_order_relaxed);
+#else
     m_need_build = true;
+#endif
   }
 
   // Clears tree and insert a set of primitives
@@ -653,27 +683,42 @@ public:
     build();
   }
 
-        template<typename Tr>
-        template<typename ... T>
-        void AABB_tree<Tr>::build(T&& ... t)
-        {
-          set_shared_data(std::forward<T>(t)...);
-          build();
-        }
+  template<typename Tr>
+  template<typename ... T>
+  void AABB_tree<Tr>::build(T&& ... t)
+  {
+    set_shared_data(std::forward<T>(t)...);
+    build();
+  }
 
   template<typename Tr>
   void AABB_tree<Tr>::insert(const Primitive& p)
   {
+    if (m_use_default_search_tree)
+      clear_search_tree();
     m_primitives.push_back(p);
+#ifdef CGAL_HAS_THREADS
+    m_atomic_need_build.store(true, std::memory_order_relaxed);
+#else
     m_need_build = true;
+#endif
   }
 
-  // Build the data structure, after calls to insert(..)
   template<typename Tr>
   void AABB_tree<Tr>::build()
   {
+    custom_build(m_traits.compute_bbox_object(),
+                 m_traits.split_primitives_object());
+  }
+#ifndef DOXYGEN_RUNNING
+  // Build the data structure, after calls to insert(..)
+  template<typename Tr>
+  template <class ComputeBbox, class SplitPrimitives>
+  void AABB_tree<Tr>::custom_build(
+    const ComputeBbox& compute_bbox,
+    const SplitPrimitives& split_primitives)
+  {
     clear_nodes();
-
     if(m_primitives.size() > 1) {
 
       // allocates tree nodes
@@ -688,36 +733,31 @@ public:
 
       // constructs the tree
       m_p_root_node->expand(m_primitives.begin(), m_primitives.end(),
-                m_primitives.size(), m_traits);
+                            m_primitives.size(),
+                            compute_bbox,
+                            split_primitives,
+                            m_traits);
     }
-
-
-    // In case the users has switched on the accelerated distance query
-    // data structure with the default arguments, then it has to be
-    // /built/rebuilt.
-    if(m_default_search_tree_constructed)
-      build_kd_tree();
+#ifdef CGAL_HAS_THREADS
+    m_atomic_need_build.store(false, std::memory_order_release); // in case build() is triggered by a call to root_node()
+#else
     m_need_build = false;
+#endif
   }
+#endif
   // constructs the search KD tree from given points
   // to accelerate the distance queries
   template<typename Tr>
-  bool AABB_tree<Tr>::build_kd_tree() const
+  bool AABB_tree<Tr>::build_kd_tree()
   {
     // iterate over primitives to get reference points on them
     std::vector<Point_and_primitive_id> points;
     points.reserve(m_primitives.size());
-    typename Primitives::const_iterator it;
-    for(it = m_primitives.begin(); it != m_primitives.end(); ++it)
-      points.push_back( Point_and_primitive_id(
-        internal::Primitive_helper<AABB_traits>::get_reference_point(
-          *it,m_traits), it->id() ) );
+    for(const Primitive& p : m_primitives)
+      points.push_back( Point_and_primitive_id( Helper::get_reference_point(p, m_traits), p.id() ) );
 
     // clears current KD tree
-    clear_search_tree();
-    bool res = build_kd_tree(points.begin(), points.end());
-    m_default_search_tree_constructed = true;
-    return res;
+    return build_kd_tree(points.begin(), points.end());
   }
 
   // constructs the search KD tree from given points
@@ -725,13 +765,18 @@ public:
   template<typename Tr>
   template<typename ConstPointIterator>
   bool AABB_tree<Tr>::build_kd_tree(ConstPointIterator first,
-    ConstPointIterator beyond) const
+                                    ConstPointIterator beyond)
   {
+    clear_search_tree();
     m_p_search_tree = new Search_tree(first, beyond);
-                m_default_search_tree_constructed = true;
+
     if(m_p_search_tree != nullptr)
     {
+#ifdef CGAL_HAS_THREADS
+      m_atomic_search_tree_constructed.store(true, std::memory_order_release); // in case build_kd_tree() is triggered by a call to best_hint()
+#else
       m_search_tree_constructed = true;
+#endif
       return true;
     }
     else
@@ -741,34 +786,21 @@ public:
     }
   }
 
+  template<typename Tr>
+  void AABB_tree<Tr>::do_not_accelerate_distance_queries()
+  {
+    clear_search_tree();
+    m_use_default_search_tree = false;
+  }
+
+
   // constructs the search KD tree from internal primitives
   template<typename Tr>
-  bool AABB_tree<Tr>::accelerate_distance_queries() const
+  bool AABB_tree<Tr>::accelerate_distance_queries()
   {
+    m_use_default_search_tree = true;
     if(m_primitives.empty()) return true;
-    if (m_default_search_tree_constructed)
-    {
-      if (!m_need_build) return m_search_tree_constructed;
-      return true; // default return type, no tree built
-    }
-
-    if(!m_need_build) // the tree was already built, build the kd-tree
-    {
-      #ifdef CGAL_HAS_THREADS
-      //this ensures that this function will be done once
-      CGAL_SCOPED_LOCK(kd_tree_mutex);
-      #endif
-      if (!m_need_build)
-      {
-        // clears current KD tree
-        clear_search_tree();
-        bool res = build_kd_tree();
-        m_default_search_tree_constructed = true;
-        return res;
-      };
-    }
-    m_default_search_tree_constructed = true;
-    return m_search_tree_constructed;
+    return build_kd_tree();
   }
 
   template<typename Tr>
diff -pruN 5.0.3-1/include/CGAL/AABB_triangle_primitive.h 5.1-1/include/CGAL/AABB_triangle_primitive.h
--- 5.0.3-1/include/CGAL/AABB_triangle_primitive.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/AABB_triangle_primitive.h	2020-09-08 12:14:29.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/AABB_tree/include/CGAL/AABB_triangle_primitive.h $
-// $Id: AABB_triangle_primitive.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/AABB_tree/include/CGAL/AABB_triangle_primitive.h $
+// $Id: AABB_triangle_primitive.h 5ec5ad6 2020-04-22T16:23:57+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -88,7 +88,7 @@ class AABB_triangle_primitive
                           Tag_false,
                           CacheDatum > Base;
 public:
-  ///Constructor from an iterator
+  ///constructor from an iterator
   AABB_triangle_primitive(Iterator it) : Base(it){}
 };
 
diff -pruN 5.0.3-1/include/CGAL/AABB_triangulation_3_cell_primitive.h 5.1-1/include/CGAL/AABB_triangulation_3_cell_primitive.h
--- 5.0.3-1/include/CGAL/AABB_triangulation_3_cell_primitive.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/AABB_triangulation_3_cell_primitive.h	2020-09-08 12:14:29.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/AABB_tree/include/CGAL/AABB_triangulation_3_cell_primitive.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/AABB_tree/include/CGAL/AABB_triangulation_3_cell_primitive.h $
 // $Id: AABB_triangulation_3_cell_primitive.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/AABB_triangulation_3_triangle_primitive.h 5.1-1/include/CGAL/AABB_triangulation_3_triangle_primitive.h
--- 5.0.3-1/include/CGAL/AABB_triangulation_3_triangle_primitive.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/AABB_triangulation_3_triangle_primitive.h	2020-09-08 12:14:29.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/AABB_tree/include/CGAL/AABB_triangulation_3_triangle_primitive.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/AABB_tree/include/CGAL/AABB_triangulation_3_triangle_primitive.h $
 // $Id: AABB_triangulation_3_triangle_primitive.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Advancing_front_surface_reconstruction_cell_base_3.h 5.1-1/include/CGAL/Advancing_front_surface_reconstruction_cell_base_3.h
--- 5.0.3-1/include/CGAL/Advancing_front_surface_reconstruction_cell_base_3.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/Advancing_front_surface_reconstruction_cell_base_3.h	2020-09-08 12:14:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Advancing_front_surface_reconstruction/include/CGAL/Advancing_front_surface_reconstruction_cell_base_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Advancing_front_surface_reconstruction/include/CGAL/Advancing_front_surface_reconstruction_cell_base_3.h $
 // $Id: Advancing_front_surface_reconstruction_cell_base_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Advancing_front_surface_reconstruction.h 5.1-1/include/CGAL/Advancing_front_surface_reconstruction.h
--- 5.0.3-1/include/CGAL/Advancing_front_surface_reconstruction.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/Advancing_front_surface_reconstruction.h	2020-09-08 12:14:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Advancing_front_surface_reconstruction/include/CGAL/Advancing_front_surface_reconstruction.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Advancing_front_surface_reconstruction/include/CGAL/Advancing_front_surface_reconstruction.h $
 // $Id: Advancing_front_surface_reconstruction.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Advancing_front_surface_reconstruction_vertex_base_3.h 5.1-1/include/CGAL/Advancing_front_surface_reconstruction_vertex_base_3.h
--- 5.0.3-1/include/CGAL/Advancing_front_surface_reconstruction_vertex_base_3.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/Advancing_front_surface_reconstruction_vertex_base_3.h	2020-09-08 12:14:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Advancing_front_surface_reconstruction/include/CGAL/Advancing_front_surface_reconstruction_vertex_base_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Advancing_front_surface_reconstruction/include/CGAL/Advancing_front_surface_reconstruction_vertex_base_3.h $
 // $Id: Advancing_front_surface_reconstruction_vertex_base_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Aff_transformation_2.h 5.1-1/include/CGAL/Aff_transformation_2.h
--- 5.0.3-1/include/CGAL/Aff_transformation_2.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Aff_transformation_2.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Aff_transformation_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Aff_transformation_2.h $
 // $Id: Aff_transformation_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Aff_transformation_3.h 5.1-1/include/CGAL/Aff_transformation_3.h
--- 5.0.3-1/include/CGAL/Aff_transformation_3.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Aff_transformation_3.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Aff_transformation_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Aff_transformation_3.h $
 // $Id: Aff_transformation_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/aff_transformation_tags.h 5.1-1/include/CGAL/aff_transformation_tags.h
--- 5.0.3-1/include/CGAL/aff_transformation_tags.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/aff_transformation_tags.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/aff_transformation_tags.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/aff_transformation_tags.h $
 // $Id: aff_transformation_tags.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/aff_transformation_tags_impl.h 5.1-1/include/CGAL/aff_transformation_tags_impl.h
--- 5.0.3-1/include/CGAL/aff_transformation_tags_impl.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/aff_transformation_tags_impl.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/aff_transformation_tags_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/aff_transformation_tags_impl.h $
 // $Id: aff_transformation_tags_impl.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_extension_traits.h 5.1-1/include/CGAL/Algebraic_extension_traits.h
--- 5.0.3-1/include/CGAL/Algebraic_extension_traits.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_extension_traits.h	2020-09-08 12:14:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_foundations/include/CGAL/Algebraic_extension_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_foundations/include/CGAL/Algebraic_extension_traits.h $
 // $Id: Algebraic_extension_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_converter.h 5.1-1/include/CGAL/Algebraic_kernel_converter.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_converter.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_converter.h	2020-09-08 12:14:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_for_circles/include/CGAL/Algebraic_kernel_converter.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_for_circles/include/CGAL/Algebraic_kernel_converter.h $
 // $Id: Algebraic_kernel_converter.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/Algebraic_curve_kernel_2.h 5.1-1/include/CGAL/Algebraic_kernel_d/Algebraic_curve_kernel_2.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/Algebraic_curve_kernel_2.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/Algebraic_curve_kernel_2.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Algebraic_curve_kernel_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Algebraic_curve_kernel_2.h $
 // $Id: Algebraic_curve_kernel_2.h 67b53fa 2020-04-29T14:06:37+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/algebraic_curve_kernel_2_tools.h 5.1-1/include/CGAL/Algebraic_kernel_d/algebraic_curve_kernel_2_tools.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/algebraic_curve_kernel_2_tools.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/algebraic_curve_kernel_2_tools.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/algebraic_curve_kernel_2_tools.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/algebraic_curve_kernel_2_tools.h $
 // $Id: algebraic_curve_kernel_2_tools.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/Algebraic_real_d_1.h 5.1-1/include/CGAL/Algebraic_kernel_d/Algebraic_real_d_1.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/Algebraic_real_d_1.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/Algebraic_real_d_1.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Algebraic_real_d_1.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Algebraic_real_d_1.h $
 // $Id: Algebraic_real_d_1.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/Algebraic_real_quadratic_refinement_rep_bfi.h 5.1-1/include/CGAL/Algebraic_kernel_d/Algebraic_real_quadratic_refinement_rep_bfi.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/Algebraic_real_quadratic_refinement_rep_bfi.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/Algebraic_real_quadratic_refinement_rep_bfi.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Algebraic_real_quadratic_refinement_rep_bfi.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Algebraic_real_quadratic_refinement_rep_bfi.h $
 // $Id: Algebraic_real_quadratic_refinement_rep_bfi.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/Algebraic_real_rep_bfi.h 5.1-1/include/CGAL/Algebraic_kernel_d/Algebraic_real_rep_bfi.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/Algebraic_real_rep_bfi.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/Algebraic_real_rep_bfi.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Algebraic_real_rep_bfi.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Algebraic_real_rep_bfi.h $
 // $Id: Algebraic_real_rep_bfi.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/Algebraic_real_rep.h 5.1-1/include/CGAL/Algebraic_kernel_d/Algebraic_real_rep.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/Algebraic_real_rep.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/Algebraic_real_rep.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Algebraic_real_rep.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Algebraic_real_rep.h $
 // $Id: Algebraic_real_rep.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/Bitstream_coefficient_kernel_at_alpha.h 5.1-1/include/CGAL/Algebraic_kernel_d/Bitstream_coefficient_kernel_at_alpha.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/Bitstream_coefficient_kernel_at_alpha.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/Bitstream_coefficient_kernel_at_alpha.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Bitstream_coefficient_kernel_at_alpha.h $
-// $Id: Bitstream_coefficient_kernel_at_alpha.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Bitstream_coefficient_kernel_at_alpha.h $
+// $Id: Bitstream_coefficient_kernel_at_alpha.h fdb17cb 2020-03-26T19:26:10+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -39,7 +39,7 @@ public:
 
     typedef typename Algebraic_kernel_d_1::Algebraic_real_1 Algebraic_real_1;
 
-    Bitstream_coefficient_kernel_at_alpha_rep() {}
+    Bitstream_coefficient_kernel_at_alpha_rep() : _m_kernel(nullptr) {}
 
     Bitstream_coefficient_kernel_at_alpha_rep(Algebraic_kernel_d_1* kernel,
                                               Algebraic_real_1 alpha)
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/Bitstream_coefficient_kernel.h 5.1-1/include/CGAL/Algebraic_kernel_d/Bitstream_coefficient_kernel.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/Bitstream_coefficient_kernel.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/Bitstream_coefficient_kernel.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Bitstream_coefficient_kernel.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Bitstream_coefficient_kernel.h $
 // $Id: Bitstream_coefficient_kernel.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/Bitstream_descartes_E08_tree.h 5.1-1/include/CGAL/Algebraic_kernel_d/Bitstream_descartes_E08_tree.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/Bitstream_descartes_E08_tree.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/Bitstream_descartes_E08_tree.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Bitstream_descartes_E08_tree.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Bitstream_descartes_E08_tree.h $
 // $Id: Bitstream_descartes_E08_tree.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/Bitstream_descartes.h 5.1-1/include/CGAL/Algebraic_kernel_d/Bitstream_descartes.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/Bitstream_descartes.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/Bitstream_descartes.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Bitstream_descartes.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Bitstream_descartes.h $
 // $Id: Bitstream_descartes.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/Bitstream_descartes_rndl_tree.h 5.1-1/include/CGAL/Algebraic_kernel_d/Bitstream_descartes_rndl_tree.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/Bitstream_descartes_rndl_tree.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/Bitstream_descartes_rndl_tree.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Bitstream_descartes_rndl_tree.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Bitstream_descartes_rndl_tree.h $
 // $Id: Bitstream_descartes_rndl_tree.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/Bitstream_descartes_rndl_tree_traits.h 5.1-1/include/CGAL/Algebraic_kernel_d/Bitstream_descartes_rndl_tree_traits.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/Bitstream_descartes_rndl_tree_traits.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/Bitstream_descartes_rndl_tree_traits.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Bitstream_descartes_rndl_tree_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Bitstream_descartes_rndl_tree_traits.h $
 // $Id: Bitstream_descartes_rndl_tree_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/bound_between_1.h 5.1-1/include/CGAL/Algebraic_kernel_d/bound_between_1.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/bound_between_1.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/bound_between_1.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/bound_between_1.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/bound_between_1.h $
 // $Id: bound_between_1.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/construct_binary.h 5.1-1/include/CGAL/Algebraic_kernel_d/construct_binary.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/construct_binary.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/construct_binary.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/construct_binary.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/construct_binary.h $
 // $Id: construct_binary.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/Curve_analysis_2.h 5.1-1/include/CGAL/Algebraic_kernel_d/Curve_analysis_2.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/Curve_analysis_2.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/Curve_analysis_2.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Curve_analysis_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Curve_analysis_2.h $
 // $Id: Curve_analysis_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/Curve_pair_analysis_2.h 5.1-1/include/CGAL/Algebraic_kernel_d/Curve_pair_analysis_2.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/Curve_pair_analysis_2.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/Curve_pair_analysis_2.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Curve_pair_analysis_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Curve_pair_analysis_2.h $
 // $Id: Curve_pair_analysis_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/Descartes.h 5.1-1/include/CGAL/Algebraic_kernel_d/Descartes.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/Descartes.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/Descartes.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Descartes.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Descartes.h $
 // $Id: Descartes.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/enums.h 5.1-1/include/CGAL/Algebraic_kernel_d/enums.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/enums.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/enums.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/enums.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/enums.h $
 // $Id: enums.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/Event_line_builder.h 5.1-1/include/CGAL/Algebraic_kernel_d/Event_line_builder.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/Event_line_builder.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/Event_line_builder.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Event_line_builder.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Event_line_builder.h $
 // $Id: Event_line_builder.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/exceptions.h 5.1-1/include/CGAL/Algebraic_kernel_d/exceptions.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/exceptions.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/exceptions.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/exceptions.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/exceptions.h $
 // $Id: exceptions.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/flags.h 5.1-1/include/CGAL/Algebraic_kernel_d/flags.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/flags.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/flags.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/flags.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/flags.h $
 // $Id: flags.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/Float_traits.h 5.1-1/include/CGAL/Algebraic_kernel_d/Float_traits.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/Float_traits.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/Float_traits.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Float_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Float_traits.h $
 // $Id: Float_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/Interval_evaluate_1.h 5.1-1/include/CGAL/Algebraic_kernel_d/Interval_evaluate_1.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/Interval_evaluate_1.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/Interval_evaluate_1.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Interval_evaluate_1.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Interval_evaluate_1.h $
 // $Id: Interval_evaluate_1.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/Interval_evaluate_2.h 5.1-1/include/CGAL/Algebraic_kernel_d/Interval_evaluate_2.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/Interval_evaluate_2.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/Interval_evaluate_2.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Interval_evaluate_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Interval_evaluate_2.h $
 // $Id: Interval_evaluate_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/LRU_hashed_map.h 5.1-1/include/CGAL/Algebraic_kernel_d/LRU_hashed_map.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/LRU_hashed_map.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/LRU_hashed_map.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/LRU_hashed_map.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/LRU_hashed_map.h $
 // $Id: LRU_hashed_map.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/macros.h 5.1-1/include/CGAL/Algebraic_kernel_d/macros.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/macros.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/macros.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/macros.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/macros.h $
 // $Id: macros.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/Real_embeddable_extension.h 5.1-1/include/CGAL/Algebraic_kernel_d/Real_embeddable_extension.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/Real_embeddable_extension.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/Real_embeddable_extension.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Real_embeddable_extension.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Real_embeddable_extension.h $
 // $Id: Real_embeddable_extension.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/Real_roots.h 5.1-1/include/CGAL/Algebraic_kernel_d/Real_roots.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/Real_roots.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/Real_roots.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Real_roots.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Real_roots.h $
 // $Id: Real_roots.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/refine_zero_against.h 5.1-1/include/CGAL/Algebraic_kernel_d/refine_zero_against.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/refine_zero_against.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/refine_zero_against.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/refine_zero_against.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/refine_zero_against.h $
 // $Id: refine_zero_against.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/Shear_controller.h 5.1-1/include/CGAL/Algebraic_kernel_d/Shear_controller.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/Shear_controller.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/Shear_controller.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Shear_controller.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Shear_controller.h $
 // $Id: Shear_controller.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/shear.h 5.1-1/include/CGAL/Algebraic_kernel_d/shear.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/shear.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/shear.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/shear.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/shear.h $
 // $Id: shear.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/Shear_transformation.h 5.1-1/include/CGAL/Algebraic_kernel_d/Shear_transformation.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/Shear_transformation.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/Shear_transformation.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Shear_transformation.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Shear_transformation.h $
 // $Id: Shear_transformation.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/Status_line_CA_1.h 5.1-1/include/CGAL/Algebraic_kernel_d/Status_line_CA_1.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/Status_line_CA_1.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/Status_line_CA_1.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Status_line_CA_1.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Status_line_CA_1.h $
 // $Id: Status_line_CA_1.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/Status_line_CPA_1.h 5.1-1/include/CGAL/Algebraic_kernel_d/Status_line_CPA_1.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/Status_line_CPA_1.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/Status_line_CPA_1.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Status_line_CPA_1.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Status_line_CPA_1.h $
 // $Id: Status_line_CPA_1.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/univariate_polynomial_utils.h 5.1-1/include/CGAL/Algebraic_kernel_d/univariate_polynomial_utils.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/univariate_polynomial_utils.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/univariate_polynomial_utils.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/univariate_polynomial_utils.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/univariate_polynomial_utils.h $
 // $Id: univariate_polynomial_utils.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d/Xy_coordinate_2.h 5.1-1/include/CGAL/Algebraic_kernel_d/Xy_coordinate_2.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d/Xy_coordinate_2.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d/Xy_coordinate_2.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Xy_coordinate_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Xy_coordinate_2.h $
 // $Id: Xy_coordinate_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d_1.h 5.1-1/include/CGAL/Algebraic_kernel_d_1.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d_1.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d_1.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d_1.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d_1.h $
 // $Id: Algebraic_kernel_d_1.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_d_2.h 5.1-1/include/CGAL/Algebraic_kernel_d_2.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_d_2.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_d_2.h	2020-09-08 12:14:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d_2.h $
 // $Id: Algebraic_kernel_d_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_for_circles/function_objects_on_roots_and_polynomials_2_2.h 5.1-1/include/CGAL/Algebraic_kernel_for_circles/function_objects_on_roots_and_polynomials_2_2.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_for_circles/function_objects_on_roots_and_polynomials_2_2.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_for_circles/function_objects_on_roots_and_polynomials_2_2.h	2020-09-08 12:14:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_for_circles/include/CGAL/Algebraic_kernel_for_circles/function_objects_on_roots_and_polynomials_2_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_for_circles/include/CGAL/Algebraic_kernel_for_circles/function_objects_on_roots_and_polynomials_2_2.h $
 // $Id: function_objects_on_roots_and_polynomials_2_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_for_circles/internal_functions_comparison_root_for_circles_2_2.h 5.1-1/include/CGAL/Algebraic_kernel_for_circles/internal_functions_comparison_root_for_circles_2_2.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_for_circles/internal_functions_comparison_root_for_circles_2_2.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_for_circles/internal_functions_comparison_root_for_circles_2_2.h	2020-09-08 12:14:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_for_circles/include/CGAL/Algebraic_kernel_for_circles/internal_functions_comparison_root_for_circles_2_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_for_circles/include/CGAL/Algebraic_kernel_for_circles/internal_functions_comparison_root_for_circles_2_2.h $
 // $Id: internal_functions_comparison_root_for_circles_2_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_for_circles/internal_functions_on_roots_and_polynomial_1_2_and_2_2.h 5.1-1/include/CGAL/Algebraic_kernel_for_circles/internal_functions_on_roots_and_polynomial_1_2_and_2_2.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_for_circles/internal_functions_on_roots_and_polynomial_1_2_and_2_2.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_for_circles/internal_functions_on_roots_and_polynomial_1_2_and_2_2.h	2020-09-08 12:14:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_for_circles/include/CGAL/Algebraic_kernel_for_circles/internal_functions_on_roots_and_polynomial_1_2_and_2_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_for_circles/include/CGAL/Algebraic_kernel_for_circles/internal_functions_on_roots_and_polynomial_1_2_and_2_2.h $
 // $Id: internal_functions_on_roots_and_polynomial_1_2_and_2_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_for_circles/internal_functions_on_roots_and_polynomials_2_2.h 5.1-1/include/CGAL/Algebraic_kernel_for_circles/internal_functions_on_roots_and_polynomials_2_2.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_for_circles/internal_functions_on_roots_and_polynomials_2_2.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_for_circles/internal_functions_on_roots_and_polynomials_2_2.h	2020-09-08 12:14:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_for_circles/include/CGAL/Algebraic_kernel_for_circles/internal_functions_on_roots_and_polynomials_2_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_for_circles/include/CGAL/Algebraic_kernel_for_circles/internal_functions_on_roots_and_polynomials_2_2.h $
 // $Id: internal_functions_on_roots_and_polynomials_2_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_for_circles_2_2.h 5.1-1/include/CGAL/Algebraic_kernel_for_circles_2_2.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_for_circles_2_2.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_for_circles_2_2.h	2020-09-08 12:14:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_for_circles/include/CGAL/Algebraic_kernel_for_circles_2_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_for_circles/include/CGAL/Algebraic_kernel_for_circles_2_2.h $
 // $Id: Algebraic_kernel_for_circles_2_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_for_spheres/function_objects_on_roots_and_polynomials_2_3.h 5.1-1/include/CGAL/Algebraic_kernel_for_spheres/function_objects_on_roots_and_polynomials_2_3.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_for_spheres/function_objects_on_roots_and_polynomials_2_3.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_for_spheres/function_objects_on_roots_and_polynomials_2_3.h	2020-09-08 12:14:32.000000000 +0000
@@ -9,7 +9,7 @@
 // and a STREP (FET Open) Project under Contract No  IST-006413
 // (ACS -- Algorithms for Complex Shapes)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_for_spheres/include/CGAL/Algebraic_kernel_for_spheres/function_objects_on_roots_and_polynomials_2_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_for_spheres/include/CGAL/Algebraic_kernel_for_spheres/function_objects_on_roots_and_polynomials_2_3.h $
 // $Id: function_objects_on_roots_and_polynomials_2_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_for_spheres/internal_functions_comparison_root_for_spheres_2_3.h 5.1-1/include/CGAL/Algebraic_kernel_for_spheres/internal_functions_comparison_root_for_spheres_2_3.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_for_spheres/internal_functions_comparison_root_for_spheres_2_3.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_for_spheres/internal_functions_comparison_root_for_spheres_2_3.h	2020-09-08 12:14:32.000000000 +0000
@@ -9,7 +9,7 @@
 // and a STREP (FET Open) Project under Contract No  IST-006413
 // (ACS -- Algorithms for Complex Shapes)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_for_spheres/include/CGAL/Algebraic_kernel_for_spheres/internal_functions_comparison_root_for_spheres_2_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_for_spheres/include/CGAL/Algebraic_kernel_for_spheres/internal_functions_comparison_root_for_spheres_2_3.h $
 // $Id: internal_functions_comparison_root_for_spheres_2_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_for_spheres/internal_functions_on_roots_and_polynomial_1_3_and_2_3.h 5.1-1/include/CGAL/Algebraic_kernel_for_spheres/internal_functions_on_roots_and_polynomial_1_3_and_2_3.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_for_spheres/internal_functions_on_roots_and_polynomial_1_3_and_2_3.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_for_spheres/internal_functions_on_roots_and_polynomial_1_3_and_2_3.h	2020-09-08 12:14:32.000000000 +0000
@@ -9,7 +9,7 @@
 // and a STREP (FET Open) Project under Contract No  IST-006413
 // (ACS -- Algorithms for Complex Shapes)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_for_spheres/include/CGAL/Algebraic_kernel_for_spheres/internal_functions_on_roots_and_polynomial_1_3_and_2_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_for_spheres/include/CGAL/Algebraic_kernel_for_spheres/internal_functions_on_roots_and_polynomial_1_3_and_2_3.h $
 // $Id: internal_functions_on_roots_and_polynomial_1_3_and_2_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_for_spheres/internal_functions_on_roots_and_polynomials_1_3.h 5.1-1/include/CGAL/Algebraic_kernel_for_spheres/internal_functions_on_roots_and_polynomials_1_3.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_for_spheres/internal_functions_on_roots_and_polynomials_1_3.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_for_spheres/internal_functions_on_roots_and_polynomials_1_3.h	2020-09-08 12:14:32.000000000 +0000
@@ -9,7 +9,7 @@
 // and a STREP (FET Open) Project under Contract No  IST-006413
 // (ACS -- Algorithms for Complex Shapes)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_for_spheres/include/CGAL/Algebraic_kernel_for_spheres/internal_functions_on_roots_and_polynomials_1_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_for_spheres/include/CGAL/Algebraic_kernel_for_spheres/internal_functions_on_roots_and_polynomials_1_3.h $
 // $Id: internal_functions_on_roots_and_polynomials_1_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_for_spheres/internal_functions_on_roots_and_polynomials_2_3.h 5.1-1/include/CGAL/Algebraic_kernel_for_spheres/internal_functions_on_roots_and_polynomials_2_3.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_for_spheres/internal_functions_on_roots_and_polynomials_2_3.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_for_spheres/internal_functions_on_roots_and_polynomials_2_3.h	2020-09-08 12:14:32.000000000 +0000
@@ -9,7 +9,7 @@
 // and a STREP (FET Open) Project under Contract No  IST-006413
 // (ACS -- Algorithms for Complex Shapes)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_for_spheres/include/CGAL/Algebraic_kernel_for_spheres/internal_functions_on_roots_and_polynomials_2_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_for_spheres/include/CGAL/Algebraic_kernel_for_spheres/internal_functions_on_roots_and_polynomials_2_3.h $
 // $Id: internal_functions_on_roots_and_polynomials_2_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_for_spheres_2_3.h 5.1-1/include/CGAL/Algebraic_kernel_for_spheres_2_3.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_for_spheres_2_3.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_for_spheres_2_3.h	2020-09-08 12:14:32.000000000 +0000
@@ -9,7 +9,7 @@
 // and a STREP (FET Open) Project under Contract No  IST-006413
 // (ACS -- Algorithms for Complex Shapes)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_for_spheres/include/CGAL/Algebraic_kernel_for_spheres_2_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_for_spheres/include/CGAL/Algebraic_kernel_for_spheres_2_3.h $
 // $Id: Algebraic_kernel_for_spheres_2_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_rs_gmpq_d_1.h 5.1-1/include/CGAL/Algebraic_kernel_rs_gmpq_d_1.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_rs_gmpq_d_1.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_rs_gmpq_d_1.h	2020-09-08 12:14:31.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_rs_gmpq_d_1.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_rs_gmpq_d_1.h $
 // $Id: Algebraic_kernel_rs_gmpq_d_1.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_kernel_rs_gmpz_d_1.h 5.1-1/include/CGAL/Algebraic_kernel_rs_gmpz_d_1.h
--- 5.0.3-1/include/CGAL/Algebraic_kernel_rs_gmpz_d_1.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_kernel_rs_gmpz_d_1.h	2020-09-08 12:14:31.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_rs_gmpz_d_1.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_rs_gmpz_d_1.h $
 // $Id: Algebraic_kernel_rs_gmpz_d_1.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Algebraic_structure_traits.h 5.1-1/include/CGAL/Algebraic_structure_traits.h
--- 5.0.3-1/include/CGAL/Algebraic_structure_traits.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/Algebraic_structure_traits.h	2020-09-08 12:14:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_foundations/include/CGAL/Algebraic_structure_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_foundations/include/CGAL/Algebraic_structure_traits.h $
 // $Id: Algebraic_structure_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/algorithm.h 5.1-1/include/CGAL/algorithm.h
--- 5.0.3-1/include/CGAL/algorithm.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/algorithm.h	2020-09-08 12:15:32.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/algorithm.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/algorithm.h $
 // $Id: algorithm.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/all_furthest_neighbors_2.h 5.1-1/include/CGAL/all_furthest_neighbors_2.h
--- 5.0.3-1/include/CGAL/all_furthest_neighbors_2.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/all_furthest_neighbors_2.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polytope_distance_d/include/CGAL/all_furthest_neighbors_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polytope_distance_d/include/CGAL/all_furthest_neighbors_2.h $
 // $Id: all_furthest_neighbors_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Alpha_shape_2.h 5.1-1/include/CGAL/Alpha_shape_2.h
--- 5.0.3-1/include/CGAL/Alpha_shape_2.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Alpha_shape_2.h	2020-09-08 12:14:32.000000000 +0000
@@ -5,7 +5,7 @@
 //
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Alpha_shapes_2/include/CGAL/Alpha_shape_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Alpha_shapes_2/include/CGAL/Alpha_shape_2.h $
 // $Id: Alpha_shape_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // Author(s)     : Tran Kai Frank DA
 //                 Andreas Fabri <Andreas.Fabri@geometryfactory.com>
diff -pruN 5.0.3-1/include/CGAL/Alpha_shape_3.h 5.1-1/include/CGAL/Alpha_shape_3.h
--- 5.0.3-1/include/CGAL/Alpha_shape_3.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Alpha_shape_3.h	2020-09-08 12:14:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Alpha_shapes_3/include/CGAL/Alpha_shape_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Alpha_shapes_3/include/CGAL/Alpha_shape_3.h $
 // $Id: Alpha_shape_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Alpha_shape_cell_base_3.h 5.1-1/include/CGAL/Alpha_shape_cell_base_3.h
--- 5.0.3-1/include/CGAL/Alpha_shape_cell_base_3.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Alpha_shape_cell_base_3.h	2020-09-08 12:14:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Alpha_shapes_3/include/CGAL/Alpha_shape_cell_base_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Alpha_shapes_3/include/CGAL/Alpha_shape_cell_base_3.h $
 // $Id: Alpha_shape_cell_base_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Alpha_shape_euclidean_traits_2.h 5.1-1/include/CGAL/Alpha_shape_euclidean_traits_2.h
--- 5.0.3-1/include/CGAL/Alpha_shape_euclidean_traits_2.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Alpha_shape_euclidean_traits_2.h	2020-09-08 12:14:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Alpha_shapes_2/include/CGAL/Alpha_shape_euclidean_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Alpha_shapes_2/include/CGAL/Alpha_shape_euclidean_traits_2.h $
 // $Id: Alpha_shape_euclidean_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Alpha_shape_euclidean_traits_3.h 5.1-1/include/CGAL/Alpha_shape_euclidean_traits_3.h
--- 5.0.3-1/include/CGAL/Alpha_shape_euclidean_traits_3.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Alpha_shape_euclidean_traits_3.h	2020-09-08 12:14:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Alpha_shapes_3/include/CGAL/Alpha_shape_euclidean_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Alpha_shapes_3/include/CGAL/Alpha_shape_euclidean_traits_3.h $
 // $Id: Alpha_shape_euclidean_traits_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Alpha_shape_face_base_2.h 5.1-1/include/CGAL/Alpha_shape_face_base_2.h
--- 5.0.3-1/include/CGAL/Alpha_shape_face_base_2.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Alpha_shape_face_base_2.h	2020-09-08 12:14:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Alpha_shapes_2/include/CGAL/Alpha_shape_face_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Alpha_shapes_2/include/CGAL/Alpha_shape_face_base_2.h $
 // $Id: Alpha_shape_face_base_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Alpha_shape_vertex_base_2.h 5.1-1/include/CGAL/Alpha_shape_vertex_base_2.h
--- 5.0.3-1/include/CGAL/Alpha_shape_vertex_base_2.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Alpha_shape_vertex_base_2.h	2020-09-08 12:14:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Alpha_shapes_2/include/CGAL/Alpha_shape_vertex_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Alpha_shapes_2/include/CGAL/Alpha_shape_vertex_base_2.h $
 // $Id: Alpha_shape_vertex_base_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Alpha_shape_vertex_base_3.h 5.1-1/include/CGAL/Alpha_shape_vertex_base_3.h
--- 5.0.3-1/include/CGAL/Alpha_shape_vertex_base_3.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Alpha_shape_vertex_base_3.h	2020-09-08 12:14:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Alpha_shapes_3/include/CGAL/Alpha_shape_vertex_base_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Alpha_shapes_3/include/CGAL/Alpha_shape_vertex_base_3.h $
 // $Id: Alpha_shape_vertex_base_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_2/Apollonius_graph_2_impl.h 5.1-1/include/CGAL/Apollonius_graph_2/Apollonius_graph_2_impl.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_2/Apollonius_graph_2_impl.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_2/Apollonius_graph_2_impl.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Apollonius_graph_2_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Apollonius_graph_2_impl.h $
 // $Id: Apollonius_graph_2_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_2/Apollonius_graph_hierarchy_2_impl.h 5.1-1/include/CGAL/Apollonius_graph_2/Apollonius_graph_hierarchy_2_impl.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_2/Apollonius_graph_hierarchy_2_impl.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_2/Apollonius_graph_hierarchy_2_impl.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Apollonius_graph_hierarchy_2_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Apollonius_graph_hierarchy_2_impl.h $
 // $Id: Apollonius_graph_hierarchy_2_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_2/basic.h 5.1-1/include/CGAL/Apollonius_graph_2/basic.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_2/basic.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_2/basic.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/basic.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/basic.h $
 // $Id: basic.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_2/Bounded_side_of_ccw_circle_C2.h 5.1-1/include/CGAL/Apollonius_graph_2/Bounded_side_of_ccw_circle_C2.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_2/Bounded_side_of_ccw_circle_C2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_2/Bounded_side_of_ccw_circle_C2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Bounded_side_of_ccw_circle_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Bounded_side_of_ccw_circle_C2.h $
 // $Id: Bounded_side_of_ccw_circle_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_2/comparator_profiler.h 5.1-1/include/CGAL/Apollonius_graph_2/comparator_profiler.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_2/comparator_profiler.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_2/comparator_profiler.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/comparator_profiler.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/comparator_profiler.h $
 // $Id: comparator_profiler.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_2/compare_quadratic.h 5.1-1/include/CGAL/Apollonius_graph_2/compare_quadratic.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_2/compare_quadratic.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_2/compare_quadratic.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/compare_quadratic.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/compare_quadratic.h $
 // $Id: compare_quadratic.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_2/Compare_weight_2.h 5.1-1/include/CGAL/Apollonius_graph_2/Compare_weight_2.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_2/Compare_weight_2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_2/Compare_weight_2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Compare_weight_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Compare_weight_2.h $
 // $Id: Compare_weight_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_2/Compare_x_2.h 5.1-1/include/CGAL/Apollonius_graph_2/Compare_x_2.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_2/Compare_x_2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_2/Compare_x_2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Compare_x_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Compare_x_2.h $
 // $Id: Compare_x_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_2/Compare_y_2.h 5.1-1/include/CGAL/Apollonius_graph_2/Compare_y_2.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_2/Compare_y_2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_2/Compare_y_2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Compare_y_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Compare_y_2.h $
 // $Id: Compare_y_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_2/Constructions_C2.h 5.1-1/include/CGAL/Apollonius_graph_2/Constructions_C2.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_2/Constructions_C2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_2/Constructions_C2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Constructions_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Constructions_C2.h $
 // $Id: Constructions_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_2/Constructions_ftC2.h 5.1-1/include/CGAL/Apollonius_graph_2/Constructions_ftC2.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_2/Constructions_ftC2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_2/Constructions_ftC2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Constructions_ftC2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Constructions_ftC2.h $
 // $Id: Constructions_ftC2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_2/Constructions_rtH2.h 5.1-1/include/CGAL/Apollonius_graph_2/Constructions_rtH2.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_2/Constructions_rtH2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_2/Constructions_rtH2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Constructions_rtH2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Constructions_rtH2.h $
 // $Id: Constructions_rtH2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_2/Finite_edge_test8_C2.h 5.1-1/include/CGAL/Apollonius_graph_2/Finite_edge_test8_C2.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_2/Finite_edge_test8_C2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_2/Finite_edge_test8_C2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Finite_edge_test8_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Finite_edge_test8_C2.h $
 // $Id: Finite_edge_test8_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_2/Finite_edge_test_C2.h 5.1-1/include/CGAL/Apollonius_graph_2/Finite_edge_test_C2.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_2/Finite_edge_test_C2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_2/Finite_edge_test_C2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Finite_edge_test_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Finite_edge_test_C2.h $
 // $Id: Finite_edge_test_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_2/Incircle8_C2.h 5.1-1/include/CGAL/Apollonius_graph_2/Incircle8_C2.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_2/Incircle8_C2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_2/Incircle8_C2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Incircle8_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Incircle8_C2.h $
 // $Id: Incircle8_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_2/Incircle_C2.h 5.1-1/include/CGAL/Apollonius_graph_2/Incircle_C2.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_2/Incircle_C2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_2/Incircle_C2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Incircle_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Incircle_C2.h $
 // $Id: Incircle_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_2/Infinite_edge_test_C2.h 5.1-1/include/CGAL/Apollonius_graph_2/Infinite_edge_test_C2.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_2/Infinite_edge_test_C2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_2/Infinite_edge_test_C2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Infinite_edge_test_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Infinite_edge_test_C2.h $
 // $Id: Infinite_edge_test_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_2/Is_degenerate_edge_C2.h 5.1-1/include/CGAL/Apollonius_graph_2/Is_degenerate_edge_C2.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_2/Is_degenerate_edge_C2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_2/Is_degenerate_edge_C2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Is_degenerate_edge_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Is_degenerate_edge_C2.h $
 // $Id: Is_degenerate_edge_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_2/Is_hidden_C2.h 5.1-1/include/CGAL/Apollonius_graph_2/Is_hidden_C2.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_2/Is_hidden_C2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_2/Is_hidden_C2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Is_hidden_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Is_hidden_C2.h $
 // $Id: Is_hidden_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_2/Kernel_wrapper_2.h 5.1-1/include/CGAL/Apollonius_graph_2/Kernel_wrapper_2.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_2/Kernel_wrapper_2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_2/Kernel_wrapper_2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Kernel_wrapper_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Kernel_wrapper_2.h $
 // $Id: Kernel_wrapper_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_2/Orientation_2.h 5.1-1/include/CGAL/Apollonius_graph_2/Orientation_2.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_2/Orientation_2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_2/Orientation_2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Orientation_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Orientation_2.h $
 // $Id: Orientation_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_2/Orientation8_C2.h 5.1-1/include/CGAL/Apollonius_graph_2/Orientation8_C2.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_2/Orientation8_C2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_2/Orientation8_C2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Orientation8_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Orientation8_C2.h $
 // $Id: Orientation8_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_2/Oriented_side_of_bisector_C2.h 5.1-1/include/CGAL/Apollonius_graph_2/Oriented_side_of_bisector_C2.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_2/Oriented_side_of_bisector_C2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_2/Oriented_side_of_bisector_C2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Oriented_side_of_bisector_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Oriented_side_of_bisector_C2.h $
 // $Id: Oriented_side_of_bisector_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_2/Predicate_constructions_C2.h 5.1-1/include/CGAL/Apollonius_graph_2/Predicate_constructions_C2.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_2/Predicate_constructions_C2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_2/Predicate_constructions_C2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Predicate_constructions_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Predicate_constructions_C2.h $
 // $Id: Predicate_constructions_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_2/predicate_profiler.h 5.1-1/include/CGAL/Apollonius_graph_2/predicate_profiler.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_2/predicate_profiler.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_2/predicate_profiler.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/predicate_profiler.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/predicate_profiler.h $
 // $Id: predicate_profiler.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_2/Predicates_C2.h 5.1-1/include/CGAL/Apollonius_graph_2/Predicates_C2.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_2/Predicates_C2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_2/Predicates_C2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Predicates_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Predicates_C2.h $
 // $Id: Predicates_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_2/Traits_wrapper_2.h 5.1-1/include/CGAL/Apollonius_graph_2/Traits_wrapper_2.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_2/Traits_wrapper_2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_2/Traits_wrapper_2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Traits_wrapper_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/Traits_wrapper_2.h $
 // $Id: Traits_wrapper_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_2/uncertain/uncertain_functions_on_signs.h 5.1-1/include/CGAL/Apollonius_graph_2/uncertain/uncertain_functions_on_signs.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_2/uncertain/uncertain_functions_on_signs.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_2/uncertain/uncertain_functions_on_signs.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/uncertain/uncertain_functions_on_signs.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/uncertain/uncertain_functions_on_signs.h $
 // $Id: uncertain_functions_on_signs.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_2/uncertain/Uncertain_is_hidden_C2.h 5.1-1/include/CGAL/Apollonius_graph_2/uncertain/Uncertain_is_hidden_C2.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_2/uncertain/Uncertain_is_hidden_C2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_2/uncertain/Uncertain_is_hidden_C2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/uncertain/Uncertain_is_hidden_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/uncertain/Uncertain_is_hidden_C2.h $
 // $Id: Uncertain_is_hidden_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_2/uncertain/Uncertain_oriented_side_of_bisector_C2.h 5.1-1/include/CGAL/Apollonius_graph_2/uncertain/Uncertain_oriented_side_of_bisector_C2.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_2/uncertain/Uncertain_oriented_side_of_bisector_C2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_2/uncertain/Uncertain_oriented_side_of_bisector_C2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/uncertain/Uncertain_oriented_side_of_bisector_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/uncertain/Uncertain_oriented_side_of_bisector_C2.h $
 // $Id: Uncertain_oriented_side_of_bisector_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_2/uncertain/Uncertain_vertex_conflict_2.h 5.1-1/include/CGAL/Apollonius_graph_2/uncertain/Uncertain_vertex_conflict_2.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_2/uncertain/Uncertain_vertex_conflict_2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_2/uncertain/Uncertain_vertex_conflict_2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/uncertain/Uncertain_vertex_conflict_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_2/uncertain/Uncertain_vertex_conflict_2.h $
 // $Id: Uncertain_vertex_conflict_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_2.h 5.1-1/include/CGAL/Apollonius_graph_2.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_2.h $
 // $Id: Apollonius_graph_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_adaptation_policies_2.h 5.1-1/include/CGAL/Apollonius_graph_adaptation_policies_2.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_adaptation_policies_2.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_adaptation_policies_2.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Apollonius_graph_adaptation_policies_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Apollonius_graph_adaptation_policies_2.h $
 // $Id: Apollonius_graph_adaptation_policies_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_adaptation_traits_2.h 5.1-1/include/CGAL/Apollonius_graph_adaptation_traits_2.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_adaptation_traits_2.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_adaptation_traits_2.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Apollonius_graph_adaptation_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Apollonius_graph_adaptation_traits_2.h $
 // $Id: Apollonius_graph_adaptation_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_data_structure_2.h 5.1-1/include/CGAL/Apollonius_graph_data_structure_2.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_data_structure_2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_data_structure_2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_data_structure_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_data_structure_2.h $
 // $Id: Apollonius_graph_data_structure_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_filtered_traits_2.h 5.1-1/include/CGAL/Apollonius_graph_filtered_traits_2.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_filtered_traits_2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_filtered_traits_2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_filtered_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_filtered_traits_2.h $
 // $Id: Apollonius_graph_filtered_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_hierarchy_2.h 5.1-1/include/CGAL/Apollonius_graph_hierarchy_2.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_hierarchy_2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_hierarchy_2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_hierarchy_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_hierarchy_2.h $
 // $Id: Apollonius_graph_hierarchy_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_hierarchy_vertex_base_2.h 5.1-1/include/CGAL/Apollonius_graph_hierarchy_vertex_base_2.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_hierarchy_vertex_base_2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_hierarchy_vertex_base_2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_hierarchy_vertex_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_hierarchy_vertex_base_2.h $
 // $Id: Apollonius_graph_hierarchy_vertex_base_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_traits_2.h 5.1-1/include/CGAL/Apollonius_graph_traits_2.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_traits_2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_traits_2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_traits_2.h $
 // $Id: Apollonius_graph_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_graph_vertex_base_2.h 5.1-1/include/CGAL/Apollonius_graph_vertex_base_2.h
--- 5.0.3-1/include/CGAL/Apollonius_graph_vertex_base_2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_graph_vertex_base_2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_graph_vertex_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_graph_vertex_base_2.h $
 // $Id: Apollonius_graph_vertex_base_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Apollonius_site_2.h 5.1-1/include/CGAL/Apollonius_site_2.h
--- 5.0.3-1/include/CGAL/Apollonius_site_2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Apollonius_site_2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Apollonius_site_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Apollonius_site_2.h $
 // $Id: Apollonius_site_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/apply_to_range.h 5.1-1/include/CGAL/apply_to_range.h
--- 5.0.3-1/include/CGAL/apply_to_range.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/apply_to_range.h	2020-09-08 12:15:51.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/apply_to_range.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/apply_to_range.h $
 // $Id: apply_to_range.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/approximated_offset_2.h 5.1-1/include/CGAL/approximated_offset_2.h
--- 5.0.3-1/include/CGAL/approximated_offset_2.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/approximated_offset_2.h	2020-09-08 12:15:11.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Minkowski_sum_2/include/CGAL/approximated_offset_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Minkowski_sum_2/include/CGAL/approximated_offset_2.h $
 // $Id: approximated_offset_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Approximate_min_ellipsoid_d/Approximate_min_ellipsoid_d_configure.h 5.1-1/include/CGAL/Approximate_min_ellipsoid_d/Approximate_min_ellipsoid_d_configure.h
--- 5.0.3-1/include/CGAL/Approximate_min_ellipsoid_d/Approximate_min_ellipsoid_d_configure.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Approximate_min_ellipsoid_d/Approximate_min_ellipsoid_d_configure.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Approximate_min_ellipsoid_d/Approximate_min_ellipsoid_d_configure.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Approximate_min_ellipsoid_d/Approximate_min_ellipsoid_d_configure.h $
 // $Id: Approximate_min_ellipsoid_d_configure.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Approximate_min_ellipsoid_d/Approximate_min_ellipsoid_d_debug.h 5.1-1/include/CGAL/Approximate_min_ellipsoid_d/Approximate_min_ellipsoid_d_debug.h
--- 5.0.3-1/include/CGAL/Approximate_min_ellipsoid_d/Approximate_min_ellipsoid_d_debug.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Approximate_min_ellipsoid_d/Approximate_min_ellipsoid_d_debug.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Approximate_min_ellipsoid_d/Approximate_min_ellipsoid_d_debug.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Approximate_min_ellipsoid_d/Approximate_min_ellipsoid_d_debug.h $
 // $Id: Approximate_min_ellipsoid_d_debug.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Approximate_min_ellipsoid_d/Approximate_min_ellipsoid_d_impl.h 5.1-1/include/CGAL/Approximate_min_ellipsoid_d/Approximate_min_ellipsoid_d_impl.h
--- 5.0.3-1/include/CGAL/Approximate_min_ellipsoid_d/Approximate_min_ellipsoid_d_impl.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Approximate_min_ellipsoid_d/Approximate_min_ellipsoid_d_impl.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Approximate_min_ellipsoid_d/Approximate_min_ellipsoid_d_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Approximate_min_ellipsoid_d/Approximate_min_ellipsoid_d_impl.h $
 // $Id: Approximate_min_ellipsoid_d_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Approximate_min_ellipsoid_d/Khachiyan_approximation.h 5.1-1/include/CGAL/Approximate_min_ellipsoid_d/Khachiyan_approximation.h
--- 5.0.3-1/include/CGAL/Approximate_min_ellipsoid_d/Khachiyan_approximation.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Approximate_min_ellipsoid_d/Khachiyan_approximation.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Approximate_min_ellipsoid_d/Khachiyan_approximation.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Approximate_min_ellipsoid_d/Khachiyan_approximation.h $
 // $Id: Khachiyan_approximation.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Approximate_min_ellipsoid_d/Khachiyan_approximation_impl.h 5.1-1/include/CGAL/Approximate_min_ellipsoid_d/Khachiyan_approximation_impl.h
--- 5.0.3-1/include/CGAL/Approximate_min_ellipsoid_d/Khachiyan_approximation_impl.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Approximate_min_ellipsoid_d/Khachiyan_approximation_impl.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Approximate_min_ellipsoid_d/Khachiyan_approximation_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Approximate_min_ellipsoid_d/Khachiyan_approximation_impl.h $
 // $Id: Khachiyan_approximation_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Approximate_min_ellipsoid_d.h 5.1-1/include/CGAL/Approximate_min_ellipsoid_d.h
--- 5.0.3-1/include/CGAL/Approximate_min_ellipsoid_d.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Approximate_min_ellipsoid_d.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Approximate_min_ellipsoid_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Approximate_min_ellipsoid_d.h $
 // $Id: Approximate_min_ellipsoid_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Approximate_min_ellipsoid_d_traits_2.h 5.1-1/include/CGAL/Approximate_min_ellipsoid_d_traits_2.h
--- 5.0.3-1/include/CGAL/Approximate_min_ellipsoid_d_traits_2.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Approximate_min_ellipsoid_d_traits_2.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Approximate_min_ellipsoid_d_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Approximate_min_ellipsoid_d_traits_2.h $
 // $Id: Approximate_min_ellipsoid_d_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Approximate_min_ellipsoid_d_traits_3.h 5.1-1/include/CGAL/Approximate_min_ellipsoid_d_traits_3.h
--- 5.0.3-1/include/CGAL/Approximate_min_ellipsoid_d_traits_3.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Approximate_min_ellipsoid_d_traits_3.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Approximate_min_ellipsoid_d_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Approximate_min_ellipsoid_d_traits_3.h $
 // $Id: Approximate_min_ellipsoid_d_traits_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Approximate_min_ellipsoid_d_traits_d.h 5.1-1/include/CGAL/Approximate_min_ellipsoid_d_traits_d.h
--- 5.0.3-1/include/CGAL/Approximate_min_ellipsoid_d_traits_d.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Approximate_min_ellipsoid_d_traits_d.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Approximate_min_ellipsoid_d_traits_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Approximate_min_ellipsoid_d_traits_d.h $
 // $Id: Approximate_min_ellipsoid_d_traits_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/argument_swaps.h 5.1-1/include/CGAL/argument_swaps.h
--- 5.0.3-1/include/CGAL/argument_swaps.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/argument_swaps.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/argument_swaps.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/argument_swaps.h $
 // $Id: argument_swaps.h 822bc55 2020-03-27T08:28:48+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arithmetic_kernel/Arithmetic_kernel_base.h 5.1-1/include/CGAL/Arithmetic_kernel/Arithmetic_kernel_base.h
--- 5.0.3-1/include/CGAL/Arithmetic_kernel/Arithmetic_kernel_base.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Arithmetic_kernel/Arithmetic_kernel_base.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arithmetic_kernel/include/CGAL/Arithmetic_kernel/Arithmetic_kernel_base.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arithmetic_kernel/include/CGAL/Arithmetic_kernel/Arithmetic_kernel_base.h $
 // $Id: Arithmetic_kernel_base.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arithmetic_kernel.h 5.1-1/include/CGAL/Arithmetic_kernel.h
--- 5.0.3-1/include/CGAL/Arithmetic_kernel.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Arithmetic_kernel.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arithmetic_kernel/include/CGAL/Arithmetic_kernel.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arithmetic_kernel/include/CGAL/Arithmetic_kernel.h $
 // $Id: Arithmetic_kernel.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_accessor.h 5.1-1/include/CGAL/Arr_accessor.h
--- 5.0.3-1/include/CGAL/Arr_accessor.h	2020-08-01 19:02:11.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_accessor.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_accessor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_accessor.h $
 // $Id: Arr_accessor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_algebraic_segment_traits_2.h 5.1-1/include/CGAL/Arr_algebraic_segment_traits_2.h
--- 5.0.3-1/include/CGAL/Arr_algebraic_segment_traits_2.h	2020-08-01 19:02:11.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_algebraic_segment_traits_2.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_algebraic_segment_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_algebraic_segment_traits_2.h $
 // $Id: Arr_algebraic_segment_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arrangement_2/Arrangement_2_iterators.h 5.1-1/include/CGAL/Arrangement_2/Arrangement_2_iterators.h
--- 5.0.3-1/include/CGAL/Arrangement_2/Arrangement_2_iterators.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arrangement_2/Arrangement_2_iterators.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arrangement_2/Arrangement_2_iterators.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arrangement_2/Arrangement_2_iterators.h $
 // $Id: Arrangement_2_iterators.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arrangement_2/Arrangement_on_surface_2_global.h 5.1-1/include/CGAL/Arrangement_2/Arrangement_on_surface_2_global.h
--- 5.0.3-1/include/CGAL/Arrangement_2/Arrangement_on_surface_2_global.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arrangement_2/Arrangement_on_surface_2_global.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arrangement_2/Arrangement_on_surface_2_global.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arrangement_2/Arrangement_on_surface_2_global.h $
 // $Id: Arrangement_on_surface_2_global.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arrangement_2/Arrangement_on_surface_2_impl.h 5.1-1/include/CGAL/Arrangement_2/Arrangement_on_surface_2_impl.h
--- 5.0.3-1/include/CGAL/Arrangement_2/Arrangement_on_surface_2_impl.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arrangement_2/Arrangement_on_surface_2_impl.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arrangement_2/Arrangement_on_surface_2_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arrangement_2/Arrangement_on_surface_2_impl.h $
 // $Id: Arrangement_on_surface_2_impl.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arrangement_2/arrangement_type_traits.h 5.1-1/include/CGAL/Arrangement_2/arrangement_type_traits.h
--- 5.0.3-1/include/CGAL/Arrangement_2/arrangement_type_traits.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arrangement_2/arrangement_type_traits.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arrangement_2/arrangement_type_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arrangement_2/arrangement_type_traits.h $
 // $Id: arrangement_type_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arrangement_2/Arrangement_zone_2_impl.h 5.1-1/include/CGAL/Arrangement_2/Arrangement_zone_2_impl.h
--- 5.0.3-1/include/CGAL/Arrangement_2/Arrangement_zone_2_impl.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arrangement_2/Arrangement_zone_2_impl.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arrangement_2/Arrangement_zone_2_impl.h $
-// $Id: Arrangement_zone_2_impl.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arrangement_2/Arrangement_zone_2_impl.h $
+// $Id: Arrangement_zone_2_impl.h 58276ed 2020-03-31T18:34:28+03:00 Efi Fogel
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -132,8 +132,8 @@ void Arrangement_zone_2<Arrangement, Zon
         // In this case m_cv overlaps the curve associated with m_intersect_he.
         // Compute the overlapping subcurve.
         bool dummy;
-        m_obj = _compute_next_intersection(m_intersect_he, false, dummy);
-        m_overlap_cv = object_cast<X_monotone_curve_2>(m_obj);
+        auto obj = _compute_next_intersection(m_intersect_he, false, dummy);
+        m_overlap_cv = boost::get<X_monotone_curve_2>(*obj);
 
         // Remove the overlap from the map.
         _remove_next_intersection(m_intersect_he);
@@ -148,8 +148,8 @@ void Arrangement_zone_2<Arrangement, Zon
       m_intersect_he = m_arr.non_const_handle(*hh);
 
       bool dummy;
-      m_obj = _compute_next_intersection(m_intersect_he, false, dummy);
-      m_overlap_cv = object_cast<X_monotone_curve_2>(m_obj);
+      auto obj = _compute_next_intersection(m_intersect_he, false, dummy);
+      m_overlap_cv = boost::get<X_monotone_curve_2>(*obj);
 
       // Remove the overlap from the map.
       _remove_next_intersection(m_intersect_he);
@@ -207,8 +207,8 @@ void Arrangement_zone_2<Arrangement, Zon
         // In this case m_cv overlaps the curve associated with m_intersect_he.
         // Compute the overlapping subcurve to the right of curr_v.
         bool dummy;
-        m_obj = _compute_next_intersection(m_intersect_he, false, dummy);
-        m_overlap_cv = object_cast<X_monotone_curve_2>(m_obj);
+        auto obj = _compute_next_intersection(m_intersect_he, false, dummy);
+        m_overlap_cv = boost::get<X_monotone_curve_2>(*obj);
 
         // Remove the overlap from the map.
         _remove_next_intersection(m_intersect_he);
@@ -800,7 +800,7 @@ _direct_intersecting_edge_to_left(const
 // Get the next intersection of cv with the given halfedge.
 //
 template <typename Arrangement, typename ZoneVisitor>
-CGAL::Object
+typename Arrangement_zone_2<Arrangement, ZoneVisitor>::Optional_intersection
 Arrangement_zone_2<Arrangement, ZoneVisitor>::
 _compute_next_intersection(Halfedge_handle he,
                            bool skip_first_point,
@@ -811,7 +811,7 @@ _compute_next_intersection(Halfedge_hand
 
   // Try to locate the intersections with this curve in the intersections map.
   Intersect_map_iterator iter = m_inter_map.find(p_curve);
-  const Intersect_point_2* ip;
+  const Intersection_point* ip;
   const X_monotone_curve_2* icv;
   bool valid_intersection;
 
@@ -821,13 +821,13 @@ _compute_next_intersection(Halfedge_hand
     // Retrieve the intersections list from the map.
     Intersect_list& inter_list = iter->second;
 
-    if (inter_list.empty()) return CGAL::Object();
+    if (inter_list.empty()) return Optional_intersection();
 
     // Locate the first intersection that lies to the right of m_left_pt
     // (if the left point exists).
     while (! inter_list.empty()) {
       // Compare that current object with m_left_pt (if exists).
-      ip = object_cast<Intersect_point_2>(&(inter_list.front()));
+      ip = boost::get<Intersection_point>(&(inter_list.front()));
 
       if (m_left_on_boundary) {
         // The left end lie on the left boundary, so all intersections are
@@ -851,7 +851,7 @@ _compute_next_intersection(Halfedge_hand
       }
       else {
         // We have an overlapping subcurve.
-        icv = object_cast<X_monotone_curve_2>(&(inter_list.front()));
+        icv = boost::get<X_monotone_curve_2>(&(inter_list.front()));
         CGAL_assertion(icv != nullptr);
 
         if (m_geom_traits->is_closed_2_object()(*icv, ARR_MIN_END)) {
@@ -869,14 +869,14 @@ _compute_next_intersection(Halfedge_hand
       }
 
       // Found an intersection to m_left_pt's right.
-      if (valid_intersection) return (inter_list.front());
+      if (valid_intersection) return Optional_intersection(inter_list.front());
 
       // Discard the current intersection, which lies to m_left_pt's left.
       inter_list.pop_front();
     }
 
     // If we reached here, the list of intersections is empty:
-    return CGAL::Object();
+    return Optional_intersection();
   }
 
   // The intersections with the curve have not been computed yet, so we
@@ -894,7 +894,7 @@ _compute_next_intersection(Halfedge_hand
   // Discard all intersection lying to the left of m_left_pt (if exists).
   while (! inter_list.empty()) {
     // Compare that current object with m_left_pt (if exists).
-    ip = object_cast<Intersect_point_2>(&(inter_list.front()));
+    ip = boost::get<Intersection_point>(&(inter_list.front()));
 
     if (ip != nullptr) {
       // We have a simple intersection point - if we don't have to skip it,
@@ -920,7 +920,7 @@ _compute_next_intersection(Halfedge_hand
     }
     else {
       // We have an overlapping subcurve.
-      icv = object_cast<X_monotone_curve_2>(&(inter_list.front()));
+      icv = boost::get<X_monotone_curve_2>(&(inter_list.front()));
       CGAL_assertion(icv != nullptr);
 
       if (m_geom_traits->is_closed_2_object()(*icv, ARR_MIN_END)) {
@@ -947,8 +947,8 @@ _compute_next_intersection(Halfedge_hand
   m_inter_map[p_curve] = inter_list;
 
   // Return the first intersection object computed (may be empty).
-  if (inter_list.empty()) return CGAL::Object();
-  else return (inter_list.front());
+  if (inter_list.empty()) return Optional_intersection();
+  else return Optional_intersection(inter_list.front());
 }
 
 //-----------------------------------------------------------------------------
@@ -1105,14 +1105,14 @@ _leftmost_intersection(Ccb_halfedge_circ
 
   // Compute the next intersection of m_cv and the current halfedge.
   bool intersection_on_right_boundary;
-  CGAL::Object iobj =
+  Optional_intersection iobj =
     _compute_next_intersection(he_curr, left_equals_curr_endpoint,
                                intersection_on_right_boundary);
 
-  if (! iobj.is_empty()) {
+  if (iobj) {
     // We have found an intersection (either a simple point or an
     // overlapping x-monotone curve).
-    const Intersect_point_2* int_p = object_cast<Intersect_point_2>(&iobj);
+    const Intersection_point* int_p = boost::get<Intersection_point>(&*iobj);
     if (int_p != nullptr) {
       Point_2 ip = int_p->first;
 
@@ -1134,7 +1134,7 @@ _leftmost_intersection(Ccb_halfedge_circ
     else {
       // We have located an overlapping curve. Assign ip as its left
       // endpoint.
-      const X_monotone_curve_2* icv = object_cast<X_monotone_curve_2>(&iobj);
+      const X_monotone_curve_2* icv = boost::get<X_monotone_curve_2>(&*iobj);
       CGAL_assertion(icv != nullptr);
       Point_2 ip = min_vertex(*icv);
 
diff -pruN 5.0.3-1/include/CGAL/Arrangement_2/Arr_compute_zone_visitor.h 5.1-1/include/CGAL/Arrangement_2/Arr_compute_zone_visitor.h
--- 5.0.3-1/include/CGAL/Arrangement_2/Arr_compute_zone_visitor.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arrangement_2/Arr_compute_zone_visitor.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arrangement_2/Arr_compute_zone_visitor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arrangement_2/Arr_compute_zone_visitor.h $
 // $Id: Arr_compute_zone_visitor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arrangement_2/Arr_default_planar_topology.h 5.1-1/include/CGAL/Arrangement_2/Arr_default_planar_topology.h
--- 5.0.3-1/include/CGAL/Arrangement_2/Arr_default_planar_topology.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arrangement_2/Arr_default_planar_topology.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arrangement_2/Arr_default_planar_topology.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arrangement_2/Arr_default_planar_topology.h $
 // $Id: Arr_default_planar_topology.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arrangement_2/Arr_do_intersect_zone_visitor.h 5.1-1/include/CGAL/Arrangement_2/Arr_do_intersect_zone_visitor.h
--- 5.0.3-1/include/CGAL/Arrangement_2/Arr_do_intersect_zone_visitor.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arrangement_2/Arr_do_intersect_zone_visitor.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arrangement_2/Arr_do_intersect_zone_visitor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arrangement_2/Arr_do_intersect_zone_visitor.h $
 // $Id: Arr_do_intersect_zone_visitor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arrangement_2/Arr_on_surface_with_history_2_impl.h 5.1-1/include/CGAL/Arrangement_2/Arr_on_surface_with_history_2_impl.h
--- 5.0.3-1/include/CGAL/Arrangement_2/Arr_on_surface_with_history_2_impl.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arrangement_2/Arr_on_surface_with_history_2_impl.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arrangement_2/Arr_on_surface_with_history_2_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arrangement_2/Arr_on_surface_with_history_2_impl.h $
 // $Id: Arr_on_surface_with_history_2_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arrangement_2/Arr_traits_adaptor_2_dispatching.h 5.1-1/include/CGAL/Arrangement_2/Arr_traits_adaptor_2_dispatching.h
--- 5.0.3-1/include/CGAL/Arrangement_2/Arr_traits_adaptor_2_dispatching.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arrangement_2/Arr_traits_adaptor_2_dispatching.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arrangement_2/Arr_traits_adaptor_2_dispatching.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arrangement_2/Arr_traits_adaptor_2_dispatching.h $
 // $Id: Arr_traits_adaptor_2_dispatching.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arrangement_2/Arr_traits_adaptor_2.h 5.1-1/include/CGAL/Arrangement_2/Arr_traits_adaptor_2.h
--- 5.0.3-1/include/CGAL/Arrangement_2/Arr_traits_adaptor_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arrangement_2/Arr_traits_adaptor_2.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arrangement_2/Arr_traits_adaptor_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arrangement_2/Arr_traits_adaptor_2.h $
 // $Id: Arr_traits_adaptor_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 // $Date$
diff -pruN 5.0.3-1/include/CGAL/Arrangement_2/Arr_with_history_accessor.h 5.1-1/include/CGAL/Arrangement_2/Arr_with_history_accessor.h
--- 5.0.3-1/include/CGAL/Arrangement_2/Arr_with_history_accessor.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arrangement_2/Arr_with_history_accessor.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arrangement_2/Arr_with_history_accessor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arrangement_2/Arr_with_history_accessor.h $
 // $Id: Arr_with_history_accessor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arrangement_2/graph_traits_dual.h 5.1-1/include/CGAL/Arrangement_2/graph_traits_dual.h
--- 5.0.3-1/include/CGAL/Arrangement_2/graph_traits_dual.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arrangement_2/graph_traits_dual.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arrangement_2/graph_traits_dual.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arrangement_2/graph_traits_dual.h $
 // $Id: graph_traits_dual.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arrangement_2.h 5.1-1/include/CGAL/Arrangement_2.h
--- 5.0.3-1/include/CGAL/Arrangement_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arrangement_2.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arrangement_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arrangement_2.h $
 // $Id: Arrangement_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arrangement_on_surface_2.h 5.1-1/include/CGAL/Arrangement_on_surface_2.h
--- 5.0.3-1/include/CGAL/Arrangement_on_surface_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arrangement_on_surface_2.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arrangement_on_surface_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arrangement_on_surface_2.h $
 // $Id: Arrangement_on_surface_2.h bf88dfc 2020-07-03T16:20:22+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arrangement_on_surface_with_history_2.h 5.1-1/include/CGAL/Arrangement_on_surface_with_history_2.h
--- 5.0.3-1/include/CGAL/Arrangement_on_surface_with_history_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arrangement_on_surface_with_history_2.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arrangement_on_surface_with_history_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arrangement_on_surface_with_history_2.h $
 // $Id: Arrangement_on_surface_with_history_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arrangement_with_history_2.h 5.1-1/include/CGAL/Arrangement_with_history_2.h
--- 5.0.3-1/include/CGAL/Arrangement_with_history_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arrangement_with_history_2.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arrangement_with_history_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arrangement_with_history_2.h $
 // $Id: Arrangement_with_history_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arrangement_zone_2.h 5.1-1/include/CGAL/Arrangement_zone_2.h
--- 5.0.3-1/include/CGAL/Arrangement_zone_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arrangement_zone_2.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arrangement_zone_2.h $
-// $Id: Arrangement_zone_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arrangement_zone_2.h $
+// $Id: Arrangement_zone_2.h 58276ed 2020-03-31T18:34:28+03:00 Efi Fogel
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -93,22 +93,25 @@ protected:
                                                    Right_side_category>::result
     Are_all_sides_oblivious_category;
 
-  typedef typename Arrangement_2::Vertex_const_handle    Vertex_const_handle;
-  typedef typename Arrangement_2::Halfedge_const_handle  Halfedge_const_handle;
-  typedef typename Arrangement_2::Face_const_handle      Face_const_handle;
+  typedef typename Arrangement_2::Vertex_const_handle   Vertex_const_handle;
+  typedef typename Arrangement_2::Halfedge_const_handle Halfedge_const_handle;
+  typedef typename Arrangement_2::Face_const_handle     Face_const_handle;
 
   typedef typename Arrangement_2::Ccb_halfedge_circulator
     Ccb_halfedge_circulator;
 
   // Types used for caching intersection points:
-  typedef std::pair<Point_2, Multiplicity>               Intersect_point_2;
-  typedef std::list<CGAL::Object>                        Intersect_list;
+  typedef std::pair<Point_2, Multiplicity>              Intersection_point;
+  typedef boost::variant<Intersection_point, X_monotone_curve_2>
+                                                        Intersection_result;
+  typedef boost::optional<Intersection_result>          Optional_intersection;
+  typedef std::list<Intersection_result>                Intersect_list;
   typedef std::map<const X_monotone_curve_2*, Intersect_list>
-                                                         Intersect_map;
-  typedef typename Intersect_map::iterator               Intersect_map_iterator;
+                                                        Intersect_map;
+  typedef typename Intersect_map::iterator              Intersect_map_iterator;
 
-  typedef std::set<const X_monotone_curve_2*>            Curves_set;
-  typedef typename Curves_set::iterator                  Curves_set_iterator;
+  typedef std::set<const X_monotone_curve_2*>           Curves_set;
+  typedef typename Curves_set::iterator                 Curves_set_iterator;
 
   // Data members:
   Arrangement_2& m_arr;                 // The associated arrangement.
@@ -378,14 +381,15 @@ private:
    *                                            point coincides with the right
    *                                            curve-end, which lies on the
    *                                            surface boundary.
-   * \return An object representing the next intersection: Intersect_point_2
+   * \return An object representing the next intersection: Intersection_point
    *         in case of a simple intersection point, X_monotone_curve_2 in
    *         case of an overlap, and an empty object if there is no
    *         intersection.
    */
-  CGAL::Object _compute_next_intersection(Halfedge_handle he,
-                                          bool skip_first_point,
-                                          bool& intersect_on_right_boundary);
+  Optional_intersection
+  _compute_next_intersection(Halfedge_handle he,
+                             bool skip_first_point,
+                             bool& intersect_on_right_boundary);
 
   /*! Remove the next intersection of m_cv with the given halfedge from the map.
    * \param he A handle to the halfedge.
diff -pruN 5.0.3-1/include/CGAL/arrange_offset_polygons_2.h 5.1-1/include/CGAL/arrange_offset_polygons_2.h
--- 5.0.3-1/include/CGAL/arrange_offset_polygons_2.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/arrange_offset_polygons_2.h	2020-09-08 12:15:41.000000000 +0000
@@ -3,7 +3,7 @@
 // This file is part of CGAL (www.cgal.org).
 //
 
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Straight_skeleton_2/include/CGAL/arrange_offset_polygons_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Straight_skeleton_2/include/CGAL/arrange_offset_polygons_2.h $
 // $Id: arrange_offset_polygons_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/array.h 5.1-1/include/CGAL/array.h
--- 5.0.3-1/include/CGAL/array.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/array.h	2020-09-08 12:15:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/array.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/array.h $
 // $Id: array.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_batched_point_location.h 5.1-1/include/CGAL/Arr_batched_point_location.h
--- 5.0.3-1/include/CGAL/Arr_batched_point_location.h	2020-08-01 19:02:11.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_batched_point_location.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_batched_point_location.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_batched_point_location.h $
 // $Id: Arr_batched_point_location.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_Bezier_curve_traits_2.h 5.1-1/include/CGAL/Arr_Bezier_curve_traits_2.h
--- 5.0.3-1/include/CGAL/Arr_Bezier_curve_traits_2.h	2020-08-01 19:02:11.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_Bezier_curve_traits_2.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_Bezier_curve_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_Bezier_curve_traits_2.h $
 // $Id: Arr_Bezier_curve_traits_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_bounded_planar_topology_traits_2.h 5.1-1/include/CGAL/Arr_bounded_planar_topology_traits_2.h
--- 5.0.3-1/include/CGAL/Arr_bounded_planar_topology_traits_2.h	2020-08-01 19:02:11.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_bounded_planar_topology_traits_2.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_bounded_planar_topology_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_bounded_planar_topology_traits_2.h $
 // $Id: Arr_bounded_planar_topology_traits_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_circle_segment_traits_2.h 5.1-1/include/CGAL/Arr_circle_segment_traits_2.h
--- 5.0.3-1/include/CGAL/Arr_circle_segment_traits_2.h	2020-08-01 19:02:11.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_circle_segment_traits_2.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,13 +3,14 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_circle_segment_traits_2.h $
-// $Id: Arr_circle_segment_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_circle_segment_traits_2.h $
+// $Id: Arr_circle_segment_traits_2.h c1fed0b 2020-03-31T17:30:55+03:00 Efi Fogel
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
-// Author(s)     : Ron Wein          <wein@post.tau.ac.il>
-//                 Baruch Zukerman   <baruchzu@post.tau.ac.il>
-//                 Waqar Khan <wkhan@mpi-inf.mpg.de>
+// Author(s): Ron Wein          <wein@post.tau.ac.il>
+//            Baruch Zukerman   <baruchzu@post.tau.ac.il>
+//            Waqar Khan        <wkhan@mpi-inf.mpg.de>
+//            Efi Fogel         <efifogel@gmail.com>
 
 #ifndef CGAL_ARR_CIRCLE_SEGMENT_TRAITS_2_H
 #define CGAL_ARR_CIRCLE_SEGMENT_TRAITS_2_H
@@ -546,21 +547,15 @@ public:
     return Split_2();
   }
 
-  class Intersect_2
-  {
+  class Intersect_2 {
   private:
-
-    Intersection_map&  _inter_map;       // The map of intersection points.
+    Intersection_map& _inter_map;       // The map of intersection points.
 
   public:
-
     /*! Constructor. */
-    Intersect_2 (Intersection_map& map) :
-      _inter_map (map)
-    {}
+    Intersect_2(Intersection_map& map) : _inter_map(map) {}
 
-    /*!
-     * Find the intersections of the two given curves and insert them to the
+    /*! Find the intersections of the two given curves and insert them to the
      * given output iterator. As two segments may itersect only once, only a
      * single will be contained in the iterator.
      * \param cv1 The first curve.
@@ -568,20 +563,15 @@ public:
      * \param oi The output iterator.
      * \return The past-the-end iterator.
      */
-    template<class OutputIterator>
-    OutputIterator operator() (const X_monotone_curve_2& cv1,
-                               const X_monotone_curve_2& cv2,
-                               OutputIterator oi) const
-    {
-      return (cv1.intersect (cv2, oi, &_inter_map));
-    }
+    template <typename OutputIterator>
+    OutputIterator operator()(const X_monotone_curve_2& cv1,
+                              const X_monotone_curve_2& cv2,
+                              OutputIterator oi) const
+    { return (cv1.intersect(cv2, oi, &_inter_map)); }
   };
 
   /*! Get an Intersect_2 functor object. */
-  Intersect_2 intersect_2_object () const
-  {
-    return (Intersect_2 (inter_map));
-  }
+  Intersect_2 intersect_2_object() const { return (Intersect_2(inter_map)); }
 
   class Are_mergeable_2
   {
@@ -706,14 +696,11 @@ public:
     friend class Arr_circle_segment_traits_2<Kernel, Filter>;
 
   public:
-    /*!\brief
-     * Returns a trimmed version of an arc
-     *
+    /*! Obtain a trimmed version of an arc
      * \param xcv The arc
      * \param src the new first endpoint
      * \param tgt the new second endpoint
      * \return The trimmed arc
-     *
      * \pre src != tgt
      * \pre both points must be interior and must lie on \c cv
      */
diff -pruN 5.0.3-1/include/CGAL/Arr_circular_arc_traits_2.h 5.1-1/include/CGAL/Arr_circular_arc_traits_2.h
--- 5.0.3-1/include/CGAL/Arr_circular_arc_traits_2.h	2020-08-01 19:02:11.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_circular_arc_traits_2.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_circular_arc_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_circular_arc_traits_2.h $
 // $Id: Arr_circular_arc_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_circular_line_arc_traits_2.h 5.1-1/include/CGAL/Arr_circular_line_arc_traits_2.h
--- 5.0.3-1/include/CGAL/Arr_circular_line_arc_traits_2.h	2020-08-01 19:02:11.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_circular_line_arc_traits_2.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_circular_line_arc_traits_2.h $
-// $Id: Arr_circular_line_arc_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_circular_line_arc_traits_2.h $
+// $Id: Arr_circular_line_arc_traits_2.h 6fcbee1 2020-04-21T17:12:21+03:00 Efi Fogel
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Monique Teillaud, Sylvain Pion, Julien Hazebrouck
@@ -22,7 +22,6 @@
 
 #include <CGAL/disable_warnings.h>
 
-
 /*! \file
  * This file was developed at Inria, France, and copied over to the
  * Arrangement_2 package, which it is now part of. It contains a traits
@@ -30,11 +29,12 @@
  * It is based on the circular kernel.
  */
 
-#include <CGAL/basic.h>
+#include <vector>
+
 #include <boost/variant.hpp>
-#include <CGAL/Arr_tags.h>
 
-#include <vector>
+#include <CGAL/basic.h>
+#include <CGAL/Arr_tags.h>
 
 namespace CGAL {
   namespace VariantFunctors{
@@ -47,9 +47,9 @@ namespace CGAL {
     object_to_object_variant(const std::vector<CGAL::Object>& res1,
                              OutputIterator res2)
     {
-      for(std::vector<CGAL::Object>::const_iterator it = res1.begin();
-          it != res1.end(); ++it ){
-        if(const Arc1 *arc = CGAL::object_cast< Arc1 >(&*it)){
+      for (std::vector<CGAL::Object>::const_iterator it = res1.begin();
+          it != res1.end(); ++it ) {
+        if (const Arc1 *arc = CGAL::object_cast< Arc1 >(&*it)){
           boost::variant< Arc1, Arc2 > v =  *arc;
           *res2++ = make_object(v);
         }
@@ -64,29 +64,27 @@ namespace CGAL {
       return res2;
     }
 
-
-    template <class CircularKernel, class Arc1, class Arc2>
-    class In_x_range_2
+    template <class CK, class Arc,
+              class IntersectionPoint, class XMonotoneCurve,
+              class InternaType, class OutputIterator>
+    OutputIterator
+    object_to_object_variant1(const std::vector<InternaType>& res,
+                              OutputIterator oi)
     {
-    public:
-      typedef typename CircularKernel::Circular_arc_point_2
-                                                  Circular_arc_point_2;
-      typedef bool result_type;
-
-      result_type
-        operator()(const boost::variant< Arc1, Arc2 > &a,
-                   const Circular_arc_point_2 &p) const
-      {
-        if ( const Arc1* arc1 = boost::get<Arc1>( &a ) ){
-          return CircularKernel().in_x_range_2_object()(*arc1, p);
-        }
-        else {
-          const Arc2* arc2 = boost::get<Arc2>( &a );
-          return CircularKernel().in_x_range_2_object()(*arc2, p);
+      typedef IntersectionPoint                         Intersection_point;
+      typedef XMonotoneCurve                            X_monotone_curve_2;
+      typedef boost::variant<Intersection_point, X_monotone_curve_2>
+                                                        Intersection_result;
+
+      for (auto it = res.begin(); it != res.end(); ++it) {
+        if (const Arc* arc = boost::get<Arc>(&*it)) {
+          X_monotone_curve_2 cv = *arc;
+          *oi++ = Intersection_result(cv);
         }
+        else *oi++ = Intersection_result(*it);
       }
-    };
-
+      return oi;
+    }
 
     template <class CircularKernel, class Arc1, class Arc2>
     class Compare_y_to_right_2
@@ -277,7 +275,7 @@ namespace CGAL {
           CircularKernel()
             .make_x_monotone_2_object()(*arc1,std::back_inserter(container));
           return object_to_object_variant<CircularKernel, Arc1, Arc2>
-                                         (container, res);
+                                          (container, res);
         }
         else {
           const Arc2* arc2 = boost::get<Arc2>( &A );
@@ -285,65 +283,42 @@ namespace CGAL {
           CircularKernel()
             .make_x_monotone_2_object()(*arc2,std::back_inserter(container));
           return object_to_object_variant<CircularKernel, Arc1, Arc2>
-                                         (container, res);
+                                          (container, res);
         }
       }
     };
 
-
-
     template <class CircularKernel, class Arc1, class Arc2>
     class Intersect_2
     {
     public:
-    typedef typename CircularKernel::Circular_arc_point_2
-                                                Circular_arc_point_2;
+      typedef typename CircularKernel::Circular_arc_point_2
+                                                        Circular_arc_point_2;
 
       template < class OutputIterator >
         OutputIterator
         operator()(const boost::variant< Arc1, Arc2 > &c1,
                    const boost::variant< Arc1, Arc2 > &c2,
-                   OutputIterator res) const
+                   OutputIterator oi) const
       {
         if ( const Arc1* arc1 = boost::get<Arc1>( &c1 ) ){
           if ( const Arc1* arc2 = boost::get<Arc1>( &c2 ) ){
-            std::vector<CGAL::Object> container;
-            CircularKernel()
-              .intersect_2_object()(*arc1,*arc2,std::back_inserter(container));
-            return object_to_object_variant<CircularKernel, Arc1, Arc2>
-                                           (container, res);
-          }
-          else if ( const Arc2* arc2 = boost::get<Arc2>( &c2 ) ){
-            std::vector<CGAL::Object> container;
-            CircularKernel()
-              .intersect_2_object()(*arc1,*arc2,std::back_inserter(container));
-            return object_to_object_variant<CircularKernel, Arc1, Arc2>
-                                           (container, res);
-          }
-        }
-        else {
-          const Arc2* arc1e = boost::get<Arc2>( &c1 );
-          if ( const Arc1* arc2 = boost::get<Arc1>( &c2 ) ){
-            std::vector<CGAL::Object> container;
-            CircularKernel()
-              .intersect_2_object()(*arc1e,*arc2,std::back_inserter(container));
-            return object_to_object_variant<CircularKernel, Arc1, Arc2>
-                                           (container, res);
+            return CircularKernel().intersect_2_object()(*arc1, *arc2, oi);
           }
           const Arc2* arc2 = boost::get<Arc2>( &c2 );
-          std::vector<CGAL::Object> container;
-          CircularKernel()
-            .intersect_2_object()(*arc1e,*arc2,std::back_inserter(container));
-          return object_to_object_variant<CircularKernel, Arc1, Arc2>
-                                         (container, res);
+          return CircularKernel().intersect_2_object()(*arc1, *arc2, oi);
         }
-        CGAL_error();
-        return res;//for no warning
+
+        const Arc2* arc1e = boost::get<Arc2>( &c1 );
+        if ( const Arc1* arc2 = boost::get<Arc1>( &c2 ) ){
+          return CircularKernel().intersect_2_object()(*arc1e, *arc2, oi);
+        }
+        const Arc2* arc2 = boost::get<Arc2>( &c2 );
+        return CircularKernel().intersect_2_object()(*arc1e, *arc2, oi);
       }
 
     };
 
-
     template <class CircularKernel, class Arc1, class Arc2>
     class Split_2
     {
@@ -533,7 +508,7 @@ namespace CGAL {
     typedef unsigned int                           Multiplicity;
 
     typedef CGAL::Tag_false                        Has_left_category;
-    typedef CGAL::Tag_false                            Has_merge_category;
+    typedef CGAL::Tag_false                        Has_merge_category;
     typedef CGAL::Tag_false                        Has_do_intersect_category;
 
     typedef Arr_oblivious_side_tag                 Left_side_category;
diff -pruN 5.0.3-1/include/CGAL/Arr_conic_traits_2.h 5.1-1/include/CGAL/Arr_conic_traits_2.h
--- 5.0.3-1/include/CGAL/Arr_conic_traits_2.h	2020-08-01 19:02:11.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_conic_traits_2.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,13 +3,13 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_conic_traits_2.h $
-// $Id: Arr_conic_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_conic_traits_2.h $
+// $Id: Arr_conic_traits_2.h 523fcfd 2020-03-31T17:32:38+03:00 Efi Fogel
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
-// Author(s)     : Ron Wein   <wein@post.tau.ac.il>
-//                 Waqar Khan <wkhan@mpi-inf.mpg.de>
+// Author(s): Ron Wein   <wein@post.tau.ac.il>
+//            Waqar Khan <wkhan@mpi-inf.mpg.de>
 
 #ifndef CGAL_ARR_CONIC_TRAITS_2_H
 #define CGAL_ARR_CONIC_TRAITS_2_H
@@ -22,6 +22,8 @@
  * The conic traits-class for the arrangement package.
  */
 
+#include <fstream>
+
 #include <CGAL/atomic.h>
 #include <CGAL/tags.h>
 #include <CGAL/Arr_tags.h>
@@ -29,8 +31,6 @@
 #include <CGAL/Arr_geometry_traits/Conic_x_monotone_arc_2.h>
 #include <CGAL/Arr_geometry_traits/Conic_point_2.h>
 
-#include <fstream>
-
 namespace CGAL {
 
 /*!
@@ -88,8 +88,7 @@ private:
 
   // Type definition for the intersection points mapping.
   typedef typename X_monotone_curve_2::Conic_id           Conic_id;
-  typedef typename X_monotone_curve_2::Intersection_point_2
-                                                          Intersection_point_2;
+  typedef typename X_monotone_curve_2::Intersection_point Intersection_point;
   typedef typename X_monotone_curve_2::Intersection_map   Intersection_map;
 
   mutable Intersection_map  inter_map;  // Mapping conic pairs to their
@@ -604,21 +603,15 @@ public:
     return Split_2();
   }
 
-  class Intersect_2
-  {
+  class Intersect_2 {
   private:
-
-    Intersection_map&  _inter_map;       // The map of intersection points.
+    Intersection_map& _inter_map;       // The map of intersection points.
 
   public:
-
     /*! Constructor. */
-    Intersect_2 (Intersection_map& map) :
-      _inter_map (map)
-    {}
+    Intersect_2(Intersection_map& map) : _inter_map(map) {}
 
-    /*!
-     * Find the intersections of the two given curves and insert them to the
+    /*! Find the intersections of the two given curves and insert them to the
      * given output iterator. As two segments may itersect only once, only a
      * single will be contained in the iterator.
      * \param cv1 The first curve.
@@ -626,20 +619,15 @@ public:
      * \param oi The output iterator.
      * \return The past-the-end iterator.
      */
-    template<class OutputIterator>
-    OutputIterator operator() (const X_monotone_curve_2& cv1,
-                               const X_monotone_curve_2& cv2,
-                               OutputIterator oi) const
-    {
-      return (cv1.intersect (cv2, _inter_map, oi));
-    }
+    template <typename OutputIterator>
+    OutputIterator operator()(const X_monotone_curve_2& cv1,
+                              const X_monotone_curve_2& cv2,
+                              OutputIterator oi) const
+    { return (cv1.intersect(cv2, _inter_map, oi)); }
   };
 
   /*! Get an Intersect_2 functor object. */
-  Intersect_2 intersect_2_object () const
-  {
-    return (Intersect_2 (inter_map));
-  }
+  Intersect_2 intersect_2_object () const { return (Intersect_2(inter_map)); }
 
   class Are_mergeable_2
   {
diff -pruN 5.0.3-1/include/CGAL/Arr_consolidated_curve_data_traits_2.h 5.1-1/include/CGAL/Arr_consolidated_curve_data_traits_2.h
--- 5.0.3-1/include/CGAL/Arr_consolidated_curve_data_traits_2.h	2020-08-01 19:02:11.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_consolidated_curve_data_traits_2.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_consolidated_curve_data_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_consolidated_curve_data_traits_2.h $
 // $Id: Arr_consolidated_curve_data_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_counting_traits_2.h 5.1-1/include/CGAL/Arr_counting_traits_2.h
--- 5.0.3-1/include/CGAL/Arr_counting_traits_2.h	2020-08-01 19:02:11.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_counting_traits_2.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_counting_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_counting_traits_2.h $
 // $Id: Arr_counting_traits_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_curve_data_traits_2.h 5.1-1/include/CGAL/Arr_curve_data_traits_2.h
--- 5.0.3-1/include/CGAL/Arr_curve_data_traits_2.h	2020-08-01 19:02:11.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_curve_data_traits_2.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,12 +3,12 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_curve_data_traits_2.h $
-// $Id: Arr_curve_data_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_curve_data_traits_2.h $
+// $Id: Arr_curve_data_traits_2.h 6eefda4 2020-04-06T15:34:28+03:00 Efi Fogel
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
-// Author(s)     : Ron Wein          <wein@post.tau.ac.il>
-//                 Efi Fogel         <efifogel@gmail.com>
+// Author(s): Ron Wein          <wein@post.tau.ac.il>
+//            Efi Fogel         <efifogel@gmail.com>
 
 #ifndef CGAL_ARR_CURVE_DATA_TRAITS_2_H
 #define CGAL_ARR_CURVE_DATA_TRAITS_2_H
@@ -22,6 +22,8 @@
  */
 
 #include <list>
+
+#include <boost/variant.hpp>
 #include <boost/utility/enable_if.hpp>
 #include <boost/mpl/has_xxx.hpp>
 
@@ -193,35 +195,40 @@ public:
      * \param oi The output iterator.
      * \return The past-the-end iterator.
      */
-    template<typename OutputIterator>
+    template <typename OutputIterator>
     OutputIterator operator()(const X_monotone_curve_2& cv1,
                               const X_monotone_curve_2& cv2,
                               OutputIterator oi) const
     {
+      typedef std::pair<Point_2, Multiplicity>          Intersection_point;
+      typedef boost::variant<Intersection_point, X_monotone_curve_2>
+                                                        Intersection_result;
+      typedef boost::variant<Intersection_point, Base_x_monotone_curve_2>
+        Intersection_base_result;
+
       // Use the base functor to obtain all intersection objects.
-      std::list<CGAL::Object> base_objects;
+      std::list<Intersection_base_result> base_objects;
       m_base.intersect_2_object()(cv1, cv2, std::back_inserter(base_objects));
 
       // Stop if the list is empty:
       if (base_objects.empty()) return oi;
 
       // Go over all intersection objects and prepare the output.
-      const Base_x_monotone_curve_2* base_cv;
-      for (typename std::list<CGAL::Object>::const_iterator it =
-             base_objects.begin(); it != base_objects.end(); ++it)
-      {
-        if ((base_cv = object_cast<Base_x_monotone_curve_2>(&(*it))) != nullptr) {
+      for (const auto& item : base_objects) {
+        const Base_x_monotone_curve_2* base_cv =
+          boost::get<Base_x_monotone_curve_2>(&item);
+        if (base_cv != nullptr) {
           // The current intersection object is an overlapping x-monotone
           // curve: Merge the data fields of both intersecting curves and
           // associate the result with the overlapping curve.
-          X_monotone_curve_2 cv(*base_cv, Merge() (cv1.data(), cv2.data()));
-          *oi++ = make_object(cv);
-        }
-        else {
-          // The current intersection object is an intersection point:
-          // Copy it as is.
-          *oi++ = *it;
+          X_monotone_curve_2 cv(*base_cv, Merge()(cv1.data(), cv2.data()));
+          *oi++ = Intersection_result(cv);
+          continue;
         }
+        // The current intersection object is an intersection point:
+        // Copy it as is.
+        const Intersection_point* ip = boost::get<Intersection_point>(&item);
+        *oi++ = Intersection_result(*ip);
       }
 
       return oi;
diff -pruN 5.0.3-1/include/CGAL/Arr_dcel_base.h 5.1-1/include/CGAL/Arr_dcel_base.h
--- 5.0.3-1/include/CGAL/Arr_dcel_base.h	2020-08-01 19:02:11.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_dcel_base.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_dcel_base.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_dcel_base.h $
 // $Id: Arr_dcel_base.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_default_dcel.h 5.1-1/include/CGAL/Arr_default_dcel.h
--- 5.0.3-1/include/CGAL/Arr_default_dcel.h	2020-08-01 19:02:11.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_default_dcel.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_default_dcel.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_default_dcel.h $
 // $Id: Arr_default_dcel.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_default_overlay_traits.h 5.1-1/include/CGAL/Arr_default_overlay_traits.h
--- 5.0.3-1/include/CGAL/Arr_default_overlay_traits.h	2020-08-01 19:02:11.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_default_overlay_traits.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_default_overlay_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_default_overlay_traits.h $
 // $Id: Arr_default_overlay_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_directional_non_caching_segment_basic_traits_2.h 5.1-1/include/CGAL/Arr_directional_non_caching_segment_basic_traits_2.h
--- 5.0.3-1/include/CGAL/Arr_directional_non_caching_segment_basic_traits_2.h	2020-08-01 19:02:11.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_directional_non_caching_segment_basic_traits_2.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_directional_non_caching_segment_basic_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_directional_non_caching_segment_basic_traits_2.h $
 // $Id: Arr_directional_non_caching_segment_basic_traits_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_enums.h 5.1-1/include/CGAL/Arr_enums.h
--- 5.0.3-1/include/CGAL/Arr_enums.h	2020-08-01 19:02:11.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_enums.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_enums.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_enums.h $
 // $Id: Arr_enums.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_extended_dcel.h 5.1-1/include/CGAL/Arr_extended_dcel.h
--- 5.0.3-1/include/CGAL/Arr_extended_dcel.h	2020-08-01 19:02:11.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_extended_dcel.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_extended_dcel.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_extended_dcel.h $
 // $Id: Arr_extended_dcel.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_face_index_map.h 5.1-1/include/CGAL/Arr_face_index_map.h
--- 5.0.3-1/include/CGAL/Arr_face_index_map.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_face_index_map.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_face_index_map.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_face_index_map.h $
 // $Id: Arr_face_index_map.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_face_map.h 5.1-1/include/CGAL/Arr_face_map.h
--- 5.0.3-1/include/CGAL/Arr_face_map.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_face_map.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_face_map.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_face_map.h $
 // $Id: Arr_face_map.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_geodesic_arc_on_sphere_partition_traits_2.h 5.1-1/include/CGAL/Arr_geodesic_arc_on_sphere_partition_traits_2.h
--- 5.0.3-1/include/CGAL/Arr_geodesic_arc_on_sphere_partition_traits_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_geodesic_arc_on_sphere_partition_traits_2.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_geodesic_arc_on_sphere_partition_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_geodesic_arc_on_sphere_partition_traits_2.h $
 // $Id: Arr_geodesic_arc_on_sphere_partition_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
@@ -314,7 +314,7 @@ public:
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_geodesic_arc_on_sphere_partition_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_geodesic_arc_on_sphere_partition_traits_2.h $
 // $Id: Arr_geodesic_arc_on_sphere_partition_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_geodesic_arc_on_sphere_traits_2.h 5.1-1/include/CGAL/Arr_geodesic_arc_on_sphere_traits_2.h
--- 5.0.3-1/include/CGAL/Arr_geodesic_arc_on_sphere_traits_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_geodesic_arc_on_sphere_traits_2.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,11 +3,11 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_geodesic_arc_on_sphere_traits_2.h $
-// $Id: Arr_geodesic_arc_on_sphere_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_geodesic_arc_on_sphere_traits_2.h $
+// $Id: Arr_geodesic_arc_on_sphere_traits_2.h 3d37889 2020-05-19T10:55:30+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
-// Author(s)     : Efi Fogel         <efif@post.tau.ac.il>
+// Author(s) : Efi Fogel         <efif@post.tau.ac.il>
 
 #ifndef CGAL_ARR_GEODESIC_ARC_ON_SPHERE_TRAITS_2_H
 #define CGAL_ARR_GEODESIC_ARC_ON_SPHERE_TRAITS_2_H
@@ -25,6 +25,8 @@
 
 #include <fstream>
 
+#include <boost/variant.hpp>
+
 #include <CGAL/config.h>
 #include <CGAL/tags.h>
 #include <CGAL/tss.h>
@@ -1646,7 +1648,10 @@ public:
                                         Project project,
                                         OutputIterator oi) const
     {
-      typedef std::pair<Point_2, Multiplicity>                  Point_2_pair;
+      typedef std::pair<Point_2, Multiplicity>          Intersection_point;
+      typedef boost::variant<Intersection_point, X_monotone_curve_2>
+                                                        Intersection_result;
+
       const Kernel* kernel = m_traits;
       typename Kernel::Equal_2 equal = kernel->equal_2_object();
 
@@ -1658,7 +1663,7 @@ public:
       if (equal(l1, l2)) {
         const Point_2& trg = (in_between(r1, l2, r2)) ? r1_3 : r2_3;
         X_monotone_curve_2 xc(l1_3, trg, normal, vertical, true);
-        *oi++ = make_object(xc);
+        *oi++ = Intersection_result(xc);
         return oi;
       }
 
@@ -1668,29 +1673,29 @@ public:
       if (l1_eq_start || (!l2_eq_start && in_between(l1, start, l2))) {
         // The following applies only to full circles:
         if (l1_eq_start && equal(r2, start))
-          *oi++ = make_object(Point_2_pair(r2_3, 1));
+          *oi++ = Intersection_result(Intersection_point(r2_3, 1));
         if (in_between(r1, l1, l2)) return oi;      // no intersection
         if (equal(r1, l2)) {
-          *oi++ = make_object(Point_2_pair(r1_3, 1));
+          *oi++ = Intersection_result(Intersection_point(r1_3, 1));
           return oi;
         }
         const Point_2& trg = (in_between(r1, l2, r2)) ? r1_3 : r2_3;
         X_monotone_curve_2 xc(l2_3, trg, normal, vertical, true);
-        *oi++ = make_object(xc);
+        *oi++ = Intersection_result(xc);
         return oi;
       }
       CGAL_assertion(l2_eq_start || in_between(l2, start, l1));
       // The following applies only to full circles:
       if (l2_eq_start && equal(r1, start))
-        *oi++ = make_object(Point_2_pair(r1_3, 1));
+        *oi++ = Intersection_result(Intersection_point(r1_3, 1));
       if (in_between(r2, l2, l1)) return oi;      // no intersection
       if (equal(r2, l1)) {
-        *oi++ = make_object(Point_2_pair(r2_3, 1));
+        *oi++ = Intersection_result(Intersection_point(r2_3, 1));
         return oi;
       }
       const Point_2& trg = (in_between(r1, l2, r2)) ? r1_3 : r2_3;
       X_monotone_curve_2 xc(l1_3, trg, normal, vertical, true);
-      *oi++ = make_object(xc);
+      *oi++ = Intersection_result(xc);
       return oi;
     }
 
@@ -1784,9 +1789,12 @@ public:
       typedef Arr_geodesic_arc_on_sphere_traits_2<Kernel> Traits;
       typedef typename Kernel::Counterclockwise_in_between_2
         Counterclockwise_in_between_2;
-      typedef typename Kernel::Equal_3                          Equal_3;
+      typedef typename Kernel::Equal_3                  Equal_3;
+
+      typedef std::pair<Point_2, Multiplicity>          Intersection_point;
+      typedef boost::variant<Intersection_point, X_monotone_curve_2>
+                                                        Intersection_result;
 
-      typedef std::pair<Point_2, Multiplicity>                  Point_2_pair;
       const Kernel* kernel = m_traits;
 
       Equal_3 equal_3 = kernel->equal_3_object();
@@ -1810,9 +1818,9 @@ public:
               (res && (xc1.is_directed_right() != xc2.is_directed_right())))
           {
             if (xc1.left().is_min_boundary() && xc2.left().is_min_boundary())
-              *oi++ = make_object(Point_2_pair(xc1.left(), 1));
+              *oi++ = Intersection_result(Intersection_point(xc1.left(), 1));
             if (xc1.right().is_max_boundary() && xc2.right().is_max_boundary())
-              *oi++ = make_object(Point_2_pair(xc1.right(), 1));
+              *oi++ = Intersection_result(Intersection_point(xc1.right(), 1));
             return oi;
           }
 
@@ -1820,11 +1828,11 @@ public:
            * the other arc is completely overlapping.
            */
           if (xc1.left().is_min_boundary() && xc1.right().is_max_boundary()) {
-            *oi++ = make_object(xc2);
+            *oi++ = Intersection_result(xc2);
             return oi;
           }
           if (xc2.left().is_min_boundary() && xc2.right().is_max_boundary()) {
-            *oi++ = make_object(xc1);
+            *oi++ = Intersection_result(xc1);
             return oi;
           }
           /*! Find an endpoint that does not coincide with a pole, and project
@@ -1877,14 +1885,14 @@ public:
       // Determine which one of the two directions:
       Point_2 ed(v.direction());
       if (is_in_between(ed, xc1) && is_in_between(ed, xc2)) {
-        *oi++ = make_object(Point_2_pair(ed, 1));
+        *oi++ = Intersection_result(Intersection_point(ed, 1));
         return oi;
       }
 
       Vector_3 vo(kernel->construct_opposite_vector_3_object()(v));
       Point_2 edo(vo.direction());
       if (is_in_between(edo, xc1) && is_in_between(edo, xc2)) {
-        *oi++ = make_object(Point_2_pair(edo, 1));
+        *oi++ = Intersection_result(Intersection_point(edo, 1));
         return oi;
       }
       return oi;
@@ -1927,8 +1935,13 @@ public:
       const Kernel* kernel = m_traits;
       typename Kernel::Equal_3 equal = kernel->equal_3_object();
 
+      // Down cast to pass to kernel member functions
+      const Direction_3& xc1_left = xc1.left();
+      const Direction_3& xc2_left = xc2.left();
+      const Direction_3& xc1_right = xc1.right();
+      const Direction_3& xc2_right = xc2.right();
       if (xc1.is_degenerate() && xc2.is_degenerate())
-        return equal(xc1.left(), xc2.left());
+        return equal(xc1_left, xc2_left);
       if ((xc1.is_full() || xc1.is_meridian()) && xc2.is_degenerate())
         return xc1.has_on(xc2.left());
       if ((xc2.is_full() || xc2.is_meridian()) && xc1.is_degenerate())
@@ -1941,8 +1954,8 @@ public:
       if (!equal(normal1, normal2) && !equal(opposite_normal1, normal2))
         return false;
 
-      bool eq1 = equal(xc1.right(), xc2.left());
-      bool eq2 = equal(xc1.left(), xc2.right());
+      bool eq1 = equal(xc1_right, xc2_left);
+      bool eq2 = equal(xc1_left, xc2_right);
 
 #if defined(CGAL_FULL_X_MONOTONE_GEODESIC_ARC_ON_SPHERE_IS_SUPPORTED)
       if (eq1 && eq2) return true;
@@ -2001,14 +2014,20 @@ public:
       const Kernel* kernel = m_traits;
       typename Kernel::Equal_3 equal = kernel->equal_3_object();
 
+      // Down cast to pass to kernel member functions
+      const Direction_3& xc1_right = xc1.right();
+      const Direction_3& xc2_left = xc2.left();
+
       xc.set_is_degenerate(false);
       xc.set_is_empty(false);
       xc.set_is_vertical(xc1.is_vertical());
 
-      bool eq1 = equal(xc1.right(), xc2.left());
+      bool eq1 = equal(xc1_right, xc2_left);
 
 #if defined(CGAL_FULL_X_MONOTONE_GEODESIC_ARC_ON_SPHERE_IS_SUPPORTED)
-      bool eq2 = equal(xc1.left(), xc2.right());
+      const Direction_3& xc1_left = xc1.left();
+      const Direction_3& xc2_right = xc2.right();
+      bool eq2 = equal(xc1_left, xc2_right);
       if (eq1 && eq2) {
         const Point_2& p =
           xc1.source().is_mid_boundary() ? xc1.source() : xc1.target();
@@ -2017,8 +2036,10 @@ public:
         xc.set_normal(xc1.normal());
         xc.set_is_full(true);
       }
+#else
+      CGAL_assertion_code(const Direction_3& xc1_left = xc1.left();
+                          const Direction_3& xc2_right = xc2.right());
 #endif
-
       if (xc1.is_directed_right() || xc2.is_directed_right()) {
         xc.set_normal(xc1.is_directed_right() ? xc1.normal() : xc2.normal());
         xc.set_is_directed_right(true);
@@ -2026,20 +2047,23 @@ public:
         if (eq1) {
           xc.set_source(xc1.left());
           xc.set_target(xc2.right());
-        } else {
-          CGAL_assertion(equal(xc1.left(), xc2.right()));
+        }
+        else {
+          CGAL_assertion(equal(xc1_left, xc2_right));
           xc.set_source(xc2.left());
           xc.set_target(xc1.right());
         }
-      } else {
+      }
+      else {
         xc.set_normal(xc1.normal());
         xc.set_is_directed_right(false);
 
         if (eq1) {
           xc.set_source(xc2.right());
           xc.set_target(xc1.left());
-        } else {
-          CGAL_assertion(equal(xc1.left(), xc2.right()));
+        }
+        else {
+          CGAL_assertion(equal(xc1_left, xc2_right));
           xc.set_source(xc1.right());
           xc.set_target(xc2.left());
         }
diff -pruN 5.0.3-1/include/CGAL/Arr_geometry_traits/Arr_plane_3.h 5.1-1/include/CGAL/Arr_geometry_traits/Arr_plane_3.h
--- 5.0.3-1/include/CGAL/Arr_geometry_traits/Arr_plane_3.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_geometry_traits/Arr_plane_3.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/Arr_plane_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/Arr_plane_3.h $
 // $Id: Arr_plane_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_geometry_traits/Bezier_bounding_rational_traits.h 5.1-1/include/CGAL/Arr_geometry_traits/Bezier_bounding_rational_traits.h
--- 5.0.3-1/include/CGAL/Arr_geometry_traits/Bezier_bounding_rational_traits.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_geometry_traits/Bezier_bounding_rational_traits.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/Bezier_bounding_rational_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/Bezier_bounding_rational_traits.h $
 // $Id: Bezier_bounding_rational_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_geometry_traits/Bezier_cache.h 5.1-1/include/CGAL/Arr_geometry_traits/Bezier_cache.h
--- 5.0.3-1/include/CGAL/Arr_geometry_traits/Bezier_cache.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_geometry_traits/Bezier_cache.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/Bezier_cache.h $
-// $Id: Bezier_cache.h cba160f 2020-06-30T16:37:01+02:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/Bezier_cache.h $
+// $Id: Bezier_cache.h 6d949cd 2020-08-08T17:29:55+02:00 Ahmed Essam
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -52,11 +52,11 @@ public:
   /// \name Type definitions for the intersection-point mapping.
   //@{
 
-  /*! \struct Intersection_point_2
+  /*! \struct Intersection_point
    * Representation of an intersection point (in both parameter and physical
    * spaces).
    */
-  struct Intersection_point_2
+  struct Intersection_point
   {
     Algebraic          s;      // The parameter for the first curve.
     Algebraic          t;      // The parameter for the second curve.
@@ -64,7 +64,7 @@ public:
     Algebraic          y;      // The y-coordinate.
 
     /*! Constructor. */
-    Intersection_point_2 (const Algebraic& _s, const Algebraic& _t,
+    Intersection_point (const Algebraic& _s, const Algebraic& _t,
                           const Algebraic& _x, const Algebraic& _y) :
       s(_s), t(_t),
       x(_x), y(_y)
@@ -73,7 +73,7 @@ public:
 
   typedef std::pair<Curve_id, Curve_id>              Curve_pair;
   typedef std::pair<Algebraic, Algebraic>            Parameter_pair;
-  typedef std::list<Intersection_point_2>            Intersection_list;
+  typedef std::list<Intersection_point>            Intersection_list;
   typedef
     typename Intersection_list::const_iterator       Intersection_iter;
 
@@ -378,7 +378,7 @@ _Bezier_cache<NtTraits>::get_intersectio
             CGAL::compare (nt_traits.evaluate_at (polyY_1, *t_it),
                            y) == EQUAL)
         {
-          info.first.push_back (Intersection_point_2 (*s_it, *t_it,
+          info.first.push_back (Intersection_point (*s_it, *t_it,
                                                       x / denX, y / denY));
         }
       }
@@ -528,7 +528,7 @@ _Bezier_cache<NtTraits>::get_intersectio
     if(CGAL::sign (s) != NEGATIVE && CGAL::compare (s, one) != LARGER &&
       CGAL::sign (t) != NEGATIVE && CGAL::compare (t, one) != LARGER)
     {
-      info.first.push_back (Intersection_point_2 (s, t,pit1->x, pit1->y));
+      info.first.push_back(Intersection_point(s, t,pit1->x, pit1->y));
     }
   }
 
@@ -653,7 +653,7 @@ void _Bezier_cache<NtTraits>::_self_inte
   //   II: Y(t) - Y(s) / (t - s) = 0
   //
   Integer                 *coeffs;
-  int                      i, k;
+  int                      i;
 
   // Consruct the bivariate polynomial that corresponds to Equation I.
   // Note that we represent a bivariate polynomial as a vector of univariate
@@ -667,12 +667,11 @@ void _Bezier_cache<NtTraits>::_self_inte
   coeffs = new Integer [degX];
 
   for (i = 0; i < degX; i++)
-  {
-    for (k = i + 1; k < degX; k++)
-      coeffs[k - i - 1] = nt_traits.get_coefficient (polyX, k);
+    coeffs[i] = nt_traits.get_coefficient(polyX, i + 1);
 
-    coeffsX_st[i] = nt_traits.construct_polynomial (coeffs, degX - i - 1);
-  }
+  for (i = 0; i < degX; i++)
+    coeffsX_st[degX - i - 1] =
+        nt_traits.construct_polynomial(coeffs + i, degX - i - 1);
 
   delete[] coeffs;
 
@@ -685,12 +684,11 @@ void _Bezier_cache<NtTraits>::_self_inte
   coeffs = new Integer [degY];
 
   for (i = 0; i < degY; i++)
-  {
-    for (k = i + 1; k < degY; k++)
-      coeffs[k - i - 1] = nt_traits.get_coefficient (polyY, k);
+    coeffs[i] = nt_traits.get_coefficient(polyY, i + 1);
 
-    coeffsY_st[i] = nt_traits.construct_polynomial (coeffs, degY - i - 1);
-  }
+  for (i = 0; i < degY; i++)
+    coeffsY_st[degY - i - 1] =
+        nt_traits.construct_polynomial(coeffs + i, degY - i - 1);
 
   delete[] coeffs;
 
diff -pruN 5.0.3-1/include/CGAL/Arr_geometry_traits/Bezier_curve_2.h 5.1-1/include/CGAL/Arr_geometry_traits/Bezier_curve_2.h
--- 5.0.3-1/include/CGAL/Arr_geometry_traits/Bezier_curve_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_geometry_traits/Bezier_curve_2.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/Bezier_curve_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/Bezier_curve_2.h $
 // $Id: Bezier_curve_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_geometry_traits/Bezier_point_2.h 5.1-1/include/CGAL/Arr_geometry_traits/Bezier_point_2.h
--- 5.0.3-1/include/CGAL/Arr_geometry_traits/Bezier_point_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_geometry_traits/Bezier_point_2.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/Bezier_point_2.h $
-// $Id: Bezier_point_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/Bezier_point_2.h $
+// $Id: Bezier_point_2.h ff93e2b 2020-08-11T13:18:30+02:00 Ahmed Essam
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -1641,17 +1641,43 @@ void _Bezier_point_2_rep<RatKer, AlgKer,
   const Algebraic      t_min = nt_traits.convert (orig2.point_bound().t_min);
   const Algebraic      t_max = nt_traits.convert (orig2.point_bound().t_max);
 
+  bool self_intersecting = (org_it1->curve().id() == org_it2->curve().id());
+
   for (intr_it = intr_list.begin(); intr_it != intr_list.end(); ++intr_it)
   {
-    if (CGAL::compare (intr_it->s, s_min) != SMALLER &&
-        CGAL::compare (intr_it->s, s_max) != LARGER &&
-        CGAL::compare (intr_it->t, t_min) != SMALLER &&
-        CGAL::compare (intr_it->t, t_max) != LARGER)
+    auto in_bounding_interval =
+      [](const auto& s_, const auto& s_min_, const auto& s_max_) -> bool {
+      return CGAL::compare(s_, s_min_) != SMALLER &&
+             CGAL::compare(s_, s_max_) != LARGER;
+    };
+
+    bool st_in_st_range = in_bounding_interval(intr_it->s, s_min, s_max) &&
+                          in_bounding_interval(intr_it->t, t_min, t_max);
+    bool ts_in_st_range = false;
+
+    if (st_in_st_range)
     {
       // Update the originators.
-      orig1.set_parameter (intr_it->s);
-      orig2.set_parameter (intr_it->t);
+      orig1.set_parameter(intr_it->s);
+      orig2.set_parameter(intr_it->t);
+    }
+    else if (self_intersecting)
+    {
+      // check whether s is in t range, and t is in s range
+      // s and t can be interchanged in case of self intersections
+      ts_in_st_range = in_bounding_interval(intr_it->t, s_min, s_max) &&
+                       in_bounding_interval(intr_it->s, t_min, t_max);
 
+      if (ts_in_st_range)
+      {
+        // Update the originators.
+        orig1.set_parameter(intr_it->t);
+        orig2.set_parameter(intr_it->s);
+      }
+    }
+
+    if (st_in_st_range || ts_in_st_range)
+    {
       // Set the exact point coordinates.
       p_alg_x = new Algebraic (intr_it->x);
       p_alg_y = new Algebraic (intr_it->y);
diff -pruN 5.0.3-1/include/CGAL/Arr_geometry_traits/Bezier_x_monotone_2.h 5.1-1/include/CGAL/Arr_geometry_traits/Bezier_x_monotone_2.h
--- 5.0.3-1/include/CGAL/Arr_geometry_traits/Bezier_x_monotone_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_geometry_traits/Bezier_x_monotone_2.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,12 +3,12 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/Bezier_x_monotone_2.h $
-// $Id: Bezier_x_monotone_2.h 157bdbd 2020-04-22T13:26:52+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/Bezier_x_monotone_2.h $
+// $Id: Bezier_x_monotone_2.h 3dc2804 2020-08-11T13:23:08+02:00 Ahmed Essam
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
-// Author(s)     : Ron Wein     <wein@post.tau.ac.il>
-//                 Iddo Hanniel <iddoh@cs.technion.ac.il>
+// Author(s): Ron Wein     <wein@post.tau.ac.il>
+//            Iddo Hanniel <iddoh@cs.technion.ac.il>
 
 #ifndef CGAL_BEZIER_X_MONOTONE_2_H
 #define CGAL_BEZIER_X_MONOTONE_2_H
@@ -85,27 +85,24 @@ private:
   typedef typename Bezier_cache::Intersection_iter        Intersect_iter;
 
   // Representation of an intersection point with its multiplicity:
-  typedef std::pair<Point_2,Multiplicity>                Intersection_point_2;
+  typedef std::pair<Point_2, Multiplicity>                Intersection_point;
 
   /*! \class Less_intersection_point
    * Comparison functor for intersection points.
    */
-  class Less_intersection_point
-  {
+  class Less_intersection_point {
   private:
-    Bezier_cache    *p_cache;
+    Bezier_cache* p_cache;
 
   public:
 
-    Less_intersection_point (Bezier_cache& cache) :
-      p_cache (&cache)
-    {}
+    Less_intersection_point(Bezier_cache& cache) : p_cache(&cache) {}
 
-    bool operator() (const Intersection_point_2& ip1,
-                     const Intersection_point_2& ip2) const
+    bool operator()(const Intersection_point& ip1,
+                    const Intersection_point& ip2) const
     {
       // Use an xy-lexicographic comparison.
-      return (ip1.first.compare_xy (ip2.first, *p_cache) == SMALLER);
+      return (ip1.first.compare_xy(ip2.first, *p_cache) == SMALLER);
     }
   };
 
@@ -117,7 +114,7 @@ private:
    */
   struct Less_curve_pair
   {
-    bool operator() (const Curve_pair& cp1, const Curve_pair& cp2) const
+    bool operator()(const Curve_pair& cp1, const Curve_pair& cp2) const
     {
       // Compare the pairs of IDs lexicographically.
       return (cp1.first < cp2.first ||
@@ -128,34 +125,32 @@ private:
   /*! \struct Subcurve
    * For the usage of the _exact_vertical_position() function.
    */
-  struct Subcurve
-  {
-    std::list<Rat_point_2>    control_points;
-    Rational                  t_min;
-    Rational                  t_max;
+  struct Subcurve {
+    std::list<Rat_point_2> control_points;
+    Rational t_min;
+    Rational t_max;
 
     /*! Get the rational bounding box of the subcurve. */
-    void bbox (Rational& x_min, Rational& y_min,
-               Rational& x_max, Rational& y_max) const
+    void bbox(Rational& x_min, Rational& y_min,
+              Rational& x_max, Rational& y_max) const
     {
-      typename std::list<Rat_point_2>::const_iterator  pit =
+      typename std::list<Rat_point_2>::const_iterator pit =
         control_points.begin();
 
-      CGAL_assertion (pit != control_points.end());
+      CGAL_assertion(pit != control_points.end());
 
       x_min = x_max = pit->x();
       y_min = y_max = pit->y();
 
-      for (++pit; pit != control_points.end(); ++pit)
-      {
-        if (CGAL::compare (x_min, pit->x()) == LARGER)
+      for (++pit; pit != control_points.end(); ++pit) {
+        if (CGAL::compare(x_min, pit->x()) == LARGER)
           x_min = pit->x();
-        else if (CGAL::compare (x_max, pit->x()) == SMALLER)
+        else if (CGAL::compare(x_max, pit->x()) == SMALLER)
           x_max = pit->x();
 
-        if (CGAL::compare (y_min, pit->y()) == LARGER)
+        if (CGAL::compare(y_min, pit->y()) == LARGER)
           y_min = pit->y();
-        else if (CGAL::compare (y_max, pit->y()) == SMALLER)
+        else if (CGAL::compare(y_max, pit->y()) == SMALLER)
           y_max = pit->y();
       }
 
@@ -189,10 +184,10 @@ private:
 public:
 
   /*! Default constructor. */
-  _Bezier_x_monotone_2 () :
+  _Bezier_x_monotone_2() :
     _xid(0),
-    _dir_right (false),
-    _is_vert (false)
+    _dir_right(false),
+    _is_vert(false)
   {}
 
   /*!
@@ -209,80 +204,56 @@ public:
    * \pre B should be an originator of both ps and pt.
    * \pre xid is a non-zero serial number.
    */
-  _Bezier_x_monotone_2 (const Curve_2& B, unsigned int xid,
-                        const Point_2& ps, const Point_2& pt,
-                        Bezier_cache& cache);
+  _Bezier_x_monotone_2(const Curve_2& B, unsigned int xid,
+                       const Point_2& ps, const Point_2& pt,
+                       Bezier_cache& cache);
 
   /*!
    * Get the supporting Bezier curve.
    */
-  const Curve_2& supporting_curve () const
-  {
-    return (_curve);
-  }
+  const Curve_2& supporting_curve() const { return (_curve); }
 
   /*!
    * Get the x-monotone ID of the curve.
    */
-  unsigned int xid () const
-  {
-    return (_xid);
-  }
+  unsigned int xid() const { return (_xid); }
 
   /*!
    * Get the source point.
    */
-  const Point_2& source () const
-  {
-    return (_ps);
-  }
+  const Point_2& source() const { return (_ps); }
 
   /*!
    * Get the target point.
    */
-  const Point_2& target () const
-  {
-    return (_pt);
-  }
+  const Point_2& target() const { return (_pt); }
 
   /*!
    * Get the left endpoint (the lexicographically smaller one).
    */
-  const Point_2& left () const
-  {
-    return (_dir_right ? _ps : _pt);
-  }
+  const Point_2& left() const { return (_dir_right ? _ps : _pt); }
 
   /*!
    * Get the right endpoint (the lexicographically larger one).
    */
-  const Point_2& right () const
-  {
-    return (_dir_right ? _pt : _ps);
-  }
+  const Point_2& right() const { return (_dir_right ? _pt : _ps); }
 
   /*!
    * Check if the subcurve is a vertical segment.
    */
-  bool is_vertical () const
-  {
-    return (_is_vert);
-  }
+  bool is_vertical() const { return (_is_vert); }
 
   /*!
    * Check if the subcurve is directed from left to right.
    */
-  bool is_directed_right () const
-  {
-    return (_dir_right);
-  }
+  bool is_directed_right() const { return (_dir_right); }
 
   /*!
    * Get the approximate parameter range defining the curve.
    * \return A pair of t_src and t_trg, where B(t_src) is the source point
    *         and B(t_trg) is the target point.
    */
-  std::pair<double, double> parameter_range () const;
+  std::pair<double, double> parameter_range() const;
 
   /*!
    * Get the relative position of the query point with respect to the subcurve.
@@ -293,8 +264,8 @@ public:
    *         LARGER if the point is above the arc;
    *         EQUAL if p lies on the arc.
    */
-  Comparison_result point_position (const Point_2& p,
-                                    Bezier_cache& cache) const;
+  Comparison_result point_position(const Point_2& p,
+                                   Bezier_cache& cache) const;
 
   /*!
    * Compare the relative y-position of two x-monotone subcurve to the right
@@ -307,9 +278,9 @@ public:
    *         EQUAL in case of an overlap (should not happen);
    *         LARGER if (*this) lies above cv to the right of p.
    */
-  Comparison_result compare_to_right (const Self& cv,
-                                      const Point_2& p,
-                                      Bezier_cache& cache) const;
+  Comparison_result compare_to_right(const Self& cv,
+                                     const Point_2& p,
+                                     Bezier_cache& cache) const;
 
   /*!
    * Compare the relative y-position of two x-monotone subcurve to the left
@@ -322,106 +293,90 @@ public:
    *         EQUAL in case of an overlap (should not happen);
    *         LARGER if (*this) lies above cv to the right of p.
    */
-  Comparison_result compare_to_left (const Self& cv,
-                                     const Point_2& p,
-                                     Bezier_cache& cache) const;
+  Comparison_result compare_to_left(const Self& cv,
+                                    const Point_2& p,
+                                    Bezier_cache& cache) const;
 
-  /*!
-   * Check whether the two subcurves are equal (have the same graph).
+  /*! Check whether the two subcurves are equal (have the same graph).
    * \param cv The other subcurve.
    * \param cache Caches the vertical tangency points and intersection points.
    * \return (true) if the two subcurves have the same graph;
    *         (false) otherwise.
    */
-  bool equals (const Self& cv,
-               Bezier_cache& cache) const;
+  bool equals(const Self& cv, Bezier_cache& cache) const;
 
-  /*!
-   * Compute the intersections with the given subcurve.
+  /*! Compute the intersections with the given subcurve.
    * \param cv The other subcurve.
    * \param inter_map Caches the bounded intersection points.
    * \param cache Caches the vertical tangency points and intersection points.
    * \param oi The output iterator.
    * \return The past-the-end iterator.
    */
-  template<class OutputIterator>
-  OutputIterator intersect (const Self& cv,
-                            Intersection_map& inter_map,
-                            Bezier_cache& cache,
-                            OutputIterator oi) const
+  template <typename OutputIterator>
+  OutputIterator intersect(const Self& cv,
+                           Intersection_map& inter_map,
+                           Bezier_cache& cache,
+                           OutputIterator oi) const
   {
+    typedef boost::variant<Intersection_point, Self>    Intersection_result;
+
     // In case we have two x-monotone subcurves of the same Bezier curve,
     // check if they have a common left endpoint.
-    if (_curve.is_same (cv._curve))
-    {
-      if (left().is_same (cv.left()) || left().is_same (cv.right()))
-      {
-        *oi = CGAL::make_object (Intersection_point_2 (left(), 0));
-        ++oi;
-      }
+    if (_curve.is_same(cv._curve)) {
+      if (left().is_same(cv.left()) || left().is_same(cv.right()))
+        *oi++ = Intersection_result(Intersection_point(left(), 0));
     }
 
     // Compute the intersections of the two sucurves. Note that for caching
     // purposes we always apply the _intersect() function on the subcurve whose
     // curve ID is smaller.
-    std::vector<Intersection_point_2>   ipts;
-    Self                                ovlp_cv;
-    bool                                do_ovlp;
+    std::vector<Intersection_point> ipts;
+    Self ovlp_cv;
+    bool do_ovlp;
 
     if (_curve.id() <= cv._curve.id())
-      do_ovlp = _intersect (cv, inter_map, cache, ipts, ovlp_cv);
+      do_ovlp = _intersect(cv, inter_map, cache, ipts, ovlp_cv);
     else
-      do_ovlp = cv._intersect (*this, inter_map, cache, ipts, ovlp_cv);
+      do_ovlp = cv._intersect(*this, inter_map, cache, ipts, ovlp_cv);
 
     // In case of overlap, just report the overlapping subcurve.
-    if (do_ovlp)
-    {
-      *oi = CGAL::make_object (ovlp_cv);
-      ++oi;
-      return (oi);
+    if (do_ovlp) {
+      *oi++ = Intersection_result(ovlp_cv);
+      return oi;
     }
 
     // If we have a set of intersection points, sort them in ascending
     // xy-lexicorgraphical order, and insert them to the output iterator.
-    typename std::vector<Intersection_point_2>::const_iterator  ip_it;
-
-    std::sort (ipts.begin(), ipts.end(), Less_intersection_point (cache));
-    for (ip_it = ipts.begin(); ip_it != ipts.end(); ++ip_it)
-    {
-      *oi = CGAL::make_object (*ip_it);
-      ++oi;
+    std::sort(ipts.begin(), ipts.end(), Less_intersection_point(cache));
+    for (auto ip_it = ipts.begin(); ip_it != ipts.end(); ++ip_it) {
+      *oi++ = Intersection_result(*ip_it);
     }
 
     // In case we have two x-monotone subcurves of the same Bezier curve,
     // check if they have a common right endpoint.
-    if (_curve.is_same (cv._curve))
-    {
-      if (right().is_same (cv.left()) || right().is_same (cv.right()))
-      {
-        *oi = CGAL::make_object (Intersection_point_2 (right(), 0));
-        ++oi;
+    if (_curve.is_same(cv._curve)) {
+      if (right().is_same(cv.left()) || right().is_same(cv.right())) {
+        *oi++ = Intersection_result(Intersection_point(right(), 0));
       }
     }
 
-    return (oi);
+    return oi;
   }
 
-  /*!
-   * Split the subcurve into two at a given split point.
+  /*! Split the subcurve into two at a given split point.
    * \param p The split point.
    * \param c1 Output: The first resulting arc, lying to the left of p.
    * \param c2 Output: The first resulting arc, lying to the right of p.
    * \pre p lies in the interior of the subcurve (not one of its endpoints).
    */
-  void split (const Point_2& p,
-              Self& c1, Self& c2) const;
+  void split(const Point_2& p, Self& c1, Self& c2) const;
 
   /*!
    * Check if the two subcurves are mergeable.
    * \param cv The other subcurve.
    * \return Whether the two subcurves can be merged.
    */
-  bool can_merge_with (const Self& cv) const;
+  bool can_merge_with(const Self& cv) const;
 
   /*!
    * Merge the current arc with the given arc.
@@ -429,13 +384,13 @@ public:
    * \pre The two arcs are mergeable.
    * \return The merged arc.
    */
-  Self merge (const Self& cv) const;
+  Self merge(const Self& cv) const;
 
   /*!
    * Flip the subcurve (swap its source and target points).
    * \return The flipped subcurve.
    */
-  Self flip () const
+  Self flip() const
   {
     // Note that we just swap the source and target of the original subcurve
     // and do not touch the supporting Beizer curve.
@@ -445,7 +400,7 @@ public:
     cv._pt = this->_ps;
     cv._dir_right = ! this->_dir_right;
 
-    return (cv);
+    return cv;
   }
 
   Self trim(const Point_2& src, const Point_2& tgt) const
@@ -460,26 +415,21 @@ public:
   }
 
 private:
-
-  /*!
-   * Check if the given t-value is in the range of the subcurve.
+  /*! Check if the given t-value is in the range of the subcurve.
    * \param t The parameter value.
    * \param cache Caches the vertical tangency points and intersection points.
    * \return If t in the parameter-range of the subcurve.
    */
-  bool _is_in_range (const Algebraic& t,
-                     Bezier_cache& cache) const;
+  bool _is_in_range(const Algebraic& t, Bezier_cache& cache) const;
 
-  /*!
-   * Check if the given point lies in the range of this x-monotone subcurve.
+  /*! Check if the given point lies in the range of this x-monotone subcurve.
    * \param p The point, which lies on the supporting Bezier curve.
    * \param is_certain Output: Is the answer we provide is certain.
    * \return Whether p is on the x-monotone subcurve.
    */
-  bool _is_in_range (const Point_2& p, bool& is_certain) const;
+  bool _is_in_range(const Point_2& p, bool& is_certain) const;
 
-  /*!
-   * Given a point p that lies on the supporting Bezier curve (X(t), Y(t)),
+  /*! Given a point p that lies on the supporting Bezier curve (X(t), Y(t)),
    * determine whether p lies within the t-range of the x-monotone subcurve.
    * If so, the value t0 such that p = (X(t0), Y(t0)) is also computed.
    * \param p The point, which lies on the supporting Bezier curve.
@@ -488,23 +438,20 @@ private:
    * \param is_endpoint Output: Whether p equals on the of the endpoints.
    * \return Whether p lies in the t-range of the subcurve.
    */
-  bool _is_in_range (const Point_2& p,
-                     Bezier_cache& cache,
-                     Algebraic& t0,
-                     bool& is_endpoint) const;
+  bool _is_in_range(const Point_2& p,
+                    Bezier_cache& cache,
+                    Algebraic& t0,
+                    bool& is_endpoint) const;
 
-  /*!
-   * Compute a y-coordinate of a point on the x-monotone subcurve with a
+  /*! Compute a y-coordinate of a point on the x-monotone subcurve with a
    * given x-coordinate.
    * \param x0 The given x-coodinate.
    * \param cache Caches the vertical tangency points and intersection points.
    * \return The y-coordinate.
    */
-  Algebraic _get_y (const Rational& x0,
-                    Bezier_cache& cache) const;
+  Algebraic _get_y(const Rational& x0, Bezier_cache& cache) const;
 
-  /*!
-   * Compare the slopes of the subcurve with another given Bezier subcurve at
+  /*! Compare the slopes of the subcurve with another given Bezier subcurve at
    * their given intersection point.
    * \param cv The other subcurve.
    * \param p The intersection point.
@@ -515,20 +462,18 @@ private:
    *         EQUAL if the two slopes are equal;
    *         LARGER if (*this) slope is greater than cv's.
    */
-  Comparison_result _compare_slopes (const Self& cv,
-                                     const Point_2& p,
-                                     Bezier_cache& cache) const;
+  Comparison_result _compare_slopes(const Self& cv,
+                                    const Point_2& p,
+                                    Bezier_cache& cache) const;
 
-  /*!
-   * Get the range of t-value over which the subcurve is defined.
+  /*! Get the range of t-value over which the subcurve is defined.
    * \param cache Caches the vertical tangency points and intersection points.
    * \return A pair comprised of the t-value for the source point and the
    *         t-value for the target point.
    */
-  std::pair<Algebraic, Algebraic> _t_range (Bezier_cache& cache) const;
+  std::pair<Algebraic, Algebraic> _t_range(Bezier_cache& cache) const;
 
-  /*!
-   * Compare the relative y-position of two x-monotone subcurve to the right
+  /*! Compare the relative y-position of two x-monotone subcurve to the right
    * (or to the left) of their intersection point, whose multiplicity is
    * greater than 1.
    * \param cv The other subcurve.
@@ -543,25 +488,22 @@ private:
    *         EQUAL in case of an overlap (should not happen);
    *         LARGER if (*this) lies above cv next to p.
    */
-  Comparison_result _compare_to_side (const Self& cv,
-                                      const Point_2& p,
-                                      bool to_right,
-                                      Bezier_cache& cache) const;
+  Comparison_result _compare_to_side(const Self& cv,
+                                     const Point_2& p,
+                                     bool to_right,
+                                     Bezier_cache& cache) const;
 
-  /*!
-   * Clip the control polygon of the supporting Bezier curve such that it
+  /*! Clip the control polygon of the supporting Bezier curve such that it
    * fits the current x-monotone subcurve.
    * \param ctrl Output: The clipped control polygon.
    * \param t_min Output: The minimal t-value of the clipped curve.
    * \param t_max Output: The maximal t-value of the clipped curve.
    */
-  void _clip_control_polygon
-        (typename Bounding_traits::Control_points& ctrl,
-         typename Bounding_traits::NT& t_min,
-         typename Bounding_traits::NT& t_max) const;
+  void _clip_control_polygon(typename Bounding_traits::Control_points& ctrl,
+                             typename Bounding_traits::NT& t_min,
+                             typename Bounding_traits::NT& t_max) const;
 
-  /*!
-   * Approximate the intersection points between the supporting Bezier curves
+  /*! Approximate the intersection points between the supporting Bezier curves
    * of the given x-monotone curves.
    * \param cv The x-monotone curve we intersect.
    * \param inter_pts Output: An output list of intersection points between
@@ -571,11 +513,10 @@ private:
    *                          between the subcurves are approximated.
    * \return Whether all intersection points where successfully approximated.
    */
-  bool _approximate_intersection_points (const Self& cv,
-                                         std::list<Point_2>& inter_pts) const;
+  bool _approximate_intersection_points(const Self& cv,
+                                        std::list<Point_2>& inter_pts) const;
 
-  /*!
-   * Compute the intersections with the given subcurve.
+  /*! Compute the intersections with the given subcurve.
    * \param cv The other subcurve.
    * \param inter_map Caches the bounded intersection points.
    * \param cache Caches the vertical tangency points and intersection points.
@@ -583,14 +524,13 @@ private:
    * \param ovlp_cv Output: An overlapping subcurve (if exists).
    * \return Whether an overlap has occurred.
    */
-  bool _intersect (const Self& cv,
-                   Intersection_map& inter_map,
-                   Bezier_cache& cache,
-                   std::vector<Intersection_point_2>& ipts,
-                   Self& ovlp_cv) const;
+  bool _intersect(const Self& cv,
+                  Intersection_map& inter_map,
+                  Bezier_cache& cache,
+                  std::vector<Intersection_point>& ipts,
+                  Self& ovlp_cv) const;
 
-  /*!
-   * Compute the exact vertical position of the given point with respect to
+  /*! Compute the exact vertical position of the given point with respect to
    * the x-monotone curve.
    * \param p The point.
    * \param force_exact Sould we force an exact result.
@@ -598,64 +538,63 @@ private:
    *         LARGER if the point is above the curve;
    *         EQUAL if p lies on the curve.
    */
-  Comparison_result _exact_vertical_position (const Point_2& p,
-                                              bool
+  Comparison_result _exact_vertical_position(const Point_2& p,
+                                             bool
 #if !defined(CGAL_NO_ASSERTIONS)
-                                              force_exact
+                                             force_exact
 #endif
-                                              ) const;
+                                             ) const;
 
 };
 
 /*!
  * Exporter for Bezier curves.
  */
-template <class Rat_kernel, class Alg_kernel, class Nt_traits,
-          class Bounding_traits>
+template <typename Rat_kernel, typename Alg_kernel, typename Nt_traits,
+          typename Bounding_traits>
 std::ostream&
-operator<< (std::ostream& os,
-            const _Bezier_x_monotone_2<Rat_kernel, Alg_kernel, Nt_traits,
-                                       Bounding_traits>& cv)
+operator<<(std::ostream& os,
+           const _Bezier_x_monotone_2
+           <Rat_kernel, Alg_kernel, Nt_traits, Bounding_traits>& cv)
 {
   os << cv.supporting_curve()
      << " [" << cv.xid()
      << "] | " << cv.source()
      << " --> " << cv.target();
 
-  return (os);
+  return os;
 }
 
 // ---------------------------------------------------------------------------
 // Constructor given two endpoints.
 //
-template <class RatKer, class AlgKer, class NtTrt, class BndTrt>
-_Bezier_x_monotone_2<RatKer, AlgKer, NtTrt, BndTrt>::_Bezier_x_monotone_2
-        (const Curve_2& B, unsigned int xid,
-         const Point_2& ps, const Point_2& pt,
-         Bezier_cache& cache) :
-  _curve (B),
-  _xid (xid),
-  _ps (ps),
-  _pt (pt),
-  _is_vert (false)
+template <typename RatKer, typename AlgKer, typename NtTrt, typename BndTrt>
+_Bezier_x_monotone_2<RatKer, AlgKer, NtTrt, BndTrt>::
+_Bezier_x_monotone_2(const Curve_2& B, unsigned int xid,
+                     const Point_2& ps, const Point_2& pt,
+                     Bezier_cache& cache) :
+  _curve(B),
+  _xid(xid),
+  _ps(ps),
+  _pt(pt),
+  _is_vert(false)
 {
-  CGAL_precondition (xid > 0);
+  CGAL_precondition(xid > 0);
 
   // Get the originators of the point that correspond to the curve B.
-  Originator_iterator   ps_org = ps.get_originator (B, _xid);
-  CGAL_precondition (ps_org != ps.originators_end());
+  Originator_iterator ps_org = ps.get_originator(B, _xid);
+  CGAL_precondition(ps_org != ps.originators_end());
 
-  Originator_iterator   pt_org = pt.get_originator (B, _xid);
-  CGAL_precondition (pt_org != pt.originators_end());
+  Originator_iterator pt_org = pt.get_originator(B, _xid);
+  CGAL_precondition(pt_org != pt.originators_end());
 
   // Check if the subcurve is directed left or right.
-  const Comparison_result    res = _ps.compare_x (_pt, cache);
+  const Comparison_result res = _ps.compare_x(_pt, cache);
 
-  if (res == EQUAL)
-  {
+  if (res == EQUAL) {
     // We have a vertical segment. Check if the source is below the target.
     _is_vert = true;
-    _dir_right = (CGAL::compare (_ps.y(), _pt.y()) == SMALLER);
+    _dir_right = (CGAL::compare(_ps.y(), _pt.y()) == SMALLER);
   }
   else
   {
@@ -665,89 +604,83 @@ _Bezier_x_monotone_2<RatKer, AlgKer, NtT
   // Check if the value of the parameter t increases when we traverse the
   // curve from left to right: If the curve is directed to the right, we
   // check if t_src < t_trg, otherwise we check whether t_src > t_trg.
-  Comparison_result      t_res;
+  Comparison_result t_res;
 
-  if (CGAL::compare (ps_org->point_bound().t_max,
-                     pt_org->point_bound().t_min) == SMALLER ||
-      CGAL::compare (ps_org->point_bound().t_min,
-                     pt_org->point_bound().t_max) == LARGER)
+  if (CGAL::compare(ps_org->point_bound().t_max,
+                    pt_org->point_bound().t_min) == SMALLER ||
+      CGAL::compare(ps_org->point_bound().t_min,
+                    pt_org->point_bound().t_max) == LARGER)
   {
     // Perform the comparison assuming that the possible parameter
     // values do not overlap.
-    t_res = CGAL::compare (ps_org->point_bound().t_min,
+    t_res = CGAL::compare(ps_org->point_bound().t_min,
                            pt_org->point_bound().t_min);
   }
-  else
-  {
+  else {
     // In this case both exact parameter values must be known.
     // We use them to perform an exact comparison.
-    CGAL_assertion (ps_org->has_parameter() && pt_org->has_parameter());
+    CGAL_assertion(ps_org->has_parameter() && pt_org->has_parameter());
 
-    t_res = CGAL::compare (ps_org->parameter(), pt_org->parameter());
+    t_res = CGAL::compare(ps_org->parameter(), pt_org->parameter());
   }
 
-  CGAL_precondition (t_res != EQUAL);
+  CGAL_precondition(t_res != EQUAL);
 
-  if (_dir_right)
-    _inc_to_right = (t_res == SMALLER);
-  else
-    _inc_to_right = (t_res == LARGER);
+  if (_dir_right) _inc_to_right = (t_res == SMALLER);
+  else _inc_to_right = (t_res == LARGER);
 }
 
 // ---------------------------------------------------------------------------
 // Get the approximate parameter range defining the curve.
 //
-template <class RatKer, class AlgKer, class NtTrt, class BndTrt>
+template <typename RatKer, typename AlgKer, typename NtTrt, typename BndTrt>
 std::pair<double, double>
-_Bezier_x_monotone_2<RatKer, AlgKer, NtTrt, BndTrt>::parameter_range () const
+_Bezier_x_monotone_2<RatKer, AlgKer, NtTrt, BndTrt>::parameter_range() const
 {
   // First try to use the approximate representation of the endpoints.
-  Originator_iterator  s_org = _ps.get_originator (_curve, _xid);
-  CGAL_assertion (s_org != _ps.originators_end());
+  Originator_iterator  s_org = _ps.get_originator(_curve, _xid);
+  CGAL_assertion(s_org != _ps.originators_end());
 
-  Originator_iterator  t_org = _pt.get_originator (_curve, _xid);
-  CGAL_assertion (t_org != _pt.originators_end());
+  Originator_iterator  t_org = _pt.get_originator(_curve, _xid);
+  CGAL_assertion(t_org != _pt.originators_end());
 
-  double  t_src = (CGAL::to_double (s_org->point_bound().t_min) +
-                   CGAL::to_double (s_org->point_bound().t_max)) / 2;
-  double  t_trg = (CGAL::to_double (t_org->point_bound().t_min) +
-                   CGAL::to_double (t_org->point_bound().t_max)) / 2;
+  double  t_src = (CGAL::to_double(s_org->point_bound().t_min) +
+                   CGAL::to_double(s_org->point_bound().t_max)) / 2;
+  double  t_trg = (CGAL::to_double(t_org->point_bound().t_min) +
+                   CGAL::to_double(t_org->point_bound().t_max)) / 2;
 
-  return (std::make_pair (t_src, t_trg));
+  return (std::make_pair(t_src, t_trg));
 }
 
 // ---------------------------------------------------------------------------
 // Get the relative position of the query point with respect to the subcurve.
 //
-template <class RatKer, class AlgKer, class NtTrt, class BndTrt>
+template <typename RatKer, typename AlgKer, typename NtTrt, typename BndTrt>
 Comparison_result
-_Bezier_x_monotone_2<RatKer, AlgKer, NtTrt, BndTrt>::point_position
-    (const Point_2& p,
-     Bezier_cache& cache) const
+_Bezier_x_monotone_2<RatKer, AlgKer, NtTrt, BndTrt>::
+point_position(const Point_2& p, Bezier_cache& cache) const
 {
   Nt_traits nt_traits;
 
   //First check if the bezier is a vertical segment
-  if (is_vertical())
-  {
-    if (! p.is_exact()) p.make_exact (cache);
-    if (! _ps.is_exact()) _ps.make_exact (cache);
-    if (! _pt.is_exact()) _ps.make_exact (cache);
+  if (is_vertical()) {
+    if (! p.is_exact()) p.make_exact(cache);
+    if (! _ps.is_exact()) _ps.make_exact(cache);
+    if (! _pt.is_exact()) _ps.make_exact(cache);
 
-    if (p.is_rational() && _ps.is_rational() && _pt.is_rational())
-    {
+    if (p.is_rational() && _ps.is_rational() && _pt.is_rational()) {
       const Rat_point_2&  rat_p = (Rat_point_2) p;
       const Rat_point_2&  rat_ps = (Rat_point_2) _ps;
       const Rat_point_2&  rat_pt = (Rat_point_2) _pt;
 
-      Comparison_result res1 = (CGAL::compare (rat_p.y(), rat_ps.y()));
-      Comparison_result res2 = (CGAL::compare (rat_p.y(), rat_pt.y()));
-      return (res1==res2 ? res1:EQUAL);
+      Comparison_result res1 = (CGAL::compare(rat_p.y(), rat_ps.y()));
+      Comparison_result res2 = (CGAL::compare(rat_p.y(), rat_pt.y()));
+      return (res1==  res2 ? res1:EQUAL);
     }
 
-    Comparison_result res1 = (CGAL::compare (p.y(), _ps.y()));
-    Comparison_result res2 = (CGAL::compare (p.y(), _pt.y()));
-    return (res1==res2 ? res1:EQUAL);
+    Comparison_result res1 = (CGAL::compare(p.y(), _ps.y()));
+    Comparison_result res2 = (CGAL::compare(p.y(), _pt.y()));
+    return (res1==  res2 ? res1:EQUAL);
   }
 
   if (p.identical(_ps)) {
@@ -757,106 +690,99 @@ _Bezier_x_monotone_2<RatKer, AlgKer, NtT
   // Then check whether the bezier is an horizontal segment or
   // if p has the same x-coordinate as one of the endpoint
 
-  const Comparison_result  res1 =  p.compare_x (_ps, cache);
+  const Comparison_result  res1 =  p.compare_x(_ps, cache);
 
   if (res1 == EQUAL || nt_traits.degree(_curve.y_polynomial()) <= 0)
   {
-    if (! p.is_exact()) p.make_exact (cache);
-    if (! _ps.is_exact()) _ps.make_exact (cache);
+    if (! p.is_exact()) p.make_exact(cache);
+    if (! _ps.is_exact()) _ps.make_exact(cache);
 
     // If both point are rational, compare their rational y-coordinates.
     if (p.is_rational() && _ps.is_rational())
     {
-      const Rat_point_2&  rat_p = (Rat_point_2) p;
-      const Rat_point_2&  rat_ps = (Rat_point_2) _ps;
+      const Rat_point_2& rat_p = (Rat_point_2) p;
+      const Rat_point_2& rat_ps = (Rat_point_2) _ps;
 
-      return (CGAL::compare (rat_p.y(), rat_ps.y()));
+      return (CGAL::compare(rat_p.y(), rat_ps.y()));
     }
 
     // Compare the algebraic y-coordinates.
-    return (CGAL::compare (p.y(), _ps.y()));
+    return (CGAL::compare(p.y(), _ps.y()));
   }
 
-  if (p.identical(_pt)) {
-    return EQUAL;
-  }
+  if (p.identical(_pt)) return EQUAL;
 
-  const Comparison_result  res2 = p.compare_x (_pt, cache);
+  const Comparison_result  res2 = p.compare_x(_pt, cache);
 
-  if (res2 == EQUAL)
-  {
+  if (res2 == EQUAL) {
     // In this case both points must be exact.
-    CGAL_assertion (p.is_exact() && _pt.is_exact());
+    CGAL_assertion(p.is_exact() && _pt.is_exact());
 
     // If both point are rational, compare their rational y-coordinates.
-    if (p.is_rational() && _pt.is_rational())
-    {
-      const Rat_point_2&  rat_p = (Rat_point_2) p;
-      const Rat_point_2&  rat_pt = (Rat_point_2) _pt;
+    if (p.is_rational() && _pt.is_rational()) {
+      const Rat_point_2& rat_p = (Rat_point_2) p;
+      const Rat_point_2& rat_pt = (Rat_point_2) _pt;
 
-      return (CGAL::compare (rat_p.y(), rat_pt.y()));
+      return (CGAL::compare(rat_p.y(), rat_pt.y()));
     }
 
     // Compare the algebraic y-coordinates.
-    return (CGAL::compare (p.y(), _pt.y()));
+    return (CGAL::compare(p.y(), _pt.y()));
   }
 
   // Make sure that p is in the x-range of our subcurve.
-  CGAL_precondition (res1 != res2);
+  CGAL_precondition(res1 != res2);
 
   // Check for the case when curve is an originator of the point.
-  Originator_iterator   p_org = p.get_originator (_curve, _xid);
+  Originator_iterator p_org = p.get_originator(_curve, _xid);
 
-  if (p_org != p.originators_end())
-  {
+  if (p_org != p.originators_end()) {
     CGAL_assertion_code
-      (Originator_iterator ps_org = _ps.get_originator (_curve, _xid);
+      (Originator_iterator ps_org = _ps.get_originator(_curve, _xid);
        CGAL_assertion(ps_org != _ps.originators_end());
-       Originator_iterator pt_org = _pt.get_originator (_curve, _xid);
+       Originator_iterator pt_org = _pt.get_originator(_curve, _xid);
        CGAL_assertion(pt_org != _pt.originators_end()));
 
     // Check if the point is in the parameter range of this subcurve.
     // First try an approximate check of the parameter bounds.
-    bool  correct_res;
-    bool  in_range = false;
+    bool correct_res;
+    bool in_range = false;
 
-    in_range = _is_in_range (p, correct_res);
+    in_range = _is_in_range(p, correct_res);
 
-    if (! correct_res)
-    {
+    if (! correct_res) {
       // Perform the comparsion in an exact manner.
       if (! p.is_exact())
-        p.make_exact (cache);
+        p.make_exact(cache);
 
-      CGAL_assertion (p_org->has_parameter());
+      CGAL_assertion(p_org->has_parameter());
 
-      in_range = _is_in_range (p_org->parameter(), cache);
+      in_range = _is_in_range(p_org->parameter(), cache);
     }
 
-    if (in_range)
-      return (EQUAL);
+    if (in_range) return (EQUAL);
   }
 
   // Call the vertical-position function that uses the bounding-boxes
   // to evaluate the comparsion result.
-  typename Bounding_traits::Control_points  cp;
+  typename Bounding_traits::Control_points cp;
 
-  std::copy (_curve.control_points_begin(), _curve.control_points_end(),
-             std::back_inserter(cp));
+  std::copy(_curve.control_points_begin(), _curve.control_points_end(),
+            std::back_inserter(cp));
 
-  Originator_iterator           ps_org = _ps.get_originator (_curve, _xid);
-  CGAL_assertion (ps_org != _ps.originators_end());
+  Originator_iterator ps_org = _ps.get_originator(_curve, _xid);
+  CGAL_assertion(ps_org != _ps.originators_end());
 
-  Originator_iterator           pt_org = _pt.get_originator (_curve, _xid);
-  CGAL_assertion (pt_org != _pt.originators_end());
+  Originator_iterator pt_org = _pt.get_originator(_curve, _xid);
+  CGAL_assertion(pt_org != _pt.originators_end());
 
-  Comparison_result             res_bound = EQUAL;
-  typename Bounding_traits::NT  x_min, y_min, x_max, y_max;
-  bool                          can_refine;
+  Comparison_result res_bound = EQUAL;
+  typename Bounding_traits::NT x_min, y_min, x_max, y_max;
+  bool can_refine;
 
-  p.get_bbox (x_min, y_min, x_max, y_max);
+  p.get_bbox(x_min, y_min, x_max, y_max);
 
-  if (CGAL::compare (ps_org->point_bound().t_max,
+  if (CGAL::compare(ps_org->point_bound().t_max,
                      pt_org->point_bound().t_min) == SMALLER)
   {
     // Examine the parameter range of the originator of the source point
@@ -866,14 +792,13 @@ _Bezier_x_monotone_2<RatKer, AlgKer, NtT
     can_refine = ! _ps.is_exact();
     do
     {
-      const Rat_point_2&  ps = _curve (ps_org->point_bound().t_max);
+      const Rat_point_2&  ps = _curve(ps_org->point_bound().t_max);
 
-      if ((_dir_right && CGAL::compare (ps.x(), x_min) != LARGER) ||
-          (! _dir_right && CGAL::compare (ps.x(), x_max) != SMALLER))
+      if ((_dir_right && CGAL::compare(ps.x(), x_min) != LARGER) ||
+          (! _dir_right && CGAL::compare(ps.x(), x_max) != SMALLER))
         break;
 
-      if (can_refine)
-        can_refine = _ps.refine();
+      if (can_refine) can_refine = _ps.refine();
     } while (can_refine);
 
     // Examine the parameter range of the originator of the target point
@@ -883,10 +808,10 @@ _Bezier_x_monotone_2<RatKer, AlgKer, NtT
     can_refine = ! _pt.is_exact();
     do
     {
-      const Rat_point_2&  pt = _curve (pt_org->point_bound().t_min);
+      const Rat_point_2&  pt = _curve(pt_org->point_bound().t_min);
 
-      if ((_dir_right && CGAL::compare (pt.x(), x_max) != SMALLER) ||
-          (! _dir_right && CGAL::compare (pt.x(), x_min) != LARGER))
+      if ((_dir_right && CGAL::compare(pt.x(), x_max) != SMALLER) ||
+          (! _dir_right && CGAL::compare(pt.x(), x_min) != LARGER))
         break;
 
       if (can_refine)
@@ -896,11 +821,11 @@ _Bezier_x_monotone_2<RatKer, AlgKer, NtT
     // In this case the parameter value of the source is smaller than the
     // target's, so we compare the point with the subcurve of B defined over
     // the proper parameter range.
-    res_bound = p.vertical_position (cp,
+    res_bound = p.vertical_position(cp,
                                      ps_org->point_bound().t_max,
                                      pt_org->point_bound().t_min);
   }
-  else if (CGAL::compare (pt_org->point_bound().t_max,
+  else if (CGAL::compare(pt_org->point_bound().t_max,
                           ps_org->point_bound().t_min) == SMALLER)
   {
     // Examine the parameter range of the originator of the source point
@@ -908,12 +833,11 @@ _Bezier_x_monotone_2<RatKer, AlgKer, NtT
     // lies to the left of p if the curve is directed from left to right
     // (or to the right of p, if the subcurve is directed from right to left).
     can_refine = ! _ps.is_exact();
-    do
-    {
-      const Rat_point_2&  ps = _curve (ps_org->point_bound().t_min);
+    do {
+      const Rat_point_2&  ps = _curve(ps_org->point_bound().t_min);
 
-      if ((_dir_right && CGAL::compare (ps.x(), x_min) != LARGER) ||
-          (! _dir_right && CGAL::compare (ps.x(), x_max) != SMALLER))
+      if ((_dir_right && CGAL::compare(ps.x(), x_min) != LARGER) ||
+          (! _dir_right && CGAL::compare(ps.x(), x_max) != SMALLER))
         break;
 
       if (can_refine)
@@ -925,12 +849,11 @@ _Bezier_x_monotone_2<RatKer, AlgKer, NtT
     // lies to the right of p if the curve is directed from left to right
     // (or to the left of p, if the subcurve is directed from right to left).
     can_refine = ! _pt.is_exact();
-    do
-    {
-      const Rat_point_2&  pt = _curve (pt_org->point_bound().t_max);
+    do {
+      const Rat_point_2&  pt = _curve(pt_org->point_bound().t_max);
 
-      if ((_dir_right && CGAL::compare (pt.x(), x_max) != SMALLER) ||
-          (! _dir_right && CGAL::compare (pt.x(), x_min) != LARGER))
+      if ((_dir_right && CGAL::compare(pt.x(), x_max) != SMALLER) ||
+          (! _dir_right && CGAL::compare(pt.x(), x_min) != LARGER))
         break;
 
       if (can_refine)
@@ -940,7 +863,7 @@ _Bezier_x_monotone_2<RatKer, AlgKer, NtT
     // In this case the parameter value of the source is large than the
     // target's, so we compare the point with the subcurve of B defined over
     // the proper parameter range.
-    res_bound = p.vertical_position (cp,
+    res_bound = p.vertical_position(cp,
                                      pt_org->point_bound().t_max,
                                      ps_org->point_bound().t_min);
   }
@@ -960,13 +883,14 @@ _Bezier_x_monotone_2<RatKer, AlgKer, NtT
 
     std::vector <Algebraic> roots;
     std::pair<double,double> prange = parameter_range();
-    nt_traits.compute_polynomial_roots (poly_x,prange.first,prange.second,std::back_inserter(roots));
+    nt_traits.compute_polynomial_roots(poly_x,prange.first,prange.second, std::back_inserter(roots));
 
-    CGAL_assertion(roots.size()==1); //p is in the range and the curve is x-monotone
+    //p is in the range and the curve is x-monotone
+    CGAL_assertion(roots.size() == 1);
 
     return CGAL::compare(
       ((Rat_point_2) p).y(),
-      nt_traits.evaluate_at (_curve.y_polynomial(), *roots.begin())
+      nt_traits.evaluate_at(_curve.y_polynomial(), *roots.begin())
     );
   }
 
@@ -974,22 +898,22 @@ _Bezier_x_monotone_2<RatKer, AlgKer, NtT
   // p lies of the given subcurve. We take one of p's originating curves and
   // compute its intersections with our x-monotone curve.
   if (! p.is_exact())
-    p.make_exact (cache);
+    p.make_exact(cache);
 
-  CGAL_assertion (p.originators_begin() != p.originators_end());
+  CGAL_assertion(p.originators_begin() != p.originators_end());
 
   Originator   org = *(p.originators_begin());
   bool         do_ovlp;
   bool         swap_order = (_curve.id() > org.curve().id());
   const Intersect_list&  inter_list = (! swap_order ?
-    (cache.get_intersections (_curve.id(),
+    (cache.get_intersections(_curve.id(),
                               _curve.x_polynomial(), _curve.x_norm(),
                               _curve.y_polynomial(), _curve.y_norm(),
                               org.curve().id(),
                               org.curve().x_polynomial(), org.curve().x_norm(),
                               org.curve().y_polynomial(), org.curve().y_norm(),
                               do_ovlp)) :
-    (cache.get_intersections (org.curve().id(),
+    (cache.get_intersections(org.curve().id(),
                               org.curve().x_polynomial(), org.curve().x_norm(),
                               org.curve().y_polynomial(), org.curve().y_norm(),
                               _curve.id(),
@@ -997,29 +921,24 @@ _Bezier_x_monotone_2<RatKer, AlgKer, NtT
                               _curve.y_polynomial(), _curve.y_norm(),
                               do_ovlp)));
 
-  if (do_ovlp)
-    return (EQUAL);
+  if (do_ovlp) return EQUAL;
 
   // Go over the intersection points and look for p there.
-  Intersect_iter       iit;
-
-  for (iit = inter_list.begin(); iit != inter_list.end(); ++iit)
-  {
+  for (auto iit = inter_list.begin(); iit != inter_list.end(); ++iit) {
     // Get the parameter of the originator and compare it to p's parameter.
-    const Algebraic&  s = swap_order ? iit->s : iit->t;
+    const Algebraic& s = swap_order ? iit->s : iit->t;
 
-    if (CGAL::compare (s, org.parameter()) == EQUAL)
-    {
+    if (CGAL::compare(s, org.parameter()) == EQUAL) {
       // Add this curve as an originator for p.
-      const Algebraic&  t = swap_order ? iit->t : iit->s;
+      const Algebraic& t = swap_order ? iit->t : iit->s;
 
-      CGAL_assertion (_is_in_range (t, cache));
+      CGAL_assertion(_is_in_range(t, cache));
 
-      Point_2&  pt = const_cast<Point_2&> (p);
+      Point_2& pt = const_cast<Point_2&>(p);
       pt.add_originator (Originator (_curve, _xid, t));
 
       // The point p lies on the subcurve.
-      return (EQUAL);
+      return EQUAL;
     }
   }
 
@@ -1028,21 +947,14 @@ _Bezier_x_monotone_2<RatKer, AlgKer, NtT
   // between the curve and the point. (This case should be very rare.)
   // Note that we first try to work with inexact endpoint representation, and
   // only if we fail we make the endpoints of the x-monotone curves exact.
-  if (! p.is_exact())
-    p.make_exact (cache);
-
-  Comparison_result  exact_res = _exact_vertical_position (p, false);
-
-  if (exact_res != EQUAL)
-    return (exact_res);
+  if (! p.is_exact()) p.make_exact (cache);
 
-  if (! _ps.is_exact())
-    _ps.make_exact (cache);
+  Comparison_result exact_res = _exact_vertical_position(p, false);
+  if (exact_res != EQUAL) return (exact_res);
+  if (! _ps.is_exact()) _ps.make_exact(cache);
+  if (! _pt.is_exact()) _pt.make_exact(cache);
 
-  if (! _pt.is_exact())
-    _pt.make_exact (cache);
-
-  return (_exact_vertical_position (p, true));
+  return (_exact_vertical_position(p, true));
 }
 
 // ---------------------------------------------------------------------------
@@ -1051,57 +963,45 @@ _Bezier_x_monotone_2<RatKer, AlgKer, NtT
 //
 template <class RatKer, class AlgKer, class NtTrt, class BndTrt>
 Comparison_result
-_Bezier_x_monotone_2<RatKer, AlgKer, NtTrt, BndTrt>::compare_to_right
-        (const Self& cv,
-         const Point_2& p,
-         Bezier_cache& cache) const
+_Bezier_x_monotone_2<RatKer, AlgKer, NtTrt, BndTrt>::
+compare_to_right(const Self& cv,
+                 const Point_2& p,
+                 Bezier_cache& cache) const
 {
   CGAL_precondition (p.compare_xy (right(), cache) != LARGER);
   CGAL_precondition (p.compare_xy (cv.right(), cache) != LARGER);
 
-  if (this == &cv)
-    return (EQUAL);
+  if (this == &cv) return EQUAL;
 
   // Make sure that p is incident to both curves (either equals the left
   // endpoint or lies in the curve interior). Note that this is important to
   // carry out these tests, as it assures us the eventually both curves are
   // originators of p.
-  if (! p.equals (left(), cache))
-  {
-    if (point_position (p, cache) != EQUAL)
-    {
+  if (! p.equals (left(), cache)) {
+    if (point_position (p, cache) != EQUAL) {
       CGAL_precondition_msg (false, "p is not on cv1");
     }
   }
 
-  if (! p.equals (cv.left(), cache))
-  {
-    if (cv.point_position (p, cache) != EQUAL)
-    {
+  if (! p.equals (cv.left(), cache)) {
+    if (cv.point_position (p, cache) != EQUAL) {
       CGAL_precondition_msg (false, "p is not on cv2");
     }
   }
 
   // Check for vertical subcurves. A vertical segment is above any other
   // x-monotone subcurve to the right of their common endpoint.
-  if (is_vertical())
-  {
-    if (cv.is_vertical())
-      // Both are vertical segments with a common endpoint, so they overlap:
-      return (EQUAL);
-
+  if (is_vertical()) {
+    // Both are vertical segments with a common endpoint, so they overlap:
+    if (cv.is_vertical()) return (EQUAL);
     return (LARGER);
   }
-  else if (cv.is_vertical())
-  {
-    return (SMALLER);
-  }
+  else if (cv.is_vertical()) return (SMALLER);
 
   // Check if both subcurves originate from the same Bezier curve.
-  Nt_traits       nt_traits;
+  Nt_traits nt_traits;
 
-  if (_curve.is_same (cv._curve))
-  {
+  if (_curve.is_same (cv._curve)) {
     // Get the originator, and check whether p is a vertical tangency
     // point of this originator (otherwise it is a self-intersection point,
     // and we proceed as if it is a regular intersection point).
@@ -1109,24 +1009,20 @@ _Bezier_x_monotone_2<RatKer, AlgKer, NtT
 
     CGAL_assertion (org != p.originators_end());
 
-    if (org->point_bound().type == Bez_point_bound::VERTICAL_TANGENCY_PT)
-    {
+    if (org->point_bound().type == Bez_point_bound::VERTICAL_TANGENCY_PT) {
       CGAL_assertion (_inc_to_right != cv._inc_to_right);
 
-      if (! p.is_exact())
-      {
+      if (! p.is_exact()) {
         // Comparison based on the control polygon of the bounded vertical
         // tangency point, using the fact this polygon is y-monotone.
         const typename Bounding_traits::Control_points& cp =
           org->point_bound().ctrl;
 
-        if (_inc_to_right)
-        {
-          return (CGAL::compare (cp.back().y(), cp.front().y()));
+        if (_inc_to_right) {
+          return (CGAL::compare(cp.back().y(), cp.front().y()));
         }
-        else
-        {
-          return (CGAL::compare (cp.front().y(), cp.back().y()));
+        else {
+          return (CGAL::compare(cp.front().y(), cp.back().y()));
         }
       }
 
@@ -1135,14 +1031,14 @@ _Bezier_x_monotone_2<RatKer, AlgKer, NtT
       // In this case we know that we have a vertical tangency at t0, so
       // X'(t0) = 0. We evaluate the sign of Y'(t0) in order to find the
       // vertical position of the two subcurves to the right of this point.
-      CGAL_assertion (org->has_parameter());
+      CGAL_assertion(org->has_parameter());
 
-      const Algebraic&  t0 = org->parameter();
-      Polynomial        polyY_der = nt_traits.derive (_curve.y_polynomial());
-      const CGAL::Sign  sign_der =
-        CGAL::sign (nt_traits.evaluate_at (polyY_der, t0));
+      const Algebraic& t0 = org->parameter();
+      Polynomial polyY_der = nt_traits.derive(_curve.y_polynomial());
+      const CGAL::Sign sign_der =
+        CGAL::sign(nt_traits.evaluate_at(polyY_der, t0));
 
-      CGAL_assertion (sign_der != CGAL::ZERO);
+      CGAL_assertion(sign_der != CGAL::ZERO);
 
       if (_inc_to_right)
         return ((sign_der == CGAL::POSITIVE) ? LARGER : SMALLER);
@@ -1154,28 +1050,27 @@ _Bezier_x_monotone_2<RatKer, AlgKer, NtT
   // Compare the slopes of the two supporting curves at p. In the general
   // case, the slopes are not equal and their comparison gives us the
   // vertical order to p's right.
-  Comparison_result   slope_res = _compare_slopes (cv, p, cache);
+  Comparison_result slope_res = _compare_slopes(cv, p, cache);
 
-  if (slope_res != EQUAL)
-    return (slope_res);
+  if (slope_res != EQUAL) return (slope_res);
 
   // Compare the two subcurves by choosing some point to the right of p
   // and comparing the vertical position there.
   Comparison_result   right_res;
 
-  if (right().compare_x (cv.right(), cache) != LARGER)
+  if (right().compare_x(cv.right(), cache) != LARGER)
   {
-    right_res = _compare_to_side (cv, p,
+    right_res = _compare_to_side(cv, p,
                                   true,           // Compare to p's right.
                                   cache);
   }
   else
   {
-    right_res = cv._compare_to_side (*this, p,
+    right_res = cv._compare_to_side(*this, p,
                                      true,        // Compare to p's right.
                                      cache);
 
-    right_res = CGAL::opposite (right_res);
+    right_res = CGAL::opposite(right_res);
   }
 
   return (right_res);
@@ -1192,8 +1087,8 @@ _Bezier_x_monotone_2<RatKer, AlgKer, NtT
          const Point_2& p,
          Bezier_cache& cache) const
 {
-  CGAL_precondition (p.compare_xy (left(), cache) != SMALLER);
-  CGAL_precondition (p.compare_xy (cv.left(), cache) != SMALLER);
+  CGAL_precondition(p.compare_xy(left(), cache) != SMALLER);
+  CGAL_precondition(p.compare_xy(cv.left(), cache) != SMALLER);
 
   if (this == &cv)
     return (EQUAL);
@@ -1202,19 +1097,19 @@ _Bezier_x_monotone_2<RatKer, AlgKer, NtT
   // endpoint or lies in the curve interior). Note that this is important to
   // carry out these tests, as it assures us the eventually both curves are
   // originators of p.
-  if (! p.equals (right(), cache))
+  if (! p.equals(right(), cache))
   {
-    if (point_position (p, cache) != EQUAL)
+    if (point_position(p, cache) != EQUAL)
     {
-      CGAL_precondition_msg (false, "p is not on cv1");
+      CGAL_precondition_msg(false, "p is not on cv1");
     }
   }
 
-  if (! p.equals (cv.right(), cache))
+  if (! p.equals(cv.right(), cache))
   {
-    if (cv.point_position (p, cache) != EQUAL)
+    if (cv.point_position(p, cache) != EQUAL)
     {
-      CGAL_precondition_msg (false, "p is not on cv2");
+      CGAL_precondition_msg(false, "p is not on cv2");
     }
   }
 
@@ -1236,18 +1131,19 @@ _Bezier_x_monotone_2<RatKer, AlgKer, NtT
   // Check if both subcurves originate from the same Bezier curve.
   Nt_traits       nt_traits;
 
-  if (_curve.is_same (cv._curve))
+  if (_curve.is_same(cv._curve))
   {
     // Get the originator, and check whether p is a vertical tangency
     // point of this originator (otherwise it is a self-intersection point,
     // and we proceed as if it is a regular intersection point).
-    Originator_iterator  org = p.get_originator (_curve, _xid);
+    Originator_iterator  org = p.get_originator(_curve, _xid);
 
-    CGAL_assertion (org != p.originators_end());
-    CGAL_assertion (_inc_to_right != cv._inc_to_right);
+    CGAL_assertion(org != p.originators_end());
 
     if (org->point_bound().type == Bez_point_bound::VERTICAL_TANGENCY_PT)
     {
+      CGAL_assertion(_inc_to_right != cv._inc_to_right);
+
       if (! p.is_exact())
       {
         // Comparison based on the control polygon of the bounded vertical
@@ -1270,14 +1166,14 @@ _Bezier_x_monotone_2<RatKer, AlgKer, NtT
       // In this case we know that we have a vertical tangency at t0, so
       // X'(t0) = 0. We evaluate the sign of Y'(t0) in order to find the
       // vertical position of the two subcurves to the right of this point.
-      CGAL_assertion (org->has_parameter());
+      CGAL_assertion(org->has_parameter());
 
       const Algebraic&  t0 = org->parameter();
-      Polynomial        polyY_der = nt_traits.derive (_curve.y_polynomial());
+      Polynomial        polyY_der = nt_traits.derive(_curve.y_polynomial());
       const CGAL::Sign  sign_der =
-        CGAL::sign (nt_traits.evaluate_at (polyY_der, t0));
+        CGAL::sign(nt_traits.evaluate_at(polyY_der, t0));
 
-      CGAL_assertion (sign_der != CGAL::ZERO);
+      CGAL_assertion(sign_der != CGAL::ZERO);
 
       if (_inc_to_right)
         return ((sign_der == CGAL::NEGATIVE) ? LARGER : SMALLER);
@@ -1290,7 +1186,7 @@ _Bezier_x_monotone_2<RatKer, AlgKer, NtT
   // case, the slopes are not equal and their comparison gives us the
   // vertical order to p's right; note that we swap the order of the curves
   // to obtains their position to the left.
-  Comparison_result   slope_res = cv._compare_slopes (*this, p, cache);
+  Comparison_result   slope_res = cv._compare_slopes(*this, p, cache);
 
   if (slope_res != EQUAL)
     return (slope_res);
@@ -1299,18 +1195,18 @@ _Bezier_x_monotone_2<RatKer, AlgKer, NtT
   // and compareing the vertical position there.
   Comparison_result   left_res;
 
-  if (left().compare_x (cv.left(), cache) != SMALLER)
+  if (left().compare_x(cv.left(), cache) != SMALLER)
   {
-    left_res = _compare_to_side (cv, p,
+    left_res = _compare_to_side(cv, p,
                                  false,          // Compare to p's left.
                                  cache);
   }
   else
   {
-    left_res = cv._compare_to_side (*this, p,
+    left_res = cv._compare_to_side(*this, p,
                                     false,       // Compare to p's left.
                                     cache);
-    left_res = CGAL::opposite (left_res);
+    left_res = CGAL::opposite(left_res);
   }
 
   return (left_res);
@@ -1325,7 +1221,7 @@ bool _Bezier_x_monotone_2<RatKer, AlgKer
          Bezier_cache& cache) const
 {
   // Check if the two subcurves have overlapping supporting curves.
-  if (! _curve.is_same (cv._curve))
+  if (! _curve.is_same(cv._curve))
   {
     //special case when curves are vertical
     if (cv.is_vertical()){
@@ -1335,7 +1231,7 @@ bool _Bezier_x_monotone_2<RatKer, AlgKer
     }
 
     // Check whether the two curves have the same support:
-    if (! _curve.has_same_support (cv._curve))
+    if (! _curve.has_same_support(cv._curve))
       return (false);
 
     // Mark that the two curves overlap in the cache.
@@ -1343,14 +1239,14 @@ bool _Bezier_x_monotone_2<RatKer, AlgKer
     const Curve_id               id2 = cv._curve.id();
 
     if (id1 < id2)
-      cache.mark_as_overlapping (id1, id2);
+      cache.mark_as_overlapping(id1, id2);
     else
-      cache.mark_as_overlapping (id2, id1);
+      cache.mark_as_overlapping(id2, id1);
   }
 
   // Check for equality of the endpoints.
-  return (left().equals (cv.left(), cache) &&
-          right().equals (cv.right(), cache));
+  return (left().equals(cv.left(), cache) &&
+          right().equals(cv.right(), cache));
 }
 
 // ---------------------------------------------------------------------------
@@ -1406,9 +1302,9 @@ bool _Bezier_x_monotone_2<RatKer, AlgKer
 {
   // Note that we only allow merging subcurves of the same originating
   // Bezier curve (overlapping curves will not do in this case).
-  return (_curve.is_same (cv._curve) &&
+  return (_curve.is_same(cv._curve) &&
           _xid == cv._xid &&
-          (right().is_same (cv.left()) || left().is_same (cv.right())));
+          (right().is_same(cv.left()) || left().is_same(cv.right())));
 
   return (false);
 }
@@ -1421,28 +1317,22 @@ typename _Bezier_x_monotone_2<RatKer, Al
 _Bezier_x_monotone_2<RatKer, AlgKer, NtTrt, BndTrt>::merge
         (const Self& cv) const
 {
-  CGAL_precondition (_curve.is_same (cv._curve));
-  CGAL_precondition (_xid == cv._xid);
+  CGAL_precondition(_curve.is_same(cv._curve));
+  CGAL_precondition(_xid == cv._xid);
 
-  Self    res = *this;
+  Self res = *this;
 
-  if (right().is_same (cv.left()))
-  {
+  if (right().is_same(cv.left())) {
     // Extend the subcurve to the right.
-    if (_dir_right)
-      res._pt = cv.right();
-    else
-      res._ps = cv.right();
+    if (_dir_right) res._pt = cv.right();
+    else res._ps = cv.right();
   }
-  else
-  {
-    CGAL_precondition (left().is_same (cv.right()));
+  else {
+    CGAL_precondition(left().is_same(cv.right()));
 
     // Extend the subcurve to the left.
-    if (_dir_right)
-      res._ps = cv.left();
-    else
-      res._pt = cv.left();
+    if (_dir_right) res._ps = cv.left();
+    else res._pt = cv.left();
   }
 
   return (res);
@@ -1451,42 +1341,35 @@ _Bezier_x_monotone_2<RatKer, AlgKer, NtT
 // ---------------------------------------------------------------------------
 // Check if the given t-value is in the range of the subcurve.
 //
-template <class RatKer, class AlgKer, class NtTrt, class BndTrt>
-bool _Bezier_x_monotone_2<RatKer, AlgKer, NtTrt, BndTrt>::_is_in_range
-        (const Algebraic& t,
-         Bezier_cache& cache) const
+template <typename RatKer, typename AlgKer, typename NtTrt, typename BndTrt>
+bool _Bezier_x_monotone_2<RatKer, AlgKer, NtTrt, BndTrt>::
+_is_in_range(const Algebraic& t, Bezier_cache& cache) const
 {
   // First try to use the approximate representation of the endpoints.
-  Originator_iterator  s_org = _ps.get_originator (_curve, _xid);
-  CGAL_assertion (s_org != _ps.originators_end());
+  Originator_iterator s_org = _ps.get_originator(_curve, _xid);
+  CGAL_assertion(s_org != _ps.originators_end());
 
-  Originator_iterator  t_org = _pt.get_originator (_curve, _xid);
-  CGAL_assertion (t_org != _pt.originators_end());
+  Originator_iterator t_org = _pt.get_originator (_curve, _xid);
+  CGAL_assertion(t_org != _pt.originators_end());
 
-  Nt_traits            nt_traits;
+  Nt_traits nt_traits;
 
-  bool  p_lt_ps =
-      (CGAL::compare (t, nt_traits.convert (s_org->point_bound().t_min)) ==
-       SMALLER);
-  bool  p_gt_ps =
-      (CGAL::compare (t, nt_traits.convert (s_org->point_bound().t_max)) ==
-       LARGER);
-  bool  p_lt_pt =
-      (CGAL::compare (t, nt_traits.convert (t_org->point_bound().t_min)) ==
-       SMALLER);
-  bool  p_gt_pt =
-      (CGAL::compare (t, nt_traits.convert (t_org->point_bound().t_max)) ==
-       LARGER);
+  bool p_lt_ps =
+    CGAL::compare(t, nt_traits.convert(s_org->point_bound().t_min)) == SMALLER;
+  bool p_gt_ps =
+    CGAL::compare(t, nt_traits.convert(s_org->point_bound().t_max)) == LARGER;
+  bool p_lt_pt =
+    CGAL::compare(t, nt_traits.convert(t_org->point_bound().t_min)) == SMALLER;
+  bool p_gt_pt =
+    CGAL::compare(t, nt_traits.convert(t_org->point_bound().t_max)) == LARGER;
 
-  if ((p_gt_ps && p_lt_pt) || (p_lt_ps && p_gt_pt))
-  {
+  if ((p_gt_ps && p_lt_pt) || (p_lt_ps && p_gt_pt)) {
     // The point p is definitely in the x-range of the subcurve, as its
     // parameter is between the source and target parameters.
-    return (true);
+    return true;
   }
 
-  if ((p_lt_ps && p_lt_pt) || (p_gt_ps && p_gt_pt))
-  {
+  if ((p_lt_ps && p_lt_pt) || (p_gt_ps && p_gt_pt)) {
     // The point p is definitely not in the x-range of the subcurve,
     // as its parameter is smaller than both source and target parameter
     // (or greater than both of them).
@@ -1507,60 +1390,54 @@ bool _Bezier_x_monotone_2<RatKer, AlgKer
 // ---------------------------------------------------------------------------
 // Check if the given point lies in the range of this x-monotone subcurve.
 //
-template <class RatKer, class AlgKer, class NtTrt, class BndTrt>
-bool _Bezier_x_monotone_2<RatKer, AlgKer, NtTrt, BndTrt>::_is_in_range
-        (const Point_2& p,
-         bool& is_certain) const
+template <typename RatKer, typename AlgKer, typename NtTrt, typename BndTrt>
+bool _Bezier_x_monotone_2<RatKer, AlgKer, NtTrt, BndTrt>::
+_is_in_range(const Point_2& p, bool& is_certain) const
 {
   is_certain = true;
 
   // Check the easy case that p is one of the subcurve endpoints.
-  if (p.is_same(_ps) || p.is_same(_pt))
-    return true;
+  if (p.is_same(_ps) || p.is_same(_pt)) return true;
 
   // Compare the parameter of p with the parameters of the endpoints.
-  Originator_iterator  p_org = p.get_originator (_curve, _xid);
+  Originator_iterator p_org = p.get_originator(_curve, _xid);
 
-  if (p_org == p.originators_end())
-  {
-    CGAL_assertion (p.get_originator (_curve) != p.originators_end());
+  if (p_org == p.originators_end()) {
+    CGAL_assertion(p.get_originator(_curve) != p.originators_end());
 
     // In this case a different x-monotone curve of the supporting Bezier
     // curve is an originator of the point, so we know that p does not
     // lie in the range of our x-monotone subcurve.
-    return (false);
+    return false;
   }
 
-  Originator_iterator  s_org = _ps.get_originator (_curve, _xid);
-  CGAL_assertion (s_org != _ps.originators_end());
+  Originator_iterator s_org = _ps.get_originator(_curve, _xid);
+  CGAL_assertion(s_org != _ps.originators_end());
 
-  Originator_iterator  t_org = _pt.get_originator (_curve, _xid);
-  CGAL_assertion (t_org != _pt.originators_end());
+  Originator_iterator t_org = _pt.get_originator(_curve, _xid);
+  CGAL_assertion(t_org != _pt.originators_end());
 
-  bool      can_refine_p = ! p.is_exact();
-  bool      can_refine_s = ! _ps.is_exact();
-  bool      can_refine_t = ! _pt.is_exact();
-
-  while (can_refine_p || can_refine_s || can_refine_t)
-  {
-    bool  p_lt_ps = (CGAL::compare (p_org->point_bound().t_max,
-                                    s_org->point_bound().t_min) == SMALLER);
-    bool  p_gt_ps = (CGAL::compare (p_org->point_bound().t_min,
-                                    s_org->point_bound().t_max) == LARGER);
-    bool  p_lt_pt = (CGAL::compare (p_org->point_bound().t_max,
-                                    t_org->point_bound().t_min) == SMALLER);
-    bool  p_gt_pt = (CGAL::compare (p_org->point_bound().t_min,
-                                    t_org->point_bound().t_max) == LARGER);
+  bool can_refine_p = ! p.is_exact();
+  bool can_refine_s = ! _ps.is_exact();
+  bool can_refine_t = ! _pt.is_exact();
+
+  while (can_refine_p || can_refine_s || can_refine_t) {
+    bool p_lt_ps = (CGAL::compare(p_org->point_bound().t_max,
+                                  s_org->point_bound().t_min) == SMALLER);
+    bool p_gt_ps = (CGAL::compare(p_org->point_bound().t_min,
+                                  s_org->point_bound().t_max) == LARGER);
+    bool p_lt_pt = (CGAL::compare(p_org->point_bound().t_max,
+                                  t_org->point_bound().t_min) == SMALLER);
+    bool p_gt_pt = (CGAL::compare(p_org->point_bound().t_min,
+                                  t_org->point_bound().t_max) == LARGER);
 
-    if ((p_gt_ps && p_lt_pt) || (p_lt_ps && p_gt_pt))
-    {
+    if ((p_gt_ps && p_lt_pt) || (p_lt_ps && p_gt_pt)) {
       // The point p is definitely in the x-range of the subcurve, as its
       // parameter is between the source and target parameters.
       return (true);
     }
 
-    if ((p_lt_ps && p_lt_pt) || (p_gt_ps && p_gt_pt))
-    {
+    if ((p_lt_ps && p_lt_pt) || (p_gt_ps && p_gt_pt)) {
       // The point p is definitely not in the x-range of the subcurve,
       // as its parameter is smaller than both source and target parameter
       // (or greater than both of them).
@@ -1568,19 +1445,14 @@ bool _Bezier_x_monotone_2<RatKer, AlgKer
     }
 
     // Try to refine the points.
-    if (can_refine_p)
-      can_refine_p = p.refine();
-
-    if (can_refine_s)
-      can_refine_s = _ps.refine();
-
-    if (can_refine_t)
-      can_refine_t = _pt.refine();
+    if (can_refine_p) can_refine_p = p.refine();
+    if (can_refine_s) can_refine_s = _ps.refine();
+    if (can_refine_t) can_refine_t = _pt.refine();
   }
 
   // If we reached here, we do not have a certain answer.
   is_certain = false;
-  return (false);
+  return false;
 }
 
 // ---------------------------------------------------------------------------
@@ -1588,35 +1460,32 @@ bool _Bezier_x_monotone_2<RatKer, AlgKer
 // determine whether p lies within the t-range of the x-monotone subcurve.
 // If so, the value t0 such that p = (X(t0), Y(t0)) is also computed.
 //
-template <class RatKer, class AlgKer, class NtTrt, class BndTrt>
-bool _Bezier_x_monotone_2<RatKer, AlgKer, NtTrt, BndTrt>::_is_in_range
-        (const Point_2& p,
-         Bezier_cache& cache,
-         Algebraic& t0,
-         bool& is_endpoint) const
+template <typename RatKer, typename AlgKer, typename NtTrt, typename BndTrt>
+bool _Bezier_x_monotone_2<RatKer, AlgKer, NtTrt, BndTrt>::
+_is_in_range(const Point_2& p,
+             Bezier_cache& cache,
+             Algebraic& t0,
+             bool& is_endpoint) const
 {
   // The given point p must be rational, otherwise there is no point checking
   // whether it lies in the interior of the curve.
-  if (! p.is_rational())
-  {
+  if (! p.is_rational()) {
     is_endpoint = false;
-    return (false);
+    return false;
   }
 
-  const Rat_point_2&    rat_p = (Rat_point_2) p;
+  const Rat_point_2& rat_p = (Rat_point_2) p;
 
   // Determine the parameter range [t_min, t_max] for our x-monotone
   // subcurve.
-  std::pair<Algebraic, Algebraic>          t_range = _t_range (cache);
-  Algebraic                                t_min, t_max;
+  std::pair<Algebraic, Algebraic> t_range = _t_range (cache);
+  Algebraic t_min, t_max;
 
-  if ((_dir_right && _inc_to_right) || (! _dir_right && ! _inc_to_right))
-  {
+  if ((_dir_right && _inc_to_right) || (! _dir_right && ! _inc_to_right)) {
     t_min = t_range.first;
     t_max = t_range.second;
   }
-  else
-  {
+  else {
     t_min = t_range.second;
     t_max = t_range.first;
   }
@@ -1624,90 +1493,78 @@ bool _Bezier_x_monotone_2<RatKer, AlgKer
   // The given point p must lie on (X(t), Y(t)) for some t-value. Obtain the
   // parameter value t0 for that point. We start by computing all t-values
   // such that X(t) equals the x-coordinate of p.
-  Nt_traits                                nt_traits;
-  std::list<Algebraic>                     t_vals;
-  typename std::list<Algebraic>::iterator  t_iter;
-  Comparison_result                        res1, res2;
-  Algebraic                                y0;
-
-  if ( is_vertical() ){
-    if ( compare(rat_p.x(),left().x())==EQUAL ){
-      _curve.get_t_at_y (rat_p.y(), std::back_inserter(t_vals));
+  Nt_traits nt_traits;
+  std::list<Algebraic> t_vals;
+  typename std::list<Algebraic>::iterator t_iter;
+  Comparison_result res1, res2;
+  Algebraic y0;
+
+  if (is_vertical()){
+    if ( compare(rat_p.x(),left().x()) == EQUAL) {
+      _curve.get_t_at_y(rat_p.y(), std::back_inserter(t_vals));
 
-      for (t_iter = t_vals.begin(); t_iter != t_vals.end(); ++t_iter)
-      {
+      for (t_iter = t_vals.begin(); t_iter != t_vals.end(); ++t_iter) {
         // Compare the current t-value with t_min.
-        res1 = CGAL::compare (t_min, *t_iter);
-
-        if (res1 == LARGER)
-          continue;
+        res1 = CGAL::compare(t_min, *t_iter);
 
+        if (res1 == LARGER) continue;
 
-        if (res1 == EQUAL)
-        {
+        if (res1 == EQUAL) {
           t0 = t_min;
           is_endpoint = true;
           return (true);
         }
 
         // Compare the current t-value with t_max.
-        res2 = CGAL::compare (t_max, *t_iter);
+        res2 = CGAL::compare(t_max, *t_iter);
 
-        if (res2 == EQUAL)
-        {
+        if (res2 == EQUAL) {
           t0 = t_max;
           is_endpoint = true;
-          return (true);
+          return true;
         }
 
-        if (res2 == LARGER)
-        {
+        if (res2 == LARGER) {
           t0 = *t_iter;
           is_endpoint = false;
-          return (true);
+          return true;
         }
       }
     }
     is_endpoint = false;
-    return (false);
+    return false;
   }
 
-  _curve.get_t_at_x (rat_p.x(), std::back_inserter(t_vals));
-  CGAL_assertion (! t_vals.empty() );
+  _curve.get_t_at_x(rat_p.x(), std::back_inserter(t_vals));
+  CGAL_assertion(! t_vals.empty() );
 
-  for (t_iter = t_vals.begin(); t_iter != t_vals.end(); ++t_iter)
-  {
+  for (t_iter = t_vals.begin(); t_iter != t_vals.end(); ++t_iter) {
     // Compare the current t-value with t_min.
     res1 = CGAL::compare (t_min, *t_iter);
 
-    if (res1 == LARGER)
-      continue;
+    if (res1 == LARGER) continue;
 
     // Make sure the y-coordinates match.
-    y0 = nt_traits.evaluate_at (_curve.y_polynomial(), *t_iter) /
-         nt_traits.convert (_curve.y_norm());
+    y0 = nt_traits.evaluate_at(_curve.y_polynomial(), *t_iter) /
+         nt_traits.convert(_curve.y_norm());
 
-    if (CGAL::compare (nt_traits.convert (rat_p.y()), y0) == EQUAL)
-    {
-      if (res1 == EQUAL)
-      {
+    if (CGAL::compare(nt_traits.convert(rat_p.y()), y0) == EQUAL) {
+      if (res1 == EQUAL) {
         t0 = t_min;
         is_endpoint = true;
-        return (true);
+        return true;
       }
 
       // Compare the current t-value with t_max.
       res2 = CGAL::compare (t_max, *t_iter);
 
-      if (res2 == EQUAL)
-      {
+      if (res2 == EQUAL) {
         t0 = t_max;
         is_endpoint = true;
         return (true);
       }
 
-      if (res2 == LARGER)
-      {
+      if (res2 == LARGER) {
         t0 = *t_iter;
         is_endpoint = false;
         return (true);
@@ -1718,53 +1575,48 @@ bool _Bezier_x_monotone_2<RatKer, AlgKer
   // In this case, we have not found a t-value in the range of our subcurve,
   // so p does not lie on the subcurve:
   is_endpoint = false;
-  return (false);
+  return false;
 }
 
 // ---------------------------------------------------------------------------
 // Compute a y-coordinate of a point on the x-monotone subcurve with a
 // given x-coordinate.
 //
-template <class RatKer, class AlgKer, class NtTrt, class BndTrt>
+template <typename RatKer, typename AlgKer, typename NtTrt,typename BndTrt>
 typename _Bezier_x_monotone_2<RatKer, AlgKer, NtTrt, BndTrt>::Algebraic
-_Bezier_x_monotone_2<RatKer, AlgKer, NtTrt, BndTrt>::_get_y
-        (const Rational& x0,
-         Bezier_cache& cache) const
+_Bezier_x_monotone_2<RatKer, AlgKer, NtTrt, BndTrt>::
+_get_y(const Rational& x0, Bezier_cache& cache) const
 {
   // Obtain the t-values for with the x-coordinates of the supporting
   // curve equal x0.
-  std::list<Algebraic>  t_vals;
+  std::list<Algebraic> t_vals;
 
-  _curve.get_t_at_x (x0, std::back_inserter(t_vals));
+  _curve.get_t_at_x(x0, std::back_inserter(t_vals));
 
   // Find a t-value that is in the range of the current curve.
-  Nt_traits                                nt_traits;
-  typename std::list<Algebraic>::iterator  t_iter;
-  std::pair<Algebraic, Algebraic>          t_range = _t_range (cache);
-  const Algebraic&                         t_src = t_range.first;
-  const Algebraic&                         t_trg = t_range.second;
-  Comparison_result                        res1, res2;
+  Nt_traits nt_traits;
+  typename std::list<Algebraic>::iterator t_iter;
+  std::pair<Algebraic, Algebraic> t_range = _t_range (cache);
+  const Algebraic& t_src = t_range.first;
+  const Algebraic& t_trg = t_range.second;
+  Comparison_result res1, res2;
 
-  for (t_iter = t_vals.begin(); t_iter != t_vals.end(); ++t_iter)
-  {
+  for (t_iter = t_vals.begin(); t_iter != t_vals.end(); ++t_iter) {
     res1 = CGAL::compare (*t_iter, t_src);
 
-    if (res1 == EQUAL)
-    {
+    if (res1 == EQUAL) {
       // Return the y-coordinate of the source point:
       return (_ps.y());
     }
 
     res2 = CGAL::compare (*t_iter, t_trg);
 
-    if (res2 == EQUAL)
-    {
+    if (res2 == EQUAL) {
       // Return the y-coordinate of the source point:
       return (_pt.y());
     }
 
-    if (res1 != res2)
-    {
+    if (res1 != res2) {
       // We found a t-value in the range of our x-monotone subcurve.
       // Use this value to compute the y-coordinate.
       return (nt_traits.evaluate_at (_curve.y_polynomial(), *t_iter) /
@@ -1774,42 +1626,38 @@ _Bezier_x_monotone_2<RatKer, AlgKer, NtT
 
   // If we reached here, x0 is not in the x-range of our subcurve.
   CGAL_error();
-  return (0);
+  return 0;
 }
 
 // ---------------------------------------------------------------------------
 // Compare the slopes of the subcurve with another given Bezier subcurve at
 // their given intersection point.
 //
-template <class RatKer, class AlgKer, class NtTrt, class BndTrt>
+template <typename RatKer, typename AlgKer,typename NtTrt, typename BndTrt>
 Comparison_result
-_Bezier_x_monotone_2<RatKer, AlgKer, NtTrt, BndTrt>::_compare_slopes
-        (const Self& cv,
-         const Point_2& p,
-         Bezier_cache& cache) const
+_Bezier_x_monotone_2<RatKer, AlgKer, NtTrt, BndTrt>::
+_compare_slopes(const Self& cv, const Point_2& p, Bezier_cache& cache) const
 {
   // Get the originators of p.
-  Originator_iterator     org1 = p.get_originator (_curve, _xid);
-  const bool              valid_org1 = (org1 != p.originators_end());
+  Originator_iterator org1 = p.get_originator(_curve, _xid);
+  const bool valid_org1 = (org1 != p.originators_end());
 
-  Originator_iterator     org2 = p.get_originator (cv._curve, cv._xid);
-  const bool              valid_org2 = (org2 != p.originators_end());
+  Originator_iterator org2 = p.get_originator(cv._curve, cv._xid);
+  const bool valid_org2 = (org2 != p.originators_end());
 
-  CGAL_assertion (valid_org1 || valid_org2);
+  CGAL_assertion(valid_org1 || valid_org2);
 
   // If the point is only approximated, we can carry out a comparison using
   // an approximate number type.
-  if (valid_org1 && valid_org2 && ! p.is_exact())
-  {
+  if (valid_org1 && valid_org2 && ! p.is_exact()) {
     // If the point is inexact, we assume it is a bounded intersection
     // point of two curves, and therefore the bounding angle these curves
     // span do not overlap.
-    const Bez_point_bound&  bound1 = org1->point_bound();
-    const Bez_point_bound&  bound2 = org2->point_bound();
-    Bounding_traits         bound_tr;
+    const Bez_point_bound& bound1 = org1->point_bound();
+    const Bez_point_bound& bound2 = org2->point_bound();
+    Bounding_traits bound_tr;
 
-    return (bound_tr.compare_slopes_at_intersection_point (bound1,
-                                                           bound2));
+    return (bound_tr.compare_slopes_at_intersection_point(bound1, bound2));
   }
 
   // Obtain the parameter values t1 and t2 that correspond to the point p.
@@ -1818,17 +1666,15 @@ _Bezier_x_monotone_2<RatKer, AlgKer, NtT
   // it must be a ratioal point!) and lies in its interior. In this
   // (degenerate) case we compute the parameter value and set the appropriate
   // originator for p.
-  Nt_traits         nt_traits;
-  Algebraic         t1;
-  Algebraic         t2;
+  Nt_traits nt_traits;
+  Algebraic t1;
+  Algebraic t2;
 
-  if (valid_org1)
-  {
+  if (valid_org1) {
     CGAL_assertion (org1->has_parameter());
     t1 = org1->parameter();
   }
-  else
-  {
+  else {
     bool    is_endpoint1;
     CGAL_assertion_code (bool   in_range1 =)
     _is_in_range (p, cache, t1, is_endpoint1);
@@ -1836,66 +1682,60 @@ _Bezier_x_monotone_2<RatKer, AlgKer, NtT
     p.add_originator (Originator (_curve, _xid, t1));
   }
 
-  if (valid_org2)
-  {
+  if (valid_org2) {
     CGAL_assertion (org2->has_parameter());
     t2 = org2->parameter();
   }
-  else
-  {
-    bool    is_endpoint2;
-    CGAL_assertion_code (bool   in_range2 =)
-    cv._is_in_range (p, cache, t2, is_endpoint2);
-    CGAL_assertion (in_range2);
-    p.add_originator (Originator (cv._curve, cv._xid, t2));
+  else {
+    bool is_endpoint2;
+    CGAL_assertion_code(bool in_range2 =)
+    cv._is_in_range(p, cache, t2, is_endpoint2);
+    CGAL_assertion(in_range2);
+    p.add_originator(Originator (cv._curve, cv._xid, t2));
   }
 
   // The slope of (X(t), Y(t)) at t0 is given by Y'(t0)/X'(t0).
   // Compute the slope of (*this).
   // Note that we take special care of the case X'(t0) = 0, when the tangent
   // is vertical and its slope is +/- oo.
-  Polynomial        derivX = nt_traits.derive (_curve.x_polynomial());
-  Polynomial        derivY = nt_traits.derive (_curve.y_polynomial());
-  Algebraic         numer1 = nt_traits.evaluate_at (derivY, t1) *
-                             nt_traits.convert (_curve.x_norm());
-  Algebraic         denom1 = nt_traits.evaluate_at (derivX, t1) *
-                             nt_traits.convert (_curve.y_norm());
-  CGAL::Sign        inf_slope1 = CGAL::ZERO;
-  Algebraic         slope1;
-
-  if (CGAL::sign (denom1) == CGAL::ZERO)
-  {
-    inf_slope1 = is_directed_right() ? CGAL::sign (numer1) : CGAL::opposite( CGAL::sign (numer1) );
+  Polynomial derivX = nt_traits.derive(_curve.x_polynomial());
+  Polynomial derivY = nt_traits.derive(_curve.y_polynomial());
+  Algebraic numer1 = nt_traits.evaluate_at(derivY, t1) *
+    nt_traits.convert (_curve.x_norm());
+  Algebraic denom1 = nt_traits.evaluate_at(derivX, t1) *
+    nt_traits.convert (_curve.y_norm());
+  CGAL::Sign inf_slope1 = CGAL::ZERO;
+  Algebraic slope1;
+
+  if (CGAL::sign (denom1) == CGAL::ZERO) {
+    inf_slope1 = is_directed_right() ?
+      CGAL::sign(numer1) : CGAL::opposite(CGAL::sign(numer1));
 
     // If both derivatives are zero, we cannot perform the comparison:
-    if (inf_slope1 == CGAL::ZERO)
-      return (EQUAL);
+    if (inf_slope1 == CGAL::ZERO) return EQUAL;
   }
-  else
-  {
+  else {
     slope1 = numer1 / denom1;
   }
 
   // Compute the slope of the other subcurve.
   derivX = nt_traits.derive (cv._curve.x_polynomial());
   derivY = nt_traits.derive (cv._curve.y_polynomial());
-  Algebraic         numer2 = nt_traits.evaluate_at (derivY, t2) *
-                             nt_traits.convert (cv._curve.x_norm());
-  Algebraic         denom2 = nt_traits.evaluate_at (derivX, t2) *
-                             nt_traits.convert (cv._curve.y_norm());
-  CGAL::Sign        inf_slope2 = CGAL::ZERO;
-  Algebraic         slope2;
-
-  if (CGAL::sign (denom2) == CGAL::ZERO)
-  {
-    inf_slope2 = cv.is_directed_right() ? CGAL::sign (numer2) : CGAL::opposite( CGAL::sign (numer2) );
+  Algebraic numer2 = nt_traits.evaluate_at (derivY, t2) *
+    nt_traits.convert (cv._curve.x_norm());
+  Algebraic denom2 = nt_traits.evaluate_at (derivX, t2) *
+    nt_traits.convert (cv._curve.y_norm());
+  CGAL::Sign inf_slope2 = CGAL::ZERO;
+  Algebraic slope2;
+
+  if (CGAL::sign (denom2) == CGAL::ZERO) {
+    inf_slope2 = cv.is_directed_right() ?
+      CGAL::sign (numer2) : CGAL::opposite( CGAL::sign (numer2) );
 
     // If both derivatives are zero, we cannot perform the comparison:
-    if (inf_slope2 == CGAL::ZERO)
-      return (EQUAL);
+    if (inf_slope2 == CGAL::ZERO) return (EQUAL);
   }
-  else
-  {
+  else {
     slope2 = numer2 / denom2;
   }
 
@@ -1906,11 +1746,9 @@ _Bezier_x_monotone_2<RatKer, AlgKer, NtT
   if (inf_slope1 == CGAL::NEGATIVE)
     return (inf_slope2 == CGAL::NEGATIVE ? EQUAL : SMALLER);
 
-  if (inf_slope2 == CGAL::POSITIVE)
-    return (SMALLER);
+  if (inf_slope2 == CGAL::POSITIVE) return (SMALLER);
 
-  if (inf_slope2 == CGAL::NEGATIVE)
-    return (LARGER);
+  if (inf_slope2 == CGAL::NEGATIVE) return (LARGER);
 
   // Compare the slopes.
   return (CGAL::compare (slope1, slope2));
@@ -1919,29 +1757,25 @@ _Bezier_x_monotone_2<RatKer, AlgKer, NtT
 // ---------------------------------------------------------------------------
 // Get the range of t-value over which the subcurve is defined.
 //
-template <class RatKer, class AlgKer, class NtTrt, class BndTrt>
+template <typename RatKer, typename AlgKer, typename NtTrt, typename BndTrt>
 std::pair<typename _Bezier_x_monotone_2<RatKer, AlgKer, NtTrt,
                                         BndTrt>::Algebraic,
           typename _Bezier_x_monotone_2<RatKer, AlgKer, NtTrt,
                                         BndTrt>::Algebraic>
-_Bezier_x_monotone_2<RatKer, AlgKer, NtTrt, BndTrt>::_t_range
-        (Bezier_cache& cache) const
+_Bezier_x_monotone_2<RatKer, AlgKer, NtTrt, BndTrt>::
+_t_range(Bezier_cache& cache) const
 {
-  Originator_iterator  ps_org = _ps.get_originator (_curve, _xid);
+  Originator_iterator ps_org = _ps.get_originator(_curve, _xid);
   CGAL_assertion(ps_org != _ps.originators_end());
 
-  Originator_iterator  pt_org = _pt.get_originator (_curve, _xid);
+  Originator_iterator pt_org = _pt.get_originator(_curve, _xid);
   CGAL_assertion(pt_org != _pt.originators_end());
 
   // Make sure that the two endpoints are exact.
-  if (! ps_org->has_parameter())
-    _ps.make_exact (cache);
+  if (! ps_org->has_parameter()) _ps.make_exact (cache);
+  if (! pt_org->has_parameter()) _pt.make_exact (cache);
 
-  if (! pt_org->has_parameter())
-    _pt.make_exact (cache);
-
-  return (std::make_pair (ps_org->parameter(),
-                          pt_org->parameter()));
+  return (std::make_pair(ps_org->parameter(), pt_org->parameter()));
 }
 
 // ---------------------------------------------------------------------------
@@ -1949,18 +1783,18 @@ _Bezier_x_monotone_2<RatKer, AlgKer, NtT
 // (or to the left) of their intersection point, whose multiplicity is
 // greater than 1.
 //
-template <class RatKer, class AlgKer, class NtTrt, class BndTrt>
+template <typename RatKer, typename AlgKer, typename NtTrt, typename BndTrt>
 Comparison_result
-_Bezier_x_monotone_2<RatKer, AlgKer, NtTrt, BndTrt>::_compare_to_side
-        (const Self& cv,
-         const Point_2& p,
-         bool to_right,
-         Bezier_cache& cache) const
+_Bezier_x_monotone_2<RatKer, AlgKer, NtTrt, BndTrt>::
+_compare_to_side(const Self& cv,
+                 const Point_2& p,
+                 bool to_right,
+                 Bezier_cache& cache) const
 {
   // Get the intersection points of the two curves from the cache. Note that
   // we make sure that the ID of this->_curve is smaller than of cv's curve ID.
-  const bool             no_swap_curves = (_curve.id() <= cv._curve.id());
-  bool                   do_ovlp;
+  const bool no_swap_curves = (_curve.id() <= cv._curve.id());
+  bool do_ovlp;
   const Intersect_list&  inter_list =
     (no_swap_curves ?
      (cache.get_intersections (_curve.id(),
@@ -1984,40 +1818,36 @@ _Bezier_x_monotone_2<RatKer, AlgKer, NtT
   CGAL_assertion (org != p.originators_end());
   CGAL_assertion (org->has_parameter());
 
-  const Algebraic&             t0 = org->parameter();
+  const Algebraic& t0 = org->parameter();
 
   // Get the parameter range of the curve.
-  const std::pair<Algebraic,
-                  Algebraic>&  range = _t_range (cache);
-  const Algebraic&             t_src = range.first;
-  const Algebraic&             t_trg = range.second;
+  const std::pair<Algebraic, Algebraic>& range = _t_range (cache);
+  const Algebraic& t_src = range.first;
+  const Algebraic& t_trg = range.second;
 
   // Find the next intersection point that lies to the right of p.
-  Intersect_iter               iit;
-  Algebraic                    next_t;
-  Comparison_result            res = CGAL::EQUAL;
-  bool                         found = false;
+  Intersect_iter iit;
+  Algebraic next_t;
+  Comparison_result res = CGAL::EQUAL;
+  bool found = false;
 
-  for (iit = inter_list.begin(); iit != inter_list.end(); ++iit)
-  {
+  for (iit = inter_list.begin(); iit != inter_list.end(); ++iit) {
     // Check if the current point lies to the right (left) of p. We do so by
     // considering its originating parameter value s (or t, if we swapped
     // the curves).
-    const Algebraic&     t = (no_swap_curves ? (iit->s) : iit->t);
+    const Algebraic& t = (no_swap_curves ? (iit->s) : iit->t);
 
-    res = CGAL::compare (t, t0);
+    res = CGAL::compare(t, t0);
     if ((to_right && ((_inc_to_right && res == LARGER) ||
                       (! _inc_to_right && res == SMALLER))) ||
         (! to_right && ((_inc_to_right && res == SMALLER) ||
                         (! _inc_to_right && res == LARGER))))
     {
-      if (! found)
-      {
+      if (! found) {
         next_t = t;
         found = true;
       }
-      else
-      {
+      else {
         // If we have already located an intersection point to the right
         // (left) of p, choose the leftmost (rightmost) of the two points.
         res = CGAL::compare (t, next_t);
@@ -2035,12 +1865,9 @@ _Bezier_x_monotone_2<RatKer, AlgKer, NtT
   // If the next intersection point occurs before the right (left) endpoint
   // of the subcurve, keep it. Otherwise, take the parameter value at
   // the endpoint.
-  if (found)
-  {
-    if (to_right == _dir_right)
-      res = CGAL::compare (t_trg, next_t);
-    else
-      res = CGAL::compare (t_src, next_t);
+  if (found) {
+    if (to_right == _dir_right) res = CGAL::compare (t_trg, next_t);
+    else res = CGAL::compare (t_src, next_t);
   }
 
   if (! found ||
@@ -2055,10 +1882,10 @@ _Bezier_x_monotone_2<RatKer, AlgKer, NtT
   // Find a rational value between t0 and t_next. Using this value, we
   // a point with rational coordinates on our subcurve. We also locate a point
   // on the other curve with the same x-coordinates.
-  Nt_traits           nt_traits;
-  const Rational&     mid_t = nt_traits.rational_in_interval (t0, next_t);
-  const Rat_point_2&  q1 = _curve (mid_t);
-  const Algebraic&    y2 = cv._get_y (q1.x(), cache);
+  Nt_traits nt_traits;
+  const Rational& mid_t = nt_traits.rational_in_interval(t0, next_t);
+  const Rat_point_2& q1 = _curve(mid_t);
+  const Algebraic& y2 = cv._get_y(q1.x(), cache);
 
   // We now just have to compare the y-coordinates of the two points we have
   // computed.
@@ -2069,33 +1896,32 @@ _Bezier_x_monotone_2<RatKer, AlgKer, NtT
 // Clip the control polygon of the supporting Bezier curve such that it fits
 // the current x-monotone subcurve.
 //
-template <class RatKer, class AlgKer, class NtTrt, class BndTrt>
-void _Bezier_x_monotone_2<RatKer, AlgKer, NtTrt,
-                          BndTrt>::_clip_control_polygon
-        (typename Bounding_traits::Control_points& ctrl,
-         typename Bounding_traits::NT& t_min,
-         typename Bounding_traits::NT& t_max) const
+template <typename RatKer, typename AlgKer, typename NtTrt, typename BndTrt>
+void _Bezier_x_monotone_2<RatKer, AlgKer, NtTrt, BndTrt>::
+_clip_control_polygon(typename Bounding_traits::Control_points& ctrl,
+                      typename Bounding_traits::NT& t_min,
+                      typename Bounding_traits::NT& t_max) const
 {
   // Start from the control polygon of the supporting curve.
   ctrl.clear();
-  std::copy (_curve.control_points_begin(), _curve.control_points_end(),
-             std::back_inserter (ctrl));
+  std::copy(_curve.control_points_begin(), _curve.control_points_end(),
+            std::back_inserter(ctrl));
 
   // The x-monotone subcurve is defined over a parameter range
   // 0 <= t_min < t_max <= 1. Determine the endpoint with minimal t-value and
   // the one with maximal t-value.
-  const Point_2&       p_min = (_inc_to_right ? left() : right());
-  Originator_iterator  org_min = p_min.get_originator (_curve, _xid);
-  const Point_2&       p_max = (_inc_to_right ? right() : left());
-  Originator_iterator  org_max = p_max.get_originator (_curve, _xid);
-  bool                 clipped_min = false;
+  const Point_2& p_min = (_inc_to_right ? left() : right());
+  Originator_iterator org_min = p_min.get_originator(_curve, _xid);
+  const Point_2& p_max = (_inc_to_right ? right() : left());
+  Originator_iterator org_max = p_max.get_originator(_curve, _xid);
+  bool clipped_min = false;
 
-  CGAL_assertion (org_min != p_min.originators_end());
-  CGAL_assertion (org_max != p_max.originators_end());
+  CGAL_assertion(org_min != p_min.originators_end());
+  CGAL_assertion(org_max != p_max.originators_end());
 
   // Check if t_min = 0. If so, there is no need to clip.
   if (! (org_min->point_bound().type == Bez_point_bound::RATIONAL_PT &&
-         CGAL::sign (org_min->point_bound().t_min) == CGAL::ZERO))
+         CGAL::sign(org_min->point_bound().t_min) == CGAL::ZERO))
   {
     // It is possible that the paramater range of the originator is too large.
     // We therefore make sure it fits the current bounding box of the point
@@ -2104,22 +1930,20 @@ void _Bezier_x_monotone_2<RatKer, AlgKer
 
     // Obtain two control polygons, the first for [0, t_min] and the other for
     // [t_min, 1] and take the second one.
-    typename Bounding_traits::Control_points    cp_a;
-    typename Bounding_traits::Control_points    cp_b;
+    typename Bounding_traits::Control_points cp_a;
+    typename Bounding_traits::Control_points cp_b;
 
     t_min = org_min->point_bound().t_max;
-    de_Casteljau_2 (ctrl.begin(), ctrl.end(),
-                    t_min,
-                    std::back_inserter(cp_a),
-                    std::front_inserter(cp_b));
+    de_Casteljau_2(ctrl.begin(), ctrl.end(),
+                   t_min,
+                   std::back_inserter(cp_a),
+                   std::front_inserter(cp_b));
 
     ctrl.clear();
-    std::copy (cp_b.begin(), cp_b.end(),
-               std::back_inserter (ctrl));
+    std::copy(cp_b.begin(), cp_b.end(), std::back_inserter (ctrl));
     clipped_min = true;
   }
-  else
-  {
+  else {
     t_min = 0;
   }
 
@@ -2137,27 +1961,21 @@ void _Bezier_x_monotone_2<RatKer, AlgKer
     typename Bounding_traits::Control_points    cp_a;
     typename Bounding_traits::Control_points    cp_b;
 
-    if (clipped_min)
-    {
+    if (clipped_min) {
       t_max = (org_max->point_bound().t_min - t_min) / (1 - t_min);
     }
-    else
-    {
+    else {
       t_max = org_max->point_bound().t_min;
     }
 
-    de_Casteljau_2 (ctrl.begin(), ctrl.end(),
-                    t_max,
-                    std::back_inserter(cp_a),
-                    std::front_inserter(cp_b));
+    de_Casteljau_2(ctrl.begin(), ctrl.end(), t_max,
+                   std::back_inserter(cp_a), std::front_inserter(cp_b));
 
     ctrl.clear();
-    std::copy (cp_a.begin(), cp_a.end(),
-               std::back_inserter (ctrl));
+    std::copy(cp_a.begin(), cp_a.end(), std::back_inserter (ctrl));
     t_max = org_max->point_bound().t_min;
   }
-  else
-  {
+  else {
     t_max = 1;
   }
 
@@ -2167,11 +1985,10 @@ void _Bezier_x_monotone_2<RatKer, AlgKer
 // ---------------------------------------------------------------------------
 // Approximate the intersection points between the two given curves.
 //
-template <class RatKer, class AlgKer, class NtTrt, class BndTrt>
-bool _Bezier_x_monotone_2<RatKer, AlgKer, NtTrt,
-                          BndTrt>::_approximate_intersection_points
-        (const Self& cv,
-         std::list<Point_2>& inter_pts) const
+template <typename RatKer, typename AlgKer, typename NtTrt, typename BndTrt>
+bool _Bezier_x_monotone_2<RatKer, AlgKer, NtTrt, BndTrt>::
+_approximate_intersection_points(const Self& cv,
+                                 std::list<Point_2>& inter_pts) const
 {
   typedef typename Bounding_traits::Intersection_point  Intersection_point;
 
@@ -2179,16 +1996,15 @@ bool _Bezier_x_monotone_2<RatKer, AlgKer
 
   // Get the supporting Bezier curves, and make local copies of their control
   // polygons.
-  const Curve_2&                            B1 = this->_curve;
-  const Curve_2&                            B2 = cv._curve;
-  typename Bounding_traits::Control_points  cp1;
-  typename Bounding_traits::NT              t_min1 = 0, t_max1 = 1;
+  const Curve_2& B1 = this->_curve;
+  const Curve_2& B2 = cv._curve;
+  typename Bounding_traits::Control_points cp1;
+  typename Bounding_traits::NT t_min1 = 0, t_max1 = 1;
   typename Bounding_traits::Control_points  cp2;
-  typename Bounding_traits::NT              t_min2 = 0, t_max2 = 1;
-  bool                                      is_self_intersection = false;
+  typename Bounding_traits::NT t_min2 = 0, t_max2 = 1;
+  bool is_self_intersection = false;
 
-  if (! B1.is_same (B2))
-  {
+  if (! B1.is_same (B2)) {
     // In case B1 and B2 are different curves, use their full control polygons
     // in order to approximate all intersection points between the two
     // supporting Bezier curves.
@@ -2197,8 +2013,7 @@ bool _Bezier_x_monotone_2<RatKer, AlgKer
     std::copy (B2.control_points_begin(), B2.control_points_end(),
                std::back_inserter(cp2));
   }
-  else
-  {
+  else {
     // In this case we need to approximate the (self-)intersection points of
     // two subcurves of the same Bezier curve. Clip the control polygons and
     // obtain the control polygons of the two subcurves.
@@ -2208,39 +2023,33 @@ bool _Bezier_x_monotone_2<RatKer, AlgKer
   }
 
   // Use the bounding traits to isolate the intersection points.
-  Bounding_traits                  bound_tr;
-  std::list<Intersection_point>    ipt_bounds;
+  Bounding_traits bound_tr;
+  std::list<Intersection_point> ipt_bounds;
 
-  bound_tr.compute_intersection_points (cp1, cp2,
-                                        std::back_inserter (ipt_bounds));
+  bound_tr.compute_intersection_points(cp1, cp2, std::back_inserter(ipt_bounds));
 
   // Construct the approximated points.
-  typename std::list<Intersection_point>::const_iterator  iter;
-
-  for (iter = ipt_bounds.begin(); iter != ipt_bounds.end(); ++iter)
-  {
+  for (auto iter = ipt_bounds.begin(); iter != ipt_bounds.end(); ++iter) {
     const Bez_point_bound&  bound1 = iter->bound1;
     const Bez_point_bound&  bound2 = iter->bound2;
     const Bez_point_bbox&   bbox = iter->bbox;
 
     // In case it is impossible to further refine the point, stop here.
-    if (! bound1.can_refine || ! bound2.can_refine)
-      return (false);
+    if (! bound1.can_refine || ! bound2.can_refine) return false;
 
     // Create the approximated intersection point.
-    Point_2                 pt;
+    Point_2 pt;
 
     if (bound1.type == Bounding_traits::Bez_point_bound::RATIONAL_PT &&
         bound2.type == Bounding_traits::Bez_point_bound::RATIONAL_PT)
     {
-      CGAL_assertion (CGAL::compare (bound1.t_min, bound1.t_max) == EQUAL);
-      CGAL_assertion (CGAL::compare (bound2.t_min, bound2.t_max) == EQUAL);
-      Rational   t1 = bound1.t_min;
-      Rational   t2 = bound2.t_min;
-      Nt_traits  nt_traits;
+      CGAL_assertion (CGAL::compare(bound1.t_min, bound1.t_max) == EQUAL);
+      CGAL_assertion (CGAL::compare(bound2.t_min, bound2.t_max) == EQUAL);
+      Rational t1 = bound1.t_min;
+      Rational t2 = bound2.t_min;
+      Nt_traits nt_traits;
 
-      if (is_self_intersection)
-      {
+      if (is_self_intersection) {
         // Set the originators with the curve x-monotone IDs.
         // Note that the parameter values we have computed relate to the
         // parameter range [t_min1, t_max1] and [t_min2, t_max2], respectively,
@@ -2249,63 +2058,60 @@ bool _Bezier_x_monotone_2<RatKer, AlgKer
         t1 = t_min1 + t1 * (t_max1 - t_min1);
         t2 = t_min2 + t2 * (t_max2 - t_min2);
 
-        pt = Point_2 (B1, _xid, t1);
-        pt.add_originator (Originator (B2, cv._xid, t2));
+        pt = Point_2(B1, _xid, t1);
+        pt.add_originator(Originator (B2, cv._xid, t2));
       }
-      else
-      {
+      else {
         // Set the originators referring to the entire supporting curves.
-        pt = Point_2 (B1, t1);
-        pt.add_originator (Originator (B2, nt_traits.convert (t2)));
+        pt = Point_2(B1, t1);
+        pt.add_originator(Originator (B2, nt_traits.convert (t2)));
       }
     }
-    else
-    {
-      if (is_self_intersection)
-      {
+    else {
+      if (is_self_intersection) {
         // Set the originators with the curve x-monotone IDs.
         // Note that the parameter values we have computed relate to the
         // parameter range [t_min1, t_max1] and [t_min2, t_max2], respectively,
         // so we scale them back to the parameter range [0, 1] that represent
         // the entire curve.
-        Bez_point_bound  sc_bound1 = bound1;
+        Bez_point_bound sc_bound1 = bound1;
         sc_bound1.t_min = t_min1 + bound1.t_min * (t_max1 - t_min1);
         sc_bound1.t_max = t_min1 + bound1.t_max * (t_max1 - t_min1);
-        pt.add_originator (Originator (B1, _xid, sc_bound1));
+        pt.add_originator(Originator (B1, _xid, sc_bound1));
 
-        Bez_point_bound  sc_bound2 = bound2;
+        Bez_point_bound sc_bound2 = bound2;
         sc_bound2.t_min = t_min2 + bound2.t_min * (t_max2 - t_min2);
         sc_bound2.t_max = t_min2 + bound2.t_max * (t_max2 - t_min2);
-        pt.add_originator (Originator (B2, cv._xid, sc_bound2));
+        pt.add_originator(Originator (B2, cv._xid, sc_bound2));
       }
       else
       {
         // Set the originators with the curve x-monotone IDs.
-        pt.add_originator (Originator (B1, bound1));
-        pt.add_originator (Originator (B2, bound2));
+        pt.add_originator(Originator (B1, bound1));
+        pt.add_originator(Originator (B2, bound2));
       }
     }
-    pt.set_bbox (bbox);
+    pt.set_bbox(bbox);
 
-    inter_pts.push_back (pt);
+    inter_pts.push_back(pt);
   }
 
   // The approximation process ended OK.
-  return (true);
+  return true;
 }
 
 // ---------------------------------------------------------------------------
 // Compute the intersections with the given subcurve.
 //
-template <class RatKer, class AlgKer, class NtTrt, class BndTrt>
-bool _Bezier_x_monotone_2<RatKer, AlgKer, NtTrt, BndTrt>::_intersect
-    (const Self& cv,
-     Intersection_map& inter_map,
-     Bezier_cache& cache,
-     std::vector<Intersection_point_2>& ipts,
-     Self& ovlp_cv) const
+template <typename RatKer, typename AlgKer, typename NtTrt, typename BndTrt>
+bool _Bezier_x_monotone_2<RatKer, AlgKer, NtTrt, BndTrt>::
+_intersect(const Self& cv,
+           Intersection_map& inter_map,
+           Bezier_cache& cache,
+           std::vector<Intersection_point>& ipts,
+           Self& ovlp_cv) const
 {
-  CGAL_precondition (_curve.id() <= cv._curve.id());
+  CGAL_precondition(_curve.id() <= cv._curve.id());
 
   ipts.clear();
 
@@ -2313,115 +2119,96 @@ bool _Bezier_x_monotone_2<RatKer, AlgKer
   // first check if this base curve is not self-intersecting. If this is the
   // case we can avoid any attempt of computing intersection points between
   // the two subcurves.
-  const bool                self_intersect = (_curve.id() == cv._curve.id());
-
-  if (self_intersect)
-  {
-    if (_xid == cv._xid)
-      return (false);
+  const bool self_intersect = (_curve.id() == cv._curve.id());
 
-    if (_curve.has_no_self_intersections())
-      return (false);
+  if (self_intersect) {
+    if (_xid == cv._xid) return false;
+    if (_curve.has_no_self_intersections()) return false;
   }
 
   // Construct the pair of curve IDs and look for it in the intersection map.
-  Curve_pair                 curve_pair (_curve.id(), cv._curve.id());
-  Intersection_map_iterator  map_iter = inter_map.find (curve_pair);
-  std::list<Point_2>         inter_pts;
-  bool                       app_ok = true;
+  Curve_pair curve_pair(_curve.id(), cv._curve.id());
+  Intersection_map_iterator map_iter = inter_map.find(curve_pair);
+  std::list<Point_2> inter_pts;
+  bool app_ok = true;
 
-  if (map_iter != inter_map.end())
-  {
+  if (map_iter != inter_map.end()) {
     // Get the intersection points between the two supporting curves as stored
     // in the map.
     inter_pts = map_iter->second;
   }
-  else
-  {
+  else {
     // Approximate the intersection points and store them in the map.
     // Note that we do not store approximated self-intersections in the map,
     // as they realte only to the pecific x-monotone curves, and not to the
     // entire curve.
-    app_ok = _approximate_intersection_points (cv,
-                                               inter_pts);
+    app_ok = _approximate_intersection_points(cv, inter_pts);
 
-    if (app_ok && ! self_intersect)
-      inter_map[curve_pair] = inter_pts;
+    if (app_ok && ! self_intersect) inter_map[curve_pair] = inter_pts;
   }
 
   // Try to approximate the intersection points.
-  bool                in_range1, in_range2;
-  bool                correct_res;
+  bool in_range1, in_range2;
+  bool correct_res;
 
-  if (app_ok)
-  {
+  if (app_ok) {
     // Approximations are computed using de Casteljau subdivision and
     // filtering using skewed bounding boxes. A property of these bboxes
-    // if that it can fail in the following cases: (i) there are two intersection
-    // points lying very close together, (ii) there exists an intersection point
-    // whose multiplicity is greater than 1, or (iii) the curves overlap.
-    // If the approximation went OK, then we know that we have a simple
-    // intersection point (with multiplicity 1) if intersection point
-    // is not rational (otherwise it is unknown: at this point, an intersection point
-    // is rational if it was found as a control point during the de Casteljau subdivision)
+    // if that it can fail in the following cases: (i) there are two
+    // intersection points lying very close together, (ii) there exists an
+    // intersection point whose multiplicity is greater than 1, or (iii) the
+    // curves overlap. If the approximation went OK, then we know that we have
+    // a simple intersection point (with multiplicity 1) if intersection point
+    // is not rational (otherwise it is unknown: at this point, an intersection
+    // point is rational if it was found as a control point during the de
+    // Casteljau subdivision)
     // We go over the points and report the ones lying in the parameter
     // ranges of both curves. Note that in case of self-intersections,
     // all points we get are in the respective parameter range of the curves.
     typename std::list<Point_2>::iterator  pit;
 
-    for (pit = inter_pts.begin(); pit != inter_pts.end(); ++pit)
-    {
+    for (pit = inter_pts.begin(); pit != inter_pts.end(); ++pit) {
       // Check if the point is in the range of this curve - first using
       // its parameter bounds, and if we fail we perform an exact check.
-      if (! self_intersect)
-      {
-        in_range1 = _is_in_range (*pit, correct_res);
+      if (! self_intersect) {
+        in_range1 = _is_in_range(*pit, correct_res);
       }
-      else
-      {
+      else {
         in_range1 = true;
         correct_res = true;
       }
 
-      if (! correct_res)
-      {
-        if (! pit->is_exact())
-          pit->make_exact (cache);
+      if (! correct_res) {
+        if (! pit->is_exact()) pit->make_exact(cache);
 
-        Originator_iterator  p_org = pit->get_originator (_curve, _xid);
+        Originator_iterator p_org = pit->get_originator (_curve, _xid);
         CGAL_assertion (p_org != pit->originators_end());
 
-        in_range1 = _is_in_range (p_org->parameter(), cache);
+        in_range1 = _is_in_range(p_org->parameter(), cache);
       }
 
-      if (! in_range1)
-        continue;
+      if (! in_range1) continue;
 
       // Check if the point is in the range of the other curve - first using
       // its parameter bounds, and if we fail we perform an exact check.
-      if (! self_intersect)
-      {
+      if (! self_intersect) {
         in_range2 = cv._is_in_range (*pit, correct_res);
       }
-      else
-      {
+      else {
         in_range2 = true;
         correct_res = true;
       }
 
-      if (! correct_res)
-      {
-        if (! pit->is_exact())
-          pit->make_exact (cache);
+      if (! correct_res) {
+        if (! pit->is_exact()) pit->make_exact (cache);
 
         Originator_iterator  p_org = pit->get_originator (cv._curve, cv._xid);
-        CGAL_assertion (p_org != pit->originators_end());
+        CGAL_assertion(p_org != pit->originators_end());
 
-        in_range2 = cv._is_in_range (p_org->parameter(), cache);
+        in_range2 = cv._is_in_range(p_org->parameter(), cache);
       }
 
-      if (in_range1 && in_range2)
-      {
+      if (in_range1 && in_range2) {
         // In case the originators of the intersection point are not marked
         // with x-monotone identifiers, mark them now as we know in which
         // subcurves they lie.
@@ -2429,17 +2216,17 @@ bool _Bezier_x_monotone_2<RatKer, AlgKer
         CGAL_assertion (p_org1 != pit->originators_end());
 
         if (p_org1->xid() == 0)
-          pit->update_originator_xid (*p_org1, _xid);
+          pit->update_originator_xid(*p_org1, _xid);
 
         Originator_iterator  p_org2 = pit->get_originator (cv._curve, cv._xid);
         CGAL_assertion (p_org2 != pit->originators_end());
 
         if (p_org2->xid() == 0)
-          pit->update_originator_xid (*p_org2, cv._xid);
+          pit->update_originator_xid(*p_org2, cv._xid);
 
         // The point lies within the parameter range of both curves, so we
         // report it as a valid intersection point with multiplicity 1 or unknown.
-        ipts.push_back (Intersection_point_2 (*pit, pit->is_rational()?0:1));
+        ipts.push_back(Intersection_point(*pit, pit->is_rational() ? 0 : 1));
       }
     }
 
@@ -2449,34 +2236,30 @@ bool _Bezier_x_monotone_2<RatKer, AlgKer
 
   // We did not succeed in isolate the approximate intersection points.
   // We therefore resort to the exact procedure and exactly compute them.
-  bool                   do_ovlp;
-  const Intersect_list&  inter_list =
-    cache.get_intersections (_curve.id(),
-                             _curve.x_polynomial(), _curve.x_norm(),
-                             _curve.y_polynomial(), _curve.y_norm(),
-                             cv._curve.id(),
-                             cv._curve.x_polynomial(), cv._curve.x_norm(),
-                             cv._curve.y_polynomial(), cv._curve.y_norm(),
-                             do_ovlp);
+  bool do_ovlp;
+  const Intersect_list& inter_list =
+    cache.get_intersections(_curve.id(),
+                            _curve.x_polynomial(), _curve.x_norm(),
+                            _curve.y_polynomial(), _curve.y_norm(),
+                            cv._curve.id(),
+                            cv._curve.x_polynomial(), cv._curve.x_norm(),
+                            cv._curve.y_polynomial(), cv._curve.y_norm(),
+                            do_ovlp);
 
-  if (do_ovlp)
-  {
+  if (do_ovlp) {
     // Check the case of co-inciding endpoints
-    if (left().equals (cv.left(), cache))
-    {
-      if (right().equals (cv.right(), cache))
-      {
+    if (left().equals (cv.left(), cache)) {
+      if (right().equals (cv.right(), cache)) {
         // The two curves entirely overlap one another:
         ovlp_cv = cv;
-        return (true);
+        return true;
       }
 
-      Algebraic     t_right;
-      bool          is_endpoint;
+      Algebraic t_right;
+      bool is_endpoint;
 
-      if (_is_in_range (cv.right(), cache, t_right, is_endpoint))
-      {
-        CGAL_assertion (! is_endpoint);
+      if (_is_in_range(cv.right(), cache, t_right, is_endpoint)) {
+        CGAL_assertion(! is_endpoint);
 
         // Case 1 - *this:   s +-----------+ t
         //             cv:   s'+=====+ t'
@@ -2484,13 +2267,12 @@ bool _Bezier_x_monotone_2<RatKer, AlgKer
         // Take cv as the overlapping subcurve, and add originators for its
         // right endpoint referring to *this.
         ovlp_cv = cv;
-        ovlp_cv.right().add_originator (Originator (_curve, _xid, t_right));
+        ovlp_cv.right().add_originator(Originator(_curve, _xid, t_right));
 
-        return (true);
+        return true;
       }
-      else if (cv._is_in_range (right(), cache, t_right, is_endpoint))
-      {
-        CGAL_assertion (! is_endpoint);
+      else if (cv._is_in_range(right(), cache, t_right, is_endpoint)) {
+        CGAL_assertion(! is_endpoint);
 
         // Case 2 - *this:   s +----+ t
         //             cv:   s'+==========+ t'
@@ -2498,24 +2280,21 @@ bool _Bezier_x_monotone_2<RatKer, AlgKer
         // Take this as the overlapping subcurve, and add originators for its
         // right endpoint referring to cv.
         ovlp_cv = *this;
-        ovlp_cv.right().add_originator (Originator (cv._curve, cv._xid,
-                                                    t_right));
+        ovlp_cv.right().add_originator(Originator(cv._curve, cv._xid, t_right));
 
-        return (true);
+        return true;
       }
 
       // In this case the two curves do not overlap, but have a common left
       // endpoint.
-      ipts.push_back (Intersection_point_2 (left(), 0));
+      ipts.push_back(Intersection_point(left(), 0));
       return (false);
     }
-    else if (right().equals (cv.right(), cache))
-    {
-      Algebraic     t_left;
-      bool          is_endpoint;
+    else if (right().equals (cv.right(), cache)) {
+      Algebraic t_left;
+      bool is_endpoint;
 
-      if (_is_in_range (cv.left(), cache, t_left, is_endpoint))
-      {
+      if (_is_in_range (cv.left(), cache, t_left, is_endpoint)) {
         CGAL_assertion (! is_endpoint);
 
         // Case 3 - *this:   s +-----------+ t
@@ -2528,9 +2307,8 @@ bool _Bezier_x_monotone_2<RatKer, AlgKer
 
         return (true);
       }
-      else if (cv._is_in_range (left(), cache, t_left, is_endpoint))
-      {
-        CGAL_assertion (! is_endpoint);
+      else if (cv._is_in_range(left(), cache, t_left, is_endpoint)) {
+        CGAL_assertion(! is_endpoint);
 
         // Case 4 - *this:         s +----+ t
         //             cv:   s'+==========+ t'
@@ -2538,30 +2316,26 @@ bool _Bezier_x_monotone_2<RatKer, AlgKer
         // Take this as the overlapping subcurve, and add originators for its
         // left endpoint referring to cv.
         ovlp_cv = *this;
-        ovlp_cv.left().add_originator (Originator (cv._curve, cv._xid,
-                                                   t_left));
+        ovlp_cv.left().add_originator(Originator(cv._curve, cv._xid, t_left));
 
-        return (true);
+        return true;
       }
 
       // In this case the two curves do not overlap, but have a common right
       // endpoint.
-      ipts.push_back (Intersection_point_2 (right(), 0));
-      return (false);
+      ipts.push_back(Intersection_point(right(), 0));
+      return false;
     }
 
     // If we reached here, none of the endpoints coincide.
     // Check the possible overlap scenarios.
-    Point_2         ovrp_src, ovlp_trg;
-    Algebraic       t_cv_src;
-    Algebraic       t_cv_trg;
-    bool            is_endpoint = false;
+    Point_2 ovrp_src, ovlp_trg;
+    Algebraic t_cv_src;
+    Algebraic t_cv_trg;
+    bool is_endpoint = false;
 
-    if (_is_in_range (cv._ps, cache, t_cv_src, is_endpoint) &&
-        ! is_endpoint)
-    {
-      if (_is_in_range (cv._pt, cache, t_cv_trg, is_endpoint) &&
-          ! is_endpoint)
+    if (_is_in_range (cv._ps, cache, t_cv_src, is_endpoint) && ! is_endpoint) {
+      if (_is_in_range (cv._pt, cache, t_cv_trg, is_endpoint) && ! is_endpoint)
       {
         // Case 5 - *this:   s +-----------+ t
         //             cv:     s' +=====+ t'
@@ -2569,32 +2343,30 @@ bool _Bezier_x_monotone_2<RatKer, AlgKer
         // Take cv as the overlapping subcurve, and add originators for its
         // endpoints referring to *this.
         ovlp_cv = cv;
-        ovlp_cv._ps.add_originator (Originator (_curve, _xid, t_cv_src));
-        ovlp_cv._pt.add_originator (Originator (_curve, _xid, t_cv_trg));
+        ovlp_cv._ps.add_originator(Originator (_curve, _xid, t_cv_src));
+        ovlp_cv._pt.add_originator(Originator (_curve, _xid, t_cv_trg));
 
-        return (true);
+        return true;
       }
-      else
-      {
+      else {
         // Case 6 - *this:   s +-----------+ t
         //             cv:            s' +=====+ t'
         //
         // Use *this as a base, and replace its source point.
         ovlp_cv = *this;
         ovlp_cv._ps = cv._ps;
-        ovlp_cv._ps.add_originator (Originator (_curve, _xid, t_cv_src));
+        ovlp_cv._ps.add_originator(Originator (_curve, _xid, t_cv_src));
 
         // Add an originator to the target point, referring to cv:
-        CGAL_assertion_code (bool  pt_in_cv_range =)
-        cv._is_in_range (ovlp_cv._pt, cache, t_cv_trg, is_endpoint);
-        CGAL_assertion (pt_in_cv_range);
-        ovlp_cv._pt.add_originator (Originator (cv._curve, cv._xid, t_cv_trg));
+        CGAL_assertion_code(bool  pt_in_cv_range =)
+        cv._is_in_range(ovlp_cv._pt, cache, t_cv_trg, is_endpoint);
+        CGAL_assertion(pt_in_cv_range);
+        ovlp_cv._pt.add_originator(Originator(cv._curve, cv._xid, t_cv_trg));
 
-        return (true);
+        return true;
       }
     }
-    else if (_is_in_range (cv._pt, cache, t_cv_trg, is_endpoint) &&
-             ! is_endpoint)
+    else if (_is_in_range(cv._pt, cache, t_cv_trg, is_endpoint) && ! is_endpoint)
     {
       // Case 7 - *this:      s +-----------+ t
       //             cv:   s' +=====+ t'
@@ -2606,14 +2378,14 @@ bool _Bezier_x_monotone_2<RatKer, AlgKer
 
       // Add an originator to the source point, referring to cv:
       CGAL_assertion_code (bool  ps_in_cv_range =)
-      cv._is_in_range (ovlp_cv._ps, cache, t_cv_src, is_endpoint);
+      cv._is_in_range(ovlp_cv._ps, cache, t_cv_src, is_endpoint);
       CGAL_assertion (ps_in_cv_range);
-      ovlp_cv._ps.add_originator (Originator (cv._curve, cv._xid, t_cv_src));
+      ovlp_cv._ps.add_originator(Originator(cv._curve, cv._xid, t_cv_src));
 
       return (true);
     }
-    else if (cv._is_in_range (_ps, cache, t_cv_src, is_endpoint) &&
-             cv._is_in_range (_pt, cache, t_cv_trg, is_endpoint))
+    else if (cv._is_in_range(_ps, cache, t_cv_src, is_endpoint) &&
+             cv._is_in_range(_pt, cache, t_cv_trg, is_endpoint))
     {
       // Case 8 - *this:      s +---------+ t
       //             cv:  s' +================+ t'
@@ -2621,38 +2393,32 @@ bool _Bezier_x_monotone_2<RatKer, AlgKer
       // Take *this as the overlapping subcurve, and add originators for its
       // endpoints referring to cv.
       ovlp_cv = *this;
-      ovlp_cv._ps.add_originator (Originator (cv._curve, cv._xid, t_cv_src));
-      ovlp_cv._pt.add_originator (Originator (cv._curve, cv._xid, t_cv_trg));
+      ovlp_cv._ps.add_originator(Originator(cv._curve, cv._xid, t_cv_src));
+      ovlp_cv._pt.add_originator(Originator(cv._curve, cv._xid, t_cv_trg));
 
-      return (true);
+      return true;
     }
 
     // If we reached here, there are no overlaps:
-    return (false);
+    return false;
   }
 
   // Go over the points and report the ones lying in the parameter ranges
   // of both curves.
-  Intersect_iter          iit;
-
-  for (iit = inter_list.begin(); iit != inter_list.end(); ++iit)
-  {
-    if (_is_in_range (iit->s, cache) &&
-        cv._is_in_range (iit->t, cache))
-    {
+  for (auto iit = inter_list.begin(); iit != inter_list.end(); ++iit) {
+    if (_is_in_range (iit->s, cache) && cv._is_in_range(iit->t, cache)) {
       // Construct an intersection point with unknown multiplicity.
-      Point_2                 pt (iit->x, iit->y,
-                                  true);        // Dummy parameter.
+      Point_2 pt (iit->x, iit->y, true);        // Dummy parameter.
 
-      pt.add_originator (Originator (_curve, _xid, iit->s));
-      pt.add_originator (Originator (cv._curve, cv._xid, iit->t));
+      pt.add_originator(Originator(_curve, _xid, iit->s));
+      pt.add_originator(Originator(cv._curve, cv._xid, iit->t));
 
-      ipts.push_back (Intersection_point_2 (pt, 0));
+      ipts.push_back(Intersection_point(pt, 0));
     }
   }
 
   // Mark that there is no overlap:
-  return (false);
+  return false;
 }
 
 // ---------------------------------------------------------------------------
@@ -2676,23 +2442,22 @@ _exact_vertical_position(const Point_2&
     rat_p = (Rat_point_2) p;
 
   // Get a rational approximation of the parameter values at the endpoints.
-  Nt_traits                nt_traits;
-  Originator_iterator      ps_org = _ps.get_originator (_curve, _xid);
+  Nt_traits nt_traits;
+  Originator_iterator ps_org = _ps.get_originator(_curve, _xid);
   CGAL_assertion (ps_org != _ps.originators_end());
 
-  Originator_iterator      pt_org = _pt.get_originator (_curve, _xid);
-  CGAL_assertion (pt_org != _pt.originators_end());
-
-  Rational                 my_t_min;
-  Rational                 my_t_max;
+  Originator_iterator pt_org = _pt.get_originator(_curve, _xid);
+  CGAL_assertion(pt_org != _pt.originators_end());
 
+  Rational my_t_min;
+  Rational my_t_max;
 
-  bool      can_refine_s = ! _ps.is_exact();
-  bool      can_refine_t = ! _pt.is_exact();
+  bool can_refine_s = ! _ps.is_exact();
+  bool can_refine_t = ! _pt.is_exact();
 
   do {
-    if (CGAL::compare (ps_org->point_bound().t_max,
-                       pt_org->point_bound().t_min) == SMALLER)
+    if (CGAL::compare(ps_org->point_bound().t_max,
+                      pt_org->point_bound().t_min) == SMALLER)
     {
       // In case the parameter value of the source is smaller than the target's.
       my_t_min = ps_org->point_bound().t_max;
@@ -2719,14 +2484,14 @@ _exact_vertical_position(const Point_2&
   while(can_refine_s || can_refine_t);
 
   // Start the subdivision process from the entire supporting curve.
-  std::list<Subcurve>      subcurves;
-  Subcurve                 init_scv;
-  Rational                 x_min, y_min, x_max, y_max;
-  bool                     no_x_ovlp;
-  Comparison_result        res_y_min, res_y_max;
+  std::list<Subcurve> subcurves;
+  Subcurve init_scv;
+  Rational x_min, y_min, x_max, y_max;
+  bool no_x_ovlp;
+  Comparison_result res_y_min, res_y_max;
 
-  std::copy (_curve.control_points_begin(), _curve.control_points_end(),
-             std::back_inserter (init_scv.control_points));
+  std::copy(_curve.control_points_begin(), _curve.control_points_end(),
+            std::back_inserter(init_scv.control_points));
   init_scv.t_min = 0;
   init_scv.t_max = 1;
   subcurves.push_back (init_scv);
@@ -2735,13 +2500,12 @@ _exact_vertical_position(const Point_2&
   {
     // Go over the list of subcurves and consider only those lying in the
     // given [t_min, t_max] bound.
-    typename std::list<Subcurve>::iterator  iter = subcurves.begin();
-    bool                                    is_fully_in_range;
+    typename std::list<Subcurve>::iterator iter = subcurves.begin();
+    bool is_fully_in_range;
 
-    while (iter != subcurves.end())
-    {
-      if (CGAL::compare (iter->t_max, my_t_min) == SMALLER ||
-          CGAL::compare (iter->t_min, my_t_max) == LARGER)
+    while (iter != subcurves.end()) {
+      if (CGAL::compare(iter->t_max, my_t_min) == SMALLER ||
+          CGAL::compare(iter->t_min, my_t_max) == LARGER)
       {
         // Subcurve out of bounds of the x-monotone curve we consider - erase
         // it and continue to next subcurve.
@@ -2753,43 +2517,37 @@ _exact_vertical_position(const Point_2&
       // the bounding box of the point.
       iter->bbox (x_min, y_min, x_max, y_max);
 
-      if (p.is_rational())
-      {
-        no_x_ovlp = (CGAL::compare (x_min, rat_p.x()) == LARGER ||
-                     CGAL::compare (x_max, rat_p.x()) == SMALLER);
-      }
-      else
-      {
-        no_x_ovlp = (CGAL::compare (nt_traits.convert (x_min),
-                                    p.x()) == LARGER ||
-                     CGAL::compare (nt_traits.convert (x_max),
-                                    p.x()) == SMALLER);
+      if (p.is_rational()) {
+        no_x_ovlp = (CGAL::compare(x_min, rat_p.x()) == LARGER ||
+                     CGAL::compare(x_max, rat_p.x()) == SMALLER);
+      }
+      else {
+        no_x_ovlp = (CGAL::compare(nt_traits.convert (x_min),
+                                   p.x()) == LARGER ||
+                     CGAL::compare(nt_traits.convert (x_max),
+                                   p.x()) == SMALLER);
       }
 
-      if (no_x_ovlp)
-      {
+      if (no_x_ovlp) {
         // Subcurve out of x-bounds - erase it and continue to next subcurve.
         subcurves.erase(iter++);
         continue;
       }
 
       // In this case, check if there is an overlap in the y-range.
-      if (p.is_rational())
-      {
-        res_y_min = CGAL::compare (rat_p.y(), y_min);
-        res_y_max = CGAL::compare (rat_p.y(), y_max);
+      if (p.is_rational()) {
+        res_y_min = CGAL::compare(rat_p.y(), y_min);
+        res_y_max = CGAL::compare(rat_p.y(), y_max);
       }
-      else
-      {
-        res_y_min = CGAL::compare (p.y(), nt_traits.convert (y_min));
-        res_y_max = CGAL::compare (p.y(), nt_traits.convert (y_max));
+      else {
+        res_y_min = CGAL::compare(p.y(), nt_traits.convert (y_min));
+        res_y_max = CGAL::compare(p.y(), nt_traits.convert (y_max));
       }
 
-      is_fully_in_range = (CGAL::compare (iter->t_min, my_t_min) != SMALLER) &&
-                          (CGAL::compare (iter->t_max, my_t_max) != LARGER);
+      is_fully_in_range = (CGAL::compare(iter->t_min, my_t_min) != SMALLER) &&
+                          (CGAL::compare(iter->t_max, my_t_max) != LARGER);
 
-      if (res_y_min != res_y_max || ! is_fully_in_range)
-      {
+      if (res_y_min != res_y_max || ! is_fully_in_range) {
         // Subdivide the current subcurve and replace iter with the two
         // resulting subcurves using de Casteljau's algorithm.
         Subcurve           scv_l, scv_r;
@@ -2798,21 +2556,20 @@ _exact_vertical_position(const Point_2&
         scv_r.t_max = iter->t_max;
         scv_l.t_max = scv_r.t_min = (iter->t_min + iter->t_max) / 2;
 
-        bisect_control_polygon_2 (iter->control_points.begin(),
-                                  iter->control_points.end(),
-                                  std::back_inserter(scv_l.control_points),
-                                  std::front_inserter(scv_r.control_points));
+        bisect_control_polygon_2(iter->control_points.begin(),
+                                 iter->control_points.end(),
+                                 std::back_inserter(scv_l.control_points),
+                                 std::front_inserter(scv_r.control_points));
 
-        subcurves.insert (iter, scv_l);
-        subcurves.insert (iter, scv_r);
+        subcurves.insert(iter, scv_l);
+        subcurves.insert(iter, scv_r);
         subcurves.erase(iter++);
 
         continue;
       }
 
-      if (res_y_min == res_y_max)
-      {
-        CGAL_assertion (res_y_min != EQUAL);
+      if (res_y_min == res_y_max) {
+        CGAL_assertion(res_y_min != EQUAL);
 
         // We reached a separation, as p is either strictly above or strictly
         // below the bounding box of the current subcurve.
@@ -2826,8 +2583,8 @@ _exact_vertical_position(const Point_2&
   }
 
   // We can reach here only if we do not force an exact result.
-  CGAL_assertion (! force_exact);
-  return (EQUAL);
+  CGAL_assertion(! force_exact);
+  return EQUAL;
 }
 
 } //namespace CGAL
diff -pruN 5.0.3-1/include/CGAL/Arr_geometry_traits/Circle_segment_2.h 5.1-1/include/CGAL/Arr_geometry_traits/Circle_segment_2.h
--- 5.0.3-1/include/CGAL/Arr_geometry_traits/Circle_segment_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_geometry_traits/Circle_segment_2.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,21 +3,21 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/Circle_segment_2.h $
-// $Id: Circle_segment_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/Circle_segment_2.h $
+// $Id: Circle_segment_2.h 58276ed 2020-03-31T18:34:28+03:00 Efi Fogel
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
-// Author(s)     : Ron Wein <wein@post.tau.ac.il>
-//                 Baruch Zukerman <baruchzu@post.tau.ac.il>
-//                 Waqar Khan <wkhan@mpi-inf.mpg.de>
+// Author(s): Ron Wein          <wein@post.tau.ac.il>
+//            Baruch Zukerman   <baruchzu@post.tau.ac.il>
+//            Waqar Khan        <wkhan@mpi-inf.mpg.de>
+//            Efi Fogel         <efifogel@gmail.com>
 
 #ifndef CGAL_CIRCLE_SEGMENT_2_H
 #define CGAL_CIRCLE_SEGMENT_2_H
 
 #include <CGAL/license/Arrangement_on_surface_2.h>
 
-
 /*! \file
  * Header file for the _Circle_segment_2<Kernel, Filter> class.
  */
@@ -31,126 +31,103 @@
 namespace CGAL {
 
 // Forward declaration:
-template <class NumberType_, bool Filter_> class _One_root_point_2;
+template <typename NumberType_, bool Filter_> class _One_root_point_2;
 
 /*! \class
  * Representation of a point whose coordinates are one-root numbers.
  */
-template <class NumberType_, bool Filter_>
-class _One_root_point_2_rep //: public Ref_counted
-{
+template <typename NumberType_, bool Filter_>
+class _One_root_point_2_rep {
   friend class _One_root_point_2<NumberType_, Filter_>;
 
 public:
-
   typedef NumberType_                               NT;
   typedef _One_root_point_2_rep<NT, Filter_>        Self;
-  typedef Sqrt_extension<NT,NT,Tag_true,Boolean_tag<Filter_> >    CoordNT;
+  typedef Sqrt_extension<NT, NT, Tag_true,Boolean_tag<Filter_> >    CoordNT;
 
 private:
-
-  CoordNT       _x;            // The coordinates.
-  CoordNT       _y;
+  CoordNT _x;            // The coordinates.
+  CoordNT _y;
 
 public:
-
   /*! Default constructor. */
-  _One_root_point_2_rep () :
-    _x (0),
-    _y (0)
+  _One_root_point_2_rep() :
+    _x(0),
+    _y(0)
   {}
 
   /*! Constructor of a point with one-root coefficients.
      This constructor of a point can also be used with rational coefficients
      thanks to convertor of CoordNT. */
-  _One_root_point_2_rep (const CoordNT& x, const CoordNT& y) :
-    _x (x),
-    _y (y)
+  _One_root_point_2_rep(const CoordNT& x, const CoordNT& y) :
+    _x(x),
+    _y(y)
   {}
 };
 
 /*! \class
  * A handle for a point whose coordinates are one-root numbers.
  */
-template <class NumberType_, bool Filter_>
+template <typename NumberType_, bool Filter_>
 class _One_root_point_2 :
   public Handle_for<_One_root_point_2_rep<NumberType_, Filter_> >
 {
 public:
-
   typedef NumberType_                           NT;
   typedef _One_root_point_2<NT, Filter_>        Self;
 
 private:
-
   typedef _One_root_point_2_rep<NT, Filter_>    Point_rep;
   typedef Handle_for<Point_rep>                 Point_handle;
 
 public:
-
   typedef typename Point_rep::CoordNT           CoordNT;
 
   /*! Default constructor. */
-  _One_root_point_2 () :
-    Point_handle (Point_rep())
-  {}
+  _One_root_point_2() : Point_handle(Point_rep()) {}
 
   /*! Copy constructor. */
-  _One_root_point_2 (const Self& p) :
-    Point_handle (p)
-  {}
+  _One_root_point_2(const Self& p) : Point_handle(p) {}
 
   _One_root_point_2& operator=(const _One_root_point_2&)=default;
 
   /*! Constructor of a point with one-root coefficients.
      This constructor of a point can also be used with rational coefficients
      thanks to convertor of CoordNT. */
-  _One_root_point_2 (const CoordNT& x, const CoordNT& y) :
-    Point_handle (Point_rep (x, y))
+  _One_root_point_2(const CoordNT& x, const CoordNT& y) :
+    Point_handle(Point_rep(x, y))
   {}
 
   /*! Get the x-coordinate. */
-  const CoordNT& x () const
-  {
-    return (this->ptr()->_x);
-  }
+  const CoordNT& x() const { return (this->ptr()->_x); }
 
   /*! Get the y-coordinate. */
-  const CoordNT& y () const
-  {
-    return (this->ptr()->_y);
-  }
+  const CoordNT& y() const { return (this->ptr()->_y); }
 
   /*! Check for equality. */
-  bool equals (const Self& p) const
+  bool equals(const Self& p) const
   {
-    if (this->identical (p))
-      return (true);
+    if (this->identical(p)) return (true);
 
-    return (CGAL::compare (this->ptr()->_x, p.ptr()->_x) == EQUAL &&
-            CGAL::compare (this->ptr()->_y, p.ptr()->_y) == EQUAL);
+    return (CGAL::compare(this->ptr()->_x, p.ptr()->_x) == EQUAL &&
+            CGAL::compare(this->ptr()->_y, p.ptr()->_y) == EQUAL);
   }
 
-  bool operator != (const Self& p) const
-  {
-    return !equals(p);
-  }
+  bool operator != (const Self& p) const { return ! equals(p); }
+
+  bool operator == (const Self& p) const { return equals(p); }
 
-  bool operator == (const Self& p) const
-  {
-    return equals(p);
-  }
   /*! Set the point coordinates. */
-  void set (const NT& x, const NT& y)
+  void set(const NT& x, const NT& y)
   {
     this->copy_on_write();
-    this->ptr()->_x = CoordNT (x);
-    this->ptr()->_y = CoordNT (y);
+    this->ptr()->_x = CoordNT(x);
+    this->ptr()->_y = CoordNT(y);
     return;
   }
 
   /*! Set the point coordinates. */
-  void set (const CoordNT& x, const CoordNT& y)
+  void set(const CoordNT& x, const CoordNT& y)
   {
     this->copy_on_write();
     this->ptr()->_x = x;
@@ -162,10 +139,9 @@ public:
 /*!
  * Exporter for conic arcs.
  */
-template <class NT, bool Filter>
-std::ostream&
-operator<< (std::ostream& os,
-            const _One_root_point_2<NT, Filter>& p)
+template <typename NT, bool Filter>
+std::ostream& operator<<(std::ostream& os,
+                         const _One_root_point_2<NT, Filter>& p)
 {
   os << CGAL::to_double(p.x()) << ' ' << CGAL::to_double(p.y());
   return (os);
@@ -186,11 +162,9 @@ std::istream & operator >> (std::istream
 /*! \class
  * Representation of a circle, a circular arc or a line segment.
  */
-template <class Kernel_, bool Filter_>
-class _Circle_segment_2
-{
+template <typename Kernel_, bool Filter_>
+class _Circle_segment_2 {
 public:
-
   typedef Kernel_                                          Kernel;
   typedef typename Kernel::FT                              NT;
   typedef _One_root_point_2<NT, Filter_>                   Point_2;
@@ -199,114 +173,101 @@ public:
   typedef typename Kernel::Line_2                          Line_2;
 
 protected:
-
   typedef typename Point_2::CoordNT                        CoordNT;
 
   // Data members:
-  Line_2        _line;        // The supporting line (for line segments).
-  Circle_2      _circ;        // The supporting circle (for circular arcs).
-  bool          _is_full;     // Whether we have a full circle.
-  bool          _has_radius;  // Is the radius (not just the squared radius)
+  Line_2 m_line;              // The supporting line (for line segments).
+  Circle_2 m_circ;            // The supporting circle (for circular arcs).
+  bool m_is_full;             // Whether we have a full circle.
+  bool m_has_radius;          // Is the radius (not just the squared radius)
                               // explicitly specified).
-  NT            _radius;      // The radius, in case it is specified.
-  Point_2       _source;      // The source point.
-  Point_2       _target;      // The target point.
-  Orientation   _orient;      // The orientation (COLLINEAR for line segments).
+  NT m_radius;                // The radius, in case it is specified.
+  Point_2 m_source;           // The source point.
+  Point_2 m_target;           // The target point.
+  Orientation m_orient;       // The orientation (COLLINEAR for line segments).
 
 public:
-
   /*! Default constructor. */
-  _Circle_segment_2 () :
-    _is_full (false),
-    _has_radius (false),
-    _orient (COLLINEAR)
+  _Circle_segment_2() :
+    m_is_full(false),
+    m_has_radius(false),
+    m_orient(COLLINEAR)
   {}
 
-  /*!
-   * Constructor from a line segment.
+  /*! Constructor from a line segment.
    * \param seg The segment.
    */
-  _Circle_segment_2 (const Segment_2& seg) :
-    _line (seg),
-    _is_full (false),
-    _has_radius (false),
-    _source (seg.source().x(), seg.source().y()),
-    _target (seg.target().x(), seg.target().y()),
-    _orient (COLLINEAR)
+  _Circle_segment_2(const Segment_2& seg) :
+    m_line(seg),
+    m_is_full(false),
+    m_has_radius(false),
+    m_source(seg.source().x(), seg.source().y()),
+    m_target(seg.target().x(), seg.target().y()),
+    m_orient(COLLINEAR)
   {}
 
-  /*!
-  * Constructor from of a line segment.
+  /*! Constructor from of a line segment.
    * \param ps The source point.
    * \param pt The target point.
    */
-  _Circle_segment_2 (const typename Kernel::Point_2& ps,
-                     const typename Kernel::Point_2& pt) :
-    _line (ps, pt),
-    _is_full (false),
-    _has_radius (false),
-    _source (ps.x(), ps.y()),
-    _target (pt.x(), pt.y()),
-    _orient (COLLINEAR)
+  _Circle_segment_2(const typename Kernel::Point_2& ps,
+                    const typename Kernel::Point_2& pt) :
+    m_line(ps, pt),
+    m_is_full(false),
+    m_has_radius(false),
+    m_source(ps.x(), ps.y()),
+    m_target(pt.x(), pt.y()),
+    m_orient(COLLINEAR)
   {}
 
-  /*!
-   * Constructor of a segment, given a supporting line and two endpoints,
+  /*! Constructor of a segment, given a supporting line and two endpoints,
    * which need not necessarily have rational coordinates.
    * \param line The supporting line.
    * \param source The source point.
    * \param target The target point.
    * \pre Both endpoints lie on the supporting line.
    */
-  _Circle_segment_2 (const Line_2& line,
-                     const Point_2& source, const Point_2& target) :
-    _line (line),
-    _is_full (false),
-    _has_radius (false),
-    _source (source),
-    _target (target),
-    _orient (COLLINEAR)
+  _Circle_segment_2(const Line_2& line,
+                    const Point_2& source, const Point_2& target) :
+    m_line(line),
+    m_is_full(false),
+    m_has_radius(false),
+    m_source(source),
+    m_target(target),
+    m_orient(COLLINEAR)
   {
-    CGAL_precondition (CGAL::compare (source.x()*line.a() + line.c(),
-                                      -source.y()*line.b()) == EQUAL);
+    CGAL_precondition(CGAL::compare(source.x() * line.a() + line.c(),
+                                    -source.y() * line.b()) == EQUAL);
 
-    CGAL_precondition (CGAL::compare (target.x()*line.a() + line.c(),
-                                      -target.y()*line.b()) == EQUAL);
+    CGAL_precondition(CGAL::compare(target.x() * line.a() + line.c(),
+                                    -target.y() * line.b()) == EQUAL);
   }
 
-  /*!
-   * Constructor from a circle.
+  /*! Constructor from a circle.
    * \param circ The circle.
    */
-  _Circle_segment_2 (const Circle_2& circ) :
-    _circ (circ),
-    _is_full (true),
-    _has_radius (false),
-    _orient (circ.orientation())
-  {
-    CGAL_assertion (_orient != COLLINEAR);
-  }
+  _Circle_segment_2(const Circle_2& circ) :
+    m_circ(circ),
+    m_is_full(true),
+    m_has_radius(false),
+    m_orient(circ.orientation())
+  { CGAL_assertion(m_orient != COLLINEAR); }
 
-  /*!
-   * Constructor from a circle.
+  /*! Constructor from a circle.
    * \param c The circle center.
    * \param r The radius.
    * \param orient The orientation of the circle.
    */
-  _Circle_segment_2 (const typename Kernel::Point_2& c,
-                     const NT& r,
-                     Orientation orient = COUNTERCLOCKWISE) :
-    _circ (c, r*r, orient),
-    _is_full (true),
-    _has_radius (true),
-    _radius (r),
-    _orient (orient)
-  {
-    CGAL_assertion (orient != COLLINEAR);
-  }
+  _Circle_segment_2(const typename Kernel::Point_2& c, const NT& r,
+                    Orientation orient = COUNTERCLOCKWISE) :
+    m_circ(c, r*r, orient),
+    m_is_full(true),
+    m_has_radius(true),
+    m_radius(r),
+    m_orient(orient)
+  { CGAL_assertion (orient != COLLINEAR); }
 
-  /*!
-   * Constructor of a circular arc, given a supporting circle and two
+  /*! Constructor of a circular arc, given a supporting circle and two
    * endpoints, which need not necessarily have rational coordinates.
    * The orientation of the circle determines the orientation of the arc.
    * \param circ The supporting circle.
@@ -314,30 +275,29 @@ public:
    * \param target The target point.
    * \pre Both endpoints lie on the supporting circle.
    */
-  _Circle_segment_2 (const Circle_2& circ,
-                     const Point_2& source, const Point_2& target) :
-    _circ (circ),
-    _is_full (false),
-    _has_radius (false),
-    _source (source),
-    _target (target),
-    _orient (circ.orientation())
+  _Circle_segment_2(const Circle_2& circ,
+                    const Point_2& source, const Point_2& target) :
+    m_circ(circ),
+    m_is_full(false),
+    m_has_radius(false),
+    m_source(source),
+    m_target(target),
+    m_orient(circ.orientation())
   {
-    CGAL_assertion (_orient != COLLINEAR);
+    CGAL_assertion(m_orient != COLLINEAR);
 
     CGAL_precondition
-      (CGAL::compare (CGAL::square (source.x() - circ.center().x()),
-                      circ.squared_radius() -
-                      CGAL::square (source.y() - circ.center().y())) == EQUAL);
+      (CGAL::compare(CGAL::square(source.x() - circ.center().x()),
+                     circ.squared_radius() -
+                     CGAL::square(source.y() - circ.center().y())) == EQUAL);
 
     CGAL_precondition
-      (CGAL::compare (CGAL::square (target.x() - circ.center().x()),
-                      circ.squared_radius() -
-                      CGAL::square (target.y() - circ.center().y())) == EQUAL);
+      (CGAL::compare(CGAL::square(target.x() - circ.center().x()),
+                     circ.squared_radius() -
+                     CGAL::square(target.y() - circ.center().y())) == EQUAL);
   }
 
-  /*!
-   * Constructor of a circular arc, given a supporting circle and two
+  /*! Constructor of a circular arc, given a supporting circle and two
    * endpoints, which need not necessarily have rational coordinates.
    * \param c The circle center.
    * \param r The radius.
@@ -346,86 +306,83 @@ public:
    * \param target The target point.
    * \pre Both endpoints lie on the supporting circle.
    */
-  _Circle_segment_2 (const typename Kernel::Point_2& c,
-                     const NT& r, Orientation orient,
-                     const Point_2& source, const Point_2& target) :
-    _circ (c, r*r, orient),
-    _is_full (false),
-    _has_radius (true),
-    _radius (r),
-    _source (source),
-    _target (target),
-    _orient (orient)
+  _Circle_segment_2(const typename Kernel::Point_2& c,
+                    const NT& r, Orientation orient,
+                    const Point_2& source, const Point_2& target) :
+    m_circ(c, r*r, orient),
+    m_is_full(false),
+    m_has_radius(true),
+    m_radius(r),
+    m_source(source),
+    m_target(target),
+    m_orient(orient)
   {
-    CGAL_assertion (orient != COLLINEAR);
+    CGAL_assertion(orient != COLLINEAR);
 
     CGAL_precondition
-      (CGAL::compare (CGAL::square (source.x() - c.x()),
-                      CGAL::square (r) -
-                      CGAL::square (source.y() - c.y())) == EQUAL);
+      (CGAL::compare(CGAL::square(source.x() - c.x()),
+                     CGAL::square(r) -
+                     CGAL::square(source.y() - c.y())) == EQUAL);
 
     CGAL_precondition
-      (CGAL::compare (CGAL::square (target.x() - c.x()),
-                      CGAL::square (r) -
-                      CGAL::square (target.y() - c.y())) == EQUAL);
+      (CGAL::compare(CGAL::square(target.x() - c.x()),
+                     CGAL::square(r) -
+                     CGAL::square(target.y() - c.y())) == EQUAL);
   }
 
-  /*!
-   * Constructor of a circular arc, from the given three points, in case of
+  /*! Constructor of a circular arc, from the given three points, in case of
    * three collinear points, a segment will be constructed.
    * \param p1 The arc source.
    * \param p2 A point in the interior of the arc.
    * \param p3 The arc target.
    * \pre p1 and p3 are not equal.
    */
-   _Circle_segment_2 (const typename Kernel::Point_2& p1,
-                      const typename Kernel::Point_2& p2,
-                      const typename Kernel::Point_2& p3) :
-     _is_full(false),
-     _has_radius(false),
-     _source(p1.x(), p1.y()),
-     _target(p3.x(), p3.y())
+   _Circle_segment_2(const typename Kernel::Point_2& p1,
+                     const typename Kernel::Point_2& p2,
+                     const typename Kernel::Point_2& p3) :
+     m_is_full(false),
+     m_has_radius(false),
+     m_source(p1.x(), p1.y()),
+     m_target(p3.x(), p3.y())
   {
     // Set the source and target.
-    NT          x1 = p1.x();
-    NT          y1 = p1.y();
-    NT          x2 = p2.x();
-    NT          y2 = p2.y();
-    NT          x3 = p3.x();
-    NT          y3 = p3.y();
-
+    NT x1 = p1.x();
+    NT y1 = p1.y();
+    NT x2 = p2.x();
+    NT y2 = p2.y();
+    NT x3 = p3.x();
+    NT y3 = p3.y();
 
     // Make sure that the source and the target are not the same.
-    CGAL_precondition (Kernel().compare_xy_2_object() (p1, p3) != EQUAL);
+    CGAL_precondition(Kernel().compare_xy_2_object()(p1, p3) != EQUAL);
 
     // Compute the lines: A1*x + B1*y + C1 = 0,
     //               and: A2*x + B2*y + C2 = 0,
     // where:
-    const NT  _two  = 2;
+    const NT _two  = 2;
 
-    const NT  A1 = _two*(x1 - x2);
-    const NT  B1 = _two*(y1 - y2);
-    const NT  C1 = CGAL::square(y2) - CGAL::square(y1) +
-                   CGAL::square(x2) - CGAL::square(x1);
-
-    const NT  A2 = _two*(x2 - x3);
-    const NT  B2 = _two*(y2 - y3);
-    const NT  C2 = CGAL::square(y3) - CGAL::square(y2) +
-                   CGAL::square(x3) - CGAL::square(x2);
+    const NT A1 = _two*(x1 - x2);
+    const NT B1 = _two*(y1 - y2);
+    const NT C1 =
+      CGAL::square(y2) - CGAL::square(y1) + CGAL::square(x2) - CGAL::square(x1);
+
+    const NT A2 = _two*(x2 - x3);
+    const NT B2 = _two*(y2 - y3);
+    const NT C2 =
+      CGAL::square(y3) - CGAL::square(y2) + CGAL::square(x3) - CGAL::square(x2);
 
     // Compute the coordinates of the intersection point between the
     // two lines, given by (Nx / D, Ny / D), where:
-    const NT  Nx = B1*C2 - B2*C1;
-    const NT  Ny = A2*C1 - A1*C2;
-    const NT  D = A1*B2 - A2*B1;
+    const NT Nx = B1*C2 - B2*C1;
+    const NT Ny = A2*C1 - A1*C2;
+    const NT D = A1*B2 - A2*B1;
 
     // Make sure the three points are not collinear.
-    const bool  points_collinear = (CGAL::sign (D) == ZERO);
+    const bool points_collinear = (CGAL::sign (D) == ZERO);
 
-    if (points_collinear)
-    {
-      _line  = Line_2(p1, p3);
-      _orient = COLLINEAR;
+    if (points_collinear) {
+      m_line  = Line_2(p1, p3);
+      m_orient = COLLINEAR;
       return;
     }
 
@@ -436,145 +393,118 @@ public:
 
     typename Kernel::Point_2 circ_center(x_center, y_center);
 
-
-
-    NT sqr_rad = (CGAL::square(D*x2 - Nx) + CGAL::square(D*y2 - Ny)) /
-                 CGAL::square(D);
+    NT sqr_rad =
+      (CGAL::square(D*x2 - Nx) + CGAL::square(D*y2 - Ny)) / CGAL::square(D);
 
     // Determine the orientation: If the mid-point forms a left-turn with
     // the source and the target points, the orientation is positive (going
     // counterclockwise).
     // Otherwise, it is negative (going clockwise).
-    Kernel                         ker;
+    Kernel ker;
     typename Kernel::Orientation_2 orient_f = ker.orientation_2_object();
 
-    if (orient_f(p1, p2, p3) == LEFT_TURN)
-      _orient = COUNTERCLOCKWISE;
-    else
-      _orient = CLOCKWISE;
-
-     _circ = Circle_2(circ_center, sqr_rad, _orient);
+    if (orient_f(p1, p2, p3) == LEFT_TURN) m_orient = COUNTERCLOCKWISE;
+    else m_orient = CLOCKWISE;
+     m_circ = Circle_2(circ_center, sqr_rad, m_orient);
   }
 
-  /*!
-   * Get the orientation of the curve.
+  /*! Get the orientation of the curve.
    * \return COLLINEAR in case of a line segment,
    *         CLOCKWISE or COUNTERCLOCKWISE for circular curves.
    */
-  inline Orientation orientation () const
-  {
-    return (_orient);
-  }
+  inline Orientation orientation() const { return (m_orient); }
 
   /*! Check if the arc is linear. */
-  inline bool is_linear () const
-  {
-    return (_orient == COLLINEAR);
-  }
+  inline bool is_linear() const { return (m_orient == COLLINEAR); }
 
   /*! Check if the arc is circular. */
-  inline bool is_circular () const
-  {
-    return (_orient != COLLINEAR);
-  }
+  inline bool is_circular() const { return (m_orient != COLLINEAR); }
 
-  /*!
-   * Get the supporting line.
+  /*! Obtain the supporting line.
    * \pre The curve orientation is COLLINEAR.
    */
-  const Line_2& supporting_line () const
+  const Line_2& supporting_line() const
   {
-    CGAL_precondition (_orient == COLLINEAR);
-    return (_line);
+    CGAL_precondition(m_orient == COLLINEAR);
+    return m_line;
   }
 
-  /*!
-   * Get the supporting circle.
+  /*! Obtain the supporting circle.
    * \pre The curve orientation is not COLLINEAR.
    */
-  const Circle_2& supporting_circle () const
+  const Circle_2& supporting_circle() const
   {
-    CGAL_precondition (_orient != COLLINEAR);
-    return (_circ);
+    CGAL_precondition(m_orient != COLLINEAR);
+    return m_circ;
   }
 
   /*! Check if the curve is a full circle. */
-  bool is_full () const
-  {
-    return (_is_full);
-  }
+  bool is_full() const { return (m_is_full); }
 
   /*!
    * Get the source point.
    * \pre The curve is not a full circle.
    */
-  const Point_2& source () const
+  const Point_2& source() const
   {
-    CGAL_precondition (! _is_full);
-    return (_source);
+    CGAL_precondition(! m_is_full);
+    return (m_source);
   }
 
   /*!
    * Get the target point.
    * \pre The curve is not a full circle.
    */
-  const Point_2& target () const
+  const Point_2& target() const
   {
-    CGAL_precondition (! _is_full);
-    return (_target);
+    CGAL_precondition(! m_is_full);
+    return (m_target);
   }
 
-  /*!
-   * Get the vertical tangency points the arc contains.
+  /*! Get the vertical tangency points the arc contains.
    * \param vpts Output: The vertical tangency points.
    * \pre The curve is circular.
    * \return The number of points (0, 1, or 2).
    */
-  unsigned int vertical_tangency_points (Point_2 *vpts) const
+  unsigned int vertical_tangency_points(Point_2* vpts) const
   {
-    CGAL_precondition (_orient != COLLINEAR);
-    unsigned int  n_vpts = 0;
+    CGAL_precondition(m_orient != COLLINEAR);
+    unsigned int n_vpts = 0;
 
-    if (_is_full)
-    {
+    if (m_is_full) {
       // In case of a full circle, create both vertical tangency points:
-      const NT&     x0 = _circ.center().x();
-      const NT&     y0 = _circ.center().y();
-      CoordNT       xv_left;
-      CoordNT       xv_right;
+      const NT& x0 = m_circ.center().x();
+      const NT& y0 = m_circ.center().y();
+      CoordNT xv_left;
+      CoordNT xv_right;
 
-      if (_has_radius)
-      {
+      if (m_has_radius) {
         // In case the radius is explicitly given:
-        xv_left = CoordNT (x0 - _radius);
-        xv_right = CoordNT (x0 + _radius);
+        xv_left = CoordNT(x0 - m_radius);
+        xv_right = CoordNT(x0 + m_radius);
       }
-      else
-      {
+      else {
         // In case only the squared root is given:
-        xv_left = CoordNT (x0, NT(-1), _circ.squared_radius());
-        xv_right = CoordNT (x0, NT(1), _circ.squared_radius());
+        xv_left = CoordNT(x0, NT(-1), m_circ.squared_radius());
+        xv_right = CoordNT(x0, NT(1), m_circ.squared_radius());
       }
 
-      vpts[0] = Point_2 (xv_left, y0);
-      vpts[1] = Point_2 (xv_right, y0);
+      vpts[0] = Point_2(xv_left, y0);
+      vpts[1] = Point_2(xv_right, y0);
       return (2);
     }
 
-    if (_orient == COUNTERCLOCKWISE)
-    {
+    if (m_orient == COUNTERCLOCKWISE) {
       // Compute the vertical tangency points for the arc:
-      n_vpts = _ccw_vertical_tangency_points (_source, _target, vpts);
+      n_vpts = _ccw_vertical_tangency_points(m_source, m_target, vpts);
     }
-    else
-    {
+    else {
       // Compute the vertical tangency points for the opposite arc:
-      n_vpts = _ccw_vertical_tangency_points (_target, _source, vpts);
+      n_vpts = _ccw_vertical_tangency_points(m_target, m_source, vpts);
 
       // Swap their order, if necessary.
-      if (n_vpts == 2)
-      {
-        Point_2   temp = vpts[0];
+      if (n_vpts == 2) {
+        Point_2 temp = vpts[0];
         vpts[0] = vpts[1];
         vpts[1] = temp;
       }
@@ -584,69 +514,62 @@ public:
   }
 
 private:
-
-  /*!
-   * Get the vertical tangency points the arc contains, assuming it is
+  /*! Obtain the vertical tangency points the arc contains, assuming it is
    * counterclockwise oriented.
    * \param vpts Output: The vertical tangency points.
    * \return The number of points (0, 1, or 2).
    */
-  unsigned int _ccw_vertical_tangency_points (const Point_2& src,
-                                              const Point_2& trg,
-                                              Point_2 *vpts) const
-  {
-    unsigned int  n_vpts = 0;
-    const NT&     x0 = _circ.center().x();
-    const NT&     y0 = _circ.center().y();
-    int           qs = _quart_index (src);
-    int           qt = _quart_index (trg);
+  unsigned int _ccw_vertical_tangency_points(const Point_2& src,
+                                             const Point_2& trg,
+                                             Point_2* vpts) const
+  {
+    unsigned int n_vpts = 0;
+    const NT& x0 = m_circ.center().x();
+    const NT& y0 = m_circ.center().y();
+    int qs = _quart_index(src);
+    int qt = _quart_index(trg);
 
-    if (qs == qt)
-    {
-      if ((qs == 0 || qs == 1) && CGAL::compare (src.x(), trg.x()) == LARGER)
+    if (qs == qt) {
+      if ((qs == 0 || qs == 1) && CGAL::compare(src.x(), trg.x()) == LARGER)
         // We have an x-monotone arc lying on the upper half of the circle:
         return (0);
 
-      if ((qs == 2 || qs == 3) && CGAL::compare (src.x(), trg.x()) == SMALLER)
+      if ((qs == 2 || qs == 3) && CGAL::compare(src.x(), trg.x()) == SMALLER)
         // We have an x-monotone arc lying on the lower half of the circle:
         return (0);
     }
 
     // Make sure the target quarter is larger than the source quarter, by
     // adding 4 to its index, if necessary.
-    if (qt <= qs)
-      qt += 4;
+    if (qt <= qs) qt += 4;
 
     // Start traversing the quarter-planes and collect the vertical tangency
     // points we encounter.
-    while (qs < qt)
-    {
-      if ((qs % 4) == 1)
-      {
+    while (qs < qt) {
+      if ((qs % 4) == 1) {
         // We collect the left tangency point when going from Q[1] to Q[2]:
-        if (CGAL::compare (x0, trg.x()) != LARGER ||
-            CGAL::compare (y0, trg.y()) != EQUAL)
+        if (CGAL::compare(x0, trg.x()) != LARGER ||
+            CGAL::compare(y0, trg.y()) != EQUAL)
         {
-          if (_has_radius)
-            vpts[n_vpts] = Point_2 (CoordNT (x0 - _radius), y0);
+          if (m_has_radius)
+            vpts[n_vpts] = Point_2(CoordNT(x0 - m_radius), y0);
           else
-            vpts[n_vpts] = Point_2 (CoordNT (x0, NT(-1), _circ.squared_radius()),
-                                             y0);
+            vpts[n_vpts] =
+              Point_2(CoordNT(x0, NT(-1), m_circ.squared_radius()), y0);
 
           n_vpts++;
         }
       }
-      else if ((qs % 4) == 3)
-      {
+      else if ((qs % 4) == 3) {
         // We collect the right tangency point when going from Q[3] to Q[0]:
-        if (CGAL::compare (x0, trg.x()) != SMALLER ||
-            CGAL::compare (y0, trg.y()) != EQUAL)
+        if (CGAL::compare(x0, trg.x()) != SMALLER ||
+            CGAL::compare(y0, trg.y()) != EQUAL)
         {
-          if (_has_radius)
-            vpts[n_vpts] = Point_2 (CoordNT (x0 + _radius), y0);
+          if (m_has_radius)
+            vpts[n_vpts] = Point_2(CoordNT(x0 + m_radius), y0);
           else
-            vpts[n_vpts] = Point_2 (CoordNT (x0, NT(1), _circ.squared_radius()),
-                                             y0);
+            vpts[n_vpts] =
+              Point_2(CoordNT(x0, NT(1), m_circ.squared_radius()), y0);
           n_vpts++;
         }
       }
@@ -657,11 +580,10 @@ private:
     return (n_vpts);
   }
 
-  /*!
-   * Get the index of the quarter-plane containing the given point,
+  /*! Obtain the index of the quarter-plane containing the given point,
    * where the circle center is considered to be the origin.
    */
-  int _quart_index (const Point_2& p) const
+  int _quart_index(const Point_2& p) const
   {
     // The plane looks like:
     //
@@ -673,44 +595,32 @@ private:
     //      x <  0  |   x >= 0
     //      y <= 0  |   y <  0
     //
-    const CGAL::Sign   sign_x = CGAL::sign (p.x() - _circ.center().x());
-    const CGAL::Sign   sign_y = CGAL::sign (p.y() - _circ.center().y());
+    const CGAL::Sign sign_x = CGAL::sign(p.x() - m_circ.center().x());
+    const CGAL::Sign sign_y = CGAL::sign(p.y() - m_circ.center().y());
 
-    if (sign_x == POSITIVE)
-    {
-      return ((sign_y == NEGATIVE) ? 3 : 0);
-    }
-    else if (sign_x == NEGATIVE)
-    {
-      return ((sign_y == POSITIVE) ? 1 : 2);
-    }
+    if (sign_x == POSITIVE) return ((sign_y == NEGATIVE) ? 3 : 0);
+    else if (sign_x == NEGATIVE) return ((sign_y == POSITIVE) ? 1 : 2);
 
     CGAL_assertion (sign_y != ZERO);
     return ((sign_y == POSITIVE) ? 1 : 3);
   }
 };
 
-/*!
- * Exporter for line segments and circular arcs.
+/*! Exporter for line segments and circular arcs.
  */
-template <class Kernel, bool Filter>
+template <typename Kernel, bool Filter>
 std::ostream&
-operator<< (std::ostream& os,
-            const _Circle_segment_2<Kernel, Filter>& c)
+operator<<(std::ostream& os, const _Circle_segment_2<Kernel, Filter>& c)
 {
-  if (c.orientation() == COLLINEAR)
-  {
+  if (c.orientation() == COLLINEAR) {
     os<< "segment: " << c.source() << " -> " << c.target();
   }
-  else
-  {
-    if(!c.is_full())
-    {
+  else {
+    if (! c.is_full()) {
       os << "circular arc: " << c.supporting_circle() << ' '
          << c.source() << " -> " << c.target();
     }
-    else
-    {
+    else {
       os << "circular arc: " << c.supporting_circle();
     }
   }
@@ -721,11 +631,9 @@ operator<< (std::ostream& os,
 /*! \class
  * Representation of an x-monotone circular arc.
  */
-template <class Kernel_, bool Filter_>
-class _X_monotone_circle_segment_2
-{
+template <typename Kernel_, bool Filter_>
+class _X_monotone_circle_segment_2 {
 public:
-
   typedef Kernel_                                          Kernel;
   typedef _X_monotone_circle_segment_2<Kernel, Filter_>    Self;
   typedef typename Kernel::FT                              NT;
@@ -735,17 +643,16 @@ public:
   typedef typename Point_2::CoordNT                        CoordNT;
 
   // Type definition for the intersection points mapping.
-  typedef std::pair<unsigned int, unsigned int>   Curve_id_pair;
-  typedef unsigned int                            Multiplicity;
-  typedef std::pair<Point_2,Multiplicity>         Intersection_point_2;
-  typedef std::list<Intersection_point_2>         Intersection_list;
+  typedef std::pair<unsigned int, unsigned int>         Curve_id_pair;
+  typedef unsigned int                                  Multiplicity;
+  typedef std::pair<Point_2, Multiplicity>              Intersection_point;
+  typedef std::list<Intersection_point>                 Intersection_list;
 
   /*!
    * \struct Less functor for Curve_id_pair.
    */
-  struct Less_id_pair
-  {
-    bool operator() (const Curve_id_pair& ip1, const Curve_id_pair& ip2) const
+  struct Less_id_pair {
+    bool operator()(const Curve_id_pair& ip1, const Curve_id_pair& ip2) const
     {
       // Compare the pairs of IDs lexicographically.
       return (ip1.first < ip2.first ||
@@ -753,25 +660,20 @@ public:
     }
   };
 
-  typedef std::map<Curve_id_pair,
-                   Intersection_list,
-                   Less_id_pair>                  Intersection_map;
+  typedef std::map<Curve_id_pair, Intersection_list, Less_id_pair>
+                                                  Intersection_map;
   typedef typename Intersection_map::value_type   Intersection_map_entry;
   typedef typename Intersection_map::iterator     Intersection_map_iterator;
 
 protected:
-
-  NT           _first;       // The x-coordinate of the circle center.
-                             // Or: the coefficient of x in the line equation.
-
-  NT           _second;      // The y-coordinate of the circle center.
-                             // Or: the coefficient of y in the line equation.
-
-  NT           _third;       // The squared radius of the supporting circle.
-                             // Or: the free coefficient in the line equation.
-
-  Point_2      _source;      // The source point.
-  Point_2      _target;      // The target point.
+  NT m_first;           // The x-coordinate of the circle center.
+                        // Or: the coefficient of x in the line equation.
+  NT m_second;          // The y-coordinate of the circle center.
+                        // Or: the coefficient of y in the line equation.
+  NT m_third;           // The squared radius of the supporting circle.
+                        // Or: the free coefficient in the line equation.
+  Point_2 m_source;     // The source point.
+  Point_2 m_target;     // The target point.
 
   enum {
     IS_DIRECTED_RIGHT_MASK = 1,
@@ -782,7 +684,7 @@ protected:
     INDEX_SHIFT_BITS = 4
   };
 
-  unsigned int _info;        // A bit vector, where:
+  unsigned int m_info;       // A bit vector, where:
                              // Bit 0 (the LSB): marks if the arc is directed
                              //                  from left to right.
                              // Bit 1: marks if the arc is a vertical segment.
@@ -790,408 +692,321 @@ protected:
                              // The rest of the bits represent the curve index.
 
 public:
-
-  /*!
-   * Default constructor.
+  /*! Default constructor.
    */
-  _X_monotone_circle_segment_2 () :
-    _first(),
-    _second(),
-    _third(),
-    _source(),
-    _target(),
-    _info (0)
+  _X_monotone_circle_segment_2() :
+    m_first(),
+    m_second(),
+    m_third(),
+    m_source(),
+    m_target(),
+    m_info(0)
   {}
 
-  /*!
-   * Construct an arc from a line segment.
+  /*! Construct an arc from a line segment.
    * \param line The supporting line.
    * \param source The source point.
    * \param target The target point.
    */
-  _X_monotone_circle_segment_2 (const Line_2& line,
-                                const Point_2& source, const Point_2& target,
-                                unsigned int index = 0) :
-    _first (line.a()),
-    _second (line.b()),
-    _third (line.c()),
-    _source (source),
-    _target(target),
-    _info (index << INDEX_SHIFT_BITS)
+  _X_monotone_circle_segment_2(const Line_2& line,
+                               const Point_2& source, const Point_2& target,
+                               unsigned int index = 0) :
+    m_first(line.a()),
+    m_second(line.b()),
+    m_third(line.c()),
+    m_source(source),
+    m_target(target),
+    m_info(index << INDEX_SHIFT_BITS)
   {
     // Check if the segment is directed left or right:
-    Comparison_result   res = CGAL::compare (source.x(), target.x());
+    Comparison_result res = CGAL::compare(source.x(), target.x());
 
-    if (res == EQUAL)
-    {
-      CGAL_precondition (CGAL::sign(_second) == ZERO);
+    if (res == EQUAL) {
+      CGAL_precondition(CGAL::sign(m_second) == ZERO);
 
       // We have a vertical segment - compare the points by their
       // y-coordinates:
-      _info = (_info | IS_VERTICAL_SEGMENT_MASK);
-      res = CGAL::compare (source.y(), target.y());
+      m_info = (m_info | IS_VERTICAL_SEGMENT_MASK);
+      res = CGAL::compare(source.y(), target.y());
     }
 
-    CGAL_precondition (res != EQUAL);
-    if (res == SMALLER)
-      _info = (_info | IS_DIRECTED_RIGHT_MASK);
+    CGAL_precondition(res != EQUAL);
+    if (res == SMALLER) m_info = (m_info | IS_DIRECTED_RIGHT_MASK);
   }
 
-  /*!
-   * Construct a segment arc from two kernel points
+  /*! Construct a segment arc from two kernel points
    * \param source the source point.
    * \ param target the target point.
    * \pre source and target are not equal.
    */
-  _X_monotone_circle_segment_2 (const typename Kernel::Point_2& source,
-                                const typename Kernel::Point_2& target) :
-    _source(source.x(), source.y()),
-    _target(target.x(), target.y()),
-    _info (0)
+  _X_monotone_circle_segment_2(const typename Kernel::Point_2& source,
+                               const typename Kernel::Point_2& target) :
+    m_source(source.x(), source.y()),
+    m_target(target.x(), target.y()),
+    m_info(0)
   {
     Line_2 line(source, target);
-    _first  = line.a();
-    _second = line.b();
-    _third  = line.c();
+    m_first  = line.a();
+    m_second = line.b();
+    m_third  = line.c();
 
     // Check if the segment is directed left or right:
-    Comparison_result   res = CGAL::compare (source.x(), target.x());
+    Comparison_result res = CGAL::compare(source.x(), target.x());
 
-    if (res == EQUAL)
-    {
-      CGAL_precondition (CGAL::sign(_second) == ZERO);
+    if (res == EQUAL) {
+      CGAL_precondition(CGAL::sign(m_second) == ZERO);
 
       // We have a vertical segment - compare the points by their
       // y-coordinates:
-      _info = (_info | IS_VERTICAL_SEGMENT_MASK);
-      res = CGAL::compare (source.y(), target.y());
+      m_info = (m_info | IS_VERTICAL_SEGMENT_MASK);
+      res = CGAL::compare(source.y(), target.y());
     }
 
-    CGAL_precondition (res != EQUAL);
-    if (res == SMALLER)
-      _info = (_info | IS_DIRECTED_RIGHT_MASK);
+    CGAL_precondition(res != EQUAL);
+    if (res == SMALLER) m_info = (m_info | IS_DIRECTED_RIGHT_MASK);
   }
 
-  /*!
-   * Construct a circular arc.
+  /*! Construct a circular arc.
    * \param line The supporting line.
    * \param source The source point.
    * \param target The target point.
    * \param orient The orientation of the arc.
    */
-  _X_monotone_circle_segment_2 (const Circle_2& circ,
-                                const Point_2& source, const Point_2& target,
-                                Orientation orient,
-                                unsigned int index = 0) :
-    _first (circ.center().x()),
-    _second (circ.center().y()),
-    _third (circ.squared_radius()),
-    _source (source),
-    _target(target),
-    _info (index << INDEX_SHIFT_BITS)
+  _X_monotone_circle_segment_2(const Circle_2& circ,
+                               const Point_2& source, const Point_2& target,
+                               Orientation orient,
+                               unsigned int index = 0) :
+    m_first(circ.center().x()),
+    m_second(circ.center().y()),
+    m_third(circ.squared_radius()),
+    m_source(source),
+    m_target(target),
+    m_info(index << INDEX_SHIFT_BITS)
   {
     // Check if the segment is directed left or right:
-    Comparison_result   res = CGAL::compare (source.x(), target.x());
+    Comparison_result res = CGAL::compare (source.x(), target.x());
 
-    CGAL_precondition (res != EQUAL);
-    if (res == SMALLER)
-      _info = (_info | IS_DIRECTED_RIGHT_MASK);
+    CGAL_precondition(res != EQUAL);
+    if (res == SMALLER) m_info = (m_info | IS_DIRECTED_RIGHT_MASK);
 
     // Set the orientation.
     CGAL_precondition (orient != COLLINEAR);
-    if (orient == COUNTERCLOCKWISE)
-      _info = (_info | COUNTERCLOCKWISE_CODE);
-    else
-      _info = (_info | CLOCKWISE_CODE);
+    if (orient == COUNTERCLOCKWISE) m_info = (m_info | COUNTERCLOCKWISE_CODE);
+    else m_info = (m_info | CLOCKWISE_CODE);
   }
 
   /*! Check if the arc is linear. */
-  inline bool is_linear () const
-  {
-    return ((_info & ORIENTATION_MASK) == 0);
-  }
+  inline bool is_linear () const { return ((m_info & ORIENTATION_MASK) == 0); }
 
   /*! Check if the arc is circular. */
   inline bool is_circular () const
-  {
-    return ((_info & ORIENTATION_MASK) != 0);
-  }
+  { return ((m_info & ORIENTATION_MASK) != 0); }
 
-  /*!
-   * Get the supporting line.
+  /*! Obtain the supporting line.
    * \pre The arc is linear (a line segment).
    */
-  Line_2 supporting_line () const
+  Line_2 supporting_line() const
   {
     CGAL_precondition (is_linear());
-
     return (Line_2 (a(), b(), c()));
   }
 
-  /*!
-   * Get the supporting circle.
+  /*! Obtain the supporting circle.
    * \pre The arc is circular.
    */
-  Circle_2 supporting_circle () const
+  Circle_2 supporting_circle() const
   {
     CGAL_precondition (is_circular());
 
-    typename Kernel::Point_2  center (x0(), y0());
-    return (Circle_2 (center , sqr_r(), orientation()));
+    typename Kernel::Point_2 center(x0(), y0());
+    return (Circle_2(center , sqr_r(), orientation()));
   }
 
   /*! Get the source point. */
-  inline const Point_2& source () const
-  {
-    return (_source);
-  }
+  inline const Point_2& source() const { return (m_source); }
 
   /*! Get the target point. */
-  inline const Point_2& target () const
-  {
-    return (_target);
-  }
+  inline const Point_2& target() const { return (m_target); }
 
   /*! True if the arc is directed right, false otherwise. */
-  bool is_directed_right () const
-  {
-    return ((_info & IS_DIRECTED_RIGHT_MASK) != 0);
-  }
+  bool is_directed_right() const
+  { return ((m_info & IS_DIRECTED_RIGHT_MASK) != 0); }
 
-  bool has_left() const
-  {
-    return true;
-  }
+  bool has_left() const { return true; }
 
-  bool has_right() const
-  {
-    return true;
-  }
+  bool has_right() const { return true; }
 
   /*! Get the left endpoint of the arc. */
-  inline const Point_2& left () const
-  {
-    return (((_info & IS_DIRECTED_RIGHT_MASK) != 0) ? _source : _target);
-  }
+  inline const Point_2& left() const
+  { return (((m_info & IS_DIRECTED_RIGHT_MASK) != 0) ? m_source : m_target); }
 
   /*! Get the right endpoint of the arc. */
-  inline const Point_2& right () const
-  {
-    return (((_info & IS_DIRECTED_RIGHT_MASK) != 0) ? _target : _source);
-  }
+  inline const Point_2& right() const
+  { return (((m_info & IS_DIRECTED_RIGHT_MASK) != 0) ? m_target : m_source); }
 
-  /*!
-   * Check whether the given point is in the x-range of the arc.
+  /*! Check whether the given point is in the x-range of the arc.
    */
-  bool is_in_x_range (const Point_2& p) const
+  bool is_in_x_range(const Point_2& p) const
   {
-    Comparison_result    res = CGAL::compare (p.x(), left().x());
+    Comparison_result res = CGAL::compare (p.x(), left().x());
 
-    if (res == SMALLER)
-      return (false);
-    else if (res == EQUAL)
-      return (true);
+    if (res == SMALLER) return false;
+    else if (res == EQUAL) return true;
 
     return (CGAL::compare (p.x(), right().x()) != LARGER);
   }
 
   /*! Check if the arc is a vertical segment. */
-  inline bool is_vertical () const
-  {
-    return ((_info & IS_VERTICAL_SEGMENT_MASK) != 0);
-  }
+  inline bool is_vertical() const
+  { return ((m_info & IS_VERTICAL_SEGMENT_MASK) != 0); }
 
   /*! Get the orientation of the arc. */
   inline Orientation orientation() const
   {
-    unsigned int   _or = (_info & ORIENTATION_MASK);
-
-    if (_or == COUNTERCLOCKWISE_CODE)
-      return (CGAL::COUNTERCLOCKWISE);
-    else if (_or == CLOCKWISE_CODE)
-      return (CGAL::CLOCKWISE);
+    unsigned int or_ = (m_info & ORIENTATION_MASK);
+    if (or_ == COUNTERCLOCKWISE_CODE) return (CGAL::COUNTERCLOCKWISE);
+    else if (or_ == CLOCKWISE_CODE) return (CGAL::CLOCKWISE);
 
-    CGAL_assertion (_or == 0);
+    CGAL_assertion(or_ == 0);
     return (CGAL::COLLINEAR);
   }
 
-  /*!
-   * Check the position of a given point with respect to the arc.
+  /*! Check the position of a given point with respect to the arc.
    */
-  Comparison_result point_position (const Point_2& p) const
+  Comparison_result point_position(const Point_2& p) const
   {
-    if (is_linear())
-      return (_line_point_position (p));
-    else
-      return (_circ_point_position (p));
+    if (is_linear()) return (_line_point_position(p));
+    else return (_circ_point_position (p));
   }
 
-
-  /*!
-   * Compare the two arcs to the right of their intersection point.
+  /*! Compare the two arcs to the right of their intersection point.
    */
-  Comparison_result compare_to_right (const Self& cv, const Point_2& p) const
+  Comparison_result compare_to_right(const Self& cv, const Point_2& p) const
   {
-    if (is_linear())
-    {
-      if (cv.is_linear())
-        return (_lines_compare_to_right (cv, p));
-
-      Comparison_result   res = cv._circ_line_compare_to_right (*this, p);
-
-      if (res != EQUAL)
-        res = (res == SMALLER) ? LARGER : SMALLER;
-
+    if (is_linear()) {
+      if (cv.is_linear()) return (_lines_compare_to_right (cv, p));
+      Comparison_result res = cv._circ_line_compare_to_right (*this, p);
+      if (res != EQUAL) res = (res == SMALLER) ? LARGER : SMALLER;
       return (res);
     }
-    else
-    {
-      if (cv.is_linear())
-        return (_circ_line_compare_to_right (cv, p));
-
+    else {
+      if (cv.is_linear()) return (_circ_line_compare_to_right (cv, p));
       return (_circs_compare_to_right (cv, p));
     }
   }
 
-  /*!
-   * Compare the two arcs to the left of their intersection point.
+  /*! Compare the two arcs to the left of their intersection point.
    */
-  Comparison_result compare_to_left (const Self& cv, const Point_2& p) const
+  Comparison_result compare_to_left(const Self& cv, const Point_2& p) const
   {
-    if (is_linear())
-    {
-      if (cv.is_linear())
-        return (_lines_compare_to_left (cv, p));
-
-      Comparison_result   res = cv._circ_line_compare_to_left (*this, p);
-
-      if (res != EQUAL)
-        res = (res == SMALLER) ? LARGER : SMALLER;
-
+    if (is_linear()) {
+      if (cv.is_linear()) return (_lines_compare_to_left (cv, p));
+      Comparison_result res = cv._circ_line_compare_to_left(*this, p);
+      if (res != EQUAL) res = (res == SMALLER) ? LARGER : SMALLER;
       return (res);
     }
-    else
-    {
-      if (cv.is_linear())
-        return (_circ_line_compare_to_left (cv, p));
-
+    else {
+      if (cv.is_linear()) return (_circ_line_compare_to_left(cv, p));
       return (_circs_compare_to_left (cv, p));
     }
   }
 
-  /*!
-   * Check whether the two arcs have the same supporting curve.
+  /*! Check whether the two arcs have the same supporting curve.
    */
-  bool has_same_supporting_curve (const Self& cv) const
+  bool has_same_supporting_curve(const Self& cv) const
   {
     // Check if the curve indices are the same.
-    if (_index() != 0 && _index() == cv._index())
-      return (true);
+    if (_index() != 0 && _index() == cv._index()) return true;
 
     // Make sure that the supporting curves are of the same type.
-    if (is_linear() && ! cv.is_linear())
-      return (false);
+    if (is_linear() && ! cv.is_linear()) return false;
 
-    if (! is_linear() && cv.is_linear())
-      return (false);
+    if (! is_linear() && cv.is_linear()) return false;
 
     // Compare the curve coefficients.
-    if (! is_linear())
-    {
+    if (! is_linear()) {
       // The two circles must have the same center and the same radius.
-      return (CGAL::compare (x0(), cv.x0()) == EQUAL &&
-              CGAL::compare (y0(), cv.y0()) == EQUAL &&
-              CGAL::compare (sqr_r(), cv.sqr_r()) == EQUAL);
+      return (CGAL::compare(x0(), cv.x0()) == EQUAL &&
+              CGAL::compare(y0(), cv.y0()) == EQUAL &&
+              CGAL::compare(sqr_r(), cv.sqr_r()) == EQUAL);
     }
 
     // Compare the line equations: Note that these may be scaled.
-    NT    fact1;
-    NT    fact2;
+    NT fact1;
+    NT fact2;
 
-    if (is_vertical())
-    {
-      if (! cv.is_vertical())
-        return (false);
+    if (is_vertical()) {
+      if (! cv.is_vertical()) return false;
 
       fact1 = a();
       fact2 = cv.a();
     }
-    else
-    {
+    else {
       fact1 = b();
       fact2 = cv.b();
     }
 
-    return (CGAL::compare (fact2*a(), fact1*cv.a()) == EQUAL &&
-            CGAL::compare (fact2*b(), fact1*cv.b()) == EQUAL &&
-            CGAL::compare (fact2*c(), fact1*cv.c()) == EQUAL);
+    return (CGAL::compare(fact2*a(), fact1*cv.a()) == EQUAL &&
+            CGAL::compare(fact2*b(), fact1*cv.b()) == EQUAL &&
+            CGAL::compare(fact2*c(), fact1*cv.c()) == EQUAL);
   }
 
-  /*!
-   * Check if the two curves are equal.
+  /*! Check whether the two curves are equal.
    */
-  bool equals (const Self& cv) const
+  bool equals(const Self& cv) const
   {
-    if (! this->has_same_supporting_curve (cv))
-      return (false);
+    if (! this->has_same_supporting_curve(cv)) return false;
 
-    if (is_linear())
-    {
+    if (is_linear()) {
       // In case of line segments we can swap the source and target:
-      return ((_source.equals (cv._source) && _target.equals (cv._target)) ||
-              (_source.equals (cv._target) && _target.equals (cv._source)));
+      return ((m_source.equals(cv.m_source) && m_target.equals(cv.m_target)) ||
+              (m_source.equals(cv.m_target) && m_target.equals(cv.m_source)));
     }
 
     // Once again, opposite circular arcs are considered to be equal:
     return ((orientation() == cv.orientation() &&
-             _source.equals (cv._source) && _target.equals (cv._target)) ||
+             m_source.equals(cv.m_source) && m_target.equals(cv.m_target)) ||
             (orientation() != cv.orientation() &&
-             _source.equals (cv._target) && _target.equals (cv._source)));
+             m_source.equals(cv.m_target) && m_target.equals(cv.m_source)));
   }
 
-  /*!
-   * Split the curve at a given point into two sub-arcs.
+  /*! Split the curve at a given point into two sub-arcs.
    */
-  void split (const Point_2& p, Self& c1, Self& c2) const
+  void split(const Point_2& p, Self& c1, Self& c2) const
   {
     // Copy the properties of this arc to the sub-arcs.
     c1 = *this;
     c2 = *this;
 
     // Change the endpoint, such that c1 lies to the right of c2:
-    if (is_directed_right())
-    {
-      c1._target = p;
-      c2._source = p;
+    if (is_directed_right()) {
+      c1.m_target = p;
+      c2.m_source = p;
     }
-    else
-    {
-      c1._source = p;
-      c2._target = p;
+    else {
+      c1.m_source = p;
+      c2.m_target = p;
     }
-
-    return;
   }
 
-  /*!
-   * Compute the intersections between the two arcs or segments.
+  /*! Compute the intersections between the two arcs or segments.
    */
-  template <class OutputIterator>
-  OutputIterator intersect (const Self& cv, OutputIterator oi,
-                            Intersection_map *inter_map = nullptr) const
+  template <typename OutputIterator>
+  OutputIterator intersect(const Self& cv, OutputIterator oi,
+                           Intersection_map* inter_map = nullptr) const
   {
+    typedef std::pair<Point_2, Multiplicity>            Intersection_point;
+    typedef boost::variant<Intersection_point, Self>    Intersection_result;
+
     // First check whether the two arcs have the same supporting curve.
-    if (has_same_supporting_curve (cv))
-    {
+    if (has_same_supporting_curve(cv)) {
       // Check for overlaps between the two arcs.
-      Self    overlap;
+      Self overlap;
 
-      if (_compute_overlap (cv, overlap))
-      {
+      if (_compute_overlap(cv, overlap)) {
         // There can be just a single overlap between two x-monotone arcs:
-        *oi = CGAL::make_object (overlap);
-        ++oi;
-        return (oi);
+        *oi++ = Intersection_result(overlap);
+        return oi;
       }
 
       // In case there is not overlap and the supporting curves are the same,
@@ -1199,155 +1014,121 @@ public:
       // a common end point.
       // Note that in this case we do not define the multiplicity of the
       // intersection points we report.
-      unsigned int  mult = 0;
-      if (left().equals (cv.left()) || left().equals(cv.right()))
-      {
-        *oi = CGAL::make_object (std::make_pair (left(), mult));
-        ++oi;
+      Multiplicity mult = 0;
+      if (left().equals(cv.left()) || left().equals(cv.right())) {
+        *oi++ = Intersection_result(std::make_pair(left(), mult));
       }
 
-      if (right().equals (cv.right()) || right().equals(cv.left()))
-      {
-        *oi = CGAL::make_object (std::make_pair (right(), mult));
-        ++oi;
+      if (right().equals(cv.right()) || right().equals(cv.left())) {
+        *oi++ = Intersection_result(std::make_pair(right(), mult));
       }
 
-      return (oi);
+      return oi;
     }
 
     // Before computing the intersection points between the two supporting
     // curves, check if their intersection has already been computed and
     // cached.
-    Curve_id_pair                id_pair;
-    Intersection_map_iterator    map_iter;
-    Intersection_list            inter_list;
-    bool                         invalid_ids = false;
+    Curve_id_pair id_pair;
+    Intersection_map_iterator map_iter;
+    Intersection_list inter_list;
+    bool invalid_ids = false;
 
-    if (inter_map != nullptr && _index() != 0 && cv._index() != 0)
-    {
+    if (inter_map != nullptr && _index() != 0 && cv._index() != 0) {
       if (_index() < cv._index())
         id_pair = Curve_id_pair (_index(), cv._index());
-      else
-        id_pair = Curve_id_pair (cv._index(), _index());
+      else id_pair = Curve_id_pair (cv._index(), _index());
 
-      map_iter = inter_map->find (id_pair);
+      map_iter = inter_map->find(id_pair);
     }
-    else
-    {
+    else {
       // In case one of the IDs is invalid, we do not look in the map neither
       // we cache the results.
-      if (inter_map != nullptr)
-        map_iter = inter_map->end();
+      if (inter_map != nullptr) map_iter = inter_map->end();
       invalid_ids = true;
     }
 
-    if (inter_map == nullptr || map_iter == inter_map->end())
-    {
+    if ((inter_map == nullptr) || (map_iter == inter_map->end())) {
       // Compute the intersections points between the two supporting curves.
-      if (is_linear())
-      {
-        if (cv.is_linear())
-          _lines_intersect (cv, inter_list);
-        else
-          cv._circ_line_intersect (*this, inter_list);
-      }
-      else
-      {
-        if (cv.is_linear())
-          _circ_line_intersect (cv, inter_list);
-        else
-          _circs_intersect (cv, inter_list);
+      if (is_linear()) {
+        if (cv.is_linear()) _lines_intersect(cv, inter_list);
+        else cv._circ_line_intersect(*this, inter_list);
+      }
+      else {
+        if (cv.is_linear()) _circ_line_intersect(cv, inter_list);
+        else _circs_intersect(cv, inter_list);
       }
 
       // Cache the result.
-      if (! invalid_ids)
-        (*inter_map)[id_pair] = inter_list;
+      if (! invalid_ids) (*inter_map)[id_pair] = inter_list;
     }
-    else
-    {
+    else {
       // Obtain the precomputed intersection points from the map.
       inter_list = (*map_iter).second;
     }
 
     // Report only the intersection points that lie on both arcs.
-    typename Intersection_list::const_iterator   iter;
-
-    for (iter = inter_list.begin(); iter != inter_list.end(); ++iter)
-    {
+    for (auto iter = inter_list.begin(); iter != inter_list.end(); ++iter) {
       if (this->_is_between_endpoints (iter->first) &&
           cv._is_between_endpoints (iter->first))
       {
-        *oi = CGAL::make_object (*iter);
-        ++oi;
+        *oi++ = Intersection_result(*iter);
       }
     }
 
-    return (oi);
+    return oi;
   }
 
-  /*!
-   * Check whether it is possible to merge our arc with the given arc.
+  /*! Check whether it is possible to merge our arc with the given arc.
    */
-  bool can_merge_with (const Self& cv) const
+  bool can_merge_with(const Self& cv) const
   {
     // In order to merge the two arcs, they should have the same supporting
     // curve.
-    if (! this->has_same_supporting_curve (cv))
-      return (false);
+    if (! this->has_same_supporting_curve(cv)) return false;
 
     // Check if the left endpoint of one curve is the right endpoint of the
     // other.
-    return (right().equals (cv.left()) ||
-            left().equals (cv.right()));
+    return (right().equals(cv.left()) || left().equals(cv.right()));
   }
 
-  /*!
-   * Merge our arc with the given arc.
+  /*! Merge our arc with the given arc.
    * \pre The two arcs are mergeable.
    */
-  void merge (const Self& cv)
+  void merge(const Self& cv)
   {
-    CGAL_precondition (this->can_merge_with (cv));
+    CGAL_precondition(this->can_merge_with (cv));
 
     // Check if we should extend the arc to the left or to the right.
-    if (right().equals (cv.left()))
-    {
+    if (right().equals(cv.left())) {
       // Extend the arc to the right.
-      if (is_directed_right())
-        this->_target = cv.right();
-      else
-        this->_source = cv.right();
+      if (is_directed_right()) this->m_target = cv.right();
+      else this->m_source = cv.right();
     }
-    else
-    {
-      CGAL_precondition (left().equals (cv.right()));
+    else {
+      CGAL_precondition(left().equals(cv.right()));
 
       // Extend the arc to the left.
-      if (is_directed_right())
-        this->_source = cv.left();
-      else
-        this->_target = cv.left();
+      if (is_directed_right()) this->m_source = cv.left();
+      else this->m_target = cv.left();
     }
-
-    return;
   }
 
   /*! construct an opposite arc. */
   Self construct_opposite() const
   {
     Self opp_cv;
-    opp_cv._first = this->_first;
-    opp_cv._second = this-> _second;
-    opp_cv._third = this-> _third;
-    opp_cv._source = this->_target;
-    opp_cv._target = this->_source;
+    opp_cv.m_first = this->m_first;
+    opp_cv.m_second = this->m_second;
+    opp_cv.m_third = this->m_third;
+    opp_cv.m_source = this->m_target;
+    opp_cv.m_target = this->m_source;
 
     // Take care of the information bits: We flip the orientation bits and
     // the bits that marks the direction.
-    if (is_linear())
-      opp_cv._info = (this->_info ^ IS_DIRECTED_RIGHT_MASK);
+    if (is_linear()) opp_cv.m_info = (this->m_info ^ IS_DIRECTED_RIGHT_MASK);
     else
-      opp_cv._info = (this->_info ^ IS_DIRECTED_RIGHT_MASK ^ ORIENTATION_MASK);
+      opp_cv.m_info = (this->m_info ^ IS_DIRECTED_RIGHT_MASK ^ ORIENTATION_MASK);
 
     return (opp_cv);
   }
@@ -1358,65 +1139,47 @@ public:
     double x_max = to_double(right().x());
     double y_min = to_double(left().y());
     double y_max = to_double(right().y());
-    if(y_min > y_max)
-      std::swap(y_min, y_max);
-    if(is_circular())
-    {
+    if (y_min > y_max) std::swap(y_min, y_max);
+    if (is_circular()) {
       const Circle_2& circ = this->supporting_circle();
-      if(_is_upper())
-      {
+      if (_is_upper()) {
         y_max = to_double(circ.center().y())+
-                std::sqrt(to_double(circ.squared_radius()));
+          std::sqrt(to_double(circ.squared_radius()));
       }
-      else
-      {
+      else {
         y_min = to_double(circ.center().y()) -
-                std::sqrt(to_double(circ.squared_radius()));
+          std::sqrt(to_double(circ.squared_radius()));
       }
     }
 
-
     return Bbox_2(x_min, y_min, x_max, y_max);
   }
 
 protected:
-
   /*! Get the curve index. */
-  inline unsigned int _index () const
-  {
-    return (_info >> INDEX_SHIFT_BITS);
-  }
+  inline unsigned int _index() const { return (m_info >> INDEX_SHIFT_BITS); }
 
   /// \name Accessors for circular arcs.
   //@{
 
   /*! Get the x-coordinate of the center of the supporting circle. */
-  inline const NT& x0 () const
-  {
-    return (_first);
-  }
+  inline const NT& x0() const { return (m_first); }
 
   /*! Get the y-coordinate of the center of the supporting circle. */
-  inline const NT& y0 () const
-  {
-    return (_second);
-  }
+  inline const NT& y0() const { return (m_second); }
 
   /*! Get the squared radius of the supporting circle. */
-  inline const NT& sqr_r () const
-  {
-    return (_third);
-  }
+  inline const NT& sqr_r() const { return (m_third); }
 
   /*!
    * Check if the circular arc lies on the upper half of the supporting circle.
    */
-  inline bool _is_upper () const
+  inline bool _is_upper() const
   {
-    Orientation  orient = orientation();
-    bool         dir_right = ((_info & IS_DIRECTED_RIGHT_MASK) != 0);
+    Orientation orient = orientation();
+    bool dir_right = ((m_info & IS_DIRECTED_RIGHT_MASK) != 0);
 
-    CGAL_precondition (orient != COLLINEAR);
+    CGAL_precondition(orient != COLLINEAR);
 
     return ((orient == COUNTERCLOCKWISE && !dir_right) ||
             (orient == CLOCKWISE && dir_right));
@@ -1427,22 +1190,13 @@ protected:
   //@{
 
   /*! Get the coefficient of x in the equation of the supporting line. */
-  inline const NT& a () const
-  {
-    return (_first);
-  }
+  inline const NT& a() const { return (m_first); }
 
   /*! Get the coefficient of y in the equation of the supporting line. */
-  inline const NT& b () const
-  {
-    return (_second);
-  }
+  inline const NT& b() const { return (m_second); }
 
   /*! Get the free coefficient in the equation of the supporting line. */
-  inline const NT& c () const
-  {
-    return (_third);
-  }
+  inline const NT& c() const { return (m_third); }
   //@}
 
   /// \name Auxiliary functions for the point_position predicate.
@@ -1451,27 +1205,24 @@ protected:
   /*!
    * Check the position of a given point with respect to a line segment.
    */
-  Comparison_result _line_point_position (const Point_2& p) const
+  Comparison_result _line_point_position(const Point_2& p) const
   {
     // Check if we have a vertical segment.
 
-    CGAL_precondition (is_in_x_range(p));
+    CGAL_precondition(is_in_x_range(p));
 
-    Comparison_result    res;
+    Comparison_result res;
 
-    if (is_vertical())
-    {
+    if (is_vertical()) {
       // left() is the lower endpoint:
-      res = CGAL::compare (p.y(), left().y());
+      res = CGAL::compare(p.y(), left().y());
 
-      if (res != LARGER)
-        return (res);
+      if (res != LARGER) return (res);
 
       // left() is the upper endpoint:
       res = CGAL::compare (p.y(), right().y());
 
-      if (res != SMALLER)
-        return (res);
+      if (res != SMALLER) return (res);
 
       // p lies in the interior of the vertical segment:
       return (EQUAL);
@@ -1487,19 +1238,17 @@ protected:
   /*!
    * Check the position of a given point with respect to a circular arc.
    */
-  Comparison_result _circ_point_position (const Point_2& p) const
+  Comparison_result _circ_point_position(const Point_2& p) const
   {
 
-    Comparison_result   c_res = CGAL::compare (p.y(), y0());
+    Comparison_result c_res = CGAL::compare (p.y(), y0());
 
-    if (_is_upper())
-    {
+    if (_is_upper()) {
       // Check if p lies below the "equator" (while the arc lies above it):
       if (c_res == SMALLER)
         return (SMALLER);
     }
-    else
-    {
+    else {
       // Check if p lies above the "equator" (while the arc lies below it):
       if (c_res == LARGER)
         return (LARGER);
@@ -1507,21 +1256,18 @@ protected:
 
     // Check if p lies inside the supporting circle, namely we have to check
     // whether (p.x() - x0)^2 + (p.y() - y0)^2 < r^2:
-    Comparison_result   res =
-                         CGAL::compare (CGAL::square (p.x() - x0()),
-                                        sqr_r() - CGAL::square (p.y() - y0()));
+    Comparison_result res =
+      CGAL::compare(CGAL::square (p.x() - x0()),
+                    sqr_r() - CGAL::square (p.y() - y0()));
 
-    if (res == EQUAL)
-      // p lies on the circle:
-      return (EQUAL);
+    // p lies on the circle:
+    if (res == EQUAL) return (EQUAL);
 
-    if (_is_upper())
-    {
+    if (_is_upper()) {
       // If p is inside the circle, it lies below the upper arc:
       return (res);
     }
-    else
-    {
+    else {
       // If p is inside the circle, it lies above the lower arc:
       return (res == SMALLER ? LARGER : SMALLER);
     }
@@ -1534,25 +1280,18 @@ protected:
   /*!
    * Compare two line segments to the right of their intersection point.
    */
-  Comparison_result _lines_compare_to_right (const Self& cv,
-                                             const Point_2& /* p */) const
+  Comparison_result _lines_compare_to_right(const Self& cv,
+                                            const Point_2& /* p */) const
   {
-    if (_index() != 0 && _index() == cv._index())
-      return (EQUAL);
+    if (_index() != 0 && _index() == cv._index()) return (EQUAL);
 
     // Special treatment for vertical segments: a vertical segment is larger
     // than any other non-vertical segment.
-    if (is_vertical())
-    {
-      if (cv.is_vertical())
-        return (EQUAL);
-
+    if (is_vertical()) {
+      if (cv.is_vertical()) return (EQUAL);
       return (LARGER);
     }
-    else if (cv.is_vertical())
-    {
-      return (SMALLER);
-    }
+    else if (cv.is_vertical()) return (SMALLER);
 
     // Compare the slopes: -A1/B1 and -A2/B2. We actually negate the slopes
     // and swap the result.
@@ -1563,12 +1302,11 @@ protected:
    * Compare a circular arcs (this) and a line segment (cv) to the right of
    * their intersection point.
    */
-  Comparison_result _circ_line_compare_to_right (const Self& cv,
-                                                 const Point_2& p) const
+  Comparison_result _circ_line_compare_to_right(const Self& cv,
+                                                const Point_2& p) const
   {
     // A vertical segment lies above any other circle to the right of p:
-    if (cv.is_vertical())
-      return (SMALLER);
+    if (cv.is_vertical()) return (SMALLER);
 
     // We have to compare the slopes of the supporting circles and the
     // supporting line at p:
@@ -1580,8 +1318,7 @@ protected:
     const CGAL::Sign  sign_denom1 = CGAL::sign (y0() - p.y());
 
     // Check the case of a vertical tangent.
-    if (sign_denom1 == ZERO)
-    {
+    if (sign_denom1 == ZERO) {
       // The arc lies above any line segment if it is an upper arc, or below
       // any segment if it is a lower arc.
       return (_is_upper() ? LARGER : SMALLER);
@@ -1589,16 +1326,13 @@ protected:
 
     // Compare (p.x() - x0(1)) and (A(2)/B(2)*(p.y() - y0(1)).
     // Note that if the denominator is negative, we have to swap the result.
-    const bool        swap_res = (sign_denom1 == NEGATIVE);
-    Comparison_result slope_res = CGAL::compare (p.x() - x0(),
-                                                 (p.y() - y0())*cv.a()/cv.b());
-
-    if (slope_res != EQUAL)
-    {
-      if (swap_res)
-        // Swap the comparison result, if necessary:
-        slope_res = (slope_res == SMALLER) ? LARGER : SMALLER;
-
+    const bool swap_res = (sign_denom1 == NEGATIVE);
+    Comparison_result slope_res =
+      CGAL::compare (p.x() - x0(), (p.y() - y0())*cv.a()/cv.b());
+
+    if (slope_res != EQUAL) {
+      // Swap the comparison result, if necessary:
+      if (swap_res) slope_res = (slope_res == SMALLER) ? LARGER : SMALLER;
       return (slope_res);
     }
 
@@ -1608,23 +1342,18 @@ protected:
     return (_is_upper() ? SMALLER : LARGER);
   }
 
-  /*!
-   * Compare two circular arcs to the right of their intersection point.
+  /*! Compare two circular arcs to the right of their intersection point.
    */
-  Comparison_result _circs_compare_to_right (const Self& cv,
-                                             const Point_2& p) const
+  Comparison_result _circs_compare_to_right(const Self& cv,
+                                            const Point_2& p) const
   {
-    if (_index() != 0 && _index() == cv._index())
-    {
+    if (_index() != 0 && _index() == cv._index()) {
       // Check the case of comparing two circular arcs that originate from the
       // same supporting circle. Their comparison result is not EQUAL only if
       // one is an upper arc and the other is a lower arc.
-      if (_is_upper() && ! cv._is_upper())
-        return (LARGER);
-      else if (! _is_upper() && cv._is_upper())
-        return (SMALLER);
-      else
-        return (EQUAL);
+      if (_is_upper() && ! cv._is_upper()) return (LARGER);
+      else if (! _is_upper() && cv._is_upper()) return (SMALLER);
+      else return (EQUAL);
     }
 
     // We have to compare the slopes of the two supporting circles at p:
@@ -1633,39 +1362,31 @@ protected:
     //   ---------------  and  ---------------
     //    y0(1) - p.y()         y0(2) - p.y()
     //
-    const CGAL::Sign  sign_numer1 = CGAL::sign (p.x() - x0());
-    const CGAL::Sign  sign_denom1 = CGAL::sign (y0() - p.y());
-    const CGAL::Sign  sign_numer2 = CGAL::sign (p.x() - cv.x0());
-    const CGAL::Sign  sign_denom2 = CGAL::sign (cv.y0() - p.y());
+    const CGAL::Sign  sign_numer1 = CGAL::sign(p.x() - x0());
+    const CGAL::Sign  sign_denom1 = CGAL::sign(y0() - p.y());
+    const CGAL::Sign  sign_numer2 = CGAL::sign(p.x() - cv.x0());
+    const CGAL::Sign  sign_denom2 = CGAL::sign(cv.y0() - p.y());
 
     // Check the case of vertical tangents.
-    if (sign_denom1 == ZERO)
-    {
-      if (sign_denom2 == ZERO)
-      {
-        if (_is_upper())
-        {
-          if (cv._is_upper())
-          {
+    if (sign_denom1 == ZERO) {
+      if (sign_denom2 == ZERO) {
+        if (_is_upper()) {
+          if (cv._is_upper()) {
             // The two circles have a vertical tangent:
             // The one with a larger radius is above the other.
             return (CGAL::compare (sqr_r(), cv.sqr_r()));
           }
-          else
-          {
+          else {
             // The other curve is directed downwards:
             return (LARGER);
           }
         }
-        else
-        {
-          if (cv._is_upper())
-          {
+        else {
+          if (cv._is_upper()) {
             // The other curve is directed upwards:
             return (SMALLER);
           }
-          else
-          {
+          else {
             // The two circles have a vertical tangent:
             // The one with a smaller radius is above the other.
             return (CGAL::compare (cv.sqr_r(), sqr_r()));
@@ -1676,28 +1397,21 @@ protected:
       // The other arc does not have a vertical tangent.
       return (_is_upper() ? LARGER : SMALLER);
     }
-    else if (sign_denom2 == ZERO)
-    {
+    else if (sign_denom2 == ZERO) {
       return (cv._is_upper() ? SMALLER : LARGER);
     }
 
     // Try to act according to the slope signs.
-    CGAL::Sign   sign_slope1;
-    CGAL::Sign   sign_slope2;
+    CGAL::Sign sign_slope1;
+    CGAL::Sign sign_slope2;
 
-    if (sign_numer1 == sign_denom1)
-      sign_slope1 = POSITIVE;
-    else if (sign_numer1 == ZERO)
-      sign_slope1 = ZERO;
-    else
-      sign_slope1 = NEGATIVE;
-
-    if (sign_numer2 == sign_denom2)
-      sign_slope2 = POSITIVE;
-    else if (sign_numer2 == ZERO)
-      sign_slope2 = ZERO;
-    else
-      sign_slope2 = NEGATIVE;
+    if (sign_numer1 == sign_denom1) sign_slope1 = POSITIVE;
+    else if (sign_numer1 == ZERO) sign_slope1 = ZERO;
+    else sign_slope1 = NEGATIVE;
+
+    if (sign_numer2 == sign_denom2) sign_slope2 = POSITIVE;
+    else if (sign_numer2 == ZERO) sign_slope2 = ZERO;
+    else sign_slope2 = NEGATIVE;
 
     if ((sign_slope1 == POSITIVE && sign_slope2 != POSITIVE) ||
         (sign_slope1 == ZERO && sign_slope2 == NEGATIVE))
@@ -1718,46 +1432,38 @@ protected:
     else
     {
       // Actually compare the slopes.
-      const bool    swap_res = (sign_denom1 != sign_denom2);
+      const bool swap_res = (sign_denom1 != sign_denom2);
       const CoordNT A = (cv.y0() - y0())*p.x() + (y0()*cv.x0() - cv.y0()*x0());
       const CoordNT B = (cv.x0() - x0())*p.y();
 
       slope_res = CGAL::compare (A, B);
 
-      if (slope_res != EQUAL && swap_res)
-      {
+      if (slope_res != EQUAL && swap_res) {
         // Swap the comparison result, if necessary:
         slope_res = (slope_res == SMALLER) ? LARGER : SMALLER;
       }
     }
 
     // In case the two circles have different tangent slopes at p:
-    if (slope_res != EQUAL)
-      return (slope_res);
+    if (slope_res != EQUAL) return (slope_res);
 
     // In this case we have a tangency point at p.
-    if (_is_upper())
-    {
-      if (cv._is_upper())
-      {
+    if (_is_upper()) {
+      if (cv._is_upper()) {
         // The circle with a larger radius is above the other.
-        return (CGAL::compare (sqr_r(), cv.sqr_r()));
+        return (CGAL::compare(sqr_r(), cv.sqr_r()));
       }
-      else
-      {
+      else {
         // The other curve is above our curve:
         return (SMALLER);
       }
     }
-    else
-    {
-      if (cv._is_upper())
-      {
+    else {
+      if (cv._is_upper()) {
         // Out curve is above the other curve:
         return (LARGER);
       }
-      else
-      {
+      else {
         // The circle with a smaller radius is above the other.
         return (CGAL::compare (cv.sqr_r(), sqr_r()));
       }
@@ -1771,23 +1477,18 @@ protected:
   /*!
    * Compare two line segments to the left of their intersection point.
    */
-  Comparison_result _lines_compare_to_left (const Self& cv,
-                                            const Point_2& ) const
+  Comparison_result _lines_compare_to_left(const Self& cv,
+                                           const Point_2& ) const
   {
-    if (_index() != 0 && _index() == cv._index())
-      return (EQUAL);
+    if (_index() != 0 && _index() == cv._index()) return (EQUAL);
 
     // Special treatment for vertical segments: a vertical segment is smaller
     // than any other non-vertical segment.
-    if (is_vertical())
-    {
-      if (cv.is_vertical())
-        return (EQUAL);
-
+    if (is_vertical()) {
+      if (cv.is_vertical()) return (EQUAL);
       return (SMALLER);
     }
-    else if (cv.is_vertical())
-    {
+    else if (cv.is_vertical()) {
       return (LARGER);
     }
 
@@ -1796,16 +1497,14 @@ protected:
     return (CGAL::compare (a()/b(), cv.a()/cv.b()));
   }
 
-  /*!
-   * Compare a circular arcs (this) and a line segment (cv) to the left of
+  /*! Compare a circular arcs (this) and a line segment (cv) to the left of
    * their intersection point.
    */
-  Comparison_result _circ_line_compare_to_left (const Self& cv,
-                                                const Point_2& p) const
+  Comparison_result _circ_line_compare_to_left(const Self& cv,
+                                               const Point_2& p) const
   {
     // A vertical segment lies below any other circle to the left of p:
-    if (cv.is_vertical())
-      return (LARGER);
+    if (cv.is_vertical()) return (LARGER);
 
     // We have to compare the slopes of the supporting circles and the
     // supporting line at p, and return the swapped result:
@@ -1817,8 +1516,7 @@ protected:
     const CGAL::Sign  sign_denom1 = CGAL::sign (y0() - p.y());
 
     // Check the case of a vertical tangent.
-    if (sign_denom1 == ZERO)
-    {
+    if (sign_denom1 == ZERO) {
       // The arc lies above any line segment if it is an upper arc, or below
       // any segment if it is a lower arc.
       return (_is_upper() ? LARGER : SMALLER);
@@ -1826,12 +1524,11 @@ protected:
 
     // Compare (p.x() - x0(1)) and (A(2)/B(2)*(p.y() - y0(1)).
     // Note that if the denominator is negative, we have to swap the result.
-    const bool        swap_res = (sign_denom1 == NEGATIVE);
-    Comparison_result slope_res = CGAL::compare (p.x() - x0(),
-                                                 (p.y() - y0())*cv.a()/cv.b());
+    const bool swap_res = (sign_denom1 == NEGATIVE);
+    Comparison_result slope_res =
+      CGAL::compare(p.x() - x0(), (p.y() - y0()) * cv.a() / cv.b());
 
-    if (slope_res != EQUAL)
-    {
+    if (slope_res != EQUAL) {
       if (swap_res)
         // Swap the comparison result, if necessary:
         slope_res = (slope_res == SMALLER) ? LARGER : SMALLER;
@@ -1849,20 +1546,16 @@ protected:
   /*!
    * Compare the two arcs to the left of their intersection point.
    */
-  Comparison_result _circs_compare_to_left (const Self& cv,
-                                            const Point_2& p) const
+  Comparison_result _circs_compare_to_left(const Self& cv,
+                                           const Point_2& p) const
   {
-    if (_index() != 0 && _index() == cv._index())
-    {
+    if (_index() != 0 && _index() == cv._index()) {
       // Check the case of comparing two circular arcs that originate from the
       // same supporting circle. Their comparison result is not EQUAL only if
       // one is an upper arc and the other is a lower arc.
-      if (_is_upper() && ! cv._is_upper())
-        return (LARGER);
-      else if (! _is_upper() && cv._is_upper())
-        return (SMALLER);
-      else
-        return (EQUAL);
+      if (_is_upper() && ! cv._is_upper()) return (LARGER);
+      else if (! _is_upper() && cv._is_upper()) return (SMALLER);
+      else return (EQUAL);
     }
 
     // We have to compare the slopes of the two supporting circles at p:
@@ -1872,39 +1565,31 @@ protected:
     //    y0(1) - p.y()         y0(2) - p.y()
     //
     // Eventually, we should take the opposite result.
-    const CGAL::Sign  sign_numer1 = CGAL::sign (p.x() - x0());
-    const CGAL::Sign  sign_denom1 = CGAL::sign (y0() - p.y());
-    const CGAL::Sign  sign_numer2 = CGAL::sign (p.x() - cv.x0());
-    const CGAL::Sign  sign_denom2 = CGAL::sign (cv.y0() - p.y());
+    const CGAL::Sign  sign_numer1 = CGAL::sign(p.x() - x0());
+    const CGAL::Sign  sign_denom1 = CGAL::sign(y0() - p.y());
+    const CGAL::Sign  sign_numer2 = CGAL::sign(p.x() - cv.x0());
+    const CGAL::Sign  sign_denom2 = CGAL::sign(cv.y0() - p.y());
 
     // Check the case of vertical tangents.
-    if (sign_denom1 == ZERO)
-    {
-      if (sign_denom2 == ZERO)
-      {
-        if (_is_upper())
-        {
-          if (cv._is_upper())
-          {
+    if (sign_denom1 == ZERO) {
+      if (sign_denom2 == ZERO) {
+        if (_is_upper()) {
+          if (cv._is_upper()) {
             // The two circles have a vertical tangent:
             // The one with a larger radius is above the other.
             return (CGAL::compare (sqr_r(), cv.sqr_r()));
           }
-          else
-          {
+          else {
             // The other curve is directed downwards:
             return (LARGER);
           }
         }
-        else
-        {
-          if (cv._is_upper())
-          {
+        else {
+          if (cv._is_upper()) {
             // The other curve is directed upwards:
             return (SMALLER);
           }
-          else
-          {
+          else {
             // The two circles have a vertical tangent:
             // The one with a smaller radius is above the other.
             return (CGAL::compare (cv.sqr_r(), sqr_r()));
@@ -1915,28 +1600,21 @@ protected:
       // The other arc does not have a vertical tangent.
       return (_is_upper() ? LARGER : SMALLER);
     }
-    else if (sign_denom2 == ZERO)
-    {
+    else if (sign_denom2 == ZERO) {
       return (cv._is_upper() ? SMALLER : LARGER);
     }
 
     // Try to act according to the slope signs.
-    CGAL::Sign   sign_slope1;
-    CGAL::Sign   sign_slope2;
-
-    if (sign_numer1 == sign_denom1)
-      sign_slope1 = POSITIVE;
-    else if (sign_numer1 == ZERO)
-      sign_slope1 = ZERO;
-    else
-      sign_slope1 = NEGATIVE;
+    CGAL::Sign sign_slope1;
+    CGAL::Sign sign_slope2;
 
-    if (sign_numer2 == sign_denom2)
-      sign_slope2 = POSITIVE;
-    else if (sign_numer2 == ZERO)
-      sign_slope2 = ZERO;
-    else
-      sign_slope2 = NEGATIVE;
+    if (sign_numer1 == sign_denom1) sign_slope1 = POSITIVE;
+    else if (sign_numer1 == ZERO) sign_slope1 = ZERO;
+    else sign_slope1 = NEGATIVE;
+
+    if (sign_numer2 == sign_denom2) sign_slope2 = POSITIVE;
+    else if (sign_numer2 == ZERO) sign_slope2 = ZERO;
+    else sign_slope2 = NEGATIVE;
 
     if ((sign_slope1 == POSITIVE && sign_slope2 != POSITIVE) ||
         (sign_slope1 == ZERO && sign_slope2 == NEGATIVE))
@@ -1954,17 +1632,15 @@ protected:
       // Special case were both circles have a horizontal tangent:
       slope_res = EQUAL;
     }
-    else
-    {
+    else {
       // Actually compare the slopes.
-      const bool    swap_res = (sign_denom1 != sign_denom2);
+      const bool swap_res = (sign_denom1 != sign_denom2);
       const CoordNT A = (cv.y0() - y0())*p.x() + (y0()*cv.x0() - cv.y0()*x0());
       const CoordNT B = (cv.x0() - x0())*p.y();
 
-      slope_res = CGAL::compare (A, B);
+      slope_res = CGAL::compare(A, B);
 
-      if (slope_res != EQUAL && swap_res)
-      {
+      if (slope_res != EQUAL && swap_res) {
         // Swap the comparison result, if necessary:
         slope_res = (slope_res == SMALLER) ? LARGER : SMALLER;
       }
@@ -1973,34 +1649,27 @@ protected:
     // In case the two circles have different tangent slopes at p, return
     // the opposite of the slope result (since the slope result is the
     // comparison result to the right of the intersection point):
-    if (slope_res != EQUAL)
-      return ((slope_res == SMALLER) ? LARGER : SMALLER);
+    if (slope_res != EQUAL) return ((slope_res == SMALLER) ? LARGER : SMALLER);
 
     // In this case we have a tangency point at p.
-    if (_is_upper())
-    {
-      if (cv._is_upper())
-      {
+    if (_is_upper()) {
+      if (cv._is_upper()) {
         // The circle with a larger radius is above the other.
-        return (CGAL::compare (sqr_r(), cv.sqr_r()));
+        return (CGAL::compare(sqr_r(), cv.sqr_r()));
       }
-      else
-      {
+      else {
         // The other curve is above our curve:
         return (SMALLER);
       }
     }
-    else
-    {
-      if (cv._is_upper())
-      {
+    else {
+      if (cv._is_upper()) {
         // Out curve is above the other curve:
         return (LARGER);
       }
-      else
-      {
+      else {
         // The circle with a smaller radius is above the other.
-        return (CGAL::compare (cv.sqr_r(), sqr_r()));
+        return (CGAL::compare(cv.sqr_r(), sqr_r()));
       }
     }
   }
@@ -2009,11 +1678,10 @@ protected:
   /// \name Auxiliary functions for computing intersections.
   //@{
 
-  /*!
-   * Compute the intersections between two line segments.
+  /*! Compute the intersections between two line segments.
    */
-  void _lines_intersect (const Self& cv,
-                         Intersection_list& inter_list) const
+  void _lines_intersect(const Self& cv,
+                        Intersection_list& inter_list) const
   {
     // The intersection of the lines:
     //   a1*x + b1*y + c1 = 0   and   a2*x + b2*y + c2 = 0 ,
@@ -2023,53 +1691,47 @@ protected:
     //   ( --------------- , --------------- )
     //      a1*b2 - b1*a2     a1*b2 - b1*a2
     //
-    unsigned int  mult = 1;
-    const NT      denom = a()*cv.b() - b()*cv.a();
+    unsigned int mult = 1;
+    const NT denom = a()*cv.b() - b()*cv.a();
 
     // Make sure the supporting lines are not parallel.
-    if (CGAL::sign(denom) == ZERO)
-      return;
+    if (CGAL::sign(denom) == ZERO) return;
 
-    const NT      x = (b()*cv.c() - c()*cv.b()) / denom;
-    const NT      y = (c()*cv.a() - a()*cv.c()) / denom;
-    Point_2       p (x, y);
+    const NT x = (b()*cv.c() - c()*cv.b()) / denom;
+    const NT y = (c()*cv.a() - a()*cv.c()) / denom;
+    Point_2 p (x, y);
 
-    inter_list.push_back (Intersection_point_2 (p, mult));
-    return;
+    inter_list.push_back(Intersection_point(p, mult));
   }
 
-  /*!
-   * Compute the intersections between the supporting circle of (*this) and
+  /*! Compute the intersections between the supporting circle of (*this) and
    * the supporting line of the segement cv.
    */
-  void _circ_line_intersect (const Self& cv,
-                             Intersection_list& inter_list) const
+  void _circ_line_intersect(const Self& cv,
+                            Intersection_list& inter_list) const
   {
-    Point_2       p;
-    unsigned int  mult;
+    Point_2 p;
+    unsigned int mult;
 
     // First check the special cases of vertical and horizontal lines.
-    if (cv.is_vertical())
-    {
+    if (cv.is_vertical()) {
       // The equation of the vertical line is x = -c / a.
       // The y-coordinates of the intersection points are:
       //   y =  y0 +/- sqrt(r^2 - (x - x0)^2)
       //
-      const NT   vx = -cv.c() / cv.a();
-      const NT   vdisc = sqr_r() - CGAL::square (vx - x0());
+      const NT vx = -cv.c() / cv.a();
+      const NT vdisc = sqr_r() - CGAL::square (vx - x0());
       CGAL::Sign sign_vdisc = CGAL::sign (vdisc);
 
-      if (sign_vdisc == NEGATIVE)
-      {
+      if (sign_vdisc == NEGATIVE) {
         // The circle and the vertical line do not intersect.
         return;
       }
-      else if (sign_vdisc == ZERO)
-      {
+      else if (sign_vdisc == ZERO) {
         // A single tangency point, given by:
         mult = 2;
         p = Point_2 (vx, y0());
-        inter_list.push_back (Intersection_point_2 (p, mult));
+        inter_list.push_back(Intersection_point(p, mult));
 
         return;
       }
@@ -2077,37 +1739,30 @@ protected:
       // Compute the two intersection points:
       mult = 1;
 
-      p = Point_2 (CoordNT (vx),
-                   CoordNT (y0(), NT(-1), vdisc));
-      inter_list.push_back (Intersection_point_2 (p, mult));
-
-      p = Point_2 (CoordNT (vx),
-                   CoordNT (y0(), NT(1), vdisc));
-      inter_list.push_back (Intersection_point_2 (p, mult));
+      p = Point_2(CoordNT (vx), CoordNT (y0(), NT(-1), vdisc));
+      inter_list.push_back (Intersection_point(p, mult));
+
+      p = Point_2(CoordNT (vx), CoordNT (y0(), NT(1), vdisc));
+      inter_list.push_back(Intersection_point(p, mult));
 
       return;
     }
-    else if (CGAL::sign (cv.a()) == ZERO)
-    {
+    else if (CGAL::sign (cv.a()) == ZERO) {
       // The equation of the horizontal line is y = -c / b.
       // The y-coordinates of the intersection points are:
       //   x =  x0 +/- sqrt(r^2 - (y - y0)^2)
       //
-      const NT   hy = -cv.c() / cv.b();
-      const NT   hdisc = sqr_r() - CGAL::square (hy - y0());
+      const NT hy = -cv.c() / cv.b();
+      const NT hdisc = sqr_r() - CGAL::square (hy - y0());
       CGAL::Sign sign_hdisc = CGAL::sign (hdisc);
 
-      if (sign_hdisc == NEGATIVE)
-      {
-        // The circle and the vertical line do not intersect.
-        return;
-      }
-      else if (sign_hdisc == ZERO)
-      {
+      // The circle and the vertical line do not intersect.
+      if (sign_hdisc == NEGATIVE) return;
+      else if (sign_hdisc == ZERO) {
         // A single tangency point, given by:
         mult = 2;
-        p = Point_2 (x0(), hy);
-        inter_list.push_back (Intersection_point_2 (p, mult));
+        p = Point_2(x0(), hy);
+        inter_list.push_back(Intersection_point (p, mult));
 
         return;
       }
@@ -2115,287 +1770,246 @@ protected:
       // Compute the two intersection points:
       mult = 1;
 
-      p = Point_2 (CoordNT (x0(), NT(-1), hdisc),
-                   CoordNT (hy));
-      inter_list.push_back (Intersection_point_2 (p, mult));
-
-      p = Point_2 (CoordNT (x0(), NT(1), hdisc),
-                   CoordNT (hy));
-      inter_list.push_back (Intersection_point_2 (p, mult));
+      p = Point_2(CoordNT(x0(), NT(-1), hdisc), CoordNT (hy));
+      inter_list.push_back(Intersection_point (p, mult));
+
+      p = Point_2(CoordNT(x0(), NT(1), hdisc), CoordNT(hy));
+      inter_list.push_back(Intersection_point (p, mult));
 
       return;
     }
 
     // Compute the squared distance between the line and the circle center,
     // inducing the discriminant of the quadratic equations we have to solve.
-    const NT   line_factor = CGAL::square(cv.a()) + CGAL::square(cv.b());
-    const NT   disc = line_factor*sqr_r() -
-                      CGAL::square(cv.a()*x0() + cv.b()*y0() + cv.c());
+    const NT line_factor = CGAL::square(cv.a()) + CGAL::square(cv.b());
+    const NT disc =
+      line_factor*sqr_r() - CGAL::square(cv.a()*x0() + cv.b()*y0() + cv.c());
     CGAL::Sign sign_disc = CGAL::sign (disc);
 
-    if (sign_disc == NEGATIVE)
-    {
-      // The circle and the line do not intersect:
-      return;
-    }
+    // The circle and the line do not intersect:
+    if (sign_disc == NEGATIVE) return;
 
     // Compare the square-free part of the solution:
-    const NT   aux = cv.b()*x0() - cv.a()*y0();
-    const NT   x_base = (aux*cv.b() - cv.a()*cv.c()) / line_factor;
-    const NT   y_base = (-aux*cv.a() - cv.b()*cv.c()) / line_factor;
+    const NT aux = cv.b()*x0() - cv.a()*y0();
+    const NT x_base = (aux*cv.b() - cv.a()*cv.c()) / line_factor;
+    const NT y_base = (-aux*cv.a() - cv.b()*cv.c()) / line_factor;
 
-    if (sign_disc == ZERO)
-    {
+    if (sign_disc == ZERO) {
       // A single tangency point, given by:
       mult = 2;
-      p = Point_2 (x_base, y_base);
-      inter_list.push_back (Intersection_point_2 (p, mult));
+      p = Point_2(x_base, y_base);
+      inter_list.push_back(Intersection_point(p, mult));
 
       return;
     }
 
     // We have two intersection points, whose coordinates are one-root numbers.
-    bool       minus_root_first = (CGAL::sign(cv.b()) == POSITIVE);
-    const NT   x_root_coeff = cv.b() / line_factor;
-    const NT   y_root_coeff = cv.a() / line_factor;
+    bool minus_root_first = (CGAL::sign(cv.b()) == POSITIVE);
+    const NT x_root_coeff = cv.b() / line_factor;
+    const NT y_root_coeff = cv.a() / line_factor;
 
     mult = 1;
-    if (minus_root_first)
-    {
-      p = Point_2 (CoordNT (x_base, -x_root_coeff, disc),
-                   CoordNT (y_base, y_root_coeff, disc));
-      inter_list.push_back (Intersection_point_2 (p, mult));
-
-      p = Point_2 (CoordNT (x_base, x_root_coeff, disc),
-                   CoordNT (y_base, -y_root_coeff, disc));
-      inter_list.push_back (Intersection_point_2 (p, mult));
-    }
-    else
-    {
-      p = Point_2 (CoordNT (x_base, x_root_coeff, disc),
-                   CoordNT (y_base, -y_root_coeff, disc));
-      inter_list.push_back (Intersection_point_2 (p, mult));
-
-      p = Point_2 (CoordNT (x_base, -x_root_coeff, disc),
-                   CoordNT (y_base, y_root_coeff, disc));
-      inter_list.push_back (Intersection_point_2 (p, mult));
+    if (minus_root_first) {
+      p = Point_2(CoordNT(x_base, -x_root_coeff, disc),
+                  CoordNT(y_base, y_root_coeff, disc));
+      inter_list.push_back(Intersection_point (p, mult));
+
+      p = Point_2(CoordNT(x_base, x_root_coeff, disc),
+                  CoordNT(y_base, -y_root_coeff, disc));
+      inter_list.push_back(Intersection_point(p, mult));
     }
+    else {
+      p = Point_2(CoordNT(x_base, x_root_coeff, disc),
+                  CoordNT(y_base, -y_root_coeff, disc));
+      inter_list.push_back(Intersection_point(p, mult));
 
-    return;
+      p = Point_2(CoordNT(x_base, -x_root_coeff, disc),
+                  CoordNT(y_base, y_root_coeff, disc));
+      inter_list.push_back(Intersection_point(p, mult));
+    }
   }
 
-  /*!
-   * Compute the intersections between two circles.
+  /*! Compute the intersections between two circles.
    */
-  void _circs_intersect (const Self& cv,
-                         Intersection_list& inter_list) const
+  void _circs_intersect(const Self& cv, Intersection_list& inter_list) const
   {
-    Point_2       p;
-    unsigned int  mult;
+    Point_2 p;
+    unsigned int mult;
 
     // Compute the squared distance between the circle centers, inducing the
     // discriminant of the quadratic equations we have to solve.
-    const NT   diff_x = cv.x0() - x0();
-    const NT   diff_y = cv.y0() - y0();
-    const NT   sqr_dist = CGAL::square(diff_x) + CGAL::square(diff_y);
-    const NT   diff_sqr_rad = sqr_r() - cv.sqr_r();
-    const NT   disc = 2*sqr_dist*(sqr_r() + cv.sqr_r()) -
-                      (CGAL::square(diff_sqr_rad) + CGAL::square(sqr_dist));
+    const NT diff_x = cv.x0() - x0();
+    const NT diff_y = cv.y0() - y0();
+    const NT sqr_dist = CGAL::square(diff_x) + CGAL::square(diff_y);
+    const NT diff_sqr_rad = sqr_r() - cv.sqr_r();
+    const NT disc = 2 * sqr_dist * (sqr_r() + cv.sqr_r()) -
+      (CGAL::square(diff_sqr_rad) + CGAL::square(sqr_dist));
     CGAL::Sign sign_disc = CGAL::sign (disc);
 
-    if (sign_disc == NEGATIVE)
-    {
-      // The two circles do not intersect.
-      return;
-    }
+    // The two circles do not intersect.
+    if (sign_disc == NEGATIVE) return;
 
     // Compare the square-free part of the solution:
-    const NT   x_base = ((x0() + cv.x0()) + diff_x*diff_sqr_rad/sqr_dist) / 2;
-    const NT   y_base = ((y0() + cv.y0()) + diff_y*diff_sqr_rad/sqr_dist) / 2;
+    const NT x_base = ((x0() + cv.x0()) + diff_x * diff_sqr_rad / sqr_dist) / 2;
+    const NT y_base = ((y0() + cv.y0()) + diff_y * diff_sqr_rad / sqr_dist) / 2;
 
-    if (sign_disc == ZERO)
-    {
+    if (sign_disc == ZERO) {
       // A single tangency point, given by:
       mult = 2;
-      p = Point_2 (x_base, y_base);
-      inter_list.push_back (Intersection_point_2 (p, mult));
-
+      p = Point_2(x_base, y_base);
+      inter_list.push_back(Intersection_point(p, mult));
       return;
     }
 
     // We have two intersection points, whose coordinates are one-root numbers.
     CGAL::Sign sign_diff_y = CGAL::sign (diff_y);
-    bool       minus_root_first;
+    bool minus_root_first;
 
     if (sign_diff_y == ZERO)
       minus_root_first = (CGAL::sign (diff_x) == NEGATIVE);
     else
       minus_root_first = (sign_diff_y == POSITIVE);
 
-    const NT   x_root_coeff = diff_y / (2 * sqr_dist);
-    const NT   y_root_coeff = diff_x / (2 * sqr_dist);
+    const NT x_root_coeff = diff_y / (2 * sqr_dist);
+    const NT y_root_coeff = diff_x / (2 * sqr_dist);
 
     mult = 1;
-    if (minus_root_first)
-    {
-      p = Point_2 (CoordNT (x_base, -x_root_coeff, disc),
-                   CoordNT (y_base, y_root_coeff, disc));
-      inter_list.push_back (Intersection_point_2 (p, mult));
-
-      p = Point_2 (CoordNT (x_base, x_root_coeff, disc),
-                   CoordNT (y_base, -y_root_coeff, disc));
-      inter_list.push_back (Intersection_point_2 (p, mult));
-    }
-    else
-    {
-      p = Point_2 (CoordNT (x_base, x_root_coeff, disc),
-                   CoordNT (y_base, -y_root_coeff, disc));
-      inter_list.push_back (Intersection_point_2 (p, mult));
-
-      p = Point_2 (CoordNT (x_base, -x_root_coeff, disc),
-                   CoordNT (y_base, y_root_coeff, disc));
-      inter_list.push_back (Intersection_point_2 (p, mult));
+    if (minus_root_first) {
+      p = Point_2(CoordNT(x_base, -x_root_coeff, disc),
+                  CoordNT(y_base, y_root_coeff, disc));
+      inter_list.push_back(Intersection_point (p, mult));
+
+      p = Point_2(CoordNT(x_base, x_root_coeff, disc),
+                  CoordNT(y_base, -y_root_coeff, disc));
+      inter_list.push_back(Intersection_point (p, mult));
     }
+    else {
+      p = Point_2(CoordNT(x_base, x_root_coeff, disc),
+                  CoordNT(y_base, -y_root_coeff, disc));
+      inter_list.push_back(Intersection_point (p, mult));
 
-    return;
+      p = Point_2(CoordNT(x_base, -x_root_coeff, disc),
+                  CoordNT(y_base, y_root_coeff, disc));
+      inter_list.push_back(Intersection_point(p, mult));
+    }
   }
 
-  /*!
-   * Check if the given point lies on the arc.
+  /*! Check if the given point lies on the arc.
    * \pre p lies on the supporting curve.
    */
-  bool _is_between_endpoints (const Point_2& p) const
+  bool _is_between_endpoints(const Point_2& p) const
   {
-    if (is_linear())
-    {
-      if (is_vertical())
-      {
+    if (is_linear()) {
+      if (is_vertical()) {
         // Check if the point is in the y-range of the arc.
         // Note that left() is the lower endpoint and right() is the upper
         // endpoint of the segment in this case.
-        Comparison_result    res = CGAL::compare (p.y(), left().y());
+        Comparison_result res = CGAL::compare(p.y(), left().y());
 
-        if (res == SMALLER)
-          return (false);
-        else if (res == EQUAL)
-          return (true);
+        if (res == SMALLER) return false;
+        else if (res == EQUAL) return true;
 
-        return (CGAL::compare (p.y(), right().y()) != LARGER);
+        return (CGAL::compare(p.y(), right().y()) != LARGER);
       }
 
       // For non-vertical segments, it is sufficient to check if the point
       // is in the x-range of the arc.
-      return (this->is_in_x_range (p));
+      return (this->is_in_x_range(p));
     }
 
     // The supporting curve is a circle:
     // Check whether p lies on the upper or on the lower part of the circle.
-    Comparison_result   c_res = CGAL::compare (p.y(), y0());
+    Comparison_result c_res = CGAL::compare(p.y(), y0());
 
-    if ((_is_upper() && c_res == SMALLER) ||
-        (! _is_upper() && c_res == LARGER))
+    if ((_is_upper() && c_res == SMALLER) || (! _is_upper() && c_res == LARGER))
     {
       // The point lies on the other half of the circle:
-      return (false);
+      return false;
     }
 
     // Check if the point is in the x-range of the arc.
-    return (this->is_in_x_range (p));
+    return (this->is_in_x_range(p));
   }
 
-  /*!
-   * Check if the given point lies in the interior of the arc.
+  /*! Check whether the given point lies in the interior of the arc.
    * \pre p lies on the supporting curve.
    */
-  bool _is_strictly_between_endpoints (const Point_2& p) const
+  bool _is_strictly_between_endpoints(const Point_2& p) const
   {
-    if (p.equals (_source) || p.equals (_target))
-      return (false);
-
-    return (_is_between_endpoints (p));
+    if (p.equals (m_source) || p.equals (m_target)) return false;
+    return (_is_between_endpoints(p));
   }
 
-  /*!
-   * Compute the overlap with a given arc having the same supporting curve.
+  /*! Compute the overlap with a given arc having the same supporting curve.
    * \param cv The given arc.
    * \param overlap Output: The overlapping arc (if any).
    * \return Whether we found an overlap.
    */
-  bool _compute_overlap (const Self& cv, Self& overlap) const
+  bool _compute_overlap(const Self& cv, Self& overlap) const
   {
     // Check if the two arcs are identical.
-    if (is_linear())
-    {
+    if (is_linear()) {
       // In case of line segments we can swap the source and target:
-      if (((_source.equals (cv._source) && _target.equals (cv._target)) ||
-           (_source.equals (cv._target) && _target.equals (cv._source))))
+      if (((m_source.equals(cv.m_source) && m_target.equals(cv.m_target)) ||
+           (m_source.equals(cv.m_target) && m_target.equals(cv.m_source))))
       {
         overlap = cv;
-        return (true);
+        return true;
       }
     }
-    else
-    {
+    else {
       if ((orientation() == cv.orientation() &&
-           _source.equals (cv._source) && _target.equals (cv._target)) ||
+           m_source.equals(cv.m_source) && m_target.equals(cv.m_target)) ||
           (orientation() != cv.orientation() &&
-           _source.equals (cv._target) && _target.equals (cv._source)))
+           m_source.equals(cv.m_target) && m_target.equals(cv.m_source)))
       {
         overlap = cv;
-        return (true);
+        return true;
       }
     }
 
     // Check for other overlaps:
-    if (_is_strictly_between_endpoints (cv.left()))
-    {
-      if (_is_strictly_between_endpoints (cv.right()))
-      {
+    if (_is_strictly_between_endpoints(cv.left())) {
+      if (_is_strictly_between_endpoints(cv.right())) {
         // Case 1 - *this:     +----------->
         //             cv:       +=====>
         overlap = cv;
-        return (true);
+        return true;
       }
-      else
-      {
+      else {
         // Case 2 - *this:     +----------->
         //             cv:               +=====>
         overlap = *this;
 
-        if (overlap.is_directed_right())
-          overlap._source = cv.left();
-        else
-          overlap._target = cv.left();
+        if (overlap.is_directed_right()) overlap.m_source = cv.left();
+        else overlap.m_target = cv.left();
 
-        return (true);
+        return true;
       }
     }
-    else if (_is_strictly_between_endpoints (cv.right()))
-    {
+    else if (_is_strictly_between_endpoints(cv.right())) {
       // Case 3 - *this:     +----------->
       //             cv:   +=====>
       overlap = *this;
 
-      if (overlap.is_directed_right())
-        overlap._target = cv.right();
-      else
-        overlap._source = cv.right();
-
-      return (true);
-    }
-    else if (cv._is_between_endpoints (_source) &&
-             cv._is_between_endpoints (_target) &&
-             (cv._is_strictly_between_endpoints (_source) ||
-              cv._is_strictly_between_endpoints (_target)))
+      if (overlap.is_directed_right()) overlap.m_target = cv.right();
+      else overlap.m_source = cv.right();
+
+      return true;
+    }
+    else if (cv._is_between_endpoints(m_source) &&
+             cv._is_between_endpoints(m_target) &&
+             (cv._is_strictly_between_endpoints(m_source) ||
+              cv._is_strictly_between_endpoints(m_target)))
     {
       // Case 4 - *this:     +----------->
       //             cv:   +================>
       overlap = *this;
-      return (true);
+      return true;
     }
 
     // If we reached here, there are no overlaps:
-    return (false);
+    return false;
   }
 
   public:
@@ -2407,61 +2021,49 @@ protected:
 
     const double x_right = CGAL::to_double(this->target().x());
     const double y_right = CGAL::to_double(this->target().y());
-    if(this->is_linear())
-    {
-      *oi = std::make_pair(x_left, y_left);
-      ++oi;
-
-      *oi = std::make_pair(x_right, y_right);
-      ++oi;
+    if (this->is_linear()) {
+      *oi++ = std::make_pair(x_left, y_left);
+      *oi++ = std::make_pair(x_right, y_right);
       return;
     }
 
     // Otherwise, sample (n - 1) equally-spaced points in between.
-    const double  app_xcenter = CGAL::to_double (this->_first);
-    const double  app_ycenter = CGAL::to_double (this->_second);
-    const double  app_sqr_rad = CGAL::to_double (this->_third);
-
-    const double  x_jump = (x_right - x_left) / n;
-    double        x, y;
-    double        disc;
-    unsigned int        i;
+    const double app_xcenter = CGAL::to_double(this->m_first);
+    const double app_ycenter = CGAL::to_double(this->m_second);
+    const double app_sqr_rad = CGAL::to_double(this->m_third);
+
+    const double x_jump = (x_right - x_left) / n;
+    double x, y;
+    double disc;
+    unsigned int i;
 
     const bool is_up = this->_is_upper();
-    *oi = std::make_pair (x_left, y_left);   // The left point.
-    ++oi;
-    for (i = 1; i < n; i++)
-    {
+    *oi++ = std::make_pair (x_left, y_left);   // The left point.
+    for (i = 1; i < n; ++i) {
       x = x_left + x_jump*i;
       disc = app_sqr_rad - CGAL::square(x - app_xcenter);
       if (disc < 0) disc = 0;
-      if(is_up)
-        y = app_ycenter + std::sqrt(disc);
-      else
-        y = app_ycenter - std::sqrt(disc);
+      if(is_up) y = app_ycenter + std::sqrt(disc);
+      else y = app_ycenter - std::sqrt(disc);
 
-      *oi = std::make_pair(x, y);
-      ++oi;
+      *oi++ = std::make_pair(x, y);
     }
-    *oi = std::make_pair(x_right, y_right);   // The right point.
-    ++oi;
+    *oi++ = std::make_pair(x_right, y_right);   // The right point.
   }
 
-  /*!
-   * Trim the arc given its new endpoints.
+  /*! Trim the arc given its new endpoints.
    * \param ps The new source point.
    * \param pt The new target point.
    * \return The new trimmed arc.
    * \pre Both ps and pt lies on the arc and must conform with the current
    *      direction of the arc.
    */
-  Self trim (const Point_2& ps,
-             const Point_2& pt) const
+  Self trim(const Point_2& ps, const Point_2& pt) const
   {
-    Self  arc = *this;
+    Self arc = *this;
 
-    arc._source = ps;
-    arc._target = pt;
+    arc.m_source = ps;
+    arc.m_target = pt;
 
     return arc;
   }
@@ -2469,13 +2071,12 @@ protected:
   //@}
 };
 
-/*!
- * Exporter for circular arcs (or line segments).
+/*! Exporter for circular arcs (or line segments).
  */
 template <class Kernel, bool Filter>
 std::ostream&
-operator<< (std::ostream& os,
-            const _X_monotone_circle_segment_2<Kernel, Filter> & arc)
+operator<<(std::ostream& os,
+           const _X_monotone_circle_segment_2<Kernel, Filter> & arc)
 {
   if (! arc.is_linear())
     os << "(" << arc.supporting_circle() << ") ";
diff -pruN 5.0.3-1/include/CGAL/Arr_geometry_traits/Conic_arc_2.h 5.1-1/include/CGAL/Arr_geometry_traits/Conic_arc_2.h
--- 5.0.3-1/include/CGAL/Arr_geometry_traits/Conic_arc_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_geometry_traits/Conic_arc_2.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/Conic_arc_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/Conic_arc_2.h $
 // $Id: Conic_arc_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_geometry_traits/Conic_intersections_2.h 5.1-1/include/CGAL/Arr_geometry_traits/Conic_intersections_2.h
--- 5.0.3-1/include/CGAL/Arr_geometry_traits/Conic_intersections_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_geometry_traits/Conic_intersections_2.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/Conic_intersections_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/Conic_intersections_2.h $
 // $Id: Conic_intersections_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_geometry_traits/Conic_point_2.h 5.1-1/include/CGAL/Arr_geometry_traits/Conic_point_2.h
--- 5.0.3-1/include/CGAL/Arr_geometry_traits/Conic_point_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_geometry_traits/Conic_point_2.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/Conic_point_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/Conic_point_2.h $
 // $Id: Conic_point_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_geometry_traits/Conic_x_monotone_arc_2.h 5.1-1/include/CGAL/Arr_geometry_traits/Conic_x_monotone_arc_2.h
--- 5.0.3-1/include/CGAL/Arr_geometry_traits/Conic_x_monotone_arc_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_geometry_traits/Conic_x_monotone_arc_2.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,38 +3,36 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/Conic_x_monotone_arc_2.h $
-// $Id: Conic_x_monotone_arc_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/Conic_x_monotone_arc_2.h $
+// $Id: Conic_x_monotone_arc_2.h 6fcbee1 2020-04-21T17:12:21+03:00 Efi Fogel
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
-//
-// Author(s)     : Ron Wein <wein@post.tau.ac.il>
+// Author(s): Ron Wein <wein@post.tau.ac.il>
 
 #ifndef CGAL_CONIC_X_MONOTONE_ARC_2_H
 #define CGAL_CONIC_X_MONOTONE_ARC_2_H
 
 #include <CGAL/license/Arrangement_on_surface_2.h>
 
-
 /*! \file
  * Header file for the _Conic_x_monotone_arc_2<Conic_arc_2> class.
  */
 
-#include <CGAL/Arr_geometry_traits/Conic_intersections_2.h>
-
 #include <map>
 #include <ostream>
 
+#include <boost/variant.hpp>
+
+#include <CGAL/Arr_geometry_traits/Conic_intersections_2.h>
+
 namespace CGAL {
 
-/*!
- * Representation of an x-monotone conic arc.
+/*! Representation of an x-monotone conic arc.
  * The class is templated by a representation of a general bounded conic arc.
  */
 
-template <class Conic_arc_>
-class _Conic_x_monotone_arc_2 : private Conic_arc_
-{
+template <typename Conic_arc_>
+class _Conic_x_monotone_arc_2 : private Conic_arc_ {
 public:
 
   typedef Conic_arc_                              Conic_arc_2;
@@ -49,8 +47,8 @@ public:
   // Type definition for the intersection points mapping.
   typedef typename Conic_point_2::Conic_id        Conic_id;
   typedef std::pair<Conic_id, Conic_id>           Conic_pair;
-  typedef std::pair<Conic_point_2, unsigned int>  Intersection_point_2;
-  typedef std::list<Intersection_point_2>         Intersection_list;
+  typedef std::pair<Conic_point_2, unsigned int>  Intersection_point;
+  typedef std::list<Intersection_point>           Intersection_list;
 
   using Conic_arc_2::_sign_of_extra_data;
   using Conic_arc_2::_is_between_endpoints;
@@ -59,13 +57,12 @@ public:
   /*!
    * \struct Less functor for Conic_pair.
    */
-  struct Less_conic_pair
-  {
-    bool operator() (const Conic_pair& cp1, const Conic_pair& cp2) const
+  struct Less_conic_pair {
+    bool operator()(const Conic_pair& cp1, const Conic_pair& cp2) const
     {
       // Compare the pairs of IDs lexicographically.
-      return (cp1.first < cp2.first ||
-              (cp1.first == cp2.first && cp1.second < cp2.second));
+      return ((cp1.first < cp2.first) ||
+              ((cp1.first == cp2.first) && (cp1.second < cp2.second)));
     }
   };
 
@@ -85,8 +82,7 @@ protected:
 
   // Bit masks for the _info field (the two least significant bits are already
   // used by the base class).
-  enum
-  {
+  enum {
     IS_VERTICAL_SEGMENT = 4,
     IS_DIRECTED_RIGHT = 8,
     DEGREE_1 = 16,
@@ -99,14 +95,14 @@ protected:
     IS_SPECIAL_SEGMENT = 512
   };
 
-  Algebraic      alg_r;      // The coefficients of the supporting conic curve:
-  Algebraic      alg_s;      //
-  Algebraic      alg_t;      //   r*x^2 + s*y^2 + t*xy + u*x + v*y +w = 0 ,
-  Algebraic      alg_u;      //
-  Algebraic      alg_v;      // converted to algebraic numbers.
-  Algebraic      alg_w;      //
+  Algebraic alg_r;      // The coefficients of the supporting conic curve:
+  Algebraic alg_s;      //
+  Algebraic alg_t;      //   r*x^2 + s*y^2 + t*xy + u*x + v*y +w = 0 ,
+  Algebraic alg_u;      //
+  Algebraic alg_v;      // converted to algebraic numbers.
+  Algebraic alg_w;      //
 
-  Conic_id      _id;         // The ID number of the supporting conic curve.
+  Conic_id _id;         // The ID number of the supporting conic curve.
 
 public:
 
@@ -117,68 +113,62 @@ public:
    * Default constructor.
    */
   _Conic_x_monotone_arc_2 () :
-    Base (),
-    _id ()
+    Base(),
+    _id()
   {}
 
   /*!
    * Copy constructor.
    * \param arc The copied arc.
    */
-  _Conic_x_monotone_arc_2 (const Self& arc) :
-    Base (arc),
-    alg_r (arc.alg_r),
-    alg_s (arc.alg_s),
-    alg_t (arc.alg_t),
-    alg_u (arc.alg_u),
-    alg_v (arc.alg_v),
-    alg_w (arc.alg_w),
-    _id (arc._id)
+  _Conic_x_monotone_arc_2(const Self& arc) :
+    Base(arc),
+    alg_r(arc.alg_r),
+    alg_s(arc.alg_s),
+    alg_t(arc.alg_t),
+    alg_u(arc.alg_u),
+    alg_v(arc.alg_v),
+    alg_w(arc.alg_w),
+    _id(arc._id)
   {}
 
-  /*!
-   * Construct an x-monotone arc from a conic arc.
+  /*! Construct an x-monotone arc from a conic arc.
    * \param arc The given (base) arc.
    * \pre The given arc is x-monotone.
    */
-  _Conic_x_monotone_arc_2 (const Base& arc) :
-    Base (arc),
-    _id ()
+  _Conic_x_monotone_arc_2(const Base& arc) :
+    Base(arc),
+    _id()
   {
-    CGAL_precondition (arc.is_valid() && arc.is_x_monotone());
-
+    CGAL_precondition(arc.is_valid() && arc.is_x_monotone());
     _set ();
   }
 
-  /*!
-   * Construct an x-monotone arc from a conic arc.
+  /*! Construct an x-monotone arc from a conic arc.
    * \param arc The given (base) arc.
    * \param id The ID of the base arc.
    */
-  _Conic_x_monotone_arc_2 (const Base& arc,
-                           const Conic_id& id) :
-    Base (arc),
-    _id (id)
+  _Conic_x_monotone_arc_2(const Base& arc, const Conic_id& id) :
+    Base(arc),
+    _id(id)
   {
-    CGAL_precondition (arc.is_valid() && id.is_valid());
-
+    CGAL_precondition(arc.is_valid() && id.is_valid());
     _set ();
   }
 
-  /*!
-   * Construct an x-monotone sub-arc from a conic arc.
+  /*! Construct an x-monotone sub-arc from a conic arc.
    * \param arc The given (base) arc.
    * \param source The source point.
    * \param target The target point.
    * \param id The ID of the base arc.
    */
-  _Conic_x_monotone_arc_2 (const Base& arc,
-                           const Point_2& source, const Point_2& target,
-                           const Conic_id& id) :
-    Base (arc),
-    _id (id)
+  _Conic_x_monotone_arc_2(const Base& arc,
+                          const Point_2& source, const Point_2& target,
+                          const Conic_id& id) :
+    Base(arc),
+    _id(id)
   {
-    CGAL_precondition (arc.is_valid() && id.is_valid());
+    CGAL_precondition(arc.is_valid() && id.is_valid());
 
     // Set the two endpoints.
     this->_source = source;
@@ -193,7 +183,7 @@ public:
    * \param source The source point.
    * \param target The target point.
    */
-  _Conic_x_monotone_arc_2 (const Point_2& source, const Point_2& target) :
+  _Conic_x_monotone_arc_2(const Point_2& source, const Point_2& target) :
     Base()
   {
     // Set the basic properties and clear the _info bits.
@@ -204,22 +194,21 @@ public:
 
     // Check if the arc is directed right (the target is lexicographically
     // greater than the source point), or to the left.
-    Alg_kernel         ker;
-    Comparison_result  dir_res = ker.compare_xy_2_object() (this->_source,
-                                                            this->_target);
+    Alg_kernel ker;
+    Comparison_result dir_res =
+      ker.compare_xy_2_object()(this->_source, this->_target);
 
     CGAL_precondition (dir_res != EQUAL);
-    if (dir_res == EQUAL)
-      // Invalid arc:
-      return;
+    // Invalid arc:
+    if (dir_res == EQUAL) return;
 
     this->_info = (Conic_arc_2::IS_VALID | DEGREE_1);
     if (dir_res == SMALLER)
       this->_info = (this->_info | IS_DIRECTED_RIGHT);
 
     // Compose the equation of the underlying line.
-    const Algebraic        x1 = source.x(), y1 = source.y();
-    const Algebraic        x2 = target.x(), y2 = target.y();
+    const Algebraic x1 = source.x(), y1 = source.y();
+    const Algebraic x2 = target.x(), y2 = target.y();
 
     // The supporting line is A*x + B*y + C = 0, where:
     //
@@ -249,18 +238,18 @@ public:
    * \param source The source point.
    * \param target The target point.
    */
-  _Conic_x_monotone_arc_2 (const Algebraic& a,
-                           const Algebraic& b,
-                           const Algebraic& c,
-                           const Point_2& source, const Point_2& target) :
+  _Conic_x_monotone_arc_2(const Algebraic& a,
+                          const Algebraic& b,
+                          const Algebraic& c,
+                          const Point_2& source, const Point_2& target) :
     Base()
   {
     // Make sure the two endpoints lie on the supporting line.
-    CGAL_precondition (CGAL::sign (a * source.x() +
-                                   b * source.y() + c) == CGAL::ZERO);
+    CGAL_precondition(CGAL::sign(a * source.x() +
+                                 b * source.y() + c) == CGAL::ZERO);
 
-    CGAL_precondition (CGAL::sign (a * target.x() +
-                                   b * target.y() + c) == CGAL::ZERO);
+    CGAL_precondition(CGAL::sign(a * target.x() +
+                                 b * target.y() + c) == CGAL::ZERO);
 
     // Set the basic properties and clear the _info bits.
     this->_source = source;
@@ -270,23 +259,20 @@ public:
 
     // Check if the arc is directed right (the target is lexicographically
     // greater than the source point), or to the left.
-    Alg_kernel         ker;
-    Comparison_result  res = ker.compare_x_2_object() (this->_source,
-                                                       this->_target);
+    Alg_kernel ker;
+    Comparison_result res =
+      ker.compare_x_2_object()(this->_source, this->_target);
 
     this->_info = (Conic_arc_2::IS_VALID | DEGREE_1);
-    if (res == EQUAL)
-    {
+    if (res == EQUAL) {
       // Mark that the segment is vertical.
       this->_info = (this->_info | IS_VERTICAL_SEGMENT);
 
       // Compare the endpoints lexicographically.
-      res = ker.compare_y_2_object() (this->_source,
-                                      this->_target);
+      res = ker.compare_y_2_object()(this->_source, this->_target);
 
       CGAL_precondition (res != EQUAL);
-      if (res == EQUAL)
-      {
+      if (res == EQUAL) {
         // Invalid arc:
         this->_info = 0;
         return;
@@ -313,12 +299,11 @@ public:
    * Assignment operator.
    * \param arc The copied arc.
    */
-  const Self& operator= (const Self& arc)
+  const Self& operator=(const Self& arc)
   {
     CGAL_precondition (arc.is_valid());
 
-    if (this == &arc)
-      return (*this);
+    if (this == &arc) return (*this);
 
     // Copy the base arc.
     Base::operator= (arc);
@@ -343,49 +328,35 @@ public:
   /*!
    * Get the coefficients of the underlying conic.
    */
-  const Integer& r () const {return (this->_r);}
-  const Integer& s () const {return (this->_s);}
-  const Integer& t () const {return (this->_t);}
-  const Integer& u () const {return (this->_u);}
-  const Integer& v () const {return (this->_v);}
-  const Integer& w () const {return (this->_w);}
+  const Integer& r() const { return (this->_r); }
+  const Integer& s() const { return (this->_s); }
+  const Integer& t() const { return (this->_t); }
+  const Integer& u() const { return (this->_u); }
+  const Integer& v() const { return (this->_v); }
+  const Integer& w() const { return (this->_w); }
 
   /*!
    * Get the arc's source.
    * \return The source point.
    */
-  const Conic_point_2& source () const
-  {
-    return (this->_source);
-  }
+  const Conic_point_2& source() const { return (this->_source); }
 
-  /*!
-   * Get the arc's target.
+  /*! Get the arc's target.
    * \return The target point.
    */
-  const Conic_point_2& target () const
-  {
-    return (this->_target);
-  }
+  const Conic_point_2& target() const { return (this->_target); }
 
-  /*!
-   * Get the orientation of the arc.
+  /*! Get the orientation of the arc.
    * \return The orientation.
    */
-  Orientation orientation () const
-  {
-    return (this->_orient);
-  }
+  Orientation orientation() const { return (this->_orient); }
 
-  /*!
-   * Get the left endpoint of the arc.
+  /*! Get the left endpoint of the arc.
    */
   const Conic_point_2& left () const
   {
-    if ((this->_info & IS_DIRECTED_RIGHT) != 0)
-      return (this->_source);
-    else
-      return (this->_target);
+    if ((this->_info & IS_DIRECTED_RIGHT) != 0) return (this->_source);
+    else return (this->_target);
   }
 
   /*!
@@ -393,28 +364,21 @@ public:
    */
   const Conic_point_2& right () const
   {
-    if ((this->_info & IS_DIRECTED_RIGHT) != 0)
-      return (this->_target);
-    else
-      return (this->_source);
+    if ((this->_info & IS_DIRECTED_RIGHT) != 0) return (this->_target);
+    else return (this->_source);
   }
 
   /*!
    * Return true iff the conic arc is directed right iexicographically.
    */
   bool is_directed_right() const
-  {
-    return ((this->_info & IS_DIRECTED_RIGHT) != 0);
-  }
+  { return ((this->_info & IS_DIRECTED_RIGHT) != 0); }
 
   /*!
    * Get a bounding box for the conic arc.
    * \return The bounding box.
    */
-  Bbox_2 bbox () const
-  {
-    return (Base::bbox());
-  }
+  Bbox_2 bbox() const { return (Base::bbox()); }
   //@}
 
   /// \name Predicates.
@@ -433,23 +397,20 @@ public:
    * \param p The qury point.
    * \param (true) if p lies on the arc; (false) otherwise.
    */
-  bool contains_point (const Conic_point_2& p) const
+  bool contains_point(const Conic_point_2& p) const
   {
     // First check if p lies on the supporting conic. We first check whether
     // it is one of p's generating conic curves.
     bool       p_on_conic = false;
 
-    if (p.is_generating_conic (_id))
-    {
+    if (p.is_generating_conic(_id)) {
       p_on_conic = true;
     }
-    else
-    {
+    else {
       // Check whether p satisfies the supporting conic equation.
-      p_on_conic = _is_on_supporting_conic (p.x(), p.y());
+      p_on_conic = _is_on_supporting_conic(p.x(), p.y());
 
-      if (p_on_conic)
-      {
+      if (p_on_conic) {
         // As p lies on the supporting conic of our arc, add its ID to
         // the list of generating conics for p.
         Conic_point_2&  p_non_const = const_cast<Conic_point_2&> (p);
@@ -483,17 +444,16 @@ public:
       Alg_kernel   ker;
     );
 
-    CGAL_precondition (ker.compare_x_2_object() (p, left()) != SMALLER &&
-                       ker.compare_x_2_object() (p, right()) != LARGER);
+    CGAL_precondition(ker.compare_x_2_object() (p, left()) != SMALLER &&
+                      ker.compare_x_2_object() (p, right()) != LARGER);
 
-    if (_is_special_segment())
-    {
+    if (_is_special_segment()) {
       // In case of a special segment, the equation of the supported line
       // (a*x + b*y + c) = 0 is stored with the extra data field, and we
       // simply have:
-      Algebraic        _y = -(this->_extra_data_P->a*p.x() +
-                              this->_extra_data_P->c) /
-                            this->_extra_data_P->b;
+      Algebraic _y = -(this->_extra_data_P->a*p.x() +
+                       this->_extra_data_P->c) /
+        this->_extra_data_P->b;
 
       // Return the computed point.
       return (Point_2 (p.x(), _y));
@@ -501,18 +461,16 @@ public:
 
     // Compute the y-coordinate according to the degree of the supporting
     // conic curve.
-    Nt_traits        nt_traits;
-    Algebraic        y;
+    Nt_traits nt_traits;
+    Algebraic y;
 
-    if ((this->_info & DEGREE_MASK) == DEGREE_1)
-    {
+    if ((this->_info & DEGREE_MASK) == DEGREE_1) {
       // In case of a linear curve, the y-coordinate is a simple linear
       // expression of x(p) (note that v is not 0 as the arc is not vertical):
       //   y = -(u*x(p) + w) / v
       y = -(alg_u*p.x() + alg_w) / alg_v;
     }
-    else if (this->_orient == COLLINEAR)
-    {
+    else if (this->_orient == COLLINEAR) {
       CGAL_assertion (this->_extra_data_P != nullptr);
 
       // In this case the equation of the supporting line is given by the
@@ -520,26 +478,23 @@ public:
       y = -(this->_extra_data_P->a * p.x() +
             this->_extra_data_P->c) / this->_extra_data_P->b;
     }
-    else
-    {
-      CGAL_assertion ((this->_info & DEGREE_MASK) == DEGREE_2);
+    else {
+      CGAL_assertion((this->_info & DEGREE_MASK) == DEGREE_2);
 
       // In this case the y-coordinate is one of solutions to the quadratic
       // equation:
       //  s*y^2 + (t*x(p) + v)*y + (r*x(p)^2 + u*x(p) + w) = 0
-      Algebraic  A = alg_s;
-      Algebraic  B = alg_t*p.x() + alg_v;
-      Algebraic  C = (alg_r*p.x() + alg_u)*p.x() + alg_w;
+      Algebraic A = alg_s;
+      Algebraic B = alg_t*p.x() + alg_v;
+      Algebraic C = (alg_r*p.x() + alg_u)*p.x() + alg_w;
 
-      if (CGAL::sign(this->_s) == ZERO)
-      {
+      if (CGAL::sign(this->_s) == ZERO) {
         // In this case A is 0 and we have a linear equation.
         CGAL_assertion (CGAL::sign (B) != ZERO);
 
         y = -C / B;
       }
-      else
-      {
+      else {
         // Solve the quadratic equation.
         Algebraic  disc = B*B - 4*A*C;
 
@@ -547,13 +502,10 @@ public:
 
         // We take either the root involving -sqrt(disc) or +sqrt(disc)
         // based on the information flags.
-        if ((this->_info & PLUS_SQRT_DISC_ROOT) != 0)
-        {
+        if ((this->_info & PLUS_SQRT_DISC_ROOT) != 0) {
           y = (nt_traits.sqrt (disc) - B) / (2*A);
         }
-        else
-
-        {
+        else {
           y = -(B + nt_traits.sqrt (disc)) / (2*A);
         }
       }
@@ -579,169 +531,148 @@ public:
   {
     CGAL_precondition (n != 0);
 
-    const double  x_left = CGAL::to_double (left().x());
-    const double  y_left = CGAL::to_double (left().y());
-    const double  x_right = CGAL::to_double (right().x());
-    const double  y_right = CGAL::to_double (right().y());
+    const double x_left = CGAL::to_double (left().x());
+    const double y_left = CGAL::to_double (left().y());
+    const double x_right = CGAL::to_double (right().x());
+    const double y_right = CGAL::to_double (right().y());
 
-    if (this->_orient == COLLINEAR)
-    {
+    if (this->_orient == COLLINEAR) {
       // In case of a line segment, return the two endpoints.
-      *oi = std::pair<double, double> (x_left, y_left);
-      ++oi;
-      *oi = std::pair<double, double> (x_right, y_right);
-      ++oi;
-      return (oi);
+      *oi++ = std::pair<double, double> (x_left, y_left);
+      *oi++ = std::pair<double, double> (x_right, y_right);
+      return oi;
     }
 
     // Otherwise, sample (n - 1) equally-spaced points in between.
-    const double  app_r = CGAL::to_double (this->_r);
-    const double  app_s = CGAL::to_double (this->_s);
-    const double  app_t = CGAL::to_double (this->_t);
-    const double  app_u = CGAL::to_double (this->_u);
-    const double  app_v = CGAL::to_double (this->_v);
-    const double  app_w = CGAL::to_double (this->_w);
-    const double  x_jump = (x_right - x_left) / n;
-    double        x, y;
-    const bool    A_is_zero = (CGAL::sign(this->_s) == ZERO);
-    double        A = app_s, B, C;
-    double        disc;
-    size_t        i;
+    const double app_r = CGAL::to_double (this->_r);
+    const double app_s = CGAL::to_double (this->_s);
+    const double app_t = CGAL::to_double (this->_t);
+    const double app_u = CGAL::to_double (this->_u);
+    const double app_v = CGAL::to_double (this->_v);
+    const double app_w = CGAL::to_double (this->_w);
+    const double x_jump = (x_right - x_left) / n;
+    double       x, y;
+    const bool   A_is_zero = (CGAL::sign(this->_s) == ZERO);
+    double       A = app_s, B, C;
+    double       disc;
+    size_t       i;
 
-    *oi = std::pair<double, double> (x_left, y_left);   // The left point.
+    *oi = std::pair<double, double>(x_left, y_left);   // The left point.
     ++oi;
-    for (i = 1; i < n; i++)
-    {
+    for (i = 1; i < n; i++) {
       x = x_left + x_jump*i;
 
       // Solve the quadratic equation: A*x^2 + B*x + C = 0:
       B = app_t*x + app_v;
       C = (app_r*x + app_u)*x + app_w;
 
-      if (A_is_zero)
-      {
+      if (A_is_zero) {
         y = -C / B;
       }
-      else
-      {
+      else {
         disc = B*B - 4*A*C;
 
-        if (disc < 0)
-          disc = 0;
+        if (disc < 0) disc = 0;
 
         // We take either the root involving -sqrt(disc) or +sqrt(disc)
         // based on the information flags.
-        if ((this->_info & PLUS_SQRT_DISC_ROOT) != 0)
-        {
+        if ((this->_info & PLUS_SQRT_DISC_ROOT) != 0) {
           y = (std::sqrt(disc) - B) / (2*A);
         }
-        else
-        {
+        else {
           y = -(B + std::sqrt (disc)) / (2*A);
         }
       }
 
-      *oi = std::pair<double, double> (x, y);
-      ++oi;
+      *oi++ = std::pair<double, double> (x, y);
     }
-    *oi = std::pair<double, double> (x_right, y_right);   // The right point.
-    ++oi;
+    *oi++ = std::pair<double, double> (x_right, y_right);   // The right point.
 
-    return (oi);
+    return oi;
   }
 
-  /*!
-   * Compare to arcs immediately to the right of their intersection point.
+  /*! Compare to arcs immediately to the right of their intersection point.
    * \param arc The compared arc.
    * \param p The reference intersection point.
    * \return The relative position of the arcs to the right of p.
    * \pre Both arcs we compare are not vertical segments.
    */
-  Comparison_result compare_to_right (const Self& arc,
-                                      const Conic_point_2& p) const
+  Comparison_result compare_to_right(const Self& arc,
+                                     const Conic_point_2& p) const
   {
-    CGAL_precondition ((this->_info & IS_VERTICAL_SEGMENT) == 0 &&
-                       (arc._info & IS_VERTICAL_SEGMENT) == 0);
+    CGAL_precondition((this->_info & IS_VERTICAL_SEGMENT) == 0 &&
+                      (arc._info & IS_VERTICAL_SEGMENT) == 0);
 
     // In case one arc is facing upwards and another facing downwards, it is
     // clear that the one facing upward is above the one facing downwards.
-    if (_has_same_supporting_conic (arc))
-    {
+    if (_has_same_supporting_conic (arc)) {
       if ((this->_info & FACING_UP) != 0 && (arc._info & FACING_DOWN) != 0)
-        return (LARGER);
+        return LARGER;
       else if ((this->_info & FACING_DOWN)!= 0 && (arc._info & FACING_UP) != 0)
-        return (SMALLER);
+        return SMALLER;
 
       // In this case the two arcs overlap.
-      CGAL_assertion ((this->_info & FACING_MASK) ==
-                      (arc._info & FACING_MASK));
+      CGAL_assertion((this->_info & FACING_MASK) == (arc._info & FACING_MASK));
 
-      return (EQUAL);
+      return EQUAL;
     }
 
     // Compare the slopes of the two arcs at p, using their first-order
     // partial derivatives.
-    Algebraic      slope1_numer, slope1_denom;
-    Algebraic      slope2_numer, slope2_denom;
+    Algebraic slope1_numer, slope1_denom;
+    Algebraic slope2_numer, slope2_denom;
 
     _derive_by_x_at (p, 1, slope1_numer, slope1_denom);
     arc._derive_by_x_at (p, 1, slope2_numer, slope2_denom);
 
     // Check if any of the slopes is vertical.
-    const bool     is_vertical_slope1 = (CGAL::sign (slope1_denom) == ZERO);
-    const bool     is_vertical_slope2 = (CGAL::sign (slope2_denom) == ZERO);
+    const bool is_vertical_slope1 = (CGAL::sign(slope1_denom) == ZERO);
+    const bool is_vertical_slope2 = (CGAL::sign(slope2_denom) == ZERO);
 
-    if (!is_vertical_slope1 && !is_vertical_slope2)
-    {
+    if (!is_vertical_slope1 && !is_vertical_slope2) {
       // The two derivatives at p are well-defined: use them to determine
       // which arc is above the other (the one with a larger slope is below).
-      Comparison_result slope_res = CGAL::compare (slope1_numer*slope2_denom,
-                                                   slope2_numer*slope1_denom);
+      Comparison_result slope_res =
+        CGAL::compare(slope1_numer*slope2_denom, slope2_numer*slope1_denom);
 
-      if (slope_res != EQUAL)
-        return (slope_res);
+      if (slope_res != EQUAL) return (slope_res);
 
       // Use the second-order derivative.
-      _derive_by_x_at (p, 2, slope1_numer, slope1_denom);
-      arc._derive_by_x_at (p, 2, slope2_numer, slope2_denom);
+      _derive_by_x_at(p, 2, slope1_numer, slope1_denom);
+      arc._derive_by_x_at(p, 2, slope2_numer, slope2_denom);
 
-      slope_res = CGAL::compare (slope1_numer*slope2_denom,
-                                 slope2_numer*slope1_denom);
+      slope_res =
+        CGAL::compare(slope1_numer*slope2_denom, slope2_numer*slope1_denom);
 
-      if (slope_res != EQUAL)
-        return (slope_res);
+      if (slope_res != EQUAL) return (slope_res);
 
       // Use the third-order derivative.
-      _derive_by_x_at (p, 3, slope1_numer, slope1_denom);
-      arc._derive_by_x_at (p, 3, slope2_numer, slope2_denom);
+      _derive_by_x_at(p, 3, slope1_numer, slope1_denom);
+      arc._derive_by_x_at(p, 3, slope2_numer, slope2_denom);
 
-      slope_res = CGAL::compare (slope1_numer*slope2_denom,
-                                 slope2_numer*slope1_denom);
+      slope_res =
+        CGAL::compare(slope1_numer*slope2_denom, slope2_numer*slope1_denom);
 
       // \todo Handle higher-order derivatives:
       CGAL_assertion (slope_res != EQUAL);
 
       return (slope_res);
     }
-    else if (!is_vertical_slope2)
-    {
+    else if (!is_vertical_slope2) {
       // The first arc has a vertical slope at p: check whether it is
       // facing upwards or downwards and decide accordingly.
       CGAL_assertion ((this->_info & FACING_MASK) != 0);
 
-      if ((this->_info & FACING_UP) != 0)
-        return (LARGER);
-      return (SMALLER);
+      if ((this->_info & FACING_UP) != 0) return (LARGER);
+      return SMALLER;
     }
-    else if (!is_vertical_slope1)
-    {
+    else if (!is_vertical_slope1) {
       // The second arc has a vertical slope at p_int: check whether it is
       // facing upwards or downwards and decide accordingly.
       CGAL_assertion ((arc._info & FACING_MASK) != 0);
 
-      if ((arc._info & FACING_UP) != 0)
-        return (SMALLER);
-      return (LARGER);
+      if ((arc._info & FACING_UP) != 0) return (SMALLER);
+      return LARGER;
     }
 
     // The two arcs have vertical slopes at p_int:
@@ -750,31 +681,29 @@ public:
     if ((this->_info & FACING_UP) != 0 && (arc._info & FACING_DOWN) != 0)
       return (LARGER);
     else if ((this->_info & FACING_DOWN)!= 0 && (arc._info & FACING_UP)!= 0)
-      return (SMALLER);
+      return SMALLER;
 
     // Compute the second-order derivative by y and act according to it.
     _derive_by_y_at (p, 2, slope1_numer, slope1_denom);
     arc._derive_by_y_at (p, 2, slope2_numer, slope2_denom);
 
-    Comparison_result slope_res = CGAL::compare (slope1_numer*slope2_denom,
-                                                 slope2_numer*slope1_denom);
+    Comparison_result slope_res =
+      CGAL::compare(slope1_numer*slope2_denom, slope2_numer*slope1_denom);
 
     // If necessary, use the third-order derivative by y.
-    if (slope_res == EQUAL)
-    {
+    if (slope_res == EQUAL) {
       // \todo Check this!
-      _derive_by_y_at (p, 3, slope1_numer, slope1_denom);
-      arc._derive_by_y_at (p, 3, slope2_numer, slope2_denom);
+      _derive_by_y_at(p, 3, slope1_numer, slope1_denom);
+      arc._derive_by_y_at(p, 3, slope2_numer, slope2_denom);
 
-      slope_res = CGAL::compare (slope2_numer*slope1_denom,
-                                 slope1_numer*slope2_denom);
+      slope_res =
+        CGAL::compare(slope2_numer*slope1_denom, slope1_numer*slope2_denom);
     }
 
     // \todo Handle higher-order derivatives:
     CGAL_assertion(slope_res != EQUAL);
 
-    if ((this->_info & FACING_UP) != 0 && (arc._info & FACING_UP) != 0)
-    {
+    if ((this->_info & FACING_UP) != 0 && (arc._info & FACING_UP) != 0) {
       // Both are facing up.
       return ((slope_res == LARGER) ? SMALLER : LARGER);
     }
@@ -789,50 +718,45 @@ public:
    * \return The relative position of the arcs to the left of p.
    * \pre Both arcs we compare are not vertical segments.
    */
-  Comparison_result compare_to_left (const Self& arc,
-                                     const Conic_point_2& p) const
+  Comparison_result compare_to_left(const Self& arc,
+                                    const Conic_point_2& p) const
   {
-    CGAL_precondition ((this->_info & IS_VERTICAL_SEGMENT) == 0 &&
-                       (arc._info & IS_VERTICAL_SEGMENT) == 0);
+    CGAL_precondition((this->_info & IS_VERTICAL_SEGMENT) == 0 &&
+                      (arc._info & IS_VERTICAL_SEGMENT) == 0);
 
     // In case one arc is facing upwards and another facing downwards, it is
     // clear that the one facing upward is above the one facing downwards.
-    if (_has_same_supporting_conic (arc))
-    {
+    if (_has_same_supporting_conic (arc)) {
       if ((this->_info & FACING_UP) != 0 && (arc._info & FACING_DOWN) != 0)
-        return (LARGER);
+        return LARGER;
       else if ((this->_info & FACING_DOWN)!= 0 && (arc._info & FACING_UP)!= 0)
-        return (SMALLER);
+        return SMALLER;
 
       // In this case the two arcs overlap.
-      CGAL_assertion ((this->_info & FACING_MASK) ==
-                      (arc._info & FACING_MASK));
+      CGAL_assertion((this->_info & FACING_MASK) == (arc._info & FACING_MASK));
 
-      return (EQUAL);
+      return EQUAL;
     }
 
     // Compare the slopes of the two arcs at p, using their first-order
     // partial derivatives.
-    Algebraic      slope1_numer, slope1_denom;
-    Algebraic      slope2_numer, slope2_denom;
+    Algebraic slope1_numer, slope1_denom;
+    Algebraic slope2_numer, slope2_denom;
 
-    _derive_by_x_at (p, 1, slope1_numer, slope1_denom);
-    arc._derive_by_x_at (p, 1, slope2_numer, slope2_denom);
+    _derive_by_x_at(p, 1, slope1_numer, slope1_denom);
+    arc._derive_by_x_at(p, 1, slope2_numer, slope2_denom);
 
     // Check if any of the slopes is vertical.
-    const bool     is_vertical_slope1 = (CGAL::sign (slope1_denom) == ZERO);
+    const bool is_vertical_slope1 = (CGAL::sign (slope1_denom) == ZERO);
+    const bool is_vertical_slope2 = (CGAL::sign (slope2_denom) == ZERO);
 
-    const bool     is_vertical_slope2 = (CGAL::sign (slope2_denom) == ZERO);
-
-    if (!is_vertical_slope1 && !is_vertical_slope2)
-    {
+    if (!is_vertical_slope1 && !is_vertical_slope2) {
       // The two derivatives at p are well-defined: use them to determine
       // which arc is above the other (the one with a larger slope is below).
       Comparison_result  slope_res = CGAL::compare(slope2_numer*slope1_denom,
                                                    slope1_numer*slope2_denom);
 
-      if (slope_res != EQUAL)
-        return (slope_res);
+      if (slope_res != EQUAL) return (slope_res);
 
       // Use the second-order derivative.
       _derive_by_x_at (p, 2, slope1_numer, slope1_denom);
@@ -841,73 +765,66 @@ public:
       slope_res = CGAL::compare (slope1_numer*slope2_denom,
                                  slope2_numer*slope1_denom);
 
-      if (slope_res != EQUAL)
-        return (slope_res);
+      if (slope_res != EQUAL) return (slope_res);
 
       // Use the third-order derivative.
-      _derive_by_x_at (p, 3, slope1_numer, slope1_denom);
-      arc._derive_by_x_at (p, 3, slope2_numer, slope2_denom);
+      _derive_by_x_at(p, 3, slope1_numer, slope1_denom);
+      arc._derive_by_x_at(p, 3, slope2_numer, slope2_denom);
 
-      slope_res = CGAL::compare (slope2_numer*slope1_denom,
-                                 slope1_numer*slope2_denom);
+      slope_res = CGAL::compare(slope2_numer*slope1_denom,
+                                slope1_numer*slope2_denom);
 
       // \todo Handle higher-order derivatives:
       CGAL_assertion (slope_res != EQUAL);
 
       return (slope_res);
     }
-    else if (!is_vertical_slope2)
-    {
+    else if (!is_vertical_slope2) {
       // The first arc has a vertical slope at p: check whether it is
       // facing upwards or downwards and decide accordingly.
       CGAL_assertion ((this->_info & FACING_MASK) != 0);
 
-      if ((this->_info & FACING_UP) != 0)
-        return (LARGER);
-      return (SMALLER);
+      if ((this->_info & FACING_UP) != 0) return (LARGER);
+      return SMALLER;
     }
-    else if (!is_vertical_slope1)
-    {
+    else if (!is_vertical_slope1) {
       // The second arc has a vertical slope at p_int: check whether it is
       // facing upwards or downwards and decide accordingly.
       CGAL_assertion ((arc._info & FACING_MASK) != 0);
 
-      if ((arc._info & FACING_UP) != 0)
-        return (SMALLER);
-      return (LARGER);
+      if ((arc._info & FACING_UP) != 0) return (SMALLER);
+      return LARGER;
     }
 
     // The two arcs have vertical slopes at p_int:
     // First check whether one is facing up and one down. In this case the
     // comparison result is trivial.
     if ((this->_info & FACING_UP) != 0 && (arc._info & FACING_DOWN) != 0)
-      return (LARGER);
+      return LARGER;
     else if ((this->_info & FACING_DOWN)!= 0 && (arc._info & FACING_UP)!= 0)
-      return (SMALLER);
+      return SMALLER;
 
     // Compute the second-order derivative by y and act according to it.
-    _derive_by_y_at (p, 2, slope1_numer, slope1_denom);
-    arc._derive_by_y_at (p, 2, slope2_numer, slope2_denom);
+    _derive_by_y_at(p, 2, slope1_numer, slope1_denom);
+    arc._derive_by_y_at(p, 2, slope2_numer, slope2_denom);
 
-    Comparison_result  slope_res = CGAL::compare(slope2_numer*slope1_denom,
-                                                 slope1_numer*slope2_denom);
+    Comparison_result slope_res =
+      CGAL::compare(slope2_numer*slope1_denom, slope1_numer*slope2_denom);
 
     // If necessary, use the third-order derivative by y.
-    if (slope_res == EQUAL)
-    {
+    if (slope_res == EQUAL) {
       // \todo Check this!
-      _derive_by_y_at (p, 3, slope1_numer, slope1_denom);
-      arc._derive_by_y_at (p, 3, slope2_numer, slope2_denom);
+      _derive_by_y_at(p, 3, slope1_numer, slope1_denom);
+      arc._derive_by_y_at(p, 3, slope2_numer, slope2_denom);
 
-      slope_res = CGAL::compare (slope2_numer*slope1_denom,
-                                 slope1_numer*slope2_denom);
+      slope_res =
+        CGAL::compare(slope2_numer*slope1_denom, slope1_numer*slope2_denom);
     }
 
     // \todo Handle higher-order derivatives:
     CGAL_assertion(slope_res != EQUAL);
 
-    if ((this->_info & FACING_UP) != 0 && (arc._info & FACING_UP) != 0)
-    {
+    if ((this->_info & FACING_UP) != 0 && (arc._info & FACING_UP) != 0) {
       // Both are facing up.
       return ((slope_res == LARGER) ? SMALLER : LARGER);
     }
@@ -922,22 +839,21 @@ public:
    * \param oi The output iterator.
    * \return The past-the-end iterator.
    */
-  template<class OutputIterator>
-  OutputIterator intersect (const Self& arc,
-                            Intersection_map& inter_map,
-                            OutputIterator oi) const
+  template <typename OutputIterator>
+  OutputIterator intersect(const Self& arc,
+                           Intersection_map& inter_map,
+                           OutputIterator oi) const
   {
-    if (_has_same_supporting_conic (arc))
-    {
+    typedef boost::variant<Intersection_point, Self>  Intersection_result;
+
+    if (_has_same_supporting_conic(arc)) {
       // Check for overlaps between the two arcs.
-      Self    overlap;
+      Self overlap;
 
-      if (_compute_overlap (arc, overlap))
-      {
+      if (_compute_overlap(arc, overlap)) {
         // There can be just a single overlap between two x-monotone arcs:
-        *oi = make_object (overlap);
-        oi++;
-        return (oi);
+        *oi++ = Intersection_result(overlap);
+        return oi;
       }
 
       // In case there is not overlap and the supporting conics are the same,
@@ -947,80 +863,62 @@ public:
       // intersection points we report.
       Alg_kernel  ker;
 
-      if (ker.equal_2_object() (left(), arc.left()))
-      {
-        Intersection_point_2  ip (left(), 0);
-
-        *oi = make_object (ip);
-        oi++;
+      if (ker.equal_2_object()(left(), arc.left())) {
+        Intersection_point ip(left(), 0);
+        *oi++ = Intersection_result(ip);
       }
 
-      if (ker.equal_2_object() (right(), arc.right()))
-      {
-        Intersection_point_2  ip (right(), 0);
-
-        *oi = make_object (ip);
-        oi++;
+      if (ker.equal_2_object()(right(), arc.right())) {
+        Intersection_point ip(right(), 0);
+        *oi++ = Intersection_result(ip);
       }
 
-      return (oi);
+      return oi;
     }
 
     // Search for the pair of supporting conics in the map (the first conic
     // ID in the pair should be smaller than the second one, to guarantee
     // uniqueness).
-    Conic_pair                   conic_pair;
-    Intersection_map_iterator    map_iter;
-    Intersection_list            inter_list;
-    bool                         invalid_ids = false;
-
-    if (_id.is_valid() && arc._id.is_valid())
-    {
-      if (_id < arc._id)
-        conic_pair = Conic_pair (_id, arc._id);
-      else
-        conic_pair = Conic_pair (arc._id, _id);
-
+    Conic_pair conic_pair;
+    Intersection_map_iterator map_iter;
+    Intersection_list inter_list;
+    bool invalid_ids = false;
+
+    if (_id.is_valid() && arc._id.is_valid()) {
+      if (_id < arc._id) conic_pair = Conic_pair (_id, arc._id);
+      else conic_pair = Conic_pair (arc._id, _id);
       map_iter = inter_map.find (conic_pair);
     }
-    else
-    {
+    else {
       // In case one of the IDs is invalid, we do not look in the map neither
       // we cache the results.
       map_iter = inter_map.end();
       invalid_ids = true;
     }
 
-    if (map_iter == inter_map.end())
-    {
+    if (map_iter == inter_map.end()) {
       // In case the intersection points between the supporting conics have
       // not been computed before, compute them now and store them in the map.
-      _intersect_supporting_conics (arc, inter_list);
+      _intersect_supporting_conics(arc, inter_list);
 
-      if (! invalid_ids)
-        inter_map[conic_pair] = inter_list;
+      if (! invalid_ids) inter_map[conic_pair] = inter_list;
     }
-    else
-    {
+    else {
       // Obtain the precomputed intersection points from the map.
       inter_list = (*map_iter).second;
     }
 
     // Go over the list of intersection points and report those that lie on
     // both x-monotone arcs.
-    typename Intersection_list::const_iterator  iter;
-
-    for (iter = inter_list.begin(); iter != inter_list.end(); ++iter)
-    {
-      if (_is_between_endpoints ((*iter).first) &&
-          arc._is_between_endpoints ((*iter).first))
+    for (auto iter = inter_list.begin(); iter != inter_list.end(); ++iter) {
+      if (_is_between_endpoints((*iter).first) &&
+          arc._is_between_endpoints((*iter).first))
       {
-        *oi = make_object (*iter);
-        ++oi;
+        *oi++ = Intersection_result(*iter);
       }
     }
 
-    return (oi);
+    return oi;
   }
   //@}
 
@@ -1034,13 +932,10 @@ public:
    * \param c2 Output: The first resulting arc, lying to the right of p.
    * \pre p lies in the interior of the arc (not one of its endpoints).
    */
-  void split (const Conic_point_2& p,
-              Self& c1, Self& c2) const
+  void split(const Conic_point_2& p, Self& c1, Self& c2) const
   {
     // Make sure that p lies on the interior of the arc.
-    CGAL_precondition_code (
-      Alg_kernel   ker;
-    );
+    CGAL_precondition_code(Alg_kernel ker);
 
     CGAL_precondition (this->contains_point (p) &&
                        ! ker.equal_2_object() (p, this->_source) &&
@@ -1058,8 +953,7 @@ public:
       c1._target = p;
       c2._source = p;
 
-      if (! p.is_generating_conic (_id))
-      {
+      if (! p.is_generating_conic (_id)) {
         c1._target.set_generating_conic (_id);
         c2._source.set_generating_conic (_id);
       }
@@ -1071,8 +965,7 @@ public:
       c1._source = p;
       c2._target = p;
 
-      if (! p.is_generating_conic (_id))
-      {
+      if (! p.is_generating_conic (_id)) {
         c1._source.set_generating_conic (_id);
         c2._target.set_generating_conic (_id);
       }
@@ -1085,16 +978,14 @@ public:
    * Flip the arc.
    * \return An arc with swapped source and target and a reverse orienation.
    */
-  Self flip () const
+  Self flip() const
   {
     // Make a copy of the current arc.
-    Self    arc = *this;
+    Self arc = *this;
 
     // Reverse the orientation.
-    if (this->_orient == CLOCKWISE)
-      arc._orient = COUNTERCLOCKWISE;
-    else if (this->_orient == COUNTERCLOCKWISE)
-      arc._orient = CLOCKWISE;
+    if (this->_orient == CLOCKWISE) arc._orient = COUNTERCLOCKWISE;
+    else if (this->_orient == COUNTERCLOCKWISE) arc._orient = CLOCKWISE;
 
     // Swap the source and the target.
     arc._source = this->_target;
@@ -1103,7 +994,7 @@ public:
     // Change the direction bit among the information flags.
     arc._info = (this->_info ^ IS_DIRECTED_RIGHT);
 
-    return (arc);
+    return arc;
   }
 
   /*!
@@ -1114,12 +1005,11 @@ public:
    * \pre Both ps and pt lies on the arc and must conform with the current
    *      direction of the arc.
    */
-  Self trim (const Conic_point_2& ps,
-             const Conic_point_2& pt) const
+  Self trim(const Conic_point_2& ps, const Conic_point_2& pt) const
   {
     // Make sure that both ps and pt lie on the arc.
-    CGAL_precondition (this->contains_point (ps) &&
-                       this->contains_point (pt));
+    CGAL_precondition(this->contains_point (ps) &&
+                      this->contains_point (pt));
 
     // Make sure that the endpoints conform with the direction of the arc.
     Self         arc = *this;
@@ -1136,8 +1026,7 @@ public:
     }
 
     // Make a copy of the current arc and assign its endpoints.
-    if (! ker.equal_2_object() (ps, this->_source))
-    {
+    if (! ker.equal_2_object() (ps, this->_source)) {
       arc._source = ps;
 
       if (! ps.is_generating_conic (_id))
@@ -1164,46 +1053,41 @@ public:
   {
     // The two arc must have the same supporting conic curves.
     if (! _has_same_supporting_conic (arc))
-      return (false);
+      return false;
 
     // Check that the arc endpoints are the same.
-    Alg_kernel   ker;
+    Alg_kernel ker;
 
-    if(this->_orient == COLLINEAR)
-    {
+    if (this->_orient == COLLINEAR) {
       CGAL_assertion(arc._orient == COLLINEAR);
-      return((ker.equal_2_object() (this->_source, arc._source) &&
-              ker.equal_2_object() (this->_target, arc._target)) ||
-              (ker.equal_2_object() (this->_source, arc._target) &&
-               ker.equal_2_object() (this->_target, arc._source)));
+      return((ker.equal_2_object()(this->_source, arc._source) &&
+              ker.equal_2_object()(this->_target, arc._target)) ||
+              (ker.equal_2_object()(this->_source, arc._target) &&
+               ker.equal_2_object()(this->_target, arc._source)));
     }
 
-    if (this->_orient == arc._orient)
-    {
+    if (this->_orient == arc._orient) {
       // Same orientation - the source and target points must be the same.
-      return (ker.equal_2_object() (this->_source, arc._source) &&
-              ker.equal_2_object() (this->_target, arc._target));
+      return (ker.equal_2_object()(this->_source, arc._source) &&
+              ker.equal_2_object()(this->_target, arc._target));
     }
-    else
-    {
+    else {
       // Reverse orientation - the source and target points must be swapped.
-      return (ker.equal_2_object() (this->_source, arc._target) &&
-              ker.equal_2_object() (this->_target, arc._source));
+      return (ker.equal_2_object()(this->_source, arc._target) &&
+              ker.equal_2_object()(this->_target, arc._source));
     }
   }
 
-  /*!
-   * Check whether it is possible to merge the arc with the given arc.
+  /*! Check whether it is possible to merge the arc with the given arc.
    * \param arc The query arc.
    * \return (true) if it is possible to merge the two arcs;
    *         (false) otherwise.
    */
-  bool can_merge_with (const Self& arc) const
+  bool can_merge_with(const Self& arc) const
   {
     // In order to merge the two arcs, they should have the same supporting
     // conic.
-    if (! _has_same_supporting_conic (arc))
-      return (false);
+    if (! _has_same_supporting_conic(arc)) return false;
 
     // Check if the left endpoint of one curve is the right endpoint of the
     // other.
@@ -1213,28 +1097,23 @@ public:
             ker.equal_2_object() (left(), arc.right()));
   }
 
-  /*!
-   * Merge the current arc with the given arc.
+  /*! Merge the current arc with the given arc.
    * \param arc The arc to merge with.
    * \pre The two arcs are mergeable.
    */
-  void merge (const Self& arc)
+  void merge(const Self& arc)
   {
     CGAL_precondition (this->can_merge_with (arc));
 
     // Check if we should extend the arc to the left or to the right.
     Alg_kernel   ker;
 
-    if (ker.equal_2_object() (right(), arc.left()))
-    {
+    if (ker.equal_2_object() (right(), arc.left())) {
       // Extend the arc to the right.
-      if ((this->_info & IS_DIRECTED_RIGHT) != 0)
-        this->_target = arc.right();
-      else
-        this->_source = arc.right();
+      if ((this->_info & IS_DIRECTED_RIGHT) != 0) this->_target = arc.right();
+      else this->_source = arc.right();
     }
-    else
-    {
+    else {
       CGAL_precondition (ker.equal_2_object() (left(), arc.right()));
 
       // Extend the arc to the left.
@@ -1835,36 +1714,32 @@ private:
    * \param arc The arc to intersect with.
    * \param inter_list The list of intersection points.
    */
-  void _intersect_supporting_conics (const Self& arc,
-                                     Intersection_list& inter_list) const
+  void _intersect_supporting_conics(const Self& arc,
+                                    Intersection_list& inter_list) const
   {
-    if (_is_special_segment() && ! arc._is_special_segment())
-    {
+    if (_is_special_segment() && ! arc._is_special_segment()) {
       // If one of the arcs is a special segment, make sure it is (arc).
-      arc._intersect_supporting_conics (*this, inter_list);
+      arc._intersect_supporting_conics(*this, inter_list);
       return;
     }
 
-    const int   deg1 = ((this->_info & DEGREE_MASK) == DEGREE_1) ? 1 : 2;
-    const int   deg2 = ((arc._info & DEGREE_MASK) == DEGREE_1) ? 1 : 2;
-    Nt_traits   nt_traits;
-    Algebraic   xs[4];
-    int         n_xs = 0;
-    Algebraic   ys[4];
-    int         n_ys = 0;
+    const int deg1 = ((this->_info & DEGREE_MASK) == DEGREE_1) ? 1 : 2;
+    const int deg2 = ((arc._info & DEGREE_MASK) == DEGREE_1) ? 1 : 2;
+    Nt_traits nt_traits;
+    Algebraic xs[4];
+    int n_xs = 0;
+    Algebraic ys[4];
+    int n_ys = 0;
 
-    if (arc._is_special_segment())
-    {
+    if (arc._is_special_segment()) {
       // The second arc is a special segment (a*x + b*y + c = 0).
-      if (_is_special_segment())
-      {
+      if (_is_special_segment()) {
         // Both arc are sepcial segment, so they have at most one intersection
         // point.
-        Algebraic   denom = this->_extra_data_P->a * arc._extra_data_P->b -
-                            this->_extra_data_P->b * arc._extra_data_P->a;
+        Algebraic denom = this->_extra_data_P->a * arc._extra_data_P->b -
+          this->_extra_data_P->b * arc._extra_data_P->a;
 
-        if (CGAL::sign (denom) != CGAL::ZERO)
-        {
+        if (CGAL::sign (denom) != CGAL::ZERO) {
           xs[0] = (this->_extra_data_P->b * arc._extra_data_P->c -
                    this->_extra_data_P->c * arc._extra_data_P->b) / denom;
           n_xs = 1;
@@ -1874,8 +1749,7 @@ private:
           n_ys = 1;
         }
       }
-      else
-      {
+      else {
         // Compute the x-coordinates of the intersection points.
         n_xs = _compute_resultant_roots (nt_traits,
                                          alg_r, alg_s, alg_t,
@@ -1899,8 +1773,7 @@ private:
         CGAL_assertion (n_ys <= 2);
       }
     }
-    else
-    {
+    else {
       // Compute the x-coordinates of the intersection points.
       n_xs = _compute_resultant_roots (nt_traits,
                                        this->_r, this->_s, this->_t,
@@ -1921,7 +1794,7 @@ private:
                                        arc._v, arc._u, arc._w,
                                        deg2,
                                        ys);
-      CGAL_assertion (n_ys <= 4);
+      CGAL_assertion(n_ys <= 4);
     }
 
     // Pair the coordinates of the intersection points. As the vectors of
@@ -1938,14 +1811,14 @@ private:
       if (n_xs == 1 && n_ys == 1)
       {
         // Single intersection.
-        Conic_point_2         ip (xs[0], ys[0]);
+        Conic_point_2 ip (xs[0], ys[0]);
 
         ip.set_generating_conic (_id);
         ip.set_generating_conic (arc._id);
 
         // In case the other curve is of degree 2, this is a tangency point.
         mult = (deg1 == 1 || _is_special_segment()) ? 1 : 2;
-        inter_list.push_back (Intersection_point_2 (ip, mult));
+        inter_list.push_back(Intersection_point (ip, mult));
       }
       else if (n_xs == 1 && n_ys == 2)
       {
@@ -1954,14 +1827,14 @@ private:
         ip1.set_generating_conic (_id);
         ip1.set_generating_conic (arc._id);
 
-        inter_list.push_back (Intersection_point_2 (ip1, 1));
+        inter_list.push_back(Intersection_point (ip1, 1));
 
         Conic_point_2         ip2 (xs[0], ys[1]);
 
         ip2.set_generating_conic (_id);
         ip2.set_generating_conic (arc._id);
 
-        inter_list.push_back (Intersection_point_2 (ip2, 1));
+        inter_list.push_back(Intersection_point (ip2, 1));
       }
       else if (n_xs == 2 && n_ys == 1)
       {
@@ -1970,24 +1843,23 @@ private:
         ip1.set_generating_conic (_id);
         ip1.set_generating_conic (arc._id);
 
-        inter_list.push_back (Intersection_point_2 (ip1, 1));
+        inter_list.push_back(Intersection_point (ip1, 1));
 
         Conic_point_2         ip2 (xs[1], ys[0]);
 
         ip2.set_generating_conic (_id);
         ip2.set_generating_conic (arc._id);
 
-        inter_list.push_back (Intersection_point_2 (ip2, 1));
+        inter_list.push_back(Intersection_point (ip2, 1));
 
       }
-      else
-      {
+      else {
         CGAL_assertion (n_xs == 2 && n_ys == 2);
 
         // The x-coordinates and the y-coordinates are given in ascending
         // order. If the slope of the segment is positive, we pair the
         // coordinates as is - otherwise, we swap the pairs.
-        int                   ind_first_y = 0, ind_second_y = 1;
+        int ind_first_y = 0, ind_second_y = 1;
 
         if (CGAL::sign (arc._extra_data_P->b) ==
             CGAL::sign(arc._extra_data_P->a))
@@ -1996,45 +1868,41 @@ private:
           ind_second_y = 0;
         }
 
-        Conic_point_2         ip1 (xs[0], ys[ind_first_y]);
+        Conic_point_2 ip1(xs[0], ys[ind_first_y]);
 
-        ip1.set_generating_conic (_id);
-        ip1.set_generating_conic (arc._id);
+        ip1.set_generating_conic(_id);
+        ip1.set_generating_conic(arc._id);
 
-        inter_list.push_back (Intersection_point_2 (ip1, 1));
+        inter_list.push_back(Intersection_point (ip1, 1));
 
-        Conic_point_2         ip2 (xs[1], ys[ind_second_y]);
+        Conic_point_2 ip2(xs[1], ys[ind_second_y]);
 
         ip2.set_generating_conic (_id);
         ip2.set_generating_conic (arc._id);
 
-        inter_list.push_back (Intersection_point_2 (ip2, 1));
+        inter_list.push_back(Intersection_point(ip2, 1));
       }
 
       return;
     }
 
-    for (i = 0; i < n_xs; i++)
-    {
-      for (j = 0; j < n_ys; j++)
-      {
+    for (i = 0; i < n_xs; i++) {
+      for (j = 0; j < n_ys; j++) {
         if (_is_on_supporting_conic (xs[i], ys[j]) &&
             arc._is_on_supporting_conic (xs[i], ys[j]))
         {
           // Create the intersection point and set its generating conics.
-          Conic_point_2         ip (xs[i], ys[j]);
+          Conic_point_2 ip(xs[i], ys[j]);
 
           ip.set_generating_conic (_id);
           ip.set_generating_conic (arc._id);
 
           // Compute the multiplicity of the intersection point.
-          if (deg1 == 1 && deg2 == 1)
-            mult = 1;
-          else
-            mult = _multiplicity_of_intersection_point (arc, ip);
+          if (deg1 == 1 && deg2 == 1) mult = 1;
+          else mult = _multiplicity_of_intersection_point(arc, ip);
 
           // Insert the intersection point to the output list.
-          inter_list.push_back (Intersection_point_2 (ip, mult));
+          inter_list.push_back(Intersection_point(ip, mult));
         }
       }
     }
diff -pruN 5.0.3-1/include/CGAL/Arr_geometry_traits/Consolidated_curve_data_aux.h 5.1-1/include/CGAL/Arr_geometry_traits/Consolidated_curve_data_aux.h
--- 5.0.3-1/include/CGAL/Arr_geometry_traits/Consolidated_curve_data_aux.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_geometry_traits/Consolidated_curve_data_aux.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/Consolidated_curve_data_aux.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/Consolidated_curve_data_aux.h $
 // $Id: Consolidated_curve_data_aux.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_geometry_traits/Curve_data_aux.h 5.1-1/include/CGAL/Arr_geometry_traits/Curve_data_aux.h
--- 5.0.3-1/include/CGAL/Arr_geometry_traits/Curve_data_aux.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_geometry_traits/Curve_data_aux.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/Curve_data_aux.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/Curve_data_aux.h $
 // $Id: Curve_data_aux.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_geometry_traits/de_Casteljau_2.h 5.1-1/include/CGAL/Arr_geometry_traits/de_Casteljau_2.h
--- 5.0.3-1/include/CGAL/Arr_geometry_traits/de_Casteljau_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_geometry_traits/de_Casteljau_2.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/de_Casteljau_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/de_Casteljau_2.h $
 // $Id: de_Casteljau_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_geometry_traits/IO/Polycurve_2_iostream.h 5.1-1/include/CGAL/Arr_geometry_traits/IO/Polycurve_2_iostream.h
--- 5.0.3-1/include/CGAL/Arr_geometry_traits/IO/Polycurve_2_iostream.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_geometry_traits/IO/Polycurve_2_iostream.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/IO/Polycurve_2_iostream.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/IO/Polycurve_2_iostream.h $
 // $Id: Polycurve_2_iostream.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_geometry_traits/One_root_number.h 5.1-1/include/CGAL/Arr_geometry_traits/One_root_number.h
--- 5.0.3-1/include/CGAL/Arr_geometry_traits/One_root_number.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_geometry_traits/One_root_number.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/One_root_number.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/One_root_number.h $
 // $Id: One_root_number.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_geometry_traits/Polycurve_2.h 5.1-1/include/CGAL/Arr_geometry_traits/Polycurve_2.h
--- 5.0.3-1/include/CGAL/Arr_geometry_traits/Polycurve_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_geometry_traits/Polycurve_2.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/Polycurve_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/Polycurve_2.h $
 // $Id: Polycurve_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_geometry_traits/Polyline_2.h 5.1-1/include/CGAL/Arr_geometry_traits/Polyline_2.h
--- 5.0.3-1/include/CGAL/Arr_geometry_traits/Polyline_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_geometry_traits/Polyline_2.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/Polyline_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/Polyline_2.h $
 // $Id: Polyline_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_geometry_traits/Rational_arc_2.h 5.1-1/include/CGAL/Arr_geometry_traits/Rational_arc_2.h
--- 5.0.3-1/include/CGAL/Arr_geometry_traits/Rational_arc_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_geometry_traits/Rational_arc_2.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/Rational_arc_2.h $
-// $Id: Rational_arc_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/Rational_arc_2.h $
+// $Id: Rational_arc_2.h 58276ed 2020-03-31T18:34:28+03:00 Efi Fogel
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -1830,7 +1830,7 @@ public:
   typedef typename Base::Rat_vector               Rat_vector;
   typedef typename Base::Polynomial               Polynomial;
 
-  typedef std::pair<Point_2,Multiplicity>        Intersection_point_2;
+  typedef std::pair<Point_2,Multiplicity>        Intersection_point;
 
 
   /// \name Constrcution methods.
@@ -1967,138 +1967,118 @@ public:
     CGAL_precondition (this->is_valid() && this->is_continuous());
     CGAL_precondition (arc.is_valid() && arc.is_continuous());
 
-    if (this->_has_same_base (arc))
-    {
-      Alg_kernel       ker;
+    if (this->_has_same_base (arc)) {
+      Alg_kernel ker;
 
       // Get the left and right endpoints of (*this) and their information
       // bits.
-      const Point_2&   left1 = (this->is_directed_right() ?
-                                this->_ps : this->_pt);
-      const Point_2&   right1 = (this->is_directed_right() ?
-                                 this->_pt : this->_ps);
-      int              info_left1, info_right1;
+      const Point_2& left1 =
+        (this->is_directed_right() ? this->_ps : this->_pt);
+      const Point_2& right1 =
+        (this->is_directed_right() ? this->_pt : this->_ps);
+      int info_left1, info_right1;
 
-      if (this->is_directed_right())
-      {
+      if (this->is_directed_right()) {
         info_left1 = (this->_info & this->SRC_INFO_BITS);
         info_right1 = ((this->_info & this->TRG_INFO_BITS) >> 4);
       }
-      else
-      {
+      else {
         info_right1 = (this->_info & this->SRC_INFO_BITS);
         info_left1 = ((this->_info & this->TRG_INFO_BITS) >> 4);
       }
 
       // Get the left and right endpoints of the other arc and their
       // information bits.
-      const Point_2&   left2 = (arc.is_directed_right() ? arc._ps : arc._pt);
-      const Point_2&   right2 = (arc.is_directed_right() ? arc._pt : arc._ps);
-      int              info_left2, info_right2;
+      const Point_2& left2 = (arc.is_directed_right() ? arc._ps : arc._pt);
+      const Point_2& right2 = (arc.is_directed_right() ? arc._pt : arc._ps);
+      int info_left2, info_right2;
 
-      if (arc.is_directed_right())
-      {
+      if (arc.is_directed_right()) {
         info_left2 = (arc._info & this->SRC_INFO_BITS);
         info_right2 = ((arc._info & this->TRG_INFO_BITS) >> 4);
       }
-      else
-      {
+      else {
         info_right2 = (arc._info & this->SRC_INFO_BITS);
         info_left2 = ((arc._info & this->TRG_INFO_BITS) >> 4);
       }
 
       // Locate the left curve-end with larger x-coordinate.
-      bool             at_minus_infinity = false;
-      Arr_parameter_space    inf_l1 = this->left_infinite_in_x();
-      Arr_parameter_space    inf_l2 = arc.left_infinite_in_x();
-      Point_2          p_left;
-      int              info_left;
+      bool at_minus_infinity = false;
+      Arr_parameter_space inf_l1 = this->left_infinite_in_x();
+      Arr_parameter_space inf_l2 = arc.left_infinite_in_x();
+      Point_2 p_left;
+      int info_left;
 
-      if (inf_l1 == ARR_INTERIOR && inf_l2 == ARR_INTERIOR)
-      {
+      if (inf_l1 == ARR_INTERIOR && inf_l2 == ARR_INTERIOR) {
         // Let p_left be the rightmost of the two left endpoints.
-        if (ker.compare_x_2_object() (left1, left2) == LARGER)
-        {
+        if (ker.compare_x_2_object() (left1, left2) == LARGER) {
           p_left = left1;
           info_left = info_left1;
         }
-        else
-        {
+        else {
           p_left = left2;
           info_left = info_left2;
         }
       }
-      else if (inf_l1 == ARR_INTERIOR)
-      {
+      else if (inf_l1 == ARR_INTERIOR) {
         // Let p_left be the left endpoint of (*this).
         p_left = left1;
         info_left = info_left1;
       }
-      else if (inf_l2 == ARR_INTERIOR)
-      {
+      else if (inf_l2 == ARR_INTERIOR) {
         // Let p_left be the left endpoint of the other arc.
         p_left = left2;
         info_left = info_left2;
       }
-      else
-      {
+      else {
         // Both arcs are defined at x = -oo.
         at_minus_infinity = true;
         info_left = info_left1;
       }
 
       // Locate the right curve-end with smaller x-coordinate.
-      bool             at_plus_infinity = false;
-      Arr_parameter_space    inf_r1 = this->right_infinite_in_x();
-      Arr_parameter_space    inf_r2 = arc.right_infinite_in_x();
-      Point_2          p_right;
-      int              info_right;
+      bool at_plus_infinity = false;
+      Arr_parameter_space inf_r1 = this->right_infinite_in_x();
+      Arr_parameter_space inf_r2 = arc.right_infinite_in_x();
+      Point_2 p_right;
+      int info_right;
 
-      if (inf_r1 == ARR_INTERIOR && inf_r2 == ARR_INTERIOR)
-      {
+      if (inf_r1 == ARR_INTERIOR && inf_r2 == ARR_INTERIOR) {
         // Let p_right be the rightmost of the two right endpoints.
-        if (ker.compare_x_2_object() (right1, right2) == SMALLER)
-        {
+        if (ker.compare_x_2_object() (right1, right2) == SMALLER) {
           p_right = right1;
           info_right = info_right1;
         }
-        else
-        {
+        else {
           p_right = right2;
           info_right = info_right2;
         }
       }
-      else if (inf_r1 == ARR_INTERIOR)
-      {
+      else if (inf_r1 == ARR_INTERIOR) {
         // Let p_right be the right endpoint of (*this).
         p_right = right1;
         info_right = info_right1;
       }
-      else if (inf_r2 == ARR_INTERIOR)
-      {
+      else if (inf_r2 == ARR_INTERIOR) {
         // Let p_right be the right endpoint of the other arc.
         p_right = right2;
         info_right = info_right2;
       }
-      else
-      {
+      else {
         // Both arcs are defined at x = +oo.
         at_plus_infinity = true;
         info_right = info_right2;
       }
 
       // Check the case of two bounded (in x) ends.
-      if (! at_minus_infinity && ! at_plus_infinity)
-      {
+      if (! at_minus_infinity && ! at_plus_infinity) {
         Comparison_result res = ker.compare_x_2_object() (p_left, p_right);
 
-        if (res == LARGER)
-        {
+        if (res == LARGER) {
           // The x-range of the overlap is empty, so there is no overlap.
-          return (oi);
+          return oi;
         }
-        else if (res == EQUAL)
-        {
+        if (res == EQUAL) {
           // We have a single overlapping point. Just make sure this point
           // is not at y = -/+ oo.
           if (info_left &&
@@ -2106,20 +2086,19 @@ public:
               info_right &&
               (this->SRC_AT_Y_MINUS_INFTY | this->SRC_AT_Y_PLUS_INFTY) == 0)
           {
-            Intersection_point_2  ip (p_left, 0);
+            Intersection_point  ip (p_left, 0);
 
-            *oi = make_object (ip);
-            ++oi;
+            *oi++ = make_object (ip);
           }
 
-          return (oi);
+          return oi;
         }
       }
 
       // Create the overlapping portion of the rational arc by properly setting
       // the source (left) and target (right) endpoints and their information
       // bits.
-      Self      overlap_arc (*this);
+      Self overlap_arc(*this);
 
       overlap_arc._ps = p_left;
       overlap_arc._pt = p_right;
@@ -2128,10 +2107,9 @@ public:
                            this->IS_DIRECTED_RIGHT | this->IS_CONTINUOUS |
                            this->IS_VALID);
 
-      *oi = make_object (overlap_arc);
-      ++oi;
+      *oi++ = make_object(overlap_arc);
 
-      return (oi);
+      return oi;
     }
 
     // We wish to find the intersection points between:
@@ -2140,39 +2118,34 @@ public:
     //
     // It is clear that the x-coordinates of the intersection points are
     // the roots of the polynomial: ip(x) = p1(x)*q2(x) - p2(x)*q1(x).
-    Nt_traits            nt_traits;
-    Polynomial           ipoly = this->_numer * arc._denom -
-      arc._numer * this->_denom;
-    std::list<Algebraic>                           xs;
+    Nt_traits nt_traits;
+    Polynomial ipoly = this->_numer * arc._denom - arc._numer* this->_denom;
+    std::list<Algebraic> xs;
     typename std::list<Algebraic>::const_iterator  x_iter;
 
-    nt_traits.compute_polynomial_roots (ipoly,
-                                        std::back_inserter(xs));
+    nt_traits.compute_polynomial_roots(ipoly, std::back_inserter(xs));
 
     // Go over the x-values we obtained. For each value produce an
     // intersection point if it is contained in the x-range of both curves.
-    unsigned int                     mult;
+    unsigned int mult;
 
-    for (x_iter = xs.begin(); x_iter != xs.end(); ++x_iter)
-    {
+    for (x_iter = xs.begin(); x_iter != xs.end(); ++x_iter)  {
       if (this->_is_in_true_x_range (*x_iter) &&
           arc._is_in_true_x_range (*x_iter))
       {
         // Compute the intersection point and obtain its multiplicity.
-        Point_2    p (*x_iter, nt_traits.evaluate_at (this->_numer, *x_iter) /
-                               nt_traits.evaluate_at (this->_denom, *x_iter));
+        Point_2 p(*x_iter, nt_traits.evaluate_at (this->_numer, *x_iter) /
+                  nt_traits.evaluate_at (this->_denom, *x_iter));
 
-        this->compare_slopes (arc, p, mult);
+        this->compare_slopes(arc, p, mult);
 
         // Output the intersection point:
-        Intersection_point_2  ip (p, mult);
-
-        *oi = make_object (ip);
-        ++oi;
+        Intersection_point ip(p, mult);
+        *oi++ = make_object(ip);
       }
     }
 
-    return (oi);
+    return oi;
   }
 
   /*!
diff -pruN 5.0.3-1/include/CGAL/Arr_geometry_traits/Segment_assertions.h 5.1-1/include/CGAL/Arr_geometry_traits/Segment_assertions.h
--- 5.0.3-1/include/CGAL/Arr_geometry_traits/Segment_assertions.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_geometry_traits/Segment_assertions.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/Segment_assertions.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_geometry_traits/Segment_assertions.h $
 // $Id: Segment_assertions.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_landmarks_point_location.h 5.1-1/include/CGAL/Arr_landmarks_point_location.h
--- 5.0.3-1/include/CGAL/Arr_landmarks_point_location.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_landmarks_point_location.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_landmarks_point_location.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_landmarks_point_location.h $
 // $Id: Arr_landmarks_point_location.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_line_arc_traits_2.h 5.1-1/include/CGAL/Arr_line_arc_traits_2.h
--- 5.0.3-1/include/CGAL/Arr_line_arc_traits_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_line_arc_traits_2.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_line_arc_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_line_arc_traits_2.h $
 // $Id: Arr_line_arc_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_linear_traits_2.h 5.1-1/include/CGAL/Arr_linear_traits_2.h
--- 5.0.3-1/include/CGAL/Arr_linear_traits_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_linear_traits_2.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,13 +3,14 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_linear_traits_2.h $
-// $Id: Arr_linear_traits_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_linear_traits_2.h $
+// $Id: Arr_linear_traits_2.h 5d37a1b 2020-04-22T00:31:33+03:00 Efi Fogel
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
-// Author(s)     : Ron Wein          <wein@post.tau.ac.il>
-//               : Waqar Khan        <wkhan@mpi-inf.mpg.de>
+// Author(s): Ron Wein          <wein@post.tau.ac.il>
+//            Waqar Khan        <wkhan@mpi-inf.mpg.de>
+//            Efi fogel         <efifogel@gmail.com>
 
 #ifndef CGAL_ARR_LINEAR_TRAITS_2_H
 #define CGAL_ARR_LINEAR_TRAITS_2_H
@@ -23,27 +24,29 @@
  * in the arrangement package.
  */
 
+#include <fstream>
+
+#include <boost/variant.hpp>
+
 #include <CGAL/tags.h>
 #include <CGAL/intersections.h>
 #include <CGAL/Arr_tags.h>
 #include <CGAL/Arr_enums.h>
 #include <CGAL/Arr_geometry_traits/Segment_assertions.h>
-#include <fstream>
 
 namespace CGAL {
 
-template <class Kernel_> class Arr_linear_object_2;
+template <typename Kernel_> class Arr_linear_object_2;
 
 /*! \class
  * A traits class for maintaining an arrangement of linear objects (lines,
  * rays and segments), aoviding cascading of computations as much as possible.
  */
-template <class Kernel_>
+template <typename Kernel_>
 class Arr_linear_traits_2 : public Kernel_ {
   friend class Arr_linear_object_2<Kernel_>;
 
 public:
-
   typedef Kernel_                         Kernel;
   typedef typename Kernel::FT             FT;
 
@@ -70,17 +73,14 @@ public:
   /*!
    * \class Representation of a linear with cached data.
    */
-  class _Linear_object_cached_2
-  {
+  class _Linear_object_cached_2 {
   public:
-
     typedef typename Kernel::Line_2                Line_2;
     typedef typename Kernel::Ray_2                 Ray_2;
     typedef typename Kernel::Segment_2             Segment_2;
     typedef typename Kernel::Point_2               Point_2;
 
   protected:
-
     Line_2    l;                // The supporting line.
     Point_2   ps;               // The source point (if exists).
     Point_2   pt;               // The target point (if exists).
@@ -99,39 +99,36 @@ public:
     bool      is_degen;         // Is the object degenerate (a single point).
 
   public:
-
-    /*!
-     * Default constructor.
+    /*! Default constructor.
      */
-    _Linear_object_cached_2 () :
-      has_source (true),
-      has_target (true),
-      is_vert (false),
-      is_horiz (false),
-      has_pos_slope (false),
-      is_degen (true)
+    _Linear_object_cached_2() :
+      has_source(true),
+      has_target(true),
+      is_vert(false),
+      is_horiz(false),
+      has_pos_slope(false),
+      is_degen(true)
     {}
 
-    /*!
-     * Constructor for segment from two points.
+    /*! Constructor for segment from two points.
      * \param p1 source point.
      * \param p2 target point.
      * \pre The two points must not be equal.
      */
     _Linear_object_cached_2(const Point_2& source, const Point_2& target) :
-      ps (source),
-      pt (target),
-      has_source (true),
-      has_target (true)
+      ps(source),
+      pt(target),
+      has_source(true),
+      has_target(true)
     {
-      Kernel   kernel;
+      Kernel kernel;
 
-      Comparison_result  res = kernel.compare_xy_2_object()(source, target);
+      Comparison_result res = kernel.compare_xy_2_object()(source, target);
       is_degen = (res == EQUAL);
       is_right = (res == SMALLER);
 
-      CGAL_precondition_msg (! is_degen,
-                             "Cannot construct a degenerate segment.");
+      CGAL_precondition_msg(! is_degen,
+                            "Cannot construct a degenerate segment.");
 
       l = kernel.construct_line_2_object()(source, target);
       is_vert = kernel.is_vertical_2_object()(l);
@@ -139,29 +136,25 @@ public:
       has_pos_slope = _has_positive_slope();
     }
 
-    /*!
-     * Constructor from a segment.
+    /*! Constructor from a segment.
      * \param seg The segment.
      * \pre The segment is not degenerate.
      */
-    _Linear_object_cached_2 (const Segment_2& seg)
+    _Linear_object_cached_2(const Segment_2& seg) :
+      has_source(true),
+      has_target(true)
     {
-      Kernel   kernel;
-
-      CGAL_assertion_msg (! kernel.is_degenerate_2_object() (seg),
-                          "Cannot construct a degenerate segment.");
+      Kernel kernel;
 
-      typename Kernel_::Construct_vertex_2
-        construct_vertex = kernel.construct_vertex_2_object();
+      CGAL_assertion_msg(! kernel.is_degenerate_2_object()(seg),
+                         "Cannot construct a degenerate segment.");
 
+      auto construct_vertex = kernel.construct_vertex_2_object();
       ps = construct_vertex(seg, 0);
-      has_source = true;
       pt = construct_vertex(seg, 1);
-      has_target = true;
 
-      Comparison_result  res = kernel.compare_xy_2_object()(ps, pt);
-
-      CGAL_assertion (res != EQUAL);
+      Comparison_result res = kernel.compare_xy_2_object()(ps, pt);
+      CGAL_assertion(res != EQUAL);
       is_degen = false;
       is_right = (res == SMALLER);
 
@@ -171,28 +164,25 @@ public:
       has_pos_slope = _has_positive_slope();
     }
 
-    /*!
-     * Constructor from a ray.
+    /*! Constructor from a ray.
      * \param ray The ray.
      * \pre The ray is not degenerate.
      */
-    _Linear_object_cached_2 (const Ray_2& ray)
+    _Linear_object_cached_2(const Ray_2& ray) :
+      has_source(true),
+      has_target(false)
     {
-      Kernel   kernel;
-
-      CGAL_assertion_msg (! kernel.is_degenerate_2_object() (ray),
-                          "Cannot construct a degenerate ray.");
+      Kernel kernel;
 
-      typename Kernel_::Construct_point_on_2
-        construct_vertex = kernel.construct_point_on_2_object();
+      CGAL_assertion_msg(! kernel.is_degenerate_2_object()(ray),
+                         "Cannot construct a degenerate ray.");
 
+      auto construct_vertex = kernel.construct_point_on_2_object();
       ps = construct_vertex(ray, 0);         // The source point.
-      has_source = true;
       pt = construct_vertex(ray, 1);         // Some point on the ray.
-      has_target = false;
 
       Comparison_result  res = kernel.compare_xy_2_object()(ps, pt);
-      CGAL_assertion (res != EQUAL);
+      CGAL_assertion(res != EQUAL);
       is_degen = false;
       is_right = (res == SMALLER);
 
@@ -202,31 +192,26 @@ public:
       has_pos_slope = _has_positive_slope();
     }
 
-    /*!
-     * Constructor from a line.
+    /*! Constructor from a line.
      * \param ln The line.
      * \pre The line is not degenerate.
      */
-    _Linear_object_cached_2 (const Line_2& ln) :
-      l (ln),
-      has_source (false),
-      has_target (false)
+    _Linear_object_cached_2(const Line_2& ln) :
+      l(ln),
+      has_source(false),
+      has_target(false)
     {
-      Kernel   kernel;
+      Kernel kernel;
 
-      CGAL_assertion_msg (! kernel.is_degenerate_2_object() (ln),
-                          "Cannot construct a degenerate line.");
-
-      typename Kernel_::Construct_point_on_2
-        construct_vertex = kernel.construct_point_on_2_object();
+      CGAL_assertion_msg(! kernel.is_degenerate_2_object()(ln),
+                         "Cannot construct a degenerate line.");
 
+      auto construct_vertex = kernel.construct_point_on_2_object();
       ps = construct_vertex(ln, 0);         // Some point on the line.
-      has_source = false;
       pt = construct_vertex(ln, 1);         // Some point further on the line.
-      has_target = false;
 
-      Comparison_result  res = kernel.compare_xy_2_object()(ps, pt);
-      CGAL_assertion (res != EQUAL);
+      Comparison_result res = kernel.compare_xy_2_object()(ps, pt);
+      CGAL_assertion(res != EQUAL);
       is_degen = false;
       is_right = (res == SMALLER);
 
@@ -235,31 +220,27 @@ public:
       has_pos_slope = _has_positive_slope();
     }
 
-    /*!
-     * Check whether the x-coordinate of the left point is infinite.
+    /*! Check whether the x-coordinate of the left point is infinite.
      * \return ARR_LEFT_BOUNDARY if the left point is near the boundary;
      *         ARR_INTERIOR if the x-coordinate is finite.
      */
-    Arr_parameter_space left_infinite_in_x () const
+    Arr_parameter_space left_infinite_in_x() const
     {
-      if (is_vert || is_degen)
-        return (ARR_INTERIOR);
+      if (is_vert || is_degen) return (ARR_INTERIOR);
 
       return (is_right) ?
         (has_source ? ARR_INTERIOR : ARR_LEFT_BOUNDARY) :
         (has_target ? ARR_INTERIOR : ARR_LEFT_BOUNDARY);
     }
 
-    /*!
-     * Check whether the y-coordinate of the left point is infinite.
+    /*! Check whether the y-coordinate of the left point is infinite.
      * \return ARR_BOTTOM_BOUNDARY if the left point is at y = -oo;
      *         ARR_INTERIOR if the y-coordinate is finite.
      *         ARR_TOP_BOUNDARY if the left point is at y = +oo;
      */
-    Arr_parameter_space left_infinite_in_y () const
+    Arr_parameter_space left_infinite_in_y() const
     {
-      if (is_horiz || is_degen)
-        return ARR_INTERIOR;
+      if (is_horiz || is_degen) return ARR_INTERIOR;
 
       if (is_vert) {
         return (is_right) ?
@@ -268,100 +249,80 @@ public:
       }
 
       if ((is_right && has_source) || (! is_right && has_target))
-          return ARR_INTERIOR;
+        return ARR_INTERIOR;
 
       return (has_pos_slope ? ARR_BOTTOM_BOUNDARY : ARR_TOP_BOUNDARY);
     }
 
-    /*!
-     * Check whether the left point is finite.
+    /*! Check whether the left point is finite.
      */
-    bool has_left () const
-    {
-      if (is_right)
-        return (has_source);
-      else
-        return (has_target);
-    }
+    bool has_left() const { return (is_right ? has_source : has_target); }
 
-    /*!
-     * Obtain the (lexicographically) left endpoint.
+    /*! Obtain the (lexicographically) left endpoint.
      * \pre The left point is finite.
      */
-    const Point_2& left () const
+    const Point_2& left() const
     {
-      CGAL_precondition (has_left());
+      CGAL_precondition(has_left());
       return (is_right ? ps : pt);
     }
 
-    /*!
-     * Set the (lexicographically) left endpoint.
+    /*! Set the (lexicographically) left endpoint.
      * \param p The point to set.
      * \pre p lies on the supporting line to the left of the right endpoint.
      */
-    void set_left (const Point_2& p, bool CGAL_assertion_code(check_validity) = true)
+    void set_left(const Point_2& p,
+                  bool CGAL_assertion_code(check_validity) = true)
     {
-      CGAL_precondition (! is_degen);
+      CGAL_precondition(! is_degen);
 
-      CGAL_precondition_code (
-        Kernel    kernel;
-      );
+      CGAL_precondition_code(Kernel kernel);
       CGAL_precondition
-        (Segment_assertions::_assert_is_point_on (p, l,
-                                                  Has_exact_division()) &&
+        (Segment_assertions::_assert_is_point_on(p, l, Has_exact_division()) &&
          (! check_validity || ! has_right() ||
-          kernel.compare_xy_2_object() (p, right()) == SMALLER));
+          kernel.compare_xy_2_object()(p, right()) == SMALLER));
 
-      if (is_right)
-      {
+      if (is_right) {
         ps = p;
         has_source = true;
       }
-      else
-      {
+      else {
         pt = p;
         has_target = true;
       }
     }
 
-    /*!
-     * Set the (lexicographically) left endpoint as infinite.
+    /*! Set the (lexicographically) left endpoint as infinite.
      */
-    void set_left ()
+    void set_left()
     {
-      CGAL_precondition (! is_degen);
+      CGAL_precondition(! is_degen);
 
-      if (is_right)
-        has_source = false;
-      else
-        has_target = false;
+      if (is_right) has_source = false;
+      else has_target = false;
     }
 
-    /*!
-     * Check whether the x-coordinate of the right point is infinite.
+    /*! Check whether the x-coordinate of the right point is infinite.
      * \return ARR_RIGHT_BOUNDARY if the right point is near the boundary;
      *         ARR_INTERIOR if the x-coordinate is finite.
      */
-    Arr_parameter_space right_infinite_in_x () const
+    Arr_parameter_space right_infinite_in_x() const
     {
-      if (is_vert || is_degen)
-        return ARR_INTERIOR;
+      if (is_vert || is_degen) return ARR_INTERIOR;
 
       return (is_right) ?
         (has_target ? ARR_INTERIOR : ARR_RIGHT_BOUNDARY) :
         (has_source ? ARR_INTERIOR : ARR_RIGHT_BOUNDARY);
     }
 
-    /*!
-     * Check whether the y-coordinate of the right point is infinite.
+    /*! Check whether the y-coordinate of the right point is infinite.
      * \return ARR_BOTTOM_BOUNDARY if the right point is at y = -oo;
      *         ARR_INTERIOR if the y-coordinate is finite.
      *         ARR_TOP_BOUNDARY if the right point is at y = +oo;
      */
-    Arr_parameter_space right_infinite_in_y () const
+    Arr_parameter_space right_infinite_in_y() const
     {
-      if (is_horiz || is_degen)
-        return ARR_INTERIOR;
+      if (is_horiz || is_degen) return ARR_INTERIOR;
 
       if (is_vert) {
         return (is_right) ?
@@ -375,145 +336,108 @@ public:
       return (has_pos_slope ? ARR_TOP_BOUNDARY : ARR_BOTTOM_BOUNDARY);
     }
 
-    /*!
-     * Check whether the right point is finite.
+    /*! Check whether the right point is finite.
      */
-    bool has_right () const
-    {
-      if (is_right)
-        return (has_target);
-      else
-        return (has_source);
-    }
+    bool has_right() const { return (is_right ? has_target : has_source); }
 
-    /*!
-     * Obtain the (lexicographically) right endpoint.
+    /*! Obtain the (lexicographically) right endpoint.
      * \pre The right endpoint is finite.
      */
-    const Point_2& right () const
+    const Point_2& right() const
     {
-      CGAL_precondition (has_right());
+      CGAL_precondition(has_right());
       return (is_right ? pt : ps);
     }
 
-    /*!
-     * Set the (lexicographically) right endpoint.
+    /*! Set the (lexicographically) right endpoint.
      * \param p The point to set.
      * \pre p lies on the supporting line to the right of the left endpoint.
      */
-    void set_right (const Point_2& p, bool CGAL_assertion_code(check_validity) = true)
+    void set_right(const Point_2& p,
+                   bool CGAL_assertion_code(check_validity) = true)
     {
-      CGAL_precondition (! is_degen);
-      CGAL_precondition_code (
-        Kernel    kernel;
-      );
+      CGAL_precondition(! is_degen);
+      CGAL_precondition_code(Kernel kernel);
       CGAL_precondition
-        (Segment_assertions::_assert_is_point_on (p, l,
-                                                  Has_exact_division()) &&
+        (Segment_assertions::_assert_is_point_on(p, l, Has_exact_division()) &&
          (! check_validity || ! has_left() ||
-          kernel.compare_xy_2_object() (p, left()) == LARGER));
+          kernel.compare_xy_2_object()(p, left()) == LARGER));
 
-      if (is_right)
-      {
+      if (is_right) {
         pt = p;
         has_target = true;
       }
-      else
-      {
+      else {
         ps = p;
         has_source = true;
       }
     }
 
-    /*!
-     * Set the (lexicographically) right endpoint as infinite.
+    /*! Set the (lexicographically) right endpoint as infinite.
      */
-    void set_right ()
+    void set_right()
     {
       CGAL_precondition (! is_degen);
 
-      if (is_right)
-        has_target = false;
-      else
-        has_source = false;
+      if (is_right) has_target = false;
+      else has_source = false;
     }
 
-    /*!
-     * Obtain the supporting line.
+    /*! Obtain the supporting line.
      */
-    const Line_2& supp_line () const
+    const Line_2& supp_line() const
     {
-      CGAL_precondition (! is_degen);
+      CGAL_precondition(! is_degen);
       return (l);
     }
 
-    /*!
-     * Check whether the curve is vertical.
+    /*! Check whether the curve is vertical.
      */
-    bool is_vertical () const
+    bool is_vertical() const
     {
-      CGAL_precondition (! is_degen);
+      CGAL_precondition(! is_degen);
       return (is_vert);
     }
 
-    /*!
-     * Check whether the curve is degenerate.
+    /*! Check whether the curve is degenerate.
      */
-    bool is_degenerate () const
-    {
-      return (is_degen);
-    }
+    bool is_degenerate() const { return (is_degen); }
 
-    /*!
-     * Check whether the curve is directed lexicographic from left to right
+    /*! Check whether the curve is directed lexicographic from left to right
      */
-    bool is_directed_right () const
-    {
-      return (is_right);
-    }
+    bool is_directed_right() const { return (is_right); }
 
-    /*!
-     * Check whether the given point is in the x-range of the object.
+    /*! Check whether the given point is in the x-range of the object.
      * \param p The query point.
      * \return (true) is in the x-range of the segment; (false) if it is not.
      */
-    bool is_in_x_range (const Point_2& p) const
+    bool is_in_x_range(const Point_2& p) const
     {
-      Kernel                          kernel;
-      typename Kernel_::Compare_x_2   compare_x = kernel.compare_x_2_object();
-      Comparison_result               res1;
-
-      if (left_infinite_in_x() == ARR_INTERIOR)
-      {
-        if (left_infinite_in_y() != ARR_INTERIOR)
-          // Compare with some point on the curve.
-          res1 = compare_x (p, ps);
-        else
-          res1 = compare_x (p, left());
+      Kernel kernel;
+      typename Kernel_::Compare_x_2 compare_x = kernel.compare_x_2_object();
+      Comparison_result res1;
+
+      if (left_infinite_in_x() == ARR_INTERIOR) {
+        // Compare with some point on the curve.
+        if (left_infinite_in_y() != ARR_INTERIOR) res1 = compare_x(p, ps);
+        else res1 = compare_x(p, left());
       }
-      else
-      {
+      else {
         // p is obviously to the right.
         res1 = LARGER;
       }
 
-      if (res1 == SMALLER)
-        return (false);
-      else if (res1 == EQUAL)
-        return (true);
+      if (res1 == SMALLER) return false;
+      else if (res1 == EQUAL) return true;
 
-      Comparison_result               res2;
+      Comparison_result res2;
 
-      if (right_infinite_in_x() == ARR_INTERIOR)
-      {
-        if (right_infinite_in_y() != ARR_INTERIOR)
-          // Compare with some point on the curve.
-          res2 = compare_x (p, ps);
-        else
-          res2 = compare_x (p, right());
+      if (right_infinite_in_x() == ARR_INTERIOR) {
+        // Compare with some point on the curve.
+        if (right_infinite_in_y() != ARR_INTERIOR) res2 = compare_x(p, ps);
+        else res2 = compare_x(p, right());
       }
-      else
-      {
+      else {
         // p is obviously to the right.
         res2 = SMALLER;
       }
@@ -521,68 +445,54 @@ public:
       return (res2 != LARGER);
     }
 
-    /*!
-     * Check whether the given point is in the y-range of the object.
+    /*! Check whether the given point is in the y-range of the object.
      * \param p The query point.
      * \pre The object is vertical.
      * \return (true) is in the y-range of the segment; (false) if it is not.
      */
-    bool is_in_y_range (const Point_2& p) const
+    bool is_in_y_range(const Point_2& p) const
     {
-      CGAL_precondition (is_vertical());
+      CGAL_precondition(is_vertical());
 
-      Kernel                          kernel;
-      typename Kernel_::Compare_y_2   compare_y = kernel.compare_y_2_object();
-      Arr_parameter_space             inf = left_infinite_in_y();
-      Comparison_result               res1;
-
-      CGAL_assertion (inf != ARR_TOP_BOUNDARY);
-      if (inf == ARR_INTERIOR)
-        res1 = compare_y (p, left());
-      else
-        res1 = LARGER;           // p is obviously above.
-
-      if (res1 == SMALLER)
-        return (false);
-      else if (res1 == EQUAL)
-        return (true);
+      Kernel kernel;
+      typename Kernel_::Compare_y_2 compare_y = kernel.compare_y_2_object();
+      Arr_parameter_space inf = left_infinite_in_y();
+      Comparison_result res1;
 
-      Comparison_result               res2;
+      CGAL_assertion(inf != ARR_TOP_BOUNDARY);
+      if (inf == ARR_INTERIOR) res1 = compare_y (p, left());
+      else res1 = LARGER;           // p is obviously above.
+
+      if (res1 == SMALLER) return false;
+      else if (res1 == EQUAL) return true;
+
+      Comparison_result res2;
 
       inf = right_infinite_in_y();
-      CGAL_assertion (inf != ARR_BOTTOM_BOUNDARY);
-      if (inf == ARR_INTERIOR)
-        res2 = compare_y (p, right());
-      else
-        res2 = SMALLER;          // p is obviously below.
+      CGAL_assertion(inf != ARR_BOTTOM_BOUNDARY);
+      if (inf == ARR_INTERIOR) res2 = compare_y(p, right());
+      else res2 = SMALLER;          // p is obviously below.
 
       return (res2 != LARGER);
     }
 
   private:
-
-    /*!
-     * Determine if the supporting line has a positive slope.
+    /*! Determine if the supporting line has a positive slope.
      */
-    bool _has_positive_slope () const
+    bool _has_positive_slope() const
     {
-      if (is_vert)
-        return (true);
-
-      if (is_horiz)
-        return (false);
+      if (is_vert) return true;
+      if (is_horiz) return false;
 
       // Construct a horizontal line and compare its slope the that of l.
-      Kernel     kernel;
-      Line_2     l_horiz = kernel.construct_line_2_object() (Point_2 (0, 0),
-                                                             Point_2 (1, 0));
-
-      return (kernel.compare_slope_2_object() (l, l_horiz) == LARGER);
+      Kernel kernel;
+      Line_2 l_horiz =
+        kernel.construct_line_2_object()(Point_2(0, 0), Point_2(1, 0));
+      return (kernel.compare_slope_2_object()(l, l_horiz) == LARGER);
     }
   };
 
 public:
-
   // Traits objects
   typedef typename Kernel::Point_2              Point_2;
   typedef Arr_linear_object_2<Kernel>           X_monotone_curve_2;
@@ -590,12 +500,9 @@ public:
   typedef unsigned int                          Multiplicity;
 
 public:
-
-  /*!
-   * Default constructor.
+  /*! Default constructor.
    */
-  Arr_linear_traits_2 ()
-  {}
+  Arr_linear_traits_2() {}
 
   /// \name Basic functor definitions.
   //@{
@@ -606,7 +513,7 @@ public:
     typedef Arr_linear_traits_2<Kernel> Traits;
 
     /*! The traits (in case it has state) */
-    const Traits * m_traits;
+    const Traits& m_traits;
 
     /*! Constructor
      * \param traits the traits (in case it has state)
@@ -614,32 +521,28 @@ public:
      * obtaining function, which is a member of the nesting class,
      * constructing it.
      */
-    Compare_x_2(const Traits * traits) : m_traits(traits) {}
+    Compare_x_2(const Traits& traits) : m_traits(traits) {}
 
     //! Allow its functor obtaining function calling the private constructor.
     friend class Arr_linear_traits_2<Kernel>;
 
   public:
-    /*!
-     * Compare the x-coordinates of two points.
+    /*! Compare the x-coordinates of two points.
      * \param p1 The first point.
      * \param p2 The second point.
      * \return LARGER if x(p1) > x(p2);
      *         SMALLER if x(p1) < x(p2);
      *         EQUAL if x(p1) = x(p2).
      */
-    Comparison_result operator() (const Point_2& p1, const Point_2& p2) const
+    Comparison_result operator()(const Point_2& p1, const Point_2& p2) const
     {
-      const Kernel * kernel = m_traits;
-      return (kernel->compare_x_2_object()(p1, p2));
+      const Kernel& kernel = m_traits;
+      return (kernel.compare_x_2_object()(p1, p2));
     }
   };
 
   /*! Obtain a Compare_x_2 functor. */
-  Compare_x_2 compare_x_2_object () const
-  {
-    return Compare_x_2(this);
-  }
+  Compare_x_2 compare_x_2_object() const { return Compare_x_2(*this); }
 
   /*! A functor that compares the he endpoints of an $x$-monotone curve. */
   class Compare_endpoints_xy_2{
@@ -650,21 +553,19 @@ public:
      * \return SMALLER if the curve is directed right;
      *         LARGER if the curve is directed left.
      */
-     Comparison_result operator() (const X_monotone_curve_2& xcv) const
+     Comparison_result operator()(const X_monotone_curve_2& xcv) const
     { return (xcv.is_directed_right()) ? (SMALLER) : (LARGER); }
   };
 
   Compare_endpoints_xy_2 compare_endpoints_xy_2_object() const
-  {
-    return Compare_endpoints_xy_2();
-  }
+  { return Compare_endpoints_xy_2(); }
 
-  class Trim_2{
+  class Trim_2 {
   protected:
     typedef Arr_linear_traits_2<Kernel> Traits;
 
     /*! The traits (in case it has state) */
-    const Traits* m_traits;
+    const Traits& m_traits;
 
     /*! Constructor
      * \param traits the traits (in case it has state)
@@ -672,25 +573,25 @@ public:
      * obtaining function, which is a member of the nesting class,
      * constructing it.
      */
-    Trim_2(const Traits * traits) : m_traits(traits) {}
+    Trim_2(const Traits& traits) : m_traits(traits) {}
 
     //! Allow its functor obtaining function calling the private constructor.
     friend class Arr_linear_traits_2<Kernel>;
 
   public:
-    X_monotone_curve_2 operator()( const X_monotone_curve_2 xcv,
-                                   const Point_2 src,
-                                   const Point_2 tgt )
+    X_monotone_curve_2 operator()(const X_monotone_curve_2 xcv,
+                                  const Point_2 src,
+                                  const Point_2 tgt)
     {
       /*
        * "Line_segment, line, and ray" will become line segments
        * when trimmed.
-      */
+       */
       Equal_2 equal = Equal_2();
-      Compare_y_at_x_2 compare_y_at_x = m_traits->compare_y_at_x_2_object();
+      Compare_y_at_x_2 compare_y_at_x = m_traits.compare_y_at_x_2_object();
 
       //preconditions
-      //check if source and taget are two distinct points and they lie on the line.
+      //check if source and taget are distinct points and they lie on the line.
       CGAL_precondition(!equal(src, tgt));
       CGAL_precondition(compare_y_at_x(src, xcv) == EQUAL);
       CGAL_precondition(compare_y_at_x(tgt, xcv) == EQUAL);
@@ -698,33 +599,24 @@ public:
       //create trimmed line_segment
       X_monotone_curve_2 trimmed_segment;
 
-      if( xcv.is_directed_right() && tgt.x() < src.x() )
+      if (xcv.is_directed_right() && (tgt.x() < src.x()))
         trimmed_segment = Segment_2(tgt, src);
-
-
-      else if( !xcv.is_directed_right() && tgt.x() > src.x())
+      else if (! xcv.is_directed_right() && (tgt.x() > src.x()))
         trimmed_segment = Segment_2(tgt, src);
-
-      else
-        trimmed_segment = Segment_2(src, tgt);
+      else trimmed_segment = Segment_2(src, tgt);
 
       return trimmed_segment;
     }
-
   };
 
-  Trim_2 trim_2_object() const
-  {
-    return Trim_2(this);
-  }
-
+  Trim_2 trim_2_object() const { return Trim_2(*this); }
 
   class Construct_opposite_2{
   protected:
     typedef Arr_linear_traits_2<Kernel> Traits;
 
     /*! The traits (in case it has state) */
-    const Traits* m_traits;
+    const Traits& m_traits;
 
     /*! Constructor
      * \param traits the traits (in case it has state)
@@ -732,142 +624,111 @@ public:
      * obtaining function, which is a member of the nesting class,
      * constructing it.
      */
-    Construct_opposite_2(const Traits * traits) : m_traits(traits) {}
+    Construct_opposite_2(const Traits& traits) : m_traits(traits) {}
 
     //! Allow its functor obtaining function calling the private constructor.
     friend class Arr_linear_traits_2<Kernel>;
 
   public:
-
-    X_monotone_curve_2 operator()(const X_monotone_curve_2& xcv)const
+    X_monotone_curve_2 operator()(const X_monotone_curve_2& xcv) const
     {
-      CGAL_precondition (! xcv.is_degenerate());
+      CGAL_precondition(! xcv.is_degenerate());
 
       X_monotone_curve_2 opp_xcv;
 
-      if( xcv.is_segment() )
-      {
-        opp_xcv = Segment_2(xcv.target(), xcv.source());
-      }
-
-      if( xcv.is_line() )
-      {
-        opp_xcv = Line_2(xcv.get_pt(), xcv.get_ps());
-      }
-
-      if( xcv.is_ray() )
-      {
+      if (xcv.is_segment()) opp_xcv = Segment_2(xcv.target(), xcv.source());
+      if (xcv.is_line()) opp_xcv = Line_2(xcv.get_pt(), xcv.get_ps());
+      if (xcv.is_ray()) {
         Point_2 opp_tgt = Point_2( -(xcv.get_pt().x()), -(xcv.get_pt().y()));
         opp_xcv = Ray_2( xcv.source(),  opp_tgt);
       }
 
       return opp_xcv;
-
     }
-
   };
 
   /*! Get a Construct_opposite_2 functor object. */
   Construct_opposite_2 construct_opposite_2_object() const
-  {
-    return Construct_opposite_2(this);
-  }
+  { return Construct_opposite_2(*this); }
 
   /*! A functor that compares the x-coordinates of two points */
   class Compare_xy_2 {
   public:
-    /*!
-     * Compare two points lexigoraphically: by x, then by y.
+    /*! Compare two points lexigoraphically: by x, then by y.
      * \param p1 The first point.
      * \param p2 The second point.
      * \return LARGER if x(p1) > x(p2), or if x(p1) = x(p2) and y(p1) > y(p2);
      *         SMALLER if x(p1) < x(p2), or if x(p1) = x(p2) and y(p1) < y(p2);
      *         EQUAL if the two points are equal.
      */
-    Comparison_result operator() (const Point_2& p1, const Point_2& p2) const
+    Comparison_result operator()(const Point_2& p1, const Point_2& p2) const
     {
-      Kernel    kernel;
+      Kernel kernel;
       return (kernel.compare_xy_2_object()(p1, p2));
     }
   };
 
   /*! Obtain a Compare_xy_2 functor object. */
-  Compare_xy_2 compare_xy_2_object () const
-  {
-    return Compare_xy_2();
-  }
+  Compare_xy_2 compare_xy_2_object() const { return Compare_xy_2(); }
 
   /*! A functor that obtains the left endpoint of a segment or a ray. */
-  class Construct_min_vertex_2
-  {
+  class Construct_min_vertex_2 {
   public:
-    /*!
-     * Get the left endpoint of the x-monotone curve (segment).
+    /*! Obtain the left endpoint of the x-monotone curve (segment).
      * \param cv The curve.
      * \pre The left end of cv is a valid (bounded) point.
      * \return The left endpoint.
      */
-    const Point_2& operator() (const X_monotone_curve_2& cv) const
+    const Point_2& operator()(const X_monotone_curve_2& cv) const
     {
-      CGAL_precondition (! cv.is_degenerate());
-      CGAL_precondition (cv.has_left());
+      CGAL_precondition(! cv.is_degenerate());
+      CGAL_precondition(cv.has_left());
 
       return (cv.left());
     }
   };
 
   /*! Obtain a Construct_min_vertex_2 functor object. */
-  Construct_min_vertex_2 construct_min_vertex_2_object () const
-  {
-    return Construct_min_vertex_2();
-  }
+  Construct_min_vertex_2 construct_min_vertex_2_object() const
+  { return Construct_min_vertex_2(); }
 
   /*! A functor that obtains the right endpoint of a segment or a ray. */
-  class Construct_max_vertex_2
-  {
+  class Construct_max_vertex_2 {
   public:
-    /*!
-     * Get the right endpoint of the x-monotone curve (segment).
+    /*! Obtain the right endpoint of the x-monotone curve (segment).
      * \param cv The curve.
      * \pre The right end of cv is a valid (bounded) point.
      * \return The right endpoint.
      */
-    const Point_2& operator() (const X_monotone_curve_2& cv) const
+    const Point_2& operator()(const X_monotone_curve_2& cv) const
     {
-      CGAL_precondition (! cv.is_degenerate());
-      CGAL_precondition (cv.has_right());
+      CGAL_precondition(! cv.is_degenerate());
+      CGAL_precondition(cv.has_right());
 
       return (cv.right());
     }
   };
 
   /*! Obtain a Construct_max_vertex_2 functor object. */
-  Construct_max_vertex_2 construct_max_vertex_2_object () const
-  {
-    return Construct_max_vertex_2();
-  }
+  Construct_max_vertex_2 construct_max_vertex_2_object() const
+  { return Construct_max_vertex_2(); }
 
   /*! A functor that checks whether a given linear curve is vertical. */
-  class Is_vertical_2
-  {
+  class Is_vertical_2 {
   public:
-    /*!
-     * Check whether the given x-monotone curve is a vertical segment.
+    /*! Check whether the given x-monotone curve is a vertical segment.
      * \param cv The curve.
      * \return (true) if the curve is a vertical segment; (false) otherwise.
      */
-    bool operator() (const X_monotone_curve_2& cv) const
+    bool operator()(const X_monotone_curve_2& cv) const
     {
-      CGAL_precondition (! cv.is_degenerate());
+      CGAL_precondition(! cv.is_degenerate());
       return (cv.is_vertical());
     }
   };
 
   /*! Obtain an Is_vertical_2 functor object. */
-  Is_vertical_2 is_vertical_2_object () const
-  {
-    return Is_vertical_2();
-  }
+  Is_vertical_2 is_vertical_2_object() const { return Is_vertical_2(); }
 
   /*! A functor that compares the y-coordinates of a point and a line at
    * the point x-coordinate
@@ -877,7 +738,7 @@ public:
     typedef Arr_linear_traits_2<Kernel> Traits;
 
     /*! The traits (in case it has state) */
-    const Traits* m_traits;
+    const Traits& m_traits;
 
     /*! Constructor
      * \param traits the traits (in case it has state)
@@ -885,14 +746,13 @@ public:
      * obtaining function, which is a member of the nesting class,
      * constructing it.
      */
-    Compare_y_at_x_2(const Traits * traits) : m_traits(traits) {}
+    Compare_y_at_x_2(const Traits& traits) : m_traits(traits) {}
 
     //! Allow its functor obtaining function calling the private constructor.
     friend class Arr_linear_traits_2<Kernel>;
 
   public:
-    /*!
-     * Return the location of the given point with respect to the input curve.
+    /*! Obtain the location of the given point with respect to the input curve.
      * \param cv The curve.
      * \param p The point.
      * \pre p is in the x-range of cv.
@@ -900,42 +760,38 @@ public:
      *         LARGER if y(p) > cv(x(p)), i.e. the point is above the curve;
      *         EQUAL if p lies on the curve.
      */
-    Comparison_result operator() (const Point_2& p,
-                                  const X_monotone_curve_2& cv) const
+    Comparison_result operator()(const Point_2& p,
+                                 const X_monotone_curve_2& cv) const
     {
-      CGAL_precondition (! cv.is_degenerate());
-      CGAL_precondition (cv.is_in_x_range (p));
+      CGAL_precondition(! cv.is_degenerate());
+      CGAL_precondition(cv.is_in_x_range(p));
 
-      const Kernel * kernel = m_traits;
+      const Kernel& kernel = m_traits;
       if (! cv.is_vertical())
         // Compare p with the segment's supporting line.
-        return (kernel->compare_y_at_x_2_object()(p, cv.supp_line()));
+        return (kernel.compare_y_at_x_2_object()(p, cv.supp_line()));
 
       // Compare with the vertical segment's end-points.
-      typename Kernel::Compare_y_2  compare_y = kernel->compare_y_2_object();
+      typename Kernel::Compare_y_2 compare_y = kernel.compare_y_2_object();
       const Comparison_result res1 =
-        cv.has_left() ? compare_y (p, cv.left()) : LARGER;
+        cv.has_left() ? compare_y(p, cv.left()) : LARGER;
       const Comparison_result res2 =
-        cv.has_right() ? compare_y (p, cv.right()) : SMALLER;
+        cv.has_right() ? compare_y(p, cv.right()) : SMALLER;
 
       return (res1 == res2) ? res1 : EQUAL;
     }
   };
 
   /*! Obtain a Compare_y_at_x_2 functor object. */
-  Compare_y_at_x_2 compare_y_at_x_2_object () const
-  {
-    return Compare_y_at_x_2(this);
-  }
+  Compare_y_at_x_2 compare_y_at_x_2_object() const
+  { return Compare_y_at_x_2(*this); }
 
   /*! A functor that compares compares the y-coordinates of two linear
    * curves immediately to the left of their intersection point.
    */
-  class Compare_y_at_x_left_2
-  {
+  class Compare_y_at_x_left_2 {
   public:
-    /*!
-     * Compare the y value of two x-monotone curves immediately to the left
+    /*! Compare the y value of two x-monotone curves immediately to the left
      * of their intersection point.
      * \param cv1 The first curve.
      * \param cv2 The second curve.
@@ -945,31 +801,28 @@ public:
      * \return The relative position of cv1 with respect to cv2 immdiately to
      *         the left of p: SMALLER, LARGER or EQUAL.
      */
-    Comparison_result operator() (const X_monotone_curve_2& cv1,
-                                  const X_monotone_curve_2& cv2,
-                                  const Point_2& CGAL_precondition_code(p)) const
+    Comparison_result operator()(const X_monotone_curve_2& cv1,
+                                 const X_monotone_curve_2& cv2,
+                                 const Point_2& CGAL_precondition_code(p)) const
     {
-      CGAL_precondition (! cv1.is_degenerate());
-      CGAL_precondition (! cv2.is_degenerate());
+      CGAL_precondition(! cv1.is_degenerate());
+      CGAL_precondition(! cv2.is_degenerate());
 
       Kernel                        kernel;
 
       // Make sure that p lies on both curves, and that both are defined to its
       // left (so their left endpoint is lexicographically smaller than p).
-      CGAL_precondition_code (
-        typename Kernel::Compare_xy_2 compare_xy = kernel.compare_xy_2_object();
-      );
+      CGAL_precondition_code(auto compare_xy = kernel.compare_xy_2_object());
 
       CGAL_precondition
-        (Segment_assertions::_assert_is_point_on (p, cv1,
-                                                  Has_exact_division()) &&
-         Segment_assertions::_assert_is_point_on (p, cv2,
-                                                  Has_exact_division()));
-
-      CGAL_precondition ((! cv1.has_left() ||
-                          compare_xy(cv1.left(), p) == SMALLER) &&
-                         (! cv2.has_left() ||
-                          compare_xy(cv2.left(), p) == SMALLER));
+        (Segment_assertions::_assert_is_point_on(p, cv1,
+                                                 Has_exact_division()) &&
+         Segment_assertions::_assert_is_point_on(p, cv2, Has_exact_division()));
+
+      CGAL_precondition((! cv1.has_left() ||
+                         compare_xy(cv1.left(), p) == SMALLER) &&
+                        (! cv2.has_left() ||
+                         compare_xy(cv2.left(), p) == SMALLER));
 
       // Compare the slopes of the two segments to determine thir relative
       // position immediately to the left of q.
@@ -981,19 +834,15 @@ public:
   };
 
   /*! Obtain a Compare_y_at_x_left_2 functor object. */
-  Compare_y_at_x_left_2 compare_y_at_x_left_2_object () const
-  {
-    return Compare_y_at_x_left_2();
-  }
+  Compare_y_at_x_left_2 compare_y_at_x_left_2_object() const
+  { return Compare_y_at_x_left_2(); }
 
   /*! A functor that compares compares the y-coordinates of two linear
    * curves immediately to the right of their intersection point.
    */
-  class Compare_y_at_x_right_2
-  {
+  class Compare_y_at_x_right_2 {
   public:
-    /*!
-     * Compare the y value of two x-monotone curves immediately to the right
+    /*! Compare the y value of two x-monotone curves immediately to the right
      * of their intersection point.
      * \param cv1 The first curve.
      * \param cv2 The second curve.
@@ -1003,31 +852,28 @@ public:
      * \return The relative position of cv1 with respect to cv2 immdiately to
      *         the right of p: SMALLER, LARGER or EQUAL.
      */
-    Comparison_result operator() (const X_monotone_curve_2& cv1,
-                                  const X_monotone_curve_2& cv2,
-                                  const Point_2& CGAL_precondition_code(p)) const
+    Comparison_result operator()(const X_monotone_curve_2& cv1,
+                                 const X_monotone_curve_2& cv2,
+                                 const Point_2& CGAL_precondition_code(p)) const
     {
-      CGAL_precondition (! cv1.is_degenerate());
-      CGAL_precondition (! cv2.is_degenerate());
+      CGAL_precondition(! cv1.is_degenerate());
+      CGAL_precondition(! cv2.is_degenerate());
 
-      Kernel                        kernel;
+      Kernel kernel;
 
       // Make sure that p lies on both curves, and that both are defined to its
       // right (so their right endpoint is lexicographically larger than p).
-      CGAL_precondition_code (
-        typename Kernel::Compare_xy_2 compare_xy = kernel.compare_xy_2_object();
-      );
+      CGAL_precondition_code(auto compare_xy = kernel.compare_xy_2_object());
 
       CGAL_precondition
-        (Segment_assertions::_assert_is_point_on (p, cv1,
-                                                  Has_exact_division()) &&
-         Segment_assertions::_assert_is_point_on (p, cv2,
-                                                  Has_exact_division()));
-
-      CGAL_precondition ((! cv1.has_right() ||
-                          compare_xy(cv1.right(), p) == LARGER) &&
-                         (! cv2.has_right() ||
-                          compare_xy(cv2.right(), p) == LARGER));
+        (Segment_assertions::_assert_is_point_on(p, cv1,
+                                                 Has_exact_division()) &&
+         Segment_assertions::_assert_is_point_on(p, cv2, Has_exact_division()));
+
+      CGAL_precondition((! cv1.has_right() ||
+                         compare_xy(cv1.right(), p) == LARGER) &&
+                        (! cv2.has_right() ||
+                         compare_xy(cv2.right(), p) == LARGER));
 
       // Compare the slopes of the two segments to determine thir relative
       // position immediately to the left of q.
@@ -1038,47 +884,43 @@ public:
   };
 
   /*! Obtain a Compare_y_at_x_right_2 functor object. */
-  Compare_y_at_x_right_2 compare_y_at_x_right_2_object () const
-  {
-    return Compare_y_at_x_right_2();
-  }
+  Compare_y_at_x_right_2 compare_y_at_x_right_2_object() const
+  { return Compare_y_at_x_right_2(); }
 
   /*! A functor that checks whether two points and two linear curves are
    * identical.
    */
-  class Equal_2
-  {
+  class Equal_2 {
   public:
-    /*!
-     * Check whether the two x-monotone curves are the same (have the same
+    /*! Check whether the two x-monotone curves are the same (have the same
      * graph).
      * \param cv1 The first curve.
      * \param cv2 The second curve.
      * \return (true) if the two curves are the same; (false) otherwise.
      */
-    bool operator() (const X_monotone_curve_2& cv1,
-                     const X_monotone_curve_2& cv2) const
+    bool operator()(const X_monotone_curve_2& cv1,
+                    const X_monotone_curve_2& cv2) const
     {
-      CGAL_precondition (! cv1.is_degenerate());
-      CGAL_precondition (! cv2.is_degenerate());
+      CGAL_precondition(! cv1.is_degenerate());
+      CGAL_precondition(! cv2.is_degenerate());
 
-      Kernel                    kernel;
-      typename Kernel::Equal_2  equal = kernel.equal_2_object();
+      Kernel kernel;
+      typename Kernel::Equal_2 equal = kernel.equal_2_object();
 
       // Check that the two supporting lines are the same.
-      if (! equal (cv1.supp_line(), cv2.supp_line()) &&
-          ! equal (cv1.supp_line(),
-                   kernel.construct_opposite_line_2_object()(cv2.supp_line())))
+      if (! equal(cv1.supp_line(), cv2.supp_line()) &&
+          ! equal(cv1.supp_line(),
+                  kernel.construct_opposite_line_2_object()(cv2.supp_line())))
       {
-        return (false);
+        return false;
       }
 
       // Check that either the two left endpoints are at infinity, or they
       // are bounded and equal.
       if ((cv1.has_left() != cv2.has_left()) ||
-          (cv1.has_left() && ! equal (cv1.left(), cv2.left())))
+          (cv1.has_left() && ! equal(cv1.left(), cv2.left())))
       {
-        return (false);
+        return false;
       }
 
       // Check that either the two right endpoints are at infinity, or they
@@ -1087,24 +929,20 @@ public:
               (! cv1.has_right() || equal (cv1.right(), cv2.right())));
     }
 
-    /*!
-     * Check whether the two points are the same.
+    /*! Check whether the two points are the same.
      * \param p1 The first point.
      * \param p2 The second point.
      * \return (true) if the two point are the same; (false) otherwise.
      */
-    bool operator() (const Point_2& p1, const Point_2& p2) const
+    bool operator()(const Point_2& p1, const Point_2& p2) const
     {
-      Kernel    kernel;
+      Kernel kernel;
       return (kernel.equal_2_object()(p1, p2));
     }
   };
 
   /*! Obtain an Equal_2 functor object. */
-  Equal_2 equal_2_object () const
-  {
-    return Equal_2();
-  }
+  Equal_2 equal_2_object() const { return Equal_2(); }
   //@}
 
   /// \name Functor definitions to handle boundaries
@@ -1130,7 +968,7 @@ public:
     Arr_parameter_space operator()(const X_monotone_curve_2 & xcv,
                                    Arr_curve_end ce) const
     {
-      CGAL_precondition (! xcv.is_degenerate());
+      CGAL_precondition(! xcv.is_degenerate());
       return (ce == ARR_MIN_END) ?
         xcv.left_infinite_in_x() : xcv.right_infinite_in_x();
     }
@@ -1140,9 +978,7 @@ public:
      * \return the parameter space at p.
      */
     Arr_parameter_space operator()(const Point_2 ) const
-    {
-      return ARR_INTERIOR;
-    }
+    { return ARR_INTERIOR; }
   };
 
   /*! Obtain a Parameter_space_in_x_2 function object */
@@ -1185,9 +1021,7 @@ public:
      * \return the parameter space at p.
      */
     Arr_parameter_space operator()(const Point_2 ) const
-    {
-      return ARR_INTERIOR;
-    }
+    { return ARR_INTERIOR; }
   };
 
   /*! Obtain a Parameter_space_in_y_2 function object */
@@ -1202,7 +1036,7 @@ public:
     typedef Arr_linear_traits_2<Kernel> Traits;
 
     /*! The traits (in case it has state) */
-    const Traits* m_traits;
+    const Traits& m_traits;
 
     /*! Constructor
      * \param traits the traits (in case it has state)
@@ -1210,7 +1044,7 @@ public:
      * obtaining function, which is a member of the nesting class,
      * constructing it.
      */
-    Compare_x_at_limit_2(const Traits* traits) : m_traits(traits) {}
+    Compare_x_at_limit_2(const Traits& traits) : m_traits(traits) {}
 
     //! Allow its functor obtaining function calling the private constructor.
     friend class Arr_linear_traits_2<Kernel>;
@@ -1238,8 +1072,8 @@ public:
       CGAL_precondition(! xcv.is_degenerate());
       CGAL_precondition(xcv.is_vertical());
 
-      const Kernel* kernel = m_traits;
-      return (kernel->compare_x_at_y_2_object()(p, xcv.supp_line()));
+      const Kernel& kernel = m_traits;
+      return (kernel.compare_x_at_y_2_object()(p, xcv.supp_line()));
     }
 
     /*! Compare the x-limits of 2 arcs ends on the boundary of the
@@ -1271,23 +1105,22 @@ public:
       CGAL_precondition(xcv1.is_vertical());
       CGAL_precondition(xcv2.is_vertical());
 
-      const Kernel* kernel = m_traits;
-      const Point_2 p = kernel->construct_point_2_object()(ORIGIN);
-      return (kernel->compare_x_at_y_2_object()(p, xcv1.supp_line(),
-                                                xcv2.supp_line()));
+      const Kernel& kernel = m_traits;
+      const Point_2 p = kernel.construct_point_2_object()(ORIGIN);
+      return (kernel.compare_x_at_y_2_object()(p, xcv1.supp_line(),
+                                               xcv2.supp_line()));
     }
   };
 
   /*! Obtain a Compare_x_at_limit_2 function object */
   Compare_x_at_limit_2 compare_x_at_limit_2_object() const
-  { return Compare_x_at_limit_2(this); }
+  { return Compare_x_at_limit_2(*this); }
 
   /*! A function object that compares the x-coordinates of arc ends near the
    * boundary of the parameter space
    */
   class Compare_x_near_limit_2 {
   public:
-
     /*! Compare the x-coordinates of 2 arcs ends near the boundary of the
      * parameter space at y = +/- oo.
      * \param xcv1 the first arc.
@@ -1335,7 +1168,7 @@ public:
     typedef Arr_linear_traits_2<Kernel> Traits;
 
     /*! The traits (in case it has state) */
-    const Traits* m_traits;
+    const Traits& m_traits;
 
     /*! Constructor
      * \param traits the traits (in case it has state)
@@ -1343,7 +1176,7 @@ public:
      * obtaining function, which is a member of the nesting class,
      * constructing it.
      */
-    Compare_y_near_boundary_2(const Traits* traits) : m_traits(traits) {}
+    Compare_y_near_boundary_2(const Traits& traits) : m_traits(traits) {}
 
     //! Allow its functor obtaining function calling the private constructor.
     friend class Arr_linear_traits_2<Kernel>;
@@ -1373,17 +1206,17 @@ public:
                          xcv2.right_infinite_in_x() == ARR_RIGHT_BOUNDARY));
 
       // Compare the slopes of the two supporting lines.
-      const Kernel* kernel = m_traits;
+      const Kernel& kernel = m_traits;
       const Comparison_result res_slopes =
-        kernel->compare_slope_2_object()(xcv1.supp_line(), xcv2.supp_line());
+        kernel.compare_slope_2_object()(xcv1.supp_line(), xcv2.supp_line());
 
       if (res_slopes == EQUAL) {
         // In case the two supporting line are parallel, compare their
         // relative position at x = 0, which is the same as their position
         // at infinity.
-        const Point_2 p = kernel->construct_point_2_object()(ORIGIN);
-        return (kernel->compare_y_at_x_2_object()(p, xcv1.supp_line(),
-                                                  xcv2.supp_line()));
+        const Point_2 p = kernel.construct_point_2_object()(ORIGIN);
+        return (kernel.compare_y_at_x_2_object()(p, xcv1.supp_line(),
+                                                 xcv2.supp_line()));
       }
 
       // Flip the slope result if we compare at x = -oo:
@@ -1391,21 +1224,18 @@ public:
     }
   };
 
-
   /*! Obtain a Compare_y_limit_on_boundary_2 function object */
   Compare_y_near_boundary_2 compare_y_near_boundary_2_object() const
-  { return Compare_y_near_boundary_2(this); }
+  { return Compare_y_near_boundary_2(*this); }
 
   //@}
 
   /// \name Functor definitions for supporting intersections.
   //@{
 
-  class Make_x_monotone_2
-  {
+  class Make_x_monotone_2 {
   public:
-    /*!
-     * Cut the given curve into x-monotone subcurves and insert them into the
+    /*! Cut the given curve into x-monotone subcurves and insert them into the
      * given output iterator. As segments are always x_monotone, only one
      * object will be contained in the iterator.
      * \param cv The curve.
@@ -1414,73 +1244,72 @@ public:
      *           essentially the same as the input curve.
      * \return The past-the-end iterator.
      */
-    template<class OutputIterator>
-    OutputIterator operator() (const Curve_2& cv, OutputIterator oi) const
+    template <typename OutputIterator>
+    OutputIterator operator()(const Curve_2& cv, OutputIterator oi) const
     {
       // Wrap the curve with an object.
-      *oi = make_object (cv);
-      ++oi;
-
-      return (oi);
+      *oi++ = make_object(cv);
+      return oi;
     }
   };
 
   /*! Obtain a Make_x_monotone_2 functor object. */
-  Make_x_monotone_2 make_x_monotone_2_object () const
-  {
-    return Make_x_monotone_2();
-  }
+  Make_x_monotone_2 make_x_monotone_2_object() const
+  { return Make_x_monotone_2(); }
 
-  class Split_2
-  {
+  class Split_2 {
   public:
-    /*!
-     * Split a given x-monotone curve at a given point into two sub-curves.
+    /*! Split a given x-monotone curve at a given point into two sub-curves.
      * \param cv The curve to split
      * \param p The split point.
      * \param c1 Output: The left resulting subcurve (p is its right endpoint).
      * \param c2 Output: The right resulting subcurve (p is its left endpoint).
      * \pre p lies on cv but is not one of its end-points.
      */
-    void operator() (const X_monotone_curve_2& cv, const Point_2& p,
-                     X_monotone_curve_2& c1, X_monotone_curve_2& c2) const
+    void operator()(const X_monotone_curve_2& cv, const Point_2& p,
+                    X_monotone_curve_2& c1, X_monotone_curve_2& c2) const
     {
       CGAL_precondition (! cv.is_degenerate());
 
       // Make sure that p lies on the interior of the curve.
       CGAL_precondition_code (
-        Kernel                        kernel;
+        Kernel kernel;
         typename Kernel::Compare_xy_2 compare_xy = kernel.compare_xy_2_object();
       );
 
       CGAL_precondition
-        (Segment_assertions::_assert_is_point_on (p, cv,
-                                                  Has_exact_division()) &&
+        (Segment_assertions::_assert_is_point_on(p, cv, Has_exact_division()) &&
          (! cv.has_left() || compare_xy(cv.left(), p) == SMALLER) &&
          (! cv.has_right() || compare_xy(cv.right(), p) == LARGER));
 
       // Perform the split.
       c1 = cv;
-      c1.set_right (p);
+      c1.set_right(p);
 
       c2 = cv;
-      c2.set_left (p);
-
-      return;
+      c2.set_left(p);
     }
   };
 
   /*! Obtain a Split_2 functor object. */
-  Split_2 split_2_object () const
-  {
-    return Split_2();
-  }
+  Split_2 split_2_object() const { return Split_2(); }
+
+  class Intersect_2 {
+  protected:
+    typedef Arr_linear_traits_2<Kernel>        Traits;
+
+    /*! The traits (in case it has state) */
+    const Traits& m_traits;
+
+    /*! Constructor
+     * \param traits the traits (in case it has state)
+     */
+    Intersect_2(const Traits& traits) : m_traits(traits) {}
+
+    friend class Arr_linear_traits_2<Kernel>;
 
-  class Intersect_2
-  {
   public:
-    /*!
-     * Find the intersections of the two given curves and insert them into the
+    /*! Find the intersections of the two given curves and insert them into the
      * given output iterator. As two segments may itersect only once, only a
      * single intersection will be contained in the iterator.
      * \param cv1 The first curve.
@@ -1488,195 +1317,170 @@ public:
      * \param oi The output iterator.
      * \return The past-the-end iterator.
      */
-    template<class OutputIterator>
-    OutputIterator operator() (const X_monotone_curve_2& cv1,
-                               const X_monotone_curve_2& cv2,
-                               OutputIterator oi) const
-    {
-      CGAL_precondition (! cv1.is_degenerate());
-      CGAL_precondition (! cv2.is_degenerate());
+    template <typename OutputIterator>
+    OutputIterator operator()(const X_monotone_curve_2& cv1,
+                              const X_monotone_curve_2& cv2,
+                              OutputIterator oi) const
+    {
+      typedef std::pair<Point_2, Multiplicity>          Intersection_point;
+      typedef boost::variant<Intersection_point, X_monotone_curve_2>
+                                                        Intersection_result;
+
+      CGAL_precondition(! cv1.is_degenerate());
+      CGAL_precondition(! cv2.is_degenerate());
 
       // Intersect the two supporting lines.
-      Kernel       kernel;
-      CGAL::Object obj = kernel.intersect_2_object()(cv1.supp_line(),
-                                                     cv2.supp_line());
+      const Kernel& kernel = m_traits;
+      auto res = kernel.intersect_2_object()(cv1.supp_line(), cv2.supp_line());
 
-      if (obj.is_empty())
-      {
-        // The supporting line are parallel lines and do not intersect:
-        return (oi);
-      }
+      // The supporting line are parallel lines and do not intersect:
+      if (! res) return oi;
 
       // Check whether we have a single intersection point.
-      const Point_2  *ip = object_cast<Point_2> (&obj);
-
-      if (ip != nullptr)
-      {
+      const Point_2* ip = boost::get<Point_2>(&*res);
+      if (ip != nullptr) {
         // Check whether the intersection point ip lies on both segments.
-        const bool    ip_on_cv1 = cv1.is_vertical() ? cv1.is_in_y_range(*ip) :
-                                                      cv1.is_in_x_range(*ip);
+        const bool ip_on_cv1 = cv1.is_vertical() ?
+          cv1.is_in_y_range(*ip) : cv1.is_in_x_range(*ip);
 
-        if (ip_on_cv1)
-        {
-          const bool  ip_on_cv2 = cv2.is_vertical() ? cv2.is_in_y_range(*ip) :
-                                                      cv2.is_in_x_range(*ip);
+        if (ip_on_cv1) {
+          const bool ip_on_cv2 = cv2.is_vertical() ?
+            cv2.is_in_y_range(*ip) : cv2.is_in_x_range(*ip);
 
-          if (ip_on_cv2)
-          {
+          if (ip_on_cv2) {
             // Create a pair representing the point with its multiplicity,
             // which is always 1 for line segments.
-            std::pair<Point_2,Multiplicity>   ip_mult (*ip, 1);
-            *oi = make_object (ip_mult);
-            oi++;
+            Intersection_point ip_mult(*ip, 1);
+            *oi++ = Intersection_result(ip_mult);
           }
         }
-        return (oi);
+        return oi;
       }
 
       // In this case, the two supporting lines overlap.
       // We start with the entire cv1 curve as the overlapping subcurve,
       // then clip it to form the true overlapping curve.
-      typename Kernel::Compare_xy_2  compare_xy = kernel.compare_xy_2_object();
-      X_monotone_curve_2             ovlp = cv1;
+      auto compare_xy = kernel.compare_xy_2_object();
+      X_monotone_curve_2 ovlp = cv1;
 
-      if (cv2.has_left())
-      {
+      if (cv2.has_left()) {
         // If the left endpoint of cv2 is to the right of cv1's left endpoint,
         // clip the overlapping subcurve.
-        if (! cv1.has_left())
-        {
+        if (! cv1.has_left()) {
           ovlp.set_left (cv2.left(), false);
         }
-        else
-        {
-          if (compare_xy (cv1.left(), cv2.left()) == SMALLER)
-            ovlp.set_left (cv2.left(), false);
+        else {
+          if (compare_xy(cv1.left(), cv2.left()) == SMALLER)
+            ovlp.set_left(cv2.left(), false);
         }
       }
 
-      if (cv2.has_right())
-      {
+      if (cv2.has_right()) {
         // If the right endpoint of cv2 is to the left of cv1's right endpoint,
         // clip the overlapping subcurve.
-        if (! cv1.has_right())
-        {
-          ovlp.set_right (cv2.right(), false);
+        if (! cv1.has_right()) {
+          ovlp.set_right(cv2.right(), false);
         }
-        else
-        {
-          if (compare_xy (cv1.right(), cv2.right()) == LARGER)
-            ovlp.set_right (cv2.right(), false);
+        else {
+          if (compare_xy(cv1.right(), cv2.right()) == LARGER)
+            ovlp.set_right(cv2.right(), false);
         }
       }
 
       // Examine the resulting subcurve.
-      Comparison_result        res = SMALLER;
+      Comparison_result cmp_res = SMALLER;
 
       if (ovlp.has_left() && ovlp.has_right())
-        res = compare_xy (ovlp.left(), ovlp.right());
+        cmp_res = compare_xy(ovlp.left(), ovlp.right());
 
-      if (res == SMALLER)
-      {
+      if (cmp_res == SMALLER) {
         // We have discovered a true overlapping subcurve:
-        *oi = make_object (ovlp);
-        oi++;
+        *oi++ = Intersection_result(ovlp);
       }
-      else if (res == EQUAL)
-      {
+      else if (cmp_res == EQUAL) {
         // The two objects have the same supporting line, but they just share
         // a common endpoint. Thus we have an intersection point, but we leave
         // the multiplicity of this point undefined.
-        std::pair<Point_2,Multiplicity>   ip_mult (ovlp.left(), 0);
-        *oi = make_object (ip_mult);
-        oi++;
+        Intersection_point ip_mult(ovlp.left(), 0);
+        *oi++ = Intersection_result(ip_mult);
       }
 
-      return (oi);
+      return oi;
     }
   };
 
   /*! Obtain an Intersect_2 functor object. */
-  Intersect_2 intersect_2_object () const
-  {
-    return Intersect_2();
-  }
+  Intersect_2 intersect_2_object () const { return Intersect_2(*this); }
 
-  class Are_mergeable_2
-  {
+  class Are_mergeable_2 {
   public:
-    /*!
-     * Check whether it is possible to merge two given x-monotone curves.
+    /*! Check whether it is possible to merge two given x-monotone curves.
      * \param cv1 The first curve.
      * \param cv2 The second curve.
      * \return (true) if the two curves are mergeable - if they are supported
      *         by the same line and share a common endpoint; (false) otherwise.
      */
-    bool operator() (const X_monotone_curve_2& cv1,
+    bool operator()(const X_monotone_curve_2& cv1,
                      const X_monotone_curve_2& cv2) const
     {
-      CGAL_precondition (! cv1.is_degenerate());
-      CGAL_precondition (! cv2.is_degenerate());
+      CGAL_precondition(! cv1.is_degenerate());
+      CGAL_precondition(! cv2.is_degenerate());
 
-      Kernel                    kernel;
-      typename Kernel::Equal_2  equal = kernel.equal_2_object();
+      Kernel kernel;
+      typename Kernel::Equal_2 equal = kernel.equal_2_object();
 
       // Check whether the two curves have the same supporting line.
-      if (! equal (cv1.supp_line(), cv2.supp_line()) &&
-          ! equal (cv1.supp_line(),
-                   kernel.construct_opposite_line_2_object()(cv2.supp_line())))
-        return (false);
+      if (! equal(cv1.supp_line(), cv2.supp_line()) &&
+          ! equal(cv1.supp_line(),
+                  kernel.construct_opposite_line_2_object()(cv2.supp_line())))
+        return false;
 
       // Check whether the left endpoint of one curve is the right endpoint of the
       // other.
       return ((cv1.has_right() && cv2.has_left() &&
-               equal (cv1.right(), cv2.left())) ||
+               equal(cv1.right(), cv2.left())) ||
               (cv2.has_right() && cv1.has_left() &&
-               equal (cv2.right(), cv1.left())));
+               equal(cv2.right(), cv1.left())));
     }
   };
 
   /*! Obtain an Are_mergeable_2 functor object. */
-  Are_mergeable_2 are_mergeable_2_object () const
-  {
-    return Are_mergeable_2();
-  }
+  Are_mergeable_2 are_mergeable_2_object () const { return Are_mergeable_2(); }
 
   /*! \class Merge_2
    * A functor that merges two x-monotone arcs into one.
    */
-  class Merge_2
-  {
+  class Merge_2 {
   protected:
     typedef Arr_linear_traits_2<Kernel> Traits;
 
     /*! The traits (in case it has state) */
-    const Traits* m_traits;
+    const Traits& m_traits;
 
     /*! Constructor
      * \param traits the traits (in case it has state)
      */
-    Merge_2(const Traits* traits) : m_traits(traits) {}
+    Merge_2(const Traits& traits) : m_traits(traits) {}
 
     friend class Arr_linear_traits_2<Kernel>;
 
   public:
-    /*!
-     * Merge two given x-monotone curves into a single curve (segment).
+    /*! Merge two given x-monotone curves into a single curve (segment).
      * \param cv1 The first curve.
      * \param cv2 The second curve.
      * \param c Output: The merged curve.
      * \pre The two curves are mergeable.
      */
-    void operator() (const X_monotone_curve_2& cv1,
-                     const X_monotone_curve_2& cv2,
-                     X_monotone_curve_2& c) const
+    void operator()(const X_monotone_curve_2& cv1,
+                    const X_monotone_curve_2& cv2,
+                    X_monotone_curve_2& c) const
     {
-      CGAL_precondition(m_traits->are_mergeable_2_object()(cv2, cv1));
+      CGAL_precondition(m_traits.are_mergeable_2_object()(cv2, cv1));
 
       CGAL_precondition(!cv1.is_degenerate());
       CGAL_precondition(!cv2.is_degenerate());
 
-      Equal_2 equal = m_traits->equal_2_object();
+      Equal_2 equal = m_traits.equal_2_object();
 
       // Check which curve extends to the right of the other.
       if (cv1.has_right() && cv2.has_left() &&
@@ -1685,10 +1489,8 @@ public:
         // cv2 extends cv1 to the right.
         c = cv1;
 
-        if (cv2.has_right())
-          c.set_right(cv2.right());
-        else
-          c.set_right();      // Unbounded endpoint.
+        if (cv2.has_right()) c.set_right(cv2.right());
+        else c.set_right();      // Unbounded endpoint.
       }
       else {
         CGAL_precondition(cv2.has_right() && cv1.has_left() &&
@@ -1697,87 +1499,67 @@ public:
         // cv1 extends cv2 to the right.
         c = cv2;
 
-        if (cv1.has_right())
-          c.set_right(cv1.right());
-        else
-          c.set_right();      // Unbounded endpoint.
+        if (cv1.has_right()) c.set_right(cv1.right());
+        else c.set_right();      // Unbounded endpoint.
       }
     }
   };
 
   /*! Obtain a Merge_2 functor object. */
-  Merge_2 merge_2_object () const { return Merge_2(this); }
+  Merge_2 merge_2_object() const { return Merge_2(*this); }
   //@}
 
   /// \name Functor definitions for the landmarks point-location strategy.
   //@{
   typedef double                          Approximate_number_type;
 
-  class Approximate_2
-  {
+  class Approximate_2 {
   public:
-
-    /*!
-     * Return an approximation of a point coordinate.
+    /*! Obtain an approximation of a point coordinate.
      * \param p The exact point.
      * \param i The coordinate index (either 0 or 1).
      * \pre i is either 0 or 1.
      * \return An approximation of p's x-coordinate (if i == 0), or an
      *         approximation of p's y-coordinate (if i == 1).
      */
-    Approximate_number_type operator() (const Point_2& p,
-                                        int i) const
+    Approximate_number_type operator()(const Point_2& p, int i) const
     {
-      CGAL_precondition (i == 0 || i == 1);
-
-      if (i == 0)
-        return (CGAL::to_double(p.x()));
-      else
-        return (CGAL::to_double(p.y()));
+      CGAL_precondition((i == 0) || (i == 1));
+      return (i == 0) ? CGAL::to_double(p.x()) : CGAL::to_double(p.y());
     }
   };
 
   /*! Obtain an Approximate_2 functor object. */
-  Approximate_2 approximate_2_object () const
-  {
-    return Approximate_2();
-  }
+  Approximate_2 approximate_2_object() const { return Approximate_2(); }
 
-  class Construct_x_monotone_curve_2
-  {
+  class Construct_x_monotone_curve_2 {
   public:
-
-    /*!
-     * Return an x-monotone curve connecting the two given endpoints.
+    /*! Obtain an x-monotone curve connecting the two given endpoints.
      * \param p The first point.
      * \param q The second point.
      * \pre p and q must not be the same.
      * \return A segment connecting p and q.
      */
-    X_monotone_curve_2 operator() (const Point_2& p,
-                                   const Point_2& q) const
+    X_monotone_curve_2 operator()(const Point_2& p, const Point_2& q) const
     {
-      Kernel     kernel;
-      Segment_2  seg = kernel.construct_segment_2_object() (p, q);
+      Kernel kernel;
+      Segment_2 seg = kernel.construct_segment_2_object()(p, q);
 
-      return (X_monotone_curve_2 (seg));
+      return (X_monotone_curve_2(seg));
     }
   };
 
   /*! Obtain a Construct_x_monotone_curve_2 functor object. */
-  Construct_x_monotone_curve_2 construct_x_monotone_curve_2_object () const
-  {
-    return Construct_x_monotone_curve_2();
-  }
+  Construct_x_monotone_curve_2 construct_x_monotone_curve_2_object() const
+  { return Construct_x_monotone_curve_2(); }
   //@}
-
 };
 
 /*!
  * \class A representation of a segment, as used by the Arr_segment_traits_2
  * traits-class.
  */
-template <class Kernel_>
+template <typename Kernel_>
 class Arr_linear_object_2 :
     public Arr_linear_traits_2<Kernel_>::_Linear_object_cached_2
 {
@@ -1785,7 +1567,6 @@ class Arr_linear_object_2 :
                                                                    Base;
 
 public:
-
   typedef Kernel_                                           Kernel;
 
   typedef typename Kernel::Point_2                          Point_2;
@@ -1794,121 +1575,92 @@ public:
   typedef typename Kernel::Line_2                           Line_2;
 
 public:
-
-  /*!
-   * Default constructor.
+  /*! Default constructor.
    */
-  Arr_linear_object_2 () :
-    Base()
-  {}
+  Arr_linear_object_2() : Base() {}
 
-  /*!
-   * Constructor from two points.
+  /*! Constructor from two points.
    * \param s The source point.
    * \param t The target point.
    * \pre The two points must not be the same.
    */
-  Arr_linear_object_2(const Point_2& s, const Point_2& t):
-    Base(s, t)
-  {}
+  Arr_linear_object_2(const Point_2& s, const Point_2& t) : Base(s, t) {}
 
-  /*!
-   * Constructor from a segment.
+  /*! Constructor from a segment.
    * \param seg The segment.
    * \pre The segment is not degenerate.
    */
-  Arr_linear_object_2 (const Segment_2& seg) :
-    Base (seg)
-  {}
+  Arr_linear_object_2(const Segment_2& seg) : Base(seg) {}
 
-  /*!
-   * Constructor from a ray.
+  /*! Constructor from a ray.
    * \param ray The segment.
    * \pre The ray is not degenerate.
    */
-  Arr_linear_object_2 (const Ray_2& ray) :
-    Base (ray)
-  {}
+  Arr_linear_object_2(const Ray_2& ray) : Base(ray) {}
 
-  /*!
-   * Constructor from a line.
+  /*! Constructor from a line.
    * \param line The line.
    * \pre The line is not degenerate.
    */
-  Arr_linear_object_2 (const Line_2& line) :
-    Base (line)
-  {}
+  Arr_linear_object_2(const Line_2& line) : Base(line) {}
 
-  /*!
-   * Check whether the object is actually a segment.
+  /*! Check whether the object is actually a segment.
    */
-  bool is_segment () const
-  {
-    return (! this->is_degen && this->has_source && this->has_target);
-  }
+  bool is_segment() const
+  { return (! this->is_degen && this->has_source && this->has_target); }
 
-  /*!
-   * Cast to a segment.
+  /*! Cast to a segment.
    * \pre The linear object is really a segment.
    */
-  Segment_2 segment () const
+  Segment_2 segment() const
   {
-    CGAL_precondition (is_segment());
+    CGAL_precondition(is_segment());
 
-    Kernel     kernel;
-    Segment_2  seg = kernel.construct_segment_2_object() (this->ps, this->pt);
+    Kernel kernel;
+    Segment_2 seg = kernel.construct_segment_2_object()(this->ps, this->pt);
     return seg;
   }
 
-  /*!
-   * Check whether the object is actually a ray.
+  /*! Check whether the object is actually a ray.
    */
-  bool is_ray () const
-  {
-    return (! this->is_degen && (this->has_source != this->has_target));
-  }
+  bool is_ray() const
+  { return (! this->is_degen && (this->has_source != this->has_target)); }
 
-  /*!
-   * Cast to a ray.
+  /*! Cast to a ray.
    * \pre The linear object is really a ray.
    */
-  Ray_2 ray () const
+  Ray_2 ray() const
   {
-    CGAL_precondition (is_ray());
+    CGAL_precondition(is_ray());
 
-    Kernel     kernel;
-    Ray_2      ray = (this->has_source) ?
-      kernel.construct_ray_2_object() (this->ps, this->l) :
+    Kernel kernel;
+    Ray_2 ray = (this->has_source) ?
+      kernel.construct_ray_2_object()(this->ps, this->l) :
       kernel.construct_ray_2_object()
         (this->pt, kernel.construct_opposite_line_2_object()(this->l));
     return ray;
   }
 
-  /*!
-   * Check whether the object is actually a line.
+  /*! Check whether the object is actually a line.
    */
-  bool is_line () const
-  {
-    return (! this->is_degen && ! this->has_source && ! this->has_target);
-  }
+  bool is_line() const
+  { return (! this->is_degen && ! this->has_source && ! this->has_target); }
 
-  /*!
-   * Cast to a line.
+  /*! Cast to a line.
    * \pre The linear object is really a line.
    */
-  Line_2 line () const
+  Line_2 line() const
   {
-    CGAL_precondition (is_line());
+    CGAL_precondition(is_line());
     return (this->l);
   }
 
-  /*!
-   * Get the supporting line.
+  /*! Get the supporting line.
    * \pre The object is not a point.
    */
-  const Line_2& supporting_line () const
+  const Line_2& supporting_line() const
   {
-    CGAL_precondition (! this->is_degen);
+    CGAL_precondition(! this->is_degen);
     return (this->l);
   }
 
@@ -1918,37 +1670,30 @@ public:
    */
   const Point_2& source() const
   {
-    CGAL_precondition (! is_line());
-
-    if (this->is_degen)
-      return (this->ps);      // For a point.
+    CGAL_precondition(! is_line());
 
-    if (this->has_source)
-      return (this->ps);      // For a segment or a ray.
-    else
-      return (this->pt);      // For a "flipped" ray.
+    if (this->is_degen) return (this->ps);      // For a point.
+    if (this->has_source) return (this->ps);    // For a segment or a ray.
+    else return (this->pt);                     // For a "flipped" ray.
   }
 
-  /*!
-   * Get the target point.
+  /*! Get the target point.
    * \pre The object is a point or a segment.
    */
   const Point_2& target() const
   {
-    CGAL_precondition (! is_line() && ! is_ray());
-
+    CGAL_precondition(! is_line() && ! is_ray());
     return (this->pt);
   }
 
-  /*!
-   * Create a bounding box for the linear object.
+  /*! Create a bounding box for the linear object.
    */
   Bbox_2 bbox() const
   {
     CGAL_precondition(this->is_segment());
-    Kernel     kernel;
-    Segment_2  seg = kernel.construct_segment_2_object() (this->ps, this->pt);
-    return (kernel.construct_bbox_2_object() (seg));
+    Kernel kernel;
+    Segment_2 seg = kernel.construct_segment_2_object()(this->ps, this->pt);
+    return (kernel.construct_bbox_2_object()(seg));
   }
 
   // Introducing casting operators instead from a curve to
@@ -1962,58 +1707,49 @@ public:
 /*!
  * Exporter for the segment class used by the traits-class.
  */
-template <class Kernel, class OutputStream>
-OutputStream& operator<< (OutputStream& os,
-                          const Arr_linear_object_2<Kernel>& lobj)
+template <typename Kernel, typename OutputStream>
+OutputStream& operator<<(OutputStream& os,
+                         const Arr_linear_object_2<Kernel>& lobj)
 {
   // Print a letter identifying the object type, then the object itself.
-  if (lobj.is_segment())
-    os << " S " << lobj.segment();
-  else if (lobj.is_ray())
-    os << " R " << lobj.ray();
-  else
-    os << " L " << lobj.line();
-
-  return (os);
+  if (lobj.is_segment()) os << " S " << lobj.segment();
+  else if (lobj.is_ray()) os << " R " << lobj.ray();
+  else os << " L " << lobj.line();
+  return os;
 }
 
-/*!
- * Importer for the segment class used by the traits-class.
+/*! Importer for the segment class used by the traits-class.
  */
-template <class Kernel, class InputStream>
-InputStream& operator>> (InputStream& is, Arr_linear_object_2<Kernel>& lobj)
+template <typename Kernel, typename InputStream>
+InputStream& operator>>(InputStream& is, Arr_linear_object_2<Kernel>& lobj)
 {
   // Read the object type.
-  char        c;
+  char c;
 
-  do
-  {
+  do {
     is >> c;
   } while ((c != 'S' && c != 's') &&
            (c != 'R' && c != 'r') &&
            (c != 'L' && c != 'l'));
 
   // Read the object accordingly.
-  if (c == 'S' || c == 's')
-  {
+  if (c == 'S' || c == 's') {
     typename Kernel::Segment_2  seg;
     is >> seg;
     lobj = seg;
   }
-  else if (c == 'R' || c == 'r')
-  {
-    typename Kernel::Ray_2      ray;
+  else if (c == 'R' || c == 'r') {
+    typename Kernel::Ray_2 ray;
     is >> ray;
     lobj = ray;
   }
-  else
-  {
-    typename Kernel::Line_2     line;
+  else {
+    typename Kernel::Line_2 line;
     is >> line;
     lobj = line;
   }
 
-  return (is);
+  return is;
 }
 
 } //namespace CGAL
diff -pruN 5.0.3-1/include/CGAL/Arr_naive_point_location.h 5.1-1/include/CGAL/Arr_naive_point_location.h
--- 5.0.3-1/include/CGAL/Arr_naive_point_location.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_naive_point_location.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_naive_point_location.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_naive_point_location.h $
 // $Id: Arr_naive_point_location.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_non_caching_segment_basic_traits_2.h 5.1-1/include/CGAL/Arr_non_caching_segment_basic_traits_2.h
--- 5.0.3-1/include/CGAL/Arr_non_caching_segment_basic_traits_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_non_caching_segment_basic_traits_2.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_non_caching_segment_basic_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_non_caching_segment_basic_traits_2.h $
 // $Id: Arr_non_caching_segment_basic_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_non_caching_segment_traits_2.h 5.1-1/include/CGAL/Arr_non_caching_segment_traits_2.h
--- 5.0.3-1/include/CGAL/Arr_non_caching_segment_traits_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_non_caching_segment_traits_2.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,13 +3,13 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_non_caching_segment_traits_2.h $
-// $Id: Arr_non_caching_segment_traits_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_non_caching_segment_traits_2.h $
+// $Id: Arr_non_caching_segment_traits_2.h 58276ed 2020-03-31T18:34:28+03:00 Efi Fogel
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
-// Author(s)     : Efi Fogel    <efif@post.tau.ac.il>
-//                 Ron Wein     <wein@post.tau.ac.il>
-//                 (base on old version by: Iddo Hanniel)
+// Author(s): Efi Fogel    <efif@post.tau.ac.il>
+//            Ron Wein     <wein@post.tau.ac.il>
+//            (base on old version by: Iddo Hanniel)
 
 #ifndef CGAL_ARR_NON_CACHING_SEGMENT_TRAITS_H
 #define CGAL_ARR_NON_CACHING_SEGMENT_TRAITS_H
@@ -27,6 +27,8 @@
  * functors required by the concept it models.
  */
 
+#include <boost/variant.hpp>
+
 #include <CGAL/Exact_predicates_exact_constructions_kernel.h>
 #include <CGAL/tags.h>
 #include <CGAL/Arr_tags.h>
@@ -39,7 +41,7 @@ namespace CGAL {
  * A model of the ArrangementTraits_2 concept that handles general
  * line segments.
  */
-template <class Kernel_T = Exact_predicates_exact_constructions_kernel>
+template <typename Kernel_T = Exact_predicates_exact_constructions_kernel>
 class Arr_non_caching_segment_traits_2 :
   public Arr_non_caching_segment_basic_traits_2<Kernel_T>
 {
@@ -114,12 +116,10 @@ public:
   /*! \class
    * A functor for splitting curves into x-monotone pieces.
    */
-  class Make_x_monotone_2
-  {
+  class Make_x_monotone_2 {
   public:
 
-    /*!
-     * Cut the given segment into x-monotone subcurves and insert them into
+    /*! Cut the given segment into x-monotone subcurves and insert them into
      * the given output iterator. As segments are always x_monotone, only one
      * x-monotone curve is inserted into the output iterator.
      * \param cv The segment.
@@ -127,54 +127,47 @@ public:
      *           object is a wrapper of an X_monotone_curve_2 object.
      * \return The past-the-end iterator.
      */
-    template<class OutputIterator>
-    OutputIterator operator()(const Curve_2 & cv, OutputIterator oi) const
+    template <typename OutputIterator>
+    OutputIterator operator()(const Curve_2& cv, OutputIterator oi) const
     {
-      *oi = make_object (cv);
-      ++oi;
-      return (oi);
+      *oi++ = make_object(cv);
+      return oi;
     }
   };
 
-  /*! Get a Make_x_monotone_2 functor object. */
+  /*! Obtain a Make_x_monotone_2 functor object. */
   Make_x_monotone_2 make_x_monotone_2_object() const
-  {
-    return Make_x_monotone_2();
-  }
+  { return Make_x_monotone_2(); }
 
   /*! \class
    * A functor for splitting a segment into two segements.
    */
-  class Split_2
-  {
+  class Split_2 {
     typedef Arr_non_caching_segment_traits_2<Kernel_T>    Self;
-  public:
 
-    /*!
-     * Split a given x-monotone curve at a given point into two sub-curves.
+  public:
+    /*! Split a given x-monotone curve at a given point into two sub-curves.
      * \param cv The curve to split
      * \param p The split point.
      * \param c1 Output: The left resulting subcurve (p is its right endpoint).
      * \param c2 Output: The right resulting subcurve (p is its left endpoint).
      * \pre p lies on cv but is not one of its end-points.
      */
-    void operator()(const X_monotone_curve_2 & cv, const Point_2 & p,
-                    X_monotone_curve_2 & c1, X_monotone_curve_2 & c2) const
+    void operator()(const X_monotone_curve_2& cv, const Point_2& p,
+                    X_monotone_curve_2& c1, X_monotone_curve_2& c2) const
     {
       Base base;
 
       // Make sure that p lies on the interior of the curve.
-      CGAL_precondition_code (
-        Compare_xy_2 compare_xy = base.compare_xy_2_object();
-      );
+      CGAL_precondition_code(auto compare_xy = base.compare_xy_2_object());
 
       Construct_min_vertex_2 min_vertex = base.construct_min_vertex_2_object();
       Construct_max_vertex_2 max_vertex = base.construct_max_vertex_2_object();
 
-      const Point_2 & left = min_vertex(cv);
-      const Point_2 & right = max_vertex(cv);
+      const Point_2& left = min_vertex(cv);
+      const Point_2& right = max_vertex(cv);
       CGAL_precondition
-        (Segment_assertions::_assert_is_point_on(p, cv, Has_exact_division())&&
+        (Segment_assertions::_assert_is_point_on(p, cv, Has_exact_division()) &&
          compare_xy(left, p) == SMALLER &&
          compare_xy(right, p) == LARGER);
 
@@ -182,31 +175,36 @@ public:
         base.construct_segment_2_object();
 
       Self self;
-      if(self.compare_endpoints_xy_2_object()(cv) == SMALLER)
-      {
+      if (self.compare_endpoints_xy_2_object()(cv) == SMALLER) {
         c1 = construct_segment(left, p);
         c2 = construct_segment(p, right);
       }
-      else
-      {
+      else {
         c1 = construct_segment(p, left);
         c2 = construct_segment(right, p);
       }
     }
   };
 
-  /*! Get a Split_2 functor object. */
-  Split_2 split_2_object() const
-  {
-    return Split_2();
-  }
+  /*! Obtain a Split_2 functor object. */
+  Split_2 split_2_object() const { return Split_2(); }
 
   /*! \class
    * A functor for computing intersections.
    */
-  class Intersect_2
-  {
-    typedef Arr_non_caching_segment_traits_2<Kernel_T>    Self;
+  class Intersect_2 {
+  protected:
+    typedef Arr_non_caching_segment_traits_2<Kernel>        Traits;
+
+    /*! The traits (in case it has state) */
+    const Traits& m_traits;
+
+    /*! Constructor
+     * \param traits the traits (in case it has state)
+     */
+    Intersect_2(const Traits& traits) : m_traits(traits) {}
+
+    friend class Arr_non_caching_segment_traits_2<Kernel>;
 
   public:
     /*! Find the intersections of the two given segments and insert them into
@@ -217,67 +215,56 @@ public:
      * \param oi The output iterator.
      * \return The past-the-end iterator.
      */
-    template<class OutputIterator>
-    OutputIterator operator()(const X_monotone_curve_2 & cv1,
-                              const X_monotone_curve_2 & cv2,
+    template <typename OutputIterator>
+    OutputIterator operator()(const X_monotone_curve_2& cv1,
+                              const X_monotone_curve_2& cv2,
                               OutputIterator oi) const
     {
-      Base   base;
-      Object res = base.intersect_2_object()(cv1, cv2);
+      typedef std::pair<Point_2, Multiplicity>          Intersection_point;
+      typedef boost::variant<Intersection_point, X_monotone_curve_2>
+                                                        Intersection_result;
+
+      const Kernel& kernel = m_traits;
+      auto res = kernel.intersect_2_object()(cv1, cv2);
 
       // There is no intersection:
-      if (res.is_empty())
-        return (oi);
+      if (! res) return oi;
 
       // Chack if the intersection is a point:
-      const Point_2             *ip;
-
-      if ((ip = object_cast<Point_2> (&res)) != nullptr)
-      {
+      const Point_2* p_p = boost::get<Point_2>(&*res);
+      if (p_p != nullptr) {
         // Create a pair representing the point with its multiplicity,
         // which is always 1 for line segments for all practical purposes.
         // If the two segments intersect at their endpoints, then the
         // multiplicity is undefined, but we deliberately ignore it for
         // efficieny reasons.
-        std::pair<Point_2,Multiplicity> ip_mult(*ip, 1);
-        *oi = make_object (ip_mult);
-        ++oi;
+        *oi++ = Intersection_result(Intersection_point(*p_p, 1));
+        return oi;
       }
-      else
-      {
-        // The intersection is a segment.
-
-        const X_monotone_curve_2 *ov = object_cast<X_monotone_curve_2>(&res);
-        CGAL_assertion (ov != nullptr);
-
-        Self self;
-        Comparison_result cmp1 = self.compare_endpoints_xy_2_object()(cv1);
-        Comparison_result cmp2 = self.compare_endpoints_xy_2_object()(cv2);
-
-        if(cmp1 == cmp2)
-        {
-          // cv1 and cv2 have the same directions, maintain this direction
-          // in the overlap segment
-          if(self.compare_endpoints_xy_2_object()(*ov) != cmp1)
-          {
-            Kernel k;
-            res = make_object(k.construct_opposite_segment_2_object()(*ov));
-          }
-        }
 
-        *oi = res;
-        ++oi;
+      // The intersection is a segment.
+      const X_monotone_curve_2* cv_p = boost::get<X_monotone_curve_2>(&*res);
+      CGAL_assertion(cv_p != nullptr);
+
+      Comparison_result cmp1 = m_traits.compare_endpoints_xy_2_object()(cv1);
+      Comparison_result cmp2 = m_traits.compare_endpoints_xy_2_object()(cv2);
+
+      if (cmp1 == cmp2) {
+        // cv1 and cv2 have the same directions, maintain this direction
+        // in the overlap segment
+        if (m_traits.compare_endpoints_xy_2_object()(*cv_p) != cmp1) {
+          auto ctr_opposite = kernel.construct_opposite_segment_2_object();
+          *oi++ = Intersection_result(ctr_opposite(*cv_p));
+          return oi;
+        }
       }
-
-      return (oi);
+      *oi++ = Intersection_result(*cv_p);
+      return oi;
     }
   };
 
-  /*! Get an Intersect_2 functor object. */
-  Intersect_2 intersect_2_object() const
-  {
-    return Intersect_2();
-  }
+  /*! Obtain an Intersect_2 functor object. */
+  Intersect_2 intersect_2_object() const { return Intersect_2(*this); }
 
   /*! \class
    * A functor for testing whether two segments are mergeable.
@@ -297,24 +284,22 @@ public:
     friend class Arr_non_caching_segment_traits_2<Kernel>;
 
   public:
-
-    /*!
-     * Check whether it is possible to merge two given x-monotone curves.
+    /*! Check whether it is possible to merge two given x-monotone curves.
      * \param cv1 The first curve.
      * \param cv2 The second curve.
      * \return (true) if the two curves are mergeable, that is, if they are
      *         supported by the same line; (false) otherwise.
      * \pre cv1 and cv2 share a common endpoint.
      */
-    bool operator()(const X_monotone_curve_2 & cv1,
-                    const X_monotone_curve_2 & cv2) const
+    bool operator()(const X_monotone_curve_2& cv1,
+                    const X_monotone_curve_2& cv2) const
     {
       const Base* base = m_traits;
       Equal_2 equal = base->equal_2_object();
       Construct_min_vertex_2 min_vertex = base->construct_min_vertex_2_object();
       Construct_max_vertex_2 max_vertex = base->construct_max_vertex_2_object();
-      if (!equal(max_vertex(cv1), min_vertex(cv2)) &&
-          !equal(max_vertex(cv2), min_vertex(cv1)))
+      if (! equal(max_vertex(cv1), min_vertex(cv2)) &&
+          ! equal(max_vertex(cv2), min_vertex(cv1)))
         return false;
 
       // Check if the two curves have the same supporting line.
@@ -344,16 +329,15 @@ public:
     friend class Arr_non_caching_segment_traits_2<Kernel>;
 
   public:
-    /*!
-     * Merge two given segments into a single segment.
+    /*! Merge two given segments into a single segment.
      * \param cv1 The first curve.
      * \param cv2 The second curve.
      * \param c Output: The merged curve.
      * \pre The two curves are mergeable.
      */
-    void operator()(const X_monotone_curve_2 & cv1,
-                    const X_monotone_curve_2 & cv2,
-                    X_monotone_curve_2 & c) const
+    void operator()(const X_monotone_curve_2& cv1,
+                    const X_monotone_curve_2& cv2,
+                    X_monotone_curve_2& c) const
     {
       CGAL_precondition(m_traits->are_mergeable_2_object()(cv2, cv1));
 
@@ -390,12 +374,9 @@ public:
 
   /*! Obtain a Construct_opposite_2 functor object */
   Construct_opposite_2 construct_opposite_2_object() const
-  {
-    return Construct_opposite_2();
-  }
+  { return Construct_opposite_2(); }
 
-  class Compare_endpoints_xy_2
-  {
+  class Compare_endpoints_xy_2 {
   public:
     /*!
      * Compare the two endpoints of a given curve lexigoraphically.
@@ -417,9 +398,7 @@ public:
 
   /*! Obtain a Compare_endpoints_xy_2 functor object */
   Compare_endpoints_xy_2 compare_endpoints_xy_2_object() const
-  {
-    return Compare_endpoints_xy_2();
-  }
+  { return Compare_endpoints_xy_2(); }
   //@}
 };
 
diff -pruN 5.0.3-1/include/CGAL/Arr_observer.h 5.1-1/include/CGAL/Arr_observer.h
--- 5.0.3-1/include/CGAL/Arr_observer.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_observer.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_observer.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_observer.h $
 // $Id: Arr_observer.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_overlay_2.h 5.1-1/include/CGAL/Arr_overlay_2.h
--- 5.0.3-1/include/CGAL/Arr_overlay_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_overlay_2.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_overlay_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_overlay_2.h $
 // $Id: Arr_overlay_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_overlay.h 5.1-1/include/CGAL/Arr_overlay.h
--- 5.0.3-1/include/CGAL/Arr_overlay.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_overlay.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_overlay.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_overlay.h $
 // $Id: Arr_overlay.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_point_location/Arr_batched_point_location_traits_2.h 5.1-1/include/CGAL/Arr_point_location/Arr_batched_point_location_traits_2.h
--- 5.0.3-1/include/CGAL/Arr_point_location/Arr_batched_point_location_traits_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_point_location/Arr_batched_point_location_traits_2.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Arr_batched_point_location_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Arr_batched_point_location_traits_2.h $
 // $Id: Arr_batched_point_location_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_point_location/Arr_landmarks_pl_impl.h 5.1-1/include/CGAL/Arr_point_location/Arr_landmarks_pl_impl.h
--- 5.0.3-1/include/CGAL/Arr_point_location/Arr_landmarks_pl_impl.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_point_location/Arr_landmarks_pl_impl.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Arr_landmarks_pl_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Arr_landmarks_pl_impl.h $
 // $Id: Arr_landmarks_pl_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_point_location/Arr_lm_generator_base.h 5.1-1/include/CGAL/Arr_point_location/Arr_lm_generator_base.h
--- 5.0.3-1/include/CGAL/Arr_point_location/Arr_lm_generator_base.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_point_location/Arr_lm_generator_base.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Arr_lm_generator_base.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Arr_lm_generator_base.h $
 // $Id: Arr_lm_generator_base.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_point_location/Arr_lm_grid_generator.h 5.1-1/include/CGAL/Arr_point_location/Arr_lm_grid_generator.h
--- 5.0.3-1/include/CGAL/Arr_point_location/Arr_lm_grid_generator.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_point_location/Arr_lm_grid_generator.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Arr_lm_grid_generator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Arr_lm_grid_generator.h $
 // $Id: Arr_lm_grid_generator.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_point_location/Arr_lm_halton_generator.h 5.1-1/include/CGAL/Arr_point_location/Arr_lm_halton_generator.h
--- 5.0.3-1/include/CGAL/Arr_point_location/Arr_lm_halton_generator.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_point_location/Arr_lm_halton_generator.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Arr_lm_halton_generator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Arr_lm_halton_generator.h $
 // $Id: Arr_lm_halton_generator.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_point_location/Arr_lm_middle_edges_generator.h 5.1-1/include/CGAL/Arr_point_location/Arr_lm_middle_edges_generator.h
--- 5.0.3-1/include/CGAL/Arr_point_location/Arr_lm_middle_edges_generator.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_point_location/Arr_lm_middle_edges_generator.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Arr_lm_middle_edges_generator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Arr_lm_middle_edges_generator.h $
 // $Id: Arr_lm_middle_edges_generator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_point_location/Arr_lm_nearest_neighbor.h 5.1-1/include/CGAL/Arr_point_location/Arr_lm_nearest_neighbor.h
--- 5.0.3-1/include/CGAL/Arr_point_location/Arr_lm_nearest_neighbor.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_point_location/Arr_lm_nearest_neighbor.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Arr_lm_nearest_neighbor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Arr_lm_nearest_neighbor.h $
 // $Id: Arr_lm_nearest_neighbor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_point_location/Arr_lm_random_generator.h 5.1-1/include/CGAL/Arr_point_location/Arr_lm_random_generator.h
--- 5.0.3-1/include/CGAL/Arr_point_location/Arr_lm_random_generator.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_point_location/Arr_lm_random_generator.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Arr_lm_random_generator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Arr_lm_random_generator.h $
 // $Id: Arr_lm_random_generator.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_point_location/Arr_lm_specified_points_generator.h 5.1-1/include/CGAL/Arr_point_location/Arr_lm_specified_points_generator.h
--- 5.0.3-1/include/CGAL/Arr_point_location/Arr_lm_specified_points_generator.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_point_location/Arr_lm_specified_points_generator.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Arr_lm_specified_points_generator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Arr_lm_specified_points_generator.h $
 // $Id: Arr_lm_specified_points_generator.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_point_location/Arr_lm_vertices_generator.h 5.1-1/include/CGAL/Arr_point_location/Arr_lm_vertices_generator.h
--- 5.0.3-1/include/CGAL/Arr_point_location/Arr_lm_vertices_generator.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_point_location/Arr_lm_vertices_generator.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Arr_lm_vertices_generator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Arr_lm_vertices_generator.h $
 // $Id: Arr_lm_vertices_generator.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_point_location/Arr_naive_point_location_impl.h 5.1-1/include/CGAL/Arr_point_location/Arr_naive_point_location_impl.h
--- 5.0.3-1/include/CGAL/Arr_point_location/Arr_naive_point_location_impl.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_point_location/Arr_naive_point_location_impl.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Arr_naive_point_location_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Arr_naive_point_location_impl.h $
 // $Id: Arr_naive_point_location_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_point_location/Arr_simple_point_location_impl.h 5.1-1/include/CGAL/Arr_point_location/Arr_simple_point_location_impl.h
--- 5.0.3-1/include/CGAL/Arr_point_location/Arr_simple_point_location_impl.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_point_location/Arr_simple_point_location_impl.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Arr_simple_point_location_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Arr_simple_point_location_impl.h $
 // $Id: Arr_simple_point_location_impl.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_point_location/Arr_trapezoid_ric_pl_impl.h 5.1-1/include/CGAL/Arr_point_location/Arr_trapezoid_ric_pl_impl.h
--- 5.0.3-1/include/CGAL/Arr_point_location/Arr_trapezoid_ric_pl_impl.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_point_location/Arr_trapezoid_ric_pl_impl.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Arr_trapezoid_ric_pl_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Arr_trapezoid_ric_pl_impl.h $
 // $Id: Arr_trapezoid_ric_pl_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_point_location/Arr_triangulation_pl_functions.h 5.1-1/include/CGAL/Arr_point_location/Arr_triangulation_pl_functions.h
--- 5.0.3-1/include/CGAL/Arr_point_location/Arr_triangulation_pl_functions.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_point_location/Arr_triangulation_pl_functions.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Arr_triangulation_pl_functions.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Arr_triangulation_pl_functions.h $
 // $Id: Arr_triangulation_pl_functions.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_point_location/Arr_triangulation_pl_impl.h 5.1-1/include/CGAL/Arr_point_location/Arr_triangulation_pl_impl.h
--- 5.0.3-1/include/CGAL/Arr_point_location/Arr_triangulation_pl_impl.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_point_location/Arr_triangulation_pl_impl.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Arr_triangulation_pl_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Arr_triangulation_pl_impl.h $
 // $Id: Arr_triangulation_pl_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_point_location/Arr_walk_along_line_pl_impl.h 5.1-1/include/CGAL/Arr_point_location/Arr_walk_along_line_pl_impl.h
--- 5.0.3-1/include/CGAL/Arr_point_location/Arr_walk_along_line_pl_impl.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_point_location/Arr_walk_along_line_pl_impl.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Arr_walk_along_line_pl_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Arr_walk_along_line_pl_impl.h $
 // $Id: Arr_walk_along_line_pl_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_point_location/Td_active_edge.h 5.1-1/include/CGAL/Arr_point_location/Td_active_edge.h
--- 5.0.3-1/include/CGAL/Arr_point_location/Td_active_edge.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_point_location/Td_active_edge.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Td_active_edge.h $
-// $Id: Td_active_edge.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Td_active_edge.h $
+// $Id: Td_active_edge.h 319383c 2020-05-20T09:47:58+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)         : Oren Nechushtan <theoren@math.tau.ac.il>
diff -pruN 5.0.3-1/include/CGAL/Arr_point_location/Td_active_fictitious_vertex.h 5.1-1/include/CGAL/Arr_point_location/Td_active_fictitious_vertex.h
--- 5.0.3-1/include/CGAL/Arr_point_location/Td_active_fictitious_vertex.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_point_location/Td_active_fictitious_vertex.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Td_active_fictitious_vertex.h $
-// $Id: Td_active_fictitious_vertex.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Td_active_fictitious_vertex.h $
+// $Id: Td_active_fictitious_vertex.h 319383c 2020-05-20T09:47:58+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s): Oren Nechushtan   <theoren@math.tau.ac.il>
diff -pruN 5.0.3-1/include/CGAL/Arr_point_location/Td_active_trapezoid.h 5.1-1/include/CGAL/Arr_point_location/Td_active_trapezoid.h
--- 5.0.3-1/include/CGAL/Arr_point_location/Td_active_trapezoid.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_point_location/Td_active_trapezoid.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Td_active_trapezoid.h $
-// $Id: Td_active_trapezoid.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Td_active_trapezoid.h $
+// $Id: Td_active_trapezoid.h 319383c 2020-05-20T09:47:58+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)         : Oren Nechushtan <theoren@math.tau.ac.il>
diff -pruN 5.0.3-1/include/CGAL/Arr_point_location/Td_active_vertex.h 5.1-1/include/CGAL/Arr_point_location/Td_active_vertex.h
--- 5.0.3-1/include/CGAL/Arr_point_location/Td_active_vertex.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_point_location/Td_active_vertex.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Td_active_vertex.h $
-// $Id: Td_active_vertex.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Td_active_vertex.h $
+// $Id: Td_active_vertex.h 319383c 2020-05-20T09:47:58+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s): Oren Nechushtan   <theoren@math.tau.ac.il>
diff -pruN 5.0.3-1/include/CGAL/Arr_point_location/Td_dag.h 5.1-1/include/CGAL/Arr_point_location/Td_dag.h
--- 5.0.3-1/include/CGAL/Arr_point_location/Td_dag.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_point_location/Td_dag.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Td_dag.h $
-// $Id: Td_dag.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Td_dag.h $
+// $Id: Td_dag.h 319383c 2020-05-20T09:47:58+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_point_location/Td_dag_node.h 5.1-1/include/CGAL/Arr_point_location/Td_dag_node.h
--- 5.0.3-1/include/CGAL/Arr_point_location/Td_dag_node.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_point_location/Td_dag_node.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Td_dag_node.h $
-// $Id: Td_dag_node.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Td_dag_node.h $
+// $Id: Td_dag_node.h 319383c 2020-05-20T09:47:58+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Michal Balas <balasmic@post.tau.ac.il>
diff -pruN 5.0.3-1/include/CGAL/Arr_point_location/Td_inactive_edge.h 5.1-1/include/CGAL/Arr_point_location/Td_inactive_edge.h
--- 5.0.3-1/include/CGAL/Arr_point_location/Td_inactive_edge.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_point_location/Td_inactive_edge.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Td_inactive_edge.h $
-// $Id: Td_inactive_edge.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Td_inactive_edge.h $
+// $Id: Td_inactive_edge.h 319383c 2020-05-20T09:47:58+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)         : Oren Nechushtan <theoren@math.tau.ac.il>
diff -pruN 5.0.3-1/include/CGAL/Arr_point_location/Td_inactive_fictitious_vertex.h 5.1-1/include/CGAL/Arr_point_location/Td_inactive_fictitious_vertex.h
--- 5.0.3-1/include/CGAL/Arr_point_location/Td_inactive_fictitious_vertex.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_point_location/Td_inactive_fictitious_vertex.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Td_inactive_fictitious_vertex.h $
-// $Id: Td_inactive_fictitious_vertex.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Td_inactive_fictitious_vertex.h $
+// $Id: Td_inactive_fictitious_vertex.h 319383c 2020-05-20T09:47:58+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)         : Oren Nechushtan <theoren@math.tau.ac.il>
diff -pruN 5.0.3-1/include/CGAL/Arr_point_location/Td_inactive_trapezoid.h 5.1-1/include/CGAL/Arr_point_location/Td_inactive_trapezoid.h
--- 5.0.3-1/include/CGAL/Arr_point_location/Td_inactive_trapezoid.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_point_location/Td_inactive_trapezoid.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Td_inactive_trapezoid.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Td_inactive_trapezoid.h $
 // $Id: Td_inactive_trapezoid.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_point_location/Td_inactive_vertex.h 5.1-1/include/CGAL/Arr_point_location/Td_inactive_vertex.h
--- 5.0.3-1/include/CGAL/Arr_point_location/Td_inactive_vertex.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_point_location/Td_inactive_vertex.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Td_inactive_vertex.h $
-// $Id: Td_inactive_vertex.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Td_inactive_vertex.h $
+// $Id: Td_inactive_vertex.h 319383c 2020-05-20T09:47:58+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)         : Oren Nechushtan <theoren@math.tau.ac.il>
diff -pruN 5.0.3-1/include/CGAL/Arr_point_location/Td_ninetuple.h 5.1-1/include/CGAL/Arr_point_location/Td_ninetuple.h
--- 5.0.3-1/include/CGAL/Arr_point_location/Td_ninetuple.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_point_location/Td_ninetuple.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Td_ninetuple.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Td_ninetuple.h $
 // $Id: Td_ninetuple.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_point_location/Td_predicates.h 5.1-1/include/CGAL/Arr_point_location/Td_predicates.h
--- 5.0.3-1/include/CGAL/Arr_point_location/Td_predicates.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_point_location/Td_predicates.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Td_predicates.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Td_predicates.h $
 // $Id: Td_predicates.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_point_location/Td_traits.h 5.1-1/include/CGAL/Arr_point_location/Td_traits.h
--- 5.0.3-1/include/CGAL/Arr_point_location/Td_traits.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_point_location/Td_traits.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Td_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Td_traits.h $
 // $Id: Td_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_point_location/Td_X_trapezoid.h 5.1-1/include/CGAL/Arr_point_location/Td_X_trapezoid.h
--- 5.0.3-1/include/CGAL/Arr_point_location/Td_X_trapezoid.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_point_location/Td_X_trapezoid.h	2020-09-08 12:14:36.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Td_X_trapezoid.h $
-// $Id: Td_X_trapezoid.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Td_X_trapezoid.h $
+// $Id: Td_X_trapezoid.h 319383c 2020-05-20T09:47:58+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_point_location/Trapezoidal_decomposition_2.h 5.1-1/include/CGAL/Arr_point_location/Trapezoidal_decomposition_2.h
--- 5.0.3-1/include/CGAL/Arr_point_location/Trapezoidal_decomposition_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_point_location/Trapezoidal_decomposition_2.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Trapezoidal_decomposition_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Trapezoidal_decomposition_2.h $
 // $Id: Trapezoidal_decomposition_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_point_location/Trapezoidal_decomposition_2_impl.h 5.1-1/include/CGAL/Arr_point_location/Trapezoidal_decomposition_2_impl.h
--- 5.0.3-1/include/CGAL/Arr_point_location/Trapezoidal_decomposition_2_impl.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_point_location/Trapezoidal_decomposition_2_impl.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Trapezoidal_decomposition_2_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Trapezoidal_decomposition_2_impl.h $
 // $Id: Trapezoidal_decomposition_2_impl.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_point_location/Trapezoidal_decomposition_2_iostream.h 5.1-1/include/CGAL/Arr_point_location/Trapezoidal_decomposition_2_iostream.h
--- 5.0.3-1/include/CGAL/Arr_point_location/Trapezoidal_decomposition_2_iostream.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_point_location/Trapezoidal_decomposition_2_iostream.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Trapezoidal_decomposition_2_iostream.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Trapezoidal_decomposition_2_iostream.h $
 // $Id: Trapezoidal_decomposition_2_iostream.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_point_location/Trapezoidal_decomposition_2_misc.h 5.1-1/include/CGAL/Arr_point_location/Trapezoidal_decomposition_2_misc.h
--- 5.0.3-1/include/CGAL/Arr_point_location/Trapezoidal_decomposition_2_misc.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_point_location/Trapezoidal_decomposition_2_misc.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Trapezoidal_decomposition_2_misc.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Trapezoidal_decomposition_2_misc.h $
 // $Id: Trapezoidal_decomposition_2_misc.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_point_location_result.h 5.1-1/include/CGAL/Arr_point_location_result.h
--- 5.0.3-1/include/CGAL/Arr_point_location_result.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_point_location_result.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_point_location_result.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_point_location_result.h $
 // $Id: Arr_point_location_result.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_polycurve_basic_traits_2.h 5.1-1/include/CGAL/Arr_polycurve_basic_traits_2.h
--- 5.0.3-1/include/CGAL/Arr_polycurve_basic_traits_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_polycurve_basic_traits_2.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_polycurve_basic_traits_2.h $
-// $Id: Arr_polycurve_basic_traits_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_polycurve_basic_traits_2.h $
+// $Id: Arr_polycurve_basic_traits_2.h 180c061 2020-03-26T19:10:41+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Efi Fogel <efif@post.tau.ac.il>
@@ -2521,11 +2521,8 @@ protected:
   template <typename Comparer>
   class Compare_points {
   private:
-    typedef Arr_polycurve_basic_traits_2<Subcurve_traits_2>
-      Polycurve_basic_traits_2;
-
     /*! The polycurve traits (in case it has state). */
-    const Polycurve_basic_traits_2 m_poly_traits;
+    const Subcurve_traits_2& m_subcurve_traits;
 
     const Point_2& m_point;
 
@@ -2533,9 +2530,9 @@ protected:
 
   public:
     // Constructor
-    Compare_points(const Polycurve_basic_traits_2& traits, Comparer compare,
+    Compare_points(const Subcurve_traits_2& traits, Comparer compare,
                    const Point_2& p) :
-      m_poly_traits(traits),
+      m_subcurve_traits(traits),
       m_point(p),
       m_compare(compare)
     {}
@@ -2544,10 +2541,9 @@ protected:
     Comparison_result operator()(const X_monotone_subcurve_2& xs,
                                  Arr_curve_end ce)
     {
-      const Subcurve_traits_2* geom_traits = m_poly_traits.subcurve_traits_2();
       const Point_2& p = (ce == ARR_MAX_END) ?
-        geom_traits->construct_max_vertex_2_object()(xs) :
-        geom_traits->construct_min_vertex_2_object()(xs);
+        m_subcurve_traits.construct_max_vertex_2_object()(xs) :
+        m_subcurve_traits.construct_min_vertex_2_object()(xs);
       return m_compare(p, m_point);
     }
   };
@@ -2696,12 +2692,12 @@ protected:
       Comparison_result res = compare_x(min_vertex(xcv[0]), q);
       if (res != EQUAL) return INVALID_INDEX;
 
-      Compare_points<Compare_xy_2> compare(geom_traits,
+      Compare_points<Compare_xy_2> compare(*geom_traits,
                                            compare_xy_2_object(), q);
       return locate_gen(xcv, compare);
     }
 
-    Compare_points<Compare_x_2> compare(geom_traits, compare_x_2_object(), q);
+    Compare_points<Compare_x_2> compare(*geom_traits, compare_x_2_object(), q);
     return locate_gen(xcv, compare);
   }
 
diff -pruN 5.0.3-1/include/CGAL/Arr_polycurve_traits_2.h 5.1-1/include/CGAL/Arr_polycurve_traits_2.h
--- 5.0.3-1/include/CGAL/Arr_polycurve_traits_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_polycurve_traits_2.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_polycurve_traits_2.h $
-// $Id: Arr_polycurve_traits_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_polycurve_traits_2.h $
+// $Id: Arr_polycurve_traits_2.h 6eefda4 2020-04-06T15:34:28+03:00 Efi Fogel
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Efi Fogel <efif@post.tau.ac.il>
@@ -25,6 +25,8 @@
  */
 
 #include <iterator>
+
+#include <boost/variant.hpp>
 #include <boost/type_traits/is_same.hpp>
 #include <boost/utility/enable_if.hpp>
 
@@ -193,13 +195,12 @@ public:
       // If the polycurve is empty, return.
       if (cv.number_of_subcurves() == 0) return oi;
 
-      Construct_x_monotone_curve_2 ctr_x_curve =
-        m_poly_traits.construct_x_monotone_curve_2_object();
+      auto ctr_x_curve = m_poly_traits.construct_x_monotone_curve_2_object();
 
-      typename Subcurve_traits_2::Make_x_monotone_2 make_seg_x_monotone =
+      auto make_seg_x_monotone =
         m_poly_traits.subcurve_traits_2()->make_x_monotone_2_object();
 
-      typename Subcurve_traits_2::Compare_endpoints_xy_2 cmp_seg_endpts =
+      auto cmp_seg_endpts =
         m_poly_traits.subcurve_traits_2()->compare_endpoints_xy_2_object();
 
 #ifdef CGAL_ALWAYS_LEFT_TO_RIGHT
@@ -238,12 +239,11 @@ public:
         (
          // To be used in order to verify continuity and well-orientedness
          // of the input curve cv.
-         typename Subcurve_traits_2::Construct_min_vertex_2 min_seg_v =
+         auto min_seg_v =
            m_poly_traits.subcurve_traits_2()->construct_min_vertex_2_object();
-         typename Subcurve_traits_2::Construct_max_vertex_2 max_seg_v =
+         auto max_seg_v =
            m_poly_traits.subcurve_traits_2()->construct_max_vertex_2_object();
-         typename Subcurve_traits_2::Equal_2 equal =
-           m_poly_traits.subcurve_traits_2()->equal_2_object();
+         auto equal = m_poly_traits.subcurve_traits_2()->equal_2_object();
          Point_2 last_target = (cmp_seg_endpts(x_seg) == SMALLER) ?
            max_seg_v(x_seg) : min_seg_v(x_seg);
          Point_2 next_src;
@@ -329,19 +329,18 @@ public:
       // If the polycurve is empty, return.
       if (cv.number_of_subcurves() == 0) return oi;
 
-      Construct_x_monotone_curve_2 ctr_x_curve =
-        m_poly_traits.construct_x_monotone_curve_2_object();
+      auto ctr_x_curve = m_poly_traits.construct_x_monotone_curve_2_object();
 
-      typename Subcurve_traits_2::Make_x_monotone_2 make_seg_x_monotone =
+      auto make_seg_x_monotone =
         m_poly_traits.subcurve_traits_2()->make_x_monotone_2_object();
 
-      typename Subcurve_traits_2::Compare_endpoints_xy_2 cmp_seg_endpts =
+      auto cmp_seg_endpts =
         m_poly_traits.subcurve_traits_2()->compare_endpoints_xy_2_object();
 
-      typename Subcurve_traits_2::Parameter_space_in_x_2 ps_x =
-           m_poly_traits.subcurve_traits_2()->parameter_space_in_x_2_object();
-      typename Subcurve_traits_2::Parameter_space_in_y_2 ps_y =
-           m_poly_traits.subcurve_traits_2()->parameter_space_in_y_2_object();
+      auto ps_x =
+        m_poly_traits.subcurve_traits_2()->parameter_space_in_x_2_object();
+      auto ps_y =
+        m_poly_traits.subcurve_traits_2()->parameter_space_in_y_2_object();
 
 #ifdef CGAL_ALWAYS_LEFT_TO_RIGHT
       typename Subcurve_traits_2::Construct_opposite_2 ctr_seg_opposite =
@@ -379,12 +378,11 @@ public:
         (
          // To be used in order to verify continuity and well-orientedness
          // of the input curve cv.
-         typename Subcurve_traits_2::Construct_min_vertex_2 min_seg_v =
+         auto min_seg_v =
            m_poly_traits.subcurve_traits_2()->construct_min_vertex_2_object();
-         typename Subcurve_traits_2::Construct_max_vertex_2 max_seg_v =
+         auto max_seg_v =
            m_poly_traits.subcurve_traits_2()->construct_max_vertex_2_object();
-         typename Subcurve_traits_2::Equal_2 equal =
-           m_poly_traits.subcurve_traits_2()->equal_2_object();
+         auto equal = m_poly_traits.subcurve_traits_2()->equal_2_object();
          Point_2 last_target = (cmp_seg_endpts(x_seg) == SMALLER) ?
            max_seg_v(x_seg) : min_seg_v(x_seg);
          Point_2 next_src;
@@ -498,9 +496,7 @@ public:
 
   public:
     /*! Constructor. */
-    Push_back_2(const Polycurve_traits_2& traits) :
-      Base::Push_back_2(traits)
-    {}
+    Push_back_2(const Polycurve_traits_2& traits) : Base::Push_back_2(traits) {}
 
     // Normally, the moment the compiler finds a name, it stops looking. In
     // other words, the compiler first finds the operator() in the current
@@ -584,20 +580,16 @@ public:
                     X_monotone_curve_2& xcv1, X_monotone_curve_2& xcv2) const
     {
       const Subcurve_traits_2* geom_traits = m_poly_traits.subcurve_traits_2();
-      typename Subcurve_traits_2::Construct_min_vertex_2 min_vertex =
-        geom_traits->construct_min_vertex_2_object();
-      typename Subcurve_traits_2::Construct_max_vertex_2 max_vertex =
-        geom_traits->construct_max_vertex_2_object();
-      typename Subcurve_traits_2::Equal_2 equal =
-        geom_traits->equal_2_object();
-      typename Subcurve_traits_2::Compare_endpoints_xy_2 cmp_seg_endpts =
-        geom_traits->compare_endpoints_xy_2_object();
+      auto min_vertex = geom_traits->construct_min_vertex_2_object();
+      auto max_vertex = geom_traits->construct_max_vertex_2_object();
+      auto equal = geom_traits->equal_2_object();
+      auto cmp_seg_endpts = geom_traits->compare_endpoints_xy_2_object();
 
       // Make sure the split point is not one of the curve endpoints.
-      CGAL_precondition((!equal(m_poly_traits.
-                                construct_min_vertex_2_object()(xcv), p)));
-      CGAL_precondition((!equal(m_poly_traits.
-                                construct_max_vertex_2_object()(xcv), p)));
+      CGAL_precondition((! equal(m_poly_traits.
+                                 construct_min_vertex_2_object()(xcv), p)));
+      CGAL_precondition((! equal(m_poly_traits.
+                                 construct_max_vertex_2_object()(xcv), p)));
 
       CGAL_precondition_msg(xcv.number_of_subcurves() > 0,
                             "Cannot split a polycurve of length zero.");
@@ -709,22 +701,20 @@ public:
                const X_monotone_curve_2& cv2,
                OutputIterator oi) const
     {
-      const Subcurve_traits_2* geom_traits = m_poly_traits.subcurve_traits_2();
-      Compare_y_at_x_2 cmp_y_at_x = m_poly_traits.compare_y_at_x_2_object();
-      typename Subcurve_traits_2::Equal_2 equal =
-        geom_traits->equal_2_object();
-      typename Subcurve_traits_2::Construct_min_vertex_2 min_vertex =
-        geom_traits->construct_min_vertex_2_object();
-      typename Subcurve_traits_2::Construct_max_vertex_2 max_vertex =
-        geom_traits->construct_max_vertex_2_object();
-      typename Subcurve_traits_2::Intersect_2 intersect =
-        geom_traits->intersect_2_object();
-      typename Subcurve_traits_2::Compare_endpoints_xy_2 cmp_seg_endpts =
-        geom_traits->compare_endpoints_xy_2_object();
-      typename Subcurve_traits_2::Construct_opposite_2 construct_opposite =
-        geom_traits->construct_opposite_2_object();
+      typedef std::pair<Point_2, Multiplicity>        Intersection_point;
+      typedef boost::variant<Intersection_point, X_monotone_subcurve_2>
+                                                      Intersection_base_result;
+      typedef boost::variant<Intersection_point, X_monotone_curve_2>
+                                                      Intersection_result;
 
-      typedef std::pair<Point_2,Multiplicity> Point_2_pair;
+      const Subcurve_traits_2* geom_traits = m_poly_traits.subcurve_traits_2();
+      auto cmp_y_at_x = m_poly_traits.compare_y_at_x_2_object();
+      auto equal = geom_traits->equal_2_object();
+      auto min_vertex = geom_traits->construct_min_vertex_2_object();
+      auto max_vertex = geom_traits->construct_max_vertex_2_object();
+      auto intersect = geom_traits->intersect_2_object();
+      auto cmp_seg_endpts = geom_traits->compare_endpoints_xy_2_object();
+      auto construct_opposite = geom_traits->construct_opposite_2_object();
 
       Comparison_result dir1 = cmp_seg_endpts(cv1[0]);
       Comparison_result dir2 = cmp_seg_endpts(cv2[0]);
@@ -737,7 +727,7 @@ public:
 
       X_monotone_curve_2 ocv;           // Used to represent overlaps.
 
-      Compare_xy_2 compare_xy = m_poly_traits.compare_xy_2_object();
+      auto compare_xy = m_poly_traits.compare_xy_2_object();
       Comparison_result left_res =
         compare_xy(cv1[i1], ARR_MIN_END, cv2[i2], ARR_MIN_END);
 
@@ -754,12 +744,13 @@ public:
               ((dir1 == LARGER) && (i1 == 0))){
             // cv1's right endpoint equals cv2's left endpoint
             // Thus we can return this single(!) intersection point
-            std::pair<Point_2, Multiplicity>  p(max_vertex(cv1[i1]), 0);
-            *oi++ = make_object(p);
+            Intersection_point p(max_vertex(cv1[i1]), 0);
+            *oi++ = Intersection_result(p);
             return oi;
           }
           dir1 == SMALLER ?
-            ++i1 : (i1 != 0) ? --i1 : (std::size_t) Polycurve_traits_2::INVALID_INDEX;
+            ++i1 :
+            (i1 != 0) ? --i1 : (std::size_t) Polycurve_traits_2::INVALID_INDEX;
           left_res = EQUAL;
         }
       }
@@ -776,13 +767,14 @@ public:
               ((dir2 == LARGER) && (i2 == 0))){
             // cv2's right endpoint equals cv1's left endpoint
             // Thus we can return this single(!) intersection point
-            std::pair<Point_2, Multiplicity>  p(max_vertex(cv2[i2]), 0);
-            *oi++ = make_object(p);
+            Intersection_point p(max_vertex(cv2[i2]), 0);
+            *oi++ = Intersection_result(p);
             return oi;
           }
 
           dir2 == SMALLER ?
-            ++i2 : (i2 != 0) ? --i2 : (std::size_t) Polycurve_traits_2::INVALID_INDEX;
+            ++i2 :
+            (i2 != 0) ? --i2 : (std::size_t) Polycurve_traits_2::INVALID_INDEX;
           left_res = EQUAL;
         }
       }
@@ -823,48 +815,64 @@ public:
 
         right_overlap = false;
 
-        if (!right_coincides && !left_coincides) {
+        //! EF: the following code is abit suspicious. It may erroneously
+        //      assume that the subcurves cannot overlap more than once.
+        if (! right_coincides && ! left_coincides) {
           // Non of the endpoints of the current subcurve of one polycurve
           // coincides with the curent subcurve of the other polycurve:
           // Output the intersection if exists.
-          oi = intersect(cv1[i1], cv2[i2], oi);
+          std::vector<Intersection_base_result> xections;
+          intersect(cv1[i1], cv2[i2], std::back_inserter(xections));
+          for (const auto& xection : xections) {
+            const X_monotone_subcurve_2* subcv_p =
+              boost::get<X_monotone_subcurve_2>(&xection);
+            if (subcv_p != nullptr) {
+              ocv.push_back(*subcv_p);
+              *oi++ = Intersection_result(ocv);
+              ocv.clear();
+              continue;
+            }
+
+            const Intersection_point* p_p =
+              boost::get<Intersection_point>(&xection);
+            if (p_p != nullptr) *oi++ = Intersection_result(*p_p);
+          }
         }
         else if (right_coincides && left_coincides) {
           // An overlap exists between the current subcurves of the
           // polycurves: Output the overlapping subcurve.
           right_overlap = true;
 
-          std::vector<CGAL::Object> int_seg;
-          intersect(cv1[i1], cv2[i2], std::back_inserter(int_seg));
+          std::vector<Intersection_base_result> sub_xections;
+          intersect(cv1[i1], cv2[i2], std::back_inserter(sub_xections));
 
-          for (size_t i = 0; i < int_seg.size(); ++i) {
+          for (const auto& item : sub_xections) {
             const X_monotone_subcurve_2* x_seg =
-              CGAL::object_cast<X_monotone_subcurve_2> (&(int_seg[i]));
+              boost::get<X_monotone_subcurve_2>(&item);
             if (x_seg != nullptr) {
               X_monotone_subcurve_2 seg = *x_seg;
 
               // If for some reason the subcurve intersection
               // results in left oriented curve.
-              if ( cmp_seg_endpts(seg) == LARGER)
-                seg = construct_opposite(seg);
+              if (cmp_seg_endpts(seg) == LARGER) seg = construct_opposite(seg);
               ocv.push_back(seg);
             }
 
-            const Point_2_pair* p_ptr =
-              CGAL::object_cast<Point_2_pair>(&(int_seg[i]));
+            const Intersection_point* p_ptr =
+              boost::get<Intersection_point>(&item);
             if (p_ptr != nullptr) {
               // Any point that is not equal to the max_vertex of the
               // subcurve should be inserted into oi.
               // The max_vertex of the current subcurve (if intersecting)
               // will be taken care of as the min_vertex of in the next
               // iteration.
-              if (!equal(p_ptr->first, max_vertex(cv1[i1])))
-                *oi++ = make_object(*p_ptr);
+              if (! equal(p_ptr->first, max_vertex(cv1[i1])))
+                *oi++ = Intersection_result(*p_ptr);
             }
           }
         }
 
-        else if (left_coincides && !right_coincides) {
+        else if (left_coincides && ! right_coincides) {
           // std::cout << "Left is coinciding but right is not." << std::endl;
           // The left point of the current subcurve of one polycurve
           // coincides with the current subcurve of the other polycurve.
@@ -872,7 +880,7 @@ public:
             // An overlap occurred at the previous iteration:
             // Output the overlapping polycurve.
             CGAL_assertion(ocv.number_of_subcurves() > 0);
-            *oi++ = make_object(ocv);
+            *oi++ = Intersection_result(ocv);
             ocv.clear();
           }
           else {
@@ -884,12 +892,12 @@ public:
             // polycurves is not defined at this point, so we give
             // it multiplicity 0.
             if (left_res == SMALLER) {
-              std::pair<Point_2, Multiplicity> p(min_vertex(cv2[i2]), 0);
-              *oi++ = make_object(p);
+              Intersection_point p(min_vertex(cv2[i2]), 0);
+              *oi++ = Intersection_result(p);
             }
             else {
-              std::pair<Point_2, Multiplicity> p(min_vertex(cv1[i1]), 0);
-              *oi++ = make_object(p);
+              Intersection_point p(min_vertex(cv1[i1]), 0);
+              *oi++ = Intersection_result(p);
             }
           }
         }
@@ -919,7 +927,7 @@ public:
 
         // Output the remaining overlapping polycurve, if necessary.
       if (ocv.number_of_subcurves() > 0) {
-        *oi++ = make_object(ocv);
+        *oi++ = Intersection_result(ocv);
       }
       else if (right_coincides) {
         typedef std::pair<Point_2,Multiplicity> return_point;
@@ -930,7 +938,7 @@ public:
             (i1 != Polycurve_traits_2::INVALID_INDEX) ?
             return_point(max_vertex(cv1[i1+1]), 0) :
             return_point(max_vertex(cv1[0]), 0);
-          *oi++ = make_object(ip);
+          *oi++ = Intersection_result(ip);
         }
         else if (right_res == LARGER) {
           ip = (dir2 == SMALLER) ?
@@ -938,7 +946,7 @@ public:
             (i2 != Polycurve_traits_2::INVALID_INDEX) ?
             return_point(max_vertex(cv2[i2+1]), 0) :
             return_point(max_vertex(cv2[0]), 0);
-          *oi++ = make_object(ip);
+          *oi++ = Intersection_result(ip);
         }
         else if (((i1 > 0) && (dir1 == SMALLER)) ||
                  ((i1 < n1) && (dir1 != SMALLER)) ||
@@ -950,7 +958,7 @@ public:
             (i1 != Polycurve_traits_2::INVALID_INDEX) ?
             return_point(max_vertex(cv1[i1+1]), 0) :
             return_point(max_vertex(cv1[0]), 0);
-          *oi++ = make_object(ip);
+          *oi++ = Intersection_result(ip);
         }
         else {
           CGAL_assertion_msg((dir2 == SMALLER && i2 > 0) ||
@@ -965,7 +973,7 @@ public:
             (i2 != Polycurve_traits_2::INVALID_INDEX) ?
             return_point(max_vertex(cv2[i2+1]), 0) :
             return_point(max_vertex(cv2[0]), 0);
-          *oi++ = make_object(ip);
+          *oi++ = Intersection_result(ip);
         }
       }
 
diff -pruN 5.0.3-1/include/CGAL/Arr_polyline_traits_2.h 5.1-1/include/CGAL/Arr_polyline_traits_2.h
--- 5.0.3-1/include/CGAL/Arr_polyline_traits_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_polyline_traits_2.h	2020-09-08 12:14:37.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_polyline_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_polyline_traits_2.h $
 // $Id: Arr_polyline_traits_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_rat_arc/Algebraic_point_2.h 5.1-1/include/CGAL/Arr_rat_arc/Algebraic_point_2.h
--- 5.0.3-1/include/CGAL/Arr_rat_arc/Algebraic_point_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_rat_arc/Algebraic_point_2.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_rat_arc/Algebraic_point_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_rat_arc/Algebraic_point_2.h $
 // $Id: Algebraic_point_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_rat_arc/Base_rational_arc_ds_1.h 5.1-1/include/CGAL/Arr_rat_arc/Base_rational_arc_ds_1.h
--- 5.0.3-1/include/CGAL/Arr_rat_arc/Base_rational_arc_ds_1.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_rat_arc/Base_rational_arc_ds_1.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_rat_arc/Base_rational_arc_ds_1.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_rat_arc/Base_rational_arc_ds_1.h $
 // $Id: Base_rational_arc_ds_1.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_rat_arc/Cache.h 5.1-1/include/CGAL/Arr_rat_arc/Cache.h
--- 5.0.3-1/include/CGAL/Arr_rat_arc/Cache.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_rat_arc/Cache.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_rat_arc/Cache.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_rat_arc/Cache.h $
 // $Id: Cache.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_rat_arc/Rational_arc_d_1.h 5.1-1/include/CGAL/Arr_rat_arc/Rational_arc_d_1.h
--- 5.0.3-1/include/CGAL/Arr_rat_arc/Rational_arc_d_1.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_rat_arc/Rational_arc_d_1.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_rat_arc/Rational_arc_d_1.h $
-// $Id: Rational_arc_d_1.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_rat_arc/Rational_arc_d_1.h $
+// $Id: Rational_arc_d_1.h 58276ed 2020-03-31T18:34:28+03:00 Efi Fogel
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Oren Salzman <orenzalz@post.tau.ac.il >
@@ -1848,8 +1848,8 @@ public:
 
   typedef typename Base::Cache                          Cache;
 
-  typedef std::pair<Algebraic_point_2, Multiplicity>    Intersection_point_2;
-  //typedef std::pair<Algebraic_point_2, unsigned int>  Intersection_point_2;
+  typedef std::pair<Algebraic_point_2, Multiplicity>    Intersection_point;
+  //typedef std::pair<Algebraic_point_2, unsigned int>  Intersection_point;
 
 
   /// \name Constrcution methods.
@@ -2029,8 +2029,7 @@ public:
   /// \name Constructions of points and curves.
   //@{
 
-  /*!
-   * Compute the intersections with the given arc.
+  /*! Compute the intersections with the given arc.
    * \param arc The given intersecting arc.
    * \param oi The output iterator.
    * \return The past-the-end iterator.
@@ -2039,33 +2038,31 @@ public:
   OutputIterator intersect(const Self& arc, OutputIterator oi,
                            const Cache& cache) const
   {
+    typedef boost::variant<Intersection_point, Self>    Intersection_result;
+
     CGAL_precondition(this->is_valid() && this->is_continuous());
     CGAL_precondition(arc.is_valid() && arc.is_continuous());
 
-    if (this->equals(arc))
-    {
-      Self      overlap_arc(*this);
-      *oi++ = make_object(overlap_arc);
-      return (oi);
+    if (this->equals(arc)) {
+      Self overlap_arc(*this);
+      *oi++ = Intersection_result(overlap_arc);
+      return oi;
     }
 
-    if (this->_has_same_base(arc))
-    {
+    if (this->_has_same_base(arc)) {
       // Get the left and right endpoints of (*this) and their information
       // bits.
-      const Algebraic_point_2&   left1 = (this->is_directed_right() ?
-                                          this->_ps : this->_pt);
-      const Algebraic_point_2&   right1 = (this->is_directed_right() ?
-                                           this->_pt : this->_ps);
-      int      info_left1, info_right1;
+      const Algebraic_point_2& left1 =
+        (this->is_directed_right() ? this->_ps : this->_pt);
+      const Algebraic_point_2& right1 =
+        (this->is_directed_right() ? this->_pt : this->_ps);
+      int info_left1, info_right1;
 
-      if (this->is_directed_right())
-      {
+      if (this->is_directed_right()) {
         info_left1 = (this->_info & this->SRC_INFO_BITS);
         info_right1 = ((this->_info & this->TRG_INFO_BITS) >> 4);
       }
-      else
-      {
+      else {
         info_right1 = (this->_info & this->SRC_INFO_BITS);
         info_left1 = ((this->_info & this->TRG_INFO_BITS) >> 4);
       }
@@ -2076,110 +2073,93 @@ public:
         (arc.is_directed_right() ? arc._ps : arc._pt);
       const Algebraic_point_2& right2 =
         (arc.is_directed_right() ? arc._pt : arc._ps);
-      int      info_left2, info_right2;
+      int info_left2, info_right2;
 
-      if (arc.is_directed_right())
-      {
+      if (arc.is_directed_right()) {
         info_left2 = (arc._info & this->SRC_INFO_BITS);
         info_right2 = ((arc._info & this->TRG_INFO_BITS) >> 4);
       }
-      else
-      {
+      else {
         info_right2 = (arc._info & this->SRC_INFO_BITS);
         info_left2 = ((arc._info & this->TRG_INFO_BITS) >> 4);
       }
 
       // Locate the left curve-end with larger x-coordinate.
-      bool                 at_minus_infinity = false;
-      Arr_parameter_space  inf_l1 = this->left_parameter_space_in_x();
-      Arr_parameter_space  inf_l2 = arc.left_parameter_space_in_x();
-      Algebraic_point_2    p_left;
-      int                  info_left;
+      bool at_minus_infinity = false;
+      Arr_parameter_space inf_l1 = this->left_parameter_space_in_x();
+      Arr_parameter_space inf_l2 = arc.left_parameter_space_in_x();
+      Algebraic_point_2 p_left;
+      int info_left;
 
-      if (inf_l1 == ARR_INTERIOR && inf_l2 == ARR_INTERIOR)
-      {
+      if (inf_l1 == ARR_INTERIOR && inf_l2 == ARR_INTERIOR) {
         // Let p_left be the rightmost of the two left endpoints.
-        if (left1.x() > left2.x())
-        {
+        if (left1.x() > left2.x()) {
           p_left = left1;
           info_left = info_left1;
         }
-        else
-        {
+        else {
           p_left = left2;
           info_left = info_left2;
         }
       }
-      else if (inf_l1 == ARR_INTERIOR)
-      {
+      else if (inf_l1 == ARR_INTERIOR) {
         // Let p_left be the left endpoint of (*this).
         p_left = left1;
         info_left = info_left1;
       }
-      else if (inf_l2 == ARR_INTERIOR)
-      {
+      else if (inf_l2 == ARR_INTERIOR) {
         // Let p_left be the left endpoint of the other arc.
         p_left = left2;
         info_left = info_left2;
       }
-      else
-      {
+      else {
         // Both arcs are defined at x = -oo.
         at_minus_infinity = true;
         info_left = info_left1;
       }
 
       // Locate the right curve-end with smaller x-coordinate.
-      bool                 at_plus_infinity = false;
-      Arr_parameter_space  inf_r1 = this->right_parameter_space_in_x();
-      Arr_parameter_space  inf_r2 = arc.right_parameter_space_in_x();
-      Algebraic_point_2    p_right;
-      int                  info_right;
+      bool at_plus_infinity = false;
+      Arr_parameter_space inf_r1 = this->right_parameter_space_in_x();
+      Arr_parameter_space inf_r2 = arc.right_parameter_space_in_x();
+      Algebraic_point_2 p_right;
+      int info_right;
 
-      if (inf_r1 == ARR_INTERIOR && inf_r2 == ARR_INTERIOR)
-      {
+      if (inf_r1 == ARR_INTERIOR && inf_r2 == ARR_INTERIOR) {
         // Let p_right be the rightmost of the two right endpoints.
-        if (right1.x() < right2.x())
-        {
+        if (right1.x() < right2.x()) {
           p_right = right1;
           info_right = info_right1;
         }
-        else
-        {
+        else {
           p_right = right2;
           info_right = info_right2;
         }
       }
-      else if (inf_r1 == ARR_INTERIOR)
-      {
+      else if (inf_r1 == ARR_INTERIOR) {
         // Let p_right be the right endpoint of (*this).
         p_right = right1;
         info_right = info_right1;
       }
-      else if (inf_r2 == ARR_INTERIOR)
-      {
+      else if (inf_r2 == ARR_INTERIOR) {
         // Let p_right be the right endpoint of the other arc.
         p_right = right2;
         info_right = info_right2;
       }
-      else
-      {
+      else {
         // Both arcs are defined at x = +oo.
         at_plus_infinity = true;
         info_right = info_right2;
       }
 
       // Check the case of two bounded (in x) ends.
-      if (! at_minus_infinity && ! at_plus_infinity)
-      {
+      if (! at_minus_infinity && ! at_plus_infinity) {
         Comparison_result res = CGAL::compare(p_left.x(), p_right.x());
-        if (res == LARGER)
-        {
-          // The x-range of the overlap is empty, so there is no overlap.
-          return (oi);
-        }
-        else if (res == EQUAL)
-        {
+
+        // The x-range of the overlap is empty, so there is no overlap.
+        if (res == LARGER) return oi;
+
+        if (res == EQUAL) {
           // We have a single overlapping point. Just make sure this point
           // is not at y = -/+ oo.
           if (info_left &&
@@ -2187,19 +2167,18 @@ public:
               info_right &&
               (this->SRC_AT_Y_MINUS_INFTY | this->SRC_AT_Y_PLUS_INFTY) == 0)
           {
-            Intersection_point_2  ip(p_left, 0);
-
-            *oi++ = make_object(ip);
+            Intersection_point ip(p_left, 0);
+            *oi++ = Intersection_result(ip);
           }
 
-          return (oi);
+          return oi;
         }
       }
 
       // Create the overlapping portion of the rational arc by properly setting
       // the source (left) and target (right) endpoints and their information
       // bits.
-      Self      overlap_arc(*this);
+      Self overlap_arc(*this);
 
       overlap_arc._ps = p_left;
       overlap_arc._pt = p_right;
@@ -2208,8 +2187,8 @@ public:
                            this->IS_DIRECTED_RIGHT | this->IS_CONTINUOUS |
                            this->IS_VALID);
 
-      *oi++ = make_object(overlap_arc);
-      return (oi);
+      *oi++ = Intersection_result(overlap_arc);
+      return oi;
     }
 
     // We wish to find the intersection points between:
@@ -2237,15 +2216,14 @@ public:
       if (this->_is_in_true_x_range(*x_iter) && arc._is_in_true_x_range(*x_iter))
       {
         // Compute the intersection point and obtain its multiplicity.
-        Algebraic_point_2   p(this->_f, *x_iter);
+        Algebraic_point_2 p(this->_f, *x_iter);
         // Output the intersection point:
-        Intersection_point_2  ip(p, *m_iter);
-
-        *oi++ = make_object(ip);
+        Intersection_point ip(p, *m_iter);
+        *oi++ = Intersection_result(ip);
       }
     }
 
-    return (oi);
+    return oi;
   }
 
   /*!
@@ -2255,7 +2233,8 @@ public:
    * \param c2 Output: The first resulting arc, lying to the right of p.
    * \pre p lies in the interior of the arc (not one of its endpoints).
    */
-  void split(const Algebraic_point_2& p, Self& c1, Self& c2, const Cache& CGAL_assertion_code(cache)) const
+  void split(const Algebraic_point_2& p, Self& c1, Self& c2,
+             const Cache& CGAL_assertion_code(cache)) const
   {
     CGAL_precondition(this->is_valid() && this->is_continuous());
 
@@ -2598,4 +2577,3 @@ public:
 }       //namespace CGAL {
 
 #endif //CGAL_RATIONAL_ARC_D_1_H
-
diff -pruN 5.0.3-1/include/CGAL/Arr_rat_arc/Rational_function_canonicalized_pair.h 5.1-1/include/CGAL/Arr_rat_arc/Rational_function_canonicalized_pair.h
--- 5.0.3-1/include/CGAL/Arr_rat_arc/Rational_function_canonicalized_pair.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_rat_arc/Rational_function_canonicalized_pair.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_rat_arc/Rational_function_canonicalized_pair.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_rat_arc/Rational_function_canonicalized_pair.h $
 // $Id: Rational_function_canonicalized_pair.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_rat_arc/Rational_function.h 5.1-1/include/CGAL/Arr_rat_arc/Rational_function.h
--- 5.0.3-1/include/CGAL/Arr_rat_arc/Rational_function.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_rat_arc/Rational_function.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_rat_arc/Rational_function.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_rat_arc/Rational_function.h $
 // $Id: Rational_function.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_rat_arc/Rational_function_ordered_pair.h 5.1-1/include/CGAL/Arr_rat_arc/Rational_function_ordered_pair.h
--- 5.0.3-1/include/CGAL/Arr_rat_arc/Rational_function_ordered_pair.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_rat_arc/Rational_function_ordered_pair.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_rat_arc/Rational_function_ordered_pair.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_rat_arc/Rational_function_ordered_pair.h $
 // $Id: Rational_function_ordered_pair.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_rat_arc/Rational_function_pair.h 5.1-1/include/CGAL/Arr_rat_arc/Rational_function_pair.h
--- 5.0.3-1/include/CGAL/Arr_rat_arc/Rational_function_pair.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_rat_arc/Rational_function_pair.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_rat_arc/Rational_function_pair.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_rat_arc/Rational_function_pair.h $
 // $Id: Rational_function_pair.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_rat_arc/Singleton.h 5.1-1/include/CGAL/Arr_rat_arc/Singleton.h
--- 5.0.3-1/include/CGAL/Arr_rat_arc/Singleton.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_rat_arc/Singleton.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_rat_arc/Singleton.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_rat_arc/Singleton.h $
 // $Id: Singleton.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_rational_function_traits_2.h 5.1-1/include/CGAL/Arr_rational_function_traits_2.h
--- 5.0.3-1/include/CGAL/Arr_rational_function_traits_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_rational_function_traits_2.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_rational_function_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_rational_function_traits_2.h $
 // $Id: Arr_rational_function_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_segment_traits_2.h 5.1-1/include/CGAL/Arr_segment_traits_2.h
--- 5.0.3-1/include/CGAL/Arr_segment_traits_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_segment_traits_2.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,13 +3,13 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_segment_traits_2.h $
-// $Id: Arr_segment_traits_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_segment_traits_2.h $
+// $Id: Arr_segment_traits_2.h 58276ed 2020-03-31T18:34:28+03:00 Efi Fogel
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
-// Author(s)     : Ron Wein          <wein@post.tau.ac.il>
-//                 Efi Fogel         <efif@post.tau.ac.il>
-//                 Waqar Khan        <wkhan@mpi-inf.mpg.de>
+// Author(s): Ron Wein          <wein@post.tau.ac.il>
+//            Efi Fogel         <efif@post.tau.ac.il>
+//            Waqar Khan        <wkhan@mpi-inf.mpg.de>
 
 #ifndef CGAL_ARR_SEGMENT_TRAITS_2_H
 #define CGAL_ARR_SEGMENT_TRAITS_2_H
@@ -22,21 +22,23 @@
  * The segment traits-class for the arrangement package.
  */
 
+#include <fstream>
+
+#include <boost/variant.hpp>
+
 #include <CGAL/tags.h>
 #include <CGAL/intersections.h>
 #include <CGAL/Arr_tags.h>
 #include <CGAL/Arr_geometry_traits/Segment_assertions.h>
 #include <CGAL/Exact_predicates_exact_constructions_kernel.h>
 #include <CGAL/Arr_enums.h>
-#include <fstream>
 
 namespace CGAL {
 
-template <class Kernel_ = Exact_predicates_exact_constructions_kernel>
+template <typename Kernel_ = Exact_predicates_exact_constructions_kernel>
 class Arr_segment_2;
 
-/*!
- * \class A traits class for maintaining an arrangement of segments, avoiding
+/*! \class A traits class for maintaining an arrangement of segments, avoiding
  * cascading of computations as much as possible.
  *
  * The class is derived from the parameterized kernel to extend the traits
@@ -79,17 +81,17 @@ public:
     typedef typename Kernel::Point_2               Point_2;
 
   protected:
-    Line_2    l;                // The line that supports the segment.
-    Point_2   ps;               // The source point of the segment.
-    Point_2   pt;               // The target point of the segment.
-    bool      is_pt_max;        // Is the target (lexicographically) larger
-                                // than the source.
-    bool      is_vert;          // Is this a vertical segment.
-    bool      is_degen;         // Is the segment degenerate (a single point).
+    Line_2 m_l;               // The line that supports the segment.
+    Point_2 m_ps;             // The source point of the segment.
+    Point_2 m_pt;             // The target point of the segment.
+    bool m_is_pt_max;         // Is the target (lexicographically) larger
+                              // than the source.
+    bool m_is_vert;           // Is this a vertical segment.
+    bool m_is_degen;          // Is the segment degenerate (a single point).
 
   public:
     /*! Default constructor. */
-    _Segment_cached_2() : is_vert(false), is_degen(true) {}
+    _Segment_cached_2() : m_is_vert(false), m_is_degen(true) {}
 
     /*! Constructor from a segment.
      * \param seg The segment.
@@ -97,50 +99,46 @@ public:
      */
     _Segment_cached_2(const Segment_2& seg)
     {
-      Kernel   kernel;
+      Kernel kernel;
+      auto construct_vertex = kernel.construct_vertex_2_object();
 
-      typename Kernel_::Construct_vertex_2
-        construct_vertex = kernel.construct_vertex_2_object();
+      m_ps = construct_vertex(seg, 0);
+      m_pt = construct_vertex(seg, 1);
 
-      ps = construct_vertex(seg, 0);
-      pt = construct_vertex(seg, 1);
+      Comparison_result res = kernel.compare_xy_2_object()(m_ps, m_pt);
+      m_is_degen = (res == EQUAL);
+      m_is_pt_max = (res == SMALLER);
 
-      Comparison_result  res = kernel.compare_xy_2_object()(ps, pt);
-      is_degen = (res == EQUAL);
-      is_pt_max = (res == SMALLER);
-
-      CGAL_precondition_msg (! is_degen,
+      CGAL_precondition_msg (! m_is_degen,
                              "Cannot construct a degenerate segment.");
 
-      l = kernel.construct_line_2_object()(seg);
-      is_vert = kernel.is_vertical_2_object()(seg);
+      m_l = kernel.construct_line_2_object()(seg);
+      m_is_vert = kernel.is_vertical_2_object()(seg);
     }
 
-    /*!
-     * Construct a segment from two end-points.
+    /*! Construct a segment from two end-points.
      * \param source The source point.
      * \param target The target point.
      * \param The two points must not be equal.
      */
     _Segment_cached_2(const Point_2& source, const Point_2& target) :
-      ps(source),
-      pt(target)
+      m_ps(source),
+      m_pt(target)
     {
-      Kernel   kernel;
+      Kernel kernel;
 
-      Comparison_result  res = kernel.compare_xy_2_object()(ps, pt);
-      is_degen = (res == EQUAL);
-      is_pt_max = (res == SMALLER);
+      Comparison_result res = kernel.compare_xy_2_object()(m_ps, m_pt);
+      m_is_degen = (res == EQUAL);
+      m_is_pt_max = (res == SMALLER);
 
-      CGAL_precondition_msg(! is_degen,
+      CGAL_precondition_msg(! m_is_degen,
                             "Cannot construct a degenerate segment.");
 
-      l = kernel.construct_line_2_object()(source, target);
-      is_vert = kernel.is_vertical_2_object()(l);
+      m_l = kernel.construct_line_2_object()(source, target);
+      m_is_vert = kernel.is_vertical_2_object()(m_l);
     }
 
-    /*!
-     * Construct a segment from two end-points on a supporting line.
+    /*! Construct a segment from two end-points on a supporting line.
      * \param supp_line The supporting line.
      * \param source The source point.
      * \param target The target point.
@@ -148,60 +146,56 @@ public:
      */
     _Segment_cached_2(const Line_2& supp_line,
                       const Point_2& source, const Point_2& target) :
-      l(supp_line),
-      ps(source),
-      pt(target)
-    {
-      Kernel   kernel;
-
-      CGAL_precondition(
-        Segment_assertions::_assert_is_point_on(source, l,
-                                                Has_exact_division()) &&
-        Segment_assertions::_assert_is_point_on(target,l,
-                                                Has_exact_division())
-        );
-
-      is_vert = kernel.is_vertical_2_object()(l);
-
-      Comparison_result  res = kernel.compare_xy_2_object()(ps, pt);
-      is_degen = (res == EQUAL);
-      is_pt_max = (res == SMALLER);
+      m_l(supp_line),
+      m_ps(source),
+      m_pt(target)
+    {
+      Kernel kernel;
+
+      CGAL_precondition
+        (Segment_assertions::_assert_is_point_on(source, m_l,
+                                                 Has_exact_division()) &&
+         Segment_assertions::_assert_is_point_on(target, m_l,
+                                                 Has_exact_division()));
+
+      m_is_vert = kernel.is_vertical_2_object()(m_l);
+
+      Comparison_result res = kernel.compare_xy_2_object()(m_ps, m_pt);
+      m_is_degen = (res == EQUAL);
+      m_is_pt_max = (res == SMALLER);
 
-      CGAL_precondition_msg(! is_degen,
+      CGAL_precondition_msg(! m_is_degen,
                             "Cannot construct a degenerate segment.");
     }
 
-    /*!
-     * Assignment operator.
+    /*! Assignment operator.
      * \param seg the source segment to copy from
      * \pre The segment is not degenerate.
      */
-    const _Segment_cached_2& operator= (const Segment_2& seg)
+    const _Segment_cached_2& operator=(const Segment_2& seg)
     {
-      Kernel   kernel;
+      Kernel kernel;
+      auto construct_vertex = kernel.construct_vertex_2_object();
 
-      typename Kernel_::Construct_vertex_2
-        construct_vertex = kernel.construct_vertex_2_object();
+      m_ps = construct_vertex(seg, 0);
+      m_pt = construct_vertex(seg, 1);
 
-      ps = construct_vertex(seg, 0);
-      pt = construct_vertex(seg, 1);
+      Comparison_result res = kernel.compare_xy_2_object()(m_ps, m_pt);
+      m_is_degen = (res == EQUAL);
+      m_is_pt_max = (res == SMALLER);
 
-      Comparison_result res = kernel.compare_xy_2_object()(ps, pt);
-      is_degen = (res == EQUAL);
-      is_pt_max = (res == SMALLER);
-
-      CGAL_precondition_msg(! is_degen,
+      CGAL_precondition_msg(! m_is_degen,
                             "Cannot construct a degenerate segment.");
 
-      l = kernel.construct_line_2_object()(seg);
-      is_vert = kernel.is_vertical_2_object()(seg);
+      m_l = kernel.construct_line_2_object()(seg);
+      m_is_vert = kernel.is_vertical_2_object()(seg);
 
       return (*this);
     }
 
     /*! Obtain the (lexicographically) left endpoint.
      */
-    const Point_2& left() const { return (is_pt_max ? ps : pt); }
+    const Point_2& left() const { return (m_is_pt_max ? m_ps : m_pt); }
 
     /*! Set the (lexicographically) left endpoint.
      * \param p The point to set.
@@ -209,22 +203,19 @@ public:
      */
     void set_left(const Point_2& p)
     {
-      CGAL_precondition (! is_degen);
-      CGAL_precondition_code (
-        Kernel    kernel;
-      );
+      CGAL_precondition(! m_is_degen);
+      CGAL_precondition_code(Kernel kernel);
       CGAL_precondition
-        (Segment_assertions::_assert_is_point_on (p, l,
-                                                  Has_exact_division()) &&
-         kernel.compare_xy_2_object() (p, right()) == SMALLER);
+        (Segment_assertions::_assert_is_point_on(p, m_l, Has_exact_division()) &&
+         (kernel.compare_xy_2_object()(p, right()) == SMALLER));
 
-      if (is_pt_max) ps = p;
-      else pt = p;
+      if (m_is_pt_max) m_ps = p;
+      else m_pt = p;
     }
 
     /*! Obtain the (lexicographically) right endpoint.
      */
-    const Point_2& right() const { return (is_pt_max ? pt : ps); }
+    const Point_2& right() const { return (m_is_pt_max ? m_pt : m_ps); }
 
     /*! Set the (lexicographically) right endpoint.
      * \param p The point to set.
@@ -232,38 +223,35 @@ public:
      */
     void set_right(const Point_2& p)
     {
-      CGAL_precondition(! is_degen);
-      CGAL_precondition_code(
-        Kernel    kernel;
-      );
+      CGAL_precondition(! m_is_degen);
+      CGAL_precondition_code(Kernel kernel);
       CGAL_precondition
-        (Segment_assertions::_assert_is_point_on (p, l,
-                                                  Has_exact_division()) &&
-         kernel.compare_xy_2_object() (p, left()) == LARGER);
+        (Segment_assertions::_assert_is_point_on(p, m_l, Has_exact_division()) &&
+         (kernel.compare_xy_2_object()(p, left()) == LARGER));
 
-      if (is_pt_max) pt = p;
-      else ps = p;
+      if (m_is_pt_max) m_pt = p;
+      else m_ps = p;
     }
 
     /*! Obtain the supporting line.
      */
     const Line_2& line() const
     {
-      CGAL_precondition(! is_degen);
-      return (l);
+      CGAL_precondition(! m_is_degen);
+      return m_l;
     }
 
     /*! Determine whether the curve is vertical.
      */
     bool is_vertical() const
     {
-      CGAL_precondition(! is_degen);
-      return (is_vert);
+      CGAL_precondition(! m_is_degen);
+      return m_is_vert;
     }
 
     /*! Determine whether the curve is directed lexicographic from left to right
      */
-    bool is_directed_right() const { return (is_pt_max); }
+    bool is_directed_right() const { return (m_is_pt_max); }
 
     /*! Determine whether the given point is in the x-range of the segment.
      * \param p The query point.
@@ -271,12 +259,12 @@ public:
      */
     bool is_in_x_range(const Point_2& p) const
     {
-      Kernel                          kernel;
-      typename Kernel_::Compare_x_2   compare_x = kernel.compare_x_2_object();
-      const Comparison_result         res1 = compare_x(p, left());
+      Kernel kernel;
+      typename Kernel_::Compare_x_2 compare_x = kernel.compare_x_2_object();
+      const Comparison_result res1 = compare_x(p, left());
 
-      if (res1 == SMALLER) return (false);
-      else if (res1 == EQUAL) return (true);
+      if (res1 == SMALLER) return false;
+      else if (res1 == EQUAL) return true;
 
       const Comparison_result res2 = compare_x(p, right());
       return (res2 != LARGER);
@@ -288,14 +276,14 @@ public:
      */
     bool is_in_y_range(const Point_2& p) const
     {
-      Kernel                          kernel;
-      typename Kernel_::Compare_y_2   compare_y = kernel.compare_y_2_object();
-      const Comparison_result         res1 = compare_y (p, left());
+      Kernel kernel;
+      typename Kernel_::Compare_y_2 compare_y = kernel.compare_y_2_object();
+      const Comparison_result res1 = compare_y(p, left());
 
-      if (res1 == SMALLER) return (false);
-      else if (res1 == EQUAL) return (true);
+      if (res1 == SMALLER) return false;
+      else if (res1 == EQUAL) return true;
 
-      const Comparison_result res2 = compare_y (p, right());
+      const Comparison_result res2 = compare_y(p, right());
       return (res2 != LARGER);
     }
   };
@@ -329,23 +317,21 @@ public:
     friend class Arr_segment_traits_2<Kernel>;
 
   public:
-    /*!
-     * Compare the x-coordinates of two points.
+    /*! Compare the x-coordinates of two points.
      * \param p1 The first point.
      * \param p2 The second point.
      * \return LARGER if x(p1) > x(p2);
      *         SMALLER if x(p1) < x(p2);
      *         EQUAL if x(p1) = x(p2).
      */
-    Comparison_result operator() (const Point_2& p1, const Point_2& p2) const
+    Comparison_result operator()(const Point_2& p1, const Point_2& p2) const
     {
       const Kernel& kernel = m_traits;
-
       return (kernel.compare_x_2_object()(p1, p2));
     }
   };
 
-  /*! Get a Compare_x_2 functor object. */
+  /*! Obtain a Compare_x_2 functor object. */
   Compare_x_2 compare_x_2_object() const { return Compare_x_2(*this); }
 
   class Compare_xy_2 {
@@ -363,8 +349,7 @@ public:
     friend class Arr_segment_traits_2<Kernel>;
 
   public:
-    /*!
-     * Compare two points lexicographically: by x, then by y.
+    /*! Compare two points lexicographically: by x, then by y.
      * \param p1 The first point.
      * \param p2 The second point.
      * \return LARGER if x(p1) > x(p2), or if x(p1) = x(p2) and y(p1) > y(p2);
@@ -378,13 +363,12 @@ public:
     }
   };
 
-  /*! Get a Compare_xy_2 functor object. */
+  /*! Obtain a Compare_xy_2 functor object. */
   Compare_xy_2 compare_xy_2_object() const { return Compare_xy_2(*this); }
 
   class Construct_min_vertex_2 {
   public:
-    /*!
-     * Get the left endpoint of the x-monotone curve (segment).
+    /*! Obtain the left endpoint of the x-monotone curve (segment).
      * \param cv The curve.
      * \return The left endpoint.
      */
@@ -392,29 +376,27 @@ public:
     { return (cv.left()); }
   };
 
-  /*! Get a Construct_min_vertex_2 functor object. */
+  /*! Obtain a Construct_min_vertex_2 functor object. */
   Construct_min_vertex_2 construct_min_vertex_2_object() const
   { return Construct_min_vertex_2(); }
 
   class Construct_max_vertex_2 {
   public:
-    /*!
-     * Get the right endpoint of the x-monotone curve (segment).
+    /*! Obtain the right endpoint of the x-monotone curve (segment).
      * \param cv The curve.
      * \return The right endpoint.
      */
-    const Point_2& operator() (const X_monotone_curve_2& cv) const
+    const Point_2& operator()(const X_monotone_curve_2& cv) const
     { return (cv.right()); }
   };
 
-  /*! Get a Construct_max_vertex_2 functor object. */
+  /*! Obtain a Construct_max_vertex_2 functor object. */
   Construct_max_vertex_2 construct_max_vertex_2_object() const
   { return Construct_max_vertex_2(); }
 
   class Is_vertical_2 {
   public:
-    /*!
-     * Check whether the given x-monotone curve is a vertical segment.
+    /*! Check whether the given x-monotone curve is a vertical segment.
      * \param cv The curve.
      * \return (true) if the curve is a vertical segment; (false) otherwise.
      */
@@ -422,9 +404,8 @@ public:
     { return (cv.is_vertical()); }
   };
 
-  /*! Get an Is_vertical_2 functor object. */
-  Is_vertical_2 is_vertical_2_object () const
-  { return Is_vertical_2(); }
+  /*! Obtain an Is_vertical_2 functor object. */
+  Is_vertical_2 is_vertical_2_object () const { return Is_vertical_2(); }
 
   class Compare_y_at_x_2 {
   protected:
@@ -441,8 +422,7 @@ public:
     friend class Arr_segment_traits_2<Kernel>;
 
   public:
-    /*!
-     * Return the location of the given point with respect to the input curve.
+    /*! Return the location of the given point with respect to the input curve.
      * \param cv The curve.
      * \param p The point.
      * \pre p is in the x-range of cv.
@@ -453,26 +433,26 @@ public:
     Comparison_result operator()(const Point_2& p,
                                  const X_monotone_curve_2& cv) const
     {
-      CGAL_precondition (cv.is_in_x_range(p));
+      CGAL_precondition(cv.is_in_x_range(p));
 
       const Kernel& kernel = m_traits;
 
       if (! cv.is_vertical()) {
         // Compare p with the segment's supporting line.
-        return (kernel.compare_y_at_x_2_object()(p, cv.line()));
+        CGAL_assertion_code(auto cmp_x = kernel.compare_x_2_object());
+        CGAL_assertion(cmp_x(cv.left(), cv.right()) == SMALLER);
+        return kernel.orientation_2_object()(cv.left(), cv.right(), p);
       }
-      else {
-        // Compare with the vertical segment's end-points.
-        typename Kernel::Compare_y_2  compare_y = kernel.compare_y_2_object();
-        Comparison_result res1 = compare_y(p, cv.left());
-        Comparison_result res2 = compare_y(p, cv.right());
 
-        return (res1 == res2) ? res1 : EQUAL;
-      }
+      // Compare with the vertical segment's end-points.
+      typename Kernel::Compare_y_2 compare_y = kernel.compare_y_2_object();
+      Comparison_result res1 = compare_y(p, cv.left());
+      Comparison_result res2 = compare_y(p, cv.right());
+      return (res1 == res2) ? res1 : EQUAL;
     }
   };
 
-  /*! Get a Compare_y_at_x_2 functor object. */
+  /*! Obtain a Compare_y_at_x_2 functor object. */
   Compare_y_at_x_2 compare_y_at_x_2_object() const
   { return Compare_y_at_x_2(*this); }
 
@@ -491,8 +471,7 @@ public:
     friend class Arr_segment_traits_2<Kernel>;
 
   public:
-    /*!
-     * Compare the y value of two x-monotone curves immediately to the left
+    /*! Compare the y value of two x-monotone curves immediately to the left
      * of their intersection point.
      * \param cv1 The first curve.
      * \param cv2 The second curve.
@@ -510,14 +489,10 @@ public:
 
       // Make sure that p lies on both curves, and that both are defined to its
       // left (so their left endpoint is lexicographically smaller than p).
-      CGAL_precondition_code(
-        typename Kernel::Compare_xy_2 compare_xy =
-          kernel.compare_xy_2_object();
-      );
-
-      CGAL_precondition(
-         (m_traits.compare_y_at_x_2_object()(p, cv1) == EQUAL) &&
-         (m_traits.compare_y_at_x_2_object()(p, cv2) == EQUAL));
+      CGAL_precondition_code(auto compare_xy = kernel.compare_xy_2_object());
+
+      CGAL_precondition((m_traits.compare_y_at_x_2_object()(p, cv1) == EQUAL) &&
+                        (m_traits.compare_y_at_x_2_object()(p, cv2) == EQUAL));
 
       CGAL_precondition(compare_xy(cv1.left(), p) == SMALLER &&
                         compare_xy(cv2.left(), p) == SMALLER);
@@ -531,8 +506,8 @@ public:
     }
   };
 
-  /*! Get a Compare_y_at_x_left_2 functor object. */
-  Compare_y_at_x_left_2 compare_y_at_x_left_2_object () const
+  /*! Obtain a Compare_y_at_x_left_2 functor object. */
+  Compare_y_at_x_left_2 compare_y_at_x_left_2_object() const
   { return Compare_y_at_x_left_2(*this); }
 
   class Compare_y_at_x_right_2 {
@@ -550,8 +525,7 @@ public:
     friend class Arr_segment_traits_2<Kernel>;
 
   public:
-    /*!
-     * Compare the y value of two x-monotone curves immediately to the right
+    /*! Compare the y value of two x-monotone curves immediately to the right
      * of their intersection point.
      * \param cv1 The first curve.
      * \param cv2 The second curve.
@@ -569,14 +543,10 @@ public:
 
       // Make sure that p lies on both curves, and that both are defined to its
       // right (so their right endpoint is lexicographically larger than p).
-      CGAL_precondition_code (
-        typename Kernel::Compare_xy_2 compare_xy =
-          kernel.compare_xy_2_object();
-      );
-
-      CGAL_precondition(
-        (m_traits.compare_y_at_x_2_object()(p, cv1) == EQUAL) &&
-        (m_traits.compare_y_at_x_2_object()(p, cv2) == EQUAL));
+      CGAL_precondition_code(auto compare_xy = kernel.compare_xy_2_object());
+
+      CGAL_precondition((m_traits.compare_y_at_x_2_object()(p, cv1) == EQUAL) &&
+                        (m_traits.compare_y_at_x_2_object()(p, cv2) == EQUAL));
 
       CGAL_precondition(compare_xy(cv1.right(), p) == LARGER &&
                         compare_xy(cv2.right(), p) == LARGER);
@@ -588,7 +558,7 @@ public:
     }
   };
 
-  /*! Get a Compare_y_at_x_right_2 functor object. */
+  /*! Obtain a Compare_y_at_x_right_2 functor object. */
   Compare_y_at_x_right_2 compare_y_at_x_right_2_object() const
   { return Compare_y_at_x_right_2(*this); }
 
@@ -607,8 +577,8 @@ public:
     friend class Arr_segment_traits_2<Kernel>;
 
   public:
-    /*!
-     * Check if the two x-monotone curves are the same (have the same graph).
+    /*! Check whether the two x-monotone curves are the same (have the same
+     * graph).
      * \param cv1 The first curve.
      * \param cv2 The second curve.
      * \return (true) if the two curves are the same; (false) otherwise.
@@ -628,14 +598,14 @@ public:
      * \param p2 The second point.
      * \return (true) if the two point are the same; (false) otherwise.
      */
-    bool operator() (const Point_2& p1, const Point_2& p2) const
+    bool operator()(const Point_2& p1, const Point_2& p2) const
     {
       const Kernel& kernel = m_traits;
       return (kernel.equal_2_object()(p1, p2));
     }
   };
 
-  /*! Get an Equal_2 functor object. */
+  /*! Obtain an Equal_2 functor object. */
   Equal_2 equal_2_object() const { return Equal_2(*this); }
   //@}
 
@@ -644,25 +614,23 @@ public:
 
   class Make_x_monotone_2 {
   public:
-    /*!
-     * Cut the given curve into x-monotone subcurves and insert them into the
+    /*! Cut the given curve into x-monotone subcurves and insert them into the
      * given output iterator. As segments are always x_monotone, only one
      * object will be contained in the iterator.
      * \param cv The curve.
-     * \param oi The output iterator, whose value-type is Object.
+     * \param oi The output iterator, whose value-type is variant<....
      * \return The past-the-end iterator.
      */
-    template<class OutputIterator>
+    template <typename OutputIterator>
     OutputIterator operator()(const Curve_2& cv, OutputIterator oi) const
     {
       // Wrap the segment with an object.
-      *oi = make_object (cv);
-      ++oi;
-      return (oi);
+      *oi++ = make_object(cv);
+      return oi;
     }
   };
 
-  /*! Get a Make_x_monotone_2 functor object. */
+  /*! Obtain a Make_x_monotone_2 functor object. */
   Make_x_monotone_2 make_x_monotone_2_object() const
   { return Make_x_monotone_2(); }
 
@@ -681,8 +649,7 @@ public:
     friend class Arr_segment_traits_2<Kernel>;
 
   public:
-    /*!
-     * Split a given x-monotone curve at a given point into two sub-curves.
+    /*! Split a given x-monotone curve at a given point into two sub-curves.
      * \param cv The curve to split
      * \param p The split point.
      * \param c1 Output: The left resulting subcurve (p is its right endpoint).
@@ -693,28 +660,24 @@ public:
                     X_monotone_curve_2& c1, X_monotone_curve_2& c2) const
     {
       // Make sure that p lies on the interior of the curve.
-      CGAL_precondition_code (
-        const Kernel& kernel = m_traits;
-        typename Kernel::Compare_xy_2 compare_xy =
-          kernel.compare_xy_2_object();
-      );
-
-      CGAL_precondition(
-        (m_traits.compare_y_at_x_2_object()(p, cv) == EQUAL) &&
-        compare_xy(cv.left(), p) == SMALLER &&
-        compare_xy(cv.right(), p) == LARGER);
+      CGAL_precondition_code(const Kernel& kernel = m_traits;
+                             auto compare_xy = kernel.compare_xy_2_object());
+
+      CGAL_precondition((m_traits.compare_y_at_x_2_object()(p, cv) == EQUAL) &&
+                        compare_xy(cv.left(), p) == SMALLER &&
+                        compare_xy(cv.right(), p) == LARGER);
 
       // Perform the split.
       c1 = cv;
-      c1.set_right (p);
+      c1.set_right(p);
 
       c2 = cv;
-      c2.set_left (p);
+      c2.set_left(p);
     }
   };
 
-  /*! Get a Split_2 functor object. */
-  Split_2 split_2_object () const { return Split_2(*this); }
+  /*! Obtain a Split_2 functor object. */
+  Split_2 split_2_object() const { return Split_2(*this); }
 
   class Intersect_2 {
   protected:
@@ -731,8 +694,7 @@ public:
     friend class Arr_segment_traits_2<Kernel>;
 
   public:
-    /*!
-     * Find the intersections of the two given curves and insert them into the
+    /*! Find the intersections of the two given curves and insert them into the
      * given output iterator. As two segments may intersect only once, only a
      * single intersection will be contained in the iterator.
      * \param cv1 The first curve.
@@ -740,93 +702,90 @@ public:
      * \param oi The output iterator.
      * \return The past-the-end iterator.
      */
-    template<class OutputIterator>
-    OutputIterator operator() (const X_monotone_curve_2& cv1,
-                               const X_monotone_curve_2& cv2,
-                               OutputIterator oi) const
-    {
+    template <typename OutputIterator>
+    OutputIterator operator()(const X_monotone_curve_2& cv1,
+                              const X_monotone_curve_2& cv2,
+                              OutputIterator oi) const
+    {
+      typedef std::pair<Point_2, Multiplicity>          Intersection_point;
+      typedef boost::variant<Intersection_point, X_monotone_curve_2>
+                                                        Intersection_result;
+
       // Intersect the two supporting lines.
       const Kernel& kernel = m_traits;
-      CGAL::Object obj = kernel.intersect_2_object()(cv1.line(), cv2.line());
+      auto res = kernel.intersect_2_object()(cv1.line(), cv2.line());
 
-      if (obj.is_empty()) {
-        // The supporting line are parallel lines and do not intersect:
-        return (oi);
-      }
+      // The supporting line are parallel lines and do not intersect:
+      if (! res) return oi;
 
       // Check if we have a single intersection point.
-      const Point_2  *ip = object_cast<Point_2> (&obj);
-
+      const Point_2* ip = boost::get<Point_2>(&*res);
       if (ip != nullptr) {
         // Check if the intersection point ip lies on both segments.
-        const bool    ip_on_cv1 = cv1.is_vertical() ? cv1.is_in_y_range(*ip) :
-                                                      cv1.is_in_x_range(*ip);
+        const bool ip_on_cv1 = cv1.is_vertical() ?
+          cv1.is_in_y_range(*ip) : cv1.is_in_x_range(*ip);
 
         if (ip_on_cv1) {
-          const bool  ip_on_cv2 = cv2.is_vertical() ? cv2.is_in_y_range(*ip) :
-                                                      cv2.is_in_x_range(*ip);
+          const bool ip_on_cv2 = cv2.is_vertical() ?
+            cv2.is_in_y_range(*ip) : cv2.is_in_x_range(*ip);
 
           if (ip_on_cv2) {
             // Create a pair representing the point with its multiplicity,
             // which is always 1 for line segments.
-            std::pair<Point_2, Multiplicity>   ip_mult (*ip, 1);
-            *oi = make_object (ip_mult);
-            oi++;
+            Intersection_point ip_mult(*ip, 1);
+            *oi++ = Intersection_result(ip_mult);
           }
         }
-        return (oi);
+        return oi;
       }
 
       // In this case, the two supporting lines overlap.
       // The overlapping segment is therefore [p_l,p_r], where p_l is the
       // rightmost of the two left endpoints and p_r is the leftmost of the
       // two right endpoints.
-      typename Kernel::Compare_xy_2  compare_xy = kernel.compare_xy_2_object();
-      Point_2                        p_l, p_r;
-
-      if (compare_xy (cv1.left(), cv2.left()) == SMALLER) p_l = cv2.left();
-      else p_l = cv1.left();
-
-      if (compare_xy (cv1.right(), cv2.right()) == SMALLER) p_r = cv1.right();
-      else p_r = cv2.right();
+      auto compare_xy = kernel.compare_xy_2_object();
+      Point_2 p_l = (compare_xy(cv1.left(), cv2.left()) == SMALLER) ?
+        cv2.left() : cv1.left();
+      Point_2 p_r = (compare_xy(cv1.right(), cv2.right()) == SMALLER) ?
+        cv1.right() : cv2.right();
 
       // Examine the resulting segment.
-      const Comparison_result        res = compare_xy (p_l, p_r);
+      const Comparison_result cmp_res = compare_xy(p_l, p_r);
 
-      if (res == SMALLER) {
+      if (cmp_res == SMALLER) {
         // We have discovered an overlapping segment:
         if (cv1.is_directed_right() == cv2.is_directed_right()) {
           // cv1 and cv2 have the same directions, maintain this direction
           // in the overlap segment
           if (cv1.is_directed_right()) {
-            X_monotone_curve_2  overlap_seg(cv1.line(), p_l, p_r);
-            *oi++ = make_object(overlap_seg);
-          }
-          else {
-            X_monotone_curve_2  overlap_seg(cv1.line(), p_r, p_l);
-            *oi++ = make_object(overlap_seg);
+            X_monotone_curve_2 overlap_seg(cv1.line(), p_l, p_r);
+            *oi++ = Intersection_result(overlap_seg);
+            return oi;
           }
+          X_monotone_curve_2 overlap_seg(cv1.line(), p_r, p_l);
+          *oi++ = Intersection_result(overlap_seg);
+          return oi;
         }
-        else {
-          // cv1 and cv2 have opposite directions, the overlap segment
-          // will be directed from left to right
-          X_monotone_curve_2  overlap_seg(cv1.line(), p_l, p_r);
-          *oi++ = make_object(overlap_seg);
-        }
+        // cv1 and cv2 have opposite directions, the overlap segment
+        // will be directed from left to right
+        X_monotone_curve_2 overlap_seg(cv1.line(), p_l, p_r);
+        *oi++ = Intersection_result(overlap_seg);
+        return oi;
       }
-      else if (res == EQUAL) {
+      if (cmp_res == EQUAL) {
         // The two segment have the same supporting line, but they just share
         // a common endpoint. Thus we have an intersection point, but we leave
         // the multiplicity of this point undefined.
-        std::pair<Point_2, Multiplicity> ip_mult(p_r, 0);
-        *oi++ = make_object(ip_mult);
+        Intersection_point ip_mult(p_r, 0);
+        *oi++ = Intersection_result(ip_mult);
+        return oi;
       }
 
-      return (oi);
+      return oi;
     }
   };
 
-  /*! Get an Intersect_2 functor object. */
+  /*! Obtain an Intersect_2 functor object. */
   Intersect_2 intersect_2_object() const { return Intersect_2(*this); }
 
   class Are_mergeable_2 {
@@ -844,8 +803,7 @@ public:
     friend class Arr_segment_traits_2<Kernel>;
 
   public:
-    /*!
-     * Check whether it is possible to merge two given x-monotone curves.
+    /*! Check whether it is possible to merge two given x-monotone curves.
      * \param cv1 The first curve.
      * \param cv2 The second curve.
      * \return (true) if the two curves are mergeable, that is, if they are
@@ -855,20 +813,20 @@ public:
     bool operator()(const X_monotone_curve_2& cv1,
                     const X_monotone_curve_2& cv2) const
     {
-      if (!m_traits.equal_2_object()(cv1.right(), cv2.left()) &&
-          !m_traits.equal_2_object()(cv2.right(), cv1.left()))
+      const Kernel& kernel = m_traits;
+      typename Kernel::Equal_2 equal = kernel.equal_2_object();
+      if (! equal(cv1.right(), cv2.left()) &&
+          ! equal(cv2.right(), cv1.left()))
         return false;
 
       // Check whether the two curves have the same supporting line.
-      const Kernel& kernel = m_traits;
-      typename Kernel::Equal_2 equal = kernel.equal_2_object();
       return (equal(cv1.line(), cv2.line()) ||
               equal(cv1.line(),
                     kernel.construct_opposite_line_2_object()(cv2.line())));
     }
   };
 
-  /*! Get an Are_mergeable_2 functor object. */
+  /*! Obtain an Are_mergeable_2 functor object. */
   Are_mergeable_2 are_mergeable_2_object() const
   { return Are_mergeable_2(*this); }
 
@@ -890,8 +848,7 @@ public:
     friend class Arr_segment_traits_2<Kernel>;
 
   public:
-    /*!
-     * Merge two given x-monotone curves into a single curve (segment).
+    /*! Merge two given x-monotone curves into a single curve (segment).
      * \param cv1 The first curve.
      * \param cv2 The second curve.
      * \param c Output: The merged curve.
@@ -903,25 +860,26 @@ public:
     {
       CGAL_precondition(m_traits.are_mergeable_2_object()(cv1, cv2));
 
-      Equal_2 equal = m_traits.equal_2_object();
+      const Kernel& kernel = m_traits;
+      auto equal = kernel.equal_2_object();
 
       // Check which curve extends to the right of the other.
       if (equal(cv1.right(), cv2.left())) {
         // cv2 extends cv1 to the right.
         c = cv1;
         c.set_right(cv2.right());
+        return;
       }
-      else {
-        CGAL_precondition(equal(cv2.right(), cv1.left()));
 
-        // cv1 extends cv2 to the right.
-        c = cv2;
-        c.set_right(cv1.right());
-      }
+      CGAL_precondition(equal(cv2.right(), cv1.left()));
+
+      // cv1 extends cv2 to the right.
+      c = cv2;
+      c.set_right(cv1.right());
     }
   };
 
-  /*! Get a Merge_2 functor object. */
+  /*! Obtain a Merge_2 functor object. */
   Merge_2 merge_2_object() const { return Merge_2(*this); }
   //@}
 
@@ -931,8 +889,7 @@ public:
 
   class Approximate_2 {
   public:
-    /*!
-     * Return an approximation of a point coordinate.
+    /*! Obtain an approximation of a point coordinate.
      * \param p The exact point.
      * \param i The coordinate index (either 0 or 1).
      * \pre i is either 0 or 1.
@@ -946,13 +903,12 @@ public:
     }
   };
 
-  /*! Get an Approximate_2 functor object. */
+  /*! Obtain an Approximate_2 functor object. */
   Approximate_2 approximate_2_object() const { return Approximate_2(); }
 
   class Construct_x_monotone_curve_2 {
   public:
-    /*!
-     * Return an x-monotone curve connecting the two given endpoints.
+    /*! Obtain an x-monotone curve connecting the two given endpoints.
      * \param p The first point.
      * \param q The second point.
      * \pre p and q must not be the same.
@@ -962,7 +918,7 @@ public:
     { return (X_monotone_curve_2(p, q)); }
   };
 
-  /*! Get a Construct_x_monotone_curve_2 functor object. */
+  /*! Obtain a Construct_x_monotone_curve_2 functor object. */
   Construct_x_monotone_curve_2 construct_x_monotone_curve_2_object() const
   { return Construct_x_monotone_curve_2(); }
   //@}
@@ -1010,41 +966,37 @@ public:
 
       // exchange src and tgt IF they do not conform with the direction
       X_monotone_curve_2 trimmed_segment;
-
       if (xcv.is_directed_right() && compare_x_2(src, tgt) == LARGER)
         trimmed_segment = X_monotone_curve_2(tgt, src);
-      else if (!xcv.is_directed_right() && compare_x_2(src, tgt) == SMALLER )
+      else if (! xcv.is_directed_right() && (compare_x_2(src, tgt) == SMALLER))
         trimmed_segment = X_monotone_curve_2(tgt, src);
       else trimmed_segment = X_monotone_curve_2(src, tgt);
-      return (trimmed_segment);
+      return trimmed_segment;
     }
   };
 
-  //get a Trim_2 functor object
+  /*! Obtain a Trim_2 functor object */
   Trim_2 trim_2_object() const { return Trim_2(*this); }
 
-  class Compare_endpoints_xy_2
-  {
+  class Compare_endpoints_xy_2 {
   public:
-    /*!
-     * Compare the endpoints of an $x$-monotone curve lexicographically.
+    /*! Compare the endpoints of an $x$-monotone curve lexicographically.
      * (assuming the curve has a designated source and target points).
      * \param cv The curve.
      * \return SMALLER if the curve is directed right;
      *         LARGER if the curve is directed left.
      */
-    Comparison_result operator() (const X_monotone_curve_2& cv) const
+    Comparison_result operator()(const X_monotone_curve_2& cv) const
     { return (cv.is_directed_right()) ? (SMALLER) : (LARGER); }
   };
 
-  /*! Get a Compare_endpoints_xy_2 functor object. */
+  /*! Obtain a Compare_endpoints_xy_2 functor object. */
   Compare_endpoints_xy_2 compare_endpoints_xy_2_object() const
   { return Compare_endpoints_xy_2(); }
 
   class Construct_opposite_2 {
   public:
-    /*!
-     * Construct an opposite x-monotone (with swapped source and target).
+    /*! Construct an opposite x-monotone (with swapped source and target).
      * \param cv The curve.
      * \return The opposite curve.
      */
@@ -1052,17 +1004,16 @@ public:
     { return (cv.flip()); }
   };
 
-  /*! Get a Construct_opposite_2 functor object. */
+  /*! Obtain a Construct_opposite_2 functor object. */
   Construct_opposite_2 construct_opposite_2_object() const
   { return Construct_opposite_2(); }
   //@}
 };
 
-/*!
- * \class A representation of a segment, as used by the Arr_segment_traits_2
+/*! \class A representation of a segment, as used by the Arr_segment_traits_2
  * traits-class.
  */
-template <class Kernel_>
+template <typename Kernel_>
 class Arr_segment_2 :
     public Arr_segment_traits_2<Kernel_>::_Segment_cached_2
 {
@@ -1088,7 +1039,7 @@ public:
    * \pre The two points are not the same.
    */
   Arr_segment_2(const Point_2& source, const Point_2& target) :
-    Base(source,target)
+    Base(source, target)
   {}
 
   /*! Construct a segment from a line and two end-points.
@@ -1100,7 +1051,7 @@ public:
    */
   Arr_segment_2(const Line_2& line,
                 const Point_2& source, const Point_2& target) :
-    Base(line,source,target)
+    Base(line,source, target)
   {}
 
   /*! Cast to a segment.
@@ -1108,8 +1059,8 @@ public:
   operator Segment_2() const
   {
     Kernel kernel;
-    Segment_2 seg = kernel.construct_segment_2_object()(this->ps, this->pt);
-    return (seg);
+    Segment_2 seg = kernel.construct_segment_2_object()(this->m_ps, this->m_pt);
+    return seg;
   }
 
   /*! Create a bounding box for the segment.
@@ -1117,37 +1068,37 @@ public:
   Bbox_2 bbox() const
   {
     Kernel kernel;
-    Segment_2 seg = kernel.construct_segment_2_object()(this->ps, this->pt);
-    return (kernel.construct_bbox_2_object() (seg));
+    Segment_2 seg = kernel.construct_segment_2_object()(this->m_ps, this->m_pt);
+    return (kernel.construct_bbox_2_object()(seg));
   }
 
   /*! Obtain the segment source.
    */
-  const Point_2& source() const { return (this->ps); }
+  const Point_2& source() const { return (this->m_ps); }
 
   /*! Obtain the segment target.
    */
-  const Point_2& target() const { return (this->pt); }
+  const Point_2& target() const { return (this->m_pt); }
 
   /*! Flip the segment (swap its source and target).
    */
   Arr_segment_2 flip() const
   {
-    Arr_segment_2   opp;
-    opp.l = this->l;
-    opp.ps = this->pt;
-    opp.pt = this->ps;
-    opp.is_pt_max = !(this->is_pt_max);
-    opp.is_vert = this->is_vert;
-    opp.is_degen = this->is_degen;
+    Arr_segment_2 opp;
+    opp.m_l = this->m_l;
+    opp.m_ps = this->m_pt;
+    opp.m_pt = this->m_ps;
+    opp.m_is_pt_max = !(this->m_is_pt_max);
+    opp.m_is_vert = this->m_is_vert;
+    opp.m_is_degen = this->m_is_degen;
 
-    return (opp);
+    return opp;
   }
 };
 
 /*! Exporter for the segment class used by the traits-class.
  */
-template <class Kernel, class OutputStream>
+template <typename Kernel, typename OutputStream>
 OutputStream& operator<<(OutputStream& os, const Arr_segment_2<Kernel>& seg)
 {
   os << static_cast<typename Kernel::Segment_2>(seg);
@@ -1156,13 +1107,13 @@ OutputStream& operator<<(OutputStream& o
 
 /*! Importer for the segment class used by the traits-class.
  */
-template <class Kernel, class InputStream>
+template <typename Kernel, typename InputStream>
 InputStream& operator>>(InputStream& is, Arr_segment_2<Kernel>& seg)
 {
   typename Kernel::Segment_2   kernel_seg;
   is >> kernel_seg;
   seg = kernel_seg;
-  return (is);
+  return is;
 }
 
 } //namespace CGAL
diff -pruN 5.0.3-1/include/CGAL/Arr_simple_point_location.h 5.1-1/include/CGAL/Arr_simple_point_location.h
--- 5.0.3-1/include/CGAL/Arr_simple_point_location.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_simple_point_location.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_simple_point_location.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_simple_point_location.h $
 // $Id: Arr_simple_point_location.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_on_sphere_transformation.h 5.1-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_on_sphere_transformation.h
--- 5.0.3-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_on_sphere_transformation.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_on_sphere_transformation.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_spherical_gaussian_map_3/Arr_on_sphere_transformation.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_spherical_gaussian_map_3/Arr_on_sphere_transformation.h $
 // $Id: Arr_on_sphere_transformation.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_arr_dcel.h 5.1-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_arr_dcel.h
--- 5.0.3-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_arr_dcel.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_arr_dcel.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_arr_dcel.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_arr_dcel.h $
 // $Id: Arr_polyhedral_sgm_arr_dcel.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm.h 5.1-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm.h
--- 5.0.3-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm.h $
 // $Id: Arr_polyhedral_sgm.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_initializer_visitor.h 5.1-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_initializer_visitor.h
--- 5.0.3-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_initializer_visitor.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_initializer_visitor.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_initializer_visitor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_initializer_visitor.h $
 // $Id: Arr_polyhedral_sgm_initializer_visitor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_overlay.h 5.1-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_overlay.h
--- 5.0.3-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_overlay.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_overlay.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_overlay.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_overlay.h $
 // $Id: Arr_polyhedral_sgm_overlay.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_polyhedron_3.h 5.1-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_polyhedron_3.h
--- 5.0.3-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_polyhedron_3.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_polyhedron_3.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_polyhedron_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_polyhedron_3.h $
 // $Id: Arr_polyhedral_sgm_polyhedron_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_traits.h 5.1-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_traits.h
--- 5.0.3-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_traits.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_traits.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_traits.h $
 // $Id: Arr_polyhedral_sgm_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_transformation.h 5.1-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_transformation.h
--- 5.0.3-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_transformation.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_transformation.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_transformation.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_spherical_gaussian_map_3/Arr_polyhedral_sgm_transformation.h $
 // $Id: Arr_polyhedral_sgm_transformation.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_spherical_gaussian_map_3.h 5.1-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_spherical_gaussian_map_3.h
--- 5.0.3-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_spherical_gaussian_map_3.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_spherical_gaussian_map_3.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_spherical_gaussian_map_3/Arr_spherical_gaussian_map_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_spherical_gaussian_map_3/Arr_spherical_gaussian_map_3.h $
 // $Id: Arr_spherical_gaussian_map_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_transform_on_sphere.h 5.1-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_transform_on_sphere.h
--- 5.0.3-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_transform_on_sphere.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_spherical_gaussian_map_3/Arr_transform_on_sphere.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_spherical_gaussian_map_3/Arr_transform_on_sphere.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_spherical_gaussian_map_3/Arr_transform_on_sphere.h $
 // $Id: Arr_transform_on_sphere.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_spherical_topology_traits_2.h 5.1-1/include/CGAL/Arr_spherical_topology_traits_2.h
--- 5.0.3-1/include/CGAL/Arr_spherical_topology_traits_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_spherical_topology_traits_2.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_spherical_topology_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_spherical_topology_traits_2.h $
 // $Id: Arr_spherical_topology_traits_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_tags.h 5.1-1/include/CGAL/Arr_tags.h
--- 5.0.3-1/include/CGAL/Arr_tags.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_tags.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_tags.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_tags.h $
 // $Id: Arr_tags.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_bounded_planar_batched_pl_helper.h 5.1-1/include/CGAL/Arr_topology_traits/Arr_bounded_planar_batched_pl_helper.h
--- 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_bounded_planar_batched_pl_helper.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_topology_traits/Arr_bounded_planar_batched_pl_helper.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_bounded_planar_batched_pl_helper.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_bounded_planar_batched_pl_helper.h $
 // $Id: Arr_bounded_planar_batched_pl_helper.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_bounded_planar_construction_helper.h 5.1-1/include/CGAL/Arr_topology_traits/Arr_bounded_planar_construction_helper.h
--- 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_bounded_planar_construction_helper.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_topology_traits/Arr_bounded_planar_construction_helper.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_bounded_planar_construction_helper.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_bounded_planar_construction_helper.h $
 // $Id: Arr_bounded_planar_construction_helper.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_bounded_planar_insertion_helper.h 5.1-1/include/CGAL/Arr_topology_traits/Arr_bounded_planar_insertion_helper.h
--- 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_bounded_planar_insertion_helper.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_topology_traits/Arr_bounded_planar_insertion_helper.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_bounded_planar_insertion_helper.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_bounded_planar_insertion_helper.h $
 // $Id: Arr_bounded_planar_insertion_helper.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_bounded_planar_overlay_helper.h 5.1-1/include/CGAL/Arr_topology_traits/Arr_bounded_planar_overlay_helper.h
--- 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_bounded_planar_overlay_helper.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_topology_traits/Arr_bounded_planar_overlay_helper.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_bounded_planar_overlay_helper.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_bounded_planar_overlay_helper.h $
 // $Id: Arr_bounded_planar_overlay_helper.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_bounded_planar_topology_traits_2_impl.h 5.1-1/include/CGAL/Arr_topology_traits/Arr_bounded_planar_topology_traits_2_impl.h
--- 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_bounded_planar_topology_traits_2_impl.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_topology_traits/Arr_bounded_planar_topology_traits_2_impl.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_bounded_planar_topology_traits_2_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_bounded_planar_topology_traits_2_impl.h $
 // $Id: Arr_bounded_planar_topology_traits_2_impl.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_bounded_planar_vert_decomp_helper.h 5.1-1/include/CGAL/Arr_topology_traits/Arr_bounded_planar_vert_decomp_helper.h
--- 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_bounded_planar_vert_decomp_helper.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_topology_traits/Arr_bounded_planar_vert_decomp_helper.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_bounded_planar_vert_decomp_helper.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_bounded_planar_vert_decomp_helper.h $
 // $Id: Arr_bounded_planar_vert_decomp_helper.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_inc_insertion_zone_visitor.h 5.1-1/include/CGAL/Arr_topology_traits/Arr_inc_insertion_zone_visitor.h
--- 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_inc_insertion_zone_visitor.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_topology_traits/Arr_inc_insertion_zone_visitor.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_inc_insertion_zone_visitor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_inc_insertion_zone_visitor.h $
 // $Id: Arr_inc_insertion_zone_visitor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_planar_topology_traits_base_2.h 5.1-1/include/CGAL/Arr_topology_traits/Arr_planar_topology_traits_base_2.h
--- 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_planar_topology_traits_base_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_topology_traits/Arr_planar_topology_traits_base_2.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_planar_topology_traits_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_planar_topology_traits_base_2.h $
 // $Id: Arr_planar_topology_traits_base_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_spherical_batched_pl_helper.h 5.1-1/include/CGAL/Arr_topology_traits/Arr_spherical_batched_pl_helper.h
--- 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_spherical_batched_pl_helper.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_topology_traits/Arr_spherical_batched_pl_helper.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_spherical_batched_pl_helper.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_spherical_batched_pl_helper.h $
 // $Id: Arr_spherical_batched_pl_helper.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_spherical_construction_helper.h 5.1-1/include/CGAL/Arr_topology_traits/Arr_spherical_construction_helper.h
--- 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_spherical_construction_helper.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_topology_traits/Arr_spherical_construction_helper.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_spherical_construction_helper.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_spherical_construction_helper.h $
 // $Id: Arr_spherical_construction_helper.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_spherical_insertion_helper.h 5.1-1/include/CGAL/Arr_topology_traits/Arr_spherical_insertion_helper.h
--- 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_spherical_insertion_helper.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_topology_traits/Arr_spherical_insertion_helper.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_spherical_insertion_helper.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_spherical_insertion_helper.h $
 // $Id: Arr_spherical_insertion_helper.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_spherical_overlay_helper.h 5.1-1/include/CGAL/Arr_topology_traits/Arr_spherical_overlay_helper.h
--- 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_spherical_overlay_helper.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_topology_traits/Arr_spherical_overlay_helper.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_spherical_overlay_helper.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_spherical_overlay_helper.h $
 // $Id: Arr_spherical_overlay_helper.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_spherical_topology_traits_2_impl.h 5.1-1/include/CGAL/Arr_topology_traits/Arr_spherical_topology_traits_2_impl.h
--- 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_spherical_topology_traits_2_impl.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_topology_traits/Arr_spherical_topology_traits_2_impl.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_spherical_topology_traits_2_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_spherical_topology_traits_2_impl.h $
 // $Id: Arr_spherical_topology_traits_2_impl.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_spherical_vert_decomp_helper.h 5.1-1/include/CGAL/Arr_topology_traits/Arr_spherical_vert_decomp_helper.h
--- 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_spherical_vert_decomp_helper.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_topology_traits/Arr_spherical_vert_decomp_helper.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_spherical_vert_decomp_helper.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_spherical_vert_decomp_helper.h $
 // $Id: Arr_spherical_vert_decomp_helper.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_unb_planar_batched_pl_helper.h 5.1-1/include/CGAL/Arr_topology_traits/Arr_unb_planar_batched_pl_helper.h
--- 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_unb_planar_batched_pl_helper.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_topology_traits/Arr_unb_planar_batched_pl_helper.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_unb_planar_batched_pl_helper.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_unb_planar_batched_pl_helper.h $
 // $Id: Arr_unb_planar_batched_pl_helper.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_unb_planar_construction_helper.h 5.1-1/include/CGAL/Arr_topology_traits/Arr_unb_planar_construction_helper.h
--- 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_unb_planar_construction_helper.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_topology_traits/Arr_unb_planar_construction_helper.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_unb_planar_construction_helper.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_unb_planar_construction_helper.h $
 // $Id: Arr_unb_planar_construction_helper.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_unb_planar_insertion_helper.h 5.1-1/include/CGAL/Arr_topology_traits/Arr_unb_planar_insertion_helper.h
--- 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_unb_planar_insertion_helper.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_topology_traits/Arr_unb_planar_insertion_helper.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_unb_planar_insertion_helper.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_unb_planar_insertion_helper.h $
 // $Id: Arr_unb_planar_insertion_helper.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_unb_planar_overlay_helper.h 5.1-1/include/CGAL/Arr_topology_traits/Arr_unb_planar_overlay_helper.h
--- 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_unb_planar_overlay_helper.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_topology_traits/Arr_unb_planar_overlay_helper.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_unb_planar_overlay_helper.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_unb_planar_overlay_helper.h $
 // $Id: Arr_unb_planar_overlay_helper.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_unb_planar_topology_traits_2_impl.h 5.1-1/include/CGAL/Arr_topology_traits/Arr_unb_planar_topology_traits_2_impl.h
--- 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_unb_planar_topology_traits_2_impl.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_topology_traits/Arr_unb_planar_topology_traits_2_impl.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_unb_planar_topology_traits_2_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_unb_planar_topology_traits_2_impl.h $
 // $Id: Arr_unb_planar_topology_traits_2_impl.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_unb_planar_vert_decomp_helper.h 5.1-1/include/CGAL/Arr_topology_traits/Arr_unb_planar_vert_decomp_helper.h
--- 5.0.3-1/include/CGAL/Arr_topology_traits/Arr_unb_planar_vert_decomp_helper.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_topology_traits/Arr_unb_planar_vert_decomp_helper.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_unb_planar_vert_decomp_helper.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_unb_planar_vert_decomp_helper.h $
 // $Id: Arr_unb_planar_vert_decomp_helper.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_tracing_traits_2.h 5.1-1/include/CGAL/Arr_tracing_traits_2.h
--- 5.0.3-1/include/CGAL/Arr_tracing_traits_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_tracing_traits_2.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,11 +3,11 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_tracing_traits_2.h $
-// $Id: Arr_tracing_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_tracing_traits_2.h $
+// $Id: Arr_tracing_traits_2.h 6eefda4 2020-04-06T15:34:28+03:00 Efi Fogel
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
-// Author(s)     : Efi Fogel    <efif@post.tau.ac.il>
+// Author(s): Efi Fogel    <efif@post.tau.ac.il>
 
 #ifndef CGAL_ARR_TRACING_TRAITS_H
 #define CGAL_ARR_TRACING_TRAITS_H
@@ -26,6 +26,8 @@
 #include <iostream>
 #include <list>
 
+#include <boost/variant.hpp>
+
 #include <CGAL/basic.h>
 #include <CGAL/Arr_enums.h>
 #include <CGAL/Arr_tags.h>
@@ -592,7 +594,7 @@ public:
 
   public:
     /*! Construct */
-    Intersect_2(const Base * base, bool enabled = true) :
+    Intersect_2(const Base* base, bool enabled = true) :
       m_object(base->intersect_2_object()), m_enabled(enabled) {}
 
     /*! Operate
@@ -604,37 +606,41 @@ public:
      * multiplicity
      * \return the output iterator
      */
-    template<typename OutputIterator>
+    template <typename OutputIterator>
     OutputIterator operator()(const X_monotone_curve_2 & xcv1,
                               const X_monotone_curve_2 & xcv2,
                               OutputIterator oi) const
     {
-      if (!m_enabled) return m_object(xcv1, xcv2, oi);
+      typedef std::pair<Point_2, Multiplicity>          Intersection_point;
+      typedef boost::variant<Intersection_point, X_monotone_curve_2>
+                                                        Intersection_result;
+
+      if (! m_enabled) return m_object(xcv1, xcv2, oi);
+
       std::cout << "intersect" << std::endl
                 << "  xcv1: " << xcv1 << std::endl
                 << "  xcv2: " << xcv2 << std::endl;
-      std::list<CGAL::Object> container;
+      std::list<Intersection_result> container;
       m_object(xcv1, xcv2, std::back_inserter(container));
       if (container.empty()) return oi;
 
-      std::list<CGAL::Object>::iterator it;
       unsigned int i = 0;
-      for (it = container.begin(); it != container.end(); ++it) {
-        X_monotone_curve_2 xcv;
-        if (assign (xcv, *it)) {
-          std::cout << "  result[" << i++ << "]: xcv: " << xcv << std::endl;
+      for (const auto& item : container) {
+        const X_monotone_curve_2* xcv = boost::get<X_monotone_curve_2>(&item);
+        if (xcv != nullptr) {
+          std::cout << "  result[" << i++ << "]: xcv: " << *xcv << std::endl;
           continue;
         }
 
-        std::pair<Point_2,Multiplicity> point_pair;
-        if (assign (point_pair, *it)) {
-          std::cout << "  result[" << i++ << "]: p: " << point_pair.first
-                    << ", multiplicity: " << point_pair.second << std::endl;
+        const Intersection_point* ip = boost::get<Intersection_point>(&item);
+        if (ip != nullptr) {
+          std::cout << "  result[" << i++ << "]: p: " << ip->first
+                    << ", multiplicity: " << ip->second << std::endl;
           continue;
         }
       }
 
-      for (it = container.begin(); it != container.end(); ++it) *oi++ = *it;
+      for (auto it = container.begin(); it != container.end(); ++it) *oi++ = *it;
       container.clear();
       return oi;
     }
diff -pruN 5.0.3-1/include/CGAL/Arr_trapezoid_ric_point_location.h 5.1-1/include/CGAL/Arr_trapezoid_ric_point_location.h
--- 5.0.3-1/include/CGAL/Arr_trapezoid_ric_point_location.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_trapezoid_ric_point_location.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_trapezoid_ric_point_location.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_trapezoid_ric_point_location.h $
 // $Id: Arr_trapezoid_ric_point_location.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_triangulation_point_location.h 5.1-1/include/CGAL/Arr_triangulation_point_location.h
--- 5.0.3-1/include/CGAL/Arr_triangulation_point_location.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_triangulation_point_location.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_triangulation_point_location.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_triangulation_point_location.h $
 // $Id: Arr_triangulation_point_location.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_unb_planar_topology_traits_2.h 5.1-1/include/CGAL/Arr_unb_planar_topology_traits_2.h
--- 5.0.3-1/include/CGAL/Arr_unb_planar_topology_traits_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_unb_planar_topology_traits_2.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_unb_planar_topology_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_unb_planar_topology_traits_2.h $
 // $Id: Arr_unb_planar_topology_traits_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_vertex_index_map.h 5.1-1/include/CGAL/Arr_vertex_index_map.h
--- 5.0.3-1/include/CGAL/Arr_vertex_index_map.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_vertex_index_map.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_vertex_index_map.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_vertex_index_map.h $
 // $Id: Arr_vertex_index_map.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_vertex_map.h 5.1-1/include/CGAL/Arr_vertex_map.h
--- 5.0.3-1/include/CGAL/Arr_vertex_map.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_vertex_map.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_vertex_map.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_vertex_map.h $
 // $Id: Arr_vertex_map.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_vertical_decomposition_2.h 5.1-1/include/CGAL/Arr_vertical_decomposition_2.h
--- 5.0.3-1/include/CGAL/Arr_vertical_decomposition_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_vertical_decomposition_2.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_vertical_decomposition_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_vertical_decomposition_2.h $
 // $Id: Arr_vertical_decomposition_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Arr_walk_along_line_point_location.h 5.1-1/include/CGAL/Arr_walk_along_line_point_location.h
--- 5.0.3-1/include/CGAL/Arr_walk_along_line_point_location.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Arr_walk_along_line_point_location.h	2020-09-08 12:14:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Arr_walk_along_line_point_location.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Arr_walk_along_line_point_location.h $
 // $Id: Arr_walk_along_line_point_location.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/assertions_behaviour.h 5.1-1/include/CGAL/assertions_behaviour.h
--- 5.0.3-1/include/CGAL/assertions_behaviour.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/assertions_behaviour.h	2020-09-08 12:15:32.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/assertions_behaviour.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/assertions_behaviour.h $
 // $Id: assertions_behaviour.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/assertions.h 5.1-1/include/CGAL/assertions.h
--- 5.0.3-1/include/CGAL/assertions.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/assertions.h	2020-09-08 12:15:32.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/assertions.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/assertions.h $
 // $Id: assertions.h 822bc55 2020-03-27T08:28:48+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/assertions_impl.h 5.1-1/include/CGAL/assertions_impl.h
--- 5.0.3-1/include/CGAL/assertions_impl.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/assertions_impl.h	2020-09-08 12:15:32.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/assertions_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/assertions_impl.h $
 // $Id: assertions_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/atomic.h 5.1-1/include/CGAL/atomic.h
--- 5.0.3-1/include/CGAL/atomic.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/atomic.h	2020-09-08 12:14:54.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/atomic.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/atomic.h $
 // $Id: atomic.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 
diff -pruN 5.0.3-1/include/CGAL/auto_link/auto_link.h 5.1-1/include/CGAL/auto_link/auto_link.h
--- 5.0.3-1/include/CGAL/auto_link/auto_link.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/auto_link/auto_link.h	2020-09-08 12:14:54.000000000 +0000
@@ -9,7 +9,7 @@
 //  (C) Copyright John Maddock 2003.
 //
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/auto_link/auto_link.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/auto_link/auto_link.h $
 // $Id: auto_link.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: BSL-1.0
 //
diff -pruN 5.0.3-1/include/CGAL/auto_link/CGAL.h 5.1-1/include/CGAL/auto_link/CGAL.h
--- 5.0.3-1/include/CGAL/auto_link/CGAL.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/auto_link/CGAL.h	2020-09-08 12:14:54.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/auto_link/CGAL.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/auto_link/CGAL.h $
 // $Id: CGAL.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/auto_link/CORE.h 5.1-1/include/CGAL/auto_link/CORE.h
--- 5.0.3-1/include/CGAL/auto_link/CORE.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/auto_link/CORE.h	2020-09-08 12:14:54.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/auto_link/CORE.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/auto_link/CORE.h $
 // $Id: CORE.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/auto_link/ImageIO.h 5.1-1/include/CGAL/auto_link/ImageIO.h
--- 5.0.3-1/include/CGAL/auto_link/ImageIO.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/auto_link/ImageIO.h	2020-09-08 12:14:54.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/auto_link/ImageIO.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/auto_link/ImageIO.h $
 // $Id: ImageIO.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/auto_link/Qt.h 5.1-1/include/CGAL/auto_link/Qt.h
--- 5.0.3-1/include/CGAL/auto_link/Qt.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/auto_link/Qt.h	2020-09-08 12:14:53.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/auto_link/Qt.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/auto_link/Qt.h $
 // $Id: Qt.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/barycenter.h 5.1-1/include/CGAL/barycenter.h
--- 5.0.3-1/include/CGAL/barycenter.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/barycenter.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Principal_component_analysis_LGPL/include/CGAL/barycenter.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Principal_component_analysis_LGPL/include/CGAL/barycenter.h $
 // $Id: barycenter.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Barycentric_coordinates_2/barycentric_enum_2.h 5.1-1/include/CGAL/Barycentric_coordinates_2/barycentric_enum_2.h
--- 5.0.3-1/include/CGAL/Barycentric_coordinates_2/barycentric_enum_2.h	2020-08-01 19:02:15.000000000 +0000
+++ 5.1-1/include/CGAL/Barycentric_coordinates_2/barycentric_enum_2.h	2020-09-08 12:14:41.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is a part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Barycentric_coordinates_2/include/CGAL/Barycentric_coordinates_2/barycentric_enum_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Barycentric_coordinates_2/include/CGAL/Barycentric_coordinates_2/barycentric_enum_2.h $
 // $Id: barycentric_enum_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Barycentric_coordinates_2/Discrete_harmonic_2.h 5.1-1/include/CGAL/Barycentric_coordinates_2/Discrete_harmonic_2.h
--- 5.0.3-1/include/CGAL/Barycentric_coordinates_2/Discrete_harmonic_2.h	2020-08-01 19:02:15.000000000 +0000
+++ 5.1-1/include/CGAL/Barycentric_coordinates_2/Discrete_harmonic_2.h	2020-09-08 12:14:40.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is a part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Barycentric_coordinates_2/include/CGAL/Barycentric_coordinates_2/Discrete_harmonic_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Barycentric_coordinates_2/include/CGAL/Barycentric_coordinates_2/Discrete_harmonic_2.h $
 // $Id: Discrete_harmonic_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Barycentric_coordinates_2/Generalized_barycentric_coordinates_2.h 5.1-1/include/CGAL/Barycentric_coordinates_2/Generalized_barycentric_coordinates_2.h
--- 5.0.3-1/include/CGAL/Barycentric_coordinates_2/Generalized_barycentric_coordinates_2.h	2020-08-01 19:02:15.000000000 +0000
+++ 5.1-1/include/CGAL/Barycentric_coordinates_2/Generalized_barycentric_coordinates_2.h	2020-09-08 12:14:40.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is a part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Barycentric_coordinates_2/include/CGAL/Barycentric_coordinates_2/Generalized_barycentric_coordinates_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Barycentric_coordinates_2/include/CGAL/Barycentric_coordinates_2/Generalized_barycentric_coordinates_2.h $
 // $Id: Generalized_barycentric_coordinates_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Barycentric_coordinates_2/Mean_value_2.h 5.1-1/include/CGAL/Barycentric_coordinates_2/Mean_value_2.h
--- 5.0.3-1/include/CGAL/Barycentric_coordinates_2/Mean_value_2.h	2020-08-01 19:02:15.000000000 +0000
+++ 5.1-1/include/CGAL/Barycentric_coordinates_2/Mean_value_2.h	2020-09-08 12:14:40.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is a part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Barycentric_coordinates_2/include/CGAL/Barycentric_coordinates_2/Mean_value_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Barycentric_coordinates_2/include/CGAL/Barycentric_coordinates_2/Mean_value_2.h $
 // $Id: Mean_value_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Barycentric_coordinates_2/Segment_coordinates_2.h 5.1-1/include/CGAL/Barycentric_coordinates_2/Segment_coordinates_2.h
--- 5.0.3-1/include/CGAL/Barycentric_coordinates_2/Segment_coordinates_2.h	2020-08-01 19:02:15.000000000 +0000
+++ 5.1-1/include/CGAL/Barycentric_coordinates_2/Segment_coordinates_2.h	2020-09-08 12:14:40.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is a part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Barycentric_coordinates_2/include/CGAL/Barycentric_coordinates_2/Segment_coordinates_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Barycentric_coordinates_2/include/CGAL/Barycentric_coordinates_2/Segment_coordinates_2.h $
 // $Id: Segment_coordinates_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Barycentric_coordinates_2/Triangle_coordinates_2.h 5.1-1/include/CGAL/Barycentric_coordinates_2/Triangle_coordinates_2.h
--- 5.0.3-1/include/CGAL/Barycentric_coordinates_2/Triangle_coordinates_2.h	2020-08-01 19:02:15.000000000 +0000
+++ 5.1-1/include/CGAL/Barycentric_coordinates_2/Triangle_coordinates_2.h	2020-09-08 12:14:40.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is a part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Barycentric_coordinates_2/include/CGAL/Barycentric_coordinates_2/Triangle_coordinates_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Barycentric_coordinates_2/include/CGAL/Barycentric_coordinates_2/Triangle_coordinates_2.h $
 // $Id: Triangle_coordinates_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Barycentric_coordinates_2/Wachspress_2.h 5.1-1/include/CGAL/Barycentric_coordinates_2/Wachspress_2.h
--- 5.0.3-1/include/CGAL/Barycentric_coordinates_2/Wachspress_2.h	2020-08-01 19:02:15.000000000 +0000
+++ 5.1-1/include/CGAL/Barycentric_coordinates_2/Wachspress_2.h	2020-09-08 12:14:41.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is a part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Barycentric_coordinates_2/include/CGAL/Barycentric_coordinates_2/Wachspress_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Barycentric_coordinates_2/include/CGAL/Barycentric_coordinates_2/Wachspress_2.h $
 // $Id: Wachspress_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Barycentric_coordinates_2.h 5.1-1/include/CGAL/Barycentric_coordinates_2.h
--- 5.0.3-1/include/CGAL/Barycentric_coordinates_2.h	2020-08-01 19:02:15.000000000 +0000
+++ 5.1-1/include/CGAL/Barycentric_coordinates_2.h	2020-09-08 12:14:40.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is a part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Barycentric_coordinates_2/include/CGAL/Barycentric_coordinates_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Barycentric_coordinates_2/include/CGAL/Barycentric_coordinates_2.h $
 // $Id: Barycentric_coordinates_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/basic_classes.h 5.1-1/include/CGAL/basic_classes.h
--- 5.0.3-1/include/CGAL/basic_classes.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/basic_classes.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/basic_classes.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/basic_classes.h $
 // $Id: basic_classes.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/basic_constructions_2.h 5.1-1/include/CGAL/basic_constructions_2.h
--- 5.0.3-1/include/CGAL/basic_constructions_2.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/basic_constructions_2.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/basic_constructions_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/basic_constructions_2.h $
 // $Id: basic_constructions_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/basic_constructions_3.h 5.1-1/include/CGAL/basic_constructions_3.h
--- 5.0.3-1/include/CGAL/basic_constructions_3.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/basic_constructions_3.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/basic_constructions_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/basic_constructions_3.h $
 // $Id: basic_constructions_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/basic.h 5.1-1/include/CGAL/basic.h
--- 5.0.3-1/include/CGAL/basic.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/basic.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/basic.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/basic.h $
 // $Id: basic.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Bbox_2.h 5.1-1/include/CGAL/Bbox_2.h
--- 5.0.3-1/include/CGAL/Bbox_2.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Bbox_2.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Bbox_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Bbox_2.h $
 // $Id: Bbox_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Bbox_3.h 5.1-1/include/CGAL/Bbox_3.h
--- 5.0.3-1/include/CGAL/Bbox_3.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Bbox_3.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Bbox_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Bbox_3.h $
 // $Id: Bbox_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Bigfloat_interval_traits.h 5.1-1/include/CGAL/Bigfloat_interval_traits.h
--- 5.0.3-1/include/CGAL/Bigfloat_interval_traits.h	2020-08-01 19:02:25.000000000 +0000
+++ 5.1-1/include/CGAL/Bigfloat_interval_traits.h	2020-09-08 12:14:59.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Interval_support/include/CGAL/Bigfloat_interval_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Interval_support/include/CGAL/Bigfloat_interval_traits.h $
 // $Id: Bigfloat_interval_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/bilateral_smooth_point_set.h 5.1-1/include/CGAL/bilateral_smooth_point_set.h
--- 5.0.3-1/include/CGAL/bilateral_smooth_point_set.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/bilateral_smooth_point_set.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_processing_3/include/CGAL/bilateral_smooth_point_set.h $
-// $Id: bilateral_smooth_point_set.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/bilateral_smooth_point_set.h $
+// $Id: bilateral_smooth_point_set.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s) : Shihao Wu, Clement Jamin, Pierre Alliez
@@ -16,18 +16,20 @@
 
 #include <CGAL/disable_warnings.h>
 
-#include <CGAL/Search_traits_3.h>
-#include <CGAL/Orthogonal_k_neighbor_search.h>
-#include <CGAL/Point_set_processing_3/internal/neighbor_query.h>
+#include <CGAL/number_type_config.h>
+#include <CGAL/Point_set_processing_3/internal/Neighbor_query.h>
+#include <CGAL/Point_set_processing_3/internal/Callback_wrapper.h>
+#include <CGAL/for_each.h>
 #include <CGAL/property_map.h>
 #include <CGAL/point_set_processing_assertions.h>
-#include <CGAL/Point_with_normal_3.h>
 #include <CGAL/squared_distance_3.h>
 #include <functional>
 
 #include <CGAL/boost/graph/Named_function_parameters.h>
 #include <CGAL/boost/graph/named_params_helper.h>
 
+#include <boost/iterator/zip_iterator.hpp>
+
 #include <iterator>
 #include <set>
 #include <algorithm>
@@ -37,23 +39,6 @@
 #include <CGAL/Memory_sizer.h>
 #include <CGAL/property_map.h>
 
-#ifdef CGAL_LINKED_WITH_TBB
-
-#include <CGAL/Point_set_processing_3/internal/Parallel_callback.h>
-#include <tbb/parallel_for.h>
-#include <tbb/blocked_range.h>
-#include <tbb/scalable_allocator.h>
-#include <atomic>
-#endif // CGAL_LINKED_WITH_TBB
-
-// Default allocator: use TBB allocators if available
-#ifdef CGAL_LINKED_WITH_TBB
-# define CGAL_PSP3_DEFAULT_ALLOCATOR tbb::scalable_allocator
-#else // CGAL_LINKED_WITH_TBB
-# define CGAL_PSP3_DEFAULT_ALLOCATOR std::allocator
-#endif // CGAL_LINKED_WITH_TBB
-
-
 //#define CGAL_PSP3_VERBOSE
 
 namespace CGAL {
@@ -65,47 +50,6 @@ namespace CGAL {
 
 namespace bilateral_smooth_point_set_internal{
 
-// Item in the Kd-tree: position (Point_3) + index
-template <typename Kernel>
-class Kd_tree_element : public Point_with_normal_3<Kernel>
-{
-public:
-  unsigned int index;
-
-  // basic geometric types
-  typedef typename CGAL::Origin Origin;
-  typedef CGAL::Point_with_normal_3<Kernel> Base;
-
-  Kd_tree_element(const Origin& o = ORIGIN, unsigned int id=0)
-    : Base(o), index(id)
-  {}
-  Kd_tree_element(const Base& p, unsigned int id=0)
-    : Base(p), index(id)
-  {}
-  Kd_tree_element(const Kd_tree_element& other)
-    : Base(other), index(other.index)
-  {}
-
-  Kd_tree_element& operator=(const Kd_tree_element&)=default;
-};
-
-
-// Helper class for the Kd-tree
-template <typename Kernel>
-class Kd_tree_gt : public Kernel
-{
-public:
-  typedef Kd_tree_element<Kernel> Point_3;
-};
-
-template <typename Kernel>
-class Kd_tree_traits : public CGAL::Search_traits_3<Kd_tree_gt<Kernel> >
-{
-public:
-  typedef typename Kernel::Point_3 PointType;
-};
-
-
 /// Compute bilateral projection for each point
 /// according to their KNN neighborhood points
 ///
@@ -116,12 +60,14 @@ public:
 ///
 /// @return
 
-template <typename Kernel>
-CGAL::Point_with_normal_3<Kernel>
+template <typename Kernel, typename PointRange,
+          typename PointMap, typename VectorMap>
+std::pair<typename Kernel::Point_3, typename Kernel::Vector_3>
 compute_denoise_projection(
-  const CGAL::Point_with_normal_3<Kernel>& query, ///< 3D point to project
-  const std::vector<CGAL::Point_with_normal_3<Kernel>,
-  CGAL_PSP3_DEFAULT_ALLOCATOR<CGAL::Point_with_normal_3<Kernel> > >& neighbor_pwns,  //
+  const typename PointRange::iterator::value_type& vt,
+  PointMap point_map,
+  VectorMap normal_map,
+  const std::vector<typename PointRange::iterator>& neighbor_pwns,
   typename Kernel::FT radius,                   ///< accept neighborhood radius
   typename Kernel::FT sharpness_angle           ///< control sharpness(0-90)
 )
@@ -132,7 +78,6 @@ compute_denoise_projection(
 
   // basic geometric types
   typedef typename Kernel::FT FT;
-  typedef CGAL::Point_with_normal_3<Kernel> Pwn;
   typedef typename Kernel::Vector_3 Vector;
   typedef typename Kernel::Point_3 Point;
 
@@ -144,26 +89,24 @@ compute_denoise_projection(
   FT project_weight_sum = FT(0.0);
   Vector normal_sum = CGAL::NULL_VECTOR;
 
-  FT cos_sigma = cos(sharpness_angle / 180.0 * 3.1415926);
+  FT cos_sigma = cos(sharpness_angle * CGAL_PI / 180.0);
   FT sharpness_bandwidth = std::pow((CGAL::max)(1e-8, 1 - cos_sigma), 2);
 
-  typename std::vector<Pwn,CGAL_PSP3_DEFAULT_ALLOCATOR<Pwn> >::const_iterator
-    pwn_iter = neighbor_pwns.begin();
-  for (; pwn_iter != neighbor_pwns.end(); ++pwn_iter)
+  for (typename PointRange::iterator it : neighbor_pwns)
   {
-    const Point& np = pwn_iter->position();
-    const Vector& nn = pwn_iter->normal();
+    const Point& np = get(point_map, *it);
+    const Vector& nn = get(normal_map, *it);
 
-    FT dist2 = CGAL::squared_distance(query.position(), np);
+    FT dist2 = CGAL::squared_distance(get(point_map, vt), np);
     if (dist2 < radius2)
     {
       FT theta = std::exp(dist2 * iradius16);
-      FT psi = std::exp(-std::pow(1 - query.normal() * nn, 2)
+      FT psi = std::exp(-std::pow(1 - get(normal_map, vt) * nn, 2)
         / sharpness_bandwidth);
 
       weight = theta * psi;
 
-      project_dist_sum += ((query.position() - np) * nn) * weight;
+      project_dist_sum += ((get(point_map, vt) - np) * nn) * weight;
       project_weight_sum += weight;
       normal_sum = normal_sum + nn * weight;
     }
@@ -172,10 +115,10 @@ compute_denoise_projection(
   Vector update_normal = normal_sum / project_weight_sum;
   update_normal = update_normal / sqrt(update_normal.squared_length());
 
-  Point update_point = query.position() - update_normal *
+  Point update_point = get(point_map, vt) - update_normal *
                       (project_dist_sum / project_weight_sum);
 
-  return Pwn(update_point, update_normal);
+  return std::make_pair (update_point, update_normal);
 }
 
 /// Computes max-spacing of one query point from K nearest neighbors.
@@ -186,41 +129,30 @@ compute_denoise_projection(
 /// @tparam Tree KD-tree.
 ///
 /// @return max spacing.
-template < typename Kernel,
-           typename Tree >
-typename Kernel::FT
+template <typename NeighborQuery>
+typename NeighborQuery::Kernel::FT
 compute_max_spacing(
-  const CGAL::Point_with_normal_3<Kernel>& query, ///< 3D point
-  Tree& tree,                                     ///< KD-tree
+  const typename NeighborQuery::value_type& vt,
+  typename NeighborQuery::Point_map point_map,
+  NeighborQuery& neighbor_query,                                     ///< KD-tree
   unsigned int k)                                 ///< number of neighbors
 {
   // basic geometric types
+  typedef typename NeighborQuery::Kernel Kernel;
   typedef typename Kernel::FT FT;
-  typedef CGAL::Point_with_normal_3<Kernel> Pwn;
-
-  // types for K nearest neighbors search
-  typedef bilateral_smooth_point_set_internal::Kd_tree_traits<Kernel> Tree_traits;
-  typedef CGAL::Orthogonal_k_neighbor_search<Tree_traits> Neighbor_search;
-  typedef typename Neighbor_search::iterator Search_iterator;
 
   // performs k + 1 queries (if unique the query point is
   // output first). search may be aborted when k is greater
   // than number of input points
-  Neighbor_search search(tree,query,k+1);
-  Search_iterator search_iterator = search.begin();
-  ++search_iterator;
   FT max_distance = (FT)0.0;
-  unsigned int i;
-  for(i = 0; i < (k+1) ; ++i)
-  {
-    if(search_iterator == search.end())
-      break; // premature ending
-
-    Pwn pwn = search_iterator->first;
-    double dist2 = CGAL::squared_distance(query.position(), pwn.position());
-    max_distance = (CGAL::max)(dist2, max_distance);
-    ++search_iterator;
-  }
+  neighbor_query.get_iterators
+    (get(point_map, vt), k, (FT)(0.0),
+     boost::make_function_output_iterator
+     ([&](const typename NeighborQuery::input_iterator& it)
+      {
+        double dist2 = CGAL::squared_distance (get(point_map, vt), get(point_map, *it));
+        max_distance = (CGAL::max)(dist2, max_distance);
+      }));
 
   // output max spacing
   return std::sqrt(max_distance);
@@ -230,102 +162,6 @@ compute_max_spacing(
 
 /// \endcond
 
-#ifdef CGAL_LINKED_WITH_TBB
-/// \cond SKIP_IN_MANUAL
-/// This is for parallelization of function: bilateral_smooth_point_set()
-template <typename Kernel, typename Tree>
-class Compute_pwns_neighbors
-{
-  typedef typename CGAL::Point_with_normal_3<Kernel> Pwn;
-  typedef typename std::vector<Pwn,CGAL_PSP3_DEFAULT_ALLOCATOR<Pwn> > Pwns;
-  typedef typename std::vector<Pwns,CGAL_PSP3_DEFAULT_ALLOCATOR<Pwns> >
-                                                                Pwns_neighbors;
-  typedef typename Kernel::FT FT;
-
-  unsigned int                                              m_k;
-  FT                                                        m_neighbor_radius;
-  const Tree                                              & m_tree;
-  const Pwns                                              & m_pwns;
-  Pwns_neighbors                                          & m_pwns_neighbors;
-  cpp11::atomic<std::size_t>& advancement;
-  cpp11::atomic<bool>& interrupted;
-
-public:
-  Compute_pwns_neighbors(unsigned int k, FT neighbor_radius, const Tree &tree,
-                         const Pwns &pwns, Pwns_neighbors &neighbors,
-                         cpp11::atomic<std::size_t>& advancement,
-                         cpp11::atomic<bool>& interrupted)
-    : m_k(k), m_neighbor_radius (neighbor_radius), m_tree(tree)
-    , m_pwns(pwns), m_pwns_neighbors(neighbors)
-    , advancement (advancement), interrupted (interrupted) {}
-
-  void operator() ( const tbb::blocked_range<size_t>& r ) const
-  {
-    for (size_t i = r.begin(); i!=r.end(); i++)
-    {
-      if (interrupted)
-        break;
-
-      CGAL::Point_set_processing_3::internal::neighbor_query
-        (m_pwns[i], m_tree, m_k, m_neighbor_radius, m_pwns_neighbors[i]);
-
-      ++ advancement;
-    }
-  }
-};
-/// \endcond
-
-/// \cond SKIP_IN_MANUAL
-/// This is for parallelization of function: compute_denoise_projection()
-template <typename Kernel>
-class Pwn_updater
-{
-  typedef typename CGAL::Point_with_normal_3<Kernel> Pwn;
-  typedef typename std::vector<Pwn,CGAL_PSP3_DEFAULT_ALLOCATOR<Pwn> > Pwns;
-  typedef typename Kernel::FT FT;
-
-  FT sharpness_angle;
-  FT radius;
-  Pwns* pwns;
-  Pwns* update_pwns;
-  std::vector<Pwns,CGAL_PSP3_DEFAULT_ALLOCATOR<Pwns> >* pwns_neighbors;
-  cpp11::atomic<std::size_t>& advancement;
-  cpp11::atomic<bool>& interrupted;
-
-public:
-  Pwn_updater(FT sharpness,
-    FT r,
-    Pwns *in,
-    Pwns *out,
-    std::vector<Pwns,CGAL_PSP3_DEFAULT_ALLOCATOR<Pwns> >* neighbors,
-    cpp11::atomic<std::size_t>& advancement,
-    cpp11::atomic<bool>& interrupted):
-  sharpness_angle(sharpness),
-    radius(r),
-    pwns(in),
-    update_pwns(out),
-    pwns_neighbors(neighbors),
-    advancement (advancement),
-    interrupted (interrupted) {}
-
-  void operator() ( const tbb::blocked_range<size_t>& r ) const
-  {
-    for (size_t i = r.begin(); i != r.end(); ++i)
-    {
-      if (interrupted)
-        break;
-      (*update_pwns)[i] = bilateral_smooth_point_set_internal::
-        compute_denoise_projection<Kernel>((*pwns)[i],
-        (*pwns_neighbors)[i],
-        radius,
-        sharpness_angle);
-      ++ advancement;
-    }
-  }
-};
-/// \endcond
-#endif // CGAL_LINKED_WITH_TBB
-
 
 // ----------------------------------------------------------------------------
 // Public section
@@ -350,37 +186,70 @@ public:
    \pre Normals must be unit vectors
    \pre k >= 2
 
-   \tparam ConcurrencyTag enables sequential versus parallel algorithm.
-   Possible values are `Sequential_tag`
-   And `Parallel_tag`.
+   \tparam ConcurrencyTag enables sequential versus parallel algorithm. Possible values are `Sequential_tag`,
+                          `Parallel_tag`, and `Parallel_if_available_tag`.
    \tparam PointRange is a model of `Range`. The value type of
    its iterator is the key type of the named parameter `point_map`.
 
    \param points input point range.
    \param k size of the neighborhood for the implicit surface patch fitting.
    The larger the value is, the smoother the result will be.
-   \param np optional sequence of \ref psp_namedparameters "Named Parameters" among the ones listed below.
+   \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
    \cgalNamedParamsBegin
-     \cgalParamBegin{point_map} a model of `ReadWritePropertyMap` with value type `geom_traits::Point_3`.
-     If this parameter is omitted, `CGAL::Identity_property_map<geom_traits::Point_3>` is used.\cgalParamEnd
-     \cgalParamBegin{normal_map} a model of `ReadWritePropertyMap` with value type
-     `geom_traits::Vector_3`.\cgalParamEnd
-     \cgalParamBegin{neighbor_radius} spherical neighborhood radius. If
-     provided, the neighborhood of a query point is computed with a fixed spherical
-     radius instead of a fixed number of neighbors. In that case, the parameter
-     `k` is used as a limit on the number of points returned by each spherical
-     query (to avoid overly large number of points in high density areas). If no
-     limit is wanted, use `k=0`.\cgalParamEnd
-     \cgalParamBegin{sharpness_angle} controls the sharpness of the result.\cgalParamEnd
-     \cgalParamBegin{callback} an instance of
-      `std::function<bool(double)>`. It is called regularly when the
-      algorithm is running: the current advancement (between 0. and
-      1.) is passed as parameter. If it returns `true`, then the
-      algorithm continues its execution normally; if it returns
-      `false`, the algorithm is stopped, all points are left unchanged
-      and the function return `NaN`.\cgalParamEnd
-     \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+     \cgalParamNBegin{point_map}
+       \cgalParamDescription{a property map associating points to the elements of the point set `points`}
+       \cgalParamType{a model of `ReadWritePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange` and whose value type is `geom_traits::Point_3`}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{normal_map}
+       \cgalParamDescription{a property map associating normals to the elements of the point set `points`}
+       \cgalParamType{a model of `ReadWritePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange` and whose value type is `geom_traits::Vector_3`}
+       \cgalParamDefault{Normals are computed and stored internally.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{neighbor_radius}
+       \cgalParamDescription{the spherical neighborhood radius}
+       \cgalParamType{floating scalar value}
+       \cgalParamDefault{`0` (no limit)}
+       \cgalParamExtra{If provided, the neighborhood of a query point is computed with a fixed spherical
+                       radius instead of a fixed number of neighbors. In that case, the parameter
+                       `k` is used as a limit on the number of points returned by each spherical
+                       query (to avoid overly large number of points in high density areas).}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{sharpness_angle}
+       \cgalParamDescription{controls the sharpness of the result}
+       \cgalParamType{floating scalar value}
+       \cgalParamDefault{`30`}
+       \cgalParamExtra{The larger the value is, the smoother the result will be.
+                       The range of possible value is `[0, 90]`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{callback}
+       \cgalParamDescription{a mechanism to get feedback on the advancement of the algorithm
+                             while it's running and to interrupt it if needed}
+       \cgalParamType{an instance of `std::function<bool(double)>`.}
+       \cgalParamDefault{unused}
+       \cgalParamExtra{It is called regularly when the
+                       algorithm is running: the current advancement (between 0. and
+                       1.) is passed as parameter. If it returns `true`, then the
+                       algorithm continues its execution normally; if it returns
+                       `false`, the algorithm is stopped, all points are left unchanged
+                       and the function return `NaN`.}
+       \cgalParamExtra{The callback will be copied and therefore needs to be lightweight.}
+       \cgalParamExtra{When `CGAL::Parallel_tag` is used, the `callback` mechanism is called asynchronously
+                       on a separate thread and shouldn't access or modify the variables that are parameters of the algorithm.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{geom_traits}
+       \cgalParamDescription{an instance of a geometric traits class}
+       \cgalParamType{a model of `Kernel`}
+       \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+     \cgalParamNEnd
    \cgalNamedParamsEnd
 
    \return Average point movement error. It's a convergence criterium for the algorithm.
@@ -400,16 +269,18 @@ bilateral_smooth_point_set(
   using parameters::get_parameter;
 
   // basic geometric types
-  typedef typename Point_set_processing_3::GetPointMap<PointRange, NamedParameters>::type PointMap;
+  typedef typename PointRange::iterator iterator;
+  typedef typename iterator::value_type value_type;
+  typedef typename CGAL::GetPointMap<PointRange, NamedParameters>::type PointMap;
   typedef typename Point_set_processing_3::GetNormalMap<PointRange, NamedParameters>::type NormalMap;
   typedef typename Point_set_processing_3::GetK<PointRange, NamedParameters>::Kernel Kernel;
+  typedef typename Kernel::Point_3 Point_3;
+  typedef typename Kernel::Vector_3 Vector_3;
 
   CGAL_static_assertion_msg(!(boost::is_same<NormalMap,
                               typename Point_set_processing_3::GetNormalMap<PointRange, NamedParameters>::NoMap>::value),
                             "Error: no normal map");
 
-  typedef typename CGAL::Point_with_normal_3<Kernel> Pwn;
-  typedef typename std::vector<Pwn,CGAL_PSP3_DEFAULT_ALLOCATOR<Pwn> > Pwns;
   typedef typename Kernel::FT FT;
 
   double sharpness_angle = choose_parameter(get_parameter(np, internal_np::sharpness_angle), 30.);
@@ -420,43 +291,20 @@ bilateral_smooth_point_set(
   CGAL_point_set_processing_precondition(k > 1);
 
   // types for K nearest neighbors search structure
-  typedef bilateral_smooth_point_set_internal::
-                                       Kd_tree_element<Kernel> Kd_tree_element;
-  typedef bilateral_smooth_point_set_internal::Kd_tree_traits<Kernel> Tree_traits;
-  typedef CGAL::Orthogonal_k_neighbor_search<Tree_traits> Neighbor_search;
-  typedef typename Neighbor_search::Tree Tree;
+  typedef Point_set_processing_3::internal::Neighbor_query<Kernel, PointRange&, PointMap> Neighbor_query;
 
-  PointMap point_map = choose_parameter(get_parameter(np, internal_np::point_map), PointMap());
-  NormalMap normal_map = choose_parameter(get_parameter(np, internal_np::normal_map), NormalMap());
+  PointMap point_map = choose_parameter<PointMap>(get_parameter(np, internal_np::point_map));
+  NormalMap normal_map = choose_parameter<NormalMap>(get_parameter(np, internal_np::normal_map));
   FT neighbor_radius = choose_parameter(get_parameter(np, internal_np::neighbor_radius), FT(0));
 
-  // copy points and normals
-  Pwns pwns;
-  for(typename PointRange::iterator it = points.begin(); it != points.end(); ++it)
-  {
-    typename boost::property_traits<PointMap>::reference p = get(point_map, *it);
-    typename boost::property_traits<NormalMap>::reference n = get(normal_map, *it);
-    CGAL_point_set_processing_precondition(n.squared_length() > 1e-10);
-
-    pwns.push_back(Pwn(p, n));
-  }
-
-  std::size_t nb_points = pwns.size();
+  std::size_t nb_points = points.size();
 
 #ifdef CGAL_PSP3_VERBOSE
    std::cout << "Initialization and compute max spacing: " << std::endl;
 #endif
    // initiate a KD-tree search for points
-   std::vector<Kd_tree_element,
-     CGAL_PSP3_DEFAULT_ALLOCATOR<Kd_tree_element> > treeElements;
-   treeElements.reserve(pwns.size());
-   typename std::vector<Pwn,CGAL_PSP3_DEFAULT_ALLOCATOR<Pwn> >::iterator
-     pwn_iter = pwns.begin();
-   for (unsigned int i = 0; pwn_iter != pwns.end(); ++pwn_iter)
-   {
-     treeElements.push_back(Kd_tree_element(*pwn_iter, i));
-   }
-   Tree tree(treeElements.begin(), treeElements.end());
+   Neighbor_query neighbor_query (points, point_map);
+
    // Guess spacing
 #ifdef CGAL_PSP3_VERBOSE
    CGAL::Real_timer task_timer;
@@ -464,10 +312,10 @@ bilateral_smooth_point_set(
 #endif
    FT guess_neighbor_radius = 0.0;
 
-   for(pwn_iter = pwns.begin(); pwn_iter != pwns.end(); ++pwn_iter)
+   for (const value_type& vt : points)
    {
      FT max_spacing = bilateral_smooth_point_set_internal::
-       compute_max_spacing<Kernel,Tree>(*pwn_iter, tree, k);
+       compute_max_spacing (vt, point_map, neighbor_query, k);
      guess_neighbor_radius = (CGAL::max)(max_spacing, guess_neighbor_radius);
    }
 
@@ -486,49 +334,40 @@ bilateral_smooth_point_set(
    task_timer.start();
 #endif
    // compute all neighbors
-   std::vector<Pwns,CGAL_PSP3_DEFAULT_ALLOCATOR<Pwns> > pwns_neighbors;
+   typedef std::vector<iterator> iterators;
+   std::vector<iterators> pwns_neighbors;
    pwns_neighbors.resize(nb_points);
 
-#ifndef CGAL_LINKED_WITH_TBB
-  CGAL_static_assertion_msg (!(boost::is_convertible<ConcurrencyTag, Parallel_tag>::value),
-                             "Parallel_tag is enabled but TBB is unavailable.");
-#else
-   if (boost::is_convertible<ConcurrencyTag,Parallel_tag>::value)
-   {
-     Point_set_processing_3::internal::Parallel_callback
-       parallel_callback (callback, 2 * nb_points);
+   Point_set_processing_3::internal::Callback_wrapper<ConcurrencyTag>
+     callback_wrapper (callback, 2 * nb_points);
 
-     Compute_pwns_neighbors<Kernel, Tree> f(k, neighbor_radius, tree, pwns, pwns_neighbors,
-                                            parallel_callback.advancement(),
-                                            parallel_callback.interrupted());
-     tbb::parallel_for(tbb::blocked_range<size_t>(0, nb_points), f);
-
-     bool interrupted = parallel_callback.interrupted();
-
-     // We interrupt by hand as counter only goes halfway and won't terminate by itself
-     parallel_callback.interrupted() = true;
-     parallel_callback.join();
-
-     // If interrupted during this step, nothing is computed, we return NaN
-     if (interrupted)
-       return std::numeric_limits<double>::quiet_NaN();
-   }
-   else
-#endif
-   {
-     typename std::vector<Pwns,CGAL_PSP3_DEFAULT_ALLOCATOR<Pwns> >::iterator
-       pwns_iter = pwns_neighbors.begin();
+   typedef boost::zip_iterator<boost::tuple<iterator, typename std::vector<iterators>::iterator> > Zip_iterator;
 
-     std::size_t nb = 0;
-     for(pwn_iter = pwns.begin(); pwn_iter != pwns.end(); ++pwn_iter, ++pwns_iter, ++ nb)
-     {
-       CGAL::Point_set_processing_3::internal::neighbor_query
-         (*pwn_iter, tree, k, neighbor_radius, *pwns_iter);
-
-       if (callback && !callback ((nb+1) / double(2. * nb_points)))
-         return std::numeric_limits<double>::quiet_NaN();
-     }
-   }
+   CGAL::for_each<ConcurrencyTag>
+     (CGAL::make_range (boost::make_zip_iterator (boost::make_tuple (points.begin(), pwns_neighbors.begin())),
+                        boost::make_zip_iterator (boost::make_tuple (points.end(), pwns_neighbors.end()))),
+      [&](const typename Zip_iterator::reference& t)
+      {
+        if (callback_wrapper.interrupted())
+          return false;
+
+        neighbor_query.get_iterators (get(point_map, get<0>(t)), k, neighbor_radius,
+                                      std::back_inserter (get<1>(t)));
+
+        ++ callback_wrapper.advancement();
+
+        return true;
+      });
+
+   bool interrupted = callback_wrapper.interrupted();
+
+   // We interrupt by hand as counter only goes halfway and won't terminate by itself
+   callback_wrapper.interrupted() = true;
+   callback_wrapper.join();
+
+   // If interrupted during this step, nothing is computed, we return NaN
+   if (interrupted)
+     return std::numeric_limits<double>::quiet_NaN();
 
 #ifdef CGAL_PSP3_VERBOSE
    task_timer.stop();
@@ -541,53 +380,45 @@ bilateral_smooth_point_set(
    task_timer.start();
 #endif
    // update points and normals
-   Pwns update_pwns(nb_points);
+   std::vector<std::pair<Point_3, Vector_3> > update_pwns(nb_points);
 
-#ifdef CGAL_LINKED_WITH_TBB
-   if(boost::is_convertible<ConcurrencyTag, CGAL::Parallel_tag>::value)
-   {
-     Point_set_processing_3::internal::Parallel_callback
-       parallel_callback (callback, 2 * nb_points, nb_points);
+   callback_wrapper.reset (2 * nb_points, nb_points);
 
-     //tbb::task_scheduler_init init(4);
-     tbb::blocked_range<size_t> block(0, nb_points);
-     Pwn_updater<Kernel> pwn_updater(sharpness_angle,
-                                     guess_neighbor_radius,
-                                     &pwns,
-                                     &update_pwns,
-                                     &pwns_neighbors,
-                                     parallel_callback.advancement(),
-                                     parallel_callback.interrupted());
-     tbb::parallel_for(block, pwn_updater);
-
-     parallel_callback.join();
-
-     // If interrupted during this step, nothing is computed, we return NaN
-     if (parallel_callback.interrupted())
-       return std::numeric_limits<double>::quiet_NaN();
-   }
-   else
-#endif // CGAL_LINKED_WITH_TBB
-   {
-     std::size_t nb = nb_points;
+   typedef boost::zip_iterator
+     <boost::tuple<iterator,
+                   typename std::vector<iterators>::iterator,
+                   typename std::vector<std::pair<Point_3, Vector_3> >::iterator> > Zip_iterator_2;
+
+
+   CGAL::for_each<ConcurrencyTag>
+     (CGAL::make_range (boost::make_zip_iterator (boost::make_tuple
+                                                  (points.begin(), pwns_neighbors.begin(), update_pwns.begin())),
+                        boost::make_zip_iterator (boost::make_tuple
+                                                  (points.end(), pwns_neighbors.end(), update_pwns.end()))),
+      [&](const typename Zip_iterator_2::reference& t)
+      {
+        if (callback_wrapper.interrupted())
+          return false;
+
+        get<2>(t) = bilateral_smooth_point_set_internal::
+          compute_denoise_projection<Kernel, PointRange>
+          (get<0>(t),
+           point_map, normal_map,
+           get<1>(t),
+           guess_neighbor_radius,
+           sharpness_angle);
+
+        ++ callback_wrapper.advancement();
+
+        return true;
+      });
+
+   callback_wrapper.join();
+
+   // If interrupted during this step, nothing is computed, we return NaN
+   if (callback_wrapper.interrupted())
+     return std::numeric_limits<double>::quiet_NaN();
 
-     typename std::vector<Pwn,CGAL_PSP3_DEFAULT_ALLOCATOR<Pwn> >::iterator
-       update_iter = update_pwns.begin();
-     typename std::vector<Pwns,CGAL_PSP3_DEFAULT_ALLOCATOR<Pwns> >::iterator
-       neighbor_iter = pwns_neighbors.begin();
-     for(pwn_iter = pwns.begin(); pwn_iter != pwns.end();
-         ++pwn_iter, ++update_iter, ++neighbor_iter, ++ nb)
-     {
-       *update_iter = bilateral_smooth_point_set_internal::
-         compute_denoise_projection<Kernel>
-         (*pwn_iter,
-          *neighbor_iter,
-          guess_neighbor_radius,
-          sharpness_angle);
-       if (callback && !callback ((nb+1) / double(2. * nb_points)))
-         return std::numeric_limits<double>::quiet_NaN();
-     }
-   }
 #ifdef CGAL_PSP3_VERBOSE
    task_timer.stop();
    memory = CGAL::Memory_sizer().virtual_size();
@@ -596,13 +427,13 @@ bilateral_smooth_point_set(
 #endif
    // save results
    FT sum_move_error = 0;
-   typename PointRange::iterator it = points.begin();
-   for(unsigned int i = 0 ; it != points.end(); ++it, ++i)
+   std::size_t nb = 0;
+   for (value_type& vt : points)
    {
-     typename boost::property_traits<PointMap>::reference p = get(point_map, *it);
-     sum_move_error += CGAL::squared_distance(p, update_pwns[i].position());
-     put (point_map, *it, update_pwns[i].position());
-     put (normal_map, *it, update_pwns[i].normal());
+     sum_move_error += CGAL::squared_distance(get(point_map, vt), update_pwns[nb].first);
+     put (point_map, vt, update_pwns[nb].first);
+     put (normal_map, vt, update_pwns[nb].second);
+     ++ nb;
    }
 
    return sum_move_error / nb_points;
diff -pruN 5.0.3-1/include/CGAL/Boolean_set_operations_2/Bso_internal_functions.h 5.1-1/include/CGAL/Boolean_set_operations_2/Bso_internal_functions.h
--- 5.0.3-1/include/CGAL/Boolean_set_operations_2/Bso_internal_functions.h	2020-08-01 19:02:15.000000000 +0000
+++ 5.1-1/include/CGAL/Boolean_set_operations_2/Bso_internal_functions.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Bso_internal_functions.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Bso_internal_functions.h $
 // $Id: Bso_internal_functions.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Boolean_set_operations_2/Ccb_curve_iterator.h 5.1-1/include/CGAL/Boolean_set_operations_2/Ccb_curve_iterator.h
--- 5.0.3-1/include/CGAL/Boolean_set_operations_2/Ccb_curve_iterator.h	2020-08-01 19:02:15.000000000 +0000
+++ 5.1-1/include/CGAL/Boolean_set_operations_2/Ccb_curve_iterator.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Ccb_curve_iterator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Ccb_curve_iterator.h $
 // $Id: Ccb_curve_iterator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Boolean_set_operations_2/Curve_with_halfedge.h 5.1-1/include/CGAL/Boolean_set_operations_2/Curve_with_halfedge.h
--- 5.0.3-1/include/CGAL/Boolean_set_operations_2/Curve_with_halfedge.h	2020-08-01 19:02:15.000000000 +0000
+++ 5.1-1/include/CGAL/Boolean_set_operations_2/Curve_with_halfedge.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Curve_with_halfedge.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Curve_with_halfedge.h $
 // $Id: Curve_with_halfedge.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_agg_meta_traits.h 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_agg_meta_traits.h
--- 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_agg_meta_traits.h	2020-08-01 19:02:15.000000000 +0000
+++ 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_agg_meta_traits.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,12 +3,13 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_agg_meta_traits.h $
-// $Id: Gps_agg_meta_traits.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_agg_meta_traits.h $
+// $Id: Gps_agg_meta_traits.h 7436c14 2020-04-06T01:23:45+03:00 Efi Fogel
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
-// Author(s)     : Baruch Zukerman <baruchzu@post.tau.ac.il>
+// Author(s): Baruch Zukerman <baruchzu@post.tau.ac.il>
+//            Efi Fogel       <efifogel@gmail.com>
 
 #ifndef CGAL_BSO_2_GPS_AGG_META_TRAITS_H
 #define CGAL_BSO_2_GPS_AGG_META_TRAITS_H
@@ -81,8 +82,8 @@ class Gps_agg_meta_traits :
   typedef typename Arr::Traits_adaptor_2        Traits;
   typedef Traits                                Gt2;
 
-  typedef typename Gt2::X_monotone_curve_2      Base_X_monotone_curve_2;
-  typedef typename Gt2::Point_2                 Base_Point_2;
+  typedef typename Gt2::X_monotone_curve_2      Base_x_monotone_curve_2;
+  typedef typename Gt2::Point_2                 Base_point_2;
   typedef typename Gt2::Construct_min_vertex_2  Base_Construct_min_vertex_2;
   typedef typename Gt2::Construct_max_vertex_2  Base_Construct_max_vertex_2;
   typedef typename Gt2::Compare_endpoints_xy_2  Base_Compare_endpoints_xy_2;
@@ -106,8 +107,8 @@ public:
   typedef Point_with_vertex<Arr>                Point_data;
 
 private:
-  typedef Gps_traits_decorator<Gt2, Curve_data, Point_data>
-                                                Base;
+  typedef Gps_agg_meta_traits<Arrangement>                      Self;
+  typedef Gps_traits_decorator<Gt2, Curve_data, Point_data>     Base;
 
 public:
   typedef typename Base::X_monotone_curve_2     X_monotone_curve_2;
@@ -145,93 +146,88 @@ public:
 
   class Intersect_2 {
   private:
-    Base_Intersect_2 m_base;
-    Base_Compare_endpoints_xy_2 m_base_cmp_endpoints;
-    Base_Compare_xy_2 m_base_cmp_xy;
-    Base_Construct_min_vertex_2 m_base_ctr_min_v;
+    const Self& m_traits;
 
-  public:
-    /*! Construct. */
-    Intersect_2(const Base_Intersect_2& base,
-                const Base_Compare_endpoints_xy_2& base_cmp_endpoints,
-                const Base_Compare_xy_2& base_cmp_xy,
-                const Base_Construct_min_vertex_2& base_ctr_min_v) :
-      m_base(base),
-      m_base_cmp_endpoints(base_cmp_endpoints),
-      m_base_cmp_xy(base_cmp_xy),
-      m_base_ctr_min_v(base_ctr_min_v)
-    {}
+    /*! Constructor. */
+    Intersect_2(const Self& traits) : m_traits(traits) {}
+
+    friend Self;
 
+  public:
     template <typename OutputIterator>
     OutputIterator operator()(const X_monotone_curve_2& cv1,
                               const X_monotone_curve_2& cv2,
                               OutputIterator oi) const
     {
-      if (cv1.data().arr() == cv2.data().arr()) {
-        return oi;      // the curves are disjoint-interior because they
-                        // are already at the same arrangement.
-      }
-
-      const std::pair<Base_Point_2, Multiplicity>* base_pt;
-      const Base_X_monotone_curve_2* overlap_cv;
-      OutputIterator oi_end;
-      if(m_base_cmp_xy(m_base_ctr_min_v(cv1.base()),
-                       m_base_ctr_min_v(cv2.base())) == LARGER)
-        oi_end = m_base(cv1.base(), cv2.base(), oi);
+      // Check whether the curves are already in the same arrangement, and thus
+      // must be interior-disjoint
+      if (cv1.data().arr() == cv2.data().arr()) return oi;
+
+      typedef const std::pair<Base_point_2, Multiplicity>
+        Intersection_base_point;
+      typedef boost::variant<Intersection_base_point, Base_x_monotone_curve_2>
+                                                        Intersection_base_result;
+      typedef const std::pair<Point_2, Multiplicity>    Intersection_point;
+      typedef boost::variant<Intersection_point, X_monotone_curve_2>
+                                                        Intersection_result;
+
+      const auto* base_traits = m_traits.m_base_traits;
+      auto base_cmp_xy = base_traits->compare_xy_2_object();
+      auto base_cmp_endpoints = base_traits->compare_endpoints_xy_2_object();
+      auto base_ctr_min_vertex = base_traits->construct_min_vertex_2_object();
+      auto base_intersect = base_traits->intersect_2_object();
+
+      std::vector<Intersection_base_result> xections;
+      if (base_cmp_xy(base_ctr_min_vertex(cv1.base()),
+                      base_ctr_min_vertex(cv2.base())) == LARGER)
+        base_intersect(cv1.base(), cv2.base(), back_inserter(xections));
       else
-        oi_end = m_base(cv2.base(), cv1.base(), oi);
+        base_intersect(cv2.base(), cv1.base(), back_inserter(xections));
 
-      // convert objects that are associated with Base_X_monotone_curve_2 to
+      // convert objects that are associated with Base_x_monotone_curve_2 to
       // the extenede X_monotone_curve_2
-      for (; oi != oi_end; ++oi) {
-        base_pt = object_cast<std::pair<Base_Point_2, Multiplicity> >(&(*oi));
-
+      for (const auto& xection : xections) {
+        const Intersection_base_point* base_pt =
+          boost::get<Intersection_base_point>(&xection);
         if (base_pt != nullptr) {
           Point_2 point_plus(base_pt->first); // the extended point
-          *oi = CGAL::make_object(std::make_pair(point_plus,
-                                                 base_pt->second));
+          *oi++ =
+            Intersection_result(std::make_pair(point_plus, base_pt->second));
+          continue;
+        }
+
+        const Base_x_monotone_curve_2* overlap_cv =
+          boost::get<Base_x_monotone_curve_2>(&xection);
+        CGAL_assertion(overlap_cv != nullptr);
+        unsigned int ov_bc;
+        unsigned int ov_twin_bc;
+        if (base_cmp_endpoints(cv1) == base_cmp_endpoints(cv2)) {
+          // cv1 and cv2 have the same directions
+          ov_bc = cv1.data().bc() + cv2.data().bc();
+          ov_twin_bc = cv1.data().twin_bc() + cv2.data().twin_bc();
         }
         else {
-          overlap_cv = object_cast<Base_X_monotone_curve_2>(&(*oi));
+          // cv1 and cv2 have opposite directions
+          ov_bc = cv1.data().bc() + cv2.data().twin_bc();
+          ov_twin_bc = cv1.data().twin_bc() + cv2.data().bc();
+        }
 
-          if (overlap_cv != nullptr) {
-            unsigned int ov_bc;
-            unsigned int ov_twin_bc;
-            if (m_base_cmp_endpoints(cv1) == m_base_cmp_endpoints(cv2)) {
-              // cv1 and cv2 have the same directions
-              ov_bc = cv1.data().bc() + cv2.data().bc();
-              ov_twin_bc = cv1.data().twin_bc() + cv2.data().twin_bc();
-            }
-            else {
-              // cv1 and cv2 have opposite directions
-              ov_bc = cv1.data().bc() + cv2.data().twin_bc();
-              ov_twin_bc = cv1.data().twin_bc() + cv2.data().bc();
-            }
-
-            if(m_base_cmp_endpoints(*overlap_cv) != m_base_cmp_endpoints(cv1)) {
-              // overlap_cv, cv1 have opposite directions
-              std::swap(ov_bc, ov_twin_bc);
-            }
-
-            Curve_data cv_data(cv1.data().arr(), Halfedge_handle(),
-                               ov_bc, ov_twin_bc);
-              *oi = CGAL::make_object(X_monotone_curve_2(*overlap_cv, cv_data));
-          }
+        if (base_cmp_endpoints(*overlap_cv) != base_cmp_endpoints(cv1)) {
+          // overlap_cv, cv1 have opposite directions
+          std::swap(ov_bc, ov_twin_bc);
         }
+
+        Curve_data cv_data(cv1.data().arr(), Halfedge_handle(),
+                           ov_bc, ov_twin_bc);
+        *oi++ = Intersection_result(X_monotone_curve_2(*overlap_cv, cv_data));
       }
-      //return past-end iterator
-      return oi_end;
+
+      return oi;
     }
   };
 
   /*! Obtain an Intersect_2 functor object. */
-  Intersect_2 intersect_2_object() const
-  {
-    return Intersect_2(this->m_base_tr->intersect_2_object(),
-                       this->m_base_tr->compare_endpoints_xy_2_object(),
-                       this->m_base_tr->compare_xy_2_object(),
-                       this->m_base_tr->construct_min_vertex_2_object());
-  }
+  Intersect_2 intersect_2_object() const { return Intersect_2(*this); }
 
   class Split_2 {
   private:
@@ -256,7 +252,7 @@ public:
 
   /*! Obtain a Split_2 functor object. */
   Split_2 split_2_object() const
-  { return Split_2(this->m_base_tr->split_2_object()); }
+  { return Split_2(this->m_base_traits->split_2_object()); }
 
   class Construct_min_vertex_2 {
   private:
@@ -286,7 +282,7 @@ public:
   /*! Get a Construct_min_vertex_2 functor object. */
   Construct_min_vertex_2 construct_min_vertex_2_object() const
   {
-    return Construct_min_vertex_2(this->m_base_tr->
+    return Construct_min_vertex_2(this->m_base_traits->
                                   construct_min_vertex_2_object());
   }
 
@@ -318,7 +314,7 @@ public:
   /*! Get a Construct_min_vertex_2 functor object. */
   Construct_max_vertex_2 construct_max_vertex_2_object() const
   {
-    return Construct_max_vertex_2(this->m_base_tr->
+    return Construct_max_vertex_2(this->m_base_traits->
                                   construct_max_vertex_2_object());
   }
 
@@ -348,7 +344,7 @@ public:
 
   /*! Obtain a Construct_min_vertex_2 functor object. */
   Compare_xy_2 compare_xy_2_object() const
-  { return Compare_xy_2(this->m_base_tr->compare_xy_2_object()); }
+  { return Compare_xy_2(this->m_base_traits->compare_xy_2_object()); }
 
   // left-right
   class Parameter_space_in_x_2 {
@@ -380,7 +376,7 @@ public:
   /*! Obtain a Construct_min_vertex_2 functor object. */
   Parameter_space_in_x_2 parameter_space_in_x_2_object() const
   {
-    return Parameter_space_in_x_2(this->m_base_tr->
+    return Parameter_space_in_x_2(this->m_base_traits->
                                   parameter_space_in_x_2_object());
   }
 
@@ -404,7 +400,7 @@ public:
   /*! Obtain a Construct_min_vertex_2 functor object. */
   Compare_y_near_boundary_2 compare_y_near_boundary_2_object() const
   {
-    return Compare_y_near_boundary_2(this->m_base_tr->
+    return Compare_y_near_boundary_2(this->m_base_traits->
                                      compare_y_near_boundary_2_object()
     );
   }
@@ -443,7 +439,7 @@ public:
   /*! Obtain a Construct_min_vertex_2 functor object. */
   Parameter_space_in_y_2 parameter_space_in_y_2_object() const
   {
-    return Parameter_space_in_y_2(this->m_base_tr->
+    return Parameter_space_in_y_2(this->m_base_traits->
                                   parameter_space_in_y_2_object());
   }
 
@@ -476,7 +472,7 @@ public:
   /*! Obtain a Construct_min_vertex_2 functor object. */
   Compare_x_near_boundary_2 compare_x_near_boundary_2_object() const
   {
-    return Compare_x_near_boundary_2(this->m_base_tr->
+    return Compare_x_near_boundary_2(this->m_base_traits->
                                      compare_x_near_boundary_2_object());
   }
 
diff -pruN 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_agg_op.h 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_agg_op.h
--- 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_agg_op.h	2020-08-01 19:02:15.000000000 +0000
+++ 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_agg_op.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_agg_op.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_agg_op.h $
 // $Id: Gps_agg_op.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_agg_op_surface_sweep_2.h 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_agg_op_surface_sweep_2.h
--- 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_agg_op_surface_sweep_2.h	2020-08-01 19:02:15.000000000 +0000
+++ 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_agg_op_surface_sweep_2.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_agg_op_surface_sweep_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_agg_op_surface_sweep_2.h $
 // $Id: Gps_agg_op_surface_sweep_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_agg_op_visitor.h 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_agg_op_visitor.h
--- 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_agg_op_visitor.h	2020-08-01 19:02:15.000000000 +0000
+++ 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_agg_op_visitor.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_agg_op_visitor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_agg_op_visitor.h $
 // $Id: Gps_agg_op_visitor.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_base_functor.h 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_base_functor.h
--- 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_base_functor.h	2020-08-01 19:02:15.000000000 +0000
+++ 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_base_functor.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_base_functor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_base_functor.h $
 // $Id: Gps_base_functor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_bfs_base_visitor.h 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_bfs_base_visitor.h
--- 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_bfs_base_visitor.h	2020-08-01 19:02:15.000000000 +0000
+++ 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_bfs_base_visitor.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_bfs_base_visitor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_bfs_base_visitor.h $
 // $Id: Gps_bfs_base_visitor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_bfs_intersection_visitor.h 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_bfs_intersection_visitor.h
--- 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_bfs_intersection_visitor.h	2020-08-01 19:02:15.000000000 +0000
+++ 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_bfs_intersection_visitor.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_bfs_intersection_visitor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_bfs_intersection_visitor.h $
 // $Id: Gps_bfs_intersection_visitor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_bfs_join_visitor.h 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_bfs_join_visitor.h
--- 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_bfs_join_visitor.h	2020-08-01 19:02:15.000000000 +0000
+++ 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_bfs_join_visitor.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_bfs_join_visitor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_bfs_join_visitor.h $
 // $Id: Gps_bfs_join_visitor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_bfs_scanner.h 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_bfs_scanner.h
--- 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_bfs_scanner.h	2020-08-01 19:02:15.000000000 +0000
+++ 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_bfs_scanner.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_bfs_scanner.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_bfs_scanner.h $
 // $Id: Gps_bfs_scanner.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_bfs_xor_visitor.h 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_bfs_xor_visitor.h
--- 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_bfs_xor_visitor.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_bfs_xor_visitor.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_bfs_xor_visitor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_bfs_xor_visitor.h $
 // $Id: Gps_bfs_xor_visitor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_default_dcel.h 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_default_dcel.h
--- 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_default_dcel.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_default_dcel.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_default_dcel.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_default_dcel.h $
 // $Id: Gps_default_dcel.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_default_traits.h 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_default_traits.h
--- 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_default_traits.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_default_traits.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_default_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_default_traits.h $
 // $Id: Gps_default_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_difference_functor.h 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_difference_functor.h
--- 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_difference_functor.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_difference_functor.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_difference_functor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_difference_functor.h $
 // $Id: Gps_difference_functor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_do_intersect_functor.h 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_do_intersect_functor.h
--- 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_do_intersect_functor.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_do_intersect_functor.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_do_intersect_functor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_do_intersect_functor.h $
 // $Id: Gps_do_intersect_functor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_insertion_meta_traits.h 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_insertion_meta_traits.h
--- 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_insertion_meta_traits.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_insertion_meta_traits.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_insertion_meta_traits.h $
-// $Id: Gps_insertion_meta_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_insertion_meta_traits.h $
+// $Id: Gps_insertion_meta_traits.h 7436c14 2020-04-06T01:23:45+03:00 Efi Fogel
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -75,7 +75,7 @@ public:
   Construct_min_vertex_2 construct_min_vertex_2_object () const
   {
     return Construct_min_vertex_2
-      (this->m_base_tr->construct_min_vertex_2_object());
+      (this->m_base_traits->construct_min_vertex_2_object());
   }
 
 
@@ -100,7 +100,7 @@ public:
   Construct_max_vertex_2 construct_max_vertex_2_object () const
   {
     return Construct_max_vertex_2
-      (this->m_base_tr->construct_max_vertex_2_object());
+      (this->m_base_traits->construct_max_vertex_2_object());
   }
 
   class Compare_xy_2
@@ -123,7 +123,7 @@ public:
   /*! Get a Compare_xy_2 functor object. */
   Compare_xy_2 compare_xy_2_object () const
   {
-    return Compare_xy_2(m_base_tr->compare_xy_2_object());
+    return Compare_xy_2(m_base_traits->compare_xy_2_object());
   }
 
 };
diff -pruN 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_intersection_functor.h 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_intersection_functor.h
--- 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_intersection_functor.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_intersection_functor.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_intersection_functor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_intersection_functor.h $
 // $Id: Gps_intersection_functor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_join_functor.h 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_join_functor.h
--- 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_join_functor.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_join_functor.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_join_functor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_join_functor.h $
 // $Id: Gps_join_functor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_merge.h 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_merge.h
--- 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_merge.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_merge.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_merge.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_merge.h $
 // $Id: Gps_merge.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_on_surface_base_2.h 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_on_surface_base_2.h
--- 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_on_surface_base_2.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_on_surface_base_2.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_on_surface_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_on_surface_base_2.h $
 // $Id: Gps_on_surface_base_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_on_surface_base_2_impl.h 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_on_surface_base_2_impl.h
--- 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_on_surface_base_2_impl.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_on_surface_base_2_impl.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_on_surface_base_2_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_on_surface_base_2_impl.h $
 // $Id: Gps_on_surface_base_2_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_polygon_simplifier.h 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_polygon_simplifier.h
--- 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_polygon_simplifier.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_polygon_simplifier.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_polygon_simplifier.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_polygon_simplifier.h $
 // $Id: Gps_polygon_simplifier.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_polygon_validation.h 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_polygon_validation.h
--- 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_polygon_validation.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_polygon_validation.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_polygon_validation.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_polygon_validation.h $
 // $Id: Gps_polygon_validation.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_simplifier_traits.h 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_simplifier_traits.h
--- 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_simplifier_traits.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_simplifier_traits.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,12 +3,13 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_simplifier_traits.h $
-// $Id: Gps_simplifier_traits.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_simplifier_traits.h $
+// $Id: Gps_simplifier_traits.h 6fcbee1 2020-04-21T17:12:21+03:00 Efi Fogel
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
-// Author(s)     : Baruch Zukerman <baruchzu@post.tau.ac.il>
+// Author(s): Baruch Zukerman <baruchzu@post.tau.ac.il>
+//            Efi Fogel       <efifogel@gmail.com>
 
 #ifndef CGAL_GPS_SIMPLIFIER_TRAITS_H
 #define CGAL_GPS_SIMPLIFIER_TRAITS_H
@@ -20,103 +21,67 @@
 
 namespace CGAL {
 
-class Gps_simplifier_curve_data
-{
+class Gps_simplifier_curve_data {
 protected:
   unsigned int m_bc;
   unsigned int m_twin_bc;
   unsigned int m_index;
 
 public:
-  Gps_simplifier_curve_data()
-  {}
+  Gps_simplifier_curve_data() {}
 
-  Gps_simplifier_curve_data(unsigned int bc,
-                            unsigned int twin_bc,
+  Gps_simplifier_curve_data(unsigned int bc, unsigned int twin_bc,
                             unsigned int index):
     m_bc(bc),
     m_twin_bc(twin_bc),
     m_index(index)
   {}
 
-  unsigned int bc() const
-  {
-    return m_bc;
-  }
-
-  unsigned int twin_bc() const
-  {
-    return m_twin_bc;
-  }
-
-  unsigned int index() const
-  {
-    return m_index;
-  }
-
-  unsigned int& index()
-  {
-    return m_index;
-  }
-
-  unsigned int& twin_bc()
-  {
-    return m_twin_bc;
-  }
-
-  void set_bc(unsigned int bc)
-  {
-    m_bc = bc;
-  }
-
-  void set_twin_bc(unsigned int twin_bc)
-  {
-    m_twin_bc = twin_bc;
-  }
-
-  void set_index(unsigned int index)
-  {
-    m_index = index;
-  }
+  unsigned int bc() const { return m_bc; }
+
+  unsigned int twin_bc() const { return m_twin_bc; }
+
+  unsigned int index() const { return m_index; }
+
+  unsigned int& index() { return m_index; }
+
+  unsigned int& twin_bc() { return m_twin_bc; }
+
+  void set_bc(unsigned int bc) { m_bc = bc; }
+
+  void set_twin_bc(unsigned int twin_bc) { m_twin_bc = twin_bc; }
+
+  void set_index(unsigned int index) { m_index = index; }
 };
 
-struct Gps_simplifier_point_data
-{
+struct Gps_simplifier_point_data {
 protected:
   unsigned int m_index;
 
 public:
-  Gps_simplifier_point_data()
-  {}
+  Gps_simplifier_point_data() {}
 
-  Gps_simplifier_point_data(unsigned int index) : m_index(index)
-  {}
+  Gps_simplifier_point_data(unsigned int index) : m_index(index) {}
+
+  unsigned int index() const { return m_index; }
 
-  unsigned int index() const
-  {
-    return m_index;
-  }
-
-  void set_index(unsigned int index)
-  {
-    m_index = index;
-  }
+  void set_index(unsigned int index) { m_index = index; }
 };
 
-template <class Traits_>
+template <typename Traits_>
 class Gps_simplifier_traits :
   public Gps_traits_decorator<Traits_,
                               Gps_simplifier_curve_data,
                               Gps_simplifier_point_data>
 {
 public:
-  typedef Traits_    Traits;
+  typedef Traits_                                           Traits;
   typedef Gps_traits_decorator<Traits_,
                                Gps_simplifier_curve_data,
-                               Gps_simplifier_point_data>    Base;
-  typedef Gps_simplifier_traits<Traits>                      Self;
-  typedef typename Traits::X_monotone_curve_2     Base_X_monotone_curve_2;
-  typedef typename Traits::Point_2                Base_Point_2;
+                               Gps_simplifier_point_data>   Base;
+  typedef Gps_simplifier_traits<Traits>                     Self;
+  typedef typename Traits::X_monotone_curve_2     Base_x_monotone_curve_2;
+  typedef typename Traits::Point_2                Base_point_2;
   typedef typename Traits::Construct_min_vertex_2 Base_Construct_min_vertex_2;
   typedef typename Traits::Construct_max_vertex_2 Base_Construct_max_vertex_2;
   typedef typename Traits::Compare_endpoints_xy_2 Base_Compare_endpoints_xy_2;
@@ -129,9 +94,7 @@ public:
 protected:
   mutable unsigned int m_pgn_size;
 
-
 public:
-
   typedef typename Base::X_monotone_curve_2       X_monotone_curve_2;
   typedef typename Base::Point_2                  Point_2;
   typedef typename Base::Multiplicity             Multiplicity;
@@ -139,333 +102,256 @@ public:
   typedef typename Base::Curve_data               Curve_data;
   typedef typename Base::Point_data               Point_data;
 
-  Gps_simplifier_traits()
-  {}
+  Gps_simplifier_traits() {}
 
-  Gps_simplifier_traits(const Traits & tr) : Base(tr)
-  {}
+  Gps_simplifier_traits(const Traits& tr) : Base(tr) {}
 
-  unsigned int polygon_size() const
-  {
-    return m_pgn_size;
-  }
-
-  void set_polygon_size(unsigned int pgn_size) const
-  {
-    m_pgn_size = pgn_size;
-  }
+  unsigned int polygon_size() const { return m_pgn_size; }
+
+  void set_polygon_size(unsigned int pgn_size) const { m_pgn_size = pgn_size; }
 
   bool is_valid_index(unsigned int index) const
-  {
-    return (index < m_pgn_size);
-  }
-
-  unsigned int invalid_index() const
-  {
-    return (m_pgn_size);
-  }
+  { return (index < m_pgn_size); }
 
+  unsigned int invalid_index() const { return (m_pgn_size); }
 
-  class Intersect_2
-  {
+  class Intersect_2 {
   private:
+    /*! The traits (in case it has state) */
+    const Self& m_traits;
 
-    Base_Intersect_2             m_base;
-    Base_Compare_endpoints_xy_2  m_base_cmp_endpoints;
-    Base_Compare_xy_2            m_base_cmp_xy;
-    Base_Construct_min_vertex_2  m_ctr_min_v;
-    const Self * m_self_tr;
+    /*! Constructor. */
+    Intersect_2(const Self& tr) : m_traits(tr) {}
 
-  public:
+    friend Self;
 
-    /*! Constructor. */
-    Intersect_2 (const Base_Intersect_2& base,
-                 const Base_Compare_endpoints_xy_2& base_cmp_endpoints,
-                 const Base_Compare_xy_2& base_cmp_xy,
-                 const Base_Construct_min_vertex_2& ,
-                 const Self*  tr) :
-      m_base(base),
-      m_base_cmp_endpoints(base_cmp_endpoints),
-      m_base_cmp_xy(base_cmp_xy),
-      m_self_tr(tr)
-    {}
-
-    template<class OutputIterator>
-    OutputIterator operator() (const X_monotone_curve_2& cv1,
-                               const X_monotone_curve_2& cv2,
-                               OutputIterator oi) const
+  public:
+    template <typename OutputIterator>
+    OutputIterator operator()(const X_monotone_curve_2& cv1,
+                              const X_monotone_curve_2& cv2,
+                              OutputIterator oi) const
     {
+      typedef const std::pair<Base_point_2, Multiplicity>
+        Intersection_base_point;
+      typedef boost::variant<Intersection_base_point, Base_x_monotone_curve_2>
+                                                        Intersection_base_result;
+      typedef const std::pair<Point_2, Multiplicity>    Intersection_point;
+      typedef boost::variant<Intersection_point, X_monotone_curve_2>
+                                                        Intersection_result;
+
+      const auto* base_traits = m_traits.m_base_traits;
+      auto base_cmp_xy = base_traits->compare_xy_2_object();
+      auto base_cmp_endpoints = base_traits->compare_endpoints_xy_2_object();
+      auto base_ctr_min_vertex = base_traits->construct_min_vertex_2_object();
+      auto base_intersect = base_traits->intersect_2_object();
+
       //// if the two curves are incident, do not intersect them
-      //if(m_self_tr->is_valid_index(cv1.data().index()) &&
-      //   m_self_tr->is_valid_index(cv2.data().index()))
+      //if (m_traits.is_valid_index(cv1.data().index()) &&
+      //    m_traits.is_valid_index(cv2.data().index()))
       //{
       //  unsigned int index_diff =
       //    (cv1.data().index() > cv2.data().index()) ?
       //    (cv1.data().index() - cv2.data().index()):
       //    (cv2.data().index() - cv1.data().index());
 
-      //  if(index_diff == 1 ||index_diff == m_self_tr->polygon_size() -1)
+      //  if(index_diff == 1 ||index_diff == m_traits.polygon_size() -1)
       //  {
       //    return (oi);
       //  }
       //}
-      const std::pair<Base_Point_2, Multiplicity>   *base_pt;
-      const Base_X_monotone_curve_2                 *overlap_cv;
-      OutputIterator oi_end;
-      if(m_base_cmp_xy(m_ctr_min_v(cv1.base()),
-                       m_ctr_min_v(cv2.base())) == LARGER)
-        oi_end = m_base(cv1.base(), cv2.base(), oi);
+      std::vector<Intersection_base_result> xections;
+      if (base_cmp_xy(base_ctr_min_vertex(cv1.base()),
+                      base_ctr_min_vertex(cv2.base())) == LARGER)
+        base_intersect(cv1.base(), cv2.base(), back_inserter(xections));
       else
-        oi_end = m_base(cv2.base(), cv1.base(), oi);
+        base_intersect(cv2.base(), cv1.base(), back_inserter(xections));
 
-      // convert objects that are associated with Base_X_monotone_curve_2 to
+      // convert objects that are associated with Base_x_monotone_curve_2 to
       // the extenede X_monotone_curve_2
-      for(; oi != oi_end; ++oi)
-      {
-        base_pt = object_cast<std::pair<Base_Point_2, Multiplicity> >(&(*oi));
-
-        if (base_pt != nullptr)
-        {
-          Point_data pt_data(m_self_tr->invalid_index());
-          Point_2 point_plus (base_pt->first, pt_data); // the extended point
-          *oi = CGAL::make_object(std::make_pair(point_plus,
-                                                 base_pt->second));
+      for (const auto& xection : xections) {
+        const Intersection_base_point* base_pt =
+          boost::get<Intersection_base_point>(&xection);
+        if (base_pt != nullptr) {
+          Point_data pt_data(m_traits.invalid_index());
+          Point_2 point_plus(base_pt->first, pt_data); // the extended point
+          *oi++ =
+            Intersection_result(std::make_pair(point_plus, base_pt->second));
+          continue;
+        }
+
+        const Base_x_monotone_curve_2* overlap_cv =
+          boost::get<Base_x_monotone_curve_2>(&xection);
+
+        CGAL_assertion(overlap_cv != nullptr);
+        unsigned int ov_bc;
+        unsigned int ov_twin_bc;
+        if (base_cmp_endpoints(cv1) == base_cmp_endpoints(cv2)) {
+          // cv1 and cv2 have the same directions
+          ov_bc = cv1.data().bc() + cv2.data().bc();
+          ov_twin_bc = cv1.data().twin_bc() + cv2.data().twin_bc();
         }
-        else
-        {
-          overlap_cv = object_cast<Base_X_monotone_curve_2> (&(*oi));
-
-          if (overlap_cv != nullptr)
-          {
-            unsigned int ov_bc;
-            unsigned int ov_twin_bc;
-            if(m_base_cmp_endpoints(cv1) == m_base_cmp_endpoints(cv2))
-            {
-              // cv1 and cv2 have the same directions
-              ov_bc = cv1.data().bc() + cv2.data().bc();
-              ov_twin_bc = cv1.data().twin_bc() + cv2.data().twin_bc();
-            }
-            else
-            {
-              // cv1 and cv2 have opposite directions
-              ov_bc = cv1.data().bc() + cv2.data().twin_bc();
-              ov_twin_bc = cv1.data().twin_bc() + cv2.data().bc();
-            }
-
-            if(m_base_cmp_endpoints(*overlap_cv) != m_base_cmp_endpoints(cv1))
-            {
-              // overlap_cv, cv1 have opposite directions
-              std::swap(ov_bc, ov_twin_bc);
-            }
-
-            Curve_data cv_data(ov_bc, ov_twin_bc, m_self_tr->invalid_index());
-            *oi = CGAL::make_object (X_monotone_curve_2 (*overlap_cv, cv_data));
-          }
+        else {
+          // cv1 and cv2 have opposite directions
+          ov_bc = cv1.data().bc() + cv2.data().twin_bc();
+          ov_twin_bc = cv1.data().twin_bc() + cv2.data().bc();
         }
+
+        if (base_cmp_endpoints(*overlap_cv) != base_cmp_endpoints(cv1)) {
+          // overlap_cv, cv1 have opposite directions
+          std::swap(ov_bc, ov_twin_bc);
+        }
+
+        Curve_data cv_data(ov_bc, ov_twin_bc, m_traits.invalid_index());
+        *oi++ = Intersection_result(X_monotone_curve_2(*overlap_cv, cv_data));
       }
-      //return past-end iterator
-      return oi_end;
+
+      return oi;
     }
   };
 
-   /*! Get an Intersect_2 functor object. */
-  Intersect_2 intersect_2_object () const
-  {
-    return Intersect_2(this->m_base_tr->intersect_2_object(),
-                       this->m_base_tr->compare_endpoints_xy_2_object(),
-                       this->m_base_tr->compare_xy_2_object(),
-                       this->m_base_tr->construct_min_vertex_2_object(),
-                       this);
-  }
+  /*! Obtain an Intersect_2 functor object. */
+  Intersect_2 intersect_2_object () const { return Intersect_2(*this); }
 
-  class Split_2
-  {
+  class Split_2 {
   private:
-    Base_Split_2      m_base_split;
-    const Self * m_self_tr;
-
-  public:
+    const Self& m_traits;
 
     /*! Constructor. */
-    Split_2 (const Base_Split_2& base, const Self* tr) :
-      m_base_split(base),
-      m_self_tr(tr)
-    {}
+    Split_2(const Self& tr) : m_traits(tr) {}
+
+    friend Self;
 
-    void operator() (const X_monotone_curve_2& cv, const Point_2 & p,
-                     X_monotone_curve_2& c1, X_monotone_curve_2& c2) const
+  public:
+    void operator()(const X_monotone_curve_2& cv, const Point_2 & p,
+                    X_monotone_curve_2& c1, X_monotone_curve_2& c2) const
     {
-      m_base_split(cv.base(),
-                   p.base(),
-                   c1.base(),
-                   c2.base());
+      const auto* base_traits = m_traits.m_base_traits;
+      auto base_split = base_traits->split_2_object();
+      base_split(cv.base(), p.base(), c1.base(), c2.base());
       const Curve_data& cv_data = cv.data();
-      c1.set_data(Curve_data(cv_data.bc(),
-                             cv_data.twin_bc(),
-                             m_self_tr->invalid_index()));
-
-      c2.set_data(Curve_data(cv_data.bc(),
-                             cv_data.twin_bc(),
-                             m_self_tr->invalid_index()));
+      c1.set_data(Curve_data(cv_data.bc(), cv_data.twin_bc(),
+                             m_traits.invalid_index()));
+
+      c2.set_data(Curve_data(cv_data.bc(), cv_data.twin_bc(),
+                             m_traits.invalid_index()));
     }
   };
 
   /*! Get a Split_2 functor object. */
-  Split_2 split_2_object () const
-  {
-    return Split_2(this->m_base_tr->split_2_object(), this);
-  }
+  Split_2 split_2_object () const { return Split_2(*this); }
 
-  class Construct_min_vertex_2
-  {
+  class Construct_min_vertex_2 {
   private:
-    Base_Construct_min_vertex_2 m_base;
-    Base_Compare_endpoints_xy_2 m_base_cmp_endpoints;
-    const Self * m_self_tr;
+    const Self& m_traits;
 
-  public:
+    Construct_min_vertex_2(const Self& tr) : m_traits(tr) {}
 
-    Construct_min_vertex_2(const Base_Construct_min_vertex_2& base,
-                          const Base_Compare_endpoints_xy_2& base_cmp_endpoints,
-                          const Self * tr):
-      m_base(base),
-      m_base_cmp_endpoints(base_cmp_endpoints),
-      m_self_tr(tr)
-    {}
+    friend Self;
 
-    /*!
-      * Get the left endpoint of the x-monotone curve (segment).
+  public:
+    /*! Obtain the left endpoint of the x-monotone curve (segment).
       * \param cv The curve.
       * \return The left endpoint.
       */
-    Point_2 operator() (const X_monotone_curve_2 & cv) const
+    Point_2 operator()(const X_monotone_curve_2 & cv) const
     {
-      if(!m_self_tr->is_valid_index(cv.data().index()))
-      {
-        return Point_2 (m_base(cv.base()), m_self_tr->invalid_index());
-      }
+      const auto* base_traits = m_traits.m_base_traits;
+      auto base_ctr_min_vertex = base_traits->construct_min_vertex_2_object();
 
-      Comparison_result res = m_base_cmp_endpoints(cv);
+      if (! m_traits.is_valid_index(cv.data().index()))
+        return Point_2(base_ctr_min_vertex(cv.base()), m_traits.invalid_index());
+
+      auto base_cmp_endpoints = base_traits->compare_endpoints_xy_2_object();
+      Comparison_result res = base_cmp_endpoints(cv);
       Point_data pt_data;
-      if(res == SMALLER)
-      {
+      if (res == SMALLER) {
         // min vertex is the source
         pt_data.set_index(cv.data().index());
       }
-      else
-      {
+      else {
         // min vertex is the target
-        pt_data.set_index((cv.data().index() + 1) % m_self_tr->polygon_size());
+        pt_data.set_index((cv.data().index() + 1) % m_traits.polygon_size());
       }
-      return Point_2 (m_base(cv.base()), pt_data);
+      return Point_2(base_ctr_min_vertex(cv.base()), pt_data);
     }
   };
 
   /*! Get a Construct_min_vertex_2 functor object. */
   Construct_min_vertex_2 construct_min_vertex_2_object () const
-  {
-    return Construct_min_vertex_2
-      (this->m_base_tr->construct_min_vertex_2_object(),
-       this->m_base_tr->compare_endpoints_xy_2_object(),
-       this);
-  }
-
+  { return Construct_min_vertex_2(*this); }
 
-  class Construct_max_vertex_2
-  {
+  class Construct_max_vertex_2 {
   private:
-    Base_Construct_max_vertex_2      m_base;
-    Base_Compare_endpoints_xy_2      m_base_cmp_endpoints;
-    const Self * m_self_tr;
+    const Self& m_traits;
 
-  public:
+    Construct_max_vertex_2(const Self& tr) : m_traits(tr) {}
 
-    Construct_max_vertex_2(const Base_Construct_max_vertex_2& base,
-                          const Base_Compare_endpoints_xy_2& base_cmp_endpoints,
-                          const Self * tr):
-      m_base(base),
-      m_base_cmp_endpoints(base_cmp_endpoints),
-      m_self_tr(tr)
-    {}
+    friend Self;
 
-    /*!
-      * Get the right endpoint of the x-monotone curve (segment).
+  public:
+    /*! Obtain the right endpoint of the x-monotone curve (segment).
       * \param cv The curve.
       * \return The left endpoint.
       */
     Point_2 operator() (const X_monotone_curve_2 & cv) const
     {
-      if(!m_self_tr->is_valid_index(cv.data().index()))
-      {
-        return Point_2 (m_base(cv.base()), m_self_tr->invalid_index());
-      }
-      Comparison_result res = m_base_cmp_endpoints(cv);
+      const auto* base_traits = m_traits.m_base_traits;
+      auto base_ctr_max_vertex = base_traits->construct_max_vertex_2_object();
+      if (! m_traits.is_valid_index(cv.data().index()))
+        return Point_2(base_ctr_max_vertex(cv.base()), m_traits.invalid_index());
+
+      auto base_cmp_endpoints = base_traits->compare_endpoints_xy_2_object();
+      Comparison_result res = base_cmp_endpoints(cv);
       Point_data pt_data;
-      if(res == SMALLER)
-      {
+      if (res == SMALLER) {
         // min vertex is the target
-        pt_data.set_index((cv.data().index() + 1) % m_self_tr->polygon_size());
+        pt_data.set_index((cv.data().index() + 1) % m_traits.polygon_size());
       }
-      else
-      {
+      else {
         // min vertex is the source
         pt_data.set_index(cv.data().index());
       }
-      return Point_2 (m_base(cv.base()), pt_data);
+      return Point_2(base_ctr_max_vertex(cv.base()), pt_data);
     }
   };
 
   /*! Get a Construct_min_vertex_2 functor object. */
   Construct_max_vertex_2 construct_max_vertex_2_object () const
-  {
-    return Construct_max_vertex_2
-      (this->m_base_tr->construct_max_vertex_2_object(),
-       this->m_base_tr->compare_endpoints_xy_2_object(),
-       this);
-  }
+  { return Construct_max_vertex_2(*this); }
 
-  class Compare_xy_2
-  {
+  class Compare_xy_2 {
   private:
-    Base_Compare_xy_2       m_base;
-    const Self * m_self_tr;
+    const Self& m_traits;
 
-  public:
-    Compare_xy_2(const Base_Compare_xy_2& base,
-                 const Self * tr):
-      m_base(base),
-      m_self_tr(tr)
-    {}
+    Compare_xy_2(const Self& tr) : m_traits(tr) {}
 
+    friend Self;
 
-    /*!
-      * Get the left endpoint of the x-monotone curve (segment).
+  public:
+    /*! Obtain the left endpoint of the x-monotone curve (segment).
       * \param cv The curve.
       * \return The left endpoint.
       */
-    Comparison_result operator() (const Point_2& p1, const Point_2& p2) const
+    Comparison_result operator()(const Point_2& p1, const Point_2& p2) const
     {
+      const auto* base_traits = m_traits.m_base_traits;
+      auto base_cmp_xy = base_traits->compare_xy_2_object();
+
       //if one of the indexes is invalid, compare p1 and p2
-      if(! m_self_tr->is_valid_index(p1.data().index()) ||
-        ! m_self_tr->is_valid_index(p2.data().index()))
-        return (m_base(p1.base(), p2.base()));
+      if (! m_traits.is_valid_index(p1.data().index()) ||
+          ! m_traits.is_valid_index(p2.data().index()))
+        return (base_cmp_xy(p1.base(), p2.base()));
 
       // if the two point has the same index, return EQUAL
-      if(p1.data().index() == p2.data().index())
-      {
-        return EQUAL;
-      }
+      if (p1.data().index() == p2.data().index()) return EQUAL;
 
-      return (m_base(p1.base(), p2.base()));
+      return (base_cmp_xy(p1.base(), p2.base()));
     }
   };
 
 
   /*! Get a Construct_min_vertex_2 functor object. */
-  Compare_xy_2 compare_xy_2_object () const
-  {
-    return Compare_xy_2(this->m_base_tr->compare_xy_2_object(), this);
-  }
+  Compare_xy_2 compare_xy_2_object () const { return Compare_xy_2(*this); }
 };
 
 } //namespace CGAL
diff -pruN 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_sym_diff_functor.h 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_sym_diff_functor.h
--- 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_sym_diff_functor.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_sym_diff_functor.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_sym_diff_functor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_sym_diff_functor.h $
 // $Id: Gps_sym_diff_functor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_traits_adaptor.h 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_traits_adaptor.h
--- 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_traits_adaptor.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_traits_adaptor.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_traits_adaptor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_traits_adaptor.h $
 // $Id: Gps_traits_adaptor.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_traits_decorator.h 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_traits_decorator.h
--- 5.0.3-1/include/CGAL/Boolean_set_operations_2/Gps_traits_decorator.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Boolean_set_operations_2/Gps_traits_decorator.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_traits_decorator.h $
-// $Id: Gps_traits_decorator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_traits_decorator.h $
+// $Id: Gps_traits_decorator.h 7436c14 2020-04-06T01:23:45+03:00 Efi Fogel
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -202,25 +202,27 @@ public:
   protected:
 
   //Data members
-  const Base * m_base_tr;
+  const Base* m_base_traits;
   bool m_traits_owner;
 
 public:
 
   Gps_traits_decorator() :
-    m_base_tr(new Base()),
+    m_base_traits(new Base()),
     m_traits_owner(true)
   {}
 
-  Gps_traits_decorator(const Base & base_traits) :
-    m_base_tr(&base_traits),
+  Gps_traits_decorator(const Base& base_traits) :
+    m_base_traits(&base_traits),
     m_traits_owner(false)
   {}
 
   ~Gps_traits_decorator()
   {
-    if (m_traits_owner)
-      delete m_base_tr;
+    if (m_traits_owner) {
+      delete m_base_traits;
+      m_base_traits = nullptr;
+    }
   }
 
   class Compare_x_2
@@ -242,7 +244,7 @@ public:
   /*! Get a Compare_x_2 functor object. */
   Compare_x_2 compare_x_2_object () const
   {
-    return Compare_x_2(m_base_tr->compare_x_2_object());
+    return Compare_x_2(m_base_traits->compare_x_2_object());
   }
 
 
@@ -265,7 +267,7 @@ public:
   /*! Get a Compare_xy_2 functor object. */
   Compare_xy_2 compare_xy_2_object () const
   {
-    return Compare_xy_2(m_base_tr->compare_xy_2_object());
+    return Compare_xy_2(m_base_traits->compare_xy_2_object());
   }
 
   class Construct_min_vertex_2
@@ -288,7 +290,7 @@ public:
   /*! Get a Construct_min_vertex_2 functor object. */
   Construct_min_vertex_2 construct_min_vertex_2_object () const
   {
-    return Construct_min_vertex_2(m_base_tr->construct_min_vertex_2_object());
+    return Construct_min_vertex_2(m_base_traits->construct_min_vertex_2_object());
   }
 
   class Construct_max_vertex_2
@@ -311,7 +313,7 @@ public:
   /*! Get a Construct_max_vertex_2 functor object. */
   Construct_max_vertex_2 construct_max_vertex_2_object () const
   {
-    return Construct_max_vertex_2(m_base_tr->construct_max_vertex_2_object());
+    return Construct_max_vertex_2(m_base_traits->construct_max_vertex_2_object());
   }
 
 
@@ -334,7 +336,7 @@ public:
   /*! Get a Is_vertical_2 functor object. */
   Is_vertical_2 is_vertical_2_object() const
   {
-    return Is_vertical_2(m_base_tr->is_vertical_2_object());
+    return Is_vertical_2(m_base_traits->is_vertical_2_object());
   }
 
 
@@ -358,7 +360,7 @@ public:
   /*! Get a compare_y_at_x_2_object functor object. */
   Compare_y_at_x_2 compare_y_at_x_2_object() const
   {
-    return Compare_y_at_x_2(m_base_tr->compare_y_at_x_2_object());
+    return Compare_y_at_x_2(m_base_traits->compare_y_at_x_2_object());
   }
 
 
@@ -384,7 +386,7 @@ public:
   /*! Get a Compare_y_at_x_right_2 functor object. */
   Compare_y_at_x_right_2 compare_y_at_x_right_2_object() const
   {
-    return Compare_y_at_x_right_2(m_base_tr->compare_y_at_x_right_2_object());
+    return Compare_y_at_x_right_2(m_base_traits->compare_y_at_x_right_2_object());
   }
 
 
@@ -407,7 +409,7 @@ public:
   /*! Get a Equal_2 functor object. */
   Equal_2 equal_2_object() const
   {
-    return Equal_2(m_base_tr->equal_2_object());
+    return Equal_2(m_base_traits->equal_2_object());
   }
 
 
@@ -432,7 +434,7 @@ public:
   /*! Get a Split_2 functor object. */
   Split_2 split_2_object() const
   {
-    return Split_2(m_base_tr->split_2_object());
+    return Split_2(m_base_traits->split_2_object());
   }
 
 
@@ -495,9 +497,9 @@ public:
   /*! Get a Intersect_2 functor object. */
   Intersect_2 intersect_2_object() const
   {
-    return Intersect_2(m_base_tr->intersect_2_object(),
-                       m_base_tr->compare_xy_2_object(),
-                       m_base_tr->construct_min_vertex_2_object());
+    return Intersect_2(m_base_traits->intersect_2_object(),
+                       m_base_traits->compare_xy_2_object(),
+                       m_base_traits->construct_min_vertex_2_object());
   }
 
 
@@ -522,7 +524,7 @@ public:
   /*! Get a Compare_endpoints_xy_2 functor object. */
   Compare_endpoints_xy_2 compare_endpoints_xy_2_object() const
   {
-    return Compare_endpoints_xy_2(m_base_tr->compare_endpoints_xy_2_object());
+    return Compare_endpoints_xy_2(m_base_traits->compare_endpoints_xy_2_object());
   }
 
 
@@ -545,7 +547,7 @@ public:
   /*! Get a Construct_opposite_2 functor object. */
   Construct_opposite_2 construct_opposite_2_object() const
   {
-    return Construct_opposite_2(m_base_tr->construct_opposite_2_object());
+    return Construct_opposite_2(m_base_traits->construct_opposite_2_object());
   }
 
 };
diff -pruN 5.0.3-1/include/CGAL/Boolean_set_operations_2/Indexed_event.h 5.1-1/include/CGAL/Boolean_set_operations_2/Indexed_event.h
--- 5.0.3-1/include/CGAL/Boolean_set_operations_2/Indexed_event.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Boolean_set_operations_2/Indexed_event.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Indexed_event.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Indexed_event.h $
 // $Id: Indexed_event.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Boolean_set_operations_2/Point_with_vertex.h 5.1-1/include/CGAL/Boolean_set_operations_2/Point_with_vertex.h
--- 5.0.3-1/include/CGAL/Boolean_set_operations_2/Point_with_vertex.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Boolean_set_operations_2/Point_with_vertex.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Point_with_vertex.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Point_with_vertex.h $
 // $Id: Point_with_vertex.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Boolean_set_operations_2/Polygon_2_curve_iterator.h 5.1-1/include/CGAL/Boolean_set_operations_2/Polygon_2_curve_iterator.h
--- 5.0.3-1/include/CGAL/Boolean_set_operations_2/Polygon_2_curve_iterator.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Boolean_set_operations_2/Polygon_2_curve_iterator.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Polygon_2_curve_iterator.h $
-// $Id: Polygon_2_curve_iterator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Polygon_2_curve_iterator.h $
+// $Id: Polygon_2_curve_iterator.h 2756b2a 2020-05-24T11:11:32+02:00 Marc Glisse
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -60,9 +60,9 @@ public:
     Edge_const_iterator m_curr_edge;   // points to the current edge iterator
 
 public:
-    Polygon_2_curve_iterator< X_monotone_curve_2_, Polygon_ >(){}
+    Polygon_2_curve_iterator(){}
 
-    Polygon_2_curve_iterator< X_monotone_curve_2_, Polygon_ >
+    Polygon_2_curve_iterator
       (const Polygon* pgn, Edge_const_iterator ci) : m_pgn(pgn),
                                                      m_curr_edge(ci) {}
 
diff -pruN 5.0.3-1/include/CGAL/Boolean_set_operations_2.h 5.1-1/include/CGAL/Boolean_set_operations_2.h
--- 5.0.3-1/include/CGAL/Boolean_set_operations_2.h	2020-08-01 19:02:15.000000000 +0000
+++ 5.1-1/include/CGAL/Boolean_set_operations_2.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2.h $
 // $Id: Boolean_set_operations_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/bimap/multiset_of.hpp 5.1-1/include/CGAL/boost/bimap/multiset_of.hpp
--- 5.0.3-1/include/CGAL/boost/bimap/multiset_of.hpp	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/boost/bimap/multiset_of.hpp	2020-09-08 12:14:54.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/boost/bimap/multiset_of.hpp $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/boost/bimap/multiset_of.hpp $
 // $Id: multiset_of.hpp 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/bimap.hpp 5.1-1/include/CGAL/boost/bimap.hpp
--- 5.0.3-1/include/CGAL/boost/bimap.hpp	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/boost/bimap.hpp	2020-09-08 12:14:54.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/boost/bimap.hpp $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/boost/bimap.hpp $
 // $Id: bimap.hpp 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/alpha_expansion_graphcut.h 5.1-1/include/CGAL/boost/graph/alpha_expansion_graphcut.h
--- 5.0.3-1/include/CGAL/boost/graph/alpha_expansion_graphcut.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/alpha_expansion_graphcut.h	2020-09-08 12:14:40.000000000 +0000
@@ -0,0 +1,754 @@
+#ifndef CGAL_BOOST_GRAPH_ALPHA_EXPANSION_GRAPHCUT_H
+// Copyright (c) 2014  GeometryFactory (France).  All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/alpha_expansion_graphcut.h $
+// $Id: alpha_expansion_graphcut.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
+// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Ilker O. Yaz, Simon Giraudot
+
+#define CGAL_BOOST_GRAPH_ALPHA_EXPANSION_GRAPHCUT_H
+
+#include <CGAL/Iterator_range.h>
+#include <CGAL/assertions.h>
+#include <CGAL/property_map.h>
+#ifdef CGAL_SEGMENTATION_BENCH_GRAPHCUT
+#include <CGAL/Timer.h>
+#endif
+#include <CGAL/IO/trace.h>
+
+#include <CGAL/boost/graph/Named_function_parameters.h>
+#include <CGAL/boost/graph/named_params_helper.h>
+
+#include <boost/version.hpp>
+
+#include <boost/graph/adjacency_list.hpp>
+#include <boost/graph/compressed_sparse_row_graph.hpp>
+
+#if BOOST_VERSION >= 104400 // at this version kolmogorov_max_flow become depricated.
+#  include <boost/graph/boykov_kolmogorov_max_flow.hpp>
+#else
+#  include <boost/graph/kolmogorov_max_flow.hpp>
+#endif
+
+#include <vector>
+
+
+
+
+namespace CGAL
+{
+
+/// \cond SKIP_IN_MANUAL
+namespace internal
+{
+
+struct Alpha_expansion_old_API_wrapper_graph
+{
+  typedef std::size_t vertex_descriptor;
+  typedef std::size_t edge_descriptor;
+  typedef boost::directed_tag directed_category;
+  typedef boost::disallow_parallel_edge_tag edge_parallel_category;
+  typedef boost::edge_list_graph_tag traversal_category;
+
+  typedef boost::counting_iterator<std::size_t> counting_iterator;
+  typedef CGAL::Iterator_range<counting_iterator> counting_range;
+
+  typedef CGAL::Identity_property_map<std::size_t> Vertex_index_map;
+  typedef CGAL::Pointer_property_map<std::size_t>::type Vertex_label_map;
+
+  struct Vertex_label_cost_map
+  {
+    typedef std::size_t key_type;
+    typedef std::vector<double> value_type;
+    typedef value_type reference;
+    typedef boost::readable_property_map_tag category;
+
+    const std::vector<std::vector<double> >* cost_matrix;
+
+    Vertex_label_cost_map (const std::vector<std::vector<double> >* cost_matrix)
+      : cost_matrix (cost_matrix)
+    { }
+
+    friend reference get (const Vertex_label_cost_map& pmap, key_type idx)
+    {
+      std::vector<double> out;
+      out.reserve (pmap.cost_matrix->size());
+      for (std::size_t i = 0; i < pmap.cost_matrix->size(); ++ i)
+        out.push_back ((*pmap.cost_matrix)[i][idx]);
+      return out;
+    }
+
+  };
+
+  typedef CGAL::Pointer_property_map<double>::const_type Edge_cost_map;
+
+  const std::vector<std::pair<std::size_t, std::size_t> >& edges;
+  const std::vector<double>& edge_costs;
+  const std::vector<std::vector<double> >& cost_matrix;
+  std::vector<std::size_t>& labels;
+
+  Alpha_expansion_old_API_wrapper_graph (const std::vector<std::pair<std::size_t, std::size_t> >& edges,
+                                         const std::vector<double>& edge_costs,
+                                         const std::vector<std::vector<double> >& cost_matrix,
+                                         std::vector<std::size_t>& labels)
+    : edges (edges), edge_costs (edge_costs), cost_matrix (cost_matrix), labels (labels)
+  { }
+
+  friend counting_range vertices (const Alpha_expansion_old_API_wrapper_graph& graph)
+  {
+    return CGAL::make_range (boost::counting_iterator<std::size_t>(0),
+                             boost::counting_iterator<std::size_t>(graph.labels.size()));
+  }
+
+  friend std::size_t num_vertices (const Alpha_expansion_old_API_wrapper_graph& graph) { return graph.labels.size(); }
+
+  friend counting_range edges (const Alpha_expansion_old_API_wrapper_graph& graph)
+  {
+    return CGAL::make_range (boost::counting_iterator<std::size_t>(0),
+                             boost::counting_iterator<std::size_t>(graph.edges.size()));
+  }
+
+  friend vertex_descriptor source (edge_descriptor ed, const Alpha_expansion_old_API_wrapper_graph& graph)
+  { return graph.edges[ed].first; }
+  friend vertex_descriptor target (edge_descriptor ed, const Alpha_expansion_old_API_wrapper_graph& graph)
+  { return graph.edges[ed].second; }
+
+  Vertex_index_map vertex_index_map() const { return Vertex_index_map(); }
+  Vertex_label_map vertex_label_map() { return CGAL::make_property_map(labels); }
+  Vertex_label_cost_map vertex_label_cost_map() const
+  { return Vertex_label_cost_map(&cost_matrix); }
+  Edge_cost_map edge_cost_map() const { return CGAL::make_property_map(edge_costs); }
+};
+
+////////////////////////////////////////////////////////////////////////////////////////
+//   Comments about performance:
+//
+// 1) With BGL:
+//     * Using adjacency_list:
+//     ** Without pre-allocating vertex-list
+//       | OutEdgeList | VertexList | Performance |
+//       |    listS    |   listS    |     25.2    |
+//       |    vecS     |   listS    |     22.7    |
+//       |    listS    |   vecS     |     30.7    |
+//       |    vecS     |   vecS     |     26.1    |
+//
+//     ** With pre-allocating vertex-list with max-node size
+//        (Note: exact number of vertices are not certain at the beginning)
+//       | OutEdgeList | VertexList | Performance |
+//       |    listS    |   vecS     |     25.2    |
+//       |    vecS     |   vecS     |     23.4    |
+//
+//     * Didn't try adjacency_matrix since our graph is sparse
+//     ( Also one can check BGL book, performance section )
+//
+//    Decision:
+//     * Alpha_expansion_graph_cut_boost: use adjacency_list<vecS, listS> without
+//       pre-allocating vertex-list.
+//
+// 2) With Boykov-Kolmogorov MAXFLOW software:
+//   (http://pub.ist.ac.at/~vnk/software/maxflow-v2.21.src.tar.gz)
+//                                  | Performance |
+//                                  |     3.1     |
+//     * Alpha_expansion_graph_cut_boykov_kolmogorov provides an implementation.
+//       MAXFLOW does not provide any option for pre-allocation (It is possible with v_3.02 though).
+//
+// Typical Benchmark result provided by Ilker
+//                                 | construction of vertices  |  construction of edges    | graph cut  | Total
+//   -----------------------------------------------------------------------------------------------------------
+//   boost with an adjacency list  |         1.53              | 1.51                      |  3.00      | 6.04
+//   boost with CSR                | 0.11 (gather in a vector) | 0.15 (gather in a vector) |  2.67      | 2.93
+//   MaxFlow                       |       0.042               | 0.076                     |  1.043     | 1.161
+//
+// The main issue for now with CSR is the construction of the opposite edge map that is too costly,
+// since it is done by exploring all edges to find opposite
+////////////////////////////////////////////////////////////////////////////////////////
+
+} // namespace internal
+
+/**
+ * @brief Implements alpha-expansion graph cut algorithm.
+ *
+ * For representing graph, it uses adjacency_list with OutEdgeList = vecS, VertexList = listS.
+ * Also no pre-allocation is made for vertex-list.
+ */
+class Alpha_expansion_boost_adjacency_list_impl
+{
+private:
+  typedef boost::adjacency_list_traits<boost::vecS, boost::listS, boost::directedS>
+  Adjacency_list_traits;
+
+  typedef boost::adjacency_list<boost::vecS, boost::listS, boost::directedS,
+          // 4 vertex properties
+          boost::property<boost::vertex_index_t, std::size_t,
+          boost::property<boost::vertex_color_t, boost::default_color_type,
+          boost::property<boost::vertex_distance_t, double,
+          boost::property<boost::vertex_predecessor_t, Adjacency_list_traits::edge_descriptor >
+          > > >,
+          // 3 edge properties
+          boost::property<boost::edge_capacity_t, double,
+          boost::property<boost::edge_residual_capacity_t, double,
+          boost::property<boost::edge_reverse_t, Adjacency_list_traits::edge_descriptor> >
+          > > Graph;
+
+  typedef boost::graph_traits<Graph> Traits;
+  typedef boost::color_traits<boost::default_color_type> ColorTraits;
+
+public:
+
+  typedef Traits::vertex_descriptor Vertex_descriptor;
+  typedef Traits::vertex_iterator   Vertex_iterator;
+  typedef Traits::edge_descriptor   Edge_descriptor;
+  typedef Traits::edge_iterator     Edge_iterator;
+
+private:
+
+  Graph graph;
+  Vertex_descriptor cluster_source;
+  Vertex_descriptor cluster_sink;
+
+public:
+
+  void clear_graph()
+  {
+    graph.clear();
+    cluster_source = boost::add_vertex(graph);
+    cluster_sink = boost::add_vertex(graph);
+  }
+
+  Vertex_descriptor add_vertex()
+  {
+    return boost::add_vertex(graph);
+  }
+
+  void add_tweight (Vertex_descriptor& v, double w1, double w2)
+  {
+    add_edge (cluster_source, v, w1, 0);
+    add_edge (v, cluster_sink, w2, 0);
+  }
+
+  void init_vertices()
+  {
+    // initialize vertex indices, it is necessary since we are using VertexList = listS
+    Vertex_iterator v_begin, v_end;
+    Traits::vertices_size_type index = 0;
+    for(boost::tie(v_begin, v_end) = vertices(graph); v_begin != v_end; ++v_begin) {
+      boost::put(boost::vertex_index, graph, *v_begin, index++);
+    }
+  }
+
+  double max_flow()
+  {
+#if BOOST_VERSION >= 104400
+    return boost::boykov_kolmogorov_max_flow(graph, cluster_source,
+                                                      cluster_sink);
+#else
+    return boost::kolmogorov_max_flow(graph, cluster_source, cluster_sink);
+#endif
+  }
+
+  template <typename VertexLabelMap, typename InputVertexDescriptor>
+  void update(VertexLabelMap vertex_label_map,
+              const std::vector<Vertex_descriptor>& inserted_vertices,
+              InputVertexDescriptor vd,
+              std::size_t vertex_i,
+              std::size_t alpha)
+  {
+    boost::default_color_type color = boost::get(boost::vertex_color, graph,
+                                                 inserted_vertices[vertex_i]);
+    if(std::size_t(get (vertex_label_map, vd)) != alpha
+       && color == ColorTraits::white()) //new comers (expansion occurs)
+      put (vertex_label_map, vd,
+           static_cast<typename boost::property_traits<VertexLabelMap>::value_type>(alpha));
+  }
+
+  void add_edge (Vertex_descriptor& v1, Vertex_descriptor& v2, double w1, double w2)
+  {
+    Edge_descriptor v1_v2, v2_v1;
+    bool v1_v2_added, v2_v1_added;
+
+    boost::tie(v1_v2, v1_v2_added) = boost::add_edge(v1, v2, graph);
+    boost::tie(v2_v1, v2_v1_added) = boost::add_edge(v2, v1, graph);
+
+    CGAL_assertion(v1_v2_added && v2_v1_added);
+    //put edge capacities
+    boost::put(boost::edge_reverse, graph, v1_v2, v2_v1);
+    boost::put(boost::edge_reverse, graph, v2_v1, v1_v2);
+
+    //map reverse edges
+    boost::put(boost::edge_capacity, graph, v1_v2, w1);
+    boost::put(boost::edge_capacity, graph, v2_v1, w2);
+
+  }
+};
+
+// another implementation using compressed_sparse_row_graph
+// for now there is a performance problem while setting reverse edges
+// if that can be solved, it is faster than Alpha_expansion_graph_cut_boost
+class Alpha_expansion_boost_compressed_sparse_row_impl
+{
+private:
+  // CSR only accepts bundled props
+  struct VertexP {
+    boost::default_color_type vertex_color;
+    double vertex_distance_t;
+    // ? do not now there is another way to take it, I think since edge_descriptor does not rely on properties
+    // this should be fine...
+    boost::compressed_sparse_row_graph<boost::directedS>::edge_descriptor
+    vertex_predecessor;
+  };
+
+  struct EdgeP {
+    double edge_capacity;
+    double edge_residual_capacity;
+    boost::compressed_sparse_row_graph<boost::directedS>::edge_descriptor
+    edge_reverse;
+  };
+
+  typedef boost::compressed_sparse_row_graph<boost::directedS,
+          VertexP, EdgeP> Graph;
+
+  typedef boost::graph_traits<Graph> Traits;
+  typedef boost::color_traits<boost::default_color_type> ColorTraits;
+
+public:
+
+  typedef Traits::vertex_descriptor Vertex_descriptor;
+  typedef Traits::vertex_iterator   Vertex_iterator;
+  typedef Traits::edge_descriptor   Edge_descriptor;
+  typedef Traits::edge_iterator     Edge_iterator;
+
+private:
+
+  Graph graph;
+  std::size_t nb_vertices;
+  std::vector<std::pair<std::size_t, std::size_t> > edge_map;
+  std::vector<EdgeP>                edge_map_weights;
+
+public:
+  void clear_graph()
+  {
+    nb_vertices = 2;
+    edge_map.clear();
+    edge_map_weights.clear();
+    // edge_map.reserve(labels.size() *
+    //                  8); // there is no way to know exact edge count, it is a heuristic value
+    // edge_map_weights.reserve(labels.size() * 8);
+  }
+
+  Vertex_descriptor add_vertex()
+  {
+    return (nb_vertices ++);
+  }
+
+  void add_tweight (Vertex_descriptor& v, double w1, double w2)
+  {
+    add_edge (0, v, w1, 0);
+    add_edge (v, 1, w2, 0);
+  }
+
+  void init_vertices()
+  {
+#if BOOST_VERSION >= 104000
+    graph = Graph(boost::edges_are_unsorted, edge_map.begin(), edge_map.end(),
+                  edge_map_weights.begin(), nb_vertices);
+#else
+    graph= Graph(edge_map.begin(), edge_map.end(),
+                 edge_map_weights.begin(), nb_vertices);
+#endif
+
+    // PERFORMANCE PROBLEM
+    // need to set reverse edge map, I guess there is no way to do that before creating the graph
+    // since we do not have edge_descs
+    // however from our edge_map, we know that each (2i, 2i + 1) is reverse pairs, how to facilitate that ?
+    // will look it back
+    Graph::edge_iterator ei, ee;
+    for(boost::tie(ei, ee) = boost::edges(graph); ei != ee; ++ei) {
+      Graph::vertex_descriptor v1 = boost::source(*ei, graph);
+      Graph::vertex_descriptor v2 = boost::target(*ei, graph);
+      std::pair<Graph::edge_descriptor, bool> opp_edge = boost::edge(v2, v1, graph);
+
+      CGAL_assertion(opp_edge.second);
+      graph[opp_edge.first].edge_reverse =
+        *ei; // and edge_reverse of *ei will be (or already have been) set by the opp_edge
+    }
+  }
+
+  double max_flow()
+  {
+#if BOOST_VERSION >= 104400
+    // since properties are bundled, defaults does not work need to specify them
+    return boost::boykov_kolmogorov_max_flow
+      (graph,
+       boost::get(&EdgeP::edge_capacity, graph),
+       boost::get(&EdgeP::edge_residual_capacity, graph),
+       boost::get(&EdgeP::edge_reverse, graph),
+       boost::get(&VertexP::vertex_predecessor, graph),
+       boost::get(&VertexP::vertex_color, graph),
+       boost::get(&VertexP::vertex_distance_t, graph),
+       boost::get(boost::vertex_index,
+                  graph), // this is not bundled, get it from graph (CRS provides one)
+       0, 1);
+#else
+    return boost::kolmogorov_max_flow
+       (graph,
+        boost::get(&EdgeP::edge_capacity, graph),
+        boost::get(&EdgeP::edge_residual_capacity, graph),
+        boost::get(&EdgeP::edge_reverse, graph),
+        boost::get(&VertexP::vertex_predecessor, graph),
+        boost::get(&VertexP::vertex_color, graph),
+        boost::get(&VertexP::vertex_distance_t, graph),
+        boost::get(boost::vertex_index,
+                   graph), // this is not bundled, get it from graph
+        0, 1);
+#endif
+  }
+
+  template <typename VertexLabelMap, typename InputVertexDescriptor>
+  void update(VertexLabelMap vertex_label_map,
+              const std::vector<Vertex_descriptor>&,
+              InputVertexDescriptor vd,
+              std::size_t vertex_i,
+              std::size_t alpha)
+  {
+    boost::default_color_type color =  graph[vertex_i + 2].vertex_color;
+    if(get(vertex_label_map, vd)!= alpha
+       && color == ColorTraits::white()) //new comers (expansion occurs)
+      put(vertex_label_map, vd, alpha);
+  }
+
+  void add_edge(Vertex_descriptor v1, Vertex_descriptor v2, double w1, double w2)
+  {
+    edge_map.push_back(std::make_pair(v1, v2));
+    EdgeP p1;
+    p1.edge_capacity = w1;
+    edge_map_weights.push_back(p1);
+
+    edge_map.push_back(std::make_pair(v2, v1));
+    EdgeP p2;
+    p2.edge_capacity = w2;
+    edge_map_weights.push_back(p2);
+  }
+
+
+};
+
+// tags
+struct Alpha_expansion_boost_adjacency_list_tag { };
+struct Alpha_expansion_boost_compressed_sparse_row_tag { };
+struct Alpha_expansion_MaxFlow_tag { };
+
+// forward declaration
+class Alpha_expansion_MaxFlow_impl;
+
+/// \endcond
+
+// NOTE: latest performances check (2019-07-22)
+//
+// Using a random graph with 50000 vertices, 100000 edges and 30 labels:
+//
+// METHOD                 TIMING           MEMORY
+// Boost Adjacency list   49s              122MiB
+// Boost CSR              187s             77MiB
+// MaxFlow                12s              717MiB
+
+/**
+   \ingroup PkgBGLPartition
+
+   regularizes a partition of a graph into `n` labels using the alpha
+   expansion algorithm \cgalCite{Boykov2001FastApproximate}.
+
+   For a graph \f$(V,E)\f$, this function computes a partition `f`
+   that minimizes the following cost function:
+
+   \f[
+   \mathrm{C}(f) = \sum_{\{v0,v1\} \in E} C_E(v0,v1) + \sum_{v \in V} C_V(f_v)
+   \f]
+
+   where \f$C_E(v0,v1)\f$ is the edge cost of assigning a different
+   label to \f$v0\f$ and \f$v1\f$, and \f$C_V(f_v)\f$ is the vertex
+   cost of assigning the label \f$f\f$ to the vertex \f$v\f$.
+
+   \tparam InputGraph a model of `VertexAndEdgeListGraph`
+
+   \tparam EdgeCostMap a model of `ReadablePropertyMap` with
+   `boost::graph_traits<InputGraph>::%edge_descriptor` as key and `double`
+   as value
+
+   \tparam VertexLabelCostMap a model of `ReadablePropertyMap`
+   with `boost::graph_traits<InputGraph>::%vertex_descriptor` as key and
+   `std::vector<double>` as value
+
+   \tparam VertexLabelMap a model of `ReadWritePropertyMap` with
+   `boost::graph_traits<InputGraph>::%vertex_descriptor` as key and
+   `std::size_t` as value
+
+   \tparam NamedParameters a sequence of named parameters
+
+   \param input_graph the input graph.
+
+   \param edge_cost_map a property map providing the weight of each
+   edge.
+
+   \param vertex_label_map a property map providing the label of each
+   vertex. This map will be updated by the algorithm with the
+   regularized version of the partition.
+
+   \param vertex_label_cost_map a property map providing, for each
+   vertex, an `std::vector` containing the cost of this vertex to
+   belong to each label. Each `std::vector` should have the same size
+   `n` (which is the number of labels), each label being indexed from
+   `0` to `n-1`. For example, `get(vertex_label_cost_map,
+   vd)[label_idx]` returns the cost of vertex `vd` to belong to the
+   label `label_idx`.
+
+   \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+
+   \cgalNamedParamsBegin
+     \cgalParamNBegin{vertex_index_map}
+       \cgalParamDescription{a property map associating to each vertex of `input_graph` a unique index between `0` and `num_vertices(input_graph) - 1`}
+       \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<InputGraph>::%vertex_descriptor`
+                      as key type and `std::size_t` as value type}
+       \cgalParamDefault{an automatically indexed internal map}
+       \cgalParamExtra{If this parameter is not passed, internal machinery will create and initialize
+                       a face index property map, either using the internal property map if it exists
+                       or using an external map. The latter might result in  - slightly - worsened performance
+                       in case of non-constant complexity for index access.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{implementation_tag}
+       \cgalParamDescription{a tag used to select which implementation of the alpha expansion should be used.
+                             Available implementation tags are:
+                             - `CGAL::Alpha_expansion_boost_adjacency_list`
+                             - `CGAL::Alpha_expansion_boost_compressed_sparse_row_tag`
+                             - `CGAL::Alpha_expansion_MaxFlow_tag`}
+       \cgalParamDefault{`CGAL::Alpha_expansion_boost_adjacency_list`}
+     \cgalParamNEnd
+   \cgalNamedParamsEnd
+
+   \note The `MaxFlow` implementation is provided by the \ref PkgSurfaceMeshSegmentationRef
+   under GPL license. The header `<CGAL/boost/graph/Alpha_expansion_MaxFlow_tag.h>`
+   must be included if users want to use this implementation.
+*/
+template <typename InputGraph,
+          typename EdgeCostMap,
+          typename VertexLabelCostMap,
+          typename VertexLabelMap,
+          typename NamedParameters>
+double alpha_expansion_graphcut (const InputGraph& input_graph,
+                                 EdgeCostMap edge_cost_map,
+                                 VertexLabelCostMap vertex_label_cost_map,
+                                 VertexLabelMap vertex_label_map,
+                                 const NamedParameters& np)
+{
+  using parameters::choose_parameter;
+  using parameters::get_parameter;
+
+  typedef boost::graph_traits<InputGraph> GT;
+  typedef typename GT::edge_descriptor input_edge_descriptor;
+  typedef typename GT::vertex_descriptor input_vertex_descriptor;
+
+  typedef typename GetInitializedVertexIndexMap<InputGraph, NamedParameters>::type VertexIndexMap;
+  VertexIndexMap vertex_index_map = CGAL::get_initialized_vertex_index_map(input_graph, np);
+
+  typedef typename GetImplementationTag<NamedParameters>::type Impl_tag;
+
+  // select implementation
+  typedef typename std::conditional
+    <std::is_same<Impl_tag, Alpha_expansion_boost_adjacency_list_tag>::value,
+     Alpha_expansion_boost_adjacency_list_impl,
+     typename std::conditional
+     <std::is_same<Impl_tag, Alpha_expansion_boost_compressed_sparse_row_tag>::value,
+      Alpha_expansion_boost_compressed_sparse_row_impl,
+      Alpha_expansion_MaxFlow_impl>::type>::type
+    Alpha_expansion;
+
+  typedef typename Alpha_expansion::Vertex_descriptor Vertex_descriptor;
+
+  Alpha_expansion alpha_expansion;
+
+  // TODO: check this hardcoded parameter
+  const double tolerance = 1e-10;
+
+  double min_cut = (std::numeric_limits<double>::max)();
+
+#ifdef CGAL_SEGMENTATION_BENCH_GRAPHCUT
+  double vertex_creation_time, edge_creation_time, cut_time;
+  vertex_creation_time = edge_creation_time = cut_time = 0.0;
+#endif
+
+  std::vector<Vertex_descriptor> inserted_vertices;
+  inserted_vertices.resize(num_vertices (input_graph));
+
+  std::size_t number_of_labels = get(vertex_label_cost_map, *(vertices(input_graph).first)).size();
+
+  bool success;
+  do {
+    success = false;
+
+    for (std::size_t alpha = 0; alpha < number_of_labels; ++ alpha)
+    {
+      alpha_expansion.clear_graph();
+
+#ifdef CGAL_SEGMENTATION_BENCH_GRAPHCUT
+      Timer timer;
+      timer.start();
+#endif
+
+      // For E-Data
+      // add every input vertex as a vertex to the graph, put edges to source & sink vertices
+      for (input_vertex_descriptor vd : CGAL::make_range(vertices(input_graph)))
+      {
+        std::size_t vertex_i = get(vertex_index_map, vd);
+        Vertex_descriptor new_vertex = alpha_expansion.add_vertex();
+        inserted_vertices[vertex_i] = new_vertex;
+        double source_weight = get(vertex_label_cost_map, vd)[alpha];
+        // since it is expansion move, current alpha labeled vertices will be assigned to alpha again,
+        // making sink_weight 'infinity' guarantee this.
+        double sink_weight = (std::size_t(get(vertex_label_map, vd)) == alpha ?
+                              (std::numeric_limits<double>::max)()
+                              : get(vertex_label_cost_map, vd)[get(vertex_label_map, vd)]);
+
+        alpha_expansion.add_tweight(new_vertex, source_weight, sink_weight);
+      }
+#ifdef CGAL_SEGMENTATION_BENCH_GRAPHCUT
+      vertex_creation_time += timer.time();
+      timer.reset();
+#endif
+
+      // For E-Smooth
+      // add edge between every vertex,
+      for (input_edge_descriptor ed : CGAL::make_range(edges(input_graph)))
+      {
+        input_vertex_descriptor vd1 = source(ed, input_graph);
+        input_vertex_descriptor vd2 = target(ed, input_graph);
+        std::size_t idx1 = get (vertex_index_map, vd1);
+        std::size_t idx2 = get (vertex_index_map, vd2);
+
+        double weight = get (edge_cost_map, ed);
+
+        Vertex_descriptor v1 = inserted_vertices[idx1],
+          v2 = inserted_vertices[idx2];
+
+        std::size_t label_1 = get (vertex_label_map, vd1);
+        std::size_t label_2 = get (vertex_label_map, vd2);
+        if(label_1 == label_2) {
+          if(label_1 != alpha) {
+            alpha_expansion.add_edge(v1, v2, weight, weight);
+          }
+        } else {
+          Vertex_descriptor inbetween = alpha_expansion.add_vertex();
+
+          double w1 = (label_1 == alpha) ? 0 : weight;
+          double w2 = (label_2 == alpha) ? 0 : weight;
+          alpha_expansion.add_edge(inbetween, v1, w1, w1);
+          alpha_expansion.add_edge(inbetween, v2, w2, w2);
+          alpha_expansion.add_tweight(inbetween, 0., weight);
+        }
+      }
+#ifdef CGAL_SEGMENTATION_BENCH_GRAPHCUT
+      edge_creation_time += timer.time();
+#endif
+
+      alpha_expansion.init_vertices();
+
+#ifdef CGAL_SEGMENTATION_BENCH_GRAPHCUT
+      timer.reset();
+#endif
+
+      double flow = alpha_expansion.max_flow();
+
+#ifdef CGAL_SEGMENTATION_BENCH_GRAPHCUT
+      cut_time += timer.time();
+#endif
+
+      if(min_cut - flow <= flow * tolerance) {
+        continue;
+      }
+      min_cut = flow;
+      success = true;
+      //update labeling
+      for (input_vertex_descriptor vd : CGAL::make_range(vertices (input_graph)))
+      {
+        std::size_t vertex_i = get (vertex_index_map, vd);
+        alpha_expansion.update(vertex_label_map, inserted_vertices, vd, vertex_i, alpha);
+      }
+    }
+  } while(success);
+
+#ifdef CGAL_SEGMENTATION_BENCH_GRAPHCUT
+  CGAL_TRACE_STREAM << "vertex creation time: " << vertex_creation_time <<
+    std::endl;
+  CGAL_TRACE_STREAM << "edge creation time: " << edge_creation_time << std::endl;
+  CGAL_TRACE_STREAM << "max flow algorithm time: " << cut_time << std::endl;
+#endif
+
+  return min_cut;
+}
+
+
+/// \cond SKIP_IN_MANUAL
+// variant with default NP
+template <typename InputGraph,
+          typename EdgeCostMap,
+          typename VertexLabelCostMap,
+          typename VertexLabelMap>
+double alpha_expansion_graphcut (const InputGraph& input_graph,
+                                 EdgeCostMap edge_cost_map,
+                                 VertexLabelCostMap vertex_label_cost_map,
+                                 VertexLabelMap vertex_label_map)
+{
+  return alpha_expansion_graphcut (input_graph, edge_cost_map,
+                                   vertex_label_cost_map, vertex_label_map,
+                                   CGAL::parameters::all_default());
+}
+
+// Old API
+inline double alpha_expansion_graphcut (const std::vector<std::pair<std::size_t, std::size_t> >& edges,
+                                        const std::vector<double>& edge_costs,
+                                        const std::vector<std::vector<double> >& cost_matrix,
+                                        std::vector<std::size_t>& labels)
+{
+  internal::Alpha_expansion_old_API_wrapper_graph graph (edges, edge_costs, cost_matrix, labels);
+  return alpha_expansion_graphcut(graph,
+                                  graph.edge_cost_map(),
+                                  graph.vertex_label_cost_map(),
+                                  graph.vertex_label_map(),
+                                  CGAL::parameters::vertex_index_map (graph.vertex_index_map()));
+}
+
+template <typename AlphaExpansionImplementationTag>
+double alpha_expansion_graphcut (const std::vector<std::pair<std::size_t, std::size_t> >& edges,
+                                 const std::vector<double>& edge_costs,
+                                 const std::vector<std::vector<double> >& cost_matrix,
+                                 std::vector<std::size_t>& labels,
+                                 const AlphaExpansionImplementationTag&)
+{
+  internal::Alpha_expansion_old_API_wrapper_graph graph (edges, edge_costs, cost_matrix, labels);
+
+  return alpha_expansion_graphcut(graph,
+                                  graph.edge_cost_map(),
+                                  graph.vertex_label_cost_map(),
+                                  graph.vertex_label_map(),
+                                  CGAL::parameters::vertex_index_map (graph.vertex_index_map()).
+                                  implementation_tag (AlphaExpansionImplementationTag()));
+}
+/// \endcond
+
+}//namespace CGAL
+
+namespace boost
+{
+
+template <>
+struct property_map<CGAL::internal::Alpha_expansion_old_API_wrapper_graph, boost::vertex_index_t>
+{
+  typedef CGAL::internal::Alpha_expansion_old_API_wrapper_graph::Vertex_index_map type;
+  typedef CGAL::internal::Alpha_expansion_old_API_wrapper_graph::Vertex_index_map const_type;
+};
+}
+
+#endif //CGAL_BOOST_GRAPH_ALPHA_EXPANSION_GRAPHCUT_H
diff -pruN 5.0.3-1/include/CGAL/boost/graph/Alpha_expansion_MaxFlow_tag.h 5.1-1/include/CGAL/boost/graph/Alpha_expansion_MaxFlow_tag.h
--- 5.0.3-1/include/CGAL/boost/graph/Alpha_expansion_MaxFlow_tag.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/Alpha_expansion_MaxFlow_tag.h	2020-09-08 12:15:44.000000000 +0000
@@ -0,0 +1,94 @@
+#ifndef CGAL_BOOST_GRAPH_ALPHA_EXPANSION_MAXFLOW_IMPL_H
+// Copyright (c) 2014  GeometryFactory Sarl (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_segmentation/include/CGAL/boost/graph/Alpha_expansion_MaxFlow_tag.h $
+// $Id: Alpha_expansion_MaxFlow_tag.h 47027d6 2020-03-26T18:59:19+01:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Ilker O. Yaz, Simon Giraudot
+
+#define CGAL_BOOST_GRAPH_ALPHA_EXPANSION_MAXFLOW_IMPL_H
+
+#include <CGAL/license/Surface_mesh_segmentation.h>
+
+/// \cond SKIP_IN_MANUAL
+
+#include <CGAL/boost/graph/alpha_expansion_graphcut.h>
+
+namespace MaxFlow
+{
+#include <CGAL/internal/auxiliary/graph.h>
+}
+
+
+namespace CGAL
+{
+
+/**
+ * @brief Implements alpha-expansion graph cut algorithm.
+ *
+ * For underlying max-flow algorithm, it uses the MAXFLOW software implemented by Boykov & Kolmogorov.
+ *  Also no pre-allocation is made.
+ */
+class Alpha_expansion_MaxFlow_impl
+{
+public:
+
+  typedef MaxFlow::Graph::node_id Vertex_descriptor;
+
+private:
+
+  MaxFlow::Graph graph;
+
+public:
+
+  void clear_graph()
+  {
+    graph = MaxFlow::Graph();
+  }
+
+  Vertex_descriptor add_vertex()
+  {
+    return graph.add_node();
+  }
+
+  void add_tweight (Vertex_descriptor& v, double w1, double w2)
+  {
+    graph.add_tweights(v, w1, w2);
+  }
+
+  void init_vertices()
+  {
+  }
+
+  double max_flow()
+  {
+    return graph.maxflow();
+  }
+
+  template <typename VertexLabelMap, typename InputVertexDescriptor>
+  void update(VertexLabelMap vertex_label_map,
+              const std::vector<Vertex_descriptor>& inserted_vertices,
+              InputVertexDescriptor vd,
+              std::size_t vertex_i,
+              std::size_t alpha)
+  {
+    if(get(vertex_label_map, vd) != alpha
+       && graph.what_segment(inserted_vertices[vertex_i]) == MaxFlow::Graph::SINK)
+      put(vertex_label_map, vd, alpha);
+  }
+
+  void add_edge (Vertex_descriptor& v1, Vertex_descriptor& v2, double w1, double w2)
+  {
+    graph.add_edge(v1, v2, w1, w2);
+  }
+};
+
+}//namespace CGAL
+
+/// \endcond
+
+#endif //CGAL_BOOST_GRAPH_ALPHA_EXPANSION_MAXFLOW_IMPL_H
diff -pruN 5.0.3-1/include/CGAL/boost/graph/backward_compatibility_functions.h 5.1-1/include/CGAL/boost/graph/backward_compatibility_functions.h
--- 5.0.3-1/include/CGAL/boost/graph/backward_compatibility_functions.h	2020-08-01 19:02:14.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/backward_compatibility_functions.h	2020-09-08 12:14:40.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/backward_compatibility_functions.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/backward_compatibility_functions.h $
 // $Id: backward_compatibility_functions.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/convert_nef_polyhedron_to_polygon_mesh.h 5.1-1/include/CGAL/boost/graph/convert_nef_polyhedron_to_polygon_mesh.h
--- 5.0.3-1/include/CGAL/boost/graph/convert_nef_polyhedron_to_polygon_mesh.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/convert_nef_polyhedron_to_polygon_mesh.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/boost/graph/convert_nef_polyhedron_to_polygon_mesh.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/boost/graph/convert_nef_polyhedron_to_polygon_mesh.h $
 // $Id: convert_nef_polyhedron_to_polygon_mesh.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/copy_face_graph.h 5.1-1/include/CGAL/boost/graph/copy_face_graph.h
--- 5.0.3-1/include/CGAL/boost/graph/copy_face_graph.h	2020-08-01 19:02:14.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/copy_face_graph.h	2020-09-08 12:14:40.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/copy_face_graph.h $
-// $Id: copy_face_graph.h 437dec1 2020-03-20T15:42:48+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/copy_face_graph.h $
+// $Id: copy_face_graph.h 93f1cd9 2020-07-16T09:53:31+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -40,7 +40,6 @@ void copy_face_graph_impl(const SourceMe
 {
   typedef typename boost::graph_traits<SourceMesh>::vertex_descriptor sm_vertex_descriptor;
   typedef typename boost::graph_traits<TargetMesh>::vertex_descriptor tm_vertex_descriptor;
-  typedef typename boost::graph_traits<TargetMesh>::halfedge_iterator tm_halfedge_iterator;
 
   typedef typename boost::graph_traits<SourceMesh>::face_descriptor sm_face_descriptor;
   typedef typename boost::graph_traits<TargetMesh>::face_descriptor tm_face_descriptor;
@@ -64,14 +63,17 @@ void copy_face_graph_impl(const SourceMe
   const tm_face_descriptor tm_null_face = boost::graph_traits<TargetMesh>::null_face();
   const tm_vertex_descriptor tm_null_vertex = boost::graph_traits<TargetMesh>::null_vertex();
 
-  reserve(tm, static_cast<typename boost::graph_traits<TargetMesh>::vertices_size_type>(vertices(sm).size()),
-              static_cast<typename boost::graph_traits<TargetMesh>::edges_size_type>(edges(sm).size()),
-              static_cast<typename boost::graph_traits<TargetMesh>::faces_size_type>(faces(sm).size()) );
+  reserve(tm, static_cast<typename boost::graph_traits<TargetMesh>::vertices_size_type>(vertices(tm).size()+vertices(sm).size()),
+              static_cast<typename boost::graph_traits<TargetMesh>::edges_size_type>(edges(tm).size()+edges(sm).size()),
+              static_cast<typename boost::graph_traits<TargetMesh>::faces_size_type>(faces(tm).size()+faces(sm).size()) );
 
   //insert halfedges and create each vertex when encountering its halfedge
+  std::vector<tm_edge_descriptor> new_edges;
+  new_edges.reserve(edges(sm).size());
   for(sm_edge_descriptor sm_e : edges(sm))
   {
     tm_edge_descriptor tm_e = add_edge(tm);
+    new_edges.push_back(tm_e);
     sm_halfedge_descriptor sm_h = halfedge(sm_e, sm), sm_h_opp = opposite(sm_h, sm);
     tm_halfedge_descriptor tm_h = halfedge(tm_e, tm), tm_h_opp = opposite(tm_h, tm);
 
@@ -173,9 +175,10 @@ void copy_face_graph_impl(const SourceMe
   }
 
   // detect if there are some non-manifold umbrellas and fix missing halfedge target pointers
-  for (tm_halfedge_iterator it=halfedges(tm).first; it!=halfedges(tm).second; ++it)
+  typedef typename std::vector<tm_edge_descriptor>::iterator edge_iterator;
+  for (edge_iterator it=new_edges.begin(); it!=new_edges.end(); ++it)
   {
-    if (target(*it, tm) == tm_null_vertex)
+    if (target(*it, tm) == tm_null_vertex || source(*it, tm) == tm_null_vertex)
     {
       // create and fill a map from target halfedge to source halfedge
       typedef CGAL::dynamic_halfedge_property_t<sm_halfedge_descriptor> Dyn_th_tag;
@@ -183,17 +186,22 @@ void copy_face_graph_impl(const SourceMe
       for (sm_halfedge_descriptor hs : halfedges(sm))
         put(ht_to_hs, get(hs_to_ht, hs), hs);
 
-      for(; it!=halfedges(tm).second; ++it)
+      for(; it!=new_edges.end(); ++it)
       {
-        if (target(*it, tm) == tm_null_vertex)
+        tm_halfedge_descriptor nh_t = halfedge(*it, tm);
+        for (int i=0; i<2; ++i)
         {
-          // we recover tm_v using the halfedge associated to the target vertex of
-          // the halfedge in sm corresponding to *it. This is working because we
-          // set the vertex halfedge pointer to the "same" halfedges.
-          tm_vertex_descriptor tm_v =
-            target( get(hs_to_ht, halfedge(target(get(ht_to_hs, *it), sm), sm)), tm);
-          for(tm_halfedge_descriptor ht : halfedges_around_target(*it, tm))
-            set_target(ht, tm_v, tm);
+          if (target(nh_t, tm) == tm_null_vertex)
+          {
+            // we recover tm_v using the halfedge associated to the target vertex of
+            // the halfedge in sm corresponding to nh_t. This is working because we
+            // set the vertex halfedge pointer to the "same" halfedges.
+            tm_vertex_descriptor tm_v =
+              target( get(hs_to_ht, halfedge(target(get(ht_to_hs, nh_t), sm), sm)), tm);
+            for(tm_halfedge_descriptor ht : halfedges_around_target(nh_t, tm))
+              set_target(ht, tm_v, tm);
+          }
+          nh_t = opposite(nh_t, tm);
         }
       }
       break;
@@ -246,58 +254,97 @@ inline Emptyset_iterator make_functor(co
           and `boost::graph_traits<SourceMesh>::%face_descriptor` must be
           models of `Hashable`.
   \tparam TargetMesh a model of `FaceListGraph`
-  \tparam NamedParameters1 a sequence of \ref pmp_namedparameters "Named Parameters"
-  \tparam NamedParameters2 a sequence of \ref pmp_namedparameters "Named Parameters"
+  \tparam NamedParameters1 a sequence of \ref bgl_namedparameters "Named Parameters"
+  \tparam NamedParameters2 a sequence of \ref bgl_namedparameters "Named Parameters"
 
   The types `sm_vertex_descriptor` and `sm_face_descriptor` must be models of the concept `Hashable`.
 
   \param sm the source mesh
   \param tm the target mesh
-  \param np1 optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+  \param np1 an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
   \cgalNamedParamsBegin
-    \cgalParamBegin{vertex_point_map}
-      the property map with the points associated to the vertices of `sm` .
-      If this parameter is omitted, an internal property map for
-      `CGAL::vertex_point_t` should be available in `SourceMesh`
-    \cgalParamEnd
-    \cgalParamBegin{vertex_to_vertex_output_iterator} an `OutputIterator` containing the
-      pairs source-vertex, target-vertex. If this parameter is given, then
-      `vertex_to_vertex_map` cannot be used.
-    \cgalParamEnd
-    \cgalParamBegin{halfedge_to_halfedge_output_iterator} an `OutputIterator` containing the
-      pairs source-halfedge, target-halfedge. If this parameter is given, then
-      `halfedge_to_halfedge_map` cannot be used.
-    \cgalParamEnd
-    \cgalParamBegin{face_to_face_output_iterator} an `OutputIterator` containing the
-      pairs source-face, target-face. If this parameter is given, then
-      `face_to_face_map` cannot be used.
-    \cgalParamEnd
-    \cgalParamBegin{vertex_to_vertex_map} a `ReadWritePropertyMap` containing the
-      pairs source-vertex, target-vertex.
-    \cgalParamEnd
-    \cgalParamBegin{halfedge_to_halfedge_map} a `ReadWritePropertyMap` containing the
-      pairs source-halfedge, target-halfedge.
-    \cgalParamEnd
-    \cgalParamBegin{face_to_face_map} a `ReadWritePropertyMap` containing the
-      pairs source-face, target-face.
-    \cgalParamEnd
+    \cgalParamNBegin{vertex_point_map}
+      \cgalParamDescription{a property map associating points to the vertices of `sm`}
+      \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<SourceMesh>::%vertex_descriptor`
+                     as key type and `%Point_3` as value type}
+      \cgalParamDefault{`boost::get(CGAL::vertex_point, sm)`}
+      \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+                      must be available in `SourceMesh`.}
+    \cgalParamNEnd
+
+    \cgalParamNBegin{vertex_to_vertex_output_iterator}
+      \cgalParamDescription{an `OutputIterator` containing the pairs source-vertex, target-vertex.}
+      \cgalParamType{a class model of `OutputIterator` accepting
+                     `std::pair<`boost::graph_traits<SourceMesh>::%vertex_descriptor, `boost::graph_traits<TargetMesh>::%vertex_descriptor>`}
+      \cgalParamDefault{`Emptyset_iterator`}
+      \cgalParamExtra{If this parameter is given, then `vertex_to_vertex_map` cannot be used.}
+    \cgalParamNEnd
+
+    \cgalParamNBegin{halfedge_to_halfedge_output_iterator}
+      \cgalParamDescription{an `OutputIterator` containing the pairs source-halfedge, target-halfedge.}
+      \cgalParamType{a class model of `OutputIterator` accepting
+                     `std::pair<`boost::graph_traits<SourceMesh>::%halfedge_descriptor, `boost::graph_traits<TargetMesh>::%halfedge_descriptor>`}
+      \cgalParamDefault{`Emptyset_iterator`}
+      \cgalParamExtra{If this parameter is given, then `halfedge_to_halfedge_map` cannot be used.}
+    \cgalParamNEnd
+
+    \cgalParamNBegin{face_to_face_output_iterator}
+      \cgalParamDescription{an `OutputIterator` containing the pairs source-face, target-face.}
+      \cgalParamType{a class model of `OutputIterator` accepting
+                     `std::pair<`boost::graph_traits<SourceMesh>::%face_descriptor, `boost::graph_traits<TargetMesh>::%face_descriptor>`}
+      \cgalParamDefault{`Emptyset_iterator`}
+      \cgalParamExtra{If this parameter is given, then `face_to_face_map` cannot be used.}
+    \cgalParamNEnd
+
+    \cgalParamNBegin{vertex_to_vertex_map}
+      \cgalParamDescription{a property map storing for each vertex of a source mesh the corresponding vertex of another mesh}
+      \cgalParamType{a class model of `ReadWritePropertyMap` with
+                     `boost::graph_traits<SourceMesh>::%vertex_descriptor` as key type and
+                     `boost::graph_traits<TargetMesh>::%vertex_descriptor` as value type.}
+      \cgalParamDefault{unused}
+      \cgalParamExtra{A typical use case is mapping the vertices from a source mesh
+                      to its copy's after a `copy_face_graph()` operation.}
+    \cgalParamNEnd
+
+    \cgalParamNBegin{halfedge_to_halfedge_map}
+      \cgalParamDescription{a property map storing for each halfedge of a source mesh the corresponding halfedge of another mesh}
+      \cgalParamType{a class model of `ReadWritePropertyMap` with
+                     `boost::graph_traits<SourceMesh>::%halfedge_descriptor` as key type and
+                     `boost::graph_traits<TargetMesh>::%halfedge_descriptor` as value type}
+      \cgalParamDefault{unused}
+      \cgalParamExtra{A typical use case is mapping the halfedges from a source mesh to its copy's
+                      after a `copy_face_graph()`operation.}
+    \cgalParamNEnd
+
+    \cgalParamNBegin{face_to_face_map}
+      \cgalParamDescription{a property map storing for each face of a source mesh the corresponding face of another mesh}
+      \cgalParamType{a class model of `ReadWritePropertyMap` with
+                     `boost::graph_traits<SourceMesh>::%face_descriptor` as key type and
+                     `boost::graph_traits<TargetMesh>::%face_descriptor` as value type}
+      \cgalParamDefault{unused}
+      \cgalParamExtra{A typical use case is mapping the faces from a source mesh to its copy's
+                      after a `copy_face_graph()` operation}
+    \cgalParamNEnd
   \cgalNamedParamsEnd
 
-  \param np2 optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+  \param np2 an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
   \cgalNamedParamsBegin
-    \cgalParamBegin{vertex_point_map}
-      the property map with the points associated to the vertices of `tm`.
-      If this parameter is omitted, an internal property map for
-      `CGAL::vertex_point_t` should be available in `TargetMesh`
-    \cgalParamEnd
+    \cgalParamNBegin{vertex_point_map}
+      \cgalParamDescription{a property map associating points to the vertices of `tm`}
+      \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<TargetMesh>::%vertex_descriptor`
+                     as key type and `%Point_3` as value type}
+      \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+      \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+                      must be available in `TargetMesh`.}
+    \cgalParamNEnd
   \cgalNamedParamsEnd
 
   The points from `sm` to `tm` are converted using
   `CGAL::Cartesian_converter<SourceKernel, TargetKernel>`.
   `SourceKernel` and `TargetKernel` are deduced using `CGAL::Kernel_traits`
-  from the value types of the vertex_point_maps.
+  from the value types of the vertex point maps.
 
   Other properties are not copied.
 */
@@ -322,19 +369,17 @@ void copy_face_graph(const SourceMesh& s
   using parameters::choose_parameter;
   using parameters::get_parameter;
 
-  internal::copy_face_graph_impl(
-    sm,
-    tm,
-    choose_parameter(get_parameter(np1, internal_np::vertex_to_vertex_output_iterator),
-                 impl::make_functor(get_parameter(np1, internal_np::vertex_to_vertex_map))),
-    choose_parameter(get_parameter(np1, internal_np::halfedge_to_halfedge_output_iterator),
-                 impl::make_functor(get_parameter(np1, internal_np::halfedge_to_halfedge_map))),
-    choose_parameter(get_parameter(np1, internal_np::face_to_face_output_iterator),
-                 impl::make_functor(get_parameter(np1, internal_np::face_to_face_map))),
-    choose_parameter(get_parameter(np1, internal_np::vertex_point),
-                 get(vertex_point, sm)),
-    choose_parameter(get_parameter(np2, internal_np::vertex_point),
-                 get(vertex_point, tm)));
+  internal::copy_face_graph_impl(sm, tm,
+                            choose_parameter(get_parameter(np1, internal_np::vertex_to_vertex_output_iterator),
+                                             impl::make_functor(get_parameter(np1, internal_np::vertex_to_vertex_map))),
+                            choose_parameter(get_parameter(np1, internal_np::halfedge_to_halfedge_output_iterator),
+                                             impl::make_functor(get_parameter(np1, internal_np::halfedge_to_halfedge_map))),
+                            choose_parameter(get_parameter(np1, internal_np::face_to_face_output_iterator),
+                                             impl::make_functor(get_parameter(np1, internal_np::face_to_face_map))),
+                            choose_parameter(get_parameter(np1, internal_np::vertex_point),
+                                             get(vertex_point, sm)),
+                            choose_parameter(get_parameter(np2, internal_np::vertex_point),
+                                             get(vertex_point, tm)));
 }
 
 template <typename SourceMesh, typename TargetMesh>
diff -pruN 5.0.3-1/include/CGAL/boost/graph/dijkstra_shortest_paths.h 5.1-1/include/CGAL/boost/graph/dijkstra_shortest_paths.h
--- 5.0.3-1/include/CGAL/boost/graph/dijkstra_shortest_paths.h	2020-08-01 19:02:14.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/dijkstra_shortest_paths.h	2020-09-08 12:14:40.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/dijkstra_shortest_paths.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/dijkstra_shortest_paths.h $
 // $Id: dijkstra_shortest_paths.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/dijkstra_shortest_paths.hpp 5.1-1/include/CGAL/boost/graph/dijkstra_shortest_paths.hpp
--- 5.0.3-1/include/CGAL/boost/graph/dijkstra_shortest_paths.hpp	2020-08-01 19:02:14.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/dijkstra_shortest_paths.hpp	2020-09-08 12:14:40.000000000 +0000
@@ -13,7 +13,7 @@
 // https://www.boost.org/LICENSE_1_0.txt)
 //=======================================================================
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/dijkstra_shortest_paths.hpp $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/dijkstra_shortest_paths.hpp $
 // $Id: dijkstra_shortest_paths.hpp 590612a 2020-03-26T13:25:16+01:00 Sébastien Loriot
 // SPDX-License-Identifier: BSL-1.0
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/Dual.h 5.1-1/include/CGAL/boost/graph/Dual.h
--- 5.0.3-1/include/CGAL/boost/graph/Dual.h	2020-08-01 19:02:14.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/Dual.h	2020-09-08 12:14:40.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/Dual.h $
-// $Id: Dual.h 2dd06eb 2020-04-13T10:44:26+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/Dual.h $
+// $Id: Dual.h ebd65c9 2020-07-20T11:47:30+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -56,18 +56,18 @@ public:
   /*! The underlying primal type. */
   typedef Primal_ Primal;
 
-  /*! Construct a Dual from a given primal. */
+  /*! constructs a Dual from a given primal. */
   Dual(const Primal& primal)
     : primal_(primal) {}
 
-  /*! Returns the underlying primal. */
+  /*! returns the underlying primal. */
   const Primal& primal() const
   { return primal_; }
 };
 
 
 /*!
-  Construct a `Dual` from a given `primal`.
+  constructs a `Dual` from a given `primal`.
   \relates CGAL::Dual
  */
 template<typename Primal>
diff -pruN 5.0.3-1/include/CGAL/boost/graph/Euler_operations.h 5.1-1/include/CGAL/boost/graph/Euler_operations.h
--- 5.0.3-1/include/CGAL/boost/graph/Euler_operations.h	2020-08-01 19:02:14.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/Euler_operations.h	2020-09-08 12:14:40.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/Euler_operations.h $
-// $Id: Euler_operations.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/Euler_operations.h $
+// $Id: Euler_operations.h 38d1745 2020-04-10T17:57:17+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -21,6 +21,9 @@
 #include <CGAL/boost/graph/helpers.h>
 #include <CGAL/boost/graph/internal/helpers.h>
 #include <CGAL/boost/graph/iterator.h>
+#include <CGAL/boost/graph/named_params_helper.h>
+
+#include <boost/container/small_vector.hpp>
 
 namespace CGAL {
 
@@ -753,6 +756,231 @@ add_face(const VertexRange& vr, Graph& g
   return f;
 }
 
+// TODO: add a visitor for new edge/vertex/face created
+// TODO: doc (VertexRange is random access for now, making a copy to a vector as an noticeable impact on the runtime)
+// TODO: handle and return false in case of non valid input?
+// An interesting property of this function is that in case the mesh contains non-manifold boundary vertices,
+// the connected components of faces incident to such a vertex will not be linked together around the
+// vertex (boundary edges are connected by turning around the vertex in the interior of the mesh).
+// This produce a deterministic behavior for non-manifold vertices.
+template <class PolygonMesh, class RangeofVertexRange>
+void add_faces(const RangeofVertexRange& faces_to_add, PolygonMesh& pm)
+{
+  typedef typename boost::graph_traits<PolygonMesh> GT;
+  typedef typename GT::halfedge_descriptor halfedge_descriptor;
+  typedef typename GT::edge_descriptor edge_descriptor;
+  typedef typename GT::vertex_descriptor vertex_descriptor;
+  typedef typename GT::face_descriptor face_descriptor;
+
+  typedef typename RangeofVertexRange::const_iterator VTR_const_it;
+  typedef typename std::iterator_traits<VTR_const_it>::value_type Vertex_range;
+
+  typedef boost::container::small_vector<halfedge_descriptor,8> Halfedges;
+
+  typedef typename CGAL::GetInitializedVertexIndexMap<PolygonMesh>::type Vid_map;
+  Vid_map vid = CGAL::get_initialized_vertex_index_map(pm);
+
+  // TODO: add also this lambda as an Euler function?
+  auto add_new_edge = [&pm](vertex_descriptor v1, vertex_descriptor v2)
+  {
+    halfedge_descriptor v1v2 = halfedge(add_edge(pm), pm), v2v1=opposite(v1v2, pm);
+    if (halfedge(v1,pm)==GT::null_halfedge()) set_halfedge(v1, v2v1, pm);
+    if (halfedge(v2,pm)==GT::null_halfedge()) set_halfedge(v2, v1v2, pm);
+    set_target(v1v2, v2, pm);
+    set_target(v2v1, v1, pm);
+    set_next(v1v2,v2v1, pm);
+    set_next(v2v1,v1v2, pm);
+    return v1v2;
+  };
+
+  // used to collect existing border halfedges that will no longer be on the border.
+  // Some update is needed in case of non-manifold vertex at the source/target of those
+  // edges are present.
+  std::vector<halfedge_descriptor> former_border_hedges;
+
+  std::vector<Halfedges> outgoing_hedges(num_vertices(pm));
+
+  for (const Vertex_range& vr : faces_to_add)
+  {
+    std::size_t nbh=vr.size();
+    for (std::size_t i=0; i<nbh; ++i)
+    {
+      vertex_descriptor v1=vr[i], v2=vr[(i+1)%nbh];
+      std::pair<edge_descriptor, bool> edge_and_bool = edge(v1, v2, pm);
+      if (v2<v1){
+        // needed in case an existing border edge won't be found
+        // because the outgoing edge from the smallest vertex is on the patch boundary
+        if (edge_and_bool.second && is_border(halfedge(edge_and_bool.first, pm), pm))
+        {
+          outgoing_hedges[get(vid,v2)].push_back(opposite(halfedge(edge_and_bool.first, pm), pm));
+          former_border_hedges.push_back(halfedge(edge_and_bool.first, pm));
+        }
+        continue;
+      }
+      if (edge_and_bool.second)
+      {
+        halfedge_descriptor h = halfedge(edge_and_bool.first, pm);
+        outgoing_hedges[get(vid,v1)].push_back(h);
+        if (is_border(h, pm))
+          former_border_hedges.push_back(h);
+      }
+      else
+        outgoing_hedges[get(vid,v1)].push_back(add_new_edge(v1,v2));
+      CGAL_assertion( source(outgoing_hedges[get(vid,v1)].back(), pm)==v1 );
+      CGAL_assertion( target(outgoing_hedges[get(vid,v1)].back(), pm)==v2 );
+    }
+  }
+
+  // disconnect hand-fans (umbrellas being not affected) at non-manifold vertices
+  // in case the location on the boundary of the mesh where they are attached is closed.
+  // Note that we link the boundary of the hand fans together, making them
+  // independant boundary cycles (even if the non-manifold vertex is not duplicated)
+  if ( !former_border_hedges.empty() )
+  {
+    std::sort(former_border_hedges.begin(), former_border_hedges.end()); // TODO: is it better to use a dynamic pmap?
+    for (halfedge_descriptor h : former_border_hedges)
+    {
+    // update link around target vertex
+      halfedge_descriptor nh = next(h, pm);
+      if ( !std::binary_search(former_border_hedges.begin(), former_border_hedges.end(), nh) )
+      {
+        do
+        {
+          // look for a new prev for h
+          halfedge_descriptor candidate = opposite(next(opposite(nh, pm), pm), pm);
+          while (!is_border(candidate, pm))
+            candidate = opposite(next(candidate, pm), pm);
+          halfedge_descriptor for_next_iteration = next(candidate, pm);
+          set_next(candidate, nh, pm);
+          nh = for_next_iteration;
+          if (candidate==h) break; // stop condition for a vertex that will stay on the boundary after the operation
+          if ( std::binary_search(former_border_hedges.begin(), former_border_hedges.end(), nh) )
+          {
+            // linking halfedges that will no longer be on the boundary
+            set_next(h, nh, pm);
+            break;
+          }
+        }
+        while(true);
+      }
+
+
+    // update link around source vertex
+      halfedge_descriptor ph = prev(h, pm);
+      if ( !std::binary_search(former_border_hedges.begin(), former_border_hedges.end(), ph) )
+      {
+        do
+        {
+          // look for a new next for h
+          halfedge_descriptor candidate = opposite(prev(opposite(ph, pm), pm), pm);
+          while (!is_border(candidate, pm))
+            candidate = opposite(prev(candidate, pm), pm);
+          halfedge_descriptor for_next_iteration = prev(candidate, pm);
+          set_next(ph, candidate, pm);
+          ph = for_next_iteration;
+          if (candidate==h) break;; // stop condition for a vertex that will stay on the boundary after the operation
+          if( std::binary_search(former_border_hedges.begin(), former_border_hedges.end(), ph) )
+          {
+            // linking halfedges that will no longer be on the boundary
+            set_next(ph, h, pm);
+            break;
+          }
+        }
+        while(true);
+      }
+    }
+  }
+
+  for (Halfedges& hedges: outgoing_hedges)
+  {
+    if (!hedges.empty())
+      std::sort(hedges.begin(), hedges.end(), [&pm](halfedge_descriptor h1, halfedge_descriptor h2)
+                                              {
+                                                return target(h1, pm) < target(h2,pm);
+                                              });
+  }
+  std::vector<halfedge_descriptor> new_border_halfedges;
+  auto get_hedge = [&pm, &add_new_edge, &new_border_halfedges, &outgoing_hedges,&vid](vertex_descriptor v1, vertex_descriptor v2)
+  {
+    bool return_opposite = v2 < v1;
+    if (return_opposite) std::swap(v1,v2);
+    const Halfedges& v1_outgoing_hedges = outgoing_hedges[get(vid,v1)];
+    typename Halfedges::const_iterator it_find =
+      std::lower_bound(v1_outgoing_hedges.begin(),
+                       v1_outgoing_hedges.end(),
+                       v2, [&pm](halfedge_descriptor h, vertex_descriptor v){return target(h,pm) < v;});
+    if (it_find!=v1_outgoing_hedges.end() && target(*it_find, pm)==v2)
+    {
+      return return_opposite ? opposite(*it_find, pm) : *it_find;
+    }
+
+    // fall onto a border edge
+    halfedge_descriptor v1v2=add_new_edge(v1,v2);
+    if (return_opposite)
+    {
+      new_border_halfedges.push_back(v1v2);
+      return opposite(v1v2, pm);
+    }
+    new_border_halfedges.push_back(opposite(v1v2, pm));
+    return v1v2;
+  };
+
+  // link interior halfedges
+  for (const Vertex_range& vr : faces_to_add)
+  {
+    std::size_t nbh=vr.size();
+    face_descriptor f = add_face(pm);
+    halfedge_descriptor first = get_hedge(vr[nbh-1],vr[0]), prev=first;
+    set_halfedge(f, first, pm);
+    set_face(first, f, pm);
+    for(std::size_t i=0; i<nbh-1; ++i)
+    {
+      halfedge_descriptor curr = get_hedge(vr[i], vr[i+1]);
+      set_face(curr, f, pm);
+      set_next(prev, curr, pm);
+      prev=curr;
+    }
+    set_next(prev, first, pm);
+  }
+
+  // link border halfedges by turning around the vertex in the interior of the mesh
+  for (const Halfedges& hedges : outgoing_hedges)
+  {
+    for (halfedge_descriptor h : hedges)
+    {
+      halfedge_descriptor hopp = opposite(h, pm);
+      if (is_border(h, pm) && next(h, pm)==hopp)
+        new_border_halfedges.push_back(h);
+      if (is_border(hopp, pm) && next(hopp, pm)==h)
+        new_border_halfedges.push_back(hopp);
+    }
+  }
+  for (halfedge_descriptor h : new_border_halfedges)
+  {
+    CGAL_assertion(is_border(h, pm));
+    halfedge_descriptor hopp = opposite(h, pm);
+    // look around the target
+    if (next(h, pm)==hopp)
+    {
+      halfedge_descriptor candidate = hopp;
+      while(!is_border(candidate, pm))
+      {
+        candidate = opposite(prev(candidate, pm), pm);
+      }
+      set_next(h, candidate, pm);
+    }
+    //look around the source
+    if (prev(h, pm)==hopp)
+    {
+      halfedge_descriptor candidate = hopp;
+      while(!is_border(candidate, pm))
+      {
+        candidate = opposite(next(candidate, pm), pm);
+      }
+      set_next(candidate, h, pm);
+    }
+  }
+}
 
   /**
    * removes the incident face of `h` and changes all halfedges incident to the face into border halfedges. See `remove_face(g,h)` for a more generalized variant.
diff -pruN 5.0.3-1/include/CGAL/boost/graph/Face_filtered_graph.h 5.1-1/include/CGAL/boost/graph/Face_filtered_graph.h
--- 5.0.3-1/include/CGAL/boost/graph/Face_filtered_graph.h	2020-08-01 19:02:14.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/Face_filtered_graph.h	2020-09-08 12:14:40.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/Face_filtered_graph.h $
-// $Id: Face_filtered_graph.h 2782bf0 2020-03-13T06:01:10+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/Face_filtered_graph.h $
+// $Id: Face_filtered_graph.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -16,23 +16,24 @@
 #include <CGAL/boost/graph/properties.h>
 #include <CGAL/boost/graph/iterator.h>
 #include <CGAL/boost/graph/Named_function_parameters.h>
+#include <CGAL/boost/graph/named_params_helper.h>
 #include <CGAL/boost/graph/helpers.h>
+#include <CGAL/boost/iterator/transform_iterator.hpp>
+#include <CGAL/Default.h>
 #include <CGAL/Dynamic_property_map.h>
-#include <CGAL/assertions.h>
-#include <boost/unordered_set.hpp>
+
+#include <boost/dynamic_bitset.hpp>
 #include <boost/graph/graph_traits.hpp>
-#include <CGAL/boost/iterator/transform_iterator.hpp>
 #include <boost/iterator/filter_iterator.hpp>
-#include <boost/dynamic_bitset.hpp>
 #include <boost/range/has_range_iterator.hpp>
+#include <boost/unordered_set.hpp>
 
 #ifdef DOXYGEN_RUNNING
 #define CGAL_BGL_NP_TEMPLATE_PARAMETERS NamedParameters
 #define CGAL_BGL_NP_CLASS NamedParameters
 #endif
 
-namespace CGAL
-{
+namespace CGAL {
 
   /*!
    * \ingroup PkgBGLAdaptors
@@ -62,18 +63,18 @@ namespace CGAL
    * missing if the default is fine.
    *
    * \tparam Graph must be a model of a `FaceListGraph`, `HalfedgeListGraph`, and \bgllink{VertexListGraph}.
-   * \tparam FIMap a model of `ReadablePropertyMap` with `face_descriptor` as key and `graph_traits<Graph>::%faces_size_type` as value
-   * \tparam VIMap a model of `ReadablePropertyMap` with `vertex_descriptor` as key and `graph_traits<Graph>::%vertices_size_type` as value
-   * \tparam HIMap a model of `ReadablePropertyMap` with `halfedge_descriptor` as key and `graph_traits<Graph>::%halfedges_size_type` as value
+   * \tparam FIMap a model of `ReadablePropertyMap` with `graph_traits<Graph>::%face_descriptor` as key and `graph_traits<Graph>::%faces_size_type` as value
+   * \tparam VIMap a model of `ReadablePropertyMap` with `graph_traits<Graph>::%vertex_descriptor` as key and `graph_traits<Graph>::%vertices_size_type` as value
+   * \tparam HIMap a model of `ReadablePropertyMap` with `graph_traits<Graph>::%halfedge_descriptor` as key and `graph_traits<Graph>::%halfedges_size_type` as value
    *
    * \cgalModels `FaceListGraph`
    * \cgalModels `HalfedgeListGraph`
    * \cgalModels \bgllink{VertexListGraph}
    */
 template<typename Graph,
-         typename FIMap = typename boost::property_map<Graph, CGAL::face_index_t>::type,
-         typename VIMap = typename boost::property_map<Graph, boost::vertex_index_t>::type,
-         typename HIMap = typename boost::property_map<Graph, CGAL::halfedge_index_t>::type>
+         typename FIMap = Default,
+         typename VIMap = Default,
+         typename HIMap = Default>
 struct Face_filtered_graph
 {
   typedef boost::graph_traits<Graph>                  gt;
@@ -93,10 +94,15 @@ struct Face_filtered_graph
   #endif
 
   // non documented types
-  typedef typename boost::property_traits< FIMap >::value_type face_index_type;
-  typedef typename boost::property_traits< VIMap >::value_type vertex_index_type;
-  typedef typename boost::property_traits< HIMap >::value_type halfedge_index_type;
-  typedef Face_filtered_graph<Graph, FIMap, VIMap, HIMap>   Self;
+  typedef typename Default::Get<FIMap, typename CGAL::GetInitializedFaceIndexMap<Graph>::const_type>::type FIM;
+  typedef typename Default::Get<VIMap, typename CGAL::GetInitializedVertexIndexMap<Graph>::const_type>::type VIM;
+  typedef typename Default::Get<HIMap, typename CGAL::GetInitializedHalfedgeIndexMap<Graph>::const_type>::type HIM;
+
+  typedef typename boost::property_traits<FIM>::value_type face_index_type;
+  typedef typename boost::property_traits<VIM>::value_type vertex_index_type;
+  typedef typename boost::property_traits<HIM>::value_type halfedge_index_type;
+
+  typedef Face_filtered_graph<Graph, FIMap, VIMap, HIMap> Self;
 
   /*!
    * \brief Constructor where the set of selected faces is specified as a range of patch ids.
@@ -107,39 +113,61 @@ struct Face_filtered_graph
    * \tparam FacePatchIndexRange a model of `ConstRange` with `boost::property_traits<FacePatchIndexMap>::%value_type` as value type.
    * \tparam NamedParameters a sequence of named parameters
    *
-   * \param graph the underlying graph.
-   * \param face_patch_index_map the property_map that assigns a patch index to each face, with
-      `face_descriptor` as key type and `boost::graph_traits<Graph>::%faces_size_type` as value type.
-   * \param selected_face_patch_indices a range of the face patch indices to select.
-   * \param np optional sequence of named parameters among the ones listed below
+   * \param graph the underlying graph
+   * \param face_patch_index_map the property_map that assigns a patch index to each face
+   * \param selected_face_patch_indices a range of the face patch indices to select
+   * \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
    *
    * \cgalNamedParamsBegin
-   *   \cgalParamBegin{face_index_map}
-   *     a property map containing an index for each face initialized from 0 to `num_vertices(graph)`
-   *   \cgalParamEnd
-   *   \cgalParamBegin{vertex_index_map}
-   *     a property map containing an index for each vertex initialized 0 to `num_vertices(graph)`
-   *   \cgalParamEnd
-   *   \cgalParamBegin{halfedge_index_map}
-   *     a property map containing an index for each halfedge initialized 0 to `num_halfedges(graph)`
-   *   \cgalParamEnd
+   *   \cgalParamNBegin{vertex_index_map}
+   *     \cgalParamDescription{a property map associating to each vertex of `graph` a unique index between `0` and `num_vertices(graph) - 1`}
+   *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<Graph>::%vertex_descriptor`
+   *                    as key type and `std::size_t` as value type}
+   *     \cgalParamDefault{an automatically indexed internal map}
+   *     \cgalParamExtra{If this parameter is not passed, internal machinery will create and initialize
+   *                     a face index property map, either using the internal property map if it exists
+   *                     or using an external map. The latter might result in  - slightly - worsened performance
+   *                     in case of non-constant complexity for index access.}
+   *   \cgalParamNEnd
+   *
+   *   \cgalParamNBegin{halfedge_index_map}
+   *     \cgalParamDescription{a property map associating to each halfedge of `graph` a unique index between `0` and `num_halfedges(graph) - 1`}
+   *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<Graph>::%halfedge_descriptor`
+   *                    as key type and `std::size_t` as value type}
+   *     \cgalParamDefault{an automatically indexed internal map}
+   *     \cgalParamExtra{If this parameter is not passed, internal machinery will create and initialize
+   *                     a face index property map, either using the internal property map if it exists
+   *                     or using an external map. The latter might result in  - slightly - worsened performance
+   *                     in case of non-constant complexity for index access.}
+   *   \cgalParamNEnd
+   *
+   *   \cgalParamNBegin{face_index_map}
+   *     \cgalParamDescription{a property map associating to each face of `graph` a unique index between `0` and `num_faces(graph) - 1`}
+   *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<Graph>::%face_descriptor`
+   *                    as key type and `std::size_t` as value type}
+   *     \cgalParamDefault{an automatically indexed internal map}
+   *     \cgalParamExtra{If this parameter is not passed, internal machinery will create and initialize
+   *                     a face index property map, either using the internal property map if it exists
+   *                     or using an external map. The latter might result in  - slightly - worsened performance
+   *                     in case of non-constant complexity for index access.}
+   *   \cgalParamNEnd
    * \cgalNamedParamsEnd
    */
   template <typename FacePatchIndexMap, class FacePatchIndexRange, class CGAL_BGL_NP_TEMPLATE_PARAMETERS>
   Face_filtered_graph(const Graph& graph,
                       const FacePatchIndexRange& selected_face_patch_indices,
-                            FacePatchIndexMap face_patch_index_map,
-                            const CGAL_BGL_NP_CLASS& np
-                           #ifndef DOXYGEN_RUNNING
-                              , typename boost::enable_if<
-                                  typename boost::has_range_const_iterator<FacePatchIndexRange>::type
-                                >::type* = 0
-                           #endif
-                             )
-    : _graph(const_cast<Graph&>(graph))
-    , fimap(parameters::choose_parameter(parameters::get_parameter(np, internal_np::face_index), get_const_property_map(face_index, graph)))
-    , vimap(parameters::choose_parameter(parameters::get_parameter(np, internal_np::vertex_index), get_const_property_map(boost::vertex_index, graph)))
-    , himap(parameters::choose_parameter(parameters::get_parameter(np, internal_np::halfedge_index), get_const_property_map(halfedge_index, graph)))
+                      FacePatchIndexMap face_patch_index_map,
+                      const CGAL_BGL_NP_CLASS& np
+#ifndef DOXYGEN_RUNNING
+                    , typename boost::enable_if<
+                        typename boost::has_range_const_iterator<FacePatchIndexRange>::type
+                      >::type* = 0
+#endif
+                      )
+    : _graph(const_cast<Graph&>(graph)),
+      fimap(CGAL::get_initialized_face_index_map(graph, np)),
+      vimap(CGAL::get_initialized_vertex_index_map(graph, np)),
+      himap(CGAL::get_initialized_halfedge_index_map(graph, np))
   {
     set_selected_faces(selected_face_patch_indices, face_patch_index_map);
   }
@@ -152,10 +180,10 @@ struct Face_filtered_graph
                       typename boost::has_range_const_iterator<FacePatchIndexRange>::type
                       >::type* = 0
                       )
-    : _graph(const_cast<Graph&>(graph))
-    , fimap(get(CGAL::face_index, graph))
-    , vimap(get(boost::vertex_index, graph))
-    , himap(get(CGAL::halfedge_index, graph))
+    : _graph(const_cast<Graph&>(graph)),
+      fimap(CGAL::get_initialized_face_index_map(graph)),
+      vimap(CGAL::get_initialized_vertex_index_map(graph)),
+      himap(CGAL::get_initialized_halfedge_index_map(graph))
   {
     set_selected_faces(selected_face_patch_indices, face_patch_index_map);
   }
@@ -166,47 +194,68 @@ struct Face_filtered_graph
       `face_descriptor` as key type and
       `graph_traits<Graph>::%faces_size_type` as value type.
    * \tparam NamedParameters a sequence of named parameters
+   *
    * \param graph the underlying graph.
-   * \param face_patch_index_map the property_map that assigns a patch index to each face, with
-      `face_descriptor` as key type and
-      `graph_traits<Graph>::%faces_size_type` as value type.
-   * \param selected_face_patch_index the index of the face patch selected.
-   * \param np optional sequence of named parameters among the ones listed below
+   * \param face_patch_index_map the property_map that assigns a patch index to each face
+   * \param selected_face_patch_index the index of the face patch selected
+   * \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
    *
    * \cgalNamedParamsBegin
-   *   \cgalParamBegin{face_index_map}
-   *     a property map containing an index for each face initialized from 0 to `num_vertices(graph)`
-   *   \cgalParamEnd
-   *   \cgalParamBegin{vertex_index_map}
-   *     a property map containing an index for each vertex initialized 0 to `num_vertices(graph)`
-   *   \cgalParamEnd
-   *   \cgalParamBegin{halfedge_index_map}
-   *     a property map containing an index for each halfedge initialized 0 to `num_halfedges(graph)`
-   *   \cgalParamEnd
+   *   \cgalParamNBegin{vertex_index_map}
+   *     \cgalParamDescription{a property map associating to each vertex of `graph` a unique index between `0` and `num_vertices(graph) - 1`}
+   *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<Graph>::%vertex_descriptor`
+   *                    as key type and `std::size_t` as value type}
+   *     \cgalParamDefault{an automatically indexed internal map}
+   *     \cgalParamExtra{If this parameter is not passed, internal machinery will create and initialize
+   *                     a face index property map, either using the internal property map if it exists
+   *                     or using an external map. The latter might result in  - slightly - worsened performance
+   *                     in case of non-constant complexity for index access.}
+   *   \cgalParamNEnd
+   *
+   *   \cgalParamNBegin{halfedge_index_map}
+   *     \cgalParamDescription{a property map associating to each halfedge of `graph` a unique index between `0` and `num_halfedges(graph) - 1`}
+   *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<Graph>::%halfedge_descriptor`
+   *                    as key type and `std::size_t` as value type}
+   *     \cgalParamDefault{an automatically indexed internal map}
+   *     \cgalParamExtra{If this parameter is not passed, internal machinery will create and initialize
+   *                     a face index property map, either using the internal property map if it exists
+   *                     or using an external map. The latter might result in  - slightly - worsened performance
+   *                     in case of non-constant complexity for index access.}
+   *   \cgalParamNEnd
+   *
+   *   \cgalParamNBegin{face_index_map}
+   *     \cgalParamDescription{a property map associating to each face of `graph` a unique index between `0` and `num_faces(graph) - 1`}
+   *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<Graph>::%face_descriptor`
+   *                    as key type and `std::size_t` as value type}
+   *     \cgalParamDefault{an automatically indexed internal map}
+   *     \cgalParamExtra{If this parameter is not passed, internal machinery will create and initialize
+   *                     a face index property map, either using the internal property map if it exists
+   *                     or using an external map. The latter might result in  - slightly - worsened performance
+   *                     in case of non-constant complexity for index access.}
+   *   \cgalParamNEnd
    * \cgalNamedParamsEnd
    */
   template <typename FacePatchIndexMap, class CGAL_BGL_NP_TEMPLATE_PARAMETERS>
   Face_filtered_graph(const Graph& graph,
-                            typename boost::property_traits<FacePatchIndexMap>::value_type selected_face_patch_index,
-                            FacePatchIndexMap face_patch_index_map,
-                            const CGAL_BGL_NP_CLASS& np
-                             )
-    : _graph(const_cast<Graph&>(graph))
-    , fimap(parameters::choose_parameter(parameters::get_parameter(np, internal_np::face_index), get_const_property_map(face_index, graph)))
-    , vimap(parameters::choose_parameter(parameters::get_parameter(np, internal_np::vertex_index), get_const_property_map(boost::vertex_index, graph)))
-    , himap(parameters::choose_parameter(parameters::get_parameter(np, internal_np::halfedge_index), get_const_property_map(halfedge_index, graph)))
+                      typename boost::property_traits<FacePatchIndexMap>::value_type selected_face_patch_index,
+                      FacePatchIndexMap face_patch_index_map,
+                      const CGAL_BGL_NP_CLASS& np)
+    : _graph(const_cast<Graph&>(graph)),
+      fimap(CGAL::get_initialized_face_index_map(graph, np)),
+      vimap(CGAL::get_initialized_vertex_index_map(graph, np)),
+      himap(CGAL::get_initialized_halfedge_index_map(graph, np))
   {
     set_selected_faces(selected_face_patch_index, face_patch_index_map);
   }
 
   template <typename FacePatchIndexMap>
   Face_filtered_graph(const Graph& graph,
-                            typename boost::property_traits<FacePatchIndexMap>::value_type pid,
-                            FacePatchIndexMap face_patch_index_map)
-    : _graph(const_cast<Graph&>(graph))
-    , fimap(get(CGAL::face_index, graph))
-    , vimap(get(boost::vertex_index, graph))
-    , himap(get(CGAL::halfedge_index, graph))
+                      typename boost::property_traits<FacePatchIndexMap>::value_type pid,
+                      FacePatchIndexMap face_patch_index_map)
+    : _graph(const_cast<Graph&>(graph)),
+      fimap(CGAL::get_initialized_face_index_map(graph)),
+      vimap(CGAL::get_initialized_vertex_index_map(graph)),
+      himap(CGAL::get_initialized_halfedge_index_map(graph))
   {
     set_selected_faces(pid, face_patch_index_map);
   }
@@ -216,30 +265,54 @@ struct Face_filtered_graph
    *
    * \tparam FaceRange a model of `ConstRange` with `face_descriptor` as value type.
    * \tparam NamedParameters a sequence of named parameters
-   * \param graph the graph containing the wanted patch.
-   * \param selected_faces the set of selected faces.
-   * \param np optional sequence of named parameters among the ones listed below
+   *
+   * \param graph the graph containing the wanted patch
+   * \param selected_faces the set of selected faces
+   * \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
    *
    * \cgalNamedParamsBegin
-   *   \cgalParamBegin{face_index_map}
-   *     a property map containing an index for each face initialized from 0 to `num_vertices(graph)`
-   *   \cgalParamEnd
-   *   \cgalParamBegin{vertex_index_map}
-   *     a property map containing an index for each vertex initialized 0 to `num_vertices(graph)`
-   *   \cgalParamEnd
-   *   \cgalParamBegin{halfedge_index_map}
-   *     a property map containing an index for each halfedge initialized 0 to `num_halfedges(graph)`
-   *   \cgalParamEnd
+   *   \cgalParamNBegin{vertex_index_map}
+   *     \cgalParamDescription{a property map associating to each vertex of `graph` a unique index between `0` and `num_vertices(graph) - 1`}
+   *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<Graph>::%vertex_descriptor`
+   *                    as key type and `std::size_t` as value type}
+   *     \cgalParamDefault{an automatically indexed internal map}
+   *     \cgalParamExtra{If this parameter is not passed, internal machinery will create and initialize
+   *                     a face index property map, either using the internal property map if it exists
+   *                     or using an external map. The latter might result in  - slightly - worsened performance
+   *                     in case of non-constant complexity for index access.}
+   *   \cgalParamNEnd
+   *
+   *   \cgalParamNBegin{halfedge_index_map}
+   *     \cgalParamDescription{a property map associating to each halfedge of `graph` a unique index between `0` and `num_halfedges(graph) - 1`}
+   *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<Graph>::%halfedge_descriptor`
+   *                    as key type and `std::size_t` as value type}
+   *     \cgalParamDefault{an automatically indexed internal map}
+   *     \cgalParamExtra{If this parameter is not passed, internal machinery will create and initialize
+   *                     a face index property map, either using the internal property map if it exists
+   *                     or using an external map. The latter might result in  - slightly - worsened performance
+   *                     in case of non-constant complexity for index access.}
+   *   \cgalParamNEnd
+   *
+   *   \cgalParamNBegin{face_index_map}
+   *     \cgalParamDescription{a property map associating to each face of `graph` a unique index between `0` and `num_faces(graph) - 1`}
+   *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<Graph>::%face_descriptor`
+   *                    as key type and `std::size_t` as value type}
+   *     \cgalParamDefault{an automatically indexed internal map}
+   *     \cgalParamExtra{If this parameter is not passed, internal machinery will create and initialize
+   *                     a face index property map, either using the internal property map if it exists
+   *                     or using an external map. The latter might result in  - slightly - worsened performance
+   *                     in case of non-constant complexity for index access.}
+   *   \cgalParamNEnd
    * \cgalNamedParamsEnd
    */
   template <typename FaceRange, class CGAL_BGL_NP_TEMPLATE_PARAMETERS>
   Face_filtered_graph(const Graph& graph,
                       const FaceRange& selected_faces,
                       const CGAL_BGL_NP_CLASS& np)
-    : _graph(const_cast<Graph&>(graph))
-    , fimap(parameters::choose_parameter(parameters::get_parameter(np, internal_np::face_index), get_const_property_map(face_index, graph)))
-    , vimap(parameters::choose_parameter(parameters::get_parameter(np, internal_np::vertex_index), get_const_property_map(boost::vertex_index, graph)))
-    , himap(parameters::choose_parameter(parameters::get_parameter(np, internal_np::halfedge_index), get_const_property_map(halfedge_index, graph)))
+    : _graph(const_cast<Graph&>(graph)),
+      fimap(CGAL::get_initialized_face_index_map(graph, np)),
+      vimap(CGAL::get_initialized_vertex_index_map(graph, np)),
+      himap(CGAL::get_initialized_halfedge_index_map(graph, np))
   {
     set_selected_faces(selected_faces);
   }
@@ -247,10 +320,10 @@ struct Face_filtered_graph
   template <typename FaceRange>
   Face_filtered_graph(const Graph& graph,
                       const FaceRange& selected_faces)
-    : _graph(const_cast<Graph&>(graph))
-    , fimap(get(CGAL::face_index, graph))
-    , vimap(get(boost::vertex_index, graph))
-    , himap(get(CGAL::halfedge_index, graph))
+    : _graph(const_cast<Graph&>(graph)),
+      fimap(CGAL::get_initialized_face_index_map(graph)),
+      vimap(CGAL::get_initialized_vertex_index_map(graph)),
+      himap(CGAL::get_initialized_halfedge_index_map(graph))
   {
     set_selected_faces(selected_faces);
   }
@@ -293,12 +366,12 @@ struct Face_filtered_graph
   template<class FacePatchIndexRange, class FacePatchIndexMap>
   void set_selected_faces(const FacePatchIndexRange& selected_face_patch_indices,
                           FacePatchIndexMap face_patch_index_map
-  #ifndef DOXYGEN_RUNNING
+#ifndef DOXYGEN_RUNNING
                           , typename boost::enable_if<
                               typename boost::has_range_const_iterator<FacePatchIndexRange>::type
                             >::type* = 0
-  #endif
-  )
+#endif
+                          )
   {
     face_indices.clear();
     vertex_indices.clear();
@@ -407,7 +480,7 @@ struct Face_filtered_graph
     return selected_halfedges.count();
   }
 
-  Property_map_binder< FIMap, typename Pointer_property_map< typename boost::property_traits< FIMap >::value_type >::type >
+  Property_map_binder<FIM, typename Pointer_property_map<typename boost::property_traits<FIM>::value_type>::type>
   get_face_index_map() const
   {
     if (face_indices.empty())
@@ -422,7 +495,7 @@ struct Face_filtered_graph
     return bind_property_maps(fimap, make_property_map(face_indices) );
   }
 
-  Property_map_binder< VIMap, typename Pointer_property_map< typename boost::property_traits< VIMap >::value_type >::type >
+  Property_map_binder<VIM, typename Pointer_property_map<typename boost::property_traits<VIM>::value_type>::type>
   get_vertex_index_map() const
   {
     if (vertex_indices.empty())
@@ -437,7 +510,7 @@ struct Face_filtered_graph
     return bind_property_maps(vimap, make_property_map(vertex_indices) );
   }
 
-  Property_map_binder< HIMap, typename Pointer_property_map< typename boost::property_traits< HIMap >::value_type >::type >
+  Property_map_binder<HIM, typename Pointer_property_map<typename boost::property_traits<HIM>::value_type >::type>
   get_halfedge_index_map() const
   {
     if (halfedge_indices.empty())
@@ -510,9 +583,9 @@ struct Face_filtered_graph
 
 private:
   Graph& _graph;
-  FIMap fimap;
-  VIMap vimap;
-  HIMap himap;
+  FIM fimap;
+  VIM vimap;
+  HIM himap;
   boost::dynamic_bitset<> selected_faces;
   boost::dynamic_bitset<> selected_vertices;
   boost::dynamic_bitset<> selected_halfedges;
@@ -1172,38 +1245,40 @@ CGAL_FILTERED_FACE_GRAPH_DYNAMIC_PMAP_SP
 
 #undef CGAL_FILTERED_FACE_GRAPH_DYNAMIC_PMAP_SPECIALIZATION
 
-
-
 //specializations for indices
-template<typename Graph,
-         typename FIMap,
-         typename VIMap,
-         typename HIMap>
-struct property_map<CGAL::Face_filtered_graph<Graph, FIMap, VIMap, HIMap>, CGAL::face_index_t>{
-  typedef typename CGAL::Property_map_binder< FIMap,
-  typename CGAL::Pointer_property_map< typename boost::property_traits< FIMap >::value_type >::type > type;
-  typedef type const_type;
+template<typename Graph, typename FIMap, typename VIMap, typename HIMap>
+struct property_map<CGAL::Face_filtered_graph<Graph, FIMap, VIMap, HIMap>, CGAL::face_index_t>
+{
+  typedef typename CGAL::Face_filtered_graph<Graph, FIMap, VIMap, HIMap>::FIM         FIM;
+  typedef typename CGAL::Property_map_binder<FIM,
+                     typename CGAL::Pointer_property_map<
+                       typename boost::property_traits<FIM>::value_type>::type>       type;
+  typedef type                                                                        const_type;
 };
 
-template<typename Graph,
-         typename FIMap,
-         typename VIMap,
-         typename HIMap>
-struct property_map<CGAL::Face_filtered_graph<Graph, FIMap, VIMap, HIMap>, boost::vertex_index_t>{
-  typedef typename CGAL::Property_map_binder< VIMap,
-  typename CGAL::Pointer_property_map< typename boost::property_traits< VIMap >::value_type >::type > type;
-  typedef type const_type;
+template<typename Graph, typename FIMap, typename VIMap, typename HIMap>
+struct property_map<CGAL::Face_filtered_graph<Graph, FIMap, VIMap, HIMap>, boost::vertex_index_t>
+{
+  typedef typename CGAL::Face_filtered_graph<Graph, FIMap, VIMap, HIMap>::VIM         VIM;
+  typedef typename CGAL::Property_map_binder<VIM,
+                     typename CGAL::Pointer_property_map<
+                       typename boost::property_traits<VIM>::value_type>::type>       type;
+  typedef type                                                                        const_type;
 };
 
 template<typename Graph,
          typename FIMap,
          typename VIMap,
          typename HIMap>
-
-struct property_map<CGAL::Face_filtered_graph<Graph, FIMap, VIMap, HIMap>, CGAL::halfedge_index_t>{
-  typedef typename CGAL::Property_map_binder< HIMap,
-  typename CGAL::Pointer_property_map< typename boost::property_traits< HIMap >::value_type >::type > type;
-  typedef type const_type;
+struct property_map<CGAL::Face_filtered_graph<Graph, FIMap, VIMap, HIMap>, CGAL::halfedge_index_t>
+{
+  typedef typename CGAL::Face_filtered_graph<Graph, FIMap, VIMap, HIMap>::HIM         HIM;
+  typedef typename CGAL::Property_map_binder<HIM,
+                     typename CGAL::Pointer_property_map<
+                       typename boost::property_traits<HIM>::value_type>::type>       type;
+  typedef type                                                                        const_type;
 };
-}// namespace boost
+
+} // namespace boost
+
 #endif // CGAL_BOOST_GRAPH_FACE_FILTERED_GRAPH_H
diff -pruN 5.0.3-1/include/CGAL/boost/graph/generators.h 5.1-1/include/CGAL/boost/graph/generators.h
--- 5.0.3-1/include/CGAL/boost/graph/generators.h	2020-08-01 19:02:14.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/generators.h	2020-09-08 12:14:40.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/generators.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/generators.h $
 // $Id: generators.h 416cbbc 2019-10-29T15:23:41+01:00 Laurent Rineau
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/graph_concepts.h 5.1-1/include/CGAL/boost/graph/graph_concepts.h
--- 5.0.3-1/include/CGAL/boost/graph/graph_concepts.h	2020-08-01 19:02:14.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/graph_concepts.h	2020-09-08 12:14:40.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/graph_concepts.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/graph_concepts.h $
 // $Id: graph_concepts.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/graph_traits_Arrangement_2.h 5.1-1/include/CGAL/boost/graph/graph_traits_Arrangement_2.h
--- 5.0.3-1/include/CGAL/boost/graph/graph_traits_Arrangement_2.h	2020-08-01 19:02:13.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/graph_traits_Arrangement_2.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/boost/graph/graph_traits_Arrangement_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/boost/graph/graph_traits_Arrangement_2.h $
 // $Id: graph_traits_Arrangement_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/graph_traits_Constrained_Delaunay_triangulation_2.h 5.1-1/include/CGAL/boost/graph/graph_traits_Constrained_Delaunay_triangulation_2.h
--- 5.0.3-1/include/CGAL/boost/graph/graph_traits_Constrained_Delaunay_triangulation_2.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/graph_traits_Constrained_Delaunay_triangulation_2.h	2020-09-08 12:15:51.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/boost/graph/graph_traits_Constrained_Delaunay_triangulation_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/boost/graph/graph_traits_Constrained_Delaunay_triangulation_2.h $
 // $Id: graph_traits_Constrained_Delaunay_triangulation_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/graph_traits_Constrained_triangulation_2.h 5.1-1/include/CGAL/boost/graph/graph_traits_Constrained_triangulation_2.h
--- 5.0.3-1/include/CGAL/boost/graph/graph_traits_Constrained_triangulation_2.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/graph_traits_Constrained_triangulation_2.h	2020-09-08 12:15:51.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/boost/graph/graph_traits_Constrained_triangulation_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/boost/graph/graph_traits_Constrained_triangulation_2.h $
 // $Id: graph_traits_Constrained_triangulation_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/graph_traits_Constrained_triangulation_plus_2.h 5.1-1/include/CGAL/boost/graph/graph_traits_Constrained_triangulation_plus_2.h
--- 5.0.3-1/include/CGAL/boost/graph/graph_traits_Constrained_triangulation_plus_2.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/graph_traits_Constrained_triangulation_plus_2.h	2020-09-08 12:15:51.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/boost/graph/graph_traits_Constrained_triangulation_plus_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/boost/graph/graph_traits_Constrained_triangulation_plus_2.h $
 // $Id: graph_traits_Constrained_triangulation_plus_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/graph_traits_Delaunay_triangulation_2.h 5.1-1/include/CGAL/boost/graph/graph_traits_Delaunay_triangulation_2.h
--- 5.0.3-1/include/CGAL/boost/graph/graph_traits_Delaunay_triangulation_2.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/graph_traits_Delaunay_triangulation_2.h	2020-09-08 12:15:51.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/boost/graph/graph_traits_Delaunay_triangulation_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/boost/graph/graph_traits_Delaunay_triangulation_2.h $
 // $Id: graph_traits_Delaunay_triangulation_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/graph_traits_Dual_Arrangement_2.h 5.1-1/include/CGAL/boost/graph/graph_traits_Dual_Arrangement_2.h
--- 5.0.3-1/include/CGAL/boost/graph/graph_traits_Dual_Arrangement_2.h	2020-08-01 19:02:13.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/graph_traits_Dual_Arrangement_2.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/boost/graph/graph_traits_Dual_Arrangement_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/boost/graph/graph_traits_Dual_Arrangement_2.h $
 // $Id: graph_traits_Dual_Arrangement_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/graph_traits_HalfedgeDS_default.h 5.1-1/include/CGAL/boost/graph/graph_traits_HalfedgeDS_default.h
--- 5.0.3-1/include/CGAL/boost/graph/graph_traits_HalfedgeDS_default.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/graph_traits_HalfedgeDS_default.h	2020-09-08 12:14:53.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/HalfedgeDS/include/CGAL/boost/graph/graph_traits_HalfedgeDS_default.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/HalfedgeDS/include/CGAL/boost/graph/graph_traits_HalfedgeDS_default.h $
 // $Id: graph_traits_HalfedgeDS_default.h 620dfa4 2020-03-27T08:37:32+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/graph_traits_HalfedgeDS.h 5.1-1/include/CGAL/boost/graph/graph_traits_HalfedgeDS.h
--- 5.0.3-1/include/CGAL/boost/graph/graph_traits_HalfedgeDS.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/graph_traits_HalfedgeDS.h	2020-09-08 12:14:53.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/HalfedgeDS/include/CGAL/boost/graph/graph_traits_HalfedgeDS.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/HalfedgeDS/include/CGAL/boost/graph/graph_traits_HalfedgeDS.h $
 // $Id: graph_traits_HalfedgeDS.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/graph_traits_Linear_cell_complex_for_combinatorial_map.h 5.1-1/include/CGAL/boost/graph/graph_traits_Linear_cell_complex_for_combinatorial_map.h
--- 5.0.3-1/include/CGAL/boost/graph/graph_traits_Linear_cell_complex_for_combinatorial_map.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/graph_traits_Linear_cell_complex_for_combinatorial_map.h	2020-09-08 12:15:06.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Linear_cell_complex/include/CGAL/boost/graph/graph_traits_Linear_cell_complex_for_combinatorial_map.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Linear_cell_complex/include/CGAL/boost/graph/graph_traits_Linear_cell_complex_for_combinatorial_map.h $
 // $Id: graph_traits_Linear_cell_complex_for_combinatorial_map.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/graph_traits_OpenMesh.h 5.1-1/include/CGAL/boost/graph/graph_traits_OpenMesh.h
--- 5.0.3-1/include/CGAL/boost/graph/graph_traits_OpenMesh.h	2020-08-01 19:02:14.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/graph_traits_OpenMesh.h	2020-09-08 12:14:40.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/graph_traits_OpenMesh.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/graph_traits_OpenMesh.h $
 // $Id: graph_traits_OpenMesh.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/graph_traits_Polyhedron_3.h 5.1-1/include/CGAL/boost/graph/graph_traits_Polyhedron_3.h
--- 5.0.3-1/include/CGAL/boost/graph/graph_traits_Polyhedron_3.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/graph_traits_Polyhedron_3.h	2020-09-08 12:15:28.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron/include/CGAL/boost/graph/graph_traits_Polyhedron_3.h $
-// $Id: graph_traits_Polyhedron_3.h 414bb92 2020-06-30T16:36:55+02:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron/include/CGAL/boost/graph/graph_traits_Polyhedron_3.h $
+// $Id: graph_traits_Polyhedron_3.h 10834e4 2020-06-30T16:37:59+02:00 Laurent Rineau
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/graph_traits_PolyMesh_ArrayKernelT.h 5.1-1/include/CGAL/boost/graph/graph_traits_PolyMesh_ArrayKernelT.h
--- 5.0.3-1/include/CGAL/boost/graph/graph_traits_PolyMesh_ArrayKernelT.h	2020-08-01 19:02:14.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/graph_traits_PolyMesh_ArrayKernelT.h	2020-09-08 12:14:40.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/graph_traits_PolyMesh_ArrayKernelT.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/graph_traits_PolyMesh_ArrayKernelT.h $
 // $Id: graph_traits_PolyMesh_ArrayKernelT.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/graph_traits_Regular_triangulation_2.h 5.1-1/include/CGAL/boost/graph/graph_traits_Regular_triangulation_2.h
--- 5.0.3-1/include/CGAL/boost/graph/graph_traits_Regular_triangulation_2.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/graph_traits_Regular_triangulation_2.h	2020-09-08 12:15:51.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/boost/graph/graph_traits_Regular_triangulation_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/boost/graph/graph_traits_Regular_triangulation_2.h $
 // $Id: graph_traits_Regular_triangulation_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/graph_traits_Seam_mesh.h 5.1-1/include/CGAL/boost/graph/graph_traits_Seam_mesh.h
--- 5.0.3-1/include/CGAL/boost/graph/graph_traits_Seam_mesh.h	2020-08-01 19:02:14.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/graph_traits_Seam_mesh.h	2020-09-08 12:14:40.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/graph_traits_Seam_mesh.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/graph_traits_Seam_mesh.h $
 // $Id: graph_traits_Seam_mesh.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/graph_traits_Surface_mesh.h 5.1-1/include/CGAL/boost/graph/graph_traits_Surface_mesh.h
--- 5.0.3-1/include/CGAL/boost/graph/graph_traits_Surface_mesh.h	2020-08-01 19:02:54.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/graph_traits_Surface_mesh.h	2020-09-08 12:15:43.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh/include/CGAL/boost/graph/graph_traits_Surface_mesh.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh/include/CGAL/boost/graph/graph_traits_Surface_mesh.h $
 // $Id: graph_traits_Surface_mesh.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/graph_traits_Triangulation_2.h 5.1-1/include/CGAL/boost/graph/graph_traits_Triangulation_2.h
--- 5.0.3-1/include/CGAL/boost/graph/graph_traits_Triangulation_2.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/graph_traits_Triangulation_2.h	2020-09-08 12:15:51.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/boost/graph/graph_traits_Triangulation_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/boost/graph/graph_traits_Triangulation_2.h $
 // $Id: graph_traits_Triangulation_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/graph_traits_Triangulation_data_structure_2.h 5.1-1/include/CGAL/boost/graph/graph_traits_Triangulation_data_structure_2.h
--- 5.0.3-1/include/CGAL/boost/graph/graph_traits_Triangulation_data_structure_2.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/graph_traits_Triangulation_data_structure_2.h	2020-09-08 12:15:51.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/boost/graph/graph_traits_Triangulation_data_structure_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/boost/graph/graph_traits_Triangulation_data_structure_2.h $
 // $Id: graph_traits_Triangulation_data_structure_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/graph_traits_Triangulation_hierarchy_2.h 5.1-1/include/CGAL/boost/graph/graph_traits_Triangulation_hierarchy_2.h
--- 5.0.3-1/include/CGAL/boost/graph/graph_traits_Triangulation_hierarchy_2.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/graph_traits_Triangulation_hierarchy_2.h	2020-09-08 12:15:51.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/boost/graph/graph_traits_Triangulation_hierarchy_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/boost/graph/graph_traits_Triangulation_hierarchy_2.h $
 // $Id: graph_traits_Triangulation_hierarchy_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/graph_traits_TriMesh_ArrayKernelT.h 5.1-1/include/CGAL/boost/graph/graph_traits_TriMesh_ArrayKernelT.h
--- 5.0.3-1/include/CGAL/boost/graph/graph_traits_TriMesh_ArrayKernelT.h	2020-08-01 19:02:14.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/graph_traits_TriMesh_ArrayKernelT.h	2020-09-08 12:14:40.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/graph_traits_TriMesh_ArrayKernelT.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/graph_traits_TriMesh_ArrayKernelT.h $
 // $Id: graph_traits_TriMesh_ArrayKernelT.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/Graph_with_descriptor_with_graph_fwd.h 5.1-1/include/CGAL/boost/graph/Graph_with_descriptor_with_graph_fwd.h
--- 5.0.3-1/include/CGAL/boost/graph/Graph_with_descriptor_with_graph_fwd.h	2020-08-01 19:02:14.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/Graph_with_descriptor_with_graph_fwd.h	2020-09-08 12:14:40.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/Graph_with_descriptor_with_graph_fwd.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/Graph_with_descriptor_with_graph_fwd.h $
 // $Id: Graph_with_descriptor_with_graph_fwd.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/Graph_with_descriptor_with_graph.h 5.1-1/include/CGAL/boost/graph/Graph_with_descriptor_with_graph.h
--- 5.0.3-1/include/CGAL/boost/graph/Graph_with_descriptor_with_graph.h	2020-08-01 19:02:14.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/Graph_with_descriptor_with_graph.h	2020-09-08 12:14:40.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/Graph_with_descriptor_with_graph.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/Graph_with_descriptor_with_graph.h $
 // $Id: Graph_with_descriptor_with_graph.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/halfedge_graph_traits.h 5.1-1/include/CGAL/boost/graph/halfedge_graph_traits.h
--- 5.0.3-1/include/CGAL/boost/graph/halfedge_graph_traits.h	2020-08-01 19:02:14.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/halfedge_graph_traits.h	2020-09-08 12:14:40.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/halfedge_graph_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/halfedge_graph_traits.h $
 // $Id: halfedge_graph_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/halfedge_graph_traits_HalfedgeDS.h 5.1-1/include/CGAL/boost/graph/halfedge_graph_traits_HalfedgeDS.h
--- 5.0.3-1/include/CGAL/boost/graph/halfedge_graph_traits_HalfedgeDS.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/halfedge_graph_traits_HalfedgeDS.h	2020-09-08 12:14:53.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/HalfedgeDS/include/CGAL/boost/graph/halfedge_graph_traits_HalfedgeDS.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/HalfedgeDS/include/CGAL/boost/graph/halfedge_graph_traits_HalfedgeDS.h $
 // $Id: halfedge_graph_traits_HalfedgeDS.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/halfedge_graph_traits_Polyhedron_3.h 5.1-1/include/CGAL/boost/graph/halfedge_graph_traits_Polyhedron_3.h
--- 5.0.3-1/include/CGAL/boost/graph/halfedge_graph_traits_Polyhedron_3.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/halfedge_graph_traits_Polyhedron_3.h	2020-09-08 12:15:28.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron/include/CGAL/boost/graph/halfedge_graph_traits_Polyhedron_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron/include/CGAL/boost/graph/halfedge_graph_traits_Polyhedron_3.h $
 // $Id: halfedge_graph_traits_Polyhedron_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/helpers.h 5.1-1/include/CGAL/boost/graph/helpers.h
--- 5.0.3-1/include/CGAL/boost/graph/helpers.h	2020-08-01 19:02:14.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/helpers.h	2020-09-08 12:14:40.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/helpers.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/helpers.h $
 // $Id: helpers.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/internal/graph_traits_2D_triangulation.h 5.1-1/include/CGAL/boost/graph/internal/graph_traits_2D_triangulation.h
--- 5.0.3-1/include/CGAL/boost/graph/internal/graph_traits_2D_triangulation.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/internal/graph_traits_2D_triangulation.h	2020-09-08 12:15:51.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/boost/graph/internal/graph_traits_2D_triangulation.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/boost/graph/internal/graph_traits_2D_triangulation.h $
 // $Id: graph_traits_2D_triangulation.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/internal/graph_traits_2D_triangulation_helper.h 5.1-1/include/CGAL/boost/graph/internal/graph_traits_2D_triangulation_helper.h
--- 5.0.3-1/include/CGAL/boost/graph/internal/graph_traits_2D_triangulation_helper.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/internal/graph_traits_2D_triangulation_helper.h	2020-09-08 12:15:51.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/boost/graph/internal/graph_traits_2D_triangulation_helper.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/boost/graph/internal/graph_traits_2D_triangulation_helper.h $
 // $Id: graph_traits_2D_triangulation_helper.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/internal/Has_member_clear.h 5.1-1/include/CGAL/boost/graph/internal/Has_member_clear.h
--- 5.0.3-1/include/CGAL/boost/graph/internal/Has_member_clear.h	2020-08-01 19:02:14.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/internal/Has_member_clear.h	2020-09-08 12:14:40.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/internal/Has_member_clear.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/internal/Has_member_clear.h $
 // $Id: Has_member_clear.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/internal/Has_member_id.h 5.1-1/include/CGAL/boost/graph/internal/Has_member_id.h
--- 5.0.3-1/include/CGAL/boost/graph/internal/Has_member_id.h	2020-08-01 19:02:14.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/internal/Has_member_id.h	2020-09-08 12:14:40.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/internal/Has_member_id.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/internal/Has_member_id.h $
 // $Id: Has_member_id.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/internal/helpers.h 5.1-1/include/CGAL/boost/graph/internal/helpers.h
--- 5.0.3-1/include/CGAL/boost/graph/internal/helpers.h	2020-08-01 19:02:14.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/internal/helpers.h	2020-09-08 12:14:40.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/internal/helpers.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/internal/helpers.h $
 // $Id: helpers.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/internal/initialized_index_maps_helpers.h 5.1-1/include/CGAL/boost/graph/internal/initialized_index_maps_helpers.h
--- 5.0.3-1/include/CGAL/boost/graph/internal/initialized_index_maps_helpers.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/internal/initialized_index_maps_helpers.h	2020-09-08 12:14:40.000000000 +0000
@@ -0,0 +1,309 @@
+// Copyright (c) 2020 GeometryFactory (France).  All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/internal/initialized_index_maps_helpers.h $
+// $Id: initialized_index_maps_helpers.h 47027d6 2020-03-26T18:59:19+01:00 Sébastien Loriot
+// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s) : Mael Rouxel-Labbé
+//             Maxime Gimeno
+
+#ifndef CGAL_BOOST_GRAPH_INITIALIZED_INTERNAL_INDEX_MAPS_HELPERS
+#define CGAL_BOOST_GRAPH_INITIALIZED_INTERNAL_INDEX_MAPS_HELPERS
+
+#include <CGAL/assertions.h>
+#include <CGAL/boost/graph/Named_function_parameters.h>
+#include <CGAL/boost/graph/properties.h>
+#include <CGAL/Dynamic_property_map.h>
+#include <CGAL/use.h>
+
+#include <vector>
+#include <type_traits>
+
+namespace CGAL {
+namespace BGL {
+namespace internal {
+
+// Check that an index map has been correctly initialized
+template <typename DescriptorRange, typename IndexMap>
+bool is_index_map_valid(IndexMap idmap,
+                        const std::size_t num_simplices,
+                        const DescriptorRange& range)
+{
+  typedef typename boost::property_traits<IndexMap>::value_type Id_type;
+
+  Id_type max_id = static_cast<Id_type>(num_simplices);
+  std::vector<bool> indices(max_id);
+
+  // According to concepts, the descriptor ranges such as 'vertices(g)' return a 'std::pair<it, it>'
+  for(auto it = range.first; it != range.second; ++it)
+  {
+    const Id_type id = get(idmap, *it);
+    if(id >= 0 && id < max_id && !indices[id])
+    {
+      indices[id] = true;
+    }
+    else
+    {
+#ifdef CGAL_BGL_INDEX_MAP_DEBUG
+      std::cerr << "Invalid ID: " << id << " num_simplices: " << num_simplices << std::endl;
+#endif
+      return false;
+    }
+  }
+
+  return true;
+}
+
+template <typename VertexIndexPropertyMap, typename Graph>
+bool is_index_map_valid(const CGAL::internal_np::vertex_index_t, VertexIndexPropertyMap vertex_index_map, const Graph& g)
+{
+  return is_index_map_valid(vertex_index_map, num_vertices(g), vertices(g));
+}
+
+template <typename HalfedgeIndexPropertyMap, typename Graph>
+bool is_index_map_valid(const CGAL::internal_np::halfedge_index_t, HalfedgeIndexPropertyMap halfedge_index_map, const Graph& g)
+{
+  return is_index_map_valid(halfedge_index_map, num_halfedges(g), halfedges(g));
+}
+
+template <typename EdgeIndexPropertyMap, typename Graph>
+bool is_index_map_valid(const CGAL::internal_np::edge_index_t, EdgeIndexPropertyMap edge_index_map, const Graph& g)
+{
+  return is_index_map_valid(edge_index_map, num_edges(g), edges(g));
+}
+
+template <typename FaceIndexPropertyMap, typename Graph>
+bool is_index_map_valid(const CGAL::internal_np::face_index_t, FaceIndexPropertyMap face_index_map, const Graph& g)
+{
+  return is_index_map_valid(face_index_map, num_faces(g), faces(g));
+}
+
+template <typename PropertyTag, typename IndexPropertyMap, typename Graph>
+void initialize_index_map(const PropertyTag, IndexPropertyMap, const Graph&)
+{
+  // Unknown parameter; should never be here.
+  CGAL_assertion(false);
+}
+
+template <typename IndexPropertyMap,
+          typename Graph,
+          bool is_writable = CGAL::internal::Is_writable_property_map<IndexPropertyMap>::value>
+struct Index_map_initializer
+{
+  void operator()(const CGAL::internal_np::vertex_index_t, IndexPropertyMap vertex_index_map, const Graph& g)
+  {
+    typename boost::property_traits<IndexPropertyMap>::value_type i = 0;
+    for(typename boost::graph_traits<Graph>::vertex_descriptor vd : vertices(g))
+      put(vertex_index_map, vd, i++);
+  }
+
+  void operator()(const CGAL::internal_np::halfedge_index_t, IndexPropertyMap halfedge_index_map, const Graph& g)
+  {
+    typename boost::property_traits<IndexPropertyMap>::value_type i = 0;
+    for(typename boost::graph_traits<Graph>::halfedge_descriptor hd : halfedges(g))
+      put(halfedge_index_map, hd, i++);
+  }
+
+  void operator()(const CGAL::internal_np::edge_index_t, IndexPropertyMap edge_index_map, const Graph& g)
+  {
+    typename boost::property_traits<IndexPropertyMap>::value_type i = 0;
+    for(typename boost::graph_traits<Graph>::edge_descriptor ed : edges(g))
+      put(edge_index_map, ed, i++);
+  }
+
+  void operator()(const CGAL::internal_np::face_index_t, IndexPropertyMap face_index_map, const Graph& g)
+  {
+    typename boost::property_traits<IndexPropertyMap>::value_type i = 0;
+    for(typename boost::graph_traits<Graph>::face_descriptor fd : faces(g))
+      put(face_index_map, fd, i++);
+  }
+
+  template <typename PropertyTag>
+  void operator()(const PropertyTag, IndexPropertyMap, const Graph&)
+  {
+    // Unknown parameter; should never be here.
+    CGAL_assertion(false);
+  }
+};
+
+template <typename IndexPropertyMap, typename Graph>
+struct Index_map_initializer<IndexPropertyMap, Graph, false>
+{
+  template <typename PropertyTag>
+  void operator()(const PropertyTag, IndexPropertyMap, const Graph&)
+  {
+    // The property map is not writable; should never be here.
+    CGAL_assertion_msg(false, "You are trying to initialize a non-writable property map");
+  }
+};
+
+// Just for convenience, define the following functions:
+//
+// BGL::internal::initialize_vertex_index_map()
+// BGL::internal::initialize_halfedge_index_map()
+// BGL::internal::initialize_edge_index_map()
+// BGL::internal::initialize_face_index_map()
+
+#define CGAL_DEF_INITIALIZE_ID_MAP_FUNCTION(TYPE)                                                  \
+template <typename WritableIndexPropertyMap, typename Graph>                                       \
+void initialize_##TYPE##_index_map(WritableIndexPropertyMap index_map,                             \
+                                   const Graph& g)                                                 \
+{                                                                                                  \
+  Index_map_initializer<WritableIndexPropertyMap, Graph> initializer;                              \
+  initializer(CGAL::internal_np::TYPE##_index_t{}, index_map, g);                                  \
+}
+
+CGAL_DEF_INITIALIZE_ID_MAP_FUNCTION(vertex)
+CGAL_DEF_INITIALIZE_ID_MAP_FUNCTION(halfedge)
+CGAL_DEF_INITIALIZE_ID_MAP_FUNCTION(edge)
+CGAL_DEF_INITIALIZE_ID_MAP_FUNCTION(face)
+
+#undef CGAL_DEF_INITIALIZE_ID_FUCNTION
+
+// Using the pmap passed in named parameters -------------------------------------------------------
+template <typename IndexMap, typename PropertyTag, typename Tag, typename DynamicTag, typename Graph>
+IndexMap get_initialized_index_map_const(const IndexMap index_map,
+                                         const PropertyTag p, Tag, DynamicTag,
+                                         const Graph& g)
+{
+  CGAL_USE(g);
+  CGAL_USE(p);
+
+  // If you are passing a pmap via NPs, it must be initialized
+  CGAL_assertion(is_index_map_valid(p, index_map, g));
+
+  return index_map;
+}
+
+template <typename IndexMap, typename PropertyTag, typename Tag, typename DynamicTag, typename Graph>
+IndexMap get_initialized_index_map(const IndexMap index_map,
+                                   const PropertyTag p, Tag, DynamicTag,
+                                   Graph& g)
+{
+  CGAL_USE(g);
+  CGAL_USE(p);
+
+  // If you are passing a pmap via NPs, it must be initialized
+  CGAL_assertion(is_index_map_valid(p, index_map, g));
+
+  return index_map;
+}
+
+// Using the internal to the mesh ------------------------------------------------------------------
+template <typename InternalIndexMap, typename PropertyTag, typename Graph>
+InternalIndexMap
+get_initialized_internal_index_map(InternalIndexMap index_map,
+                                   const PropertyTag p,
+                                   const Graph& g)
+{
+  if(CGAL::internal::Is_writable_property_map<InternalIndexMap>::value)
+  {
+    if(!is_index_map_valid(p, index_map, g))
+      Index_map_initializer<InternalIndexMap, Graph>{}(p, index_map, g);
+  }
+  else // not writable
+  {
+    CGAL_assertion(is_index_map_valid(p, index_map, g));
+  }
+
+  return index_map;
+}
+
+template <typename PropertyTag, typename Tag, typename DynamicTag, typename Graph>
+typename boost::property_map<Graph, Tag>::const_type
+get_initialized_index_map_const(CGAL::internal_np::Param_not_found,
+                                const PropertyTag p, const Tag tag, DynamicTag,
+                                const Graph& g)
+{
+  return get_initialized_internal_index_map(get(tag, g), p, g);
+}
+
+// same as above, non-const graph overload
+template <typename PropertyTag, typename Tag, typename DynamicTag, typename Graph>
+typename boost::property_map<Graph, Tag>::type
+get_initialized_index_map(CGAL::internal_np::Param_not_found,
+                          const PropertyTag p, const Tag tag, DynamicTag,
+                          Graph& g)
+{
+  // From now on the correct property map has been acquired
+  // and there is no need to distinguish between const and non-const mesh
+  return get_initialized_internal_index_map(get(tag, g), p, g);
+}
+
+// Create a dynamic property and initialize it -----------------------------------------------------
+template <typename DynamicIndexMap, typename PropertyTag, typename Graph>
+DynamicIndexMap
+get_initialized_dynamic_index_map(DynamicIndexMap index_map,
+                                  const PropertyTag p,
+                                  const Graph& g)
+{
+#ifdef CGAL_PERFORMANCE_WARNINGS
+  std::cerr << "Warning: the automatically selected index map is a dynamic property map,"
+            << " which might not have constant-time access complexity." << std::endl;
+#endif
+
+  Index_map_initializer<DynamicIndexMap, Graph>{}(p, index_map, g);
+  return index_map;
+}
+
+template <typename PropertyTag, typename DynamicTag, typename Graph>
+typename boost::property_map<Graph, DynamicTag>::const_type
+get_initialized_index_map_const(CGAL::internal_np::Param_not_found,
+                                const PropertyTag p, const DynamicTag dtag, DynamicTag,
+                                const Graph& g)
+{
+  return get_initialized_dynamic_index_map(get(dtag, g), p, g);
+}
+
+// same as above, non-const graph overload
+template <typename PropertyTag, typename DynamicTag, typename Graph>
+typename boost::property_map<Graph, DynamicTag>::type
+get_initialized_index_map(CGAL::internal_np::Param_not_found,
+                          const PropertyTag p, const DynamicTag dtag, DynamicTag,
+                          Graph& g)
+{
+  // From now on the correct property map has been acquired
+  // and there is no need to distinguish between const and non-const mesh
+  return get_initialized_dynamic_index_map(get(dtag, g), p, g);
+}
+
+template <typename PropertyTag, typename Tag, typename DynamicTag,
+          typename Graph,
+          typename NamedParameters = Named_function_parameters<bool, internal_np::all_default_t> >
+class GetInitializedIndexMap
+{
+public:
+  // Check if there is an internal property map; if not, we must a dynamic property map
+  typedef typename boost::mpl::if_c<
+      CGAL::graph_has_property<Graph, Tag>::value, Tag, DynamicTag>::type    Final_tag;
+
+  typedef typename internal_np::Lookup_named_param_def<
+      PropertyTag,
+      NamedParameters,
+      typename boost::property_map<Graph, Final_tag>::const_type>::type      const_type;
+
+  typedef typename internal_np::Lookup_named_param_def<
+      PropertyTag,
+      NamedParameters,
+      typename boost::property_map<Graph, Final_tag>::type>::type            type;
+
+  static const_type get_const(const PropertyTag p, const Graph& g, const NamedParameters& np)
+  {
+    return BGL::internal::get_initialized_index_map_const(parameters::get_parameter(np, p),
+                                                          p, Final_tag{}, DynamicTag{}, g);
+  }
+
+  static type get(const PropertyTag p, Graph& g, const NamedParameters& np)
+  {
+    return BGL::internal::get_initialized_index_map(parameters::get_parameter(np, p),
+                                                    p, Final_tag{}, DynamicTag{}, g);
+  }
+};
+
+} // namespace internal
+} // namespace BGL
+} // namespace CGAL
+
+#endif // CGAL_BOOST_GRAPH_INITIALIZED_INTERNAL_INDEX_MAPS_HELPERS
diff -pruN 5.0.3-1/include/CGAL/boost/graph/internal/OM_iterator_from_circulator.h 5.1-1/include/CGAL/boost/graph/internal/OM_iterator_from_circulator.h
--- 5.0.3-1/include/CGAL/boost/graph/internal/OM_iterator_from_circulator.h	2020-08-01 19:02:14.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/internal/OM_iterator_from_circulator.h	2020-09-08 12:14:40.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/internal/OM_iterator_from_circulator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/internal/OM_iterator_from_circulator.h $
 // $Id: OM_iterator_from_circulator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/internal/properties_2D_triangulation.h 5.1-1/include/CGAL/boost/graph/internal/properties_2D_triangulation.h
--- 5.0.3-1/include/CGAL/boost/graph/internal/properties_2D_triangulation.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/internal/properties_2D_triangulation.h	2020-09-08 12:15:51.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/boost/graph/internal/properties_2D_triangulation.h $
-// $Id: properties_2D_triangulation.h 0903946 2020-03-17T17:53:21+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/boost/graph/internal/properties_2D_triangulation.h $
+// $Id: properties_2D_triangulation.h a8f9858 2020-04-16T16:29:55+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Mael Rouxel-Labbé
@@ -111,7 +111,7 @@ public:
   T2_halfedge_id_map(const Tr& tr) : tr(tr) { }
 
   // Halfedge id is twice the edge id, and +0/+1 depending whether
-  // h.first is such that h.first < opposite(h).first --> different ids
+  // h.first is such that h.first < opposite(h).first
   value_type operator[](key_type h) const
   {
     const Face_handle f1 = h.first;
@@ -119,13 +119,13 @@ public:
     CGAL_assertion(!tr.is_infinite(f1) || !tr.is_infinite(f2));
 
     if(tr.is_infinite(f1))
-      return 2*(3 * f2->id() + f2->index(f1));
+      return 2*(f2->edge_id(f2->index(f1)));
     else if(tr.is_infinite(f2))
-      return 2*(3 * f1->id() + h.second) + 1;
+      return 2*(f1->edge_id(h.second)) + 1;
     else if(f1->id() < f2->id())
-      return 2*(3 * f1->id() + h.second);
+      return 2*(f1->edge_id(h.second));
     else
-      return 2*(3 * f2->id() + f2->index(f1)) + 1;
+      return 2*(f1->edge_id(h.second)) + 1;
   }
 
 private:
@@ -152,13 +152,9 @@ public:
     CGAL_assertion(!tr.is_infinite(f1) || !tr.is_infinite(f2));
 
     if(tr.is_infinite(f1))
-      return 3 * f2->id() + f2->index(f1);
-    else if(tr.is_infinite(f2))
-      return 3 * f1->id() + e.second;
-    else if(f1->id() < f2->id())
-      return 3 * f1->id() + e.second;
+      return f2->edge_id(f2->index(f1));
     else
-      return 3 * f2->id() + f2->index(f1);
+      return f1->edge_id(e.second);
   }
 
 private:
@@ -376,6 +372,39 @@ put(PropertyTag p, CGAL_2D_TRIANGULATION
   put(pmap, key, value);
 }
 
+template < CGAL_2D_TRIANGULATION_TEMPLATE_PARAMETERS >
+void set_triangulation_ids(CGAL_2D_TRIANGULATION& g)
+{
+  typedef typename boost::graph_traits< CGAL_2D_TRIANGULATION >::vertex_descriptor   vertex_descriptor;
+  typedef typename boost::graph_traits< CGAL_2D_TRIANGULATION >::halfedge_descriptor halfedge_descriptor;
+  typedef typename boost::graph_traits< CGAL_2D_TRIANGULATION >::edge_descriptor     edge_descriptor;
+  typedef typename boost::graph_traits< CGAL_2D_TRIANGULATION >::face_descriptor     face_descriptor;
+
+  int vid = 0;
+  for(vertex_descriptor vd : vertices(g))
+    vd->id() = vid++;
+
+  int eid = 0;
+  for(edge_descriptor ed : edges(g))
+  {
+    halfedge_descriptor hd = halfedge(ed, g);
+    face_descriptor fd = face(hd, g);
+    if(fd != boost::graph_traits< CGAL_2D_TRIANGULATION >::null_face())
+      fd->edge_id(hd.second) = eid;
+
+    halfedge_descriptor opp_hd = opposite(hd, g);
+    face_descriptor opp_fd = face(opp_hd, g);
+    if(opp_fd != boost::graph_traits< CGAL_2D_TRIANGULATION >::null_face())
+      opp_fd->edge_id(opp_hd.second) = eid;
+
+    ++eid;
+  }
+
+  int fid = 0;
+  for(face_descriptor fd : faces(g))
+    fd->id() = fid++;
+}
+
 } // namespace CGAL
 
 #undef CGAL_2D_TRIANGULATION_TEMPLATE_PARAMETERS
diff -pruN 5.0.3-1/include/CGAL/boost/graph/io.h 5.1-1/include/CGAL/boost/graph/io.h
--- 5.0.3-1/include/CGAL/boost/graph/io.h	2020-08-01 19:02:14.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/io.h	2020-09-08 12:14:40.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/io.h $
-// $Id: io.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/io.h $
+// $Id: io.h d567a0d 2020-07-16T09:52:32+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -27,16 +27,27 @@
 #include <CGAL/IO/write_vtk.h>
 
 namespace CGAL {
-  /*!
-   \ingroup PkgBGLIOFct
-    writes the graph `g` in the wrl format (VRML 2.0).
 
-    \cgalNamedParamsBegin
-    *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `g`.
-    *       If this parameter is omitted, an internal property map for
-    *       `CGAL::vertex_point_t` should be available in `FaceGraph`\cgalParamEnd
-    * \cgalNamedParamsEnd
-    */
+/*!
+ \ingroup PkgBGLIOFct
+
+  writes the graph `g` in the wrl format (VRML 2.0).
+
+  \param os the output stream
+  \param g the graph to be written
+  \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+
+  \cgalNamedParamsBegin
+    \cgalParamNBegin{vertex_point_map}
+      \cgalParamDescription{a property map associating points to the vertices of `g`}
+      \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<FaceGraph>::%vertex_descriptor`
+                     as key type and `%Point_3` as value type}
+      \cgalParamDefault{`boost::get(CGAL::vertex_point, g)`}
+      \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+                      must be available in `FaceGraph`.}
+    \cgalParamNEnd
+  \cgalNamedParamsEnd
+*/
 template <typename FaceGraph, typename NamedParameters>
 bool write_wrl(std::ostream& os,
                const FaceGraph& g,
@@ -49,7 +60,7 @@ bool write_wrl(std::ostream& os,
   using parameters::get_parameter;
   using parameters::choose_parameter;
 
-  typename Polygon_mesh_processing::GetVertexPointMap<FaceGraph, NamedParameters>::const_type
+  typename CGAL::GetVertexPointMap<FaceGraph, NamedParameters>::const_type
       vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
                              get_const_property_map(CGAL::vertex_point, g));
 
@@ -114,18 +125,27 @@ bool write_wrl(std::ostream& os,
 }
 
 /*!
-   \ingroup PkgBGLIOFct
-    writes the graph `g` in the OFF format.
+ \ingroup PkgBGLIOFct
 
-    \cgalNamedParamsBegin
-    *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `g`.
-    *       If this parameter is omitted, an internal property map for
-    *       `CGAL::vertex_point_t` should be available in `FaceGraph`\cgalParamEnd
-    * \cgalNamedParamsEnd
+  writes the graph `g` in the OFF format.
 
-    \sa Overloads of this function for specific models of the concept `FaceGraph`.
+  \param os the output stream
+  \param g the graph to be written
+  \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+
+  \cgalNamedParamsBegin
+    \cgalParamNBegin{vertex_point_map}
+      \cgalParamDescription{a property map associating points to the vertices of `g`}
+      \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<FaceGraph>::%vertex_descriptor`
+                     as key type and `%Point_3` as value type}
+      \cgalParamDefault{`boost::get(CGAL::vertex_point, g)`}
+      \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+                      must be available in `FaceGraph`.}
+    \cgalParamNEnd
+  \cgalNamedParamsEnd
 
-  */
+  \sa Overloads of this function for specific models of the concept `FaceGraph`.
+*/
 template <typename FaceGraph, typename NamedParameters>
 bool write_off(std::ostream& os,
                const FaceGraph& g,
@@ -139,7 +159,7 @@ bool write_off(std::ostream& os,
   using parameters::choose_parameter;
   using parameters::get_parameter;
 
-  typename Polygon_mesh_processing::GetVertexPointMap<FaceGraph, NamedParameters>::const_type
+  typename CGAL::GetVertexPointMap<FaceGraph, NamedParameters>::const_type
       vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
                              get_const_property_map(CGAL::vertex_point, g));
   vertices_size_type nv = static_cast<vertices_size_type>(std::distance(vertices(g).first, vertices(g).second));
@@ -236,19 +256,30 @@ inline std::string next_non_comment(std:
 
 
 /*!
-   \ingroup PkgBGLIOFct
-    reads the graph `g` from data in the OFF format. Ignores comment lines which start with a hash, and lines with whitespace.
+ \ingroup PkgBGLIOFct
 
-    \cgalNamedParamsBegin
-    *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `g`.
-    *       If this parameter is omitted, an internal property map for
-    *       `CGAL::vertex_point_t` should be available in `FaceGraph`\cgalParamEnd
-    * \cgalNamedParamsEnd
-    \sa Overloads of this function for specific models of the concept `FaceGraph`.
-    \pre The data must represent a 2-manifold
-    \attention The graph `g` is not cleared, and the data from the stream are added.
+ reads the graph `g` from data in the OFF format. Ignores comment lines which start with a hash, and lines with whitespace.
 
-  */
+ \param is the input stream
+ \param g the graph to be read
+ \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+
+ \cgalNamedParamsBegin
+   \cgalParamNBegin{vertex_point_map}
+     \cgalParamDescription{a property map associating points to the vertices of `g`}
+     \cgalParamType{a class model of `WritablePropertyMap` with `boost::graph_traits<FaceGraph>::%vertex_descriptor`
+                    as key type and `%Point_3` as value type}
+     \cgalParamDefault{`boost::get(CGAL::vertex_point, g)`}
+     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+                     must be available in `FaceGraph`.}
+   \cgalParamNEnd
+ \cgalNamedParamsEnd
+
+ \sa Overloads of this function for specific models of the concept `FaceGraph`.
+ \pre The data must represent a 2-manifold
+
+ \attention The graph `g` is not cleared, and the data from the stream are added.
+*/
 template <typename FaceGraph, typename NamedParameters>
 bool read_off(std::istream& is,
               FaceGraph& g,
@@ -262,7 +293,7 @@ bool read_off(std::istream& is,
   typedef typename boost::graph_traits<FaceGraph>::vertices_size_type vertices_size_type;
   typedef typename boost::graph_traits<FaceGraph>::faces_size_type faces_size_type;
 
-  typedef typename Polygon_mesh_processing::GetVertexPointMap<FaceGraph, NamedParameters>::type Vpm;
+  typedef typename CGAL::GetVertexPointMap<FaceGraph, NamedParameters>::type Vpm;
   typedef  typename boost::property_traits<Vpm>::value_type Point_3;
 
   Vpm vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
@@ -365,7 +396,7 @@ bool write_inp(std::ostream& os,
   typedef typename boost::graph_traits<FaceGraph>::face_descriptor face_descriptor;
   typedef typename boost::graph_traits<FaceGraph>::vertices_size_type vertices_size_type;
 
-  typedef typename Polygon_mesh_processing::GetVertexPointMap<FaceGraph, NamedParameters>::const_type VPM;
+  typedef typename CGAL::GetVertexPointMap<FaceGraph, NamedParameters>::const_type VPM;
   typedef typename boost::property_traits<VPM>::value_type Point_3;
 
   using parameters::choose_parameter;
@@ -417,11 +448,9 @@ write_polys(std::ostream& os,
 {
   typedef typename boost::graph_traits<Mesh>::vertex_descriptor vertex_descriptor;
   typedef typename boost::graph_traits<Mesh>::face_iterator face_iterator;
-  typedef typename CGAL::Polygon_mesh_processing::GetVertexIndexMap<Mesh, NamedParameters>::type Vimap;
-  using parameters::get_parameter;
-  using parameters::choose_parameter;
-  Vimap V = choose_parameter(get_parameter(np, internal_np::vertex_index),
-                             get_const_property_map(boost::vertex_index, mesh));
+
+  typedef typename CGAL::GetInitializedVertexIndexMap<Mesh, NamedParameters>::const_type Vimap;
+  Vimap V = CGAL::get_initialized_vertex_index_map(mesh, np);
 
   std::vector<std::size_t> connectivity_table;
   std::vector<std::size_t> offsets;
@@ -454,11 +483,9 @@ write_polys_tag(std::ostream& os,
 {
   typedef typename boost::graph_traits<Mesh>::vertex_descriptor vertex_descriptor;
   typedef typename boost::graph_traits<Mesh>::face_iterator face_iterator;
-  typedef typename CGAL::Polygon_mesh_processing::GetVertexIndexMap<Mesh, NamedParameters>::type Vimap;
-  using parameters::get_parameter;
-  using parameters::choose_parameter;
-  Vimap V = choose_parameter(get_parameter(np, internal_np::vertex_index),
-                             get_const_property_map(boost::vertex_index, mesh));
+
+  typedef typename CGAL::GetInitializedVertexIndexMap<Mesh, NamedParameters>::const_type Vimap;
+  Vimap V = CGAL::get_initialized_vertex_index_map(mesh, np);
 
   std::string formatattribute =
     binary ? " format=\"appended\"" : " format=\"ascii\"";
@@ -545,7 +572,7 @@ write_points_tag(std::ostream& os,
                  const NamedParameters& np)
 {
   typedef typename boost::graph_traits<Mesh>::vertex_iterator vertex_iterator;
-  typedef typename CGAL::Polygon_mesh_processing::GetVertexPointMap<Mesh, NamedParameters>::const_type Vpmap;
+  typedef typename CGAL::GetVertexPointMap<Mesh, NamedParameters>::const_type Vpmap;
   using parameters::get_parameter;
   using parameters::choose_parameter;
   Vpmap vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
@@ -590,7 +617,7 @@ write_polys_points(std::ostream& os,
                    const NamedParameters& np)
 {
   typedef typename boost::graph_traits<Mesh>::vertex_iterator vertex_iterator;
-  typedef typename CGAL::Polygon_mesh_processing::GetVertexPointMap<Mesh, NamedParameters>::const_type Vpmap;
+  typedef typename CGAL::GetVertexPointMap<Mesh, NamedParameters>::const_type Vpmap;
   using parameters::get_parameter;
   using parameters::choose_parameter;
   Vpmap vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
@@ -618,25 +645,34 @@ write_polys_points(std::ostream& os,
  * \brief  writes a triangulated surface mesh in the `PolyData` XML format.
  *
  * \tparam TriangleMesh a model of `FaceListGraph` with only triangle faces.
- * \tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+ * \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
  *
  * \param os the stream used for writing.
  * \param mesh the triangle mesh to be written.
- * \param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the
- * ones listed below
+ * \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{use_binary_mode} a Boolean indicating if the
- *    data should be written in binary (`true`, the default) or in ASCII (`false`).
- *     \cgalParamEnd
- *    \cgalParamBegin{vertex_point_map} the property map with the points associated to
- * the vertices of `mesh`. If this parameter is omitted, an internal property map for
- *       `CGAL::vertex_point_t` must be available in `TriangleMesh`.
- *     \cgalParamEnd
- *    \cgalParamBegin{vertex_index_map} the property map with the indices associated to
- * the vertices of `mesh`. If this parameter is omitted, an internal property map for
- *       `CGAL::vertex_index_t` must be available in `TriangleMesh`.
- *     \cgalParamEnd
+ *   \cgalParamNBegin{use_binary_mode}
+ *     \cgalParamDescription{Boolean indicating if the data should be written in binary (`true`) or in ASCII (`false`)}
+ *     \cgalParamType{Boolean}
+ *     \cgalParamDefault{`true`}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `mesh`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+ *                    as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, mesh)`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     must be available in `TriangleMesh`.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{vertex_index_map}
+ *     \cgalParamDescription{a property map associating to each vertex of `mesh` a unique index between `0` and `num_vertices(mesh) - 1`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+ *                    as key type and `std::size_t` as value type}
+ *     \cgalParamDefault{an automatically indexed internal map}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  */
 template<class TriangleMesh,
diff -pruN 5.0.3-1/include/CGAL/boost/graph/iterator.h 5.1-1/include/CGAL/boost/graph/iterator.h
--- 5.0.3-1/include/CGAL/boost/graph/iterator.h	2020-08-01 19:02:14.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/iterator.h	2020-09-08 12:14:40.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/iterator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/iterator.h $
 // $Id: iterator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/METIS/partition_dual_graph.h 5.1-1/include/CGAL/boost/graph/METIS/partition_dual_graph.h
--- 5.0.3-1/include/CGAL/boost/graph/METIS/partition_dual_graph.h	2020-08-01 19:02:14.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/METIS/partition_dual_graph.h	2020-09-08 12:14:40.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/METIS/partition_dual_graph.h $
-// $Id: partition_dual_graph.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/METIS/partition_dual_graph.h $
+// $Id: partition_dual_graph.h a84927d 2020-07-23T17:15:44+02:00 Laurent Rineau
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Mael Rouxel-Labbé
@@ -42,17 +42,14 @@ void partition_dual_graph(const Triangle
   CGAL_precondition(CGAL::is_triangle_mesh(tm));
   CGAL_precondition_msg(nparts > 1, ("Partitioning requires a number of parts > 1"));
 
-  using parameters::choose_parameter;
   using parameters::get_parameter;
 
   typedef typename boost::graph_traits<TriangleMesh>::vertex_descriptor   vertex_descriptor;
   typedef typename boost::graph_traits<TriangleMesh>::halfedge_descriptor halfedge_descriptor;
   typedef typename boost::graph_traits<TriangleMesh>::face_iterator       face_iterator;
 
-  // vertex index map
-  typedef typename CGAL::Polygon_mesh_processing::GetVertexIndexMap<TriangleMesh, NamedParameters>::type Indices;
-  Indices indices = choose_parameter(get_parameter(np, internal_np::vertex_index),
-                                 get_const_property_map(boost::vertex_index, tm));
+  typedef typename CGAL::GetInitializedVertexIndexMap<TriangleMesh, NamedParameters>::type Indices;
+  Indices indices = CGAL::get_initialized_vertex_index_map(tm, np);
 
   idx_t nn = static_cast<idx_t>(num_vertices(tm));
   idx_t ne = static_cast<idx_t>(num_faces(tm));
@@ -132,38 +129,56 @@ void partition_dual_graph(const Triangle
 
 /// \ingroup PkgBGLPartition
 ///
-/// Computes a partition of the input triangular mesh into `nparts` parts,
+/// computes a partition of the input triangular mesh into `nparts` parts,
 /// based on the mesh's dual graph. The resulting partition is stored in the vertex and/or face
 /// property maps that are passed as parameters using \ref bgl_namedparameters "Named Parameters".
 ///
-/// Property map for `CGAL::vertex_index_t` should be either available
-/// as an internal property map to `tm` or provided as \ref bgl_namedparameters "Named Parameters".
-///
 /// \param tm a triangle mesh
 /// \param nparts the number of parts in the final partition
-/// \param np optional \ref bgl_namedparameters "Named Parameters" described below
+/// \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 ///
-/// \tparam TriangleMesh is a model of the `FaceListGraph` concept.
+/// \tparam TriangleMesh is a model of the `FaceListGraph` concept
 /// \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 ///
 /// \cgalNamedParamsBegin
-///   \cgalParamBegin{vertex_index_map}
-///     is a property map containing the index of each vertex of `tm` intialized from `0` to `num_vertices(tm)-1`.
-///   \cgalParamEnd
-///   \cgalParamBegin{METIS_options}
-///     is a parameter used in to pass options to the METIS mesh
-///     partitioner. The many options of METIS are not described here. Instead, users
-///     should refer to the <a href="http://glaros.dtc.umn.edu/gkhome/fetch/sw/metis/manual.pdf">documentation</a>
-///     of METIS directly.
-///   \cgalParamEnd
-///   \cgalParamBegin{vertex_partition_id_map}
-///     is a property map that contains (after the function has been run)
-///     the ID of the subpart for each vertex of `tm`.
-///   \cgalParamEnd
-///   \cgalParamBegin{face_partition_id_map}
-///     is a property map that contains (after the function has been run)
-///     the ID of the subpart for each face of `tm`.
-///   \cgalParamEnd
+///   \cgalParamNBegin{vertex_index_map}
+///     \cgalParamDescription{a property map associating to each vertex of `tm` a unique index between `0` and `num_vertices(tm) - 1`}
+///     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+///                    as key type and `std::size_t` as value type}
+///     \cgalParamDefault{an automatically indexed internal map}
+///     \cgalParamExtra{If this parameter is not passed, internal machinery will create and initialize
+///                     a face index property map, either using the internal property map if it exists
+///                     or using an external map. The latter might result in  - slightly - worsened performance
+///                     in case of non-constant complexity for index access.}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{METIS_options}
+///     \cgalParamDescription{a parameter used in to pass options to the METIS mesh partitioner}
+///     \cgalParamType{an array of size `METIS_NOPTIONS` with value type `idx_t` (an integer type defined by METIS)}
+///     \cgalParamDefault{an array of size `METIS_NOPTIONS` with value type `idx_t`,
+///                       initialized using the function `METIS_SetDefaultOptions()`}
+///     \cgalParamExtra{The many options of METIS are not described here. Instead, users should refer
+///                     to the <a href="http://glaros.dtc.umn.edu/gkhome/fetch/sw/metis/manual.pdf">documentation</a>
+///                     of METIS directly.}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{vertex_partition_id_map}
+///     \cgalParamDescription{a property map that contains (after the function has been run)
+///                           the ID of the subpart for each vertex of `tm`}
+///     \cgalParamType{a class model of `ReadWritePropertyMap` with
+///                    `boost::graph_traits<TriangleMesh>::%vertex_descriptor` as key type and
+///                    `int` as value type}
+///     \cgalParamDefault{unused}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{face_partition_id_map}
+///     \cgalParamDescription{a property map that contains (after the function has been run)
+///                           the ID of the subpart for each face of `tm`}
+///     \cgalParamType{a class model of `ReadWritePropertyMap` with
+///                    `boost::graph_traits<TriangleMesh>::%face_descriptor` as key type and
+///                    `int` as value type}
+///     \cgalParamDefault{unused}
+///   \cgalParamNEnd
 /// \cgalNamedParamsEnd
 ///
 /// \pre `tm` is a pure triangular surface mesh: there are no edges
diff -pruN 5.0.3-1/include/CGAL/boost/graph/METIS/partition_graph.h 5.1-1/include/CGAL/boost/graph/METIS/partition_graph.h
--- 5.0.3-1/include/CGAL/boost/graph/METIS/partition_graph.h	2020-08-01 19:02:14.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/METIS/partition_graph.h	2020-09-08 12:14:40.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/METIS/partition_graph.h $
-// $Id: partition_graph.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/METIS/partition_graph.h $
+// $Id: partition_graph.h a84927d 2020-07-23T17:15:44+02:00 Laurent Rineau
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Mael Rouxel-Labbé
@@ -77,17 +77,14 @@ void partition_graph(const TriangleMesh&
   CGAL_precondition(CGAL::is_triangle_mesh(tm));
   CGAL_precondition_msg(nparts > 1, ("Partitioning requires a number of parts > 1"));
 
-  using parameters::choose_parameter;
   using parameters::get_parameter;
 
   typedef typename boost::graph_traits<TriangleMesh>::vertex_descriptor   vertex_descriptor;
   typedef typename boost::graph_traits<TriangleMesh>::halfedge_descriptor halfedge_descriptor;
   typedef typename boost::graph_traits<TriangleMesh>::face_iterator       face_iterator;
 
-  //Vertex index map
-  typedef typename CGAL::Polygon_mesh_processing::GetVertexIndexMap<TriangleMesh, NamedParameters>::type Indices;
-  Indices indices = choose_parameter(get_parameter(np, internal_np::vertex_index),
-                                 get_const_property_map(boost::vertex_index, tm));
+  typedef typename CGAL::GetInitializedVertexIndexMap<TriangleMesh, NamedParameters>::type Indices;
+  Indices indices = CGAL::get_initialized_vertex_index_map(tm, np);
 
   idx_t nn = static_cast<idx_t>(num_vertices(tm));
   idx_t ne = static_cast<idx_t>(num_faces(tm));
@@ -164,38 +161,56 @@ void partition_graph(const TriangleMesh&
 
 /// \ingroup PkgBGLPartition
 ///
-/// Computes a partition of the input triangular mesh into `nparts` parts, based on the
+/// computes a partition of the input triangular mesh into `nparts` parts, based on the
 /// mesh's nodal graph. The resulting partition is stored in the vertex and/or face
 /// property maps that are passed as parameters using \ref bgl_namedparameters "Named Parameters".
 ///
-/// Property map for `CGAL::vertex_index_t` should be either available
-/// as an internal property map to `tm` or provided as \ref bgl_namedparameters "Named Parameters".
-///
 /// \param tm a triangle mesh
 /// \param nparts the number of parts in the final partition
-/// \param np optional \ref bgl_namedparameters "Named Parameters" described below
+/// \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 ///
 /// \tparam TriangleMesh is a model of the `FaceListGraph` concept.
 /// \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 ///
 /// \cgalNamedParamsBegin
-///   \cgalParamBegin{vertex_index_map}
-///     is a property map containing the index of each vertex of `tm` intialized from `0` to `num_vertices(tm)-1`.
-///   \cgalParamEnd
-///   \cgalParamBegin{METIS_options}
-///     is a parameter used in to pass options to the METIS mesh
-///     partitioner. The many options of METIS are not described here. Instead, users
-///     should refer to the <a href="http://glaros.dtc.umn.edu/gkhome/fetch/sw/metis/manual.pdf">documentation</a>
-///     of METIS directly.
-///   \cgalParamEnd
-///   \cgalParamBegin{vertex_partition_id_map}
-///     is a property map that contains (after the function has been run)
-///     the ID of the subpart for each vertex of `tm`.
-///   \cgalParamEnd
-///   \cgalParamBegin{face_partition_id_map}
-///     is a property map that contains (after the function has been run)
-///     the ID of the subpart for each face of `tm`.
-///   \cgalParamEnd
+///   \cgalParamNBegin{vertex_index_map}
+///     \cgalParamDescription{a property map associating to each vertex of `tm` a unique index between `0` and `num_vertices(tm) - 1`}
+///     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+///                    as key type and `std::size_t` as value type}
+///     \cgalParamDefault{an automatically indexed internal map}
+///     \cgalParamExtra{If this parameter is not passed, internal machinery will create and initialize
+///                     a face index property map, either using the internal property map if it exists
+///                     or using an external map. The latter might result in  - slightly - worsened performance
+///                     in case of non-constant complexity for index access.}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{METIS_options}
+///     \cgalParamDescription{a parameter used in to pass options to the METIS mesh partitioner}
+///     \cgalParamType{an array of size `METIS_NOPTIONS` with value type `idx_t` (an integer type defined by METIS)}
+///     \cgalParamDefault{an array of size `METIS_NOPTIONS` with value type `idx_t`,
+///                       initialized using the function `METIS_SetDefaultOptions()`}
+///     \cgalParamExtra{The many options of METIS are not described here. Instead, users should refer
+///                     to the <a href="http://glaros.dtc.umn.edu/gkhome/fetch/sw/metis/manual.pdf">documentation</a>
+///                     of METIS directly.}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{vertex_partition_id_map}
+///     \cgalParamDescription{a property map that contains (after the function has been run)
+///                           the ID of the subpart for each vertex of `tm`}
+///     \cgalParamType{a class model of `ReadWritePropertyMap` with
+///                    `boost::graph_traits<TriangleMesh>::%vertex_descriptor` as key type and
+///                    `int` as value type}
+///     \cgalParamDefault{unused}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{face_partition_id_map}
+///     \cgalParamDescription{a property map that contains (after the function has been run)
+///                           the ID of the subpart for each face of `tm`}
+///     \cgalParamType{a class model of `ReadWritePropertyMap` with
+///                    `boost::graph_traits<TriangleMesh>::%face_descriptor` as key type and
+///                    `int` as value type}
+///     \cgalParamDefault{unused}
+///   \cgalParamNEnd
 /// \cgalNamedParamsEnd
 ///
 /// \pre `tm` is a pure triangular surface mesh: there are no edges
diff -pruN 5.0.3-1/include/CGAL/boost/graph/Named_function_parameters.h 5.1-1/include/CGAL/boost/graph/Named_function_parameters.h
--- 5.0.3-1/include/CGAL/boost/graph/Named_function_parameters.h	2020-08-01 19:02:14.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/Named_function_parameters.h	2020-09-08 12:14:40.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/Named_function_parameters.h $
-// $Id: Named_function_parameters.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/Named_function_parameters.h $
+// $Id: Named_function_parameters.h 2556196 2020-03-23T08:56:47+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -194,6 +194,7 @@ get_parameter(const Named_function_param
   return internal_np::get_parameter_impl(static_cast<const internal_np::Named_params_impl<T, Tag, Base>&>(np), tag);
 }
 
+// Two parameters, non-trivial default value
 template <typename D>
 D choose_parameter(const internal_np::Param_not_found&, const D& d)
 {
@@ -201,8 +202,20 @@ D choose_parameter(const internal_np::Pa
 }
 
 template <typename T, typename D>
-const T&
-choose_parameter(const T& t, const D&)
+const T& choose_parameter(const T& t, const D&)
+{
+  return t;
+}
+
+// single parameter so that we can avoid a default construction
+template <typename D>
+D choose_parameter(const internal_np::Param_not_found&)
+{
+  return D();
+}
+
+template <typename D, typename T>
+const T& choose_parameter(const T& t)
 {
   return t;
 }
diff -pruN 5.0.3-1/include/CGAL/boost/graph/named_params_helper.h 5.1-1/include/CGAL/boost/graph/named_params_helper.h
--- 5.0.3-1/include/CGAL/boost/graph/named_params_helper.h	2020-08-01 19:02:14.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/named_params_helper.h	2020-09-08 12:14:40.000000000 +0000
@@ -10,8 +10,8 @@
 //=======================================================================
 // Copyright (c) 2007-2015  GeometryFactory (France).  All rights reserved.
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/named_params_helper.h $
-// $Id: named_params_helper.h 590612a 2020-03-26T13:25:16+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/named_params_helper.h $
+// $Id: named_params_helper.h fefdd13 2020-05-06T08:59:40+02:00 Simon Giraudot
 // SPDX-License-Identifier: BSL-1.0
 //
 // Author(s)     : Andreas Fabri, Fernando Cacciola, Jane Tournois
@@ -19,18 +19,21 @@
 #ifndef CGAL_BOOST_GRAPH_NAMED_PARAMETERS_HELPERS_H
 #define CGAL_BOOST_GRAPH_NAMED_PARAMETERS_HELPERS_H
 
+#include <CGAL/boost/graph/internal/initialized_index_maps_helpers.h>
 #include <CGAL/boost/graph/Named_function_parameters.h>
-
+#include <CGAL/boost/graph/properties.h>
+#include <CGAL/Dynamic_property_map.h>
 #include <CGAL/Kernel_traits.h>
 #include <CGAL/Origin.h>
+#include <CGAL/iterator.h>
 
 #include <CGAL/property_map.h>
-#include <CGAL/boost/graph/properties.h>
+
 #include <boost/mpl/if.hpp>
 #include <boost/mpl/has_xxx.hpp>
-
 #include <boost/type_traits/is_same.hpp>
 
+#include <type_traits>
 
 namespace CGAL {
 
@@ -39,6 +42,7 @@ namespace CGAL {
   class Default_diagonalize_traits;
   class Eigen_svd;
   class Lapack_svd;
+  struct Alpha_expansion_boost_adjacency_list_tag;
   //
 
 
@@ -104,7 +108,8 @@ namespace CGAL {
     property_map_selector<PolygonMesh, PropertyTag> pms;
     return pms.get_const_pmap(p, pmesh);
   }
-// shortcut for accessing the value type of the property map
+
+  // Shortcut for accessing the value type of the property map
   template <class Graph, class Property>
   class property_map_value {
     typedef typename boost::property_map<Graph, Property>::const_type PMap;
@@ -112,10 +117,8 @@ namespace CGAL {
     typedef typename boost::property_traits<PMap>::value_type type;
   };
 
-  namespace Polygon_mesh_processing
-  {
-
-  template<typename PolygonMesh, typename NamedParameters>
+  template<typename PolygonMesh,
+           typename NamedParameters = Named_function_parameters<bool, internal_np::all_default_t> >
   class GetVertexPointMap
   {
     typedef typename property_map_selector<PolygonMesh, boost::vertex_point_t>::const_type
@@ -135,6 +138,8 @@ namespace CGAL {
       > ::type  const_type;
   };
 
+  namespace Polygon_mesh_processing {
+
   template<typename PolygonMesh, typename NamedParameters>
   class GetK
   {
@@ -145,6 +150,8 @@ namespace CGAL {
     typedef typename CGAL::Kernel_traits<Point>::Kernel Kernel;
   };
 
+  } // namespace Polygon_mesh_processing
+
   template<typename PolygonMesh,
            typename NamedParametersGT = Named_function_parameters<bool, internal_np::all_default_t>,
            typename NamedParametersVPM = NamedParametersGT >
@@ -161,10 +168,9 @@ namespace CGAL {
 
     struct Fake_GT {};//to be used if there is no internal vertex_point_map in PolygonMesh
 
-    typedef typename boost::mpl::if_c< Has_internal_pmap::value || !boost::is_same<internal_np::Param_not_found, NP_vpm>::value
-                                     , typename GetK<PolygonMesh, NamedParametersVPM>::Kernel
-                                     , Fake_GT
-    >::type DefaultKernel;
+    typedef typename boost::mpl::if_c<Has_internal_pmap::value || !boost::is_same<internal_np::Param_not_found, NP_vpm>::value,
+                                     typename Polygon_mesh_processing::GetK<PolygonMesh, NamedParametersVPM>::Kernel,
+                                     Fake_GT>::type DefaultKernel;
 
   public:
     typedef typename internal_np::Lookup_named_param_def <
@@ -174,37 +180,105 @@ namespace CGAL {
     > ::type  type;
   };
 
-  template<typename PolygonMesh, typename NamedParameters>
-  class GetFaceIndexMap
-  {
-    typedef typename property_map_selector<PolygonMesh, boost::face_index_t>::type DefaultMap;
-    typedef typename property_map_selector<PolygonMesh, boost::face_index_t>::const_type DefaultMap_const;
-  public:
-    typedef typename internal_np::Lookup_named_param_def <
-    internal_np::face_index_t,
-    NamedParameters,
-    DefaultMap
-    > ::type  type;
-    typedef typename internal_np::Lookup_named_param_def <
-      internal_np::face_index_t,
-      NamedParameters,
-      DefaultMap_const
-      > ::type  const_type;
-    typedef typename boost::is_same<type, DefaultMap>::type Is_internal_map;
-    typedef typename boost::is_same<const_type, DefaultMap_const>::type Is_internal_map_const;
-  };
+// Define the following structs:
+//
+// GetInitializedVertexIndexMap
+// GetInitializedHalfedgeIndexMap
+// GetInitializedEdgeIndexMap
+// GetInitializedFaceIndexMap
+
+#define CGAL_DEF_GET_INDEX_TYPE(CTYPE, DTYPE, STYPE)                                               \
+template <typename Graph,                                                                          \
+          typename NamedParameters =                                                               \
+            CGAL::Named_function_parameters<bool, CGAL::internal_np::all_default_t> >              \
+struct GetInitialized##CTYPE##IndexMap                                                             \
+  : public BGL::internal::GetInitializedIndexMap<internal_np::DTYPE##_index_t,                     \
+                                                 boost::DTYPE##_index_t,                           \
+                                                 CGAL::dynamic_##DTYPE##_property_t<STYPE>,        \
+                                                 Graph, NamedParameters>                           \
+{ };
+
+CGAL_DEF_GET_INDEX_TYPE(Vertex, vertex, typename boost::graph_traits<Graph>::vertices_size_type)
+CGAL_DEF_GET_INDEX_TYPE(Halfedge, halfedge, typename boost::graph_traits<Graph>::halfedges_size_type)
+CGAL_DEF_GET_INDEX_TYPE(Edge, edge, typename boost::graph_traits<Graph>::edges_size_type)
+CGAL_DEF_GET_INDEX_TYPE(Face, face, typename boost::graph_traits<Graph>::faces_size_type)
 
-  template<typename PolygonMesh, typename NamedParameters>
-  class GetVertexIndexMap
-  {
-    typedef typename property_map_selector<PolygonMesh, boost::vertex_index_t>::type DefaultMap;
-  public:
-    typedef typename internal_np::Lookup_named_param_def <
-    internal_np::vertex_index_t,
-    NamedParameters,
-    DefaultMap
-    > ::type  type;
-  };
+#undef CGAL_DEF_GET_INDEX_TYPE
+
+// Define the following functions:
+//
+// get_initialized_vertex_index_map()
+// get_initialized_halfedge_index_map()
+// get_initialized_edge_index_map()
+// get_initialized_face_index_map()
+//
+// The function returns:
+// - the index property map passed in the NPs, if passed in the NPs; it must be initialized by the user;
+// - the internal index property map if it is the graph has one. It is initialized if needed and possible;
+// - an initialized dynamic pmap otherwise.
+
+#define CGAL_DEF_GET_INITIALIZED_INDEX_MAP(DTYPE, STYPE)                                           \
+template <typename Graph,                                                                          \
+          typename NamedParameters>                                                                \
+typename BGL::internal::GetInitializedIndexMap<CGAL::internal_np::DTYPE##_index_t,                 \
+                                               boost::DTYPE##_index_t,                             \
+                                               CGAL::dynamic_##DTYPE##_property_t<STYPE>,          \
+                                               Graph, NamedParameters>::const_type                 \
+get_initialized_##DTYPE##_index_map(const Graph& g,                                                \
+                                    const NamedParameters& np)                                     \
+{                                                                                                  \
+  typedef BGL::internal::GetInitializedIndexMap<CGAL::internal_np::DTYPE##_index_t,                \
+                                                boost::DTYPE##_index_t,                            \
+                                                CGAL::dynamic_##DTYPE##_property_t<STYPE>,         \
+                                                Graph, NamedParameters>          Index_map_getter; \
+  return Index_map_getter::get_const(CGAL::internal_np::DTYPE##_index_t{}, g, np);                 \
+}                                                                                                  \
+template <typename Graph>                                                                          \
+typename BGL::internal::GetInitializedIndexMap<CGAL::internal_np::DTYPE##_index_t,                 \
+                                               boost::DTYPE##_index_t,                             \
+                                               CGAL::dynamic_##DTYPE##_property_t<STYPE>,          \
+                                               Graph>::const_type                                  \
+get_initialized_##DTYPE##_index_map(const Graph& g)                                                \
+{                                                                                                  \
+  return get_initialized_##DTYPE##_index_map(g, CGAL::parameters::all_default());                  \
+}                                                                                                  \
+/* same as above, non-const version*/                                                              \
+template <typename Graph,                                                                          \
+          typename NamedParameters,                                                                \
+          /*otherwise compilers will try to use 'Graph := const PM' and things will go badly*/     \
+          std::enable_if_t<                                                                        \
+            !std::is_const<typename std::remove_reference<Graph>::type>::value, int> = 0>          \
+typename BGL::internal::GetInitializedIndexMap<CGAL::internal_np::DTYPE##_index_t,                 \
+                                               boost::DTYPE##_index_t,                             \
+                                               CGAL::dynamic_##DTYPE##_property_t<STYPE>,          \
+                                               Graph, NamedParameters>::type                       \
+get_initialized_##DTYPE##_index_map(Graph& g,                                                      \
+                                    const NamedParameters& np)                                     \
+{                                                                                                  \
+  typedef BGL::internal::GetInitializedIndexMap<CGAL::internal_np::DTYPE##_index_t,                \
+                                                boost::DTYPE##_index_t,                            \
+                                                CGAL::dynamic_##DTYPE##_property_t<STYPE>,         \
+                                                Graph, NamedParameters>          Index_map_getter; \
+  return Index_map_getter::get(CGAL::internal_np::DTYPE##_index_t{}, g, np);                       \
+}                                                                                                  \
+template <typename Graph,                                                                          \
+          std::enable_if_t<                                                                        \
+            !std::is_const<typename std::remove_reference<Graph>::type>::value, int> = 0>          \
+typename BGL::internal::GetInitializedIndexMap<CGAL::internal_np::DTYPE##_index_t,                 \
+                                               boost::DTYPE##_index_t,                             \
+                                               CGAL::dynamic_##DTYPE##_property_t<STYPE>,          \
+                                               Graph>::type                                        \
+get_initialized_##DTYPE##_index_map(Graph& g)                                                      \
+{                                                                                                  \
+  return get_initialized_##DTYPE##_index_map(g, CGAL::parameters::all_default());                  \
+}
+
+CGAL_DEF_GET_INITIALIZED_INDEX_MAP(vertex, typename boost::graph_traits<Graph>::vertices_size_type)
+CGAL_DEF_GET_INITIALIZED_INDEX_MAP(halfedge, typename boost::graph_traits<Graph>::halfedges_size_type)
+CGAL_DEF_GET_INITIALIZED_INDEX_MAP(edge, typename boost::graph_traits<Graph>::edges_size_type)
+CGAL_DEF_GET_INITIALIZED_INDEX_MAP(face, typename boost::graph_traits<Graph>::faces_size_type)
+
+#undef CGAL_DEF_GET_INITIALIZED_INDEX_MAP
 
   template<typename PolygonMesh, typename NamedParameters>
   class GetFaceNormalMap
@@ -229,10 +303,45 @@ namespace CGAL {
       > ::type  type;
   };
 
-  } // namespace Polygon_mesh_processing
 
-  namespace Point_set_processing_3
+  namespace internal {
+    BOOST_MPL_HAS_XXX_TRAIT_NAMED_DEF(Has_nested_type_iterator, iterator, false)
+  }
+
+  template<typename PointRange,
+           typename NamedParameters = Named_function_parameters<bool, internal_np::all_default_t>,
+           bool has_nested_iterator = internal::Has_nested_type_iterator<PointRange>::value>
+  class GetPointMap
   {
+    typedef typename std::iterator_traits<typename PointRange::iterator>::value_type Point;
+    typedef typename CGAL::Identity_property_map<Point> DefaultPMap;
+
+  public:
+    typedef typename internal_np::Lookup_named_param_def<
+    internal_np::point_t,
+    NamedParameters,
+    DefaultPMap
+    > ::type  type;
+
+    typedef typename internal_np::Lookup_named_param_def<
+    internal_np::point_t,
+    NamedParameters,
+    DefaultPMap
+    > ::type  const_type;
+  };
+
+  // to please compiler instantiating non valid overloads
+  template<typename PointRange, typename NamedParameters>
+  class GetPointMap<PointRange, NamedParameters, false>
+  {
+    struct Dummy_point{};
+  public:
+    typedef typename CGAL::Identity_property_map<Dummy_point> type;
+    typedef typename CGAL::Identity_property_map<Dummy_point> const_type;
+  };
+
+  namespace Point_set_processing_3 {
+
     template <typename ValueType>
     struct Fake_point_range
     {
@@ -256,41 +365,6 @@ namespace CGAL {
       }
     }
 
-    namespace internal{
-      BOOST_MPL_HAS_XXX_TRAIT_NAMED_DEF(Has_nested_type_iterator, iterator, false)
-    }
-
-    template<typename PointRange, typename NamedParameters,
-             bool has_nested_iterator=internal::Has_nested_type_iterator<PointRange>::value>
-    class GetPointMap
-    {
-      typedef typename std::iterator_traits<typename PointRange::iterator>::value_type Point;
-      typedef typename CGAL::Identity_property_map<Point> DefaultPMap;
-
-    public:
-      typedef typename internal_np::Lookup_named_param_def<
-      internal_np::point_t,
-      NamedParameters,
-      DefaultPMap
-      > ::type  type;
-
-      typedef typename internal_np::Lookup_named_param_def<
-      internal_np::point_t,
-      NamedParameters,
-      DefaultPMap
-      > ::type  const_type;
-    };
-
-    // to please compiler instantiating non valid overloads
-    template<typename PointRange, typename NamedParameters>
-    class GetPointMap<PointRange, NamedParameters, false>
-    {
-      struct Dummy_point{};
-    public:
-      typedef typename CGAL::Identity_property_map<Dummy_point> type;
-      typedef typename CGAL::Identity_property_map<Dummy_point> const_type;
-    };
-
     template<typename PointRange>
     class GetFT
     {
@@ -428,6 +502,18 @@ namespace CGAL {
         > ::type  type;
     };
 
+    template<typename PointRange, typename NamedParameters>
+    class GetAdjacencies
+    {
+    public:
+      typedef Emptyset_iterator Empty;
+      typedef typename internal_np::Lookup_named_param_def <
+        internal_np::adjacencies_t,
+        NamedParameters,
+        Empty//default
+        > ::type  type;
+    };
+
   } // namespace Point_set_processing_3
 
   template<typename NamedParameters, typename DefaultSolver>
@@ -478,6 +564,17 @@ namespace CGAL {
 #endif
     > ::type type;
   };
+
+  template<typename NamedParameters>
+  class GetImplementationTag
+  {
+  public:
+    typedef typename internal_np::Lookup_named_param_def <
+    internal_np::implementation_tag_t,
+    NamedParameters,
+    Alpha_expansion_boost_adjacency_list_tag
+    >::type type;
+  };
 } //namespace CGAL
 
 
diff -pruN 5.0.3-1/include/CGAL/boost/graph/parameters_interface.h 5.1-1/include/CGAL/boost/graph/parameters_interface.h
--- 5.0.3-1/include/CGAL/boost/graph/parameters_interface.h	2020-08-01 19:02:14.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/parameters_interface.h	2020-09-08 12:14:40.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/parameters_interface.h $
-// $Id: parameters_interface.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/parameters_interface.h $
+// $Id: parameters_interface.h 42b272d 2020-07-23T12:19:58+02:00 Jane Tournois
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -17,6 +17,8 @@ CGAL_add_named_parameter(face_index_t, f
 CGAL_add_named_parameter(vertex_index_t, vertex_index, vertex_index_map)
 CGAL_add_named_parameter(graph_visitor_t, graph_visitor, visitor)
 
+CGAL_add_named_parameter(point_t, point_map, point_map)
+
 CGAL_add_named_parameter(edge_is_constrained_t, edge_is_constrained, edge_is_constrained_map)
 CGAL_add_named_parameter(first_index_t, first_index, first_index)
 CGAL_add_named_parameter(number_of_iterations_t, number_of_iterations, number_of_iterations)
@@ -34,6 +36,8 @@ CGAL_add_named_parameter(face_to_face_ou
 CGAL_add_named_parameter(vertex_to_vertex_map_t, vertex_to_vertex_map, vertex_to_vertex_map)
 CGAL_add_named_parameter(halfedge_to_halfedge_map_t, halfedge_to_halfedge_map, halfedge_to_halfedge_map)
 CGAL_add_named_parameter(face_to_face_map_t, face_to_face_map, face_to_face_map)
+CGAL_add_named_parameter(implementation_tag_t, implementation_tag, implementation_tag)
+CGAL_add_named_parameter(prevent_unselection_t, prevent_unselection, prevent_unselection)
 
 // List of named parameters that we use in the package 'Mesh_3'
 CGAL_add_named_parameter(vertex_feature_degree_t, vertex_feature_degree, vertex_feature_degree_map)
@@ -82,6 +86,21 @@ CGAL_add_named_parameter(use_angle_smoot
 CGAL_add_named_parameter(use_area_smoothing_t, use_area_smoothing, use_area_smoothing)
 CGAL_add_named_parameter(use_Delaunay_flips_t, use_Delaunay_flips, use_Delaunay_flips)
 CGAL_add_named_parameter(do_project_t, do_project, do_project)
+CGAL_add_named_parameter(do_orientation_tests_t, do_orientation_tests, do_orientation_tests)
+CGAL_add_named_parameter(do_self_intersection_tests_t, do_self_intersection_tests, do_self_intersection_tests)
+CGAL_add_named_parameter(error_codes_t, error_codes, error_codes)
+CGAL_add_named_parameter(volume_inclusions_t, volume_inclusions, volume_inclusions)
+CGAL_add_named_parameter(face_cc_map_t, face_connected_component_map, face_connected_component_map)
+CGAL_add_named_parameter(ccid_to_vid_vector_t, connected_component_id_to_volume_id, connected_component_id_to_volume_id)
+CGAL_add_named_parameter(is_cc_outward_oriented_bs_t, is_cc_outward_oriented, is_cc_outward_oriented);
+CGAL_add_named_parameter(intersecting_volume_pairs_t, intersecting_volume_pairs_output_iterator, intersecting_volume_pairs_output_iterator);
+CGAL_add_named_parameter(i_used_as_a_predicate_t, i_used_as_a_predicate, i_used_as_a_predicate);
+CGAL_add_named_parameter(nesting_levels_t, nesting_levels, nesting_levels);
+CGAL_add_named_parameter(i_used_for_volume_orientation_t, i_used_for_volume_orientation, i_used_for_volume_orientation);
+CGAL_add_named_parameter(area_threshold_t, area_threshold, area_threshold)
+CGAL_add_named_parameter(halfedges_keeper_t, halfedges_keeper, halfedges_keeper)
+CGAL_add_named_parameter(volume_threshold_t, volume_threshold, volume_threshold)
+CGAL_add_named_parameter(dry_run_t, dry_run, dry_run)
 
 // List of named parameters that we use in the package 'Surface Mesh Simplification'
 CGAL_add_named_parameter(get_cost_policy_t, get_cost_policy, get_cost)
@@ -90,14 +109,14 @@ CGAL_add_named_parameter(get_placement_p
 //to be documented
 CGAL_add_named_parameter(face_normal_t, face_normal, face_normal_map)
 CGAL_add_named_parameter(random_seed_t, random_seed, random_seed)
-CGAL_add_named_parameter(tolerance_map_t, tolerance_map, tolerance_map)
+CGAL_add_named_parameter(do_lock_mesh_t, do_lock_mesh, do_lock_mesh)
+CGAL_add_named_parameter(do_simplify_border_t, do_simplify_border, do_simplify_border)
 
 //internal
 CGAL_add_named_parameter(weight_calculator_t, weight_calculator, weight_calculator)
 CGAL_add_named_parameter(use_bool_op_to_clip_surface_t, use_bool_op_to_clip_surface, use_bool_op_to_clip_surface)
 
 // List of named parameters used in the Point Set Processing package
-CGAL_add_named_parameter(point_t, point_map, point_map)
 CGAL_add_named_parameter(query_point_t, query_point_map, query_point_map)
 CGAL_add_named_parameter(normal_t, normal_map, normal_map)
 CGAL_add_named_parameter(diagonalize_traits_t, diagonalize_traits, diagonalize_traits)
@@ -119,6 +138,17 @@ CGAL_add_named_parameter(plane_index_t,
 CGAL_add_named_parameter(select_percentage_t, select_percentage, select_percentage)
 CGAL_add_named_parameter(require_uniform_sampling_t, require_uniform_sampling, require_uniform_sampling)
 CGAL_add_named_parameter(point_is_constrained_t, point_is_constrained, point_is_constrained_map)
+CGAL_add_named_parameter(maximum_number_of_faces_t, maximum_number_of_faces, maximum_number_of_faces)
+CGAL_add_named_parameter(transformation_t, transformation, transformation)
+CGAL_add_named_parameter(point_set_filters_t, point_set_filters, point_set_filters)
+CGAL_add_named_parameter(matcher_t, matcher, matcher)
+CGAL_add_named_parameter(outlier_filters_t, outlier_filters, outlier_filters)
+CGAL_add_named_parameter(error_minimizer_t, error_minimizer, error_minimizer)
+CGAL_add_named_parameter(transformation_checkers_t, transformation_checkers, transformation_checkers)
+CGAL_add_named_parameter(inspector_t, inspector, inspector)
+CGAL_add_named_parameter(logger_t, logger, logger)
+CGAL_add_named_parameter(pointmatcher_config_t, pointmatcher_config, pointmatcher_config)
+CGAL_add_named_parameter(adjacencies_t, adjacencies, adjacencies)
 
 // List of named parameters used in Surface_mesh_approximation package
 CGAL_add_named_parameter(verbose_level_t, verbose_level, verbose_level)
@@ -127,6 +157,9 @@ CGAL_add_named_parameter(max_number_of_p
 CGAL_add_named_parameter(min_error_drop_t, min_error_drop, min_error_drop)
 CGAL_add_named_parameter(number_of_relaxations_t, number_of_relaxations, number_of_relaxations)
 
+// List of named parameters used in Optimal_bounding_box package
+CGAL_add_named_parameter(use_convex_hull_t, use_convex_hull, use_convex_hull)
+
 // meshing parameters
 CGAL_add_named_parameter(subdivision_ratio_t, subdivision_ratio, subdivision_ratio)
 CGAL_add_named_parameter(relative_to_chord_t, relative_to_chord, relative_to_chord)
@@ -134,9 +167,21 @@ CGAL_add_named_parameter(with_dihedral_a
 CGAL_add_named_parameter(optimize_anchor_location_t, optimize_anchor_location, optimize_anchor_location)
 CGAL_add_named_parameter(pca_plane_t, pca_plane, pca_plane)
 
+// tetrahedral remeshing parameters
+CGAL_add_named_parameter(remesh_boundaries_t, remesh_boundaries, remesh_boundaries)
+CGAL_add_named_parameter(cell_selector_t, cell_selector, cell_selector)
+CGAL_add_named_parameter(facet_is_constrained_t, facet_is_constrained, facet_is_constrained_map)
+CGAL_add_named_parameter(remeshing_visitor_t, remeshing_visitor, remeshing_visitor)
+CGAL_add_named_parameter(smooth_constrained_edges_t, smooth_constrained_edges, smooth_constrained_edges)
+
 // output parameters
 CGAL_add_named_parameter(face_proxy_map_t, face_proxy_map, face_proxy_map)
 CGAL_add_named_parameter(proxies_t, proxies, proxies)
 CGAL_add_named_parameter(anchors_t, anchors, anchors)
 CGAL_add_named_parameter(triangles_t, triangles, triangles)
 
+CGAL_add_named_parameter(number_of_samples_t, number_of_samples, number_of_samples)
+CGAL_add_named_parameter(accuracy_t, accuracy, accuracy)
+CGAL_add_named_parameter(maximum_running_time_t, maximum_running_time, maximum_running_time)
+CGAL_add_named_parameter(overlap_t, overlap, overlap)
+CGAL_add_named_parameter(maximum_normal_deviation_t, maximum_normal_deviation, maximum_normal_deviation)
diff -pruN 5.0.3-1/include/CGAL/boost/graph/partition.h 5.1-1/include/CGAL/boost/graph/partition.h
--- 5.0.3-1/include/CGAL/boost/graph/partition.h	2020-08-01 19:02:14.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/partition.h	2020-09-08 12:14:40.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/partition.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/partition.h $
 // $Id: partition.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/properties_Constrained_Delaunay_triangulation_2.h 5.1-1/include/CGAL/boost/graph/properties_Constrained_Delaunay_triangulation_2.h
--- 5.0.3-1/include/CGAL/boost/graph/properties_Constrained_Delaunay_triangulation_2.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/properties_Constrained_Delaunay_triangulation_2.h	2020-09-08 12:15:51.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/boost/graph/properties_Constrained_Delaunay_triangulation_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/boost/graph/properties_Constrained_Delaunay_triangulation_2.h $
 // $Id: properties_Constrained_Delaunay_triangulation_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/properties_Constrained_triangulation_2.h 5.1-1/include/CGAL/boost/graph/properties_Constrained_triangulation_2.h
--- 5.0.3-1/include/CGAL/boost/graph/properties_Constrained_triangulation_2.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/properties_Constrained_triangulation_2.h	2020-09-08 12:15:51.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/boost/graph/properties_Constrained_triangulation_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/boost/graph/properties_Constrained_triangulation_2.h $
 // $Id: properties_Constrained_triangulation_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/properties_Constrained_triangulation_plus_2.h 5.1-1/include/CGAL/boost/graph/properties_Constrained_triangulation_plus_2.h
--- 5.0.3-1/include/CGAL/boost/graph/properties_Constrained_triangulation_plus_2.h	2020-08-01 19:02:59.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/properties_Constrained_triangulation_plus_2.h	2020-09-08 12:15:51.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/boost/graph/properties_Constrained_triangulation_plus_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/boost/graph/properties_Constrained_triangulation_plus_2.h $
 // $Id: properties_Constrained_triangulation_plus_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/properties_Delaunay_triangulation_2.h 5.1-1/include/CGAL/boost/graph/properties_Delaunay_triangulation_2.h
--- 5.0.3-1/include/CGAL/boost/graph/properties_Delaunay_triangulation_2.h	2020-08-01 19:02:59.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/properties_Delaunay_triangulation_2.h	2020-09-08 12:15:51.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/boost/graph/properties_Delaunay_triangulation_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/boost/graph/properties_Delaunay_triangulation_2.h $
 // $Id: properties_Delaunay_triangulation_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/properties.h 5.1-1/include/CGAL/boost/graph/properties.h
--- 5.0.3-1/include/CGAL/boost/graph/properties.h	2020-08-01 19:02:14.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/properties.h	2020-09-08 12:14:40.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/properties.h $
-// $Id: properties.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/properties.h $
+// $Id: properties.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -14,14 +14,15 @@
 #define CGAL_BOOST_GRAPH_BGL_PROPERTIES_H
 
 #include <CGAL/property_map.h>
+#include <CGAL/Dynamic_property_map.h>
+#include <CGAL/basic.h>
+
 #include <boost/graph/properties.hpp>
 #include <boost/graph/graph_traits.hpp>
-#include <boost/type_traits/is_const.hpp>
-#include <boost/type_traits/remove_reference.hpp>
-#include <CGAL/Dynamic_property_map.h>
 
-#include <CGAL/basic.h>
 #include <string>
+#include <vector>
+#include <type_traits>
 
 namespace CGAL {
 
@@ -69,12 +70,16 @@ BOOST_INSTALL_PROPERTY(face, external_in
 namespace CGAL {
 using boost::vertex_point_t;
 using boost::vertex_point;
+using boost::vertex_index_t;
+using boost::vertex_index;
 using boost::vertex_external_index_t;
 using boost::vertex_external_index;
 using boost::halfedge_index_t;
 using boost::halfedge_index;
 using boost::halfedge_external_index_t;
 using boost::halfedge_external_index;
+using boost::edge_index_t;
+using boost::edge_index;
 using boost::edge_external_index_t;
 using boost::edge_external_index;
 using boost::face_index_t;
@@ -83,128 +88,7 @@ using boost::face_external_index_t;
 using boost::face_external_index;
 } // CGAL
 
-namespace CGAL{
-namespace helpers {
-
-// matches read-write property maps
-template <class PolygonMesh, class FaceIndexMap, class Tag>
-void init_face_indices(PolygonMesh& pm,
-                       FaceIndexMap& fid,
-                       boost::read_write_property_map_tag,
-                       Tag)
-{
-  typename boost::property_traits<FaceIndexMap>::value_type i = 0;
-  for(typename boost::graph_traits<PolygonMesh>::face_descriptor fd :
-                faces(pm))
-  {
-    put(fid, fd, i);
-    ++i;
-  }
-}
-template <class PolygonMesh, class VertexIndexMap, class Tag>
-void init_vertex_indices(PolygonMesh& pm,
-                         VertexIndexMap& vid,
-                         boost::read_write_property_map_tag,
-                         Tag)
-{
-  typename boost::property_traits<VertexIndexMap>::value_type i = 0;
-  for(typename boost::graph_traits<PolygonMesh>::vertex_descriptor vd :
-                vertices(pm))
-  {
-    put(vid, vd, i);
-    ++i;
-  }
-}
-template <class PolygonMesh, class HalfedgeIndexMap, class Tag>
-void init_halfedge_indices(PolygonMesh& pm,
-                           HalfedgeIndexMap& hid,
-                           boost::read_write_property_map_tag,
-                           Tag)
-{
-  typename boost::property_traits<HalfedgeIndexMap>::value_type i = 0;
-  for(typename boost::graph_traits<PolygonMesh>::halfedge_descriptor hd :
-                halfedges(pm))
-  {
-    put(hid, hd, i);
-    ++i;
-  }
-}
-
-// matches mutable Lvalue property maps
-template <class PolygonMesh, class FaceIndexMap>
-void init_face_indices(PolygonMesh& pm,
-                       FaceIndexMap& fid,
-                       boost::lvalue_property_map_tag,
-                       boost::false_type)
-{
-  init_face_indices(pm, fid,
-    boost::read_write_property_map_tag(), boost::false_type());
-}
-template <class PolygonMesh, class VertexIndexMap>
-void init_vertex_indices(PolygonMesh& pm,
-                         VertexIndexMap& vid,
-                         boost::lvalue_property_map_tag,
-                         boost::false_type)
-{
-  init_vertex_indices(pm, vid,
-    boost::read_write_property_map_tag(), boost::false_type());
-}
-template <class PolygonMesh, class HalfedgeIndexMap>
-void init_halfedge_indices(PolygonMesh& pm,
-                         HalfedgeIndexMap& hid,
-                         boost::lvalue_property_map_tag,
-                         boost::false_type)
-{
-  init_halfedge_indices(pm, hid,
-    boost::read_write_property_map_tag(), boost::false_type());
-}
-
-// matches all other types of property map
-template <class PolygonMesh, class FaceIndexMap, class MapTag, class Tag>
-void init_face_indices(PolygonMesh&, FaceIndexMap, MapTag, Tag)
-{}
-template <class PolygonMesh, class VertexIndexMap, class MapTag, class Tag>
-void init_vertex_indices(PolygonMesh&, VertexIndexMap, MapTag, Tag)
-{}
-template <class PolygonMesh, class HalfedgeIndexMap, class MapTag, class Tag>
-void init_halfedge_indices(PolygonMesh&, HalfedgeIndexMap, MapTag, Tag)
-{}
-
-template <class PolygonMesh, class FaceIndexMap>
-void init_face_indices(PolygonMesh& pm, FaceIndexMap fid)
-{
-  init_face_indices(pm, fid,
-                    typename boost::property_traits<FaceIndexMap>::category(),
-                    typename boost::is_const<
-                      typename boost::remove_reference<
-                        typename boost::property_traits<FaceIndexMap>::reference
-                            >::type >::type() );
-}
-
-template <class PolygonMesh, class VertexIndexMap>
-void init_vertex_indices(PolygonMesh& pm, VertexIndexMap vid)
-{
-  init_vertex_indices(pm, vid,
-                      typename boost::property_traits<VertexIndexMap>::category(),
-                      typename boost::is_const<
-                        typename boost::remove_reference<
-                          typename boost::property_traits<VertexIndexMap>::reference
-                            >::type >::type() );
-}
-
-template <class PolygonMesh, class HalfedgeIndexMap>
-void init_halfedge_indices(PolygonMesh& pm, HalfedgeIndexMap hid)
-{
-  init_halfedge_indices(pm, hid,
-                        typename boost::property_traits<HalfedgeIndexMap>::category(),
-                        typename boost::is_const<
-                          typename boost::remove_reference<
-                            typename boost::property_traits<HalfedgeIndexMap>::reference
-                              >::type >::type() );
-}
-
-} //namespace helpers
-
+namespace CGAL {
 namespace internal {
 
   template<typename Polyhedron, typename Handle>
@@ -232,8 +116,8 @@ struct Edge_index_accessor
 };
 
 template<typename Handle, typename ValueType, typename Reference,
-         bool is_const = boost::is_const<
-                           typename boost::remove_reference<Reference>::type >::value>
+         bool is_const = std::is_const<
+                           typename std::remove_reference<Reference>::type >::value>
 struct Point_accessor
   : boost::put_get_helper< Reference, Point_accessor<Handle, ValueType, Reference> >
 {
@@ -265,6 +149,32 @@ struct Point_accessor<Handle, ValueType,
   reference operator[](Handle h) const { return h->point(); }
 };
 
+// this one is basically 'readable_property_map_tag'
+template <typename PropertyMap,
+          typename PropertyMapCategory = typename boost::property_traits<PropertyMap>::category>
+struct Is_writable_property_map : CGAL::Tag_false { };
+
+template <typename PropertyMap>
+struct Is_writable_property_map<PropertyMap, boost::writable_property_map_tag> : CGAL::Tag_true { };
+
+template <typename PropertyMap>
+struct Is_writable_property_map<PropertyMap, boost::read_write_property_map_tag> : CGAL::Tag_true { };
+
+// 'lvalue_pmap_tag' is annoying, because the property map is allowed to be non-mutable,
+// but boost::lvalue_property_map_tag is defined as:
+//   struct lvalue_property_map_tag : public read_write_property_map_tag
+// so we can't just check that 'writable_property_map_tag' is a base of the the lvalue tag.
+//
+// This checks if the reference is non-const, which is not completely correct: map[key] returning
+// a non-const reference doesn't mean that 'put(map, key, val)' exists, which is what a writable
+// property map must define.
+template <typename PropertyMap>
+struct Is_writable_property_map<PropertyMap, boost::lvalue_property_map_tag>
+  : boost::mpl::if_c<std::is_const<typename std::remove_reference<
+                       typename boost::property_traits<PropertyMap>::reference>::type>::value,
+                     CGAL::Tag_false, CGAL::Tag_true>::type
+{ };
+
 } // namespace internal
 
 // Needed by PMP::detec_features and Mesh_3
@@ -275,12 +185,12 @@ enum vertex_time_stamp_t        { vertex
 enum halfedge_time_stamp_t      { halfedge_time_stamp};
 enum face_time_stamp_t          { face_time_stamp};
 
-template <typename ID>
+template<typename ID>
 struct vertex_incident_patches_t {
   typedef ID type;
 };
 
-template <typename ID>
+template<typename ID>
 struct face_patch_id_t {
   typedef ID type;
 };
diff -pruN 5.0.3-1/include/CGAL/boost/graph/properties_Linear_cell_complex_for_combinatorial_map.h 5.1-1/include/CGAL/boost/graph/properties_Linear_cell_complex_for_combinatorial_map.h
--- 5.0.3-1/include/CGAL/boost/graph/properties_Linear_cell_complex_for_combinatorial_map.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/properties_Linear_cell_complex_for_combinatorial_map.h	2020-09-08 12:15:06.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Linear_cell_complex/include/CGAL/boost/graph/properties_Linear_cell_complex_for_combinatorial_map.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Linear_cell_complex/include/CGAL/boost/graph/properties_Linear_cell_complex_for_combinatorial_map.h $
 // $Id: properties_Linear_cell_complex_for_combinatorial_map.h b4c0e6e 2020-03-27T19:12:07+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/properties_OpenMesh.h 5.1-1/include/CGAL/boost/graph/properties_OpenMesh.h
--- 5.0.3-1/include/CGAL/boost/graph/properties_OpenMesh.h	2020-08-01 19:02:14.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/properties_OpenMesh.h	2020-09-08 12:14:40.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/properties_OpenMesh.h $
-// $Id: properties_OpenMesh.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/properties_OpenMesh.h $
+// $Id: properties_OpenMesh.h bd3f47c 2020-03-24T11:02:57+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -423,7 +423,11 @@ template<typename K>
 void
 put(boost::vertex_point_t p, OPEN_MESH_CLASS& g,
     typename boost::graph_traits< OPEN_MESH_CLASS >::vertex_descriptor vd,
+#if defined(CGAL_USE_OM_POINTS)
     const typename K::Point& point)
+#else
+    const CGAL::Exact_predicates_inexact_constructions_kernel::Point_3& point)
+#endif
 {
   put(get(p,g), vd, point);
 }
diff -pruN 5.0.3-1/include/CGAL/boost/graph/properties_Polyhedron_3_features.h 5.1-1/include/CGAL/boost/graph/properties_Polyhedron_3_features.h
--- 5.0.3-1/include/CGAL/boost/graph/properties_Polyhedron_3_features.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/properties_Polyhedron_3_features.h	2020-09-08 12:15:28.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron/include/CGAL/boost/graph/properties_Polyhedron_3_features.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron/include/CGAL/boost/graph/properties_Polyhedron_3_features.h $
 // $Id: properties_Polyhedron_3_features.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/properties_Polyhedron_3.h 5.1-1/include/CGAL/boost/graph/properties_Polyhedron_3.h
--- 5.0.3-1/include/CGAL/boost/graph/properties_Polyhedron_3.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/properties_Polyhedron_3.h	2020-09-08 12:15:28.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron/include/CGAL/boost/graph/properties_Polyhedron_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron/include/CGAL/boost/graph/properties_Polyhedron_3.h $
 // $Id: properties_Polyhedron_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/properties_Polyhedron_3_time_stamp.h 5.1-1/include/CGAL/boost/graph/properties_Polyhedron_3_time_stamp.h
--- 5.0.3-1/include/CGAL/boost/graph/properties_Polyhedron_3_time_stamp.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/properties_Polyhedron_3_time_stamp.h	2020-09-08 12:15:28.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron/include/CGAL/boost/graph/properties_Polyhedron_3_time_stamp.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron/include/CGAL/boost/graph/properties_Polyhedron_3_time_stamp.h $
 // $Id: properties_Polyhedron_3_time_stamp.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/properties_PolyMesh_ArrayKernelT.h 5.1-1/include/CGAL/boost/graph/properties_PolyMesh_ArrayKernelT.h
--- 5.0.3-1/include/CGAL/boost/graph/properties_PolyMesh_ArrayKernelT.h	2020-08-01 19:02:14.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/properties_PolyMesh_ArrayKernelT.h	2020-09-08 12:14:40.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/properties_PolyMesh_ArrayKernelT.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/properties_PolyMesh_ArrayKernelT.h $
 // $Id: properties_PolyMesh_ArrayKernelT.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/properties_Regular_triangulation_2.h 5.1-1/include/CGAL/boost/graph/properties_Regular_triangulation_2.h
--- 5.0.3-1/include/CGAL/boost/graph/properties_Regular_triangulation_2.h	2020-08-01 19:02:59.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/properties_Regular_triangulation_2.h	2020-09-08 12:15:51.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/boost/graph/properties_Regular_triangulation_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/boost/graph/properties_Regular_triangulation_2.h $
 // $Id: properties_Regular_triangulation_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/properties_Seam_mesh.h 5.1-1/include/CGAL/boost/graph/properties_Seam_mesh.h
--- 5.0.3-1/include/CGAL/boost/graph/properties_Seam_mesh.h	2020-08-01 19:02:14.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/properties_Seam_mesh.h	2020-09-08 12:14:40.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/properties_Seam_mesh.h $
-// $Id: properties_Seam_mesh.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/properties_Seam_mesh.h $
+// $Id: properties_Seam_mesh.h c999ce1 2020-02-19T10:07:53+01:00 Maxime Gimeno
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -112,6 +112,39 @@ struct property_map<CGAL::Seam_mesh<TM,
   typedef CGAL::Seam_mesh_point_map<TM, SEM, SVM>        type;
   typedef type                                           const_type;
 };
+
+template <class TM, class SEM, class SVM, typename T>
+struct property_map<CGAL::Seam_mesh<TM, SEM, SVM>, CGAL::dynamic_vertex_property_t<T> >
+{
+  typedef typename boost::graph_traits<CGAL::Seam_mesh<TM, SEM, SVM> >::vertex_descriptor vertex_descriptor;
+  typedef CGAL::internal::Dynamic_property_map<vertex_descriptor,T> type;
+  typedef type const_type;
+};
+
+template <class TM, class SEM, class SVM, typename T>
+struct property_map<CGAL::Seam_mesh<TM, SEM, SVM>, CGAL::dynamic_halfedge_property_t<T> >
+{
+  typedef typename boost::graph_traits<CGAL::Seam_mesh<TM, SEM, SVM> >::halfedge_descriptor halfedge_descriptor;
+  typedef CGAL::internal::Dynamic_property_map<halfedge_descriptor,T> type;
+  typedef type const_type;
+};
+
+
+template <class TM, class SEM, class SVM, typename T>
+struct property_map<CGAL::Seam_mesh<TM, SEM, SVM>, CGAL::dynamic_edge_property_t<T> >
+{
+  typedef typename boost::graph_traits<CGAL::Seam_mesh<TM, SEM, SVM> >::edge_descriptor edge_descriptor;
+  typedef CGAL::internal::Dynamic_property_map<edge_descriptor,T> type;
+  typedef type const_type;
+};
+
+template <class TM, class SEM, class SVM, typename T>
+struct property_map<CGAL::Seam_mesh<TM, SEM, SVM>, CGAL::dynamic_face_property_t<T> >
+{
+  typedef typename boost::graph_traits<CGAL::Seam_mesh<TM, SEM, SVM> >::face_descriptor face_descriptor;
+  typedef CGAL::internal::Dynamic_property_map<face_descriptor,T> type;
+  typedef type const_type;
+};
 } // namespace boost
 
 namespace CGAL {
diff -pruN 5.0.3-1/include/CGAL/boost/graph/properties_Surface_mesh_features.h 5.1-1/include/CGAL/boost/graph/properties_Surface_mesh_features.h
--- 5.0.3-1/include/CGAL/boost/graph/properties_Surface_mesh_features.h	2020-08-01 19:02:54.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/properties_Surface_mesh_features.h	2020-09-08 12:15:43.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh/include/CGAL/boost/graph/properties_Surface_mesh_features.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh/include/CGAL/boost/graph/properties_Surface_mesh_features.h $
 // $Id: properties_Surface_mesh_features.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/properties_Surface_mesh.h 5.1-1/include/CGAL/boost/graph/properties_Surface_mesh.h
--- 5.0.3-1/include/CGAL/boost/graph/properties_Surface_mesh.h	2020-08-01 19:02:54.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/properties_Surface_mesh.h	2020-09-08 12:15:43.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh/include/CGAL/boost/graph/properties_Surface_mesh.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh/include/CGAL/boost/graph/properties_Surface_mesh.h $
 // $Id: properties_Surface_mesh.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/properties_Surface_mesh_time_stamp.h 5.1-1/include/CGAL/boost/graph/properties_Surface_mesh_time_stamp.h
--- 5.0.3-1/include/CGAL/boost/graph/properties_Surface_mesh_time_stamp.h	2020-08-01 19:02:54.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/properties_Surface_mesh_time_stamp.h	2020-09-08 12:15:43.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh/include/CGAL/boost/graph/properties_Surface_mesh_time_stamp.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh/include/CGAL/boost/graph/properties_Surface_mesh_time_stamp.h $
 // $Id: properties_Surface_mesh_time_stamp.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/properties_Triangulation_2.h 5.1-1/include/CGAL/boost/graph/properties_Triangulation_2.h
--- 5.0.3-1/include/CGAL/boost/graph/properties_Triangulation_2.h	2020-08-01 19:02:59.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/properties_Triangulation_2.h	2020-09-08 12:15:51.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/boost/graph/properties_Triangulation_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/boost/graph/properties_Triangulation_2.h $
 // $Id: properties_Triangulation_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/properties_Triangulation_data_structure_2.h 5.1-1/include/CGAL/boost/graph/properties_Triangulation_data_structure_2.h
--- 5.0.3-1/include/CGAL/boost/graph/properties_Triangulation_data_structure_2.h	2020-08-01 19:02:59.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/properties_Triangulation_data_structure_2.h	2020-09-08 12:15:51.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/boost/graph/properties_Triangulation_data_structure_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/boost/graph/properties_Triangulation_data_structure_2.h $
 // $Id: properties_Triangulation_data_structure_2.h 0903946 2020-03-17T17:53:21+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/properties_Triangulation_hierarchy_2.h 5.1-1/include/CGAL/boost/graph/properties_Triangulation_hierarchy_2.h
--- 5.0.3-1/include/CGAL/boost/graph/properties_Triangulation_hierarchy_2.h	2020-08-01 19:02:59.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/properties_Triangulation_hierarchy_2.h	2020-09-08 12:15:51.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/boost/graph/properties_Triangulation_hierarchy_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/boost/graph/properties_Triangulation_hierarchy_2.h $
 // $Id: properties_Triangulation_hierarchy_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/properties_TriMesh_ArrayKernelT.h 5.1-1/include/CGAL/boost/graph/properties_TriMesh_ArrayKernelT.h
--- 5.0.3-1/include/CGAL/boost/graph/properties_TriMesh_ArrayKernelT.h	2020-08-01 19:02:15.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/properties_TriMesh_ArrayKernelT.h	2020-09-08 12:14:40.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/properties_TriMesh_ArrayKernelT.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/properties_TriMesh_ArrayKernelT.h $
 // $Id: properties_TriMesh_ArrayKernelT.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/property_maps.h 5.1-1/include/CGAL/boost/graph/property_maps.h
--- 5.0.3-1/include/CGAL/boost/graph/property_maps.h	2020-08-01 19:02:15.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/property_maps.h	2020-09-08 12:14:40.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/property_maps.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/property_maps.h $
 // $Id: property_maps.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/graph/Seam_mesh.h 5.1-1/include/CGAL/boost/graph/Seam_mesh.h
--- 5.0.3-1/include/CGAL/boost/graph/Seam_mesh.h	2020-08-01 19:02:14.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/Seam_mesh.h	2020-09-08 12:14:40.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/Seam_mesh.h $
-// $Id: Seam_mesh.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/Seam_mesh.h $
+// $Id: Seam_mesh.h ebd65c9 2020-07-20T11:47:30+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -124,8 +124,8 @@ public:
   /// The type for the objects used to identify halfedges in the underlying mesh.
   typedef typename boost::graph_traits<TM>::halfedge_descriptor   TM_halfedge_descriptor;
 
- /// The type for the iterators that traverse through the complete halfedge set of the underlying mesh.
- typedef typename boost::graph_traits<TM>::halfedge_iterator     TM_halfedge_iterator;
+  /// The type for the iterators that traverse through the complete halfedge set of the underlying mesh.
+  typedef typename boost::graph_traits<TM>::halfedge_iterator     TM_halfedge_iterator;
 
   /// The type for the objects used to identify edges in the underlying mesh.
   typedef typename boost::graph_traits<TM>::edge_descriptor       TM_edge_descriptor;
@@ -168,7 +168,7 @@ private:
   mutable vertices_size_type number_of_vertices;
 
 public:
-  /// Returns the underlying mesh.
+  /// returns the underlying mesh.
   const TM& mesh() const
   {
     return tm;
@@ -439,6 +439,11 @@ public:
     {
       return ! (e1 == e2);
     }
+
+    friend std::size_t hash_value(const edge_descriptor& ed)
+    {
+      return hash_value((std::min)(ed.hd, ed.mesh_->opposite(ed.hd)));
+    }
   };
 
 #ifndef DOXYGEN_RUNNING
@@ -538,31 +543,31 @@ public:
   /// \name Seam query functions
   /// @{
 
-  /// Returns `true` if the vertex is on the seam.
+  /// returns `true` if the vertex is on the seam.
   bool has_on_seam(TM_vertex_descriptor vd) const
   {
     return get(svm, vd);
   }
 
-  /// Returns `true` if the edge is on the seam.
+  /// returns `true` if the edge is on the seam.
   bool has_on_seam(TM_edge_descriptor ed) const
   {
     return get(sem, ed);
   }
 
-  /// Returns `true` if the halfedge is on the seam.
+  /// returns `true` if the halfedge is on the seam.
   bool has_on_seam(TM_halfedge_descriptor tmhd) const
   {
     return get(sem, CGAL::edge(tmhd, tm));
   }
 
-  /// Returns `true` if the halfedge is on the seam.
+  /// returns `true` if the halfedge is on the seam.
   bool has_on_seam(const halfedge_descriptor& hd) const
   {
     return has_on_seam(CGAL::edge(hd, tm));
   }
 
-  /// Return the number of seam edges in the seam mesh.
+  /// returns the number of seam edges in the seam mesh.
   edges_size_type number_of_seam_edges() const
   {
     return number_of_seams;
@@ -596,7 +601,7 @@ public:
   }
   /// @endcond
 
-  /// Returns the iterator range of the vertices of the mesh.
+  /// returns the iterator range of the vertices of the mesh.
   Iterator_range<vertex_iterator> vertices() const
   {
     Iterator_range<TM_halfedge_iterator> ir = CGAL::halfedges(tm);
@@ -622,7 +627,7 @@ public:
   }
   /// @endcond
 
-  /// Returns the iterator range of the halfedges of the mesh.
+  /// returns the iterator range of the halfedges of the mesh.
   Iterator_range<halfedge_iterator> halfedges() const
   {
     Iterator_range<TM_halfedge_iterator> ir = CGAL::halfedges(tm);
@@ -647,7 +652,7 @@ public:
   }
   /// @endcond
 
-  /// Returns the iterator range of the edges of the mesh.
+  /// returns the iterator range of the edges of the mesh.
   Iterator_range<edge_iterator> edges() const
   {
     Iterator_range<TM_halfedge_iterator> ir = CGAL::halfedges(tm);
@@ -670,7 +675,7 @@ public:
   }
   /// @endcond
 
-  /// Returns the iterator range of the faces of the mesh.
+  /// returns the iterator range of the faces of the mesh.
   Iterator_range<face_iterator> faces() const
   {
     return CGAL::faces(tm);
@@ -681,7 +686,7 @@ public:
   /// \name Memory Management
   /// @{
 
-  /// Returns the number of vertices in the seam mesh.
+  /// returns the number of vertices in the seam mesh.
   vertices_size_type num_vertices() const
   {
     if(number_of_vertices == static_cast<vertices_size_type>(-1)) {
@@ -691,19 +696,19 @@ public:
     return number_of_vertices;
   }
 
-  /// Returns the number of halfedges in the seam mesh.
+  /// returns the number of halfedges in the seam mesh.
   halfedges_size_type num_halfedges() const
   {
     return CGAL::num_halfedges(tm) + 2 * number_of_seams;
   }
 
-  /// Returns the number of edges in the seam mesh.
+  /// returns the number of edges in the seam mesh.
   halfedges_size_type num_edges() const
   {
     return CGAL::num_edges(tm) + number_of_seams;
   }
 
-  /// Returns the number of faces in the seam mesh.
+  /// returns the number of faces in the seam mesh.
   faces_size_type num_faces() const
   {
     return CGAL::num_faces(tm);
@@ -714,7 +719,7 @@ public:
   /// \name Degree Functions
   /// @{
 
-  /// Returns the number of incident halfedges of vertex `v`.
+  /// returns the number of incident halfedges of vertex `v`.
   degree_size_type degree(vertex_descriptor v) const
   {
     degree_size_type count(0);
@@ -738,13 +743,13 @@ public:
 #ifndef DOXYGEN_RUNNING
   ///@{
 
-  /// Returns the edge that contains halfedge `h` as one of its two halfedges.
+  /// returns the edge that contains halfedge `h` as one of its two halfedges.
   edge_descriptor edge(halfedge_descriptor h) const
   {
     return edge_descriptor(h,this);
   }
 
-  /// Returns the halfedge corresponding to the edge `e`.
+  /// returns the halfedge corresponding to the edge `e`.
   halfedge_descriptor halfedge(edge_descriptor e) const
   {
     return e.hd;
@@ -753,7 +758,7 @@ public:
 
   ///@{
 
-  /// Returns an incoming halfedge of vertex `v`.
+  /// returns an incoming halfedge of vertex `v`.
   /// If `v` is a seam vertex, this will be the halfedge whose target is `v` and
   /// whose opposite is a virtual border halfedge.
   /// Otherwise, the rules of the underlying mesh are followed.
@@ -764,7 +769,7 @@ public:
     return halfedge_descriptor(h, false /*not on seam*/);
   }
 
-  /// Finds a halfedge between two vertices. Returns a default constructed
+  /// finds a halfedge between two vertices. Returns a default constructed
   /// `halfedge_descriptor`, if  `source` and  `target` are not connected.
   std::pair<halfedge_descriptor, bool> halfedge(vertex_descriptor u,
                                                 vertex_descriptor v) const
@@ -795,7 +800,7 @@ public:
     return std::make_pair(halfedge_descriptor(), false/*invalid*/);
   }
 
-  /// Finds an edge between two vertices. Returns a default constructed
+  /// finds an edge between two vertices. Returns a default constructed
   /// `edge`, if  `source` and  `target` are not connected.
   std::pair<edge_descriptor, bool> edge(vertex_descriptor u, vertex_descriptor v) const
   {
@@ -810,7 +815,7 @@ public:
     return halfedge_descriptor(hd, false/*not on seam*/);
   }
 
-  /// Returns the face incident to halfedge `h`.
+  /// returns the face incident to halfedge `h`.
   face_descriptor face(halfedge_descriptor h) const
   {
     if(h.seam)
@@ -820,7 +825,7 @@ public:
   }
 
 public:
-  /// Returns the next halfedge within the incident face.
+  /// returns the next halfedge within the incident face.
   halfedge_descriptor next(const halfedge_descriptor& hd) const
   {
     if((!hd.seam) && (!is_border(hd.tmhd, tm)))
@@ -835,7 +840,7 @@ public:
                                !is_border(CGAL::opposite(*hatc, tm), tm));
   }
 
-  /// Returns the previous halfedge within the incident face.
+  /// returns the previous halfedge within the incident face.
   halfedge_descriptor prev(const halfedge_descriptor& hd) const
   {
     if((!hd.seam) && (!is_border(hd.tmhd, tm)))
@@ -850,7 +855,7 @@ public:
                                !is_border(CGAL::opposite(*hatc, tm), tm));
   }
 
-  /// Returns the opposite halfedge of `hd`.
+  /// returns the opposite halfedge of `hd`.
   halfedge_descriptor opposite(const halfedge_descriptor& hd) const
   {
     if(!hd.seam)
@@ -859,7 +864,7 @@ public:
     return halfedge_descriptor(CGAL::opposite(hd.tmhd, tm), false /*not on seam*/);
   }
 
-  /// Returns the vertex the halfedge `h` emanates from.
+  /// returns the vertex the halfedge `h` emanates from.
   vertex_descriptor target(halfedge_descriptor hd) const
   {
     TM_halfedge_descriptor tmhd(hd);
@@ -879,7 +884,7 @@ public:
     return vertex_descriptor(halfedge_descriptor(tmhd));
   }
 
-  /// Returns the vertex the halfedge `h` emanates from.
+  /// returns the vertex the halfedge `h` emanates from.
   vertex_descriptor source(const halfedge_descriptor& hd) const
   {
     return target(opposite(hd));
@@ -917,7 +922,7 @@ public:
   /// \name Seam selection
   /// @{
 
-  /// Mark the edge of the underlying mesh that has extremities the vertices
+  /// marks the edge of the underlying mesh that has extremities the vertices
   /// `tm_vd_s` and `tm_vd_s` as a seam edge.
   ///
   /// \return whether the edge was successfully marked or not.
@@ -958,7 +963,7 @@ public:
     return true;
   }
 
-  /// Create new seams.
+  /// creates new seams.
   ///
   /// The edges to be marked as seams are described by the range [first, last) of
   /// vertices of the underlying mesh. Each edge to be marked is described
@@ -993,7 +998,7 @@ public:
     return tmhd;
   }
 
-  /// Create new seams.
+  /// creates new seams.
   ///
   /// A seam edge is described by a pair of integers. The integer index
   /// of a vertex of the underlying mesh is given by its position
@@ -1019,7 +1024,7 @@ public:
     return add_seams(seam_vertices.begin(), seam_vertices.end());
   }
 
-  /// Create new seams.
+  /// creates new seams.
   ///
   /// A seam edge is described by a pair of integers. The integer
   /// index of a vertex of the underlying mesh is defined as its position when
@@ -1036,7 +1041,7 @@ public:
     return add_seams(in, tm_vds);
   }
 
-  /// Create new seams.
+  /// creates new seams.
   ///
   /// A seam edge is described by a pair of integers. The integer index
   /// of a vertex of the underlying mesh is given by its position
@@ -1080,7 +1085,7 @@ public:
     return add_seams(in, tm_vds);
   }
 
-  /// Create new seams.
+  /// creates new seams.
   ///
   /// A seam edge is described by a pair of integers. The integer
   /// index of a vertex of the underlying mesh is defined as its position when
@@ -1099,7 +1104,7 @@ public:
 
   /// @}
 
-  /// Constructs a seam mesh for a triangle mesh and an edge and vertex property map
+  /// constructs a seam mesh for a triangle mesh and an edge and vertex property map
   ///
   /// \param tm the underlying mesh
   /// \param sem the edge property map with value `true` for seam edges
diff -pruN 5.0.3-1/include/CGAL/boost/graph/selection.h 5.1-1/include/CGAL/boost/graph/selection.h
--- 5.0.3-1/include/CGAL/boost/graph/selection.h	2020-08-01 19:02:15.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/selection.h	2020-09-08 12:14:40.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/selection.h $
-// $Id: selection.h 952e0d9 2020-03-05T14:50:52+01:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/selection.h $
+// $Id: selection.h a84927d 2020-07-23T17:15:44+02:00 Laurent Rineau
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Sebastien Loriot
@@ -16,6 +16,12 @@
 #include <CGAL/boost/graph/iterator.h>
 #include <boost/unordered_set.hpp>
 
+#include <CGAL/boost/graph/Dual.h>
+#include <boost/graph/filtered_graph.hpp>
+#include <boost/iterator/filter_iterator.hpp>
+
+#include <CGAL/boost/graph/alpha_expansion_graphcut.h>
+#include <CGAL/squared_distance_3.h>
 
 namespace CGAL {
 
@@ -56,12 +62,231 @@ extract_selection_boundary(
   }
   return out;
 }
+
+template <typename GeomTraits,
+          typename FaceGraph,
+          typename IsSelectedMap,
+          typename FaceIndexMap,
+          typename VertexPointMap>
+struct Regularization_graph
+{
+  typedef boost::graph_traits<FaceGraph> GT;
+  typedef typename GT::face_descriptor fg_face_descriptor;
+  typedef typename GT::face_iterator fg_face_iterator;
+  typedef typename GT::halfedge_descriptor fg_halfedge_descriptor;
+  typedef typename GT::edge_descriptor fg_edge_descriptor;
+  typedef typename GT::edge_iterator fg_edge_iterator;
+  typedef typename GT::vertex_descriptor fg_vertex_descriptor;
+
+  typedef fg_face_descriptor vertex_descriptor;
+  typedef fg_face_iterator vertex_iterator;
+  typedef fg_edge_descriptor edge_descriptor;
+  typedef boost::undirected_tag directed_category;
+  typedef boost::disallow_parallel_edge_tag edge_parallel_category;
+  typedef boost::edge_list_graph_tag traversal_category;
+
+  struct Filter_border_edges
+  {
+    FaceGraph* fg;
+    Filter_border_edges (FaceGraph& fg) : fg (&fg) { }
+    bool operator() (const fg_edge_descriptor ed) const
+    {
+      return !is_border (ed, *fg);
+    }
+  };
+
+  typedef boost::filter_iterator<Filter_border_edges, fg_edge_iterator> edge_iterator;
+
+  struct Vertex_label_map
+  {
+    typedef vertex_descriptor key_type;
+    typedef std::size_t value_type;
+    typedef std::size_t& reference;
+    typedef boost::lvalue_property_map_tag category;
+
+    Regularization_graph* rg;
+
+    Vertex_label_map (Regularization_graph* rg)
+      : rg (rg) { }
+
+    friend reference get (const Vertex_label_map& map, key_type k)
+    {
+      return (map.rg->labels)[get(map.rg->face_index_map,k)];
+    }
+    friend void put (const Vertex_label_map& map, key_type k, const value_type& v)
+    {
+      (map.rg->labels)[get(map.rg->face_index_map,k)] = v;
+    }
+  };
+
+  struct Vertex_label_probability_map
+  {
+    typedef vertex_descriptor key_type;
+    typedef std::vector<double> value_type;
+    typedef value_type reference;
+    typedef boost::readable_property_map_tag category;
+
+    const Regularization_graph* rg;
+
+    Vertex_label_probability_map (const Regularization_graph* rg)
+      : rg (rg)
+    { }
+
+    friend reference get (const Vertex_label_probability_map& pmap, key_type fd)
+    {
+      double value = (1. - pmap.rg->weight) * pmap.rg->area (fd) / pmap.rg->total_area;
+
+      std::vector<double> out(2);
+      if (get(pmap.rg->is_selected_map, fd))
+      {
+        if (pmap.rg->prevent_unselection)
+          out[0] = (std::numeric_limits<double>::max)();
+        else
+          out[0] = value;
+        out[1] = 0.;
+      }
+      else
+      {
+        out[0] = 0.;
+        out[1] = value;
+      }
+
+      return out;
+    }
+  };
+
+  struct Edge_cost_map
+  {
+    typedef edge_descriptor key_type;
+    typedef double value_type;
+    typedef value_type reference;
+    typedef boost::readable_property_map_tag category;
+
+    const Regularization_graph* rg;
+
+    Edge_cost_map (const Regularization_graph* rg)
+      : rg (rg) { }
+
+    friend reference get (const Edge_cost_map& pmap, key_type ed)
+    {
+      fg_vertex_descriptor esource = source(ed, pmap.rg->fg);
+      fg_vertex_descriptor etarget = target(ed, pmap.rg->fg);
+
+      // Cost
+      double edge_length = std::sqrt(CGAL::squared_distance (get (pmap.rg->vertex_point_map, esource),
+                                                             get (pmap.rg->vertex_point_map, etarget)));
+      return pmap.rg->weight * edge_length / pmap.rg->total_length;
+    }
+  };
+
+  FaceGraph& fg;
+  IsSelectedMap is_selected_map;
+  FaceIndexMap face_index_map;
+  VertexPointMap vertex_point_map;
+  double total_length;
+  double total_area;
+  double weight;
+  bool prevent_unselection;
+  std::vector<std::size_t> labels;
+
+  Regularization_graph (FaceGraph& fg,
+                        IsSelectedMap is_selected_map,
+                        FaceIndexMap face_index_map,
+                        VertexPointMap vertex_point_map,
+                        double weight,
+                        bool prevent_unselection)
+    : fg (fg),
+      is_selected_map (is_selected_map),
+      face_index_map (face_index_map),
+      vertex_point_map (vertex_point_map),
+      total_length(0),
+      total_area(0),
+      weight (weight),
+      prevent_unselection (prevent_unselection)
+  {
+    labels.reserve(num_faces(fg));
+    std::size_t nb_selected = 0;
+    for (fg_face_descriptor fd : faces(fg))
+    {
+      if (get(is_selected_map,fd))
+      {
+        labels.push_back(1);
+        ++ nb_selected;
+      }
+      else
+        labels.push_back(0);
+    }
+
+    // Compute normalization factors
+    for (fg_edge_descriptor ed : edges(fg))
+      total_length += length (ed);
+    for (fg_face_descriptor fd : faces(fg))
+      total_area += area (fd);
+  }
+
+  double length (fg_edge_descriptor ed) const
+  {
+    fg_vertex_descriptor esource = source(ed, fg);
+    fg_vertex_descriptor etarget = target(ed, fg);
+    return approximate_sqrt (typename GeomTraits::Compute_squared_distance_3()
+                             (get (vertex_point_map, esource),
+                              get (vertex_point_map, etarget)));
+  }
+
+  double area (fg_face_descriptor fd) const
+  {
+    fg_halfedge_descriptor hd = halfedge (fd, fg);
+    fg_halfedge_descriptor nhd = next (hd, fg);
+
+    return approximate_sqrt (typename GeomTraits::Compute_squared_area_3()
+                             (get (vertex_point_map, source (hd, fg)),
+                              get (vertex_point_map, target (hd, fg)),
+                              get (vertex_point_map, target (nhd, fg))));
+  }
+
+  friend CGAL::Iterator_range<vertex_iterator>
+  vertices (const Regularization_graph& graph)
+  {
+    return faces (graph.fg);
+  }
+
+  friend std::size_t num_vertices (const Regularization_graph& graph) { return num_faces(graph.fg); }
+
+  friend CGAL::Iterator_range<edge_iterator>
+  edges (const Regularization_graph& graph)
+  {
+    return CGAL::make_range (boost::make_filter_iterator
+                             (Filter_border_edges(graph.fg),
+                              begin(edges(graph.fg)), end(edges(graph.fg))),
+                             boost::make_filter_iterator
+                             (Filter_border_edges(graph.fg),
+                              end(edges(graph.fg)), end(edges(graph.fg))));
+  }
+
+  friend vertex_descriptor source (edge_descriptor ed, const Regularization_graph& graph)
+  {
+    return face (halfedge (ed, graph.fg), graph.fg);
+  }
+
+  friend vertex_descriptor target (edge_descriptor ed, const Regularization_graph& graph)
+  {
+    return face (opposite(halfedge (ed, graph.fg), graph.fg), graph.fg);
+  }
+
+  Vertex_label_map vertex_label_map() { return Vertex_label_map(this); }
+  Vertex_label_probability_map vertex_label_probability_map() const
+  { return Vertex_label_probability_map(this); }
+  Edge_cost_map edge_cost_map() const
+  { return Edge_cost_map(this); }
+};
+
+
 } //end of namespace internal
 
 
 /*!
 \ingroup PkgBGLSelectionFct
-Augments a selection with faces of `fg` that are adjacent
+augments a selection with faces of `fg` that are adjacent
 to a face in `selection`. This process is applied `k` times considering
 all faces added in the previous steps.
 Two faces are said to be adjacent if they share a vertex or an edge.
@@ -131,7 +356,7 @@ expand_face_selection(
 
 /*!
 \ingroup PkgBGLSelectionFct
-Diminishes a selection of faces from faces adjacent to a non-selected face.
+diminishes a selection of faces from faces adjacent to a non-selected face.
 This process is applied `k` times considering all faces removed in the previous steps.
 Two faces are said to be adjacent if they share a vertex or an edge.
 Each face removed from the selection is added exactly once in `out`.
@@ -203,6 +428,283 @@ reduce_face_selection(
   return out;
 }
 
+/*!
+  \ingroup PkgBGLSelectionFct
+
+  regularizes a selection in order to minimize the length of the
+  border of the selection.
+
+  The alpha expansion algorithm is used (see
+  `CGAL::alpha_expansion_graphcut()`) using the length of the edge
+  between two faces as the edge cost and the initial
+  selected/unselected property of a face as the face cost.
+
+  If `prevent_unselection` is set to `true`, the cost of unselecting a
+  face is set to infinity, which forces the regularization to only
+  select new faces and ensures that the regularization keeps all
+  selected faces.
+
+  \tparam TriangleMesh a model of `FaceGraph`
+
+  \tparam IsSelectedMap a model of `ReadWritePropertyMap` with
+  `boost::graph_traits<TriangleMesh>::%face_descriptor` as key type and
+  `bool` as value type
+
+  \tparam NamedParameters a sequence of named parameters
+
+  \param mesh the mesh containing the selected faces.
+
+  \param is_selected indicates if a face is part of the selection. It
+  is updated by the function to accommodate faces added or removed
+  from the selection.
+
+  \param weight sets the tradeoff between data fidelity and
+  regularity, ranging from 0 (no regularization at all, selection is
+  left unaltered) to 1 (maximum regularization, usually selects or
+  unselects everything so that the length of the border of the
+  selection is 0)
+
+  \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+
+  \cgalNamedParamsBegin
+    \cgalParamNBegin{vertex_point_map}
+      \cgalParamDescription{a property map associating points to the vertices of `tm`}
+      \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+                     as key type and `%Point_3` as value type}
+      \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+      \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+                      must be available in `TriangleMesh`.}
+    \cgalParamNEnd
+
+    \cgalParamNBegin{face_index_map}
+      \cgalParamDescription{a property map associating to each face of `tm` a unique index between `0` and `num_faces(tm) - 1`}
+      \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%face_descriptor`
+                     as key type and `std::size_t` as value type}
+      \cgalParamDefault{an automatically indexed internal map}
+    \cgalParamNEnd
+
+    \cgalParamNBegin{prevent_unselection}
+      \cgalParamDescription{Boolean used to indicate if selection can be only extended or if it can also be shrinked.}
+      \cgalParamType{`bool`}
+      \cgalParamDefault{`false`}
+      \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+    \cgalParamNEnd
+
+    \cgalParamNBegin{geom_traits}
+      \cgalParamDescription{an instance of a geometric traits class}
+      \cgalParamType{a class model of `Kernel`}
+      \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+      \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+    \cgalParamNEnd
+  \cgalNamedParamsEnd
+*/
+template <typename TriangleMesh, typename IsSelectedMap, typename NamedParameters>
+void
+regularize_face_selection_borders(
+  TriangleMesh& mesh,
+  IsSelectedMap is_selected,
+  double weight,
+  const NamedParameters& np)
+{
+  using parameters::choose_parameter;
+  using parameters::get_parameter;
+
+  CGAL_precondition (0.0 <= weight && weight < 1.0);
+
+  typedef boost::graph_traits<TriangleMesh> GT;
+  typedef typename GT::face_descriptor mesh_face_descriptor;
+
+  typedef typename GetInitializedFaceIndexMap<TriangleMesh, NamedParameters>::type FaceIndexMap;
+  FaceIndexMap face_index_map = CGAL::get_initialized_face_index_map(mesh, np);
+
+  typedef typename GetVertexPointMap<TriangleMesh, NamedParameters>::const_type VertexPointMap;
+  VertexPointMap vertex_point_map
+    = choose_parameter(get_parameter(np, internal_np::vertex_point),
+                       get_const_property_map(vertex_point, mesh));
+
+  typedef typename GetGeomTraits<TriangleMesh, NamedParameters>::type Kernel;
+
+  bool prevent_unselection = choose_parameter(get_parameter(np, internal_np::prevent_unselection),
+                                              false);
+
+  internal::Regularization_graph<Kernel, TriangleMesh, IsSelectedMap, FaceIndexMap,
+                                 VertexPointMap>
+    graph (mesh, is_selected,
+           face_index_map,
+           vertex_point_map,
+           weight,
+           prevent_unselection);
+
+  alpha_expansion_graphcut (graph,
+                            graph.edge_cost_map(),
+                            graph.vertex_label_probability_map(),
+                            graph.vertex_label_map(),
+                            CGAL::parameters::vertex_index_map
+                            (face_index_map));
+
+  for (mesh_face_descriptor fd : faces(mesh))
+    put(is_selected, fd, graph.labels[get(face_index_map,fd)]);
+}
+
+/// \cond SKIP_IN_MANUAL
+// variant with default np
+template <typename TriangleMesh, typename IsSelectedMap>
+void
+regularize_face_selection_borders(
+  TriangleMesh& fg,
+  IsSelectedMap is_selected,
+  double weight)
+{
+  regularize_face_selection_borders (fg, is_selected, weight,
+                                     CGAL::parameters::all_default());
+}
+/// \endcond
+
+/// \cond SKIP_IN_MANUAL
+// TODO: improve and document if useful
+//
+// Variant of regularization without graphcut but with brut-force
+// local expansions. Can be interesting in some cases but too
+// experimental/messy so far to be officially integrated.
+template <class FaceGraph, class IsSelectedMap, class VertexPointMap>
+void
+regularize_face_selection_borders(
+  FaceGraph& fg,
+  IsSelectedMap is_selected,
+  VertexPointMap vertex_point_map)
+{
+  typedef boost::graph_traits<FaceGraph> GT;
+  typedef typename GT::face_descriptor fg_face_descriptor;
+  typedef typename GT::halfedge_descriptor fg_halfedge_descriptor;
+  typedef typename GT::edge_descriptor fg_edge_descriptor;
+  typedef typename GT::vertex_descriptor fg_vertex_descriptor;
+
+  // TODO: this is a quick and dirty version, the complexity is
+  // crazy and it should be easy to do better (with priority queues,
+  // for example)
+
+  auto border_length =
+    [&]() -> double
+    {
+      double out = 0.;
+      for(fg_edge_descriptor ed : edges(fg))
+      {
+        fg_face_descriptor f0 = face (halfedge (ed, fg), fg);
+        fg_face_descriptor f1 = face (opposite(halfedge (ed, fg), fg), fg);
+        if (get(is_selected,f0) == get(is_selected,f1))
+          continue;
+
+        fg_vertex_descriptor esource = source(ed, fg);
+        fg_vertex_descriptor etarget = target(ed, fg);
+
+        out += std::sqrt(CGAL::squared_distance (get (vertex_point_map, esource),
+                                                 get (vertex_point_map, etarget)));
+      }
+      return out;
+    };
+
+  // First: try edges
+  while (true)
+  {
+    fg_edge_descriptor chosen;
+    double length_before = border_length();
+    double shortest_length = length_before;
+
+    for (fg_edge_descriptor ed : edges(fg))
+    {
+      fg_face_descriptor selected = face (halfedge (ed, fg), fg);
+      fg_face_descriptor unselected = face (opposite(halfedge (ed, fg), fg), fg);
+      if (get(is_selected,selected) == get(is_selected,unselected))
+        continue;
+
+      if (get(is_selected, unselected))
+        std::swap (selected, unselected);
+
+      put(is_selected, unselected, true);
+      double length_after = border_length();
+
+      if (length_after < shortest_length)
+      {
+        chosen = ed;
+        shortest_length = length_after;
+      }
+
+      // Cancel
+      put(is_selected, unselected, false);
+    }
+
+    if (shortest_length == length_before)
+      break;
+
+    fg_face_descriptor selected = face (halfedge (chosen, fg), fg);
+    fg_face_descriptor unselected = face (opposite(halfedge (chosen, fg), fg), fg);
+    if (get(is_selected,selected) == get(is_selected,unselected))
+      continue;
+
+    if (get(is_selected, unselected))
+      std::swap (selected, unselected);
+
+    put(is_selected, unselected, true);
+  }
+
+  // Second: try 1-ring of vertices
+  while (true)
+  {
+    fg_vertex_descriptor chosen;
+    double length_before = border_length();
+    double shortest_length = length_before;
+
+    for (fg_vertex_descriptor vd : vertices(fg))
+    {
+      fg_halfedge_descriptor hd = halfedge(vd, fg);
+      bool adjacent_to_selected = false, adjacent_to_nonselected = false;
+      for (fg_face_descriptor fd : faces_around_target (hd, fg))
+      {
+        if (get(is_selected, fd))
+          adjacent_to_selected = true;
+        else
+          adjacent_to_nonselected = true;
+
+        if (adjacent_to_selected && adjacent_to_nonselected)
+          break;
+      }
+
+      if (!(adjacent_to_selected && adjacent_to_nonselected))
+        continue;
+
+      std::vector<fg_face_descriptor> newly_selected;
+      for (fg_face_descriptor fd : faces_around_target (hd, fg))
+      {
+        if (!get(is_selected, fd))
+        {
+          newly_selected.push_back (fd);
+          put(is_selected, fd, true);
+        }
+      }
+      double length_after = border_length();
+
+      if (length_after < shortest_length)
+      {
+        chosen = vd;
+        shortest_length = length_after;
+      }
+
+      // Cancel
+      for (fg_face_descriptor fd : newly_selected)
+        put(is_selected, fd, false);
+    }
+
+    if (shortest_length == length_before)
+      break;
+
+    fg_halfedge_descriptor hd = halfedge (chosen, fg);
+
+    for (fg_face_descriptor fd : faces_around_target (hd, fg))
+      put(is_selected, fd, true);
+  }
+}
+/// \endcond
+
 
 /*!
 \ingroup PkgBGLSelectionFct
@@ -247,7 +749,7 @@ select_incident_faces(
 
 /*!
 \ingroup PkgBGLSelectionFct
-Augments a selection with edges of `fg` that are adjacent
+augments a selection with edges of `fg` that are adjacent
 to an edge in `selection`. This process is applied `k` times considering
 all edges added in the previous steps.
 Two edges are said to be adjacent if they are incident to the same face or vertex.
@@ -314,7 +816,7 @@ expand_edge_selection(
 
 /*!
 \ingroup PkgBGLSelectionFct
-Diminishes a selection of edges from edges adjacent to a non-selected edge.
+diminishes a selection of edges from edges adjacent to a non-selected edge.
 This process is applied `k` times considering all edges removed in the previous steps.
 Two edges are said to be adjacent if they are incident to the same face or vertex.
 Each edge removed from the selection is added exactly once in `out`.
@@ -400,7 +902,7 @@ reduce_edge_selection(
 
 /*!
 \ingroup PkgBGLSelectionFct
-Augments a selection with vertices of `fg` that are adjacent
+augments a selection with vertices of `fg` that are adjacent
 to a vertex in `selection`. This process is applied `k` times considering
 all vertices added in the previous steps.
 Two vertices are said to be adjacent if they are part of the same face.
@@ -455,7 +957,7 @@ expand_vertex_selection(
 
 /*!
 \ingroup PkgBGLSelectionFct
-Diminishes a selection of vertices from vertices adjacent to a non-selected vertex.
+diminishes a selection of vertices from vertices adjacent to a non-selected vertex.
 This process is applied `k` times considering all vertices removed in the previous steps.
 Two vertices are said to be adjacent if they are part of the same face.
 Each vertex removed from the selection is added exactly once in `out`.
@@ -652,4 +1154,3 @@ bool is_selection_a_topological_disk(con
 } //end of namespace CGAL
 
 #endif //CGAL_BOOST_GRAPH_SELECTION_H
-
diff -pruN 5.0.3-1/include/CGAL/boost/graph/split_graph_into_polylines.h 5.1-1/include/CGAL/boost/graph/split_graph_into_polylines.h
--- 5.0.3-1/include/CGAL/boost/graph/split_graph_into_polylines.h	2020-08-01 19:02:15.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/split_graph_into_polylines.h	2020-09-08 12:14:40.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/split_graph_into_polylines.h $
-// $Id: split_graph_into_polylines.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/split_graph_into_polylines.h $
+// $Id: split_graph_into_polylines.h ebd65c9 2020-07-20T11:47:30+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Laurent Rineau, Xiang Gao
@@ -131,7 +131,7 @@ public:
   }
 }; // end class Less_on_G_copy_vertex_descriptors
 
-// Splits a graph at vertices with degree higher than two and at vertices where `is_terminal` returns `true`
+// splits a graph at vertices with degree higher than two and at vertices where `is_terminal` returns `true`
 // The vertices are duplicated, and new incident edges created.
 // `OrigGraph` must be undirected
 template <typename Graph,
@@ -192,6 +192,7 @@ void duplicate_terminal_vertices(Graph&
 
 } // namespace internal
 
+#ifndef DOXYGEN_RUNNING
 template <typename Graph,
           typename Visitor,
           typename IsTerminal,
@@ -201,6 +202,7 @@ split_graph_into_polylines(const Graph&
                            Visitor& polyline_visitor,
                            IsTerminal is_terminal,
                            LessForVertexDescriptors less);
+#endif
 
 /*!
 \ingroup PkgBGLRef
diff -pruN 5.0.3-1/include/CGAL/boost/graph/visitor.h 5.1-1/include/CGAL/boost/graph/visitor.h
--- 5.0.3-1/include/CGAL/boost/graph/visitor.h	2020-08-01 19:02:15.000000000 +0000
+++ 5.1-1/include/CGAL/boost/graph/visitor.h	2020-09-08 12:14:40.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/graph/visitor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/graph/visitor.h $
 // $Id: visitor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/iterator/counting_iterator.hpp 5.1-1/include/CGAL/boost/iterator/counting_iterator.hpp
--- 5.0.3-1/include/CGAL/boost/iterator/counting_iterator.hpp	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/boost/iterator/counting_iterator.hpp	2020-09-08 12:14:54.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/boost/iterator/counting_iterator.hpp $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/boost/iterator/counting_iterator.hpp $
 // $Id: counting_iterator.hpp 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/iterator/transform_iterator.hpp 5.1-1/include/CGAL/boost/iterator/transform_iterator.hpp
--- 5.0.3-1/include/CGAL/boost/iterator/transform_iterator.hpp	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/boost/iterator/transform_iterator.hpp	2020-09-08 12:14:54.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/boost/iterator/transform_iterator.hpp $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/boost/iterator/transform_iterator.hpp $
 // $Id: transform_iterator.hpp 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost/parameter.h 5.1-1/include/CGAL/boost/parameter.h
--- 5.0.3-1/include/CGAL/boost/parameter.h	2020-08-01 19:02:15.000000000 +0000
+++ 5.1-1/include/CGAL/boost/parameter.h	2020-09-08 12:14:40.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/BGL/include/CGAL/boost/parameter.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/BGL/include/CGAL/boost/parameter.h $
 // $Id: parameter.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/BOOST_MP_arithmetic_kernel.h 5.1-1/include/CGAL/BOOST_MP_arithmetic_kernel.h
--- 5.0.3-1/include/CGAL/BOOST_MP_arithmetic_kernel.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/BOOST_MP_arithmetic_kernel.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arithmetic_kernel/include/CGAL/BOOST_MP_arithmetic_kernel.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arithmetic_kernel/include/CGAL/BOOST_MP_arithmetic_kernel.h $
 // $Id: BOOST_MP_arithmetic_kernel.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/boost_mp.h 5.1-1/include/CGAL/boost_mp.h
--- 5.0.3-1/include/CGAL/boost_mp.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/boost_mp.h	2020-09-08 12:15:16.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/boost_mp.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/boost_mp.h $
 // $Id: boost_mp.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Bounded_kernel.h 5.1-1/include/CGAL/Bounded_kernel.h
--- 5.0.3-1/include/CGAL/Bounded_kernel.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Bounded_kernel.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_2/include/CGAL/Bounded_kernel.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_2/include/CGAL/Bounded_kernel.h $
 // $Id: Bounded_kernel.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/bounding_box.h 5.1-1/include/CGAL/bounding_box.h
--- 5.0.3-1/include/CGAL/bounding_box.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/bounding_box.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Principal_component_analysis_LGPL/include/CGAL/bounding_box.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Principal_component_analysis_LGPL/include/CGAL/bounding_box.h $
 // $Id: bounding_box.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Box_intersection_d/Box_d.h 5.1-1/include/CGAL/Box_intersection_d/Box_d.h
--- 5.0.3-1/include/CGAL/Box_intersection_d/Box_d.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Box_intersection_d/Box_d.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Box_intersection_d/include/CGAL/Box_intersection_d/Box_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Box_intersection_d/include/CGAL/Box_intersection_d/Box_d.h $
 // $Id: Box_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Box_intersection_d/box_limits.h 5.1-1/include/CGAL/Box_intersection_d/box_limits.h
--- 5.0.3-1/include/CGAL/Box_intersection_d/box_limits.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Box_intersection_d/box_limits.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Box_intersection_d/include/CGAL/Box_intersection_d/box_limits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Box_intersection_d/include/CGAL/Box_intersection_d/box_limits.h $
 // $Id: box_limits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Box_intersection_d/Box_traits_d.h 5.1-1/include/CGAL/Box_intersection_d/Box_traits_d.h
--- 5.0.3-1/include/CGAL/Box_intersection_d/Box_traits_d.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Box_intersection_d/Box_traits_d.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Box_intersection_d/include/CGAL/Box_intersection_d/Box_traits_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Box_intersection_d/include/CGAL/Box_intersection_d/Box_traits_d.h $
 // $Id: Box_traits_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Box_intersection_d/Box_with_handle_d.h 5.1-1/include/CGAL/Box_intersection_d/Box_with_handle_d.h
--- 5.0.3-1/include/CGAL/Box_intersection_d/Box_with_handle_d.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Box_intersection_d/Box_with_handle_d.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Box_intersection_d/include/CGAL/Box_intersection_d/Box_with_handle_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Box_intersection_d/include/CGAL/Box_intersection_d/Box_with_handle_d.h $
 // $Id: Box_with_handle_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Box_intersection_d/Box_with_info_d.h 5.1-1/include/CGAL/Box_intersection_d/Box_with_info_d.h
--- 5.0.3-1/include/CGAL/Box_intersection_d/Box_with_info_d.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Box_intersection_d/Box_with_info_d.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Box_intersection_d/include/CGAL/Box_intersection_d/Box_with_info_d.h $
-// $Id: Box_with_info_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Box_intersection_d/include/CGAL/Box_intersection_d/Box_with_info_d.h $
+// $Id: Box_with_info_d.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -16,34 +16,34 @@
 
 #include <CGAL/license/Box_intersection_d.h>
 
-
 #include <CGAL/basic.h>
 #include <CGAL/Box_intersection_d/Box_d.h>
 
 namespace CGAL {
-
 namespace Box_intersection_d {
 
-template<class NT_, int N, class Info_>
-class Box_with_info_d : public Box_d< NT_, N, ID_FROM_BOX_ADDRESS> {
+template<class NT_, int N, class Info_, class IdPolicy = ID_EXPLICIT>
+class Box_with_info_d
+  : public Box_d< NT_, N, IdPolicy>
+{
 protected:
     Info_ m_info;
 public:
-    typedef Box_d< NT_, N, ID_FROM_BOX_ADDRESS> Base;
-    typedef NT_                      NT;
-    typedef Info_                     Info;
+    typedef Box_d< NT_, N, IdPolicy>    Base;
+    typedef NT_                         NT;
+    typedef Info_                       Info;
+
     Box_with_info_d() {}
     Box_with_info_d( Info h) : m_info(h) {}
     Box_with_info_d( bool complete, Info h): Base(complete), m_info(h) {}
     Box_with_info_d(NT l[N], NT h[N], Info n) : Base( l, h), m_info(n) {}
     Box_with_info_d( const Bbox_2& b, Info h) : Base( b), m_info(h) {}
     Box_with_info_d( const Bbox_3& b, Info h) : Base( b), m_info(h) {}
+
     Info info() const { return m_info; }
 };
 
-} // end namespace Box_intersection_d
-
-
-} //namespace CGAL
+} // namespace Box_intersection_d
+} // namespace CGAL
 
-#endif
+#endif // CGAL_BOX_INTERSECTION_D_BOX_WITH_INFO_D_H
diff -pruN 5.0.3-1/include/CGAL/Box_intersection_d/segment_tree.h 5.1-1/include/CGAL/Box_intersection_d/segment_tree.h
--- 5.0.3-1/include/CGAL/Box_intersection_d/segment_tree.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Box_intersection_d/segment_tree.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Box_intersection_d/include/CGAL/Box_intersection_d/segment_tree.h $
-// $Id: segment_tree.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Box_intersection_d/include/CGAL/Box_intersection_d/segment_tree.h $
+// $Id: segment_tree.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -16,7 +16,6 @@
 
 #include <CGAL/license/Box_intersection_d.h>
 
-
 #include <CGAL/basic.h>
 #include <CGAL/Box_intersection_d/box_limits.h>
 
@@ -24,7 +23,6 @@
 #include <boost/random/uniform_int.hpp>
 #include <boost/random/variate_generator.hpp>
 
-
 #include <algorithm>
 #include <iterator>
 #include <functional>
@@ -93,6 +91,8 @@ void one_way_scan( RandomAccessIter1 p_b
                    bool in_order = true )
 {
     typedef typename Traits::Compare Compare;
+
+    // Putting a parallel sort here slows down the overall parallel algorithm
     std::sort( p_begin, p_end, Compare( 0 ) );
     std::sort( i_begin, i_end, Compare( 0 ) );
 
@@ -323,7 +323,7 @@ void segment_tree( RandomAccessIter1 p_b
                    RandomAccessIter2 i_begin, RandomAccessIter2 i_end,
                    T lo, T hi,
                    Callback callback, Predicate_traits traits,
-                   std::ptrdiff_t cutoff, int dim, bool in_order )
+                   std::ptrdiff_t cutoff, int dim, bool in_order)
 {
     typedef typename Predicate_traits::Spanning   Spanning;
     typedef typename Predicate_traits::Lo_less    Lo_less;
diff -pruN 5.0.3-1/include/CGAL/box_intersection_d.h 5.1-1/include/CGAL/box_intersection_d.h
--- 5.0.3-1/include/CGAL/box_intersection_d.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/box_intersection_d.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Box_intersection_d/include/CGAL/box_intersection_d.h $
-// $Id: box_intersection_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Box_intersection_d/include/CGAL/box_intersection_d.h $
+// $Id: box_intersection_d.h 8773cee 2020-02-05T10:54:11+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -25,12 +25,191 @@
 #include <CGAL/Box_intersection_d/Box_traits_d.h>
 #include <CGAL/Box_intersection_d/box_limits.h>
 
+#include <CGAL/use.h>
+#include <CGAL/tags.h>
+
+#ifdef CGAL_LINKED_WITH_TBB
+#include <tbb/task_group.h>
+#endif
+
+#include <iterator>
 #include <vector>
 
+////////////////////////////////////////////////////////////////////////////////////////////////
+/// THE CALLBACK MUST BE THREADSAFE IF YOU ARE USING THE PARALLEL MODE
+////////////////////////////////////////////////////////////////////////////////////////////////
+
 namespace CGAL {
+namespace internal {
 
 // Generic call with custom predicate traits parameter.
-template< class RandomAccessIter1, class RandomAccessIter2,
+template< class ConcurrencyTag,
+          class RandomAccessIter1, class RandomAccessIter2,
+          class Callback, class Traits >
+void box_intersection_segment_tree_d(
+    RandomAccessIter1 begin1, RandomAccessIter1 end1,
+    RandomAccessIter2 begin2, RandomAccessIter2 end2,
+    Callback callback,
+    const Traits& traits,
+    const std::ptrdiff_t cutoff,
+    const bool in_order)
+{
+  typedef typename Traits::NT NT;
+
+  CGAL_assertion(Traits::dimension() > 0);
+  const int dim = Traits::dimension() - 1;
+
+  const NT inf = Box_intersection_d::box_limits<NT>::inf();
+  const NT sup = Box_intersection_d::box_limits<NT>::sup();
+
+#ifndef CGAL_LINKED_WITH_TBB
+  CGAL_static_assertion_msg (!(boost::is_convertible<ConcurrencyTag, Parallel_tag>::value),
+                             "Parallel_tag is enabled but TBB is unavailable.");
+#else // CGAL_LINKED_WITH_TBB
+  if(boost::is_convertible<ConcurrencyTag, Parallel_tag>::value)
+  {
+    // Here is an illustration for n=2.
+    //
+    // Doing a R1-R2 intersection with a 2-split is doing 4 subpairs in parallel
+    // a-c and b-d
+    // r1[0][0] --a-- r1[0][1] --b-- r1[0][2]
+    // r2[0][0] --c-- r2[0][1] --d-- r2[0][2]
+    //
+    // a-d and b-c
+    // r1[1][0] --a-- r1[1][1] --b-- r1[1][2]
+    // r2[1][0] --c-- r2[1][1] --d-- r2[1][2]
+    //
+    // Ranges must be duplicates since sorting is performed
+
+    typedef typename std::iterator_traits<RandomAccessIter1>::value_type         val_t;
+    typedef typename std::iterator_traits<RandomAccessIter1>::difference_type    diff_size;
+
+    typedef std::vector<val_t>                                                   val_container;
+    typedef typename val_container::iterator                                     It;
+
+    static constexpr int n = 4;
+
+    const diff_size r1s = std::distance(begin1, end1);
+    const diff_size r2s = std::distance(begin2, end2);
+
+    val_container range_1_copies, range_2_copies;
+    range_1_copies.reserve(r1s * n);
+    range_2_copies.reserve(r2s * n);
+
+    const diff_size r1_step = r1s / n;
+    const diff_size r2_step = r2s / n;
+
+    for(int i=0; i<n; ++i)
+    {
+      range_1_copies.insert(range_1_copies.end(), begin1, end1);
+      range_2_copies.insert(range_2_copies.end(), begin2, end2);
+    }
+
+    // for example for n=2, there's 'begin', 'mid', and 'end' but we leave out 'end' for convenience
+    std::array<std::array<It, n>, n> range_1_iterators;
+    std::array<std::array<It, n>, n> range_2_iterators;
+
+    for(int i=0; i<n; ++i)
+    {
+      for(int j=0; j<n; ++j)
+      {
+        range_1_iterators[i][j] = range_1_copies.begin() + i * r1s + j * r1_step;
+        range_2_iterators[i][j] = range_2_copies.begin() + i * r2s + j * r2_step;
+      }
+    }
+
+    tbb::task_group g;
+
+    for(int i=0; i<n; ++i)
+    {
+      for(int j=0; j<n; ++j)
+      {
+        // 'j' vs 'j+i', meaning for each value of 'i' we're matching r1[i] with a shifted version of r2[i]
+        int r1_endi, r1_endj;
+        int r2_endi, r2_endj;
+
+        if(i == (n-1))
+        {
+          if(j == (n-1)) // both very last range and very last iterator
+          {
+            r1_endi = -1;
+            r1_endj = -1;
+          }
+          else
+          {
+            r1_endi = i;
+            r1_endj = j+1;
+          }
+        }
+        else // i != (n-1)
+        {
+          if(j == (n-1)) // end of that range, but not of the full container
+          {
+            r1_endi = i+1;
+            r1_endj = 0;
+          }
+          else
+          {
+            r1_endi = i;
+            r1_endj = j+1;
+          }
+        }
+
+        if(i == (n-1))
+        {
+          if(j+i == (n-1)) // both very last range and very last iterator
+          {
+            r2_endi = -1;
+            r2_endj = -1;
+          }
+          else
+          {
+            r2_endi = i;
+            r2_endj = (j+i+1)%n;
+          }
+        }
+        else // i != (n-1)
+        {
+          if(j+i == (n-1)) // end of that range, but not of the full container
+          {
+            r2_endi = i+1;
+            r2_endj = 0;
+          }
+          else
+          {
+            r2_endi = i;
+            r2_endj = (j+i+1)%n;
+          }
+        }
+
+        It r1_start = range_1_iterators[i][j];
+        It r1_end = (r1_endi == -1) ? range_1_copies.end() : range_1_iterators[r1_endi][r1_endj];
+        It r2_start = range_2_iterators[i][(j+i)%n];
+        It r2_end = (r2_endi == -1) ? range_2_copies.end() : range_2_iterators[r2_endi][r2_endj];
+        CGAL_assertion(range_1_copies.begin() <= r1_start && r1_start <= r1_end && r1_end <= range_1_copies.end());
+        CGAL_assertion(range_2_copies.begin() <= r2_start && r2_start <= r2_end && r2_end <= range_2_copies.end());
+
+        // Specify "copy by value" otherwise the values of iterators for next (i,j) iterations
+        // become shared with different lambdas being run in parallel, and things go wrong
+        g.run([=]{ Box_intersection_d::segment_tree( r1_start, r1_end, r2_start, r2_end,
+                                                     inf, sup, callback, traits, cutoff, dim, in_order); });
+      }
+    }
+
+    g.wait();
+  }
+  else
+#endif // CGAL_LINKED_WITH_TBB
+  {
+    Box_intersection_d::segment_tree(begin1, end1, begin2, end2, inf, sup, callback, traits, cutoff, dim, in_order);
+  }
+}
+
+} // namespace internal
+
+// Generic call with custom predicate traits parameter.
+template< class ConcurrencyTag = Sequential_tag,
+          class RandomAccessIter1, class RandomAccessIter2,
           class Callback, class BoxPredicateTraits >
 void box_intersection_custom_predicates_d(
     RandomAccessIter1 begin1, RandomAccessIter1 end1,
@@ -40,23 +219,15 @@ void box_intersection_custom_predicates_
     std::ptrdiff_t cutoff = 10,
     Box_intersection_d::Setting setting = Box_intersection_d::BIPARTITE)
 {
-    typedef BoxPredicateTraits Traits;
-    typedef typename Traits::NT NT;
-    CGAL_assertion( Traits::dimension() > 0 );
-    const int dim = Traits::dimension() - 1;
-    const NT inf = Box_intersection_d::box_limits<NT>::inf();
-    const NT sup = Box_intersection_d::box_limits<NT>::sup();
-    Box_intersection_d::segment_tree(begin1, end1, begin2, end2,
-                              inf, sup, callback, traits, cutoff, dim, true);
-    if(setting == Box_intersection_d::BIPARTITE)
-        Box_intersection_d::segment_tree(begin2, end2, begin1, end1,
-                              inf, sup, callback, traits, cutoff, dim, false);
+  internal::box_intersection_segment_tree_d<ConcurrencyTag>(begin1, end1, begin2, end2, callback, traits, cutoff, true);
+  if(setting == Box_intersection_d::BIPARTITE)
+    internal::box_intersection_segment_tree_d<ConcurrencyTag>(begin2, end2, begin1, end1, callback, traits, cutoff, false);
 }
 
-
 // Generic call with box traits parameter.
 // - make all default parameters explicit overloads (workaround)
-template< class RandomAccessIter1, class RandomAccessIter2,
+template< class ConcurrencyTag = Sequential_tag,
+          class RandomAccessIter1, class RandomAccessIter2,
           class Callback, class BoxTraits >
 void box_intersection_d(
     RandomAccessIter1 begin1, RandomAccessIter1 end1,
@@ -67,18 +238,19 @@ void box_intersection_d(
     Box_intersection_d::Topology topology,
     Box_intersection_d::Setting  setting)
 {
-    if (topology == Box_intersection_d::CLOSED) {
-        typedef Box_intersection_d::Predicate_traits_d<BoxTraits,true> Traits;
-        box_intersection_custom_predicates_d(begin1, end1, begin2, end2,
-                                         callback, Traits(), cutoff, setting);
-    } else {
-        typedef Box_intersection_d::Predicate_traits_d<BoxTraits,false> Traits;
-        box_intersection_custom_predicates_d(begin1, end1, begin2, end2,
-                                         callback, Traits(), cutoff, setting);
-    }
+  if (topology == Box_intersection_d::CLOSED) {
+    typedef Box_intersection_d::Predicate_traits_d<BoxTraits,true> Traits;
+    box_intersection_custom_predicates_d<ConcurrencyTag>(begin1, end1, begin2, end2,
+                                                         callback, Traits(), cutoff, setting);
+  } else {
+    typedef Box_intersection_d::Predicate_traits_d<BoxTraits,false> Traits;
+    box_intersection_custom_predicates_d<ConcurrencyTag>(begin1, end1, begin2, end2,
+                                                         callback, Traits(), cutoff, setting);
+  }
 }
 
-template< class RandomAccessIter1, class RandomAccessIter2,
+template< class ConcurrencyTag = Sequential_tag,
+          class RandomAccessIter1, class RandomAccessIter2,
           class Callback, class BoxTraits >
 void box_intersection_d(
     RandomAccessIter1 begin1, RandomAccessIter1 end1,
@@ -86,35 +258,39 @@ void box_intersection_d(
     Callback callback, BoxTraits box_traits, std::ptrdiff_t cutoff,
     Box_intersection_d::Topology topology)
 {
-    box_intersection_d( begin1, end1, begin2, end2, callback, box_traits,
-                        cutoff, topology, Box_intersection_d::BIPARTITE);
+    box_intersection_d<ConcurrencyTag>( begin1, end1, begin2, end2, callback, box_traits,
+                                        cutoff, topology, Box_intersection_d::BIPARTITE);
 }
-template< class RandomAccessIter1, class RandomAccessIter2,
+template< class ConcurrencyTag = Sequential_tag,
+          class RandomAccessIter1, class RandomAccessIter2,
           class Callback, class BoxTraits >
 void box_intersection_d(
     RandomAccessIter1 begin1, RandomAccessIter1 end1,
     RandomAccessIter2 begin2, RandomAccessIter2 end2,
     Callback callback, BoxTraits box_traits, std::ptrdiff_t cutoff)
 {
-    box_intersection_d( begin1, end1, begin2, end2, callback, box_traits,
-                        cutoff, Box_intersection_d::CLOSED,
-                        Box_intersection_d::BIPARTITE);
+  box_intersection_d<ConcurrencyTag>( begin1, end1, begin2, end2, callback, box_traits,
+                                      cutoff, Box_intersection_d::CLOSED,
+                                      Box_intersection_d::BIPARTITE);
 }
-template< class RandomAccessIter1, class RandomAccessIter2,
+
+template< class ConcurrencyTag = Sequential_tag,
+          class RandomAccessIter1, class RandomAccessIter2,
           class Callback, class BoxTraits >
 void box_intersection_d(
     RandomAccessIter1 begin1, RandomAccessIter1 end1,
     RandomAccessIter2 begin2, RandomAccessIter2 end2,
     Callback callback, BoxTraits box_traits)
 {
-    box_intersection_d( begin1, end1, begin2, end2, callback, box_traits,
-                        10, Box_intersection_d::CLOSED,
-                        Box_intersection_d::BIPARTITE);
+  box_intersection_d<ConcurrencyTag>( begin1, end1, begin2, end2, callback, box_traits,
+                                      10, Box_intersection_d::CLOSED,
+                                      Box_intersection_d::BIPARTITE);
 }
 
 // Specialized call with default box traits.
 // - make all default parameters explicit overloads (workaround)
-template< class RandomAccessIter1, class RandomAccessIter2, class Callback >
+template< class ConcurrencyTag = Sequential_tag,
+          class RandomAccessIter1, class RandomAccessIter2, class Callback >
 void box_intersection_d(
     RandomAccessIter1 begin1, RandomAccessIter1 end1,
     RandomAccessIter2 begin2, RandomAccessIter2 end2,
@@ -122,53 +298,58 @@ void box_intersection_d(
     Box_intersection_d::Topology topology,
     Box_intersection_d::Setting  setting)
 {
-    typedef typename std::iterator_traits<RandomAccessIter1>::value_type val_t;
-    typedef Box_intersection_d::Box_traits_d< val_t>  Box_traits;
-    box_intersection_d( begin1, end1, begin2, end2, callback, Box_traits(),
-                        cutoff, topology, setting);
+  typedef typename std::iterator_traits<RandomAccessIter1>::value_type val_t;
+  typedef Box_intersection_d::Box_traits_d< val_t>                     Box_traits;
+
+  box_intersection_d<ConcurrencyTag>( begin1, end1, begin2, end2, callback, Box_traits(),
+                                      cutoff, topology, setting);
 }
 
-template< class RandomAccessIter1, class RandomAccessIter2, class Callback >
+template< class ConcurrencyTag = Sequential_tag,
+          class RandomAccessIter1, class RandomAccessIter2, class Callback >
 void box_intersection_d(
     RandomAccessIter1 begin1, RandomAccessIter1 end1,
     RandomAccessIter2 begin2, RandomAccessIter2 end2,
     Callback callback, std::ptrdiff_t cutoff,
     Box_intersection_d::Topology topology)
 {
-    typedef typename std::iterator_traits<RandomAccessIter1>::value_type val_t;
-    typedef Box_intersection_d::Box_traits_d< val_t>  Box_traits;
-    box_intersection_d( begin1, end1, begin2, end2, callback, Box_traits(),
-                        cutoff, topology, Box_intersection_d::BIPARTITE);
+  typedef typename std::iterator_traits<RandomAccessIter1>::value_type val_t;
+  typedef Box_intersection_d::Box_traits_d< val_t>                     Box_traits;
+
+  box_intersection_d<ConcurrencyTag>( begin1, end1, begin2, end2, callback, Box_traits(),
+                                      cutoff, topology, Box_intersection_d::BIPARTITE);
 }
-template< class RandomAccessIter1, class RandomAccessIter2, class Callback >
+template< class ConcurrencyTag = Sequential_tag,
+          class RandomAccessIter1, class RandomAccessIter2, class Callback >
 void box_intersection_d(
     RandomAccessIter1 begin1, RandomAccessIter1 end1,
     RandomAccessIter2 begin2, RandomAccessIter2 end2,
     Callback callback, std::ptrdiff_t cutoff)
 {
-    typedef typename std::iterator_traits<RandomAccessIter1>::value_type val_t;
-    typedef Box_intersection_d::Box_traits_d< val_t>  Box_traits;
-    box_intersection_d( begin1, end1, begin2, end2, callback, Box_traits(),
-                        cutoff, Box_intersection_d::CLOSED,
-                        Box_intersection_d::BIPARTITE);
+  typedef typename std::iterator_traits<RandomAccessIter1>::value_type val_t;
+  typedef Box_intersection_d::Box_traits_d< val_t>  Box_traits;
+  box_intersection_d<ConcurrencyTag>( begin1, end1, begin2, end2, callback, Box_traits(),
+                                      cutoff, Box_intersection_d::CLOSED,
+                                      Box_intersection_d::BIPARTITE);
 }
-template< class RandomAccessIter1, class RandomAccessIter2, class Callback >
+template< class ConcurrencyTag = Sequential_tag,
+          class RandomAccessIter1, class RandomAccessIter2, class Callback >
 void box_intersection_d(
     RandomAccessIter1 begin1, RandomAccessIter1 end1,
     RandomAccessIter2 begin2, RandomAccessIter2 end2,
     Callback callback)
 {
-    typedef typename std::iterator_traits<RandomAccessIter1>::value_type val_t;
-    typedef Box_intersection_d::Box_traits_d< val_t>  Box_traits;
-    box_intersection_d( begin1, end1, begin2, end2, callback, Box_traits(),
-                        10, Box_intersection_d::CLOSED,
-                        Box_intersection_d::BIPARTITE);
+  typedef typename std::iterator_traits<RandomAccessIter1>::value_type val_t;
+  typedef Box_intersection_d::Box_traits_d< val_t>  Box_traits;
+  box_intersection_d<ConcurrencyTag>( begin1, end1, begin2, end2, callback, Box_traits(),
+                                      10, Box_intersection_d::CLOSED,
+                                      Box_intersection_d::BIPARTITE);
 }
 
-
 // Generic call with box traits parameter, specialized for self-intersection.
 // - make all default parameters explicit overloads (workaround)
-template< class RandomAccessIter, class Callback, class BoxTraits >
+template< class ConcurrencyTag = Sequential_tag,
+          class RandomAccessIter, class Callback, class BoxTraits >
 void box_self_intersection_d(
     RandomAccessIter begin, RandomAccessIter end,
     Callback callback,
@@ -176,49 +357,52 @@ void box_self_intersection_d(
     std::ptrdiff_t cutoff,
     Box_intersection_d::Topology topology)
 {
-    // Copying rather than calling 'box_intersection_d(begin, end, begin, end, ...'
-    // is necessary because the 'std::partition' and range splits on the first range
-    // would be messed up by sorts on the second range otherwise.
-    typedef typename std::iterator_traits<RandomAccessIter>::value_type val_t;
-    std::vector< val_t> i( begin, end);
-    box_intersection_d( begin, end, i.begin(), i.end(),
-                        callback, box_traits, cutoff, topology,
-                        Box_intersection_d::COMPLETE);
+  // Copying rather than calling 'box_intersection_d(begin, end, begin, end, ...'
+  // is necessary because the 'std::partition' and range splits on the first range
+  // would be messed up by sorts on the second range otherwise.
+  typedef typename std::iterator_traits<RandomAccessIter>::value_type val_t;
+  std::vector< val_t> i( begin, end);
+
+  box_intersection_d<ConcurrencyTag>( begin, end, i.begin(), i.end(),
+                                      callback, box_traits, cutoff, topology,
+                                      Box_intersection_d::COMPLETE);
 }
 
-template< class RandomAccessIter, class Callback, class BoxTraits >
+template< class ConcurrencyTag = Sequential_tag,
+          class RandomAccessIter, class Callback, class BoxTraits >
 void box_self_intersection_d(
     RandomAccessIter begin, RandomAccessIter end,
     Callback callback,
     BoxTraits box_traits,
     std::ptrdiff_t cutoff)
 {
-    return box_self_intersection_d(begin, end, callback, box_traits, cutoff,
-                                   Box_intersection_d::CLOSED);
+    return box_self_intersection_d<ConcurrencyTag>(begin, end, callback, box_traits, cutoff,
+                                                   Box_intersection_d::CLOSED);
 }
 
-template< class RandomAccessIter, class Callback, class BoxTraits >
+template< class ConcurrencyTag = Sequential_tag,
+          class RandomAccessIter, class Callback, class BoxTraits >
 void box_self_intersection_d(
     RandomAccessIter begin, RandomAccessIter end,
     Callback callback,
     BoxTraits box_traits)
 {
-    return box_self_intersection_d(begin, end, callback, box_traits, 10);
+  return box_self_intersection_d<ConcurrencyTag>(begin, end, callback, box_traits, 10);
 }
 
 // Specialized call with default box traits, specialized for self-intersection.
 // - make all default parameters explicit overloads (workaround)
-template< class RandomAccessIter, class Callback >
+template< class ConcurrencyTag = Sequential_tag, class RandomAccessIter, class Callback >
 void box_self_intersection_d(
     RandomAccessIter begin, RandomAccessIter end,
     Callback callback)
 {
     typedef typename std::iterator_traits<RandomAccessIter>::value_type val_t;
     typedef Box_intersection_d::Box_traits_d< val_t>  Box_traits;
-    box_self_intersection_d(begin, end, callback, Box_traits());
+    box_self_intersection_d<ConcurrencyTag>(begin, end, callback, Box_traits());
 }
 
-template< class RandomAccessIter, class Callback >
+template< class ConcurrencyTag = Sequential_tag, class RandomAccessIter, class Callback >
 void box_self_intersection_d(
     RandomAccessIter begin, RandomAccessIter end,
     Callback callback,
@@ -226,10 +410,10 @@ void box_self_intersection_d(
 {
     typedef typename std::iterator_traits<RandomAccessIter>::value_type val_t;
     typedef Box_intersection_d::Box_traits_d< val_t>  Box_traits;
-    box_self_intersection_d(begin, end, callback, Box_traits(), cutoff);
+    box_self_intersection_d<ConcurrencyTag>(begin, end, callback, Box_traits(), cutoff);
 }
 
-template< class RandomAccessIter, class Callback >
+  template< class ConcurrencyTag = Sequential_tag, class RandomAccessIter, class Callback >
 void box_self_intersection_d(
     RandomAccessIter begin, RandomAccessIter end,
     Callback callback,
@@ -238,11 +422,10 @@ void box_self_intersection_d(
 {
     typedef typename std::iterator_traits<RandomAccessIter>::value_type val_t;
     typedef Box_intersection_d::Box_traits_d< val_t>  Box_traits;
-    box_self_intersection_d(begin, end, callback,
-                            Box_traits(), cutoff, topology );
+    box_self_intersection_d<ConcurrencyTag>(begin, end, callback,
+                                            Box_traits(), cutoff, topology );
 }
 
-
 // Generic call for trivial all-pairs algorithm with box traits parameter.
 // - make all default parameters explicit overloads (workaround)
 template< class ForwardIter1, class ForwardIter2,
diff -pruN 5.0.3-1/include/CGAL/Buffer_for_vao.h 5.1-1/include/CGAL/Buffer_for_vao.h
--- 5.0.3-1/include/CGAL/Buffer_for_vao.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Buffer_for_vao.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Buffer_for_vao.h $
-// $Id: Buffer_for_vao.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Buffer_for_vao.h $
+// $Id: Buffer_for_vao.h 58b10a3 2020-03-26T18:58:50+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -98,6 +98,11 @@ namespace internal
       CGAL::Cartesian_converter<K, Local_kernel> converter;
       return converter(v);
     }
+    static typename Local_kernel::Ray_2 get_local_ray(const typename K::Ray_2& r)
+    {
+      CGAL::Cartesian_converter<K, Local_kernel> converter;
+      return converter(r);
+    }
   };
 
   // Specialization when K==Local_kernel, because there is no need of convertion here.
@@ -116,6 +121,8 @@ namespace internal
     { return typename Local_kernel::Vector_3(v.x(), v.y(), 0); }
     static const typename Local_kernel::Vector_3& get_local_vector(const typename Local_kernel::Vector_3& v)
     { return v; }
+    static const typename Local_kernel::Ray_2& get_local_ray(const typename Local_kernel::Ray_2& r)
+    { return r; }
   };
 } // End namespace internal
 
@@ -127,6 +134,7 @@ public:
   typedef CGAL::Exact_predicates_inexact_constructions_kernel Local_kernel;
   typedef Local_kernel::Point_3  Local_point;
   typedef Local_kernel::Vector_3 Local_vector;
+  typedef Local_kernel::Ray_2    Local_ray;
 
   Buffer_for_vao(std::vector<BufferType>* pos=nullptr,
                  std::vector<IndexType>* indices=nullptr,
@@ -143,6 +151,7 @@ public:
     m_zero_x(true),
     m_zero_y(true),
     m_zero_z(true),
+    m_inverse_normal(false),
     m_face_started(false)
   {}
 
@@ -193,6 +202,17 @@ public:
   bool has_zero_z() const
   { return m_zero_z; }
 
+  void negate_normals()
+  {
+    m_inverse_normal=!m_inverse_normal;
+    for (std::vector<BufferType>*array=m_flat_normal_buffer; array!=nullptr;
+         array=(array==m_gouraud_normal_buffer?nullptr:m_gouraud_normal_buffer))
+    {
+      for (std::size_t i=0; i<array->size(); ++i)
+      { (*array)[i]=-(*array)[i]; }
+    }
+  }
+
   // 1.1) Add a point, without color. Return the index of the added point.
   template<typename KPoint>
   std::size_t add_point(const KPoint& kp)
@@ -212,6 +232,16 @@ public:
     return m_pos_buffer->size()-3;
   }
 
+  template<typename KPoint>
+  std::size_t add_point_infinity(const KPoint& kp)
+  {
+      if (!has_position()) return (std::size_t)-1;
+
+      Local_point p=get_local_point(kp);
+      add_point_in_buffer(p, *m_pos_buffer);
+      return m_pos_buffer->size()-3;
+  }
+
   // 1.2) Add a point, with color.
   template<typename KPoint>
   void add_point(const KPoint& kp, const CGAL::Color& c)
@@ -253,6 +283,44 @@ public:
     add_indexed_point(index2);
   }
 
+  // 3.1) Add a ray segment, without color
+  template<typename KPoint, typename KVector>
+  void add_ray_segment(const KPoint& kp1, const KVector& kp2)
+  {
+    add_point(kp1);
+    add_point_infinity(kp2);
+  }
+
+  //3.2) Add a ray segment, with color
+  template<typename KPoint, typename KVector>
+  void add_ray_segment(const KPoint& kp1, const KVector& kp2,
+                       const CGAL::Color& c)
+  {
+    add_point(kp1);
+    add_point_infinity(kp2);
+    add_color(c);
+    add_color(c);
+  }
+
+  // 4.1) Add a line, without color
+  template<typename KPoint>
+  void add_line_segment(const KPoint& kp1, const KPoint& kp2)
+  {
+    add_point_infinity(kp1);
+    add_point_infinity(kp2);
+  }
+
+  // 4.1) Add a line, with color
+  template<typename KPoint>
+  void add_line_segment(const KPoint& kp1, const KPoint& kp2,
+                        const CGAL::Color& c)
+  {
+    add_point_infinity(kp1);
+    add_point_infinity(kp2);
+    add_color(c);
+    add_color(c);
+  }
+
   /// @return true iff a face has begun.
   bool is_a_face_started() const
   { return m_face_started; }
@@ -338,8 +406,8 @@ public:
 
     if (m_points_of_face.size()<3)
     {
-      std::cerr<<"PB: you try to triangulate a face with "<<m_points_of_face.size()<<" vertices."
-               <<std::endl;
+      /* std::cerr<<"PB: you try to triangulate a face with "<<m_points_of_face.size()<<" vertices."
+               <<std::endl; */
 
       m_face_started=false;
       m_points_of_face.clear();
@@ -392,19 +460,20 @@ public:
   static void add_point_in_buffer(const KPoint& kp, std::vector<float>& buffer)
   {
     Local_point p=get_local_point(kp);
-    buffer.push_back(p.x());
-    buffer.push_back(p.y());
-    buffer.push_back(p.z());
+    buffer.push_back(static_cast<float>(p.x()));
+    buffer.push_back(static_cast<float>(p.y()));
+    buffer.push_back(static_cast<float>(p.z()));
   }
 
   /// adds `kv` coordinates to `buffer`
   template<typename KVector>
-  static void add_normal_in_buffer(const KVector& kv, std::vector<float>& buffer)
+  static void add_normal_in_buffer(const KVector& kv, std::vector<float>& buffer,
+                                   bool inverse_normal=false)
   {
-    Local_vector n=get_local_vector(kv);
-    buffer.push_back(n.x());
-    buffer.push_back(n.y());
-    buffer.push_back(n.z());
+    Local_vector n=(inverse_normal?-get_local_vector(kv):get_local_vector(kv));
+    buffer.push_back(static_cast<float>(n.x()));
+    buffer.push_back(static_cast<float>(n.y()));
+    buffer.push_back(static_cast<float>(n.z()));
   }
 
   ///adds `acolor` RGB components to `buffer`
@@ -458,6 +527,8 @@ public:
     return true;
   }
 
+  CGAL::Bbox_3 *bb() const { return m_bb; }
+
 protected:
   void face_begin_internal(bool has_color, bool has_normal)
   {
@@ -766,14 +837,14 @@ protected:
   void add_flat_normal(const KVector& kv)
   {
     if(m_flat_normal_buffer != nullptr)
-    { add_normal_in_buffer(kv, *m_flat_normal_buffer); }
+    { add_normal_in_buffer(kv, *m_flat_normal_buffer, m_inverse_normal); }
   }
 
   template<typename KVector>
   void add_gouraud_normal(const KVector& kv)
   {
     if(m_gouraud_normal_buffer != nullptr)
-    { add_normal_in_buffer(kv, *m_gouraud_normal_buffer); }
+    { add_normal_in_buffer(kv, *m_gouraud_normal_buffer, m_inverse_normal); }
   }
 
 protected:
@@ -827,7 +898,9 @@ protected:
   bool m_zero_y; /// True iff all points have y==0
   bool m_zero_z; /// True iff all points have z==0
 
-  // Local variables, used when we started a new face.
+  bool m_inverse_normal;;
+
+  // Local variables, used when we started a new face.g
   bool m_face_started;
   bool m_started_face_is_colored;
   bool m_started_face_has_normal;
diff -pruN 5.0.3-1/include/CGAL/Cache.h 5.1-1/include/CGAL/Cache.h
--- 5.0.3-1/include/CGAL/Cache.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Cache.h	2020-09-08 12:15:32.000000000 +0000
@@ -5,7 +5,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Cache.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Cache.h $
 // $Id: Cache.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/Aff_transformation_2.h 5.1-1/include/CGAL/Cartesian/Aff_transformation_2.h
--- 5.0.3-1/include/CGAL/Cartesian/Aff_transformation_2.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/Aff_transformation_2.h	2020-09-08 12:14:44.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/Aff_transformation_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/Aff_transformation_2.h $
 // $Id: Aff_transformation_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/Aff_transformation_3.h 5.1-1/include/CGAL/Cartesian/Aff_transformation_3.h
--- 5.0.3-1/include/CGAL/Cartesian/Aff_transformation_3.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/Aff_transformation_3.h	2020-09-08 12:14:44.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/Aff_transformation_3.h $
-// $Id: Aff_transformation_3.h 4c30c49 2020-05-18T12:22:27+02:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/Aff_transformation_3.h $
+// $Id: Aff_transformation_3.h 9a3e038 2020-05-18T12:24:25+02:00 Laurent Rineau
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/Aff_transformation_rep_2.h 5.1-1/include/CGAL/Cartesian/Aff_transformation_rep_2.h
--- 5.0.3-1/include/CGAL/Cartesian/Aff_transformation_rep_2.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/Aff_transformation_rep_2.h	2020-09-08 12:14:44.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/Aff_transformation_rep_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/Aff_transformation_rep_2.h $
 // $Id: Aff_transformation_rep_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/Aff_transformation_rep_3.h 5.1-1/include/CGAL/Cartesian/Aff_transformation_rep_3.h
--- 5.0.3-1/include/CGAL/Cartesian/Aff_transformation_rep_3.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/Aff_transformation_rep_3.h	2020-09-08 12:14:44.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/Aff_transformation_rep_3.h $
-// $Id: Aff_transformation_rep_3.h 4c30c49 2020-05-18T12:22:27+02:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/Aff_transformation_rep_3.h $
+// $Id: Aff_transformation_rep_3.h 9a3e038 2020-05-18T12:24:25+02:00 Laurent Rineau
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/basic_constructions_2.h 5.1-1/include/CGAL/Cartesian/basic_constructions_2.h
--- 5.0.3-1/include/CGAL/Cartesian/basic_constructions_2.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/basic_constructions_2.h	2020-09-08 12:14:45.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/basic_constructions_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/basic_constructions_2.h $
 // $Id: basic_constructions_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/basic_constructions_3.h 5.1-1/include/CGAL/Cartesian/basic_constructions_3.h
--- 5.0.3-1/include/CGAL/Cartesian/basic_constructions_3.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/basic_constructions_3.h	2020-09-08 12:14:45.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/basic_constructions_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/basic_constructions_3.h $
 // $Id: basic_constructions_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/Cartesian_base.h 5.1-1/include/CGAL/Cartesian/Cartesian_base.h
--- 5.0.3-1/include/CGAL/Cartesian/Cartesian_base.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/Cartesian_base.h	2020-09-08 12:14:44.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/Cartesian_base.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/Cartesian_base.h $
 // $Id: Cartesian_base.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/Circle_2.h 5.1-1/include/CGAL/Cartesian/Circle_2.h
--- 5.0.3-1/include/CGAL/Cartesian/Circle_2.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/Circle_2.h	2020-09-08 12:14:44.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/Circle_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/Circle_2.h $
 // $Id: Circle_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/Circle_3.h 5.1-1/include/CGAL/Cartesian/Circle_3.h
--- 5.0.3-1/include/CGAL/Cartesian/Circle_3.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/Circle_3.h	2020-09-08 12:14:44.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/Circle_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/Circle_3.h $
 // $Id: Circle_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/ConicCPA2.h 5.1-1/include/CGAL/Cartesian/ConicCPA2.h
--- 5.0.3-1/include/CGAL/Cartesian/ConicCPA2.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/ConicCPA2.h	2020-09-08 12:14:44.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/ConicCPA2.h $
-// $Id: ConicCPA2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/ConicCPA2.h $
+// $Id: ConicCPA2.h fdb17cb 2020-03-26T19:26:10+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -312,15 +312,12 @@ class ConicCPA2
     PT center () const
     {
         CGAL_kernel_precondition (type != PARABOLA);
-        // PT p;
-        // replaced previous line by following hack (no idea
-        // why original version doesn't work)
-        typename DA::Point p;
-        FT two = FT(2);
-        FT div = -det();
-        dao.set( p, (two*s()*u() - t()*v()) / div,
-                    (two*r()*v() - t()*u()) / div);
-        return p;
+
+        const FT two = FT(2);
+        const FT div = -det();
+
+        return PT((two*s()*u() - t()*v()) / div,
+                  (two*r()*v() - t()*u()) / div);
     }
 
     Conic_type conic_type () const
diff -pruN 5.0.3-1/include/CGAL/Cartesian/Data_accessor_2.h 5.1-1/include/CGAL/Cartesian/Data_accessor_2.h
--- 5.0.3-1/include/CGAL/Cartesian/Data_accessor_2.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/Data_accessor_2.h	2020-09-08 12:14:44.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/Data_accessor_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/Data_accessor_2.h $
 // $Id: Data_accessor_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/Direction_2.h 5.1-1/include/CGAL/Cartesian/Direction_2.h
--- 5.0.3-1/include/CGAL/Cartesian/Direction_2.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/Direction_2.h	2020-09-08 12:14:44.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/Direction_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/Direction_2.h $
 // $Id: Direction_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/Direction_3.h 5.1-1/include/CGAL/Cartesian/Direction_3.h
--- 5.0.3-1/include/CGAL/Cartesian/Direction_3.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/Direction_3.h	2020-09-08 12:14:44.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/Direction_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/Direction_3.h $
 // $Id: Direction_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/ft_constructions_2.h 5.1-1/include/CGAL/Cartesian/ft_constructions_2.h
--- 5.0.3-1/include/CGAL/Cartesian/ft_constructions_2.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/ft_constructions_2.h	2020-09-08 12:14:45.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/ft_constructions_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/ft_constructions_2.h $
 // $Id: ft_constructions_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/ft_constructions_3.h 5.1-1/include/CGAL/Cartesian/ft_constructions_3.h
--- 5.0.3-1/include/CGAL/Cartesian/ft_constructions_3.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/ft_constructions_3.h	2020-09-08 12:14:45.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/ft_constructions_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/ft_constructions_3.h $
 // $Id: ft_constructions_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/function_objects.h 5.1-1/include/CGAL/Cartesian/function_objects.h
--- 5.0.3-1/include/CGAL/Cartesian/function_objects.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/function_objects.h	2020-09-08 12:14:45.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/function_objects.h $
-// $Id: function_objects.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/function_objects.h $
+// $Id: function_objects.h 4527b1f 2020-03-26T19:01:49+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -527,6 +527,39 @@ namespace CartesianKernelFunctors {
   };
 
   template <typename K>
+  class Compare_signed_distance_to_line_2
+  {
+    typedef typename K::Point_2   Point_2;
+    typedef typename K::Line_2    Line_2;
+    typedef typename K::Equal_2   Equal_2;
+    typedef typename K::Less_signed_distance_to_line_2 Less_signed_distance_to_line_2;
+
+  public:
+    typedef typename K::Comparison_result     result_type;
+
+    result_type
+    operator()(const Point_2& a, const Point_2& b,
+               const Point_2& c, const Point_2& d) const
+    {
+      CGAL_kernel_precondition_code(Equal_2 equal;)
+          CGAL_kernel_precondition(! equal(a,b));
+      return cmp_signed_dist_to_lineC2( a.x(), a.y(),
+                                        b.x(), b.y(),
+                                        c.x(), c.y(),
+                                        d.x(), d.y());
+    }
+
+    result_type
+    operator()(const Line_2& l, const Point_2& p, const Point_2& q) const
+    {
+      Less_signed_distance_to_line_2 less = K().less_signed_distance_to_line_2_object();
+      if (less(l, p, q)) return SMALLER;
+      if (less(l, q, p)) return LARGER;
+      return EQUAL;
+    }
+  };
+
+  template <typename K>
   class Compare_squared_radius_3
   {
     typedef typename K::Point_3            Point_3;
@@ -3884,7 +3917,7 @@ namespace CartesianKernelFunctors {
     }
   };
 
-  // TODO ...
+
   template <typename K>
   class Less_signed_distance_to_line_2
   {
diff -pruN 5.0.3-1/include/CGAL/Cartesian/Iso_cuboid_3.h 5.1-1/include/CGAL/Cartesian/Iso_cuboid_3.h
--- 5.0.3-1/include/CGAL/Cartesian/Iso_cuboid_3.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/Iso_cuboid_3.h	2020-09-08 12:14:44.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/Iso_cuboid_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/Iso_cuboid_3.h $
 // $Id: Iso_cuboid_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/Iso_rectangle_2.h 5.1-1/include/CGAL/Cartesian/Iso_rectangle_2.h
--- 5.0.3-1/include/CGAL/Cartesian/Iso_rectangle_2.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/Iso_rectangle_2.h	2020-09-08 12:14:44.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/Iso_rectangle_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/Iso_rectangle_2.h $
 // $Id: Iso_rectangle_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/Line_2.h 5.1-1/include/CGAL/Cartesian/Line_2.h
--- 5.0.3-1/include/CGAL/Cartesian/Line_2.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/Line_2.h	2020-09-08 12:14:44.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/Line_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/Line_2.h $
 // $Id: Line_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/Line_3.h 5.1-1/include/CGAL/Cartesian/Line_3.h
--- 5.0.3-1/include/CGAL/Cartesian/Line_3.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/Line_3.h	2020-09-08 12:14:44.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/Line_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/Line_3.h $
 // $Id: Line_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/line_constructions_2.h 5.1-1/include/CGAL/Cartesian/line_constructions_2.h
--- 5.0.3-1/include/CGAL/Cartesian/line_constructions_2.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/line_constructions_2.h	2020-09-08 12:14:45.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/line_constructions_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/line_constructions_2.h $
 // $Id: line_constructions_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/MatrixC33.h 5.1-1/include/CGAL/Cartesian/MatrixC33.h
--- 5.0.3-1/include/CGAL/Cartesian/MatrixC33.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/MatrixC33.h	2020-09-08 12:15:45.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_simplification/include/CGAL/Cartesian/MatrixC33.h $
-// $Id: MatrixC33.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_simplification/include/CGAL/Cartesian/MatrixC33.h $
+// $Id: MatrixC33.h ff09c5d 2019-10-25T16:35:53+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Fernando Cacciola <fernando.cacciola@geometryfactory.com>
@@ -13,11 +13,10 @@
 
 #include <CGAL/license/Surface_mesh_simplification.h>
 
-
-#include <CGAL/Vector_3.h>
 #include <CGAL/determinant.h>
-#include <CGAL/number_utils.h>
 #include <CGAL/Null_matrix.h>
+#include <CGAL/number_utils.h>
+#include <CGAL/Vector_3.h>
 
 #include <boost/optional/optional.hpp>
 
@@ -27,124 +26,116 @@ template <class R_>
 class MatrixC33
 {
 public:
+  typedef R_                                                R;
 
-  typedef R_ R ;
-
-  typedef typename R::FT       RT ;
-  typedef typename R::Vector_3 Vector_3;
+  typedef typename R::FT                                    RT;
+  typedef typename R::Vector_3                              Vector_3;
 
-  MatrixC33 ( Null_matrix )
-   :
-    mR0(NULL_VECTOR)
-   ,mR1(NULL_VECTOR)
-   ,mR2(NULL_VECTOR)
+  MatrixC33(Null_matrix)
+   : mR0(NULL_VECTOR),
+     mR1(NULL_VECTOR),
+     mR2(NULL_VECTOR)
   {}
 
-  MatrixC33 ( RT const& r0x, RT const& r0y, RT const& r0z
-            , RT const& r1x, RT const& r1y, RT const& r1z
-            , RT const& r2x, RT const& r2y, RT const& r2z
-            )
-    :
-     mR0(r0x,r0y,r0z)
-    ,mR1(r1x,r1y,r1z)
-    ,mR2(r2x,r2y,r2z)
+  MatrixC33(const RT& r0x, const RT& r0y, const RT& r0z,
+            const RT& r1x, const RT& r1y, const RT& r1z,
+            const RT& r2x, const RT& r2y, const RT& r2z)
+    : mR0(r0x,r0y,r0z),
+      mR1(r1x,r1y,r1z),
+      mR2(r2x,r2y,r2z)
   {}
 
-  MatrixC33 ( Vector_3 const& r0, Vector_3 const& r1, Vector_3 const& r2 )
-    :
-     mR0(r0)
-    ,mR1(r1)
-    ,mR2(r2)
+  MatrixC33(const Vector_3& r0, const Vector_3& r1, const Vector_3& r2)
+    : mR0(r0),
+      mR1(r1),
+      mR2(r2)
   {}
 
-  Vector_3 const& r0() const { return mR0; }
-  Vector_3 const& r1() const { return mR1; }
-  Vector_3 const& r2() const { return mR2; }
+  const Vector_3& r0() const { return mR0; }
+  const Vector_3& r1() const { return mR1; }
+  const Vector_3& r2() const { return mR2; }
 
   Vector_3& r0() { return mR0; }
   Vector_3& r1() { return mR1; }
   Vector_3& r2() { return mR2; }
 
-  Vector_3 const& operator[] ( int row ) const { return row == 0 ? mR0 : ( row == 1 ? mR1 : mR2 ) ; }
-  Vector_3&       operator[] ( int row )       { return row == 0 ? mR0 : ( row == 1 ? mR1 : mR2 ) ; }
+  const Vector_3& operator[](int row) const { return row == 0 ? mR0 : (row == 1 ? mR1 : mR2); }
+  Vector_3& operator[](int row) { return row == 0 ? mR0 : (row == 1 ? mR1 : mR2); }
 
-  MatrixC33& operator+= ( MatrixC33 const& m )
+  MatrixC33& operator+=(const MatrixC33& m)
   {
-    mR0 = mR0 + m.r0() ;
-    mR1 = mR1 + m.r1() ;
-    mR2 = mR2 + m.r2() ;
-    return *this ;
+    mR0 = mR0 + m.r0();
+    mR1 = mR1 + m.r1();
+    mR2 = mR2 + m.r2();
+    return *this;
   }
 
-  MatrixC33& operator-= ( MatrixC33 const& m )
+  MatrixC33& operator-=(const MatrixC33& m)
   {
-    mR0 = mR0 - m.r0() ;
-    mR1 = mR1 - m.r1() ;
-    mR2 = mR2 - m.r2() ;
-    return *this ;
+    mR0 = mR0 - m.r0();
+    mR1 = mR1 - m.r1();
+    mR2 = mR2 - m.r2();
+    return *this;
   }
 
-  MatrixC33& operator*= ( RT const& c )
+  MatrixC33& operator*=(const RT& c)
   {
-    mR0 = mR0 * c ;
-    mR1 = mR1 * c ;
-    mR2 = mR2 * c ;
-    return *this ;
+    mR0 = mR0 * c;
+    mR1 = mR1 * c;
+    mR2 = mR2 * c;
+    return *this;
   }
 
-  MatrixC33& operator/= ( RT const& c )
+  MatrixC33& operator/=(const RT& c)
   {
-    mR0 = mR0 / c ;
-    mR1 = mR1 / c ;
-    mR2 = mR2 / c ;
-    return *this ;
+    mR0 = mR0 / c;
+    mR1 = mR1 / c;
+    mR2 = mR2 / c;
+    return *this;
   }
 
-  friend MatrixC33 operator+ ( MatrixC33 const& a, MatrixC33 const& b )
+  friend MatrixC33 operator+(const MatrixC33& a, const MatrixC33& b)
   {
-    return MatrixC33(a.r0()+b.r0()
-                    ,a.r1()+b.r1()
-                    ,a.r2()+b.r2()
-                    );
+    return MatrixC33(a.r0() + b.r0(),
+                     a.r1() + b.r1(),
+                     a.r2() + b.r2());
   }
 
-  friend MatrixC33 operator- ( MatrixC33 const& a, MatrixC33 const& b )
+  friend MatrixC33 operator-(const MatrixC33& a, const MatrixC33& b)
   {
-    return MatrixC33(a.r0()-b.r0()
-                    ,a.r1()-b.r1()
-                    ,a.r2()-b.r2()
-                    );
+    return MatrixC33(a.r0() - b.r0(),
+                     a.r1() - b.r1(),
+                     a.r2() - b.r2());
   }
 
-  friend MatrixC33 operator* ( MatrixC33 const& m, RT const& c )
+  friend MatrixC33 operator*(const MatrixC33& m, const RT& c)
   {
-    return MatrixC33(m.r0()*c,m.r1()*c,m.r2()*c);
+    return MatrixC33(m.r0()*c, m.r1()*c, m.r2()*c);
   }
-  friend MatrixC33 operator* ( RT const& c, MatrixC33 const& m )
+  friend MatrixC33 operator*(const RT& c, const MatrixC33& m)
   {
-    return MatrixC33(m.r0()*c,m.r1()*c,m.r2()*c);
+    return MatrixC33(m.r0()*c, m.r1()*c, m.r2()*c);
   }
 
-  friend MatrixC33 operator/ ( MatrixC33 const& m, RT const& c )
+  friend MatrixC33 operator/(const MatrixC33& m, const RT& c)
   {
-    return MatrixC33(m.r0()/c,m.r1()/c,m.r2()/c);
+    return MatrixC33(m.r0()/c, m.r1()/c, m.r2()/c);
   }
 
-  friend Vector_3 operator* ( MatrixC33 const& m, Vector_3 const& v )
+  friend Vector_3 operator*(const MatrixC33& m, const Vector_3& v)
   {
-    return Vector_3(m.r0()*v,m.r1()*v,m.r2()*v);
+    return Vector_3(m.r0()*v, m.r1()*v, m.r2()*v);
   }
-  friend Vector_3 operator* ( Vector_3 const& v, MatrixC33 const& m )
+  friend Vector_3 operator*(const Vector_3& v, const MatrixC33& m)
   {
-    return Vector_3(v*m.r0(),v*m.r1(),v*m.r2());
+    return Vector_3(v*m.r0(), v*m.r1(), v*m.r2());
   }
 
   RT determinant() const
   {
-    return CGAL::determinant(r0().x(),r0().y(),r0().z()
-                            ,r1().x(),r1().y(),r1().z()
-                            ,r2().x(),r2().y(),r2().z()
-                            );
+    return CGAL::determinant(r0().x(), r0().y(), r0().z(),
+                             r1().x(), r1().y(), r1().z(),
+                             r2().x(), r2().y(), r2().z());
   }
 
   MatrixC33& transpose()
@@ -152,38 +143,38 @@ public:
     mR0 = Vector_3(r0().x(),r1().x(),r2().x());
     mR1 = Vector_3(r0().y(),r1().y(),r2().y());
     mR2 = Vector_3(r0().z(),r1().z(),r2().z());
-    return *this ;
+    return *this;
   }
 
 private:
 
-  Vector_3 mR0 ;
-  Vector_3 mR1 ;
-  Vector_3 mR2 ;
-} ;
+  Vector_3 mR0;
+  Vector_3 mR1;
+  Vector_3 mR2;
+};
 
 template<class R>
 inline
-MatrixC33<R> direct_product ( Vector_3<R> const& u, Vector_3<R> const& v )
+MatrixC33<R> direct_product(const Vector_3<R>& u,
+                            const Vector_3<R>& v)
 {
-  return MatrixC33<R>( v * u.x()
-                     , v * u.y()
-                     , v * u.z()
-                     ) ;
+  return MatrixC33<R>(v * u.x(),
+                      v * u.y(),
+                      v * u.z());
 }
 
 template<class R>
-MatrixC33<R> transposed_matrix ( MatrixC33<R> const& m )
+MatrixC33<R> transposed_matrix(const MatrixC33<R>& m)
 {
-  MatrixC33<R> copy = m ;
+  MatrixC33<R> copy = m;
   copy.Transpose();
-  return copy ;
+  return copy;
 }
 
 template<class R>
-MatrixC33<R> cofactors_matrix ( MatrixC33<R> const& m )
+MatrixC33<R> cofactors_matrix(const MatrixC33<R>& m)
 {
-  typedef typename R::RT RT ;
+  typedef typename R::RT RT;
 
   RT c00 =  determinant(m.r1().y(),m.r1().z(),m.r2().y(),m.r2().z());
   RT c01 = -determinant(m.r1().x(),m.r1().z(),m.r2().x(),m.r2().z());
@@ -197,32 +188,29 @@ MatrixC33<R> cofactors_matrix ( MatrixC3
   RT c21 = -determinant(m.r0().x(),m.r0().z(),m.r1().x(),m.r1().z());
   RT c22 =  determinant(m.r0().x(),m.r0().y(),m.r1().x(),m.r1().y());
 
-  return MatrixC33<R>(c00,c01,c02
-                     ,c10,c11,c12
-                     ,c20,c21,c22
-                     );
+  return MatrixC33<R>(c00,c01,c02,
+                      c10,c11,c12,
+                      c20,c21,c22);
 }
 
 template<class R>
-MatrixC33<R> adjoint_matrix ( MatrixC33<R> const& m )
+MatrixC33<R> adjoint_matrix(const MatrixC33<R>& m)
 {
-  return cofactors_matrix(m).transpose()  ;
+  return cofactors_matrix(m).transpose();
 }
 
 template<class R>
-boost::optional< MatrixC33<R> > inverse_matrix ( MatrixC33<R> const& m )
+boost::optional< MatrixC33<R> > inverse_matrix(const MatrixC33<R>& m)
 {
-  typedef typename R::RT RT ;
-
-  typedef MatrixC33<R> Matrix ;
-
-  typedef boost::optional<Matrix> result_type ;
+  typedef typename R::RT                                        RT;
+  typedef MatrixC33<R>                                          Matrix;
+  typedef boost::optional<Matrix>                               result_type;
 
-  result_type rInverse ;
+  result_type rInverse;
 
   RT det = m.determinant();
 
-  if ( ! CGAL_NTS is_zero(det) )
+  if(! CGAL_NTS is_zero(det))
   {
     RT c00 = (m.r1().y()*m.r2().z() - m.r1().z()*m.r2().y()) / det;
     RT c01 = (m.r2().y()*m.r0().z() - m.r0().y()*m.r2().z()) / det;
@@ -236,17 +224,15 @@ boost::optional< MatrixC33<R> > inverse_
     RT c21 = (m.r2().x()*m.r0().y() - m.r0().x()*m.r2().y()) / det;
     RT c22 = (m.r0().x()*m.r1().y() - m.r0().y()*m.r1().x()) / det;
 
-    rInverse = result_type( Matrix(c00,c01,c02
-                                  ,c10,c11,c12
-                                  ,c20,c21,c22
-                                  )
-                          ) ;
+    rInverse = result_type(Matrix(c00,c01,c02,
+                                  c10,c11,c12,
+                                  c20,c21,c22));
   }
 
-  return rInverse ;
+  return rInverse;
 }
 
-} //namespace CGAL
+} // namespace CGAL
 
 #endif // CGAL_CARTESIAN_MATRIXC33_H //
 // EOF //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/Plane_3.h 5.1-1/include/CGAL/Cartesian/Plane_3.h
--- 5.0.3-1/include/CGAL/Cartesian/Plane_3.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/Plane_3.h	2020-09-08 12:14:44.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/Plane_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/Plane_3.h $
 // $Id: Plane_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/plane_constructions_3.h 5.1-1/include/CGAL/Cartesian/plane_constructions_3.h
--- 5.0.3-1/include/CGAL/Cartesian/plane_constructions_3.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/plane_constructions_3.h	2020-09-08 12:14:45.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/plane_constructions_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/plane_constructions_3.h $
 // $Id: plane_constructions_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/Point_2.h 5.1-1/include/CGAL/Cartesian/Point_2.h
--- 5.0.3-1/include/CGAL/Cartesian/Point_2.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/Point_2.h	2020-09-08 12:14:44.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/Point_2.h $
-// $Id: Point_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/Point_2.h $
+// $Id: Point_2.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -50,6 +50,15 @@ public:
   PointC2(const FT &hx, const FT &hy, const FT &hw)
     : base(hx, hy, hw) {}
 
+  friend void swap(Self& a, Self& b)
+#ifdef __cpp_lib_is_swappable
+    noexcept(std::is_nothrow_swappable_v<Vector_2_>)
+#endif
+  {
+    using std::swap;
+    swap(a.base, b.base);
+  }
+
   const FT& x() const
   {
       return base.x();
diff -pruN 5.0.3-1/include/CGAL/Cartesian/Point_3.h 5.1-1/include/CGAL/Cartesian/Point_3.h
--- 5.0.3-1/include/CGAL/Cartesian/Point_3.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/Point_3.h	2020-09-08 12:14:44.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/Point_3.h $
-// $Id: Point_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/Point_3.h $
+// $Id: Point_3.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -24,6 +24,7 @@ namespace CGAL {
 template < class R_ >
 class PointC3
 {
+  typedef PointC3<R_>                       Self;
   typedef typename R_::Vector_3             Vector_3;
   typedef typename R_::Point_3              Point_3;
   typedef typename R_::Aff_transformation_3 Aff_transformation_3;
@@ -47,6 +48,15 @@ public:
   PointC3(const FT &x, const FT &y, const FT &z, const FT &w)
     : base(x, y, z, w) {}
 
+  friend void swap(Self& a, Self& b)
+#ifdef __cpp_lib_is_swappable
+    noexcept(std::is_nothrow_swappable_v<Vector_3>)
+#endif
+  {
+    using std::swap;
+    swap(a.base, b.base);
+  }
+
   const FT & x() const
   {
       return base.x();
diff -pruN 5.0.3-1/include/CGAL/Cartesian/point_constructions_2.h 5.1-1/include/CGAL/Cartesian/point_constructions_2.h
--- 5.0.3-1/include/CGAL/Cartesian/point_constructions_2.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/point_constructions_2.h	2020-09-08 12:14:45.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/point_constructions_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/point_constructions_2.h $
 // $Id: point_constructions_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/point_constructions_3.h 5.1-1/include/CGAL/Cartesian/point_constructions_3.h
--- 5.0.3-1/include/CGAL/Cartesian/point_constructions_3.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/point_constructions_3.h	2020-09-08 12:14:45.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/point_constructions_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/point_constructions_3.h $
 // $Id: point_constructions_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/predicates_on_directions_2.h 5.1-1/include/CGAL/Cartesian/predicates_on_directions_2.h
--- 5.0.3-1/include/CGAL/Cartesian/predicates_on_directions_2.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/predicates_on_directions_2.h	2020-09-08 12:14:45.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/predicates_on_directions_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/predicates_on_directions_2.h $
 // $Id: predicates_on_directions_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/predicates_on_planes_3.h 5.1-1/include/CGAL/Cartesian/predicates_on_planes_3.h
--- 5.0.3-1/include/CGAL/Cartesian/predicates_on_planes_3.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/predicates_on_planes_3.h	2020-09-08 12:14:45.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/predicates_on_planes_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/predicates_on_planes_3.h $
 // $Id: predicates_on_planes_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/predicates_on_points_2.h 5.1-1/include/CGAL/Cartesian/predicates_on_points_2.h
--- 5.0.3-1/include/CGAL/Cartesian/predicates_on_points_2.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/predicates_on_points_2.h	2020-09-08 12:14:45.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/predicates_on_points_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/predicates_on_points_2.h $
 // $Id: predicates_on_points_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/predicates_on_points_3.h 5.1-1/include/CGAL/Cartesian/predicates_on_points_3.h
--- 5.0.3-1/include/CGAL/Cartesian/predicates_on_points_3.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/predicates_on_points_3.h	2020-09-08 12:14:45.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/predicates_on_points_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/predicates_on_points_3.h $
 // $Id: predicates_on_points_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/Ray_2.h 5.1-1/include/CGAL/Cartesian/Ray_2.h
--- 5.0.3-1/include/CGAL/Cartesian/Ray_2.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/Ray_2.h	2020-09-08 12:14:44.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/Ray_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/Ray_2.h $
 // $Id: Ray_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/Ray_3.h 5.1-1/include/CGAL/Cartesian/Ray_3.h
--- 5.0.3-1/include/CGAL/Cartesian/Ray_3.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/Ray_3.h	2020-09-08 12:14:44.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/Ray_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/Ray_3.h $
 // $Id: Ray_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/Reflection_rep_2.h 5.1-1/include/CGAL/Cartesian/Reflection_rep_2.h
--- 5.0.3-1/include/CGAL/Cartesian/Reflection_rep_2.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/Reflection_rep_2.h	2020-09-08 12:14:44.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/Reflection_rep_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/Reflection_rep_2.h $
 // $Id: Reflection_rep_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/Rotation_rep_2.h 5.1-1/include/CGAL/Cartesian/Rotation_rep_2.h
--- 5.0.3-1/include/CGAL/Cartesian/Rotation_rep_2.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/Rotation_rep_2.h	2020-09-08 12:14:44.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/Rotation_rep_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/Rotation_rep_2.h $
 // $Id: Rotation_rep_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/Scaling_rep_2.h 5.1-1/include/CGAL/Cartesian/Scaling_rep_2.h
--- 5.0.3-1/include/CGAL/Cartesian/Scaling_rep_2.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/Scaling_rep_2.h	2020-09-08 12:14:44.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/Scaling_rep_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/Scaling_rep_2.h $
 // $Id: Scaling_rep_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/Scaling_rep_3.h 5.1-1/include/CGAL/Cartesian/Scaling_rep_3.h
--- 5.0.3-1/include/CGAL/Cartesian/Scaling_rep_3.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/Scaling_rep_3.h	2020-09-08 12:14:44.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/Scaling_rep_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/Scaling_rep_3.h $
 // $Id: Scaling_rep_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/Segment_2.h 5.1-1/include/CGAL/Cartesian/Segment_2.h
--- 5.0.3-1/include/CGAL/Cartesian/Segment_2.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/Segment_2.h	2020-09-08 12:14:44.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/Segment_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/Segment_2.h $
 // $Id: Segment_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/Segment_3.h 5.1-1/include/CGAL/Cartesian/Segment_3.h
--- 5.0.3-1/include/CGAL/Cartesian/Segment_3.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/Segment_3.h	2020-09-08 12:14:44.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/Segment_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/Segment_3.h $
 // $Id: Segment_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/solve_3.h 5.1-1/include/CGAL/Cartesian/solve_3.h
--- 5.0.3-1/include/CGAL/Cartesian/solve_3.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/solve_3.h	2020-09-08 12:14:45.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/solve_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/solve_3.h $
 // $Id: solve_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/Sphere_3.h 5.1-1/include/CGAL/Cartesian/Sphere_3.h
--- 5.0.3-1/include/CGAL/Cartesian/Sphere_3.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/Sphere_3.h	2020-09-08 12:14:44.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/Sphere_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/Sphere_3.h $
 // $Id: Sphere_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/Tetrahedron_3.h 5.1-1/include/CGAL/Cartesian/Tetrahedron_3.h
--- 5.0.3-1/include/CGAL/Cartesian/Tetrahedron_3.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/Tetrahedron_3.h	2020-09-08 12:14:44.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/Tetrahedron_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/Tetrahedron_3.h $
 // $Id: Tetrahedron_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/Translation_rep_2.h 5.1-1/include/CGAL/Cartesian/Translation_rep_2.h
--- 5.0.3-1/include/CGAL/Cartesian/Translation_rep_2.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/Translation_rep_2.h	2020-09-08 12:14:44.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/Translation_rep_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/Translation_rep_2.h $
 // $Id: Translation_rep_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/Translation_rep_3.h 5.1-1/include/CGAL/Cartesian/Translation_rep_3.h
--- 5.0.3-1/include/CGAL/Cartesian/Translation_rep_3.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/Translation_rep_3.h	2020-09-08 12:14:44.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/Translation_rep_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/Translation_rep_3.h $
 // $Id: Translation_rep_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/Triangle_2.h 5.1-1/include/CGAL/Cartesian/Triangle_2.h
--- 5.0.3-1/include/CGAL/Cartesian/Triangle_2.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/Triangle_2.h	2020-09-08 12:14:44.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/Triangle_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/Triangle_2.h $
 // $Id: Triangle_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/Triangle_3.h 5.1-1/include/CGAL/Cartesian/Triangle_3.h
--- 5.0.3-1/include/CGAL/Cartesian/Triangle_3.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/Triangle_3.h	2020-09-08 12:14:44.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/Triangle_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/Triangle_3.h $
 // $Id: Triangle_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/Vector_2.h 5.1-1/include/CGAL/Cartesian/Vector_2.h
--- 5.0.3-1/include/CGAL/Cartesian/Vector_2.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/Vector_2.h	2020-09-08 12:14:45.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/Vector_2.h $
-// $Id: Vector_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/Vector_2.h $
+// $Id: Vector_2.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -27,6 +27,7 @@ namespace CGAL {
 template < class R_ >
 class VectorC2
 {
+  typedef VectorC2<R_>                      Self;
   typedef typename R_::FT                   FT;
   typedef typename R_::Point_2              Point_2;
   typedef typename R_::Vector_2             Vector_2;
@@ -55,6 +56,15 @@ public:
     : base( hw != FT(1) ? CGAL::make_array<FT>(hx/hw, hy/hw)
                         : CGAL::make_array(hx, hy) ) {}
 
+  friend void swap(Self& a, Self& b)
+#ifdef __cpp_lib_is_swappable
+    noexcept(std::is_nothrow_swappable_v<Base>)
+#endif
+  {
+    using std::swap;
+    swap(a.base, b.base);
+  }
+
   const FT & x() const
   {
       return CGAL::get_pointee_or_identity(base)[0];
diff -pruN 5.0.3-1/include/CGAL/Cartesian/Vector_3.h 5.1-1/include/CGAL/Cartesian/Vector_3.h
--- 5.0.3-1/include/CGAL/Cartesian/Vector_3.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/Vector_3.h	2020-09-08 12:14:45.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/Vector_3.h $
-// $Id: Vector_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/Vector_3.h $
+// $Id: Vector_3.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -27,6 +27,7 @@ template < class R_ >
 class VectorC3
 {
 // https://doc.cgal.org/latest/Manual/devman_code_format.html#secprogramming_conventions
+  typedef VectorC3<R_>                      Self;
   typedef typename R_::FT                   FT_;
   typedef typename R_::Point_3              Point_3;
   typedef typename R_::Vector_3             Vector_3;
@@ -70,6 +71,15 @@ public:
     : base( w != FT_(1) ? CGAL::make_array<FT_>(x/w, y/w, z/w)
                        : CGAL::make_array(x, y, z) ) {}
 
+  friend void swap(Self& a, Self& b)
+#ifdef __cpp_lib_is_swappable
+    noexcept(std::is_nothrow_swappable_v<Base>)
+#endif
+  {
+    using std::swap;
+    swap(a.base, b.base);
+  }
+
   const FT_ & x() const
   {
       return get_pointee_or_identity(base)[0];
diff -pruN 5.0.3-1/include/CGAL/Cartesian/Weighted_point_2.h 5.1-1/include/CGAL/Cartesian/Weighted_point_2.h
--- 5.0.3-1/include/CGAL/Cartesian/Weighted_point_2.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/Weighted_point_2.h	2020-09-08 12:14:45.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/Weighted_point_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/Weighted_point_2.h $
 // $Id: Weighted_point_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian/Weighted_point_3.h 5.1-1/include/CGAL/Cartesian/Weighted_point_3.h
--- 5.0.3-1/include/CGAL/Cartesian/Weighted_point_3.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian/Weighted_point_3.h	2020-09-08 12:14:45.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian/Weighted_point_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian/Weighted_point_3.h $
 // $Id: Weighted_point_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian_converter_fwd.h 5.1-1/include/CGAL/Cartesian_converter_fwd.h
--- 5.0.3-1/include/CGAL/Cartesian_converter_fwd.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian_converter_fwd.h	2020-09-08 12:14:54.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/Cartesian_converter_fwd.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/Cartesian_converter_fwd.h $
 // $Id: Cartesian_converter_fwd.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian_converter.h 5.1-1/include/CGAL/Cartesian_converter.h
--- 5.0.3-1/include/CGAL/Cartesian_converter.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian_converter.h	2020-09-08 12:14:45.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian_converter.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian_converter.h $
 // $Id: Cartesian_converter.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian_d.h 5.1-1/include/CGAL/Cartesian_d.h
--- 5.0.3-1/include/CGAL/Cartesian_d.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian_d.h	2020-09-08 12:15:03.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Cartesian_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Cartesian_d.h $
 // $Id: Cartesian_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian.h 5.1-1/include/CGAL/Cartesian.h
--- 5.0.3-1/include/CGAL/Cartesian.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian.h	2020-09-08 12:14:44.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Cartesian.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Cartesian.h $
 // $Id: Cartesian.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/cartesian_homogeneous_conversion.h 5.1-1/include/CGAL/cartesian_homogeneous_conversion.h
--- 5.0.3-1/include/CGAL/cartesian_homogeneous_conversion.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/cartesian_homogeneous_conversion.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/cartesian_homogeneous_conversion.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/cartesian_homogeneous_conversion.h $
 // $Id: cartesian_homogeneous_conversion.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cartesian_matrix.h 5.1-1/include/CGAL/Cartesian_matrix.h
--- 5.0.3-1/include/CGAL/Cartesian_matrix.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Cartesian_matrix.h	2020-09-08 12:15:06.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Matrix_search/include/CGAL/Cartesian_matrix.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Matrix_search/include/CGAL/Cartesian_matrix.h $
 // $Id: Cartesian_matrix.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/CC_safe_handle.h 5.1-1/include/CGAL/CC_safe_handle.h
--- 5.0.3-1/include/CGAL/CC_safe_handle.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/CC_safe_handle.h	2020-09-08 12:15:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/CC_safe_handle.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/CC_safe_handle.h $
 // $Id: CC_safe_handle.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cell_attribute.h 5.1-1/include/CGAL/Cell_attribute.h
--- 5.0.3-1/include/CGAL/Cell_attribute.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Cell_attribute.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Combinatorial_map/include/CGAL/Cell_attribute.h $
-// $Id: Cell_attribute.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Combinatorial_map/include/CGAL/Cell_attribute.h $
+// $Id: Cell_attribute.h d1a323c 2020-03-26T19:24:14+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
@@ -197,8 +197,8 @@ namespace CGAL {
 
     void * for_compact_container() const
     { return vp; }
-    void * & for_compact_container()
-    { return vp; }
+    void for_compact_container(void *p)
+    { vp = p; }
 
   private:
     /// Reference counting: the number of darts linked to this cell.
@@ -310,8 +310,8 @@ namespace CGAL {
 
     void * for_compact_container() const
     { return mdart.for_compact_container(); }
-    void * & for_compact_container()
-    { return mdart.for_compact_container(); }
+    void for_compact_container(void *p)
+    { mdart.for_compact_container(p); }
 
   private:
     /// The dart handle associated with the cell.
diff -pruN 5.0.3-1/include/CGAL/Cell_attribute_with_id.h 5.1-1/include/CGAL/Cell_attribute_with_id.h
--- 5.0.3-1/include/CGAL/Cell_attribute_with_id.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Cell_attribute_with_id.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Combinatorial_map/include/CGAL/Cell_attribute_with_id.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Combinatorial_map/include/CGAL/Cell_attribute_with_id.h $
 // $Id: Cell_attribute_with_id.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cell_attribute_with_point_and_id.h 5.1-1/include/CGAL/Cell_attribute_with_point_and_id.h
--- 5.0.3-1/include/CGAL/Cell_attribute_with_point_and_id.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Cell_attribute_with_point_and_id.h	2020-09-08 12:15:05.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Linear_cell_complex/include/CGAL/Cell_attribute_with_point_and_id.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Linear_cell_complex/include/CGAL/Cell_attribute_with_point_and_id.h $
 // $Id: Cell_attribute_with_point_and_id.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cell_attribute_with_point.h 5.1-1/include/CGAL/Cell_attribute_with_point.h
--- 5.0.3-1/include/CGAL/Cell_attribute_with_point.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Cell_attribute_with_point.h	2020-09-08 12:15:05.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Linear_cell_complex/include/CGAL/Cell_attribute_with_point.h $
-// $Id: Cell_attribute_with_point.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Linear_cell_complex/include/CGAL/Cell_attribute_with_point.h $
+// $Id: Cell_attribute_with_point.h 9e250c0 2020-02-12T10:33:57+01:00 Guillaume Damiand
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
@@ -25,11 +25,11 @@ namespace CGAL {
   class Point_for_cell
   {
   public:
-    /// Contructor without parameter.
+    /// Constructor without parameter.
     Point_for_cell()
     {}
 
-    /// Contructor with a point in parameter.
+    /// Constructor with a point in parameter.
     Point_for_cell(const Point& apoint) : mpoint(apoint)
     {}
 
@@ -87,15 +87,15 @@ namespace CGAL {
     { return !operator==(other); }
 
   protected:
-    /// Default contructor.
+    /// Default constructor.
     Cell_attribute_with_point()
     {}
 
-    /// Contructor with a point in parameter.
+    /// Constructor with a point in parameter.
     Cell_attribute_with_point(const Point& apoint) : Base2(apoint)
     {}
 
-    /// Contructor with a point and an attribute in parameters.
+    /// Constructor with a point and an attribute in parameters.
     Cell_attribute_with_point(const Point& apoint, const Info& ainfo) :
       Base1(ainfo),
       Base2(apoint)
@@ -144,11 +144,11 @@ namespace CGAL {
     { return false; }
 
   protected:
-    /// Default contructor.
+    /// Default constructor.
     Cell_attribute_with_point()
     {}
 
-    /// Contructor with a point in parameter.
+    /// Constructor with a point in parameter.
     Cell_attribute_with_point(const Point& apoint) : Base2(apoint)
     {}
   };
diff -pruN 5.0.3-1/include/CGAL/Cell_const_iterators.h 5.1-1/include/CGAL/Cell_const_iterators.h
--- 5.0.3-1/include/CGAL/Cell_const_iterators.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Cell_const_iterators.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Combinatorial_map/include/CGAL/Cell_const_iterators.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Combinatorial_map/include/CGAL/Cell_const_iterators.h $
 // $Id: Cell_const_iterators.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cell_iterators.h 5.1-1/include/CGAL/Cell_iterators.h
--- 5.0.3-1/include/CGAL/Cell_iterators.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Cell_iterators.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Combinatorial_map/include/CGAL/Cell_iterators.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Combinatorial_map/include/CGAL/Cell_iterators.h $
 // $Id: Cell_iterators.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/centroid.h 5.1-1/include/CGAL/centroid.h
--- 5.0.3-1/include/CGAL/centroid.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/centroid.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Principal_component_analysis_LGPL/include/CGAL/centroid.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Principal_component_analysis_LGPL/include/CGAL/centroid.h $
 // $Id: centroid.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/certified_numeric_predicates.h 5.1-1/include/CGAL/certified_numeric_predicates.h
--- 5.0.3-1/include/CGAL/certified_numeric_predicates.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/certified_numeric_predicates.h	2020-09-08 12:15:41.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Straight_skeleton_2/include/CGAL/certified_numeric_predicates.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Straight_skeleton_2/include/CGAL/certified_numeric_predicates.h $
 // $Id: certified_numeric_predicates.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/certified_quotient_predicates.h 5.1-1/include/CGAL/certified_quotient_predicates.h
--- 5.0.3-1/include/CGAL/certified_quotient_predicates.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/certified_quotient_predicates.h	2020-09-08 12:15:41.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Straight_skeleton_2/include/CGAL/certified_quotient_predicates.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Straight_skeleton_2/include/CGAL/certified_quotient_predicates.h $
 // $Id: certified_quotient_predicates.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/CGAL_Ipelet_base.h 5.1-1/include/CGAL/CGAL_Ipelet_base.h
--- 5.0.3-1/include/CGAL/CGAL_Ipelet_base.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/CGAL_Ipelet_base.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ipelets/include/CGAL/CGAL_Ipelet_base.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ipelets/include/CGAL/CGAL_Ipelet_base.h $
 // $Id: CGAL_Ipelet_base.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/CGAL_Ipelet_base_v6.h 5.1-1/include/CGAL/CGAL_Ipelet_base_v6.h
--- 5.0.3-1/include/CGAL/CGAL_Ipelet_base_v6.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/CGAL_Ipelet_base_v6.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ipelets/include/CGAL/CGAL_Ipelet_base_v6.h $
-// $Id: CGAL_Ipelet_base_v6.h 6a51a30 2020-04-22T11:02:16+02:00 Maxime Gimeno
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ipelets/include/CGAL/CGAL_Ipelet_base_v6.h $
+// $Id: CGAL_Ipelet_base_v6.h 9bf61b7 2020-04-22T11:02:16+02:00 Maxime Gimeno
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
diff -pruN 5.0.3-1/include/CGAL/CGAL_Ipelet_base_v7.h 5.1-1/include/CGAL/CGAL_Ipelet_base_v7.h
--- 5.0.3-1/include/CGAL/CGAL_Ipelet_base_v7.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/CGAL_Ipelet_base_v7.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ipelets/include/CGAL/CGAL_Ipelet_base_v7.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ipelets/include/CGAL/CGAL_Ipelet_base_v7.h $
 // $Id: CGAL_Ipelet_base_v7.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/ch_akl_toussaint.h 5.1-1/include/CGAL/ch_akl_toussaint.h
--- 5.0.3-1/include/CGAL/ch_akl_toussaint.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/ch_akl_toussaint.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_2/include/CGAL/ch_akl_toussaint.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_2/include/CGAL/ch_akl_toussaint.h $
 // $Id: ch_akl_toussaint.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/ch_bykat.h 5.1-1/include/CGAL/ch_bykat.h
--- 5.0.3-1/include/CGAL/ch_bykat.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/ch_bykat.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_2/include/CGAL/ch_bykat.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_2/include/CGAL/ch_bykat.h $
 // $Id: ch_bykat.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/ch_eddy.h 5.1-1/include/CGAL/ch_eddy.h
--- 5.0.3-1/include/CGAL/ch_eddy.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/ch_eddy.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_2/include/CGAL/ch_eddy.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_2/include/CGAL/ch_eddy.h $
 // $Id: ch_eddy.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/ch_function_objects_2.h 5.1-1/include/CGAL/ch_function_objects_2.h
--- 5.0.3-1/include/CGAL/ch_function_objects_2.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/ch_function_objects_2.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_2/include/CGAL/ch_function_objects_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_2/include/CGAL/ch_function_objects_2.h $
 // $Id: ch_function_objects_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/ch_graham_andrew.h 5.1-1/include/CGAL/ch_graham_andrew.h
--- 5.0.3-1/include/CGAL/ch_graham_andrew.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/ch_graham_andrew.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_2/include/CGAL/ch_graham_andrew.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_2/include/CGAL/ch_graham_andrew.h $
 // $Id: ch_graham_andrew.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Chinese_remainder_traits.h 5.1-1/include/CGAL/Chinese_remainder_traits.h
--- 5.0.3-1/include/CGAL/Chinese_remainder_traits.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/Chinese_remainder_traits.h	2020-09-08 12:14:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_foundations/include/CGAL/Chinese_remainder_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_foundations/include/CGAL/Chinese_remainder_traits.h $
 // $Id: Chinese_remainder_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/ch_jarvis.h 5.1-1/include/CGAL/ch_jarvis.h
--- 5.0.3-1/include/CGAL/ch_jarvis.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/ch_jarvis.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_2/include/CGAL/ch_jarvis.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_2/include/CGAL/ch_jarvis.h $
 // $Id: ch_jarvis.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/ch_melkman.h 5.1-1/include/CGAL/ch_melkman.h
--- 5.0.3-1/include/CGAL/ch_melkman.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/ch_melkman.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_2/include/CGAL/ch_melkman.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_2/include/CGAL/ch_melkman.h $
 // $Id: ch_melkman.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/ch_selected_extreme_points_2.h 5.1-1/include/CGAL/ch_selected_extreme_points_2.h
--- 5.0.3-1/include/CGAL/ch_selected_extreme_points_2.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/ch_selected_extreme_points_2.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_2/include/CGAL/ch_selected_extreme_points_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_2/include/CGAL/ch_selected_extreme_points_2.h $
 // $Id: ch_selected_extreme_points_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circle_2.h 5.1-1/include/CGAL/Circle_2.h
--- 5.0.3-1/include/CGAL/Circle_2.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Circle_2.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Circle_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Circle_2.h $
 // $Id: Circle_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circle_3.h 5.1-1/include/CGAL/Circle_3.h
--- 5.0.3-1/include/CGAL/Circle_3.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Circle_3.h	2020-09-08 12:15:01.000000000 +0000
@@ -9,7 +9,7 @@
 // and a STREP (FET Open) Project under Contract No  IST-006413
 // (ACS -- Algorithms for Complex Shapes)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Circle_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Circle_3.h $
 // $Id: Circle_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circle_type.h 5.1-1/include/CGAL/Circle_type.h
--- 5.0.3-1/include/CGAL/Circle_type.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Circle_type.h	2020-09-08 12:14:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_3/include/CGAL/Circle_type.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_3/include/CGAL/Circle_type.h $
 // $Id: Circle_type.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_arc_2.h 5.1-1/include/CGAL/Circular_arc_2.h
--- 5.0.3-1/include/CGAL/Circular_arc_2.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_arc_2.h	2020-09-08 12:14:46.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_2/include/CGAL/Circular_arc_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_2/include/CGAL/Circular_arc_2.h $
 // $Id: Circular_arc_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_arc_3.h 5.1-1/include/CGAL/Circular_arc_3.h
--- 5.0.3-1/include/CGAL/Circular_arc_3.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_arc_3.h	2020-09-08 12:14:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_3/include/CGAL/Circular_arc_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_3/include/CGAL/Circular_arc_3.h $
 // $Id: Circular_arc_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_arc_point_2.h 5.1-1/include/CGAL/Circular_arc_point_2.h
--- 5.0.3-1/include/CGAL/Circular_arc_point_2.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_arc_point_2.h	2020-09-08 12:14:46.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_2/include/CGAL/Circular_arc_point_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_2/include/CGAL/Circular_arc_point_2.h $
 // $Id: Circular_arc_point_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_arc_point_3.h 5.1-1/include/CGAL/Circular_arc_point_3.h
--- 5.0.3-1/include/CGAL/Circular_arc_point_3.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_arc_point_3.h	2020-09-08 12:14:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_3/include/CGAL/Circular_arc_point_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_3/include/CGAL/Circular_arc_point_3.h $
 // $Id: Circular_arc_point_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_kernel_2/Circular_arc_2.h 5.1-1/include/CGAL/Circular_kernel_2/Circular_arc_2.h
--- 5.0.3-1/include/CGAL/Circular_kernel_2/Circular_arc_2.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_kernel_2/Circular_arc_2.h	2020-09-08 12:14:46.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_2/include/CGAL/Circular_kernel_2/Circular_arc_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_2/include/CGAL/Circular_kernel_2/Circular_arc_2.h $
 // $Id: Circular_arc_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_kernel_2/Circular_arc_point_2.h 5.1-1/include/CGAL/Circular_kernel_2/Circular_arc_point_2.h
--- 5.0.3-1/include/CGAL/Circular_kernel_2/Circular_arc_point_2.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_kernel_2/Circular_arc_point_2.h	2020-09-08 12:14:46.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_2/include/CGAL/Circular_kernel_2/Circular_arc_point_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_2/include/CGAL/Circular_kernel_2/Circular_arc_point_2.h $
 // $Id: Circular_arc_point_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_kernel_2/function_objects_on_circle_2.h 5.1-1/include/CGAL/Circular_kernel_2/function_objects_on_circle_2.h
--- 5.0.3-1/include/CGAL/Circular_kernel_2/function_objects_on_circle_2.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_kernel_2/function_objects_on_circle_2.h	2020-09-08 12:14:46.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_2/include/CGAL/Circular_kernel_2/function_objects_on_circle_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_2/include/CGAL/Circular_kernel_2/function_objects_on_circle_2.h $
 // $Id: function_objects_on_circle_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_kernel_2/function_objects_on_line_2.h 5.1-1/include/CGAL/Circular_kernel_2/function_objects_on_line_2.h
--- 5.0.3-1/include/CGAL/Circular_kernel_2/function_objects_on_line_2.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_kernel_2/function_objects_on_line_2.h	2020-09-08 12:14:46.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_2/include/CGAL/Circular_kernel_2/function_objects_on_line_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_2/include/CGAL/Circular_kernel_2/function_objects_on_line_2.h $
 // $Id: function_objects_on_line_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_kernel_2/function_objects_polynomial_circular.h 5.1-1/include/CGAL/Circular_kernel_2/function_objects_polynomial_circular.h
--- 5.0.3-1/include/CGAL/Circular_kernel_2/function_objects_polynomial_circular.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_kernel_2/function_objects_polynomial_circular.h	2020-09-08 12:14:46.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_2/include/CGAL/Circular_kernel_2/function_objects_polynomial_circular.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_2/include/CGAL/Circular_kernel_2/function_objects_polynomial_circular.h $
 // $Id: function_objects_polynomial_circular.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_kernel_2/interface_macros.h 5.1-1/include/CGAL/Circular_kernel_2/interface_macros.h
--- 5.0.3-1/include/CGAL/Circular_kernel_2/interface_macros.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_kernel_2/interface_macros.h	2020-09-08 12:14:46.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_2/include/CGAL/Circular_kernel_2/interface_macros.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_2/include/CGAL/Circular_kernel_2/interface_macros.h $
 // $Id: interface_macros.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_kernel_2/internal_functions_on_circle_2.h 5.1-1/include/CGAL/Circular_kernel_2/internal_functions_on_circle_2.h
--- 5.0.3-1/include/CGAL/Circular_kernel_2/internal_functions_on_circle_2.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_kernel_2/internal_functions_on_circle_2.h	2020-09-08 12:14:46.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_2/include/CGAL/Circular_kernel_2/internal_functions_on_circle_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_2/include/CGAL/Circular_kernel_2/internal_functions_on_circle_2.h $
 // $Id: internal_functions_on_circle_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_kernel_2/internal_functions_on_circular_arc_2.h 5.1-1/include/CGAL/Circular_kernel_2/internal_functions_on_circular_arc_2.h
--- 5.0.3-1/include/CGAL/Circular_kernel_2/internal_functions_on_circular_arc_2.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_kernel_2/internal_functions_on_circular_arc_2.h	2020-09-08 12:14:46.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_2/include/CGAL/Circular_kernel_2/internal_functions_on_circular_arc_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_2/include/CGAL/Circular_kernel_2/internal_functions_on_circular_arc_2.h $
 // $Id: internal_functions_on_circular_arc_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_kernel_2/internal_functions_on_line_2.h 5.1-1/include/CGAL/Circular_kernel_2/internal_functions_on_line_2.h
--- 5.0.3-1/include/CGAL/Circular_kernel_2/internal_functions_on_line_2.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_kernel_2/internal_functions_on_line_2.h	2020-09-08 12:14:46.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_2/include/CGAL/Circular_kernel_2/internal_functions_on_line_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_2/include/CGAL/Circular_kernel_2/internal_functions_on_line_2.h $
 // $Id: internal_functions_on_line_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_kernel_2/internal_functions_on_line_arc_2.h 5.1-1/include/CGAL/Circular_kernel_2/internal_functions_on_line_arc_2.h
--- 5.0.3-1/include/CGAL/Circular_kernel_2/internal_functions_on_line_arc_2.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_kernel_2/internal_functions_on_line_arc_2.h	2020-09-08 12:14:46.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_2/include/CGAL/Circular_kernel_2/internal_functions_on_line_arc_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_2/include/CGAL/Circular_kernel_2/internal_functions_on_line_arc_2.h $
 // $Id: internal_functions_on_line_arc_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_kernel_2/intersection_line_2_circle_2_map.h 5.1-1/include/CGAL/Circular_kernel_2/intersection_line_2_circle_2_map.h
--- 5.0.3-1/include/CGAL/Circular_kernel_2/intersection_line_2_circle_2_map.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_kernel_2/intersection_line_2_circle_2_map.h	2020-09-08 12:14:46.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_2/include/CGAL/Circular_kernel_2/intersection_line_2_circle_2_map.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_2/include/CGAL/Circular_kernel_2/intersection_line_2_circle_2_map.h $
 // $Id: intersection_line_2_circle_2_map.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_kernel_2/Intersection_traits.h 5.1-1/include/CGAL/Circular_kernel_2/Intersection_traits.h
--- 5.0.3-1/include/CGAL/Circular_kernel_2/Intersection_traits.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_kernel_2/Intersection_traits.h	2020-09-08 12:14:46.000000000 +0000
@@ -3,11 +3,11 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_2/include/CGAL/Circular_kernel_2/Intersection_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_2/include/CGAL/Circular_kernel_2/Intersection_traits.h $
 // $Id: Intersection_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_2/include/CGAL/Circular_kernel_2/Intersection_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_2/include/CGAL/Circular_kernel_2/Intersection_traits.h $
 // $Id: Intersection_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_kernel_2/Line_arc_2.h 5.1-1/include/CGAL/Circular_kernel_2/Line_arc_2.h
--- 5.0.3-1/include/CGAL/Circular_kernel_2/Line_arc_2.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_kernel_2/Line_arc_2.h	2020-09-08 12:14:46.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_2/include/CGAL/Circular_kernel_2/Line_arc_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_2/include/CGAL/Circular_kernel_2/Line_arc_2.h $
 // $Id: Line_arc_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_kernel_2.h 5.1-1/include/CGAL/Circular_kernel_2.h
--- 5.0.3-1/include/CGAL/Circular_kernel_2.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_kernel_2.h	2020-09-08 12:14:46.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_2/include/CGAL/Circular_kernel_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_2/include/CGAL/Circular_kernel_2.h $
 // $Id: Circular_kernel_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_kernel_3/Circular_arc_3.h 5.1-1/include/CGAL/Circular_kernel_3/Circular_arc_3.h
--- 5.0.3-1/include/CGAL/Circular_kernel_3/Circular_arc_3.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_kernel_3/Circular_arc_3.h	2020-09-08 12:14:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_3/include/CGAL/Circular_kernel_3/Circular_arc_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_3/include/CGAL/Circular_kernel_3/Circular_arc_3.h $
 // $Id: Circular_arc_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_kernel_3/Circular_arc_point_3.h 5.1-1/include/CGAL/Circular_kernel_3/Circular_arc_point_3.h
--- 5.0.3-1/include/CGAL/Circular_kernel_3/Circular_arc_point_3.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_kernel_3/Circular_arc_point_3.h	2020-09-08 12:14:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_3/include/CGAL/Circular_kernel_3/Circular_arc_point_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_3/include/CGAL/Circular_kernel_3/Circular_arc_point_3.h $
 // $Id: Circular_arc_point_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_kernel_3/function_objects_polynomial_sphere.h 5.1-1/include/CGAL/Circular_kernel_3/function_objects_polynomial_sphere.h
--- 5.0.3-1/include/CGAL/Circular_kernel_3/function_objects_polynomial_sphere.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_kernel_3/function_objects_polynomial_sphere.h	2020-09-08 12:14:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_3/include/CGAL/Circular_kernel_3/function_objects_polynomial_sphere.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_3/include/CGAL/Circular_kernel_3/function_objects_polynomial_sphere.h $
 // $Id: function_objects_polynomial_sphere.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_kernel_3/get_equation_object_on_curved_kernel_3.h 5.1-1/include/CGAL/Circular_kernel_3/get_equation_object_on_curved_kernel_3.h
--- 5.0.3-1/include/CGAL/Circular_kernel_3/get_equation_object_on_curved_kernel_3.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_kernel_3/get_equation_object_on_curved_kernel_3.h	2020-09-08 12:14:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_3/include/CGAL/Circular_kernel_3/get_equation_object_on_curved_kernel_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_3/include/CGAL/Circular_kernel_3/get_equation_object_on_curved_kernel_3.h $
 // $Id: get_equation_object_on_curved_kernel_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_kernel_3/interface_macros.h 5.1-1/include/CGAL/Circular_kernel_3/interface_macros.h
--- 5.0.3-1/include/CGAL/Circular_kernel_3/interface_macros.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_kernel_3/interface_macros.h	2020-09-08 12:14:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_3/include/CGAL/Circular_kernel_3/interface_macros.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_3/include/CGAL/Circular_kernel_3/interface_macros.h $
 // $Id: interface_macros.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_kernel_3/internal_function_compare_spherical_kernel.h 5.1-1/include/CGAL/Circular_kernel_3/internal_function_compare_spherical_kernel.h
--- 5.0.3-1/include/CGAL/Circular_kernel_3/internal_function_compare_spherical_kernel.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_kernel_3/internal_function_compare_spherical_kernel.h	2020-09-08 12:14:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_3/include/CGAL/Circular_kernel_3/internal_function_compare_spherical_kernel.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_3/include/CGAL/Circular_kernel_3/internal_function_compare_spherical_kernel.h $
 // $Id: internal_function_compare_spherical_kernel.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_kernel_3/internal_function_compare_to_right_spherical_kernel.h 5.1-1/include/CGAL/Circular_kernel_3/internal_function_compare_to_right_spherical_kernel.h
--- 5.0.3-1/include/CGAL/Circular_kernel_3/internal_function_compare_to_right_spherical_kernel.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_kernel_3/internal_function_compare_to_right_spherical_kernel.h	2020-09-08 12:14:47.000000000 +0000
@@ -9,7 +9,7 @@
 // and a STREP (FET Open) Project under Contract No  IST-006413
 // (ACS -- Algorithms for Complex Shapes)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_3/include/CGAL/Circular_kernel_3/internal_function_compare_to_right_spherical_kernel.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_3/include/CGAL/Circular_kernel_3/internal_function_compare_to_right_spherical_kernel.h $
 // $Id: internal_function_compare_to_right_spherical_kernel.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_kernel_3/internal_function_has_on_spherical_kernel.h 5.1-1/include/CGAL/Circular_kernel_3/internal_function_has_on_spherical_kernel.h
--- 5.0.3-1/include/CGAL/Circular_kernel_3/internal_function_has_on_spherical_kernel.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_kernel_3/internal_function_has_on_spherical_kernel.h	2020-09-08 12:14:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_3/include/CGAL/Circular_kernel_3/internal_function_has_on_spherical_kernel.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_3/include/CGAL/Circular_kernel_3/internal_function_has_on_spherical_kernel.h $
 // $Id: internal_function_has_on_spherical_kernel.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_kernel_3/internal_functions_on_circle_3.h 5.1-1/include/CGAL/Circular_kernel_3/internal_functions_on_circle_3.h
--- 5.0.3-1/include/CGAL/Circular_kernel_3/internal_functions_on_circle_3.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_kernel_3/internal_functions_on_circle_3.h	2020-09-08 12:14:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_3/include/CGAL/Circular_kernel_3/internal_functions_on_circle_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_3/include/CGAL/Circular_kernel_3/internal_functions_on_circle_3.h $
 // $Id: internal_functions_on_circle_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_kernel_3/internal_functions_on_circular_arc_3.h 5.1-1/include/CGAL/Circular_kernel_3/internal_functions_on_circular_arc_3.h
--- 5.0.3-1/include/CGAL/Circular_kernel_3/internal_functions_on_circular_arc_3.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_kernel_3/internal_functions_on_circular_arc_3.h	2020-09-08 12:14:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_3/include/CGAL/Circular_kernel_3/internal_functions_on_circular_arc_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_3/include/CGAL/Circular_kernel_3/internal_functions_on_circular_arc_3.h $
 // $Id: internal_functions_on_circular_arc_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_kernel_3/internal_functions_on_circular_arc_point_3.h 5.1-1/include/CGAL/Circular_kernel_3/internal_functions_on_circular_arc_point_3.h
--- 5.0.3-1/include/CGAL/Circular_kernel_3/internal_functions_on_circular_arc_point_3.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_kernel_3/internal_functions_on_circular_arc_point_3.h	2020-09-08 12:14:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_3/include/CGAL/Circular_kernel_3/internal_functions_on_circular_arc_point_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_3/include/CGAL/Circular_kernel_3/internal_functions_on_circular_arc_point_3.h $
 // $Id: internal_functions_on_circular_arc_point_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_kernel_3/internal_functions_on_line_3.h 5.1-1/include/CGAL/Circular_kernel_3/internal_functions_on_line_3.h
--- 5.0.3-1/include/CGAL/Circular_kernel_3/internal_functions_on_line_3.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_kernel_3/internal_functions_on_line_3.h	2020-09-08 12:14:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_3/include/CGAL/Circular_kernel_3/internal_functions_on_line_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_3/include/CGAL/Circular_kernel_3/internal_functions_on_line_3.h $
 // $Id: internal_functions_on_line_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_kernel_3/internal_functions_on_line_arc_3.h 5.1-1/include/CGAL/Circular_kernel_3/internal_functions_on_line_arc_3.h
--- 5.0.3-1/include/CGAL/Circular_kernel_3/internal_functions_on_line_arc_3.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_kernel_3/internal_functions_on_line_arc_3.h	2020-09-08 12:14:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_3/include/CGAL/Circular_kernel_3/internal_functions_on_line_arc_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_3/include/CGAL/Circular_kernel_3/internal_functions_on_line_arc_3.h $
 // $Id: internal_functions_on_line_arc_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_kernel_3/internal_functions_on_plane_3.h 5.1-1/include/CGAL/Circular_kernel_3/internal_functions_on_plane_3.h
--- 5.0.3-1/include/CGAL/Circular_kernel_3/internal_functions_on_plane_3.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_kernel_3/internal_functions_on_plane_3.h	2020-09-08 12:14:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_3/include/CGAL/Circular_kernel_3/internal_functions_on_plane_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_3/include/CGAL/Circular_kernel_3/internal_functions_on_plane_3.h $
 // $Id: internal_functions_on_plane_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_kernel_3/internal_functions_on_sphere_3.h 5.1-1/include/CGAL/Circular_kernel_3/internal_functions_on_sphere_3.h
--- 5.0.3-1/include/CGAL/Circular_kernel_3/internal_functions_on_sphere_3.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_kernel_3/internal_functions_on_sphere_3.h	2020-09-08 12:14:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_3/include/CGAL/Circular_kernel_3/internal_functions_on_sphere_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_3/include/CGAL/Circular_kernel_3/internal_functions_on_sphere_3.h $
 // $Id: internal_functions_on_sphere_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_kernel_3/Intersection_traits.h 5.1-1/include/CGAL/Circular_kernel_3/Intersection_traits.h
--- 5.0.3-1/include/CGAL/Circular_kernel_3/Intersection_traits.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_kernel_3/Intersection_traits.h	2020-09-08 12:14:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_3/include/CGAL/Circular_kernel_3/Intersection_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_3/include/CGAL/Circular_kernel_3/Intersection_traits.h $
 // $Id: Intersection_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_kernel_3/Line_arc_3.h 5.1-1/include/CGAL/Circular_kernel_3/Line_arc_3.h
--- 5.0.3-1/include/CGAL/Circular_kernel_3/Line_arc_3.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_kernel_3/Line_arc_3.h	2020-09-08 12:14:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_3/include/CGAL/Circular_kernel_3/Line_arc_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_3/include/CGAL/Circular_kernel_3/Line_arc_3.h $
 // $Id: Line_arc_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_kernel_converter.h 5.1-1/include/CGAL/Circular_kernel_converter.h
--- 5.0.3-1/include/CGAL/Circular_kernel_converter.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_kernel_converter.h	2020-09-08 12:14:46.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_2/include/CGAL/Circular_kernel_converter.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_2/include/CGAL/Circular_kernel_converter.h $
 // $Id: Circular_kernel_converter.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_kernel_intersections.h 5.1-1/include/CGAL/Circular_kernel_intersections.h
--- 5.0.3-1/include/CGAL/Circular_kernel_intersections.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_kernel_intersections.h	2020-09-08 12:14:46.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_2/include/CGAL/Circular_kernel_intersections.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_2/include/CGAL/Circular_kernel_intersections.h $
 // $Id: Circular_kernel_intersections.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circular_kernel_type_equality_wrapper.h 5.1-1/include/CGAL/Circular_kernel_type_equality_wrapper.h
--- 5.0.3-1/include/CGAL/Circular_kernel_type_equality_wrapper.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Circular_kernel_type_equality_wrapper.h	2020-09-08 12:14:46.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_2/include/CGAL/Circular_kernel_type_equality_wrapper.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_2/include/CGAL/Circular_kernel_type_equality_wrapper.h $
 // $Id: Circular_kernel_type_equality_wrapper.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circulator/Circulator_adapters.h 5.1-1/include/CGAL/Circulator/Circulator_adapters.h
--- 5.0.3-1/include/CGAL/Circulator/Circulator_adapters.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Circulator/Circulator_adapters.h	2020-09-08 12:14:47.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circulator/include/CGAL/Circulator/Circulator_adapters.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circulator/include/CGAL/Circulator/Circulator_adapters.h $
 // $Id: Circulator_adapters.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circulator/Circulator_concepts.h 5.1-1/include/CGAL/Circulator/Circulator_concepts.h
--- 5.0.3-1/include/CGAL/Circulator/Circulator_concepts.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Circulator/Circulator_concepts.h	2020-09-08 12:14:47.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circulator/include/CGAL/Circulator/Circulator_concepts.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circulator/include/CGAL/Circulator/Circulator_concepts.h $
 // $Id: Circulator_concepts.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circulator/Safe_circulator_from_iterator.h 5.1-1/include/CGAL/Circulator/Safe_circulator_from_iterator.h
--- 5.0.3-1/include/CGAL/Circulator/Safe_circulator_from_iterator.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Circulator/Safe_circulator_from_iterator.h	2020-09-08 12:14:47.000000000 +0000
@@ -8,7 +8,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circulator/include/CGAL/Circulator/Safe_circulator_from_iterator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circulator/include/CGAL/Circulator/Safe_circulator_from_iterator.h $
 // $Id: Safe_circulator_from_iterator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/circulator_bases.h 5.1-1/include/CGAL/circulator_bases.h
--- 5.0.3-1/include/CGAL/circulator_bases.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/circulator_bases.h	2020-09-08 12:14:47.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circulator/include/CGAL/circulator_bases.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circulator/include/CGAL/circulator_bases.h $
 // $Id: circulator_bases.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/circulator.h 5.1-1/include/CGAL/circulator.h
--- 5.0.3-1/include/CGAL/circulator.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/circulator.h	2020-09-08 12:14:47.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circulator/include/CGAL/circulator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circulator/include/CGAL/circulator.h $
 // $Id: circulator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circulator_identity.h 5.1-1/include/CGAL/Circulator_identity.h
--- 5.0.3-1/include/CGAL/Circulator_identity.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Circulator_identity.h	2020-09-08 12:15:32.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Circulator_identity.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Circulator_identity.h $
 // $Id: Circulator_identity.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circulator_on_node.h 5.1-1/include/CGAL/Circulator_on_node.h
--- 5.0.3-1/include/CGAL/Circulator_on_node.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Circulator_on_node.h	2020-09-08 12:15:32.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Circulator_on_node.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Circulator_on_node.h $
 // $Id: Circulator_on_node.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Circulator_project.h 5.1-1/include/CGAL/Circulator_project.h
--- 5.0.3-1/include/CGAL/Circulator_project.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Circulator_project.h	2020-09-08 12:15:32.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Circulator_project.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Circulator_project.h $
 // $Id: Circulator_project.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Classification/classify.h 5.1-1/include/CGAL/Classification/classify.h
--- 5.0.3-1/include/CGAL/Classification/classify.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/classify.h	2020-09-08 12:14:48.000000000 +0000
@@ -4,8 +4,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/classify.h $
-// $Id: classify.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/classify.h $
+// $Id: classify.h 47027d6 2020-03-26T18:59:19+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Simon Giraudot, Florent Lafarge
@@ -15,7 +15,7 @@
 
 #include <CGAL/license/Classification.h>
 
-#include <CGAL/internal/Surface_mesh_segmentation/Alpha_expansion_graph_cut.h>
+#include <CGAL/boost/graph/alpha_expansion_graphcut.h>
 #include <CGAL/Bbox_3.h>
 #include <CGAL/Classification/Label_set.h>
 #include <CGAL/property_map.h>
@@ -234,12 +234,6 @@ namespace internal {
     const std::vector<std::pair<std::size_t, std::size_t> >& m_input_to_indices;
     LabelIndexRange& m_out;
 
-#ifdef CGAL_DO_NOT_USE_BOYKOV_KOLMOGOROV_MAXFLOW_SOFTWARE
-    typedef CGAL::internal::Alpha_expansion_graph_cut_boost             Alpha_expansion;
-#else
-    typedef CGAL::internal::Alpha_expansion_graph_cut_boykov_kolmogorov Alpha_expansion;
-#endif
-
   public:
 
     Classify_functor_graphcut (const ItemRange& input,
@@ -310,8 +304,7 @@ namespace internal {
         assigned_label[j] = nb_class_best;
       }
 
-      Alpha_expansion graphcut;
-      graphcut(edges, edge_weights, probability_matrix, assigned_label);
+      CGAL::alpha_expansion_graphcut (edges, edge_weights, probability_matrix, assigned_label);
 
       for (std::size_t i = 0; i < assigned_label.size(); ++ i)
         m_out[m_indices[sub][i]] = static_cast<typename LabelIndexRange::iterator::value_type>(assigned_label[i]);
@@ -334,7 +327,7 @@ namespace internal {
     suboptimal results.
 
     \tparam ConcurrencyTag enables sequential versus parallel
-    algorithm. Possible values are `Parallel_tag` or `Sequential_tag`.
+    algorithm. Possible values are `Parallel_if_available_tag`, `Parallel_tag` or `Sequential_tag`.
 
     \tparam ItemRange model of `ConstRange`. Its iterator type is
     `RandomAccessIterator`. Its value type depends on the data that is
@@ -424,7 +417,7 @@ namespace internal {
     efficiency and better quality results.
 
     \tparam ConcurrencyTag enables sequential versus parallel
-    algorithm. Possible values are `Parallel_tag` or `Sequential_tag`.
+    algorithm. Possible values are `Parallel_if_available_tag`, `Parallel_tag` or `Sequential_tag`.
     \tparam ItemRange model of `ConstRange`. Its iterator type is
     `RandomAccessIterator`.
     \tparam ItemMap model of `ReadablePropertyMap` whose key
@@ -502,7 +495,7 @@ namespace internal {
     results.
 
     \tparam ConcurrencyTag enables sequential versus parallel
-    algorithm. Possible values are `Parallel_tag` or `Sequential_tag`.
+    algorithm. Possible values are `Parallel_if_available_tag`, `Parallel_tag` or `Sequential_tag`.
     \tparam ItemRange model of `ConstRange`. Its iterator type is
     `RandomAccessIterator`.
     \tparam ItemMap model of `ReadablePropertyMap` whose key
diff -pruN 5.0.3-1/include/CGAL/Classification/Cluster.h 5.1-1/include/CGAL/Classification/Cluster.h
--- 5.0.3-1/include/CGAL/Classification/Cluster.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/Cluster.h	2020-09-08 12:14:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/Cluster.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/Cluster.h $
 // $Id: Cluster.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Classification/compressed_float.h 5.1-1/include/CGAL/Classification/compressed_float.h
--- 5.0.3-1/include/CGAL/Classification/compressed_float.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/compressed_float.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/compressed_float.h $
-// $Id: compressed_float.h 5b2d46c 2020-04-21T10:03:00+02:00 Maxime Gimeno
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/compressed_float.h $
+// $Id: compressed_float.h e9d41d7 2020-04-21T10:03:00+02:00 Maxime Gimeno
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Simon Giraudot
diff -pruN 5.0.3-1/include/CGAL/Classification/ETHZ/internal/dataview.h 5.1-1/include/CGAL/Classification/ETHZ/internal/dataview.h
--- 5.0.3-1/include/CGAL/Classification/ETHZ/internal/dataview.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/ETHZ/internal/dataview.h	2020-09-08 12:14:47.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/ETHZ/internal/dataview.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/ETHZ/internal/dataview.h $
 // $Id: dataview.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LicenseRef-RFL
 // License notice in Installation/LICENSE.RFL
diff -pruN 5.0.3-1/include/CGAL/Classification/ETHZ/internal/random-forest/common-libraries.hpp 5.1-1/include/CGAL/Classification/ETHZ/internal/random-forest/common-libraries.hpp
--- 5.0.3-1/include/CGAL/Classification/ETHZ/internal/random-forest/common-libraries.hpp	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/ETHZ/internal/random-forest/common-libraries.hpp	2020-09-08 12:14:47.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/ETHZ/internal/random-forest/common-libraries.hpp $
-// $Id: common-libraries.hpp 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/ETHZ/internal/random-forest/common-libraries.hpp $
+// $Id: common-libraries.hpp 7cfe6df 2020-04-07T11:02:16+02:00 Simon Giraudot
 // SPDX-License-Identifier: LicenseRef-RFL
 // License notice in Installation/LICENSE.RFL
 //
@@ -40,7 +40,9 @@
 #endif
 #include <boost/random/uniform_01.hpp>
 #include <boost/random/normal_distribution.hpp>
+#if defined(CGAL_LINKED_WITH_BOOST_IOSTREAMS) && defined(CGAL_LINKED_WITH_BOOST_SERIALIZATION)
 #include <boost/serialization/vector.hpp>
+#endif
 #include <boost/scoped_ptr.hpp>
 #include <boost/shared_ptr.hpp>
 #include <boost/make_shared.hpp>
@@ -93,6 +95,7 @@ struct ForestParams {
         min_samples_per_node(5),
         sample_reduction(0.368f)
     {}
+#if defined(CGAL_LINKED_WITH_BOOST_IOSTREAMS) && defined(CGAL_LINKED_WITH_BOOST_SERIALIZATION)
     template <typename Archive>
     void serialize(Archive& ar, unsigned /*version*/)
     {
@@ -105,6 +108,7 @@ struct ForestParams {
         ar & BOOST_SERIALIZATION_NVP(min_samples_per_node);
         ar & BOOST_SERIALIZATION_NVP(sample_reduction);
     }
+#endif
 };
 
 struct QuadraticSplitter {
@@ -149,6 +153,7 @@ struct QuadraticSplitter {
             data_points[i_sample] = std::make_pair(sample_fval, sample_class);
         }
     }
+#if defined(CGAL_LINKED_WITH_BOOST_IOSTREAMS) && defined(CGAL_LINKED_WITH_BOOST_SERIALIZATION)
     template <typename Archive>
     void serialize(Archive& ar, unsigned /*version*/)
     {
@@ -156,6 +161,7 @@ struct QuadraticSplitter {
         ar & BOOST_SERIALIZATION_NVP(w);
         ar & BOOST_SERIALIZATION_NVP(threshold);
     }
+#endif
 };
 
 struct LinearSplitter {
@@ -187,12 +193,14 @@ struct LinearSplitter {
             data_points[i_sample] = std::make_pair(sample_fval, sample_class);
         }
     }
+#if defined(CGAL_LINKED_WITH_BOOST_IOSTREAMS) && defined(CGAL_LINKED_WITH_BOOST_SERIALIZATION)
     template <typename Archive>
     void serialize(Archive& ar, unsigned /*version*/)
     {
         ar & BOOST_SERIALIZATION_NVP(w);
         ar & BOOST_SERIALIZATION_NVP(threshold);
     }
+#endif
 };
 
 struct AxisAlignedSplitter {
@@ -232,12 +240,14 @@ struct AxisAlignedSplitter {
         data_points.push_back(std::make_pair(sample_fval, sample_class));
       }
     }
+#if defined(CGAL_LINKED_WITH_BOOST_IOSTREAMS) && defined(CGAL_LINKED_WITH_BOOST_SERIALIZATION)
     template <typename Archive>
     void serialize(Archive& ar, unsigned /*version*/)
     {
         ar & BOOST_SERIALIZATION_NVP(feature);
         ar & BOOST_SERIALIZATION_NVP(threshold);
     }
+#endif
 };
 
 struct AxisAlignedRandomSplitGenerator {
diff -pruN 5.0.3-1/include/CGAL/Classification/ETHZ/internal/random-forest/forest.hpp 5.1-1/include/CGAL/Classification/ETHZ/internal/random-forest/forest.hpp
--- 5.0.3-1/include/CGAL/Classification/ETHZ/internal/random-forest/forest.hpp	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/ETHZ/internal/random-forest/forest.hpp	2020-09-08 12:14:47.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/ETHZ/internal/random-forest/forest.hpp $
-// $Id: forest.hpp 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/ETHZ/internal/random-forest/forest.hpp $
+// $Id: forest.hpp 7cfe6df 2020-04-07T11:02:16+02:00 Simon Giraudot
 // SPDX-License-Identifier: LicenseRef-RFL
 // License notice in Installation/LICENSE.RFL
 //
@@ -223,12 +223,14 @@ public:
         return sum/trees.size();
     }
 #endif
+#if defined(CGAL_LINKED_WITH_BOOST_IOSTREAMS) && defined(CGAL_LINKED_WITH_BOOST_SERIALIZATION)
     template <typename Archive>
     void serialize(Archive& ar, unsigned /* version */)
     {
         ar & BOOST_SERIALIZATION_NVP(params);
         ar & BOOST_SERIALIZATION_NVP(trees);
     }
+#endif
 
     void get_feature_usage (std::vector<std::size_t>& count) const
     {
diff -pruN 5.0.3-1/include/CGAL/Classification/ETHZ/internal/random-forest/node-gini.hpp 5.1-1/include/CGAL/Classification/ETHZ/internal/random-forest/node-gini.hpp
--- 5.0.3-1/include/CGAL/Classification/ETHZ/internal/random-forest/node-gini.hpp	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/ETHZ/internal/random-forest/node-gini.hpp	2020-09-08 12:14:47.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/ETHZ/internal/random-forest/node-gini.hpp $
-// $Id: node-gini.hpp 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/ETHZ/internal/random-forest/node-gini.hpp $
+// $Id: node-gini.hpp 7cfe6df 2020-04-07T11:02:16+02:00 Simon Giraudot
 // SPDX-License-Identifier: LicenseRef-RFL
 // License notice in Installation/LICENSE.RFL
 //
@@ -97,11 +97,13 @@ public:
         return std::make_pair(best_thresh, float(best_loss));
     }
 
+#if defined(CGAL_LINKED_WITH_BOOST_IOSTREAMS) && defined(CGAL_LINKED_WITH_BOOST_SERIALIZATION)
     template <typename Archive>
     void serialize(Archive& ar, unsigned /* version */)
     {
         ar & boost::serialization::make_nvp("base",  boost::serialization::base_object< Node< NodeGini<Splitter>, ForestParams, Splitter > >(*this));
     }
+#endif
 };
 
 }
diff -pruN 5.0.3-1/include/CGAL/Classification/ETHZ/internal/random-forest/node.hpp 5.1-1/include/CGAL/Classification/ETHZ/internal/random-forest/node.hpp
--- 5.0.3-1/include/CGAL/Classification/ETHZ/internal/random-forest/node.hpp	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/ETHZ/internal/random-forest/node.hpp	2020-09-08 12:14:47.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/ETHZ/internal/random-forest/node.hpp $
-// $Id: node.hpp 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/ETHZ/internal/random-forest/node.hpp $
+// $Id: node.hpp 7cfe6df 2020-04-07T11:02:16+02:00 Simon Giraudot
 // SPDX-License-Identifier: LicenseRef-RFL
 // License notice in Installation/LICENSE.RFL
 //
@@ -21,8 +21,15 @@
 #define CGAL_INTERNAL_LIBLEARNING_RANDOMFORESTS_NODE_H
 #include "../dataview.h"
 #include "common-libraries.hpp"
+
+#if defined(CGAL_LINKED_WITH_BOOST_IOSTREAMS) && defined(CGAL_LINKED_WITH_BOOST_SERIALIZATION)
 #include <boost/serialization/scoped_ptr.hpp>
 #include <boost/serialization/vector.hpp>
+#else
+#include <boost/scoped_ptr.hpp>
+#include <vector>
+#endif
+
 #if VERBOSE_NODE_LEARNING
 #include <cstdio>
 #endif
@@ -228,6 +235,7 @@ public:
         right->train(samples, labels, sample_idxes + offset_right, n_samples_right, split_generator, gen);
     }
 
+#if defined(CGAL_LINKED_WITH_BOOST_IOSTREAMS) && defined(CGAL_LINKED_WITH_BOOST_SERIALIZATION)
     template <typename Archive>
     void serialize(Archive& ar, unsigned /*version*/)
     {
@@ -243,6 +251,7 @@ public:
           ar & BOOST_SERIALIZATION_NVP(right);
         }
     }
+#endif
 
     void get_feature_usage (std::vector<std::size_t>& count) const
     {
diff -pruN 5.0.3-1/include/CGAL/Classification/ETHZ/internal/random-forest/tree.hpp 5.1-1/include/CGAL/Classification/ETHZ/internal/random-forest/tree.hpp
--- 5.0.3-1/include/CGAL/Classification/ETHZ/internal/random-forest/tree.hpp	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/ETHZ/internal/random-forest/tree.hpp	2020-09-08 12:14:47.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/ETHZ/internal/random-forest/tree.hpp $
-// $Id: tree.hpp 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/ETHZ/internal/random-forest/tree.hpp $
+// $Id: tree.hpp 7cfe6df 2020-04-07T11:02:16+02:00 Simon Giraudot
 // SPDX-License-Identifier: LicenseRef-RFL
 // License notice in Installation/LICENSE.RFL
 //
@@ -18,7 +18,11 @@
 #define CGAL_INTERNAL_LIBLEARNING_RANDOMFOREST_TREE_H
 #include "../dataview.h"
 #include "common-libraries.hpp"
+#if defined(CGAL_LINKED_WITH_BOOST_IOSTREAMS) && defined(CGAL_LINKED_WITH_BOOST_SERIALIZATION)
 #include <boost/serialization/scoped_ptr.hpp>
+#else
+#include <boost/scoped_ptr.hpp>
+#endif
 
 namespace CGAL { namespace internal {
 
@@ -115,12 +119,16 @@ public:
         return n_common/sqrt((n_common + n_1)*(n_common + n_2));
     }
 #endif
+
+#if defined(CGAL_LINKED_WITH_BOOST_IOSTREAMS) && defined(CGAL_LINKED_WITH_BOOST_SERIALIZATION)
     template <typename Archive>
     void serialize(Archive& ar, unsigned /*version*/)
     {
         ar & BOOST_SERIALIZATION_NVP(params);
         ar & BOOST_SERIALIZATION_NVP(root_node);
     }
+#endif
+
     void get_feature_usage (std::vector<std::size_t>& count) const
     {
       root_node->get_feature_usage(count);
diff -pruN 5.0.3-1/include/CGAL/Classification/ETHZ/Random_forest_classifier.h 5.1-1/include/CGAL/Classification/ETHZ/Random_forest_classifier.h
--- 5.0.3-1/include/CGAL/Classification/ETHZ/Random_forest_classifier.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/ETHZ/Random_forest_classifier.h	2020-09-08 12:14:47.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/ETHZ/Random_forest_classifier.h $
-// $Id: Random_forest_classifier.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/ETHZ/Random_forest_classifier.h $
+// $Id: Random_forest_classifier.h a08a1e9 2020-04-16T16:56:13+02:00 Simon Giraudot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Simon Giraudot
@@ -38,10 +38,12 @@
 
 #include <CGAL/tags.h>
 
+#if defined(CGAL_LINKED_WITH_BOOST_IOSTREAMS) && defined(CGAL_LINKED_WITH_BOOST_SERIALIZATION)
 #include <boost/archive/text_iarchive.hpp>
 #include <boost/archive/text_oarchive.hpp>
 #include <boost/iostreams/filtering_stream.hpp>
 #include <boost/iostreams/filter/gzip.hpp>
+#endif
 
 #ifdef BOOST_MSVC
 #  pragma warning(pop)
@@ -98,6 +100,9 @@ public:
     than the ones used by `other`, and in the same order.
 
   */
+#if defined(DOXYGEN_RUNNING) || \
+  (defined(CGAL_LINKED_WITH_BOOST_IOSTREAMS) && \
+   defined(CGAL_LINKED_WITH_BOOST_SERIALIZATION))
   Random_forest_classifier (const Random_forest_classifier& other,
                             const Feature_set& features)
     : m_labels (other.m_labels), m_features (features), m_rfc (nullptr)
@@ -106,6 +111,7 @@ public:
     other.save_configuration(stream);
     this->load_configuration(stream);
   }
+#endif
 
   /// \cond SKIP_IN_MANUAL
   ~Random_forest_classifier ()
@@ -127,11 +133,7 @@ public:
               std::size_t num_trees = 25,
               std::size_t max_depth = 20)
   {
-#ifdef CGAL_LINKED_WITH_TBB
-    train<CGAL::Parallel_tag>(ground_truth, reset_trees, num_trees, max_depth);
-#else
-    train<CGAL::Sequential_tag>(ground_truth, reset_trees, num_trees, max_depth);
-#endif
+    train<CGAL::Parallel_if_available_tag>(ground_truth, reset_trees, num_trees, max_depth);
   }
   /// \endcond
 
@@ -146,7 +148,7 @@ public:
     label.
 
     \tparam ConcurrencyTag enables sequential versus parallel
-    algorithm. Possible values are `Parallel_tag` (default value is
+    algorithm. Possible values are `Parallel_tag` (default value if
     %CGAL is linked with TBB) or `Sequential_tag` (default value
     otherwise).
 
@@ -184,15 +186,17 @@ public:
     std::vector<int> gt;
     std::vector<float> ft;
 
-    for (std::size_t i = 0; i < ground_truth.size(); ++ i)
+    std::size_t idx = 0;
+    for (const auto& ig : ground_truth)
     {
-      int g = int(ground_truth[i]);
+      int g = int(ig);
       if (g != -1)
       {
         for (std::size_t f = 0; f < m_features.size(); ++ f)
-          ft.push_back(m_features[f]->value(i));
+          ft.push_back(m_features[f]->value(idx));
         gt.push_back(g);
       }
+      ++ idx;
     }
 
     CGAL_CLASSIFICATION_CERR << "Using " << gt.size() << " inliers" << std::endl;
@@ -284,6 +288,9 @@ public:
     The output file is written in an GZIP container that is readable
     by the `load_configuration()` method.
   */
+#if defined(DOXYGEN_RUNNING) || \
+  (defined(CGAL_LINKED_WITH_BOOST_IOSTREAMS) && \
+   defined(CGAL_LINKED_WITH_BOOST_SERIALIZATION))
   void save_configuration (std::ostream& output) const
   {
     boost::iostreams::filtering_ostream outs;
@@ -292,6 +299,7 @@ public:
     boost::archive::text_oarchive oas(outs);
     oas << BOOST_SERIALIZATION_NVP(*m_rfc);
   }
+#endif
 
   /*!
     \brief Loads a configuration from the stream `input`.
@@ -302,6 +310,9 @@ public:
     the ones present when the file was generated using
     `save_configuration()`.
   */
+#if defined(DOXYGEN_RUNNING) || \
+  (defined(CGAL_LINKED_WITH_BOOST_IOSTREAMS) && \
+   defined(CGAL_LINKED_WITH_BOOST_SERIALIZATION))
   void load_configuration (std::istream& input)
   {
     CGAL::internal::liblearning::RandomForest::ForestParams params;
@@ -315,6 +326,7 @@ public:
     boost::archive::text_iarchive ias(ins);
     ias >> BOOST_SERIALIZATION_NVP(*m_rfc);
   }
+#endif
 
 /// @}
 
diff -pruN 5.0.3-1/include/CGAL/Classification/Evaluation.h 5.1-1/include/CGAL/Classification/Evaluation.h
--- 5.0.3-1/include/CGAL/Classification/Evaluation.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/Evaluation.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/Evaluation.h $
-// $Id: Evaluation.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/Evaluation.h $
+// $Id: Evaluation.h c03e69d 2020-04-20T08:56:08+02:00 Simon Giraudot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Simon Giraudot
@@ -16,6 +16,10 @@
 
 #include <CGAL/Classification/Label.h>
 #include <CGAL/Classification/Label_set.h>
+
+#include <boost/iterator/zip_iterator.hpp>
+#include <CGAL/Iterator_range.h>
+
 #include <map>
 #include <cmath> // for std::isnan
 
@@ -80,10 +84,13 @@ public:
     std::size_t sum_true_positives = 0;
     std::size_t total = 0;
 
-    for (std::size_t j = 0; j < ground_truth.size(); ++ j)
+    for (const auto& zip : CGAL::make_range (boost::make_zip_iterator
+                                             (boost::make_tuple(ground_truth.begin(), result.begin())),
+                                             boost::make_zip_iterator
+                                             (boost::make_tuple(ground_truth.end(), result.end()))))
     {
-      int gt = static_cast<int>(ground_truth[j]);
-      int res = static_cast<int>(result[j]);
+      int gt = static_cast<int>(boost::get<0>(zip));
+      int res = static_cast<int>(boost::get<1>(zip));
       if (gt == -1 || res == -1)
         continue;
       ++ total;
@@ -221,4 +228,3 @@ public:
 } // namespace CGAL
 
 #endif // CGAL_CLASSIFICATION_EVALUATION_H
-
diff -pruN 5.0.3-1/include/CGAL/Classification/Feature/Cluster_mean_of_feature.h 5.1-1/include/CGAL/Classification/Feature/Cluster_mean_of_feature.h
--- 5.0.3-1/include/CGAL/Classification/Feature/Cluster_mean_of_feature.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/Feature/Cluster_mean_of_feature.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/Feature/Cluster_mean_of_feature.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/Feature/Cluster_mean_of_feature.h $
 // $Id: Cluster_mean_of_feature.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Classification/Feature/Cluster_size.h 5.1-1/include/CGAL/Classification/Feature/Cluster_size.h
--- 5.0.3-1/include/CGAL/Classification/Feature/Cluster_size.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/Feature/Cluster_size.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/Feature/Cluster_size.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/Feature/Cluster_size.h $
 // $Id: Cluster_size.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Classification/Feature/Cluster_variance_of_feature.h 5.1-1/include/CGAL/Classification/Feature/Cluster_variance_of_feature.h
--- 5.0.3-1/include/CGAL/Classification/Feature/Cluster_variance_of_feature.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/Feature/Cluster_variance_of_feature.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/Feature/Cluster_variance_of_feature.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/Feature/Cluster_variance_of_feature.h $
 // $Id: Cluster_variance_of_feature.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Classification/Feature/Cluster_vertical_extent.h 5.1-1/include/CGAL/Classification/Feature/Cluster_vertical_extent.h
--- 5.0.3-1/include/CGAL/Classification/Feature/Cluster_vertical_extent.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/Feature/Cluster_vertical_extent.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/Feature/Cluster_vertical_extent.h $
-// $Id: Cluster_vertical_extent.h 5b2d46c 2020-04-21T10:03:00+02:00 Maxime Gimeno
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/Feature/Cluster_vertical_extent.h $
+// $Id: Cluster_vertical_extent.h e9d41d7 2020-04-21T10:03:00+02:00 Maxime Gimeno
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Simon Giraudot
diff -pruN 5.0.3-1/include/CGAL/Classification/Feature/Color_channel.h 5.1-1/include/CGAL/Classification/Feature/Color_channel.h
--- 5.0.3-1/include/CGAL/Classification/Feature/Color_channel.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/Feature/Color_channel.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/Feature/Color_channel.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/Feature/Color_channel.h $
 // $Id: Color_channel.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Classification/Feature/Distance_to_plane.h 5.1-1/include/CGAL/Classification/Feature/Distance_to_plane.h
--- 5.0.3-1/include/CGAL/Classification/Feature/Distance_to_plane.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/Feature/Distance_to_plane.h	2020-09-08 12:14:48.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/Feature/Distance_to_plane.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/Feature/Distance_to_plane.h $
 // $Id: Distance_to_plane.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Classification/Feature/Echo_scatter.h 5.1-1/include/CGAL/Classification/Feature/Echo_scatter.h
--- 5.0.3-1/include/CGAL/Classification/Feature/Echo_scatter.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/Feature/Echo_scatter.h	2020-09-08 12:14:48.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/Feature/Echo_scatter.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/Feature/Echo_scatter.h $
 // $Id: Echo_scatter.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Classification/Feature/Eigenvalue.h 5.1-1/include/CGAL/Classification/Feature/Eigenvalue.h
--- 5.0.3-1/include/CGAL/Classification/Feature/Eigenvalue.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/Feature/Eigenvalue.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/Feature/Eigenvalue.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/Feature/Eigenvalue.h $
 // $Id: Eigenvalue.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Classification/Feature/Elevation.h 5.1-1/include/CGAL/Classification/Feature/Elevation.h
--- 5.0.3-1/include/CGAL/Classification/Feature/Elevation.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/Feature/Elevation.h	2020-09-08 12:14:48.000000000 +0000
@@ -4,8 +4,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/Feature/Elevation.h $
-// $Id: Elevation.h 5b2d46c 2020-04-21T10:03:00+02:00 Maxime Gimeno
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/Feature/Elevation.h $
+// $Id: Elevation.h e9d41d7 2020-04-21T10:03:00+02:00 Maxime Gimeno
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Florent Lafarge, Simon Giraudot
diff -pruN 5.0.3-1/include/CGAL/Classification/Feature/Gradient_of_feature.h 5.1-1/include/CGAL/Classification/Feature/Gradient_of_feature.h
--- 5.0.3-1/include/CGAL/Classification/Feature/Gradient_of_feature.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/Feature/Gradient_of_feature.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/Feature/Gradient_of_feature.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/Feature/Gradient_of_feature.h $
 // $Id: Gradient_of_feature.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Classification/Feature/Height_above.h 5.1-1/include/CGAL/Classification/Feature/Height_above.h
--- 5.0.3-1/include/CGAL/Classification/Feature/Height_above.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/Feature/Height_above.h	2020-09-08 12:14:48.000000000 +0000
@@ -4,8 +4,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/Feature/Height_above.h $
-// $Id: Height_above.h 5b2d46c 2020-04-21T10:03:00+02:00 Maxime Gimeno
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/Feature/Height_above.h $
+// $Id: Height_above.h e9d41d7 2020-04-21T10:03:00+02:00 Maxime Gimeno
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Florent Lafarge, Simon Giraudot
diff -pruN 5.0.3-1/include/CGAL/Classification/Feature/Height_below.h 5.1-1/include/CGAL/Classification/Feature/Height_below.h
--- 5.0.3-1/include/CGAL/Classification/Feature/Height_below.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/Feature/Height_below.h	2020-09-08 12:14:48.000000000 +0000
@@ -4,8 +4,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/Feature/Height_below.h $
-// $Id: Height_below.h 5b2d46c 2020-04-21T10:03:00+02:00 Maxime Gimeno
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/Feature/Height_below.h $
+// $Id: Height_below.h e9d41d7 2020-04-21T10:03:00+02:00 Maxime Gimeno
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Florent Lafarge, Simon Giraudot
diff -pruN 5.0.3-1/include/CGAL/Classification/Feature/Simple_feature.h 5.1-1/include/CGAL/Classification/Feature/Simple_feature.h
--- 5.0.3-1/include/CGAL/Classification/Feature/Simple_feature.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/Feature/Simple_feature.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/Feature/Simple_feature.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/Feature/Simple_feature.h $
 // $Id: Simple_feature.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Classification/Feature/Vertical_dispersion.h 5.1-1/include/CGAL/Classification/Feature/Vertical_dispersion.h
--- 5.0.3-1/include/CGAL/Classification/Feature/Vertical_dispersion.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/Feature/Vertical_dispersion.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/Feature/Vertical_dispersion.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/Feature/Vertical_dispersion.h $
 // $Id: Vertical_dispersion.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Classification/Feature/Verticality.h 5.1-1/include/CGAL/Classification/Feature/Verticality.h
--- 5.0.3-1/include/CGAL/Classification/Feature/Verticality.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/Feature/Verticality.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/Feature/Verticality.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/Feature/Verticality.h $
 // $Id: Verticality.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Classification/Feature/Vertical_range.h 5.1-1/include/CGAL/Classification/Feature/Vertical_range.h
--- 5.0.3-1/include/CGAL/Classification/Feature/Vertical_range.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/Feature/Vertical_range.h	2020-09-08 12:14:48.000000000 +0000
@@ -4,8 +4,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/Feature/Vertical_range.h $
-// $Id: Vertical_range.h 5b2d46c 2020-04-21T10:03:00+02:00 Maxime Gimeno
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/Feature/Vertical_range.h $
+// $Id: Vertical_range.h e9d41d7 2020-04-21T10:03:00+02:00 Maxime Gimeno
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Florent Lafarge, Simon Giraudot
diff -pruN 5.0.3-1/include/CGAL/Classification/Feature_base.h 5.1-1/include/CGAL/Classification/Feature_base.h
--- 5.0.3-1/include/CGAL/Classification/Feature_base.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/Feature_base.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/Feature_base.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/Feature_base.h $
 // $Id: Feature_base.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Classification/Feature_set.h 5.1-1/include/CGAL/Classification/Feature_set.h
--- 5.0.3-1/include/CGAL/Classification/Feature_set.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/Feature_set.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/Feature_set.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/Feature_set.h $
 // $Id: Feature_set.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Classification/Image.h 5.1-1/include/CGAL/Classification/Image.h
--- 5.0.3-1/include/CGAL/Classification/Image.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/Image.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/Image.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/Image.h $
 // $Id: Image.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Classification/internal/verbosity.h 5.1-1/include/CGAL/Classification/internal/verbosity.h
--- 5.0.3-1/include/CGAL/Classification/internal/verbosity.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/internal/verbosity.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/internal/verbosity.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/internal/verbosity.h $
 // $Id: verbosity.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Classification/Label.h 5.1-1/include/CGAL/Classification/Label.h
--- 5.0.3-1/include/CGAL/Classification/Label.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/Label.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/Label.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/Label.h $
 // $Id: Label.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Classification/Label_set.h 5.1-1/include/CGAL/Classification/Label_set.h
--- 5.0.3-1/include/CGAL/Classification/Label_set.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/Label_set.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/Label_set.h $
-// $Id: Label_set.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/Label_set.h $
+// $Id: Label_set.h 9e2ce81 2020-04-07T10:15:50+02:00 Simon Giraudot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Simon Giraudot
@@ -38,6 +38,15 @@ public:
 
   Label_set() { }
 
+  /*!
+    \brief Initializes the set with the provided `labels` names.
+  */
+  Label_set (const std::initializer_list<const char*>& labels)
+  {
+    for (const char* l : labels)
+      add(l);
+  }
+
   /// \cond SKIP_IN_MANUAL
   virtual ~Label_set() { }
   /// \endcond
diff -pruN 5.0.3-1/include/CGAL/Classification/Local_eigen_analysis.h 5.1-1/include/CGAL/Classification/Local_eigen_analysis.h
--- 5.0.3-1/include/CGAL/Classification/Local_eigen_analysis.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/Local_eigen_analysis.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/Local_eigen_analysis.h $
-// $Id: Local_eigen_analysis.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/Local_eigen_analysis.h $
+// $Id: Local_eigen_analysis.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Simon Giraudot
@@ -226,7 +226,7 @@ public:
     is `CGAL::Point_3`.
     \tparam NeighborQuery model of `NeighborQuery`
     \tparam ConcurrencyTag enables sequential versus parallel
-    algorithm. Possible values are `Parallel_tag` (default value is %CGAL
+    algorithm. Possible values are `Parallel_tag` (default value if %CGAL
     is linked with TBB) or `Sequential_tag` (default value otherwise).
     \tparam DiagonalizeTraits model of `DiagonalizeTraits` used for
     matrix diagonalization. It can be omitted if Eigen 3 (or greater)
@@ -242,10 +242,8 @@ public:
             typename NeighborQuery,
 #if defined(DOXYGEN_RUNNING)
             typename ConcurrencyTag,
-#elif defined(CGAL_LINKED_WITH_TBB)
-            typename ConcurrencyTag = CGAL::Parallel_tag,
 #else
-            typename ConcurrencyTag = CGAL::Sequential_tag,
+            typename ConcurrencyTag = CGAL::Parallel_if_available_tag,
 #endif
 #if defined(DOXYGEN_RUNNING)
             typename DiagonalizeTraits>
@@ -310,7 +308,7 @@ public:
     \tparam FaceListGraph model of `FaceListGraph`.
     \tparam NeighborQuery model of `NeighborQuery`
     \tparam ConcurrencyTag enables sequential versus parallel
-    algorithm. Possible values are `Parallel_tag` (default value is %CGAL
+    algorithm. Possible values are `Parallel_tag` (default value if %CGAL
     is linked with TBB) or `Sequential_tag` (default value otherwise).
     \tparam DiagonalizeTraits model of `DiagonalizeTraits` used for
     matrix diagonalization. It can be omitted: if Eigen 3 (or greater)
@@ -325,10 +323,8 @@ public:
             typename NeighborQuery,
 #if defined(DOXYGEN_RUNNING)
             typename ConcurrencyTag,
-#elif defined(CGAL_LINKED_WITH_TBB)
-            typename ConcurrencyTag = CGAL::Parallel_tag,
 #else
-            typename ConcurrencyTag = CGAL::Sequential_tag,
+            typename ConcurrencyTag = CGAL::Parallel_if_available_tag,
 #endif
 #if defined(DOXYGEN_RUNNING)
             typename DiagonalizeTraits>
@@ -396,7 +392,7 @@ public:
     `RandomAccessIterator` and its value type is the key type of
     `PointMap`.
     \tparam ConcurrencyTag enables sequential versus parallel
-    algorithm. Possible values are `Parallel_tag` (default value is %CGAL
+    algorithm. Possible values are `Parallel_tag` (default value if %CGAL
     is linked with TBB) or `Sequential_tag` (default value otherwise).
     \tparam DiagonalizeTraits model of `DiagonalizeTraits` used for
     matrix diagonalization. It can be omitted: if Eigen 3 (or greater)
@@ -409,10 +405,8 @@ public:
   template <typename ClusterRange,
 #if defined(DOXYGEN_RUNNING)
             typename ConcurrencyTag,
-#elif defined(CGAL_LINKED_WITH_TBB)
-            typename ConcurrencyTag = CGAL::Parallel_tag,
 #else
-            typename ConcurrencyTag = CGAL::Sequential_tag,
+            typename ConcurrencyTag = CGAL::Parallel_if_available_tag,
 #endif
 #if defined(DOXYGEN_RUNNING)
             typename DiagonalizeTraits>
diff -pruN 5.0.3-1/include/CGAL/Classification/Mesh_feature_generator.h 5.1-1/include/CGAL/Classification/Mesh_feature_generator.h
--- 5.0.3-1/include/CGAL/Classification/Mesh_feature_generator.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/Mesh_feature_generator.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/Mesh_feature_generator.h $
-// $Id: Mesh_feature_generator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/Mesh_feature_generator.h $
+// $Id: Mesh_feature_generator.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Simon Giraudot
@@ -79,7 +79,7 @@ namespace Classification {
   is `GeomTraits::Point_3`.
   \tparam ConcurrencyTag enables sequential versus parallel
   computation of `CGAL::Classification::Local_eigen_analysis`
-  objects. Possible values are `Parallel_tag` (default value is %CGAL
+  objects. Possible values are `Parallel_tag` (default value if %CGAL
   is linked with TBB) or `Sequential_tag` (default value otherwise).
   \tparam DiagonalizeTraits model of `DiagonalizeTraits` used for
   matrix diagonalization. It can be omitted: if Eigen 3 (or greater)
@@ -93,10 +93,8 @@ template <typename GeomTraits,
           typename PointMap,
 #if defined(DOXYGEN_RUNNING)
           typename ConcurrencyTag,
-#elif defined(CGAL_LINKED_WITH_TBB)
-          typename ConcurrencyTag = CGAL::Parallel_tag,
 #else
-          typename ConcurrencyTag = CGAL::Sequential_tag,
+          typename ConcurrencyTag = CGAL::Parallel_if_available_tag,
 #endif
           typename DiagonalizeTraits = CGAL::Default_diagonalize_traits<float,3> >
 class Mesh_feature_generator
diff -pruN 5.0.3-1/include/CGAL/Classification/Mesh_neighborhood.h 5.1-1/include/CGAL/Classification/Mesh_neighborhood.h
--- 5.0.3-1/include/CGAL/Classification/Mesh_neighborhood.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/Mesh_neighborhood.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/Mesh_neighborhood.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/Mesh_neighborhood.h $
 // $Id: Mesh_neighborhood.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Classification/OpenCV/Random_forest_classifier.h 5.1-1/include/CGAL/Classification/OpenCV/Random_forest_classifier.h
--- 5.0.3-1/include/CGAL/Classification/OpenCV/Random_forest_classifier.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/OpenCV/Random_forest_classifier.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/OpenCV/Random_forest_classifier.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/OpenCV/Random_forest_classifier.h $
 // $Id: Random_forest_classifier.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Classification/Planimetric_grid.h 5.1-1/include/CGAL/Classification/Planimetric_grid.h
--- 5.0.3-1/include/CGAL/Classification/Planimetric_grid.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/Planimetric_grid.h	2020-09-08 12:14:48.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/Planimetric_grid.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/Planimetric_grid.h $
 // $Id: Planimetric_grid.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Classification/Point_set_feature_generator.h 5.1-1/include/CGAL/Classification/Point_set_feature_generator.h
--- 5.0.3-1/include/CGAL/Classification/Point_set_feature_generator.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/Point_set_feature_generator.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/Point_set_feature_generator.h $
-// $Id: Point_set_feature_generator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/Point_set_feature_generator.h $
+// $Id: Point_set_feature_generator.h d42113b 2020-03-26T19:35:20+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Simon Giraudot
@@ -90,10 +90,8 @@ template <typename GeomTraits,
           typename PointMap,
 #if defined(DOXYGEN_RUNNING)
           typename ConcurrencyTag,
-#elif defined(CGAL_LINKED_WITH_TBB)
-          typename ConcurrencyTag = CGAL::Parallel_tag,
 #else
-          typename ConcurrencyTag = CGAL::Sequential_tag,
+          typename ConcurrencyTag = CGAL::Parallel_if_available_tag,
 #endif
 #if defined(DOXYGEN_RUNNING)
           typename DiagonalizeTraits>
@@ -163,9 +161,9 @@ private:
       CGAL::Real_timer t;
       t.start();
       if (lower_grid == nullptr)
-        neighborhood = new Neighborhood (input, point_map);
+        neighborhood = new Neighborhood (input, point_map, ConcurrencyTag());
       else
-        neighborhood = new Neighborhood (input, point_map, voxel_size);
+        neighborhood = new Neighborhood (input, point_map, voxel_size, ConcurrencyTag());
       t.stop();
 
       if (lower_grid == nullptr)
diff -pruN 5.0.3-1/include/CGAL/Classification/Point_set_neighborhood.h 5.1-1/include/CGAL/Classification/Point_set_neighborhood.h
--- 5.0.3-1/include/CGAL/Classification/Point_set_neighborhood.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/Point_set_neighborhood.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/Point_set_neighborhood.h $
-// $Id: Point_set_neighborhood.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/Point_set_neighborhood.h $
+// $Id: Point_set_neighborhood.h d42113b 2020-03-26T19:35:20+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Simon Giraudot
@@ -81,7 +81,7 @@ class Point_set_neighborhood
   typedef Search_traits_adapter <boost::uint32_t, My_point_property_map, SearchTraits_3> Search_traits;
   typedef Sliding_midpoint<Search_traits> Splitter;
   typedef Distance_adapter<boost::uint32_t, My_point_property_map, Euclidean_distance<SearchTraits_3> > Distance;
-  typedef Kd_tree<Search_traits, Splitter, Tag_true> Tree;
+  typedef Kd_tree<Search_traits, Splitter, Tag_true, Tag_true> Tree;
   typedef Fuzzy_sphere<Search_traits> Sphere;
   typedef Orthogonal_k_neighbor_search<Search_traits, Distance, Splitter, Tree> Knn;
 
@@ -173,21 +173,42 @@ public:
   /*!
     \brief Constructs a neighborhood object based on the input range.
 
+    \tparam ConcurrencyTag enables sequential versus parallel
+    algorithm. Possible values are `Sequential_tag`, `Parallel_tag`,
+    and `Parallel_if_available_tag`. If no tag is provided,
+    `Parallel_if_available_tag` is used.
+
     \param input point range.
     \param point_map property map to access the input points.
   */
+  template <typename ConcurrencyTag>
   Point_set_neighborhood (const PointRange& input,
-                          PointMap point_map)
+                          PointMap point_map,
+                          const ConcurrencyTag&)
+    : m_tree (nullptr)
+  {
+    init<ConcurrencyTag> (input, point_map);
+  }
+
+  /// \cond SKIP_IN_MANUAL
+  Point_set_neighborhood (const PointRange& input, PointMap point_map)
     : m_tree (nullptr)
   {
+    init<Parallel_if_available_tag> (input, point_map);
+  }
+
+  template <typename ConcurrencyTag>
+  void init (const PointRange& input, PointMap point_map)
+  {
     My_point_property_map pmap (&input, point_map);
     m_tree = new Tree (boost::counting_iterator<boost::uint32_t> (0),
                        boost::counting_iterator<boost::uint32_t> (boost::uint32_t(input.size())),
                        Splitter(),
                        Search_traits (pmap));
     m_distance = Distance (pmap);
-    m_tree->build();
+    m_tree->template build<ConcurrencyTag>();
   }
+  /// \endcond
 
   /*!
     \brief Constructs a simplified neighborhood object based on the input range.
@@ -197,15 +218,37 @@ public:
     present in one cell, only the point closest to the centroid of
     this subset is used.
 
+    \tparam ConcurrencyTag enables sequential versus parallel
+    algorithm. Possible values are `Sequential_tag`, `Parallel_tag`,
+    and `Parallel_if_available_tag`. If no tag is provided,
+    `Parallel_if_available_tag` is used.
+
     \param input input range.
     \param point_map property map to access the input points.
     \param voxel_size size of the cells of the 3D grid used for simplification.
   */
+  template <typename ConcurrencyTag>
+  Point_set_neighborhood (const PointRange& input,
+                          PointMap point_map,
+                          float voxel_size,
+                          const ConcurrencyTag&)
+    : m_tree (nullptr)
+  {
+    init<ConcurrencyTag> (input, point_map, voxel_size);
+  }
+
+  /// \cond SKIP_IN_MANUAL
   Point_set_neighborhood (const PointRange& input,
                           PointMap point_map,
                           float voxel_size)
     : m_tree (nullptr)
   {
+    init<Parallel_if_available_tag> (input, point_map, voxel_size);
+  }
+
+  template <typename ConcurrencyTag>
+  void init (const PointRange& input, PointMap point_map, float voxel_size)
+  {
     // First, simplify
     std::vector<boost::uint32_t> indices;
     My_point_property_map pmap (&input, point_map);
@@ -215,8 +258,9 @@ public:
                        Splitter(),
                        Search_traits (pmap));
     m_distance = Distance (pmap);
-    m_tree->build();
+    m_tree->template build<ConcurrencyTag>();
   }
+  /// \endcond
 
   /// @}
 
diff -pruN 5.0.3-1/include/CGAL/Classification/property_maps.h 5.1-1/include/CGAL/Classification/property_maps.h
--- 5.0.3-1/include/CGAL/Classification/property_maps.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/property_maps.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/property_maps.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/property_maps.h $
 // $Id: property_maps.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Classification/Sum_of_weighted_features_classifier.h 5.1-1/include/CGAL/Classification/Sum_of_weighted_features_classifier.h
--- 5.0.3-1/include/CGAL/Classification/Sum_of_weighted_features_classifier.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/Sum_of_weighted_features_classifier.h	2020-09-08 12:14:48.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/Sum_of_weighted_features_classifier.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/Sum_of_weighted_features_classifier.h $
 // $Id: Sum_of_weighted_features_classifier.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Classification/TensorFlow/Neural_network_classifier.h 5.1-1/include/CGAL/Classification/TensorFlow/Neural_network_classifier.h
--- 5.0.3-1/include/CGAL/Classification/TensorFlow/Neural_network_classifier.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Classification/TensorFlow/Neural_network_classifier.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification/TensorFlow/Neural_network_classifier.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification/TensorFlow/Neural_network_classifier.h $
 // $Id: Neural_network_classifier.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Classification.h 5.1-1/include/CGAL/Classification.h
--- 5.0.3-1/include/CGAL/Classification.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Classification.h	2020-09-08 12:14:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Classification/include/CGAL/Classification.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Classification/include/CGAL/Classification.h $
 // $Id: Classification.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/cluster_point_set.h 5.1-1/include/CGAL/cluster_point_set.h
--- 5.0.3-1/include/CGAL/cluster_point_set.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/cluster_point_set.h	2020-09-08 12:15:22.000000000 +0000
@@ -0,0 +1,276 @@
+// Copyright (c) 2020 GeometryFactory Sarl (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/cluster_point_set.h $
+// $Id: cluster_point_set.h 93f1cd9 2020-07-16T09:53:31+02:00 Mael Rouxel-Labbé
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Simon Giraudot
+
+#ifndef CGAL_CLUSTER_POINT_SET_H
+#define CGAL_CLUSTER_POINT_SET_H
+
+#include <CGAL/license/Point_set_processing_3.h>
+
+#include <CGAL/squared_distance_3.h>
+#include <CGAL/Point_set_processing_3/internal/Neighbor_query.h>
+#include <CGAL/Point_set_processing_3/internal/Callback_wrapper.h>
+#include <CGAL/Point_set_processing_3/internal/bbox_diagonal.h>
+#include <CGAL/for_each.h>
+
+#include <CGAL/boost/graph/Named_function_parameters.h>
+#include <CGAL/boost/graph/named_params_helper.h>
+
+#include <queue>
+
+namespace CGAL
+{
+
+
+/// \cond SKIP_IN_MANUAL
+namespace Point_set_processing_3
+{
+
+namespace internal
+{
+
+// Trick to both compile version with Emptyset_iterator and with
+// user-provided OutputIterator. Many output iterators (such as
+// `std::back_insert_iterator`) cannot be default constructed, which
+// makes the mechanism `choose_param(get_param(...),Default())` fails.
+template <typename NamedParameters, typename OutputIterator>
+OutputIterator get_adjacencies (const NamedParameters& np, OutputIterator*)
+{
+  return CGAL::parameters::get_parameter(np, internal_np::adjacencies);
+}
+
+template <typename NamedParameters>
+CGAL::Emptyset_iterator get_adjacencies (const NamedParameters&, CGAL::Emptyset_iterator*)
+{
+  return CGAL::Emptyset_iterator();
+}
+
+} // namespace internal
+
+} // namespace Point_set_processing_3
+/// \endcond
+
+// ----------------------------------------------------------------------------
+// Public section
+// ----------------------------------------------------------------------------
+
+/**
+   \ingroup PkgPointSetProcessing3Algorithms
+   Identifies connected components on a nearest neighbor graph built
+   using a query sphere of fixed radius centered on each point.
+
+   \tparam PointRange is a model of `Range`. The value type of its
+   iterator is the key type of the named parameter `point_map`.
+   \tparam ClusterMap is a model of `ReadWritePropertyMap` with value
+   type `std::size_t`.
+
+   \param points input point range.
+   \param cluster_map maps each point to the index of the cluster it belongs to.
+   \param np optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below.
+
+   \cgalNamedParamsBegin
+     \cgalParamNBegin{point_map}
+       \cgalParamDescription{a property map associating points to the elements of the point set `points`}
+       \cgalParamType{a model of `ReadablePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange` and whose value type is `geom_traits::Point_3`}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{callback}
+       \cgalParamDescription{a mechanism to get feedback on the advancement of the algorithm
+                             while it's running and to interrupt it if needed}
+       \cgalParamType{an instance of `std::function<bool(double)>`.}
+       \cgalParamDefault{unused}
+       \cgalParamExtra{It is called regularly when the
+                       algorithm is running: the current advancement (between 0. and
+                       1.) is passed as parameter. If it returns `true`, then the
+                       algorithm continues its execution normally; if it returns
+                       `false`, the algorithm is stopped and the number of already
+                       computed clusters is returned.}
+       \cgalParamExtra{The callback will be copied and therefore needs to be lightweight.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{neighbor_radius}
+       \cgalParamDescription{the spherical neighborhood radius}
+       \cgalParamType{floating scalar value}
+       \cgalParamDefault{`1` percent of the bounding box diagonal}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{attraction_factor}
+       \cgalParamDescription{used to compute adjacencies between clusters.
+                             Adjacencies are computed using a nearest neighbor graph built similarly
+                             to the one used for clustering, using `attraction_factor * neighbor_radius` as
+                             parameter.}
+       \cgalParamType{floating scalar value}
+       \cgalParamDefault{`2`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{adjacencies}
+       \cgalParamDescription{an output iterator used to output pairs containing the indices of two adjacent clusters.}
+       \cgalParamType{a model of `OutputIterator` that accepts objects of type `std::pair<std::size_t, std::size_t>`}
+       \cgalParamDefault{`CGAL::Emptyset_iterator`}
+       \cgalParamExtra{If this parameter is not used, adjacencies are not computed at all.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{geom_traits}
+       \cgalParamDescription{an instance of a geometric traits class}
+       \cgalParamType{a model of `Kernel`}
+       \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+     \cgalParamNEnd
+   \cgalNamedParamsEnd
+
+   \return the number of clusters identified.
+*/
+template <typename PointRange, typename ClusterMap, typename NamedParameters>
+std::size_t cluster_point_set (PointRange& points,
+                               ClusterMap cluster_map,
+                               const NamedParameters& np)
+{
+  using parameters::choose_parameter;
+  using parameters::get_parameter;
+
+  // basic geometric types
+  typedef typename PointRange::iterator iterator;
+  typedef typename iterator::value_type value_type;
+  typedef typename CGAL::GetPointMap<PointRange, NamedParameters>::type PointMap;
+  typedef typename Point_set_processing_3::GetK<PointRange, NamedParameters>::Kernel Kernel;
+  typedef typename Point_set_processing_3::GetAdjacencies<PointRange, NamedParameters>::type Adjacencies;
+
+  CGAL_static_assertion_msg(!(boost::is_same<typename GetSvdTraits<NamedParameters>::type,
+                                             typename GetSvdTraits<NamedParameters>::NoTraits>::value),
+                            "Error: no SVD traits");
+
+  PointMap point_map = choose_parameter(get_parameter(np, internal_np::point_map), PointMap());
+  typename Kernel::FT neighbor_radius = choose_parameter(get_parameter(np, internal_np::neighbor_radius),
+                                                         typename Kernel::FT(-1));
+  typename Kernel::FT factor = choose_parameter(get_parameter(np, internal_np::attraction_factor),
+                                                typename Kernel::FT(2));
+
+  const std::function<bool(double)>& callback = choose_parameter(get_parameter(np, internal_np::callback),
+                                                               std::function<bool(double)>());
+
+  double callback_factor = 1.;
+  if (!std::is_same<Adjacencies,
+      typename Point_set_processing_3::GetAdjacencies<PointRange, NamedParameters>::Empty>::value)
+    callback_factor = 0.5;
+
+  // types for K nearest neighbors search structure
+  typedef Point_set_processing_3::internal::Neighbor_query<Kernel, PointRange&, PointMap> Neighbor_query;
+
+  // precondition: at least one element in the container.
+  // to fix: should have at least three distinct points
+  // but this is costly to check
+  CGAL_point_set_processing_precondition(points.begin() != points.end());
+
+  // If no radius is given, init with 1% of bbox diagonal
+  if (neighbor_radius < 0)
+    neighbor_radius = 0.01 * Point_set_processing_3::internal::bbox_diagonal (points, point_map);
+
+  // Init cluster map with -1
+  for (const value_type& p : points)
+    put (cluster_map, p, -1);
+
+  Neighbor_query neighbor_query (points, point_map);
+
+  std::queue<iterator> todo;
+  std::size_t nb_clusters = 0;
+
+  // Flooding algorithm from each point
+  std::size_t done = 0;
+  std::size_t size = points.size();
+
+  for (iterator it = points.begin(); it != points.end(); ++ it)
+  {
+    const value_type& p = *it;
+
+    if (int(get (cluster_map, p)) != -1)
+      continue;
+
+    todo.push (it);
+
+    while (!todo.empty())
+    {
+      iterator current = todo.front();
+      todo.pop();
+
+      if (int(get (cluster_map, *current)) != -1)
+        continue;
+
+      put (cluster_map, *current, nb_clusters);
+      ++ done;
+
+      if (callback && !callback (callback_factor * (done + 1) / double(size)))
+        return (nb_clusters + 1);
+
+      neighbor_query.get_iterators (get (point_map, *current), 0, neighbor_radius,
+                                    boost::make_function_output_iterator
+                                    ([&](const iterator& it) { todo.push(it); }), true);
+
+    }
+
+    ++ nb_clusters;
+  }
+
+  if (!std::is_same<Adjacencies,
+      typename Point_set_processing_3::GetAdjacencies<PointRange, NamedParameters>::Empty>::value)
+  {
+    Adjacencies adjacencies = Point_set_processing_3::internal::get_adjacencies(np, (Adjacencies*)(nullptr));
+    neighbor_radius *= factor;
+
+    std::vector<iterator> neighbors;
+    std::vector<std::pair<std::size_t, std::size_t> > adj;
+
+    done = 0;
+    for (const value_type& p : points)
+    {
+      std::size_t c0 = get (cluster_map, p);
+
+      neighbors.clear();
+      neighbor_query.get_iterators (get (point_map, p), 0, neighbor_radius,
+                                    std::back_inserter (neighbors), 0);
+
+      for (const iterator& it : neighbors)
+      {
+        std::size_t c1 = get (cluster_map, *it);
+        if (c0 < c1)
+          adj.push_back (std::make_pair (c0, c1));
+        else if (c0 > c1)
+          adj.push_back (std::make_pair (c1, c0));
+        // else c0 == c1, ignore
+      }
+
+      ++ done;
+      if (callback && !callback (callback_factor + callback_factor * (done + 1) / double(size)))
+        return nb_clusters;
+    }
+    std::sort (adj.begin(), adj.end());
+    auto last = std::unique (adj.begin(), adj.end());
+    std::copy (adj.begin(), last, adjacencies);
+  }
+
+  return nb_clusters;
+}
+
+/// \cond SKIP_IN_MANUAL
+// overload with default NP
+template <typename PointRange, typename ClusterMap>
+std::size_t cluster_point_set (PointRange& points,
+                               ClusterMap cluster_map,
+                               unsigned int k)
+{
+  return cluster_point_set (points, cluster_map, k,
+                            CGAL::Point_set_processing_3::parameters::all_default(points));
+}
+/// \endcond
+
+} // namespace CGAL
+
+
+#endif // CGAL_CLUSTER_POINT_SET_H
diff -pruN 5.0.3-1/include/CGAL/CMap_linear_cell_complex_storages.h 5.1-1/include/CGAL/CMap_linear_cell_complex_storages.h
--- 5.0.3-1/include/CGAL/CMap_linear_cell_complex_storages.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/CMap_linear_cell_complex_storages.h	2020-09-08 12:15:05.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Linear_cell_complex/include/CGAL/CMap_linear_cell_complex_storages.h $
-// $Id: CMap_linear_cell_complex_storages.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Linear_cell_complex/include/CGAL/CMap_linear_cell_complex_storages.h $
+// $Id: CMap_linear_cell_complex_storages.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
@@ -150,6 +150,8 @@ namespace CGAL {
       CGAL_assertion(i <= dimension);
       return dh->mf[i]==null_dart_handle;
     }
+    bool is_perforated(Dart_const_handle /*dh*/) const
+    { return false; }
 
     /// Set simultaneously all the marks of this dart to a given value.
     void set_dart_marks(Dart_const_handle ADart,
diff -pruN 5.0.3-1/include/CGAL/Coercion_traits.h 5.1-1/include/CGAL/Coercion_traits.h
--- 5.0.3-1/include/CGAL/Coercion_traits.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/Coercion_traits.h	2020-09-08 12:14:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_foundations/include/CGAL/Coercion_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_foundations/include/CGAL/Coercion_traits.h $
 // $Id: Coercion_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Combination_enumerator.h 5.1-1/include/CGAL/Combination_enumerator.h
--- 5.0.3-1/include/CGAL/Combination_enumerator.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Combination_enumerator.h	2020-09-08 12:14:51.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Generator/include/CGAL/Combination_enumerator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Generator/include/CGAL/Combination_enumerator.h $
 // $Id: Combination_enumerator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Combinatorial_map_basic_operations.h 5.1-1/include/CGAL/Combinatorial_map_basic_operations.h
--- 5.0.3-1/include/CGAL/Combinatorial_map_basic_operations.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Combinatorial_map_basic_operations.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Combinatorial_map/include/CGAL/Combinatorial_map_basic_operations.h $
-// $Id: Combinatorial_map_basic_operations.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Combinatorial_map/include/CGAL/Combinatorial_map_basic_operations.h $
+// $Id: Combinatorial_map_basic_operations.h 3fb644e 2020-04-10T17:40:30+02:00 Guillaume Damiand
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
@@ -81,7 +81,7 @@ namespace CGAL
                                typename Map::size_type amark)
   {
     amap.negate_mark(amark);
-    bool res=CGAL::is_whole_orbit_marked<Map,Iterator>(amap, adart, amark);
+    bool res=CGAL::template is_whole_orbit_marked<Map,Iterator>(amap, adart, amark);
     amap.negate_mark(amark);
     return res;
   }
@@ -128,7 +128,7 @@ namespace CGAL
   {
     amap.negate_mark(amark);
     typename Map::size_type
-        res=CGAL::mark_orbit<Map, Iterator>(amap, adart, amark);
+        res=CGAL::template mark_orbit<Map, Iterator>(amap, adart, amark);
     amap.negate_mark(amark);
     return res;
   }
@@ -154,7 +154,7 @@ namespace CGAL
                            typename Map::Dart_const_handle adart1,
                            typename Map::Dart_const_handle adart2)
   {
-    return CGAL::belong_to_same_cell<Map,i,Map::dimension>(amap,adart1,adart2);
+    return CGAL::template belong_to_same_cell<Map,i,Map::dimension>(amap,adart1,adart2);
   }
 
 
@@ -169,7 +169,7 @@ namespace CGAL
                             typename Map::Dart_const_handle adart,
                             typename Map::size_type amark)
   {
-    return CGAL::is_whole_orbit_marked<Map,
+    return CGAL::template is_whole_orbit_marked<Map,
         typename Map::template Dart_of_cell_range<i,d>::const_iterator>
         (amap, adart, amark);
   }
@@ -179,7 +179,8 @@ namespace CGAL
                             typename Map::Dart_const_handle adart,
                             typename Map::size_type amark)
   {
-    return CGAL::is_whole_cell_marked<Map,i,Map::dimension>(amap,adart,amark);
+    return CGAL::template is_whole_cell_marked<Map,i,Map::dimension>
+      (amap,adart,amark);
   }
 
   /** Test if all the darts of a given cell are unmarked.
@@ -193,8 +194,8 @@ namespace CGAL
                               typename Map::Dart_const_handle adart,
                               typename Map::size_type amark)
   {
-    return CGAL::is_whole_orbit_unmarked<Map,
-        typename Map::template Dart_of_cell_range<i,d>::iterator>
+    return CGAL::template is_whole_orbit_unmarked<Map,
+        typename Map::template Dart_of_cell_range<i,d>::const_iterator>
         (amap, adart, amark);
   }
 
@@ -203,7 +204,7 @@ namespace CGAL
                               typename Map::Dart_const_handle adart,
                               typename Map::size_type amark)
   {
-    return CGAL::is_whole_cell_unmarked<Map,i,Map::dimension>
+    return CGAL::template is_whole_cell_unmarked<Map,i,Map::dimension>
         (amap,adart,amark);
   }
 
@@ -218,7 +219,7 @@ namespace CGAL
   typename Map::size_type mark_cell(const Map & amap,
                                     typename Map::Dart_const_handle adart,
                                     typename Map::size_type amark)
-  { return CGAL::mark_orbit<Map,
+  { return CGAL::template mark_orbit<Map,
         typename Map::template Dart_of_cell_basic_range<i,d>::const_iterator>
         (amap, adart, amark); }
 
@@ -226,9 +227,9 @@ namespace CGAL
   typename Map::size_type mark_cell(const Map & amap,
                                     typename Map::Dart_const_handle adart,
                                     typename Map::size_type amark)
-  { return CGAL::mark_cell<Map,i,Map::dimension>(amap, adart, amark);}
+  { return CGAL::template mark_cell<Map,i,Map::dimension>(amap, adart, amark);}
 
-  /** Unmark a given orbit with a given mark.
+  /** Unmark a given cell with a given mark.
    * @param amap a combinatorial map.
    * @param adart a dart of the cell.
    * @param amark the mark.
@@ -239,7 +240,7 @@ namespace CGAL
   typename Map::size_type unmark_cell(const Map & amap,
                                       typename Map::Dart_const_handle adart,
                                       typename Map::size_type amark)
-  { return CGAL::unmark_orbit<Map,
+  { return CGAL::template unmark_orbit<Map,
         typename Map::template Dart_of_cell_basic_range<i,d>::const_iterator>
         (amap, adart, amark);}
 
@@ -247,7 +248,156 @@ namespace CGAL
   typename Map::size_type unmark_cell(const Map & amap,
                                       typename Map::Dart_const_handle adart,
                                       typename Map::size_type amark)
-  { return CGAL::unmark_cell<Map,i,Map::dimension>(amap, adart, amark); }
+  { return CGAL::template unmark_cell<Map,i,Map::dimension>(amap, adart, amark); }
+
+  /// Functor to test if the given dart must be marked, when we want
+  /// to mark an oriented orbit. By default (for CMap), return true.
+  template<class Map, class T=typename Map::Combinatorial_data_structure>
+  struct Need_to_mark_for_oriented_cell
+  {
+    static bool run(const Map & /*amap*/,
+                    typename Map::Dart_const_handle /*adart*/,
+                    typename Map::size_type /*amark*/,
+                    OperationState /*op*/)
+    { return true; }
+  };
+
+  struct Generalized_map_tag;
+
+  /// Specialization for GMap. We mark one out of two darts.
+  template<class Map>
+  struct Need_to_mark_for_oriented_cell<Map, CGAL::Generalized_map_tag>
+  {
+    static bool run(const Map & amap,
+                    typename Map::Dart_const_handle adart,
+                    typename Map::size_type amark,
+                    OperationState op)
+    {
+      CGAL_assertion(op!=OP_END);
+
+      if (op==OP_NONE) return true;
+
+      bool all_darts_unmarked=true;
+      for (int i=0; all_darts_unmarked && i<=(int)Map::dimension; ++i)
+      {
+        if (!amap.is_free(adart, i) &&
+            amap.is_marked(amap.alpha(adart, i), amark))
+        { all_darts_unmarked=false; }
+      }
+      return all_darts_unmarked;
+    }
+  };
+
+  /** Mark a given oriented orbit with a given mark. For combinatorial map,
+   *  equivalent to mark_orbit. For generalized map, mark one out of two dart
+   *  if the orbit is orientable. If amark2!=INVALID_MARK, mark also all the darts
+   *  of the orbit with amark2.
+   * @param amap a combinatorial map / generalized.
+   * @param adart a dart of the orbit.
+   * @param amark the mark.
+   * @param amark2 a second mark.
+   * @return the number of darts of the marked orbit.
+   * @pre The whole orbit must be unmarked.
+   */
+  template < class Map, class Iterator >
+  typename Map::size_type mark_oriented_orbit(const Map & amap,
+                                              typename Map::Dart_const_handle adart,
+                                              typename Map::size_type amark,
+                                              typename Map::size_type amark2=Map::INVALID_MARK)
+  {
+    CGAL_static_assertion( (boost::is_same<typename Iterator::Basic_iterator,
+                            Tag_true>::value) );
+    CGAL_assertion( (is_whole_orbit_unmarked<Map,
+                     CMap_non_basic_iterator<Map,Iterator> >
+                     (amap, adart, amark)) );
+    typename Map::size_type res=0;
+    typename Map::size_type amark3=(amark2==Map::INVALID_MARK?amap.get_new_mark():amark2);
+    for (Iterator it(amap, adart, amark3); it.cont(); ++it)
+    {
+      amap.mark(it, amark3);
+      if (CGAL::template Need_to_mark_for_oriented_cell<Map>::
+          run(amap, it, amark, it.prev_operation()))
+      {
+        amap.mark(it, amark);
+        ++res;
+      }
+    }
+
+    if (amark2==Map::INVALID_MARK)
+    {
+      CGAL::template unmark_orbit<Map, Iterator>(amap, adart, amark3);
+      amap.free_mark(amark3);
+    }
+
+    return res;
+  }
+
+  template < class Map, class Iterator >
+  typename Map::size_type unmark_oriented_orbit(const Map & amap,
+                                                typename Map::Dart_const_handle adart,
+                                                typename Map::size_type amark,
+                                                typename Map::size_type amark2=Map::INVALID_MARK)
+  {
+    amap.negate_mark(amark);
+    if (amark2!=Map::INVALID_MARK) { amap.negate_mark(amark2); }
+    typename Map::size_type
+      res=CGAL::template mark_oriented_orbit<Map, Iterator>(amap, adart, amark, amark2);
+    amap.negate_mark(amark);
+    if (amark2!=Map::INVALID_MARK) { amap.negate_mark(amark2); }
+    return res;
+  }
+
+  /** Mark a given oriented cell with a given mark. For combinatorial map,
+   *  equivalent to mark_cell. For generalized map, mark one out of two dart
+   *  if the cell is orientable. If amark2!=INVALID_MARK, mark also all the darts
+   *  of the cell with amark2.
+   * @param amap a combinatorial map / generalized.
+   * @param adart a dart of the cell.
+   * @param amark the mark.
+   * @param amark2 a second mark.
+   * @return the number of darts of the marked cell.
+   * @pre The whole cell must be unmarked.
+   */
+  template < class Map, unsigned int i, unsigned int d >
+  typename Map::size_type mark_oriented_cell(const Map & amap,
+                                             typename Map::Dart_const_handle adart,
+                                             typename Map::size_type amark,
+                                             typename Map::size_type amark2=Map::INVALID_MARK)
+  { return CGAL::template mark_oriented_orbit
+      <Map, typename Map::template Dart_of_cell_basic_range<i,d>::const_iterator>
+      (amap, adart, amark, amark2); }
+
+  template < class Map, unsigned int i>
+  typename Map::size_type mark_oriented_cell(const Map & amap,
+                                             typename Map::Dart_const_handle adart,
+                                             typename Map::size_type amark,
+                                             typename Map::size_type amark2=Map::INVALID_MARK)
+  { return CGAL::template mark_oriented_cell<Map,i,Map::dimension>
+      (amap, adart, amark, amark2);}
+
+  /** Unmark a given oriented cell with a given mark.
+   * @param amap a combinatorial map.
+   * @param adart a dart of the cell.
+   * @param amark the mark.
+   * @return the number of darts of the unmarked cell.
+   * @pre The whole cell must be marked.
+   */
+  template < class Map, unsigned int i, unsigned int d >
+  typename Map::size_type unmark_oriented_cell(const Map & amap,
+                                               typename Map::Dart_const_handle adart,
+                                               typename Map::size_type amark,
+                                               typename Map::size_type amark2=Map::INVALID_MARK)
+  { return CGAL::template unmark_oriented_orbit
+      <Map, typename Map::template Dart_of_cell_basic_range<i,d>::const_iterator>
+      (amap, adart, amark, amark2);}
+
+  template < class Map, unsigned int i >
+  typename Map::size_type unmark_oriented_cell(const Map & amap,
+                                               typename Map::Dart_const_handle adart,
+                                               typename Map::size_type amark,
+                                               typename Map::size_type amark2=Map::INVALID_MARK)
+  { return CGAL::template unmark_oriented_cell<Map,i,Map::dimension>
+      (amap, adart, amark, amark2); }
 
   /** Compute the degree of a given i-cell c.
    * The degree is the number of distinct i+1 cells incident to c.
diff -pruN 5.0.3-1/include/CGAL/Combinatorial_map_constructors.h 5.1-1/include/CGAL/Combinatorial_map_constructors.h
--- 5.0.3-1/include/CGAL/Combinatorial_map_constructors.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Combinatorial_map_constructors.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Combinatorial_map/include/CGAL/Combinatorial_map_constructors.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Combinatorial_map/include/CGAL/Combinatorial_map_constructors.h $
 // $Id: Combinatorial_map_constructors.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Combinatorial_map_functors.h 5.1-1/include/CGAL/Combinatorial_map_functors.h
--- 5.0.3-1/include/CGAL/Combinatorial_map_functors.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Combinatorial_map_functors.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Combinatorial_map/include/CGAL/Combinatorial_map_functors.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Combinatorial_map/include/CGAL/Combinatorial_map_functors.h $
 // $Id: Combinatorial_map_functors.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Combinatorial_map_fwd.h 5.1-1/include/CGAL/Combinatorial_map_fwd.h
--- 5.0.3-1/include/CGAL/Combinatorial_map_fwd.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Combinatorial_map_fwd.h	2020-09-08 12:14:48.000000000 +0000
@@ -0,0 +1,51 @@
+// Copyright (c) 2010-2011 CNRS and LIRIS' Establishments (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Combinatorial_map/include/CGAL/Combinatorial_map_fwd.h $
+// $Id: Combinatorial_map_fwd.h 0308d1a 2020-03-27T18:35:15+01:00 Guillaume Damiand
+// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
+//
+#ifndef COMBINATORIAL_MAP_FWD_H
+#define COMBINATORIAL_MAP_FWD_H 1
+
+#include <CGAL/memory.h>
+
+namespace CGAL {
+
+#if defined(CGAL_CMAP_DART_DEPRECATED) && !defined(CGAL_NO_DEPRECATED_CODE)
+template <unsigned int d>
+struct Combinatorial_map_min_items;
+#else
+struct Generic_map_min_items;
+#endif
+
+template<unsigned int d_, class Items_, class Alloc_ >
+class Combinatorial_map_storage_1;
+
+template < unsigned int d_, class Refs_,
+#if defined(CGAL_CMAP_DART_DEPRECATED) && !defined(CGAL_NO_DEPRECATED_CODE)
+           class Items_=Combinatorial_map_min_items<d_>,
+#else
+           class Items_=Generic_map_min_items,
+#endif
+           class Alloc_=CGAL_ALLOCATOR(int),
+           class Storage_= Combinatorial_map_storage_1<d_, Items_, Alloc_> >
+class Combinatorial_map_base;
+
+template < unsigned int d_,
+#if defined(CGAL_CMAP_DART_DEPRECATED) && !defined(CGAL_NO_DEPRECATED_CODE)
+           class Items_=Combinatorial_map_min_items<d_>,
+#else
+           class Items_=Generic_map_min_items,
+#endif
+           class Alloc_=CGAL_ALLOCATOR(int),
+           class Storage_= Combinatorial_map_storage_1<d_, Items_, Alloc_> >
+class Combinatorial_map;
+
+} // CGAL
+
+#endif // COMBINATORIAL_MAP_FWD_H
diff -pruN 5.0.3-1/include/CGAL/Combinatorial_map.h 5.1-1/include/CGAL/Combinatorial_map.h
--- 5.0.3-1/include/CGAL/Combinatorial_map.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Combinatorial_map.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Combinatorial_map/include/CGAL/Combinatorial_map.h $
-// $Id: Combinatorial_map.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Combinatorial_map/include/CGAL/Combinatorial_map.h $
+// $Id: Combinatorial_map.h e6536aa 2020-06-09T20:15:16+02:00 Laurent Rineau
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
@@ -14,6 +14,8 @@
 
 #include <CGAL/disable_warnings.h>
 
+#include <CGAL/Combinatorial_map_fwd.h>
+
 #include <CGAL/internal/Combinatorial_map_internal_functors.h>
 #include <CGAL/internal/Combinatorial_map_utility.h>
 #include <CGAL/internal/Combinatorial_map_group_functors.h>
@@ -39,8 +41,12 @@
 #include <bitset>
 #include <vector>
 #include <deque>
+#include <tuple>
 #include <boost/type_traits/is_same.hpp>
 #include <boost/unordered_map.hpp>
+#include <boost/graph/graph_traits.hpp>
+#include <CGAL/boost/graph/helpers.h>
+
 #include <CGAL/config.h>
 
 #if defined( __INTEL_COMPILER )
@@ -55,26 +61,31 @@ _Pragma("GCC diagnostic ignored \"-Warra
 #endif
 
 namespace CGAL {
+  // functions to allow the call to next/opposite by ADL
+  template <typename G, typename Desc>
+  auto CM_ADL_next(Desc&& d, G&& g) {
+    return next(std::forward<Desc>(d), std::forward<G>(g));
+  }
+
+  template <typename G, typename Desc>
+  auto CM_ADL_opposite(Desc&& d, G&& g) {
+    return opposite(std::forward<Desc>(d), std::forward<G>(g));
+  }
 
   /** @file Combinatorial_map.h
    * Definition of generic dD Combinatorial map.
    */
 
   struct Combinatorial_map_tag {};
+  struct Generalized_map_tag;
 
   /** Generic definition of combinatorial map in dD.
    * The Combinatorial_map class describes an dD combinatorial map. It allows
    * mainly to create darts, to use marks onto these darts, to get and set
    * the beta links, and to manage enabled attributes.
    */
-  template < unsigned int d_, class Refs,
-#if defined(CGAL_CMAP_DART_DEPRECATED) && !defined(CGAL_NO_DEPRECATED_CODE)
-             class Items_=Combinatorial_map_min_items<d_>,
-#else
-             class Items_=Generic_map_min_items,
-#endif
-             class Alloc_=CGAL_ALLOCATOR(int),
-             class Storage_= Combinatorial_map_storage_1<d_, Items_, Alloc_> >
+  template < unsigned int d_, class Refs_, class Items_,
+             class Alloc_, class Storage_ >
   class Combinatorial_map_base: public Storage_
   {
     template<typename CMap,unsigned int i,typename Enabled>
@@ -101,9 +112,10 @@ namespace CGAL {
     typedef Combinatorial_map_tag Combinatorial_data_structure;
 
     /// Types definition
-    typedef Storage_                                                    Storage;
-    typedef Storage                                                     Base;
-    typedef Combinatorial_map_base<d_, Refs, Items_, Alloc_, Storage_ > Self;
+    typedef Storage_                                                     Storage;
+    typedef Storage                                                      Base;
+    typedef Combinatorial_map_base<d_, Refs_, Items_, Alloc_, Storage_ > Self;
+    typedef Refs_                                                        Refs;
     typedef typename Base::Dart Dart;
     typedef typename Base::Dart_handle Dart_handle;
     typedef typename Base::Dart_const_handle Dart_const_handle;
@@ -199,21 +211,36 @@ namespace CGAL {
       CGAL_assertion(number_of_darts()==0);
     }
 
-    /** Copy the given combinatorial map into *this.
+    /** Copy the given combinatorial map 'amap' into *this.
      *  Note that both CMap can have different dimensions and/or non void attributes.
+     *  Here CMap2 is necessarily non const; while Dart_handle_2 can be a const or non const handle.
+     *  This is the "generic" method, called by the different variants below.
+     *  Marks reserved and automatic attributes management are not updated.
      *  @param amap the combinatorial map to copy.
+     *  @param origin_to_copy associative array from original darts to copy darts
+     *  @param origin_to_copy associative array from copy darts to original darts
+     *  @param converters tuple of functors, one per attribute, to transform original attributes into copies
+     *  @param dartinfoconverter functor to transform original information of darts into information of copies
+     *  @param pointconverter functor to transform points in original map into points of copies.
+     *  @param copy_perforated_darts true to copy also darts marked perforated (if any)
+     *  @param mark_perforated_darts true to mark darts wich are copies of perforated darts (if any)
      *  @post *this is valid.
      */
-    template <typename CMap2, typename Converters, typename DartInfoConverter,
+    template <typename CMap2, typename Dart_handle_2,
+              typename Converters, typename DartInfoConverter,
               typename PointConverter>
-    void copy(const CMap2& amap, const Converters& converters,
-              const DartInfoConverter& dartinfoconverter,
-              const PointConverter& pointconverter,
-              boost::unordered_map<typename CMap2::Dart_const_handle, Dart_handle>* dart_mapping=nullptr)
+    void generic_copy(CMap2& amap,
+                      boost::unordered_map<Dart_handle_2, Dart_handle>* origin_to_copy,
+                      boost::unordered_map<Dart_handle, Dart_handle_2>* copy_to_origin,
+                      const Converters& converters,
+                      const DartInfoConverter& dartinfoconverter,
+                      const PointConverter& pointconverter,
+                      bool copy_perforated_darts=false,
+                      size_type mark_perforated=INVALID_MARK)
     {
       this->clear();
 
-      this->mnb_used_marks = amap.mnb_used_marks;
+      /*this->mnb_used_marks = amap.mnb_used_marks;
       this->mmask_marks    = amap.mmask_marks;
       this->automatic_attributes_management =
           amap.automatic_attributes_management;
@@ -227,121 +254,260 @@ namespace CGAL {
         this->mnb_times_reserved_marks[i] = amap.mnb_times_reserved_marks[i];
       }
 
-      init_dart(null_dart_handle, amap.get_marks(amap.null_dart_handle));
+      init_dart(null_dart_handle, amap.get_marks(amap.null_dart_handle));*/
 
       // Create an mapping between darts of the two maps (originals->copies).
       // (here we cannot use CGAL::Unique_hash_map because it does not provide
       // iterators...
-      boost::unordered_map<typename CMap2::Dart_const_handle, Dart_handle> local_dartmap;
-      if (dart_mapping==nullptr)
-      { dart_mapping=&local_dartmap; }
+      boost::unordered_map<Dart_handle_2, Dart_handle> local_dartmap;
+      if (origin_to_copy==NULL) // Use local_dartmap if user does not provides its own unordered_map
+      { origin_to_copy=&local_dartmap; }
 
-      for (typename CMap2::Dart_const_range::const_iterator
-             it=amap.darts().begin(), itend=amap.darts().end();
-           it!=itend; ++it)
+      Dart_handle new_dart;
+      for (typename CMap2::Dart_range::iterator it=amap.darts().begin(),
+             itend=amap.darts().end(); it!=itend; ++it)
       {
-        (*dart_mapping)[it]=mdarts.emplace();
-        init_dart((*dart_mapping)[it], amap.get_marks(it));
-        internal::Copy_dart_info_functor<CMap2, Refs, DartInfoConverter>::run
-            (amap, static_cast<Refs&>(*this), it, (*dart_mapping)[it],
-             dartinfoconverter);
+        if (copy_perforated_darts || !amap.is_perforated(it))
+        {
+          new_dart=mdarts.emplace();
+          init_dart(new_dart); // , amap.get_marks(it));
+
+          if (mark_perforated!=INVALID_MARK && amap.is_perforated(it))
+          { mark(new_dart, mark_perforated); }
+
+          (*origin_to_copy)[it]=new_dart;
+          if (copy_to_origin!=NULL) { (*copy_to_origin)[new_dart]=it; }
+
+          internal::Copy_dart_info_functor
+            <typename CMap2::Refs, Refs, DartInfoConverter>::run
+            (static_cast<typename CMap2::Refs&>(amap), static_cast<Refs&>(*this),
+             it, new_dart, dartinfoconverter);
+        }
       }
 
       unsigned int min_dim=(dimension<amap.dimension?dimension:amap.dimension);
 
-      typename boost::unordered_map<typename CMap2::Dart_const_handle,Dart_handle>
-        ::iterator dartmap_iter, dartmap_iter_end=dart_mapping->end();
-      for (dartmap_iter=dart_mapping->begin(); dartmap_iter!=dartmap_iter_end;
+      typename boost::unordered_map<Dart_handle_2,Dart_handle>::iterator
+        dartmap_iter, dartmap_iter_end=origin_to_copy->end();
+      for (dartmap_iter=origin_to_copy->begin(); dartmap_iter!=dartmap_iter_end;
            ++dartmap_iter)
       {
         for (unsigned int i=0; i<=min_dim; i++)
         {
           if (!amap.is_free(dartmap_iter->first,i) &&
-              (dartmap_iter->first)<(amap.beta(dartmap_iter->first,i)))
+              is_free(dartmap_iter->second,i))
           {
             basic_link_beta(dartmap_iter->second,
-                            (*dart_mapping)[amap.beta(dartmap_iter->first,i)], i);
+                            (*origin_to_copy)[amap.beta(dartmap_iter->first,i)], i);
           }
         }
       }
 
       /** Copy attributes */
-      for (dartmap_iter=dart_mapping->begin(); dartmap_iter!=dartmap_iter_end;
+      for (dartmap_iter=origin_to_copy->begin(); dartmap_iter!=dartmap_iter_end;
            ++dartmap_iter)
       {
         Helper::template Foreach_enabled_attributes
-          < internal::Copy_attributes_functor <CMap2, Refs, Converters,
-            PointConverter> >::
-          run(amap, static_cast<Refs&>(*this),
+          < internal::Copy_attributes_functor<typename CMap2::Refs, Refs,
+                                              Converters, PointConverter> >::
+          run(static_cast<const typename CMap2::Refs&>(amap), static_cast<Refs&>(*this),
               dartmap_iter->first, dartmap_iter->second,
               converters, pointconverter);
       }
 
-      CGAL_assertion (is_valid () == 1);
+      CGAL_assertion (is_valid());
     }
 
-    template <typename CMap2>
-    void copy(const CMap2& amap,
-              boost::unordered_map<typename CMap2::Dart_const_handle, Dart_handle>* dart_mapping=nullptr)
-
+    // (1a) copy(amap, converters, dartinfoconverter, pointconverter)
+    template<typename CMap2, typename Converters, typename DartInfoConverter,
+             typename PointConverter>
+    void copy(CMap2& amap,
+              boost::unordered_map
+              <typename CMap2::Dart_handle, Dart_handle>* origin_to_copy,
+              boost::unordered_map
+              <Dart_handle, typename CMap2::Dart_handle>* copy_to_origin,
+              const Converters& converters,
+              const DartInfoConverter& dartinfoconverter,
+              const PointConverter& pointconverter,
+              bool copy_perforated_darts=false,
+              size_type mark_perforated=INVALID_MARK)
     {
-      std::tuple<> converters;
-      Default_converter_dart_info<CMap2, Refs> dartinfoconverter;
-      Default_converter_cmap_0attributes_with_point<CMap2, Refs> pointconverter;
-      copy(amap, converters, dartinfoconverter, pointconverter, dart_mapping);
+      generic_copy<CMap2, typename CMap2::Dart_handle, Converters,
+          DartInfoConverter, PointConverter>
+          (amap,  origin_to_copy, copy_to_origin,
+           converters, dartinfoconverter, pointconverter,
+           copy_perforated_darts, mark_perforated);
+    }
+
+    // (1b) copy_from_const(const amap, converters, dartinfoconverter, pointconverter)
+    template<typename CMap2, typename Converters, typename DartInfoConverter,
+             typename PointConverter>
+    void copy_from_const(const CMap2& amap,
+                         boost::unordered_map
+                         <typename CMap2::Dart_const_handle, Dart_handle>* origin_to_copy,
+                         boost::unordered_map
+                         <Dart_handle, typename CMap2::Dart_const_handle>* copy_to_origin,
+                         const Converters& converters,
+                         const DartInfoConverter& dartinfoconverter,
+                         const PointConverter& pointconverter,
+                         bool copy_perforated_darts=false,
+                         size_type mark_perforated=INVALID_MARK)
+   {
+      generic_copy<CMap2, typename CMap2::Dart_const_handle, Converters,
+          DartInfoConverter, PointConverter>
+          (const_cast<CMap2&>(amap), origin_to_copy, copy_to_origin,
+           converters, dartinfoconverter, pointconverter,
+           copy_perforated_darts, mark_perforated);
+    }
+
+    // (2a) copy(amap, converters, dartinfoconverter)
+    template<typename CMap2, typename Converters, typename DartInfoConverter>
+    void copy(CMap2& amap,
+              boost::unordered_map
+              <typename CMap2::Dart_handle, Dart_handle>* origin_to_copy,
+              boost::unordered_map
+              <Dart_handle, typename CMap2::Dart_handle>* copy_to_origin,
+              const Converters& converters,
+              const DartInfoConverter& dartinfoconverter,
+              bool copy_perforated_darts=false,
+              size_type mark_perforated=INVALID_MARK)
+    {
+      Default_converter_cmap_0attributes_with_point<typename CMap2::Refs, Refs>
+        pointconverter;
+      copy(amap, origin_to_copy, copy_to_origin,
+           converters, dartinfoconverter, pointconverter,
+           copy_perforated_darts, mark_perforated);
+    }
+
+    // (2b) copy_from_const(const amap, converters, dartinfoconverter)
+    template <typename CMap2, typename Converters, typename DartInfoConverter>
+    void copy_from_const(const CMap2& amap,
+                         boost::unordered_map
+                         <typename CMap2::Dart_const_handle, Dart_handle>* origin_to_copy,
+                         boost::unordered_map
+                         <Dart_handle, typename CMap2::Dart_const_handle>* copy_to_origin,
+                         const Converters& converters,
+                         const DartInfoConverter& dartinfoconverter,
+                         bool copy_perforated_darts=false,
+                         size_type mark_perforated=INVALID_MARK)
+    {
+      Default_converter_cmap_0attributes_with_point<typename CMap2::Refs, Refs>
+          pointconverter;
+      copy_from_const(amap, origin_to_copy, copy_to_origin,
+                      converters, dartinfoconverter, pointconverter,
+                      copy_perforated_darts, mark_perforated);
+    }
+
+    // (3a) copy(amap, converters)
+    template<typename CMap2, typename Converters>
+    void copy(CMap2& amap,
+              boost::unordered_map
+              <typename CMap2::Dart_handle, Dart_handle>* origin_to_copy,
+              boost::unordered_map
+              <Dart_handle, typename CMap2::Dart_handle>* copy_to_origin,
+              const Converters& converters,
+              bool copy_perforated_darts=false,
+              size_type mark_perforated=INVALID_MARK)
+    {
+      Default_converter_dart_info<typename CMap2::Refs, Refs> dartinfoconverter;
+      copy(amap, origin_to_copy, copy_to_origin, converters, dartinfoconverter,
+           copy_perforated_darts, mark_perforated);
     }
 
+    // (3b) copy_from_const(const amap, converters)
     template <typename CMap2, typename Converters>
-    void copy(const CMap2& amap, const Converters& converters,
-              boost::unordered_map<typename CMap2::Dart_const_handle, Dart_handle>* dart_mapping=nullptr)
+    void copy_from_const(const CMap2& amap,
+                         boost::unordered_map
+                         <typename CMap2::Dart_const_handle, Dart_handle>* origin_to_copy,
+                         boost::unordered_map
+                         <Dart_handle, typename CMap2::Dart_const_handle>* copy_to_origin,
+                         const Converters& converters,
+                         bool copy_perforated_darts=false,
+                         size_type mark_perforated=INVALID_MARK)
+    {
+      Default_converter_dart_info<typename CMap2::Refs, Refs> dartinfoconverter;
+      copy_from_const(amap, origin_to_copy, copy_to_origin, converters, dartinfoconverter,
+                      copy_perforated_darts, mark_perforated);
+    }
+
+    // (4a) copy(amap)
+    template<typename CMap2>
+    void copy(CMap2& amap,
+              boost::unordered_map
+              <typename CMap2::Dart_handle, Dart_handle>* origin_to_copy=nullptr,
+              boost::unordered_map
+              <Dart_handle, typename CMap2::Dart_handle>* copy_to_origin=nullptr,
+              bool copy_perforated_darts=false,
+              size_type mark_perforated=INVALID_MARK)
     {
-      Default_converter_cmap_0attributes_with_point<CMap2, Refs> pointconverter;
-      Default_converter_dart_info<CMap2, Refs> dartinfoconverter;
-      copy(amap, converters, dartinfoconverter, pointconverter, dart_mapping);
+      std::tuple<> converters;
+      copy(amap, origin_to_copy, copy_to_origin, converters,
+           copy_perforated_darts, mark_perforated);
     }
 
-    template <typename CMap2, typename Converters, typename DartInfoConverter>
-    void copy(const CMap2& amap, const Converters& converters,
-              const DartInfoConverter& dartinfoconverter,
-              boost::unordered_map<typename CMap2::Dart_const_handle, Dart_handle>* dart_mapping=nullptr)
+    // (4b) copy_from_const(const amap)
+    template <typename CMap2>
+    void copy_from_const(const CMap2& amap,
+                         boost::unordered_map
+                         <typename CMap2::Dart_const_handle, Dart_handle>* origin_to_copy=nullptr,
+                         boost::unordered_map
+                         <Dart_handle, typename CMap2::Dart_const_handle>* copy_to_origin=nullptr,
+                         bool copy_perforated_darts=false,
+                         size_type mark_perforated=INVALID_MARK)
     {
-      Default_converter_cmap_0attributes_with_point<CMap2, Refs> pointconverter;
-      copy(amap, converters, dartinfoconverter, pointconverter, dart_mapping);
+      std::tuple<> converters;
+      copy_from_const(amap, origin_to_copy, copy_to_origin, converters,
+                      copy_perforated_darts, mark_perforated);
     }
 
     // Copy constructor from a map having exactly the same type.
-    Combinatorial_map_base (const Self & amap)
-    { copy(amap); }
+    Combinatorial_map_base (const Self & amap): Combinatorial_map_base()
+    { copy_from_const(amap); }
 
     // "Copy constructor" from a map having different type.
-    template <typename CMap2>
-    Combinatorial_map_base(const CMap2& amap)
-    { copy(amap); }
+    template <unsigned int d2, typename Refs2, typename Items2, typename Alloc2,
+              typename Storage2>
+    Combinatorial_map_base(const Combinatorial_map_base<d2, Refs2, Items2,
+                           Alloc2, Storage2>& amap): Combinatorial_map_base()
+    { copy_from_const(amap); }
 
     // "Copy constructor" from a map having different type.
-    template <typename CMap2, typename Converters>
-    Combinatorial_map_base(const CMap2& amap, Converters& converters)
-    { copy(amap, converters); }
+    template <unsigned int d2, typename Refs2, typename Items2, typename Alloc2,
+              typename Storage2, typename Converters>
+    Combinatorial_map_base(const Combinatorial_map_base<d2, Refs2, Items2,
+                           Alloc2, Storage2>& amap,
+                           const Converters& converters):
+      Combinatorial_map_base()
+    { copy_from_const(amap, nullptr, nullptr, converters); }
 
     // "Copy constructor" from a map having different type.
-    template <typename CMap2, typename Converters, typename DartInfoConverter>
-    Combinatorial_map_base(const CMap2& amap, Converters& converters,
-                           const DartInfoConverter& dartinfoconverter)
-    { copy(amap, converters, dartinfoconverter); }
+    template <unsigned int d2, typename Refs2, typename Items2, typename Alloc2,
+              typename Storage2, typename Converters, typename DartInfoConverter>
+    Combinatorial_map_base(const Combinatorial_map_base<d2, Refs2, Items2,
+                           Alloc2, Storage2>& amap,
+                           const Converters& converters,
+                           const DartInfoConverter& dartinfoconverter):
+      Combinatorial_map_base()
+    { copy_from_const(amap, nullptr, nullptr, converters, dartinfoconverter); }
 
     // "Copy constructor" from a map having different type.
-    template <typename CMap2, typename Converters, typename DartInfoConverter,
-                           typename PointConverter>
-    Combinatorial_map_base(const CMap2& amap, Converters& converters,
+    template <unsigned int d2, typename Refs2, typename Items2, typename Alloc2,
+              typename Storage2,
+              typename Converters, typename DartInfoConverter,
+              typename PointConverter>
+    Combinatorial_map_base(const Combinatorial_map_base<d2, Refs2, Items2,
+                           Alloc2, Storage2>& amap,
+                           const Converters& converters,
                            const DartInfoConverter& dartinfoconverter,
-                           const PointConverter& pointconverter)
-    { copy(amap, converters, dartinfoconverter, pointconverter); }
+                           const PointConverter& pointconverter):
+      Combinatorial_map_base()
+    { copy_from_const(amap, nullptr, nullptr, converters, dartinfoconverter, pointconverter); }
 
     /** Affectation operation. Copies one map to the other.
      * @param amap a combinatorial map.
      * @return A copy of that combinatorial map.
      */
-    Self & operator= (const Self & amap)
+    Self & operator=(const Self & amap)
     {
       if (this!=&amap)
       {
@@ -385,6 +551,68 @@ namespace CGAL {
       }
     }
 
+    /** Import the given hds which should be a model of an halfedge graph. */
+    template<class HEG>
+    void import_from_halfedge_graph(const HEG& heg,
+                                    boost::unordered_map
+                                    <typename boost::graph_traits<HEG>::halfedge_descriptor,
+                                    Dart_handle>* origin_to_copy=NULL,
+                                    boost::unordered_map
+                                    <Dart_handle,
+                                    typename boost::graph_traits<HEG>::halfedge_descriptor>*
+                                    copy_to_origin=NULL,
+                                    bool copy_perforated_darts=false,
+                                    size_type mark_perforated=INVALID_MARK)
+    {
+      // Create an mapping between darts of the two maps (originals->copies).
+      // (here we cannot use CGAL::Unique_hash_map because it does not provide
+      // iterators...
+      boost::unordered_map
+        <typename boost::graph_traits<HEG>::halfedge_descriptor,
+         Dart_handle> local_dartmap;
+      if (origin_to_copy==NULL) // Used local_dartmap if user does not provides its own unordered_map
+      { origin_to_copy=&local_dartmap; }
+
+      Dart_handle new_dart;
+      for (typename boost::graph_traits<HEG>::halfedge_iterator
+           it=halfedges(heg).begin(), itend=halfedges(heg).end();
+           it!=itend; ++it)
+      {
+        if (copy_perforated_darts || !CGAL::is_border(*it, heg))
+        {
+          new_dart=mdarts.emplace();
+
+          if (mark_perforated!=INVALID_MARK && CGAL::is_border(*it, heg))
+          { mark(new_dart, mark_perforated); }
+
+          (*origin_to_copy)[*it]=new_dart;
+          if (copy_to_origin!=NULL) { (*copy_to_origin)[new_dart]=*it; }
+        }
+      }
+
+      typename boost::unordered_map
+        <typename boost::graph_traits<HEG>::halfedge_descriptor,
+         Dart_handle>::iterator dartmap_iter, dartmap_iter_end=origin_to_copy->end();
+      for (dartmap_iter=origin_to_copy->begin(); dartmap_iter!=dartmap_iter_end;
+           ++dartmap_iter)
+      {
+        basic_link_beta(dartmap_iter->second,
+                        (*origin_to_copy)[CM_ADL_next(dartmap_iter->first, heg)],
+            1);
+
+        if ((copy_perforated_darts ||
+             !CGAL::is_border(CM_ADL_opposite(dartmap_iter->first, heg), heg)) &&
+            (dartmap_iter->first)<CM_ADL_opposite(dartmap_iter->first, heg))
+        {
+          basic_link_beta(dartmap_iter->second,
+                          (*origin_to_copy)
+                          [CM_ADL_opposite(dartmap_iter->first, heg)], 2);
+        }
+      }
+
+      CGAL_assertion (is_valid());
+    }
+
     /** Clear the combinatorial map. Remove all darts and all attributes.
      *  Note that reserved marks are not free.
      */
@@ -498,6 +726,16 @@ namespace CGAL {
     { return mdarts.iterator_to(adart); }
     Dart_const_handle dart_handle(const Dart& adart) const
     { return mdarts.iterator_to(adart); }
+    Dart_handle dart_handle(size_type i)
+    {
+      CGAL_assertion(darts().is_used(i));
+      return mdarts.iterator_to(darts()[i]);
+    }
+    Dart_const_handle dart_handle(size_type i) const
+    {
+      CGAL_assertion(darts().is_used(i));
+      return mdarts.iterator_to(darts()[i]);
+    }
 
     /** Return the highest dimension for which dh is not free.
      * @param dh a dart handle
@@ -673,6 +911,11 @@ namespace CGAL {
     Dart_const_handle next(Dart_const_handle ADart) const
     { return this->template beta<1>(ADart); }
 
+    Dart_handle opposite2(Dart_handle ADart)
+    { return this->template beta<2>(ADart); }
+    Dart_const_handle opposite2(Dart_const_handle ADart) const
+    { return this->template beta<2>(ADart); }
+
     template<unsigned int dim>
     Dart_handle opposite(Dart_handle ADart)
     { return this->template beta<dim>(ADart); }
@@ -950,6 +1193,40 @@ namespace CGAL {
       mnb_times_reserved_marks[amark]=0;
     }
 
+    template <unsigned int i, unsigned int d=dimension>
+    bool belong_to_same_cell(Dart_const_handle adart1,
+                             Dart_const_handle adart2) const
+    { return CGAL::belong_to_same_cell<Self, i, d>(*this, adart1, adart2); }
+
+    template <unsigned int i, unsigned int d=dimension>
+    bool is_whole_cell_unmarked(Dart_const_handle adart, size_type amark) const
+    { return CGAL::is_whole_cell_unmarked<Self, i, d>(*this, adart, amark); }
+
+    template <unsigned int i, unsigned int d=dimension>
+    bool is_whole_cell_marked(Dart_const_handle adart, size_type amark) const
+    { return CGAL::is_whole_cell_marked<Self, i, d>(*this, adart, amark); }
+
+    template <unsigned int i, unsigned int d=dimension>
+    size_type mark_cell(Dart_const_handle adart, size_type amark) const
+    { return CGAL::mark_cell<Self, i, d>(*this, adart, amark); }
+
+    template <unsigned int i, unsigned int d=dimension>
+    size_type unmark_cell(Dart_const_handle adart, size_type amark) const
+    { return CGAL::unmark_cell<Self, i, d>(*this, adart, amark); }
+
+    template <unsigned int i, unsigned int d=dimension>
+    size_type mark_oriented_cell(Dart_const_handle adart, size_type amark,
+                                 size_type amark2=INVALID_MARK) const
+    { return CGAL::mark_oriented_cell<Self, i, d>(*this, adart, amark, amark2); }
+
+    template <unsigned int i, unsigned int d=dimension>
+    size_type unmark_oriented_cell(Dart_const_handle adart, size_type amark,
+                                 size_type amark2=INVALID_MARK) const
+    { return CGAL::unmark_oriented_cell<Self, i, d>(*this, adart, amark, amark2); }
+
+    std::size_t orient(size_type amark) const
+    { negate_mark(amark); return number_of_darts(); }
+
     /** Test if this map is without boundary for a given dimension.
      * @param i the dimension.
      * @return true iff all the darts are not i-free.
@@ -2300,6 +2577,30 @@ namespace CGAL {
         run(*this, adart, amark);
     }
 
+    /// Keep the biggest connected component.
+    /// @return the size (in number of darts) of the biggest cc.
+    std::size_t keep_biggest_connected_component()
+    {
+      std::map<std::size_t, Dart_handle> ccs;
+
+      size_type treated=get_new_mark();
+      for (auto it=darts().begin(), itend=darts().end(); it!=itend; ++it)
+      {
+        if (!is_marked(it, treated))
+        { ccs[mark_cell<dimension+1>(it, treated)]=it; }
+      }
+
+      if (ccs.size()>1)
+      { // Here all darts are marked
+        this->template unmark_cell<dimension+1>(ccs.rbegin()->second, treated); // Unmark the biggest cc
+        erase_marked_darts(treated);
+      }
+
+      free_mark(treated);
+
+      return ccs.rbegin()->first;
+    }
+
     /** Count the marked cells (at least one marked dart).
      * @param amark the mark to consider.
      * @param avector containing the dimensions of the cells to count.
@@ -2433,6 +2734,64 @@ namespace CGAL {
     }
 
   public:
+
+    /// @return the positive turn between the two given darts.
+    //  @pre beta1(d1) and d2 must belong to the same vertex.
+    std::size_t positive_turn(Dart_const_handle d1, Dart_const_handle d2) const
+    {
+      CGAL_assertion((!this->template is_free<1>(d1)));
+      /* CGAL_assertion((belong_to_same_cell<0>(this->template beta<1>(d1),
+                                                d2))); */
+
+      if (d2==beta<2>(d1)) { return 0; }
+
+      Dart_const_handle dd1=d1;
+      std::size_t res=1;
+      while (beta<1>(dd1)!=d2)
+      {
+        if (this->template is_free<2>(beta<1>(dd1)))
+        { return (std::numeric_limits<std::size_t>::max)(); }
+
+        ++res;
+        dd1=beta<1, 2>(dd1);
+
+        CGAL_assertion(!this->template is_free<1>(dd1));
+        CGAL_assertion(beta<1>(dd1)==d2 || dd1!=d1);
+      }
+      return res;
+    }
+
+    /// @return the negative turn between the two given darts.
+    //  @pre beta1(d1) and d2 must belong to the same vertex.
+    std::size_t negative_turn(Dart_const_handle d1, Dart_const_handle d2) const
+    {
+      CGAL_assertion((!this->template is_free<1>(d1)));
+      /* CGAL_assertion((belong_to_same_cell<0>(this->template beta<1>(d1),
+                                                d2))); */
+
+      if (d2==beta<2>(d1)) { return 0; }
+
+      if (this->template is_free<2>(d1) || this->template is_free<2>(d2))
+      { return (std::numeric_limits<std::size_t>::max)(); }
+
+      d1=beta<2>(d1);
+      d2=beta<2>(d2);
+      Dart_const_handle dd1=d1;
+      std::size_t res=1;
+      while (beta<0>(dd1)!=d2)
+      {
+        if (this->template is_free<2>(beta<0>(dd1)))
+        { return (std::numeric_limits<std::size_t>::max)(); }
+
+        ++res;
+        dd1=beta<0, 2>(dd1);
+
+        CGAL_assertion(!this->template is_free<0>(dd1));
+        CGAL_assertion(beta<0>(dd1)==d2 || dd1!=d1);
+      }
+      return res;
+    }
+
     /** Erase marked darts from the map.
      * Marked darts are unlinked before to be removed, thus surviving darts
      * are correctly linked, but the map is not necessarily valid depending
@@ -3513,12 +3872,12 @@ namespace CGAL {
            !is_face_combinatorial_polygon(d4, 3) ) return false;
 
       // TODO do better with marks (?).
-      if ( belong_to_same_cell<Self,2,1>(*this, d1, d2) ||
-           belong_to_same_cell<Self,2,1>(*this, d1, d3) ||
-           belong_to_same_cell<Self,2,1>(*this, d1, d4) ||
-           belong_to_same_cell<Self,2,1>(*this, d2, d3) ||
-           belong_to_same_cell<Self,2,1>(*this, d2, d4) ||
-           belong_to_same_cell<Self,2,1>(*this, d3, d4) ) return false;
+      if ( belong_to_same_cell<2,1>(d1, d2) ||
+           belong_to_same_cell<2,1>(d1, d3) ||
+           belong_to_same_cell<2,1>(d1, d4) ||
+           belong_to_same_cell<2,1>(d2, d3) ||
+           belong_to_same_cell<2,1>(d2, d4) ||
+           belong_to_same_cell<2,1>(d3, d4) ) return false;
 
       if ( beta(d1,1,2)!=beta(d3,0) ||
            beta(d4,0,2)!=beta(d3,1) ||
@@ -3611,21 +3970,21 @@ namespace CGAL {
           !is_face_combinatorial_polygon(d6, 4) ) return false;
 
       // TODO do better with marks.
-      if ( belong_to_same_cell<Self,2,1>(*this, d1, d2) ||
-           belong_to_same_cell<Self,2,1>(*this, d1, d3) ||
-           belong_to_same_cell<Self,2,1>(*this, d1, d4) ||
-           belong_to_same_cell<Self,2,1>(*this, d1, d5) ||
-           belong_to_same_cell<Self,2,1>(*this, d1, d6) ||
-           belong_to_same_cell<Self,2,1>(*this, d2, d3) ||
-           belong_to_same_cell<Self,2,1>(*this, d2, d4) ||
-           belong_to_same_cell<Self,2,1>(*this, d2, d5) ||
-           belong_to_same_cell<Self,2,1>(*this, d2, d6) ||
-           belong_to_same_cell<Self,2,1>(*this, d3, d4) ||
-           belong_to_same_cell<Self,2,1>(*this, d3, d5) ||
-           belong_to_same_cell<Self,2,1>(*this, d3, d6) ||
-           belong_to_same_cell<Self,2,1>(*this, d4, d5) ||
-           belong_to_same_cell<Self,2,1>(*this, d4, d6) ||
-           belong_to_same_cell<Self,2,1>(*this, d5, d6) )
+      if ( belong_to_same_cell<2,1>(d1, d2) ||
+           belong_to_same_cell<2,1>(d1, d3) ||
+           belong_to_same_cell<2,1>(d1, d4) ||
+           belong_to_same_cell<2,1>(d1, d5) ||
+           belong_to_same_cell<2,1>(d1, d6) ||
+           belong_to_same_cell<2,1>(d2, d3) ||
+           belong_to_same_cell<2,1>(d2, d4) ||
+           belong_to_same_cell<2,1>(d2, d5) ||
+           belong_to_same_cell<2,1>(d2, d6) ||
+           belong_to_same_cell<2,1>(d3, d4) ||
+           belong_to_same_cell<2,1>(d3, d5) ||
+           belong_to_same_cell<2,1>(d3, d6) ||
+           belong_to_same_cell<2,1>(d4, d5) ||
+           belong_to_same_cell<2,1>(d4, d6) ||
+           belong_to_same_cell<2,1>(d5, d6) )
         return false;
 
       if ( beta(d1,2)    !=beta(d4,1,1) ||
@@ -3781,7 +4140,7 @@ namespace CGAL {
       return this->template beta<1>(adart);
     }
 
-    /** Insert a vertex in the given 2-cell which is splitted in triangles,
+    /** Insert a vertex in the given 2-cell which is split in triangles,
      *  once for each inital edge of the facet.
      * @param adart a dart of the facet to triangulate.
      * @param update_attributes a boolean to update the enabled attributes
@@ -4213,7 +4572,7 @@ namespace CGAL {
           if ( od==null_handle ) return false;
 
           // of and *it must belong to the same vertex of the same volume
-          if ( !belong_to_same_cell<Self, 0, 2>(*this, od, *it) )
+          if ( !belong_to_same_cell<0, 2>(od, *it) )
             return false;
         }
         prec = *it;
@@ -4223,7 +4582,7 @@ namespace CGAL {
       od = other_extremity(prec);
       if ( od==null_handle ) return false;
 
-      if (!belong_to_same_cell<Self, 0, 2>(*this, od, *afirst))
+      if (!belong_to_same_cell<0, 2>(od, *afirst))
         return false;
 
       return true;
@@ -4408,14 +4767,7 @@ namespace CGAL {
     typename Helper::Merge_functors m_onmerge_functors;
   };
 
-  template < unsigned int d_,
-#if defined(CGAL_CMAP_DART_DEPRECATED) && !defined(CGAL_NO_DEPRECATED_CODE)
-             class Items_=Combinatorial_map_min_items<d_>,
-#else
-             class Items_=Generic_map_min_items,
-#endif
-             class Alloc_=CGAL_ALLOCATOR(int),
-             class Storage_= Combinatorial_map_storage_1<d_, Items_, Alloc_> >
+  template < unsigned int d_, class Items_, class Alloc_, class Storage_ >
   class Combinatorial_map :
     public Combinatorial_map_base<d_,
                                   Combinatorial_map<d_,Items_,Alloc_, Storage_>,
@@ -4437,24 +4789,33 @@ namespace CGAL {
     Combinatorial_map(const Self & amap) : Base(amap)
     {}
 
-    template < class CMap >
-    Combinatorial_map(const CMap & amap) : Base(amap)
+    template <unsigned int d2, typename Refs2, typename Items2, typename Alloc2,
+              typename Storage2>
+    Combinatorial_map(const Combinatorial_map_base<d2, Refs2, Items2, Alloc2, Storage2>&
+                      amap) : Base(amap)
     {}
 
-    template < class CMap, typename Converters >
-    Combinatorial_map(const CMap & amap, const Converters& converters) :
+    template <unsigned int d2, typename Refs2, typename Items2, typename Alloc2,
+              typename Storage2, typename Converters>
+    Combinatorial_map(const Combinatorial_map_base<d2, Refs2, Items2, Alloc2, Storage2>&
+                      amap, const Converters& converters) :
       Base(amap, converters)
     {}
 
-    template < class CMap, typename Converters, typename DartInfoConverter >
-    Combinatorial_map(const CMap & amap, const Converters& converters,
+    template <unsigned int d2, typename Refs2, typename Items2, typename Alloc2,
+              typename Storage2, typename Converters,
+              typename DartInfoConverter>
+    Combinatorial_map(const Combinatorial_map_base<d2, Refs2, Items2, Alloc2, Storage2>&
+                      amap, const Converters& converters,
                       const DartInfoConverter& dartinfoconverter) :
       Base(amap, converters, dartinfoconverter)
     {}
 
-    template < class CMap, typename Converters, typename DartInfoConverter,
-               typename PointConverter >
-    Combinatorial_map(const CMap & amap, const Converters& converters,
+    template <unsigned int d2, typename Refs2, typename Items2, typename Alloc2,
+              typename Storage2, typename Converters,
+              typename DartInfoConverter, typename PointConverter >
+    Combinatorial_map(const Combinatorial_map_base<d2, Refs2, Items2, Alloc2, Storage2>&
+                      amap, const Converters& converters,
                       const DartInfoConverter& dartinfoconverter,
                       const PointConverter& pointconverter) :
       Base(amap, converters, dartinfoconverter, pointconverter)
diff -pruN 5.0.3-1/include/CGAL/Combinatorial_map_insertions.h 5.1-1/include/CGAL/Combinatorial_map_insertions.h
--- 5.0.3-1/include/CGAL/Combinatorial_map_insertions.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Combinatorial_map_insertions.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Combinatorial_map/include/CGAL/Combinatorial_map_insertions.h $
-// $Id: Combinatorial_map_insertions.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Combinatorial_map/include/CGAL/Combinatorial_map_insertions.h $
+// $Id: Combinatorial_map_insertions.h 48fedba 2019-12-11T16:57:34+01:00 Andreas Fabri
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
@@ -39,7 +39,7 @@ insert_cell_0_in_cell_1( CMap& amap, typ
   return amap.insert_cell_0_in_cell_1(adart, ah, update_attributes);
 }
 
-/** Insert a vertex in the given 2-cell which is splitted in triangles,
+/** Insert a vertex in the given 2-cell which is split in triangles,
  * once for each inital edge of the facet.
  * @param amap the used combinatorial map.
  * @param adart a dart of the facet to triangulate.
diff -pruN 5.0.3-1/include/CGAL/Combinatorial_map_iterators_base.h 5.1-1/include/CGAL/Combinatorial_map_iterators_base.h
--- 5.0.3-1/include/CGAL/Combinatorial_map_iterators_base.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Combinatorial_map_iterators_base.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Combinatorial_map/include/CGAL/Combinatorial_map_iterators_base.h $
-// $Id: Combinatorial_map_iterators_base.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Combinatorial_map/include/CGAL/Combinatorial_map_iterators_base.h $
+// $Id: Combinatorial_map_iterators_base.h 9a50c0f 2019-11-26T13:22:03+01:00 Guillaume Damiand
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
@@ -106,7 +106,7 @@ namespace CGAL {
     {
       return ( ((*this==mmap->null_handle) && (aiterator==mmap->null_handle)) ||
                (mfirst_dart == aiterator.mfirst_dart &&
-               static_cast<const Base&>(*this)==
+                static_cast<const Base&>(*this)==
                 static_cast<const Base&>(aiterator)) );
     }
 
diff -pruN 5.0.3-1/include/CGAL/Combinatorial_map_min_items.h 5.1-1/include/CGAL/Combinatorial_map_min_items.h
--- 5.0.3-1/include/CGAL/Combinatorial_map_min_items.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Combinatorial_map_min_items.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Combinatorial_map/include/CGAL/Combinatorial_map_min_items.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Combinatorial_map/include/CGAL/Combinatorial_map_min_items.h $
 // $Id: Combinatorial_map_min_items.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Combinatorial_map_operations.h 5.1-1/include/CGAL/Combinatorial_map_operations.h
--- 5.0.3-1/include/CGAL/Combinatorial_map_operations.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Combinatorial_map_operations.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Combinatorial_map/include/CGAL/Combinatorial_map_operations.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Combinatorial_map/include/CGAL/Combinatorial_map_operations.h $
 // $Id: Combinatorial_map_operations.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Combinatorial_map_save_load.h 5.1-1/include/CGAL/Combinatorial_map_save_load.h
--- 5.0.3-1/include/CGAL/Combinatorial_map_save_load.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Combinatorial_map_save_load.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Combinatorial_map/include/CGAL/Combinatorial_map_save_load.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Combinatorial_map/include/CGAL/Combinatorial_map_save_load.h $
 // $Id: Combinatorial_map_save_load.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Combinatorial_map_storages.h 5.1-1/include/CGAL/Combinatorial_map_storages.h
--- 5.0.3-1/include/CGAL/Combinatorial_map_storages.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Combinatorial_map_storages.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Combinatorial_map/include/CGAL/Combinatorial_map_storages.h $
-// $Id: Combinatorial_map_storages.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Combinatorial_map/include/CGAL/Combinatorial_map_storages.h $
+// $Id: Combinatorial_map_storages.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
@@ -127,6 +127,8 @@ namespace CGAL {
       CGAL_assertion(i <= dimension);
       return dh->mf[i]==null_dart_handle;
     }
+    bool is_perforated(Dart_const_handle /*dh*/) const
+    { return false; }
 
     /// Set simultaneously all the marks of this dart to a given value.
     void set_dart_marks(Dart_const_handle ADart,
diff -pruN 5.0.3-1/include/CGAL/Compact_container.h 5.1-1/include/CGAL/Compact_container.h
--- 5.0.3-1/include/CGAL/Compact_container.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Compact_container.h	2020-09-08 12:15:32.000000000 +0000
@@ -4,8 +4,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Compact_container.h $
-// $Id: Compact_container.h a8fca35 2020-05-27T16:32:49+02:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Compact_container.h $
+// $Id: Compact_container.h 871c972 2020-06-03T16:23:22+02:00 Laurent Rineau
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Sylvain Pion
@@ -25,11 +25,13 @@
 #include <vector>
 #include <cstring>
 #include <functional>
+#include <atomic>
 
 #include <CGAL/memory.h>
 #include <CGAL/iterator.h>
 #include <CGAL/CC_safe_handle.h>
 #include <CGAL/Time_stamper.h>
+#include <CGAL/Has_member.h>
 
 #include <boost/mpl/if.hpp>
 
@@ -82,24 +84,6 @@
 
 namespace CGAL {
 
-#define CGAL_GENERATE_MEMBER_DETECTOR(X)                                             \
-template<typename T> class has_##X {                                          \
-    struct Fallback { int X; };                                               \
-    struct Derived : T, Fallback { };                                         \
-                                                                              \
-    template<typename U, U> struct Check;                                     \
-                                                                              \
-    typedef char ArrayOfOne[1];                                               \
-    typedef char ArrayOfTwo[2];                                               \
-                                                                              \
-    template<typename U> static ArrayOfOne & func(                            \
-                                            Check<int Fallback::*, &U::X> *); \
-    template<typename U> static ArrayOfTwo & func(...);                       \
-  public:                                                                     \
-    typedef has_##X type;                                                     \
-    enum { value = sizeof(func<Derived>(0)) == 2 };                           \
-} // semicolon is after the macro call
-
 #define CGAL_INIT_COMPACT_CONTAINER_BLOCK_SIZE 14
 #define CGAL_INCREMENT_COMPACT_CONTAINER_BLOCK_SIZE 16
 
@@ -165,15 +149,15 @@ public:
   Compact_container_base()
   : p(nullptr) {}
   void *   for_compact_container() const { return p; }
-  void * & for_compact_container()       { return p; }
+  void for_compact_container(void* ptr)  { p = ptr; }
 };
 
 // The traits class describes the way to access the pointer.
 // It can be specialized.
 template < class T >
 struct Compact_container_traits {
-  static void *   pointer(const T &t) { return t.for_compact_container(); }
-  static void * & pointer(T &t)       { return t.for_compact_container(); }
+  static void *   pointer(const T &t)    { return t.for_compact_container(); }
+  static void set_pointer(T &t, void* p) { t.for_compact_container(p); }
 };
 
 namespace internal {
@@ -240,7 +224,8 @@ class Compact_container
 public:
   typedef typename Default::Get< TimeStamper_,
                                  CGAL::Time_stamper_impl<T> >::type
-                                                    Time_stamper_impl;
+                                                    Time_stamper;
+  typedef Time_stamper                              Time_stamper_impl; // backward-compatibility
 
   typedef T                                         value_type;
   typedef Allocator                                 allocator_type;
@@ -267,16 +252,14 @@ public:
 
   explicit Compact_container(const Allocator &a = Allocator())
   : alloc(a)
-  , time_stamper(new Time_stamper_impl())
   {
-    init ();
+    init();
   }
 
   template < class InputIterator >
   Compact_container(InputIterator first, InputIterator last,
                     const Allocator & a = Allocator())
   : alloc(a)
-  , time_stamper(new Time_stamper_impl())
   {
     init();
     std::copy(first, last, CGAL::inserter(*this));
@@ -285,14 +268,19 @@ public:
   // The copy constructor and assignment operator preserve the iterator order
   Compact_container(const Compact_container &c)
   : alloc(c.get_allocator())
-  , time_stamper(new Time_stamper_impl())
   {
     init();
     block_size = c.block_size;
-    *time_stamper = *c.time_stamper;
+    time_stamp = c.time_stamp.load();
     std::copy(c.begin(), c.end(), CGAL::inserter(*this));
   }
 
+  Compact_container(Compact_container&& c) noexcept
+  : alloc(c.get_allocator())
+  {
+    c.swap(*this);
+  }
+
   Compact_container & operator=(const Compact_container &c)
   {
     if (&c != this) {
@@ -302,10 +290,16 @@ public:
     return *this;
   }
 
+  Compact_container & operator=(Compact_container&& c) noexcept
+  {
+    Self tmp(std::move(c));
+    tmp.swap(*this);
+    return *this;
+  }
+
   ~Compact_container()
   {
     clear();
-    delete time_stamper;
   }
 
   bool is_used(const_iterator ptr) const
@@ -345,17 +339,8 @@ public:
     return all_items[block_number].first[index_in_block];
   }
 
-  void swap(Self &c)
-  {
-    std::swap(alloc, c.alloc);
-    std::swap(capacity_, c.capacity_);
-    std::swap(size_, c.size_);
-    std::swap(block_size, c.block_size);
-    std::swap(first_item, c.first_item);
-    std::swap(last_item, c.last_item);
-    std::swap(free_list, c.free_list);
-    all_items.swap(c.all_items);
-    std::swap(time_stamper, c.time_stamper);
+  friend void swap(Compact_container& a, Compact_container b) {
+    a.swap(b);
   }
 
   iterator begin() { return iterator(first_item, 0, 0); }
@@ -400,7 +385,7 @@ public:
     new (ret) value_type(args...);
     CGAL_assertion(type(ret) == USED);
     ++size_;
-    time_stamper->set_time_stamp(ret);
+    Time_stamper::set_time_stamp(ret, time_stamp);
     return iterator(ret, 0);
   }
 
@@ -414,7 +399,7 @@ public:
     std::allocator_traits<allocator_type>::construct(alloc, ret, t);
     CGAL_assertion(type(ret) == USED);
     ++size_;
-    time_stamper->set_time_stamp(ret);
+    Time_stamper::set_time_stamp(ret, time_stamp);
     return iterator(ret, 0);
   }
 
@@ -660,8 +645,8 @@ private:
     // This out of range compare is always true and causes lots of
     // unnecessary warnings.
     // CGAL_precondition(0 <= t && t < 4);
-    Traits::pointer(*ptr) = reinterpret_cast<void *>
-      (reinterpret_cast<std::ptrdiff_t>(clean_pointer((char *) p)) + (int) t);
+    Traits::set_pointer(*ptr, reinterpret_cast<void *>
+      (reinterpret_cast<std::ptrdiff_t>(clean_pointer((char *) p)) + (int) t));
   }
 
 public:
@@ -669,7 +654,21 @@ public:
   static bool is_begin_or_end(const_pointer ptr)
   { return type(ptr)==START_END; }
 
+  void swap(Self &c)
+  {
+    std::swap(alloc, c.alloc);
+    std::swap(capacity_, c.capacity_);
+    std::swap(size_, c.size_);
+    std::swap(block_size, c.block_size);
+    std::swap(first_item, c.first_item);
+    std::swap(last_item, c.last_item);
+    std::swap(free_list, c.free_list);
+    all_items.swap(c.all_items);
 
+    // non-atomic swap of time_stamp:
+    c.time_stamp = time_stamp.exchange(c.time_stamp.load());
+  }
+private:
   // We store a vector of pointers to all allocated blocks and their sizes.
   // Knowing all pointers, we don't have to walk to the end of a block to reach
   // the pointer to the next block.
@@ -677,7 +676,9 @@ public:
   // by walking through the block till its end.
   // This opens up the possibility for the compiler to optimize the clear()
   // function considerably when has_trivial_destructor<T>.
-  typedef std::vector<std::pair<pointer, size_type> >  All_items;
+  using All_items = std::vector<std::pair<pointer, size_type> >;
+
+  using time_stamp_t = std::atomic<std::size_t>;
 
   void init()
   {
@@ -688,21 +689,18 @@ public:
     first_item = nullptr;
     last_item  = nullptr;
     all_items  = All_items();
-    time_stamper->reset();
+    time_stamp = 0;
   }
 
   allocator_type   alloc;
-  size_type        capacity_;
-  size_type        size_;
-  size_type        block_size;
-  pointer          free_list;
-  pointer          first_item;
-  pointer          last_item;
-  All_items        all_items;
-
-  // This is a pointer, so that the definition of Compact_container does
-  // not require a complete type `T`.
-  Time_stamper_impl* time_stamper;
+  size_type        capacity_   = 0;
+  size_type        size_       = 0;
+  size_type        block_size  = Increment_policy::first_block_size;
+  pointer          free_list   = nullptr;
+  pointer          first_item  = nullptr;
+  pointer          last_item   = nullptr;
+  All_items        all_items   = {};
+  time_stamp_t     time_stamp  = {};
 };
 
 template < class T, class Allocator, class Increment_policy, class TimeStamper >
@@ -776,7 +774,7 @@ void Compact_container<T, Allocator, Inc
   for (size_type i = block_size; i >= 1; --i)
   {
     EraseCounterStrategy::set_erase_counter(*(new_block + i), 0);
-    time_stamper->initialize_time_stamp(new_block + i);
+    Time_stamper::initialize_time_stamp(new_block + i);
     put_on_free_list(new_block + i);
   }
   // We insert this new block at the end.
@@ -856,7 +854,6 @@ namespace internal {
   template < class DSC, bool Const >
   class CC_iterator
   {
-    typedef typename DSC::iterator                    iterator;
     typedef CC_iterator<DSC, Const>                   Self;
   public:
     typedef DSC                                       CC;
@@ -875,49 +872,51 @@ namespace internal {
       : ts(0)
 #endif
     {
-      m_ptr.p = nullptr;
+      m_ptr = nullptr;
     }
 
-    // Either a harmless copy-ctor,
-    // or a conversion from iterator to const_iterator.
-    CC_iterator (const iterator &it)
+    // Converting constructor from mutable to constant iterator
+    template <bool OtherConst>
+    CC_iterator(const CC_iterator<
+                typename std::enable_if<(!OtherConst && Const), DSC>::type,
+                OtherConst> &const_it)
 #ifdef CGAL_COMPACT_CONTAINER_DEBUG_TIME_STAMP
-      : ts(Time_stamper_impl::time_stamp(it.operator->()))
+        : ts(Time_stamper::time_stamp(const_it.operator->()))
 #endif
     {
-      m_ptr.p = it.operator->();
+      m_ptr = const_it.operator->();
     }
 
-    // Same for assignment operator (otherwise MipsPro warns)
-    CC_iterator & operator= (const iterator &it)
+    // Assignment operator from mutable to constant iterator
+    template <bool OtherConst>
+    CC_iterator & operator= (const CC_iterator<
+                typename std::enable_if<(!OtherConst && Const), DSC>::type,
+                OtherConst> &const_it)
     {
-      m_ptr.p = it.operator->();
+      m_ptr = const_it.operator->();
 #ifdef CGAL_COMPACT_CONTAINER_DEBUG_TIME_STAMP
-      ts = Time_stamper_impl::time_stamp(it.operator->());
+      ts = Time_stamper::time_stamp(const_it.operator->());
 #endif
       return *this;
     }
 
     // Construction from nullptr
-    CC_iterator (std::nullptr_t CGAL_assertion_code(n))
+    CC_iterator (std::nullptr_t /*CGAL_assertion_code(n)*/)
 #ifdef CGAL_COMPACT_CONTAINER_DEBUG_TIME_STAMP
       : ts(0)
 #endif
     {
-      CGAL_assertion (n == nullptr);
-      m_ptr.p = nullptr;
+      //CGAL_assertion (n == nullptr);
+      m_ptr = nullptr;
     }
 
   private:
 
-    typedef typename DSC::Time_stamper_impl           Time_stamper_impl;
+    typedef typename DSC::Time_stamper           Time_stamper;
 #ifdef CGAL_COMPACT_CONTAINER_DEBUG_TIME_STAMP
     std::size_t ts;
 #endif
-    union {
-      pointer      p;
-      void        *vp;
-    } m_ptr;
+    pointer m_ptr;
 
     // Only Compact_container and Concurrent_compact_container should
     // access these constructors.
@@ -933,16 +932,16 @@ namespace internal {
       : ts(0)
 #endif
     {
-      m_ptr.p = ptr;
-      if (m_ptr.p == nullptr) // empty container.
+      m_ptr = ptr;
+      if (m_ptr == nullptr) // empty container.
         return;
 
-      ++(m_ptr.p); // if not empty, p = start
-      if (DSC::type(m_ptr.p) == DSC::FREE)
+      ++(m_ptr); // if not empty, p = start
+      if (DSC::type(m_ptr) == DSC::FREE)
         increment();
 #ifdef CGAL_COMPACT_CONTAINER_DEBUG_TIME_STAMP
       else
-        ts = Time_stamper_impl::time_stamp(m_ptr.p);
+        ts = Time_stamper::time_stamp(m_ptr);
 #endif // CGAL_COMPACT_CONTAINER_DEBUG_TIME_STAMP
     }
 
@@ -952,10 +951,10 @@ namespace internal {
       : ts(0)
 #endif
     {
-      m_ptr.p = ptr;
+      m_ptr = ptr;
 #ifdef CGAL_COMPACT_CONTAINER_DEBUG_TIME_STAMP
       if(ptr != nullptr){
-        ts = Time_stamper_impl::time_stamp(m_ptr.p);
+        ts = Time_stamper::time_stamp(m_ptr);
       }
 #endif // end CGAL_COMPACT_CONTAINER_DEBUG_TIME_STAMP
     }
@@ -964,49 +963,49 @@ namespace internal {
     void increment()
     {
       // It's either pointing to end(), or valid.
-      CGAL_assertion_msg(m_ptr.p != nullptr,
+      CGAL_assertion_msg(m_ptr != nullptr,
          "Incrementing a singular iterator or an empty container iterator ?");
-      CGAL_assertion_msg(DSC::type(m_ptr.p) != DSC::START_END,
+      CGAL_assertion_msg(DSC::type(m_ptr) != DSC::START_END,
          "Incrementing end() ?");
 
       // If it's not end(), then it's valid, we can do ++.
       do {
-        ++(m_ptr.p);
-        if (DSC::type(m_ptr.p) == DSC::USED ||
-            DSC::type(m_ptr.p) == DSC::START_END)
+        ++(m_ptr);
+        if (DSC::type(m_ptr) == DSC::USED ||
+            DSC::type(m_ptr) == DSC::START_END)
         {
 #ifdef CGAL_COMPACT_CONTAINER_DEBUG_TIME_STAMP
-          ts = Time_stamper_impl::time_stamp(m_ptr.p);
+          ts = Time_stamper::time_stamp(m_ptr);
 #endif
           return;
         }
-        if (DSC::type(m_ptr.p) == DSC::BLOCK_BOUNDARY)
-          m_ptr.p = DSC::clean_pointee(m_ptr.p);
+        if (DSC::type(m_ptr) == DSC::BLOCK_BOUNDARY)
+          m_ptr = DSC::clean_pointee(m_ptr);
       } while (true);
     }
 
     void decrement()
     {
       // It's either pointing to end(), or valid.
-      CGAL_assertion_msg(m_ptr.p != nullptr,
+      CGAL_assertion_msg(m_ptr != nullptr,
          "Decrementing a singular iterator or an empty container iterator ?");
-      CGAL_assertion_msg(DSC::type(m_ptr.p - 1) != DSC::START_END,
+      CGAL_assertion_msg(DSC::type(m_ptr - 1) != DSC::START_END,
          "Decrementing begin() ?");
 
       // If it's not begin(), then it's valid, we can do --.
       do {
-        --m_ptr.p;
-        if (DSC::type(m_ptr.p) == DSC::USED ||
-            DSC::type(m_ptr.p) == DSC::START_END)
+        --m_ptr;
+        if (DSC::type(m_ptr) == DSC::USED ||
+            DSC::type(m_ptr) == DSC::START_END)
         {
 #ifdef CGAL_COMPACT_CONTAINER_DEBUG_TIME_STAMP
-          ts = Time_stamper_impl::time_stamp(m_ptr.p);
+          ts = Time_stamper::time_stamp(m_ptr);
 #endif
           return;
         }
 
-        if (DSC::type(m_ptr.p) == DSC::BLOCK_BOUNDARY)
-          m_ptr.p = DSC::clean_pointee(m_ptr.p);
+        if (DSC::type(m_ptr) == DSC::BLOCK_BOUNDARY)
+          m_ptr = DSC::clean_pointee(m_ptr);
       } while (true);
     }
 
@@ -1014,9 +1013,9 @@ namespace internal {
 
     Self & operator++()
     {
-      CGAL_assertion_msg(m_ptr.p != nullptr,
+      CGAL_assertion_msg(m_ptr != nullptr,
          "Incrementing a singular iterator or an empty container iterator ?");
-      /* CGAL_assertion_msg(DSC::type(m_ptr.p) == DSC::USED,
+      /* CGAL_assertion_msg(DSC::type(m_ptr) == DSC::USED,
          "Incrementing an invalid iterator."); */
       increment();
       return *this;
@@ -1024,10 +1023,10 @@ namespace internal {
 
     Self & operator--()
     {
-      CGAL_assertion_msg(m_ptr.p != nullptr,
+      CGAL_assertion_msg(m_ptr != nullptr,
          "Decrementing a singular iterator or an empty container iterator ?");
-      /*CGAL_assertion_msg(DSC::type(m_ptr.p) == DSC::USED
-                      || DSC::type(m_ptr.p) == DSC::START_END,
+      /*CGAL_assertion_msg(DSC::type(m_ptr) == DSC::USED
+                      || DSC::type(m_ptr) == DSC::START_END,
                       "Decrementing an invalid iterator.");*/
       decrement();
       return *this;
@@ -1039,13 +1038,13 @@ namespace internal {
 #ifdef CGAL_COMPACT_CONTAINER_DEBUG_TIME_STAMP
     bool is_time_stamp_valid() const
     {
-      return (ts == 0) || (ts == Time_stamper_impl::time_stamp(m_ptr.p));
+      return (ts == 0) || (ts == Time_stamper::time_stamp(m_ptr));
     }
 #endif // CGAL_COMPACT_CONTAINER_DEBUG_TIME_STAMP
 
-    reference operator*() const { return *(m_ptr.p); }
+    reference operator*() const { return *(m_ptr); }
 
-    pointer   operator->() const { return (m_ptr.p); }
+    pointer   operator->() const { return (m_ptr); }
 
     // For std::less...
     bool operator<(const CC_iterator& other) const
@@ -1053,7 +1052,7 @@ namespace internal {
 #ifdef CGAL_COMPACT_CONTAINER_DEBUG_TIME_STAMP
       assert( is_time_stamp_valid() );
 #endif
-      return Time_stamper_impl::less(m_ptr.p, other.m_ptr.p);
+      return Time_stamper::less(m_ptr, other.m_ptr);
     }
 
     bool operator>(const CC_iterator& other) const
@@ -1061,7 +1060,7 @@ namespace internal {
 #ifdef CGAL_COMPACT_CONTAINER_DEBUG_TIME_STAMP
       assert( is_time_stamp_valid() );
 #endif
-      return Time_stamper_impl::less(other.m_ptr.p, m_ptr.p);
+      return Time_stamper::less(other.m_ptr, m_ptr);
     }
 
     bool operator<=(const CC_iterator& other) const
@@ -1069,7 +1068,7 @@ namespace internal {
 #ifdef CGAL_COMPACT_CONTAINER_DEBUG_TIME_STAMP
       assert( is_time_stamp_valid() );
 #endif
-      return Time_stamper_impl::less(m_ptr.p, other.m_ptr.p)
+      return Time_stamper::less(m_ptr, other.m_ptr)
           || (*this == other);
     }
 
@@ -1078,13 +1077,13 @@ namespace internal {
 #ifdef CGAL_COMPACT_CONTAINER_DEBUG_TIME_STAMP
       assert( is_time_stamp_valid() );
 #endif
-      return Time_stamper_impl::less(other.m_ptr.p, m_ptr.p)
+      return Time_stamper::less(other.m_ptr, m_ptr)
           || (*this == other);
     }
 
     // Can itself be used for bit-squatting.
-    void *   for_compact_container() const { return (m_ptr.vp); }
-    void * & for_compact_container()       { return (m_ptr.vp); }
+    void * for_compact_container() const { return m_ptr; }
+    void for_compact_container(void* p) { m_ptr = static_cast<pointer>(p); }
   };
 
   template < class DSC, bool Const1, bool Const2 >
@@ -1107,18 +1106,18 @@ namespace internal {
   template < class DSC, bool Const >
   inline
   bool operator==(const CC_iterator<DSC, Const> &rhs,
-                  std::nullptr_t CGAL_assertion_code(n))
+                  std::nullptr_t /*CGAL_assertion_code(n)*/)
   {
-    CGAL_assertion( n == nullptr);
+    //CGAL_assertion( n == nullptr);
     return rhs.operator->() == nullptr;
   }
 
   template < class DSC, bool Const >
   inline
   bool operator!=(const CC_iterator<DSC, Const> &rhs,
-                  std::nullptr_t CGAL_assertion_code(n))
+                  std::nullptr_t /*CGAL_assertion_code(n)*/)
   {
-    CGAL_assertion( n == nullptr);
+    //CGAL_assertion( n == nullptr);
     return rhs.operator->() != nullptr;
   }
 
diff -pruN 5.0.3-1/include/CGAL/Compact_mesh_cell_base_3.h 5.1-1/include/CGAL/Compact_mesh_cell_base_3.h
--- 5.0.3-1/include/CGAL/Compact_mesh_cell_base_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Compact_mesh_cell_base_3.h	2020-09-08 12:15:08.000000000 +0000
@@ -4,8 +4,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Compact_mesh_cell_base_3.h $
-// $Id: Compact_mesh_cell_base_3.h 113c3d1 2020-06-15T15:56:28+02:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Compact_mesh_cell_base_3.h $
+// $Id: Compact_mesh_cell_base_3.h bd7d3c4 2020-06-23T18:49:58+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Laurent Rineau, Stephane Tayeb, Andreas Fabri
@@ -491,7 +491,7 @@ public:
 
   // For use by Compact_container.
   void * for_compact_container() const { return N[0].for_compact_container(); }
-  void * & for_compact_container()     { return N[0].for_compact_container(); }
+  void for_compact_container(void *p) { N[0].for_compact_container(p); }
 
   // TDS internal data access functions.
         TDS_data& tds_data()       { return _tds_data; }
diff -pruN 5.0.3-1/include/CGAL/Compare_handles_with_or_without_timestamps.h 5.1-1/include/CGAL/Compare_handles_with_or_without_timestamps.h
--- 5.0.3-1/include/CGAL/Compare_handles_with_or_without_timestamps.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Compare_handles_with_or_without_timestamps.h	2020-09-08 12:15:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Compare_handles_with_or_without_timestamps.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Compare_handles_with_or_without_timestamps.h $
 // $Id: Compare_handles_with_or_without_timestamps.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/compare_vertices.h 5.1-1/include/CGAL/compare_vertices.h
--- 5.0.3-1/include/CGAL/compare_vertices.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/compare_vertices.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_2/include/CGAL/compare_vertices.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_2/include/CGAL/compare_vertices.h $
 // $Id: compare_vertices.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Complex_2_in_triangulation_3.h 5.1-1/include/CGAL/Complex_2_in_triangulation_3.h
--- 5.0.3-1/include/CGAL/Complex_2_in_triangulation_3.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Complex_2_in_triangulation_3.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Complex_2_in_triangulation_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Complex_2_in_triangulation_3.h $
 // $Id: Complex_2_in_triangulation_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Complex_2_in_triangulation_cell_base_3.h 5.1-1/include/CGAL/Complex_2_in_triangulation_cell_base_3.h
--- 5.0.3-1/include/CGAL/Complex_2_in_triangulation_cell_base_3.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Complex_2_in_triangulation_cell_base_3.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Complex_2_in_triangulation_cell_base_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Complex_2_in_triangulation_cell_base_3.h $
 // $Id: Complex_2_in_triangulation_cell_base_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Complex_2_in_triangulation_vertex_base_3.h 5.1-1/include/CGAL/Complex_2_in_triangulation_vertex_base_3.h
--- 5.0.3-1/include/CGAL/Complex_2_in_triangulation_vertex_base_3.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Complex_2_in_triangulation_vertex_base_3.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Complex_2_in_triangulation_vertex_base_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Complex_2_in_triangulation_vertex_base_3.h $
 // $Id: Complex_2_in_triangulation_vertex_base_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Complexity_tags.h 5.1-1/include/CGAL/Complexity_tags.h
--- 5.0.3-1/include/CGAL/Complexity_tags.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Complexity_tags.h	2020-09-08 12:15:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Complexity_tags.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Complexity_tags.h $
 // $Id: Complexity_tags.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Compute_anchor_3.h 5.1-1/include/CGAL/Compute_anchor_3.h
--- 5.0.3-1/include/CGAL/Compute_anchor_3.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Compute_anchor_3.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Skin_surface_3/include/CGAL/Compute_anchor_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Skin_surface_3/include/CGAL/Compute_anchor_3.h $
 // $Id: Compute_anchor_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/compute_average_spacing.h 5.1-1/include/CGAL/compute_average_spacing.h
--- 5.0.3-1/include/CGAL/compute_average_spacing.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/compute_average_spacing.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_processing_3/include/CGAL/compute_average_spacing.h $
-// $Id: compute_average_spacing.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/compute_average_spacing.h $
+// $Id: compute_average_spacing.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s) : Pierre Alliez and Laurent Saboret
@@ -18,7 +18,9 @@
 
 #include <CGAL/Search_traits_3.h>
 #include <CGAL/squared_distance_3.h>
-#include <CGAL/Orthogonal_k_neighbor_search.h>
+#include <CGAL/Point_set_processing_3/internal/Neighbor_query.h>
+#include <CGAL/Point_set_processing_3/internal/Callback_wrapper.h>
+#include <CGAL/for_each.h>
 #include <CGAL/property_map.h>
 #include <CGAL/point_set_processing_assertions.h>
 #include <CGAL/assertions.h>
@@ -27,15 +29,12 @@
 #include <CGAL/boost/graph/Named_function_parameters.h>
 #include <CGAL/boost/graph/named_params_helper.h>
 
+#include <boost/iterator/zip_iterator.hpp>
+
 #include <iterator>
 #include <list>
 
-#ifdef CGAL_LINKED_WITH_TBB
-#include <CGAL/Point_set_processing_3/internal/Parallel_callback.h>
-#include <tbb/parallel_for.h>
-#include <tbb/blocked_range.h>
-#include <tbb/scalable_allocator.h>
-#endif // CGAL_LINKED_WITH_TBB
+
 
 #ifdef DOXYGEN_RUNNING
 #define CGAL_BGL_NP_TEMPLATE_PARAMETERS NamedParameters
@@ -60,81 +59,37 @@ namespace internal {
 /// @tparam Tree KD-tree.
 ///
 /// @return average spacing (scalar).
-template < typename Kernel,
-           typename Tree >
-typename Kernel::FT
-compute_average_spacing(const typename Kernel::Point_3& query, ///< 3D point whose spacing we want to compute
-                        const Tree& tree,                      ///< KD-tree
+template <typename NeighborQuery>
+typename NeighborQuery::Kernel::FT
+compute_average_spacing(const typename NeighborQuery::Kernel::Point_3& query, ///< 3D point whose spacing we want to compute
+                        const NeighborQuery& neighbor_query,                      ///< KD-tree
                         unsigned int k)                        ///< number of neighbors
 {
   // basic geometric types
+  typedef typename NeighborQuery::Kernel Kernel;
   typedef typename Kernel::FT FT;
   typedef typename Kernel::Point_3 Point;
 
-  // types for K nearest neighbors search
-  typedef Search_traits_3<Kernel> Tree_traits;
-  typedef Orthogonal_k_neighbor_search<Tree_traits> Neighbor_search;
-  typedef typename Neighbor_search::iterator Search_iterator;
 
   // performs k + 1 queries (if unique the query point is
   // output first). search may be aborted when k is greater
   // than number of input points
-  Neighbor_search search(tree,query,k+1);
-  Search_iterator search_iterator = search.begin();
   FT sum_distances = (FT)0.0;
-  unsigned int i;
-  for(i=0;i<(k+1);i++)
-  {
-    if(search_iterator == search.end())
-      break; // premature ending
-
-    Point p = search_iterator->first;
-    sum_distances += std::sqrt(CGAL::squared_distance(query,p));
-    search_iterator++;
-  }
+  unsigned int i = 0;
+  neighbor_query.get_points
+    (query, k, 0,
+     boost::make_function_output_iterator
+     ([&](const Point& p)
+      {
+        sum_distances += std::sqrt(CGAL::squared_distance (query,p));
+        ++ i;
+      }));
 
   // output average spacing
   return sum_distances / (FT)i;
 }
 
 
-#ifdef CGAL_LINKED_WITH_TBB
-  template <typename Kernel, typename Tree>
-  class Compute_average_spacings {
-    typedef typename Kernel::Point_3 Point;
-    typedef typename Kernel::FT FT;
-    const Tree& tree;
-    const unsigned int k;
-    const std::vector<Point>& input;
-    std::vector<FT>& output;
-    cpp11::atomic<std::size_t>& advancement;
-    cpp11::atomic<bool>& interrupted;
-
-  public:
-    Compute_average_spacings(Tree& tree, unsigned int k, std::vector<Point>& points,
-                             std::vector<FT>& output,
-                             cpp11::atomic<std::size_t>& advancement,
-                             cpp11::atomic<bool>& interrupted)
-      : tree(tree), k (k), input (points), output (output)
-      , advancement (advancement)
-      , interrupted (interrupted)
-    { }
-
-    void operator()(const tbb::blocked_range<std::size_t>& r) const
-    {
-      for( std::size_t i = r.begin(); i != r.end(); ++i)
-      {
-        if (interrupted)
-          break;
-
-        output[i] = CGAL::internal::compute_average_spacing<Kernel,Tree>(input[i], tree, k);
-        ++ advancement;
-      }
-    }
-
-  };
-#endif // CGAL_LINKED_WITH_TBB
-
 } /* namespace internal */
 /// \endcond
 
@@ -150,31 +105,48 @@ compute_average_spacing(const typename K
 
    \pre `k >= 2.`
 
-   \tparam ConcurrencyTag enables sequential versus parallel algorithm.
-   Possible values are `Sequential_tag`
-   and `Parallel_tag`.
+   \tparam ConcurrencyTag enables sequential versus parallel algorithm. Possible values are `Sequential_tag`,
+                          `Parallel_tag`, and `Parallel_if_available_tag`.
    \tparam PointRange is a model of `ConstRange`. The value type of
    its iterator is the key type of the named parameter `point_map`.
 
    \param points input point range.
    \param k number of neighbors.
-   \param np optional sequence of \ref psp_namedparameters "Named Parameters" among the ones listed below.
+   \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
    \cgalNamedParamsBegin
-     \cgalParamBegin{point_map} a model of `ReadablePropertyMap` with value type `geom_traits::Point_3`.
-     If this parameter is omitted, `CGAL::Identity_property_map<geom_traits::Point_3>` is used.\cgalParamEnd
-     \cgalParamBegin{callback} an instance of
-      `std::function<bool(double)>`. It is called regularly when the
-      algorithm is running: the current advancement (between 0. and
-      1.) is passed as parameter. If it returns `true`, then the
-      algorithm continues its execution normally; if it returns
-      `false`, the algorithm is stopped and the average spacing value
-      estimated on the processed subset is returned.\cgalParamEnd
-     \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+     \cgalParamNBegin{point_map}
+       \cgalParamDescription{a property map associating points to the elements of the point set `points`}
+       \cgalParamType{a model of `ReadablePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange` and whose value type is `geom_traits::Point_3`}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{callback}
+       \cgalParamDescription{a mechanism to get feedback on the advancement of the algorithm
+                             while it's running and to interrupt it if needed}
+       \cgalParamType{an instance of `std::function<bool(double)>`.}
+       \cgalParamDefault{unused}
+       \cgalParamExtra{It is called regularly when the
+                       algorithm is running: the current advancement (between 0. and
+                       1.) is passed as parameter. If it returns `true`, then the
+                       algorithm continues its execution normally; if it returns
+                       `false`, the algorithm is stopped, the average spacing value estimated
+                       on the processed subset is returned.}
+       \cgalParamExtra{The callback will be copied and therefore needs to be lightweight.}
+       \cgalParamExtra{When `CGAL::Parallel_tag` is used, the `callback` mechanism is called asynchronously
+                       on a separate thread and shouldn't access or modify the variables that are parameters of the algorithm.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{geom_traits}
+       \cgalParamDescription{an instance of a geometric traits class}
+       \cgalParamType{a model of `Kernel`}
+       \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+     \cgalParamNEnd
    \cgalNamedParamsEnd
 
    \return average spacing (scalar). The return type `FT` is a number type. It is
-   either deduced from the `geom_traits` \ref psp_namedparameters "Named Parameters" if provided,
+   either deduced from the `geom_traits` \ref bgl_namedparameters "Named Parameters" if provided,
    or the geometric traits class deduced from the point property map
    of `points`.
 */
@@ -196,20 +168,17 @@ compute_average_spacing(
   using parameters::get_parameter;
 
   // basic geometric types
-  typedef typename Point_set_processing_3::GetPointMap<PointRange, CGAL_BGL_NP_CLASS>::const_type PointMap;
+  typedef typename PointRange::const_iterator iterator;
+  typedef typename CGAL::GetPointMap<PointRange, CGAL_BGL_NP_CLASS>::const_type PointMap;
   typedef typename Point_set_processing_3::GetK<PointRange, CGAL_BGL_NP_CLASS>::Kernel Kernel;
 
-  typedef typename Kernel::Point_3 Point;
-
   PointMap point_map = choose_parameter(get_parameter(np, internal_np::point_map), PointMap());
   const std::function<bool(double)>& callback = choose_parameter(get_parameter(np, internal_np::callback),
                                                                  std::function<bool(double)>());
 
   // types for K nearest neighbors search structure
   typedef typename Kernel::FT FT;
-  typedef Search_traits_3<Kernel> Tree_traits;
-  typedef Orthogonal_k_neighbor_search<Tree_traits> Neighbor_search;
-  typedef typename Neighbor_search::Tree Tree;
+  typedef Point_set_processing_3::internal::Neighbor_query<Kernel, const PointRange&, PointMap> Neighbor_query;
 
   // precondition: at least one element in the container.
   // to fix: should have at least three distinct points
@@ -220,60 +189,46 @@ compute_average_spacing(
   CGAL_point_set_processing_precondition(k >= 2);
 
   // Instanciate a KD-tree search.
-  // Note: We have to convert each input iterator to Point_3.
-  std::vector<Point> kd_tree_points;
-  for(typename PointRange::const_iterator it = points.begin(); it != points.end(); it++)
-    kd_tree_points.push_back(get(point_map, *it));
-  Tree tree(kd_tree_points.begin(), kd_tree_points.end());
+  Neighbor_query neighbor_query (points, point_map);
 
   // iterate over input points, compute and output normal
   // vectors (already normalized)
   FT sum_spacings = (FT)0.0;
   std::size_t nb = 0;
+  std::size_t nb_points = std::distance(points.begin(), points.end());
 
-#ifndef CGAL_LINKED_WITH_TBB
-  CGAL_static_assertion_msg (!(boost::is_convertible<ConcurrencyTag, Parallel_tag>::value),
-                             "Parallel_tag is enabled but TBB is unavailable.");
-#else
-   if (boost::is_convertible<ConcurrencyTag,Parallel_tag>::value)
-   {
-     Point_set_processing_3::internal::Parallel_callback
-       parallel_callback (callback, kd_tree_points.size());
-
-     std::vector<FT> spacings (kd_tree_points.size (), -1);
-     CGAL::internal::Compute_average_spacings<Kernel, Tree>
-       f (tree, k, kd_tree_points, spacings,
-          parallel_callback.advancement(),
-          parallel_callback.interrupted());
-     tbb::parallel_for(tbb::blocked_range<size_t>(0, kd_tree_points.size ()), f);
-
-     for (unsigned int i = 0; i < spacings.size (); ++ i)
-       if (spacings[i] >= 0.)
-       {
-         sum_spacings += spacings[i];
-         ++ nb;
-       }
-
-     parallel_callback.join();
-   }
-   else
-#endif
+  Point_set_processing_3::internal::Callback_wrapper<ConcurrencyTag>
+    callback_wrapper (callback, nb_points);
+
+  std::vector<FT> spacings (nb_points, -1);
+
+  typedef boost::zip_iterator<boost::tuple<iterator, typename std::vector<FT>::iterator> > Zip_iterator;
+
+  CGAL::for_each<ConcurrencyTag>
+    (CGAL::make_range (boost::make_zip_iterator (boost::make_tuple (points.begin(), spacings.begin())),
+                       boost::make_zip_iterator (boost::make_tuple (points.end(), spacings.end()))),
+     [&](const typename Zip_iterator::reference& t)
      {
-       for(typename PointRange::const_iterator it = points.begin(); it != points.end(); it++, nb++)
-       {
-         sum_spacings += internal::compute_average_spacing<Kernel,Tree>(
-           get(point_map,*it),
-           tree,k);
-         if (callback && !callback ((nb+1) / double(kd_tree_points.size())))
-         {
-           ++ nb;
-           break;
-         }
-       }
-     }
+       if (callback_wrapper.interrupted())
+         return false;
+
+       get<1>(t) = CGAL::internal::compute_average_spacing<Neighbor_query>
+         (get(point_map, get<0>(t)), neighbor_query, k);
+       ++ callback_wrapper.advancement();
+
+       return true;
+     });
+
+  for (unsigned int i = 0; i < spacings.size (); ++ i)
+    if (spacings[i] >= 0.)
+    {
+      sum_spacings += spacings[i];
+      ++ nb;
+    }
+  callback_wrapper.join();
 
   // return average spacing
-   return sum_spacings / (FT)(nb);
+  return sum_spacings / (FT)(nb);
 }
 
 /// \cond SKIP_IN_MANUAL
diff -pruN 5.0.3-1/include/CGAL/Compute_cone_boundaries_2.h 5.1-1/include/CGAL/Compute_cone_boundaries_2.h
--- 5.0.3-1/include/CGAL/Compute_cone_boundaries_2.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Compute_cone_boundaries_2.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cone_spanners_2/include/CGAL/Compute_cone_boundaries_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cone_spanners_2/include/CGAL/Compute_cone_boundaries_2.h $
 // $Id: Compute_cone_boundaries_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/compute_outer_frame_margin.h 5.1-1/include/CGAL/compute_outer_frame_margin.h
--- 5.0.3-1/include/CGAL/compute_outer_frame_margin.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/compute_outer_frame_margin.h	2020-09-08 12:15:41.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Straight_skeleton_2/include/CGAL/compute_outer_frame_margin.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Straight_skeleton_2/include/CGAL/compute_outer_frame_margin.h $
 // $Id: compute_outer_frame_margin.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Concatenate_iterator.h 5.1-1/include/CGAL/Concatenate_iterator.h
--- 5.0.3-1/include/CGAL/Concatenate_iterator.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Concatenate_iterator.h	2020-09-08 12:15:32.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Concatenate_iterator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Concatenate_iterator.h $
 // $Id: Concatenate_iterator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Concurrent_compact_container.h 5.1-1/include/CGAL/Concurrent_compact_container.h
--- 5.0.3-1/include/CGAL/Concurrent_compact_container.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Concurrent_compact_container.h	2020-09-08 12:15:32.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Concurrent_compact_container.h $
-// $Id: Concurrent_compact_container.h 83fed40 2020-06-19T14:45:32+02:00 Maxime Gimeno
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Concurrent_compact_container.h $
+// $Id: Concurrent_compact_container.h e0b2bc6 2020-06-23T18:49:49+02:00 Laurent Rineau
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Clement Jamin
@@ -66,7 +66,7 @@ template<typename T> class has_##X {
 template < class T >
 struct Concurrent_compact_container_traits {
   static void *   pointer(const T &t) { return t.for_compact_container(); }
-  static void * & pointer(T &t)       { return t.for_compact_container(); }
+  static void set_pointer(T &t, void* p) { t.for_compact_container(p); }
 };
 
 namespace CCC_internal {
@@ -114,6 +114,8 @@ namespace CCC_internal {
 // Free list (head and size)
 template< typename pointer, typename size_type, typename CCC >
 class Free_list {
+  // Not that the implicitly-defined member functions copy the
+  // pointer, and not the pointed data.
 public:
   Free_list() : m_head(nullptr), m_size(0) {
 #if CGAL_CONCURRENT_COMPACT_CONTAINER_APPROXIMATE_SIZE
@@ -150,13 +152,6 @@ public:
 #endif // CGAL_CONCURRENT_COMPACT_CONTAINER_APPROXIMATE_SIZE
   }
   bool empty()               { return size() == 0; }
-  // Warning: copy the pointer, not the data!
-  Free_list& operator= (const Free_list& other)
-  {
-    m_head = other.m_head;
-    m_size = other.m_size;
-    return *this;
-  }
 
   void merge(Free_list &other)
   {
@@ -210,7 +205,8 @@ class Concurrent_compact_container
   typedef Concurrent_compact_container_traits <T>                   Traits;
 
 public:
-  typedef CGAL::Time_stamper_impl<T>                Time_stamper_impl;
+  typedef CGAL::Time_stamper_impl<T>                Time_stamper;
+  typedef Time_stamper                              Time_stamper_impl; // backward compatibility
 
   typedef T                                         value_type;
   typedef Allocator                                 allocator_type;
@@ -241,7 +237,6 @@ public:
 
   explicit Concurrent_compact_container(const Allocator &a = Allocator())
   : m_alloc(a)
-  , m_time_stamper(new Time_stamper_impl())
   {
     init ();
   }
@@ -250,7 +245,6 @@ public:
   Concurrent_compact_container(InputIterator first, InputIterator last,
                     const Allocator & a = Allocator())
   : m_alloc(a)
-  , m_time_stamper(new Time_stamper_impl())
   {
     init();
     std::copy(first, last, CGAL::inserter(*this));
@@ -259,13 +253,18 @@ public:
   // The copy constructor and assignment operator preserve the iterator order
   Concurrent_compact_container(const Concurrent_compact_container &c)
   : m_alloc(c.get_allocator())
-  , m_time_stamper(new Time_stamper_impl())
   {
     init();
     m_block_size = c.m_block_size;
     std::copy(c.begin(), c.end(), CGAL::inserter(*this));
   }
 
+  Concurrent_compact_container(Concurrent_compact_container&& c) noexcept
+  : m_alloc(c.get_allocator())
+  {
+    c.swap(*this);
+  }
+
   Concurrent_compact_container & operator=(const Concurrent_compact_container &c)
   {
     if (&c != this) {
@@ -275,10 +274,16 @@ public:
     return *this;
   }
 
+  Concurrent_compact_container & operator=(Concurrent_compact_container&& c) noexcept
+  {
+    Self tmp(std::move(c));
+    tmp.swap(*this);
+    return *this;
+  }
+
   ~Concurrent_compact_container()
   {
     clear();
-    delete m_time_stamper;
   }
 
   bool is_used(const_iterator ptr) const
@@ -290,11 +295,8 @@ public:
   {
     std::swap(m_alloc, c.m_alloc);
 #if CGAL_CONCURRENT_COMPACT_CONTAINER_APPROXIMATE_SIZE
-    { // non-atomic swap
-      size_type other_capacity = c.m_capacity;
-      c.m_capacity = size_type(m_capacity);
-      m_capacity = other_capacity;
-    }
+    // non-atomic swap of m_capacity
+    c.m_capacity = m_capacity.exchange(c.m_capacity.load());
 #else // not CGAL_CONCURRENT_COMPACT_CONTAINER_APPROXIMATE_SIZE
     std::swap(m_capacity, c.m_capacity);
 #endif // not CGAL_CONCURRENT_COMPACT_CONTAINER_APPROXIMATE_SIZE
@@ -304,7 +306,12 @@ public:
     std::swap(m_last_item, c.m_last_item);
     std::swap(m_free_lists, c.m_free_lists);
     m_all_items.swap(c.m_all_items);
-    std::swap(m_time_stamper, c.m_time_stamper);
+    // non-atomic swap of m_time_stamp
+    c.m_time_stamp = m_time_stamp.exchange(c.m_time_stamp.load());
+  }
+
+  friend void swap(Concurrent_compact_container& a, Concurrent_compact_container& b) {
+    a.swap(b);
   }
 
   iterator begin() { return iterator(m_first_item, 0, 0); }
@@ -552,7 +559,7 @@ private:
   {
     CGAL_assertion(type(ret) == USED);
     fl->dec_size();
-    m_time_stamper->set_time_stamp(ret);
+    Time_stamper::set_time_stamp(ret, m_time_stamp);
     return iterator(ret, 0);
   }
 
@@ -614,8 +621,8 @@ private:
     // This out of range compare is always true and causes lots of
     // unnecessary warnings.
     // CGAL_precondition(0 <= t && t < 4);
-    Traits::pointer(*ptr) = reinterpret_cast<void *>
-      (reinterpret_cast<std::ptrdiff_t>(clean_pointer((char *) p)) + (int) t);
+    Traits::set_pointer(*ptr, reinterpret_cast<void *>
+      (reinterpret_cast<std::ptrdiff_t>(clean_pointer((char *) p)) + (int) t));
   }
 
   typedef tbb::queuing_mutex Mutex;
@@ -627,8 +634,9 @@ private:
   // by walking through the block till its end.
   // This opens up the possibility for the compiler to optimize the clear()
   // function considerably when has_trivial_destructor<T>.
-  typedef std::vector<std::pair<pointer, size_type> >  All_items;
+  using All_items = std::vector<std::pair<pointer, size_type> >;
 
+  using time_stamp_t = std::atomic<std::size_t>;
 
   void init()
   {
@@ -644,25 +652,23 @@ private:
     m_first_item = nullptr;
     m_last_item  = nullptr;
     m_all_items  = All_items();
-    m_time_stamper->reset();
+    m_time_stamp = 0;
   }
 
   allocator_type    m_alloc;
 #if CGAL_CONCURRENT_COMPACT_CONTAINER_APPROXIMATE_SIZE
-  std::atomic<size_type> m_capacity;
+  std::atomic<size_type> m_capacity = {};
 #else // not CGAL_CONCURRENT_COMPACT_CONTAINER_APPROXIMATE_SIZE
-  size_type         m_capacity;
+  size_type         m_capacity      = {};
 #endif // not CGAL_CONCURRENT_COMPACT_CONTAINER_APPROXIMATE_SIZE
-  size_type         m_block_size;
+  size_type         m_block_size    = CGAL_INIT_CONCURRENT_COMPACT_CONTAINER_BLOCK_SIZE;
   Free_lists        m_free_lists;
-  pointer           m_first_item;
-  pointer           m_last_item;
-  All_items         m_all_items;
+  pointer           m_first_item    = nullptr;
+  pointer           m_last_item     = nullptr;
+  All_items         m_all_items     = {};
   mutable Mutex     m_mutex;
+  time_stamp_t      m_time_stamp    = {};
 
-  // This is a pointer, so that the definition of Compact_container does
-  // not require a complete type `T`.
-  Time_stamper_impl* m_time_stamper;
 };
 
 template < class T, class Allocator >
@@ -726,7 +732,7 @@ void Concurrent_compact_container<T, All
     size_type s = it->second;
     for (pointer pp = p + 1; pp != p + s - 1; ++pp) {
       if (type(pp) == USED)
-        m_alloc.destroy(pp);
+        std::allocator_traits<allocator_type>::destroy(m_alloc, pp);
     }
     m_alloc.deallocate(p, s);
   }
@@ -778,7 +784,7 @@ void Concurrent_compact_container<T, All
   for (size_type i = old_block_size; i >= 1; --i)
   {
     EraseCounterStrategy::set_erase_counter(*(new_block + i), 0);
-    m_time_stamper->initialize_time_stamp(new_block + i);
+    Time_stamper::initialize_time_stamp(new_block + i);
     put_on_free_list(new_block + i, fl);
   }
 }
diff -pruN 5.0.3-1/include/CGAL/Cone_spanners_2/Less_by_direction_2.h 5.1-1/include/CGAL/Cone_spanners_2/Less_by_direction_2.h
--- 5.0.3-1/include/CGAL/Cone_spanners_2/Less_by_direction_2.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Cone_spanners_2/Less_by_direction_2.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cone_spanners_2/include/CGAL/Cone_spanners_2/Less_by_direction_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cone_spanners_2/include/CGAL/Cone_spanners_2/Less_by_direction_2.h $
 // $Id: Less_by_direction_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cone_spanners_2/Plane_scan_tree.h 5.1-1/include/CGAL/Cone_spanners_2/Plane_scan_tree.h
--- 5.0.3-1/include/CGAL/Cone_spanners_2/Plane_scan_tree.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Cone_spanners_2/Plane_scan_tree.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cone_spanners_2/include/CGAL/Cone_spanners_2/Plane_scan_tree.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cone_spanners_2/include/CGAL/Cone_spanners_2/Plane_scan_tree.h $
 // $Id: Plane_scan_tree.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cone_spanners_2/Plane_scan_tree_impl.h 5.1-1/include/CGAL/Cone_spanners_2/Plane_scan_tree_impl.h
--- 5.0.3-1/include/CGAL/Cone_spanners_2/Plane_scan_tree_impl.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Cone_spanners_2/Plane_scan_tree_impl.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cone_spanners_2/include/CGAL/Cone_spanners_2/Plane_scan_tree_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cone_spanners_2/include/CGAL/Cone_spanners_2/Plane_scan_tree_impl.h $
 // $Id: Plane_scan_tree_impl.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Cone_spanners_enum_2.h 5.1-1/include/CGAL/Cone_spanners_enum_2.h
--- 5.0.3-1/include/CGAL/Cone_spanners_enum_2.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Cone_spanners_enum_2.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cone_spanners_2/include/CGAL/Cone_spanners_enum_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cone_spanners_2/include/CGAL/Cone_spanners_enum_2.h $
 // $Id: Cone_spanners_enum_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/config.h 5.1-1/include/CGAL/config.h
--- 5.0.3-1/include/CGAL/config.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/config.h	2020-09-08 12:14:54.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/config.h $
-// $Id: config.h 2e8a59d 2020-07-21T15:25:54+02:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/config.h $
+// $Id: config.h 022b1a7 2020-07-21T15:27:49+02:00 Laurent Rineau
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -36,6 +36,11 @@
 #  define WIN64
 #endif
 
+#ifdef _MSC_VER
+#define _SILENCE_CXX17_ALLOCATOR_VOID_DEPRECATION_WARNING 1
+#define _SILENCE_CXX17_OLD_ALLOCATOR_MEMBERS_DEPRECATION_WARNING 1
+#endif
+
 #ifdef CGAL_INCLUDE_WINDOWS_DOT_H
 // Mimic users including this file which defines min max macros
 // and other names leading to name clashes
@@ -101,6 +106,10 @@
 // <boost/type_traits/detail/has_postfix_operator.hpp> fails as well
 #  define BOOST_TT_HAS_POST_DECREMENT_HPP_INCLUDED
 #  define BOOST_TT_HAS_POST_INCREMENT_HPP_INCLUDED
+//work around for moc bug : https://bugreports.qt.io/browse/QTBUG-80990
+#if defined(CGAL_LINKED_WITH_TBB)
+#undef CGAL_LINKED_WITH_TBB
+#endif
 #endif
 
 // Macro used by Boost Parameter. Mesh_3 needs at least 12, before the
@@ -288,6 +297,10 @@
 #if __cplusplus >= 201402L || _MSVC_LANG >= 201402L
 #  define CGAL_CXX14 1
 #endif
+// Same for C++17
+#if __cplusplus >= 201703L || _MSVC_LANG >= 201703L
+#  define CGAL_CXX17 1
+#endif
 
 #if defined(BOOST_NO_CXX11_HDR_FUNCTIONAL) || BOOST_VERSION < 105000
 #define CGAL_CFG_NO_STD_HASH 1
@@ -737,5 +750,4 @@ typedef const void * Nullptr_t;   // Ant
 /// @}
 #include <CGAL/license/lgpl.h>
 
-
 #endif // CGAL_CONFIG_H
diff -pruN 5.0.3-1/include/CGAL/Conic_2.h 5.1-1/include/CGAL/Conic_2.h
--- 5.0.3-1/include/CGAL/Conic_2.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Conic_2.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Conic_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Conic_2.h $
 // $Id: Conic_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/connect_holes.h 5.1-1/include/CGAL/connect_holes.h
--- 5.0.3-1/include/CGAL/connect_holes.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/connect_holes.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/connect_holes.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/connect_holes.h $
 // $Id: connect_holes.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/constant.h 5.1-1/include/CGAL/constant.h
--- 5.0.3-1/include/CGAL/constant.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/constant.h	2020-09-08 12:15:16.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/constant.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/constant.h $
 // $Id: constant.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Constrained_Delaunay_triangulation_2.h 5.1-1/include/CGAL/Constrained_Delaunay_triangulation_2.h
--- 5.0.3-1/include/CGAL/Constrained_Delaunay_triangulation_2.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Constrained_Delaunay_triangulation_2.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/Constrained_Delaunay_triangulation_2.h $
-// $Id: Constrained_Delaunay_triangulation_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/Constrained_Delaunay_triangulation_2.h $
+// $Id: Constrained_Delaunay_triangulation_2.h 86e25f7 2020-03-26T19:23:54+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -179,6 +179,19 @@ public:
   virtual ~Constrained_Delaunay_triangulation_2() {}
 
 
+  // Ensure rule-of-five: define the copy- and move- constructors
+  // as well as the copy- and move- assignment operators.
+  Constrained_Delaunay_triangulation_2(
+      const Constrained_Delaunay_triangulation_2 &) = default;
+  Constrained_Delaunay_triangulation_2(
+      Constrained_Delaunay_triangulation_2 &&) = default;
+
+  Constrained_Delaunay_triangulation_2 &
+  operator=(const Constrained_Delaunay_triangulation_2 &) = default;
+
+  Constrained_Delaunay_triangulation_2 &
+  operator=(Constrained_Delaunay_triangulation_2 &&) = default;
+
   // FLIPS
   bool is_flipable(Face_handle f, int i, bool perturb = true) const;
   void flip(Face_handle& f, int i);
diff -pruN 5.0.3-1/include/CGAL/Constrained_Delaunay_triangulation_face_base_2.h 5.1-1/include/CGAL/Constrained_Delaunay_triangulation_face_base_2.h
--- 5.0.3-1/include/CGAL/Constrained_Delaunay_triangulation_face_base_2.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Constrained_Delaunay_triangulation_face_base_2.h	2020-09-08 12:15:06.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_2/include/CGAL/Constrained_Delaunay_triangulation_face_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_2/include/CGAL/Constrained_Delaunay_triangulation_face_base_2.h $
 // $Id: Constrained_Delaunay_triangulation_face_base_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Constrained_triangulation_2.h 5.1-1/include/CGAL/Constrained_triangulation_2.h
--- 5.0.3-1/include/CGAL/Constrained_triangulation_2.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Constrained_triangulation_2.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/Constrained_triangulation_2.h $
-// $Id: Constrained_triangulation_2.h a8fca35 2020-05-27T16:32:49+02:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/Constrained_triangulation_2.h $
+// $Id: Constrained_triangulation_2.h 871c972 2020-06-03T16:23:22+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -34,13 +34,21 @@
 
 #include <boost/utility/result_of.hpp>
 #include <boost/type_traits/is_floating_point.hpp>
+#include <boost/type_traits/is_same.hpp>
 
 namespace CGAL {
 
-struct No_intersection_tag{};
+struct No_constraint_intersection_tag{};
+struct No_constraint_intersection_requiring_constructions_tag{};
 struct Exact_intersections_tag{}; // to be used with an exact number type
 struct Exact_predicates_tag{}; // to be used with filtered exact number
 
+// This was deprecated and replaced by ` No_constraint_intersection_tag` and `No_constraint_intersection_requiring_constructions_tag`
+// due to an inconsistency between the code and the documenation.
+struct CGAL_DEPRECATED No_intersection_tag :
+  public No_constraint_intersection_requiring_constructions_tag
+{ };
+
 namespace internal {
 
 #ifdef CGAL_CDT_2_DEBUG_INTERSECTIONS
@@ -86,7 +94,7 @@ public:
                                         Triangulation_vertex_base_2<Gt>,
                                         Constrained_triangulation_face_base_2<Gt> > >::type Tds;
 
-  typedef typename Default::Get<Itag_, No_intersection_tag>::type Itag;
+  typedef typename Default::Get<Itag_, No_constraint_intersection_requiring_constructions_tag>::type Itag;
 
   typedef Triangulation_2<Gt,Tds> Triangulation;
   typedef Constrained_triangulation_2<Gt,Tds_,Itag_>  Constrained_triangulation;
@@ -98,6 +106,7 @@ public:
   typedef typename Triangulation::size_type size_type;
   typedef typename Triangulation::Locate_type Locate_type;
   typedef typename Triangulation::All_faces_iterator All_faces_iterator;
+  typedef typename Triangulation::Finite_edges_iterator Finite_edges_iterator;
   typedef typename Triangulation::Face_circulator Face_circulator;
   typedef typename Triangulation::Edge_circulator Edge_circulator;
   typedef typename Triangulation::Vertex_circulator Vertex_circulator;
@@ -122,7 +131,7 @@ public:
   {
     const char* what() const throw ()
     {
-      return "Intersection of constraints while using No_intersection_tag";
+      return "Unauthorized intersections of constraints";
     }
   };
 
@@ -140,6 +149,8 @@ public:
   using Triangulation::geom_traits;
   using Triangulation::all_faces_begin;
   using Triangulation::all_faces_end;
+  using Triangulation::finite_edges_begin;
+  using Triangulation::finite_edges_end;
   using Triangulation::side_of_oriented_circle;
   using Triangulation::is_infinite;
   using Triangulation::collinear_between;
@@ -159,6 +170,7 @@ public:
   using Triangulation::all_edges_begin;
   using Triangulation::all_edges_end;
   using Triangulation::mirror_index;
+  using Triangulation::mirror_edge;
   using Triangulation::orientation;
 #endif
 
@@ -208,6 +220,17 @@ public:
   //TODO Is that destructor correct ?
   virtual ~Constrained_triangulation_2() {}
 
+  // Ensure rule-of-five: define the copy- and move- constructors
+  // as well as the copy- and move- assignment operators.
+  Constrained_triangulation_2(const Constrained_triangulation_2 &) = default;
+  Constrained_triangulation_2(Constrained_triangulation_2 &&) = default;
+
+  Constrained_triangulation_2 &
+  operator=(const Constrained_triangulation_2 &) = default;
+
+  Constrained_triangulation_2 &
+  operator=(Constrained_triangulation_2 &&) = default;
+
 
   Constrained_edges_iterator constrained_edges_begin() const
   {
@@ -306,7 +329,7 @@ template <class OutputIterator>
 void
 insert_constraint(Vertex_handle  vaa, Vertex_handle vbb, OutputIterator out)
 // forces the constrained [va,vb]
-// [va,vb] will eventually be splitted into several edges
+// [va,vb] will eventually be split into several edges
 // if a vertex vc of t lies on segment ab
 // of if ab intersect some constrained edges
 {
@@ -315,7 +338,21 @@ insert_constraint(Vertex_handle  vaa, Ve
 
   Face_handle fr;
   int i;
-  if(includes_edge(vaa,vbb,vi,fr,i)) {
+  if(includes_edge(vaa,vbb,vi,fr,i))
+  {
+    // if the segment (or a subpart of the segment) that we are trying to constraint is already
+    // present in the triangulation and is already marked as constrained,
+    // then this is an intersection
+    if(boost::is_same<Itag, No_constraint_intersection_tag>::value) {
+      if(dimension() == 1) {
+        if(fr->is_constrained(2))
+          throw Intersection_of_constraints_exception();
+      } else {
+        if(fr->is_constrained(i))
+          throw Intersection_of_constraints_exception();
+      }
+    }
+
     mark_constraint(fr,i);
     if (vi != vbb)  {
       insert_constraint(vi,vbb,out);
@@ -449,16 +486,20 @@ protected:
   void mark_constraint(Face_handle fr, int i);
 
   virtual Vertex_handle intersect(Face_handle f, int i,
+                                  Vertex_handle vaa,
+                                  Vertex_handle vbb);
+  Vertex_handle intersect(Face_handle f, int i,
                           Vertex_handle vaa,
-                          Vertex_handle vbb);
+                          Vertex_handle vbb,
+                          No_constraint_intersection_tag);
   Vertex_handle intersect(Face_handle f, int i,
                           Vertex_handle vaa,
                           Vertex_handle vbb,
-                          No_intersection_tag);
+                          No_constraint_intersection_requiring_constructions_tag);
   Vertex_handle intersect(Face_handle f, int i,
                           Vertex_handle vaa,
                           Vertex_handle vbb,
-                           Exact_intersections_tag);
+                          Exact_intersections_tag);
   Vertex_handle intersect(Face_handle f, int i,
                           Vertex_handle vaa,
                           Vertex_handle vbb,
@@ -530,7 +571,7 @@ public:
     {
       Edge_circulator ec=incident_edges(v), done(ec);
       bool are_there = false;
-      if (ec == 0) return are_there;
+      if (ec == nullptr) return are_there;
       do {
         if(is_constrained(*ec)) {
           *out++ = *ec;
@@ -546,7 +587,7 @@ public:
  OutputItEdges  incident_constraints(Vertex_handle v,
                                       OutputItEdges out) const {
    Edge_circulator ec=incident_edges(v), done(ec);
-   if (ec == 0) return  out;
+   if (ec == nullptr) return  out;
    do {
      if(is_constrained(*ec))    *out++ = *ec;
      ec++;
@@ -649,16 +690,51 @@ insert(const Point& a, Locate_type lt, F
   Vertex_handle v1, v2;
   bool insert_in_constrained_edge = false;
 
-  if ( lt == Triangulation::EDGE && loc->is_constrained(li) ){
+  std::list<std::pair<Vertex_handle,Vertex_handle> > constrained_edges;
+  bool one_dimensional = false;
+  if(dimension() == 1){
+    one_dimensional = true;
+    for(Finite_edges_iterator it = finite_edges_begin();
+        it != finite_edges_end();
+        ++it){
+      if(is_constrained(*it)){
+        constrained_edges.emplace_back(it->first->vertex(cw(it->second)),
+                                       it->first->vertex(ccw(it->second)));
+      }
+    }
+  }
+  if ( lt == Triangulation::EDGE && loc->is_constrained(li) )
+  {
+    if(boost::is_same<Itag, No_constraint_intersection_tag>::value)
+      throw Intersection_of_constraints_exception();
+
     insert_in_constrained_edge = true;
     v1=loc->vertex(ccw(li)); //endpoint of the constraint
     v2=loc->vertex(cw(li)); // endpoint of the constraint
   }
 
   va = Triangulation::insert(a,lt,loc,li);
-  if (insert_in_constrained_edge) update_constraints_incident(va, v1,v2);
-  else if(lt != Triangulation::VERTEX) clear_constraints_incident(va);
-  if (dimension() == 2) update_constraints_opposite(va);
+
+  if(one_dimensional && (dimension() == 2)){
+    for(const std::pair<Vertex_handle,Vertex_handle>& vp : constrained_edges){
+      Face_handle fh;
+      int i;
+      if(this->is_edge(vp.first, vp.second, fh,i)){
+        fh->set_constraint(i,true);
+        boost::tie(fh,i) = mirror_edge(Edge(fh,i));
+        fh->set_constraint(i,true);
+      }
+    }
+  }
+
+  if (insert_in_constrained_edge)
+    update_constraints_incident(va, v1,v2);
+  else if(lt != Triangulation::VERTEX)
+    clear_constraints_incident(va);
+
+  if (dimension() == 2)
+    update_constraints_opposite(va);
+
   return va;
 }
 
@@ -734,7 +810,21 @@ insert_constraint(Vertex_handle  vaa, Ve
 
     Face_handle fr;
     int i;
-    if(includes_edge(vaa,vbb,vi,fr,i)) {
+    if(includes_edge(vaa,vbb,vi,fr,i))
+    {
+      // if the segment (or a subpart of the segment) that we are trying to constraint is already
+      // present in the triangulation and is already marked as constrained,
+      // then this is an intersection
+      if(boost::is_same<Itag, No_constraint_intersection_tag>::value) {
+        if(dimension() == 1) {
+          if(fr->is_constrained(2))
+            throw Intersection_of_constraints_exception();
+        } else {
+          if(fr->is_constrained(i))
+            throw Intersection_of_constraints_exception();
+        }
+      }
+
       mark_constraint(fr,i);
       if (vi != vbb)  {
         stack.push(std::make_pair(vi,vbb));
@@ -744,7 +834,6 @@ insert_constraint(Vertex_handle  vaa, Ve
 
     List_faces intersected_faces;
     List_edges conflict_boundary_ab, conflict_boundary_ba;
-
     bool intersection  = find_intersected_faces( vaa, vbb,
                                                  intersected_faces,
                                                  conflict_boundary_ab,
@@ -926,9 +1015,20 @@ Constrained_triangulation_2<Gt,Tds,Itag>
 intersect(Face_handle , int ,
           Vertex_handle ,
           Vertex_handle ,
-          No_intersection_tag)
+          No_constraint_intersection_tag)
 {
+  throw Intersection_of_constraints_exception();
+  return Vertex_handle() ;
+}
 
+template <class Gt, class Tds, class Itag >
+typename Constrained_triangulation_2<Gt,Tds,Itag>::Vertex_handle
+Constrained_triangulation_2<Gt,Tds,Itag>::
+intersect(Face_handle , int ,
+          Vertex_handle ,
+          Vertex_handle ,
+          No_constraint_intersection_requiring_constructions_tag)
+{
   throw Intersection_of_constraints_exception();
   return Vertex_handle() ;
 }
@@ -953,7 +1053,7 @@ intersect(Face_handle f, int i,
   "would avoid cascading intersection computation\n"
   " and be much more efficient\n"
   "This message is shown only if CGAL_NO_CDT_2_WARNING"
-  "is not defined.\n");
+  " is not defined.\n");
 #endif
   const Point& pa = vaa->point();
   const Point& pb = vbb->point();
@@ -1072,7 +1172,7 @@ update_constraints_incident(Vertex_handl
     //dimension() ==2
     int cwi, ccwi, indf;
     Face_circulator fc=incident_faces(va), done(fc);
-    CGAL_triangulation_assertion(fc != 0);
+    CGAL_triangulation_assertion(fc != nullptr);
     do {
       indf = fc->index(va);
       cwi=cw(indf);
@@ -1099,7 +1199,7 @@ clear_constraints_incident(Vertex_handle
  Edge_circulator ec=incident_edges(va), done(ec);
  Face_handle f;
  int indf;
-  if ( ec != 0){
+  if ( ec != nullptr){
     do {
       f = (*ec).first ;
       indf = (*ec).second;
@@ -1294,7 +1394,7 @@ Constrained_triangulation_2<Gt,Tds,Itag>
 remove_incident_constraints(Vertex_handle v)
 {
    Edge_circulator ec=incident_edges(v), done(ec);
-   if (ec == 0) return;
+   if (ec == nullptr) return;
    do {
         if(is_constrained(*ec)) { remove_constrained_edge((*ec).first,
                                                    (*ec).second);}
@@ -1492,7 +1592,20 @@ intersection(const Gt& ,
              const typename Gt::Point_2& ,
              const typename Gt::Point_2& ,
              typename Gt::Point_2& ,
-             No_intersection_tag)
+             No_constraint_intersection_tag)
+{
+  return false;
+}
+
+template<class Gt>
+bool
+intersection(const Gt& ,
+             const typename Gt::Point_2& ,
+             const typename Gt::Point_2& ,
+             const typename Gt::Point_2& ,
+             const typename Gt::Point_2& ,
+             typename Gt::Point_2& ,
+             No_constraint_intersection_requiring_constructions_tag)
 {
   return false;
 }
@@ -1624,7 +1737,19 @@ limit_intersection(const Gt& ,
                    const typename Gt::Point_2& ,
                    const typename Gt::Point_2& ,
                    const typename Gt::Point_2& ,
-                   No_intersection_tag)
+                   No_constraint_intersection_tag)
+{
+  return 0;
+}
+
+template<class Gt>
+int
+limit_intersection(const Gt& ,
+                   const typename Gt::Point_2& ,
+                   const typename Gt::Point_2& ,
+                   const typename Gt::Point_2& ,
+                   const typename Gt::Point_2& ,
+                   No_constraint_intersection_requiring_constructions_tag)
 {
   return 0;
 }
diff -pruN 5.0.3-1/include/CGAL/Constrained_triangulation_face_base_2.h 5.1-1/include/CGAL/Constrained_triangulation_face_base_2.h
--- 5.0.3-1/include/CGAL/Constrained_triangulation_face_base_2.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Constrained_triangulation_face_base_2.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/Constrained_triangulation_face_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/Constrained_triangulation_face_base_2.h $
 // $Id: Constrained_triangulation_face_base_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Constrained_triangulation_plus_2.h 5.1-1/include/CGAL/Constrained_triangulation_plus_2.h
--- 5.0.3-1/include/CGAL/Constrained_triangulation_plus_2.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Constrained_triangulation_plus_2.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/Constrained_triangulation_plus_2.h $
-// $Id: Constrained_triangulation_plus_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/Constrained_triangulation_plus_2.h $
+// $Id: Constrained_triangulation_plus_2.h dcc4fb1 2020-05-20T09:46:26+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -21,7 +21,9 @@
 #include <CGAL/triangulation_assertions.h>
 #include <CGAL/Polygon_2.h>
 #include <CGAL/Triangulation_2/internal/Polyline_constraint_hierarchy_2.h>
+#include <CGAL/Triangulation_2/internal/CTP2_subconstraint_graph.h>
 #include <boost/tuple/tuple.hpp>
+#include <boost/type_traits/is_same.hpp>
 
 #include <CGAL/Default.h>
 #include <CGAL/Constrained_Delaunay_triangulation_2.h>
@@ -215,6 +217,8 @@ public:
     , hierarchy(Vh_less_xy(this))
   { copy_triangulation(ctp);}
 
+  Constrained_triangulation_plus_2(Constrained_triangulation_plus_2&&) = default;
+
   virtual ~Constrained_triangulation_plus_2() {}
 
   Constrained_triangulation_plus_2 & operator=(const Constrained_triangulation_plus_2& ctp)
@@ -223,6 +227,8 @@ public:
     return *this;
   }
 
+  Constrained_triangulation_plus_2& operator=(Constrained_triangulation_plus_2&&) = default;
+
   template<class InputIterator>
   Constrained_triangulation_plus_2(InputIterator first,
                                    InputIterator last,
@@ -314,6 +320,22 @@ public:
     return n;
   }
   */
+
+  void split_subconstraint_graph_into_constraints(const std::function<bool(Vertex_handle)>& is_terminal
+                                                  = std::function<bool(Vertex_handle)>())
+  {
+    internal::CTP2_graph_visitor<Self> visitor(*this);
+    if (is_terminal)
+      CGAL::split_graph_into_polylines (internal::CTP2_subconstraint_graph<Self>(*this), visitor,
+                                        [&is_terminal](Vertex_handle vh,
+                                                       const internal::CTP2_subconstraint_graph<Self>&) -> bool
+                                        {
+                                          return is_terminal(vh);
+                                        });
+    else
+      CGAL::split_graph_into_polylines (internal::CTP2_subconstraint_graph<Self>(*this), visitor);
+  }
+
   Vertex_handle push_back(const Point& p)
   {
     return insert(p);
@@ -397,7 +419,7 @@ public:
     ++beg;
     Face_container<Constrained_triangulation_plus_2> fc(*this);
 
-    Constraint_id head = 0, tail = 0;
+    Constraint_id head = nullptr, tail = nullptr;
     if(pos != beg){
       // split off head
       --pos;
@@ -478,7 +500,7 @@ public:
       return pos;
 
     }
-    Constraint_id head = 0, tail = 0;
+    Constraint_id head = nullptr, tail = nullptr;
     Vertices_in_constraint_iterator bit = vertices_in_constraint_begin(cid);
     Vertices_in_constraint_iterator pred = pos;
     --pred;
@@ -677,12 +699,16 @@ public:
   }
 
   virtual Vertex_handle intersect(Face_handle f, int i,
+                                  Vertex_handle vaa,
+                                  Vertex_handle vbb);
+  Vertex_handle intersect(Face_handle f, int i,
                           Vertex_handle vaa,
-                          Vertex_handle vbb);
+                          Vertex_handle vbb,
+                          No_constraint_intersection_tag);
   Vertex_handle intersect(Face_handle f, int i,
                           Vertex_handle vaa,
                           Vertex_handle vbb,
-                          No_intersection_tag);
+                          No_constraint_intersection_requiring_constructions_tag);
   Vertex_handle intersect(Face_handle f, int i,
                           Vertex_handle vaa,
                           Vertex_handle vbb,
@@ -852,7 +878,7 @@ insert_subconstraint(Vertex_handle vaa,
                      Vertex_handle vbb,
                      OutputItertator out)
   // insert the subconstraint [vaa vbb]
-  // it will eventually be splitted into several subconstraints
+  // it will eventually be split into several subconstraints
 {
   std::stack<std::pair<Vertex_handle, Vertex_handle> > stack;
   stack.push(std::make_pair(vaa,vbb));
@@ -1018,11 +1044,16 @@ insert(const Point& a, Locate_type lt, F
   Vertex_handle v1, v2;
   bool insert_in_constrained_edge = false;
 
-  if ( lt == Triangulation::EDGE && loc->is_constrained(li) ){
+  if ( lt == Triangulation::EDGE && loc->is_constrained(li) )
+  {
+    if(boost::is_same<typename Tr::Itag, No_constraint_intersection_tag>::value)
+      throw typename Tr::Intersection_of_constraints_exception();
+
     insert_in_constrained_edge = true;
     v1=loc->vertex(ccw(li)); //endpoint of the constraint
     v2=loc->vertex(cw(li)); // endpoint of the constraint
   }
+
   Vertex_handle va = Triangulation::insert(a,lt,loc,li);
   // update the hierarchy
   if (insert_in_constrained_edge) {
@@ -1044,16 +1075,24 @@ intersect(Face_handle f, int i,
 template <class Tr>
 typename Constrained_triangulation_plus_2<Tr>:: Vertex_handle
 Constrained_triangulation_plus_2<Tr>::
+intersect(Face_handle, int,
+          Vertex_handle,
+          Vertex_handle,
+          No_constraint_intersection_tag)
+{
+  throw typename Tr::Intersection_of_constraints_exception();
+  return Vertex_handle();
+}
 
-intersect(Face_handle , int ,
-          Vertex_handle ,
-          Vertex_handle ,
-          No_intersection_tag)
+template <class Tr>
+typename Constrained_triangulation_plus_2<Tr>:: Vertex_handle
+Constrained_triangulation_plus_2<Tr>::
+intersect(Face_handle, int,
+          Vertex_handle,
+          Vertex_handle,
+          No_constraint_intersection_requiring_constructions_tag)
 {
-  std::cerr << " sorry, this triangulation does not deal with"
-            <<    std::endl
-            << " intersecting constraints" << std::endl;
-  CGAL_triangulation_assertion(false);
+  throw typename Tr::Intersection_of_constraints_exception();
   return Vertex_handle();
 }
 
diff -pruN 5.0.3-1/include/CGAL/Constrained_voronoi_diagram_2.h 5.1-1/include/CGAL/Constrained_voronoi_diagram_2.h
--- 5.0.3-1/include/CGAL/Constrained_voronoi_diagram_2.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Constrained_voronoi_diagram_2.h	2020-09-08 12:15:06.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_2/include/CGAL/Constrained_voronoi_diagram_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_2/include/CGAL/Constrained_voronoi_diagram_2.h $
 // $Id: Constrained_voronoi_diagram_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/constructions/constructions_for_voronoi_intersection_cartesian_2_3.h 5.1-1/include/CGAL/constructions/constructions_for_voronoi_intersection_cartesian_2_3.h
--- 5.0.3-1/include/CGAL/constructions/constructions_for_voronoi_intersection_cartesian_2_3.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/constructions/constructions_for_voronoi_intersection_cartesian_2_3.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Interpolation/include/CGAL/constructions/constructions_for_voronoi_intersection_cartesian_2_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Interpolation/include/CGAL/constructions/constructions_for_voronoi_intersection_cartesian_2_3.h $
 // $Id: constructions_for_voronoi_intersection_cartesian_2_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/constructions/kernel_ftC2.h 5.1-1/include/CGAL/constructions/kernel_ftC2.h
--- 5.0.3-1/include/CGAL/constructions/kernel_ftC2.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/constructions/kernel_ftC2.h	2020-09-08 12:14:45.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/constructions/kernel_ftC2.h $
-// $Id: kernel_ftC2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/constructions/kernel_ftC2.h $
+// $Id: kernel_ftC2.h 5c41857 2020-04-08T13:03:50+02:00 Maxime Gimeno
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -268,6 +268,13 @@ line_y_at_xC2(const FT &a, const FT &b,
   return (-a*x-c) / b;
 }
 
+// Silence a warning for MSVC 2017
+// > include\cgal\constructions\kernel_ftc2.h(287) :
+// >   warning C4723: potential divide by 0
+#if defined(BOOST_MSVC)
+#pragma warning(push)
+#pragma warning(disable:4723)
+#endif
 template < class FT >
 inline
 void
@@ -276,14 +283,6 @@ line_get_pointC2(const FT &a, const FT &
 {
   if (CGAL_NTS is_zero(b))
     {
-      // Laurent Rineau, 2018/12/07: I add this CGAL_assume to calm
-      // down a warning from MSVC 2017:
-      // > include\cgal\constructions\kernel_ftc2.h(287) :
-      // >   warning C4723: potential divide by 0
-      // The test `!boost::is_integral<FT>::value` is there to avoid
-      // that `a != 0` is tested on anything but integral types, for
-      // performance reasons.
-      CGAL_assume(!boost::is_integral<FT>::value || a != FT(0));
 
       x = -c/a;
       y = 1 - i * a;
@@ -294,6 +293,9 @@ line_get_pointC2(const FT &a, const FT &
       y = -(a+c)/b - i * a;
     }
 }
+#if defined(BOOST_MSVC)
+#pragma warning(pop)
+#endif
 
 template < class FT >
 inline
diff -pruN 5.0.3-1/include/CGAL/constructions/kernel_ftC3.h 5.1-1/include/CGAL/constructions/kernel_ftC3.h
--- 5.0.3-1/include/CGAL/constructions/kernel_ftC3.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/constructions/kernel_ftC3.h	2020-09-08 12:14:45.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/constructions/kernel_ftC3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/constructions/kernel_ftC3.h $
 // $Id: kernel_ftC3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/constructions/Polygon_offset_cons_ftC2.h 5.1-1/include/CGAL/constructions/Polygon_offset_cons_ftC2.h
--- 5.0.3-1/include/CGAL/constructions/Polygon_offset_cons_ftC2.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/constructions/Polygon_offset_cons_ftC2.h	2020-09-08 12:15:41.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Straight_skeleton_2/include/CGAL/constructions/Polygon_offset_cons_ftC2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Straight_skeleton_2/include/CGAL/constructions/Polygon_offset_cons_ftC2.h $
 // $Id: Polygon_offset_cons_ftC2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/constructions/Straight_skeleton_cons_ftC2.h 5.1-1/include/CGAL/constructions/Straight_skeleton_cons_ftC2.h
--- 5.0.3-1/include/CGAL/constructions/Straight_skeleton_cons_ftC2.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/constructions/Straight_skeleton_cons_ftC2.h	2020-09-08 12:15:41.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Straight_skeleton_2/include/CGAL/constructions/Straight_skeleton_cons_ftC2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Straight_skeleton_2/include/CGAL/constructions/Straight_skeleton_cons_ftC2.h $
 // $Id: Straight_skeleton_cons_ftC2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/constructions_d.h 5.1-1/include/CGAL/constructions_d.h
--- 5.0.3-1/include/CGAL/constructions_d.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/constructions_d.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/constructions_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/constructions_d.h $
 // $Id: constructions_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Construct_theta_graph_2.h 5.1-1/include/CGAL/Construct_theta_graph_2.h
--- 5.0.3-1/include/CGAL/Construct_theta_graph_2.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Construct_theta_graph_2.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cone_spanners_2/include/CGAL/Construct_theta_graph_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cone_spanners_2/include/CGAL/Construct_theta_graph_2.h $
 // $Id: Construct_theta_graph_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Construct_yao_graph_2.h 5.1-1/include/CGAL/Construct_yao_graph_2.h
--- 5.0.3-1/include/CGAL/Construct_yao_graph_2.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Construct_yao_graph_2.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cone_spanners_2/include/CGAL/Construct_yao_graph_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cone_spanners_2/include/CGAL/Construct_yao_graph_2.h $
 // $Id: Construct_yao_graph_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Container_helper.h 5.1-1/include/CGAL/Container_helper.h
--- 5.0.3-1/include/CGAL/Container_helper.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Container_helper.h	2020-09-08 12:15:32.000000000 +0000
@@ -0,0 +1,80 @@
+// Copyright (c) 2014  GeometryFactory (France).  All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Container_helper.h $
+// $Id: Container_helper.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
+// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Andreas Fabri
+
+#ifndef CGAL_CONTAINER_HELPER_H
+#define CGAL_CONTAINER_HELPER_H
+
+#include <CGAL/array.h>
+#include <CGAL/Has_member.h>
+
+#include <CGAL/assertions.h>
+#include <boost/mpl/logical.hpp>
+#include <boost/utility/enable_if.hpp>
+
+#include <cstddef>
+
+namespace CGAL {
+namespace internal {
+
+CGAL_GENERATE_MEMBER_DETECTOR(size);
+CGAL_GENERATE_MEMBER_DETECTOR(resize);
+
+// Typical container
+template <class Container>
+void resize(Container& c, std::size_t size,
+            typename boost::enable_if_c<has_resize<Container>::value>::type* = nullptr)
+{
+  c.resize(size);
+}
+
+// Container without a resize() function, but with a size() function (e.g. an array)
+template <class Container>
+void resize(Container& CGAL_assertion_code(array), std::size_t CGAL_assertion_code(size),
+            typename boost::enable_if<
+              boost::mpl::and_<
+                boost::mpl::not_<has_resize<Container> >,
+                                 has_size<Container> > >::type* = nullptr)
+{
+  CGAL_assertion(array.size() == size);
+}
+
+// A class with neither resize() nor size(), can't enforce size (it better be correct!)
+template <class Container>
+void resize(Container&, std::size_t,
+            typename boost::disable_if<
+              boost::mpl::or_<has_resize<Container>,
+                              has_size<Container> > >::type* = nullptr)
+{
+}
+
+// Plenty of times we ask for a model of RandomAccessContainer and use reserve, but some models
+// of the concept do not have a reserve (e.g. deque)
+CGAL_GENERATE_MEMBER_DETECTOR(reserve);
+
+// Container with 'reserve'
+template <class Container>
+void reserve(Container& c, std::size_t size,
+             typename boost::enable_if_c<has_reserve<Container>::value>::type* = nullptr)
+{
+  c.reserve(size);
+}
+
+// Container with no 'reserve'
+template <class Container>
+void reserve(Container&, std::size_t,
+             typename boost::disable_if_c<has_reserve<Container>::value>::type* = nullptr)
+{
+}
+
+} // namespace internal
+} // namespace CGAL
+
+#endif // CGAL_CONTAINER_HELPER_H
diff -pruN 5.0.3-1/include/CGAL/convert_to_bfi.h 5.1-1/include/CGAL/convert_to_bfi.h
--- 5.0.3-1/include/CGAL/convert_to_bfi.h	2020-08-01 19:02:25.000000000 +0000
+++ 5.1-1/include/CGAL/convert_to_bfi.h	2020-09-08 12:14:59.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Interval_support/include/CGAL/convert_to_bfi.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Interval_support/include/CGAL/convert_to_bfi.h $
 // $Id: convert_to_bfi.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_decomposition_3/Edge_sorter.h 5.1-1/include/CGAL/Convex_decomposition_3/Edge_sorter.h
--- 5.0.3-1/include/CGAL/Convex_decomposition_3/Edge_sorter.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_decomposition_3/Edge_sorter.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_decomposition_3/include/CGAL/Convex_decomposition_3/Edge_sorter.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_decomposition_3/include/CGAL/Convex_decomposition_3/Edge_sorter.h $
 // $Id: Edge_sorter.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_decomposition_3/External_structure_builder.h 5.1-1/include/CGAL/Convex_decomposition_3/External_structure_builder.h
--- 5.0.3-1/include/CGAL/Convex_decomposition_3/External_structure_builder.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_decomposition_3/External_structure_builder.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_decomposition_3/include/CGAL/Convex_decomposition_3/External_structure_builder.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_decomposition_3/include/CGAL/Convex_decomposition_3/External_structure_builder.h $
 // $Id: External_structure_builder.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_decomposition_3/Insert_vertex_into_edge.h 5.1-1/include/CGAL/Convex_decomposition_3/Insert_vertex_into_edge.h
--- 5.0.3-1/include/CGAL/Convex_decomposition_3/Insert_vertex_into_edge.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_decomposition_3/Insert_vertex_into_edge.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_decomposition_3/include/CGAL/Convex_decomposition_3/Insert_vertex_into_edge.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_decomposition_3/include/CGAL/Convex_decomposition_3/Insert_vertex_into_edge.h $
 // $Id: Insert_vertex_into_edge.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_decomposition_3/is_reflex_sedge.h 5.1-1/include/CGAL/Convex_decomposition_3/is_reflex_sedge.h
--- 5.0.3-1/include/CGAL/Convex_decomposition_3/is_reflex_sedge.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_decomposition_3/is_reflex_sedge.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_decomposition_3/include/CGAL/Convex_decomposition_3/is_reflex_sedge.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_decomposition_3/include/CGAL/Convex_decomposition_3/is_reflex_sedge.h $
 // $Id: is_reflex_sedge.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_decomposition_3/Ray_hit_generator2.h 5.1-1/include/CGAL/Convex_decomposition_3/Ray_hit_generator2.h
--- 5.0.3-1/include/CGAL/Convex_decomposition_3/Ray_hit_generator2.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_decomposition_3/Ray_hit_generator2.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_decomposition_3/include/CGAL/Convex_decomposition_3/Ray_hit_generator2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_decomposition_3/include/CGAL/Convex_decomposition_3/Ray_hit_generator2.h $
 // $Id: Ray_hit_generator2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_decomposition_3/Ray_hit_generator.h 5.1-1/include/CGAL/Convex_decomposition_3/Ray_hit_generator.h
--- 5.0.3-1/include/CGAL/Convex_decomposition_3/Ray_hit_generator.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_decomposition_3/Ray_hit_generator.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_decomposition_3/include/CGAL/Convex_decomposition_3/Ray_hit_generator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_decomposition_3/include/CGAL/Convex_decomposition_3/Ray_hit_generator.h $
 // $Id: Ray_hit_generator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_decomposition_3/Reflex_edge_searcher.h 5.1-1/include/CGAL/Convex_decomposition_3/Reflex_edge_searcher.h
--- 5.0.3-1/include/CGAL/Convex_decomposition_3/Reflex_edge_searcher.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_decomposition_3/Reflex_edge_searcher.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_decomposition_3/include/CGAL/Convex_decomposition_3/Reflex_edge_searcher.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_decomposition_3/include/CGAL/Convex_decomposition_3/Reflex_edge_searcher.h $
 // $Id: Reflex_edge_searcher.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_decomposition_3/Reflex_vertex_searcher.h 5.1-1/include/CGAL/Convex_decomposition_3/Reflex_vertex_searcher.h
--- 5.0.3-1/include/CGAL/Convex_decomposition_3/Reflex_vertex_searcher.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_decomposition_3/Reflex_vertex_searcher.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_decomposition_3/include/CGAL/Convex_decomposition_3/Reflex_vertex_searcher.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_decomposition_3/include/CGAL/Convex_decomposition_3/Reflex_vertex_searcher.h $
 // $Id: Reflex_vertex_searcher.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_decomposition_3/SFace_separator.h 5.1-1/include/CGAL/Convex_decomposition_3/SFace_separator.h
--- 5.0.3-1/include/CGAL/Convex_decomposition_3/SFace_separator.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_decomposition_3/SFace_separator.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_decomposition_3/include/CGAL/Convex_decomposition_3/SFace_separator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_decomposition_3/include/CGAL/Convex_decomposition_3/SFace_separator.h $
 // $Id: SFace_separator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_decomposition_3/Single_wall_creator2.h 5.1-1/include/CGAL/Convex_decomposition_3/Single_wall_creator2.h
--- 5.0.3-1/include/CGAL/Convex_decomposition_3/Single_wall_creator2.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_decomposition_3/Single_wall_creator2.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_decomposition_3/include/CGAL/Convex_decomposition_3/Single_wall_creator2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_decomposition_3/include/CGAL/Convex_decomposition_3/Single_wall_creator2.h $
 // $Id: Single_wall_creator2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_decomposition_3/Single_wall_creator3.h 5.1-1/include/CGAL/Convex_decomposition_3/Single_wall_creator3.h
--- 5.0.3-1/include/CGAL/Convex_decomposition_3/Single_wall_creator3.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_decomposition_3/Single_wall_creator3.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_decomposition_3/include/CGAL/Convex_decomposition_3/Single_wall_creator3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_decomposition_3/include/CGAL/Convex_decomposition_3/Single_wall_creator3.h $
 // $Id: Single_wall_creator3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_decomposition_3/Single_wall_creator.h 5.1-1/include/CGAL/Convex_decomposition_3/Single_wall_creator.h
--- 5.0.3-1/include/CGAL/Convex_decomposition_3/Single_wall_creator.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_decomposition_3/Single_wall_creator.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_decomposition_3/include/CGAL/Convex_decomposition_3/Single_wall_creator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_decomposition_3/include/CGAL/Convex_decomposition_3/Single_wall_creator.h $
 // $Id: Single_wall_creator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_decomposition_3/SM_walls.h 5.1-1/include/CGAL/Convex_decomposition_3/SM_walls.h
--- 5.0.3-1/include/CGAL/Convex_decomposition_3/SM_walls.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_decomposition_3/SM_walls.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_decomposition_3/include/CGAL/Convex_decomposition_3/SM_walls.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_decomposition_3/include/CGAL/Convex_decomposition_3/SM_walls.h $
 // $Id: SM_walls.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_decomposition_3/YVertical_wall_builder.h 5.1-1/include/CGAL/Convex_decomposition_3/YVertical_wall_builder.h
--- 5.0.3-1/include/CGAL/Convex_decomposition_3/YVertical_wall_builder.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_decomposition_3/YVertical_wall_builder.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_decomposition_3/include/CGAL/Convex_decomposition_3/YVertical_wall_builder.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_decomposition_3/include/CGAL/Convex_decomposition_3/YVertical_wall_builder.h $
 // $Id: YVertical_wall_builder.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/convex_decomposition_3.h 5.1-1/include/CGAL/convex_decomposition_3.h
--- 5.0.3-1/include/CGAL/convex_decomposition_3.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/convex_decomposition_3.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_decomposition_3/include/CGAL/convex_decomposition_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_decomposition_3/include/CGAL/convex_decomposition_3.h $
 // $Id: convex_decomposition_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_hull_2/ch_akl_toussaint_impl.h 5.1-1/include/CGAL/Convex_hull_2/ch_akl_toussaint_impl.h
--- 5.0.3-1/include/CGAL/Convex_hull_2/ch_akl_toussaint_impl.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_hull_2/ch_akl_toussaint_impl.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_2/include/CGAL/Convex_hull_2/ch_akl_toussaint_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_2/include/CGAL/Convex_hull_2/ch_akl_toussaint_impl.h $
 // $Id: ch_akl_toussaint_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_hull_2/ch_assertions.h 5.1-1/include/CGAL/Convex_hull_2/ch_assertions.h
--- 5.0.3-1/include/CGAL/Convex_hull_2/ch_assertions.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_hull_2/ch_assertions.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_2/include/CGAL/Convex_hull_2/ch_assertions.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_2/include/CGAL/Convex_hull_2/ch_assertions.h $
 // $Id: ch_assertions.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_hull_2/ch_bykat_impl.h 5.1-1/include/CGAL/Convex_hull_2/ch_bykat_impl.h
--- 5.0.3-1/include/CGAL/Convex_hull_2/ch_bykat_impl.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_hull_2/ch_bykat_impl.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_2/include/CGAL/Convex_hull_2/ch_bykat_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_2/include/CGAL/Convex_hull_2/ch_bykat_impl.h $
 // $Id: ch_bykat_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_hull_2/ch_eddy_impl.h 5.1-1/include/CGAL/Convex_hull_2/ch_eddy_impl.h
--- 5.0.3-1/include/CGAL/Convex_hull_2/ch_eddy_impl.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_hull_2/ch_eddy_impl.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_2/include/CGAL/Convex_hull_2/ch_eddy_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_2/include/CGAL/Convex_hull_2/ch_eddy_impl.h $
 // $Id: ch_eddy_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_hull_2/ch_graham_andrew_impl.h 5.1-1/include/CGAL/Convex_hull_2/ch_graham_andrew_impl.h
--- 5.0.3-1/include/CGAL/Convex_hull_2/ch_graham_andrew_impl.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_hull_2/ch_graham_andrew_impl.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_2/include/CGAL/Convex_hull_2/ch_graham_andrew_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_2/include/CGAL/Convex_hull_2/ch_graham_andrew_impl.h $
 // $Id: ch_graham_andrew_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_hull_2/ch_jarvis_impl.h 5.1-1/include/CGAL/Convex_hull_2/ch_jarvis_impl.h
--- 5.0.3-1/include/CGAL/Convex_hull_2/ch_jarvis_impl.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_hull_2/ch_jarvis_impl.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_2/include/CGAL/Convex_hull_2/ch_jarvis_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_2/include/CGAL/Convex_hull_2/ch_jarvis_impl.h $
 // $Id: ch_jarvis_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_hull_2/ch_melkman_impl.h 5.1-1/include/CGAL/Convex_hull_2/ch_melkman_impl.h
--- 5.0.3-1/include/CGAL/Convex_hull_2/ch_melkman_impl.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_hull_2/ch_melkman_impl.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_2/include/CGAL/Convex_hull_2/ch_melkman_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_2/include/CGAL/Convex_hull_2/ch_melkman_impl.h $
 // $Id: ch_melkman_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_hull_2/ch_selected_extreme_points_2_impl.h 5.1-1/include/CGAL/Convex_hull_2/ch_selected_extreme_points_2_impl.h
--- 5.0.3-1/include/CGAL/Convex_hull_2/ch_selected_extreme_points_2_impl.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_hull_2/ch_selected_extreme_points_2_impl.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_2/include/CGAL/Convex_hull_2/ch_selected_extreme_points_2_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_2/include/CGAL/Convex_hull_2/ch_selected_extreme_points_2_impl.h $
 // $Id: ch_selected_extreme_points_2_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_hull_2/convexity_check_2_impl.h 5.1-1/include/CGAL/Convex_hull_2/convexity_check_2_impl.h
--- 5.0.3-1/include/CGAL/Convex_hull_2/convexity_check_2_impl.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_hull_2/convexity_check_2_impl.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_2/include/CGAL/Convex_hull_2/convexity_check_2_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_2/include/CGAL/Convex_hull_2/convexity_check_2_impl.h $
 // $Id: convexity_check_2_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/convex_hull_2.h 5.1-1/include/CGAL/convex_hull_2.h
--- 5.0.3-1/include/CGAL/convex_hull_2.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/convex_hull_2.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_2/include/CGAL/convex_hull_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_2/include/CGAL/convex_hull_2.h $
 // $Id: convex_hull_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_hull_3/dual/Convex_hull_traits_dual_2.h 5.1-1/include/CGAL/Convex_hull_3/dual/Convex_hull_traits_dual_2.h
--- 5.0.3-1/include/CGAL/Convex_hull_3/dual/Convex_hull_traits_dual_2.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_hull_3/dual/Convex_hull_traits_dual_2.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_3/include/CGAL/Convex_hull_3/dual/Convex_hull_traits_dual_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_3/include/CGAL/Convex_hull_3/dual/Convex_hull_traits_dual_2.h $
 // $Id: Convex_hull_traits_dual_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_hull_3/dual/Convex_hull_traits_dual_3.h 5.1-1/include/CGAL/Convex_hull_3/dual/Convex_hull_traits_dual_3.h
--- 5.0.3-1/include/CGAL/Convex_hull_3/dual/Convex_hull_traits_dual_3.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_hull_3/dual/Convex_hull_traits_dual_3.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_3/include/CGAL/Convex_hull_3/dual/Convex_hull_traits_dual_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_3/include/CGAL/Convex_hull_3/dual/Convex_hull_traits_dual_3.h $
 // $Id: Convex_hull_traits_dual_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_hull_3/dual/halfspace_intersection_3.h 5.1-1/include/CGAL/Convex_hull_3/dual/halfspace_intersection_3.h
--- 5.0.3-1/include/CGAL/Convex_hull_3/dual/halfspace_intersection_3.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_hull_3/dual/halfspace_intersection_3.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_3/include/CGAL/Convex_hull_3/dual/halfspace_intersection_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_3/include/CGAL/Convex_hull_3/dual/halfspace_intersection_3.h $
 // $Id: halfspace_intersection_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_hull_3/dual/halfspace_intersection_with_constructions_3.h 5.1-1/include/CGAL/Convex_hull_3/dual/halfspace_intersection_with_constructions_3.h
--- 5.0.3-1/include/CGAL/Convex_hull_3/dual/halfspace_intersection_with_constructions_3.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_hull_3/dual/halfspace_intersection_with_constructions_3.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_3/include/CGAL/Convex_hull_3/dual/halfspace_intersection_with_constructions_3.h $
-// $Id: halfspace_intersection_with_constructions_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_3/include/CGAL/Convex_hull_3/dual/halfspace_intersection_with_constructions_3.h $
+// $Id: halfspace_intersection_with_constructions_3.h 9f2eafd 2020-03-26T19:17:02+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -142,7 +142,7 @@ namespace CGAL
                                                             boost::optional<typename Kernel_traits<typename std::iterator_traits<PlaneIterator>::value_type>::Kernel::Point_3> const& origin = boost::none) {
           typedef typename Kernel_traits<typename std::iterator_traits<PlaneIterator>::value_type>::Kernel K;
           typedef typename K::Point_3 Point_3;
-          typedef typename internal::Convex_hull_3::Default_traits_for_Chull_3<Point_3>::type Traits;
+          typedef typename Convex_hull_3::internal::Default_traits_for_Chull_3<Point_3>::type Traits;
 
           halfspace_intersection_with_constructions_3(pbegin, pend, P, origin, Traits());
         }
diff -pruN 5.0.3-1/include/CGAL/Convex_hull_3/dual/interior_polyhedron_3.h 5.1-1/include/CGAL/Convex_hull_3/dual/interior_polyhedron_3.h
--- 5.0.3-1/include/CGAL/Convex_hull_3/dual/interior_polyhedron_3.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_hull_3/dual/interior_polyhedron_3.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_3/include/CGAL/Convex_hull_3/dual/interior_polyhedron_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_3/include/CGAL/Convex_hull_3/dual/interior_polyhedron_3.h $
 // $Id: interior_polyhedron_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_hull_3/dual/predicates.h 5.1-1/include/CGAL/Convex_hull_3/dual/predicates.h
--- 5.0.3-1/include/CGAL/Convex_hull_3/dual/predicates.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_hull_3/dual/predicates.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_3/include/CGAL/Convex_hull_3/dual/predicates.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_3/include/CGAL/Convex_hull_3/dual/predicates.h $
 // $Id: predicates.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/convex_hull_3.h 5.1-1/include/CGAL/convex_hull_3.h
--- 5.0.3-1/include/CGAL/convex_hull_3.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/convex_hull_3.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_3/include/CGAL/convex_hull_3.h $
-// $Id: convex_hull_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_3/include/CGAL/convex_hull_3.h $
+// $Id: convex_hull_3.h 9f2eafd 2020-03-26T19:17:02+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -31,31 +31,36 @@
 #include <CGAL/Triangulation_vertex_base_with_info_2.h>
 #include <CGAL/Cartesian_converter.h>
 #include <CGAL/Simple_cartesian.h>
-#include <iostream>
-#include <algorithm>
-#include <utility>
-#include <memory>
-#include <list>
-#include <vector>
-#include <boost/bind.hpp>
-#include <boost/next_prior.hpp>
-#include <boost/type_traits/is_floating_point.hpp>
-#include <boost/type_traits/is_same.hpp>
-#include <boost/mpl/has_xxx.hpp>
-#include <boost/graph/graph_traits.hpp>
+
 #include <CGAL/internal/Exact_type_selector.h>
 #include <CGAL/boost/graph/copy_face_graph.h>
+#include <CGAL/boost/graph/Named_function_parameters.h>
 #include <CGAL/boost/graph/graph_traits_Triangulation_data_structure_2.h>
 #include <CGAL/boost/graph/properties_Triangulation_data_structure_2.h>
 #include <CGAL/Polyhedron_3_fwd.h>
 #include <CGAL/boost/graph/Euler_operations.h>
+#include <CGAL/boost/iterator/transform_iterator.hpp>
+#include <CGAL/boost/graph/named_params_helper.h>
+#include <CGAL/is_iterator.h>
 
-#include <boost/unordered_map.hpp>
+#include <boost/bind.hpp>
+#include <boost/next_prior.hpp>
+#include <boost/type_traits/is_floating_point.hpp>
+#include <boost/type_traits/is_same.hpp>
+#include <boost/mpl/has_xxx.hpp>
+#include <boost/graph/graph_traits.hpp>
 
 #ifndef CGAL_CH_NO_POSTCONDITIONS
 #include <CGAL/convexity_check_3.h>
 #endif // CGAL_CH_NO_POSTCONDITIONS
 
+#include <algorithm>
+#include <iostream>
+#include <list>
+#include <memory>
+#include <vector>
+#include <type_traits>
+#include <utility>
 
 // first some internal stuff to avoid using a true Face_graph model for extreme_points_3
 namespace CGAL {
@@ -63,7 +68,8 @@ namespace CGAL {
 // Forward declaration
 template<class VertexPointMap,class Base_traits> class Extreme_points_traits_adapter_3;
 
-namespace internal{  namespace Convex_hull_3{
+namespace Convex_hull_3 {
+namespace internal {
 
 // wrapper used as a MutableFaceGraph to extract extreme points
 template <class OutputIterator>
@@ -77,29 +83,30 @@ struct Output_iterator_wrapper
 
 template <class Point_3, class OutputIterator>
 void add_isolated_points(const Point_3& point,
-                         internal::Convex_hull_3::Output_iterator_wrapper<OutputIterator>& w)
+                         Convex_hull_3::internal::Output_iterator_wrapper<OutputIterator>& w)
 {
     *w.out++ = point;
 }
 
-template <class Point_3, class Polyhedron_3>
-void add_isolated_points(const Point_3& point, Polyhedron_3& P)
+template <class Point_3, class PolygonMesh>
+void add_isolated_points(const Point_3& point, PolygonMesh& P)
 {
   put(get(CGAL::vertex_point, P), add_vertex(P), point);
 }
 
 template <class Point_3, class OutputIterator>
 void copy_ch2_to_face_graph(const std::list<Point_3>& CH_2,
-                            internal::Convex_hull_3::Output_iterator_wrapper<OutputIterator>& w)
+                            Convex_hull_3::internal::Output_iterator_wrapper<OutputIterator>& w)
 {
   for(const Point_3& p : CH_2)
     *w.out++ = p;
 }
 
-} } // internal::Convex_hull_3
+} // namespace internal
+} // namespace Convex_hull_3
 
 template <class TDS, class OutputIterator>
-void copy_face_graph(const TDS& tds, internal::Convex_hull_3::Output_iterator_wrapper<OutputIterator>& wrapper)
+void copy_face_graph(const TDS& tds, Convex_hull_3::internal::Output_iterator_wrapper<OutputIterator>& wrapper)
 {
   typedef typename boost::graph_traits<TDS>::vertex_descriptor vertex_descriptor;
   typename boost::property_map<TDS, boost::vertex_point_t >::const_type vpm = get(boost::vertex_point, tds);
@@ -110,12 +117,12 @@ void copy_face_graph(const TDS& tds, int
 }
 
 template <class OutputIterator>
-void clear(internal::Convex_hull_3::Output_iterator_wrapper<OutputIterator>&)
+void clear(Convex_hull_3::internal::Output_iterator_wrapper<OutputIterator>&)
 {}
 
 template <class Point, class OutputIterator>
 void make_tetrahedron(const Point& p0, const Point& p1, const Point& p2, const Point& p3,
-                      internal::Convex_hull_3::Output_iterator_wrapper<OutputIterator>& w)
+                      Convex_hull_3::internal::Output_iterator_wrapper<OutputIterator>& w)
 {
   *w.out++ = p0;
   *w.out++ = p1;
@@ -125,19 +132,20 @@ void make_tetrahedron(const Point& p0, c
 
 } // CGAL
 
-namespace boost{
+namespace boost {
+
 // needed so that the regular make_tetrahedron of CGAL does not complain when tried to be instantiated
 template <class OutputIterator>
-struct graph_traits<CGAL::internal::Convex_hull_3::Output_iterator_wrapper<OutputIterator> >
+struct graph_traits<CGAL::Convex_hull_3::internal::Output_iterator_wrapper<OutputIterator> >
 {
   typedef void* halfedge_descriptor;
 };
-}
-
-namespace CGAL {
 
+} // namespace boost
 
-namespace internal{  namespace Convex_hull_3{
+namespace CGAL {
+namespace Convex_hull_3 {
+namespace internal {
 
 //struct to select the default traits class for computing convex hull
 template< class Point_3,
@@ -403,12 +411,12 @@ struct Projection_traits<T,true>{
   {return traits.construct_traits_xz_3_object();}
 };
 
-template <class Point_3, class Polyhedron_3>
-void copy_ch2_to_face_graph(const std::list<Point_3>& CH_2, Polyhedron_3& P)
+template <class Point_3, class PolygonMesh>
+void copy_ch2_to_face_graph(const std::list<Point_3>& CH_2, PolygonMesh& P)
 {
-  typename boost::property_map<Polyhedron_3, CGAL::vertex_point_t>::type vpm
+  typename boost::property_map<PolygonMesh, CGAL::vertex_point_t>::type vpm
     = get(CGAL::vertex_point, P);
-  typedef boost::graph_traits<Polyhedron_3> Graph_traits;
+  typedef boost::graph_traits<PolygonMesh> Graph_traits;
   typedef typename Graph_traits::vertex_descriptor vertex_descriptor;
   typedef typename Graph_traits::halfedge_descriptor halfedge_descriptor;
   typedef typename Graph_traits::face_descriptor face_descriptor;
@@ -431,12 +439,12 @@ void copy_ch2_to_face_graph(const std::l
 }
 
 
-template <class InputIterator, class Point_3, class Polyhedron_3, class Traits>
+template <class InputIterator, class Point_3, class PolygonMesh, class Traits>
 void coplanar_3_hull(InputIterator first, InputIterator beyond,
                      const Point_3& p1, const Point_3& p2, const Point_3& p3,
-                     Polyhedron_3& P, const Traits&  traits )
+                     PolygonMesh& P, const Traits&  traits )
 {
-  typedef typename internal::Convex_hull_3::Projection_traits<Traits> PTraits;
+  typedef typename Convex_hull_3::internal::Projection_traits<Traits> PTraits;
   typedef typename PTraits::Traits_xy_3 Traits_xy_3;
   typedef typename PTraits::Traits_yz_3 Traits_yz_3;
   typedef typename PTraits::Traits_xz_3 Traits_xz_3;
@@ -754,12 +762,12 @@ void non_coplanar_quickhull_3(std::list<
 //  CGAL_ch_postcondition(is_strongly_convex_3(P, traits));
 }
 
-template <class InputIterator, class Polyhedron_3, class Traits>
+template <class InputIterator, class PolygonMesh, class Traits>
 void
-ch_quickhull_polyhedron_3(std::list<typename Traits::Point_3>& points,
-                          InputIterator point1_it, InputIterator point2_it,
-                          InputIterator point3_it, Polyhedron_3& P,
-                          const Traits& traits)
+ch_quickhull_face_graph(std::list<typename Traits::Point_3>& points,
+                        InputIterator point1_it, InputIterator point2_it, InputIterator point3_it,
+                        PolygonMesh& P,
+                        const Traits& traits)
 {
   typedef typename Traits::Point_3                            Point_3;
   typedef typename Traits::Plane_3                                Plane_3;
@@ -844,12 +852,14 @@ ch_quickhull_polyhedron_3(std::list<type
 
 }
 
-} } //namespace internal::Convex_hull_3
+} // namespace internal
+} // namespace Convex_hull_3
 
 template <class InputIterator, class Traits>
 void
 convex_hull_3(InputIterator first, InputIterator beyond,
-              Object& ch_object, const Traits& traits)
+              Object& ch_object,
+              const Traits& traits)
 {
   typedef typename Traits::Point_3                            Point_3;
   typedef std::list<Point_3>                              Point_3_list;
@@ -932,7 +942,7 @@ convex_hull_3(InputIterator first, Input
   }
 
   // result will be a polyhedron
-  typedef typename internal::Convex_hull_3::Default_polyhedron_for_Chull_3<Traits>::type Polyhedron;
+  typedef typename Convex_hull_3::internal::Default_polyhedron_for_Chull_3<Traits>::type Polyhedron;
   Polyhedron P;
 
   P3_iterator minx, maxx, miny, it;
@@ -944,9 +954,9 @@ convex_hull_3(InputIterator first, Input
     if(it->y() < miny->y()) miny = it;
   }
   if(! collinear(*minx, *maxx, *miny) ){
-    internal::Convex_hull_3::ch_quickhull_polyhedron_3(points, minx, maxx, miny, P, traits);
+    Convex_hull_3::internal::ch_quickhull_face_graph(points, minx, maxx, miny, P, traits);
   } else {
-    internal::Convex_hull_3::ch_quickhull_polyhedron_3(points, point1_it, point2_it, point3_it, P, traits);
+    Convex_hull_3::internal::ch_quickhull_face_graph(points, point1_it, point2_it, point3_it, P, traits);
   }
   CGAL_assertion(num_vertices(P)>=3);
   typename boost::graph_traits<Polyhedron>::vertex_iterator b,e;
@@ -969,18 +979,18 @@ convex_hull_3(InputIterator first, Input
 
 
 template <class InputIterator>
-void convex_hull_3(InputIterator first, InputIterator beyond, Object& ch_object)
+void convex_hull_3(InputIterator first, InputIterator beyond,
+                   Object& ch_object)
 {
    typedef typename std::iterator_traits<InputIterator>::value_type Point_3;
-   typedef typename internal::Convex_hull_3::Default_traits_for_Chull_3<Point_3>::type Traits;
+   typedef typename Convex_hull_3::internal::Default_traits_for_Chull_3<Point_3>::type Traits;
    convex_hull_3(first, beyond, ch_object, Traits());
 }
 
-
-
-template <class InputIterator, class Polyhedron_3, class Traits>
+template <class InputIterator, class PolygonMesh, class Traits>
 void convex_hull_3(InputIterator first, InputIterator beyond,
-                   Polyhedron_3& polyhedron,  const Traits& traits)
+                   PolygonMesh& polyhedron,
+                   const Traits& traits)
 {
   typedef typename Traits::Point_3                Point_3;
   typedef std::list<Point_3>                      Point_3_list;
@@ -1005,7 +1015,7 @@ void convex_hull_3(InputIterator first,
   // if there is only one point or all points are equal
   if(point2_it == points.end())
   {
-    internal::Convex_hull_3::add_isolated_points(*point1_it, polyhedron);
+    Convex_hull_3::internal::add_isolated_points(*point1_it, polyhedron);
     return;
   }
 
@@ -1024,25 +1034,50 @@ void convex_hull_3(InputIterator first,
     min_max_element(points.begin(), points.end(),
                     boost::bind(less_dist, *points.begin(), _1, _2),
                     boost::bind(less_dist, *points.begin(), _1, _2));
-    internal::Convex_hull_3::add_isolated_points(*endpoints.first, polyhedron);
-    internal::Convex_hull_3::add_isolated_points(*endpoints.second, polyhedron);
+    Convex_hull_3::internal::add_isolated_points(*endpoints.first, polyhedron);
+    Convex_hull_3::internal::add_isolated_points(*endpoints.second, polyhedron);
     return;
   }
 
-  internal::Convex_hull_3::ch_quickhull_polyhedron_3(points, point1_it, point2_it, point3_it,
+  Convex_hull_3::internal::ch_quickhull_face_graph(points, point1_it, point2_it, point3_it,
     polyhedron, traits);
 }
 
-
-template <class InputIterator, class Polyhedron_3>
+template <class InputIterator, class PolygonMesh>
 void convex_hull_3(InputIterator first, InputIterator beyond,
-                   Polyhedron_3& polyhedron)
-{
-   typedef typename std::iterator_traits<InputIterator>::value_type Point_3;
-   typedef typename internal::Convex_hull_3::Default_traits_for_Chull_3<Point_3, Polyhedron_3>::type Traits;
-   convex_hull_3(first, beyond, polyhedron, Traits());
+                   PolygonMesh& polyhedron,
+                   // workaround to avoid ambiguity with next overload.
+                   typename std::enable_if<CGAL::is_iterator<InputIterator>::value>::type* = 0)
+{
+  typedef typename std::iterator_traits<InputIterator>::value_type Point_3;
+  typedef typename Convex_hull_3::internal::Default_traits_for_Chull_3<Point_3, PolygonMesh>::type Traits;
+  convex_hull_3(first, beyond, polyhedron, Traits());
+}
+
+template <class VertexListGraph, class PolygonMesh, class NamedParameters>
+void convex_hull_3(const VertexListGraph& g,
+                   PolygonMesh& pm,
+                   const NamedParameters& np)
+{
+  using CGAL::parameters::choose_parameter;
+  using CGAL::parameters::get_parameter;
+
+  typedef typename GetVertexPointMap<VertexListGraph, NamedParameters>::const_type Vpmap;
+  typedef CGAL::Property_map_to_unary_function<Vpmap> Vpmap_fct;
+  Vpmap vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
+                               get_const_property_map(boost::vertex_point, g));
+
+  Vpmap_fct v2p(vpm);
+  convex_hull_3(boost::make_transform_iterator(vertices(g).begin(), v2p),
+                boost::make_transform_iterator(vertices(g).end(), v2p), pm);
 }
 
+template <class VertexListGraph, class PolygonMesh>
+void convex_hull_3(const VertexListGraph& g,
+                   PolygonMesh& pm)
+{
+  convex_hull_3(g,pm,CGAL::parameters::all_default());
+}
 
 template <class InputRange, class OutputIterator, class Traits>
 OutputIterator
@@ -1050,7 +1085,7 @@ extreme_points_3(const InputRange& range
                  OutputIterator out,
                  const Traits& traits)
 {
-  internal::Convex_hull_3::Output_iterator_wrapper<OutputIterator> wrapper(out);
+  Convex_hull_3::internal::Output_iterator_wrapper<OutputIterator> wrapper(out);
   convex_hull_3(range.begin(), range.end(), wrapper, traits);
   return out;
 }
@@ -1061,7 +1096,7 @@ extreme_points_3(const InputRange& range
 {
   typedef typename InputRange::const_iterator Iterator_type;
   typedef typename std::iterator_traits<Iterator_type>::value_type Point_3;
-  typedef typename internal::Convex_hull_3::Default_traits_for_Chull_3<Point_3>::type Traits;
+  typedef typename Convex_hull_3::internal::Default_traits_for_Chull_3<Point_3>::type Traits;
 
   return extreme_points_3(range, out, Traits());
 }
diff -pruN 5.0.3-1/include/CGAL/convex_hull_3_to_face_graph.h 5.1-1/include/CGAL/convex_hull_3_to_face_graph.h
--- 5.0.3-1/include/CGAL/convex_hull_3_to_face_graph.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/convex_hull_3_to_face_graph.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_3/include/CGAL/convex_hull_3_to_face_graph.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_3/include/CGAL/convex_hull_3_to_face_graph.h $
 // $Id: convex_hull_3_to_face_graph.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/convex_hull_3_to_polyhedron_3.h 5.1-1/include/CGAL/convex_hull_3_to_polyhedron_3.h
--- 5.0.3-1/include/CGAL/convex_hull_3_to_polyhedron_3.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/convex_hull_3_to_polyhedron_3.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,37 +0,0 @@
-// Copyright (c) 2011  GeometryFactory (France).
-// All rights reserved.
-//
-// This file is part of CGAL (www.cgal.org).
-//
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_3/include/CGAL/convex_hull_3_to_polyhedron_3.h $
-// $Id: convex_hull_3_to_polyhedron_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
-// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
-//
-//
-// Author(s)     : Sebastien Loriot
-//
-
-#ifndef CGAL_CONVEX_HULL_3_TO_POLYHEDRON_3_H
-#define CGAL_CONVEX_HULL_3_TO_POLYHEDRON_3_H
-
-#include <CGAL/license/Convex_hull_3.h>
-
-
-#define CGAL_DEPRECATED_HEADER "<CGAL/convex_hull_3_to_polyhedron_3.h>"
-#define CGAL_REPLACEMENT_HEADER "<CGAL/convex_hull_3_to_face_graph.h>"
-#include <CGAL/internal/deprecation_warning.h>
-
-
-#include <CGAL/Polyhedron_3_fwd.h>
-
-namespace CGAL {
-
-template<class Triangulation_3,class Polyhedron_3>
-CGAL_DEPRECATED void convex_hull_3_to_polyhedron_3(const Triangulation_3& T,Polyhedron_3& P){
-  clear(P);
-  link_to_face_graph(T,T.infinite_vertex(), P);
-}
-
-} //namespace CGAL
-
-#endif //CGAL_CONVEX_HULL_3_TO_POLYHEDRON_3_H
diff -pruN 5.0.3-1/include/CGAL/convex_hull_constructive_traits_2.h 5.1-1/include/CGAL/convex_hull_constructive_traits_2.h
--- 5.0.3-1/include/CGAL/convex_hull_constructive_traits_2.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/convex_hull_constructive_traits_2.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_2/include/CGAL/convex_hull_constructive_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_2/include/CGAL/convex_hull_constructive_traits_2.h $
 // $Id: convex_hull_constructive_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_hull_d.h 5.1-1/include/CGAL/Convex_hull_d.h
--- 5.0.3-1/include/CGAL/Convex_hull_d.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_hull_d.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_d/include/CGAL/Convex_hull_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_d/include/CGAL/Convex_hull_d.h $
 // $Id: Convex_hull_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_hull_d_to_polyhedron_3.h 5.1-1/include/CGAL/Convex_hull_d_to_polyhedron_3.h
--- 5.0.3-1/include/CGAL/Convex_hull_d_to_polyhedron_3.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_hull_d_to_polyhedron_3.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_d/include/CGAL/Convex_hull_d_to_polyhedron_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_d/include/CGAL/Convex_hull_d_to_polyhedron_3.h $
 // $Id: Convex_hull_d_to_polyhedron_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_hull_d_traits_3.h 5.1-1/include/CGAL/Convex_hull_d_traits_3.h
--- 5.0.3-1/include/CGAL/Convex_hull_d_traits_3.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_hull_d_traits_3.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_d/include/CGAL/Convex_hull_d_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_d/include/CGAL/Convex_hull_d_traits_3.h $
 // $Id: Convex_hull_d_traits_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_hull_face_base_2.h 5.1-1/include/CGAL/Convex_hull_face_base_2.h
--- 5.0.3-1/include/CGAL/Convex_hull_face_base_2.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_hull_face_base_2.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_3/include/CGAL/Convex_hull_face_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_3/include/CGAL/Convex_hull_face_base_2.h $
 // $Id: Convex_hull_face_base_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_hull_projective_xy_traits_2.h 5.1-1/include/CGAL/Convex_hull_projective_xy_traits_2.h
--- 5.0.3-1/include/CGAL/Convex_hull_projective_xy_traits_2.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_hull_projective_xy_traits_2.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_2/include/CGAL/Convex_hull_projective_xy_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_2/include/CGAL/Convex_hull_projective_xy_traits_2.h $
 // $Id: Convex_hull_projective_xy_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_hull_projective_xz_traits_2.h 5.1-1/include/CGAL/Convex_hull_projective_xz_traits_2.h
--- 5.0.3-1/include/CGAL/Convex_hull_projective_xz_traits_2.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_hull_projective_xz_traits_2.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_2/include/CGAL/Convex_hull_projective_xz_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_2/include/CGAL/Convex_hull_projective_xz_traits_2.h $
 // $Id: Convex_hull_projective_xz_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_hull_projective_yz_traits_2.h 5.1-1/include/CGAL/Convex_hull_projective_yz_traits_2.h
--- 5.0.3-1/include/CGAL/Convex_hull_projective_yz_traits_2.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_hull_projective_yz_traits_2.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_2/include/CGAL/Convex_hull_projective_yz_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_2/include/CGAL/Convex_hull_projective_yz_traits_2.h $
 // $Id: Convex_hull_projective_yz_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/convex_hull_traits_2.h 5.1-1/include/CGAL/convex_hull_traits_2.h
--- 5.0.3-1/include/CGAL/convex_hull_traits_2.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/convex_hull_traits_2.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_2/include/CGAL/convex_hull_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_2/include/CGAL/convex_hull_traits_2.h $
 // $Id: convex_hull_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_hull_traits_3.h 5.1-1/include/CGAL/Convex_hull_traits_3.h
--- 5.0.3-1/include/CGAL/Convex_hull_traits_3.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_hull_traits_3.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_3/include/CGAL/Convex_hull_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_3/include/CGAL/Convex_hull_traits_3.h $
 // $Id: Convex_hull_traits_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Convex_hull_traits_adapter_2.h 5.1-1/include/CGAL/Convex_hull_traits_adapter_2.h
--- 5.0.3-1/include/CGAL/Convex_hull_traits_adapter_2.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Convex_hull_traits_adapter_2.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_2/include/CGAL/Convex_hull_traits_adapter_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_2/include/CGAL/Convex_hull_traits_adapter_2.h $
 // $Id: Convex_hull_traits_adapter_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/convexity_check_2.h 5.1-1/include/CGAL/convexity_check_2.h
--- 5.0.3-1/include/CGAL/convexity_check_2.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/convexity_check_2.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_2/include/CGAL/convexity_check_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_2/include/CGAL/convexity_check_2.h $
 // $Id: convexity_check_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/convexity_check_3.h 5.1-1/include/CGAL/convexity_check_3.h
--- 5.0.3-1/include/CGAL/convexity_check_3.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/convexity_check_3.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_3/include/CGAL/convexity_check_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_3/include/CGAL/convexity_check_3.h $
 // $Id: convexity_check_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/copy_n.h 5.1-1/include/CGAL/copy_n.h
--- 5.0.3-1/include/CGAL/copy_n.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/copy_n.h	2020-09-08 12:15:32.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/copy_n.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/copy_n.h $
 // $Id: copy_n.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/CORE/BigFloat.h 5.1-1/include/CGAL/CORE/BigFloat.h
--- 5.0.3-1/include/CGAL/CORE/BigFloat.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/CORE/BigFloat.h	2020-09-08 12:14:43.000000000 +0000
@@ -17,7 +17,7 @@
  * WWW URL: http://cs.nyu.edu/exact/
  * Email: exact@cs.nyu.edu
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_Core/include/CGAL/CORE/BigFloat.h $
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_Core/include/CGAL/CORE/BigFloat.h $
  * $Id: BigFloat.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
  * SPDX-License-Identifier: LGPL-3.0-or-later
  ***************************************************************************/
diff -pruN 5.0.3-1/include/CGAL/CORE/BigFloat_impl.h 5.1-1/include/CGAL/CORE/BigFloat_impl.h
--- 5.0.3-1/include/CGAL/CORE/BigFloat_impl.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/CORE/BigFloat_impl.h	2020-09-08 12:14:43.000000000 +0000
@@ -26,8 +26,8 @@
  * WWW URL: http://cs.nyu.edu/exact/
  * Email: exact@cs.nyu.edu
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_Core/include/CGAL/CORE/BigFloat_impl.h $
- * $Id: BigFloat_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_Core/include/CGAL/CORE/BigFloat_impl.h $
+ * $Id: BigFloat_impl.h bd172e5 2020-07-21T17:15:45+02:00 Laurent Rineau
  * SPDX-License-Identifier: LGPL-3.0-or-later
  ***************************************************************************/
 
@@ -78,12 +78,14 @@ BigInt FiveTo(unsigned long exp) {
 // ZERO
 CGAL_INLINE_FUNCTION
 const BigFloat& BigFloat::getZero() {
+  init_CORE();
   CGAL_STATIC_THREAD_LOCAL_VARIABLE(BigFloat, Zero,0);
   return Zero;
 }
 // ONE
 CGAL_INLINE_FUNCTION
 const BigFloat& BigFloat::getOne() {
+  init_CORE();
   CGAL_STATIC_THREAD_LOCAL_VARIABLE(BigFloat, One,1);
   return One;
 }
diff -pruN 5.0.3-1/include/CGAL/CORE/BigFloatRep.h 5.1-1/include/CGAL/CORE/BigFloatRep.h
--- 5.0.3-1/include/CGAL/CORE/BigFloatRep.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/CORE/BigFloatRep.h	2020-09-08 12:14:43.000000000 +0000
@@ -17,7 +17,7 @@
  * WWW URL: http://cs.nyu.edu/exact/
  * Email: exact@cs.nyu.edu
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_Core/include/CGAL/CORE/BigFloatRep.h $
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_Core/include/CGAL/CORE/BigFloatRep.h $
  * $Id: BigFloatRep.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
  * SPDX-License-Identifier: LGPL-3.0-or-later
  ***************************************************************************/
diff -pruN 5.0.3-1/include/CGAL/CORE/BigInt.h 5.1-1/include/CGAL/CORE/BigInt.h
--- 5.0.3-1/include/CGAL/CORE/BigInt.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/CORE/BigInt.h	2020-09-08 12:14:43.000000000 +0000
@@ -17,7 +17,7 @@
  * WWW URL: http://cs.nyu.edu/exact/
  * Email: exact@cs.nyu.edu
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_Core/include/CGAL/CORE/BigInt.h $
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_Core/include/CGAL/CORE/BigInt.h $
  * $Id: BigInt.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
  * SPDX-License-Identifier: LGPL-3.0-or-later
  ***************************************************************************/
diff -pruN 5.0.3-1/include/CGAL/CORE/BigRat.h 5.1-1/include/CGAL/CORE/BigRat.h
--- 5.0.3-1/include/CGAL/CORE/BigRat.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/CORE/BigRat.h	2020-09-08 12:14:43.000000000 +0000
@@ -17,7 +17,7 @@
  * WWW URL: http://cs.nyu.edu/exact/
  * Email: exact@cs.nyu.edu
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_Core/include/CGAL/CORE/BigRat.h $
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_Core/include/CGAL/CORE/BigRat.h $
  * $Id: BigRat.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
  * SPDX-License-Identifier: LGPL-3.0-or-later
  ***************************************************************************/
diff -pruN 5.0.3-1/include/CGAL/CORE/Config.h 5.1-1/include/CGAL/CORE/Config.h
--- 5.0.3-1/include/CGAL/CORE/Config.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/CORE/Config.h	2020-09-08 12:14:43.000000000 +0000
@@ -5,7 +5,7 @@
  *
  * This file is part of CGAL (www.cgal.org).
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_Core/include/CGAL/CORE/Config.h $
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_Core/include/CGAL/CORE/Config.h $
  * $Id: Config.h 26fb266 2019-10-19T16:28:33+02:00 Sébastien Loriot
  * SPDX-License-Identifier: LGPL-3.0-or-later
  ***************************************************************************/
diff -pruN 5.0.3-1/include/CGAL/CORE/CoreAux.h 5.1-1/include/CGAL/CORE/CoreAux.h
--- 5.0.3-1/include/CGAL/CORE/CoreAux.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/CORE/CoreAux.h	2020-09-08 12:14:43.000000000 +0000
@@ -17,7 +17,7 @@
  * WWW URL: http://cs.nyu.edu/exact/
  * Email: exact@cs.nyu.edu
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_Core/include/CGAL/CORE/CoreAux.h $
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_Core/include/CGAL/CORE/CoreAux.h $
  * $Id: CoreAux.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
  * SPDX-License-Identifier: LGPL-3.0-or-later
  ***************************************************************************/
diff -pruN 5.0.3-1/include/CGAL/CORE/CoreAux_impl.h 5.1-1/include/CGAL/CORE/CoreAux_impl.h
--- 5.0.3-1/include/CGAL/CORE/CoreAux_impl.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/CORE/CoreAux_impl.h	2020-09-08 12:14:43.000000000 +0000
@@ -18,7 +18,7 @@
  * WWW URL: http://cs.nyu.edu/exact/
  * Email: exact@cs.nyu.edu
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_Core/include/CGAL/CORE/CoreAux_impl.h $
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_Core/include/CGAL/CORE/CoreAux_impl.h $
  * $Id: CoreAux_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
  * SPDX-License-Identifier: LGPL-3.0-or-later
  ***************************************************************************/
diff -pruN 5.0.3-1/include/CGAL/CORE/CoreDefs.h 5.1-1/include/CGAL/CORE/CoreDefs.h
--- 5.0.3-1/include/CGAL/CORE/CoreDefs.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/CORE/CoreDefs.h	2020-09-08 12:14:43.000000000 +0000
@@ -20,7 +20,7 @@
  * WWW URL: http://cs.nyu.edu/exact/
  * Email: exact@cs.nyu.edu
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_Core/include/CGAL/CORE/CoreDefs.h $
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_Core/include/CGAL/CORE/CoreDefs.h $
  * $Id: CoreDefs.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
  * SPDX-License-Identifier: LGPL-3.0-or-later
  ***************************************************************************/
diff -pruN 5.0.3-1/include/CGAL/CORE/CoreDefs_impl.h 5.1-1/include/CGAL/CORE/CoreDefs_impl.h
--- 5.0.3-1/include/CGAL/CORE/CoreDefs_impl.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/CORE/CoreDefs_impl.h	2020-09-08 12:14:43.000000000 +0000
@@ -17,7 +17,7 @@
  * WWW URL: http://cs.nyu.edu/exact/
  * Email: exact@cs.nyu.edu
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_Core/include/CGAL/CORE/CoreDefs_impl.h $
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_Core/include/CGAL/CORE/CoreDefs_impl.h $
  * $Id: CoreDefs_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
  * SPDX-License-Identifier: LGPL-3.0-or-later
  ***************************************************************************/
diff -pruN 5.0.3-1/include/CGAL/CORE/CORE.h 5.1-1/include/CGAL/CORE/CORE.h
--- 5.0.3-1/include/CGAL/CORE/CORE.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/CORE/CORE.h	2020-09-08 12:14:43.000000000 +0000
@@ -18,7 +18,7 @@
  * WWW URL: http://cs.nyu.edu/exact/
  * Email: exact@cs.nyu.edu
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_Core/include/CGAL/CORE/CORE.h $
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_Core/include/CGAL/CORE/CORE.h $
  * $Id: CORE.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
  * SPDX-License-Identifier: LGPL-3.0-or-later
  ***************************************************************************/
diff -pruN 5.0.3-1/include/CGAL/CORE/CoreIO_impl.h 5.1-1/include/CGAL/CORE/CoreIO_impl.h
--- 5.0.3-1/include/CGAL/CORE/CoreIO_impl.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/CORE/CoreIO_impl.h	2020-09-08 12:14:43.000000000 +0000
@@ -14,7 +14,7 @@
  * WWW URL: http://cs.nyu.edu/exact/
  * Email: exact@cs.nyu.edu
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_Core/include/CGAL/CORE/CoreIO_impl.h $
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_Core/include/CGAL/CORE/CoreIO_impl.h $
  * $Id: CoreIO_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
  * SPDX-License-Identifier: LGPL-3.0-or-later
  ***************************************************************************/
diff -pruN 5.0.3-1/include/CGAL/CORE/Expr.h 5.1-1/include/CGAL/CORE/Expr.h
--- 5.0.3-1/include/CGAL/CORE/Expr.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/CORE/Expr.h	2020-09-08 12:14:43.000000000 +0000
@@ -21,7 +21,7 @@
  * WWW URL: http://cs.nyu.edu/exact/
  * Email: exact@cs.nyu.edu
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_Core/include/CGAL/CORE/Expr.h $
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_Core/include/CGAL/CORE/Expr.h $
  * $Id: Expr.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
  * SPDX-License-Identifier: LGPL-3.0-or-later
  ***************************************************************************/
diff -pruN 5.0.3-1/include/CGAL/CORE/Expr_impl.h 5.1-1/include/CGAL/CORE/Expr_impl.h
--- 5.0.3-1/include/CGAL/CORE/Expr_impl.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/CORE/Expr_impl.h	2020-09-08 12:14:43.000000000 +0000
@@ -19,8 +19,8 @@
  * WWW URL: http://cs.nyu.edu/exact/
  * Email: exact@cs.nyu.edu
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_Core/include/CGAL/CORE/Expr_impl.h $
- * $Id: Expr_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_Core/include/CGAL/CORE/Expr_impl.h $
+ * $Id: Expr_impl.h bd172e5 2020-07-21T17:15:45+02:00 Laurent Rineau
  * SPDX-License-Identifier: LGPL-3.0-or-later
  ***************************************************************************/
 
@@ -61,11 +61,13 @@ const char* Sub::name = "-";
  ********************************************************/
 CGAL_INLINE_FUNCTION
 const Expr& Expr::getZero() {
+  init_CORE();
   CGAL_STATIC_THREAD_LOCAL_VARIABLE(Expr, Zero,0);
   return Zero;
 }
 CGAL_INLINE_FUNCTION
 const Expr& Expr::getOne() {
+  init_CORE();
   CGAL_STATIC_THREAD_LOCAL_VARIABLE(Expr, One,1);
   return One;
 }
diff -pruN 5.0.3-1/include/CGAL/CORE/ExprRep.h 5.1-1/include/CGAL/CORE/ExprRep.h
--- 5.0.3-1/include/CGAL/CORE/ExprRep.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/CORE/ExprRep.h	2020-09-08 12:14:43.000000000 +0000
@@ -21,7 +21,7 @@
  * WWW URL: http://cs.nyu.edu/exact/
  * Email: exact@cs.nyu.edu
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_Core/include/CGAL/CORE/ExprRep.h $
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_Core/include/CGAL/CORE/ExprRep.h $
  * $Id: ExprRep.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
  * SPDX-License-Identifier: LGPL-3.0-or-later
  ***************************************************************************/
diff -pruN 5.0.3-1/include/CGAL/CORE/extLong.h 5.1-1/include/CGAL/CORE/extLong.h
--- 5.0.3-1/include/CGAL/CORE/extLong.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/CORE/extLong.h	2020-09-08 12:14:44.000000000 +0000
@@ -20,7 +20,7 @@
  * WWW URL: http://cs.nyu.edu/exact/
  * Email: exact@cs.nyu.edu
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_Core/include/CGAL/CORE/extLong.h $
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_Core/include/CGAL/CORE/extLong.h $
  * $Id: extLong.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
  * SPDX-License-Identifier: LGPL-3.0-or-later
  ***************************************************************************/
diff -pruN 5.0.3-1/include/CGAL/CORE/extLong_impl.h 5.1-1/include/CGAL/CORE/extLong_impl.h
--- 5.0.3-1/include/CGAL/CORE/extLong_impl.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/CORE/extLong_impl.h	2020-09-08 12:14:44.000000000 +0000
@@ -24,7 +24,7 @@
  * WWW URL: http://cs.nyu.edu/exact/
  * Email: exact@cs.nyu.edu
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_Core/include/CGAL/CORE/extLong_impl.h $
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_Core/include/CGAL/CORE/extLong_impl.h $
  * $Id: extLong_impl.h 627a7b2 2020-04-27T17:08:35+02:00 Maxime Gimeno
  * SPDX-License-Identifier: LGPL-3.0-or-later
  ***************************************************************************/
diff -pruN 5.0.3-1/include/CGAL/CORE/Filter.h 5.1-1/include/CGAL/CORE/Filter.h
--- 5.0.3-1/include/CGAL/CORE/Filter.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/CORE/Filter.h	2020-09-08 12:14:43.000000000 +0000
@@ -20,7 +20,7 @@
  * WWW URL: http://cs.nyu.edu/exact/
  * Email: exact@cs.nyu.edu
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_Core/include/CGAL/CORE/Filter.h $
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_Core/include/CGAL/CORE/Filter.h $
  * $Id: Filter.h 6c94c15 2020-05-05T14:40:55+02:00 Sébastien Loriot
  * SPDX-License-Identifier: LGPL-3.0-or-later
  ***************************************************************************/
diff -pruN 5.0.3-1/include/CGAL/CORE/Gmp.h 5.1-1/include/CGAL/CORE/Gmp.h
--- 5.0.3-1/include/CGAL/CORE/Gmp.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/CORE/Gmp.h	2020-09-08 12:14:43.000000000 +0000
@@ -5,7 +5,7 @@
  *
  * This file is part of CGAL (www.cgal.org).
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_Core/include/CGAL/CORE/Gmp.h $
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_Core/include/CGAL/CORE/Gmp.h $
  * $Id: Gmp.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
  * SPDX-License-Identifier: LGPL-3.0-or-later
  ***************************************************************************/
diff -pruN 5.0.3-1/include/CGAL/CORE/Gmp_impl.h 5.1-1/include/CGAL/CORE/Gmp_impl.h
--- 5.0.3-1/include/CGAL/CORE/Gmp_impl.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/CORE/Gmp_impl.h	2020-09-08 12:14:43.000000000 +0000
@@ -8,7 +8,7 @@
  *
  * Zilin Du, 2003
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_Core/include/CGAL/CORE/Gmp_impl.h $
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_Core/include/CGAL/CORE/Gmp_impl.h $
  * $Id: Gmp_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
  * SPDX-License-Identifier: LGPL-3.0-only
  ***************************************************************************/
diff -pruN 5.0.3-1/include/CGAL/CORE/Impl.h 5.1-1/include/CGAL/CORE/Impl.h
--- 5.0.3-1/include/CGAL/CORE/Impl.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/CORE/Impl.h	2020-09-08 12:14:43.000000000 +0000
@@ -5,7 +5,7 @@
  *
  * This file is part of CGAL (www.cgal.org).
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_Core/include/CGAL/CORE/Impl.h $
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_Core/include/CGAL/CORE/Impl.h $
  * $Id: Impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
  * SPDX-License-Identifier: LGPL-3.0-or-later
  ***************************************************************************/
diff -pruN 5.0.3-1/include/CGAL/CORE/linearAlgebra.h 5.1-1/include/CGAL/CORE/linearAlgebra.h
--- 5.0.3-1/include/CGAL/CORE/linearAlgebra.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/CORE/linearAlgebra.h	2020-09-08 12:14:44.000000000 +0000
@@ -5,7 +5,7 @@
  *
  * This file is part of CGAL (www.cgal.org).
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_Core/include/CGAL/CORE/linearAlgebra.h $
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_Core/include/CGAL/CORE/linearAlgebra.h $
  * $Id: linearAlgebra.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
  * SPDX-License-Identifier: LGPL-3.0-or-later
  ***************************************************************************/
diff -pruN 5.0.3-1/include/CGAL/CORE/MemoryPool.h 5.1-1/include/CGAL/CORE/MemoryPool.h
--- 5.0.3-1/include/CGAL/CORE/MemoryPool.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/CORE/MemoryPool.h	2020-09-08 12:14:43.000000000 +0000
@@ -17,8 +17,8 @@
  * WWW URL: http://cs.nyu.edu/exact/
  * Email: exact@cs.nyu.edu
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_Core/include/CGAL/CORE/MemoryPool.h $
- * $Id: MemoryPool.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_Core/include/CGAL/CORE/MemoryPool.h $
+ * $Id: MemoryPool.h 8a7d3fa 2020-07-21T17:32:30+02:00 Laurent Rineau
  * SPDX-License-Identifier: LGPL-3.0-or-later
  ***************************************************************************/
 #ifndef _CORE_MEMORYPOOL_H_
@@ -26,14 +26,6 @@
 
 #include <CGAL/config.h>
 #include <CGAL/tss.h>
-#include <boost/config.hpp>
-#if defined(CGAL_HAS_THREADS) && defined(BOOST_GCC) && BOOST_GCC < 90100
-// Force the use of Boost.Thread with g++ and C++11, because of the PR66944
-//   https://gcc.gnu.org/bugzilla/show_bug.cgi?id=66944
-// See also CGAL PR #1888
-//   https://github.com/CGAL/cgal/pull/1888#issuecomment-278284232
-#  include <boost/thread/tss.hpp>
-#endif
 
 #include <new>           // for placement new
 #include <cassert>
@@ -85,10 +77,7 @@ public:
 
   // Access the corresponding static global allocator.
   static MemoryPool<T,nObjects>& global_allocator() {
-#if defined(CGAL_HAS_THREADS) && defined(BOOST_GCC) && BOOST_GCC < 90100
-    if(memPool_ptr.get() == nullptr) {memPool_ptr.reset(new Self());}
-    Self& memPool =  * memPool_ptr.get();
-#elif defined(CGAL_HAS_THREADS) // use the C++11 implementation
+#if defined(CGAL_HAS_THREADS) // use the C++11 implementation
     static thread_local Self memPool;
 #else // not CGAL_HAS_THREADS
     static Self memPool;
@@ -99,18 +88,8 @@ public:
 private:
    Thunk* head; // next available block in the pool
   std::vector<void*> blocks;
-
-#if defined(CGAL_HAS_THREADS) && defined(BOOST_GCC) && BOOST_GCC < 90100
-  static boost::thread_specific_ptr<Self> memPool_ptr;
-#endif // not CGAL_HAS_THREADS
 };
 
-#if defined(CGAL_HAS_THREADS) && defined(BOOST_GCC) && BOOST_GCC < 90100
-template <class T, int nObjects >
-boost::thread_specific_ptr<MemoryPool<T, nObjects> >
-MemoryPool<T, nObjects>::memPool_ptr;
-#endif
-
 template< class T, int nObjects >
 void* MemoryPool< T, nObjects >::allocate(std::size_t) {
    if ( head == 0 ) { // if no more memory in the pool
diff -pruN 5.0.3-1/include/CGAL/CORE/poly/Curves.h 5.1-1/include/CGAL/CORE/poly/Curves.h
--- 5.0.3-1/include/CGAL/CORE/poly/Curves.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/CORE/poly/Curves.h	2020-09-08 12:14:44.000000000 +0000
@@ -52,7 +52,7 @@
  * WWW URL: http://cs.nyu.edu/exact/
  * Email: exact@cs.nyu.edu
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_Core/include/CGAL/CORE/poly/Curves.h $
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_Core/include/CGAL/CORE/poly/Curves.h $
  * $Id: Curves.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
  * SPDX-License-Identifier: LGPL-3.0-or-later
  ***************************************************************************/
diff -pruN 5.0.3-1/include/CGAL/CORE/poly/Poly.h 5.1-1/include/CGAL/CORE/poly/Poly.h
--- 5.0.3-1/include/CGAL/CORE/poly/Poly.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/CORE/poly/Poly.h	2020-09-08 12:14:44.000000000 +0000
@@ -39,8 +39,8 @@
  * WWW URL: http://cs.nyu.edu/exact/
  * Email: exact@cs.nyu.edu
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_Core/include/CGAL/CORE/poly/Poly.h $
- * $Id: Poly.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_Core/include/CGAL/CORE/poly/Poly.h $
+ * $Id: Poly.h bd172e5 2020-07-21T17:15:45+02:00 Laurent Rineau
  * SPDX-License-Identifier: LGPL-3.0-or-later
  ***************************************************************************/
 
@@ -252,6 +252,7 @@ public:
 template < class NT >
 CORE_INLINE
 const Polynomial<NT> & Polynomial<NT>::polyZero() {
+  init_CORE();
   CGAL_STATIC_THREAD_LOCAL_VARIABLE_0(Polynomial<NT>, zeroP);
   return zeroP;
 }
@@ -259,6 +260,7 @@ const Polynomial<NT> & Polynomial<NT>::p
 template < class NT >
 CORE_INLINE
 const Polynomial<NT> & Polynomial<NT>::polyUnity() {
+  init_CORE();
   static const NT c[] = {1};
   CGAL_STATIC_THREAD_LOCAL_VARIABLE_2(Polynomial<NT>, unityP, 0, c);
   return unityP;
diff -pruN 5.0.3-1/include/CGAL/CORE/poly/Poly.tcc 5.1-1/include/CGAL/CORE/poly/Poly.tcc
--- 5.0.3-1/include/CGAL/CORE/poly/Poly.tcc	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/include/CGAL/CORE/poly/Poly.tcc	2020-09-08 12:12:19.000000000 +0000
@@ -670,19 +670,20 @@ Polynomial<NT> Polynomial<NT>::pseudoRem
   contract();         // Let A = (*this).  Contract A.
   Polynomial<NT> tmpB(B);
   tmpB.contract();    // local copy of B
+  int bTrueDegree = tmpB.degree;
   C = NT(1);  // Initialized to C=1.
-  if (B.degree == -1)  {
+  if (bTrueDegree == -1)  {
     core_error("ERROR in Polynomial<NT>::pseudoRemainder :\n    -- divide by zero polynomial", __FILE__, __LINE__, false);
     return Polynomial(0);  // Unit Polynomial (arbitrary!)
   }
-  if (B.degree > degree) {
+  if (bTrueDegree > degree) {
     return Polynomial(); // Zero Polynomial
     // CHECK: 1*THIS = 0*B + THAT,  deg(THAT) < deg(B)
   }
 
   Polynomial<NT> Quo;  // accumulate the return polynomial, Quo
   Polynomial<NT> tmpQuo;
-  while (degree >= B.degree) {  // INVARIANT: C*A = B*Quo + (*this)
+  while (degree >= bTrueDegree) {  // INVARIANT: C*A = B*Quo + (*this)
     tmpQuo = reduceStep(tmpB);  // Let (*this) be (*oldthis), which
                                 // is transformed into (*newthis). Then,
                                 //     c*(*oldthis) = B*m + (*newthis)
diff -pruN 5.0.3-1/include/CGAL/CORE/poly/Sturm.h 5.1-1/include/CGAL/CORE/poly/Sturm.h
--- 5.0.3-1/include/CGAL/CORE/poly/Sturm.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/CORE/poly/Sturm.h	2020-09-08 12:14:44.000000000 +0000
@@ -40,7 +40,7 @@
  * WWW URL: http://cs.nyu.edu/exact/
  * Email: exact@cs.nyu.edu
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_Core/include/CGAL/CORE/poly/Sturm.h $
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_Core/include/CGAL/CORE/poly/Sturm.h $
  * $Id: Sturm.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
  * SPDX-License-Identifier: LGPL-3.0-or-later
  ***************************************************************************/
diff -pruN 5.0.3-1/include/CGAL/CORE/Promote.h 5.1-1/include/CGAL/CORE/Promote.h
--- 5.0.3-1/include/CGAL/CORE/Promote.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/CORE/Promote.h	2020-09-08 12:14:43.000000000 +0000
@@ -21,7 +21,7 @@
  * WWW URL: http://cs.nyu.edu/exact/
  * Email: exact@cs.nyu.edu
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_Core/include/CGAL/CORE/Promote.h $
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_Core/include/CGAL/CORE/Promote.h $
  * $Id: Promote.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
  * SPDX-License-Identifier: LGPL-3.0-or-later
  ***************************************************************************/
diff -pruN 5.0.3-1/include/CGAL/CORE/Real.h 5.1-1/include/CGAL/CORE/Real.h
--- 5.0.3-1/include/CGAL/CORE/Real.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/CORE/Real.h	2020-09-08 12:14:43.000000000 +0000
@@ -21,13 +21,15 @@
  * WWW URL: http://cs.nyu.edu/exact/
  * Email: exact@cs.nyu.edu
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_Core/include/CGAL/CORE/Real.h $
- * $Id: Real.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_Core/include/CGAL/CORE/Real.h $
+ * $Id: Real.h 10e99cb 2020-07-22T09:51:07+02:00 Laurent Rineau
  * SPDX-License-Identifier: LGPL-3.0-or-later
  ***************************************************************************/
 #ifndef _CORE_REAL_H_
 #define _CORE_REAL_H_
+
 #include "RealRep.h"
+#include <CGAL/use.h>
 
 namespace CORE {
 // class Real
@@ -483,6 +485,12 @@ inline Real RealLong::operator-() const
   return ker < -LONG_MAX ? -BigInt(ker) : -ker;
 }
 
+inline void init_CORE() {
+  using RealRep = CORE::RealDouble;
+  CGAL_STATIC_THREAD_LOCAL_VARIABLE(MemoryPool<RealRep>*, pool_real_rep, &MemoryPool<RealRep>::global_allocator());
+  CGAL_USE(pool_real_rep);
+}
+
 } //namespace CORE
 
 #ifdef CGAL_HEADER_ONLY
diff -pruN 5.0.3-1/include/CGAL/CORE/Real_impl.h 5.1-1/include/CGAL/CORE/Real_impl.h
--- 5.0.3-1/include/CGAL/CORE/Real_impl.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/CORE/Real_impl.h	2020-09-08 12:14:44.000000000 +0000
@@ -20,8 +20,8 @@
  * WWW URL: http://cs.nyu.edu/exact/
  * Email: exact@cs.nyu.edu
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_Core/include/CGAL/CORE/Real_impl.h $
- * $Id: Real_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_Core/include/CGAL/CORE/Real_impl.h $
+ * $Id: Real_impl.h bd172e5 2020-07-21T17:15:45+02:00 Laurent Rineau
  * SPDX-License-Identifier: LGPL-3.0-or-later
  ***************************************************************************/
 
@@ -44,6 +44,7 @@ namespace CORE {
 
 CGAL_INLINE_FUNCTION
 const Real& Real::getZero() {
+  init_CORE();
   CGAL_STATIC_THREAD_LOCAL_VARIABLE(Real, Zero, 0);
   return Zero;
 }
diff -pruN 5.0.3-1/include/CGAL/CORE/RealRep.h 5.1-1/include/CGAL/CORE/RealRep.h
--- 5.0.3-1/include/CGAL/CORE/RealRep.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/CORE/RealRep.h	2020-09-08 12:14:43.000000000 +0000
@@ -19,7 +19,7 @@
  * WWW URL: http://cs.nyu.edu/exact/
  * Email: exact@cs.nyu.edu
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_Core/include/CGAL/CORE/RealRep.h $
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_Core/include/CGAL/CORE/RealRep.h $
  * $Id: RealRep.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
  * SPDX-License-Identifier: LGPL-3.0-or-later
  ***************************************************************************/
diff -pruN 5.0.3-1/include/CGAL/CORE/RefCount.h 5.1-1/include/CGAL/CORE/RefCount.h
--- 5.0.3-1/include/CGAL/CORE/RefCount.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/CORE/RefCount.h	2020-09-08 12:14:44.000000000 +0000
@@ -38,7 +38,7 @@
  * WWW URL: http://cs.nyu.edu/exact/
  * Email: exact@cs.nyu.edu
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_Core/include/CGAL/CORE/RefCount.h $
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_Core/include/CGAL/CORE/RefCount.h $
  * $Id: RefCount.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
  * SPDX-License-Identifier: LGPL-3.0-or-later
  ***************************************************************************/
diff -pruN 5.0.3-1/include/CGAL/CORE/Timer.h 5.1-1/include/CGAL/CORE/Timer.h
--- 5.0.3-1/include/CGAL/CORE/Timer.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/CORE/Timer.h	2020-09-08 12:14:44.000000000 +0000
@@ -26,7 +26,7 @@
  * WWW URL: http://cs.nyu.edu/exact/
  * Email: exact@cs.nyu.edu
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_Core/include/CGAL/CORE/Timer.h $
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_Core/include/CGAL/CORE/Timer.h $
  * $Id: Timer.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
  * SPDX-License-Identifier: LGPL-3.0-or-later
  ***************************************************************************/
diff -pruN 5.0.3-1/include/CGAL/CORE_algebraic_number_traits.h 5.1-1/include/CGAL/CORE_algebraic_number_traits.h
--- 5.0.3-1/include/CGAL/CORE_algebraic_number_traits.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/CORE_algebraic_number_traits.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/CORE_algebraic_number_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/CORE_algebraic_number_traits.h $
 // $Id: CORE_algebraic_number_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/CORE_arithmetic_kernel.h 5.1-1/include/CGAL/CORE_arithmetic_kernel.h
--- 5.0.3-1/include/CGAL/CORE_arithmetic_kernel.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/CORE_arithmetic_kernel.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arithmetic_kernel/include/CGAL/CORE_arithmetic_kernel.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arithmetic_kernel/include/CGAL/CORE_arithmetic_kernel.h $
 // $Id: CORE_arithmetic_kernel.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/CORE_BigFloat.h 5.1-1/include/CGAL/CORE_BigFloat.h
--- 5.0.3-1/include/CGAL/CORE_BigFloat.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/CORE_BigFloat.h	2020-09-08 12:15:16.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/CORE_BigFloat.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/CORE_BigFloat.h $
 // $Id: CORE_BigFloat.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/CORE_BigInt.h 5.1-1/include/CGAL/CORE_BigInt.h
--- 5.0.3-1/include/CGAL/CORE_BigInt.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/CORE_BigInt.h	2020-09-08 12:15:16.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/CORE_BigInt.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/CORE_BigInt.h $
 // $Id: CORE_BigInt.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/CORE_BigRat.h 5.1-1/include/CGAL/CORE_BigRat.h
--- 5.0.3-1/include/CGAL/CORE_BigRat.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/CORE_BigRat.h	2020-09-08 12:15:16.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/CORE_BigRat.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/CORE_BigRat.h $
 // $Id: CORE_BigRat.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/CORE_coercion_traits.h 5.1-1/include/CGAL/CORE_coercion_traits.h
--- 5.0.3-1/include/CGAL/CORE_coercion_traits.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/CORE_coercion_traits.h	2020-09-08 12:15:16.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/CORE_coercion_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/CORE_coercion_traits.h $
 // $Id: CORE_coercion_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/CORE_Expr.h 5.1-1/include/CGAL/CORE_Expr.h
--- 5.0.3-1/include/CGAL/CORE_Expr.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/CORE_Expr.h	2020-09-08 12:15:16.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/CORE_Expr.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/CORE_Expr.h $
 // $Id: CORE_Expr.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Counted_number.h 5.1-1/include/CGAL/Counted_number.h
--- 5.0.3-1/include/CGAL/Counted_number.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Counted_number.h	2020-09-08 12:15:16.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Counted_number.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Counted_number.h $
 // $Id: Counted_number.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Counting_iterator.h 5.1-1/include/CGAL/Counting_iterator.h
--- 5.0.3-1/include/CGAL/Counting_iterator.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Counting_iterator.h	2020-09-08 12:15:32.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Counting_iterator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Counting_iterator.h $
 // $Id: Counting_iterator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/create_offset_polygons_2.h 5.1-1/include/CGAL/create_offset_polygons_2.h
--- 5.0.3-1/include/CGAL/create_offset_polygons_2.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/create_offset_polygons_2.h	2020-09-08 12:15:41.000000000 +0000
@@ -3,7 +3,7 @@
 // This file is part of CGAL (www.cgal.org).
 //
 
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Straight_skeleton_2/include/CGAL/create_offset_polygons_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Straight_skeleton_2/include/CGAL/create_offset_polygons_2.h $
 // $Id: create_offset_polygons_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/create_offset_polygons_from_polygon_with_holes_2.h 5.1-1/include/CGAL/create_offset_polygons_from_polygon_with_holes_2.h
--- 5.0.3-1/include/CGAL/create_offset_polygons_from_polygon_with_holes_2.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/create_offset_polygons_from_polygon_with_holes_2.h	2020-09-08 12:15:41.000000000 +0000
@@ -3,7 +3,7 @@
 // This file is part of CGAL (www.cgal.org).
 //
 
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Straight_skeleton_2/include/CGAL/create_offset_polygons_from_polygon_with_holes_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Straight_skeleton_2/include/CGAL/create_offset_polygons_from_polygon_with_holes_2.h $
 // $Id: create_offset_polygons_from_polygon_with_holes_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/create_straight_skeleton_2.h 5.1-1/include/CGAL/create_straight_skeleton_2.h
--- 5.0.3-1/include/CGAL/create_straight_skeleton_2.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/create_straight_skeleton_2.h	2020-09-08 12:15:41.000000000 +0000
@@ -3,7 +3,7 @@
 // This file is part of CGAL (www.cgal.org).
 //
 
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Straight_skeleton_2/include/CGAL/create_straight_skeleton_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Straight_skeleton_2/include/CGAL/create_straight_skeleton_2.h $
 // $Id: create_straight_skeleton_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/create_straight_skeleton_from_polygon_with_holes_2.h 5.1-1/include/CGAL/create_straight_skeleton_from_polygon_with_holes_2.h
--- 5.0.3-1/include/CGAL/create_straight_skeleton_from_polygon_with_holes_2.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/create_straight_skeleton_from_polygon_with_holes_2.h	2020-09-08 12:15:41.000000000 +0000
@@ -3,7 +3,7 @@
 // This file is part of CGAL (www.cgal.org).
 //
 
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Straight_skeleton_2/include/CGAL/create_straight_skeleton_from_polygon_with_holes_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Straight_skeleton_2/include/CGAL/create_straight_skeleton_from_polygon_with_holes_2.h $
 // $Id: create_straight_skeleton_from_polygon_with_holes_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/Arc_2.h 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/Arc_2.h
--- 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/Arc_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/Arc_2.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/Arc_2.h $
-// $Id: Arc_2.h 5b2d46c 2020-04-21T10:03:00+02:00 Maxime Gimeno
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/Arc_2.h $
+// $Id: Arc_2.h e9d41d7 2020-04-21T10:03:00+02:00 Maxime Gimeno
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Eric Berberich <eric@mpi-inf.mpg.de>
diff -pruN 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/Curved_kernel_via_analysis_2_functors.h 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/Curved_kernel_via_analysis_2_functors.h
--- 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/Curved_kernel_via_analysis_2_functors.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/Curved_kernel_via_analysis_2_functors.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/Curved_kernel_via_analysis_2_functors.h $
-// $Id: Curved_kernel_via_analysis_2_functors.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/Curved_kernel_via_analysis_2_functors.h $
+// $Id: Curved_kernel_via_analysis_2_functors.h 58276ed 2020-03-31T18:34:28+03:00 Efi Fogel
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -1459,29 +1459,24 @@ public:
     OutputIterator operator()(const Arc_2& cv1, const Arc_2& cv2,
                               OutputIterator oi) const {
 
+      typedef unsigned int                              Multiplicity;
+      typedef std::pair<Point_2, Multiplicity>          Intersection_point;
+      typedef boost::variant<Intersection_point, Arc_2> Intersection_result;
+
         CERR("\nintersect; cv1: " << cv1
              << ";\n cv2:" << cv2 << "");
 
         // if arcs overlap, just store their common part, otherwise compute
         // point-wise intersections
-        std::vector< Arc_2 > common_arcs;
-        if (cv1._trim_if_overlapped(cv2, std::back_inserter(common_arcs))) {
-            typename std::vector< Arc_2 >::const_iterator it;
-            for(it = common_arcs.begin(); it < common_arcs.end(); it++) {
-                *oi++ = CGAL::make_object(*it);
-            }
+        std::vector<Arc_2> arcs;
+        if (cv1._trim_if_overlapped(cv2, std::back_inserter(arcs))) {
+            for (const auto& item : arcs) *oi++ = Intersection_result(item);
             return oi;
         }
         // process non-ov erlapping case
-        typedef std::pair< Point_2, unsigned int > Point_and_mult;
-        typedef std::vector< Point_and_mult > Point_vector;
-        Point_vector vec;
-        typename Point_vector::const_iterator it;
+        std::vector<Intersection_point> vec;
         Arc_2::_intersection_points(cv1, cv2, std::back_inserter(vec));
-
-        for (it = vec.begin(); it != vec.end(); it++) {
-            *oi++ = CGAL::make_object(*it);
-        }
+        for (const auto& item : vec) *oi++ = Intersection_result(item);
         return oi;
     }
 
diff -pruN 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/Curved_kernel_via_analysis_2_impl.h 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/Curved_kernel_via_analysis_2_impl.h
--- 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/Curved_kernel_via_analysis_2_impl.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/Curved_kernel_via_analysis_2_impl.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/Curved_kernel_via_analysis_2_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/Curved_kernel_via_analysis_2_impl.h $
 // $Id: Curved_kernel_via_analysis_2_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/Curve_interval_arcno_cache.h 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/Curve_interval_arcno_cache.h
--- 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/Curve_interval_arcno_cache.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/Curve_interval_arcno_cache.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/Curve_interval_arcno_cache.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/Curve_interval_arcno_cache.h $
 // $Id: Curve_interval_arcno_cache.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/Curve_renderer_facade.h 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/Curve_renderer_facade.h
--- 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/Curve_renderer_facade.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/Curve_renderer_facade.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/Curve_renderer_facade.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/Curve_renderer_facade.h $
 // $Id: Curve_renderer_facade.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/Fig_stream_Curve_renderer_2.h 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/Fig_stream_Curve_renderer_2.h
--- 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/Fig_stream_Curve_renderer_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/Fig_stream_Curve_renderer_2.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/Fig_stream_Curve_renderer_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/Fig_stream_Curve_renderer_2.h $
 // $Id: Fig_stream_Curve_renderer_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/Filtered_curved_kernel_via_analysis_2_impl.h 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/Filtered_curved_kernel_via_analysis_2_impl.h
--- 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/Filtered_curved_kernel_via_analysis_2_impl.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/Filtered_curved_kernel_via_analysis_2_impl.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/Filtered_curved_kernel_via_analysis_2_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/Filtered_curved_kernel_via_analysis_2_impl.h $
 // $Id: Filtered_curved_kernel_via_analysis_2_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/Generic_arc_2.h 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/Generic_arc_2.h
--- 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/Generic_arc_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/Generic_arc_2.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/Generic_arc_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/Generic_arc_2.h $
 // $Id: Generic_arc_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/Generic_point_2.h 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/Generic_point_2.h
--- 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/Generic_point_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/Generic_point_2.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/Generic_point_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/Generic_point_2.h $
 // $Id: Generic_point_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/gfx/Curve_renderer_2.h 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/gfx/Curve_renderer_2.h
--- 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/gfx/Curve_renderer_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/gfx/Curve_renderer_2.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/gfx/Curve_renderer_2.h $
-// $Id: Curve_renderer_2.h 5b2d46c 2020-04-21T10:03:00+02:00 Maxime Gimeno
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/gfx/Curve_renderer_2.h $
+// $Id: Curve_renderer_2.h e9d41d7 2020-04-21T10:03:00+02:00 Maxime Gimeno
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Pavel Emeliyanenko <asm@mpi-sb.mpg.de>
diff -pruN 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/gfx/Curve_renderer_internals.h 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/gfx/Curve_renderer_internals.h
--- 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/gfx/Curve_renderer_internals.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/gfx/Curve_renderer_internals.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/gfx/Curve_renderer_internals.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/gfx/Curve_renderer_internals.h $
 // $Id: Curve_renderer_internals.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/gfx/Curve_renderer_traits.h 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/gfx/Curve_renderer_traits.h
--- 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/gfx/Curve_renderer_traits.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/gfx/Curve_renderer_traits.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/gfx/Curve_renderer_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/gfx/Curve_renderer_traits.h $
 // $Id: Curve_renderer_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/gfx/Subdivision_1.h 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/gfx/Subdivision_1.h
--- 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/gfx/Subdivision_1.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/gfx/Subdivision_1.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/gfx/Subdivision_1.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/gfx/Subdivision_1.h $
 // $Id: Subdivision_1.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/gfx/Subdivision_2.h 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/gfx/Subdivision_2.h
--- 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/gfx/Subdivision_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/gfx/Subdivision_2.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/gfx/Subdivision_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/gfx/Subdivision_2.h $
 // $Id: Subdivision_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/Make_x_monotone_2.h 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/Make_x_monotone_2.h
--- 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/Make_x_monotone_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/Make_x_monotone_2.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/Make_x_monotone_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/Make_x_monotone_2.h $
 // $Id: Make_x_monotone_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/Non_x_monotone_arc_2.h 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/Non_x_monotone_arc_2.h
--- 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/Non_x_monotone_arc_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/Non_x_monotone_arc_2.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/Non_x_monotone_arc_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/Non_x_monotone_arc_2.h $
 // $Id: Non_x_monotone_arc_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/Point_2.h 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/Point_2.h
--- 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/Point_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/Point_2.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/Point_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/Point_2.h $
 // $Id: Point_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/Sweep_curves_adapter_2.h 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/Sweep_curves_adapter_2.h
--- 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/Sweep_curves_adapter_2.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/Sweep_curves_adapter_2.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/Sweep_curves_adapter_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/Sweep_curves_adapter_2.h $
 // $Id: Sweep_curves_adapter_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/test/simple_models.h 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/test/simple_models.h
--- 5.0.3-1/include/CGAL/Curved_kernel_via_analysis_2/test/simple_models.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/Curved_kernel_via_analysis_2/test/simple_models.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/test/simple_models.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/test/simple_models.h $
 // $Id: simple_models.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Curves_on_surface_topology.h 5.1-1/include/CGAL/Curves_on_surface_topology.h
--- 5.0.3-1/include/CGAL/Curves_on_surface_topology.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Curves_on_surface_topology.h	2020-09-08 12:15:45.000000000 +0000
@@ -0,0 +1,180 @@
+// Copyright (c) 2019 CNRS and LIRIS' Establishments (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_topology/include/CGAL/Curves_on_surface_topology.h $
+// $Id: Curves_on_surface_topology.h 5e3d85a 2020-04-20T14:14:29+02:00 Guillaume Damiand
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
+//
+#ifndef CGAL_CURVES_ON_SURFACE_TOPOLOGY_H
+#define CGAL_CURVES_ON_SURFACE_TOPOLOGY_H 1
+
+#include <CGAL/license/Surface_mesh_topology.h>
+
+#include <CGAL/Surface_mesh_topology/internal/Minimal_quadrangulation.h>
+#include <CGAL/Surface_mesh_topology/internal/Shortest_noncontractible_cycle.h>
+#include <CGAL/Surface_mesh_topology/internal/Facewidth.h>
+#include <CGAL/Face_graph_wrapper.h>
+#include <memory>
+
+namespace CGAL {
+namespace Surface_mesh_topology {
+
+template<typename Mesh>
+class Curves_on_surface_topology
+{
+public:
+  // Types for minimal quadrangulation
+  using Minimal_quadrangulation=internal::Minimal_quadrangulation<Mesh>;
+  using Reduced_map            =typename Minimal_quadrangulation::Local_map;
+
+  // Types for shortest noncontractible cycle
+  using Shortest_noncontractible_cycle=typename internal::Shortest_noncontractible_cycle<Mesh, true>;
+  using Facewidth                     =typename internal::Facewidth<Mesh>;
+  using Dart_const_handle             =typename Shortest_noncontractible_cycle::Original_dart_const_handle;
+  using halfedge_descriptor           =Dart_const_handle ; // To be compatible with BGL
+
+  // Constructor
+  Curves_on_surface_topology(const Mesh& amesh, bool /* display_time */=false) :
+    m_original_mesh(amesh),
+    m_minimal_quadrangulation(nullptr),
+    m_shortest_noncontractible_cycle(nullptr),
+    m_facewidth(nullptr)
+  {}
+
+//================================================================================
+// Homotopy test
+
+  /// @return true iff the minimal quadrangulation is computed.
+  bool is_minimal_quadrangulation_computed() const
+  { return m_minimal_quadrangulation!=nullptr; }
+
+  /// Computes the minimal quadrangulation if it is not yet computed.
+  void compute_minimal_quadrangulation(bool display_time=true) const
+  {
+    if (m_minimal_quadrangulation==nullptr)
+    {
+      m_minimal_quadrangulation=std::make_unique<Minimal_quadrangulation>
+        (m_original_mesh, display_time);
+    }
+  }
+
+  /// Return the reduced map computed in the minimal quadrangulation.
+  /// @pre is_minimal_quadrangulation_computed()
+  const Reduced_map& get_minimal_quadrangulation() const
+  {
+    CGAL_assertion(is_minimal_quadrangulation_computed());
+    return m_minimal_quadrangulation->get_local_map();
+  }
+
+  /// @return true iff 'p' is contractible.
+  bool is_contractible(const Path_on_surface<Mesh>& p,
+                       bool display_time=false) const
+  {
+    compute_minimal_quadrangulation(display_time);
+    return m_minimal_quadrangulation->is_contractible(p, display_time);
+  }
+
+  /// @return true iff 'p1' and 'p2' are freely homotopic.
+  bool are_freely_homotopic(const Path_on_surface<Mesh>& p1,
+                            const Path_on_surface<Mesh>& p2,
+                            bool display_time=false) const
+  {
+    compute_minimal_quadrangulation(display_time);
+    return m_minimal_quadrangulation->are_freely_homotopic(p1, p2,
+                                                           display_time);
+  }
+
+  /// @return true iff 'p1' and 'p2' are base point freely homotopic.
+  bool are_base_point_homotopic(const Path_on_surface<Mesh>& p1,
+                                const Path_on_surface<Mesh>& p2,
+                                bool display_time=false) const
+  {
+    compute_minimal_quadrangulation(display_time);
+    return m_minimal_quadrangulation->are_base_point_homotopic(p1, p2,
+                                                               display_time);
+  }
+
+//================================================================================
+// Shortest noncontractible cycle; edge and face width
+
+  bool is_shortest_noncontractible_cycle_representation_computed() const
+  { return m_shortest_noncontractible_cycle!=nullptr; }
+
+  void compute_shortest_non_contractible_cycle_representation(bool display_time=false) const
+  {
+    if (m_shortest_noncontractible_cycle==nullptr)
+    {
+      m_shortest_noncontractible_cycle=std::make_unique
+        <Shortest_noncontractible_cycle>(m_original_mesh, display_time);
+    }
+  }
+
+  template <class WeightFunctor>
+  Path_on_surface<Mesh> compute_shortest_non_contractible_cycle_with_base_point
+  (Dart_const_handle dh, const WeightFunctor& wf, bool display_time=false) const
+  {
+    compute_shortest_non_contractible_cycle_representation(display_time);
+    return m_shortest_noncontractible_cycle->compute_cycle(dh, NULL, wf, display_time);
+  }
+
+  Path_on_surface<Mesh> compute_shortest_non_contractible_cycle_with_base_point
+  (Dart_const_handle dh, bool display_time=false) const
+  {
+    compute_shortest_non_contractible_cycle_representation(display_time);
+    return m_shortest_noncontractible_cycle->compute_cycle(dh, display_time);
+  }
+
+  Path_on_surface<Mesh> compute_shortest_non_contractible_cycle
+  (bool display_time=false) const
+  {
+    compute_shortest_non_contractible_cycle_representation(display_time);
+    return m_shortest_noncontractible_cycle->
+      compute_shortest_non_contractible_cycle(nullptr, display_time);
+  }
+
+  template <class WeightFunctor>
+  Path_on_surface<Mesh> compute_shortest_non_contractible_cycle
+  (const WeightFunctor& wf, bool display_time=false) const
+  {
+    compute_shortest_non_contractible_cycle_representation(display_time);
+    return m_shortest_noncontractible_cycle->
+      compute_shortest_non_contractible_cycle(nullptr, wf, display_time);
+  }
+
+  Path_on_surface<Mesh> compute_edge_width(bool display_time=false) const
+  {
+    compute_shortest_non_contractible_cycle_representation(display_time);
+    return m_shortest_noncontractible_cycle->compute_edge_width(display_time);
+  }
+
+  bool is_facewidth_representation_computed() const
+  { return m_facewidth!=nullptr; }
+
+  void compute_face_width_representation(bool display_time=false) const
+  {
+    if (m_facewidth==nullptr)
+    { m_facewidth=std::make_unique<Facewidth>(m_original_mesh, display_time); }
+  }
+
+  std::vector<Dart_const_handle> compute_face_width(bool display_time=false) const
+  {
+    compute_face_width_representation(display_time);
+    return m_facewidth->compute_face_width(display_time);
+  }
+
+protected:
+  const Mesh&                                             m_original_mesh;
+  mutable std::unique_ptr<Minimal_quadrangulation>        m_minimal_quadrangulation;
+  mutable std::unique_ptr<Shortest_noncontractible_cycle> m_shortest_noncontractible_cycle;
+  mutable std::unique_ptr<Facewidth>                      m_facewidth;
+};
+
+} // namespace Surface_mesh_topology
+} // namespace CGAL
+
+#endif // CGAL_CURVES_ON_SURFACE_TOPOLOGY_H //
+// EOF //
diff -pruN 5.0.3-1/include/CGAL/Dart_const_iterators.h 5.1-1/include/CGAL/Dart_const_iterators.h
--- 5.0.3-1/include/CGAL/Dart_const_iterators.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Dart_const_iterators.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Combinatorial_map/include/CGAL/Dart_const_iterators.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Combinatorial_map/include/CGAL/Dart_const_iterators.h $
 // $Id: Dart_const_iterators.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Dart.h 5.1-1/include/CGAL/Dart.h
--- 5.0.3-1/include/CGAL/Dart.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Dart.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Combinatorial_map/include/CGAL/Dart.h $
-// $Id: Dart.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Combinatorial_map/include/CGAL/Dart.h $
+// $Id: Dart.h d1a323c 2020-03-26T19:24:14+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
@@ -105,8 +105,8 @@ namespace CGAL {
 
     void * for_compact_container() const
     { return mf[0].for_compact_container(); }
-    void * & for_compact_container()
-    { return mf[0].for_compact_container(); }
+    void for_compact_container(void *p)
+    { mf[0].for_compact_container(p); }
 
     Dart_handle get_f(unsigned int i) const
     {
diff -pruN 5.0.3-1/include/CGAL/Dart_iterators.h 5.1-1/include/CGAL/Dart_iterators.h
--- 5.0.3-1/include/CGAL/Dart_iterators.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Dart_iterators.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Combinatorial_map/include/CGAL/Dart_iterators.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Combinatorial_map/include/CGAL/Dart_iterators.h $
 // $Id: Dart_iterators.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Default_diagonalize_traits.h 5.1-1/include/CGAL/Default_diagonalize_traits.h
--- 5.0.3-1/include/CGAL/Default_diagonalize_traits.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Default_diagonalize_traits.h	2020-09-08 12:15:39.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Solver_interface/include/CGAL/Default_diagonalize_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Solver_interface/include/CGAL/Default_diagonalize_traits.h $
 // $Id: Default_diagonalize_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Default.h 5.1-1/include/CGAL/Default.h
--- 5.0.3-1/include/CGAL/Default.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Default.h	2020-09-08 12:15:32.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Default.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Default.h $
 // $Id: Default.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Deformation_Eigen_closest_rotation_traits_3.h 5.1-1/include/CGAL/Deformation_Eigen_closest_rotation_traits_3.h
--- 5.0.3-1/include/CGAL/Deformation_Eigen_closest_rotation_traits_3.h	2020-08-01 19:02:54.000000000 +0000
+++ 5.1-1/include/CGAL/Deformation_Eigen_closest_rotation_traits_3.h	2020-09-08 12:15:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_deformation/include/CGAL/Deformation_Eigen_closest_rotation_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_deformation/include/CGAL/Deformation_Eigen_closest_rotation_traits_3.h $
 // $Id: Deformation_Eigen_closest_rotation_traits_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Deformation_Eigen_polar_closest_rotation_traits_3.h 5.1-1/include/CGAL/Deformation_Eigen_polar_closest_rotation_traits_3.h
--- 5.0.3-1/include/CGAL/Deformation_Eigen_polar_closest_rotation_traits_3.h	2020-08-01 19:02:54.000000000 +0000
+++ 5.1-1/include/CGAL/Deformation_Eigen_polar_closest_rotation_traits_3.h	2020-09-08 12:15:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_deformation/include/CGAL/Deformation_Eigen_polar_closest_rotation_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_deformation/include/CGAL/Deformation_Eigen_polar_closest_rotation_traits_3.h $
 // $Id: Deformation_Eigen_polar_closest_rotation_traits_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Delaunay_d.h 5.1-1/include/CGAL/Delaunay_d.h
--- 5.0.3-1/include/CGAL/Delaunay_d.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Delaunay_d.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_d/include/CGAL/Delaunay_d.h $
-// $Id: Delaunay_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_d/include/CGAL/Delaunay_d.h $
+// $Id: Delaunay_d.h adb8210 2020-04-06T21:45:32+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -822,8 +822,7 @@ locate(const Point_d& x) const
     lifted_kernel().lift_to_paraboloid_d_object();;
   Lifted_point_d lp = lift(x);
   if ( is_dimension_jump(lp) ) {
-    Simplex_iterator s;
-    for (s = const_cast<Self*>(this)->simplices_begin(NEAREST);
+    for (Simplex_iterator s = const_cast<Self*>(this)->simplices_begin(NEAREST);
          s != const_cast<Self*>(this)->simplices_end(); ++s)
       if ( contains(s,x) ) return s;
     return Simplex_handle();
diff -pruN 5.0.3-1/include/CGAL/Delaunay_mesh_area_criteria_2.h 5.1-1/include/CGAL/Delaunay_mesh_area_criteria_2.h
--- 5.0.3-1/include/CGAL/Delaunay_mesh_area_criteria_2.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Delaunay_mesh_area_criteria_2.h	2020-09-08 12:15:06.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_2/include/CGAL/Delaunay_mesh_area_criteria_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_2/include/CGAL/Delaunay_mesh_area_criteria_2.h $
 // $Id: Delaunay_mesh_area_criteria_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Delaunay_mesh_criteria_2.h 5.1-1/include/CGAL/Delaunay_mesh_criteria_2.h
--- 5.0.3-1/include/CGAL/Delaunay_mesh_criteria_2.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Delaunay_mesh_criteria_2.h	2020-09-08 12:15:06.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_2/include/CGAL/Delaunay_mesh_criteria_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_2/include/CGAL/Delaunay_mesh_criteria_2.h $
 // $Id: Delaunay_mesh_criteria_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Delaunay_mesher_2.h 5.1-1/include/CGAL/Delaunay_mesher_2.h
--- 5.0.3-1/include/CGAL/Delaunay_mesher_2.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Delaunay_mesher_2.h	2020-09-08 12:15:06.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_2/include/CGAL/Delaunay_mesher_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_2/include/CGAL/Delaunay_mesher_2.h $
 // $Id: Delaunay_mesher_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Delaunay_mesher_no_edge_refinement_2.h 5.1-1/include/CGAL/Delaunay_mesher_no_edge_refinement_2.h
--- 5.0.3-1/include/CGAL/Delaunay_mesher_no_edge_refinement_2.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Delaunay_mesher_no_edge_refinement_2.h	2020-09-08 12:15:06.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_2/include/CGAL/Delaunay_mesher_no_edge_refinement_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_2/include/CGAL/Delaunay_mesher_no_edge_refinement_2.h $
 // $Id: Delaunay_mesher_no_edge_refinement_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Delaunay_mesh_face_base_2.h 5.1-1/include/CGAL/Delaunay_mesh_face_base_2.h
--- 5.0.3-1/include/CGAL/Delaunay_mesh_face_base_2.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Delaunay_mesh_face_base_2.h	2020-09-08 12:15:06.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_2/include/CGAL/Delaunay_mesh_face_base_2.h $
-// $Id: Delaunay_mesh_face_base_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_2/include/CGAL/Delaunay_mesh_face_base_2.h $
+// $Id: Delaunay_mesh_face_base_2.h 7a6bdc1 2020-05-15T14:16:56+01:00 Andreas Fabri
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -17,6 +17,7 @@
 
 
 #include <CGAL/Constrained_Delaunay_triangulation_face_base_2.h>
+#include <CGAL/Has_timestamp.h>
 
 namespace CGAL {
 
@@ -67,6 +68,14 @@ public:
   /** compatibility with CGAL-3.2 */
   inline
   void set_marked(const bool b) { in_domain=b; }
+
+  typedef Tag_true Has_timestamp;
+
+  std::size_t time_stamp() const { return time_stamp_; }
+
+  void set_time_stamp(const std::size_t& ts) { time_stamp_ = ts; }
+
+  std::size_t time_stamp_;
 };
 
 } // namespace CGAL
diff -pruN 5.0.3-1/include/CGAL/Delaunay_mesh_local_size_criteria_2.h 5.1-1/include/CGAL/Delaunay_mesh_local_size_criteria_2.h
--- 5.0.3-1/include/CGAL/Delaunay_mesh_local_size_criteria_2.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Delaunay_mesh_local_size_criteria_2.h	2020-09-08 12:15:06.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_2/include/CGAL/Delaunay_mesh_local_size_criteria_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_2/include/CGAL/Delaunay_mesh_local_size_criteria_2.h $
 // $Id: Delaunay_mesh_local_size_criteria_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Delaunay_mesh_size_criteria_2.h 5.1-1/include/CGAL/Delaunay_mesh_size_criteria_2.h
--- 5.0.3-1/include/CGAL/Delaunay_mesh_size_criteria_2.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Delaunay_mesh_size_criteria_2.h	2020-09-08 12:15:06.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_2/include/CGAL/Delaunay_mesh_size_criteria_2.h $
-// $Id: Delaunay_mesh_size_criteria_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_2/include/CGAL/Delaunay_mesh_size_criteria_2.h $
+// $Id: Delaunay_mesh_size_criteria_2.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -111,10 +111,8 @@ public:
       typedef typename Geom_traits::Compute_squared_distance_2
         Compute_squared_distance_2;
 
-      Geom_traits traits; /** @warning traits with data!! */
-
       Compute_squared_distance_2 squared_distance =
-        traits.compute_squared_distance_2_object();
+        this->traits.compute_squared_distance_2_object();
 
       const Point_2& pa = fh->vertex(0)->point();
       const Point_2& pb = fh->vertex(1)->point();
@@ -165,7 +163,7 @@ public:
             }
         }
 
-      Compute_area_2 area_2 = traits.compute_area_2_object();
+      Compute_area_2 area_2 = this->traits.compute_area_2_object();
 
       double area = 2*CGAL::to_double(area_2(pa, pb, pc));
 
diff -pruN 5.0.3-1/include/CGAL/Delaunay_mesh_vertex_base_2.h 5.1-1/include/CGAL/Delaunay_mesh_vertex_base_2.h
--- 5.0.3-1/include/CGAL/Delaunay_mesh_vertex_base_2.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Delaunay_mesh_vertex_base_2.h	2020-09-08 12:15:06.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_2/include/CGAL/Delaunay_mesh_vertex_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_2/include/CGAL/Delaunay_mesh_vertex_base_2.h $
 // $Id: Delaunay_mesh_vertex_base_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Delaunay_triangulation_2.h 5.1-1/include/CGAL/Delaunay_triangulation_2.h
--- 5.0.3-1/include/CGAL/Delaunay_triangulation_2.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Delaunay_triangulation_2.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/Delaunay_triangulation_2.h $
-// $Id: Delaunay_triangulation_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/Delaunay_triangulation_2.h $
+// $Id: Delaunay_triangulation_2.h d1a323c 2020-03-26T19:24:14+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Mariette Yvinec
@@ -90,6 +90,11 @@ public:
        : Triangulation_2<Gt,Tds>(tr)
   {   CGAL_triangulation_postcondition(is_valid());  }
 
+  Delaunay_triangulation_2(Delaunay_triangulation_2&&) = default;
+  Delaunay_triangulation_2& operator=(const Delaunay_triangulation_2&) = default;
+  Delaunay_triangulation_2& operator=(Delaunay_triangulation_2&&) = default;
+  ~Delaunay_triangulation_2() = default;
+
  template <class InputIterator>
  Delaunay_triangulation_2(InputIterator first, InputIterator last,
                           const Gt& gt = Gt())
diff -pruN 5.0.3-1/include/CGAL/Delaunay_triangulation_3.h 5.1-1/include/CGAL/Delaunay_triangulation_3.h
--- 5.0.3-1/include/CGAL/Delaunay_triangulation_3.h	2020-08-01 19:02:59.000000000 +0000
+++ 5.1-1/include/CGAL/Delaunay_triangulation_3.h	2020-09-08 12:15:52.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_3/include/CGAL/Delaunay_triangulation_3.h $
-// $Id: Delaunay_triangulation_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_3/include/CGAL/Delaunay_triangulation_3.h $
+// $Id: Delaunay_triangulation_3.h d1a323c 2020-03-26T19:24:14+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -93,6 +93,7 @@ class Delaunay_triangulation_3<Gt, Tds_,
   typedef Delaunay_triangulation_3<Gt, Tds_, Default, Lock_data_structure_> Self;
 
 public:
+  typedef typename Tds::Concurrency_tag                     Concurrency_tag;
   typedef Triangulation_3<Gt, Tds, Lock_data_structure_>    Tr_Base;
 
   typedef Tds                                               Triangulation_data_structure;
@@ -349,7 +350,7 @@ public:
 
     size_type n = number_of_vertices();
     std::vector<Point> points(first, last);
-    spatial_sort(points.begin(), points.end(), geom_traits());
+    spatial_sort<Concurrency_tag>(points.begin(), points.end(), geom_traits());
 
     // Parallel
 #ifdef CGAL_LINKED_WITH_TBB
@@ -425,8 +426,8 @@ private:
     typedef typename Pointer_property_map<Point>::type Pmap;
     typedef Spatial_sort_traits_adapter_3<Geom_traits,Pmap> Search_traits;
 
-    spatial_sort(indices.begin(), indices.end(),
-                 Search_traits(make_property_map(points),geom_traits()));
+    spatial_sort<Concurrency_tag>(indices.begin(), indices.end(),
+                                  Search_traits(make_property_map(points),geom_traits()));
 
 #ifdef CGAL_LINKED_WITH_TBB
     if(this->is_parallel())
@@ -887,11 +888,6 @@ protected:
     : m_dt(dt), m_points(points), m_tls_hint(tls_hint)
     {}
 
-    // Constructor
-    Insert_point(const Insert_point& ip)
-    : m_dt(ip.m_dt), m_points(ip.m_points), m_tls_hint(ip.m_tls_hint)
-    {}
-
     // operator()
     void operator()(const tbb::blocked_range<size_t>& r) const
     {
@@ -967,12 +963,6 @@ protected:
       m_tls_hint(tls_hint)
     {}
 
-    // Constructor
-    Insert_point_with_info(const Insert_point_with_info& ip)
-      : m_dt(ip.m_dt), m_points(ip.m_points), m_infos(ip.m_infos),
-      m_indices(ip.m_indices), m_tls_hint(ip.m_tls_hint)
-    {}
-
     // operator()
     void operator()(const tbb::blocked_range<size_t>& r) const
     {
@@ -1046,12 +1036,6 @@ protected:
       m_vertices_to_remove_sequentially(vertices_to_remove_sequentially)
     {}
 
-    // Constructor
-    Remove_point(const Remove_point& rp)
-    : m_dt(rp.m_dt), m_vertices(rp.m_vertices),
-      m_vertices_to_remove_sequentially(rp.m_vertices_to_remove_sequentially)
-    {}
-
     // operator()
     void operator()(const tbb::blocked_range<size_t>& r) const
     {
diff -pruN 5.0.3-1/include/CGAL/Delaunay_triangulation_adaptation_policies_2.h 5.1-1/include/CGAL/Delaunay_triangulation_adaptation_policies_2.h
--- 5.0.3-1/include/CGAL/Delaunay_triangulation_adaptation_policies_2.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Delaunay_triangulation_adaptation_policies_2.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Delaunay_triangulation_adaptation_policies_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Delaunay_triangulation_adaptation_policies_2.h $
 // $Id: Delaunay_triangulation_adaptation_policies_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Delaunay_triangulation_adaptation_traits_2.h 5.1-1/include/CGAL/Delaunay_triangulation_adaptation_traits_2.h
--- 5.0.3-1/include/CGAL/Delaunay_triangulation_adaptation_traits_2.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Delaunay_triangulation_adaptation_traits_2.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Delaunay_triangulation_adaptation_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Delaunay_triangulation_adaptation_traits_2.h $
 // $Id: Delaunay_triangulation_adaptation_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Delaunay_triangulation_cell_base_3.h 5.1-1/include/CGAL/Delaunay_triangulation_cell_base_3.h
--- 5.0.3-1/include/CGAL/Delaunay_triangulation_cell_base_3.h	2020-08-01 19:02:59.000000000 +0000
+++ 5.1-1/include/CGAL/Delaunay_triangulation_cell_base_3.h	2020-09-08 12:15:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_3/include/CGAL/Delaunay_triangulation_cell_base_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_3/include/CGAL/Delaunay_triangulation_cell_base_3.h $
 // $Id: Delaunay_triangulation_cell_base_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Delaunay_triangulation_cell_base_with_circumcenter_3.h 5.1-1/include/CGAL/Delaunay_triangulation_cell_base_with_circumcenter_3.h
--- 5.0.3-1/include/CGAL/Delaunay_triangulation_cell_base_with_circumcenter_3.h	2020-08-01 19:02:59.000000000 +0000
+++ 5.1-1/include/CGAL/Delaunay_triangulation_cell_base_with_circumcenter_3.h	2020-09-08 12:15:52.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_3/include/CGAL/Delaunay_triangulation_cell_base_with_circumcenter_3.h $
-// $Id: Delaunay_triangulation_cell_base_with_circumcenter_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_3/include/CGAL/Delaunay_triangulation_cell_base_with_circumcenter_3.h $
+// $Id: Delaunay_triangulation_cell_base_with_circumcenter_3.h d1a323c 2020-03-26T19:24:14+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Monique Teillaud <Monique.Teillaud@sophia.inria.fr>
@@ -63,6 +63,12 @@ public:
     : Cb(c), circumcenter_(c.circumcenter_ != nullptr ? new Point(*(c.circumcenter_)) : nullptr)
   {}
 
+  Delaunay_triangulation_cell_base_with_circumcenter_3
+        (Delaunay_triangulation_cell_base_with_circumcenter_3 &&c)
+    : Cb(std::move(c)), circumcenter_(std::exchange(c.circumcenter_, nullptr))
+  {
+  }
+
   Delaunay_triangulation_cell_base_with_circumcenter_3&
   operator=(const Delaunay_triangulation_cell_base_with_circumcenter_3 &c)
   {
@@ -71,6 +77,14 @@ public:
       return *this;
   }
 
+  Delaunay_triangulation_cell_base_with_circumcenter_3&
+  operator=(Delaunay_triangulation_cell_base_with_circumcenter_3 &&c)
+  {
+      Cb::operator=(std::move(c));
+      circumcenter_ = std::exchange(c.circumcenter_, nullptr);
+      return *this;
+  }
+
   Delaunay_triangulation_cell_base_with_circumcenter_3(
                             Vertex_handle v0, Vertex_handle v1,
                             Vertex_handle v2, Vertex_handle v3)
diff -pruN 5.0.3-1/include/CGAL/Delaunay_triangulation.h 5.1-1/include/CGAL/Delaunay_triangulation.h
--- 5.0.3-1/include/CGAL/Delaunay_triangulation.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Delaunay_triangulation.h	2020-09-08 12:15:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation/include/CGAL/Delaunay_triangulation.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation/include/CGAL/Delaunay_triangulation.h $
 // $Id: Delaunay_triangulation.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/demangle.h 5.1-1/include/CGAL/demangle.h
--- 5.0.3-1/include/CGAL/demangle.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/demangle.h	2020-09-08 12:15:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/demangle.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/demangle.h $
 // $Id: demangle.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/determinant.h 5.1-1/include/CGAL/determinant.h
--- 5.0.3-1/include/CGAL/determinant.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/determinant.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/determinant.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/determinant.h $
 // $Id: determinant.h cb08f67 2020-05-12T11:56:18+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/determinant_of_vectors.h 5.1-1/include/CGAL/determinant_of_vectors.h
--- 5.0.3-1/include/CGAL/determinant_of_vectors.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/determinant_of_vectors.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/determinant_of_vectors.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/determinant_of_vectors.h $
 // $Id: determinant_of_vectors.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Diagonalize_traits.h 5.1-1/include/CGAL/Diagonalize_traits.h
--- 5.0.3-1/include/CGAL/Diagonalize_traits.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Diagonalize_traits.h	2020-09-08 12:15:39.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Solver_interface/include/CGAL/Diagonalize_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Solver_interface/include/CGAL/Diagonalize_traits.h $
 // $Id: Diagonalize_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Dimension.h 5.1-1/include/CGAL/Dimension.h
--- 5.0.3-1/include/CGAL/Dimension.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Dimension.h	2020-09-08 12:15:01.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Dimension.h $
-// $Id: Dimension.h 5b2d46c 2020-04-21T10:03:00+02:00 Maxime Gimeno
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Dimension.h $
+// $Id: Dimension.h e9d41d7 2020-04-21T10:03:00+02:00 Maxime Gimeno
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Sylvain Pion
diff -pruN 5.0.3-1/include/CGAL/Direction_2.h 5.1-1/include/CGAL/Direction_2.h
--- 5.0.3-1/include/CGAL/Direction_2.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Direction_2.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Direction_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Direction_2.h $
 // $Id: Direction_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Direction_3.h 5.1-1/include/CGAL/Direction_3.h
--- 5.0.3-1/include/CGAL/Direction_3.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Direction_3.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Direction_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Direction_3.h $
 // $Id: Direction_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/disable_warnings.h 5.1-1/include/CGAL/disable_warnings.h
--- 5.0.3-1/include/CGAL/disable_warnings.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/disable_warnings.h	2020-09-08 12:14:54.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/disable_warnings.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/disable_warnings.h $
 // $Id: disable_warnings.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/distance_predicates_2.h 5.1-1/include/CGAL/distance_predicates_2.h
--- 5.0.3-1/include/CGAL/distance_predicates_2.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/distance_predicates_2.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/distance_predicates_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/distance_predicates_2.h $
 // $Id: distance_predicates_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/distance_predicates_3.h 5.1-1/include/CGAL/distance_predicates_3.h
--- 5.0.3-1/include/CGAL/distance_predicates_3.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/distance_predicates_3.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/distance_predicates_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/distance_predicates_3.h $
 // $Id: distance_predicates_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/double.h 5.1-1/include/CGAL/double.h
--- 5.0.3-1/include/CGAL/double.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/double.h	2020-09-08 12:15:16.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/double.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/double.h $
 // $Id: double.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Double_map.h 5.1-1/include/CGAL/Double_map.h
--- 5.0.3-1/include/CGAL/Double_map.h	2020-08-01 19:02:32.000000000 +0000
+++ 5.1-1/include/CGAL/Double_map.h	2020-09-08 12:15:10.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesher_level/include/CGAL/Double_map.h $
-// $Id: Double_map.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesher_level/include/CGAL/Double_map.h $
+// $Id: Double_map.h 58b10a3 2020-03-26T18:58:50+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -129,7 +129,6 @@ public :
 
   size_type size() const
   {
-    CGAL_assertion(is_valid());
     return direct_func().size();
   }
 
diff -pruN 5.0.3-1/include/CGAL/draw_face_graph_with_paths.h 5.1-1/include/CGAL/draw_face_graph_with_paths.h
--- 5.0.3-1/include/CGAL/draw_face_graph_with_paths.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/draw_face_graph_with_paths.h	2020-09-08 12:15:46.000000000 +0000
@@ -0,0 +1,481 @@
+﻿// Copyright (c) 2019 CNRS and LIRIS' Establishments (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_topology/include/CGAL/draw_face_graph_with_paths.h $
+// $Id: draw_face_graph_with_paths.h 9b5205f 2020-04-22T19:58:34+02:00 Guillaume Damiand
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
+//
+#ifndef CGAL_DRAW_FACE_GRAPH_WITH_PATHS_H
+#define CGAL_DRAW_FACE_GRAPH_WITH_PATHS_H
+
+#include <CGAL/license/Surface_mesh_topology.h>
+
+#include <iostream>
+#include <initializer_list>
+#include <CGAL/draw_linear_cell_complex.h>
+#include <CGAL/Path_on_surface.h>
+
+#ifdef CGAL_USE_BASIC_VIEWER
+
+#include <CGAL/Random.h>
+
+namespace CGAL {
+
+// Specialisation for face graph; otherwise use the LCC_geom_utils of LCC.
+template<class Mesh, class Local_kernel>
+struct LCC_geom_utils<CGAL::Face_graph_wrapper<Mesh>, Local_kernel, 3>
+{
+  static typename Get_traits<Mesh>::Vector
+  get_face_normal(const CGAL::Face_graph_wrapper<Mesh>& mesh,
+                  typename CGAL::Face_graph_wrapper<Mesh>::Dart_const_handle dh)
+  {
+    typename Get_traits<Mesh>::Vector normal(CGAL::NULL_VECTOR);
+    const typename Get_traits<Mesh>::Point*
+        curr=&Get_traits<Mesh>::get_point(mesh.get_fg(), dh);
+    typename CGAL::Face_graph_wrapper<Mesh>::Dart_const_handle adart=dh;
+    unsigned int nb=0;
+
+    do
+    {
+      const typename Get_traits<Mesh>::Point*
+          next=&Get_traits<Mesh>::get_point(mesh.get_fg(),
+                                            mesh.other_extremity(adart));
+      internal::newell_single_step_3_for_lcc(*curr, *next, normal);
+      ++nb;
+      curr=next;
+      adart=mesh.next(adart);
+    }
+    while(adart!=dh);
+
+    assert(nb>0);
+    return typename Get_traits<Mesh>::Kernel::Construct_scaled_vector_3()
+      (normal, 1.0/nb);
+  }
+  static typename Local_kernel::Vector_3
+  get_vertex_normal(const CGAL::Face_graph_wrapper<Mesh>& mesh,
+                    typename CGAL::Face_graph_wrapper<Mesh>::Dart_const_handle dh)
+  {
+    typename Get_traits<Mesh>::Vector normal(CGAL::NULL_VECTOR);
+    unsigned int nb = 0;
+
+    for ( typename CGAL::Face_graph_wrapper<Mesh>::template Dart_of_cell_range<0>::
+          const_iterator it=mesh.template darts_of_cell<0>(dh).begin(),
+          itend=mesh.template darts_of_cell<0>(dh).end(); it!=itend; ++it )
+    {
+      normal=typename Get_traits<Mesh>::Kernel::Construct_sum_of_vectors_3()
+          (normal, get_face_normal(mesh, it));
+      ++nb;
+    }
+
+    if ( nb<2 ) return internal::Geom_utils
+                  <typename Get_traits<Mesh>::Kernel, Local_kernel>::
+                  get_local_vector(normal);
+
+    return internal::Geom_utils
+      <typename Get_traits<Mesh>::Kernel, Local_kernel>::
+      get_local_vector(typename Get_traits<Mesh>::Kernel::
+                       Construct_scaled_vector_3()(normal, 1.0/nb));
+  }
+};
+
+// Viewer class for Face_graph with paths.
+template<class Mesh, class DrawingFunctorLCC>
+class Face_graph_with_path_viewer : public Basic_viewer_qt
+{
+  typedef Basic_viewer_qt                         Base;
+  typedef typename Get_map<Mesh, Mesh>::type      LCC;
+  typedef typename LCC::Dart_const_handle         Dart_const_handle;
+  typedef typename LCC::size_type                 size_type;
+  typedef typename CGAL::Get_traits<Mesh>::Kernel Kernel;
+  typedef typename CGAL::Get_traits<Mesh>::Point  Point;
+  typedef typename CGAL::Get_traits<Mesh>::Vector Vector;
+
+public:
+  /// Construct the viewer.
+  /// @param alcc the lcc to view
+  /// @param title the title of the window
+  /// @param anofaces if true, do not draw faces (faces are not computed;
+  ///     this can be usefull for very big object where this time could be long)
+  Face_graph_with_path_viewer(QWidget* parent,
+                              const Mesh& amesh,
+                              const std::vector
+                              <Surface_mesh_topology::Path_on_surface<Mesh> >
+                              *paths=nullptr,
+                              size_type amark=LCC::INVALID_MARK,
+                              const char* title="", bool anofaces=false,
+                              const DrawingFunctorLCC&
+                              drawing_functor=DrawingFunctorLCC()) :
+    Base(parent, title, true, true, true, false, true),
+    mesh(amesh),
+    lcc(amesh),
+    m_oriented_mark(lcc.get_new_mark()),
+    m_nofaces(anofaces),
+    m_drawing_functor(drawing_functor),
+    m_paths(paths),
+    m_current_path(m_paths->size()),
+    m_current_dart(lcc.darts().end()),
+    m_draw_marked_darts(true),
+    m_amark(amark==(std::numeric_limits<size_type>::max)()?
+            LCC::INVALID_MARK:amark)
+  {
+    lcc.orient(m_oriented_mark);
+    compute_elements();
+  }
+
+  ~Face_graph_with_path_viewer()
+  {
+    lcc.free_mark(m_oriented_mark);
+  }
+
+protected:
+
+  const Point& get_point(Dart_const_handle dh) const
+  { return CGAL::Get_traits<Mesh>::get_point(mesh, dh); }
+
+  void compute_elements()
+  {
+    clear();
+
+    typename LCC::size_type markfaces    = lcc.get_new_mark();
+    typename LCC::size_type markedges    = lcc.get_new_mark();
+    typename LCC::size_type markvertices = lcc.get_new_mark();
+
+    if (m_current_dart!=lcc.darts().end())
+    { // We want to draw only one dart
+      Dart_const_handle selected_dart=m_current_dart; //lcc.dart_handle(m_current_dart);
+      compute_edge(selected_dart, CGAL::Color(255,0,0));
+      lcc.template mark_cell<1>(selected_dart, markedges);
+      compute_vertex(selected_dart);
+
+      if ( !m_nofaces )
+      { compute_face(selected_dart); }
+
+      for (typename LCC::Dart_range::const_iterator it=lcc.darts().begin(),
+           itend=lcc.darts().end(); it!=itend; ++it )
+      {
+        if ( !lcc.is_marked(it, markedges) )
+        {
+          compute_edge(it);
+          lcc.template mark_cell<1>(it, markedges);
+        }
+      }
+    }
+    else
+    {
+      if (m_current_path==m_paths->size())
+      {
+        for (std::size_t i=0; i<m_paths->size(); ++i)
+        { compute_path(i, markedges); }
+      }
+      else if (m_current_path!=m_paths->size()+1)
+      { compute_path(m_current_path, markedges); }
+
+      for (typename LCC::Dart_range::const_iterator it=lcc.darts().begin(),
+           itend=lcc.darts().end(); it!=itend; ++it )
+      {
+        if (!m_nofaces && !lcc.is_marked(it, markfaces) &&
+            !lcc.is_perforated(it) && lcc.is_marked(it, m_oriented_mark))
+        {
+          compute_face(it);
+          lcc.template mark_cell<2>(it, markfaces);
+        }
+
+        if ( !lcc.is_marked(it, markedges) )
+        {
+          compute_edge(it);
+          lcc.template mark_cell<1>(it, markedges);
+        }
+
+        if ( !lcc.is_marked(it, markvertices) )
+        {
+          compute_vertex(it);
+          lcc.template mark_cell<0>(it, markvertices);
+        }
+      }
+    }
+
+    lcc.free_mark(markfaces);
+    lcc.free_mark(markedges);
+    lcc.free_mark(markvertices);
+  }
+
+  void compute_face(Dart_const_handle dh)
+  {
+    // We fill only closed faces.
+    Dart_const_handle cur=dh;
+    Dart_const_handle min=dh;
+    do
+    {
+      if (!lcc.is_next_exist(cur)) return; // open face=>not filled
+      if (cur<min) min=cur;
+      cur=lcc.next(cur);
+    }
+    while(cur!=dh);
+
+    // CGAL::Color c=m_fcolor.run(*lcc, dh);
+    face_begin(); //c);
+
+    cur=dh;
+    do
+    {
+      add_point_in_face(get_point(cur),
+                        LCC_geom_utils<LCC, Local_kernel>::
+                        get_vertex_normal(lcc, cur));
+      cur=lcc.next(cur);
+    }
+    while(cur!=dh);
+
+    face_end();
+  }
+
+  void compute_edge(Dart_const_handle dh)
+  {
+    Point p1 = get_point(dh);
+    Dart_const_handle d2 = lcc.other_extremity(dh);
+    if (d2!=LCC::null_handle)
+    {
+      if (m_draw_marked_darts && m_amark!=LCC::INVALID_MARK &&
+          (lcc.is_marked(dh, m_amark) || lcc.is_marked(lcc.opposite2(dh), m_amark)))
+      { add_segment(p1, get_point(d2), CGAL::Color(0, 0, 255)); }
+      else
+      { add_segment(p1, get_point(d2)); }
+    }
+  }
+
+  void compute_edge(Dart_const_handle dh, const CGAL::Color& color)
+  {
+    Point p1 = get_point(dh);
+    Dart_const_handle d2 = lcc.other_extremity(dh);
+    if (d2!=LCC::null_handle)
+    { add_segment(p1, get_point(d2), color); }
+  }
+
+  void compute_vertex(Dart_const_handle dh)
+  { add_point(get_point(dh)); }
+
+  virtual void init()
+  {
+    Base::init();
+    setKeyDescription(::Qt::Key_D, "Increase current dart drawing");
+    setKeyDescription(::Qt::Key_D+::Qt::ControlModifier, "Decrease current dart drawing");
+    setKeyDescription(::Qt::Key_D+::Qt::ShiftModifier, "Draw all darts");
+
+    setKeyDescription(::Qt::Key_X, "Toggles marked darts display");
+
+    setKeyDescription(::Qt::Key_P, "Increase current path drawing");
+    setKeyDescription(::Qt::Key_P+::Qt::ControlModifier, "Decrease current path drawing");
+    setKeyDescription(::Qt::Key_P+::Qt::ShiftModifier, "Draw all paths");
+  }
+
+  virtual void keyPressEvent(QKeyEvent *e)
+  {
+    const ::Qt::KeyboardModifiers modifiers = e->modifiers();
+
+    if ((e->key()==::Qt::Key_D) && (modifiers==::Qt::NoButton))
+    {
+      if (m_current_dart==lcc.darts().end())
+      { m_current_dart=lcc.darts().begin(); }
+      else
+      { ++m_current_dart; }
+      if (m_current_dart==lcc.darts().end())
+      { displayMessage(QString("Draw all darts.")); }
+      else
+      { displayMessage(QString("Draw dart=%1.").arg(lcc.darts().index(m_current_dart))); }
+      compute_elements();
+      redraw();
+    }
+    else if ((e->key()==::Qt::Key_D) && (modifiers==::Qt::ControlModifier))
+    {
+      if (m_current_dart==lcc.darts().begin())
+      { m_current_dart=lcc.darts().end(); }
+      else
+      { --m_current_dart; }
+      if (m_current_dart==lcc.darts().end())
+      { displayMessage(QString("Draw all darts.")); }
+      else
+      { displayMessage(QString("Draw dart=%1.").arg(lcc.darts().index(m_current_dart))); }
+      compute_elements();
+      redraw();
+    }
+    else if ((e->key()==::Qt::Key_D) && (modifiers==::Qt::ShiftModifier))
+    {
+      m_current_dart=lcc.darts().end();
+      displayMessage(QString("Draw all darts."));
+      compute_elements();
+      redraw();
+    }
+    else if ((e->key()==::Qt::Key_P) && (modifiers==::Qt::NoButton))
+    {
+      m_current_path=(m_current_path+1)%(m_paths->size()+2);
+      if (m_current_path==m_paths->size())
+      { displayMessage(QString("Draw all paths.")); }
+      else if (m_current_path==m_paths->size()+1)
+      { displayMessage(QString("Do not draw paths.")); }
+      else
+      { displayMessage(QString("Draw path=%1, nb_darts=%2.").
+                       arg(m_current_path).
+                       arg((*m_paths)[m_current_path].length())); }
+      compute_elements();
+      redraw();
+    }
+    else if ((e->key()==::Qt::Key_P) && (modifiers==::Qt::ControlModifier))
+    {
+      m_current_path=(m_current_path==0?m_paths->size()+1:m_current_path-1);
+      if (m_current_path==m_paths->size())
+      { displayMessage(QString("Draw all paths.")); }
+      else if (m_current_path==m_paths->size()+1)
+      { displayMessage(QString("Do not draw paths.")); }
+      else
+      { displayMessage(QString("Draw path=%1, nb_darts=%2.").
+                       arg(m_current_path).
+                       arg((*m_paths)[m_current_path].length())); }
+      compute_elements();
+      redraw();
+    }
+    else if ((e->key()==::Qt::Key_P) && (modifiers==::Qt::ShiftModifier))
+    {
+      m_current_path=m_paths->size();
+      displayMessage(QString("Draw all paths."));
+      compute_elements();
+      redraw();
+    }
+    else if ((e->key()==::Qt::Key_X) && (modifiers==::Qt::NoButton))
+    {
+      m_draw_marked_darts=!m_draw_marked_darts;
+
+      if (m_draw_marked_darts)
+      { displayMessage(QString("Draw marked darts in blue.")); }
+      else
+      {
+        displayMessage(QString("Do not draw marked darts in different color."));
+      }
+      compute_elements();
+      redraw();
+    }
+    else
+    { Base::keyPressEvent(e); }
+  }
+
+  void compute_path(std::size_t i, typename LCC::size_type amark)
+  {
+    if ((*m_paths)[i].is_empty())
+    { return; }
+
+    CGAL::Random random(static_cast<unsigned int>(i));
+    CGAL::Color color=get_random_color(random);
+
+    add_point(get_point((*m_paths)[i].get_ith_dart(0)), color);
+    for (std::size_t j=0; j<(*m_paths)[i].length(); ++j)
+    {
+      if ( !lcc.is_marked( (*m_paths)[i].get_ith_dart(j), amark) )
+      {
+        compute_edge((*m_paths)[i].get_ith_dart(j), color);
+        lcc.template mark_cell<1>((*m_paths)[i].get_ith_dart(j), amark);
+      }
+    }
+  }
+
+protected:
+  const Mesh& mesh;
+  const typename Get_map<Mesh, Mesh>::storage_type lcc;
+  typename LCC::size_type m_oriented_mark;
+  bool m_nofaces;
+  const DrawingFunctorLCC& m_drawing_functor;
+  const std::vector<Surface_mesh_topology::Path_on_surface<Mesh> >* m_paths;
+  std::size_t m_current_path;
+  typename LCC::Dart_range::const_iterator m_current_dart;
+  bool m_draw_marked_darts;
+  typename LCC::size_type m_amark; // If !=INVALID_MARK, show darts marked with this mark
+};
+
+template<class Mesh, class DrawingFunctor>
+void draw(const Mesh& alcc,
+          const std::vector<Surface_mesh_topology::Path_on_surface<Mesh> >& paths,
+          const char* title="Mesh Viewer With Path",
+          typename Get_map<Mesh, Mesh>::type::size_type amark=
+          (std::numeric_limits<typename Get_map<Mesh, Mesh>::type::size_type>::max)(),
+          bool nofill=false,
+          const DrawingFunctor& drawing_functor=DrawingFunctor())
+{
+#if defined(CGAL_TEST_SUITE)
+  bool cgal_test_suite=true;
+#else
+  bool cgal_test_suite=qEnvironmentVariableIsSet("CGAL_TEST_SUITE");
+#endif
+
+  if (!cgal_test_suite)
+  {
+    int argc=1;
+    const char* argv[2]={"lccviewer","\0"};
+    QApplication app(argc,const_cast<char**>(argv));
+    Face_graph_with_path_viewer<Mesh, DrawingFunctor> mainwindow(app.activeWindow(),
+                                                                 alcc, &paths, amark,
+                                                                 title, nofill,
+                                                                 drawing_functor);
+    mainwindow.show();
+    app.exec();
+  }
+}
+
+template<class Mesh>
+void draw(const Mesh& alcc,
+          const std::vector<Surface_mesh_topology::Path_on_surface<Mesh> >& paths,
+          const char* title="Mesh Viewer With Path",
+          typename Get_map<Mesh, Mesh>::type::size_type amark=
+          (std::numeric_limits<typename Get_map<Mesh, Mesh>::type::size_type>::max)(),
+          bool nofill=false)
+{
+  DefaultDrawingFunctorLCC f;
+  draw<Mesh, DefaultDrawingFunctorLCC>(alcc, paths, title, amark, nofill, f);
+}
+
+template<class Mesh>
+void draw(const Mesh& alcc,
+          std::initializer_list<Surface_mesh_topology::Path_on_surface<Mesh>> l,
+          const char* title="Mesh Viewer With Path",
+          typename Get_map<Mesh, Mesh>::type::size_type amark=
+          (std::numeric_limits<typename Get_map<Mesh, Mesh>::type::size_type>::max)(),
+          bool nofill=false)
+{
+  std::vector<Surface_mesh_topology::Path_on_surface<Mesh> > paths=l;
+  draw(alcc, paths, title, amark, nofill);
+}
+
+} // End namespace CGAL
+
+#else  // CGAL_USE_BASIC_VIEWER
+
+namespace CGAL
+{
+
+  template<class Mesh>
+  void draw(const Mesh&,
+            const std::vector<Surface_mesh_topology::Path_on_surface<Mesh> >& ,
+            const char* ="",
+            typename Get_map<Mesh, Mesh>::type::size_type=
+            (std::numeric_limits<typename Get_map<Mesh, Mesh>::type::size_type>::max)(),
+            bool=false)
+  {
+    std::cerr<<"Impossible to draw, CGAL_USE_BASIC_VIEWER is not defined."<<std::endl;
+  }
+
+  template<class Mesh>
+  void draw(const Mesh&,
+            std::initializer_list<Surface_mesh_topology::Path_on_surface<Mesh>>,
+            const char* ="",
+            typename Get_map<Mesh, Mesh>::type::size_type=
+            (std::numeric_limits<typename Get_map<Mesh, Mesh>::type::size_type>::max)(),
+            bool=false)
+  {
+    std::cerr<<"Impossible to draw, CGAL_USE_BASIC_VIEWER is not defined."<<std::endl;
+  }
+
+} // End namespace CGAL
+
+#endif // CGAL_USE_BASIC_VIEWER
+
+#endif // CGAL_DRAW_FACE_GRAPH_WITH_PATHS_H
diff -pruN 5.0.3-1/include/CGAL/draw_linear_cell_complex.h 5.1-1/include/CGAL/draw_linear_cell_complex.h
--- 5.0.3-1/include/CGAL/draw_linear_cell_complex.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/draw_linear_cell_complex.h	2020-09-08 12:15:06.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Linear_cell_complex/include/CGAL/draw_linear_cell_complex.h $
-// $Id: draw_linear_cell_complex.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Linear_cell_complex/include/CGAL/draw_linear_cell_complex.h $
+// $Id: draw_linear_cell_complex.h 3fb644e 2020-04-10T17:40:30+02:00 Guillaume Damiand
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
@@ -16,7 +16,7 @@
 
 #ifdef CGAL_USE_BASIC_VIEWER
 
-#include <CGAL/Linear_cell_complex_base.h>
+#include <CGAL/Linear_cell_complex_operations.h>
 #include <CGAL/Random.h>
 
 namespace CGAL
@@ -170,17 +170,28 @@ public:
     // First draw: vertices; edges, faces; multi-color; inverse normal
     Base(parent, title, true, true, true, false, false),
     lcc(alcc),
+    m_oriented_mark(lcc->get_new_mark()),
     m_nofaces(anofaces),
     m_random_face_color(false),
     m_drawing_functor(drawing_functor)
   {
+    lcc->orient(m_oriented_mark);
     compute_elements();
   }
 
+  ~SimpleLCCViewerQt()
+  { lcc->free_mark(m_oriented_mark); }
+
 protected:
   void set_lcc(const LCC* alcc, bool doredraw=true)
   {
+    if (lcc!=nullptr)
+    { lcc->free_mark(m_oriented_mark); }
+
     lcc=alcc;
+    m_oriented_mark=lcc->get_new_mark();
+    lcc->orient(m_oriented_mark);
+
     compute_elements();
     if (doredraw) { redraw(); }
   }
@@ -279,6 +290,7 @@ protected:
         {
           lcc->mark(itv, markvolumes); // To be sure that all darts of the basic iterator will be marked
           if (!lcc->is_marked(itv, markfaces) &&
+              lcc->is_marked(itv, m_oriented_mark) &&
               m_drawing_functor.draw_face(*lcc, itv))
           {
             if (!m_drawing_functor.volume_wireframe(*lcc, itv) &&
@@ -335,13 +347,13 @@ protected:
   virtual void init()
   {
     Base::init();
-    setKeyDescription(::Qt::Key_C, "Toggles random face colors");
+    setKeyDescription(::Qt::Key_R, "Toggles random face colors");
   }
 
   virtual void keyPressEvent(QKeyEvent *e)
   {
     const ::Qt::KeyboardModifiers modifiers = e->modifiers();
-    if ((e->key()==::Qt::Key_C) && (modifiers==::Qt::NoButton))
+    if ((e->key()==::Qt::Key_R) && (modifiers==::Qt::NoButton))
     {
       m_random_face_color=!m_random_face_color;
       displayMessage(QString("Random face color=%1.").arg(m_random_face_color?"true":"false"));
@@ -360,6 +372,7 @@ protected:
 
 protected:
   const LCC* lcc;
+  typename LCC::size_type m_oriented_mark;
   bool m_nofaces;
   bool m_random_face_color;
   const DrawingFunctorLCC& m_drawing_functor;
@@ -376,10 +389,12 @@ template < unsigned int d_, unsigned int
            template<unsigned int,class,class,class,class>
            class Map,
            class Refs,
-           class Storage_>
+           class Storage_,
+           class DrawingFunctorLCC=DefaultDrawingFunctorLCC>
 void draw(const CGAL_LCC_TYPE& alcc,
           const char* title="LCC for CMap Basic Viewer",
-          bool nofill=false)
+          bool nofill=false,
+          const DrawingFunctorLCC& drawing_functor=DrawingFunctorLCC())
 {
 #if defined(CGAL_TEST_SUITE)
   bool cgal_test_suite=true;
@@ -392,9 +407,8 @@ void draw(const CGAL_LCC_TYPE& alcc,
     int argc=1;
     const char* argv[2]={"lccviewer","\0"};
     QApplication app(argc,const_cast<char**>(argv));
-    DefaultDrawingFunctorLCC fcolor;
-    SimpleLCCViewerQt<CGAL_LCC_TYPE, DefaultDrawingFunctorLCC>
-      mainwindow(app.activeWindow(), &alcc, title, nofill, fcolor);
+    SimpleLCCViewerQt<CGAL_LCC_TYPE, DrawingFunctorLCC>
+      mainwindow(app.activeWindow(), &alcc, title, nofill, drawing_functor);
     mainwindow.show();
     app.exec();
   }
diff -pruN 5.0.3-1/include/CGAL/draw_nef_3.h 5.1-1/include/CGAL/draw_nef_3.h
--- 5.0.3-1/include/CGAL/draw_nef_3.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/draw_nef_3.h	2020-09-08 12:15:14.000000000 +0000
@@ -0,0 +1,272 @@
+// Copyright (c) 2019 Max-Planck-Institute Saarbruecken (Germany).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/draw_nef_3.h $
+// $Id: draw_nef_3.h 1bf02b2 2019-11-13T15:28:56+01:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Jasmeet Singh    <jasmeet.singh.mec11@iitbhu.ac.in>
+
+#ifndef DRAW_NEF_3_H
+#define DRAW_NEF_3_H
+
+#include <CGAL/license/Nef_3.h>
+#include <CGAL/Qt/Basic_viewer_qt.h>
+
+#ifdef CGAL_USE_BASIC_VIEWER
+
+#include <CGAL/Nef_3/SNC_iteration.h>
+#include <CGAL/circulator.h>
+#include <CGAL/Random.h>
+#include <unordered_map>
+
+namespace CGAL {
+
+// Default color functor; user can change it to have its own face color
+struct DefaultColorFunctorNefPolyhedron
+{
+  template<typename NefPolyhedron>
+  static CGAL::Color run(const NefPolyhedron&,
+                         typename NefPolyhedron::Halffacet_const_handle fh)
+  {
+    if (fh == nullptr) // use to get the mono color
+      return CGAL::Color(100, 125, 200); // R G B between 0-255
+
+    CGAL::Random random((unsigned int)(std::size_t)(&(*fh)));
+    return get_random_color(random);
+  }
+};
+
+// Viewer class for Nef Polyhedron
+template<class Nef_Polyhedron, class ColorFunctor>
+class SimpleNefPolyhedronViewerQt : public Basic_viewer_qt
+{
+  typedef Basic_viewer_qt                                   Base;
+  typedef typename Nef_Polyhedron::Kernel                   Kernel;
+
+  typedef typename Nef_Polyhedron::Halffacet_cycle_const_iterator            Halffacet_cycle_const_iterator;
+  typedef typename Nef_Polyhedron::SHalfedge_around_facet_const_circulator   SHalfedge_around_facet_const_circulator;
+
+  typedef typename Nef_Polyhedron::Shell_entry_const_iterator   Shell_entry_const_iterator;
+  typedef typename Nef_Polyhedron::SHalfedge_const_iterator     SHalfedge_const_iterator;
+  typedef typename Nef_Polyhedron::Volume_const_iterator        Volume_const_iterator;
+
+  typedef typename Nef_Polyhedron::Vertex_const_handle       Vertex_const_handle;
+  typedef typename Nef_Polyhedron::SFace_const_handle        SFace_const_handle;
+  typedef typename Nef_Polyhedron::Halfedge_const_handle     Halfedge_const_handle;
+  typedef typename Nef_Polyhedron::Halffacet_const_handle    Halffacet_const_handle;
+  typedef typename Nef_Polyhedron::SHalfedge_const_handle    SHalfedge_const_handle;
+  typedef typename Nef_Polyhedron::SHalfloop_const_handle    SHalfloop_const_handle;
+
+public:
+  /// Construct the viewer
+  /// @param anef the nef polyhedron to view
+  /// @param title the title of the window
+  /// @param anofaces if true, do not draw faces (faces are not
+  ///        computed: this can be useful for big objects)
+  SimpleNefPolyhedronViewerQt(QWidget* parent,
+                              const Nef_Polyhedron& anef,
+                              const char* title="Basic Nef Polyhedron Viewer",
+                              bool anofaces=false,
+                              const ColorFunctor& fcolor=ColorFunctor()) :
+  //First draw: vertex; edges, faces; mon-color; inverse normal
+    Base(parent, title, false, true, true, true, false),
+    nef(anef),
+    m_nofaces(anofaces),
+    m_fcolor(fcolor)
+  {
+    compute_elements();
+  }
+protected:
+  // Visitor class to iterate through shell objects
+  class Nef_Visitor {
+  public:
+    Nef_Visitor(SimpleNefPolyhedronViewerQt &v)
+      : n_faces(0), n_edges(0), viewer(v) {}
+
+    void visit(Vertex_const_handle vh) {
+      viewer.add_point(vh->point());
+    }
+
+    void visit(Halffacet_const_handle opposite_facet)
+    {
+      Halffacet_const_handle f = opposite_facet->twin();
+
+      if (facets_done.find(f) != facets_done.end() ||
+          facets_done.find(opposite_facet) != facets_done.end()) {
+        return;
+      }
+
+      SHalfedge_const_handle se;
+      Halffacet_cycle_const_iterator fc;
+      fc = f->facet_cycles_begin();
+
+      se = SHalfedge_const_handle(fc); // non-zero if shalfedge is returned
+      if(se == 0)
+      { //return if not-shalfedge
+        return;
+      }
+
+      CGAL::Color c = viewer.run_color(f);
+      viewer.face_begin(c);
+
+      SHalfedge_around_facet_const_circulator hc_start(se);
+      SHalfedge_around_facet_const_circulator hc_end(hc_start);
+      CGAL_For_all(hc_start, hc_end) {
+        Vertex_const_handle vh = hc_start->source()->center_vertex();
+        viewer.add_point_in_face(vh->point(),
+                                 viewer.get_vertex_normal(vh));
+      }
+      viewer.face_end();
+      facets_done[f] = true;
+      n_faces++;
+    }
+
+    void visit(Halfedge_const_handle he)
+    {
+      Halfedge_const_handle twin = he->twin();
+      if (edges_done.find(he) != edges_done.end() ||
+          edges_done.find(twin) != edges_done.end())
+      {
+        // Edge already added
+        return;
+      }
+
+      viewer.add_segment(he->source()->point(), he->target()->point());
+      edges_done[he] = true;
+      n_edges++;
+    }
+
+    void visit(SHalfedge_const_handle ) {}
+    void visit(SHalfloop_const_handle ) {}
+    void visit(SFace_const_handle ) {}
+    int n_faces;
+    int n_edges;
+  protected:
+    std::unordered_map<Halffacet_const_handle, bool> facets_done;
+    std::unordered_map<Halfedge_const_handle, bool> edges_done;
+    SimpleNefPolyhedronViewerQt& viewer;
+  };
+
+  void compute_elements()
+  {
+    clear();
+
+    Volume_const_iterator c;
+
+    Nef_Visitor V(*this);
+    CGAL_forall_volumes(c, nef)
+    {
+      Shell_entry_const_iterator it;
+      CGAL_forall_shells_of(it, c)
+      {
+        nef.visit_shell_objects(SFace_const_handle(it), V);
+      }
+    }
+
+    negate_all_normals();
+  }
+
+  CGAL::Color run_color(Halffacet_const_handle fh)
+  {
+    return m_fcolor.run(nef, fh);
+  }
+
+  virtual void keyPressEvent(QKeyEvent *e)
+  {
+    // Test key pressed:
+    //    const ::Qt::KeyboardModifiers modifiers = e->modifiers();
+    //    if ((e->key()==Qt::Key_PageUp) && (modifiers==Qt::NoButton)) { ... }
+
+    // Call: * compute_elements() if the model changed, followed by
+    //       * redraw() if some viewing parameters changed that implies some
+    //                  modifications of the buffers
+    //                  (eg. type of normal, color/mono)
+    //       * update() just to update the drawing
+
+    // Call the base method to process others/classicals key
+    Base::keyPressEvent(e);
+  }
+
+protected:
+  Local_vector get_face_normal(SHalfedge_const_handle she)
+  {
+    SHalfedge_around_facet_const_circulator he(she);
+    Local_vector normal = CGAL::NULL_VECTOR;
+    SHalfedge_around_facet_const_circulator end = he;
+    unsigned int nb = 0;
+
+    CGAL_For_all(he, end)
+    {
+      internal::newell_single_step_3(this->get_local_point
+                                     (he->next()->source()->center_vertex()->point()),
+                                     this->get_local_point(he->source()->center_vertex()->
+                                                           point()), normal);
+      ++nb;
+    }
+
+    assert(nb > 0);
+    return (typename Local_kernel::Construct_scaled_vector_3()(normal, 1.0 / nb));
+  }
+
+  Local_vector get_vertex_normal(Vertex_const_handle vh)
+  {
+    Local_vector normal = CGAL::NULL_VECTOR;
+
+    SHalfedge_const_iterator it = vh->shalfedges_begin();
+    SHalfedge_const_handle end = it;
+    do {
+      Local_vector n = get_face_normal(it);
+      normal = typename Local_kernel::Construct_sum_of_vectors_3()(normal, n);
+      it = it->snext();
+    } while( it != end );
+
+    if (!typename Local_kernel::Equal_3()(normal, CGAL::NULL_VECTOR))
+    {
+      normal = (typename Local_kernel::Construct_scaled_vector_3()(
+          normal, 1.0 / CGAL::sqrt(normal.squared_length())));
+    }
+
+    return normal;
+  }
+
+protected:
+  const Nef_Polyhedron &nef;
+  bool m_nofaces;
+  const ColorFunctor &m_fcolor;
+};
+
+#define CGAL_NEF3_TYPE Nef_polyhedron_3<Kernel_, Items_, Mark_>
+
+template <typename Kernel_, typename Items_, typename Mark_>
+void draw(const CGAL_NEF3_TYPE &anef,
+          const char *title = "Nef Polyhedron Viewer",
+          bool nofill = false) {
+#if defined(CGAL_TEST_SUITE)
+  bool cgal_test_suite = true;
+#else
+  bool cgal_test_suite = qEnvironmentVariableIsSet("CGAL_TEST_SUITE");
+#endif
+
+  if (!cgal_test_suite)
+  {
+    int argc = 1;
+    const char *argv[2] = {"nef_polyhedron_viewer", "\0"};
+    QApplication app(argc, const_cast<char **>(argv));
+    DefaultColorFunctorNefPolyhedron fcolor;
+    SimpleNefPolyhedronViewerQt<CGAL_NEF3_TYPE,
+                                DefaultColorFunctorNefPolyhedron>
+        mainwindow(app.activeWindow(), anef, title, nofill, fcolor);
+    mainwindow.show();
+    app.exec();
+  }
+}
+
+} // End namespace CGAL
+
+#endif // CGAL_USE_BASIC_VIEWER
+
+#endif // DRAW_NEF_3_H
diff -pruN 5.0.3-1/include/CGAL/draw_periodic_2_triangulation_2.h 5.1-1/include/CGAL/draw_periodic_2_triangulation_2.h
--- 5.0.3-1/include/CGAL/draw_periodic_2_triangulation_2.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/draw_periodic_2_triangulation_2.h	2020-09-08 12:15:19.000000000 +0000
@@ -0,0 +1,279 @@
+// Copyright (c) 2019 INRIA Sophia-Antipolis (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_2_triangulation_2/include/CGAL/draw_periodic_2_triangulation_2.h $
+// $Id: draw_periodic_2_triangulation_2.h e9d41d7 2020-04-21T10:03:00+02:00 Maxime Gimeno
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Jasmeet Singh <jasmeet.singh.mec11@iitbhu.ac.in>
+
+#ifndef DRAW_PERIODIC_2_TRIANGULATION_2_H
+#define DRAW_PERIODIC_2_TRIANGULATION_2_H
+
+#include <CGAL/license/Periodic_2_triangulation_2.h>
+#include <CGAL/Qt/Basic_viewer_qt.h>
+
+#ifdef CGAL_USE_BASIC_VIEWER
+
+#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
+#include <CGAL/Random.h>
+
+namespace CGAL {
+
+// Default color functor; user can change it to have its own face color
+struct DefaultColorFunctorP2T2 {
+  template <typename P2T2>
+  static CGAL::Color run(const P2T2 &,
+                         const typename P2T2::Periodic_triangle_iterator /*ti*/) {
+    //CGAL::Random random((unsigned int)(std::size_t)(&*ti));
+    //return get_random_color(random);
+    return CGAL::Color(73, 250, 117);
+  }
+};
+
+// Viewer class for P2T2
+template <class P2T2, class ColorFunctor>
+class SimplePeriodic2Triangulation2ViewerQt : public Basic_viewer_qt
+{
+  typedef Basic_viewer_qt                         Base;
+
+  typedef typename P2T2::Iterator_type            Iterator_type;
+
+  // Vertex iterators
+  typedef typename P2T2::Periodic_point_iterator      Periodic_point_iterator;
+
+  // Edge iterators
+  typedef typename P2T2::Periodic_segment_iterator    Periodic_segment_iterator;
+  typedef typename P2T2::Segment                      Segment;
+
+  // Face iterators
+  typedef typename P2T2::Periodic_triangle_iterator   Periodic_triangle_iterator;
+  typedef typename P2T2::Triangle                     Triangle;
+
+  typedef CGAL::Exact_predicates_inexact_constructions_kernel Kernel;
+
+public:
+  /// Construct the viewer.
+  /// @param ap2t2 the p2t2 to view
+  /// @param title the title of the window
+  /// @param anofaces if true, do not draw faces (faces are not computed; this can be
+  ///        usefull for very big object where this time could be long)
+  SimplePeriodic2Triangulation2ViewerQt(QWidget* parent, const P2T2& ap2t2,
+                               const char* title="Basic P2T2 Viewer",
+                               bool anofaces=false,
+                               const ColorFunctor& fcolor=ColorFunctor()) :
+    // First draw: vertices; edges, faces; multi-color; no inverse normal
+    Base(parent, title, true, true, true, false, false),
+    p2t2(ap2t2),
+    m_nofaces(anofaces),
+    m_fcolor(fcolor),
+    m_display_type(STORED_COVER_DOMAIN),
+    m_domain(true)
+  {
+    // Add custom key description (see keyPressEvent).
+    setKeyDescription(::Qt::Key_1, "STORED: Display all geometric primitives as they are stored in "
+                                   "Triangulation_data_structure_2");
+    setKeyDescription(::Qt::Key_2, "UNIQUE: Display only one representative of each geometric primitive even "
+                                   "if the triangulation is computed in multiply sheeted covering space.");
+    setKeyDescription(::Qt::Key_3, "STORED_COVER_DOMAIN: Same as STORED but also display "
+                                   "all primitives whose intersection with the original "
+                                   "domain of the current covering space is non-empty");
+    setKeyDescription(::Qt::Key_4, "UNIQUE_COVER_DOMAIN: Same as UNIQUE but also display "
+                                   "all primitives whose intersection with the original "
+                                   "domain of the current covering space is non-empty");
+    setKeyDescription(::Qt::Key_D, "Toggle 9-sheeted domain display");
+
+    compute_elements();
+  }
+protected:
+  void compute_vertex(Periodic_point_iterator pi)
+  {
+    // Construct the point in 9-sheeted covering space and add to viewer
+    add_point(p2t2.point(*pi));
+  }
+
+  void compute_edge(Periodic_segment_iterator si)
+  {
+    // Construct the segment in 9-sheeted covering space and add to viewer
+    Segment s(p2t2.segment(*si));
+    add_segment(s[0], s[1]);
+  }
+
+  void compute_face(Periodic_triangle_iterator ti)
+  {
+    // Construct the triangle in 9-sheeted covering space and add to viewer
+    Triangle t(p2t2.triangle(*ti));
+
+    CGAL::Color c=m_fcolor.run(p2t2, ti);
+    face_begin(c);
+    add_point_in_face(t[0]);
+    add_point_in_face(t[1]);
+    add_point_in_face(t[2]);
+    face_end();
+
+    // Display the edges of the faces as segments with a
+    // light gray color for better visualization
+    add_segment(t[0], t[1], CGAL::Color(207, 213, 211));
+    add_segment(t[1], t[2], CGAL::Color(207, 213, 211));
+    add_segment(t[2], t[0], CGAL::Color(207, 213, 211));
+  }
+
+  void compute_domain()
+  {
+    Kernel::Iso_rectangle_2 orig_domain =  p2t2.domain();
+    std::array<int, 2> covering_sheets = p2t2.number_of_sheets();
+
+    for(int i = 0; i < covering_sheets[0]; i++){
+      for(int j = 0; j < covering_sheets[1]; j++){
+        Kernel::Vector_2 shift(i * (orig_domain.xmax() - orig_domain.xmin()),
+                               j * orig_domain.ymax() - orig_domain.ymin());
+        Kernel::Point_2 p1((orig_domain.min)());
+        Kernel::Point_2 p2(orig_domain.xmin(), orig_domain.ymax());
+        Kernel::Point_2 p3(orig_domain.xmax(), orig_domain.ymin());
+        Kernel::Point_2 p4((orig_domain.max)());
+
+        add_segment(p1 + shift, p2 + shift, CGAL::Color(96, 104, 252));
+        add_segment(p1 + shift, p3 + shift, CGAL::Color(96, 104, 252));
+        add_segment(p2 + shift, p4 + shift, CGAL::Color(96, 104, 252));
+        add_segment(p3 + shift, p4 + shift, CGAL::Color(96, 104, 252));
+      }
+    }
+  }
+
+  void compute_elements() {
+    // Clear the buffers
+    clear();
+
+    // Get the display type, iterate through periodic elements according
+    // to the display type
+    Iterator_type it_type = (Iterator_type)m_display_type;
+
+    // Iterate through vertices, edges and faces, add elements to buffer
+    for (Periodic_point_iterator it =
+             p2t2.periodic_points_begin(it_type);
+         it != p2t2.periodic_points_end(it_type); it++)
+    {
+      compute_vertex(it);
+    }
+
+    for (Periodic_segment_iterator it =
+             p2t2.periodic_segments_begin(it_type);
+         it != p2t2.periodic_segments_end(it_type); it++)
+    {
+      compute_edge(it);
+    }
+
+    for (Periodic_triangle_iterator it =
+             p2t2.periodic_triangles_begin(it_type);
+         it != p2t2.periodic_triangles_end(it_type); it++)
+    {
+      compute_face(it);
+    }
+
+    if(m_domain){
+      // Compute the (9-sheet covering space) domain of the periodic triangulation
+      compute_domain();
+    }
+  }
+
+  virtual void keyPressEvent(QKeyEvent *e)
+  {
+    // Test key pressed:
+    //    const ::Qt::KeyboardModifiers modifiers = e->modifiers();
+    //    if ((e->key()==Qt::Key_PageUp) && (modifiers==Qt::NoButton)) { ... }
+
+    // Call: * compute_elements() if the model changed, followed by
+    //       * redraw() if some viewing parameters changed that implies some
+    //                  modifications of the buffers
+    //                  (eg. type of normal, color/mono)
+    //       * update() just to update the drawing
+
+    // Call the base method to process others/classicals key
+    const ::Qt::KeyboardModifiers modifiers = e->modifiers();
+    if (e->text()=="1")
+    {
+      m_display_type = Display_type::UNIQUE;
+      displayMessage(QString("Display type= UNIQUE"));
+      compute_elements();
+      redraw();
+    } else if (e->text()=="2")
+    {
+      m_display_type = Display_type::UNIQUE_COVER_DOMAIN;
+      displayMessage(QString("Display type= UNIQUE_COVER_DOMAIN"));
+      compute_elements();
+      redraw();
+    } else if (e->text()=="3")
+    {
+      m_display_type = Display_type::STORED;
+      displayMessage(QString("Display type= STORED"));
+      compute_elements();
+      redraw();
+    } else if (e->text()=="4")
+    {
+      m_display_type = Display_type::STORED_COVER_DOMAIN;
+      displayMessage(QString("Display type= STORED_COVER_DOMAIN"));
+      compute_elements();
+      redraw();
+    } else if ((e->key() == ::Qt::Key_D) && (modifiers == ::Qt::NoButton))
+    {
+      m_domain=!m_domain;
+      displayMessage(QString("Draw domain=%1.").arg(m_domain?"true":"false"));
+      compute_elements();
+      redraw();
+    } else {
+      Base::keyPressEvent(e);
+    }
+  }
+
+protected:
+  const P2T2& p2t2;
+  bool m_nofaces;
+  const ColorFunctor& m_fcolor;
+  enum Display_type
+  {
+    STORED = 0,
+    UNIQUE, // 1
+    STORED_COVER_DOMAIN, // 2
+    UNIQUE_COVER_DOMAIN // 3
+  };
+  Display_type m_display_type;
+  bool m_domain;
+};
+
+// Specialization of draw function
+#define CGAL_P2T2_TYPE CGAL::Periodic_2_triangulation_2 \
+  <Gt, Tds >
+
+template < class Gt,
+           class Tds >
+void draw(const CGAL_P2T2_TYPE& ap2t2,
+          const char* title = "2D Periodic Triangulation Viewer",
+          bool nofill = false)
+{
+#if defined(CGAL_TEST_SUITE)
+  bool cgal_test_suite=true;
+#else
+  bool cgal_test_suite=qEnvironmentVariableIsSet("CGAL_TEST_SUITE");
+#endif
+
+  if (!cgal_test_suite)
+  {
+    int argc=1;
+    const char* argv[2]={"p2t2_viewer","\0"};
+    QApplication app(argc,const_cast<char**>(argv));
+    DefaultColorFunctorP2T2 fcolor;
+    SimplePeriodic2Triangulation2ViewerQt<CGAL_P2T2_TYPE,
+                                          DefaultColorFunctorP2T2>
+        mainwindow(app.activeWindow(), ap2t2, title, nofill, fcolor);
+    mainwindow.show();
+    app.exec();
+  }
+}
+
+} // namespace CGAL
+
+#endif // CGAL_USE_BASIC_VIEWER
+
+#endif // DRAW_PERIODIC_2_TRIANGULATION_2_H
diff -pruN 5.0.3-1/include/CGAL/draw_point_set_3.h 5.1-1/include/CGAL/draw_point_set_3.h
--- 5.0.3-1/include/CGAL/draw_point_set_3.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/draw_point_set_3.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_3/include/CGAL/draw_point_set_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_3/include/CGAL/draw_point_set_3.h $
 // $Id: draw_point_set_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/draw_polygon_2.h 5.1-1/include/CGAL/draw_polygon_2.h
--- 5.0.3-1/include/CGAL/draw_polygon_2.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/draw_polygon_2.h	2020-09-08 12:15:22.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon/include/CGAL/draw_polygon_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon/include/CGAL/draw_polygon_2.h $
 // $Id: draw_polygon_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/draw_polygon_with_holes_2.h 5.1-1/include/CGAL/draw_polygon_with_holes_2.h
--- 5.0.3-1/include/CGAL/draw_polygon_with_holes_2.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/draw_polygon_with_holes_2.h	2020-09-08 12:15:22.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon/include/CGAL/draw_polygon_with_holes_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon/include/CGAL/draw_polygon_with_holes_2.h $
 // $Id: draw_polygon_with_holes_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/draw_polyhedron.h 5.1-1/include/CGAL/draw_polyhedron.h
--- 5.0.3-1/include/CGAL/draw_polyhedron.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/draw_polyhedron.h	2020-09-08 12:15:28.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron/include/CGAL/draw_polyhedron.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron/include/CGAL/draw_polyhedron.h $
 // $Id: draw_polyhedron.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/draw_surface_mesh.h 5.1-1/include/CGAL/draw_surface_mesh.h
--- 5.0.3-1/include/CGAL/draw_surface_mesh.h	2020-08-01 19:02:54.000000000 +0000
+++ 5.1-1/include/CGAL/draw_surface_mesh.h	2020-09-08 12:15:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh/include/CGAL/draw_surface_mesh.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh/include/CGAL/draw_surface_mesh.h $
 // $Id: draw_surface_mesh.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/draw_triangulation_2.h 5.1-1/include/CGAL/draw_triangulation_2.h
--- 5.0.3-1/include/CGAL/draw_triangulation_2.h	2020-08-01 19:02:59.000000000 +0000
+++ 5.1-1/include/CGAL/draw_triangulation_2.h	2020-09-08 12:15:51.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/draw_triangulation_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/draw_triangulation_2.h $
 // $Id: draw_triangulation_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/draw_triangulation_3.h 5.1-1/include/CGAL/draw_triangulation_3.h
--- 5.0.3-1/include/CGAL/draw_triangulation_3.h	2020-08-01 19:02:59.000000000 +0000
+++ 5.1-1/include/CGAL/draw_triangulation_3.h	2020-09-08 12:15:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_3/include/CGAL/draw_triangulation_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_3/include/CGAL/draw_triangulation_3.h $
 // $Id: draw_triangulation_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/draw_voronoi_diagram_2.h 5.1-1/include/CGAL/draw_voronoi_diagram_2.h
--- 5.0.3-1/include/CGAL/draw_voronoi_diagram_2.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/draw_voronoi_diagram_2.h	2020-09-08 12:15:53.000000000 +0000
@@ -0,0 +1,335 @@
+// Copyright(c) 2019  Foundation for Research and Technology-Hellas (Greece).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/draw_voronoi_diagram_2.h $
+// $Id: draw_voronoi_diagram_2.h 1bf02b2 2019-11-13T15:28:56+01:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Jasmeet Singh <jasmeet.singh.mec11@iitbhu.ac.in>
+
+#ifndef CGAL_DRAW_VORONOI_DIAGRAM_2_H
+#define CGAL_DRAW_VORONOI_DIAGRAM_2_H
+
+#include <CGAL/Qt/Basic_viewer_qt.h>
+#include <CGAL/license/Voronoi_diagram_2.h>
+
+#ifdef CGAL_USE_BASIC_VIEWER
+
+#include <CGAL/Random.h>
+#include <CGAL/Triangulation_utils_2.h>
+#include <CGAL/Voronoi_diagram_2/Face.h>
+#include <CGAL/Voronoi_diagram_2/Handle_adaptor.h>
+#include <CGAL/Voronoi_diagram_2/Vertex.h>
+#include <CGAL/Voronoi_diagram_2/basic.h>
+#include <CGAL/Voronoi_diagram_2/Accessor.h>
+
+namespace CGAL {
+
+// Default color functor; user can change it to have its own face color
+struct DefaultColorFunctorV2
+{
+  template <typename V2>
+  static CGAL::Color run(const V2 &, const typename V2::Face_iterator /*fh*/) {
+    //CGAL::Random random((unsigned int)(std::size_t)(&*fh));
+    //return get_random_color(random);
+    return CGAL::Color(73, 250, 117);
+  }
+};
+
+// Viewer for Voronoi diagram
+template <class V2, class ColorFunctor>
+class SimpleVoronoiDiagram2ViewerQt : public Basic_viewer_qt
+{
+  typedef Basic_viewer_qt                                       Base;
+  typedef typename V2::Vertex_iterator                          Vertex_const_handle;
+  typedef typename V2::Delaunay_vertex_handle                   Delaunay_vertex_const_handle;
+  typedef typename V2::Delaunay_graph::Finite_vertices_iterator Dual_vertices_iterator;
+
+  typedef typename V2::Halfedge_iterator                        Halfedge_const_handle;
+  typedef typename V2::Ccb_halfedge_circulator                  Ccb_halfedge_circulator;
+  typedef typename V2::Halfedge_handle                          Halfedge_handle;
+
+  typedef typename V2::Face_iterator                            Face_const_handle;
+
+  typedef CGAL::Exact_predicates_inexact_constructions_kernel Kernel;
+
+public:
+  /// Construct the viewer.
+  /// @param av2 the voronoi diagram to view
+  /// @param title the title of the window
+  /// @param anofaces if true, do not draw faces (faces are not computed; this
+  /// can be useful for very big object where this time could be long)
+  SimpleVoronoiDiagram2ViewerQt(QWidget *parent, const V2 &av2,
+                                const char *title = "Basic Voronoi Viewer",
+                                bool anofaces = false,
+                                bool draw_voronoi_vertices = true,
+                                bool draw_delaunay_vertices = true,
+                                const ColorFunctor &fcolor = ColorFunctor())
+      : // First draw: vertices; half-edges; faces; multi-color; no inverse
+        // normal
+        Base(parent, title, true, true, true, false, false, true, true),
+        v2(av2), m_nofaces(anofaces),
+        m_draw_voronoi_vertices(draw_voronoi_vertices),
+        m_draw_dual_vertices(draw_delaunay_vertices), m_fcolor(fcolor) {
+    // Add custom key description (see keyPressEvent)
+    setKeyDescription(::Qt::Key_R, "Toggles rays display");
+    setKeyDescription(::Qt::Key_D, "Toggles dual vertices display");
+    setKeyDescription(::Qt::Key_V, "Toggles voronoi vertices display");
+
+    compute_elements();
+  }
+
+protected:
+
+  void compute_vertex(Vertex_const_handle vh) { add_point(vh->point()); }
+
+  void compute_dual_vertex(Dual_vertices_iterator vi)
+  {
+    add_point(vi->point(), CGAL::Color(50, 100, 180));
+  }
+
+  void add_segments_and_update_bounding_box(Halfedge_handle he)
+  {
+    if (he->is_segment()) {
+      add_segment(he->source()->point(), he->target()->point());
+    } else {
+      Delaunay_vertex_const_handle v1 = he->up();
+      Delaunay_vertex_const_handle v2 = he->down();
+
+      Kernel::Vector_2 direction(v1->point().y() - v2->point().y(),
+                                 v2->point().x() - v1->point().x());
+      if (he->is_ray()) {
+        Kernel::Point_2 end_point;
+        if (he->has_source()) {
+          end_point = he->source()->point();
+          update_bounding_box_for_ray(end_point, direction);
+        }
+      } else if (he->is_bisector()) {
+        Kernel::Point_2 pointOnLine((v1->point().x() + v2->point().x()) / 2,
+                                    (v1->point().y() + v2->point().y()) / 2);
+        Kernel::Vector_2 perpendicularDirection(
+            v2->point().x() - v1->point().x(),
+            v2->point().y() - v1->point().y());
+        update_bounding_box_for_line(pointOnLine, direction,
+                                     perpendicularDirection);
+      }
+    }
+  }
+
+  Local_kernel::Point_2 get_second_point(Halfedge_handle ray)
+  {
+    Delaunay_vertex_const_handle v1 = ray->up();
+    Delaunay_vertex_const_handle v2 = ray->down();
+
+    // calculate direction of ray and its inverse
+    Kernel::Vector_2 v(v1->point().y() - v2->point().y(),
+                       v2->point().x() - v1->point().x());
+    Local_kernel::Vector_2 inv(1 / v.x(), 1 / v.y());
+
+    // origin of the ray
+    Kernel::Point_2 p;
+    if (ray->has_source()) {
+      p = ray->source()->point();
+    } else {
+      p = ray->target()->point();
+    }
+
+    // get the bounding box of the viewer
+    Local_kernel::Vector_2 boundsMin(m_bounding_box.xmin(),
+                                     m_bounding_box.zmin());
+    Local_kernel::Vector_2 boundsMax(m_bounding_box.xmax(),
+                                     m_bounding_box.zmax());
+    // calculate intersection
+    double txmax, txmin, tymax, tymin;
+
+    if (inv.x() >= 0) {
+      txmax = (boundsMax.x() - p.x()) * inv.x();
+      txmin = (boundsMin.x() - p.x()) * inv.x();
+    } else {
+      txmax = (boundsMin.x() - p.x()) * inv.x();
+      txmin = (boundsMax.x() - p.x()) * inv.x();
+    }
+
+    if (inv.y() >= 0) {
+      tymax = (boundsMax.y() - p.y()) * inv.y();
+      tymin = (boundsMin.y() - p.y()) * inv.y();
+    } else {
+      tymax = (boundsMin.y() - p.y()) * inv.y();
+      tymin = (boundsMax.y() - p.y()) * inv.y();
+    }
+
+    if (tymin > txmin)
+      txmin = tymin;
+    if (tymax < txmax)
+      txmax = tymax;
+
+    Local_kernel::Point_2 p1;
+    if (v.x() == 0) {
+      p1 = Local_kernel::Point_2(p.x(), p.y() + tymax * v.y());
+    } else if (v.y() == 0) {
+      p1 = Local_kernel::Point_2(p.x() + txmax * v.x(), p.y());
+    } else {
+      p1 = Local_kernel::Point_2(p.x() + txmax * v.x(), p.y() + tymax * v.y());
+    }
+    return p1;
+  }
+
+  void compute_rays_and_bisectors(Halfedge_const_handle he)
+  {
+    Delaunay_vertex_const_handle v1 = he->up();
+    Delaunay_vertex_const_handle v2 = he->down();
+
+    Kernel::Vector_2 direction(v1->point().y() - v2->point().y(),
+                               v2->point().x() - v1->point().x());
+    if (he->is_ray()) {
+      if (he->has_source()) {
+        // add_ray_segment(he->source()->point(), get_second_point(he));
+        add_ray(he->source()->point(), direction, CGAL::Color(100, 0, 0));
+      }
+    } else if (he->is_bisector()) {
+      Kernel::Point_2 pointOnLine((v1->point().x() + v2->point().x()) / 2,
+                                  (v1->point().y() + v2->point().y()) / 2);
+      add_line(pointOnLine, direction);
+    }
+  }
+
+  void compute_face(Face_const_handle fh)
+  {
+    CGAL::Color c = m_fcolor.run(v2, fh);
+
+    Ccb_halfedge_circulator ec_start = fh->ccb();
+    Ccb_halfedge_circulator ec = ec_start;
+
+    if (!fh->is_unbounded()) {
+      face_begin(c);
+      do {
+        add_point_in_face(ec->source()->point());
+      } while (++ec != ec_start);
+      face_end();
+    }
+    // Test: for unbounded faces
+    //      else {
+    //            do{
+    //                if( ec->has_source() ){
+    //                    add_point_in_face(ec->source()->point());
+    //                }
+    //                else{
+    //                    add_point_in_face(get_second_point(ec->twin()));
+    //                }
+    //            } while(++ec != ec_start);
+    //        }
+  }
+
+  void compute_elements()
+  {
+    clear();
+
+    // Draw the voronoi vertices
+    if (m_draw_voronoi_vertices) {
+      for (typename V2::Vertex_iterator it = v2.vertices_begin();
+           it != v2.vertices_end(); ++it) {
+        compute_vertex(it);
+      }
+    }
+
+    // Draw the dual vertices
+    if (m_draw_dual_vertices) {
+      for (Dual_vertices_iterator it = v2.dual().finite_vertices_begin();
+           it != v2.dual().finite_vertices_end(); ++it) {
+        compute_dual_vertex(it);
+      }
+    }
+
+    // Add segments and update bounding box
+    for (typename V2::Halfedge_iterator it = v2.halfedges_begin();
+         it != v2.halfedges_end(); ++it) {
+      add_segments_and_update_bounding_box(it);
+    }
+
+    for (typename V2::Halfedge_iterator it = v2.halfedges_begin();
+         it != v2.halfedges_end(); ++it) {
+      compute_rays_and_bisectors(it);
+    }
+
+    if (!m_nofaces) {
+      for (typename V2::Face_iterator it = v2.faces_begin();
+           it != v2.faces_end(); ++it) {
+        compute_face(it);
+      }
+    }
+  }
+
+  virtual void keyPressEvent(QKeyEvent *e)
+  {
+    /// [Keypress]
+    const ::Qt::KeyboardModifiers modifiers = e->modifiers();
+    if ((e->key() == ::Qt::Key_R) && (modifiers == ::Qt::NoButton)) {
+      m_draw_rays = !m_draw_rays;
+      displayMessage(
+          QString("Draw rays=%1.").arg(m_draw_rays ? "true" : "false"));
+      update();
+    } else if ((e->key() == ::Qt::Key_V) && (modifiers == ::Qt::NoButton)) {
+      m_draw_voronoi_vertices = !m_draw_voronoi_vertices;
+      displayMessage(
+          QString("Voronoi vertices=%1.").arg(m_draw_voronoi_vertices? "true" : "false"));
+      compute_elements();
+      redraw();
+    } else if ((e->key() == ::Qt::Key_D) && (modifiers == ::Qt::NoButton)) {
+      m_draw_dual_vertices = !m_draw_dual_vertices;
+      displayMessage(QString("Dual vertices=%1.")
+                         .arg(m_draw_dual_vertices ? "true" : "false"));
+      compute_elements();
+      redraw();
+    } else {
+      // Call the base method to process others/classicals key
+      Base::keyPressEvent(e);
+    }
+    /// [Keypress]
+  }
+
+protected:
+  const V2 &v2;
+  bool m_nofaces;
+  bool m_draw_voronoi_vertices;
+  bool m_draw_dual_vertices;
+  const ColorFunctor &m_fcolor;
+};
+
+// Specialization of draw function.
+#define CGAL_VORONOI_TYPE CGAL::Voronoi_diagram_2 <DG, AT, AP>
+
+template<class DG,
+         class AT,
+         class AP>
+void draw(const CGAL_VORONOI_TYPE &av2,
+          const char *title="2D Voronoi Diagram Basic Viewer",
+          bool nofill = false,
+          bool draw_voronoi_vertices = true,
+          bool draw_dual_vertices = true)
+{
+#if defined(CGAL_TEST_SUITE)
+  bool cgal_test_suite = true;
+#else
+  bool cgal_test_suite = qEnvironmentVariableIsSet("CGAL_TEST_SUITE");
+#endif
+
+  if (!cgal_test_suite) {
+    int argc = 1;
+    const char *argv[2] = {"voronoi_2_viewer", "\0"};
+    QApplication app(argc, const_cast<char **>(argv));
+    DefaultColorFunctorV2 fcolor;
+    SimpleVoronoiDiagram2ViewerQt<CGAL_VORONOI_TYPE, DefaultColorFunctorV2>
+        mainwindow(app.activeWindow(), av2, title, nofill,
+                   draw_voronoi_vertices, draw_dual_vertices, fcolor);
+    mainwindow.show();
+    app.exec();
+  }
+}
+
+} // End namespace CGAL
+
+#endif // CGAL_USE_BASIC_VIEWER
+
+#endif // CGAL_DRAW_VORONOI_DIAGRAM_2_H
diff -pruN 5.0.3-1/include/CGAL/Dummy_tds_2.h 5.1-1/include/CGAL/Dummy_tds_2.h
--- 5.0.3-1/include/CGAL/Dummy_tds_2.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Dummy_tds_2.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/TDS_2/include/CGAL/Dummy_tds_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/TDS_2/include/CGAL/Dummy_tds_2.h $
 // $Id: Dummy_tds_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Dynamic_matrix.h 5.1-1/include/CGAL/Dynamic_matrix.h
--- 5.0.3-1/include/CGAL/Dynamic_matrix.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Dynamic_matrix.h	2020-09-08 12:15:06.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Matrix_search/include/CGAL/Dynamic_matrix.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Matrix_search/include/CGAL/Dynamic_matrix.h $
 // $Id: Dynamic_matrix.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Dynamic_property_map.h 5.1-1/include/CGAL/Dynamic_property_map.h
--- 5.0.3-1/include/CGAL/Dynamic_property_map.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Dynamic_property_map.h	2020-09-08 12:15:30.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Property_map/include/CGAL/Dynamic_property_map.h $
-// $Id: Dynamic_property_map.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Property_map/include/CGAL/Dynamic_property_map.h $
+// $Id: Dynamic_property_map.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -13,11 +13,13 @@
 #define CGAL_DYNAMIC_PROPERTY_MAP_H
 
 #include <boost/graph/graph_traits.hpp>
+#include <boost/graph/properties.hpp>
+
 #include <CGAL/boost/graph/properties.h>
 #include <CGAL/property_map.h>
+
 #include <boost/shared_ptr.hpp>
 #include <boost/unordered_map.hpp>
-
 #include <boost/type_traits/is_same.hpp>
 #include <boost/mpl/if.hpp>
 
diff -pruN 5.0.3-1/include/CGAL/edge_aware_upsample_point_set.h 5.1-1/include/CGAL/edge_aware_upsample_point_set.h
--- 5.0.3-1/include/CGAL/edge_aware_upsample_point_set.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/edge_aware_upsample_point_set.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_processing_3/include/CGAL/edge_aware_upsample_point_set.h $
-// $Id: edge_aware_upsample_point_set.h 67b53fa 2020-04-29T14:06:37+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/edge_aware_upsample_point_set.h $
+// $Id: edge_aware_upsample_point_set.h b6a5502 2020-08-06T11:33:47+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s) : Shihao Wu, Clement Jamin, Pierre Alliez
@@ -147,10 +147,8 @@ update_new_point(
   typedef typename rich_grid_internal::Rich_point<Kernel> Rich_point;
 
   CGAL_assertion_code( unsigned int size = static_cast<unsigned int>(rich_point_set.size()) );
-  CGAL_point_set_processing_precondition(father_index >= 0 &&
-                                         father_index < size);
-  CGAL_point_set_processing_precondition(mother_index >= 0 &&
-                                           mother_index < size);
+  CGAL_point_set_processing_precondition(father_index < size);
+  CGAL_point_set_processing_precondition(mother_index < size);
 
   // 1, get neighbor information from the two "parent points"
   Rich_point& new_v = rich_point_set[new_point_index];
@@ -280,8 +278,8 @@ update_new_point(
    Normals of points are required as input. For more details, please refer to \cgalCite{ear-2013}.
 
    \tparam ConcurrencyTag enables sequential versus parallel versions
-   of `compute_average_spacing()` (called internally).  Possible
-   values are `Sequential_tag` and `Parallel_tag`.
+   of `compute_average_spacing()` (called internally). Possible
+   values are `Sequential_tag`, `Parallel_tag`, and `Parallel_if_available_tag`.
    \tparam PointRange is a model of `ConstRange`. The value type of
    its iterator is the key type of the named parameter `point_map`.
    \tparam OutputIterator Type of the output iterator.
@@ -293,21 +291,61 @@ update_new_point(
 
    \param points input point range.
    \param output iterator where output points and normals are put.
-   \param np optional sequence of \ref psp_namedparameters "Named Parameters" among the ones listed below.
+   \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
    \cgalNamedParamsBegin
-     \cgalParamBegin{point_map} a model of `ReadablePropertyMap` with value type `geom_traits::Point_3`.
-     If this parameter is omitted, `CGAL::Identity_property_map<geom_traits::Point_3>` is used.\cgalParamEnd
-     \cgalParamBegin{normal_map} a model of `ReadablePropertyMap` with value type
-     `geom_traits::Vector_3`.\cgalParamEnd
-     \cgalParamBegin{sharpness_angle} controls the sharpness of the result.\cgalParamEnd
-     \cgalParamBegin{edge_sensitivity} controls the priority of points inserted along sharp features. See
-     section \ref Point_set_processing_3Upsample_Parameter1 for an example.\cgalParamEnd
-     \cgalParamBegin{neighbor_radius} spherical neighborhood radius.\cgalParamEnd
-     \cgalParamBegin{number_of_output_points} is the number of output points to generate.\cgalParamEnd
-     \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+     \cgalParamNBegin{point_map}
+       \cgalParamDescription{a property map associating points to the elements of the point set `points`}
+       \cgalParamType{a model of `ReadablePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange` and whose value type is `geom_traits::Point_3`}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{normal_map}
+       \cgalParamDescription{a property map associating normals to the elements of the point set `points`}
+       \cgalParamType{a model of `ReadablePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange` and whose value type is `geom_traits::Vector_3`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{sharpness_angle}
+       \cgalParamDescription{controls the sharpness of the result}
+       \cgalParamType{floating scalar value}
+       \cgalParamDefault{`30.00`}
+       \cgalParamExtra{The larger the value is, the smoother the result will be.
+                       The range of possible value is `[0, 90]`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{edge_sensitivity}
+       \cgalParamDescription{controls the priority of points inserted along sharp features}
+       \cgalParamType{floating scalar value}
+       \cgalParamDefault{`1`}
+       \cgalParamExtra{Larger values of edge-sensitivity give higher priority to inserting points
+                       along sharp features. The range of possible values is `[0, 1]`.
+                       See section \ref Point_set_processing_3Upsample_Parameter1 for an example}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{number_of_output_points}
+       \cgalParamDescription{the number of output points to generate}
+       \cgalParamType{unsigned int}
+       \cgalParamDefault{`1000`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{neighbor_radius}
+       \cgalParamDescription{the spherical neighborhood radius}
+       \cgalParamType{floating scalar value}
+       \cgalParamDefault{`0` (no limit)}
+       \cgalParamExtra{If provided, the neighborhood of a query point is computed with a fixed spherical
+                       radius instead of a fixed number of neighbors. In that case, the parameter
+                       `k` is used as a limit on the number of points returned by each spherical
+                       query (to avoid overly large number of points in high density areas).}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{geom_traits}
+       \cgalParamDescription{an instance of a geometric traits class}
+       \cgalParamType{a model of `Kernel`}
+       \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+     \cgalParamNEnd
    \cgalNamedParamsEnd
-
 */
 template <typename ConcurrencyTag,
           typename PointRange,
@@ -323,7 +361,7 @@ edge_aware_upsample_point_set(
   using parameters::get_parameter;
 
   // basic geometric types
-  typedef typename Point_set_processing_3::GetPointMap<PointRange, NamedParameters>::type PointMap;
+  typedef typename CGAL::GetPointMap<PointRange, NamedParameters>::type PointMap;
   typedef typename Point_set_processing_3::GetNormalMap<PointRange, NamedParameters>::type NormalMap;
   typedef typename Point_set_processing_3::GetK<PointRange, NamedParameters>::Kernel Kernel;
 
@@ -336,8 +374,8 @@ edge_aware_upsample_point_set(
   typedef typename Kernel::FT FT;
   typedef typename rich_grid_internal::Rich_point<Kernel> Rich_point;
 
-  PointMap point_map = choose_parameter(get_parameter(np, internal_np::point_map), PointMap());
-  NormalMap normal_map = choose_parameter(get_parameter(np, internal_np::normal_map), NormalMap());
+  PointMap point_map = choose_parameter<PointMap>(get_parameter(np, internal_np::point_map));
+  NormalMap normal_map = choose_parameter<NormalMap>(get_parameter(np, internal_np::normal_map));
   double sharpness_angle = choose_parameter(get_parameter(np, internal_np::sharpness_angle), 30.);
   double edge_sensitivity = choose_parameter(get_parameter(np, internal_np::edge_sensitivity), 1);
   double neighbor_radius = choose_parameter(get_parameter(np, internal_np::neighbor_radius), -1);
@@ -355,7 +393,6 @@ edge_aware_upsample_point_set(
                                        &&sharpness_angle <= 90);
   CGAL_point_set_processing_precondition(edge_sensitivity >= 0
                                        &&edge_sensitivity <= 1);
-  CGAL_point_set_processing_precondition(neighbor_radius > 0);
 
   edge_sensitivity *= 10;  // just project [0, 1] to [0, 10].
 
diff -pruN 5.0.3-1/include/CGAL/Eigen_diagonalize_traits.h 5.1-1/include/CGAL/Eigen_diagonalize_traits.h
--- 5.0.3-1/include/CGAL/Eigen_diagonalize_traits.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Eigen_diagonalize_traits.h	2020-09-08 12:15:39.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Solver_interface/include/CGAL/Eigen_diagonalize_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Solver_interface/include/CGAL/Eigen_diagonalize_traits.h $
 // $Id: Eigen_diagonalize_traits.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Eigen_matrix.h 5.1-1/include/CGAL/Eigen_matrix.h
--- 5.0.3-1/include/CGAL/Eigen_matrix.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Eigen_matrix.h	2020-09-08 12:15:39.000000000 +0000
@@ -2,383 +2,90 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Solver_interface/include/CGAL/Eigen_matrix.h $
-// $Id: Eigen_matrix.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Solver_interface/include/CGAL/Eigen_matrix.h $
+// $Id: Eigen_matrix.h 9f2eafd 2020-03-26T19:17:02+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Gael Guennebaud
 
-#ifndef CGAL_EIGEN_MATRIX_H
-#define CGAL_EIGEN_MATRIX_H
+#ifndef CGAL_SOLVER_INTERFACE_EIGEN_MATRIX_H
+#define CGAL_SOLVER_INTERFACE_EIGEN_MATRIX_H
 
 #include <CGAL/basic.h> // include basic.h before testing #defines
 
-#include <Eigen/Sparse>
+#include <CGAL/Eigen_sparse_matrix.h> // for backward compatibility
+
+#include <Eigen/Dense>
 
 namespace CGAL {
 
 /*!
 \ingroup PkgSolverInterfaceRef
 
-The class `Eigen_sparse_matrix` is a wrapper around `Eigen` matrix type
-<a href="http://eigen.tuxfamily.org/dox/classEigen_1_1SparseMatrix.html">`Eigen::SparseMatrix`</a>
-that represents general matrices, be they symmetric or not.
+The class `Eigen_matrix` is a wrapper around `Eigen` matrix type
+<a href="http://eigen.tuxfamily.org/dox/classEigen_1_1Matrix.html">`Eigen::Matrix`</a>.
 
-\cgalModels `SparseLinearAlgebraTraits_d::Matrix`
+\cgalModels `SvdTraits::Matrix`
 
 \tparam T Number type.
+\tparam D1 Number of rows, or Dynamic
+\tparam D2 Number of columns, or Dynamic
 
 \sa `CGAL::Eigen_vector<T>`
-\sa `CGAL::Eigen_matrix<T>`
+\sa `CGAL::Eigen_sparse_matrix<T>`
 \sa `CGAL::Eigen_sparse_symmetric_matrix<T>`
 */
-template<class T>
-struct Eigen_sparse_matrix
+template <class FT, int D1 = ::Eigen::Dynamic, int D2 = ::Eigen::Dynamic>
+struct Eigen_matrix
+  : public ::Eigen::Matrix<FT, D1, D2>
 {
-  // Public types
-public:
-  /// \name Types
-  /// @{
-
   /// The internal matrix type from \ref thirdpartyEigen "Eigen".
-  typedef Eigen::SparseMatrix<T>      EigenType;
-
-  typedef T                           NT;
-  /// @}
+  typedef ::Eigen::Matrix<FT, D1, D2> EigenType;
 
-  Eigen_sparse_matrix(const EigenType& et)
-    : m_is_already_built(true), m_matrix(et), m_is_symmetric(false)
-  {}
-
-  // Public operations
-public:
-  Eigen_sparse_matrix()  :
-      m_is_already_built(false)
-  {}
-
-  /// Create a square matrix initialized with zeros.
-  Eigen_sparse_matrix(std::size_t  dim,           ///< Matrix dimension.
-                      bool is_symmetric = false)  ///< Symmetric/hermitian?
-    :
-      m_is_already_built(false),
-      m_matrix(static_cast<int>(dim), static_cast<int>(dim))
-  {
-    CGAL_precondition(dim > 0);
-
-    m_is_symmetric = is_symmetric;
-    m_triplets.reserve(dim); // reserve memory for a regular 3D grid
-  }
+  /// Constructs a null matrix.
+  Eigen_matrix() { }
 
-  /// Create a square matrix initialized with zeros.
-  Eigen_sparse_matrix(int dim,                    ///< Matrix dimension.
-                      bool is_symmetric = false)  ///< Symmetric/hermitian?
-    : m_is_already_built(false),
-      m_matrix(dim, dim)
-  {
-    CGAL_precondition(dim > 0);
-
-    m_is_symmetric = is_symmetric;
-    // reserve memory for a regular 3D grid
-    m_triplets.reserve(dim);
-  }
-
-  /// Create a rectangular matrix initialized with zeros.
-  ///
-  /// \pre rows == columns if `is_symmetric` is true.
-  Eigen_sparse_matrix(std::size_t rows,          ///< Number of rows.
-                      std::size_t columns,       ///< Number of columns.
-                      bool is_symmetric = false) ///< Symmetric/hermitian?
-    : m_is_already_built(false),
-      m_matrix(static_cast<int>(rows), static_cast<int>(columns))
-  {
-    CGAL_precondition(rows > 0);
-    CGAL_precondition(columns > 0);
-    if(m_is_symmetric)
-    {
-      CGAL_precondition(rows == columns);
-    }
-
-    m_is_symmetric = is_symmetric;
-    // reserve memory for a regular 3D grid
-    m_triplets.reserve(rows);
-  }
-
-  void swap(Eigen_sparse_matrix& other)
-  {
-    std::swap(m_is_already_built, other.m_is_already_built);
-    std::swap(m_is_symmetric, other.m_is_symmetric);
-    m_matrix.swap(other.m_matrix);
-    m_triplets.swap(other.m_triplets);
-  }
-
-
-  /// Delete this object and the wrapped matrix.
-  ~Eigen_sparse_matrix() { }
-
-  /// Create a rectangular matrix initialized with zeros.
-  ///
-  /// \pre rows == columns if `is_symmetric` is true.
-  Eigen_sparse_matrix(int rows,                  ///< Number of rows.
-                      int columns,               ///< Number of columns.
-                      bool is_symmetric = false) ///< Symmetric/hermitian?
-    : m_is_already_built(false),
-      m_matrix(rows,columns)
-  {
-    CGAL_precondition(rows > 0);
-    CGAL_precondition(columns > 0);
-    if(is_symmetric)
-    {
-      CGAL_precondition(rows == columns);
-    }
-
-    m_is_symmetric = is_symmetric;
-    // reserve memory for a regular 3D grid
-    m_triplets.reserve(rows);
-  }
-
-  /// Return the matrix number of rows
-  int row_dimension() const { return static_cast<int>(m_matrix.rows()); }
-  /// Return the matrix number of columns
-  int column_dimension() const { return static_cast<int>(m_matrix.cols()); }
-
-  /// Write access to a matrix coefficient: a_ij <- val.
-  ///
-  /// Users can optimize calls to this function by setting 'new_coef' to `true`
-  /// if the coefficient does not already exist in the matrix.
-  ///
-  /// \warning For symmetric matrices, `Eigen_sparse_matrix` only stores the lower triangle
-  ///   and `set_coef()` does nothing if (i, j) belongs to the upper triangle.
-  ///
-  /// \pre 0 <= i < row_dimension().
-  /// \pre 0 <= j < column_dimension().
-  void set_coef(std::size_t i_, std::size_t j_, T val, bool new_coef = false)
-  {
-    int i = static_cast<int>(i_);
-    int j = static_cast<int>(j_);
-    CGAL_precondition(i < row_dimension());
-    CGAL_precondition(j < column_dimension());
-
-    if (m_is_symmetric && (j > i))
-      return;
-
-    if(m_is_already_built)
-    {
-      m_matrix.coeffRef(i,j) = val;
-    }
-    else
-    {
-      if(new_coef == false)
-      {
-        assemble_matrix();
-        m_matrix.coeffRef(i,j) = val;
-      }
-      else
-      {
-        m_triplets.push_back(Triplet(i,j,val));
-      }
-    }
-  }
-
-  /// Write access to a matrix coefficient: a_ij <- a_ij + val.
-  ///
-  /// \warning For symmetric matrices, Eigen_sparse_matrix only stores the lower triangle
-  /// `add_coef()` does nothing if (i, j) belongs to the upper triangle.
-  ///
-  /// \pre 0 <= i < row_dimension().
-  /// \pre 0 <= j < column_dimension().
-  void add_coef(std::size_t i_, std::size_t j_, T  val)
-  {
-    int i = static_cast<int>(i_);
-    int j = static_cast<int>(j_);
-    if(m_is_symmetric && (j > i))
-      return;
-
-    if(m_is_already_built){
-      CGAL_precondition(i < row_dimension());
-      CGAL_precondition(j < column_dimension());
-      m_matrix.coeffRef(i,j) += val;
-    }else{
-      m_triplets.push_back(Triplet(i,j,val));
-    }
-  }
-
-  /// Read access to a matrix coefficient.
-  ///
-  /// \warning Complexity:
-  /// - O(log(n)) if the matrix is already built.
-  /// - O(n) if the matrix is not built.
-  /// `n` being the number of entries in the matrix.
-  ///
-  /// \pre 0 <= i < row_dimension().
-  /// \pre 0 <= j < column_dimension().
-  NT get_coef (std::size_t i_, std::size_t j_) const
-  {
-    int i = static_cast<int>(i_);
-    int j = static_cast<int>(j_);
-    CGAL_precondition(i < row_dimension());
-    CGAL_precondition(j < column_dimension());
-
-    if(m_is_symmetric && j > i)
-      std::swap(i, j);
-
-    if (m_is_already_built)
-      return m_matrix.coeffRef(i,j);
-    else
-    {
-      NT val = 0;
-      for(std::size_t t=0; t<m_triplets.size(); ++t)
-      {
-        if(m_triplets[t].col() == j &&
-           m_triplets[t].row() == i)
-          val += m_triplets[t].value();
-      }
-      return val;
-    }
-  }
-
-  /// \cond SKIP_IN_MANUAL
-  void assemble_matrix() const
-  {
-    m_matrix.setFromTriplets(m_triplets.begin(), m_triplets.end());
-    m_is_already_built = true;
-    m_triplets.clear(); // the matrix is built and will not be rebuilt
-  }
-  /// \endcond
+  /// Constructs an uninitialized matrix with `nr` rows and `nc` columns.
+  /// This is useful for dynamic-size matrices.
+  /// For fixed-size matrices, it is redundant to pass these parameters.
+  Eigen_matrix(std::size_t nr, std::size_t nc) : EigenType(nr, nc) { }
 
-  /// Return the internal matrix, with type `EigenType`.
-  const EigenType& eigen_object() const
-  {
-    if(!m_is_already_built)
-      assemble_matrix();
+  /// Constructs a matrix from an Eigen matrix.
+  Eigen_matrix(const EigenType& b) : EigenType(b) { }
 
-    // turns the matrix into compressed mode:
-    //  -> release some memory
-    //  -> required for some external solvers
-    m_matrix.makeCompressed();
-    return m_matrix;
-  }
+  /// Returns the matrix number of rows.
+  std::size_t number_of_rows() const { return this->rows(); }
+  /// Returns the matrix number of columns.
+  std::size_t number_of_columns() const { return this->cols(); }
 
-  /// Return the internal matrix, with type `EigenType`.
-  EigenType& eigen_object()
-  {
-    if(!m_is_already_built)
-      assemble_matrix();
+  /// Returns the value of the matrix at position (i,j).
+  FT operator()( std::size_t i , std::size_t j ) const { return EigenType::operator()(i,j); }
 
-    // turns the matrix into compressed mode:
-    //  -> release some memory
-    //  -> required for some external solvers
-    m_matrix.makeCompressed();
-    return m_matrix;
-  }
+  /// Writes access to a matrix coefficient: `a_ij` <- `val`.
+  void set(std::size_t i, std::size_t j, FT value) { this->coeffRef(i,j) = value; }
 
+  /// Returns the internal matrix, with type `EigenType`.
+  const EigenType& eigen_object() const { return static_cast<const EigenType&>(*this); }
 
 public:
-
   /// \cond SKIP_IN_MANUAL
-  friend Eigen_sparse_matrix
-  operator*(const T& c, const Eigen_sparse_matrix& M)
+  friend Eigen_matrix operator*(const FT c, const Eigen_matrix& M)
   {
-    return Eigen_sparse_matrix(c* M.eigen_object());
+    return Eigen_matrix(c * M.eigen_object());
   }
 
-
-  friend Eigen_sparse_matrix
-  operator+(const Eigen_sparse_matrix& M0, const Eigen_sparse_matrix& M1)
-  {
-    return Eigen_sparse_matrix(M0.eigen_object()+ M1.eigen_object());
-  }
-  /// \endcond
-
-
-  // Fields
-private:
-  mutable bool m_is_already_built;
-
-  typedef Eigen::Triplet<T,int> Triplet;
-  mutable std::vector<Triplet> m_triplets;
-
-  mutable EigenType m_matrix;
-
-  // Symmetric/hermitian?
-  bool m_is_symmetric;
-}; // Eigen_sparse_matrix
-
-
-/*!
-\ingroup PkgSolverInterfaceRef
-
-The class `Eigen_sparse_symmetric_matrix` is a wrapper around `Eigen` matrix type
-<a href="http://eigen.tuxfamily.org/dox/classEigen_1_1SparseMatrix.html">`Eigen::SparseMatrix` </a>
-
-Since the matrix is symmetric, only the lower triangle part is stored.
-
-\cgalModels `SparseLinearAlgebraTraits_d::Matrix`
-
-\tparam T Number type.
-
-\sa `CGAL::Eigen_vector<T>`
-\sa `CGAL::Eigen_sparse_matrix<T>`
-*/
-
-template<class T>
-struct Eigen_sparse_symmetric_matrix
-  : public Eigen_sparse_matrix<T>
-{
-  /// Create a square *symmetric* matrix initialized with zeros.
-  Eigen_sparse_symmetric_matrix(int dim)                  ///< Matrix dimension.
-    : Eigen_sparse_matrix<T>(dim, true /* symmetric */)
+  friend Eigen_matrix operator*(const Eigen_matrix& M0, const Eigen_matrix& M1)
   {
+    return Eigen_matrix(M0.eigen_object() * M1.eigen_object());
   }
 
-  /// Create a square *symmetric* matrix initialized with zeros.
-  ///
-  /// \pre rows == columns.
-  Eigen_sparse_symmetric_matrix(int rows,                 ///< Number of rows.
-                                int columns)              ///< Number of columns.
-    : Eigen_sparse_matrix<T>(rows, columns, true /* symmetric */)
+  friend Eigen_matrix operator+(const Eigen_matrix& M0, const Eigen_matrix& M1)
   {
+    return Eigen_matrix(M0.eigen_object() + M1.eigen_object());
   }
-};
-
-/*!
-\ingroup PkgSolverInterfaceRef
-
-The class `Eigen_matrix` is a wrapper around `Eigen` matrix type
-<a href="http://eigen.tuxfamily.org/dox/classEigen_1_1Matrix.html">`Eigen::Matrix`</a>.
-
-\cgalModels `SvdTraits::Matrix`
-
-\tparam T Number type.
-
-\sa `CGAL::Eigen_vector<T>`
-\sa `CGAL::Eigen_sparse_matrix<T>`
-\sa `CGAL::Eigen_sparse_symmetric_matrix<T>`
-*/
-template <class FT>
-struct Eigen_matrix
-  : public ::Eigen::Matrix<FT, ::Eigen::Dynamic, ::Eigen::Dynamic>
-{
-  /// The internal matrix type from \ref thirdpartyEigen "Eigen".
-  typedef ::Eigen::Matrix<FT, ::Eigen::Dynamic, ::Eigen::Dynamic> EigenType;
-
-  /// Construct a matrix with `nr` rows and `nc` columns.
-  Eigen_matrix(std::size_t nr, std::size_t nc) : EigenType(nr, nc) { }
-
-  /// Return the matrix number of rows.
-  std::size_t number_of_rows() const { return this->rows(); }
-  /// Return the matrix number of columns.
-  std::size_t number_of_columns() const { return this->cols(); }
-
-  /// Return the value of the matrix at position (i,j).
-  FT operator()( std::size_t i , std::size_t j ) const { return EigenType::operator()(i,j); }
-
-  /// Write access to a matrix coefficient: `a_ij` <- `val`.
-  void set(std::size_t i, std::size_t j, FT value) { this->coeffRef(i,j) = value; }
-
-  /// Return the internal matrix, with type `EigenType`.
-  const EigenType& eigen_object() const { return static_cast<const EigenType&>(*this); }
+  /// \endcond
 };
 
 } //namespace CGAL
 
-#endif // CGAL_EIGEN_MATRIX_H
+#endif // CGAL_SOLVER_INTERFACE_EIGEN_MATRIX_H
diff -pruN 5.0.3-1/include/CGAL/Eigen_solver_traits.h 5.1-1/include/CGAL/Eigen_solver_traits.h
--- 5.0.3-1/include/CGAL/Eigen_solver_traits.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Eigen_solver_traits.h	2020-09-08 12:15:39.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Solver_interface/include/CGAL/Eigen_solver_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Solver_interface/include/CGAL/Eigen_solver_traits.h $
 // $Id: Eigen_solver_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Eigen_sparse_matrix.h 5.1-1/include/CGAL/Eigen_sparse_matrix.h
--- 5.0.3-1/include/CGAL/Eigen_sparse_matrix.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Eigen_sparse_matrix.h	2020-09-08 12:15:39.000000000 +0000
@@ -0,0 +1,335 @@
+// Copyright (c) 2012  INRIA Bordeaux Sud-Ouest (France), All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Solver_interface/include/CGAL/Eigen_sparse_matrix.h $
+// $Id: Eigen_sparse_matrix.h 127bf54 2019-12-09T16:39:19+01:00 Mael Rouxel-Labbé
+// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Gael Guennebaud
+
+#ifndef CGAL_SOLVER_INTERFACE_EIGEN_SPARSE_MATRIX_H
+#define CGAL_SOLVER_INTERFACE_EIGEN_SPARSE_MATRIX_H
+
+#include <CGAL/basic.h> // include basic.h before testing #defines
+
+#include <Eigen/Sparse>
+
+namespace CGAL {
+
+/*!
+\ingroup PkgSolverInterfaceRef
+
+The class `Eigen_sparse_matrix` is a wrapper around `Eigen` matrix type
+<a href="http://eigen.tuxfamily.org/dox/classEigen_1_1SparseMatrix.html">`Eigen::SparseMatrix`</a>
+that represents general matrices, be they symmetric or not.
+
+\cgalModels `SparseLinearAlgebraTraits_d::Matrix`
+
+\tparam T Number type.
+
+\sa `CGAL::Eigen_vector<T>`
+\sa `CGAL::Eigen_matrix<T>`
+\sa `CGAL::Eigen_sparse_symmetric_matrix<T>`
+*/
+template<class T>
+struct Eigen_sparse_matrix
+{
+  // Public types
+public:
+  /// \name Types
+  /// @{
+
+  /// The internal matrix type from \ref thirdpartyEigen "Eigen".
+  typedef Eigen::SparseMatrix<T>      EigenType;
+
+  typedef T                           NT;
+  /// @}
+
+  Eigen_sparse_matrix(const EigenType& et)
+    : m_is_already_built(true), m_matrix(et), m_is_symmetric(false)
+  {}
+
+  // Public operations
+public:
+  Eigen_sparse_matrix() : m_is_already_built(false) { }
+
+  /// Create a square matrix initialized with zeros.
+  Eigen_sparse_matrix(std::size_t dim,            ///< Matrix dimension.
+                      bool is_symmetric = false)  ///< Symmetric/hermitian?
+    :
+      m_is_already_built(false),
+      m_matrix(static_cast<int>(dim), static_cast<int>(dim))
+  {
+    CGAL_precondition(dim > 0);
+
+    m_is_symmetric = is_symmetric;
+    m_triplets.reserve(dim); // reserve memory for a regular 3D grid
+  }
+
+  /// Create a square matrix initialized with zeros.
+  Eigen_sparse_matrix(int dim,                    ///< Matrix dimension.
+                      bool is_symmetric = false)  ///< Symmetric/hermitian?
+    : m_is_already_built(false),
+      m_matrix(dim, dim)
+  {
+    CGAL_precondition(dim > 0);
+
+    m_is_symmetric = is_symmetric;
+    // reserve memory for a regular 3D grid
+    m_triplets.reserve(dim);
+  }
+
+  /// Create a rectangular matrix initialized with zeros.
+  ///
+  /// \pre rows == columns if `is_symmetric` is true.
+  Eigen_sparse_matrix(std::size_t rows,          ///< Number of rows.
+                      std::size_t columns,       ///< Number of columns.
+                      bool is_symmetric = false) ///< Symmetric/hermitian?
+    : m_is_already_built(false),
+      m_matrix(static_cast<int>(rows), static_cast<int>(columns))
+  {
+    CGAL_precondition(rows > 0);
+    CGAL_precondition(columns > 0);
+    if(m_is_symmetric)
+    {
+      CGAL_precondition(rows == columns);
+    }
+
+    m_is_symmetric = is_symmetric;
+    // reserve memory for a regular 3D grid
+    m_triplets.reserve(rows);
+  }
+
+  void swap(Eigen_sparse_matrix& other)
+  {
+    std::swap(m_is_already_built, other.m_is_already_built);
+    std::swap(m_is_symmetric, other.m_is_symmetric);
+    m_matrix.swap(other.m_matrix);
+    m_triplets.swap(other.m_triplets);
+  }
+
+  /// Delete this object and the wrapped matrix.
+  ~Eigen_sparse_matrix() { }
+
+  /// Create a rectangular matrix initialized with zeros.
+  ///
+  /// \pre rows == columns if `is_symmetric` is true.
+  Eigen_sparse_matrix(int rows,                  ///< Number of rows.
+                      int columns,               ///< Number of columns.
+                      bool is_symmetric = false) ///< Symmetric/hermitian?
+    : m_is_already_built(false),
+      m_matrix(rows,columns)
+  {
+    CGAL_precondition(rows > 0);
+    CGAL_precondition(columns > 0);
+    if(is_symmetric)
+    {
+      CGAL_precondition(rows == columns);
+    }
+
+    m_is_symmetric = is_symmetric;
+    // reserve memory for a regular 3D grid
+    m_triplets.reserve(rows);
+  }
+
+  /// Return the matrix number of rows
+  int row_dimension() const { return static_cast<int>(m_matrix.rows()); }
+  /// Return the matrix number of columns
+  int column_dimension() const { return static_cast<int>(m_matrix.cols()); }
+
+  /// Write access to a matrix coefficient: a_ij <- val.
+  ///
+  /// Users can optimize calls to this function by setting 'new_coef' to `true`
+  /// if the coefficient does not already exist in the matrix.
+  ///
+  /// \warning For symmetric matrices, `Eigen_sparse_matrix` only stores the lower triangle
+  ///   and `set_coef()` does nothing if (i, j) belongs to the upper triangle.
+  ///
+  /// \pre 0 <= i < row_dimension().
+  /// \pre 0 <= j < column_dimension().
+  void set_coef(std::size_t i_, std::size_t j_, T val, bool new_coef = false)
+  {
+    int i = static_cast<int>(i_);
+    int j = static_cast<int>(j_);
+    CGAL_precondition(i < row_dimension());
+    CGAL_precondition(j < column_dimension());
+
+    if (m_is_symmetric && (j > i))
+      return;
+
+    if(m_is_already_built)
+    {
+      m_matrix.coeffRef(i,j) = val;
+    }
+    else
+    {
+      if(new_coef == false)
+      {
+        assemble_matrix();
+        m_matrix.coeffRef(i,j) = val;
+      }
+      else
+      {
+        m_triplets.push_back(Triplet(i,j,val));
+      }
+    }
+  }
+
+  /// Write access to a matrix coefficient: a_ij <- a_ij + val.
+  ///
+  /// \warning For symmetric matrices, Eigen_sparse_matrix only stores the lower triangle
+  /// `add_coef()` does nothing if (i, j) belongs to the upper triangle.
+  ///
+  /// \pre 0 <= i < row_dimension().
+  /// \pre 0 <= j < column_dimension().
+  void add_coef(std::size_t i_, std::size_t j_, T val)
+  {
+    int i = static_cast<int>(i_);
+    int j = static_cast<int>(j_);
+    if(m_is_symmetric && (j > i))
+      return;
+
+    if(m_is_already_built){
+      CGAL_precondition(i < row_dimension());
+      CGAL_precondition(j < column_dimension());
+      m_matrix.coeffRef(i,j) += val;
+    }else{
+      m_triplets.push_back(Triplet(i,j,val));
+    }
+  }
+
+  /// Read access to a matrix coefficient.
+  ///
+  /// \warning Complexity:
+  /// - O(log(n)) if the matrix is already built.
+  /// - O(n) if the matrix is not built.
+  /// `n` being the number of entries in the matrix.
+  ///
+  /// \pre 0 <= i < row_dimension().
+  /// \pre 0 <= j < column_dimension().
+  NT get_coef (std::size_t i_, std::size_t j_) const
+  {
+    int i = static_cast<int>(i_);
+    int j = static_cast<int>(j_);
+    CGAL_precondition(i < row_dimension());
+    CGAL_precondition(j < column_dimension());
+
+    if(m_is_symmetric && j > i)
+      std::swap(i, j);
+
+    if (m_is_already_built)
+      return m_matrix.coeffRef(i,j);
+    else
+    {
+      NT val = 0;
+      for(std::size_t t=0; t<m_triplets.size(); ++t)
+      {
+        if(m_triplets[t].col() == j &&
+           m_triplets[t].row() == i)
+          val += m_triplets[t].value();
+      }
+      return val;
+    }
+  }
+
+  /// \cond SKIP_IN_MANUAL
+  void assemble_matrix() const
+  {
+    m_matrix.setFromTriplets(m_triplets.begin(), m_triplets.end());
+    m_is_already_built = true;
+    m_triplets.clear(); // the matrix is built and will not be rebuilt
+  }
+  /// \endcond
+
+  /// Return the internal matrix, with type `EigenType`.
+  const EigenType& eigen_object() const
+  {
+    if(!m_is_already_built)
+      assemble_matrix();
+
+    // turns the matrix into compressed mode:
+    //  -> release some memory
+    //  -> required for some external solvers
+    m_matrix.makeCompressed();
+    return m_matrix;
+  }
+
+  /// Return the internal matrix, with type `EigenType`.
+  EigenType& eigen_object()
+  {
+    if(!m_is_already_built)
+      assemble_matrix();
+
+    // turns the matrix into compressed mode:
+    //  -> release some memory
+    //  -> required for some external solvers
+    m_matrix.makeCompressed();
+    return m_matrix;
+  }
+
+public:
+  /// \cond SKIP_IN_MANUAL
+  friend Eigen_sparse_matrix
+  operator*(const T& c, const Eigen_sparse_matrix& M)
+  {
+    return Eigen_sparse_matrix(c* M.eigen_object());
+  }
+
+  friend Eigen_sparse_matrix
+  operator+(const Eigen_sparse_matrix& M0, const Eigen_sparse_matrix& M1)
+  {
+    return Eigen_sparse_matrix(M0.eigen_object()+ M1.eigen_object());
+  }
+  /// \endcond
+
+  // Fields
+private:
+  mutable bool m_is_already_built;
+
+  typedef Eigen::Triplet<T,int> Triplet;
+  mutable std::vector<Triplet> m_triplets;
+
+  mutable EigenType m_matrix;
+
+  // Symmetric/hermitian?
+  bool m_is_symmetric;
+}; // Eigen_sparse_matrix
+
+
+/*!
+\ingroup PkgSolverInterfaceRef
+
+The class `Eigen_sparse_symmetric_matrix` is a wrapper around `Eigen` matrix type
+<a href="http://eigen.tuxfamily.org/dox/classEigen_1_1SparseMatrix.html">`Eigen::SparseMatrix` </a>
+
+Since the matrix is symmetric, only the lower triangle part is stored.
+
+\cgalModels `SparseLinearAlgebraTraits_d::Matrix`
+
+\tparam T Number type.
+
+\sa `CGAL::Eigen_vector<T>`
+\sa `CGAL::Eigen_sparse_matrix<T>`
+*/
+template<class T>
+struct Eigen_sparse_symmetric_matrix
+  : public Eigen_sparse_matrix<T>
+{
+  /// Create a square *symmetric* matrix initialized with zeros.
+  Eigen_sparse_symmetric_matrix(int dim)                  ///< Matrix dimension.
+    : Eigen_sparse_matrix<T>(dim, true /* symmetric */)
+  { }
+
+  /// Create a square *symmetric* matrix initialized with zeros.
+  ///
+  /// \pre rows == columns.
+  Eigen_sparse_symmetric_matrix(int rows,                 ///< Number of rows.
+                                int columns)              ///< Number of columns.
+    : Eigen_sparse_matrix<T>(rows, columns, true /* symmetric */)
+  { }
+};
+
+} //namespace CGAL
+
+#endif // CGAL_SOLVER_INTERFACE_EIGEN_SPARSE_MATRIX_H
diff -pruN 5.0.3-1/include/CGAL/Eigen_svd.h 5.1-1/include/CGAL/Eigen_svd.h
--- 5.0.3-1/include/CGAL/Eigen_svd.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Eigen_svd.h	2020-09-08 12:15:39.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Solver_interface/include/CGAL/Eigen_svd.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Solver_interface/include/CGAL/Eigen_svd.h $
 // $Id: Eigen_svd.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Eigen_vector.h 5.1-1/include/CGAL/Eigen_vector.h
--- 5.0.3-1/include/CGAL/Eigen_vector.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Eigen_vector.h	2020-09-08 12:15:39.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Solver_interface/include/CGAL/Eigen_vector.h $
-// $Id: Eigen_vector.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Solver_interface/include/CGAL/Eigen_vector.h $
+// $Id: Eigen_vector.h d6c1aba 2020-03-27T18:17:06+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Gael Guennebaud
@@ -33,9 +33,9 @@ which is a simple array of numbers.
 \sa `CGAL::Eigen_sparse_symmetric_matrix<T>`
 */
 
-template<class T>
+template<class T, int D = ::Eigen::Dynamic>
 class Eigen_vector
-  : public Eigen::Matrix<T, Eigen::Dynamic, 1>
+  : public ::Eigen::Matrix<T, D, 1>
 {
 // Public types
 public:
@@ -44,23 +44,23 @@ public:
   typedef T                                      NT;
 
   /// The internal vector type from \ref thirdpartyEigen "Eigen".
-  typedef Eigen::Matrix<T, Eigen::Dynamic, 1>    EigenType;
+  typedef ::Eigen::Matrix<T, D, 1>               EigenType;
   /// @}
 
 // Public operations
 public:
-  Eigen_vector<T>& operator=(const Eigen_vector<T>& other)
+  Eigen_vector& operator=(const Eigen_vector& other)
   {
     return static_cast<EigenType&>(*this) = other.eigen_object();
   }
 
-  Eigen_vector<T>& operator=(const EigenType& other)
+  Eigen_vector& operator=(const EigenType& other)
   {
-    return static_cast<Eigen_vector<T>&>(static_cast<EigenType&>(*this) = other);
+    return static_cast<Eigen_vector&>(static_cast<EigenType&>(*this) = other);
   }
-  Eigen_vector()
-    : EigenType()
-  {}
+
+  /// Constructs a null vector.
+  Eigen_vector() : EigenType() {}
 
   /// Create a vector initialized with zeros.
   Eigen_vector(std::size_t dimension)
diff -pruN 5.0.3-1/include/CGAL/enable_warnings.h 5.1-1/include/CGAL/enable_warnings.h
--- 5.0.3-1/include/CGAL/enable_warnings.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/enable_warnings.h	2020-09-08 12:14:54.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/enable_warnings.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/enable_warnings.h $
 // $Id: enable_warnings.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Enum_converter.h 5.1-1/include/CGAL/Enum_converter.h
--- 5.0.3-1/include/CGAL/Enum_converter.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Enum_converter.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Enum_converter.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Enum_converter.h $
 // $Id: Enum_converter.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/enum.h 5.1-1/include/CGAL/enum.h
--- 5.0.3-1/include/CGAL/enum.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/enum.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/enum.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/enum.h $
 // $Id: enum.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Env_default_diagram_1.h 5.1-1/include/CGAL/Env_default_diagram_1.h
--- 5.0.3-1/include/CGAL/Env_default_diagram_1.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Env_default_diagram_1.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Envelope_2/include/CGAL/Env_default_diagram_1.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Envelope_2/include/CGAL/Env_default_diagram_1.h $
 // $Id: Env_default_diagram_1.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Envelope_2/Env_divide_and_conquer_2.h 5.1-1/include/CGAL/Envelope_2/Env_divide_and_conquer_2.h
--- 5.0.3-1/include/CGAL/Envelope_2/Env_divide_and_conquer_2.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Envelope_2/Env_divide_and_conquer_2.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Envelope_2/include/CGAL/Envelope_2/Env_divide_and_conquer_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Envelope_2/include/CGAL/Envelope_2/Env_divide_and_conquer_2.h $
 // $Id: Env_divide_and_conquer_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Envelope_2/Env_divide_and_conquer_2_impl.h 5.1-1/include/CGAL/Envelope_2/Env_divide_and_conquer_2_impl.h
--- 5.0.3-1/include/CGAL/Envelope_2/Env_divide_and_conquer_2_impl.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Envelope_2/Env_divide_and_conquer_2_impl.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Envelope_2/include/CGAL/Envelope_2/Env_divide_and_conquer_2_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Envelope_2/include/CGAL/Envelope_2/Env_divide_and_conquer_2_impl.h $
 // $Id: Env_divide_and_conquer_2_impl.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/envelope_2.h 5.1-1/include/CGAL/envelope_2.h
--- 5.0.3-1/include/CGAL/envelope_2.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/envelope_2.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Envelope_2/include/CGAL/envelope_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Envelope_2/include/CGAL/envelope_2.h $
 // $Id: envelope_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Envelope_3/Envelope_base.h 5.1-1/include/CGAL/Envelope_3/Envelope_base.h
--- 5.0.3-1/include/CGAL/Envelope_3/Envelope_base.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Envelope_3/Envelope_base.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Envelope_3/include/CGAL/Envelope_3/Envelope_base.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Envelope_3/include/CGAL/Envelope_3/Envelope_base.h $
 // $Id: Envelope_base.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Envelope_3/Envelope_diagram_on_surface_2.h 5.1-1/include/CGAL/Envelope_3/Envelope_diagram_on_surface_2.h
--- 5.0.3-1/include/CGAL/Envelope_3/Envelope_diagram_on_surface_2.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Envelope_3/Envelope_diagram_on_surface_2.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Envelope_3/include/CGAL/Envelope_3/Envelope_diagram_on_surface_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Envelope_3/include/CGAL/Envelope_3/Envelope_diagram_on_surface_2.h $
 // $Id: Envelope_diagram_on_surface_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Envelope_3/Envelope_divide_and_conquer_3.h 5.1-1/include/CGAL/Envelope_3/Envelope_divide_and_conquer_3.h
--- 5.0.3-1/include/CGAL/Envelope_3/Envelope_divide_and_conquer_3.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Envelope_3/Envelope_divide_and_conquer_3.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Envelope_3/include/CGAL/Envelope_3/Envelope_divide_and_conquer_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Envelope_3/include/CGAL/Envelope_3/Envelope_divide_and_conquer_3.h $
 // $Id: Envelope_divide_and_conquer_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Envelope_3/Envelope_element_visitor_3.h 5.1-1/include/CGAL/Envelope_3/Envelope_element_visitor_3.h
--- 5.0.3-1/include/CGAL/Envelope_3/Envelope_element_visitor_3.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Envelope_3/Envelope_element_visitor_3.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Envelope_3/include/CGAL/Envelope_3/Envelope_element_visitor_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Envelope_3/include/CGAL/Envelope_3/Envelope_element_visitor_3.h $
 // $Id: Envelope_element_visitor_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Envelope_3/Envelope_overlay_2.h 5.1-1/include/CGAL/Envelope_3/Envelope_overlay_2.h
--- 5.0.3-1/include/CGAL/Envelope_3/Envelope_overlay_2.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Envelope_3/Envelope_overlay_2.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Envelope_3/include/CGAL/Envelope_3/Envelope_overlay_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Envelope_3/include/CGAL/Envelope_3/Envelope_overlay_2.h $
 // $Id: Envelope_overlay_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Envelope_3/Envelope_overlay_functor.h 5.1-1/include/CGAL/Envelope_3/Envelope_overlay_functor.h
--- 5.0.3-1/include/CGAL/Envelope_3/Envelope_overlay_functor.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Envelope_3/Envelope_overlay_functor.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Envelope_3/include/CGAL/Envelope_3/Envelope_overlay_functor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Envelope_3/include/CGAL/Envelope_3/Envelope_overlay_functor.h $
 // $Id: Envelope_overlay_functor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Envelope_3/Envelope_pm_dcel.h 5.1-1/include/CGAL/Envelope_3/Envelope_pm_dcel.h
--- 5.0.3-1/include/CGAL/Envelope_3/Envelope_pm_dcel.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Envelope_3/Envelope_pm_dcel.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Envelope_3/include/CGAL/Envelope_3/Envelope_pm_dcel.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Envelope_3/include/CGAL/Envelope_3/Envelope_pm_dcel.h $
 // $Id: Envelope_pm_dcel.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Envelope_3/Env_plane_traits_3_functions.h 5.1-1/include/CGAL/Envelope_3/Env_plane_traits_3_functions.h
--- 5.0.3-1/include/CGAL/Envelope_3/Env_plane_traits_3_functions.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Envelope_3/Env_plane_traits_3_functions.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Envelope_3/include/CGAL/Envelope_3/Env_plane_traits_3_functions.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Envelope_3/include/CGAL/Envelope_3/Env_plane_traits_3_functions.h $
 // $Id: Env_plane_traits_3_functions.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Envelope_3/set_dividors.h 5.1-1/include/CGAL/Envelope_3/set_dividors.h
--- 5.0.3-1/include/CGAL/Envelope_3/set_dividors.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Envelope_3/set_dividors.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Envelope_3/include/CGAL/Envelope_3/set_dividors.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Envelope_3/include/CGAL/Envelope_3/set_dividors.h $
 // $Id: set_dividors.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/envelope_3.h 5.1-1/include/CGAL/envelope_3.h
--- 5.0.3-1/include/CGAL/envelope_3.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/envelope_3.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Envelope_3/include/CGAL/envelope_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Envelope_3/include/CGAL/envelope_3.h $
 // $Id: envelope_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Envelope_diagram_1.h 5.1-1/include/CGAL/Envelope_diagram_1.h
--- 5.0.3-1/include/CGAL/Envelope_diagram_1.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Envelope_diagram_1.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Envelope_2/include/CGAL/Envelope_diagram_1.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Envelope_2/include/CGAL/Envelope_diagram_1.h $
 // $Id: Envelope_diagram_1.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Env_plane_traits_3.h 5.1-1/include/CGAL/Env_plane_traits_3.h
--- 5.0.3-1/include/CGAL/Env_plane_traits_3.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Env_plane_traits_3.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Envelope_3/include/CGAL/Env_plane_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Envelope_3/include/CGAL/Env_plane_traits_3.h $
 // $Id: Env_plane_traits_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Env_sphere_traits_3.h 5.1-1/include/CGAL/Env_sphere_traits_3.h
--- 5.0.3-1/include/CGAL/Env_sphere_traits_3.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Env_sphere_traits_3.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Envelope_3/include/CGAL/Env_sphere_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Envelope_3/include/CGAL/Env_sphere_traits_3.h $
 // $Id: Env_sphere_traits_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Env_surface_data_traits_3.h 5.1-1/include/CGAL/Env_surface_data_traits_3.h
--- 5.0.3-1/include/CGAL/Env_surface_data_traits_3.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Env_surface_data_traits_3.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Envelope_3/include/CGAL/Env_surface_data_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Envelope_3/include/CGAL/Env_surface_data_traits_3.h $
 // $Id: Env_surface_data_traits_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Env_tracing_traits_3.h 5.1-1/include/CGAL/Env_tracing_traits_3.h
--- 5.0.3-1/include/CGAL/Env_tracing_traits_3.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Env_tracing_traits_3.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Envelope_3/include/CGAL/Env_tracing_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Envelope_3/include/CGAL/Env_tracing_traits_3.h $
 // $Id: Env_tracing_traits_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Env_triangle_traits_3.h 5.1-1/include/CGAL/Env_triangle_traits_3.h
--- 5.0.3-1/include/CGAL/Env_triangle_traits_3.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Env_triangle_traits_3.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Envelope_3/include/CGAL/Env_triangle_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Envelope_3/include/CGAL/Env_triangle_traits_3.h $
 // $Id: Env_triangle_traits_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Epeck_d.h 5.1-1/include/CGAL/Epeck_d.h
--- 5.0.3-1/include/CGAL/Epeck_d.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Epeck_d.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/Epeck_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/Epeck_d.h $
 // $Id: Epeck_d.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Epic_converter.h 5.1-1/include/CGAL/Epic_converter.h
--- 5.0.3-1/include/CGAL/Epic_converter.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Epic_converter.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/include/CGAL/Epic_converter.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/Epic_converter.h $
 // $Id: Epic_converter.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Epick_d.h 5.1-1/include/CGAL/Epick_d.h
--- 5.0.3-1/include/CGAL/Epick_d.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Epick_d.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/Epick_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/Epick_d.h $
 // $Id: Epick_d.h 7929c7d 2020-04-08T14:42:47+02:00 Marc Glisse
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/estimate_scale.h 5.1-1/include/CGAL/estimate_scale.h
--- 5.0.3-1/include/CGAL/estimate_scale.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/estimate_scale.h	2020-09-08 12:15:22.000000000 +0000
@@ -4,8 +4,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_processing_3/include/CGAL/estimate_scale.h $
-// $Id: estimate_scale.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/estimate_scale.h $
+// $Id: estimate_scale.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s) : Simon Giraudot
@@ -452,22 +452,30 @@ public:
    \param points input point range.
    \param queries range of locations where scale must be estimated
    \param output iterator to store the computed scales
-   \param np optional sequence of \ref psp_namedparameters "Named Parameters" among the ones listed below.
+   \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
    \cgalNamedParamsBegin
-     \cgalParamBegin{point_map} a model of `ReadablePropertyMap` with
-     value type `geom_traits::Point_3` (or `geom_traits::Point_2`).
-     If this parameter is omitted,
-     `CGAL::Identity_property_map<geom_traits::Point_3>` (or
-     `CGAL::Identity_property_map<geom_traits::Point_2>`) is
-     used.\cgalParamEnd
-     \cgalParamBegin{query_point_map} a model of `ReadablePropertyMap` with
-     value type `geom_traits::Point_3` (or `geom_traits::Point_2`).
-     If this parameter is omitted,
-     `CGAL::Identity_property_map<geom_traits::Point_3>` (or
-     `CGAL::Identity_property_map<geom_traits::Point_2>`) is
-     used.\cgalParamEnd
-     \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+     \cgalParamNBegin{point_map}
+       \cgalParamDescription{a property map associating points to the elements of the point set `points`}
+       \cgalParamType{a model of `ReadablePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange` and whose value type is `geom_traits::Point_3`
+                      (or `geom_traits::Point_2`)}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>` (or
+                         `CGAL::Identity_property_map<geom_traits::Point_2>`)}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{query_point_map}
+       \cgalParamDescription{the property map containing the points associated to the elements of the point range `queries`}
+       \cgalParamType{a model of `ReadablePropertyMap` with value type `geom_traits::Point_3` (or `geom_traits::Point_2`)}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>` (or
+                         `CGAL::Identity_property_map<geom_traits::Point_2>`)}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{geom_traits}
+       \cgalParamDescription{an instance of a geometric traits class}
+       \cgalParamType{a model of `Kernel`}
+       \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+     \cgalParamNEnd
    \cgalNamedParamsEnd
 
    \note This function accepts both 2D and 3D points, but sample
@@ -488,14 +496,14 @@ estimate_local_k_neighbor_scales(
   using parameters::choose_parameter;
   using parameters::get_parameter;
 
-  typedef typename Point_set_processing_3::GetPointMap<PointRange, NamedParameters>::const_type PointMap;
+  typedef typename CGAL::GetPointMap<PointRange, NamedParameters>::const_type PointMap;
   typedef typename Point_set_processing_3::GetQueryPointMap<QueryPointRange, NamedParameters>::const_type QueryPointMap;
   typedef typename Point_set_processing_3::GetK<PointRange, NamedParameters>::Kernel Kernel;
 
   typedef typename boost::property_traits<PointMap>::value_type Point_d;
 
-  PointMap point_map = choose_parameter(get_parameter(np, internal_np::point_map), PointMap());
-  QueryPointMap query_point_map = choose_parameter(get_parameter(np, internal_np::query_point_map), QueryPointMap());
+  PointMap point_map = choose_parameter<PointMap>(get_parameter(np, internal_np::point_map));
+  QueryPointMap query_point_map = choose_parameter<QueryPointMap>(get_parameter(np, internal_np::query_point_map));
 
   // Build multi-scale KD-tree
   internal::Quick_multiscale_approximate_knn_distance<Kernel, Point_d> kdtree (points.begin(),
@@ -539,16 +547,23 @@ estimate_local_k_neighbor_scales(
    its iterator is the key type of the named parameter `point_map`.
 
    \param points input point range.
-   \param np optional sequence of \ref psp_namedparameters "Named Parameters" among the ones listed below.
+   \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
    \cgalNamedParamsBegin
-     \cgalParamBegin{point_map} a model of `ReadablePropertyMap` with
-     value type `geom_traits::Point_3` (or `geom_traits::Point_2`).
-     If this parameter is omitted,
-     `CGAL::Identity_property_map<geom_traits::Point_3>` (or
-     `CGAL::Identity_property_map<geom_traits::Point_2>`) is
-     used.\cgalParamEnd
-     \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+     \cgalParamNBegin{point_map}
+       \cgalParamDescription{a property map associating points to the elements of the point set `points`}
+       \cgalParamType{a model of `ReadablePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange` and whose value type is `geom_traits::Point_3`
+                      (or `geom_traits::Point_2`)}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>` (or
+                         `CGAL::Identity_property_map<geom_traits::Point_2>`)}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{geom_traits}
+       \cgalParamDescription{an instance of a geometric traits class}
+       \cgalParamType{a model of `Kernel`}
+       \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+     \cgalParamNEnd
    \cgalNamedParamsEnd
 
    \note This function accepts both 2D and 3D points.
@@ -566,8 +581,8 @@ estimate_global_k_neighbor_scale(
   using parameters::choose_parameter;
   using parameters::get_parameter;
 
-  typedef typename Point_set_processing_3::GetPointMap<PointRange, NamedParameters>::const_type PointMap;
-  PointMap point_map = choose_parameter(get_parameter(np, internal_np::point_map), PointMap());
+  typedef typename CGAL::GetPointMap<PointRange, NamedParameters>::const_type PointMap;
+  PointMap point_map = choose_parameter<PointMap>(get_parameter(np, internal_np::point_map));
   std::vector<std::size_t> scales;
   estimate_local_k_neighbor_scales (points, points, std::back_inserter (scales), np.query_point_map(point_map));
   std::sort (scales.begin(), scales.end());
@@ -605,22 +620,30 @@ estimate_global_k_neighbor_scale(const P
    \param points input point range.
    \param queries range of locations where scale must be estimated
    \param output iterator to store the computed scales
-   \param np optional sequence of \ref psp_namedparameters "Named Parameters" among the ones listed below.
+   \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
    \cgalNamedParamsBegin
-     \cgalParamBegin{point_map} a model of `ReadablePropertyMap` with
-     value type `geom_traits::Point_3` (or `geom_traits::Point_2`).
-     If this parameter is omitted,
-     `CGAL::Identity_property_map<geom_traits::Point_3>` (or
-     `CGAL::Identity_property_map<geom_traits::Point_2>`) is
-     used.\cgalParamEnd
-     \cgalParamBegin{query_point_map} a model of `ReadablePropertyMap` with
-     value type `geom_traits::Point_3` (or `geom_traits::Point_2`).
-     If this parameter is omitted,
-     `CGAL::Identity_property_map<geom_traits::Point_3>` (or
-     `CGAL::Identity_property_map<geom_traits::Point_2>`) is
-     used.\cgalParamEnd
-     \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+     \cgalParamNBegin{point_map}
+       \cgalParamDescription{a property map associating points to the elements of the point set `points`}
+       \cgalParamType{a model of `ReadablePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange` and whose value type is `geom_traits::Point_3`
+                      (or `geom_traits::Point_2`)}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>` (or
+                         `CGAL::Identity_property_map<geom_traits::Point_2>`)}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{query_point_map}
+       \cgalParamDescription{the property map containing the points associated to the elements of the point range `queries`}
+       \cgalParamType{a model of `ReadablePropertyMap` with value type `geom_traits::Point_3` (or `geom_traits::Point_2`)}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>` (or
+                         `CGAL::Identity_property_map<geom_traits::Point_2>`)}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{geom_traits}
+       \cgalParamDescription{an instance of a geometric traits class}
+       \cgalParamType{a model of `Kernel`}
+       \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+     \cgalParamNEnd
    \cgalNamedParamsEnd
 
    \note This function accepts both 2D and 3D points, but sample
@@ -641,14 +664,14 @@ estimate_local_range_scales(
   using parameters::choose_parameter;
   using parameters::get_parameter;
 
-  typedef typename Point_set_processing_3::GetPointMap<PointRange, NamedParameters>::const_type PointMap;
+  typedef typename CGAL::GetPointMap<PointRange, NamedParameters>::const_type PointMap;
   typedef typename Point_set_processing_3::GetQueryPointMap<QueryPointRange, NamedParameters>::const_type QueryPointMap;
   typedef typename Point_set_processing_3::GetK<PointRange, NamedParameters>::Kernel Kernel;
 
   typedef typename boost::property_traits<PointMap>::value_type Point_d;
 
-  PointMap point_map = choose_parameter(get_parameter(np, internal_np::point_map), PointMap());
-  QueryPointMap query_point_map = choose_parameter(get_parameter(np, internal_np::query_point_map), QueryPointMap());
+  PointMap point_map = choose_parameter<PointMap>(get_parameter(np, internal_np::point_map));
+  QueryPointMap query_point_map = choose_parameter<QueryPointMap>(get_parameter(np, internal_np::query_point_map));
 
   // Build multi-scale KD-tree
   internal::Quick_multiscale_approximate_knn_distance<Kernel, Point_d> kdtree (points.begin(),
@@ -692,22 +715,29 @@ estimate_local_range_scales(
    its iterator is the key type of the named parameter `point_map`.
 
    \param points input point range.
-   \param np optional sequence of \ref psp_namedparameters "Named Parameters" among the ones listed below.
+   \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
    \cgalNamedParamsBegin
-     \cgalParamBegin{point_map} a model of `ReadablePropertyMap` with
-     value type `geom_traits::Point_3` (or `geom_traits::Point_2`).
-     If this parameter is omitted,
-     `CGAL::Identity_property_map<geom_traits::Point_3>` (or
-     `CGAL::Identity_property_map<geom_traits::Point_2>`) is
-     used.\cgalParamEnd
-     \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+     \cgalParamNBegin{point_map}
+       \cgalParamDescription{a property map associating points to the elements of the point set `points`}
+       \cgalParamType{a model of `ReadablePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange` and whose value type is `geom_traits::Point_3`
+                      (or `geom_traits::Point_2`)}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>` (or
+                         `CGAL::Identity_property_map<geom_traits::Point_2>`)}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{geom_traits}
+       \cgalParamDescription{an instance of a geometric traits class}
+       \cgalParamType{a model of `Kernel`}
+       \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+     \cgalParamNEnd
    \cgalNamedParamsEnd
 
    \note This function accepts both 2D and 3D points.
 
    \return The estimated scale in the range sense. The return type `FT` is a number type. It is
-   either deduced from the `geom_traits` \ref psp_namedparameters "Named Parameters" if provided,
+   either deduced from the `geom_traits` \ref bgl_namedparameters "Named Parameters" if provided,
    or the geometric traits class deduced from the point property map
    of `points`.
 */
@@ -727,8 +757,8 @@ estimate_global_range_scale(
   using parameters::get_parameter;
 
   std::vector<double> scales;
-  typedef typename Point_set_processing_3::GetPointMap<PointRange, NamedParameters>::const_type PointMap;
-  PointMap point_map = choose_parameter(get_parameter(np, internal_np::point_map), PointMap());
+  typedef typename CGAL::GetPointMap<PointRange, NamedParameters>::const_type PointMap;
+  PointMap point_map = choose_parameter<PointMap>(get_parameter(np, internal_np::point_map));
   estimate_local_range_scales (points, points, std::back_inserter (scales), np.query_point_map(point_map));
   std::sort (scales.begin(), scales.end());
   return std::sqrt (scales[scales.size() / 2]);
diff -pruN 5.0.3-1/include/CGAL/Euclidean_distance.h 5.1-1/include/CGAL/Euclidean_distance.h
--- 5.0.3-1/include/CGAL/Euclidean_distance.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Euclidean_distance.h	2020-09-08 12:15:39.000000000 +0000
@@ -3,12 +3,13 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_searching/include/CGAL/Euclidean_distance.h $
-// $Id: Euclidean_distance.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_searching/include/CGAL/Euclidean_distance.h $
+// $Id: Euclidean_distance.h a774b36 2020-03-26T19:41:07+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
 // Author(s)     : Hans Tangelder (<hanst@cs.uu.nl>)
+//                 Clement Jamin (clement.jamin.pro@gmail.com)
 
 
 #ifndef CGAL_EUCLIDEAN_DISTANCE_H
@@ -21,6 +22,7 @@
 #include <CGAL/number_utils.h>
 #include <CGAL/internal/Get_dimension_tag.h>
 #include <vector>
+#include <iterator>
 
 namespace CGAL {
 
@@ -52,147 +54,201 @@ namespace CGAL {
     // default constructor
     Euclidean_distance(const SearchTraits& traits_=SearchTraits()):traits(traits_) {}
 
-
-    inline FT transformed_distance(const Query_item& q, const Point_d& p) const {
-        return transformed_distance(q,p, D());
+    inline FT transformed_distance(const Query_item& q, const Point_d& p) const
+    {
+      typename SearchTraits::Construct_cartesian_const_iterator_d construct_it = traits.construct_cartesian_const_iterator_d_object();
+      typename SearchTraits::Cartesian_const_iterator_d p_begin = construct_it(p), p_end = construct_it(p, 0);
+      return transformed_distance_from_coordinates(q, p_begin, p_end);
     }
 
-    //Dynamic version for runtime dimension
-    inline FT transformed_distance(const Query_item& q, const Point_d& p, Dynamic_dimension_tag) const {
-        FT distance = FT(0);
-        typename SearchTraits::Construct_cartesian_const_iterator_d construct_it=traits.construct_cartesian_const_iterator_d_object();
-        typename SearchTraits::Cartesian_const_iterator_d qit = construct_it(q),
-        qe = construct_it(q,1), pit = construct_it(p);
-        for(; qit != qe; qit++, pit++){
-            distance += ((*qit)-(*pit))*((*qit)-(*pit));
-        }
-        return distance;
+    template <typename Coord_iterator>
+    inline FT transformed_distance_from_coordinates(const Query_item& q,
+      Coord_iterator it_coord_begin, Coord_iterator it_coord_end) const
+    {
+      return transformed_distance_from_coordinates(q, it_coord_begin, it_coord_end, D());
     }
 
-    //Generic version for DIM > 3
-    template < int DIM >
-    inline FT transformed_distance(const Query_item& q, const Point_d& p, Dimension_tag<DIM>) const {
-        FT distance = FT(0);
-        typename SearchTraits::Construct_cartesian_const_iterator_d construct_it=traits.construct_cartesian_const_iterator_d_object();
-        typename SearchTraits::Cartesian_const_iterator_d qit = construct_it(q),
-          qe = construct_it(q,1), pit = construct_it(p);
-        for(; qit != qe; qit++, pit++){
-          distance += ((*qit)-(*pit))*((*qit)-(*pit));
-        }
-        return distance;
+    // Static dim = 2 loop unrolled
+    template <typename Coord_iterator>
+    inline FT transformed_distance_from_coordinates(const Query_item& q,
+                                                    Coord_iterator it_coord_begin, Coord_iterator /*unused*/,
+                                                    Dimension_tag<2>) const
+    {
+      typename SearchTraits::Construct_cartesian_const_iterator_d construct_it = traits.construct_cartesian_const_iterator_d_object();
+      typename SearchTraits::Cartesian_const_iterator_d qit = construct_it(q);
+      FT distance = square(*qit - *it_coord_begin);
+      qit++; it_coord_begin++;
+      distance += square(*qit - *it_coord_begin);
+      return distance;
     }
 
-    //DIM = 2 loop unrolled
-    inline FT transformed_distance(const Query_item& q, const Point_d& p, Dimension_tag<2> ) const {
-        typename SearchTraits::Construct_cartesian_const_iterator_d construct_it=traits.construct_cartesian_const_iterator_d_object();
-        typename SearchTraits::Cartesian_const_iterator_d qit = construct_it(q),pit = construct_it(p);
-        FT distance = square(*qit - *pit);
-        qit++;pit++;
-        distance += square(*qit - *pit);
-        return distance;
-    }
-
-    //DIM = 3 loop unrolled
-    inline FT transformed_distance(const Query_item& q, const Point_d& p, Dimension_tag<3> ) const {
-        typename SearchTraits::Construct_cartesian_const_iterator_d construct_it=traits.construct_cartesian_const_iterator_d_object();
-        typename SearchTraits::Cartesian_const_iterator_d qit = construct_it(q),pit = construct_it(p);
-        FT distance = square(*qit - *pit);
-        qit++;pit++;
-        distance += square(*qit - *pit);
-        qit++;pit++;
-        distance += square(*qit - *pit);
-        return distance;
+    // Static dim = 3 loop unrolled
+    template <typename Coord_iterator>
+    inline FT transformed_distance_from_coordinates(const Query_item& q,
+                                                    Coord_iterator it_coord_begin, Coord_iterator /*unused*/,
+                                                    Dimension_tag<3>) const
+    {
+      typename SearchTraits::Construct_cartesian_const_iterator_d construct_it = traits.construct_cartesian_const_iterator_d_object();
+      typename SearchTraits::Cartesian_const_iterator_d qit = construct_it(q);
+      FT distance = square(*qit - *it_coord_begin);
+      qit++; it_coord_begin++;
+      distance += square(*qit - *it_coord_begin);
+      qit++; it_coord_begin++;
+      distance += square(*qit - *it_coord_begin);
+      return distance;
     }
 
+    // Other cases: static dim > 3 or dynamic dim
+    template <typename Coord_iterator, typename Dim>
+    inline FT transformed_distance_from_coordinates(const Query_item& q,
+                                                    Coord_iterator it_coord_begin, Coord_iterator /*unused*/,
+                                                    Dim) const
+    {
+      FT distance = FT(0);
+      typename SearchTraits::Construct_cartesian_const_iterator_d construct_it = traits.construct_cartesian_const_iterator_d_object();
+      typename SearchTraits::Cartesian_const_iterator_d qit = construct_it(q), qe = construct_it(q, 1);
+      for (; qit != qe; ++qit, ++it_coord_begin)
+      {
+        FT diff = (*qit) - (*it_coord_begin);
+        distance += diff*diff;
+      }
+      return distance;
+    }
 
+    // During the computation, if the partially-computed distance `pcd` gets greater or equal
+    // to `stop_if_geq_to_this`, the computation is stopped and `pcd` is returned
+    template <typename Coord_iterator>
+    inline FT interruptible_transformed_distance(const Query_item& q,
+                                                 Coord_iterator it_coord_begin, Coord_iterator /*unused*/,
+                                                 FT stop_if_geq_to_this) const
+    {
+      FT distance = FT(0);
+      typename SearchTraits::Construct_cartesian_const_iterator_d construct_it = traits.construct_cartesian_const_iterator_d_object();
+      typename SearchTraits::Cartesian_const_iterator_d qit = construct_it(q), qe = construct_it(q, 1);
+      if (std::distance(qit, qe) >= 6)
+      {
+        // Every 4 coordinates, the current partially-computed distance
+        // is compared to stop_if_geq_to_this
+        // Note: the concept SearchTraits specifies that Cartesian_const_iterator_d
+        //       must be a random-access iterator
+        typename SearchTraits::Cartesian_const_iterator_d qe_minus_5 = qe;
+        std::advance(qe_minus_5, -5);
+        for (;;)
+        {
+          FT diff = (*qit) - (*it_coord_begin);
+          distance += diff*diff;
+          ++qit; ++it_coord_begin;
+          diff = (*qit) - (*it_coord_begin);
+          distance += diff*diff;
+          ++qit; ++it_coord_begin;
+          diff = (*qit) - (*it_coord_begin);
+          distance += diff*diff;
+          ++qit; ++it_coord_begin;
+          diff = (*qit) - (*it_coord_begin);
+          distance += diff*diff;
+          ++qit, ++it_coord_begin;
+
+          if (distance >= stop_if_geq_to_this)
+            return distance;
+
+          if (std::distance(qe_minus_5, qit) >= 0)
+            break;
+        }
+      }
+      for (; qit != qe; ++qit, ++it_coord_begin)
+      {
+        FT diff = (*qit) - (*it_coord_begin);
+        distance += diff*diff;
+      }
+      return distance;
+    }
 
+    inline FT min_distance_to_rectangle(const Query_item& q,
+      const Kd_tree_rectangle<FT, D>& r) const {
+      FT distance = FT(0);
+      typename SearchTraits::Construct_cartesian_const_iterator_d construct_it = traits.construct_cartesian_const_iterator_d_object();
+      typename SearchTraits::Cartesian_const_iterator_d qit = construct_it(q),
+        qe = construct_it(q, 1);
+      for (unsigned int i = 0; qit != qe; i++, qit++) {
+        if ((*qit) < r.min_coord(i))
+          distance +=
+          (r.min_coord(i) - (*qit))*(r.min_coord(i) - (*qit));
+        else if ((*qit) > r.max_coord(i))
+          distance +=
+          ((*qit) - r.max_coord(i))*((*qit) - r.max_coord(i));
 
-        inline FT min_distance_to_rectangle(const Query_item& q,
-                                            const Kd_tree_rectangle<FT,D>& r) const {
-                FT distance = FT(0);
-                typename SearchTraits::Construct_cartesian_const_iterator_d construct_it=traits.construct_cartesian_const_iterator_d_object();
-                typename SearchTraits::Cartesian_const_iterator_d qit = construct_it(q),
-                  qe = construct_it(q,1);
-                for(unsigned int i = 0;qit != qe; i++, qit++){
-                  if((*qit) < r.min_coord(i))
-                                distance +=
-                                (r.min_coord(i)-(*qit))*(r.min_coord(i)-(*qit));
-                  else if ((*qit) > r.max_coord(i))
-                                distance +=
-                                ((*qit)-r.max_coord(i))*((*qit)-r.max_coord(i));
+      }
+      return distance;
+    }
 
-                }
-                return distance;
+    inline FT min_distance_to_rectangle(const Query_item& q,
+      const Kd_tree_rectangle<FT, D>& r, std::vector<FT>& dists) const {
+      FT distance = FT(0);
+      typename SearchTraits::Construct_cartesian_const_iterator_d construct_it = traits.construct_cartesian_const_iterator_d_object();
+      typename SearchTraits::Cartesian_const_iterator_d qit = construct_it(q),
+        qe = construct_it(q, 1);
+      for (unsigned int i = 0; qit != qe; i++, qit++) {
+        if ((*qit) < r.min_coord(i)) {
+          dists[i] = (r.min_coord(i) - (*qit));
+          distance += dists[i] * dists[i];
+        }
+        else if ((*qit) > r.max_coord(i)) {
+          dists[i] = ((*qit) - r.max_coord(i));
+          distance += dists[i] * dists[i];
         }
 
-        inline FT min_distance_to_rectangle(const Query_item& q,
-                                            const Kd_tree_rectangle<FT,D>& r,std::vector<FT>& dists) const {
-                FT distance = FT(0);
-                typename SearchTraits::Construct_cartesian_const_iterator_d construct_it=traits.construct_cartesian_const_iterator_d_object();
-                typename SearchTraits::Cartesian_const_iterator_d qit = construct_it(q),
-                  qe = construct_it(q,1);
-                for(unsigned int i = 0;qit != qe; i++, qit++){
-                  if((*qit) < r.min_coord(i)){
-                                dists[i] = (r.min_coord(i)-(*qit));
-                                distance += dists[i] * dists[i];
-                  }
-                  else if ((*qit) > r.max_coord(i)){
-                                dists[i] = ((*qit)-r.max_coord(i));
-                                distance +=  dists[i] * dists[i];
-                  }
-
-                }
-                return distance;
-        }
+      }
+      return distance;
+    }
 
-        inline FT max_distance_to_rectangle(const Query_item& q,
-                                             const Kd_tree_rectangle<FT,D>& r) const {
-                FT distance=FT(0);
-                typename SearchTraits::Construct_cartesian_const_iterator_d construct_it=traits.construct_cartesian_const_iterator_d_object();
-                typename SearchTraits::Cartesian_const_iterator_d qit = construct_it(q),
-                  qe = construct_it(q,1);
-                for(unsigned int i = 0;qit != qe; i++, qit++){
-                                if ((*qit) <= (r.min_coord(i)+r.max_coord(i))/FT(2.0))
-                                        distance += (r.max_coord(i)-(*qit))*(r.max_coord(i)-(*qit));
-                                else
-                                        distance += ((*qit)-r.min_coord(i))*((*qit)-r.min_coord(i));
-                };
-                return distance;
-        }
+    inline FT max_distance_to_rectangle(const Query_item& q,
+      const Kd_tree_rectangle<FT, D>& r) const {
+      FT distance = FT(0);
+      typename SearchTraits::Construct_cartesian_const_iterator_d construct_it = traits.construct_cartesian_const_iterator_d_object();
+      typename SearchTraits::Cartesian_const_iterator_d qit = construct_it(q),
+        qe = construct_it(q, 1);
+      for (unsigned int i = 0; qit != qe; i++, qit++) {
+        if ((*qit) <= (r.min_coord(i) + r.max_coord(i)) / FT(2.0))
+          distance += (r.max_coord(i) - (*qit))*(r.max_coord(i) - (*qit));
+        else
+          distance += ((*qit) - r.min_coord(i))*((*qit) - r.min_coord(i));
+      };
+      return distance;
+    }
 
-        inline FT max_distance_to_rectangle(const Query_item& q,
-                                             const Kd_tree_rectangle<FT,D>& r,std::vector<FT>& dists ) const {
-                FT distance=FT(0);
-                typename SearchTraits::Construct_cartesian_const_iterator_d construct_it=traits.construct_cartesian_const_iterator_d_object();
-                typename SearchTraits::Cartesian_const_iterator_d qit = construct_it(q),
-                  qe = construct_it(q,1);
-                for(unsigned int i = 0;qit != qe; i++, qit++){
-                                if ((*qit) <= (r.min_coord(i)+r.max_coord(i))/FT(2.0)){
-                                        dists[i] = (r.max_coord(i)-(*qit));
-                                        distance += dists[i] * dists[i];
-                                }
-                                else{
-                                        dists[i] = ((*qit)-r.min_coord(i));
-                                        distance += dists[i] * dists[i];
-                                }
-                };
-                return distance;
+    inline FT max_distance_to_rectangle(const Query_item& q,
+      const Kd_tree_rectangle<FT, D>& r, std::vector<FT>& dists) const {
+      FT distance = FT(0);
+      typename SearchTraits::Construct_cartesian_const_iterator_d construct_it = traits.construct_cartesian_const_iterator_d_object();
+      typename SearchTraits::Cartesian_const_iterator_d qit = construct_it(q),
+        qe = construct_it(q, 1);
+      for (unsigned int i = 0; qit != qe; i++, qit++) {
+        if ((*qit) <= (r.min_coord(i) + r.max_coord(i)) / FT(2.0)) {
+          dists[i] = (r.max_coord(i) - (*qit));
+          distance += dists[i] * dists[i];
+        }
+        else {
+          dists[i] = ((*qit) - r.min_coord(i));
+          distance += dists[i] * dists[i];
         }
+      };
+      return distance;
+    }
 
-        inline FT new_distance(FT dist, FT old_off, FT new_off,
-                               int /* cutting_dimension */)  const {
+    inline FT new_distance(FT dist, FT old_off, FT new_off,
+      int /* cutting_dimension */)  const {
 
-                FT new_dist = dist + (new_off*new_off - old_off*old_off);
-                return new_dist;
-        }
+      FT new_dist = dist + (new_off*new_off - old_off*old_off);
+      return new_dist;
+    }
 
-  inline FT transformed_distance(FT d) const {
-                return d*d;
-        }
+    inline FT transformed_distance(FT d) const {
+      return d*d;
+    }
 
-  inline FT inverse_of_transformed_distance(FT d) const {
-                return CGAL::sqrt(d);
-        }
+    inline FT inverse_of_transformed_distance(FT d) const {
+      return CGAL::sqrt(d);
+    }
 
   }; // class Euclidean_distance
 
diff -pruN 5.0.3-1/include/CGAL/Euclidean_distance_sphere_point.h 5.1-1/include/CGAL/Euclidean_distance_sphere_point.h
--- 5.0.3-1/include/CGAL/Euclidean_distance_sphere_point.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Euclidean_distance_sphere_point.h	2020-09-08 12:15:39.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_searching/include/CGAL/Euclidean_distance_sphere_point.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_searching/include/CGAL/Euclidean_distance_sphere_point.h $
 // $Id: Euclidean_distance_sphere_point.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Euler_integrator_2.h 5.1-1/include/CGAL/Euler_integrator_2.h
--- 5.0.3-1/include/CGAL/Euler_integrator_2.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Euler_integrator_2.h	2020-09-08 12:15:41.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_lines_2/include/CGAL/Euler_integrator_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_lines_2/include/CGAL/Euler_integrator_2.h $
 // $Id: Euler_integrator_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Exact_algebraic.h 5.1-1/include/CGAL/Exact_algebraic.h
--- 5.0.3-1/include/CGAL/Exact_algebraic.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Exact_algebraic.h	2020-09-08 12:15:16.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Exact_algebraic.h $
-// $Id: Exact_algebraic.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Exact_algebraic.h $
+// $Id: Exact_algebraic.h 66040cb 2020-07-20T17:13:01+02:00 Laurent Rineau
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -49,12 +49,10 @@ typedef unspecified_type Exact_algebraic
 
 #else // not DOXYGEN_RUNNING
 
-#ifdef CGAL_USE_CORE
-  typedef CORE::Expr Exact_algebraic;
-#endif
-
 #ifdef CGAL_USE_LEDA
 typedef leda_real Exact_algebraic;
+#elif defined CGAL_USE_CORE
+  typedef CORE::Expr Exact_algebraic;
 #endif
 
 #endif
diff -pruN 5.0.3-1/include/CGAL/Exact_circular_kernel_2.h 5.1-1/include/CGAL/Exact_circular_kernel_2.h
--- 5.0.3-1/include/CGAL/Exact_circular_kernel_2.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Exact_circular_kernel_2.h	2020-09-08 12:14:46.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_2/include/CGAL/Exact_circular_kernel_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_2/include/CGAL/Exact_circular_kernel_2.h $
 // $Id: Exact_circular_kernel_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Exact_integer.h 5.1-1/include/CGAL/Exact_integer.h
--- 5.0.3-1/include/CGAL/Exact_integer.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Exact_integer.h	2020-09-08 12:15:16.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Exact_integer.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Exact_integer.h $
 // $Id: Exact_integer.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Exact_kernel_selector.h 5.1-1/include/CGAL/Exact_kernel_selector.h
--- 5.0.3-1/include/CGAL/Exact_kernel_selector.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Exact_kernel_selector.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Exact_kernel_selector.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Exact_kernel_selector.h $
 // $Id: Exact_kernel_selector.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Exact_predicates_exact_constructions_kernel.h 5.1-1/include/CGAL/Exact_predicates_exact_constructions_kernel.h
--- 5.0.3-1/include/CGAL/Exact_predicates_exact_constructions_kernel.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Exact_predicates_exact_constructions_kernel.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Exact_predicates_exact_constructions_kernel.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Exact_predicates_exact_constructions_kernel.h $
 // $Id: Exact_predicates_exact_constructions_kernel.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Exact_predicates_exact_constructions_kernel_with_kth_root.h 5.1-1/include/CGAL/Exact_predicates_exact_constructions_kernel_with_kth_root.h
--- 5.0.3-1/include/CGAL/Exact_predicates_exact_constructions_kernel_with_kth_root.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Exact_predicates_exact_constructions_kernel_with_kth_root.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Exact_predicates_exact_constructions_kernel_with_kth_root.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Exact_predicates_exact_constructions_kernel_with_kth_root.h $
 // $Id: Exact_predicates_exact_constructions_kernel_with_kth_root.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Exact_predicates_exact_constructions_kernel_with_root_of.h 5.1-1/include/CGAL/Exact_predicates_exact_constructions_kernel_with_root_of.h
--- 5.0.3-1/include/CGAL/Exact_predicates_exact_constructions_kernel_with_root_of.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Exact_predicates_exact_constructions_kernel_with_root_of.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Exact_predicates_exact_constructions_kernel_with_root_of.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Exact_predicates_exact_constructions_kernel_with_root_of.h $
 // $Id: Exact_predicates_exact_constructions_kernel_with_root_of.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Exact_predicates_exact_constructions_kernel_with_sqrt.h 5.1-1/include/CGAL/Exact_predicates_exact_constructions_kernel_with_sqrt.h
--- 5.0.3-1/include/CGAL/Exact_predicates_exact_constructions_kernel_with_sqrt.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Exact_predicates_exact_constructions_kernel_with_sqrt.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Exact_predicates_exact_constructions_kernel_with_sqrt.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Exact_predicates_exact_constructions_kernel_with_sqrt.h $
 // $Id: Exact_predicates_exact_constructions_kernel_with_sqrt.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Exact_predicates_inexact_constructions_kernel.h 5.1-1/include/CGAL/Exact_predicates_inexact_constructions_kernel.h
--- 5.0.3-1/include/CGAL/Exact_predicates_inexact_constructions_kernel.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Exact_predicates_inexact_constructions_kernel.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Exact_predicates_inexact_constructions_kernel.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Exact_predicates_inexact_constructions_kernel.h $
 // $Id: Exact_predicates_inexact_constructions_kernel.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Exact_rational.h 5.1-1/include/CGAL/Exact_rational.h
--- 5.0.3-1/include/CGAL/Exact_rational.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Exact_rational.h	2020-09-08 12:15:16.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Exact_rational.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Exact_rational.h $
 // $Id: Exact_rational.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Exact_spherical_kernel_3.h 5.1-1/include/CGAL/Exact_spherical_kernel_3.h
--- 5.0.3-1/include/CGAL/Exact_spherical_kernel_3.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Exact_spherical_kernel_3.h	2020-09-08 12:14:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_3/include/CGAL/Exact_spherical_kernel_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_3/include/CGAL/Exact_spherical_kernel_3.h $
 // $Id: Exact_spherical_kernel_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/exceptions.h 5.1-1/include/CGAL/exceptions.h
--- 5.0.3-1/include/CGAL/exceptions.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/exceptions.h	2020-09-08 12:15:32.000000000 +0000
@@ -4,8 +4,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/exceptions.h $
-// $Id: exceptions.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/exceptions.h $
+// $Id: exceptions.h 04305dc 2020-06-13T23:39:35+05:30 Abhay Raj Singh
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -97,8 +97,6 @@ public:
     m_msg( msg)
     {}
 
-    ~Failure_exception() throw() {}
-
     //! the name of the library that issues this message.
     std::string      library() const { return m_lib; }
 
@@ -185,7 +183,19 @@ public:
                              "warning condition failed") {}
 };
 
+namespace internal {
+
+// The following classes are useful to create output iterators (with the help
+// of boost::function_output_iterator) that will throw as soon as something is being written.
+class Throw_at_output_exception : public std::exception { };
+
+struct Throw_at_output
+{
+  template<class T>
+  void operator()(const T& /* t */) const { throw Throw_at_output_exception(); }
+};
 
+} // namespace internal
 } //namespace CGAL
 
 #endif // CGAL_EXCEPTIONS_H
diff -pruN 5.0.3-1/include/CGAL/Exponent_vector.h 5.1-1/include/CGAL/Exponent_vector.h
--- 5.0.3-1/include/CGAL/Exponent_vector.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Exponent_vector.h	2020-09-08 12:15:29.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/Exponent_vector.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/Exponent_vector.h $
 // $Id: Exponent_vector.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/export/CGAL.h 5.1-1/include/CGAL/export/CGAL.h
--- 5.0.3-1/include/CGAL/export/CGAL.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/export/CGAL.h	2020-09-08 12:14:54.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/export/CGAL.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/export/CGAL.h $
 // $Id: CGAL.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/export/CORE.h 5.1-1/include/CGAL/export/CORE.h
--- 5.0.3-1/include/CGAL/export/CORE.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/export/CORE.h	2020-09-08 12:14:44.000000000 +0000
@@ -21,7 +21,7 @@
  * WWW URL: http://cs.nyu.edu/exact/
  * Email: exact@cs.nyu.edu
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_Core/include/CGAL/export/CORE.h $
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_Core/include/CGAL/export/CORE.h $
  * $Id: CORE.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
  * SPDX-License-Identifier: LGPL-3.0-or-later
  ***************************************************************************/
diff -pruN 5.0.3-1/include/CGAL/export/helpers.h 5.1-1/include/CGAL/export/helpers.h
--- 5.0.3-1/include/CGAL/export/helpers.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/export/helpers.h	2020-09-08 12:14:54.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/export/helpers.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/export/helpers.h $
 // $Id: helpers.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/export/ImageIO.h 5.1-1/include/CGAL/export/ImageIO.h
--- 5.0.3-1/include/CGAL/export/ImageIO.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/export/ImageIO.h	2020-09-08 12:14:44.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/export/ImageIO.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/export/ImageIO.h $
 // $Id: ImageIO.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/export/Qt.h 5.1-1/include/CGAL/export/Qt.h
--- 5.0.3-1/include/CGAL/export/Qt.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/export/Qt.h	2020-09-08 12:14:53.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/export/Qt.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/export/Qt.h $
 // $Id: Qt.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Extended_cartesian.h 5.1-1/include/CGAL/Extended_cartesian.h
--- 5.0.3-1/include/CGAL/Extended_cartesian.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Extended_cartesian.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_2/include/CGAL/Extended_cartesian.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_2/include/CGAL/Extended_cartesian.h $
 // $Id: Extended_cartesian.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/extended_euclidean_algorithm.h 5.1-1/include/CGAL/extended_euclidean_algorithm.h
--- 5.0.3-1/include/CGAL/extended_euclidean_algorithm.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/extended_euclidean_algorithm.h	2020-09-08 12:14:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_foundations/include/CGAL/extended_euclidean_algorithm.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_foundations/include/CGAL/extended_euclidean_algorithm.h $
 // $Id: extended_euclidean_algorithm.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Extended_homogeneous.h 5.1-1/include/CGAL/Extended_homogeneous.h
--- 5.0.3-1/include/CGAL/Extended_homogeneous.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Extended_homogeneous.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_2/include/CGAL/Extended_homogeneous.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_2/include/CGAL/Extended_homogeneous.h $
 // $Id: Extended_homogeneous.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/extract_mean_curvature_flow_skeleton.h 5.1-1/include/CGAL/extract_mean_curvature_flow_skeleton.h
--- 5.0.3-1/include/CGAL/extract_mean_curvature_flow_skeleton.h	2020-08-01 19:02:56.000000000 +0000
+++ 5.1-1/include/CGAL/extract_mean_curvature_flow_skeleton.h	2020-09-08 12:15:45.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_skeletonization/include/CGAL/extract_mean_curvature_flow_skeleton.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_skeletonization/include/CGAL/extract_mean_curvature_flow_skeleton.h $
 // $Id: extract_mean_curvature_flow_skeleton.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/extremal_polygon_2.h 5.1-1/include/CGAL/extremal_polygon_2.h
--- 5.0.3-1/include/CGAL/extremal_polygon_2.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/extremal_polygon_2.h	2020-09-08 12:14:54.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Inscribed_areas/include/CGAL/extremal_polygon_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Inscribed_areas/include/CGAL/extremal_polygon_2.h $
 // $Id: extremal_polygon_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Extremal_polygon_traits_2.h 5.1-1/include/CGAL/Extremal_polygon_traits_2.h
--- 5.0.3-1/include/CGAL/Extremal_polygon_traits_2.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Extremal_polygon_traits_2.h	2020-09-08 12:14:54.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Inscribed_areas/include/CGAL/Extremal_polygon_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Inscribed_areas/include/CGAL/Extremal_polygon_traits_2.h $
 // $Id: Extremal_polygon_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Extreme_points_traits_adapter_3.h 5.1-1/include/CGAL/Extreme_points_traits_adapter_3.h
--- 5.0.3-1/include/CGAL/Extreme_points_traits_adapter_3.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Extreme_points_traits_adapter_3.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_3/include/CGAL/Extreme_points_traits_adapter_3.h $
-// $Id: Extreme_points_traits_adapter_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_3/include/CGAL/Extreme_points_traits_adapter_3.h $
+// $Id: Extreme_points_traits_adapter_3.h 4e01ec7 2020-03-24T16:22:06+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -24,216 +24,254 @@
 #include <CGAL/convex_hull_3.h>
 #include <CGAL/result_of.h>
 
-
-
 namespace CGAL {
+namespace Convex_hull_3 {
+namespace internal {
 
-namespace Convex_hull_impl{
 template <class F, class PointPropertyMap>
 struct Forward_functor
   : public F
 {
   PointPropertyMap vpm_;
 
-  Forward_functor(const PointPropertyMap& vpm,
-                  const F& f) : F(f), vpm_(vpm) {}
-
+  Forward_functor(const PointPropertyMap& vpm, const F& f) : F(f), vpm_(vpm) {}
 
   template <class Vertex>
   typename cpp11::result_of<F(const Vertex&, const Vertex&)>::type
-  operator() (const Vertex& p, const Vertex& q) const
+  operator()(const Vertex& p, const Vertex& q) const
   {
-    return static_cast<const F*>(this)->operator()(get(vpm_,p),get(vpm_,q));
+    return static_cast<const F*>(this)->operator()(get(vpm_, p), get(vpm_, q));
   }
 
   template <class Vertex>
   typename cpp11::result_of<F(const Vertex&, const Vertex&, const Vertex&)>::type
-  operator() (const Vertex& p, const Vertex& q, const Vertex& r) const
+  operator()(const Vertex& p, const Vertex& q, const Vertex& r) const
   {
-    return static_cast<const F*>(this)->operator()(
-          get(vpm_,p),
-          get(vpm_,q),
-          get(vpm_,r));
+    return static_cast<const F*>(this)->operator()(get(vpm_, p),
+                                                   get(vpm_, q),
+                                                   get(vpm_, r));
   }
 
   template <class Vertex>
   typename cpp11::result_of<F(const Vertex&, const Vertex&, const Vertex&, const Vertex&)>::type
-  operator() (const Vertex& p, const Vertex& q, const Vertex& r, const Vertex& s) const
+  operator()(const Vertex& p, const Vertex& q, const Vertex& r, const Vertex& s) const
   {
-    return static_cast<const F*>(this)->operator()(
-          get(vpm_,p),
-          get(vpm_,q),
-          get(vpm_,r),
-          get(vpm_,s));
+    return static_cast<const F*>(this)->operator()(get(vpm_, p),
+                                                   get(vpm_, q),
+                                                   get(vpm_, r),
+                                                   get(vpm_, s));
   }
 };
-}//end Convex_hull_impl
-template<
-    class PointPropertyMap,
-    class Base_traits=typename internal::Convex_hull_3::Default_traits_for_Chull_3<
-      typename boost::property_traits<PointPropertyMap>::value_type>::type
+
+} // namespace internal
+} // namespace Convex_hull_3
+
+template<class PointPropertyMap,
+         class Base_traits = typename Convex_hull_3::internal::Default_traits_for_Chull_3<
+                               typename boost::property_traits<PointPropertyMap>::value_type>::type
          >
 class Extreme_points_traits_adapter_3
-    :public Base_traits
+  : public Base_traits
 {
   PointPropertyMap vpm_;
 
-
 public:
-  Extreme_points_traits_adapter_3(const PointPropertyMap& vpmap, Base_traits base=Base_traits())
-    :Base_traits(base), vpm_(vpmap)
-  {}
-  typedef typename boost::property_traits<PointPropertyMap>::key_type Vertex;
-  typedef Vertex Point_3;
-  typedef Convex_hull_impl::Forward_functor<typename Base_traits::Equal_3, PointPropertyMap> Equal_3;
-  typedef Convex_hull_impl::Forward_functor<typename Base_traits::Collinear_3, PointPropertyMap> Collinear_3;
-  typedef Convex_hull_impl::Forward_functor<typename Base_traits::Coplanar_3, PointPropertyMap> Coplanar_3;
-  typedef Convex_hull_impl::Forward_functor<typename Base_traits::Less_distance_to_point_3, PointPropertyMap> Less_distance_to_point_3;
+  Extreme_points_traits_adapter_3(const PointPropertyMap& vpmap,
+                                  Base_traits base = Base_traits())
+    :
+      Base_traits(base), vpm_(vpmap)
+  { }
+
+  typedef typename boost::property_traits<PointPropertyMap>::key_type          Vertex;
+  typedef Vertex                                                               Point_3;
+  typedef Convex_hull_3::internal::Forward_functor<
+            typename Base_traits::Equal_3, PointPropertyMap>                   Equal_3;
+  typedef Convex_hull_3::internal::Forward_functor<
+            typename Base_traits::Collinear_3, PointPropertyMap>               Collinear_3;
+  typedef Convex_hull_3::internal::Forward_functor<
+            typename Base_traits::Coplanar_3, PointPropertyMap>                Coplanar_3;
+  typedef Convex_hull_3::internal::Forward_functor<
+            typename Base_traits::Less_distance_to_point_3, PointPropertyMap>  Less_distance_to_point_3;
+
   class Less_signed_distance_to_plane_3
-      :public Base_traits::Less_signed_distance_to_plane_3
+    : public Base_traits::Less_signed_distance_to_plane_3
   {
     PointPropertyMap vpm_;
     const typename Base_traits::Less_signed_distance_to_plane_3& base;
-  public:
-    typedef typename Base_traits::Plane_3 Plane_3;
 
-    typedef bool             result_type;
+  public:
+    typedef typename Base_traits::Plane_3                                      Plane_3;
+    typedef bool                                                               result_type;
 
-    Less_signed_distance_to_plane_3(
-        const PointPropertyMap& map,
-        const typename Base_traits::Less_signed_distance_to_plane_3& base):
-      Base_traits::Less_signed_distance_to_plane_3(base),vpm_(map), base(base){}
+    Less_signed_distance_to_plane_3(const PointPropertyMap& map,
+                                    const typename Base_traits::Less_signed_distance_to_plane_3& base)
+      : Base_traits::Less_signed_distance_to_plane_3(base), vpm_(map), base(base)
+    { }
 
-    bool
-    operator()( const Plane_3& h, const Vertex& p, const Vertex& q) const
+    bool operator()(const Plane_3& h, const Vertex& p, const Vertex& q) const
     {
-      return base(h, get(vpm_,p), get(vpm_,q));
+      return base(h, get(vpm_, p), get(vpm_, q));
     }
   };
 
+  Equal_3 equal_3_object() const
+  { return Equal_3(vpm_, static_cast<const Base_traits*>(this)->equal_3_object()); }
+  Collinear_3 collinear_3_object() const
+  { return Collinear_3(vpm_, static_cast<const Base_traits*>(this)->collinear_3_object()); }
+  Coplanar_3 coplanar_3_object() const
+  { return Coplanar_3(vpm_, static_cast<const Base_traits*>(this)->coplanar_3_object()); }
+  Less_distance_to_point_3 less_distance_to_point_3_object() const
+  { return Less_distance_to_point_3(vpm_, static_cast<const Base_traits*>(this)->less_distance_to_point_3_object()); }
+  Less_signed_distance_to_plane_3 less_signed_distance_to_plane_3_object() const
+  { return Less_signed_distance_to_plane_3(vpm_, static_cast<const Base_traits*>(this)->less_signed_distance_to_plane_3_object()); }
 
-  Equal_3 equal_3_object () const {return Equal_3(vpm_,static_cast<const Base_traits*>(this)->equal_3_object() );}
-  Collinear_3 collinear_3_object () const {return Collinear_3(vpm_,static_cast<const Base_traits*>(this)->collinear_3_object() );}
-  Coplanar_3 coplanar_3_object () const {return Coplanar_3(vpm_,static_cast<const Base_traits*>(this)->coplanar_3_object() );}
-  Less_distance_to_point_3 less_distance_to_point_3_object() const {
-    return Less_distance_to_point_3(vpm_,static_cast<const Base_traits*>(this)->less_distance_to_point_3_object() );}
-  Less_signed_distance_to_plane_3 less_signed_distance_to_plane_3_object() const {
-    return Less_signed_distance_to_plane_3(
-          vpm_,static_cast<const Base_traits*>(this)->less_signed_distance_to_plane_3_object() );
-  }
-
-  class Construct_plane_3:public Base_traits::Construct_plane_3
+  class Construct_plane_3
+    : public Base_traits::Construct_plane_3
   {
     PointPropertyMap vpm_;
     const typename Base_traits::Construct_plane_3& base;
+
   public:
-    Construct_plane_3(const PointPropertyMap& map, const typename Base_traits::Construct_plane_3& base):
-      Base_traits::Construct_plane_3(base),vpm_(map), base(base){}
-    typename Base_traits::Plane_3 operator()(const Vertex& p, const Vertex& q, const Vertex& r)const
+    Construct_plane_3(const PointPropertyMap& map,
+                      const typename Base_traits::Construct_plane_3& base)
+      : Base_traits::Construct_plane_3(base), vpm_(map), base(base)
+    { }
+
+    typename Base_traits::Plane_3 operator()(const Vertex& p, const Vertex& q, const Vertex& r) const
     {
-      return base(get(vpm_,p),get(vpm_,q),get(vpm_,r));
+      return base(get(vpm_, p), get(vpm_, q), get(vpm_, r));
     }
   };
+
   Construct_plane_3 construct_plane_3_object() const
-  {return Construct_plane_3(vpm_,static_cast<const Base_traits*>(this)->construct_plane_3_object());}
+  {return Construct_plane_3(vpm_, static_cast<const Base_traits*>(this)->construct_plane_3_object());}
 
-  class Has_on_positive_side_3:public Base_traits::Has_on_positive_side_3
+  class Has_on_positive_side_3
+    : public Base_traits::Has_on_positive_side_3
   {
     PointPropertyMap vpm_;
     const typename Base_traits::Has_on_positive_side_3& base;
+
   public:
-    Has_on_positive_side_3(const PointPropertyMap& map,const typename Base_traits::Has_on_positive_side_3& base):
-      Base_traits::Has_on_positive_side_3(base),vpm_(map), base(base){}
+    Has_on_positive_side_3(const PointPropertyMap& map,
+                           const typename Base_traits::Has_on_positive_side_3& base)
+      : Base_traits::Has_on_positive_side_3(base), vpm_(map), base(base)
+    { }
 
-    typedef typename Base_traits::Plane_3          Plane_3;
+    typedef typename Base_traits::Plane_3                               Plane_3;
   public:
-    typedef bool          result_type;
+    typedef bool                                                        result_type;
 
-    result_type
-    operator()( const Plane_3& pl, const Vertex& p) const
+    result_type operator()( const Plane_3& pl, const Vertex& p) const
     {
       return base(pl, get(vpm_, p));
     }
   };
-  Has_on_positive_side_3 has_on_positive_side_3_object() const {return Has_on_positive_side_3(
-          vpm_,static_cast<const Base_traits*>(this)->has_on_positive_side_3_object() );}
+
+  Has_on_positive_side_3 has_on_positive_side_3_object() const
+  { return Has_on_positive_side_3(vpm_, static_cast<const Base_traits*>(this)->has_on_positive_side_3_object()); }
 
   template<class Base_proj_traits>
-  class Proj_traits_3:public Base_proj_traits
+  class Proj_traits_3
+    : public Base_proj_traits
   {
     PointPropertyMap vpm_;
     typedef Base_proj_traits Btt;
+
   public:
-    Proj_traits_3(const PointPropertyMap& map,const Btt& base):
-      Base_proj_traits(base),vpm_(map){}
-    typedef Point_3 Point_2;
-    typedef Convex_hull_impl::Forward_functor<typename Btt::Equal_2, PointPropertyMap> Equal_2;
-    typedef Convex_hull_impl::Forward_functor<typename Btt::Less_xy_2, PointPropertyMap> Less_xy_2;
-    typedef Convex_hull_impl::Forward_functor<typename Btt::Less_yx_2, PointPropertyMap> Less_yx_2;
-    typedef Convex_hull_impl::Forward_functor<typename Btt::Less_signed_distance_to_line_2, PointPropertyMap> Less_signed_distance_to_line_2;
-    typedef Convex_hull_impl::Forward_functor<typename Btt::Left_turn_2, PointPropertyMap> Left_turn_2;
+    Proj_traits_3(const PointPropertyMap& map, const Btt& base)
+      : Base_proj_traits(base), vpm_(map)
+    { }
+
+    typedef Point_3                                                           Point_2;
+    typedef Convex_hull_3::internal::Forward_functor<
+              typename Btt::Equal_2, PointPropertyMap>                        Equal_2;
+    typedef Convex_hull_3::internal::Forward_functor<
+              typename Btt::Less_xy_2, PointPropertyMap>                      Less_xy_2;
+    typedef Convex_hull_3::internal::Forward_functor<
+              typename Btt::Less_yx_2, PointPropertyMap>                      Less_yx_2;
+    typedef Convex_hull_3::internal::Forward_functor<
+              typename Btt::Less_signed_distance_to_line_2, PointPropertyMap> Less_signed_distance_to_line_2;
+    typedef Convex_hull_3::internal::Forward_functor<
+              typename Btt::Left_turn_2, PointPropertyMap>                    Left_turn_2;
 
-    class Less_rotate_ccw_2:public Btt::Less_rotate_ccw_2
+    class Less_rotate_ccw_2
+      : public Btt::Less_rotate_ccw_2
     {
       PointPropertyMap vpm_;
       const typename Btt::Less_rotate_ccw_2& base;
+
     public:
-      Less_rotate_ccw_2(const PointPropertyMap& map,const typename Btt::Less_rotate_ccw_2& base):
-        Btt::Less_rotate_ccw_2(base),vpm_(map), base(base){}
+      Less_rotate_ccw_2(const PointPropertyMap& map,
+                        const typename Btt::Less_rotate_ccw_2& base)
+        : Btt::Less_rotate_ccw_2(base), vpm_(map), base(base)
+      { }
+
     public:
-      bool operator()(Point_2 e, Point_2 p,Point_2 q) const
+      bool operator()(const Point_2& e, const Point_2& p, const Point_2& q) const
       {
         return base(get(vpm_, e), get(vpm_, p), get(vpm_, q));
       }
     };
 
-    Equal_2 equal_2_object () const {return Equal_2(vpm_,static_cast<const Btt*>(this)->equal_2_object() );}
-    Less_xy_2 less_xy_2_object ()const{return Less_xy_2(vpm_,static_cast<const Btt*>(this)->less_xy_2_object() );}
-    Less_yx_2 less_yx_2_object ()const{return Less_yx_2(vpm_,static_cast<const Btt*>(this)->less_yx_2_object() );}
-    Less_signed_distance_to_line_2 less_signed_distance_to_line_2_object ()const
-    {return Less_signed_distance_to_line_2(vpm_,static_cast<const Btt*>(this)->Less_signed_distance_to_line_2() );}
-    Less_rotate_ccw_2 less_rotate_ccw_2_object ()const
-    {return Less_rotate_ccw_2(vpm_,static_cast<const Btt*>(this)->less_rotate_ccw_2_object() );}
-    Left_turn_2 left_turn_2_object ()const{return Left_turn_2(vpm_,static_cast<const Btt*>(this)->left_turn_2_object() );}
+    Equal_2 equal_2_object() const
+    { return Equal_2(vpm_, static_cast<const Btt*>(this)->equal_2_object()); }
+    Less_xy_2 less_xy_2_object() const
+    { return Less_xy_2(vpm_, static_cast<const Btt*>(this)->less_xy_2_object()); }
+    Less_yx_2 less_yx_2_object() const
+    { return Less_yx_2(vpm_, static_cast<const Btt*>(this)->less_yx_2_object()); }
+    Less_signed_distance_to_line_2 less_signed_distance_to_line_2_object() const
+    { return Less_signed_distance_to_line_2(vpm_, static_cast<const Btt*>(this)->Less_signed_distance_to_line_2()); }
+    Less_rotate_ccw_2 less_rotate_ccw_2_object() const
+    { return Less_rotate_ccw_2(vpm_, static_cast<const Btt*>(this)->less_rotate_ccw_2_object()); }
+    Left_turn_2 left_turn_2_object() const
+    { return Left_turn_2(vpm_, static_cast<const Btt*>(this)->left_turn_2_object()); }
 
-    class Orientation_2:public Btt::Orientation_2
+    class Orientation_2
+      : public Btt::Orientation_2
     {
       PointPropertyMap vpm_;
       const typename Btt::Orientation_2& base;
+
     public:
-      Orientation_2(const PointPropertyMap& map,const typename Btt::Orientation_2& base):
-        Btt::Orientation_2(base),vpm_(map), base(base){}
+      Orientation_2(const PointPropertyMap& map,
+                    const typename Btt::Orientation_2& base)
+        : Btt::Orientation_2(base), vpm_(map), base(base)
+      { }
 
-      typename CGAL::Orientation operator()(Point_2 e,Point_2 p, Point_2 q) const
+      typename CGAL::Orientation operator()(const Point_2& e, const Point_2& p, const Point_2& q) const
       {
         return base(get(vpm_, e), get(vpm_, p), get(vpm_, q));
       }
     };
-    Orientation_2 orientation_2_object ()const{return Orientation_2(vpm_,static_cast<const Btt*>(this)->orientation_2_object() );}
+
+    Orientation_2 orientation_2_object() const
+    { return Orientation_2(vpm_, static_cast<const Btt*>(this)->orientation_2_object()); }
   };
 
-  typedef internal::Convex_hull_3::Projection_traits<Base_traits> Base_PTraits;
-  typedef Proj_traits_3<typename Base_PTraits::Traits_xy_3> Traits_xy_3;
-  typedef Proj_traits_3<typename Base_PTraits::Traits_yz_3> Traits_yz_3;
-  typedef Proj_traits_3<typename Base_PTraits::Traits_xz_3> Traits_xz_3;
-
-  Traits_xy_3 construct_traits_xy_3_object()const
-  {return Traits_xy_3(vpm_, Base_PTraits(static_cast<const Base_traits&>(*this)).construct_traits_xy_3_object());}
-  Traits_yz_3 construct_traits_yz_3_object()const
-  {return Traits_yz_3(vpm_, Base_PTraits(static_cast<const Base_traits&>(*this)).construct_traits_yz_3_object());}
-  Traits_xz_3 construct_traits_xz_3_object()const
-  {return Traits_xz_3(vpm_, Base_PTraits(static_cast<const Base_traits&>(*this)).construct_traits_xz_3_object());}
+  typedef Convex_hull_3::internal::Projection_traits<Base_traits> Base_PTraits;
+  typedef Proj_traits_3<typename Base_PTraits::Traits_xy_3>       Traits_xy_3;
+  typedef Proj_traits_3<typename Base_PTraits::Traits_yz_3>       Traits_yz_3;
+  typedef Proj_traits_3<typename Base_PTraits::Traits_xz_3>       Traits_xz_3;
+
+  Traits_xy_3 construct_traits_xy_3_object() const
+  { return Traits_xy_3(vpm_, Base_PTraits(static_cast<const Base_traits&>(*this)).construct_traits_xy_3_object()); }
+  Traits_yz_3 construct_traits_yz_3_object() const
+  { return Traits_yz_3(vpm_, Base_PTraits(static_cast<const Base_traits&>(*this)).construct_traits_yz_3_object()); }
+  Traits_xz_3 construct_traits_xz_3_object() const
+  { return Traits_xz_3(vpm_, Base_PTraits(static_cast<const Base_traits&>(*this)).construct_traits_xz_3_object()); }
 
   typename boost::property_traits<PointPropertyMap>::reference
   get_point(const typename boost::property_traits<PointPropertyMap>::key_type& k) const
   {
     return get(vpm_, k);
   }
-
 };
+
 template<class PointPropertyMap,class Base_traits>
 Extreme_points_traits_adapter_3<PointPropertyMap, Base_traits>
-make_extreme_points_traits_adapter(const PointPropertyMap& pmap, Base_traits traits = Base_traits())
+make_extreme_points_traits_adapter(const PointPropertyMap& pmap,
+                                   Base_traits traits = Base_traits())
 {
   return Extreme_points_traits_adapter_3<PointPropertyMap, Base_traits>(pmap, traits);
 }
@@ -245,9 +283,6 @@ make_extreme_points_traits_adapter(const
   return Extreme_points_traits_adapter_3<PointPropertyMap>(pmap);
 }
 
-
-//helper function
-
-}//end CGAL
+} // namespace CGAL
 
 #endif // CGAL_EXTREME_POINTS_TRAITS_ADAPTER_3_H
diff -pruN 5.0.3-1/include/CGAL/exude_mesh_3.h 5.1-1/include/CGAL/exude_mesh_3.h
--- 5.0.3-1/include/CGAL/exude_mesh_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/exude_mesh_3.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/exude_mesh_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/exude_mesh_3.h $
 // $Id: exude_mesh_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Face_graph_wrapper.h 5.1-1/include/CGAL/Face_graph_wrapper.h
--- 5.0.3-1/include/CGAL/Face_graph_wrapper.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Face_graph_wrapper.h	2020-09-08 12:15:45.000000000 +0000
@@ -0,0 +1,933 @@
+// Copyright (c) 2019 CNRS and LIRIS' Establishments (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_topology/include/CGAL/Face_graph_wrapper.h $
+// $Id: Face_graph_wrapper.h 52186a0 2020-05-14T11:38:15+02:00 Guillaume Damiand
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
+//
+#ifndef CGAL_FACE_GRAPH_WRAPPER_H
+#define CGAL_FACE_GRAPH_WRAPPER_H 1
+
+#include <CGAL/license/Surface_mesh_topology.h>
+
+#include <CGAL/Surface_mesh_topology/internal/Functors_for_face_graph_wrapper.h>
+#include <CGAL/Surface_mesh_topology/internal/Iterators_for_face_graph_wrapper.h>
+#include <CGAL/internal/Combinatorial_map_internal_functors.h>
+#include <CGAL/Polyhedron_3_fwd.h>
+#include <CGAL/Surface_mesh/Surface_mesh_fwd.h>
+#include <CGAL/Combinatorial_map_fwd.h>
+#include <CGAL/Generalized_map_fwd.h>
+#include <CGAL/Linear_cell_complex_fwd.h>
+#include <CGAL/Polygonal_schema_fwd.h>
+#include <bitset>
+
+namespace CGAL
+{
+////////////////////////////////////////////////////////////////////////////////
+/** Class Face_graph_wrapper: to wrap any model of FaceGraph into a
+ *  Combinatorial map. For now, only for const models, i.e. does not support
+ *  modification operators.
+ */
+template<typename HEG_>
+class Face_graph_wrapper
+{
+public:
+  typedef HEG_                    HEG;
+  typedef Face_graph_wrapper<HEG> Self;
+  typedef boost::uint32_t /*std::size_t*/ size_type;
+  typedef Self                    Refs;
+
+  struct Dart_container
+  {
+    typedef typename boost::graph_traits<HEG>::halfedge_iterator iterator;
+    typedef typename boost::graph_traits<HEG>::halfedge_iterator const_iterator; // TODO ?
+    // typedef My_halfedge_iterator<HEG> iterator;
+    // typedef My_halfedge_iterator<HEG> const_iterator; // TODO ?
+  };
+
+  typedef typename boost::graph_traits<HEG>::halfedge_descriptor Dart_handle;
+  typedef typename boost::graph_traits<HEG>::halfedge_descriptor Dart_const_handle;
+
+   typedef Dart_handle Null_handle_type;
+  // typedef CGAL::Void* Null_handle_type;
+   static const Null_handle_type null_handle; //=Dart_handle();
+   static const Null_handle_type null_dart_handle; //=Dart_handle();
+
+  /// Number of marks
+  static const size_type NB_MARKS = 32;
+  static const size_type INVALID_MARK = NB_MARKS;
+
+  /// The dimension of the combinatorial map.
+  static const unsigned int dimension=2;
+  static const unsigned int ambient_dimension=3;
+
+  typedef typename boost::graph_traits<HEG>::vertex_descriptor vertex_descriptor;
+  typedef typename boost::graph_traits<HEG>::edge_descriptor   edge_descriptor;
+  typedef typename boost::graph_traits<HEG>::face_descriptor   face_descriptor;
+  typedef boost::undirected_tag                                directed_category;
+  typedef boost::disallow_parallel_edge_tag                    edge_parallel_category;
+
+  struct SM_graph_traversal_category : public virtual boost::bidirectional_graph_tag,
+                                       public virtual boost::vertex_list_graph_tag,
+                                       public virtual boost::edge_list_graph_tag
+  {};
+  typedef SM_graph_traversal_category traversal_category;
+
+
+  Face_graph_wrapper(const HEG& f) : m_fg(f),
+                                     mdarts(*this),
+                                     m_nb_darts(0),
+                                     m_marks_initialized(false),
+                                     mnb_used_marks(0)
+
+  {
+    // Store locally the number of darts: the HEG must not be modified
+    m_nb_darts=darts().size();
+  }
+
+  void initialize_marks() const
+  {
+    if (m_marks_initialized) return;
+
+    mmask_marks.reset();
+
+    for (size_type i=0; i<NB_MARKS; ++i)
+    {
+      mfree_marks_stack[i]       =i;
+      mindex_marks[i]            =i;
+      mnb_marked_darts[i]        =0;
+      mnb_times_reserved_marks[i]=0;
+    }
+
+    m_all_marks=get(CGAL::dynamic_halfedge_property_t<std::bitset<NB_MARKS> >(), m_fg);
+    for (typename Dart_range::const_iterator it(darts().begin()),
+           itend(darts().end()); it!=itend; ++it)
+    { put(m_all_marks, it, std::bitset<NB_MARKS>()); }
+
+    m_marks_initialized=true;
+  }
+
+  const HEG& get_fg() const
+  { return m_fg; }
+
+  template<unsigned int i>
+  bool is_free(Dart_const_handle /* dh */) const
+  { return false; } // Not possible to have a free dart with an HEG.
+  bool is_free(Dart_const_handle /*dh*/, unsigned int /*i*/) const
+  { return false; } // Not possible to have a free dart with an HEG.
+
+  bool is_perforated(Dart_const_handle dh) const
+  { return is_border(dh, m_fg); }
+
+  Dart_const_handle get_beta(Dart_const_handle ADart, int B1) const
+  {
+    CGAL_assertion(B1>=0 && B1<=static_cast<int>(dimension));
+    if (B1==1) return internal::Get_beta<HEG, 1>::value(m_fg, ADart);
+    if (B1==2) return internal::Get_beta<HEG, 2>::value(m_fg, ADart);
+    return internal::Get_beta<HEG, 0>::value(m_fg, ADart);
+  }
+  template<int B1>
+  Dart_const_handle get_beta(Dart_const_handle ADart) const
+  {
+    CGAL_assertion(B1>=0 && B1<=static_cast<int>(dimension));
+    return internal::Get_beta<HEG, B1>::value(m_fg, ADart);
+  }
+
+  bool is_empty() const
+  { return number_of_darts()==0; }
+
+  /* ??  bool is_dart_used(Dart_const_handle dh) const
+      { return true; ?? } */
+
+  int highest_nonfree_dimension(Dart_const_handle /* dh */) const
+  { return 2; }
+
+  Dart_const_handle previous(Dart_const_handle ADart) const
+  { return get_beta<0>(ADart); }
+  Dart_const_handle next(Dart_const_handle ADart) const
+  { return get_beta<1>(ADart); }
+  Dart_const_handle opposite(Dart_const_handle dh) const
+  { return get_beta<2>(dh); }
+  Dart_const_handle opposite2(Dart_const_handle dh) const
+  { return get_beta<2>(dh); }
+  Dart_const_handle other_extremity(Dart_const_handle dh) const
+  { return get_beta<1>(dh); }
+
+  template<unsigned int dim>
+  Dart_const_handle opposite(Dart_const_handle ADart) const
+  { return this->template get_beta<dim>(ADart); }
+  Dart_const_handle other_orientation(Dart_const_handle ADart) const
+  { return ADart; }
+
+  bool is_previous_exist(Dart_const_handle) const
+  { return true; }
+  bool is_next_exist(Dart_const_handle) const
+  { return true; }
+  template<unsigned int dim>
+  bool is_opposite_exist(Dart_const_handle /* ADart */) const
+  { return true; }
+
+  template<typename ...Betas>
+  Dart_handle beta(Dart_handle ADart, Betas... betas)
+  { return CGAL::internal::Beta_functor<Self, Dart_handle, Betas...>::
+      run(*this, ADart, betas...); }
+  template<typename ...Betas>
+  Dart_const_handle beta(Dart_const_handle ADart, Betas... betas) const
+  { return CGAL::internal::Beta_functor<const Self, Dart_const_handle, Betas...>::
+      run(*this, ADart, betas...); }
+  template<int... Betas>
+  Dart_handle beta(Dart_handle ADart)
+  { return CGAL::internal::Beta_functor_static<Self, Dart_handle, Betas...>::
+      run(*this, ADart); }
+  template<int... Betas>
+  Dart_const_handle beta(Dart_const_handle ADart) const
+  { return CGAL::internal::Beta_functor_static<const Self, Dart_const_handle, Betas...>::
+      run(*this, ADart); }
+
+  size_type number_of_darts() const
+  { return m_nb_darts; }
+
+  size_type number_of_halfedges() const
+  { return number_of_darts(); }
+
+  size_type number_of_used_marks() const
+  { return mnb_used_marks; }
+
+  bool is_reserved(size_type amark) const
+  {
+    CGAL_assertion(amark<NB_MARKS);
+    return (m_marks_initialized && mnb_times_reserved_marks[amark]!=0);
+  }
+
+  size_type number_of_marked_darts(size_type amark) const
+  {
+    CGAL_assertion( is_reserved(amark) );
+    return mnb_marked_darts[amark];
+  }
+
+  size_type number_of_unmarked_darts(size_type amark) const
+  {
+    return number_of_darts() - number_of_marked_darts(amark);
+  }
+
+  bool is_whole_map_unmarked(size_type amark) const
+  { return number_of_marked_darts(amark)==0; }
+
+  bool is_whole_map_marked(size_type amark) const
+  {  return number_of_marked_darts(amark)==number_of_darts(); }
+
+  class Exception_no_more_available_mark {};
+
+  size_type get_new_mark() const
+  {
+    initialize_marks();
+    if (mnb_used_marks==NB_MARKS)
+    {
+      std::cerr << "Not enough Boolean marks: "
+        "increase NB_MARKS in item class." << std::endl;
+      std::cerr << "  (exception launched)" << std::endl;
+      throw Exception_no_more_available_mark();
+    }
+
+    size_type m=mfree_marks_stack[mnb_used_marks];
+    mused_marks_stack[mnb_used_marks]=m;
+
+    mindex_marks[m]=mnb_used_marks;
+    mnb_times_reserved_marks[m]=1;
+
+    ++mnb_used_marks;
+    CGAL_assertion(is_whole_map_unmarked(m));
+
+    return m;
+  }
+
+  void share_a_mark(size_type amark) const
+  {
+    CGAL_assertion( is_reserved(amark) );
+    ++mnb_times_reserved_marks[amark];
+  }
+
+  size_type get_number_of_times_mark_reserved(size_type amark) const
+  {
+    CGAL_assertion( is_reserved(amark) );
+    return mnb_times_reserved_marks[amark];
+  }
+
+  void negate_mark(size_type amark) const
+  {
+    CGAL_assertion(is_reserved(amark));
+
+    mnb_marked_darts[amark]=number_of_darts()-mnb_marked_darts[amark];
+    mmask_marks.flip(amark);
+  }
+
+  void mark_null_dart( size_type /*amark*/) const
+  {}
+
+  bool get_dart_mark(Dart_const_handle ADart, size_type amark) const
+  {
+    CGAL_assertion(is_reserved(amark));
+    return get(m_all_marks, ADart)[amark];
+  }
+  void set_dart_mark(Dart_const_handle ADart, size_type amark, bool avalue) const
+  {
+    CGAL_assertion(is_reserved(amark));
+    const_cast<std::bitset<NB_MARKS>& >(get(m_all_marks, ADart)).set(amark, avalue);
+  }
+
+  void flip_dart_mark(Dart_const_handle ADart, size_type amark) const
+  { set_dart_mark(ADart, amark, !get_dart_mark(ADart, amark)); }
+
+  bool is_marked(Dart_const_handle adart, size_type amark) const
+  {
+    CGAL_assertion(is_reserved(amark));
+    return get_dart_mark(adart, amark)!=mmask_marks[amark];
+  }
+
+  void set_mark_to(Dart_const_handle adart, size_type amark,
+                   bool astate) const
+  {
+    CGAL_assertion(is_reserved(amark));
+
+    if (is_marked(adart, amark)!=astate)
+    {
+      if (astate) ++mnb_marked_darts[amark];
+      else --mnb_marked_darts[amark];
+
+      flip_dart_mark(adart, amark);
+    }
+  }
+
+  void mark(Dart_const_handle adart, size_type amark) const
+  {
+    CGAL_assertion(is_reserved(amark));
+
+    if (is_marked(adart, amark)) return;
+
+    ++mnb_marked_darts[amark];
+    flip_dart_mark(adart, amark);
+  }
+
+  void unmark(Dart_const_handle adart, size_type amark) const
+  {
+    CGAL_assertion( adart!=this->null_dart_handle );
+    CGAL_assertion( is_reserved(amark) );
+
+    if (!is_marked(adart, amark)) return;
+
+    --mnb_marked_darts[amark];
+    flip_dart_mark(adart, amark);
+  }
+
+  void unmark_all(size_type amark) const
+  {
+    CGAL_assertion( is_reserved(amark) );
+
+    if ( is_whole_map_marked(amark) )
+    {
+      negate_mark(amark);
+    }
+    else if ( !is_whole_map_unmarked(amark) )
+    {
+      for (typename Dart_range::const_iterator it(darts().begin()),
+           itend(darts().end()); it!=itend; ++it)
+        unmark(*it, amark);
+    }
+    CGAL_assertion(is_whole_map_unmarked(amark));
+  }
+
+  void free_mark(size_type amark) const
+  {
+    CGAL_assertion( is_reserved(amark) );
+
+    if ( mnb_times_reserved_marks[amark]>1 )
+    {
+      --mnb_times_reserved_marks[amark];
+      return;
+    }
+
+    unmark_all(amark);
+
+    // 1) We remove amark from the array mused_marks_stack by
+    //    replacing it with the last mark in this array.
+    mused_marks_stack[mindex_marks[amark]] =
+      mused_marks_stack[--mnb_used_marks];
+    mindex_marks[mused_marks_stack[mnb_used_marks]] =
+      mindex_marks[amark];
+
+    // 2) We add amark in the array mfree_marks_stack and update its index.
+    mfree_marks_stack[ mnb_used_marks ]=amark;
+    mindex_marks[amark] = mnb_used_marks;
+
+    mnb_times_reserved_marks[amark]=0;
+  }
+
+  bool is_without_boundary(unsigned int i) const
+  {
+    CGAL_assertion(1<=i && i<=dimension);
+    if (i==1) return true;
+
+    for ( typename Dart_range::const_iterator it(darts().begin()),
+            itend(darts().end()); it!=itend; ++it)
+    { if (is_perforated(it)) return false; }
+    return true;
+  }
+
+  bool is_without_boundary() const
+  { return is_without_boundary(2); }
+
+  //**************************************************************************
+  // Dart_of_cell_range
+  template<unsigned int i>
+  struct Dart_of_cell_range
+  {
+    typedef CGAL::internal::FGW_cell_iterator<Self, i> iterator;
+    typedef CGAL::internal::FGW_cell_iterator<Self, i> const_iterator;
+    Dart_of_cell_range(const Self &amap, Dart_handle adart) : mmap(amap),
+                                                              m_initdart(adart),
+                                                              msize(0)
+    {}
+    const_iterator begin() const { return const_iterator(mmap, m_initdart); }
+    const_iterator end() const   { return const_iterator(mmap, m_initdart, mmap.null_handle); }
+    size_type size() const
+    {
+      if (msize==0)
+      {
+        for (const_iterator it=begin(), itend=end(); it!=itend; ++it)
+        { ++msize; }
+      }
+      return msize;
+    }
+
+    bool empty() const
+    { return mmap.is_empty(); }
+  private:
+    const Self & mmap;
+    Dart_handle m_initdart;
+    mutable typename Self::size_type msize;
+  };
+  //**************************************************************************
+  // Dart_of_cell_const_range
+  /*  template<unsigned int i,int dim=Self::dimension>
+  struct Dart_of_cell_const_range // TODO REMOVE ??
+  {}; */
+  //--------------------------------------------------------------------------
+  template<unsigned int i>
+  Dart_of_cell_range<i> darts_of_cell(Dart_handle adart)
+  { return Dart_of_cell_range<i>(*this,adart); }
+  //--------------------------------------------------------------------------
+  template<unsigned int i>
+  Dart_of_cell_range<i> darts_of_cell(Dart_const_handle adart) const
+  { return Dart_of_cell_range<i>(*this,adart); } // Before it was Dart_of_cell_const_range<i>
+  //**************************************************************************
+  // Dart_range
+  struct Dart_range {
+    typedef CGAL::internal::FGW_dart_iterator_basic_of_all<Self> iterator;
+    typedef CGAL::internal::FGW_dart_iterator_basic_of_all<Self> const_iterator;
+    Dart_range(const Self &amap) : mmap(amap), msize(0)
+    {}
+    iterator begin() { return iterator(mmap); }
+    iterator end()   { return iterator(mmap,mmap.null_handle); }
+    const_iterator begin() const { return const_iterator(mmap); }
+    const_iterator end() const   { return const_iterator(mmap,mmap.null_handle); }
+    size_type size() const
+    {
+      if (msize==0)
+      { msize=static_cast<size_type>(halfedges(mmap.get_fg()).size()); }
+      return msize;
+    }
+    bool empty() const
+    { return mmap.is_empty(); }
+
+    size_type capacity() const
+    { return static_cast<size_type>(num_halfedges(mmap.get_fg())); }
+
+    bool is_used(size_type i) const
+    { return internal::Is_index_used<HEG>::run(mmap.get_fg(), i); }
+
+    size_type index(const_iterator it) const
+    {
+      return internal::Index_from_halfedge_descriptor<HEG>::
+        run(mmap.get_fg(), *it);
+    }
+
+    size_type index(Dart_const_handle it) const
+    {
+      return internal::Index_from_halfedge_descriptor<HEG>::
+        run(mmap.get_fg(), it);
+    }
+
+  private:
+    const Self & mmap;
+    mutable typename Self::size_type msize;
+  };
+  //**************************************************************************
+  // Dart_const_range // TODO REMOVE ?
+  /*  struct Dart_const_range {
+    typedef CGAL::FGW_dart_iterator_basic_of_all<Self, true> const_iterator;
+    Dart_const_range(const Self &amap) : mmap(amap), msize(0)
+    {}
+    const_iterator begin() const { return const_iterator(mmap); }
+    const_iterator end() const   { return const_iterator(mmap,mmap.null_handle); }
+    size_type size() const
+    {
+      if (msize==0)
+      {
+        for (const_iterator it=begin(), itend=end(); it!=itend; ++it)
+        { ++msize; }
+      }
+      return msize;
+    }
+
+    bool empty() const
+    { return mmap.is_empty(); }
+  private:
+    const Self & mmap;
+    mutable typename Self::size_type msize;
+    };*/
+  //**************************************************************************
+  Dart_range& darts()
+  { return mdarts; }
+  //--------------------------------------------------------------------------
+  const Dart_range& darts() const
+  { return mdarts; } // Before it was Dart_const_range(*this)
+  //**************************************************************************
+  Dart_handle dart_handle(size_type i)
+  {
+    CGAL_assertion(darts().is_used(i));
+    return internal::Halfedge_descriptor_from_index<HEG>::run(get_fg(), i);
+  }
+  Dart_const_handle dart_handle(size_type i) const
+  {
+    CGAL_assertion(darts().is_used(i));
+    return internal::Halfedge_descriptor_from_index<HEG>::run(get_fg(), i);
+  }
+
+  template <unsigned int i>
+  bool belong_to_same_cell(Dart_const_handle adart1,
+                           Dart_const_handle adart2) const
+  {
+    for (typename Dart_of_cell_range<i>::iterator it=darts_of_cell<i>(adart1).begin(),
+           itend=darts_of_cell<i>(adart1).end(); it!=itend; ++it)
+    { if (*it==adart2) { return true; } }
+    return false;
+  }
+
+  template <unsigned int i>
+  bool is_whole_cell_unmarked(Dart_const_handle adart, size_type amark) const
+  {
+    for (typename Dart_of_cell_range<i>::iterator it=darts_of_cell<i>(adart).begin(),
+           itend=darts_of_cell<i>(adart).end(); it!=itend; ++it)
+    { if (is_marked(*it, amark)) { return false; } }
+    return true;
+  }
+
+  template <unsigned int i>
+  bool is_whole_cell_marked(Dart_const_handle adart, size_type amark) const
+  {
+    for (typename Dart_of_cell_range<i>::iterator it=darts_of_cell<i>(adart).begin(),
+           itend=darts_of_cell<i>(adart).end(); it!=itend; ++it)
+    { if (!is_marked(*it, amark)) { return false; } }
+    return true;
+  }
+
+  template <unsigned int i>
+  size_type mark_cell(Dart_const_handle adart, size_type amark) const
+  {
+    size_type res=0;
+    for (typename Dart_of_cell_range<i>::iterator it=darts_of_cell<i>(adart).begin(),
+           itend=darts_of_cell<i>(adart).end(); it!=itend; ++it)
+    { mark(*it, amark); ++res; }
+    return res;
+}
+
+  size_type mark_cell(Dart_const_handle adart, unsigned int i, size_type amark) const
+  {
+    if (i==0) { return mark_cell<0>(adart, amark); }
+    else if (i==1) { return mark_cell<1>(adart, amark); }
+    else if (i==2) { return mark_cell<2>(adart, amark); }
+    return mark_cell<3>(adart, amark);
+  }
+
+  template <unsigned int i>
+  size_type unmark_cell(Dart_const_handle adart, size_type amark) const
+  {
+    size_type res=0;
+    for (typename Dart_of_cell_range<i>::iterator it=darts_of_cell<i>(adart).begin(),
+           itend=darts_of_cell<i>(adart).end(); it!=itend; ++it)
+    { unmark(*it, amark); ++res; }
+    return res;
+  }
+
+  template <unsigned int i>
+  size_type mark_oriented_cell(Dart_const_handle adart, size_type amark,
+                               size_type amark2=INVALID_MARK) const
+  {
+    size_type res=0;
+    for (typename Dart_of_cell_range<i>::iterator it=darts_of_cell<i>(adart).begin(),
+           itend=darts_of_cell<i>(adart).end(); it!=itend; ++it)
+    {
+      mark(*it, amark); ++res;
+      if (amark2!=INVALID_MARK) { mark(*it, amark2); }
+    }
+    return res;
+  }
+
+  template <unsigned int i>
+  size_type unmark_oriented_cell(Dart_const_handle adart, size_type amark,
+                                 size_type amark2=INVALID_MARK) const
+  {
+    size_type res=0;
+    for (typename Dart_of_cell_range<i>::iterator it=darts_of_cell<i>(adart).begin(),
+           itend=darts_of_cell<i>(adart).end(); it!=itend; ++it)
+    {
+      unmark(*it, amark); ++res;
+      if (amark2!=INVALID_MARK) { unmark(*it, amark2); }
+    }
+    return res;
+  }
+
+  std::size_t orient(size_type amark) const
+  { negate_mark(amark); return number_of_darts(); }
+
+  std::vector<unsigned int>
+  count_marked_cells(size_type amark, const std::vector<unsigned int>& acells) const
+  {
+    std::vector<unsigned int> res(dimension+2);
+    std::vector<size_type> marks(dimension+2);
+
+    // Initialization of the result
+    for (unsigned int i=0; i<dimension+2; ++i)
+    {
+      res[i]=0;
+      marks[i]=INVALID_MARK;
+    }
+
+    // Mark reservation
+    for (unsigned int i=0; i<acells.size(); ++i)
+    {
+      CGAL_assertion(acells[i]<=dimension+1);
+      if (marks[acells[i]]==INVALID_MARK )
+      {
+        marks[acells[i]]=get_new_mark();
+        assert(is_whole_map_unmarked(marks[acells[i]]));
+      }
+    }
+
+    // Counting and marking cells
+    for (typename Dart_range::const_iterator it(darts().begin()),
+           itend(darts().end()); it!=itend; ++it)
+    {
+      if (is_marked(*it, amark))
+      {
+        for (unsigned int i=0; i<acells.size(); ++i)
+        {
+          if (!is_marked(*it, marks[acells[i]]))
+          {
+            mark_cell(*it, acells[i], marks[acells[i]]);
+            ++res[acells[i]];
+          }
+        }
+      }
+    }
+
+    // Unmarking darts
+    std::vector<size_type> tounmark;
+    for (unsigned int i=0; i<acells.size(); ++i)
+    {
+      if (is_whole_map_marked(marks[acells[i]]) ||
+          is_whole_map_unmarked(marks[acells[i]]))
+      { free_mark(marks[acells[i]]); }
+      else
+      { tounmark.push_back(marks[acells[i]]); }
+    }
+
+    if (tounmark.size()>0)
+    {
+      for (typename Dart_range::const_iterator it(darts().begin()),
+             itend(darts().end()); it!=itend; ++it)
+      {
+        for (unsigned int i=0; i<tounmark.size(); ++i)
+        { unmark(*it, tounmark[i]); }
+      }
+      for (unsigned int i=0; i<tounmark.size(); ++i)
+      {
+        CGAL_assertion(is_whole_map_unmarked(tounmark[i]));
+        free_mark(tounmark[i]);
+      }
+    }
+
+    return res;
+  }
+
+  std::vector<unsigned int>
+  count_cells(const std::vector<unsigned int>& acells) const
+  {
+    std::vector<unsigned int> res;
+    size_type m=get_new_mark();
+    negate_mark(m); // We mark all the cells.
+
+    res=count_marked_cells(m, acells);
+
+    negate_mark(m); // We unmark the cells
+    free_mark(m);
+
+    return res;
+  }
+
+  std::vector<unsigned int> count_all_cells() const
+  {
+    std::vector<unsigned int> dim(dimension+2);
+
+    for ( unsigned int i=0; i<=dimension+1; ++i)
+    { dim[i]=i; }
+
+    return count_cells(dim);
+  }
+
+  std::ostream& display_characteristics(std::ostream & os) const
+  {
+    std::vector<unsigned int> cells(dimension+2);
+    for ( unsigned int i=0; i<=dimension+1; ++i)
+    { cells[i]=i; }
+
+    std::vector<unsigned int> res=count_cells(cells);
+
+    os<<"#Darts="<<number_of_darts();
+    for (unsigned int i=0; i<=dimension; ++i)
+      os<<", #"<<i<<"-cells="<<res[i];
+    os<<", #ccs="<<res[dimension+1];
+
+    return os;
+  }
+
+protected:
+  const HEG& m_fg;
+  Dart_range mdarts;
+  size_type m_nb_darts;
+  mutable bool m_marks_initialized; /// True iff marks are initialized (we use lazy initialization)
+
+  /// Number of times each mark is reserved. 0 if the mark is free.
+  mutable size_type mnb_times_reserved_marks[NB_MARKS];
+
+  /// Mask marks to know the value of unmark dart, for each index i.
+  mutable std::bitset<NB_MARKS> mmask_marks;
+
+  /// Number of used marks.
+  mutable size_type mnb_used_marks;
+
+  /// Index of each mark, in mfree_marks_stack or in mfree_marks_stack.
+  mutable size_type mindex_marks[NB_MARKS];
+
+  /// "Stack" of free marks.
+  mutable size_type mfree_marks_stack[NB_MARKS];
+
+  /// "Stack" of used marks.
+  mutable size_type mused_marks_stack[NB_MARKS];
+
+  /// Number of marked darts for each used marks.
+  mutable size_type mnb_marked_darts[NB_MARKS];
+
+  /// Array of property maps; one for each reserved mark.
+  typedef typename boost::property_map
+  <HEG, CGAL::dynamic_halfedge_property_t<std::bitset<NB_MARKS> > >::const_type MarkPMap;
+  mutable MarkPMap m_all_marks;
+};
+
+  /// null_handle
+  // template <typename HEG>
+  // const typename Face_graph_wrapper<HEG>::Null_handle_type
+  // Face_graph_wrapper<HEG>::null_handle=nullptr;
+  template <typename HEG>
+  const typename Face_graph_wrapper<HEG>::Null_handle_type
+  Face_graph_wrapper<HEG>::null_handle=typename Face_graph_wrapper<HEG>::Dart_handle();
+
+  /// null_dart_handle
+  // template <typename HEG>
+  // const typename Face_graph_wrapper<HEG>::Null_handle_type
+  // Face_graph_wrapper<HEG>::null_dart_handle=nullptr;
+  template <typename HEG>
+  const typename Face_graph_wrapper<HEG>::Null_handle_type
+  Face_graph_wrapper<HEG>::null_dart_handle=typename Face_graph_wrapper<HEG>::Dart_handle();
+
+  template<class Base, class HEG>
+  struct Get_map
+  {
+    typedef Face_graph_wrapper<HEG>       type;
+    typedef const Face_graph_wrapper<HEG> storage_type;
+    Get_map(const HEG& heg): m_map(heg) {}
+    static const HEG& get_mesh(const storage_type& amap)
+    { return amap.get_fg(); }
+
+    storage_type m_map;
+  };
+
+  template <unsigned int d, typename Refs, typename Items, typename Alloc,
+            typename Storage, class Map>
+  struct Get_map<CGAL::Combinatorial_map_base<d, Refs, Items, Alloc, Storage>, Map>
+  {
+    typedef Map        type;
+    typedef const Map& storage_type;
+    Get_map(const Map& heg): m_map(heg) {}
+    static const Map& get_mesh(storage_type& amap)
+    { return amap; }
+   storage_type m_map;
+  };
+
+  template <unsigned int d, typename Refs, typename Items, typename Alloc,
+            typename Storage, class Map>
+  struct Get_map<CGAL::Generalized_map_base<d, Refs, Items, Alloc, Storage>, Map>
+  {
+    typedef Map        type;
+    typedef const Map& storage_type;
+    Get_map(const Map& heg): m_map(heg) {}
+    static const Map& get_mesh(storage_type& amap)
+    { return amap; }
+    storage_type m_map;
+  };
+
+  template <unsigned int d, unsigned int d2, typename Traits, typename Items,
+            typename Alloc,
+            template<unsigned int,class,class,class,class>
+            class Map, typename Refs, typename Storage, class LCC>
+  struct Get_map<CGAL::Linear_cell_complex_base<d, d2, Traits, Items, Alloc,
+      Map, Refs, Storage>, LCC>
+  {
+    typedef LCC        type;
+    typedef const LCC& storage_type;
+    Get_map(const LCC& heg): m_map(heg) {}
+     static const LCC& get_mesh(storage_type& amap)
+    { return amap; }
+    storage_type m_map;
+  };
+
+  template <unsigned int d, typename Items, typename Alloc,
+            typename Storage, class Map>
+  struct Get_map<CGAL::Combinatorial_map<d, Items, Alloc, Storage>, Map>
+  {
+    typedef Map        type;
+    typedef const Map& storage_type;
+    Get_map(const Map& heg): m_map(heg) {}
+     static const Map& get_mesh(storage_type& amap)
+    { return amap; }
+    storage_type m_map;
+  };
+
+  template <typename Items, typename Alloc, typename Storage, class Map>
+  struct Get_map<CGAL::Surface_mesh_topology::
+                 Polygonal_schema_with_combinatorial_map<Items, Alloc, Storage>, Map>
+  {
+    typedef Map        type;
+    typedef const Map& storage_type;
+    Get_map(const Map& heg): m_map(heg) {}
+     static const Map& get_mesh(storage_type& amap)
+    { return amap; }
+   storage_type m_map;
+  };
+
+  template <unsigned int d, typename Items, typename Alloc,
+            typename Storage, class Map>
+  struct Get_map<CGAL::Generalized_map<d, Items, Alloc, Storage>, Map>
+  {
+    typedef Map        type;
+    typedef const Map& storage_type;
+    Get_map(const Map& heg): m_map(heg) {}
+    static const Map& get_mesh(storage_type& amap)
+    { return amap; }
+    storage_type m_map;
+  };
+
+  template <typename Items, typename Alloc, typename Storage, class Map>
+  struct Get_map<CGAL::Surface_mesh_topology::
+                 Polygonal_schema_with_generalized_map<Items, Alloc, Storage>, Map>
+  {
+    typedef Map        type;
+    typedef const Map& storage_type;
+    Get_map(const Map& heg): m_map(heg) {}
+    static const Map& get_mesh(storage_type& amap)
+    { return amap; }
+    storage_type m_map;
+  };
+
+  template <unsigned int d, unsigned int d2, typename Traits, typename Items,
+            typename Alloc,
+            template<unsigned int,class,class,class,class>
+            class Map, typename Storage, class LCC>
+  struct Get_map<CGAL::Linear_cell_complex_for_combinatorial_map
+      <d, d2, Traits, Items, Alloc, Map, Storage>, LCC>
+  {
+    typedef LCC        type;
+    typedef const LCC& storage_type;
+    Get_map(const LCC& heg): m_map(heg) {}
+    static const LCC& get_mesh(storage_type& amap)
+    { return amap; }
+    storage_type m_map;
+  };
+
+  template <unsigned int d, unsigned int d2, typename Traits, typename Items,
+            typename Alloc,
+            template<unsigned int,class,class,class,class>
+            class Map, typename Storage, class LCC>
+  struct Get_map<CGAL::Linear_cell_complex_for_generalized_map
+      <d, d2, Traits, Items, Alloc, Map, Storage>, LCC>
+  {
+    typedef LCC        type;
+    typedef const LCC& storage_type;
+    Get_map(const LCC& heg): m_map(heg) {}
+    static const LCC& get_mesh(storage_type& amap)
+    { return amap; }
+    storage_type m_map;
+  };
+
+  template<class Mesh_>
+  struct Get_traits
+  {
+    typedef Mesh_                 Mesh;
+    typedef typename Mesh::Traits Kernel;
+    typedef typename Mesh::Point  Point;
+    typedef typename Mesh::Vector Vector;
+
+    template<class Dart_handle>
+    static const Point& get_point(const Mesh& m, Dart_handle dh)
+    { return m.point(dh); }
+  };
+
+  template<class P>
+  struct Get_traits<CGAL::Surface_mesh<P> >
+  {
+    typedef CGAL::Surface_mesh<P>                   Mesh;
+    typedef typename CGAL::Kernel_traits<P>::Kernel Kernel;
+    typedef typename Kernel::Point_3                Point;
+    typedef typename Kernel::Vector_3               Vector;
+
+    template<class Dart_handle>
+    static const Point& get_point(const Mesh& m, Dart_handle dh)
+    { return m.point(m.source(dh)); }
+  };
+
+  template<class PolyhedronTraits_3,
+      class PolyhedronItems_3,
+      template<class T, class I, class A> class T_HDS,
+      class Alloc>
+  struct Get_traits<CGAL::Polyhedron_3<PolyhedronTraits_3,
+      PolyhedronItems_3, T_HDS, Alloc> >
+  {
+    typedef CGAL::Polyhedron_3<PolyhedronTraits_3, PolyhedronItems_3,
+    T_HDS, Alloc>                    Mesh;
+    typedef PolyhedronTraits_3        Kernel;
+    typedef typename Kernel::Point_3  Point;
+    typedef typename Kernel::Vector_3 Vector;
+
+    template<class Dart_handle>
+    static const Point& get_point(const Mesh& /*m*/, Dart_handle dh)
+    { return dh->opposite()->vertex()->point(); }
+  };
+
+} // Namespace CGAL
+
+#endif // CGAL_FACE_GRAPH_WRAPPER_H //
+// EOF //
diff -pruN 5.0.3-1/include/CGAL/Filter_circulator.h 5.1-1/include/CGAL/Filter_circulator.h
--- 5.0.3-1/include/CGAL/Filter_circulator.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Filter_circulator.h	2020-09-08 12:15:06.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_2/include/CGAL/Filter_circulator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_2/include/CGAL/Filter_circulator.h $
 // $Id: Filter_circulator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Filtered_bbox_circular_kernel_2/bbox_filtered_predicates.h 5.1-1/include/CGAL/Filtered_bbox_circular_kernel_2/bbox_filtered_predicates.h
--- 5.0.3-1/include/CGAL/Filtered_bbox_circular_kernel_2/bbox_filtered_predicates.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Filtered_bbox_circular_kernel_2/bbox_filtered_predicates.h	2020-09-08 12:14:46.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_2/include/CGAL/Filtered_bbox_circular_kernel_2/bbox_filtered_predicates.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_2/include/CGAL/Filtered_bbox_circular_kernel_2/bbox_filtered_predicates.h $
 // $Id: bbox_filtered_predicates.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Filtered_bbox_circular_kernel_2/interface_macros.h 5.1-1/include/CGAL/Filtered_bbox_circular_kernel_2/interface_macros.h
--- 5.0.3-1/include/CGAL/Filtered_bbox_circular_kernel_2/interface_macros.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Filtered_bbox_circular_kernel_2/interface_macros.h	2020-09-08 12:14:46.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_2/include/CGAL/Filtered_bbox_circular_kernel_2/interface_macros.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_2/include/CGAL/Filtered_bbox_circular_kernel_2/interface_macros.h $
 // $Id: interface_macros.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Filtered_bbox_circular_kernel_2.h 5.1-1/include/CGAL/Filtered_bbox_circular_kernel_2.h
--- 5.0.3-1/include/CGAL/Filtered_bbox_circular_kernel_2.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Filtered_bbox_circular_kernel_2.h	2020-09-08 12:14:46.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_2/include/CGAL/Filtered_bbox_circular_kernel_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_2/include/CGAL/Filtered_bbox_circular_kernel_2.h $
 // $Id: Filtered_bbox_circular_kernel_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Filtered_construction.h 5.1-1/include/CGAL/Filtered_construction.h
--- 5.0.3-1/include/CGAL/Filtered_construction.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Filtered_construction.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/include/CGAL/Filtered_construction.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/Filtered_construction.h $
 // $Id: Filtered_construction.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Filtered_extended_homogeneous.h 5.1-1/include/CGAL/Filtered_extended_homogeneous.h
--- 5.0.3-1/include/CGAL/Filtered_extended_homogeneous.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Filtered_extended_homogeneous.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_2/include/CGAL/Filtered_extended_homogeneous.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_2/include/CGAL/Filtered_extended_homogeneous.h $
 // $Id: Filtered_extended_homogeneous.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Filtered_kernel/Cartesian_coordinate_iterator_2.h 5.1-1/include/CGAL/Filtered_kernel/Cartesian_coordinate_iterator_2.h
--- 5.0.3-1/include/CGAL/Filtered_kernel/Cartesian_coordinate_iterator_2.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Filtered_kernel/Cartesian_coordinate_iterator_2.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/include/CGAL/Filtered_kernel/Cartesian_coordinate_iterator_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/Filtered_kernel/Cartesian_coordinate_iterator_2.h $
 // $Id: Cartesian_coordinate_iterator_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Filtered_kernel/Cartesian_coordinate_iterator_3.h 5.1-1/include/CGAL/Filtered_kernel/Cartesian_coordinate_iterator_3.h
--- 5.0.3-1/include/CGAL/Filtered_kernel/Cartesian_coordinate_iterator_3.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Filtered_kernel/Cartesian_coordinate_iterator_3.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/include/CGAL/Filtered_kernel/Cartesian_coordinate_iterator_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/Filtered_kernel/Cartesian_coordinate_iterator_3.h $
 // $Id: Cartesian_coordinate_iterator_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Filtered_kernel_d.h 5.1-1/include/CGAL/Filtered_kernel_d.h
--- 5.0.3-1/include/CGAL/Filtered_kernel_d.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Filtered_kernel_d.h	2020-09-08 12:15:03.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Filtered_kernel_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Filtered_kernel_d.h $
 // $Id: Filtered_kernel_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Filtered_kernel_fwd.h 5.1-1/include/CGAL/Filtered_kernel_fwd.h
--- 5.0.3-1/include/CGAL/Filtered_kernel_fwd.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Filtered_kernel_fwd.h	2020-09-08 12:14:50.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/include/CGAL/Filtered_kernel_fwd.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/Filtered_kernel_fwd.h $
 // $Id: Filtered_kernel_fwd.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Filtered_kernel.h 5.1-1/include/CGAL/Filtered_kernel.h
--- 5.0.3-1/include/CGAL/Filtered_kernel.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Filtered_kernel.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/include/CGAL/Filtered_kernel.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/Filtered_kernel.h $
 // $Id: Filtered_kernel.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Filtered_predicate.h 5.1-1/include/CGAL/Filtered_predicate.h
--- 5.0.3-1/include/CGAL/Filtered_predicate.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Filtered_predicate.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/include/CGAL/Filtered_predicate.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/Filtered_predicate.h $
 // $Id: Filtered_predicate.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Filtered_predicate_with_state.h 5.1-1/include/CGAL/Filtered_predicate_with_state.h
--- 5.0.3-1/include/CGAL/Filtered_predicate_with_state.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Filtered_predicate_with_state.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/include/CGAL/Filtered_predicate_with_state.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/Filtered_predicate_with_state.h $
 // $Id: Filtered_predicate_with_state.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Fixed_alpha_shape_3.h 5.1-1/include/CGAL/Fixed_alpha_shape_3.h
--- 5.0.3-1/include/CGAL/Fixed_alpha_shape_3.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Fixed_alpha_shape_3.h	2020-09-08 12:14:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Alpha_shapes_3/include/CGAL/Fixed_alpha_shape_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Alpha_shapes_3/include/CGAL/Fixed_alpha_shape_3.h $
 // $Id: Fixed_alpha_shape_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Fixed_alpha_shape_cell_base_3.h 5.1-1/include/CGAL/Fixed_alpha_shape_cell_base_3.h
--- 5.0.3-1/include/CGAL/Fixed_alpha_shape_cell_base_3.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Fixed_alpha_shape_cell_base_3.h	2020-09-08 12:14:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Alpha_shapes_3/include/CGAL/Fixed_alpha_shape_cell_base_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Alpha_shapes_3/include/CGAL/Fixed_alpha_shape_cell_base_3.h $
 // $Id: Fixed_alpha_shape_cell_base_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Fixed_alpha_shape_vertex_base_3.h 5.1-1/include/CGAL/Fixed_alpha_shape_vertex_base_3.h
--- 5.0.3-1/include/CGAL/Fixed_alpha_shape_vertex_base_3.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Fixed_alpha_shape_vertex_base_3.h	2020-09-08 12:14:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Alpha_shapes_3/include/CGAL/Fixed_alpha_shape_vertex_base_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Alpha_shapes_3/include/CGAL/Fixed_alpha_shape_vertex_base_3.h $
 // $Id: Fixed_alpha_shape_vertex_base_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Flattening_iterator.h 5.1-1/include/CGAL/Flattening_iterator.h
--- 5.0.3-1/include/CGAL/Flattening_iterator.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Flattening_iterator.h	2020-09-08 12:15:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Flattening_iterator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Flattening_iterator.h $
 // $Id: Flattening_iterator.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/float.h 5.1-1/include/CGAL/float.h
--- 5.0.3-1/include/CGAL/float.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/float.h	2020-09-08 12:15:16.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/float.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/float.h $
 // $Id: float.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/for_each.h 5.1-1/include/CGAL/for_each.h
--- 5.0.3-1/include/CGAL/for_each.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/for_each.h	2020-09-08 12:15:32.000000000 +0000
@@ -0,0 +1,134 @@
+// Copyright (c) 2020 GeometryFactory (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/for_each.h $
+// $Id: for_each.h 6ae64e7 2020-03-27T16:41:53+01:00 Andreas Fabri
+// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s) : Simon Giraudot
+
+#ifndef CGAL_FOR_EACH_H
+#define CGAL_FOR_EACH_H
+
+#include <CGAL/iterator.h>
+
+#ifdef CGAL_LINKED_WITH_TBB
+#include <tbb/parallel_for.h>
+#include <tbb/blocked_range.h>
+#include <tbb/scalable_allocator.h>
+#endif // CGAL_LINKED_WITH_TBB
+
+/*
+  CGAL::for_each<ConcurrencyTag>(Range, Function) does the following:
+
+  - if Sequential_tag is used, apply Function to all elements of Range
+  - if Parallel_tag is used:
+    * static assert that TBB is available
+    * if Range has random access iterators, use a TBB parallel_for
+    loop to apply Function to all elements of Range
+    * if Range doesn't have random access iterators, first copy the
+    iterators in a vector and then use a TBB parallel_for loop to
+    apply Function to all elements of Range
+
+  The loop is interrupted if `functor` returns false (it carries on
+  until the end otherwise).
+*/
+
+namespace CGAL {
+
+namespace internal {
+
+template <typename RangeRef, typename IteratorCategory>
+void for_each (RangeRef range,
+               const std::function<bool(typename std::iterator_traits
+                                        <typename Range_iterator_type<RangeRef>::type>::reference)>& functor,
+               const Sequential_tag&,
+               IteratorCategory)
+{
+  for (typename std::iterator_traits
+         <typename Range_iterator_type<RangeRef>::type>::reference r : range)
+    if (!functor(r))
+      break;
+}
+
+#ifdef CGAL_LINKED_WITH_TBB
+template <typename RangeRef, typename Fct, typename IteratorCategory>
+void for_each (RangeRef range,
+               const Fct& functor,
+               const Parallel_tag&,
+               IteratorCategory)
+{
+  std::size_t range_size = std::distance (range.begin(), range.end());
+
+  std::vector<typename Range_iterator_type<RangeRef>::type> iterators;
+  iterators.reserve (range_size);
+  for (typename Range_iterator_type<RangeRef>::type it = range.begin(); it != range.end(); ++ it)
+    iterators.push_back (it);
+
+  tbb::parallel_for (tbb::blocked_range<std::size_t>(0, range_size),
+                     [&](const tbb::blocked_range<std::size_t>& r)
+                     {
+                       for (std::size_t i = r.begin(); i != r.end(); ++ i)
+                         if (!functor (*(iterators[i])))
+                           break;
+                     });
+}
+
+template <typename RangeRef, typename Fct>
+void for_each (RangeRef range,
+               const Fct& functor,
+               const Parallel_tag&,
+               std::random_access_iterator_tag)
+{
+  std::size_t range_size = std::distance (range.begin(), range.end());
+
+  tbb::parallel_for (tbb::blocked_range<std::size_t>(0, range_size),
+                     [&](const tbb::blocked_range<std::size_t>& r)
+                     {
+                       for (std::size_t i = r.begin(); i != r.end(); ++ i)
+                         if (!functor (*(range.begin() + i)))
+                           break;
+                     });
+}
+#endif
+
+} // namespace internal
+
+template <typename ConcurrencyTag, typename Range>
+void for_each (const Range& range,
+               const std::function<bool(typename std::iterator_traits
+                                        <typename Range::const_iterator>::reference)>& functor)
+{
+#ifndef CGAL_LINKED_WITH_TBB
+  CGAL_static_assertion_msg (!(boost::is_convertible<ConcurrencyTag, Parallel_tag>::value),
+                             "Parallel_tag is enabled but TBB is unavailable.");
+#endif
+
+  internal::for_each<const Range&>
+    (range, functor,
+     ConcurrencyTag(),
+     typename std::iterator_traits<typename Range::const_iterator>::iterator_category());
+}
+
+template <typename ConcurrencyTag, typename Range>
+void for_each (Range& range,
+               const std::function<bool(typename std::iterator_traits
+                                        <typename Range::iterator>::reference)>& functor)
+{
+#ifndef CGAL_LINKED_WITH_TBB
+  CGAL_static_assertion_msg (!(boost::is_convertible<ConcurrencyTag, Parallel_tag>::value),
+                             "Parallel_tag is enabled but TBB is unavailable.");
+#endif
+
+  internal::for_each<Range&>
+    (range, functor,
+     ConcurrencyTag(),
+     typename std::iterator_traits<typename Range::iterator>::iterator_category());
+}
+
+} // namespace CGAL
+
+
+#endif // CGAL_FOR_EACH_H
diff -pruN 5.0.3-1/include/CGAL/Fourtuple.h 5.1-1/include/CGAL/Fourtuple.h
--- 5.0.3-1/include/CGAL/Fourtuple.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Fourtuple.h	2020-09-08 12:15:32.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Fourtuple.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Fourtuple.h $
 // $Id: Fourtuple.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/FPU_extension.h 5.1-1/include/CGAL/FPU_extension.h
--- 5.0.3-1/include/CGAL/FPU_extension.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/FPU_extension.h	2020-09-08 12:15:16.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/FPU_extension.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/FPU_extension.h $
 // $Id: FPU_extension.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/FPU_gcc_i386.h 5.1-1/include/CGAL/FPU_gcc_i386.h
--- 5.0.3-1/include/CGAL/FPU_gcc_i386.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/FPU_gcc_i386.h	2020-09-08 12:15:16.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/FPU_gcc_i386.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/FPU_gcc_i386.h $
 // $Id: FPU_gcc_i386.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/FPU_gcc_i386_sse2.h 5.1-1/include/CGAL/FPU_gcc_i386_sse2.h
--- 5.0.3-1/include/CGAL/FPU_gcc_i386_sse2.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/FPU_gcc_i386_sse2.h	2020-09-08 12:15:16.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/FPU_gcc_i386_sse2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/FPU_gcc_i386_sse2.h $
 // $Id: FPU_gcc_i386_sse2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/FPU.h 5.1-1/include/CGAL/FPU.h
--- 5.0.3-1/include/CGAL/FPU.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/FPU.h	2020-09-08 12:15:16.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/FPU.h $
-// $Id: FPU.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/FPU.h $
+// $Id: FPU.h 406b6fa 2020-02-11T13:59:56+01:00 Laurent Rineau
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -408,7 +408,7 @@ typedef unsigned int FPU_CW_t;
 
 #elif defined __mips__
 #define CGAL_IA_SETFPCW(CW) asm volatile ("ctc1 %0,$31" : :"r" (CW))
-#define CGAL_IA_GETFPCW(CW) asm volatile ("cfc1 %0,$31" : "=r" (CW))
+#define CGAL_IA_GETFPCW(CW) asm volatile ("cfc1 %0,$31" : "=r" (CW)); CW &= 3
 typedef unsigned int FPU_CW_t;
 #define CGAL_FE_TONEAREST    (0x0)
 #define CGAL_FE_TOWARDZERO   (0x1)
diff -pruN 5.0.3-1/include/CGAL/FPU_msvc.h 5.1-1/include/CGAL/FPU_msvc.h
--- 5.0.3-1/include/CGAL/FPU_msvc.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/FPU_msvc.h	2020-09-08 12:15:16.000000000 +0000
@@ -5,7 +5,7 @@
 // Licensees holding a valid commercial license may use this file in
 // accordance with the commercial license agreement provided with the software.
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/FPU_msvc.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/FPU_msvc.h $
 // $Id: FPU_msvc.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Fraction_traits.h 5.1-1/include/CGAL/Fraction_traits.h
--- 5.0.3-1/include/CGAL/Fraction_traits.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/Fraction_traits.h	2020-09-08 12:14:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_foundations/include/CGAL/Fraction_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_foundations/include/CGAL/Fraction_traits.h $
 // $Id: Fraction_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/functional.h 5.1-1/include/CGAL/functional.h
--- 5.0.3-1/include/CGAL/functional.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/functional.h	2020-09-08 12:15:32.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/functional.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/functional.h $
 // $Id: functional.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/function_objects.h 5.1-1/include/CGAL/function_objects.h
--- 5.0.3-1/include/CGAL/function_objects.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/function_objects.h	2020-09-08 12:15:32.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/function_objects.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/function_objects.h $
 // $Id: function_objects.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/functions_on_enums.h 5.1-1/include/CGAL/functions_on_enums.h
--- 5.0.3-1/include/CGAL/functions_on_enums.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/functions_on_enums.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/functions_on_enums.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/functions_on_enums.h $
 // $Id: functions_on_enums.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/functions_on_signs.h 5.1-1/include/CGAL/functions_on_signs.h
--- 5.0.3-1/include/CGAL/functions_on_signs.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/functions_on_signs.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/functions_on_signs.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/functions_on_signs.h $
 // $Id: functions_on_signs.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Fuzzy_iso_box.h 5.1-1/include/CGAL/Fuzzy_iso_box.h
--- 5.0.3-1/include/CGAL/Fuzzy_iso_box.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Fuzzy_iso_box.h	2020-09-08 12:15:39.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_searching/include/CGAL/Fuzzy_iso_box.h $
-// $Id: Fuzzy_iso_box.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_searching/include/CGAL/Fuzzy_iso_box.h $
+// $Id: Fuzzy_iso_box.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -120,6 +120,17 @@ namespace CGAL {
     return true;
   }
 
+  template <typename Coord_iterator>
+  bool contains_point_given_as_coordinates(Coord_iterator it_coord_begin, Coord_iterator /*unused*/) const {
+          Construct_cartesian_const_iterator_d construct_it=traits.construct_cartesian_const_iterator_d_object();
+          Cartesian_const_iterator_d minit= min_begin, maxit = max_begin;
+                for (unsigned int i = 0; i < dim; ++i, ++it_coord_begin, ++minit, ++maxit) {
+                        if ( ((*it_coord_begin) < (*minit)) || ((*it_coord_begin) > (*maxit)) )
+        return false;
+    }
+    return true;
+  }
+
   bool inner_range_intersects(const Kd_tree_rectangle<FT,Dimension>& rectangle) const {
     // test whether the box eroded by 'eps' intersects 'rectangle'
     Cartesian_const_iterator_d minit= min_begin, maxit = max_begin;
@@ -131,7 +142,6 @@ namespace CGAL {
     return true;
   }
 
-
   bool outer_range_contains(const Kd_tree_rectangle<FT,Dimension>& rectangle) const {
     // test whether the box dilated by 'eps' contains 'rectangle'
     Cartesian_const_iterator_d minit= min_begin, maxit = max_begin;
diff -pruN 5.0.3-1/include/CGAL/Fuzzy_sphere.h 5.1-1/include/CGAL/Fuzzy_sphere.h
--- 5.0.3-1/include/CGAL/Fuzzy_sphere.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Fuzzy_sphere.h	2020-09-08 12:15:39.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_searching/include/CGAL/Fuzzy_sphere.h $
-// $Id: Fuzzy_sphere.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_searching/include/CGAL/Fuzzy_sphere.h $
+// $Id: Fuzzy_sphere.h 65cde3c 2019-12-03T19:18:15+01:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -72,6 +72,21 @@ public:
     return (distance <= sq_radius);
   }
 
+  template <typename Coord_iterator>
+  bool contains_point_given_as_coordinates(Coord_iterator it_coord_begin, Coord_iterator /*unused*/) const {
+    // test whether the distance between c and p is less than the radius
+    FT distance = FT(0);
+    typename SearchTraits::Construct_cartesian_const_iterator_d construct_it =
+      traits.construct_cartesian_const_iterator_d_object();
+    typename SearchTraits::Cartesian_const_iterator_d cit = construct_it(c),
+      end = construct_it(c, 0);
+
+    for (; cit != end && (distance <= sq_radius); ++cit, ++it_coord_begin)
+      distance += ((*cit) - (*it_coord_begin))*((*cit) - (*it_coord_begin));
+
+    return (distance < sq_radius);
+  }
+
   bool inner_range_intersects(const Kd_tree_rectangle<FT,Dimension>& rectangle) const {
     // test whether the sphere with radius (r-eps) intersects 'rectangle', i.e.
     // if the minimal distance of c to 'rectangle' is less than r-eps
diff -pruN 5.0.3-1/include/CGAL/Generalized_map_fwd.h 5.1-1/include/CGAL/Generalized_map_fwd.h
--- 5.0.3-1/include/CGAL/Generalized_map_fwd.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Generalized_map_fwd.h	2020-09-08 12:14:51.000000000 +0000
@@ -0,0 +1,38 @@
+// Copyright (c) 2016 CNRS and LIRIS' Establishments (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Generalized_map/include/CGAL/Generalized_map_fwd.h $
+// $Id: Generalized_map_fwd.h 0308d1a 2020-03-27T18:35:15+01:00 Guillaume Damiand
+// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
+//
+#ifndef GENERALIZED_MAP_FWD_H
+#define GENERALIZED_MAP_FWD_H 1
+
+#include <CGAL/memory.h>
+
+namespace CGAL {
+
+template<unsigned int d_, class Items_, class Alloc_ >
+class Generalized_map_storage_1;
+
+struct Generic_map_min_items;
+
+template < unsigned int d_, class Refs,
+           class Items_=Generic_map_min_items,
+           class Alloc_=CGAL_ALLOCATOR(int),
+           class Storage_= Generalized_map_storage_1<d_, Items_, Alloc_> >
+class Generalized_map_base;
+
+template < unsigned int d_,
+           class Items_=Generic_map_min_items,
+           class Alloc_=CGAL_ALLOCATOR(int),
+           class Storage_= Generalized_map_storage_1<d_, Items_, Alloc_> >
+class Generalized_map;
+
+} // CGAL
+
+#endif // GENERALIZED_MAP_FWD_H
diff -pruN 5.0.3-1/include/CGAL/Generalized_map.h 5.1-1/include/CGAL/Generalized_map.h
--- 5.0.3-1/include/CGAL/Generalized_map.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Generalized_map.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Generalized_map/include/CGAL/Generalized_map.h $
-// $Id: Generalized_map.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Generalized_map/include/CGAL/Generalized_map.h $
+// $Id: Generalized_map.h e6536aa 2020-06-09T20:15:16+02:00 Laurent Rineau
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
@@ -12,6 +12,7 @@
 #ifndef CGAL_GENERALIZED_MAP_H
 #define CGAL_GENERALIZED_MAP_H 1
 
+#include <CGAL/Generalized_map_fwd.h>
 #include <CGAL/internal/Combinatorial_map_utility.h>
 #include <CGAL/internal/Generalized_map_group_functors.h>
 #include <CGAL/internal/Combinatorial_map_copy_functors.h>
@@ -30,6 +31,7 @@
 #include <bitset>
 #include <vector>
 #include <deque>
+#include <tuple>
 #include <boost/type_traits/is_same.hpp>
 #include <boost/unordered_map.hpp>
 #include <CGAL/config.h>
@@ -51,6 +53,7 @@ namespace CGAL {
    * Definition of generic dD Generalized map.
    */
 
+  struct Combinatorial_map_tag;
   struct Generalized_map_tag {};
 
   /** Generic definition of generalized map in dD.
@@ -58,10 +61,7 @@ namespace CGAL {
    * mainly to create darts, to use marks onto these darts, to get and set
    * the alpha links, and to manage enabled attributes.
    */
-  template < unsigned int d_, class Refs,
-             class Items_=Generic_map_min_items,
-             class Alloc_=CGAL_ALLOCATOR(int),
-             class Storage_= Generalized_map_storage_1<d_, Items_, Alloc_> >
+  template<unsigned int d_, class Refs_, class Items_, class Alloc_, class Storage_>
   class Generalized_map_base: public Storage_
   {
     template<typename Gmap,unsigned int i,typename Enabled>
@@ -84,9 +84,10 @@ namespace CGAL {
     typedef Generalized_map_tag Combinatorial_data_structure;
 
     /// Types definition
-    typedef Storage_                                                  Storage;
-    typedef Storage                                                   Base;
-    typedef Generalized_map_base<d_, Refs, Items_, Alloc_, Storage_ > Self;
+    typedef Storage_                                                   Storage;
+    typedef Storage                                                    Base;
+    typedef Generalized_map_base<d_, Refs_, Items_, Alloc_, Storage_ > Self;
+    typedef Refs_                                                      Refs;
 
     typedef typename Base::Dart Dart;
     typedef typename Base::Dart_handle Dart_handle;
@@ -106,7 +107,7 @@ namespace CGAL {
 
     typedef typename Base::Null_handle_type Null_handle_type;
     using Base::null_handle;
-
+    using Base::null_dart_handle;
     using Base::mdarts;
     using Base::get_alpha;
     using Base::is_free;
@@ -177,20 +178,35 @@ namespace CGAL {
     }
 
     /** Copy the given generalized map into *this.
-     *  Note that both Gmap can have different dimensions and/or non void attributes.
+     *  Note that both GMap can have different dimensions and/or non void attributes.
+     *  Here GMap2 is necessarily non const; while Dart_handle_2 can be a const or non const handle.
+     *  This is the "generic" method, called by the different variants below.
+     *  Marks reserved and automatic attributes management are not updated.
      *  @param amap the generalized map to copy.
+     *  @param origin_to_copy associative array from original darts to copy darts
+     *  @param origin_to_copy associative array from copy darts to original darts
+     *  @param converters tuple of functors, one per attribute, to transform original attributes into copies
+     *  @param dartinfoconverter functor to transform original information of darts into information of copies
+     *  @param pointconverter functor to transform points in original map into points of copies.
+     *  @param copy_perforated_darts true to copy also darts marked perforated (if any)
+     *  @param mark_perforated_darts true to mark darts wich are copies of perforated darts (if any)
      *  @post *this is valid.
      */
-    template <typename GMap2, typename Converters, typename DartInfoConverter,
+    template <typename GMap2, typename Dart_handle_2,
+              typename Converters, typename DartInfoConverter,
               typename PointConverter>
-    void copy(const GMap2& amap, const Converters& converters,
-              const DartInfoConverter& dartinfoconverter,
-              const PointConverter& pointconverter,
-              boost::unordered_map<typename GMap2::Dart_const_handle, Dart_handle>* dart_mapping=nullptr)
+    void generic_copy(GMap2& amap,
+                      boost::unordered_map<Dart_handle_2, Dart_handle>* origin_to_copy,
+                      boost::unordered_map<Dart_handle, Dart_handle_2>* copy_to_origin,
+                      const Converters& converters,
+                      const DartInfoConverter& dartinfoconverter,
+                      const PointConverter& pointconverter,
+                      bool copy_perforated_darts=false,
+                      size_type mark_perforated=INVALID_MARK)
     {
       this->clear();
 
-      this->mnb_used_marks = amap.mnb_used_marks;
+      /*this->mnb_used_marks = amap.mnb_used_marks;
       this->mmask_marks    = amap.mmask_marks;
       this->automatic_attributes_management =
           amap.automatic_attributes_management;
@@ -202,121 +218,257 @@ namespace CGAL {
         this->mindex_marks[i]             = amap.mindex_marks[i];
         this->mnb_marked_darts[i]         = amap.mnb_marked_darts[i];
         this->mnb_times_reserved_marks[i] = amap.mnb_times_reserved_marks[i];
-      }
+      } */
 
       // Create an mapping between darts of the two maps (originals->copies).
       // (here we cannot use CGAL::Unique_hash_map because it does not provide
       // iterators...
-      boost::unordered_map<typename GMap2::Dart_const_handle, Dart_handle> local_dartmap;
-      if (dart_mapping==nullptr)
-      { dart_mapping=&local_dartmap; }
+      boost::unordered_map<Dart_handle_2, Dart_handle> local_dartmap;
+      if (origin_to_copy==NULL) // Use local_dartmap if user does not provides its own unordered_map
+      { origin_to_copy=&local_dartmap; }
+
+      Dart_handle new_dart;
+      for (typename GMap2::Dart_range::iterator it=amap.darts().begin(),
+             itend=amap.darts().end(); it!=itend; ++it)
+      {
+        if (copy_perforated_darts || !amap.is_perforated(it))
+        {
+          new_dart=mdarts.emplace();
+          init_dart(new_dart); //, amap.get_marks(it));
+
+          if (mark_perforated!=INVALID_MARK && amap.is_perforated(it))
+          { mark(new_dart, mark_perforated); }
 
+          (*origin_to_copy)[it]=new_dart;
+          if (copy_to_origin!=NULL) { (*copy_to_origin)[new_dart]=it; }
 
-      for (typename GMap2::Dart_const_range::const_iterator
-             it=amap.darts().begin(), itend=amap.darts().end();
-           it!=itend; ++it)
-      {
-        (*dart_mapping)[it]=mdarts.emplace();
-        init_dart((*dart_mapping)[it], amap.get_marks(it));
-        internal::Copy_dart_info_functor<GMap2, Refs, DartInfoConverter>::
-          run(amap, static_cast<Refs&>(*this), it, (*dart_mapping)[it],
-              dartinfoconverter);
+          internal::Copy_dart_info_functor
+            <typename GMap2::Refs, Refs, DartInfoConverter>::run
+            (static_cast<const typename GMap2::Refs&>(amap), static_cast<Refs&>(*this),
+             it, new_dart, dartinfoconverter);
+        }
       }
 
       unsigned int min_dim=(dimension<amap.dimension?dimension:amap.dimension);
 
-      typename boost::unordered_map<typename GMap2::Dart_const_handle,Dart_handle>
-        ::iterator dartmap_iter, dartmap_iter_end=dart_mapping->end();
-      for (dartmap_iter=dart_mapping->begin(); dartmap_iter!=dartmap_iter_end;
+      typename boost::unordered_map<Dart_handle_2, Dart_handle>::iterator
+        dartmap_iter, dartmap_iter_end=origin_to_copy->end();
+      for (dartmap_iter=origin_to_copy->begin(); dartmap_iter!=dartmap_iter_end;
            ++dartmap_iter)
       {
         for (unsigned int i=0; i<=min_dim; i++)
         {
           if (!amap.is_free(dartmap_iter->first,i) &&
-              (dartmap_iter->first)<(amap.alpha(dartmap_iter->first,i)))
+              is_free(dartmap_iter->second,i))
           {
             basic_link_alpha(dartmap_iter->second,
-                            (*dart_mapping)[amap.alpha(dartmap_iter->first,i)], i);
+                            (*origin_to_copy)[amap.alpha(dartmap_iter->first,i)], i);
           }
         }
       }
 
       /** Copy attributes */
-      for (dartmap_iter=dart_mapping->begin(); dartmap_iter!=dartmap_iter_end;
+      for (dartmap_iter=origin_to_copy->begin(); dartmap_iter!=dartmap_iter_end;
            ++dartmap_iter)
       {
         Helper::template Foreach_enabled_attributes
-          < internal::Copy_attributes_functor <GMap2, Refs, Converters,
-            PointConverter> >::
-          run(amap, static_cast<Refs&>(*this),
+          < internal::Copy_attributes_functor<typename GMap2::Refs, Refs,
+                                              Converters, PointConverter> >::
+          run(static_cast<const typename GMap2::Refs&>(amap), static_cast<Refs&>(*this),
               dartmap_iter->first, dartmap_iter->second,
               converters, pointconverter);
       }
 
-      CGAL_assertion (is_valid () == 1);
+      CGAL_assertion(is_valid());
     }
 
-    template <typename GMap2>
-    void copy(const GMap2& amap,
-              boost::unordered_map<typename GMap2::Dart_const_handle, Dart_handle>* dart_mapping=nullptr)
+    // (1a) copy(amap, converters, dartinfoconverter, pointconverter)
+    template<typename GMap2, typename Converters, typename DartInfoConverter,
+             typename PointConverter>
+    void copy(GMap2& amap,
+              boost::unordered_map
+              <typename GMap2::Dart_handle, Dart_handle>* origin_to_copy,
+              boost::unordered_map
+              <Dart_handle, typename GMap2::Dart_handle>* copy_to_origin,
+              const Converters& converters,
+              const DartInfoConverter& dartinfoconverter,
+              const PointConverter& pointconverter,
+              bool copy_perforated_darts=false,
+              size_type mark_perforated=INVALID_MARK)
     {
-      std::tuple<> converters;
-      Default_converter_dart_info<GMap2, Refs> dartinfoconverter;
-      Default_converter_cmap_0attributes_with_point<GMap2, Refs> pointconverter;
-      copy(amap, converters, dartinfoconverter, pointconverter, dart_mapping);
+      generic_copy<GMap2, typename GMap2::Dart_handle, Converters,
+          DartInfoConverter, PointConverter>
+          (amap,  origin_to_copy, copy_to_origin,
+           converters, dartinfoconverter, pointconverter,
+           copy_perforated_darts, mark_perforated);
+    }
+
+    // (1b) copy_from_const(const amap, converters, dartinfoconverter, pointconverter)
+    template<typename GMap2, typename Converters, typename DartInfoConverter,
+             typename PointConverter>
+    void copy_from_const(const GMap2& amap,
+                         boost::unordered_map
+                         <typename GMap2::Dart_const_handle, Dart_handle>* origin_to_copy,
+                         boost::unordered_map
+                         <Dart_handle, typename GMap2::Dart_const_handle>* copy_to_origin,
+                         const Converters& converters,
+                         const DartInfoConverter& dartinfoconverter,
+                         const PointConverter& pointconverter,
+                         bool copy_perforated_darts=false,
+                         size_type mark_perforated=INVALID_MARK)
+    {
+      generic_copy<GMap2, typename GMap2::Dart_const_handle,
+          Converters, DartInfoConverter, PointConverter>
+          (const_cast<GMap2&>(amap), origin_to_copy, copy_to_origin,
+           converters, dartinfoconverter, pointconverter,
+           copy_perforated_darts, mark_perforated);
+    }
+
+    // (2a) copy(amap, converters, dartinfoconverter)
+    template<typename GMap2, typename Converters, typename DartInfoConverter>
+    void copy(GMap2& amap,
+              boost::unordered_map
+              <typename GMap2::Dart_handle, Dart_handle>* origin_to_copy,
+              boost::unordered_map
+              <Dart_handle, typename GMap2::Dart_handle>* copy_to_origin,
+              const Converters& converters,
+              const DartInfoConverter& dartinfoconverter,
+              bool copy_perforated_darts=false,
+              size_type mark_perforated=INVALID_MARK)
+    {
+      Default_converter_cmap_0attributes_with_point<typename GMap2::Refs, Refs>
+        pointconverter;
+      copy(amap, origin_to_copy, copy_to_origin, converters,
+           dartinfoconverter, pointconverter,
+           copy_perforated_darts, mark_perforated);
     }
 
+    // (2b) copy_from_const(const amap, converters, dartinfoconverter)
+    template <typename GMap2, typename Converters, typename DartInfoConverter>
+    void copy_from_const(const GMap2& amap,
+                         boost::unordered_map
+                         <typename GMap2::Dart_const_handle, Dart_handle>* origin_to_copy,
+                         boost::unordered_map
+                         <Dart_handle, typename GMap2::Dart_const_handle>* copy_to_origin,
+                         const Converters& converters,
+                         const DartInfoConverter& dartinfoconverter,
+                         bool copy_perforated_darts=false,
+                         size_type mark_perforated=INVALID_MARK)
+    {
+      Default_converter_cmap_0attributes_with_point<typename GMap2::Refs, Refs>
+          pointconverter;
+      copy_from_const(amap, origin_to_copy, copy_to_origin, converters,
+                      dartinfoconverter, pointconverter,
+                      copy_perforated_darts, mark_perforated);
+    }
+
+    // (3a) copy(amap, converters)
+    template<typename GMap2, typename Converters>
+    void copy(GMap2& amap,
+              boost::unordered_map
+              <typename GMap2::Dart_handle, Dart_handle>* origin_to_copy,
+              boost::unordered_map
+              <Dart_handle, typename GMap2::Dart_handle>* copy_to_origin,
+              const Converters& converters,
+              bool copy_perforated_darts=false,
+              size_type mark_perforated=INVALID_MARK)
+    {
+      Default_converter_dart_info<typename GMap2::Refs, Refs> dartinfoconverter;
+      copy(amap, origin_to_copy, copy_to_origin, converters, dartinfoconverter,
+           copy_perforated_darts, mark_perforated);
+    }
+
+    // (3b) copy_from_const(const amap, converters)
     template <typename GMap2, typename Converters>
-    void copy(const GMap2& amap, const Converters& converters,
-              boost::unordered_map<typename GMap2::Dart_const_handle, Dart_handle>* dart_mapping=nullptr)
+    void copy_from_const(const GMap2& amap,
+                         boost::unordered_map
+                         <typename GMap2::Dart_const_handle, Dart_handle>* origin_to_copy,
+                         boost::unordered_map
+                         <Dart_handle, typename GMap2::Dart_const_handle>* copy_to_origin,
+                         const Converters& converters,
+                         bool copy_perforated_darts=false,
+                         size_type mark_perforated=INVALID_MARK)
+    {
+      Default_converter_dart_info<typename GMap2::Refs, Refs> dartinfoconverter;
+      copy_from_const(amap, origin_to_copy, copy_to_origin, converters, dartinfoconverter,
+                      copy_perforated_darts, mark_perforated);
+    }
+
+    // (4a) copy(amap)
+    template<typename GMap2>
+    void copy(GMap2& amap,
+              boost::unordered_map
+              <typename GMap2::Dart_handle, Dart_handle>* origin_to_copy=nullptr,
+              boost::unordered_map
+              <Dart_handle, typename GMap2::Dart_handle>* copy_to_origin=nullptr,
+              bool copy_perforated_darts=false,
+              size_type mark_perforated=INVALID_MARK)
     {
-      Default_converter_cmap_0attributes_with_point<GMap2, Refs> pointconverter;
-      Default_converter_dart_info<GMap2, Refs> dartinfoconverter;
-      copy(amap, converters, dartinfoconverter, pointconverter, dart_mapping);
+      std::tuple<> converters;
+      copy(amap, origin_to_copy, copy_to_origin, converters,
+           copy_perforated_darts, mark_perforated);
     }
 
-    template <typename GMap2, typename Converters, typename DartInfoConverter>
-    void copy(const GMap2& amap, const Converters& converters,
-              const DartInfoConverter& dartinfoconverter,
-              boost::unordered_map<typename GMap2::Dart_const_handle, Dart_handle>* dart_mapping=nullptr)
+    // (4b) copy_from_const(const amap)
+    template <typename GMap2>
+    void copy_from_const(const GMap2& amap,
+                         boost::unordered_map
+                         <typename GMap2::Dart_const_handle, Dart_handle>* origin_to_copy=nullptr,
+                         boost::unordered_map
+                         <Dart_handle, typename GMap2::Dart_const_handle>* copy_to_origin=nullptr,
+                         bool copy_perforated_darts=false,
+                         size_type mark_perforated=INVALID_MARK)
     {
-      Default_converter_cmap_0attributes_with_point<GMap2, Refs> pointconverter;
-      copy(amap, converters, dartinfoconverter, pointconverter, dart_mapping);
+      std::tuple<> converters;
+      copy_from_const(amap, origin_to_copy, copy_to_origin, converters,
+                      copy_perforated_darts, mark_perforated);
     }
 
     // Copy constructor from a map having exactly the same type.
-    Generalized_map_base (const Self & amap)
-    { copy<Self>(amap); }
+    Generalized_map_base(const Self & amap) : Generalized_map_base()
+    { copy_from_const(amap); }
 
     // "Copy constructor" from a map having different type.
-    template <typename GMap2>
-    Generalized_map_base(const GMap2& amap)
-    { copy(amap); }
+    template <unsigned int d2, typename Refs2, typename Items2, typename Alloc2,
+              typename Storage2>
+    Generalized_map_base(const Generalized_map_base<d2, Refs2, Items2, Alloc2, Storage2>& amap):
+       Generalized_map_base()
+    { copy_from_const(amap); }
 
     // "Copy constructor" from a map having different type.
-    template <typename GMap2, typename Converters>
-    Generalized_map_base(const GMap2& amap, Converters& converters)
-    { copy(amap, converters); }
+    template <unsigned int d2, typename Refs2, typename Items2, typename Alloc2,
+              typename Storage2, typename Converters>
+    Generalized_map_base(const Generalized_map_base<d2, Refs2, Items2, Alloc2, Storage2>& amap,
+                         const Converters& converters): Generalized_map_base()
+    { copy_from_const(amap, nullptr, nullptr, converters); }
 
     // "Copy constructor" from a map having different type.
-    template <typename GMap2, typename Converters, typename DartInfoConverter>
-    Generalized_map_base(const GMap2& amap, Converters& converters,
-                         const DartInfoConverter& dartinfoconverter)
-    { copy(amap, converters, dartinfoconverter); }
+    template <unsigned int d2, typename Refs2, typename Items2, typename Alloc2,
+              typename Storage2,
+              typename Converters, typename DartInfoConverter>
+    Generalized_map_base(const Generalized_map_base<d2, Refs2, Items2, Alloc2, Storage2>& amap,
+                         const Converters& converters,
+                         const DartInfoConverter& dartinfoconverter):
+       Generalized_map_base()
+    { copy_from_const(amap, nullptr, nullptr, converters, dartinfoconverter); }
 
     // "Copy constructor" from a map having different type.
-    template <typename GMap2, typename Converters, typename DartInfoConverter,
+    template <unsigned int d2, typename Refs2, typename Items2, typename Alloc2,
+              typename Storage2,
+              typename Converters, typename DartInfoConverter,
               typename PointConverter>
-    Generalized_map_base(const GMap2& amap, Converters& converters,
+    Generalized_map_base(const Generalized_map_base<d2, Refs2, Items2, Alloc2, Storage2>& amap,
+                         const Converters& converters,
                          const DartInfoConverter& dartinfoconverter,
-                         const PointConverter& pointconverter)
-    { copy(amap, converters, dartinfoconverter, pointconverter); }
+                         const PointConverter& pointconverter):
+      Generalized_map_base()
+    { copy_from_const(amap, nullptr, nullptr, converters, dartinfoconverter, pointconverter); }
 
     /** Affectation operation. Copies one map to the other.
      * @param amap a generalized map.
      * @return A copy of that generalized map.
      */
-    Self & operator= (const Self & amap)
+    Self & operator=(const Self & amap)
     {
       if (this!=&amap)
       {
@@ -449,6 +601,16 @@ namespace CGAL {
     { return mdarts.iterator_to(adart); }
     Dart_const_handle dart_handle(const Dart& adart) const
     { return mdarts.iterator_to(adart); }
+    Dart_handle dart_handle(size_type i)
+    {
+      CGAL_assertion(darts().is_used(i));
+      return mdarts.iterator_to(darts()[i]);
+    }
+    Dart_const_handle dart_handle(size_type i) const
+    {
+      CGAL_assertion(darts().is_used(i));
+      return mdarts.iterator_to(darts()[i]);
+    }
 
     /** Return the highest dimension for which dh is not free.
      * @param dh a dart handle
@@ -579,6 +741,11 @@ namespace CGAL {
     Dart_const_handle next(Dart_const_handle ADart) const
     { return this->template alpha<0, 1>(ADart); }
 
+    Dart_handle opposite2(Dart_handle ADart)
+    { return this->template alpha<0, 2>(ADart); }
+    Dart_const_handle opposite2(Dart_const_handle ADart) const
+    { return this->template alpha<0, 2>(ADart); }
+
     template<unsigned int dim>
     Dart_handle opposite(Dart_handle ADart)
     { return this->template alpha<0, dim>(ADart); }
@@ -849,6 +1016,52 @@ namespace CGAL {
       mnb_times_reserved_marks[amark]=0;
     }
 
+    template <unsigned int i, unsigned int d=dimension>
+    bool belong_to_same_cell(Dart_const_handle adart1,
+                             Dart_const_handle adart2) const
+    { return CGAL::belong_to_same_cell<Self, i, d>(*this, adart1, adart2); }
+
+    template <unsigned int i, unsigned int d=dimension>
+    bool is_whole_cell_unmarked(Dart_const_handle adart, size_type amark) const
+    { return CGAL::is_whole_cell_unmarked<Self, i, d>(*this, adart, amark); }
+
+    template <unsigned int i, unsigned int d=dimension>
+    bool is_whole_cell_marked(Dart_const_handle adart, size_type amark) const
+    { return CGAL::is_whole_cell_marked<Self, i, d>(*this, adart, amark); }
+
+    template <unsigned int i, unsigned int d=dimension>
+    size_type mark_cell(Dart_const_handle adart, size_type amark) const
+    { return CGAL::mark_cell<Self, i, d>(*this, adart, amark); }
+
+    template <unsigned int i, unsigned int d=dimension>
+    size_type unmark_cell(Dart_const_handle adart, size_type amark) const
+    { return CGAL::unmark_cell<Self, i, d>(*this, adart, amark); }
+
+    template <unsigned int i, unsigned int d=dimension>
+    size_type mark_oriented_cell(Dart_const_handle adart, size_type amark,
+                                 size_type amark2=INVALID_MARK) const
+    { return CGAL::mark_oriented_cell<Self, i, d>(*this, adart, amark, amark2); }
+
+    template <unsigned int i, unsigned int d=dimension>
+    size_type unmark_oriented_cell(Dart_const_handle adart, size_type amark,
+                                 size_type amark2=INVALID_MARK) const
+    { return CGAL::unmark_oriented_cell<Self, i, d>(*this, adart, amark, amark2); }
+
+    std::size_t orient(size_type amark) const
+    {
+      size_type amark2=get_new_mark();
+      std::size_t res=0;
+      for (auto it=darts().begin(), itend=darts().end(); it!=itend; ++it)
+      {
+        if (!is_marked(it, amark2))
+        { res+=mark_oriented_cell<dimension+1>(it, amark, amark2); } // Mark the connected componend
+      }
+      CGAL_assertion(is_whole_map_marked(amark2));
+      free_mark(amark2);
+
+      return res;
+    }
+
     /** Test if this map is without boundary for a given dimension.
      * @param i the dimension.
      * @return true iff all the darts are not i-free.
@@ -1647,6 +1860,61 @@ namespace CGAL {
     }
 
   public:
+    /// @return the positive turn between the two given darts.
+    //  @pre next(d1) and d2 must belong to the same vertex.
+    std::size_t positive_turn(Dart_const_handle d1, Dart_const_handle d2) const
+    {
+      CGAL_assertion((!this->template is_free<1>(d1)));
+      /* CGAL_assertion((belong_to_same_cell<0>(this->next(d1), d2))); */
+
+      if (d2==opposite2(d1)) { return 0; }
+
+      Dart_const_handle dd1=d1;
+      std::size_t res=1;
+      while (next(dd1)!=d2)
+      {
+        if (this->template is_free<2>(next(dd1)))
+        { return (std::numeric_limits<std::size_t>::max)(); }
+
+        ++res;
+        dd1=opposite2(next(dd1));
+
+        CGAL_assertion(!this->template is_free<1>(dd1));
+        CGAL_assertion(next(dd1)==d2 || dd1!=d1);
+      }
+      return res;
+    }
+
+    /// @return the negative turn between the two given darts.
+    //  @pre next(d1) and d2 must belong to the same vertex.
+    std::size_t negative_turn(Dart_const_handle d1, Dart_const_handle d2) const
+    {
+      CGAL_assertion((!this->template is_free<1>(d1)));
+      /* CGAL_assertion((belong_to_same_cell<0>(this->next(d1), d2))); */
+
+      if (d2==opposite2(d1)) { return 0; }
+
+      if (this->template is_free<2>(d1) || this->template is_free<2>(d2))
+      { return (std::numeric_limits<std::size_t>::max)(); }
+
+      d1=opposite2(d1);
+      d2=opposite2(d2);
+      Dart_const_handle dd1=d1;
+      std::size_t res=1;
+      while (previous(dd1)!=d2)
+      {
+        if (this->template is_free<2>(previous(dd1)))
+        { return (std::numeric_limits<std::size_t>::max)(); }
+
+        ++res;
+        dd1=opposite2(previous(dd1));
+
+        CGAL_assertion(!this->template is_free<0>(dd1));
+        CGAL_assertion(previous(dd1)==d2 || dd1!=d1);
+      }
+      return res;
+    }
+
     /** Erase marked darts from the map.
      * Marked darts are unlinked before to be removed, thus surviving darts
      * are correctly linked, but the map is not necessarily valid depending
@@ -1666,7 +1934,7 @@ namespace CGAL {
         if (is_marked(d, amark))
         {
           for ( i = 0; i <= dimension; ++i)
-          { if (!is_free(d, i)) unlink_beta(d, i); }
+          { if (!is_free(d, i)) topo_unsew(d, i); }
           erase_dart(d); ++res;
         }
       }
@@ -2766,12 +3034,12 @@ namespace CGAL {
            !is_face_combinatorial_polygon(d4, 3) ) return false;
 
       // TODO do better with marks (?).
-      if ( belong_to_same_cell<Self,2,1>(*this, d1, d2) ||
-           belong_to_same_cell<Self,2,1>(*this, d1, d3) ||
-           belong_to_same_cell<Self,2,1>(*this, d1, d4) ||
-           belong_to_same_cell<Self,2,1>(*this, d2, d3) ||
-           belong_to_same_cell<Self,2,1>(*this, d2, d4) ||
-           belong_to_same_cell<Self,2,1>(*this, d3, d4) ) return false;
+      if ( belong_to_same_cell<2,1>(d1, d2) ||
+           belong_to_same_cell<2,1>(d1, d3) ||
+           belong_to_same_cell<2,1>(d1, d4) ||
+           belong_to_same_cell<2,1>(d2, d3) ||
+           belong_to_same_cell<2,1>(d2, d4) ||
+           belong_to_same_cell<2,1>(d3, d4) ) return false;
 
       if ( alpha(d1, 0,1,2)!=alpha(d3,1) ||
            alpha(d4, 1,2)!=alpha(d3,0,1) ||
@@ -2847,21 +3115,21 @@ namespace CGAL {
           !is_face_combinatorial_polygon(d6, 4) ) return false;
 
       // TODO do better with marks.
-      if ( belong_to_same_cell<Self,2,1>(*this, d1, d2) ||
-           belong_to_same_cell<Self,2,1>(*this, d1, d3) ||
-           belong_to_same_cell<Self,2,1>(*this, d1, d4) ||
-           belong_to_same_cell<Self,2,1>(*this, d1, d5) ||
-           belong_to_same_cell<Self,2,1>(*this, d1, d6) ||
-           belong_to_same_cell<Self,2,1>(*this, d2, d3) ||
-           belong_to_same_cell<Self,2,1>(*this, d2, d4) ||
-           belong_to_same_cell<Self,2,1>(*this, d2, d5) ||
-           belong_to_same_cell<Self,2,1>(*this, d2, d6) ||
-           belong_to_same_cell<Self,2,1>(*this, d3, d4) ||
-           belong_to_same_cell<Self,2,1>(*this, d3, d5) ||
-           belong_to_same_cell<Self,2,1>(*this, d3, d6) ||
-           belong_to_same_cell<Self,2,1>(*this, d4, d5) ||
-           belong_to_same_cell<Self,2,1>(*this, d4, d6) ||
-           belong_to_same_cell<Self,2,1>(*this, d5, d6) )
+      if ( belong_to_same_cell<2,1>(d1, d2) ||
+           belong_to_same_cell<2,1>(d1, d3) ||
+           belong_to_same_cell<2,1>(d1, d4) ||
+           belong_to_same_cell<2,1>(d1, d5) ||
+           belong_to_same_cell<2,1>(d1, d6) ||
+           belong_to_same_cell<2,1>(d2, d3) ||
+           belong_to_same_cell<2,1>(d2, d4) ||
+           belong_to_same_cell<2,1>(d2, d5) ||
+           belong_to_same_cell<2,1>(d2, d6) ||
+           belong_to_same_cell<2,1>(d3, d4) ||
+           belong_to_same_cell<2,1>(d3, d5) ||
+           belong_to_same_cell<2,1>(d3, d6) ||
+           belong_to_same_cell<2,1>(d4, d5) ||
+           belong_to_same_cell<2,1>(d4, d6) ||
+           belong_to_same_cell<2,1>(d5, d6) )
         return false;
 
       if ( alpha(d1,2)      !=alpha(d4,1,0,1) ||
@@ -3020,7 +3288,7 @@ namespace CGAL {
       return alpha<0, 1>(adart);
     }
 
-    /** Insert a vertex in the given 2-cell which is splitted in triangles,
+    /** Insert a vertex in the given 2-cell which is split in triangles,
      * once for each inital edge of the facet.
      * @param adart a dart of the facet to triangulate.
      * @return A dart incident to the new vertex.
@@ -3341,14 +3609,14 @@ namespace CGAL {
       if (prec!=null_handle)
       {
         // prec and *it must belong to the same vertex of the same volume
-        if ( !CGAL::belong_to_same_cell<Self, 0, 2>(*this,  prec, *it) )
+        if ( !belong_to_same_cell<0, 2>(prec, *it) )
           return false;
       }
       prec = this->template alpha<0>(*it);
     }
 
     // The path must be closed.
-    if (!CGAL::belong_to_same_cell<Self, 0, 2>(*this, prec, *afirst))
+    if (!belong_to_same_cell<0, 2>(prec, *afirst))
       return false;
 
     return true;
@@ -3564,10 +3832,7 @@ namespace CGAL {
     typename Helper::Merge_functors m_onmerge_functors;
   };
 
-  template < unsigned int d_,
-             class Items_=Generic_map_min_items,
-             class Alloc_=CGAL_ALLOCATOR(int),
-             class Storage_= Generalized_map_storage_1<d_, Items_, Alloc_> >
+  template<unsigned int d_, class Items_, class Alloc_,class Storage_>
   class Generalized_map :
     public Generalized_map_base<d_,
                                   Generalized_map<d_,Items_,Alloc_, Storage_>,
@@ -3589,24 +3854,34 @@ namespace CGAL {
     Generalized_map(const Self & amap) : Base(amap)
     {}
 
-    template < class Gmap >
-    Generalized_map(const Gmap & amap) : Base(amap)
+    template <unsigned int d2, typename Refs2, typename Items2, typename Alloc2,
+              typename Storage2>
+    Generalized_map(const Generalized_map_base<d2, Refs2, Items2, Alloc2, Storage2>& amap) :
+      Base(amap)
     {}
 
-    template < class Gmap, typename Converters >
-    Generalized_map(const Gmap & amap, const Converters& converters) :
+    template <unsigned int d2, typename Refs2, typename Items2, typename Alloc2,
+              typename Storage2, typename Converters>
+    Generalized_map(const Generalized_map_base<d2, Refs2, Items2, Alloc2, Storage2>& amap,
+                    const Converters& converters) :
       Base(amap, converters)
     {}
 
-    template < class Gmap, typename Converters, typename DartInfoConverter >
-    Generalized_map(const Gmap & amap, const Converters& converters,
+    template <unsigned int d2, typename Refs2, typename Items2, typename Alloc2,
+              typename Storage2,
+              typename Converters, typename DartInfoConverter>
+    Generalized_map(const Generalized_map_base<d2, Refs2, Items2, Alloc2, Storage2>& amap,
+                    const Converters& converters,
                     const DartInfoConverter& dartinfoconverter) :
       Base(amap, converters, dartinfoconverter)
     {}
 
-    template < class Gmap, typename Converters, typename DartInfoConverter,
-               typename PointConverter >
-    Generalized_map(const Gmap & amap, const Converters& converters,
+    template <unsigned int d2, typename Refs2, typename Items2, typename Alloc2,
+              typename Storage2,
+              typename Converters, typename DartInfoConverter,
+              typename PointConverter >
+    Generalized_map(const Generalized_map_base<d2, Refs2, Items2, Alloc2, Storage2>& amap,
+                    const Converters& converters,
                     const DartInfoConverter& dartinfoconverter,
                     const PointConverter& pointconverter) :
       Base(amap, converters, dartinfoconverter, pointconverter)
diff -pruN 5.0.3-1/include/CGAL/Generalized_map_iterators_base.h 5.1-1/include/CGAL/Generalized_map_iterators_base.h
--- 5.0.3-1/include/CGAL/Generalized_map_iterators_base.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Generalized_map_iterators_base.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Generalized_map/include/CGAL/Generalized_map_iterators_base.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Generalized_map/include/CGAL/Generalized_map_iterators_base.h $
 // $Id: Generalized_map_iterators_base.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Generalized_map_operations.h 5.1-1/include/CGAL/Generalized_map_operations.h
--- 5.0.3-1/include/CGAL/Generalized_map_operations.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Generalized_map_operations.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Generalized_map/include/CGAL/Generalized_map_operations.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Generalized_map/include/CGAL/Generalized_map_operations.h $
 // $Id: Generalized_map_operations.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Generalized_map_save_load.h 5.1-1/include/CGAL/Generalized_map_save_load.h
--- 5.0.3-1/include/CGAL/Generalized_map_save_load.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Generalized_map_save_load.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Generalized_map/include/CGAL/Generalized_map_save_load.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Generalized_map/include/CGAL/Generalized_map_save_load.h $
 // $Id: Generalized_map_save_load.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Generalized_map_storages.h 5.1-1/include/CGAL/Generalized_map_storages.h
--- 5.0.3-1/include/CGAL/Generalized_map_storages.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Generalized_map_storages.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Generalized_map/include/CGAL/Generalized_map_storages.h $
-// $Id: Generalized_map_storages.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Generalized_map/include/CGAL/Generalized_map_storages.h $
+// $Id: Generalized_map_storages.h daab969 2020-04-08T09:23:59+02:00 Guillaume Damiand
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
@@ -103,7 +103,7 @@ namespace CGAL {
 
     // Init
     void init_storage()
-    {}
+    { null_dart_handle=nullptr; }
 
    /** Return if this dart is free for adimension.
      * @param dh a dart handle
@@ -123,6 +123,8 @@ namespace CGAL {
       CGAL_assertion(i <= dimension);
       return dh->mf[i]==dh;
     }
+    bool is_perforated(Dart_const_handle /*dh*/) const
+    { return false; }
 
     /// Set simultaneously all the marks of this dart to a given value.
     void set_dart_marks(Dart_const_handle ADart,
@@ -400,6 +402,8 @@ namespace CGAL {
     }
 
   protected:
+    Dart_handle null_dart_handle; // To be compatible with combinatorial map
+
     /// Dart container.
     Dart_container mdarts;
 
diff -pruN 5.0.3-1/include/CGAL/General_polygon_2.h 5.1-1/include/CGAL/General_polygon_2.h
--- 5.0.3-1/include/CGAL/General_polygon_2.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/General_polygon_2.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/General_polygon_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/General_polygon_2.h $
 // $Id: General_polygon_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/General_polygon_set_2.h 5.1-1/include/CGAL/General_polygon_set_2.h
--- 5.0.3-1/include/CGAL/General_polygon_set_2.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/General_polygon_set_2.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/General_polygon_set_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/General_polygon_set_2.h $
 // $Id: General_polygon_set_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/General_polygon_set_on_surface_2.h 5.1-1/include/CGAL/General_polygon_set_on_surface_2.h
--- 5.0.3-1/include/CGAL/General_polygon_set_on_surface_2.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/General_polygon_set_on_surface_2.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/General_polygon_set_on_surface_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/General_polygon_set_on_surface_2.h $
 // $Id: General_polygon_set_on_surface_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/General_polygon_with_holes_2.h 5.1-1/include/CGAL/General_polygon_with_holes_2.h
--- 5.0.3-1/include/CGAL/General_polygon_with_holes_2.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/General_polygon_with_holes_2.h	2020-09-08 12:15:22.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon/include/CGAL/General_polygon_with_holes_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon/include/CGAL/General_polygon_with_holes_2.h $
 // $Id: General_polygon_with_holes_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/generators.h 5.1-1/include/CGAL/generators.h
--- 5.0.3-1/include/CGAL/generators.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/generators.h	2020-09-08 12:14:51.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Generator/include/CGAL/generators.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Generator/include/CGAL/generators.h $
 // $Id: generators.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Generic_map_min_items.h 5.1-1/include/CGAL/Generic_map_min_items.h
--- 5.0.3-1/include/CGAL/Generic_map_min_items.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/Generic_map_min_items.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Combinatorial_map/include/CGAL/Generic_map_min_items.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Combinatorial_map/include/CGAL/Generic_map_min_items.h $
 // $Id: Generic_map_min_items.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/generic_sweep.h 5.1-1/include/CGAL/generic_sweep.h
--- 5.0.3-1/include/CGAL/generic_sweep.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/generic_sweep.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_2/include/CGAL/generic_sweep.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_2/include/CGAL/generic_sweep.h $
 // $Id: generic_sweep.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Get_arithmetic_kernel.h 5.1-1/include/CGAL/Get_arithmetic_kernel.h
--- 5.0.3-1/include/CGAL/Get_arithmetic_kernel.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Get_arithmetic_kernel.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arithmetic_kernel/include/CGAL/Get_arithmetic_kernel.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arithmetic_kernel/include/CGAL/Get_arithmetic_kernel.h $
 // $Id: Get_arithmetic_kernel.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/gl.h 5.1-1/include/CGAL/gl.h
--- 5.0.3-1/include/CGAL/gl.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/gl.h	2020-09-08 12:14:54.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/gl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/gl.h $
 // $Id: gl.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 
diff -pruN 5.0.3-1/include/CGAL/global_functions_circular_kernel_2.h 5.1-1/include/CGAL/global_functions_circular_kernel_2.h
--- 5.0.3-1/include/CGAL/global_functions_circular_kernel_2.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/global_functions_circular_kernel_2.h	2020-09-08 12:14:46.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_2/include/CGAL/global_functions_circular_kernel_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_2/include/CGAL/global_functions_circular_kernel_2.h $
 // $Id: global_functions_circular_kernel_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/global_functions_on_root_for_sphere_2_3.h 5.1-1/include/CGAL/global_functions_on_root_for_sphere_2_3.h
--- 5.0.3-1/include/CGAL/global_functions_on_root_for_sphere_2_3.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/global_functions_on_root_for_sphere_2_3.h	2020-09-08 12:14:32.000000000 +0000
@@ -9,7 +9,7 @@
 // and a STREP (FET Open) Project under Contract No  IST-006413
 // (ACS -- Algorithms for Complex Shapes)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_for_spheres/include/CGAL/global_functions_on_root_for_sphere_2_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_for_spheres/include/CGAL/global_functions_on_root_for_sphere_2_3.h $
 // $Id: global_functions_on_root_for_sphere_2_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/global_functions_on_roots_and_polynomials_1_3.h 5.1-1/include/CGAL/global_functions_on_roots_and_polynomials_1_3.h
--- 5.0.3-1/include/CGAL/global_functions_on_roots_and_polynomials_1_3.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/global_functions_on_roots_and_polynomials_1_3.h	2020-09-08 12:14:32.000000000 +0000
@@ -9,7 +9,7 @@
 // and a STREP (FET Open) Project under Contract No  IST-006413
 // (ACS -- Algorithms for Complex Shapes)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_for_spheres/include/CGAL/global_functions_on_roots_and_polynomials_1_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_for_spheres/include/CGAL/global_functions_on_roots_and_polynomials_1_3.h $
 // $Id: global_functions_on_roots_and_polynomials_1_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/global_functions_on_roots_and_polynomials_2_3.h 5.1-1/include/CGAL/global_functions_on_roots_and_polynomials_2_3.h
--- 5.0.3-1/include/CGAL/global_functions_on_roots_and_polynomials_2_3.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/global_functions_on_roots_and_polynomials_2_3.h	2020-09-08 12:14:32.000000000 +0000
@@ -9,7 +9,7 @@
 // and a STREP (FET Open) Project under Contract No  IST-006413
 // (ACS -- Algorithms for Complex Shapes)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_for_spheres/include/CGAL/global_functions_on_roots_and_polynomials_2_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_for_spheres/include/CGAL/global_functions_on_roots_and_polynomials_2_3.h $
 // $Id: global_functions_on_roots_and_polynomials_2_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/global_functions_spherical_kernel_3.h 5.1-1/include/CGAL/global_functions_spherical_kernel_3.h
--- 5.0.3-1/include/CGAL/global_functions_spherical_kernel_3.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/global_functions_spherical_kernel_3.h	2020-09-08 12:14:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_3/include/CGAL/global_functions_spherical_kernel_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_3/include/CGAL/global_functions_spherical_kernel_3.h $
 // $Id: global_functions_spherical_kernel_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/GLPK_mixed_integer_program_traits.h 5.1-1/include/CGAL/GLPK_mixed_integer_program_traits.h
--- 5.0.3-1/include/CGAL/GLPK_mixed_integer_program_traits.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/GLPK_mixed_integer_program_traits.h	2020-09-08 12:15:39.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Solver_interface/include/CGAL/GLPK_mixed_integer_program_traits.h $
-// $Id: GLPK_mixed_integer_program_traits.h 202430d 2020-04-20T15:51:48+02:00 Dmitry Anisimov
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Solver_interface/include/CGAL/GLPK_mixed_integer_program_traits.h $
+// $Id: GLPK_mixed_integer_program_traits.h 3b6c100 2020-04-29T14:12:29+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s) : Liangliang Nan
@@ -13,277 +13,262 @@
 
 #include <CGAL/Mixed_integer_program_traits.h>
 
-#ifdef CGAL_USE_GLPK
+#if defined(CGAL_USE_GLPK) || defined(DOXYGEN_RUNNING)
 
 #include <glpk.h>
 
+#include <iostream>
+#include <vector>
 
 namespace CGAL {
+namespace internal {
 
-        /// \ingroup PkgSolverInterfaceRef
-        ///
-        /// This class provides an interface for formulating and solving
-        /// mixed integer programs (constrained or unconstrained) using
-        /// \ref thirdpartyGLPK.
-        ///
-        /// \ref thirdpartyGLPK must be available on the system.
-        /// \note For better performance, please consider using
-        ///                  `CGAL::SCIP_mixed_integer_program_traits`, or derive a new
-        ///                  model from `CGAL::Mixed_integer_program_traits`.
-        ///
-        /// \cond SKIP_IN_MANUAL
-        /// \tparam FT Number type
-        /// \endcond
-        ///
-        /// \cgalModels `MixedIntegerProgramTraits`
-        ///
-        /// \sa `SCIP_mixed_integer_program_traits`
-        template <typename FT>
-        class GLPK_mixed_integer_program_traits : public Mixed_integer_program_traits<FT>
-        {
-                /// \cond SKIP_IN_MANUAL
-        public:
-                typedef CGAL::Mixed_integer_program_traits<FT>                Base_class;
-                typedef typename Base_class::Variable                        Variable;
-                typedef typename Base_class::Linear_constraint          Linear_constraint;
-                typedef typename Base_class::Linear_objective           Linear_objective;
-                typedef typename Linear_objective::Sense                Sense;
-                typedef typename Variable::Variable_type                Variable_type;
-
-        public:
-
-                /// Solves the program. Returns false if fails.
-                virtual bool solve();
-                /// \endcond
-        };
-
-        //////////////////////////////////////////////////////////////////////////
-
-        // implementation
-
-        /// \cond SKIP_IN_MANUAL
-
-        namespace internal {
-
-                // Infers "bound type" (required by GLPK) from the bounds values
-                template <typename FT>
-                int bound_type(FT lb, FT ub) {
-                        typedef CGAL::Variable<FT> Variable;
-
-                        if (lb <= -Variable::infinity() && ub >= Variable::infinity())
-                                return GLP_FR;                // free (unbounded) variable
-
-                        else if (lb > -Variable::infinity() && ub >= Variable::infinity())
-                                return GLP_LO;                // variable with lower bound
-
-                        else if (lb <= -Variable::infinity() && ub < Variable::infinity())
-                                return GLP_UP;                // variable with upper bound
-
-                        else {// lb > -Variable::infinity() && ub < Variable::infinity()
-                                if (lb == ub)
-                                        return GLP_FX;        // fixed variable
-                                else
-                                        return GLP_DB;  // double-bounded variable
-                        }
-                }
+// Infers "bound type" (required by GLPK) from the bounds values
+template <typename FT>
+int bound_type(FT lb, FT ub)
+{
+  typedef CGAL::Variable<FT> Variable;
+
+  if (lb <= -Variable::infinity() && ub >= Variable::infinity()) {
+    return GLP_FR; // free (unbounded) variable
+  } else if (lb > -Variable::infinity() && ub >= Variable::infinity()) {
+    return GLP_LO; // variable with lower bound
+  } else if (lb <= -Variable::infinity() && ub < Variable::infinity()) {
+    return GLP_UP; // variable with upper bound
+  } else {
+    // lb > -Variable::infinity() && ub < Variable::infinity()
+    if (lb == ub)
+      return GLP_FX; // fixed variable
+    else
+      return GLP_DB; // double-bounded variable
+  }
+}
+
+} // namespace internal
+
+/// \ingroup PkgSolverInterfaceRef
+///
+/// This class provides an interface for formulating and solving
+/// constrained or unconstrained mixed integer programs using
+/// \ref thirdpartyGLPK, which must be available on the system.
+///
+/// \note For better performance, please consider using
+///      `CGAL::SCIP_mixed_integer_program_traits`, or derive a new
+///      model from `CGAL::Mixed_integer_program_traits`.
+///
+/// \cgalModels `MixedIntegerProgramTraits`
+///
+/// \sa `SCIP_mixed_integer_program_traits`
+template <typename FT>
+class GLPK_mixed_integer_program_traits
+  : public Mixed_integer_program_traits<FT>
+{
+  /// \cond SKIP_IN_MANUAL
+public:
+  typedef CGAL::Mixed_integer_program_traits<FT>          Base_class;
+  typedef typename Base_class::Variable                   Variable;
+  typedef typename Base_class::Linear_constraint          Linear_constraint;
+  typedef typename Base_class::Linear_objective           Linear_objective;
+  typedef typename Linear_objective::Sense                Sense;
+  typedef typename Variable::Variable_type                Variable_type;
+
+public:
+  /// Solves the program. Returns `false` if fails.
+  virtual bool solve()
+  {
+    Base_class::error_message_.clear();
+
+    glp_prob* lp = glp_create_prob();
+    if (!lp) {
+      Base_class::error_message_ = "failed creating GLPK program";
+      return false;
+    }
+
+    std::size_t num_integer_variables = 0;
+
+    // Creates variables
+
+    // This "static_cast<>" suppresses many annoying warnings: "conversion from 'size_t' to 'int', possible loss of data"
+    int num_variables = static_cast<int>(Base_class::variables_.size());
+
+    glp_add_cols(lp, num_variables);
+    for (int i = 0; i < num_variables; ++i) {
+      const Variable* var = Base_class::variables_[i];
+      glp_set_col_name(lp, i + 1, var->name().c_str());
+
+      if (var->variable_type() == Variable::INTEGER) {
+        glp_set_col_kind(lp, i + 1, GLP_IV);  // glpk uses 1-based arrays
+        ++num_integer_variables;
+      } else if (var->variable_type() == Variable::BINARY) {
+        glp_set_col_kind(lp, i + 1, GLP_BV);  // glpk uses 1-based arrays
+        ++num_integer_variables;
+      } else {
+        glp_set_col_kind(lp, i + 1, GLP_CV);  // Continuous variable
+      }
+
+      FT lb, ub;
+      var->get_bounds(lb, ub);
+
+      int type = internal::bound_type(lb, ub);
+      glp_set_col_bnds(lp, i + 1, type, lb, ub);
+    }
+
+    // Adds constraints
+
+    // This "static_cast<>" suppresses many annoying warnings: "conversion from 'size_t' to 'int', possible loss of data"
+    int num_constraints = static_cast<int>(Base_class::constraints_.size());
+    glp_add_rows(lp, num_constraints);
+
+    for (int i = 0; i < num_constraints; ++i) {
+      const Linear_constraint* c = Base_class::constraints_[i];
+      const std::unordered_map<const Variable*, FT>& coeffs = c->coefficients();
+      typename std::unordered_map<const Variable*, FT>::const_iterator cur = coeffs.begin();
+
+      std::vector<int>  indices(coeffs.size() + 1, 0); // glpk uses 1-based arrays
+      std::vector<FT> coefficients(coeffs.size() + 1, 0.0); // glpk uses 1-based arrays
+      std::size_t idx = 1; // glpk uses 1-based arrays
+      for (; cur != coeffs.end(); ++cur) {
+        int var_idx = cur->first->index();
+        FT coeff = cur->second;
+
+        indices[idx] = var_idx + 1;   // glpk uses 1-based arrays
+        coefficients[idx] = coeff;
+        ++idx;
+      }
+
+      glp_set_mat_row(lp, i + 1, static_cast<int>(coeffs.size()), indices.data(), coefficients.data());
+
+      FT lb, ub;
+      c->get_bounds(lb, ub);
+
+      int type = internal::bound_type(lb, ub);
+      glp_set_row_bnds(lp, i + 1, type, lb, ub);
+
+      glp_set_row_name(lp, i + 1, c->name().c_str());
+    }
+
+    // Sets objective
+
+    // Determines the coefficient of each variable in the objective function
+    const std::unordered_map<const Variable*, FT>& obj_coeffs = Base_class::objective_->coefficients();
+    typename std::unordered_map<const Variable*, FT>::const_iterator cur = obj_coeffs.begin();
+    for (; cur != obj_coeffs.end(); ++cur) {
+      int var_idx = cur->first->index();
+      FT coeff = cur->second;
+      glp_set_obj_coef(lp, var_idx + 1, coeff); // glpk uses 1-based arrays
+    }
+
+    // Sets objective function sense
+    bool minimize = (Base_class::objective_->sense() == Linear_objective::MINIMIZE);
+    glp_set_obj_dir(lp, minimize ? GLP_MIN : GLP_MAX);
+    int msg_level = GLP_MSG_ERR;
+    int status = -1;
+    if (num_integer_variables == 0) { // Continuous problem
+      glp_smcp parm;
+      glp_init_smcp(&parm);
+      parm.msg_lev = msg_level;
+      status = glp_simplex(lp, &parm);
+    }
+    else { // Solves as MIP problem
+      glp_iocp parm;
+      glp_init_iocp(&parm);
+      parm.msg_lev = msg_level;
+      parm.presolve = GLP_ON;
+      // The routine glp_intopt is a driver to the MIP solver based on the branch-and-cut method,
+      // which is a hybrid of branch-and-bound and cutting plane methods.
+      status = glp_intopt(lp, &parm);
+    }
+
+    switch (status) {
+      case 0: {
+        if (num_integer_variables == 0) { // continuous problem
+          Base_class::result_.resize(num_variables);
+          for (int i = 0; i < num_variables; ++i) {
+            FT x = glp_get_col_prim(lp, i + 1);  // glpk uses 1-based arrays
+            Variable* v = Base_class::variables_[i];
+            v->set_solution_value(x);
+            Base_class::result_[i] = x;
+          }
         }
-        /// \endcond
-
-        template<typename FT>
-        bool GLPK_mixed_integer_program_traits<FT>::solve() {
-                Base_class::error_message_.clear();
-
-                glp_prob* lp = glp_create_prob();
-                if (!lp) {
-                        Base_class::error_message_ = "failed creating GLPK program";
-                        return false;
-                }
-
-                std::size_t num_integer_variables = 0;
-
-                // Creates variables
-
-                // This "static_cast<>" suppresses many annoying warnings: "conversion from 'size_t' to 'int', possible loss of data"
-                int num_variables = static_cast<int>(Base_class::variables_.size());
-
-                glp_add_cols(lp, num_variables);
-                for (int i = 0; i < num_variables; ++i) {
-                        const Variable* var = Base_class::variables_[i];
-                        glp_set_col_name(lp, i + 1, var->name().c_str());
-
-                        if (var->variable_type() == Variable::INTEGER) {
-                                glp_set_col_kind(lp, i + 1, GLP_IV);        // glpk uses 1-based arrays
-                                ++num_integer_variables;
-                        }
-                        else if (var->variable_type() == Variable::BINARY) {
-                                glp_set_col_kind(lp, i + 1, GLP_BV);        // glpk uses 1-based arrays
-                                ++num_integer_variables;
-                        }
-                        else
-                                glp_set_col_kind(lp, i + 1, GLP_CV);        // Continuous variable
-
-                        FT lb, ub;
-                        var->get_bounds(lb, ub);
-
-                        int type = internal::bound_type(lb, ub);
-                        glp_set_col_bnds(lp, i + 1, type, lb, ub);
-                }
-
-                // Adds constraints
-
-                // This "static_cast<>" suppresses many annoying warnings: "conversion from 'size_t' to 'int', possible loss of data"
-                int num_constraints = static_cast<int>(Base_class::constraints_.size());
-                glp_add_rows(lp, num_constraints);
-
-                for (int i = 0; i < num_constraints; ++i) {
-                        const Linear_constraint* c = Base_class::constraints_[i];
-                        const std::unordered_map<const Variable*, FT>& coeffs = c->coefficients();
-                        typename std::unordered_map<const Variable*, FT>::const_iterator cur = coeffs.begin();
-
-                        std::vector<int>        indices(coeffs.size() + 1, 0);                // glpk uses 1-based arrays
-                        std::vector<FT> coefficients(coeffs.size() + 1, 0.0);        // glpk uses 1-based arrays
-                        std::size_t idx = 1; // glpk uses 1-based arrays
-                        for (; cur != coeffs.end(); ++cur) {
-                                int var_idx = cur->first->index();
-                                FT coeff = cur->second;
-
-                                indices[idx] = var_idx + 1;         // glpk uses 1-based arrays
-                                coefficients[idx] = coeff;
-                                ++idx;
-                        }
-
-                        glp_set_mat_row(lp, i + 1, static_cast<int>(coeffs.size()), indices.data(), coefficients.data());
-
-                        FT lb, ub;
-                        c->get_bounds(lb, ub);
-
-                        int type = internal::bound_type(lb, ub);
-                        glp_set_row_bnds(lp, i + 1, type, lb, ub);
-
-                        glp_set_row_name(lp, i + 1, c->name().c_str());
-                }
-
-                // Sets objective
-
-                // Determines the coefficient of each variable in the objective function
-                const std::unordered_map<const Variable*, FT>& obj_coeffs = Base_class::objective_->coefficients();
-                typename std::unordered_map<const Variable*, FT>::const_iterator cur = obj_coeffs.begin();
-                for (; cur != obj_coeffs.end(); ++cur) {
-                        int var_idx = cur->first->index();
-                        FT coeff = cur->second;
-                        glp_set_obj_coef(lp, var_idx + 1, coeff); // glpk uses 1-based arrays
-                }
-
-                // Sets objective function sense
-                bool minimize = (Base_class::objective_->sense() == Linear_objective::MINIMIZE);
-                glp_set_obj_dir(lp, minimize ? GLP_MIN : GLP_MAX);
-                int msg_level = GLP_MSG_ERR;
-                int status = -1;
-                if (num_integer_variables == 0) { // Continuous problem
-                        glp_smcp parm;
-                        glp_init_smcp(&parm);
-                        parm.msg_lev = msg_level;
-                        status = glp_simplex(lp, &parm);
-                }
-                else { // Solves as MIP problem
-                        glp_iocp parm;
-                        glp_init_iocp(&parm);
-                        parm.msg_lev = msg_level;
-                        parm.presolve = GLP_ON;
-                        // The routine glp_intopt is a driver to the MIP solver based on the branch-and-cut method,
-                        // which is a hybrid of branch-and-bound and cutting plane methods.
-                        status = glp_intopt(lp, &parm);
-                }
-
-                switch (status) {
-                case 0: {
-                        if (num_integer_variables == 0) { // continuous problem
-                                Base_class::result_.resize(num_variables);
-                                for (int i = 0; i < num_variables; ++i) {
-                                        FT x = glp_get_col_prim(lp, i + 1);        // glpk uses 1-based arrays
-                                        Variable* v = Base_class::variables_[i];
-                                        v->set_solution_value(x);
-                                        Base_class::result_[i] = x;
-                                }
-                        }
-                        else { // MIP problem
-                                Base_class::result_.resize(num_variables);
-                                for (int i = 0; i < num_variables; ++i) {
-                                        FT x = glp_mip_col_val(lp, i + 1);                // glpk uses 1-based arrays
-
-                                        Variable* v = Base_class::variables_[i];
-                                        if (v->variable_type() != Variable::CONTINUOUS)
-                                                x = static_cast<int>(std::round(x));
-
-                                        v->set_solution_value(x);
-                                        Base_class::result_[i] = x;
-                                }
-                        }
-                        break;
-                }
-
-                case GLP_EBOUND:
-                        Base_class::error_message_ =
-                                "Unable to start the search, because some FT-bounded variables have incorrect"
-                                "bounds or some integer variables have non - integer(fractional) bounds.";
-                        break;
-
-                case GLP_EROOT:
-                        Base_class::error_message_ =
-                                "Unable to start the search, because optimal basis for initial LP relaxation is not"
-                                "provided. (This code may appear only if the presolver is disabled.)";
-                        break;
-
-                case GLP_ENOPFS:
-                        Base_class::error_message_ =
-                                "Unable to start the search, because LP relaxation of the MIP problem instance has"
-                                "no primal feasible solution. (This code may appear only if the presolver is enabled.)";
-                        break;
-
-                case GLP_ENODFS:
-                        Base_class::error_message_ =
-                                "Unable to start the search, because LP relaxation of the MIP problem instance has"
-                                "no dual feasible solution.In other word, this code means that if the LP relaxation"
-                                "has at least one primal feasible solution, its optimal solution is unbounded, so if the"
-                                "MIP problem has at least one integer feasible solution, its(integer) optimal solution"
-                                "is also unbounded. (This code may appear only if the presolver is enabled.)";
-                        break;
-
-                case GLP_EFAIL:
-                        Base_class::error_message_ =
-                                "The search was prematurely terminated due to the solver failure.";
-                        break;
-
-                case GLP_EMIPGAP:
-                        Base_class::error_message_ =
-                                "The search was prematurely terminated, because the relative mip gap tolerance has been reached.";
-                        break;
-
-                case GLP_ETMLIM:
-                        Base_class::error_message_ =
-                                "The search was prematurely terminated, because the time limit has been exceeded.";
-                        break;
-
-                case GLP_ESTOP:
-                        Base_class::error_message_ =
-                                "The search was prematurely terminated by application. (This code may appear only"
-                                "if the advanced solver interface is used.)";
-                        break;
-
-                default:
-                        Base_class::error_message_ =
-                                "optimization was stopped with status code " + std::to_string(status);
-                        break;
-                }
-
-                glp_delete_prob(lp);
-
-                return (status == 0);
+        else { // MIP problem
+          Base_class::result_.resize(num_variables);
+          for (int i = 0; i < num_variables; ++i) {
+            FT x = glp_mip_col_val(lp, i + 1); // glpk uses 1-based arrays
+
+            Variable* v = Base_class::variables_[i];
+            if (v->variable_type() != Variable::CONTINUOUS)
+              x = static_cast<int>(std::round(x));
+
+            v->set_solution_value(x);
+            Base_class::result_[i] = x;
+          }
         }
+        break;
+      }
 
+      case GLP_EBOUND:
+        Base_class::error_message_ =
+            "Unable to start the search, because some FT-bounded variables have incorrect"
+            "bounds or some integer variables have non - integer(fractional) bounds.";
+        break;
+
+      case GLP_EROOT:
+        Base_class::error_message_ =
+            "Unable to start the search, because optimal basis for initial LP relaxation is not"
+            "provided. (This code may appear only if the presolver is disabled.)";
+        break;
+
+      case GLP_ENOPFS:
+        Base_class::error_message_ =
+            "Unable to start the search, because LP relaxation of the MIP problem instance has"
+            "no primal feasible solution. (This code may appear only if the presolver is enabled.)";
+        break;
+
+      case GLP_ENODFS:
+        Base_class::error_message_ =
+            "Unable to start the search, because LP relaxation of the MIP problem instance has"
+            "no dual feasible solution.In other word, this code means that if the LP relaxation"
+            "has at least one primal feasible solution, its optimal solution is unbounded, so if the"
+            "MIP problem has at least one integer feasible solution, its(integer) optimal solution"
+            "is also unbounded. (This code may appear only if the presolver is enabled.)";
+        break;
+
+      case GLP_EFAIL:
+        Base_class::error_message_ =
+            "The search was prematurely terminated due to the solver failure.";
+        break;
+
+      case GLP_EMIPGAP:
+        Base_class::error_message_ =
+            "The search was prematurely terminated, because the relative mip gap tolerance has been reached.";
+        break;
+
+      case GLP_ETMLIM:
+        Base_class::error_message_ =
+            "The search was prematurely terminated, because the time limit has been exceeded.";
+        break;
+
+      case GLP_ESTOP:
+        Base_class::error_message_ =
+            "The search was prematurely terminated by application. (This code may appear only"
+            "if the advanced solver interface is used.)";
+        break;
+
+      default:
+        Base_class::error_message_ =
+            "optimization was stopped with status code " + std::to_string(status);
+        break;
+    }
+
+    glp_delete_prob(lp);
+
+    return (status == 0);
+  }
+  /// \endcond
+};
 
 } // namespace CGAL
 
-#endif // CGAL_USE_GLPK
+#endif // CGAL_USE_GLPK or DOXYGEN_RUNNING
 
 #endif // CGAL_GLPK_MIXED_INTEGER_PROGRAM_TRAITS_H
diff -pruN 5.0.3-1/include/CGAL/glu.h 5.1-1/include/CGAL/glu.h
--- 5.0.3-1/include/CGAL/glu.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/glu.h	2020-09-08 12:14:54.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/glu.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/glu.h $
 // $Id: glu.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 
diff -pruN 5.0.3-1/include/CGAL/GMap_cell_const_iterators.h 5.1-1/include/CGAL/GMap_cell_const_iterators.h
--- 5.0.3-1/include/CGAL/GMap_cell_const_iterators.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/GMap_cell_const_iterators.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Generalized_map/include/CGAL/GMap_cell_const_iterators.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Generalized_map/include/CGAL/GMap_cell_const_iterators.h $
 // $Id: GMap_cell_const_iterators.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/GMap_cell_iterators.h 5.1-1/include/CGAL/GMap_cell_iterators.h
--- 5.0.3-1/include/CGAL/GMap_cell_iterators.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/GMap_cell_iterators.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Generalized_map/include/CGAL/GMap_cell_iterators.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Generalized_map/include/CGAL/GMap_cell_iterators.h $
 // $Id: GMap_cell_iterators.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/GMap_dart_const_iterators.h 5.1-1/include/CGAL/GMap_dart_const_iterators.h
--- 5.0.3-1/include/CGAL/GMap_dart_const_iterators.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/GMap_dart_const_iterators.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Generalized_map/include/CGAL/GMap_dart_const_iterators.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Generalized_map/include/CGAL/GMap_dart_const_iterators.h $
 // $Id: GMap_dart_const_iterators.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/GMap_dart_iterators.h 5.1-1/include/CGAL/GMap_dart_iterators.h
--- 5.0.3-1/include/CGAL/GMap_dart_iterators.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/GMap_dart_iterators.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Generalized_map/include/CGAL/GMap_dart_iterators.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Generalized_map/include/CGAL/GMap_dart_iterators.h $
 // $Id: GMap_dart_iterators.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/GMap_linear_cell_complex_storages.h 5.1-1/include/CGAL/GMap_linear_cell_complex_storages.h
--- 5.0.3-1/include/CGAL/GMap_linear_cell_complex_storages.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/GMap_linear_cell_complex_storages.h	2020-09-08 12:15:05.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Linear_cell_complex/include/CGAL/GMap_linear_cell_complex_storages.h $
-// $Id: GMap_linear_cell_complex_storages.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Linear_cell_complex/include/CGAL/GMap_linear_cell_complex_storages.h $
+// $Id: GMap_linear_cell_complex_storages.h daab969 2020-04-08T09:23:59+02:00 Guillaume Damiand
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
@@ -122,7 +122,7 @@ namespace CGAL {
 
     // Init
     void init_storage()
-    {}
+    { null_dart_handle=nullptr; }
 
    /** Return if this dart is free for adimension.
      * @param dh a dart handle
@@ -142,6 +142,8 @@ namespace CGAL {
       CGAL_assertion(i <= dimension);
       return dh->mf[i]==dh;
     }
+    bool is_perforated(Dart_const_handle /*dh*/) const
+    { return false; }
 
     /// Set simultaneously all the marks of this dart to a given value.
     void set_dart_marks(Dart_const_handle ADart,
@@ -433,6 +435,8 @@ namespace CGAL {
     }
 
   protected:
+    Dart_handle null_dart_handle; // To be compatible with combinatorial map
+
     /// Dart container.
     Dart_container mdarts;
 
diff -pruN 5.0.3-1/include/CGAL/GMP/Gmpfi_type.h 5.1-1/include/CGAL/GMP/Gmpfi_type.h
--- 5.0.3-1/include/CGAL/GMP/Gmpfi_type.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/GMP/Gmpfi_type.h	2020-09-08 12:15:16.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/GMP/Gmpfi_type.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/GMP/Gmpfi_type.h $
 // $Id: Gmpfi_type.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/GMP/Gmpfi_type_static.h 5.1-1/include/CGAL/GMP/Gmpfi_type_static.h
--- 5.0.3-1/include/CGAL/GMP/Gmpfi_type_static.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/GMP/Gmpfi_type_static.h	2020-09-08 12:15:16.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/GMP/Gmpfi_type_static.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/GMP/Gmpfi_type_static.h $
 // $Id: Gmpfi_type_static.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/GMP/Gmpfr_type.h 5.1-1/include/CGAL/GMP/Gmpfr_type.h
--- 5.0.3-1/include/CGAL/GMP/Gmpfr_type.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/GMP/Gmpfr_type.h	2020-09-08 12:15:16.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/GMP/Gmpfr_type.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/GMP/Gmpfr_type.h $
 // $Id: Gmpfr_type.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/GMP/Gmpfr_type_static.h 5.1-1/include/CGAL/GMP/Gmpfr_type_static.h
--- 5.0.3-1/include/CGAL/GMP/Gmpfr_type_static.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/GMP/Gmpfr_type_static.h	2020-09-08 12:15:16.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/GMP/Gmpfr_type_static.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/GMP/Gmpfr_type_static.h $
 // $Id: Gmpfr_type_static.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/GMP/Gmpq_type.h 5.1-1/include/CGAL/GMP/Gmpq_type.h
--- 5.0.3-1/include/CGAL/GMP/Gmpq_type.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/GMP/Gmpq_type.h	2020-09-08 12:15:16.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/GMP/Gmpq_type.h $
-// $Id: Gmpq_type.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/GMP/Gmpq_type.h $
+// $Id: Gmpq_type.h ad758d0 2020-05-20T11:59:03+02:00 Marc Glisse
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -222,20 +222,23 @@ public:
   Gmpq& operator*=(const Gmpq &q);
   Gmpq& operator/=(const Gmpq &q);
 
-  bool operator==(const Gmpq &q) const { return mpq_equal(this->mpq(), q.mpq()) != 0;}
+  bool operator==(const Gmpq &q) const noexcept { return mpq_equal(this->mpq(), q.mpq()) != 0;}
   bool operator< (const Gmpq &q) const { return mpq_cmp(this->mpq(), q.mpq()) < 0; }
 
-  double to_double() const;
-  Sign sign() const;
+  double to_double() const noexcept;
+  Sign sign() const noexcept;
 
-  const mpq_t & mpq() const { return Ptr()->mpQ; }
-  mpq_t & mpq() { return ptr()->mpQ; }
+  const mpq_t & mpq() const noexcept { return Ptr()->mpQ; }
+  mpq_t & mpq() noexcept { return ptr()->mpQ; }
 
+  friend void swap(Gmpq &x, Gmpq &y) noexcept { x.Base::swap(y); }
+#ifdef CGAL_PROFILE
   ~Gmpq()
   {
      CGAL_HISTOGRAM_PROFILER("[Gmpq sizes in log2 scale]",
                              (unsigned) ( ::log(double(size())) / ::log(double(2)) )  );
   }
+#endif
 
   // Interoperability with int
   Gmpq& operator+=(int z){return (*this)+= Gmpq(z);}
@@ -446,12 +449,12 @@ Gmpq& Gmpq::operator/=(const Gmpz &z){
 
 inline
 double
-Gmpq::to_double() const
+Gmpq::to_double() const noexcept
 { return mpq_get_d(mpq()); }
 
 inline
 Sign
-Gmpq::sign() const
+Gmpq::sign() const noexcept
 { return static_cast<Sign>(mpq_sgn(mpq())); }
 
 inline
diff -pruN 5.0.3-1/include/CGAL/GMP/Gmpzf_type.h 5.1-1/include/CGAL/GMP/Gmpzf_type.h
--- 5.0.3-1/include/CGAL/GMP/Gmpzf_type.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/GMP/Gmpzf_type.h	2020-09-08 12:15:16.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/GMP/Gmpzf_type.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/GMP/Gmpzf_type.h $
 // $Id: Gmpzf_type.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/GMP/Gmpz_type.h 5.1-1/include/CGAL/GMP/Gmpz_type.h
--- 5.0.3-1/include/CGAL/GMP/Gmpz_type.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/GMP/Gmpz_type.h	2020-09-08 12:15:16.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/GMP/Gmpz_type.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/GMP/Gmpz_type.h $
 // $Id: Gmpz_type.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/GMP_arithmetic_kernel.h 5.1-1/include/CGAL/GMP_arithmetic_kernel.h
--- 5.0.3-1/include/CGAL/GMP_arithmetic_kernel.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/GMP_arithmetic_kernel.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arithmetic_kernel/include/CGAL/GMP_arithmetic_kernel.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arithmetic_kernel/include/CGAL/GMP_arithmetic_kernel.h $
 // $Id: GMP_arithmetic_kernel.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Gmp_coercion_traits.h 5.1-1/include/CGAL/Gmp_coercion_traits.h
--- 5.0.3-1/include/CGAL/Gmp_coercion_traits.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Gmp_coercion_traits.h	2020-09-08 12:15:16.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Gmp_coercion_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Gmp_coercion_traits.h $
 // $Id: Gmp_coercion_traits.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Gmpfi.h 5.1-1/include/CGAL/Gmpfi.h
--- 5.0.3-1/include/CGAL/Gmpfi.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Gmpfi.h	2020-09-08 12:15:16.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Gmpfi.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Gmpfi.h $
 // $Id: Gmpfi.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Gmpfr.h 5.1-1/include/CGAL/Gmpfr.h
--- 5.0.3-1/include/CGAL/Gmpfr.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Gmpfr.h	2020-09-08 12:15:16.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Gmpfr.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Gmpfr.h $
 // $Id: Gmpfr.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/gmp.h 5.1-1/include/CGAL/gmp.h
--- 5.0.3-1/include/CGAL/gmp.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/gmp.h	2020-09-08 12:15:16.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/gmp.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/gmp.h $
 // $Id: gmp.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Gmpq.h 5.1-1/include/CGAL/Gmpq.h
--- 5.0.3-1/include/CGAL/Gmpq.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Gmpq.h	2020-09-08 12:15:16.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Gmpq.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Gmpq.h $
 // $Id: Gmpq.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/GMPXX_arithmetic_kernel.h 5.1-1/include/CGAL/GMPXX_arithmetic_kernel.h
--- 5.0.3-1/include/CGAL/GMPXX_arithmetic_kernel.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/GMPXX_arithmetic_kernel.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arithmetic_kernel/include/CGAL/GMPXX_arithmetic_kernel.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arithmetic_kernel/include/CGAL/GMPXX_arithmetic_kernel.h $
 // $Id: GMPXX_arithmetic_kernel.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/gmpxx_coercion_traits.h 5.1-1/include/CGAL/gmpxx_coercion_traits.h
--- 5.0.3-1/include/CGAL/gmpxx_coercion_traits.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/gmpxx_coercion_traits.h	2020-09-08 12:15:16.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/gmpxx_coercion_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/gmpxx_coercion_traits.h $
 // $Id: gmpxx_coercion_traits.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/gmpxx.h 5.1-1/include/CGAL/gmpxx.h
--- 5.0.3-1/include/CGAL/gmpxx.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/gmpxx.h	2020-09-08 12:15:16.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/gmpxx.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/gmpxx.h $
 // $Id: gmpxx.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Gmpzf.h 5.1-1/include/CGAL/Gmpzf.h
--- 5.0.3-1/include/CGAL/Gmpzf.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Gmpzf.h	2020-09-08 12:15:16.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Gmpzf.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Gmpzf.h $
 // $Id: Gmpzf.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Gmpz.h 5.1-1/include/CGAL/Gmpz.h
--- 5.0.3-1/include/CGAL/Gmpz.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Gmpz.h	2020-09-08 12:15:16.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Gmpz.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Gmpz.h $
 // $Id: Gmpz.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/gnuplot_output_2.h 5.1-1/include/CGAL/gnuplot_output_2.h
--- 5.0.3-1/include/CGAL/gnuplot_output_2.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/gnuplot_output_2.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cone_spanners_2/include/CGAL/gnuplot_output_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cone_spanners_2/include/CGAL/gnuplot_output_2.h $
 // $Id: gnuplot_output_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Gps_circle_segment_traits_2.h 5.1-1/include/CGAL/Gps_circle_segment_traits_2.h
--- 5.0.3-1/include/CGAL/Gps_circle_segment_traits_2.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Gps_circle_segment_traits_2.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Gps_circle_segment_traits_2.h $
-// $Id: Gps_circle_segment_traits_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Gps_circle_segment_traits_2.h $
+// $Id: Gps_circle_segment_traits_2.h e2a77e5 2020-05-24T11:18:01+02:00 Marc Glisse
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -27,7 +27,7 @@ class Gps_circle_segment_traits_2 :
   public Gps_traits_2<Arr_circle_segment_traits_2<Kernel_, Filer_> >
 {
 public:
-  Gps_circle_segment_traits_2<Kernel_, Filer_>(bool use_cache = false) :
+  Gps_circle_segment_traits_2(bool use_cache = false) :
     Gps_traits_2<Arr_circle_segment_traits_2<Kernel_, Filer_> >()
   {
     this->m_use_cache = use_cache;
diff -pruN 5.0.3-1/include/CGAL/Gps_segment_traits_2.h 5.1-1/include/CGAL/Gps_segment_traits_2.h
--- 5.0.3-1/include/CGAL/Gps_segment_traits_2.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Gps_segment_traits_2.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Gps_segment_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Gps_segment_traits_2.h $
 // $Id: Gps_segment_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Gps_traits_2.h 5.1-1/include/CGAL/Gps_traits_2.h
--- 5.0.3-1/include/CGAL/Gps_traits_2.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Gps_traits_2.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Gps_traits_2.h $
-// $Id: Gps_traits_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Gps_traits_2.h $
+// $Id: Gps_traits_2.h 7436c14 2020-04-06T01:23:45+03:00 Efi Fogel
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -35,6 +35,8 @@ public:
 
   typedef typename Base::Point_2                        Point_2;
   typedef typename Base::X_monotone_curve_2             X_monotone_curve_2;
+  typedef typename Base::Multiplicity                   Multiplicity;
+
   //Polygon_2 type is required by GeneralPolygonSetTraits Concept
   typedef General_polygon_t                             Polygon_2;
   //Polygon_2 is a model of the GeneralPolygon2 concept
diff -pruN 5.0.3-1/include/CGAL/grabbers.h 5.1-1/include/CGAL/grabbers.h
--- 5.0.3-1/include/CGAL/grabbers.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/grabbers.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ipelets/include/CGAL/grabbers.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ipelets/include/CGAL/grabbers.h $
 // $Id: grabbers.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/graph_traits_Arrangement_2.h 5.1-1/include/CGAL/graph_traits_Arrangement_2.h
--- 5.0.3-1/include/CGAL/graph_traits_Arrangement_2.h	2020-08-01 19:02:13.000000000 +0000
+++ 5.1-1/include/CGAL/graph_traits_Arrangement_2.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/graph_traits_Arrangement_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/graph_traits_Arrangement_2.h $
 // $Id: graph_traits_Arrangement_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/graph_traits_dual_arrangement_2.h 5.1-1/include/CGAL/graph_traits_dual_arrangement_2.h
--- 5.0.3-1/include/CGAL/graph_traits_dual_arrangement_2.h	2020-08-01 19:02:13.000000000 +0000
+++ 5.1-1/include/CGAL/graph_traits_dual_arrangement_2.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/graph_traits_dual_arrangement_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/graph_traits_dual_arrangement_2.h $
 // $Id: graph_traits_dual_arrangement_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/graph_traits_dual_arrangement_on_surface_2.h 5.1-1/include/CGAL/graph_traits_dual_arrangement_on_surface_2.h
--- 5.0.3-1/include/CGAL/graph_traits_dual_arrangement_on_surface_2.h	2020-08-01 19:02:13.000000000 +0000
+++ 5.1-1/include/CGAL/graph_traits_dual_arrangement_on_surface_2.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/graph_traits_dual_arrangement_on_surface_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/graph_traits_dual_arrangement_on_surface_2.h $
 // $Id: graph_traits_dual_arrangement_on_surface_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/graph_traits_dual_arrangement_on_surface_with_history_2.h 5.1-1/include/CGAL/graph_traits_dual_arrangement_on_surface_with_history_2.h
--- 5.0.3-1/include/CGAL/graph_traits_dual_arrangement_on_surface_with_history_2.h	2020-08-01 19:02:13.000000000 +0000
+++ 5.1-1/include/CGAL/graph_traits_dual_arrangement_on_surface_with_history_2.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/graph_traits_dual_arrangement_on_surface_with_history_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/graph_traits_dual_arrangement_on_surface_with_history_2.h $
 // $Id: graph_traits_dual_arrangement_on_surface_with_history_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/graph_traits_dual_arrangement_with_history_2.h 5.1-1/include/CGAL/graph_traits_dual_arrangement_with_history_2.h
--- 5.0.3-1/include/CGAL/graph_traits_dual_arrangement_with_history_2.h	2020-08-01 19:02:13.000000000 +0000
+++ 5.1-1/include/CGAL/graph_traits_dual_arrangement_with_history_2.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/graph_traits_dual_arrangement_with_history_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/graph_traits_dual_arrangement_with_history_2.h $
 // $Id: graph_traits_dual_arrangement_with_history_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Gray_image_mesh_domain_3.h 5.1-1/include/CGAL/Gray_image_mesh_domain_3.h
--- 5.0.3-1/include/CGAL/Gray_image_mesh_domain_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Gray_image_mesh_domain_3.h	2020-09-08 12:15:08.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Gray_image_mesh_domain_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Gray_image_mesh_domain_3.h $
 // $Id: Gray_image_mesh_domain_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Gray_level_image_3.h 5.1-1/include/CGAL/Gray_level_image_3.h
--- 5.0.3-1/include/CGAL/Gray_level_image_3.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Gray_level_image_3.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Gray_level_image_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Gray_level_image_3.h $
 // $Id: Gray_level_image_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/grid_simplify_point_set.h 5.1-1/include/CGAL/grid_simplify_point_set.h
--- 5.0.3-1/include/CGAL/grid_simplify_point_set.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/grid_simplify_point_set.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_processing_3/include/CGAL/grid_simplify_point_set.h $
-// $Id: grid_simplify_point_set.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/grid_simplify_point_set.h $
+// $Id: grid_simplify_point_set.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s) : Nader Salman and Laurent Saboret
@@ -175,19 +175,34 @@ public:
 
    \param points input point range.
    \param epsilon tolerance value when merging 3D points.
-   \param np optional sequence of \ref psp_namedparameters "Named Parameters" among the ones listed below.
+   \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
    \cgalNamedParamsBegin
-     \cgalParamBegin{point_map} a model of `ReadWritePropertyMap` with value type `geom_traits::Point_3`.
-     If this parameter is omitted, `CGAL::Identity_property_map<geom_traits::Point_3>` is used.\cgalParamEnd
-     \cgalParamBegin{callback} an instance of
-      `std::function<bool(double)>`. It is called regularly when the
-      algorithm is running: the current advancement (between 0. and
-      1.) is passed as parameter. If it returns `true`, then the
-      algorithm continues its execution normally; if it returns
-      `false`, the algorithm is stopped and simplification stops with
-      no guarantee on the output.\cgalParamEnd
-     \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+     \cgalParamNBegin{point_map}
+       \cgalParamDescription{a property map associating points to the elements of the point set `points`}
+       \cgalParamType{a model of `ReadWritePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange` and whose value type is `geom_traits::Point_3`}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{callback}
+       \cgalParamDescription{a mechanism to get feedback on the advancement of the algorithm
+                             while it's running and to interrupt it if needed}
+       \cgalParamType{an instance of `std::function<bool(double)>`.}
+       \cgalParamDefault{unused}
+       \cgalParamExtra{It is called regularly when the
+                       algorithm is running: the current advancement (between 0. and
+                       1.) is passed as parameter. If it returns `true`, then the
+                       algorithm continues its execution normally; if it returns
+                       `false`, the algorithm is stopped and simplification stops with no guarantee on the output. }
+       \cgalParamExtra{The callback will be copied and therefore needs to be lightweight.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{geom_traits}
+       \cgalParamDescription{an instance of a geometric traits class}
+       \cgalParamType{a model of `Kernel`}
+       \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+     \cgalParamNEnd
    \cgalNamedParamsEnd
 
    \return iterator over the first point to remove.
@@ -202,8 +217,8 @@ grid_simplify_point_set(
   using parameters::choose_parameter;
   using parameters::get_parameter;
 
-  typedef typename Point_set_processing_3::GetPointMap<PointRange, NamedParameters>::const_type PointMap;
-  PointMap point_map = choose_parameter(get_parameter(np, internal_np::point_map), PointMap());
+  typedef typename CGAL::GetPointMap<PointRange, NamedParameters>::const_type PointMap;
+  PointMap point_map = choose_parameter<PointMap>(get_parameter(np, internal_np::point_map));
   const std::function<bool(double)>& callback = choose_parameter(get_parameter(np, internal_np::callback),
                                                                  std::function<bool(double)>());
 
diff -pruN 5.0.3-1/include/CGAL/halfedgeds_connected_components.h 5.1-1/include/CGAL/halfedgeds_connected_components.h
--- 5.0.3-1/include/CGAL/halfedgeds_connected_components.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/halfedgeds_connected_components.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/HalfedgeDS/include/CGAL/halfedgeds_connected_components.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/HalfedgeDS/include/CGAL/halfedgeds_connected_components.h $
 // $Id: halfedgeds_connected_components.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/HalfedgeDS_const_decorator.h 5.1-1/include/CGAL/HalfedgeDS_const_decorator.h
--- 5.0.3-1/include/CGAL/HalfedgeDS_const_decorator.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/HalfedgeDS_const_decorator.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/HalfedgeDS/include/CGAL/HalfedgeDS_const_decorator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/HalfedgeDS/include/CGAL/HalfedgeDS_const_decorator.h $
 // $Id: HalfedgeDS_const_decorator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/halfedgeDS_cut_component.h 5.1-1/include/CGAL/halfedgeDS_cut_component.h
--- 5.0.3-1/include/CGAL/halfedgeDS_cut_component.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/halfedgeDS_cut_component.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/HalfedgeDS/include/CGAL/halfedgeDS_cut_component.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/HalfedgeDS/include/CGAL/halfedgeDS_cut_component.h $
 // $Id: halfedgeDS_cut_component.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/HalfedgeDS_decorator.h 5.1-1/include/CGAL/HalfedgeDS_decorator.h
--- 5.0.3-1/include/CGAL/HalfedgeDS_decorator.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/HalfedgeDS_decorator.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/HalfedgeDS/include/CGAL/HalfedgeDS_decorator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/HalfedgeDS/include/CGAL/HalfedgeDS_decorator.h $
 // $Id: HalfedgeDS_decorator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/HalfedgeDS_default.h 5.1-1/include/CGAL/HalfedgeDS_default.h
--- 5.0.3-1/include/CGAL/HalfedgeDS_default.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/HalfedgeDS_default.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/HalfedgeDS/include/CGAL/HalfedgeDS_default.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/HalfedgeDS/include/CGAL/HalfedgeDS_default.h $
 // $Id: HalfedgeDS_default.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/HalfedgeDS_face_base.h 5.1-1/include/CGAL/HalfedgeDS_face_base.h
--- 5.0.3-1/include/CGAL/HalfedgeDS_face_base.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/HalfedgeDS_face_base.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/HalfedgeDS/include/CGAL/HalfedgeDS_face_base.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/HalfedgeDS/include/CGAL/HalfedgeDS_face_base.h $
 // $Id: HalfedgeDS_face_base.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/HalfedgeDS_face_max_base_with_id.h 5.1-1/include/CGAL/HalfedgeDS_face_max_base_with_id.h
--- 5.0.3-1/include/CGAL/HalfedgeDS_face_max_base_with_id.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/HalfedgeDS_face_max_base_with_id.h	2020-09-08 12:14:53.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/HalfedgeDS/include/CGAL/HalfedgeDS_face_max_base_with_id.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/HalfedgeDS/include/CGAL/HalfedgeDS_face_max_base_with_id.h $
 // $Id: HalfedgeDS_face_max_base_with_id.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/HalfedgeDS_face_min_base.h 5.1-1/include/CGAL/HalfedgeDS_face_min_base.h
--- 5.0.3-1/include/CGAL/HalfedgeDS_face_min_base.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/HalfedgeDS_face_min_base.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/HalfedgeDS/include/CGAL/HalfedgeDS_face_min_base.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/HalfedgeDS/include/CGAL/HalfedgeDS_face_min_base.h $
 // $Id: HalfedgeDS_face_min_base.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/HalfedgeDS_halfedge_base.h 5.1-1/include/CGAL/HalfedgeDS_halfedge_base.h
--- 5.0.3-1/include/CGAL/HalfedgeDS_halfedge_base.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/HalfedgeDS_halfedge_base.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/HalfedgeDS/include/CGAL/HalfedgeDS_halfedge_base.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/HalfedgeDS/include/CGAL/HalfedgeDS_halfedge_base.h $
 // $Id: HalfedgeDS_halfedge_base.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/HalfedgeDS_halfedge_max_base_with_id.h 5.1-1/include/CGAL/HalfedgeDS_halfedge_max_base_with_id.h
--- 5.0.3-1/include/CGAL/HalfedgeDS_halfedge_max_base_with_id.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/HalfedgeDS_halfedge_max_base_with_id.h	2020-09-08 12:14:53.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/HalfedgeDS/include/CGAL/HalfedgeDS_halfedge_max_base_with_id.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/HalfedgeDS/include/CGAL/HalfedgeDS_halfedge_max_base_with_id.h $
 // $Id: HalfedgeDS_halfedge_max_base_with_id.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/HalfedgeDS_halfedge_min_base.h 5.1-1/include/CGAL/HalfedgeDS_halfedge_min_base.h
--- 5.0.3-1/include/CGAL/HalfedgeDS_halfedge_min_base.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/HalfedgeDS_halfedge_min_base.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/HalfedgeDS/include/CGAL/HalfedgeDS_halfedge_min_base.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/HalfedgeDS/include/CGAL/HalfedgeDS_halfedge_min_base.h $
 // $Id: HalfedgeDS_halfedge_min_base.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/HalfedgeDS_items_2.h 5.1-1/include/CGAL/HalfedgeDS_items_2.h
--- 5.0.3-1/include/CGAL/HalfedgeDS_items_2.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/HalfedgeDS_items_2.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/HalfedgeDS/include/CGAL/HalfedgeDS_items_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/HalfedgeDS/include/CGAL/HalfedgeDS_items_2.h $
 // $Id: HalfedgeDS_items_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/HalfedgeDS_items_decorator.h 5.1-1/include/CGAL/HalfedgeDS_items_decorator.h
--- 5.0.3-1/include/CGAL/HalfedgeDS_items_decorator.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/HalfedgeDS_items_decorator.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/HalfedgeDS/include/CGAL/HalfedgeDS_items_decorator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/HalfedgeDS/include/CGAL/HalfedgeDS_items_decorator.h $
 // $Id: HalfedgeDS_items_decorator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/HalfedgeDS_iterator_adaptor.h 5.1-1/include/CGAL/HalfedgeDS_iterator_adaptor.h
--- 5.0.3-1/include/CGAL/HalfedgeDS_iterator_adaptor.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/HalfedgeDS_iterator_adaptor.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/HalfedgeDS/include/CGAL/HalfedgeDS_iterator_adaptor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/HalfedgeDS/include/CGAL/HalfedgeDS_iterator_adaptor.h $
 // $Id: HalfedgeDS_iterator_adaptor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/HalfedgeDS_iterator.h 5.1-1/include/CGAL/HalfedgeDS_iterator.h
--- 5.0.3-1/include/CGAL/HalfedgeDS_iterator.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/HalfedgeDS_iterator.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/HalfedgeDS/include/CGAL/HalfedgeDS_iterator.h $
-// $Id: HalfedgeDS_iterator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/HalfedgeDS/include/CGAL/HalfedgeDS_iterator.h $
+// $Id: HalfedgeDS_iterator.h 1d3c8bb 2020-06-10T22:33:26+02:00 Laurent Rineau
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -262,6 +262,8 @@ public:
     bool operator!=( std::nullptr_t p) const { return !(*this == p); }
     bool operator==( const Self& i) const { return  It::operator==(i); }
     bool operator!=( const Self& i) const { return !(*this == i); }
+    bool operator==( const It& i) const { return  It::operator==(i); }
+    bool operator!=( const It& i) const { return !(*this == i); }
 
     Self& operator++() {
         this->nt = (*this->nt).next();
diff -pruN 5.0.3-1/include/CGAL/HalfedgeDS_list.h 5.1-1/include/CGAL/HalfedgeDS_list.h
--- 5.0.3-1/include/CGAL/HalfedgeDS_list.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/HalfedgeDS_list.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/HalfedgeDS/include/CGAL/HalfedgeDS_list.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/HalfedgeDS/include/CGAL/HalfedgeDS_list.h $
 // $Id: HalfedgeDS_list.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/HalfedgeDS_min_items.h 5.1-1/include/CGAL/HalfedgeDS_min_items.h
--- 5.0.3-1/include/CGAL/HalfedgeDS_min_items.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/HalfedgeDS_min_items.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/HalfedgeDS/include/CGAL/HalfedgeDS_min_items.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/HalfedgeDS/include/CGAL/HalfedgeDS_min_items.h $
 // $Id: HalfedgeDS_min_items.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/HalfedgeDS_vector.h 5.1-1/include/CGAL/HalfedgeDS_vector.h
--- 5.0.3-1/include/CGAL/HalfedgeDS_vector.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/HalfedgeDS_vector.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/HalfedgeDS/include/CGAL/HalfedgeDS_vector.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/HalfedgeDS/include/CGAL/HalfedgeDS_vector.h $
 // $Id: HalfedgeDS_vector.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/HalfedgeDS_vertex_base.h 5.1-1/include/CGAL/HalfedgeDS_vertex_base.h
--- 5.0.3-1/include/CGAL/HalfedgeDS_vertex_base.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/HalfedgeDS_vertex_base.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/HalfedgeDS/include/CGAL/HalfedgeDS_vertex_base.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/HalfedgeDS/include/CGAL/HalfedgeDS_vertex_base.h $
 // $Id: HalfedgeDS_vertex_base.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/HalfedgeDS_vertex_max_base_with_id.h 5.1-1/include/CGAL/HalfedgeDS_vertex_max_base_with_id.h
--- 5.0.3-1/include/CGAL/HalfedgeDS_vertex_max_base_with_id.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/HalfedgeDS_vertex_max_base_with_id.h	2020-09-08 12:14:53.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/HalfedgeDS/include/CGAL/HalfedgeDS_vertex_max_base_with_id.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/HalfedgeDS/include/CGAL/HalfedgeDS_vertex_max_base_with_id.h $
 // $Id: HalfedgeDS_vertex_max_base_with_id.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/HalfedgeDS_vertex_min_base.h 5.1-1/include/CGAL/HalfedgeDS_vertex_min_base.h
--- 5.0.3-1/include/CGAL/HalfedgeDS_vertex_min_base.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/HalfedgeDS_vertex_min_base.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/HalfedgeDS/include/CGAL/HalfedgeDS_vertex_min_base.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/HalfedgeDS/include/CGAL/HalfedgeDS_vertex_min_base.h $
 // $Id: HalfedgeDS_vertex_min_base.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Handle_for.h 5.1-1/include/CGAL/Handle_for.h
--- 5.0.3-1/include/CGAL/Handle_for.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Handle_for.h	2020-09-08 12:15:32.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Handle_for.h $
-// $Id: Handle_for.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Handle_for.h $
+// $Id: Handle_for.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -102,7 +102,7 @@ public:
         ptr_ = p;
     }
 
-    Handle_for(const Handle_for& h)
+    Handle_for(const Handle_for& h) noexcept
       : ptr_(h.ptr_)
     {
         CGAL_assume (ptr_->count > 0);
@@ -110,7 +110,7 @@ public:
     }
 
     Handle_for&
-    operator=(const Handle_for& h)
+    operator=(const Handle_for& h) noexcept
     {
         Handle_for tmp = h;
         swap(tmp);
@@ -132,7 +132,7 @@ public:
     //        from e.g. using nullptr as a ptr value, but this is drastic.
 
     Handle_for&
-    operator=(Handle_for && h)
+    operator=(Handle_for && h) noexcept
     {
         swap(h);
         return *this;
@@ -164,15 +164,15 @@ public:
         *this = t;
     }
 
-    Id_type id() const { return Ptr() - static_cast<T const*>(0); }
+    Id_type id() const noexcept { return Ptr() - static_cast<T const*>(0); }
 
-    bool identical(const Handle_for& h) const { return Ptr() == h.Ptr(); }
+    bool identical(const Handle_for& h) const noexcept { return Ptr() == h.Ptr(); }
 
 
     // Ptr() is the "public" access to the pointer to the object.
     // The non-const version asserts that the instance is not shared.
     const element_type *
-    Ptr() const
+    Ptr() const noexcept
     {
        return &(ptr_->t);
     }
@@ -188,25 +188,25 @@ public:
     */
 
     bool
-    is_shared() const
+    is_shared() const noexcept
     {
         return ptr_->count > 1;
     }
 
     bool
-    unique() const
+    unique() const noexcept
     {
         return !is_shared();
     }
 
     long
-    use_count() const
+    use_count() const noexcept
     {
         return ptr_->count;
     }
 
     void
-    swap(Handle_for& h)
+    swap(Handle_for& h) noexcept
     {
       std::swap(ptr_, h.ptr_);
     }
@@ -222,11 +222,11 @@ protected:
     // ptr() is the protected access to the pointer.  Both const and non-const.
     // Redundant with Ptr().
     element_type *
-    ptr()
+    ptr() noexcept
     { return &(ptr_->t); }
 
     const element_type *
-    ptr() const
+    ptr() const noexcept
     { return &(ptr_->t); }
 };
 
diff -pruN 5.0.3-1/include/CGAL/Handle_for_virtual.h 5.1-1/include/CGAL/Handle_for_virtual.h
--- 5.0.3-1/include/CGAL/Handle_for_virtual.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Handle_for_virtual.h	2020-09-08 12:15:32.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Handle_for_virtual.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Handle_for_virtual.h $
 // $Id: Handle_for_virtual.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Handle.h 5.1-1/include/CGAL/Handle.h
--- 5.0.3-1/include/CGAL/Handle.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Handle.h	2020-09-08 12:15:32.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Handle.h $
-// $Id: Handle.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Handle.h $
+// $Id: Handle.h 8b474dd 2020-05-20T10:32:03+02:00 Laurent Rineau
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -39,13 +39,15 @@ class Handle
 
     typedef std::ptrdiff_t Id_type ;
 
-    Handle()
+    Handle() noexcept
         : PTR(static_cast<Rep*>(0)) {}
 
-    Handle(const Handle& x)
+    // FIXME: if the precondition throws in a noexcept function, the program terminates
+    Handle(const Handle& x) noexcept
     {
       CGAL_precondition( x.PTR != static_cast<Rep*>(0) );
       PTR = x.PTR;
+      CGAL_assume (PTR->count > 0);
       PTR->count++;
     }
 
@@ -56,7 +58,7 @@ class Handle
     }
 
     Handle&
-    operator=(const Handle& x)
+    operator=(const Handle& x) noexcept
     {
       CGAL_precondition( x.PTR != static_cast<Rep*>(0) );
       x.PTR->count++;
@@ -66,6 +68,8 @@ class Handle
       return *this;
     }
 
+    friend void swap(Handle& a, Handle& b) noexcept { std::swap(a.PTR, b.PTR); }
+
     void reset()
     {
       if (PTR)
@@ -77,19 +81,21 @@ class Handle
     }
 
     int
-    refs()  const { return PTR->count; }
+    refs()  const noexcept { return PTR->count; }
 
-    Id_type id() const { return PTR - static_cast<Rep*>(0); }
+    Id_type id() const noexcept { return PTR - static_cast<Rep*>(0); }
 
-    bool identical(const Handle& h) const { return PTR == h.PTR; }
+    bool identical(const Handle& h) const noexcept { return PTR == h.PTR; }
 
+    void * for_compact_container() const { return PTR; }
+    void for_compact_container(void* p) { PTR = static_cast<Rep*>(p); }
   protected:
     Rep* PTR;
 };
 
 //inline Handle::Id_type id(const Handle& x) { return x.id() ; }
 
-inline bool identical(const Handle &h1, const Handle &h2) { return h1.identical(h2); }
+inline bool identical(const Handle &h1, const Handle &h2) noexcept { return h1.identical(h2); }
 
 } //namespace CGAL
 
diff -pruN 5.0.3-1/include/CGAL/Handle_hash_function.h 5.1-1/include/CGAL/Handle_hash_function.h
--- 5.0.3-1/include/CGAL/Handle_hash_function.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Handle_hash_function.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Hash_map/include/CGAL/Handle_hash_function.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Hash_map/include/CGAL/Handle_hash_function.h $
 // $Id: Handle_hash_function.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Handle_with_policy.h 5.1-1/include/CGAL/Handle_with_policy.h
--- 5.0.3-1/include/CGAL/Handle_with_policy.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Handle_with_policy.h	2020-09-08 12:15:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Handle_with_policy.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Handle_with_policy.h $
 // $Id: Handle_with_policy.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Has_conversion.h 5.1-1/include/CGAL/Has_conversion.h
--- 5.0.3-1/include/CGAL/Has_conversion.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Has_conversion.h	2020-09-08 12:15:01.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Has_conversion.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Has_conversion.h $
 // $Id: Has_conversion.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Hash_handles_with_or_without_timestamps.h 5.1-1/include/CGAL/Hash_handles_with_or_without_timestamps.h
--- 5.0.3-1/include/CGAL/Hash_handles_with_or_without_timestamps.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Hash_handles_with_or_without_timestamps.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,50 +0,0 @@
-// Copyright (c) 2014-2017 GeometryFactory Sarl (France)
-// All rights reserved.
-//
-// This file is part of CGAL (www.cgal.org)
-//
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Hash_handles_with_or_without_timestamps.h $
-// $Id: Hash_handles_with_or_without_timestamps.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
-// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
-//
-// Author(s)     : Laurent Rineau,
-//                 Mael Rouxel-Labbé
-
-#ifndef CGAL_HASH_HANDLES_WITH_OR_WITHOUT_TIMESTAMPS_H
-#define CGAL_HASH_HANDLES_WITH_OR_WITHOUT_TIMESTAMPS_H
-
-#include <CGAL/Has_timestamp.h>
-
-#include <boost/functional/hash.hpp>
-
-#include <cstddef>
-#include <iterator>
-
-namespace CGAL {
-
-struct Hash_handles_with_or_without_timestamps
-{
-  template <typename Handle>
-  std::size_t operator()(const Handle h) const
-  {
-    typedef typename std::iterator_traits<Handle>::value_type Type;
-
-    return hash(h, Boolean_tag<CGAL::internal::Has_timestamp<Type>::value>());
-  }
-
-  template<typename Handle>
-  std::size_t hash(const Handle h, Tag_false) const
-  {
-    return boost::hash_value(&*h);
-  }
-
-  template<typename Handle>
-  std::size_t hash(const Handle h, Tag_true) const
-  {
-    return h->time_stamp();
-  }
-};
-
-} // namespace CGAL
-
-#endif // CGAL_HASH_HANDLES_WITH_OR_WITHOUT_TIMESTAMPS_H
diff -pruN 5.0.3-1/include/CGAL/hash_openmesh.h 5.1-1/include/CGAL/hash_openmesh.h
--- 5.0.3-1/include/CGAL/hash_openmesh.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/hash_openmesh.h	2020-09-08 12:15:32.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/hash_openmesh.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/hash_openmesh.h $
 // $Id: hash_openmesh.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Has_member.h 5.1-1/include/CGAL/Has_member.h
--- 5.0.3-1/include/CGAL/Has_member.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Has_member.h	2020-09-08 12:15:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Has_member.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Has_member.h $
 // $Id: Has_member.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Has_timestamp.h 5.1-1/include/CGAL/Has_timestamp.h
--- 5.0.3-1/include/CGAL/Has_timestamp.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Has_timestamp.h	2020-09-08 12:15:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Has_timestamp.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Has_timestamp.h $
 // $Id: Has_timestamp.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Heat_method_3/internal/Intrinsic_Delaunay_triangulation_3.h 5.1-1/include/CGAL/Heat_method_3/internal/Intrinsic_Delaunay_triangulation_3.h
--- 5.0.3-1/include/CGAL/Heat_method_3/internal/Intrinsic_Delaunay_triangulation_3.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Heat_method_3/internal/Intrinsic_Delaunay_triangulation_3.h	2020-09-08 12:14:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Heat_method_3/include/CGAL/Heat_method_3/internal/Intrinsic_Delaunay_triangulation_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Heat_method_3/include/CGAL/Heat_method_3/internal/Intrinsic_Delaunay_triangulation_3.h $
 // $Id: Intrinsic_Delaunay_triangulation_3.h 67b53fa 2020-04-29T14:06:37+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Heat_method_3/internal/V2V.h 5.1-1/include/CGAL/Heat_method_3/internal/V2V.h
--- 5.0.3-1/include/CGAL/Heat_method_3/internal/V2V.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Heat_method_3/internal/V2V.h	2020-09-08 12:14:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Heat_method_3/include/CGAL/Heat_method_3/internal/V2V.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Heat_method_3/include/CGAL/Heat_method_3/internal/V2V.h $
 // $Id: V2V.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Heat_method_3/Surface_mesh_geodesic_distances_3.h 5.1-1/include/CGAL/Heat_method_3/Surface_mesh_geodesic_distances_3.h
--- 5.0.3-1/include/CGAL/Heat_method_3/Surface_mesh_geodesic_distances_3.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Heat_method_3/Surface_mesh_geodesic_distances_3.h	2020-09-08 12:14:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Heat_method_3/include/CGAL/Heat_method_3/Surface_mesh_geodesic_distances_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Heat_method_3/include/CGAL/Heat_method_3/Surface_mesh_geodesic_distances_3.h $
 // $Id: Surface_mesh_geodesic_distances_3.h 67b53fa 2020-04-29T14:06:37+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Hidden_point_memory_policy.h 5.1-1/include/CGAL/Hidden_point_memory_policy.h
--- 5.0.3-1/include/CGAL/Hidden_point_memory_policy.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Hidden_point_memory_policy.h	2020-09-08 12:15:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Hidden_point_memory_policy.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Hidden_point_memory_policy.h $
 // $Id: Hidden_point_memory_policy.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/hierarchy_simplify_point_set.h 5.1-1/include/CGAL/hierarchy_simplify_point_set.h
--- 5.0.3-1/include/CGAL/hierarchy_simplify_point_set.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/hierarchy_simplify_point_set.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_processing_3/include/CGAL/hierarchy_simplify_point_set.h $
-// $Id: hierarchy_simplify_point_set.h 84d7e9a 2020-06-23T18:48:11+02:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/hierarchy_simplify_point_set.h $
+// $Id: hierarchy_simplify_point_set.h 93f1cd9 2020-07-16T09:53:31+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s) : Simon Giraudot, Pierre Alliez
@@ -123,25 +123,54 @@ namespace CGAL {
      its iterator is the key type of the named parameter `point_map`.
 
      \param points input point range.
-     \param np optional sequence of \ref psp_namedparameters "Named Parameters" among the ones listed below.
+     \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
      \cgalNamedParamsBegin
-       \cgalParamBegin{point_map} a model of `ReadWritePropertyMap` with value type `geom_traits::Point_3`.
-       If this parameter is omitted, `CGAL::Identity_property_map<geom_traits::Point_3>` is used.\cgalParamEnd
-       \cgalParamBegin{size} maximum cluster size.\cgalParamEnd
-       \cgalParamBegin{maximum_variation} maximum cluster variation value.\cgalParamEnd
-       \cgalParamBegin{diagonalize_traits} a model of `DiagonalizeTraits`. It can be omitted:
-       if Eigen 3 (or greater) is available and `CGAL_EIGEN3_ENABLED` is defined then an overload
-       using `Eigen_diagonalize_traits` is provided. Otherwise, the internal implementation
-       `CGAL::Diagonalize_traits` is used.\cgalParamEnd
-       \cgalParamBegin{callback} an instance of
-       `std::function<bool(double)>`. It is called regularly when the
-       algorithm is running: the current advancement (between 0. and
-       1.) is passed as parameter. If it returns `true`, then the
-       algorithm continues its execution normally; if it returns
-       `false`, the algorithm is stopped and simplification stops with
-       no guarantee on the output.\cgalParamEnd
-       \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+       \cgalParamNBegin{point_map}
+         \cgalParamDescription{a property map associating points to the elements of the point set `points`}
+         \cgalParamType{a model of `ReadablePropertyMap` whose key type is the value type
+                        of the iterator of `PointRange` and whose value type is `geom_traits::Point_3`}
+         \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>`}
+       \cgalParamNEnd
+
+       \cgalParamNBegin{callback}
+         \cgalParamDescription{a mechanism to get feedback on the advancement of the algorithm
+                               while it's running and to interrupt it if needed}
+         \cgalParamType{an instance of `std::function<bool(double)>`.}
+         \cgalParamDefault{unused}
+         \cgalParamExtra{It is called regularly when the
+                         algorithm is running: the current advancement (between 0. and
+                         1.) is passed as parameter. If it returns `true`, then the
+                         algorithm continues its execution normally; if it returns
+                         `false`, the algorithm is stopped and simplification stops with no guarantee on the output.}
+         \cgalParamExtra{The callback will be copied and therefore needs to be lightweight.}
+       \cgalParamNEnd
+
+       \cgalParamNBegin{size}
+         \cgalParamDescription{a value for cluster size}
+         \cgalParamType{unsigned int}
+         \cgalParamDefault{`10`}
+       \cgalParamNEnd
+
+       \cgalParamNBegin{maximum_variation}
+         \cgalParamDescription{a value for maximum cluster variation}
+         \cgalParamType{floating scalar value}
+         \cgalParamDefault{`1/3`}
+       \cgalParamNEnd
+
+       \cgalParamNBegin{diagonalize_traits}
+         \cgalParamDescription{the solver used for diagonalizing covariance matrices}
+         \cgalParamType{a model of `DiagonalizeTraits`}
+         \cgalParamDefault{If Eigen 3 (or greater) is available and `CGAL_EIGEN3_ENABLED` is defined
+                           then an overload using `Eigen_diagonalize_traits` is provided.
+                           Otherwise, the internal implementation `CGAL::Diagonalize_traits` is used.}
+       \cgalParamNEnd
+
+       \cgalParamNBegin{geom_traits}
+         \cgalParamDescription{an instance of a geometric traits class}
+         \cgalParamType{a model of `Kernel`}
+         \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+       \cgalParamNEnd
      \cgalNamedParamsEnd
 
      \return iterator over the first point to remove.
@@ -156,7 +185,7 @@ namespace CGAL {
     using parameters::get_parameter;
 
     // basic geometric types
-    typedef typename Point_set_processing_3::GetPointMap<PointRange, NamedParameters>::type PointMap;
+    typedef typename CGAL::GetPointMap<PointRange, NamedParameters>::type PointMap;
     typedef typename Point_set_processing_3::GetK<PointRange, NamedParameters>::Kernel Kernel;
     typedef typename GetDiagonalizeTraits<NamedParameters, double, 3>::type DiagonalizeTraits;
 
@@ -164,7 +193,7 @@ namespace CGAL {
     typedef typename Kernel::Vector_3 Vector;
     typedef typename Kernel::FT FT;
 
-    PointMap point_map = choose_parameter(get_parameter(np, internal_np::point_map), PointMap());
+    PointMap point_map = choose_parameter<PointMap>(get_parameter(np, internal_np::point_map));
     unsigned int size = choose_parameter(get_parameter(np, internal_np::size), 10);
     double var_max = choose_parameter(get_parameter(np, internal_np::maximum_variation), 1./3.);
     const std::function<bool(double)>& callback = choose_parameter(get_parameter(np, internal_np::callback),
diff -pruN 5.0.3-1/include/CGAL/Hilbert_policy_tags.h 5.1-1/include/CGAL/Hilbert_policy_tags.h
--- 5.0.3-1/include/CGAL/Hilbert_policy_tags.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Hilbert_policy_tags.h	2020-09-08 12:15:40.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_sorting/include/CGAL/Hilbert_policy_tags.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_sorting/include/CGAL/Hilbert_policy_tags.h $
 // $Id: Hilbert_policy_tags.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Hilbert_sort_2.h 5.1-1/include/CGAL/Hilbert_sort_2.h
--- 5.0.3-1/include/CGAL/Hilbert_sort_2.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Hilbert_sort_2.h	2020-09-08 12:15:40.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_sorting/include/CGAL/Hilbert_sort_2.h $
-// $Id: Hilbert_sort_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_sorting/include/CGAL/Hilbert_sort_2.h $
+// $Id: Hilbert_sort_2.h 5c41b10 2020-01-02T10:26:44+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Olivier Devillers
@@ -18,27 +18,27 @@
 
 namespace CGAL {
 
-template <class K,  class Hilbert_policy >
-  class Hilbert_sort_2;
+template <class K,  class Hilbert_policy, class ConcurrencyTag = Sequential_tag >
+class Hilbert_sort_2;
 
-template <class K>
-  class Hilbert_sort_2<K, Hilbert_sort_median_policy >
-  : public Hilbert_sort_median_2<K>
+template <class K, class ConcurrencyTag>
+class Hilbert_sort_2<K, Hilbert_sort_median_policy, ConcurrencyTag >
+  : public Hilbert_sort_median_2<K, ConcurrencyTag>
 {
- public:
-  Hilbert_sort_2 (const K &k=K() , std::ptrdiff_t limit=1 )
-   : Hilbert_sort_median_2<K> (k,limit)
-    {}
+public:
+  Hilbert_sort_2 (const K &k=K(), std::ptrdiff_t limit=1 )
+    : Hilbert_sort_median_2<K, ConcurrencyTag> (k,limit)
+  {}
 };
 
-template <class K>
-  class Hilbert_sort_2<K, Hilbert_sort_middle_policy >
+template <class K, class ConcurrencyTag>
+class Hilbert_sort_2<K, Hilbert_sort_middle_policy, ConcurrencyTag >
   : public Hilbert_sort_middle_2<K>
 {
- public:
- Hilbert_sort_2 (const K &k=K() , std::ptrdiff_t limit=1 )
-   : Hilbert_sort_middle_2<K> (k,limit)
-    {}
+public:
+  Hilbert_sort_2 (const K &k=K(), std::ptrdiff_t limit=1 )
+    : Hilbert_sort_middle_2<K> (k,limit)
+  {}
 };
 
 } // namespace CGAL
diff -pruN 5.0.3-1/include/CGAL/Hilbert_sort_3.h 5.1-1/include/CGAL/Hilbert_sort_3.h
--- 5.0.3-1/include/CGAL/Hilbert_sort_3.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Hilbert_sort_3.h	2020-09-08 12:15:40.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_sorting/include/CGAL/Hilbert_sort_3.h $
-// $Id: Hilbert_sort_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_sorting/include/CGAL/Hilbert_sort_3.h $
+// $Id: Hilbert_sort_3.h 5c41b10 2020-01-02T10:26:44+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Olivier Devillers
@@ -18,27 +18,27 @@
 
 namespace CGAL {
 
-template <class K,  class Hilbert_policy >
-  class Hilbert_sort_3;
+template <class K, class Hilbert_policy, class ConcurrencyTag = Sequential_tag>
+class Hilbert_sort_3;
 
-template <class K>
-  class Hilbert_sort_3<K, Hilbert_sort_median_policy >
-  : public Hilbert_sort_median_3<K>
+template <class K, class ConcurrencyTag>
+class Hilbert_sort_3<K, Hilbert_sort_median_policy, ConcurrencyTag >
+  : public Hilbert_sort_median_3<K, ConcurrencyTag>
 {
- public:
- Hilbert_sort_3 (const K &k=K() , std::ptrdiff_t limit=1 )
-   : Hilbert_sort_median_3<K> (k,limit)
-    {}
+public:
+  Hilbert_sort_3 (const K &k=K(), std::ptrdiff_t limit=1 )
+    : Hilbert_sort_median_3<K, ConcurrencyTag> (k,limit)
+  {}
 };
 
-template <class K>
-  class Hilbert_sort_3<K, Hilbert_sort_middle_policy >
+template <class K, class ConcurrencyTag >
+class Hilbert_sort_3<K, Hilbert_sort_middle_policy, ConcurrencyTag >
   : public Hilbert_sort_middle_3<K>
 {
- public:
- Hilbert_sort_3 (const K &k=K() , std::ptrdiff_t limit=1 )
-   : Hilbert_sort_middle_3<K> (k,limit)
-    {}
+public:
+  Hilbert_sort_3 (const K &k=K(), std::ptrdiff_t limit=1 )
+    : Hilbert_sort_middle_3<K> (k,limit)
+  {}
 };
 
 } // namespace CGAL
diff -pruN 5.0.3-1/include/CGAL/Hilbert_sort_base.h 5.1-1/include/CGAL/Hilbert_sort_base.h
--- 5.0.3-1/include/CGAL/Hilbert_sort_base.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Hilbert_sort_base.h	2020-09-08 12:15:40.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_sorting/include/CGAL/Hilbert_sort_base.h $
-// $Id: Hilbert_sort_base.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_sorting/include/CGAL/Hilbert_sort_base.h $
+// $Id: Hilbert_sort_base.h 5c41b10 2020-01-02T10:26:44+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Christophe Delage
@@ -15,30 +15,33 @@
 #include <CGAL/config.h>
 #include <algorithm>
 #include <CGAL/algorithm.h>
+#include <CGAL/use.h>
 
 namespace CGAL {
 
 namespace internal {
 
-    template <class RandomAccessIterator, class Cmp>
-    RandomAccessIterator
-    hilbert_split (RandomAccessIterator begin, RandomAccessIterator end,
-                   Cmp cmp = Cmp ())
-    {
-        if (begin >= end) return begin;
-      #if defined(CGAL_HILBERT_SORT_WITH_MEDIAN_POLICY_CROSS_PLATFORM_BEHAVIOR)
-        RandomAccessIterator middle = begin + (end - begin) / 2;
-        CGAL::nth_element (begin, middle, end, cmp);
-        return middle;
-      #else
-        RandomAccessIterator middle = begin + (end - begin) / 2;
-        std::nth_element (begin, middle, end, cmp);
-        return middle;
-      #endif
-
-    }
+template <class RandomAccessIterator, class Cmp>
+RandomAccessIterator
+hilbert_split (RandomAccessIterator begin, RandomAccessIterator end,
+               Cmp cmp = Cmp ())
+{
+  if (begin >= end)
+    return begin;
+
+#if defined(CGAL_HILBERT_SORT_WITH_MEDIAN_POLICY_CROSS_PLATFORM_BEHAVIOR)
+  RandomAccessIterator middle = begin + (end - begin) / 2;
+  CGAL::nth_element (begin, middle, end, cmp);
+  return middle;
+#else
+  RandomAccessIterator middle = begin + (end - begin) / 2;
+  std::nth_element (begin, middle, end, cmp);
+  return middle;
+#endif
 }
 
+} // namespace internal
+
 } // namespace CGAL
 
 #endif//CGAL_HILBERT_SORT_BASE_H
diff -pruN 5.0.3-1/include/CGAL/Hilbert_sort_d.h 5.1-1/include/CGAL/Hilbert_sort_d.h
--- 5.0.3-1/include/CGAL/Hilbert_sort_d.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Hilbert_sort_d.h	2020-09-08 12:15:40.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_sorting/include/CGAL/Hilbert_sort_d.h $
-// $Id: Hilbert_sort_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_sorting/include/CGAL/Hilbert_sort_d.h $
+// $Id: Hilbert_sort_d.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Olivier Devillers
@@ -19,26 +19,26 @@
 namespace CGAL {
 
 template <class K,  class Hilbert_policy >
-  class Hilbert_sort_d;
+class Hilbert_sort_d;
 
 template <class K>
-  class Hilbert_sort_d<K, Hilbert_sort_median_policy >
-  : public Hilbert_sort_median_d<K>
+class Hilbert_sort_d<K, Hilbert_sort_median_policy >
+    : public Hilbert_sort_median_d<K>
 {
- public:
- Hilbert_sort_d (const K &k=K() , std::ptrdiff_t limit=1 )
-   : Hilbert_sort_median_d<K> (k,limit)
-    {}
+public:
+  Hilbert_sort_d (const K &k=K() , std::ptrdiff_t limit=1 )
+    : Hilbert_sort_median_d<K> (k,limit)
+  {}
 };
 
 template <class K>
-  class Hilbert_sort_d<K, Hilbert_sort_middle_policy >
-  : public Hilbert_sort_middle_d<K>
+class Hilbert_sort_d<K, Hilbert_sort_middle_policy >
+    : public Hilbert_sort_middle_d<K>
 {
- public:
- Hilbert_sort_d (const K &k=K() , std::ptrdiff_t limit=1 )
-   : Hilbert_sort_middle_d<K> (k,limit)
-    {}
+public:
+  Hilbert_sort_d (const K &k=K() , std::ptrdiff_t limit=1 )
+    : Hilbert_sort_middle_d<K> (k,limit)
+  {}
 };
 
 } // namespace CGAL
diff -pruN 5.0.3-1/include/CGAL/hilbert_sort.h 5.1-1/include/CGAL/hilbert_sort.h
--- 5.0.3-1/include/CGAL/hilbert_sort.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/hilbert_sort.h	2020-09-08 12:15:40.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_sorting/include/CGAL/hilbert_sort.h $
-// $Id: hilbert_sort.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_sorting/include/CGAL/hilbert_sort.h $
+// $Id: hilbert_sort.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Christophe Delage
@@ -27,120 +27,115 @@
 #include <algorithm>
 
 namespace CGAL {
+namespace internal {
 
+template <class ConcurrencyTag = Sequential_tag, class RandomAccessIterator, class Kernel, class Policy>
+void hilbert_sort (RandomAccessIterator begin,
+                   RandomAccessIterator end,
+                   const Kernel &k,
+                   Policy /*policy*/,
+                   typename Kernel::Point_2 *)
+{
+  typedef std::iterator_traits<RandomAccessIterator> ITraits;
+  typedef typename ITraits::difference_type Diff_t;
+  boost::rand48 random;
+  boost::random_number_generator<boost::rand48, Diff_t> rng(random);
+  CGAL::cpp98::random_shuffle(begin,end, rng);
+  (Hilbert_sort_2<Kernel, Policy, ConcurrencyTag> (k))(begin, end);
+}
 
-namespace internal {
+template <class ConcurrencyTag = Sequential_tag, class RandomAccessIterator, class Kernel, class Policy>
+void hilbert_sort (RandomAccessIterator begin,
+                   RandomAccessIterator end,
+                   const Kernel &k,
+                   Policy /*policy*/,
+                   typename Kernel::Point_3 *)
+{
+  typedef std::iterator_traits<RandomAccessIterator> ITraits;
+  typedef typename ITraits::difference_type Diff_t;
+  boost::rand48 random;
+  boost::random_number_generator<boost::rand48, Diff_t> rng(random);
+  CGAL::cpp98::random_shuffle(begin,end, rng);
+  (Hilbert_sort_3<Kernel, Policy, ConcurrencyTag> (k))(begin, end);
+}
 
-  template <class RandomAccessIterator, class Kernel, class Policy>
-    void hilbert_sort (RandomAccessIterator begin,
-                       RandomAccessIterator end,
-                       const Kernel &k,
-                       Policy /*policy*/,
-                       typename Kernel::Point_2 *)
-    {
-        typedef std::iterator_traits<RandomAccessIterator> ITraits;
-        typedef typename ITraits::difference_type Diff_t;
-        boost::rand48 random;
-        boost::random_number_generator<boost::rand48, Diff_t> rng(random);
-        CGAL::cpp98::random_shuffle(begin,end, rng);
-        (Hilbert_sort_2<Kernel, Policy> (k))(begin, end);
-    }
-
-    template <class RandomAccessIterator, class Kernel, class Policy>
-      void hilbert_sort (RandomAccessIterator begin,
-                         RandomAccessIterator end,
-                         const Kernel &k,
-                         Policy /*policy*/,
-                         typename Kernel::Point_3 *)
-    {
-        typedef std::iterator_traits<RandomAccessIterator> ITraits;
-        typedef typename ITraits::difference_type Diff_t;
-        boost::rand48 random;
-        boost::random_number_generator<boost::rand48, Diff_t> rng(random);
-        CGAL::cpp98::random_shuffle(begin,end, rng);
-        (Hilbert_sort_3<Kernel, Policy> (k))(begin, end);
-    }
-
-    template <class RandomAccessIterator, class Kernel, class Policy>
-      void hilbert_sort (RandomAccessIterator begin,
-                         RandomAccessIterator end,
-                         const Kernel &k,
-                         Policy /*policy*/,
-                         typename Kernel::Point_d *)
-    {
-        typedef std::iterator_traits<RandomAccessIterator> ITraits;
-        typedef typename ITraits::difference_type Diff_t;
-        boost::rand48 random;
-        boost::random_number_generator<boost::rand48, Diff_t> rng(random);
-        CGAL::cpp98::random_shuffle(begin,end, rng);
-        (Hilbert_sort_d<Kernel, Policy> (k))(begin, end);
-    }
+template <class ConcurrencyTag = Sequential_tag, class RandomAccessIterator, class Kernel, class Policy>
+void hilbert_sort (RandomAccessIterator begin,
+                   RandomAccessIterator end,
+                   const Kernel &k,
+                   Policy /*policy*/,
+                   typename Kernel::Point_d *)
+{
+  typedef std::iterator_traits<RandomAccessIterator> ITraits;
+  typedef typename ITraits::difference_type Diff_t;
+  boost::rand48 random;
+  boost::random_number_generator<boost::rand48, Diff_t> rng(random);
+  CGAL::cpp98::random_shuffle(begin,end, rng);
+  (Hilbert_sort_d<Kernel, Policy> (k))(begin, end);
 }
 
-template <class RandomAccessIterator>
+} // namespace internal
+
+template <class ConcurrencyTag = Sequential_tag, class RandomAccessIterator>
 void hilbert_sort (RandomAccessIterator begin, RandomAccessIterator end)
 {
 
-    typedef std::iterator_traits<RandomAccessIterator> ITraits;
-    typedef typename ITraits::value_type               value_type;
-    typedef CGAL::Kernel_traits<value_type>            KTraits;
-    typedef typename KTraits::Kernel                   Kernel;
+  typedef std::iterator_traits<RandomAccessIterator> ITraits;
+  typedef typename ITraits::value_type               value_type;
+  typedef CGAL::Kernel_traits<value_type>            KTraits;
+  typedef typename KTraits::Kernel                   Kernel;
 
-    internal::hilbert_sort(begin, end, Kernel(), Hilbert_sort_median_policy(),
-                                  static_cast<value_type *> (0));
+  internal::hilbert_sort<ConcurrencyTag>(begin, end, Kernel(), Hilbert_sort_median_policy(),
+                                         static_cast<value_type *> (0));
 
 }
 
-template <class RandomAccessIterator, class Kernel>
+template <class ConcurrencyTag = Sequential_tag, class RandomAccessIterator, class Kernel>
 void hilbert_sort (RandomAccessIterator begin, RandomAccessIterator end,
                    const Kernel &k)
 {
-    typedef std::iterator_traits<RandomAccessIterator> ITraits;
-    typedef typename ITraits::value_type               value_type;
+  typedef std::iterator_traits<RandomAccessIterator> ITraits;
+  typedef typename ITraits::value_type               value_type;
 
-    internal::hilbert_sort(begin, end, k, Hilbert_sort_median_policy(),
-                                  static_cast<value_type *> (0));
+  internal::hilbert_sort<ConcurrencyTag>(begin, end, k, Hilbert_sort_median_policy(),
+                                         static_cast<value_type *> (0));
 }
 
-  template <class RandomAccessIterator>
+template <class ConcurrencyTag = Sequential_tag, class RandomAccessIterator>
 void hilbert_sort (RandomAccessIterator begin, RandomAccessIterator end,
                    Hilbert_sort_median_policy policy)
 {
-    typedef std::iterator_traits<RandomAccessIterator> ITraits;
-    typedef typename ITraits::value_type               value_type;
-    typedef CGAL::Kernel_traits<value_type>            KTraits;
-    typedef typename KTraits::Kernel                   Kernel;
-
-    internal::hilbert_sort(begin, end, Kernel(), policy,
-                                  static_cast<value_type *> (0));
+  typedef std::iterator_traits<RandomAccessIterator> ITraits;
+  typedef typename ITraits::value_type               value_type;
+  typedef CGAL::Kernel_traits<value_type>            KTraits;
+  typedef typename KTraits::Kernel                   Kernel;
 
+  internal::hilbert_sort<ConcurrencyTag>(begin, end, Kernel(), policy,
+                                         static_cast<value_type *> (0));
 }
 
-
-  template <class RandomAccessIterator>
+template <class ConcurrencyTag = Sequential_tag, class RandomAccessIterator>
 void hilbert_sort (RandomAccessIterator begin, RandomAccessIterator end,
                    Hilbert_sort_middle_policy policy)
 {
-    typedef std::iterator_traits<RandomAccessIterator> ITraits;
-    typedef typename ITraits::value_type               value_type;
-    typedef CGAL::Kernel_traits<value_type>            KTraits;
-    typedef typename KTraits::Kernel                   Kernel;
-
-    internal::hilbert_sort(begin, end, Kernel(), policy,
-                                  static_cast<value_type *> (0));
+  typedef std::iterator_traits<RandomAccessIterator> ITraits;
+  typedef typename ITraits::value_type               value_type;
+  typedef CGAL::Kernel_traits<value_type>            KTraits;
+  typedef typename KTraits::Kernel                   Kernel;
 
+  internal::hilbert_sort<ConcurrencyTag>(begin, end, Kernel(), policy,
+                                         static_cast<value_type *> (0));
 }
 
-
-  template <class RandomAccessIterator, class Kernel, class Policy>
+template <class ConcurrencyTag = Sequential_tag, class RandomAccessIterator, class Kernel, class Policy>
 void hilbert_sort (RandomAccessIterator begin, RandomAccessIterator end,
                    const Kernel &k, Policy policy)
 {
-    typedef std::iterator_traits<RandomAccessIterator> ITraits;
-    typedef typename ITraits::value_type               value_type;
+  typedef std::iterator_traits<RandomAccessIterator> ITraits;
+  typedef typename ITraits::value_type               value_type;
 
-    internal::hilbert_sort(begin, end,
-                           k, policy, static_cast<value_type *> (0));
+  internal::hilbert_sort<ConcurrencyTag>(begin, end,
+                                         k, policy, static_cast<value_type *> (0));
 }
 
 } // namespace CGAL
diff -pruN 5.0.3-1/include/CGAL/Hilbert_sort_median_2.h 5.1-1/include/CGAL/Hilbert_sort_median_2.h
--- 5.0.3-1/include/CGAL/Hilbert_sort_median_2.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Hilbert_sort_median_2.h	2020-09-08 12:15:40.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_sorting/include/CGAL/Hilbert_sort_median_2.h $
-// $Id: Hilbert_sort_median_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_sorting/include/CGAL/Hilbert_sort_median_2.h $
+// $Id: Hilbert_sort_median_2.h 4eeb7f1 2020-02-06T11:57:04+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Christophe Delage
@@ -12,101 +12,188 @@
 #ifndef CGAL_HILBERT_SORT_MEDIAN_2_H
 #define CGAL_HILBERT_SORT_MEDIAN_2_H
 
+#include <CGAL/assertions.h>
 #include <CGAL/config.h>
+#include <CGAL/tags.h>
+#include <CGAL/Hilbert_sort_base.h>
+
+#include <boost/type_traits/is_convertible.hpp>
+
+#ifdef CGAL_LINKED_WITH_TBB
+#include <tbb/parallel_invoke.h>
+#endif
+
 #include <functional>
 #include <cstddef>
-#include <CGAL/Hilbert_sort_base.h>
 
 namespace CGAL {
 
 namespace internal {
-    template <class K, int x, bool up> struct Hilbert_cmp_2;
 
-    template <class K, int x>
-    struct Hilbert_cmp_2<K,x,true>
-        : public CGAL::cpp98::binary_function<typename K::Point_2,
-                                              typename K::Point_2, bool>
-    {
-        typedef typename K::Point_2 Point;
-        K k;
-        Hilbert_cmp_2 (const K &_k = K()) : k(_k) {}
-        bool operator() (const Point &p, const Point &q) const
-        {
-            return Hilbert_cmp_2<K,x,false> (k) (q, p);
-        }
-    };
-
-    template <class K>
-    struct Hilbert_cmp_2<K,0,false>
-        : public CGAL::cpp98::binary_function<typename K::Point_2,
-                                              typename K::Point_2, bool>
-    {
-        typedef typename K::Point_2 Point;
-        K k;
-        Hilbert_cmp_2 (const K &_k = K()) : k(_k) {}
-        bool operator() (const Point &p, const Point &q) const
-        {
-          return k.less_x_2_object() (p, q);
-        }
-    };
-
-    template <class K>
-    struct Hilbert_cmp_2<K,1,false>
-        : public CGAL::cpp98::binary_function<typename K::Point_2,
-                                              typename K::Point_2, bool>
-    {
-        typedef typename K::Point_2 Point;
-        K k;
-        Hilbert_cmp_2 (const K &_k = K()) : k(_k) {}
-        bool operator() (const Point &p, const Point &q) const
-        {
-            return k.less_y_2_object() (p, q);
-        }
-    };
-}
+template <class K, int x, bool up> struct Hilbert_cmp_2;
+
+template <class K, int x>
+struct Hilbert_cmp_2<K,x,true>
+  : public CGAL::cpp98::binary_function<typename K::Point_2, typename K::Point_2, bool>
+{
+  typedef typename K::Point_2 Point;
+  K k;
+  Hilbert_cmp_2 (const K &_k = K()) : k(_k) {}
+  bool operator() (const Point &p, const Point &q) const
+  {
+    return Hilbert_cmp_2<K,x,false> (k) (q, p);
+  }
+};
 
 template <class K>
+struct Hilbert_cmp_2<K,0,false>
+    : public CGAL::cpp98::binary_function<typename K::Point_2, typename K::Point_2, bool>
+{
+  typedef typename K::Point_2 Point;
+  K k;
+  Hilbert_cmp_2 (const K &_k = K()) : k(_k) {}
+  bool operator() (const Point &p, const Point &q) const
+  {
+    return k.less_x_2_object() (p, q);
+  }
+};
+
+template <class K>
+struct Hilbert_cmp_2<K,1,false>
+  : public CGAL::cpp98::binary_function<typename K::Point_2, typename K::Point_2, bool>
+{
+  typedef typename K::Point_2 Point;
+  K k;
+  Hilbert_cmp_2 (const K &_k = K()) : k(_k) {}
+  bool operator() (const Point &p, const Point &q) const
+  {
+    return k.less_y_2_object() (p, q);
+  }
+};
+
+} // namespace internal
+
+template <class K, class ConcurrencyTag>
 class Hilbert_sort_median_2
 {
 public:
-    typedef K Kernel;
-    typedef typename Kernel::Point_2 Point;
+  typedef Hilbert_sort_median_2<K, ConcurrencyTag> Self;
+  typedef K Kernel;
+  typedef typename Kernel::Point_2 Point;
 
 private:
-    Kernel _k;
-    std::ptrdiff_t _limit;
+  Kernel _k;
+  std::ptrdiff_t _limit;
 
-    template <int x, bool up> struct Cmp : public internal::Hilbert_cmp_2<Kernel,x,up>
-    { Cmp (const Kernel &k) : internal::Hilbert_cmp_2<Kernel,x,up> (k) {} };
+  template <int x, bool up>
+  struct Cmp
+    : public internal::Hilbert_cmp_2<Kernel,x,up>
+  {
+    Cmp (const Kernel &k) : internal::Hilbert_cmp_2<Kernel,x,up> (k) {}
+  };
 
 public:
-    Hilbert_sort_median_2 (const Kernel &k = Kernel(), std::ptrdiff_t limit = 1)
-        : _k(k), _limit (limit)
+  Hilbert_sort_median_2 (const Kernel &k = Kernel(), std::ptrdiff_t limit = 1)
+    : _k(k), _limit (limit)
+  {}
+
+  template <int x, bool upx, bool upy, class RandomAccessIterator>
+  void recursive_sort (RandomAccessIterator begin, RandomAccessIterator end) const
+  {
+    const int y = (x + 1) % 2;
+    if (end - begin <= _limit)
+      return;
+
+    RandomAccessIterator m0 = begin, m4 = end;
+
+    RandomAccessIterator m2 = internal::hilbert_split (m0, m4, Cmp< x,  upx> (_k));
+    RandomAccessIterator m1 = internal::hilbert_split (m0, m2, Cmp< y,  upy> (_k));
+    RandomAccessIterator m3 = internal::hilbert_split (m2, m4, Cmp< y, !upy> (_k));
+
+    recursive_sort<y, upy, upx> (m0, m1);
+    recursive_sort<x, upx, upy> (m1, m2);
+    recursive_sort<x, upx, upy> (m2, m3);
+    recursive_sort<y,!upy,!upx> (m3, m4);
+  }
+
+  template <int x, bool upx, bool upy, class RandomAccessIterator>
+  struct Recursive_sort
+  {
+    const Self& hs;
+    RandomAccessIterator begin,end;
+
+    Recursive_sort(const Self& hs, RandomAccessIterator begin, RandomAccessIterator end)
+      : hs(hs), begin(begin), end(end)
     {}
 
-    template <int x, bool upx, bool upy, class RandomAccessIterator>
-    void sort (RandomAccessIterator begin, RandomAccessIterator end) const
+    void operator()() const
     {
-        const int y = (x + 1) % 2;
-        if (end - begin <= _limit) return;
+      hs.recursive_sort<x,upx,upy>(begin,end);
+    }
+  };
 
-        RandomAccessIterator m0 = begin, m4 = end;
+  template <class RandomAccessIterator, class Comp>
+  struct Hilbert_split
+  {
+    RandomAccessIterator& res;
+    RandomAccessIterator begin, end;
+    const Comp& comp;
+
+    Hilbert_split(RandomAccessIterator& res,
+                  RandomAccessIterator begin, RandomAccessIterator end,
+                  const Comp& comp)
+      : res(res), begin(begin), end(end), comp(comp)
+    {}
 
-        RandomAccessIterator m2 = internal::hilbert_split (m0, m4, Cmp< x,  upx> (_k));
-        RandomAccessIterator m1 = internal::hilbert_split (m0, m2, Cmp< y,  upy> (_k));
-        RandomAccessIterator m3 = internal::hilbert_split (m2, m4, Cmp< y, !upy> (_k));
-
-        sort<y, upy, upx> (m0, m1);
-        sort<x, upx, upy> (m1, m2);
-        sort<x, upx, upy> (m2, m3);
-        sort<y,!upy,!upx> (m3, m4);
+    void operator()() const
+    {
+      res = internal::hilbert_split(begin, end, comp);
     }
+  };
 
-    template <class RandomAccessIterator>
-    void operator() (RandomAccessIterator begin, RandomAccessIterator end) const
-    {
-        sort <0, false, false> (begin, end);
+  template <int x, bool upx, bool upy, class RandomAccessIterator>
+  void sort (RandomAccessIterator begin, RandomAccessIterator end, Parallel_tag) const
+  {
+#ifndef CGAL_LINKED_WITH_TBB
+    CGAL_USE(begin);
+    CGAL_USE(end);
+    CGAL_static_assertion_msg (!(boost::is_convertible<ConcurrencyTag, Parallel_tag>::value),
+                               "Parallel_tag is enabled but TBB is unavailable.");
+#else
+    const int y = (x + 1) % 2;
+    if (std::distance(begin,end) <= _limit)
+      return;
+
+    RandomAccessIterator m0 = begin, m4 = end;
+
+    if(std::distance(begin,end) > 8192){ // 2^13, empirically a good cutoff
+      RandomAccessIterator m1, m2, m3;
+      m2 = internal::hilbert_split (m0, m4, Cmp< x,  upx> (_k));
+
+      tbb::parallel_invoke(Hilbert_split<RandomAccessIterator, Cmp< y,  upy> > (m1, m0, m2, Cmp< y,  upy> (_k)),
+                           Hilbert_split<RandomAccessIterator, Cmp< y, !upy> > (m3, m2, m4, Cmp< y, !upy> (_k)));
+
+      tbb::parallel_invoke(Recursive_sort<y, upy, upx, RandomAccessIterator> (*this, m0, m1),
+                           Recursive_sort<x, upx, upy, RandomAccessIterator> (*this, m1, m2),
+                           Recursive_sort<x, upx, upy, RandomAccessIterator> (*this, m2, m3),
+                           Recursive_sort<y,!upy,!upx, RandomAccessIterator> (*this, m3, m4));
+    } else {
+      recursive_sort<0, false, false>(begin, end);
     }
+#endif
+  }
+
+  template <int x, bool upx, bool upy, class RandomAccessIterator>
+  void sort (RandomAccessIterator begin, RandomAccessIterator end, Sequential_tag) const
+  {
+    recursive_sort<0, false, false>(begin, end);
+  }
+
+  template <class RandomAccessIterator>
+  void operator() (RandomAccessIterator begin, RandomAccessIterator end) const
+  {
+    sort <0, false, false> (begin, end, ConcurrencyTag());
+  }
 };
 
 } // namespace CGAL
diff -pruN 5.0.3-1/include/CGAL/Hilbert_sort_median_3.h 5.1-1/include/CGAL/Hilbert_sort_median_3.h
--- 5.0.3-1/include/CGAL/Hilbert_sort_median_3.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Hilbert_sort_median_3.h	2020-09-08 12:15:40.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_sorting/include/CGAL/Hilbert_sort_median_3.h $
-// $Id: Hilbert_sort_median_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_sorting/include/CGAL/Hilbert_sort_median_3.h $
+// $Id: Hilbert_sort_median_3.h 4eeb7f1 2020-02-06T11:57:04+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Christophe Delage
@@ -13,122 +13,211 @@
 #define CGAL_HILBERT_SORT_MEDIAN_3_H
 
 #include <CGAL/config.h>
+#include <CGAL/tags.h>
 #include <functional>
 #include <cstddef>
 #include <CGAL/Hilbert_sort_base.h>
 
+#include <boost/type_traits/is_convertible.hpp>
+
+#ifdef CGAL_LINKED_WITH_TBB
+#include <tbb/parallel_invoke.h>
+#endif
+
 namespace CGAL {
 
 namespace internal {
-    template <class K, int x, bool up> struct Hilbert_cmp_3;
+template <class K, int x, bool up> struct Hilbert_cmp_3;
 
-    template <class K, int x>
-    struct Hilbert_cmp_3<K,x,true>
-        : public CGAL::cpp98::binary_function<typename K::Point_3,
-                                              typename K::Point_3, bool>
-    {
-        typedef typename K::Point_3 Point;
-        K k;
-        Hilbert_cmp_3 (const K &_k = K()) : k(_k) {}
-        bool operator() (const Point &p, const Point &q) const
-        {
-            return Hilbert_cmp_3<K,x,false> (k) (q, p);
-        }
-    };
-
-    template <class K>
-    struct Hilbert_cmp_3<K,0,false>
-        : public CGAL::cpp98::binary_function<typename K::Point_3,
-                                              typename K::Point_3, bool>
-    {
-        typedef typename K::Point_3 Point;
-        K k;
-        Hilbert_cmp_3 (const K &_k = K()) : k(_k) {}
-        bool operator() (const Point &p, const Point &q) const
-        {
-            return k.less_x_3_object() (p, q);
-        }
-    };
-
-    template <class K>
-    struct Hilbert_cmp_3<K,1,false>
-        : public CGAL::cpp98::binary_function<typename K::Point_3,
-                                              typename K::Point_3, bool>
-    {
-        typedef typename K::Point_3 Point;
-        K k;
-        Hilbert_cmp_3 (const K &_k = K()) : k(_k) {}
-        bool operator() (const Point &p, const Point &q) const
-        {
-            return k.less_y_3_object() (p, q);
-        }
-    };
-
-    template <class K>
-    struct Hilbert_cmp_3<K,2,false>
-        : public CGAL::cpp98::binary_function<typename K::Point_3,
-                                              typename K::Point_3, bool>
-    {
-        typedef typename K::Point_3 Point;
-        K k;
-        Hilbert_cmp_3 (const K &_k = K()) : k(_k) {}
-        bool operator() (const Point &p, const Point &q) const
-        {
-            return k.less_z_3_object() (p, q);
-        }
-    };
-}
+template <class K, int x>
+struct Hilbert_cmp_3<K,x,true>
+  : public CGAL::cpp98::binary_function<typename K::Point_3, typename K::Point_3, bool>
+{
+  typedef typename K::Point_3 Point;
+  K k;
+  Hilbert_cmp_3 (const K &_k = K()) : k(_k) {}
+  bool operator() (const Point &p, const Point &q) const
+  {
+    return Hilbert_cmp_3<K,x,false> (k) (q, p);
+  }
+};
+
+template <class K>
+struct Hilbert_cmp_3<K,0,false>
+  : public CGAL::cpp98::binary_function<typename K::Point_3, typename K::Point_3, bool>
+{
+  typedef typename K::Point_3 Point;
+  K k;
+  Hilbert_cmp_3 (const K &_k = K()) : k(_k) {}
+  bool operator() (const Point &p, const Point &q) const
+  {
+    return k.less_x_3_object() (p, q);
+  }
+};
+
+template <class K>
+struct Hilbert_cmp_3<K,1,false>
+  : public CGAL::cpp98::binary_function<typename K::Point_3, typename K::Point_3, bool>
+{
+  typedef typename K::Point_3 Point;
+  K k;
+  Hilbert_cmp_3 (const K &_k = K()) : k(_k) {}
+  bool operator() (const Point &p, const Point &q) const
+  {
+    return k.less_y_3_object() (p, q);
+  }
+};
 
 template <class K>
+struct Hilbert_cmp_3<K,2,false>
+  : public CGAL::cpp98::binary_function<typename K::Point_3, typename K::Point_3, bool>
+{
+  typedef typename K::Point_3 Point;
+  K k;
+  Hilbert_cmp_3 (const K &_k = K()) : k(_k) {}
+  bool operator() (const Point &p, const Point &q) const
+  {
+    return k.less_z_3_object() (p, q);
+  }
+};
+
+} // namespace internal
+
+template <class K, class ConcurrencyTag>
 class Hilbert_sort_median_3
 {
 public:
-    typedef K Kernel;
-    typedef typename Kernel::Point_3 Point;
+  typedef Hilbert_sort_median_3<K, ConcurrencyTag> Self;
+  typedef K Kernel;
+  typedef typename Kernel::Point_3 Point;
 
 private:
-    Kernel _k;
-    std::ptrdiff_t _limit;
+  Kernel _k;
+  std::ptrdiff_t _limit;
 
-    template <int x, bool up> struct Cmp : public internal::Hilbert_cmp_3<Kernel,x,up>
-    { Cmp (const Kernel &k) : internal::Hilbert_cmp_3<Kernel,x,up> (k) {} };
+  template <int x, bool up>
+  struct Cmp
+    : public internal::Hilbert_cmp_3<Kernel,x,up>
+  {
+    Cmp (const Kernel &k) : internal::Hilbert_cmp_3<Kernel,x,up> (k) {}
+  };
 
 public:
-    Hilbert_sort_median_3 (const Kernel &k = Kernel(), std::ptrdiff_t limit = 1)
-        : _k(k), _limit (limit)
+  Hilbert_sort_median_3 (const Kernel &k = Kernel(), std::ptrdiff_t limit = 1)
+    : _k(k), _limit (limit)
+  {}
+
+  template <int x, bool upx, bool upy, bool upz, class RandomAccessIterator>
+  void recursive_sort (RandomAccessIterator begin, RandomAccessIterator end) const
+  {
+    const int y = (x + 1) % 3, z = (x + 2) % 3;
+    if (end - begin <= _limit) return;
+
+    RandomAccessIterator m0 = begin, m8 = end;
+
+    RandomAccessIterator m4 = internal::hilbert_split (m0, m8, Cmp< x,  upx> (_k));
+    RandomAccessIterator m2 = internal::hilbert_split (m0, m4, Cmp< y,  upy> (_k));
+    RandomAccessIterator m1 = internal::hilbert_split (m0, m2, Cmp< z,  upz> (_k));
+    RandomAccessIterator m3 = internal::hilbert_split (m2, m4, Cmp< z, !upz> (_k));
+    RandomAccessIterator m6 = internal::hilbert_split (m4, m8, Cmp< y, !upy> (_k));
+    RandomAccessIterator m5 = internal::hilbert_split (m4, m6, Cmp< z,  upz> (_k));
+    RandomAccessIterator m7 = internal::hilbert_split (m6, m8, Cmp< z, !upz> (_k));
+
+    recursive_sort<z, upz, upx, upy> (m0, m1);
+    recursive_sort<y, upy, upz, upx> (m1, m2);
+    recursive_sort<y, upy, upz, upx> (m2, m3);
+    recursive_sort<x, upx,!upy,!upz> (m3, m4);
+    recursive_sort<x, upx,!upy,!upz> (m4, m5);
+    recursive_sort<y,!upy, upz,!upx> (m5, m6);
+    recursive_sort<y,!upy, upz,!upx> (m6, m7);
+    recursive_sort<z,!upz,!upx, upy> (m7, m8);
+  }
+
+  template <int x, bool upx, bool upy, bool upz, class RandomAccessIterator>
+  struct Recursive_sort
+  {
+    const Self& hs;
+    RandomAccessIterator begin,end;
+
+    Recursive_sort(const Self& hs, RandomAccessIterator begin, RandomAccessIterator end)
+      : hs(hs), begin(begin), end(end)
     {}
 
-    template <int x, bool upx, bool upy, bool upz, class RandomAccessIterator>
-    void sort (RandomAccessIterator begin, RandomAccessIterator end) const
+    void operator()() const
     {
-        const int y = (x + 1) % 3, z = (x + 2) % 3;
-        if (end - begin <= _limit) return;
+      hs.recursive_sort<x,upx,upy,upz>(begin,end);
+    }
+  };
 
-        RandomAccessIterator m0 = begin, m8 = end;
+  template <class RandomAccessIterator, class Comp>
+  struct Hilbert_split
+  {
+    RandomAccessIterator& res;
+    RandomAccessIterator begin, end;
+    const Comp& comp;
+
+    Hilbert_split(RandomAccessIterator& res,
+                  RandomAccessIterator begin, RandomAccessIterator end,
+                  const Comp& comp)
+      : res(res), begin(begin), end(end), comp(comp)
+    {}
 
-        RandomAccessIterator m4 = internal::hilbert_split (m0, m8, Cmp< x,  upx> (_k));
-        RandomAccessIterator m2 = internal::hilbert_split (m0, m4, Cmp< y,  upy> (_k));
-        RandomAccessIterator m1 = internal::hilbert_split (m0, m2, Cmp< z,  upz> (_k));
-        RandomAccessIterator m3 = internal::hilbert_split (m2, m4, Cmp< z, !upz> (_k));
-        RandomAccessIterator m6 = internal::hilbert_split (m4, m8, Cmp< y, !upy> (_k));
-        RandomAccessIterator m5 = internal::hilbert_split (m4, m6, Cmp< z,  upz> (_k));
-        RandomAccessIterator m7 = internal::hilbert_split (m6, m8, Cmp< z, !upz> (_k));
-
-        sort<z, upz, upx, upy> (m0, m1);
-        sort<y, upy, upz, upx> (m1, m2);
-        sort<y, upy, upz, upx> (m2, m3);
-        sort<x, upx,!upy,!upz> (m3, m4);
-        sort<x, upx,!upy,!upz> (m4, m5);
-        sort<y,!upy, upz,!upx> (m5, m6);
-        sort<y,!upy, upz,!upx> (m6, m7);
-        sort<z,!upz,!upx, upy> (m7, m8);
+    void operator()() const
+    {
+      res = internal::hilbert_split(begin, end, comp);
     }
+  };
 
-    template <class RandomAccessIterator>
-    void operator() (RandomAccessIterator begin, RandomAccessIterator end) const
-    {
-        sort <0, false, false, false> (begin, end);
+  template <int x, bool upx, bool upy, bool upz, class RandomAccessIterator>
+  void sort (RandomAccessIterator begin, RandomAccessIterator end, Parallel_tag) const
+  {
+#ifndef CGAL_LINKED_WITH_TBB
+    CGAL_USE(begin);
+    CGAL_USE(end);
+    CGAL_static_assertion_msg (!(boost::is_convertible<ConcurrencyTag, Parallel_tag>::value),
+                               "Parallel_tag is enabled but TBB is unavailable.");
+#else
+    const int y = (x + 1) % 3, z = (x + 2) % 3;
+    if ((end - begin) <= _limit) return;
+
+    RandomAccessIterator m0 = begin, m8 = end;
+    if((end - begin) > 2048){ // 2^11, empirically a good cutoff
+      RandomAccessIterator m1, m2, m3, m4, m5, m6, m7;
+      m4 = internal::hilbert_split(m0, m8, Cmp<x, upx>(_k));
+
+      tbb::parallel_invoke(Hilbert_split<RandomAccessIterator,Cmp<y,  upy> >(m2, m0, m4, Cmp<y,  upy>(_k)),
+                           Hilbert_split<RandomAccessIterator,Cmp<y, !upy> >(m6, m4, m8, Cmp<y, !upy>(_k)));
+
+      tbb::parallel_invoke(Hilbert_split<RandomAccessIterator,Cmp<z,  upz> >(m1, m0, m2, Cmp<z,  upz>(_k)),
+                           Hilbert_split<RandomAccessIterator,Cmp<z, !upz> >(m3, m2, m4, Cmp<z, !upz>(_k)),
+                           Hilbert_split<RandomAccessIterator,Cmp<z,  upz> >(m5, m4, m6, Cmp<z,  upz>(_k)),
+                           Hilbert_split<RandomAccessIterator,Cmp<z, !upz> >(m7, m6, m8, Cmp<z, !upz>(_k)));
+
+      tbb::parallel_invoke(Recursive_sort<z,  upz,  upx,  upy, RandomAccessIterator>(*this, m0, m1),
+                           Recursive_sort<y,  upy,  upz,  upx, RandomAccessIterator>(*this, m1, m2),
+                           Recursive_sort<y,  upy,  upz,  upx, RandomAccessIterator>(*this, m2, m3),
+                           Recursive_sort<x,  upx, !upy, !upz, RandomAccessIterator>(*this, m3, m4),
+                           Recursive_sort<x,  upx, !upy, !upz, RandomAccessIterator>(*this, m4, m5),
+                           Recursive_sort<y, !upy,  upz, !upx, RandomAccessIterator>(*this, m5, m6),
+                           Recursive_sort<y, !upy,  upz, !upx, RandomAccessIterator>(*this, m6, m7),
+                           Recursive_sort<z, !upz, !upx,  upy, RandomAccessIterator>(*this, m7, m8));
+    } else {
+      recursive_sort<0, false, false, false>(begin, end);
     }
+#endif
+  }
+
+  template <int x, bool upx, bool upy, bool upz, class RandomAccessIterator>
+  void sort (RandomAccessIterator begin, RandomAccessIterator end, Sequential_tag) const
+  {
+    recursive_sort<0, false, false, false>(begin, end);
+  }
+
+  template <class RandomAccessIterator>
+  void operator() (RandomAccessIterator begin, RandomAccessIterator end) const
+  {
+    sort <0, false, false, false> (begin, end, ConcurrencyTag());
+  }
 };
 
 } // namespace CGAL
diff -pruN 5.0.3-1/include/CGAL/Hilbert_sort_median_d.h 5.1-1/include/CGAL/Hilbert_sort_median_d.h
--- 5.0.3-1/include/CGAL/Hilbert_sort_median_d.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Hilbert_sort_median_d.h	2020-09-08 12:15:40.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_sorting/include/CGAL/Hilbert_sort_median_d.h $
-// $Id: Hilbert_sort_median_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_sorting/include/CGAL/Hilbert_sort_median_d.h $
+// $Id: Hilbert_sort_median_d.h 5c41b10 2020-01-02T10:26:44+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     :  Olivier Devillers
@@ -23,137 +23,138 @@ namespace CGAL {
 
 namespace internal {
 
-    template <class K>
-    struct Hilbert_cmp_d
-        : public CGAL::cpp98::binary_function<typename K::Point_d,
-                                              typename K::Point_d, bool>
-    {
-        typedef typename K::Point_d Point;
-        K k;
-        int axe;
-        bool orient;
-        Hilbert_cmp_d (int a, bool o, const K &_k = K())
-          : k(_k), axe(a),  orient(o) {}
-        bool operator() (const Point &p, const Point &q) const
-        {
-          return (orient  ? (k.less_coordinate_d_object() (q,p,axe) )
-                          : (k.less_coordinate_d_object() (p,q,axe) ));
-        }
-    };
+template <class K>
+struct Hilbert_cmp_d
+    : public CGAL::cpp98::binary_function<typename K::Point_d,
+    typename K::Point_d, bool>
+{
+  typedef typename K::Point_d Point;
+  K k;
+  int axe;
+  bool orient;
+  Hilbert_cmp_d (int a, bool o, const K &_k = K()) : k(_k), axe(a),  orient(o) {}
+
+  bool operator() (const Point &p, const Point &q) const
+  {
+    return (orient  ? (k.less_coordinate_d_object() (q,p,axe) )
+                    : (k.less_coordinate_d_object() (p,q,axe) ));
+  }
+};
 
-}
+} // namespace internal
 
 template <class K>
 class Hilbert_sort_median_d
 {
 public:
-    typedef K Kernel;
-    typedef typename Kernel::Point_d Point;
-    typedef std::vector< bool > Starting_position;
+  typedef K Kernel;
+  typedef typename Kernel::Point_d Point;
+  typedef std::vector< bool > Starting_position;
 
 private:
-    Kernel _k;
-    std::ptrdiff_t _limit;
-    mutable int _dimension;
-    mutable int two_to_dim;
-
-    struct Cmp : public internal::Hilbert_cmp_d<Kernel>
-    { Cmp (int a, bool dir, const Kernel &k)
-          : internal::Hilbert_cmp_d<Kernel> (a,dir,k) {} };
+  Kernel _k;
+  std::ptrdiff_t _limit;
+  mutable int _dimension;
+  mutable int two_to_dim;
+
+  struct Cmp
+    : public internal::Hilbert_cmp_d<Kernel>
+  {
+    Cmp (int a, bool dir, const Kernel &k) : internal::Hilbert_cmp_d<Kernel> (a,dir,k) {}
+  };
 
 public:
-    Hilbert_sort_median_d(const Kernel &k = Kernel(), std::ptrdiff_t limit = 1)
-        : _k(k), _limit (limit)
-    {}
-
-    template <class RandomAccessIterator>
-    void sort (RandomAccessIterator begin, RandomAccessIterator end,
-               Starting_position start, int direction) const
-   {
-     if (end - begin <= _limit) return;
-
-     int nb_directions = _dimension;
-     int nb_splits     = two_to_dim;
-
-     if ( (end-begin) < (two_to_dim/2) ) { // not many points
-       nb_splits = 1;
-       nb_directions = 0;
-       while ( (end-begin) > nb_splits) {
-         ++nb_directions;
-         nb_splits *= 2;        // compute 2^nb_directions
-       }
-     }
-
-     std::vector<RandomAccessIterator> places(nb_splits +1);
-     std::vector<int>                  dir   (nb_splits +1);
-     places[0]=begin;
-     places[nb_splits]=end;
-
-     int last_dir = (direction + nb_directions) % _dimension;
-     int current_dir = direction;
-     int current_level_step =nb_splits;
-     do{
-       int half_step = current_level_step/2;
-       int left=0;
-       int middle = half_step;
-       int right=current_level_step;
-       bool orient = start[current_dir];
-       do{
-         dir[middle]    = current_dir;
-         places[middle] = internal::hilbert_split
-           (places[left], places[right], Cmp (current_dir,orient,_k));
-         left =right;
-         right+=current_level_step;
-         middle+=current_level_step;
-         orient = ! orient;
-       }while( left< nb_splits);
-       current_level_step = half_step;
-       current_dir = (current_dir +1) % _dimension;
-     }while (current_dir != last_dir);
-
-     if ( end-begin < two_to_dim) return; // less than 2^dim points
-
-     /////////////start recursive calls
-     last_dir = (direction + _dimension -1) % _dimension;
-     // first step is special
-     sort( places[0], places[1], start, last_dir);
-
-     for(int i=1; i<two_to_dim-1; i +=2){
-       sort( places[i  ], places[i+1], start, dir[i+1]);
-       sort( places[i+1], places[i+2], start, dir[i+1]);
-       start[dir[i+1]] = !  start[dir[i+1]];
-       start[last_dir] = !  start[last_dir];
-     }
-
-     //last step is special
-     sort( places[two_to_dim-1], places[two_to_dim], start, last_dir);
-    }
-
-
-    template <class RandomAccessIterator>
-    void operator() (RandomAccessIterator begin, RandomAccessIterator end) const
-    {
-      _dimension = _k.point_dimension_d_object()(*begin);
-      two_to_dim = 1;
-      Starting_position start(_dimension);
-
-      typename std::iterator_traits<RandomAccessIterator>::difference_type N=end-begin;
-      N*=2;
-      for (int i=0; i<_dimension; ++i)         start[i]=false;         // we start below in all coordinates
-      for (int i=0; i<_dimension; ++i) {
-        two_to_dim *= 2;        // compute 2^_dimension
-        N/=2;
-        if (N==0) break;  // not many points, this number of dimension is enough
+  Hilbert_sort_median_d(const Kernel &k = Kernel(), std::ptrdiff_t limit = 1)
+    : _k(k), _limit (limit)
+  {}
+
+  template <class RandomAccessIterator>
+  void sort (RandomAccessIterator begin, RandomAccessIterator end,
+             Starting_position start, int direction) const
+  {
+    if (end - begin <= _limit)
+      return;
+
+    int nb_directions = _dimension;
+    int nb_splits     = two_to_dim;
+
+    if ( (end-begin) < (two_to_dim/2) ) { // not many points
+      nb_splits = 1;
+      nb_directions = 0;
+      while ( (end-begin) > nb_splits) {
+        ++nb_directions;
+        nb_splits *= 2;        // compute 2^nb_directions
       }
-
-
-      // we start with  direction 0;
-      sort (begin, end, start, 0);
     }
-};
 
+    std::vector<RandomAccessIterator> places(nb_splits +1);
+    std::vector<int>                  dir   (nb_splits +1);
+    places[0]=begin;
+    places[nb_splits]=end;
+
+    int last_dir = (direction + nb_directions) % _dimension;
+    int current_dir = direction;
+    int current_level_step =nb_splits;
+    do{
+      int half_step = current_level_step/2;
+      int left=0;
+      int middle = half_step;
+      int right=current_level_step;
+      bool orient = start[current_dir];
+      do{
+        dir[middle]    = current_dir;
+        places[middle] = internal::hilbert_split
+                         (places[left], places[right], Cmp (current_dir,orient,_k));
+        left =right;
+        right+=current_level_step;
+        middle+=current_level_step;
+        orient = ! orient;
+      }while( left< nb_splits);
+      current_level_step = half_step;
+      current_dir = (current_dir +1) % _dimension;
+    }while (current_dir != last_dir);
+
+    if ( end-begin < two_to_dim) return; // less than 2^dim points
+
+    /////////////start recursive calls
+    last_dir = (direction + _dimension -1) % _dimension;
+    // first step is special
+    sort( places[0], places[1], start, last_dir);
+
+    for(int i=1; i<two_to_dim-1; i +=2){
+      sort( places[i  ], places[i+1], start, dir[i+1]);
+      sort( places[i+1], places[i+2], start, dir[i+1]);
+      start[dir[i+1]] = !  start[dir[i+1]];
+      start[last_dir] = !  start[last_dir];
+    }
 
+    //last step is special
+    sort( places[two_to_dim-1], places[two_to_dim], start, last_dir);
+  }
+
+  template <class RandomAccessIterator>
+  void operator() (RandomAccessIterator begin, RandomAccessIterator end) const
+  {
+    _dimension = _k.point_dimension_d_object()(*begin);
+    two_to_dim = 1;
+    Starting_position start(_dimension);
+
+    typename std::iterator_traits<RandomAccessIterator>::difference_type N=end-begin;
+    N*=2;
+    for (int i=0; i<_dimension; ++i)
+      start[i]=false; // we start below in all coordinates
+
+    for (int i=0; i<_dimension; ++i) {
+      two_to_dim *= 2;        // compute 2^_dimension
+      N/=2;
+      if (N==0)
+        break;  // not many points, this number of dimension is enough
+    }
 
+    // we start with  direction 0;
+    sort (begin, end, start, 0);
+  }
+};
 
 } // namespace CGAL
 
diff -pruN 5.0.3-1/include/CGAL/Hilbert_sort_middle_2.h 5.1-1/include/CGAL/Hilbert_sort_middle_2.h
--- 5.0.3-1/include/CGAL/Hilbert_sort_middle_2.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Hilbert_sort_middle_2.h	2020-09-08 12:15:40.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_sorting/include/CGAL/Hilbert_sort_middle_2.h $
-// $Id: Hilbert_sort_middle_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_sorting/include/CGAL/Hilbert_sort_middle_2.h $
+// $Id: Hilbert_sort_middle_2.h 5c41b10 2020-01-02T10:26:44+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     :  Olivier Devillers
@@ -21,130 +21,130 @@
 namespace CGAL {
 
 namespace internal {
-    template <class K, int x, bool up> struct Fixed_hilbert_cmp_2;
+template <class K, int x, bool up> struct Fixed_hilbert_cmp_2;
 
-    template <class K, int x>
-    struct Fixed_hilbert_cmp_2<K,x,true>
-        : public CGAL::cpp98::binary_function<typename K::Point_2,
-                                              typename K::Point_2, bool>
-    {
-        typedef typename K::Point_2 Point;
-        K k;
-        double value;
-        Fixed_hilbert_cmp_2 (double v, const K &_k = K()) : k(_k),value(v) {}
-        bool operator() (const Point &p) const
-        {
-          return ! Fixed_hilbert_cmp_2<K,x,false> (value, k) (p);
-        }
-    };
-
-    template <class K>
-    struct Fixed_hilbert_cmp_2<K,0,false>
-        : public CGAL::cpp98::binary_function<typename K::Point_2,
-                                              typename K::Point_2, bool>
-    {
-        typedef typename K::Point_2 Point;
-        K k;
-        double value;
-        Fixed_hilbert_cmp_2 (double v, const K &_k = K()) : k(_k),value(v) {}
-        bool operator() (const Point &p) const
-        {
-          return to_double(k.compute_x_2_object()(p)) < value;
-        }
-    };
-
-    template <class K>
-    struct Fixed_hilbert_cmp_2<K,1,false>
-        : public CGAL::cpp98::binary_function<typename K::Point_2,
-                                              typename K::Point_2, bool>
-    {
-        typedef typename K::Point_2 Point;
-        K k;
-        double value;
-        Fixed_hilbert_cmp_2 (double v, const K &_k = K()) : k(_k),value(v) {}
-        bool operator() (const Point &p) const
-        {
-          return to_double(k.compute_y_2_object()(p)) < value;
-        }
-    };
-}
+template <class K, int x>
+struct Fixed_hilbert_cmp_2<K,x,true>
+    : public CGAL::cpp98::binary_function<typename K::Point_2,
+    typename K::Point_2, bool>
+{
+  typedef typename K::Point_2 Point;
+  K k;
+  double value;
+  Fixed_hilbert_cmp_2 (double v, const K &_k = K()) : k(_k),value(v) {}
+  bool operator() (const Point &p) const
+  {
+    return ! Fixed_hilbert_cmp_2<K,x,false> (value, k) (p);
+  }
+};
 
+template <class K>
+struct Fixed_hilbert_cmp_2<K,0,false>
+    : public CGAL::cpp98::binary_function<typename K::Point_2,
+    typename K::Point_2, bool>
+{
+  typedef typename K::Point_2 Point;
+  K k;
+  double value;
+  Fixed_hilbert_cmp_2 (double v, const K &_k = K()) : k(_k),value(v) {}
+  bool operator() (const Point &p) const
+  {
+    return to_double(k.compute_x_2_object()(p)) < value;
+  }
+};
+
+template <class K>
+struct Fixed_hilbert_cmp_2<K,1,false>
+    : public CGAL::cpp98::binary_function<typename K::Point_2,
+    typename K::Point_2, bool>
+{
+  typedef typename K::Point_2 Point;
+  K k;
+  double value;
+  Fixed_hilbert_cmp_2 (double v, const K &_k = K()) : k(_k),value(v) {}
+  bool operator() (const Point &p) const
+  {
+    return to_double(k.compute_y_2_object()(p)) < value;
+  }
+};
+
+} // namespace internal
 
 template <class K>
 class Hilbert_sort_middle_2
 {
 public:
-    typedef K Kernel;
-    typedef typename Kernel::Point_2 Point;
+  typedef K Kernel;
+  typedef typename Kernel::Point_2 Point;
 
 private:
-    Kernel _k;
-    std::ptrdiff_t _limit;
+  Kernel _k;
+  std::ptrdiff_t _limit;
 
-    template <int x, bool up> struct Cmp : public internal::Fixed_hilbert_cmp_2<Kernel,x,up>
-      { Cmp (double v, const Kernel &k) : internal::Fixed_hilbert_cmp_2<Kernel,x,up> (v, k) {} };
+  template <int x, bool up>
+  struct Cmp
+    : public internal::Fixed_hilbert_cmp_2<Kernel,x,up>
+  {
+    Cmp (double v, const Kernel &k) : internal::Fixed_hilbert_cmp_2<Kernel,x,up> (v, k) {}
+  };
 
 public:
-    Hilbert_sort_middle_2 (const Kernel &k = Kernel(), std::ptrdiff_t limit = 1)
-        : _k(k), _limit (limit)
-    {}
-
-    template <int x, bool upx, bool upy, class RandomAccessIterator>
-    void sort (RandomAccessIterator begin, RandomAccessIterator end,
-               double xmin, double ymin, double xmax, double ymax) const
-    {
-        const int y = (x + 1) % 2;
-        if (end - begin <= _limit) return;
-
-        double xmed= (xmin+xmax)/2;
-        double ymed= (ymin+ymax)/2;
-
-        RandomAccessIterator m0 = begin, m4 = end;
-
-        RandomAccessIterator m2 =
-          internal::fixed_hilbert_split (m0, m4, Cmp< x,  upx> (xmed,_k));
-        RandomAccessIterator m1 =
-          internal::fixed_hilbert_split (m0, m2, Cmp< y,  upy> (ymed,_k));
-        RandomAccessIterator m3 =
-          internal::fixed_hilbert_split (m2, m4, Cmp< y, !upy> (ymed,_k));
-
-        if (m1!=m4)
-          sort<y, upy, upx> (m0, m1, ymin, xmin, ymed, xmed);
-        if (m1!=m0 || m2!=m4)
-          sort<x, upx, upy> (m1, m2, xmin, ymed, xmed, ymax);
-        if (m2!=m0 || m3!=m4)
-          sort<x, upx, upy> (m2, m3, xmed, ymed, xmax, ymax);
-        if (m3!=m0)
-          sort<y,!upy,!upx> (m3, m4, ymed, xmax, ymin, xmed);
+  Hilbert_sort_middle_2 (const Kernel &k = Kernel(), std::ptrdiff_t limit = 1)
+    : _k(k), _limit (limit)
+  {}
+
+  template <int x, bool upx, bool upy, class RandomAccessIterator>
+  void sort (RandomAccessIterator begin, RandomAccessIterator end,
+             double xmin, double ymin, double xmax, double ymax) const
+  {
+    const int y = (x + 1) % 2;
+    if (end - begin <= _limit) return;
+
+    double xmed= (xmin+xmax)/2;
+    double ymed= (ymin+ymax)/2;
+
+    RandomAccessIterator m0 = begin, m4 = end;
+
+    RandomAccessIterator m2 = internal::fixed_hilbert_split (m0, m4, Cmp< x,  upx> (xmed,_k));
+    RandomAccessIterator m1 = internal::fixed_hilbert_split (m0, m2, Cmp< y,  upy> (ymed,_k));
+    RandomAccessIterator m3 = internal::fixed_hilbert_split (m2, m4, Cmp< y, !upy> (ymed,_k));
+
+    if (m1!=m4)
+      sort<y, upy, upx> (m0, m1, ymin, xmin, ymed, xmed);
+    if (m1!=m0 || m2!=m4)
+      sort<x, upx, upy> (m1, m2, xmin, ymed, xmed, ymax);
+    if (m2!=m0 || m3!=m4)
+      sort<x, upx, upy> (m2, m3, xmed, ymed, xmax, ymax);
+    if (m3!=m0)
+      sort<y,!upy,!upx> (m3, m4, ymed, xmax, ymin, xmed);
+  }
+
+  template <class RandomAccessIterator>
+  void operator() (RandomAccessIterator begin, RandomAccessIterator end) const
+  {
+    //Bbox_2 box=bbox_2(begin, end); BUG: WE NEED TO FIX THIS
+
+    K k;
+    double xmin=to_double(k.compute_x_2_object()(*begin)),
+           ymin=to_double(k.compute_y_2_object()(*begin)),
+           xmax=xmin,
+           ymax=ymin;
+
+    for(RandomAccessIterator it=begin+1; it<end; ++it){
+      if ( to_double(k.compute_x_2_object()(*it)) < xmin)
+        xmin = to_double(k.compute_x_2_object()(*it));
+      if ( to_double(k.compute_y_2_object()(*it)) < ymin)
+        ymin = to_double(k.compute_y_2_object()(*it));
+      if ( to_double(k.compute_x_2_object()(*it)) > xmax)
+        xmax = to_double(k.compute_x_2_object()(*it));
+      if ( to_double(k.compute_y_2_object()(*it)) > ymax)
+        ymax = to_double(k.compute_y_2_object()(*it));
     }
 
-    template <class RandomAccessIterator>
-    void operator() (RandomAccessIterator begin, RandomAccessIterator end) const
-    {
-      //Bbox_2 box=bbox_2(begin, end); BUG: WE NEED TO FIX THIS
-
-                K k;
-            double xmin=to_double(k.compute_x_2_object()(*begin)),
-                     ymin=to_double(k.compute_y_2_object()(*begin)),
-                       xmax=xmin,
-                     ymax=ymin;
-
-            for(RandomAccessIterator it=begin+1; it<end; ++it){
-                        if ( to_double(k.compute_x_2_object()(*it)) < xmin)
-                                  xmin = to_double(k.compute_x_2_object()(*it));
-                        if ( to_double(k.compute_y_2_object()(*it)) < ymin)
-                                  ymin = to_double(k.compute_y_2_object()(*it));
-                        if ( to_double(k.compute_x_2_object()(*it)) > xmax)
-                                  xmax = to_double(k.compute_x_2_object()(*it));
-                        if ( to_double(k.compute_y_2_object()(*it)) > ymax)
-                                  ymax = to_double(k.compute_y_2_object()(*it));
-            }
-
-        sort <0, false, false> (begin, end, xmin, ymin, xmax, ymax);
-    }
+    sort <0, false, false> (begin, end, xmin, ymin, xmax, ymax);
+  }
 };
 
-
 } // namespace CGAL
 
 #endif//CGAL_HILBERT_SORT_MIDDLE_2_H
diff -pruN 5.0.3-1/include/CGAL/Hilbert_sort_middle_3.h 5.1-1/include/CGAL/Hilbert_sort_middle_3.h
--- 5.0.3-1/include/CGAL/Hilbert_sort_middle_3.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Hilbert_sort_middle_3.h	2020-09-08 12:15:40.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_sorting/include/CGAL/Hilbert_sort_middle_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_sorting/include/CGAL/Hilbert_sort_middle_3.h $
 // $Id: Hilbert_sort_middle_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Hilbert_sort_middle_base.h 5.1-1/include/CGAL/Hilbert_sort_middle_base.h
--- 5.0.3-1/include/CGAL/Hilbert_sort_middle_base.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Hilbert_sort_middle_base.h	2020-09-08 12:15:40.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_sorting/include/CGAL/Hilbert_sort_middle_base.h $
-// $Id: Hilbert_sort_middle_base.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_sorting/include/CGAL/Hilbert_sort_middle_base.h $
+// $Id: Hilbert_sort_middle_base.h 5c41b10 2020-01-02T10:26:44+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     :  Olivier Devillers
@@ -19,17 +19,19 @@ namespace CGAL {
 
 namespace internal {
 
-    template <class RandomAccessIterator, class Cmp>
-    RandomAccessIterator
-    fixed_hilbert_split (RandomAccessIterator begin, RandomAccessIterator end,
-                   Cmp cmp = Cmp ())
-    {
-        if (begin >= end) return begin;
+template <class RandomAccessIterator, class Cmp>
+RandomAccessIterator
+fixed_hilbert_split (RandomAccessIterator begin, RandomAccessIterator end,
+                     Cmp cmp = Cmp ())
+{
+  if (begin >= end)
+    return begin;
 
-        return std::partition (begin, end, cmp);
-    }
+  return std::partition (begin, end, cmp);
 }
 
+} // namespace internal
+
 } // namespace CGAL
 
 #endif//CGAL_HILBERT_SORT_MIDDLE_BASE_H
diff -pruN 5.0.3-1/include/CGAL/Hilbert_sort_middle_d.h 5.1-1/include/CGAL/Hilbert_sort_middle_d.h
--- 5.0.3-1/include/CGAL/Hilbert_sort_middle_d.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Hilbert_sort_middle_d.h	2020-09-08 12:15:40.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_sorting/include/CGAL/Hilbert_sort_middle_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_sorting/include/CGAL/Hilbert_sort_middle_d.h $
 // $Id: Hilbert_sort_middle_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Hilbert_sort_on_sphere_3.h 5.1-1/include/CGAL/Hilbert_sort_on_sphere_3.h
--- 5.0.3-1/include/CGAL/Hilbert_sort_on_sphere_3.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Hilbert_sort_on_sphere_3.h	2020-09-08 12:15:40.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_sorting/include/CGAL/Hilbert_sort_on_sphere_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_sorting/include/CGAL/Hilbert_sort_on_sphere_3.h $
 // $Id: Hilbert_sort_on_sphere_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/hilbert_sort_on_sphere.h 5.1-1/include/CGAL/hilbert_sort_on_sphere.h
--- 5.0.3-1/include/CGAL/hilbert_sort_on_sphere.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/hilbert_sort_on_sphere.h	2020-09-08 12:15:40.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_sorting/include/CGAL/hilbert_sort_on_sphere.h $
-// $Id: hilbert_sort_on_sphere.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_sorting/include/CGAL/hilbert_sort_on_sphere.h $
+// $Id: hilbert_sort_on_sphere.h 5c41b10 2020-01-02T10:26:44+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Olivier Devillers
@@ -22,12 +22,12 @@ namespace internal {
 
 template <class RandomAccessIterator, class Kernel, class Policy>
 void hilbert_sort_on_sphere (RandomAccessIterator begin,
-                   RandomAccessIterator end,
-       const Kernel &k,
-                   Policy,
-                   typename Kernel::Point_3 *,
-                   double sq_r,
-                   const typename Kernel::Point_3 &p)
+                             RandomAccessIterator end,
+                             const Kernel &k,
+                             Policy,
+                             typename Kernel::Point_3 *,
+                             double sq_r,
+                             const typename Kernel::Point_3 &p)
 {
   typedef std::iterator_traits<RandomAccessIterator> ITraits;
   typedef typename ITraits::difference_type Diff_t;
@@ -41,62 +41,63 @@ void hilbert_sort_on_sphere (RandomAcces
 
 template <class RandomAccessIterator>
 void hilbert_sort_on_sphere (RandomAccessIterator begin, RandomAccessIterator end,
-  double sq_r = 1.0,
-  const typename CGAL::Kernel_traits<typename std::iterator_traits<RandomAccessIterator>::value_type>::Kernel::Point_3 &p =
-            typename CGAL::Kernel_traits<typename std::iterator_traits<RandomAccessIterator>::value_type>::Kernel::Point_3(0,0,0))
+                             double sq_r = 1.0,
+                             const typename CGAL::Kernel_traits<
+                               typename std::iterator_traits<RandomAccessIterator>::value_type>::Kernel::Point_3 &p =
+                             typename CGAL::Kernel_traits<
+                               typename std::iterator_traits<RandomAccessIterator>::value_type>::Kernel::Point_3(0,0,0))
 {
 
-    typedef std::iterator_traits<RandomAccessIterator> ITraits;
-    typedef typename ITraits::value_type               value_type;
-    typedef CGAL::Kernel_traits<value_type>            KTraits;
-    typedef typename KTraits::Kernel                   Kernel;
-
-    internal::hilbert_sort_on_sphere(begin, end, Kernel(), Hilbert_sort_median_policy(),
-                                  static_cast<value_type *> (0), sq_r, p);
+  typedef std::iterator_traits<RandomAccessIterator> ITraits;
+  typedef typename ITraits::value_type               value_type;
+  typedef CGAL::Kernel_traits<value_type>            KTraits;
+  typedef typename KTraits::Kernel                   Kernel;
 
+  internal::hilbert_sort_on_sphere(begin, end, Kernel(), Hilbert_sort_median_policy(), static_cast<value_type *> (0), sq_r, p);
 }
 
 template <class RandomAccessIterator>
 void hilbert_sort_on_sphere (RandomAccessIterator begin, RandomAccessIterator end, Hilbert_sort_median_policy policy,
-  double sq_r = 1.0,
-  const typename CGAL::Kernel_traits<typename std::iterator_traits<RandomAccessIterator>::value_type>::Kernel::Point_3 &p =
-    typename CGAL::Kernel_traits<typename std::iterator_traits<RandomAccessIterator>::value_type>::Kernel::Point_3(0,0,0))
+                             double sq_r = 1.0,
+                             const typename CGAL::Kernel_traits<
+                               typename std::iterator_traits<RandomAccessIterator>::value_type>::Kernel::Point_3 &p =
+                             typename CGAL::Kernel_traits<
+                               typename std::iterator_traits<RandomAccessIterator>::value_type>::Kernel::Point_3(0,0,0))
 {
-    typedef std::iterator_traits<RandomAccessIterator> ITraits;
-    typedef typename ITraits::value_type               value_type;
-    typedef CGAL::Kernel_traits<value_type>            KTraits;
-    typedef typename KTraits::Kernel                   Kernel;
+  typedef std::iterator_traits<RandomAccessIterator> ITraits;
+  typedef typename ITraits::value_type               value_type;
+  typedef CGAL::Kernel_traits<value_type>            KTraits;
+  typedef typename KTraits::Kernel                   Kernel;
 
-    internal::hilbert_sort_on_sphere(begin, end, Kernel(), policy,
-                                  static_cast<value_type *> (0), sq_r, p);
+  internal::hilbert_sort_on_sphere(begin, end, Kernel(), policy, static_cast<value_type *> (0), sq_r, p);
 }
 
-
 template <class RandomAccessIterator>
 void hilbert_sort_on_sphere (RandomAccessIterator begin, RandomAccessIterator end, Hilbert_sort_middle_policy policy,
-  double sq_r = 1.0,
-  const typename CGAL::Kernel_traits<typename std::iterator_traits<RandomAccessIterator>::value_type>::Kernel::Point_3 &p =
-    typename CGAL::Kernel_traits<typename std::iterator_traits<RandomAccessIterator>::value_type>::Kernel::Point_3(0,0,0))
+                             double sq_r = 1.0,
+                             const typename CGAL::Kernel_traits<
+                               typename std::iterator_traits<RandomAccessIterator>::value_type>::Kernel::Point_3 &p =
+                             typename CGAL::Kernel_traits<
+                               typename std::iterator_traits<RandomAccessIterator>::value_type>::Kernel::Point_3(0,0,0))
 {
-    typedef std::iterator_traits<RandomAccessIterator> ITraits;
-    typedef typename ITraits::value_type               value_type;
-    typedef CGAL::Kernel_traits<value_type>            KTraits;
-    typedef typename KTraits::Kernel                   Kernel;
+  typedef std::iterator_traits<RandomAccessIterator> ITraits;
+  typedef typename ITraits::value_type               value_type;
+  typedef CGAL::Kernel_traits<value_type>            KTraits;
+  typedef typename KTraits::Kernel                   Kernel;
 
-    internal::hilbert_sort_on_sphere(begin, end, Kernel(), policy,
-                                  static_cast<value_type *> (0), sq_r, p);
+  internal::hilbert_sort_on_sphere(begin, end, Kernel(), policy, static_cast<value_type *> (0), sq_r, p);
 }
 
-
 template <class RandomAccessIterator, class Kernel, class Policy>
-void hilbert_sort_on_sphere (RandomAccessIterator begin, RandomAccessIterator end, const Kernel &k, Policy policy,
-        double sq_r = 1.0, const typename Kernel::Point_3 &p = typename Kernel::Point_3(0,0,0))
+void hilbert_sort_on_sphere (RandomAccessIterator begin, RandomAccessIterator end,
+                             const Kernel &k, Policy policy,
+                             double sq_r = 1.0,
+                             const typename Kernel::Point_3 &p = typename Kernel::Point_3(0,0,0))
 {
-    typedef std::iterator_traits<RandomAccessIterator> ITraits;
-    typedef typename ITraits::value_type               value_type;
+  typedef std::iterator_traits<RandomAccessIterator> ITraits;
+  typedef typename ITraits::value_type               value_type;
 
-    internal::hilbert_sort_on_sphere(begin, end,
-                           k, policy, static_cast<value_type *> (0), sq_r, p);
+  internal::hilbert_sort_on_sphere(begin, end, k, policy, static_cast<value_type *> (0), sq_r, p);
 }
 
 } // end of namespace CGAL
diff -pruN 5.0.3-1/include/CGAL/Homogeneous/Aff_transformationH2.h 5.1-1/include/CGAL/Homogeneous/Aff_transformationH2.h
--- 5.0.3-1/include/CGAL/Homogeneous/Aff_transformationH2.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Homogeneous/Aff_transformationH2.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Homogeneous_kernel/include/CGAL/Homogeneous/Aff_transformationH2.h $
-// $Id: Aff_transformationH2.h 4c30c49 2020-05-18T12:22:27+02:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Homogeneous_kernel/include/CGAL/Homogeneous/Aff_transformationH2.h $
+// $Id: Aff_transformationH2.h 9a3e038 2020-05-18T12:24:25+02:00 Laurent Rineau
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
diff -pruN 5.0.3-1/include/CGAL/Homogeneous/Aff_transformationH3.h 5.1-1/include/CGAL/Homogeneous/Aff_transformationH3.h
--- 5.0.3-1/include/CGAL/Homogeneous/Aff_transformationH3.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Homogeneous/Aff_transformationH3.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Homogeneous_kernel/include/CGAL/Homogeneous/Aff_transformationH3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Homogeneous_kernel/include/CGAL/Homogeneous/Aff_transformationH3.h $
 // $Id: Aff_transformationH3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Homogeneous/basic_constructionsH2.h 5.1-1/include/CGAL/Homogeneous/basic_constructionsH2.h
--- 5.0.3-1/include/CGAL/Homogeneous/basic_constructionsH2.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Homogeneous/basic_constructionsH2.h	2020-09-08 12:14:54.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Homogeneous_kernel/include/CGAL/Homogeneous/basic_constructionsH2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Homogeneous_kernel/include/CGAL/Homogeneous/basic_constructionsH2.h $
 // $Id: basic_constructionsH2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Homogeneous/basic_constructionsH3.h 5.1-1/include/CGAL/Homogeneous/basic_constructionsH3.h
--- 5.0.3-1/include/CGAL/Homogeneous/basic_constructionsH3.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Homogeneous/basic_constructionsH3.h	2020-09-08 12:14:54.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Homogeneous_kernel/include/CGAL/Homogeneous/basic_constructionsH3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Homogeneous_kernel/include/CGAL/Homogeneous/basic_constructionsH3.h $
 // $Id: basic_constructionsH3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Homogeneous/CircleH2.h 5.1-1/include/CGAL/Homogeneous/CircleH2.h
--- 5.0.3-1/include/CGAL/Homogeneous/CircleH2.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Homogeneous/CircleH2.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Homogeneous_kernel/include/CGAL/Homogeneous/CircleH2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Homogeneous_kernel/include/CGAL/Homogeneous/CircleH2.h $
 // $Id: CircleH2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Homogeneous/ConicHPA2.h 5.1-1/include/CGAL/Homogeneous/ConicHPA2.h
--- 5.0.3-1/include/CGAL/Homogeneous/ConicHPA2.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Homogeneous/ConicHPA2.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Homogeneous_kernel/include/CGAL/Homogeneous/ConicHPA2.h $
-// $Id: ConicHPA2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Homogeneous_kernel/include/CGAL/Homogeneous/ConicHPA2.h $
+// $Id: ConicHPA2.h 29b2957 2020-04-06T21:46:00+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -310,13 +310,8 @@ class ConicHPA2
     PT center () const
     {
         CGAL_kernel_precondition (type != PARABOLA);
-        // PT p;
-        // replaced previous line by following hack (no idea
-        // why original version doesn't work)
-        typename DA::Point_2 p;
         RT two = RT(2);
-        dao.set( p, two*s()*u() - t()*v(), two*r()*v() - t()*u(), -det());
-        return p;
+        return PT(two*s()*u() - t()*v(), two*r()*v() - t()*u(), -det());
     }
 
     Conic_type conic_type () const
diff -pruN 5.0.3-1/include/CGAL/Homogeneous/Data_accessorH2.h 5.1-1/include/CGAL/Homogeneous/Data_accessorH2.h
--- 5.0.3-1/include/CGAL/Homogeneous/Data_accessorH2.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Homogeneous/Data_accessorH2.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Homogeneous_kernel/include/CGAL/Homogeneous/Data_accessorH2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Homogeneous_kernel/include/CGAL/Homogeneous/Data_accessorH2.h $
 // $Id: Data_accessorH2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Homogeneous/DirectionH2.h 5.1-1/include/CGAL/Homogeneous/DirectionH2.h
--- 5.0.3-1/include/CGAL/Homogeneous/DirectionH2.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Homogeneous/DirectionH2.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Homogeneous_kernel/include/CGAL/Homogeneous/DirectionH2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Homogeneous_kernel/include/CGAL/Homogeneous/DirectionH2.h $
 // $Id: DirectionH2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Homogeneous/DirectionH3.h 5.1-1/include/CGAL/Homogeneous/DirectionH3.h
--- 5.0.3-1/include/CGAL/Homogeneous/DirectionH3.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Homogeneous/DirectionH3.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Homogeneous_kernel/include/CGAL/Homogeneous/DirectionH3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Homogeneous_kernel/include/CGAL/Homogeneous/DirectionH3.h $
 // $Id: DirectionH3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Homogeneous/distance_predicatesH2.h 5.1-1/include/CGAL/Homogeneous/distance_predicatesH2.h
--- 5.0.3-1/include/CGAL/Homogeneous/distance_predicatesH2.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Homogeneous/distance_predicatesH2.h	2020-09-08 12:14:54.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Homogeneous_kernel/include/CGAL/Homogeneous/distance_predicatesH2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Homogeneous_kernel/include/CGAL/Homogeneous/distance_predicatesH2.h $
 // $Id: distance_predicatesH2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Homogeneous/distance_predicatesH3.h 5.1-1/include/CGAL/Homogeneous/distance_predicatesH3.h
--- 5.0.3-1/include/CGAL/Homogeneous/distance_predicatesH3.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Homogeneous/distance_predicatesH3.h	2020-09-08 12:14:54.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Homogeneous_kernel/include/CGAL/Homogeneous/distance_predicatesH3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Homogeneous_kernel/include/CGAL/Homogeneous/distance_predicatesH3.h $
 // $Id: distance_predicatesH3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Homogeneous/function_objects.h 5.1-1/include/CGAL/Homogeneous/function_objects.h
--- 5.0.3-1/include/CGAL/Homogeneous/function_objects.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Homogeneous/function_objects.h	2020-09-08 12:14:54.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Homogeneous_kernel/include/CGAL/Homogeneous/function_objects.h $
-// $Id: function_objects.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Homogeneous_kernel/include/CGAL/Homogeneous/function_objects.h $
+// $Id: function_objects.h 4527b1f 2020-03-26T19:01:49+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -762,6 +762,52 @@ namespace HomogeneousKernelFunctors {
   };
 
   template <typename K>
+  class Compare_signed_distance_to_line_2
+  {
+    typedef typename K::Point_2   Point_2;
+    typedef typename K::Line_2    Line_2;
+    typedef typename K::Less_signed_distance_to_line_2 Less_signed_distance_to_line_2;
+
+  public:
+    typedef Comparison_result   result_type;
+
+    result_type
+    operator()(const Point_2& p, const Point_2& q,
+               const Point_2& r, const Point_2& s) const
+    {
+      typedef typename K::RT RT;
+
+      const RT & phx = p.hx();
+      const RT & phy = p.hy();
+      const RT & phw = p.hw();
+      const RT & qhx = q.hx();
+      const RT & qhy = q.hy();
+      const RT & qhw = q.hw();
+      const RT & rhx = r.hx();
+      const RT & rhy = r.hy();
+      const RT & rhw = r.hw();
+      const RT & shx = s.hx();
+      const RT & shy = s.hy();
+      const RT & shw = s.hw();
+
+      RT  scaled_dist_r_minus_scaled_dist_s =
+          ( rhx*shw - shx*rhw ) * (phy*qhw - qhy*phw)
+          - ( rhy*shw - shy*rhw ) * (phx*qhw - qhx*phw);
+
+      return compare(scaled_dist_r_minus_scaled_dist_s, 0);
+    }
+
+    result_type
+    operator()(const Line_2& l, const Point_2& p, const Point_2& q) const
+    {
+      Less_signed_distance_to_line_2 less = K().less_signed_distance_to_line_2_object();
+      if (less(l, p, q)) return SMALLER;
+      if (less(l, q, p)) return LARGER;
+      return EQUAL;
+    }
+  };
+
+  template <typename K>
   class Compare_slope_2
   {
     typedef typename K::Line_2             Line_2;
@@ -4109,26 +4155,7 @@ namespace HomogeneousKernelFunctors {
     operator()(const Point_2& p, const Point_2& q,
                const Point_2& r, const Point_2& s) const
     {
-      typedef typename K::RT RT;
-
-      const RT & phx= p.hx();
-      const RT & phy= p.hy();
-      const RT & phw= p.hw();
-      const RT & qhx= q.hx();
-      const RT & qhy= q.hy();
-      const RT & qhw= q.hw();
-      const RT & rhx= r.hx();
-      const RT & rhy= r.hy();
-      const RT & rhw= r.hw();
-      const RT & shx= s.hx();
-      const RT & shy= s.hy();
-      const RT & shw= s.hw();
-
-      RT  scaled_dist_r_minus_scaled_dist_s =
-        ( rhx*shw - shx*rhw ) * (phy*qhw - qhy*phw)
-        - ( rhy*shw - shy*rhw ) * (phx*qhw - qhx*phw);
-
-      return scaled_dist_r_minus_scaled_dist_s < 0;
+      return Compare_signed_distance_to_line_2<K>().operator()(p, q, r, s) == SMALLER;
     }
 
     result_type
diff -pruN 5.0.3-1/include/CGAL/Homogeneous/Homogeneous_base.h 5.1-1/include/CGAL/Homogeneous/Homogeneous_base.h
--- 5.0.3-1/include/CGAL/Homogeneous/Homogeneous_base.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Homogeneous/Homogeneous_base.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Homogeneous_kernel/include/CGAL/Homogeneous/Homogeneous_base.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Homogeneous_kernel/include/CGAL/Homogeneous/Homogeneous_base.h $
 // $Id: Homogeneous_base.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Homogeneous/Iso_cuboidH3.h 5.1-1/include/CGAL/Homogeneous/Iso_cuboidH3.h
--- 5.0.3-1/include/CGAL/Homogeneous/Iso_cuboidH3.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Homogeneous/Iso_cuboidH3.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Homogeneous_kernel/include/CGAL/Homogeneous/Iso_cuboidH3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Homogeneous_kernel/include/CGAL/Homogeneous/Iso_cuboidH3.h $
 // $Id: Iso_cuboidH3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Homogeneous/Iso_rectangleH2.h 5.1-1/include/CGAL/Homogeneous/Iso_rectangleH2.h
--- 5.0.3-1/include/CGAL/Homogeneous/Iso_rectangleH2.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Homogeneous/Iso_rectangleH2.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Homogeneous_kernel/include/CGAL/Homogeneous/Iso_rectangleH2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Homogeneous_kernel/include/CGAL/Homogeneous/Iso_rectangleH2.h $
 // $Id: Iso_rectangleH2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Homogeneous/LineH2.h 5.1-1/include/CGAL/Homogeneous/LineH2.h
--- 5.0.3-1/include/CGAL/Homogeneous/LineH2.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Homogeneous/LineH2.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Homogeneous_kernel/include/CGAL/Homogeneous/LineH2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Homogeneous_kernel/include/CGAL/Homogeneous/LineH2.h $
 // $Id: LineH2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Homogeneous/PlaneH3.h 5.1-1/include/CGAL/Homogeneous/PlaneH3.h
--- 5.0.3-1/include/CGAL/Homogeneous/PlaneH3.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Homogeneous/PlaneH3.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Homogeneous_kernel/include/CGAL/Homogeneous/PlaneH3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Homogeneous_kernel/include/CGAL/Homogeneous/PlaneH3.h $
 // $Id: PlaneH3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Homogeneous/PointH2.h 5.1-1/include/CGAL/Homogeneous/PointH2.h
--- 5.0.3-1/include/CGAL/Homogeneous/PointH2.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Homogeneous/PointH2.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Homogeneous_kernel/include/CGAL/Homogeneous/PointH2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Homogeneous_kernel/include/CGAL/Homogeneous/PointH2.h $
 // $Id: PointH2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Homogeneous/PointH3.h 5.1-1/include/CGAL/Homogeneous/PointH3.h
--- 5.0.3-1/include/CGAL/Homogeneous/PointH3.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Homogeneous/PointH3.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Homogeneous_kernel/include/CGAL/Homogeneous/PointH3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Homogeneous_kernel/include/CGAL/Homogeneous/PointH3.h $
 // $Id: PointH3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Homogeneous/predicates_on_directionsH2.h 5.1-1/include/CGAL/Homogeneous/predicates_on_directionsH2.h
--- 5.0.3-1/include/CGAL/Homogeneous/predicates_on_directionsH2.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Homogeneous/predicates_on_directionsH2.h	2020-09-08 12:14:54.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Homogeneous_kernel/include/CGAL/Homogeneous/predicates_on_directionsH2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Homogeneous_kernel/include/CGAL/Homogeneous/predicates_on_directionsH2.h $
 // $Id: predicates_on_directionsH2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Homogeneous/predicates_on_pointsH2.h 5.1-1/include/CGAL/Homogeneous/predicates_on_pointsH2.h
--- 5.0.3-1/include/CGAL/Homogeneous/predicates_on_pointsH2.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Homogeneous/predicates_on_pointsH2.h	2020-09-08 12:14:54.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Homogeneous_kernel/include/CGAL/Homogeneous/predicates_on_pointsH2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Homogeneous_kernel/include/CGAL/Homogeneous/predicates_on_pointsH2.h $
 // $Id: predicates_on_pointsH2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Homogeneous/predicates_on_pointsH3.h 5.1-1/include/CGAL/Homogeneous/predicates_on_pointsH3.h
--- 5.0.3-1/include/CGAL/Homogeneous/predicates_on_pointsH3.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Homogeneous/predicates_on_pointsH3.h	2020-09-08 12:14:54.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Homogeneous_kernel/include/CGAL/Homogeneous/predicates_on_pointsH3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Homogeneous_kernel/include/CGAL/Homogeneous/predicates_on_pointsH3.h $
 // $Id: predicates_on_pointsH3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Homogeneous/RayH3.h 5.1-1/include/CGAL/Homogeneous/RayH3.h
--- 5.0.3-1/include/CGAL/Homogeneous/RayH3.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Homogeneous/RayH3.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Homogeneous_kernel/include/CGAL/Homogeneous/RayH3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Homogeneous_kernel/include/CGAL/Homogeneous/RayH3.h $
 // $Id: RayH3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Homogeneous/SphereH3.h 5.1-1/include/CGAL/Homogeneous/SphereH3.h
--- 5.0.3-1/include/CGAL/Homogeneous/SphereH3.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Homogeneous/SphereH3.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Homogeneous_kernel/include/CGAL/Homogeneous/SphereH3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Homogeneous_kernel/include/CGAL/Homogeneous/SphereH3.h $
 // $Id: SphereH3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Homogeneous/VectorH2.h 5.1-1/include/CGAL/Homogeneous/VectorH2.h
--- 5.0.3-1/include/CGAL/Homogeneous/VectorH2.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Homogeneous/VectorH2.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Homogeneous_kernel/include/CGAL/Homogeneous/VectorH2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Homogeneous_kernel/include/CGAL/Homogeneous/VectorH2.h $
 // $Id: VectorH2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Homogeneous/VectorH3.h 5.1-1/include/CGAL/Homogeneous/VectorH3.h
--- 5.0.3-1/include/CGAL/Homogeneous/VectorH3.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Homogeneous/VectorH3.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Homogeneous_kernel/include/CGAL/Homogeneous/VectorH3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Homogeneous_kernel/include/CGAL/Homogeneous/VectorH3.h $
 // $Id: VectorH3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Homogeneous/Weighted_point_2.h 5.1-1/include/CGAL/Homogeneous/Weighted_point_2.h
--- 5.0.3-1/include/CGAL/Homogeneous/Weighted_point_2.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Homogeneous/Weighted_point_2.h	2020-09-08 12:14:54.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Homogeneous_kernel/include/CGAL/Homogeneous/Weighted_point_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Homogeneous_kernel/include/CGAL/Homogeneous/Weighted_point_2.h $
 // $Id: Weighted_point_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Homogeneous/Weighted_point_3.h 5.1-1/include/CGAL/Homogeneous/Weighted_point_3.h
--- 5.0.3-1/include/CGAL/Homogeneous/Weighted_point_3.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Homogeneous/Weighted_point_3.h	2020-09-08 12:14:54.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Homogeneous_kernel/include/CGAL/Homogeneous/Weighted_point_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Homogeneous_kernel/include/CGAL/Homogeneous/Weighted_point_3.h $
 // $Id: Weighted_point_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Homogeneous_converter.h 5.1-1/include/CGAL/Homogeneous_converter.h
--- 5.0.3-1/include/CGAL/Homogeneous_converter.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Homogeneous_converter.h	2020-09-08 12:14:54.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Homogeneous_kernel/include/CGAL/Homogeneous_converter.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Homogeneous_kernel/include/CGAL/Homogeneous_converter.h $
 // $Id: Homogeneous_converter.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Homogeneous_d.h 5.1-1/include/CGAL/Homogeneous_d.h
--- 5.0.3-1/include/CGAL/Homogeneous_d.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Homogeneous_d.h	2020-09-08 12:15:03.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Homogeneous_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Homogeneous_d.h $
 // $Id: Homogeneous_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Homogeneous.h 5.1-1/include/CGAL/Homogeneous.h
--- 5.0.3-1/include/CGAL/Homogeneous.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Homogeneous.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Homogeneous_kernel/include/CGAL/Homogeneous.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Homogeneous_kernel/include/CGAL/Homogeneous.h $
 // $Id: Homogeneous.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Hyperbola_2.h 5.1-1/include/CGAL/Hyperbola_2.h
--- 5.0.3-1/include/CGAL/Hyperbola_2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Hyperbola_2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Hyperbola_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Hyperbola_2.h $
 // $Id: Hyperbola_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Hyperbola_ray_2.h 5.1-1/include/CGAL/Hyperbola_ray_2.h
--- 5.0.3-1/include/CGAL/Hyperbola_ray_2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Hyperbola_ray_2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Hyperbola_ray_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Hyperbola_ray_2.h $
 // $Id: Hyperbola_ray_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Hyperbola_segment_2.h 5.1-1/include/CGAL/Hyperbola_segment_2.h
--- 5.0.3-1/include/CGAL/Hyperbola_segment_2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Hyperbola_segment_2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Hyperbola_segment_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Hyperbola_segment_2.h $
 // $Id: Hyperbola_segment_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Hyperbolic_Delaunay_triangulation_2.h 5.1-1/include/CGAL/Hyperbolic_Delaunay_triangulation_2.h
--- 5.0.3-1/include/CGAL/Hyperbolic_Delaunay_triangulation_2.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Hyperbolic_Delaunay_triangulation_2.h	2020-09-08 12:14:54.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Hyperbolic_triangulation_2/include/CGAL/Hyperbolic_Delaunay_triangulation_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Hyperbolic_triangulation_2/include/CGAL/Hyperbolic_Delaunay_triangulation_2.h $
 // $Id: Hyperbolic_Delaunay_triangulation_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Hyperbolic_Delaunay_triangulation_CK_traits_2.h 5.1-1/include/CGAL/Hyperbolic_Delaunay_triangulation_CK_traits_2.h
--- 5.0.3-1/include/CGAL/Hyperbolic_Delaunay_triangulation_CK_traits_2.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Hyperbolic_Delaunay_triangulation_CK_traits_2.h	2020-09-08 12:14:54.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Hyperbolic_triangulation_2/include/CGAL/Hyperbolic_Delaunay_triangulation_CK_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Hyperbolic_triangulation_2/include/CGAL/Hyperbolic_Delaunay_triangulation_CK_traits_2.h $
 // $Id: Hyperbolic_Delaunay_triangulation_CK_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Hyperbolic_Delaunay_triangulation_traits_2.h 5.1-1/include/CGAL/Hyperbolic_Delaunay_triangulation_traits_2.h
--- 5.0.3-1/include/CGAL/Hyperbolic_Delaunay_triangulation_traits_2.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Hyperbolic_Delaunay_triangulation_traits_2.h	2020-09-08 12:14:54.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Hyperbolic_triangulation_2/include/CGAL/Hyperbolic_Delaunay_triangulation_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Hyperbolic_triangulation_2/include/CGAL/Hyperbolic_Delaunay_triangulation_traits_2.h $
 // $Id: Hyperbolic_Delaunay_triangulation_traits_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Hyperbolic_octagon_translation.h 5.1-1/include/CGAL/Hyperbolic_octagon_translation.h
--- 5.0.3-1/include/CGAL/Hyperbolic_octagon_translation.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/Hyperbolic_octagon_translation.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_4_hyperbolic_triangulation_2/include/CGAL/Hyperbolic_octagon_translation.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_4_hyperbolic_triangulation_2/include/CGAL/Hyperbolic_octagon_translation.h $
 // $Id: Hyperbolic_octagon_translation.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Hyperbolic_triangulation_face_base_2.h 5.1-1/include/CGAL/Hyperbolic_triangulation_face_base_2.h
--- 5.0.3-1/include/CGAL/Hyperbolic_triangulation_face_base_2.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Hyperbolic_triangulation_face_base_2.h	2020-09-08 12:14:54.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Hyperbolic_triangulation_2/include/CGAL/Hyperbolic_triangulation_face_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Hyperbolic_triangulation_2/include/CGAL/Hyperbolic_triangulation_face_base_2.h $
 // $Id: Hyperbolic_triangulation_face_base_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Identity_policy_2.h 5.1-1/include/CGAL/Identity_policy_2.h
--- 5.0.3-1/include/CGAL/Identity_policy_2.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Identity_policy_2.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Identity_policy_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Identity_policy_2.h $
 // $Id: Identity_policy_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IEEE_754_unions.h 5.1-1/include/CGAL/IEEE_754_unions.h
--- 5.0.3-1/include/CGAL/IEEE_754_unions.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/IEEE_754_unions.h	2020-09-08 12:15:16.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/IEEE_754_unions.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/IEEE_754_unions.h $
 // $Id: IEEE_754_unions.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Image_3.h 5.1-1/include/CGAL/Image_3.h
--- 5.0.3-1/include/CGAL/Image_3.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Image_3.h	2020-09-08 12:14:44.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/Image_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/Image_3.h $
 // $Id: Image_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Image_3_impl.h 5.1-1/include/CGAL/Image_3_impl.h
--- 5.0.3-1/include/CGAL/Image_3_impl.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Image_3_impl.h	2020-09-08 12:14:44.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/Image_3_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/Image_3_impl.h $
 // $Id: Image_3_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Image_3_vtk_interface.h 5.1-1/include/CGAL/Image_3_vtk_interface.h
--- 5.0.3-1/include/CGAL/Image_3_vtk_interface.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Image_3_vtk_interface.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/Image_3_vtk_interface.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/Image_3_vtk_interface.h $
 // $Id: Image_3_vtk_interface.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/ImageIO/analyze.h 5.1-1/include/CGAL/ImageIO/analyze.h
--- 5.0.3-1/include/CGAL/ImageIO/analyze.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/ImageIO/analyze.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of the ImageIO Library, and as been adapted for CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/ImageIO/analyze.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/ImageIO/analyze.h $
 // $Id: analyze.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later
 //
diff -pruN 5.0.3-1/include/CGAL/ImageIO/analyze_impl.h 5.1-1/include/CGAL/ImageIO/analyze_impl.h
--- 5.0.3-1/include/CGAL/ImageIO/analyze_impl.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/ImageIO/analyze_impl.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of the ImageIO Library, and as been adapted for CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/ImageIO/analyze_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/ImageIO/analyze_impl.h $
 // $Id: analyze_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later
 //
diff -pruN 5.0.3-1/include/CGAL/ImageIO/bmpendian.h 5.1-1/include/CGAL/ImageIO/bmpendian.h
--- 5.0.3-1/include/CGAL/ImageIO/bmpendian.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/ImageIO/bmpendian.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of the ImageIO Library, and as been adapted for CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/ImageIO/bmpendian.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/ImageIO/bmpendian.h $
 // $Id: bmpendian.h 07c4ada 2019-10-19T15:50:09+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later
 //
diff -pruN 5.0.3-1/include/CGAL/ImageIO/bmpendian_impl.h 5.1-1/include/CGAL/ImageIO/bmpendian_impl.h
--- 5.0.3-1/include/CGAL/ImageIO/bmpendian_impl.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/ImageIO/bmpendian_impl.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of the ImageIO Library, and as been adapted for CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/ImageIO/bmpendian_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/ImageIO/bmpendian_impl.h $
 // $Id: bmpendian_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later
 //
diff -pruN 5.0.3-1/include/CGAL/ImageIO/bmp.h 5.1-1/include/CGAL/ImageIO/bmp.h
--- 5.0.3-1/include/CGAL/ImageIO/bmp.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/ImageIO/bmp.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of the ImageIO Library, and as been adapted for CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/ImageIO/bmp.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/ImageIO/bmp.h $
 // $Id: bmp.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later
 //
diff -pruN 5.0.3-1/include/CGAL/ImageIO/bmp_impl.h 5.1-1/include/CGAL/ImageIO/bmp_impl.h
--- 5.0.3-1/include/CGAL/ImageIO/bmp_impl.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/ImageIO/bmp_impl.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of the ImageIO Library, and as been adapted for CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/ImageIO/bmp_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/ImageIO/bmp_impl.h $
 // $Id: bmp_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later
 //
diff -pruN 5.0.3-1/include/CGAL/ImageIO/bmpread.h 5.1-1/include/CGAL/ImageIO/bmpread.h
--- 5.0.3-1/include/CGAL/ImageIO/bmpread.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/ImageIO/bmpread.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of the ImageIO Library, and as been adapted for CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/ImageIO/bmpread.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/ImageIO/bmpread.h $
 // $Id: bmpread.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later
 //
diff -pruN 5.0.3-1/include/CGAL/ImageIO/bmpread_impl.h 5.1-1/include/CGAL/ImageIO/bmpread_impl.h
--- 5.0.3-1/include/CGAL/ImageIO/bmpread_impl.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/ImageIO/bmpread_impl.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of the ImageIO Library, and as been adapted for CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/ImageIO/bmpread_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/ImageIO/bmpread_impl.h $
 // $Id: bmpread_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later
 //
diff -pruN 5.0.3-1/include/CGAL/ImageIO/bmptypes.h 5.1-1/include/CGAL/ImageIO/bmptypes.h
--- 5.0.3-1/include/CGAL/ImageIO/bmptypes.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/ImageIO/bmptypes.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of the ImageIO Library, and as been adapted for CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/ImageIO/bmptypes.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/ImageIO/bmptypes.h $
 // $Id: bmptypes.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later
 //
diff -pruN 5.0.3-1/include/CGAL/ImageIO/convert.h 5.1-1/include/CGAL/ImageIO/convert.h
--- 5.0.3-1/include/CGAL/ImageIO/convert.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/ImageIO/convert.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of the ImageIO Library, and as been adapted for CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/ImageIO/convert.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/ImageIO/convert.h $
 // $Id: convert.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later
 //
diff -pruN 5.0.3-1/include/CGAL/ImageIO/convert_impl.h 5.1-1/include/CGAL/ImageIO/convert_impl.h
--- 5.0.3-1/include/CGAL/ImageIO/convert_impl.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/ImageIO/convert_impl.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of the ImageIO Library, and as been adapted for CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/ImageIO/convert_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/ImageIO/convert_impl.h $
 // $Id: convert_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later
 //
diff -pruN 5.0.3-1/include/CGAL/ImageIO/fgetns.h 5.1-1/include/CGAL/ImageIO/fgetns.h
--- 5.0.3-1/include/CGAL/ImageIO/fgetns.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/ImageIO/fgetns.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of the ImageIO Library, and as been adapted for CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/ImageIO/fgetns.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/ImageIO/fgetns.h $
 // $Id: fgetns.h 07c4ada 2019-10-19T15:50:09+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later
 //
diff -pruN 5.0.3-1/include/CGAL/ImageIO/fgetns_impl.h 5.1-1/include/CGAL/ImageIO/fgetns_impl.h
--- 5.0.3-1/include/CGAL/ImageIO/fgetns_impl.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/ImageIO/fgetns_impl.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of the ImageIO Library, and as been adapted for CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/ImageIO/fgetns_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/ImageIO/fgetns_impl.h $
 // $Id: fgetns_impl.h 07c4ada 2019-10-19T15:50:09+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later
 //
diff -pruN 5.0.3-1/include/CGAL/ImageIO/gif.h 5.1-1/include/CGAL/ImageIO/gif.h
--- 5.0.3-1/include/CGAL/ImageIO/gif.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/ImageIO/gif.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of the ImageIO Library, and as been adapted for CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/ImageIO/gif.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/ImageIO/gif.h $
 // $Id: gif.h 07c4ada 2019-10-19T15:50:09+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later
 //
diff -pruN 5.0.3-1/include/CGAL/ImageIO/gif_impl.h 5.1-1/include/CGAL/ImageIO/gif_impl.h
--- 5.0.3-1/include/CGAL/ImageIO/gif_impl.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/ImageIO/gif_impl.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of the ImageIO Library, and as been adapted for CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/ImageIO/gif_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/ImageIO/gif_impl.h $
 // $Id: gif_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later
 //
diff -pruN 5.0.3-1/include/CGAL/ImageIO/gis.h 5.1-1/include/CGAL/ImageIO/gis.h
--- 5.0.3-1/include/CGAL/ImageIO/gis.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/ImageIO/gis.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of the ImageIO Library, and as been adapted for CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/ImageIO/gis.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/ImageIO/gis.h $
 // $Id: gis.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later
 //
diff -pruN 5.0.3-1/include/CGAL/ImageIO/gis_impl.h 5.1-1/include/CGAL/ImageIO/gis_impl.h
--- 5.0.3-1/include/CGAL/ImageIO/gis_impl.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/ImageIO/gis_impl.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of the ImageIO Library, and as been adapted for CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/ImageIO/gis_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/ImageIO/gis_impl.h $
 // $Id: gis_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later
 //
diff -pruN 5.0.3-1/include/CGAL/ImageIO/inr.h 5.1-1/include/CGAL/ImageIO/inr.h
--- 5.0.3-1/include/CGAL/ImageIO/inr.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/ImageIO/inr.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of the ImageIO Library, and as been adapted for CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/ImageIO/inr.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/ImageIO/inr.h $
 // $Id: inr.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later
 //
diff -pruN 5.0.3-1/include/CGAL/ImageIO/inr_impl.h 5.1-1/include/CGAL/ImageIO/inr_impl.h
--- 5.0.3-1/include/CGAL/ImageIO/inr_impl.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/ImageIO/inr_impl.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of the ImageIO Library, and as been adapted for CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/ImageIO/inr_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/ImageIO/inr_impl.h $
 // $Id: inr_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later
 //
diff -pruN 5.0.3-1/include/CGAL/ImageIO/iris.h 5.1-1/include/CGAL/ImageIO/iris.h
--- 5.0.3-1/include/CGAL/ImageIO/iris.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/ImageIO/iris.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of the ImageIO Library, and as been adapted for CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/ImageIO/iris.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/ImageIO/iris.h $
 // $Id: iris.h 07c4ada 2019-10-19T15:50:09+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later
 //
diff -pruN 5.0.3-1/include/CGAL/ImageIO/iris_impl.h 5.1-1/include/CGAL/ImageIO/iris_impl.h
--- 5.0.3-1/include/CGAL/ImageIO/iris_impl.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/ImageIO/iris_impl.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of the ImageIO Library, and as been adapted for CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/ImageIO/iris_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/ImageIO/iris_impl.h $
 // $Id: iris_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later
 //
diff -pruN 5.0.3-1/include/CGAL/ImageIO/mincio.h 5.1-1/include/CGAL/ImageIO/mincio.h
--- 5.0.3-1/include/CGAL/ImageIO/mincio.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/ImageIO/mincio.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of the ImageIO Library, and as been adapted for CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/ImageIO/mincio.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/ImageIO/mincio.h $
 // $Id: mincio.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later
 //
diff -pruN 5.0.3-1/include/CGAL/ImageIO/mincio_impl.h 5.1-1/include/CGAL/ImageIO/mincio_impl.h
--- 5.0.3-1/include/CGAL/ImageIO/mincio_impl.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/ImageIO/mincio_impl.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of the ImageIO Library, and as been adapted for CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/ImageIO/mincio_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/ImageIO/mincio_impl.h $
 // $Id: mincio_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later
 //
diff -pruN 5.0.3-1/include/CGAL/ImageIO/pnm.h 5.1-1/include/CGAL/ImageIO/pnm.h
--- 5.0.3-1/include/CGAL/ImageIO/pnm.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/ImageIO/pnm.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of the ImageIO Library, and as been adapted for CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/ImageIO/pnm.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/ImageIO/pnm.h $
 // $Id: pnm.h 07c4ada 2019-10-19T15:50:09+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later
 //
diff -pruN 5.0.3-1/include/CGAL/ImageIO/pnm_impl.h 5.1-1/include/CGAL/ImageIO/pnm_impl.h
--- 5.0.3-1/include/CGAL/ImageIO/pnm_impl.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/ImageIO/pnm_impl.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of the ImageIO Library, and as been adapted for CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/ImageIO/pnm_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/ImageIO/pnm_impl.h $
 // $Id: pnm_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later
 //
diff -pruN 5.0.3-1/include/CGAL/ImageIO/recbuffer.h 5.1-1/include/CGAL/ImageIO/recbuffer.h
--- 5.0.3-1/include/CGAL/ImageIO/recbuffer.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/ImageIO/recbuffer.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of the ImageIO Library, and as been adapted for CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/ImageIO/recbuffer.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/ImageIO/recbuffer.h $
 // $Id: recbuffer.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later
 //
diff -pruN 5.0.3-1/include/CGAL/ImageIO/recbuffer_impl.h 5.1-1/include/CGAL/ImageIO/recbuffer_impl.h
--- 5.0.3-1/include/CGAL/ImageIO/recbuffer_impl.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/ImageIO/recbuffer_impl.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of the ImageIO Library, and as been adapted for CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/ImageIO/recbuffer_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/ImageIO/recbuffer_impl.h $
 // $Id: recbuffer_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later
 //
diff -pruN 5.0.3-1/include/CGAL/ImageIO/recline.h 5.1-1/include/CGAL/ImageIO/recline.h
--- 5.0.3-1/include/CGAL/ImageIO/recline.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/ImageIO/recline.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of the ImageIO Library, and as been adapted for CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/ImageIO/recline.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/ImageIO/recline.h $
 // $Id: recline.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later
 //
diff -pruN 5.0.3-1/include/CGAL/ImageIO/recline_impl.h 5.1-1/include/CGAL/ImageIO/recline_impl.h
--- 5.0.3-1/include/CGAL/ImageIO/recline_impl.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/ImageIO/recline_impl.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of the ImageIO Library, and as been adapted for CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/ImageIO/recline_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/ImageIO/recline_impl.h $
 // $Id: recline_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later
 //
diff -pruN 5.0.3-1/include/CGAL/ImageIO/reech4x4.h 5.1-1/include/CGAL/ImageIO/reech4x4.h
--- 5.0.3-1/include/CGAL/ImageIO/reech4x4.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/ImageIO/reech4x4.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of the ImageIO Library, and as been adapted for CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/ImageIO/reech4x4.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/ImageIO/reech4x4.h $
 // $Id: reech4x4.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later
 //
diff -pruN 5.0.3-1/include/CGAL/ImageIO/reech4x4_impl.h 5.1-1/include/CGAL/ImageIO/reech4x4_impl.h
--- 5.0.3-1/include/CGAL/ImageIO/reech4x4_impl.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/ImageIO/reech4x4_impl.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of the ImageIO Library, and as been adapted for CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/ImageIO/reech4x4_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/ImageIO/reech4x4_impl.h $
 // $Id: reech4x4_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later
 //
diff -pruN 5.0.3-1/include/CGAL/ImageIO/typedefs.h 5.1-1/include/CGAL/ImageIO/typedefs.h
--- 5.0.3-1/include/CGAL/ImageIO/typedefs.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/ImageIO/typedefs.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of the ImageIO Library, and as been adapted for CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/ImageIO/typedefs.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/ImageIO/typedefs.h $
 // $Id: typedefs.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later
 //
diff -pruN 5.0.3-1/include/CGAL/ImageIO.h 5.1-1/include/CGAL/ImageIO.h
--- 5.0.3-1/include/CGAL/ImageIO.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/ImageIO.h	2020-09-08 12:14:44.000000000 +0000
@@ -5,7 +5,7 @@
 //
 // This file is part of the ImageIO Library, and as been adapted for CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/ImageIO.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/ImageIO.h $
 // $Id: ImageIO.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later
 //
diff -pruN 5.0.3-1/include/CGAL/ImageIO_impl.h 5.1-1/include/CGAL/ImageIO_impl.h
--- 5.0.3-1/include/CGAL/ImageIO_impl.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/ImageIO_impl.h	2020-09-08 12:14:44.000000000 +0000
@@ -5,7 +5,7 @@
 //
 // This file is part of the ImageIO Library, and as been adapted for CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/ImageIO_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/ImageIO_impl.h $
 // $Id: ImageIO_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later
 //
diff -pruN 5.0.3-1/include/CGAL/Implicit_mesh_domain_3.h 5.1-1/include/CGAL/Implicit_mesh_domain_3.h
--- 5.0.3-1/include/CGAL/Implicit_mesh_domain_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Implicit_mesh_domain_3.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Implicit_mesh_domain_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Implicit_mesh_domain_3.h $
 // $Id: Implicit_mesh_domain_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Implicit_surface_3.h 5.1-1/include/CGAL/Implicit_surface_3.h
--- 5.0.3-1/include/CGAL/Implicit_surface_3.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Implicit_surface_3.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Implicit_surface_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Implicit_surface_3.h $
 // $Id: Implicit_surface_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Implicit_to_labeled_subdomains_function_wrapper.h 5.1-1/include/CGAL/Implicit_to_labeled_subdomains_function_wrapper.h
--- 5.0.3-1/include/CGAL/Implicit_to_labeled_subdomains_function_wrapper.h	2020-08-01 19:02:38.000000000 +0000
+++ 5.1-1/include/CGAL/Implicit_to_labeled_subdomains_function_wrapper.h	2020-09-08 12:15:20.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_mesh_3/include/CGAL/Implicit_to_labeled_subdomains_function_wrapper.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_mesh_3/include/CGAL/Implicit_to_labeled_subdomains_function_wrapper.h $
 // $Id: Implicit_to_labeled_subdomains_function_wrapper.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Implicit_to_labeling_function_wrapper.h 5.1-1/include/CGAL/Implicit_to_labeling_function_wrapper.h
--- 5.0.3-1/include/CGAL/Implicit_to_labeling_function_wrapper.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Implicit_to_labeling_function_wrapper.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Implicit_to_labeling_function_wrapper.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Implicit_to_labeling_function_wrapper.h $
 // $Id: Implicit_to_labeling_function_wrapper.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Incremental_neighbor_search.h 5.1-1/include/CGAL/Incremental_neighbor_search.h
--- 5.0.3-1/include/CGAL/Incremental_neighbor_search.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Incremental_neighbor_search.h	2020-09-08 12:15:39.000000000 +0000
@@ -3,12 +3,13 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_searching/include/CGAL/Incremental_neighbor_search.h $
-// $Id: Incremental_neighbor_search.h c1ffa15 2020-05-18T12:22:15+02:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_searching/include/CGAL/Incremental_neighbor_search.h $
+// $Id: Incremental_neighbor_search.h dfe272f 2020-05-18T12:24:05+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
 // Author(s)     : Hans Tangelder (<hanst@cs.uu.nl>)
+//                 Clement Jamin (clement.jamin.pro@gmail.com)
 
 #ifndef CGAL_INCREMENTAL_NEIGHBOR_SEARCH_H
 #define CGAL_INCREMENTAL_NEIGHBOR_SEARCH_H
@@ -16,21 +17,23 @@
 #include <CGAL/license/Spatial_searching.h>
 
 #include <CGAL/disable_warnings.h>
+#include <CGAL/Kd_tree_node.h>
+#include <CGAL/Kd_tree_rectangle.h>
+#include <CGAL/Euclidean_distance.h>
+#include <CGAL/internal/Search_helpers.h>
 
 #include <cstring>
 #include <list>
 #include <queue>
 #include <memory>
-#include <CGAL/Kd_tree_node.h>
-#include <CGAL/Kd_tree_rectangle.h>
-#include <CGAL/Euclidean_distance.h>
+#include <iterator> // for std::distance
 
 namespace CGAL {
 
   template <class SearchTraits,
             class Distance_=typename internal::Spatial_searching_default_distance<SearchTraits>::type,
             class Splitter_ = Sliding_midpoint<SearchTraits>,
-            class Tree_=Kd_tree<SearchTraits, Splitter_, Tag_false> >
+            class Tree_=Kd_tree<SearchTraits, Splitter_, Tag_false, Tag_false> >
   class Incremental_neighbor_search {
 
   public:
@@ -264,6 +267,9 @@ namespace CGAL {
 
         FT rd;
 
+        internal::Distance_helper<Distance, SearchTraits> m_distance_helper;
+        int m_dim;
+        Tree const& m_tree;
 
         class Priority_higher {
 
@@ -324,6 +330,8 @@ namespace CGAL {
         Iterator_implementation(const Tree& tree, const Query_item& q,const Distance& tr,
                                 FT Eps, bool search_nearest)
           : query_point(q), search_nearest_neighbour(search_nearest),
+          m_distance_helper(tr, tree.traits()),
+          m_tree(tree),
           PriorityQueue(Priority_higher(search_nearest)),
           Item_PriorityQueue(Distance_smaller(search_nearest)),
           distance(tr), reference_count(1), number_of_internal_nodes_visited(0),
@@ -332,6 +340,10 @@ namespace CGAL {
         {
           if (tree.empty()) return;
 
+          typename SearchTraits::Construct_cartesian_const_iterator_d construct_it =
+            m_tree.traits().construct_cartesian_const_iterator_d_object();
+          m_dim = static_cast<int>(std::distance(construct_it(q), construct_it(q, 0)));
+
           multiplication_factor= distance.transformed_distance(FT(1)+Eps);
 
           Node_box *bounding_box = new Node_box((tree.bounding_box()));
@@ -422,6 +434,81 @@ namespace CGAL {
           delete The_item_top;
         }
 
+
+        // With cache
+        bool search_in_leaf(typename Tree::Leaf_node_const_handle node, Tag_true, bool search_furthest)
+        {
+          typename Tree::iterator it_node_point = node->begin(), it_node_point_end = node->end();
+          typename std::vector<FT>::const_iterator cache_point_begin = m_tree.cache_begin() + m_dim*(it_node_point - m_tree.begin());
+
+          for (; it_node_point != node->end(); ++it_node_point)
+          {
+            number_of_items_visited++;
+            FT distance_to_query_point =
+              m_distance_helper.transformed_distance_from_coordinates(
+                query_point, *it_node_point, cache_point_begin, cache_point_begin + m_dim);
+
+            Point_with_transformed_distance *NN_Candidate =
+              new Point_with_transformed_distance(*it_node_point, distance_to_query_point);
+            Item_PriorityQueue.push(NN_Candidate);
+
+            cache_point_begin += m_dim;
+          }
+          // old top of PriorityQueue has been processed,
+          // hence update rd
+
+          bool next_neighbour_found;
+          if (!(PriorityQueue.empty()))
+          {
+            rd = PriorityQueue.top()->second;
+            next_neighbour_found = (search_furthest ?
+              multiplication_factor*rd < Item_PriorityQueue.top()->second
+              : multiplication_factor*rd > Item_PriorityQueue.top()->second);
+          }
+          else // priority queue empty => last neighbour found
+          {
+            next_neighbour_found = true;
+          }
+
+          number_of_neighbours_computed++;
+          return next_neighbour_found;
+        }
+
+        // Without cache
+        bool search_in_leaf(typename Tree::Leaf_node_const_handle node, Tag_false, bool search_furthest)
+        {
+          typename Tree::iterator it_node_point = node->begin(), it_node_point_end = node->end();
+
+          for (; it_node_point != it_node_point_end; ++it_node_point)
+          {
+            number_of_items_visited++;
+            FT distance_to_query_point =
+              distance.transformed_distance(query_point, *it_node_point);
+
+            Point_with_transformed_distance *NN_Candidate =
+              new Point_with_transformed_distance(*it_node_point, distance_to_query_point);
+            Item_PriorityQueue.push(NN_Candidate);
+          }
+          // old top of PriorityQueue has been processed,
+          // hence update rd
+
+          bool next_neighbour_found;
+          if (!(PriorityQueue.empty()))
+          {
+            rd = PriorityQueue.top()->second;
+            next_neighbour_found = (search_furthest ?
+              multiplication_factor*rd < Item_PriorityQueue.top()->second
+              : multiplication_factor*rd > Item_PriorityQueue.top()->second);
+          }
+          else // priority queue empty => last neighbour found
+          {
+            next_neighbour_found = true;
+          }
+
+          number_of_neighbours_computed++;
+          return next_neighbour_found;
+        }
+
         void
         Compute_the_next_nearest_neighbour()
         {
@@ -501,36 +588,14 @@ namespace CGAL {
               }
             }
             delete B;
-             typename Tree::Leaf_node_const_handle node =
+
+            // N is a leaf
+            typename Tree::Leaf_node_const_handle node =
               static_cast<typename Tree::Leaf_node_const_handle>(N);
             number_of_leaf_nodes_visited++;
             if (node->size() > 0) {
-              for (typename Tree::iterator it = node->begin(); it != node->end(); it++) {
-                number_of_items_visited++;
-                FT distance_to_query_point=
-                  distance.transformed_distance(query_point,*it);
-                Point_with_transformed_distance *NN_Candidate=
-                  new Point_with_transformed_distance(*it,distance_to_query_point);
-                Item_PriorityQueue.push(NN_Candidate);
-              }
-              // old top of PriorityQueue has been processed,
-              // hence update rd
-              if (!PriorityQueue.empty()) {
-                rd = PriorityQueue.top()->second;
-                if (search_nearest_neighbour)
-                  next_neighbour_found =
-                    (multiplication_factor*rd >
-                     Item_PriorityQueue.top()->second);
-                else
-                  next_neighbour_found =
-                    (multiplication_factor*rd <
-                     Item_PriorityQueue.top()->second);
-              }
-              else // priority queue empty => last neighbour found
-                {
-                  next_neighbour_found=true;
-                };
-              number_of_neighbours_computed++;
+              typename internal::Has_points_cache<Tree, internal::has_Enable_points_cache<Tree>::type::value>::type dummy;
+              next_neighbour_found = search_in_leaf(node, dummy, !search_nearest_neighbour);
             }
           }   // next_neighbour_found or priority queue is empty
           // in the latter case also the item priority queue is empty
diff -pruN 5.0.3-1/include/CGAL/Index_property_map.h 5.1-1/include/CGAL/Index_property_map.h
--- 5.0.3-1/include/CGAL/Index_property_map.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Index_property_map.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Property_map/include/CGAL/Index_property_map.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Property_map/include/CGAL/Index_property_map.h $
 // $Id: Index_property_map.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/in_place_edge_list.h 5.1-1/include/CGAL/in_place_edge_list.h
--- 5.0.3-1/include/CGAL/in_place_edge_list.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/in_place_edge_list.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/in_place_edge_list.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/in_place_edge_list.h $
 // $Id: in_place_edge_list.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/In_place_list.h 5.1-1/include/CGAL/In_place_list.h
--- 5.0.3-1/include/CGAL/In_place_list.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/In_place_list.h	2020-09-08 12:15:32.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/In_place_list.h $
-// $Id: In_place_list.h dc12dea 2020-05-04T17:52:26+02:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/In_place_list.h $
+// $Id: In_place_list.h 704cdab 2020-04-26T07:43:22+02:00 Michael Hemmer
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
diff -pruN 5.0.3-1/include/CGAL/internal/AABB_tree/AABB_drawing_traits.h 5.1-1/include/CGAL/internal/AABB_tree/AABB_drawing_traits.h
--- 5.0.3-1/include/CGAL/internal/AABB_tree/AABB_drawing_traits.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/internal/AABB_tree/AABB_drawing_traits.h	2020-09-08 12:14:29.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/AABB_tree/include/CGAL/internal/AABB_tree/AABB_drawing_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/AABB_tree/include/CGAL/internal/AABB_tree/AABB_drawing_traits.h $
 // $Id: AABB_drawing_traits.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/AABB_tree/AABB_node.h 5.1-1/include/CGAL/internal/AABB_tree/AABB_node.h
--- 5.0.3-1/include/CGAL/internal/AABB_tree/AABB_node.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/internal/AABB_tree/AABB_node.h	2020-09-08 12:14:29.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/AABB_tree/include/CGAL/internal/AABB_tree/AABB_node.h $
-// $Id: AABB_node.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/AABB_tree/include/CGAL/internal/AABB_tree/AABB_node.h $
+// $Id: AABB_node.h f56cd59 2020-04-23T16:13:32+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -56,10 +56,12 @@ public:
    *
    * [first,last[ is the range of primitives to be added to the tree.
    */
-  template<typename ConstPrimitiveIterator>
+  template<typename ConstPrimitiveIterator, typename ComputeBbox, typename SplitPrimitives>
   void expand(ConstPrimitiveIterator first,
               ConstPrimitiveIterator beyond,
               const std::size_t range,
+              const ComputeBbox& compute_bbox,
+              const SplitPrimitives& split_primitives,
               const AABBTraits&);
 
   /**
@@ -117,19 +119,20 @@ private:
 
 };  // end class AABB_node
 
-
 template<typename Tr>
-template<typename ConstPrimitiveIterator>
+template<typename ConstPrimitiveIterator, typename ComputeBbox, typename SplitPrimitives>
 void
 AABB_node<Tr>::expand(ConstPrimitiveIterator first,
                       ConstPrimitiveIterator beyond,
                       const std::size_t range,
+                      const ComputeBbox& compute_bbox,
+                      const SplitPrimitives& split_primitives,
                       const Tr& traits)
 {
-  m_bbox = traits.compute_bbox_object()(first, beyond);
+  m_bbox = compute_bbox(first, beyond);
 
   // sort primitives along longest axis aabb
-  traits.split_primitives_object()(first, beyond, m_bbox);
+  split_primitives(first, beyond, m_bbox);
 
   switch(range)
   {
@@ -140,14 +143,14 @@ AABB_node<Tr>::expand(ConstPrimitiveIter
   case 3:
     m_p_left_child = &(*first);
     m_p_right_child = static_cast<Node*>(this)+1;
-    right_child().expand(first+1, beyond, 2,traits);
+    right_child().expand(first+1, beyond, 2, compute_bbox, split_primitives, traits);
     break;
   default:
     const std::size_t new_range = range/2;
     m_p_left_child = static_cast<Node*>(this) + 1;
     m_p_right_child = static_cast<Node*>(this) + new_range;
-    left_child().expand(first, first + new_range, new_range,traits);
-    right_child().expand(first + new_range, beyond, range - new_range,traits);
+    left_child().expand(first, first + new_range, new_range, compute_bbox, split_primitives, traits);
+    right_child().expand(first + new_range, beyond, range - new_range, compute_bbox, split_primitives, traits);
   }
 }
 
diff -pruN 5.0.3-1/include/CGAL/internal/AABB_tree/AABB_ray_intersection.h 5.1-1/include/CGAL/internal/AABB_tree/AABB_ray_intersection.h
--- 5.0.3-1/include/CGAL/internal/AABB_tree/AABB_ray_intersection.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/internal/AABB_tree/AABB_ray_intersection.h	2020-09-08 12:14:29.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/AABB_tree/include/CGAL/internal/AABB_tree/AABB_ray_intersection.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/AABB_tree/include/CGAL/internal/AABB_tree/AABB_ray_intersection.h $
 // $Id: AABB_ray_intersection.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/AABB_tree/AABB_search_tree.h 5.1-1/include/CGAL/internal/AABB_tree/AABB_search_tree.h
--- 5.0.3-1/include/CGAL/internal/AABB_tree/AABB_search_tree.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/internal/AABB_tree/AABB_search_tree.h	2020-09-08 12:14:29.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/AABB_tree/include/CGAL/internal/AABB_tree/AABB_search_tree.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/AABB_tree/include/CGAL/internal/AABB_tree/AABB_search_tree.h $
 // $Id: AABB_search_tree.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/AABB_tree/AABB_traversal_traits.h 5.1-1/include/CGAL/internal/AABB_tree/AABB_traversal_traits.h
--- 5.0.3-1/include/CGAL/internal/AABB_tree/AABB_traversal_traits.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/internal/AABB_tree/AABB_traversal_traits.h	2020-09-08 12:14:29.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/AABB_tree/include/CGAL/internal/AABB_tree/AABB_traversal_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/AABB_tree/include/CGAL/internal/AABB_tree/AABB_traversal_traits.h $
 // $Id: AABB_traversal_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/AABB_tree/Has_nested_type_Shared_data.h 5.1-1/include/CGAL/internal/AABB_tree/Has_nested_type_Shared_data.h
--- 5.0.3-1/include/CGAL/internal/AABB_tree/Has_nested_type_Shared_data.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/internal/AABB_tree/Has_nested_type_Shared_data.h	2020-09-08 12:14:29.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/AABB_tree/include/CGAL/internal/AABB_tree/Has_nested_type_Shared_data.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/AABB_tree/include/CGAL/internal/AABB_tree/Has_nested_type_Shared_data.h $
 // $Id: Has_nested_type_Shared_data.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/AABB_tree/Is_ray_intersection_geomtraits.h 5.1-1/include/CGAL/internal/AABB_tree/Is_ray_intersection_geomtraits.h
--- 5.0.3-1/include/CGAL/internal/AABB_tree/Is_ray_intersection_geomtraits.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/internal/AABB_tree/Is_ray_intersection_geomtraits.h	2020-09-08 12:14:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/AABB_tree/include/CGAL/internal/AABB_tree/Is_ray_intersection_geomtraits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/AABB_tree/include/CGAL/internal/AABB_tree/Is_ray_intersection_geomtraits.h $
 // $Id: Is_ray_intersection_geomtraits.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/AABB_tree/Primitive_helper.h 5.1-1/include/CGAL/internal/AABB_tree/Primitive_helper.h
--- 5.0.3-1/include/CGAL/internal/AABB_tree/Primitive_helper.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/internal/AABB_tree/Primitive_helper.h	2020-09-08 12:14:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/AABB_tree/include/CGAL/internal/AABB_tree/Primitive_helper.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/AABB_tree/include/CGAL/internal/AABB_tree/Primitive_helper.h $
 // $Id: Primitive_helper.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/AFSR/construct_polyhedron.h 5.1-1/include/CGAL/internal/AFSR/construct_polyhedron.h
--- 5.0.3-1/include/CGAL/internal/AFSR/construct_polyhedron.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/internal/AFSR/construct_polyhedron.h	2020-09-08 12:14:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Advancing_front_surface_reconstruction/include/CGAL/internal/AFSR/construct_polyhedron.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Advancing_front_surface_reconstruction/include/CGAL/internal/AFSR/construct_polyhedron.h $
 // $Id: construct_polyhedron.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/AFSR/construct_surface_2.h 5.1-1/include/CGAL/internal/AFSR/construct_surface_2.h
--- 5.0.3-1/include/CGAL/internal/AFSR/construct_surface_2.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/internal/AFSR/construct_surface_2.h	2020-09-08 12:14:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Advancing_front_surface_reconstruction/include/CGAL/internal/AFSR/construct_surface_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Advancing_front_surface_reconstruction/include/CGAL/internal/AFSR/construct_surface_2.h $
 // $Id: construct_surface_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/AFSR/orient.h 5.1-1/include/CGAL/internal/AFSR/orient.h
--- 5.0.3-1/include/CGAL/internal/AFSR/orient.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/internal/AFSR/orient.h	2020-09-08 12:14:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Advancing_front_surface_reconstruction/include/CGAL/internal/AFSR/orient.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Advancing_front_surface_reconstruction/include/CGAL/internal/AFSR/orient.h $
 // $Id: orient.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/AFSR/Surface_face_base_2.h 5.1-1/include/CGAL/internal/AFSR/Surface_face_base_2.h
--- 5.0.3-1/include/CGAL/internal/AFSR/Surface_face_base_2.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/internal/AFSR/Surface_face_base_2.h	2020-09-08 12:14:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Advancing_front_surface_reconstruction/include/CGAL/internal/AFSR/Surface_face_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Advancing_front_surface_reconstruction/include/CGAL/internal/AFSR/Surface_face_base_2.h $
 // $Id: Surface_face_base_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/AFSR/Surface_vertex_base_2.h 5.1-1/include/CGAL/internal/AFSR/Surface_vertex_base_2.h
--- 5.0.3-1/include/CGAL/internal/AFSR/Surface_vertex_base_2.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/internal/AFSR/Surface_vertex_base_2.h	2020-09-08 12:14:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Advancing_front_surface_reconstruction/include/CGAL/internal/AFSR/Surface_vertex_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Advancing_front_surface_reconstruction/include/CGAL/internal/AFSR/Surface_vertex_base_2.h $
 // $Id: Surface_vertex_base_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/AFSR/write_triple_indices.h 5.1-1/include/CGAL/internal/AFSR/write_triple_indices.h
--- 5.0.3-1/include/CGAL/internal/AFSR/write_triple_indices.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/internal/AFSR/write_triple_indices.h	2020-09-08 12:14:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Advancing_front_surface_reconstruction/include/CGAL/internal/AFSR/write_triple_indices.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Advancing_front_surface_reconstruction/include/CGAL/internal/AFSR/write_triple_indices.h $
 // $Id: write_triple_indices.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/alpha_shape_mesh.h 5.1-1/include/CGAL/internal/alpha_shape_mesh.h
--- 5.0.3-1/include/CGAL/internal/alpha_shape_mesh.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/internal/alpha_shape_mesh.h	2020-09-08 12:15:23.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygonal_surface_reconstruction/include/CGAL/internal/alpha_shape_mesh.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygonal_surface_reconstruction/include/CGAL/internal/alpha_shape_mesh.h $
 // $Id: alpha_shape_mesh.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/auxiliary/graph.h 5.1-1/include/CGAL/internal/auxiliary/graph.h
--- 5.0.3-1/include/CGAL/internal/auxiliary/graph.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/internal/auxiliary/graph.h	2020-09-08 12:15:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_segmentation/include/CGAL/internal/auxiliary/graph.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_segmentation/include/CGAL/internal/auxiliary/graph.h $
 // $Id: graph.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // Re-licensed for CGAL distribution to:
 // SPDX-License-Identifier: GPL-3.0-or-later
diff -pruN 5.0.3-1/include/CGAL/internal/boost/array_binary_tree.hpp 5.1-1/include/CGAL/internal/boost/array_binary_tree.hpp
--- 5.0.3-1/include/CGAL/internal/boost/array_binary_tree.hpp	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/internal/boost/array_binary_tree.hpp	2020-09-08 12:15:32.000000000 +0000
@@ -8,7 +8,7 @@
 // https://www.boost.org/LICENSE_1_0.txt)
 //=======================================================================
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/internal/boost/array_binary_tree.hpp $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/internal/boost/array_binary_tree.hpp $
 // $Id: array_binary_tree.hpp 590612a 2020-03-26T13:25:16+01:00 Sébastien Loriot
 // SPDX-License-Identifier: BSL-1.0
 //
diff -pruN 5.0.3-1/include/CGAL/internal/boost/function_property_map.hpp 5.1-1/include/CGAL/internal/boost/function_property_map.hpp
--- 5.0.3-1/include/CGAL/internal/boost/function_property_map.hpp	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/internal/boost/function_property_map.hpp	1970-01-01 00:00:00.000000000 +0000
@@ -1,84 +0,0 @@
-//
-//=======================================================================
-// Author: Philipp Moeller
-//
-// Copyright 2012, Philipp Moeller
-//
-// Distributed under the Boost Software License, Version 1.0. (See
-// accompanying file LICENSE_1_0.txt or copy at
-// https://www.boost.org/LICENSE_1_0.txt)
-//=======================================================================
-//
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/internal/boost/function_property_map.hpp $
-// $Id: function_property_map.hpp a472c24 2019-03-20T08:59:22+01:00 Andreas Fabri
-// SPDX-License-Identifier: BSL-1.0
-//
-
-// Note: This file was copied from Boost v1.64 because it was introduced
-//       in a release (~v1.51) that is newer than the oldest release of Boost
-//       that CGAL supports.
-//       the property map 'function_property_map' is (currently) used
-//       in the packages Triangulation_2 and Triangulation_3.
-
-#ifndef CGAL_INTERNAL_BOOST_PROPERTY_MAP_FUNCTION_PROPERTY_MAP_HPP
-#define CGAL_INTERNAL_BOOST_PROPERTY_MAP_FUNCTION_PROPERTY_MAP_HPP
-
-#include <boost/config.hpp>
-#include <boost/property_map/property_map.hpp>
-#include <boost/type_traits.hpp>
-#include <boost/utility/result_of.hpp>
-#include <boost/mpl/and.hpp>
-#include <boost/mpl/not.hpp>
-#include <utility>
-
-namespace CGAL {
-namespace internal {
-namespace boost_ {
-
-template<typename Func,
-         typename Key,
-         typename Ret = typename boost::result_of<const Func(const Key&)>::type>
-class function_property_map
-  : public boost::put_get_helper<Ret, function_property_map<Func, Key, Ret> >
-{
-public:
-  typedef Key key_type;
-  typedef Ret reference;
-  typedef typename boost::remove_cv<typename boost::remove_reference<Ret>::type>::type value_type;
-
-  typedef typename boost::mpl::if_<
-                     boost::mpl::and_<
-                       boost::is_reference<Ret>,
-                       boost::mpl::not_< boost::is_const<Ret> >
-                     >,
-                     boost::lvalue_property_map_tag,
-                     boost::readable_property_map_tag>::type
-    category;
-
-  function_property_map(Func f = Func()) : f(f) {}
-
-  reference operator[](const Key& k) const {
-    return f(k);
-  }
-
-private:
-  Func f;
-};
-
-template<typename Key, typename Func>
-function_property_map<Func, Key>
-make_function_property_map(const Func& f) {
-  return function_property_map<Func, Key>(f);
-}
-
-template<typename Key, typename Ret, typename Func>
-function_property_map<Func, Key, Ret>
-make_function_property_map(const Func& f) {
-  return function_property_map<Func, Key, Ret>(f);
-}
-
-} // boost_
-} // internal
-} // CGAL
-
-#endif /* CGAL_INTERNAL_BOOST_PROPERTY_MAP_FUNCTION_PROPERTY_MAP_HPP */
diff -pruN 5.0.3-1/include/CGAL/internal/boost/mutable_heap.hpp 5.1-1/include/CGAL/internal/boost/mutable_heap.hpp
--- 5.0.3-1/include/CGAL/internal/boost/mutable_heap.hpp	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/internal/boost/mutable_heap.hpp	2020-09-08 12:15:32.000000000 +0000
@@ -8,7 +8,7 @@
 // https://www.boost.org/LICENSE_1_0.txt)
 //=======================================================================
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/internal/boost/mutable_heap.hpp $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/internal/boost/mutable_heap.hpp $
 // $Id: mutable_heap.hpp 590612a 2020-03-26T13:25:16+01:00 Sébastien Loriot
 // SPDX-License-Identifier: BSL-1.0
 //
diff -pruN 5.0.3-1/include/CGAL/internal/boost/mutable_queue.hpp 5.1-1/include/CGAL/internal/boost/mutable_queue.hpp
--- 5.0.3-1/include/CGAL/internal/boost/mutable_queue.hpp	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/internal/boost/mutable_queue.hpp	2020-09-08 12:15:32.000000000 +0000
@@ -8,7 +8,7 @@
 // https://www.boost.org/LICENSE_1_0.txt)
 //=======================================================================
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/internal/boost/mutable_queue.hpp $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/internal/boost/mutable_queue.hpp $
 // $Id: mutable_queue.hpp 590612a 2020-03-26T13:25:16+01:00 Sébastien Loriot
 // SPDX-License-Identifier: BSL-1.0
 //
diff -pruN 5.0.3-1/include/CGAL/internal/bounded_priority_queue.h 5.1-1/include/CGAL/internal/bounded_priority_queue.h
--- 5.0.3-1/include/CGAL/internal/bounded_priority_queue.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/internal/bounded_priority_queue.h	2020-09-08 12:15:40.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_searching/include/CGAL/internal/bounded_priority_queue.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_searching/include/CGAL/internal/bounded_priority_queue.h $
 // $Id: bounded_priority_queue.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/canonicalize_helper.h 5.1-1/include/CGAL/internal/canonicalize_helper.h
--- 5.0.3-1/include/CGAL/internal/canonicalize_helper.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/internal/canonicalize_helper.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_triangulation_3/include/CGAL/internal/canonicalize_helper.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_triangulation_3/include/CGAL/internal/canonicalize_helper.h $
 // $Id: canonicalize_helper.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Classification_type.h 5.1-1/include/CGAL/internal/Classification_type.h
--- 5.0.3-1/include/CGAL/internal/Classification_type.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Classification_type.h	2020-09-08 12:14:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Alpha_shapes_3/include/CGAL/internal/Classification_type.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Alpha_shapes_3/include/CGAL/internal/Classification_type.h $
 // $Id: Classification_type.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Combination_enumerator.h 5.1-1/include/CGAL/internal/Combination_enumerator.h
--- 5.0.3-1/include/CGAL/internal/Combination_enumerator.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Combination_enumerator.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation/include/CGAL/internal/Combination_enumerator.h $
-// $Id: Combination_enumerator.h 5b2d46c 2020-04-21T10:03:00+02:00 Maxime Gimeno
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation/include/CGAL/internal/Combination_enumerator.h $
+// $Id: Combination_enumerator.h e9d41d7 2020-04-21T10:03:00+02:00 Maxime Gimeno
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)    : Samuel Hornus
diff -pruN 5.0.3-1/include/CGAL/internal/Combinatorial_map_copy_functors.h 5.1-1/include/CGAL/internal/Combinatorial_map_copy_functors.h
--- 5.0.3-1/include/CGAL/internal/Combinatorial_map_copy_functors.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Combinatorial_map_copy_functors.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Combinatorial_map/include/CGAL/internal/Combinatorial_map_copy_functors.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Combinatorial_map/include/CGAL/internal/Combinatorial_map_copy_functors.h $
 // $Id: Combinatorial_map_copy_functors.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Combinatorial_map_group_functors.h 5.1-1/include/CGAL/internal/Combinatorial_map_group_functors.h
--- 5.0.3-1/include/CGAL/internal/Combinatorial_map_group_functors.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Combinatorial_map_group_functors.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Combinatorial_map/include/CGAL/internal/Combinatorial_map_group_functors.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Combinatorial_map/include/CGAL/internal/Combinatorial_map_group_functors.h $
 // $Id: Combinatorial_map_group_functors.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Combinatorial_map_internal_functors.h 5.1-1/include/CGAL/internal/Combinatorial_map_internal_functors.h
--- 5.0.3-1/include/CGAL/internal/Combinatorial_map_internal_functors.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Combinatorial_map_internal_functors.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Combinatorial_map/include/CGAL/internal/Combinatorial_map_internal_functors.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Combinatorial_map/include/CGAL/internal/Combinatorial_map_internal_functors.h $
 // $Id: Combinatorial_map_internal_functors.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Combinatorial_map_sewable.h 5.1-1/include/CGAL/internal/Combinatorial_map_sewable.h
--- 5.0.3-1/include/CGAL/internal/Combinatorial_map_sewable.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Combinatorial_map_sewable.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Combinatorial_map/include/CGAL/internal/Combinatorial_map_sewable.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Combinatorial_map/include/CGAL/internal/Combinatorial_map_sewable.h $
 // $Id: Combinatorial_map_sewable.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Combinatorial_map_utility.h 5.1-1/include/CGAL/internal/Combinatorial_map_utility.h
--- 5.0.3-1/include/CGAL/internal/Combinatorial_map_utility.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Combinatorial_map_utility.h	2020-09-08 12:14:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Combinatorial_map/include/CGAL/internal/Combinatorial_map_utility.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Combinatorial_map/include/CGAL/internal/Combinatorial_map_utility.h $
 // $Id: Combinatorial_map_utility.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/compute_confidences.h 5.1-1/include/CGAL/internal/compute_confidences.h
--- 5.0.3-1/include/CGAL/internal/compute_confidences.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/internal/compute_confidences.h	2020-09-08 12:15:23.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygonal_surface_reconstruction/include/CGAL/internal/compute_confidences.h $
-// $Id: compute_confidences.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygonal_surface_reconstruction/include/CGAL/internal/compute_confidences.h $
+// $Id: compute_confidences.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s) : Liangliang Nan
@@ -24,17 +24,10 @@
 
 
 // Concurrency
-#ifdef CGAL_LINKED_WITH_TBB
-typedef CGAL::Parallel_tag Concurrency_tag;
-#else
-typedef CGAL::Sequential_tag Concurrency_tag;
-#endif
-
+typedef CGAL::Parallel_if_available_tag Concurrency_tag;
 
 namespace CGAL {
-
-
-        namespace internal {
+namespace internal {
 
                 /**
                 *        Computes the confidences of the candidate faces.
diff -pruN 5.0.3-1/include/CGAL/internal/container_fwd_fixed.hpp 5.1-1/include/CGAL/internal/container_fwd_fixed.hpp
--- 5.0.3-1/include/CGAL/internal/container_fwd_fixed.hpp	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/internal/container_fwd_fixed.hpp	2020-09-08 12:15:06.000000000 +0000
@@ -7,7 +7,7 @@
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at https://www.boost.org/LICENSE_1_0.txt)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_2/include/CGAL/internal/container_fwd_fixed.hpp $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_2/include/CGAL/internal/container_fwd_fixed.hpp $
 // $Id: container_fwd_fixed.hpp a472c24 2019-03-20T08:59:22+01:00 Andreas Fabri
 // SPDX-License-Identifier: BSL-1.0
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Dehn_hyperbolic_octagon_translation_word.h 5.1-1/include/CGAL/internal/Dehn_hyperbolic_octagon_translation_word.h
--- 5.0.3-1/include/CGAL/internal/Dehn_hyperbolic_octagon_translation_word.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Dehn_hyperbolic_octagon_translation_word.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_4_hyperbolic_triangulation_2/include/CGAL/internal/Dehn_hyperbolic_octagon_translation_word.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_4_hyperbolic_triangulation_2/include/CGAL/internal/Dehn_hyperbolic_octagon_translation_word.h $
 // $Id: Dehn_hyperbolic_octagon_translation_word.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Delaunay_triangulation_hierarchy_3.h 5.1-1/include/CGAL/internal/Delaunay_triangulation_hierarchy_3.h
--- 5.0.3-1/include/CGAL/internal/Delaunay_triangulation_hierarchy_3.h	2020-08-01 19:02:59.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Delaunay_triangulation_hierarchy_3.h	2020-09-08 12:15:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_3/include/CGAL/internal/Delaunay_triangulation_hierarchy_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_3/include/CGAL/internal/Delaunay_triangulation_hierarchy_3.h $
 // $Id: Delaunay_triangulation_hierarchy_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/deprecation_warning.h 5.1-1/include/CGAL/internal/deprecation_warning.h
--- 5.0.3-1/include/CGAL/internal/deprecation_warning.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/internal/deprecation_warning.h	2020-09-08 12:14:54.000000000 +0000
@@ -6,7 +6,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/internal/deprecation_warning.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/internal/deprecation_warning.h $
 // $Id: deprecation_warning.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/disable_deprecation_warnings_and_errors.h 5.1-1/include/CGAL/internal/disable_deprecation_warnings_and_errors.h
--- 5.0.3-1/include/CGAL/internal/disable_deprecation_warnings_and_errors.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/internal/disable_deprecation_warnings_and_errors.h	2020-09-08 12:14:54.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/internal/disable_deprecation_warnings_and_errors.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/internal/disable_deprecation_warnings_and_errors.h $
 // $Id: disable_deprecation_warnings_and_errors.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Dummy_tds_3.h 5.1-1/include/CGAL/internal/Dummy_tds_3.h
--- 5.0.3-1/include/CGAL/internal/Dummy_tds_3.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Dummy_tds_3.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/TDS_3/include/CGAL/internal/Dummy_tds_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/TDS_3/include/CGAL/internal/Dummy_tds_3.h $
 // $Id: Dummy_tds_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/enable_third_party_libraries.h 5.1-1/include/CGAL/internal/enable_third_party_libraries.h
--- 5.0.3-1/include/CGAL/internal/enable_third_party_libraries.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/internal/enable_third_party_libraries.h	2020-09-08 12:14:54.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/internal/enable_third_party_libraries.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/internal/enable_third_party_libraries.h $
 // $Id: enable_third_party_libraries.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Exact_complex.h 5.1-1/include/CGAL/internal/Exact_complex.h
--- 5.0.3-1/include/CGAL/internal/Exact_complex.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Exact_complex.h	2020-09-08 12:14:54.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Hyperbolic_triangulation_2/include/CGAL/internal/Exact_complex.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Hyperbolic_triangulation_2/include/CGAL/internal/Exact_complex.h $
 // $Id: Exact_complex.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Exact_type_selector.h 5.1-1/include/CGAL/internal/Exact_type_selector.h
--- 5.0.3-1/include/CGAL/internal/Exact_type_selector.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Exact_type_selector.h	2020-09-08 12:15:16.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/internal/Exact_type_selector.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/internal/Exact_type_selector.h $
 // $Id: Exact_type_selector.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Functor_with_offset_points_adaptor_2.h 5.1-1/include/CGAL/internal/Functor_with_offset_points_adaptor_2.h
--- 5.0.3-1/include/CGAL/internal/Functor_with_offset_points_adaptor_2.h	2020-08-01 19:02:38.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Functor_with_offset_points_adaptor_2.h	2020-09-08 12:15:19.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_2_triangulation_2/include/CGAL/internal/Functor_with_offset_points_adaptor_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_2_triangulation_2/include/CGAL/internal/Functor_with_offset_points_adaptor_2.h $
 // $Id: Functor_with_offset_points_adaptor_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Functor_with_offset_points_adaptor_3.h 5.1-1/include/CGAL/internal/Functor_with_offset_points_adaptor_3.h
--- 5.0.3-1/include/CGAL/internal/Functor_with_offset_points_adaptor_3.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Functor_with_offset_points_adaptor_3.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_triangulation_3/include/CGAL/internal/Functor_with_offset_points_adaptor_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_triangulation_3/include/CGAL/internal/Functor_with_offset_points_adaptor_3.h $
 // $Id: Functor_with_offset_points_adaptor_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Functor_with_offset_weighted_points_adaptor_3.h 5.1-1/include/CGAL/internal/Functor_with_offset_weighted_points_adaptor_3.h
--- 5.0.3-1/include/CGAL/internal/Functor_with_offset_weighted_points_adaptor_3.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Functor_with_offset_weighted_points_adaptor_3.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_triangulation_3/include/CGAL/internal/Functor_with_offset_weighted_points_adaptor_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_triangulation_3/include/CGAL/internal/Functor_with_offset_weighted_points_adaptor_3.h $
 // $Id: Functor_with_offset_weighted_points_adaptor_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Generalized_map_group_functors.h 5.1-1/include/CGAL/internal/Generalized_map_group_functors.h
--- 5.0.3-1/include/CGAL/internal/Generalized_map_group_functors.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Generalized_map_group_functors.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Generalized_map/include/CGAL/internal/Generalized_map_group_functors.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Generalized_map/include/CGAL/internal/Generalized_map_group_functors.h $
 // $Id: Generalized_map_group_functors.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Generalized_map_internal_functors.h 5.1-1/include/CGAL/internal/Generalized_map_internal_functors.h
--- 5.0.3-1/include/CGAL/internal/Generalized_map_internal_functors.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Generalized_map_internal_functors.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Generalized_map/include/CGAL/internal/Generalized_map_internal_functors.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Generalized_map/include/CGAL/internal/Generalized_map_internal_functors.h $
 // $Id: Generalized_map_internal_functors.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Generalized_map_sewable.h 5.1-1/include/CGAL/internal/Generalized_map_sewable.h
--- 5.0.3-1/include/CGAL/internal/Generalized_map_sewable.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Generalized_map_sewable.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Generalized_map/include/CGAL/internal/Generalized_map_sewable.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Generalized_map/include/CGAL/internal/Generalized_map_sewable.h $
 // $Id: Generalized_map_sewable.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Generic_random_point_generator.h 5.1-1/include/CGAL/internal/Generic_random_point_generator.h
--- 5.0.3-1/include/CGAL/internal/Generic_random_point_generator.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Generic_random_point_generator.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Generator/include/CGAL/internal/Generic_random_point_generator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Generator/include/CGAL/internal/Generic_random_point_generator.h $
 // $Id: Generic_random_point_generator.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Geometry_container.h 5.1-1/include/CGAL/internal/Geometry_container.h
--- 5.0.3-1/include/CGAL/internal/Geometry_container.h	2020-08-01 19:02:54.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Geometry_container.h	2020-09-08 12:15:42.000000000 +0000
@@ -6,7 +6,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/internal/Geometry_container.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/internal/Geometry_container.h $
 // $Id: Geometry_container.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Get_dimension_tag.h 5.1-1/include/CGAL/internal/Get_dimension_tag.h
--- 5.0.3-1/include/CGAL/internal/Get_dimension_tag.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Get_dimension_tag.h	2020-09-08 12:15:40.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_searching/include/CGAL/internal/Get_dimension_tag.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_searching/include/CGAL/internal/Get_dimension_tag.h $
 // $Id: Get_dimension_tag.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Has_boolean_tags.h 5.1-1/include/CGAL/internal/Has_boolean_tags.h
--- 5.0.3-1/include/CGAL/internal/Has_boolean_tags.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Has_boolean_tags.h	2020-09-08 12:15:02.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/internal/Has_boolean_tags.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/internal/Has_boolean_tags.h $
 // $Id: Has_boolean_tags.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Has_member_visited.h 5.1-1/include/CGAL/internal/Has_member_visited.h
--- 5.0.3-1/include/CGAL/internal/Has_member_visited.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Has_member_visited.h	2020-09-08 12:15:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/internal/Has_member_visited.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/internal/Has_member_visited.h $
 // $Id: Has_member_visited.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Has_nested_type_Bare_point.h 5.1-1/include/CGAL/internal/Has_nested_type_Bare_point.h
--- 5.0.3-1/include/CGAL/internal/Has_nested_type_Bare_point.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Has_nested_type_Bare_point.h	2020-09-08 12:15:32.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/internal/Has_nested_type_Bare_point.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/internal/Has_nested_type_Bare_point.h $
 // $Id: Has_nested_type_Bare_point.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Hyperbolic_Delaunay_triangulation_traits_2_functions.h 5.1-1/include/CGAL/internal/Hyperbolic_Delaunay_triangulation_traits_2_functions.h
--- 5.0.3-1/include/CGAL/internal/Hyperbolic_Delaunay_triangulation_traits_2_functions.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Hyperbolic_Delaunay_triangulation_traits_2_functions.h	2020-09-08 12:14:54.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Hyperbolic_triangulation_2/include/CGAL/internal/Hyperbolic_Delaunay_triangulation_traits_2_functions.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Hyperbolic_triangulation_2/include/CGAL/internal/Hyperbolic_Delaunay_triangulation_traits_2_functions.h $
 // $Id: Hyperbolic_Delaunay_triangulation_traits_2_functions.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Hyperbolic_octagon_translation_matrix.h 5.1-1/include/CGAL/internal/Hyperbolic_octagon_translation_matrix.h
--- 5.0.3-1/include/CGAL/internal/Hyperbolic_octagon_translation_matrix.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Hyperbolic_octagon_translation_matrix.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_4_hyperbolic_triangulation_2/include/CGAL/internal/Hyperbolic_octagon_translation_matrix.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_4_hyperbolic_triangulation_2/include/CGAL/internal/Hyperbolic_octagon_translation_matrix.h $
 // $Id: Hyperbolic_octagon_translation_matrix.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Hyperbolic_octagon_translation_word.h 5.1-1/include/CGAL/internal/Hyperbolic_octagon_translation_word.h
--- 5.0.3-1/include/CGAL/internal/Hyperbolic_octagon_translation_word.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Hyperbolic_octagon_translation_word.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_4_hyperbolic_triangulation_2/include/CGAL/internal/Hyperbolic_octagon_translation_word.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_4_hyperbolic_triangulation_2/include/CGAL/internal/Hyperbolic_octagon_translation_word.h $
 // $Id: Hyperbolic_octagon_translation_word.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/hypothesis.h 5.1-1/include/CGAL/internal/hypothesis.h
--- 5.0.3-1/include/CGAL/internal/hypothesis.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/internal/hypothesis.h	2020-09-08 12:15:23.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygonal_surface_reconstruction/include/CGAL/internal/hypothesis.h $
-// $Id: hypothesis.h 5b2d46c 2020-04-21T10:03:00+02:00 Maxime Gimeno
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygonal_surface_reconstruction/include/CGAL/internal/hypothesis.h $
+// $Id: hypothesis.h e9d41d7 2020-04-21T10:03:00+02:00 Maxime Gimeno
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s) : Liangliang Nan
diff -pruN 5.0.3-1/include/CGAL/internal/info_check.h 5.1-1/include/CGAL/internal/info_check.h
--- 5.0.3-1/include/CGAL/internal/info_check.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/internal/info_check.h	2020-09-08 12:15:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/internal/info_check.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/internal/info_check.h $
 // $Id: info_check.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/K_neighbor_search.h 5.1-1/include/CGAL/internal/K_neighbor_search.h
--- 5.0.3-1/include/CGAL/internal/K_neighbor_search.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/internal/K_neighbor_search.h	2020-09-08 12:15:40.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_searching/include/CGAL/internal/K_neighbor_search.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_searching/include/CGAL/internal/K_neighbor_search.h $
 // $Id: K_neighbor_search.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Lazy_alpha_nt_2.h 5.1-1/include/CGAL/internal/Lazy_alpha_nt_2.h
--- 5.0.3-1/include/CGAL/internal/Lazy_alpha_nt_2.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Lazy_alpha_nt_2.h	2020-09-08 12:14:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Alpha_shapes_2/include/CGAL/internal/Lazy_alpha_nt_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Alpha_shapes_2/include/CGAL/internal/Lazy_alpha_nt_2.h $
 // $Id: Lazy_alpha_nt_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Lazy_alpha_nt_3.h 5.1-1/include/CGAL/internal/Lazy_alpha_nt_3.h
--- 5.0.3-1/include/CGAL/internal/Lazy_alpha_nt_3.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Lazy_alpha_nt_3.h	2020-09-08 12:14:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Alpha_shapes_3/include/CGAL/internal/Lazy_alpha_nt_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Alpha_shapes_3/include/CGAL/internal/Lazy_alpha_nt_3.h $
 // $Id: Lazy_alpha_nt_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Mesh_3/Boundary_of_subdomain_of_complex_3_in_triangulation_3_to_off.h 5.1-1/include/CGAL/internal/Mesh_3/Boundary_of_subdomain_of_complex_3_in_triangulation_3_to_off.h
--- 5.0.3-1/include/CGAL/internal/Mesh_3/Boundary_of_subdomain_of_complex_3_in_triangulation_3_to_off.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Mesh_3/Boundary_of_subdomain_of_complex_3_in_triangulation_3_to_off.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/internal/Mesh_3/Boundary_of_subdomain_of_complex_3_in_triangulation_3_to_off.h $
-// $Id: Boundary_of_subdomain_of_complex_3_in_triangulation_3_to_off.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/internal/Mesh_3/Boundary_of_subdomain_of_complex_3_in_triangulation_3_to_off.h $
+// $Id: Boundary_of_subdomain_of_complex_3_in_triangulation_3_to_off.h 4dda7b6 2020-05-27T15:53:05+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Laurent Rineau and Sebastien Loriot
@@ -12,7 +12,7 @@
 #ifndef CGAL_INTERNAL_MESH_3_BOUNDARY_OF_SUDDOMAIN_OF_COMPLEX_3_IN_TRIANGULATION_3_TO_OFF_H
 #define CGAL_INTERNAL_MESH_3_BOUNDARY_OF_SUDDOMAIN_OF_COMPLEX_3_IN_TRIANGULATION_3_TO_OFF_H
 
-#include <CGAL/license/Mesh_3.h>
+#include <CGAL/license/Triangulation_3.h>
 
 #include <CGAL/IO/facets_in_complex_3_to_triangle_mesh.h>
 
diff -pruN 5.0.3-1/include/CGAL/internal/Mesh_3/check_weights.h 5.1-1/include/CGAL/internal/Mesh_3/check_weights.h
--- 5.0.3-1/include/CGAL/internal/Mesh_3/check_weights.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Mesh_3/check_weights.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/internal/Mesh_3/check_weights.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/internal/Mesh_3/check_weights.h $
 // $Id: check_weights.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Mesh_3/get_index.h 5.1-1/include/CGAL/internal/Mesh_3/get_index.h
--- 5.0.3-1/include/CGAL/internal/Mesh_3/get_index.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Mesh_3/get_index.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/internal/Mesh_3/get_index.h $
-// $Id: get_index.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/internal/Mesh_3/get_index.h $
+// $Id: get_index.h 4dda7b6 2020-05-27T15:53:05+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Laurent Rineau
@@ -17,7 +17,7 @@
 #ifndef CGAL_INTERNAL_MESH_3_GET_INDEX_3_H
 #define CGAL_INTERNAL_MESH_3_GET_INDEX_3_H
 
-#include <CGAL/license/Mesh_3.h>
+#include <CGAL/license/Triangulation_3.h>
 
 #include <CGAL/internal/Mesh_3/indices_management.h>
 
diff -pruN 5.0.3-1/include/CGAL/internal/Mesh_3/Graph_manipulations.h 5.1-1/include/CGAL/internal/Mesh_3/Graph_manipulations.h
--- 5.0.3-1/include/CGAL/internal/Mesh_3/Graph_manipulations.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Mesh_3/Graph_manipulations.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/internal/Mesh_3/Graph_manipulations.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/internal/Mesh_3/Graph_manipulations.h $
 // $Id: Graph_manipulations.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Mesh_3/Handle_IO_for_pair_of_int.h 5.1-1/include/CGAL/internal/Mesh_3/Handle_IO_for_pair_of_int.h
--- 5.0.3-1/include/CGAL/internal/Mesh_3/Handle_IO_for_pair_of_int.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Mesh_3/Handle_IO_for_pair_of_int.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/internal/Mesh_3/Handle_IO_for_pair_of_int.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/internal/Mesh_3/Handle_IO_for_pair_of_int.h $
 // $Id: Handle_IO_for_pair_of_int.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Mesh_3/helpers.h 5.1-1/include/CGAL/internal/Mesh_3/helpers.h
--- 5.0.3-1/include/CGAL/internal/Mesh_3/helpers.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Mesh_3/helpers.h	2020-09-08 12:15:09.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/internal/Mesh_3/helpers.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/internal/Mesh_3/helpers.h $
 // $Id: helpers.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Mesh_3/indices_management.h 5.1-1/include/CGAL/internal/Mesh_3/indices_management.h
--- 5.0.3-1/include/CGAL/internal/Mesh_3/indices_management.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Mesh_3/indices_management.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/internal/Mesh_3/indices_management.h $
-// $Id: indices_management.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/internal/Mesh_3/indices_management.h $
+// $Id: indices_management.h 4dda7b6 2020-05-27T15:53:05+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Stéphane Tayeb
@@ -18,7 +18,7 @@
 #ifndef CGAL_INTERNAL_MESH_3_INDICES_MANAGEMENT_H
 #define CGAL_INTERNAL_MESH_3_INDICES_MANAGEMENT_H
 
-#include <CGAL/license/Mesh_3.h>
+#include <CGAL/license/Triangulation_3.h>
 
 
 #include <boost/type_traits/is_same.hpp>
diff -pruN 5.0.3-1/include/CGAL/internal/parameters.h 5.1-1/include/CGAL/internal/parameters.h
--- 5.0.3-1/include/CGAL/internal/parameters.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/internal/parameters.h	2020-09-08 12:15:23.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygonal_surface_reconstruction/include/CGAL/internal/parameters.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygonal_surface_reconstruction/include/CGAL/internal/parameters.h $
 // $Id: parameters.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Periodic_2_construct_point_2.h 5.1-1/include/CGAL/internal/Periodic_2_construct_point_2.h
--- 5.0.3-1/include/CGAL/internal/Periodic_2_construct_point_2.h	2020-08-01 19:02:38.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Periodic_2_construct_point_2.h	2020-09-08 12:15:19.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_2_triangulation_2/include/CGAL/internal/Periodic_2_construct_point_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_2_triangulation_2/include/CGAL/internal/Periodic_2_construct_point_2.h $
 // $Id: Periodic_2_construct_point_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Periodic_2_Delaunay_triangulation_filtered_traits_2.h 5.1-1/include/CGAL/internal/Periodic_2_Delaunay_triangulation_filtered_traits_2.h
--- 5.0.3-1/include/CGAL/internal/Periodic_2_Delaunay_triangulation_filtered_traits_2.h	2020-08-01 19:02:38.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Periodic_2_Delaunay_triangulation_filtered_traits_2.h	2020-09-08 12:15:19.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_2_triangulation_2/include/CGAL/internal/Periodic_2_Delaunay_triangulation_filtered_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_2_triangulation_2/include/CGAL/internal/Periodic_2_Delaunay_triangulation_filtered_traits_2.h $
 // $Id: Periodic_2_Delaunay_triangulation_filtered_traits_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Periodic_2_Delaunay_triangulation_statically_filtered_traits_2.h 5.1-1/include/CGAL/internal/Periodic_2_Delaunay_triangulation_statically_filtered_traits_2.h
--- 5.0.3-1/include/CGAL/internal/Periodic_2_Delaunay_triangulation_statically_filtered_traits_2.h	2020-08-01 19:02:38.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Periodic_2_Delaunay_triangulation_statically_filtered_traits_2.h	2020-09-08 12:15:19.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_2_triangulation_2/include/CGAL/internal/Periodic_2_Delaunay_triangulation_statically_filtered_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_2_triangulation_2/include/CGAL/internal/Periodic_2_Delaunay_triangulation_statically_filtered_traits_2.h $
 // $Id: Periodic_2_Delaunay_triangulation_statically_filtered_traits_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Periodic_2_triangulation_filtered_traits_2.h 5.1-1/include/CGAL/internal/Periodic_2_triangulation_filtered_traits_2.h
--- 5.0.3-1/include/CGAL/internal/Periodic_2_triangulation_filtered_traits_2.h	2020-08-01 19:02:38.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Periodic_2_triangulation_filtered_traits_2.h	2020-09-08 12:15:19.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_2_triangulation_2/include/CGAL/internal/Periodic_2_triangulation_filtered_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_2_triangulation_2/include/CGAL/internal/Periodic_2_triangulation_filtered_traits_2.h $
 // $Id: Periodic_2_triangulation_filtered_traits_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Periodic_2_triangulation_statically_filtered_traits_2.h 5.1-1/include/CGAL/internal/Periodic_2_triangulation_statically_filtered_traits_2.h
--- 5.0.3-1/include/CGAL/internal/Periodic_2_triangulation_statically_filtered_traits_2.h	2020-08-01 19:02:38.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Periodic_2_triangulation_statically_filtered_traits_2.h	2020-09-08 12:15:19.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_2_triangulation_2/include/CGAL/internal/Periodic_2_triangulation_statically_filtered_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_2_triangulation_2/include/CGAL/internal/Periodic_2_triangulation_statically_filtered_traits_2.h $
 // $Id: Periodic_2_triangulation_statically_filtered_traits_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Periodic_3_construct_point_3.h 5.1-1/include/CGAL/internal/Periodic_3_construct_point_3.h
--- 5.0.3-1/include/CGAL/internal/Periodic_3_construct_point_3.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Periodic_3_construct_point_3.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_triangulation_3/include/CGAL/internal/Periodic_3_construct_point_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_triangulation_3/include/CGAL/internal/Periodic_3_construct_point_3.h $
 // $Id: Periodic_3_construct_point_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Periodic_3_construct_weighted_point_3.h 5.1-1/include/CGAL/internal/Periodic_3_construct_weighted_point_3.h
--- 5.0.3-1/include/CGAL/internal/Periodic_3_construct_weighted_point_3.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Periodic_3_construct_weighted_point_3.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_triangulation_3/include/CGAL/internal/Periodic_3_construct_weighted_point_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_triangulation_3/include/CGAL/internal/Periodic_3_construct_weighted_point_3.h $
 // $Id: Periodic_3_construct_weighted_point_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Periodic_3_Delaunay_triangulation_filtered_traits_3.h 5.1-1/include/CGAL/internal/Periodic_3_Delaunay_triangulation_filtered_traits_3.h
--- 5.0.3-1/include/CGAL/internal/Periodic_3_Delaunay_triangulation_filtered_traits_3.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Periodic_3_Delaunay_triangulation_filtered_traits_3.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_triangulation_3/include/CGAL/internal/Periodic_3_Delaunay_triangulation_filtered_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_triangulation_3/include/CGAL/internal/Periodic_3_Delaunay_triangulation_filtered_traits_3.h $
 // $Id: Periodic_3_Delaunay_triangulation_filtered_traits_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Periodic_3_Delaunay_triangulation_remove_traits_3.h 5.1-1/include/CGAL/internal/Periodic_3_Delaunay_triangulation_remove_traits_3.h
--- 5.0.3-1/include/CGAL/internal/Periodic_3_Delaunay_triangulation_remove_traits_3.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Periodic_3_Delaunay_triangulation_remove_traits_3.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_triangulation_3/include/CGAL/internal/Periodic_3_Delaunay_triangulation_remove_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_triangulation_3/include/CGAL/internal/Periodic_3_Delaunay_triangulation_remove_traits_3.h $
 // $Id: Periodic_3_Delaunay_triangulation_remove_traits_3.h 2e8a59d 2020-07-21T15:25:54+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Periodic_3_Delaunay_triangulation_statically_filtered_traits_3.h 5.1-1/include/CGAL/internal/Periodic_3_Delaunay_triangulation_statically_filtered_traits_3.h
--- 5.0.3-1/include/CGAL/internal/Periodic_3_Delaunay_triangulation_statically_filtered_traits_3.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Periodic_3_Delaunay_triangulation_statically_filtered_traits_3.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_triangulation_3/include/CGAL/internal/Periodic_3_Delaunay_triangulation_statically_filtered_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_triangulation_3/include/CGAL/internal/Periodic_3_Delaunay_triangulation_statically_filtered_traits_3.h $
 // $Id: Periodic_3_Delaunay_triangulation_statically_filtered_traits_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Periodic_3_regular_triangulation_dummy_288.h 5.1-1/include/CGAL/internal/Periodic_3_regular_triangulation_dummy_288.h
--- 5.0.3-1/include/CGAL/internal/Periodic_3_regular_triangulation_dummy_288.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Periodic_3_regular_triangulation_dummy_288.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_triangulation_3/include/CGAL/internal/Periodic_3_regular_triangulation_dummy_288.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_triangulation_3/include/CGAL/internal/Periodic_3_regular_triangulation_dummy_288.h $
 // $Id: Periodic_3_regular_triangulation_dummy_288.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Periodic_3_regular_triangulation_filtered_traits_3.h 5.1-1/include/CGAL/internal/Periodic_3_regular_triangulation_filtered_traits_3.h
--- 5.0.3-1/include/CGAL/internal/Periodic_3_regular_triangulation_filtered_traits_3.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Periodic_3_regular_triangulation_filtered_traits_3.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_triangulation_3/include/CGAL/internal/Periodic_3_regular_triangulation_filtered_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_triangulation_3/include/CGAL/internal/Periodic_3_regular_triangulation_filtered_traits_3.h $
 // $Id: Periodic_3_regular_triangulation_filtered_traits_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Periodic_3_regular_triangulation_remove_traits_3.h 5.1-1/include/CGAL/internal/Periodic_3_regular_triangulation_remove_traits_3.h
--- 5.0.3-1/include/CGAL/internal/Periodic_3_regular_triangulation_remove_traits_3.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Periodic_3_regular_triangulation_remove_traits_3.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_triangulation_3/include/CGAL/internal/Periodic_3_regular_triangulation_remove_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_triangulation_3/include/CGAL/internal/Periodic_3_regular_triangulation_remove_traits_3.h $
 // $Id: Periodic_3_regular_triangulation_remove_traits_3.h 2e8a59d 2020-07-21T15:25:54+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Periodic_3_regular_triangulation_statically_filtered_traits_3.h 5.1-1/include/CGAL/internal/Periodic_3_regular_triangulation_statically_filtered_traits_3.h
--- 5.0.3-1/include/CGAL/internal/Periodic_3_regular_triangulation_statically_filtered_traits_3.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Periodic_3_regular_triangulation_statically_filtered_traits_3.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_triangulation_3/include/CGAL/internal/Periodic_3_regular_triangulation_statically_filtered_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_triangulation_3/include/CGAL/internal/Periodic_3_regular_triangulation_statically_filtered_traits_3.h $
 // $Id: Periodic_3_regular_triangulation_statically_filtered_traits_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Periodic_3_triangulation_dummy_36.h 5.1-1/include/CGAL/internal/Periodic_3_triangulation_dummy_36.h
--- 5.0.3-1/include/CGAL/internal/Periodic_3_triangulation_dummy_36.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Periodic_3_triangulation_dummy_36.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_triangulation_3/include/CGAL/internal/Periodic_3_triangulation_dummy_36.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_triangulation_3/include/CGAL/internal/Periodic_3_triangulation_dummy_36.h $
 // $Id: Periodic_3_triangulation_dummy_36.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Periodic_3_triangulation_filtered_traits_3.h 5.1-1/include/CGAL/internal/Periodic_3_triangulation_filtered_traits_3.h
--- 5.0.3-1/include/CGAL/internal/Periodic_3_triangulation_filtered_traits_3.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Periodic_3_triangulation_filtered_traits_3.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_triangulation_3/include/CGAL/internal/Periodic_3_triangulation_filtered_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_triangulation_3/include/CGAL/internal/Periodic_3_triangulation_filtered_traits_3.h $
 // $Id: Periodic_3_triangulation_filtered_traits_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Periodic_3_triangulation_iterators_3.h 5.1-1/include/CGAL/internal/Periodic_3_triangulation_iterators_3.h
--- 5.0.3-1/include/CGAL/internal/Periodic_3_triangulation_iterators_3.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Periodic_3_triangulation_iterators_3.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_triangulation_3/include/CGAL/internal/Periodic_3_triangulation_iterators_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_triangulation_3/include/CGAL/internal/Periodic_3_triangulation_iterators_3.h $
 // $Id: Periodic_3_triangulation_iterators_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Periodic_3_triangulation_remove_traits_3.h 5.1-1/include/CGAL/internal/Periodic_3_triangulation_remove_traits_3.h
--- 5.0.3-1/include/CGAL/internal/Periodic_3_triangulation_remove_traits_3.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Periodic_3_triangulation_remove_traits_3.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_triangulation_3/include/CGAL/internal/Periodic_3_triangulation_remove_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_triangulation_3/include/CGAL/internal/Periodic_3_triangulation_remove_traits_3.h $
 // $Id: Periodic_3_triangulation_remove_traits_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Periodic_3_triangulation_statically_filtered_traits_3.h 5.1-1/include/CGAL/internal/Periodic_3_triangulation_statically_filtered_traits_3.h
--- 5.0.3-1/include/CGAL/internal/Periodic_3_triangulation_statically_filtered_traits_3.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Periodic_3_triangulation_statically_filtered_traits_3.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_triangulation_3/include/CGAL/internal/Periodic_3_triangulation_statically_filtered_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_triangulation_3/include/CGAL/internal/Periodic_3_triangulation_statically_filtered_traits_3.h $
 // $Id: Periodic_3_triangulation_statically_filtered_traits_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Periodic_4_hyperbolic_triangulation_dummy_14.h 5.1-1/include/CGAL/internal/Periodic_4_hyperbolic_triangulation_dummy_14.h
--- 5.0.3-1/include/CGAL/internal/Periodic_4_hyperbolic_triangulation_dummy_14.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Periodic_4_hyperbolic_triangulation_dummy_14.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_4_hyperbolic_triangulation_2/include/CGAL/internal/Periodic_4_hyperbolic_triangulation_dummy_14.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_4_hyperbolic_triangulation_2/include/CGAL/internal/Periodic_4_hyperbolic_triangulation_dummy_14.h $
 // $Id: Periodic_4_hyperbolic_triangulation_dummy_14.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/point_set_with_planes.h 5.1-1/include/CGAL/internal/point_set_with_planes.h
--- 5.0.3-1/include/CGAL/internal/point_set_with_planes.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/internal/point_set_with_planes.h	2020-09-08 12:15:23.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygonal_surface_reconstruction/include/CGAL/internal/point_set_with_planes.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygonal_surface_reconstruction/include/CGAL/internal/point_set_with_planes.h $
 // $Id: point_set_with_planes.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Projection_traits_3.h 5.1-1/include/CGAL/internal/Projection_traits_3.h
--- 5.0.3-1/include/CGAL/internal/Projection_traits_3.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Projection_traits_3.h	2020-09-08 12:15:02.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/internal/Projection_traits_3.h $
-// $Id: Projection_traits_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/internal/Projection_traits_3.h $
+// $Id: Projection_traits_3.h 9f2eafd 2020-03-26T19:17:02+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Mariette Yvinec, Sebastien Loriot, Mael Rouxel-Labbé
@@ -521,7 +521,7 @@ public:
 
   Weighted_point_2 project(const Weighted_point_3& wp) const
   {
-    Point_3 p = R().construct_point_3_object()(wp);
+    const Point_3& p = R().construct_point_3_object()(wp);
     return Weighted_point_2(Point_2(x(p), y(p)), wp.weight());
   }
 
@@ -550,7 +550,7 @@ public:
 
   Weighted_point_2 project(const Weighted_point_3& wp) const
   {
-    Point_3 p = R().construct_point_3_object()(wp);
+    const Point_3& p = R().construct_point_3_object()(wp);
     return Weighted_point_2(Point_2(x(p), y(p)), wp.weight());
   }
 
@@ -577,7 +577,7 @@ public:
 
   Weighted_point_2 project(const Weighted_point_3& wp) const
   {
-    Point_3 p = R().construct_point_3_object()(wp);
+    const Point_3& p = R().construct_point_3_object()(wp);
     return Weighted_point_2(Point_2(x(p), y(p)), wp.weight());
   }
 
@@ -614,7 +614,7 @@ public:
 
   Weighted_point_2 project(const Weighted_point_3& wp) const
   {
-    Point_3 p = R().construct_point_3_object()(wp);
+    const Point_3& p = R().construct_point_3_object()(wp);
     return Weighted_point_2(Point_2(x(p), y(p)), wp.weight());
   }
 
@@ -659,7 +659,7 @@ public:
 
   Weighted_point_2 project(const Weighted_point_3& wp) const
   {
-    Point_3 p = R().construct_point_3_object()(wp);
+    const Point_3& p = R().construct_point_3_object()(wp);
     return Weighted_point_2(Point_2(x(p), y(p)), wp.weight());
   }
 
@@ -697,7 +697,7 @@ public:
 
   Weighted_point_2 project(const Weighted_point_3& wp) const
   {
-    Point_3 p = R().construct_point_3_object()(wp);
+    const Point_3& p = R().construct_point_3_object()(wp);
     return Weighted_point_2(Point_2(x(p), y(p)), wp.weight());
   }
 
@@ -739,7 +739,7 @@ public:
 
   Weighted_point_2 project(const Weighted_point_3& wp) const
   {
-    Point_3 p = R().construct_point_3_object()(wp);
+    const Point_3& p = R().construct_point_3_object()(wp);
     return Weighted_point_2(Point_2(x(p), y(p)), wp.weight());
   }
 
diff -pruN 5.0.3-1/include/CGAL/internal/Robust_periodic_weighted_circumcenter_traits_3.h 5.1-1/include/CGAL/internal/Robust_periodic_weighted_circumcenter_traits_3.h
--- 5.0.3-1/include/CGAL/internal/Robust_periodic_weighted_circumcenter_traits_3.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Robust_periodic_weighted_circumcenter_traits_3.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_triangulation_3/include/CGAL/internal/Robust_periodic_weighted_circumcenter_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_triangulation_3/include/CGAL/internal/Robust_periodic_weighted_circumcenter_traits_3.h $
 // $Id: Robust_periodic_weighted_circumcenter_traits_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Search_helpers.h 5.1-1/include/CGAL/internal/Search_helpers.h
--- 5.0.3-1/include/CGAL/internal/Search_helpers.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Search_helpers.h	2020-09-08 12:15:40.000000000 +0000
@@ -0,0 +1,240 @@
+// Copyright (c) 2002,2011 Utrecht University (The Netherlands).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_searching/include/CGAL/internal/Search_helpers.h $
+// $Id: Search_helpers.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Clement Jamin (clement.jamin.pro@gmail.com)
+
+#ifndef CGAL_INTERNAL_SEARCH_HELPERS_H
+#define CGAL_INTERNAL_SEARCH_HELPERS_H
+
+#include <CGAL/license/Spatial_searching.h>
+
+#include <CGAL/Has_member.h>
+
+#include <boost/mpl/has_xxx.hpp>
+
+namespace CGAL {
+namespace internal {
+
+// Helper struct to know at compile-time if there is a cache of the points
+// stored in the tree
+template <typename Tree, bool has_enable_points_cache>
+struct Has_points_cache;
+
+template <typename Tree>
+struct Has_points_cache<Tree, true>
+{
+  typedef typename Tree::Enable_points_cache type;
+  static const bool value = type::value;
+};
+
+template <typename Tree>
+struct Has_points_cache<Tree, false>
+{
+  typedef Tag_false type;
+  static const bool value = false;
+};
+
+CGAL_GENERATE_MEMBER_DETECTOR(transformed_distance_from_coordinates);
+CGAL_GENERATE_MEMBER_DETECTOR(interruptible_transformed_distance);
+BOOST_MPL_HAS_XXX_TRAIT_NAMED_DEF(has_Enable_points_cache, Enable_points_cache, false)
+
+
+
+// If transformed_distance_from_coordinates does not exist in `Distance`
+template <typename Distance, typename SearchTraits, bool has_transformed_distance_from_coordinates>
+class Transformed_distance_from_coordinates
+{
+  typedef typename Distance::FT         FT;
+  typedef typename Distance::Point_d    Point;
+  typedef typename Distance::Query_item Query_item;
+
+public:
+  Transformed_distance_from_coordinates(Distance const& distance)
+    : m_distance(distance)
+  {}
+
+  FT operator() (
+    const Query_item& q,
+    Point const& p,
+    typename std::vector<FT>::const_iterator /*it_coord_begin*/,
+    typename std::vector<FT>::const_iterator /*it_coord_end*/) const
+  {
+    return m_distance.transformed_distance(q, p);
+  }
+
+private:
+  Distance const& m_distance;
+};
+// ... or if it exists
+template <typename Distance, typename SearchTraits>
+class Transformed_distance_from_coordinates<Distance, SearchTraits, true>
+{
+  typedef typename Distance::FT         FT;
+  typedef typename Distance::Point_d    Point;
+  typedef typename Distance::Query_item Query_item;
+
+public:
+  Transformed_distance_from_coordinates(Distance const& distance)
+    : m_distance(distance)
+  {}
+
+  FT operator() (
+    const Query_item& q,
+    Point const& /*p*/,
+    typename std::vector<FT>::const_iterator it_coord_begin,
+    typename std::vector<FT>::const_iterator it_coord_end) const
+  {
+    return m_distance.transformed_distance_from_coordinates(q, it_coord_begin, it_coord_end);
+  }
+
+private:
+  Distance const& m_distance;
+};
+
+// If interruptible_transformed_distance does not exist in `Distance`
+template <typename Distance, typename SearchTraits, bool has_interruptible_transformed_distance>
+class Interruptible_transformed_distance
+{
+  typedef typename Distance::FT         FT;
+  typedef typename Distance::Point_d    Point;
+  typedef typename Distance::Query_item Query_item;
+
+public:
+  typedef Transformed_distance_from_coordinates<Distance, SearchTraits,
+    has_transformed_distance_from_coordinates<Distance>::value> Tdfc;
+
+  Interruptible_transformed_distance(
+    SearchTraits const&, Distance const& distance, Tdfc const& tdfc)
+    : m_distance(distance), m_ref_to_tdfc(tdfc)
+  {}
+
+  FT operator() (
+    const Query_item& q,
+    Point const& p,
+    FT) const
+  {
+    return m_distance.transformed_distance(q, p);
+  }
+
+  FT operator() (
+    const Query_item& q,
+    Point const& p,
+    typename std::vector<FT>::const_iterator it_coord_begin,
+    typename std::vector<FT>::const_iterator it_coord_end,
+    FT) const
+  {
+    return m_ref_to_tdfc(q, p, it_coord_begin, it_coord_end);
+  }
+
+private:
+  Distance const& m_distance;
+  Tdfc const& m_ref_to_tdfc;
+};
+// ... or if it exists
+template <typename Distance, typename SearchTraits>
+class Interruptible_transformed_distance<Distance, SearchTraits, true>
+{
+  typedef typename Distance::FT         FT;
+  typedef typename Distance::Point_d    Point;
+  typedef typename Distance::Query_item Query_item;
+
+public:
+  typedef Transformed_distance_from_coordinates<Distance, SearchTraits,
+    has_transformed_distance_from_coordinates<Distance>::value> Tdfc;
+
+  Interruptible_transformed_distance(
+    SearchTraits const& traits, Distance const& distance, Tdfc const&)
+    : m_traits(traits), m_distance(distance)
+  {}
+
+  FT operator() (
+    const Query_item& q,
+    Point const& p,
+    FT stop_if_geq_to_this) const
+  {
+    typename SearchTraits::Construct_cartesian_const_iterator_d construct_it =
+      m_traits.construct_cartesian_const_iterator_d_object();
+    return m_distance.interruptible_transformed_distance(
+      q, construct_it(p), construct_it(p, 0), stop_if_geq_to_this);
+  }
+
+  FT operator() (
+    const Query_item& q,
+    Point const& /*p*/,
+    typename std::vector<FT>::const_iterator it_coord_begin,
+    typename std::vector<FT>::const_iterator it_coord_end,
+    FT stop_if_geq_to_this) const
+  {
+    return m_distance.interruptible_transformed_distance(
+      q, it_coord_begin, it_coord_end, stop_if_geq_to_this);
+  }
+
+private:
+  SearchTraits const& m_traits;
+  Distance const& m_distance;
+};
+
+
+
+template <typename Distance, typename SearchTraits>
+class Distance_helper
+{
+  typedef typename Distance::FT         FT;
+  typedef typename Distance::Point_d    Point;
+  typedef typename Distance::Query_item Query_item;
+
+public:
+
+  Distance_helper(Distance const& distance, SearchTraits const& traits)
+    : m_distance(distance), m_tdfc(m_distance), m_itd(traits, distance, m_tdfc)
+  {}
+
+  FT
+  transformed_distance_from_coordinates(
+    const Query_item& q,
+    Point const& p,
+    typename std::vector<FT>::const_iterator it_coord_begin,
+    typename std::vector<FT>::const_iterator it_coord_end)
+  {
+    return m_tdfc(q, p, it_coord_begin, it_coord_end);
+  }
+
+  FT
+  interruptible_transformed_distance(
+    const Query_item& q,
+    Point const& p,
+    FT stop_if_geq_to_this)
+  {
+    return m_itd(q, p, stop_if_geq_to_this);
+  }
+
+  FT
+  interruptible_transformed_distance(
+    const Query_item& q,
+    Point const& p,
+    typename std::vector<FT>::const_iterator it_coord_begin,
+    typename std::vector<FT>::const_iterator it_coord_end,
+    FT stop_if_geq_to_this)
+  {
+    return m_itd(q, p, it_coord_begin, it_coord_end, stop_if_geq_to_this);
+  }
+
+private:
+  Distance     const& m_distance;
+  Transformed_distance_from_coordinates<Distance, SearchTraits,
+    has_transformed_distance_from_coordinates<Distance>::value> m_tdfc;
+  Interruptible_transformed_distance<Distance, SearchTraits,
+    has_interruptible_transformed_distance<Distance>::value> m_itd;
+}; // Distance_helper
+
+
+}} // namespace CGAL::internal
+
+#endif  // CGAL_INTERNAL_SEARCH_HELPERSS_H
diff -pruN 5.0.3-1/include/CGAL/internal/Static_filters/Angle_3.h 5.1-1/include/CGAL/internal/Static_filters/Angle_3.h
--- 5.0.3-1/include/CGAL/internal/Static_filters/Angle_3.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Static_filters/Angle_3.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/include/CGAL/internal/Static_filters/Angle_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/internal/Static_filters/Angle_3.h $
 // $Id: Angle_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Static_filters/Collinear_3.h 5.1-1/include/CGAL/internal/Static_filters/Collinear_3.h
--- 5.0.3-1/include/CGAL/internal/Static_filters/Collinear_3.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Static_filters/Collinear_3.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/include/CGAL/internal/Static_filters/Collinear_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/internal/Static_filters/Collinear_3.h $
 // $Id: Collinear_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Static_filters/Compare_squared_radius_3.h 5.1-1/include/CGAL/internal/Static_filters/Compare_squared_radius_3.h
--- 5.0.3-1/include/CGAL/internal/Static_filters/Compare_squared_radius_3.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Static_filters/Compare_squared_radius_3.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/include/CGAL/internal/Static_filters/Compare_squared_radius_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/internal/Static_filters/Compare_squared_radius_3.h $
 // $Id: Compare_squared_radius_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Static_filters/Compare_weighted_squared_radius_3.h 5.1-1/include/CGAL/internal/Static_filters/Compare_weighted_squared_radius_3.h
--- 5.0.3-1/include/CGAL/internal/Static_filters/Compare_weighted_squared_radius_3.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Static_filters/Compare_weighted_squared_radius_3.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/include/CGAL/internal/Static_filters/Compare_weighted_squared_radius_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/internal/Static_filters/Compare_weighted_squared_radius_3.h $
 // $Id: Compare_weighted_squared_radius_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Static_filters/Compare_x_2.h 5.1-1/include/CGAL/internal/Static_filters/Compare_x_2.h
--- 5.0.3-1/include/CGAL/internal/Static_filters/Compare_x_2.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Static_filters/Compare_x_2.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/include/CGAL/internal/Static_filters/Compare_x_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/internal/Static_filters/Compare_x_2.h $
 // $Id: Compare_x_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Static_filters/Compare_y_2.h 5.1-1/include/CGAL/internal/Static_filters/Compare_y_2.h
--- 5.0.3-1/include/CGAL/internal/Static_filters/Compare_y_2.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Static_filters/Compare_y_2.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/include/CGAL/internal/Static_filters/Compare_y_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/internal/Static_filters/Compare_y_2.h $
 // $Id: Compare_y_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Static_filters/Compare_y_at_x_2.h 5.1-1/include/CGAL/internal/Static_filters/Compare_y_at_x_2.h
--- 5.0.3-1/include/CGAL/internal/Static_filters/Compare_y_at_x_2.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Static_filters/Compare_y_at_x_2.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/include/CGAL/internal/Static_filters/Compare_y_at_x_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/internal/Static_filters/Compare_y_at_x_2.h $
 // $Id: Compare_y_at_x_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Static_filters/Coplanar_3.h 5.1-1/include/CGAL/internal/Static_filters/Coplanar_3.h
--- 5.0.3-1/include/CGAL/internal/Static_filters/Coplanar_3.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Static_filters/Coplanar_3.h	2020-09-08 12:14:51.000000000 +0000
@@ -0,0 +1,54 @@
+// Copyright (c) 2011 GeometryFactory (France)
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/internal/Static_filters/Coplanar_3.h $
+// $Id: Coplanar_3.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
+// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Andreas Fabri, Laurent Rineau
+
+
+#ifndef CGAL_INTERNAL_STATIC_FILTERS_COPLANAR_3_H
+#define CGAL_INTERNAL_STATIC_FILTERS_COPLANAR_3_H
+
+namespace CGAL {
+
+namespace internal {
+
+namespace Static_filters_predicates {
+
+
+template < typename K_base, typename SFK >
+class Coplanar_3
+  : public K_base::Coplanar_3
+{
+  typedef typename K_base::Point_3 Point_3;
+  typedef typename K_base::Coplanar_3 Base;
+  typedef typename SFK::Orientation_3 Orientation_3;
+
+public:
+
+  typedef typename Base::result_type  result_type;
+
+
+
+  result_type
+  operator()(const Point_3& p,const Point_3& q, const Point_3& r, const Point_3& s) const
+  {
+    return Orientation_3()(p,q,r,s) == COPLANAR;
+  }
+
+
+
+}; // end class Coplanar_3
+
+} // end namespace Static_filters_predicates
+
+} // end namespace internal
+
+} // end namespace CGAL
+
+#endif  // CGAL_INTERNAL_STATIC_FILTERS_COPLANAR_3_H
diff -pruN 5.0.3-1/include/CGAL/internal/Static_filters/Coplanar_orientation_3.h 5.1-1/include/CGAL/internal/Static_filters/Coplanar_orientation_3.h
--- 5.0.3-1/include/CGAL/internal/Static_filters/Coplanar_orientation_3.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Static_filters/Coplanar_orientation_3.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/include/CGAL/internal/Static_filters/Coplanar_orientation_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/internal/Static_filters/Coplanar_orientation_3.h $
 // $Id: Coplanar_orientation_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Static_filters/Coplanar_side_of_bounded_circle_3.h 5.1-1/include/CGAL/internal/Static_filters/Coplanar_side_of_bounded_circle_3.h
--- 5.0.3-1/include/CGAL/internal/Static_filters/Coplanar_side_of_bounded_circle_3.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Static_filters/Coplanar_side_of_bounded_circle_3.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/include/CGAL/internal/Static_filters/Coplanar_side_of_bounded_circle_3.h $
-// $Id: Coplanar_side_of_bounded_circle_3.h 5b2d46c 2020-04-21T10:03:00+02:00 Maxime Gimeno
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/internal/Static_filters/Coplanar_side_of_bounded_circle_3.h $
+// $Id: Coplanar_side_of_bounded_circle_3.h e9d41d7 2020-04-21T10:03:00+02:00 Maxime Gimeno
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Static_filters/Do_intersect_2.h 5.1-1/include/CGAL/internal/Static_filters/Do_intersect_2.h
--- 5.0.3-1/include/CGAL/internal/Static_filters/Do_intersect_2.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Static_filters/Do_intersect_2.h	2020-09-08 12:14:51.000000000 +0000
@@ -5,7 +5,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/include/CGAL/internal/Static_filters/Do_intersect_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/internal/Static_filters/Do_intersect_2.h $
 // $Id: Do_intersect_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Static_filters/Do_intersect_3.h 5.1-1/include/CGAL/internal/Static_filters/Do_intersect_3.h
--- 5.0.3-1/include/CGAL/internal/Static_filters/Do_intersect_3.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Static_filters/Do_intersect_3.h	2020-09-08 12:14:51.000000000 +0000
@@ -5,7 +5,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/include/CGAL/internal/Static_filters/Do_intersect_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/internal/Static_filters/Do_intersect_3.h $
 // $Id: Do_intersect_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Static_filters/Equal_2.h 5.1-1/include/CGAL/internal/Static_filters/Equal_2.h
--- 5.0.3-1/include/CGAL/internal/Static_filters/Equal_2.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Static_filters/Equal_2.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/include/CGAL/internal/Static_filters/Equal_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/internal/Static_filters/Equal_2.h $
 // $Id: Equal_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Static_filters/Equal_3.h 5.1-1/include/CGAL/internal/Static_filters/Equal_3.h
--- 5.0.3-1/include/CGAL/internal/Static_filters/Equal_3.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Static_filters/Equal_3.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/include/CGAL/internal/Static_filters/Equal_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/internal/Static_filters/Equal_3.h $
 // $Id: Equal_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Static_filters/Is_degenerate_3.h 5.1-1/include/CGAL/internal/Static_filters/Is_degenerate_3.h
--- 5.0.3-1/include/CGAL/internal/Static_filters/Is_degenerate_3.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Static_filters/Is_degenerate_3.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/include/CGAL/internal/Static_filters/Is_degenerate_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/internal/Static_filters/Is_degenerate_3.h $
 // $Id: Is_degenerate_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Static_filters/Orientation_2.h 5.1-1/include/CGAL/internal/Static_filters/Orientation_2.h
--- 5.0.3-1/include/CGAL/internal/Static_filters/Orientation_2.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Static_filters/Orientation_2.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/include/CGAL/internal/Static_filters/Orientation_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/internal/Static_filters/Orientation_2.h $
 // $Id: Orientation_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Static_filters/Orientation_3.h 5.1-1/include/CGAL/internal/Static_filters/Orientation_3.h
--- 5.0.3-1/include/CGAL/internal/Static_filters/Orientation_3.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Static_filters/Orientation_3.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/include/CGAL/internal/Static_filters/Orientation_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/internal/Static_filters/Orientation_3.h $
 // $Id: Orientation_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Static_filters/Periodic_2_orientation_2.h 5.1-1/include/CGAL/internal/Static_filters/Periodic_2_orientation_2.h
--- 5.0.3-1/include/CGAL/internal/Static_filters/Periodic_2_orientation_2.h	2020-08-01 19:02:38.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Static_filters/Periodic_2_orientation_2.h	2020-09-08 12:15:19.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_2_triangulation_2/include/CGAL/internal/Static_filters/Periodic_2_orientation_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_2_triangulation_2/include/CGAL/internal/Static_filters/Periodic_2_orientation_2.h $
 // $Id: Periodic_2_orientation_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Static_filters/Periodic_2_side_of_oriented_circle_2.h 5.1-1/include/CGAL/internal/Static_filters/Periodic_2_side_of_oriented_circle_2.h
--- 5.0.3-1/include/CGAL/internal/Static_filters/Periodic_2_side_of_oriented_circle_2.h	2020-08-01 19:02:38.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Static_filters/Periodic_2_side_of_oriented_circle_2.h	2020-09-08 12:15:19.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_2_triangulation_2/include/CGAL/internal/Static_filters/Periodic_2_side_of_oriented_circle_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_2_triangulation_2/include/CGAL/internal/Static_filters/Periodic_2_side_of_oriented_circle_2.h $
 // $Id: Periodic_2_side_of_oriented_circle_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Static_filters/Periodic_3_orientation_3.h 5.1-1/include/CGAL/internal/Static_filters/Periodic_3_orientation_3.h
--- 5.0.3-1/include/CGAL/internal/Static_filters/Periodic_3_orientation_3.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Static_filters/Periodic_3_orientation_3.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_triangulation_3/include/CGAL/internal/Static_filters/Periodic_3_orientation_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_triangulation_3/include/CGAL/internal/Static_filters/Periodic_3_orientation_3.h $
 // $Id: Periodic_3_orientation_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Static_filters/Periodic_3_power_side_of_oriented_power_sphere_3.h 5.1-1/include/CGAL/internal/Static_filters/Periodic_3_power_side_of_oriented_power_sphere_3.h
--- 5.0.3-1/include/CGAL/internal/Static_filters/Periodic_3_power_side_of_oriented_power_sphere_3.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Static_filters/Periodic_3_power_side_of_oriented_power_sphere_3.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_triangulation_3/include/CGAL/internal/Static_filters/Periodic_3_power_side_of_oriented_power_sphere_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_triangulation_3/include/CGAL/internal/Static_filters/Periodic_3_power_side_of_oriented_power_sphere_3.h $
 // $Id: Periodic_3_power_side_of_oriented_power_sphere_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Static_filters/Periodic_3_side_of_oriented_sphere_3.h 5.1-1/include/CGAL/internal/Static_filters/Periodic_3_side_of_oriented_sphere_3.h
--- 5.0.3-1/include/CGAL/internal/Static_filters/Periodic_3_side_of_oriented_sphere_3.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Static_filters/Periodic_3_side_of_oriented_sphere_3.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_triangulation_3/include/CGAL/internal/Static_filters/Periodic_3_side_of_oriented_sphere_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_triangulation_3/include/CGAL/internal/Static_filters/Periodic_3_side_of_oriented_sphere_3.h $
 // $Id: Periodic_3_side_of_oriented_sphere_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Static_filters/Power_side_of_oriented_power_sphere_3.h 5.1-1/include/CGAL/internal/Static_filters/Power_side_of_oriented_power_sphere_3.h
--- 5.0.3-1/include/CGAL/internal/Static_filters/Power_side_of_oriented_power_sphere_3.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Static_filters/Power_side_of_oriented_power_sphere_3.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/include/CGAL/internal/Static_filters/Power_side_of_oriented_power_sphere_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/internal/Static_filters/Power_side_of_oriented_power_sphere_3.h $
 // $Id: Power_side_of_oriented_power_sphere_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Static_filters/Side_of_oriented_circle_2.h 5.1-1/include/CGAL/internal/Static_filters/Side_of_oriented_circle_2.h
--- 5.0.3-1/include/CGAL/internal/Static_filters/Side_of_oriented_circle_2.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Static_filters/Side_of_oriented_circle_2.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/include/CGAL/internal/Static_filters/Side_of_oriented_circle_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/internal/Static_filters/Side_of_oriented_circle_2.h $
 // $Id: Side_of_oriented_circle_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Static_filters/Side_of_oriented_sphere_3.h 5.1-1/include/CGAL/internal/Static_filters/Side_of_oriented_sphere_3.h
--- 5.0.3-1/include/CGAL/internal/Static_filters/Side_of_oriented_sphere_3.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Static_filters/Side_of_oriented_sphere_3.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/include/CGAL/internal/Static_filters/Side_of_oriented_sphere_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/internal/Static_filters/Side_of_oriented_sphere_3.h $
 // $Id: Side_of_oriented_sphere_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Static_filters/Static_filter_error.h 5.1-1/include/CGAL/internal/Static_filters/Static_filter_error.h
--- 5.0.3-1/include/CGAL/internal/Static_filters/Static_filter_error.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Static_filters/Static_filter_error.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/include/CGAL/internal/Static_filters/Static_filter_error.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/internal/Static_filters/Static_filter_error.h $
 // $Id: Static_filter_error.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Static_filters/Static_filters.h 5.1-1/include/CGAL/internal/Static_filters/Static_filters.h
--- 5.0.3-1/include/CGAL/internal/Static_filters/Static_filters.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Static_filters/Static_filters.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/include/CGAL/internal/Static_filters/Static_filters.h $
-// $Id: Static_filters.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/internal/Static_filters/Static_filters.h $
+// $Id: Static_filters.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -41,7 +41,6 @@
 
 #endif // CGAL_DISABLE_STATIC_FILTERS_ADDED_2011
 
-
 #ifndef CGAL_NO_EQUAL_3_STATIC_FILTERS
 #  include <CGAL/internal/Static_filters/Equal_3.h>
 #  include <CGAL/internal/Static_filters/Equal_2.h>
@@ -65,6 +64,7 @@
 #  include <CGAL/internal/Static_filters/Do_intersect_2.h>
 #endif // NOT NOT CGAL_NO_DO_INTERSECT_STATIC_FILTERS
 
+#include <CGAL/internal/Static_filters/Coplanar_3.h>
 #include <CGAL/internal/Static_filters/Compare_y_at_x_2.h>
 #include <CGAL/internal/Static_filters/Side_of_oriented_circle_2.h>
 #include <CGAL/internal/Static_filters/Side_of_oriented_sphere_3.h>
@@ -121,6 +121,7 @@ public:
   typedef Static_filters_predicates::Side_of_oriented_circle_2<K_base>      Side_of_oriented_circle_2;
   typedef Static_filters_predicates::Side_of_oriented_sphere_3<K_base>      Side_of_oriented_sphere_3;
   typedef Static_filters_predicates::Compare_squared_radius_3<K_base>       Compare_squared_radius_3;
+  typedef Static_filters_predicates::Coplanar_3<K_base,Self>                Coplanar_3;
 
   typedef Static_filters_predicates::Compare_weighted_squared_radius_3<K_base>     Compare_weighted_squared_radius_3;
   typedef Static_filters_predicates::Power_side_of_oriented_power_sphere_3<K_base>                          Power_side_of_oriented_power_sphere_3;
@@ -138,28 +139,28 @@ public:
   { return Collinear_3(); }
 
 #ifndef CGAL_NO_EQUAL_3_STATIC_FILTERS
- Equal_2
+  Equal_2
   equal_2_object() const
   { return Equal_2(); }
 
- Equal_3
+  Equal_3
   equal_3_object() const
   { return Equal_3(); }
 #endif // NOT CGAL_NO_EQUAL_3_STATIC_FILTERS
 
 #ifndef CGAL_NO_COMPARE_X_2_STATIC_FILTERS
- Compare_x_2
+  Compare_x_2
   compare_x_2_object() const
   { return Compare_x_2(); }
 
-Compare_y_2
+  Compare_y_2
   compare_y_2_object() const
   { return Compare_y_2(); }
 
 #endif // NOT CGAL_NO_COMPARE_Y_2_STATIC_FILTERS
 
 #ifndef CGAL_NO_IS_DEGENERATE_3_STATIC_FILTERS
- Is_degenerate_3
+  Is_degenerate_3
   is_degenerate_3_object() const
   { return Is_degenerate_3(); }
 #endif // NOT CGAL_NO_IS_DEGENERATE_3_STATIC_FILTERS
@@ -182,7 +183,12 @@ Compare_y_2
   compare_squared_radius_3_object() const
   { return Compare_squared_radius_3(); }
 
-  Power_side_of_oriented_power_sphere_3 power_side_of_oriented_power_sphere_3_object() const
+  Coplanar_3
+  coplanar_3_object() const
+  { return Coplanar_3(); }
+
+  Power_side_of_oriented_power_sphere_3
+  power_side_of_oriented_power_sphere_3_object() const
   { return Power_side_of_oriented_power_sphere_3();}
 
   Compare_weighted_squared_radius_3
diff -pruN 5.0.3-1/include/CGAL/internal/Static_filters/tools.h 5.1-1/include/CGAL/internal/Static_filters/tools.h
--- 5.0.3-1/include/CGAL/internal/Static_filters/tools.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Static_filters/tools.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/include/CGAL/internal/Static_filters/tools.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/internal/Static_filters/tools.h $
 // $Id: tools.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Static_or_dynamic_array.h 5.1-1/include/CGAL/internal/Static_or_dynamic_array.h
--- 5.0.3-1/include/CGAL/internal/Static_or_dynamic_array.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Static_or_dynamic_array.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation/include/CGAL/internal/Static_or_dynamic_array.h $
-// $Id: Static_or_dynamic_array.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation/include/CGAL/internal/Static_or_dynamic_array.h $
+// $Id: Static_or_dynamic_array.h 378554e 2020-03-06T16:42:23+01:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)    : Samuel Hornus
@@ -66,9 +66,9 @@ struct S_or_D_array< Containee, Dimensio
     {
         return (*this)[0].for_compact_container();
     }
-    void* & for_compact_container()
+    void    for_compact_container(void *p)
     {
-        return (*this)[0].for_compact_container();
+        (*this)[0].for_compact_container(p);
     }
 };
 
@@ -101,7 +101,7 @@ struct S_or_D_array< Containee, Dynamic_
     {}
     void* fcc_;
     void*   for_compact_container() const { return fcc_; }
-    void* & for_compact_container()       { return fcc_; }
+    void    for_compact_container(void* p)       { fcc_ = p; }
 };
 
 } // end of namespace internal
diff -pruN 5.0.3-1/include/CGAL/internal/Surface_mesh_deformation/Spokes_and_rims_iterator.h 5.1-1/include/CGAL/internal/Surface_mesh_deformation/Spokes_and_rims_iterator.h
--- 5.0.3-1/include/CGAL/internal/Surface_mesh_deformation/Spokes_and_rims_iterator.h	2020-08-01 19:02:54.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Surface_mesh_deformation/Spokes_and_rims_iterator.h	2020-09-08 12:15:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_deformation/include/CGAL/internal/Surface_mesh_deformation/Spokes_and_rims_iterator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_deformation/include/CGAL/internal/Surface_mesh_deformation/Spokes_and_rims_iterator.h $
 // $Id: Spokes_and_rims_iterator.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Surface_mesh_segmentation/AABB_traits.h 5.1-1/include/CGAL/internal/Surface_mesh_segmentation/AABB_traits.h
--- 5.0.3-1/include/CGAL/internal/Surface_mesh_segmentation/AABB_traits.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Surface_mesh_segmentation/AABB_traits.h	2020-09-08 12:15:44.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_segmentation/include/CGAL/internal/Surface_mesh_segmentation/AABB_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_segmentation/include/CGAL/internal/Surface_mesh_segmentation/AABB_traits.h $
 // $Id: AABB_traits.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Surface_mesh_segmentation/AABB_traversal_traits.h 5.1-1/include/CGAL/internal/Surface_mesh_segmentation/AABB_traversal_traits.h
--- 5.0.3-1/include/CGAL/internal/Surface_mesh_segmentation/AABB_traversal_traits.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Surface_mesh_segmentation/AABB_traversal_traits.h	2020-09-08 12:15:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_segmentation/include/CGAL/internal/Surface_mesh_segmentation/AABB_traversal_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_segmentation/include/CGAL/internal/Surface_mesh_segmentation/AABB_traversal_traits.h $
 // $Id: AABB_traversal_traits.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Surface_mesh_segmentation/Alpha_expansion_graph_cut.h 5.1-1/include/CGAL/internal/Surface_mesh_segmentation/Alpha_expansion_graph_cut.h
--- 5.0.3-1/include/CGAL/internal/Surface_mesh_segmentation/Alpha_expansion_graph_cut.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Surface_mesh_segmentation/Alpha_expansion_graph_cut.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,687 +0,0 @@
-#ifndef CGAL_SURFACE_MESH_SEGMENTATION_ALPHA_EXPANSION_GRAPH_CUT_H
-// Copyright (c) 2014  GeometryFactory Sarl (France).
-// All rights reserved.
-//
-// This file is part of CGAL (www.cgal.org).
-//
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_segmentation/include/CGAL/internal/Surface_mesh_segmentation/Alpha_expansion_graph_cut.h $
-// $Id: Alpha_expansion_graph_cut.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
-// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
-//
-// Author(s)     : Ilker O. Yaz
-
-#define CGAL_SURFACE_MESH_SEGMENTATION_ALPHA_EXPANSION_GRAPH_CUT_H
-
-#include <CGAL/license/Surface_mesh_segmentation.h>
-
-
-/// @cond CGAL_DOCUMENT_INTERNAL
-
-/**
- * @file Alpha_expansion_graph_cut.h
- * @brief This file contains 3 graph-cut algorithms, which can be used as a template parameter for CGAL::internal::Surface_mesh_segmentation.
- *
- * Main differences between implementations are underlying max-flow algorithm and graph type (i.e. results are the same, performance differs).
- *
- * By default, we use MAXFLOW and the class Alpha_expansion_graph_cut_boykov_kolmogorov.
- * For deactivating MAXFLOW software and using boost implementation instead, define CGAL_DO_NOT_USE_BOYKOV_KOLMOGOROV_MAXFLOW_SOFTWARE.
- * It deactivates Alpha_expansion_graph_cut_boykov_kolmogorov, activate boost versions
- * and makes CGAL::internal::Surface_mesh_segmentation using Alpha_expansion_graph_cut_boost
- * as default implementation for the graph-cut.
- *
- * Also algorithms can be used by their-own for applying alpha-expansion graph-cut on any graph.
- *
- */
-#include <CGAL/assertions.h>
-#ifdef CGAL_SEGMENTATION_BENCH_GRAPHCUT
-#include <CGAL/Timer.h>
-#endif
-#include <CGAL/IO/trace.h>
-
-#include <CGAL/boost/graph/Named_function_parameters.h>
-
-#include <boost/version.hpp>
-#ifdef CGAL_DO_NOT_USE_BOYKOV_KOLMOGOROV_MAXFLOW_SOFTWARE
-#include <boost/graph/adjacency_list.hpp>
-#include <boost/graph/compressed_sparse_row_graph.hpp>
-#if BOOST_VERSION >= 104400 // at this version kolmogorov_max_flow become depricated.
-#include <boost/graph/boykov_kolmogorov_max_flow.hpp>
-#else
-#include <boost/graph/kolmogorov_max_flow.hpp>
-#endif
-#else
-namespace MaxFlow
-{
-#include <CGAL/internal/auxiliary/graph.h>
-}
-#endif
-
-#include <vector>
-
-
-
-
-namespace CGAL
-{
-namespace internal
-{
-
-////////////////////////////////////////////////////////////////////////////////////////
-//   Comments about performance:
-//
-// 1) With BGL:
-//     * Using adjacency_list:
-//     ** Without pre-allocating vertex-list
-//       | OutEdgeList | VertexList | Performance |
-//       |    listS    |   listS    |     25.2    |
-//       |    vecS     |   listS    |     22.7    |
-//       |    listS    |   vecS     |     30.7    |
-//       |    vecS     |   vecS     |     26.1    |
-//
-//     ** With pre-allocating vertex-list with max-node size
-//        (Note: exact number of vertices are not certain at the beginning)
-//       | OutEdgeList | VertexList | Performance |
-//       |    listS    |   vecS     |     25.2    |
-//       |    vecS     |   vecS     |     23.4    |
-//
-//     * Didn't try adjacency_matrix since our graph is sparse
-//     ( Also one can check BGL book, performance section )
-//
-//    Decision:
-//     * Alpha_expansion_graph_cut_boost: use adjacency_list<vecS, listS> without
-//       pre-allocating vertex-list.
-//
-// 2) With Boykov-Kolmogorov MAXFLOW software:
-//   (http://pub.ist.ac.at/~vnk/software/maxflow-v2.21.src.tar.gz)
-//                                  | Performance |
-//                                  |     3.1     |
-//     * Alpha_expansion_graph_cut_boykov_kolmogorov provides an implementation.
-//       MAXFLOW does not provide any option for pre-allocation (It is possible with v_3.02 though).
-//
-// Typical Benchmark result provided by Ilker
-//                                 | construction of vertices  |  construction of edges    | graph cut  | Total
-//   -----------------------------------------------------------------------------------------------------------
-//   boost with an adjacency list  |         1.53              | 1.51                      |  3.00      | 6.04
-//   boost with CSR                | 0.11 (gather in a vector) | 0.15 (gather in a vector) |  2.67      | 2.93
-//   MaxFlow                       |       0.042               | 0.076                     |  1.043     | 1.161
-//
-// The main issue for now with CSR is the construction of the opposite edge map that is too costly,
-// since it is done by exploring all edges to find opposite
-////////////////////////////////////////////////////////////////////////////////////////
-
-#ifdef CGAL_DO_NOT_USE_BOYKOV_KOLMOGOROV_MAXFLOW_SOFTWARE
-/**
- * @brief Implements alpha-expansion graph cut algorithm.
- *
- * For representing graph, it uses adjacency_list with OutEdgeList = vecS, VertexList = listS.
- * Also no pre-allocation is made for vertex-list.
- */
-class Alpha_expansion_graph_cut_boost
-{
-private:
-  typedef boost::adjacency_list_traits<boost::vecS, boost::listS, boost::directedS>
-  Adjacency_list_traits;
-
-  typedef boost::adjacency_list<boost::vecS, boost::listS, boost::directedS,
-          // 4 vertex properties
-          boost::property<boost::vertex_index_t, std::size_t,
-          boost::property<boost::vertex_color_t, boost::default_color_type,
-          boost::property<boost::vertex_distance_t, double,
-          boost::property<boost::vertex_predecessor_t, Adjacency_list_traits::edge_descriptor >
-          > > >,
-          // 3 edge properties
-          boost::property<boost::edge_capacity_t, double,
-          boost::property<boost::edge_residual_capacity_t, double,
-          boost::property<boost::edge_reverse_t, Adjacency_list_traits::edge_descriptor> >
-          > > Graph;
-
-  typedef boost::graph_traits<Graph> Traits;
-  typedef boost::color_traits<boost::default_color_type> ColorTraits;
-
-  typedef Traits::vertex_descriptor Vertex_descriptor;
-  typedef Traits::vertex_iterator   Vertex_iterator;
-  typedef Traits::edge_descriptor   Edge_descriptor;
-  typedef Traits::edge_iterator     Edge_iterator;
-
-  /**
-   * Adds two directional edges between @a v1 and @a v2
-   * @param v1 first vertex
-   * @param v2 second vertex
-   * @param w1 weight for edge from v1 to v2 (v1->v2)
-   * @param w2 weight for edge from v2 to v1 (v2->v1)
-   * @param graph to be added
-   * @return pair of added edges, first: v1->v2 and second: v2->v1
-   */
-  boost::tuple<Edge_descriptor, Edge_descriptor>
-  add_edge_and_reverse(Vertex_descriptor& v1, Vertex_descriptor& v2, double w1,
-                       double w2, Graph& graph) const {
-    Edge_descriptor v1_v2, v2_v1;
-    bool v1_v2_added, v2_v1_added;
-
-    boost::tie(v1_v2, v1_v2_added) = boost::add_edge(v1, v2, graph);
-    boost::tie(v2_v1, v2_v1_added) = boost::add_edge(v2, v1, graph);
-
-    CGAL_assertion(v1_v2_added && v2_v1_added);
-    //put edge capacities
-    boost::put(boost::edge_reverse, graph, v1_v2, v2_v1);
-    boost::put(boost::edge_reverse, graph, v2_v1, v1_v2);
-
-    //map reverse edges
-    boost::put(boost::edge_capacity, graph, v1_v2, w1);
-    boost::put(boost::edge_capacity, graph, v2_v1, w2);
-
-    return boost::make_tuple(v1_v2, v2_v1);
-  }
-
-public:
-  /**
-   * Applies alpha-expansion graph-cut for energy minimization.
-   * @param edges contains incident vertex-id pairs for each edge (vertex-ids should be between [0, number of vertices -1])
-   * @param edge_weights contains weights for each edge in @a edges (correspondence according to order)
-   * @param probability_matrix contains responsibility of the center on the vertex probability[center][vertex]
-   * @param[in, out] labels as input it contains initial labeling of vertices (i.e. a center-id between [0, number of centers -1]),
-   * and as output it returns final labeling of vertices (i.e. assigned cluster-id to each facet)
-   * @return result of energy function
-   */
-  double operator()(const std::vector<std::pair<std::size_t, std::size_t> >&
-                    edges,
-                    const std::vector<double>& edge_weights,
-                    const std::vector<std::vector<double> >& probability_matrix,
-                    std::vector<std::size_t>& labels) const {
-    const double tolerance = 1e-10;
-
-    double min_cut = (std::numeric_limits<double>::max)();
-
-    #ifdef CGAL_SEGMENTATION_BENCH_GRAPHCUT
-    double vertex_creation_time, edge_creation_time, cut_time;
-    vertex_creation_time = edge_creation_time = cut_time = 0.0;
-    #endif
-
-    std::vector<Vertex_descriptor> inserted_vertices;
-    inserted_vertices.resize(labels.size());
-
-    Graph graph;
-
-    bool success;
-    do {
-      success = false;
-      std::size_t alpha = 0;
-
-      for(std::vector<std::vector<double> >::const_iterator it =
-            probability_matrix.begin();
-          it != probability_matrix.end(); ++it, ++alpha) {
-        graph.clear();
-
-        Vertex_descriptor cluster_source = boost::add_vertex(graph);
-        Vertex_descriptor cluster_sink = boost::add_vertex(graph);
-
-        #ifdef CGAL_SEGMENTATION_BENCH_GRAPHCUT
-        Timer timer;
-        timer.start();
-        #endif
-
-        // For E-Data
-        // add every facet as a vertex to the graph, put edges to source & sink vertices
-        for(std::size_t vertex_i = 0; vertex_i < labels.size(); ++vertex_i) {
-          Vertex_descriptor new_vertex = boost::add_vertex(graph);
-          inserted_vertices[vertex_i] = new_vertex;
-          double source_weight = probability_matrix[alpha][vertex_i];
-          // since it is expansion move, current alpha labeled vertices will be assigned to alpha again,
-          // making sink_weight 'infinity' guarantee this.
-          double sink_weight = (labels[vertex_i] == alpha) ?
-                               (std::numeric_limits<double>::max)()
-                               : probability_matrix[labels[vertex_i]][vertex_i];
-
-          add_edge_and_reverse(cluster_source, new_vertex, source_weight, 0.0, graph);
-          add_edge_and_reverse(new_vertex, cluster_sink, sink_weight, 0.0, graph);
-        }
-        #ifdef CGAL_SEGMENTATION_BENCH_GRAPHCUT
-        vertex_creation_time += timer.time();
-        timer.reset();
-        #endif
-
-        // For E-Smooth
-        // add edge between every vertex,
-        std::vector<double>::const_iterator weight_it = edge_weights.begin();
-        for(std::vector<std::pair<std::size_t, std::size_t> >::const_iterator edge_it =
-              edges.begin(); edge_it != edges.end();
-            ++edge_it, ++weight_it) {
-          Vertex_descriptor v1 = inserted_vertices[edge_it->first],
-                            v2 = inserted_vertices[edge_it->second];
-          std::size_t label_1 = labels[edge_it->first], label_2 = labels[edge_it->second];
-          if(label_1 == label_2) {
-            if(label_1 != alpha) {
-              add_edge_and_reverse(v1, v2, *weight_it, *weight_it, graph);
-            }
-          } else {
-            Vertex_descriptor inbetween = boost::add_vertex(graph);
-
-            double w1 = (label_1 == alpha) ? 0 : *weight_it;
-            double w2 = (label_2 == alpha) ? 0 : *weight_it;
-            add_edge_and_reverse(inbetween, v1, w1, w1, graph);
-            add_edge_and_reverse(inbetween, v2, w2, w2, graph);
-            add_edge_and_reverse(inbetween, cluster_sink, *weight_it, 0.0, graph);
-          }
-        }
-        #ifdef CGAL_SEGMENTATION_BENCH_GRAPHCUT
-        edge_creation_time += timer.time();
-        #endif
-
-        // initialize vertex indices, it is necessary since we are using VertexList = listS
-        Vertex_iterator v_begin, v_end;
-        Traits::vertices_size_type index = 0;
-        for(boost::tie(v_begin, v_end) = vertices(graph); v_begin != v_end; ++v_begin) {
-          boost::put(boost::vertex_index, graph, *v_begin, index++);
-        }
-
-        #ifdef CGAL_SEGMENTATION_BENCH_GRAPHCUT
-        timer.reset();
-        #endif
-#if BOOST_VERSION >= 104400
-        double flow = boost::boykov_kolmogorov_max_flow(graph, cluster_source,
-                      cluster_sink);
-#else
-        double flow = boost::kolmogorov_max_flow(graph, cluster_source, cluster_sink);
-#endif
-        #ifdef CGAL_SEGMENTATION_BENCH_GRAPHCUT
-        cut_time += timer.time();
-        #endif
-
-        if(min_cut - flow <= flow * tolerance) {
-          continue;
-        }
-        min_cut = flow;
-        success = true;
-        //update labeling
-        for(std::size_t vertex_i = 0; vertex_i < inserted_vertices.size(); ++vertex_i) {
-          boost::default_color_type color = boost::get(boost::vertex_color, graph,
-                                            inserted_vertices[vertex_i]);
-          if(labels[vertex_i] != alpha
-              && color == ColorTraits::white()) { //new comers (expansion occurs)
-            labels[vertex_i] = alpha;
-          }
-        }
-      }
-    } while(success);
-
-    #ifdef CGAL_SEGMENTATION_BENCH_GRAPHCUT
-    CGAL_TRACE_STREAM << "vertex creation time: " << vertex_creation_time <<
-                      std::endl;
-    CGAL_TRACE_STREAM << "edge creation time: " << edge_creation_time << std::endl;
-    CGAL_TRACE_STREAM << "max flow algorithm time: " << cut_time << std::endl;
-    #endif
-
-    return min_cut;
-  }
-};
-
-// another implementation using compressed_sparse_row_graph
-// for now there is a performance problem while setting reverse edges
-// if that can be solved, it is faster than Alpha_expansion_graph_cut_boost
-class Alpha_expansion_graph_cut_boost_CSR
-{
-private:
-  // CSR only accepts bundled props
-  struct VertexP {
-    boost::default_color_type vertex_color;
-    double vertex_distance_t;
-    // ? do not now there is another way to take it, I think since edge_descriptor does not rely on properties
-    // this should be fine...
-    boost::compressed_sparse_row_graph<boost::directedS>::edge_descriptor
-    vertex_predecessor;
-  };
-
-  struct EdgeP {
-    double edge_capacity;
-    double edge_residual_capacity;
-    boost::compressed_sparse_row_graph<boost::directedS>::edge_descriptor
-    edge_reverse;
-  };
-
-  typedef boost::compressed_sparse_row_graph<boost::directedS,
-          VertexP, EdgeP> Graph;
-
-  typedef boost::graph_traits<Graph> Traits;
-  typedef boost::color_traits<boost::default_color_type> ColorTraits;
-
-  typedef Traits::vertex_descriptor Vertex_descriptor;
-  typedef Traits::vertex_iterator   Vertex_iterator;
-  typedef Traits::edge_descriptor   Edge_descriptor;
-  typedef Traits::edge_iterator     Edge_iterator;
-
-  void
-  add_edge_and_reverse(std::size_t v1 , std::size_t v2, double w1, double w2,
-                       std::vector<std::pair<std::size_t, std::size_t> >& edge_map,
-                       std::vector<EdgeP>& edge_weights) const {
-    edge_map.push_back(std::make_pair(v1, v2));
-    EdgeP p1;
-    p1.edge_capacity = w1;
-    edge_weights.push_back(p1);
-
-    edge_map.push_back(std::make_pair(v2, v1));
-    EdgeP p2;
-    p2.edge_capacity = w2;
-    edge_weights.push_back(p2);
-  }
-
-public:
-  /**
-   * Applies alpha-expansion graph-cut for energy minimization.
-   * @param edges contains incident vertex-id pairs for each edge (vertex-ids should be between [0, number of vertices -1])
-   * @param edge_weights contains weights for each edge in @a edges (correspondence according to order)
-   * @param probability_matrix contains responsibility of the center on the vertex probability[center][vertex]
-   * @param[in, out] labels as input it contains initial labeling of vertices (i.e. a center-id between [0, number of centers -1]),
-   * and as output it returns final labeling of vertices (i.e. assigned cluster-id to each facet)
-   * @return result of energy function
-   */
-  double operator()(const std::vector<std::pair<std::size_t, std::size_t> >&
-                    edges,
-                    const std::vector<double>& edge_weights,
-                    const std::vector<std::vector<double> >& probability_matrix,
-                    std::vector<std::size_t>& labels) const {
-    const double tolerance = 1e-10;
-
-    double min_cut = (std::numeric_limits<double>::max)();
-
-    #ifdef CGAL_SEGMENTATION_BENCH_GRAPHCUT
-    double vertex_creation_time, edge_creation_time, graph_creation_time,
-           reverse_mapping_time, cut_time;
-    vertex_creation_time = edge_creation_time = graph_creation_time =
-                             reverse_mapping_time = cut_time = 0.0;
-    #endif
-
-    Graph graph;
-
-    bool success;
-    do {
-      success = false;
-      std::size_t alpha = 0;
-
-      for(std::vector<std::vector<double> >::const_iterator it =
-            probability_matrix.begin();
-          it != probability_matrix.end(); ++it, ++alpha) {
-        std::vector<std::pair<std::size_t, std::size_t> > edge_map;
-        std::vector<EdgeP>                edge_map_weights;
-        edge_map.reserve(labels.size() *
-                         8); // there is no way to know exact edge count, it is a heuristic value
-        edge_map_weights.reserve(labels.size() * 8);
-
-        std::size_t cluster_source = 0;
-        std::size_t cluster_sink = 1;
-
-        #ifdef CGAL_SEGMENTATION_BENCH_GRAPHCUT
-        Timer timer;
-        timer.start();
-        #endif
-        // For E-Data
-        // add every facet as a vertex to the graph, put edges to source & sink vertices
-        for(std::size_t vertex_i = 0; vertex_i < labels.size(); ++vertex_i) {
-          double source_weight = probability_matrix[alpha][vertex_i];
-          // since it is expansion move, current alpha labeled vertices will be assigned to alpha again,
-          // making sink_weight 'infinity' guarantee this.
-          double sink_weight = (labels[vertex_i] == alpha) ?
-                               (std::numeric_limits<double>::max)()
-                               : probability_matrix[labels[vertex_i]][vertex_i];
-
-          add_edge_and_reverse(cluster_source, vertex_i + 2, source_weight, 0.0, edge_map,
-                               edge_map_weights);
-          add_edge_and_reverse(vertex_i + 2, cluster_sink, sink_weight, 0.0, edge_map,
-                               edge_map_weights);
-        }
-        #ifdef CGAL_SEGMENTATION_BENCH_GRAPHCUT
-        vertex_creation_time += timer.time();
-        timer.reset();
-        #endif
-        // For E-Smooth
-        // add edge between every vertex,
-        std::size_t num_vert = labels.size() + 2;
-        std::vector<double>::const_iterator weight_it = edge_weights.begin();
-        for(std::vector<std::pair<std::size_t, std::size_t> >::const_iterator edge_it =
-              edges.begin(); edge_it != edges.end();
-            ++edge_it, ++weight_it) {
-          std::size_t v1 = edge_it->first + 2, v2 = edge_it->second + 2;
-          std::size_t label_1 = labels[edge_it->first], label_2 = labels[edge_it->second];
-          if(label_1 == label_2) {
-            if(label_1 != alpha) {
-              add_edge_and_reverse(v1, v2, *weight_it, *weight_it, edge_map,
-                                   edge_map_weights);
-            }
-          } else {
-            std::size_t inbetween = num_vert++;
-
-            double w1 = (label_1 == alpha) ? 0 : *weight_it;
-            double w2 = (label_2 == alpha) ? 0 : *weight_it;
-            add_edge_and_reverse(inbetween, v1, w1, w1, edge_map, edge_map_weights);
-            add_edge_and_reverse(inbetween, v2, w2, w2, edge_map, edge_map_weights);
-            add_edge_and_reverse(inbetween, cluster_sink, *weight_it, 0.0, edge_map,
-                                 edge_map_weights);
-          }
-        }
-        #ifdef CGAL_SEGMENTATION_BENCH_GRAPHCUT
-        edge_creation_time += timer.time();
-        timer.reset();
-        #endif
-#if BOOST_VERSION >= 104000
-        Graph graph(boost::edges_are_unsorted, edge_map.begin(), edge_map.end(),
-                    edge_map_weights.begin(), num_vert);
-#else
-        Graph graph(edge_map.begin(), edge_map.end(),
-                    edge_map_weights.begin(), num_vert);
-#endif
-        #ifdef CGAL_SEGMENTATION_BENCH_GRAPHCUT
-        graph_creation_time += timer.time();
-        timer.reset();
-        #endif
-
-        // PERFORMANCE PROBLEM
-        // need to set reverse edge map, I guess there is no way to do that before creating the graph
-        // since we do not have edge_descs
-        // however from our edge_map, we know that each (2i, 2i + 1) is reverse pairs, how to facilitate that ?
-        // will look it back
-        Graph::edge_iterator ei, ee;
-        for(boost::tie(ei, ee) = boost::edges(graph); ei != ee; ++ei) {
-          Graph::vertex_descriptor v1 = boost::source(*ei, graph);
-          Graph::vertex_descriptor v2 = boost::target(*ei, graph);
-          std::pair<Graph::edge_descriptor, bool> opp_edge = boost::edge(v2, v1, graph);
-
-          CGAL_assertion(opp_edge.second);
-          graph[opp_edge.first].edge_reverse =
-            *ei; // and edge_reverse of *ei will be (or already have been) set by the opp_edge
-        }
-        #ifdef CGAL_SEGMENTATION_BENCH_GRAPHCUT
-        reverse_mapping_time += timer.time();
-        timer.reset();
-        #endif
-
-#if BOOST_VERSION >= 104400
-        // since properties are bundled, defaults does not work need to specify them
-        double flow = boost::boykov_kolmogorov_max_flow(graph,
-                      boost::get(&EdgeP::edge_capacity, graph),
-                      boost::get(&EdgeP::edge_residual_capacity, graph),
-                      boost::get(&EdgeP::edge_reverse, graph),
-                      boost::get(&VertexP::vertex_predecessor, graph),
-                      boost::get(&VertexP::vertex_color, graph),
-                      boost::get(&VertexP::vertex_distance_t, graph),
-                      boost::get(boost::vertex_index,
-                                 graph), // this is not bundled, get it from graph (CRS provides one)
-                      cluster_source,
-                      cluster_sink
-                                                       );
-#else
-        double flow = boost::kolmogorov_max_flow(graph,
-                      boost::get(&EdgeP::edge_capacity, graph),
-                      boost::get(&EdgeP::edge_residual_capacity, graph),
-                      boost::get(&EdgeP::edge_reverse, graph),
-                      boost::get(&VertexP::vertex_predecessor, graph),
-                      boost::get(&VertexP::vertex_color, graph),
-                      boost::get(&VertexP::vertex_distance_t, graph),
-                      boost::get(boost::vertex_index,
-                                 graph), // this is not bundled, get it from graph
-                      cluster_source,
-                      cluster_sink
-                                                );
-#endif
-        #ifdef CGAL_SEGMENTATION_BENCH_GRAPHCUT
-        cut_time += timer.time();
-        #endif
-
-        if(min_cut - flow <= flow * tolerance) {
-          continue;
-        }
-        min_cut = flow;
-        success = true;
-        //update labeling
-        for(std::size_t vertex_i = 0; vertex_i < labels.size(); ++vertex_i) {
-          boost::default_color_type color =  graph[vertex_i + 2].vertex_color;
-          if(labels[vertex_i] != alpha
-              && color == ColorTraits::white()) { //new comers (expansion occurs)
-            labels[vertex_i] = alpha;
-          }
-        }
-      }
-    } while(success);
-
-    #ifdef CGAL_SEGMENTATION_BENCH_GRAPHCUT
-    CGAL_TRACE_STREAM << "vertex creation time: " << vertex_creation_time <<
-                      std::endl;
-    CGAL_TRACE_STREAM << "edge creation time: " << edge_creation_time << std::endl;
-    CGAL_TRACE_STREAM << "graph creation time: " << graph_creation_time <<
-                      std::endl;
-    CGAL_TRACE_STREAM << "reverse mapping time: " << reverse_mapping_time <<
-                      std::endl;
-    CGAL_TRACE_STREAM << "max flow algorithm time: " << cut_time << std::endl;
-    #endif
-    return min_cut;
-  }
-};
-#endif
-
-#ifndef CGAL_DO_NOT_USE_BOYKOV_KOLMOGOROV_MAXFLOW_SOFTWARE
-/**
- * @brief Implements alpha-expansion graph cut algorithm.
- *
- * For underlying max-flow algorithm, it uses the MAXFLOW software implemented by Boykov & Kolmogorov.
- *  Also no pre-allocation is made.
- */
-class Alpha_expansion_graph_cut_boykov_kolmogorov
-{
-public:
-  /**
-   * Applies alpha-expansion graph-cut for energy minimization.
-   * @param edges contains incident vertex-id pairs for each edge (vertex-ids should be between [0, number of vertices -1])
-   * @param edge_weights contains weights for each edge in @a edges (correspondence according to order)
-   * @param probability_matrix contains responsibility of the center on the vertex probability[center][vertex]
-   * @param[in, out] labels as input it contains initial labeling of vertices (i.e. a center-id between [0, number of centers -1]),
-   * and as output it returns final labeling of vertices
-   * @return result of energy function
-   */
-  double operator()(const std::vector<std::pair<std::size_t, std::size_t> >&
-                    edges,
-                    const std::vector<double>& edge_weights,
-                    const std::vector<std::vector<double> >& probability_matrix,
-                    std::vector<std::size_t>& labels) const {
-    const double tolerance = 1e-10;
-
-    double min_cut = (std::numeric_limits<double>::max)();
-
-    #ifdef CGAL_SEGMENTATION_BENCH_GRAPHCUT
-    double vertex_creation_time, edge_creation_time, cut_time;
-    vertex_creation_time = edge_creation_time = cut_time = 0.0;
-    #endif
-
-    std::vector<MaxFlow::Graph::node_id> inserted_vertices;
-    inserted_vertices.resize(labels.size());
-    bool success;
-    do {
-      success = false;
-      std::size_t alpha = 0;
-      for(std::vector<std::vector<double> >::const_iterator it =
-            probability_matrix.begin();
-          it != probability_matrix.end(); ++it, ++alpha) {
-        MaxFlow::Graph graph;
-        #ifdef CGAL_SEGMENTATION_BENCH_GRAPHCUT
-        Timer timer;
-        timer.start();
-        #endif
-        // For E-Data
-        // add every facet as a vertex to graph, put edges to source-sink vertices
-        for(std::size_t vertex_i = 0; vertex_i <  labels.size(); ++vertex_i) {
-          MaxFlow::Graph::node_id new_vertex = graph.add_node();
-          inserted_vertices[vertex_i] = new_vertex;
-
-          double source_weight = probability_matrix[alpha][vertex_i];
-          // since it is expansion move, current alpha labeled vertices will be assigned to alpha again,
-          // making sink_weight 'infinity' guarantee this.
-          double sink_weight = (labels[vertex_i] == alpha) ?
-                               (std::numeric_limits<double>::max)()
-                               : probability_matrix[labels[vertex_i]][vertex_i];
-          graph.add_tweights(new_vertex, source_weight, sink_weight);
-        }
-        #ifdef CGAL_SEGMENTATION_BENCH_GRAPHCUT
-        vertex_creation_time += timer.time();
-        timer.reset();
-        #endif
-        // For E-Smooth
-        // add edge between every vertex,
-        std::vector<double>::const_iterator weight_it = edge_weights.begin();
-        for(std::vector<std::pair<std::size_t, std::size_t> >::const_iterator edge_it =
-              edges.begin(); edge_it != edges.end();
-            ++edge_it, ++weight_it) {
-          MaxFlow::Graph::node_id v1 = inserted_vertices[edge_it->first];
-          MaxFlow::Graph::node_id v2 = inserted_vertices[edge_it->second];
-          std::size_t label_1 = labels[edge_it->first], label_2 = labels[edge_it->second];
-          if(label_1 == label_2) {
-            if(label_1 != alpha) {
-              graph.add_edge(v1, v2, *weight_it, *weight_it);
-            }
-          } else {
-            MaxFlow::Graph::node_id inbetween = graph.add_node();
-
-            double w1 = (label_1 == alpha) ? 0 : *weight_it;
-            double w2 = (label_2 == alpha) ? 0 : *weight_it;
-            graph.add_edge(inbetween, v1, w1, w1);
-            graph.add_edge(inbetween, v2, w2, w2);
-
-            graph.add_tweights(inbetween, 0.0, *weight_it);
-          }
-        }
-        #ifdef CGAL_SEGMENTATION_BENCH_GRAPHCUT
-        edge_creation_time += timer.time();
-        timer.reset();
-        #endif
-
-        double flow = graph.maxflow();
-        #ifdef CGAL_SEGMENTATION_BENCH_GRAPHCUT
-        cut_time += timer.time();
-        #endif
-
-        if(min_cut - flow <= flow * tolerance) {
-          continue;
-        }
-
-        min_cut = flow;
-        success = true;
-        //update labeling
-        for(std::size_t vertex_i = 0; vertex_i < labels.size(); ++vertex_i) {
-          if(labels[vertex_i] != alpha
-              && graph.what_segment(inserted_vertices[vertex_i]) == MaxFlow::Graph::SINK) {
-            labels[vertex_i] = alpha;
-          }
-        }
-      }
-    } while(success);
-
-    #ifdef CGAL_SEGMENTATION_BENCH_GRAPHCUT
-    CGAL_TRACE_STREAM << "vertex creation time: " << vertex_creation_time <<
-                      std::endl;
-    CGAL_TRACE_STREAM << "edge creation time: " << edge_creation_time << std::endl;
-    CGAL_TRACE_STREAM << "max flow algorithm time: " << cut_time << std::endl;
-    #endif
-    return min_cut;
-  }
-};
-#endif //CGAL_DO_NOT_USE_BOYKOV_KOLMOGOROV_MAXFLOW_SOFTWARE
-}//namespace internal
-/// @endcond
-}//namespace CGAL
-#endif //CGAL_SURFACE_MESH_SEGMENTATION_ALPHA_EXPANSION_GRAPH_CUT_H
diff -pruN 5.0.3-1/include/CGAL/internal/Surface_mesh_segmentation/Disk_samplers.h 5.1-1/include/CGAL/internal/Surface_mesh_segmentation/Disk_samplers.h
--- 5.0.3-1/include/CGAL/internal/Surface_mesh_segmentation/Disk_samplers.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Surface_mesh_segmentation/Disk_samplers.h	2020-09-08 12:15:44.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_segmentation/include/CGAL/internal/Surface_mesh_segmentation/Disk_samplers.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_segmentation/include/CGAL/internal/Surface_mesh_segmentation/Disk_samplers.h $
 // $Id: Disk_samplers.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Surface_mesh_segmentation/Expectation_maximization.h 5.1-1/include/CGAL/internal/Surface_mesh_segmentation/Expectation_maximization.h
--- 5.0.3-1/include/CGAL/internal/Surface_mesh_segmentation/Expectation_maximization.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Surface_mesh_segmentation/Expectation_maximization.h	2020-09-08 12:15:44.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_segmentation/include/CGAL/internal/Surface_mesh_segmentation/Expectation_maximization.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_segmentation/include/CGAL/internal/Surface_mesh_segmentation/Expectation_maximization.h $
 // $Id: Expectation_maximization.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Surface_mesh_segmentation/Filters.h 5.1-1/include/CGAL/internal/Surface_mesh_segmentation/Filters.h
--- 5.0.3-1/include/CGAL/internal/Surface_mesh_segmentation/Filters.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Surface_mesh_segmentation/Filters.h	2020-09-08 12:15:44.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_segmentation/include/CGAL/internal/Surface_mesh_segmentation/Filters.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_segmentation/include/CGAL/internal/Surface_mesh_segmentation/Filters.h $
 // $Id: Filters.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Surface_mesh_segmentation/K_means_clustering.h 5.1-1/include/CGAL/internal/Surface_mesh_segmentation/K_means_clustering.h
--- 5.0.3-1/include/CGAL/internal/Surface_mesh_segmentation/K_means_clustering.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Surface_mesh_segmentation/K_means_clustering.h	2020-09-08 12:15:44.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_segmentation/include/CGAL/internal/Surface_mesh_segmentation/K_means_clustering.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_segmentation/include/CGAL/internal/Surface_mesh_segmentation/K_means_clustering.h $
 // $Id: K_means_clustering.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Surface_mesh_segmentation/SDF_calculation.h 5.1-1/include/CGAL/internal/Surface_mesh_segmentation/SDF_calculation.h
--- 5.0.3-1/include/CGAL/internal/Surface_mesh_segmentation/SDF_calculation.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Surface_mesh_segmentation/SDF_calculation.h	2020-09-08 12:15:44.000000000 +0000
@@ -4,8 +4,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_segmentation/include/CGAL/internal/Surface_mesh_segmentation/SDF_calculation.h $
-// $Id: SDF_calculation.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_segmentation/include/CGAL/internal/Surface_mesh_segmentation/SDF_calculation.h $
+// $Id: SDF_calculation.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Ilker O. Yaz
@@ -163,11 +163,10 @@ public:
         if(!test)
           tree.insert(Primitive(it, mesh, vertex_point_map));
     }
-    tree.build();
-
-    if(build_kd_tree) {
-      tree.accelerate_distance_queries();
+    if(!build_kd_tree) {
+      tree.do_not_accelerate_distance_queries();
     }
+    tree.build();
 
     if(use_diagonal) {
       CGAL::Bbox_3 bbox = tree.bbox();
diff -pruN 5.0.3-1/include/CGAL/internal/Surface_mesh_segmentation/Surface_mesh_segmentation.h 5.1-1/include/CGAL/internal/Surface_mesh_segmentation/Surface_mesh_segmentation.h
--- 5.0.3-1/include/CGAL/internal/Surface_mesh_segmentation/Surface_mesh_segmentation.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Surface_mesh_segmentation/Surface_mesh_segmentation.h	2020-09-08 12:15:44.000000000 +0000
@@ -4,8 +4,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_segmentation/include/CGAL/internal/Surface_mesh_segmentation/Surface_mesh_segmentation.h $
-// $Id: Surface_mesh_segmentation.h 2c5826d 2020-01-17T16:29:49+01:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_segmentation/include/CGAL/internal/Surface_mesh_segmentation/Surface_mesh_segmentation.h $
+// $Id: Surface_mesh_segmentation.h 855003b 2020-03-24T08:18:23+01:00 Mael
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Ilker O. Yaz
@@ -18,9 +18,13 @@
 
 #include <CGAL/internal/Surface_mesh_segmentation/Expectation_maximization.h>
 #include <CGAL/internal/Surface_mesh_segmentation/Filters.h>
-#include <CGAL/internal/Surface_mesh_segmentation/Alpha_expansion_graph_cut.h>
 #include <CGAL/internal/Surface_mesh_segmentation/SDF_calculation.h>
 
+#include <CGAL/boost/graph/alpha_expansion_graphcut.h>
+#ifndef CGAL_DO_NOT_USE_BOYKOV_KOLMOGOROV_MAXFLOW_SOFTWARE
+#include <CGAL/boost/graph/Alpha_expansion_MaxFlow_tag.h>
+#endif
+
 #include <CGAL/Kernel/global_functions_3.h>
 
 #include <CGAL/property_map.h>
@@ -199,9 +203,9 @@ class Polyhedron,
   class VertexPointPmap,
       bool fast_bbox_intersection = true,
 #ifndef CGAL_DO_NOT_USE_BOYKOV_KOLMOGOROV_MAXFLOW_SOFTWARE
-      class GraphCut = Alpha_expansion_graph_cut_boykov_kolmogorov,
+      class AlphaExpansionImplementationTag = CGAL::Alpha_expansion_MaxFlow_tag,
 #else
-      class GraphCut = Alpha_expansion_graph_cut_boost,
+      class AlphaExpansionImplementationTag = CGAL::Alpha_expansion_boost_adjacency_list_tag,
 #endif
       class Filter = Bilateral_filtering<Polyhedron>
       >
@@ -289,7 +293,8 @@ public:
         edge_weights);
 
     // apply graph cut
-    GraphCut()(edges, edge_weights, probability_matrix, labels);
+    CGAL::alpha_expansion_graphcut (edges, edge_weights, probability_matrix, labels,
+                                    AlphaExpansionImplementationTag());
     std::vector<std::size_t>::iterator label_it = labels.begin();
     face_iterator facet_it, fend;
     for(boost::tie(facet_it,fend) = faces(mesh);
diff -pruN 5.0.3-1/include/CGAL/internal/Surface_mesh_skeletonization/Curve_skeleton.h 5.1-1/include/CGAL/internal/Surface_mesh_skeletonization/Curve_skeleton.h
--- 5.0.3-1/include/CGAL/internal/Surface_mesh_skeletonization/Curve_skeleton.h	2020-08-01 19:02:56.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Surface_mesh_skeletonization/Curve_skeleton.h	2020-09-08 12:15:45.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_skeletonization/include/CGAL/internal/Surface_mesh_skeletonization/Curve_skeleton.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_skeletonization/include/CGAL/internal/Surface_mesh_skeletonization/Curve_skeleton.h $
 // $Id: Curve_skeleton.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Surface_mesh_skeletonization/Debug.h 5.1-1/include/CGAL/internal/Surface_mesh_skeletonization/Debug.h
--- 5.0.3-1/include/CGAL/internal/Surface_mesh_skeletonization/Debug.h	2020-08-01 19:02:56.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Surface_mesh_skeletonization/Debug.h	2020-09-08 12:15:45.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_skeletonization/include/CGAL/internal/Surface_mesh_skeletonization/Debug.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_skeletonization/include/CGAL/internal/Surface_mesh_skeletonization/Debug.h $
 // $Id: Debug.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Surface_mesh_skeletonization/Detect_degeneracy.h 5.1-1/include/CGAL/internal/Surface_mesh_skeletonization/Detect_degeneracy.h
--- 5.0.3-1/include/CGAL/internal/Surface_mesh_skeletonization/Detect_degeneracy.h	2020-08-01 19:02:56.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Surface_mesh_skeletonization/Detect_degeneracy.h	2020-09-08 12:15:45.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_skeletonization/include/CGAL/internal/Surface_mesh_skeletonization/Detect_degeneracy.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_skeletonization/include/CGAL/internal/Surface_mesh_skeletonization/Detect_degeneracy.h $
 // $Id: Detect_degeneracy.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/TDS_2/Edge_hash_function.h 5.1-1/include/CGAL/internal/TDS_2/Edge_hash_function.h
--- 5.0.3-1/include/CGAL/internal/TDS_2/Edge_hash_function.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/internal/TDS_2/Edge_hash_function.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/TDS_2/include/CGAL/internal/TDS_2/Edge_hash_function.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/TDS_2/include/CGAL/internal/TDS_2/Edge_hash_function.h $
 // $Id: Edge_hash_function.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/TDS_2/edge_list.h 5.1-1/include/CGAL/internal/TDS_2/edge_list.h
--- 5.0.3-1/include/CGAL/internal/TDS_2/edge_list.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/internal/TDS_2/edge_list.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/TDS_2/include/CGAL/internal/TDS_2/edge_list.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/TDS_2/include/CGAL/internal/TDS_2/edge_list.h $
 // $Id: edge_list.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Transform_coordinates_traits_3.h 5.1-1/include/CGAL/internal/Transform_coordinates_traits_3.h
--- 5.0.3-1/include/CGAL/internal/Transform_coordinates_traits_3.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Transform_coordinates_traits_3.h	2020-09-08 12:15:40.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_sorting/include/CGAL/internal/Transform_coordinates_traits_3.h $
-// $Id: Transform_coordinates_traits_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_sorting/include/CGAL/internal/Transform_coordinates_traits_3.h $
+// $Id: Transform_coordinates_traits_3.h 5c41b10 2020-01-02T10:26:44+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Olivier Devillers
@@ -52,146 +52,146 @@ struct Coordinate_value_adaptor;
 
 template <class R>
 struct Coordinate_value_adaptor<R,0> {
-        static typename R::FT x(const typename R::Point_3& p) {return p.x();}
-        static typename R::FT y(const typename R::Point_3& p) {return p.y();}
+  static typename R::FT x(const typename R::Point_3& p) {return p.x();}
+  static typename R::FT y(const typename R::Point_3& p) {return p.y();}
 };
 
 template <class R>
 struct Coordinate_value_adaptor<R,1> {
-        static typename R::FT x(const typename R::Point_3& p) {return -p.x();}
-        static typename R::FT y(const typename R::Point_3& p) {return p.y();}
+  static typename R::FT x(const typename R::Point_3& p) {return -p.x();}
+  static typename R::FT y(const typename R::Point_3& p) {return p.y();}
 };
 
 template <class R>
 struct Coordinate_value_adaptor<R,2> {
-        static typename R::FT x(const typename R::Point_3& p) {return p.x();}
-        static typename R::FT y(const typename R::Point_3& p) {return -p.y();}
+  static typename R::FT x(const typename R::Point_3& p) {return p.x();}
+  static typename R::FT y(const typename R::Point_3& p) {return -p.y();}
 };
 
 template <class R>
 struct Coordinate_value_adaptor<R,3> {
-        static typename R::FT x(const typename R::Point_3& p) {return -p.x();}
-        static typename R::FT y(const typename R::Point_3& p) {return -p.y();}
+  static typename R::FT x(const typename R::Point_3& p) {return -p.x();}
+  static typename R::FT y(const typename R::Point_3& p) {return -p.y();}
 };
 
 template <class R>
 struct Coordinate_value_adaptor<R,4> {
-        static typename R::FT x(const typename R::Point_3& p) {return p.y();}
-        static typename R::FT y(const typename R::Point_3& p) {return p.x();}
+  static typename R::FT x(const typename R::Point_3& p) {return p.y();}
+  static typename R::FT y(const typename R::Point_3& p) {return p.x();}
 };
 
 template <class R>
 struct Coordinate_value_adaptor<R,5> {
-        static typename R::FT x(const typename R::Point_3& p) {return -p.y();}
-        static typename R::FT y(const typename R::Point_3& p) {return p.x();}
+  static typename R::FT x(const typename R::Point_3& p) {return -p.y();}
+  static typename R::FT y(const typename R::Point_3& p) {return p.x();}
 };
 
 template <class R>
 struct Coordinate_value_adaptor<R,6> {
-        static typename R::FT x(const typename R::Point_3& p) {return p.y();}
-        static typename R::FT y(const typename R::Point_3& p) {return -p.x();}
+  static typename R::FT x(const typename R::Point_3& p) {return p.y();}
+  static typename R::FT y(const typename R::Point_3& p) {return -p.x();}
 };
 
 template <class R>
 struct Coordinate_value_adaptor<R,7> {
-        static typename R::FT x(const typename R::Point_3& p) {return -p.y();}
-        static typename R::FT y(const typename R::Point_3& p) {return -p.x();}
+  static typename R::FT x(const typename R::Point_3& p) {return -p.y();}
+  static typename R::FT y(const typename R::Point_3& p) {return -p.x();}
 };
 
 template <class R>
 struct Coordinate_value_adaptor<R,8> {
-        static typename R::FT x(const typename R::Point_3& p) {return p.x();}
-        static typename R::FT y(const typename R::Point_3& p) {return p.z();}
+  static typename R::FT x(const typename R::Point_3& p) {return p.x();}
+  static typename R::FT y(const typename R::Point_3& p) {return p.z();}
 };
 
 template <class R>
 struct Coordinate_value_adaptor<R,9> {
-        static typename R::FT x(const typename R::Point_3& p) {return -p.x();}
-        static typename R::FT y(const typename R::Point_3& p) {return p.z();}
+  static typename R::FT x(const typename R::Point_3& p) {return -p.x();}
+  static typename R::FT y(const typename R::Point_3& p) {return p.z();}
 };
 
 template <class R>
 struct Coordinate_value_adaptor<R,10> {
-        static typename R::FT x(const typename R::Point_3& p) {return p.x();}
-        static typename R::FT y(const typename R::Point_3& p) {return -p.z();}
+  static typename R::FT x(const typename R::Point_3& p) {return p.x();}
+  static typename R::FT y(const typename R::Point_3& p) {return -p.z();}
 };
 
 template <class R>
 struct Coordinate_value_adaptor<R,11> {
-        static typename R::FT x(const typename R::Point_3& p) {return -p.x();}
-        static typename R::FT y(const typename R::Point_3& p) {return -p.z();}
+  static typename R::FT x(const typename R::Point_3& p) {return -p.x();}
+  static typename R::FT y(const typename R::Point_3& p) {return -p.z();}
 };
 
 template <class R>
 struct Coordinate_value_adaptor<R,12> {
-        static typename R::FT x(const typename R::Point_3& p) {return p.z();}
-        static typename R::FT y(const typename R::Point_3& p) {return p.x();}
+  static typename R::FT x(const typename R::Point_3& p) {return p.z();}
+  static typename R::FT y(const typename R::Point_3& p) {return p.x();}
 };
 
 template <class R>
 struct Coordinate_value_adaptor<R,13> {
-        static typename R::FT x(const typename R::Point_3& p) {return -p.z();}
-        static typename R::FT y(const typename R::Point_3& p) {return p.x();}
+  static typename R::FT x(const typename R::Point_3& p) {return -p.z();}
+  static typename R::FT y(const typename R::Point_3& p) {return p.x();}
 };
 
 template <class R>
 struct Coordinate_value_adaptor<R,14> {
-        static typename R::FT x(const typename R::Point_3& p) {return p.z();}
-        static typename R::FT y(const typename R::Point_3& p) {return -p.x();}
+  static typename R::FT x(const typename R::Point_3& p) {return p.z();}
+  static typename R::FT y(const typename R::Point_3& p) {return -p.x();}
 };
 
 template <class R>
 struct Coordinate_value_adaptor<R,15> {
-        static typename R::FT x(const typename R::Point_3& p) {return -p.z();}
-        static typename R::FT y(const typename R::Point_3& p) {return -p.x();}
+  static typename R::FT x(const typename R::Point_3& p) {return -p.z();}
+  static typename R::FT y(const typename R::Point_3& p) {return -p.x();}
 };
 
 template <class R>
 struct Coordinate_value_adaptor<R,16> {
-        static typename R::FT x(const typename R::Point_3& p) {return p.y();}
-        static typename R::FT y(const typename R::Point_3& p) {return p.z();}
+  static typename R::FT x(const typename R::Point_3& p) {return p.y();}
+  static typename R::FT y(const typename R::Point_3& p) {return p.z();}
 };
 
 template <class R>
 struct Coordinate_value_adaptor<R,17> {
-        static typename R::FT x(const typename R::Point_3& p) {return -p.y();}
-        static typename R::FT y(const typename R::Point_3& p) {return p.z();}
+  static typename R::FT x(const typename R::Point_3& p) {return -p.y();}
+  static typename R::FT y(const typename R::Point_3& p) {return p.z();}
 };
 
 template <class R>
 struct Coordinate_value_adaptor<R,18> {
-        static typename R::FT x(const typename R::Point_3& p) {return p.y();}
-        static typename R::FT y(const typename R::Point_3& p) {return -p.z();}
+  static typename R::FT x(const typename R::Point_3& p) {return p.y();}
+  static typename R::FT y(const typename R::Point_3& p) {return -p.z();}
 };
 
 template <class R>
 struct Coordinate_value_adaptor<R,19> {
-        static typename R::FT x(const typename R::Point_3& p) {return -p.y();}
-        static typename R::FT y(const typename R::Point_3& p) {return -p.z();}
+  static typename R::FT x(const typename R::Point_3& p) {return -p.y();}
+  static typename R::FT y(const typename R::Point_3& p) {return -p.z();}
 };
 
 template <class R>
 struct Coordinate_value_adaptor<R,20> {
-        static typename R::FT x(const typename R::Point_3& p) {return p.z();}
-        static typename R::FT y(const typename R::Point_3& p) {return p.y();}
+  static typename R::FT x(const typename R::Point_3& p) {return p.z();}
+  static typename R::FT y(const typename R::Point_3& p) {return p.y();}
 };
 
 template <class R>
 struct Coordinate_value_adaptor<R,21> {
-        static typename R::FT x(const typename R::Point_3& p) {return -p.z();}
-        static typename R::FT y(const typename R::Point_3& p) {return p.y();}
+  static typename R::FT x(const typename R::Point_3& p) {return -p.z();}
+  static typename R::FT y(const typename R::Point_3& p) {return p.y();}
 };
 
 template <class R>
 struct Coordinate_value_adaptor<R,22> {
-        static typename R::FT x(const typename R::Point_3& p) {return p.z();}
-        static typename R::FT y(const typename R::Point_3& p) {return -p.y();}
+  static typename R::FT x(const typename R::Point_3& p) {return p.z();}
+  static typename R::FT y(const typename R::Point_3& p) {return -p.y();}
 };
 
 template <class R>
 struct Coordinate_value_adaptor<R,23> {
-        static typename R::FT x(const typename R::Point_3& p) {return -p.z();}
-        static typename R::FT y(const typename R::Point_3& p) {return -p.y();}
+  static typename R::FT x(const typename R::Point_3& p) {return -p.z();}
+  static typename R::FT y(const typename R::Point_3& p) {return -p.y();}
 };
 
 template <class R, int opt>
@@ -232,28 +232,27 @@ public:
 
 template <class R, int x, int y, int z, int ord>
 struct Transform_coordinates_traits_3 {
-        private:
-                enum {opt = Transform_constant_struct<x,y,z,ord>::value};
+private:
+  enum {opt = Transform_constant_struct<x,y,z,ord>::value};
 
-        public:
-                typedef Transform_coordinates_traits_3<R,x,y,z,ord>                            Traits;
-                typedef R                                                               Rp;
-                typedef typename Rp::Point_3                                            Point_2;
-                typedef Less_x_2<R,opt>                                                                                Less_x;
-                typedef Less_y_2<R,opt>                                                                               Less_y;
-                typedef Compute_x_2<R,opt>                                                                             Compute_x;
-                typedef Compute_y_2<R,opt>                                                                             Compute_y;
-
-                Transform_coordinates_traits_3(){}
-                Transform_coordinates_traits_3(const Transform_coordinates_traits_3&){}
-
-                Less_x less_x_2_object() const { return Less_x(); }
-                Less_y less_y_2_object() const { return Less_y(); }
-                Compute_x compute_x_2_object() const { return Compute_x(); }
-                Compute_y compute_y_2_object() const { return Compute_y(); }
+public:
+  typedef Transform_coordinates_traits_3<R,x,y,z,ord>  Traits;
+  typedef R                                            Rp;
+  typedef typename Rp::Point_3                         Point_2;
+  typedef Less_x_2<R,opt>                              Less_x;
+  typedef Less_y_2<R,opt>                              Less_y;
+  typedef Compute_x_2<R,opt>                           Compute_x;
+  typedef Compute_y_2<R,opt>                           Compute_y;
+
+  Transform_coordinates_traits_3(){}
+  Transform_coordinates_traits_3(const Transform_coordinates_traits_3&){}
+
+  Less_x less_x_2_object() const { return Less_x(); }
+  Less_y less_y_2_object() const { return Less_y(); }
+  Compute_x compute_x_2_object() const { return Compute_x(); }
+  Compute_y compute_y_2_object() const { return Compute_y(); }
 };
 
-
 } } //namespace CGAL::internal
 
 #endif // CGAL_INTERNAL_SCALE_COORDINATES_ADAPTOR_TRAITS_3_H
diff -pruN 5.0.3-1/include/CGAL/internal/Triangulation/Dummy_TDS.h 5.1-1/include/CGAL/internal/Triangulation/Dummy_TDS.h
--- 5.0.3-1/include/CGAL/internal/Triangulation/Dummy_TDS.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Triangulation/Dummy_TDS.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation/include/CGAL/internal/Triangulation/Dummy_TDS.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation/include/CGAL/internal/Triangulation/Dummy_TDS.h $
 // $Id: Dummy_TDS.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Triangulation/Triangulation_ds_iterators.h 5.1-1/include/CGAL/internal/Triangulation/Triangulation_ds_iterators.h
--- 5.0.3-1/include/CGAL/internal/Triangulation/Triangulation_ds_iterators.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Triangulation/Triangulation_ds_iterators.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation/include/CGAL/internal/Triangulation/Triangulation_ds_iterators.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation/include/CGAL/internal/Triangulation/Triangulation_ds_iterators.h $
 // $Id: Triangulation_ds_iterators.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Triangulation/utilities.h 5.1-1/include/CGAL/internal/Triangulation/utilities.h
--- 5.0.3-1/include/CGAL/internal/Triangulation/utilities.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Triangulation/utilities.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation/include/CGAL/internal/Triangulation/utilities.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation/include/CGAL/internal/Triangulation/utilities.h $
 // $Id: utilities.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Triangulation_ds_circulators_3.h 5.1-1/include/CGAL/internal/Triangulation_ds_circulators_3.h
--- 5.0.3-1/include/CGAL/internal/Triangulation_ds_circulators_3.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Triangulation_ds_circulators_3.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/TDS_3/include/CGAL/internal/Triangulation_ds_circulators_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/TDS_3/include/CGAL/internal/Triangulation_ds_circulators_3.h $
 // $Id: Triangulation_ds_circulators_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/internal/Triangulation_ds_iterators_3.h 5.1-1/include/CGAL/internal/Triangulation_ds_iterators_3.h
--- 5.0.3-1/include/CGAL/internal/Triangulation_ds_iterators_3.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/internal/Triangulation_ds_iterators_3.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/TDS_3/include/CGAL/internal/Triangulation_ds_iterators_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/TDS_3/include/CGAL/internal/Triangulation_ds_iterators_3.h $
 // $Id: Triangulation_ds_iterators_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Interpolation/internal/helpers.h 5.1-1/include/CGAL/Interpolation/internal/helpers.h
--- 5.0.3-1/include/CGAL/Interpolation/internal/helpers.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/Interpolation/internal/helpers.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Interpolation/include/CGAL/Interpolation/internal/helpers.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Interpolation/include/CGAL/Interpolation/internal/helpers.h $
 // $Id: helpers.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/interpolation_functions.h 5.1-1/include/CGAL/interpolation_functions.h
--- 5.0.3-1/include/CGAL/interpolation_functions.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/interpolation_functions.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Interpolation/include/CGAL/interpolation_functions.h $
-// $Id: interpolation_functions.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Interpolation/include/CGAL/interpolation_functions.h $
+// $Id: interpolation_functions.h d7224e5 2020-05-24T11:08:44+02:00 Marc Glisse
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Julia Floetotto
@@ -34,7 +34,7 @@ struct Data_access
   typedef typename Map::mapped_type   Data_type;
   typedef typename Map::key_type      Key_type;
 
-  Data_access<Map>(const Map& m): map(m){}
+  Data_access(const Map& m): map(m){}
 
   std::pair< Data_type, bool>
   operator()(const Key_type& p) const
diff -pruN 5.0.3-1/include/CGAL/Interpolation_gradient_fitting_traits_2.h 5.1-1/include/CGAL/Interpolation_gradient_fitting_traits_2.h
--- 5.0.3-1/include/CGAL/Interpolation_gradient_fitting_traits_2.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/Interpolation_gradient_fitting_traits_2.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Interpolation/include/CGAL/Interpolation_gradient_fitting_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Interpolation/include/CGAL/Interpolation_gradient_fitting_traits_2.h $
 // $Id: Interpolation_gradient_fitting_traits_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Interpolation_traits_2.h 5.1-1/include/CGAL/Interpolation_traits_2.h
--- 5.0.3-1/include/CGAL/Interpolation_traits_2.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/Interpolation_traits_2.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Interpolation/include/CGAL/Interpolation_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Interpolation/include/CGAL/Interpolation_traits_2.h $
 // $Id: Interpolation_traits_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/intersection_2.h 5.1-1/include/CGAL/intersection_2.h
--- 5.0.3-1/include/CGAL/intersection_2.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/intersection_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_2/include/CGAL/intersection_2.h $
-// $Id: intersection_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/intersection_2.h $
+// $Id: intersection_2.h 8b41189 2020-03-26T18:58:21+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -28,6 +28,10 @@
 #include <CGAL/Intersections_2/Circle_2_Iso_rectangle_2.h>
 #include <CGAL/Intersections_2/Circle_2_Line_2.h>
 #include <CGAL/Intersections_2/Circle_2_Point_2.h>
+#include <CGAL/Intersections_2/Circle_2_Segment_2.h>
+#include <CGAL/Intersections_2/Circle_2_Ray_2.h>
+#include <CGAL/Intersections_2/Circle_2_Triangle_2.h>
+
 
 #include <CGAL/Intersections_2/Iso_rectangle_2_Iso_rectangle_2.h>
 #include <CGAL/Intersections_2/Iso_rectangle_2_Line_2.h>
diff -pruN 5.0.3-1/include/CGAL/intersection_3_1.h 5.1-1/include/CGAL/intersection_3_1.h
--- 5.0.3-1/include/CGAL/intersection_3_1.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/intersection_3_1.h	2020-09-08 12:14:58.000000000 +0000
@@ -8,7 +8,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/intersection_3_1.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/intersection_3_1.h $
 // $Id: intersection_3_1.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/intersection_3.h 5.1-1/include/CGAL/intersection_3.h
--- 5.0.3-1/include/CGAL/intersection_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/intersection_3.h	2020-09-08 12:14:58.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/intersection_3.h $
-// $Id: intersection_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/intersection_3.h $
+// $Id: intersection_3.h 90d2e03 2020-01-15T13:32:11+01:00 Maxime Gimeno
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -83,6 +83,7 @@
 
 #include <CGAL/Intersections_3/Tetrahedron_3_Tetrahedron_3.h>
 #include <CGAL/Intersections_3/Tetrahedron_3_Triangle_3.h>
+#include <CGAL/Intersections_3/Tetrahedron_3_Triangle_3.h>
 
 #include <CGAL/Intersections_3/Triangle_3_Triangle_3.h>
 
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Bbox_2_Bbox_2.h 5.1-1/include/CGAL/Intersections_2/Bbox_2_Bbox_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Bbox_2_Bbox_2.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Bbox_2_Bbox_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -0,0 +1,50 @@
+// Copyright (c) 2019 GeometryFactory (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Bbox_2_Bbox_2.h $
+// $Id: Bbox_2_Bbox_2.h ce4cbe6 2020-03-19T11:41:57+01:00 Maxime Gimeno
+// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Maxime Gimeno
+
+#ifndef CGAL_INTERSECTIONS_BBOX_2_BBOX_2_H
+#define CGAL_INTERSECTIONS_BBOX_2_BBOX_2_H
+
+#include <CGAL/Bbox_2.h>
+
+namespace CGAL {
+
+bool
+inline
+do_intersect(const CGAL::Bbox_2& c,
+             const CGAL::Bbox_2& bbox)
+{
+  return CGAL::do_overlap(c, bbox);
+}
+
+typename boost::optional< typename boost::variant<Bbox_2> >
+inline
+intersection(const CGAL::Bbox_2& a,
+             const CGAL::Bbox_2& b)
+{
+  typedef typename boost::variant<Bbox_2> variant_type;
+  typedef typename boost::optional<variant_type> result_type;
+
+  if(!do_intersect(a, b))
+    return result_type();
+
+  double xmin = (std::max)(a.xmin(), b.xmin());
+  double xmax = (std::min)(a.xmax(), b.xmax());
+
+  double ymin = (std::max)(a.ymin(), b.ymin());
+  double ymax = (std::min)(a.ymax(), b.ymax());
+
+  return result_type(std::forward<Bbox_2>(Bbox_2(xmin, ymin, xmax, ymax)));
+}
+
+} // namespace CGAL
+
+#endif // CGAL_INTERSECTIONS_BBOX_2_BBOX_2_H
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Bbox_2_Circle_2.h 5.1-1/include/CGAL/Intersections_2/Bbox_2_Circle_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Bbox_2_Circle_2.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Bbox_2_Circle_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_2/include/CGAL/Intersections_2/Bbox_2_Circle_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Bbox_2_Circle_2.h $
 // $Id: Bbox_2_Circle_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Bbox_2_Iso_rectangle_2.h 5.1-1/include/CGAL/Intersections_2/Bbox_2_Iso_rectangle_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Bbox_2_Iso_rectangle_2.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Bbox_2_Iso_rectangle_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -0,0 +1,56 @@
+// Copyright (c) 2019
+// GeometryFactory (France).  All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Bbox_2_Iso_rectangle_2.h $
+// $Id: Bbox_2_Iso_rectangle_2.h ce4cbe6 2020-03-19T11:41:57+01:00 Maxime Gimeno
+// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Maxime Gimeno
+
+
+#ifndef CGAL_INTERSECTIONS_BBOX_2_ISO_RECTANGLE_2_H
+#define CGAL_INTERSECTIONS_BBOX_2_ISO_RECTANGLE_2_H
+
+#include <CGAL/Bbox_2.h>
+#include <CGAL/Iso_rectangle_2.h>
+#include <CGAL/Intersections_2/Iso_rectangle_2_Iso_rectangle_2.h>
+
+namespace CGAL {
+
+template <typename K>
+inline bool do_intersect(const Iso_rectangle_2<K> &rect,
+                         const Bbox_2 &box)
+{
+  return do_intersect(K::Iso_rectangle_2(box), rect);
+}
+
+template <typename K>
+inline bool do_intersect(const Bbox_2 &box,
+                         const Iso_rectangle_2<K> &rect)
+{
+  return do_intersect(rect, box);
+}
+
+template<typename K>
+typename Intersection_traits<K, typename K::Iso_rectangle_2, Bbox_2>::result_type
+intersection(const Bbox_2& box,
+             const Iso_rectangle_2<K>& rect)
+{
+  typename K::Iso_rectangle_2 rec(box.xmin(), box.ymin(), box.xmax(), box.ymax());
+  return intersection(rec, rect);
+}
+
+template<typename K>
+typename Intersection_traits<K, typename K::Iso_rectangle_2, Bbox_2>::result_type
+intersection(const Iso_rectangle_2<K>& rect,
+             const Bbox_2& box)
+{
+  return intersection(box, rect);
+}
+
+} // namespace CGAL
+
+#endif // CGAL_INTERSECTIONS_BBOX_2_ISO_RECTANGLE_2_H
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Bbox_2_Line_2.h 5.1-1/include/CGAL/Intersections_2/Bbox_2_Line_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Bbox_2_Line_2.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Bbox_2_Line_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_2/include/CGAL/Intersections_2/Bbox_2_Line_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Bbox_2_Line_2.h $
 // $Id: Bbox_2_Line_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Bbox_2_Point_2.h 5.1-1/include/CGAL/Intersections_2/Bbox_2_Point_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Bbox_2_Point_2.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Bbox_2_Point_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_2/include/CGAL/Intersections_2/Bbox_2_Point_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Bbox_2_Point_2.h $
 // $Id: Bbox_2_Point_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Bbox_2_Ray_2.h 5.1-1/include/CGAL/Intersections_2/Bbox_2_Ray_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Bbox_2_Ray_2.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Bbox_2_Ray_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_2/include/CGAL/Intersections_2/Bbox_2_Ray_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Bbox_2_Ray_2.h $
 // $Id: Bbox_2_Ray_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Bbox_2_Segment_2.h 5.1-1/include/CGAL/Intersections_2/Bbox_2_Segment_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Bbox_2_Segment_2.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Bbox_2_Segment_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -0,0 +1,57 @@
+// Copyright (c) 2019
+// GeometryFactory (France).  All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Bbox_2_Segment_2.h $
+// $Id: Bbox_2_Segment_2.h ce4cbe6 2020-03-19T11:41:57+01:00 Maxime Gimeno
+// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Maxime Gimeno
+
+
+#ifndef CGAL_INTERSECTIONS_BBOX_2_SEGMENT_2_H
+#define CGAL_INTERSECTIONS_BBOX_2_SEGMENT_2_H
+
+#include <CGAL/Bbox_2.h>
+#include <CGAL/Segment_2.h>
+#include <CGAL/Intersections_2/Iso_rectangle_2_Segment_2.h>
+
+namespace CGAL {
+
+
+template <class K>
+inline bool do_intersect(
+    const Segment_2<K> &seg,
+    const Bbox_2 &box)
+{
+  typename K::Iso_rectangle_2 rec(box.xmin(), box.ymin(), box.xmax(), box.ymax());
+  return do_intersect(rec, seg);
+}
+
+template <class K>
+inline bool do_intersect(
+    const Bbox_2 &box,
+    const Segment_2<K> &seg)
+{
+  return do_intersect(seg, box);
+}
+
+template<typename K>
+typename Intersection_traits<K, typename K::Segment_2, Bbox_2>::result_type
+intersection(const CGAL::Bbox_2& box,
+             const Segment_2<K>& seg) {
+  typename K::Iso_rectangle_2 rec(box.xmin(), box.ymin(), box.xmax(), box.ymax());
+  return intersection(rec, seg);
+}
+
+template<typename K>
+typename Intersection_traits<K, typename K::Segment_2, Bbox_2>::result_type
+intersection(const Segment_2<K>& seg,
+             const CGAL::Bbox_2& box) {
+  return intersection(box, seg);
+}
+
+}
+#endif // CGAL_INTERSECTIONS_BBOX_2_SEGMENT_2_H
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Bbox_2_Triangle_2.h 5.1-1/include/CGAL/Intersections_2/Bbox_2_Triangle_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Bbox_2_Triangle_2.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Bbox_2_Triangle_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -0,0 +1,57 @@
+// Copyright (c) 2019
+// GeometryFactory (France).  All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Bbox_2_Triangle_2.h $
+// $Id: Bbox_2_Triangle_2.h ce4cbe6 2020-03-19T11:41:57+01:00 Maxime Gimeno
+// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Maxime Gimeno
+
+
+#ifndef CGAL_INTERSECTIONS_BBOX_2_TRIANGLE_2_H
+#define CGAL_INTERSECTIONS_BBOX_2_TRIANGLE_2_H
+
+#include <CGAL/Bbox_2.h>
+#include <CGAL/Triangle_2.h>
+#include <CGAL/Intersections_2/Iso_rectangle_2_Triangle_2.h>
+
+namespace CGAL {
+
+
+template <class K>
+inline bool do_intersect(
+    const Triangle_2<K> &tr,
+    const Bbox_2 &box)
+{
+  typename K::Iso_rectangle_2 rec(box.xmin(), box.ymin(), box.xmax(), box.ymax());
+  return do_intersect(rec, tr);
+}
+
+template <class K>
+inline bool do_intersect(
+    const Bbox_2 &box,
+    const Triangle_2<K> &tr)
+{
+  return do_intersect(tr, box);
+}
+
+template<typename K>
+typename Intersection_traits<K, typename K::Triangle_2, Bbox_2>::result_type
+intersection(const Bbox_2& box,
+             const Triangle_2<K>& tr) {
+  typename K::Iso_rectangle_2 rec(box.xmin(), box.ymin(), box.xmax(), box.ymax());
+  return intersection(rec, tr);
+}
+
+template<typename K>
+typename Intersection_traits<K, typename K::Triangle_2, Bbox_2>::result_type
+intersection(const Triangle_2<K>& tr,
+             const Bbox_2& box) {
+  return intersection(box, tr);
+}
+
+}
+#endif // CGAL_INTERSECTIONS_BBOX_2_TRIANGLE_2_H
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Circle_2_Circle_2.h 5.1-1/include/CGAL/Intersections_2/Circle_2_Circle_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Circle_2_Circle_2.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Circle_2_Circle_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_2/include/CGAL/Intersections_2/Circle_2_Circle_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Circle_2_Circle_2.h $
 // $Id: Circle_2_Circle_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Circle_2_Iso_rectangle_2.h 5.1-1/include/CGAL/Intersections_2/Circle_2_Iso_rectangle_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Circle_2_Iso_rectangle_2.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Circle_2_Iso_rectangle_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_2/include/CGAL/Intersections_2/Circle_2_Iso_rectangle_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Circle_2_Iso_rectangle_2.h $
 // $Id: Circle_2_Iso_rectangle_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Circle_2_Line_2.h 5.1-1/include/CGAL/Intersections_2/Circle_2_Line_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Circle_2_Line_2.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Circle_2_Line_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_2/include/CGAL/Intersections_2/Circle_2_Line_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Circle_2_Line_2.h $
 // $Id: Circle_2_Line_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Circle_2_Point_2.h 5.1-1/include/CGAL/Intersections_2/Circle_2_Point_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Circle_2_Point_2.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Circle_2_Point_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_2/include/CGAL/Intersections_2/Circle_2_Point_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Circle_2_Point_2.h $
 // $Id: Circle_2_Point_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Circle_2_Ray_2.h 5.1-1/include/CGAL/Intersections_2/Circle_2_Ray_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Circle_2_Ray_2.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Circle_2_Ray_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -0,0 +1,49 @@
+// Copyright (c) 2019
+// GeometryFactory.  All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Circle_2_Ray_2.h $
+// $Id: Circle_2_Ray_2.h ce4cbe6 2020-03-19T11:41:57+01:00 Maxime Gimeno
+// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Maxime Gimeno
+
+#ifndef CGAL_INTERSECTIONS_2_CIRCLE_2_RAY_2_H
+#define CGAL_INTERSECTIONS_2_CIRCLE_2_RAY_2_H
+
+#include <CGAL/Circle_2.h>
+#include <CGAL/Ray_2.h>
+#include <CGAL/squared_distance_2_1.h>
+#include <CGAL/Intersection_traits_2.h>
+
+namespace CGAL {
+namespace Intersections {
+namespace internal {
+
+template <class K>
+bool
+do_intersect(const typename K::Circle_2 & c,
+             const typename K::Ray_2& r,
+             const K&)
+{
+  return squared_distance(c.center(), r) <= c.squared_radius();
+}
+
+template <class K>
+bool
+do_intersect(const typename K::Ray_2& r,
+             const typename K::Circle_2 & c,
+             const K&)
+{
+  return squared_distance(c.center(), r) <= c.squared_radius();
+}
+
+} // namespace internal
+} // namespace Intersections
+
+CGAL_DO_INTERSECT_FUNCTION(Circle_2, Ray_2, 2)
+
+} // namespace CGAL
+#endif // CGAL_INTERSECTIONS_2_CIRCLE_2_RAY_2_H
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Circle_2_Segment_2.h 5.1-1/include/CGAL/Intersections_2/Circle_2_Segment_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Circle_2_Segment_2.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Circle_2_Segment_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -0,0 +1,49 @@
+// Copyright (c) 2019
+// GeometryFactory.  All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Circle_2_Segment_2.h $
+// $Id: Circle_2_Segment_2.h ce4cbe6 2020-03-19T11:41:57+01:00 Maxime Gimeno
+// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Maxime Gimeno
+
+#ifndef CGAL_INTERSECTIONS_2_CIRCLE_2_SEGMENT_2_H
+#define CGAL_INTERSECTIONS_2_CIRCLE_2_SEGMENT_2_H
+
+#include <CGAL/Circle_2.h>
+#include <CGAL/Segment_2.h>
+#include <CGAL/squared_distance_2_1.h>
+#include <CGAL/Intersection_traits_2.h>
+
+namespace CGAL {
+namespace Intersections {
+namespace internal {
+
+template <class K>
+bool
+do_intersect(const typename K::Circle_2 & c,
+             const typename K::Segment_2& s,
+             const K&)
+{
+  return squared_distance(c.center(), s) <= c.squared_radius();
+}
+
+template <class K>
+bool
+do_intersect(const typename K::Segment_2& s,
+             const typename K::Circle_2 & c,
+             const K&)
+{
+  return squared_distance(c.center(), s) <= c.squared_radius();
+}
+
+} // namespace internal
+} // namespace Intersections
+
+CGAL_DO_INTERSECT_FUNCTION(Circle_2, Segment_2, 2)
+
+} // namespace CGAL
+#endif // CGAL_INTERSECTIONS_2_CIRCLE_2_SEGMENT_2_H
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Circle_2_Triangle_2.h 5.1-1/include/CGAL/Intersections_2/Circle_2_Triangle_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Circle_2_Triangle_2.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Circle_2_Triangle_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -0,0 +1,63 @@
+// Copyright (c) 2019
+// GeometryFactory.  All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Circle_2_Triangle_2.h $
+// $Id: Circle_2_Triangle_2.h ce4cbe6 2020-03-19T11:41:57+01:00 Maxime Gimeno
+// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Maxime Gimeno
+
+#ifndef CGAL_INTERSECTIONS_2_CIRCLE_2_TRIANGLE_2_H
+#define CGAL_INTERSECTIONS_2_CIRCLE_2_TRIANGLE_2_H
+
+#include <CGAL/Circle_2.h>
+#include <CGAL/Triangle_2.h>
+#include <CGAL/squared_distance_2_1.h>
+#include <CGAL/Intersection_traits_2.h>
+
+namespace CGAL {
+namespace Intersections {
+namespace internal {
+
+template <class K>
+bool
+do_intersect(const typename K::Circle_2 & c,
+             const typename K::Triangle_2& t,
+             const K&)
+{
+  char nb_close(0);
+//if all three points are inside the circle, there is no intersection.
+  if(squared_distance(c.center(), t) < c.squared_radius())
+  {
+    for(int i = 0; i< 3; ++i)
+    {
+      if(squared_distance(c.center(), t[i]) == c.squared_radius())
+        return true;
+      if(squared_distance(c.center(), t[i]) < c.squared_radius())
+        ++nb_close;
+    }
+    return (nb_close < 3);
+  }
+  else
+    return squared_distance(c.center(), t) == c.squared_radius();
+}
+
+template <class K>
+bool
+do_intersect(const typename K::Triangle_2& t,
+             const typename K::Circle_2 & c,
+             const K&)
+{
+  return do_intersect(c,t);
+}
+
+} // namespace internal
+} // namespace Intersections
+
+CGAL_DO_INTERSECT_FUNCTION(Circle_2, Triangle_2, 2)
+
+} // namespace CGAL
+#endif // CGAL_INTERSECTIONS_2_CIRCLE_2_TRIANGLE_2_H
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/internal/Straight_2.h 5.1-1/include/CGAL/Intersections_2/internal/Straight_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/internal/Straight_2.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/internal/Straight_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_2/include/CGAL/Intersections_2/internal/Straight_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/internal/Straight_2.h $
 // $Id: Straight_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/internal/Triangle_2_Triangle_2_do_intersect_impl.h 5.1-1/include/CGAL/Intersections_2/internal/Triangle_2_Triangle_2_do_intersect_impl.h
--- 5.0.3-1/include/CGAL/Intersections_2/internal/Triangle_2_Triangle_2_do_intersect_impl.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/internal/Triangle_2_Triangle_2_do_intersect_impl.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_2/include/CGAL/Intersections_2/internal/Triangle_2_Triangle_2_do_intersect_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/internal/Triangle_2_Triangle_2_do_intersect_impl.h $
 // $Id: Triangle_2_Triangle_2_do_intersect_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/internal/Triangle_2_Triangle_2_intersection_impl.h 5.1-1/include/CGAL/Intersections_2/internal/Triangle_2_Triangle_2_intersection_impl.h
--- 5.0.3-1/include/CGAL/Intersections_2/internal/Triangle_2_Triangle_2_intersection_impl.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/internal/Triangle_2_Triangle_2_intersection_impl.h	2020-09-08 12:14:57.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_2/include/CGAL/Intersections_2/internal/Triangle_2_Triangle_2_intersection_impl.h $
-// $Id: Triangle_2_Triangle_2_intersection_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/internal/Triangle_2_Triangle_2_intersection_impl.h $
+// $Id: Triangle_2_Triangle_2_intersection_impl.h 8b41189 2020-03-26T18:58:21+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -21,6 +21,7 @@
 
 #include <CGAL/Intersections_2/Line_2_Line_2.h>
 #include <CGAL/Intersection_traits_2.h>
+#include <CGAL/Algebraic_structure_traits.h>
 
 namespace CGAL {
 
@@ -277,12 +278,22 @@ typename K::Triangle_2
 Triangle_2_Triangle_2_pair<K>::intersection_triangle() const
 {
   typedef typename K::Triangle_2 Triangle_2;
-    if (!_known)
-        intersection_type();
-    CGAL_kernel_assertion(_result == TRIANGLE);
+  if (!_known)
+    intersection_type();
+  CGAL_kernel_assertion(_result == TRIANGLE);
+  if(CGAL::left_turn(_pointlist.first->point,
+                     _pointlist.first->next->point,
+                     _pointlist.first->next->next->point))
+  {
     return Triangle_2(_pointlist.first->point,
                       _pointlist.first->next->point,
                       _pointlist.first->next->next->point);
+  }
+  else {
+    return Triangle_2(_pointlist.first->point,
+                      _pointlist.first->next->next->point,
+                      _pointlist.first->next->point);
+  }
 }
 
 template <class K>
@@ -308,6 +319,29 @@ Triangle_2_Triangle_2_pair<K>::intersect
 }
 
 
+//algorithm taken from here : https://stackoverflow.com/questions/1165647/how-to-determine-if-a-list-of-polygon-points-are-in-clockwise-order
+template <typename K,  typename Exact>
+struct Is_cw
+{
+  bool operator()(const std::vector<typename K::Point_2>& ps)
+  {
+    typename K::FT res(0);
+    std::size_t length = ps.size();
+    for(std::size_t i = 0; i<length; ++i)
+      res += (ps[(i+1)%length].x() - ps[i].x())*(ps[(i+1)%length].y()+ps[i].y());
+
+    return res > 0;
+  }
+};
+
+template <typename K>
+struct Is_cw<K, Tag_true>
+{
+  bool operator()(const std::vector<typename K::Point_2>& ps)
+  {
+    return CGAL::right_turn(ps[0], ps[1], ps[2]);
+  }
+};
 
 template <class K>
 typename CGAL::Intersection_traits
@@ -316,27 +350,34 @@ intersection(const typename K::Triangle_
              const typename K::Triangle_2 &tr2,
              const K&)
 {
-    typedef Triangle_2_Triangle_2_pair<K> is_t;
-    is_t ispair(&tr1, &tr2);
-    switch (ispair.intersection_type()) {
-    case is_t::NO_INTERSECTION:
+  typedef Triangle_2_Triangle_2_pair<K> Intersection_type;
+  Intersection_type ispair(&tr1, &tr2);
+  switch (ispair.intersection_type())
+  {
+    case Intersection_type::NO_INTERSECTION:
     default:
-        return intersection_return<typename K::Intersect_2, typename K::Triangle_2, typename K::Triangle_2>();
-    case is_t::POINT:
-        return intersection_return<typename K::Intersect_2, typename K::Triangle_2, typename K::Triangle_2>(ispair.intersection_point());
-    case is_t::SEGMENT:
-        return intersection_return<typename K::Intersect_2, typename K::Triangle_2, typename K::Triangle_2>(ispair.intersection_segment());
-    case is_t::TRIANGLE:
-        return intersection_return<typename K::Intersect_2, typename K::Triangle_2, typename K::Triangle_2>(ispair.intersection_triangle());
-    case is_t::POLYGON: {
-        typedef std::vector<typename K::Point_2> Container;
-        Container points(ispair.vertex_count());
-        for (int i =0; i < ispair.vertex_count(); i++) {
-            points[i] = ispair.vertex(i);
-        }
-        return intersection_return<typename K::Intersect_2, typename K::Triangle_2, typename K::Triangle_2>(points);
-    }
+      return intersection_return<typename K::Intersect_2, typename K::Triangle_2, typename K::Triangle_2>();
+    case Intersection_type::POINT:
+      return intersection_return<typename K::Intersect_2, typename K::Triangle_2, typename K::Triangle_2>(ispair.intersection_point());
+    case Intersection_type::SEGMENT:
+      return intersection_return<typename K::Intersect_2, typename K::Triangle_2, typename K::Triangle_2>(ispair.intersection_segment());
+    case Intersection_type::TRIANGLE:
+      return intersection_return<typename K::Intersect_2, typename K::Triangle_2, typename K::Triangle_2>(ispair.intersection_triangle());
+    case Intersection_type::POLYGON:
+    {
+      typedef std::vector<typename K::Point_2> Container;
+      Container points(ispair.vertex_count());
+      for (int i =0; i < ispair.vertex_count(); i++)
+        points[i] = ispair.vertex(i);
+
+      if(Is_cw<K, typename Algebraic_structure_traits<typename K::FT>::Is_exact>()(points))
+      {
+        std::reverse(points.begin(), points.end());
+
+      }
+      return intersection_return<typename K::Intersect_2, typename K::Triangle_2, typename K::Triangle_2>(points);
     }
+  }
 }
 
 } // namespace internal
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Iso_rectangle_2_Iso_rectangle_2.h 5.1-1/include/CGAL/Intersections_2/Iso_rectangle_2_Iso_rectangle_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Iso_rectangle_2_Iso_rectangle_2.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Iso_rectangle_2_Iso_rectangle_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_2/include/CGAL/Intersections_2/Iso_rectangle_2_Iso_rectangle_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Iso_rectangle_2_Iso_rectangle_2.h $
 // $Id: Iso_rectangle_2_Iso_rectangle_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Iso_rectangle_2_Line_2.h 5.1-1/include/CGAL/Intersections_2/Iso_rectangle_2_Line_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Iso_rectangle_2_Line_2.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Iso_rectangle_2_Line_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_2/include/CGAL/Intersections_2/Iso_rectangle_2_Line_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Iso_rectangle_2_Line_2.h $
 // $Id: Iso_rectangle_2_Line_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Iso_rectangle_2_Point_2.h 5.1-1/include/CGAL/Intersections_2/Iso_rectangle_2_Point_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Iso_rectangle_2_Point_2.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Iso_rectangle_2_Point_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_2/include/CGAL/Intersections_2/Iso_rectangle_2_Point_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Iso_rectangle_2_Point_2.h $
 // $Id: Iso_rectangle_2_Point_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Iso_rectangle_2_Ray_2.h 5.1-1/include/CGAL/Intersections_2/Iso_rectangle_2_Ray_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Iso_rectangle_2_Ray_2.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Iso_rectangle_2_Ray_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_2/include/CGAL/Intersections_2/Iso_rectangle_2_Ray_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Iso_rectangle_2_Ray_2.h $
 // $Id: Iso_rectangle_2_Ray_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Iso_rectangle_2_Segment_2.h 5.1-1/include/CGAL/Intersections_2/Iso_rectangle_2_Segment_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Iso_rectangle_2_Segment_2.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Iso_rectangle_2_Segment_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_2/include/CGAL/Intersections_2/Iso_rectangle_2_Segment_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Iso_rectangle_2_Segment_2.h $
 // $Id: Iso_rectangle_2_Segment_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Iso_rectangle_2_Triangle_2.h 5.1-1/include/CGAL/Intersections_2/Iso_rectangle_2_Triangle_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Iso_rectangle_2_Triangle_2.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Iso_rectangle_2_Triangle_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_2/include/CGAL/Intersections_2/Iso_rectangle_2_Triangle_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Iso_rectangle_2_Triangle_2.h $
 // $Id: Iso_rectangle_2_Triangle_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Line_2_Line_2.h 5.1-1/include/CGAL/Intersections_2/Line_2_Line_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Line_2_Line_2.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Line_2_Line_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_2/include/CGAL/Intersections_2/Line_2_Line_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Line_2_Line_2.h $
 // $Id: Line_2_Line_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Line_2_Point_2.h 5.1-1/include/CGAL/Intersections_2/Line_2_Point_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Line_2_Point_2.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Line_2_Point_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_2/include/CGAL/Intersections_2/Line_2_Point_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Line_2_Point_2.h $
 // $Id: Line_2_Point_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Line_2_Ray_2.h 5.1-1/include/CGAL/Intersections_2/Line_2_Ray_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Line_2_Ray_2.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Line_2_Ray_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_2/include/CGAL/Intersections_2/Line_2_Ray_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Line_2_Ray_2.h $
 // $Id: Line_2_Ray_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Line_2_Segment_2.h 5.1-1/include/CGAL/Intersections_2/Line_2_Segment_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Line_2_Segment_2.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Line_2_Segment_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_2/include/CGAL/Intersections_2/Line_2_Segment_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Line_2_Segment_2.h $
 // $Id: Line_2_Segment_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Line_2_Triangle_2.h 5.1-1/include/CGAL/Intersections_2/Line_2_Triangle_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Line_2_Triangle_2.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Line_2_Triangle_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_2/include/CGAL/Intersections_2/Line_2_Triangle_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Line_2_Triangle_2.h $
 // $Id: Line_2_Triangle_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Point_2_Point_2.h 5.1-1/include/CGAL/Intersections_2/Point_2_Point_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Point_2_Point_2.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Point_2_Point_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_2/include/CGAL/Intersections_2/Point_2_Point_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Point_2_Point_2.h $
 // $Id: Point_2_Point_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Point_2_Ray_2.h 5.1-1/include/CGAL/Intersections_2/Point_2_Ray_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Point_2_Ray_2.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Point_2_Ray_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_2/include/CGAL/Intersections_2/Point_2_Ray_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Point_2_Ray_2.h $
 // $Id: Point_2_Ray_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Point_2_Segment_2.h 5.1-1/include/CGAL/Intersections_2/Point_2_Segment_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Point_2_Segment_2.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Point_2_Segment_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_2/include/CGAL/Intersections_2/Point_2_Segment_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Point_2_Segment_2.h $
 // $Id: Point_2_Segment_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Point_2_Triangle_2.h 5.1-1/include/CGAL/Intersections_2/Point_2_Triangle_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Point_2_Triangle_2.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Point_2_Triangle_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_2/include/CGAL/Intersections_2/Point_2_Triangle_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Point_2_Triangle_2.h $
 // $Id: Point_2_Triangle_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Ray_2_Ray_2.h 5.1-1/include/CGAL/Intersections_2/Ray_2_Ray_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Ray_2_Ray_2.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Ray_2_Ray_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_2/include/CGAL/Intersections_2/Ray_2_Ray_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Ray_2_Ray_2.h $
 // $Id: Ray_2_Ray_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Ray_2_Segment_2.h 5.1-1/include/CGAL/Intersections_2/Ray_2_Segment_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Ray_2_Segment_2.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Ray_2_Segment_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_2/include/CGAL/Intersections_2/Ray_2_Segment_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Ray_2_Segment_2.h $
 // $Id: Ray_2_Segment_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Ray_2_Triangle_2.h 5.1-1/include/CGAL/Intersections_2/Ray_2_Triangle_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Ray_2_Triangle_2.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Ray_2_Triangle_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_2/include/CGAL/Intersections_2/Ray_2_Triangle_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Ray_2_Triangle_2.h $
 // $Id: Ray_2_Triangle_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Segment_2_Segment_2.h 5.1-1/include/CGAL/Intersections_2/Segment_2_Segment_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Segment_2_Segment_2.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Segment_2_Segment_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_2/include/CGAL/Intersections_2/Segment_2_Segment_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Segment_2_Segment_2.h $
 // $Id: Segment_2_Segment_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Segment_2_Triangle_2.h 5.1-1/include/CGAL/Intersections_2/Segment_2_Triangle_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Segment_2_Triangle_2.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Segment_2_Triangle_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_2/include/CGAL/Intersections_2/Segment_2_Triangle_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Segment_2_Triangle_2.h $
 // $Id: Segment_2_Triangle_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_2/Triangle_2_Triangle_2.h 5.1-1/include/CGAL/Intersections_2/Triangle_2_Triangle_2.h
--- 5.0.3-1/include/CGAL/Intersections_2/Triangle_2_Triangle_2.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_2/Triangle_2_Triangle_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_2/include/CGAL/Intersections_2/Triangle_2_Triangle_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersections_2/Triangle_2_Triangle_2.h $
 // $Id: Triangle_2_Triangle_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Bbox_3_Bbox_3.h 5.1-1/include/CGAL/Intersections_3/Bbox_3_Bbox_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Bbox_3_Bbox_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Bbox_3_Bbox_3.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Bbox_3_Bbox_3.h $
-// $Id: Bbox_3_Bbox_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Bbox_3_Bbox_3.h $
+// $Id: Bbox_3_Bbox_3.h 0f3305f 2020-01-16T17:20:13+01:00 Maxime Gimeno
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -14,10 +14,11 @@
 #ifndef CGAL_INTERSECTIONS_3_BBOX_3_BBOX_3_H
 #define CGAL_INTERSECTIONS_3_BBOX_3_BBOX_3_H
 
-
 #include <CGAL/Bbox_3.h>
+#include <CGAL/Intersection_traits_3.h>
 
 namespace CGAL {
+
 bool
 inline
 do_intersect(const CGAL::Bbox_3& c,
@@ -26,6 +27,27 @@ do_intersect(const CGAL::Bbox_3& c,
   return CGAL::do_overlap(c, bbox);
 }
 
+typename boost::optional< typename boost::variant< Bbox_3> >
+inline
+intersection(const CGAL::Bbox_3& a,
+             const CGAL::Bbox_3& b)
+{
+  typedef typename boost::variant<Bbox_3> variant_type;
+  typedef typename boost::optional<variant_type> result_type;
+
+  if(!do_intersect(a,b))
+    return result_type();
+
+  double xmin = (std::max)(a.xmin(), b.xmin());
+  double xmax = (std::min)(a.xmax(), b.xmax());
+  double ymin = (std::max)(a.ymin(), b.ymin());
+  double ymax = (std::min)(a.ymax(), b.ymax());
+  double zmin = (std::max)(a.zmin(), b.zmin());
+  double zmax = (std::min)(a.zmax(), b.zmax());
+
+  return result_type(std::forward<Bbox_3>(Bbox_3(xmin, ymin, zmin, xmax, ymax, zmax)));
+}
+
 } //namespace CGAL
 
 
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Bbox_3_Iso_cuboid_3.h 5.1-1/include/CGAL/Intersections_3/Bbox_3_Iso_cuboid_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Bbox_3_Iso_cuboid_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Bbox_3_Iso_cuboid_3.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Bbox_3_Iso_cuboid_3.h $
-// $Id: Bbox_3_Iso_cuboid_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Bbox_3_Iso_cuboid_3.h $
+// $Id: Bbox_3_Iso_cuboid_3.h 90d2e03 2020-01-15T13:32:11+01:00 Maxime Gimeno
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -18,6 +18,7 @@
 #include <CGAL/Iso_cuboid_3.h>
 
 #include <CGAL/Intersections_3/internal/Bbox_3_Iso_cuboid_3_do_intersect.h>
+#include <CGAL/Intersections_3/internal/intersection_3_1_impl.h>
 
 namespace CGAL {
 
@@ -33,6 +34,19 @@ bool do_intersect(const Iso_cuboid_3<K>&
   return K().do_intersect_3_object()(b, a);
 }
 
+template<typename K>
+typename Intersection_traits<K, typename K::Iso_cuboid_3, Bbox_3>::result_type
+intersection(const CGAL::Bbox_3& a,
+             const Iso_cuboid_3<K>& b) {
+  return K().intersect_3_object()(a, b);
+}
+
+template<typename K>
+typename Intersection_traits<K, typename K::Iso_cuboid_3, Bbox_3>::result_type
+intersection(const Iso_cuboid_3<K>& a,
+             const CGAL::Bbox_3& b) {
+  return K().intersect_3_object()(a, b);
+}
 } // namespace CGAL
 
 #endif // CGAL_INTERSECTIONS_3_BBOX_3_ISO_CUBOID_3_H
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Bbox_3_Line_3.h 5.1-1/include/CGAL/Intersections_3/Bbox_3_Line_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Bbox_3_Line_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Bbox_3_Line_3.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Bbox_3_Line_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Bbox_3_Line_3.h $
 // $Id: Bbox_3_Line_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Bbox_3_Plane_3.h 5.1-1/include/CGAL/Intersections_3/Bbox_3_Plane_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Bbox_3_Plane_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Bbox_3_Plane_3.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Bbox_3_Plane_3.h $
-// $Id: Bbox_3_Plane_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Bbox_3_Plane_3.h $
+// $Id: Bbox_3_Plane_3.h 90d2e03 2020-01-15T13:32:11+01:00 Maxime Gimeno
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -18,6 +18,7 @@
 #include <CGAL/Plane_3.h>
 
 #include <CGAL/Intersections_3/internal/Bbox_3_Plane_3_do_intersect.h>
+#include <CGAL/Intersections_3/internal/Iso_cuboid_3_Plane_3_intersection.h>
 
 
 namespace CGAL {
@@ -34,6 +35,23 @@ bool do_intersect(const Plane_3<K>& a,
   return K().do_intersect_3_object()(a, b);
 }
 
+template<typename K>
+typename Intersection_traits<K, typename K::Plane_3, Bbox_3>::result_type
+intersection(const CGAL::Bbox_3& box,
+             const Plane_3<K>& pl) {
+  typename K::Iso_cuboid_3 cub(box.xmin(), box.ymin(), box.zmin(),
+                   box.xmax(), box.ymax(), box.zmax());
+  return typename K::Intersect_3()(cub, pl);
+}
+
+template<typename K>
+typename Intersection_traits<K, typename K::Triangle_3, Bbox_3>::result_type
+intersection(const Plane_3<K>& a,
+             const CGAL::Bbox_3& b) {
+  return intersection(b,a);
+}
+
+
 }
 
 #endif // CGAL_INTERSECTIONS_3_BBOX_3_PLANE_3_H
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Bbox_3_Point_3.h 5.1-1/include/CGAL/Intersections_3/Bbox_3_Point_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Bbox_3_Point_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Bbox_3_Point_3.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Bbox_3_Point_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Bbox_3_Point_3.h $
 // $Id: Bbox_3_Point_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Bbox_3_Ray_3.h 5.1-1/include/CGAL/Intersections_3/Bbox_3_Ray_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Bbox_3_Ray_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Bbox_3_Ray_3.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Bbox_3_Ray_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Bbox_3_Ray_3.h $
 // $Id: Bbox_3_Ray_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Bbox_3_Segment_3.h 5.1-1/include/CGAL/Intersections_3/Bbox_3_Segment_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Bbox_3_Segment_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Bbox_3_Segment_3.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Bbox_3_Segment_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Bbox_3_Segment_3.h $
 // $Id: Bbox_3_Segment_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Bbox_3_Sphere_3.h 5.1-1/include/CGAL/Intersections_3/Bbox_3_Sphere_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Bbox_3_Sphere_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Bbox_3_Sphere_3.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Bbox_3_Sphere_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Bbox_3_Sphere_3.h $
 // $Id: Bbox_3_Sphere_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Bbox_3_Tetrahedron_3.h 5.1-1/include/CGAL/Intersections_3/Bbox_3_Tetrahedron_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Bbox_3_Tetrahedron_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Bbox_3_Tetrahedron_3.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Bbox_3_Tetrahedron_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Bbox_3_Tetrahedron_3.h $
 // $Id: Bbox_3_Tetrahedron_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Bbox_3_Triangle_3.h 5.1-1/include/CGAL/Intersections_3/Bbox_3_Triangle_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Bbox_3_Triangle_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Bbox_3_Triangle_3.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Bbox_3_Triangle_3.h $
-// $Id: Bbox_3_Triangle_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Bbox_3_Triangle_3.h $
+// $Id: Bbox_3_Triangle_3.h 90d2e03 2020-01-15T13:32:11+01:00 Maxime Gimeno
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -18,6 +18,7 @@
 #include <CGAL/Triangle_3.h>
 
 #include <CGAL/Intersections_3/internal/Bbox_3_Triangle_3_do_intersect.h>
+#include <CGAL/Intersections_3/internal/Iso_cuboid_3_Triangle_3_intersection.h>
 
 namespace CGAL {
 
@@ -33,6 +34,21 @@ bool do_intersect(const Triangle_3<K>& a
   return K().do_intersect_3_object()(a, b);
 }
 
+template<typename K>
+typename Intersection_traits<K, typename K::Triangle_3, Bbox_3>::result_type
+intersection(const CGAL::Bbox_3& box,
+             const Triangle_3<K>& tr) {
+  typename K::Iso_cuboid_3 cub(box.xmin(), box.ymin(), box.zmin(),
+                   box.xmax(), box.ymax(), box.zmax());
+  return typename K::Intersect_3()(cub, tr);
+}
+
+template<typename K>
+typename Intersection_traits<K, typename K::Triangle_3, Bbox_3>::result_type
+intersection(const Triangle_3<K>& a,
+             const CGAL::Bbox_3& b) {
+  return intersection(b,a);
+}
 
 } // namespace CGAL
 
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/internal/Bbox_3_Bbox_3_do_intersect.h 5.1-1/include/CGAL/Intersections_3/internal/Bbox_3_Bbox_3_do_intersect.h
--- 5.0.3-1/include/CGAL/Intersections_3/internal/Bbox_3_Bbox_3_do_intersect.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/internal/Bbox_3_Bbox_3_do_intersect.h	2020-09-08 12:14:58.000000000 +0000
@@ -5,7 +5,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/internal/Bbox_3_Bbox_3_do_intersect.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/internal/Bbox_3_Bbox_3_do_intersect.h $
 // $Id: Bbox_3_Bbox_3_do_intersect.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/internal/Bbox_3_Iso_cuboid_3_do_intersect.h 5.1-1/include/CGAL/Intersections_3/internal/Bbox_3_Iso_cuboid_3_do_intersect.h
--- 5.0.3-1/include/CGAL/Intersections_3/internal/Bbox_3_Iso_cuboid_3_do_intersect.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/internal/Bbox_3_Iso_cuboid_3_do_intersect.h	2020-09-08 12:14:58.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/internal/Bbox_3_Iso_cuboid_3_do_intersect.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/internal/Bbox_3_Iso_cuboid_3_do_intersect.h $
 // $Id: Bbox_3_Iso_cuboid_3_do_intersect.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/internal/Bbox_3_Line_3_do_intersect.h 5.1-1/include/CGAL/Intersections_3/internal/Bbox_3_Line_3_do_intersect.h
--- 5.0.3-1/include/CGAL/Intersections_3/internal/Bbox_3_Line_3_do_intersect.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/internal/Bbox_3_Line_3_do_intersect.h	2020-09-08 12:14:58.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/internal/Bbox_3_Line_3_do_intersect.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/internal/Bbox_3_Line_3_do_intersect.h $
 // $Id: Bbox_3_Line_3_do_intersect.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/internal/Bbox_3_Plane_3_do_intersect.h 5.1-1/include/CGAL/Intersections_3/internal/Bbox_3_Plane_3_do_intersect.h
--- 5.0.3-1/include/CGAL/Intersections_3/internal/Bbox_3_Plane_3_do_intersect.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/internal/Bbox_3_Plane_3_do_intersect.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/internal/Bbox_3_Plane_3_do_intersect.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/internal/Bbox_3_Plane_3_do_intersect.h $
 // $Id: Bbox_3_Plane_3_do_intersect.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/internal/Bbox_3_Ray_3_do_intersect.h 5.1-1/include/CGAL/Intersections_3/internal/Bbox_3_Ray_3_do_intersect.h
--- 5.0.3-1/include/CGAL/Intersections_3/internal/Bbox_3_Ray_3_do_intersect.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/internal/Bbox_3_Ray_3_do_intersect.h	2020-09-08 12:14:58.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/internal/Bbox_3_Ray_3_do_intersect.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/internal/Bbox_3_Ray_3_do_intersect.h $
 // $Id: Bbox_3_Ray_3_do_intersect.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/internal/Bbox_3_Segment_3_do_intersect.h 5.1-1/include/CGAL/Intersections_3/internal/Bbox_3_Segment_3_do_intersect.h
--- 5.0.3-1/include/CGAL/Intersections_3/internal/Bbox_3_Segment_3_do_intersect.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/internal/Bbox_3_Segment_3_do_intersect.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/internal/Bbox_3_Segment_3_do_intersect.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/internal/Bbox_3_Segment_3_do_intersect.h $
 // $Id: Bbox_3_Segment_3_do_intersect.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/internal/Bbox_3_Sphere_3_do_intersect.h 5.1-1/include/CGAL/Intersections_3/internal/Bbox_3_Sphere_3_do_intersect.h
--- 5.0.3-1/include/CGAL/Intersections_3/internal/Bbox_3_Sphere_3_do_intersect.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/internal/Bbox_3_Sphere_3_do_intersect.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/internal/Bbox_3_Sphere_3_do_intersect.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/internal/Bbox_3_Sphere_3_do_intersect.h $
 // $Id: Bbox_3_Sphere_3_do_intersect.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/internal/Bbox_3_Triangle_3_do_intersect.h 5.1-1/include/CGAL/Intersections_3/internal/Bbox_3_Triangle_3_do_intersect.h
--- 5.0.3-1/include/CGAL/Intersections_3/internal/Bbox_3_Triangle_3_do_intersect.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/internal/Bbox_3_Triangle_3_do_intersect.h	2020-09-08 12:14:58.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/internal/Bbox_3_Triangle_3_do_intersect.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/internal/Bbox_3_Triangle_3_do_intersect.h $
 // $Id: Bbox_3_Triangle_3_do_intersect.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/internal/bbox_intersection_3.h 5.1-1/include/CGAL/Intersections_3/internal/bbox_intersection_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/internal/bbox_intersection_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/internal/bbox_intersection_3.h	2020-09-08 12:14:58.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/internal/bbox_intersection_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/internal/bbox_intersection_3.h $
 // $Id: bbox_intersection_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/internal/intersection_3_1_impl.h 5.1-1/include/CGAL/Intersections_3/internal/intersection_3_1_impl.h
--- 5.0.3-1/include/CGAL/Intersections_3/internal/intersection_3_1_impl.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/internal/intersection_3_1_impl.h	2020-09-08 12:14:58.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/internal/intersection_3_1_impl.h $
-// $Id: intersection_3_1_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/internal/intersection_3_1_impl.h $
+// $Id: intersection_3_1_impl.h 8b41189 2020-03-26T18:58:21+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -25,6 +25,7 @@
 #include <CGAL/Intersections_3/Iso_cuboid_3_Iso_cuboid_3.h>
 #include <CGAL/Intersections_3/Iso_cuboid_3_Line_3.h>
 #include <CGAL/utils_classes.h>
+#include <CGAL/squared_distance_3.h>
 
 #include <CGAL/Intersections_3/internal/bbox_intersection_3.h>
 namespace CGAL {
@@ -1243,6 +1244,37 @@ intersection(const Bbox_3 &box,
   return intersection(seg, box, k);
 }
 
+template <class K>
+inline
+typename Intersection_traits<K, Bbox_3, Bbox_3>::result_type
+intersection(const Bbox_3 &a,
+             const Bbox_3 &b,
+             const K&)
+{
+  return CGAL::intersection<K>(a, b);
+}
+
+template <class K>
+typename Intersection_traits<K, CGAL::Bbox_3, typename K::Iso_cuboid_3>::result_type
+intersection(const CGAL::Bbox_3 &box,
+             const typename K::Iso_cuboid_3 &cub,
+             const K&)
+{
+  typename K::Iso_cuboid_3 iso_cub(box);
+    return intersection(iso_cub, cub);
+}
+
+
+template <class K>
+inline
+typename Intersection_traits<K, typename K::Iso_cuboid_3, CGAL::Bbox_3>::result_type
+intersection(const typename K::Iso_cuboid_3 &cub,
+             const CGAL::Bbox_3 &box,
+             const K& k)
+{
+  return intersection(box,cub, k);
+}
+
 
 template <class K>
 typename Intersection_traits<K, typename K::Line_3, typename K::Iso_cuboid_3>::result_type
@@ -1567,7 +1599,6 @@ do_intersect(const Iso_cuboid_3<R> &j, c
 {
   return bool(CGAL::intersection(l, j));
 }
-
 } // namespace internal
 } // namespace Intersections
 } // namespace CGAL
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/internal/Iso_cuboid_3_Plane_3_intersection.h 5.1-1/include/CGAL/Intersections_3/internal/Iso_cuboid_3_Plane_3_intersection.h
--- 5.0.3-1/include/CGAL/Intersections_3/internal/Iso_cuboid_3_Plane_3_intersection.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/internal/Iso_cuboid_3_Plane_3_intersection.h	2020-09-08 12:14:58.000000000 +0000
@@ -0,0 +1,347 @@
+// Copyright (c) 2019 GeometryFactory(France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/internal/Iso_cuboid_3_Plane_3_intersection.h $
+// $Id: Iso_cuboid_3_Plane_3_intersection.h 8b41189 2020-03-26T18:58:21+01:00 Sébastien Loriot
+// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Maxime Gimeno
+//
+
+#ifndef CGAL_INTERNAL_INTERSECTIONS_3_ISO_CUBOID_3_PLANE_3_INTERSECTION_H
+#define CGAL_INTERNAL_INTERSECTIONS_3_ISO_CUBOID_3_PLANE_3_INTERSECTION_H
+
+#include <CGAL/kernel_basic.h>
+#include <CGAL/intersections.h>
+
+#include <CGAL/Intersections_3/internal/tetrahedron_intersection_helpers.h>
+#include <CGAL/Intersections_3/Iso_cuboid_3_Segment_3.h>
+#include <CGAL/Intersections_3/Plane_3_Plane_3.h>
+
+#include <set>
+
+namespace CGAL {
+namespace Intersections {
+namespace internal {
+
+template<typename Point>
+void filter_points(std::vector<Point>& input,
+                   std::vector<Point>& output)
+{
+  std::sort(input.begin(), input.end());
+  auto last = std::unique(input.begin(), input.end());
+  input.erase(last, input.end());
+  output = std::move(input);
+}
+
+//Tetrahedron_3 Line_3
+template <class K>
+typename Intersection_traits<K, typename K::Iso_cuboid_3, typename K::Plane_3>::result_type
+intersection(
+    const typename K::Iso_cuboid_3 &cub,
+    const typename K::Plane_3 &pl,
+    const K& k)
+{
+  typedef typename K::Point_3 Point_3;
+  typedef typename K::Segment_3 Segment_3;
+  typedef typename K::Line_3 Line_3;
+  typedef typename K::Plane_3 Plane_3;
+  typedef std::vector<Point_3> Poly;
+
+  typedef typename Intersection_traits<K, CGAL::Iso_cuboid_3<K>,
+      CGAL::Plane_3<K> >::result_type result_type;
+
+  typedef typename Intersection_traits<K, CGAL::Segment_3<K>,
+      CGAL::Plane_3<K> >::result_type Inter_type;
+
+  std::vector<Segment_3> edges;
+  edges.reserve(12);
+
+  //get all edges of cub
+  for(int i=0; i< 4; ++i)
+    edges.emplace_back(cub.vertex(i), cub.vertex((i+1)%4));
+  for(int i=0; i < 4; ++i)
+    edges.emplace_back(cub.vertex(i+4), cub.vertex((i+1)%4+4));
+  for(int i=0; i < 4; ++i)
+    edges.emplace_back(cub.vertex(i), cub.vertex((i+1)%4+4));
+
+  //get all intersections between pl and cub edges
+  std::vector<Segment_3> segments;
+  std::vector<Point_3> points;
+
+  for(int i=0; i < 12; ++i)
+  {
+    // Intersect_3 checks the orientation of the segment's extremities to avoid actually computing
+    // the intersection if possible
+    Inter_type inter = typename K::Intersect_3()(pl, edges[i]);
+    if(inter)
+    {
+      if(const Segment_3* seg = boost::get<Segment_3>(&*inter))
+        segments.push_back(*seg);
+      else if(const Point_3* p = boost::get<Point_3>(&*inter))
+        points.push_back(*p);
+    }
+  }
+
+  if(segments.empty() && points.empty())
+    return result_type();
+
+  switch(segments.size())
+  {
+    case 0:
+    //points dealt with later
+    break;
+    case 1:
+    {
+      //adj to an edge
+      if(points.size() == 4)
+      {
+        return result_type(std::forward<Segment_3>(segments.front()));
+      }
+      //plane intersecting through an edge (not 2)
+      else
+      {
+        Poly res(4);
+        const Segment_3& entry_seg(segments.front());
+        Point_3 p1, p2;
+        bool p1_found(false),
+             p2_found(false);
+
+        for(const Point_3& p : points)
+        {
+          if(!k.equal_3_object()(entry_seg.source(), p)
+             && ! k.equal_3_object()(entry_seg.target(), p))
+          {
+            if(!p1_found)
+            {
+              p1 = p;
+              p1_found = true;
+            }
+            else {
+              p2 = p;
+              p2_found = true;
+              break;
+            }
+          }
+        }
+        CGAL_USE(p2_found);
+        CGAL_assertion(p1_found && p2_found);
+        res[0] = entry_seg.target();
+        res[1] = p2;
+        res[2] = p1;
+        res[3] = entry_seg.source();
+
+        typename K::Coplanar_orientation_3 coplanar_orientation =
+          k.coplanar_orientation_3_object();
+
+        if( coplanar_orientation(res[0], res[1], res[2])
+            != coplanar_orientation(res[0], res[1], res[3]))
+        {
+          std::swap(res[1], res[2]);
+        }
+
+        return result_type(std::forward<Poly>(res));
+      }
+    }
+      break;
+
+    case 2: //intersects diagonally
+    {
+      Poly res(4);
+      Segment_3 &front(segments.front()),
+          &back(segments.back());
+      res[0] = front.target();
+      res[1] = back.target();
+      res[2] = back.source();
+      res[3] = front.source();
+      typename K::Coplanar_orientation_3 coplanar_orientation =
+        k.coplanar_orientation_3_object();
+
+      if( coplanar_orientation(res[0], res[1], res[2])
+          != coplanar_orientation(res[0], res[1], res[3]))
+      {
+        std::swap(res[1], res[2]);
+      }
+
+      return result_type(std::forward<Poly>(res));
+    }
+    break;
+  case 4: // intersects a face
+  {
+    Poly res;
+    res.reserve(4);
+    typename K::Has_on_3 has_on;
+    if(has_on(pl, cub.vertex(0))
+       && has_on(pl, cub.vertex(5))
+       && has_on(pl, cub.vertex(4)))
+    {
+      res.push_back(cub.vertex(0));
+      res.push_back(cub.vertex(5));
+      res.push_back(cub.vertex(4));
+      res.push_back(cub.vertex(3));
+    }
+    else if(has_on(pl, cub.vertex(0))
+            && has_on(pl, cub.vertex(1))
+            && has_on(pl, cub.vertex(6)))
+    {
+      res.push_back(cub.vertex(0));
+      res.push_back(cub.vertex(1));
+      res.push_back(cub.vertex(6));
+      res.push_back(cub.vertex(5));
+
+    }
+    else if(has_on(pl, cub.vertex(1))
+            && has_on(pl, cub.vertex(2))
+            && has_on(pl, cub.vertex(7)))
+    {
+      res.push_back(cub.vertex(1));
+      res.push_back(cub.vertex(2));
+      res.push_back(cub.vertex(7));
+      res.push_back(cub.vertex(6));
+    }
+    else if(has_on(pl, cub.vertex(2))
+            && has_on(pl, cub.vertex(3))
+            && has_on(pl, cub.vertex(4)))
+    {
+      res.push_back(cub.vertex(2));
+      res.push_back(cub.vertex(3));
+      res.push_back(cub.vertex(4));
+      res.push_back(cub.vertex(7));
+    }
+    else if(has_on(pl, cub.vertex(6))
+            && has_on(pl, cub.vertex(7))
+            && has_on(pl, cub.vertex(4)))
+    {
+      res.push_back(cub.vertex(6));
+      res.push_back(cub.vertex(7));
+      res.push_back(cub.vertex(4));
+      res.push_back(cub.vertex(5));
+    }
+    else if(has_on(pl, cub.vertex(2))
+            && has_on(pl, cub.vertex(1))
+            && has_on(pl, cub.vertex(0)))
+    {
+      res.push_back(cub.vertex(2));
+      res.push_back(cub.vertex(1));
+      res.push_back(cub.vertex(0));
+      res.push_back(cub.vertex(3));
+    }
+    return result_type(std::forward<Poly>(res));
+  }
+    break;
+  default:
+    break;
+  }
+
+  Poly filtered_points;
+  filter_points(points, filtered_points);
+
+  //adjacent to a vertex
+  if(filtered_points.size() == 1)
+  {
+    return result_type(std::forward<Point_3>(filtered_points.front()));
+  }
+
+  //get intersections between pl and each face -> line. Foreach line, creates segment with points. Then use helper_function to recover polygon.
+  typedef typename Intersection_traits<K,
+      CGAL::Plane_3<K>,
+      CGAL::Plane_3<K> >::result_type Pl_pl_type;
+
+  std::vector<Line_3> plane_intersections;
+  Pl_pl_type pl_inter = CGAL::intersection(pl, Plane_3(cub.vertex(0),
+                                                       cub.vertex(1),
+                                                       cub.vertex(5)));
+  if(const Line_3* line = boost::get<Line_3>(&*pl_inter)){
+    plane_intersections.push_back(*line);
+  }
+  pl_inter = CGAL::intersection(pl, Plane_3(cub.vertex(0),
+                                            cub.vertex(3),
+                                            cub.vertex(4)));
+  if(const Line_3* line = boost::get<Line_3>(&*pl_inter)){
+    plane_intersections.push_back(*line);
+  }
+  pl_inter = CGAL::intersection(pl, Plane_3(cub.vertex(0),
+                                            cub.vertex(1),
+                                            cub.vertex(3)));
+  if(const Line_3* line = boost::get<Line_3>(&*pl_inter)){
+    plane_intersections.push_back(*line);
+  }
+  pl_inter = CGAL::intersection(pl, Plane_3(cub.vertex(7),
+                                            cub.vertex(6),
+                                            cub.vertex(1)));
+  if(const Line_3* line = boost::get<Line_3>(&*pl_inter)){
+    plane_intersections.push_back(*line);
+  }
+  pl_inter = CGAL::intersection(pl, Plane_3(cub.vertex(7),
+                                            cub.vertex(4),
+                                            cub.vertex(3)));
+  if(const Line_3* line = boost::get<Line_3>(&*pl_inter)){
+    plane_intersections.push_back(*line);
+  }
+  pl_inter = CGAL::intersection(pl, Plane_3(cub.vertex(7),
+                                            cub.vertex(6),
+                                            cub.vertex(4)));
+  if(const Line_3* line = boost::get<Line_3>(&*pl_inter)){
+    plane_intersections.push_back(*line);
+  }
+
+  std::list<Segment_3> tmp_segs;
+  for(const auto& line : plane_intersections)
+  {
+    bool first_found = false;
+    Point_3 first_p;
+    typename K::Has_on_3 has_on;
+    for(const auto  &p : filtered_points)
+    {
+      if(has_on(line, p))
+      {
+        if(!first_found)
+        {
+          first_found = true;
+          first_p = p;
+        }
+        else
+        {
+          tmp_segs.emplace_back(first_p, p);
+          break;
+        }
+      }
+    }
+  }
+
+  if(tmp_segs.size() < 3)
+    return result_type();
+
+  std::list<Point_3> tmp_pts;
+  fill_points_list(tmp_segs,tmp_pts);
+
+  Poly res;
+  for(const auto& p : tmp_pts)
+    res.push_back(p);
+
+  if(res.size() == 3){
+    typename K::Triangle_3 tr(res[0], res[1], res[2]);
+    return result_type(std::forward<typename K::Triangle_3>(tr));
+  }
+  else
+  {
+    return result_type(std::forward<Poly>(res));
+  }
+}
+
+template <class K>
+typename Intersection_traits<K, typename K::Iso_cuboid_3, typename K::Plane_3>::result_type
+intersection(
+    const typename K::Plane_3 &pl,
+    const typename K::Iso_cuboid_3 &cub,
+    const K& k)
+{
+  return intersection(cub, pl, k);
+}
+
+}}}
+
+#endif // CGAL_INTERNAL_INTERSECTIONS_3_ISO_CUBOID_3_PLANE_3_INTERSECTION_H
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/internal/Iso_cuboid_3_Ray_3_do_intersect.h 5.1-1/include/CGAL/Intersections_3/internal/Iso_cuboid_3_Ray_3_do_intersect.h
--- 5.0.3-1/include/CGAL/Intersections_3/internal/Iso_cuboid_3_Ray_3_do_intersect.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/internal/Iso_cuboid_3_Ray_3_do_intersect.h	2020-09-08 12:14:58.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/internal/Iso_cuboid_3_Ray_3_do_intersect.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/internal/Iso_cuboid_3_Ray_3_do_intersect.h $
 // $Id: Iso_cuboid_3_Ray_3_do_intersect.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/internal/Iso_cuboid_3_Segment_3_do_intersect.h 5.1-1/include/CGAL/Intersections_3/internal/Iso_cuboid_3_Segment_3_do_intersect.h
--- 5.0.3-1/include/CGAL/Intersections_3/internal/Iso_cuboid_3_Segment_3_do_intersect.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/internal/Iso_cuboid_3_Segment_3_do_intersect.h	2020-09-08 12:14:58.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/internal/Iso_cuboid_3_Segment_3_do_intersect.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/internal/Iso_cuboid_3_Segment_3_do_intersect.h $
 // $Id: Iso_cuboid_3_Segment_3_do_intersect.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/internal/Iso_cuboid_3_Sphere_3_do_intersect.h 5.1-1/include/CGAL/Intersections_3/internal/Iso_cuboid_3_Sphere_3_do_intersect.h
--- 5.0.3-1/include/CGAL/Intersections_3/internal/Iso_cuboid_3_Sphere_3_do_intersect.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/internal/Iso_cuboid_3_Sphere_3_do_intersect.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/internal/Iso_cuboid_3_Sphere_3_do_intersect.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/internal/Iso_cuboid_3_Sphere_3_do_intersect.h $
 // $Id: Iso_cuboid_3_Sphere_3_do_intersect.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/internal/Iso_cuboid_3_Triangle_3_do_intersect.h 5.1-1/include/CGAL/Intersections_3/internal/Iso_cuboid_3_Triangle_3_do_intersect.h
--- 5.0.3-1/include/CGAL/Intersections_3/internal/Iso_cuboid_3_Triangle_3_do_intersect.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/internal/Iso_cuboid_3_Triangle_3_do_intersect.h	2020-09-08 12:14:58.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/internal/Iso_cuboid_3_Triangle_3_do_intersect.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/internal/Iso_cuboid_3_Triangle_3_do_intersect.h $
 // $Id: Iso_cuboid_3_Triangle_3_do_intersect.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/internal/Iso_cuboid_3_Triangle_3_intersection.h 5.1-1/include/CGAL/Intersections_3/internal/Iso_cuboid_3_Triangle_3_intersection.h
--- 5.0.3-1/include/CGAL/Intersections_3/internal/Iso_cuboid_3_Triangle_3_intersection.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/internal/Iso_cuboid_3_Triangle_3_intersection.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/internal/Iso_cuboid_3_Triangle_3_intersection.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/internal/Iso_cuboid_3_Triangle_3_intersection.h $
 // $Id: Iso_cuboid_3_Triangle_3_intersection.h bbe02d9 2019-10-29T15:22:49+01:00 Laurent Rineau
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/internal/Tetrahedron_3_Bounded_3_do_intersect.h 5.1-1/include/CGAL/Intersections_3/internal/Tetrahedron_3_Bounded_3_do_intersect.h
--- 5.0.3-1/include/CGAL/Intersections_3/internal/Tetrahedron_3_Bounded_3_do_intersect.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/internal/Tetrahedron_3_Bounded_3_do_intersect.h	2020-09-08 12:14:58.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/internal/Tetrahedron_3_Bounded_3_do_intersect.h $
-// $Id: Tetrahedron_3_Bounded_3_do_intersect.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/internal/Tetrahedron_3_Bounded_3_do_intersect.h $
+// $Id: Tetrahedron_3_Bounded_3_do_intersect.h dc1ccb5 2020-08-11T16:59:55+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -153,31 +153,49 @@ do_intersect(const typename K::Sphere_3
 template <class K>
 inline
 typename K::Boolean
-do_intersect(const typename K::Tetrahedron_3 &tet,
-             const typename K::Tetrahedron_3 &sp,
+do_intersect(const typename K::Tetrahedron_3 &lh_tet,
+             const typename K::Tetrahedron_3 &rh_tet,
              const K & k)
 {
-  return do_intersect_tetrahedron_bounded(sp, tet, tet[0], k);
+  return do_intersect_tetrahedron_bounded(lh_tet, rh_tet, lh_tet[0], k);
 }
 
+// BBox_3 specific code since it is ok for BBox_3 to degenerate.
 template <class K>
-inline
-typename K::Boolean
-do_intersect(const typename K::Tetrahedron_3 &tet,
-             const CGAL::Bbox_3 &bb,
-             const K & k)
-{
-  return do_intersect_tetrahedron_bounded(bb, tet, typename K::Point_3(bb.xmin(), bb.ymin(), bb.zmin()), k);
+inline typename K::Boolean do_intersect(const CGAL::Bbox_3 &aabb,
+                                        const typename K::Tetrahedron_3 &tet,
+                                        const K &k) {
+  typename K::Construct_triangle_3 tr = k.construct_triangle_3_object();
+  typename K::Boolean result = false;
+  typename K::Boolean b = false;
+
+  b = do_intersect(aabb, tr(tet[0], tet[1], tet[2]), k);
+  if (certainly(b)) return b;
+  if (is_indeterminate(b)) result = b;
+  b = do_intersect(aabb, tr(tet[1], tet[2], tet[3]), k);
+  if (certainly(b)) return b;
+  if (is_indeterminate(b)) result = b;
+  b = do_intersect(aabb, tr(tet[2], tet[3], tet[0]), k);
+  if (certainly(b)) return b;
+  if (is_indeterminate(b)) result = b;
+  b = do_intersect(aabb, tr(tet[3], tet[0], tet[1]), k);
+  if (certainly(b)) return b;
+  if (is_indeterminate(b)) result = b;
+
+  b = k.has_on_bounded_side_3_object()(
+      tet, k.construct_point_3_object()(aabb.xmin(), aabb.ymin(), aabb.zmin()));
+  if (certainly(b)) return b;
+  if (is_indeterminate(b)) result = b;
+
+  return result;
 }
 
-  template <class K>
-inline
-typename K::Boolean
-do_intersect(const CGAL::Bbox_3 &bb,
-             const typename K::Tetrahedron_3 &tet,
-             const K & k)
-{
-  return do_intersect_tetrahedron_bounded(bb, tet, typename K::Point_3(bb.xmin(), bb.ymin(), bb.zmin()), k);
+
+template <class K>
+inline typename K::Boolean do_intersect(const typename K::Tetrahedron_3 &tet,
+                                        const CGAL::Bbox_3 &bb, const K &k) {
+  // Swap arguments.
+  return do_intersect(bb, tet, k);
 }
 
 } // namespace internal
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/internal/Tetrahedron_3_Line_3_intersection.h 5.1-1/include/CGAL/Intersections_3/internal/Tetrahedron_3_Line_3_intersection.h
--- 5.0.3-1/include/CGAL/Intersections_3/internal/Tetrahedron_3_Line_3_intersection.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/internal/Tetrahedron_3_Line_3_intersection.h	2020-09-08 12:14:58.000000000 +0000
@@ -0,0 +1,75 @@
+// Copyright (c) 2019 GeometryFactory(France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/internal/Tetrahedron_3_Line_3_intersection.h $
+// $Id: Tetrahedron_3_Line_3_intersection.h 8b41189 2020-03-26T18:58:21+01:00 Sébastien Loriot
+// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Maxime Gimeno
+//
+
+#ifndef CGAL_INTERNAL_INTERSECTIONS_3_TETRAHEDRON_3_LINE_3_INTERSECTION_H
+#define CGAL_INTERNAL_INTERSECTIONS_3_TETRAHEDRON_3_LINE_3_INTERSECTION_H
+
+#include <CGAL/kernel_basic.h>
+#include <CGAL/intersections.h>
+#include <CGAL/Intersections_3/internal/tetrahedron_lines_intersections_3.h>
+
+namespace CGAL {
+
+namespace Intersections {
+
+namespace internal {
+
+template<class K>
+struct Tetrahedron_Line_intersection_3
+    :public Tetrahedron_lines_intersection_3_base<K, typename K::Line_3, Tetrahedron_Line_intersection_3<K> >
+{
+  typedef Tetrahedron_lines_intersection_3_base<K, typename K::Line_3,
+  Tetrahedron_Line_intersection_3<K> > Base;
+  typedef typename Base::Result_type Result_type;
+  Tetrahedron_Line_intersection_3(
+      const typename K::Tetrahedron_3& tet,
+      const typename K::Line_3& o):Base(tet,o)
+  {}
+
+  bool all_inside_test()
+  {
+    return false;
+  }
+
+  bool are_extremities_inside_test()
+  {
+    return false;
+  }
+
+};
+
+//Tetrahedron_3 Line_3
+template <class K>
+typename Intersection_traits<K, typename K::Tetrahedron_3, typename K::Line_3>::result_type
+intersection(
+    const typename K::Tetrahedron_3 &tet,
+    const typename K::Line_3 &line,
+    const K&)
+{
+  Tetrahedron_Line_intersection_3<K> solver(tet, line);
+  solver.do_procede();
+  return solver.output;
+}
+
+template <class K>
+typename Intersection_traits<K, typename K::Tetrahedron_3, typename K::Line_3>::result_type
+intersection(
+    const typename K::Line_3 &line,
+    const typename K::Tetrahedron_3 &tet,
+    const K& k)
+{
+  return intersection(tet, line, k);
+}
+
+}}}
+#endif // CGAL_INTERNAL_INTERSECTIONS_3_TETRAHEDRON_3_LINE_3_INTERSECTION_H
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/internal/Tetrahedron_3_Plane_3_intersection.h 5.1-1/include/CGAL/Intersections_3/internal/Tetrahedron_3_Plane_3_intersection.h
--- 5.0.3-1/include/CGAL/Intersections_3/internal/Tetrahedron_3_Plane_3_intersection.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/internal/Tetrahedron_3_Plane_3_intersection.h	2020-09-08 12:14:58.000000000 +0000
@@ -0,0 +1,143 @@
+// Copyright (c) 2019 GeometryFactory(France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/internal/Tetrahedron_3_Plane_3_intersection.h $
+// $Id: Tetrahedron_3_Plane_3_intersection.h d395e09 2020-03-20T10:35:16+01:00 Maxime Gimeno
+// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Maxime Gimeno
+//
+
+
+#ifndef CGAL_INTERNAL_INTERSECTIONS_3_TETRAHEDRON_3_PLANE_3_INTERSECTIONS_H
+#define CGAL_INTERNAL_INTERSECTIONS_3_TETRAHEDRON_3_PLANE_3_INTERSECTIONS_H
+
+#include <CGAL/kernel_basic.h>
+#include <CGAL/intersections.h>
+#include <CGAL/Intersections_3/internal/Triangle_3_Plane_3_do_intersect.h>
+#include <CGAL/Intersections_3/internal/tetrahedron_intersection_helpers.h>
+#include <set>
+namespace CGAL {
+
+namespace Intersections {
+
+namespace internal {
+
+
+//Tetrahedron_3 Plane_3
+template <class K>
+typename Intersection_traits<K, typename K::Tetrahedron_3, typename K::Plane_3>::result_type
+intersection(
+    const typename K::Tetrahedron_3 &tet,
+    const typename K::Plane_3 &pl,
+    const K&)
+{
+  typedef typename Intersection_traits<K,
+      typename K::Tetrahedron_3,
+      typename K::Plane_3>::result_type result_type;
+
+  typedef typename Intersection_traits<K,
+      typename K::Triangle_3,
+      typename K::Plane_3>::result_type Inter_type;
+
+  typedef typename K::Segment_3 Segment_3;
+  Inter_type intersections[4];
+  int p_id = -1;
+  std::vector<typename K::Point_3> points;
+  std::vector<Segment_3> segments;
+  for(int i = 0; i < 4; ++i)
+  {
+    const typename K::Triangle_3 triangle(tet.vertex((i+1)%4),
+                                          tet.vertex((i+2)%4),
+                                          tet.vertex((i+3)%4));
+    intersections[i] = typename K::Intersect_3()(pl, triangle);
+    if(intersections[i]){
+      if(const typename K::Triangle_3* tr = boost::get<typename K::Triangle_3>(&*intersections[i]))
+      {
+        typename K::Triangle_3 res = *tr;
+        return result_type(std::forward<typename K::Triangle_3>(res));
+      }
+      else if( const Segment_3* s
+               = boost::get<Segment_3>(&*intersections[i]))
+      {
+        segments.push_back(*s);
+      }
+      else if( const typename K::Point_3* p
+               = boost::get<typename K::Point_3>(&*intersections[i]))
+      {
+        points.push_back(*p);
+        p_id = i;
+      }
+    }
+  }
+  CGAL_assertion(segments.size() != 1);
+
+  switch(segments.size())
+  {
+  case 0:
+  {
+    if(p_id == -1)
+      return result_type();
+    else
+    {
+      typename K::Point_3 p
+          = *boost::get<typename K::Point_3>(&*intersections[p_id]);
+
+      return result_type(std::forward<typename  K::Point_3>(p));
+    }
+  }
+    break;
+  case 2:
+  {
+    return result_type(std::forward<typename  K::Segment_3>(segments.back()));
+  }
+    break;
+  default:
+  {
+    std::set<typename K::Point_3> all_points;
+    for (const auto& s : segments)
+    {
+      all_points.insert(s.source());
+      all_points.insert(s.target());
+    }
+    if(all_points.size() == 3)
+    {
+      auto p_it = all_points.begin();
+      ++p_it;
+      typename K::Point_3 mid_point = *p_it;
+      ++p_it;
+      typename K::Triangle_3 result(*all_points.begin(), mid_point, *p_it );
+      return result_type(std::forward<typename K::Triangle_3>(result));
+    }
+    else //size = 4
+    {
+      std::list<Segment_3> segs(segments.begin(), segments.end());
+      std::list<typename K::Point_3> tmp;
+      fill_points_list(segs, tmp);
+      std::vector<typename K::Point_3> res;
+      for( const auto& p : tmp)
+        res.push_back(p);
+      return result_type(std::forward<std::vector<typename K::Point_3> >(res));
+    }
+  }
+    break;
+  }
+  CGAL_assertion(false);
+  return result_type();
+}
+
+template <class K>
+typename Intersection_traits<K, typename K::Tetrahedron_3, typename K::Plane_3>::result_type
+intersection(
+    const typename K::Plane_3 &pl,
+    const typename K::Tetrahedron_3 &tet,
+    const K& k)
+{
+  return intersection(tet, pl, k);
+}
+
+}}}
+#endif // CGAL_INTERNAL_INTERSECTIONS_3_TETRAHEDRON_3_PLANE_3_INTERSECTIONS_H
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/internal/Tetrahedron_3_Ray_3_intersection.h 5.1-1/include/CGAL/Intersections_3/internal/Tetrahedron_3_Ray_3_intersection.h
--- 5.0.3-1/include/CGAL/Intersections_3/internal/Tetrahedron_3_Ray_3_intersection.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/internal/Tetrahedron_3_Ray_3_intersection.h	2020-09-08 12:14:58.000000000 +0000
@@ -0,0 +1,78 @@
+// Copyright (c) 2019 GeometryFactory(France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/internal/Tetrahedron_3_Ray_3_intersection.h $
+// $Id: Tetrahedron_3_Ray_3_intersection.h ce4cbe6 2020-03-19T11:41:57+01:00 Maxime Gimeno
+// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Maxime Gimeno
+//
+
+#ifndef CGAL_INTERNAL_INTERSECTIONS_3_TETRAHEDRON_3_RAY_3_INTERSECTION_H
+#define CGAL_INTERNAL_INTERSECTIONS_3_TETRAHEDRON_3_RAY_3_INTERSECTION_H
+
+#include <CGAL/kernel_basic.h>
+#include <CGAL/intersections.h>
+#include <CGAL/Intersections_3/internal/tetrahedron_lines_intersections_3.h>
+
+namespace CGAL {
+
+namespace Intersections {
+
+namespace internal {
+
+template<class K>
+struct Tetrahedron_ray_intersection_3
+    :public Tetrahedron_lines_intersection_3_base<K,
+      typename K::Ray_3, Tetrahedron_ray_intersection_3<K> >
+{
+  typedef typename K::Ray_3 O;
+  typedef Tetrahedron_lines_intersection_3_base<K, typename K::Ray_3,
+    Tetrahedron_ray_intersection_3<K> > Base;
+  typedef typename Base::Result_type Result_type;
+
+  Tetrahedron_ray_intersection_3(const typename K::Tetrahedron_3& tet,
+                                 const O& o):Base(tet,o) {}
+
+  bool are_extremities_inside_test()
+  {
+    //If one extremity is inside tet : return a segment
+    if(this->tet.has_on_bounded_side(this->o.source()))
+    {
+      typename K::Segment_3 result(this->o.source(), this->res_points.front());
+      this->output = Result_type(std::forward<typename K::Segment_3>(result));
+      return true;
+    }
+    return false;
+  }
+};
+
+
+//Tetrahedron_3 Ray_3
+template <class K>
+typename Intersection_traits<K, typename K::Tetrahedron_3, typename K::Ray_3>::result_type
+intersection(
+    const typename K::Tetrahedron_3 &tet,
+    const typename K::Ray_3 &ray,
+    const K&)
+{
+  Tetrahedron_ray_intersection_3<K> solver(tet, ray);
+  solver.do_procede();
+  return solver.output;
+}
+
+template <class K>
+typename Intersection_traits<K, typename K::Tetrahedron_3, typename K::Ray_3>::result_type
+intersection(
+    const typename K::Ray_3 &seg,
+    const typename K::Tetrahedron_3 &tet,
+    const K& k)
+{
+  return intersection(tet, seg, k);
+}
+
+}}}
+#endif // CGAL_INTERNAL_INTERSECTIONS_3_TETRAHEDRON_3_RAY_3_INTERSECTION_H
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/internal/Tetrahedron_3_Segment_3_intersection.h 5.1-1/include/CGAL/Intersections_3/internal/Tetrahedron_3_Segment_3_intersection.h
--- 5.0.3-1/include/CGAL/Intersections_3/internal/Tetrahedron_3_Segment_3_intersection.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/internal/Tetrahedron_3_Segment_3_intersection.h	2020-09-08 12:14:58.000000000 +0000
@@ -0,0 +1,96 @@
+// Copyright (c) 2019 GeometryFactory(France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/internal/Tetrahedron_3_Segment_3_intersection.h $
+// $Id: Tetrahedron_3_Segment_3_intersection.h fe0d4d0 2020-03-19T16:41:59+01:00 Maxime Gimeno
+// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Maxime Gimeno
+//
+
+#ifndef CGAL_INTERNAL_INTERSECTIONS_3_TETRAHEDRON_3_SEGMENT_3_INTERSECTION_H
+#define CGAL_INTERNAL_INTERSECTIONS_3_TETRAHEDRON_3_SEGMENT_3_INTERSECTION_H
+
+#include <CGAL/kernel_basic.h>
+#include <CGAL/intersections.h>
+#include <CGAL/Intersections_3/internal/tetrahedron_lines_intersections_3.h>
+
+namespace CGAL {
+
+namespace Intersections {
+
+namespace internal {
+
+template<class K>
+struct Tetrahedron_segment_intersection_3
+    :public Tetrahedron_lines_intersection_3_base<K, typename K::Segment_3,
+    Tetrahedron_segment_intersection_3<K> >
+{
+  typedef typename K::Segment_3 O;
+  typedef Tetrahedron_lines_intersection_3_base<K,typename K::Segment_3,
+    Tetrahedron_segment_intersection_3<K> > Base;
+  typedef typename Base::Result_type result_type;
+
+  Tetrahedron_segment_intersection_3(const typename K::Tetrahedron_3& tet,
+                                   const O& o):Base(tet,o) {}
+
+  bool all_inside_test()
+  {
+    //if both segment extremities are inside tet, return the input segment.
+    if(this->tet.has_on_bounded_side(this->o.source())
+       && this->tet.has_on_bounded_side(this->o.target())){
+      typename K::Segment_3 result = this->o;
+      this->output = result_type(std::forward<typename K::Segment_3>(result));
+      return true;
+    }
+    return false;
+  }
+
+  bool are_extremities_inside_test()
+  {
+    //If one extremity is inside tet : return a segment
+    if(this->tet.has_on_bounded_side(this->o.source()))
+    {
+      typename K::Segment_3 result(this->o.source(), this->res_points.front());
+      this->output = result_type(std::forward<typename K::Segment_3>(result));
+      return true;
+    }
+    else if(this->tet.has_on_bounded_side(this->o.target())){
+      typename K::Segment_3 result(this->res_points.front(), this->o.target());
+      this->output = result_type(std::forward<typename K::Segment_3>(result));
+      return true;
+    }
+    return false;
+  }
+};
+
+
+//Tetrahedron_3 Segment_3
+template <class K>
+typename Intersection_traits<K, typename K::Tetrahedron_3, typename K::Segment_3>::result_type
+intersection(
+    const typename K::Tetrahedron_3 &tet,
+    const typename K::Segment_3 &seg,
+    const K&)
+{
+  Tetrahedron_segment_intersection_3<K> solver(tet, seg);
+  solver.do_procede();
+  return solver.output;
+
+}
+
+template <class K>
+typename Intersection_traits<K, typename K::Tetrahedron_3, typename K::Segment_3>::result_type
+intersection(
+    const typename K::Segment_3 &seg,
+    const typename K::Tetrahedron_3 &tet,
+    const K& k)
+{
+  return intersection(tet, seg, k);
+}
+
+}}}
+#endif // CGAL_INTERNAL_INTERSECTIONS_3_TETRAHEDRON_3_Segment_3_INTERSECTION_H
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/internal/Tetrahedron_3_Triangle_3_intersection.h 5.1-1/include/CGAL/Intersections_3/internal/Tetrahedron_3_Triangle_3_intersection.h
--- 5.0.3-1/include/CGAL/Intersections_3/internal/Tetrahedron_3_Triangle_3_intersection.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/internal/Tetrahedron_3_Triangle_3_intersection.h	2020-09-08 12:14:58.000000000 +0000
@@ -0,0 +1,296 @@
+// Copyright (c) 2019 GeometryFactory(France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/internal/Tetrahedron_3_Triangle_3_intersection.h $
+// $Id: Tetrahedron_3_Triangle_3_intersection.h 8b41189 2020-03-26T18:58:21+01:00 Sébastien Loriot
+// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Maxime Gimeno
+//
+
+
+#ifndef CGAL_INTERNAL_INTERSECTIONS_3_TETRAHEDRON_3_TRIANGLE_3_INTERSECTIONS_H
+#define CGAL_INTERNAL_INTERSECTIONS_3_TETRAHEDRON_3_TRIANGLE_3_INTERSECTIONS_H
+
+#include <CGAL/kernel_basic.h>
+#include <CGAL/intersections.h>
+#include <CGAL/Intersections_3/internal/Triangle_3_Triangle_3_intersection.h>
+#include <CGAL/Intersections_3/internal/Tetrahedron_3_Plane_3_intersection.h>
+#include <CGAL/Intersections_3/internal/tetrahedron_intersection_helpers.h>
+namespace CGAL {
+
+namespace Intersections {
+
+namespace internal {
+
+//Tetrahedron_3 Segment_3
+template <class K>
+typename Intersection_traits<K, typename K::Tetrahedron_3, typename K::Triangle_3>::result_type
+intersection(
+    const typename K::Tetrahedron_3 &tet,
+    const typename K::Triangle_3 &tr,
+    const K& k)
+{
+  typedef typename Intersection_traits<K,
+      typename K::Tetrahedron_3,
+      typename K::Triangle_3>::result_type result_type;
+
+  typedef typename Intersection_traits<K,
+      typename K::Triangle_3,
+      typename K::Triangle_3>::result_type Inter_type;
+
+  typedef typename K::Segment_3 Segment_3;
+  typedef typename K::Point_3 Point_3;
+  typedef typename K::Triangle_3 Triangle_3;
+  typedef std::vector<Point_3> Poly;
+
+  std::vector<Point_3> inside_points;
+  for(int i = 0; i< 3; ++i)
+  {
+    if(tet.has_on_bounded_side(tr.vertex(i))
+       || tet.has_on_boundary(tr.vertex(i))){
+      inside_points.push_back(tr.vertex(i));
+    }
+  }
+  switch(inside_points.size())
+  {
+  case 0:
+  {
+    Inter_type intersections[4];
+    std::vector<Segment_3> segments;
+    std::vector<std::size_t> seg_ids;
+    std::vector<Point_3> points;
+    for(std::size_t i = 0; i < 4; ++i)
+    {
+     const typename K::Triangle_3 triangle(tet.vertex((i+1)%4),
+                             tet.vertex((i+2)%4),
+                             tet.vertex((i+3)%4));
+      intersections[i] = typename K::Intersect_3()(tr, triangle);
+      if(intersections[i]){
+        //a face is inside the input tr
+        if(const Triangle_3* t = boost::get<typename K::Triangle_3>(&*intersections[i]))
+        {
+          Triangle_3 res = *t;
+          return result_type(std::forward<Triangle_3>(res));
+        }
+        //get segs and pts to construct poly
+        else if( const Segment_3* s
+                 = boost::get<Segment_3>(&*intersections[i]))
+        {
+          segments.push_back(*s);
+          seg_ids.push_back(i);
+        }
+        else if( const typename K::Point_3* p
+                 = boost::get<typename K::Point_3>(&*intersections[i]))
+        {
+          points.push_back(*p);
+        }
+        //if poly : then the input is in a supporting plane of a face, return the poly.
+        else if( const Poly* p
+                 = boost::get<Poly>(&*intersections[i]))
+        {
+          Poly res = *p;
+          return result_type(std::forward<Poly>(res));
+        }
+      }
+    }
+    if(segments.size() > 1)
+    {
+      std::vector<Segment_3> filtered;
+      filter_segments(segments, filtered);
+      segments = filtered;
+    }
+    //if there are several segments, then we need to compute the polygone.
+    if(segments.size() > 1)
+    {
+      std::list<Point_3> tmp;
+      fill_segments_infos(segments,tmp, tr);
+      Poly res;
+      res.reserve(4);
+      for(const auto& p : tmp)
+        res.push_back(p);
+      return result_type(std::forward<Poly>(res));
+    }
+    //else it must be adjacent to an vertex, so we return the point
+    else if(segments.size() == 1)
+    {
+      //adjacency to an edge, return resulting segment.
+      return result_type(std::forward<Segment_3>(segments.front()));
+    }
+    else
+    {
+      //no segment = adjacency to an vertex or an edge : return result point
+      return result_type(std::forward<Point_3>(points.front()));
+
+    }
+  }
+    break;
+  case 1:
+  case 2:
+  {
+    //tricky cases
+    Inter_type intersections[4];
+    std::vector<typename K::Point_3> points;
+    std::vector<Segment_3> segments;
+    for(std::size_t i = 0; i < 4; ++i)
+    {
+     const typename K::Triangle_3 triangle(tet.vertex((i+1)%4),
+                             tet.vertex((i+2)%4),
+                             tet.vertex((i+3)%4));
+      intersections[i] = typename K::Intersect_3()(tr, triangle);
+      if(intersections[i]){
+        if(const Triangle_3* t = boost::get<typename K::Triangle_3>(&*intersections[i]))
+        {
+          Triangle_3 res = *t;
+          return result_type(std::forward<Triangle_3>(res));
+        }
+        //get segs and pts to construct poly
+        else if( const Segment_3* s
+                 = boost::get<Segment_3>(&*intersections[i]))
+        {
+          segments.push_back(*s);
+        }
+        else if( const typename K::Point_3* p
+                 = boost::get<typename K::Point_3>(&*intersections[i]))
+        {
+          points.push_back(*p);
+        }
+        //if poly : then the input is in a supporting plane of a face, return the poly.
+        else if( const Poly* p
+                 = boost::get<Poly>(&*intersections[i]))
+        {
+          Poly res = *p;
+          return result_type(std::forward<Poly>(res));
+        }
+      }
+    }
+    if(segments.empty())
+    {
+      //then there is only one point of contact. Return it:
+      return result_type(std::forward<Point_3>(points.front()));
+    }
+
+    if(segments.size() > 1)
+    {
+      std::vector<Segment_3> filtered;
+      filter_segments(segments, filtered);
+      segments = filtered;
+    }
+
+    switch(segments.size())
+    {
+    case 1:
+    {
+      bool return_solo_seg = true;
+      //only one intersection, a triangle edge is one of the tet edges, and
+      //the 3rd point is outside. This is the only intersection.
+      for(const auto& p : inside_points)
+      {
+        if(!tet.has_on_boundary(p))
+        {
+          return_solo_seg = false;
+          break;
+        }
+      }
+      if(return_solo_seg)
+      {
+        return result_type(std::forward<Segment_3>(segments.front()));
+      }
+
+      if(inside_points.size() == 1)
+      {
+        Triangle_3 res(inside_points.front(), segments.front().source(),
+                       segments.front().target());
+        return result_type(std::forward<Triangle_3>(res));
+      }
+      else //size 2
+      {
+        Poly res(4);
+        res[0] = inside_points.front();
+        res[1] = inside_points.back();
+        if((inside_points.front() - inside_points.back()) *
+           (segments.front().source() - segments.front().target()) > 0)
+        {
+          res[2] = segments.front().target();
+          res[3] = segments.front().source();
+        }
+        else
+        {
+          res[3] = segments.front().target();
+          res[2] = segments.front().source();
+        }
+        return result_type(std::forward<Poly>(res));
+      }
+      }
+      break;
+    case 2:
+    case 3:
+    {
+      std::list<Segment_3> segs(segments.begin(), segments.end());
+      std::list<Point_3> tmp;
+      fill_points_list(segs, tmp);
+      if(inside_points.size() == 1)
+      {
+        Poly res;
+        res.reserve(4);
+        res.push_back(inside_points.front());
+        for(const auto& p : tmp)
+          res.push_back(p);
+        return result_type(std::forward<Poly>(res));
+      }
+      else //size 2
+      {
+        Poly res;
+        res.reserve(5);
+        typename K::Compute_scalar_product_3 scalar =
+          k.compute_scalar_product_3_object();
+        typename K::Construct_vector_3 construct_vector =
+          k.construct_vector_3_object();
+
+        if(scalar(construct_vector(inside_points.front(), inside_points.back()),
+           construct_vector(tmp.front(), tmp.back())) > 0)
+        {
+          res.push_back(inside_points.front());
+        }
+        res.push_back(inside_points.back());
+        for(const auto& p : tmp)
+          res.push_back(p);
+        return result_type(std::forward<Poly>(res));
+      }
+    }
+      break;
+    default:
+      //3 faces max if a point or more are inside tetrahedron
+      break;
+    }
+  }
+    break;
+
+  case 3:
+  {
+    //triangle entirely inside tetra : return input triangle
+    return result_type(tr);
+  }
+    break;
+  default:
+    //never happens (only 3 pts in a tr)
+    break;
+  }
+  return result_type();
+}
+
+template <class K>
+typename Intersection_traits<K, typename K::Tetrahedron_3, typename K::Triangle_3>::result_type
+intersection(
+    const typename K::Triangle_3 &pl,
+    const typename K::Tetrahedron_3 &tet,
+    const K& k)
+{
+  return intersection(tet, pl, k);
+}
+
+}}}
+#endif // CGAL_INTERNAL_INTERSECTIONS_3_TETRAHEDRON_3_TRIANGLE_3_INTERSECTIONS_H
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/internal/Tetrahedron_3_Unbounded_3_do_intersect.h 5.1-1/include/CGAL/Intersections_3/internal/Tetrahedron_3_Unbounded_3_do_intersect.h
--- 5.0.3-1/include/CGAL/Intersections_3/internal/Tetrahedron_3_Unbounded_3_do_intersect.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/internal/Tetrahedron_3_Unbounded_3_do_intersect.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/internal/Tetrahedron_3_Unbounded_3_do_intersect.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/internal/Tetrahedron_3_Unbounded_3_do_intersect.h $
 // $Id: Tetrahedron_3_Unbounded_3_do_intersect.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/internal/tetrahedron_intersection_helpers.h 5.1-1/include/CGAL/Intersections_3/internal/tetrahedron_intersection_helpers.h
--- 5.0.3-1/include/CGAL/Intersections_3/internal/tetrahedron_intersection_helpers.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/internal/tetrahedron_intersection_helpers.h	2020-09-08 12:14:58.000000000 +0000
@@ -0,0 +1,261 @@
+// Copyright (c) 2019 GeometryFactory(France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/internal/tetrahedron_intersection_helpers.h $
+// $Id: tetrahedron_intersection_helpers.h 401e06c 2020-03-24T16:33:54+01:00 Maxime Gimeno
+// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Maxime Gimeno
+//
+
+#ifndef CGAL_INTERNAL_TETRAHEDRON_INTERSECTION_HELPERS_H
+#define CGAL_INTERNAL_TETRAHEDRON_INTERSECTION_HELPERS_H
+
+#include <CGAL/kernel_basic.h>
+#include <list>
+#include <vector>
+
+namespace CGAL {
+namespace Intersections {
+namespace internal {
+
+template<typename Segment>
+void filter_segments(const std::vector<Segment>& input,
+                     std::vector<Segment>& output)
+{
+  std::list<Segment> tmp(input.begin(), input.end());
+
+  do
+  {
+    Segment s = tmp.back();
+    tmp.pop_back();
+    for(auto s_it = tmp.begin(); s_it != tmp.end();)
+    {
+      if(s == *s_it || s == s_it->opposite())
+      {
+        s_it = tmp.erase(s_it);
+      }
+      else {
+        ++s_it;
+      }
+    }
+    output.push_back(s);
+  }while (!tmp.empty());
+}
+
+template <typename Segment,
+          typename Point,
+          typename Triangle>
+void fill_segments_infos(std::vector<Segment>& segments,
+                         std::list<Point>& points, const Triangle& input_tr)
+{
+  struct Wrapped_segment
+  {
+    Segment segment;
+    bool s_dangling;
+    bool t_dangling;
+    std::size_t s_neighbor;
+    std::size_t t_neighbor;
+    Wrapped_segment(const Segment& s)
+      :segment(s), s_dangling(true),
+        t_dangling(true), s_neighbor(0),
+        t_neighbor(0){}
+  };
+
+  std::vector<Wrapped_segment> wrapped_segments;
+
+  for(const Segment& s:segments)
+    wrapped_segments.push_back(Wrapped_segment(s));
+
+  std::vector<Segment> bis = segments;
+  while(!bis.empty())
+  {
+    Segment s = bis.back();
+    bis.pop_back();
+    Wrapped_segment& w_s = wrapped_segments.back();
+    if(!w_s.s_dangling && ! w_s.t_dangling)
+      continue;
+    for(std::size_t i = 0; i< bis.size(); ++i)
+    {
+      const Segment& s2 = bis[i];
+      if(s2.target() == s.source())
+      {
+        w_s.s_dangling = false;
+        w_s.s_neighbor = i;
+        //same i because we empty from the bottom
+        wrapped_segments[i].t_dangling = false;
+        wrapped_segments[i].t_neighbor = bis.size();
+      }
+      else if(s2.target() == s.target())
+      {
+        w_s.t_dangling = false;
+        w_s.t_neighbor = i;
+        wrapped_segments[i].s_dangling = false;
+        wrapped_segments[i].s_neighbor = bis.size();
+        wrapped_segments[i].segment = wrapped_segments[i].segment.opposite(); //also orient the structure
+
+      }
+      else if(s2.source() == s.source())
+      {
+        w_s.s_dangling = false;
+        w_s.s_neighbor = i;
+        wrapped_segments[i].t_dangling = false;
+        wrapped_segments[i].t_neighbor = bis.size();
+        wrapped_segments[i].segment = wrapped_segments[i].segment.opposite();
+      }
+      else if(s2.source() == s.target())
+      {
+        w_s.t_dangling = false;
+        w_s.t_neighbor = i;
+        wrapped_segments[i].s_dangling = false;
+        wrapped_segments[i].s_neighbor = bis.size();
+      }
+    }
+
+    //fill dangling extremities using triangle edges
+    if(w_s.s_dangling)
+    {
+      for(int e_id = 0; e_id < 3; ++e_id)
+      {
+        Segment edge(input_tr.vertex(e_id), input_tr.vertex(e_id+1));
+        if(!edge.has_on(s.source()))
+        {
+          continue;
+        }
+        for(std::size_t i = 0; i< bis.size(); ++i)
+        {
+          if(edge.has_on(bis[i].source()))
+          {
+            w_s.s_dangling = false;
+            w_s.s_neighbor = i;
+            //same i because we empty from the bottom
+            wrapped_segments[i].t_dangling = false;
+            wrapped_segments[i].t_neighbor = bis.size();
+            wrapped_segments[i].segment = wrapped_segments[i].segment.opposite();
+          }
+          else if(edge.has_on(bis[i].target()))
+          {
+            w_s.s_dangling = false;
+            w_s.s_neighbor = i;
+            //same i because we empty from the bottom
+            wrapped_segments[i].t_dangling = false;
+            wrapped_segments[i].t_neighbor = bis.size();
+          }
+        }
+      }
+    }
+    if(w_s.t_dangling)
+    {
+      for(int e_id = 0; e_id < 3; ++e_id)
+      {
+        Segment edge(input_tr.vertex(e_id), input_tr.vertex(e_id+1));
+        if(!edge.has_on(s.target()))
+        {
+          continue;
+        }
+        for(std::size_t i = 0; i< bis.size(); ++i)
+        {
+          if(edge.has_on(bis[i].source()))
+          {
+            w_s.t_dangling = false;
+            w_s.t_neighbor = i;
+            //same i because we empty from the bottom
+            wrapped_segments[i].s_dangling = false;
+            wrapped_segments[i].s_neighbor = bis.size();
+          }
+          else if(edge.has_on(bis[i].target()))
+          {
+            w_s.t_dangling = false;
+            w_s.t_neighbor = i;
+            //same i because we empty from the bottom
+            wrapped_segments[i].s_dangling = false;
+            wrapped_segments[i].s_neighbor = bis.size();
+            wrapped_segments[i].segment = wrapped_segments[i].segment.opposite();
+          }
+        }
+      }
+    }
+
+    if(w_s.s_dangling || w_s.t_dangling)
+    {
+      std::cerr<<"Error. Kernel must have exact constructions to compute this intersection."<<std::endl;
+      return;
+    }
+  }
+
+  //finally fill points
+  std::size_t n = 0;
+  do
+  {
+    Wrapped_segment& ws = wrapped_segments[n];
+    points.push_back(ws.segment.source());
+    if(wrapped_segments[ws.t_neighbor].segment.source() != ws.segment.target())
+      points.push_back(ws.segment.target());
+    n = ws.t_neighbor;
+  }while(n!=0);
+}
+
+
+template <typename Segment,
+          typename Point>
+void fill_points_list(std::list<Segment>& segments, std::list<Point>& points)
+{
+  CGAL_assertion(segments.size() > 1);
+
+  Segment seg = segments.front();
+  segments.pop_front();
+
+  points.push_back(seg.source());
+  points.push_back(seg.target());
+
+  //find first seg with a point in common with seg.front = s2.
+  do
+  {
+    auto seg_it = segments.begin();
+    bool found = false;
+    for(;seg_it != segments.end(); ++seg_it)
+    {
+      if(seg_it->source() == points.front())
+      {
+        points.push_front(seg_it->target());
+        found = true;
+      }
+      else if(seg_it->source() == points.back())
+      {
+        points.push_back(seg_it->target());
+        found = true;
+      }
+      else if(seg_it->target() == points.front())
+      {
+        points.push_front(seg_it->source());
+        found = true;
+      }
+      else if(seg_it->target() == points.back())
+      {
+        points.push_back(seg_it->source());
+        found = true;
+      }
+      if(found)
+      {
+        break;
+      }
+    }
+    if(!found)
+    {
+      std::cerr<<"Error. Kernel must have exact constructions to compute this intersection."<<std::endl;
+      return;
+    }
+    segments.erase(seg_it);
+    //if loop, pop first point to avoid double
+    if(points.front() == points.back())
+      points.pop_front();
+  }while(!segments.empty());
+}
+
+}
+}
+}//end namespaces
+#endif // CGAL_INTERNAL_TETRAHEDRON_INTERSECTION_HELPERS_H
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/internal/tetrahedron_lines_intersections_3.h 5.1-1/include/CGAL/Intersections_3/internal/tetrahedron_lines_intersections_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/internal/tetrahedron_lines_intersections_3.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/internal/tetrahedron_lines_intersections_3.h	2020-09-08 12:14:58.000000000 +0000
@@ -0,0 +1,158 @@
+// Copyright (c) 2019 GeometryFactory(France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/internal/tetrahedron_lines_intersections_3.h $
+// $Id: tetrahedron_lines_intersections_3.h 586987e 2020-03-23T11:34:37+01:00 Maxime Gimeno
+// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Maxime Gimeno
+//
+
+#ifndef CGAL_INTERNAL_INTERSECTIONS_3_TETRAHEDRON_LINES_INTERSECTIONS_3_H
+#define CGAL_INTERNAL_INTERSECTIONS_3_TETRAHEDRON_LINES_INTERSECTIONS_3_H
+
+#include <vector>
+#include <CGAL/kernel_basic.h>
+#include <CGAL/intersections.h>
+
+namespace CGAL {
+namespace Intersections {
+namespace internal {
+
+template<class K, class O, class Derived>
+struct Tetrahedron_lines_intersection_3_base
+{
+  typedef typename Intersection_traits<K,
+  CGAL::Tetrahedron_3<K>,
+  O>::result_type Result_type;
+
+  typedef typename Intersection_traits<K,
+  CGAL::Triangle_3<K>,
+  O>::result_type Inter_type;
+
+  const typename K::Tetrahedron_3& tet;
+  const O& o;
+  std::vector<typename K::Point_3> res_points;
+  Result_type output;
+
+  Tetrahedron_lines_intersection_3_base(const typename K::Tetrahedron_3& tet,
+                                        const O& o):tet(tet), o(o)
+  {}
+
+  bool all_inside_test()
+  {
+    return false;
+  }
+
+  bool are_extremities_inside_test()
+  {
+    return false;
+  }
+
+  void do_procede()
+  {
+    if(static_cast<Derived*>(this)->all_inside_test())
+      return;
+
+    int res_id = -1;
+
+    Inter_type tr_seg[4];
+    for(int i = 0; i < 4; ++i)
+    {
+      const typename K::Triangle_3 triangle(tet.vertex((i+1)%4),
+                                            tet.vertex((i+2)%4),
+                                            tet.vertex((i+3)%4));
+      if(do_intersect(o, triangle))
+      {
+        tr_seg[i] = typename K::Intersect_3()(o, triangle);
+        if( boost::get<typename K::Segment_3>(&*tr_seg[i]) != nullptr)
+        {
+          res_id = i;
+          break;
+        }
+      }
+    }
+
+    //if there is a segment in the intersections, then we return it
+    if(res_id !=-1)
+    {
+      output = tr_seg[res_id];
+      return;
+    }
+
+    //else if there is only 1 intersection
+    res_points.reserve(4);
+    res_id = -1;
+
+    for(int i = 0; i< 4; ++i)
+    {
+      if(tr_seg[i])
+      {
+        if (const typename K::Point_3* p = boost::get<typename K::Point_3>(&*tr_seg[i]))
+        {
+          if(res_points.empty())
+          {
+            res_id = i;
+          }
+          else {
+            if(*p != res_points.front())
+            {
+              res_id = -1;
+            }
+          }
+          res_points.push_back(*p);
+        }
+      }
+    }
+
+    if(res_points.empty())
+    {
+      return;
+    }
+
+    if(res_id != -1)
+    {
+      if(static_cast<Derived*>(this)->are_extremities_inside_test())
+        return;
+      //else point and segment entirely not inside or one intersection on boundary:
+      output = tr_seg[res_id];
+      return;
+    }
+
+    //else, we return a segment of the 2 intersection points (the most far away, in case of inexact)
+    typename K::FT max_dist = 0;
+    std::size_t res_id_2 = -1;
+    std::vector<std::vector<typename K::FT> > sq_distances(res_points.size());
+
+    for(std::size_t i = 0; i< res_points.size(); ++i)
+    {
+      const auto& p1 = res_points[i];
+      for(const auto& p2 : res_points)
+      {
+        sq_distances[i].push_back(CGAL::squared_distance(p1, p2));
+        if(sq_distances[i].back() > max_dist)
+        {
+          res_id = static_cast<int>(i);
+          res_id_2 = sq_distances[i].size()-1;
+          max_dist = sq_distances[i].back();
+        }
+      }
+    }
+
+    CGAL_assertion(res_id != -1);
+    CGAL_assertion(res_id_2 != static_cast<std::size_t>(-1));
+    CGAL_assertion(max_dist >0 );
+
+    typename K::Segment_3 res_seg(res_points[res_id], res_points[res_id_2]);
+    output = Result_type(std::forward<typename K::Segment_3>(res_seg));
+    return;
+  }
+};
+
+}
+}
+} //end namespaces
+#endif // CGAL_INTERNAL_INTERSECTIONS_3_TETRAHEDRON_LINES_INTERSECTIONS_3_H
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/internal/Triangle_3_Line_3_do_intersect.h 5.1-1/include/CGAL/Intersections_3/internal/Triangle_3_Line_3_do_intersect.h
--- 5.0.3-1/include/CGAL/Intersections_3/internal/Triangle_3_Line_3_do_intersect.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/internal/Triangle_3_Line_3_do_intersect.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/internal/Triangle_3_Line_3_do_intersect.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/internal/Triangle_3_Line_3_do_intersect.h $
 // $Id: Triangle_3_Line_3_do_intersect.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/internal/Triangle_3_Line_3_intersection.h 5.1-1/include/CGAL/Intersections_3/internal/Triangle_3_Line_3_intersection.h
--- 5.0.3-1/include/CGAL/Intersections_3/internal/Triangle_3_Line_3_intersection.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/internal/Triangle_3_Line_3_intersection.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/internal/Triangle_3_Line_3_intersection.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/internal/Triangle_3_Line_3_intersection.h $
 // $Id: Triangle_3_Line_3_intersection.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/internal/Triangle_3_Plane_3_do_intersect.h 5.1-1/include/CGAL/Intersections_3/internal/Triangle_3_Plane_3_do_intersect.h
--- 5.0.3-1/include/CGAL/Intersections_3/internal/Triangle_3_Plane_3_do_intersect.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/internal/Triangle_3_Plane_3_do_intersect.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/internal/Triangle_3_Plane_3_do_intersect.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/internal/Triangle_3_Plane_3_do_intersect.h $
 // $Id: Triangle_3_Plane_3_do_intersect.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/internal/Triangle_3_Ray_3_do_intersect.h 5.1-1/include/CGAL/Intersections_3/internal/Triangle_3_Ray_3_do_intersect.h
--- 5.0.3-1/include/CGAL/Intersections_3/internal/Triangle_3_Ray_3_do_intersect.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/internal/Triangle_3_Ray_3_do_intersect.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/internal/Triangle_3_Ray_3_do_intersect.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/internal/Triangle_3_Ray_3_do_intersect.h $
 // $Id: Triangle_3_Ray_3_do_intersect.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/internal/Triangle_3_Ray_3_intersection.h 5.1-1/include/CGAL/Intersections_3/internal/Triangle_3_Ray_3_intersection.h
--- 5.0.3-1/include/CGAL/Intersections_3/internal/Triangle_3_Ray_3_intersection.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/internal/Triangle_3_Ray_3_intersection.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/internal/Triangle_3_Ray_3_intersection.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/internal/Triangle_3_Ray_3_intersection.h $
 // $Id: Triangle_3_Ray_3_intersection.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/internal/Triangle_3_Segment_3_do_intersect.h 5.1-1/include/CGAL/Intersections_3/internal/Triangle_3_Segment_3_do_intersect.h
--- 5.0.3-1/include/CGAL/Intersections_3/internal/Triangle_3_Segment_3_do_intersect.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/internal/Triangle_3_Segment_3_do_intersect.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/internal/Triangle_3_Segment_3_do_intersect.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/internal/Triangle_3_Segment_3_do_intersect.h $
 // $Id: Triangle_3_Segment_3_do_intersect.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/internal/Triangle_3_Segment_3_intersection.h 5.1-1/include/CGAL/Intersections_3/internal/Triangle_3_Segment_3_intersection.h
--- 5.0.3-1/include/CGAL/Intersections_3/internal/Triangle_3_Segment_3_intersection.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/internal/Triangle_3_Segment_3_intersection.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/internal/Triangle_3_Segment_3_intersection.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/internal/Triangle_3_Segment_3_intersection.h $
 // $Id: Triangle_3_Segment_3_intersection.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/internal/Triangle_3_Sphere_3_do_intersect.h 5.1-1/include/CGAL/Intersections_3/internal/Triangle_3_Sphere_3_do_intersect.h
--- 5.0.3-1/include/CGAL/Intersections_3/internal/Triangle_3_Sphere_3_do_intersect.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/internal/Triangle_3_Sphere_3_do_intersect.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/internal/Triangle_3_Sphere_3_do_intersect.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/internal/Triangle_3_Sphere_3_do_intersect.h $
 // $Id: Triangle_3_Sphere_3_do_intersect.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/internal/Triangle_3_Triangle_3_do_intersect.h 5.1-1/include/CGAL/Intersections_3/internal/Triangle_3_Triangle_3_do_intersect.h
--- 5.0.3-1/include/CGAL/Intersections_3/internal/Triangle_3_Triangle_3_do_intersect.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/internal/Triangle_3_Triangle_3_do_intersect.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/internal/Triangle_3_Triangle_3_do_intersect.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/internal/Triangle_3_Triangle_3_do_intersect.h $
 // $Id: Triangle_3_Triangle_3_do_intersect.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/internal/Triangle_3_Triangle_3_intersection.h 5.1-1/include/CGAL/Intersections_3/internal/Triangle_3_Triangle_3_intersection.h
--- 5.0.3-1/include/CGAL/Intersections_3/internal/Triangle_3_Triangle_3_intersection.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/internal/Triangle_3_Triangle_3_intersection.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/internal/Triangle_3_Triangle_3_intersection.h $
-// $Id: Triangle_3_Triangle_3_intersection.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/internal/Triangle_3_Triangle_3_intersection.h $
+// $Id: Triangle_3_Triangle_3_intersection.h 8b41189 2020-03-26T18:58:21+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -128,7 +128,7 @@ struct Triangle_Line_visitor {
 
   result_type
   operator()(const typename K::Point_3& p, const typename K::Segment_3& s) const {
-    if ( s.has_on(p) )
+    if ( typename K::Has_on_3()(s,p) )
       return intersection_return<typename K::Intersect_3, typename K::Triangle_3, typename K::Triangle_3>(p);
     else
       return result_type();
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Iso_cuboid_3_Iso_cuboid_3.h 5.1-1/include/CGAL/Intersections_3/Iso_cuboid_3_Iso_cuboid_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Iso_cuboid_3_Iso_cuboid_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Iso_cuboid_3_Iso_cuboid_3.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Iso_cuboid_3_Iso_cuboid_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Iso_cuboid_3_Iso_cuboid_3.h $
 // $Id: Iso_cuboid_3_Iso_cuboid_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Iso_cuboid_3_Line_3.h 5.1-1/include/CGAL/Intersections_3/Iso_cuboid_3_Line_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Iso_cuboid_3_Line_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Iso_cuboid_3_Line_3.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Iso_cuboid_3_Line_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Iso_cuboid_3_Line_3.h $
 // $Id: Iso_cuboid_3_Line_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Iso_cuboid_3_Plane_3.h 5.1-1/include/CGAL/Intersections_3/Iso_cuboid_3_Plane_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Iso_cuboid_3_Plane_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Iso_cuboid_3_Plane_3.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Iso_cuboid_3_Plane_3.h $
-// $Id: Iso_cuboid_3_Plane_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Iso_cuboid_3_Plane_3.h $
+// $Id: Iso_cuboid_3_Plane_3.h 90d2e03 2020-01-15T13:32:11+01:00 Maxime Gimeno
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -17,10 +17,12 @@
 #include <CGAL/Iso_cuboid_3.h>
 #include <CGAL/Plane_3.h>
 #include <CGAL/Intersections_3/internal/Bbox_3_Plane_3_do_intersect.h>
+#include <CGAL/Intersections_3/internal/Iso_cuboid_3_Plane_3_intersection.h>
 
 
 namespace CGAL {
   CGAL_DO_INTERSECT_FUNCTION(Iso_cuboid_3, Plane_3, 3)
+  CGAL_INTERSECTION_FUNCTION(Iso_cuboid_3, Plane_3, 3)
 }
 
 #endif // CGAL_INTERSECTIONS_3_ISO_CUBOID_3_PLANE_3_H
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Iso_cuboid_3_Point_3.h 5.1-1/include/CGAL/Intersections_3/Iso_cuboid_3_Point_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Iso_cuboid_3_Point_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Iso_cuboid_3_Point_3.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Iso_cuboid_3_Point_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Iso_cuboid_3_Point_3.h $
 // $Id: Iso_cuboid_3_Point_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Iso_cuboid_3_Ray_3.h 5.1-1/include/CGAL/Intersections_3/Iso_cuboid_3_Ray_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Iso_cuboid_3_Ray_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Iso_cuboid_3_Ray_3.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Iso_cuboid_3_Ray_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Iso_cuboid_3_Ray_3.h $
 // $Id: Iso_cuboid_3_Ray_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Iso_cuboid_3_Segment_3.h 5.1-1/include/CGAL/Intersections_3/Iso_cuboid_3_Segment_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Iso_cuboid_3_Segment_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Iso_cuboid_3_Segment_3.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Iso_cuboid_3_Segment_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Iso_cuboid_3_Segment_3.h $
 // $Id: Iso_cuboid_3_Segment_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Iso_cuboid_3_Sphere_3.h 5.1-1/include/CGAL/Intersections_3/Iso_cuboid_3_Sphere_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Iso_cuboid_3_Sphere_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Iso_cuboid_3_Sphere_3.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Iso_cuboid_3_Sphere_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Iso_cuboid_3_Sphere_3.h $
 // $Id: Iso_cuboid_3_Sphere_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Iso_cuboid_3_Tetrahedron_3.h 5.1-1/include/CGAL/Intersections_3/Iso_cuboid_3_Tetrahedron_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Iso_cuboid_3_Tetrahedron_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Iso_cuboid_3_Tetrahedron_3.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Iso_cuboid_3_Tetrahedron_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Iso_cuboid_3_Tetrahedron_3.h $
 // $Id: Iso_cuboid_3_Tetrahedron_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Iso_cuboid_3_Triangle_3.h 5.1-1/include/CGAL/Intersections_3/Iso_cuboid_3_Triangle_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Iso_cuboid_3_Triangle_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Iso_cuboid_3_Triangle_3.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Iso_cuboid_3_Triangle_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Iso_cuboid_3_Triangle_3.h $
 // $Id: Iso_cuboid_3_Triangle_3.h ee357fd 2019-10-29T15:19:24+01:00 Laurent Rineau
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Line_3_Line_3.h 5.1-1/include/CGAL/Intersections_3/Line_3_Line_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Line_3_Line_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Line_3_Line_3.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Line_3_Line_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Line_3_Line_3.h $
 // $Id: Line_3_Line_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Line_3_Plane_3.h 5.1-1/include/CGAL/Intersections_3/Line_3_Plane_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Line_3_Plane_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Line_3_Plane_3.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Line_3_Plane_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Line_3_Plane_3.h $
 // $Id: Line_3_Plane_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Line_3_Point_3.h 5.1-1/include/CGAL/Intersections_3/Line_3_Point_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Line_3_Point_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Line_3_Point_3.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Line_3_Point_3.h $
-// $Id: Line_3_Point_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Line_3_Point_3.h $
+// $Id: Line_3_Point_3.h 0f3305f 2020-01-16T17:20:13+01:00 Maxime Gimeno
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -27,18 +27,18 @@ template <class K>
 inline bool
 do_intersect(const typename K::Point_3 &pt,
              const typename K::Line_3 &line,
-             const K&)
+             const K& k)
 {
-  return line.has_on(pt);
+  return k.has_on_3_object()(line,pt);
 }
 
 template <class K>
 inline bool
 do_intersect(const typename K::Line_3 &line,
              const typename K::Point_3 &pt,
-             const K&)
+             const K& k)
 {
-  return line.has_on(pt);
+  return k.has_on_3_object()(line, pt);
 }
 
 template <class K>
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Line_3_Ray_3.h 5.1-1/include/CGAL/Intersections_3/Line_3_Ray_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Line_3_Ray_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Line_3_Ray_3.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Line_3_Ray_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Line_3_Ray_3.h $
 // $Id: Line_3_Ray_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Line_3_Segment_3.h 5.1-1/include/CGAL/Intersections_3/Line_3_Segment_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Line_3_Segment_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Line_3_Segment_3.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Line_3_Segment_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Line_3_Segment_3.h $
 // $Id: Line_3_Segment_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Line_3_Sphere_3.h 5.1-1/include/CGAL/Intersections_3/Line_3_Sphere_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Line_3_Sphere_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Line_3_Sphere_3.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Line_3_Sphere_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Line_3_Sphere_3.h $
 // $Id: Line_3_Sphere_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Line_3_Tetrahedron_3.h 5.1-1/include/CGAL/Intersections_3/Line_3_Tetrahedron_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Line_3_Tetrahedron_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Line_3_Tetrahedron_3.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Line_3_Tetrahedron_3.h $
-// $Id: Line_3_Tetrahedron_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Line_3_Tetrahedron_3.h $
+// $Id: Line_3_Tetrahedron_3.h 90d2e03 2020-01-15T13:32:11+01:00 Maxime Gimeno
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -17,10 +17,24 @@
 #include <CGAL/Line_3.h>
 #include <CGAL/Tetrahedron_3.h>
 
-#include <CGAL/Intersections_3/internal/Tetrahedron_3_Unbounded_3_do_intersect.h>
+#include <CGAL/Intersections_3/internal/Tetrahedron_3_Line_3_intersection.h>
 
 namespace CGAL {
   CGAL_DO_INTERSECT_FUNCTION(Line_3, Tetrahedron_3, 3)
+
+  template<typename K>
+  typename Intersection_traits<K, typename K::Line_3, typename K::Tetrahedron_3>::result_type
+  intersection(const Line_3<K>& line,
+               const Tetrahedron_3<K>& tet) {
+    return K().intersect_3_object()(line, tet);
+  }
+
+  template<typename K>
+  typename Intersection_traits<K, typename K::Line_3, typename K::Tetrahedron_3>::result_type
+  intersection(const Tetrahedron_3<K>& tet,
+      const Line_3<K>& line) {
+    return K().intersect_3_object()(tet, line);
+  }
 }
 
 #endif // CGAL_INTERSECTIONS_3_LINE_3_TETRAHEDRON_3_H
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Line_3_Triangle_3.h 5.1-1/include/CGAL/Intersections_3/Line_3_Triangle_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Line_3_Triangle_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Line_3_Triangle_3.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Line_3_Triangle_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Line_3_Triangle_3.h $
 // $Id: Line_3_Triangle_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Plane_3_Plane_3.h 5.1-1/include/CGAL/Intersections_3/Plane_3_Plane_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Plane_3_Plane_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Plane_3_Plane_3.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Plane_3_Plane_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Plane_3_Plane_3.h $
 // $Id: Plane_3_Plane_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Plane_3_Point_3.h 5.1-1/include/CGAL/Intersections_3/Plane_3_Point_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Plane_3_Point_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Plane_3_Point_3.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Plane_3_Point_3.h $
-// $Id: Plane_3_Point_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Plane_3_Point_3.h $
+// $Id: Plane_3_Point_3.h 0f3305f 2020-01-16T17:20:13+01:00 Maxime Gimeno
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -27,18 +27,18 @@ template <class K>
 inline bool
 do_intersect(const typename K::Point_3 &pt,
              const typename K::Plane_3 &plane,
-             const K&)
+             const K& k)
 {
-  return plane.has_on(pt);
+  return k.has_on_3_object()(plane,pt);
 }
 
 template <class K>
 inline bool
 do_intersect(const typename K::Plane_3 &plane,
              const typename K::Point_3 &pt,
-             const K&)
+             const K& k)
 {
-  return plane.has_on(pt);
+  return k.has_on_3_object()(plane,pt);
 }
 
 template <class K>
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Plane_3_Ray_3.h 5.1-1/include/CGAL/Intersections_3/Plane_3_Ray_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Plane_3_Ray_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Plane_3_Ray_3.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Plane_3_Ray_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Plane_3_Ray_3.h $
 // $Id: Plane_3_Ray_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Plane_3_Segment_3.h 5.1-1/include/CGAL/Intersections_3/Plane_3_Segment_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Plane_3_Segment_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Plane_3_Segment_3.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Plane_3_Segment_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Plane_3_Segment_3.h $
 // $Id: Plane_3_Segment_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Plane_3_Sphere_3.h 5.1-1/include/CGAL/Intersections_3/Plane_3_Sphere_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Plane_3_Sphere_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Plane_3_Sphere_3.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Plane_3_Sphere_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Plane_3_Sphere_3.h $
 // $Id: Plane_3_Sphere_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Plane_3_Tetrahedron_3.h 5.1-1/include/CGAL/Intersections_3/Plane_3_Tetrahedron_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Plane_3_Tetrahedron_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Plane_3_Tetrahedron_3.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Plane_3_Tetrahedron_3.h $
-// $Id: Plane_3_Tetrahedron_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Plane_3_Tetrahedron_3.h $
+// $Id: Plane_3_Tetrahedron_3.h 90d2e03 2020-01-15T13:32:11+01:00 Maxime Gimeno
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -18,9 +18,11 @@
 #include <CGAL/Tetrahedron_3.h>
 
 #include <CGAL/Intersections_3/internal/Tetrahedron_3_Unbounded_3_do_intersect.h>
+#include <CGAL/Intersections_3/internal/Tetrahedron_3_Plane_3_intersection.h>
 
 namespace CGAL {
   CGAL_DO_INTERSECT_FUNCTION(Tetrahedron_3, Plane_3, 3)
+  CGAL_INTERSECTION_FUNCTION(Tetrahedron_3, Plane_3, 3)
 }
 
 #endif // CGAL_INTERSECTIONS_3_PLANE_3_TETRAHEDRON_3_H
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Plane_3_Triangle_3.h 5.1-1/include/CGAL/Intersections_3/Plane_3_Triangle_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Plane_3_Triangle_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Plane_3_Triangle_3.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Plane_3_Triangle_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Plane_3_Triangle_3.h $
 // $Id: Plane_3_Triangle_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Point_3_Point_3.h 5.1-1/include/CGAL/Intersections_3/Point_3_Point_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Point_3_Point_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Point_3_Point_3.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Point_3_Point_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Point_3_Point_3.h $
 // $Id: Point_3_Point_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Point_3_Ray_3.h 5.1-1/include/CGAL/Intersections_3/Point_3_Ray_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Point_3_Ray_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Point_3_Ray_3.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Point_3_Ray_3.h $
-// $Id: Point_3_Ray_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Point_3_Ray_3.h $
+// $Id: Point_3_Ray_3.h 0f3305f 2020-01-16T17:20:13+01:00 Maxime Gimeno
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -28,9 +28,9 @@ inline
 bool
 do_intersect(const typename K::Point_3 &pt,
              const typename K::Ray_3 &ray,
-             const K&)
+             const K& k)
 {
-  return ray.has_on(pt);
+  return k.has_on_3_object()(ray,pt);
 }
 
 
@@ -39,9 +39,9 @@ inline
 bool
 do_intersect(const typename K::Ray_3 &ray,
              const typename K::Point_3 &pt,
-             const K&)
+             const K& k)
 {
-  return ray.has_on(pt);
+  return k.has_on_3_object()(ray,pt);
 }
 
 
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Point_3_Segment_3.h 5.1-1/include/CGAL/Intersections_3/Point_3_Segment_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Point_3_Segment_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Point_3_Segment_3.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Point_3_Segment_3.h $
-// $Id: Point_3_Segment_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Point_3_Segment_3.h $
+// $Id: Point_3_Segment_3.h 0f3305f 2020-01-16T17:20:13+01:00 Maxime Gimeno
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -28,9 +28,9 @@ inline
 bool
 do_intersect(const typename K::Point_3 &pt,
              const typename K::Segment_3 &seg,
-             const K&)
+             const K& k)
 {
-    return seg.has_on(pt);
+    return k.has_on_3_object()(seg, pt);
 }
 
 template <class K>
@@ -38,9 +38,9 @@ inline
 bool
 do_intersect(const typename K::Segment_3 &seg,
              const typename K::Point_3 &pt,
-             const K&)
+             const K& k)
 {
-    return seg.has_on(pt);
+    return k.has_on_3_object()(seg, pt);
 }
 
 
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Point_3_Sphere_3.h 5.1-1/include/CGAL/Intersections_3/Point_3_Sphere_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Point_3_Sphere_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Point_3_Sphere_3.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Point_3_Sphere_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Point_3_Sphere_3.h $
 // $Id: Point_3_Sphere_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Point_3_Tetrahedron_3.h 5.1-1/include/CGAL/Intersections_3/Point_3_Tetrahedron_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Point_3_Tetrahedron_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Point_3_Tetrahedron_3.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Point_3_Tetrahedron_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Point_3_Tetrahedron_3.h $
 // $Id: Point_3_Tetrahedron_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Point_3_Triangle_3.h 5.1-1/include/CGAL/Intersections_3/Point_3_Triangle_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Point_3_Triangle_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Point_3_Triangle_3.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Point_3_Triangle_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Point_3_Triangle_3.h $
 // $Id: Point_3_Triangle_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Ray_3_Ray_3.h 5.1-1/include/CGAL/Intersections_3/Ray_3_Ray_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Ray_3_Ray_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Ray_3_Ray_3.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Ray_3_Ray_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Ray_3_Ray_3.h $
 // $Id: Ray_3_Ray_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Ray_3_Segment_3.h 5.1-1/include/CGAL/Intersections_3/Ray_3_Segment_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Ray_3_Segment_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Ray_3_Segment_3.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Ray_3_Segment_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Ray_3_Segment_3.h $
 // $Id: Ray_3_Segment_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Ray_3_Sphere_3.h 5.1-1/include/CGAL/Intersections_3/Ray_3_Sphere_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Ray_3_Sphere_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Ray_3_Sphere_3.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Ray_3_Sphere_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Ray_3_Sphere_3.h $
 // $Id: Ray_3_Sphere_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Ray_3_Tetrahedron_3.h 5.1-1/include/CGAL/Intersections_3/Ray_3_Tetrahedron_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Ray_3_Tetrahedron_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Ray_3_Tetrahedron_3.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Ray_3_Tetrahedron_3.h $
-// $Id: Ray_3_Tetrahedron_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Ray_3_Tetrahedron_3.h $
+// $Id: Ray_3_Tetrahedron_3.h 90d2e03 2020-01-15T13:32:11+01:00 Maxime Gimeno
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -18,9 +18,11 @@
 #include <CGAL/Tetrahedron_3.h>
 
 #include <CGAL/Intersections_3/internal/Tetrahedron_3_Unbounded_3_do_intersect.h>
+#include <CGAL/Intersections_3/internal/Tetrahedron_3_Ray_3_intersection.h>
 
 namespace CGAL {
   CGAL_DO_INTERSECT_FUNCTION(Ray_3, Tetrahedron_3, 3)
+  CGAL_INTERSECTION_FUNCTION(Ray_3, Tetrahedron_3, 3)
 }
 
 #endif // CGAL_INTERSECTIONS_3_RAY_3_TETRAHEDRON_3_H
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Ray_3_Triangle_3.h 5.1-1/include/CGAL/Intersections_3/Ray_3_Triangle_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Ray_3_Triangle_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Ray_3_Triangle_3.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Ray_3_Triangle_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Ray_3_Triangle_3.h $
 // $Id: Ray_3_Triangle_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Segment_3_Segment_3.h 5.1-1/include/CGAL/Intersections_3/Segment_3_Segment_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Segment_3_Segment_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Segment_3_Segment_3.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Segment_3_Segment_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Segment_3_Segment_3.h $
 // $Id: Segment_3_Segment_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Segment_3_Sphere_3.h 5.1-1/include/CGAL/Intersections_3/Segment_3_Sphere_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Segment_3_Sphere_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Segment_3_Sphere_3.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Segment_3_Sphere_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Segment_3_Sphere_3.h $
 // $Id: Segment_3_Sphere_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Segment_3_Tetrahedron_3.h 5.1-1/include/CGAL/Intersections_3/Segment_3_Tetrahedron_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Segment_3_Tetrahedron_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Segment_3_Tetrahedron_3.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Segment_3_Tetrahedron_3.h $
-// $Id: Segment_3_Tetrahedron_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Segment_3_Tetrahedron_3.h $
+// $Id: Segment_3_Tetrahedron_3.h 90d2e03 2020-01-15T13:32:11+01:00 Maxime Gimeno
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -18,9 +18,11 @@
 #include <CGAL/Tetrahedron_3.h>
 
 #include <CGAL/Intersections_3/internal/Tetrahedron_3_Bounded_3_do_intersect.h>
+#include <CGAL/Intersections_3/internal/Tetrahedron_3_Segment_3_intersection.h>
 
 namespace CGAL {
   CGAL_DO_INTERSECT_FUNCTION(Segment_3, Tetrahedron_3, 3)
+  CGAL_INTERSECTION_FUNCTION(Segment_3, Tetrahedron_3, 3)
 }
 
 #endif // CGAL_INTERSECTIONS_3_SEGMENT_3_TETRAHEDRON_3_H
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Segment_3_Triangle_3.h 5.1-1/include/CGAL/Intersections_3/Segment_3_Triangle_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Segment_3_Triangle_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Segment_3_Triangle_3.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Segment_3_Triangle_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Segment_3_Triangle_3.h $
 // $Id: Segment_3_Triangle_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Sphere_3_Sphere_3.h 5.1-1/include/CGAL/Intersections_3/Sphere_3_Sphere_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Sphere_3_Sphere_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Sphere_3_Sphere_3.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Sphere_3_Sphere_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Sphere_3_Sphere_3.h $
 // $Id: Sphere_3_Sphere_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Sphere_3_Tetrahedron_3.h 5.1-1/include/CGAL/Intersections_3/Sphere_3_Tetrahedron_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Sphere_3_Tetrahedron_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Sphere_3_Tetrahedron_3.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Sphere_3_Tetrahedron_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Sphere_3_Tetrahedron_3.h $
 // $Id: Sphere_3_Tetrahedron_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Sphere_3_Triangle_3.h 5.1-1/include/CGAL/Intersections_3/Sphere_3_Triangle_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Sphere_3_Triangle_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Sphere_3_Triangle_3.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Sphere_3_Triangle_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Sphere_3_Triangle_3.h $
 // $Id: Sphere_3_Triangle_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Tetrahedron_3_Tetrahedron_3.h 5.1-1/include/CGAL/Intersections_3/Tetrahedron_3_Tetrahedron_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Tetrahedron_3_Tetrahedron_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Tetrahedron_3_Tetrahedron_3.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Tetrahedron_3_Tetrahedron_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Tetrahedron_3_Tetrahedron_3.h $
 // $Id: Tetrahedron_3_Tetrahedron_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Tetrahedron_3_Triangle_3.h 5.1-1/include/CGAL/Intersections_3/Tetrahedron_3_Triangle_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Tetrahedron_3_Triangle_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Tetrahedron_3_Triangle_3.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Tetrahedron_3_Triangle_3.h $
-// $Id: Tetrahedron_3_Triangle_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Tetrahedron_3_Triangle_3.h $
+// $Id: Tetrahedron_3_Triangle_3.h 90d2e03 2020-01-15T13:32:11+01:00 Maxime Gimeno
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -18,9 +18,11 @@
 #include <CGAL/Tetrahedron_3.h>
 
 #include <CGAL/Intersections_3/internal/Tetrahedron_3_Bounded_3_do_intersect.h>
+#include <CGAL/Intersections_3/internal/Tetrahedron_3_Triangle_3_intersection.h>
 
 namespace CGAL {
   CGAL_DO_INTERSECT_FUNCTION(Tetrahedron_3, Triangle_3, 3)
+  CGAL_INTERSECTION_FUNCTION(Tetrahedron_3, Triangle_3, 3)
 }
 
 #endif // CGAL_INTERSECTIONS_3_TETRAHEDRON_3_TRIANGLE_3_H
diff -pruN 5.0.3-1/include/CGAL/Intersections_3/Triangle_3_Triangle_3.h 5.1-1/include/CGAL/Intersections_3/Triangle_3_Triangle_3.h
--- 5.0.3-1/include/CGAL/Intersections_3/Triangle_3_Triangle_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersections_3/Triangle_3_Triangle_3.h	2020-09-08 12:14:58.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersections_3/Triangle_3_Triangle_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersections_3/Triangle_3_Triangle_3.h $
 // $Id: Triangle_3_Triangle_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/intersections_d.h 5.1-1/include/CGAL/intersections_d.h
--- 5.0.3-1/include/CGAL/intersections_d.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/intersections_d.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/intersections_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/intersections_d.h $
 // $Id: intersections_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/intersections.h 5.1-1/include/CGAL/intersections.h
--- 5.0.3-1/include/CGAL/intersections.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/intersections.h	2020-09-08 12:14:58.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/intersections.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/intersections.h $
 // $Id: intersections.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Intersection_traits_2.h 5.1-1/include/CGAL/Intersection_traits_2.h
--- 5.0.3-1/include/CGAL/Intersection_traits_2.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersection_traits_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_2/include/CGAL/Intersection_traits_2.h $
-// $Id: Intersection_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersection_traits_2.h $
+// $Id: Intersection_traits_2.h 8b41189 2020-03-26T18:58:21+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -105,6 +105,28 @@ template<typename K>
 struct Intersection_traits<K, typename K::Triangle_2, typename K::Iso_rectangle_2>
   : public Intersection_traits<K, typename K::Iso_rectangle_2, typename K::Triangle_2> {};
 
+template<typename K, class B>
+struct Intersection_traits<K, CGAL::Bbox_2, B>  {
+  typedef typename Intersection_traits<K,typename K::Iso_rectangle_2, B>::result_type result_type;
+};
+
+template<typename K, class A>
+struct Intersection_traits<K, A, CGAL::Bbox_2>  {
+  typedef typename Intersection_traits<K,typename K::Iso_rectangle_2, A>::result_type result_type;
+};
+
+template<typename K>
+struct Intersection_traits<K, CGAL::Bbox_2, typename K::Point_2>  {
+  typedef typename boost::variant<typename K::Point_2> variant_type;
+  typedef boost::optional<variant_type> result_type;
+};
+
+template<typename K>
+struct Intersection_traits<K, typename K::Point_2, CGAL::Bbox_2>  {
+  typedef typename boost::variant<typename K::Point_2> variant_type;
+  typedef boost::optional<variant_type> result_type;
+};
+
 } // namespace CGAL
 
 #endif /* CGAL_INTERSECTION_TRAITS_2_H */
diff -pruN 5.0.3-1/include/CGAL/Intersection_traits_3.h 5.1-1/include/CGAL/Intersection_traits_3.h
--- 5.0.3-1/include/CGAL/Intersection_traits_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersection_traits_3.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_3/include/CGAL/Intersection_traits_3.h $
-// $Id: Intersection_traits_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_3/include/CGAL/Intersection_traits_3.h $
+// $Id: Intersection_traits_3.h 8b41189 2020-03-26T18:58:21+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -32,6 +32,9 @@ CGAL_INTERSECTION_TRAITS_2(Segment_3, Li
 CGAL_INTERSECTION_TRAITS_2(Line_3, Triangle_3, Point_3, Segment_3)
 CGAL_INTERSECTION_TRAITS_2(Triangle_3, Line_3, Point_3, Segment_3)
 
+CGAL_INTERSECTION_TRAITS_2(Line_3, Tetrahedron_3, Point_3, Segment_3)
+CGAL_INTERSECTION_TRAITS_2(Tetrahedron_3, Line_3, Point_3, Segment_3)
+
 CGAL_INTERSECTION_TRAITS_2(Plane_3, Plane_3, Line_3, Plane_3)
 
 CGAL_INTERSECTION_TRAITS_2(Plane_3, Ray_3, Point_3, Ray_3)
@@ -54,11 +57,17 @@ CGAL_INTERSECTION_TRAITS_2(Segment_3, Ra
 CGAL_INTERSECTION_TRAITS_2(Ray_3, Triangle_3, Point_3, Segment_3)
 CGAL_INTERSECTION_TRAITS_2(Triangle_3, Ray_3, Point_3, Segment_3)
 
+CGAL_INTERSECTION_TRAITS_2(Ray_3, Tetrahedron_3, Point_3, Segment_3)
+CGAL_INTERSECTION_TRAITS_2(Tetrahedron_3, Ray_3, Point_3, Segment_3)
+
 CGAL_INTERSECTION_TRAITS_2(Segment_3, Segment_3, Point_3, Segment_3)
 
 CGAL_INTERSECTION_TRAITS_2(Segment_3, Triangle_3, Point_3, Segment_3)
 CGAL_INTERSECTION_TRAITS_2(Triangle_3, Segment_3, Point_3, Segment_3)
 
+CGAL_INTERSECTION_TRAITS_2(Segment_3, Tetrahedron_3, Point_3, Segment_3)
+CGAL_INTERSECTION_TRAITS_2(Tetrahedron_3, Segment_3, Point_3, Segment_3)
+
 CGAL_INTERSECTION_TRAITS_3(Sphere_3, Sphere_3, Point_3, Circle_3, Sphere_3)
 
 template<typename K>
@@ -147,11 +156,43 @@ struct Intersection_traits<K, typename K
   typedef typename boost::optional< variant_type > result_type;
 };
 
+
+// Bbox_3 Iso_cuboid_3, variant of 1
+template<typename K>
+struct Intersection_traits<K, CGAL::Bbox_3, typename K::Iso_cuboid_3>  {
+  typedef typename
+  boost::variant<  typename K::Iso_cuboid_3 > variant_type;
+  typedef typename boost::optional< variant_type > result_type;
+};
+
+template<typename K>
+struct Intersection_traits<K, typename K::Iso_cuboid_3, CGAL::Bbox_3>
+    : public Intersection_traits<K, CGAL::Bbox_3, typename K::Iso_cuboid_3> {};
+
 // Iso_cuboid_3 Point_3, variant of one
 template<typename K>
 struct Intersection_traits<K, typename K::Iso_cuboid_3, typename K::Point_3>  {
   typedef typename
-  boost::variant< typename K::Point_3 > variant_type;
+    boost::variant< typename K::Point_3 > variant_type;
+    typedef typename boost::optional< variant_type > result_type;
+};
+
+// Iso_cuboid_3 Plane_3, variant of 4
+template<typename K>
+struct Intersection_traits<K, typename K::Iso_cuboid_3, typename K::Plane_3>  {
+  typedef typename
+  boost::variant< typename K::Point_3, typename K::Segment_3,
+                  typename K::Triangle_3, std::vector<typename K::Point_3> > variant_type;
+
+  typedef typename boost::optional< variant_type > result_type;
+};
+
+template<typename K>
+struct Intersection_traits<K, typename K::Plane_3, typename K::Iso_cuboid_3>  {
+  typedef typename
+  boost::variant< typename K::Point_3, typename K::Segment_3,
+                  typename K::Triangle_3, std::vector<typename K::Point_3> > variant_type;
+
   typedef typename boost::optional< variant_type > result_type;
 };
 
@@ -160,7 +201,7 @@ template<typename K>
 struct Intersection_traits<K, typename K::Iso_cuboid_3, typename K::Triangle_3>  {
   typedef typename
   boost::variant< typename K::Point_3, typename K::Segment_3,
-  typename K::Triangle_3, std::vector<typename K::Point_3> > variant_type;
+                  typename K::Triangle_3, std::vector<typename K::Point_3> > variant_type;
 
   typedef typename boost::optional< variant_type > result_type;
 };
@@ -169,7 +210,45 @@ template<typename K>
 struct Intersection_traits<K, typename K::Triangle_3, typename K::Iso_cuboid_3>  {
   typedef typename
   boost::variant< typename K::Point_3, typename K::Segment_3,
-  typename K::Triangle_3, std::vector<typename K::Point_3> > variant_type;
+                  typename K::Triangle_3, std::vector<typename K::Point_3> > variant_type;
+
+  typedef typename boost::optional< variant_type > result_type;
+};
+
+// Bbox_3 Plane_3, variant of 4
+template<typename K>
+struct Intersection_traits<K, typename CGAL::Bbox_3, typename K::Plane_3>  {
+  typedef typename
+  boost::variant< typename K::Point_3, typename K::Segment_3,
+                  typename K::Triangle_3, std::vector<typename K::Point_3> > variant_type;
+
+  typedef typename boost::optional< variant_type > result_type;
+};
+
+template<typename K>
+struct Intersection_traits<K, typename K::Plane_3, typename CGAL::Bbox_3>  {
+  typedef typename
+  boost::variant< typename K::Point_3, typename K::Segment_3,
+                  typename K::Triangle_3, std::vector<typename K::Point_3> > variant_type;
+
+  typedef typename boost::optional< variant_type > result_type;
+};
+
+// Bbox_3 Triangle_3, variant of 4
+template<typename K>
+struct Intersection_traits<K, typename CGAL::Bbox_3, typename K::Triangle_3>  {
+  typedef typename
+  boost::variant< typename K::Point_3, typename K::Segment_3,
+                  typename K::Triangle_3, std::vector<typename K::Point_3> > variant_type;
+
+  typedef typename boost::optional< variant_type > result_type;
+};
+
+template<typename K>
+struct Intersection_traits<K, typename K::Triangle_3, typename CGAL::Bbox_3>  {
+  typedef typename
+  boost::variant< typename K::Point_3, typename K::Segment_3,
+                  typename K::Triangle_3, std::vector<typename K::Point_3> > variant_type;
 
   typedef typename boost::optional< variant_type > result_type;
 };
@@ -253,6 +332,7 @@ struct Intersection_traits<K, typename K
   boost::variant< typename K::Point_3 > variant_type;
   typedef typename boost::optional< variant_type > result_type;
 };
+
 // Triangle_3 Point_3, variant of one
 template<typename K>
 struct Intersection_traits<K, typename K::Triangle_3, typename K::Point_3>  {
@@ -268,6 +348,7 @@ struct Intersection_traits<K, typename K
   boost::variant< typename K::Point_3 > variant_type;
   typedef typename boost::optional< variant_type > result_type;
 };
+
 // Tetrahedron_3 Point_3, variant of one
 template<typename K>
 struct Intersection_traits<K, typename K::Tetrahedron_3, typename K::Point_3>  {
@@ -283,6 +364,7 @@ struct Intersection_traits<K, typename K
   boost::variant< typename K::Point_3 > variant_type;
   typedef typename boost::optional< variant_type > result_type;
 };
+
 // Sphere_3 Point_3, variant of one
 template<typename K>
 struct Intersection_traits<K, typename K::Sphere_3, typename K::Point_3>  {
@@ -290,6 +372,48 @@ struct Intersection_traits<K, typename K
   boost::variant< typename K::Point_3 > variant_type;
   typedef typename boost::optional< variant_type > result_type;
 };
+
+//Tetrahedron_3 Plane_3, variant of 4
+template<class K>
+struct Intersection_traits<K, typename K::Tetrahedron_3, typename K::Plane_3>
+{
+  typedef typename
+  boost::variant< typename K::Point_3 , typename K::Segment_3,
+                  typename K::Triangle_3, std::vector<typename K::Point_3> > variant_type;
+  typedef typename boost::optional< variant_type > result_type;
+};
+
+//Plane_3 Tetrahedron_3, variant of 4
+template<class K>
+struct Intersection_traits<K, typename K::Plane_3,typename K::Tetrahedron_3>
+{
+  typedef typename
+  boost::variant< typename K::Point_3 , typename K::Segment_3,
+                  typename K::Triangle_3, std::vector<typename K::Point_3> > variant_type;
+  typedef typename boost::optional< variant_type > result_type;
+};
+
+//Triangle_3 Tetrahedron_3, variant of 4
+template<class K>
+struct Intersection_traits<K, typename K::Triangle_3,typename K::Tetrahedron_3>
+{
+  typedef typename
+  boost::variant< typename K::Point_3 , typename K::Segment_3,
+                  typename K::Triangle_3, std::vector<typename K::Point_3> > variant_type;
+  typedef typename boost::optional< variant_type > result_type;
+};
+
+// Tetrahedron_3 Triangle_3, variant of 4
+template<class K>
+struct Intersection_traits<K, typename K::Tetrahedron_3,typename K::Triangle_3>
+{
+  typedef typename
+  boost::variant< typename K::Point_3 , typename K::Segment_3,
+                  typename K::Triangle_3, std::vector<typename K::Point_3> > variant_type;
+  typedef typename boost::optional< variant_type > result_type;
+};
+
+
 } // namespace
 
 #endif /* CGAL_INTERSECTION_TRAITS_3_H */
diff -pruN 5.0.3-1/include/CGAL/Intersection_traits.h 5.1-1/include/CGAL/Intersection_traits.h
--- 5.0.3-1/include/CGAL/Intersection_traits.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/Intersection_traits.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Intersections_2/include/CGAL/Intersection_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Intersections_2/include/CGAL/Intersection_traits.h $
 // $Id: Intersection_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Interval_arithmetic.h 5.1-1/include/CGAL/Interval_arithmetic.h
--- 5.0.3-1/include/CGAL/Interval_arithmetic.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Interval_arithmetic.h	2020-09-08 12:15:16.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Interval_arithmetic.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Interval_arithmetic.h $
 // $Id: Interval_arithmetic.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Interval_arithmetic_impl.h 5.1-1/include/CGAL/Interval_arithmetic_impl.h
--- 5.0.3-1/include/CGAL/Interval_arithmetic_impl.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Interval_arithmetic_impl.h	2020-09-08 12:15:16.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Interval_arithmetic_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Interval_arithmetic_impl.h $
 // $Id: Interval_arithmetic_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Interval_nt.h 5.1-1/include/CGAL/Interval_nt.h
--- 5.0.3-1/include/CGAL/Interval_nt.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Interval_nt.h	2020-09-08 12:15:16.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Interval_nt.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Interval_nt.h $
 // $Id: Interval_nt.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Interval_skip_list.h 5.1-1/include/CGAL/Interval_skip_list.h
--- 5.0.3-1/include/CGAL/Interval_skip_list.h	2020-08-01 19:02:25.000000000 +0000
+++ 5.1-1/include/CGAL/Interval_skip_list.h	2020-09-08 12:14:59.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Interval_skip_list/include/CGAL/Interval_skip_list.h $
-// $Id: Interval_skip_list.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Interval_skip_list/include/CGAL/Interval_skip_list.h $
+// $Id: Interval_skip_list.h d1a323c 2020-03-26T19:24:14+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -115,7 +115,7 @@ class Interval_for_container : public In
       {}
 
       void *   for_compact_container() const { return p; }
-      void * & for_compact_container()       { return p; }
+      void for_compact_container(void *ptr) { p = ptr; }
     };
 
 
@@ -457,7 +457,7 @@ class Interval_for_container : public In
   public:
 #ifdef CGAL_ISL_USE_CCC
     void *   for_compact_container() const { return p; }
-    void * & for_compact_container()       { return p; }
+    void for_compact_container(void *ptr) { p = ptr; }
 #endif
 
     bool operator==(const IntervalListElt& e)
diff -pruN 5.0.3-1/include/CGAL/Interval_skip_list_interval.h 5.1-1/include/CGAL/Interval_skip_list_interval.h
--- 5.0.3-1/include/CGAL/Interval_skip_list_interval.h	2020-08-01 19:02:25.000000000 +0000
+++ 5.1-1/include/CGAL/Interval_skip_list_interval.h	2020-09-08 12:14:59.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Interval_skip_list/include/CGAL/Interval_skip_list_interval.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Interval_skip_list/include/CGAL/Interval_skip_list_interval.h $
 // $Id: Interval_skip_list_interval.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Interval_traits.h 5.1-1/include/CGAL/Interval_traits.h
--- 5.0.3-1/include/CGAL/Interval_traits.h	2020-08-01 19:02:25.000000000 +0000
+++ 5.1-1/include/CGAL/Interval_traits.h	2020-09-08 12:14:59.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Interval_support/include/CGAL/Interval_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Interval_support/include/CGAL/Interval_traits.h $
 // $Id: Interval_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/int.h 5.1-1/include/CGAL/int.h
--- 5.0.3-1/include/CGAL/int.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/int.h	2020-09-08 12:15:16.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/int.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/int.h $
 // $Id: int.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Inverse_index.h 5.1-1/include/CGAL/Inverse_index.h
--- 5.0.3-1/include/CGAL/Inverse_index.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Inverse_index.h	2020-09-08 12:15:32.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Inverse_index.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Inverse_index.h $
 // $Id: Inverse_index.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Alpha_shape_3_VRML_2_ostream.h 5.1-1/include/CGAL/IO/Alpha_shape_3_VRML_2_ostream.h
--- 5.0.3-1/include/CGAL/IO/Alpha_shape_3_VRML_2_ostream.h	2020-08-01 19:02:25.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Alpha_shape_3_VRML_2_ostream.h	2020-09-08 12:14:59.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Inventor/include/CGAL/IO/Alpha_shape_3_VRML_2_ostream.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Inventor/include/CGAL/IO/Alpha_shape_3_VRML_2_ostream.h $
 // $Id: Alpha_shape_3_VRML_2_ostream.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/alpha_shape_geomview_ostream_3.h 5.1-1/include/CGAL/IO/alpha_shape_geomview_ostream_3.h
--- 5.0.3-1/include/CGAL/IO/alpha_shape_geomview_ostream_3.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/IO/alpha_shape_geomview_ostream_3.h	2020-09-08 12:14:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Alpha_shapes_3/include/CGAL/IO/alpha_shape_geomview_ostream_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Alpha_shapes_3/include/CGAL/IO/alpha_shape_geomview_ostream_3.h $
 // $Id: alpha_shape_geomview_ostream_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Arrangement_2_reader.h 5.1-1/include/CGAL/IO/Arrangement_2_reader.h
--- 5.0.3-1/include/CGAL/IO/Arrangement_2_reader.h	2020-08-01 19:02:13.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Arrangement_2_reader.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/IO/Arrangement_2_reader.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/IO/Arrangement_2_reader.h $
 // $Id: Arrangement_2_reader.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Arrangement_2_writer.h 5.1-1/include/CGAL/IO/Arrangement_2_writer.h
--- 5.0.3-1/include/CGAL/IO/Arrangement_2_writer.h	2020-08-01 19:02:13.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Arrangement_2_writer.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/IO/Arrangement_2_writer.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/IO/Arrangement_2_writer.h $
 // $Id: Arrangement_2_writer.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Arr_iostream.h 5.1-1/include/CGAL/IO/Arr_iostream.h
--- 5.0.3-1/include/CGAL/IO/Arr_iostream.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Arr_iostream.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/IO/Arr_iostream.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/IO/Arr_iostream.h $
 // $Id: Arr_iostream.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Arr_text_formatter.h 5.1-1/include/CGAL/IO/Arr_text_formatter.h
--- 5.0.3-1/include/CGAL/IO/Arr_text_formatter.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Arr_text_formatter.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/IO/Arr_text_formatter.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/IO/Arr_text_formatter.h $
 // $Id: Arr_text_formatter.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Arr_with_history_2_reader.h 5.1-1/include/CGAL/IO/Arr_with_history_2_reader.h
--- 5.0.3-1/include/CGAL/IO/Arr_with_history_2_reader.h	2020-08-01 19:02:12.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Arr_with_history_2_reader.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/IO/Arr_with_history_2_reader.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/IO/Arr_with_history_2_reader.h $
 // $Id: Arr_with_history_2_reader.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Arr_with_history_2_writer.h 5.1-1/include/CGAL/IO/Arr_with_history_2_writer.h
--- 5.0.3-1/include/CGAL/IO/Arr_with_history_2_writer.h	2020-08-01 19:02:13.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Arr_with_history_2_writer.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/IO/Arr_with_history_2_writer.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/IO/Arr_with_history_2_writer.h $
 // $Id: Arr_with_history_2_writer.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Arr_with_history_iostream.h 5.1-1/include/CGAL/IO/Arr_with_history_iostream.h
--- 5.0.3-1/include/CGAL/IO/Arr_with_history_iostream.h	2020-08-01 19:02:13.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Arr_with_history_iostream.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/IO/Arr_with_history_iostream.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/IO/Arr_with_history_iostream.h $
 // $Id: Arr_with_history_iostream.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Arr_with_history_text_formatter.h 5.1-1/include/CGAL/IO/Arr_with_history_text_formatter.h
--- 5.0.3-1/include/CGAL/IO/Arr_with_history_text_formatter.h	2020-08-01 19:02:13.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Arr_with_history_text_formatter.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/IO/Arr_with_history_text_formatter.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/IO/Arr_with_history_text_formatter.h $
 // $Id: Arr_with_history_text_formatter.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/binary_file_io.h 5.1-1/include/CGAL/IO/binary_file_io.h
--- 5.0.3-1/include/CGAL/IO/binary_file_io.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/IO/binary_file_io.h	2020-09-08 12:15:42.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/binary_file_io.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/binary_file_io.h $
 // $Id: binary_file_io.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Color.h 5.1-1/include/CGAL/IO/Color.h
--- 5.0.3-1/include/CGAL/IO/Color.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Color.h	2020-09-08 12:15:41.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/Color.h $
-// $Id: Color.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/Color.h $
+// $Id: Color.h 401d266 2020-08-23T05:55:48+02:00 oboes
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -221,8 +221,6 @@ public:
   /*!
     replaces the rgb values of the colors by the conversion to rgb of
     the hsv values given as parameters.
-
-    Double values given as parameters should take range between 0 and 1.
   */
   void set_hsv (double hue,
                 double saturation,
@@ -232,8 +230,8 @@ public:
     saturation /= 100.;
     value /= 100.;
     double C = value*saturation;
-    int hh = (int)(hue/60.);
-    double X = C * (1-std::abs (hh % 2 - 1));
+    double hh = (hue/60.);
+    double X = C * (1-std::abs(std::fmod(hh, 2) - 1));
     double r = 0, g = 0, b = 0;
 
     if( hh>=0 && hh<1 )
diff -pruN 5.0.3-1/include/CGAL/IO/Complex_2_in_triangulation_3_file_writer.h 5.1-1/include/CGAL/IO/Complex_2_in_triangulation_3_file_writer.h
--- 5.0.3-1/include/CGAL/IO/Complex_2_in_triangulation_3_file_writer.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Complex_2_in_triangulation_3_file_writer.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/IO/Complex_2_in_triangulation_3_file_writer.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/IO/Complex_2_in_triangulation_3_file_writer.h $
 // $Id: Complex_2_in_triangulation_3_file_writer.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Complex_2_in_triangulation_3_polyhedron_builder.h 5.1-1/include/CGAL/IO/Complex_2_in_triangulation_3_polyhedron_builder.h
--- 5.0.3-1/include/CGAL/IO/Complex_2_in_triangulation_3_polyhedron_builder.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Complex_2_in_triangulation_3_polyhedron_builder.h	2020-09-08 12:15:47.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/IO/Complex_2_in_triangulation_3_polyhedron_builder.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/IO/Complex_2_in_triangulation_3_polyhedron_builder.h $
 // $Id: Complex_2_in_triangulation_3_polyhedron_builder.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Complex_2_in_triangulation_3_to_medit.h 5.1-1/include/CGAL/IO/Complex_2_in_triangulation_3_to_medit.h
--- 5.0.3-1/include/CGAL/IO/Complex_2_in_triangulation_3_to_medit.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Complex_2_in_triangulation_3_to_medit.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/IO/Complex_2_in_triangulation_3_to_medit.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/IO/Complex_2_in_triangulation_3_to_medit.h $
 // $Id: Complex_2_in_triangulation_3_to_medit.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Complex_2_in_triangulation_3_to_vtk.h 5.1-1/include/CGAL/IO/Complex_2_in_triangulation_3_to_vtk.h
--- 5.0.3-1/include/CGAL/IO/Complex_2_in_triangulation_3_to_vtk.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Complex_2_in_triangulation_3_to_vtk.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/IO/Complex_2_in_triangulation_3_to_vtk.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/IO/Complex_2_in_triangulation_3_to_vtk.h $
 // $Id: Complex_2_in_triangulation_3_to_vtk.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Complex_3_in_triangulation_3_to_vtk.h 5.1-1/include/CGAL/IO/Complex_3_in_triangulation_3_to_vtk.h
--- 5.0.3-1/include/CGAL/IO/Complex_3_in_triangulation_3_to_vtk.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Complex_3_in_triangulation_3_to_vtk.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/IO/Complex_3_in_triangulation_3_to_vtk.h $
-// $Id: Complex_3_in_triangulation_3_to_vtk.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/IO/Complex_3_in_triangulation_3_to_vtk.h $
+// $Id: Complex_3_in_triangulation_3_to_vtk.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -15,7 +15,7 @@
 
 #include <CGAL/license/Mesh_3.h>
 
-#include <CGAL/Hash_handles_with_or_without_timestamps.h>
+#include <CGAL/Time_stamper.h>
 
 #include <vtkPoints.h>
 #include <vtkUnstructuredGrid.h>
diff -pruN 5.0.3-1/include/CGAL/IO/Dxf_bsop_reader.h 5.1-1/include/CGAL/IO/Dxf_bsop_reader.h
--- 5.0.3-1/include/CGAL/IO/Dxf_bsop_reader.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Dxf_bsop_reader.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/IO/Dxf_bsop_reader.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/IO/Dxf_bsop_reader.h $
 // $Id: Dxf_bsop_reader.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Dxf_reader_doubles.h 5.1-1/include/CGAL/IO/Dxf_reader_doubles.h
--- 5.0.3-1/include/CGAL/IO/Dxf_reader_doubles.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Dxf_reader_doubles.h	2020-09-08 12:14:46.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_2/include/CGAL/IO/Dxf_reader_doubles.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_2/include/CGAL/IO/Dxf_reader_doubles.h $
 // $Id: Dxf_reader_doubles.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Dxf_reader.h 5.1-1/include/CGAL/IO/Dxf_reader.h
--- 5.0.3-1/include/CGAL/IO/Dxf_reader.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Dxf_reader.h	2020-09-08 12:14:46.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_2/include/CGAL/IO/Dxf_reader.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_2/include/CGAL/IO/Dxf_reader.h $
 // $Id: Dxf_reader.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Dxf_stream.h 5.1-1/include/CGAL/IO/Dxf_stream.h
--- 5.0.3-1/include/CGAL/IO/Dxf_stream.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Dxf_stream.h	2020-09-08 12:15:41.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Straight_skeleton_2/include/CGAL/IO/Dxf_stream.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Straight_skeleton_2/include/CGAL/IO/Dxf_stream.h $
 // $Id: Dxf_stream.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Dxf_variant_reader.h 5.1-1/include/CGAL/IO/Dxf_variant_reader.h
--- 5.0.3-1/include/CGAL/IO/Dxf_variant_reader.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Dxf_variant_reader.h	2020-09-08 12:14:46.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_2/include/CGAL/IO/Dxf_variant_reader.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_2/include/CGAL/IO/Dxf_variant_reader.h $
 // $Id: Dxf_variant_reader.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Dxf_writer.h 5.1-1/include/CGAL/IO/Dxf_writer.h
--- 5.0.3-1/include/CGAL/IO/Dxf_writer.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Dxf_writer.h	2020-09-08 12:15:41.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Straight_skeleton_2/include/CGAL/IO/Dxf_writer.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Straight_skeleton_2/include/CGAL/IO/Dxf_writer.h $
 // $Id: Dxf_writer.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/facets_in_complex_2_to_triangle_mesh.h 5.1-1/include/CGAL/IO/facets_in_complex_2_to_triangle_mesh.h
--- 5.0.3-1/include/CGAL/IO/facets_in_complex_2_to_triangle_mesh.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/IO/facets_in_complex_2_to_triangle_mesh.h	2020-09-08 12:15:47.000000000 +0000
@@ -6,7 +6,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/IO/facets_in_complex_2_to_triangle_mesh.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/IO/facets_in_complex_2_to_triangle_mesh.h $
 // $Id: facets_in_complex_2_to_triangle_mesh.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/facets_in_complex_3_to_triangle_mesh.h 5.1-1/include/CGAL/IO/facets_in_complex_3_to_triangle_mesh.h
--- 5.0.3-1/include/CGAL/IO/facets_in_complex_3_to_triangle_mesh.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/IO/facets_in_complex_3_to_triangle_mesh.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/IO/facets_in_complex_3_to_triangle_mesh.h $
-// $Id: facets_in_complex_3_to_triangle_mesh.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/IO/facets_in_complex_3_to_triangle_mesh.h $
+// $Id: facets_in_complex_3_to_triangle_mesh.h 4dda7b6 2020-05-27T15:53:05+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -14,13 +14,13 @@
 #ifndef CGAL_FACETS_IN_COMPLEX_3_TO_TRIANGLE_MESH_H
 #define CGAL_FACETS_IN_COMPLEX_3_TO_TRIANGLE_MESH_H
 
-#include <CGAL/license/Mesh_3.h>
+#include <CGAL/license/Triangulation_3.h>
 
 #include <CGAL/array.h>
 #include <CGAL/boost/graph/Euler_operations.h>
-#include <CGAL/Hash_handles_with_or_without_timestamps.h>
 #include <CGAL/Polygon_mesh_processing/orient_polygon_soup.h>
 #include <CGAL/Polygon_mesh_processing/polygon_soup_to_polygon_mesh.h>
+#include <CGAL/Time_stamper.h>
 
 #include <boost/unordered_map.hpp>
 #include <boost/tuple/tuple.hpp>
diff -pruN 5.0.3-1/include/CGAL/IO/Fig_stream_Conic_arc_2.h 5.1-1/include/CGAL/IO/Fig_stream_Conic_arc_2.h
--- 5.0.3-1/include/CGAL/IO/Fig_stream_Conic_arc_2.h	2020-08-01 19:02:13.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Fig_stream_Conic_arc_2.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/IO/Fig_stream_Conic_arc_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/IO/Fig_stream_Conic_arc_2.h $
 // $Id: Fig_stream_Conic_arc_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Fig_stream.h 5.1-1/include/CGAL/IO/Fig_stream.h
--- 5.0.3-1/include/CGAL/IO/Fig_stream.h	2020-08-01 19:02:13.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Fig_stream.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/IO/Fig_stream.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/IO/Fig_stream.h $
 // $Id: Fig_stream.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/File_avizo.h 5.1-1/include/CGAL/IO/File_avizo.h
--- 5.0.3-1/include/CGAL/IO/File_avizo.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/IO/File_avizo.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/IO/File_avizo.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/IO/File_avizo.h $
 // $Id: File_avizo.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/File_binary_mesh_3.h 5.1-1/include/CGAL/IO/File_binary_mesh_3.h
--- 5.0.3-1/include/CGAL/IO/File_binary_mesh_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/IO/File_binary_mesh_3.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/IO/File_binary_mesh_3.h $
-// $Id: File_binary_mesh_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/IO/File_binary_mesh_3.h $
+// $Id: File_binary_mesh_3.h 4dda7b6 2020-05-27T15:53:05+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -13,7 +13,7 @@
 #ifndef CGAL_IO_FILE_BINARY_MESH_3_H
 #define CGAL_IO_FILE_BINARY_MESH_3_H
 
-#include <CGAL/license/Mesh_3.h>
+#include <CGAL/license/Triangulation_3.h>
 
 
 #include <iostream>
diff -pruN 5.0.3-1/include/CGAL/IO/File_header_extended_OFF.h 5.1-1/include/CGAL/IO/File_header_extended_OFF.h
--- 5.0.3-1/include/CGAL/IO/File_header_extended_OFF.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/IO/File_header_extended_OFF.h	2020-09-08 12:15:41.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/File_header_extended_OFF.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/File_header_extended_OFF.h $
 // $Id: File_header_extended_OFF.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/File_header_extended_OFF_impl.h 5.1-1/include/CGAL/IO/File_header_extended_OFF_impl.h
--- 5.0.3-1/include/CGAL/IO/File_header_extended_OFF_impl.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/IO/File_header_extended_OFF_impl.h	2020-09-08 12:15:41.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/File_header_extended_OFF_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/File_header_extended_OFF_impl.h $
 // $Id: File_header_extended_OFF_impl.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/File_header_OFF.h 5.1-1/include/CGAL/IO/File_header_OFF.h
--- 5.0.3-1/include/CGAL/IO/File_header_OFF.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/IO/File_header_OFF.h	2020-09-08 12:15:41.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/File_header_OFF.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/File_header_OFF.h $
 // $Id: File_header_OFF.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/File_header_OFF_impl.h 5.1-1/include/CGAL/IO/File_header_OFF_impl.h
--- 5.0.3-1/include/CGAL/IO/File_header_OFF_impl.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/IO/File_header_OFF_impl.h	2020-09-08 12:15:41.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/File_header_OFF_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/File_header_OFF_impl.h $
 // $Id: File_header_OFF_impl.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/File_maya.h 5.1-1/include/CGAL/IO/File_maya.h
--- 5.0.3-1/include/CGAL/IO/File_maya.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/IO/File_maya.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/IO/File_maya.h $
-// $Id: File_maya.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/IO/File_maya.h $
+// $Id: File_maya.h 4dda7b6 2020-05-27T15:53:05+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Clement Jamin
@@ -12,9 +12,9 @@
 #ifndef CGAL_IO_FILE_MAYA_H
 #define CGAL_IO_FILE_MAYA_H
 
-#include <CGAL/license/Mesh_3.h>
+#include <CGAL/license/Triangulation_3.h>
 
-#include <CGAL/Hash_handles_with_or_without_timestamps.h>
+#include <CGAL/Time_stamper.h>
 #include <CGAL/utility.h>
 
 #include <boost/unordered_map.hpp>
diff -pruN 5.0.3-1/include/CGAL/IO/File_medit.h 5.1-1/include/CGAL/IO/File_medit.h
--- 5.0.3-1/include/CGAL/IO/File_medit.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/IO/File_medit.h	2020-09-08 12:15:08.000000000 +0000
@@ -4,8 +4,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/IO/File_medit.h $
-// $Id: File_medit.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/IO/File_medit.h $
+// $Id: File_medit.h 93d62b9 2020-08-18T14:07:27+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -14,9 +14,10 @@
 #ifndef CGAL_IO_FILE_MEDIT_H
 #define CGAL_IO_FILE_MEDIT_H
 
-#include <CGAL/license/Mesh_3.h>
+#include <CGAL/license/Triangulation_3.h>
 
 #include <CGAL/Mesh_3/config.h>
+#include <CGAL/Mesh_3/Mesh_complex_3_in_triangulation_3_fwd.h>
 
 #include <CGAL/utility.h>
 #include <CGAL/basic.h>
@@ -754,7 +755,7 @@ output_to_medit(std::ostream& os,
 
   typedef typename Tr::Finite_vertices_iterator Finite_vertices_iterator;
   typedef typename Tr::Vertex_handle Vertex_handle;
-  typedef typename Tr::Weighted_point Weighted_point;
+  typedef typename Tr::Point Point; //can be weighted or not
 
   const Tr& tr = c3t3.triangulation();
 
@@ -783,7 +784,7 @@ output_to_medit(std::ostream& os,
        ++vit)
   {
     V[vit] = inum++;
-    Weighted_point p = tr.point(vit);
+    Point p = tr.point(vit);
     os << CGAL::to_double(p.x()) << ' '
        << CGAL::to_double(p.y()) << ' '
        << CGAL::to_double(p.z()) << ' '
@@ -893,6 +894,23 @@ output_to_medit(std::ostream& os,
   }
 }
 
+
+template<typename T3>
+void write_MEDIT(std::ostream& os, const T3& t3)
+{
+  CGAL::Mesh_complex_3_in_triangulation_3<T3, int, int> c3t3;
+  c3t3.triangulation() = t3;
+  c3t3.rescan_after_load_of_triangulation();
+  output_to_medit(os, c3t3);
+}
+
+template<typename T3>
+bool read_MEDIT(std::istream& in, T3& t3)
+{
+  CGAL_assertion(!(!in));
+  return CGAL::build_triangulation_from_file<T3, true>(in, t3);
+}
+
 } // end namespace CGAL
 
 #endif // CGAL_IO_FILE_MEDIT_H
diff -pruN 5.0.3-1/include/CGAL/IO/File_poly.h 5.1-1/include/CGAL/IO/File_poly.h
--- 5.0.3-1/include/CGAL/IO/File_poly.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/IO/File_poly.h	2020-09-08 12:15:06.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_2/include/CGAL/IO/File_poly.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_2/include/CGAL/IO/File_poly.h $
 // $Id: File_poly.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/File_scanner_OFF.h 5.1-1/include/CGAL/IO/File_scanner_OFF.h
--- 5.0.3-1/include/CGAL/IO/File_scanner_OFF.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/IO/File_scanner_OFF.h	2020-09-08 12:15:42.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/File_scanner_OFF.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/File_scanner_OFF.h $
 // $Id: File_scanner_OFF.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/File_scanner_OFF_impl.h 5.1-1/include/CGAL/IO/File_scanner_OFF_impl.h
--- 5.0.3-1/include/CGAL/IO/File_scanner_OFF_impl.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/IO/File_scanner_OFF_impl.h	2020-09-08 12:15:42.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/File_scanner_OFF_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/File_scanner_OFF_impl.h $
 // $Id: File_scanner_OFF_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/File_tetgen.h 5.1-1/include/CGAL/IO/File_tetgen.h
--- 5.0.3-1/include/CGAL/IO/File_tetgen.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/IO/File_tetgen.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/IO/File_tetgen.h $
-// $Id: File_tetgen.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/IO/File_tetgen.h $
+// $Id: File_tetgen.h 53d4c9b 2019-10-28T11:29:08+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -15,7 +15,7 @@
 
 #include <CGAL/license/Mesh_3.h>
 
-#include <CGAL/Hash_handles_with_or_without_timestamps.h>
+#include <CGAL/Time_stamper.h>
 #include <CGAL/IO/File_medit.h>
 
 #include <iostream>
diff -pruN 5.0.3-1/include/CGAL/IO/File_writer_inventor.h 5.1-1/include/CGAL/IO/File_writer_inventor.h
--- 5.0.3-1/include/CGAL/IO/File_writer_inventor.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/IO/File_writer_inventor.h	2020-09-08 12:15:42.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/File_writer_inventor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/File_writer_inventor.h $
 // $Id: File_writer_inventor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/File_writer_inventor_impl.h 5.1-1/include/CGAL/IO/File_writer_inventor_impl.h
--- 5.0.3-1/include/CGAL/IO/File_writer_inventor_impl.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/IO/File_writer_inventor_impl.h	2020-09-08 12:15:42.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/File_writer_inventor_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/File_writer_inventor_impl.h $
 // $Id: File_writer_inventor_impl.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/File_writer_OFF.h 5.1-1/include/CGAL/IO/File_writer_OFF.h
--- 5.0.3-1/include/CGAL/IO/File_writer_OFF.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/IO/File_writer_OFF.h	2020-09-08 12:15:42.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/File_writer_OFF.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/File_writer_OFF.h $
 // $Id: File_writer_OFF.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/File_writer_OFF_impl.h 5.1-1/include/CGAL/IO/File_writer_OFF_impl.h
--- 5.0.3-1/include/CGAL/IO/File_writer_OFF_impl.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/IO/File_writer_OFF_impl.h	2020-09-08 12:15:42.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/File_writer_OFF_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/File_writer_OFF_impl.h $
 // $Id: File_writer_OFF_impl.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/File_writer_VRML_2.h 5.1-1/include/CGAL/IO/File_writer_VRML_2.h
--- 5.0.3-1/include/CGAL/IO/File_writer_VRML_2.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/IO/File_writer_VRML_2.h	2020-09-08 12:15:42.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/File_writer_VRML_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/File_writer_VRML_2.h $
 // $Id: File_writer_VRML_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/File_writer_VRML_2_impl.h 5.1-1/include/CGAL/IO/File_writer_VRML_2_impl.h
--- 5.0.3-1/include/CGAL/IO/File_writer_VRML_2_impl.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/IO/File_writer_VRML_2_impl.h	2020-09-08 12:15:42.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/File_writer_VRML_2_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/File_writer_VRML_2_impl.h $
 // $Id: File_writer_VRML_2_impl.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/File_writer_wavefront.h 5.1-1/include/CGAL/IO/File_writer_wavefront.h
--- 5.0.3-1/include/CGAL/IO/File_writer_wavefront.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/IO/File_writer_wavefront.h	2020-09-08 12:15:42.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/File_writer_wavefront.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/File_writer_wavefront.h $
 // $Id: File_writer_wavefront.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/File_writer_wavefront_impl.h 5.1-1/include/CGAL/IO/File_writer_wavefront_impl.h
--- 5.0.3-1/include/CGAL/IO/File_writer_wavefront_impl.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/IO/File_writer_wavefront_impl.h	2020-09-08 12:15:42.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/File_writer_wavefront_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/File_writer_wavefront_impl.h $
 // $Id: File_writer_wavefront_impl.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/generic_copy_OFF.h 5.1-1/include/CGAL/IO/generic_copy_OFF.h
--- 5.0.3-1/include/CGAL/IO/generic_copy_OFF.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/IO/generic_copy_OFF.h	2020-09-08 12:15:42.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/generic_copy_OFF.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/generic_copy_OFF.h $
 // $Id: generic_copy_OFF.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/generic_print_polyhedron.h 5.1-1/include/CGAL/IO/generic_print_polyhedron.h
--- 5.0.3-1/include/CGAL/IO/generic_print_polyhedron.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/IO/generic_print_polyhedron.h	2020-09-08 12:15:28.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron_IO/include/CGAL/IO/generic_print_polyhedron.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron_IO/include/CGAL/IO/generic_print_polyhedron.h $
 // $Id: generic_print_polyhedron.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Generic_writer.h 5.1-1/include/CGAL/IO/Generic_writer.h
--- 5.0.3-1/include/CGAL/IO/Generic_writer.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Generic_writer.h	2020-09-08 12:15:42.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/Generic_writer.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/Generic_writer.h $
 // $Id: Generic_writer.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Geomview_stream.h 5.1-1/include/CGAL/IO/Geomview_stream.h
--- 5.0.3-1/include/CGAL/IO/Geomview_stream.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Geomview_stream.h	2020-09-08 12:14:51.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Geomview/include/CGAL/IO/Geomview_stream.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Geomview/include/CGAL/IO/Geomview_stream.h $
 // $Id: Geomview_stream.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Geomview_stream_impl.h 5.1-1/include/CGAL/IO/Geomview_stream_impl.h
--- 5.0.3-1/include/CGAL/IO/Geomview_stream_impl.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Geomview_stream_impl.h	2020-09-08 12:14:51.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Geomview/include/CGAL/IO/Geomview_stream_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Geomview/include/CGAL/IO/Geomview_stream_impl.h $
 // $Id: Geomview_stream_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Gps_iostream.h 5.1-1/include/CGAL/IO/Gps_iostream.h
--- 5.0.3-1/include/CGAL/IO/Gps_iostream.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Gps_iostream.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/IO/Gps_iostream.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/IO/Gps_iostream.h $
 // $Id: Gps_iostream.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Inventor_ostream.h 5.1-1/include/CGAL/IO/Inventor_ostream.h
--- 5.0.3-1/include/CGAL/IO/Inventor_ostream.h	2020-08-01 19:02:25.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Inventor_ostream.h	2020-09-08 12:14:59.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Inventor/include/CGAL/IO/Inventor_ostream.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Inventor/include/CGAL/IO/Inventor_ostream.h $
 // $Id: Inventor_ostream.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/io.h 5.1-1/include/CGAL/IO/io.h
--- 5.0.3-1/include/CGAL/IO/io.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/IO/io.h	2020-09-08 12:15:42.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/io.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/io.h $
 // $Id: io.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/io_impl.h 5.1-1/include/CGAL/IO/io_impl.h
--- 5.0.3-1/include/CGAL/IO/io_impl.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/IO/io_impl.h	2020-09-08 12:15:42.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/io_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/io_impl.h $
 // $Id: io_impl.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/io_tags.h 5.1-1/include/CGAL/IO/io_tags.h
--- 5.0.3-1/include/CGAL/IO/io_tags.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/IO/io_tags.h	2020-09-08 12:15:42.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/io_tags.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/io_tags.h $
 // $Id: io_tags.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Istream_iterator.h 5.1-1/include/CGAL/IO/Istream_iterator.h
--- 5.0.3-1/include/CGAL/IO/Istream_iterator.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Istream_iterator.h	2020-09-08 12:15:42.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/Istream_iterator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/Istream_iterator.h $
 // $Id: Istream_iterator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Nef_polyhedron_2_PS_stream.h 5.1-1/include/CGAL/IO/Nef_polyhedron_2_PS_stream.h
--- 5.0.3-1/include/CGAL/IO/Nef_polyhedron_2_PS_stream.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Nef_polyhedron_2_PS_stream.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_2/include/CGAL/IO/Nef_polyhedron_2_PS_stream.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_2/include/CGAL/IO/Nef_polyhedron_2_PS_stream.h $
 // $Id: Nef_polyhedron_2_PS_stream.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Nef_polyhedron_iostream_3.h 5.1-1/include/CGAL/IO/Nef_polyhedron_iostream_3.h
--- 5.0.3-1/include/CGAL/IO/Nef_polyhedron_iostream_3.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Nef_polyhedron_iostream_3.h	2020-09-08 12:15:13.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/IO/Nef_polyhedron_iostream_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/IO/Nef_polyhedron_iostream_3.h $
 // $Id: Nef_polyhedron_iostream_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/OBJ_reader.h 5.1-1/include/CGAL/IO/OBJ_reader.h
--- 5.0.3-1/include/CGAL/IO/OBJ_reader.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/IO/OBJ_reader.h	2020-09-08 12:15:28.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron_IO/include/CGAL/IO/OBJ_reader.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron_IO/include/CGAL/IO/OBJ_reader.h $
 // $Id: OBJ_reader.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/OFF_reader.h 5.1-1/include/CGAL/IO/OFF_reader.h
--- 5.0.3-1/include/CGAL/IO/OFF_reader.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/IO/OFF_reader.h	2020-09-08 12:15:28.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron_IO/include/CGAL/IO/OFF_reader.h $
-// $Id: OFF_reader.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron_IO/include/CGAL/IO/OFF_reader.h $
+// $Id: OFF_reader.h 922b861 2020-04-14T15:40:38+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Laurent Rineau and Sebastien Loriot
@@ -14,12 +14,14 @@
 #include <CGAL/IO/File_scanner_OFF.h>
 #include <CGAL/IO/reader_helpers.h>
 
-#include <vector>
-#include <iostream>
 #include <CGAL/array.h>
 #include <CGAL/assertions.h>
+#include <CGAL/Container_helper.h>
 #include <CGAL/use.h>
 
+#include <vector>
+#include <iostream>
+
 namespace CGAL {
 
   template <class Point_3, class Polygon_3>
@@ -47,13 +49,13 @@ namespace CGAL {
       std::size_t no;
 
       scanner.scan_facet( no, i);
-      IO::internal::resize(polygons[i], no);
+      CGAL::internal::resize(polygons[i], no);
       for(std::size_t j = 0; j < no; ++j) {
         std::size_t id;
         scanner.scan_facet_vertex_index(id, i);
         if(id < scanner.size_of_vertices())
         {
-          polygons[i][j] = id;
+          polygons[i][j] = typename std::remove_cv<typename std::iterator_traits<typename Polygon_3::const_iterator>::value_type>::type(id);
         }
         else
           return false;
@@ -114,7 +116,7 @@ namespace CGAL {
       scanner.scan_facet( no, i);
       if(!in)
         return false;
-      IO::internal::resize(polygons[i], no);
+      CGAL::internal::resize(polygons[i], no);
       for(std::size_t j = 0; j < no; ++j) {
         std::size_t id;
         scanner.scan_facet_vertex_index(id, i);
diff -pruN 5.0.3-1/include/CGAL/IO/Ostream_iterator.h 5.1-1/include/CGAL/IO/Ostream_iterator.h
--- 5.0.3-1/include/CGAL/IO/Ostream_iterator.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Ostream_iterator.h	2020-09-08 12:15:42.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/Ostream_iterator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/Ostream_iterator.h $
 // $Id: Ostream_iterator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/output_surface_facets_to_polyhedron.h 5.1-1/include/CGAL/IO/output_surface_facets_to_polyhedron.h
--- 5.0.3-1/include/CGAL/IO/output_surface_facets_to_polyhedron.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/IO/output_surface_facets_to_polyhedron.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/IO/output_surface_facets_to_polyhedron.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/IO/output_surface_facets_to_polyhedron.h $
 // $Id: output_surface_facets_to_polyhedron.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/output_surface_facets_to_triangle_soup.h 5.1-1/include/CGAL/IO/output_surface_facets_to_triangle_soup.h
--- 5.0.3-1/include/CGAL/IO/output_surface_facets_to_triangle_soup.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/IO/output_surface_facets_to_triangle_soup.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Poisson_surface_reconstruction_3/include/CGAL/IO/output_surface_facets_to_triangle_soup.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Poisson_surface_reconstruction_3/include/CGAL/IO/output_surface_facets_to_triangle_soup.h $
 // $Id: output_surface_facets_to_triangle_soup.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/output_to_vtu.h 5.1-1/include/CGAL/IO/output_to_vtu.h
--- 5.0.3-1/include/CGAL/IO/output_to_vtu.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/IO/output_to_vtu.h	2020-09-08 12:15:08.000000000 +0000
@@ -5,7 +5,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/IO/output_to_vtu.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/IO/output_to_vtu.h $
 // $Id: output_to_vtu.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/PLY.h 5.1-1/include/CGAL/IO/PLY.h
--- 5.0.3-1/include/CGAL/IO/PLY.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/IO/PLY.h	2020-09-08 12:15:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/PLY.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/PLY.h $
 // $Id: PLY.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/PLY_reader.h 5.1-1/include/CGAL/IO/PLY_reader.h
--- 5.0.3-1/include/CGAL/IO/PLY_reader.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/IO/PLY_reader.h	2020-09-08 12:15:28.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron_IO/include/CGAL/IO/PLY_reader.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron_IO/include/CGAL/IO/PLY_reader.h $
 // $Id: PLY_reader.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/PLY_writer.h 5.1-1/include/CGAL/IO/PLY_writer.h
--- 5.0.3-1/include/CGAL/IO/PLY_writer.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/IO/PLY_writer.h	2020-09-08 12:15:28.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron_IO/include/CGAL/IO/PLY_writer.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron_IO/include/CGAL/IO/PLY_writer.h $
 // $Id: PLY_writer.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Polyhedron_builder_from_STL.h 5.1-1/include/CGAL/IO/Polyhedron_builder_from_STL.h
--- 5.0.3-1/include/CGAL/IO/Polyhedron_builder_from_STL.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Polyhedron_builder_from_STL.h	2020-09-08 12:15:28.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron_IO/include/CGAL/IO/Polyhedron_builder_from_STL.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron_IO/include/CGAL/IO/Polyhedron_builder_from_STL.h $
 // $Id: Polyhedron_builder_from_STL.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Polyhedron_geomview_ostream.h 5.1-1/include/CGAL/IO/Polyhedron_geomview_ostream.h
--- 5.0.3-1/include/CGAL/IO/Polyhedron_geomview_ostream.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Polyhedron_geomview_ostream.h	2020-09-08 12:15:28.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron_IO/include/CGAL/IO/Polyhedron_geomview_ostream.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron_IO/include/CGAL/IO/Polyhedron_geomview_ostream.h $
 // $Id: Polyhedron_geomview_ostream.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Polyhedron_inventor_ostream.h 5.1-1/include/CGAL/IO/Polyhedron_inventor_ostream.h
--- 5.0.3-1/include/CGAL/IO/Polyhedron_inventor_ostream.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Polyhedron_inventor_ostream.h	2020-09-08 12:15:28.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron_IO/include/CGAL/IO/Polyhedron_inventor_ostream.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron_IO/include/CGAL/IO/Polyhedron_inventor_ostream.h $
 // $Id: Polyhedron_inventor_ostream.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Polyhedron_iostream.h 5.1-1/include/CGAL/IO/Polyhedron_iostream.h
--- 5.0.3-1/include/CGAL/IO/Polyhedron_iostream.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Polyhedron_iostream.h	2020-09-08 12:15:28.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron_IO/include/CGAL/IO/Polyhedron_iostream.h $
-// $Id: Polyhedron_iostream.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron_IO/include/CGAL/IO/Polyhedron_iostream.h $
+// $Id: Polyhedron_iostream.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -38,9 +38,9 @@ write_off( std::ostream& out, const Poly
   // writes P to `out' in PRETTY, ASCII or BINARY format
     // as the stream indicates.
     File_header_OFF header( is_binary( out), ! is_pretty( out), false);
-    typename CGAL::Polygon_mesh_processing::GetVertexPointMap<Polyhedron_3<Traits,Items,HDS,Alloc>, NamedParameters>::const_type
+    typename CGAL::GetVertexPointMap<Polyhedron_3<Traits,Items,HDS,Alloc>, NamedParameters>::const_type
         vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
-                           get_const_property_map(CGAL::vertex_point, P));
+                               get_const_property_map(CGAL::vertex_point, P));
     CGAL::print_polyhedron_with_header_OFF( out, P, header, vpm);
     return out.good();
 }
@@ -64,12 +64,12 @@ read_off(std::istream& in,
          Polyhedron_3<Traits,Items,HDS,Alloc>& P,
          NamedParameters np) {
     // reads a polyhedron from `in' and appends it to P.
-    typedef typename CGAL::Polygon_mesh_processing::GetVertexPointMap<Polyhedron_3<Traits,Items,HDS,Alloc>, NamedParameters>::type Vpm;
+    typedef typename CGAL::GetVertexPointMap<Polyhedron_3<Traits,Items,HDS,Alloc>, NamedParameters>::type Vpm;
     using parameters::choose_parameter;
     using parameters::get_parameter;
 
     Vpm vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
-                           get_property_map(CGAL::vertex_point, P));
+                               get_property_map(CGAL::vertex_point, P));
     CGAL::scan_OFF( in, P);
     if(!parameters::is_default_parameter(get_parameter(np, internal_np::vertex_point)))
     {
diff -pruN 5.0.3-1/include/CGAL/IO/Polyhedron_scan_OFF.h 5.1-1/include/CGAL/IO/Polyhedron_scan_OFF.h
--- 5.0.3-1/include/CGAL/IO/Polyhedron_scan_OFF.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Polyhedron_scan_OFF.h	2020-09-08 12:15:28.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron_IO/include/CGAL/IO/Polyhedron_scan_OFF.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron_IO/include/CGAL/IO/Polyhedron_scan_OFF.h $
 // $Id: Polyhedron_scan_OFF.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Polyhedron_VRML_1_ostream.h 5.1-1/include/CGAL/IO/Polyhedron_VRML_1_ostream.h
--- 5.0.3-1/include/CGAL/IO/Polyhedron_VRML_1_ostream.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Polyhedron_VRML_1_ostream.h	2020-09-08 12:15:28.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron_IO/include/CGAL/IO/Polyhedron_VRML_1_ostream.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron_IO/include/CGAL/IO/Polyhedron_VRML_1_ostream.h $
 // $Id: Polyhedron_VRML_1_ostream.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Polyhedron_VRML_2_ostream.h 5.1-1/include/CGAL/IO/Polyhedron_VRML_2_ostream.h
--- 5.0.3-1/include/CGAL/IO/Polyhedron_VRML_2_ostream.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Polyhedron_VRML_2_ostream.h	2020-09-08 12:15:28.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron_IO/include/CGAL/IO/Polyhedron_VRML_2_ostream.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron_IO/include/CGAL/IO/Polyhedron_VRML_2_ostream.h $
 // $Id: Polyhedron_VRML_2_ostream.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/print_inventor.h 5.1-1/include/CGAL/IO/print_inventor.h
--- 5.0.3-1/include/CGAL/IO/print_inventor.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/IO/print_inventor.h	2020-09-08 12:15:28.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron_IO/include/CGAL/IO/print_inventor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron_IO/include/CGAL/IO/print_inventor.h $
 // $Id: print_inventor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/print_OFF.h 5.1-1/include/CGAL/IO/print_OFF.h
--- 5.0.3-1/include/CGAL/IO/print_OFF.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/IO/print_OFF.h	2020-09-08 12:15:28.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron_IO/include/CGAL/IO/print_OFF.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron_IO/include/CGAL/IO/print_OFF.h $
 // $Id: print_OFF.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/print_VRML_1.h 5.1-1/include/CGAL/IO/print_VRML_1.h
--- 5.0.3-1/include/CGAL/IO/print_VRML_1.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/IO/print_VRML_1.h	2020-09-08 12:15:28.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron_IO/include/CGAL/IO/print_VRML_1.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron_IO/include/CGAL/IO/print_VRML_1.h $
 // $Id: print_VRML_1.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/print_VRML_2.h 5.1-1/include/CGAL/IO/print_VRML_2.h
--- 5.0.3-1/include/CGAL/IO/print_VRML_2.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/IO/print_VRML_2.h	2020-09-08 12:15:28.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron_IO/include/CGAL/IO/print_VRML_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron_IO/include/CGAL/IO/print_VRML_2.h $
 // $Id: print_VRML_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/print_wavefront.h 5.1-1/include/CGAL/IO/print_wavefront.h
--- 5.0.3-1/include/CGAL/IO/print_wavefront.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/IO/print_wavefront.h	2020-09-08 12:15:28.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron_IO/include/CGAL/IO/print_wavefront.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron_IO/include/CGAL/IO/print_wavefront.h $
 // $Id: print_wavefront.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/read_3mf.h 5.1-1/include/CGAL/IO/read_3mf.h
--- 5.0.3-1/include/CGAL/IO/read_3mf.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/IO/read_3mf.h	2020-09-08 12:15:42.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/read_3mf.h $
-// $Id: read_3mf.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/read_3mf.h $
+// $Id: read_3mf.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s) : Maxime Gimeno
@@ -65,12 +65,6 @@ bool extract_soups (NMR::PLib3MFModelMes
     pBuffer.resize(nNeededChars + 1);
     hResult = NMR::lib3mf_object_getnameutf8(pMeshObject, &pBuffer[0], nNeededChars + 1, NULL);
     pBuffer[nNeededChars] = 0;
-    std::string temp(&pBuffer[0]);
-    if(temp.find("_cgal_pc") != std::string::npos
-       || temp.find("_cgal_pl")!= std::string::npos) //ignore point clouds and polylines
-    {
-      return false;
-    }
     name = std::string(&pBuffer[0]);
   }
   else
@@ -120,151 +114,6 @@ bool extract_soups (NMR::PLib3MFModelMes
   return true;
 }
 
-template<typename PointRange,
-         typename PolygonRange,
-         typename ColorRange>
-bool extract_polylines (NMR::PLib3MFModelMeshObject *pMeshObject,
-                        const NMR::MODELTRANSFORM& ,
-                        PointRange& points,
-                        PolygonRange&,
-                        ColorRange& colors,
-                        std::string& name) {
-  typedef typename PointRange::value_type Point_3;
-
-  HRESULT hResult;
-  DWORD nNeededChars;
-  std::vector<char> pBuffer;
-  // Retrieve Mesh Name Length
-  hResult = NMR::lib3mf_object_getnameutf8(pMeshObject, NULL, 0, &nNeededChars);
-  if (hResult != LIB3MF_OK)
-  {
-    points.resize(0);
-    std::cerr<<"Error during name extraction.";
-    return false;
-  }
-
-  // Retrieve Mesh Name
-  if (nNeededChars > 0) {
-    pBuffer.resize(nNeededChars + 1);
-    hResult = NMR::lib3mf_object_getnameutf8(pMeshObject, &pBuffer[0], nNeededChars + 1, NULL);
-    pBuffer[nNeededChars] = 0;
-    std::string temp(&pBuffer[0]);
-    if(temp.find("_cgal_pl")== std::string::npos) //ignore not polylines
-    {
-      points.resize(0);
-      return false;
-    }
-    name = std::string(&pBuffer[0]);
-  }
-  else
-  {
-    points.resize(0);
-    return false;
-  }
-
-  NMR::PLib3MFPropertyHandler * pPropertyHandler;
-  hResult = NMR::lib3mf_meshobject_createpropertyhandler(pMeshObject, &pPropertyHandler);
-  if (hResult != LIB3MF_OK) {
-    DWORD nErrorMessage;
-    LPCSTR pszErrorMessage;
-    std::cerr << "could not create property handler: " << std::hex << hResult << std::endl;
-    NMR::lib3mf_getlasterror(pMeshObject, &nErrorMessage, &pszErrorMessage);
-    std::cerr << "error #" << std::hex << nErrorMessage << ": " << pszErrorMessage << std::endl;
-    NMR::lib3mf_release(pMeshObject);
-    return false;
-  }
-
-  points.resize(points.size()-3);//ignore dummy_vertices
-  for(DWORD vid = 0; vid < points.size(); ++vid)
-  {
-    NMR::MODELMESHVERTEX pVertex;
-    NMR::lib3mf_meshobject_getvertex(pMeshObject, vid+3, &pVertex);
-    points[vid] =
-        Point_3(pVertex.m_fPosition[0],
-        pVertex.m_fPosition[1],
-        pVertex.m_fPosition[2]);
-  }
-
-  NMR::MODELMESH_TRIANGLECOLOR_SRGB pColor;
-  NMR::lib3mf_propertyhandler_getcolor(pPropertyHandler, 0, &pColor);//get color of the dummy triangle
-  NMR::MODELMESHCOLOR_SRGB mColor = pColor.m_Colors[0];
-  colors[0]=CGAL::Color(mColor.m_Red, mColor.m_Green,
-                        mColor.m_Blue, mColor.m_Alpha);
-  return true;
-}
-
-template<typename PointRange,
-         typename PolygonRange,
-         typename ColorRange>
-bool extract_point_clouds (NMR::PLib3MFModelMeshObject *pMeshObject,
-                           const NMR::MODELTRANSFORM&,
-                           PointRange& points,
-                           PolygonRange&,
-                           ColorRange& colors,
-                           std::string& name) {
-  typedef typename PointRange::value_type Point_3;
-
-  HRESULT hResult;
-  DWORD nNeededChars;
-  std::vector<char> pBuffer;
-  // Retrieve Mesh Name Length
-  hResult = NMR::lib3mf_object_getnameutf8(pMeshObject, NULL, 0, &nNeededChars);
-  if (hResult != LIB3MF_OK)
-  {
-    std::cerr<<"Error during name extraction.";
-    points.resize(0);
-    return false;
-  }
-
-  // Retrieve Mesh Name
-  if (nNeededChars > 0) {
-    pBuffer.resize(nNeededChars + 1);
-    hResult = NMR::lib3mf_object_getnameutf8(pMeshObject, &pBuffer[0], nNeededChars + 1, NULL);
-    pBuffer[nNeededChars] = 0;
-    std::string temp(&pBuffer[0]);
-    if(temp.find("_cgal_pc")== std::string::npos) //ignore not point_cloud
-    {
-      points.resize(0);
-      return false;
-    }
-    name = std::string(&pBuffer[0]);
-  }
-  else{
-    points.resize(0);
-    return false;
-  }
-
-  NMR::PLib3MFPropertyHandler * pPropertyHandler;
-  hResult = NMR::lib3mf_meshobject_createpropertyhandler(pMeshObject, &pPropertyHandler);
-  if (hResult != LIB3MF_OK) {
-    DWORD nErrorMessage;
-    LPCSTR pszErrorMessage;
-    std::cerr << "could not create property handler: " << std::hex << hResult << std::endl;
-    NMR::lib3mf_getlasterror(pMeshObject, &nErrorMessage, &pszErrorMessage);
-    std::cerr << "error #" << std::hex << nErrorMessage << ": " << pszErrorMessage << std::endl;
-    NMR::lib3mf_release(pMeshObject);
-    return -1;
-  }
-
-  points.resize(points.size()-3);//ignore dummy_vertices
-  for(DWORD vid = 0; vid < points.size(); ++vid)
-  {
-    NMR::MODELMESHVERTEX pVertex;
-    NMR::lib3mf_meshobject_getvertex(pMeshObject, vid+3, &pVertex);
-    points[vid] =
-        Point_3(pVertex.m_fPosition[0],
-        pVertex.m_fPosition[1],
-        pVertex.m_fPosition[2]);
-  }
-
-  NMR::MODELMESH_TRIANGLECOLOR_SRGB pColor;
-  NMR::lib3mf_propertyhandler_getcolor(pPropertyHandler, 0, &pColor);//get color of the dummy triangle
-  NMR::MODELMESHCOLOR_SRGB mColor = pColor.m_Colors[0];
-  colors[0]=CGAL::Color(mColor.m_Red, mColor.m_Green,
-                        mColor.m_Blue, mColor.m_Alpha);
-  return true;
-}
-
 
 template<typename PointRanges, typename PolygonRanges, typename ColorRanges,
          typename PointRange, typename PolygonRange, typename ColorRange>
@@ -607,43 +456,6 @@ int read_triangle_soups_from_3mf(const s
        all_colors, names, extract_soups<PointRange, PolygonRange, ColorRange>);
 }
 
-
-template<typename PointRanges, typename ColorRanges>
-int read_polylines_from_3mf(const std::string& file_name,
-                            PointRanges& all_points,
-                            ColorRanges& all_colors,
-                            std::vector<std::string>& names
-                            )
-{
-  typedef typename PointRanges::value_type PointRange;
-  typedef std::vector<std::size_t> Polygon;
-  typedef std::vector<Polygon> PolygonRange;
-  typedef std::vector<CGAL::Color> ColorRange;
-  std::vector<PolygonRange> all_polygons;
-  return read_from_3mf<PointRanges,std::vector<PolygonRange>,
-      std::vector<ColorRange>, PointRange, PolygonRange, ColorRange>
-      (file_name, all_points, all_polygons, all_colors, names,
-       extract_polylines<PointRange, PolygonRange, ColorRange>);
-}
-
-
-template<typename PointRanges, typename ColorRanges>
-int read_point_clouds_from_3mf(const std::string& file_name,
-                               PointRanges& all_points,
-                               ColorRanges& all_colors,
-                               std::vector<std::string>& names
-                               )
-{
-  typedef typename PointRanges::value_type PointRange;
-  typedef std::vector<std::size_t> Polygon;
-  typedef std::vector<Polygon> PolygonRange;
-  typedef std::vector<CGAL::Color> ColorRange;
-  std::vector<PolygonRange> all_polygons;
-  return read_from_3mf<PointRanges,std::vector<PolygonRange>,
-      std::vector<ColorRange>, PointRange, PolygonRange, ColorRange>
-      (file_name, all_points, all_polygons, all_colors, names,
-       extract_point_clouds<PointRange, PolygonRange, ColorRange>);
-}
 }//end CGAL
 
 #endif // CGAL_IO_READ_3MF_H
diff -pruN 5.0.3-1/include/CGAL/IO/reader_helpers.h 5.1-1/include/CGAL/IO/reader_helpers.h
--- 5.0.3-1/include/CGAL/IO/reader_helpers.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/IO/reader_helpers.h	2020-09-08 12:15:28.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron_IO/include/CGAL/IO/reader_helpers.h $
-// $Id: reader_helpers.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron_IO/include/CGAL/IO/reader_helpers.h $
+// $Id: reader_helpers.h 752fc00 2020-02-20T12:14:40+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Mael Rouxel-Labbé
@@ -11,50 +11,13 @@
 #ifndef CGAL_IO_READER_HELPERS_H
 #define CGAL_IO_READER_HELPERS_H
 
-#include <CGAL/array.h>
-#include <CGAL/assertions.h>
-#include <CGAL/Has_member.h>
-
-#include <boost/mpl/logical.hpp>
-#include <boost/utility/enable_if.hpp>
+#include <CGAL/Container_helper.h>
+#include <CGAL/Point_3.h>
 
 namespace CGAL{
-
 namespace IO {
-
 namespace internal {
 
-CGAL_GENERATE_MEMBER_DETECTOR(size);
-CGAL_GENERATE_MEMBER_DETECTOR(resize);
-
-// Typical container
-template <class Container>
-void resize(Container& c, std::size_t size,
-            typename boost::enable_if_c<has_resize<Container>::value>::type* = nullptr)
-{
-  c.resize(size);
-}
-
-// Container without a resize() function, but with a size() function (e.g. an array)
-template <class Container>
-void resize(Container& CGAL_assertion_code(array), std::size_t CGAL_assertion_code(size),
-            typename boost::enable_if<
-              boost::mpl::and_<
-                boost::mpl::not_<has_resize<Container> >,
-                                 has_size<Container> > >::type* = nullptr)
-{
-  CGAL_assertion(array.size() == size);
-}
-
-// A class with neither resize() nor size(), can't enforce size (it better be correct!)
-template <class Container>
-void resize(Container&, std::size_t,
-            typename boost::disable_if<
-              boost::mpl::or_<has_resize<Container>,
-                              has_size<Container> > >::type* = nullptr)
-{
-}
-
 // Ideally this should be a std::is_constructible(double, double, double) but boost::is_constructible
 // is not safe to use without CXX11
 template <typename Kernel>
@@ -67,15 +30,13 @@ template <typename Point_3>
 void fill_point(const double x, const double y, const double z, Point_3& pt)
 {
   // just in case something weirder than arrays or CGAL points are used as points...
-  resize(pt, 3);
+  CGAL::internal::resize(pt, 3);
 
   pt[0] = x; pt[1] = y; pt[2] = z;
 }
 
 } // end namespace internal
-
 } // end namespace IO
-
 } // namespace CGAL
 
 #endif // CGAL_IO_READER_HELPERS_H
diff -pruN 5.0.3-1/include/CGAL/IO/read_las_points.h 5.1-1/include/CGAL/IO/read_las_points.h
--- 5.0.3-1/include/CGAL/IO/read_las_points.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/IO/read_las_points.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_processing_3/include/CGAL/IO/read_las_points.h $
-// $Id: read_las_points.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/IO/read_las_points.h $
+// $Id: read_las_points.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s) : Simon Giraudot
@@ -428,12 +428,20 @@ bool read_las_points_with_properties (st
    \param stream input stream.
    \param output output iterator over points.
 
-   \param np optional sequence of \ref psp_namedparameters "Named Parameters" among the ones listed below.
+   \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
    \cgalNamedParamsBegin
-     \cgalParamBegin{point_map} a model of `WritablePropertyMap` with value type `geom_traits::Point_3`.
-     If this parameter is omitted, `CGAL::Identity_property_map<geom_traits::Point_3>` is used.\cgalParamEnd
-     \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+     \cgalParamNBegin{point_map}
+       \cgalParamDescription{a property map associating points to the elements of the point range}
+       \cgalParamType{a model of `WritablePropertyMap` with value type `geom_traits::Point_3`}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{geom_traits}
+       \cgalParamDescription{an instance of a geometric traits class}
+       \cgalParamType{a model of `Kernel`}
+       \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+     \cgalParamNEnd
    \cgalNamedParamsEnd
 
    \return true on success.
@@ -461,8 +469,8 @@ bool read_las_points(std::istream& strea
 
   typedef Point_set_processing_3::Fake_point_range<OutputIteratorValueType> PointRange;
 
-  typedef typename Point_set_processing_3::GetPointMap<PointRange, CGAL_BGL_NP_CLASS>::type PointMap;
-  PointMap point_map = choose_parameter(get_parameter(np, internal_np::point_map), PointMap());
+  typedef typename CGAL::GetPointMap<PointRange, CGAL_BGL_NP_CLASS>::type PointMap;
+  PointMap point_map = choose_parameter<PointMap>(get_parameter(np, internal_np::point_map));
 
   return read_las_points_with_properties (stream, output,
                                           make_las_point_reader (point_map));
diff -pruN 5.0.3-1/include/CGAL/IO/read_off_points.h 5.1-1/include/CGAL/IO/read_off_points.h
--- 5.0.3-1/include/CGAL/IO/read_off_points.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/IO/read_off_points.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_processing_3/include/CGAL/IO/read_off_points.h $
-// $Id: read_off_points.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/IO/read_off_points.h $
+// $Id: read_off_points.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s) : Pierre Alliez and Laurent Saboret
@@ -45,15 +45,26 @@ namespace CGAL {
 
    \param stream input stream.
    \param output output iterator over points.
-   \param np optional sequence of \ref psp_namedparameters "Named Parameters" among the ones listed below.
+   \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
    \cgalNamedParamsBegin
-     \cgalParamBegin{point_map} a model of `WritablePropertyMap` with value type `geom_traits::Point_3`.
-     If this parameter is omitted, `CGAL::Identity_property_map<geom_traits::Point_3>` is used.\cgalParamEnd
-     \cgalParamBegin{normal_map} a model of `ReadWritePropertyMap` with value type
-     `geom_traits::Vector_3`. If this parameter is omitted, normals in the input stream are
-     ignored.\cgalParamEnd
-     \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+     \cgalParamNBegin{point_map}
+       \cgalParamDescription{a property map associating points to the elements of the point range}
+       \cgalParamType{a model of `WritablePropertyMap` with value type `geom_traits::Point_3`}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{normal_map}
+       \cgalParamDescription{a property map associating normals to the elements of the poing range}
+       \cgalParamType{a model of `ReadWritePropertyMap` with value type `geom_traits::Vector_3`}
+       \cgalParamDefault{If this parameter is omitted, normals in the input stream are ignored.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{geom_traits}
+       \cgalParamDescription{an instance of a geometric traits class}
+       \cgalParamType{a model of `Kernel`}
+       \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+     \cgalParamNEnd
    \cgalNamedParamsEnd
 
    \return true on success.
@@ -82,15 +93,15 @@ read_off_points(
   typedef Point_set_processing_3::Fake_point_range<OutputIteratorValueType> PointRange;
 
   // basic geometric types
-  typedef typename Point_set_processing_3::GetPointMap<PointRange, CGAL_BGL_NP_CLASS>::type PointMap;
+  typedef typename CGAL::GetPointMap<PointRange, CGAL_BGL_NP_CLASS>::type PointMap;
   typedef typename Point_set_processing_3::GetNormalMap<PointRange, CGAL_BGL_NP_CLASS>::type NormalMap;
   typedef typename Point_set_processing_3::GetK<PointRange, CGAL_BGL_NP_CLASS>::Kernel Kernel;
 
   bool has_normals = !(boost::is_same<NormalMap,
                        typename Point_set_processing_3::GetNormalMap<PointRange, CGAL_BGL_NP_CLASS>::NoMap>::value);
 
-  PointMap point_map = choose_parameter(get_parameter(np, internal_np::point_map), PointMap());
-  NormalMap normal_map = choose_parameter(get_parameter(np, internal_np::normal_map), NormalMap());
+  PointMap point_map = choose_parameter<PointMap>(get_parameter(np, internal_np::point_map));
+  NormalMap normal_map = choose_parameter<NormalMap>(get_parameter(np, internal_np::normal_map));
 
   // value_type_traits is a workaround as back_insert_iterator's value_type is void
   // typedef typename value_type_traits<OutputIterator>::type Enriched_point;
diff -pruN 5.0.3-1/include/CGAL/IO/read_ply_points.h 5.1-1/include/CGAL/IO/read_ply_points.h
--- 5.0.3-1/include/CGAL/IO/read_ply_points.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/IO/read_ply_points.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_processing_3/include/CGAL/IO/read_ply_points.h $
-// $Id: read_ply_points.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/IO/read_ply_points.h $
+// $Id: read_ply_points.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s) : Simon Giraudot
@@ -201,15 +201,26 @@ bool read_ply_points_with_properties (st
 
    \param stream input stream.
    \param output output iterator over points.
-   \param np optional sequence of \ref psp_namedparameters "Named Parameters" among the ones listed below.
+   \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
    \cgalNamedParamsBegin
-     \cgalParamBegin{point_map} a model of `WritablePropertyMap` with value type `geom_traits::Point_3`.
-     If this parameter is omitted, `CGAL::Identity_property_map<geom_traits::Point_3>` is used.\cgalParamEnd
-     \cgalParamBegin{normal_map} a model of `ReadWritePropertyMap` with value type
-     `geom_traits::Vector_3`. If this parameter is omitted, normals in the input stream are
-     ignored.\cgalParamEnd
-     \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+     \cgalParamNBegin{point_map}
+       \cgalParamDescription{a property map associating points to the elements of the point range}
+       \cgalParamType{a model of `WritablePropertyMap` with value type `geom_traits::Point_3`}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{normal_map}
+       \cgalParamDescription{a property map associating normals to the elements of the poing range}
+       \cgalParamType{a model of `ReadWritePropertyMap` with value type `geom_traits::Vector_3`}
+       \cgalParamDefault{If this parameter is omitted, normals in the input stream are ignored.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{geom_traits}
+       \cgalParamDescription{an instance of a geometric traits class}
+       \cgalParamType{a model of `Kernel`}
+       \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+     \cgalParamNEnd
    \cgalNamedParamsEnd
 
    \return true on success.
@@ -238,14 +249,14 @@ bool read_ply_points(std::istream& strea
   typedef Point_set_processing_3::Fake_point_range<OutputIteratorValueType> PointRange;
 
   // basic geometric types
-  typedef typename Point_set_processing_3::GetPointMap<PointRange, CGAL_BGL_NP_CLASS>::type PointMap;
+  typedef typename CGAL::GetPointMap<PointRange, CGAL_BGL_NP_CLASS>::type PointMap;
   typedef typename Point_set_processing_3::GetNormalMap<PointRange, CGAL_BGL_NP_CLASS>::type NormalMap;
 
   bool has_normals = !(boost::is_same<NormalMap,
                        typename Point_set_processing_3::GetNormalMap<PointRange, CGAL_BGL_NP_CLASS>::NoMap>::value);
 
-  PointMap point_map = choose_parameter(get_parameter(np, internal_np::point_map), PointMap());
-  NormalMap normal_map = choose_parameter(get_parameter(np, internal_np::normal_map), NormalMap());
+  PointMap point_map = choose_parameter<PointMap>(get_parameter(np, internal_np::point_map));
+  NormalMap normal_map = choose_parameter<NormalMap>(get_parameter(np, internal_np::normal_map));
 
   if (has_normals)
     return read_ply_points_with_properties (stream, output,
diff -pruN 5.0.3-1/include/CGAL/IO/read_xyz_points.h 5.1-1/include/CGAL/IO/read_xyz_points.h
--- 5.0.3-1/include/CGAL/IO/read_xyz_points.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/IO/read_xyz_points.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_processing_3/include/CGAL/IO/read_xyz_points.h $
-// $Id: read_xyz_points.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/IO/read_xyz_points.h $
+// $Id: read_xyz_points.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s) : Pierre Alliez and Laurent Saboret
@@ -44,15 +44,26 @@ namespace CGAL {
 
    \param stream input stream.
    \param output output iterator over points.
-   \param np optional sequence of \ref psp_namedparameters "Named Parameters" among the ones listed below.
+   \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
    \cgalNamedParamsBegin
-     \cgalParamBegin{point_map} a model of `WritablePropertyMap` with value type `geom_traits::Point_3`.
-     If this parameter is omitted, `CGAL::Identity_property_map<geom_traits::Point_3>` is used.\cgalParamEnd
-     \cgalParamBegin{normal_map} a model of `ReadWritePropertyMap` with value type
-     `geom_traits::Vector_3`. If this parameter is omitted, normals in the input stream are
-     ignored.\cgalParamEnd
-     \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+     \cgalParamNBegin{point_map}
+       \cgalParamDescription{a property map associating points to the elements of the point range}
+       \cgalParamType{a model of `WritablePropertyMap` with value type `geom_traits::Point_3`}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{normal_map}
+       \cgalParamDescription{a property map associating normals to the elements of the poing range}
+       \cgalParamType{a model of `ReadWritePropertyMap` with value type `geom_traits::Vector_3`}
+       \cgalParamDefault{If this parameter is omitted, normals in the input stream are ignored.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{geom_traits}
+       \cgalParamDescription{an instance of a geometric traits class}
+       \cgalParamType{a model of `Kernel`}
+       \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+     \cgalParamNEnd
    \cgalNamedParamsEnd
 
    \return true on success.
@@ -81,15 +92,15 @@ read_xyz_points(
   typedef Point_set_processing_3::Fake_point_range<OutputIteratorValueType> PointRange;
 
   // basic geometric types
-  typedef typename Point_set_processing_3::GetPointMap<PointRange, CGAL_BGL_NP_CLASS>::type PointMap;
+  typedef typename CGAL::GetPointMap<PointRange, CGAL_BGL_NP_CLASS>::type PointMap;
   typedef typename Point_set_processing_3::GetNormalMap<PointRange, CGAL_BGL_NP_CLASS>::type NormalMap;
   typedef typename Point_set_processing_3::GetK<PointRange, CGAL_BGL_NP_CLASS>::Kernel Kernel;
 
   bool has_normals = !(boost::is_same<NormalMap,
                        typename Point_set_processing_3::GetNormalMap<PointRange, CGAL_BGL_NP_CLASS>::NoMap>::value);
 
-  PointMap point_map = choose_parameter(get_parameter(np, internal_np::point_map), PointMap());
-  NormalMap normal_map = choose_parameter(get_parameter(np, internal_np::normal_map), NormalMap());
+  PointMap point_map = choose_parameter<PointMap>(get_parameter(np, internal_np::point_map));
+  NormalMap normal_map = choose_parameter<NormalMap>(get_parameter(np, internal_np::normal_map));
 
   // value_type_traits is a workaround as back_insert_iterator's value_type is void
   //typedef typename value_type_traits<OutputIterator>::type Enriched_point;
diff -pruN 5.0.3-1/include/CGAL/IO/Scanner_OFF.h 5.1-1/include/CGAL/IO/Scanner_OFF.h
--- 5.0.3-1/include/CGAL/IO/Scanner_OFF.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Scanner_OFF.h	2020-09-08 12:15:42.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/Scanner_OFF.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/Scanner_OFF.h $
 // $Id: Scanner_OFF.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/scan_OFF.h 5.1-1/include/CGAL/IO/scan_OFF.h
--- 5.0.3-1/include/CGAL/IO/scan_OFF.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/IO/scan_OFF.h	2020-09-08 12:15:28.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron_IO/include/CGAL/IO/scan_OFF.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron_IO/include/CGAL/IO/scan_OFF.h $
 // $Id: scan_OFF.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/STL_reader.h 5.1-1/include/CGAL/IO/STL_reader.h
--- 5.0.3-1/include/CGAL/IO/STL_reader.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/IO/STL_reader.h	2020-09-08 12:15:28.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron_IO/include/CGAL/IO/STL_reader.h $
-// $Id: STL_reader.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron_IO/include/CGAL/IO/STL_reader.h $
+// $Id: STL_reader.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Andreas Fabri,
@@ -15,6 +15,8 @@
 #include <CGAL/IO/io.h>
 #include <CGAL/IO/reader_helpers.h>
 
+#include <CGAL/Container_helper.h>
+
 #include <boost/cstdint.hpp>
 
 #include <cctype>
@@ -43,7 +45,7 @@ bool read_ASCII_facet(std::istream& inpu
   double x,y,z;
   Point p;
   Triangle ijk;
-  IO::internal::resize(ijk, 3);
+  CGAL::internal::resize(ijk, 3);
 
   while(input >> s)
   {
@@ -213,7 +215,7 @@ bool parse_binary_STL(std::istream& inpu
     }
 
     Triangle ijk;
-    IO::internal::resize(ijk, 3);
+    CGAL::internal::resize(ijk, 3);
 
     for(int j=0; j<3; ++j)
     {
diff -pruN 5.0.3-1/include/CGAL/IO/STL_writer.h 5.1-1/include/CGAL/IO/STL_writer.h
--- 5.0.3-1/include/CGAL/IO/STL_writer.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/IO/STL_writer.h	2020-09-08 12:15:28.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron_IO/include/CGAL/IO/STL_writer.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron_IO/include/CGAL/IO/STL_writer.h $
 // $Id: STL_writer.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Tee_for_output_iterator.h 5.1-1/include/CGAL/IO/Tee_for_output_iterator.h
--- 5.0.3-1/include/CGAL/IO/Tee_for_output_iterator.h	2020-08-01 19:02:19.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Tee_for_output_iterator.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_2/include/CGAL/IO/Tee_for_output_iterator.h $
-// $Id: Tee_for_output_iterator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_2/include/CGAL/IO/Tee_for_output_iterator.h $
+// $Id: Tee_for_output_iterator.h 319383c 2020-05-20T09:47:58+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
diff -pruN 5.0.3-1/include/CGAL/IO/trace.h 5.1-1/include/CGAL/IO/trace.h
--- 5.0.3-1/include/CGAL/IO/trace.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/IO/trace.h	2020-09-08 12:15:42.000000000 +0000
@@ -2,7 +2,7 @@
 // All rights reserved.
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/trace.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/trace.h $
 // $Id: trace.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/traits_linestring.h 5.1-1/include/CGAL/IO/traits_linestring.h
--- 5.0.3-1/include/CGAL/IO/traits_linestring.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/IO/traits_linestring.h	2020-09-08 12:15:42.000000000 +0000
@@ -6,7 +6,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/traits_linestring.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/traits_linestring.h $
 // $Id: traits_linestring.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/traits_multilinestring.h 5.1-1/include/CGAL/IO/traits_multilinestring.h
--- 5.0.3-1/include/CGAL/IO/traits_multilinestring.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/IO/traits_multilinestring.h	2020-09-08 12:15:42.000000000 +0000
@@ -6,7 +6,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/traits_multilinestring.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/traits_multilinestring.h $
 // $Id: traits_multilinestring.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/traits_multipoint.h 5.1-1/include/CGAL/IO/traits_multipoint.h
--- 5.0.3-1/include/CGAL/IO/traits_multipoint.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/IO/traits_multipoint.h	2020-09-08 12:15:42.000000000 +0000
@@ -6,7 +6,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/traits_multipoint.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/traits_multipoint.h $
 // $Id: traits_multipoint.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/traits_multipolygon.h 5.1-1/include/CGAL/IO/traits_multipolygon.h
--- 5.0.3-1/include/CGAL/IO/traits_multipolygon.h	2020-08-01 19:02:54.000000000 +0000
+++ 5.1-1/include/CGAL/IO/traits_multipolygon.h	2020-09-08 12:15:42.000000000 +0000
@@ -6,7 +6,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/traits_multipolygon.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/traits_multipolygon.h $
 // $Id: traits_multipolygon.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/traits_point_3.h 5.1-1/include/CGAL/IO/traits_point_3.h
--- 5.0.3-1/include/CGAL/IO/traits_point_3.h	2020-08-01 19:02:54.000000000 +0000
+++ 5.1-1/include/CGAL/IO/traits_point_3.h	2020-09-08 12:15:42.000000000 +0000
@@ -6,7 +6,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/traits_point_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/traits_point_3.h $
 // $Id: traits_point_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/traits_point.h 5.1-1/include/CGAL/IO/traits_point.h
--- 5.0.3-1/include/CGAL/IO/traits_point.h	2020-08-01 19:02:54.000000000 +0000
+++ 5.1-1/include/CGAL/IO/traits_point.h	2020-09-08 12:15:42.000000000 +0000
@@ -6,7 +6,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/traits_point.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/traits_point.h $
 // $Id: traits_point.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/traits_polygon.h 5.1-1/include/CGAL/IO/traits_polygon.h
--- 5.0.3-1/include/CGAL/IO/traits_polygon.h	2020-08-01 19:02:54.000000000 +0000
+++ 5.1-1/include/CGAL/IO/traits_polygon.h	2020-09-08 12:15:42.000000000 +0000
@@ -6,7 +6,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/traits_polygon.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/traits_polygon.h $
 // $Id: traits_polygon.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Triangulation_geomview_ostream_2.h 5.1-1/include/CGAL/IO/Triangulation_geomview_ostream_2.h
--- 5.0.3-1/include/CGAL/IO/Triangulation_geomview_ostream_2.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Triangulation_geomview_ostream_2.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/IO/Triangulation_geomview_ostream_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/IO/Triangulation_geomview_ostream_2.h $
 // $Id: Triangulation_geomview_ostream_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Triangulation_geomview_ostream_3.h 5.1-1/include/CGAL/IO/Triangulation_geomview_ostream_3.h
--- 5.0.3-1/include/CGAL/IO/Triangulation_geomview_ostream_3.h	2020-08-01 19:02:59.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Triangulation_geomview_ostream_3.h	2020-09-08 12:15:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_3/include/CGAL/IO/Triangulation_geomview_ostream_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_3/include/CGAL/IO/Triangulation_geomview_ostream_3.h $
 // $Id: Triangulation_geomview_ostream_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Triangulation_off_ostream_2.h 5.1-1/include/CGAL/IO/Triangulation_off_ostream_2.h
--- 5.0.3-1/include/CGAL/IO/Triangulation_off_ostream_2.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Triangulation_off_ostream_2.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/IO/Triangulation_off_ostream_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/IO/Triangulation_off_ostream_2.h $
 // $Id: Triangulation_off_ostream_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Triangulation_off_ostream_3.h 5.1-1/include/CGAL/IO/Triangulation_off_ostream_3.h
--- 5.0.3-1/include/CGAL/IO/Triangulation_off_ostream_3.h	2020-08-01 19:02:59.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Triangulation_off_ostream_3.h	2020-09-08 12:15:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_3/include/CGAL/IO/Triangulation_off_ostream_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_3/include/CGAL/IO/Triangulation_off_ostream_3.h $
 // $Id: Triangulation_off_ostream_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Triangulation_off_ostream.h 5.1-1/include/CGAL/IO/Triangulation_off_ostream.h
--- 5.0.3-1/include/CGAL/IO/Triangulation_off_ostream.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Triangulation_off_ostream.h	2020-09-08 12:15:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation/include/CGAL/IO/Triangulation_off_ostream.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation/include/CGAL/IO/Triangulation_off_ostream.h $
 // $Id: Triangulation_off_ostream.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Triangulation_ps_stream.h 5.1-1/include/CGAL/IO/Triangulation_ps_stream.h
--- 5.0.3-1/include/CGAL/IO/Triangulation_ps_stream.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Triangulation_ps_stream.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/IO/Triangulation_ps_stream.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/IO/Triangulation_ps_stream.h $
 // $Id: Triangulation_ps_stream.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/Verbose_ostream.h 5.1-1/include/CGAL/IO/Verbose_ostream.h
--- 5.0.3-1/include/CGAL/IO/Verbose_ostream.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Verbose_ostream.h	2020-09-08 12:15:42.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/Verbose_ostream.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/Verbose_ostream.h $
 // $Id: Verbose_ostream.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/VRML_1_ostream.h 5.1-1/include/CGAL/IO/VRML_1_ostream.h
--- 5.0.3-1/include/CGAL/IO/VRML_1_ostream.h	2020-08-01 19:02:25.000000000 +0000
+++ 5.1-1/include/CGAL/IO/VRML_1_ostream.h	2020-09-08 12:14:59.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Inventor/include/CGAL/IO/VRML_1_ostream.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Inventor/include/CGAL/IO/VRML_1_ostream.h $
 // $Id: VRML_1_ostream.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/VRML_2_ostream.h 5.1-1/include/CGAL/IO/VRML_2_ostream.h
--- 5.0.3-1/include/CGAL/IO/VRML_2_ostream.h	2020-08-01 19:02:25.000000000 +0000
+++ 5.1-1/include/CGAL/IO/VRML_2_ostream.h	2020-09-08 12:14:59.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Inventor/include/CGAL/IO/VRML_2_ostream.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Inventor/include/CGAL/IO/VRML_2_ostream.h $
 // $Id: VRML_2_ostream.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/WKT.h 5.1-1/include/CGAL/IO/WKT.h
--- 5.0.3-1/include/CGAL/IO/WKT.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/IO/WKT.h	2020-09-08 12:15:42.000000000 +0000
@@ -6,7 +6,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/WKT.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/WKT.h $
 // $Id: WKT.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/write_3mf.h 5.1-1/include/CGAL/IO/write_3mf.h
--- 5.0.3-1/include/CGAL/IO/write_3mf.h	2020-08-01 19:02:54.000000000 +0000
+++ 5.1-1/include/CGAL/IO/write_3mf.h	2020-09-08 12:15:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/write_3mf.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/write_3mf.h $
 // $Id: write_3mf.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/write_las_points.h 5.1-1/include/CGAL/IO/write_las_points.h
--- 5.0.3-1/include/CGAL/IO/write_las_points.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/IO/write_las_points.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_processing_3/include/CGAL/IO/write_las_points.h $
-// $Id: write_las_points.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/IO/write_las_points.h $
+// $Id: write_las_points.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s) : Simon Giraudot
@@ -254,12 +254,19 @@ bool write_las_points_with_properties (s
 
    \param stream output stream.
    \param points input point range.
-   \param np optional sequence of \ref psp_namedparameters "Named Parameters" among the ones listed below.
+   \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
    \cgalNamedParamsBegin
-     \cgalParamBegin{point_map} a model of `ReadablePropertyMap` with value type `geom_traits::Point_3`.
-     If this parameter is omitted, `CGAL::Identity_property_map<geom_traits::Point_3>` is used.\cgalParamEnd
-     \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+     \cgalParamNBegin{point_map}
+       \cgalParamDescription{a property map associating points to the elements of the point range}
+       \cgalParamType{a model of `ReadablePropertyMap` with value type `geom_traits::Point_3`}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>`}
+     \cgalParamNEnd
+     \cgalParamNBegin{geom_traits}
+       \cgalParamDescription{an instance of a geometric traits class}
+       \cgalParamType{a model of `Kernel`}
+       \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+     \cgalParamNEnd
    \cgalNamedParamsEnd
 
    \return true on success.
@@ -276,8 +283,8 @@ write_las_points(
   using parameters::choose_parameter;
   using parameters::get_parameter;
 
-  typedef typename Point_set_processing_3::GetPointMap<PointRange, NamedParameters>::type PointMap;
-  PointMap point_map = choose_parameter(get_parameter(np, internal_np::point_map), PointMap());
+  typedef typename CGAL::GetPointMap<PointRange, NamedParameters>::type PointMap;
+  PointMap point_map = choose_parameter<PointMap>(get_parameter(np, internal_np::point_map));
 
   return write_las_points_with_properties (stream, points, make_las_point_writer(point_map));
 }
diff -pruN 5.0.3-1/include/CGAL/IO/write_off_points.h 5.1-1/include/CGAL/IO/write_off_points.h
--- 5.0.3-1/include/CGAL/IO/write_off_points.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/IO/write_off_points.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_processing_3/include/CGAL/IO/write_off_points.h $
-// $Id: write_off_points.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/IO/write_off_points.h $
+// $Id: write_off_points.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s) : Pierre Alliez and Laurent Saboret
@@ -40,15 +40,26 @@ namespace CGAL {
 
    \param stream output stream.
    \param points input point range.
-   \param np optional sequence of \ref psp_namedparameters "Named Parameters" among the ones listed below.
+   \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
    \cgalNamedParamsBegin
-     \cgalParamBegin{point_map} a model of `ReadablePropertyMap` with value type `geom_traits::Point_3`.
-     If this parameter is omitted, `CGAL::Identity_property_map<geom_traits::Point_3>` is used.\cgalParamEnd
-     \cgalParamBegin{normal_map} a model of `ReadablePropertyMap` with value type
-     `geom_traits::Vector_3`.\cgalParamEnd If this parameter is omitted, normals are not written to the
-     output stream.\cgalParamEnd
-     \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+     \cgalParamNBegin{point_map}
+       \cgalParamDescription{a property map associating points to the elements of the point range}
+       \cgalParamType{a model of `ReadablePropertyMap` with value type `geom_traits::Point_3`}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{normal_map}
+       \cgalParamDescription{a property map associating normals to the elements of the poing range}
+       \cgalParamType{a model of `ReadablePropertyMap` with value type `geom_traits::Vector_3`}
+       \cgalParamDefault{If this parameter is omitted, normals are not written in the output stream.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{geom_traits}
+       \cgalParamDescription{an instance of a geometric traits class}
+       \cgalParamType{a model of `Kernel`}
+       \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+     \cgalParamNEnd
    \cgalNamedParamsEnd
 
    \return `true` on success.
@@ -66,14 +77,14 @@ write_off_points(
   using parameters::get_parameter;
 
   // basic geometric types
-  typedef typename Point_set_processing_3::GetPointMap<PointRange, NamedParameters>::type PointMap;
+  typedef typename CGAL::GetPointMap<PointRange, NamedParameters>::type PointMap;
   typedef typename Point_set_processing_3::GetNormalMap<PointRange, NamedParameters>::type NormalMap;
 
   bool has_normals = !(boost::is_same<NormalMap,
                        typename Point_set_processing_3::GetNormalMap<PointRange, NamedParameters>::NoMap>::value);
 
-  PointMap point_map = choose_parameter(get_parameter(np, internal_np::point_map), PointMap());
-  NormalMap normal_map = choose_parameter(get_parameter(np, internal_np::normal_map), NormalMap());
+  PointMap point_map = choose_parameter<PointMap>(get_parameter(np, internal_np::point_map));
+  NormalMap normal_map = choose_parameter<NormalMap>(get_parameter(np, internal_np::normal_map));
 
   CGAL_point_set_processing_precondition(points.begin() != points.end());
 
diff -pruN 5.0.3-1/include/CGAL/IO/write_ply_points.h 5.1-1/include/CGAL/IO/write_ply_points.h
--- 5.0.3-1/include/CGAL/IO/write_ply_points.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/IO/write_ply_points.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_processing_3/include/CGAL/IO/write_ply_points.h $
-// $Id: write_ply_points.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/IO/write_ply_points.h $
+// $Id: write_ply_points.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s) : Simon Giraudot
@@ -159,15 +159,26 @@ write_ply_points_with_properties(
 
    \param stream output stream.
    \param points input point range.
-   \param np optional sequence of \ref psp_namedparameters "Named Parameters" among the ones listed below.
+   \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
    \cgalNamedParamsBegin
-     \cgalParamBegin{point_map} a model of `ReadablePropertyMap` with value type `geom_traits::Point_3`.
-     If this parameter is omitted, `CGAL::Identity_property_map<geom_traits::Point_3>` is used.\cgalParamEnd
-     \cgalParamBegin{normal_map} a model of `ReadablePropertyMap` with value type
-     `geom_traits::Vector_3`.\cgalParamEnd If this parameter is omitted, normals are not written to the
-     output stream.\cgalParamEnd
-     \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+     \cgalParamNBegin{point_map}
+       \cgalParamDescription{a property map associating points to the elements of the point range}
+       \cgalParamType{a model of `ReadablePropertyMap` with value type `geom_traits::Point_3`}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{normal_map}
+       \cgalParamDescription{a property map associating normals to the elements of the poing range}
+       \cgalParamType{a model of `ReadablePropertyMap` with value type `geom_traits::Vector_3`}
+       \cgalParamDefault{If this parameter is omitted, normals are not written in the output stream.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{geom_traits}
+       \cgalParamDescription{an instance of a geometric traits class}
+       \cgalParamType{a model of `Kernel`}
+       \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+     \cgalParamNEnd
    \cgalNamedParamsEnd
 
    \return true on success.
@@ -185,14 +196,14 @@ write_ply_points(
   using parameters::get_parameter;
 
   // basic geometric types
-  typedef typename Point_set_processing_3::GetPointMap<PointRange, NamedParameters>::type PointMap;
+  typedef typename CGAL::GetPointMap<PointRange, NamedParameters>::type PointMap;
   typedef typename Point_set_processing_3::GetNormalMap<PointRange, NamedParameters>::type NormalMap;
 
   bool has_normals = !(boost::is_same<NormalMap,
                        typename Point_set_processing_3::GetNormalMap<PointRange, NamedParameters>::NoMap>::value);
 
-  PointMap point_map = choose_parameter(get_parameter(np, internal_np::point_map), PointMap());
-  NormalMap normal_map = choose_parameter(get_parameter(np, internal_np::normal_map), NormalMap());
+  PointMap point_map = choose_parameter<PointMap>(get_parameter(np, internal_np::point_map));
+  NormalMap normal_map = choose_parameter<NormalMap>(get_parameter(np, internal_np::normal_map));
 
   if (has_normals)
     return write_ply_points_with_properties(
diff -pruN 5.0.3-1/include/CGAL/IO/Writer_OFF.h 5.1-1/include/CGAL/IO/Writer_OFF.h
--- 5.0.3-1/include/CGAL/IO/Writer_OFF.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/IO/Writer_OFF.h	2020-09-08 12:15:42.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/Writer_OFF.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/Writer_OFF.h $
 // $Id: Writer_OFF.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/write_vtk.h 5.1-1/include/CGAL/IO/write_vtk.h
--- 5.0.3-1/include/CGAL/IO/write_vtk.h	2020-08-01 19:02:54.000000000 +0000
+++ 5.1-1/include/CGAL/IO/write_vtk.h	2020-09-08 12:15:42.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_support/include/CGAL/IO/write_vtk.h $
-// $Id: write_vtk.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_support/include/CGAL/IO/write_vtk.h $
+// $Id: write_vtk.h c8b73c9 2020-08-07T06:40:23+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -13,8 +13,6 @@
 #ifndef CGAL_WRITE_VTK_IO_H
 #define CGAL_WRITE_VTK_IO_H
 
-#include <CGAL/license/Polyhedron.h>
-
 #include <fstream>
 #include <vector>
 template <class FT>
diff -pruN 5.0.3-1/include/CGAL/IO/write_vtu.h 5.1-1/include/CGAL/IO/write_vtu.h
--- 5.0.3-1/include/CGAL/IO/write_vtu.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/IO/write_vtu.h	2020-09-08 12:15:06.000000000 +0000
@@ -5,7 +5,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_2/include/CGAL/IO/write_vtu.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_2/include/CGAL/IO/write_vtu.h $
 // $Id: write_vtu.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/IO/write_xyz_points.h 5.1-1/include/CGAL/IO/write_xyz_points.h
--- 5.0.3-1/include/CGAL/IO/write_xyz_points.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/IO/write_xyz_points.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_processing_3/include/CGAL/IO/write_xyz_points.h $
-// $Id: write_xyz_points.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/IO/write_xyz_points.h $
+// $Id: write_xyz_points.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s) : Pierre Alliez and Laurent Saboret
@@ -40,15 +40,26 @@ namespace CGAL {
 
    \param stream output stream.
    \param points input point range.
-   \param np optional sequence of \ref psp_namedparameters "Named Parameters" among the ones listed below.
+   \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
    \cgalNamedParamsBegin
-     \cgalParamBegin{point_map} a model of `ReadablePropertyMap` with value type `geom_traits::Point_3`.
-     If this parameter is omitted, `CGAL::Identity_property_map<geom_traits::Point_3>` is used.\cgalParamEnd
-     \cgalParamBegin{normal_map} a model of `ReadablePropertyMap` with value type
-     `geom_traits::Vector_3`.\cgalParamEnd If this parameter is omitted, normals are not written to the
-     output stream.\cgalParamEnd
-     \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+     \cgalParamNBegin{point_map}
+       \cgalParamDescription{a property map associating points to the elements of the point range}
+       \cgalParamType{a model of `ReadablePropertyMap` with value type `geom_traits::Point_3`}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{normal_map}
+       \cgalParamDescription{a property map associating normals to the elements of the poing range}
+       \cgalParamType{a model of `ReadablePropertyMap` with value type `geom_traits::Vector_3`}
+       \cgalParamDefault{If this parameter is omitted, normals are not written in the output stream.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{geom_traits}
+       \cgalParamDescription{an instance of a geometric traits class}
+       \cgalParamType{a model of `Kernel`}
+       \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+     \cgalParamNEnd
    \cgalNamedParamsEnd
 
    \return true on success.
@@ -66,14 +77,14 @@ write_xyz_points(
   using parameters::get_parameter;
 
   // basic geometric types
-  typedef typename Point_set_processing_3::GetPointMap<PointRange, NamedParameters>::type PointMap;
+  typedef typename CGAL::GetPointMap<PointRange, NamedParameters>::type PointMap;
   typedef typename Point_set_processing_3::GetNormalMap<PointRange, NamedParameters>::type NormalMap;
 
   bool has_normals = !(boost::is_same<NormalMap,
                        typename Point_set_processing_3::GetNormalMap<PointRange, NamedParameters>::NoMap>::value);
 
-  PointMap point_map = choose_parameter(get_parameter(np, internal_np::point_map), PointMap());
-  NormalMap normal_map = choose_parameter(get_parameter(np, internal_np::normal_map), NormalMap());
+  PointMap point_map = choose_parameter<PointMap>(get_parameter(np, internal_np::point_map));
+  NormalMap normal_map = choose_parameter<NormalMap>(get_parameter(np, internal_np::normal_map));
 
   CGAL_point_set_processing_precondition(points.begin() != points.end());
 
diff -pruN 5.0.3-1/include/CGAL/ipower.h 5.1-1/include/CGAL/ipower.h
--- 5.0.3-1/include/CGAL/ipower.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/ipower.h	2020-09-08 12:14:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_foundations/include/CGAL/ipower.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_foundations/include/CGAL/ipower.h $
 // $Id: ipower.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
@@ -13,7 +13,7 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_foundations/include/CGAL/ipower.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_foundations/include/CGAL/ipower.h $
 
 #ifndef CGAL_IPOWER_H
 #define CGAL_IPOWER_H
diff -pruN 5.0.3-1/include/CGAL/Is_a_predicate.h 5.1-1/include/CGAL/Is_a_predicate.h
--- 5.0.3-1/include/CGAL/Is_a_predicate.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Is_a_predicate.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Is_a_predicate.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Is_a_predicate.h $
 // $Id: Is_a_predicate.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/is_convertible.h 5.1-1/include/CGAL/is_convertible.h
--- 5.0.3-1/include/CGAL/is_convertible.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/is_convertible.h	2020-09-08 12:15:32.000000000 +0000
@@ -6,7 +6,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/is_convertible.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/is_convertible.h $
 // $Id: is_convertible.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Is_extended_kernel.h 5.1-1/include/CGAL/Is_extended_kernel.h
--- 5.0.3-1/include/CGAL/Is_extended_kernel.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Is_extended_kernel.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_2/include/CGAL/Is_extended_kernel.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_2/include/CGAL/Is_extended_kernel.h $
 // $Id: Is_extended_kernel.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/is_iterator.h 5.1-1/include/CGAL/is_iterator.h
--- 5.0.3-1/include/CGAL/is_iterator.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/is_iterator.h	2020-09-08 12:15:32.000000000 +0000
@@ -6,7 +6,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/is_iterator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/is_iterator.h $
 // $Id: is_iterator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Iso_cuboid_3.h 5.1-1/include/CGAL/Iso_cuboid_3.h
--- 5.0.3-1/include/CGAL/Iso_cuboid_3.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Iso_cuboid_3.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Iso_cuboid_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Iso_cuboid_3.h $
 // $Id: Iso_cuboid_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Iso_rectangle_2.h 5.1-1/include/CGAL/Iso_rectangle_2.h
--- 5.0.3-1/include/CGAL/Iso_rectangle_2.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Iso_rectangle_2.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Iso_rectangle_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Iso_rectangle_2.h $
 // $Id: Iso_rectangle_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Iso_rectangle_d.h 5.1-1/include/CGAL/Iso_rectangle_d.h
--- 5.0.3-1/include/CGAL/Iso_rectangle_d.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Iso_rectangle_d.h	2020-09-08 12:15:39.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_searching/include/CGAL/Iso_rectangle_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_searching/include/CGAL/Iso_rectangle_d.h $
 // $Id: Iso_rectangle_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/is_streamable.h 5.1-1/include/CGAL/is_streamable.h
--- 5.0.3-1/include/CGAL/is_streamable.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/is_streamable.h	2020-09-08 12:15:32.000000000 +0000
@@ -1,7 +1,7 @@
 // Copyright (c) 2012  GeometryFactory Sarl (France)
 // All rights reserved.
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/is_streamable.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/is_streamable.h $
 // $Id: is_streamable.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: BSL-1.0
 //
diff -pruN 5.0.3-1/include/CGAL/is_y_monotone_2.h 5.1-1/include/CGAL/is_y_monotone_2.h
--- 5.0.3-1/include/CGAL/is_y_monotone_2.h	2020-08-01 19:02:37.000000000 +0000
+++ 5.1-1/include/CGAL/is_y_monotone_2.h	2020-09-08 12:15:18.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Partition_2/include/CGAL/is_y_monotone_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Partition_2/include/CGAL/is_y_monotone_2.h $
 // $Id: is_y_monotone_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/iterator_from_indices.h 5.1-1/include/CGAL/iterator_from_indices.h
--- 5.0.3-1/include/CGAL/iterator_from_indices.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/iterator_from_indices.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/iterator_from_indices.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/iterator_from_indices.h $
 // $Id: iterator_from_indices.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/iterator.h 5.1-1/include/CGAL/iterator.h
--- 5.0.3-1/include/CGAL/iterator.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/iterator.h	2020-09-08 12:15:32.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/iterator.h $
-// $Id: iterator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/iterator.h $
+// $Id: iterator.h c1d8fb6 2020-04-22T13:47:33+02:00 Simon Giraudot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -1471,6 +1471,15 @@ dispatch_or_drop_output(O... o)
   return Dispatch_or_drop_output_iterator<std::tuple<V...>, std::tuple<O...> >(o...);
 }
 
+
+// Trick to select iterator or const_iterator depending on the range constness
+template <typename RangeRef>
+struct Range_iterator_type;
+template <typename RangeRef>
+struct Range_iterator_type<RangeRef&>       { typedef typename RangeRef::iterator       type; };
+template <typename RangeRef>
+struct Range_iterator_type<const RangeRef&> { typedef typename RangeRef::const_iterator type; };
+
 } //namespace CGAL
 
 #include <CGAL/enable_warnings.h>
diff -pruN 5.0.3-1/include/CGAL/Iterator_project.h 5.1-1/include/CGAL/Iterator_project.h
--- 5.0.3-1/include/CGAL/Iterator_project.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Iterator_project.h	2020-09-08 12:15:32.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Iterator_project.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Iterator_project.h $
 // $Id: Iterator_project.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Iterator_range.h 5.1-1/include/CGAL/Iterator_range.h
--- 5.0.3-1/include/CGAL/Iterator_range.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Iterator_range.h	2020-09-08 12:15:32.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Iterator_range.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Iterator_range.h $
 // $Id: Iterator_range.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Iterator_transform.h 5.1-1/include/CGAL/Iterator_transform.h
--- 5.0.3-1/include/CGAL/Iterator_transform.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Iterator_transform.h	2020-09-08 12:15:32.000000000 +0000
@@ -8,7 +8,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Iterator_transform.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Iterator_transform.h $
 // $Id: Iterator_transform.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/jet_estimate_normals.h 5.1-1/include/CGAL/jet_estimate_normals.h
--- 5.0.3-1/include/CGAL/jet_estimate_normals.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/jet_estimate_normals.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_processing_3/include/CGAL/jet_estimate_normals.h $
-// $Id: jet_estimate_normals.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/jet_estimate_normals.h $
+// $Id: jet_estimate_normals.h 93f1cd9 2020-07-16T09:53:31+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s) : Pierre Alliez and Laurent Saboret and Marc Pouget and Frederic Cazals
@@ -17,9 +17,9 @@
 #include <CGAL/disable_warnings.h>
 
 #include <CGAL/IO/trace.h>
-#include <CGAL/Search_traits_3.h>
-#include <CGAL/Orthogonal_k_neighbor_search.h>
-#include <CGAL/Point_set_processing_3/internal/neighbor_query.h>
+#include <CGAL/Point_set_processing_3/internal/Neighbor_query.h>
+#include <CGAL/Point_set_processing_3/internal/Callback_wrapper.h>
+#include <CGAL/for_each.h>
 #include <CGAL/Monge_via_jet_fitting.h>
 #include <CGAL/property_map.h>
 #include <CGAL/point_set_processing_assertions.h>
@@ -32,13 +32,6 @@
 #include <iterator>
 #include <list>
 
-#ifdef CGAL_LINKED_WITH_TBB
-#include <CGAL/Point_set_processing_3/internal/Parallel_callback.h>
-#include <tbb/parallel_for.h>
-#include <tbb/blocked_range.h>
-#include <tbb/scalable_allocator.h>
-#endif // CGAL_LINKED_WITH_TBB
-
 namespace CGAL {
 
 
@@ -58,18 +51,16 @@ namespace internal {
 /// @tparam Tree KD-tree.
 ///
 /// @return Computed normal. Orientation is random.
-template < typename Kernel,
-           typename SvdTraits,
-           typename Tree
->
-typename Kernel::Vector_3
-jet_estimate_normal(const typename Kernel::Point_3& query, ///< point to compute the normal at
-                    Tree& tree, ///< KD-tree
+template <typename SvdTraits, typename NeighborQuery>
+typename NeighborQuery::Kernel::Vector_3
+jet_estimate_normal(const typename NeighborQuery::Point_3& query, ///< point to compute the normal at
+                    const NeighborQuery& neighbor_query, ///< KD-tree
                     unsigned int k, ///< number of neighbors
-                    typename Kernel::FT neighbor_radius,
+                    typename NeighborQuery::FT neighbor_radius,
                     unsigned int degree_fitting)
 {
   // basic geometric types
+  typedef typename NeighborQuery::Kernel Kernel;
   typedef typename Kernel::Point_3  Point;
 
   // types for jet fitting
@@ -79,8 +70,10 @@ jet_estimate_normal(const typename Kerne
   typedef typename Monge_jet_fitting::Monge_form Monge_form;
 
   std::vector<Point> points;
-  CGAL::Point_set_processing_3::internal::neighbor_query
-    (query, tree, k, neighbor_radius, points);
+
+  // query using as fallback minimum requires nb points for jet fitting (d+1)*(d+2)/2
+  neighbor_query.get_points (query, k, neighbor_radius, std::back_inserter(points),
+                             (degree_fitting + 1) * (degree_fitting + 2) / 2);
 
   // performs jet fitting
   Monge_jet_fitting monge_fit;
@@ -92,49 +85,6 @@ jet_estimate_normal(const typename Kerne
   return monge_form.normal_direction();
 }
 
-#ifdef CGAL_LINKED_WITH_TBB
-  template <typename Kernel, typename SvdTraits, typename Tree>
-  class Jet_estimate_normals {
-    typedef typename Kernel::FT FT;
-    typedef typename Kernel::Point_3 Point;
-    typedef typename Kernel::Vector_3 Vector;
-    const Tree& tree;
-    const unsigned int k;
-    const FT neighbor_radius;
-    const unsigned int degree_fitting;
-    const std::vector<Point>& input;
-    std::vector<Vector>& output;
-    cpp11::atomic<std::size_t>& advancement;
-    cpp11::atomic<bool>& interrupted;
-
-  public:
-    Jet_estimate_normals(Tree& tree, unsigned int k, FT neighbor_radius,
-                         std::vector<Point>& points,
-                         unsigned int degree_fitting, std::vector<Vector>& output,
-                         cpp11::atomic<std::size_t>& advancement,
-                         cpp11::atomic<bool>& interrupted)
-      : tree(tree), k (k), neighbor_radius (neighbor_radius)
-      , degree_fitting (degree_fitting), input (points), output (output)
-      , advancement (advancement)
-      , interrupted (interrupted)
-    { }
-
-    void operator()(const tbb::blocked_range<std::size_t>& r) const
-    {
-      for( std::size_t i = r.begin(); i != r.end(); ++i)
-      {
-        if (interrupted)
-          break;
-        output[i] = CGAL::internal::jet_estimate_normal<Kernel,SvdTraits>(input[i], tree, k, neighbor_radius, degree_fitting);
-        ++ advancement;
-      }
-    }
-
-  };
-#endif // CGAL_LINKED_WITH_TBB
-
-
-
 } /* namespace internal */
 /// \endcond
 
@@ -152,39 +102,73 @@ jet_estimate_normal(const typename Kerne
 
    \pre `k >= 2`
 
-   \tparam ConcurrencyTag enables sequential versus parallel algorithm.
-   Possible values are `Sequential_tag`
-   and `Parallel_tag`.
+   \tparam ConcurrencyTag enables sequential versus parallel algorithm. Possible values are `Sequential_tag`,
+                          `Parallel_tag`, and `Parallel_if_available_tag`.
    \tparam PointRange is a model of `Range`. The value type of
    its iterator is the key type of the named parameter `point_map`.
 
    \param points input point range.
    \param k number of neighbors
-   \param np optional sequence of \ref psp_namedparameters "Named Parameters" among the ones listed below.
+   \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
    \cgalNamedParamsBegin
-     \cgalParamBegin{point_map} a model of `ReadablePropertyMap` with value type `geom_traits::Point_3`.
-     If this parameter is omitted, `CGAL::Identity_property_map<geom_traits::Point_3>` is used.\cgalParamEnd
-     \cgalParamBegin{normal_map} a model of `ReadWritePropertyMap` with value type
-     `geom_traits::Vector_3`.\cgalParamEnd
-     \cgalParamBegin{neighbor_radius} spherical neighborhood radius. If
-     provided, the neighborhood of a query point is computed with a fixed spherical
-     radius instead of a fixed number of neighbors. In that case, the parameter
-     `k` is used as a limit on the number of points returned by each spherical
-     query (to avoid overly large number of points in high density areas). If no
-     limit is wanted, use `k=0`.\cgalParamEnd
-     \cgalParamBegin{degree_fitting} degree of jet fitting.\cgalParamEnd
-     \cgalParamBegin{svd_traits} template parameter for the class `Monge_via_jet_fitting`. If
-     \ref thirdpartyEigen "Eigen" 3.2 (or greater) is available and `CGAL_EIGEN3_ENABLED` is defined,
-     then `CGAL::Eigen_svd` is used.\cgalParamEnd
-     \cgalParamBegin{callback} an instance of
-      `std::function<bool(double)>`. It is called regularly when the
-      algorithm is running: the current advancement (between 0. and
-      1.) is passed as parameter. If it returns `true`, then the
-      algorithm continues its execution normally; if it returns
-      `false`, the algorithm is stopped and the remaining normals are
-      left unchanged.\cgalParamEnd
-     \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+     \cgalParamNBegin{point_map}
+       \cgalParamDescription{a property map associating points to the elements of the point set `points`}
+       \cgalParamType{a model of `ReadablePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange` and whose value type is `geom_traits::Point_3`}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{normal_map}
+       \cgalParamDescription{a property map associating normals to the elements of the point set `points`}
+       \cgalParamType{a model of `WritablePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange` and whose value type is `geom_traits::Vector_3`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{neighbor_radius}
+       \cgalParamDescription{the spherical neighborhood radius}
+       \cgalParamType{floating scalar value}
+       \cgalParamDefault{`0` (no limit)}
+       \cgalParamExtra{If provided, the neighborhood of a query point is computed with a fixed spherical
+                       radius instead of a fixed number of neighbors. In that case, the parameter
+                       `k` is used as a limit on the number of points returned by each spherical
+                       query (to avoid overly large number of points in high density areas).}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{degree_fitting}
+       \cgalParamDescription{the degree of fitting}
+       \cgalParamType{unsigned int}
+       \cgalParamDefault{`2`}
+       \cgalParamExtra{see `CGAL::Monge_via_jet_fitting`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{svd_traits}
+       \cgalParamDescription{the linear algebra algorithm used in the class `CGAL::Monge_via_jet_fitting`}
+       \cgalParamType{a class fitting the requirements of `CGAL::Monge_via_jet_fitting`}
+       \cgalParamDefault{If \ref thirdpartyEigen "Eigen" 3.2 (or greater) is available
+                         and `CGAL_EIGEN3_ENABLED` is defined, then `CGAL::Eigen_svd` is used.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{callback}
+       \cgalParamDescription{a mechanism to get feedback on the advancement of the algorithm
+                             while it's running and to interrupt it if needed}
+       \cgalParamType{an instance of `std::function<bool(double)>`.}
+       \cgalParamDefault{unused}
+       \cgalParamExtra{It is called regularly when the
+                       algorithm is running: the current advancement (between 0. and
+                       1.) is passed as parameter. If it returns `true`, then the
+                       algorithm continues its execution normally; if it returns
+                       `false`, the algorithm is stopped and the remaining normals are left unchanged.}
+       \cgalParamExtra{The callback will be copied and therefore needs to be lightweight.}
+       \cgalParamExtra{When `CGAL::Parallel_tag` is used, the `callback` mechanism is called asynchronously
+                       on a separate thread and shouldn't access or modify the variables that are parameters of the algorithm.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{geom_traits}
+       \cgalParamDescription{an instance of a geometric traits class}
+       \cgalParamType{a model of `Kernel`}
+       \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+     \cgalParamNEnd
    \cgalNamedParamsEnd
 */
 template <typename ConcurrencyTag,
@@ -203,7 +187,9 @@ jet_estimate_normals(
   CGAL_TRACE("Calls jet_estimate_normals()\n");
 
   // basic geometric types
-  typedef typename Point_set_processing_3::GetPointMap<PointRange, NamedParameters>::type PointMap;
+  typedef typename PointRange::iterator iterator;
+  typedef typename iterator::value_type value_type;
+  typedef typename CGAL::GetPointMap<PointRange, NamedParameters>::type PointMap;
   typedef typename Point_set_processing_3::GetNormalMap<PointRange, NamedParameters>::type NormalMap;
   typedef typename Point_set_processing_3::GetK<PointRange, NamedParameters>::Kernel Kernel;
   typedef typename Kernel::FT FT;
@@ -216,23 +202,16 @@ jet_estimate_normals(
                               typename GetSvdTraits<NamedParameters>::NoTraits>::value),
                             "Error: no SVD traits");
 
-  PointMap point_map = choose_parameter(get_parameter(np, internal_np::point_map), PointMap());
-  NormalMap normal_map = choose_parameter(get_parameter(np, internal_np::normal_map), NormalMap());
+  PointMap point_map = choose_parameter<PointMap>(get_parameter(np, internal_np::point_map));
+  NormalMap normal_map = choose_parameter<NormalMap>(get_parameter(np, internal_np::normal_map));
   unsigned int degree_fitting = choose_parameter(get_parameter(np, internal_np::degree_fitting), 2);
   FT neighbor_radius = choose_parameter(get_parameter(np, internal_np::neighbor_radius), FT(0));
 
   const std::function<bool(double)>& callback = choose_parameter(get_parameter(np, internal_np::callback),
                                                                std::function<bool(double)>());
 
-  typedef typename Kernel::Point_3 Point;
-
-  // Input points types
-  typedef typename boost::property_traits<NormalMap>::value_type Vector;
-
   // types for K nearest neighbors search structure
-  typedef typename CGAL::Search_traits_3<Kernel> Tree_traits;
-  typedef typename CGAL::Orthogonal_k_neighbor_search<Tree_traits> Neighbor_search;
-  typedef typename Neighbor_search::Tree Tree;
+  typedef Point_set_processing_3::internal::Neighbor_query<Kernel, PointRange&, PointMap> Neighbor_query;
 
   // precondition: at least one element in the container.
   // to fix: should have at least three distinct points
@@ -245,60 +224,32 @@ jet_estimate_normals(
   std::size_t memory = CGAL::Memory_sizer().virtual_size(); CGAL_TRACE("  %ld Mb allocated\n", memory>>20);
   CGAL_TRACE("  Creates KD-tree\n");
 
-  typename PointRange::iterator it;
-
-  // Instanciate a KD-tree search.
-  // Note: We have to convert each input iterator to Point_3.
-  std::vector<Point> kd_tree_points;
-  for(it = points.begin(); it != points.end(); it++)
-    kd_tree_points.push_back(get(point_map, *it));
-  Tree tree(kd_tree_points.begin(), kd_tree_points.end());
+  Neighbor_query neighbor_query (points, point_map);
 
   memory = CGAL::Memory_sizer().virtual_size(); CGAL_TRACE("  %ld Mb allocated\n", memory>>20);
   CGAL_TRACE("  Computes normals\n");
 
-  // iterate over input points, compute and output normal
-  // vectors (already normalized)
-#ifndef CGAL_LINKED_WITH_TBB
-  CGAL_static_assertion_msg (!(boost::is_convertible<ConcurrencyTag, Parallel_tag>::value),
-                             "Parallel_tag is enabled but TBB is unavailable.");
-#else
-   if (boost::is_convertible<ConcurrencyTag,Parallel_tag>::value)
-   {
-     Point_set_processing_3::internal::Parallel_callback
-       parallel_callback (callback, kd_tree_points.size());
-
-     std::vector<Vector> normals (kd_tree_points.size (),
-                                  CGAL::NULL_VECTOR);
-     CGAL::internal::Jet_estimate_normals<Kernel, SvdTraits, Tree>
-       f (tree, k, neighbor_radius,
-          kd_tree_points, degree_fitting, normals,
-          parallel_callback.advancement(),
-          parallel_callback.interrupted());
-     tbb::parallel_for(tbb::blocked_range<size_t>(0, kd_tree_points.size ()), f);
-     std::size_t i = 0;
-     for(it = points.begin(); it != points.end(); ++ it, ++ i)
-       if (normals[i] != CGAL::NULL_VECTOR)
-         put (normal_map, *it, normals[i]);
-
-     parallel_callback.join();
-   }
-   else
-#endif
+  std::size_t nb_points = points.size();
+
+  Point_set_processing_3::internal::Callback_wrapper<ConcurrencyTag>
+    callback_wrapper (callback, nb_points);
+
+  CGAL::for_each<ConcurrencyTag>
+    (points,
+     [&](value_type& vt)
      {
-       std::size_t nb = 0;
-       for(it = points.begin(); it != points.end(); it++, ++ nb)
-         {
-           Vector normal = internal::jet_estimate_normal<Kernel,SvdTraits,Tree>(
-                                                                                get(point_map,*it),
-                                                                                tree, k, neighbor_radius, degree_fitting);
-
-           put(normal_map, *it, normal); // normal_map[it] = normal
-           if (callback && !callback ((nb+1) / double(kd_tree_points.size())))
-             break;
-             }
-     }
+       if (callback_wrapper.interrupted())
+         return false;
+
+       put (normal_map, vt,
+            CGAL::internal::jet_estimate_normal<SvdTraits>
+            (get(point_map, vt), neighbor_query, k, neighbor_radius, degree_fitting));
+       ++ callback_wrapper.advancement();
+
+       return true;
+     });
 
+  callback_wrapper.join();
 
   memory = CGAL::Memory_sizer().virtual_size(); CGAL_TRACE("  %ld Mb allocated\n", memory>>20);
   CGAL_TRACE("End of jet_estimate_normals()\n");
diff -pruN 5.0.3-1/include/CGAL/jet_smooth_point_set.h 5.1-1/include/CGAL/jet_smooth_point_set.h
--- 5.0.3-1/include/CGAL/jet_smooth_point_set.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/jet_smooth_point_set.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_processing_3/include/CGAL/jet_smooth_point_set.h $
-// $Id: jet_smooth_point_set.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/jet_smooth_point_set.h $
+// $Id: jet_smooth_point_set.h 93f1cd9 2020-07-16T09:53:31+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s) : Pierre Alliez, Marc Pouget and Laurent Saboret
@@ -17,9 +17,9 @@
 #include <CGAL/disable_warnings.h>
 
 #include <CGAL/IO/trace.h>
-#include <CGAL/Search_traits_3.h>
-#include <CGAL/Orthogonal_k_neighbor_search.h>
-#include <CGAL/Point_set_processing_3/internal/neighbor_query.h>
+#include <CGAL/Point_set_processing_3/internal/Neighbor_query.h>
+#include <CGAL/Point_set_processing_3/internal/Callback_wrapper.h>
+#include <CGAL/for_each.h>
 #include <CGAL/Monge_via_jet_fitting.h>
 #include <CGAL/property_map.h>
 #include <CGAL/point_set_processing_assertions.h>
@@ -28,16 +28,11 @@
 #include <CGAL/boost/graph/Named_function_parameters.h>
 #include <CGAL/boost/graph/named_params_helper.h>
 
+#include <boost/iterator/zip_iterator.hpp>
+
 #include <iterator>
 #include <list>
 
-#ifdef CGAL_LINKED_WITH_TBB
-#include <CGAL/Point_set_processing_3/internal/Parallel_callback.h>
-#include <tbb/parallel_for.h>
-#include <tbb/blocked_range.h>
-#include <tbb/scalable_allocator.h>
-#endif // CGAL_LINKED_WITH_TBB
-
 namespace CGAL {
 
 
@@ -57,20 +52,20 @@ namespace internal {
 /// @tparam Tree KD-tree.
 ///
 /// @return computed point
-template <typename Kernel,
-          typename SvdTraits,
-          typename Tree
+template <typename SvdTraits,
+          typename NeighborQuery
           >
-typename Kernel::Point_3
+typename NeighborQuery::Kernel::Point_3
 jet_smooth_point(
-  const typename Kernel::Point_3& query, ///< 3D point to project
-  Tree& tree, ///< KD-tree
+  const typename NeighborQuery::Kernel::Point_3& query, ///< 3D point to project
+  NeighborQuery& neighbor_query, ///< KD-tree
   const unsigned int k, ///< number of neighbors.
-  typename Kernel::FT neighbor_radius,
+  typename NeighborQuery::Kernel::FT neighbor_radius,
   const unsigned int degree_fitting,
   const unsigned int degree_monge)
 {
   // basic geometric types
+  typedef typename NeighborQuery::Kernel Kernel;
   typedef typename Kernel::Point_3 Point;
 
   // types for jet fitting
@@ -80,8 +75,10 @@ jet_smooth_point(
   typedef typename Monge_jet_fitting::Monge_form Monge_form;
 
   std::vector<Point> points;
-  CGAL::Point_set_processing_3::internal::neighbor_query
-    (query, tree, k, neighbor_radius, points);
+
+  // query using as fallback minimum requires nb points for jet fitting (d+1)*(d+2)/2
+  neighbor_query.get_points (query, k, neighbor_radius, std::back_inserter(points),
+                             (degree_fitting + 1) * (degree_fitting + 2) / 2);
 
   // performs jet fitting
   Monge_jet_fitting monge_fit;
@@ -92,50 +89,6 @@ jet_smooth_point(
   return monge_form.origin();
 }
 
-#ifdef CGAL_LINKED_WITH_TBB
-  template <typename Kernel, typename SvdTraits, typename Tree>
-  class Jet_smooth_pwns {
-    typedef typename Kernel::Point_3 Point;
-    const Tree& tree;
-    const unsigned int k;
-    const typename Kernel::FT neighbor_radius;
-    unsigned int degree_fitting;
-    unsigned int degree_monge;
-    const std::vector<Point>& input;
-    std::vector<Point>& output;
-    cpp11::atomic<std::size_t>& advancement;
-    cpp11::atomic<bool>& interrupted;
-
-  public:
-    Jet_smooth_pwns (Tree& tree, unsigned int k, typename Kernel::FT neighbor_radius,
-                     std::vector<Point>& points,
-                     unsigned int degree_fitting, unsigned int degree_monge, std::vector<Point>& output,
-                     cpp11::atomic<std::size_t>& advancement,
-                     cpp11::atomic<bool>& interrupted)
-      : tree(tree), k (k), neighbor_radius(neighbor_radius)
-      , degree_fitting (degree_fitting)
-      , degree_monge (degree_monge), input (points), output (output)
-      , advancement (advancement)
-      , interrupted (interrupted)
-    { }
-
-    void operator()(const tbb::blocked_range<std::size_t>& r) const
-    {
-      for( std::size_t i = r.begin(); i != r.end(); ++i)
-      {
-        if (interrupted)
-          break;
-        output[i] = CGAL::internal::jet_smooth_point<Kernel, SvdTraits>(input[i], tree, k,
-                  neighbor_radius,
-                                                                        degree_fitting,
-                                                                        degree_monge);
-        ++ advancement;
-      }
-    }
-
-  };
-#endif // CGAL_LINKED_WITH_TBB
-
 
 } /* namespace internal */
 
@@ -156,38 +109,74 @@ jet_smooth_point(
 
    \pre `k >= 2`
 
-   \tparam ConcurrencyTag enables sequential versus parallel algorithm.
-   Possible values are `Sequential_tag`
-   and `Parallel_tag`.
+   \tparam ConcurrencyTag enables sequential versus parallel algorithm. Possible values are `Sequential_tag`,
+                          `Parallel_tag`, and `Parallel_if_available_tag`.
    \tparam PointRange is a model of `Range`. The value type of
    its iterator is the key type of the named parameter `point_map`.
 
    \param points input point range.
    \param k number of neighbors
-   \param np optional sequence of \ref psp_namedparameters "Named Parameters" among the ones listed below.
+   \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
    \cgalNamedParamsBegin
-     \cgalParamBegin{point_map} a model of `ReadablePropertyMap` with value type `geom_traits::Point_3`.
-     If this parameter is omitted, `CGAL::Identity_property_map<geom_traits::Point_3>` is used.\cgalParamEnd
-     \cgalParamBegin{neighbor_radius} spherical neighborhood radius. If
-     provided, the neighborhood of a query point is computed with a fixed spherical
-     radius instead of a fixed number of neighbors. In that case, the parameter
-     `k` is used as a limit on the number of points returned by each spherical
-     query (to avoid overly large number of points in high density areas). If no
-     limit is wanted, use `k=0`.\cgalParamEnd
-     \cgalParamBegin{degree_fitting} degree of jet fitting.\cgalParamEnd
-     \cgalParamBegin{degree_monge} Monge degree.\cgalParamEnd
-     \cgalParamBegin{svd_traits} template parameter for the class `Monge_via_jet_fitting`. If
-     \ref thirdpartyEigen "Eigen" 3.2 (or greater) is available and `CGAL_EIGEN3_ENABLED` is defined,
-     then `CGAL::Eigen_svd` is used.\cgalParamEnd
-     \cgalParamBegin{callback} an instance of
-      `std::function<bool(double)>`. It is called regularly when the
-      algorithm is running: the current advancement (between 0. and
-      1.) is passed as parameter. If it returns `true`, then the
-      algorithm continues its execution normally; if it returns
-      `false`, the algorithm is stopped and the remaining points are
-      left unchanged.\cgalParamEnd
-     \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+     \cgalParamNBegin{point_map}
+       \cgalParamDescription{a property map associating points to the elements of the point set `points`}
+       \cgalParamType{a model of `ReadablePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange` and whose value type is `geom_traits::Point_3`}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{neighbor_radius}
+       \cgalParamDescription{the spherical neighborhood radius}
+       \cgalParamType{floating scalar value}
+       \cgalParamDefault{`0` (no limit)}
+       \cgalParamExtra{If provided, the neighborhood of a query point is computed with a fixed spherical
+                       radius instead of a fixed number of neighbors. In that case, the parameter
+                       `k` is used as a limit on the number of points returned by each spherical
+                       query (to avoid overly large number of points in high density areas).}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{degree_fitting}
+       \cgalParamDescription{the degree of fitting}
+       \cgalParamType{unsigned int}
+       \cgalParamDefault{`2`}
+       \cgalParamExtra{see `CGAL::Monge_via_jet_fitting`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{degree_monge}
+       \cgalParamDescription{the Monge degree}
+       \cgalParamType{unsigned int}
+       \cgalParamDefault{`2`}
+       \cgalParamExtra{see `CGAL::Monge_via_jet_fitting`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{svd_traits}
+       \cgalParamDescription{the linear algebra algorithm used in the class `CGAL::Monge_via_jet_fitting`}
+       \cgalParamType{a class fitting the requirements of `CGAL::Monge_via_jet_fitting`}
+       \cgalParamDefault{If \ref thirdpartyEigen "Eigen" 3.2 (or greater) is available
+                         and `CGAL_EIGEN3_ENABLED` is defined, then `CGAL::Eigen_svd` is used.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{callback}
+       \cgalParamDescription{a mechanism to get feedback on the advancement of the algorithm
+                             while it's running and to interrupt it if needed}
+       \cgalParamType{an instance of `std::function<bool(double)>`.}
+       \cgalParamDefault{unused}
+       \cgalParamExtra{It is called regularly when the
+                       algorithm is running: the current advancement (between 0. and
+                       1.) is passed as parameter. If it returns `true`, then the
+                       algorithm continues its execution normally; if it returns
+                       `false`, the algorithm is stopped and the remaining points are left unchanged.}
+       \cgalParamExtra{The callback will be copied and therefore needs to be lightweight.}
+       \cgalParamExtra{When `CGAL::Parallel_tag` is used, the `callback` mechanism is called asynchronously
+                       on a separate thread and shouldn't access or modify the variables that are parameters of the algorithm.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{geom_traits}
+       \cgalParamDescription{an instance of a geometric traits class}
+       \cgalParamType{a model of `Kernel`}
+       \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+     \cgalParamNEnd
    \cgalNamedParamsEnd
 
 */
@@ -205,7 +194,8 @@ jet_smooth_point_set(
   using parameters::get_parameter;
 
   // basic geometric types
-  typedef typename Point_set_processing_3::GetPointMap<PointRange, NamedParameters>::type PointMap;
+  typedef typename PointRange::iterator iterator;
+  typedef typename CGAL::GetPointMap<PointRange, NamedParameters>::type PointMap;
   typedef typename Point_set_processing_3::GetK<PointRange, NamedParameters>::Kernel Kernel;
   typedef typename GetSvdTraits<NamedParameters>::type SvdTraits;
 
@@ -213,7 +203,7 @@ jet_smooth_point_set(
                               typename GetSvdTraits<NamedParameters>::NoTraits>::value),
                             "Error: no SVD traits");
 
-  PointMap point_map = choose_parameter(get_parameter(np, internal_np::point_map), PointMap());
+  PointMap point_map = choose_parameter<PointMap>(get_parameter(np, internal_np::point_map));
   typename Kernel::FT neighbor_radius = choose_parameter(get_parameter(np, internal_np::neighbor_radius),
                                                          typename Kernel::FT(0));
   unsigned int degree_fitting = choose_parameter(get_parameter(np, internal_np::degree_fitting), 2);
@@ -221,12 +211,8 @@ jet_smooth_point_set(
   const std::function<bool(double)>& callback = choose_parameter(get_parameter(np, internal_np::callback),
                                                                std::function<bool(double)>());
 
-  typedef typename Kernel::Point_3 Point;
-
   // types for K nearest neighbors search structure
-  typedef typename CGAL::Search_traits_3<Kernel> Tree_traits;
-  typedef typename CGAL::Orthogonal_k_neighbor_search<Tree_traits> Neighbor_search;
-  typedef typename Neighbor_search::Tree Tree;
+  typedef Point_set_processing_3::internal::Neighbor_query<Kernel, PointRange&, PointMap> Neighbor_query;
 
   // precondition: at least one element in the container.
   // to fix: should have at least three distinct points
@@ -236,56 +222,53 @@ jet_smooth_point_set(
   // precondition: at least 2 nearest neighbors
   CGAL_point_set_processing_precondition(k >= 2);
 
-  typename PointRange::iterator it;
-
   // Instanciate a KD-tree search.
-  // Note: We have to convert each input iterator to Point_3.
-  std::vector<Point> kd_tree_points;
-  for(it = points.begin(); it != points.end(); it++)
-    kd_tree_points.push_back(get(point_map, *it));
-  Tree tree(kd_tree_points.begin(), kd_tree_points.end());
+  Neighbor_query neighbor_query (points, point_map);
 
   // Iterates over input points and mutates them.
   // Implementation note: the cast to Point& allows to modify only the point's position.
 
-#ifndef CGAL_LINKED_WITH_TBB
-  CGAL_static_assertion_msg (!(boost::is_convertible<ConcurrencyTag, Parallel_tag>::value),
-                             "Parallel_tag is enabled but TBB is unavailable.");
-#else
-   if (boost::is_convertible<ConcurrencyTag,Parallel_tag>::value)
-   {
-     Point_set_processing_3::internal::Parallel_callback
-       parallel_callback (callback, kd_tree_points.size());
-
-     std::vector<Point> mutated_points (kd_tree_points.size (), CGAL::ORIGIN);
-     CGAL::internal::Jet_smooth_pwns<Kernel, SvdTraits, Tree>
-       f (tree, k, neighbor_radius, kd_tree_points, degree_fitting, degree_monge,
-          mutated_points,
-          parallel_callback.advancement(),
-          parallel_callback.interrupted());
-     tbb::parallel_for(tbb::blocked_range<size_t>(0, kd_tree_points.size ()), f);
-     unsigned int i = 0;
-     for(it = points.begin(); it != points.end(); ++ it, ++ i)
-       if (mutated_points[i] != CGAL::ORIGIN)
-         put(point_map, *it, mutated_points[i]);
-
-     parallel_callback.join();
-
-   }
-   else
-#endif
+  std::size_t nb_points = points.size();
+
+  Point_set_processing_3::internal::Callback_wrapper<ConcurrencyTag>
+    callback_wrapper (callback, nb_points);
+
+  std::vector<typename Kernel::Point_3> smoothed (points.size());
+
+  typedef boost::zip_iterator
+     <boost::tuple<iterator,
+                   typename std::vector<typename Kernel::Point_3>::iterator> > Zip_iterator;
+
+  CGAL::for_each<ConcurrencyTag>
+    (CGAL::make_range (boost::make_zip_iterator (boost::make_tuple (points.begin(), smoothed.begin())),
+                       boost::make_zip_iterator (boost::make_tuple (points.end(), smoothed.end()))),
+     [&](const typename Zip_iterator::reference& t)
+     {
+       if (callback_wrapper.interrupted())
+         return false;
+
+       get<1>(t) = CGAL::internal::jet_smooth_point<SvdTraits>
+         (get (point_map, get<0>(t)), neighbor_query,
+          k,
+          neighbor_radius,
+          degree_fitting,
+          degree_monge);
+       ++ callback_wrapper.advancement();
+
+       return true;
+     });
+
+  callback_wrapper.join();
+
+  // Finally, update points
+  CGAL::for_each<ConcurrencyTag>
+    (CGAL::make_range (boost::make_zip_iterator (boost::make_tuple (points.begin(), smoothed.begin())),
+                       boost::make_zip_iterator (boost::make_tuple (points.end(), smoothed.end()))),
+     [&](const typename Zip_iterator::reference& t)
      {
-       std::size_t nb = 0;
-       for(it = points.begin(); it != points.end(); it++, ++ nb)
-         {
-           const typename boost::property_traits<PointMap>::reference p = get(point_map, *it);
-           put(point_map, *it ,
-               internal::jet_smooth_point<Kernel, SvdTraits>(
-                   p,tree,k,neighbor_radius,degree_fitting,degree_monge) );
-           if (callback && !callback ((nb+1) / double(kd_tree_points.size())))
-             break;
-         }
-     }
+       put (point_map, get<0>(t), get<1>(t));
+       return true;
+     });
 }
 
 
diff -pruN 5.0.3-1/include/CGAL/Join_input_iterator.h 5.1-1/include/CGAL/Join_input_iterator.h
--- 5.0.3-1/include/CGAL/Join_input_iterator.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Join_input_iterator.h	2020-09-08 12:15:32.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Join_input_iterator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Join_input_iterator.h $
 // $Id: Join_input_iterator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kd_tree.h 5.1-1/include/CGAL/Kd_tree.h
--- 5.0.3-1/include/CGAL/Kd_tree.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Kd_tree.h	2020-09-08 12:15:39.000000000 +0000
@@ -3,12 +3,13 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_searching/include/CGAL/Kd_tree.h $
-// $Id: Kd_tree.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_searching/include/CGAL/Kd_tree.h $
+// $Id: Kd_tree.h 74f1cad 2020-04-14T09:32:02+02:00 Simon Giraudot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Hans Tangelder (<hanst@cs.uu.nl>),
-//               : Waqar Khan <wkhan@mpi-inf.mpg.de>
+//               : Waqar Khan <wkhan@mpi-inf.mpg.de>,
+//                 Clement Jamin (clement.jamin.pro@gmail.com)
 
 #ifndef CGAL_KD_TREE_H
 #define CGAL_KD_TREE_H
@@ -26,7 +27,6 @@
 #include <CGAL/Splitters.h>
 #include <CGAL/internal/Get_dimension_tag.h>
 
-#include <deque>
 #include <boost/container/deque.hpp>
 #include <boost/optional.hpp>
 
@@ -34,10 +34,38 @@
 #include <CGAL/mutex.h>
 #endif
 
+/*
+  For building the KD Tree in parallel, TBB is needed. If TBB is
+  linked, the internal structures `deque` will be replaced by
+  `tbb::concurrent_vector`, even if the KD Tree is built in sequential
+  mode (this is to avoid changing the type of the KD Tree when
+  changing the concurrency mode of `build()`).
+
+  Experimentally, using the `tbb::concurrent_vector` in sequential
+  mode does not trigger any loss of performance, so from a user's
+  point of view, it should be transparent.
+
+  However, in case one wants to compile the KD Tree *without using TBB
+  structure even though CGAL is linked with TBB*, the macro
+  `CGAL_DISABLE_TBB_STRUCTURE_IN_KD_TREE` can be defined. In that
+  case, even if TBB is linked, the standard `deque` will be used
+  internally. Note that of course, in that case, parallel build will
+  be disabled.
+ */
+#if defined(CGAL_LINKED_WITH_TBB) && !defined(CGAL_DISABLE_TBB_STRUCTURE_IN_KD_TREE)
+#  include <tbb/parallel_invoke.h>
+#  include <tbb/concurrent_vector.h>
+#  define CGAL_TBB_STRUCTURE_IN_KD_TREE
+#endif
+
 namespace CGAL {
 
 //template <class SearchTraits, class Splitter_=Median_of_rectangle<SearchTraits>, class UseExtendedNode = Tag_true >
-template <class SearchTraits, class Splitter_=Sliding_midpoint<SearchTraits>, class UseExtendedNode = Tag_true >
+template <
+  class SearchTraits,
+  class Splitter_=Sliding_midpoint<SearchTraits>,
+  class UseExtendedNode = Tag_true,
+  class EnablePointsCache = Tag_false>
 class Kd_tree {
 
 public:
@@ -47,11 +75,11 @@ public:
   typedef typename Splitter::Container Point_container;
 
   typedef typename SearchTraits::FT FT;
-  typedef Kd_tree_node<SearchTraits, Splitter, UseExtendedNode > Node;
-  typedef Kd_tree_leaf_node<SearchTraits, Splitter, UseExtendedNode > Leaf_node;
-  typedef Kd_tree_internal_node<SearchTraits, Splitter, UseExtendedNode > Internal_node;
-  typedef Kd_tree<SearchTraits, Splitter> Tree;
-  typedef Kd_tree<SearchTraits, Splitter,UseExtendedNode> Self;
+  typedef Kd_tree_node<SearchTraits, Splitter, UseExtendedNode, EnablePointsCache> Node;
+  typedef Kd_tree_leaf_node<SearchTraits, Splitter, UseExtendedNode, EnablePointsCache> Leaf_node;
+  typedef Kd_tree_internal_node<SearchTraits, Splitter, UseExtendedNode, EnablePointsCache> Internal_node;
+  typedef Kd_tree<SearchTraits, Splitter, UseExtendedNode, EnablePointsCache> Tree;
+  typedef Kd_tree<SearchTraits, Splitter, UseExtendedNode, EnablePointsCache> Self;
 
   typedef Node* Node_handle;
   typedef const Node* Node_const_handle;
@@ -69,15 +97,16 @@ public:
 
   typedef typename internal::Get_dimension_tag<SearchTraits>::Dimension D;
 
+  typedef EnablePointsCache Enable_points_cache;
+
 private:
+
   SearchTraits traits_;
   Splitter split;
 
-
-  // wokaround for https://svn.boost.org/trac/boost/ticket/9332
-#if   (_MSC_VER == 1800) && (BOOST_VERSION == 105500)
-  std::deque<Internal_node> internal_nodes;
-  std::deque<Leaf_node> leaf_nodes;
+#if defined(CGAL_TBB_STRUCTURE_IN_KD_TREE)
+  tbb::concurrent_vector<Internal_node> internal_nodes;
+  tbb::concurrent_vector<Leaf_node> leaf_nodes;
 #else
   boost::container::deque<Internal_node> internal_nodes;
   boost::container::deque<Leaf_node> leaf_nodes;
@@ -88,12 +117,18 @@ private:
   Kd_tree_rectangle<FT,D>* bbox;
   std::vector<Point_d> pts;
 
+  // Store a contiguous copy of the point coordinates
+  // for faster queries (reduce the number of cache misses)
+  std::vector<FT> points_cache;
+
   // Instead of storing the points in arrays in the Kd_tree_node
   // we put all the data in a vector in the Kd_tree.
   // and we only store an iterator range in the Kd_tree_node.
   //
   std::vector<const Point_d*> data;
 
+  // Dimension of the points
+  int dim_;
 
   #ifdef CGAL_HAS_THREADS
   mutable CGAL_MUTEX building_mutex;//mutex used to protect const calls inducing build()
@@ -103,10 +138,9 @@ private:
 
   // protected copy constructor
   Kd_tree(const Tree& tree)
-    : traits_(tree.traits_),built_(tree.built_)
+    : traits_(tree.traits_),built_(tree.built_),dim_(-1)
   {};
 
-
   // Instead of the recursive construction of the tree in the class Kd_tree_node
   // we do this in the tree class. The advantage is that we then can optimize
   // the allocation of the nodes.
@@ -115,50 +149,69 @@ private:
   Node_handle
   create_leaf_node(Point_container& c)
   {
-    Leaf_node node(true , static_cast<unsigned int>(c.size()));
+    Leaf_node node(static_cast<unsigned int>(c.size()));
     std::ptrdiff_t tmp = c.begin() - data.begin();
     node.data = pts.begin() + tmp;
 
-    leaf_nodes.push_back(node);
-    Leaf_node_handle nh = &leaf_nodes.back();
-
-
-    return nh;
+#ifdef CGAL_TBB_STRUCTURE_IN_KD_TREE
+    return &*(leaf_nodes.push_back(node));
+#else
+    leaf_nodes.emplace_back (node);
+    return &(leaf_nodes.back());
+#endif
   }
 
-
   // The internal node
-
-  Node_handle
-  create_internal_node(Point_container& c, const Tag_true&)
+  Node_handle new_internal_node()
   {
-    return create_internal_node_use_extension(c);
-  }
-
-  Node_handle
-  create_internal_node(Point_container& c, const Tag_false&)
-  {
-    return create_internal_node(c);
+#ifdef CGAL_TBB_STRUCTURE_IN_KD_TREE
+    return &*(internal_nodes.push_back(Internal_node()));
+#else
+    internal_nodes.emplace_back ();
+    return &(internal_nodes.back());
+#endif
   }
 
-
-
   // TODO: Similiar to the leaf_init function above, a part of the code should be
   //       moved to a the class Kd_tree_node.
   //       It is not proper yet, but the goal was to see if there is
   //       a potential performance gain through the Compact_container
-  Node_handle
-  create_internal_node_use_extension(Point_container& c)
+  template <typename ConcurrencyTag>
+  void
+  create_internal_node(Node_handle n, Point_container& c, const ConcurrencyTag& tag)
   {
-    Internal_node node(false);
-    internal_nodes.push_back(node);
-    Internal_node_handle nh = &internal_nodes.back();
+    Internal_node_handle nh = static_cast<Internal_node_handle>(n);
+    CGAL_assertion (nh != nullptr);
 
     Separator sep;
     Point_container c_low(c.dimension(),traits_);
     split(sep, c, c_low);
     nh->set_separator(sep);
 
+    handle_extended_node (nh, c, c_low, UseExtendedNode());
+
+    if (try_parallel_internal_node_creation (nh, c, c_low, tag))
+      return;
+
+    if (c_low.size() > split.bucket_size())
+    {
+      nh->lower_ch = new_internal_node();
+      create_internal_node (nh->lower_ch, c_low, tag);
+    }
+    else
+      nh->lower_ch = create_leaf_node(c_low);
+
+    if (c.size() > split.bucket_size())
+    {
+      nh->upper_ch = new_internal_node();
+      create_internal_node (nh->upper_ch, c, tag);
+    }
+    else
+      nh->upper_ch = create_leaf_node(c);
+  }
+
+  void handle_extended_node (Internal_node_handle nh, Point_container& c, Point_container& c_low, const Tag_true&)
+  {
     int cd  = nh->cutting_dimension();
     if(!c_low.empty()){
       nh->lower_low_val = c_low.tight_bounding_box().min_coord(cd);
@@ -179,56 +232,45 @@ private:
 
     CGAL_assertion(nh->cutting_value() >= nh->lower_low_val);
     CGAL_assertion(nh->cutting_value() <= nh->upper_high_val);
+  }
 
-    if (c_low.size() > split.bucket_size()){
-      nh->lower_ch = create_internal_node_use_extension(c_low);
-    }else{
-      nh->lower_ch = create_leaf_node(c_low);
-    }
-    if (c.size() > split.bucket_size()){
-      nh->upper_ch = create_internal_node_use_extension(c);
-    }else{
-      nh->upper_ch = create_leaf_node(c);
-    }
-
-
-
+  inline void handle_extended_node (Internal_node_handle, Point_container&, Point_container&, const Tag_false&) { }
 
-    return nh;
+  inline bool try_parallel_internal_node_creation (Internal_node_handle, Point_container&,
+                                                   Point_container&, const Sequential_tag&)
+  {
+    return false;
   }
 
+#ifdef CGAL_TBB_STRUCTURE_IN_KD_TREE
 
-  // Note also that I duplicated the code to get rid if the if's for
-  // the boolean use_extension which was constant over the construction
-  Node_handle
-  create_internal_node(Point_container& c)
+  inline bool try_parallel_internal_node_creation (Internal_node_handle nh, Point_container& c,
+                                                   Point_container& c_low, const Parallel_tag& tag)
   {
-    Internal_node node(false);
-    internal_nodes.push_back(node);
-    Internal_node_handle nh = &internal_nodes.back();
-    Separator sep;
+    /*
+      The two child branches are computed in parallel if and only if:
 
-    Point_container c_low(c.dimension(),traits_);
-    split(sep, c, c_low);
-    nh->set_separator(sep);
+      * both branches lead to internal nodes (if at least one branch
+        is a leaf, it's useless)
 
-    if (c_low.size() > split.bucket_size()){
-      nh->lower_ch = create_internal_node(c_low);
-    }else{
-      nh->lower_ch = create_leaf_node(c_low);
+      * the current number of points is sufficiently high to be worth
+        the cost of launching new threads. Experimentally, using 10
+        times the bucket size as a limit gives the best timings.
+    */
+    if (c_low.size() > split.bucket_size() && c.size() > split.bucket_size()
+        && (c_low.size() + c.size() > 10 * split.bucket_size()))
+    {
+      nh->lower_ch = new_internal_node();
+      nh->upper_ch = new_internal_node();
+      tbb::parallel_invoke (std::bind (&Self::create_internal_node<Parallel_tag>, this, nh->lower_ch, std::ref(c_low), std::cref(tag)),
+                            std::bind (&Self::create_internal_node<Parallel_tag>, this, nh->upper_ch, std::ref(c), std::cref(tag)));
+      return true;
     }
-    if (c.size() > split.bucket_size()){
-      nh->upper_ch = create_internal_node(c);
-    }else{
-      nh->upper_ch = create_leaf_node(c);
-    }
-
 
-
-    return nh;
+    return false;
   }
 
-
+#endif
 
 public:
 
@@ -248,6 +290,32 @@ public:
     return pts.empty();
   }
 
+  void build()
+  {
+    build<Sequential_tag>();
+  }
+
+  /*
+    Note about parallel `build()`. Several different strategies have
+    been tried, among which:
+
+    * keeping the `deque` and using mutex structures to secure the
+      insertions in them
+    * using free stand-alone pointers generated with `new` instead of
+      pushing elements in a container
+    * using a global `tbb::task_group` to handle the internal node
+      computations
+    * using one `tbb::task_group` per internal node to handle the
+      internal node computations
+
+    Experimentally, the options giving the best timings is the one
+    kept, namely:
+
+    * nodes are stored in `tbb::concurrent_vector` structures
+    * the parallel computations are launched using
+      `tbb::parallel_invoke`
+  */
+  template <typename ConcurrencyTag>
   void
   build()
   {
@@ -258,26 +326,42 @@ public:
     CGAL_assertion(!removed_);
     const Point_d& p = *pts.begin();
     typename SearchTraits::Construct_cartesian_const_iterator_d ccci=traits_.construct_cartesian_const_iterator_d_object();
-    int dim = static_cast<int>(std::distance(ccci(p), ccci(p,0)));
+    dim_ = static_cast<int>(std::distance(ccci(p), ccci(p,0)));
 
     data.reserve(pts.size());
     for(unsigned int i = 0; i < pts.size(); i++){
       data.push_back(&pts[i]);
     }
-    Point_container c(dim, data.begin(), data.end(),traits_);
+
+#ifndef CGAL_TBB_STRUCTURE_IN_KD_TREE
+    CGAL_static_assertion_msg (!(boost::is_convertible<ConcurrencyTag, Parallel_tag>::value),
+                               "Parallel_tag is enabled but TBB is unavailable.");
+#endif
+
+    Point_container c(dim_, data.begin(), data.end(),traits_);
     bbox = new Kd_tree_rectangle<FT,D>(c.bounding_box());
     if (c.size() <= split.bucket_size()){
       tree_root = create_leaf_node(c);
     }else {
-      tree_root = create_internal_node(c, UseExtendedNode());
+       tree_root = new_internal_node();
+       create_internal_node (tree_root, c, ConcurrencyTag());
     }
 
     //Reorder vector for spatial locality
     std::vector<Point_d> ptstmp;
     ptstmp.resize(pts.size());
-    for (std::size_t i = 0; i < pts.size(); ++i){
+    for (std::size_t i = 0; i < pts.size(); ++i)
       ptstmp[i] = *data[i];
+
+    // Cache?
+    if (Enable_points_cache::value)
+    {
+      typename SearchTraits::Construct_cartesian_const_iterator_d construct_it = traits_.construct_cartesian_const_iterator_d_object();
+      points_cache.reserve(dim_ * pts.size());
+      for (std::size_t i = 0; i < pts.size(); ++i)
+        points_cache.insert(points_cache.end(), construct_it(ptstmp[i]), construct_it(ptstmp[i], 0));
     }
+
     for(std::size_t i = 0; i < leaf_nodes.size(); ++i){
       std::ptrdiff_t tmp = leaf_nodes[i].begin() - pts.begin();
       leaf_nodes[i].data = ptstmp.begin() + tmp;
@@ -289,6 +373,12 @@ public:
     built_ = true;
   }
 
+  // Only correct when build() has been called
+  int dim() const
+  {
+    return dim_;
+  }
+
 private:
   //any call to this function is for the moment not threadsafe
   void const_build() const {
@@ -463,7 +553,7 @@ public:
         const_build();
       }
       Kd_tree_rectangle<FT,D> b(*bbox);
-      return tree_root->search(it,q,b);
+      return tree_root->search(it,q,b,begin(),cache_begin(),dim_);
     }
     return it;
   }
@@ -479,7 +569,7 @@ public:
         const_build();
       }
       Kd_tree_rectangle<FT,D> b(*bbox);
-      return tree_root->search_any_point(q,b);
+      return tree_root->search_any_point(q,b,begin(),cache_begin(),dim_);
     }
     return boost::none;
   }
@@ -538,6 +628,13 @@ public:
     return *bbox;
   }
 
+
+  typename std::vector<FT>::const_iterator
+    cache_begin() const
+  {
+    return points_cache.begin();
+  }
+
   const_iterator
   begin() const
   {
diff -pruN 5.0.3-1/include/CGAL/Kd_tree_node.h 5.1-1/include/CGAL/Kd_tree_node.h
--- 5.0.3-1/include/CGAL/Kd_tree_node.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Kd_tree_node.h	2020-09-08 12:15:39.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_searching/include/CGAL/Kd_tree_node.h $
-// $Id: Kd_tree_node.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_searching/include/CGAL/Kd_tree_node.h $
+// $Id: Kd_tree_node.h bd08ba8 2020-04-27T11:26:43+02:00 Simon Giraudot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -19,41 +19,44 @@
 
 #include <CGAL/Splitters.h>
 #include <CGAL/Compact_container.h>
+#include <CGAL/Has_member.h>
+#include <CGAL/internal/Search_helpers.h>
 #include <boost/cstdint.hpp>
 
 namespace CGAL {
 
-  template <class SearchTraits, class Splitter, class UseExtendedNode>
+  CGAL_GENERATE_MEMBER_DETECTOR(contains_point_given_as_coordinates);
+
+  template <class SearchTraits, class Splitter, class UseExtendedNode, class EnablePointsCache>
   class Kd_tree;
 
-  template < class TreeTraits, class Splitter, class UseExtendedNode >
+  template < class TreeTraits, class Splitter, class UseExtendedNode, class EnablePointsCache >
   class Kd_tree_node {
 
-     friend class Kd_tree<TreeTraits,Splitter,UseExtendedNode>;
+    friend class Kd_tree<TreeTraits, Splitter, UseExtendedNode, EnablePointsCache>;
+
+    typedef Kd_tree<TreeTraits, Splitter, UseExtendedNode, EnablePointsCache> Kdt;
 
-    typedef typename Kd_tree<TreeTraits,Splitter,UseExtendedNode>::Node_handle Node_handle;
-    typedef typename Kd_tree<TreeTraits,Splitter,UseExtendedNode>::Node_const_handle Node_const_handle;
-     typedef typename Kd_tree<TreeTraits,Splitter,UseExtendedNode>::Internal_node_handle Internal_node_handle;
-    typedef typename Kd_tree<TreeTraits,Splitter,UseExtendedNode>::Internal_node_const_handle Internal_node_const_handle;
-     typedef typename Kd_tree<TreeTraits,Splitter,UseExtendedNode>::Leaf_node_handle Leaf_node_handle;
-    typedef typename Kd_tree<TreeTraits,Splitter,UseExtendedNode>::Leaf_node_const_handle Leaf_node_const_handle;
+    typedef typename Kdt::Node_handle Node_handle;
+    typedef typename Kdt::Node_const_handle Node_const_handle;
+    typedef typename Kdt::Internal_node_handle Internal_node_handle;
+    typedef typename Kdt::Internal_node_const_handle Internal_node_const_handle;
+    typedef typename Kdt::Leaf_node_handle Leaf_node_handle;
+    typedef typename Kdt::Leaf_node_const_handle Leaf_node_const_handle;
     typedef typename TreeTraits::Point_d Point_d;
 
     typedef typename TreeTraits::FT FT;
-    typedef typename Kd_tree<TreeTraits,Splitter,UseExtendedNode>::Separator Separator;
-    typedef typename Kd_tree<TreeTraits,Splitter,UseExtendedNode>::Point_d_iterator Point_d_iterator;
-    typedef typename Kd_tree<TreeTraits,Splitter,UseExtendedNode>::iterator iterator;
-    typedef typename Kd_tree<TreeTraits,Splitter,UseExtendedNode>::D D;
+    typedef typename Kdt::Separator Separator;
+    typedef typename Kdt::Point_d_iterator Point_d_iterator;
+    typedef typename Kdt::iterator iterator;
+    typedef typename Kdt::D D;
 
     bool leaf;
 
   public :
-    Kd_tree_node(bool leaf_)
-      :leaf(leaf_){}
+    Kd_tree_node(bool leaf) : leaf(leaf) { }
 
-    bool is_leaf() const{
-      return leaf;
-    }
+    bool is_leaf() const { return leaf; }
 
     std::size_t
     num_items() const
@@ -91,8 +94,8 @@ namespace CGAL {
         Internal_node_const_handle node =
           static_cast<Internal_node_const_handle>(this);
         return
-             (std::max)( node->lower()->depth(current_max_depth + 1),
-                         node->upper()->depth(current_max_depth + 1));
+          (std::max)( node->lower()->depth(current_max_depth + 1),
+                      node->upper()->depth(current_max_depth + 1));
       }
     }
 
@@ -106,17 +109,17 @@ namespace CGAL {
     OutputIterator
     tree_items(OutputIterator it) const {
       if (is_leaf()) {
-         Leaf_node_const_handle node =
+        Leaf_node_const_handle node =
           static_cast<Leaf_node_const_handle>(this);
-         if (node->size()>0)
-            for (iterator i=node->begin(); i != node->end(); i++)
-              {*it=*i; ++it;}
-        }
+        if (node->size()>0)
+          for (iterator i=node->begin(); i != node->end(); i++)
+          {*it=*i; ++it;}
+      }
       else {
-         Internal_node_const_handle node =
+        Internal_node_const_handle node =
           static_cast<Internal_node_const_handle>(this);
-          it=node->lower()->tree_items(it);
-          it=node->upper()->tree_items(it);
+        it=node->lower()->tree_items(it);
+        it=node->upper()->tree_items(it);
       }
       return it;
     }
@@ -183,33 +186,37 @@ namespace CGAL {
     template <class OutputIterator, class FuzzyQueryItem>
     OutputIterator
     search(OutputIterator it, const FuzzyQueryItem& q,
-           Kd_tree_rectangle<FT,D>& b) const
+           Kd_tree_rectangle<FT,D>& b,
+           typename Kdt::const_iterator tree_points_begin,
+           typename std::vector<FT>::const_iterator cache_begin,
+           int dim) const
     {
       if (is_leaf()) {
         Leaf_node_const_handle node =
           static_cast<Leaf_node_const_handle>(this);
-        if (node->size()>0)
-          for (iterator i=node->begin(); i != node->end(); i++)
-            if (q.contains(*i))
-              {*it++=*i;}
+        if (node->size() > 0)
+        {
+          typename internal::Has_points_cache<Kdt, internal::has_Enable_points_cache<Kdt>::type::value>::type dummy;
+          it = search_in_leaf(node, q, tree_points_begin, cache_begin, dim, it, dummy);
+        }
       }
       else {
          Internal_node_const_handle node =
           static_cast<Internal_node_const_handle>(this);
-        // after splitting b denotes the lower part of b
-        Kd_tree_rectangle<FT,D> b_upper(b);
-        node->split_bbox(b, b_upper);
-
-        if (q.outer_range_contains(b))
-          it=node->lower()->tree_items(it);
-        else
-          if (q.inner_range_intersects(b))
-            it=node->lower()->search(it,q,b);
-        if  (q.outer_range_contains(b_upper))
-          it=node->upper()->tree_items(it);
-        else
-          if (q.inner_range_intersects(b_upper))
-            it=node->upper()->search(it,q,b_upper);
+         // after splitting b denotes the lower part of b
+         Kd_tree_rectangle<FT,D> b_upper(b);
+         node->split_bbox(b, b_upper);
+
+         if (q.outer_range_contains(b))
+           it=node->lower()->tree_items(it);
+         else
+           if (q.inner_range_intersects(b))
+             it=node->lower()->search(it,q,b,tree_points_begin,cache_begin,dim);
+         if  (q.outer_range_contains(b_upper))
+           it=node->upper()->tree_items(it);
+         else
+           if (q.inner_range_intersects(b_upper))
+             it=node->upper()->search(it,q,b_upper,tree_points_begin,cache_begin,dim);
       };
       return it;
     }
@@ -218,16 +225,20 @@ namespace CGAL {
     template <class FuzzyQueryItem>
     boost::optional<Point_d>
     search_any_point(const FuzzyQueryItem& q,
-                     Kd_tree_rectangle<FT,D>& b) const
+                     Kd_tree_rectangle<FT,D>& b,
+                     typename Kdt::const_iterator tree_points_begin,
+                     typename std::vector<FT>::const_iterator cache_begin,
+                     int dim) const
     {
       boost::optional<Point_d> result = boost::none;
       if (is_leaf()) {
         Leaf_node_const_handle node =
           static_cast<Leaf_node_const_handle>(this);
         if (node->size()>0)
-          for (iterator i=node->begin(); i != node->end(); i++)
-            if (q.contains(*i))
-              { result = *i; break; }
+        {
+          typename internal::Has_points_cache<Kdt, internal::has_Enable_points_cache<Kdt>::type::value>::type dummy;
+          result = search_any_point_in_leaf(node, q, tree_points_begin, cache_begin, dim, dummy);
+        }
       }
       else {
          Internal_node_const_handle node =
@@ -237,26 +248,138 @@ namespace CGAL {
         node->split_bbox(b, b_upper);
 
         if (q.inner_range_intersects(b)) {
-          result = node->lower()->search_any_point(q,b);
+          result = node->lower()->search_any_point(q,b,tree_points_begin,cache_begin,dim);
           if(result)
             return result;
         }
         if (q.inner_range_intersects(b_upper))
-          result = node->upper()->search_any_point(q,b_upper);
+          result = node->upper()->search_any_point(q,b_upper,tree_points_begin,cache_begin,dim);
+      }
+      return result;
+    }
+
+  private:
+
+    // If contains_point_given_as_coordinates does not exist in `FuzzyQueryItem`
+    template <typename FuzzyQueryItem>
+    bool contains(
+      const FuzzyQueryItem& q,
+      Point_d const& p,
+      typename std::vector<FT>::const_iterator /*it_coord_begin*/,
+      typename std::vector<FT>::const_iterator /*it_coord_end*/,
+      Tag_false /*has_contains_point_given_as_coordinates*/) const
+    {
+      return q.contains(p);
+    }
+    // ... or if it exists
+    template <typename FuzzyQueryItem>
+    bool contains(
+      const FuzzyQueryItem& q,
+      Point_d const& /*p*/,
+      typename std::vector<FT>::const_iterator it_coord_begin,
+      typename std::vector<FT>::const_iterator it_coord_end,
+      Tag_true /*has_contains_point_given_as_coordinates*/) const
+    {
+      return q.contains_point_given_as_coordinates(it_coord_begin, it_coord_end);
+    }
+
+    // With cache
+    template<class FuzzyQueryItem, class OutputIterator>
+    OutputIterator search_in_leaf(
+      Leaf_node_const_handle node,
+      const FuzzyQueryItem &q,
+      typename Kdt::const_iterator tree_points_begin,
+      typename std::vector<FT>::const_iterator cache_begin,
+      int dim,
+      OutputIterator oit,
+      Tag_true /*has_points_cache*/) const
+    {
+      typename Kdt::iterator it_node_point = node->begin(), it_node_point_end = node->end();
+      typename std::vector<FT>::const_iterator cache_point_it = cache_begin + dim*(it_node_point - tree_points_begin);
+      for (; it_node_point != it_node_point_end; ++it_node_point, cache_point_it += dim)
+      {
+        Boolean_tag<has_contains_point_given_as_coordinates<FuzzyQueryItem>::value> dummy;
+        if (contains(q, *it_node_point, cache_point_it, cache_point_it + dim, dummy))
+          *oit++ = *it_node_point;
+      }
+      return oit;
+    }
+
+    // Without cache
+    template<class FuzzyQueryItem, class OutputIterator>
+    OutputIterator search_in_leaf(
+      Leaf_node_const_handle node,
+      const FuzzyQueryItem &q,
+      typename Kdt::const_iterator /*tree_points_begin*/,
+      typename std::vector<FT>::const_iterator /*cache_begin*/,
+      int /*dim*/,
+      OutputIterator oit,
+      Tag_false /*has_points_cache*/) const
+    {
+      for (iterator i = node->begin(); i != node->end(); ++i)
+      {
+        if (q.contains(*i))
+          *oit++ = *i;
+      }
+      return oit;
+    }
+
+    // With cache
+    template<class FuzzyQueryItem>
+    boost::optional<Point_d> search_any_point_in_leaf(
+      Leaf_node_const_handle node,
+      const FuzzyQueryItem &q,
+      typename Kdt::const_iterator tree_points_begin,
+      typename std::vector<FT>::const_iterator cache_begin,
+      int dim,
+      Tag_true /*has_points_cache*/) const
+    {
+      boost::optional<Point_d> result = boost::none;
+      typename Kdt::iterator it_node_point = node->begin(), it_node_point_end = node->end();
+      typename std::vector<FT>::const_iterator cache_point_it = cache_begin + dim*(it_node_point - tree_points_begin);
+      for (; it_node_point != it_node_point_end; ++it_node_point, cache_point_it += dim)
+      {
+        Boolean_tag<has_contains_point_given_as_coordinates<FuzzyQueryItem>::value> dummy;
+        if (contains(q, *it_node_point, cache_point_it, cache_point_it + dim, dummy))
+        {
+          result = *it_node_point;
+          break;
+        }
       }
       return result;
     }
 
+    // Without cache
+    template<class FuzzyQueryItem>
+    boost::optional<Point_d> search_any_point_in_leaf(
+      Leaf_node_const_handle node,
+      const FuzzyQueryItem &q,
+      typename Kdt::const_iterator /*tree_points_begin*/,
+      typename std::vector<FT>::const_iterator /*cache_begin*/,
+      int /*dim*/,
+      Tag_false /*has_points_cache*/) const
+    {
+      boost::optional<Point_d> result = boost::none;
+      for (iterator i = node->begin(); i != node->end(); ++i)
+      {
+        if (q.contains(*i))
+        {
+          result = *i;
+          break;
+        }
+      }
+      return result;
+    }
   };
 
 
-  template < class TreeTraits, class Splitter, class UseExtendedNode >
-  class Kd_tree_leaf_node : public Kd_tree_node< TreeTraits, Splitter, UseExtendedNode >{
+  template < class TreeTraits, class Splitter, class UseExtendedNode, class EnablePointsCache >
+  class Kd_tree_leaf_node : public Kd_tree_node< TreeTraits, Splitter, UseExtendedNode, EnablePointsCache >{
 
-    friend class Kd_tree<TreeTraits,Splitter,UseExtendedNode>;
+    friend class Kd_tree<TreeTraits, Splitter, UseExtendedNode, EnablePointsCache>;
 
-    typedef typename Kd_tree<TreeTraits,Splitter,UseExtendedNode>::iterator iterator;
-    typedef Kd_tree_node< TreeTraits, Splitter, UseExtendedNode> Base;
+    typedef typename Kd_tree<TreeTraits, Splitter, UseExtendedNode, EnablePointsCache>::iterator iterator;
+    typedef Kd_tree_node< TreeTraits, Splitter, UseExtendedNode, EnablePointsCache> Base;
     typedef typename TreeTraits::Point_d Point_d;
 
   private:
@@ -270,14 +393,11 @@ namespace CGAL {
 
     // default constructor
     Kd_tree_leaf_node()
+      : Base(true)
     {}
 
-    Kd_tree_leaf_node(bool leaf_ )
-      : Base(leaf_)
-    {}
-
-    Kd_tree_leaf_node(bool leaf_,unsigned int n_ )
-      : Base(leaf_), n(n_)
+    Kd_tree_leaf_node(unsigned int n_ )
+      : Base(true), n(n_)
     {}
 
     // members for all nodes
@@ -315,18 +435,20 @@ namespace CGAL {
 
 
 
-  template < class TreeTraits, class Splitter, class UseExtendedNode>
-  class Kd_tree_internal_node : public Kd_tree_node< TreeTraits, Splitter, UseExtendedNode >{
+  template < class TreeTraits, class Splitter, class UseExtendedNode, class EnablePointsCache>
+  class Kd_tree_internal_node : public Kd_tree_node< TreeTraits, Splitter, UseExtendedNode, EnablePointsCache >{
+
+    friend class Kd_tree<TreeTraits, Splitter, UseExtendedNode, EnablePointsCache>;
 
-    friend class Kd_tree<TreeTraits,Splitter,UseExtendedNode>;
+    typedef Kd_tree<TreeTraits, Splitter, UseExtendedNode, EnablePointsCache> Kdt;
 
-    typedef Kd_tree_node< TreeTraits, Splitter, UseExtendedNode> Base;
-    typedef typename Kd_tree<TreeTraits,Splitter,UseExtendedNode>::Node_handle Node_handle;
-    typedef typename Kd_tree<TreeTraits,Splitter,UseExtendedNode>::Node_const_handle Node_const_handle;
+    typedef Kd_tree_node< TreeTraits, Splitter, UseExtendedNode, EnablePointsCache> Base;
+    typedef typename Kdt::Node_handle Node_handle;
+    typedef typename Kdt::Node_const_handle Node_const_handle;
 
     typedef typename TreeTraits::FT FT;
-    typedef typename Kd_tree<TreeTraits,Splitter,UseExtendedNode>::Separator Separator;
-    typedef typename Kd_tree<TreeTraits,Splitter,UseExtendedNode>::D D;
+    typedef typename Kdt::Separator Separator;
+    typedef typename Kdt::D D;
 
   private:
 
@@ -347,13 +469,12 @@ namespace CGAL {
 
     // default constructor
     Kd_tree_internal_node()
+      : Base(false), cut_dim(-1), cut_val(0)
+      , lower_ch (nullptr), upper_ch (nullptr)
+      , upper_low_val(0), upper_high_val(0)
+      , lower_low_val(0), lower_high_val(0)
     {}
 
-    Kd_tree_internal_node(bool leaf_)
-      : Base(leaf_)
-    {}
-
-
     // members for internal node and extended internal node
 
     inline
@@ -463,18 +584,21 @@ namespace CGAL {
     }
   };//internal node
 
- template < class TreeTraits, class Splitter>
- class Kd_tree_internal_node<TreeTraits,Splitter,Tag_false> : public Kd_tree_node< TreeTraits, Splitter, Tag_false >{
-
-    friend class Kd_tree<TreeTraits,Splitter,Tag_false>;
-
-    typedef Kd_tree_node< TreeTraits, Splitter, Tag_false> Base;
-    typedef typename Kd_tree<TreeTraits,Splitter,Tag_false>::Node_handle Node_handle;
-    typedef typename Kd_tree<TreeTraits,Splitter,Tag_false>::Node_const_handle Node_const_handle;
+ template < class TreeTraits, class Splitter, class EnablePointsCache>
+ class Kd_tree_internal_node<TreeTraits,Splitter,Tag_false,EnablePointsCache>
+   : public Kd_tree_node< TreeTraits, Splitter, Tag_false, EnablePointsCache >
+ {
+    friend class Kd_tree<TreeTraits, Splitter, Tag_false, EnablePointsCache>;
+
+    typedef Kd_tree<TreeTraits, Splitter, Tag_false, EnablePointsCache> Kdt;
+
+    typedef Kd_tree_node< TreeTraits, Splitter, Tag_false, EnablePointsCache> Base;
+    typedef typename Kdt::Node_handle Node_handle;
+    typedef typename Kdt::Node_const_handle Node_const_handle;
 
     typedef typename TreeTraits::FT FT;
-    typedef typename Kd_tree<TreeTraits,Splitter,Tag_false>::Separator Separator;
-    typedef typename Kd_tree<TreeTraits,Splitter,Tag_false>::D D;
+    typedef typename Kdt::Separator Separator;
+    typedef typename Kdt::D D;
 
   private:
 
@@ -488,13 +612,9 @@ namespace CGAL {
 
     // default constructor
     Kd_tree_internal_node()
+      : Base(false)
     {}
 
-    Kd_tree_internal_node(bool leaf_)
-      : Base(leaf_)
-    {}
-
-
     // members for internal node and extended internal node
 
     inline
diff -pruN 5.0.3-1/include/CGAL/Kd_tree_rectangle.h 5.1-1/include/CGAL/Kd_tree_rectangle.h
--- 5.0.3-1/include/CGAL/Kd_tree_rectangle.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Kd_tree_rectangle.h	2020-09-08 12:15:39.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_searching/include/CGAL/Kd_tree_rectangle.h $
-// $Id: Kd_tree_rectangle.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_searching/include/CGAL/Kd_tree_rectangle.h $
+// $Id: Kd_tree_rectangle.h 2e180ac 2020-03-26T19:29:44+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -105,6 +105,7 @@ namespace CGAL {
     }
 
     Kd_tree_rectangle()
+      : max_span_coord_(-1)
     {}
 
 
@@ -138,6 +139,7 @@ namespace CGAL {
 
     template <class Construct_cartesian_const_iterator_d,class PointPointerIter> // was PointIter
     Kd_tree_rectangle(int,  PointPointerIter begin,  PointPointerIter end,const Construct_cartesian_const_iterator_d& construct_it)
+      : max_span_coord_(-1)
     {
       update_from_point_pointers<Construct_cartesian_const_iterator_d>(begin,end,construct_it);
     }
@@ -288,7 +290,7 @@ namespace CGAL {
     }
 
     Kd_tree_rectangle()
-      : coords_(0), dim(0)
+      : coords_(0), dim(0), max_span_coord_(-1)
     {
 }
 
@@ -323,7 +325,7 @@ namespace CGAL {
 
     template <class Construct_cartesian_const_iterator_d,class PointPointerIter> // was PointIter
     Kd_tree_rectangle(int d,  PointPointerIter begin,  PointPointerIter end,const Construct_cartesian_const_iterator_d& construct_it)
-      : coords_(new FT[2*d]), dim(d)
+      : coords_(new FT[2*d]), dim(d), max_span_coord_(-1)
     {
       update_from_point_pointers<Construct_cartesian_const_iterator_d>(begin,end,construct_it);
     }
diff -pruN 5.0.3-1/include/CGAL/Kernel/Conic_misc.h 5.1-1/include/CGAL/Kernel/Conic_misc.h
--- 5.0.3-1/include/CGAL/Kernel/Conic_misc.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel/Conic_misc.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Kernel/Conic_misc.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Kernel/Conic_misc.h $
 // $Id: Conic_misc.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel/Dimension_utils.h 5.1-1/include/CGAL/Kernel/Dimension_utils.h
--- 5.0.3-1/include/CGAL/Kernel/Dimension_utils.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel/Dimension_utils.h	2020-09-08 12:15:01.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Kernel/Dimension_utils.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Kernel/Dimension_utils.h $
 // $Id: Dimension_utils.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel/function_objects.h 5.1-1/include/CGAL/Kernel/function_objects.h
--- 5.0.3-1/include/CGAL/Kernel/function_objects.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel/function_objects.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Kernel/function_objects.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Kernel/function_objects.h $
 // $Id: function_objects.h 4476663 2020-04-08T16:14:44+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel/global_functions_2.h 5.1-1/include/CGAL/Kernel/global_functions_2.h
--- 5.0.3-1/include/CGAL/Kernel/global_functions_2.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel/global_functions_2.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Kernel/global_functions_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Kernel/global_functions_2.h $
 // $Id: global_functions_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel/global_functions_3.h 5.1-1/include/CGAL/Kernel/global_functions_3.h
--- 5.0.3-1/include/CGAL/Kernel/global_functions_3.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel/global_functions_3.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Kernel/global_functions_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Kernel/global_functions_3.h $
 // $Id: global_functions_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel/global_functions.h 5.1-1/include/CGAL/Kernel/global_functions.h
--- 5.0.3-1/include/CGAL/Kernel/global_functions.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel/global_functions.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Kernel/global_functions.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Kernel/global_functions.h $
 // $Id: global_functions.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel/global_functions_internal_2.h 5.1-1/include/CGAL/Kernel/global_functions_internal_2.h
--- 5.0.3-1/include/CGAL/Kernel/global_functions_internal_2.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel/global_functions_internal_2.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Kernel/global_functions_internal_2.h $
-// $Id: global_functions_internal_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Kernel/global_functions_internal_2.h $
+// $Id: global_functions_internal_2.h a943082 2020-03-26T19:01:31+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -341,9 +341,7 @@ compare_signed_distance_to_line(const ty
                                 const typename K::Point_2& s,
                                 const K& k)
 {
-  if (k.less_signed_distance_to_line_2_object()(p, q, r, s)) return SMALLER;
-  if (k.less_signed_distance_to_line_2_object()(p, q, s, r)) return LARGER;
-  return EQUAL;
+  return k.compare_signed_distance_to_line_2_object()(p, q, r, s);
 }
 
 template <class K>
@@ -354,9 +352,7 @@ compare_signed_distance_to_line(const ty
                                 const typename K::Point_2& q,
                                 const K& k)
 {
-  if (k.less_signed_distance_to_line_2_object()(l, p, q)) return SMALLER;
-  if (k.less_signed_distance_to_line_2_object()(l, q, p)) return LARGER;
-  return EQUAL;
+  return k.compare_signed_distance_to_line_2_object()(l, p, q);
 }
 
 template < class K >
diff -pruN 5.0.3-1/include/CGAL/Kernel/global_functions_internal_3.h 5.1-1/include/CGAL/Kernel/global_functions_internal_3.h
--- 5.0.3-1/include/CGAL/Kernel/global_functions_internal_3.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel/global_functions_internal_3.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Kernel/global_functions_internal_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Kernel/global_functions_internal_3.h $
 // $Id: global_functions_internal_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel/hash_functions.h 5.1-1/include/CGAL/Kernel/hash_functions.h
--- 5.0.3-1/include/CGAL/Kernel/hash_functions.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel/hash_functions.h	2020-09-08 12:15:01.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Kernel/hash_functions.h $
-// $Id: hash_functions.h 6a51a30 2020-04-22T11:02:16+02:00 Maxime Gimeno
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Kernel/hash_functions.h $
+// $Id: hash_functions.h 9bf61b7 2020-04-22T11:02:16+02:00 Maxime Gimeno
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel/interface_macros.h 5.1-1/include/CGAL/Kernel/interface_macros.h
--- 5.0.3-1/include/CGAL/Kernel/interface_macros.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel/interface_macros.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Kernel/interface_macros.h $
-// $Id: interface_macros.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Kernel/interface_macros.h $
+// $Id: interface_macros.h a943082 2020-03-26T19:01:31+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -116,6 +116,8 @@ CGAL_Kernel_pred_RT(Compare_power_distan
                     compare_power_distance_2_object)
 CGAL_Kernel_pred_RT(Compare_power_distance_3,
                     compare_power_distance_3_object)
+CGAL_Kernel_pred(Compare_signed_distance_to_line_2,
+                 compare_signed_distance_to_line_2_object)
 CGAL_Kernel_pred(Compare_slope_2,
                  compare_slope_2_object)
 CGAL_Kernel_pred(Compare_slope_3,
@@ -488,7 +490,7 @@ CGAL_Kernel_pred_RT(Coplanar_orientation
                     coplanar_orientation_3_object)
 CGAL_Kernel_pred_RT(Coplanar_side_of_bounded_circle_3,
                     coplanar_side_of_bounded_circle_3_object)
-CGAL_Kernel_pred(Coplanar_3,
+CGAL_Kernel_pred_RT(Coplanar_3,
                  coplanar_3_object)
 CGAL_Kernel_pred(Counterclockwise_in_between_2,
                  counterclockwise_in_between_2_object)
diff -pruN 5.0.3-1/include/CGAL/Kernel/mpl.h 5.1-1/include/CGAL/Kernel/mpl.h
--- 5.0.3-1/include/CGAL/Kernel/mpl.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel/mpl.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Kernel/mpl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Kernel/mpl.h $
 // $Id: mpl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel/Return_base_tag.h 5.1-1/include/CGAL/Kernel/Return_base_tag.h
--- 5.0.3-1/include/CGAL/Kernel/Return_base_tag.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel/Return_base_tag.h	2020-09-08 12:15:01.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Kernel/Return_base_tag.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Kernel/Return_base_tag.h $
 // $Id: Return_base_tag.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel/Same_uncertainty.h 5.1-1/include/CGAL/Kernel/Same_uncertainty.h
--- 5.0.3-1/include/CGAL/Kernel/Same_uncertainty.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel/Same_uncertainty.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Kernel/Same_uncertainty.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Kernel/Same_uncertainty.h $
 // $Id: Same_uncertainty.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel/solve.h 5.1-1/include/CGAL/Kernel/solve.h
--- 5.0.3-1/include/CGAL/Kernel/solve.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel/solve.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Kernel/solve.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Kernel/solve.h $
 // $Id: solve.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel/Type_equality_wrapper.h 5.1-1/include/CGAL/Kernel/Type_equality_wrapper.h
--- 5.0.3-1/include/CGAL/Kernel/Type_equality_wrapper.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel/Type_equality_wrapper.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Kernel/Type_equality_wrapper.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Kernel/Type_equality_wrapper.h $
 // $Id: Type_equality_wrapper.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel/Type_mapper.h 5.1-1/include/CGAL/Kernel/Type_mapper.h
--- 5.0.3-1/include/CGAL/Kernel/Type_mapper.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel/Type_mapper.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Kernel/Type_mapper.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Kernel/Type_mapper.h $
 // $Id: Type_mapper.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel/Wutils.h 5.1-1/include/CGAL/Kernel/Wutils.h
--- 5.0.3-1/include/CGAL/Kernel/Wutils.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel/Wutils.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Kernel/Wutils.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Kernel/Wutils.h $
 // $Id: Wutils.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/kernel_assertions.h 5.1-1/include/CGAL/kernel_assertions.h
--- 5.0.3-1/include/CGAL/kernel_assertions.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/kernel_assertions.h	2020-09-08 12:15:02.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/kernel_assertions.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/kernel_assertions.h $
 // $Id: kernel_assertions.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/kernel_basic.h 5.1-1/include/CGAL/kernel_basic.h
--- 5.0.3-1/include/CGAL/kernel_basic.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/kernel_basic.h	2020-09-08 12:15:02.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/kernel_basic.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/kernel_basic.h $
 // $Id: kernel_basic.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_checker.h 5.1-1/include/CGAL/Kernel_checker.h
--- 5.0.3-1/include/CGAL/Kernel_checker.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_checker.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Kernel_checker.h $
-// $Id: Kernel_checker.h 2e8a59d 2020-07-21T15:25:54+02:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Kernel_checker.h $
+// $Id: Kernel_checker.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -79,205 +79,21 @@ public:
     : p1(pp1), p2(pp2), cmp(c)
   { }
 
-  template <typename F>
-  struct result;
-
-  template <typename F, typename A1>
-  struct result<F(A1)> {
-    typedef typename Pairify<typename CGAL::cpp11::result_of<P1(const A1&)>::type,
-                             typename CGAL::cpp11::result_of<P2(const A1&)>::type>::result_type type;
-  };
-
-  template <class A1>
-  typename Pairify<typename CGAL::cpp11::result_of<P1(const A1&)>::type,
-                   typename CGAL::cpp11::result_of<P2(const A1&)>::type>::result_type
-  operator()(const A1 &a1) const
-  {
-    typedef typename CGAL::cpp11::result_of<P1(const A1&)>::type result_type_1;
-    typedef typename CGAL::cpp11::result_of<P2(const A1&)>::type result_type_2;
-    result_type_1 res1 = p1(a1.first);
-    result_type_2 res2 = p2(a1.second);
-    if (! cmp(res1, res2))
-    {
-      std::cerr << "Kernel_checker error : " << res1 << " != " << res2
-                << " for the inputs : " << std::endl;
-      std::cerr << a1.first << std::endl;
-      std::cerr << a1.second << std::endl;
-      std::cerr << "functor first kernel : "
-                << typeid(p1).name() << std::endl;
-      std::cerr << "functor second kernel: "
-                << typeid(p2).name() << std::endl;
-      std::cerr << CGAL_PRETTY_FUNCTION << std::endl;
-      CGAL_kernel_assertion(false);
-    }
-    return Pairify<result_type_1, result_type_2>()(res1, res2);
-  }
-
-  template <typename F, typename A1, typename A2>
-  struct result<F(A1, A2)> {
-    typedef typename Pairify<typename CGAL::cpp11::result_of<P1(const A1&, const A2&)>::type,
-                             typename CGAL::cpp11::result_of<P2(const A1&, const A2&)>::type>::result_type type;
-  };
-
-  template <class A1, class A2>
-  typename Pairify<typename CGAL::cpp11::result_of<P1(const A1&, const A2&)>::type,
-                   typename CGAL::cpp11::result_of<P2(const A1&, const A2&)>::type>::result_type
-  operator()(const A1 &a1, const A2 &a2) const
-  {
-    typedef typename CGAL::cpp11::result_of<P1(const A1&, const A2&)>::type result_type_1;
-    typedef typename CGAL::cpp11::result_of<P2(const A1&, const A2&)>::type result_type_2;
-    result_type_1 res1 = p1(a1.first, a2.first);
-    result_type_2 res2 = p2(a1.second, a2.second);
-    if (! cmp(res1, res2))
-    {
-      std::cerr << "Kernel_checker error : " << res1 << " != " << res2
-                << " for the inputs : " << std::endl;
-      std::cerr << a1.first << std::endl;
-      std::cerr << a1.second << std::endl;
-      std::cerr << a2.first << std::endl;
-      std::cerr << a2.second << std::endl;
-      std::cerr << "functor first kernel : "
-                << typeid(p1).name() << std::endl;
-      std::cerr << "functor second kernel: "
-                << typeid(p2).name() << std::endl;
-      std::cerr << CGAL_PRETTY_FUNCTION << std::endl;
-      CGAL_kernel_assertion(false);
-    }
-    return Pairify<result_type_1, result_type_2>()(res1, res2);
-  }
-
-  template <typename F, typename A1, typename A2, typename A3>
-  struct result<F(A1, A2, A3)> {
-    typedef typename Pairify<typename CGAL::cpp11::result_of<P1(const A1&, const A2&, const A3&)>::type,
-                             typename CGAL::cpp11::result_of<P2(const A1&, const A2&, const A3&)>::type>::result_type type;
-  };
-
-  template <class A1, class A2, class A3>
-  typename Pairify<typename CGAL::cpp11::result_of<P1(
-                     const A1&, const A2&, const A3&)>::type,
-                   typename CGAL::cpp11::result_of<P2(
-                     const A1&, const A2&, const A3&)>::type>::result_type
-  operator()(const A1 &a1, const A2 &a2, const A3 &a3) const
-  {
-    typedef typename CGAL::cpp11::result_of<P1(const A1&, const A2&, const A3&)>::type result_type_1;
-    typedef typename CGAL::cpp11::result_of<P2(const A1&, const A2&, const A3&)>::type result_type_2;
-    result_type_1 res1 = p1(a1.first, a2.first, a3.first);
-    result_type_2 res2 = p2(a1.second, a2.second, a3.second);
-    if (! cmp(res1, res2))
-    {
-      std::cerr << "Kernel_checker error : " << res1 << " != " << res2
-                << " for the inputs : " << std::endl;
-      std::cerr << a1.first << std::endl;
-      std::cerr << a1.second << std::endl;
-      std::cerr << a2.first << std::endl;
-      std::cerr << a2.second << std::endl;
-      std::cerr << a3.first << std::endl;
-      std::cerr << a3.second << std::endl;
-      std::cerr << "functor first kernel : "
-                << typeid(p1).name() << std::endl;
-      std::cerr << "functor second kernel: "
-                << typeid(p2).name() << std::endl;
-      std::cerr << CGAL_PRETTY_FUNCTION << std::endl;
-      CGAL_kernel_assertion(false);
-    }
-    return Pairify<result_type_1, result_type_2>()(res1, res2);
-  }
-
-  template <typename F, typename A1, typename A2, typename A3, typename A4>
-  struct result<F(A1, A2, A3, A4)> {
-    typedef
-      typename Pairify<typename CGAL::cpp11::result_of<P1(
-                         const A1&, const A2&, const A3&, const A4&)>::type,
-                       typename CGAL::cpp11::result_of<P2(
-                         const A1&, const A2&, const A3&, const A4&)>::type>::result_type
-    type;
-  };
-
-  template <class A1, class A2, class A3, class A4>
-  typename Pairify<typename CGAL::cpp11::result_of<P1(
-                     const A1&, const A2&, const A3&, const A4&)>::type,
-                   typename CGAL::cpp11::result_of<P2(
-                     const A1&, const A2&, const A3&, const A4&)>::type>::result_type
-  operator()(const A1 &a1, const A2 &a2, const A3 &a3, const A4 &a4) const
-  {
-    typedef typename CGAL::cpp11::result_of<P1(
-                       const A1&, const A2&, const A3&, const A4&)>::type result_type_1;
-    typedef typename CGAL::cpp11::result_of<P2(
-                       const A1&, const A2&, const A3&, const A4&)>::type result_type_2;
-
-    result_type_1 res1 = p1(a1.first, a2.first, a3.first, a4.first);
-    result_type_2 res2 = p2(a1.second, a2.second, a3.second, a4.second);
-    if (! cmp(res1, res2))
-    {
-      std::cerr << "Kernel_checker error : " << res1 << " != " << res2
-                << " for the inputs : " << std::endl;
-      std::cerr << a1.first << std::endl;
-      std::cerr << a1.second << std::endl;
-      std::cerr << a2.first << std::endl;
-      std::cerr << a2.second << std::endl;
-      std::cerr << a3.first << std::endl;
-      std::cerr << a3.second << std::endl;
-      std::cerr << a4.first << std::endl;
-      std::cerr << a4.second << std::endl;
-      std::cerr << "functor first kernel : "
-                << typeid(p1).name() << std::endl;
-      std::cerr << "functor second kernel: "
-                << typeid(p2).name() << std::endl;
-      std::cerr << CGAL_PRETTY_FUNCTION << std::endl;
-      CGAL_kernel_assertion(false);
-    }
-    return Pairify<result_type_1, result_type_2>()(res1, res2);
-  }
-
-  template <typename F, typename A1, typename A2, typename A3, typename A4, typename A5>
-  struct result<F(A1, A2, A3, A4, A5)> {
-    typedef
-      typename Pairify<typename CGAL::cpp11::result_of<P1(
-                         const A1&, const A2&, const A3&, const A4&, const A5&)>::type,
-                       typename CGAL::cpp11::result_of<P2(
-                         const A1&, const A2&, const A3&, const A4&, const A5&)>::type>::result_type
-    type;
-  };
-
-  template <class A1, class A2, class A3, class A4, class A5>
-  typename Pairify<typename CGAL::cpp11::result_of<P1(
-                     const A1&, const A2&, const A3&, const A4&, const A5&)>::type,
-                   typename CGAL::cpp11::result_of<P2(
-                     const A1&, const A2&, const A3&, const A4&, const A5&)>::type>::result_type
-  operator()(const A1 &a1, const A2 &a2, const A3 &a3, const A4 &a4, const A5 &a5) const
+  template <class ... A>
+  typename Pairify<typename CGAL::cpp11::result_of<P1(const A&...)>::type,
+                   typename CGAL::cpp11::result_of<P2(const A&...)>::type>::result_type
+  operator()(const A&... a) const
   {
-    typedef typename CGAL::cpp11::result_of<P1(
-      const A1&, const A2&, const A3&, const A4&, const A5&)>::type result_type_1;
-    typedef typename CGAL::cpp11::result_of<P2(
-      const A1&, const A2&, const A3&, const A4&, const A5&)>::type result_type_2;
-
-    result_type_1 res1 = p1(a1.first, a2.first, a3.first, a4.first, a5.first);
-    result_type_2 res2 = p2(a1.second, a2.second, a3.second, a4.second, a5.second);
+    typedef typename CGAL::cpp11::result_of<P1(const A&...)>::type result_type_1;
+    typedef typename CGAL::cpp11::result_of<P2(const A&...)>::type result_type_2;
+    result_type_1 res1 = p1(a.first...);
+    result_type_2 res2 = p2(a.second...);
     if (! cmp(res1, res2))
     {
-      std::cerr << "Kernel_checker error : " << res1 << " != " << res2
-                << " for the inputs : " << std::endl;
-      std::cerr << a1.first << std::endl;
-      std::cerr << a1.second << std::endl;
-      std::cerr << a2.first << std::endl;
-      std::cerr << a2.second << std::endl;
-      std::cerr << a3.first << std::endl;
-      std::cerr << a3.second << std::endl;
-      std::cerr << a4.first << std::endl;
-      std::cerr << a4.second << std::endl;
-      std::cerr << a5.first << std::endl;
-      std::cerr << a5.second << std::endl;
-      std::cerr << "functor first kernel : "
-                << typeid(p1).name() << std::endl;
-      std::cerr << "functor second kernel: "
-                << typeid(p2).name() << std::endl;
-      std::cerr << CGAL_PRETTY_FUNCTION << std::endl;
       CGAL_kernel_assertion(false);
     }
     return Pairify<result_type_1, result_type_2>()(res1, res2);
   }
-
-  // Same thing with more arguments...
 };
 
 struct dont_check_equal {
diff -pruN 5.0.3-1/include/CGAL/kernel_config.h 5.1-1/include/CGAL/kernel_config.h
--- 5.0.3-1/include/CGAL/kernel_config.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/kernel_config.h	2020-09-08 12:15:02.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/kernel_config.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/kernel_config.h $
 // $Id: kernel_config.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/Aff_transformationCd.h 5.1-1/include/CGAL/Kernel_d/Aff_transformationCd.h
--- 5.0.3-1/include/CGAL/Kernel_d/Aff_transformationCd.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/Aff_transformationCd.h	2020-09-08 12:15:03.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/Aff_transformationCd.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/Aff_transformationCd.h $
 // $Id: Aff_transformationCd.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/Aff_transformation_d.h 5.1-1/include/CGAL/Kernel_d/Aff_transformation_d.h
--- 5.0.3-1/include/CGAL/Kernel_d/Aff_transformation_d.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/Aff_transformation_d.h	2020-09-08 12:15:03.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/Aff_transformation_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/Aff_transformation_d.h $
 // $Id: Aff_transformation_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/Aff_transformationHd.h 5.1-1/include/CGAL/Kernel_d/Aff_transformationHd.h
--- 5.0.3-1/include/CGAL/Kernel_d/Aff_transformationHd.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/Aff_transformationHd.h	2020-09-08 12:15:03.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/Aff_transformationHd.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/Aff_transformationHd.h $
 // $Id: Aff_transformationHd.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/Cartesian_const_iterator_d.h 5.1-1/include/CGAL/Kernel_d/Cartesian_const_iterator_d.h
--- 5.0.3-1/include/CGAL/Kernel_d/Cartesian_const_iterator_d.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/Cartesian_const_iterator_d.h	2020-09-08 12:15:03.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/Cartesian_const_iterator_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/Cartesian_const_iterator_d.h $
 // $Id: Cartesian_const_iterator_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/Cartesian_converter_d.h 5.1-1/include/CGAL/Kernel_d/Cartesian_converter_d.h
--- 5.0.3-1/include/CGAL/Kernel_d/Cartesian_converter_d.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/Cartesian_converter_d.h	2020-09-08 12:15:03.000000000 +0000
@@ -6,7 +6,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/Cartesian_converter_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/Cartesian_converter_d.h $
 // $Id: Cartesian_converter_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/debug.h 5.1-1/include/CGAL/Kernel_d/debug.h
--- 5.0.3-1/include/CGAL/Kernel_d/debug.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/debug.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/debug.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/debug.h $
 // $Id: debug.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/DirectionCd.h 5.1-1/include/CGAL/Kernel_d/DirectionCd.h
--- 5.0.3-1/include/CGAL/Kernel_d/DirectionCd.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/DirectionCd.h	2020-09-08 12:15:03.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/DirectionCd.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/DirectionCd.h $
 // $Id: DirectionCd.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/DirectionCd_impl.h 5.1-1/include/CGAL/Kernel_d/DirectionCd_impl.h
--- 5.0.3-1/include/CGAL/Kernel_d/DirectionCd_impl.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/DirectionCd_impl.h	2020-09-08 12:15:03.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/DirectionCd_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/DirectionCd_impl.h $
 // $Id: DirectionCd_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/Direction_d.h 5.1-1/include/CGAL/Kernel_d/Direction_d.h
--- 5.0.3-1/include/CGAL/Kernel_d/Direction_d.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/Direction_d.h	2020-09-08 12:15:03.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/Direction_d.h $
-// $Id: Direction_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/Direction_d.h $
+// $Id: Direction_d.h 127d76c 2020-06-10T17:57:54+02:00 Laurent Rineau
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -62,6 +62,10 @@ class Direction_d : public pR::Direction
   { return Base::operator==(w); }
   bool operator!=(const Self& w) const
   { return Base::operator!=(w); }
+  bool operator==(const Base& w) const
+  { return Base::operator==(w); }
+  bool operator!=(const Base& w) const
+  { return Base::operator!=(w); }
 };
 
 } //namespace CGAL
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/DirectionHd.h 5.1-1/include/CGAL/Kernel_d/DirectionHd.h
--- 5.0.3-1/include/CGAL/Kernel_d/DirectionHd.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/DirectionHd.h	2020-09-08 12:15:03.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/DirectionHd.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/DirectionHd.h $
 // $Id: DirectionHd.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/DirectionHd_impl.h 5.1-1/include/CGAL/Kernel_d/DirectionHd_impl.h
--- 5.0.3-1/include/CGAL/Kernel_d/DirectionHd_impl.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/DirectionHd_impl.h	2020-09-08 12:15:03.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/DirectionHd_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/DirectionHd_impl.h $
 // $Id: DirectionHd_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/function_objectsCd.h 5.1-1/include/CGAL/Kernel_d/function_objectsCd.h
--- 5.0.3-1/include/CGAL/Kernel_d/function_objectsCd.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/function_objectsCd.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/function_objectsCd.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/function_objectsCd.h $
 // $Id: function_objectsCd.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/function_objects.h 5.1-1/include/CGAL/Kernel_d/function_objects.h
--- 5.0.3-1/include/CGAL/Kernel_d/function_objects.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/function_objects.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/function_objects.h $
-// $Id: function_objects.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/function_objects.h $
+// $Id: function_objects.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -32,81 +32,10 @@ class Construct
   public:
     typedef ToBeConstructed  result_type;
 
+    template <class ... A>
     ToBeConstructed
-    operator()() const
-    { return ToBeConstructed(); }
-
-    template <class A1>
-    ToBeConstructed
-    operator()( const A1& a1) const
-    { return ToBeConstructed(a1); }
-
-    template <class A1, class A2>
-    ToBeConstructed
-    operator()( const A1& a1, const A2& a2) const
-    { return ToBeConstructed(a1,a2); }
-
-    template <class A1, class A2, class A3>
-    ToBeConstructed
-    operator()( const A1& a1, const A2& a2, const A3& a3) const
-    { return ToBeConstructed(a1,a2,a3); }
-
-    template <class A1, class A2, class A3, class A4>
-    ToBeConstructed
-    operator()( const A1& a1, const A2& a2, const A3& a3, const A4& a4) const
-    { return ToBeConstructed(a1,a2,a3,a4); }
-
-    template <class A1, class A2, class A3, class A4, class A5>
-    ToBeConstructed
-    operator()( const A1& a1, const A2& a2, const A3& a3, const A4& a4,
-            const A5& a5) const
-    { return ToBeConstructed(a1,a2,a3,a4,a5); }
-
-    template <class A>
-    ToBeConstructed
-    operator()( const A& a1, const A& a2, const A& a3,
-                const A& a4, const A& a5, const A& a6 ) const
-    { return ToBeConstructed(a1,a2,a3,a4,a5,a6); }
-
-    template <class A>
-    ToBeConstructed
-    operator()( const A& a1, const A& a2, const A& a3,
-                const A& a4, const A& a5, const A& a6,
-                const A& a7 ) const
-    { return ToBeConstructed(a1,a2,a3,a4,a5,a6,a7); }
-
-    template <class A>
-    ToBeConstructed
-    operator()( const A& a1, const A& a2, const A& a3,
-                const A& a4, const A& a5, const A& a6,
-                const A& a7, const A& a8, const A& a9) const
-    { return ToBeConstructed(a1,a2,a3,a4,a5,a6,a7,a8,a9); }
-
-    template <class A>
-    ToBeConstructed
-    operator()( const A& a1, const A& a2, const A& a3,
-                const A& a4, const A& a5, const A& a6,
-                const A& a7, const A& a8, const A& a9,
-                const A& a10) const
-    { return ToBeConstructed(a1,a2,a3,a4,a5,a6,a7,a8,a9,a10); }
-
-    template <class A>
-    ToBeConstructed
-    operator()( const A& a1, const A& a2, const A& a3,
-                const A& a4, const A& a5, const A& a6,
-                const A& a7, const A& a8, const A& a9,
-                const A& a10,const A& a11,const A& a12) const
-    { return ToBeConstructed(a1,a2,a3,a4,a5,a6,a7,a8,a9,a10,a11,a12); }
-
-    template <class A>
-    ToBeConstructed
-    operator()( const A& a1, const A& a2, const A& a3,
-                const A& a4, const A& a5, const A& a6,
-                const A& a7, const A& a8, const A& a9,
-                const A& a10,const A& a11,const A& a12,
-                const A& a13) const
-    { return ToBeConstructed(a1,a2,a3,a4,a5,a6,a7,a8,a9,a10,a11,a12,a13); }
-
+    operator()( A&& ... a) const
+    { return ToBeConstructed(std::forward<A>(a)...); }
 };
 
 class Call_has_on_positive_side
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/function_objectsHd.h 5.1-1/include/CGAL/Kernel_d/function_objectsHd.h
--- 5.0.3-1/include/CGAL/Kernel_d/function_objectsHd.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/function_objectsHd.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/function_objectsHd.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/function_objectsHd.h $
 // $Id: function_objectsHd.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/HyperplaneCd.h 5.1-1/include/CGAL/Kernel_d/HyperplaneCd.h
--- 5.0.3-1/include/CGAL/Kernel_d/HyperplaneCd.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/HyperplaneCd.h	2020-09-08 12:15:03.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/HyperplaneCd.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/HyperplaneCd.h $
 // $Id: HyperplaneCd.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/HyperplaneCd_impl.h 5.1-1/include/CGAL/Kernel_d/HyperplaneCd_impl.h
--- 5.0.3-1/include/CGAL/Kernel_d/HyperplaneCd_impl.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/HyperplaneCd_impl.h	2020-09-08 12:15:03.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/HyperplaneCd_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/HyperplaneCd_impl.h $
 // $Id: HyperplaneCd_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/Hyperplane_d.h 5.1-1/include/CGAL/Kernel_d/Hyperplane_d.h
--- 5.0.3-1/include/CGAL/Kernel_d/Hyperplane_d.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/Hyperplane_d.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/Hyperplane_d.h $
-// $Id: Hyperplane_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/Hyperplane_d.h $
+// $Id: Hyperplane_d.h 489e853 2020-06-10T18:25:33+02:00 Laurent Rineau
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -75,6 +75,10 @@ public:
   { return Base::operator==(w); }
   bool operator!=(const Self& w) const
   { return Base::operator!=(w); }
+  bool operator==(const Base& w) const
+  { return Base::operator==(w); }
+  bool operator!=(const Base& w) const
+  { return Base::operator!=(w); }
 };
 
 } //namespace CGAL
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/HyperplaneHd.h 5.1-1/include/CGAL/Kernel_d/HyperplaneHd.h
--- 5.0.3-1/include/CGAL/Kernel_d/HyperplaneHd.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/HyperplaneHd.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/HyperplaneHd.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/HyperplaneHd.h $
 // $Id: HyperplaneHd.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/HyperplaneHd_impl.h 5.1-1/include/CGAL/Kernel_d/HyperplaneHd_impl.h
--- 5.0.3-1/include/CGAL/Kernel_d/HyperplaneHd_impl.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/HyperplaneHd_impl.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/HyperplaneHd_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/HyperplaneHd_impl.h $
 // $Id: HyperplaneHd_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/Interface_classes.h 5.1-1/include/CGAL/Kernel_d/Interface_classes.h
--- 5.0.3-1/include/CGAL/Kernel_d/Interface_classes.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/Interface_classes.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/Interface_classes.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/Interface_classes.h $
 // $Id: Interface_classes.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/interface_macros_d.h 5.1-1/include/CGAL/Kernel_d/interface_macros_d.h
--- 5.0.3-1/include/CGAL/Kernel_d/interface_macros_d.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/interface_macros_d.h	2020-09-08 12:15:04.000000000 +0000
@@ -6,7 +6,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/interface_macros_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/interface_macros_d.h $
 // $Id: interface_macros_d.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/intersection_objectsCd.h 5.1-1/include/CGAL/Kernel_d/intersection_objectsCd.h
--- 5.0.3-1/include/CGAL/Kernel_d/intersection_objectsCd.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/intersection_objectsCd.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/intersection_objectsCd.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/intersection_objectsCd.h $
 // $Id: intersection_objectsCd.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/intersection_objects_d.h 5.1-1/include/CGAL/Kernel_d/intersection_objects_d.h
--- 5.0.3-1/include/CGAL/Kernel_d/intersection_objects_d.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/intersection_objects_d.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/intersection_objects_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/intersection_objects_d.h $
 // $Id: intersection_objects_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/intersection_objectsHd.h 5.1-1/include/CGAL/Kernel_d/intersection_objectsHd.h
--- 5.0.3-1/include/CGAL/Kernel_d/intersection_objectsHd.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/intersection_objectsHd.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/intersection_objectsHd.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/intersection_objectsHd.h $
 // $Id: intersection_objectsHd.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/Interval_linear_algebra.h 5.1-1/include/CGAL/Kernel_d/Interval_linear_algebra.h
--- 5.0.3-1/include/CGAL/Kernel_d/Interval_linear_algebra.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/Interval_linear_algebra.h	2020-09-08 12:15:04.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/Interval_linear_algebra.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/Interval_linear_algebra.h $
 // $Id: Interval_linear_algebra.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/Iso_box_d.h 5.1-1/include/CGAL/Kernel_d/Iso_box_d.h
--- 5.0.3-1/include/CGAL/Kernel_d/Iso_box_d.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/Iso_box_d.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/Iso_box_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/Iso_box_d.h $
 // $Id: Iso_box_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/Kernel_classesCd.h 5.1-1/include/CGAL/Kernel_d/Kernel_classesCd.h
--- 5.0.3-1/include/CGAL/Kernel_d/Kernel_classesCd.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/Kernel_classesCd.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/Kernel_classesCd.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/Kernel_classesCd.h $
 // $Id: Kernel_classesCd.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/Kernel_classesHd.h 5.1-1/include/CGAL/Kernel_d/Kernel_classesHd.h
--- 5.0.3-1/include/CGAL/Kernel_d/Kernel_classesHd.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/Kernel_classesHd.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/Kernel_classesHd.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/Kernel_classesHd.h $
 // $Id: Kernel_classesHd.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/Linear_algebraCd_impl.h 5.1-1/include/CGAL/Kernel_d/Linear_algebraCd_impl.h
--- 5.0.3-1/include/CGAL/Kernel_d/Linear_algebraCd_impl.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/Linear_algebraCd_impl.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/Linear_algebraCd_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/Linear_algebraCd_impl.h $
 // $Id: Linear_algebraCd_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/Linear_algebraHd_impl.h 5.1-1/include/CGAL/Kernel_d/Linear_algebraHd_impl.h
--- 5.0.3-1/include/CGAL/Kernel_d/Linear_algebraHd_impl.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/Linear_algebraHd_impl.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/Linear_algebraHd_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/Linear_algebraHd_impl.h $
 // $Id: Linear_algebraHd_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/Line_d.h 5.1-1/include/CGAL/Kernel_d/Line_d.h
--- 5.0.3-1/include/CGAL/Kernel_d/Line_d.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/Line_d.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/Line_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/Line_d.h $
 // $Id: Line_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/Line_d_impl.h 5.1-1/include/CGAL/Kernel_d/Line_d_impl.h
--- 5.0.3-1/include/CGAL/Kernel_d/Line_d_impl.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/Line_d_impl.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/Line_d_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/Line_d_impl.h $
 // $Id: Line_d_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/Matrix__.h 5.1-1/include/CGAL/Kernel_d/Matrix__.h
--- 5.0.3-1/include/CGAL/Kernel_d/Matrix__.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/Matrix__.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/Matrix__.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/Matrix__.h $
 // $Id: Matrix__.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/Pair_d.h 5.1-1/include/CGAL/Kernel_d/Pair_d.h
--- 5.0.3-1/include/CGAL/Kernel_d/Pair_d.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/Pair_d.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/Pair_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/Pair_d.h $
 // $Id: Pair_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/PointCd.h 5.1-1/include/CGAL/Kernel_d/PointCd.h
--- 5.0.3-1/include/CGAL/Kernel_d/PointCd.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/PointCd.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/PointCd.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/PointCd.h $
 // $Id: PointCd.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/PointCd_impl.h 5.1-1/include/CGAL/Kernel_d/PointCd_impl.h
--- 5.0.3-1/include/CGAL/Kernel_d/PointCd_impl.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/PointCd_impl.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/PointCd_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/PointCd_impl.h $
 // $Id: PointCd_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/Point_d.h 5.1-1/include/CGAL/Kernel_d/Point_d.h
--- 5.0.3-1/include/CGAL/Kernel_d/Point_d.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/Point_d.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/Point_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/Point_d.h $
 // $Id: Point_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/PointHd.h 5.1-1/include/CGAL/Kernel_d/PointHd.h
--- 5.0.3-1/include/CGAL/Kernel_d/PointHd.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/PointHd.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/PointHd.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/PointHd.h $
 // $Id: PointHd.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/PointHd_impl.h 5.1-1/include/CGAL/Kernel_d/PointHd_impl.h
--- 5.0.3-1/include/CGAL/Kernel_d/PointHd_impl.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/PointHd_impl.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/PointHd_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/PointHd_impl.h $
 // $Id: PointHd_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/Ray_d.h 5.1-1/include/CGAL/Kernel_d/Ray_d.h
--- 5.0.3-1/include/CGAL/Kernel_d/Ray_d.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/Ray_d.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/Ray_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/Ray_d.h $
 // $Id: Ray_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/Segment_d.h 5.1-1/include/CGAL/Kernel_d/Segment_d.h
--- 5.0.3-1/include/CGAL/Kernel_d/Segment_d.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/Segment_d.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/Segment_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/Segment_d.h $
 // $Id: Segment_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/simple_objects.h 5.1-1/include/CGAL/Kernel_d/simple_objects.h
--- 5.0.3-1/include/CGAL/Kernel_d/simple_objects.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/simple_objects.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/simple_objects.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/simple_objects.h $
 // $Id: simple_objects.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/Sphere_d.h 5.1-1/include/CGAL/Kernel_d/Sphere_d.h
--- 5.0.3-1/include/CGAL/Kernel_d/Sphere_d.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/Sphere_d.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/Sphere_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/Sphere_d.h $
 // $Id: Sphere_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/Tuple_d.h 5.1-1/include/CGAL/Kernel_d/Tuple_d.h
--- 5.0.3-1/include/CGAL/Kernel_d/Tuple_d.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/Tuple_d.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/Tuple_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/Tuple_d.h $
 // $Id: Tuple_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/VectorCd.h 5.1-1/include/CGAL/Kernel_d/VectorCd.h
--- 5.0.3-1/include/CGAL/Kernel_d/VectorCd.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/VectorCd.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/VectorCd.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/VectorCd.h $
 // $Id: VectorCd.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/VectorCd_impl.h 5.1-1/include/CGAL/Kernel_d/VectorCd_impl.h
--- 5.0.3-1/include/CGAL/Kernel_d/VectorCd_impl.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/VectorCd_impl.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/VectorCd_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/VectorCd_impl.h $
 // $Id: VectorCd_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/Vector_d.h 5.1-1/include/CGAL/Kernel_d/Vector_d.h
--- 5.0.3-1/include/CGAL/Kernel_d/Vector_d.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/Vector_d.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/Vector_d.h $
-// $Id: Vector_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/Vector_d.h $
+// $Id: Vector_d.h 127d76c 2020-06-10T17:57:54+02:00 Laurent Rineau
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Michael Seel
@@ -91,6 +91,10 @@ class Vector_d : public pR::Vector_d_bas
   { return Base::operator==(w); }
   bool operator!=(const Self& w) const
   { return Base::operator!=(w); }
+  bool operator==(const Base& w) const
+  { return Base::operator==(w); }
+  bool operator!=(const Base& w) const
+  { return Base::operator!=(w); }
 
 };
 
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/Vector__.h 5.1-1/include/CGAL/Kernel_d/Vector__.h
--- 5.0.3-1/include/CGAL/Kernel_d/Vector__.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/Vector__.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/Vector__.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/Vector__.h $
 // $Id: Vector__.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/VectorHd.h 5.1-1/include/CGAL/Kernel_d/VectorHd.h
--- 5.0.3-1/include/CGAL/Kernel_d/VectorHd.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/VectorHd.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/VectorHd.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/VectorHd.h $
 // $Id: VectorHd.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_d/VectorHd_impl.h 5.1-1/include/CGAL/Kernel_d/VectorHd_impl.h
--- 5.0.3-1/include/CGAL/Kernel_d/VectorHd_impl.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_d/VectorHd_impl.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Kernel_d/VectorHd_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Kernel_d/VectorHd_impl.h $
 // $Id: VectorHd_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_profiler.h 5.1-1/include/CGAL/Kernel_profiler.h
--- 5.0.3-1/include/CGAL/Kernel_profiler.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_profiler.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/include/CGAL/Kernel_profiler.h $
-// $Id: Kernel_profiler.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/Kernel_profiler.h $
+// $Id: Kernel_profiler.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -34,77 +34,13 @@ struct Primitive_profiler
     Primitive_profiler(const P& p = P())
       : P(p) {}
 
-    template <class A1>
+    template <class ... A>
     result_type
-    operator()(const A1 &a1) const
+    operator()(A&& ... a) const
     {
         CGAL_KERNEL_PROFILER;
-        return P::operator()(a1);
+        return P::operator()(std::forward<A>(a)...);
     }
-
-    template <class A1, class A2>
-    result_type
-    operator()(const A1 &a1, const A2 &a2) const
-    {
-        CGAL_KERNEL_PROFILER;
-        return P::operator()(a1, a2);
-    }
-
-    template <class A1, class A2, class A3>
-    result_type
-    operator()(const A1 &a1, const A2 &a2, const A3 &a3) const
-    {
-        CGAL_KERNEL_PROFILER;
-        return P::operator()(a1, a2, a3);
-    }
-
-    template <class A1, class A2, class A3, class A4>
-    result_type
-    operator()(const A1 &a1, const A2 &a2, const A3 &a3, const A4 &a4) const
-    {
-        CGAL_KERNEL_PROFILER;
-        return P::operator()(a1, a2, a3, a4);
-    }
-
-    template <class A1, class A2, class A3, class A4, class A5>
-    result_type
-    operator()(const A1 &a1, const A2 &a2, const A3 &a3, const A4 &a4,
-               const A5 &a5) const
-    {
-        CGAL_KERNEL_PROFILER;
-        return P::operator()(a1, a2, a3, a4, a5);
-    }
-
-    template <class A1, class A2, class A3, class A4, class A5, class A6>
-    result_type
-    operator()(const A1 &a1, const A2 &a2, const A3 &a3, const A4 &a4,
-               const A5 &a5, const A6 &a6) const
-    {
-        CGAL_KERNEL_PROFILER;
-        return P::operator()(a1, a2, a3, a4, a5, a6);
-    }
-
-    template <class A1, class A2, class A3, class A4,
-              class A5, class A6, class A7>
-    result_type
-    operator()(const A1 &a1, const A2 &a2, const A3 &a3, const A4 &a4,
-               const A5 &a5, const A6 &a6, const A7 &a7) const
-    {
-        CGAL_KERNEL_PROFILER;
-        return P::operator()(a1, a2, a3, a4, a5, a6, a7);
-    }
-
-    template <class A1, class A2, class A3, class A4,
-              class A5, class A6, class A7, class A8>
-    result_type
-    operator()(const A1 &a1, const A2 &a2, const A3 &a3, const A4 &a4,
-               const A5 &a5, const A6 &a6, const A7 &a7, const A8 &a8) const
-    {
-        CGAL_KERNEL_PROFILER;
-        return P::operator()(a1, a2, a3, a4, a5, a6, a7, a8);
-    }
-
-    // ...
 };
 
 // We inherit all geometric objects from K, and just replace the primitives.
diff -pruN 5.0.3-1/include/CGAL/kernel_to_kernel.h 5.1-1/include/CGAL/kernel_to_kernel.h
--- 5.0.3-1/include/CGAL/kernel_to_kernel.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/kernel_to_kernel.h	2020-09-08 12:15:02.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/kernel_to_kernel.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/kernel_to_kernel.h $
 // $Id: kernel_to_kernel.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_traits_fwd.h 5.1-1/include/CGAL/Kernel_traits_fwd.h
--- 5.0.3-1/include/CGAL/Kernel_traits_fwd.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_traits_fwd.h	2020-09-08 12:14:54.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/Kernel_traits_fwd.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/Kernel_traits_fwd.h $
 // $Id: Kernel_traits_fwd.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Kernel_traits.h 5.1-1/include/CGAL/Kernel_traits.h
--- 5.0.3-1/include/CGAL/Kernel_traits.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Kernel_traits.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Kernel_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Kernel_traits.h $
 // $Id: Kernel_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/K_neighbor_search.h 5.1-1/include/CGAL/K_neighbor_search.h
--- 5.0.3-1/include/CGAL/K_neighbor_search.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/K_neighbor_search.h	2020-09-08 12:15:39.000000000 +0000
@@ -3,12 +3,13 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_searching/include/CGAL/K_neighbor_search.h $
-// $Id: K_neighbor_search.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_searching/include/CGAL/K_neighbor_search.h $
+// $Id: K_neighbor_search.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
-// Author(s)     : Hans Tangelder (<hanst@cs.uu.nl>)
+// Author(s)     : Hans Tangelder (<hanst@cs.uu.nl>),
+//                 Clement Jamin (clement.jamin.pro@gmail.com)
 
 #ifndef  CGAL_K_NEIGHBOR_SEARCH_H
 #define  CGAL_K_NEIGHBOR_SEARCH_H
@@ -19,8 +20,9 @@
 
 #include <CGAL/internal/K_neighbor_search.h>
 #include <CGAL/internal/Get_dimension_tag.h>
+#include <CGAL/internal/Search_helpers.h>
 
-
+#include <iterator> // for std::distance
 
 
 namespace CGAL {
@@ -30,9 +32,11 @@ class K_neighbor_search;
 template <class SearchTraits,
           class Distance= typename internal::Spatial_searching_default_distance<SearchTraits>::type,
           class Splitter= Sliding_midpoint<SearchTraits> ,
-          class Tree= Kd_tree<SearchTraits, Splitter, Tag_true> >
-class K_neighbor_search: public internal::K_neighbor_search<SearchTraits,Distance,Splitter,Tree> {
+          class Tree= Kd_tree<SearchTraits, Splitter, Tag_true, Tag_false> >
+class K_neighbor_search: public internal::K_neighbor_search<SearchTraits,Distance,Splitter,Tree>
+{
   typedef  internal::K_neighbor_search<SearchTraits,Distance,Splitter,Tree> Base;
+  typedef typename Tree::Point_d Point;
 
 public:
   typedef typename Base::FT FT;
@@ -40,9 +44,12 @@ public:
 
   K_neighbor_search(const Tree& tree, const typename Base::Query_item& q,
     unsigned int k=1, FT Eps=FT(0.0), bool Search_nearest=true, const Distance& d=Distance(),bool sorted=true)
-    : Base(q,k,Eps,Search_nearest,d)
+  : Base(q,k,Eps,Search_nearest,d),
+    m_distance_helper(this->distance_instance, tree.traits()),
+    m_tree(tree)
   {
     if (tree.empty()) return;
+
     compute_neighbors_general(tree.root(),tree.bounding_box());
     if (sorted) this->queue.sort();
   };
@@ -51,6 +58,9 @@ private:
   typedef typename Base::Node_const_handle Node_const_handle;
   using Base::branch;
 
+  internal::Distance_helper<Distance, SearchTraits> m_distance_helper;
+  Tree const& m_tree;
+
   void
   compute_neighbors_general(typename Base::Node_const_handle N, const Kd_tree_rectangle<FT,D>& r)
   {
@@ -115,19 +125,127 @@ private:
 
     }
     else
+    {
+      // n is a leaf
+      typename Tree::Leaf_node_const_handle node =
+        static_cast<typename Tree::Leaf_node_const_handle>(N);
+      this->number_of_leaf_nodes_visited++;
+      if (node->size() > 0)
+      {
+        typename internal::Has_points_cache<Tree, internal::has_Enable_points_cache<Tree>::type::value>::type dummy;
+        if (this->search_nearest)
+          search_nearest_in_leaf(node, dummy);
+        else
+          search_furthest_in_leaf(node, dummy);
+      }
+    }
+  }
+
+  // With cache
+  void search_nearest_in_leaf(typename Tree::Leaf_node_const_handle node, Tag_true)
+  {
+    typename Tree::iterator it_node_point = node->begin(), it_node_point_end = node->end();
+    int dim = m_tree.dim();
+    typename std::vector<FT>::const_iterator cache_point_begin = m_tree.cache_begin() + dim*(it_node_point - m_tree.begin());
+    // As long as the queue is not full, the node is just inserted
+    for (; !this->queue.full() && it_node_point != it_node_point_end; ++it_node_point)
+    {
+      this->number_of_items_visited++;
+
+      FT distance_to_query_object =
+        m_distance_helper.transformed_distance_from_coordinates(
+          this->query_object, *it_node_point, cache_point_begin, cache_point_begin + dim);
+      this->queue.insert(std::make_pair(&(*it_node_point), distance_to_query_object));
+
+      cache_point_begin += dim;
+    }
+    // Now that the queue is full, we can gain time by keeping track
+    // of the current worst distance to interrupt the distance computation earlier
+    FT worst_dist = this->queue.top().second;
+    for (; it_node_point != it_node_point_end; ++it_node_point)
+    {
+      this->number_of_items_visited++;
+
+      FT distance_to_query_object =
+        m_distance_helper.interruptible_transformed_distance(
+          this->query_object, *it_node_point, cache_point_begin, cache_point_begin + dim, worst_dist);
+
+      if (distance_to_query_object < worst_dist)
+      {
+        this->queue.insert(std::make_pair(&(*it_node_point), distance_to_query_object));
+        worst_dist = this->queue.top().second;
+      }
+
+      cache_point_begin += dim;
+    }
+  }
+
+  // Without cache
+  void search_nearest_in_leaf(typename Tree::Leaf_node_const_handle node, Tag_false)
+  {
+    typename Tree::iterator it_node_point = node->begin(), it_node_point_end = node->end();
+    // As long as the queue is not full, the node is just inserted
+    for (; !this->queue.full() && it_node_point != it_node_point_end; ++it_node_point)
+    {
+      this->number_of_items_visited++;
+
+      FT distance_to_query_object =
+        this->distance_instance.transformed_distance(this->query_object, *it_node_point);
+      this->queue.insert(std::make_pair(&(*it_node_point), distance_to_query_object));
+    }
+    // Now that the queue is full, we can gain time by keeping track
+    // of the current worst distance to interrupt the distance computation earlier
+    FT worst_dist = this->queue.top().second;
+    for (; it_node_point != it_node_point_end; ++it_node_point)
+    {
+      this->number_of_items_visited++;
+
+      FT distance_to_query_object =
+        m_distance_helper.interruptible_transformed_distance(
+          this->query_object, *it_node_point, worst_dist);
+
+      if (distance_to_query_object < worst_dist)
       {
-        // n is a leaf
-         typename Tree::Leaf_node_const_handle node =
-          static_cast<typename Tree::Leaf_node_const_handle>(N);
-        this->number_of_leaf_nodes_visited++;
-        if (node->size() > 0)
-          for (typename Tree::iterator it = node->begin(); it != node->end(); it++) {
-            this->number_of_items_visited++;
-            FT distance_to_query_object =
-              this->distance_instance.transformed_distance(this->query_object,*it);
-            this->queue.insert(std::make_pair(&(*it),distance_to_query_object));
-          }
+        this->queue.insert(std::make_pair(&(*it_node_point), distance_to_query_object));
+        worst_dist = this->queue.top().second;
       }
+    }
+  }
+
+
+  // With cache
+  void search_furthest_in_leaf(typename Tree::Leaf_node_const_handle node, Tag_true)
+  {
+    typename Tree::iterator it_node_point = node->begin(), it_node_point_end = node->end();
+    int dim = m_tree.dim();
+    typename std::vector<FT>::const_iterator cache_point_begin = m_tree.cache_begin() + dim*(it_node_point - m_tree.begin());
+    // In furthest search mode, the interruptible distance cannot be used to optimize
+    for (; it_node_point != it_node_point_end; ++it_node_point)
+    {
+      this->number_of_items_visited++;
+
+      FT distance_to_query_object =
+        m_distance_helper.transformed_distance_from_coordinates(
+          this->query_object, *it_node_point, cache_point_begin, cache_point_begin + dim);
+      this->queue.insert(std::make_pair(&(*it_node_point), distance_to_query_object));
+
+      cache_point_begin += dim;
+    }
+  }
+
+  // Without cache
+  void search_furthest_in_leaf(typename Tree::Leaf_node_const_handle node, Tag_false)
+  {
+    typename Tree::iterator it_node_point = node->begin(), it_node_point_end = node->end();
+    // In furthest search mode, the interruptible distance cannot be used to optimize
+    for (; it_node_point != it_node_point_end; ++it_node_point)
+    {
+      this->number_of_items_visited++;
+
+      FT distance_to_query_object =
+        this->distance_instance.transformed_distance(this->query_object, *it_node_point);
+      this->queue.insert(std::make_pair(&(*it_node_point), distance_to_query_object));
+    }
   }
 
 }; // class
diff -pruN 5.0.3-1/include/CGAL/known_bit_size_integers.h 5.1-1/include/CGAL/known_bit_size_integers.h
--- 5.0.3-1/include/CGAL/known_bit_size_integers.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/known_bit_size_integers.h	2020-09-08 12:15:16.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/known_bit_size_integers.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/known_bit_size_integers.h $
 // $Id: known_bit_size_integers.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Labeled_image_mesh_domain_3.h 5.1-1/include/CGAL/Labeled_image_mesh_domain_3.h
--- 5.0.3-1/include/CGAL/Labeled_image_mesh_domain_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Labeled_image_mesh_domain_3.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Labeled_image_mesh_domain_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Labeled_image_mesh_domain_3.h $
 // $Id: Labeled_image_mesh_domain_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Labeled_mesh_domain_3.h 5.1-1/include/CGAL/Labeled_mesh_domain_3.h
--- 5.0.3-1/include/CGAL/Labeled_mesh_domain_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Labeled_mesh_domain_3.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Labeled_mesh_domain_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Labeled_mesh_domain_3.h $
 // $Id: Labeled_mesh_domain_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Largest_empty_iso_rectangle_2.h 5.1-1/include/CGAL/Largest_empty_iso_rectangle_2.h
--- 5.0.3-1/include/CGAL/Largest_empty_iso_rectangle_2.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Largest_empty_iso_rectangle_2.h	2020-09-08 12:14:54.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Inscribed_areas/include/CGAL/Largest_empty_iso_rectangle_2.h $
-// $Id: Largest_empty_iso_rectangle_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Inscribed_areas/include/CGAL/Largest_empty_iso_rectangle_2.h $
+// $Id: Largest_empty_iso_rectangle_2.h 0fcbaf5 2020-05-24T11:15:12+02:00 Marc Glisse
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -233,12 +233,11 @@ public:
   ~Largest_empty_iso_rectangle_2();
 
   //! An operator=
-  Largest_empty_iso_rectangle_2<T>&
-    operator =(const Largest_empty_iso_rectangle_2<T>& ler);
+  Largest_empty_iso_rectangle_2&
+    operator =(const Largest_empty_iso_rectangle_2& ler);
 
   //! A copy constructor
-  Largest_empty_iso_rectangle_2<T>(
-               const Largest_empty_iso_rectangle_2<T>& ler);
+  Largest_empty_iso_rectangle_2(const Largest_empty_iso_rectangle_2& ler);
 
   struct Internal_point {
     Point_2 x_part;// the x coordinate of the point
diff -pruN 5.0.3-1/include/CGAL/Lazy_exact_nt.h 5.1-1/include/CGAL/Lazy_exact_nt.h
--- 5.0.3-1/include/CGAL/Lazy_exact_nt.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Lazy_exact_nt.h	2020-09-08 12:15:16.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Lazy_exact_nt.h $
-// $Id: Lazy_exact_nt.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Lazy_exact_nt.h $
+// $Id: Lazy_exact_nt.h c1f2fde 2020-04-11T12:12:27+02:00 Marc Glisse
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -383,6 +383,9 @@ public :
   typename boost::disable_if<is_implicit_convertible<ET1,ET>,int>::type=0)
     : Base(new Lazy_lazy_exact_Cst<ET, ET1>(x)){}
 
+  friend void swap(Lazy_exact_nt& a, Lazy_exact_nt& b) noexcept
+  { swap(static_cast<Base&>(a), static_cast<Base&>(b)); }
+
   Self operator+ () const
   { return *this; }
 
@@ -435,6 +438,69 @@ public :
     return *this = new Lazy_exact_Div<ET>(*this, b);
   }
 
+  // Mixed comparisons with int.
+  friend bool operator<(const Lazy_exact_nt& a, int b)
+  {
+    CGAL_BRANCH_PROFILER(std::string(" failures/calls to   : ") + std::string(CGAL_PRETTY_FUNCTION), tmp);
+    Uncertain<bool> res = a.approx() < b;
+    if (is_certain(res))
+      return res;
+    CGAL_BRANCH_PROFILER_BRANCH(tmp);
+    return a.exact() < b;
+  }
+
+  friend bool operator>(const Lazy_exact_nt& a, int b)
+  {
+    CGAL_BRANCH_PROFILER(std::string(" failures/calls to   : ") + std::string(CGAL_PRETTY_FUNCTION), tmp);
+    Uncertain<bool> res = b < a.approx();
+    if (is_certain(res))
+      return get_certain(res);
+    CGAL_BRANCH_PROFILER_BRANCH(tmp);
+    return b < a.exact();
+  }
+
+  friend bool operator==(const Lazy_exact_nt& a, int b)
+  {
+    CGAL_BRANCH_PROFILER(std::string(" failures/calls to   : ") + std::string(CGAL_PRETTY_FUNCTION), tmp);
+    Uncertain<bool> res = b == a.approx();
+    if (is_certain(res))
+      return get_certain(res);
+    CGAL_BRANCH_PROFILER_BRANCH(tmp);
+    return b == a.exact();
+  }
+
+
+  // Mixed comparisons with double.
+  friend bool operator<(const Lazy_exact_nt& a, double b)
+  {
+    CGAL_BRANCH_PROFILER(std::string(" failures/calls to   : ") + std::string(CGAL_PRETTY_FUNCTION), tmp);
+    Uncertain<bool> res = a.approx() < b;
+    if (is_certain(res))
+      return res;
+    CGAL_BRANCH_PROFILER_BRANCH(tmp);
+    return a.exact() < b;
+  }
+
+  friend bool operator>(const Lazy_exact_nt& a, double b)
+  {
+    CGAL_BRANCH_PROFILER(std::string(" failures/calls to   : ") + std::string(CGAL_PRETTY_FUNCTION), tmp);
+    Uncertain<bool> res = b < a.approx();
+    if (is_certain(res))
+      return res;
+    CGAL_BRANCH_PROFILER_BRANCH(tmp);
+    return b < a.exact();
+  }
+
+  friend bool operator==(const Lazy_exact_nt& a, double b)
+  {
+    CGAL_BRANCH_PROFILER(std::string(" failures/calls to   : ") + std::string(CGAL_PRETTY_FUNCTION), tmp);
+    Uncertain<bool> res = b == a.approx();
+    if (is_certain(res))
+      return res;
+    CGAL_BRANCH_PROFILER_BRANCH(tmp);
+    return b == a.exact();
+  }
+
   // % kills filtering
   Self & operator%=(const Self& b)
   {
@@ -559,84 +625,6 @@ operator%(const Lazy_exact_nt<ET>& a, co
 }
 
 
-
-// Mixed operators with int.
-template <typename ET>
-bool
-operator<(const Lazy_exact_nt<ET>& a, int b)
-{
-  CGAL_BRANCH_PROFILER(std::string(" failures/calls to   : ") + std::string(CGAL_PRETTY_FUNCTION), tmp);
-  Uncertain<bool> res = a.approx() < b;
-  if (is_certain(res))
-    return res;
-  CGAL_BRANCH_PROFILER_BRANCH(tmp);
-  return a.exact() < b;
-}
-
-template <typename ET>
-bool
-operator>(const Lazy_exact_nt<ET>& a, int b)
-{
-  CGAL_BRANCH_PROFILER(std::string(" failures/calls to   : ") + std::string(CGAL_PRETTY_FUNCTION), tmp);
-  Uncertain<bool> res = b < a.approx();
-  if (is_certain(res))
-    return get_certain(res);
-  CGAL_BRANCH_PROFILER_BRANCH(tmp);
-  return b < a.exact();
-}
-
-template <typename ET>
-bool
-operator==(const Lazy_exact_nt<ET>& a, int b)
-{
-  CGAL_BRANCH_PROFILER(std::string(" failures/calls to   : ") + std::string(CGAL_PRETTY_FUNCTION), tmp);
-  Uncertain<bool> res = b == a.approx();
-  if (is_certain(res))
-    return get_certain(res);
-  CGAL_BRANCH_PROFILER_BRANCH(tmp);
-  return b == a.exact();
-}
-
-
-// Mixed operators with double.
-template <typename ET>
-bool
-operator<(const Lazy_exact_nt<ET>& a, double b)
-{
-  CGAL_BRANCH_PROFILER(std::string(" failures/calls to   : ") + std::string(CGAL_PRETTY_FUNCTION), tmp);
-  Uncertain<bool> res = a.approx() < b;
-  if (is_certain(res))
-    return res;
-  CGAL_BRANCH_PROFILER_BRANCH(tmp);
-  return a.exact() < b;
-}
-
-template <typename ET>
-bool
-operator>(const Lazy_exact_nt<ET>& a, double b)
-{
-  CGAL_BRANCH_PROFILER(std::string(" failures/calls to   : ") + std::string(CGAL_PRETTY_FUNCTION), tmp);
-  Uncertain<bool> res = b < a.approx();
-  if (is_certain(res))
-    return res;
-  CGAL_BRANCH_PROFILER_BRANCH(tmp);
-  return b < a.exact();
-}
-
-template <typename ET>
-bool
-operator==(const Lazy_exact_nt<ET>& a, double b)
-{
-  CGAL_BRANCH_PROFILER(std::string(" failures/calls to   : ") + std::string(CGAL_PRETTY_FUNCTION), tmp);
-  Uncertain<bool> res = b == a.approx();
-  if (is_certain(res))
-    return res;
-  CGAL_BRANCH_PROFILER_BRANCH(tmp);
-  return b == a.exact();
-}
-
-
-
 template <typename ET1, typename ET2>
 Lazy_exact_nt< typename Coercion_traits<ET1, ET2>::Type >
 operator+(const Lazy_exact_nt<ET1>& a, const Lazy_exact_nt<ET2>& b)
diff -pruN 5.0.3-1/include/CGAL/Lazy.h 5.1-1/include/CGAL/Lazy.h
--- 5.0.3-1/include/CGAL/Lazy.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Lazy.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/include/CGAL/Lazy.h $
-// $Id: Lazy.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/Lazy.h $
+// $Id: Lazy.h 319383c 2020-05-20T09:47:58+02:00 Laurent Rineau
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -116,9 +116,9 @@ template<class T>inline std::enable_if_t
 
 // For an iterator, exact/approx applies to the objects it points to
 template <class T, class=std::enable_if_t<is_iterator_type<T,std::input_iterator_tag>::value>>
-auto exact(T const& t) {return make_transforming_iterator(t,[](auto const&u){return CGAL::exact(u);});}
+auto exact(T const& t) {return make_transforming_iterator(t,[](auto const&u)->decltype(auto){return CGAL::exact(u);});}
 template <class T, class=std::enable_if_t<is_iterator_type<T,std::input_iterator_tag>::value>>
-auto approx(T const& t) {return make_transforming_iterator(t,[](auto const&u){return CGAL::approx(u);});}
+auto approx(T const& t) {return make_transforming_iterator(t,[](auto const&u)->decltype(auto){return CGAL::approx(u);});}
 template <class T, class=std::enable_if_t<is_iterator_type<T,std::input_iterator_tag>::value>>
 unsigned depth(T const&) {return 1;} // FIXME: depth(*t) would be better when t is valid, but not for end iterators, and the true answer would iterate on the range, but we can't do that with only one iterator... We need to replace iterators with ranges to solve that.
 
@@ -731,6 +731,9 @@ public :
     PTR = new Lazy_rep_0<AT,ET,E2A>(std::move(e));
   }
 
+  friend void swap(Lazy& a, Lazy& b) noexcept
+  { swap(static_cast<Handle&>(a), static_cast<Handle&>(b)); }
+
   const AT& approx() const
   { return ptr()->approx(); }
 
diff -pruN 5.0.3-1/include/CGAL/Lazy_kernel.h 5.1-1/include/CGAL/Lazy_kernel.h
--- 5.0.3-1/include/CGAL/Lazy_kernel.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Lazy_kernel.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/include/CGAL/Lazy_kernel.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/Lazy_kernel.h $
 // $Id: Lazy_kernel.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/LEDA_arithmetic_kernel.h 5.1-1/include/CGAL/LEDA_arithmetic_kernel.h
--- 5.0.3-1/include/CGAL/LEDA_arithmetic_kernel.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/LEDA_arithmetic_kernel.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arithmetic_kernel/include/CGAL/LEDA_arithmetic_kernel.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arithmetic_kernel/include/CGAL/LEDA_arithmetic_kernel.h $
 // $Id: LEDA_arithmetic_kernel.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/LEDA_basic.h 5.1-1/include/CGAL/LEDA_basic.h
--- 5.0.3-1/include/CGAL/LEDA_basic.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/LEDA_basic.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/LEDA/include/CGAL/LEDA_basic.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/LEDA/include/CGAL/LEDA_basic.h $
 // $Id: LEDA_basic.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/leda_bigfloat.h 5.1-1/include/CGAL/leda_bigfloat.h
--- 5.0.3-1/include/CGAL/leda_bigfloat.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/leda_bigfloat.h	2020-09-08 12:15:16.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/leda_bigfloat.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/leda_bigfloat.h $
 // $Id: leda_bigfloat.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/leda_bigfloat_interval.h 5.1-1/include/CGAL/leda_bigfloat_interval.h
--- 5.0.3-1/include/CGAL/leda_bigfloat_interval.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/leda_bigfloat_interval.h	2020-09-08 12:15:16.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/leda_bigfloat_interval.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/leda_bigfloat_interval.h $
 // $Id: leda_bigfloat_interval.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/leda_coercion_traits.h 5.1-1/include/CGAL/leda_coercion_traits.h
--- 5.0.3-1/include/CGAL/leda_coercion_traits.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/leda_coercion_traits.h	2020-09-08 12:15:16.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/leda_coercion_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/leda_coercion_traits.h $
 // $Id: leda_coercion_traits.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/leda_integer.h 5.1-1/include/CGAL/leda_integer.h
--- 5.0.3-1/include/CGAL/leda_integer.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/leda_integer.h	2020-09-08 12:15:16.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/leda_integer.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/leda_integer.h $
 // $Id: leda_integer.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/leda_rational.h 5.1-1/include/CGAL/leda_rational.h
--- 5.0.3-1/include/CGAL/leda_rational.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/leda_rational.h	2020-09-08 12:15:16.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/leda_rational.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/leda_rational.h $
 // $Id: leda_rational.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/leda_real.h 5.1-1/include/CGAL/leda_real.h
--- 5.0.3-1/include/CGAL/leda_real.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/leda_real.h	2020-09-08 12:15:16.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/leda_real.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/leda_real.h $
 // $Id: leda_real.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Level_interval.h 5.1-1/include/CGAL/Level_interval.h
--- 5.0.3-1/include/CGAL/Level_interval.h	2020-08-01 19:02:25.000000000 +0000
+++ 5.1-1/include/CGAL/Level_interval.h	2020-09-08 12:14:59.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Interval_skip_list/include/CGAL/Level_interval.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Interval_skip_list/include/CGAL/Level_interval.h $
 // $Id: Level_interval.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/AABB_tree.h 5.1-1/include/CGAL/license/AABB_tree.h
--- 5.0.3-1/include/CGAL/license/AABB_tree.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/license/AABB_tree.h	2020-09-08 12:14:54.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/AABB_tree.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/AABB_tree.h $
 // $Id: AABB_tree.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Advancing_front_surface_reconstruction.h 5.1-1/include/CGAL/license/Advancing_front_surface_reconstruction.h
--- 5.0.3-1/include/CGAL/license/Advancing_front_surface_reconstruction.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/license/Advancing_front_surface_reconstruction.h	2020-09-08 12:14:54.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Advancing_front_surface_reconstruction.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Advancing_front_surface_reconstruction.h $
 // $Id: Advancing_front_surface_reconstruction.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Alpha_shapes_2.h 5.1-1/include/CGAL/license/Alpha_shapes_2.h
--- 5.0.3-1/include/CGAL/license/Alpha_shapes_2.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/license/Alpha_shapes_2.h	2020-09-08 12:14:54.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Alpha_shapes_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Alpha_shapes_2.h $
 // $Id: Alpha_shapes_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Alpha_shapes_3.h 5.1-1/include/CGAL/license/Alpha_shapes_3.h
--- 5.0.3-1/include/CGAL/license/Alpha_shapes_3.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/license/Alpha_shapes_3.h	2020-09-08 12:14:54.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Alpha_shapes_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Alpha_shapes_3.h $
 // $Id: Alpha_shapes_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Apollonius_graph_2.h 5.1-1/include/CGAL/license/Apollonius_graph_2.h
--- 5.0.3-1/include/CGAL/license/Apollonius_graph_2.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/license/Apollonius_graph_2.h	2020-09-08 12:14:54.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Apollonius_graph_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Apollonius_graph_2.h $
 // $Id: Apollonius_graph_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Arrangement_on_surface_2.h 5.1-1/include/CGAL/license/Arrangement_on_surface_2.h
--- 5.0.3-1/include/CGAL/license/Arrangement_on_surface_2.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/license/Arrangement_on_surface_2.h	2020-09-08 12:14:54.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Arrangement_on_surface_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Arrangement_on_surface_2.h $
 // $Id: Arrangement_on_surface_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Barycentric_coordinates_2.h 5.1-1/include/CGAL/license/Barycentric_coordinates_2.h
--- 5.0.3-1/include/CGAL/license/Barycentric_coordinates_2.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/license/Barycentric_coordinates_2.h	2020-09-08 12:14:54.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Barycentric_coordinates_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Barycentric_coordinates_2.h $
 // $Id: Barycentric_coordinates_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Boolean_set_operations_2.h 5.1-1/include/CGAL/license/Boolean_set_operations_2.h
--- 5.0.3-1/include/CGAL/license/Boolean_set_operations_2.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/license/Boolean_set_operations_2.h	2020-09-08 12:14:54.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Boolean_set_operations_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Boolean_set_operations_2.h $
 // $Id: Boolean_set_operations_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Bounding_volumes.h 5.1-1/include/CGAL/license/Bounding_volumes.h
--- 5.0.3-1/include/CGAL/license/Bounding_volumes.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/license/Bounding_volumes.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Bounding_volumes.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Bounding_volumes.h $
 // $Id: Bounding_volumes.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Box_intersection_d.h 5.1-1/include/CGAL/license/Box_intersection_d.h
--- 5.0.3-1/include/CGAL/license/Box_intersection_d.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/license/Box_intersection_d.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Box_intersection_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Box_intersection_d.h $
 // $Id: Box_intersection_d.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Circular_kernel_2.h 5.1-1/include/CGAL/license/Circular_kernel_2.h
--- 5.0.3-1/include/CGAL/license/Circular_kernel_2.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/license/Circular_kernel_2.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Circular_kernel_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Circular_kernel_2.h $
 // $Id: Circular_kernel_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Circular_kernel_3.h 5.1-1/include/CGAL/license/Circular_kernel_3.h
--- 5.0.3-1/include/CGAL/license/Circular_kernel_3.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/license/Circular_kernel_3.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Circular_kernel_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Circular_kernel_3.h $
 // $Id: Circular_kernel_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Classification.h 5.1-1/include/CGAL/license/Classification.h
--- 5.0.3-1/include/CGAL/license/Classification.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/license/Classification.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Classification.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Classification.h $
 // $Id: Classification.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Cone_spanners_2.h 5.1-1/include/CGAL/license/Cone_spanners_2.h
--- 5.0.3-1/include/CGAL/license/Cone_spanners_2.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/license/Cone_spanners_2.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Cone_spanners_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Cone_spanners_2.h $
 // $Id: Cone_spanners_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Convex_decomposition_3.h 5.1-1/include/CGAL/license/Convex_decomposition_3.h
--- 5.0.3-1/include/CGAL/license/Convex_decomposition_3.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/license/Convex_decomposition_3.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Convex_decomposition_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Convex_decomposition_3.h $
 // $Id: Convex_decomposition_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Convex_hull_2.h 5.1-1/include/CGAL/license/Convex_hull_2.h
--- 5.0.3-1/include/CGAL/license/Convex_hull_2.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/license/Convex_hull_2.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Convex_hull_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Convex_hull_2.h $
 // $Id: Convex_hull_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Convex_hull_3.h 5.1-1/include/CGAL/license/Convex_hull_3.h
--- 5.0.3-1/include/CGAL/license/Convex_hull_3.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/license/Convex_hull_3.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Convex_hull_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Convex_hull_3.h $
 // $Id: Convex_hull_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Convex_hull_d.h 5.1-1/include/CGAL/license/Convex_hull_d.h
--- 5.0.3-1/include/CGAL/license/Convex_hull_d.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/license/Convex_hull_d.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Convex_hull_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Convex_hull_d.h $
 // $Id: Convex_hull_d.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Envelope_2.h 5.1-1/include/CGAL/license/Envelope_2.h
--- 5.0.3-1/include/CGAL/license/Envelope_2.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/license/Envelope_2.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Envelope_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Envelope_2.h $
 // $Id: Envelope_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Envelope_3.h 5.1-1/include/CGAL/license/Envelope_3.h
--- 5.0.3-1/include/CGAL/license/Envelope_3.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/license/Envelope_3.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Envelope_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Envelope_3.h $
 // $Id: Envelope_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/gpl_package_list.txt 5.1-1/include/CGAL/license/gpl_package_list.txt
--- 5.0.3-1/include/CGAL/license/gpl_package_list.txt	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/include/CGAL/license/gpl_package_list.txt	2020-09-08 12:12:19.000000000 +0000
@@ -32,6 +32,7 @@ Minkowski_sum_3 3D Minkowski Sum of Poly
 Nef_2 2D Boolean Operations on Nef Polygons
 Nef_3 3D Boolean Operations on Nef Polyhedra
 Nef_S2 2D Boolean Operations on Nef Polygons Embedded on the Sphere
+Optimal_bounding_box Optimal Bounding Box
 Optimal_transportation_reconstruction_2 Optimal Transportation Curve Reconstruction
 Partition_2 2D Polygon Partitioning
 Periodic_2_triangulation_2 2D Periodic Triangulations
@@ -82,6 +83,7 @@ Surface_mesh_segmentation Triangulated S
 Surface_mesh_shortest_path Triangulated Surface Mesh Shortest Paths
 Surface_mesh_simplification Triangulated Surface Mesh Simplification
 Surface_mesh_skeletonization Triangulated Surface Mesh Skeletonization
+Surface_mesh_topology Surface Mesh Topology
 Surface_sweep_2 2D Intersection of Curves
 TDS_2 2D Triangulation Data Structure
 TDS_3 3D Triangulation Data Structure
@@ -91,3 +93,4 @@ Triangulation_3 3D Triangulations
 Triangulation dD Triangulations
 Visibility_2 2D Visibility Computation
 Voronoi_diagram_2 2D Voronoi Diagram Adaptor
+Tetrahedral_remeshing Tetrahedral Remeshing
diff -pruN 5.0.3-1/include/CGAL/license/GraphicsView.h 5.1-1/include/CGAL/license/GraphicsView.h
--- 5.0.3-1/include/CGAL/license/GraphicsView.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/license/GraphicsView.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/GraphicsView.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/GraphicsView.h $
 // $Id: GraphicsView.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Heat_method_3.h 5.1-1/include/CGAL/license/Heat_method_3.h
--- 5.0.3-1/include/CGAL/license/Heat_method_3.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/license/Heat_method_3.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Heat_method_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Heat_method_3.h $
 // $Id: Heat_method_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Hyperbolic_triangulation_2.h 5.1-1/include/CGAL/license/Hyperbolic_triangulation_2.h
--- 5.0.3-1/include/CGAL/license/Hyperbolic_triangulation_2.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/license/Hyperbolic_triangulation_2.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Hyperbolic_triangulation_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Hyperbolic_triangulation_2.h $
 // $Id: Hyperbolic_triangulation_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Inscribed_areas.h 5.1-1/include/CGAL/license/Inscribed_areas.h
--- 5.0.3-1/include/CGAL/license/Inscribed_areas.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/license/Inscribed_areas.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Inscribed_areas.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Inscribed_areas.h $
 // $Id: Inscribed_areas.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Interpolation.h 5.1-1/include/CGAL/license/Interpolation.h
--- 5.0.3-1/include/CGAL/license/Interpolation.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/license/Interpolation.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Interpolation.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Interpolation.h $
 // $Id: Interpolation.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Interval_skip_list.h 5.1-1/include/CGAL/license/Interval_skip_list.h
--- 5.0.3-1/include/CGAL/license/Interval_skip_list.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/license/Interval_skip_list.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Interval_skip_list.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Interval_skip_list.h $
 // $Id: Interval_skip_list.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Jet_fitting_3.h 5.1-1/include/CGAL/license/Jet_fitting_3.h
--- 5.0.3-1/include/CGAL/license/Jet_fitting_3.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/license/Jet_fitting_3.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Jet_fitting_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Jet_fitting_3.h $
 // $Id: Jet_fitting_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/lgpl.h 5.1-1/include/CGAL/license/lgpl.h
--- 5.0.3-1/include/CGAL/license/lgpl.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/lgpl.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/lgpl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/lgpl.h $
 // $Id: lgpl.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Matrix_search.h 5.1-1/include/CGAL/license/Matrix_search.h
--- 5.0.3-1/include/CGAL/license/Matrix_search.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/license/Matrix_search.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Matrix_search.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Matrix_search.h $
 // $Id: Matrix_search.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Mesh_2.h 5.1-1/include/CGAL/license/Mesh_2.h
--- 5.0.3-1/include/CGAL/license/Mesh_2.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/license/Mesh_2.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Mesh_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Mesh_2.h $
 // $Id: Mesh_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Mesh_3.h 5.1-1/include/CGAL/license/Mesh_3.h
--- 5.0.3-1/include/CGAL/license/Mesh_3.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/license/Mesh_3.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Mesh_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Mesh_3.h $
 // $Id: Mesh_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Minkowski_sum_2.h 5.1-1/include/CGAL/license/Minkowski_sum_2.h
--- 5.0.3-1/include/CGAL/license/Minkowski_sum_2.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/license/Minkowski_sum_2.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Minkowski_sum_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Minkowski_sum_2.h $
 // $Id: Minkowski_sum_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Minkowski_sum_3.h 5.1-1/include/CGAL/license/Minkowski_sum_3.h
--- 5.0.3-1/include/CGAL/license/Minkowski_sum_3.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Minkowski_sum_3.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Minkowski_sum_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Minkowski_sum_3.h $
 // $Id: Minkowski_sum_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Nef_2.h 5.1-1/include/CGAL/license/Nef_2.h
--- 5.0.3-1/include/CGAL/license/Nef_2.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Nef_2.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Nef_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Nef_2.h $
 // $Id: Nef_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Nef_3.h 5.1-1/include/CGAL/license/Nef_3.h
--- 5.0.3-1/include/CGAL/license/Nef_3.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Nef_3.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Nef_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Nef_3.h $
 // $Id: Nef_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Nef_S2.h 5.1-1/include/CGAL/license/Nef_S2.h
--- 5.0.3-1/include/CGAL/license/Nef_S2.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Nef_S2.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Nef_S2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Nef_S2.h $
 // $Id: Nef_S2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Optimal_bounding_box.h 5.1-1/include/CGAL/license/Optimal_bounding_box.h
--- 5.0.3-1/include/CGAL/license/Optimal_bounding_box.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/license/Optimal_bounding_box.h	2020-09-08 12:14:55.000000000 +0000
@@ -0,0 +1,54 @@
+// Copyright (c) 2016  GeometryFactory SARL (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Optimal_bounding_box.h $
+// $Id: Optimal_bounding_box.h 844cd55 2019-12-09T15:43:30+01:00 Mael Rouxel-Labbé
+// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s) : Andreas Fabri
+//
+// Warning: this file is generated, see include/CGAL/licence/README.md
+
+#ifndef CGAL_LICENSE_OPTIMAL_BOUNDING_BOX_H
+#define CGAL_LICENSE_OPTIMAL_BOUNDING_BOX_H
+
+#include <CGAL/config.h>
+#include <CGAL/license.h>
+
+#ifdef CGAL_OPTIMAL_BOUNDING_BOX_COMMERCIAL_LICENSE
+
+#  if CGAL_OPTIMAL_BOUNDING_BOX_COMMERCIAL_LICENSE < CGAL_RELEASE_DATE
+
+#    if defined(CGAL_LICENSE_WARNING)
+
+       CGAL_pragma_warning("Your commercial license for CGAL does not cover "
+                           "this release of the Optimal Bounding Box package.")
+#    endif
+
+#    ifdef CGAL_LICENSE_ERROR
+#      error "Your commercial license for CGAL does not cover this release \
+              of the Optimal Bounding Box package. \
+              You get this error, as you defined CGAL_LICENSE_ERROR."
+#    endif // CGAL_LICENSE_ERROR
+
+#  endif // CGAL_OPTIMAL_BOUNDING_BOX_COMMERCIAL_LICENSE < CGAL_RELEASE_DATE
+
+#else // no CGAL_OPTIMAL_BOUNDING_BOX_COMMERCIAL_LICENSE
+
+#  if defined(CGAL_LICENSE_WARNING)
+     CGAL_pragma_warning("\nThe macro CGAL_OPTIMAL_BOUNDING_BOX_COMMERCIAL_LICENSE is not defined."
+                          "\nYou use the CGAL Optimal Bounding Box package under "
+                          "the terms of the GPLv3+.")
+#  endif // CGAL_LICENSE_WARNING
+
+#  ifdef CGAL_LICENSE_ERROR
+#    error "The macro CGAL_OPTIMAL_BOUNDING_BOX_COMMERCIAL_LICENSE is not defined.\
+            You use the CGAL Optimal Bounding Box package under the terms of \
+            the GPLv3+. You get this error, as you defined CGAL_LICENSE_ERROR."
+#  endif // CGAL_LICENSE_ERROR
+
+#endif // no CGAL_OPTIMAL_BOUNDING_BOX_COMMERCIAL_LICENSE
+
+#endif // CGAL_LICENSE_OPTIMAL_BOUNDING_BOX_H
diff -pruN 5.0.3-1/include/CGAL/license/Optimal_transportation_reconstruction_2.h 5.1-1/include/CGAL/license/Optimal_transportation_reconstruction_2.h
--- 5.0.3-1/include/CGAL/license/Optimal_transportation_reconstruction_2.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Optimal_transportation_reconstruction_2.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Optimal_transportation_reconstruction_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Optimal_transportation_reconstruction_2.h $
 // $Id: Optimal_transportation_reconstruction_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Partition_2.h 5.1-1/include/CGAL/license/Partition_2.h
--- 5.0.3-1/include/CGAL/license/Partition_2.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Partition_2.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Partition_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Partition_2.h $
 // $Id: Partition_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Periodic_2_triangulation_2.h 5.1-1/include/CGAL/license/Periodic_2_triangulation_2.h
--- 5.0.3-1/include/CGAL/license/Periodic_2_triangulation_2.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Periodic_2_triangulation_2.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Periodic_2_triangulation_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Periodic_2_triangulation_2.h $
 // $Id: Periodic_2_triangulation_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Periodic_3_mesh_3.h 5.1-1/include/CGAL/license/Periodic_3_mesh_3.h
--- 5.0.3-1/include/CGAL/license/Periodic_3_mesh_3.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Periodic_3_mesh_3.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Periodic_3_mesh_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Periodic_3_mesh_3.h $
 // $Id: Periodic_3_mesh_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Periodic_3_triangulation_3.h 5.1-1/include/CGAL/license/Periodic_3_triangulation_3.h
--- 5.0.3-1/include/CGAL/license/Periodic_3_triangulation_3.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Periodic_3_triangulation_3.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Periodic_3_triangulation_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Periodic_3_triangulation_3.h $
 // $Id: Periodic_3_triangulation_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Periodic_4_hyperbolic_triangulation_2.h 5.1-1/include/CGAL/license/Periodic_4_hyperbolic_triangulation_2.h
--- 5.0.3-1/include/CGAL/license/Periodic_4_hyperbolic_triangulation_2.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Periodic_4_hyperbolic_triangulation_2.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Periodic_4_hyperbolic_triangulation_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Periodic_4_hyperbolic_triangulation_2.h $
 // $Id: Periodic_4_hyperbolic_triangulation_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Point_set_2.h 5.1-1/include/CGAL/license/Point_set_2.h
--- 5.0.3-1/include/CGAL/license/Point_set_2.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Point_set_2.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Point_set_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Point_set_2.h $
 // $Id: Point_set_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Point_set_3.h 5.1-1/include/CGAL/license/Point_set_3.h
--- 5.0.3-1/include/CGAL/license/Point_set_3.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Point_set_3.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Point_set_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Point_set_3.h $
 // $Id: Point_set_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Point_set_processing_3.h 5.1-1/include/CGAL/license/Point_set_processing_3.h
--- 5.0.3-1/include/CGAL/license/Point_set_processing_3.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Point_set_processing_3.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Point_set_processing_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Point_set_processing_3.h $
 // $Id: Point_set_processing_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Poisson_surface_reconstruction_3.h 5.1-1/include/CGAL/license/Poisson_surface_reconstruction_3.h
--- 5.0.3-1/include/CGAL/license/Poisson_surface_reconstruction_3.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Poisson_surface_reconstruction_3.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Poisson_surface_reconstruction_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Poisson_surface_reconstruction_3.h $
 // $Id: Poisson_surface_reconstruction_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Polygonal_surface_reconstruction.h 5.1-1/include/CGAL/license/Polygonal_surface_reconstruction.h
--- 5.0.3-1/include/CGAL/license/Polygonal_surface_reconstruction.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Polygonal_surface_reconstruction.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Polygonal_surface_reconstruction.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Polygonal_surface_reconstruction.h $
 // $Id: Polygonal_surface_reconstruction.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Polygon_mesh_processing/collision_detection.h 5.1-1/include/CGAL/license/Polygon_mesh_processing/collision_detection.h
--- 5.0.3-1/include/CGAL/license/Polygon_mesh_processing/collision_detection.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Polygon_mesh_processing/collision_detection.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Polygon_mesh_processing/collision_detection.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Polygon_mesh_processing/collision_detection.h $
 // $Id: collision_detection.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Polygon_mesh_processing/Compute_normal.h 5.1-1/include/CGAL/license/Polygon_mesh_processing/Compute_normal.h
--- 5.0.3-1/include/CGAL/license/Polygon_mesh_processing/Compute_normal.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Polygon_mesh_processing/Compute_normal.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Polygon_mesh_processing/Compute_normal.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Polygon_mesh_processing/Compute_normal.h $
 // $Id: Compute_normal.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Polygon_mesh_processing/connected_components.h 5.1-1/include/CGAL/license/Polygon_mesh_processing/connected_components.h
--- 5.0.3-1/include/CGAL/license/Polygon_mesh_processing/connected_components.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Polygon_mesh_processing/connected_components.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Polygon_mesh_processing/connected_components.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Polygon_mesh_processing/connected_components.h $
 // $Id: connected_components.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Polygon_mesh_processing/corefinement.h 5.1-1/include/CGAL/license/Polygon_mesh_processing/corefinement.h
--- 5.0.3-1/include/CGAL/license/Polygon_mesh_processing/corefinement.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Polygon_mesh_processing/corefinement.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Polygon_mesh_processing/corefinement.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Polygon_mesh_processing/corefinement.h $
 // $Id: corefinement.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Polygon_mesh_processing/core.h 5.1-1/include/CGAL/license/Polygon_mesh_processing/core.h
--- 5.0.3-1/include/CGAL/license/Polygon_mesh_processing/core.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Polygon_mesh_processing/core.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Polygon_mesh_processing/core.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Polygon_mesh_processing/core.h $
 // $Id: core.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Polygon_mesh_processing/detect_features.h 5.1-1/include/CGAL/license/Polygon_mesh_processing/detect_features.h
--- 5.0.3-1/include/CGAL/license/Polygon_mesh_processing/detect_features.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Polygon_mesh_processing/detect_features.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Polygon_mesh_processing/detect_features.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Polygon_mesh_processing/detect_features.h $
 // $Id: detect_features.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Polygon_mesh_processing/distance.h 5.1-1/include/CGAL/license/Polygon_mesh_processing/distance.h
--- 5.0.3-1/include/CGAL/license/Polygon_mesh_processing/distance.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Polygon_mesh_processing/distance.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Polygon_mesh_processing/distance.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Polygon_mesh_processing/distance.h $
 // $Id: distance.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Polygon_mesh_processing/locate.h 5.1-1/include/CGAL/license/Polygon_mesh_processing/locate.h
--- 5.0.3-1/include/CGAL/license/Polygon_mesh_processing/locate.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Polygon_mesh_processing/locate.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Polygon_mesh_processing/locate.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Polygon_mesh_processing/locate.h $
 // $Id: locate.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Polygon_mesh_processing/measure.h 5.1-1/include/CGAL/license/Polygon_mesh_processing/measure.h
--- 5.0.3-1/include/CGAL/license/Polygon_mesh_processing/measure.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Polygon_mesh_processing/measure.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Polygon_mesh_processing/measure.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Polygon_mesh_processing/measure.h $
 // $Id: measure.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Polygon_mesh_processing/meshing_hole_filling.h 5.1-1/include/CGAL/license/Polygon_mesh_processing/meshing_hole_filling.h
--- 5.0.3-1/include/CGAL/license/Polygon_mesh_processing/meshing_hole_filling.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Polygon_mesh_processing/meshing_hole_filling.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Polygon_mesh_processing/meshing_hole_filling.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Polygon_mesh_processing/meshing_hole_filling.h $
 // $Id: meshing_hole_filling.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Polygon_mesh_processing/miscellaneous.h 5.1-1/include/CGAL/license/Polygon_mesh_processing/miscellaneous.h
--- 5.0.3-1/include/CGAL/license/Polygon_mesh_processing/miscellaneous.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Polygon_mesh_processing/miscellaneous.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Polygon_mesh_processing/miscellaneous.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Polygon_mesh_processing/miscellaneous.h $
 // $Id: miscellaneous.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Polygon_mesh_processing/orientation.h 5.1-1/include/CGAL/license/Polygon_mesh_processing/orientation.h
--- 5.0.3-1/include/CGAL/license/Polygon_mesh_processing/orientation.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Polygon_mesh_processing/orientation.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Polygon_mesh_processing/orientation.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Polygon_mesh_processing/orientation.h $
 // $Id: orientation.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Polygon_mesh_processing/predicate.h 5.1-1/include/CGAL/license/Polygon_mesh_processing/predicate.h
--- 5.0.3-1/include/CGAL/license/Polygon_mesh_processing/predicate.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Polygon_mesh_processing/predicate.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Polygon_mesh_processing/predicate.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Polygon_mesh_processing/predicate.h $
 // $Id: predicate.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Polygon_mesh_processing/repair.h 5.1-1/include/CGAL/license/Polygon_mesh_processing/repair.h
--- 5.0.3-1/include/CGAL/license/Polygon_mesh_processing/repair.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Polygon_mesh_processing/repair.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Polygon_mesh_processing/repair.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Polygon_mesh_processing/repair.h $
 // $Id: repair.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Polygon_mesh_processing.h 5.1-1/include/CGAL/license/Polygon_mesh_processing.h
--- 5.0.3-1/include/CGAL/license/Polygon_mesh_processing.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Polygon_mesh_processing.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Polygon_mesh_processing.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Polygon_mesh_processing.h $
 // $Id: Polygon_mesh_processing.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Polyhedron.h 5.1-1/include/CGAL/license/Polyhedron.h
--- 5.0.3-1/include/CGAL/license/Polyhedron.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Polyhedron.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Polyhedron.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Polyhedron.h $
 // $Id: Polyhedron.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Polyline_simplification_2.h 5.1-1/include/CGAL/license/Polyline_simplification_2.h
--- 5.0.3-1/include/CGAL/license/Polyline_simplification_2.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Polyline_simplification_2.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Polyline_simplification_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Polyline_simplification_2.h $
 // $Id: Polyline_simplification_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Polytope_distance_d.h 5.1-1/include/CGAL/license/Polytope_distance_d.h
--- 5.0.3-1/include/CGAL/license/Polytope_distance_d.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Polytope_distance_d.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Polytope_distance_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Polytope_distance_d.h $
 // $Id: Polytope_distance_d.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Principal_component_analysis.h 5.1-1/include/CGAL/license/Principal_component_analysis.h
--- 5.0.3-1/include/CGAL/license/Principal_component_analysis.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Principal_component_analysis.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Principal_component_analysis.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Principal_component_analysis.h $
 // $Id: Principal_component_analysis.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/QP_solver.h 5.1-1/include/CGAL/license/QP_solver.h
--- 5.0.3-1/include/CGAL/license/QP_solver.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/QP_solver.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/QP_solver.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/QP_solver.h $
 // $Id: QP_solver.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Ridges_3.h 5.1-1/include/CGAL/license/Ridges_3.h
--- 5.0.3-1/include/CGAL/license/Ridges_3.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Ridges_3.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Ridges_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Ridges_3.h $
 // $Id: Ridges_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Scale_space_reconstruction_3.h 5.1-1/include/CGAL/license/Scale_space_reconstruction_3.h
--- 5.0.3-1/include/CGAL/license/Scale_space_reconstruction_3.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Scale_space_reconstruction_3.h	2020-09-08 12:14:55.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Scale_space_reconstruction_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Scale_space_reconstruction_3.h $
 // $Id: Scale_space_reconstruction_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/SearchStructures.h 5.1-1/include/CGAL/license/SearchStructures.h
--- 5.0.3-1/include/CGAL/license/SearchStructures.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/SearchStructures.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/SearchStructures.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/SearchStructures.h $
 // $Id: SearchStructures.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Segment_Delaunay_graph_2.h 5.1-1/include/CGAL/license/Segment_Delaunay_graph_2.h
--- 5.0.3-1/include/CGAL/license/Segment_Delaunay_graph_2.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Segment_Delaunay_graph_2.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Segment_Delaunay_graph_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Segment_Delaunay_graph_2.h $
 // $Id: Segment_Delaunay_graph_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Segment_Delaunay_graph_Linf_2.h 5.1-1/include/CGAL/license/Segment_Delaunay_graph_Linf_2.h
--- 5.0.3-1/include/CGAL/license/Segment_Delaunay_graph_Linf_2.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Segment_Delaunay_graph_Linf_2.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Segment_Delaunay_graph_Linf_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Segment_Delaunay_graph_Linf_2.h $
 // $Id: Segment_Delaunay_graph_Linf_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Set_movable_separability_2.h 5.1-1/include/CGAL/license/Set_movable_separability_2.h
--- 5.0.3-1/include/CGAL/license/Set_movable_separability_2.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Set_movable_separability_2.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Set_movable_separability_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Set_movable_separability_2.h $
 // $Id: Set_movable_separability_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Shape_detection.h 5.1-1/include/CGAL/license/Shape_detection.h
--- 5.0.3-1/include/CGAL/license/Shape_detection.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Shape_detection.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Shape_detection.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Shape_detection.h $
 // $Id: Shape_detection.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Skin_surface_3.h 5.1-1/include/CGAL/license/Skin_surface_3.h
--- 5.0.3-1/include/CGAL/license/Skin_surface_3.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Skin_surface_3.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Skin_surface_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Skin_surface_3.h $
 // $Id: Skin_surface_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Snap_rounding_2.h 5.1-1/include/CGAL/license/Snap_rounding_2.h
--- 5.0.3-1/include/CGAL/license/Snap_rounding_2.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Snap_rounding_2.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Snap_rounding_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Snap_rounding_2.h $
 // $Id: Snap_rounding_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Spatial_searching.h 5.1-1/include/CGAL/license/Spatial_searching.h
--- 5.0.3-1/include/CGAL/license/Spatial_searching.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Spatial_searching.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Spatial_searching.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Spatial_searching.h $
 // $Id: Spatial_searching.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Straight_skeleton_2.h 5.1-1/include/CGAL/license/Straight_skeleton_2.h
--- 5.0.3-1/include/CGAL/license/Straight_skeleton_2.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Straight_skeleton_2.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Straight_skeleton_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Straight_skeleton_2.h $
 // $Id: Straight_skeleton_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Stream_lines_2.h 5.1-1/include/CGAL/license/Stream_lines_2.h
--- 5.0.3-1/include/CGAL/license/Stream_lines_2.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Stream_lines_2.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Stream_lines_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Stream_lines_2.h $
 // $Id: Stream_lines_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Surface_mesh_approximation.h 5.1-1/include/CGAL/license/Surface_mesh_approximation.h
--- 5.0.3-1/include/CGAL/license/Surface_mesh_approximation.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Surface_mesh_approximation.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Surface_mesh_approximation.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Surface_mesh_approximation.h $
 // $Id: Surface_mesh_approximation.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Surface_mesh_deformation.h 5.1-1/include/CGAL/license/Surface_mesh_deformation.h
--- 5.0.3-1/include/CGAL/license/Surface_mesh_deformation.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Surface_mesh_deformation.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Surface_mesh_deformation.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Surface_mesh_deformation.h $
 // $Id: Surface_mesh_deformation.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Surface_mesher.h 5.1-1/include/CGAL/license/Surface_mesher.h
--- 5.0.3-1/include/CGAL/license/Surface_mesher.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Surface_mesher.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Surface_mesher.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Surface_mesher.h $
 // $Id: Surface_mesher.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Surface_mesh.h 5.1-1/include/CGAL/license/Surface_mesh.h
--- 5.0.3-1/include/CGAL/license/Surface_mesh.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Surface_mesh.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Surface_mesh.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Surface_mesh.h $
 // $Id: Surface_mesh.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Surface_mesh_parameterization.h 5.1-1/include/CGAL/license/Surface_mesh_parameterization.h
--- 5.0.3-1/include/CGAL/license/Surface_mesh_parameterization.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Surface_mesh_parameterization.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Surface_mesh_parameterization.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Surface_mesh_parameterization.h $
 // $Id: Surface_mesh_parameterization.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Surface_mesh_segmentation.h 5.1-1/include/CGAL/license/Surface_mesh_segmentation.h
--- 5.0.3-1/include/CGAL/license/Surface_mesh_segmentation.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Surface_mesh_segmentation.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Surface_mesh_segmentation.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Surface_mesh_segmentation.h $
 // $Id: Surface_mesh_segmentation.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Surface_mesh_shortest_path.h 5.1-1/include/CGAL/license/Surface_mesh_shortest_path.h
--- 5.0.3-1/include/CGAL/license/Surface_mesh_shortest_path.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Surface_mesh_shortest_path.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Surface_mesh_shortest_path.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Surface_mesh_shortest_path.h $
 // $Id: Surface_mesh_shortest_path.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Surface_mesh_simplification.h 5.1-1/include/CGAL/license/Surface_mesh_simplification.h
--- 5.0.3-1/include/CGAL/license/Surface_mesh_simplification.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Surface_mesh_simplification.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Surface_mesh_simplification.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Surface_mesh_simplification.h $
 // $Id: Surface_mesh_simplification.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Surface_mesh_skeletonization.h 5.1-1/include/CGAL/license/Surface_mesh_skeletonization.h
--- 5.0.3-1/include/CGAL/license/Surface_mesh_skeletonization.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Surface_mesh_skeletonization.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Surface_mesh_skeletonization.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Surface_mesh_skeletonization.h $
 // $Id: Surface_mesh_skeletonization.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Surface_mesh_topology.h 5.1-1/include/CGAL/license/Surface_mesh_topology.h
--- 5.0.3-1/include/CGAL/license/Surface_mesh_topology.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/license/Surface_mesh_topology.h	2020-09-08 12:14:56.000000000 +0000
@@ -0,0 +1,54 @@
+// Copyright (c) 2016  GeometryFactory SARL (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Surface_mesh_topology.h $
+// $Id: Surface_mesh_topology.h 90457b3 2019-11-26T14:21:39+01:00 Guillaume Damiand
+// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s) : Andreas Fabri
+//
+// Warning: this file is generated, see include/CGAL/licence/README.md
+
+#ifndef CGAL_LICENSE_SURFACE_MESH_TOPOLOGY_H
+#define CGAL_LICENSE_SURFACE_MESH_TOPOLOGY_H
+
+#include <CGAL/config.h>
+#include <CGAL/license.h>
+
+#ifdef CGAL_SURFACE_MESH_TOPOLOGY_COMMERCIAL_LICENSE
+
+#  if CGAL_SURFACE_MESH_TOPOLOGY_COMMERCIAL_LICENSE < CGAL_RELEASE_DATE
+
+#    if defined(CGAL_LICENSE_WARNING)
+
+       CGAL_pragma_warning("Your commercial license for CGAL does not cover "
+                           "this release of the Surface Mesh Topology package.")
+#    endif
+
+#    ifdef CGAL_LICENSE_ERROR
+#      error "Your commercial license for CGAL does not cover this release \
+              of the Surface Mesh Topology package. \
+              You get this error, as you defined CGAL_LICENSE_ERROR."
+#    endif // CGAL_LICENSE_ERROR
+
+#  endif // CGAL_SURFACE_MESH_TOPOLOGY_COMMERCIAL_LICENSE < CGAL_RELEASE_DATE
+
+#else // no CGAL_SURFACE_MESH_TOPOLOGY_COMMERCIAL_LICENSE
+
+#  if defined(CGAL_LICENSE_WARNING)
+     CGAL_pragma_warning("\nThe macro CGAL_SURFACE_MESH_TOPOLOGY_COMMERCIAL_LICENSE is not defined."
+                          "\nYou use the CGAL Surface Mesh Topology package under "
+                          "the terms of the GPLv3+.")
+#  endif // CGAL_LICENSE_WARNING
+
+#  ifdef CGAL_LICENSE_ERROR
+#    error "The macro CGAL_SURFACE_MESH_TOPOLOGY_COMMERCIAL_LICENSE is not defined.\
+            You use the CGAL Surface Mesh Topology package under the terms of \
+            the GPLv3+. You get this error, as you defined CGAL_LICENSE_ERROR."
+#  endif // CGAL_LICENSE_ERROR
+
+#endif // no CGAL_SURFACE_MESH_TOPOLOGY_COMMERCIAL_LICENSE
+
+#endif // CGAL_LICENSE_SURFACE_MESH_TOPOLOGY_H
diff -pruN 5.0.3-1/include/CGAL/license/Surface_sweep_2.h 5.1-1/include/CGAL/license/Surface_sweep_2.h
--- 5.0.3-1/include/CGAL/license/Surface_sweep_2.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Surface_sweep_2.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Surface_sweep_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Surface_sweep_2.h $
 // $Id: Surface_sweep_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/TDS_2.h 5.1-1/include/CGAL/license/TDS_2.h
--- 5.0.3-1/include/CGAL/license/TDS_2.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/TDS_2.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/TDS_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/TDS_2.h $
 // $Id: TDS_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/TDS_3.h 5.1-1/include/CGAL/license/TDS_3.h
--- 5.0.3-1/include/CGAL/license/TDS_3.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/TDS_3.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/TDS_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/TDS_3.h $
 // $Id: TDS_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Tetrahedral_remeshing.h 5.1-1/include/CGAL/license/Tetrahedral_remeshing.h
--- 5.0.3-1/include/CGAL/license/Tetrahedral_remeshing.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/license/Tetrahedral_remeshing.h	2020-09-08 12:14:56.000000000 +0000
@@ -0,0 +1,54 @@
+// Copyright (c) 2016  GeometryFactory SARL (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Tetrahedral_remeshing.h $
+// $Id: Tetrahedral_remeshing.h 825f77b 2020-04-10T14:14:30+02:00 Sébastien Loriot
+// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s) : Andreas Fabri
+//
+// Warning: this file is generated, see include/CGAL/licence/README.md
+
+#ifndef CGAL_LICENSE_TETRAHEDRAL_REMESHING_H
+#define CGAL_LICENSE_TETRAHEDRAL_REMESHING_H
+
+#include <CGAL/config.h>
+#include <CGAL/license.h>
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_COMMERCIAL_LICENSE
+
+#  if CGAL_TETRAHEDRAL_REMESHING_COMMERCIAL_LICENSE < CGAL_RELEASE_DATE
+
+#    if defined(CGAL_LICENSE_WARNING)
+
+       CGAL_pragma_warning("Your commercial license for CGAL does not cover "
+                           "this release of the Tetrahedral Remeshing package.")
+#    endif
+
+#    ifdef CGAL_LICENSE_ERROR
+#      error "Your commercial license for CGAL does not cover this release \
+              of the Tetrahedral Remeshing package. \
+              You get this error, as you defined CGAL_LICENSE_ERROR."
+#    endif // CGAL_LICENSE_ERROR
+
+#  endif // CGAL_TETRAHEDRAL_REMESHING_COMMERCIAL_LICENSE < CGAL_RELEASE_DATE
+
+#else // no CGAL_TETRAHEDRAL_REMESHING_COMMERCIAL_LICENSE
+
+#  if defined(CGAL_LICENSE_WARNING)
+     CGAL_pragma_warning("\nThe macro CGAL_TETRAHEDRAL_REMESHING_COMMERCIAL_LICENSE is not defined."
+                          "\nYou use the CGAL Tetrahedral Remeshing package under "
+                          "the terms of the GPLv3+.")
+#  endif // CGAL_LICENSE_WARNING
+
+#  ifdef CGAL_LICENSE_ERROR
+#    error "The macro CGAL_TETRAHEDRAL_REMESHING_COMMERCIAL_LICENSE is not defined.\
+            You use the CGAL Tetrahedral Remeshing package under the terms of \
+            the GPLv3+. You get this error, as you defined CGAL_LICENSE_ERROR."
+#  endif // CGAL_LICENSE_ERROR
+
+#endif // no CGAL_TETRAHEDRAL_REMESHING_COMMERCIAL_LICENSE
+
+#endif // CGAL_LICENSE_TETRAHEDRAL_REMESHING_H
diff -pruN 5.0.3-1/include/CGAL/license/Three.h 5.1-1/include/CGAL/license/Three.h
--- 5.0.3-1/include/CGAL/license/Three.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Three.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Three.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Three.h $
 // $Id: Three.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Triangulation_2.h 5.1-1/include/CGAL/license/Triangulation_2.h
--- 5.0.3-1/include/CGAL/license/Triangulation_2.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Triangulation_2.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Triangulation_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Triangulation_2.h $
 // $Id: Triangulation_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Triangulation_3.h 5.1-1/include/CGAL/license/Triangulation_3.h
--- 5.0.3-1/include/CGAL/license/Triangulation_3.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Triangulation_3.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Triangulation_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Triangulation_3.h $
 // $Id: Triangulation_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Triangulation.h 5.1-1/include/CGAL/license/Triangulation.h
--- 5.0.3-1/include/CGAL/license/Triangulation.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Triangulation.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Triangulation.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Triangulation.h $
 // $Id: Triangulation.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Visibility_2.h 5.1-1/include/CGAL/license/Visibility_2.h
--- 5.0.3-1/include/CGAL/license/Visibility_2.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Visibility_2.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Visibility_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Visibility_2.h $
 // $Id: Visibility_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license/Voronoi_diagram_2.h 5.1-1/include/CGAL/license/Voronoi_diagram_2.h
--- 5.0.3-1/include/CGAL/license/Voronoi_diagram_2.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/license/Voronoi_diagram_2.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license/Voronoi_diagram_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license/Voronoi_diagram_2.h $
 // $Id: Voronoi_diagram_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/license.h 5.1-1/include/CGAL/license.h
--- 5.0.3-1/include/CGAL/license.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/license.h	2020-09-08 12:14:54.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/license.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/license.h $
 // $Id: license.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Lightweight_vector_3.h 5.1-1/include/CGAL/Lightweight_vector_3.h
--- 5.0.3-1/include/CGAL/Lightweight_vector_3.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Lightweight_vector_3.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Poisson_surface_reconstruction_3/include/CGAL/Lightweight_vector_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Poisson_surface_reconstruction_3/include/CGAL/Lightweight_vector_3.h $
 // $Id: Lightweight_vector_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Line_2.h 5.1-1/include/CGAL/Line_2.h
--- 5.0.3-1/include/CGAL/Line_2.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Line_2.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Line_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Line_2.h $
 // $Id: Line_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Line_3.h 5.1-1/include/CGAL/Line_3.h
--- 5.0.3-1/include/CGAL/Line_3.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Line_3.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Line_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Line_3.h $
 // $Id: Line_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Linear_algebraCd.h 5.1-1/include/CGAL/Linear_algebraCd.h
--- 5.0.3-1/include/CGAL/Linear_algebraCd.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Linear_algebraCd.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Linear_algebraCd.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Linear_algebraCd.h $
 // $Id: Linear_algebraCd.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Linear_algebraHd.h 5.1-1/include/CGAL/Linear_algebraHd.h
--- 5.0.3-1/include/CGAL/Linear_algebraHd.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Linear_algebraHd.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Linear_algebraHd.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Linear_algebraHd.h $
 // $Id: Linear_algebraHd.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Line_arc_2.h 5.1-1/include/CGAL/Line_arc_2.h
--- 5.0.3-1/include/CGAL/Line_arc_2.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Line_arc_2.h	2020-09-08 12:14:46.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_2/include/CGAL/Line_arc_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_2/include/CGAL/Line_arc_2.h $
 // $Id: Line_arc_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Line_arc_3.h 5.1-1/include/CGAL/Line_arc_3.h
--- 5.0.3-1/include/CGAL/Line_arc_3.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Line_arc_3.h	2020-09-08 12:14:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_3/include/CGAL/Line_arc_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_3/include/CGAL/Line_arc_3.h $
 // $Id: Line_arc_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Linear_cell_complex_base.h 5.1-1/include/CGAL/Linear_cell_complex_base.h
--- 5.0.3-1/include/CGAL/Linear_cell_complex_base.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Linear_cell_complex_base.h	2020-09-08 12:15:05.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Linear_cell_complex/include/CGAL/Linear_cell_complex_base.h $
-// $Id: Linear_cell_complex_base.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Linear_cell_complex/include/CGAL/Linear_cell_complex_base.h $
+// $Id: Linear_cell_complex_base.h daab969 2020-04-08T09:23:59+02:00 Guillaume Damiand
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
@@ -12,6 +12,7 @@
 #ifndef CGAL_LINEAR_CELL_COMPLEX_BASE_H
 #define CGAL_LINEAR_CELL_COMPLEX_BASE_H 1
 
+#include <CGAL/Linear_cell_complex_fwd.h>
 #include <CGAL/Combinatorial_map_functors.h>
 #include <CGAL/internal/Combinatorial_map_internal_functors.h>
 #include <CGAL/Linear_cell_complex_operations.h>
@@ -42,21 +43,22 @@ namespace CGAL {
              class Alloc_,
              template<unsigned int,class,class,class,class>
              class Map,
-             class Refs,
+             class Refs_,
              class Storage_>
   class Linear_cell_complex_base:
-    public Map<d_, Refs, Items_, Alloc_, Storage_>
+    public Map<d_, Refs_, Items_, Alloc_, Storage_>
   {
   public:
     typedef Linear_cell_complex_base<d_, ambient_dim,
                                 Traits_, Items_, Alloc_, Map,
-                                Refs, Storage_>  Self;
-    typedef Map<d_, Refs, Items_, Alloc_, Storage_> Base;
+                                Refs_, Storage_>  Self;
+    typedef Map<d_, Refs_, Items_, Alloc_, Storage_> Base;
 
-    typedef Traits_ Traits;
-    typedef Items_  Items;
-    typedef Alloc_  Alloc;
+    typedef Traits_  Traits;
+    typedef Items_   Items;
+    typedef Alloc_   Alloc;
     typedef Storage_ Storage;
+    typedef Refs_     Refs;
 
     static const unsigned int ambient_dimension = ambient_dim;
     static const unsigned int dimension = Base::dimension;
@@ -110,6 +112,7 @@ namespace CGAL {
     using Base::create_dart;
     using Base::attribute;
     using Base::null_handle;
+    using Base::null_dart_handle;
     using Base::point_of_vertex_attribute;
     using Base::other_extremity;
     using Base::darts;
@@ -147,26 +150,48 @@ namespace CGAL {
     Linear_cell_complex_base(const Self& alcc) : Base(alcc)
     {}
 
-    template < class LCC2 >
-    Linear_cell_complex_base(const LCC2& alcc) : Base(alcc)
-    {}
-
-    template < class LCC2, typename Converters >
-    Linear_cell_complex_base(const LCC2& alcc, Converters& converters) :
+    template <unsigned int d2,  unsigned int ambient_dim2, class Traits2,
+              class Items2, class Alloc2,
+              template<unsigned int,class,class,class,class> class CMap2,
+              class Refs2, class Storage2>
+    Linear_cell_complex_base
+    (const Linear_cell_complex_base<d2, ambient_dim2,
+     Traits2, Items2, Alloc2, CMap2, Refs2, Storage2>& alcc) : Base(alcc)
+    {}
+
+    template <unsigned int d2,  unsigned int ambient_dim2, class Traits2,
+              class Items2, class Alloc2,
+              template<unsigned int,class,class,class,class> class CMap2,
+              class Refs2,
+              class Storage2, typename Converters>
+    Linear_cell_complex_base
+    (const Linear_cell_complex_base<d2, ambient_dim2, Traits2, Items2,
+     Alloc2, CMap2, Refs2, Storage2>& alcc, Converters& converters) :
       Base(alcc, converters)
     {}
 
-    template < class LCC2, typename Converters, typename DartInfoConverter >
-    Linear_cell_complex_base(const LCC2& alcc, Converters& converters,
-                             const DartInfoConverter& dartinfoconverter) :
+    template <unsigned int d2,  unsigned int ambient_dim2, class Traits2,
+              class Items2, class Alloc2,
+              template<unsigned int,class,class,class,class> class CMap2,
+              class Refs2, class Storage2, typename Converters,
+              typename DartInfoConverter>
+    Linear_cell_complex_base
+    (const Linear_cell_complex_base<d2, ambient_dim2, Traits2, Items2,
+     Alloc2, CMap2, Refs2, Storage2>& alcc, Converters& converters,
+     const DartInfoConverter& dartinfoconverter) :
       Base(alcc, converters, dartinfoconverter)
     {}
 
-    template < class LCC2, typename Converters, typename DartInfoConverter,
-               typename Pointconverter >
-    Linear_cell_complex_base(const LCC2& alcc, Converters& converters,
-                             const DartInfoConverter& dartinfoconverter,
-                             const Pointconverter& pointconverter) :
+    template <unsigned int d2,  unsigned int ambient_dim2, class Traits2,
+              class Items2, class Alloc2,
+              template<unsigned int,class,class,class,class> class CMap2,
+              class Refs2, class Storage2, typename Converters,
+              typename DartInfoConverter, typename Pointconverter>
+    Linear_cell_complex_base
+    (const Linear_cell_complex_base<d2, ambient_dim2, Traits2, Items2,
+     Alloc2, CMap2, Refs2, Storage2>& alcc, Converters& converters,
+     const DartInfoConverter& dartinfoconverter,
+     const Pointconverter& pointconverter) :
       Base(alcc, converters, dartinfoconverter, pointconverter)
     {}
 
diff -pruN 5.0.3-1/include/CGAL/Linear_cell_complex_bgl_min_items.h 5.1-1/include/CGAL/Linear_cell_complex_bgl_min_items.h
--- 5.0.3-1/include/CGAL/Linear_cell_complex_bgl_min_items.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Linear_cell_complex_bgl_min_items.h	2020-09-08 12:15:05.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Linear_cell_complex/include/CGAL/Linear_cell_complex_bgl_min_items.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Linear_cell_complex/include/CGAL/Linear_cell_complex_bgl_min_items.h $
 // $Id: Linear_cell_complex_bgl_min_items.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Linear_cell_complex_constructors.h 5.1-1/include/CGAL/Linear_cell_complex_constructors.h
--- 5.0.3-1/include/CGAL/Linear_cell_complex_constructors.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Linear_cell_complex_constructors.h	2020-09-08 12:15:05.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Linear_cell_complex/include/CGAL/Linear_cell_complex_constructors.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Linear_cell_complex/include/CGAL/Linear_cell_complex_constructors.h $
 // $Id: Linear_cell_complex_constructors.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Linear_cell_complex_for_bgl_combinatorial_map_helper.h 5.1-1/include/CGAL/Linear_cell_complex_for_bgl_combinatorial_map_helper.h
--- 5.0.3-1/include/CGAL/Linear_cell_complex_for_bgl_combinatorial_map_helper.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Linear_cell_complex_for_bgl_combinatorial_map_helper.h	2020-09-08 12:15:05.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Linear_cell_complex/include/CGAL/Linear_cell_complex_for_bgl_combinatorial_map_helper.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Linear_cell_complex/include/CGAL/Linear_cell_complex_for_bgl_combinatorial_map_helper.h $
 // $Id: Linear_cell_complex_for_bgl_combinatorial_map_helper.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Linear_cell_complex_for_combinatorial_map.h 5.1-1/include/CGAL/Linear_cell_complex_for_combinatorial_map.h
--- 5.0.3-1/include/CGAL/Linear_cell_complex_for_combinatorial_map.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Linear_cell_complex_for_combinatorial_map.h	2020-09-08 12:15:05.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Linear_cell_complex/include/CGAL/Linear_cell_complex_for_combinatorial_map.h $
-// $Id: Linear_cell_complex_for_combinatorial_map.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Linear_cell_complex/include/CGAL/Linear_cell_complex_for_combinatorial_map.h $
+// $Id: Linear_cell_complex_for_combinatorial_map.h 0308d1a 2020-03-27T18:35:15+01:00 Guillaume Damiand
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
@@ -12,11 +12,13 @@
 #ifndef CGAL_LINEAR_CELL_COMPLEX_FOR_COMBINATORIAL_MAP_H
 #define CGAL_LINEAR_CELL_COMPLEX_FOR_COMBINATORIAL_MAP_H 1
 
+#include <CGAL/Linear_cell_complex_fwd.h>
 #include <CGAL/Linear_cell_complex_base.h>
 #include <CGAL/Linear_cell_complex_traits.h>
 #include <CGAL/Linear_cell_complex_min_items.h>
 #include <CGAL/Combinatorial_map.h>
 #include <CGAL/CMap_linear_cell_complex_storages.h>
+#include <CGAL/boost/graph/properties.h>
 
 namespace CGAL {
 
@@ -28,19 +30,10 @@ namespace CGAL {
   // Linear_cell_complex_for_combinatorial_map class.
   // No difference with class Linear_cell_complex_base except the default
   // template parameters for Refs class which is a combinatorial map.
-  template < unsigned int d_, unsigned int ambient_dim = d_,
-             class Traits_ = Linear_cell_complex_traits<ambient_dim>,
-#if defined(CGAL_CMAP_DART_DEPRECATED) && !defined(CGAL_NO_DEPRECATED_CODE)
-             class Items_ = Linear_cell_complex_min_items<d_>,
-#else
-             class Items_ = Linear_cell_complex_min_items,
-#endif
-             class Alloc_ = CGAL_ALLOCATOR(int),
-             template<unsigned int,class,class,class,class>
-             class CMap = Combinatorial_map_base,
-             class Storage_ = CMap_linear_cell_complex_storage_1<d_, ambient_dim,
-                                                                 Traits_, Items_,
-                                                                 Alloc_> >
+  template < unsigned int d_, unsigned int ambient_dim,
+             class Traits_, class Items_, class Alloc_,
+             template<unsigned int,class,class,class,class> class CMap,
+             class Storage_ >
     class Linear_cell_complex_for_combinatorial_map:
         public Linear_cell_complex_base<d_, ambient_dim, Traits_,
                                         Items_, Alloc_, CMap,
@@ -100,40 +93,119 @@ namespace CGAL {
        *  @param alcc the linear cell complex to copy.
        *  @post *this is valid.
        */
-#ifdef DOXYGEN_RUNNING
       Linear_cell_complex_for_combinatorial_map(const Self& alcc) : Base(alcc)
       {}
-#endif
 
-      template < class LCC2 >
-      Linear_cell_complex_for_combinatorial_map(const LCC2& alcc) : Base(alcc)
-      {}
-
-      template < class LCC2, typename Converters >
-      Linear_cell_complex_for_combinatorial_map(const LCC2& alcc,
-                                                Converters& converters) :
-        Base(alcc, converters)
-      {}
-
-      template < class LCC2, typename Converters, typename DartInfoConverter >
-      Linear_cell_complex_for_combinatorial_map(const LCC2& alcc,
-                                                Converters& converters,
-                                                const DartInfoConverter&
-                                                dartinfoconverter) :
+      template <unsigned int d2,  unsigned int ambient_dim2, class Traits2,
+                class Items2, class Alloc2,
+                template<unsigned int,class,class,class,class> class CMap2,
+                class Storage2>
+      Linear_cell_complex_for_combinatorial_map
+      (const Linear_cell_complex_for_combinatorial_map<d2, ambient_dim2,
+       Traits2, Items2, Alloc2, CMap2, Storage2>& alcc) : Base(alcc)
+      {}
+
+      template <unsigned int d2,  unsigned int ambient_dim2, class Traits2,
+                class Items2, class Alloc2,
+                template<unsigned int,class,class,class,class> class CMap2,
+                class Storage2, typename Converters>
+      Linear_cell_complex_for_combinatorial_map
+      (const Linear_cell_complex_for_combinatorial_map<d2, ambient_dim2,
+       Traits2, Items2, Alloc2, CMap2, Storage2>& alcc,
+       const Converters& converters) : Base(alcc, converters)
+      {}
+
+      template <unsigned int d2,  unsigned int ambient_dim2, class Traits2,
+                class Items2, class Alloc2,
+                template<unsigned int,class,class,class,class> class CMap2,
+                class Storage2, typename Converters, typename DartInfoConverter>
+      Linear_cell_complex_for_combinatorial_map
+      (const Linear_cell_complex_for_combinatorial_map<d2, ambient_dim2,
+       Traits2, Items2, Alloc2, CMap2, Storage2>& alcc,
+       const Converters& converters,
+       const DartInfoConverter& dartinfoconverter) :
         Base(alcc, converters, dartinfoconverter)
       {}
 
-      template < class LCC2, typename Converters, typename DartInfoConverter,
-                 typename PointConverter >
-      Linear_cell_complex_for_combinatorial_map(const LCC2& alcc,
-                                                Converters& converters,
-                                                const DartInfoConverter&
-                                                dartinfoconverter,
-                                                const PointConverter&
-                                                pointconverter) :
+      template <unsigned int d2,  unsigned int ambient_dim2, class Traits2,
+                class Items2, class Alloc2,
+                template<unsigned int,class,class,class,class> class CMap2,
+                class Storage2, typename Converters,
+                typename DartInfoConverter, typename PointConverter>
+      Linear_cell_complex_for_combinatorial_map
+      (const Linear_cell_complex_for_combinatorial_map<d2, ambient_dim2,
+       Traits2, Items2, Alloc2, CMap2, Storage2>& alcc,
+       const Converters& converters, const DartInfoConverter& dartinfoconverter,
+       const PointConverter& pointconverter) :
         Base(alcc, converters, dartinfoconverter, pointconverter)
       {}
 
+      Self & operator= (const Self & alcc)
+      {
+        Base::operator=(alcc);
+        return *this;
+      }
+
+      /** Import the given hds which should be a model of an halfedge graph. */
+      template<class HEG, class PointConverter>
+      void import_from_halfedge_graph(const HEG& heg              ,
+                                      const PointConverter& pointconverter,
+                                      boost::unordered_map
+                                      <typename boost::graph_traits<HEG>::halfedge_descriptor,
+                                      Dart_handle>* origin_to_copy=NULL,
+                                      boost::unordered_map
+                                      <Dart_handle,
+                                      typename boost::graph_traits<HEG>::halfedge_descriptor>*
+                                      copy_to_origin=NULL)
+
+      {
+        boost::unordered_map
+            <typename boost::graph_traits<HEG>::halfedge_descriptor,
+            Dart_handle> local_dartmap;
+        if (origin_to_copy==NULL) // Used local_dartmap if user does not provides its own unordered_map
+        { origin_to_copy=&local_dartmap; }
+
+        Base::import_from_halfedge_graph(heg, origin_to_copy, copy_to_origin);
+
+        typedef typename boost::property_map<HEG,vertex_point_t>::const_type
+            Point_property_map;
+        Point_property_map ppmap = get(CGAL::vertex_point, heg);
+
+        typename boost::unordered_map
+          <typename boost::graph_traits<HEG>::halfedge_descriptor,
+           Dart_handle>::iterator dartmap_iter, dartmap_iter_end=origin_to_copy->end();
+        for (dartmap_iter=origin_to_copy->begin(); dartmap_iter!=dartmap_iter_end;
+             ++dartmap_iter)
+        {
+          if (this->vertex_attribute(dartmap_iter->second)==NULL)
+          {
+            this->set_vertex_attribute(dartmap_iter->second,
+                                 this->create_vertex_attribute());
+            pointconverter.run(ppmap[source(dartmap_iter->first, heg)],
+                this->point(dartmap_iter->second));
+          }
+        }
+      }
+
+      /** Import the given hds which should be a model of an halfedge graph. */
+      template<class HEG>
+      void import_from_halfedge_graph(const HEG& heg,
+                                      boost::unordered_map
+                                      <typename boost::graph_traits<HEG>::halfedge_descriptor,
+                                      Dart_handle>* origin_to_copy=NULL,
+                                      boost::unordered_map
+                                      <Dart_handle,
+                                      typename boost::graph_traits<HEG>::halfedge_descriptor>*
+                                      copy_to_origin=NULL)
+      {
+         typedef typename boost::property_traits<typename boost::property_map
+            <HEG, vertex_point_t>::type>::value_type HEG_point;
+
+        CGAL::internal::Set_point_if_possible_cmap<HEG_point, Point> default_point_converter;
+        import_from_halfedge_graph(heg, default_point_converter,
+                                   origin_to_copy, copy_to_origin);
+      }
+
     };
 
 } // namespace CGAL
diff -pruN 5.0.3-1/include/CGAL/Linear_cell_complex_for_generalized_map.h 5.1-1/include/CGAL/Linear_cell_complex_for_generalized_map.h
--- 5.0.3-1/include/CGAL/Linear_cell_complex_for_generalized_map.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Linear_cell_complex_for_generalized_map.h	2020-09-08 12:15:06.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Linear_cell_complex/include/CGAL/Linear_cell_complex_for_generalized_map.h $
-// $Id: Linear_cell_complex_for_generalized_map.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Linear_cell_complex/include/CGAL/Linear_cell_complex_for_generalized_map.h $
+// $Id: Linear_cell_complex_for_generalized_map.h 0308d1a 2020-03-27T18:35:15+01:00 Guillaume Damiand
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
@@ -12,6 +12,7 @@
 #ifndef CGAL_LINEAR_CELL_COMPLEX_FOR_GENERALIZED_MAP_H
 #define CGAL_LINEAR_CELL_COMPLEX_FOR_GENERALIZED_MAP_H 1
 
+#include <CGAL/Linear_cell_complex_fwd.h>
 #include <CGAL/Linear_cell_complex_base.h>
 #include <CGAL/Linear_cell_complex_traits.h>
 #include <CGAL/Linear_cell_complex_min_items.h>
@@ -28,15 +29,10 @@ namespace CGAL {
   // Linear_cell_complex_for_generalized_map class.
   // No difference with class Linear_cell_complex_base except the default
   // template parameters for Refs class which is a generalized map.
-  template < unsigned int d_, unsigned int ambient_dim = d_,
-             class Traits_ = Linear_cell_complex_traits<ambient_dim>,
-             class Items_ = Linear_cell_complex_min_items,
-             class Alloc_ = CGAL_ALLOCATOR(int),
-             template<unsigned int,class,class,class,class>
-             class CMap = Generalized_map_base,
-             class Storage_ = GMap_linear_cell_complex_storage_1<d_, ambient_dim,
-                                                                 Traits_, Items_,
-                                                                 Alloc_> >
+  template < unsigned int d_, unsigned int ambient_dim,
+             class Traits_, class Items_, class Alloc_,
+             template<unsigned int,class,class,class,class> class CMap,
+             class Storage_ >
     class Linear_cell_complex_for_generalized_map:
         public Linear_cell_complex_base<d_, ambient_dim, Traits_,
                                         Items_, Alloc_, CMap,
@@ -96,40 +92,59 @@ namespace CGAL {
        *  @param alcc the linear cell complex to copy.
        *  @post *this is valid.
        */
-#ifdef DOXYGEN_RUNNING
       Linear_cell_complex_for_generalized_map(const Self & alcc) : Base(alcc)
       {}
-#endif
 
-      template < class LCC2 >
-      Linear_cell_complex_for_generalized_map(const LCC2& alcc) : Base(alcc)
-      {}
-
-      template < class LCC2, typename Converters >
-      Linear_cell_complex_for_generalized_map(const LCC2& alcc,
-                                              Converters& converters) :
-        Base(alcc, converters)
-      {}
-
-      template < class LCC2, typename Converters, typename DartInfoConverter >
-      Linear_cell_complex_for_generalized_map(const LCC2& alcc,
-                                              Converters& converters,
-                                              const DartInfoConverter&
-                                              dartinfoconverter) :
+      template <unsigned int d2,  unsigned int ambient_dim2, class Traits2,
+                class Items2, class Alloc2,
+                template<unsigned int,class,class,class,class> class CMap2,
+                class Storage2>
+      Linear_cell_complex_for_generalized_map
+      (const Linear_cell_complex_for_generalized_map<d2, ambient_dim2,
+       Traits2, Items2, Alloc2, CMap2, Storage2>& alcc) : Base(alcc)
+      {}
+
+      template <unsigned int d2,  unsigned int ambient_dim2, class Traits2,
+                class Items2, class Alloc2,
+                template<unsigned int,class,class,class,class> class CMap2,
+                class Storage2, typename Converters>
+      Linear_cell_complex_for_generalized_map
+      (const Linear_cell_complex_for_generalized_map<d2, ambient_dim2,
+       Traits2, Items2, Alloc2, CMap2, Storage2>& alcc,
+       const Converters& converters) : Base(alcc, converters)
+      {}
+
+      template <unsigned int d2,  unsigned int ambient_dim2, class Traits2,
+                class Items2, class Alloc2,
+                template<unsigned int,class,class,class,class> class CMap2,
+                class Storage2, typename Converters, typename DartInfoConverter>
+      Linear_cell_complex_for_generalized_map
+      (const Linear_cell_complex_for_generalized_map<d2, ambient_dim2,
+       Traits2, Items2, Alloc2, CMap2, Storage2>& alcc,
+       const Converters& converters,
+       const DartInfoConverter& dartinfoconverter) :
         Base(alcc, converters, dartinfoconverter)
       {}
 
-      template < class LCC2, typename Converters, typename DartInfoConverter,
-                 typename PointConverter >
-      Linear_cell_complex_for_generalized_map(const LCC2& alcc,
-                                              Converters& converters,
-                                              const DartInfoConverter&
-                                              dartinfoconverter,
-                                              const PointConverter&
-                                              pointconverter) :
+      template <unsigned int d2,  unsigned int ambient_dim2, class Traits2,
+                class Items2, class Alloc2,
+                template<unsigned int,class,class,class,class> class CMap2,
+                class Storage2, typename Converters, typename DartInfoConverter,
+                typename PointConverter>
+      Linear_cell_complex_for_generalized_map
+      (const Linear_cell_complex_for_generalized_map<d2, ambient_dim2,
+       Traits2, Items2, Alloc2, CMap2, Storage2>& alcc,
+       const Converters& converters,
+       const DartInfoConverter& dartinfoconverter,
+       const PointConverter& pointconverter) :
         Base(alcc, converters, dartinfoconverter, pointconverter)
       {}
 
+      Self & operator= (const Self & alcc)
+      {
+        Base::operator=(alcc);
+        return *this;
+      }
     };
 
 } // namespace CGAL
diff -pruN 5.0.3-1/include/CGAL/Linear_cell_complex_fwd.h 5.1-1/include/CGAL/Linear_cell_complex_fwd.h
--- 5.0.3-1/include/CGAL/Linear_cell_complex_fwd.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Linear_cell_complex_fwd.h	2020-09-08 12:15:06.000000000 +0000
@@ -0,0 +1,98 @@
+// Copyright (c) 2016 CNRS and LIRIS' Establishments (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Linear_cell_complex/include/CGAL/Linear_cell_complex_fwd.h $
+// $Id: Linear_cell_complex_fwd.h 35bff47 2020-03-29T18:11:39+02:00 Guillaume Damiand
+// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
+//
+#ifndef LINEAR_CELL_COMPLEX_FWD_H
+#define LINEAR_CELL_COMPLEX_FWD_H 1
+
+#include <CGAL/memory.h>
+#include <CGAL/Combinatorial_map_fwd.h>
+#include <CGAL/Generalized_map_fwd.h>
+
+namespace CGAL {
+
+template<unsigned int d_, unsigned int ambient_dim,
+         class Traits_, class Items_, class Alloc_ >
+class CMap_linear_cell_complex_storage_1;
+
+template<unsigned int d_, unsigned int ambient_dim,
+         class Traits_, class Items_, class Alloc_ >
+class GMap_linear_cell_complex_storage_1;
+
+template <unsigned int d>
+struct LCC_default_kernel;
+
+template <unsigned int d_,
+          class Kernel=typename LCC_default_kernel<d_>::type >
+struct Linear_cell_complex_traits;
+
+#if defined(CGAL_CMAP_DART_DEPRECATED) && !defined(CGAL_NO_DEPRECATED_CODE)
+  template <unsigned int d>
+  struct CGAL_DEPRECATED Linear_cell_complex_min_items;
+#else
+  struct Linear_cell_complex_min_items;
+#endif
+
+template < unsigned int d_, unsigned int ambient_dim,
+           class Traits_,
+           class Items_,
+           class Alloc_,
+           template<unsigned int,class,class,class,class>
+           class Map,
+           class Refs_,
+           class Storage_>
+class Linear_cell_complex_base;
+
+template < unsigned int d_, unsigned int ambient_dim = d_,
+           class Traits_ = Linear_cell_complex_traits<ambient_dim>,
+#if defined(CGAL_CMAP_DART_DEPRECATED) && !defined(CGAL_NO_DEPRECATED_CODE)
+           class Items_ = Linear_cell_complex_min_items<d_>,
+#else
+           class Items_ = Linear_cell_complex_min_items,
+#endif
+           class Alloc_ = CGAL_ALLOCATOR(int),
+           template<unsigned int,class,class,class,class>
+           class CMap = Combinatorial_map_base,
+           class Storage_ = CMap_linear_cell_complex_storage_1<d_, ambient_dim,
+                                                               Traits_, Items_,
+                                                               Alloc_> >
+  class Linear_cell_complex_for_combinatorial_map;
+
+  template < unsigned int d_, unsigned int ambient_dim = d_,
+             class Traits_ = Linear_cell_complex_traits<ambient_dim>,
+             class Items_ = Linear_cell_complex_min_items,
+             class Alloc_ = CGAL_ALLOCATOR(int),
+             template<unsigned int,class,class,class,class>
+             class CMap = Generalized_map_base,
+             class Storage_ = GMap_linear_cell_complex_storage_1<d_, ambient_dim,
+                                                                 Traits_, Items_,
+                                                                 Alloc_> >
+    class Linear_cell_complex_for_generalized_map;
+
+#if !defined(CGAL_NO_DEPRECATED_CODE)
+  template < unsigned int d_, unsigned int ambient_dim = d_,
+             class Traits_ = Linear_cell_complex_traits<ambient_dim>,
+#if defined(CGAL_CMAP_DART_DEPRECATED)
+             class Items_ = Linear_cell_complex_min_items<d_>,
+#else
+             class Items_ = Linear_cell_complex_min_items,
+#endif
+             class Alloc_ = CGAL_ALLOCATOR(int),
+             template<unsigned int,class,class,class,class>
+             class CMap = Combinatorial_map_base,
+             class Storage_ = CMap_linear_cell_complex_storage_1<d_, ambient_dim,
+                                                                 Traits_, Items_,
+                                                                 Alloc_> >
+  class Linear_cell_complex;
+#endif
+
+} // CGAL
+
+#endif // LINEAR_CELL_COMPLEX_FWD_H
diff -pruN 5.0.3-1/include/CGAL/Linear_cell_complex.h 5.1-1/include/CGAL/Linear_cell_complex.h
--- 5.0.3-1/include/CGAL/Linear_cell_complex.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Linear_cell_complex.h	2020-09-08 12:15:05.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Linear_cell_complex/include/CGAL/Linear_cell_complex.h $
-// $Id: Linear_cell_complex.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Linear_cell_complex/include/CGAL/Linear_cell_complex.h $
+// $Id: Linear_cell_complex.h 0308d1a 2020-03-27T18:35:15+01:00 Guillaume Damiand
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
@@ -12,6 +12,7 @@
 #ifndef CGAL_LINEAR_CELL_COMPLEX_H
 #define CGAL_LINEAR_CELL_COMPLEX_H 1
 
+#include <CGAL/Linear_cell_complex_fwd.h>
 #include <CGAL/Linear_cell_complex_for_combinatorial_map.h>
 #include <CGAL/Linear_cell_complex_traits.h>
 #include <CGAL/Linear_cell_complex_min_items.h>
@@ -26,19 +27,10 @@ namespace CGAL {
    */
 
 #if !defined(CGAL_NO_DEPRECATED_CODE)
-  template < unsigned int d_, unsigned int ambient_dim = d_,
-             class Traits_ = Linear_cell_complex_traits<ambient_dim>,
-#if defined(CGAL_CMAP_DART_DEPRECATED)
-             class Items_ = Linear_cell_complex_min_items<d_>,
-#else
-             class Items_ = Linear_cell_complex_min_items,
-#endif
-             class Alloc_ = CGAL_ALLOCATOR(int),
-             template<unsigned int,class,class,class,class>
-             class CMap = Combinatorial_map_base,
-             class Storage_ = CMap_linear_cell_complex_storage_1<d_, ambient_dim,
-                                                                 Traits_, Items_,
-                                                                 Alloc_> >
+  template < unsigned int d_, unsigned int ambient_dim,
+             class Traits_, class Items_, class Alloc_,
+             template<unsigned int,class,class,class,class> class CMap,
+             class Storage_ >
   class CGAL_DEPRECATED Linear_cell_complex:
     public Linear_cell_complex_for_combinatorial_map<d_, ambient_dim, Traits_, Items_,
                                                      Alloc_, CMap, Storage_>
diff -pruN 5.0.3-1/include/CGAL/Linear_cell_complex_incremental_builder.h 5.1-1/include/CGAL/Linear_cell_complex_incremental_builder.h
--- 5.0.3-1/include/CGAL/Linear_cell_complex_incremental_builder.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Linear_cell_complex_incremental_builder.h	2020-09-08 12:15:06.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Linear_cell_complex/include/CGAL/Linear_cell_complex_incremental_builder.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Linear_cell_complex/include/CGAL/Linear_cell_complex_incremental_builder.h $
 // $Id: Linear_cell_complex_incremental_builder.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Linear_cell_complex_min_items.h 5.1-1/include/CGAL/Linear_cell_complex_min_items.h
--- 5.0.3-1/include/CGAL/Linear_cell_complex_min_items.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Linear_cell_complex_min_items.h	2020-09-08 12:15:06.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Linear_cell_complex/include/CGAL/Linear_cell_complex_min_items.h $
-// $Id: Linear_cell_complex_min_items.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Linear_cell_complex/include/CGAL/Linear_cell_complex_min_items.h $
+// $Id: Linear_cell_complex_min_items.h 0308d1a 2020-03-27T18:35:15+01:00 Guillaume Damiand
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
@@ -16,6 +16,7 @@
 #include <CGAL/Dart.h>
 #endif
 
+#include <CGAL/Linear_cell_complex_fwd.h>
 #include <CGAL/Cell_attribute_with_point.h>
 #include <CGAL/tuple.h>
 
diff -pruN 5.0.3-1/include/CGAL/Linear_cell_complex_operations.h 5.1-1/include/CGAL/Linear_cell_complex_operations.h
--- 5.0.3-1/include/CGAL/Linear_cell_complex_operations.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Linear_cell_complex_operations.h	2020-09-08 12:15:06.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Linear_cell_complex/include/CGAL/Linear_cell_complex_operations.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Linear_cell_complex/include/CGAL/Linear_cell_complex_operations.h $
 // $Id: Linear_cell_complex_operations.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Linear_cell_complex_traits.h 5.1-1/include/CGAL/Linear_cell_complex_traits.h
--- 5.0.3-1/include/CGAL/Linear_cell_complex_traits.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Linear_cell_complex_traits.h	2020-09-08 12:15:06.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Linear_cell_complex/include/CGAL/Linear_cell_complex_traits.h $
-// $Id: Linear_cell_complex_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Linear_cell_complex/include/CGAL/Linear_cell_complex_traits.h $
+// $Id: Linear_cell_complex_traits.h 0308d1a 2020-03-27T18:35:15+01:00 Guillaume Damiand
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
@@ -12,6 +12,7 @@
 #ifndef CGAL_LINEAR_CELL_COMPLEX_TRAITS_H
 #define CGAL_LINEAR_CELL_COMPLEX_TRAITS_H 1
 
+#include <CGAL/Linear_cell_complex_fwd.h>
 #include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
 #include <CGAL/Cartesian_d.h>
 #include <CGAL/predicates_d.h>
@@ -32,8 +33,7 @@ namespace CGAL {
   /** Trait class for Linear_cell_complex class.
    *  dD version (for the moment there is only one dD kernel in CGAL).
    */
-  template <unsigned int d_,
-            class Kernel=typename LCC_default_kernel<d_>::type >
+  template <unsigned int d_, class Kernel>
   struct Linear_cell_complex_traits : public Kernel
   {
     static const unsigned int ambient_dimension = d_;
diff -pruN 5.0.3-1/include/CGAL/linear_least_squares_fitting_2.h 5.1-1/include/CGAL/linear_least_squares_fitting_2.h
--- 5.0.3-1/include/CGAL/linear_least_squares_fitting_2.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/linear_least_squares_fitting_2.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Principal_component_analysis/include/CGAL/linear_least_squares_fitting_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Principal_component_analysis/include/CGAL/linear_least_squares_fitting_2.h $
 // $Id: linear_least_squares_fitting_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/linear_least_squares_fitting_3.h 5.1-1/include/CGAL/linear_least_squares_fitting_3.h
--- 5.0.3-1/include/CGAL/linear_least_squares_fitting_3.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/linear_least_squares_fitting_3.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Principal_component_analysis/include/CGAL/linear_least_squares_fitting_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Principal_component_analysis/include/CGAL/linear_least_squares_fitting_3.h $
 // $Id: linear_least_squares_fitting_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/linear_least_squares_fitting_circles_2.h 5.1-1/include/CGAL/linear_least_squares_fitting_circles_2.h
--- 5.0.3-1/include/CGAL/linear_least_squares_fitting_circles_2.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/linear_least_squares_fitting_circles_2.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Principal_component_analysis/include/CGAL/linear_least_squares_fitting_circles_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Principal_component_analysis/include/CGAL/linear_least_squares_fitting_circles_2.h $
 // $Id: linear_least_squares_fitting_circles_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/linear_least_squares_fitting_cuboids_3.h 5.1-1/include/CGAL/linear_least_squares_fitting_cuboids_3.h
--- 5.0.3-1/include/CGAL/linear_least_squares_fitting_cuboids_3.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/linear_least_squares_fitting_cuboids_3.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Principal_component_analysis/include/CGAL/linear_least_squares_fitting_cuboids_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Principal_component_analysis/include/CGAL/linear_least_squares_fitting_cuboids_3.h $
 // $Id: linear_least_squares_fitting_cuboids_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/linear_least_squares_fitting_points_2.h 5.1-1/include/CGAL/linear_least_squares_fitting_points_2.h
--- 5.0.3-1/include/CGAL/linear_least_squares_fitting_points_2.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/linear_least_squares_fitting_points_2.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Principal_component_analysis/include/CGAL/linear_least_squares_fitting_points_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Principal_component_analysis/include/CGAL/linear_least_squares_fitting_points_2.h $
 // $Id: linear_least_squares_fitting_points_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/linear_least_squares_fitting_points_3.h 5.1-1/include/CGAL/linear_least_squares_fitting_points_3.h
--- 5.0.3-1/include/CGAL/linear_least_squares_fitting_points_3.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/linear_least_squares_fitting_points_3.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Principal_component_analysis/include/CGAL/linear_least_squares_fitting_points_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Principal_component_analysis/include/CGAL/linear_least_squares_fitting_points_3.h $
 // $Id: linear_least_squares_fitting_points_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/linear_least_squares_fitting_rectangles_2.h 5.1-1/include/CGAL/linear_least_squares_fitting_rectangles_2.h
--- 5.0.3-1/include/CGAL/linear_least_squares_fitting_rectangles_2.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/linear_least_squares_fitting_rectangles_2.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Principal_component_analysis/include/CGAL/linear_least_squares_fitting_rectangles_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Principal_component_analysis/include/CGAL/linear_least_squares_fitting_rectangles_2.h $
 // $Id: linear_least_squares_fitting_rectangles_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/linear_least_squares_fitting_segments_2.h 5.1-1/include/CGAL/linear_least_squares_fitting_segments_2.h
--- 5.0.3-1/include/CGAL/linear_least_squares_fitting_segments_2.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/linear_least_squares_fitting_segments_2.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Principal_component_analysis/include/CGAL/linear_least_squares_fitting_segments_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Principal_component_analysis/include/CGAL/linear_least_squares_fitting_segments_2.h $
 // $Id: linear_least_squares_fitting_segments_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/linear_least_squares_fitting_segments_3.h 5.1-1/include/CGAL/linear_least_squares_fitting_segments_3.h
--- 5.0.3-1/include/CGAL/linear_least_squares_fitting_segments_3.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/linear_least_squares_fitting_segments_3.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Principal_component_analysis/include/CGAL/linear_least_squares_fitting_segments_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Principal_component_analysis/include/CGAL/linear_least_squares_fitting_segments_3.h $
 // $Id: linear_least_squares_fitting_segments_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/linear_least_squares_fitting_spheres_3.h 5.1-1/include/CGAL/linear_least_squares_fitting_spheres_3.h
--- 5.0.3-1/include/CGAL/linear_least_squares_fitting_spheres_3.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/linear_least_squares_fitting_spheres_3.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Principal_component_analysis/include/CGAL/linear_least_squares_fitting_spheres_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Principal_component_analysis/include/CGAL/linear_least_squares_fitting_spheres_3.h $
 // $Id: linear_least_squares_fitting_spheres_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/linear_least_squares_fitting_tetrahedra_3.h 5.1-1/include/CGAL/linear_least_squares_fitting_tetrahedra_3.h
--- 5.0.3-1/include/CGAL/linear_least_squares_fitting_tetrahedra_3.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/linear_least_squares_fitting_tetrahedra_3.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Principal_component_analysis/include/CGAL/linear_least_squares_fitting_tetrahedra_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Principal_component_analysis/include/CGAL/linear_least_squares_fitting_tetrahedra_3.h $
 // $Id: linear_least_squares_fitting_tetrahedra_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/linear_least_squares_fitting_triangles_2.h 5.1-1/include/CGAL/linear_least_squares_fitting_triangles_2.h
--- 5.0.3-1/include/CGAL/linear_least_squares_fitting_triangles_2.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/linear_least_squares_fitting_triangles_2.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Principal_component_analysis/include/CGAL/linear_least_squares_fitting_triangles_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Principal_component_analysis/include/CGAL/linear_least_squares_fitting_triangles_2.h $
 // $Id: linear_least_squares_fitting_triangles_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/linear_least_squares_fitting_triangles_3.h 5.1-1/include/CGAL/linear_least_squares_fitting_triangles_3.h
--- 5.0.3-1/include/CGAL/linear_least_squares_fitting_triangles_3.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/linear_least_squares_fitting_triangles_3.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Principal_component_analysis/include/CGAL/linear_least_squares_fitting_triangles_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Principal_component_analysis/include/CGAL/linear_least_squares_fitting_triangles_3.h $
 // $Id: linear_least_squares_fitting_triangles_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/link_to_face_graph.h 5.1-1/include/CGAL/link_to_face_graph.h
--- 5.0.3-1/include/CGAL/link_to_face_graph.h	2020-08-01 19:02:59.000000000 +0000
+++ 5.1-1/include/CGAL/link_to_face_graph.h	2020-09-08 12:15:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_3/include/CGAL/link_to_face_graph.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_3/include/CGAL/link_to_face_graph.h $
 // $Id: link_to_face_graph.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/lloyd_optimize_mesh_2.h 5.1-1/include/CGAL/lloyd_optimize_mesh_2.h
--- 5.0.3-1/include/CGAL/lloyd_optimize_mesh_2.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/lloyd_optimize_mesh_2.h	2020-09-08 12:15:06.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_2/include/CGAL/lloyd_optimize_mesh_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_2/include/CGAL/lloyd_optimize_mesh_2.h $
 // $Id: lloyd_optimize_mesh_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/lloyd_optimize_mesh_3.h 5.1-1/include/CGAL/lloyd_optimize_mesh_3.h
--- 5.0.3-1/include/CGAL/lloyd_optimize_mesh_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/lloyd_optimize_mesh_3.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/lloyd_optimize_mesh_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/lloyd_optimize_mesh_3.h $
 // $Id: lloyd_optimize_mesh_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Location_policy.h 5.1-1/include/CGAL/Location_policy.h
--- 5.0.3-1/include/CGAL/Location_policy.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Location_policy.h	2020-09-08 12:15:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Location_policy.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Location_policy.h $
 // $Id: Location_policy.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/long_double.h 5.1-1/include/CGAL/long_double.h
--- 5.0.3-1/include/CGAL/long_double.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/long_double.h	2020-09-08 12:15:16.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/long_double.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/long_double.h $
 // $Id: long_double.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/long_long.h 5.1-1/include/CGAL/long_long.h
--- 5.0.3-1/include/CGAL/long_long.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/long_long.h	2020-09-08 12:15:16.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/long_long.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/long_long.h $
 // $Id: long_long.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/make_mesh_3.h 5.1-1/include/CGAL/make_mesh_3.h
--- 5.0.3-1/include/CGAL/make_mesh_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/make_mesh_3.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/make_mesh_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/make_mesh_3.h $
 // $Id: make_mesh_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/make_periodic_3_mesh_3.h 5.1-1/include/CGAL/make_periodic_3_mesh_3.h
--- 5.0.3-1/include/CGAL/make_periodic_3_mesh_3.h	2020-08-01 19:02:38.000000000 +0000
+++ 5.1-1/include/CGAL/make_periodic_3_mesh_3.h	2020-09-08 12:15:20.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_mesh_3/include/CGAL/make_periodic_3_mesh_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_mesh_3/include/CGAL/make_periodic_3_mesh_3.h $
 // $Id: make_periodic_3_mesh_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/make_piecewise_smooth_surface_mesh.h 5.1-1/include/CGAL/make_piecewise_smooth_surface_mesh.h
--- 5.0.3-1/include/CGAL/make_piecewise_smooth_surface_mesh.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/make_piecewise_smooth_surface_mesh.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/make_piecewise_smooth_surface_mesh.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/make_piecewise_smooth_surface_mesh.h $
 // $Id: make_piecewise_smooth_surface_mesh.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/make_skin_surface_mesh_3.h 5.1-1/include/CGAL/make_skin_surface_mesh_3.h
--- 5.0.3-1/include/CGAL/make_skin_surface_mesh_3.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/make_skin_surface_mesh_3.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Skin_surface_3/include/CGAL/make_skin_surface_mesh_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Skin_surface_3/include/CGAL/make_skin_surface_mesh_3.h $
 // $Id: make_skin_surface_mesh_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/make_surface_mesh.h 5.1-1/include/CGAL/make_surface_mesh.h
--- 5.0.3-1/include/CGAL/make_surface_mesh.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/make_surface_mesh.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/make_surface_mesh.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/make_surface_mesh.h $
 // $Id: make_surface_mesh.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/make_union_of_balls_3.h 5.1-1/include/CGAL/make_union_of_balls_3.h
--- 5.0.3-1/include/CGAL/make_union_of_balls_3.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/make_union_of_balls_3.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Skin_surface_3/include/CGAL/make_union_of_balls_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Skin_surface_3/include/CGAL/make_union_of_balls_3.h $
 // $Id: make_union_of_balls_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Manhattan_distance_iso_box_point.h 5.1-1/include/CGAL/Manhattan_distance_iso_box_point.h
--- 5.0.3-1/include/CGAL/Manhattan_distance_iso_box_point.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Manhattan_distance_iso_box_point.h	2020-09-08 12:15:39.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_searching/include/CGAL/Manhattan_distance_iso_box_point.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_searching/include/CGAL/Manhattan_distance_iso_box_point.h $
 // $Id: Manhattan_distance_iso_box_point.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/marching_tetrahedra_3.h 5.1-1/include/CGAL/marching_tetrahedra_3.h
--- 5.0.3-1/include/CGAL/marching_tetrahedra_3.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/marching_tetrahedra_3.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Skin_surface_3/include/CGAL/marching_tetrahedra_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Skin_surface_3/include/CGAL/marching_tetrahedra_3.h $
 // $Id: marching_tetrahedra_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Marching_tetrahedra_observer_default_3.h 5.1-1/include/CGAL/Marching_tetrahedra_observer_default_3.h
--- 5.0.3-1/include/CGAL/Marching_tetrahedra_observer_default_3.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Marching_tetrahedra_observer_default_3.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Skin_surface_3/include/CGAL/Marching_tetrahedra_observer_default_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Skin_surface_3/include/CGAL/Marching_tetrahedra_observer_default_3.h $
 // $Id: Marching_tetrahedra_observer_default_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Marching_tetrahedra_traits_skin_surface_3.h 5.1-1/include/CGAL/Marching_tetrahedra_traits_skin_surface_3.h
--- 5.0.3-1/include/CGAL/Marching_tetrahedra_traits_skin_surface_3.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Marching_tetrahedra_traits_skin_surface_3.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Skin_surface_3/include/CGAL/Marching_tetrahedra_traits_skin_surface_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Skin_surface_3/include/CGAL/Marching_tetrahedra_traits_skin_surface_3.h $
 // $Id: Marching_tetrahedra_traits_skin_surface_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mean_curvature_flow_skeletonization.h 5.1-1/include/CGAL/Mean_curvature_flow_skeletonization.h
--- 5.0.3-1/include/CGAL/Mean_curvature_flow_skeletonization.h	2020-08-01 19:02:56.000000000 +0000
+++ 5.1-1/include/CGAL/Mean_curvature_flow_skeletonization.h	2020-09-08 12:15:45.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_skeletonization/include/CGAL/Mean_curvature_flow_skeletonization.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_skeletonization/include/CGAL/Mean_curvature_flow_skeletonization.h $
 // $Id: Mean_curvature_flow_skeletonization.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/memory.h 5.1-1/include/CGAL/memory.h
--- 5.0.3-1/include/CGAL/memory.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/memory.h	2020-09-08 12:15:32.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/memory.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/memory.h $
 // $Id: memory.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Memory_sizer.h 5.1-1/include/CGAL/Memory_sizer.h
--- 5.0.3-1/include/CGAL/Memory_sizer.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Memory_sizer.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Profiling_tools/include/CGAL/Memory_sizer.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Profiling_tools/include/CGAL/Memory_sizer.h $
 // $Id: Memory_sizer.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_2/Clusters.h 5.1-1/include/CGAL/Mesh_2/Clusters.h
--- 5.0.3-1/include/CGAL/Mesh_2/Clusters.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_2/Clusters.h	2020-09-08 12:15:06.000000000 +0000
@@ -4,8 +4,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_2/include/CGAL/Mesh_2/Clusters.h $
-// $Id: Clusters.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_2/include/CGAL/Mesh_2/Clusters.h $
+// $Id: Clusters.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -89,7 +89,7 @@ public:
 
     /**
      * The following map tells what vertices are in the cluster and if
-     * the corresponding segment has been splitted once.
+     * the corresponding segment has been split once.
      */
     typedef std::map<Vertex_handle, bool> Vertices_map;
     Vertices_map vertices;
diff -pruN 5.0.3-1/include/CGAL/Mesh_2/Do_not_refine_edges.h 5.1-1/include/CGAL/Mesh_2/Do_not_refine_edges.h
--- 5.0.3-1/include/CGAL/Mesh_2/Do_not_refine_edges.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_2/Do_not_refine_edges.h	2020-09-08 12:15:06.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_2/include/CGAL/Mesh_2/Do_not_refine_edges.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_2/include/CGAL/Mesh_2/Do_not_refine_edges.h $
 // $Id: Do_not_refine_edges.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_2/Face_badness.h 5.1-1/include/CGAL/Mesh_2/Face_badness.h
--- 5.0.3-1/include/CGAL/Mesh_2/Face_badness.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_2/Face_badness.h	2020-09-08 12:15:06.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_2/include/CGAL/Mesh_2/Face_badness.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_2/include/CGAL/Mesh_2/Face_badness.h $
 // $Id: Face_badness.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_2/Lipschitz_sizing_field_2.h 5.1-1/include/CGAL/Mesh_2/Lipschitz_sizing_field_2.h
--- 5.0.3-1/include/CGAL/Mesh_2/Lipschitz_sizing_field_2.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_2/Lipschitz_sizing_field_2.h	2020-09-08 12:15:06.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_2/include/CGAL/Mesh_2/Lipschitz_sizing_field_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_2/include/CGAL/Mesh_2/Lipschitz_sizing_field_2.h $
 // $Id: Lipschitz_sizing_field_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_2/Lloyd_move_2.h 5.1-1/include/CGAL/Mesh_2/Lloyd_move_2.h
--- 5.0.3-1/include/CGAL/Mesh_2/Lloyd_move_2.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_2/Lloyd_move_2.h	2020-09-08 12:15:06.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_2/include/CGAL/Mesh_2/Lloyd_move_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_2/include/CGAL/Mesh_2/Lloyd_move_2.h $
 // $Id: Lloyd_move_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_2/Mesh_global_optimizer_2.h 5.1-1/include/CGAL/Mesh_2/Mesh_global_optimizer_2.h
--- 5.0.3-1/include/CGAL/Mesh_2/Mesh_global_optimizer_2.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_2/Mesh_global_optimizer_2.h	2020-09-08 12:15:06.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_2/include/CGAL/Mesh_2/Mesh_global_optimizer_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_2/include/CGAL/Mesh_2/Mesh_global_optimizer_2.h $
 // $Id: Mesh_global_optimizer_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_2/Mesh_sizing_field.h 5.1-1/include/CGAL/Mesh_2/Mesh_sizing_field.h
--- 5.0.3-1/include/CGAL/Mesh_2/Mesh_sizing_field.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_2/Mesh_sizing_field.h	2020-09-08 12:15:06.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_2/include/CGAL/Mesh_2/Mesh_sizing_field.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_2/include/CGAL/Mesh_2/Mesh_sizing_field.h $
 // $Id: Mesh_sizing_field.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_2/Output_stream.h 5.1-1/include/CGAL/Mesh_2/Output_stream.h
--- 5.0.3-1/include/CGAL/Mesh_2/Output_stream.h	2020-08-01 19:02:32.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_2/Output_stream.h	2020-09-08 12:15:10.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesher_level/include/CGAL/Mesh_2/Output_stream.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesher_level/include/CGAL/Mesh_2/Output_stream.h $
 // $Id: Output_stream.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_2/Refine_edges.h 5.1-1/include/CGAL/Mesh_2/Refine_edges.h
--- 5.0.3-1/include/CGAL/Mesh_2/Refine_edges.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_2/Refine_edges.h	2020-09-08 12:15:06.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_2/include/CGAL/Mesh_2/Refine_edges.h $
-// $Id: Refine_edges.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_2/include/CGAL/Mesh_2/Refine_edges.h $
+// $Id: Refine_edges.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -493,7 +493,7 @@ public:
   /** This version computes the refinement point without handling
       clusters. The refinement point of an edge is just the middle point of
       the segment.
-      Saves the handles of the edge that will be splitted.
+      Saves the handles of the edge that will be split.
       This function is overridden in class Refine_edge_with_clusters.
   */
   Point refinement_point_impl(const Edge& edge)
diff -pruN 5.0.3-1/include/CGAL/Mesh_2/Refine_edges_visitor.h 5.1-1/include/CGAL/Mesh_2/Refine_edges_visitor.h
--- 5.0.3-1/include/CGAL/Mesh_2/Refine_edges_visitor.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_2/Refine_edges_visitor.h	2020-09-08 12:15:06.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_2/include/CGAL/Mesh_2/Refine_edges_visitor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_2/include/CGAL/Mesh_2/Refine_edges_visitor.h $
 // $Id: Refine_edges_visitor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_2/Refine_edges_with_clusters.h 5.1-1/include/CGAL/Mesh_2/Refine_edges_with_clusters.h
--- 5.0.3-1/include/CGAL/Mesh_2/Refine_edges_with_clusters.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_2/Refine_edges_with_clusters.h	2020-09-08 12:15:06.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_2/include/CGAL/Mesh_2/Refine_edges_with_clusters.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_2/include/CGAL/Mesh_2/Refine_edges_with_clusters.h $
 // $Id: Refine_edges_with_clusters.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_2/Refine_faces.h 5.1-1/include/CGAL/Mesh_2/Refine_faces.h
--- 5.0.3-1/include/CGAL/Mesh_2/Refine_faces.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_2/Refine_faces.h	2020-09-08 12:15:06.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_2/include/CGAL/Mesh_2/Refine_faces.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_2/include/CGAL/Mesh_2/Refine_faces.h $
 // $Id: Refine_faces.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_2/Sizing_field_2.h 5.1-1/include/CGAL/Mesh_2/Sizing_field_2.h
--- 5.0.3-1/include/CGAL/Mesh_2/Sizing_field_2.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_2/Sizing_field_2.h	2020-09-08 12:15:06.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_2/include/CGAL/Mesh_2/Sizing_field_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_2/include/CGAL/Mesh_2/Sizing_field_2.h $
 // $Id: Sizing_field_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_2/Uniform_sizing_field_2.h 5.1-1/include/CGAL/Mesh_2/Uniform_sizing_field_2.h
--- 5.0.3-1/include/CGAL/Mesh_2/Uniform_sizing_field_2.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_2/Uniform_sizing_field_2.h	2020-09-08 12:15:06.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_2/include/CGAL/Mesh_2/Uniform_sizing_field_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_2/include/CGAL/Mesh_2/Uniform_sizing_field_2.h $
 // $Id: Uniform_sizing_field_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/C3T3_helpers.h 5.1-1/include/CGAL/Mesh_3/C3T3_helpers.h
--- 5.0.3-1/include/CGAL/Mesh_3/C3T3_helpers.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/C3T3_helpers.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/C3T3_helpers.h $
-// $Id: C3T3_helpers.h ad23db2 2020-06-30T16:36:58+02:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/C3T3_helpers.h $
+// $Id: C3T3_helpers.h 25eeff8 2020-07-22T12:29:55+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -26,7 +26,7 @@
 
 #include <CGAL/linear_least_squares_fitting_3.h>
 #include <CGAL/Mesh_3/Triangulation_helpers.h>
-#include <CGAL/Hash_handles_with_or_without_timestamps.h>
+#include <CGAL/Time_stamper.h>
 #include <CGAL/tuple.h>
 #include <CGAL/iterator.h>
 #include <CGAL/array.h>
@@ -1224,8 +1224,8 @@ private:
       if(update_c3t3)
       {
         // Update status in c3t3
-        if(surface != boost::none)
-          c3t3_.add_to_complex(facet, surface.get());
+        if((bool)surface)
+          c3t3_.add_to_complex(facet, *surface);
         else
           c3t3_.remove_from_complex(facet);
       }
@@ -1246,7 +1246,7 @@ private:
         }
       }
 
-      return surface;
+      return surface ? surface : Surface_patch();
     }
 
 
@@ -1366,7 +1366,6 @@ private:
 
     std::size_t vertex_id(const std::size_t& i) const
     {
-      CGAL_precondition(i >= 0);
       CGAL_precondition((infinite_ && i < 3) || i < 4);
       return vertices_[i];
     }
@@ -1573,10 +1572,17 @@ private:
   Cell_vector c3t3_cells(const Cell_vector& cells) const
   {
     Cell_vector c3t3_cells;
+#ifdef CGAL_CXX17
+    std::remove_copy_if(cells.begin(),
+                        cells.end(),
+                        std::back_inserter(c3t3_cells),
+                        std::not_fn(Is_in_c3t3<Cell_handle>(c3t3_)));
+#else
     std::remove_copy_if(cells.begin(),
                         cells.end(),
                         std::back_inserter(c3t3_cells),
                         std::not1(Is_in_c3t3<Cell_handle>(c3t3_)) );
+#endif
     return c3t3_cells;
   }
 
@@ -2048,7 +2054,7 @@ private:
                                        true); /* update surface centers */
       // false means "do not update the c3t3"
       if ( c3t3_.is_in_complex(*fit) != (bool)sp ||
-           ((bool)sp && !(c3t3_.surface_patch_index(*fit) == sp.get()) ) )
+           ((bool)sp && !(c3t3_.surface_patch_index(*fit) == *sp) ) )
         return false;
     }
 
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Cell_criteria_visitor_with_balls.h 5.1-1/include/CGAL/Mesh_3/Cell_criteria_visitor_with_balls.h
--- 5.0.3-1/include/CGAL/Mesh_3/Cell_criteria_visitor_with_balls.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Cell_criteria_visitor_with_balls.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Cell_criteria_visitor_with_balls.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Cell_criteria_visitor_with_balls.h $
 // $Id: Cell_criteria_visitor_with_balls.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Concurrent_mesher_config.h 5.1-1/include/CGAL/Mesh_3/Concurrent_mesher_config.h
--- 5.0.3-1/include/CGAL/Mesh_3/Concurrent_mesher_config.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Concurrent_mesher_config.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Concurrent_mesher_config.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Concurrent_mesher_config.h $
 // $Id: Concurrent_mesher_config.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/config.h 5.1-1/include/CGAL/Mesh_3/config.h
--- 5.0.3-1/include/CGAL/Mesh_3/config.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/config.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/config.h $
-// $Id: config.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/config.h $
+// $Id: config.h 4dda7b6 2020-05-27T15:53:05+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Laurent Rineau
@@ -12,7 +12,7 @@
 #ifndef CGAL_MESH_3_CONFIG_H
 #define CGAL_MESH_3_CONFIG_H 1
 
-#include <CGAL/license/Mesh_3.h>
+#include <CGAL/license/Triangulation_3.h>
 
 
 #include <CGAL/config.h>
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Detect_polylines_in_polyhedra_fwd.h 5.1-1/include/CGAL/Mesh_3/Detect_polylines_in_polyhedra_fwd.h
--- 5.0.3-1/include/CGAL/Mesh_3/Detect_polylines_in_polyhedra_fwd.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Detect_polylines_in_polyhedra_fwd.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,36 +0,0 @@
-// Copyright (c) 2010 GeometryFactory Sarl (France).
-// All rights reserved.
-//
-// This file is part of CGAL (www.cgal.org).
-//
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Detect_polylines_in_polyhedra_fwd.h $
-// $Id: Detect_polylines_in_polyhedra_fwd.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
-// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
-//
-//
-// Author(s)     : Laurent Rineau
-//
-
-#ifndef CGAL_DETECT_POLYLINES_IN_POLYHEDRA_FWD_H
-#define CGAL_DETECT_POLYLINES_IN_POLYHEDRA_FWD_H
-
-#include <CGAL/license/Mesh_3.h>
-
-
-namespace CGAL { namespace Mesh_3 {
-
-template <typename Polyhedron>
-struct Detect_polylines;
-
-template <typename Polyhedron,
-          typename Polyline_and_context,
-          typename Polylines_output_iterator>
-Polylines_output_iterator
-detect_polylines(Polyhedron* pMesh,
-                 Polylines_output_iterator out_it);
-
-} // end namespace CGAL::Mesh_3
-} // end namespace CGAL
-
-
-#endif // CGAL_DETECT_POLYLINES_IN_POLYHEDRA_FWD_H
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Detect_polylines_in_polyhedra.h 5.1-1/include/CGAL/Mesh_3/Detect_polylines_in_polyhedra.h
--- 5.0.3-1/include/CGAL/Mesh_3/Detect_polylines_in_polyhedra.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Detect_polylines_in_polyhedra.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,384 +0,0 @@
-// Copyright (c) 2010, 2012 GeometryFactory Sarl (France).
-// All rights reserved.
-//
-// This file is part of CGAL (www.cgal.org).
-//
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Detect_polylines_in_polyhedra.h $
-// $Id: Detect_polylines_in_polyhedra.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
-// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
-//
-//
-// Author(s)     : Laurent Rineau
-//
-
-#ifndef CGAL_MESH_3_DETECT_POLYLINES_IN_POLYHEDRA_H
-#define CGAL_MESH_3_DETECT_POLYLINES_IN_POLYHEDRA_H
-
-#include <CGAL/license/Mesh_3.h>
-
-#include <CGAL/Compare_handles_with_or_without_timestamps.h>
-#include <CGAL/Mesh_3/Detect_polylines_in_polyhedra_fwd.h>
-#include <CGAL/Default.h>
-#include <CGAL/Hash_handles_with_or_without_timestamps.h>
-
-#include <boost/mpl/if.hpp>
-#include <boost/unordered_map.hpp>
-#include <boost/unordered_set.hpp>
-
-#include <algorithm>
-
-namespace CGAL { namespace Mesh_3 {
-
-template <typename Polyhedron>
-struct Detect_polylines
-{
-  typedef typename Polyhedron::Traits Geom_traits;
-  typedef typename Geom_traits::Point_3 Point_3;
-  typedef typename Polyhedron::Halfedge_const_handle Halfedge_const_handle;
-  typedef typename Polyhedron::Halfedge_handle Halfedge_handle;
-  typedef typename Polyhedron::Vertex_const_handle Vertex_const_handle;
-  typedef typename Polyhedron::Vertex_handle Vertex_handle;
-  typedef typename Polyhedron::size_type size_type;
-  typedef CGAL::Compare_handles_with_or_without_timestamps Compare_handles;
-
-  typedef CGAL::Hash_handles_with_or_without_timestamps   Hash_fct;
-  typedef boost::unordered_set<Vertex_handle, Hash_fct>   Vertices_set;
-  typedef boost::unordered_map<Vertex_handle,
-                               size_type,
-                               Hash_fct>                  Vertices_counter;
-
-  typedef boost::unordered_set<Halfedge_handle, Hash_fct> Feature_edges_set;
-
-  Feature_edges_set edges_to_consider;
-  Vertices_set corner_vertices;
-
-  // typedef std::vector<Point_3> Polyline_and_context;
-
-  typedef typename Polyhedron::Vertex Polyhedron_vertex;
-  typedef typename Polyhedron_vertex::Set_of_indices Set_of_indices;
-
-  template <typename T>
-  static
-  void display_index(std::ostream& stream, const T& x)
-  {
-    stream << x;
-  }
-
-  template <typename T, typename U>
-  static
-  void display_index(std::ostream& stream, const std::pair<T,U>& p)
-  {
-    stream << p.first << "+" << p.second;
-  }
-
-  static
-  void display_set(std::ostream& stream, Set_of_indices set) {
-    stream << "( ";
-    for(typename Set_of_indices::value_type i : set) {
-      display_index(stream, i);
-      stream << " ";
-    }
-    stream << ")";
-  }
-
-  static Set_of_indices
-  edge_indices(const Halfedge_handle he) {
-    Set_of_indices set_of_indices;
-    const Set_of_indices& source_set =
-      he->opposite()->vertex()->incident_patches_ids_set();
-    const Set_of_indices& target_set =
-      he->vertex()->incident_patches_ids_set();
-    std::set_intersection(source_set.begin(), source_set.end(),
-                          target_set.begin(), target_set.end(),
-                          std::inserter(set_of_indices,
-                                        set_of_indices.begin()));
-    if(set_of_indices.empty()) {
-      std::cerr << "Indices set of following edge is empty:\n";
-      std::cerr << "  " << he->opposite()->vertex()->point()
-                << " ";
-      display_set(std::cerr, source_set);
-      std::cerr << "\n";
-      std::cerr << "  " << he->vertex()->point()
-                << " ";
-      display_set(std::cerr, target_set);
-      std::cerr << "\n";
-    }
-    return set_of_indices;
-  }
-
-  static Halfedge_handle canonical(Halfedge_handle he)
-  {
-    const Halfedge_handle& op = he->opposite();
-    if(Compare_handles()(he, op))
-      return he;
-    else
-      return op;
-  }
-
-  static bool is_feature(const Halfedge_handle he) {
-    return
-      he->is_feature_edge() || he->opposite()->is_feature_edge();
-  }
-
-  /** Follow a polyline or a polygon, from the halfedge he. */
-  template <typename Polyline_and_context, typename Polylines_output_iterator>
-  Polylines_output_iterator
-  follow_half_edge(const Halfedge_handle he,
-                   Polylines_output_iterator polylines_out,
-                   Polyline_and_context = Polyline_and_context())
-  {
-    typename Feature_edges_set::iterator it =
-      edges_to_consider.find(canonical(he));
-    if(it == edges_to_consider.end()) {
-      return polylines_out;
-    }
-
-    Polyline_and_context polyline;
-    polyline.polyline_content.push_back(he->opposite()->vertex()->point());
-
-    Halfedge_handle current_he = he;
-
-    Set_of_indices set_of_indices_of_current_edge
-      = edge_indices(current_he);
-
-    do {
-      CGAL_assertion(!set_of_indices_of_current_edge.empty());
-      CGAL_assertion(is_feature(current_he));
-      CGAL_assertion_code(const size_type n = )
-        edges_to_consider.erase(canonical(current_he));
-      CGAL_assertion(n > 0);
-      Vertex_handle v = current_he->vertex();
-      polyline.polyline_content.push_back(v->point());
-      // std::cerr << v->point() << std::endl;
-      if(corner_vertices.count(v) > 0) break;
-      typename Polyhedron::Halfedge_around_vertex_circulator
-        loop_he = v->vertex_begin();
-      ++loop_he;
-      // CGAL_assertion((&*loop_he) != (&*current_he) );
-      while((&*loop_he) == (&*current_he) ||
-            (!is_feature(loop_he)) ) {
-        ++loop_he;
-        // CGAL_assertion((&*loop_he) != (&*current_he) );
-      }
-
-      Set_of_indices set_of_indices_of_next_edge =
-        edge_indices(loop_he);
-
-      if(! (set_of_indices_of_next_edge.size() ==
-            set_of_indices_of_current_edge.size()
-            &&
-            std::equal(set_of_indices_of_next_edge.begin(),
-                       set_of_indices_of_next_edge.end(),
-                       set_of_indices_of_current_edge.begin())) )
-      {
-        // the vertex is a special vertex, a new corner
-#ifdef CGAL_MESH_3_PROTECTION_DEBUG
-        std::cerr << "New corner vertex " << v->point() << std::endl;
-        std::cerr << "  indices were: ";
-        for(typename Set_of_indices::value_type i :
-                      set_of_indices_of_current_edge) {
-          std::cerr << i << " ";
-        }
-        std::cerr << "\n           now: ";
-        for(typename Set_of_indices::value_type i :
-                      set_of_indices_of_next_edge) {
-          std::cerr << i << " ";
-        }
-        std::cerr << "\n";
-#endif
-        ++v->nb_of_feature_edges;
-        corner_vertices.insert(v);
-        polyline.context.adjacent_patches_ids=set_of_indices_of_current_edge;
-        *polylines_out++ = polyline;
-        polyline.polyline_content.clear();
-        polyline.polyline_content.push_back(loop_he->vertex()->point());
-        set_of_indices_of_current_edge = set_of_indices_of_next_edge;
-      }
-
-      current_he = loop_he->opposite();
-    } while(current_he != he );
-
-    polyline.context.adjacent_patches_ids=set_of_indices_of_current_edge;
-    *polylines_out++ = polyline;
-    return polylines_out;
-  }
-
-  /** Loop around a corner vertex, and try to follow a polyline of feature
-      edges, from each incident edge. */
-  template <typename Polyline_and_context, typename Polylines_output_iterator>
-  Polylines_output_iterator
-  loop_around_corner(const Vertex_handle v,
-                     Polylines_output_iterator polylines_out,
-                     Polyline_and_context empty_polyline =
-                     Polyline_and_context() )
-  {
-    typename Polyhedron::Halfedge_around_vertex_circulator
-      he = v->vertex_begin(), end(he);
-    do {
-      CGAL_assertion(he->vertex() == v);
-      polylines_out = follow_half_edge(he->opposite(),
-                                       polylines_out,
-                                       empty_polyline);
-      ++he;
-    } while(he != end);
-    return polylines_out;
-  }
-
-  /** For a non-corner vertex v (that is incident to two feature edges),
-      measure the angle between the two edges, and mark the vertex as corner
-      edge, if the angle is < 120°. **/
-  static bool measure_angle(const Vertex_handle v)
-  {
-    Halfedge_handle e1;
-    Halfedge_handle e2;
-    typename Polyhedron::Halfedge_around_vertex_circulator he =
-      v->vertex_begin(), end(he);
-    // std::cerr << "measure_handle(" << (void*)(&*v)
-    //           << " = " << v->point() << ")";
-    bool first = true;
-    bool done = false;
-    do {
-      CGAL_assertion(he->vertex() == v);
-      // std::cerr << he->opposite()->vertex()->point() << std::endl;
-      if(is_feature(he)) {
-        if(first) {
-          e1 = he;
-          first = false;
-        }
-        else {
-          if(done) {
-            std::cerr << v->point() << " should be a corner!\n"
-                      << "  Too many adjacent feature edges!\n";
-            return false;
-          }
-          e2 = he;
-          done = true;
-        }
-        // std::cerr << "x";
-      }
-      // else
-        // std::cerr << ".";
-      ++he;
-    } while(he != end);
-    if(!done) {
-      std::cerr << v->point() << " should be a corner!\n"
-                << "  Not enough adjacent feature edge!\n";
-      return false;
-    }
-      // std::cerr << "\n";
-    const Point_3 pv = v->point();
-    const Point_3 pa = e1->opposite()->vertex()->point();
-    const Point_3 pb = e2->opposite()->vertex()->point();
-    const typename Geom_traits::Vector_3 av = pv - pa;
-    const typename Geom_traits::Vector_3 bv = pv - pb;
-    const typename Geom_traits::FT sc_prod = av * bv;
-    if( sc_prod >= 0 ||
-        (sc_prod < 0 &&
-         CGAL::square(sc_prod) < (av * av) * (bv * bv) / 4 ) )
-    {
-      // std::cerr << "Corner (" << pa << ", " << pv
-      //           << ", " << pb << ")\n";
-      return true;
-    }
-    else {
-      return false;
-    }
-  }
-
-  template <typename Polyline_and_context,
-            typename Polylines_output_iterator>
-  Polylines_output_iterator
-  operator()(Polyhedron* pMesh,
-             Polylines_output_iterator out_it,
-             Polyline_and_context empty_polyline)
-  {
-    // That call orders the set of edges of the polyhedron, so that the
-    // feature edges are at the end of the sequence of edges.
-    // pMesh->normalize_border();
-    Vertices_counter feature_vertices;
-
-    // Iterate over all edges, and find out which vertices are corner
-    // vertices (more than two incident feature edges).
-    for(typename Polyhedron::Edge_iterator
-          eit = pMesh->edges_begin (),
-          end = pMesh->edges_end();
-        eit != end; ++eit)
-    {
-      if(!eit->is_feature_edge()) continue;
-      edges_to_consider.insert(canonical(eit));
-      typename Polyhedron::Vertex_handle v = eit->vertex();
-      for(unsigned i = 0; i < 2; ++i) {
-        if(++feature_vertices[v] == 3)
-          corner_vertices.insert(v);
-        v = eit->opposite()->vertex();
-      }
-    }
-
-    for(typename Polyhedron::Vertex_iterator
-          vit = pMesh->vertices_begin (),
-          end = pMesh->vertices_end();
-        vit != end; ++vit)
-    {
-      if(feature_vertices.count(vit) !=0 &&
-         feature_vertices[vit] == 1) {
-        corner_vertices.insert(vit);
-      }
-    }
-
-#ifdef CGAL_MESH_3_PROTECTION_DEBUG
-    std::cerr << "Corner vertices: " << corner_vertices.size() << std::endl;
-    std::cerr << "Feature vertices: " << feature_vertices.size() << std::endl;
-#endif
-
-    // // Iterate over non-corner feature vertices, and measure the angle.
-    for(typename Vertices_counter::iterator it = feature_vertices.begin(),
-        end = feature_vertices.end(); it != end; ++it)
-    {
-      const Vertex_handle v = it->first;
-      if(corner_vertices.count(v) == 0) {
-        CGAL_assertion(it->second == 2);
-        if(measure_angle(v)) {
-          corner_vertices.insert(v);
-        }
-      }
-    }
-#ifdef CGAL_MESH_3_PROTECTION_DEBUG
-    std::cerr << "New corner vertices: "
-              << corner_vertices.size() << std::endl;
-#endif
-
-    // Follow the polylines...
-    for(typename Vertices_set::iterator it = corner_vertices.begin(),
-        end = corner_vertices.end(); it != end; ++it)
-    {
-      out_it = loop_around_corner(*it, out_it, empty_polyline);
-    }
-
-    // ... and the cycles.
-    while(! edges_to_consider.empty() ) {
-      out_it = follow_half_edge(*edges_to_consider.begin(),
-                                out_it,
-                                empty_polyline);
-    }
-
-    return out_it;
-  }
-};
-
-template <typename Polyhedron,
-          typename Polyline_and_context,
-          typename Polylines_output_iterator>
-Polylines_output_iterator
-detect_polylines(Polyhedron* pMesh,
-                 Polylines_output_iterator out_it) {
-
-  Detect_polylines<Polyhedron> go;
-  Polyline_and_context empty_polyline;
-  return go(pMesh, out_it, empty_polyline);
-}
-
-} // end namespace CGAL::Mesh_3
-} // end namespace CGAL
-
-
-#endif // CGAL_MESH_3_DETECT_POLYLINES_IN_POLYHEDRA_H
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/dihedral_angle_3.h 5.1-1/include/CGAL/Mesh_3/dihedral_angle_3.h
--- 5.0.3-1/include/CGAL/Mesh_3/dihedral_angle_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/dihedral_angle_3.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/dihedral_angle_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/dihedral_angle_3.h $
 // $Id: dihedral_angle_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Dump_c3t3.h 5.1-1/include/CGAL/Mesh_3/Dump_c3t3.h
--- 5.0.3-1/include/CGAL/Mesh_3/Dump_c3t3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Dump_c3t3.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Dump_c3t3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Dump_c3t3.h $
 // $Id: Dump_c3t3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/experimental/AABB_filtered_projection_traits.h 5.1-1/include/CGAL/Mesh_3/experimental/AABB_filtered_projection_traits.h
--- 5.0.3-1/include/CGAL/Mesh_3/experimental/AABB_filtered_projection_traits.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/experimental/AABB_filtered_projection_traits.h	2020-09-08 12:15:09.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/experimental/AABB_filtered_projection_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/experimental/AABB_filtered_projection_traits.h $
 // $Id: AABB_filtered_projection_traits.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/experimental/Facet_topological_criterion_with_adjacency.h 5.1-1/include/CGAL/Mesh_3/experimental/Facet_topological_criterion_with_adjacency.h
--- 5.0.3-1/include/CGAL/Mesh_3/experimental/Facet_topological_criterion_with_adjacency.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/experimental/Facet_topological_criterion_with_adjacency.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/experimental/Facet_topological_criterion_with_adjacency.h $
-// $Id: Facet_topological_criterion_with_adjacency.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/experimental/Facet_topological_criterion_with_adjacency.h $
+// $Id: Facet_topological_criterion_with_adjacency.h 58b10a3 2020-03-26T18:58:50+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Laurent Rineau
@@ -87,8 +87,16 @@ protected:
 
           Index_set set;
           domain->get_corner_incidences(corner_id, std::back_inserter(set));
-          if(std::find(set.begin(), set.end(), patch_index) == set.end())
+          if(std::find(set.begin(), set.end(), patch_index) == set.end()) {
+#ifdef CGAL_MESH_3_DEBUG_FACET_CRITERIA
+            std::cerr << "Bad facet "
+                         "(Facet_topological_criterion_with_adjacency: corner #"
+                      << corner_id << ", point " << v->point()
+                      << ", is not incident to patch #" << patch_index << ")"
+                      << std::endl;
+#endif
             return Is_bad(Quality(1)); // bad!
+          }
         }
         break;
       case 1:
@@ -98,13 +106,28 @@ protected:
             domain->curve_index(v->index());
           Index_set set;
           domain->get_incidences(curve_id, std::back_inserter(set));
-          if(std::find(set.begin(), set.end(), patch_index) == set.end())
+          if(std::find(set.begin(), set.end(), patch_index) == set.end()) {
+#ifdef CGAL_MESH_3_DEBUG_FACET_CRITERIA
+            std::cerr << "Bad facet "
+                         "(Facet_topological_criterion_with_adjacency: curve #"
+                      << curve_id << ", at point " << v->point()
+                      << ", is not incident to patch #" << patch_index << ")"
+                      << std::endl;
+#endif
             return Is_bad(Quality(1)); // bad!
+          }
         }
         break;
       case 2:
-        if(domain->surface_patch_index(v->index()) != patch_index)
+        if(domain->surface_patch_index(v->index()) != patch_index) {
+#ifdef CGAL_MESH_3_DEBUG_FACET_CRITERIA
+          std::cerr << "Bad facet (Facet_topological_criterion_with_adjacency: "
+                       "vertex at point "
+                    << v->point() << " is not on patch #" << patch_index << ")"
+                    << std::endl;
+#endif
           return Is_bad(Quality(1)); // bad!
+        }
         break;
       default:
         return Is_bad(Quality(1));
@@ -112,6 +135,10 @@ protected:
       }
     }
     if(nb_vertices_on_curves == 3) {
+#ifdef CGAL_MESH_3_DEBUG_FACET_CRITERIA
+      std::cerr << "Bad facet (Facet_topological_criterion_with_adjacency: "
+                   "three points on a curve)\n";
+#endif
       return Is_bad(Quality(1)); // bad!
       // All vertices are on curves. That means that the facet could be on
       // several different patches. Let's disallow that.
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/experimental/Get_curve_index.h 5.1-1/include/CGAL/Mesh_3/experimental/Get_curve_index.h
--- 5.0.3-1/include/CGAL/Mesh_3/experimental/Get_curve_index.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/experimental/Get_curve_index.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/experimental/Get_curve_index.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/experimental/Get_curve_index.h $
 // $Id: Get_curve_index.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/experimental/Get_facet_patch_id.h 5.1-1/include/CGAL/Mesh_3/experimental/Get_facet_patch_id.h
--- 5.0.3-1/include/CGAL/Mesh_3/experimental/Get_facet_patch_id.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/experimental/Get_facet_patch_id.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/experimental/Get_facet_patch_id.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/experimental/Get_facet_patch_id.h $
 // $Id: Get_facet_patch_id.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/experimental/Lipschitz_sizing_experimental.h 5.1-1/include/CGAL/Mesh_3/experimental/Lipschitz_sizing_experimental.h
--- 5.0.3-1/include/CGAL/Mesh_3/experimental/Lipschitz_sizing_experimental.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/experimental/Lipschitz_sizing_experimental.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/experimental/Lipschitz_sizing_experimental.h $
-// $Id: Lipschitz_sizing_experimental.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/experimental/Lipschitz_sizing_experimental.h $
+// $Id: Lipschitz_sizing_experimental.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -310,7 +310,6 @@ public:
 
     m_own_ptree.reset(new Tree(triangles.begin(), triangles.end()));
     m_own_ptree->build();
-    m_own_ptree->accelerate_distance_queries();
   }
 
 private:
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/experimental/Lipschitz_sizing_parameters.h 5.1-1/include/CGAL/Mesh_3/experimental/Lipschitz_sizing_parameters.h
--- 5.0.3-1/include/CGAL/Mesh_3/experimental/Lipschitz_sizing_parameters.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/experimental/Lipschitz_sizing_parameters.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/experimental/Lipschitz_sizing_parameters.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/experimental/Lipschitz_sizing_parameters.h $
 // $Id: Lipschitz_sizing_parameters.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/experimental/Lipschitz_sizing_polyhedron.h 5.1-1/include/CGAL/Mesh_3/experimental/Lipschitz_sizing_polyhedron.h
--- 5.0.3-1/include/CGAL/Mesh_3/experimental/Lipschitz_sizing_polyhedron.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/experimental/Lipschitz_sizing_polyhedron.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/experimental/Lipschitz_sizing_polyhedron.h $
-// $Id: Lipschitz_sizing_polyhedron.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/experimental/Lipschitz_sizing_polyhedron.h $
+// $Id: Lipschitz_sizing_polyhedron.h 0fa0c4f 2019-12-05T11:05:35+01:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -131,7 +131,6 @@ public:
 
     m_own_ptree.reset(new Tree(triangles.begin(), triangles.end()));
     m_own_ptree->build();
-    m_own_ptree->accelerate_distance_queries();
   }
 
 private:
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/experimental/Sizing_field_minimum.h 5.1-1/include/CGAL/Mesh_3/experimental/Sizing_field_minimum.h
--- 5.0.3-1/include/CGAL/Mesh_3/experimental/Sizing_field_minimum.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/experimental/Sizing_field_minimum.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/experimental/Sizing_field_minimum.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/experimental/Sizing_field_minimum.h $
 // $Id: Sizing_field_minimum.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/experimental/Sizing_field_with_aabb_tree.h 5.1-1/include/CGAL/Mesh_3/experimental/Sizing_field_with_aabb_tree.h
--- 5.0.3-1/include/CGAL/Mesh_3/experimental/Sizing_field_with_aabb_tree.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/experimental/Sizing_field_with_aabb_tree.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/experimental/Sizing_field_with_aabb_tree.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/experimental/Sizing_field_with_aabb_tree.h $
 // $Id: Sizing_field_with_aabb_tree.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Facet_criteria_visitor_with_balls.h 5.1-1/include/CGAL/Mesh_3/Facet_criteria_visitor_with_balls.h
--- 5.0.3-1/include/CGAL/Mesh_3/Facet_criteria_visitor_with_balls.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Facet_criteria_visitor_with_balls.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Facet_criteria_visitor_with_balls.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Facet_criteria_visitor_with_balls.h $
 // $Id: Facet_criteria_visitor_with_balls.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Facet_on_same_surface_criterion.h 5.1-1/include/CGAL/Mesh_3/Facet_on_same_surface_criterion.h
--- 5.0.3-1/include/CGAL/Mesh_3/Facet_on_same_surface_criterion.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Facet_on_same_surface_criterion.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Facet_on_same_surface_criterion.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Facet_on_same_surface_criterion.h $
 // $Id: Facet_on_same_surface_criterion.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Has_features.h 5.1-1/include/CGAL/Mesh_3/Has_features.h
--- 5.0.3-1/include/CGAL/Mesh_3/Has_features.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Has_features.h	2020-09-08 12:15:08.000000000 +0000
@@ -4,8 +4,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Has_features.h $
-// $Id: Has_features.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Has_features.h $
+// $Id: Has_features.h 4dda7b6 2020-05-27T15:53:05+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Stéphane Tayeb, Laurent Rineau
@@ -13,7 +13,7 @@
 #ifndef CGAL_MESH_3_HAS_FEATURES_H
 #define CGAL_MESH_3_HAS_FEATURES_H
 
-#include <CGAL/license/Mesh_3.h>
+#include <CGAL/license/Triangulation_3.h>
 
 
 #include <boost/mpl/has_xxx.hpp>
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Image_to_labeled_function_wrapper.h 5.1-1/include/CGAL/Mesh_3/Image_to_labeled_function_wrapper.h
--- 5.0.3-1/include/CGAL/Mesh_3/Image_to_labeled_function_wrapper.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Image_to_labeled_function_wrapper.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Image_to_labeled_function_wrapper.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Image_to_labeled_function_wrapper.h $
 // $Id: Image_to_labeled_function_wrapper.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Implicit_surface_mesher_visitor.h 5.1-1/include/CGAL/Mesh_3/Implicit_surface_mesher_visitor.h
--- 5.0.3-1/include/CGAL/Mesh_3/Implicit_surface_mesher_visitor.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Implicit_surface_mesher_visitor.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Implicit_surface_mesher_visitor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Implicit_surface_mesher_visitor.h $
 // $Id: Implicit_surface_mesher_visitor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/initialize_triangulation_from_labeled_image.h 5.1-1/include/CGAL/Mesh_3/initialize_triangulation_from_labeled_image.h
--- 5.0.3-1/include/CGAL/Mesh_3/initialize_triangulation_from_labeled_image.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/initialize_triangulation_from_labeled_image.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/initialize_triangulation_from_labeled_image.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/initialize_triangulation_from_labeled_image.h $
 // $Id: initialize_triangulation_from_labeled_image.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/io_signature.h 5.1-1/include/CGAL/Mesh_3/io_signature.h
--- 5.0.3-1/include/CGAL/Mesh_3/io_signature.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/io_signature.h	2020-09-08 12:15:09.000000000 +0000
@@ -4,8 +4,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/io_signature.h $
-// $Id: io_signature.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/io_signature.h $
+// $Id: io_signature.h 4dda7b6 2020-05-27T15:53:05+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Laurent RINEAU
@@ -13,7 +13,7 @@
 #ifndef CGAL_MESH_3_IO_SIGNATURE_H
 #define CGAL_MESH_3_IO_SIGNATURE_H
 
-#include <CGAL/license/Mesh_3.h>
+#include <CGAL/license/Triangulation_3.h>
 
 #define CGAL_MESH_3_IO_H // the old include macro, tested by other files
 
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Is_mesh_domain_field_3.h 5.1-1/include/CGAL/Mesh_3/Is_mesh_domain_field_3.h
--- 5.0.3-1/include/CGAL/Mesh_3/Is_mesh_domain_field_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Is_mesh_domain_field_3.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Is_mesh_domain_field_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Is_mesh_domain_field_3.h $
 // $Id: Is_mesh_domain_field_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Lloyd_move.h 5.1-1/include/CGAL/Mesh_3/Lloyd_move.h
--- 5.0.3-1/include/CGAL/Mesh_3/Lloyd_move.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Lloyd_move.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Lloyd_move.h $
-// $Id: Lloyd_move.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Lloyd_move.h $
+// $Id: Lloyd_move.h 53d4c9b 2019-10-28T11:29:08+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -23,7 +23,7 @@
 #include <CGAL/Mesh_3/config.h>
 #include <CGAL/Mesh_3/Uniform_sizing_field.h>
 
-#include <CGAL/Hash_handles_with_or_without_timestamps.h>
+#include <CGAL/Time_stamper.h>
 #include <CGAL/convex_hull_2.h>
 #include <CGAL/ch_graham_andrew.h>
 
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Mesh_complex_3_in_triangulation_3_base.h 5.1-1/include/CGAL/Mesh_3/Mesh_complex_3_in_triangulation_3_base.h
--- 5.0.3-1/include/CGAL/Mesh_3/Mesh_complex_3_in_triangulation_3_base.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Mesh_complex_3_in_triangulation_3_base.h	2020-09-08 12:15:08.000000000 +0000
@@ -4,8 +4,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Mesh_complex_3_in_triangulation_3_base.h $
-// $Id: Mesh_complex_3_in_triangulation_3_base.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Mesh_complex_3_in_triangulation_3_base.h $
+// $Id: Mesh_complex_3_in_triangulation_3_base.h 5572d9d 2020-06-24T10:59:19+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -18,7 +18,7 @@
 #ifndef CGAL_MESH_3_MESH_COMPLEX_3_IN_TRIANGULATION_3_BASE_H
 #define CGAL_MESH_3_MESH_COMPLEX_3_IN_TRIANGULATION_3_BASE_H
 
-#include <CGAL/license/Mesh_3.h>
+#include <CGAL/license/Triangulation_3.h>
 
 #include <CGAL/disable_warnings.h>
 
@@ -31,7 +31,7 @@
 #include <CGAL/Bbox_3.h>
 #include <CGAL/Mesh_3/io_signature.h>
 #include <CGAL/Union_find.h>
-#include <CGAL/Hash_handles_with_or_without_timestamps.h>
+#include <CGAL/Time_stamper.h>
 
 #include <boost/functional/hash.hpp>
 #include <boost/unordered_map.hpp>
@@ -191,6 +191,20 @@ public:
     init(number_of_cells_, rhs.number_of_cells_);
   }
 
+  /// Move constructor
+  Mesh_complex_3_in_triangulation_3_base(Self&& rhs)
+    : Base()
+    , tr_(std::move(rhs.tr_))
+    , edge_facet_counter_(std::move(rhs.edge_facet_counter_))
+    , manifold_info_initialized_(std::exchange(rhs.manifold_info_initialized_, false))
+  {
+    Init_number_of_elements<Concurrency_tag> init;
+    init(number_of_facets_, rhs.number_of_facets_);
+    init(number_of_cells_, rhs.number_of_cells_);
+    init(rhs.number_of_facets_); // set to 0
+    init(rhs.number_of_cells_); // set to 0
+  }
+
   /// Destructor
   ~Mesh_complex_3_in_triangulation_3_base() {}
 
@@ -208,6 +222,13 @@ public:
     return *this;
   }
 
+  /// Assignment operator, also serves as move-assignment
+  Self& operator=(Self&& rhs)
+  {
+    swap(rhs);
+    return *this;
+  }
+
   /// Returns the reference to the triangulation
   Triangulation& triangulation() { return tr_; }
   /// Returns a const reference to the triangulation
@@ -861,6 +882,11 @@ private:
     {
       a = b;
     }
+    template<typename T>
+    void operator()(T& a)
+    {
+      a = 0;
+    }
   };
 
   template<typename Concurrency_tag2, typename dummy = void>
@@ -888,6 +914,11 @@ private:
     {
       a = b.load();
     }
+    template<typename T>
+    void operator()(T& a)
+    {
+      a = 0;
+    }
   };
 
   template<typename dummy>
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Mesh_complex_3_in_triangulation_3_fwd.h 5.1-1/include/CGAL/Mesh_3/Mesh_complex_3_in_triangulation_3_fwd.h
--- 5.0.3-1/include/CGAL/Mesh_3/Mesh_complex_3_in_triangulation_3_fwd.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Mesh_complex_3_in_triangulation_3_fwd.h	2020-09-08 12:14:54.000000000 +0000
@@ -0,0 +1,34 @@
+// Copyright (C) 2020  GeometryFactory Sarl
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/Mesh_3/Mesh_complex_3_in_triangulation_3_fwd.h $
+// $Id: Mesh_complex_3_in_triangulation_3_fwd.h 93d62b9 2020-08-18T14:07:27+02:00 Sébastien Loriot
+// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
+//
+
+#ifndef CGAL_MESH_3_MESH_COMPLEX_3_IN_TRIANGULATION_3_FWD_H
+#define CGAL_MESH_3_MESH_COMPLEX_3_IN_TRIANGULATION_3_FWD_H
+
+/// \file Mesh_complex_3_in_triangulation_3_fwd.h
+/// Forward declarations of the Mesh_3 package.
+
+#ifndef DOXYGEN_RUNNING
+namespace CGAL {
+
+// fwdS for the public interface
+template <typename Tr,
+          typename CornerIndex = int,
+          typename CurveIndex = int>
+class Mesh_complex_3_in_triangulation_3;
+
+template<class Tr, bool c3t3_loader_failed>
+bool build_triangulation_from_file(std::istream& is,
+                                   Tr& tr);
+
+} // CGAL
+#endif
+
+#endif /* CGAL_MESH_3_MESH_COMPLEX_3_IN_TRIANGULATION_3_FWD_H */
+
+
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Mesher_3.h 5.1-1/include/CGAL/Mesh_3/Mesher_3.h
--- 5.0.3-1/include/CGAL/Mesh_3/Mesher_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Mesher_3.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Mesher_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Mesher_3.h $
 // $Id: Mesher_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Mesher_level_default_implementations.h 5.1-1/include/CGAL/Mesh_3/Mesher_level_default_implementations.h
--- 5.0.3-1/include/CGAL/Mesh_3/Mesher_level_default_implementations.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Mesher_level_default_implementations.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Mesher_level_default_implementations.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Mesher_level_default_implementations.h $
 // $Id: Mesher_level_default_implementations.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Mesher_level.h 5.1-1/include/CGAL/Mesh_3/Mesher_level.h
--- 5.0.3-1/include/CGAL/Mesh_3/Mesher_level.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Mesher_level.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Mesher_level.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Mesher_level.h $
 // $Id: Mesher_level.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Mesh_global_optimizer.h 5.1-1/include/CGAL/Mesh_3/Mesh_global_optimizer.h
--- 5.0.3-1/include/CGAL/Mesh_3/Mesh_global_optimizer.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Mesh_global_optimizer.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Mesh_global_optimizer.h $
-// $Id: Mesh_global_optimizer.h 85712ba 2020-01-14T15:03:20+01:00 Maxime Gimeno
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Mesh_global_optimizer.h $
+// $Id: Mesh_global_optimizer.h 315a0b5 2020-01-29T16:27:53+01:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -29,8 +29,8 @@
 #include <CGAL/Origin.h>
 #include <CGAL/Mesh_optimization_return_code.h>
 #include <CGAL/Mesh_3/Null_global_optimizer_visitor.h>
+#include <CGAL/Time_stamper.h>
 
-#include <CGAL/Hash_handles_with_or_without_timestamps.h>
 #include <CGAL/iterator.h>
 #include <CGAL/tuple.h>
 
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Mesh_sizing_field.h 5.1-1/include/CGAL/Mesh_3/Mesh_sizing_field.h
--- 5.0.3-1/include/CGAL/Mesh_3/Mesh_sizing_field.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Mesh_sizing_field.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Mesh_sizing_field.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Mesh_sizing_field.h $
 // $Id: Mesh_sizing_field.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/mesh_standard_cell_criteria.h 5.1-1/include/CGAL/Mesh_3/mesh_standard_cell_criteria.h
--- 5.0.3-1/include/CGAL/Mesh_3/mesh_standard_cell_criteria.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/mesh_standard_cell_criteria.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/mesh_standard_cell_criteria.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/mesh_standard_cell_criteria.h $
 // $Id: mesh_standard_cell_criteria.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/mesh_standard_criteria.h 5.1-1/include/CGAL/Mesh_3/mesh_standard_criteria.h
--- 5.0.3-1/include/CGAL/Mesh_3/mesh_standard_criteria.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/mesh_standard_criteria.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/mesh_standard_criteria.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/mesh_standard_criteria.h $
 // $Id: mesh_standard_criteria.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/mesh_standard_facet_criteria.h 5.1-1/include/CGAL/Mesh_3/mesh_standard_facet_criteria.h
--- 5.0.3-1/include/CGAL/Mesh_3/mesh_standard_facet_criteria.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/mesh_standard_facet_criteria.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/mesh_standard_facet_criteria.h $
-// $Id: mesh_standard_facet_criteria.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/mesh_standard_facet_criteria.h $
+// $Id: mesh_standard_facet_criteria.h 58b10a3 2020-03-26T18:58:50+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -565,6 +565,10 @@ protected:
       {
         if ( !(v2->index() == index) )
         {
+#ifdef CGAL_MESH_3_DEBUG_FACET_CRITERIA
+          std::cerr << "Bad facet (on same surface criterion: "
+                    << v2->index() << " != " << index << ")" << std::endl;
+#endif
           return Is_bad(Quality(1));
         }
       }
@@ -579,6 +583,10 @@ protected:
     {
       if ( is_index_initialized && !(v3->index() == index) )
       {
+#ifdef CGAL_MESH_3_DEBUG_FACET_CRITERIA
+          std::cerr << "Bad facet (on same surface criterion: "
+                    << v3->index() << " != " << index << ")" << std::endl;
+#endif
         return Is_bad(Quality(1));
       }
     }
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Mesh_surface_cell_base_3.h 5.1-1/include/CGAL/Mesh_3/Mesh_surface_cell_base_3.h
--- 5.0.3-1/include/CGAL/Mesh_3/Mesh_surface_cell_base_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Mesh_surface_cell_base_3.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Mesh_surface_cell_base_3.h $
-// $Id: Mesh_surface_cell_base_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Mesh_surface_cell_base_3.h $
+// $Id: Mesh_surface_cell_base_3.h 4dda7b6 2020-05-27T15:53:05+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -18,7 +18,7 @@
 #ifndef CGAL_MESH_3_MESH_SURFACE_CELL_BASE_3_H
 #define CGAL_MESH_3_MESH_SURFACE_CELL_BASE_3_H
 
-#include <CGAL/license/Mesh_3.h>
+#include <CGAL/license/Triangulation_3.h>
 
 
 #include <CGAL/Mesh_3/config.h>
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/min_dihedral_angle.h 5.1-1/include/CGAL/Mesh_3/min_dihedral_angle.h
--- 5.0.3-1/include/CGAL/Mesh_3/min_dihedral_angle.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/min_dihedral_angle.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/min_dihedral_angle.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/min_dihedral_angle.h $
 // $Id: min_dihedral_angle.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Null_exuder_visitor.h 5.1-1/include/CGAL/Mesh_3/Null_exuder_visitor.h
--- 5.0.3-1/include/CGAL/Mesh_3/Null_exuder_visitor.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Null_exuder_visitor.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Null_exuder_visitor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Null_exuder_visitor.h $
 // $Id: Null_exuder_visitor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Null_global_optimizer_visitor.h 5.1-1/include/CGAL/Mesh_3/Null_global_optimizer_visitor.h
--- 5.0.3-1/include/CGAL/Mesh_3/Null_global_optimizer_visitor.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Null_global_optimizer_visitor.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Null_global_optimizer_visitor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Null_global_optimizer_visitor.h $
 // $Id: Null_global_optimizer_visitor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Null_perturber_visitor.h 5.1-1/include/CGAL/Mesh_3/Null_perturber_visitor.h
--- 5.0.3-1/include/CGAL/Mesh_3/Null_perturber_visitor.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Null_perturber_visitor.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Null_perturber_visitor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Null_perturber_visitor.h $
 // $Id: Null_perturber_visitor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Odt_move.h 5.1-1/include/CGAL/Mesh_3/Odt_move.h
--- 5.0.3-1/include/CGAL/Mesh_3/Odt_move.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Odt_move.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Odt_move.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Odt_move.h $
 // $Id: Odt_move.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/parameters_defaults.h 5.1-1/include/CGAL/Mesh_3/parameters_defaults.h
--- 5.0.3-1/include/CGAL/Mesh_3/parameters_defaults.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/parameters_defaults.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/parameters_defaults.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/parameters_defaults.h $
 // $Id: parameters_defaults.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Poisson_refine_cells_3.h 5.1-1/include/CGAL/Mesh_3/Poisson_refine_cells_3.h
--- 5.0.3-1/include/CGAL/Mesh_3/Poisson_refine_cells_3.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Poisson_refine_cells_3.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Poisson_surface_reconstruction_3/include/CGAL/Mesh_3/Poisson_refine_cells_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Poisson_surface_reconstruction_3/include/CGAL/Mesh_3/Poisson_refine_cells_3.h $
 // $Id: Poisson_refine_cells_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/polyhedral_to_labeled_function_wrapper.h 5.1-1/include/CGAL/Mesh_3/polyhedral_to_labeled_function_wrapper.h
--- 5.0.3-1/include/CGAL/Mesh_3/polyhedral_to_labeled_function_wrapper.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/polyhedral_to_labeled_function_wrapper.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/polyhedral_to_labeled_function_wrapper.h $
-// $Id: polyhedral_to_labeled_function_wrapper.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/polyhedral_to_labeled_function_wrapper.h $
+// $Id: polyhedral_to_labeled_function_wrapper.h 0fa0c4f 2019-12-05T11:05:35+01:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -90,7 +90,7 @@ public:
     , step_size_(step_size)
     , first_level_(first_level)
   {
-    tree_->accelerate_distance_queries();
+
   }
 
   // Default copy constructor and assignment operator are ok
@@ -207,7 +207,6 @@ public:
     , sq_tolerance_size_(tolerance_size*tolerance_size)
     , hint_(p.facets_begin()->halfedge()->vertex()->point())
   {
-    tree_->accelerate_distance_queries();
   }
 
   // Default copy constructor and assignment operator are ok
@@ -283,7 +282,6 @@ public:
     , sq_tolerance_size_(tolerance_size*tolerance_size)
     , hint_(p.facets_begin()->halfedge()->vertex()->point())
   {
-    tree_->accelerate_distance_queries();
   }
 
   // Default copy constructor and assignment operator are ok
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/polylines_to_protect.h 5.1-1/include/CGAL/Mesh_3/polylines_to_protect.h
--- 5.0.3-1/include/CGAL/Mesh_3/polylines_to_protect.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/polylines_to_protect.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/polylines_to_protect.h $
-// $Id: polylines_to_protect.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/polylines_to_protect.h $
+// $Id: polylines_to_protect.h daf2e68 2020-01-09T16:52:05+01:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -264,7 +264,7 @@ struct Polyline_visitor
 
   ~Polyline_visitor()
   {//DEBUG
-#if CGAL_MESH_3_PROTECTION_DEBUG > 1
+#if CGAL_MESH_3_PROTECTION_DEBUG & 2
     std::ofstream og("polylines_graph.polylines.txt");
     og.precision(17);
     for(const std::vector<P>& poly : polylines)
@@ -274,7 +274,7 @@ struct Polyline_visitor
         og << p << " ";
       og << std::endl;
     }
-#endif // CGAL_MESH_3_PROTECTION_DEBUG > 1
+#endif // CGAL_MESH_3_PROTECTION_DEBUG & 2
   }
 
   void start_new_polyline()
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Polyline_with_context.h 5.1-1/include/CGAL/Mesh_3/Polyline_with_context.h
--- 5.0.3-1/include/CGAL/Mesh_3/Polyline_with_context.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Polyline_with_context.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Polyline_with_context.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Polyline_with_context.h $
 // $Id: Polyline_with_context.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Profile_counter.h 5.1-1/include/CGAL/Mesh_3/Profile_counter.h
--- 5.0.3-1/include/CGAL/Mesh_3/Profile_counter.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Profile_counter.h	2020-09-08 12:15:08.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Profile_counter.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Profile_counter.h $
 // $Id: Profile_counter.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Profiling_tools.h 5.1-1/include/CGAL/Mesh_3/Profiling_tools.h
--- 5.0.3-1/include/CGAL/Mesh_3/Profiling_tools.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Profiling_tools.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Profiling_tools.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Profiling_tools.h $
 // $Id: Profiling_tools.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Protect_edges_sizing_field.h 5.1-1/include/CGAL/Mesh_3/Protect_edges_sizing_field.h
--- 5.0.3-1/include/CGAL/Mesh_3/Protect_edges_sizing_field.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Protect_edges_sizing_field.h	2020-09-08 12:15:08.000000000 +0000
@@ -4,8 +4,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Protect_edges_sizing_field.h $
-// $Id: Protect_edges_sizing_field.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Protect_edges_sizing_field.h $
+// $Id: Protect_edges_sizing_field.h 612d677 2020-07-07T11:13:16+02:00 Jane Tournois
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -41,8 +41,7 @@
 #endif
 
 #include <CGAL/enum.h>
-#include <CGAL/Has_timestamp.h>
-#include <CGAL/Hash_handles_with_or_without_timestamps.h>
+#include <CGAL/Time_stamper.h>
 #include <CGAL/internal/Has_member_visited.h>
 #include <CGAL/iterator.h>
 #include <CGAL/number_utils.h>
@@ -445,7 +444,14 @@ private:
       if(dim == 0) msg << "corner (";
       else msg << "point (";
       msg << p << ")";
+#if CGAL_MESH_3_PROTECTION_DEBUG & 4
+      CGAL_error_msg(([this, str = msg.str()](){
+        dump_c3t3(this->c3t3_, "dump-bug");
+        return str.c_str();
+      }()));
+#else // not CGAL_MESH_3_PROTECTION_DEBUG & 4
       CGAL_error_msg(msg.str().c_str());
+#endif
     }
     return s;
   }
@@ -731,7 +737,7 @@ smart_insert_point(const Bare_point& p,
     Vertex_handle nearest_vh = tr.nearest_power_vertex(p, ch);
     FT sq_d = sq_distance(p, cp(tr.point(nearest_vh)));
 
-#if CGAL_MESH_3_PROTECTION_DEBUG & 2
+#if CGAL_MESH_3_PROTECTION_DEBUG & 16
     std::cerr << "Nearest power vertex of (" << p << ") is "
               << &*nearest_vh << " (" << c3t3_.triangulation().point(nearest_vh) << ") "
               << "at distance: " << sq_d << std::endl;
@@ -1149,6 +1155,7 @@ insert_balls(const Vertex_handle& vp,
   //   n = 2(d-sq) / (sp+sq)
   // =======================
 
+  const FT d_signF = static_cast<FT>(d_sign);
   int n = static_cast<int>(std::floor(FT(2)*(d-sq) / (sp+sq))+.5);
   // if( minimal_weight_ != 0 && n == 0 ) return;
 
@@ -1174,7 +1181,7 @@ insert_balls(const Vertex_handle& vp,
       const Bare_point new_point =
         domain_.construct_point_on_curve(cp(vp_wp),
                                          curve_index,
-                                         d_sign * d / 2);
+                                         d_signF * d / 2);
       const int dim = 1; // new_point is on edge
       const Index index = domain_.index_from_curve_index(curve_index);
       const FT point_weight = CGAL::square(size_(new_point, dim, index));
@@ -1224,7 +1231,6 @@ insert_balls(const Vertex_handle& vp,
   FT norm_step_size = dleft_frac * step_size;
 
   // Initial distance
-  FT d_signF = static_cast<FT>(d_sign);
   FT pt_dist = d_signF * norm_step_size;
   Vertex_handle prev = vp;
   const Bare_point& p = cp(c3t3_.triangulation().point(vp));
@@ -1248,8 +1254,17 @@ insert_balls(const Vertex_handle& vp,
     norm_step_size = step_size;
   } else {
     CGAL_assertion_code(using boost::math::float_prior);
-    CGAL_assertion(n==0 ||
-                   dleft_frac >= float_prior(float_prior(1.)));
+#if CGAL_MESH_3_PROTECTION_DEBUG & 4
+    CGAL_assertion_msg(n==0 ||
+                       dleft_frac >= float_prior(float_prior(1.)),
+                       ([this](){
+                         CGAL_USE(this);
+                         dump_c3t3(this->c3t3_, "dump-bug");
+                         return "the sampling of protecting balls is not possible";
+                       }()));
+#else
+    CGAL_assertion(n==0 || dleft_frac >= float_prior(float_prior(1.)));
+#endif
   }
 
   // Launch balls
@@ -1305,7 +1320,7 @@ void
 Protect_edges_sizing_field<C3T3, MD, Sf>::
 refine_balls()
 {
-#if CGAL_MESH_3_PROTECTION_DEBUG & 4
+#if CGAL_MESH_3_PROTECTION_DEBUG & 8
   dump_c3t3(c3t3_, "dump-before-refine_balls");
   dump_c3t3_edges(c3t3_, "dump-before-refine_balls");
 #endif
@@ -1419,7 +1434,7 @@ refine_balls()
       }
     }
 
-#if CGAL_MESH_3_PROTECTION_DEBUG & 4
+#if CGAL_MESH_3_PROTECTION_DEBUG & 8
     dump_c3t3(c3t3_, "dump-before-check_and_repopulate_edges");
     dump_c3t3_edges(c3t3_, "dump-before-check_and_repopulate_edges");
 #endif
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/radius_ratio.h 5.1-1/include/CGAL/Mesh_3/radius_ratio.h
--- 5.0.3-1/include/CGAL/Mesh_3/radius_ratio.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/radius_ratio.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/radius_ratio.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/radius_ratio.h $
 // $Id: radius_ratio.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Refine_cells_3.h 5.1-1/include/CGAL/Mesh_3/Refine_cells_3.h
--- 5.0.3-1/include/CGAL/Mesh_3/Refine_cells_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Refine_cells_3.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Refine_cells_3.h $
-// $Id: Refine_cells_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Refine_cells_3.h $
+// $Id: Refine_cells_3.h 58b10a3 2020-03-26T18:58:50+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Laurent Rineau, Stéphane Tayeb
@@ -434,6 +434,7 @@ public:
    && !defined(CGAL_MESH_3_USE_LAZY_UNSORTED_REFINEMENT_QUEUE)
     this->remove_element(c);
   #endif
+    CGAL_USE(c);
   }
   // Parallel: it's always lazy, so do nothing
   void remove_element_from_refinement_queue(Cell_handle, Parallel_tag) {}
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Refine_facets_3.h 5.1-1/include/CGAL/Mesh_3/Refine_facets_3.h
--- 5.0.3-1/include/CGAL/Mesh_3/Refine_facets_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Refine_facets_3.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Refine_facets_3.h $
-// $Id: Refine_facets_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Refine_facets_3.h $
+// $Id: Refine_facets_3.h a4991fd 2020-01-20T18:17:14+01:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -38,6 +38,7 @@
 #ifdef CGAL_MESH_3_PROFILING
   #include <CGAL/Mesh_3/Profiling_tools.h>
 #endif
+#include <CGAL/Mesh_3/Dump_c3t3.h>
 
 #include <CGAL/Object.h>
 #include <CGAL/atomic.h>
@@ -570,6 +571,7 @@ protected:
     Facet canonical_facet = this->canonical_facet(facet);
     this->remove_element(canonical_facet);
 #endif
+    CGAL_USE(facet);
   }
 #ifdef CGAL_LINKED_WITH_TBB
   /// Removes facet from refinement queue
@@ -846,8 +848,12 @@ public:
     if( zone.locate_type == Tr::VERTEX )
     {
       std::stringstream sstr;
+      sstr.precision(17);
       sstr << "(" << p << ") is already inserted on surface.\n";
-      CGAL_error_msg(sstr.str().c_str());
+      CGAL_error_msg(([this,str=sstr.str()] {
+        dump_c3t3(this->r_c3t3_, "dump-bug");
+        return str.c_str();
+      }()));
       return CONFLICT_AND_ELEMENT_SHOULD_BE_DROPPED;
     }
     else
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Refine_facets_manifold_base.h 5.1-1/include/CGAL/Mesh_3/Refine_facets_manifold_base.h
--- 5.0.3-1/include/CGAL/Mesh_3/Refine_facets_manifold_base.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Refine_facets_manifold_base.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Refine_facets_manifold_base.h $
-// $Id: Refine_facets_manifold_base.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Refine_facets_manifold_base.h $
+// $Id: Refine_facets_manifold_base.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -19,9 +19,9 @@
 
 #include <CGAL/Mesh_facet_topology.h>
 
-#include <CGAL/Hash_handles_with_or_without_timestamps.h>
-#include <CGAL/utility.h>
 #include <CGAL/atomic.h>
+#include <CGAL/utility.h>
+#include <CGAL/Time_stamper.h>
 
 #include <boost/bimap.hpp>
 #include <boost/bimap/unordered_set_of.hpp>
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Refine_tets_visitor.h 5.1-1/include/CGAL/Mesh_3/Refine_tets_visitor.h
--- 5.0.3-1/include/CGAL/Mesh_3/Refine_tets_visitor.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Refine_tets_visitor.h	2020-09-08 12:15:08.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Refine_tets_visitor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Refine_tets_visitor.h $
 // $Id: Refine_tets_visitor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Robust_intersection_kernel.h 5.1-1/include/CGAL/Mesh_3/Robust_intersection_kernel.h
--- 5.0.3-1/include/CGAL/Mesh_3/Robust_intersection_kernel.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Robust_intersection_kernel.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Robust_intersection_kernel.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Robust_intersection_kernel.h $
 // $Id: Robust_intersection_kernel.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Robust_intersection_traits_3.h 5.1-1/include/CGAL/Mesh_3/Robust_intersection_traits_3.h
--- 5.0.3-1/include/CGAL/Mesh_3/Robust_intersection_traits_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Robust_intersection_traits_3.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Robust_intersection_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Robust_intersection_traits_3.h $
 // $Id: Robust_intersection_traits_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/search_for_connected_components_in_labeled_image.h 5.1-1/include/CGAL/Mesh_3/search_for_connected_components_in_labeled_image.h
--- 5.0.3-1/include/CGAL/Mesh_3/search_for_connected_components_in_labeled_image.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/search_for_connected_components_in_labeled_image.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/search_for_connected_components_in_labeled_image.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/search_for_connected_components_in_labeled_image.h $
 // $Id: search_for_connected_components_in_labeled_image.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Sizing_grid.h 5.1-1/include/CGAL/Mesh_3/Sizing_grid.h
--- 5.0.3-1/include/CGAL/Mesh_3/Sizing_grid.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Sizing_grid.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Sizing_grid.h $
-// $Id: Sizing_grid.h 5b2d46c 2020-04-21T10:03:00+02:00 Maxime Gimeno
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Sizing_grid.h $
+// $Id: Sizing_grid.h e9d41d7 2020-04-21T10:03:00+02:00 Maxime Gimeno
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/sliver_criteria.h 5.1-1/include/CGAL/Mesh_3/sliver_criteria.h
--- 5.0.3-1/include/CGAL/Mesh_3/sliver_criteria.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/sliver_criteria.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/sliver_criteria.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/sliver_criteria.h $
 // $Id: sliver_criteria.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Sliver_perturber.h 5.1-1/include/CGAL/Mesh_3/Sliver_perturber.h
--- 5.0.3-1/include/CGAL/Mesh_3/Sliver_perturber.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Sliver_perturber.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Sliver_perturber.h $
-// $Id: Sliver_perturber.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Sliver_perturber.h $
+// $Id: Sliver_perturber.h 53d4c9b 2019-10-28T11:29:08+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -42,8 +42,7 @@
 #include <CGAL/Real_timer.h>
 #include <CGAL/Mesh_3/Null_perturber_visitor.h>
 #include <CGAL/Mesh_3/sliver_criteria.h>
-#include <CGAL/Has_timestamp.h>
-#include <CGAL/Hash_handles_with_or_without_timestamps.h>
+#include <CGAL/Time_stamper.h>
 
 #include <CGAL/Mesh_3/Concurrent_mesher_config.h>
 #include <CGAL/Mesh_3/Worksharing_data_structures.h>
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Slivers_exuder_cell_attributes_traits.h 5.1-1/include/CGAL/Mesh_3/Slivers_exuder_cell_attributes_traits.h
--- 5.0.3-1/include/CGAL/Mesh_3/Slivers_exuder_cell_attributes_traits.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Slivers_exuder_cell_attributes_traits.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Slivers_exuder_cell_attributes_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Slivers_exuder_cell_attributes_traits.h $
 // $Id: Slivers_exuder_cell_attributes_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Slivers_exuder.h 5.1-1/include/CGAL/Mesh_3/Slivers_exuder.h
--- 5.0.3-1/include/CGAL/Mesh_3/Slivers_exuder.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Slivers_exuder.h	2020-09-08 12:15:09.000000000 +0000
@@ -5,7 +5,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Slivers_exuder.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Slivers_exuder.h $
 // $Id: Slivers_exuder.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/squared_distance_Point_3_Triangle_3.h 5.1-1/include/CGAL/Mesh_3/squared_distance_Point_3_Triangle_3.h
--- 5.0.3-1/include/CGAL/Mesh_3/squared_distance_Point_3_Triangle_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/squared_distance_Point_3_Triangle_3.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/squared_distance_Point_3_Triangle_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/squared_distance_Point_3_Triangle_3.h $
 // $Id: squared_distance_Point_3_Triangle_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/tet_soup_to_c3t3.h 5.1-1/include/CGAL/Mesh_3/tet_soup_to_c3t3.h
--- 5.0.3-1/include/CGAL/Mesh_3/tet_soup_to_c3t3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/tet_soup_to_c3t3.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/tet_soup_to_c3t3.h $
-// $Id: tet_soup_to_c3t3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/tet_soup_to_c3t3.h $
+// $Id: tet_soup_to_c3t3.h 2a4b4a2 2020-07-03T07:35:20+02:00 Jane Tournois
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -17,18 +17,32 @@
 #ifndef CGAL_MESH_3_TET_SOUP_TO_C3T3_H
 #define CGAL_MESH_3_TET_SOUP_TO_C3T3_H
 
-#include <CGAL/license/Mesh_3.h>
+#include <CGAL/license/Triangulation_3.h>
 
 #include <CGAL/disable_warnings.h>
 #include <CGAL/assertions.h>
 #include <CGAL/IO/File_medit.h>
 
-#include <boost/array.hpp>
+#include <array>
+#include <vector>
+#include <utility>
+#include <map>
 #include <boost/unordered_map.hpp>
 
 
 namespace CGAL
 {
+
+template<typename Vh>
+std::array<Vh, 3> make_ordered_vertex_array(const Vh vh0, const Vh vh1, const Vh vh2)
+{
+  std::array<Vh, 3> ft = { {vh0, vh1, vh2} };
+  if (ft[1] < ft[0]) std::swap(ft[0], ft[1]);
+  if (ft[2] < ft[1]) std::swap(ft[1], ft[2]);
+  if (ft[1] < ft[0]) std::swap(ft[0], ft[1]);
+  return ft;
+}
+
 template<class Tr>
 void build_vertices(Tr& tr,
                     const std::vector<typename Tr::Point>& points,
@@ -49,22 +63,22 @@ void build_vertices(Tr& tr,
 }
 
 template<class Tr>
-void add_facet_to_incident_cells_map(const typename Tr::Cell_handle c, int i,
-                                     boost::unordered_map<std::set<typename Tr::Vertex_handle>,
-                                     std::vector<std::pair<typename Tr::Cell_handle, int> > >& incident_cells_map)
+bool add_facet_to_incident_cells_map(const typename Tr::Cell_handle c, int i,
+    boost::unordered_map<std::array<typename Tr::Vertex_handle, 3>,
+                         std::vector<std::pair<typename Tr::Cell_handle, int> > >& incident_cells_map,
+    const bool verbose)
 {
   typedef typename Tr::Vertex_handle                                Vertex_handle;
   typedef typename Tr::Cell_handle                                  Cell_handle;
-  typedef std::set<Vertex_handle>                                   Facet;
+  typedef std::array<Vertex_handle, 3>                              Facet_vvv;
   typedef std::pair<Cell_handle, int>                               Incident_cell;
-  typedef boost::unordered_map<Facet, std::vector<Incident_cell> >  Incident_cells_map;
+  typedef boost::unordered_map<Facet_vvv, std::vector<Incident_cell> > Incident_cells_map;
 
   // the opposite vertex of f in c is i
-  Facet f;
-  f.insert(c->vertex((i + 1) % 4));
-  f.insert(c->vertex((i + 2) % 4));
-  f.insert(c->vertex((i + 3) % 4));
-  CGAL_precondition(f.size() == 3);
+  Facet_vvv f = make_ordered_vertex_array(c->vertex((i + 1) % 4),
+                                          c->vertex((i + 2) % 4),
+                                          c->vertex((i + 3) % 4));
+  CGAL_precondition(f[0] != f[1] && f[1] != f[2]);
 
   Incident_cell e = std::make_pair(c, i);
   std::vector<Incident_cell> vec;
@@ -74,23 +88,31 @@ void add_facet_to_incident_cells_map(con
   if(!is_insert_successful.second) // the entry already exists in the map
   {
     // a facet must have exactly two incident cells
-    CGAL_assertion(is_insert_successful.first->second.size() == 1);
+    if (is_insert_successful.first->second.size() != 1)
+    {
+      if(verbose)
+        std::cout << "Error in add_facet_to_incident_cells_map" << std::endl;
+      return false;
+    }
     is_insert_successful.first->second.push_back(e);
   }
+  return true;
 }
 
 template<class Tr>
-void build_finite_cells(Tr& tr,
-                        const std::vector<boost::array<int,5> >& finite_cells,
-                        const std::vector<typename Tr::Vertex_handle>& vertex_handle_vector,
-                        boost::unordered_map<std::set<typename Tr::Vertex_handle>,
+bool build_finite_cells(Tr& tr,
+    const std::vector<std::array<int,5> >& finite_cells,
+    const std::vector<typename Tr::Vertex_handle>& vertex_handle_vector,
+    boost::unordered_map<std::array<typename Tr::Vertex_handle, 3>,
                         std::vector<std::pair<typename Tr::Cell_handle, int> > >& incident_cells_map,
-                        const std::map<boost::array<int,3>, int>& border_facets)
+    const std::map<std::array<int,3>, typename Tr::Cell::Surface_patch_index>& border_facets,
+    const bool verbose)
 {
-  typedef boost::array<int, 5>              Tet_with_ref; // 4 ids + 1 reference
+  typedef std::array<int, 5>              Tet_with_ref; // 4 ids + 1 reference
 
   typedef typename Tr::Vertex_handle                            Vertex_handle;
   typedef typename Tr::Cell_handle                              Cell_handle;
+  typedef typename Tr::Cell::Surface_patch_index                Surface_patch_index;
 
   CGAL_assertion_code(
     typename Tr::Geom_traits::Construct_point_3 cp =
@@ -101,7 +123,7 @@ void build_finite_cells(Tr& tr,
   for(std::size_t i=0; i<finite_cells.size(); ++i)
   {
     const Tet_with_ref& tet = finite_cells[i];
-    boost::array<Vertex_handle, 4> vs;
+    std::array<Vertex_handle, 4> vs;
 
     for(int j=0; j<4; ++j)
     {
@@ -118,9 +140,8 @@ void build_finite_cells(Tr& tr,
                      == POSITIVE);
 
     Cell_handle c = tr.tds().create_cell(vs[0], vs[1], vs[2], vs[3]);
-    c->info() = tet[4]; // the cell's info keeps the reference of the tetrahedron
+    c->set_subdomain_index(tet[4]); // the cell's info keeps the reference of the tetrahedron
 
-    CGAL_precondition(tet[4] > 0);
     // assign cells to vertices
     for(int j=0; j<4; ++j)
     {
@@ -131,17 +152,18 @@ void build_finite_cells(Tr& tr,
     // build the map used for adjacency later
     for(int j=0; j<4; ++j)
     {
-      add_facet_to_incident_cells_map<Tr>(c, j, incident_cells_map);
+      if(!add_facet_to_incident_cells_map<Tr>(c, j, incident_cells_map, verbose))
+        return false;
       if(border_facets.size() != 0)
       {
-        boost::array<int,3> facet;
+        std::array<int,3> facet;
         facet[0]=tet[(j+1) % 4];
         facet[1]=tet[(j+2) % 4];
         facet[2]=tet[(j+3) % 4];
         //find the circular permutation that puts the smallest index in the first place.
         int n0 = (std::min)((std::min)(facet[0], facet[1]), facet[2]);
         int k=0;
-        boost::array<int,3> f;
+        std::array<int,3> f;
         do
         {
           f[0]=facet[(0+k)%3];
@@ -150,7 +172,8 @@ void build_finite_cells(Tr& tr,
           ++k;
         } while(f[0] != n0);
 
-        typename std::map<boost::array<int,3>, int>::const_iterator it = border_facets.find(f);
+        typename std::map<std::array<int,3>, Surface_patch_index>::const_iterator
+          it = border_facets.find(f);
         if(it != border_facets.end())
         {
           c->set_surface_patch_index(j, it->second);
@@ -165,39 +188,43 @@ void build_finite_cells(Tr& tr,
           if(it != border_facets.end())
             c->set_surface_patch_index(j, it->second);
           else
-            c->set_surface_patch_index(j, 0);
+            c->set_surface_patch_index(j, Surface_patch_index());
         }
       }
     }
   }
+  return true;
 }
 
 template<class Tr>
-void add_infinite_facets_to_incident_cells_map(typename Tr::Cell_handle c,
-                                               int inf_vert_pos,
-                                               std::map<std::set<typename Tr::Vertex_handle>,
-                                               std::vector<std::pair<typename Tr::Cell_handle,
-                                               int> > >& incident_cells_map)
+bool add_infinite_facets_to_incident_cells_map(typename Tr::Cell_handle c,
+     int inf_vert_pos,
+     boost::unordered_map<std::array<typename Tr::Vertex_handle, 3>,
+                          std::vector<std::pair<typename Tr::Cell_handle, int> > >& incident_cells_map,
+     const bool verbose)
 {
   int l = (inf_vert_pos + 1) % 4;
-  add_facet_to_incident_cells_map<Tr>(c, l, incident_cells_map);
+  bool b1 = add_facet_to_incident_cells_map<Tr>(c, l, incident_cells_map, verbose);
   l = (inf_vert_pos + 2) % 4;
-  add_facet_to_incident_cells_map<Tr>(c, l, incident_cells_map);
+  bool b2 = add_facet_to_incident_cells_map<Tr>(c, l, incident_cells_map, verbose);
   l = (inf_vert_pos + 3) % 4;
-  add_facet_to_incident_cells_map<Tr>(c, l, incident_cells_map);
+  bool b3 = add_facet_to_incident_cells_map<Tr>(c, l, incident_cells_map, verbose);
+  return b1 && b2 && b3;
 }
 
 template<class Tr>
-void build_infinite_cells(Tr& tr,
-                          boost::unordered_map<std::set<typename Tr::Vertex_handle>,
-                          std::vector<std::pair<typename Tr::Cell_handle,
-                          int> > >& incident_cells_map)
+bool build_infinite_cells(Tr& tr,
+  boost::unordered_map<std::array<typename Tr::Vertex_handle, 3>,
+                       std::vector<std::pair<typename Tr::Cell_handle, int> > >& incident_cells_map,
+  const bool verbose)
 {
   typedef typename Tr::Vertex_handle                               Vertex_handle;
   typedef typename Tr::Cell_handle                                 Cell_handle;
-  typedef std::set<Vertex_handle>                                  Facet;
+  typedef std::array<Vertex_handle, 3>                             Facet_vvv;
   typedef std::pair<Cell_handle, int>                              Incident_cell;
-  typedef boost::unordered_map<Facet, std::vector<Incident_cell> > Incident_cells_map;
+  typedef boost::unordered_map<Facet_vvv, std::vector<Incident_cell> > Incident_cells_map;
+
+  std::vector<Cell_handle> infinite_cells;
 
   // check the incident cells map for facets who only have one incident cell
   // and build the infinite cell on the opposite side
@@ -214,41 +241,69 @@ void build_infinite_cells(Tr& tr,
 
     Cell_handle opp_c;
     // the infinite cell that we are creating needs to be well oriented...
-    int inf_vert_position_in_opp_c = 0;
     if(i == 0 || i == 2)
       opp_c = tr.tds().create_cell(tr.infinite_vertex(),
-                                   c->vertex((i+1)%4),
-                                   c->vertex((i+2)%4),
-                                   c->vertex((i+3)%4));
+                                   c->vertex((i + 2) % 4),
+                                   c->vertex((i + 1) % 4),
+                                   c->vertex((i + 3) % 4));
     else
       opp_c = tr.tds().create_cell(tr.infinite_vertex(),
-                                   c->vertex((i+1)%4),
-                                   c->vertex((i+3)%4),
-                                   c->vertex((i+2)%4));
+                                   c->vertex((i + 3) % 4),
+                                   c->vertex((i + 1) % 4),
+                                   c->vertex((i + 2) % 4));
+
+    infinite_cells.push_back(opp_c);
 
     // set the infinite_vertex's incident cell
     if(tr.infinite_vertex()->cell() == Cell_handle())
       tr.infinite_vertex()->set_cell(opp_c);
 
-    // add the facets to the incident cells map
-
     // the only finite facet
-    it->second.push_back(std::make_pair(opp_c, inf_vert_position_in_opp_c));
+    it->second.push_back(std::make_pair(opp_c, 0));
     CGAL_assertion(it->second.size() == 2);
+
+    opp_c->set_surface_patch_index(0, c->surface_patch_index(i));
   }
+
+#ifdef CGAL_TET_SOUP_TO_C3T3_DEBUG
+  for (auto icit : incident_cells_map)
+    CGAL_assertion(icit.second.size() == 2);
+
+  std::map<Facet_vvv, int> facets;
+  for (const Cell_handle c : infinite_cells)
+  {
+    for (int i = 1; i < 4; ++i)
+    {
+      std::array<Vertex_handle, 3> vs = make_ordered_vertex_array(c->vertex((i + 1) % 4),
+        c->vertex((i + 2) % 4),
+        c->vertex((i + 3) % 4));
+      if (facets.find(vs) == facets.end())
+        facets.insert(std::make_pair(vs, 1));
+      else
+        facets[vs]++;
+    }
+  }
+  for (auto fp : facets)
+    CGAL_assertion(fp.second == 2);
+#endif
+
+  // add the facets to the incident cells map
+  for (const Cell_handle c : infinite_cells)
+    if(!add_infinite_facets_to_incident_cells_map<Tr>(c, 0, incident_cells_map, verbose))
+      return false;
+
+  return true;
 }
 
 template<class Tr>
 bool assign_neighbors(Tr& tr,
-                      const boost::unordered_map<std::set<typename Tr::Vertex_handle>,
-                      std::vector<std::pair<typename Tr::Cell_handle,
-                      int> > >& incident_cells_map)
+  const boost::unordered_map<std::array<typename Tr::Vertex_handle, 3>,
+                             std::vector<std::pair<typename Tr::Cell_handle, int> > >& incident_cells_map)
 {
-  typedef typename Tr::Vertex_handle                                 Vertex_handle;
   typedef typename Tr::Cell_handle                                   Cell_handle;
-  typedef std::set<Vertex_handle>                                    Facet;
   typedef std::pair<Cell_handle, int>                                Incident_cell;
-  typedef boost::unordered_map<Facet, std::vector<Incident_cell> >   Incident_cells_map;
+  typedef boost::unordered_map<std::array<typename Tr::Vertex_handle, 3>,
+                               std::vector<Incident_cell> >          Incident_cells_map;
 
   typename Incident_cells_map::const_iterator icit = incident_cells_map.begin();
   for(; icit!=incident_cells_map.end(); ++icit)
@@ -270,19 +325,23 @@ bool assign_neighbors(Tr& tr,
 template<class Tr, bool c3t3_loader_failed>
 bool build_triangulation(Tr& tr,
                          const std::vector<typename Tr::Point>& points,
-                         const std::vector<boost::array<int,5> >& finite_cells,
-                         const std::map<boost::array<int,3>, int>& border_facets)
+                         const std::vector<std::array<int,5> >& finite_cells,
+                         const std::map<std::array<int,3>, typename Tr::Cell::Surface_patch_index>& border_facets,
+                         std::vector<typename Tr::Vertex_handle>& vertex_handle_vector,
+                         const bool verbose = false)
 {
   typedef typename Tr::Vertex_handle            Vertex_handle;
   typedef typename Tr::Cell_handle              Cell_handle;
-  typedef std::set<Vertex_handle>               Facet;
+  typedef std::array<Vertex_handle, 3>          Facet_vvv;
 
   // associate to a face the two (at most) incident tets and the id of the face in the cell
   typedef std::pair<Cell_handle, int>                   Incident_cell;
-  typedef boost::unordered_map<Facet, std::vector<Incident_cell> >  Incident_cells_map;
+  typedef boost::unordered_map<Facet_vvv, std::vector<Incident_cell> >  Incident_cells_map;
 
   Incident_cells_map incident_cells_map;
-  std::vector<Vertex_handle> vertex_handle_vector(points.size() + 1); // id to vertex_handle
+  vertex_handle_vector.resize(points.size() + 1); // id to vertex_handle
+                                        //index 0 is for infinite vertex
+                                        // 1 to n for points in `points`
 
   CGAL_precondition(!points.empty());
 
@@ -298,22 +357,29 @@ bool build_triangulation(Tr& tr,
   {
     vh->set_dimension(-1);
   }
-  if(!finite_cells.empty())
+  if (!finite_cells.empty())
   {
-    build_finite_cells<Tr>(tr, finite_cells, vertex_handle_vector, incident_cells_map, border_facets);
-    build_infinite_cells<Tr>(tr, incident_cells_map);
+    if(!build_finite_cells<Tr>(tr, finite_cells, vertex_handle_vector, incident_cells_map,
+                           border_facets, verbose))
+      return false;
+    if(!build_infinite_cells<Tr>(tr, incident_cells_map, verbose))
+      return false;
     tr.tds().set_dimension(3);
-    if(!assign_neighbors<Tr>(tr, incident_cells_map))
+    if (!assign_neighbors<Tr>(tr, incident_cells_map))
       return false;
-    std::cout << "built triangulation : " << std::endl;
-    std::cout << tr.number_of_cells() << " cells" << std::endl;
+    if (verbose)
+    {
+      std::cout << "built triangulation : " << std::endl;
+      std::cout << tr.number_of_cells() << " cells" << std::endl;
+    }
   }
-  std::cout << tr.number_of_vertices() << " vertices" << std::endl;
+  if(verbose)
+    std::cout << tr.number_of_vertices() << " vertices" << std::endl;
 
   if(c3t3_loader_failed)
     return true;
   else
-    return tr.is_valid(true);
+    return tr.tds().is_valid();
 }
 
 template<class Tr, bool c3t3_loader_failed>
@@ -322,12 +388,12 @@ bool build_triangulation_from_file(std::
 {
   typedef typename Tr::Point                                  Point_3;
 
-  typedef boost::array<int, 3> Facet; // 3 = id
-  typedef boost::array<int, 5> Tet_with_ref; // first 4 = id, fifth = reference
+  typedef std::array<int, 3> Facet; // 3 = id
+  typedef std::array<int, 5> Tet_with_ref; // first 4 = id, fifth = reference
 
   std::vector<Tet_with_ref> finite_cells;
   std::vector<Point_3> points;
-  std::map<Facet, int> border_facets;
+  std::map<Facet, typename Tr::Cell::Surface_patch_index> border_facets;
 
   // grab the vertices
   int dim;
@@ -358,7 +424,8 @@ bool build_triangulation_from_file(std::
       is >> nf;
       for(int i=0; i<nf; ++i)
       {
-        int n1, n2, n3, surface_patch_id;
+        int n1, n2, n3;
+        typename Tr::Cell::Surface_patch_index surface_patch_id;
         is >> n1 >> n2 >> n3 >> surface_patch_id;
         Facet facet;
         facet[0] = n1 - 1;
@@ -404,7 +471,9 @@ bool build_triangulation_from_file(std::
   if(finite_cells.empty())
     return false;
 
-  bool is_well_built = build_triangulation<Tr, c3t3_loader_failed>(tr, points, finite_cells, border_facets);
+  std::vector<typename Tr::Vertex_handle> vertices(points.size() + 1);
+  bool is_well_built = build_triangulation<Tr, c3t3_loader_failed>(tr,
+    points, finite_cells, border_facets, vertices);
   return is_well_built;
 }
 
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Triangle_accessor_primitive.h 5.1-1/include/CGAL/Mesh_3/Triangle_accessor_primitive.h
--- 5.0.3-1/include/CGAL/Mesh_3/Triangle_accessor_primitive.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Triangle_accessor_primitive.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Triangle_accessor_primitive.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Triangle_accessor_primitive.h $
 // $Id: Triangle_accessor_primitive.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Triangulation_helpers.h 5.1-1/include/CGAL/Mesh_3/Triangulation_helpers.h
--- 5.0.3-1/include/CGAL/Mesh_3/Triangulation_helpers.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Triangulation_helpers.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Triangulation_helpers.h $
-// $Id: Triangulation_helpers.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Triangulation_helpers.h $
+// $Id: Triangulation_helpers.h 53d4c9b 2019-10-28T11:29:08+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -21,7 +21,7 @@
 
 #include <CGAL/enum.h>
 #include <CGAL/internal/Has_nested_type_Bare_point.h>
-#include <CGAL/Hash_handles_with_or_without_timestamps.h>
+#include <CGAL/Time_stamper.h>
 
 #include <boost/mpl/if.hpp>
 #include <boost/mpl/identity.hpp>
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Triangulation_sizing_field.h 5.1-1/include/CGAL/Mesh_3/Triangulation_sizing_field.h
--- 5.0.3-1/include/CGAL/Mesh_3/Triangulation_sizing_field.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Triangulation_sizing_field.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Triangulation_sizing_field.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Triangulation_sizing_field.h $
 // $Id: Triangulation_sizing_field.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Uniform_sizing_field.h 5.1-1/include/CGAL/Mesh_3/Uniform_sizing_field.h
--- 5.0.3-1/include/CGAL/Mesh_3/Uniform_sizing_field.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Uniform_sizing_field.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Uniform_sizing_field.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Uniform_sizing_field.h $
 // $Id: Uniform_sizing_field.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/utilities.h 5.1-1/include/CGAL/Mesh_3/utilities.h
--- 5.0.3-1/include/CGAL/Mesh_3/utilities.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/utilities.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/utilities.h $
-// $Id: utilities.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/utilities.h $
+// $Id: utilities.h 4dda7b6 2020-05-27T15:53:05+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -17,7 +17,7 @@
 #ifndef CGAL_MESH_3_UTILITIES_H
 #define CGAL_MESH_3_UTILITIES_H
 
-#include <CGAL/license/Mesh_3.h>
+#include <CGAL/license/Triangulation_3.h>
 
 #include <CGAL/Has_timestamp.h>
 #include <iterator>
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/vertex_perturbation.h 5.1-1/include/CGAL/Mesh_3/vertex_perturbation.h
--- 5.0.3-1/include/CGAL/Mesh_3/vertex_perturbation.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/vertex_perturbation.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/vertex_perturbation.h $
-// $Id: vertex_perturbation.h 85712ba 2020-01-14T15:03:20+01:00 Maxime Gimeno
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/vertex_perturbation.h $
+// $Id: vertex_perturbation.h 315a0b5 2020-01-29T16:27:53+01:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -24,6 +24,7 @@
 
 #include <CGAL/Mesh_3/C3T3_helpers.h>
 #include <CGAL/Mesh_3/Triangulation_helpers.h>
+#include <CGAL/Time_stamper.h>
 
 #ifdef CGAL_MESH_3_PERTURBER_VERBOSE
   #include <CGAL/Timer.h>
diff -pruN 5.0.3-1/include/CGAL/Mesh_3/Worksharing_data_structures.h 5.1-1/include/CGAL/Mesh_3/Worksharing_data_structures.h
--- 5.0.3-1/include/CGAL/Mesh_3/Worksharing_data_structures.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_3/Worksharing_data_structures.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/Worksharing_data_structures.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_3/Worksharing_data_structures.h $
 // $Id: Worksharing_data_structures.h 85712ba 2020-01-14T15:03:20+01:00 Maxime Gimeno
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_cell_base_3.h 5.1-1/include/CGAL/Mesh_cell_base_3.h
--- 5.0.3-1/include/CGAL/Mesh_cell_base_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_cell_base_3.h	2020-09-08 12:15:09.000000000 +0000
@@ -4,8 +4,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_cell_base_3.h $
-// $Id: Mesh_cell_base_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_cell_base_3.h $
+// $Id: Mesh_cell_base_3.h 4dda7b6 2020-05-27T15:53:05+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -15,7 +15,7 @@
 #ifndef CGAL_MESH_CELL_BASE_3_H
 #define CGAL_MESH_CELL_BASE_3_H
 
-#include <CGAL/license/Mesh_3.h>
+#include <CGAL/license/Triangulation_3.h>
 
 
 // #define CGAL_DEPRECATED_HEADER "<CGAL/Mesh_cell_base_3.h>"
diff -pruN 5.0.3-1/include/CGAL/Mesh_cell_criteria_3.h 5.1-1/include/CGAL/Mesh_cell_criteria_3.h
--- 5.0.3-1/include/CGAL/Mesh_cell_criteria_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_cell_criteria_3.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_cell_criteria_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_cell_criteria_3.h $
 // $Id: Mesh_cell_criteria_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_complex_3_in_triangulation_3.h 5.1-1/include/CGAL/Mesh_complex_3_in_triangulation_3.h
--- 5.0.3-1/include/CGAL/Mesh_complex_3_in_triangulation_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_complex_3_in_triangulation_3.h	2020-09-08 12:15:09.000000000 +0000
@@ -4,8 +4,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_complex_3_in_triangulation_3.h $
-// $Id: Mesh_complex_3_in_triangulation_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_complex_3_in_triangulation_3.h $
+// $Id: Mesh_complex_3_in_triangulation_3.h 93d62b9 2020-08-18T14:07:27+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -18,14 +18,15 @@
 #ifndef CGAL_MESH_COMPLEX_3_IN_TRIANGULATION_3_H
 #define CGAL_MESH_COMPLEX_3_IN_TRIANGULATION_3_H
 
-#include <CGAL/license/Mesh_3.h>
+#include <CGAL/license/Triangulation_3.h>
 
+#include <CGAL/Mesh_3/Mesh_complex_3_in_triangulation_3_fwd.h>
 #include <CGAL/disable_warnings.h>
 #include <CGAL/iterator.h>
-#include <CGAL/Hash_handles_with_or_without_timestamps.h>
 #include <CGAL/Mesh_3/utilities.h>
 #include <CGAL/Mesh_3/Mesh_complex_3_in_triangulation_3_base.h>
 #include <CGAL/internal/Mesh_3/Boundary_of_subdomain_of_complex_3_in_triangulation_3_to_off.h>
+#include <CGAL/Time_stamper.h>
 
 #include <boost/bimap/bimap.hpp>
 #include <boost/bimap/multiset_of.hpp>
@@ -38,8 +39,8 @@ namespace CGAL {
 
 
 template <typename Tr,
-          typename CornerIndex = int,
-          typename CurveIndex = int>
+          typename CornerIndex,
+          typename CurveIndex>
 class Mesh_complex_3_in_triangulation_3 :
   public Mesh_3::Mesh_complex_3_in_triangulation_3_base<
     Tr, typename Tr::Concurrency_tag>
@@ -101,10 +102,7 @@ public:
   /**
    * Constructor
    */
-  Mesh_complex_3_in_triangulation_3()
-    : Base()
-    , edges_()
-    , corners_() {}
+  Mesh_complex_3_in_triangulation_3() = default;
 
   /**
    * Copy constructor
@@ -112,12 +110,17 @@ public:
   Mesh_complex_3_in_triangulation_3(const Self& rhs);
 
   /**
-   * Destructor
+   * Move constructor
    */
-  virtual ~Mesh_complex_3_in_triangulation_3() {}
+  Mesh_complex_3_in_triangulation_3(Self&& rhs)
+    : Base(std::move(rhs))
+    , edges_(std::move(rhs.edges_))
+    , corners_(std::move(rhs.corners_))
+    , far_vertices_(std::move(rhs.far_vertices_))
+  {}
 
   /**
-   * Assignement operator
+   * Assignement operator, also serves as move-assignement
    */
   Self& operator=(Self rhs)
   {
@@ -144,6 +147,7 @@ public:
     Base::clear();
     edges_.clear();
     corners_.clear();
+    far_vertices_.clear();
   }
 
   /// Import Base functions
diff -pruN 5.0.3-1/include/CGAL/Mesh_constant_domain_field_3.h 5.1-1/include/CGAL/Mesh_constant_domain_field_3.h
--- 5.0.3-1/include/CGAL/Mesh_constant_domain_field_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_constant_domain_field_3.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_constant_domain_field_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_constant_domain_field_3.h $
 // $Id: Mesh_constant_domain_field_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_criteria_3.h 5.1-1/include/CGAL/Mesh_criteria_3.h
--- 5.0.3-1/include/CGAL/Mesh_criteria_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_criteria_3.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_criteria_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_criteria_3.h $
 // $Id: Mesh_criteria_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_domain_with_polyline_features_3.h 5.1-1/include/CGAL/Mesh_domain_with_polyline_features_3.h
--- 5.0.3-1/include/CGAL/Mesh_domain_with_polyline_features_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_domain_with_polyline_features_3.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_domain_with_polyline_features_3.h $
-// $Id: Mesh_domain_with_polyline_features_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_domain_with_polyline_features_3.h $
+// $Id: Mesh_domain_with_polyline_features_3.h 58b10a3 2020-03-26T18:58:50+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -1165,6 +1165,13 @@ add_features_and_incidences(InputIterato
 
     Curve_index curve_id = insert_edge(polyline.begin(), polyline.end());
     edges_incidences_[curve_id].insert(patches_ids.begin(), patches_ids.end());
+#if CGAL_MESH_3_PROTECTION_DEBUG & 1
+    std::cerr << "Curve #" << curve_id << " is incident to the following patches: {";
+    for(auto id: patches_ids) {
+      std::cerr << " " << id;
+    }
+    std::cerr << "}\n";
+#endif // CGAL_MESH_3_PROTECTION_DEBUG & 1
     *indices_out++ = curve_id;
   }
 
@@ -1410,7 +1417,7 @@ compute_corners_incidences()
                      std::inserter(incidences,
                                    incidences.begin()));
     }
-#ifdef CGAL_MESH_3_PROTECTION_DEBUG
+#if CGAL_MESH_3_PROTECTION_DEBUG & 1
     display_corner_incidences(std::cerr, cit->first, id);
 #endif // CGAL_MESH_3_PROTECTION_DEBUG
 
diff -pruN 5.0.3-1/include/CGAL/Mesh_edge_criteria_3.h 5.1-1/include/CGAL/Mesh_edge_criteria_3.h
--- 5.0.3-1/include/CGAL/Mesh_edge_criteria_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_edge_criteria_3.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_edge_criteria_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_edge_criteria_3.h $
 // $Id: Mesh_edge_criteria_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesher_level_default_implementations.h 5.1-1/include/CGAL/Mesher_level_default_implementations.h
--- 5.0.3-1/include/CGAL/Mesher_level_default_implementations.h	2020-08-01 19:02:32.000000000 +0000
+++ 5.1-1/include/CGAL/Mesher_level_default_implementations.h	2020-09-08 12:15:10.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesher_level/include/CGAL/Mesher_level_default_implementations.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesher_level/include/CGAL/Mesher_level_default_implementations.h $
 // $Id: Mesher_level_default_implementations.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesher_level.h 5.1-1/include/CGAL/Mesher_level.h
--- 5.0.3-1/include/CGAL/Mesher_level.h	2020-08-01 19:02:32.000000000 +0000
+++ 5.1-1/include/CGAL/Mesher_level.h	2020-09-08 12:15:10.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesher_level/include/CGAL/Mesher_level.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesher_level/include/CGAL/Mesher_level.h $
 // $Id: Mesher_level.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesher_level_visitors.h 5.1-1/include/CGAL/Mesher_level_visitors.h
--- 5.0.3-1/include/CGAL/Mesher_level_visitors.h	2020-08-01 19:02:32.000000000 +0000
+++ 5.1-1/include/CGAL/Mesher_level_visitors.h	2020-09-08 12:15:10.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesher_level/include/CGAL/Mesher_level_visitors.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesher_level/include/CGAL/Mesher_level_visitors.h $
 // $Id: Mesher_level_visitors.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_error_code.h 5.1-1/include/CGAL/Mesh_error_code.h
--- 5.0.3-1/include/CGAL/Mesh_error_code.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_error_code.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_error_code.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_error_code.h $
 // $Id: Mesh_error_code.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Meshes/Double_map_container.h 5.1-1/include/CGAL/Meshes/Double_map_container.h
--- 5.0.3-1/include/CGAL/Meshes/Double_map_container.h	2020-08-01 19:02:32.000000000 +0000
+++ 5.1-1/include/CGAL/Meshes/Double_map_container.h	2020-09-08 12:15:10.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesher_level/include/CGAL/Meshes/Double_map_container.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesher_level/include/CGAL/Meshes/Double_map_container.h $
 // $Id: Double_map_container.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Meshes/Filtered_deque_container.h 5.1-1/include/CGAL/Meshes/Filtered_deque_container.h
--- 5.0.3-1/include/CGAL/Meshes/Filtered_deque_container.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Meshes/Filtered_deque_container.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Meshes/Filtered_deque_container.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Meshes/Filtered_deque_container.h $
 // $Id: Filtered_deque_container.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Meshes/Filtered_multimap_container.h 5.1-1/include/CGAL/Meshes/Filtered_multimap_container.h
--- 5.0.3-1/include/CGAL/Meshes/Filtered_multimap_container.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Meshes/Filtered_multimap_container.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Meshes/Filtered_multimap_container.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Meshes/Filtered_multimap_container.h $
 // $Id: Filtered_multimap_container.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Meshes/Filtered_queue_container.h 5.1-1/include/CGAL/Meshes/Filtered_queue_container.h
--- 5.0.3-1/include/CGAL/Meshes/Filtered_queue_container.h	2020-08-01 19:02:32.000000000 +0000
+++ 5.1-1/include/CGAL/Meshes/Filtered_queue_container.h	2020-09-08 12:15:10.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesher_level/include/CGAL/Meshes/Filtered_queue_container.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesher_level/include/CGAL/Meshes/Filtered_queue_container.h $
 // $Id: Filtered_queue_container.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Meshes/Simple_map_container.h 5.1-1/include/CGAL/Meshes/Simple_map_container.h
--- 5.0.3-1/include/CGAL/Meshes/Simple_map_container.h	2020-08-01 19:02:32.000000000 +0000
+++ 5.1-1/include/CGAL/Meshes/Simple_map_container.h	2020-09-08 12:15:10.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesher_level/include/CGAL/Meshes/Simple_map_container.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesher_level/include/CGAL/Meshes/Simple_map_container.h $
 // $Id: Simple_map_container.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Meshes/Simple_queue_container.h 5.1-1/include/CGAL/Meshes/Simple_queue_container.h
--- 5.0.3-1/include/CGAL/Meshes/Simple_queue_container.h	2020-08-01 19:02:32.000000000 +0000
+++ 5.1-1/include/CGAL/Meshes/Simple_queue_container.h	2020-09-08 12:15:10.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesher_level/include/CGAL/Meshes/Simple_queue_container.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesher_level/include/CGAL/Meshes/Simple_queue_container.h $
 // $Id: Simple_queue_container.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Meshes/Simple_set_container.h 5.1-1/include/CGAL/Meshes/Simple_set_container.h
--- 5.0.3-1/include/CGAL/Meshes/Simple_set_container.h	2020-08-01 19:02:32.000000000 +0000
+++ 5.1-1/include/CGAL/Meshes/Simple_set_container.h	2020-09-08 12:15:10.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesher_level/include/CGAL/Meshes/Simple_set_container.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesher_level/include/CGAL/Meshes/Simple_set_container.h $
 // $Id: Simple_set_container.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Meshes/Triangulation_mesher_level_traits_2.h 5.1-1/include/CGAL/Meshes/Triangulation_mesher_level_traits_2.h
--- 5.0.3-1/include/CGAL/Meshes/Triangulation_mesher_level_traits_2.h	2020-08-01 19:02:32.000000000 +0000
+++ 5.1-1/include/CGAL/Meshes/Triangulation_mesher_level_traits_2.h	2020-09-08 12:15:10.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesher_level/include/CGAL/Meshes/Triangulation_mesher_level_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesher_level/include/CGAL/Meshes/Triangulation_mesher_level_traits_2.h $
 // $Id: Triangulation_mesher_level_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Meshes/Triangulation_mesher_level_traits_3.h 5.1-1/include/CGAL/Meshes/Triangulation_mesher_level_traits_3.h
--- 5.0.3-1/include/CGAL/Meshes/Triangulation_mesher_level_traits_3.h	2020-08-01 19:02:32.000000000 +0000
+++ 5.1-1/include/CGAL/Meshes/Triangulation_mesher_level_traits_3.h	2020-09-08 12:15:10.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesher_level/include/CGAL/Meshes/Triangulation_mesher_level_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesher_level/include/CGAL/Meshes/Triangulation_mesher_level_traits_3.h $
 // $Id: Triangulation_mesher_level_traits_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_facet_criteria_3.h 5.1-1/include/CGAL/Mesh_facet_criteria_3.h
--- 5.0.3-1/include/CGAL/Mesh_facet_criteria_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_facet_criteria_3.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_facet_criteria_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_facet_criteria_3.h $
 // $Id: Mesh_facet_criteria_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_facet_topology.h 5.1-1/include/CGAL/Mesh_facet_topology.h
--- 5.0.3-1/include/CGAL/Mesh_facet_topology.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_facet_topology.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_facet_topology.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_facet_topology.h $
 // $Id: Mesh_facet_topology.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_optimization_return_code.h 5.1-1/include/CGAL/Mesh_optimization_return_code.h
--- 5.0.3-1/include/CGAL/Mesh_optimization_return_code.h	2020-08-01 19:02:32.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_optimization_return_code.h	2020-09-08 12:15:10.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesher_level/include/CGAL/Mesh_optimization_return_code.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesher_level/include/CGAL/Mesh_optimization_return_code.h $
 // $Id: Mesh_optimization_return_code.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_polyhedron_3.h 5.1-1/include/CGAL/Mesh_polyhedron_3.h
--- 5.0.3-1/include/CGAL/Mesh_polyhedron_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_polyhedron_3.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_polyhedron_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_polyhedron_3.h $
 // $Id: Mesh_polyhedron_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/mesh_segmentation.h 5.1-1/include/CGAL/mesh_segmentation.h
--- 5.0.3-1/include/CGAL/mesh_segmentation.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/mesh_segmentation.h	2020-09-08 12:15:44.000000000 +0000
@@ -4,8 +4,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_segmentation/include/CGAL/mesh_segmentation.h $
-// $Id: mesh_segmentation.h c43d6bd 2020-02-25T14:47:03+01:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_segmentation/include/CGAL/mesh_segmentation.h $
+// $Id: mesh_segmentation.h 358b2d2 2020-03-26T11:10:23+01:00 Simon Giraudot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Ilker O. Yaz
@@ -198,8 +198,7 @@ segmentation_from_sdf_values( const Tria
                               PointPropertyMap ppmap=PointPropertyMap(),
                               GeomTraits traits=GeomTraits())
 {
-  typedef typename boost::property_map<TriangleMesh, boost::vertex_point_t>::type VPMap;
-  internal::Surface_mesh_segmentation<TriangleMesh, GeomTraits, VPMap> algorithm(triangle_mesh, traits, ppmap);
+  internal::Surface_mesh_segmentation<TriangleMesh, GeomTraits, PointPropertyMap> algorithm(triangle_mesh, traits, ppmap);
   return algorithm.partition(number_of_clusters, smoothing_lambda, sdf_values_map,
                              segment_ids, !output_cluster_ids);
 }
diff -pruN 5.0.3-1/include/CGAL/mesh_skin_surface_3.h 5.1-1/include/CGAL/mesh_skin_surface_3.h
--- 5.0.3-1/include/CGAL/mesh_skin_surface_3.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/mesh_skin_surface_3.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Skin_surface_3/include/CGAL/mesh_skin_surface_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Skin_surface_3/include/CGAL/mesh_skin_surface_3.h $
 // $Id: mesh_skin_surface_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_triangulation_3.h 5.1-1/include/CGAL/Mesh_triangulation_3.h
--- 5.0.3-1/include/CGAL/Mesh_triangulation_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_triangulation_3.h	2020-09-08 12:15:09.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_triangulation_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_triangulation_3.h $
 // $Id: Mesh_triangulation_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/mesh_union_of_balls_3.h 5.1-1/include/CGAL/mesh_union_of_balls_3.h
--- 5.0.3-1/include/CGAL/mesh_union_of_balls_3.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/mesh_union_of_balls_3.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Skin_surface_3/include/CGAL/mesh_union_of_balls_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Skin_surface_3/include/CGAL/mesh_union_of_balls_3.h $
 // $Id: mesh_union_of_balls_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mesh_vertex_base_3.h 5.1-1/include/CGAL/Mesh_vertex_base_3.h
--- 5.0.3-1/include/CGAL/Mesh_vertex_base_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Mesh_vertex_base_3.h	2020-09-08 12:15:09.000000000 +0000
@@ -4,8 +4,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_vertex_base_3.h $
-// $Id: Mesh_vertex_base_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Mesh_vertex_base_3.h $
+// $Id: Mesh_vertex_base_3.h 4dda7b6 2020-05-27T15:53:05+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Stéphane Tayeb, Andreas Fabri
@@ -20,7 +20,7 @@
 #ifndef CGAL_COMPACT_MESH_VERTEX_BASE_3_H
 #define CGAL_COMPACT_MESH_VERTEX_BASE_3_H
 
-#include <CGAL/license/Mesh_3.h>
+#include <CGAL/license/Triangulation_3.h>
 
 
 #include <CGAL/Regular_triangulation_vertex_base_3.h>
diff -pruN 5.0.3-1/include/CGAL/Min_annulus_d.h 5.1-1/include/CGAL/Min_annulus_d.h
--- 5.0.3-1/include/CGAL/Min_annulus_d.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Min_annulus_d.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Min_annulus_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Min_annulus_d.h $
 // $Id: Min_annulus_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Min_circle_2/Min_circle_2_adapterC2.h 5.1-1/include/CGAL/Min_circle_2/Min_circle_2_adapterC2.h
--- 5.0.3-1/include/CGAL/Min_circle_2/Min_circle_2_adapterC2.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Min_circle_2/Min_circle_2_adapterC2.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Min_circle_2/Min_circle_2_adapterC2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Min_circle_2/Min_circle_2_adapterC2.h $
 // $Id: Min_circle_2_adapterC2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Min_circle_2/Min_circle_2_adapterH2.h 5.1-1/include/CGAL/Min_circle_2/Min_circle_2_adapterH2.h
--- 5.0.3-1/include/CGAL/Min_circle_2/Min_circle_2_adapterH2.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Min_circle_2/Min_circle_2_adapterH2.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Min_circle_2/Min_circle_2_adapterH2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Min_circle_2/Min_circle_2_adapterH2.h $
 // $Id: Min_circle_2_adapterH2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Min_circle_2/Min_circle_2_impl.h 5.1-1/include/CGAL/Min_circle_2/Min_circle_2_impl.h
--- 5.0.3-1/include/CGAL/Min_circle_2/Min_circle_2_impl.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Min_circle_2/Min_circle_2_impl.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Min_circle_2/Min_circle_2_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Min_circle_2/Min_circle_2_impl.h $
 // $Id: Min_circle_2_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Min_circle_2/Optimisation_circle_2.h 5.1-1/include/CGAL/Min_circle_2/Optimisation_circle_2.h
--- 5.0.3-1/include/CGAL/Min_circle_2/Optimisation_circle_2.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Min_circle_2/Optimisation_circle_2.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Min_circle_2/Optimisation_circle_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Min_circle_2/Optimisation_circle_2.h $
 // $Id: Optimisation_circle_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Min_circle_2/Optimisation_circle_2_impl.h 5.1-1/include/CGAL/Min_circle_2/Optimisation_circle_2_impl.h
--- 5.0.3-1/include/CGAL/Min_circle_2/Optimisation_circle_2_impl.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Min_circle_2/Optimisation_circle_2_impl.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Min_circle_2/Optimisation_circle_2_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Min_circle_2/Optimisation_circle_2_impl.h $
 // $Id: Optimisation_circle_2_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Min_circle_2.h 5.1-1/include/CGAL/Min_circle_2.h
--- 5.0.3-1/include/CGAL/Min_circle_2.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Min_circle_2.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Min_circle_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Min_circle_2.h $
 // $Id: Min_circle_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Min_circle_2_traits_2.h 5.1-1/include/CGAL/Min_circle_2_traits_2.h
--- 5.0.3-1/include/CGAL/Min_circle_2_traits_2.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Min_circle_2_traits_2.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Min_circle_2_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Min_circle_2_traits_2.h $
 // $Id: Min_circle_2_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Min_ellipse_2/Min_ellipse_2_adapterC2.h 5.1-1/include/CGAL/Min_ellipse_2/Min_ellipse_2_adapterC2.h
--- 5.0.3-1/include/CGAL/Min_ellipse_2/Min_ellipse_2_adapterC2.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Min_ellipse_2/Min_ellipse_2_adapterC2.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Min_ellipse_2/Min_ellipse_2_adapterC2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Min_ellipse_2/Min_ellipse_2_adapterC2.h $
 // $Id: Min_ellipse_2_adapterC2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Min_ellipse_2/Min_ellipse_2_adapterH2.h 5.1-1/include/CGAL/Min_ellipse_2/Min_ellipse_2_adapterH2.h
--- 5.0.3-1/include/CGAL/Min_ellipse_2/Min_ellipse_2_adapterH2.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Min_ellipse_2/Min_ellipse_2_adapterH2.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Min_ellipse_2/Min_ellipse_2_adapterH2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Min_ellipse_2/Min_ellipse_2_adapterH2.h $
 // $Id: Min_ellipse_2_adapterH2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Min_ellipse_2/Min_ellipse_2_impl.h 5.1-1/include/CGAL/Min_ellipse_2/Min_ellipse_2_impl.h
--- 5.0.3-1/include/CGAL/Min_ellipse_2/Min_ellipse_2_impl.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Min_ellipse_2/Min_ellipse_2_impl.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Min_ellipse_2/Min_ellipse_2_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Min_ellipse_2/Min_ellipse_2_impl.h $
 // $Id: Min_ellipse_2_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Min_ellipse_2/Optimisation_ellipse_2.h 5.1-1/include/CGAL/Min_ellipse_2/Optimisation_ellipse_2.h
--- 5.0.3-1/include/CGAL/Min_ellipse_2/Optimisation_ellipse_2.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Min_ellipse_2/Optimisation_ellipse_2.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Min_ellipse_2/Optimisation_ellipse_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Min_ellipse_2/Optimisation_ellipse_2.h $
 // $Id: Optimisation_ellipse_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Min_ellipse_2/Optimisation_ellipse_2_impl.h 5.1-1/include/CGAL/Min_ellipse_2/Optimisation_ellipse_2_impl.h
--- 5.0.3-1/include/CGAL/Min_ellipse_2/Optimisation_ellipse_2_impl.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Min_ellipse_2/Optimisation_ellipse_2_impl.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Min_ellipse_2/Optimisation_ellipse_2_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Min_ellipse_2/Optimisation_ellipse_2_impl.h $
 // $Id: Optimisation_ellipse_2_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Min_ellipse_2.h 5.1-1/include/CGAL/Min_ellipse_2.h
--- 5.0.3-1/include/CGAL/Min_ellipse_2.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Min_ellipse_2.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Min_ellipse_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Min_ellipse_2.h $
 // $Id: Min_ellipse_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Min_ellipse_2_traits_2.h 5.1-1/include/CGAL/Min_ellipse_2_traits_2.h
--- 5.0.3-1/include/CGAL/Min_ellipse_2_traits_2.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Min_ellipse_2_traits_2.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Min_ellipse_2_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Min_ellipse_2_traits_2.h $
 // $Id: Min_ellipse_2_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/minimum_enclosing_quadrilateral_2.h 5.1-1/include/CGAL/minimum_enclosing_quadrilateral_2.h
--- 5.0.3-1/include/CGAL/minimum_enclosing_quadrilateral_2.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/minimum_enclosing_quadrilateral_2.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/minimum_enclosing_quadrilateral_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/minimum_enclosing_quadrilateral_2.h $
 // $Id: minimum_enclosing_quadrilateral_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Minimum_enclosing_quadrilateral_traits_2.h 5.1-1/include/CGAL/Minimum_enclosing_quadrilateral_traits_2.h
--- 5.0.3-1/include/CGAL/Minimum_enclosing_quadrilateral_traits_2.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Minimum_enclosing_quadrilateral_traits_2.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Minimum_enclosing_quadrilateral_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Minimum_enclosing_quadrilateral_traits_2.h $
 // $Id: Minimum_enclosing_quadrilateral_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Minkowski_sum_2/AABB_collision_detector_2.h 5.1-1/include/CGAL/Minkowski_sum_2/AABB_collision_detector_2.h
--- 5.0.3-1/include/CGAL/Minkowski_sum_2/AABB_collision_detector_2.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/Minkowski_sum_2/AABB_collision_detector_2.h	2020-09-08 12:15:11.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/AABB_collision_detector_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/AABB_collision_detector_2.h $
 // $Id: AABB_collision_detector_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Minkowski_sum_2/AABB_node_with_join.h 5.1-1/include/CGAL/Minkowski_sum_2/AABB_node_with_join.h
--- 5.0.3-1/include/CGAL/Minkowski_sum_2/AABB_node_with_join.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/Minkowski_sum_2/AABB_node_with_join.h	2020-09-08 12:15:11.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/AABB_node_with_join.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/AABB_node_with_join.h $
 // $Id: AABB_node_with_join.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Minkowski_sum_2/AABB_segment_2_primitive.h 5.1-1/include/CGAL/Minkowski_sum_2/AABB_segment_2_primitive.h
--- 5.0.3-1/include/CGAL/Minkowski_sum_2/AABB_segment_2_primitive.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/Minkowski_sum_2/AABB_segment_2_primitive.h	2020-09-08 12:15:11.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/AABB_segment_2_primitive.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/AABB_segment_2_primitive.h $
 // $Id: AABB_segment_2_primitive.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Minkowski_sum_2/AABB_traits_2.h 5.1-1/include/CGAL/Minkowski_sum_2/AABB_traits_2.h
--- 5.0.3-1/include/CGAL/Minkowski_sum_2/AABB_traits_2.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/Minkowski_sum_2/AABB_traits_2.h	2020-09-08 12:15:11.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/AABB_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/AABB_traits_2.h $
 // $Id: AABB_traits_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Minkowski_sum_2/AABB_traversal_traits_with_join.h 5.1-1/include/CGAL/Minkowski_sum_2/AABB_traversal_traits_with_join.h
--- 5.0.3-1/include/CGAL/Minkowski_sum_2/AABB_traversal_traits_with_join.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/Minkowski_sum_2/AABB_traversal_traits_with_join.h	2020-09-08 12:15:11.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/AABB_traversal_traits_with_join.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/AABB_traversal_traits_with_join.h $
 // $Id: AABB_traversal_traits_with_join.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Minkowski_sum_2/AABB_tree_with_join.h 5.1-1/include/CGAL/Minkowski_sum_2/AABB_tree_with_join.h
--- 5.0.3-1/include/CGAL/Minkowski_sum_2/AABB_tree_with_join.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/Minkowski_sum_2/AABB_tree_with_join.h	2020-09-08 12:15:11.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/AABB_tree_with_join.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/AABB_tree_with_join.h $
 // $Id: AABB_tree_with_join.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Minkowski_sum_2/Approx_offset_base_2.h 5.1-1/include/CGAL/Minkowski_sum_2/Approx_offset_base_2.h
--- 5.0.3-1/include/CGAL/Minkowski_sum_2/Approx_offset_base_2.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/Minkowski_sum_2/Approx_offset_base_2.h	2020-09-08 12:15:11.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/Approx_offset_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/Approx_offset_base_2.h $
 // $Id: Approx_offset_base_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Minkowski_sum_2/Arr_labeled_traits_2.h 5.1-1/include/CGAL/Minkowski_sum_2/Arr_labeled_traits_2.h
--- 5.0.3-1/include/CGAL/Minkowski_sum_2/Arr_labeled_traits_2.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/Minkowski_sum_2/Arr_labeled_traits_2.h	2020-09-08 12:15:11.000000000 +0000
@@ -3,11 +3,11 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/Arr_labeled_traits_2.h $
-// $Id: Arr_labeled_traits_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/Arr_labeled_traits_2.h $
+// $Id: Arr_labeled_traits_2.h 3863b96 2020-04-06T02:01:25+03:00 Efi Fogel
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
-// Author(s)     : Ron Wein   <wein_r@yahoo.com>
+// Author(s): Ron Wein   <wein_r@yahoo.com>
 
 #ifndef CGAL_ARR_LABELED_TRAITS_2_H
 #define CGAL_ARR_LABELED_TRAITS_2_H
@@ -25,95 +25,79 @@ namespace CGAL {
  * such that the comparison of two points, as well as the computation of the
  * intersections between two segments can be easily filtered.
  */
-template <class Traits_>
-class Arr_labeled_traits_2 : public Traits_
-{
+template <typename Traits_>
+class Arr_labeled_traits_2 : public Traits_ {
 private:
+  typedef Traits_                                       Base_traits_2;
+  typedef Arr_labeled_traits_2<Base_traits_2>           Traits;
 
-  typedef Traits_                                      Base_traits_2;
-  typedef typename Base_traits_2::Point_2              Base_point_2;
-  typedef typename Base_traits_2::X_monotone_curve_2   Base_x_monotone_curve_2;
+  typedef typename Base_traits_2::Point_2               Base_point_2;
+  typedef typename Base_traits_2::X_monotone_curve_2    Base_x_monotone_curve_2;
 
 public:
+  typedef typename Base_traits_2::Multiplicity          Multiplicity;
 
   /*! \class
    * A point extended by a label.
    */
-  class Point_2 : public Base_point_2
-  {
+  class Point_2 : public Base_point_2 {
   private:
-
-    Point_label         _label;
+    Point_label m_label;
 
   public:
-
     /*! Default constructor. */
-    Point_2 ()
-    {}
+    Point_2() {}
 
     /*! Constructor from a base point. */
-    Point_2 (const Base_point_2& p) :
-      Base_point_2 (p),
-      _label()
+    Point_2(const Base_point_2& p) :
+      Base_point_2(p),
+      m_label()
     {}
 
     /*! Constructor from a point an a label. */
-    Point_2 (const Base_point_2& p, const Point_label& label) :
-      Base_point_2 (p),
-      _label (label)
+    Point_2(const Base_point_2& p, const Point_label& label) :
+      Base_point_2(p),
+      m_label(label)
     {}
 
     /*! Get the label. */
-    const Point_label& label () const
-    {
-      return (_label);
-    }
+    const Point_label& label() const { return (m_label); }
   };
 
   /*! \class
    * An x-monotone curve extended by a label.
    */
-  class X_monotone_curve_2 : public Base_x_monotone_curve_2
-  {
+  class X_monotone_curve_2 : public Base_x_monotone_curve_2 {
   private:
-
-    X_curve_label         _label;
+    X_curve_label m_label;
 
   public:
-
     /*! Default constructor. */
-    X_monotone_curve_2 ()
-    {}
+    X_monotone_curve_2() {}
 
     /*! Constructor from a base x-monotone curve. */
-    X_monotone_curve_2 (const Base_x_monotone_curve_2& p) :
-      Base_x_monotone_curve_2 (p),
-      _label()
+    X_monotone_curve_2(const Base_x_monotone_curve_2& p) :
+      Base_x_monotone_curve_2(p),
+      m_label()
     {}
 
     /*! Constructor from an x-monotone curve an a label. */
-    X_monotone_curve_2 (const Base_x_monotone_curve_2& p,
-                        const X_curve_label& label) :
-      Base_x_monotone_curve_2 (p),
-      _label (label)
+    X_monotone_curve_2(const Base_x_monotone_curve_2& p,
+                       const X_curve_label& label) :
+      Base_x_monotone_curve_2(p),
+      m_label(label)
     {}
 
     /*! Get the label (const version). */
-    const X_curve_label& label () const
-    {
-      return (_label);
-    }
+    const X_curve_label& label() const { return m_label; }
 
     /*! Get the label (non-const version). */
-    X_curve_label& label ()
-    {
-      return (_label);
-    }
+    X_curve_label& label() { return m_label; }
 
     /*! Set the label. */
-    void set_label (const X_curve_label& label)
+    void set_label(const X_curve_label& label)
     {
-      _label = label;
+      m_label = label;
       return;
     }
   };
@@ -122,8 +106,7 @@ public:
   typedef Tag_false                                      Has_merge_category;
 
   /*! Default constructor. */
-  Arr_labeled_traits_2 ()
-  {}
+  Arr_labeled_traits_2() {}
 
   // Inherited functors:
   typedef typename Base_traits_2::Is_vertical_2         Is_vertical_2;
@@ -134,280 +117,232 @@ public:
 
   /// \name Overriden functors.
   //@{
-  class Compare_x_2
-  {
+  class Compare_x_2 {
   private:
-
-    const Base_traits_2 * base;
-
-  public:
+    const Base_traits_2& m_base_traits;
 
     /*! Constructor. */
-    Compare_x_2 (const Base_traits_2 * _base) :
-      base (_base)
-    {}
+    Compare_x_2(const Base_traits_2& base_tr) : m_base_traits(base_tr) {}
 
-    /*!
-     * Compare the x-coordinates of two points.
+    friend Traits;
+
+  public:
+    /*! Compare the x-coordinates of two points.
      */
-    Comparison_result operator() (const Point_2& p1, const Point_2& p2) const
+    Comparison_result operator()(const Point_2& p1, const Point_2& p2) const
     {
       // If two points have the same label, they are equal.
-      if (p1.label() == p2.label())
-        return (EQUAL);
+      if (p1.label() == p2.label()) return (EQUAL);
 
-      return (base->compare_x_2_object()(p1, p2));
+      return (m_base_traits.compare_x_2_object()(p1, p2));
     }
   };
 
   /*! Get a Compare_x_2 functor object. */
-  Compare_x_2 compare_x_2_object () const
-  {
-    return (Compare_x_2 (this));
-  }
+  Compare_x_2 compare_x_2_object() const { return (Compare_x_2(*this)); }
 
-
-  class Compare_xy_2
-  {
+  class Compare_xy_2 {
   private:
-
-    const Base_traits_2 * base;
-
-  public:
+    const Base_traits_2& m_base_traits;
 
     /*! Constructor. */
-    Compare_xy_2 (const Base_traits_2 *_base) :
-      base (_base)
-    {}
+    Compare_xy_2(const Base_traits_2& base_tr) : m_base_traits(base_tr) {}
+
+    friend Traits;
 
-    /*!
-     * Compare two points lexigoraphically: by x, then by y.
+  public:
+    /*! Compare two points lexigoraphically: by x, then by y.
      */
-    Comparison_result operator() (const Point_2& p1, const Point_2& p2) const
+    Comparison_result operator()(const Point_2& p1, const Point_2& p2) const
     {
       // If two points have the same label, they are equal.
-      if (p1.label() == p2.label())
-        return (EQUAL);
+      if (p1.label() == p2.label()) return (EQUAL);
 
-      return (base->compare_xy_2_object()(p1, p2));
+      return (m_base_traits.compare_xy_2_object()(p1, p2));
     }
   };
 
-  /*! Get a Compare_xy_2 functor object. */
-  Compare_xy_2 compare_xy_2_object () const
-  {
-    return (Compare_xy_2 (this));
-  }
-
+  /*! Obtain a Compare_xy_2 functor object. */
+  Compare_xy_2 compare_xy_2_object() const { return Compare_xy_2(*this); }
 
-  class Construct_min_vertex_2
-  {
+  class Construct_min_vertex_2 {
   private:
-
-    const Base_traits_2 * base;
-
-  public:
+    const Base_traits_2& m_base_traits;
 
     /*! Constructor. */
-    Construct_min_vertex_2 (const Base_traits_2 *_base) :
-      base (_base)
+    Construct_min_vertex_2(const Base_traits_2& base_tr) :
+      m_base_traits(base_tr)
     {}
 
-    /*!
-     * Get the left endpoint of the x-monotone curve.
+    friend Traits;
+
+  public:
+    /*! Obtain the left endpoint of the x-monotone curve.
      */
-    Point_2 operator() (const X_monotone_curve_2& cv) const
+    Point_2 operator()(const X_monotone_curve_2& cv) const
     {
-      const Base_point_2&  pt = base->construct_min_vertex_2_object() (cv);
+      auto base_ctr_min_vertex = m_base_traits.construct_min_vertex_2_object();
+      const Base_point_2& pt = base_ctr_min_vertex(cv);
 
-      if (cv.label().right_count() == 1 && cv.label().left_count() == 0)
-      {
+      if ((cv.label().right_count() == 1) && (cv.label().left_count() == 0)) {
         // A curve directed from left to right:
-        Point_label   label (cv.label().component(), cv.label().index());
+        Point_label label(cv.label().component(), cv.label().index());
 
         return (Point_2 (pt, label));
       }
-      else if (cv.label().right_count() == 0 && cv.label().left_count() == 1)
+      else if ((cv.label().right_count() == 0) &&
+               (cv.label().left_count() == 1))
       {
         // A curve directed from right to left:
-        Point_label   label (cv.label().component(),
-                             cv.label().is_last() ? 0 : cv.label().index()+1);
+        Point_label label(cv.label().component(),
+                           cv.label().is_last() ? 0 : cv.label().index()+1);
 
-        return (Point_2 (pt, label));
+        return (Point_2(pt, label));
       }
 
       // Assign an invalid label to the point.
-      return (Point_2 (pt));
+      return Point_2(pt);
     }
   };
 
   /*! Get a Construct_min_vertex_2 functor object. */
   Construct_min_vertex_2 construct_min_vertex_2_object () const
-  {
-    return (Construct_min_vertex_2 (this));
-  }
+  { return Construct_min_vertex_2(*this); }
 
-
-  class Construct_max_vertex_2
-  {
+  class Construct_max_vertex_2 {
   private:
-
-    const Base_traits_2 * base;
-
-  public:
+    const Base_traits_2& m_base_traits;
 
     /*! Constructor. */
-    Construct_max_vertex_2 (const Base_traits_2 *_base) :
-      base (_base)
+    Construct_max_vertex_2(const Base_traits_2& base_tr) :
+      m_base_traits(base_tr)
     {}
 
-    /*!
-     * Get the right endpoint of the x-monotone curve.
+    friend Traits;
+
+  public:
+    /*! Obtain the right endpoint of the x-monotone curve.
      */
-    Point_2 operator() (const X_monotone_curve_2& cv) const
+    Point_2 operator()(const X_monotone_curve_2& cv) const
     {
-      const Base_point_2&  pt = base->construct_max_vertex_2_object() (cv);
+      auto base_ctr_max_vertex = m_base_traits.construct_max_vertex_2_object();
+      const Base_point_2& pt = base_ctr_max_vertex(cv);
 
-      if (cv.label().right_count() == 1 && cv.label().left_count() == 0)
-      {
+      if ((cv.label().right_count() == 1) && (cv.label().left_count() == 0)) {
         // A curve directed from left to right:
-        Point_label   label (cv.label().component(),
-                             cv.label().is_last() ? 0 : cv.label().index()+1);
+        Point_label label(cv.label().component(),
+                          cv.label().is_last() ? 0 : cv.label().index()+1);
 
-        return (Point_2 (pt, label));
+        return Point_2(pt, label);
       }
-      else if (cv.label().right_count() == 0 && cv.label().left_count() == 1)
+      else if ((cv.label().right_count() == 0) &&
+               (cv.label().left_count() == 1))
       {
         // A curve directed from right to left:
-        Point_label   label (cv.label().component(), cv.label().index());
+        Point_label label(cv.label().component(), cv.label().index());
 
-        return (Point_2 (pt, label));
+        return Point_2(pt, label);
       }
 
       // Assign an invalid label to the point.
-      return (Point_2 (pt));
+      return Point_2(pt);
     }
   };
 
   /*! Get a Construct_max_vertex_2 functor object. */
   Construct_max_vertex_2 construct_max_vertex_2_object () const
-  {
-    return (Construct_max_vertex_2 (this));
-  }
-
+  { return Construct_max_vertex_2(*this); }
 
-  class Split_2
-  {
+  class Split_2 {
   private:
-
-    const Base_traits_2 * base;
-
-  public:
+    const Base_traits_2& m_base_traits;
 
     /*! Constructor. */
-    Split_2 (const Base_traits_2 * _base) :
-      base (_base)
-    {}
+    Split_2(const Base_traits_2& base_tr) : m_base_traits(base_tr) {}
+
+    friend Traits;
 
-    /*!
-     * Split a given x-monotone curve at a given point into two sub-curves.
+  public:
+    /*! Split a given x-monotone curve at a given point into two sub-curves.
      */
-    void operator() (const X_monotone_curve_2& cv, const Point_2& p,
-                     X_monotone_curve_2& c1, X_monotone_curve_2& c2) const
+    void operator()(const X_monotone_curve_2& cv, const Point_2& p,
+                    X_monotone_curve_2& c1, X_monotone_curve_2& c2) const
     {
       // Split the base curve into two.
-      base->split_2_object() (cv, p, c1, c2);
+      m_base_traits.split_2_object()(cv, p, c1, c2);
 
       // Duplicate the label to both subcurves.
-      c1.set_label (cv.label());
-      c2.set_label (cv.label());
-
-      return;
+      c1.set_label(cv.label());
+      c2.set_label(cv.label());
     }
   };
 
   /*! Get a Split_2 functor object. */
-  Split_2 split_2_object () const
-  {
-    return (Split_2 (this));
-  }
-
+  Split_2 split_2_object() const { return Split_2(*this); }
 
-  class Intersect_2
-  {
+  class Intersect_2 {
   private:
-
-    const Base_traits_2 * base;
+    const Base_traits_2& m_base_traits;
 
   public:
-
     /*! Constructor. */
-    Intersect_2 (const Base_traits_2 * _base) :
-      base (_base)
-    {}
+    Intersect_2(const Base_traits_2& traits) : m_base_traits(traits) {}
 
-    /*!
-     * Find the intersections of the two given curves and insert them to the
+    /*! Find the intersections of the two given curves and insert them to the
      * given output iterator.
      */
-    template<class OutputIterator>
-    OutputIterator operator() (const X_monotone_curve_2& cv1,
-                               const X_monotone_curve_2& cv2,
-                               OutputIterator oi) const
+    template <typename OutputIterator>
+    OutputIterator operator()(const X_monotone_curve_2& cv1,
+                              const X_monotone_curve_2& cv2,
+                              OutputIterator oi) const
     {
+      typedef std::pair<Base_point_2, Multiplicity>     Intersection_base_point;
+      typedef boost::variant<Intersection_base_point, Base_x_monotone_curve_2>
+                                                        Intersection_base_result;
+      typedef std::pair<Point_2, Multiplicity>     Intersection_point;
+      typedef boost::variant<Intersection_point, X_monotone_curve_2>
+                                                        Intersection_result;
+
       // In case the curves are adjacent in their curve sequence, we do
       // not have to compute their intersection (we already know that they
       // have just one common endpoint).
-      if (cv1.label().is_adjacent (cv2.label()))
-        return (oi);
+      if (cv1.label().is_adjacent(cv2.label())) return oi;
 
       // Compute the intersection.
-      std::list<CGAL::Object>            base_objs;
+      std::list<Intersection_base_result> xections;
+      m_base_traits.intersect_2_object()(cv1, cv2, std::back_inserter(xections));
 
-      base->intersect_2_object() (cv1, cv2, std::back_inserter (base_objs));
-
-      if (base_objs.empty())
-        return (oi);
+      if (xections.empty()) return oi;
 
       // Attach labels to the intersection objects.
-      std::list<CGAL::Object>::iterator             obj_it;
-      const std::pair<Base_point_2, unsigned int>  *base_pt;
-      const Base_x_monotone_curve_2                *base_xcv;
+      for (const auto& xection : xections) {
+        const Intersection_base_point* base_pt =
+          boost::get<Intersection_base_point>(&xection);
 
-      for (obj_it = base_objs.begin(); obj_it != base_objs.end(); ++obj_it)
-      {
-        base_pt =
-          object_cast<std::pair<Base_point_2, unsigned int> > (&(*obj_it));
-
-        if (base_pt != nullptr)
-        {
+        if (base_pt != nullptr) {
           // Attach an invalid label to an itersection point.
-          *oi = CGAL::make_object
-            (std::make_pair (Point_2 (base_pt->first), base_pt->second));
-          ++oi;
-        }
-        else
-        {
-          base_xcv = object_cast<Base_x_monotone_curve_2> (&(*obj_it));
-          CGAL_assertion (base_xcv != nullptr);
-
-          // Attach a merged label to the overlapping curve.
-          *oi = CGAL::make_object
-            (X_monotone_curve_2 (*base_xcv,
-                                 X_curve_label (cv1.label(), cv2.label())));
-          ++oi;
+          *oi++ = Intersection_result(std::make_pair(Point_2(base_pt->first),
+                                                     base_pt->second));
+          continue;
         }
+
+        const Base_x_monotone_curve_2* base_xcv =
+          boost::get<Base_x_monotone_curve_2>(&xection);
+        CGAL_assertion(base_xcv != nullptr);
+
+        // Attach a merged label to the overlapping curve.
+        *oi++ =
+          Intersection_result(X_monotone_curve_2(*base_xcv,
+                                                 X_curve_label(cv1.label(),
+                                                               cv2.label())));
       }
 
-      return (oi);
+      return oi;
     }
   };
 
   /*! Get an Intersect_2 functor object. */
-  Intersect_2 intersect_2_object () const
-  {
-    return (Intersect_2 (this));
-  }
+  Intersect_2 intersect_2_object() const { return Intersect_2(*this); }
   //@}
 };
 
diff -pruN 5.0.3-1/include/CGAL/Minkowski_sum_2/Decomposition_strategy_adapter.h 5.1-1/include/CGAL/Minkowski_sum_2/Decomposition_strategy_adapter.h
--- 5.0.3-1/include/CGAL/Minkowski_sum_2/Decomposition_strategy_adapter.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/Minkowski_sum_2/Decomposition_strategy_adapter.h	2020-09-08 12:15:11.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/Decomposition_strategy_adapter.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/Decomposition_strategy_adapter.h $
 // $Id: Decomposition_strategy_adapter.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Minkowski_sum_2/Exact_offset_base_2.h 5.1-1/include/CGAL/Minkowski_sum_2/Exact_offset_base_2.h
--- 5.0.3-1/include/CGAL/Minkowski_sum_2/Exact_offset_base_2.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/Minkowski_sum_2/Exact_offset_base_2.h	2020-09-08 12:15:11.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/Exact_offset_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/Exact_offset_base_2.h $
 // $Id: Exact_offset_base_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Minkowski_sum_2/Hole_filter_2.h 5.1-1/include/CGAL/Minkowski_sum_2/Hole_filter_2.h
--- 5.0.3-1/include/CGAL/Minkowski_sum_2/Hole_filter_2.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/Minkowski_sum_2/Hole_filter_2.h	2020-09-08 12:15:11.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/Hole_filter_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/Hole_filter_2.h $
 // $Id: Hole_filter_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Minkowski_sum_2/Labels.h 5.1-1/include/CGAL/Minkowski_sum_2/Labels.h
--- 5.0.3-1/include/CGAL/Minkowski_sum_2/Labels.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/Minkowski_sum_2/Labels.h	2020-09-08 12:15:11.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/Labels.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/Labels.h $
 // $Id: Labels.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Minkowski_sum_2/Minkowski_sum_by_reduced_convolution_2.h 5.1-1/include/CGAL/Minkowski_sum_2/Minkowski_sum_by_reduced_convolution_2.h
--- 5.0.3-1/include/CGAL/Minkowski_sum_2/Minkowski_sum_by_reduced_convolution_2.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/Minkowski_sum_2/Minkowski_sum_by_reduced_convolution_2.h	2020-09-08 12:15:11.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/Minkowski_sum_by_reduced_convolution_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/Minkowski_sum_by_reduced_convolution_2.h $
 // $Id: Minkowski_sum_by_reduced_convolution_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Minkowski_sum_2/Minkowski_sum_conv_2.h 5.1-1/include/CGAL/Minkowski_sum_2/Minkowski_sum_conv_2.h
--- 5.0.3-1/include/CGAL/Minkowski_sum_2/Minkowski_sum_conv_2.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/Minkowski_sum_2/Minkowski_sum_conv_2.h	2020-09-08 12:15:11.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/Minkowski_sum_conv_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/Minkowski_sum_conv_2.h $
 // $Id: Minkowski_sum_conv_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Minkowski_sum_2/Minkowski_sum_decomp_2.h 5.1-1/include/CGAL/Minkowski_sum_2/Minkowski_sum_decomp_2.h
--- 5.0.3-1/include/CGAL/Minkowski_sum_2/Minkowski_sum_decomp_2.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/Minkowski_sum_2/Minkowski_sum_decomp_2.h	2020-09-08 12:15:11.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/Minkowski_sum_decomp_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/Minkowski_sum_decomp_2.h $
 // $Id: Minkowski_sum_decomp_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Minkowski_sum_2/Offset_conv_2.h 5.1-1/include/CGAL/Minkowski_sum_2/Offset_conv_2.h
--- 5.0.3-1/include/CGAL/Minkowski_sum_2/Offset_conv_2.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/Minkowski_sum_2/Offset_conv_2.h	2020-09-08 12:15:11.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/Offset_conv_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/Offset_conv_2.h $
 // $Id: Offset_conv_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Minkowski_sum_2/Offset_decomp_2.h 5.1-1/include/CGAL/Minkowski_sum_2/Offset_decomp_2.h
--- 5.0.3-1/include/CGAL/Minkowski_sum_2/Offset_decomp_2.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/Minkowski_sum_2/Offset_decomp_2.h	2020-09-08 12:15:11.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/Offset_decomp_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/Offset_decomp_2.h $
 // $Id: Offset_decomp_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Minkowski_sum_2/Polygon_convex_decomposition.h 5.1-1/include/CGAL/Minkowski_sum_2/Polygon_convex_decomposition.h
--- 5.0.3-1/include/CGAL/Minkowski_sum_2/Polygon_convex_decomposition.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/Minkowski_sum_2/Polygon_convex_decomposition.h	2020-09-08 12:15:11.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/Polygon_convex_decomposition.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/Polygon_convex_decomposition.h $
 // $Id: Polygon_convex_decomposition.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Minkowski_sum_2/Union_of_curve_cycles_2.h 5.1-1/include/CGAL/Minkowski_sum_2/Union_of_curve_cycles_2.h
--- 5.0.3-1/include/CGAL/Minkowski_sum_2/Union_of_curve_cycles_2.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/Minkowski_sum_2/Union_of_curve_cycles_2.h	2020-09-08 12:15:11.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/Union_of_curve_cycles_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/Union_of_curve_cycles_2.h $
 // $Id: Union_of_curve_cycles_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Minkowski_sum_2/Union_of_cycles_2.h 5.1-1/include/CGAL/Minkowski_sum_2/Union_of_cycles_2.h
--- 5.0.3-1/include/CGAL/Minkowski_sum_2/Union_of_cycles_2.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/Minkowski_sum_2/Union_of_cycles_2.h	2020-09-08 12:15:11.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/Union_of_cycles_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/Union_of_cycles_2.h $
 // $Id: Union_of_cycles_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Minkowski_sum_2/Union_of_segment_cycles_2.h 5.1-1/include/CGAL/Minkowski_sum_2/Union_of_segment_cycles_2.h
--- 5.0.3-1/include/CGAL/Minkowski_sum_2/Union_of_segment_cycles_2.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/Minkowski_sum_2/Union_of_segment_cycles_2.h	2020-09-08 12:15:11.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/Union_of_segment_cycles_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Minkowski_sum_2/include/CGAL/Minkowski_sum_2/Union_of_segment_cycles_2.h $
 // $Id: Union_of_segment_cycles_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/minkowski_sum_2.h 5.1-1/include/CGAL/minkowski_sum_2.h
--- 5.0.3-1/include/CGAL/minkowski_sum_2.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/minkowski_sum_2.h	2020-09-08 12:15:11.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Minkowski_sum_2/include/CGAL/minkowski_sum_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Minkowski_sum_2/include/CGAL/minkowski_sum_2.h $
 // $Id: minkowski_sum_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Minkowski_sum_3/bipartite_nary_union_sorted_combined.h 5.1-1/include/CGAL/Minkowski_sum_3/bipartite_nary_union_sorted_combined.h
--- 5.0.3-1/include/CGAL/Minkowski_sum_3/bipartite_nary_union_sorted_combined.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/Minkowski_sum_3/bipartite_nary_union_sorted_combined.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Minkowski_sum_3/include/CGAL/Minkowski_sum_3/bipartite_nary_union_sorted_combined.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Minkowski_sum_3/include/CGAL/Minkowski_sum_3/bipartite_nary_union_sorted_combined.h $
 // $Id: bipartite_nary_union_sorted_combined.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Minkowski_sum_3/Gaussian_map.h 5.1-1/include/CGAL/Minkowski_sum_3/Gaussian_map.h
--- 5.0.3-1/include/CGAL/Minkowski_sum_3/Gaussian_map.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/Minkowski_sum_3/Gaussian_map.h	2020-09-08 12:15:11.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Minkowski_sum_3/include/CGAL/Minkowski_sum_3/Gaussian_map.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Minkowski_sum_3/include/CGAL/Minkowski_sum_3/Gaussian_map.h $
 // $Id: Gaussian_map.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Minkowski_sum_3/Gaussian_map_to_nef_3.h 5.1-1/include/CGAL/Minkowski_sum_3/Gaussian_map_to_nef_3.h
--- 5.0.3-1/include/CGAL/Minkowski_sum_3/Gaussian_map_to_nef_3.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/Minkowski_sum_3/Gaussian_map_to_nef_3.h	2020-09-08 12:15:11.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Minkowski_sum_3/include/CGAL/Minkowski_sum_3/Gaussian_map_to_nef_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Minkowski_sum_3/include/CGAL/Minkowski_sum_3/Gaussian_map_to_nef_3.h $
 // $Id: Gaussian_map_to_nef_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Minkowski_sum_3/PointMark.h 5.1-1/include/CGAL/Minkowski_sum_3/PointMark.h
--- 5.0.3-1/include/CGAL/Minkowski_sum_3/PointMark.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/Minkowski_sum_3/PointMark.h	2020-09-08 12:15:11.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Minkowski_sum_3/include/CGAL/Minkowski_sum_3/PointMark.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Minkowski_sum_3/include/CGAL/Minkowski_sum_3/PointMark.h $
 // $Id: PointMark.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/minkowski_sum_3.h 5.1-1/include/CGAL/minkowski_sum_3.h
--- 5.0.3-1/include/CGAL/minkowski_sum_3.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/minkowski_sum_3.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Minkowski_sum_3/include/CGAL/minkowski_sum_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Minkowski_sum_3/include/CGAL/minkowski_sum_3.h $
 // $Id: minkowski_sum_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/min_max_n.h 5.1-1/include/CGAL/min_max_n.h
--- 5.0.3-1/include/CGAL/min_max_n.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/min_max_n.h	2020-09-08 12:15:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/min_max_n.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/min_max_n.h $
 // $Id: min_max_n.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/min_quadrilateral_2.h 5.1-1/include/CGAL/min_quadrilateral_2.h
--- 5.0.3-1/include/CGAL/min_quadrilateral_2.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/min_quadrilateral_2.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/min_quadrilateral_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/min_quadrilateral_2.h $
 // $Id: min_quadrilateral_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Min_quadrilateral_traits_2.h 5.1-1/include/CGAL/Min_quadrilateral_traits_2.h
--- 5.0.3-1/include/CGAL/Min_quadrilateral_traits_2.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Min_quadrilateral_traits_2.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Min_quadrilateral_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Min_quadrilateral_traits_2.h $
 // $Id: Min_quadrilateral_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Min_sphere_annulus_d_traits_2.h 5.1-1/include/CGAL/Min_sphere_annulus_d_traits_2.h
--- 5.0.3-1/include/CGAL/Min_sphere_annulus_d_traits_2.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Min_sphere_annulus_d_traits_2.h	2020-09-08 12:14:43.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Min_sphere_annulus_d_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Min_sphere_annulus_d_traits_2.h $
 // $Id: Min_sphere_annulus_d_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Min_sphere_annulus_d_traits_3.h 5.1-1/include/CGAL/Min_sphere_annulus_d_traits_3.h
--- 5.0.3-1/include/CGAL/Min_sphere_annulus_d_traits_3.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Min_sphere_annulus_d_traits_3.h	2020-09-08 12:14:43.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Min_sphere_annulus_d_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Min_sphere_annulus_d_traits_3.h $
 // $Id: Min_sphere_annulus_d_traits_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Min_sphere_annulus_d_traits_d.h 5.1-1/include/CGAL/Min_sphere_annulus_d_traits_d.h
--- 5.0.3-1/include/CGAL/Min_sphere_annulus_d_traits_d.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Min_sphere_annulus_d_traits_d.h	2020-09-08 12:14:43.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Min_sphere_annulus_d_traits_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Min_sphere_annulus_d_traits_d.h $
 // $Id: Min_sphere_annulus_d_traits_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Min_sphere_d/Min_sphere_d_impl.h 5.1-1/include/CGAL/Min_sphere_d/Min_sphere_d_impl.h
--- 5.0.3-1/include/CGAL/Min_sphere_d/Min_sphere_d_impl.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Min_sphere_d/Min_sphere_d_impl.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Min_sphere_d/Min_sphere_d_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Min_sphere_d/Min_sphere_d_impl.h $
 // $Id: Min_sphere_d_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Min_sphere_d/Optimisation_sphere_d.h 5.1-1/include/CGAL/Min_sphere_d/Optimisation_sphere_d.h
--- 5.0.3-1/include/CGAL/Min_sphere_d/Optimisation_sphere_d.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Min_sphere_d/Optimisation_sphere_d.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Min_sphere_d/Optimisation_sphere_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Min_sphere_d/Optimisation_sphere_d.h $
 // $Id: Optimisation_sphere_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Min_sphere_d.h 5.1-1/include/CGAL/Min_sphere_d.h
--- 5.0.3-1/include/CGAL/Min_sphere_d.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Min_sphere_d.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Min_sphere_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Min_sphere_d.h $
 // $Id: Min_sphere_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Min_sphere_of_points_d_traits_2.h 5.1-1/include/CGAL/Min_sphere_of_points_d_traits_2.h
--- 5.0.3-1/include/CGAL/Min_sphere_of_points_d_traits_2.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Min_sphere_of_points_d_traits_2.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Min_sphere_of_points_d_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Min_sphere_of_points_d_traits_2.h $
 // $Id: Min_sphere_of_points_d_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Min_sphere_of_points_d_traits_3.h 5.1-1/include/CGAL/Min_sphere_of_points_d_traits_3.h
--- 5.0.3-1/include/CGAL/Min_sphere_of_points_d_traits_3.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Min_sphere_of_points_d_traits_3.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Min_sphere_of_points_d_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Min_sphere_of_points_d_traits_3.h $
 // $Id: Min_sphere_of_points_d_traits_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Min_sphere_of_points_d_traits_d.h 5.1-1/include/CGAL/Min_sphere_of_points_d_traits_d.h
--- 5.0.3-1/include/CGAL/Min_sphere_of_points_d_traits_d.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Min_sphere_of_points_d_traits_d.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Min_sphere_of_points_d_traits_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Min_sphere_of_points_d_traits_d.h $
 // $Id: Min_sphere_of_points_d_traits_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_configure.h 5.1-1/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_configure.h
--- 5.0.3-1/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_configure.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_configure.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_configure.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_configure.h $
 // $Id: Min_sphere_of_spheres_d_configure.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_impl.h 5.1-1/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_impl.h
--- 5.0.3-1/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_impl.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_impl.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_impl.h $
 // $Id: Min_sphere_of_spheres_d_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_pair.h 5.1-1/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_pair.h
--- 5.0.3-1/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_pair.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_pair.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_pair.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_pair.h $
 // $Id: Min_sphere_of_spheres_d_pair.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_pivot_impl.h 5.1-1/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_pivot_impl.h
--- 5.0.3-1/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_pivot_impl.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_pivot_impl.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_pivot_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_pivot_impl.h $
 // $Id: Min_sphere_of_spheres_d_pivot_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_support_set.h 5.1-1/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_support_set.h
--- 5.0.3-1/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_support_set.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_support_set.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_support_set.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_support_set.h $
 // $Id: Min_sphere_of_spheres_d_support_set.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_support_set_impl.h 5.1-1/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_support_set_impl.h
--- 5.0.3-1/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_support_set_impl.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_support_set_impl.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_support_set_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Min_sphere_of_spheres_d/Min_sphere_of_spheres_d_support_set_impl.h $
 // $Id: Min_sphere_of_spheres_d_support_set_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Min_sphere_of_spheres_d.h 5.1-1/include/CGAL/Min_sphere_of_spheres_d.h
--- 5.0.3-1/include/CGAL/Min_sphere_of_spheres_d.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Min_sphere_of_spheres_d.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Min_sphere_of_spheres_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Min_sphere_of_spheres_d.h $
 // $Id: Min_sphere_of_spheres_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Min_sphere_of_spheres_d_traits_2.h 5.1-1/include/CGAL/Min_sphere_of_spheres_d_traits_2.h
--- 5.0.3-1/include/CGAL/Min_sphere_of_spheres_d_traits_2.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Min_sphere_of_spheres_d_traits_2.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Min_sphere_of_spheres_d_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Min_sphere_of_spheres_d_traits_2.h $
 // $Id: Min_sphere_of_spheres_d_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Min_sphere_of_spheres_d_traits_3.h 5.1-1/include/CGAL/Min_sphere_of_spheres_d_traits_3.h
--- 5.0.3-1/include/CGAL/Min_sphere_of_spheres_d_traits_3.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Min_sphere_of_spheres_d_traits_3.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Min_sphere_of_spheres_d_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Min_sphere_of_spheres_d_traits_3.h $
 // $Id: Min_sphere_of_spheres_d_traits_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Min_sphere_of_spheres_d_traits_d.h 5.1-1/include/CGAL/Min_sphere_of_spheres_d_traits_d.h
--- 5.0.3-1/include/CGAL/Min_sphere_of_spheres_d_traits_d.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Min_sphere_of_spheres_d_traits_d.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Min_sphere_of_spheres_d_traits_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Min_sphere_of_spheres_d_traits_d.h $
 // $Id: Min_sphere_of_spheres_d_traits_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mixed_integer_program_traits.h 5.1-1/include/CGAL/Mixed_integer_program_traits.h
--- 5.0.3-1/include/CGAL/Mixed_integer_program_traits.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Mixed_integer_program_traits.h	2020-09-08 12:15:39.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Solver_interface/include/CGAL/Mixed_integer_program_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Solver_interface/include/CGAL/Mixed_integer_program_traits.h $
 // $Id: Mixed_integer_program_traits.h 202430d 2020-04-20T15:51:48+02:00 Dmitry Anisimov
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Modifiable_priority_queue.h 5.1-1/include/CGAL/Modifiable_priority_queue.h
--- 5.0.3-1/include/CGAL/Modifiable_priority_queue.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Modifiable_priority_queue.h	2020-09-08 12:15:32.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Modifiable_priority_queue.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Modifiable_priority_queue.h $
 // $Id: Modifiable_priority_queue.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Modifier_base.h 5.1-1/include/CGAL/Modifier_base.h
--- 5.0.3-1/include/CGAL/Modifier_base.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/Modifier_base.h	2020-09-08 12:15:12.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Modifier/include/CGAL/Modifier_base.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Modifier/include/CGAL/Modifier_base.h $
 // $Id: Modifier_base.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Modular_arithmetic/Residue_type.h 5.1-1/include/CGAL/Modular_arithmetic/Residue_type.h
--- 5.0.3-1/include/CGAL/Modular_arithmetic/Residue_type.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/Modular_arithmetic/Residue_type.h	2020-09-08 12:15:12.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Modular_arithmetic/include/CGAL/Modular_arithmetic/Residue_type.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Modular_arithmetic/include/CGAL/Modular_arithmetic/Residue_type.h $
 // $Id: Residue_type.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Modular_traits.h 5.1-1/include/CGAL/Modular_traits.h
--- 5.0.3-1/include/CGAL/Modular_traits.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/Modular_traits.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Modular_arithmetic/include/CGAL/Modular_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Modular_arithmetic/include/CGAL/Modular_traits.h $
 // $Id: Modular_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Monge_via_jet_fitting.h 5.1-1/include/CGAL/Monge_via_jet_fitting.h
--- 5.0.3-1/include/CGAL/Monge_via_jet_fitting.h	2020-08-01 19:02:25.000000000 +0000
+++ 5.1-1/include/CGAL/Monge_via_jet_fitting.h	2020-09-08 12:14:59.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Jet_fitting_3/include/CGAL/Monge_via_jet_fitting.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Jet_fitting_3/include/CGAL/Monge_via_jet_fitting.h $
 // $Id: Monge_via_jet_fitting.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/monotone_matrix_search.h 5.1-1/include/CGAL/monotone_matrix_search.h
--- 5.0.3-1/include/CGAL/monotone_matrix_search.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/monotone_matrix_search.h	2020-09-08 12:15:06.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Matrix_search/include/CGAL/monotone_matrix_search.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Matrix_search/include/CGAL/monotone_matrix_search.h $
 // $Id: monotone_matrix_search.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/more_functions_on_signs.h 5.1-1/include/CGAL/more_functions_on_signs.h
--- 5.0.3-1/include/CGAL/more_functions_on_signs.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/more_functions_on_signs.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/more_functions_on_signs.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/more_functions_on_signs.h $
 // $Id: more_functions_on_signs.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/mpfi_coercion_traits.h 5.1-1/include/CGAL/mpfi_coercion_traits.h
--- 5.0.3-1/include/CGAL/mpfi_coercion_traits.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/mpfi_coercion_traits.h	2020-09-08 12:15:16.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/mpfi_coercion_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/mpfi_coercion_traits.h $
 // $Id: mpfi_coercion_traits.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/MP_Float_arithmetic_kernel.h 5.1-1/include/CGAL/MP_Float_arithmetic_kernel.h
--- 5.0.3-1/include/CGAL/MP_Float_arithmetic_kernel.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/MP_Float_arithmetic_kernel.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arithmetic_kernel/include/CGAL/MP_Float_arithmetic_kernel.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arithmetic_kernel/include/CGAL/MP_Float_arithmetic_kernel.h $
 // $Id: MP_Float_arithmetic_kernel.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/MP_Float.h 5.1-1/include/CGAL/MP_Float.h
--- 5.0.3-1/include/CGAL/MP_Float.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/MP_Float.h	2020-09-08 12:15:16.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/MP_Float.h $
-// $Id: MP_Float.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/MP_Float.h $
+// $Id: MP_Float.h 12c7bb2 2020-05-19T11:24:12+02:00 Marc Glisse
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -27,6 +27,7 @@
 #include <iostream>
 #include <vector>
 #include <algorithm>
+#include <boost/operators.hpp>
 
 // MP_Float : multiprecision scaled integers.
 
@@ -106,7 +107,13 @@ MP_Float operator*(const MP_Float &a, co
 MP_Float operator%(const MP_Float &a, const MP_Float &b);
 
 
-class MP_Float
+class MP_Float : boost::totally_ordered1<MP_Float
+#ifdef _MSC_VER
+                 , boost::ordered_ring_operators2<MP_Float, int
+                 , boost::ordered_ring_operators2<MP_Float, double
+                 > >
+#endif
+                 >
 {
 public:
   typedef short          limb;
@@ -223,6 +230,22 @@ public:
   MP_Float& operator*=(const MP_Float &a) { return *this = *this * a; }
   MP_Float& operator%=(const MP_Float &a) { return *this = *this % a; }
 
+  friend bool operator<(const MP_Float &a, const MP_Float &b)
+  { return INTERN_MP_FLOAT::compare(a, b) == SMALLER; }
+
+  friend bool operator==(const MP_Float &a, const MP_Float &b)
+  { return (a.v == b.v) && (a.v.empty() || (a.exp == b.exp)); }
+
+#ifdef _MSC_VER
+  // Needed because without /permissive-, it makes hidden friends visible (from Quotient)
+  friend bool operator==(const MP_Float &a, int    b) { return a == MP_Float(b); }
+  friend bool operator==(const MP_Float &a, double b) { return a == MP_Float(b); }
+  friend bool operator< (const MP_Float &a, int    b) { return a <  MP_Float(b); }
+  friend bool operator< (const MP_Float &a, double b) { return a <  MP_Float(b); }
+  friend bool operator> (const MP_Float &a, int    b) { return a >  MP_Float(b); }
+  friend bool operator> (const MP_Float &a, double b) { return a >  MP_Float(b); }
+#endif
+
   exponent_type max_exp() const
   {
     return exponent_type(v.size()) + exp;
@@ -365,30 +388,6 @@ inline
 void swap(MP_Float &m, MP_Float &n)
 { m.swap(n); }
 
-inline
-bool operator<(const MP_Float &a, const MP_Float &b)
-{ return INTERN_MP_FLOAT::compare(a, b) == SMALLER; }
-
-inline
-bool operator>(const MP_Float &a, const MP_Float &b)
-{ return b < a; }
-
-inline
-bool operator>=(const MP_Float &a, const MP_Float &b)
-{ return ! (a < b); }
-
-inline
-bool operator<=(const MP_Float &a, const MP_Float &b)
-{ return ! (a > b); }
-
-inline
-bool operator==(const MP_Float &a, const MP_Float &b)
-{ return (a.v == b.v) && (a.v.empty() || (a.exp == b.exp)); }
-
-inline
-bool operator!=(const MP_Float &a, const MP_Float &b)
-{ return ! (a == b); }
-
 MP_Float
 approximate_sqrt(const MP_Float &d);
 
diff -pruN 5.0.3-1/include/CGAL/MP_Float_impl.h 5.1-1/include/CGAL/MP_Float_impl.h
--- 5.0.3-1/include/CGAL/MP_Float_impl.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/MP_Float_impl.h	2020-09-08 12:15:16.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/MP_Float_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/MP_Float_impl.h $
 // $Id: MP_Float_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/mpfr_coercion_traits.h 5.1-1/include/CGAL/mpfr_coercion_traits.h
--- 5.0.3-1/include/CGAL/mpfr_coercion_traits.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/mpfr_coercion_traits.h	2020-09-08 12:15:16.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/mpfr_coercion_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/mpfr_coercion_traits.h $
 // $Id: mpfr_coercion_traits.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/mpq_class.h 5.1-1/include/CGAL/mpq_class.h
--- 5.0.3-1/include/CGAL/mpq_class.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/mpq_class.h	2020-09-08 12:15:16.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/mpq_class.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/mpq_class.h $
 // $Id: mpq_class.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/mpz_class.h 5.1-1/include/CGAL/mpz_class.h
--- 5.0.3-1/include/CGAL/mpz_class.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/mpz_class.h	2020-09-08 12:15:16.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/mpz_class.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/mpz_class.h $
 // $Id: mpz_class.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Mpzf.h 5.1-1/include/CGAL/Mpzf.h
--- 5.0.3-1/include/CGAL/Mpzf.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Mpzf.h	2020-09-08 12:15:16.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Mpzf.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Mpzf.h $
 // $Id: Mpzf.h c15d630 2020-05-14T10:49:12+02:00 Maxime Gimeno
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/mst_orient_normals.h 5.1-1/include/CGAL/mst_orient_normals.h
--- 5.0.3-1/include/CGAL/mst_orient_normals.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/mst_orient_normals.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_processing_3/include/CGAL/mst_orient_normals.h $
-// $Id: mst_orient_normals.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/mst_orient_normals.h $
+// $Id: mst_orient_normals.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s) : Laurent Saboret and Andreas Fabri
@@ -17,10 +17,7 @@
 #include <CGAL/disable_warnings.h>
 
 #include <CGAL/IO/trace.h>
-#include <CGAL/Search_traits_3.h>
-#include <CGAL/Orthogonal_k_neighbor_search.h>
-#include <CGAL/Point_set_processing_3/internal/neighbor_query.h>
-#include <CGAL/Point_set_processing_3/internal/Search_traits_vertex_handle_3.h>
+#include <CGAL/Point_set_processing_3/internal/Neighbor_query.h>
 #include <CGAL/property_map.h>
 #include <CGAL/Index_property_map.h>
 #include <CGAL/Memory_sizer.h>
@@ -296,17 +293,16 @@ mst_find_source(
 /// @tparam Kernel Geometric traits class.
 ///
 /// @return the Riemannian graph
-template <typename ForwardIterator,
+template <typename PointRange,
           typename PointMap,
           typename NormalMap,
           typename IndexMap,
           typename ConstrainedMap,
           typename Kernel
 >
-Riemannian_graph<ForwardIterator>
+Riemannian_graph<typename PointRange::iterator>
 create_riemannian_graph(
-    ForwardIterator first,  ///< iterator over the first input point.
-    ForwardIterator beyond, ///< past-the-end iterator over the input points.
+    PointRange& points, ///< input points
     PointMap point_map, ///< property map: value_type of ForwardIterator -> Point_3
     NormalMap normal_map, ///< property map: value_type of ForwardIterator -> Vector_3
     IndexMap index_map, ///< property map ForwardIterator -> index
@@ -320,43 +316,23 @@ create_riemannian_graph(
     typedef typename boost::property_traits<NormalMap>::reference Vector_ref;
 
     // Types for K nearest neighbors search structure
-    typedef Point_vertex_handle_3<ForwardIterator> Point_vertex_handle_3;
-    typedef internal::Search_traits_vertex_handle_3<ForwardIterator> Traits;
-    typedef Euclidean_distance_vertex_handle_3<ForwardIterator> KDistance;
-    typedef Orthogonal_k_neighbor_search<Traits,KDistance> Neighbor_search;
-    typedef typename Neighbor_search::Tree Tree;
+    typedef typename PointRange::iterator ForwardIterator;
+    typedef Point_set_processing_3::internal::Neighbor_query<Kernel, PointRange&, PointMap> Neighbor_query;
 
     // Riemannian_graph types
     typedef internal::Riemannian_graph<ForwardIterator> Riemannian_graph;
     typedef typename boost::property_map<Riemannian_graph, boost::edge_weight_t>::type Riemannian_graph_weight_map;
 
-    // Precondition: at least one element in the container.
-    CGAL_point_set_processing_precondition(first != beyond);
-
     // Precondition: at least 2 nearest neighbors
     CGAL_point_set_processing_precondition(k >= 2);
 
     // Number of input points
-    const std::size_t num_input_points = distance(first, beyond);
+    const std::size_t num_input_points = points.size();
 
     std::size_t memory = CGAL::Memory_sizer().virtual_size(); CGAL_TRACE("  %ld Mb allocated\n", memory>>20);
     CGAL_TRACE("  Creates KD-tree\n");
 
-    // Instanciate a KD-tree search.
-    // Notes: We have to wrap each input point by a Point_vertex_handle_3.
-    //        The KD-tree is allocated dynamically to recover RAM as soon as possible.
-    std::vector<Point_vertex_handle_3> kd_tree_points; kd_tree_points.reserve(num_input_points);
-    for (ForwardIterator it = first; it != beyond; it++)
-    {
-
-        Point_ref point = get(point_map, *it);
-        Point_vertex_handle_3 point_wrapper(point.x(), point.y(), point.z(), it);
-        kd_tree_points.push_back(point_wrapper);
-    }
-    boost::shared_ptr<Tree> tree( new Tree(kd_tree_points.begin(), kd_tree_points.end()) );
-
-    // Recover RAM
-    kd_tree_points.clear();
+    Neighbor_query neighbor_query (points, point_map);
 
     memory = CGAL::Memory_sizer().virtual_size(); CGAL_TRACE("  %ld Mb allocated\n", memory>>20);
     CGAL_TRACE("  Creates Riemannian Graph\n");
@@ -369,7 +345,7 @@ create_riemannian_graph(
     Riemannian_graph riemannian_graph;
     //
     // add vertices
-    for (ForwardIterator it = first; it != beyond; it++)
+    for (ForwardIterator it = points.begin(); it != points.end(); it++)
     {
         typename Riemannian_graph::vertex_descriptor v = add_vertex(riemannian_graph);
         CGAL_point_set_processing_assertion(v == get(index_map,it));
@@ -383,16 +359,14 @@ create_riemannian_graph(
     //
     // add edges
     Riemannian_graph_weight_map riemannian_graph_weight_map = get(boost::edge_weight, riemannian_graph);
-    for (ForwardIterator it = first; it != beyond; it++)
+    for (ForwardIterator it = points.begin(); it != points.end(); it++)
     {
         std::size_t it_index = get(index_map,it);
         Vector_ref it_normal_vector = get(normal_map,*it);
 
         Point_ref point = get(point_map, *it);
-        Point_vertex_handle_3 point_wrapper(point.x(), point.y(), point.z(), it);
-        std::vector<Point_vertex_handle_3> neighbor_points;
-        CGAL::Point_set_processing_3::internal::neighbor_query
-          (point_wrapper, *tree, k, neighbor_radius, neighbor_points);
+        std::vector<ForwardIterator> neighbor_points;
+        neighbor_query.get_iterators (point, k, neighbor_radius, std::back_inserter(neighbor_points));
 
         for (std::size_t i = 0; i < neighbor_points.size(); ++ i)
         {
@@ -573,25 +547,48 @@ create_mst_graph(
 
    \param points input point range.
    \param k number of neighbors.
-   \param np optional sequence of \ref psp_namedparameters "Named Parameters" among the ones listed below.
+   \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
    \cgalNamedParamsBegin
-     \cgalParamBegin{point_map} a model of `ReadablePropertyMap` with value type `geom_traits::Point_3`.
-     If this parameter is omitted, `CGAL::Identity_property_map<geom_traits::Point_3>` is used.\cgalParamEnd
-     \cgalParamBegin{normal_map} a model of `ReadWritePropertyMap` with value type
-     `geom_traits::Vector_3`.\cgalParamEnd
-     \cgalParamBegin{neighbor_radius} spherical neighborhood radius. If
-     provided, the neighborhood of a query point is computed with a fixed spherical
-     radius instead of a fixed number of neighbors. In that case, the parameter
-     `k` is used as a limit on the number of points returned by each spherical
-     query (to avoid overly large number of points in high density areas). If no
-     limit is wanted, use `k=0`.\cgalParamEnd
-     \cgalParamBegin{point_is_constrained_map} a model of `ReadablePropertyMap` with value type
-     `bool`. Points with a `true` value will be used as seed points: their normal will be considered as already
-     oriented, it won't be altered and it will be propagated to its neighbors. If this parameter is omitted,
-     the highest point (highest Z coordinate) will be used as the unique seed with an upward oriented
-     normal\cgalParamEnd
-     \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+     \cgalParamNBegin{point_map}
+       \cgalParamDescription{a property map associating points to the elements of the point set `points`}
+       \cgalParamType{a model of `ReadablePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange` and whose value type is `geom_traits::Point_3`}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{normal_map}
+       \cgalParamDescription{a property map associating normals to the elements of the point set `points`}
+       \cgalParamType{a model of `WritablePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange` and whose value type is `geom_traits::Vector_3`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{neighbor_radius}
+       \cgalParamDescription{the spherical neighborhood radius}
+       \cgalParamType{floating scalar value}
+       \cgalParamDefault{`0` (no limit)}
+       \cgalParamExtra{If provided, the neighborhood of a query point is computed with a fixed spherical
+                       radius instead of a fixed number of neighbors. In that case, the parameter
+                       `k` is used as a limit on the number of points returned by each spherical
+                       query (to avoid overly large number of points in high density areas).}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{point_is_constrained_map}
+       \cgalParamDescription{a property map containing information about points being constrained or not}
+       \cgalParamType{a class model of `ReadablePropertyMap` with `PointRange::iterator::value_type`
+                      as key type and `bool` as value type}
+       \cgalParamDefault{If this parameter is omitted, the highest point (highest Z coordinate)
+                         will be used as the unique seed with an upward oriented.}
+       \cgalParamExtra{Points with a `true` value will be used as seed points: their normal
+                       will be considered as already oriented, it won't be altered
+                       and it will be propagated to its neighbors.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{geom_traits}
+       \cgalParamDescription{an instance of a geometric traits class}
+       \cgalParamType{a model of `Kernel`}
+       \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+     \cgalParamNEnd
    \cgalNamedParamsEnd
 
    \return iterator over the first point with an unoriented normal.
@@ -610,7 +607,7 @@ mst_orient_normals(
 
     CGAL_TRACE("Calls mst_orient_normals()\n");
 
-    typedef typename Point_set_processing_3::GetPointMap<PointRange, NamedParameters>::type PointMap;
+    typedef typename CGAL::GetPointMap<PointRange, NamedParameters>::type PointMap;
     typedef typename Point_set_processing_3::GetNormalMap<PointRange, NamedParameters>::type NormalMap;
     typedef typename Point_set_processing_3::GetK<PointRange, NamedParameters>::Kernel Kernel;
     typedef typename Point_set_processing_3::GetIsConstrainedMap<PointRange, NamedParameters>::type ConstrainedMap;
@@ -619,11 +616,11 @@ mst_orient_normals(
                                 typename Point_set_processing_3::GetNormalMap<PointRange, NamedParameters>::NoMap>::value),
                               "Error: no normal map");
 
-    PointMap point_map = choose_parameter(get_parameter(np, internal_np::point_map), PointMap());
-    NormalMap normal_map = choose_parameter(get_parameter(np, internal_np::normal_map), NormalMap());
+    PointMap point_map = choose_parameter<PointMap>(get_parameter(np, internal_np::point_map));
+    NormalMap normal_map = choose_parameter<NormalMap>(get_parameter(np, internal_np::normal_map));
     typename Kernel::FT neighbor_radius = choose_parameter(get_parameter(np, internal_np::neighbor_radius),
-                                                       typename Kernel::FT(0));
-    ConstrainedMap constrained_map = choose_parameter(get_parameter(np, internal_np::point_is_constrained), ConstrainedMap());
+                                                           typename Kernel::FT(0));
+    ConstrainedMap constrained_map = choose_parameter<ConstrainedMap>(get_parameter(np, internal_np::point_is_constrained));
     Kernel kernel;
 
   // Bring private stuff to scope
@@ -665,7 +662,7 @@ mst_orient_normals(
 
     if (boost::is_same<ConstrainedMap,
         typename CGAL::Point_set_processing_3::GetIsConstrainedMap<PointRange, NamedParameters>::NoMap>::value)
-      riemannian_graph = create_riemannian_graph(points.begin(), points.end(),
+      riemannian_graph = create_riemannian_graph(points,
                                                  point_map, normal_map, index_map,
                                                  Default_constrained_map<typename PointRange::iterator>
                                                  (mst_find_source(points.begin(), points.end(),
@@ -675,7 +672,7 @@ mst_orient_normals(
                                                  neighbor_radius,
                                                  kernel);
     else
-      riemannian_graph = create_riemannian_graph(points.begin(), points.end(),
+      riemannian_graph = create_riemannian_graph(points,
                                                  point_map, normal_map, index_map,
                                                  constrained_map,
                                                  k,
diff -pruN 5.0.3-1/include/CGAL/MSVC_compiler_config.h 5.1-1/include/CGAL/MSVC_compiler_config.h
--- 5.0.3-1/include/CGAL/MSVC_compiler_config.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/MSVC_compiler_config.h	2020-09-08 12:14:54.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/MSVC_compiler_config.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/MSVC_compiler_config.h $
 // $Id: MSVC_compiler_config.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Multiscale_sort.h 5.1-1/include/CGAL/Multiscale_sort.h
--- 5.0.3-1/include/CGAL/Multiscale_sort.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Multiscale_sort.h	2020-09-08 12:15:40.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_sorting/include/CGAL/Multiscale_sort.h $
-// $Id: Multiscale_sort.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_sorting/include/CGAL/Multiscale_sort.h $
+// $Id: Multiscale_sort.h 5c41b10 2020-01-02T10:26:44+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Christophe Delage
@@ -22,28 +22,28 @@ namespace CGAL {
 template <class Sort>
 class Multiscale_sort
 {
-    Sort _sort;
-    std::ptrdiff_t _threshold;
-    double _ratio;
+  Sort _sort;
+  std::ptrdiff_t _threshold;
+  double _ratio;
 
 public:
-    Multiscale_sort (const Sort &sort = Sort(), std::ptrdiff_t threshold = 1, double ratio = 0.5)
-        : _sort (sort), _threshold (threshold), _ratio (ratio)
-    {
-        CGAL_precondition (0. <= ratio && ratio <= 1.);
-    }
-
-    template <class RandomAccessIterator>
-    void operator() (RandomAccessIterator begin, RandomAccessIterator end) const
-    {
-        typedef typename std::iterator_traits<RandomAccessIterator>::difference_type difference_type;
-        RandomAccessIterator middle = begin;
-        if (end - begin >= _threshold) {
-            middle = begin + difference_type (double(end - begin) * _ratio);
-            this->operator() (begin, middle);
-        }
-        _sort (middle, end);
+  Multiscale_sort (const Sort &sort = Sort(), std::ptrdiff_t threshold = 1, double ratio = 0.5)
+    : _sort (sort), _threshold (threshold), _ratio (ratio)
+  {
+    CGAL_precondition (0. <= ratio && ratio <= 1.);
+  }
+
+  template <class RandomAccessIterator>
+  void operator() (RandomAccessIterator begin, RandomAccessIterator end) const
+  {
+    typedef typename std::iterator_traits<RandomAccessIterator>::difference_type difference_type;
+    RandomAccessIterator middle = begin;
+    if (end - begin >= _threshold) {
+      middle = begin + difference_type (double(end - begin) * _ratio);
+      this->operator() (begin, middle);
     }
+    _sort (middle, end);
+  }
 };
 
 } // namespace CGAL
diff -pruN 5.0.3-1/include/CGAL/multiset_assertions.h 5.1-1/include/CGAL/multiset_assertions.h
--- 5.0.3-1/include/CGAL/multiset_assertions.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/multiset_assertions.h	2020-09-08 12:15:32.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/multiset_assertions.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/multiset_assertions.h $
 // $Id: multiset_assertions.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Multiset.h 5.1-1/include/CGAL/Multiset.h
--- 5.0.3-1/include/CGAL/Multiset.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Multiset.h	2020-09-08 12:15:32.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Multiset.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Multiset.h $
 // $Id: Multiset.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Multi_surface_3.h 5.1-1/include/CGAL/Multi_surface_3.h
--- 5.0.3-1/include/CGAL/Multi_surface_3.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Multi_surface_3.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Multi_surface_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Multi_surface_3.h $
 // $Id: Multi_surface_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/mutex.h 5.1-1/include/CGAL/mutex.h
--- 5.0.3-1/include/CGAL/mutex.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/mutex.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/mutex.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/mutex.h $
 // $Id: mutex.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 
diff -pruN 5.0.3-1/include/CGAL/natural_neighbor_coordinates_2.h 5.1-1/include/CGAL/natural_neighbor_coordinates_2.h
--- 5.0.3-1/include/CGAL/natural_neighbor_coordinates_2.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/natural_neighbor_coordinates_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Interpolation/include/CGAL/natural_neighbor_coordinates_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Interpolation/include/CGAL/natural_neighbor_coordinates_2.h $
 // $Id: natural_neighbor_coordinates_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/natural_neighbor_coordinates_3.h 5.1-1/include/CGAL/natural_neighbor_coordinates_3.h
--- 5.0.3-1/include/CGAL/natural_neighbor_coordinates_3.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/natural_neighbor_coordinates_3.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Interpolation/include/CGAL/natural_neighbor_coordinates_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Interpolation/include/CGAL/natural_neighbor_coordinates_3.h $
 // $Id: natural_neighbor_coordinates_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/nearest_neighbor_delaunay_2.h 5.1-1/include/CGAL/nearest_neighbor_delaunay_2.h
--- 5.0.3-1/include/CGAL/nearest_neighbor_delaunay_2.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/nearest_neighbor_delaunay_2.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_2/include/CGAL/nearest_neighbor_delaunay_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_2/include/CGAL/nearest_neighbor_delaunay_2.h $
 // $Id: nearest_neighbor_delaunay_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Needs_parens_as_product.h 5.1-1/include/CGAL/Needs_parens_as_product.h
--- 5.0.3-1/include/CGAL/Needs_parens_as_product.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/Needs_parens_as_product.h	2020-09-08 12:14:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_foundations/include/CGAL/Needs_parens_as_product.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_foundations/include/CGAL/Needs_parens_as_product.h $
 // $Id: Needs_parens_as_product.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_2/Bounding_box_2.h 5.1-1/include/CGAL/Nef_2/Bounding_box_2.h
--- 5.0.3-1/include/CGAL/Nef_2/Bounding_box_2.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_2/Bounding_box_2.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_2/include/CGAL/Nef_2/Bounding_box_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_2/include/CGAL/Nef_2/Bounding_box_2.h $
 // $Id: Bounding_box_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_2/Constrained_triang_traits.h 5.1-1/include/CGAL/Nef_2/Constrained_triang_traits.h
--- 5.0.3-1/include/CGAL/Nef_2/Constrained_triang_traits.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_2/Constrained_triang_traits.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_2/include/CGAL/Nef_2/Constrained_triang_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_2/include/CGAL/Nef_2/Constrained_triang_traits.h $
 // $Id: Constrained_triang_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_2/debug.h 5.1-1/include/CGAL/Nef_2/debug.h
--- 5.0.3-1/include/CGAL/Nef_2/debug.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_2/debug.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_2/include/CGAL/Nef_2/debug.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_2/include/CGAL/Nef_2/debug.h $
 // $Id: debug.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_2/geninfo.h 5.1-1/include/CGAL/Nef_2/geninfo.h
--- 5.0.3-1/include/CGAL/Nef_2/geninfo.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_2/geninfo.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_2/include/CGAL/Nef_2/geninfo.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_2/include/CGAL/Nef_2/geninfo.h $
 // $Id: geninfo.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_2/gen_point_location.h 5.1-1/include/CGAL/Nef_2/gen_point_location.h
--- 5.0.3-1/include/CGAL/Nef_2/gen_point_location.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_2/gen_point_location.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_2/include/CGAL/Nef_2/gen_point_location.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_2/include/CGAL/Nef_2/gen_point_location.h $
 // $Id: gen_point_location.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_2/HDS_items.h 5.1-1/include/CGAL/Nef_2/HDS_items.h
--- 5.0.3-1/include/CGAL/Nef_2/HDS_items.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_2/HDS_items.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_2/include/CGAL/Nef_2/HDS_items.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_2/include/CGAL/Nef_2/HDS_items.h $
 // $Id: HDS_items.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_2/iterator_tools.h 5.1-1/include/CGAL/Nef_2/iterator_tools.h
--- 5.0.3-1/include/CGAL/Nef_2/iterator_tools.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_2/iterator_tools.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_2/include/CGAL/Nef_2/iterator_tools.h $
-// $Id: iterator_tools.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_2/include/CGAL/Nef_2/iterator_tools.h $
+// $Id: iterator_tools.h 9d16a42 2020-06-15T17:07:35+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -48,6 +48,12 @@ public:
     bool operator!=( const Self& i) const {
         return !(*this == i);
     }
+    bool operator==( const Iter& i ) const {
+      return Iter::operator==(i);
+    }
+    bool operator!=( const Iter& i) const {
+        return !(*this == i);
+    }
 
     Self& operator++() {
       Move move;
diff -pruN 5.0.3-1/include/CGAL/Nef_2/Line_to_epoint.h 5.1-1/include/CGAL/Nef_2/Line_to_epoint.h
--- 5.0.3-1/include/CGAL/Nef_2/Line_to_epoint.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_2/Line_to_epoint.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_2/include/CGAL/Nef_2/Line_to_epoint.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_2/include/CGAL/Nef_2/Line_to_epoint.h $
 // $Id: Line_to_epoint.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_2/Object_handle.h 5.1-1/include/CGAL/Nef_2/Object_handle.h
--- 5.0.3-1/include/CGAL/Nef_2/Object_handle.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_2/Object_handle.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_2/include/CGAL/Nef_2/Object_handle.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_2/include/CGAL/Nef_2/Object_handle.h $
 // $Id: Object_handle.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_2/Object_index.h 5.1-1/include/CGAL/Nef_2/Object_index.h
--- 5.0.3-1/include/CGAL/Nef_2/Object_index.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_2/Object_index.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_2/include/CGAL/Nef_2/Object_index.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_2/include/CGAL/Nef_2/Object_index.h $
 // $Id: Object_index.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_2/PM_checker.h 5.1-1/include/CGAL/Nef_2/PM_checker.h
--- 5.0.3-1/include/CGAL/Nef_2/PM_checker.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_2/PM_checker.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_2/include/CGAL/Nef_2/PM_checker.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_2/include/CGAL/Nef_2/PM_checker.h $
 // $Id: PM_checker.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_2/PM_const_decorator.h 5.1-1/include/CGAL/Nef_2/PM_const_decorator.h
--- 5.0.3-1/include/CGAL/Nef_2/PM_const_decorator.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_2/PM_const_decorator.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_2/include/CGAL/Nef_2/PM_const_decorator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_2/include/CGAL/Nef_2/PM_const_decorator.h $
 // $Id: PM_const_decorator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_2/PM_decorator.h 5.1-1/include/CGAL/Nef_2/PM_decorator.h
--- 5.0.3-1/include/CGAL/Nef_2/PM_decorator.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_2/PM_decorator.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_2/include/CGAL/Nef_2/PM_decorator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_2/include/CGAL/Nef_2/PM_decorator.h $
 // $Id: PM_decorator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_2/PM_explorer.h 5.1-1/include/CGAL/Nef_2/PM_explorer.h
--- 5.0.3-1/include/CGAL/Nef_2/PM_explorer.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_2/PM_explorer.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_2/include/CGAL/Nef_2/PM_explorer.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_2/include/CGAL/Nef_2/PM_explorer.h $
 // $Id: PM_explorer.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_2/PM_io_parser.h 5.1-1/include/CGAL/Nef_2/PM_io_parser.h
--- 5.0.3-1/include/CGAL/Nef_2/PM_io_parser.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_2/PM_io_parser.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_2/include/CGAL/Nef_2/PM_io_parser.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_2/include/CGAL/Nef_2/PM_io_parser.h $
 // $Id: PM_io_parser.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_2/PM_overlayer.h 5.1-1/include/CGAL/Nef_2/PM_overlayer.h
--- 5.0.3-1/include/CGAL/Nef_2/PM_overlayer.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_2/PM_overlayer.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_2/include/CGAL/Nef_2/PM_overlayer.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_2/include/CGAL/Nef_2/PM_overlayer.h $
 // $Id: PM_overlayer.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_2/PM_persistent_PL.h 5.1-1/include/CGAL/Nef_2/PM_persistent_PL.h
--- 5.0.3-1/include/CGAL/Nef_2/PM_persistent_PL.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_2/PM_persistent_PL.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_2/include/CGAL/Nef_2/PM_persistent_PL.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_2/include/CGAL/Nef_2/PM_persistent_PL.h $
 // $Id: PM_persistent_PL.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_2/PM_point_locator.h 5.1-1/include/CGAL/Nef_2/PM_point_locator.h
--- 5.0.3-1/include/CGAL/Nef_2/PM_point_locator.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_2/PM_point_locator.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_2/include/CGAL/Nef_2/PM_point_locator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_2/include/CGAL/Nef_2/PM_point_locator.h $
 // $Id: PM_point_locator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_2/Polynomial.h 5.1-1/include/CGAL/Nef_2/Polynomial.h
--- 5.0.3-1/include/CGAL/Nef_2/Polynomial.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_2/Polynomial.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_2/include/CGAL/Nef_2/Polynomial.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_2/include/CGAL/Nef_2/Polynomial.h $
 // $Id: Polynomial.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_2/Polynomial_impl.h 5.1-1/include/CGAL/Nef_2/Polynomial_impl.h
--- 5.0.3-1/include/CGAL/Nef_2/Polynomial_impl.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_2/Polynomial_impl.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_2/include/CGAL/Nef_2/Polynomial_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_2/include/CGAL/Nef_2/Polynomial_impl.h $
 // $Id: Polynomial_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_2/Segment_overlay_traits.h 5.1-1/include/CGAL/Nef_2/Segment_overlay_traits.h
--- 5.0.3-1/include/CGAL/Nef_2/Segment_overlay_traits.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_2/Segment_overlay_traits.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_2/include/CGAL/Nef_2/Segment_overlay_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_2/include/CGAL/Nef_2/Segment_overlay_traits.h $
 // $Id: Segment_overlay_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/Binary_operation.h 5.1-1/include/CGAL/Nef_3/Binary_operation.h
--- 5.0.3-1/include/CGAL/Nef_3/Binary_operation.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/Binary_operation.h	2020-09-08 12:15:13.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/Binary_operation.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/Binary_operation.h $
 // $Id: Binary_operation.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/binop_intersection_tests.h 5.1-1/include/CGAL/Nef_3/binop_intersection_tests.h
--- 5.0.3-1/include/CGAL/Nef_3/binop_intersection_tests.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/binop_intersection_tests.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/binop_intersection_tests.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/binop_intersection_tests.h $
 // $Id: binop_intersection_tests.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/bounded_side_3.h 5.1-1/include/CGAL/Nef_3/bounded_side_3.h
--- 5.0.3-1/include/CGAL/Nef_3/bounded_side_3.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/bounded_side_3.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/bounded_side_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/bounded_side_3.h $
 // $Id: bounded_side_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/Bounding_box_3.h 5.1-1/include/CGAL/Nef_3/Bounding_box_3.h
--- 5.0.3-1/include/CGAL/Nef_3/Bounding_box_3.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/Bounding_box_3.h	2020-09-08 12:15:13.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/Bounding_box_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/Bounding_box_3.h $
 // $Id: Bounding_box_3.h b1c09a5 2020-07-20T16:52:24+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/Combine_with_halfspace.h 5.1-1/include/CGAL/Nef_3/Combine_with_halfspace.h
--- 5.0.3-1/include/CGAL/Nef_3/Combine_with_halfspace.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/Combine_with_halfspace.h	2020-09-08 12:15:13.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/Combine_with_halfspace.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/Combine_with_halfspace.h $
 // $Id: Combine_with_halfspace.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/Default_items.h 5.1-1/include/CGAL/Nef_3/Default_items.h
--- 5.0.3-1/include/CGAL/Nef_3/Default_items.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/Default_items.h	2020-09-08 12:15:13.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/Default_items.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/Default_items.h $
 // $Id: Default_items.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/Edge_edge_overlay.h 5.1-1/include/CGAL/Nef_3/Edge_edge_overlay.h
--- 5.0.3-1/include/CGAL/Nef_3/Edge_edge_overlay.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/Edge_edge_overlay.h	2020-09-08 12:15:13.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/Edge_edge_overlay.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/Edge_edge_overlay.h $
 // $Id: Edge_edge_overlay.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/Exact_triangulation_euclidean_traits_xy_3.h 5.1-1/include/CGAL/Nef_3/Exact_triangulation_euclidean_traits_xy_3.h
--- 5.0.3-1/include/CGAL/Nef_3/Exact_triangulation_euclidean_traits_xy_3.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/Exact_triangulation_euclidean_traits_xy_3.h	2020-09-08 12:15:13.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/Exact_triangulation_euclidean_traits_xy_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/Exact_triangulation_euclidean_traits_xy_3.h $
 // $Id: Exact_triangulation_euclidean_traits_xy_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/Exact_triangulation_euclidean_traits_xz_3.h 5.1-1/include/CGAL/Nef_3/Exact_triangulation_euclidean_traits_xz_3.h
--- 5.0.3-1/include/CGAL/Nef_3/Exact_triangulation_euclidean_traits_xz_3.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/Exact_triangulation_euclidean_traits_xz_3.h	2020-09-08 12:15:13.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/Exact_triangulation_euclidean_traits_xz_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/Exact_triangulation_euclidean_traits_xz_3.h $
 // $Id: Exact_triangulation_euclidean_traits_xz_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/Exact_triangulation_euclidean_traits_yz_3.h 5.1-1/include/CGAL/Nef_3/Exact_triangulation_euclidean_traits_yz_3.h
--- 5.0.3-1/include/CGAL/Nef_3/Exact_triangulation_euclidean_traits_yz_3.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/Exact_triangulation_euclidean_traits_yz_3.h	2020-09-08 12:15:13.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/Exact_triangulation_euclidean_traits_yz_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/Exact_triangulation_euclidean_traits_yz_3.h $
 // $Id: Exact_triangulation_euclidean_traits_yz_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/Halfedge.h 5.1-1/include/CGAL/Nef_3/Halfedge.h
--- 5.0.3-1/include/CGAL/Nef_3/Halfedge.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/Halfedge.h	2020-09-08 12:15:13.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/Halfedge.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/Halfedge.h $
 // $Id: Halfedge.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/Halffacet.h 5.1-1/include/CGAL/Nef_3/Halffacet.h
--- 5.0.3-1/include/CGAL/Nef_3/Halffacet.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/Halffacet.h	2020-09-08 12:15:13.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/Halffacet.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/Halffacet.h $
 // $Id: Halffacet.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/ID_support_handler.h 5.1-1/include/CGAL/Nef_3/ID_support_handler.h
--- 5.0.3-1/include/CGAL/Nef_3/ID_support_handler.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/ID_support_handler.h	2020-09-08 12:15:13.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/ID_support_handler.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/ID_support_handler.h $
 // $Id: ID_support_handler.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/Infimaximal_box.h 5.1-1/include/CGAL/Nef_3/Infimaximal_box.h
--- 5.0.3-1/include/CGAL/Nef_3/Infimaximal_box.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/Infimaximal_box.h	2020-09-08 12:15:13.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/Infimaximal_box.h $
-// $Id: Infimaximal_box.h 5b2d46c 2020-04-21T10:03:00+02:00 Maxime Gimeno
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/Infimaximal_box.h $
+// $Id: Infimaximal_box.h e9d41d7 2020-04-21T10:03:00+02:00 Maxime Gimeno
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/K3_tree.h 5.1-1/include/CGAL/Nef_3/K3_tree.h
--- 5.0.3-1/include/CGAL/Nef_3/K3_tree.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/K3_tree.h	2020-09-08 12:15:13.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/K3_tree.h $
-// $Id: K3_tree.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/K3_tree.h $
+// $Id: K3_tree.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -162,11 +162,11 @@ private:
   template<typename SNC_structure, typename Kernel>
   class Triangulation_handler {
 
-    typedef typename CGAL::Triangulation_vertex_base_2<Kernel>               Vb;
-    typedef typename CGAL::Constrained_triangulation_face_base_2<Kernel>     Fb;
-    typedef typename CGAL::Triangulation_data_structure_2<Vb,Fb>             TDS;
-    typedef typename CGAL::No_intersection_tag                               Itag;
-    typedef typename CGAL::Constrained_triangulation_2<Kernel,TDS,Itag>      CT;
+    typedef typename CGAL::Triangulation_vertex_base_2<Kernel>                      Vb;
+    typedef typename CGAL::Constrained_triangulation_face_base_2<Kernel>            Fb;
+    typedef typename CGAL::Triangulation_data_structure_2<Vb,Fb>                    TDS;
+    typedef typename CGAL::No_constraint_intersection_requiring_constructions_tag   Itag;
+    typedef typename CGAL::Constrained_triangulation_2<Kernel,TDS,Itag>             CT;
 
     typedef typename CT::Face_handle           Face_handle;
     typedef typename CT::Finite_faces_iterator Finite_face_iterator;
diff -pruN 5.0.3-1/include/CGAL/Nef_3/Mark_bounded_volumes.h 5.1-1/include/CGAL/Nef_3/Mark_bounded_volumes.h
--- 5.0.3-1/include/CGAL/Nef_3/Mark_bounded_volumes.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/Mark_bounded_volumes.h	2020-09-08 12:15:13.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/Mark_bounded_volumes.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/Mark_bounded_volumes.h $
 // $Id: Mark_bounded_volumes.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/Nef_box.h 5.1-1/include/CGAL/Nef_3/Nef_box.h
--- 5.0.3-1/include/CGAL/Nef_3/Nef_box.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/Nef_box.h	2020-09-08 12:15:13.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/Nef_box.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/Nef_box.h $
 // $Id: Nef_box.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/OGL_helper.h 5.1-1/include/CGAL/Nef_3/OGL_helper.h
--- 5.0.3-1/include/CGAL/Nef_3/OGL_helper.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/OGL_helper.h	2020-09-08 12:15:13.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/OGL_helper.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/OGL_helper.h $
 // $Id: OGL_helper.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/Pluecker_line_3.h 5.1-1/include/CGAL/Nef_3/Pluecker_line_3.h
--- 5.0.3-1/include/CGAL/Nef_3/Pluecker_line_3.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/Pluecker_line_3.h	2020-09-08 12:15:13.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/Pluecker_line_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/Pluecker_line_3.h $
 // $Id: Pluecker_line_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/polygon_mesh_to_nef_3.h 5.1-1/include/CGAL/Nef_3/polygon_mesh_to_nef_3.h
--- 5.0.3-1/include/CGAL/Nef_3/polygon_mesh_to_nef_3.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/polygon_mesh_to_nef_3.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/polygon_mesh_to_nef_3.h $
-// $Id: polygon_mesh_to_nef_3.h 7b601f4 2020-04-19T14:54:19+02:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/polygon_mesh_to_nef_3.h $
+// $Id: polygon_mesh_to_nef_3.h adb0b9d 2020-04-19T14:27:48+02:00 R.E. Wolff
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/quotient_coordinates_to_homogeneous_point.h 5.1-1/include/CGAL/Nef_3/quotient_coordinates_to_homogeneous_point.h
--- 5.0.3-1/include/CGAL/Nef_3/quotient_coordinates_to_homogeneous_point.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/quotient_coordinates_to_homogeneous_point.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/quotient_coordinates_to_homogeneous_point.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/quotient_coordinates_to_homogeneous_point.h $
 // $Id: quotient_coordinates_to_homogeneous_point.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/SFace.h 5.1-1/include/CGAL/Nef_3/SFace.h
--- 5.0.3-1/include/CGAL/Nef_3/SFace.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/SFace.h	2020-09-08 12:15:13.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/SFace.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/SFace.h $
 // $Id: SFace.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/SHalfedge.h 5.1-1/include/CGAL/Nef_3/SHalfedge.h
--- 5.0.3-1/include/CGAL/Nef_3/SHalfedge.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/SHalfedge.h	2020-09-08 12:15:13.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/SHalfedge.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/SHalfedge.h $
 // $Id: SHalfedge.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/SHalfloop.h 5.1-1/include/CGAL/Nef_3/SHalfloop.h
--- 5.0.3-1/include/CGAL/Nef_3/SHalfloop.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/SHalfloop.h	2020-09-08 12:15:13.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/SHalfloop.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/SHalfloop.h $
 // $Id: SHalfloop.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/shell_to_nef_3.h 5.1-1/include/CGAL/Nef_3/shell_to_nef_3.h
--- 5.0.3-1/include/CGAL/Nef_3/shell_to_nef_3.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/shell_to_nef_3.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/shell_to_nef_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/shell_to_nef_3.h $
 // $Id: shell_to_nef_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/SM_visualizor.h 5.1-1/include/CGAL/Nef_3/SM_visualizor.h
--- 5.0.3-1/include/CGAL/Nef_3/SM_visualizor.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/SM_visualizor.h	2020-09-08 12:15:13.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/SM_visualizor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/SM_visualizor.h $
 // $Id: SM_visualizor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/SNC_const_decorator.h 5.1-1/include/CGAL/Nef_3/SNC_const_decorator.h
--- 5.0.3-1/include/CGAL/Nef_3/SNC_const_decorator.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/SNC_const_decorator.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/SNC_const_decorator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/SNC_const_decorator.h $
 // $Id: SNC_const_decorator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/SNC_constructor.h 5.1-1/include/CGAL/Nef_3/SNC_constructor.h
--- 5.0.3-1/include/CGAL/Nef_3/SNC_constructor.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/SNC_constructor.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/SNC_constructor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/SNC_constructor.h $
 // $Id: SNC_constructor.h b1c09a5 2020-07-20T16:52:24+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/SNC_decorator.h 5.1-1/include/CGAL/Nef_3/SNC_decorator.h
--- 5.0.3-1/include/CGAL/Nef_3/SNC_decorator.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/SNC_decorator.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/SNC_decorator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/SNC_decorator.h $
 // $Id: SNC_decorator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/SNC_decorator_traits.h 5.1-1/include/CGAL/Nef_3/SNC_decorator_traits.h
--- 5.0.3-1/include/CGAL/Nef_3/SNC_decorator_traits.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/SNC_decorator_traits.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/SNC_decorator_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/SNC_decorator_traits.h $
 // $Id: SNC_decorator_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/SNC_external_structure.h 5.1-1/include/CGAL/Nef_3/SNC_external_structure.h
--- 5.0.3-1/include/CGAL/Nef_3/SNC_external_structure.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/SNC_external_structure.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/SNC_external_structure.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/SNC_external_structure.h $
 // $Id: SNC_external_structure.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/SNC_FM_decorator.h 5.1-1/include/CGAL/Nef_3/SNC_FM_decorator.h
--- 5.0.3-1/include/CGAL/Nef_3/SNC_FM_decorator.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/SNC_FM_decorator.h	2020-09-08 12:15:13.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/SNC_FM_decorator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/SNC_FM_decorator.h $
 // $Id: SNC_FM_decorator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/SNC_indexed_items.h 5.1-1/include/CGAL/Nef_3/SNC_indexed_items.h
--- 5.0.3-1/include/CGAL/Nef_3/SNC_indexed_items.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/SNC_indexed_items.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/SNC_indexed_items.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/SNC_indexed_items.h $
 // $Id: SNC_indexed_items.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/SNC_intersection.h 5.1-1/include/CGAL/Nef_3/SNC_intersection.h
--- 5.0.3-1/include/CGAL/Nef_3/SNC_intersection.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/SNC_intersection.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/SNC_intersection.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/SNC_intersection.h $
 // $Id: SNC_intersection.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/SNC_io_parser.h 5.1-1/include/CGAL/Nef_3/SNC_io_parser.h
--- 5.0.3-1/include/CGAL/Nef_3/SNC_io_parser.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/SNC_io_parser.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/SNC_io_parser.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/SNC_io_parser.h $
 // $Id: SNC_io_parser.h b1c09a5 2020-07-20T16:52:24+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/SNC_items.h 5.1-1/include/CGAL/Nef_3/SNC_items.h
--- 5.0.3-1/include/CGAL/Nef_3/SNC_items.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/SNC_items.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/SNC_items.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/SNC_items.h $
 // $Id: SNC_items.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/SNC_iteration.h 5.1-1/include/CGAL/Nef_3/SNC_iteration.h
--- 5.0.3-1/include/CGAL/Nef_3/SNC_iteration.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/SNC_iteration.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/SNC_iteration.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/SNC_iteration.h $
 // $Id: SNC_iteration.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/SNC_k3_tree_traits.h 5.1-1/include/CGAL/Nef_3/SNC_k3_tree_traits.h
--- 5.0.3-1/include/CGAL/Nef_3/SNC_k3_tree_traits.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/SNC_k3_tree_traits.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/SNC_k3_tree_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/SNC_k3_tree_traits.h $
 // $Id: SNC_k3_tree_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/SNC_list.h 5.1-1/include/CGAL/Nef_3/SNC_list.h
--- 5.0.3-1/include/CGAL/Nef_3/SNC_list.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/SNC_list.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/SNC_list.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/SNC_list.h $
 // $Id: SNC_list.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/SNC_point_locator.h 5.1-1/include/CGAL/Nef_3/SNC_point_locator.h
--- 5.0.3-1/include/CGAL/Nef_3/SNC_point_locator.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/SNC_point_locator.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/SNC_point_locator.h $
-// $Id: SNC_point_locator.h 213221e 2020-07-24T14:02:40+02:00 Maxime Gimeno
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/SNC_point_locator.h $
+// $Id: SNC_point_locator.h 84ec799 2020-07-28T16:09:28+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -212,11 +212,11 @@ public:
   template<typename Kernel>
   class Triangulation_handler {
 
-    typedef typename CGAL::Triangulation_vertex_base_2<Kernel>               Vb;
-    typedef typename CGAL::Constrained_triangulation_face_base_2<Kernel>     Fb;
-    typedef typename CGAL::Triangulation_data_structure_2<Vb,Fb>             TDS;
-    typedef typename CGAL::No_intersection_tag                               Itag;
-    typedef typename CGAL::Constrained_triangulation_2<Kernel,TDS,Itag>      CT;
+    typedef typename CGAL::Triangulation_vertex_base_2<Kernel>                      Vb;
+    typedef typename CGAL::Constrained_triangulation_face_base_2<Kernel>            Fb;
+    typedef typename CGAL::Triangulation_data_structure_2<Vb,Fb>                    TDS;
+    typedef typename CGAL::No_constraint_intersection_requiring_constructions_tag   Itag;
+    typedef typename CGAL::Constrained_triangulation_2<Kernel,TDS,Itag>             CT;
 
     typedef typename CT::Face_handle           Face_handle;
     typedef typename CT::Finite_faces_iterator Finite_face_iterator;
diff -pruN 5.0.3-1/include/CGAL/Nef_3/SNC_simplify.h 5.1-1/include/CGAL/Nef_3/SNC_simplify.h
--- 5.0.3-1/include/CGAL/Nef_3/SNC_simplify.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/SNC_simplify.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/SNC_simplify.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/SNC_simplify.h $
 // $Id: SNC_simplify.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/SNC_SM_explorer.h 5.1-1/include/CGAL/Nef_3/SNC_SM_explorer.h
--- 5.0.3-1/include/CGAL/Nef_3/SNC_SM_explorer.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/SNC_SM_explorer.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/SNC_SM_explorer.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/SNC_SM_explorer.h $
 // $Id: SNC_SM_explorer.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/SNC_SM_overlayer.h 5.1-1/include/CGAL/Nef_3/SNC_SM_overlayer.h
--- 5.0.3-1/include/CGAL/Nef_3/SNC_SM_overlayer.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/SNC_SM_overlayer.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/SNC_SM_overlayer.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/SNC_SM_overlayer.h $
 // $Id: SNC_SM_overlayer.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/SNC_SM_visualizor.h 5.1-1/include/CGAL/Nef_3/SNC_SM_visualizor.h
--- 5.0.3-1/include/CGAL/Nef_3/SNC_SM_visualizor.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/SNC_SM_visualizor.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/SNC_SM_visualizor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/SNC_SM_visualizor.h $
 // $Id: SNC_SM_visualizor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/SNC_sphere_map.h 5.1-1/include/CGAL/Nef_3/SNC_sphere_map.h
--- 5.0.3-1/include/CGAL/Nef_3/SNC_sphere_map.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/SNC_sphere_map.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/SNC_sphere_map.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/SNC_sphere_map.h $
 // $Id: SNC_sphere_map.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/SNC_structure.h 5.1-1/include/CGAL/Nef_3/SNC_structure.h
--- 5.0.3-1/include/CGAL/Nef_3/SNC_structure.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/SNC_structure.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/SNC_structure.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/SNC_structure.h $
 // $Id: SNC_structure.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/vertex_cycle_to_nef_3.h 5.1-1/include/CGAL/Nef_3/vertex_cycle_to_nef_3.h
--- 5.0.3-1/include/CGAL/Nef_3/vertex_cycle_to_nef_3.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/vertex_cycle_to_nef_3.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/vertex_cycle_to_nef_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/vertex_cycle_to_nef_3.h $
 // $Id: vertex_cycle_to_nef_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/Vertex.h 5.1-1/include/CGAL/Nef_3/Vertex.h
--- 5.0.3-1/include/CGAL/Nef_3/Vertex.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/Vertex.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/Vertex.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/Vertex.h $
 // $Id: Vertex.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_3/Volume.h 5.1-1/include/CGAL/Nef_3/Volume.h
--- 5.0.3-1/include/CGAL/Nef_3/Volume.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_3/Volume.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_3/Volume.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_3/Volume.h $
 // $Id: Volume.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_nary_intersection_3.h 5.1-1/include/CGAL/Nef_nary_intersection_3.h
--- 5.0.3-1/include/CGAL/Nef_nary_intersection_3.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_nary_intersection_3.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_nary_intersection_3.h $
-// $Id: Nef_nary_intersection_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_nary_intersection_3.h $
+// $Id: Nef_nary_intersection_3.h a86712d 2020-06-07T15:25:18+02:00 Thomas Krijnen
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -52,7 +52,8 @@ class Nef_nary_intersection_3 {
   }
 
   Polyhedron get_intersection() {
-
+    if (queue.empty())
+      return empty;
     while(queue.size() > 1)
       intersect();
     inserted = 0;
diff -pruN 5.0.3-1/include/CGAL/Nef_nary_union_3.h 5.1-1/include/CGAL/Nef_nary_union_3.h
--- 5.0.3-1/include/CGAL/Nef_nary_union_3.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_nary_union_3.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_nary_union_3.h $
-// $Id: Nef_nary_union_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_nary_union_3.h $
+// $Id: Nef_nary_union_3.h a86712d 2020-06-07T15:25:18+02:00 Thomas Krijnen
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -52,7 +52,8 @@ class Nef_nary_union_3 {
   }
 
   Polyhedron get_union() {
-
+    if (queue.empty())
+      return empty;
     while(queue.size() > 1)
       unite();
     inserted = 0;
diff -pruN 5.0.3-1/include/CGAL/Nef_polyhedron_2.h 5.1-1/include/CGAL/Nef_polyhedron_2.h
--- 5.0.3-1/include/CGAL/Nef_polyhedron_2.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_polyhedron_2.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_2/include/CGAL/Nef_polyhedron_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_2/include/CGAL/Nef_polyhedron_2.h $
 // $Id: Nef_polyhedron_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_polyhedron_3.h 5.1-1/include/CGAL/Nef_polyhedron_3.h
--- 5.0.3-1/include/CGAL/Nef_polyhedron_3.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_polyhedron_3.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/Nef_polyhedron_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/Nef_polyhedron_3.h $
 // $Id: Nef_polyhedron_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_polyhedron_S2.h 5.1-1/include/CGAL/Nef_polyhedron_S2.h
--- 5.0.3-1/include/CGAL/Nef_polyhedron_S2.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_polyhedron_S2.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_S2/include/CGAL/Nef_polyhedron_S2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_S2/include/CGAL/Nef_polyhedron_S2.h $
 // $Id: Nef_polyhedron_S2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_polynomial_fwd.h 5.1-1/include/CGAL/Nef_polynomial_fwd.h
--- 5.0.3-1/include/CGAL/Nef_polynomial_fwd.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_polynomial_fwd.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_2/include/CGAL/Nef_polynomial_fwd.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_2/include/CGAL/Nef_polynomial_fwd.h $
 // $Id: Nef_polynomial_fwd.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_polynomial.h 5.1-1/include/CGAL/Nef_polynomial.h
--- 5.0.3-1/include/CGAL/Nef_polynomial.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_polynomial.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_2/include/CGAL/Nef_polynomial.h $
-// $Id: Nef_polynomial.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_2/include/CGAL/Nef_polynomial.h $
+// $Id: Nef_polynomial.h 9d16a42 2020-06-15T17:07:35+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -69,6 +69,41 @@ class Nef_polynomial
       CGAL_STATIC_THREAD_LOCAL_VARIABLE(NT, R_, 1);
       return R_;
     }
+
+  friend bool operator==(const Nef_polynomial<NT> &a, const Nef_polynomial<NT> &b)
+  {
+    return a.polynomial() == b.polynomial();
+  }
+
+  friend bool operator==(const Nef_polynomial<NT> &a, const NT& b)
+  {
+    return a.polynomial() == b;
+  }
+
+  friend bool operator==(const Nef_polynomial<NT> &a, int b)
+  {
+    return a.polynomial() == b;
+  }
+
+  friend bool operator<(const Nef_polynomial<NT> &a, const Nef_polynomial<NT> &b)
+  {
+    return a.polynomial() < b.polynomial();
+  }
+
+  friend bool operator<(const Nef_polynomial<NT> &a, const NT& b)
+  {
+    return a.polynomial() < b;
+  }
+
+  friend bool operator<(const Nef_polynomial<NT> &a, int b)
+  {
+    return a.polynomial() < b;
+  }
+
+  friend bool operator>(const Nef_polynomial<NT> &a, int b)
+  {
+    return a.polynomial() > b;
+  }
 };
 
 template <class NT>
@@ -85,42 +120,6 @@ Nef_polynomial<NT> operator-(const Nef_p
   return - a.polynomial();
 }
 
-template <class NT>
-inline
-bool operator<(const Nef_polynomial<NT> &a, const Nef_polynomial<NT> &b)
-{
-  return a.polynomial() < b.polynomial();
-}
-
-template <class NT>
-inline
-bool operator==(const Nef_polynomial<NT> &a, const Nef_polynomial<NT> &b)
-{
-  return a.polynomial() == b.polynomial();
-}
-
-template <class NT>
-inline
-bool operator==(const Nef_polynomial<NT> &a, int b)
-{
-  return a.polynomial() == b;
-}
-
-template <class NT>
-inline
-bool operator<(const Nef_polynomial<NT> &a, int b)
-{
-  return a.polynomial() < b;
-}
-
-template <class NT>
-inline
-bool operator>(const Nef_polynomial<NT> &a, int b)
-{
-  return a.polynomial() > b;
-}
-
-
 #undef CGAL_double
 #undef CGAL_int
 
diff -pruN 5.0.3-1/include/CGAL/Nef_S2/Generic_handle_map.h 5.1-1/include/CGAL/Nef_S2/Generic_handle_map.h
--- 5.0.3-1/include/CGAL/Nef_S2/Generic_handle_map.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_S2/Generic_handle_map.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_S2/include/CGAL/Nef_S2/Generic_handle_map.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_S2/include/CGAL/Nef_S2/Generic_handle_map.h $
 // $Id: Generic_handle_map.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_S2/ID_support_handler.h 5.1-1/include/CGAL/Nef_S2/ID_support_handler.h
--- 5.0.3-1/include/CGAL/Nef_S2/ID_support_handler.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_S2/ID_support_handler.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_S2/include/CGAL/Nef_S2/ID_support_handler.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_S2/include/CGAL/Nef_S2/ID_support_handler.h $
 // $Id: ID_support_handler.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_S2/leda_sphere_map.h 5.1-1/include/CGAL/Nef_S2/leda_sphere_map.h
--- 5.0.3-1/include/CGAL/Nef_S2/leda_sphere_map.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_S2/leda_sphere_map.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_S2/include/CGAL/Nef_S2/leda_sphere_map.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_S2/include/CGAL/Nef_S2/leda_sphere_map.h $
 // $Id: leda_sphere_map.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_S2/Normalizing.h 5.1-1/include/CGAL/Nef_S2/Normalizing.h
--- 5.0.3-1/include/CGAL/Nef_S2/Normalizing.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_S2/Normalizing.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_S2/include/CGAL/Nef_S2/Normalizing.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_S2/include/CGAL/Nef_S2/Normalizing.h $
 // $Id: Normalizing.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_S2/OGL_base_object.h 5.1-1/include/CGAL/Nef_S2/OGL_base_object.h
--- 5.0.3-1/include/CGAL/Nef_S2/OGL_base_object.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_S2/OGL_base_object.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_S2/include/CGAL/Nef_S2/OGL_base_object.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_S2/include/CGAL/Nef_S2/OGL_base_object.h $
 // $Id: OGL_base_object.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_S2/SM_checker.h 5.1-1/include/CGAL/Nef_S2/SM_checker.h
--- 5.0.3-1/include/CGAL/Nef_S2/SM_checker.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_S2/SM_checker.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_S2/include/CGAL/Nef_S2/SM_checker.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_S2/include/CGAL/Nef_S2/SM_checker.h $
 // $Id: SM_checker.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_S2/SM_const_decorator.h 5.1-1/include/CGAL/Nef_S2/SM_const_decorator.h
--- 5.0.3-1/include/CGAL/Nef_S2/SM_const_decorator.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_S2/SM_const_decorator.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_S2/include/CGAL/Nef_S2/SM_const_decorator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_S2/include/CGAL/Nef_S2/SM_const_decorator.h $
 // $Id: SM_const_decorator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_S2/SM_constrained_triang_traits.h 5.1-1/include/CGAL/Nef_S2/SM_constrained_triang_traits.h
--- 5.0.3-1/include/CGAL/Nef_S2/SM_constrained_triang_traits.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_S2/SM_constrained_triang_traits.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_S2/include/CGAL/Nef_S2/SM_constrained_triang_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_S2/include/CGAL/Nef_S2/SM_constrained_triang_traits.h $
 // $Id: SM_constrained_triang_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_S2/SM_decorator.h 5.1-1/include/CGAL/Nef_S2/SM_decorator.h
--- 5.0.3-1/include/CGAL/Nef_S2/SM_decorator.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_S2/SM_decorator.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_S2/include/CGAL/Nef_S2/SM_decorator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_S2/include/CGAL/Nef_S2/SM_decorator.h $
 // $Id: SM_decorator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_S2/SM_decorator_traits.h 5.1-1/include/CGAL/Nef_S2/SM_decorator_traits.h
--- 5.0.3-1/include/CGAL/Nef_S2/SM_decorator_traits.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_S2/SM_decorator_traits.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_S2/include/CGAL/Nef_S2/SM_decorator_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_S2/include/CGAL/Nef_S2/SM_decorator_traits.h $
 // $Id: SM_decorator_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_S2/SM_io_parser.h 5.1-1/include/CGAL/Nef_S2/SM_io_parser.h
--- 5.0.3-1/include/CGAL/Nef_S2/SM_io_parser.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_S2/SM_io_parser.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_S2/include/CGAL/Nef_S2/SM_io_parser.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_S2/include/CGAL/Nef_S2/SM_io_parser.h $
 // $Id: SM_io_parser.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_S2/SM_items.h 5.1-1/include/CGAL/Nef_S2/SM_items.h
--- 5.0.3-1/include/CGAL/Nef_S2/SM_items.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_S2/SM_items.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_S2/include/CGAL/Nef_S2/SM_items.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_S2/include/CGAL/Nef_S2/SM_items.h $
 // $Id: SM_items.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_S2/SM_iteration.h 5.1-1/include/CGAL/Nef_S2/SM_iteration.h
--- 5.0.3-1/include/CGAL/Nef_S2/SM_iteration.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_S2/SM_iteration.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_S2/include/CGAL/Nef_S2/SM_iteration.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_S2/include/CGAL/Nef_S2/SM_iteration.h $
 // $Id: SM_iteration.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_S2/SM_list.h 5.1-1/include/CGAL/Nef_S2/SM_list.h
--- 5.0.3-1/include/CGAL/Nef_S2/SM_list.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_S2/SM_list.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_S2/include/CGAL/Nef_S2/SM_list.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_S2/include/CGAL/Nef_S2/SM_list.h $
 // $Id: SM_list.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_S2/SM_overlayer.h 5.1-1/include/CGAL/Nef_S2/SM_overlayer.h
--- 5.0.3-1/include/CGAL/Nef_S2/SM_overlayer.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_S2/SM_overlayer.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_S2/include/CGAL/Nef_S2/SM_overlayer.h $
-// $Id: SM_overlayer.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_S2/include/CGAL/Nef_S2/SM_overlayer.h $
+// $Id: SM_overlayer.h fdb17cb 2020-03-26T19:26:10+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -540,8 +540,13 @@ public:
     Mark m[2];
     Object_handle o_supp[2];
     SHalfedge_handle e_below;
+
     vertex_info()
-    { o_supp[0]=o_supp[1]=Object_handle(); }
+    {
+      m[0]=m[1]=Mark();
+      o_supp[0]=o_supp[1]=Object_handle();
+    }
+
     LEDA_MEMORY(vertex_info)
   }; // vertex_info
 
diff -pruN 5.0.3-1/include/CGAL/Nef_S2/SM_point_locator.h 5.1-1/include/CGAL/Nef_S2/SM_point_locator.h
--- 5.0.3-1/include/CGAL/Nef_S2/SM_point_locator.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_S2/SM_point_locator.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_S2/include/CGAL/Nef_S2/SM_point_locator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_S2/include/CGAL/Nef_S2/SM_point_locator.h $
 // $Id: SM_point_locator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_S2/SM_triangulator.h 5.1-1/include/CGAL/Nef_S2/SM_triangulator.h
--- 5.0.3-1/include/CGAL/Nef_S2/SM_triangulator.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_S2/SM_triangulator.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_S2/include/CGAL/Nef_S2/SM_triangulator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_S2/include/CGAL/Nef_S2/SM_triangulator.h $
 // $Id: SM_triangulator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_S2/SM_visualizor.h 5.1-1/include/CGAL/Nef_S2/SM_visualizor.h
--- 5.0.3-1/include/CGAL/Nef_S2/SM_visualizor.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_S2/SM_visualizor.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_S2/include/CGAL/Nef_S2/SM_visualizor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_S2/include/CGAL/Nef_S2/SM_visualizor.h $
 // $Id: SM_visualizor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_S2/Sphere_circle.h 5.1-1/include/CGAL/Nef_S2/Sphere_circle.h
--- 5.0.3-1/include/CGAL/Nef_S2/Sphere_circle.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_S2/Sphere_circle.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_S2/include/CGAL/Nef_S2/Sphere_circle.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_S2/include/CGAL/Nef_S2/Sphere_circle.h $
 // $Id: Sphere_circle.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_S2/Sphere_direction.h 5.1-1/include/CGAL/Nef_S2/Sphere_direction.h
--- 5.0.3-1/include/CGAL/Nef_S2/Sphere_direction.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_S2/Sphere_direction.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_S2/include/CGAL/Nef_S2/Sphere_direction.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_S2/include/CGAL/Nef_S2/Sphere_direction.h $
 // $Id: Sphere_direction.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_S2/Sphere_geometry.h 5.1-1/include/CGAL/Nef_S2/Sphere_geometry.h
--- 5.0.3-1/include/CGAL/Nef_S2/Sphere_geometry.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_S2/Sphere_geometry.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_S2/include/CGAL/Nef_S2/Sphere_geometry.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_S2/include/CGAL/Nef_S2/Sphere_geometry.h $
 // $Id: Sphere_geometry.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_S2/Sphere_geometry_OGL.h 5.1-1/include/CGAL/Nef_S2/Sphere_geometry_OGL.h
--- 5.0.3-1/include/CGAL/Nef_S2/Sphere_geometry_OGL.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_S2/Sphere_geometry_OGL.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_S2/include/CGAL/Nef_S2/Sphere_geometry_OGL.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_S2/include/CGAL/Nef_S2/Sphere_geometry_OGL.h $
 // $Id: Sphere_geometry_OGL.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_S2/Sphere_map.h 5.1-1/include/CGAL/Nef_S2/Sphere_map.h
--- 5.0.3-1/include/CGAL/Nef_S2/Sphere_map.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_S2/Sphere_map.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_S2/include/CGAL/Nef_S2/Sphere_map.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_S2/include/CGAL/Nef_S2/Sphere_map.h $
 // $Id: Sphere_map.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_S2/Sphere_point.h 5.1-1/include/CGAL/Nef_S2/Sphere_point.h
--- 5.0.3-1/include/CGAL/Nef_S2/Sphere_point.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_S2/Sphere_point.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_S2/include/CGAL/Nef_S2/Sphere_point.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_S2/include/CGAL/Nef_S2/Sphere_point.h $
 // $Id: Sphere_point.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_S2/sphere_predicates.h 5.1-1/include/CGAL/Nef_S2/sphere_predicates.h
--- 5.0.3-1/include/CGAL/Nef_S2/sphere_predicates.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_S2/sphere_predicates.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_S2/include/CGAL/Nef_S2/sphere_predicates.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_S2/include/CGAL/Nef_S2/sphere_predicates.h $
 // $Id: sphere_predicates.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_S2/Sphere_segment.h 5.1-1/include/CGAL/Nef_S2/Sphere_segment.h
--- 5.0.3-1/include/CGAL/Nef_S2/Sphere_segment.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_S2/Sphere_segment.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_S2/include/CGAL/Nef_S2/Sphere_segment.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_S2/include/CGAL/Nef_S2/Sphere_segment.h $
 // $Id: Sphere_segment.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nef_S2/Sphere_triangle.h 5.1-1/include/CGAL/Nef_S2/Sphere_triangle.h
--- 5.0.3-1/include/CGAL/Nef_S2/Sphere_triangle.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/Nef_S2/Sphere_triangle.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_S2/include/CGAL/Nef_S2/Sphere_triangle.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_S2/include/CGAL/Nef_S2/Sphere_triangle.h $
 // $Id: Sphere_triangle.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Nested_iterator.h 5.1-1/include/CGAL/Nested_iterator.h
--- 5.0.3-1/include/CGAL/Nested_iterator.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Nested_iterator.h	2020-09-08 12:15:32.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Nested_iterator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Nested_iterator.h $
 // $Id: Nested_iterator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Cartesian_base.h 5.1-1/include/CGAL/NewKernel_d/Cartesian_base.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Cartesian_base.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Cartesian_base.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Cartesian_base.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Cartesian_base.h $
 // $Id: Cartesian_base.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Cartesian_change_FT.h 5.1-1/include/CGAL/NewKernel_d/Cartesian_change_FT.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Cartesian_change_FT.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Cartesian_change_FT.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Cartesian_change_FT.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Cartesian_change_FT.h $
 // $Id: Cartesian_change_FT.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Cartesian_complete.h 5.1-1/include/CGAL/NewKernel_d/Cartesian_complete.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Cartesian_complete.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Cartesian_complete.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Cartesian_complete.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Cartesian_complete.h $
 // $Id: Cartesian_complete.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Cartesian_filter_K.h 5.1-1/include/CGAL/NewKernel_d/Cartesian_filter_K.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Cartesian_filter_K.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Cartesian_filter_K.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Cartesian_filter_K.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Cartesian_filter_K.h $
 // $Id: Cartesian_filter_K.h 6aec7dc 2020-04-09T23:50:26+02:00 Marc Glisse
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Cartesian_filter_NT.h 5.1-1/include/CGAL/NewKernel_d/Cartesian_filter_NT.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Cartesian_filter_NT.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Cartesian_filter_NT.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Cartesian_filter_NT.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Cartesian_filter_NT.h $
 // $Id: Cartesian_filter_NT.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Cartesian_LA_base.h 5.1-1/include/CGAL/NewKernel_d/Cartesian_LA_base.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Cartesian_LA_base.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Cartesian_LA_base.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Cartesian_LA_base.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Cartesian_LA_base.h $
 // $Id: Cartesian_LA_base.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Cartesian_LA_functors.h 5.1-1/include/CGAL/NewKernel_d/Cartesian_LA_functors.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Cartesian_LA_functors.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Cartesian_LA_functors.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Cartesian_LA_functors.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Cartesian_LA_functors.h $
 // $Id: Cartesian_LA_functors.h 822bc55 2020-03-27T08:28:48+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Cartesian_per_dimension.h 5.1-1/include/CGAL/NewKernel_d/Cartesian_per_dimension.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Cartesian_per_dimension.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Cartesian_per_dimension.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Cartesian_per_dimension.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Cartesian_per_dimension.h $
 // $Id: Cartesian_per_dimension.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Cartesian_static_filters.h 5.1-1/include/CGAL/NewKernel_d/Cartesian_static_filters.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Cartesian_static_filters.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Cartesian_static_filters.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Cartesian_static_filters.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Cartesian_static_filters.h $
 // $Id: Cartesian_static_filters.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Coaffine.h 5.1-1/include/CGAL/NewKernel_d/Coaffine.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Coaffine.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Coaffine.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Coaffine.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Coaffine.h $
 // $Id: Coaffine.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Define_kernel_types.h 5.1-1/include/CGAL/NewKernel_d/Define_kernel_types.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Define_kernel_types.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Define_kernel_types.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Define_kernel_types.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Define_kernel_types.h $
 // $Id: Define_kernel_types.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Dimension_base.h 5.1-1/include/CGAL/NewKernel_d/Dimension_base.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Dimension_base.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Dimension_base.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Dimension_base.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Dimension_base.h $
 // $Id: Dimension_base.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Filtered_predicate2.h 5.1-1/include/CGAL/NewKernel_d/Filtered_predicate2.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Filtered_predicate2.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Filtered_predicate2.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Filtered_predicate2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Filtered_predicate2.h $
 // $Id: Filtered_predicate2.h 822bc55 2020-03-27T08:28:48+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/function_objects_cartesian.h 5.1-1/include/CGAL/NewKernel_d/function_objects_cartesian.h
--- 5.0.3-1/include/CGAL/NewKernel_d/function_objects_cartesian.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/function_objects_cartesian.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/function_objects_cartesian.h $
-// $Id: function_objects_cartesian.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/function_objects_cartesian.h $
+// $Id: function_objects_cartesian.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Marc Glisse
@@ -665,11 +665,13 @@ template <class R_> struct Construct_cir
       Vec b(k);
       Vec l(k);
       int j,i=0;
+      // We are doing a quadratic number of *f, which can be costly with transforming_iterator.
       for(Iter f2=f;f2!=e;++f2,++i){
-        b(i)=m(i,i)=sdo(*f2);
+        Point const& p2=*f2;
+        b(i)=m(i,i)=sdo(p2);
         j=0;
         for(Iter f3=f;f3!=e;++f3,++j){
-          m(j,i)=m(i,j)=sp(*f2,*f3);
+          m(j,i)=m(i,j)=sp(p2,*f3);
         }
       }
       for(i=1;i<k;++i){
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/functor_properties.h 5.1-1/include/CGAL/NewKernel_d/functor_properties.h
--- 5.0.3-1/include/CGAL/NewKernel_d/functor_properties.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/functor_properties.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/functor_properties.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/functor_properties.h $
 // $Id: functor_properties.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/functor_tags.h 5.1-1/include/CGAL/NewKernel_d/functor_tags.h
--- 5.0.3-1/include/CGAL/NewKernel_d/functor_tags.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/functor_tags.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/functor_tags.h $
-// $Id: functor_tags.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/functor_tags.h $
+// $Id: functor_tags.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Marc Glisse
@@ -303,6 +303,7 @@ namespace CGAL {
         CGAL_DECL_PREDICATE(Contained_in_simplex);
         CGAL_DECL_PREDICATE(Power_side_of_power_sphere_raw);
         CGAL_DECL_PREDICATE(Power_side_of_power_sphere);
+        CGAL_DECL_PREDICATE(Power_side_of_bounded_power_circumsphere);
         CGAL_DECL_PREDICATE(In_flat_power_side_of_power_sphere_raw);
         CGAL_DECL_PREDICATE(In_flat_power_side_of_power_sphere);
 #undef CGAL_DECL_PREDICATE
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Kernel_2_interface.h 5.1-1/include/CGAL/NewKernel_d/Kernel_2_interface.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Kernel_2_interface.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Kernel_2_interface.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Kernel_2_interface.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Kernel_2_interface.h $
 // $Id: Kernel_2_interface.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Kernel_3_interface.h 5.1-1/include/CGAL/NewKernel_d/Kernel_3_interface.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Kernel_3_interface.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Kernel_3_interface.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Kernel_3_interface.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Kernel_3_interface.h $
 // $Id: Kernel_3_interface.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/KernelD_converter.h 5.1-1/include/CGAL/NewKernel_d/KernelD_converter.h
--- 5.0.3-1/include/CGAL/NewKernel_d/KernelD_converter.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/KernelD_converter.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/KernelD_converter.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/KernelD_converter.h $
 // $Id: KernelD_converter.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Kernel_d_interface.h 5.1-1/include/CGAL/NewKernel_d/Kernel_d_interface.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Kernel_d_interface.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Kernel_d_interface.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Kernel_d_interface.h $
-// $Id: Kernel_d_interface.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Kernel_d_interface.h $
+// $Id: Kernel_d_interface.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Marc Glisse
@@ -58,6 +58,7 @@ template <class Base_> struct Kernel_d_i
         typedef typename Get_functor<Base, Point_dimension_tag>::type Point_dimension_d;
         typedef typename Get_functor<Base, Side_of_oriented_sphere_tag>::type Side_of_oriented_sphere_d;
         typedef typename Get_functor<Base, Power_side_of_power_sphere_tag>::type Power_side_of_power_sphere_d;
+        typedef typename Get_functor<Base, Power_side_of_bounded_power_circumsphere_tag>::type Power_side_of_bounded_power_sphere_d;
         typedef typename Get_functor<Base, Power_center_tag>::type Power_center_d;
         typedef typename Get_functor<Base, Power_distance_tag>::type Power_distance_d;
         typedef typename Get_functor<Base, Contained_in_affine_hull_tag>::type Contained_in_affine_hull_d;
@@ -166,6 +167,16 @@ template <class Base_> struct Kernel_d_i
             return typename Get_functor<Base, Squared_circumradius_tag>::type(this->kernel())(b,e);
           }
         };
+        struct Compute_squared_radius_smallest_orthogonal_sphere_d : private Store_kernel<Kernel> {
+          typedef Kernel R_; // for the macro
+          CGAL_FUNCTOR_INIT_STORE(Compute_squared_radius_smallest_orthogonal_sphere_d)
+          typedef FT result_type;
+          template<class I> FT operator()(I b, I e)const{
+            typename Get_functor<Base, Point_weight_tag>::type pw(this->kernel());
+            typename Get_functor<Base, Power_center_tag>::type pc(this->kernel());
+            return pw(pc(b,e));
+          }
+        };
         typedef typename Construct_cartesian_const_iterator_d::result_type Cartesian_const_iterator_d;
         typedef typename Get_functor<Base, Squared_distance_tag>::type Squared_distance_d;
         typedef typename Get_functor<Base, Squared_length_tag>::type Squared_length_d;
@@ -210,6 +221,7 @@ template <class Base_> struct Kernel_d_i
         Point_of_sphere_d point_of_sphere_d_object()const{ return Point_of_sphere_d(*this); }
         Side_of_oriented_sphere_d side_of_oriented_sphere_d_object()const{ return Side_of_oriented_sphere_d(*this); }
         Power_side_of_power_sphere_d power_side_of_power_sphere_d_object()const{ return Power_side_of_power_sphere_d(*this); }
+        Power_side_of_bounded_power_sphere_d power_side_of_bounded_power_sphere_d_object()const{ return Power_side_of_bounded_power_sphere_d(*this); }
         Power_center_d power_center_d_object()const{ return Power_center_d(*this); }
         Power_distance_d power_distance_d_object()const{ return Power_distance_d(*this); }
         Side_of_bounded_sphere_d side_of_bounded_sphere_d_object()const{ return Side_of_bounded_sphere_d(*this); }
@@ -243,6 +255,7 @@ template <class Base_> struct Kernel_d_i
         Construct_sphere_d construct_sphere_d_object()const{ return Construct_sphere_d(*this); }
         Construct_hyperplane_d construct_hyperplane_d_object()const{ return Construct_hyperplane_d(*this); }
         Compute_squared_radius_d compute_squared_radius_d_object()const{ return Compute_squared_radius_d(*this); }
+        Compute_squared_radius_smallest_orthogonal_sphere_d compute_squared_radius_smallest_orthogonal_sphere_d_object()const{ return Compute_squared_radius_smallest_orthogonal_sphere_d(*this); }
         Squared_distance_d squared_distance_d_object()const{ return Squared_distance_d(*this); }
         Squared_length_d squared_length_d_object()const{ return Squared_length_d(*this); }
         Scalar_product_d scalar_product_d_object()const{ return Scalar_product_d(*this); }
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Kernel_object_converter.h 5.1-1/include/CGAL/NewKernel_d/Kernel_object_converter.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Kernel_object_converter.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Kernel_object_converter.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Kernel_object_converter.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Kernel_object_converter.h $
 // $Id: Kernel_object_converter.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/LA_eigen/constructors.h 5.1-1/include/CGAL/NewKernel_d/LA_eigen/constructors.h
--- 5.0.3-1/include/CGAL/NewKernel_d/LA_eigen/constructors.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/LA_eigen/constructors.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/LA_eigen/constructors.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/LA_eigen/constructors.h $
 // $Id: constructors.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/LA_eigen/LA.h 5.1-1/include/CGAL/NewKernel_d/LA_eigen/LA.h
--- 5.0.3-1/include/CGAL/NewKernel_d/LA_eigen/LA.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/LA_eigen/LA.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/LA_eigen/LA.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/LA_eigen/LA.h $
 // $Id: LA.h 8a52dd3 2020-05-12T11:21:06+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Lazy_cartesian.h 5.1-1/include/CGAL/NewKernel_d/Lazy_cartesian.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Lazy_cartesian.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Lazy_cartesian.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Lazy_cartesian.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Lazy_cartesian.h $
 // $Id: Lazy_cartesian.h 822bc55 2020-03-27T08:28:48+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/static_int.h 5.1-1/include/CGAL/NewKernel_d/static_int.h
--- 5.0.3-1/include/CGAL/NewKernel_d/static_int.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/static_int.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/static_int.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/static_int.h $
 // $Id: static_int.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/store_kernel.h 5.1-1/include/CGAL/NewKernel_d/store_kernel.h
--- 5.0.3-1/include/CGAL/NewKernel_d/store_kernel.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/store_kernel.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/store_kernel.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/store_kernel.h $
 // $Id: store_kernel.h a03eff6 2020-04-08T16:03:18+02:00 Marc Glisse
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Types/Aff_transformation.h 5.1-1/include/CGAL/NewKernel_d/Types/Aff_transformation.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Types/Aff_transformation.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Types/Aff_transformation.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Types/Aff_transformation.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Types/Aff_transformation.h $
 // $Id: Aff_transformation.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Types/Hyperplane.h 5.1-1/include/CGAL/NewKernel_d/Types/Hyperplane.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Types/Hyperplane.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Types/Hyperplane.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Types/Hyperplane.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Types/Hyperplane.h $
 // $Id: Hyperplane.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Types/Iso_box.h 5.1-1/include/CGAL/NewKernel_d/Types/Iso_box.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Types/Iso_box.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Types/Iso_box.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Types/Iso_box.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Types/Iso_box.h $
 // $Id: Iso_box.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Types/Line.h 5.1-1/include/CGAL/NewKernel_d/Types/Line.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Types/Line.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Types/Line.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Types/Line.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Types/Line.h $
 // $Id: Line.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Types/Ray.h 5.1-1/include/CGAL/NewKernel_d/Types/Ray.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Types/Ray.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Types/Ray.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Types/Ray.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Types/Ray.h $
 // $Id: Ray.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Types/Segment.h 5.1-1/include/CGAL/NewKernel_d/Types/Segment.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Types/Segment.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Types/Segment.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Types/Segment.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Types/Segment.h $
 // $Id: Segment.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Types/Sphere.h 5.1-1/include/CGAL/NewKernel_d/Types/Sphere.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Types/Sphere.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Types/Sphere.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Types/Sphere.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Types/Sphere.h $
 // $Id: Sphere.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Types/Weighted_point.h 5.1-1/include/CGAL/NewKernel_d/Types/Weighted_point.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Types/Weighted_point.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Types/Weighted_point.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Types/Weighted_point.h $
-// $Id: Weighted_point.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Types/Weighted_point.h $
+// $Id: Weighted_point.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Marc Glisse
@@ -169,29 +169,107 @@ template <class R_> struct Power_center
     WPoint const& wp0 = *f;
     Point const& p0 = pdw(wp0);
     int d = pd(p0);
-    FT const& n0 = sdo(p0) - pw(wp0);
-    Matrix m(d,d);
-    Vec b = typename CVec::Dimension()(d);
-    // Write the point coordinates in lines.
-    int i;
-    for(i=0; ++f!=e; ++i) {
-      WPoint const& wp=*f;
-      Point const& p=pdw(wp);
-      for(int j=0;j<d;++j) {
-        m(i,j)=2*(c(p,j)-c(p0,j));
+    int k = static_cast<int>(std::distance(f,e));
+    if (d+1 == k)
+    {
+      FT const& n0 = sdo(p0) - pw(wp0);
+      Matrix m(d,d);
+      Vec b = typename CVec::Dimension()(d);
+      // Write the point coordinates in lines.
+      int i;
+      for(i=0; ++f!=e; ++i) {
+        WPoint const& wp=*f;
+        Point const& p=pdw(wp);
+        for(int j=0;j<d;++j) {
+          m(i,j)=2*(c(p,j)-c(p0,j));
+        }
+        b[i] = sdo(p) - pw(wp) - n0;
       }
-      b[i] = sdo(p) - pw(wp) - n0;
+      CGAL_assertion (i == d);
+      Vec res = typename CVec::Dimension()(d);;
+      //std::cout << "Mat: " << m << "\n Vec: " << one << std::endl;
+      LA::solve(res, std::move(m), std::move(b));
+      //std::cout << "Sol: " << res << std::endl;
+      Point center = cp(d,LA::vector_begin(res),LA::vector_end(res));
+      FT const& r2 = pdp (wp0, center);
+      return cwp(std::move(center), r2);
+    }
+    else
+    {
+      /*
+       * Matrix P=(p1, p2, ...) (each point as a column)
+       * Matrix Q=2*t(p2-p1,p3-p1, ...) (each vector as a line)
+       * Matrix M: QP, adding a line of 1 at the top
+       * Vector B: (1, p2^2-p1^2, p3^2-p1^2, ...) plus weights
+       * Solve ML=B, the center of the sphere is PL
+       *
+       * It would likely be faster to write P then transpose, multiply,
+       * etc instead of doing it by hand.
+       */
+      // TODO: check for degenerate cases?
+
+      typedef typename R_::Max_ambient_dimension D2;
+      typedef typename R_::LA::template Rebind_dimension<Dynamic_dimension_tag,D2>::Other LAd;
+      typedef typename LAd::Square_matrix Matrix;
+      typedef typename LAd::Vector Vec;
+      typename Get_functor<R_, Scalar_product_tag>::type sp(this->kernel());
+      Matrix m(k,k);
+      Vec b(k);
+      Vec l(k);
+      int j,i=0;
+      for(Iter f2=f; f2!=e; ++f2,++i){
+        WPoint const& wp = *f2;
+        Point const& p = pdw(wp);
+        b(i) = m(i,i) = sdo(p) - pw(wp);
+        j=0;
+        for(Iter f3=f; f3!=e; ++f3,++j){
+          // FIXME: scalar product of points ???
+          m(j,i) = m(i,j) = sp(p,pdw(*f3));
+        }
+      }
+      for(i=1;i<k;++i){
+        b(i)-=b(0);
+        for(j=0;j<k;++j){
+          m(i,j)=2*(m(i,j)-m(0,j));
+        }
+      }
+      for(j=0;j<k;++j) m(0,j)=1;
+      b(0)=1;
+
+      LAd::solve(l,std::move(m),std::move(b));
+
+      typename LA::Vector center=typename LA::Construct_vector::Dimension()(d);
+      for(i=0;i<d;++i) center(i)=0;
+      j=0;
+      for(Iter f2=f;f2!=e;++f2,++j){
+        WPoint const& wp = *f2;
+        Point const& p = pdw(wp);
+        for(i=0;i<d;++i){
+          center(i)+=l(j)*c(p,i);
+        }
+      }
+
+      Point c = cp(d, LA::vector_begin(center), LA::vector_end(center));
+      FT r2 = pdp (wp0, c);
+      return cwp(std::move(c), std::move(r2));
     }
-    CGAL_assertion (i == d);
-    Vec res = typename CVec::Dimension()(d);;
-    //std::cout << "Mat: " << m << "\n Vec: " << one << std::endl;
-    LA::solve(res, std::move(m), std::move(b));
-    //std::cout << "Sol: " << res << std::endl;
-    Point center = cp(d,LA::vector_begin(res),LA::vector_end(res));
-    FT const& r2 = pdp (wp0, center);
-    return cwp(std::move(center), r2);
   }
 };
+template<class R_> struct Power_side_of_bounded_power_circumsphere : private Store_kernel<R_> {
+        CGAL_FUNCTOR_INIT_STORE(Power_side_of_bounded_power_circumsphere)
+        typedef typename Get_type<R_, Bounded_side_tag>::type result_type;
+
+        template<class Iter, class P>
+        result_type operator()(Iter f, Iter const& e, P const& p0) const {
+          // TODO: Special case when the dimension is full.
+          typename Get_functor<R_, Power_center_tag>::type pc(this->kernel());
+          typename Get_functor<R_, Power_distance_tag>::type pd(this->kernel());
+
+          // ON_UNBOUNDED_SIDE = -1
+          return enum_cast<Bounded_side>(-CGAL::sign(pd(pc(f, e), p0)));
+        }
+};
+
 }
 CGAL_KD_DEFAULT_TYPE(Weighted_point_tag,(CGAL::KerD::Weighted_point<K>),(Point_tag),());
 CGAL_KD_DEFAULT_FUNCTOR(Construct_ttag<Weighted_point_tag>,(CartesianDKernelFunctors::Construct_weighted_point<K>),(Weighted_point_tag,Point_tag),());
@@ -202,5 +280,6 @@ CGAL_KD_DEFAULT_FUNCTOR(In_flat_power_si
 CGAL_KD_DEFAULT_FUNCTOR(Power_distance_tag,(CartesianDKernelFunctors::Power_distance<K>),(Weighted_point_tag,Point_tag),(Squared_distance_tag,Point_drop_weight_tag,Point_weight_tag));
 CGAL_KD_DEFAULT_FUNCTOR(Power_distance_to_point_tag,(CartesianDKernelFunctors::Power_distance_to_point<K>),(Weighted_point_tag,Point_tag),(Squared_distance_tag,Point_drop_weight_tag,Point_weight_tag));
 CGAL_KD_DEFAULT_FUNCTOR(Power_center_tag,(CartesianDKernelFunctors::Power_center<K>),(Weighted_point_tag,Point_tag),(Compute_point_cartesian_coordinate_tag,Construct_ttag<Point_tag>,Construct_ttag<Weighted_point_tag>,Point_dimension_tag,Squared_distance_to_origin_tag,Point_drop_weight_tag,Point_weight_tag,Power_distance_to_point_tag));
+CGAL_KD_DEFAULT_FUNCTOR(Power_side_of_bounded_power_circumsphere_tag,(CartesianDKernelFunctors::Power_side_of_bounded_power_circumsphere<K>),(Weighted_point_tag),(Power_distance_tag,Power_center_tag));
 } // namespace CGAL
 #endif
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/utils.h 5.1-1/include/CGAL/NewKernel_d/utils.h
--- 5.0.3-1/include/CGAL/NewKernel_d/utils.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/utils.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/utils.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/utils.h $
 // $Id: utils.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Vector/array.h 5.1-1/include/CGAL/NewKernel_d/Vector/array.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Vector/array.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Vector/array.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Vector/array.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Vector/array.h $
 // $Id: array.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Vector/avx4.h 5.1-1/include/CGAL/NewKernel_d/Vector/avx4.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Vector/avx4.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Vector/avx4.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Vector/avx4.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Vector/avx4.h $
 // $Id: avx4.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Vector/determinant_of_iterator_to_points_from_iterator_to_vectors.h 5.1-1/include/CGAL/NewKernel_d/Vector/determinant_of_iterator_to_points_from_iterator_to_vectors.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Vector/determinant_of_iterator_to_points_from_iterator_to_vectors.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Vector/determinant_of_iterator_to_points_from_iterator_to_vectors.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Vector/determinant_of_iterator_to_points_from_iterator_to_vectors.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Vector/determinant_of_iterator_to_points_from_iterator_to_vectors.h $
 // $Id: determinant_of_iterator_to_points_from_iterator_to_vectors.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Vector/determinant_of_iterator_to_points_from_points.h 5.1-1/include/CGAL/NewKernel_d/Vector/determinant_of_iterator_to_points_from_points.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Vector/determinant_of_iterator_to_points_from_points.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Vector/determinant_of_iterator_to_points_from_points.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Vector/determinant_of_iterator_to_points_from_points.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Vector/determinant_of_iterator_to_points_from_points.h $
 // $Id: determinant_of_iterator_to_points_from_points.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Vector/determinant_of_iterator_to_vectors_from_vectors.h 5.1-1/include/CGAL/NewKernel_d/Vector/determinant_of_iterator_to_vectors_from_vectors.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Vector/determinant_of_iterator_to_vectors_from_vectors.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Vector/determinant_of_iterator_to_vectors_from_vectors.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Vector/determinant_of_iterator_to_vectors_from_vectors.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Vector/determinant_of_iterator_to_vectors_from_vectors.h $
 // $Id: determinant_of_iterator_to_vectors_from_vectors.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Vector/determinant_of_points_from_vectors.h 5.1-1/include/CGAL/NewKernel_d/Vector/determinant_of_points_from_vectors.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Vector/determinant_of_points_from_vectors.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Vector/determinant_of_points_from_vectors.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Vector/determinant_of_points_from_vectors.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Vector/determinant_of_points_from_vectors.h $
 // $Id: determinant_of_points_from_vectors.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Vector/determinant_of_vectors_small_dim.h 5.1-1/include/CGAL/NewKernel_d/Vector/determinant_of_vectors_small_dim.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Vector/determinant_of_vectors_small_dim.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Vector/determinant_of_vectors_small_dim.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Vector/determinant_of_vectors_small_dim.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Vector/determinant_of_vectors_small_dim.h $
 // $Id: determinant_of_vectors_small_dim.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Vector/determinant_of_vectors_small_dim_internal.h 5.1-1/include/CGAL/NewKernel_d/Vector/determinant_of_vectors_small_dim_internal.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Vector/determinant_of_vectors_small_dim_internal.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Vector/determinant_of_vectors_small_dim_internal.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Vector/determinant_of_vectors_small_dim_internal.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Vector/determinant_of_vectors_small_dim_internal.h $
 // $Id: determinant_of_vectors_small_dim_internal.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Vector/mix.h 5.1-1/include/CGAL/NewKernel_d/Vector/mix.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Vector/mix.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Vector/mix.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Vector/mix.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Vector/mix.h $
 // $Id: mix.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Vector/sse2.h 5.1-1/include/CGAL/NewKernel_d/Vector/sse2.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Vector/sse2.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Vector/sse2.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Vector/sse2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Vector/sse2.h $
 // $Id: sse2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Vector/v2int.h 5.1-1/include/CGAL/NewKernel_d/Vector/v2int.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Vector/v2int.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Vector/v2int.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Vector/v2int.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Vector/v2int.h $
 // $Id: v2int.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Vector/vector.h 5.1-1/include/CGAL/NewKernel_d/Vector/vector.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Vector/vector.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Vector/vector.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Vector/vector.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Vector/vector.h $
 // $Id: vector.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Wrapper/Cartesian_wrap.h 5.1-1/include/CGAL/NewKernel_d/Wrapper/Cartesian_wrap.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Wrapper/Cartesian_wrap.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Wrapper/Cartesian_wrap.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Wrapper/Cartesian_wrap.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Wrapper/Cartesian_wrap.h $
 // $Id: Cartesian_wrap.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Wrapper/Hyperplane_d.h 5.1-1/include/CGAL/NewKernel_d/Wrapper/Hyperplane_d.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Wrapper/Hyperplane_d.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Wrapper/Hyperplane_d.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Wrapper/Hyperplane_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Wrapper/Hyperplane_d.h $
 // $Id: Hyperplane_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Wrapper/Point_d.h 5.1-1/include/CGAL/NewKernel_d/Wrapper/Point_d.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Wrapper/Point_d.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Wrapper/Point_d.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Wrapper/Point_d.h $
-// $Id: Point_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Wrapper/Point_d.h $
+// $Id: Point_d.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Marc Glisse
@@ -51,12 +51,12 @@ public:
   typedef typename Get_type<Kbase, Point_tag>::type      Rep;
   //typedef typename CGAL::decay<typename boost::result_of<CPI(Rep,Begin_tag)>::type>::type Cartesian_const_iterator;
 
-  const Rep& rep() const
+  const Rep& rep() const noexcept
   {
     return *this;
   }
 
-  Rep& rep()
+  Rep& rep() noexcept
   {
     return *this;
   }
@@ -101,6 +101,14 @@ public:
   Point_d(Origin&& v)
     : Rep(CPBase()(std::move(v))) {}
 
+  friend void swap(Self& a, Self& b)
+#ifdef __cpp_lib_is_swappable
+    noexcept(std::is_nothrow_swappable_v<Rep>)
+#endif
+    {
+      using std::swap;
+      swap(a.rep(), b.rep());
+    }
 
   decltype(auto) cartesian(int i)const{
           return CCBase()(rep(),i);
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Wrapper/Ref_count_obj.h 5.1-1/include/CGAL/NewKernel_d/Wrapper/Ref_count_obj.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Wrapper/Ref_count_obj.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Wrapper/Ref_count_obj.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Wrapper/Ref_count_obj.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Wrapper/Ref_count_obj.h $
 // $Id: Ref_count_obj.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Wrapper/Segment_d.h 5.1-1/include/CGAL/NewKernel_d/Wrapper/Segment_d.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Wrapper/Segment_d.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Wrapper/Segment_d.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Wrapper/Segment_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Wrapper/Segment_d.h $
 // $Id: Segment_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Wrapper/Sphere_d.h 5.1-1/include/CGAL/NewKernel_d/Wrapper/Sphere_d.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Wrapper/Sphere_d.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Wrapper/Sphere_d.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Wrapper/Sphere_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Wrapper/Sphere_d.h $
 // $Id: Sphere_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Wrapper/Vector_d.h 5.1-1/include/CGAL/NewKernel_d/Wrapper/Vector_d.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Wrapper/Vector_d.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Wrapper/Vector_d.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Wrapper/Vector_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Wrapper/Vector_d.h $
 // $Id: Vector_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NewKernel_d/Wrapper/Weighted_point_d.h 5.1-1/include/CGAL/NewKernel_d/Wrapper/Weighted_point_d.h
--- 5.0.3-1/include/CGAL/NewKernel_d/Wrapper/Weighted_point_d.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/NewKernel_d/Wrapper/Weighted_point_d.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/NewKernel_d/Wrapper/Weighted_point_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/NewKernel_d/Wrapper/Weighted_point_d.h $
 // $Id: Weighted_point_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/No_intersection_surface_sweep_2.h 5.1-1/include/CGAL/No_intersection_surface_sweep_2.h
--- 5.0.3-1/include/CGAL/No_intersection_surface_sweep_2.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/No_intersection_surface_sweep_2.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_sweep_2/include/CGAL/No_intersection_surface_sweep_2.h $
-// $Id: No_intersection_surface_sweep_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_sweep_2/include/CGAL/No_intersection_surface_sweep_2.h $
+// $Id: No_intersection_surface_sweep_2.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s) : Baruch Zukerman <baruchzu@post.tau.ac.il>
@@ -280,7 +280,7 @@ public:
   /*! Run the sweep-line alogrithm on a range of x-monotone curves, a range
    * of action event points (if a curve passed through an action point, it will
    * be split) and a range of query points (if a curve passed through a
-   * query point,it will not be splitted).
+   * query point,it will not be split).
    * \param curves_begin An iterator for the first x-monotone curve in the
    *                     range.
    * \param curves_end A past-the-end iterator for this range.
diff -pruN 5.0.3-1/include/CGAL/normal_vector_newell_3.h 5.1-1/include/CGAL/normal_vector_newell_3.h
--- 5.0.3-1/include/CGAL/normal_vector_newell_3.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/normal_vector_newell_3.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/normal_vector_newell_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/normal_vector_newell_3.h $
 // $Id: normal_vector_newell_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/N_step_adaptor_derived.h 5.1-1/include/CGAL/N_step_adaptor_derived.h
--- 5.0.3-1/include/CGAL/N_step_adaptor_derived.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/N_step_adaptor_derived.h	2020-09-08 12:15:32.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/N_step_adaptor_derived.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/N_step_adaptor_derived.h $
 // $Id: N_step_adaptor_derived.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/N_step_adaptor.h 5.1-1/include/CGAL/N_step_adaptor.h
--- 5.0.3-1/include/CGAL/N_step_adaptor.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/N_step_adaptor.h	2020-09-08 12:15:32.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/N_step_adaptor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/N_step_adaptor.h $
 // $Id: N_step_adaptor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/NT_converter.h 5.1-1/include/CGAL/NT_converter.h
--- 5.0.3-1/include/CGAL/NT_converter.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/NT_converter.h	2020-09-08 12:15:16.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/NT_converter.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/NT_converter.h $
 // $Id: NT_converter.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Null_matrix.h 5.1-1/include/CGAL/Null_matrix.h
--- 5.0.3-1/include/CGAL/Null_matrix.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Null_matrix.h	2020-09-08 12:15:45.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_simplification/include/CGAL/Null_matrix.h $
-// $Id: Null_matrix.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_simplification/include/CGAL/Null_matrix.h $
+// $Id: Null_matrix.h ff09c5d 2019-10-25T16:35:53+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Fernando Cacciola <fernando.cacciola@geometryfactory.com>
@@ -13,19 +13,12 @@
 
 #include <CGAL/license/Surface_mesh_simplification.h>
 
-
 namespace CGAL {
 
 class Null_matrix {};
 
-static const Null_matrix NULL_MATRIX = Null_matrix() ;
-
-} //namespace CGAL
-
-#endif // CGAL_NULL_MATRIX_H //
-
-// EOF //
-
-
+static const Null_matrix NULL_MATRIX = Null_matrix();
 
+} // namespace CGAL
 
+#endif // CGAL_NULL_MATRIX_H
diff -pruN 5.0.3-1/include/CGAL/number_type_basic.h 5.1-1/include/CGAL/number_type_basic.h
--- 5.0.3-1/include/CGAL/number_type_basic.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/number_type_basic.h	2020-09-08 12:15:16.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/number_type_basic.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/number_type_basic.h $
 // $Id: number_type_basic.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Number_type_checker.h 5.1-1/include/CGAL/Number_type_checker.h
--- 5.0.3-1/include/CGAL/Number_type_checker.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Number_type_checker.h	2020-09-08 12:15:16.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Number_type_checker.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Number_type_checker.h $
 // $Id: Number_type_checker.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/number_type_config.h 5.1-1/include/CGAL/number_type_config.h
--- 5.0.3-1/include/CGAL/number_type_config.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/number_type_config.h	2020-09-08 12:15:17.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/number_type_config.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/number_type_config.h $
 // $Id: number_type_config.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Number_types/internal_functions_comparison_root_of_2.h 5.1-1/include/CGAL/Number_types/internal_functions_comparison_root_of_2.h
--- 5.0.3-1/include/CGAL/Number_types/internal_functions_comparison_root_of_2.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Number_types/internal_functions_comparison_root_of_2.h	2020-09-08 12:15:16.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Number_types/internal_functions_comparison_root_of_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Number_types/internal_functions_comparison_root_of_2.h $
 // $Id: internal_functions_comparison_root_of_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/number_utils_classes.h 5.1-1/include/CGAL/number_utils_classes.h
--- 5.0.3-1/include/CGAL/number_utils_classes.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/number_utils_classes.h	2020-09-08 12:14:30.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_foundations/include/CGAL/number_utils_classes.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_foundations/include/CGAL/number_utils_classes.h $
 // $Id: number_utils_classes.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/number_utils.h 5.1-1/include/CGAL/number_utils.h
--- 5.0.3-1/include/CGAL/number_utils.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/number_utils.h	2020-09-08 12:14:30.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_foundations/include/CGAL/number_utils.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_foundations/include/CGAL/number_utils.h $
 // $Id: number_utils.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Object.h 5.1-1/include/CGAL/Object.h
--- 5.0.3-1/include/CGAL/Object.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Object.h	2020-09-08 12:15:32.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Object.h $
-// $Id: Object.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Object.h $
+// $Id: Object.h bcf8409 2020-06-23T18:50:20+02:00 Laurent Rineau
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -132,10 +132,10 @@ class Object
 
 #ifndef CGAL_NO_DEPRECATED_CODE
     // The comparisons with nullptr are only there for Nef...
-    bool operator==(std::nullptr_t CGAL_assertion_code(n)) const
-    { CGAL_assertion(n == 0); return empty(); }
-    bool operator!=(std::nullptr_t CGAL_assertion_code(n)) const
-    { CGAL_assertion(n == 0); return !empty(); }
+  bool operator==(std::nullptr_t /*CGAL_assertion_code(n)*/) const
+  { /*CGAL_assertion(n == 0);*/ return empty(); }
+  bool operator!=(std::nullptr_t /*CGAL_assertion_code(n)*/) const
+  { /*CGAL_assertion(n == 0);*/ return !empty(); }
 #endif // CGAL_NO_DEPRECATED_CODE
 
 };
@@ -161,7 +161,7 @@ assign(T& t, const Object& o)
 struct Bad_object_cast
   : public std::bad_cast
 {
-    virtual const char * what() const throw()
+    virtual const char * what() const noexcept
     {
         return "CGAL::bad_object_cast: "
                "failed conversion using CGAL::object_cast";
diff -pruN 5.0.3-1/include/CGAL/odt_optimize_mesh_3.h 5.1-1/include/CGAL/odt_optimize_mesh_3.h
--- 5.0.3-1/include/CGAL/odt_optimize_mesh_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/odt_optimize_mesh_3.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/odt_optimize_mesh_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/odt_optimize_mesh_3.h $
 // $Id: odt_optimize_mesh_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/offset_polygon_2.h 5.1-1/include/CGAL/offset_polygon_2.h
--- 5.0.3-1/include/CGAL/offset_polygon_2.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/offset_polygon_2.h	2020-09-08 12:15:11.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Minkowski_sum_2/include/CGAL/offset_polygon_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Minkowski_sum_2/include/CGAL/offset_polygon_2.h $
 // $Id: offset_polygon_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/OFF_to_nef_3.h 5.1-1/include/CGAL/OFF_to_nef_3.h
--- 5.0.3-1/include/CGAL/OFF_to_nef_3.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/OFF_to_nef_3.h	2020-09-08 12:15:14.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_3/include/CGAL/OFF_to_nef_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_3/include/CGAL/OFF_to_nef_3.h $
 // $Id: OFF_to_nef_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/OpenGR/compute_registration_transformation.h 5.1-1/include/CGAL/OpenGR/compute_registration_transformation.h
--- 5.0.3-1/include/CGAL/OpenGR/compute_registration_transformation.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/OpenGR/compute_registration_transformation.h	2020-09-08 12:15:22.000000000 +0000
@@ -0,0 +1,370 @@
+// Copyright (c) 2019  GeometryFactory(France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/OpenGR/compute_registration_transformation.h $
+// $Id: compute_registration_transformation.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s) : Sebastien Loriot, Necip Fazil Yildiran
+
+#ifndef CGAL_OPENGR_COMPUTE_REGISTRATION_TRANSFORMATION_H
+#define CGAL_OPENGR_COMPUTE_REGISTRATION_TRANSFORMATION_H
+
+#include <CGAL/license/Point_set_processing_3.h>
+
+#if defined(CGAL_LINKED_WITH_OPENGR) || defined(DOXYGEN_RUNNING)
+
+#include <CGAL/Aff_transformation_3.h>
+#include <CGAL/assertions.h>
+#include <CGAL/boost/graph/Named_function_parameters.h>
+#include <CGAL/boost/graph/named_params_helper.h>
+#include <CGAL/Iterator_range.h>
+
+#include <boost/type_traits/is_same.hpp>
+#include <boost/range/iterator_range.hpp>
+
+#include <gr/algorithms/FunctorSuper4pcs.h>
+#include <gr/algorithms/PointPairFilter.h>
+
+#include <Eigen/Dense>
+
+namespace CGAL {
+
+namespace OpenGR {
+
+template<typename Kernel>
+using Options = typename gr::Match4pcsBase<gr::FunctorSuper4PCS,
+                                           gr::Point3D<typename Kernel::FT>,
+                                           gr::DummyTransformVisitor,
+                                           gr::AdaptivePointFilter,
+                                           gr::AdaptivePointFilter::Options>::OptionsType;
+
+namespace internal {
+
+template <typename Scalar, typename InputRange, typename PointMap, typename VectorMap>
+struct CGAL_range_and_pmaps_to_opengr_point3d_range
+{
+  typedef typename InputRange::const_iterator::value_type argument_type;
+  typedef gr::Point3D<Scalar> result_type;
+  typedef typename result_type::VectorType vector_type;
+
+  PointMap point_map;
+  VectorMap normal_map;
+
+  CGAL_range_and_pmaps_to_opengr_point3d_range (PointMap point_map, VectorMap normal_map)
+    : point_map (point_map), normal_map (normal_map)
+  { }
+
+  result_type operator() (const argument_type& arg) const
+  {
+    const auto& p = get (point_map, arg);
+    const auto& n = get (normal_map, arg);
+
+    result_type out (p.x(), p.y(), p.z());
+    out.set_normal ( vector_type(n.x(), n.y(), n.z()) );
+
+    return out;
+  }
+};
+
+template <class Kernel,
+          class PointRange1,
+          class PointRange2,
+          class PointMap1,
+          class PointMap2,
+          class VectorMap1,
+          class VectorMap2>
+std::pair<typename Kernel::Aff_transformation_3, double>
+compute_registration_transformation(const PointRange1& range1,    const PointRange2& range2,
+                                    PointMap1 point_map1,   PointMap2 point_map2,
+                                    VectorMap1 vector_map1, VectorMap2 vector_map2,
+                                    Options<Kernel>& options)
+{
+  typedef gr::Point3D<typename Kernel::FT> PointType;
+
+  namespace GR=gr;
+
+  // TODO: see if should allow user to change those types
+  typedef Eigen::Matrix<typename PointType::Scalar, 4, 4> MatrixType;
+  typedef gr::UniformDistSampler<PointType> SamplerType;
+  typedef gr::DummyTransformVisitor TrVisitorType;
+  typedef gr::Match4pcsBase<gr::FunctorSuper4PCS,
+                            PointType,
+                            TrVisitorType,
+                            gr::AdaptivePointFilter,
+                            gr::AdaptivePointFilter::Options> MatcherType;
+
+  MatrixType mat (MatrixType::Identity());
+  SamplerType sampler;
+  TrVisitorType visitor;
+
+  // Unary functions that convert value_type of point ranges to gr::Point3D
+  CGAL_range_and_pmaps_to_opengr_point3d_range<typename Kernel::FT, PointRange1, PointMap1, VectorMap1> // TODO: remove deductible ones
+    unary_function_1 (point_map1, vector_map1);
+
+  CGAL_range_and_pmaps_to_opengr_point3d_range<typename Kernel::FT, PointRange2, PointMap2, VectorMap2> // TODO: remove deductible ones
+    unary_function_2 (point_map2, vector_map2);
+
+  auto gr_point_range_1 = boost::make_iterator_range(
+    boost::make_transform_iterator (range1.begin(), unary_function_1),
+    boost::make_transform_iterator (range1.end(),   unary_function_1));
+
+  auto gr_point_range_2 = boost::make_iterator_range(
+    boost::make_transform_iterator (range2.begin(), unary_function_2),
+    boost::make_transform_iterator (range2.end(),   unary_function_2));
+
+  // logger
+  GR::Utils::Logger logger(GR::Utils::NoLog);
+
+  // matcher
+  MatcherType matcher(options, logger);
+  double score =
+    matcher.ComputeTransformation(gr_point_range_1, gr_point_range_2, mat, sampler, visitor);
+
+#ifdef CGAL_OPENGR_VERBOSE
+  std::cerr << "Transformation matrix: " << std::endl;
+  for (std::size_t i = 0; i < 4; ++ i)
+  {
+    for (std::size_t j = 0; j < 4; ++ j)
+      std::cerr << mat.coeff(i,j) << " ";
+    std::cerr << std::endl;
+  }
+#endif
+
+  typename Kernel::Aff_transformation_3 cgal_trsf(
+    mat.coeff(0,0), mat.coeff(0,1), mat.coeff(0,2), mat.coeff(0,3),
+    mat.coeff(1,0), mat.coeff(1,1), mat.coeff(1,2), mat.coeff(1,3),
+    mat.coeff(2,0), mat.coeff(2,1), mat.coeff(2,2), mat.coeff(2,3));
+
+  return std::make_pair(cgal_trsf, score);
+}
+
+} // end of namespace internal
+
+/**
+   \ingroup PkgPointSetProcessing3Algorithms
+
+   Computes the registration of `point_set_2` with respect to `point_set_1` and
+   returns the corresponding affine transformation along with the registration
+   score.
+
+   Registration is computed using the Super4PCS algorithm \cgalCite{cgal:mam-sffgp-14}.
+
+   \note This function requires the \ref thirdpartyOpenGR library.
+
+   \warning Although this may seem counter-intuitive, if one of the
+   two point set matches only a small section of the other one, it is
+   advised to _use the small point set as reference_ instead of the
+   big one. The reason is that the reference point set is used to
+   construct a base that is sought after in the other point set: if
+   the big point set is used as reference, chances are the constructed
+   base will not be present in the small point set.
+
+   \tparam PointRange1 is a model of `Range`. The value type of its iterator is
+   the key type of the named parameter `point_map` in `NamedParameters1`.
+   \tparam PointRange2 is a model of `Range`. The value type of its iterator is
+   the key type of the named parameter `point_map` in `NamedParameters2`.
+
+   \param point_set_1 input point range used as reference.
+   \param point_set_2 input point range whose registration w.r.t. `point_set_1` will be computed.
+   \param np1 an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+
+   \cgalNamedParamsBegin
+     \cgalParamNBegin{point_map}
+       \cgalParamDescription{a property map associating points to the elements of the point set `point_set_1`}
+       \cgalParamType{a model of `ReadablePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange1` and whose value type is `geom_traits::Point_3`}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{normal_map}
+       \cgalParamDescription{a property map associating normals to the elements of the point set `point_set_1`}
+       \cgalParamType{a model of `ReadablePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange1` and whose value type is `geom_traits::Vector_3`}
+       \cgalParamDefault{Normals are computed and stored internally.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{geom_traits}
+       \cgalParamDescription{an instance of a geometric traits class}
+       \cgalParamType{a model of `Kernel`}
+       \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{number_of_samples}
+       \cgalParamDescription{size of the subset of input points used to compute registration}
+       \cgalParamType{unsigned int}
+       \cgalParamDefault{`200`}
+       \cgalParamExtra{Input clouds are sub-sampled prior exploration, to ensure fast computations.
+                       Super4PCS has a linear complexity w.r.t. the number of input samples,
+                       allowing to use larger values than 4PCS. Simple geometry with large overlap
+                       can be matched with only 200 samples. However, with Super4PCS, smaller details
+                       can be used during the process by using up to thousands of points.
+                       There is no theoretical limit to this parameter, however using too large values
+                       leads to very a large congruent set, which requires more time and memory
+                       to be explored. Using a large number of samples is recommended when:
+                       geometrical details are required to perform the matching, for instance
+                       to disambiguate between several similar configurations; the clouds
+                       have a very low overlap: using a too sparse sampling can prevent
+                       to have samples in the overlapping area, causing the algorithm to fail;
+                       the clouds are very noisy, and require a dense sampling.
+                       Note that Super4PCS is a global registration algorithm, which
+                       finds a good approximate of the rigid transformation aligning too
+                       clouds. Increasing the number of samples in order to get a fine
+                       registration is not optimal: it is usually faster to use less samples, and
+                       refine the transformation using a local algorithm, like the ICP, or its
+                       variant SparseICP.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{maximum_normal_deviation}
+       \cgalParamDescription{angle threshold (in degrees) used to filter pairs of points
+                             according to their normal consistency}
+       \cgalParamType{floating scalar value}
+       \cgalParamDefault{`90.00`}
+       \cgalParamExtra{Small values decrease computation time but may also decrease the quality
+                       if pairs of points that should match have a normal deviation higher than the threshold.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{accuracy}
+       \cgalParamDescription{registration accuracy (delta in the paper)}
+       \cgalParamType{floating scalar value}
+       \cgalParamDefault{`5.00`}
+       \cgalParamExtra{Setting a small value means that the two clouds needs to be very
+                       close to be considered as well aligned. It is expressed in scene units. A
+                       simple way to understand its impact is to consider the computation of the
+                       Largest Common Pointset (LCP), the metric used to verify how much the
+                       clouds are aligned. For each transformation matrix produced by Super4PCS,
+                       we compute the LCP measure by considering a shell around the reference
+                       cloud, and count the percentage of points of the target cloud lying in the
+                       shell. The thickness of the shell is defined by the parameter
+                       delta.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{overlap}
+       \cgalParamDescription{ratio of expected overlap between the two point sets:
+                             it is ranging between `0` (no overlap) to `1` (100% overlap)}
+       \cgalParamType{floating scalar value}
+       \cgalParamDefault{`0.20`}
+       \cgalParamExtra{The overlap parameter controls the size of the basis used for
+                       registration. Usually, the larger the overlap, the faster the
+                       algorithm. When the overlap is unknown, a simple way to set this parameter
+                       is to start from 100% overlap, and decrease the value until obtaining a
+                       good result. Using too small values will slow down the algorithm, and
+                       reduce the accuracy of the result.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{maximum_running_time}
+       \cgalParamDescription{maximum number of seconds after which the algorithm terminates.}
+       \cgalParamType{floating scalar value}
+       \cgalParamDefault{`1000`}
+       \cgalParamExtra{Super4PCS explores the transformation space to align the two input clouds.
+                       Since the exploration is performed randomly, it is recommended
+                       to use a large time value to explore the whole space.}
+     \cgalParamNEnd
+   \cgalNamedParamsEnd
+
+   \param np2 an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+
+   \cgalNamedParamsBegin
+     \cgalParamNBegin{point_map}
+       \cgalParamDescription{a property map associating points to the elements of the point set `point_set_2`}
+       \cgalParamType{a model of `ReadablePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange2` and whose value type is `geom_traits::Point_3`}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{normal_map}
+       \cgalParamDescription{a property map associating normals to the elements of the poing set `point_set_2`}
+       \cgalParamType{a model of `ReadablePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange2` and whose value type is `geom_traits::Vector_3`}
+       \cgalParamDefault{Normals are computed and stored internally.}
+     \cgalParamNEnd
+   \cgalNamedParamsEnd
+
+   \return a pair containing the affine transformation that should be applied
+   to `point_set_2` to make it registered w.r.t. `point_set_1` and the
+   registration score.
+*/
+template <class PointRange1, class PointRange2,
+          class NamedParameters1, class NamedParameters2>
+#ifdef DOXYGEN_RUNNING
+std::pair<geom_traits::Aff_transformation_3, double>
+#else
+std::pair<typename CGAL::Point_set_processing_3::GetK<PointRange1, NamedParameters1>
+  ::Kernel::Aff_transformation_3, double>
+#endif
+compute_registration_transformation (const PointRange1& point_set_1, const PointRange2& point_set_2,
+                                     const NamedParameters1& np1, const NamedParameters2& np2)
+{
+  namespace PSP = CGAL::Point_set_processing_3;
+  namespace GR = gr;
+  using parameters::choose_parameter;
+  using parameters::get_parameter;
+
+  // property map types
+  typedef typename CGAL::GetPointMap<PointRange1, NamedParameters1>::type PointMap1;
+  typedef typename CGAL::GetPointMap<PointRange2, NamedParameters2>::type PointMap2;
+  CGAL_static_assertion_msg((boost::is_same< typename boost::property_traits<PointMap1>::value_type,
+                                             typename boost::property_traits<PointMap2>::value_type> ::value),
+                            "The point type of input ranges must be the same");
+
+  typedef typename PSP::GetNormalMap<PointRange1, NamedParameters1>::type NormalMap1;
+  typedef typename PSP::GetNormalMap<PointRange2, NamedParameters2>::type NormalMap2;
+  CGAL_static_assertion_msg((boost::is_same< typename boost::property_traits<NormalMap1>::value_type,
+                                             typename boost::property_traits<NormalMap2>::value_type> ::value),
+                            "The vector type of input ranges must be the same");
+
+  typedef typename PSP::GetK<PointRange1, NamedParameters1>::Kernel Kernel;
+
+  PointMap1 point_map1 = choose_parameter(get_parameter(np1, internal_np::point_map), PointMap1());
+  NormalMap1 normal_map1 = choose_parameter(get_parameter(np1, internal_np::normal_map), NormalMap1());
+  PointMap2 point_map2 = choose_parameter(get_parameter(np2, internal_np::point_map), PointMap2());
+  NormalMap2 normal_map2 = choose_parameter(get_parameter(np2, internal_np::normal_map), NormalMap2());
+
+  Options<Kernel> options;
+  options.sample_size = choose_parameter(get_parameter(np1, internal_np::number_of_samples), 200);
+  options.delta = choose_parameter(get_parameter(np1, internal_np::accuracy), 5.00);
+  options.max_time_seconds = choose_parameter(get_parameter(np1, internal_np::maximum_running_time), 1000);
+  options.max_normal_difference = choose_parameter(get_parameter(np1, internal_np::maximum_normal_deviation), 90.);
+
+  bool overlap_ok = options.configureOverlap (choose_parameter(get_parameter(np1, internal_np::overlap), 0.20));
+  CGAL_USE (overlap_ok);
+  CGAL_assertion_msg (overlap_ok, "Invalid overlap configuration.");
+
+  return internal::compute_registration_transformation<Kernel>(point_set_1, point_set_2,
+                                                               point_map1, point_map2,
+                                                               normal_map1, normal_map2,
+                                                               options);
+
+}
+
+// convenience overloads
+template <class PointRange1, class PointRange2,
+          class NamedParameters1>
+std::pair<typename CGAL::Point_set_processing_3::GetK<PointRange1, NamedParameters1>
+  ::Kernel::Aff_transformation_3, double>
+compute_registration_transformation(const PointRange1& point_set_1, PointRange2& point_set_2,
+      const NamedParameters1& np1)
+{
+  namespace params = CGAL::Point_set_processing_3::parameters;
+  return compute_registration_transformation(point_set_1, point_set_2, np1, params::all_default(point_set_1));
+}
+
+template <class PointRange1, class PointRange2>
+std::pair<typename CGAL::Point_set_processing_3::GetK<PointRange1,
+          Named_function_parameters<bool, internal_np::all_default_t> >
+  ::Kernel::Aff_transformation_3, double>
+compute_registration_transformation(const PointRange1& point_set_1, PointRange2& point_set_2)
+{
+  namespace params = CGAL::Point_set_processing_3::parameters;
+  return compute_registration_transformation(point_set_1, point_set_2,
+                                             params::all_default(point_set_1),
+                                             params::all_default(point_set_2));
+}
+
+} } // end of namespace CGAL::OpenGR
+
+#endif // CGAL_LINKED_WITH_OPENGR
+
+#endif // CGAL_OPENGR_COMPUTE_REGISTRATION_TRANSFORMATION_H
diff -pruN 5.0.3-1/include/CGAL/OpenGR/register_point_sets.h 5.1-1/include/CGAL/OpenGR/register_point_sets.h
--- 5.0.3-1/include/CGAL/OpenGR/register_point_sets.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/OpenGR/register_point_sets.h	2020-09-08 12:15:22.000000000 +0000
@@ -0,0 +1,280 @@
+// Copyright (c) 2019  GeometryFactory(France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/OpenGR/register_point_sets.h $
+// $Id: register_point_sets.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s) : Sebastien Loriot, Necip Fazil Yildiran
+
+#ifndef CGAL_OPENGR_REGISTER_POINT_SETS_H
+#define CGAL_OPENGR_REGISTER_POINT_SETS_H
+
+#include <CGAL/license/Point_set_processing_3.h>
+
+#if defined(CGAL_LINKED_WITH_OPENGR) || defined(DOXYGEN_RUNNING)
+
+#include <CGAL/Aff_transformation_3.h>
+#include <CGAL/assertions.h>
+#include <CGAL/boost/graph/Named_function_parameters.h>
+#include <CGAL/boost/graph/named_params_helper.h>
+
+#include <CGAL/OpenGR/compute_registration_transformation.h>
+
+#include <boost/type_traits/is_same.hpp>
+
+#include <Eigen/Dense>
+
+namespace CGAL {
+
+namespace OpenGR {
+
+namespace internal {
+
+template <class Kernel,
+          class PointRange1,
+          class PointRange2,
+          class PointMap1,
+          class PointMap2,
+          class VectorMap1,
+          class VectorMap2>
+double
+register_point_sets(const PointRange1& range1,    PointRange2& range2,
+                    PointMap1 point_map1,   PointMap2 point_map2,
+                    VectorMap1 vector_map1, VectorMap2 vector_map2,
+                    Options<Kernel>& options)
+{
+  std::pair<typename Kernel::Aff_transformation_3, double> res =
+    compute_registration_transformation<Kernel>(range1, range2,
+                                                point_map1, point_map2,
+                                                vector_map1, vector_map2,
+                                                options);
+
+  // update CGAL points
+  for (typename PointRange2::iterator it=range2.begin(),
+                                      end=range2.end(); it!=end; ++it)
+  {
+    put(point_map2, *it, get(point_map2, *it).transform(res.first));
+  }
+
+  return res.second;
+}
+
+} // end of namespace internal
+
+/**
+   \ingroup PkgPointSetProcessing3Algorithms
+
+   Computes the registration of `point_set_2` with respect to `point_set_1` and
+   applies it.
+
+   Registration is computed using the Super4PCS algorithm
+   \cgalCite{cgal:mam-sffgp-14}. Parameters documentation is copy-pasted from [the official documentation of OpenGR](https://storm-irit.github.io/OpenGR/a00012.html). For more details on this method, please refer to it.
+
+   \note This function requires the \ref thirdpartyOpenGR library.
+
+   \warning Although this may seem counter-intuitive, if one of the two
+   point set matches only a small section of the other one, it is
+   advised to _use the small point set as reference_ instead of the
+   big one. The reason is that the reference point set is used to
+   construct a base that is sought after in the other point set: if
+   the big point set is used as reference, chances are the constructed
+   base will not be present in the small point set.
+
+   \tparam PointRange1 is a model of `Range`. The value type of its iterator is
+   the key type of the named parameter `point_map` in `NamedParameters1`.
+   \tparam PointRange2 is a model of `Range`. The value type of its iterator is
+   the key type of the named parameter `point_map` in `NamedParameters2`.
+
+   \param point_set_1 input point range used as reference.
+   \param point_set_2 input point range whose registration w.r.t. `point_set_1` will be computed.
+   \param np1 an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+
+   \cgalNamedParamsBegin
+     \cgalParamNBegin{point_map}
+       \cgalParamDescription{a property map associating points to the elements of the point set `point_set_1`}
+       \cgalParamType{a model of `ReadablePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange1` and whose value type is `geom_traits::Point_3`}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{normal_map}
+       \cgalParamDescription{a property map associating normals to the elements of the point set `point_set_1`}
+       \cgalParamType{a model of `ReadablePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange1` and whose value type is `geom_traits::Vector_3`}
+       \cgalParamDefault{Normals are computed and stored internally.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{geom_traits}
+       \cgalParamDescription{an instance of a geometric traits class}
+       \cgalParamType{a model of `Kernel`}
+       \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{number_of_samples}
+       \cgalParamDescription{size of the subset of input points used to compute registration}
+       \cgalParamType{unsigned int}
+       \cgalParamDefault{`200`}
+       \cgalParamExtra{Input clouds are sub-sampled prior exploration, to ensure fast computations.
+                       Super4PCS has a linear complexity w.r.t. the number of input samples,
+                       allowing to use larger values than 4PCS. Simple geometry with large overlap
+                       can be matched with only 200 samples. However, with Super4PCS, smaller details
+                       can be used during the process by using up to thousands of points.
+                       There is no theoretical limit to this parameter, however using too large values
+                       leads to very a large congruent set, which requires more time and memory
+                       to be explored. Using a large number of samples is recommended when:
+                       geometrical details are required to perform the matching, for instance
+                       to disambiguate between several similar configurations; the clouds
+                       have a very low overlap: using a too sparse sampling can prevent
+                       to have samples in the overlapping area, causing the algorithm to fail;
+                       the clouds are very noisy, and require a dense sampling.
+                       Note that Super4PCS is a global registration algorithm, which
+                       finds a good approximate of the rigid transformation aligning too
+                       clouds. Increasing the number of samples in order to get a fine
+                       registration is not optimal: it is usually faster to use less samples, and
+                       refine the transformation using a local algorithm, like the ICP, or its
+                       variant SparseICP.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{maximum_normal_deviation}
+       \cgalParamDescription{angle threshold (in degrees) used to filter pairs of points
+                             according to their normal consistency}
+       \cgalParamType{floating scalar value}
+       \cgalParamDefault{`90.00`}
+       \cgalParamExtra{Small values decrease computation time but may also decrease the quality
+                       if pairs of points that should match have a normal deviation higher than the threshold.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{accuracy}
+       \cgalParamDescription{registration accuracy (delta in the paper)}
+       \cgalParamType{floating scalar value}
+       \cgalParamDefault{`5.00`}
+       \cgalParamExtra{Setting a small value means that the two clouds needs to be very
+                       close to be considered as well aligned. It is expressed in scene units. A
+                       simple way to understand its impact is to consider the computation of the
+                       Largest Common Pointset (LCP), the metric used to verify how much the
+                       clouds are aligned. For each transformation matrix produced by Super4PCS,
+                       we compute the LCP measure by considering a shell around the reference
+                       cloud, and count the percentage of points of the target cloud lying in the
+                       shell. The thickness of the shell is defined by the parameter
+                       delta.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{overlap}
+       \cgalParamDescription{ratio of expected overlap between the two point sets:
+                             it is ranging between `0` (no overlap) to `1` (100% overlap)}
+       \cgalParamType{floating scalar value}
+       \cgalParamDefault{`0.20`}
+       \cgalParamExtra{The overlap parameter controls the size of the basis used for
+                       registration. Usually, the larger the overlap, the faster the
+                       algorithm. When the overlap is unknown, a simple way to set this parameter
+                       is to start from 100% overlap, and decrease the value until obtaining a
+                       good result. Using too small values will slow down the algorithm, and
+                       reduce the accuracy of the result.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{maximum_running_time}
+       \cgalParamDescription{maximum number of seconds after which the algorithm terminates.}
+       \cgalParamType{floating scalar value}
+       \cgalParamDefault{`1000`}
+       \cgalParamExtra{Super4PCS explores the transformation space to align the two input clouds.
+                       Since the exploration is performed randomly, it is recommended
+                       to use a large time value to explore the whole space.}
+     \cgalParamNEnd
+   \cgalNamedParamsEnd
+
+   \param np2 an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+
+   \cgalNamedParamsBegin
+     \cgalParamNBegin{point_map}
+       \cgalParamDescription{a property map associating points to the elements of the point set `point_set_2`}
+       \cgalParamType{a model of `ReadablePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange2` and whose value type is `geom_traits::Point_3`}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{normal_map}
+       \cgalParamDescription{a property map associating normals to the elements of the poing set `point_set_2`}
+       \cgalParamType{a model of `ReadablePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange2` and whose value type is `geom_traits::Vector_3`}
+       \cgalParamDefault{Normals are computed and stored internally.}
+     \cgalParamNEnd
+   \cgalNamedParamsEnd
+
+   \return the registration score.
+*/
+template <class PointRange1, class PointRange2,
+          class NamedParameters1, class NamedParameters2>
+double
+register_point_sets (const PointRange1& point_set_1, PointRange2& point_set_2,
+                     const NamedParameters1& np1, const NamedParameters2& np2)
+{
+  namespace PSP = CGAL::Point_set_processing_3;
+  namespace GR = gr;
+  using parameters::choose_parameter;
+  using parameters::get_parameter;
+
+  // property map types
+  typedef typename CGAL::GetPointMap<PointRange1, NamedParameters1>::type PointMap1;
+  typedef typename CGAL::GetPointMap<PointRange2, NamedParameters2>::type PointMap2;
+  CGAL_static_assertion_msg((boost::is_same< typename boost::property_traits<PointMap1>::value_type,
+                                             typename boost::property_traits<PointMap2>::value_type> ::value),
+                            "The point type of input ranges must be the same");
+
+  typedef typename PSP::GetNormalMap<PointRange1, NamedParameters1>::type NormalMap1;
+  typedef typename PSP::GetNormalMap<PointRange2, NamedParameters2>::type NormalMap2;
+  CGAL_static_assertion_msg((boost::is_same< typename boost::property_traits<NormalMap1>::value_type,
+                                             typename boost::property_traits<NormalMap2>::value_type> ::value),
+                            "The vector type of input ranges must be the same");
+
+  typedef typename PSP::GetK<PointRange1, NamedParameters1>::Kernel Kernel;
+
+  PointMap1 point_map1 = choose_parameter(get_parameter(np1, internal_np::point_map), PointMap1());
+  NormalMap1 normal_map1 = choose_parameter(get_parameter(np1, internal_np::normal_map), NormalMap1());
+  PointMap1 point_map2 = choose_parameter(get_parameter(np2, internal_np::point_map), PointMap2());
+  NormalMap2 normal_map2 = choose_parameter(get_parameter(np2, internal_np::normal_map), NormalMap2());
+
+  Options<Kernel> options;
+  options.sample_size = choose_parameter(get_parameter(np1, internal_np::number_of_samples), 200);
+  options.delta = choose_parameter(get_parameter(np1, internal_np::accuracy), 5.00);
+  options.max_time_seconds = choose_parameter(get_parameter(np1, internal_np::maximum_running_time), 1000);
+  options.max_normal_difference = choose_parameter(get_parameter(np1, internal_np::maximum_normal_deviation), 90.);
+
+  bool overlap_ok = options.configureOverlap (choose_parameter(get_parameter(np1, internal_np::overlap), 0.20));
+  CGAL_USE (overlap_ok);
+  CGAL_assertion_msg (overlap_ok, "Invalid overlap configuration.");
+
+  return internal::register_point_sets<Kernel>(point_set_1, point_set_2,
+                                               point_map1, point_map2,
+                                               normal_map1, normal_map2,
+                                               options);
+}
+
+// convenience overloads
+template <class PointRange1, class PointRange2,
+          class NamedParameters1>
+double
+register_point_sets(const PointRange1& point_set_1, PointRange2& point_set_2,
+                    const NamedParameters1& np1)
+{
+  namespace params = CGAL::Point_set_processing_3::parameters;
+  return register_point_sets(point_set_1, point_set_2, np1, params::all_default(point_set_1));
+}
+
+template <class PointRange1, class PointRange2>
+double
+register_point_sets(const PointRange1& point_set_1, PointRange2& point_set_2)
+{
+  namespace params = CGAL::Point_set_processing_3::parameters;
+  return register_point_sets(point_set_1, point_set_2,
+                             params::all_default(point_set_1),
+                             params::all_default(point_set_2));
+}
+
+} } // end of namespace CGAL::OpenGR
+
+#endif // CGAL_LINKED_WITH_OPENGR
+
+#endif // CGAL_OPENGR_REGISTER_POINT_SETS_H
diff -pruN 5.0.3-1/include/CGAL/OpenNL/bicgstab.h 5.1-1/include/CGAL/OpenNL/bicgstab.h
--- 5.0.3-1/include/CGAL/OpenNL/bicgstab.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/OpenNL/bicgstab.h	2020-09-08 12:15:17.000000000 +0000
@@ -23,7 +23,7 @@
  *      - added comments and traces
  *      - copied BICGSTAB algorithm WITH preconditioner from Graphite 1.9 code
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/OpenNL/include/CGAL/OpenNL/bicgstab.h $
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/OpenNL/include/CGAL/OpenNL/bicgstab.h $
  * $Id: bicgstab.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
  * SPDX-License-Identifier: LGPL-3.0-or-later
  */
diff -pruN 5.0.3-1/include/CGAL/OpenNL/blas.h 5.1-1/include/CGAL/OpenNL/blas.h
--- 5.0.3-1/include/CGAL/OpenNL/blas.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/OpenNL/blas.h	2020-09-08 12:15:17.000000000 +0000
@@ -21,7 +21,7 @@
  *  Andreas Meyer 2007 changes for CGAL:
  *      - replaced assert with CGAL_assertion/CGAL_error etc.
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/OpenNL/include/CGAL/OpenNL/blas.h $
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/OpenNL/include/CGAL/OpenNL/blas.h $
  * $Id: blas.h fcd0529 2019-10-20T00:47:56+02:00 Sébastien Loriot
  * SPDX-License-Identifier: LGPL-3.0-or-later
 */
diff -pruN 5.0.3-1/include/CGAL/OpenNL/conjugate_gradient.h 5.1-1/include/CGAL/OpenNL/conjugate_gradient.h
--- 5.0.3-1/include/CGAL/OpenNL/conjugate_gradient.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/OpenNL/conjugate_gradient.h	2020-09-08 12:15:17.000000000 +0000
@@ -23,7 +23,7 @@
  *      - added comments
  *      - copied Conjugate Gradient algorithm WITH preconditioner from Graphite 1.9 code
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/OpenNL/include/CGAL/OpenNL/conjugate_gradient.h $
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/OpenNL/include/CGAL/OpenNL/conjugate_gradient.h $
  * $Id: conjugate_gradient.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
  * SPDX-License-Identifier: LGPL-3.0-or-later
  */
diff -pruN 5.0.3-1/include/CGAL/OpenNL/full_vector.h 5.1-1/include/CGAL/OpenNL/full_vector.h
--- 5.0.3-1/include/CGAL/OpenNL/full_vector.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/OpenNL/full_vector.h	2020-09-08 12:15:17.000000000 +0000
@@ -21,7 +21,7 @@
  *      - FullVector is now a model of the SparseLinearAlgebraTraits_d::Vector concept
  *      - Coefficients are initialized with zeros
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/OpenNL/include/CGAL/OpenNL/full_vector.h $
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/OpenNL/include/CGAL/OpenNL/full_vector.h $
  * $Id: full_vector.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
  * SPDX-License-Identifier: LGPL-3.0-or-later
  */
diff -pruN 5.0.3-1/include/CGAL/OpenNL/linear_solver.h 5.1-1/include/CGAL/OpenNL/linear_solver.h
--- 5.0.3-1/include/CGAL/OpenNL/linear_solver.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/OpenNL/linear_solver.h	2020-09-08 12:15:17.000000000 +0000
@@ -22,7 +22,7 @@
  *      - Added SymmetricLinearSolverTraits
  *      - copied Jacobi preconditioner from Graphite 1.9 code
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/OpenNL/include/CGAL/OpenNL/linear_solver.h $
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/OpenNL/include/CGAL/OpenNL/linear_solver.h $
  * $Id: linear_solver.h fcd0529 2019-10-20T00:47:56+02:00 Sébastien Loriot
  * SPDX-License-Identifier: LGPL-3.0-or-later
  */
diff -pruN 5.0.3-1/include/CGAL/OpenNL/preconditioner.h 5.1-1/include/CGAL/OpenNL/preconditioner.h
--- 5.0.3-1/include/CGAL/OpenNL/preconditioner.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/OpenNL/preconditioner.h	2020-09-08 12:15:17.000000000 +0000
@@ -20,7 +20,7 @@
  *      - copied Jacobi preconditioner from Graphite 1.9 code
  *      - Added OpenNL namespace
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/OpenNL/include/CGAL/OpenNL/preconditioner.h $
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/OpenNL/include/CGAL/OpenNL/preconditioner.h $
  * $Id: preconditioner.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
  * SPDX-License-Identifier: LGPL-3.0-or-later
  */
diff -pruN 5.0.3-1/include/CGAL/OpenNL/sparse_matrix.h 5.1-1/include/CGAL/OpenNL/sparse_matrix.h
--- 5.0.3-1/include/CGAL/OpenNL/sparse_matrix.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/OpenNL/sparse_matrix.h	2020-09-08 12:15:17.000000000 +0000
@@ -20,7 +20,7 @@
  *      - Added OpenNL namespace
  *      - SparseMatrix is now a model of the SparseLinearAlgebraTraits_d::Matrix concept
  *
- * $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/OpenNL/include/CGAL/OpenNL/sparse_matrix.h $
+ * $URL: https://github.com/CGAL/cgal/blob/v5.1/OpenNL/include/CGAL/OpenNL/sparse_matrix.h $
  * $Id: sparse_matrix.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
  * SPDX-License-Identifier: LGPL-3.0-or-later
 */
diff -pruN 5.0.3-1/include/CGAL/Optimal_bounding_box/internal/evolution.h 5.1-1/include/CGAL/Optimal_bounding_box/internal/evolution.h
--- 5.0.3-1/include/CGAL/Optimal_bounding_box/internal/evolution.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Optimal_bounding_box/internal/evolution.h	2020-09-08 12:15:17.000000000 +0000
@@ -0,0 +1,228 @@
+// Copyright (c) 2018-2019 GeometryFactory (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Optimal_bounding_box/include/CGAL/Optimal_bounding_box/internal/evolution.h $
+// $Id: evolution.h 6fe47ed 2020-05-06T12:10:48+02:00 Mael Rouxel-Labbé
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Mael Rouxel-Labbé
+//                 Konstantinos Katrioplas
+//
+#ifndef CGAL_OPTIMAL_BOUNDING_BOX_EVOLUTION_H
+#define CGAL_OPTIMAL_BOUNDING_BOX_EVOLUTION_H
+
+#include <CGAL/license/Optimal_bounding_box.h>
+
+#include <CGAL/Optimal_bounding_box/internal/fitness_function.h>
+#include <CGAL/Optimal_bounding_box/internal/helper.h>
+#include <CGAL/Optimal_bounding_box/internal/nelder_mead_functions.h>
+#include <CGAL/Optimal_bounding_box/internal/optimize_2.h>
+#include <CGAL/Optimal_bounding_box/internal/population.h>
+
+#include <CGAL/Random.h>
+#include <CGAL/number_utils.h>
+
+#include <algorithm>
+#include <array>
+#include <iostream>
+#include <vector>
+
+namespace CGAL {
+namespace Optimal_bounding_box {
+namespace internal {
+
+template <typename PointRange, typename Traits>
+class Evolution
+{
+public:
+  typedef typename Traits::FT                                               FT;
+  typedef typename Traits::Matrix                                           Matrix;
+
+  typedef internal::Population<Traits>                                      Population;
+  typedef typename Population::Simplex                                      Simplex;
+  typedef typename Population::Vertex                                       Vertex;
+
+  Evolution(const PointRange& points,
+            CGAL::Random& rng,
+            const Traits& traits)
+    :
+      m_best_v(nullptr),
+      m_population(traits),
+      m_rng(rng),
+      m_points(points),
+      m_traits(traits)
+  { }
+
+  void genetic_algorithm()
+  {
+    // This evolves an existing population
+    CGAL_precondition(m_population.size() != 0);
+
+    //groups 1,2 : size = floor(m/2) groups 3,4 : size = ceil(m/2).
+    const std::size_t m = m_population.size();
+    const std::size_t first_group_size = m / 2;
+    const std::size_t second_group_size = m - first_group_size;
+
+    std::vector<std::size_t> group1(first_group_size), group2(first_group_size);
+    std::vector<std::size_t> group3(second_group_size), group4(second_group_size);
+
+    int im = static_cast<int>(m);
+    std::generate(group1.begin(), group1.end(), [&]{ return m_rng.get_int(0, im); });
+    std::generate(group2.begin(), group2.end(), [&]{ return m_rng.get_int(0, im); });
+    std::generate(group3.begin(), group3.end(), [&]{ return m_rng.get_int(0, im); });
+    std::generate(group4.begin(), group4.end(), [&]{ return m_rng.get_int(0, im); });
+
+    // crossover I, pick A or B
+    const FT lweight = 0.4, uweight = 0.6;
+
+    std::vector<Simplex> new_simplices(m);
+
+    for(std::size_t i=0; i<first_group_size; ++i)
+    {
+      Simplex offspring;
+      for(int j=0; j<4; ++j)
+      {
+        const FT r{m_rng.get_double()};
+        const FT fitnessA = m_population[group1[i]][j].fitness();
+        const FT fitnessB = m_population[group2[i]][j].fitness();
+        const FT threshold = (fitnessA < fitnessB) ? uweight : lweight;
+
+        if(r < threshold)
+          offspring[j] = m_population[group1[i]][j];
+        else
+          offspring[j] = m_population[group2[i]][j];
+      }
+
+      new_simplices[i] = std::move(offspring);
+    }
+
+    // crossover II, combine information from A and B
+    for(std::size_t i=0; i<second_group_size; ++i)
+    {
+      Simplex offspring;
+      for(int j=0; j<4; ++j)
+      {
+        const FT fitnessA = m_population[group3[i]][j].fitness();
+        const FT fitnessB = m_population[group4[i]][j].fitness();
+        const FT lambda = (fitnessA < fitnessB) ? uweight : lweight;
+        const FT rambda = FT(1) - lambda; // because the 'l' in 'lambda' stands for left
+
+        const Matrix& lm = m_population[group3[i]][j].matrix();
+        const Matrix& rm = m_population[group4[i]][j].matrix();
+
+        offspring[j] = Vertex{m_traits.get_Q(lambda*lm + rambda*rm), m_points, m_traits};
+      }
+
+      new_simplices[first_group_size + i] = std::move(offspring);
+    }
+
+    m_population.simplices() = std::move(new_simplices);
+  }
+
+  // @todo re-enable random mutations as it is -on theory- useful, but don't allow them to override
+  // the best (current) candidates
+  void evolve(const std::size_t max_generations,
+              const std::size_t population_size,
+              const std::size_t nelder_mead_iterations,
+              const std::size_t max_random_mutations = 0)
+  {
+    // stopping criteria prameters
+    FT prev_fit_value = 0;
+    const FT tolerance = 1e-10;
+    int stale = 0;
+
+    m_population.initialize(population_size, m_points, m_rng);
+
+    std::size_t gen_iter = 0;
+    for(;;)
+    {
+#ifdef CGAL_OPTIMAL_BOUNDING_BOX_DEBUG_PP
+      std::cout << "- - - - generation #" << gen_iter << "\n";
+#endif
+
+      genetic_algorithm();
+
+#ifdef CGAL_OPTIMAL_BOUNDING_BOX_DEBUG_PP
+      std::cout << "population after genetic" << std::endl;
+      pop.show_population();
+      std::cout << std::endl;
+#endif
+
+      for(std::size_t s=0; s<population_size; ++s)
+        nelder_mead(m_population[s], nelder_mead_iterations, m_points, m_traits);
+
+#ifdef CGAL_OPTIMAL_BOUNDING_BOX_DEBUG_PP
+      std::cout << "population after nelder mead: " << std::endl;
+      pop.show_population();
+      std::cout << std::endl;
+#endif
+
+      m_best_v = &(m_population.get_best_vertex());
+      Matrix& best_m = m_best_v->matrix();
+
+#ifdef CGAL_OPTIMAL_BOUNDING_BOX_DEBUG_PP
+      std::cout << "new best matrix: " << std::endl << best_m << std::endl;
+      std::cout << "fitness: " << m_best_v->fitness() << std::endl;
+#endif
+
+      // optimize the current best rotation by using the exact OBB 2D algorithm
+      // along the axes of the current best OBB
+      Optimizer_along_axes<Traits> optimizer_2D;
+      optimizer_2D(best_m, m_points, m_traits);
+      m_best_v->fitness() = compute_fitness(best_m, m_points, m_traits);
+
+      // stopping criteria
+      const FT new_fit_value = m_best_v->fitness();
+      const FT difference = new_fit_value - prev_fit_value;
+
+#ifdef CGAL_OPTIMAL_BOUNDING_BOX_DEBUG_PP
+      std::cout << "post 2D optimization matrix: " << std::endl << best_m << std::endl;
+      std::cout << "new fit value: " << new_fit_value << std::endl;
+      std::cout << "difference: " << difference << std::endl;
+#endif
+
+      if(CGAL::abs(difference) < tolerance * new_fit_value)
+        ++stale;
+
+      if(stale == 5 || gen_iter++ >= max_generations)
+        break;
+
+      prev_fit_value = new_fit_value;
+
+      // random mutations, swap #random_mutations random simplices with a new, random simplex
+      if(max_random_mutations <= 0)
+        continue;
+
+      CGAL_warning(max_random_mutations <= population_size);
+      const int random_mutations = m_rng.get_int(0, static_cast<int>(max_random_mutations+1));
+      for(int i=0; i<random_mutations; ++i)
+      {
+        const int random_pos = m_rng.get_int(0, static_cast<int>(population_size));
+        m_population[random_pos] = m_population.create_simplex(m_points, m_rng);
+      }
+    }
+  }
+
+  const Vertex& get_best_vertex() const
+  {
+    CGAL_assertion(m_best_v != nullptr);
+    return *m_best_v;
+  }
+
+private:
+  Vertex* m_best_v;
+  Population m_population;
+
+  CGAL::Random& m_rng;
+  const PointRange& m_points;
+  const Traits& m_traits;
+};
+
+} // namespace internal
+} // namespace Optimal_bounding_box
+} // namespace CGAL
+
+#endif // CGAL_OPTIMAL_BOUNDING_BOX_EVOLUTION_H
diff -pruN 5.0.3-1/include/CGAL/Optimal_bounding_box/internal/fitness_function.h 5.1-1/include/CGAL/Optimal_bounding_box/internal/fitness_function.h
--- 5.0.3-1/include/CGAL/Optimal_bounding_box/internal/fitness_function.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Optimal_bounding_box/internal/fitness_function.h	2020-09-08 12:15:17.000000000 +0000
@@ -0,0 +1,127 @@
+// Copyright (c) 2018-2019 GeometryFactory (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Optimal_bounding_box/include/CGAL/Optimal_bounding_box/internal/fitness_function.h $
+// $Id: fitness_function.h e9d41d7 2020-04-21T10:03:00+02:00 Maxime Gimeno
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Mael Rouxel-Labbé
+//                 Konstantinos Katrioplas
+//
+#ifndef CGAL_OPTIMAL_BOUNDING_FITNESS_FUNCTION_H
+#define CGAL_OPTIMAL_BOUNDING_FITNESS_FUNCTION_H
+
+#include <CGAL/license/Optimal_bounding_box.h>
+
+#include <CGAL/assertions.h>
+
+#include <algorithm>
+#include <limits>
+
+namespace CGAL {
+namespace Optimal_bounding_box {
+namespace internal {
+
+template <typename Traits, typename PointRange>
+typename Traits::FT
+compute_fitness(const typename Traits::Matrix& R, // rotation matrix
+                const PointRange& points,
+                const Traits& /*traits*/)
+{
+  typedef typename Traits::FT                                   FT;
+  typedef typename Traits::Point_3                              Point;
+  typedef typename Traits::Vector                               Vector;
+
+  CGAL_assertion(R.number_of_rows() == 3 && R.number_of_columns() == 3);
+  CGAL_assertion(points.size() >= 3);
+
+  FT xmin, ymin, zmin, xmax, ymax, zmax;
+  xmin = ymin = zmin = FT{(std::numeric_limits<double>::max)()};
+  xmax = ymax = zmax = FT{std::numeric_limits<double>::lowest()};
+
+  for(const Point& pt : points)
+  {
+    Vector pv(3);
+    pv.set(0, pt.x());
+    pv.set(1, pt.y());
+    pv.set(2, pt.z());
+    pv = R * pv;
+
+    xmin = (std::min)(xmin, pv(0));
+    ymin = (std::min)(ymin, pv(1));
+    zmin = (std::min)(zmin, pv(2));
+    xmax = (std::max)(xmax, pv(0));
+    ymax = (std::max)(ymax, pv(1));
+    zmax = (std::max)(zmax, pv(2));
+  }
+
+  // volume
+  return ((xmax - xmin) * (ymax - ymin) * (zmax - zmin));
+}
+
+// In this function, we only care about the fitness if it is smaller.
+// Since it likely to not be smaller, we compute the volume and the value
+// that is interesting from time to time.
+// Since the volume only grows, we exit early if greater.
+template <typename Traits, typename PointRange>
+std::pair<typename Traits::FT, bool>
+compute_fitness_if_smaller(const typename Traits::Matrix& R, // rotation matrix
+                           const PointRange& points,
+                           const typename Traits::FT cmp_val,
+                           const Traits& /*traits*/)
+{
+  typedef typename Traits::FT                                   FT;
+  typedef typename Traits::Point_3                              Point;
+  typedef typename Traits::Vector                               Vector;
+
+  CGAL_assertion(R.number_of_rows() == 3 && R.number_of_columns() == 3);
+  CGAL_assertion(points.size() >= 3);
+
+  FT xmin, ymin, zmin, xmax, ymax, zmax;
+  xmin = ymin = zmin = FT{(std::numeric_limits<double>::max)()};
+  xmax = ymax = zmax = FT{std::numeric_limits<double>::lowest()};
+
+  // compute every 1%, with a minimum of 1000 iterations
+  const std::size_t pn = points.size();
+  const std::size_t mpn = (std::max)(pn / 10, std::size_t(1000));
+  std::size_t next_check = mpn; // to avoid a costly modulo
+
+//  std::cout << "pn, mpn, next_check: " << pn << " " << mpn << std::endl;
+
+  std::size_t i = 0;
+  for(const Point& pt : points)
+  {
+    Vector pv(3);
+    pv.set(0, pt.x());
+    pv.set(1, pt.y());
+    pv.set(2, pt.z());
+    pv = R * pv;
+
+    xmin = (std::min)(xmin, pv(0));
+    ymin = (std::min)(ymin, pv(1));
+    zmin = (std::min)(zmin, pv(2));
+    xmax = (std::max)(xmax, pv(0));
+    ymax = (std::max)(ymax, pv(1));
+    zmax = (std::max)(zmax, pv(2));
+
+    if(i++ == next_check)
+    {
+      if((xmax - xmin) * (ymax - ymin) * (zmax - zmin) > cmp_val)
+        return std::make_pair(FT(0), false);
+
+      next_check += mpn;
+    }
+  }
+
+  // volume
+  return std::make_pair((xmax - xmin) * (ymax - ymin) * (zmax - zmin), true);
+}
+
+} // namespace internal
+} // namespace Optimal_bounding_box
+} // namespace CGAL
+
+#endif // CGAL_OPTIMAL_BOUNDING_FITNESS_FUNCTION_H
diff -pruN 5.0.3-1/include/CGAL/Optimal_bounding_box/internal/helper.h 5.1-1/include/CGAL/Optimal_bounding_box/internal/helper.h
--- 5.0.3-1/include/CGAL/Optimal_bounding_box/internal/helper.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Optimal_bounding_box/internal/helper.h	2020-09-08 12:15:17.000000000 +0000
@@ -0,0 +1,39 @@
+// Copyright (c) 2018-2019 GeometryFactory (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Optimal_bounding_box/include/CGAL/Optimal_bounding_box/internal/helper.h $
+// $Id: helper.h ba99311 2019-12-12T11:18:29+01:00 Mael Rouxel-Labbé
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Konstantinos Katrioplas
+//                 Mael Rouxel-Labbé
+//
+#ifndef CGAL_OPTIMAL_BOUNDING_BOX_INTERNAL_HELPER_H
+#define CGAL_OPTIMAL_BOUNDING_BOX_INTERNAL_HELPER_H
+
+#include <CGAL/license/Optimal_bounding_box.h>
+
+namespace CGAL {
+namespace Optimal_bounding_box {
+namespace internal {
+
+template <typename Matrix>
+Matrix transpose(const Matrix& m)
+{
+  Matrix tm;
+
+  tm.set(0, 0, m(0, 0)); tm.set(0, 1, m(1, 0)); tm.set(0, 2, m(2, 0));
+  tm.set(1, 0, m(0, 1)); tm.set(1, 1, m(1, 1)); tm.set(1, 2, m(2, 1));
+  tm.set(2, 0, m(0, 2)); tm.set(2, 1, m(1, 2)); tm.set(2, 2, m(2, 2));
+
+  return tm;
+}
+
+} // namespace internal
+} // namespace Optimal_bounding_box
+} // namespace CGAL
+
+#endif // CGAL_OPTIMAL_BOUNDING_BOX_INTERNAL_HELPER_H
diff -pruN 5.0.3-1/include/CGAL/Optimal_bounding_box/internal/nelder_mead_functions.h 5.1-1/include/CGAL/Optimal_bounding_box/internal/nelder_mead_functions.h
--- 5.0.3-1/include/CGAL/Optimal_bounding_box/internal/nelder_mead_functions.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Optimal_bounding_box/internal/nelder_mead_functions.h	2020-09-08 12:15:17.000000000 +0000
@@ -0,0 +1,151 @@
+// Copyright (c) 2018-2019 GeometryFactory (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Optimal_bounding_box/include/CGAL/Optimal_bounding_box/internal/nelder_mead_functions.h $
+// $Id: nelder_mead_functions.h f0f2799 2020-03-28T12:18:16+01:00 Mael
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Mael Rouxel-Labbé
+//                 Konstantinos Katrioplas
+//
+#ifndef CGAL_OPTIMAL_BOUNDING_BOX_NEALDER_MEAD_FUNCTIONS_H
+#define CGAL_OPTIMAL_BOUNDING_BOX_NEALDER_MEAD_FUNCTIONS_H
+
+#include <CGAL/license/Optimal_bounding_box.h>
+
+#include <CGAL/Optimal_bounding_box/internal/fitness_function.h>
+#include <CGAL/Optimal_bounding_box/internal/helper.h>
+
+#include <CGAL/assertions.h>
+
+#include <algorithm>
+#include <array>
+
+namespace CGAL {
+namespace Optimal_bounding_box {
+namespace internal {
+
+template <typename Matrix>
+Matrix reflection(const Matrix& S_centroid,
+                  const Matrix& S_worst)
+{
+  return S_centroid * transpose(S_worst) * S_centroid;
+}
+
+template <typename Matrix>
+Matrix expansion(const Matrix& S_centroid,
+                 const Matrix& S_worst,
+                 const Matrix& S_reflection)
+{
+  return S_centroid * transpose(S_worst) * S_reflection;
+}
+
+template <typename Matrix, typename Traits>
+Matrix mean(const Matrix& m1,
+            const Matrix& m2,
+            const Traits& traits)
+{
+  typedef typename Traits::FT                                 FT;
+
+  const FT half = FT(1) / FT(2);
+  const Matrix reduction = half * (m1 + m2);
+
+  return traits.get_Q(reduction);
+}
+
+template <typename Matrix, typename Traits>
+const Matrix nm_centroid(const Matrix& S1,
+                         const Matrix& S2,
+                         const Matrix& S3,
+                         const Traits& traits)
+{
+  typedef typename Traits::FT                                 FT;
+
+  const FT third = FT(1) / FT(3);
+  const Matrix mean = third * (S1 + S2 + S3);
+
+  return traits.get_Q(mean);
+}
+
+// It's a 3D simplex with 4 rotation matrices as vertices
+template <typename Simplex, typename PointRange, typename Traits>
+void nelder_mead(Simplex& simplex,
+                 const std::size_t nelder_mead_iterations,
+                 const PointRange& points,
+                 const Traits& traits)
+{
+  typedef typename Simplex::value_type                        Vertex;
+  typedef typename Traits::FT                                 FT;
+  typedef typename Traits::Matrix                             Matrix;
+
+  for(std::size_t t=0; t<nelder_mead_iterations; ++t)
+  {
+    std::sort(simplex.begin(), simplex.end(),
+              [](const Vertex& vi, const Vertex& vj) -> bool
+              { return vi.fitness() < vj.fitness(); });
+
+    // centroid
+    const Matrix centroid_m = nm_centroid(simplex[0].matrix(),
+                                          simplex[1].matrix(),
+                                          simplex[2].matrix(), traits);
+
+    // find worst's vertex reflection
+    const Matrix& worst_m = simplex[3].matrix();
+    const Matrix refl_m = reflection(centroid_m, worst_m);
+    const FT second_worst_fitness = simplex[2].fitness();
+
+    // we are only interested in the reflection vertex if it's better than the second worst,
+    // so compute the fitness, but exit early if the volume grows too big.
+    const std::pair<FT, bool> refl_fb = compute_fitness_if_smaller(refl_m, points, second_worst_fitness, traits);
+
+    // if the reflected point is better than the second worst
+    if(refl_fb.second && refl_fb.first < second_worst_fitness)
+    {
+      const FT refl_f = refl_fb.first;
+
+      // if the reflected point is not better than the best
+      if(refl_f >= simplex[0].fitness())
+      {
+        // reflection
+        simplex[3] = Vertex{refl_m, refl_f};
+      }
+      else
+      {
+        // expansion
+        const Matrix expand_m = expansion(centroid_m, worst_m, refl_m);
+        const std::pair<FT, bool> expand_fb = compute_fitness_if_smaller(expand_m, points, refl_f, traits);
+        if(expand_fb.second && expand_fb.first < refl_f)
+          simplex[3] = Vertex{expand_m, expand_fb.first};
+        else
+          simplex[3] = Vertex{refl_m, refl_f};
+      }
+    }
+    else // the reflected vertex is worse
+    {
+      const Matrix mean_m = mean(centroid_m, worst_m, traits);
+      const FT worst_fitness = simplex[3].fitness();
+      const std::pair<FT, bool> mean_fb = compute_fitness_if_smaller(mean_m, points, worst_fitness, traits);
+
+      if(mean_fb.second && mean_fb.first <= worst_fitness)
+      {
+        // contraction of worst
+        simplex[3] = Vertex{mean_m, mean_fb.first};
+      }
+      else
+      {
+        // reduction: move all vertices towards the best
+        for(std::size_t i=1; i<4; ++i)
+          simplex[i] = Vertex{mean(simplex[i].matrix(), simplex[0].matrix(), traits), points, traits};
+      }
+    }
+  } // nelder mead iterations
+}
+
+} // namespace internal
+} // namespace Optimal_bounding_box
+} // namespace CGAL
+
+#endif
diff -pruN 5.0.3-1/include/CGAL/Optimal_bounding_box/internal/optimize_2.h 5.1-1/include/CGAL/Optimal_bounding_box/internal/optimize_2.h
--- 5.0.3-1/include/CGAL/Optimal_bounding_box/internal/optimize_2.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Optimal_bounding_box/internal/optimize_2.h	2020-09-08 12:15:17.000000000 +0000
@@ -0,0 +1,240 @@
+// Copyright (c) 2020 GeometryFactory (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Optimal_bounding_box/include/CGAL/Optimal_bounding_box/internal/optimize_2.h $
+// $Id: optimize_2.h 6fe47ed 2020-05-06T12:10:48+02:00 Mael Rouxel-Labbé
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Mael Rouxel-Labbé
+//
+#ifndef CGAL_OPTIMAL_BOUNDING_BOX_INTERNAL_OPTIMIZE_2_H
+#define CGAL_OPTIMAL_BOUNDING_BOX_INTERNAL_OPTIMIZE_2_H
+
+#include <CGAL/license/Optimal_bounding_box.h>
+
+#include <CGAL/ch_akl_toussaint.h>
+#include <CGAL/min_quadrilateral_2.h>
+#include <CGAL/Polygon_2.h>
+#include <CGAL/number_type_config.h>
+
+#include <iostream>
+#include <iterator>
+#include <limits>
+#include <utility>
+#include <vector>
+
+namespace CGAL {
+namespace Optimal_bounding_box {
+namespace internal {
+
+enum PROJECTION_DIRECTION
+{
+  ALONG_X = 0,
+  ALONG_Y,
+  ALONG_Z
+};
+
+// Now, we would like to do all of this with projection traits... Unfortunately, it's missing
+// a couple of functors (Has_on_negative_side_2, for example), which are necessary in
+// CGAL::Min_quadrilateral_default_traits_2. And while we know we have a generic case here,
+// it's a bit tedious to get something well-defined in the generic case: for example,
+// what should Has_on_negative_side_2 do if the Line_3 is orthogonal to the projection plane?
+//
+// So, easier to just bail out to real 2D...
+template <typename Traits, typename PointRange>
+std::pair<typename Traits::FT, typename Traits::FT>
+compute_2D_deviation(const PointRange& points,
+                     const PROJECTION_DIRECTION dir,
+                     const Traits& traits)
+{
+  typedef typename Traits::FT                                        FT;
+  typedef typename Traits::Point_2                                   Point_2;
+  typedef typename Traits::Vector_2                                  Vector_2;
+  typedef typename Traits::Point_3                                   Point_3;
+
+  std::vector<Point_2> points_2D;
+  points_2D.reserve(points.size());
+  for(const Point_3& pt : points)
+  {
+    if(dir == ALONG_X)
+      points_2D.emplace_back(pt.y(), pt.z());
+    else if(dir == ALONG_Y)
+      points_2D.emplace_back(pt.x(), pt.z());
+    else if(dir == ALONG_Z)
+      points_2D.emplace_back(pt.x(), pt.y());
+  }
+
+  std::vector<Point_2> extreme_points;
+  ch_akl_toussaint(points_2D.begin(), points_2D.end(), std::back_inserter(extreme_points), traits);
+
+  CGAL::Polygon_2<Traits> pol;
+  CGAL::Min_quadrilateral_default_traits_2<Traits> mrt;
+  CGAL::min_rectangle_2(extreme_points.begin(), extreme_points.end(), std::back_inserter(pol), mrt);
+
+  if(pol.size() == 4 || !pol.is_simple() || pol.is_clockwise_oriented())
+    return std::make_pair(0., 0.);
+
+  // Compute the angle between the angle necessary to rotate the rectangle onto the reference frame
+  auto bot_pos = pol.bottom_vertex();
+  auto next_pos = bot_pos;
+  ++next_pos;
+  if(next_pos == pol.vertices_end())
+    next_pos = pol.begin();
+
+  const Point_2& p = *bot_pos;
+  const Point_2& q = *next_pos;
+
+  const Vector_2 pq = traits.construct_vector_2_object()(p, q);
+  double n = sqrt(to_double(traits.compute_squared_length_2_object()(pq)));
+
+  if(n == 0.) // degenerate input, maybe? Let's just not do anything
+    return std::make_pair(pol.area(), 0.);
+
+  const double dot = pq.x(); // that's the scalar product of PQ with V(1, 0) (Ox)
+  double cosine = dot / n;
+
+  if(cosine > 1.)
+    cosine = 1.;
+  if(cosine < -1.)
+    cosine = -1.;
+
+  double theta = std::acos(cosine);
+  if(theta > 0.25 * CGAL_PI) // @todo is there a point to this
+    theta = 0.5 * CGAL_PI - theta;
+
+  return std::make_pair(pol.area(), FT{theta});
+}
+
+template <typename PointRange, typename Traits>
+void optimize_along_OBB_axes(typename Traits::Matrix& rot,
+                             const PointRange& points,
+                             const Traits& traits)
+{
+  typedef typename Traits::FT                                        FT;
+  typedef typename Traits::Point_3                                   Point;
+  typedef typename Traits::Matrix                                    Matrix;
+  typedef typename Traits::Vector                                    Vector;
+
+  CGAL_static_assertion((std::is_same<typename boost::range_value<PointRange>::type, Point>::value));
+
+  std::vector<Point> rotated_points;
+  rotated_points.reserve(points.size());
+
+  FT xmin, ymin, zmin, xmax, ymax, zmax;
+  xmin = ymin = zmin = FT{(std::numeric_limits<double>::max)()};
+  xmax = ymax = zmax = FT{std::numeric_limits<double>::lowest()};
+
+  for(const Point& pt : points)
+  {
+    Vector pv(3);
+    pv.set(0, pt.x());
+    pv.set(1, pt.y());
+    pv.set(2, pt.z());
+    pv = rot * pv;
+
+    rotated_points.emplace_back(pv(0), pv(1), pv(2));
+
+    xmin = (std::min)(xmin, pv(0));
+    ymin = (std::min)(ymin, pv(1));
+    zmin = (std::min)(zmin, pv(2));
+    xmax = (std::max)(xmax, pv(0));
+    ymax = (std::max)(ymax, pv(1));
+    zmax = (std::max)(zmax, pv(2));
+  }
+
+  const FT lx = xmax - xmin;
+  const FT ly = ymax - ymin;
+  const FT lz = zmax - zmin;
+
+  std::array<FT, 3> angles;
+  std::array<FT, 3> volumes;
+
+  FT area_xy;
+  std::tie(area_xy, angles[0]) = compute_2D_deviation(rotated_points, ALONG_Z, traits);
+  volumes[0] = lz * area_xy;
+
+  FT area_xz;
+  std::tie(area_xz, angles[1]) = compute_2D_deviation(rotated_points, ALONG_Y, traits);
+  volumes[1] = ly * area_xz;
+
+  FT area_yz;
+  std::tie(area_yz, angles[2]) = compute_2D_deviation(rotated_points, ALONG_X, traits);
+  volumes[2] = lx * area_yz;
+
+  auto it = std::min_element(volumes.begin(), volumes.end());
+  typename std::iterator_traits<decltype(it)>::difference_type d = std::distance(volumes.begin(), it);
+
+#ifdef CGAL_OPTIMAL_BOUNDING_BOX_DEBUG_PP
+  std::cout << "volumes: " << volumes[0] << " " << volumes[1] << " " << volumes[2] << std::endl;
+  std::cout << "angles: " << angles[0] << " " << angles[1] << " " << angles[2] << std::endl;
+  std::cout << "min at " << d << std::endl;
+#endif
+
+  if(d == 0) // Along_Z
+  {
+    const double c = std::cos(angles[0]);
+    const double s = std::sin(angles[0]);
+
+    Matrix opt;
+    opt.set(0, 0,  c); opt.set(0, 1, s); opt.set(0, 2, 0);
+    opt.set(1, 0, -s); opt.set(1, 1, c); opt.set(1, 2, 0);
+    opt.set(2, 0,  0); opt.set(2, 1, 0); opt.set(2, 2, 1);
+
+    rot = opt * rot;
+  }
+  else if(d == 1) // Along_Y
+  {
+    const double c = std::cos(angles[1]);
+    const double s = std::sin(angles[1]);
+
+    Matrix opt;
+    opt.set(0, 0, c); opt.set(0, 1, 0); opt.set(0, 2, -s);
+    opt.set(1, 0, 0); opt.set(1, 1, 1); opt.set(1, 2, 0);
+    opt.set(2, 0, s); opt.set(2, 1, 0); opt.set(2, 2, c);
+
+    rot = opt * rot;
+  }
+  else if(d == 2) // Along_X
+  {
+    const double c = std::cos(angles[2]);
+    const double s = std::sin(angles[2]);
+
+    Matrix opt;
+    opt.set(0, 0, 1); opt.set(0, 1,  0); opt.set(0, 2, 0);
+    opt.set(1, 0, 0); opt.set(1, 1,  c); opt.set(1, 2, s);
+    opt.set(2, 0, 0); opt.set(2, 1, -s); opt.set(2, 2, c);
+
+    rot = opt * rot;
+  }
+  else
+  {
+    CGAL_assertion(false);
+  }
+}
+
+// This operation makes no sense if an exact number type is used, so skip it, if so
+template <typename Traits,
+          typename IsFTExact = typename Algebraic_structure_traits<typename Traits::FT>::Is_exact>
+struct Optimizer_along_axes
+{
+  template <typename PointRange>
+  void operator()(typename Traits::Matrix& rot, const PointRange& points, const Traits& traits)
+  {
+    return optimize_along_OBB_axes(rot, points, traits);
+  }
+};
+
+template <typename Traits>
+struct Optimizer_along_axes<Traits, CGAL::Tag_true>
+{
+  template <typename PointRange>
+  void operator()(typename Traits::Matrix&, const PointRange&, const Traits&) { }
+};
+
+} // namespace internal
+} // namespace Optimal_bounding_box
+} // namespace CGAL
+
+#endif // CGAL_OPTIMAL_BOUNDING_BOX_INTERNAL_OPTIMIZE_2_H
diff -pruN 5.0.3-1/include/CGAL/Optimal_bounding_box/internal/population.h 5.1-1/include/CGAL/Optimal_bounding_box/internal/population.h
--- 5.0.3-1/include/CGAL/Optimal_bounding_box/internal/population.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Optimal_bounding_box/internal/population.h	2020-09-08 12:15:17.000000000 +0000
@@ -0,0 +1,168 @@
+// Copyright (c) 2018-2019 GeometryFactory (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Optimal_bounding_box/include/CGAL/Optimal_bounding_box/internal/population.h $
+// $Id: population.h e9d41d7 2020-04-21T10:03:00+02:00 Maxime Gimeno
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Mael Rouxel-Labbé
+//                 Konstantinos Katrioplas
+//
+#ifndef CGAL_OPTIMAL_BOUNDING_BOX_POPULATION_H
+#define CGAL_OPTIMAL_BOUNDING_BOX_POPULATION_H
+
+#include <CGAL/license/Optimal_bounding_box.h>
+
+#include <CGAL/Optimal_bounding_box/internal/fitness_function.h>
+
+#include <CGAL/assertions.h>
+#include <CGAL/Random.h>
+
+#include <utility>
+#include <vector>
+
+namespace CGAL {
+namespace Optimal_bounding_box {
+namespace internal {
+
+template<typename Traits>
+struct Vertex_with_fitness
+{
+  typedef typename Traits::FT                                 FT;
+  typedef typename Traits::Matrix                             Matrix;
+
+  Vertex_with_fitness() { CGAL_assertion_code(m_is_val_initialized = false;) }
+  Vertex_with_fitness(const Matrix m, const FT v) : m_mat(std::move(m)), m_val(v)
+  {
+    CGAL_assertion_code(m_is_val_initialized = true;)
+  }
+
+  template <typename PointRange>
+  Vertex_with_fitness(const Matrix m,
+                      const PointRange& points,
+                      const Traits& traits)
+    :
+      m_mat(std::move(m))
+  {
+    m_val = compute_fitness(m, points, traits);
+    CGAL_assertion_code(m_is_val_initialized = true;)
+  }
+
+  Matrix& matrix() { return m_mat; }
+  const Matrix& matrix() const { return m_mat; }
+  FT& fitness() { return m_val; }
+  FT fitness() const { CGAL_assertion(m_is_val_initialized); return m_val; }
+
+private:
+  Matrix m_mat;
+  FT m_val;
+  CGAL_assertion_code(bool m_is_val_initialized;)
+};
+
+template<typename Traits>
+class Population
+{
+public:
+  typedef typename Traits::FT                                 FT;
+  typedef typename Traits::Matrix                             Matrix;
+
+  typedef Vertex_with_fitness<Traits>                         Vertex;
+  typedef std::array<Vertex, 4>                               Simplex;
+  typedef std::vector<Simplex>                                Simplex_container;
+
+public:
+  Population(const Traits& traits) : m_traits(traits) { }
+
+  // Access
+  std::size_t size() const { return m_simplices.size(); }
+  Simplex& operator[](const std::size_t i) { CGAL_assertion(i < m_simplices.size()); return m_simplices[i]; }
+  const Simplex& operator[](const std::size_t i) const { CGAL_assertion(i < m_simplices.size()); return m_simplices[i]; }
+  Simplex_container& simplices() { return m_simplices; }
+
+private:
+  Matrix create_random_matrix(CGAL::Random& rng) const
+  {
+    Matrix m;
+
+    for(std::size_t i=0; i<3; ++i)
+      for(std::size_t j=0; j<3; ++j)
+        m.set(i, j, FT(rng.get_double()));
+
+    return m;
+  }
+
+public:
+  template <typename PointRange>
+  Simplex create_simplex(const PointRange& points,
+                         CGAL::Random& rng) const
+  {
+    Simplex s;
+    for(std::size_t i=0; i<4; ++i)
+      s[i] = Vertex{m_traits.get_Q(create_random_matrix(rng)), points, m_traits};
+
+    return s;
+  }
+
+  // create random population
+  template <typename PointRange>
+  void initialize(const std::size_t population_size,
+                  const PointRange& points,
+                  CGAL::Random& rng)
+  {
+    m_simplices.clear();
+    m_simplices.reserve(population_size);
+    for(std::size_t i=0; i<population_size; ++i)
+      m_simplices.emplace_back(create_simplex(points, rng));
+  }
+
+  Vertex& get_best_vertex()
+  {
+    std::size_t simplex_id = static_cast<std::size_t>(-1), vertex_id = static_cast<std::size_t>(-1);
+    FT best_fitness = FT{(std::numeric_limits<double>::max)()};
+    for(std::size_t i=0, ps=m_simplices.size(); i<ps; ++i)
+    {
+      for(std::size_t j=0; j<4; ++j)
+      {
+        const Vertex& vertex = m_simplices[i][j];
+        const FT fitness = vertex.fitness();
+        if(fitness < best_fitness)
+        {
+          simplex_id = i;
+          vertex_id = j;
+          best_fitness = fitness;
+        }
+      }
+    }
+
+    return m_simplices[simplex_id][vertex_id];
+  }
+
+  // Debug
+#ifdef CGAL_OPTIMAL_BOUNDING_BOX_DEBUG
+  void show_population() const
+  {
+    std::size_t id = 0;
+    for(const Simplex& s : m_simplices)
+    {
+      std::cout << "Simplex: " << id++ << std::endl;
+      for(const Matrix& m : s)
+        std::cout << m << "\n\n";
+      std:: cout << std:: endl;
+    }
+  }
+#endif
+
+private:
+  std::vector<Simplex> m_simplices;
+
+  const Traits& m_traits;
+};
+
+} // namespace internal
+} // namespace Optimal_bounding_box
+} // namespace CGAL
+
+#endif // CGAL_OPTIMAL_BOUNDING_BOX_POPULATION_H
diff -pruN 5.0.3-1/include/CGAL/Optimal_bounding_box/oriented_bounding_box.h 5.1-1/include/CGAL/Optimal_bounding_box/oriented_bounding_box.h
--- 5.0.3-1/include/CGAL/Optimal_bounding_box/oriented_bounding_box.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Optimal_bounding_box/oriented_bounding_box.h	2020-09-08 12:15:17.000000000 +0000
@@ -0,0 +1,464 @@
+// Copyright (c) 2018-2019 GeometryFactory (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Optimal_bounding_box/include/CGAL/Optimal_bounding_box/oriented_bounding_box.h $
+// $Id: oriented_bounding_box.h d567a0d 2020-07-16T09:52:32+02:00 Mael Rouxel-Labbé
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Mael Rouxel-Labbé
+//
+#ifndef CGAL_OPTIMAL_BOUNDING_BOX_ORIENTED_BOUNDING_BOX_H
+#define CGAL_OPTIMAL_BOUNDING_BOX_ORIENTED_BOUNDING_BOX_H
+
+#include <CGAL/license/Optimal_bounding_box.h>
+
+#include <CGAL/Optimal_bounding_box/internal/evolution.h>
+#include <CGAL/Optimal_bounding_box/internal/population.h>
+#include <CGAL/Optimal_bounding_box/Oriented_bounding_box_traits_3.h>
+
+#include <CGAL/boost/graph/Named_function_parameters.h>
+#include <CGAL/boost/graph/named_params_helper.h>
+
+#include <CGAL/assertions.h>
+#include <CGAL/boost/graph/helpers.h>
+#include <CGAL/convex_hull_3.h>
+#include <CGAL/Convex_hull_traits_3.h>
+#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
+#include <CGAL/Iso_cuboid_3.h>
+#include <CGAL/Iterator_range.h>
+#include <CGAL/Kernel_traits.h>
+#include <CGAL/Random.h>
+#include <CGAL/Simple_cartesian.h>
+
+#ifdef CGAL_OPTIMAL_BOUNDING_BOX_BENCHMARKS
+#include <CGAL/Real_timer.h>
+#endif
+
+#include <boost/iterator/transform_iterator.hpp>
+#include <boost/range/value_type.hpp>
+#include <boost/utility/enable_if.hpp>
+
+#include <array>
+#include <iostream>
+#include <iterator>
+#include <type_traits>
+#include <vector>
+
+#ifdef DOXYGEN_RUNNING
+#define CGAL_BGL_NP_TEMPLATE_PARAMETERS NamedParameters
+#define CGAL_BGL_NP_CLASS NamedParameters
+#endif
+
+namespace CGAL {
+namespace Optimal_bounding_box {
+namespace internal {
+
+template <typename PointRange, typename Traits>
+void construct_oriented_bounding_box(const PointRange& points,
+                                     const typename Traits::Aff_transformation_3& transformation,
+                                     const typename Traits::Aff_transformation_3& inverse_transformation,
+                                     std::array<typename Traits::Point_3, 8>& obb_points,
+                                     const Traits& traits)
+{
+  typedef typename Traits::FT                                        FT;
+  typedef typename Traits::Point_3                                   Point;
+
+  CGAL_precondition(!points.empty());
+
+  // Construct the bbox of the transformed point set
+  typename PointRange::const_iterator pit = std::begin(points);
+  const Point& first_pt = *pit++;
+  const Point first_rot_pt = transformation.transform(first_pt);
+  FT xmin = first_rot_pt.x(), xmax = first_rot_pt.x();
+  FT ymin = first_rot_pt.y(), ymax = first_rot_pt.y();
+  FT zmin = first_rot_pt.z(), zmax = first_rot_pt.z();
+
+  for(typename PointRange::const_iterator end=std::end(points); pit!=end; ++pit)
+  {
+    const Point rot_pt = transformation.transform(*pit);
+
+    xmin = (std::min)(rot_pt.x(), xmin);
+    ymin = (std::min)(rot_pt.y(), ymin);
+    zmin = (std::min)(rot_pt.z(), zmin);
+    xmax = (std::max)(rot_pt.x(), xmax);
+    ymax = (std::max)(rot_pt.y(), ymax);
+    zmax = (std::max)(rot_pt.z(), zmax);
+  }
+
+  typename Traits::Construct_point_3 cp = traits.construct_point_3_object();
+
+  obb_points[0] = cp(xmin, ymin, zmin);
+  obb_points[1] = cp(xmax, ymin, zmin);
+  obb_points[2] = cp(xmax, ymax, zmin);
+  obb_points[3] = cp(xmin, ymax, zmin);
+
+  obb_points[4] = cp(xmin, ymax, zmax); // see order in make_hexahedron()...
+  obb_points[5] = cp(xmin, ymin, zmax);
+  obb_points[6] = cp(xmax, ymin, zmax);
+  obb_points[7] = cp(xmax, ymax, zmax);
+
+  // Apply the inverse rotation to the rotated axis aligned bounding box
+  for(std::size_t i=0; i<8; ++i)
+  {
+    obb_points[i] = inverse_transformation.transform(obb_points[i]);
+#ifdef CGAL_OPTIMAL_BOUNDING_BOX_DEBUG
+    std::cout << "  OBB[" << i << "] = " << obb_points[i] << std::endl;
+#endif
+  }
+}
+
+template <typename PointRange, typename Traits>
+void compute_best_transformation(const PointRange& points,
+                                 typename Traits::Aff_transformation_3& transformation,
+                                 typename Traits::Aff_transformation_3& inverse_transformation,
+                                 CGAL::Random& rng,
+                                 const Traits& traits)
+{
+  typedef typename Traits::Matrix                                    Matrix;
+  typedef typename Traits::Aff_transformation_3                      Aff_transformation_3;
+
+  CGAL_assertion(points.size() >= 3);
+
+  const std::size_t max_generations = 100;
+  const std::size_t population_size = 30;
+  const std::size_t nelder_mead_iterations = 20;
+
+#ifdef CGAL_OPTIMAL_BOUNDING_BOX_BENCHMARKS
+  CGAL::Real_timer timer;
+  timer.start();
+#endif
+
+  Evolution<PointRange, Traits> search_solution(points, rng, traits);
+  search_solution.evolve(max_generations, population_size, nelder_mead_iterations);
+
+#ifdef CGAL_OPTIMAL_BOUNDING_BOX_BENCHMARKS
+  std::cout << "evolve: " << timer.time() << std::endl;
+  timer.reset();
+#endif
+
+  const Matrix& rot = search_solution.get_best_vertex().matrix();
+
+#ifdef CGAL_OPTIMAL_BOUNDING_BOX_BENCHMARKS
+  std::cout << "get best: " << timer.time() << std::endl;
+#endif
+
+  transformation = Aff_transformation_3(rot(0, 0), rot(0, 1), rot(0, 2),
+                                        rot(1, 0), rot(1, 1), rot(1, 2),
+                                        rot(2, 0), rot(2, 1), rot(2, 2));
+
+  // inverse transformation is simply the transposed since the matrix is unitary
+  inverse_transformation = Aff_transformation_3(rot(0, 0), rot(1, 0), rot(2, 0),
+                                                rot(0, 1), rot(1, 1), rot(2, 1),
+                                                rot(0, 2), rot(1, 2), rot(2, 2));
+}
+
+// Following two functions are overloads to dispatch depending on return type
+template <typename PointRange, typename K, typename Traits>
+void construct_oriented_bounding_box(const PointRange& points,
+                                     CGAL::Aff_transformation_3<K>& transformation,
+                                     CGAL::Random& rng,
+                                     const Traits& traits)
+{
+  typename Traits::Aff_transformation_3 inverse_transformation;
+  compute_best_transformation(points, transformation, inverse_transformation, rng, traits);
+}
+
+template <typename PointRange, typename Array, typename Traits>
+void construct_oriented_bounding_box(const PointRange& points,
+                                     Array& obb_points,
+                                     CGAL::Random& rng,
+                                     const Traits& traits,
+                                     typename boost::enable_if<
+                                       typename boost::has_range_iterator<Array>
+                                     >::type* = 0)
+{
+  typename Traits::Aff_transformation_3 transformation, inverse_transformation;
+  compute_best_transformation(points, transformation, inverse_transformation, rng, traits);
+
+  construct_oriented_bounding_box(points, transformation, inverse_transformation, obb_points, traits);
+}
+
+template <typename PointRange, typename PolygonMesh, typename Traits>
+void construct_oriented_bounding_box(const PointRange& points,
+                                     PolygonMesh& pm,
+                                     CGAL::Random& rng,
+                                     const Traits& traits,
+                                     typename boost::disable_if<
+                                       typename boost::has_range_iterator<PolygonMesh>
+                                     >::type* = 0)
+{
+  typename Traits::Aff_transformation_3 transformation, inverse_transformation;
+  compute_best_transformation(points, transformation, inverse_transformation, rng, traits);
+
+  std::array<typename Traits::Point_3, 8> obb_points;
+  construct_oriented_bounding_box(points, transformation, inverse_transformation, obb_points, traits);
+
+  CGAL::make_hexahedron(obb_points[0], obb_points[1], obb_points[2], obb_points[3],
+                        obb_points[4], obb_points[5], obb_points[6], obb_points[7], pm);
+}
+
+// Entry point, decide whether to compute the CH_3 or not
+template <typename PointRange, typename Output, typename Traits>
+void construct_oriented_bounding_box(const PointRange& points,
+                                     const bool use_ch,
+                                     Output& output,
+                                     CGAL::Random& rng,
+                                     const Traits& traits)
+{
+  typedef typename Traits::Point_3                                   Point;
+
+  CGAL_static_assertion((std::is_same<typename boost::range_value<PointRange>::type, Point>::value));
+
+  if(use_ch) // construct the convex hull to reduce the number of points
+  {
+    std::vector<Point> ch_points;
+
+#ifdef CGAL_OPTIMAL_BOUNDING_BOX_BENCHMARKS
+  CGAL::Real_timer timer;
+  timer.start();
+#endif
+
+    CGAL::Convex_hull_traits_3<Traits> CH_traits;
+    extreme_points_3(points, std::back_inserter(ch_points), CH_traits);
+
+#ifdef CGAL_OPTIMAL_BOUNDING_BOX_BENCHMARKS
+    std::cout << "CH time: " << timer.time() << std::endl;
+#endif
+
+#ifdef CGAL_OPTIMAL_BOUNDING_BOX_DEBUG
+    std::cout << ch_points.size() << " points on the convex hull" << std::endl;
+#endif
+
+    return construct_oriented_bounding_box(ch_points, output, rng, traits);
+  }
+  else
+  {
+    return construct_oriented_bounding_box(points, output, rng, traits);
+  }
+}
+
+} // namespace internal
+} // namespace Optimal_bounding_box
+
+/// \addtogroup PkgOptimalBoundingBox_Oriented_bounding_box
+///
+/// The function `oriented_bounding_box` computes an approximation of the <i>optimal bounding box</i>,
+/// which is defined as the rectangular box with smallest volume of all the rectangular boxes containing
+/// the input points.
+///
+/// Internally, the algorithm uses an optimization process to compute a transformation (rotation)
+/// \f$ {\mathcal R}_b\f$ such that the axis-aligned box of the rotated input point set
+/// has a volume that is as small as possible given a fixed maximal number of optimization iterations.
+///
+/// \cgalHeading{Input}
+///
+/// The input can be either a range of 3D points, or a polygon mesh.
+///
+/// \cgalHeading{Output}
+///
+/// The result of the algorithm can be retrieved as either:
+/// - the best affine transformation \f${\mathcal R}_b\f$ that the algorithm has found;
+/// - an array of eight points, representing the best oriented bounding box (\f${\mathcal B}_b\f$)
+///   that the algorithm has constructed, which is related to \f$ {\mathcal R}_b\f$ as it is
+///   the inverse transformation of the axis-aligned bounding box of the transformed point set.
+///   The order of the points in the array is the same as in the function
+///   \link PkgBGLHelperFct `CGAL::make_hexahedron()` \endlink,
+///   which can be used to construct a mesh from these points.
+/// - a model of `MutableFaceGraph`
+///
+/// Note that when returning an array of points, these points are constructed from the axis-aligned
+/// bounding box and some precision loss should therefore be expected if a kernel not providing
+/// exact constructions is used.
+///
+/// The algorithm is based on a paper by Chang, Gorissen, and Melchior \cgalCite{cgal:cgm-fobbo-11}.
+
+/// \ingroup PkgOptimalBoundingBox_Oriented_bounding_box
+///
+/// The function `oriented_bounding_box` computes an approximation of the <i>optimal bounding box</i>,
+/// which is defined as the rectangular box with smallest volume of all the rectangular boxes containing
+/// the input points.
+///
+/// See \ref PkgOptimalBoundingBox_Oriented_bounding_box for more information.
+///
+/// \tparam PointRange a model of `Range`. The value type may not be equal to the type `%Point_3` of the traits class
+///                    if a point map is provided via named parameters (see below) to access points.
+/// \tparam Output either the type `Aff_transformation_3` of the traits class,
+///                or `std::array<Point, 8>` with `Point` being equivalent to the type `%Point_3` of the traits class,
+///                or a model of `MutableFaceGraph`
+/// \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
+///
+/// \param points the input range
+/// \param out the resulting array of points or affine transformation
+/// \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+///
+/// \cgalNamedParamsBegin
+///   \cgalParamNBegin{point_map}
+///     \cgalParamDescription{a property map associating points to the elements of the point range}
+///     \cgalParamType{a model of `ReadablePropertyMap` with value type `geom_traits::Point_3`}
+///     \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>`}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{geom_traits}
+///     \cgalParamDescription{an instance of a geometric traits class}
+///     \cgalParamType{a model of `OrientedBoundingBoxTraits_3`}
+///     \cgalParamDefault{a default-constructed object of type `CGAL::Oriented_bounding_box_traits_3<K>`,
+///                       where `K` is a kernel type deduced from the point type.}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{use_convex_hull}
+///     \cgalParamDescription{Parameter used in the construction of oriented bounding box to indicate
+///                           whether the algorithm should first extract the extreme points (points
+///                           that are on the 3D convex hull) of the input data range
+///                           to accelerate the computation of the bounding box.}
+///     \cgalParamType{Boolean}
+///     \cgalParamDefault{`true`}
+///   \cgalParamNEnd
+/// \cgalNamedParamsEnd
+///
+template <typename PointRange,
+          typename Output,
+          typename CGAL_BGL_NP_TEMPLATE_PARAMETERS>
+void oriented_bounding_box(const PointRange& points,
+                           Output& out,
+                           const CGAL_BGL_NP_CLASS& np
+#ifndef DOXYGEN_RUNNING
+                           , typename boost::enable_if<
+                               typename boost::has_range_iterator<PointRange>
+                           >::type* = 0
+#endif
+                           )
+{
+  using CGAL::parameters::choose_parameter;
+  using CGAL::parameters::get_parameter;
+
+  typedef typename CGAL::GetPointMap<PointRange, CGAL_BGL_NP_CLASS>::type               PointMap;
+
+#if defined(CGAL_EIGEN3_ENABLED)
+  typedef typename boost::property_traits<PointMap>::value_type                         Point;
+  typedef typename CGAL::Kernel_traits<Point>::type                                     K;
+  typedef Oriented_bounding_box_traits_3<K>                                             Default_traits;
+#else
+  typedef void                                                                          Default_traits;
+#endif
+
+  typedef typename internal_np::Lookup_named_param_def<internal_np::geom_traits_t,
+                                                       CGAL_BGL_NP_CLASS,
+                                                       Default_traits>::type            Geom_traits;
+
+  CGAL_static_assertion_msg(!(std::is_same<Geom_traits, void>::value),
+                            "You must provide a traits class or have Eigen enabled!");
+
+  Geom_traits traits = choose_parameter<Geom_traits>(get_parameter(np, internal_np::geom_traits));
+  PointMap point_map = choose_parameter<PointMap>(get_parameter(np, internal_np::point_map));
+
+  const bool use_ch = choose_parameter(get_parameter(np, internal_np::use_convex_hull), true);
+  const unsigned int seed = choose_parameter(get_parameter(np, internal_np::random_seed), -1); // undocumented
+
+  CGAL::Random fixed_seed_rng(seed);
+  CGAL::Random& rng = (seed == unsigned(-1)) ? CGAL::get_default_random() : fixed_seed_rng;
+
+#ifdef CGAL_OPTIMAL_BOUNDING_BOX_DEBUG
+  std::cout << "Random seed: " << rng.get_seed() << std::endl;
+#endif
+
+  // @todo handle those cases (or call min_rectangle_2 with a projection)
+  if(points.size() <= 3)
+  {
+    std::cerr << "The oriented bounding box cannot (yet) be computed for a mesh with fewer than 4 vertices!\n";
+    return;
+  }
+
+  return Optimal_bounding_box::internal::construct_oriented_bounding_box(
+           CGAL::make_range(
+             boost::make_transform_iterator(points.begin(), CGAL::Property_map_to_unary_function<PointMap>(point_map)),
+             boost::make_transform_iterator(points.end(), CGAL::Property_map_to_unary_function<PointMap>(point_map))),
+           use_ch, out, rng, traits);
+}
+
+/// \ingroup PkgOptimalBoundingBox_Oriented_bounding_box
+///
+/// Extracts the vertices of the mesh as a point range and calls the overload using points as input.
+///
+/// \tparam PolygonMesh a model of `VertexListGraph`
+/// \tparam Output either the type `Aff_transformation_3` of the traits class,
+///                or `std::array<Point, 8>` with `Point` being equivalent to the type `%Point_3` of the traits class,
+///                or a model of `MutableFaceGraph`
+/// \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
+///
+/// \param pmesh the input mesh
+/// \param out the resulting array of points or affine transformation
+/// \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+///
+/// \cgalNamedParamsBegin
+///   \cgalParamNBegin{vertex_point_map}
+///     \cgalParamDescription{a property map associating points to the vertices of `pmesh`}
+///     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+///                    as key type and `%Point_3` as value type}
+///     \cgalParamDefault{`boost::get(CGAL::vertex_point, pmesh)`}
+///     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+///                     should be available for the vertices of `pmesh`.}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{geom_traits}
+///     \cgalParamDescription{an instance of a geometric traits class}
+///     \cgalParamType{a model of `OrientedBoundingBoxTraits_3`}
+///     \cgalParamDefault{a default-constructed object of type `CGAL::Oriented_bounding_box_traits_3<K>`,
+///                       where `K` is a kernel type deduced from the point type.}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{use_convex_hull}
+///     \cgalParamDescription{Parameter used in the construction of oriented bounding box to indicate
+///                           whether the algorithm should first extract the extreme points (points
+///                           that are on the 3D convex hull) of the input data range
+///                           to accelerate the computation of the bounding box.}
+///     \cgalParamType{Boolean}
+///     \cgalParamDefault{`true`}
+///   \cgalParamNEnd
+/// \cgalNamedParamsEnd
+///
+template <typename PolygonMesh,
+          typename Output,
+          typename CGAL_BGL_NP_TEMPLATE_PARAMETERS>
+void oriented_bounding_box(const PolygonMesh& pmesh,
+                           Output& out,
+                           const CGAL_BGL_NP_CLASS& np
+#ifndef DOXYGEN_RUNNING
+                           , typename boost::disable_if<
+                              typename boost::has_range_iterator<PolygonMesh>
+                           >::type* = 0
+#endif
+                           )
+{
+  namespace PMP = CGAL::Polygon_mesh_processing;
+
+  using CGAL::parameters::choose_parameter;
+  using CGAL::parameters::get_parameter;
+
+  typedef typename CGAL::GetVertexPointMap<PolygonMesh, CGAL_BGL_NP_CLASS>::const_type  VPM;
+
+  VPM vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
+                             get_const_property_map(vertex_point, pmesh));
+
+  oriented_bounding_box(vertices(pmesh), out, np.point_map(vpm));
+}
+
+/// \cond SKIP_IN_MANUAL
+
+///////////////////////////////////////////////////////////////////////////////////////////////////
+/// Convenience overloads
+/////////////////////////////////////////////////////////////////////////////////////////////////
+
+template <typename Input /*range or mesh*/, typename Output /*transformation, array, or mesh*/>
+void oriented_bounding_box(const Input& in, Output& out)
+{
+  return oriented_bounding_box(in, out, CGAL::parameters::all_default());
+}
+
+/// \endcond
+
+} // end namespace CGAL
+
+#endif // CGAL_OPTIMAL_BOUNDING_BOX_ORIENTED_BOUNDING_BOX_H
diff -pruN 5.0.3-1/include/CGAL/Optimal_bounding_box/Oriented_bounding_box_traits_3.h 5.1-1/include/CGAL/Optimal_bounding_box/Oriented_bounding_box_traits_3.h
--- 5.0.3-1/include/CGAL/Optimal_bounding_box/Oriented_bounding_box_traits_3.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Optimal_bounding_box/Oriented_bounding_box_traits_3.h	2020-09-08 12:15:17.000000000 +0000
@@ -0,0 +1,74 @@
+// Copyright (c) 2018-2019 GeometryFactory (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Optimal_bounding_box/include/CGAL/Optimal_bounding_box/Oriented_bounding_box_traits_3.h $
+// $Id: Oriented_bounding_box_traits_3.h b3968d2 2020-03-27T18:19:39+01:00 Mael Rouxel-Labbé
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Mael Rouxel-Labbé
+//
+#ifndef CGAL_OPTIMAL_BOUNDING_BOX_BOUNDING_BOX_TRAITS_H
+#define CGAL_OPTIMAL_BOUNDING_BOX_BOUNDING_BOX_TRAITS_H
+
+#include <CGAL/license/Optimal_bounding_box.h>
+
+#include <CGAL/Aff_transformation_3.h>
+
+#ifdef CGAL_EIGEN3_ENABLED
+#include <CGAL/Eigen_matrix.h>
+#include <CGAL/Eigen_vector.h>
+#include <Eigen/QR>
+#endif
+
+namespace CGAL {
+
+#if defined(CGAL_EIGEN3_ENABLED) || defined(DOXYGEN_RUNNING)
+
+/// \ingroup PkgOptimalBoundingBoxClasses
+///
+/// The class `CGAL::Oriented_bounding_box_traits_3` is a traits type
+/// to be used with the overloads of the function `CGAL::oriented_bounding_box()`.
+///
+/// \attention This class requires the \ref thirdpartyEigen "Eigen" library.
+///
+/// \tparam K must be a model of `Kernel`
+///
+/// \cgalModels `OrientedBoundingBoxTraits_3`
+///
+template <typename K>
+class Oriented_bounding_box_traits_3
+  : public K
+{
+public:
+  /// The field number type
+  typedef typename K::FT                               FT;
+
+  /// The affine transformation type
+  typedef typename CGAL::Aff_transformation_3<K>       Aff_transformation_3;
+
+  /// The matrix type
+  typedef CGAL::Eigen_matrix<FT, 3, 3>                 Matrix;
+
+  /// The matrix type
+  typedef CGAL::Eigen_vector<FT, 3>                    Vector;
+
+private:
+  typedef typename Matrix::EigenType                   EigenType;
+
+public:
+  /// Performs the QR-decomposition of the matrix `m` to a unitary matrix and an upper triagonal
+  /// and returns the unitary matrix
+  static Matrix get_Q(const Matrix& m)
+  {
+    Eigen::HouseholderQR<EigenType> qr(m.eigen_object());
+    return Matrix(EigenType(qr.householderQ()));
+  }
+};
+#endif // defined(CGAL_EIGEN3_ENABLED) || defined(DOXYGEN_RUNNING)
+
+} // namespace CGAL
+
+#endif // CGAL_OPTIMAL_BOUNDING_BOX_BOUNDING_BOX_TRAITS_H
diff -pruN 5.0.3-1/include/CGAL/optimal_bounding_box.h 5.1-1/include/CGAL/optimal_bounding_box.h
--- 5.0.3-1/include/CGAL/optimal_bounding_box.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/optimal_bounding_box.h	2020-09-08 12:15:17.000000000 +0000
@@ -0,0 +1,30 @@
+// Copyright (c) 2018-2019 GeometryFactory (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Optimal_bounding_box/include/CGAL/optimal_bounding_box.h $
+// $Id: optimal_bounding_box.h ae19de5 2020-01-10T10:11:00+01:00 Mael Rouxel-Labbé
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Konstantinos Katrioplas
+//                 Mael Rouxel-Labbé
+//
+#ifndef CGAL_OPTIMAL_BOUNDING_BOX_OBB_H
+#define CGAL_OPTIMAL_BOUNDING_BOX_OBB_H
+
+#ifndef DOXYGEN_RUNNING
+#include <CGAL/license/Optimal_bounding_box.h>
+#endif
+
+/**
+* \ingroup PkgOptimalBoundingBoxRef
+* \file CGAL/optimal_bounding_box.h
+* Convenience header file including the headers for all the classes and functions of this package.
+*/
+
+#include <CGAL/Optimal_bounding_box/Oriented_bounding_box_traits_3.h>
+#include <CGAL/Optimal_bounding_box/oriented_bounding_box.h>
+
+#endif // CGAL_OPTIMAL_BOUNDING_BOX_OBB_H
diff -pruN 5.0.3-1/include/CGAL/Optimal_transportation_reconstruction_2.h 5.1-1/include/CGAL/Optimal_transportation_reconstruction_2.h
--- 5.0.3-1/include/CGAL/Optimal_transportation_reconstruction_2.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Optimal_transportation_reconstruction_2.h	2020-09-08 12:15:17.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Optimal_transportation_reconstruction_2/include/CGAL/Optimal_transportation_reconstruction_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Optimal_transportation_reconstruction_2/include/CGAL/Optimal_transportation_reconstruction_2.h $
 // $Id: Optimal_transportation_reconstruction_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Optimisation/Access_coordinates_begin_2.h 5.1-1/include/CGAL/Optimisation/Access_coordinates_begin_2.h
--- 5.0.3-1/include/CGAL/Optimisation/Access_coordinates_begin_2.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Optimisation/Access_coordinates_begin_2.h	2020-09-08 12:15:17.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Optimisation_basic/include/CGAL/Optimisation/Access_coordinates_begin_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Optimisation_basic/include/CGAL/Optimisation/Access_coordinates_begin_2.h $
 // $Id: Access_coordinates_begin_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Optimisation/Access_coordinates_begin_3.h 5.1-1/include/CGAL/Optimisation/Access_coordinates_begin_3.h
--- 5.0.3-1/include/CGAL/Optimisation/Access_coordinates_begin_3.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Optimisation/Access_coordinates_begin_3.h	2020-09-08 12:15:17.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Optimisation_basic/include/CGAL/Optimisation/Access_coordinates_begin_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Optimisation_basic/include/CGAL/Optimisation/Access_coordinates_begin_3.h $
 // $Id: Access_coordinates_begin_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Optimisation/Access_coordinates_begin_d.h 5.1-1/include/CGAL/Optimisation/Access_coordinates_begin_d.h
--- 5.0.3-1/include/CGAL/Optimisation/Access_coordinates_begin_d.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Optimisation/Access_coordinates_begin_d.h	2020-09-08 12:15:17.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Optimisation_basic/include/CGAL/Optimisation/Access_coordinates_begin_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Optimisation_basic/include/CGAL/Optimisation/Access_coordinates_begin_d.h $
 // $Id: Access_coordinates_begin_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Optimisation/Access_dimension_2.h 5.1-1/include/CGAL/Optimisation/Access_dimension_2.h
--- 5.0.3-1/include/CGAL/Optimisation/Access_dimension_2.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Optimisation/Access_dimension_2.h	2020-09-08 12:15:17.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Optimisation_basic/include/CGAL/Optimisation/Access_dimension_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Optimisation_basic/include/CGAL/Optimisation/Access_dimension_2.h $
 // $Id: Access_dimension_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Optimisation/Access_dimension_3.h 5.1-1/include/CGAL/Optimisation/Access_dimension_3.h
--- 5.0.3-1/include/CGAL/Optimisation/Access_dimension_3.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Optimisation/Access_dimension_3.h	2020-09-08 12:15:17.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Optimisation_basic/include/CGAL/Optimisation/Access_dimension_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Optimisation_basic/include/CGAL/Optimisation/Access_dimension_3.h $
 // $Id: Access_dimension_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Optimisation/Access_dimension_d.h 5.1-1/include/CGAL/Optimisation/Access_dimension_d.h
--- 5.0.3-1/include/CGAL/Optimisation/Access_dimension_d.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Optimisation/Access_dimension_d.h	2020-09-08 12:15:17.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Optimisation_basic/include/CGAL/Optimisation/Access_dimension_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Optimisation_basic/include/CGAL/Optimisation/Access_dimension_d.h $
 // $Id: Access_dimension_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Optimisation/assertions.h 5.1-1/include/CGAL/Optimisation/assertions.h
--- 5.0.3-1/include/CGAL/Optimisation/assertions.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Optimisation/assertions.h	2020-09-08 12:15:17.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Optimisation_basic/include/CGAL/Optimisation/assertions.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Optimisation_basic/include/CGAL/Optimisation/assertions.h $
 // $Id: assertions.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Optimisation/basic.h 5.1-1/include/CGAL/Optimisation/basic.h
--- 5.0.3-1/include/CGAL/Optimisation/basic.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Optimisation/basic.h	2020-09-08 12:15:17.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Optimisation_basic/include/CGAL/Optimisation/basic.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Optimisation_basic/include/CGAL/Optimisation/basic.h $
 // $Id: basic.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Optimisation/Construct_point_2.h 5.1-1/include/CGAL/Optimisation/Construct_point_2.h
--- 5.0.3-1/include/CGAL/Optimisation/Construct_point_2.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Optimisation/Construct_point_2.h	2020-09-08 12:15:17.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Optimisation_basic/include/CGAL/Optimisation/Construct_point_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Optimisation_basic/include/CGAL/Optimisation/Construct_point_2.h $
 // $Id: Construct_point_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Optimisation/Construct_point_3.h 5.1-1/include/CGAL/Optimisation/Construct_point_3.h
--- 5.0.3-1/include/CGAL/Optimisation/Construct_point_3.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Optimisation/Construct_point_3.h	2020-09-08 12:15:17.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Optimisation_basic/include/CGAL/Optimisation/Construct_point_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Optimisation_basic/include/CGAL/Optimisation/Construct_point_3.h $
 // $Id: Construct_point_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Optimisation/Construct_point_d.h 5.1-1/include/CGAL/Optimisation/Construct_point_d.h
--- 5.0.3-1/include/CGAL/Optimisation/Construct_point_d.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Optimisation/Construct_point_d.h	2020-09-08 12:15:17.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Optimisation_basic/include/CGAL/Optimisation/Construct_point_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Optimisation_basic/include/CGAL/Optimisation/Construct_point_d.h $
 // $Id: Construct_point_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Optimisation/debug.h 5.1-1/include/CGAL/Optimisation/debug.h
--- 5.0.3-1/include/CGAL/Optimisation/debug.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Optimisation/debug.h	2020-09-08 12:15:17.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Optimisation_basic/include/CGAL/Optimisation/debug.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Optimisation_basic/include/CGAL/Optimisation/debug.h $
 // $Id: debug.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Optimisation_d_traits_2.h 5.1-1/include/CGAL/Optimisation_d_traits_2.h
--- 5.0.3-1/include/CGAL/Optimisation_d_traits_2.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Optimisation_d_traits_2.h	2020-09-08 12:15:17.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Optimisation_basic/include/CGAL/Optimisation_d_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Optimisation_basic/include/CGAL/Optimisation_d_traits_2.h $
 // $Id: Optimisation_d_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Optimisation_d_traits_3.h 5.1-1/include/CGAL/Optimisation_d_traits_3.h
--- 5.0.3-1/include/CGAL/Optimisation_d_traits_3.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Optimisation_d_traits_3.h	2020-09-08 12:15:17.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Optimisation_basic/include/CGAL/Optimisation_d_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Optimisation_basic/include/CGAL/Optimisation_d_traits_3.h $
 // $Id: Optimisation_d_traits_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Optimisation_d_traits_d.h 5.1-1/include/CGAL/Optimisation_d_traits_d.h
--- 5.0.3-1/include/CGAL/Optimisation_d_traits_d.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Optimisation_d_traits_d.h	2020-09-08 12:15:17.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Optimisation_basic/include/CGAL/Optimisation_d_traits_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Optimisation_basic/include/CGAL/Optimisation_d_traits_d.h $
 // $Id: Optimisation_d_traits_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/optimize_mesh_3.h 5.1-1/include/CGAL/optimize_mesh_3.h
--- 5.0.3-1/include/CGAL/optimize_mesh_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/optimize_mesh_3.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/optimize_mesh_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/optimize_mesh_3.h $
 // $Id: optimize_mesh_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/optimize_periodic_3_mesh_3.h 5.1-1/include/CGAL/optimize_periodic_3_mesh_3.h
--- 5.0.3-1/include/CGAL/optimize_periodic_3_mesh_3.h	2020-08-01 19:02:38.000000000 +0000
+++ 5.1-1/include/CGAL/optimize_periodic_3_mesh_3.h	2020-09-08 12:15:20.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_mesh_3/include/CGAL/optimize_periodic_3_mesh_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_mesh_3/include/CGAL/optimize_periodic_3_mesh_3.h $
 // $Id: optimize_periodic_3_mesh_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Orientation_Linf_2.h 5.1-1/include/CGAL/Orientation_Linf_2.h
--- 5.0.3-1/include/CGAL/Orientation_Linf_2.h	2020-08-01 19:02:50.000000000 +0000
+++ 5.1-1/include/CGAL/Orientation_Linf_2.h	2020-09-08 12:15:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_Linf_2/include/CGAL/Orientation_Linf_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_Linf_2/include/CGAL/Orientation_Linf_2.h $
 // $Id: Orientation_Linf_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Origin.h 5.1-1/include/CGAL/Origin.h
--- 5.0.3-1/include/CGAL/Origin.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Origin.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Origin.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Origin.h $
 // $Id: Origin.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Origin_impl.h 5.1-1/include/CGAL/Origin_impl.h
--- 5.0.3-1/include/CGAL/Origin_impl.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Origin_impl.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Origin_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Origin_impl.h $
 // $Id: Origin_impl.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Orthogonal_incremental_neighbor_search.h 5.1-1/include/CGAL/Orthogonal_incremental_neighbor_search.h
--- 5.0.3-1/include/CGAL/Orthogonal_incremental_neighbor_search.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Orthogonal_incremental_neighbor_search.h	2020-09-08 12:15:39.000000000 +0000
@@ -3,12 +3,13 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_searching/include/CGAL/Orthogonal_incremental_neighbor_search.h $
-// $Id: Orthogonal_incremental_neighbor_search.h ec1dd74 2020-06-09T18:27:39+02:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_searching/include/CGAL/Orthogonal_incremental_neighbor_search.h $
+// $Id: Orthogonal_incremental_neighbor_search.h e6536aa 2020-06-09T20:15:16+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
 // Author(s)     : Hans Tangelder (<hanst@cs.uu.nl>)
+//                 Clement Jamin (clement.jamin.pro@gmail.com)
 
 #ifndef CGAL_ORTHOGONAL_INCREMENTAL_NEIGHBOR_SEARCH
 #define CGAL_ORTHOGONAL_INCREMENTAL_NEIGHBOR_SEARCH
@@ -16,13 +17,16 @@
 #include <CGAL/license/Spatial_searching.h>
 
 #include <CGAL/disable_warnings.h>
+#include <CGAL/Kd_tree.h>
+#include <CGAL/Euclidean_distance.h>
+#include <CGAL/tuple.h>
+#include <CGAL/internal/Search_helpers.h>
 
 #include <cstring>
 #include <list>
 #include <queue>
 #include <memory>
-#include <CGAL/Kd_tree.h>
-#include <CGAL/Euclidean_distance.h>
+#include <iterator> // for std::distance
 #include <tuple> // std::get for tuple
 
 namespace CGAL {
@@ -30,7 +34,7 @@ namespace CGAL {
   template <class SearchTraits,
             class Distance_= typename internal::Spatial_searching_default_distance<SearchTraits>::type,
             class Splitter_ = Sliding_midpoint<SearchTraits>,
-            class Tree_= Kd_tree<SearchTraits, Splitter_, Tag_true> >
+            class Tree_= Kd_tree<SearchTraits, Splitter_, Tag_true, Tag_false> >
   class Orthogonal_incremental_neighbor_search {
 
   public:
@@ -72,7 +76,8 @@ namespace CGAL {
 
       Distance_vector dists;
 
-      Distance Orthogonal_distance_instance;
+      Distance orthogonal_distance_instance;
+      internal::Distance_helper<Distance, SearchTraits> m_distance_helper;
 
       FT multiplication_factor;
 
@@ -84,6 +89,8 @@ namespace CGAL {
 
       FT rd;
 
+      int m_dim;
+      Tree const& m_tree;
 
       class Priority_higher {
       public:
@@ -139,27 +146,30 @@ namespace CGAL {
                               FT Eps=FT(0.0), bool search_nearest=true)
         : traits(tree.traits()),number_of_neighbours_computed(0), number_of_internal_nodes_visited(0),
         number_of_leaf_nodes_visited(0), number_of_items_visited(0),
-        Orthogonal_distance_instance(tr), multiplication_factor(Orthogonal_distance_instance.transformed_distance(FT(1.0)+Eps)),
+        orthogonal_distance_instance(tr),
+        m_distance_helper(orthogonal_distance_instance, traits),
+        multiplication_factor(orthogonal_distance_instance.transformed_distance(FT(1.0)+Eps)),
         query_point(q), search_nearest_neighbour(search_nearest),
+        m_tree(tree),
         PriorityQueue(Priority_higher(search_nearest)), Item_PriorityQueue(Distance_smaller(search_nearest)),
         reference_count(1)
 
 
       {
-        if (tree.empty()) return;
+        if (m_tree.empty()) return;
 
         typename SearchTraits::Construct_cartesian_const_iterator_d ccci=traits.construct_cartesian_const_iterator_d_object();
-        int dim = static_cast<int>(std::distance(ccci(q), ccci(q,0)));
+        m_dim = static_cast<int>(std::distance(ccci(q), ccci(q,0)));
 
-        dists.resize(dim);
-        for(int i=0 ; i<dim ; ++i){
+        dists.resize(m_dim);
+        for(int i=0 ; i<m_dim ; ++i){
           dists[i] = 0;
         }
 
         if (search_nearest){
           distance_to_root=
-            Orthogonal_distance_instance.min_distance_to_rectangle(q, tree.bounding_box(),dists);
-          Node_with_distance *The_Root = new Node_with_distance(tree.root(),
+            orthogonal_distance_instance.min_distance_to_rectangle(q, m_tree.bounding_box(),dists);
+          Node_with_distance *The_Root = new Node_with_distance(m_tree.root(),
                                                                 distance_to_root, dists);
           PriorityQueue.push(The_Root);
 
@@ -169,9 +179,9 @@ namespace CGAL {
         }
          else{
            distance_to_root=
-            Orthogonal_distance_instance.max_distance_to_rectangle(q,
-                                                tree.bounding_box(), dists);
-           Node_with_distance *The_Root = new Node_with_distance(tree.root(),
+            orthogonal_distance_instance.max_distance_to_rectangle(q,
+                                                m_tree.bounding_box(), dists);
+           Node_with_distance *The_Root = new Node_with_distance(m_tree.root(),
                                                                  distance_to_root, dists);
         PriorityQueue.push(The_Root);
 
@@ -253,6 +263,83 @@ namespace CGAL {
         delete The_item_top;
       }
 
+
+
+      // With cache
+      bool search_in_leaf(typename Tree::Leaf_node_const_handle node, Tag_true, bool search_furthest)
+      {
+        typename Tree::iterator it_node_point = node->begin(), it_node_point_end = node->end();
+        typename std::vector<FT>::const_iterator cache_point_begin = m_tree.cache_begin() + m_dim*(it_node_point - m_tree.begin());
+
+        for (; it_node_point != it_node_point_end; ++it_node_point)
+        {
+          number_of_items_visited++;
+          FT distance_to_query_point =
+            m_distance_helper.transformed_distance_from_coordinates(
+              query_point, *it_node_point, cache_point_begin, cache_point_begin + m_dim);
+
+          Point_with_transformed_distance *NN_Candidate =
+            new Point_with_transformed_distance(*it_node_point, distance_to_query_point);
+          Item_PriorityQueue.push(NN_Candidate);
+
+          cache_point_begin += m_dim;
+        }
+        // old top of PriorityQueue has been processed,
+        // hence update rd
+
+        bool next_neighbour_found;
+        if (!(PriorityQueue.empty()))
+        {
+          rd = std::get<1>(*PriorityQueue.top());
+          next_neighbour_found = (search_furthest ?
+            multiplication_factor*rd < Item_PriorityQueue.top()->second
+            : multiplication_factor*rd > Item_PriorityQueue.top()->second);
+        }
+        else // priority queue empty => last neighbour found
+        {
+          next_neighbour_found = true;
+        }
+
+        number_of_neighbours_computed++;
+        return next_neighbour_found;
+      }
+
+      // Without cache
+      bool search_in_leaf(typename Tree::Leaf_node_const_handle node, Tag_false, bool search_furthest)
+      {
+        typename Tree::iterator it_node_point = node->begin(), it_node_point_end = node->end();
+
+        for (; it_node_point != it_node_point_end; ++it_node_point)
+        {
+          number_of_items_visited++;
+          FT distance_to_query_point=
+            orthogonal_distance_instance.transformed_distance(query_point, *it_node_point);
+
+          Point_with_transformed_distance *NN_Candidate =
+            new Point_with_transformed_distance(*it_node_point, distance_to_query_point);
+          Item_PriorityQueue.push(NN_Candidate);
+        }
+        // old top of PriorityQueue has been processed,
+        // hence update rd
+
+        bool next_neighbour_found;
+        if (!(PriorityQueue.empty()))
+        {
+          rd = std::get<1>(*PriorityQueue.top());
+          next_neighbour_found = (search_furthest ?
+            multiplication_factor*rd < Item_PriorityQueue.top()->second
+            : multiplication_factor*rd > Item_PriorityQueue.top()->second);
+        }
+        else // priority queue empty => last neighbour found
+        {
+          next_neighbour_found=true;
+        }
+
+        number_of_neighbours_computed++;
+        return next_neighbour_found;
+      }
+
+
       void
       Compute_the_next_nearest_neighbour()
       {
@@ -283,7 +370,7 @@ namespace CGAL {
             FT diff2 = val - node->lower_high_value();
             if (diff1 + diff2 < FT(0.0)) {
               new_rd=
-                Orthogonal_distance_instance.new_distance(copy_rd,dst,diff1,new_cut_dim);
+                orthogonal_distance_instance.new_distance(copy_rd,dst,diff1,new_cut_dim);
 
               CGAL_assertion(new_rd >= copy_rd);
                 dists[new_cut_dim] = diff1;
@@ -295,7 +382,7 @@ namespace CGAL {
 
             }
             else { // compute new distance
-              new_rd=Orthogonal_distance_instance.new_distance(copy_rd,dst,diff2,new_cut_dim);
+              new_rd=orthogonal_distance_instance.new_distance(copy_rd,dst,diff2,new_cut_dim);
               CGAL_assertion(new_rd >= copy_rd);
                 dists[new_cut_dim] = diff2;
                 Node_with_distance *Lower_Child =
@@ -309,31 +396,10 @@ namespace CGAL {
           typename Tree::Leaf_node_const_handle node =
             static_cast<typename Tree::Leaf_node_const_handle>(N);
           number_of_leaf_nodes_visited++;
-          if (node->size() > 0) {
-            for (typename Tree::iterator it=node->begin(); it != node->end(); it++) {
-              number_of_items_visited++;
-              FT distance_to_query_point=
-                Orthogonal_distance_instance.transformed_distance(query_point,*it);
-              Point_with_transformed_distance *NN_Candidate=
-                new Point_with_transformed_distance(*it,distance_to_query_point);
-              Item_PriorityQueue.push(NN_Candidate);
-            }
-            // old top of PriorityQueue has been processed,
-            // hence update rd
-
-            if (!(PriorityQueue.empty()))  {
-              rd = std::get<1>(*PriorityQueue.top());
-                next_neighbour_found =
-                  (multiplication_factor*rd >
-                   Item_PriorityQueue.top()->second);
-            }
-            else // priority queue empty => last neighbour found
-              {
-                next_neighbour_found=true;
-              }
-
-            number_of_neighbours_computed++;
-          }
+    if (node->size() > 0) {
+      typename internal::Has_points_cache<Tree, internal::has_Enable_points_cache<Tree>::type::value>::type dummy;
+      next_neighbour_found = search_in_leaf(node, dummy, false);
+    }
         }   // next_neighbour_found or priority queue is empty
         // in the latter case also the item priority quee is empty
       }
@@ -370,7 +436,7 @@ namespace CGAL {
             if (diff1 + diff2 < FT(0.0)) {
               diff1 = val - node->upper_high_value();
               new_rd=
-                Orthogonal_distance_instance.new_distance(copy_rd,dst,diff1,new_cut_dim);
+                orthogonal_distance_instance.new_distance(copy_rd,dst,diff1,new_cut_dim);
                 Node_with_distance *Lower_Child =
                   new Node_with_distance(node->lower(), copy_rd, dists);
                 PriorityQueue.push(Lower_Child);
@@ -381,7 +447,7 @@ namespace CGAL {
             }
             else { // compute new distance
               diff2 = val - node->lower_low_value();
-              new_rd=Orthogonal_distance_instance.new_distance(copy_rd,dst,diff2,new_cut_dim);
+              new_rd=orthogonal_distance_instance.new_distance(copy_rd,dst,diff2,new_cut_dim);
                 Node_with_distance *Upper_Child =
                   new Node_with_distance(node->upper(), copy_rd, dists);
                 PriorityQueue.push(Upper_Child);
@@ -395,29 +461,8 @@ namespace CGAL {
             static_cast<typename Tree::Leaf_node_const_handle>(N);
           number_of_leaf_nodes_visited++;
           if (node->size() > 0) {
-            for (typename Tree::iterator it=node->begin(); it != node->end(); it++) {
-              number_of_items_visited++;
-              FT distance_to_query_point=
-                Orthogonal_distance_instance.transformed_distance(query_point,*it);
-              Point_with_transformed_distance *NN_Candidate=
-                new Point_with_transformed_distance(*it,distance_to_query_point);
-              Item_PriorityQueue.push(NN_Candidate);
-            }
-            // old top of PriorityQueue has been processed,
-            // hence update rd
-
-            if (!(PriorityQueue.empty()))  {
-              rd = std::get<1>(*PriorityQueue.top());
-                next_neighbour_found =
-                  (multiplication_factor*rd <
-                   Item_PriorityQueue.top()->second);
-            }
-            else // priority queue empty => last neighbour found
-              {
-                next_neighbour_found=true;
-              }
-
-            number_of_neighbours_computed++;
+            typename internal::Has_points_cache<Tree, internal::has_Enable_points_cache<Tree>::type::value>::type dummy;
+            next_neighbour_found = search_in_leaf(node, dummy, true);
           }
         }   // next_neighbour_found or priority queue is empty
         // in the latter case also the item priority quee is empty
diff -pruN 5.0.3-1/include/CGAL/Orthogonal_k_neighbor_search.h 5.1-1/include/CGAL/Orthogonal_k_neighbor_search.h
--- 5.0.3-1/include/CGAL/Orthogonal_k_neighbor_search.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Orthogonal_k_neighbor_search.h	2020-09-08 12:15:39.000000000 +0000
@@ -3,12 +3,14 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_searching/include/CGAL/Orthogonal_k_neighbor_search.h $
-// $Id: Orthogonal_k_neighbor_search.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_searching/include/CGAL/Orthogonal_k_neighbor_search.h $
+// $Id: Orthogonal_k_neighbor_search.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
-// Author(s)     : Gael Guennebaud (gael.guennebaud@inria.fr), Hans Tangelder (<hanst@cs.uu.nl>)
+// Author(s)     : Gael Guennebaud (gael.guennebaud@inria.fr),
+//                 Hans Tangelder (<hanst@cs.uu.nl>),
+//                 Clement Jamin (clement.jamin.pro@gmail.com)
 
 #ifndef CGAL_ORTHOGONAL_K_NEIGHBOR_SEARCH_H
 #define CGAL_ORTHOGONAL_K_NEIGHBOR_SEARCH_H
@@ -18,36 +20,49 @@
 #include <CGAL/disable_warnings.h>
 
 #include <CGAL/internal/K_neighbor_search.h>
+#include <CGAL/internal/Search_helpers.h>
+
+#include <iterator> // for std::distance
 
 namespace CGAL {
 
 template <class SearchTraits,
           class Distance= typename internal::Spatial_searching_default_distance<SearchTraits>::type,
           class Splitter= Sliding_midpoint<SearchTraits> ,
-          class Tree= Kd_tree<SearchTraits, Splitter, Tag_true> >
-class Orthogonal_k_neighbor_search: public internal::K_neighbor_search<SearchTraits,Distance,Splitter,Tree> {
-  typedef  internal::K_neighbor_search<SearchTraits,Distance,Splitter,Tree> Base;
-
-  typename SearchTraits::Cartesian_const_iterator_d query_object_it;
+          class Tree= Kd_tree<SearchTraits, Splitter, Tag_true, Tag_false> >
+class Orthogonal_k_neighbor_search: public internal::K_neighbor_search<SearchTraits,Distance,Splitter,Tree>
+{
+  typedef internal::K_neighbor_search<SearchTraits,Distance,Splitter,Tree> Base;
+  typedef typename Tree::Point_d Point;
 
-  std::vector<typename Base::FT> dists;
 public:
   typedef typename Base::FT FT;
 
+private:
+  typename SearchTraits::Cartesian_const_iterator_d query_object_it;
+
+  internal::Distance_helper<Distance, SearchTraits> m_distance_helper;
+  std::vector<FT> dists;
+  int m_dim;
+  Tree const& m_tree;
+
+public:
 
   Orthogonal_k_neighbor_search(const Tree& tree, const typename Base::Query_item& q,
                                unsigned int k=1, FT Eps=FT(0.0), bool Search_nearest=true, const Distance& d=Distance(),bool sorted=true)
-    : Base(q,k,Eps,Search_nearest,d)
+  : Base(q,k,Eps,Search_nearest,d),
+    m_distance_helper(this->distance_instance, tree.traits()),
+    m_tree(tree)
   {
     if (tree.empty()) return;
 
     typename SearchTraits::Construct_cartesian_const_iterator_d construct_it=tree.traits().construct_cartesian_const_iterator_d_object();
     query_object_it = construct_it(this->query_object);
 
-    int dim = static_cast<int>(std::distance(query_object_it, construct_it(this->query_object,0)));
+    m_dim = static_cast<int>(std::distance(query_object_it, construct_it(this->query_object,0)));
 
-    dists.resize(dim);
-    for(int i=0;i<dim;i++)
+    dists.resize(m_dim);
+    for(int i=0;i<m_dim;i++)
         dists[i]=0;
 
     FT distance_to_root;
@@ -60,75 +75,178 @@ public:
       compute_furthest_neighbors_orthogonally(tree.root(), distance_to_root);
     }
 
+    if (sorted) this->queue.sort();
+  }
+private:
+
+  // With cache
+  void search_nearest_in_leaf(typename Tree::Leaf_node_const_handle node, Tag_true)
+  {
+    typename Tree::iterator it_node_point = node->begin(), it_node_point_end = node->end();
+    typename std::vector<FT>::const_iterator cache_point_begin = m_tree.cache_begin() + m_dim*(it_node_point - m_tree.begin());
+    // As long as the queue is not full, the node is just inserted
+    for (; !this->queue.full() && it_node_point != it_node_point_end; ++it_node_point)
+    {
+      this->number_of_items_visited++;
 
+      FT distance_to_query_object =
+        m_distance_helper.transformed_distance_from_coordinates(
+          this->query_object, *it_node_point, cache_point_begin, cache_point_begin + m_dim);
+      this->queue.insert(std::make_pair(&(*it_node_point), distance_to_query_object));
 
+      cache_point_begin += m_dim;
+    }
+    // Now that the queue is full, we can gain time by keeping track
+    // of the current worst distance to interrupt the distance computation earlier
+    FT worst_dist = this->queue.top().second;
+    for (; it_node_point != it_node_point_end; ++it_node_point)
+    {
+      this->number_of_items_visited++;
 
+      FT distance_to_query_object =
+        m_distance_helper.interruptible_transformed_distance(
+          this->query_object, *it_node_point, cache_point_begin, cache_point_begin + m_dim, worst_dist);
 
-    if (sorted) this->queue.sort();
+      if (distance_to_query_object < worst_dist)
+      {
+        this->queue.insert(std::make_pair(&(*it_node_point), distance_to_query_object));
+        worst_dist = this->queue.top().second;
+      }
+
+      cache_point_begin += m_dim;
+    }
+  }
+
+  // Without cache
+  void search_nearest_in_leaf(typename Tree::Leaf_node_const_handle node, Tag_false)
+  {
+    typename Tree::iterator it_node_point = node->begin(), it_node_point_end = node->end();
+    // As long as the queue is not full, the node is just inserted
+    for (; !this->queue.full() && it_node_point != it_node_point_end; ++it_node_point)
+    {
+      this->number_of_items_visited++;
+
+      FT distance_to_query_object =
+        this->distance_instance.transformed_distance(this->query_object, *it_node_point);
+      this->queue.insert(std::make_pair(&(*it_node_point), distance_to_query_object));
+    }
+    // Now that the queue is full, we can gain time by keeping track
+    // of the current worst distance to interrupt the distance computation earlier
+    FT worst_dist = this->queue.top().second;
+    for (; it_node_point != it_node_point_end; ++it_node_point)
+    {
+      this->number_of_items_visited++;
+
+      FT distance_to_query_object =
+        m_distance_helper.interruptible_transformed_distance(
+          this->query_object, *it_node_point, worst_dist);
+
+      if (distance_to_query_object < worst_dist)
+      {
+        this->queue.insert(std::make_pair(&(*it_node_point), distance_to_query_object));
+        worst_dist = this->queue.top().second;
+      }
+    }
   }
-private:
 
   void compute_nearest_neighbors_orthogonally(typename Base::Node_const_handle N, FT rd)
   {
-    if (!(N->is_leaf()))
+    if (N->is_leaf())
+    {
+      // n is a leaf
+      typename Tree::Leaf_node_const_handle node =
+        static_cast<typename Tree::Leaf_node_const_handle>(N);
+      this->number_of_leaf_nodes_visited++;
+      if (node->size() > 0)
+      {
+        typename internal::Has_points_cache<Tree, internal::has_Enable_points_cache<Tree>::type::value>::type dummy;
+        search_nearest_in_leaf(node, dummy);
+      }
+    }
+    else
     {
       typename Tree::Internal_node_const_handle node =
         static_cast<typename Tree::Internal_node_const_handle>(N);
       this->number_of_internal_nodes_visited++;
-      int new_cut_dim=node->cutting_dimension();
+      int new_cut_dim = node->cutting_dimension();
       typename Base::Node_const_handle bestChild, otherChild;
       FT new_off;
       FT val = *(query_object_it + new_cut_dim);
       FT diff1 = val - node->upper_low_value();
       FT diff2 = val - node->lower_high_value();
-      if ( (diff1 + diff2 <  FT(0.0)) )
+      if ((diff1 + diff2 <  FT(0.0)))
       {
-          new_off = diff1;
-          bestChild = node->lower();
-          otherChild = node->upper();
+        new_off = diff1;
+        bestChild = node->lower();
+        otherChild = node->upper();
       }
       else // compute new distance
       {
-          new_off= diff2;
-          bestChild = node->upper();
-          otherChild = node->lower();
+        new_off = diff2;
+        bestChild = node->upper();
+        otherChild = node->lower();
       }
       compute_nearest_neighbors_orthogonally(bestChild, rd);
-      FT dst=dists[new_cut_dim];
-      FT new_rd = this->distance_instance.new_distance(rd,dst,new_off,new_cut_dim);
-      dists[new_cut_dim]=new_off;
-        if (this->branch_nearest(new_rd))
-        {
-          compute_nearest_neighbors_orthogonally(otherChild, new_rd);
-        }
-      dists[new_cut_dim]=dst;
+      FT dst = dists[new_cut_dim];
+      FT new_rd = this->distance_instance.new_distance(rd, dst, new_off, new_cut_dim);
+      dists[new_cut_dim] = new_off;
+      if (this->branch_nearest(new_rd))
+      {
+        compute_nearest_neighbors_orthogonally(otherChild, new_rd);
+      }
+      dists[new_cut_dim] = dst;
     }
-    else
+  }
+
+  // With cache
+  void search_furthest_in_leaf(typename Tree::Leaf_node_const_handle node, Tag_true)
+  {
+    typename Tree::iterator it_node_point = node->begin(), it_node_point_end = node->end();
+    typename std::vector<FT>::const_iterator cache_point_begin = m_tree.cache_begin() + m_dim*(it_node_point - m_tree.begin());
+    // In furthest search mode, the interruptible distance cannot be used to optimize
+    for (; it_node_point != it_node_point_end; ++it_node_point)
+    {
+      this->number_of_items_visited++;
+
+      FT distance_to_query_object =
+        m_distance_helper.transformed_distance_from_coordinates(
+          this->query_object, *it_node_point, cache_point_begin, cache_point_begin + m_dim);
+      this->queue.insert(std::make_pair(&(*it_node_point), distance_to_query_object));
+
+      cache_point_begin += m_dim;
+    }
+  }
+
+  // Without cache
+  void search_furthest_in_leaf(typename Tree::Leaf_node_const_handle node, Tag_false)
+  {
+    typename Tree::iterator it_node_point = node->begin(), it_node_point_end = node->end();
+    // In furthest search mode, the interruptible distance cannot be used to optimize
+    for (; it_node_point != it_node_point_end; ++it_node_point)
+    {
+      this->number_of_items_visited++;
+
+      FT distance_to_query_object =
+        this->distance_instance.transformed_distance(this->query_object, *it_node_point);
+      this->queue.insert(std::make_pair(&(*it_node_point), distance_to_query_object));
+    }
+  }
+
+  void compute_furthest_neighbors_orthogonally(typename Base::Node_const_handle N, FT rd)
+  {
+    if (N->is_leaf())
     {
       // n is a leaf
       typename Tree::Leaf_node_const_handle node =
         static_cast<typename Tree::Leaf_node_const_handle>(N);
       this->number_of_leaf_nodes_visited++;
-      bool full = this->queue.full();
-      FT worst_dist = this->queue.top().second;
       if (node->size() > 0)
       {
-        for (typename Tree::iterator it=node->begin(); it != node->end(); it++)
-        {
-          this->number_of_items_visited++;
-          FT distance_to_query_object=
-            this->distance_instance.transformed_distance(this->query_object,*it);
-
-          if(!full || distance_to_query_object < worst_dist)
-            this->queue.insert(std::make_pair(&(*it),distance_to_query_object));
-        }
+        typename internal::Has_points_cache<Tree, internal::has_Enable_points_cache<Tree>::type::value>::type dummy;
+        search_furthest_in_leaf(node, dummy);
       }
     }
-  }
-
-   void compute_furthest_neighbors_orthogonally(typename Base::Node_const_handle N, FT rd)
-  {
-    if (!(N->is_leaf()))
+    else
     {
       typename Tree::Internal_node_const_handle node =
         static_cast<typename Tree::Internal_node_const_handle>(N);
@@ -163,23 +281,6 @@ private:
           compute_furthest_neighbors_orthogonally(otherChild, new_rd);
       dists[new_cut_dim]=dst;
     }
-    else
-    {
-      // n is a leaf
-      typename Tree::Leaf_node_const_handle node =
-        static_cast<typename Tree::Leaf_node_const_handle>(N);
-      this->number_of_leaf_nodes_visited++;
-      if (node->size() > 0)
-      {
-        for (typename Tree::iterator it=node->begin(); it != node->end(); it++)
-        {
-          this->number_of_items_visited++;
-          FT distance_to_query_object=
-            this->distance_instance.transformed_distance(this->query_object,*it);
-          this->queue.insert(std::make_pair(&(*it),distance_to_query_object));
-        }
-      }
-    }
   }
 
 }; // class
diff -pruN 5.0.3-1/include/CGAL/OTR_2/Cost.h 5.1-1/include/CGAL/OTR_2/Cost.h
--- 5.0.3-1/include/CGAL/OTR_2/Cost.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/OTR_2/Cost.h	2020-09-08 12:15:17.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Optimal_transportation_reconstruction_2/include/CGAL/OTR_2/Cost.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Optimal_transportation_reconstruction_2/include/CGAL/OTR_2/Cost.h $
 // $Id: Cost.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/OTR_2/Reconstruction_edge_2.h 5.1-1/include/CGAL/OTR_2/Reconstruction_edge_2.h
--- 5.0.3-1/include/CGAL/OTR_2/Reconstruction_edge_2.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/OTR_2/Reconstruction_edge_2.h	2020-09-08 12:15:17.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Optimal_transportation_reconstruction_2/include/CGAL/OTR_2/Reconstruction_edge_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Optimal_transportation_reconstruction_2/include/CGAL/OTR_2/Reconstruction_edge_2.h $
 // $Id: Reconstruction_edge_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/OTR_2/Reconstruction_face_base_2.h 5.1-1/include/CGAL/OTR_2/Reconstruction_face_base_2.h
--- 5.0.3-1/include/CGAL/OTR_2/Reconstruction_face_base_2.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/OTR_2/Reconstruction_face_base_2.h	2020-09-08 12:15:17.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Optimal_transportation_reconstruction_2/include/CGAL/OTR_2/Reconstruction_face_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Optimal_transportation_reconstruction_2/include/CGAL/OTR_2/Reconstruction_face_base_2.h $
 // $Id: Reconstruction_face_base_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/OTR_2/Reconstruction_triangulation_2.h 5.1-1/include/CGAL/OTR_2/Reconstruction_triangulation_2.h
--- 5.0.3-1/include/CGAL/OTR_2/Reconstruction_triangulation_2.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/OTR_2/Reconstruction_triangulation_2.h	2020-09-08 12:15:17.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Optimal_transportation_reconstruction_2/include/CGAL/OTR_2/Reconstruction_triangulation_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Optimal_transportation_reconstruction_2/include/CGAL/OTR_2/Reconstruction_triangulation_2.h $
 // $Id: Reconstruction_triangulation_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/OTR_2/Reconstruction_vertex_base_2.h 5.1-1/include/CGAL/OTR_2/Reconstruction_vertex_base_2.h
--- 5.0.3-1/include/CGAL/OTR_2/Reconstruction_vertex_base_2.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/OTR_2/Reconstruction_vertex_base_2.h	2020-09-08 12:15:17.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Optimal_transportation_reconstruction_2/include/CGAL/OTR_2/Reconstruction_vertex_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Optimal_transportation_reconstruction_2/include/CGAL/OTR_2/Reconstruction_vertex_base_2.h $
 // $Id: Reconstruction_vertex_base_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/OTR_2/Sample.h 5.1-1/include/CGAL/OTR_2/Sample.h
--- 5.0.3-1/include/CGAL/OTR_2/Sample.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/OTR_2/Sample.h	2020-09-08 12:15:17.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Optimal_transportation_reconstruction_2/include/CGAL/OTR_2/Sample.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Optimal_transportation_reconstruction_2/include/CGAL/OTR_2/Sample.h $
 // $Id: Sample.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Parabola_2.h 5.1-1/include/CGAL/Parabola_2.h
--- 5.0.3-1/include/CGAL/Parabola_2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Parabola_2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Parabola_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Parabola_2.h $
 // $Id: Parabola_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Parabola_segment_2.h 5.1-1/include/CGAL/Parabola_segment_2.h
--- 5.0.3-1/include/CGAL/Parabola_segment_2.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Parabola_segment_2.h	2020-09-08 12:14:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Apollonius_graph_2/include/CGAL/Parabola_segment_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Apollonius_graph_2/include/CGAL/Parabola_segment_2.h $
 // $Id: Parabola_segment_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Partition_2/Circulator_pair.h 5.1-1/include/CGAL/Partition_2/Circulator_pair.h
--- 5.0.3-1/include/CGAL/Partition_2/Circulator_pair.h	2020-08-01 19:02:37.000000000 +0000
+++ 5.1-1/include/CGAL/Partition_2/Circulator_pair.h	2020-09-08 12:15:17.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Partition_2/include/CGAL/Partition_2/Circulator_pair.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Partition_2/include/CGAL/Partition_2/Circulator_pair.h $
 // $Id: Circulator_pair.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Partition_2/Indirect_edge_compare.h 5.1-1/include/CGAL/Partition_2/Indirect_edge_compare.h
--- 5.0.3-1/include/CGAL/Partition_2/Indirect_edge_compare.h	2020-08-01 19:02:37.000000000 +0000
+++ 5.1-1/include/CGAL/Partition_2/Indirect_edge_compare.h	2020-09-08 12:15:18.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Partition_2/include/CGAL/Partition_2/Indirect_edge_compare.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Partition_2/include/CGAL/Partition_2/Indirect_edge_compare.h $
 // $Id: Indirect_edge_compare.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Partition_2/Indirect_less_xy_2.h 5.1-1/include/CGAL/Partition_2/Indirect_less_xy_2.h
--- 5.0.3-1/include/CGAL/Partition_2/Indirect_less_xy_2.h	2020-08-01 19:02:37.000000000 +0000
+++ 5.1-1/include/CGAL/Partition_2/Indirect_less_xy_2.h	2020-09-08 12:15:18.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Partition_2/include/CGAL/Partition_2/Indirect_less_xy_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Partition_2/include/CGAL/Partition_2/Indirect_less_xy_2.h $
 // $Id: Indirect_less_xy_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Partition_2/Indirect_not_less_yx_2.h 5.1-1/include/CGAL/Partition_2/Indirect_not_less_yx_2.h
--- 5.0.3-1/include/CGAL/Partition_2/Indirect_not_less_yx_2.h	2020-08-01 19:02:37.000000000 +0000
+++ 5.1-1/include/CGAL/Partition_2/Indirect_not_less_yx_2.h	2020-09-08 12:15:18.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Partition_2/include/CGAL/Partition_2/Indirect_not_less_yx_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Partition_2/include/CGAL/Partition_2/Indirect_not_less_yx_2.h $
 // $Id: Indirect_not_less_yx_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Partition_2/is_degenerate_polygon_2.h 5.1-1/include/CGAL/Partition_2/is_degenerate_polygon_2.h
--- 5.0.3-1/include/CGAL/Partition_2/is_degenerate_polygon_2.h	2020-08-01 19:02:37.000000000 +0000
+++ 5.1-1/include/CGAL/Partition_2/is_degenerate_polygon_2.h	2020-09-08 12:15:18.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Partition_2/include/CGAL/Partition_2/is_degenerate_polygon_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Partition_2/include/CGAL/Partition_2/is_degenerate_polygon_2.h $
 // $Id: is_degenerate_polygon_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Partition_2/Iterator_list.h 5.1-1/include/CGAL/Partition_2/Iterator_list.h
--- 5.0.3-1/include/CGAL/Partition_2/Iterator_list.h	2020-08-01 19:02:37.000000000 +0000
+++ 5.1-1/include/CGAL/Partition_2/Iterator_list.h	2020-09-08 12:15:18.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Partition_2/include/CGAL/Partition_2/Iterator_list.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Partition_2/include/CGAL/Partition_2/Iterator_list.h $
 // $Id: Iterator_list.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Partition_2/Matrix.h 5.1-1/include/CGAL/Partition_2/Matrix.h
--- 5.0.3-1/include/CGAL/Partition_2/Matrix.h	2020-08-01 19:02:37.000000000 +0000
+++ 5.1-1/include/CGAL/Partition_2/Matrix.h	2020-09-08 12:15:18.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Partition_2/include/CGAL/Partition_2/Matrix.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Partition_2/include/CGAL/Partition_2/Matrix.h $
 // $Id: Matrix.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Partition_2/partition_approx_convex_2.h 5.1-1/include/CGAL/Partition_2/partition_approx_convex_2.h
--- 5.0.3-1/include/CGAL/Partition_2/partition_approx_convex_2.h	2020-08-01 19:02:37.000000000 +0000
+++ 5.1-1/include/CGAL/Partition_2/partition_approx_convex_2.h	2020-09-08 12:15:18.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Partition_2/include/CGAL/Partition_2/partition_approx_convex_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Partition_2/include/CGAL/Partition_2/partition_approx_convex_2.h $
 // $Id: partition_approx_convex_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Partition_2/partition_assertions.h 5.1-1/include/CGAL/Partition_2/partition_assertions.h
--- 5.0.3-1/include/CGAL/Partition_2/partition_assertions.h	2020-08-01 19:02:37.000000000 +0000
+++ 5.1-1/include/CGAL/Partition_2/partition_assertions.h	2020-09-08 12:15:18.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Partition_2/include/CGAL/Partition_2/partition_assertions.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Partition_2/include/CGAL/Partition_2/partition_assertions.h $
 // $Id: partition_assertions.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Partition_2/Partitioned_polygon_2.h 5.1-1/include/CGAL/Partition_2/Partitioned_polygon_2.h
--- 5.0.3-1/include/CGAL/Partition_2/Partitioned_polygon_2.h	2020-08-01 19:02:37.000000000 +0000
+++ 5.1-1/include/CGAL/Partition_2/Partitioned_polygon_2.h	2020-09-08 12:15:18.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Partition_2/include/CGAL/Partition_2/Partitioned_polygon_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Partition_2/include/CGAL/Partition_2/Partitioned_polygon_2.h $
 // $Id: Partitioned_polygon_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Partition_2/partition_greene_approx_convex_2.h 5.1-1/include/CGAL/Partition_2/partition_greene_approx_convex_2.h
--- 5.0.3-1/include/CGAL/Partition_2/partition_greene_approx_convex_2.h	2020-08-01 19:02:37.000000000 +0000
+++ 5.1-1/include/CGAL/Partition_2/partition_greene_approx_convex_2.h	2020-09-08 12:15:18.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Partition_2/include/CGAL/Partition_2/partition_greene_approx_convex_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Partition_2/include/CGAL/Partition_2/partition_greene_approx_convex_2.h $
 // $Id: partition_greene_approx_convex_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Partition_2/Partition_opt_cvx_diagonal_list.h 5.1-1/include/CGAL/Partition_2/Partition_opt_cvx_diagonal_list.h
--- 5.0.3-1/include/CGAL/Partition_2/Partition_opt_cvx_diagonal_list.h	2020-08-01 19:02:37.000000000 +0000
+++ 5.1-1/include/CGAL/Partition_2/Partition_opt_cvx_diagonal_list.h	2020-09-08 12:15:18.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Partition_2/include/CGAL/Partition_2/Partition_opt_cvx_diagonal_list.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Partition_2/include/CGAL/Partition_2/Partition_opt_cvx_diagonal_list.h $
 // $Id: Partition_opt_cvx_diagonal_list.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Partition_2/Partition_opt_cvx_edge.h 5.1-1/include/CGAL/Partition_2/Partition_opt_cvx_edge.h
--- 5.0.3-1/include/CGAL/Partition_2/Partition_opt_cvx_edge.h	2020-08-01 19:02:37.000000000 +0000
+++ 5.1-1/include/CGAL/Partition_2/Partition_opt_cvx_edge.h	2020-09-08 12:15:18.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Partition_2/include/CGAL/Partition_2/Partition_opt_cvx_edge.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Partition_2/include/CGAL/Partition_2/Partition_opt_cvx_edge.h $
 // $Id: Partition_opt_cvx_edge.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Partition_2/Partition_opt_cvx_vertex.h 5.1-1/include/CGAL/Partition_2/Partition_opt_cvx_vertex.h
--- 5.0.3-1/include/CGAL/Partition_2/Partition_opt_cvx_vertex.h	2020-08-01 19:02:37.000000000 +0000
+++ 5.1-1/include/CGAL/Partition_2/Partition_opt_cvx_vertex.h	2020-09-08 12:15:18.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Partition_2/include/CGAL/Partition_2/Partition_opt_cvx_vertex.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Partition_2/include/CGAL/Partition_2/Partition_opt_cvx_vertex.h $
 // $Id: Partition_opt_cvx_vertex.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Partition_2/partition_optimal_convex_2.h 5.1-1/include/CGAL/Partition_2/partition_optimal_convex_2.h
--- 5.0.3-1/include/CGAL/Partition_2/partition_optimal_convex_2.h	2020-08-01 19:02:37.000000000 +0000
+++ 5.1-1/include/CGAL/Partition_2/partition_optimal_convex_2.h	2020-09-08 12:15:18.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Partition_2/include/CGAL/Partition_2/partition_optimal_convex_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Partition_2/include/CGAL/Partition_2/partition_optimal_convex_2.h $
 // $Id: partition_optimal_convex_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Partition_2/Partition_traits_2_base.h 5.1-1/include/CGAL/Partition_2/Partition_traits_2_base.h
--- 5.0.3-1/include/CGAL/Partition_2/Partition_traits_2_base.h	2020-08-01 19:02:37.000000000 +0000
+++ 5.1-1/include/CGAL/Partition_2/Partition_traits_2_base.h	2020-09-08 12:15:18.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Partition_2/include/CGAL/Partition_2/Partition_traits_2_base.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Partition_2/include/CGAL/Partition_2/Partition_traits_2_base.h $
 // $Id: Partition_traits_2_base.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Partition_2/Partition_vertex_map.h 5.1-1/include/CGAL/Partition_2/Partition_vertex_map.h
--- 5.0.3-1/include/CGAL/Partition_2/Partition_vertex_map.h	2020-08-01 19:02:37.000000000 +0000
+++ 5.1-1/include/CGAL/Partition_2/Partition_vertex_map.h	2020-09-08 12:15:18.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Partition_2/include/CGAL/Partition_2/Partition_vertex_map.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Partition_2/include/CGAL/Partition_2/Partition_vertex_map.h $
 // $Id: Partition_vertex_map.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Partition_2/partition_y_monotone_2.h 5.1-1/include/CGAL/Partition_2/partition_y_monotone_2.h
--- 5.0.3-1/include/CGAL/Partition_2/partition_y_monotone_2.h	2020-08-01 19:02:37.000000000 +0000
+++ 5.1-1/include/CGAL/Partition_2/partition_y_monotone_2.h	2020-09-08 12:15:18.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Partition_2/include/CGAL/Partition_2/partition_y_monotone_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Partition_2/include/CGAL/Partition_2/partition_y_monotone_2.h $
 // $Id: partition_y_monotone_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Partition_2/Point_pair_less_xy_2.h 5.1-1/include/CGAL/Partition_2/Point_pair_less_xy_2.h
--- 5.0.3-1/include/CGAL/Partition_2/Point_pair_less_xy_2.h	2020-08-01 19:02:37.000000000 +0000
+++ 5.1-1/include/CGAL/Partition_2/Point_pair_less_xy_2.h	2020-09-08 12:15:18.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Partition_2/include/CGAL/Partition_2/Point_pair_less_xy_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Partition_2/include/CGAL/Partition_2/Point_pair_less_xy_2.h $
 // $Id: Point_pair_less_xy_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Partition_2/Rotation_tree_2.h 5.1-1/include/CGAL/Partition_2/Rotation_tree_2.h
--- 5.0.3-1/include/CGAL/Partition_2/Rotation_tree_2.h	2020-08-01 19:02:37.000000000 +0000
+++ 5.1-1/include/CGAL/Partition_2/Rotation_tree_2.h	2020-09-08 12:15:18.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Partition_2/include/CGAL/Partition_2/Rotation_tree_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Partition_2/include/CGAL/Partition_2/Rotation_tree_2.h $
 // $Id: Rotation_tree_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Partition_2/Rotation_tree_2_impl.h 5.1-1/include/CGAL/Partition_2/Rotation_tree_2_impl.h
--- 5.0.3-1/include/CGAL/Partition_2/Rotation_tree_2_impl.h	2020-08-01 19:02:37.000000000 +0000
+++ 5.1-1/include/CGAL/Partition_2/Rotation_tree_2_impl.h	2020-09-08 12:15:18.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Partition_2/include/CGAL/Partition_2/Rotation_tree_2_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Partition_2/include/CGAL/Partition_2/Rotation_tree_2_impl.h $
 // $Id: Rotation_tree_2_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Partition_2/Rotation_tree_node_2.h 5.1-1/include/CGAL/Partition_2/Rotation_tree_node_2.h
--- 5.0.3-1/include/CGAL/Partition_2/Rotation_tree_node_2.h	2020-08-01 19:02:37.000000000 +0000
+++ 5.1-1/include/CGAL/Partition_2/Rotation_tree_node_2.h	2020-09-08 12:15:18.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Partition_2/include/CGAL/Partition_2/Rotation_tree_node_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Partition_2/include/CGAL/Partition_2/Rotation_tree_node_2.h $
 // $Id: Rotation_tree_node_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Partition_2/Segment_less_yx_2.h 5.1-1/include/CGAL/Partition_2/Segment_less_yx_2.h
--- 5.0.3-1/include/CGAL/Partition_2/Segment_less_yx_2.h	2020-08-01 19:02:37.000000000 +0000
+++ 5.1-1/include/CGAL/Partition_2/Segment_less_yx_2.h	2020-09-08 12:15:18.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Partition_2/include/CGAL/Partition_2/Segment_less_yx_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Partition_2/include/CGAL/Partition_2/Segment_less_yx_2.h $
 // $Id: Segment_less_yx_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Partition_2/Triangulation_indirect_traits_2.h 5.1-1/include/CGAL/Partition_2/Triangulation_indirect_traits_2.h
--- 5.0.3-1/include/CGAL/Partition_2/Triangulation_indirect_traits_2.h	2020-08-01 19:02:37.000000000 +0000
+++ 5.1-1/include/CGAL/Partition_2/Triangulation_indirect_traits_2.h	2020-09-08 12:15:18.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Partition_2/include/CGAL/Partition_2/Triangulation_indirect_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Partition_2/include/CGAL/Partition_2/Triangulation_indirect_traits_2.h $
 // $Id: Triangulation_indirect_traits_2.h 2e8a59d 2020-07-21T15:25:54+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Partition_2/Turn_reverser.h 5.1-1/include/CGAL/Partition_2/Turn_reverser.h
--- 5.0.3-1/include/CGAL/Partition_2/Turn_reverser.h	2020-08-01 19:02:37.000000000 +0000
+++ 5.1-1/include/CGAL/Partition_2/Turn_reverser.h	2020-09-08 12:15:18.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Partition_2/include/CGAL/Partition_2/Turn_reverser.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Partition_2/include/CGAL/Partition_2/Turn_reverser.h $
 // $Id: Turn_reverser.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Partition_2/Vertex_visibility_graph_2.h 5.1-1/include/CGAL/Partition_2/Vertex_visibility_graph_2.h
--- 5.0.3-1/include/CGAL/Partition_2/Vertex_visibility_graph_2.h	2020-08-01 19:02:37.000000000 +0000
+++ 5.1-1/include/CGAL/Partition_2/Vertex_visibility_graph_2.h	2020-09-08 12:15:18.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Partition_2/include/CGAL/Partition_2/Vertex_visibility_graph_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Partition_2/include/CGAL/Partition_2/Vertex_visibility_graph_2.h $
 // $Id: Vertex_visibility_graph_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Partition_2/Vertex_visibility_graph_2_impl.h 5.1-1/include/CGAL/Partition_2/Vertex_visibility_graph_2_impl.h
--- 5.0.3-1/include/CGAL/Partition_2/Vertex_visibility_graph_2_impl.h	2020-08-01 19:02:37.000000000 +0000
+++ 5.1-1/include/CGAL/Partition_2/Vertex_visibility_graph_2_impl.h	2020-09-08 12:15:18.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Partition_2/include/CGAL/Partition_2/Vertex_visibility_graph_2_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Partition_2/include/CGAL/Partition_2/Vertex_visibility_graph_2_impl.h $
 // $Id: Vertex_visibility_graph_2_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/partition_2.h 5.1-1/include/CGAL/partition_2.h
--- 5.0.3-1/include/CGAL/partition_2.h	2020-08-01 19:02:37.000000000 +0000
+++ 5.1-1/include/CGAL/partition_2.h	2020-09-08 12:15:18.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Partition_2/include/CGAL/partition_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Partition_2/include/CGAL/partition_2.h $
 // $Id: partition_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/partition_is_valid_2.h 5.1-1/include/CGAL/partition_is_valid_2.h
--- 5.0.3-1/include/CGAL/partition_is_valid_2.h	2020-08-01 19:02:37.000000000 +0000
+++ 5.1-1/include/CGAL/partition_is_valid_2.h	2020-09-08 12:15:18.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Partition_2/include/CGAL/partition_is_valid_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Partition_2/include/CGAL/partition_is_valid_2.h $
 // $Id: partition_is_valid_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Partition_is_valid_traits_2.h 5.1-1/include/CGAL/Partition_is_valid_traits_2.h
--- 5.0.3-1/include/CGAL/Partition_is_valid_traits_2.h	2020-08-01 19:02:37.000000000 +0000
+++ 5.1-1/include/CGAL/Partition_is_valid_traits_2.h	2020-09-08 12:15:18.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Partition_2/include/CGAL/Partition_is_valid_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Partition_2/include/CGAL/Partition_is_valid_traits_2.h $
 // $Id: Partition_is_valid_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Partition_traits_2.h 5.1-1/include/CGAL/Partition_traits_2.h
--- 5.0.3-1/include/CGAL/Partition_traits_2.h	2020-08-01 19:02:37.000000000 +0000
+++ 5.1-1/include/CGAL/Partition_traits_2.h	2020-09-08 12:15:18.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Partition_2/include/CGAL/Partition_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Partition_2/include/CGAL/Partition_traits_2.h $
 // $Id: Partition_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Partition_traits_adapter_2.h 5.1-1/include/CGAL/Partition_traits_adapter_2.h
--- 5.0.3-1/include/CGAL/Partition_traits_adapter_2.h	2020-08-01 19:02:37.000000000 +0000
+++ 5.1-1/include/CGAL/Partition_traits_adapter_2.h	2020-09-08 12:15:18.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Partition_2/include/CGAL/Partition_traits_adapter_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Partition_2/include/CGAL/Partition_traits_adapter_2.h $
 // $Id: Partition_traits_adapter_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Path_on_surface.h 5.1-1/include/CGAL/Path_on_surface.h
--- 5.0.3-1/include/CGAL/Path_on_surface.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Path_on_surface.h	2020-09-08 12:15:45.000000000 +0000
@@ -0,0 +1,1270 @@
+// Copyright (c) 2019 CNRS and LIRIS' Establishments (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_topology/include/CGAL/Path_on_surface.h $
+// $Id: Path_on_surface.h 52186a0 2020-05-14T11:38:15+02:00 Guillaume Damiand
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
+//
+#ifndef CGAL_PATH_ON_SURFACE_H
+#define CGAL_PATH_ON_SURFACE_H 1
+
+#include <CGAL/license/Surface_mesh_topology.h>
+
+#include <CGAL/Combinatorial_map_operations.h>
+#include <CGAL/Combinatorial_map.h>
+#include <CGAL/Random.h>
+#include <CGAL/Face_graph_wrapper.h>
+#include <CGAL/Surface_mesh_topology/internal/Path_on_surface_with_rle.h>
+#include <boost/algorithm/searching/knuth_morris_pratt.hpp>
+#include <utility>
+#include <string>
+#include <vector>
+#include <iostream>
+#include <sstream>
+#include <initializer_list>
+
+// A Path_on_surface contains two vectors of equal length n
+// The first one is a vector of darts called m_path and the second one a vector
+// of booleans called m_flip.
+// If n = 0, the path represented by those vectors is the empty path.
+// Else, it is the path represented by the n-1 first elements of both vectors,
+// at the one we add the m_path[n-1] dart if m_flip[n-1] is false and the
+// opposite of this dart if m_flip[n-1] is true i.e. if m_flip[i] is true means
+// that the i-th dart m_path[i] has to be flipped.
+// We use flips because sometimes opposite darts doesn't exist on surfaces with
+// boundaries. But if m_flip[i] is true doesn't necesary mean that
+// m_path[i] is 2-free
+
+namespace CGAL {
+namespace Surface_mesh_topology {
+
+template<typename Mesh_>
+class Path_on_surface
+{
+public:
+  typedef Path_on_surface<Mesh_>             Self;
+  typedef Mesh_                              Mesh;
+  typedef typename Get_map<Mesh, Mesh>::type Map; // Mesh seen as a 2-map
+  typedef typename Map::Dart_const_handle    Dart_const_handle;
+
+  typedef Dart_const_handle halfedge_descriptor; // To be compatible with BGL
+
+  Path_on_surface(const Mesh& amesh) : m_map(amesh), m_is_closed(false)
+  {}
+
+  template<class COST>
+  Path_on_surface(const internal::Path_on_surface_with_rle<COST>& apath) :
+    m_map(apath.get_map()),
+    m_is_closed(apath.is_closed())
+  {
+    for (auto it=apath.m_path.begin(), itend=apath.m_path.end(); it!=itend; ++it)
+    {
+      push_back(it->begin, false, false);
+      if (it->length>0)
+      { extend_straight_positive(it->length, false); }
+      else if (it->length<0)
+      { extend_straight_negative(-(it->length), false); }
+    }
+    update_is_closed();
+    CGAL_assertion(is_valid(true));
+  }
+
+  Path_on_surface(const Self& apath) : m_map(apath.m_map),
+                                       m_path(apath.m_path),
+                                       m_is_closed(apath.m_is_closed),
+                                       m_flip(apath.m_flip)
+  {}
+
+  void swap(Self& p2)
+  {
+    if (this==&p2) { return; }
+
+    CGAL_assertion(&get_mesh()==&(p2.get_mesh()));
+    m_path.swap(p2.m_path);
+    std::swap(m_is_closed, p2.m_is_closed);
+    m_flip.swap(p2.m_flip);
+  }
+
+  Self& operator=(const Self& other)
+  {
+    CGAL_assertion(&get_mesh()==&(other.get_mesh()));
+    if (this!=&other)
+    {
+      m_path=other.m_path;
+      m_is_closed=other.m_is_closed;
+      m_flip=other.m_flip;
+    }
+    return *this;
+  }
+
+  /// @return true iff the path is empty
+  bool is_empty() const
+  { return m_path.empty(); }
+
+  /// @return the length of the path, i.e. its number of darts.
+  std::size_t length() const
+  { return m_path.size(); }
+
+  /// @return true iff the path is closed.
+  ///  (m_is_closed is updated after each path modification).
+  bool is_closed() const
+  { return m_is_closed; }
+
+  /// @return the combinatorial map supporting this path.
+  const Map& get_map() const
+  { return m_map; }
+
+  /// @return the combinatorial map supporting this path.
+  const Mesh& get_mesh() const
+  { return Get_map<Mesh, Mesh>::get_mesh(m_map); }
+
+  const std::vector<bool>& get_flip() const
+  { return m_flip; }
+
+  /// clear the path.
+  void clear()
+  {
+    m_path.clear();
+    m_flip.clear();
+    m_is_closed=false;
+  }
+
+  /// @return true iff the prev index exists
+  bool prev_index_exists(std::size_t i) const
+  { return is_closed() || i>0; }
+
+  /// @return true iff the next index exists
+  bool next_index_exists(std::size_t i) const
+  { return is_closed() || i<(m_path.size()-1); }
+
+  /// @return the index after index i.
+  std::size_t next_index(std::size_t i) const
+  { return ((is_closed() && i==(m_path.size()-1))?0:(i+1)); }
+
+  /// @return the index before index i.
+  std::size_t prev_index(std::size_t i) const
+  { return ((is_closed() && i==0)?(m_path.size()-1):(i-1)); }
+
+  /// @return the ith dart of the path.
+  Dart_const_handle get_ith_dart(std::size_t i) const
+  {
+    CGAL_assertion(i<m_path.size());
+    return m_path[i];
+  }
+
+  /// @return true iff the ith dart is flipped
+  bool get_ith_flip(std::size_t i) const
+  {
+    CGAL_assertion(i<m_path.size());
+    return m_flip[i];
+  }
+
+  /// @return the ith dart of the path.
+  Dart_const_handle operator[] (std::size_t i) const
+  { return get_ith_dart(i); }
+
+  /// @return the dart before the ith dart of the path,
+  ///          Map::null_handle if such a dart does not exist.
+  Dart_const_handle get_prev_dart(std::size_t i) const
+  {
+    CGAL_assertion(i<m_path.size());
+    if (i==0 && !is_closed()) return Map::null_handle;
+    return m_path[prev_index(i)];
+  }
+
+  /// @return the dart after the ith dart of the path,
+  ///          Map::null_handle if such a dart does not exist.
+  Dart_const_handle get_next_dart(std::size_t i) const
+  {
+    CGAL_assertion(i<m_path.size());
+    if (i==m_path.size()-1 && !is_closed()) return Map::null_handle;
+    return m_path[next_index(i)];
+  }
+
+  /// @return the flip before the ith flip of the path,
+  ///          false if such a flip does not exist.
+  bool get_prev_flip(std::size_t i) const
+  {
+    CGAL_assertion(i<m_path.size());
+    if (i==0 && !is_closed()) return false;
+    return m_flip[prev_index(i)];
+  }
+
+  /// @return the flip after the ith flip of the path,
+  ///          false if such a flip does not exist.
+  bool get_next_flip(std::size_t i) const
+  {
+    CGAL_assertion(i<m_path.size());
+    if (i==m_path.size()-1 && !is_closed()) return false;
+    return m_flip[next_index(i)];
+  }
+
+  /// @return the first dart of the path.
+  /// @pre !is_empty()
+  Dart_const_handle front() const
+  {
+    CGAL_assertion(!is_empty());
+    return m_path.front();
+  }
+
+  /// @return the last dart of the path.
+  /// @pre !is_empty()
+  Dart_const_handle back() const
+  {
+    CGAL_assertion(!is_empty());
+    return m_path.back();
+  }
+
+  /// @return the first flip of the path.
+  /// @pre !is_empty()
+  bool front_flip() const
+  {
+    CGAL_assertion(!is_empty());
+    return m_flip.front();
+  }
+
+  /// @return the last flip of the path.
+  /// @pre !is_empty()
+  bool back_flip() const
+  {
+    CGAL_assertion(!is_empty());
+    return m_flip.back();
+  }
+
+  /// @return the index of the first dart of the path.
+  /// @pre !is_empty()
+  std::size_t front_index() const
+  { return get_map().darts().index(front()); }
+
+  /// @return the index of the last dart of the path.
+  /// @pre !is_empty()
+  std::size_t back_index() const
+  { return get_map().darts().index(back()); }
+
+  /// @return the ith dart of the path taking into account flip.
+  /// return null_handle if flip and there is no beta2
+  Dart_const_handle get_ith_real_dart(std::size_t i) const
+  {
+    CGAL_assertion(i<m_path.size());
+    return (get_ith_flip(i)?get_map().opposite2(get_ith_dart(i)):
+                            get_ith_dart(i));
+  }
+
+  /// @return the opposite of the ith dart of the path taking into account flip.
+  /// return null_handle if !flip and there is no beta2
+  Dart_const_handle get_opposite_ith_real_dart(std::size_t i) const
+  {
+    CGAL_assertion(i<m_path.size());
+    return (get_ith_flip(i)?get_ith_dart(i):
+                            get_map().opposite2(get_ith_dart(i)));
+  }
+
+  /// @return the first dart of the path, taking into account flip.
+  /// @pre !is_empty()
+  Dart_const_handle real_front() const
+  {
+    CGAL_assertion(!is_empty());
+    return get_ith_real_dart(0);
+  }
+
+  /// @return the last dart of the path, taking into account flip.
+  /// @pre !is_empty()
+  Dart_const_handle real_back() const
+  {
+    CGAL_assertion(!is_empty());
+    return get_ith_real_dart(length()-1);
+  }
+
+  /// @return true iff df can be added at the end of the path.
+  bool can_be_pushed(Dart_const_handle dh, bool flip=false) const
+  {
+    // This assert is too long CGAL_assertion(m_map.darts().owns(dh));
+
+    if (is_empty()) return true;
+
+    return m_map.template belong_to_same_cell<0>
+        (m_flip.back() ? back() : m_map.other_extremity(back()),
+         flip ? m_map.other_extremity(dh) : dh);
+  }
+
+  /// Add the given dart at the end of this path.
+  /// @pre can_be_pushed(dh)
+  void push_back(Dart_const_handle dh, bool flip=false,
+                 bool update_isclosed=true)
+  {
+    CGAL_assertion(dh!=Map::null_handle);
+    /* This assert is too long, it is tested in the is_valid method. */
+    //  CGAL_assertion(can_be_pushed(dh, flip));
+
+    m_path.push_back(dh);
+    m_flip.push_back(flip);
+    if (update_isclosed) { update_is_closed(); }
+  }
+
+  /// @return true iff the ith dart can be added at the end of the path.
+  bool can_be_pushed_by_index(typename Map::size_type i, bool flip=false,
+                              bool update_isclosed=true) const
+  { return can_be_pushed(get_map().dart_handle(i), flip, update_isclosed); }
+
+  /// Add the given ith dart at the end of this path.
+  void push_back_by_index(typename Map::size_type i, bool flip=false,
+                          bool update_isclosed=true)
+  { push_back(get_map().dart_handle(i), flip, update_isclosed); }
+
+  void push_back_by_index(std::initializer_list<typename Map::size_type> l,
+                          bool update_isclosed=true)
+  {
+    for (std::size_t i : l)
+    { push_back_by_index(i, false, update_isclosed); }
+  }
+
+  /// @return true iff the dart labeled e can be added at the end of the path.
+  bool can_be_pushed_by_label(const std::string& e, bool flip=false) const
+  {
+    Dart_const_handle dh=get_map().get_dart_labeled(e);
+    if (dh==Map::null_handle) { return false; }
+    return can_be_pushed(dh, flip);
+  }
+
+  /// Add the dart having the given labels at the end of this path.
+  /// Each label is a word, possibly starting by -, words are separated by spaces
+  void push_back_by_label(const std::string& s, bool update_isclosed=true)
+  {
+    std::istringstream iss(s);
+    for (std::string e; std::getline(iss, e, ' '); )
+    {
+      Dart_const_handle dh=get_map().get_dart_labeled(e);
+      if (dh!=Map::null_handle) { push_back(dh, false, update_isclosed); }
+    }
+  }
+
+  void push_back_by_label(std::initializer_list<const char*> l,
+                          bool update_isclosed=true)
+  {
+    for (const char* e : l)
+    { push_back_by_label(e, false, update_isclosed); }
+  }
+
+  Self& operator+=(const Self& other)
+  {
+    m_path.reserve(m_path.size()+other.m_path.size());
+    // Be careful to the special case when *this==other
+    // this is the reason of the iend.
+    for (std::size_t i=0, iend=other.length(); i<iend; ++i)
+    { push_back(other[i], other.m_flip[i], false); }
+    update_is_closed();
+    return *this;
+  }
+
+  Self operator+(const Self& other) const
+  {
+    Self res=*this;
+    res+=other;
+    return res;
+  }
+
+  /// change m_path and m_flip in order to get the lower number of flips possible
+  void simplify_flips(bool show_flips_left=false)
+  {
+    if (show_flips_left)
+    { std::cout<<"Flips left (maybe none) : "<<std::flush; }
+    for(unsigned int i=0; i<length(); ++i)
+    {
+      if (m_flip[i] && !get_map().template is_free<2>(m_path[i]))
+      {
+        m_path[i]=get_map().opposite2(m_path[i]);
+        m_flip[i]=!m_flip[i];
+      }
+      else if (show_flips_left)
+      { std::cout<<i<<" "<<std::flush; }
+    }
+    if (show_flips_left)
+    { std::cout<<std::endl; }
+  }
+
+  /// @return the number of flips of this path.
+  unsigned int nb_flips()
+  {
+    unsigned int res=0;
+    for (unsigned int i=0; i<length(); ++i)
+    { if (m_flip[i]) ++res; }
+    return res;
+  }
+
+  /// Cut this path to keep only the n first darts.
+  void cut(std::size_t n, bool update_isclosed=true)
+  {
+    if (n>=length()) return;
+    m_path.resize(n);
+    m_flip.resize(n);
+    if (update_isclosed) { update_is_closed(); }
+  }
+
+  /// copy all darts starting from begin and going to the dart before end
+  /// from this path to new_path.
+  void copy_rest_of_path(std::size_t begin, std::size_t end,
+                         Self& new_path)
+  {
+    CGAL_assertion(begin<=end);
+    CGAL_assertion(end<=length());
+    new_path.m_path.reserve(new_path.m_path.size()+end-begin+1);
+    while(begin!=end)
+    {
+      new_path.push_back(get_ith_dart(begin), get_ith_flip(begin), false);
+      ++begin;
+    }
+    update_is_closed();
+  }
+
+  /// Debugging method.
+  void display_failed_extention(const std::string& /*name_of_function*/)
+  {
+    // std::cout<<"Cant extend the path this way ("<<name_of_function<<")"
+    //          <<std::endl;
+  }
+
+  /// Extend the path straight positive.
+  /// @pre must be non empty.
+  void extend_straight_positive(std::size_t nb=1, bool update_isclosed=true)
+  {
+    if (is_empty() || nb==0)
+    { display_failed_extention("extend_straight_positive"); return; }
+
+    Dart_const_handle dh=back();
+    if(back_flip())
+    {
+      if (get_map().template is_free<2>(dh))
+      { display_failed_extention("extend_straight_positive"); return; }
+      else
+      { dh=get_map().opposite2(dh); }
+    }
+
+    for (unsigned int i=0; i<nb; ++i)
+    {
+      dh=get_map().next(dh);
+
+      if (get_map().template is_free<2>(dh))
+      { display_failed_extention("extend_straight_positive"); return; }
+      dh=get_map().next(get_map().opposite2(dh));
+
+      push_back(dh, false, false);
+    }
+
+    if (update_isclosed) { update_is_closed(); }
+  }
+
+  /// Extend the path straight negative.
+  /// @pre must be non empty.
+  void extend_straight_negative(std::size_t nb=1, bool update_isclosed=true)
+  {
+    if (is_empty() || nb==0)
+    { display_failed_extention("extend_straight_negative"); return; }
+
+    Dart_const_handle dh=back();
+    if(!back_flip())
+    {
+      if (get_map().template is_free<2>(dh))
+      { display_failed_extention("extend_straight_positive"); return; }
+      else
+      { dh=get_map().opposite2(dh); }
+    }
+
+    for (unsigned int i=0; i<nb; ++i)
+    {
+      dh=get_map().previous(dh);
+
+      if (get_map().template is_free<2>(dh))
+      { display_failed_extention("extend_straight_negative"); return; }
+      dh=get_map().previous(get_map().opposite2(dh));
+
+      push_back(dh, true, false);
+    }
+
+    if (update_isclosed) { update_is_closed(); }
+  }
+
+  /// Extend the path given a positive turn.
+  /// @pre must be non empty.
+  void extend_positive_turn(std::size_t nb=1, bool update_isclosed=true)
+  {
+    if (is_empty())
+    { display_failed_extention("extend_positive_turn"); return; }
+
+    if (nb==0)
+    {
+      push_back(back(), !back_flip(), update_isclosed);
+      return;
+    }
+
+    Dart_const_handle dh=back();
+    if(back_flip())
+    {
+      if (get_map().template is_free<2>(dh))
+      { display_failed_extention("extend_positive_turn"); return; }
+      else
+      { dh=get_map().opposite2(dh); }
+    }
+    dh=get_map().next(dh);
+
+    for (unsigned int i=1; i<nb; ++i)
+    {
+      if (get_map().template is_free<2>(dh))
+      { display_failed_extention("extend_positive_turn"); return; }
+      dh=get_map().next(get_map().opposite2(dh));
+    }
+
+    push_back(dh, false, update_isclosed);
+  }
+
+  /// Extend the path given a negative turn.
+  /// @pre must be non empty.
+  void extend_negative_turn(std::size_t nb=1, bool update_isclosed=true)
+  {
+    if (is_empty()) { display_failed_extention("extend_negative_turn"); return; }
+
+    if (nb==0)
+    {
+      push_back(back(), !back_flip(), update_isclosed);
+      return;
+    }
+
+    Dart_const_handle dh=back();
+    if(!back_flip())
+    {
+      if (get_map().template is_free<2>(dh))
+      { display_failed_extention("extend_negative_turn"); return; }
+      else
+      { dh=get_map().opposite2(dh); }
+    }
+    dh=get_map().previous(dh);
+
+    for (unsigned int i=1; i<nb; ++i)
+    {
+      if (get_map().template is_free<2>(dh))
+      { display_failed_extention("extend_negative_turn"); return; }
+      dh=get_map().previous(get_map().opposite2(dh));
+    }
+
+    push_back(dh, true, update_isclosed);
+  }
+
+  /// Initializes this path to a random starting path.
+  /// @pre must be empty.
+  bool initialize_random_starting_dart(CGAL::Random& random,
+                                       bool update_isclosed=true)
+  {
+    if (!is_empty() || get_map().is_empty()) { return false; }
+
+    // first select a random edge by taking the lower index of
+    // the two darts when it is not a boundary
+    typename Map::size_type index=static_cast<typename Map::size_type>
+      (random.get_int(0, static_cast<int>(get_map().darts().capacity())));
+    while (!get_map().darts().is_used(index) ||
+          (!get_map().template is_free<2>(get_map().dart_handle(index)) &&
+           get_map().dart_handle(index)>get_map().
+           opposite2(get_map().dart_handle(index))))
+    {
+      ++index;
+      if (index==get_map().darts().capacity()) index=0;
+    }
+
+    // second we take randomly one of the two darts of this edge
+    // (potentially with the help of a flip)
+    bool heads_or_tails=random.get_bool();
+    if (get_map().template is_free<2>(get_map().dart_handle(index)))
+    {
+      push_back(get_map().dart_handle(index), heads_or_tails, update_isclosed);
+    }
+    else
+    {
+      if (heads_or_tails)
+      { push_back(get_map().dart_handle(index), false, update_isclosed); }
+      else
+      { push_back(get_map().opposite2(get_map().dart_handle(index)),
+                  false, update_isclosed); }
+    }
+    return true;
+  }
+
+  /// Initializes this path to a random starting path.
+  /// @pre must be empty.
+  bool initialize_random_starting_dart(bool update_isclosed=true)
+  {
+    CGAL::Random& random=get_default_random();
+    return initialize_random_starting_dart(random, update_isclosed);
+  }
+
+  /// Extends this path with a random dart.
+  /// @pre must be non empty.
+  bool extend_path_randomly(CGAL::Random& random,
+                            bool allow_half_turn=true,
+                            bool update_isclosed=true)
+  {
+    if (is_empty())
+    { return initialize_random_starting_dart(random, update_isclosed); }
+
+    if(get_map().template is_free<1>(back()))
+    { return false; }
+
+    Dart_const_handle next_vertex;
+    if (back_flip())
+    { next_vertex=back(); }
+    else if (get_map().template is_free<2>(back()))
+    { next_vertex=get_map().next(back()); }
+    else
+    { next_vertex=get_map().opposite2(back()); }
+
+    std::vector<std::pair<Dart_const_handle, bool> > candidats;
+    for (auto it=get_map().template darts_of_cell<0>(next_vertex).begin(),
+           itend=get_map().template darts_of_cell<0>(next_vertex).end();
+           it!=itend; ++it )
+    {
+      if (back_flip() || !get_map().template is_free<2>(back()))
+      {
+        candidats.push_back(std::make_pair(it, false));
+        if (get_map().template is_free<2>(get_map().previous(it)))
+        { candidats.push_back
+              (std::make_pair(get_map().previous(it), true)); }
+      }
+      else
+      {
+        if (get_map().template is_free<2>(get_map().previous(it)))
+        { candidats.push_back
+              (std::make_pair(get_map().previous(it), true)); }
+        candidats.push_back(std::make_pair(it, false));
+      }
+    }
+    //candidats is now the list of all the darts that can be pushed back to
+    // the path (maybe with a flip) the first of them in the list is the
+    // opposite of back(), or back() itself if it is 2-free
+
+    std::size_t i=static_cast<std::size_t>
+      (random.get_int(allow_half_turn?0:1,static_cast<int>(candidats.size())));
+    auto it=candidats.begin();
+    for (std::size_t nb=0; nb<i; ++nb, ++it) {}
+    push_back(it->first, it->second, update_isclosed);
+    return true;
+  }
+
+  /// Extends this path with a random dart.
+  /// @pre must be non empty.
+  bool extend_path_randomly(bool allow_half_turn=false,
+                            bool update_isclosed=true)
+  {
+    CGAL::Random& random=get_default_random();
+    return extend_path_randomly(random, allow_half_turn, update_isclosed);
+  }
+
+  /// Generates a random path, with a number of darts >= length.
+  void generate_random_path(std::size_t length,
+                            CGAL::Random& random=get_default_random(),
+                            bool allow_half_turns=true,
+                            bool update_isclosed=true)
+  {
+    m_path.reserve(m_path.size()+length);
+    for (std::size_t i=0; i<length; ++i)
+    { extend_path_randomly(random, allow_half_turns, true); }
+    if (update_isclosed) { update_is_closed(); }
+  }
+
+  /// Generates a random path.
+  template<typename Path>
+  void generate_random_path(CGAL::Random& random,
+                            bool update_isclosed=true)
+  { generate_random_path(random.get_int(1, 10000),
+                         random, true, update_isclosed); }
+
+  /// Generates a random path.
+  template<typename Path>
+  void generate_random_path(std::size_t length,
+                            bool update_isclosed=true)
+  {
+    CGAL::Random& random=get_default_random();
+    generate_random_path(length, random, true, update_isclosed);
+  }
+
+  /// Generates a random path.
+  template<typename Path>
+  void generate_random_path(bool update_isclosed=true)
+  {
+    CGAL::Random& random=get_default_random();
+    generate_random_path(random, update_isclosed);
+  }
+
+  /// Generates a random closed path.
+  void generate_random_closed_path(std::size_t length, CGAL::Random& random)
+  {
+    m_path.reserve(m_path.size()+length);
+    std::size_t i=0;
+    while(i<length || !is_closed())
+    {
+      extend_path_randomly(random, true, true);
+      ++i;
+    }
+  }
+
+  /// Generates a random closed path.
+  void generate_random_closed_path(std::size_t length)
+  {
+    CGAL::Random& random=get_default_random();
+    generate_random_closed_path(length, random);
+  }
+
+  /// Generates a random closed path.
+  void generate_random_closed_path(CGAL::Random& random)
+  { generate_random_closed_path(random.get_int(1, 10000), random); }
+
+  /// Generates a random closed path.
+  void generate_random_closed_path()
+  {
+    CGAL::Random& random=get_default_random();
+    generate_random_closed_path(random.get_int(1, 10000), random);
+  }
+
+  /// Replace edge [i] by the path of darts along the face.
+  /// If this face does not exist (if it is a boundary) then replace the edge
+  /// by the face on the other side. Problem of complexity when used many times
+  /// (like in update_path_randomly).
+  bool push_around_face(std::size_t i, bool update_isclosed=true)
+  {
+    CGAL_assertion(i<length());
+
+    // It is not possible to push around a perforated face since it changes
+    // the homotopy of the path.
+    if (get_map().is_perforated(get_ith_dart(i))) { return false; }
+
+    Self p2(get_mesh());
+
+    // 1) We add in p2 the part of the path which is pushed.
+    if (get_ith_flip(i))
+    {
+      Dart_const_handle dh=get_map().next(get_ith_dart(i));
+      do
+      {
+        p2.push_back(dh, false, false);
+        dh=get_map().next(dh);
+      }
+      while(dh!=get_ith_dart(i));
+    }
+    else
+    {
+      Dart_const_handle dh=get_map().previous(get_ith_dart(i));
+      do
+      {
+        p2.push_back(dh, true, false);
+        dh=get_map().previous(dh);
+      }
+      while(dh!=get_ith_dart(i));
+    }
+
+    // 2) We copy the end of the path.
+    p2.m_path.reserve(p2.length()+length()-i);
+    for (std::size_t j=i+1; j<length(); ++j)
+    { p2.push_back(get_ith_dart(j), get_ith_flip(j), false); }
+
+    // 3) We cut this path to keep the first i darts.
+    cut(i, false);
+    m_path.reserve(length()+p2.length());
+    for (std::size_t j=0; j<p2.length(); ++j)
+    { push_back(p2[j], p2.get_ith_flip(j), false); }
+
+    if (update_isclosed) { update_is_closed(); }
+    return true;
+
+    //CGAL_assertion(is_valid());
+  }
+
+  /// Transform the current path by pushing some dart around faces.
+  /// At the end, the new path is homotopic to the original one.
+  void update_path_randomly(std::size_t nb, CGAL::Random& random,
+                            bool update_isclosed=true)
+  {
+    if (is_empty()) return;
+
+    for (unsigned int i=0; i<nb; ++i)
+    {
+      std::size_t dartn=static_cast<std::size_t>
+        (random.get_int(0, static_cast<int>(length())));
+      std::size_t j=dartn;
+      while(!push_around_face(dartn, false) && dartn!=j)
+      { ++dartn; }
+    }
+    if (update_isclosed) { update_is_closed(); }
+  }
+
+  void update_path_randomly(CGAL::Random& random,
+                            bool update_isclosed=true)
+  { update_path_randomly(random.get_int(0, 10000), update_isclosed); }
+
+  void update_path_randomly(std::size_t nb, bool update_isclosed=true)
+  {
+    CGAL::Random random;
+    update_path_randomly(nb, random, update_isclosed);
+  }
+
+  void update_path_randomly(bool update_isclosed=true)
+  {
+    CGAL::Random& random=get_default_random();
+    update_path_randomly(random, update_isclosed);
+  }
+
+  /// @return true iff the i-th dart of the path and the j-th dart of the other
+  /// are the same (taking into account the flips !)
+  bool are_same_step(std::size_t i, const Self& other, std::size_t j) const
+  {
+    if (get_ith_flip(i)==other.get_ith_flip(j))
+    { return get_ith_dart(i)==other[j]; }
+
+    if (get_map().template is_free<2>(get_ith_dart(i)) ||
+        get_map().template is_free<2>(other[j]))
+    { return false; }
+
+    return get_ith_dart(i)==get_map().opposite2(other[j]);
+  }
+
+  /// @return true if this path is equal to other path, identifying dart 0 of
+  ///          this path with dart start in other path.
+  bool are_same_paths_from(const Self& other, std::size_t start) const
+  {
+    CGAL_assertion(start==0 || start<length());
+    CGAL_assertion(is_closed() || start==0);
+    CGAL_assertion(length()==other.length() && is_closed()==other.is_closed());
+
+    for(std::size_t i=0; i<length(); ++i)
+    {
+      if (!are_same_step(i, other, start))
+      { return false; }
+      start=next_index(start);
+    }
+    return true;
+  }
+
+  /// @return true if this path is equal to other path. For closed paths, test
+  ///         all possible starting darts. Old quadratic version, new version
+  ///         (operator==) use linear version based on Knuth, Morris, Pratt
+  bool are_paths_equals(const Self& other) const
+  {
+    if (length()!=other.length() || is_closed()!=other.is_closed())
+    { return false; }
+
+    if (!is_closed())
+    { return are_same_paths_from(other, 0); }
+
+    for(std::size_t start=0; start<length(); ++start)
+    {
+      if (are_same_paths_from(other, start))
+      { return true; }
+    }
+    return false;
+  }
+
+  /// @return true if this path is equal to other path. For closed paths,
+  ///         equality is achieved whatever the first dart.
+  bool operator==(const Self& other) const
+  {
+    if (length()!=other.length() || is_closed()!=other.is_closed())
+    { return false; }
+
+    if (!is_closed())
+    { return are_same_paths_from(other, 0); }
+
+    Self pp1=*this;
+    pp1.simplify_flips();
+    Self pp2=other;
+    pp2.simplify_flips();
+    pp2+=pp2;
+    // Now we search if pp1 is a sub-motif of pp2 <=> *this==other
+
+    return boost::algorithm::knuth_morris_pratt_search(pp2.m_path.begin(),
+                                                       pp2.m_path.end(),
+                                                       pp1.m_path.begin(),
+                                                       pp1.m_path.end())
+#if BOOST_VERSION>=106200
+      .first
+#endif
+        !=pp2.m_path.end();
+  }
+  bool operator!=(const Self&  other) const
+  { return !(operator==(other)); }
+
+  /// @Return true if this path is equal to other path, identifying dart 0 of
+  ///          this path with dart start in other path. other path is given
+  ///          by index of its darts, in text format.
+  bool are_same_paths_from(const char* other, std::size_t start) const
+  {
+    CGAL_assertion(start==0 || start<length());
+    CGAL_assertion(is_closed() || start==0);
+
+    std::string sother(other);
+    std::istringstream iss(sother);
+    uint64_t nb;
+
+    for(std::size_t i=0; i<length(); ++i)
+    {
+      if (!iss.good())
+      { return false; }
+      iss>>nb;
+      if (nb!=m_map.darts().index(get_ith_dart(start)))
+      { return false; }
+      start=next_index(start);
+    }
+    iss>>nb;
+    if (iss.good())
+    { return false; } // There are more elements in other than in this path
+
+    return true;
+  }
+  /// @return true if this path is equal to other path. For closed paths, test
+  ///         all possible starting darts. other path is given by index of its
+  ///         darts, in text format.
+  bool operator==(const char* other) const
+  {
+    if (!is_closed())
+    { return are_same_paths_from(other, 0); }
+
+    for(std::size_t start=0; start<length(); ++start)
+    {
+      if (are_same_paths_from(other, start))
+      { return true; }
+    }
+    return false;
+  }
+  bool operator!=(const char*  other) const
+  { return !(operator==(other)); }
+
+
+  /// @return true iff the path is valid; i.e. a sequence of edges two by
+  ///              two adjacent.
+  bool is_valid(bool display_error=false) const
+  {
+    if (is_empty()) { return !is_closed(); } // an empty past is not closed
+    Dart_const_handle last_vertex;
+
+    for (unsigned int i=1; i<m_path.size(); ++i)
+    {
+      /* This assert is long if (!m_map.darts().owns(m_path[i]))
+      { return false; } */
+
+      if (m_path[i]==Map::null_handle || m_path[i]==m_map.null_dart_handle)
+      { return false; }
+
+      last_vertex=m_flip[i-1]?m_path[i-1]:get_map().next(m_path[i-1]);
+      if (last_vertex==Map::null_handle)
+      {
+        if (display_error)
+        { std::cout<<"Invalid path: one of the vertices doesn't exist"
+                   <<std::endl; }
+        return false;
+      }
+
+      if (!m_map.template belong_to_same_cell<0>
+          (m_flip[i]?get_map().next(m_path[i]):m_path[i], last_vertex))
+      {
+        if (display_error)
+        { std::cout<<"Invalid path: dart "<<i-1<<" and dart "<<i
+                   <<" are not adjacents"<<std::endl; }
+        return false;
+      }
+    }
+    last_vertex=back_flip()?back():get_map().next(back());
+    if (is_closed())
+    {
+      if (last_vertex==Map::null_handle)
+      {
+        if (display_error)
+        { std::cout<<"Invalid path: one of the vertices doesn't exist"
+                   <<std::endl; }
+        return false;
+      }
+      if (!m_map.template belong_to_same_cell<0>
+          (front_flip()?get_map().next(front()):front(), last_vertex))
+      {
+        if (display_error)
+        { std::cout<<"Invalid path: m_is_closed is true but the path is "
+                   <<"not closed"<<std::endl; }
+        return false;
+      }
+    }
+    else
+    {
+      if (last_vertex==Map::null_handle)
+      {
+        if (display_error)
+        { std::cout<<"Invalid path: one of the vertices doesn't exist"
+                   <<std::endl; }
+        return false;
+      }
+      if (m_map.template belong_to_same_cell<0>
+          (front_flip()?get_map().next(front()):front(), last_vertex))
+      {
+        if (display_error)
+        { std::cout<<"Invalid path: m_is_closed is false but the path "
+                   <<"is closed"<<std::endl; }
+        return false;
+      }
+    }
+
+    return true;
+  }
+
+  /// Update m_is_closed to true iff the path is closed (i.e. the second
+  ///   extremity of the last dart of the path is the same vertex than the one
+  ///   of the first dart of the path).
+  void update_is_closed()
+  {
+    // CGAL_assertion(is_valid());
+    if (is_empty()) { m_is_closed=false; }
+    else
+    {
+      Dart_const_handle
+          pend=m_flip.back()?back():m_map.other_extremity(back());
+      if (pend==Map::null_handle) { m_is_closed=false; }
+      else
+      {
+        Dart_const_handle
+            pbegin=m_flip[0]?m_map.other_extremity(m_path[0]):m_path[0];
+        m_is_closed=m_map.template belong_to_same_cell<0>(pbegin, pend);
+      }
+    }
+  }
+
+  /// @return true iff the path does not pass twice through a same edge
+  ///              or a same vertex.
+  bool is_simple() const
+  {
+    typename Map::size_type markvertex=m_map.get_new_mark();
+    typename Map::size_type markedge=m_map.get_new_mark();
+
+    bool res=true;
+    Dart_const_handle dh_vertex;
+    unsigned int i=0;
+    for (i=0; res && i<m_path.size(); ++i)
+    {
+      dh_vertex=m_flip[i]?get_map().next(m_path[i]):m_path[i];
+      if (m_map.is_marked(dh_vertex, markvertex)) { res=false; }
+      else { CGAL::mark_cell<Map, 0>(m_map, dh_vertex, markvertex); }
+
+      if (m_map.is_marked(m_path[i], markedge)) { res=false; }
+      else  { CGAL::mark_cell<Map, 1>(m_map, m_path[i], markedge); }
+    }
+
+    i=0;
+    while(m_map.number_of_marked_darts(markedge)>0 ||
+          m_map.number_of_marked_darts(markvertex)>0)
+    {
+      CGAL_assertion(i<m_path.size());
+      dh_vertex=m_flip[i]?get_map().next(m_path[i]):m_path[i];
+      if (m_map.is_marked(dh_vertex, markvertex))
+      { CGAL::unmark_cell<Map, 0>(m_map, dh_vertex, markvertex); }
+      if (m_map.is_marked(m_path[i], markedge))
+      { CGAL::unmark_cell<Map, 1>(m_map, m_path[i], markedge); }
+      ++i;
+    }
+
+    m_map.free_mark(markvertex);
+    m_map.free_mark(markedge);
+
+    return res;
+  }
+
+  /// Reverse the path (i.e. negate its orientation).
+  void reverse()
+  {
+    bool tmpbool;
+    for (unsigned int i=0; i<length()/2; ++i)
+    {
+      std::swap(m_path[i], m_path[length()-1-i]);
+      tmpbool=m_flip[i]; // Cannot swap in vector bool
+      m_flip[i]=m_flip[length()-1-i];
+      m_flip[length()-1-i]=tmpbool;
+    }
+
+    for (unsigned int i=0; i<length(); ++i)
+    { m_flip[i]=!m_flip[i]; }
+  }
+
+  /// If the given path is opened, close it by doing the same path that the
+  /// first one in reverse direction.
+  void close()
+  { // TODO follow shortest path ?
+    if (!is_closed())
+    {
+      for (int i=m_path.size()-1; i>=0; --i)
+      { m_path.push_back(m_path[i], !m_flip[i], false); }
+      m_is_closed=true;
+    }
+  }
+
+  /// @return the turn between dart number i and dart number i+1.
+  ///         (turn is position of the second edge in the cyclic ordering of
+  ///          edges starting from the first edge around the second extremity
+  ///          of the first dart)
+  std::size_t next_positive_turn(std::size_t i) const
+  {
+    // CGAL_assertion(is_valid());
+    CGAL_assertion(i<m_path.size());
+    CGAL_assertion (is_closed() || i<length()-1);
+
+    if ((get_ith_flip(i) && get_map().template is_free<2>(get_ith_dart(i))) ||
+        (get_next_flip(i) && get_map().template is_free<2>(get_next_dart(i))))
+    { return (std::numeric_limits<std::size_t>::max)(); }
+
+    return m_map.positive_turn(get_ith_real_dart(i),
+                               get_ith_real_dart(next_index(i)));
+  }
+
+  /// Same than next_positive_turn but turning in reverse orientation
+  /// around vertex.
+  std::size_t next_negative_turn(std::size_t i) const
+  {
+    // CGAL_assertion(is_valid());
+    CGAL_assertion(i<m_path.size());
+    CGAL_assertion (is_closed() || i<length()-1);
+
+    if ((!get_ith_flip(i) && get_map().template is_free<2>(get_ith_dart(i))) ||
+        (!get_next_flip(i) && get_map().template is_free<2>(get_next_dart(i))))
+    { return (std::numeric_limits<std::size_t>::max)(); }
+
+    return m_map.positive_turn(get_opposite_ith_real_dart(next_index(i)),
+                               get_opposite_ith_real_dart(i));
+  }
+
+  /// Computes all positive turns of this path.
+  std::vector<std::size_t> compute_positive_turns() const
+  {
+    std::vector<std::size_t> res;
+    if (is_empty()) return res;
+
+    std::size_t i;
+    for (i=0; i<m_path.size()-1; ++i)
+    { res.push_back(next_positive_turn(i)); }
+    if (is_closed())
+    { res.push_back(next_positive_turn(i)); }
+    return res;
+  }
+
+  /// Computes all negative turns of this path.
+  std::vector<std::size_t> compute_negative_turns() const
+  {
+    std::vector<std::size_t> res;
+    if (is_empty()) return res;
+
+    std::size_t i;
+    for (i=0; i<m_path.size()-1; ++i)
+    { res.push_back(next_negative_turn(i)); }
+    if (is_closed())
+    { res.push_back(next_negative_turn(i)); }
+    return res;
+  }
+
+  /// Computes all positive or negative turns of this path, depending on p.
+  std::vector<std::size_t> compute_turns(bool p) const
+  { return (p?compute_positive_turns():compute_negative_turns()); }
+
+  bool same_turns_from(const char* turns,
+                       const std::vector<std::size_t>& resplus,
+                       const std::vector<std::size_t>& resmoins,
+                       std::size_t start) const
+  {
+    CGAL_assertion(start==0 || start<resplus.size());
+    CGAL_assertion(resplus.size()==resmoins.size());
+
+    std::string sturns(turns);
+    std::istringstream iss(sturns);
+    int64_t nb;
+
+    for(std::size_t i=0; i<resplus.size(); ++i)
+    {
+      if (!iss.good())
+      { return false; }
+      iss>>nb;
+      if ((nb>=0 && resplus[start]!=static_cast<std::size_t>(nb)) ||
+          (nb<0 && resmoins[start]!=static_cast<std::size_t>(-nb)))
+      { return false; }
+
+      start=next_index(start);
+    }
+    iss>>nb;
+    if (iss.good())
+    { return false; } // There are more elements in turns than in res
+
+    return true;
+  }
+
+  bool same_turns(const char* turns) const
+  {
+    std::vector<std::size_t> resplus=compute_positive_turns();
+    std::vector<std::size_t> resmoins=compute_negative_turns();
+
+    if (!is_closed())
+    { return same_turns_from(turns, resplus, resmoins, 0); }
+
+    for (std::size_t start=0; start<length(); ++start)
+    {
+      if (same_turns_from(turns, resplus, resmoins, start))
+      { return true; }
+    }
+
+    return false;
+  }
+
+  void display_positive_turns() const
+  {
+    std::cout<<"+(";
+    std::vector<std::size_t> res=compute_positive_turns();
+    for (std::size_t i=0; i<res.size(); ++i)
+    { std::cout<<res[i]<<(i<res.size()-1?" ":""); }
+    std::cout<<")";
+  }
+
+  void display_negative_turns() const
+  {
+    std::cout<<"-(";
+    std::vector<std::size_t> res=compute_negative_turns();
+    for (std::size_t i=0; i<res.size(); ++i)
+    { std::cout<<res[i]<<(i<res.size()-1?" ":""); }
+    std::cout<<")";
+  }
+
+  void display_pos_and_neg_turns() const
+  {
+    display_positive_turns();
+    std::cout<<"  ";
+    display_negative_turns();
+  }
+
+  void display() const
+  {
+    for (std::size_t i=0; i<length(); ++i)
+    {
+      std::cout<<m_map.darts().index(get_ith_dart(i));
+      if (m_flip[i])
+      { std::cout<<"f"; }
+      if (i<length()-1) { std::cout<<" "; }
+    }
+     if (is_closed())
+     { std::cout<<" c "; } //<<m_map.darts().index(get_ith_dart(0)); }
+  }
+
+  friend std::ostream& operator<<(std::ostream& os, const Self& p)
+  {
+    p.display();
+    return os;
+  }
+
+protected:
+  const typename Get_map<Mesh, Mesh>::storage_type m_map; // The underlying map
+  std::vector<Dart_const_handle> m_path; /// The sequence of darts
+  bool m_is_closed;                      /// True iff the path is a cycle
+  std::vector<bool> m_flip;              /// The sequence of flips
+};
+
+} // namespace Surface_mesh_topology
+} // namespace CGAL
+
+#endif // CGAL_PATH_ON_SURFACE_H //
+// EOF //
diff -pruN 5.0.3-1/include/CGAL/pca_estimate_normals.h 5.1-1/include/CGAL/pca_estimate_normals.h
--- 5.0.3-1/include/CGAL/pca_estimate_normals.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/pca_estimate_normals.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_processing_3/include/CGAL/pca_estimate_normals.h $
-// $Id: pca_estimate_normals.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/pca_estimate_normals.h $
+// $Id: pca_estimate_normals.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s) : Pierre Alliez and Laurent Saboret
@@ -18,9 +18,9 @@
 
 #include <CGAL/IO/trace.h>
 #include <CGAL/Dimension.h>
-#include <CGAL/Search_traits_3.h>
-#include <CGAL/Orthogonal_k_neighbor_search.h>
-#include <CGAL/Point_set_processing_3/internal/neighbor_query.h>
+#include <CGAL/Point_set_processing_3/internal/Neighbor_query.h>
+#include <CGAL/Point_set_processing_3/internal/Callback_wrapper.h>
+#include <CGAL/for_each.h>
 #include <CGAL/linear_least_squares_fitting_3.h>
 #include <CGAL/property_map.h>
 #include <CGAL/point_set_processing_assertions.h>
@@ -33,13 +33,6 @@
 #include <iterator>
 #include <list>
 
-#ifdef CGAL_LINKED_WITH_TBB
-#include <CGAL/Point_set_processing_3/internal/Parallel_callback.h>
-#include <tbb/parallel_for.h>
-#include <tbb/blocked_range.h>
-#include <tbb/scalable_allocator.h>
-#endif // CGAL_LINKED_WITH_TBB
-
 namespace CGAL {
 
 
@@ -58,22 +51,20 @@ namespace internal {
 /// @tparam Tree KD-tree.
 ///
 /// @return Computed normal. Orientation is random.
-template < typename Kernel,
-           typename Tree
->
-typename Kernel::Vector_3
-pca_estimate_normal(const typename Kernel::Point_3& query, ///< point to compute the normal at
-                    const Tree& tree, ///< KD-tree
+template <typename NeighborQuery>
+typename NeighborQuery::Kernel::Vector_3
+pca_estimate_normal(const typename NeighborQuery::Kernel::Point_3& query, ///< point to compute the normal at
+                    const NeighborQuery& neighbor_query, ///< KD-tree
                     unsigned int k, ///< number of neighbors
-                    typename Kernel::FT neighbor_radius)
+                    typename NeighborQuery::Kernel::FT neighbor_radius)
 {
   // basic geometric types
+  typedef typename NeighborQuery::Kernel Kernel;
   typedef typename Kernel::Point_3  Point;
   typedef typename Kernel::Plane_3  Plane;
 
   std::vector<Point> points;
-  CGAL::Point_set_processing_3::internal::neighbor_query
-    (query, tree, k, neighbor_radius, points);
+  neighbor_query.get_points (query, k, neighbor_radius, std::back_inserter(points));
 
   // performs plane fitting by point-based PCA
   Plane plane;
@@ -83,48 +74,6 @@ pca_estimate_normal(const typename Kerne
   return plane.orthogonal_vector();
 }
 
-
-#ifdef CGAL_LINKED_WITH_TBB
-  template <typename Kernel, typename Tree>
-  class PCA_estimate_normals {
-    typedef typename Kernel::FT FT;
-    typedef typename Kernel::Point_3 Point;
-    typedef typename Kernel::Vector_3 Vector;
-    const Tree& tree;
-    const unsigned int k;
-    const FT neighbor_radius;
-    const std::vector<Point>& input;
-    std::vector<Vector>& output;
-    cpp11::atomic<std::size_t>& advancement;
-    cpp11::atomic<bool>& interrupted;
-
-  public:
-    PCA_estimate_normals(Tree& tree, unsigned int k, FT neighbor_radius,
-                         std::vector<Point>& points,
-                         std::vector<Vector>& output,
-                     cpp11::atomic<std::size_t>& advancement,
-                     cpp11::atomic<bool>& interrupted)
-      : tree(tree), k (k), neighbor_radius (neighbor_radius)
-      , input (points), output (output)
-      , advancement (advancement)
-      , interrupted (interrupted)
-    { }
-
-    void operator()(const tbb::blocked_range<std::size_t>& r) const
-    {
-      for( std::size_t i = r.begin(); i != r.end(); ++i)
-      {
-        if (interrupted)
-          break;
-        output[i] = CGAL::internal::pca_estimate_normal<Kernel,Tree>(input[i], tree, k, neighbor_radius);
-        ++ advancement;
-      }
-    }
-
-  };
-#endif // CGAL_LINKED_WITH_TBB
-
-
 } /* namespace internal */
 /// \endcond
 
@@ -142,39 +91,60 @@ pca_estimate_normal(const typename Kerne
 
    \pre `k >= 2`
 
-   \tparam ConcurrencyTag enables sequential versus parallel algorithm.
-   Possible values are `Sequential_tag`
-   and `Parallel_tag`.
+   \tparam ConcurrencyTag enables sequential versus parallel algorithm. Possible values are `Sequential_tag`,
+                          `Parallel_tag`, and `Parallel_if_available_tag`.
    \tparam PointRange is a model of `Range`. The value type of
    its iterator is the key type of the named parameter `point_map`.
 
    \param points input point range.
    \param k number of neighbors
-   \param np optional sequence of \ref psp_namedparameters "Named Parameters" among the ones listed below.
+   \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
-   \cgalNamedParamsBegin \cgalParamBegin{point_map} a model of
-     `ReadablePropertyMap` with value type `geom_traits::Point_3`.  If
-     this parameter is omitted,
-     `CGAL::Identity_property_map<geom_traits::Point_3>` is
-     used.\cgalParamEnd \cgalParamBegin{normal_map} a model of
-     `WritablePropertyMap` with value type
-     `geom_traits::Vector_3`.\cgalParamEnd
-     \cgalParamBegin{neighbor_radius} spherical neighborhood
-     radius. If provided, the neighborhood of a query point is
-     computed with a fixed spherical radius instead of a fixed number
-     of neighbors. In that case, the parameter `k` is used as a limit
-     on the number of points returned by each spherical query (to
-     avoid overly large number of points in high density areas). If no
-     limit is wanted, use `k=0`.\cgalParamEnd
-     \cgalParamBegin{callback} an instance of
-     `std::function<bool(double)>`. It is called regularly when the
-     algorithm is running: the current advancement (between 0. and 1.)
-     is passed as parameter. If it returns `true`, then the algorithm
-     continues its execution normally; if it returns `false`, the
-     algorithm is stopped and the remaining normals are left
-     unchanged.\cgalParamEnd \cgalParamBegin{geom_traits} an instance
-     of a geometric traits class, model of `Kernel`\cgalParamEnd
-     \cgalNamedParamsEnd
+   \cgalNamedParamsBegin
+     \cgalParamNBegin{point_map}
+       \cgalParamDescription{a property map associating points to the elements of the point set `points`}
+       \cgalParamType{a model of `ReadablePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange` and whose value type is `geom_traits::Point_3`}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{normal_map}
+       \cgalParamDescription{a property map associating normals to the elements of the point set `points`}
+       \cgalParamType{a model of `WritablePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange` and whose value type is `geom_traits::Vector_3`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{neighbor_radius}
+       \cgalParamDescription{the spherical neighborhood radius}
+       \cgalParamType{floating scalar value}
+       \cgalParamDefault{`0` (no limit)}
+       \cgalParamExtra{If provided, the neighborhood of a query point is computed with a fixed spherical
+                       radius instead of a fixed number of neighbors. In that case, the parameter
+                       `k` is used as a limit on the number of points returned by each spherical
+                       query (to avoid overly large number of points in high density areas).}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{callback}
+       \cgalParamDescription{a mechanism to get feedback on the advancement of the algorithm
+                             while it's running and to interrupt it if needed}
+       \cgalParamType{an instance of `std::function<bool(double)>`.}
+       \cgalParamDefault{unused}
+       \cgalParamExtra{It is called regularly when the
+                       algorithm is running: the current advancement (between 0. and
+                       1.) is passed as parameter. If it returns `true`, then the
+                       algorithm continues its execution normally; if it returns
+                       `false`, the algorithm is stopped and the remaining normals are left unchanged.}
+       \cgalParamExtra{The callback will be copied and therefore needs to be lightweight.}
+       \cgalParamExtra{When `CGAL::Parallel_tag` is used, the `callback` mechanism is called asynchronously
+                       on a separate thread and shouldn't access or modify the variables that are parameters of the algorithm.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{geom_traits}
+       \cgalParamDescription{an instance of a geometric traits class}
+       \cgalParamType{a model of `Kernel`}
+       \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+     \cgalParamNEnd
+   \cgalNamedParamsEnd
 */
 template <typename ConcurrencyTag,
           typename PointRange,
@@ -192,7 +162,7 @@ pca_estimate_normals(
   CGAL_TRACE("Calls pca_estimate_normals()\n");
 
   // basic geometric types
-  typedef typename Point_set_processing_3::GetPointMap<PointRange, NamedParameters>::type PointMap;
+  typedef typename CGAL::GetPointMap<PointRange, NamedParameters>::type PointMap;
   typedef typename Point_set_processing_3::GetNormalMap<PointRange, NamedParameters>::type NormalMap;
   typedef typename Point_set_processing_3::GetK<PointRange, NamedParameters>::Kernel Kernel;
   typedef typename Kernel::FT FT;
@@ -201,21 +171,18 @@ pca_estimate_normals(
                               typename Point_set_processing_3::GetNormalMap<PointRange, NamedParameters>::NoMap>::value),
                             "Error: no normal map");
 
-  PointMap point_map = choose_parameter(get_parameter(np, internal_np::point_map), PointMap());
-  NormalMap normal_map = choose_parameter(get_parameter(np, internal_np::normal_map), NormalMap());
+  PointMap point_map = choose_parameter<PointMap>(get_parameter(np, internal_np::point_map));
+  NormalMap normal_map = choose_parameter<NormalMap>(get_parameter(np, internal_np::normal_map));
   FT neighbor_radius = choose_parameter(get_parameter(np, internal_np::neighbor_radius), FT(0));
   const std::function<bool(double)>& callback = choose_parameter(get_parameter(np, internal_np::callback),
                                                                  std::function<bool(double)>());
 
-  typedef typename Kernel::Point_3 Point;
-
   // Input points types
-  typedef typename boost::property_traits<NormalMap>::value_type Vector;
+  typedef typename PointRange::iterator iterator;
+  typedef typename iterator::value_type value_type;
 
   // types for K nearest neighbors search structure
-  typedef typename CGAL::Search_traits_3<Kernel> Tree_traits;
-  typedef typename CGAL::Orthogonal_k_neighbor_search<Tree_traits> Neighbor_search;
-  typedef typename Neighbor_search::Tree Tree;
+  typedef Point_set_processing_3::internal::Neighbor_query<Kernel, PointRange&, PointMap> Neighbor_query;
 
   // precondition: at least one element in the container.
   // to fix: should have at least three distinct points
@@ -228,59 +195,33 @@ pca_estimate_normals(
   std::size_t memory = CGAL::Memory_sizer().virtual_size(); CGAL_TRACE("  %ld Mb allocated\n", memory>>20);
   CGAL_TRACE("  Creates KD-tree\n");
 
-  typename PointRange::iterator it;
-
-  // Instanciate a KD-tree search.
-  // Note: We have to convert each input iterator to Point_3.
-  std::vector<Point> kd_tree_points;
-  for(it = points.begin(); it != points.end(); it++)
-    kd_tree_points.push_back(get(point_map, *it));
-  Tree tree(kd_tree_points.begin(), kd_tree_points.end());
+  Neighbor_query neighbor_query (points, point_map);
 
   memory = CGAL::Memory_sizer().virtual_size(); CGAL_TRACE("  %ld Mb allocated\n", memory>>20);
   CGAL_TRACE("  Computes normals\n");
 
-  // iterate over input points, compute and output normal
-  // vectors (already normalized)
-#ifndef CGAL_LINKED_WITH_TBB
-  CGAL_static_assertion_msg (!(boost::is_convertible<ConcurrencyTag, Parallel_tag>::value),
-                             "Parallel_tag is enabled but TBB is unavailable.");
-#else
-  if (boost::is_convertible<ConcurrencyTag,Parallel_tag>::value)
-    {
-      Point_set_processing_3::internal::Parallel_callback
-        parallel_callback (callback, kd_tree_points.size());
-
-      std::vector<Vector> normals (kd_tree_points.size (),
-                                   CGAL::NULL_VECTOR);
-      CGAL::internal::PCA_estimate_normals<Kernel, Tree>
-        f (tree, k, neighbor_radius, kd_tree_points, normals,
-           parallel_callback.advancement(),
-           parallel_callback.interrupted());
-      tbb::parallel_for(tbb::blocked_range<size_t>(0, kd_tree_points.size ()), f);
-      unsigned int i = 0;
-      for(it = points.begin(); it != points.end(); ++ it, ++ i)
-        if (normals[i] != CGAL::NULL_VECTOR)
-          put (normal_map, *it, normals[i]);
-
-      parallel_callback.join();
-    }
-  else
-#endif
-    {
-      std::size_t nb = 0;
-      for(it = points.begin(); it != points.end(); it++, ++ nb)
-        {
-          Vector normal = internal::pca_estimate_normal<Kernel,Tree>(
-                                                                     get(point_map,*it),
-                                                                     tree,
-                                                                     k, neighbor_radius);
-
-          put(normal_map, *it, normal); // normal_map[it] = normal
-          if (callback && !callback ((nb+1) / double(kd_tree_points.size())))
-            break;
-        }
-    }
+  std::size_t nb_points = points.size();
+
+  Point_set_processing_3::internal::Callback_wrapper<ConcurrencyTag>
+    callback_wrapper (callback, nb_points);
+
+  CGAL::for_each<ConcurrencyTag>
+    (points,
+     [&](value_type& vt)
+     {
+       if (callback_wrapper.interrupted())
+         return false;
+
+       put (normal_map, vt,
+            CGAL::internal::pca_estimate_normal
+            (get(point_map, vt), neighbor_query, k, neighbor_radius));
+
+       ++ callback_wrapper.advancement();
+
+       return true;
+     });
+
+  callback_wrapper.join();
 
   memory = CGAL::Memory_sizer().virtual_size(); CGAL_TRACE("  %ld Mb allocated\n", memory>>20);
   CGAL_TRACE("End of pca_estimate_normals()\n");
diff -pruN 5.0.3-1/include/CGAL/PCA_util_Eigen.h 5.1-1/include/CGAL/PCA_util_Eigen.h
--- 5.0.3-1/include/CGAL/PCA_util_Eigen.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/PCA_util_Eigen.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Principal_component_analysis/include/CGAL/PCA_util_Eigen.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Principal_component_analysis/include/CGAL/PCA_util_Eigen.h $
 // $Id: PCA_util_Eigen.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/PCA_util.h 5.1-1/include/CGAL/PCA_util.h
--- 5.0.3-1/include/CGAL/PCA_util.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/PCA_util.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Principal_component_analysis/include/CGAL/PCA_util.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Principal_component_analysis/include/CGAL/PCA_util.h $
 // $Id: PCA_util.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Periodic_2_Delaunay_triangulation_2.h 5.1-1/include/CGAL/Periodic_2_Delaunay_triangulation_2.h
--- 5.0.3-1/include/CGAL/Periodic_2_Delaunay_triangulation_2.h	2020-08-01 19:02:38.000000000 +0000
+++ 5.1-1/include/CGAL/Periodic_2_Delaunay_triangulation_2.h	2020-09-08 12:15:19.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_2_triangulation_2/include/CGAL/Periodic_2_Delaunay_triangulation_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_2_triangulation_2/include/CGAL/Periodic_2_Delaunay_triangulation_2.h $
 // $Id: Periodic_2_Delaunay_triangulation_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Periodic_2_Delaunay_triangulation_traits_2.h 5.1-1/include/CGAL/Periodic_2_Delaunay_triangulation_traits_2.h
--- 5.0.3-1/include/CGAL/Periodic_2_Delaunay_triangulation_traits_2.h	2020-08-01 19:02:38.000000000 +0000
+++ 5.1-1/include/CGAL/Periodic_2_Delaunay_triangulation_traits_2.h	2020-09-08 12:15:19.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_2_triangulation_2/include/CGAL/Periodic_2_Delaunay_triangulation_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_2_triangulation_2/include/CGAL/Periodic_2_Delaunay_triangulation_traits_2.h $
 // $Id: Periodic_2_Delaunay_triangulation_traits_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Periodic_2_offset_2.h 5.1-1/include/CGAL/Periodic_2_offset_2.h
--- 5.0.3-1/include/CGAL/Periodic_2_offset_2.h	2020-08-01 19:02:38.000000000 +0000
+++ 5.1-1/include/CGAL/Periodic_2_offset_2.h	2020-09-08 12:15:19.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_2_triangulation_2/include/CGAL/Periodic_2_offset_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_2_triangulation_2/include/CGAL/Periodic_2_offset_2.h $
 // $Id: Periodic_2_offset_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Periodic_2_triangulation_2.h 5.1-1/include/CGAL/Periodic_2_triangulation_2.h
--- 5.0.3-1/include/CGAL/Periodic_2_triangulation_2.h	2020-08-01 19:02:38.000000000 +0000
+++ 5.1-1/include/CGAL/Periodic_2_triangulation_2.h	2020-09-08 12:15:19.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_2_triangulation_2/include/CGAL/Periodic_2_triangulation_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_2_triangulation_2/include/CGAL/Periodic_2_triangulation_2.h $
 // $Id: Periodic_2_triangulation_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Periodic_2_triangulation_dummy_12.h 5.1-1/include/CGAL/Periodic_2_triangulation_dummy_12.h
--- 5.0.3-1/include/CGAL/Periodic_2_triangulation_dummy_12.h	2020-08-01 19:02:38.000000000 +0000
+++ 5.1-1/include/CGAL/Periodic_2_triangulation_dummy_12.h	2020-09-08 12:15:19.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_2_triangulation_2/include/CGAL/Periodic_2_triangulation_dummy_12.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_2_triangulation_2/include/CGAL/Periodic_2_triangulation_dummy_12.h $
 // $Id: Periodic_2_triangulation_dummy_12.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Periodic_2_triangulation_face_base_2.h 5.1-1/include/CGAL/Periodic_2_triangulation_face_base_2.h
--- 5.0.3-1/include/CGAL/Periodic_2_triangulation_face_base_2.h	2020-08-01 19:02:38.000000000 +0000
+++ 5.1-1/include/CGAL/Periodic_2_triangulation_face_base_2.h	2020-09-08 12:15:19.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_2_triangulation_2/include/CGAL/Periodic_2_triangulation_face_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_2_triangulation_2/include/CGAL/Periodic_2_triangulation_face_base_2.h $
 // $Id: Periodic_2_triangulation_face_base_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Periodic_2_triangulation_hierarchy_2.h 5.1-1/include/CGAL/Periodic_2_triangulation_hierarchy_2.h
--- 5.0.3-1/include/CGAL/Periodic_2_triangulation_hierarchy_2.h	2020-08-01 19:02:38.000000000 +0000
+++ 5.1-1/include/CGAL/Periodic_2_triangulation_hierarchy_2.h	2020-09-08 12:15:19.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_2_triangulation_2/include/CGAL/Periodic_2_triangulation_hierarchy_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_2_triangulation_2/include/CGAL/Periodic_2_triangulation_hierarchy_2.h $
 // $Id: Periodic_2_triangulation_hierarchy_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Periodic_2_triangulation_iterators_2.h 5.1-1/include/CGAL/Periodic_2_triangulation_iterators_2.h
--- 5.0.3-1/include/CGAL/Periodic_2_triangulation_iterators_2.h	2020-08-01 19:02:38.000000000 +0000
+++ 5.1-1/include/CGAL/Periodic_2_triangulation_iterators_2.h	2020-09-08 12:15:19.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_2_triangulation_2/include/CGAL/Periodic_2_triangulation_iterators_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_2_triangulation_2/include/CGAL/Periodic_2_triangulation_iterators_2.h $
 // $Id: Periodic_2_triangulation_iterators_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Periodic_2_triangulation_traits_2.h 5.1-1/include/CGAL/Periodic_2_triangulation_traits_2.h
--- 5.0.3-1/include/CGAL/Periodic_2_triangulation_traits_2.h	2020-08-01 19:02:38.000000000 +0000
+++ 5.1-1/include/CGAL/Periodic_2_triangulation_traits_2.h	2020-09-08 12:15:19.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_2_triangulation_2/include/CGAL/Periodic_2_triangulation_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_2_triangulation_2/include/CGAL/Periodic_2_triangulation_traits_2.h $
 // $Id: Periodic_2_triangulation_traits_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Periodic_2_triangulation_vertex_base_2.h 5.1-1/include/CGAL/Periodic_2_triangulation_vertex_base_2.h
--- 5.0.3-1/include/CGAL/Periodic_2_triangulation_vertex_base_2.h	2020-08-01 19:02:38.000000000 +0000
+++ 5.1-1/include/CGAL/Periodic_2_triangulation_vertex_base_2.h	2020-09-08 12:15:19.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_2_triangulation_2/include/CGAL/Periodic_2_triangulation_vertex_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_2_triangulation_2/include/CGAL/Periodic_2_triangulation_vertex_base_2.h $
 // $Id: Periodic_2_triangulation_vertex_base_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Periodic_3_Delaunay_triangulation_3.h 5.1-1/include/CGAL/Periodic_3_Delaunay_triangulation_3.h
--- 5.0.3-1/include/CGAL/Periodic_3_Delaunay_triangulation_3.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/Periodic_3_Delaunay_triangulation_3.h	2020-09-08 12:15:20.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_triangulation_3/include/CGAL/Periodic_3_Delaunay_triangulation_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_triangulation_3/include/CGAL/Periodic_3_Delaunay_triangulation_3.h $
 // $Id: Periodic_3_Delaunay_triangulation_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Periodic_3_Delaunay_triangulation_traits_3.h 5.1-1/include/CGAL/Periodic_3_Delaunay_triangulation_traits_3.h
--- 5.0.3-1/include/CGAL/Periodic_3_Delaunay_triangulation_traits_3.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/Periodic_3_Delaunay_triangulation_traits_3.h	2020-09-08 12:15:20.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_triangulation_3/include/CGAL/Periodic_3_Delaunay_triangulation_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_triangulation_3/include/CGAL/Periodic_3_Delaunay_triangulation_traits_3.h $
 // $Id: Periodic_3_Delaunay_triangulation_traits_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Periodic_3_function_wrapper.h 5.1-1/include/CGAL/Periodic_3_function_wrapper.h
--- 5.0.3-1/include/CGAL/Periodic_3_function_wrapper.h	2020-08-01 19:02:38.000000000 +0000
+++ 5.1-1/include/CGAL/Periodic_3_function_wrapper.h	2020-09-08 12:15:20.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_mesh_3/include/CGAL/Periodic_3_function_wrapper.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_mesh_3/include/CGAL/Periodic_3_function_wrapper.h $
 // $Id: Periodic_3_function_wrapper.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Periodic_3_mesh_3/config.h 5.1-1/include/CGAL/Periodic_3_mesh_3/config.h
--- 5.0.3-1/include/CGAL/Periodic_3_mesh_3/config.h	2020-08-01 19:02:38.000000000 +0000
+++ 5.1-1/include/CGAL/Periodic_3_mesh_3/config.h	2020-09-08 12:15:20.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_mesh_3/include/CGAL/Periodic_3_mesh_3/config.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_mesh_3/include/CGAL/Periodic_3_mesh_3/config.h $
 // $Id: config.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Periodic_3_mesh_3/IO/File_medit.h 5.1-1/include/CGAL/Periodic_3_mesh_3/IO/File_medit.h
--- 5.0.3-1/include/CGAL/Periodic_3_mesh_3/IO/File_medit.h	2020-08-01 19:02:38.000000000 +0000
+++ 5.1-1/include/CGAL/Periodic_3_mesh_3/IO/File_medit.h	2020-09-08 12:15:20.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_mesh_3/include/CGAL/Periodic_3_mesh_3/IO/File_medit.h $
-// $Id: File_medit.h 5b2d46c 2020-04-21T10:03:00+02:00 Maxime Gimeno
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_mesh_3/include/CGAL/Periodic_3_mesh_3/IO/File_medit.h $
+// $Id: File_medit.h e9d41d7 2020-04-21T10:03:00+02:00 Maxime Gimeno
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Mikhail Bogdanov
diff -pruN 5.0.3-1/include/CGAL/Periodic_3_mesh_3/Protect_edges_sizing_field.h 5.1-1/include/CGAL/Periodic_3_mesh_3/Protect_edges_sizing_field.h
--- 5.0.3-1/include/CGAL/Periodic_3_mesh_3/Protect_edges_sizing_field.h	2020-08-01 19:02:38.000000000 +0000
+++ 5.1-1/include/CGAL/Periodic_3_mesh_3/Protect_edges_sizing_field.h	2020-09-08 12:15:20.000000000 +0000
@@ -4,8 +4,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_mesh_3/include/CGAL/Periodic_3_mesh_3/Protect_edges_sizing_field.h $
-// $Id: Protect_edges_sizing_field.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_mesh_3/include/CGAL/Periodic_3_mesh_3/Protect_edges_sizing_field.h $
+// $Id: Protect_edges_sizing_field.h 53d4c9b 2019-10-28T11:29:08+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Stephane Tayeb, Laurent Rineau, Mael Rouxel-Labbé
@@ -40,13 +40,12 @@
 #include <CGAL/Mesh_3/Triangulation_helpers.h>
 
 #include <CGAL/enum.h>
-#include <CGAL/Has_timestamp.h>
-#include <CGAL/Hash_handles_with_or_without_timestamps.h>
 #include <CGAL/internal/Has_member_visited.h>
 #include <CGAL/iterator.h>
 #include <CGAL/number_utils.h>
 #include <CGAL/Periodic_3_Delaunay_triangulation_traits_3.h>
 #include <CGAL/Periodic_3_Delaunay_triangulation_3.h>
+#include <CGAL/Time_stamper.h>
 
 #include <CGAL/boost/iterator/transform_iterator.hpp>
 
diff -pruN 5.0.3-1/include/CGAL/Periodic_3_mesh_triangulation_3.h 5.1-1/include/CGAL/Periodic_3_mesh_triangulation_3.h
--- 5.0.3-1/include/CGAL/Periodic_3_mesh_triangulation_3.h	2020-08-01 19:02:38.000000000 +0000
+++ 5.1-1/include/CGAL/Periodic_3_mesh_triangulation_3.h	2020-09-08 12:15:20.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_mesh_3/include/CGAL/Periodic_3_mesh_triangulation_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_mesh_3/include/CGAL/Periodic_3_mesh_triangulation_3.h $
 // $Id: Periodic_3_mesh_triangulation_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Periodic_3_offset_3.h 5.1-1/include/CGAL/Periodic_3_offset_3.h
--- 5.0.3-1/include/CGAL/Periodic_3_offset_3.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/Periodic_3_offset_3.h	2020-09-08 12:15:20.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_triangulation_3/include/CGAL/Periodic_3_offset_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_triangulation_3/include/CGAL/Periodic_3_offset_3.h $
 // $Id: Periodic_3_offset_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Periodic_3_regular_triangulation_3.h 5.1-1/include/CGAL/Periodic_3_regular_triangulation_3.h
--- 5.0.3-1/include/CGAL/Periodic_3_regular_triangulation_3.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/Periodic_3_regular_triangulation_3.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_triangulation_3/include/CGAL/Periodic_3_regular_triangulation_3.h $
-// $Id: Periodic_3_regular_triangulation_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_triangulation_3/include/CGAL/Periodic_3_regular_triangulation_3.h $
+// $Id: Periodic_3_regular_triangulation_3.h 1003046 2020-03-12T20:58:44+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Monique Teillaud <Monique.Teillaud@inria.fr>
@@ -27,13 +27,13 @@
 #include <CGAL/Regular_triangulation_cell_base_3.h>
 
 #include <CGAL/enum.h>
-#include <CGAL/internal/boost/function_property_map.hpp>
 #include <CGAL/internal/Has_nested_type_Bare_point.h>
 #include <CGAL/spatial_sort.h>
 #include <CGAL/utility.h>
 
 #include <boost/mpl/if.hpp>
 #include <boost/mpl/identity.hpp>
+#include <boost/property_map/function_property_map.hpp>
 #include <boost/unordered_set.hpp>
 
 #include <cstdlib>
@@ -498,12 +498,12 @@ public:
     // Spatial sorting can only be applied to bare points, so we need an adaptor
     typedef typename Geom_traits::Construct_point_3 Construct_point_3;
     typedef typename boost::result_of<const Construct_point_3(const Weighted_point&)>::type Ret;
-    typedef CGAL::internal::boost_::function_property_map<Construct_point_3, Weighted_point, Ret> fpmap;
+    typedef boost::function_property_map<Construct_point_3, Weighted_point, Ret> fpmap;
     typedef CGAL::Spatial_sort_traits_adapter_3<Geom_traits, fpmap> Search_traits_3;
 
     spatial_sort(pbegin, points.end(),
                  Search_traits_3(
-                   CGAL::internal::boost_::make_function_property_map<Weighted_point, Ret, Construct_point_3>(
+                   boost::make_function_property_map<Weighted_point, Ret, Construct_point_3>(
                        geom_traits().construct_point_3_object()), geom_traits()));
 
     Cell_handle hint;
diff -pruN 5.0.3-1/include/CGAL/Periodic_3_regular_triangulation_traits_3.h 5.1-1/include/CGAL/Periodic_3_regular_triangulation_traits_3.h
--- 5.0.3-1/include/CGAL/Periodic_3_regular_triangulation_traits_3.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/Periodic_3_regular_triangulation_traits_3.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_triangulation_3/include/CGAL/Periodic_3_regular_triangulation_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_triangulation_3/include/CGAL/Periodic_3_regular_triangulation_traits_3.h $
 // $Id: Periodic_3_regular_triangulation_traits_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Periodic_3_triangulation_3.h 5.1-1/include/CGAL/Periodic_3_triangulation_3.h
--- 5.0.3-1/include/CGAL/Periodic_3_triangulation_3.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/Periodic_3_triangulation_3.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_triangulation_3/include/CGAL/Periodic_3_triangulation_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_triangulation_3/include/CGAL/Periodic_3_triangulation_3.h $
 // $Id: Periodic_3_triangulation_3.h d10359d 2020-03-19T16:27:55+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/periodic_3_triangulation_3_io.h 5.1-1/include/CGAL/periodic_3_triangulation_3_io.h
--- 5.0.3-1/include/CGAL/periodic_3_triangulation_3_io.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/periodic_3_triangulation_3_io.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_triangulation_3/include/CGAL/periodic_3_triangulation_3_io.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_triangulation_3/include/CGAL/periodic_3_triangulation_3_io.h $
 // $Id: periodic_3_triangulation_3_io.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Periodic_3_triangulation_ds_cell_base_3.h 5.1-1/include/CGAL/Periodic_3_triangulation_ds_cell_base_3.h
--- 5.0.3-1/include/CGAL/Periodic_3_triangulation_ds_cell_base_3.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/Periodic_3_triangulation_ds_cell_base_3.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_triangulation_3/include/CGAL/Periodic_3_triangulation_ds_cell_base_3.h $
-// $Id: Periodic_3_triangulation_ds_cell_base_3.h 627a7b2 2020-04-27T17:08:35+02:00 Maxime Gimeno
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_triangulation_3/include/CGAL/Periodic_3_triangulation_ds_cell_base_3.h $
+// $Id: Periodic_3_triangulation_ds_cell_base_3.h dcc4fb1 2020-05-20T09:46:26+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -233,7 +233,7 @@ public:
 
   // For use by Compact_container.
   void * for_compact_container() const { return N[0].for_compact_container(); }
-  void * & for_compact_container()     { return N[0].for_compact_container(); }
+  void for_compact_container(void *p) { N[0].for_compact_container(p); }
 
   // TDS internal data access functions.
   TDS_data& tds_data() { return _tds_data; }
diff -pruN 5.0.3-1/include/CGAL/Periodic_3_triangulation_ds_vertex_base_3.h 5.1-1/include/CGAL/Periodic_3_triangulation_ds_vertex_base_3.h
--- 5.0.3-1/include/CGAL/Periodic_3_triangulation_ds_vertex_base_3.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/Periodic_3_triangulation_ds_vertex_base_3.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_triangulation_3/include/CGAL/Periodic_3_triangulation_ds_vertex_base_3.h $
-// $Id: Periodic_3_triangulation_ds_vertex_base_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_triangulation_3/include/CGAL/Periodic_3_triangulation_ds_vertex_base_3.h $
+// $Id: Periodic_3_triangulation_ds_vertex_base_3.h 378554e 2020-03-06T16:42:23+01:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -80,8 +80,8 @@ public:
   // For use by the Compact_container.
   void *   for_compact_container() const
   { return _c.for_compact_container(); }
-  void * & for_compact_container()
-  { return _c.for_compact_container(); }
+  void for_compact_container(void *p)
+  { _c.for_compact_container(p); }
 
 private:
   Cell_handle _c;
diff -pruN 5.0.3-1/include/CGAL/Periodic_3_triangulation_hierarchy_3.h 5.1-1/include/CGAL/Periodic_3_triangulation_hierarchy_3.h
--- 5.0.3-1/include/CGAL/Periodic_3_triangulation_hierarchy_3.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/Periodic_3_triangulation_hierarchy_3.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_triangulation_3/include/CGAL/Periodic_3_triangulation_hierarchy_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_triangulation_3/include/CGAL/Periodic_3_triangulation_hierarchy_3.h $
 // $Id: Periodic_3_triangulation_hierarchy_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Periodic_3_triangulation_traits_3.h 5.1-1/include/CGAL/Periodic_3_triangulation_traits_3.h
--- 5.0.3-1/include/CGAL/Periodic_3_triangulation_traits_3.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/Periodic_3_triangulation_traits_3.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_triangulation_3/include/CGAL/Periodic_3_triangulation_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_triangulation_3/include/CGAL/Periodic_3_triangulation_traits_3.h $
 // $Id: Periodic_3_triangulation_traits_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Periodic_4_hyperbolic_Delaunay_triangulation_2.h 5.1-1/include/CGAL/Periodic_4_hyperbolic_Delaunay_triangulation_2.h
--- 5.0.3-1/include/CGAL/Periodic_4_hyperbolic_Delaunay_triangulation_2.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/Periodic_4_hyperbolic_Delaunay_triangulation_2.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_4_hyperbolic_triangulation_2/include/CGAL/Periodic_4_hyperbolic_Delaunay_triangulation_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_4_hyperbolic_triangulation_2/include/CGAL/Periodic_4_hyperbolic_Delaunay_triangulation_2.h $
 // $Id: Periodic_4_hyperbolic_Delaunay_triangulation_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Periodic_4_hyperbolic_Delaunay_triangulation_traits_2.h 5.1-1/include/CGAL/Periodic_4_hyperbolic_Delaunay_triangulation_traits_2.h
--- 5.0.3-1/include/CGAL/Periodic_4_hyperbolic_Delaunay_triangulation_traits_2.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/Periodic_4_hyperbolic_Delaunay_triangulation_traits_2.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_4_hyperbolic_triangulation_2/include/CGAL/Periodic_4_hyperbolic_Delaunay_triangulation_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_4_hyperbolic_triangulation_2/include/CGAL/Periodic_4_hyperbolic_Delaunay_triangulation_traits_2.h $
 // $Id: Periodic_4_hyperbolic_Delaunay_triangulation_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Periodic_4_hyperbolic_triangulation_2.h 5.1-1/include/CGAL/Periodic_4_hyperbolic_triangulation_2.h
--- 5.0.3-1/include/CGAL/Periodic_4_hyperbolic_triangulation_2.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/Periodic_4_hyperbolic_triangulation_2.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_4_hyperbolic_triangulation_2/include/CGAL/Periodic_4_hyperbolic_triangulation_2.h $
-// $Id: Periodic_4_hyperbolic_triangulation_2.h 5b2d46c 2020-04-21T10:03:00+02:00 Maxime Gimeno
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_4_hyperbolic_triangulation_2/include/CGAL/Periodic_4_hyperbolic_triangulation_2.h $
+// $Id: Periodic_4_hyperbolic_triangulation_2.h e9d41d7 2020-04-21T10:03:00+02:00 Maxime Gimeno
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Iordan Iordanov  <Iordan.Iordanov@loria.fr>
diff -pruN 5.0.3-1/include/CGAL/Periodic_4_hyperbolic_triangulation_face_base_2.h 5.1-1/include/CGAL/Periodic_4_hyperbolic_triangulation_face_base_2.h
--- 5.0.3-1/include/CGAL/Periodic_4_hyperbolic_triangulation_face_base_2.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/Periodic_4_hyperbolic_triangulation_face_base_2.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_4_hyperbolic_triangulation_2/include/CGAL/Periodic_4_hyperbolic_triangulation_face_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_4_hyperbolic_triangulation_2/include/CGAL/Periodic_4_hyperbolic_triangulation_face_base_2.h $
 // $Id: Periodic_4_hyperbolic_triangulation_face_base_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Periodic_4_hyperbolic_triangulation_vertex_base_2.h 5.1-1/include/CGAL/Periodic_4_hyperbolic_triangulation_vertex_base_2.h
--- 5.0.3-1/include/CGAL/Periodic_4_hyperbolic_triangulation_vertex_base_2.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/Periodic_4_hyperbolic_triangulation_vertex_base_2.h	2020-09-08 12:15:21.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_4_hyperbolic_triangulation_2/include/CGAL/Periodic_4_hyperbolic_triangulation_vertex_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_4_hyperbolic_triangulation_2/include/CGAL/Periodic_4_hyperbolic_triangulation_vertex_base_2.h $
 // $Id: Periodic_4_hyperbolic_triangulation_vertex_base_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/perturb_mesh_3.h 5.1-1/include/CGAL/perturb_mesh_3.h
--- 5.0.3-1/include/CGAL/perturb_mesh_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/perturb_mesh_3.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/perturb_mesh_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/perturb_mesh_3.h $
 // $Id: perturb_mesh_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/pierce_rectangles_2.h 5.1-1/include/CGAL/pierce_rectangles_2.h
--- 5.0.3-1/include/CGAL/pierce_rectangles_2.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/pierce_rectangles_2.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/pierce_rectangles_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/pierce_rectangles_2.h $
 // $Id: pierce_rectangles_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Plane_3.h 5.1-1/include/CGAL/Plane_3.h
--- 5.0.3-1/include/CGAL/Plane_3.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Plane_3.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Plane_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Plane_3.h $
 // $Id: Plane_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Plane_separator.h 5.1-1/include/CGAL/Plane_separator.h
--- 5.0.3-1/include/CGAL/Plane_separator.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Plane_separator.h	2020-09-08 12:15:39.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_searching/include/CGAL/Plane_separator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_searching/include/CGAL/Plane_separator.h $
 // $Id: Plane_separator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Point_2.h 5.1-1/include/CGAL/Point_2.h
--- 5.0.3-1/include/CGAL/Point_2.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Point_2.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Point_2.h $
-// $Id: Point_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Point_2.h $
+// $Id: Point_2.h 3ebe883 2020-03-01T12:50:13+01:00 Marc Glisse
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -48,12 +48,12 @@ public:
   typedef RPoint_2 Rep;
   typedef typename R_::Cartesian_const_iterator_2 Cartesian_const_iterator;
 
-  const Rep& rep() const
+  const Rep& rep() const noexcept
   {
     return *this;
   }
 
-  Rep& rep()
+  Rep& rep() noexcept
   {
     return *this;
   }
@@ -84,6 +84,15 @@ public:
     : RPoint_2(typename R::Construct_point_2()(Return_base_tag(), hx, hy, hw))
   {}
 
+  friend void swap(Self& a, Self& b)
+#ifdef __cpp_lib_is_swappable
+    noexcept(std::is_nothrow_swappable_v<Rep>)
+#endif
+  {
+    using std::swap;
+    swap(a.rep(), b.rep());
+  }
+
   typename cpp11::result_of<typename R::Compute_x_2(Point_2)>::type
   x() const
   {
diff -pruN 5.0.3-1/include/CGAL/Point_3.h 5.1-1/include/CGAL/Point_3.h
--- 5.0.3-1/include/CGAL/Point_3.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Point_3.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Point_3.h $
-// $Id: Point_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Point_3.h $
+// $Id: Point_3.h 3ebe883 2020-03-01T12:50:13+01:00 Marc Glisse
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -46,12 +46,12 @@ public:
   typedef typename R_::Kernel_base::Point_3  Rep;
   typedef typename R_::Cartesian_const_iterator_3 Cartesian_const_iterator;
 
-  const Rep& rep() const
+  const Rep& rep() const noexcept
   {
     return *this;
   }
 
-  Rep& rep()
+  Rep& rep() noexcept
   {
     return *this;
   }
@@ -81,6 +81,15 @@ public:
     : Rep(typename R::Construct_point_3()(Return_base_tag(), hx, hy, hz, hw))
   {}
 
+  friend void swap(Self& a, Self& b)
+#ifdef __cpp_lib_is_swappable
+    noexcept(std::is_nothrow_swappable_v<Rep>)
+#endif
+  {
+    using std::swap;
+    swap(a.rep(), b.rep());
+  }
+
   typename cpp11::result_of<typename R::Compute_x_3( Point_3)>::type
   x() const
   {
diff -pruN 5.0.3-1/include/CGAL/Point_container.h 5.1-1/include/CGAL/Point_container.h
--- 5.0.3-1/include/CGAL/Point_container.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Point_container.h	2020-09-08 12:15:39.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_searching/include/CGAL/Point_container.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_searching/include/CGAL/Point_container.h $
 // $Id: Point_container.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/point_generators_2.h 5.1-1/include/CGAL/point_generators_2.h
--- 5.0.3-1/include/CGAL/point_generators_2.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/point_generators_2.h	2020-09-08 12:14:51.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Generator/include/CGAL/point_generators_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Generator/include/CGAL/point_generators_2.h $
 // $Id: point_generators_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/point_generators_3.h 5.1-1/include/CGAL/point_generators_3.h
--- 5.0.3-1/include/CGAL/point_generators_3.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/point_generators_3.h	2020-09-08 12:14:51.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Generator/include/CGAL/point_generators_3.h $
-// $Id: point_generators_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Generator/include/CGAL/point_generators_3.h $
+// $Id: point_generators_3.h 9689a1c 2020-03-27T16:16:47+01:00 Maxime Gimeno
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -539,6 +539,26 @@ public:
 
   const Tr& tr;
 };
+
+template<typename Triangle, typename PointRange>
+class Triangle_3_from_soup
+{
+  typedef typename boost::range_value<PointRange>::type  Point_3;
+  typedef typename Kernel_traits<Point_3>::Kernel        Kernel;
+public:
+  typedef typename Kernel::Triangle_3                    result_type;
+
+  Triangle_3_from_soup(const PointRange& pts) : points(pts) { }
+
+  result_type operator()(const Triangle& t) const
+  {
+    return result_type(points[t[0]], points[t[1]], points[t[2]]);
+  }
+
+private:
+  const PointRange& points;
+};
+
 }//end namespace internal
 
 template <class C3t3,
@@ -685,6 +705,52 @@ struct Random_points_in_triangles_3
     This tmp = *this;
     ++(*this);
     return tmp;
+  }
+};
+
+
+template <class PointRange,
+          class Triangle = std::vector<std::size_t>,
+          class Creator = Creator_uniform_3<
+                            typename Kernel_traits< typename PointRange::value_type >::Kernel::RT,
+                            typename PointRange::value_type> >
+struct Random_points_in_triangle_soup
+    : public Generic_random_point_generator<Triangle,
+                                            internal::Triangle_3_from_soup<Triangle, PointRange>,
+                                            Random_points_in_triangle_3<typename PointRange::value_type>,
+                                            typename PointRange::value_type>
+{
+  typedef Generic_random_point_generator<Triangle,
+                                         internal::Triangle_3_from_soup<Triangle, PointRange>,
+                                         Random_points_in_triangle_3<typename PointRange::value_type>,
+                                         typename PointRange::value_type> Base;
+  typedef typename PointRange::value_type                                 Point_3;
+  typedef typename Kernel_traits<Point_3>::Kernel                         Kernel;
+  typedef Triangle                                                        Id;
+  typedef Point_3                                                         result_type;
+  typedef Random_points_in_triangle_soup<PointRange, Triangle, Creator>   This;
+
+  template<typename TriangleRange>
+  Random_points_in_triangle_soup(const TriangleRange& triangles,
+                                 const PointRange& points,
+                                 Random& rnd = get_default_random())
+    : Base(triangles,
+           internal::Triangle_3_from_soup<Triangle, PointRange>(points),
+           internal::Apply_approx_sqrt<typename Kernel_traits<Point_3>::Kernel::Compute_squared_area_3>(),
+           rnd)
+  { }
+
+  This& operator++()
+  {
+    Base::generate_point();
+    return *this;
+  }
+
+  This operator++(int)
+  {
+    This tmp = *this;
+    ++(*this);
+    return tmp;
   }
 };
 
diff -pruN 5.0.3-1/include/CGAL/point_generators_d.h 5.1-1/include/CGAL/point_generators_d.h
--- 5.0.3-1/include/CGAL/point_generators_d.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/point_generators_d.h	2020-09-08 12:14:51.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Generator/include/CGAL/point_generators_d.h $
-// $Id: point_generators_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Generator/include/CGAL/point_generators_d.h $
+// $Id: point_generators_d.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Olivier Devillers
@@ -18,6 +18,8 @@
 
 #include <CGAL/generators.h>
 #include <CGAL/number_type_basic.h>
+#include <CGAL/number_type_config.h>
+
 #include <cmath>
 #include <vector>
 
@@ -56,11 +58,10 @@ generate_point() {
   for(int i=0; i<dimension; ++i) {
     // normal distribution
     //( a product of normal distib is a normal distrib in higher dim)
-    const double pi = 3.141592653589793238462643;
     double a=this->_rnd.get_double();
     a = std::sqrt( -2* std::log(1-a) );
     double b=this->_rnd.get_double();
-    b = std::cos(2*pi*b);
+    b = std::cos(2*CGAL_PI*b);
     coord[i]= a*b;
     norm += coord[i]*coord[i];
   }
@@ -104,11 +105,10 @@ generate_point() {
 
   for(int i=0; i<dimension; ++i) {
     // normal distribution
-    const double pi = 3.141592653589793238462643;
     double a=this->_rnd.get_double();
     a = std::sqrt( -2* std::log(1-a) );
     double b=this->_rnd.get_double();
-    b = std::cos(2*pi*b);
+    b = std::cos(2*CGAL_PI*b);
     coord[i]= a*b;
     norm += coord[i]*coord[i];
   }
diff -pruN 5.0.3-1/include/CGAL/pointmatcher/compute_registration_transformation.h 5.1-1/include/CGAL/pointmatcher/compute_registration_transformation.h
--- 5.0.3-1/include/CGAL/pointmatcher/compute_registration_transformation.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/pointmatcher/compute_registration_transformation.h	2020-09-08 12:15:22.000000000 +0000
@@ -0,0 +1,628 @@
+// Copyright (c) 2019  GeometryFactory(France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/pointmatcher/compute_registration_transformation.h $
+// $Id: compute_registration_transformation.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s) : Necip Fazil Yildiran
+
+#ifndef CGAL_POINTMATCHER_COMPUTE_REGISTRATION_TRANSFORMATION_H
+#define CGAL_POINTMATCHER_COMPUTE_REGISTRATION_TRANSFORMATION_H
+
+#include <CGAL/license/Point_set_processing_3.h>
+
+#if defined(CGAL_LINKED_WITH_POINTMATCHER) || defined(DOXYGEN_RUNNING)
+
+#include <CGAL/Aff_transformation_3.h>
+#include <CGAL/assertions.h>
+#include <CGAL/boost/graph/Named_function_parameters.h>
+#include <CGAL/boost/graph/named_params_helper.h>
+#include <CGAL/aff_transformation_tags.h>
+
+#include <boost/type_traits/is_same.hpp>
+
+#include <pointmatcher/PointMatcher.h>
+
+#include <iostream>
+#include <string>
+#include <map>
+
+namespace CGAL {
+
+namespace pointmatcher {
+
+template<typename Scalar>
+using ICP = typename PointMatcher<Scalar>::ICP;
+
+/*!
+   \ingroup PkgPointSetProcessing3Algorithms
+
+   \brief The class `ICP_config` is designed to handle preparing and passing configurations
+   to the registration methods `CGAL::pointmatcher::compute_registration_transformation()`
+   and `CGAL::pointmatcher::register_point_sets()`.
+
+   \details A configuration corresponds to a component of a configuration module
+   of \ref thirdpartylibpointmatcher library. The name and the parameters of any
+   configuration for the corresponding registration methods are directly passed
+   to \ref thirdpartylibpointmatcher library to be parsed and registered at the
+   \ref thirdpartylibpointmatcher side.
+   */
+struct ICP_config {
+  /// The name of the configuration component
+  std::string name;
+
+  /// The set of (parameter name, parameter value) pairs as a map
+  std::map<std::string, std::string> params;
+};
+
+namespace internal {
+
+void dump_invalid_point_matcher_config_exception_msg(const PointMatcherSupport::InvalidElement& err) {
+  std::cerr << "ERROR Invalid configuration for PM::ICP, omitting configuration: " << std::endl;
+        std::cerr << "   " << err.what() << std::endl;
+}
+
+template<typename Scalar, typename NamedParameters1, typename NamedParameters2>
+ICP<Scalar>
+construct_icp(const NamedParameters1& np1, const NamedParameters2& np2)
+{
+  typedef PointMatcher<Scalar> PM;
+
+  using parameters::choose_parameter;
+  using parameters::get_parameter;
+
+  ICP<Scalar> icp;
+
+  icp.setDefault();
+
+  const ICP_config null_config { "_null_pm_config_in_cgal" };
+  const std::vector<ICP_config> null_config_chain { null_config };
+  auto is_null_config = [&](const ICP_config& c) { return !c.name.compare(null_config.name); };
+
+  // Config file
+  std::istream* pointmatcher_config = choose_parameter(get_parameter(np1, internal_np::pointmatcher_config), nullptr);
+  if(pointmatcher_config != nullptr)
+  {
+    icp.loadFromYaml(*pointmatcher_config);
+    return icp;
+  }
+
+  // In CGAL, point_set_1 is the reference while point_set_2 is the data
+  // However, in pointmatcher, the order is reverse: point_set_1 is the data while point_set_2 is the reference
+  // Therefore, filter params from np1 applies to reference data points while params from np2 applies to reading data points
+
+  // np1.point_set_filters -> PM::ReferenceDataPointsFilter
+  auto reference_data_points_filter_configs = choose_parameter(get_parameter(np1, internal_np::point_set_filters), null_config_chain);
+  if(!reference_data_points_filter_configs.empty() && is_null_config(*reference_data_points_filter_configs.cbegin())) {
+    // No config provided: use default values as been set above, do nothing
+    ;
+  } else {
+    // Some config chain is given: clear default values and use the provided configs
+    icp.referenceDataPointsFilters.clear();
+
+    for(const auto& conf : reference_data_points_filter_configs)
+    {
+      try {
+        icp.referenceDataPointsFilters.push_back( PM::get().DataPointsFilterRegistrar.create(conf.name, conf.params) );
+      } catch(typename PointMatcherSupport::InvalidElement& error) {
+        dump_invalid_point_matcher_config_exception_msg(error);
+      }
+    }
+  }
+  // np2.point_set_filters -> PM::ReadingDataPointsFilters
+  auto reading_data_points_filter_configs = choose_parameter(get_parameter(np2, internal_np::point_set_filters), null_config_chain);
+  if(!reading_data_points_filter_configs.empty() && is_null_config(*reading_data_points_filter_configs.cbegin())) {
+    // No config provided: use default values as been set above, do nothing
+    ;
+  } else {
+    // Some config chain is given: clear default values and use the provided configs
+    icp.readingDataPointsFilters.clear();
+
+    for(const auto& conf : reading_data_points_filter_configs)
+    {
+      try {
+        icp.readingDataPointsFilters.push_back( PM::get().DataPointsFilterRegistrar.create(conf.name, conf.params) );
+      } catch(typename PointMatcherSupport::InvalidElement& error) {
+        dump_invalid_point_matcher_config_exception_msg(error);
+      }
+    }
+  }
+
+  // Matcher
+  auto matcher_config = choose_parameter(get_parameter(np1, internal_np::matcher), null_config);
+  if(!is_null_config(matcher_config))
+  {
+    try {
+      icp.matcher = PM::get().MatcherRegistrar.create(matcher_config.name, matcher_config.params);
+    } catch(typename PointMatcherSupport::InvalidElement& error) {
+      dump_invalid_point_matcher_config_exception_msg(error);
+    }
+  }
+
+  // Outlier Filters
+  auto outlier_filters_config = choose_parameter(get_parameter(np1, internal_np::outlier_filters), null_config_chain);
+  if(!outlier_filters_config.empty() && is_null_config(*outlier_filters_config.cbegin())) {
+    // No config provided: use default values as been set above, do nothing
+    ;
+  } else {
+    // Some config chain is given: clear default values and use the provided configs
+    icp.outlierFilters.clear();
+
+    for(const auto& conf : outlier_filters_config)
+    {
+      try {
+        icp.outlierFilters.push_back( PM::get().OutlierFilterRegistrar.create(conf.name, conf.params) );
+      } catch(typename PointMatcherSupport::InvalidElement& error) {
+        dump_invalid_point_matcher_config_exception_msg(error);
+      }
+    }
+  }
+
+  // Error Minimizer
+  auto error_minimizer_config = choose_parameter(get_parameter(np1, internal_np::error_minimizer), null_config);
+  if(!is_null_config(error_minimizer_config))
+  {
+    try {
+      icp.errorMinimizer = PM::get().ErrorMinimizerRegistrar.create(error_minimizer_config.name, error_minimizer_config.params);
+    } catch(typename PointMatcherSupport::InvalidElement& error) {
+      dump_invalid_point_matcher_config_exception_msg(error);
+    }
+  }
+
+  // Transformation Checkers
+  auto transformation_checkers_config = choose_parameter(get_parameter(np1, internal_np::transformation_checkers), null_config_chain);
+  if(!transformation_checkers_config.empty() && is_null_config(*transformation_checkers_config.cbegin())) {
+    // No config provided: use default values as been set above, do nothing
+    ;
+  } else {
+    // Some config chain is given: clear default values and use the provided configs
+    icp.transformationCheckers.clear();
+
+    for(const auto& conf : transformation_checkers_config)
+    {
+      try {
+        icp.transformationCheckers.push_back( PM::get().TransformationCheckerRegistrar.create(conf.name, conf.params) );
+      } catch(typename PointMatcherSupport::InvalidElement& error) {
+        dump_invalid_point_matcher_config_exception_msg(error);
+      }
+    }
+  }
+
+  // Inspector
+  auto inspector_config = choose_parameter(get_parameter(np1, internal_np::inspector), null_config);
+  if(!is_null_config(error_minimizer_config))
+  {
+    try {
+      icp.inspector = PM::get().InspectorRegistrar.create(inspector_config.name, inspector_config.params);
+    } catch(typename PointMatcherSupport::InvalidElement& error) {
+      dump_invalid_point_matcher_config_exception_msg(error);
+    }
+  }
+
+  // Logger
+  auto logger_config = choose_parameter(get_parameter(np1, internal_np::logger), null_config);
+  if(!is_null_config(logger_config))
+  {
+    try {
+      PointMatcherSupport::setLogger( PM::get().LoggerRegistrar.create(logger_config.name, logger_config.params) );
+    } catch(typename PointMatcherSupport::InvalidElement& error) {
+      dump_invalid_point_matcher_config_exception_msg(error);
+    }
+  }
+
+  return icp;
+}
+
+template<typename Scalar,
+         typename PointRange,
+         typename PointMap,
+         typename VectorMap,
+         typename PM_matrix>
+void
+copy_cgal_points_to_pm_matrix
+(const PointRange& prange, PointMap point_map, VectorMap vector_map, PM_matrix& pm_points, PM_matrix& pm_normals)
+{
+  int idx = 0;
+  for(const auto& p : prange)
+  {
+    // position
+    const auto& pos = get(point_map, p);
+    pm_points(0, idx) = pos.x();
+    pm_points(1, idx) = pos.y();
+    pm_points(2, idx) = pos.z();
+    pm_points(3, idx) = Scalar(1.);
+
+    // normal
+    const auto& normal = get (vector_map, p);
+    pm_normals(0, idx) = normal.x();
+    pm_normals(1, idx) = normal.y();
+    pm_normals(2, idx) = normal.z();
+
+    ++idx;
+  }
+}
+
+template <class Kernel,
+          class PointRange1,
+          class PointRange2,
+          class PointMap1,
+          class PointMap2,
+          class VectorMap1,
+          class VectorMap2>
+std::pair<typename Kernel::Aff_transformation_3, bool>
+compute_registration_transformation(const PointRange1& range1, const PointRange2& range2,
+                                    PointMap1 point_map1, PointMap2 point_map2,
+                                    VectorMap1 vector_map1, VectorMap2 vector_map2,
+                                    const typename Kernel::Aff_transformation_3& initial_transform,
+                                    ICP<typename Kernel::FT> icp)
+{
+  using Scalar    = typename Kernel::FT;
+
+  using PM                  = PointMatcher<Scalar>;
+  using PM_cloud            = typename PM::DataPoints;
+  using PM_matrix           = typename PM::Matrix;
+  using PM_transform        = typename PM::Transformation;
+  using PM_transform_params = typename PM::TransformationParameters;
+
+  // ref_points: 1, points: 2
+  std::size_t nb_ref_points = range1.size();
+  std::size_t nb_points     = range2.size();
+
+  PM_matrix ref_points_pos_matrix    = PM_matrix (4, nb_ref_points);
+  PM_matrix ref_points_normal_matrix = PM_matrix (3, nb_ref_points);
+  PM_matrix points_pos_matrix    = PM_matrix (4, nb_points);
+  PM_matrix points_normal_matrix = PM_matrix (3, nb_points);
+
+  // In CGAL, point_set_1 is the reference while point_set_2 is the data
+
+  // convert cgal points to pointmatcher points
+  internal::copy_cgal_points_to_pm_matrix<Scalar>(range1,
+                                                  point_map1,
+                                                  vector_map1,
+                                                  ref_points_pos_matrix, // out
+                                                  ref_points_normal_matrix); // out
+
+  internal::copy_cgal_points_to_pm_matrix<Scalar>(range2,
+                                                  point_map2,
+                                                  vector_map2,
+                                                  points_pos_matrix, // out
+                                                  points_normal_matrix); // out
+
+  auto construct_PM_cloud = [](const PM_matrix& positions, const PM_matrix& normals) -> PM_cloud
+  {
+    PM_cloud cloud;
+
+    cloud.addFeature("x", positions.row(0));
+    cloud.addFeature("y", positions.row(1));
+    cloud.addFeature("z", positions.row(2));
+    cloud.addFeature("pad", positions.row(3));
+    cloud.addDescriptor("normals", normals);
+
+    return cloud;
+  };
+
+  PM_cloud ref_cloud = construct_PM_cloud(ref_points_pos_matrix, ref_points_normal_matrix);
+  PM_cloud cloud     = construct_PM_cloud(points_pos_matrix,     points_normal_matrix);
+
+  PM_transform_params pm_transform_params = PM_transform_params::Identity(4,4);
+
+  // Convert CGAL transform to pm transform
+  for(int i = 0; i < 4; i++)
+    for(int j = 0; j < 4; j++)
+      pm_transform_params(i,j) = initial_transform.m(i,j);
+
+  bool converged = false;
+  try
+  {
+                const PM_transform_params prior = pm_transform_params;
+                pm_transform_params = icp(cloud, ref_cloud, prior);
+    converged = true;
+        }
+        catch (typename PM::ConvergenceError& error)
+        {
+                std::cerr << "ERROR CGAL::pointmatcher registration (PM::ICP) failed to converge: " << std::endl;
+                std::cerr << "   " << error.what() << std::endl;
+    converged = false;
+        }
+
+        // Rigid transformation
+        std::shared_ptr<PM_transform> transform = PM::get().REG(Transformation).create("RigidTransformation");
+  pm_transform_params = transform->correctParameters(pm_transform_params);
+
+  typename Kernel::Aff_transformation_3 cgal_transform
+    (pm_transform_params(0,0), pm_transform_params(0,1), pm_transform_params(0,2), pm_transform_params(0,3),
+     pm_transform_params(1,0), pm_transform_params(1,1), pm_transform_params(1,2), pm_transform_params(1,3),
+     pm_transform_params(2,0), pm_transform_params(2,1), pm_transform_params(2,2), pm_transform_params(2,3));
+
+#ifdef CGAL_POINTMATCHER_VERBOSE
+  std::cerr << "Transformation matrix: " << std::endl;
+  for (std::size_t i = 0; i < 4; ++ i)
+  {
+    for (std::size_t j = 0; j < 4; ++ j)
+      std::cerr << cgal_transform.coeff(i,j) << " ";
+    std::cerr << std::endl;
+  }
+#endif
+
+  return std::make_pair(cgal_transform, converged);
+}
+
+} // end of namespace internal
+
+// point_set_1 is reference while point_set_2 is data
+/**
+   \ingroup PkgPointSetProcessing3Algorithms
+
+   Computes the registration of `point_set_2` with respect to `point_set_1` and
+   returns the corresponding affine transformation.
+   Registration is computed using the Iterative Closest Point (ICP) algorithm.
+
+   \note This function requires the \ref thirdpartylibpointmatcher library.
+
+   \tparam PointRange1 is a model of `Range`. The value type of its iterator is
+   the key type of the named parameter `point_map` in `NamedParameters1`.
+   \tparam PointRange2 is a model of `Range`. The value type of its iterator is
+   the key type of the named parameter `point_map` in `NamedParameters2`.
+
+   \param point_set_1 input point range used as reference.
+   \param point_set_2 input point range whose registration w.r.t. `point_set_1` will be computed.
+   \param np1 an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+
+   \cgalNamedParamsBegin
+     \cgalParamNBegin{point_map}
+       \cgalParamDescription{a property map associating points to the elements of the point set `point_set_1`}
+       \cgalParamType{a model of `ReadablePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange1` and whose value type is `geom_traits::Point_3`}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{normal_map}
+       \cgalParamDescription{a property map associating normals to the elements of the point set `point_set_1`}
+       \cgalParamType{a model of `ReadablePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange1` and whose value type is `geom_traits::Vector_3`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{point_set_filters}
+       \cgalParamDescription{a chain of filters to be applied to the point set}
+       \cgalParamType{a class model of `Range`. The value type of its iterator must be `ICP_config`.}
+       \cgalParamDefault{`RandomSamplingDataPointsFilter`}
+       \cgalParamExtra{The chain of filters to be applied to the reference point cloud. The reference
+                       point cloud is processed into an intermediate point cloud with the given chain
+                       of filters to be used in the alignment procedure. The chain is organized with
+                       the forward traversal order of the point set filters range.
+
+                       The chain of point set filters are applied only once at the beginning of the
+                       ICP procedure, i.e., before the first iteration of the ICP algorithm.
+
+                       The filters can have several purposes, including but are not limited to
+                       i) removal of noisy points which render alignment of point clouds difficult,
+                       ii) removal of redundant points so as to speed up alignment, iii) addition
+                       of descriptive information to the points such as a surface normal vector,
+                       or the direction from the point to the sensor.}
+       \cgalParamExtra{Corresponds to `referenceDataPointsFilters` configuration module of \ref thirdpartylibpointmatcher
+                       library. The filters should be chosen and set from possible components of
+                       the `referenceDataPointsFilters` configuration module.
+                       See <a href="https://libpointmatcher.readthedocs.io/en/latest/Configuration/#configuration-of-an-icp-chain">libpointmatcher documentation</a>
+                       for possible configurations.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{matcher}
+       \cgalParamDescription{a method used for matching (linking) the points from `point_set_2`,
+                             to the points in the reference cloud, `point_set_1`}
+       \cgalParamType{a class model of `ICP_config`}
+       \cgalParamDefault{`KDTreeMatcher`}
+       \cgalParamExtra{Corresponds to the `matcher` configuration module of \ref thirdpartylibpointmatcher
+                       library. The matcher should be chosen and set from possible components of
+                       the `matcher` configuration module.
+                       See <a href="https://libpointmatcher.readthedocs.io/en/latest/Configuration/#configuration-of-an-icp-chain">libpointmatcher documentation</a>
+                       for possible configurations.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{outlier_filters}
+       \cgalParamDescription{a chain of filters to be applied to the matched (linked) point clouds after
+                             each processing iteration of the ICP algorithm to remove the links which do not
+                             correspond to true point correspondences}
+       \cgalParamType{a model of `Range`. The value type of its iterator must be `ICP_config`.}
+       \cgalParamDefault{`TrimmedDistOutlierFilter`}
+       \cgalParamExtra{The outliers are rejected. Points with no link are ignored
+                       in the subsequent error minimization step. The chain is organized
+                       with the forward traversal order of the outlier filters range.}
+       \cgalParamExtra{Corresponds to the `outlierFilters` configuration module of \ref thirdpartylibpointmatcher
+                       library. The filters should be chosen and set from possible components of
+                       the `outlierFilters` configuration module.
+                       See <a href="https://libpointmatcher.readthedocs.io/en/latest/Configuration/#configuration-of-an-icp-chain">libpointmatcher documentation</a>
+                       for possible configurations.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{error_minimizer}
+       \cgalParamDescription{an error minimizer that computes a transformation matrix such as to minimize
+                             the error between the point sets}
+       \cgalParamType{a class model of `ICP_config`}
+       \cgalParamDefault{`PointToPlaneErrorMinimizer`}
+       \cgalParamExtra{Corresponds to the `errorMinimizer` configuration module of \ref thirdpartylibpointmatcher
+                       library. The error minimizer should be chosen and set from possible components of
+                       the `errorMinimizer` configuration module.
+                       See <a href="https://libpointmatcher.readthedocs.io/en/latest/Configuration/#configuration-of-an-icp-chain">libpointmatcher documentation</a>
+                       for possible configurations.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{transformation_checkers}
+       \cgalParamDescription{a chain of transformation checkers}
+       \cgalParamType{a class model of `Range`. The value type of its iterator must be `ICP_config`.}
+       \cgalParamDefault{`CounterTransformationChecker` and `DifferentialTransformationChecker`}
+       \cgalParamExtra{The chain is organized with the forward traversal order of the transformation checkers range.}
+       \cgalParamExtra{A transformation checker can stop the iteration depending on the conditions it defines.}
+       \cgalParamExtra{Corresponds to the `transformationCheckers` configuration module of \ref thirdpartylibpointmatcher
+                       library. The transformation checkers should be chosen and set from possible components of
+                       the `transformationCheckers` configuration module.
+                       See <a href="https://libpointmatcher.readthedocs.io/en/latest/Configuration/#configuration-of-an-icp-chain">libpointmatcher documentation</a>
+                       for possible configurations.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{inspector}
+       \cgalParamDescription{an inspector that enables logging data at different steps for analysis.}
+       \cgalParamType{a class model of `ICP_config`}
+       \cgalParamDefault{`NullInspector`}
+       \cgalParamExtra{Inspectors typically provide deeper scrutiny than the logger.}
+       \cgalParamExtra{Corresponds to the `inspector` configuration module of \ref thirdpartylibpointmatcher
+                       library. The inspector should be chosen and set from possible components of
+                       the `inspector` configuration module.
+                       See <a href="https://libpointmatcher.readthedocs.io/en/latest/Configuration/#configuration-of-an-icp-chain">libpointmatcher documentation</a>
+                       for possible configurations.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{logger}
+       \cgalParamDescription{a method for logging information regarding the registration process
+                             outputted by \ref thirdpartylibpointmatcher library}
+       \cgalParamType{a class model of `ICP_config`}
+       \cgalParamDefault{`NullLogger`}
+       \cgalParamExtra{The logs generated by CGAL library does not get effected by this configuration.}
+       \cgalParamExtra{Corresponds to the `logger` configuration module of \ref thirdpartylibpointmatcher
+                       library. The logger should be chosen and set from possible components of
+                       the `logger` configuration module.
+                       See <a href="https://libpointmatcher.readthedocs.io/en/latest/Configuration/#configuration-of-an-icp-chain">libpointmatcher documentation</a>
+                       for possible configurations.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{geom_traits}
+       \cgalParamDescription{an instance of a geometric traits class}
+       \cgalParamType{a model of `Kernel`}
+       \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+     \cgalParamNEnd
+   \cgalNamedParamsEnd
+
+   \param np2 an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+
+   \cgalNamedParamsBegin
+     \cgalParamNBegin{point_map}
+       \cgalParamDescription{a property map associating points to the elements of the point set `point_set_2`}
+       \cgalParamType{a model of `ReadablePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange2` and whose value type is `geom_traits::Point_3`}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{normal_map}
+       \cgalParamDescription{a property map associating normals to the elements of the point set `point_set_2`}
+       \cgalParamType{a model of `ReadablePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange2` and whose value type is `geom_traits::Vector_3`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{point_set_filters}
+       \cgalParamDescription{a chain of filters to be applied to the point set}
+       \cgalParamType{a class model of `Range`. The value type of its iterator must be `ICP_config`.}
+       \cgalParamDefault{`SamplingSurfaceNormalDataPointsFilter`}
+       \cgalParamExtra{The chain of filters to be applied to the point cloud `point_set_2`. The
+                       point cloud is processed into an intermediate point cloud with the given chain
+                       of filters to be used in the alignment procedure. The chain is organized with
+                       the forward traversal order of the point set filters range.
+
+                       The chain of point set filters are applied only once at the beginning of the
+                       ICP procedure, i.e., before the first iteration of the ICP algorithm.
+
+                       The filters can have several purposes, including but are not limited to
+                       i) removal of noisy points which render alignment of point clouds difficult,
+                       ii) removal of redundant points so as to speed up alignment, iii) addition
+                       of descriptive information to the points such as a surface normal vector,
+                       or the direction from the point to the sensor.}
+       \cgalParamExtra{Corresponds to the `readingDataPointsFilters` configuration module of \ref thirdpartylibpointmatcher
+                       library. The filters should be chosen and set from possible components of
+                       the `readingDataPointsFilters` configuration module.
+                       See <a href="https://libpointmatcher.readthedocs.io/en/latest/Configuration/#configuration-of-an-icp-chain">libpointmatcher documentation</a>
+                       for possible configurations.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{transformation}
+       \cgalParamDescription{an affine transformation that is used as the initial transformation for `point_set_2`}
+       \cgalParamType{`CGAL::Aff_transformation_3`}
+       \cgalParamDefault{the identity transformation}
+     \cgalParamNEnd
+   \cgalNamedParamsEnd
+
+   \return a pair containing the affine transformation that should be applied
+   to `point_set_2` to make it registered w.r.t. `point_set_1` and the
+   boolean value indicating if the registration converged. The second
+   of the pair is `true` if converged, `false` otherwise. A log why it failed to
+   converge is written to `std::cerr` if the registration cannot converge.
+*/
+template <class PointRange1, class PointRange2,
+          class NamedParameters1, class NamedParameters2>
+#ifdef DOXYGEN_RUNNING
+std::pair<geom_traits::Aff_transformation_3, bool>
+#else
+std::pair<typename CGAL::Point_set_processing_3::GetK<PointRange1, NamedParameters1>
+  ::Kernel::Aff_transformation_3, bool>
+#endif
+compute_registration_transformation (const PointRange1& point_set_1, const PointRange2& point_set_2,
+                                     const NamedParameters1& np1, const NamedParameters2& np2)
+{
+  using parameters::choose_parameter;
+  using parameters::get_parameter;
+
+  namespace PSP = CGAL::Point_set_processing_3;
+
+  // property map types
+  typedef typename CGAL::GetPointMap<PointRange1, NamedParameters1>::type PointMap1;
+  typedef typename CGAL::GetPointMap<PointRange2, NamedParameters2>::type PointMap2;
+  CGAL_static_assertion_msg((boost::is_same< typename boost::property_traits<PointMap1>::value_type,
+                                             typename boost::property_traits<PointMap2>::value_type> ::value),
+                            "The point type of input ranges must be the same");
+
+  typedef typename PSP::GetNormalMap<PointRange1, NamedParameters1>::type NormalMap1;
+  typedef typename PSP::GetNormalMap<PointRange2, NamedParameters2>::type NormalMap2;
+  CGAL_static_assertion_msg((boost::is_same< typename boost::property_traits<NormalMap1>::value_type,
+                                             typename boost::property_traits<NormalMap2>::value_type> ::value),
+                            "The vector type of input ranges must be the same");
+
+  typedef typename PSP::GetK<PointRange1, NamedParameters1>::Kernel Kernel;
+  typedef typename Kernel::FT Scalar;
+  typedef typename Kernel::Aff_transformation_3 Transformation;
+
+  PointMap1 point_map1 = choose_parameter(get_parameter(np1, internal_np::point_map), PointMap1());
+  NormalMap1 normal_map1 = choose_parameter(get_parameter(np1, internal_np::normal_map), NormalMap1());
+  PointMap2 point_map2 = choose_parameter(get_parameter(np2, internal_np::point_map), PointMap2());
+  NormalMap2 normal_map2 = choose_parameter(get_parameter(np2, internal_np::normal_map), NormalMap2());
+
+  // initial transformation
+  Transformation initial_transformation
+    = choose_parameter(get_parameter(np2, internal_np::transformation), Transformation(Identity_transformation()));
+
+  return internal::compute_registration_transformation<Kernel>(point_set_1, point_set_2,
+                                                               point_map1, point_map2,
+                                                               normal_map1, normal_map2,
+                                                               initial_transformation,
+                                                               internal::construct_icp<Scalar>(np1, np2));
+}
+
+// convenience overloads
+template <class PointRange1, class PointRange2,
+          class NamedParameters1>
+std::pair<typename CGAL::Point_set_processing_3::GetK<PointRange1, NamedParameters1>
+  ::Kernel::Aff_transformation_3, bool>
+compute_registration_transformation(const PointRange1& point_set_1, const PointRange2& point_set_2,
+      const NamedParameters1& np1)
+{
+  namespace params = CGAL::Point_set_processing_3::parameters;
+  return compute_registration_transformation(point_set_1, point_set_2, np1, params::all_default(point_set_1));
+}
+
+template <class PointRange1, class PointRange2>
+std::pair<typename CGAL::Point_set_processing_3::GetK<PointRange1,
+          Named_function_parameters<bool, internal_np::all_default_t> >
+  ::Kernel::Aff_transformation_3, bool>
+compute_registration_transformation(const PointRange1& point_set_1, const PointRange2& point_set_2)
+{
+  namespace params = CGAL::Point_set_processing_3::parameters;
+  return compute_registration_transformation(point_set_1, point_set_2,
+                                             params::all_default(point_set_1),
+                                             params::all_default(point_set_2));
+}
+
+
+} } // end of namespace CGAL::pointmatcher
+
+#endif // CGAL_LINKED_WITH_POINTMATCHER
+
+#endif // CGAL_POINTMATCHER_COMPUTE_REGISTRATION_TRANSFORMATION_H
diff -pruN 5.0.3-1/include/CGAL/pointmatcher/register_point_sets.h 5.1-1/include/CGAL/pointmatcher/register_point_sets.h
--- 5.0.3-1/include/CGAL/pointmatcher/register_point_sets.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/pointmatcher/register_point_sets.h	2020-09-08 12:15:22.000000000 +0000
@@ -0,0 +1,282 @@
+// Copyright (c) 2019  GeometryFactory(France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/pointmatcher/register_point_sets.h $
+// $Id: register_point_sets.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s) : Necip Fazil Yildiran
+
+#ifndef CGAL_POINTMATCHER_REGISTER_POINT_SETS_H
+#define CGAL_POINTMATCHER_REGISTER_POINT_SETS_H
+
+#include <CGAL/license/Point_set_processing_3.h>
+
+#if defined(CGAL_LINKED_WITH_POINTMATCHER) || defined(DOXYGEN_RUNNING)
+
+#include <CGAL/Aff_transformation_3.h>
+#include <CGAL/assertions.h>
+#include <CGAL/boost/graph/Named_function_parameters.h>
+#include <CGAL/boost/graph/named_params_helper.h>
+
+#include <CGAL/pointmatcher/compute_registration_transformation.h>
+
+#include <boost/type_traits/is_same.hpp>
+
+#include <Eigen/Dense>
+
+namespace CGAL {
+
+namespace pointmatcher {
+
+// point_set_1 is reference while point_set_2 is data
+/**
+   \ingroup PkgPointSetProcessing3Algorithms
+
+   Computes the registration of `point_set_2` with respect to `point_set_1` and
+   applies it.
+
+   Registration is computed using the Iterative Closest Point (ICP) algorithm.
+
+   \note This function requires the \ref thirdpartylibpointmatcher library.
+
+   \tparam PointRange1 is a model of `Range`. The value type of its iterator is
+   the key type of the named parameter `point_map` in `NamedParameters1`.
+   \tparam PointRange2 is a model of `Range`. The value type of its iterator is
+   the key type of the named parameter `point_map` in `NamedParameters2`.
+
+   \param point_set_1 input point range used as reference.
+   \param point_set_2 input point range whose registration w.r.t. `point_set_1` will be computed.
+   \param np1 an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+
+   \cgalNamedParamsBegin
+     \cgalParamNBegin{point_map}
+       \cgalParamDescription{a property map associating points to the elements of the point set `point_set_1`}
+       \cgalParamType{a model of `ReadablePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange1` and whose value type is `geom_traits::Point_3`}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{normal_map}
+       \cgalParamDescription{a property map associating normals to the elements of the point set `point_set_1`}
+       \cgalParamType{a model of `ReadablePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange1` and whose value type is `geom_traits::Vector_3`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{point_set_filters}
+       \cgalParamDescription{a chain of filters to be applied to the point set}
+       \cgalParamType{a class model of `Range`. The value type of its iterator must be `ICP_config`.}
+       \cgalParamDefault{`RandomSamplingDataPointsFilter`}
+       \cgalParamExtra{The chain of filters to be applied to the reference point cloud. The reference
+                       point cloud is processed into an intermediate point cloud with the given chain
+                       of filters to be used in the alignment procedure. The chain is organized with
+                       the forward traversal order of the point set filters range.
+
+                       The chain of point set filters are applied only once at the beginning of the
+                       ICP procedure, i.e., before the first iteration of the ICP algorithm.
+
+                       The filters can have several purposes, including but are not limited to
+                       i) removal of noisy points which render alignment of point clouds difficult,
+                       ii) removal of redundant points so as to speed up alignment, iii) addition
+                       of descriptive information to the points such as a surface normal vector,
+                       or the direction from the point to the sensor.}
+       \cgalParamExtra{Corresponds to `referenceDataPointsFilters` configuration module of \ref thirdpartylibpointmatcher
+                       library. The filters should be chosen and set from possible components of
+                       the `referenceDataPointsFilters` configuration module.
+                       See <a href="https://libpointmatcher.readthedocs.io/en/latest/Configuration/#configuration-of-an-icp-chain">libpointmatcher documentation</a>
+                       for possible configurations.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{matcher}
+       \cgalParamDescription{a method used for matching (linking) the points from `point_set_2`,
+                             to the points in the reference cloud, `point_set_1`}
+       \cgalParamType{a class model of `ICP_config`}
+       \cgalParamDefault{`KDTreeMatcher`}
+       \cgalParamExtra{Corresponds to the `matcher` configuration module of \ref thirdpartylibpointmatcher
+                       library. The matcher should be chosen and set from possible components of
+                       the `matcher` configuration module.
+                       See <a href="https://libpointmatcher.readthedocs.io/en/latest/Configuration/#configuration-of-an-icp-chain">libpointmatcher documentation</a>
+                       for possible configurations.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{outlier_filters}
+       \cgalParamDescription{a chain of filters to be applied to the matched (linked) point clouds after
+                             each processing iteration of the ICP algorithm to remove the links which do not
+                             correspond to true point correspondences}
+       \cgalParamType{a model of `Range`. The value type of its iterator must be `ICP_config`.}
+       \cgalParamDefault{`TrimmedDistOutlierFilter`}
+       \cgalParamExtra{The outliers are rejected. Points with no link are ignored
+                       in the subsequent error minimization step. The chain is organized
+                       with the forward traversal order of the outlier filters range.}
+       \cgalParamExtra{Corresponds to the `outlierFilters` configuration module of \ref thirdpartylibpointmatcher
+                       library. The filters should be chosen and set from possible components of
+                       the `outlierFilters` configuration module.
+                       See <a href="https://libpointmatcher.readthedocs.io/en/latest/Configuration/#configuration-of-an-icp-chain">libpointmatcher documentation</a>
+                       for possible configurations.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{error_minimizer}
+       \cgalParamDescription{an error minimizer that computes a transformation matrix such as to minimize
+                             the error between the point sets}
+       \cgalParamType{a class model of `ICP_config`}
+       \cgalParamDefault{`PointToPlaneErrorMinimizer`}
+       \cgalParamExtra{Corresponds to the `errorMinimizer` configuration module of \ref thirdpartylibpointmatcher
+                       library. The error minimizer should be chosen and set from possible components of
+                       the `errorMinimizer` configuration module.
+                       See <a href="https://libpointmatcher.readthedocs.io/en/latest/Configuration/#configuration-of-an-icp-chain">libpointmatcher documentation</a>
+                       for possible configurations.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{transformation_checkers}
+       \cgalParamDescription{a chain of transformation checkers}
+       \cgalParamType{a class model of `Range`. The value type of its iterator must be `ICP_config`.}
+       \cgalParamDefault{`CounterTransformationChecker` and `DifferentialTransformationChecker`}
+       \cgalParamExtra{The chain is organized with the forward traversal order of the transformation checkers range.}
+       \cgalParamExtra{A transformation checker can stop the iteration depending on the conditions it defines.}
+       \cgalParamExtra{Corresponds to the `transformationCheckers` configuration module of \ref thirdpartylibpointmatcher
+                       library. The transformation checkers should be chosen and set from possible components of
+                       the `transformationCheckers` configuration module.
+                       See <a href="https://libpointmatcher.readthedocs.io/en/latest/Configuration/#configuration-of-an-icp-chain">libpointmatcher documentation</a>
+                       for possible configurations.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{inspector}
+       \cgalParamDescription{an inspector that enables logging data at different steps for analysis.}
+       \cgalParamType{a class model of `ICP_config`}
+       \cgalParamDefault{`NullInspector`}
+       \cgalParamExtra{Inspectors typically provide deeper scrutiny than the logger.}
+       \cgalParamExtra{Corresponds to the `inspector` configuration module of \ref thirdpartylibpointmatcher
+                       library. The inspector should be chosen and set from possible components of
+                       the `inspector` configuration module.
+                       See <a href="https://libpointmatcher.readthedocs.io/en/latest/Configuration/#configuration-of-an-icp-chain">libpointmatcher documentation</a>
+                       for possible configurations.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{logger}
+       \cgalParamDescription{a method for logging information regarding the registration process
+                             outputted by \ref thirdpartylibpointmatcher library}
+       \cgalParamType{a class model of `ICP_config`}
+       \cgalParamDefault{`NullLogger`}
+       \cgalParamExtra{The logs generated by CGAL library does not get effected by this configuration.}
+       \cgalParamExtra{Corresponds to the `logger` configuration module of \ref thirdpartylibpointmatcher
+                       library. The logger should be chosen and set from possible components of
+                       the `logger` configuration module.
+                       See <a href="https://libpointmatcher.readthedocs.io/en/latest/Configuration/#configuration-of-an-icp-chain">libpointmatcher documentation</a>
+                       for possible configurations.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{geom_traits}
+       \cgalParamDescription{an instance of a geometric traits class}
+       \cgalParamType{a model of `Kernel`}
+       \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+     \cgalParamNEnd
+   \cgalNamedParamsEnd
+
+   \param np2 an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+
+   \cgalNamedParamsBegin
+     \cgalParamNBegin{point_map}
+       \cgalParamDescription{a property map associating points to the elements of the point set `point_set_2`}
+       \cgalParamType{a model of `ReadablePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange2` and whose value type is `geom_traits::Point_3`}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{normal_map}
+       \cgalParamDescription{a property map associating normals to the elements of the point set `point_set_2`}
+       \cgalParamType{a model of `ReadablePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange2` and whose value type is `geom_traits::Vector_3`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{point_set_filters}
+       \cgalParamDescription{a chain of filters to be applied to the point set}
+       \cgalParamType{a class model of `Range`. The value type of its iterator must be `ICP_config`.}
+       \cgalParamDefault{`SamplingSurfaceNormalDataPointsFilter`}
+       \cgalParamExtra{The chain of filters to be applied to the point cloud `point_set_2`. The
+                       point cloud is processed into an intermediate point cloud with the given chain
+                       of filters to be used in the alignment procedure. The chain is organized with
+                       the forward traversal order of the point set filters range.
+
+                       The chain of point set filters are applied only once at the beginning of the
+                       ICP procedure, i.e., before the first iteration of the ICP algorithm.
+
+                       The filters can have several purposes, including but are not limited to
+                       i) removal of noisy points which render alignment of point clouds difficult,
+                       ii) removal of redundant points so as to speed up alignment, iii) addition
+                       of descriptive information to the points such as a surface normal vector,
+                       or the direction from the point to the sensor.}
+       \cgalParamExtra{Corresponds to the `readingDataPointsFilters` configuration module of \ref thirdpartylibpointmatcher
+                       library. The filters should be chosen and set from possible components of
+                       the `readingDataPointsFilters` configuration module.
+                       See <a href="https://libpointmatcher.readthedocs.io/en/latest/Configuration/#configuration-of-an-icp-chain">libpointmatcher documentation</a>
+                       for possible configurations.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{transformation}
+       \cgalParamDescription{an affine transformation that is used as the initial transformation for `point_set_2`}
+       \cgalParamType{`CGAL::Aff_transformation_3`}
+       \cgalParamDefault{the identity transformation}
+     \cgalParamNEnd
+   \cgalNamedParamsEnd
+
+   \return `true` if registration is converged, `false` otherwise. A log why it
+   failed to converge is written to `std::cerr` if the registration cannot converge.
+*/
+template <class PointRange1, class PointRange2,
+          class NamedParameters1, class NamedParameters2>
+bool
+register_point_sets (const PointRange1& point_set_1, PointRange2& point_set_2,
+                     const NamedParameters1& np1, const NamedParameters2& np2)
+{
+  using parameters::choose_parameter;
+  using parameters::get_parameter;
+
+  namespace PSP = CGAL::Point_set_processing_3;
+  typedef typename PSP::GetK<PointRange1, NamedParameters1>::Kernel Kernel;
+
+  // compute registration transformation
+  std::pair<typename Kernel::Aff_transformation_3, bool> res =
+    compute_registration_transformation(point_set_1, point_set_2, np1, np2);
+
+  // property map type of point_set_2
+  typedef typename CGAL::GetPointMap<PointRange2, NamedParameters2>::type PointMap2;
+  PointMap2 point_map2 = choose_parameter(get_parameter(np2, internal_np::point_map), PointMap2());
+
+  // update CGAL points
+  for (typename PointRange2::iterator it=point_set_2.begin(),
+                                      end=point_set_2.end(); it!=end; ++it)
+  {
+    put(point_map2, *it, get(point_map2, *it).transform(res.first));
+  }
+
+  return res.second;
+}
+
+// convenience overloads
+template <class PointRange1, class PointRange2,
+          class NamedParameters1>
+bool
+register_point_sets(const PointRange1& point_set_1, PointRange2& point_set_2,
+                    const NamedParameters1& np1)
+{
+  namespace params = CGAL::Point_set_processing_3::parameters;
+  return register_point_sets(point_set_1, point_set_2, np1, params::all_default(point_set_1));
+}
+
+template <class PointRange1, class PointRange2>
+bool
+register_point_sets(const PointRange1& point_set_1, PointRange2& point_set_2)
+{
+  namespace params = CGAL::Point_set_processing_3::parameters;
+  return register_point_sets(point_set_1, point_set_2,
+                             params::all_default(point_set_1),
+                             params::all_default(point_set_2));
+}
+
+} } // end of namespace CGAL::pointmatcher
+
+#endif // CGAL_LINKED_WITH_POINTMATCHER
+
+#endif // CGAL_POINTMATCHER_REGISTER_POINT_SETS_H
diff -pruN 5.0.3-1/include/CGAL/Point_set_2.h 5.1-1/include/CGAL/Point_set_2.h
--- 5.0.3-1/include/CGAL/Point_set_2.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/Point_set_2.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_2/include/CGAL/Point_set_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_2/include/CGAL/Point_set_2.h $
 // $Id: Point_set_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Point_set_3/IO.h 5.1-1/include/CGAL/Point_set_3/IO.h
--- 5.0.3-1/include/CGAL/Point_set_3/IO.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Point_set_3/IO.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_3/include/CGAL/Point_set_3/IO.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_3/include/CGAL/Point_set_3/IO.h $
 // $Id: IO.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Point_set_3.h 5.1-1/include/CGAL/Point_set_3.h
--- 5.0.3-1/include/CGAL/Point_set_3.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Point_set_3.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_3/include/CGAL/Point_set_3.h $
-// $Id: Point_set_3.h 157bdbd 2020-04-22T13:26:52+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_3/include/CGAL/Point_set_3.h $
+// $Id: Point_set_3.h 038869c 2020-05-12T15:12:21+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -947,13 +947,12 @@ public:
 
 
   /*!
-    \brief Returns a sequence of \ref psp_namedparameters "Named Parameters" for Point Set Processing algorithms.
+    \brief Returns a sequence of \ref bgl_namedparameters "Named Parameters" to be used in Point Set Processing algorithms.
 
-    \cgalNamedParamsBegin
-      \cgalParamBegin{point_map} contains the point map (see `point_map()`)\cgalParamEnd
-      \cgalParamBegin{normal_map} contains the normal map (see `normal_map()`)\cgalParamEnd
-      \cgalParamBegin{geom_traits} contains the kernel `typename Kernel_traits<Point>`::`Kernel`\cgalParamEnd
-    \cgalNamedParamsEnd
+    The following named parameters are used:
+    - `point_map`: contains the point property map (see `point_map()`)
+    - `normal_map`: contains the normal map (see `normal_map()`)
+    - `geom_traits`: contains the kernel `typename Kernel_traits<Point>`::`Kernel`
 
     \warning this method does not check if the normal map was
     instanciated or not. The normal map named parameter should not be
diff -pruN 5.0.3-1/include/CGAL/Point_set_processing_3/internal/bbox_diagonal.h 5.1-1/include/CGAL/Point_set_processing_3/internal/bbox_diagonal.h
--- 5.0.3-1/include/CGAL/Point_set_processing_3/internal/bbox_diagonal.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Point_set_processing_3/internal/bbox_diagonal.h	2020-09-08 12:15:22.000000000 +0000
@@ -0,0 +1,59 @@
+// Copyright (c) 2020 GeometryFactory (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/Point_set_processing_3/internal/bbox_diagonal.h $
+// $Id: bbox_diagonal.h d6cf972 2020-04-17T09:22:57+02:00 Mael Rouxel-Labbé
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s) : Simon Giraudot
+
+#ifndef CGAL_PSP_INTERNAL_BBOX_DIAGONAL_H
+#define CGAL_PSP_INTERNAL_BBOX_DIAGONAL_H
+
+#include <CGAL/license/Point_set_processing_3.h>
+
+namespace CGAL
+{
+namespace Point_set_processing_3
+{
+namespace internal
+{
+
+template <typename Kernel, typename PointRange, typename PointMap>
+double bbox_diagonal (const PointRange& points, PointMap point_map, const typename Kernel::Point_2&)
+{
+  CGAL::Bbox_2 bbox = CGAL::bbox_2 (CGAL::make_transform_iterator_from_property_map (points.begin(), point_map),
+                                    CGAL::make_transform_iterator_from_property_map (points.end(), point_map));
+
+  return CGAL::approximate_sqrt
+    ((bbox.xmax() - bbox.xmin()) * (bbox.xmax() - bbox.xmin())
+     + (bbox.ymax() - bbox.ymin()) * (bbox.ymax() - bbox.ymin()));
+}
+
+template <typename Kernel, typename PointRange, typename PointMap>
+double bbox_diagonal (const PointRange& points, PointMap point_map, const typename Kernel::Point_3&)
+{
+  CGAL::Bbox_3 bbox = CGAL::bbox_3 (CGAL::make_transform_iterator_from_property_map (points.begin(), point_map),
+                                    CGAL::make_transform_iterator_from_property_map (points.end(), point_map));
+
+  return CGAL::approximate_sqrt
+    ((bbox.xmax() - bbox.xmin()) * (bbox.xmax() - bbox.xmin())
+     + (bbox.ymax() - bbox.ymin()) * (bbox.ymax() - bbox.ymin())
+     + (bbox.zmax() - bbox.zmin()) * (bbox.zmax() - bbox.zmin()));
+}
+
+template <typename PointRange, typename PointMap>
+double bbox_diagonal (const PointRange& points, PointMap point_map)
+{
+  typedef typename boost::property_traits<PointMap>::value_type Point;
+  return bbox_diagonal<typename Kernel_traits<Point>::Kernel> (points, point_map, Point());
+}
+
+} // namespace internal
+} // namespace Point_set_processing_3
+} // namespace CGAL
+
+
+#endif // CGAL_PSP_INTERNAL_BBOX_DIAGONAL_H
diff -pruN 5.0.3-1/include/CGAL/Point_set_processing_3/internal/Callback_wrapper.h 5.1-1/include/CGAL/Point_set_processing_3/internal/Callback_wrapper.h
--- 5.0.3-1/include/CGAL/Point_set_processing_3/internal/Callback_wrapper.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Point_set_processing_3/internal/Callback_wrapper.h	2020-09-08 12:15:22.000000000 +0000
@@ -0,0 +1,184 @@
+// Copyright (c) 2018 GeometryFactory (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/Point_set_processing_3/internal/Callback_wrapper.h $
+// $Id: Callback_wrapper.h 2e180ac 2020-03-26T19:29:44+01:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s) : Simon Giraudot
+
+#ifndef CGAL_PSP_INTERNAL_CALLBACK_WRAPPER_H
+#define CGAL_PSP_INTERNAL_CALLBACK_WRAPPER_H
+
+#include <CGAL/license/Point_set_processing_3.h>
+
+#include <functional>
+
+#include <CGAL/thread.h>
+
+namespace CGAL {
+namespace Point_set_processing_3 {
+namespace internal {
+
+template <typename ConcurrencyTag>
+class Callback_wrapper
+{
+  std::size_t m_advancement;
+  bool m_interrupted;
+
+public:
+  Callback_wrapper (const std::function<bool(double)>&, std::size_t, std::size_t, bool = false) { }
+  void reset (std::size_t, std::size_t, bool = false) { }
+  std::size_t& advancement() { return m_advancement; }
+  bool& interrupted() { return m_interrupted; }
+  void join() { }
+};
+
+template <>
+class Callback_wrapper<CGAL::Sequential_tag>
+{
+  const std::function<bool(double)>& m_callback;
+  std::size_t m_advancement;
+  bool m_interrupted;
+  std::size_t m_size;
+
+public:
+
+  Callback_wrapper (const std::function<bool(double)>& callback,
+                    std::size_t size,
+                    std::size_t advancement = 0,
+                    bool interrupted = false)
+    : m_callback (callback)
+    , m_advancement (advancement)
+    , m_interrupted (interrupted)
+    , m_size (size)
+  { }
+
+  Callback_wrapper (const Callback_wrapper& other)
+    : m_callback (other.m_callback)
+    , m_advancement (other.m_advancement)
+    , m_interrupted (other.m_interrupted)
+    , m_size (other.m_size)
+  { }
+
+  ~Callback_wrapper () { }
+
+  void reset (std::size_t size, std::size_t advancement, bool interrupted = false)
+  {
+    m_size = size;
+    m_advancement = advancement;
+    m_interrupted = interrupted;
+  }
+  std::size_t& advancement()
+  {
+    return m_advancement;
+  }
+
+  bool& interrupted()
+  {
+    if (m_callback)
+      m_interrupted = !(m_callback(m_advancement / double(m_size)));
+    return m_interrupted;
+  }
+
+  void join() { }
+};
+
+#ifdef CGAL_LINKED_WITH_TBB
+template <>
+class Callback_wrapper<CGAL::Parallel_tag>
+{
+  const std::function<bool(double)>& m_callback;
+  cpp11::atomic<std::size_t>* m_advancement;
+  cpp11::atomic<bool>* m_interrupted;
+  std::size_t m_size;
+  bool m_creator;
+  cpp11::thread* m_thread;
+
+  // assignment operator shouldn't be used (m_callback is const ref)
+  Callback_wrapper& operator= (const Callback_wrapper&)
+  {
+    return *this;
+  }
+
+public:
+  Callback_wrapper (const std::function<bool(double)>& callback,
+                     std::size_t size,
+                     std::size_t advancement = 0,
+                     bool interrupted = false)
+    : m_callback (callback)
+    , m_advancement (new cpp11::atomic<std::size_t>())
+    , m_interrupted (new cpp11::atomic<bool>())
+    , m_size (size)
+    , m_creator (true)
+    , m_thread (nullptr)
+  {
+    // cpp11::atomic only has default constructor, initialization done in two steps
+    *m_advancement = advancement;
+    *m_interrupted = interrupted;
+    if (m_callback)
+      m_thread = new cpp11::thread (*this);
+  }
+
+  Callback_wrapper (const Callback_wrapper& other)
+    : m_callback (other.m_callback)
+    , m_advancement (other.m_advancement)
+    , m_interrupted (other.m_interrupted)
+    , m_size (other.m_size)
+    , m_creator (false)
+    , m_thread (nullptr)
+  {
+
+  }
+
+  ~Callback_wrapper ()
+  {
+    if (m_creator)
+    {
+      delete m_advancement;
+      delete m_interrupted;
+    }
+    if (m_thread != nullptr)
+      delete m_thread;
+  }
+
+  void reset (std::size_t size, std::size_t advancement, bool interrupted = false)
+  {
+    m_size = size;
+    *m_advancement = advancement;
+    *m_interrupted = interrupted;
+    if (m_callback)
+      m_thread = new cpp11::thread (*this);
+  }
+
+  cpp11::atomic<std::size_t>& advancement() { return *m_advancement; }
+  cpp11::atomic<bool>& interrupted() { return *m_interrupted; }
+  void join()
+  {
+    if (m_thread != nullptr)
+      m_thread->join();
+  }
+
+  void operator()()
+  {
+    while (*m_advancement != m_size)
+    {
+      if (m_callback && !m_callback (*m_advancement / double(m_size)))
+        *m_interrupted = true;
+      if (*m_interrupted)
+        return;
+      cpp11::sleep_for (0.00001);
+    }
+    if (m_callback)
+      m_callback (1.);
+  }
+};
+#endif
+
+} // namespace internal
+} // namespace Point_set_processing_3
+} // namespace CGAL
+
+#endif // CGAL_PSP_INTERNAL_CALLBACK_WRAPPER_H
diff -pruN 5.0.3-1/include/CGAL/Point_set_processing_3/internal/neighbor_query.h 5.1-1/include/CGAL/Point_set_processing_3/internal/neighbor_query.h
--- 5.0.3-1/include/CGAL/Point_set_processing_3/internal/neighbor_query.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Point_set_processing_3/internal/neighbor_query.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,104 +0,0 @@
-// Copyright (c) 2019 GeometryFactory Sarl (France).
-// All rights reserved.
-//
-// This file is part of CGAL (www.cgal.org).
-//
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_processing_3/include/CGAL/Point_set_processing_3/internal/neighbor_query.h $
-// $Id: neighbor_query.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
-// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
-//
-// Author(s)     : Simon Giraudot
-
-#ifndef CGAL_PSP_INTERNAL_NEIGHBOR_QUERY_H
-#define CGAL_PSP_INTERNAL_NEIGHBOR_QUERY_H
-
-#include <CGAL/license/Point_set_processing_3.h>
-
-#include <CGAL/Search_traits_3.h>
-#include <CGAL/Fuzzy_sphere.h>
-#include <CGAL/Orthogonal_k_neighbor_search.h>
-#include <CGAL/point_set_processing_assertions.h>
-
-#include <boost/function_output_iterator.hpp>
-
-namespace CGAL {
-namespace Point_set_processing_3 {
-namespace internal {
-
-struct Maximum_points_reached_exception : public std::exception { };
-
-template <typename Point,
-          typename TreeTraits,
-          typename TreeSplitter,
-          typename TreeUseExtendedNode,
-          typename FT,
-          typename PointContainer>
-void neighbor_query (const Point& query,
-                     const CGAL::Kd_tree<TreeTraits, TreeSplitter, TreeUseExtendedNode>& tree,
-                     unsigned int k,
-                     FT neighbor_radius,
-                     PointContainer& points)
-{
-  typedef typename CGAL::Orthogonal_k_neighbor_search<TreeTraits> Neighbor_search;
-  typedef typename Neighbor_search::iterator Search_iterator;
-  typedef CGAL::Fuzzy_sphere<TreeTraits> Sphere;
-
-  if (neighbor_radius != FT(0))
-  {
-    Sphere fs (query, neighbor_radius, 0, tree.traits());
-
-    // if k=0, no limit on the number of neighbors returned
-    if (k == 0)
-      k = (std::numeric_limits<unsigned int>::max)();
-
-    try
-    {
-      std::function<void(const Point&)> back_insert_with_limit
-        = [&](const Point& point) -> void
-        {
-          points.push_back (point);
-          if (points.size() == k)
-            throw Maximum_points_reached_exception();
-        };
-
-      auto function_output_iterator
-        = boost::make_function_output_iterator (back_insert_with_limit);
-
-      tree.search (function_output_iterator, fs);
-    }
-    catch (const Maximum_points_reached_exception&)
-    { }
-
-    // Fallback, if less than 3 points are return, search for the 3
-    // first points
-    if (points.size() < 3)
-      k = 3;
-    // Else, no need to search for K nearest neighbors
-    else
-      k = 0;
-  }
-
-  if (k != 0)
-  {
-    // Gather set of (k+1) neighboring points.
-    // Perform k+1 queries (as in point set, the query point is
-    // output first). Search may be aborted if k is greater
-    // than number of input points.
-    points.reserve(k+1);
-    Neighbor_search search(tree,query,k+1);
-    Search_iterator search_iterator = search.begin();
-    unsigned int i;
-    for(i=0;i<(k+1);i++)
-    {
-      if(search_iterator == search.end())
-        break; // premature ending
-      points.push_back(search_iterator->first);
-      search_iterator++;
-    }
-    CGAL_point_set_processing_precondition(points.size() >= 1);
-  }
-}
-
-} } } // namespace CGAL::Point_set_processing_3::internal
-
-#endif // CGAL_PSP_INTERNAL_NEIGHBOR_QUERY_H
diff -pruN 5.0.3-1/include/CGAL/Point_set_processing_3/internal/Neighbor_query.h 5.1-1/include/CGAL/Point_set_processing_3/internal/Neighbor_query.h
--- 5.0.3-1/include/CGAL/Point_set_processing_3/internal/Neighbor_query.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Point_set_processing_3/internal/Neighbor_query.h	2020-09-08 12:15:22.000000000 +0000
@@ -0,0 +1,188 @@
+// Copyright (c) 2019 GeometryFactory Sarl (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/Point_set_processing_3/internal/Neighbor_query.h $
+// $Id: Neighbor_query.h 684f6e9 2020-07-06T14:07:13+02:00 Simon Giraudot
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Simon Giraudot
+
+#ifndef CGAL_PSP_INTERNAL_NEIGHBOR_QUERY_H
+#define CGAL_PSP_INTERNAL_NEIGHBOR_QUERY_H
+
+#include <CGAL/license/Point_set_processing_3.h>
+
+#include <CGAL/Search_traits_2.h>
+#include <CGAL/Search_traits_3.h>
+#include <CGAL/Fuzzy_sphere.h>
+#include <CGAL/Orthogonal_k_neighbor_search.h>
+#include <CGAL/point_set_processing_assertions.h>
+
+#include <CGAL/iterator.h>
+
+#include <boost/function_output_iterator.hpp>
+
+namespace CGAL {
+namespace Point_set_processing_3 {
+namespace internal {
+
+struct Maximum_points_reached_exception : public std::exception { };
+
+template <typename Kernel_, typename PointRangeRef, typename PointMap>
+class Neighbor_query
+{
+public:
+
+  typedef Kernel_ Kernel;
+  typedef PointRangeRef Point_range;
+  typedef PointMap Point_map;
+
+  typedef typename Kernel::FT FT;
+  typedef typename boost::property_traits<PointMap>::value_type Point;
+
+  typedef typename Kernel::Point_2 Point_2;
+  typedef typename Kernel::Point_3 Point_3;
+
+  typedef std::is_same<Point, Point_2> Is_2d;
+
+  typedef typename Range_iterator_type<PointRangeRef>::type input_iterator;
+  typedef typename input_iterator::value_type value_type;
+
+  typedef CGAL::Prevent_deref<input_iterator> iterator;
+
+  struct Deref_point_map
+  {
+    typedef input_iterator key_type;
+    typedef typename boost::property_traits<PointMap>::reference reference;
+    typedef typename boost::property_traits<PointMap>::value_type value_type;
+    typedef typename boost::property_traits<PointMap>::category category;
+
+    PointMap point_map;
+
+    Deref_point_map () { }
+    Deref_point_map (PointMap point_map) : point_map(point_map) { }
+
+    friend reference get (const Deref_point_map& map, key_type it)
+    {
+      return get(map.point_map, *it);
+    }
+  };
+
+  typedef typename std::conditional<Is_2d::value,
+                                    CGAL::Search_traits_2<Kernel>,
+                                    CGAL::Search_traits_3<Kernel> >::type Tree_traits_base;
+
+  typedef CGAL::Search_traits_adapter<input_iterator, Deref_point_map, Tree_traits_base> Tree_traits;
+  typedef CGAL::Sliding_midpoint<Tree_traits> Splitter;
+  typedef CGAL::Distance_adapter<input_iterator, Deref_point_map, CGAL::Euclidean_distance<Tree_traits_base> > Distance;
+  typedef CGAL::Kd_tree<Tree_traits, Splitter, CGAL::Tag_true, CGAL::Tag_true> Tree;
+  typedef CGAL::Fuzzy_sphere<Tree_traits> Sphere;
+
+  typedef CGAL::Orthogonal_k_neighbor_search<Tree_traits, Distance, Splitter, Tree> Neighbor_search;
+  typedef typename Neighbor_search::iterator Search_iterator;
+
+private:
+
+  PointRangeRef m_points;
+  PointMap m_point_map;
+  Deref_point_map m_deref_map;
+  Tree_traits m_traits;
+  Tree m_tree;
+  Distance m_distance;
+
+  // Forbid copy
+  Neighbor_query (const Neighbor_query&) { }
+
+public:
+
+  Neighbor_query (PointRangeRef points, PointMap point_map)
+    : m_points (points)
+    , m_point_map (point_map)
+    , m_deref_map (point_map)
+    , m_traits (m_deref_map)
+    , m_tree (iterator(m_points.begin()), iterator(m_points.end()), Splitter(), m_traits)
+    , m_distance (m_deref_map)
+  {
+    m_tree.build();
+  }
+
+  PointMap point_map() const { return m_point_map; }
+
+  template <typename OutputIterator>
+  void get_iterators (const Point& query, unsigned int k, FT neighbor_radius,
+                      OutputIterator output, unsigned int fallback_k_if_sphere_empty = 3) const
+  {
+    if (neighbor_radius != FT(0))
+    {
+      Sphere fs (query, neighbor_radius, 0, m_traits);
+
+      // if k=0, no limit on the number of neighbors returned
+      if (k == 0)
+        k = (std::numeric_limits<unsigned int>::max)();
+
+      unsigned int nb = 0;
+
+      try
+      {
+        std::function<void(const input_iterator&)> output_iterator_with_limit
+          = [&](const input_iterator& it)
+          {
+            *(output ++) = it;
+            if (++ nb == k)
+              throw Maximum_points_reached_exception();
+          };
+
+        auto function_output_iterator
+          = boost::make_function_output_iterator (output_iterator_with_limit);
+
+        m_tree.search (function_output_iterator, fs);
+      }
+      catch (const Maximum_points_reached_exception&)
+      { }
+
+      // Fallback, if not enough  points are return, search for the knn
+      // first points
+      if (nb < fallback_k_if_sphere_empty)
+        k = fallback_k_if_sphere_empty;
+      else
+        k = 0;
+    }
+
+    if (k != 0)
+    {
+      // Gather set of (k+1) neighboring points.
+      // Perform k+1 queries (as in point set, the query point is
+      // output first). Search may be aborted if k is greater
+      // than number of input points.
+
+      Neighbor_search search (m_tree, query, k+1, 0, true, m_distance);
+      Search_iterator search_iterator = search.begin();
+      unsigned int i;
+      for (i = 0; i < (k+1); ++ i)
+      {
+        if(search_iterator == search.end())
+          break; // premature ending
+        *(output ++) = search_iterator->first;
+        search_iterator++;
+      }
+    }
+  }
+
+  template <typename OutputIterator>
+  void get_points (const Point& query, unsigned int k, FT neighbor_radius,
+                   OutputIterator output, unsigned int fallback_k_if_sphere_empty = 3) const
+  {
+    return get_iterators(query, k, neighbor_radius,
+                         boost::make_function_output_iterator
+                         ([&](const input_iterator& it)
+                          {
+                            *(output ++) = get (m_point_map, *it);
+                          }), fallback_k_if_sphere_empty);
+  }
+};
+
+} } } // namespace CGAL::Point_set_processing_3::internal
+
+#endif // CGAL_PSP_INTERNAL_NEIGHBOR_QUERY_H
diff -pruN 5.0.3-1/include/CGAL/Point_set_processing_3/internal/Parallel_callback.h 5.1-1/include/CGAL/Point_set_processing_3/internal/Parallel_callback.h
--- 5.0.3-1/include/CGAL/Point_set_processing_3/internal/Parallel_callback.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Point_set_processing_3/internal/Parallel_callback.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,108 +0,0 @@
-// Copyright (c) 2018 GeometryFactory (France).
-// All rights reserved.
-//
-// This file is part of CGAL (www.cgal.org).
-//
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_processing_3/include/CGAL/Point_set_processing_3/internal/Parallel_callback.h $
-// $Id: Parallel_callback.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
-// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
-//
-// Author(s) : Simon Giraudot
-
-#ifndef CGAL_PSP_INTERNAL_PARALLEL_CALLBACK_H
-#define CGAL_PSP_INTERNAL_PARALLEL_CALLBACK_H
-
-#include <CGAL/license/Point_set_processing_3.h>
-
-#include <functional>
-
-#include <CGAL/thread.h>
-
-namespace CGAL {
-namespace Point_set_processing_3 {
-namespace internal {
-
-class Parallel_callback
-{
-  const std::function<bool(double)>& m_callback;
-  cpp11::atomic<std::size_t>* m_advancement;
-  cpp11::atomic<bool>* m_interrupted;
-  std::size_t m_size;
-  bool m_creator;
-  cpp11::thread* m_thread;
-
-  // assignment operator shouldn't be used (m_callback is const ref)
-  Parallel_callback& operator= (const Parallel_callback&)
-  {
-    return *this;
-  }
-
-public:
-  Parallel_callback (const std::function<bool(double)>& callback,
-                     std::size_t size,
-                     std::size_t advancement = 0,
-                     bool interrupted = false)
-    : m_callback (callback)
-    , m_advancement (new cpp11::atomic<std::size_t>())
-    , m_interrupted (new cpp11::atomic<bool>())
-    , m_size (size)
-    , m_creator (true)
-    , m_thread (nullptr)
-  {
-    // cpp11::atomic only has default constructor, initialization done in two steps
-    *m_advancement = advancement;
-    *m_interrupted = interrupted;
-    if (m_callback)
-      m_thread = new cpp11::thread (*this);
-  }
-
-  Parallel_callback (const Parallel_callback& other)
-    : m_callback (other.m_callback)
-    , m_advancement (other.m_advancement)
-    , m_interrupted (other.m_interrupted)
-    , m_size (other.m_size)
-    , m_creator (false)
-    , m_thread (nullptr)
-  {
-
-  }
-
-
-  ~Parallel_callback ()
-  {
-    if (m_creator)
-    {
-      delete m_advancement;
-      delete m_interrupted;
-    }
-    if (m_thread != nullptr)
-      delete m_thread;
-  }
-
-  cpp11::atomic<std::size_t>& advancement() { return *m_advancement; }
-  cpp11::atomic<bool>& interrupted() { return *m_interrupted; }
-  void join()
-  {
-    if (m_thread != nullptr)
-      m_thread->join();
-  }
-
-  void operator()()
-  {
-    while (*m_advancement != m_size)
-    {
-      if (!m_callback (*m_advancement / double(m_size)))
-        *m_interrupted = true;
-      if (*m_interrupted)
-        return;
-      cpp11::sleep_for (0.00001);
-    }
-    m_callback (1.);
-  }
-};
-
-} // namespace internal
-} // namespace Point_set_processing_3
-} // namespace CGAL
-
-#endif // CGAL_PSP_INTERNAL_PARALLEL_CALLBACK_H
diff -pruN 5.0.3-1/include/CGAL/Point_set_processing_3/internal/Rich_grid.h 5.1-1/include/CGAL/Point_set_processing_3/internal/Rich_grid.h
--- 5.0.3-1/include/CGAL/Point_set_processing_3/internal/Rich_grid.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Point_set_processing_3/internal/Rich_grid.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_processing_3/include/CGAL/Point_set_processing_3/internal/Rich_grid.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/Point_set_processing_3/internal/Rich_grid.h $
 // $Id: Rich_grid.h 67b53fa 2020-04-29T14:06:37+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Point_set_processing_3/internal/Search_traits_vertex_handle_3.h 5.1-1/include/CGAL/Point_set_processing_3/internal/Search_traits_vertex_handle_3.h
--- 5.0.3-1/include/CGAL/Point_set_processing_3/internal/Search_traits_vertex_handle_3.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Point_set_processing_3/internal/Search_traits_vertex_handle_3.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_processing_3/include/CGAL/Point_set_processing_3/internal/Search_traits_vertex_handle_3.h $
-// $Id: Search_traits_vertex_handle_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/Point_set_processing_3/internal/Search_traits_vertex_handle_3.h $
+// $Id: Search_traits_vertex_handle_3.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s) : Pierre Alliez and Laurent Saboret
@@ -117,8 +117,10 @@ struct Construct_cartesian_const_iterato
 template <class Vertex_handle>
 struct Euclidean_distance_vertex_handle_3
 {
+  typedef double FT;
   typedef CGAL::internal::Point_vertex_handle_3<Vertex_handle> Point_vertex_handle_3;
   typedef Point_vertex_handle_3 Query_item;
+  typedef Point_vertex_handle_3 Point_d;
 
   double transformed_distance(const Point_vertex_handle_3& p1, const Point_vertex_handle_3& p2) const {
     double distx = p1.x()-p2.x();
diff -pruN 5.0.3-1/include/CGAL/Point_set_processing_3/internal/Voronoi_covariance_3/voronoi_covariance_3.h 5.1-1/include/CGAL/Point_set_processing_3/internal/Voronoi_covariance_3/voronoi_covariance_3.h
--- 5.0.3-1/include/CGAL/Point_set_processing_3/internal/Voronoi_covariance_3/voronoi_covariance_3.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Point_set_processing_3/internal/Voronoi_covariance_3/voronoi_covariance_3.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_processing_3/include/CGAL/Point_set_processing_3/internal/Voronoi_covariance_3/voronoi_covariance_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/Point_set_processing_3/internal/Voronoi_covariance_3/voronoi_covariance_3.h $
 // $Id: voronoi_covariance_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Point_set_processing_3/internal/Voronoi_covariance_3/voronoi_covariance_sphere_3.h 5.1-1/include/CGAL/Point_set_processing_3/internal/Voronoi_covariance_3/voronoi_covariance_sphere_3.h
--- 5.0.3-1/include/CGAL/Point_set_processing_3/internal/Voronoi_covariance_3/voronoi_covariance_sphere_3.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Point_set_processing_3/internal/Voronoi_covariance_3/voronoi_covariance_sphere_3.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_processing_3/include/CGAL/Point_set_processing_3/internal/Voronoi_covariance_3/voronoi_covariance_sphere_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/Point_set_processing_3/internal/Voronoi_covariance_3/voronoi_covariance_sphere_3.h $
 // $Id: voronoi_covariance_sphere_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/point_set_processing_assertions.h 5.1-1/include/CGAL/point_set_processing_assertions.h
--- 5.0.3-1/include/CGAL/point_set_processing_assertions.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/point_set_processing_assertions.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_processing_3/include/CGAL/point_set_processing_assertions.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/point_set_processing_assertions.h $
 // $Id: point_set_processing_assertions.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Point_traits.h 5.1-1/include/CGAL/Point_traits.h
--- 5.0.3-1/include/CGAL/Point_traits.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Point_traits.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Point_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Point_traits.h $
 // $Id: Point_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Point_with_normal_3.h 5.1-1/include/CGAL/Point_with_normal_3.h
--- 5.0.3-1/include/CGAL/Point_with_normal_3.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Point_with_normal_3.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_processing_3/include/CGAL/Point_with_normal_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/Point_with_normal_3.h $
 // $Id: Point_with_normal_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Point_with_psc_localisation.h 5.1-1/include/CGAL/Point_with_psc_localisation.h
--- 5.0.3-1/include/CGAL/Point_with_psc_localisation.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Point_with_psc_localisation.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Point_with_psc_localisation.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Point_with_psc_localisation.h $
 // $Id: Point_with_psc_localisation.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Point_with_surface_index_geom_traits.h 5.1-1/include/CGAL/Point_with_surface_index_geom_traits.h
--- 5.0.3-1/include/CGAL/Point_with_surface_index_geom_traits.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Point_with_surface_index_geom_traits.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Point_with_surface_index_geom_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Point_with_surface_index_geom_traits.h $
 // $Id: Point_with_surface_index_geom_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Point_with_surface_index.h 5.1-1/include/CGAL/Point_with_surface_index.h
--- 5.0.3-1/include/CGAL/Point_with_surface_index.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Point_with_surface_index.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Point_with_surface_index.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Point_with_surface_index.h $
 // $Id: Point_with_surface_index.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Poisson_implicit_surface_3.h 5.1-1/include/CGAL/Poisson_implicit_surface_3.h
--- 5.0.3-1/include/CGAL/Poisson_implicit_surface_3.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Poisson_implicit_surface_3.h	2020-09-08 12:15:22.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Poisson_surface_reconstruction_3/include/CGAL/Poisson_implicit_surface_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Poisson_surface_reconstruction_3/include/CGAL/Poisson_implicit_surface_3.h $
 // $Id: Poisson_implicit_surface_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Poisson_mesh_cell_criteria_3.h 5.1-1/include/CGAL/Poisson_mesh_cell_criteria_3.h
--- 5.0.3-1/include/CGAL/Poisson_mesh_cell_criteria_3.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Poisson_mesh_cell_criteria_3.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Poisson_surface_reconstruction_3/include/CGAL/Poisson_mesh_cell_criteria_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Poisson_surface_reconstruction_3/include/CGAL/Poisson_mesh_cell_criteria_3.h $
 // $Id: Poisson_mesh_cell_criteria_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Poisson_reconstruction_function.h 5.1-1/include/CGAL/Poisson_reconstruction_function.h
--- 5.0.3-1/include/CGAL/Poisson_reconstruction_function.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Poisson_reconstruction_function.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Poisson_surface_reconstruction_3/include/CGAL/Poisson_reconstruction_function.h $
-// $Id: Poisson_reconstruction_function.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Poisson_surface_reconstruction_3/include/CGAL/Poisson_reconstruction_function.h $
+// $Id: Poisson_reconstruction_function.h 6b2dad1 2020-04-01T07:59:15+02:00 Simon Giraudot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Laurent Saboret, Pierre Alliez
@@ -47,6 +47,7 @@
 #include <boost/array.hpp>
 #include <boost/type_traits/is_convertible.hpp>
 #include <boost/utility/enable_if.hpp>
+#include <boost/iterator/indirect_iterator.hpp>
 
 /*!
   \file Poisson_reconstruction_function.h
@@ -104,18 +105,6 @@ struct Poisson_visitor {
   {}
 };
 
-struct Poisson_skip_vertices {
-  double ratio;
-  Random& m_random;
-  Poisson_skip_vertices(const double ratio, Random& random)
-    : ratio(ratio), m_random(random) {}
-
-  template <typename Iterator>
-  bool operator()(Iterator) const {
-    return m_random.get_double() < ratio;
-  }
-};
-
 // Given f1 and f2, two sizing fields, that functor wrapper returns
 //   max(f1, f2*f2)
 // The wrapper stores only pointers to the two functors.
@@ -415,11 +404,15 @@ public:
       // then the cell is considered as small enough, and the first sizing
       // field, more costly, is not evaluated.
 
-      typedef Filter_iterator<typename Triangulation::Input_point_iterator,
-                              Poisson_skip_vertices> Some_points_iterator;
       //make it deterministic
       Random random(0);
-      Poisson_skip_vertices skip(1.-approximation_ratio,random);
+      double ratio = 1.-approximation_ratio;
+
+      std::vector<typename Triangulation::Input_point_iterator> some_points;
+      for (typename Triangulation::Input_point_iterator
+             it = m_tr->input_points_begin(); it != m_tr->input_points_end(); ++ it)
+        if (random.get_double() >= ratio)
+          some_points.push_back (it);
 
       CGAL_TRACE_STREAM << "SPECIAL PASS that uses an approximation of the result (approximation ratio: "
                 << approximation_ratio << ")" << std::endl;
@@ -427,11 +420,8 @@ public:
 
       CGAL::Timer sizing_field_timer; sizing_field_timer.start();
       Poisson_reconstruction_function<Geom_traits>
-        coarse_poisson_function(Some_points_iterator(m_tr->input_points_end(),
-                                                     skip,
-                                                     m_tr->input_points_begin()),
-                                Some_points_iterator(m_tr->input_points_end(),
-                                                     skip),
+        coarse_poisson_function(boost::make_indirect_iterator (some_points.begin()),
+                                boost::make_indirect_iterator (some_points.end()),
                                 Normal_of_point_with_normal_map<Geom_traits>() );
       coarse_poisson_function.compute_implicit_function(solver, Poisson_visitor(),
                                                         0.);
diff -pruN 5.0.3-1/include/CGAL/poisson_refine_triangulation.h 5.1-1/include/CGAL/poisson_refine_triangulation.h
--- 5.0.3-1/include/CGAL/poisson_refine_triangulation.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/poisson_refine_triangulation.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Poisson_surface_reconstruction_3/include/CGAL/poisson_refine_triangulation.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Poisson_surface_reconstruction_3/include/CGAL/poisson_refine_triangulation.h $
 // $Id: poisson_refine_triangulation.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/poisson_surface_reconstruction.h 5.1-1/include/CGAL/poisson_surface_reconstruction.h
--- 5.0.3-1/include/CGAL/poisson_surface_reconstruction.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/poisson_surface_reconstruction.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Poisson_surface_reconstruction_3/include/CGAL/poisson_surface_reconstruction.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Poisson_surface_reconstruction_3/include/CGAL/poisson_surface_reconstruction.h $
 // $Id: poisson_surface_reconstruction.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polychain_2.h 5.1-1/include/CGAL/Polychain_2.h
--- 5.0.3-1/include/CGAL/Polychain_2.h	2020-08-01 19:02:50.000000000 +0000
+++ 5.1-1/include/CGAL/Polychain_2.h	2020-09-08 12:15:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_Linf_2/include/CGAL/Polychain_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_Linf_2/include/CGAL/Polychain_2.h $
 // $Id: Polychain_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_2/Polygon_2_algorithms_impl.h 5.1-1/include/CGAL/Polygon_2/Polygon_2_algorithms_impl.h
--- 5.0.3-1/include/CGAL/Polygon_2/Polygon_2_algorithms_impl.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_2/Polygon_2_algorithms_impl.h	2020-09-08 12:15:22.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon/include/CGAL/Polygon_2/Polygon_2_algorithms_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon/include/CGAL/Polygon_2/Polygon_2_algorithms_impl.h $
 // $Id: Polygon_2_algorithms_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_2/Polygon_2_edge_circulator.h 5.1-1/include/CGAL/Polygon_2/Polygon_2_edge_circulator.h
--- 5.0.3-1/include/CGAL/Polygon_2/Polygon_2_edge_circulator.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_2/Polygon_2_edge_circulator.h	2020-09-08 12:15:22.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon/include/CGAL/Polygon_2/Polygon_2_edge_circulator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon/include/CGAL/Polygon_2/Polygon_2_edge_circulator.h $
 // $Id: Polygon_2_edge_circulator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_2/Polygon_2_edge_iterator.h 5.1-1/include/CGAL/Polygon_2/Polygon_2_edge_iterator.h
--- 5.0.3-1/include/CGAL/Polygon_2/Polygon_2_edge_iterator.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_2/Polygon_2_edge_iterator.h	2020-09-08 12:15:22.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon/include/CGAL/Polygon_2/Polygon_2_edge_iterator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon/include/CGAL/Polygon_2/Polygon_2_edge_iterator.h $
 // $Id: Polygon_2_edge_iterator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_2/Polygon_2_impl.h 5.1-1/include/CGAL/Polygon_2/Polygon_2_impl.h
--- 5.0.3-1/include/CGAL/Polygon_2/Polygon_2_impl.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_2/Polygon_2_impl.h	2020-09-08 12:15:22.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon/include/CGAL/Polygon_2/Polygon_2_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon/include/CGAL/Polygon_2/Polygon_2_impl.h $
 // $Id: Polygon_2_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_2/Polygon_2_simplicity.h 5.1-1/include/CGAL/Polygon_2/Polygon_2_simplicity.h
--- 5.0.3-1/include/CGAL/Polygon_2/Polygon_2_simplicity.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_2/Polygon_2_simplicity.h	2020-09-08 12:15:22.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon/include/CGAL/Polygon_2/Polygon_2_simplicity.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon/include/CGAL/Polygon_2/Polygon_2_simplicity.h $
 // $Id: Polygon_2_simplicity.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_2/Polygon_2_vertex_circulator.h 5.1-1/include/CGAL/Polygon_2/Polygon_2_vertex_circulator.h
--- 5.0.3-1/include/CGAL/Polygon_2/Polygon_2_vertex_circulator.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_2/Polygon_2_vertex_circulator.h	2020-09-08 12:15:22.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon/include/CGAL/Polygon_2/Polygon_2_vertex_circulator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon/include/CGAL/Polygon_2/Polygon_2_vertex_circulator.h $
 // $Id: Polygon_2_vertex_circulator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_2/polygon_assertions.h 5.1-1/include/CGAL/Polygon_2/polygon_assertions.h
--- 5.0.3-1/include/CGAL/Polygon_2/polygon_assertions.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_2/polygon_assertions.h	2020-09-08 12:15:22.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon/include/CGAL/Polygon_2/polygon_assertions.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon/include/CGAL/Polygon_2/polygon_assertions.h $
 // $Id: polygon_assertions.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_2_algorithms.h 5.1-1/include/CGAL/Polygon_2_algorithms.h
--- 5.0.3-1/include/CGAL/Polygon_2_algorithms.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_2_algorithms.h	2020-09-08 12:15:22.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon/include/CGAL/Polygon_2_algorithms.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon/include/CGAL/Polygon_2_algorithms.h $
 // $Id: Polygon_2_algorithms.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_2.h 5.1-1/include/CGAL/Polygon_2.h
--- 5.0.3-1/include/CGAL/Polygon_2.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_2.h	2020-09-08 12:15:22.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon/include/CGAL/Polygon_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon/include/CGAL/Polygon_2.h $
 // $Id: Polygon_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygonal_schema_fwd.h 5.1-1/include/CGAL/Polygonal_schema_fwd.h
--- 5.0.3-1/include/CGAL/Polygonal_schema_fwd.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Polygonal_schema_fwd.h	2020-09-08 12:15:45.000000000 +0000
@@ -0,0 +1,39 @@
+// Copyright (c) 2019 CNRS and LIRIS' Establishments (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_topology/include/CGAL/Polygonal_schema_fwd.h $
+// $Id: Polygonal_schema_fwd.h 0308d1a 2020-03-27T18:35:15+01:00 Guillaume Damiand
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
+//
+#ifndef CGAL_POLYGONAL_SCHEMA_FWD_H
+#define CGAL_POLYGONAL_SCHEMA_FWD_H 1
+
+#include <CGAL/license/Surface_mesh_topology.h>
+
+#include <CGAL/memory.h>
+#include <CGAL/Combinatorial_map_fwd.h>
+#include <CGAL/Generalized_map_fwd.h>
+
+namespace CGAL {
+namespace Surface_mesh_topology {
+
+struct Polygonal_schema_min_items;
+
+template <class Items_=Polygonal_schema_min_items,
+          class Alloc_=CGAL_ALLOCATOR(int),
+          class Storage_= Combinatorial_map_storage_1<2, Items_, Alloc_> >
+class Polygonal_schema_with_combinatorial_map;
+
+template <class Items_=Polygonal_schema_min_items,
+          class Alloc_=CGAL_ALLOCATOR(int),
+          class Storage_= Generalized_map_storage_1<2, Items_, Alloc_> >
+class Polygonal_schema_with_generalized_map;
+
+} // Surface_mesh_topology
+} // CGAL
+
+#endif // CGAL_POLYGONAL_SCHEMA_FWD_H
diff -pruN 5.0.3-1/include/CGAL/Polygonal_schema.h 5.1-1/include/CGAL/Polygonal_schema.h
--- 5.0.3-1/include/CGAL/Polygonal_schema.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Polygonal_schema.h	2020-09-08 12:15:45.000000000 +0000
@@ -0,0 +1,594 @@
+// Copyright (c) 2019 CNRS and LIRIS' Establishments (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_topology/include/CGAL/Polygonal_schema.h $
+// $Id: Polygonal_schema.h 52186a0 2020-05-14T11:38:15+02:00 Guillaume Damiand
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
+//
+#ifndef CGAL_POLYGONAL_SCHEMA_H
+#define CGAL_POLYGONAL_SCHEMA_H 1
+
+#include <CGAL/license/Surface_mesh_topology.h>
+
+#include <CGAL/Polygonal_schema_fwd.h>
+#include <vector>
+#include <unordered_map>
+#include <cstddef>
+#include <string>
+#include <initializer_list>
+#include <algorithm>
+#include <random>
+#include <CGAL/assertions.h>
+#include <CGAL/memory.h>
+#include <CGAL/Polygonal_schema_min_items.h>
+#include <CGAL/Combinatorial_map.h>
+#include <CGAL/Generalized_map.h>
+#include <CGAL/Random.h>
+
+namespace CGAL {
+namespace Surface_mesh_topology {
+
+  namespace internal
+  {
+    /// @return opposite label of label s
+    ///    (i.e. add/remove - depending if s is positive or negative)
+    inline std::string opposite_label(const std::string& s)
+    {
+      CGAL_assertion(!s.empty());
+      if (s[0]=='-')
+      { return s.substr(1, std::string::npos); }
+
+      return std::string("-")+s;
+    }
+
+    template<class Map, class Combinatorial_data_structure=
+             typename Map::Combinatorial_data_structure>
+    struct Polygonal_schema_tools
+    {};
+    template<class CMap>
+    struct Polygonal_schema_tools<CMap, Combinatorial_map_tag>
+    {
+      typedef typename CMap::Dart_handle Dart_handle;
+
+      static Dart_handle
+      add_edge_to_face(CMap& cmap, const std::string& s,
+                       Dart_handle prev_dart,
+                       Dart_handle dart_same_label,
+                       Dart_handle dart_opposite_label,
+                       std::unordered_map<std::string, Dart_handle>&
+                       edge_label_to_dart)
+      {
+        if (dart_same_label!=CMap::null_handle && dart_opposite_label!=CMap::null_handle)
+        {
+          std::cerr<<"Polygonal_schema ERROR: "<<"both labels "<<s
+                   <<" and "<<internal::opposite_label(s)
+                   <<" are already added in the surface."
+                   <<" This label can not be use anymore."<<std::endl;
+          return CMap::null_handle;
+        }
+
+        if (dart_same_label!=CMap::null_handle)
+        {
+          std::cerr<<"Polygonal_schema ERROR: "<<"label "<<s
+                   <<" is already added in the surface."
+                   <<" Since the surface is orientable, this label can "
+                   <<"not be use anymore."<<std::endl;
+          return CMap::null_handle;
+        }
+
+        Dart_handle res=cmap.create_dart();
+        edge_label_to_dart[s]=res;
+        cmap.info(res).m_label=s;
+
+        if (prev_dart!=cmap.null_handle)
+        { cmap.template link_beta<1>(prev_dart, res); }
+
+        if (dart_opposite_label!=CMap::null_handle)
+        { cmap.template link_beta<2>(res, dart_opposite_label); }
+
+        return res;
+      }
+
+      const std::string& get_label(CMap& cmap, Dart_handle dh) const
+      { return cmap.info(dh).m_label; }
+    };
+    template<class GMap>
+    struct Polygonal_schema_tools<GMap, Generalized_map_tag>
+    {
+      typedef typename GMap::Dart_handle Dart_handle;
+
+      // In a GMap, if an edge is 2-free, only one of its two dart has one label.
+      // Otherwise, d has one label and alpha<0,2>(d) the opposite label.
+      static Dart_handle
+      add_edge_to_face(GMap& gmap, const std::string& s,
+                       Dart_handle prev_dart,
+                       Dart_handle dart_same_label,
+                       Dart_handle dart_opposite_label,
+                       std::unordered_map<std::string, Dart_handle>&
+                       edge_label_to_dart)
+      {
+        if (dart_same_label!=GMap::null_handle && dart_opposite_label!=GMap::null_handle)
+        {
+          std::cerr<<"Polygonal_schema ERROR: "<<"both labels "<<s
+                   <<" and "<<internal::opposite_label(s)
+                   <<" are already added in the surface."
+                   <<" This label can not be use anymore."<<std::endl;
+          return GMap::null_handle;
+        }
+
+        Dart_handle res=gmap.create_dart();
+        Dart_handle dh2=gmap.create_dart();
+
+        gmap.template link_alpha<0>(res, dh2);
+        if (prev_dart!=gmap.null_handle)
+        { gmap.template link_alpha<1>(res, gmap.template alpha<0>(prev_dart)); }
+
+        if (dart_same_label!=GMap::null_handle)
+        { // Here dart_same_label!=GMap::null_handle
+          std::string s2=internal::opposite_label(s);
+          edge_label_to_dart[s2]=dh2;
+          gmap.info(dh2).m_label=s2;
+
+          gmap.template sew<2>(res, dart_same_label);
+        }
+        else
+        { // Here either dart_opposite_label!=GMap::null_handle, or both are GMap::null_handle
+          edge_label_to_dart[s]=res;
+          gmap.info(res).m_label=s;
+
+          if (dart_opposite_label!=GMap::null_handle)
+          {
+            std::string s2=internal::opposite_label(s);
+            edge_label_to_dart[s2]=res;
+            gmap.info(res).m_label=s2;
+
+            gmap.template sew<2>(dh2, dart_opposite_label);
+          }
+        }
+
+        return res;
+      }
+
+      std::string get_label(GMap& gmap, Dart_handle dh) const
+      {
+        if (gmap.info(dh).m_label.empty())
+        {
+          if (!gmap.template is_free<2>(dh))
+          { return gmap.info(gmap.template alpha<2>(dh)).m_label; }
+          else
+          {
+            return internal::opposite_label(gmap.info(gmap.template alpha<0>(dh)));
+          }
+        }
+        return gmap.info(dh).m_label;
+      }
+    };
+
+  }
+  // end namespace internal
+
+  struct Combinatorial_map_tag;
+  struct Generalized_map_tag;
+
+  template < class BaseModel >
+  class Polygonal_schema_base: public BaseModel
+  {
+  public:
+    typedef BaseModel                       Base;
+    typedef Polygonal_schema_base           Self;
+    typedef BaseModel                       Map; // Either a GMap or a CMap
+    typedef typename Map::Dart_handle       Dart_handle;
+    typedef typename Map::Dart_const_handle Dart_const_handle;
+    typedef typename Map::size_type         size_type;
+
+    Polygonal_schema_base() : Base(),
+      mark_perforated(this->get_new_mark()),
+      first_dart(this->null_handle),
+      prev_dart(this->null_handle),
+      facet_started(false)
+    {}
+
+    /// Start a new facet.
+    void init_facet()
+    {
+      if (facet_started)
+      {
+        std::cerr<<"Polygonal_schema ERROR: "
+                 <<"you try to start a facet"
+                 <<" but the previous facet is not yet ended."<<std::endl;
+        return;
+      }
+
+      first_dart = this->null_handle;
+      prev_dart  = this->null_handle;
+      facet_started=true;
+    }
+
+    /// Add one edge to the current facet, given by its label
+    ///  (any string, using minus sign for orientation)
+    void add_edge_to_facet(const std::string& s)
+    {
+      if (!facet_started)
+      {
+        std::cerr<<"Polygonal_schema ERROR: "
+                 <<"you try to add an edge to a facet"
+                 <<" but the facet is not yet started."<<std::endl;
+        return;
+      }
+
+      Dart_handle dart_same_label=get_dart_labeled(s);
+      Dart_handle dart_opposite_label=get_dart_labeled
+                                      (internal::opposite_label(s));
+
+      Dart_handle cur=internal::Polygonal_schema_tools<Map>::
+        add_edge_to_face(*this, s, prev_dart, dart_same_label,
+                         dart_opposite_label, edge_label_to_dart);
+
+      if (prev_dart==this->null_handle)
+      { first_dart=cur; }
+
+      prev_dart=cur;
+    }
+
+    /// add all the given edges to the current facet.
+    /// @param s the sequence of labels of edges to add.
+    void add_edges_to_facet(const std::string& s)
+    {
+      if (!facet_started)
+      {
+        std::cerr<<"Polygonal_schema ERROR: "
+                 <<"you try to add edges to a facet"
+                 <<" but the facet is not yet started."<<std::endl;
+        return;
+      }
+      std::istringstream iss(s);
+      for (std::string token; std::getline(iss, token, ' '); )
+      { add_edge_to_facet(token); }
+    }
+
+    /// add one facet, given a sequence of labels.
+    /// @param s the sequence of labels of edges to add.
+    void add_facet(const std::string& s)
+    {
+      if (facet_started)
+      {
+        std::cerr<<"Polygonal_schema ERROR: "
+                 <<"you try to add a new facet"
+                 <<" but the previous facet is not yet ended."<<std::endl;
+        return;
+      }
+      init_facet();
+      add_edges_to_facet(s);
+      finish_facet();
+    }
+
+    /// add edges to the current facet,
+    ///  given a sequence of labels, as an initializer list.
+    void add_edges_to_facet(std::initializer_list<const char*> l)
+    {
+      if (!facet_started)
+      {
+        std::cerr<<"Polygonal_schema ERROR: "
+                 <<"you try to add edges to a facet"
+                 <<" but the facet is not yet started."<<std::endl;
+        return;
+      }
+       for (const char* e : l)
+       { add_edge_to_facet(e); }
+    }
+
+    /// add a new facet, given a sequence of labels, as an initializer list.
+    void add_facet(std::initializer_list<const char*> l)
+    {
+      if (facet_started)
+      {
+        std::cerr<<"Polygonal_schema ERROR: "
+                 <<"you try to add a new facet"
+                 <<" but the previous facet is not yet ended."<<std::endl;
+        return;
+      }
+      init_facet();
+      add_edges_to_facet(l);
+      finish_facet();
+    }
+
+    /// End of the facet. Return the first dart of this facet.
+    Dart_handle finish_facet()
+    {
+      if (!facet_started)
+      {
+        std::cerr<<"Polygonal_schema ERROR: "
+                 <<"you try to end a facet"
+                 <<" but the facet is not yet started."<<std::endl;
+        return Map::null_handle;
+      }
+      CGAL_assertion( first_dart!=this->null_handle &&
+                                  prev_dart!=this->null_handle );
+      this->set_next(prev_dart, first_dart);
+
+      facet_started=false;
+      return first_dart;
+    }
+
+    /// @return dart with the given label, Map::null_handle if this dart does not exist.
+    Dart_handle get_dart_labeled(const std::string& s) const
+    {
+      auto ite=edge_label_to_dart.find(s);
+      if (ite==edge_label_to_dart.end())
+      { return Map::null_handle; }
+
+      return ite->second;
+    }
+
+    std::string get_label(Dart_handle dh) const
+    { return internal::Polygonal_schema_tools<Map>::get_label(dh); }
+
+    /// marks the whole facet containing dh as perforated
+    /// @return the number of darts of the marked face
+    size_type perforate_facet(Dart_handle dh)
+    {
+      if (this->is_marked(dh, mark_perforated))
+      { return 0; }
+
+      return this->template mark_cell<2>(dh, mark_perforated);
+    }
+
+    /// same method but using a label
+    size_type perforate_facet(const std::string& s)
+    {
+      auto ite=edge_label_to_dart.find(s);
+      if (ite==edge_label_to_dart.end())
+      {// maybe there is no need to put an error message
+        std::cerr<<"Polygonal_schema ERROR: "
+                 <<"you try to label "<<s<<" to be a border"
+                 <<" but this label does not exist yet."<<std::endl;
+        return 0;
+      }
+
+      return perforate_facet(ite->second);
+    }
+
+    /// unmark the facet as being perforated, now the facet is filled
+    /// @return the number of darts of the unmarked face
+    size_type fill_facet(Dart_handle dh)
+    {
+      if (!this->is_marked(dh, mark_perforated))
+      { return 0; }
+
+      return this->template unmark_cell<2>(dh, mark_perforated);
+    }
+
+    /// same fonciton but using a label
+    size_type fill_facet(const std::string& s)
+    {
+      auto ite=edge_label_to_dart.find(s);
+      if (ite==edge_label_to_dart.end())
+      {// maybe there is no need to put an error message
+        std::cerr<<"Polygonal_schema ERROR: "
+                 <<"you try to label "<<s<<" to be a non border"
+                 <<" but this label does not exist yet."<<std::endl;
+        return 0;
+      }
+
+      return fill_facet(ite->second);
+    }
+
+    /// @return true iff dh is on a perforated facet
+    bool is_perforated(Dart_const_handle dh) const
+    { return this->is_marked(dh, mark_perforated); }
+
+    /// same thing but using a label instead of a dart
+    bool is_perforated(const std::string& s) const
+    {
+      auto ite=edge_label_to_dart.find(s);
+      if (ite==edge_label_to_dart.end())
+      {// maybe there is no need to put an error message
+        std::cerr<<"Polygonal_schema ERROR: "
+                 <<"you ask if label "<<s<<" represents a dart border"
+                 <<" but this label does not exist yet."<<std::endl;
+        return false;
+      }
+
+      return is_perforated(ite->second);
+    }
+
+    std::size_t get_perforated_mark() const
+    { return mark_perforated; }
+
+    void display_perforated_darts() const
+    {
+      std::cout<<"labels is_free<2> is_perforated"<<std::endl;
+      for (auto it=edge_label_to_dart.begin(), itend=edge_label_to_dart.end();
+           it!=itend; ++it)
+      {
+        std::cout<<it->first<<" "<<Self::template is_free<2>(it->second)
+                 <<" "<<is_perforated(it->second)<<std::endl;
+      }
+    }
+
+  protected:
+    // For each edge label, its corresponding dart. Stores both association
+    // a -a, to allow users to start to add either a or -a.
+    std::unordered_map<std::string, Dart_handle> edge_label_to_dart;
+    std::size_t mark_perforated; // mark for perforated facets.
+
+    // Data members used when we create a facet.
+    Dart_handle first_dart;
+    Dart_handle prev_dart;
+    bool        facet_started;
+  };
+
+  /// Polygonal schema with combinatorial map.
+  template <class Items_, class Alloc_, class Storage_>
+  class Polygonal_schema_with_combinatorial_map:
+    public Polygonal_schema_base<CGAL::Combinatorial_map_base
+      <2,
+      Polygonal_schema_with_combinatorial_map<Items_, Alloc_, Storage_>,
+      Items_, Alloc_, Storage_> >
+  {
+  public:
+    typedef Polygonal_schema_with_combinatorial_map<Items_, Alloc_, Storage_> Self;
+    typedef Combinatorial_map_base<2, Self, Items_, Alloc_, Storage_>         CMap_base;
+    typedef Polygonal_schema_base<CMap_base>                                  Base;
+
+    typedef typename Base::Dart_handle Dart_handle;
+    typedef typename Base::Dart_const_handle Dart_const_handle;
+
+    Polygonal_schema_with_combinatorial_map() : Base()
+    {}
+
+    Polygonal_schema_with_combinatorial_map(const Self& amap) : Base(amap)
+    {}
+
+    template <unsigned int d2, typename Refs2, typename Items2, typename Alloc2,
+              typename Storage2>
+    Polygonal_schema_with_combinatorial_map(const Combinatorial_map_base
+                                            <d2, Refs2, Items2, Alloc2, Storage2>&
+                                            amap) : Base(amap)
+    {}
+
+    template <unsigned int d2, typename Refs2, typename Items2, typename Alloc2,
+              typename Storage2, typename Converters>
+    Polygonal_schema_with_combinatorial_map(const Combinatorial_map_base
+                                            <d2, Refs2, Items2, Alloc2, Storage2>&
+                                            amap, const Converters& converters) :
+      Base(amap, converters)
+    {}
+
+    template <unsigned int d2, typename Refs2, typename Items2, typename Alloc2,
+              typename Storage2, typename Converters,
+              typename DartInfoConverter>
+    Polygonal_schema_with_combinatorial_map(const Combinatorial_map_base
+                                            <d2, Refs2, Items2, Alloc2, Storage2>&
+                                            amap, const Converters& converters,
+                                            const DartInfoConverter& dartinfoconverter) :
+      Base(amap, converters, dartinfoconverter)
+    {}
+
+    template <unsigned int d2, typename Refs2, typename Items2, typename Alloc2,
+              typename Storage2, typename Converters,
+              typename DartInfoConverter, typename PointConverter >
+    Polygonal_schema_with_combinatorial_map(const Combinatorial_map_base
+                                            <d2, Refs2, Items2, Alloc2, Storage2>&
+                                            amap, const Converters& converters,
+                                            const DartInfoConverter& dartinfoconverter,
+                                            const PointConverter& pointconverter) :
+      Base(amap, converters, dartinfoconverter, pointconverter)
+    {}
+  };
+
+  /// Polygonal schema with generalized map.
+  template <class Items_, class Alloc_, class Storage_>
+  class Polygonal_schema_with_generalized_map:
+    public Polygonal_schema_base<CGAL::Generalized_map_base
+      <2,
+      Polygonal_schema_with_generalized_map<Items_, Alloc_, Storage_>,
+      Items_, Alloc_, Storage_> >
+  {
+  public:
+    typedef Polygonal_schema_with_generalized_map<Items_, Alloc_, Storage_> Self;
+    typedef Generalized_map_base<2, Self, Items_, Alloc_, Storage_>         GMap_base;
+    typedef Polygonal_schema_base<GMap_base>                                Base;
+
+    typedef typename Base::Dart_handle Dart_handle;
+    typedef typename Base::Dart_const_handle Dart_const_handle;
+
+    Polygonal_schema_with_generalized_map() : Base()
+    {}
+
+    Polygonal_schema_with_generalized_map(const Self& amap) : Base(amap)
+    {}
+
+    template <unsigned int d2, typename Refs2, typename Items2, typename Alloc2,
+              typename Storage2>
+    Polygonal_schema_with_generalized_map(const Generalized_map_base
+                                          <d2, Refs2, Items2, Alloc2, Storage2>&
+                                            amap) : Base(amap)
+    {}
+
+    template <unsigned int d2, typename Refs2, typename Items2, typename Alloc2,
+              typename Storage2, typename Converters>
+    Polygonal_schema_with_generalized_map(const Generalized_map_base
+                                          <d2, Refs2, Items2, Alloc2, Storage2>&
+                                            amap, const Converters& converters) :
+      Base(amap, converters)
+    {}
+
+    template <unsigned int d2, typename Refs2, typename Items2, typename Alloc2,
+              typename Storage2, typename Converters,
+              typename DartInfoConverter>
+    Polygonal_schema_with_generalized_map(const Generalized_map_base
+                                          <d2, Refs2, Items2, Alloc2, Storage2>&
+                                            amap, const Converters& converters,
+                                            const DartInfoConverter& dartinfoconverter) :
+      Base(amap, converters, dartinfoconverter)
+    {}
+
+    template <unsigned int d2, typename Refs2, typename Items2, typename Alloc2,
+              typename Storage2, typename Converters,
+              typename DartInfoConverter, typename PointConverter >
+    Polygonal_schema_with_generalized_map(const Generalized_map_base
+                                          <d2, Refs2, Items2, Alloc2, Storage2>&
+                                            amap, const Converters& converters,
+                                            const DartInfoConverter& dartinfoconverter,
+                                            const PointConverter& pointconverter) :
+      Base(amap, converters, dartinfoconverter, pointconverter)
+    {}
+  };
+
+  /// Generate a random polygonal schema ps.
+  /// @param nb_labels the number of labels used to generate ps.
+  /// @param seed the seed used for random
+  /// @param max_dart_per_face maximal number of darts per face
+  /// @param closed if true generates a closed polygonal schema.
+  /// @param percentage_of_perforated percentage of perforated faces. If 0
+  ///         no perforated faces.
+  template<typename PS>
+  void generate_random_polygonal_schema(PS& ps, std::size_t nb_labels,
+                                        unsigned int seed
+                                        =static_cast<unsigned int>(std::time(nullptr)),
+                                        std::size_t max_dart_per_face=30,
+                                        bool closed=true,
+                                        std::size_t percentage_of_perforated=20)
+  {
+    CGAL::Random random(seed);
+    std::vector<std::string> all_labels(nb_labels*2);
+    for (std::size_t i=0; i<nb_labels; ++i)
+    {
+      all_labels[2*i]=std::to_string(static_cast<int>(i)+1);
+      all_labels[(2*i)+1]=std::to_string(-(static_cast<int>(i)+1));
+    }
+
+    std::shuffle(all_labels.begin(), all_labels.end(),
+                 std::default_random_engine(seed));
+
+    std::size_t endlabel=all_labels.size();
+    if (!closed)
+    { endlabel-=(all_labels.size()/10); } // We remove 10% of labels.
+
+    for (std::size_t i=0; i<endlabel; )
+    {
+      ps.init_facet();
+      for (std::size_t j=0,
+           nb=static_cast<std::size_t>
+           (random.get_int(1, static_cast<int>(max_dart_per_face)));
+           i<endlabel && j<nb; ++i, ++j)
+      { ps.add_edges_to_facet(all_labels[i]); }
+      typename PS::Dart_handle dh=ps.finish_facet();
+
+      if (static_cast<std::size_t>(rand()%100)<percentage_of_perforated)
+      { ps.perforate_facet(dh); }
+    }
+
+    ps.keep_biggest_connected_component(); // We keep only the biggest cc.
+  }
+
+} //namespace Surface_mesh_topology
+} //namespace CGAL
+
+#endif // CGAL_POLYGONAL_SCHEMA_H //
+// EOF //
diff -pruN 5.0.3-1/include/CGAL/Polygonal_schema_min_items.h 5.1-1/include/CGAL/Polygonal_schema_min_items.h
--- 5.0.3-1/include/CGAL/Polygonal_schema_min_items.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Polygonal_schema_min_items.h	2020-09-08 12:15:45.000000000 +0000
@@ -0,0 +1,45 @@
+// Copyright (c) 2019 CNRS and LIRIS' Establishments (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_topology/include/CGAL/Polygonal_schema_min_items.h $
+// $Id: Polygonal_schema_min_items.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
+//
+#ifndef CGAL_POLYGONAL_SCHEMA_MIN_ITEMS_H
+#define CGAL_POLYGONAL_SCHEMA_MIN_ITEMS_H 1
+
+#include <CGAL/license/Surface_mesh_topology.h>
+#include <string>
+
+namespace CGAL {
+namespace Surface_mesh_topology {
+
+  /** @file Polygonal_schema_min_items.h
+   * Definition of min item class for Polygonal_schema.
+   */
+
+  /** Minimal items for polygonal schema.
+   * Generic_map_min_items defines what is the minimal item class for a generic map.
+   * One struct associated with darts, having one std::string named m_label..
+   */
+  struct Polygonal_schema_min_items
+  {
+    template < class Refs >
+    struct Dart_wrapper
+    {
+      struct Info_for_darts
+      { std::string m_label; };
+
+      typedef Info_for_darts Dart_info;
+    };
+  };
+
+} // namespace Surface_mesh_topology
+} // namespace CGAL
+
+#endif // CGAL_POLYGONAL_SCHEMA_MIN_ITEMS_H
+// EOF //
diff -pruN 5.0.3-1/include/CGAL/Polygonal_surface_reconstruction.h 5.1-1/include/CGAL/Polygonal_surface_reconstruction.h
--- 5.0.3-1/include/CGAL/Polygonal_surface_reconstruction.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Polygonal_surface_reconstruction.h	2020-09-08 12:15:23.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygonal_surface_reconstruction/include/CGAL/Polygonal_surface_reconstruction.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygonal_surface_reconstruction/include/CGAL/Polygonal_surface_reconstruction.h $
 // $Id: Polygonal_surface_reconstruction.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_convex_decomposition_2.h 5.1-1/include/CGAL/Polygon_convex_decomposition_2.h
--- 5.0.3-1/include/CGAL/Polygon_convex_decomposition_2.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_convex_decomposition_2.h	2020-09-08 12:15:11.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Minkowski_sum_2/include/CGAL/Polygon_convex_decomposition_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Minkowski_sum_2/include/CGAL/Polygon_convex_decomposition_2.h $
 // $Id: Polygon_convex_decomposition_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/polygon_function_objects.h 5.1-1/include/CGAL/polygon_function_objects.h
--- 5.0.3-1/include/CGAL/polygon_function_objects.h	2020-08-01 19:02:37.000000000 +0000
+++ 5.1-1/include/CGAL/polygon_function_objects.h	2020-09-08 12:15:18.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Partition_2/include/CGAL/polygon_function_objects.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Partition_2/include/CGAL/polygon_function_objects.h $
 // $Id: polygon_function_objects.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/bbox.h 5.1-1/include/CGAL/Polygon_mesh_processing/bbox.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/bbox.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/bbox.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/bbox.h $
-// $Id: bbox.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/bbox.h $
+// $Id: bbox.h 80f6f65 2020-04-05T17:56:50+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -43,21 +43,27 @@ namespace CGAL {
     *  computes a bounding box of a polygon mesh.
     *
     * @tparam PolygonMesh a model of `HalfedgeListGraph`
-    * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+    * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
     *
     * @param pmesh a polygon mesh
-    * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+    * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
     *
     * \cgalNamedParamsBegin
-    *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
-    *       If this parameter is omitted, an internal property map for
-    *       `CGAL::vertex_point_t` must be available in `PolygonMesh`\cgalParamEnd
-    *    \cgalParamBegin{geom_traits} an instance of a geometric traits class,
-    *       providing the functor `Construct_bbox_3` and the function
-    *       `Construct_bbox_3 construct_bbox_3_object()`. `Construct_bbox_3`
-    *       must provide `BBox_3 operator()(Point_3)` where `Point_3` is the value type
-    *       of the vertex point map.
-    *     \cgalParamEnd
+    *   \cgalParamNBegin{vertex_point_map}
+    *     \cgalParamDescription{a property map associating points to the vertices of `pmesh`}
+    *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+    *                    as key type and `%Point_3` as value type}
+    *     \cgalParamDefault{`boost::get(CGAL::vertex_point, pmesh)`}
+    *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+    *                     must be available in `PolygonMesh`.}
+    *   \cgalParamNEnd
+    *
+    *   \cgalParamNBegin{geom_traits}
+    *     \cgalParamDescription{an instance of a geometric traits class providing the functor `Construct_bbox_3`
+    *                           and the function `Construct_bbox_3 construct_bbox_3_object()`.
+    *                           `Construct_bbox_3` must provide `Bbox_3 operator()(Point_3)`
+    *                           where `%Point_3` is the value type of the vertex point map.}
+    *   \cgalParamNEnd
     * \cgalNamedParamsEnd
     *
     * @return a bounding box of `pmesh`
@@ -68,12 +74,13 @@ namespace CGAL {
     {
       using parameters::choose_parameter;
       using parameters::get_parameter;
+
       typename GetVertexPointMap<PolygonMesh, CGAL_PMP_NP_CLASS>::const_type
         vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
-                           get_const_property_map(CGAL::vertex_point, pmesh));
+                               get_const_property_map(CGAL::vertex_point, pmesh));
 
       typedef typename GetGeomTraits<PolygonMesh, CGAL_PMP_NP_CLASS>::type GT;
-      GT gt = choose_parameter(get_parameter(np, internal_np::geom_traits), GT());
+      GT gt = choose_parameter<GT>(get_parameter(np, internal_np::geom_traits));
       typename GT::Construct_bbox_3 get_bbox = gt.construct_bbox_3_object();
 
       typedef typename boost::graph_traits<PolygonMesh>::vertex_descriptor vertex_descriptor;
@@ -91,22 +98,28 @@ namespace CGAL {
     *  computes a bounding box of a vertex of a polygon mesh.
     *
     * @tparam PolygonMesh a model of `HalfedgeGraph`
-    * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+    * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
     *
     * @param vd a descriptor of a vertex in `pmesh`
     * @param pmesh a polygon mesh
-    * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+    * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
     *
     * \cgalNamedParamsBegin
-    *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
-    *       If this parameter is omitted, an internal property map for
-    *       `CGAL::vertex_point_t` must be available in `PolygonMesh`\cgalParamEnd
-    *    \cgalParamBegin{geom_traits} an instance of a geometric traits class,
-    *       providing the functor `Construct_bbox_3` and the function
-    *       `Construct_bbox_3 construct_bbox_3_object()`. `Construct_bbox_3`
-    *       must provide `BBox_3 operator()(Point_3)` where `Point_3` is the value type
-    *       of the vertex point map.
-    *     \cgalParamEnd
+    *   \cgalParamNBegin{vertex_point_map}
+    *     \cgalParamDescription{a property map associating points to the vertices of `pmesh`}
+    *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+    *                    as key type and `%Point_3` as value type}
+    *     \cgalParamDefault{`boost::get(CGAL::vertex_point, pmesh)`}
+    *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+    *                     must be available in `PolygonMesh`.}
+    *   \cgalParamNEnd
+    *
+    *   \cgalParamNBegin{geom_traits}
+    *     \cgalParamDescription{an instance of a geometric traits class providing the functor `Construct_bbox_3`
+    *                           and the function `Construct_bbox_3 construct_bbox_3_object()`.
+    *                           `Construct_bbox_3` must provide `Bbox_3 operator()(Point_3)`
+    *                           where `%Point_3` is the value type of the vertex point map.}
+    *   \cgalParamNEnd
     * \cgalNamedParamsEnd
     *
     * @return a bounding box of `pmesh`
@@ -120,10 +133,10 @@ namespace CGAL {
       using parameters::get_parameter;
       typename GetVertexPointMap<PolygonMesh, NamedParameters>::const_type
         vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
-                           get_const_property_map(CGAL::vertex_point, pmesh));
+                               get_const_property_map(CGAL::vertex_point, pmesh));
 
       typedef typename GetGeomTraits<PolygonMesh, NamedParameters>::type GT;
-      GT gt = choose_parameter(get_parameter(np, internal_np::geom_traits), GT());
+      GT gt = choose_parameter<GT>(get_parameter(np, internal_np::geom_traits));
       typename GT::Construct_bbox_3 get_bbox = gt.construct_bbox_3_object();
 
       return get_bbox( get(vpm, vd) );
@@ -134,22 +147,28 @@ namespace CGAL {
     *  computes a bounding box of an edge of a polygon mesh.
     *
     * @tparam PolygonMesh a model of `HalfedgeGraph`
-    * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+    * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
     *
     * @param ed a descriptor of an edge in `pmesh`
     * @param pmesh a polygon mesh
-    * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+    * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
     *
     * \cgalNamedParamsBegin
-    *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
-    *       If this parameter is omitted, an internal property map for
-    *       `CGAL::vertex_point_t` must be available in `PolygonMesh`\cgalParamEnd
-    *    \cgalParamBegin{geom_traits} an instance of a geometric traits class,
-    *       providing the functor `Construct_bbox_3` and the function
-    *       `Construct_bbox_3 construct_bbox_3_object()`. `Construct_bbox_3`
-    *       must provide `BBox_3 operator()(Point_3)` where `Point_3` is the value type
-    *       of the vertex point map.
-    *     \cgalParamEnd
+    *   \cgalParamNBegin{vertex_point_map}
+    *     \cgalParamDescription{a property map associating points to the vertices of `pmesh`}
+    *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+    *                    as key type and `%Point_3` as value type}
+    *     \cgalParamDefault{`boost::get(CGAL::vertex_point, pmesh)`}
+    *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+    *                     must be available in `PolygonMesh`.}
+    *   \cgalParamNEnd
+    *
+    *   \cgalParamNBegin{geom_traits}
+    *     \cgalParamDescription{an instance of a geometric traits class providing the functor `Construct_bbox_3`
+    *                           and the function `Construct_bbox_3 construct_bbox_3_object()`.
+    *                           `Construct_bbox_3` must provide `Bbox_3 operator()(Point_3)`
+    *                           where `%Point_3` is the value type of the vertex point map.}
+    *   \cgalParamNEnd
     * \cgalNamedParamsEnd
     *
     * @return a bounding box of `pmesh`
@@ -161,12 +180,13 @@ namespace CGAL {
     {
       using parameters::choose_parameter;
       using parameters::get_parameter;
+
       typename GetVertexPointMap<PolygonMesh, NamedParameters>::const_type
         vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
-                           get_const_property_map(CGAL::vertex_point, pmesh));
+                               get_const_property_map(CGAL::vertex_point, pmesh));
 
       typedef typename GetGeomTraits<PolygonMesh, NamedParameters>::type GT;
-      GT gt = choose_parameter(get_parameter(np, internal_np::geom_traits), GT());
+      GT gt = choose_parameter<GT>(get_parameter(np, internal_np::geom_traits));
       typename GT::Construct_bbox_3 get_bbox = gt.construct_bbox_3_object();
 
       return get_bbox( get(vpm, source(ed, pmesh)) ) +
@@ -178,22 +198,29 @@ namespace CGAL {
     *  computes a bounding box of a face of a polygon mesh.
     *
     * @tparam PolygonMesh a model of `HalfedgeGraph`
-    * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+    * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
     *
     * @param fd a descriptor of a face in `pmesh`
     * @param pmesh a polygon mesh
-    * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+    * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
     *
     * \cgalNamedParamsBegin
-    *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
-    *       If this parameter is omitted, an internal property map for
-    *       `CGAL::vertex_point_t` must be available in `PolygonMesh`\cgalParamEnd
-    *    \cgalParamBegin{geom_traits} an instance of a geometric traits class,
-    *       providing the functor `Construct_bbox_3` and the function
-    *       `Construct_bbox_3 construct_bbox_3_object()`. `Construct_bbox_3`
-    *       must provide `BBox_3 operator()(Point_3)` where `Point_3` is the value type
-    *       of the vertex point map.
-    *     \cgalParamEnd
+    *   \cgalParamNBegin{vertex_point_map}
+    *     \cgalParamDescription{a property map associating points to the vertices of `pmesh`}
+    *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+    *                    as key type and `%Point_3` as value type}
+    *     \cgalParamDefault{`boost::get(CGAL::vertex_point, pmesh)`}
+    *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+    *                     must be available in `PolygonMesh`.}
+    *   \cgalParamNEnd
+    *
+    *   \cgalParamNBegin{geom_traits}
+    *     \cgalParamDescription{an instance of a geometric traits class providing the functor `Construct_bbox_3`
+    *                           and the function `Construct_bbox_3 construct_bbox_3_object()`.
+    *                           `Construct_bbox_3` must provide `Bbox_3 operator()(Point_3)`
+    *                           where `%Point_3` is the value type of the vertex point map.}
+    *     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+    *   \cgalParamNEnd
     * \cgalNamedParamsEnd
     *
     * @return a bounding box of `pmesh`
@@ -205,12 +232,13 @@ namespace CGAL {
     {
       using parameters::choose_parameter;
       using parameters::get_parameter;
+
       typename GetVertexPointMap<PolygonMesh, NamedParameters>::const_type
         vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
-                           get_const_property_map(CGAL::vertex_point, pmesh));
+                               get_const_property_map(CGAL::vertex_point, pmesh));
 
       typedef typename GetGeomTraits<PolygonMesh, NamedParameters>::type GT;
-      GT gt = choose_parameter(get_parameter(np, internal_np::geom_traits), GT());
+      GT gt = choose_parameter<GT>(get_parameter(np, internal_np::geom_traits));
       typename GT::Construct_bbox_3 get_bbox = gt.construct_bbox_3_object();
 
       typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor halfedge_descriptor;
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/border.h 5.1-1/include/CGAL/Polygon_mesh_processing/border.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/border.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/border.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/border.h $
-// $Id: border.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/border.h $
+// $Id: border.h 80f6f65 2020-04-05T17:56:50+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -17,6 +17,7 @@
 
 #include <CGAL/algorithm.h>
 #include <CGAL/boost/graph/iterator.h>
+#include <CGAL/boost/graph/helpers.h>
 #include <CGAL/Polygon_mesh_processing/internal/named_function_params.h>
 #include <CGAL/Polygon_mesh_processing/internal/named_params_helper.h>
 
@@ -28,13 +29,31 @@
 
 namespace CGAL{
 namespace Polygon_mesh_processing {
+namespace internal {
 
-  namespace internal
+template<typename PolygonMesh>
+std::size_t border_size(typename boost::graph_traits<PolygonMesh>::halfedge_descriptor h,
+                        const PolygonMesh& pmesh)
+{
+  // if you want to use it on a non-border halfedge, just use `degree(face, mesh)`
+  CGAL_precondition(is_border(h, pmesh));
+
+  std::size_t res = 0;
+  typename boost::graph_traits<PolygonMesh>::halfedge_descriptor done = h;
+  do
   {
+    ++res;
+    h = next(h, pmesh);
+  }
+  while(h != done);
+
+  return res;
+}
+
     template<typename PM
            , typename FaceRange
            , typename HalfedgeOutputIterator>
-    HalfedgeOutputIterator border_halfedges_impl(const FaceRange& faces
+    HalfedgeOutputIterator border_halfedges_impl(const FaceRange& face_range
                                                , HalfedgeOutputIterator out
                                                , const PM& pmesh)
     {
@@ -45,7 +64,7 @@ namespace Polygon_mesh_processing {
       // the bool is true if the halfedge stored is the one of the face,
       // false if it is its opposite
       std::map<halfedge_descriptor, bool> border;
-      for(face_descriptor f : faces)
+      for(face_descriptor f : face_range)
       {
         for(halfedge_descriptor h :
           halfedges_around_face(halfedge(f, pmesh), pmesh))
@@ -75,13 +94,13 @@ namespace Polygon_mesh_processing {
            , typename FaceRange
            , typename HalfedgeOutputIterator
            , typename NamedParameters>
-    HalfedgeOutputIterator border_halfedges_impl(const FaceRange& faces
+    HalfedgeOutputIterator border_halfedges_impl(const FaceRange& face_range
                                                , typename boost::cgal_no_property::type
                                                , HalfedgeOutputIterator out
                                                , const PM& pmesh
                                                , const NamedParameters& /* np */)
     {
-      return border_halfedges_impl(faces, out, pmesh);
+      return border_halfedges_impl(face_range, out, pmesh);
     }
 
     template<typename PM
@@ -89,7 +108,7 @@ namespace Polygon_mesh_processing {
            , typename FaceIndexMap
            , typename HalfedgeOutputIterator
            , typename NamedParameters>
-    HalfedgeOutputIterator border_halfedges_impl(const FaceRange& faces
+    HalfedgeOutputIterator border_halfedges_impl(const FaceRange& face_range
                                                , const FaceIndexMap& fmap
                                                , HalfedgeOutputIterator out
                                                , const PM& pmesh
@@ -98,25 +117,13 @@ namespace Polygon_mesh_processing {
       typedef typename boost::graph_traits<PM>::halfedge_descriptor halfedge_descriptor;
       typedef typename boost::graph_traits<PM>::face_descriptor     face_descriptor;
 
-      //make a minimal check that it's properly initialized :
-      //if the 2 first faces have the same id, we know the property map is not initialized
-      if (boost::is_same<typename GetFaceIndexMap<PM, NamedParameters>::Is_internal_map,
-                         boost::true_type>::value)
-      {
-        typename boost::range_iterator<const FaceRange>::type it = boost::const_begin(faces);
-        if (get(fmap, *it) == get(fmap, *std::next(it)))
-        {
-          std::cerr << "WARNING : the internal property map for CGAL::face_index_t" << std::endl
-                    << "          is not properly initialized." << std::endl
-                    << "          Initialize it before calling border_halfedges()" << std::endl;
-        }
-      }
+      CGAL_assertion(BGL::internal::is_index_map_valid(fmap, num_faces(pmesh), faces(pmesh)));
 
       std::vector<bool> present(num_faces(pmesh), false);
-      for(face_descriptor fd : faces)
+      for(face_descriptor fd : face_range)
         present[get(fmap, fd)] = true;
 
-      for(face_descriptor fd : faces)
+      for(face_descriptor fd : face_range)
         for(halfedge_descriptor hd :
                       halfedges_around_face(halfedge(fd, pmesh), pmesh))
        {
@@ -142,28 +149,28 @@ namespace Polygon_mesh_processing {
   * For each returned halfedge `h`, `opposite(h, pmesh)` belongs to a face of the patch,
   * but `face(h, pmesh)` does not belong to the patch.
   *
-  * @tparam PolygonMesh model of `HalfedgeGraph`. If `PolygonMesh`
-  *  has an internal property map
-  *  for `CGAL::face_index_t` and no `face_index_map` is given
-  *  as a named parameter, then the internal one must be initialized
-  * @tparam FaceRange range of
-       `boost::graph_traits<PolygonMesh>::%face_descriptor`, model of `Range`.
-        Its iterator type is `InputIterator`.
+  * @tparam PolygonMesh model of `HalfedgeGraph`
+  * @tparam FaceRange a model of `Range` with value type `boost::graph_traits<PolygonMesh>::%face_descriptor`.
   * @tparam HalfedgeOutputIterator model of `OutputIterator`
      holding `boost::graph_traits<PolygonMesh>::%halfedge_descriptor`
      for patch border
-  * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+  * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
   *
-  * @param pmesh the polygon mesh to which `faces` belong
-  * @param faces the range of faces defining the patch whose border halfedges
-  *              are collected
+  * @param pmesh the polygon mesh to which the faces in `face_range` belong
+  * @param face_range the range of faces defining the patch whose border halfedges
+  *                   are collected
   * @param out the output iterator that collects the border halfedges of the patch,
   *            seen from outside.
-  * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+  * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
   * \cgalNamedParamsBegin
-      \cgalParamBegin{face_index_map} a property map containing the index of each face of `pmesh` \cgalParamEnd
-    \cgalNamedParamsEnd
+  *   \cgalParamNBegin{face_index_map}
+  *     \cgalParamDescription{a property map associating to each face of `pmesh` a unique index between `0` and `num_faces(pmesh) - 1`}
+  *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%face_descriptor`
+  *                    as key type and `std::size_t` as value type}
+  *     \cgalParamDefault{an automatically indexed internal map}
+  *   \cgalParamNEnd
+  * \cgalNamedParamsEnd
   *
   * @returns `out`
   */
@@ -171,29 +178,18 @@ namespace Polygon_mesh_processing {
          , typename FaceRange
          , typename HalfedgeOutputIterator
          , typename NamedParameters>
-  HalfedgeOutputIterator border_halfedges(const FaceRange& faces
+  HalfedgeOutputIterator border_halfedges(const FaceRange& face_range
                                   , const PolygonMesh& pmesh
                                   , HalfedgeOutputIterator out
                                   , const NamedParameters& np)
   {
-    if (faces.empty()) return out;
-
-    typedef PolygonMesh PM;
-    typedef typename GetFaceIndexMap<PM, NamedParameters>::const_type     FIMap;
-    typedef typename boost::property_map<typename internal::Dummy_PM,
-                                              CGAL::face_index_t>::type   Unset_FIMap;
-
-    if (boost::is_same<FIMap, Unset_FIMap>::value || faces.size() == 1)
-    {
-      //face index map is not given in named parameters, nor as an internal property map
-      return internal::border_halfedges_impl(faces, out, pmesh);
-    }
+    if (face_range.empty())
+      return out;
 
-    //face index map given as a named parameter, or as an internal property map
-    FIMap fim = parameters::choose_parameter(parameters::get_parameter(np, internal_np::face_index),
-                                             get_const_property_map(CGAL::face_index, pmesh));
+    typedef typename CGAL::GetInitializedFaceIndexMap<PolygonMesh, NamedParameters>::const_type FIMap;
+    FIMap fim = CGAL::get_initialized_face_index_map(pmesh, np);
 
-    return internal::border_halfedges_impl(faces, fim, out, pmesh, np);
+    return internal::border_halfedges_impl(face_range, fim, out, pmesh, np);
   }
 
   template<typename PolygonMesh
@@ -212,11 +208,11 @@ namespace Polygon_mesh_processing {
   template<typename PolygonMesh
          , typename FaceRange
          , typename HalfedgeOutputIterator>
-  HalfedgeOutputIterator border_halfedges(const FaceRange& faces
+  HalfedgeOutputIterator border_halfedges(const FaceRange& face_range
                                         , const PolygonMesh& pmesh
                                         , HalfedgeOutputIterator out)
   {
-    return border_halfedges(faces, pmesh, out,
+    return border_halfedges(face_range, pmesh, out,
       CGAL::Polygon_mesh_processing::parameters::all_default());
   }
 
@@ -225,7 +221,7 @@ namespace Polygon_mesh_processing {
   //
   // @tparam PolygonMesh model of `HalfedgeGraph`.
   //
-  // @param pmesh the polygon mesh to which `faces` belong
+  // @param pmesh the polygon mesh to which `face_range` belong
   //
   template<typename PolygonMesh>
   unsigned int number_of_borders(const PolygonMesh& pmesh)
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/clip.h 5.1-1/include/CGAL/Polygon_mesh_processing/clip.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/clip.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/clip.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/clip.h $
-// $Id: clip.h 3d66c59 2020-03-13T05:15:12+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/clip.h $
+// $Id: clip.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -15,15 +15,25 @@
 
 #include <CGAL/license/Polygon_mesh_processing/corefinement.h>
 
-
 #include <CGAL/Polygon_mesh_processing/corefinement.h>
 #include <CGAL/Polygon_mesh_processing/connected_components.h>
 #include <CGAL/Polygon_mesh_processing/bbox.h>
 #include <CGAL/Polygon_mesh_processing/triangulate_faces.h>
 #include <CGAL/Polygon_mesh_processing/orientation.h>
+#include <CGAL/Polygon_mesh_processing/triangulate_hole.h>
+#include <CGAL/Polygon_mesh_processing/border.h>
+#include <CGAL/Polygon_mesh_processing/repair.h>
+#include <CGAL/iterator.h>
 
 #include <CGAL/AABB_triangle_primitive.h>
 
+#include <CGAL/boost/graph/properties.h>
+#include <CGAL/boost/graph/Face_filtered_graph.h>
+
+#include <boost/property_map/property_map.hpp>
+
+#include <unordered_map>
+
 namespace CGAL{
 namespace Polygon_mesh_processing {
 
@@ -40,10 +50,10 @@ inter_pt_index(int i, int j,
   std::pair<std::map<std::pair<int,int>, int>::iterator, bool> res =
     id_map.insert(std::make_pair(make_sorted_pair(i,j),
                   static_cast<int> (points.size())));
-  if (res.second)
+  if(res.second)
     points.push_back(
       typename Geom_traits::Construct_plane_line_intersection_point_3()
-        (plane, points[i], points[j]) );
+        (plane, points[i], points[j]));
 
   return res.first->second;
 }
@@ -55,7 +65,7 @@ Oriented_side
 clip_to_bbox(const Plane_3& plane,
              const Bbox_3& bbox,
                    TriangleMesh& tm_out,
-             const NamedParameters& np )
+             const NamedParameters& np)
 {
   typedef typename GetGeomTraits<TriangleMesh, NamedParameters>::type Geom_traits;
   typedef typename Geom_traits::Point_3 Point_3;
@@ -63,7 +73,7 @@ clip_to_bbox(const Plane_3& plane,
                                      NamedParameters>::type Vpm;
 
   Vpm vpm_out = parameters::choose_parameter(parameters::get_parameter(np, internal_np::vertex_point),
-                                    get_property_map(boost::vertex_point, tm_out));
+                                             get_property_map(boost::vertex_point, tm_out));
 
 
   std::vector<Point_3> corners(8);
@@ -103,9 +113,9 @@ clip_to_bbox(const Plane_3& plane,
   std::set<int> in_point_ids; // to collect the set of points in the clipped bbox
 
   // for each face of the bbox, we look for intersection of the plane with its edges
-  for (int i=0; i<6; ++i)
+  for(int i=0; i<6; ++i)
   {
-    for (int k=0; k< 4; ++k)
+    for(int k=0; k< 4; ++k)
     {
       int current_id = face_indices[4*i + k];
       int next_id = face_indices[4*i + (k+1)%4];
@@ -116,14 +126,14 @@ clip_to_bbox(const Plane_3& plane,
         {
           all_out=false;
           // point on or on the negative side
-          output_faces[i].push_back( current_id );
-          in_point_ids.insert( output_faces[i].back() );
+          output_faces[i].push_back(current_id);
+          in_point_ids.insert(output_faces[i].back());
           // check for intersection of the edge
-          if (orientations[ next_id ] == ON_POSITIVE_SIDE)
+          if(orientations[ next_id ] == ON_POSITIVE_SIDE)
           {
             output_faces[i].push_back(
-              inter_pt_index<Geom_traits>(current_id, next_id, plane, corners, id_map) );
-            in_point_ids.insert( output_faces[i].back() );
+              inter_pt_index<Geom_traits>(current_id, next_id, plane, corners, id_map));
+            in_point_ids.insert(output_faces[i].back());
           }
           break;
         }
@@ -131,31 +141,31 @@ clip_to_bbox(const Plane_3& plane,
         {
           all_in = false;
           // check for intersection of the edge
-          if ( orientations[ next_id ] == ON_NEGATIVE_SIDE )
+          if(orientations[ next_id ] == ON_NEGATIVE_SIDE)
           {
             output_faces[i].push_back(
-              inter_pt_index<Geom_traits>(current_id, next_id, plane, corners, id_map) );
-            in_point_ids.insert( output_faces[i].back() );
+              inter_pt_index<Geom_traits>(current_id, next_id, plane, corners, id_map));
+            in_point_ids.insert(output_faces[i].back());
           }
           break;
         }
         case ON_ORIENTED_BOUNDARY:
         {
-          output_faces[i].push_back( current_id );
-          in_point_ids.insert( output_faces[i].back() );
+          output_faces[i].push_back(current_id);
+          in_point_ids.insert(output_faces[i].back());
         }
       }
     }
-    if (output_faces[i].size() < 3){
+    if(output_faces[i].size() < 3){
       CGAL_assertion(output_faces[i].empty() ||
-                     (output_faces[i].front()<8 && output_faces[i].back()<8) );
+                     (output_faces[i].front()<8 && output_faces[i].back()<8));
       output_faces[i].clear(); // edge of the bbox included in the plane
     }
   }
 
   // the intersection is the full bbox
-  if (all_in) return ON_NEGATIVE_SIDE;
-  if (all_out) return ON_POSITIVE_SIDE;
+  if(all_in) return ON_NEGATIVE_SIDE;
+  if(all_out) return ON_POSITIVE_SIDE;
 
   // build the clipped bbox
   typedef boost::graph_traits<TriangleMesh> graph_traits;
@@ -167,7 +177,7 @@ clip_to_bbox(const Plane_3& plane,
   for(int i : in_point_ids)
   {
     vertex_descriptor v = add_vertex(tm_out);
-    out_vertices.insert( std::make_pair(i, v ) );
+    out_vertices.insert(std::make_pair(i, v));
     put(vpm_out, v, corners[i]);
   }
 
@@ -176,7 +186,7 @@ clip_to_bbox(const Plane_3& plane,
   const face_descriptor null_fd = graph_traits::null_face();
   for(const std::vector<int>& findices : output_faces)
   {
-    if (findices.empty()) continue;
+    if(findices.empty()) continue;
     const face_descriptor fd=add_face(tm_out);
     int prev_id = findices.back();
 
@@ -189,12 +199,12 @@ clip_to_bbox(const Plane_3& plane,
 
       std::pair<typename std::map< std::pair<int,int>,
                 halfedge_descriptor>::iterator, bool> res =
-        hedge_map.insert( std::make_pair(std::make_pair(prev_id, current_id), null_hedge) );
-      if (res.second)
+        hedge_map.insert(std::make_pair(std::make_pair(prev_id, current_id), null_hedge));
+      if(res.second)
       {
-        res.first->second = halfedge( add_edge(tm_out), tm_out);
-        hedge_map.insert( std::make_pair(std::make_pair(current_id, prev_id),
-                            opposite(res.first->second, tm_out) ) );
+        res.first->second = halfedge(add_edge(tm_out), tm_out);
+        hedge_map.insert(std::make_pair(std::make_pair(current_id, prev_id),
+                            opposite(res.first->second, tm_out)));
         set_face(opposite(res.first->second, tm_out), null_fd, tm_out);
 
       }
@@ -208,7 +218,7 @@ clip_to_bbox(const Plane_3& plane,
       set_halfedge(src, opposite(hedges.back(), tm_out), tm_out);
       set_halfedge(tgt, hedges.back(), tm_out);
 
-      if (current_id==findices.front())
+      if(current_id==findices.front())
         set_halfedge(fd, hedges.back(), tm_out);
 
       prev_id = current_id;
@@ -230,7 +240,7 @@ clip_to_bbox(const Plane_3& plane,
   // until we reach another border halfedge
   for(halfedge_descriptor h : halfedges(tm_out))
   {
-    if (face(h, tm_out) == null_fd)
+    if(face(h, tm_out) == null_fd)
     {
       face_descriptor fd = add_face(tm_out);
       set_halfedge(fd, h, tm_out);
@@ -244,7 +254,7 @@ clip_to_bbox(const Plane_3& plane,
         } while(face(h_curr, tm_out) != null_fd && h_curr!=h);
         set_face(h_prev, fd, tm_out);
         set_next(h_prev, h_curr, tm_out);
-        if (h_curr==h)
+        if(h_curr==h)
           break;
         h_prev=h_curr;
       } while(true);
@@ -259,57 +269,210 @@ clip_to_bbox(const Plane_3& plane,
   return ON_ORIENTED_BOUNDARY;
 }
 
+template <class TriangleMesh, class Ecm, class VPM>
+void split_along_edges(TriangleMesh& tm,
+                       Ecm ecm,
+                       VPM vpm)
+{
+  typedef boost::graph_traits<TriangleMesh> GT;
+  typedef typename GT::face_descriptor face_descriptor;
+  typedef typename GT::edge_descriptor edge_descriptor;
+  typedef typename GT::vertex_descriptor vertex_descriptor;
+  typedef typename GT::halfedge_descriptor halfedge_descriptor;
+
+  std::vector<edge_descriptor> shared_edges;
+  for(edge_descriptor e : edges(tm))
+    if(get(ecm, e))
+      shared_edges.push_back(e);
+
+  std::size_t nb_shared_edges = shared_edges.size();
+  std::vector<halfedge_descriptor> hedges_to_update;
+
+  typedef CGAL::dynamic_halfedge_property_t<bool> H_tag;
+  typename boost::property_map<TriangleMesh, H_tag>::type
+    no_target_update = get(H_tag(), tm);
+
+  std::vector< std::pair<halfedge_descriptor, vertex_descriptor> > vertices_to_duplicate;
+
+  //collect border halfedges having as target one of the edge endpoints
+  std::set<halfedge_descriptor> extra_border_hedges;
+  for(std::size_t k=0; k<nb_shared_edges; ++k)
+  {
+    for(halfedge_descriptor h : halfedges_around_target(target(shared_edges[k], tm), tm))
+      if(is_border(h, tm))
+        extra_border_hedges.insert(h);
+    for(halfedge_descriptor h : halfedges_around_target(source(shared_edges[k], tm), tm))
+      if(is_border(h, tm))
+        extra_border_hedges.insert(h);
+  }
+
+  for(halfedge_descriptor h : extra_border_hedges)
+  {
+    put(no_target_update, h, true);
+    set_halfedge(target(h, tm), h, tm);
+    hedges_to_update.push_back(h);
+  }
+
+  // now duplicate the edge and set its pointers
+  for(std::size_t k=0; k<nb_shared_edges; ++k)
+  {
+    halfedge_descriptor h    = halfedge(shared_edges[k], tm);
+    face_descriptor fh = face(h, tm);
+    //add edge
+    halfedge_descriptor new_hedge = halfedge(add_edge(tm), tm),
+                        new_opp   = opposite(new_hedge,tm);
+
+    vertex_descriptor vt = target(h, tm);
+    vertex_descriptor vs = source(h, tm);
+
+    //replace h with new_hedge
+    set_next(new_hedge, next(h, tm), tm);
+    set_next(prev(h, tm), new_hedge, tm);
+    set_face(new_hedge, fh, tm);
+    set_halfedge(fh, new_hedge, tm);
+
+    set_target(new_hedge, vt, tm);
+    set_target(new_opp, vs, tm);
+
+    set_face(new_opp, GT::null_face(), tm);
+    set_face(h, GT::null_face(), tm);
+
+    // handle vertices to duplicate
+    halfedge_descriptor h_vt = halfedge(vt, tm);
+    if(get(no_target_update, h_vt))
+      vertices_to_duplicate.push_back(std::make_pair(h, vt));
+    else
+      set_halfedge(vt, h, tm);
+    halfedge_descriptor h_vs = halfedge(vs, tm);
+    if(get(no_target_update, h_vs))
+      vertices_to_duplicate.push_back(std::make_pair(new_opp, vs));
+    else
+      set_halfedge(vs, new_opp, tm);
+
+    hedges_to_update.push_back(h);
+    put(no_target_update, h, true);
+    hedges_to_update.push_back(new_opp);
+    put(no_target_update, new_opp, true);
+
+    CGAL_assertion(next(prev(new_hedge, tm), tm) == new_hedge);
+    CGAL_assertion(prev(next(new_hedge, tm), tm) == new_hedge);
+  }
+
+  // update next/prev relationship
+  for(halfedge_descriptor h : hedges_to_update)
+  {
+    CGAL_assertion(is_border(h, tm));
+    halfedge_descriptor h_opp = opposite(h, tm);
+
+    // set next pointer of h, visiting faces inside the patch we consider
+    halfedge_descriptor candidate = opposite(prev(h_opp, tm), tm);
+    while (!is_border(candidate, tm))
+      candidate = opposite(prev(candidate, tm), tm);
+    set_next(h, candidate, tm);
+    CGAL_assertion(prev(next(h_opp, tm), tm)==h_opp);
+
+    CGAL_assertion(prev(next(h, tm), tm) == h);
+    CGAL_assertion(is_border(next(h, tm), tm));
+  }
+
+  for(const std::pair<halfedge_descriptor, vertex_descriptor>& p : vertices_to_duplicate)
+  {
+    vertex_descriptor nv = add_vertex(tm);
+    put(vpm, nv, get(vpm, p.second));
+    for(halfedge_descriptor h : halfedges_around_target(p.first, tm))
+      set_target(h, nv, tm);
+    set_halfedge(nv, p.first, tm);
+   }
+
+  CGAL_assertion_code(for(halfedge_descriptor h : hedges_to_update))
+  {
+    CGAL_assertion(next(prev(h, tm), tm) == h);
+    CGAL_assertion(prev(next(h, tm), tm) == h);
+  }
+
+  for(halfedge_descriptor h : hedges_to_update)
+  {
+    for(halfedge_descriptor hh : halfedges_around_target(h, tm))
+        if(h!=hh)
+          set_target(hh, target(h, tm), tm);
+  }
+
+  CGAL_assertion(is_valid_polygon_mesh(tm));
+}
+
 } // end of internal namespace
 
 /**
   * \ingroup PMP_corefinement_grp
+  *
   * clips `tm` by keeping the part that is inside the volume \link coref_def_subsec bounded \endlink
   * by `clipper`.
   * If `tm` is closed, the clipped part can be closed too if the named parameter `clip_volume` is set to `true`.
-  * See subsection \ref coref_clip for more details.
+  * See Subsection \ref coref_clip for more details.
   * \attention With the current implementation, `clipper` will be modified (refined with the intersection with `tm`).
   *
-  * \pre \link CGAL::Polygon_mesh_processing::does_self_intersect() `!CGAL::Polygon_mesh_processing::does_self_intersect(tm1)` \endlink
+  * \pre \link CGAL::Polygon_mesh_processing::does_self_intersect() `!CGAL::Polygon_mesh_processing::does_self_intersect(tm)` \endlink
   * \pre \link CGAL::Polygon_mesh_processing::does_self_intersect() `!CGAL::Polygon_mesh_processing::does_self_intersect(clipper)` \endlink
   * \pre \link CGAL::Polygon_mesh_processing::does_bound_a_volume() `CGAL::Polygon_mesh_processing::does_bound_a_volume(clipper)` \endlink
   *
   * @tparam TriangleMesh a model of `MutableFaceGraph`, `HalfedgeListGraph` and `FaceListGraph`.
-  *                      If `TriangleMesh` has an internal property map for `CGAL::face_index_t`,
-  *                      as a named parameter, then it must be initialized.
-  *
-  * @tparam NamedParameters1 a sequence of \ref pmp_namedparameters "Named Parameters"
-  * @tparam NamedParameters2 a sequence of \ref pmp_namedparameters "Named Parameters"
+  * @tparam NamedParameters1 a sequence of \ref bgl_namedparameters "Named Parameters"
+  * @tparam NamedParameters2 a sequence of \ref bgl_namedparameters "Named Parameters"
   *
   * @param tm input triangulated surface mesh
   * @param clipper triangulated surface mesh used to clip `tm`
-  * @param np_tm optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
-  * @param np_c optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+  * @param np_tm an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+  * @param np_c an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+  *
   *
   * \cgalNamedParamsBegin
-  *   \cgalParamBegin{vertex_point_map}
-  *     the property map with the points associated to the vertices of `tm` (`clipper`).
-  *     If this parameter is omitted, an internal property map for
-  *     `CGAL::vertex_point_t` must be available in `TriangleMesh`
-  *   \cgalParamEnd
-  *   \cgalParamBegin{face_index_map} a property map containing the index of each face of `tm` (`clipper`).
-  *     Note that if the property map is writable, the indices of the faces
-  *     of `tm` and `clipper` will be set after refining `tm` with the intersection with `clipper`.
-  *   \cgalParamEnd
-  *   \cgalParamBegin{visitor} a class model of `PMPCorefinementVisitor`
-  *                            that is used to track the creation of new faces.
-  *   \cgalParamEnd
-  *   \cgalParamBegin{throw_on_self_intersection} if `true`,
-  *      the set of triangles closed to the intersection of `tm` and `clipper` will be
-  *      checked for self-intersections and `CGAL::Polygon_mesh_processing::Corefinement::Self_intersection_exception`
-  *      will be thrown if at least one is found.  Default value is `false`.
-  *   \cgalParamEnd
-  *   \cgalParamBegin{clip_volume} if `true` and `tm` is closed, the clipping will be done on
-  *      the volume \link coref_def_subsec bounded \endlink by `tm` rather than on its surface
-  *      (i.e. `tm` will be kept closed). Default value is `false`.
-  *   \cgalParamEnd
-  *   \cgalParamBegin{use_compact_clipper} if `false`, the parts of `tm` coplanar with `clipper`
-  *      will not be part of the output. Default value is `true`.
-  *   \cgalParamEnd
+  *   \cgalParamNBegin{vertex_point_map}
+  *     \cgalParamDescription{a property map associating points to the vertices of `tm` (resp. `clipper`)}
+  *     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+  *                    as key type and `%Point_3` as value type}
+  *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm (resp. clipper))`}
+  *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+  *                     must be available in `TriangleMesh`.}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{face_index_map}
+  *     \cgalParamDescription{a property map associating to each face of `tm` (`clipper`) a unique index between `0` and `num_faces(tm (resp. clipper)) - 1`}
+  *     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<TriangleMesh>::%face_descriptor`
+  *                    as key type and `std::size_t` as value type}
+  *     \cgalParamDefault{an automatically indexed internal map}
+  *     \cgalParamExtra{if the property map is writable, the indices of the faces of `tm` and `clipper`
+  *                     will be set after refining `tm` with the intersection with `clipper`}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{visitor}
+  *     \cgalParamDescription{a visitor used to track the creation of new faces}
+  *     \cgalParamType{a class model of `PMPCorefinementVisitor`}
+  *     \cgalParamDefault{`Corefinement::Default_visitor<TriangleMesh>`}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{throw_on_self_intersection}
+  *     \cgalParamDescription{If `true`, the set of triangles closed to the intersection of `tm` and `clipper` will be
+  *                           checked for self-intersections and `Corefinement::Self_intersection_exception`
+  *                           will be thrown if at least one self-intersection is found.}
+  *     \cgalParamType{Boolean}
+  *     \cgalParamDefault{`false`}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{clip_volume}
+  *     \cgalParamDescription{If `true`, and `tm` is closed, the clipping will be done on the volume
+  *                           \link coref_def_subsec bounded \endlink by `tm` rather than on its surface
+  *                           (i.e., `tm` will be kept closed).}
+  *     \cgalParamType{Boolean}
+  *     \cgalParamDefault{`false`}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{use_compact_clipper}
+  *     \cgalParamDescription{if `false`, the parts of `tm` coplanar with `clipper` will not be part of the output.}
+  *     \cgalParamType{Boolean}
+  *     \cgalParamDefault{`false`}
+  *     \cgalParamExtra{This option has an effect only if a surface and not a volume is clipped,
+  *                     (i.e., if `clip_volume` is `false` or if `tm` is open).}
+  *   \cgalParamNEnd
   * \cgalNamedParamsEnd
   *
   * @return `true` if the output surface mesh is manifold.
@@ -319,77 +482,79 @@ template <class TriangleMesh,
           class NamedParameters1,
           class NamedParameters2>
 bool
-clip(      TriangleMesh& tm,
-           TriangleMesh& clipper,
+clip(TriangleMesh& tm,
+     TriangleMesh& clipper,
      const NamedParameters1& np_tm,
      const NamedParameters2& np_c)
 {
   const bool clip_volume =
     parameters::choose_parameter(parameters::get_parameter(np_tm, internal_np::clip_volume), false);
 
-  if (clip_volume && is_closed(tm))
+  if(clip_volume && is_closed(tm))
     return corefine_and_compute_intersection(tm, clipper, tm, np_tm, np_c);
   return corefine_and_compute_intersection(tm, clipper, tm,
                                            np_tm.use_bool_op_to_clip_surface(true),
                                            np_c);
 }
 
-namespace internal{
-template <class TriangleMesh, class NamedParameters>
-bool dispatch_clip_call(TriangleMesh& tm, TriangleMesh& clipper,
-                        const NamedParameters& np, Tag_false)
-{
-  return clip(tm, clipper,
-              np.face_index_map(get(CGAL::dynamic_face_property_t<std::size_t>(), tm)),
-              parameters::face_index_map(get(CGAL::dynamic_face_property_t<std::size_t>(), clipper)));
-}
-
-template <class TriangleMesh, class NamedParameters>
-bool dispatch_clip_call(TriangleMesh& tm, TriangleMesh& clipper,
-                        const NamedParameters& np, Tag_true)
-{
-  return clip(tm, clipper,
-              np.face_index_map(get(face_index, tm)),
-              parameters::face_index_map(get(face_index, clipper)));
-}
-}
-
 /**
   * \ingroup PMP_corefinement_grp
   * clips `tm` by keeping the part that is on the negative side of `plane` (side opposite to its normal vector).
   * If `tm` is closed, the clipped part can be closed too if the named parameter `clip_volume` is set to `true`.
-  * See subsection \ref coref_clip for more details.
+  * See Subsection \ref coref_clip for more details.
+  *
+  * \note In the current implementation it is not possible to set the vertex point map and the default will be used. `Plane_3` must be
+  * from the same %Kernel as the point of the vertex point map.
   *
-  * \note In the current implementation it is not possible to set the vertex point map and the default will be used.
   * \pre \link CGAL::Polygon_mesh_processing::does_self_intersect() `!CGAL::Polygon_mesh_processing::does_self_intersect(tm)` \endlink
   *
   * @tparam TriangleMesh a model of `MutableFaceGraph`, `HalfedgeListGraph` and `FaceListGraph`.
-  *                      If `TriangleMesh` has an internal property map for `CGAL::face_index_t`,
-  *                      as a named parameter, then it must be initialized.
   *                      An internal property map for `CGAL::vertex_point_t` must be available.
   *
-  * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+  * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
   *
   * @param tm input triangulated surface mesh
   * @param plane plane whose negative side defines the half-space to intersect `tm` with.
   *              `Plane_3` is the plane type for the same CGAL kernel as the point of the vertex point map of `tm`.
-  * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+  * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
   *
   * \cgalNamedParamsBegin
-  *   \cgalParamBegin{visitor} a class model of `PMPCorefinementVisitor`
-  *                            that is used to track the creation of new faces.
-  *   \cgalParamEnd
-  *   \cgalParamBegin{throw_on_self_intersection} if `true`,
-  *      the set of triangles closed to the intersection of `tm` and `plane` will be
-  *      checked for self-intersections and `CGAL::Polygon_mesh_processing::Corefinement::Self_intersection_exception`
-  *      will be thrown if at least one is found. Default value is `false`.
-  *   \cgalParamEnd
-  *   \cgalParamBegin{clip_volume} if `true` and `tm` is closed, the clipping will be done on
-  *      the volume \link coref_def_subsec bounded \endlink by `tm` rather than on its surface
-  *      (i.e. `tm` will be kept closed). Default value is `false`.
-  *   \cgalParamEnd
-  *   \cgalParamBegin{use_compact_clipper} if `false` the parts of `tm` coplanar with `plane`
-  *      will not be part of the output. Default value is `true`.
+  *   \cgalParamNBegin{vertex_point_map}
+  *     \cgalParamDescription{a property map associating points to the vertices of `tm`}
+  *     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+  *                    as key type and `%Point_3` as value type}
+  *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{visitor}
+  *     \cgalParamDescription{a visitor used to track the creation of new faces}
+  *     \cgalParamType{a class model of `PMPCorefinementVisitor`}
+  *     \cgalParamDefault{`Corefinement::Default_visitor<TriangleMesh>`}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{throw_on_self_intersection}
+  *     \cgalParamDescription{If `true`, the set of triangles closed to the intersection of `tm`
+  *                           and `plane` will be checked for self-intersections
+  *                           and `CGAL::Polygon_mesh_processing::Corefinement::Self_intersection_exception`
+  *                           will be thrown if at least one self-intersection is found.}
+  *     \cgalParamType{Boolean}
+  *     \cgalParamDefault{`false`}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{clip_volume}
+  *     \cgalParamDescription{If `true`, and `tm` is closed, the clipping will be done on
+  *                           the volume \link coref_def_subsec bounded \endlink by `tm`
+  *                           rather than on its surface (i.e., `tm` will be kept closed).}
+  *     \cgalParamType{Boolean}
+  *     \cgalParamDefault{`false`}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{use_compact_clipper}
+  *     \cgalParamDescription{if `false` the parts of `tm` coplanar with `plane` will not be part of the output}
+  *     \cgalParamType{Boolean}
+  *     \cgalParamDefault{`true`}
+  *   \cgalParamNEnd
+  *
   * \cgalNamedParamsEnd
   *
   * @return `true` if the output surface mesh is manifold.
@@ -397,15 +562,15 @@ bool dispatch_clip_call(TriangleMesh& tm
   */
 template <class TriangleMesh,
           class NamedParameters>
-bool clip(      TriangleMesh& tm,
-          #ifdef DOXYGEN_RUNNING
+bool clip(TriangleMesh& tm,
+#ifdef DOXYGEN_RUNNING
           const Plane_3& plane,
-          #else
+#else
           const typename GetGeomTraits<TriangleMesh, NamedParameters>::type::Plane_3& plane,
-          #endif
+#endif
           const NamedParameters& np)
 {
-  if( boost::begin(faces(tm))==boost::end(faces(tm)) ) return true;
+  if(boost::begin(faces(tm))==boost::end(faces(tm))) return true;
 
   CGAL::Bbox_3 bbox = ::CGAL::Polygon_mesh_processing::bbox(tm);
 
@@ -427,27 +592,341 @@ bool clip(      TriangleMesh& tm,
     default:
       break;
   }
-  // dispatch is needed because face index map for tm and clipper have to be of the same time
-  return internal::dispatch_clip_call(tm, clipper,
-                                      np, CGAL::graph_has_property<TriangleMesh, CGAL::face_index_t>());
+
+  return clip(tm, clipper, np, parameters::all_default());
+}
+
+/**
+  * \ingroup PMP_corefinement_grp
+  * clips `tm` by keeping the part that is inside `iso_cuboid`.
+  * If `tm` is closed, the clipped part can be closed too if the named parameter `clip_volume` is set to `true`.
+  * See Subsection \ref coref_clip for more details.
+  *
+  * \note In the current implementation it is not possible to set the vertex point map and the default will be used. `Iso_cuboid_3` must be
+  * from the same %Kernel as the point of the vertex point map.
+  * \pre \link CGAL::Polygon_mesh_processing::does_self_intersect() `!CGAL::Polygon_mesh_processing::does_self_intersect(tm)` \endlink
+  *
+  * @tparam TriangleMesh a model of `MutableFaceGraph`, `HalfedgeListGraph` and `FaceListGraph`.
+  *                      An internal property map for `CGAL::vertex_point_t` must be available.
+  *
+  * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
+  *
+  * @param tm input triangulated surface mesh
+  * @param iso_cuboid iso-cuboid used to clip `tm`.
+  * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+  *
+  * \cgalNamedParamsBegin
+  *   \cgalParamNBegin{visitor}
+  *     \cgalParamDescription{a visitor used to track the creation of new faces}
+  *     \cgalParamType{a class model of `PMPCorefinementVisitor`}
+  *     \cgalParamDefault{`Corefinement::Default_visitor<TriangleMesh>`}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{throw_on_self_intersection}
+  *     \cgalParamDescription{If `true`, the set of triangles closed to the intersection of `tm`
+  *                           and `iso_cuboid` will be checked for self-intersections
+  *                           and `CGAL::Polygon_mesh_processing::Corefinement::Self_intersection_exception`
+  *                           will be thrown if at least one self-intersection is found.}
+  *     \cgalParamType{Boolean}
+  *     \cgalParamDefault{`false`}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{clip_volume}
+  *     \cgalParamDescription{If `true`, and `tm` is closed, the clipping will be done on
+  *                           the volume \link coref_def_subsec bounded \endlink by `tm`
+  *                           rather than on its surface (i.e., `tm` will be kept closed).}
+  *     \cgalParamType{Boolean}
+  *     \cgalParamDefault{`false`}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{use_compact_clipper}
+  *     \cgalParamDescription{if `false` the parts of `tm` coplanar with `iso_cuboid` will not be part of the output}
+  *     \cgalParamType{Boolean}
+  *     \cgalParamDefault{`true`}
+  *   \cgalParamNEnd
+  * \cgalNamedParamsEnd
+  *
+  * @return `true` if the output surface mesh is manifold.
+  *         If `false` is returned `tm` is only refined by the intersection with `iso_cuboid`.
+  */
+template <class TriangleMesh,
+          class NamedParameters>
+bool clip(TriangleMesh& tm,
+#ifdef DOXYGEN_RUNNING
+          const Iso_cuboid_3& iso_cuboid,
+#else
+          const typename GetGeomTraits<TriangleMesh, NamedParameters>::type::Iso_cuboid_3& iso_cuboid,
+#endif
+          const NamedParameters& np)
+{
+  if(boost::begin(faces(tm))==boost::end(faces(tm))) return true;
+  TriangleMesh clipper;
+
+  make_hexahedron(iso_cuboid[0], iso_cuboid[1], iso_cuboid[2], iso_cuboid[3],
+                  iso_cuboid[4], iso_cuboid[5], iso_cuboid[6], iso_cuboid[7],
+                  clipper);
+  triangulate_faces(clipper);
+
+  return clip(tm, clipper, np, parameters::all_default());
+}
+
+/*!
+  * \ingroup PMP_corefinement_grp
+  * corefines `tm` and `splitter` and duplicates edges in `tm` that are on the intersection with `splitter`.
+  *
+  * \pre \link CGAL::Polygon_mesh_processing::does_self_intersect() `!CGAL::Polygon_mesh_processing::does_self_intersect(tm)` \endlink
+  * \pre \link CGAL::Polygon_mesh_processing::does_self_intersect() `!CGAL::Polygon_mesh_processing::does_self_intersect(splitter)` \endlink
+  *
+  * @tparam TriangleMesh a model of `MutableFaceGraph`, `HalfedgeListGraph` and `FaceListGraph`.
+  *
+  * @tparam NamedParameters1 a sequence of \ref bgl_namedparameters "Named Parameters"
+  * @tparam NamedParameters2 a sequence of \ref bgl_namedparameters "Named Parameters"
+  *
+  * @param tm input triangulated surface mesh
+  * @param splitter triangulated surface mesh used to split `tm`
+  * @param np_tm an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+  * @param np_s an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+  *
+  * \cgalNamedParamsBegin
+  *   \cgalParamNBegin{vertex_point_map}
+  *     \cgalParamDescription{a property map associating points to the vertices of `tm` (`splitter`)}
+  *     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+  *                    as key type and `%Point_3` as value type}
+  *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+  *     \cgalParamDefault{If this parameter is omitted, an internal property map for
+  *                       `CGAL::vertex_point_t` must be available in `TriangleMesh`.}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{visitor}
+  *     \cgalParamDescription{a visitor used to track the creation of new faces}
+  *     \cgalParamType{a class model of `PMPCorefinementVisitor`}
+  *     \cgalParamDefault{`Corefinement::Default_visitor<TriangleMesh>`}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{throw_on_self_intersection}
+  *     \cgalParamDescription{If `true`, the set of triangles closed to the intersection of `tm`
+  *                           and `splitter` will be checked for self-intersections
+  *                           and `CGAL::Polygon_mesh_processing::Corefinement::Self_intersection_exception`
+  *                           will be thrown if at least one self-intersection is found.}
+  *     \cgalParamType{Boolean}
+  *     \cgalParamDefault{`false`}
+  *   \cgalParamNEnd
+  *
+  * \cgalNamedParamsEnd
+*/
+template <class TriangleMesh,
+          class NamedParameters1,
+          class NamedParameters2>
+void split(TriangleMesh& tm,
+           TriangleMesh& splitter,
+           const NamedParameters1& np_tm,
+           const NamedParameters2& np_s)
+{
+  namespace PMP = CGAL::Polygon_mesh_processing;
+
+  using parameters::get_parameter;
+  using parameters::choose_parameter;
+
+  typedef typename GetVertexPointMap<TriangleMesh, NamedParameters1>::type VPM1;
+  typedef typename GetVertexPointMap<TriangleMesh, NamedParameters2>::type VPM2;
+
+  typedef typename boost::template property_map<TriangleMesh, CGAL::dynamic_edge_property_t<bool> >::type Ecm;
+
+  VPM1 vpm_tm = choose_parameter(get_parameter(np_tm, internal_np::vertex_point),
+                                 get_property_map(vertex_point, tm));
+  VPM2 vpm_s = choose_parameter(get_parameter(np_s, internal_np::vertex_point),
+                                get_property_map(vertex_point, splitter));
+
+  Ecm ecm  = get(CGAL::dynamic_edge_property_t<bool>(), tm);
+
+  // create a constrained edge map and corefine input mesh with the splitter,
+  // and mark edges
+
+  PMP::corefine(tm, splitter,
+                CGAL::parameters::vertex_point_map(vpm_tm).edge_is_constrained_map(ecm),
+                CGAL::parameters::vertex_point_map(vpm_s));
+
+  //split mesh along marked edges
+  internal::split_along_edges(tm, ecm, vpm_tm);
+}
+
+/**
+  * \ingroup PMP_corefinement_grp
+  * adds intersection edges of `plane` and `tm` in `tm` and duplicates those edges.
+  *
+  * \note In the current implementation it is not possible to set the vertex point map and the default will be used.
+  *
+  * \pre \link CGAL::Polygon_mesh_processing::does_self_intersect() `!CGAL::Polygon_mesh_processing::does_self_intersect(tm)` \endlink
+  *
+  * @tparam TriangleMesh a model of `MutableFaceGraph`, `HalfedgeListGraph` and `FaceListGraph`
+  *                      An internal property map for `CGAL::vertex_point_t` must be available.
+  *
+  * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
+  *
+  * @param tm input triangulated surface mesh
+  * @param plane the plane that will be used to split `tm`.
+  *              `Plane_3` is the plane type for the same CGAL kernel as the point of the vertex point map of `tm`.
+  * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+  *
+  * \cgalNamedParamsBegin
+  *   \cgalParamNBegin{vertex_point_map}
+  *     \cgalParamDescription{a property map associating points to the vertices of `tm`}
+  *     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+  *                    as key type and `%Point_3` as value type}
+  *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{visitor}
+  *     \cgalParamDescription{a visitor used to track the creation of new faces}
+  *     \cgalParamType{a class model of `PMPCorefinementVisitor`}
+  *     \cgalParamDefault{`Corefinement::Default_visitor<TriangleMesh>`}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{throw_on_self_intersection}
+  *     \cgalParamDescription{If `true`, the set of triangles closed to the intersection of `tm`
+  *                           and `plane` will be checked for self-intersections
+  *                           and `CGAL::Polygon_mesh_processing::Corefinement::Self_intersection_exception`
+  *                           will be thrown if at least one self-intersection is found.}
+  *     \cgalParamType{Boolean}
+  *     \cgalParamDefault{`false`}
+  *   \cgalParamNEnd
+  *
+  * \cgalNamedParamsEnd
+  */
+template <class TriangleMesh,
+          class NamedParameters>
+void split(TriangleMesh& tm,
+#ifdef DOXYGEN_RUNNING
+           const Plane_3& plane,
+#else
+           const typename GetGeomTraits<TriangleMesh, NamedParameters>::type::Plane_3& plane,
+#endif
+           const NamedParameters& np)
+{
+  using parameters::get_parameter;
+  using parameters::choose_parameter;
+  namespace PMP = CGAL::Polygon_mesh_processing;
+  namespace params = PMP::parameters;
+
+  // create a splitter mesh for the splitting plane using an internal CGAL function
+  CGAL::Bbox_3 bbox = ::CGAL::Polygon_mesh_processing::bbox(tm, np);
+  double xd = (std::max)(1., 0.01 * (bbox.xmax() - bbox.xmin()));
+  double yd = (std::max)(1., 0.01 * (bbox.ymax() - bbox.ymin()));
+  double zd = (std::max)(1., 0.01 * (bbox.zmax() - bbox.zmin()));
+  bbox = CGAL::Bbox_3(bbox.xmin()-xd, bbox.ymin()-yd, bbox.zmin()-zd,
+                      bbox.xmax()+xd, bbox.ymax()+yd, bbox.zmax()+zd);
+
+  TriangleMesh splitter;
+  CGAL::Oriented_side os = PMP::internal::clip_to_bbox(plane, bbox, splitter, PMP::parameters::all_default());
+
+  if(os == CGAL::ON_ORIENTED_BOUNDARY)
+    return split(tm, splitter, np, params::all_default());
+
+  //else nothing to do, no intersection.
+}
+
+
+/**
+  * \ingroup PMP_corefinement_grp
+  * adds intersection edges of `iso_cuboid` and `tm` in `tm` and duplicates those edges.
+  *
+  * \note In the current implementation it is not possible to set the vertex point map and the default will be used.
+  * \note `Iso_cuboid_3` must be from the same %Kernel as the point of the vertex point map.
+  *
+  * \pre \link CGAL::Polygon_mesh_processing::does_self_intersect() `!CGAL::Polygon_mesh_processing::does_self_intersect(tm)` \endlink
+  *
+  * @tparam TriangleMesh a model of `MutableFaceGraph`, `HalfedgeListGraph` and `FaceListGraph`
+  *                      An internal property map for `CGAL::vertex_point_t` must be available.
+  *
+  * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
+  *
+  * @param tm input triangulated surface mesh
+  * @param iso_cuboid iso-cuboid used to split `tm`.
+  * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+  *
+  * \cgalNamedParamsBegin
+  *   \cgalParamNBegin{vertex_point_map}
+  *     \cgalParamDescription{a property map associating points to the vertices of `tm`}
+  *     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+  *                    as key type and `%Point_3` as value type}
+  *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{visitor}
+  *     \cgalParamDescription{a visitor used to track the creation of new faces}
+  *     \cgalParamType{a class model of `PMPCorefinementVisitor`}
+  *     \cgalParamDefault{`Corefinement::Default_visitor<TriangleMesh>`}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{throw_on_self_intersection}
+  *     \cgalParamDescription{If `true`, the set of triangles closed to the intersection of `tm`
+  *                           and `iso_cuboid` will be checked for self-intersections
+  *                           and `CGAL::Polygon_mesh_processing::Corefinement::Self_intersection_exception`
+  *                           will be thrown if at least one self-intersection is found.}
+  *     \cgalParamType{Boolean}
+  *     \cgalParamDefault{`false`}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{clip_volume}
+  *     \cgalParamDescription{If `true`, and `tm` is closed, the clipping will be done on
+  *                           the volume \link coref_def_subsec bounded \endlink by `tm`
+  *                           rather than on its surface (i.e., `tm` will be kept closed).}
+  *     \cgalParamType{Boolean}
+  *     \cgalParamDefault{`false`}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{use_compact_clipper}
+  *     \cgalParamDescription{if `false` the parts of `tm` coplanar with `iso_cuboid` will not be part of the output}
+  *     \cgalParamType{Boolean}
+  *     \cgalParamDefault{`true`}
+  *   \cgalParamNEnd
+  *
+  * \cgalNamedParamsEnd
+  */
+template <class TriangleMesh,
+          class NamedParameters>
+void split(TriangleMesh& tm,
+           #ifdef DOXYGEN_RUNNING
+           const Iso_cuboid_3& iso_cuboid,
+           #else
+           const typename GetGeomTraits<TriangleMesh, NamedParameters>::type::Iso_cuboid_3& iso_cuboid,
+           #endif
+           const NamedParameters& np)
+{
+  TriangleMesh splitter;
+
+  make_hexahedron(iso_cuboid[0], iso_cuboid[1], iso_cuboid[2], iso_cuboid[3],
+      iso_cuboid[4], iso_cuboid[5], iso_cuboid[6], iso_cuboid[7],
+      splitter);
+  triangulate_faces(splitter);
+
+  split(tm, splitter, np, parameters::all_default());
 }
 
 /// \cond SKIP_IN_MANUAL
 
 // convenience overloads
 template <class TriangleMesh>
-bool clip(      TriangleMesh& tm,
+bool clip(TriangleMesh& tm,
           const typename GetGeomTraits<TriangleMesh>::type::Plane_3& plane)
 {
   return clip(tm, plane, parameters::all_default());
 }
 
+// convenience overloads
+template <class TriangleMesh>
+bool clip(TriangleMesh& tm,
+          const typename GetGeomTraits<TriangleMesh>::type::Iso_cuboid_3& iso_cuboid)
+{
+  return clip(tm, iso_cuboid, parameters::all_default());
+}
+
 // convenience overload
 template <class TriangleMesh,
           class NamedParameters1>
 bool
-clip(      TriangleMesh& tm,
-           TriangleMesh& clipper,
+clip(TriangleMesh& tm,
+     TriangleMesh& clipper,
      const NamedParameters1& np_tm)
 {
   return clip(tm, clipper, np_tm, parameters::all_default());
@@ -456,11 +935,47 @@ clip(      TriangleMesh& tm,
 // convenience overload
 template <class TriangleMesh>
 bool
-clip(      TriangleMesh& tm,
-           TriangleMesh& clipper)
+clip(TriangleMesh& tm,
+     TriangleMesh& clipper)
 {
   return clip(tm, clipper, parameters::all_default());
 }
+
+
+// convenience overload
+template <class TriangleMesh,
+          class NamedParameters1>
+void
+split(TriangleMesh& tm,
+      TriangleMesh& splitter,
+      const NamedParameters1& np_tm)
+{
+  split(tm, splitter, np_tm, parameters::all_default());
+}
+
+// convenience overload
+template <class TriangleMesh>
+void
+split(TriangleMesh& tm,
+      TriangleMesh& splitter)
+{
+  split(tm, splitter, parameters::all_default());
+}
+
+template <class TriangleMesh>
+void split(TriangleMesh& tm,
+           const typename GetGeomTraits<TriangleMesh>::type::Plane_3& plane)
+{
+   split(tm, plane, parameters::all_default());
+}
+
+template <class TriangleMesh>
+void split(TriangleMesh& tm,
+           const typename GetGeomTraits<TriangleMesh>::type::Iso_cuboid_3& iso_cuboid)
+{
+  split(tm, iso_cuboid, parameters::all_default());
+}
+
 /// \endcond
 
 } } //end of namespace CGAL::Polygon_mesh_processing
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/compute_normal.h 5.1-1/include/CGAL/Polygon_mesh_processing/compute_normal.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/compute_normal.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/compute_normal.h	2020-09-08 12:15:23.000000000 +0000
@@ -1,122 +1,153 @@
-// Copyright (c) 2015 GeometryFactory (France).
+// Copyright (c) 2015-2019 GeometryFactory (France).
 // All rights reserved.
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/compute_normal.h $
-// $Id: compute_normal.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/compute_normal.h $
+// $Id: compute_normal.h 80f6f65 2020-04-05T17:56:50+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
 // Author(s)     : Jane Tournois
-
+//                 Mael Rouxel-Labbé
 
 #ifndef CGAL_POLYGON_MESH_PROCESSING_COMPUTE_NORMAL_H
 #define CGAL_POLYGON_MESH_PROCESSING_COMPUTE_NORMAL_H
 
 #include <CGAL/license/Polygon_mesh_processing/Compute_normal.h>
 
+#include <CGAL/Polygon_mesh_processing/internal/named_function_params.h>
+#include <CGAL/Polygon_mesh_processing/internal/named_params_helper.h>
 
 #include <CGAL/boost/graph/helpers.h>
 #include <CGAL/boost/graph/properties.h>
-#include <boost/graph/graph_traits.hpp>
+#include <CGAL/Dynamic_property_map.h>
 #include <CGAL/Origin.h>
-#include <CGAL/Kernel/global_functions_3.h>
-#include <CGAL/Kernel_traits.h>
-
-#include <CGAL/Polygon_mesh_processing/internal/named_function_params.h>
-#include <CGAL/Polygon_mesh_processing/internal/named_params_helper.h>
 
-#include <boost/type_traits.hpp>
+#include <boost/graph/graph_traits.hpp>
 
-namespace CGAL{
+#include <iostream>
+#include <limits>
+#include <utility>
+#include <vector>
+
+#ifdef CGAL_PMP_COMPUTE_NORMAL_DEBUG_PP
+# ifndef CGAL_PMP_COMPUTE_NORMAL_DEBUG
+#  define CGAL_PMP_COMPUTE_NORMAL_DEBUG
+# endif
+#endif
 
-namespace Polygon_mesh_processing{
+#ifdef CGAL_PMP_COMPUTE_NORMAL_DEBUG
+#include <fstream>
+#include <CGAL/Polygon_mesh_processing/bbox.h>
+#endif
 
+namespace CGAL {
+namespace Polygon_mesh_processing {
 namespace internal {
 
-  template <class GT>
-  void normalize(typename GT::Vector_3& v, const GT& traits)
+template <class GT>
+void normalize(typename GT::Vector_3& v, const GT& traits)
+{
+  typedef typename GT::FT                                             FT;
+
+  //If the vector is small enough, approx_sqrt might return 0, and then we get nan values.
+  //To avoid that, we check the resulted norm. If it is 0, we don't normalize.
+  const FT norm = CGAL::approximate_sqrt(traits.compute_squared_length_3_object()(v));
+  if(norm != FT(0))
   {
-    typename GT::FT norm = CGAL::approximate_sqrt(
-        traits.compute_squared_length_3_object()(v));
-    //If the vector is small enough, approx_sqrt might return 0, and then we get nan values.
-    //To avoid that, we check the resulted norm. If it is 0, we don't normalize.
-    if(norm != 0)
-    {
-      v = traits.construct_divided_vector_3_object()(v, norm );
-    }
+    v = traits.construct_divided_vector_3_object()(v, norm);
   }
+}
 
-  template<typename Point
-         , typename GT>
-  typename GT::Vector_3
-  triangle_normal(const Point& p0, const Point& p1, const Point& p2
-                , const GT& traits)
-  {
-    typename GT::Vector_3 n = traits.construct_cross_product_vector_3_object()(
-      traits.construct_vector_3_object()(p1, p2),
-      traits.construct_vector_3_object()(p1, p0));
+template<typename Point, typename GT>
+typename GT::Vector_3
+triangle_normal(const Point& p0, const Point& p1, const Point& p2, const GT& traits)
+{
+  typedef typename GT::FT                                             FT;
 
-    //cross-product(AB, AC)'s norm is the area of the parallelogram
-    //formed by these 2 vectors.
-    //the triangle's area is half of it
-    return traits.construct_scaled_vector_3_object()(n, 0.5);
-  }
+  typename GT::Vector_3 n = traits.construct_cross_product_vector_3_object()(
+                              traits.construct_vector_3_object()(p1, p2),
+                              traits.construct_vector_3_object()(p1, p0));
+
+  //cross-product(AB, AC)'s norm is the area of the parallelogram
+  //formed by these 2 vectors.
+  //the triangle's area is half of it
+  return traits.construct_scaled_vector_3_object()(n, FT(1)/FT(2));
 }
 
-template<typename Point, typename PM, typename VertexPointMap, typename Vector
-       , typename GT>
+template<typename Point, typename PM, typename VertexPointMap, typename Vector, typename GT>
 void sum_normals(const PM& pmesh,
                  typename boost::graph_traits<PM>::face_descriptor f,
                  VertexPointMap vpmap,
                  Vector& sum,
                  const GT& traits)
 {
-  typedef typename boost::graph_traits<PM>::vertex_descriptor   vertex_descriptor;
-  typedef typename boost::graph_traits<PM>::halfedge_descriptor halfedge_descriptor;
+  typedef typename boost::graph_traits<PM>::vertex_descriptor           vertex_descriptor;
+  typedef typename boost::graph_traits<PM>::halfedge_descriptor         halfedge_descriptor;
+
+  typedef typename boost::property_traits<VertexPointMap>::reference    Point_ref;
 
   halfedge_descriptor he = halfedge(f, pmesh);
   vertex_descriptor v = source(he, pmesh);
-  const Point& pv = get(vpmap, v);
-  while (v != target(next(he, pmesh), pmesh))
+  const Point_ref pv = get(vpmap, v);
+
+  while(v != target(next(he, pmesh), pmesh))
   {
-    const Point& pvn  = get(vpmap, target(he, pmesh));
-    const Point& pvnn = get(vpmap, target(next(he, pmesh), pmesh));
+    const Point_ref pvn = get(vpmap, target(he, pmesh));
+    const Point_ref pvnn = get(vpmap, target(next(he, pmesh), pmesh));
+
+    const Vector n = internal::triangle_normal(pv, pvn, pvnn, traits);
+
+#ifdef CGAL_PMP_COMPUTE_NORMAL_DEBUG_PP
+    std::cout << "Normal of " << f << " pts: " << pv << " ; " << pvn << " ; " << pvnn << std::endl;
+    std::cout << " --> " << n << std::endl;
+#endif
 
-    Vector n = internal::triangle_normal(pv, pvn, pvnn, traits);
     sum = traits.construct_sum_of_vectors_3_object()(sum, n);
 
     he = next(he, pmesh);
   }
 }
 
+} // namespace internal
 
 /**
 * \ingroup PMP_normal_grp
 * computes the outward unit vector normal to face `f`.
 * @tparam PolygonMesh a model of `FaceGraph`
-* @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+* @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 *
-* @param f the face on which the normal is computed
+* @param f the face whose normal is computed
 * @param pmesh the polygon mesh containing `f`
-* @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+* @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 *
 * \cgalNamedParamsBegin
-*    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
-*   If this parameter is omitted, an internal property map for
-*   `CGAL::vertex_point_t` must be available in `PolygonMesh`\cgalParamEnd
-*    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+*   \cgalParamNBegin{vertex_point_map}
+*     \cgalParamDescription{a property map associating points to the vertices of `pmesh`}
+*     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+*                    as key type and `%Point_3` as value type}
+*     \cgalParamDefault{`boost::get(CGAL::vertex_point, pmesh)`}
+*     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+*                     must be available in `PolygonMesh`.}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{geom_traits}
+*     \cgalParamDescription{an instance of a geometric traits class}
+*     \cgalParamType{a class model of `Kernel`}
+*     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+*     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+*   \cgalParamNEnd
 * \cgalNamedParamsEnd
 *
 * @return the computed normal. The return type is a 3D vector type. It is
-* either deduced from the `geom_traits` \ref pmp_namedparameters "Named Parameters" if provided,
+* either deduced from the `geom_traits` \ref bgl_namedparameters "Named Parameters" if provided,
 * or from the geometric traits class deduced from the point property map
 * of `pmesh`.
 *
 * \warning This function involves a square root computation.
-* If `Kernel::FT` does not have a `sqrt()` operation, the square root computation
-* will be done approximately.
+* If the field type (`FT`) of the traits does not support the `sqrt()` operation,
+* the square root computation will be performed approximately.
 */
 template <typename PolygonMesh, typename NamedParameters>
 #ifdef DOXYGEN_RUNNING
@@ -124,95 +155,494 @@ Vector_3
 #else
 typename GetGeomTraits<PolygonMesh, NamedParameters>::type::Vector_3
 #endif
-compute_face_normal(typename boost::graph_traits<PolygonMesh>::face_descriptor f
-                    , const PolygonMesh& pmesh
-                    , const NamedParameters& np)
+compute_face_normal(typename boost::graph_traits<PolygonMesh>::face_descriptor f,
+                    const PolygonMesh& pmesh,
+                    const NamedParameters& np)
 {
   using parameters::choose_parameter;
   using parameters::get_parameter;
 
-  typedef typename GetGeomTraits<PolygonMesh, NamedParameters>::type GT;
-  GT traits = choose_parameter(get_parameter(np, internal_np::geom_traits), GT());
+  typedef typename GetGeomTraits<PolygonMesh, NamedParameters>::type               GT;
+  GT traits = choose_parameter<GT>(get_parameter(np, internal_np::geom_traits));
 
-  typedef typename GetVertexPointMap<PolygonMesh, NamedParameters>::const_type VPMap;
+  typedef typename GetVertexPointMap<PolygonMesh, NamedParameters>::const_type     VPMap;
   VPMap vpmap = choose_parameter(get_parameter(np, internal_np::vertex_point),
-                             get_const_property_map(vertex_point, pmesh));
+                                 get_const_property_map(vertex_point, pmesh));
 
-  typedef typename GT::Point_3 Point;
-  typedef typename GT::Vector_3 Vector;
+  typedef typename GT::Point_3                                                     Point;
+  typedef typename GT::Vector_3                                                    Vector_3;
 
-  Vector normal = traits.construct_vector_3_object()(CGAL::NULL_VECTOR);
-  sum_normals<Point>(pmesh, f, vpmap, normal, traits);
+  Vector_3 normal = traits.construct_vector_3_object()(CGAL::NULL_VECTOR);
+  internal::sum_normals<Point>(pmesh, f, vpmap, normal, traits);
 
-  if (!traits.equal_3_object()(normal, CGAL::NULL_VECTOR))
+  if(!traits.equal_3_object()(normal, CGAL::NULL_VECTOR))
     internal::normalize(normal, traits);
 
   return normal;
 }
 
+template <typename PolygonMesh>
+typename GetGeomTraits<PolygonMesh>::type::Vector_3
+compute_face_normal(typename boost::graph_traits<PolygonMesh>::face_descriptor f,
+                    const PolygonMesh& pmesh)
+{
+  return compute_face_normal(f, pmesh, CGAL::parameters::all_default());
+}
+
 /**
 * \ingroup PMP_normal_grp
 * computes the outward unit vector normal for all faces of the polygon mesh.
 * @tparam PolygonMesh a model of `FaceGraph`
-* @tparam FaceNormalMap a model of `WritablePropertyMap` with
+* @tparam Face_normal_map a model of `WritablePropertyMap` with
     `boost::graph_traits<PolygonMesh>::%face_descriptor` as key type and
     `Kernel::Vector_3` as value type.
 *
 * @param pmesh the polygon mesh
-* @param fnm the property map in which the normals are written
-* @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+* @param face_normals the property map in which the normals are written
+* @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 *
 * \cgalNamedParamsBegin
-*    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
-*   If this parameter is omitted, an internal property map for
-*   `CGAL::vertex_point_t` must be available in `PolygonMesh`\cgalParamEnd
-*    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+*   \cgalParamNBegin{vertex_point_map}
+*     \cgalParamDescription{a property map associating points to the vertices of `pmesh`}
+*     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+*                    as key type and `%Point_3` as value type}
+*     \cgalParamDefault{`boost::get(CGAL::vertex_point, pmesh)`}
+*     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+*                     must be available in `PolygonMesh`.}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{geom_traits}
+*     \cgalParamDescription{an instance of a geometric traits class}
+*     \cgalParamType{a class model of `Kernel`}
+*     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+*     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+*   \cgalParamNEnd
 * \cgalNamedParamsEnd
 *
 * \warning This function involves a square root computation.
-* If `Kernel::FT` does not have a `sqrt()` operation, the square root computation
-* will be done approximately.
+* If the field type (`FT`) of the traits does not support the `sqrt()` operation,
+* the square root computation will be performed approximately.
 */
-template <typename PolygonMesh
-          , typename FaceNormalMap
-          , typename NamedParameters>
-void
-compute_face_normals(const PolygonMesh& pmesh
-                   , FaceNormalMap fnm
-                   , const NamedParameters& np)
+template <typename PolygonMesh, typename Face_normal_map, typename NamedParameters>
+void compute_face_normals(const PolygonMesh& pmesh,
+                          Face_normal_map face_normals,
+                          const NamedParameters& np)
 {
   typedef typename GetGeomTraits<PolygonMesh,NamedParameters>::type Kernel;
 
-  for(typename boost::graph_traits<PolygonMesh>::face_descriptor f : faces(pmesh)){
+  for(typename boost::graph_traits<PolygonMesh>::face_descriptor f : faces(pmesh))
+  {
     typename Kernel::Vector_3 vec = compute_face_normal(f, pmesh, np);
-    put(fnm, f, vec);
+    put(face_normals, f, vec);
+#ifdef CGAL_PMP_COMPUTE_NORMAL_DEBUG_PP
+    std::cout << "normal at face " << f << " is " << get(face_normals, f) << std::endl;
+#endif
+  }
+}
+
+template <typename PolygonMesh, typename Face_normal_map>
+void compute_face_normals(const PolygonMesh& pmesh, Face_normal_map face_normals)
+{
+  compute_face_normals(pmesh, face_normals, CGAL::parameters::all_default());
+}
+
+namespace internal {
+
+enum Vertex_normal_type {
+  NO_WEIGHT = 0,
+  SIN_WEIGHT,
+  MOST_VISIBLE
+};
+
+template <typename GT>
+bool almost_equal(const typename GT::Vector_3& v1, const typename GT::Vector_3& v2,
+                  const GT& traits)
+{
+  typedef typename GT::FT                                                     FT;
+
+  // We are doing a lot of likely inexact constructions to compute min circles on the sphere and
+  // these degenerate cases often happen (e.g. almost flat surfaces), so we don't want to to the usual
+  // "switch to the exact kernel" in degenerate cases robustness trick. Rather, use a fixed tolerance
+  // to assimilate vectors and scalar products.
+  //
+  // Tolerance is (arbitrarily) chosen as theta := 0.01°, thus we want sp(v1,v2) >= cos(theta)
+  const FT cos_theta = 0.99999998476912910;
+  return traits.compute_scalar_product_3_object()(v1, v2) >= cos_theta;
+}
+
+template <typename PolygonMesh, typename FaceNormalVector, typename K>
+bool does_enclose_other_normals(const std::size_t i, const std::size_t j, const std::size_t k,
+                                const typename K::Vector_3& nb,
+                                const typename K::FT sp_bi,
+                                const std::vector<typename boost::graph_traits<PolygonMesh>::face_descriptor>& incident_faces,
+                                const FaceNormalVector& face_normals,
+                                const K& traits)
+{
+  typedef typename K::FT                                                   FT;
+  typedef typename boost::property_traits<FaceNormalVector>::reference     Vector_ref;
+
+  typename K::Compute_scalar_product_3 sp = traits.compute_scalar_product_3_object();
+
+  const FT nbn = CGAL::approximate_sqrt(traits.compute_squared_length_3_object()(nb));
+
+  // check that this min circle defined by the diameter contains the other points
+  const std::size_t nif = incident_faces.size();
+  for(std::size_t l=0; l<nif; ++l)
+  {
+    if(l == i || l == j || l == k)
+      continue;
+
+    const Vector_ref nl = get(face_normals, incident_faces[l]);
+    if(nl == CGAL::NULL_VECTOR)
+      continue;
+
+    // this is a bound on how much the scalar product between (v1,v2) and (v1, v3) can change
+    // when the angle changes theta_bound := 0.01°
+    // The weird number is thus := max_(theta_i, theta_j)[abs(std::cos(theta_i), std::cos(theta_j))]
+    // with theta_j - theta_i = theta_bound
+    const FT sp_diff_bound = nbn * 0.00017453292431333;
+    const FT sp_bl = sp(nb, nl);
+
+    // norm of nl is 1 by construction
+    if(CGAL::abs(sp_bi - sp_bl) <= sp_diff_bound)
+      continue;
+
+    if(sp_bl < sp_bi)
+      return false;
+  }
+
+  return true;
+}
+
+template <typename GT>
+typename GT::Vector_3 compute_normals_bisector(const typename GT::Vector_3& ni,
+                                               const typename GT::Vector_3& nj,
+                                               const GT& traits)
+{
+  if(traits.equal_3_object()(ni, nj))
+    return ni;
+
+  return traits.construct_sum_of_vectors_3_object()(ni, nj); // not normalized
+}
+
+template <typename GT>
+typename GT::Vector_3 compute_normals_bisector(const typename GT::Vector_3& ni,
+                                               const typename GT::Vector_3& nj,
+                                               const typename GT::Vector_3& nk,
+                                               const GT& traits)
+{
+  typedef typename GT::FT                                                  FT;
+  typedef typename GT::Point_3                                             Point_3;
+  typedef typename GT::Vector_3                                            Vector_3;
+
+  typename GT::Construct_scaled_vector_3 cslv_3 = traits.construct_scaled_vector_3_object();
+  typename GT::Construct_sum_of_vectors_3 csv_3 = traits.construct_sum_of_vectors_3_object();
+  typename GT::Construct_vector_3 cv_3 = traits.construct_vector_3_object();
+
+  Vector_3 nb = cv_3(CGAL::NULL_VECTOR);
+
+  if(almost_equal(ni, nj, traits) || nk == CGAL::NULL_VECTOR)
+  {
+    if(almost_equal(nj, nk, traits))
+      nb = ni;
+    else // ni == nj, but nij != nk
+      nb = compute_normals_bisector(nj, nk, traits);
+  }
+  else if(almost_equal(ni, nk, traits) || nj == CGAL::NULL_VECTOR) // ni != nj
+  {
+    nb = compute_normals_bisector(nj, nk, traits);
+  }
+  else if(almost_equal(nj, nk, traits) || ni == CGAL::NULL_VECTOR) // ni != nj, ni != nk
+  {
+    nb = compute_normals_bisector(ni, nk, traits);
+  }
+  else
+  {
+    CGAL_assertion(ni != nj);
+    CGAL_assertion(ni != nk);
+    CGAL_assertion(nj != nk);
+    CGAL_assertion(!traits.collinear_3_object()(CGAL::ORIGIN + ni, CGAL::ORIGIN + nj, CGAL::ORIGIN + nk));
+
+#ifdef CGAL_PMP_COMPUTE_NORMAL_DEBUG_PP
+    std::cout << "Triplet: ni[" << ni << "] nj[" << nj << "] nk[" << nk << "]" << std::endl;
+#endif
+
+    const Point_3 c = traits.construct_circumcenter_3_object()(CGAL::ORIGIN + ni, CGAL::ORIGIN + nj, CGAL::ORIGIN + nk);
+    if(c == CGAL::ORIGIN)
+    {
+      // will happen if the three vectors live in the same plan, return some weighted sum
+      const FT third = FT(1)/FT(3);
+      return csv_3(csv_3(cslv_3(ni, third), cslv_3(nj, third)), cslv_3(nk, third));
+    }
+
+    nb = cv_3(CGAL::ORIGIN, c); // not normalized
+  }
+
+  return nb;
+}
+
+template <typename PolygonMesh, typename FaceNormalVector, typename GT>
+typename GT::Vector_3
+compute_most_visible_normal_2_points(std::vector<typename boost::graph_traits<PolygonMesh>::face_descriptor>& incident_faces,
+                                     const FaceNormalVector& face_normals,
+                                     const GT& traits)
+{
+  typedef typename GT::FT                                                  FT;
+  typedef typename GT::Vector_3                                            Vector_3;
+  typedef typename boost::property_traits<FaceNormalVector>::reference     Vector_ref;
+
+  typename GT::Compute_scalar_product_3 sp_3 = traits.compute_scalar_product_3_object();
+  typename GT::Construct_vector_3 cv_3 = traits.construct_vector_3_object();
+
+#ifdef CGAL_PMP_COMPUTE_NORMAL_DEBUG_PP
+  std::cout << "Trying to find enclosing normal with 2 normals" << std::endl;
+#endif
+
+  FT min_sp = -1;
+  Vector_3 n = cv_3(CGAL::NULL_VECTOR);
+
+  const std::size_t nif = incident_faces.size();
+  for(std::size_t i=0; i<nif; ++i)
+  {
+    for(std::size_t j=i+1; j<nif; ++j)
+    {
+      const Vector_ref ni = get(face_normals, incident_faces[i]);
+      const Vector_ref nj = get(face_normals, incident_faces[j]);
+
+      const Vector_3 nb = compute_normals_bisector(ni, nj, traits);
+
+      // Degeneracies like ni == -nj or a numerical error in the construction of 'nb' can happen.
+      if(traits.equal_3_object()(nb, CGAL::NULL_VECTOR))
+        return CGAL::NULL_VECTOR;
+
+      FT sp_bi = sp_3(nb, ni);
+      sp_bi = (std::max)(FT(0), sp_bi);
+      if(sp_bi <= min_sp)
+        continue;
+
+      if(!does_enclose_other_normals<PolygonMesh>(i, j, -1 /*NA*/, nb, sp_bi, incident_faces, face_normals, traits))
+        continue;
+
+      min_sp = sp_bi;
+      n = nb;
+    }
+  }
+
+  return n;
+}
+
+template <typename PolygonMesh, typename FaceNormalVector, typename GT>
+typename GT::Vector_3
+compute_most_visible_normal_3_points(const std::vector<typename boost::graph_traits<PolygonMesh>::face_descriptor>& incident_faces,
+                                     const FaceNormalVector& face_normals,
+                                     const GT& traits)
+{
+  typedef typename GT::FT                                                  FT;
+  typedef typename GT::Vector_3                                            Vector_3;
+  typedef typename boost::property_traits<FaceNormalVector>::reference     Vector_ref;
+
+#ifdef CGAL_PMP_COMPUTE_NORMAL_DEBUG_PP
+  std::cout << "Trying to find enclosing normal with 3 normals" << std::endl;
+#endif
+
+  FT min_sp = -1;
+
+  Vector_3 n = traits.construct_vector_3_object()(CGAL::NULL_VECTOR);
+
+  const std::size_t nif = incident_faces.size();
+  for(std::size_t i=0; i<nif; ++i)
+  {
+    for(std::size_t j=i+1; j<nif; ++j)
+    {
+      for(std::size_t k=j+1; k<nif; ++k)
+      {
+        const Vector_ref ni = get(face_normals, incident_faces[i]);
+        const Vector_ref nj = get(face_normals, incident_faces[j]);
+        const Vector_ref nk = get(face_normals, incident_faces[k]);
+
+        if(ni == CGAL::NULL_VECTOR || nj == CGAL::NULL_VECTOR || nk == CGAL::NULL_VECTOR)
+          continue;
+
+        Vector_3 nb = compute_normals_bisector(ni, nj, nk, traits);
+        if(traits.equal_3_object()(nb, CGAL::NULL_VECTOR))
+          return nb;
+
+        FT sp_bi = traits.compute_scalar_product_3_object()(nb, ni);
+        if(sp_bi < FT(0))
+        {
+          nb = traits.construct_opposite_vector_3_object()(nb);
+          sp_bi = - sp_bi;
+        }
+
+        if(sp_bi <= min_sp)
+          continue;
+
+        if(!does_enclose_other_normals<PolygonMesh>(i, j, k, nb, sp_bi, incident_faces, face_normals, traits))
+          continue;
+
+        min_sp = sp_bi;
+        n = nb;
+      }
+    }
+  }
+
+#ifdef CGAL_PMP_COMPUTE_NORMAL_DEBUG_PP
+  std::cout << "Best normal from 3-normals-approach: " << n << std::endl;
+#endif
+
+  return n;
+}
+
+// Inspired by Aubry et al. On the most 'normal' normal
+template <typename PolygonMesh, typename FaceNormalVector, typename GT>
+typename GT::Vector_3
+compute_vertex_normal_most_visible_min_circle(typename boost::graph_traits<PolygonMesh>::vertex_descriptor v,
+                                              const FaceNormalVector& face_normals,
+                                              const PolygonMesh& pmesh,
+                                              const GT& traits)
+{
+  typedef typename boost::graph_traits<PolygonMesh>::face_descriptor       face_descriptor;
+
+  typedef typename GT::Vector_3                                            Vector_3;
+
+  std::vector<face_descriptor> incident_faces;
+  for(face_descriptor f : CGAL::faces_around_target(halfedge(v, pmesh), pmesh))
+  {
+    if(f == boost::graph_traits<PolygonMesh>::null_face())
+      continue;
+
+    incident_faces.push_back(f);
   }
+
+  if(incident_faces.size() == 1)
+    return get(face_normals, incident_faces.front());
+
+  Vector_3 res = compute_most_visible_normal_2_points<PolygonMesh>(incident_faces, face_normals, traits);
+
+  if(res != CGAL::NULL_VECTOR) // found a valid normal through 2 point min circle
+    return res;
+
+  // The vertex has only two incident faces with opposite normals (fold)...
+  // @todo devise something based on the directions of the 2/3/4 incident edges?
+  if(incident_faces.size() == 2 && res == CGAL::NULL_VECTOR)
+    return res;
+
+  CGAL_assertion(incident_faces.size() >= 2);
+
+  return compute_most_visible_normal_3_points<PolygonMesh>(incident_faces, face_normals, traits);
 }
 
+template <typename PolygonMesh, typename FaceNormalVector, typename VertexPointMap, typename GT>
+typename GT::Vector_3
+compute_vertex_normal_as_sum_of_weighted_normals(typename boost::graph_traits<PolygonMesh>::vertex_descriptor v,
+                                                 const Vertex_normal_type& vn_type,
+                                                 const FaceNormalVector& face_normals,
+                                                 const VertexPointMap& vpmap,
+                                                 const PolygonMesh& pmesh,
+                                                 const GT& traits)
+{
+  typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor      halfedge_descriptor;
+
+  typedef typename GT::FT                                                     FT;
+  typedef typename GT::Vector_3                                               Vector_3;
+  typedef typename boost::property_traits<FaceNormalVector>::reference        Vector_ref;
+
+  typename GT::Construct_vector_3 cv_3 = traits.construct_vector_3_object();
+  typename GT::Compute_squared_length_3 csl_3 = traits.compute_squared_length_3_object();
+
+#ifdef CGAL_PMP_COMPUTE_NORMAL_DEBUG_PP
+  std::cout << "Compute normal as weighted sums; type: " << vn_type << std::endl;
+#endif
+
+  Vector_3 normal = cv_3(CGAL::NULL_VECTOR);
+
+  halfedge_descriptor h = halfedge(v, pmesh);
+  if(h == boost::graph_traits<PolygonMesh>::null_halfedge())
+    return normal;
+
+  halfedge_descriptor end = h;
+  do
+  {
+    if(!is_border(h, pmesh))
+    {
+      if(vn_type == NO_WEIGHT)
+      {
+        const Vector_ref n = get(face_normals, face(h, pmesh));
+        normal = traits.construct_sum_of_vectors_3_object()(normal, n);
+      }
+      else if(vn_type == SIN_WEIGHT)
+      {
+        const Vector_3 v1 = cv_3(get(vpmap, v), get(vpmap, source(h, pmesh)));
+        const Vector_3 v2 = cv_3(get(vpmap, v), get(vpmap, target(next(h, pmesh), pmesh)));
+
+        //v(i) and v(i+1) must be seen in ccw order, from v, so we reverse v1 and v2
+        Vector_3 n = traits.construct_cross_product_vector_3_object()(v2, v1);
+        const FT den = CGAL::approximate_sqrt(csl_3(v1) * csl_3(v2));
+
+        if(den == FT(0))
+        {
+#ifdef CGAL_PMP_COMPUTE_NORMAL_DEBUG_PP
+          std::cout << "Null denominator, switching to no weights" << std::endl;
+#endif
+
+          return compute_vertex_normal_as_sum_of_weighted_normals(v, NO_WEIGHT, face_normals, vpmap, pmesh, traits);
+        }
+
+        n = traits.construct_scaled_vector_3_object()(n, FT(1) / den);
+        normal = traits.construct_sum_of_vectors_3_object()(normal, n);
+      }
+      else
+      {
+        std::cerr << "Error: unknown vertex normal type" << std::endl;
+        CGAL_assertion(false);
+        return CGAL::NULL_VECTOR;
+      }
+    }
+
+    h = opposite(next(h, pmesh), pmesh);
+  }
+  while(h != end);
+
+  return normal;
+}
+
+} // end namespace internal
+
 /**
 * \ingroup PMP_normal_grp
 * computes the unit normal at vertex `v` as the average of the normals of incident faces.
 * @tparam PolygonMesh a model of `FaceGraph`
 *
-* @param v the vertex at which the normal is computed
+* @param v the vertex whose normal is computed
 * @param pmesh the polygon mesh containing `v`
-* @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+* @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 *
 * \cgalNamedParamsBegin
-*    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
-*   If this parameter is omitted, an internal property map for
-*   `CGAL::vertex_point_t` must be available in `PolygonMesh`\cgalParamEnd
-*    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+*   \cgalParamNBegin{vertex_point_map}
+*     \cgalParamDescription{a property map associating points to the vertices of `pmesh`}
+*     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+*                    as key type and `%Point_3` as value type}
+*     \cgalParamDefault{`boost::get(CGAL::vertex_point, pmesh)`}
+*     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+*                     must be available in `PolygonMesh`.}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{geom_traits}
+*     \cgalParamDescription{an instance of a geometric traits class}
+*     \cgalParamType{a class model of `Kernel`}
+*     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+*     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+*   \cgalParamNEnd
 * \cgalNamedParamsEnd
 *
 * @return the computed normal. The return type is a 3D vector type. It is
-* either deduced from the `geom_traits` \ref pmp_namedparameters "Named Parameters" if provided,
+* either deduced from the `geom_traits` \ref bgl_namedparameters "Named Parameters" if provided,
 * or the geometric traits class deduced from the point property map
 * of `pmesh`.
 *
 * \warning This function involves a square root computation.
-* If `Kernel::FT` does not have a `sqrt()` operation, the square root computation
-* will be done approximately.
+* If the field type (`FT`) of the traits does not support the `sqrt()` operation,
+* the square root computation will be performed approximately.
 */
 template<typename PolygonMesh, typename NamedParameters>
 #ifdef DOXYGEN_RUNNING
@@ -222,192 +652,244 @@ typename GetGeomTraits<PolygonMesh, Name
 #endif
 compute_vertex_normal(typename boost::graph_traits<PolygonMesh>::vertex_descriptor v,
                       const PolygonMesh& pmesh,
-                      const NamedParameters& np
-                      )
+                      const NamedParameters& np)
 {
   using parameters::choose_parameter;
+  using parameters::is_default_parameter;
   using parameters::get_parameter;
 
-  typedef typename GetGeomTraits<PolygonMesh, NamedParameters>::type GT;
-  typedef typename GT::Vector_3 Vector;
-  GT traits = choose_parameter(get_parameter(np, internal_np::geom_traits), GT());
-
-  typedef typename GetFaceNormalMap<PolygonMesh, NamedParameters>::NoMap DefaultMap;
-  typedef typename internal_np::Lookup_named_param_def <
-    internal_np::face_normal_t,
-    NamedParameters,
-    DefaultMap> ::type FaceNormalMap;
-  FaceNormalMap fnmap = choose_parameter(get_parameter(np, internal_np::face_normal), DefaultMap());
-  bool fnmap_valid
-    = !boost::is_same<FaceNormalMap,
-                      DefaultMap
-                     >::value;
+  typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor      halfedge_descriptor;
+  typedef typename boost::graph_traits<PolygonMesh>::face_descriptor          face_descriptor;
+
+  typedef typename GetGeomTraits<PolygonMesh, NamedParameters>::type          GT;
+  typedef typename GT::Vector_3                                               Vector_3;
+  GT traits = choose_parameter<GT>(get_parameter(np, internal_np::geom_traits));
 
-  typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor halfedge_descriptor;
+  typedef typename GetVertexPointMap<PolygonMesh, NamedParameters>::const_type VPMap;
+  VPMap vpmap = choose_parameter(get_parameter(np, internal_np::vertex_point),
+                                 get_const_property_map(vertex_point, pmesh));
 
-  Vector normal = traits.construct_vector_3_object()(CGAL::NULL_VECTOR);
+  typedef std::map<face_descriptor, Vector_3>                                 Face_vector_map;
+  typedef boost::associative_property_map<Face_vector_map>                    Default_map;
+
+  typedef typename internal_np::Lookup_named_param_def<internal_np::face_normal_t,
+                                                       NamedParameters,
+                                                       Default_map>::type     Face_normal_map;
+  Face_vector_map default_fvmap;
+  Face_normal_map face_normals = choose_parameter(get_parameter(np, internal_np::face_normal),
+                                                  Default_map(default_fvmap));
+  const bool must_compute_face_normals = is_default_parameter(get_parameter(np, internal_np::face_normal));
+
+#ifdef CGAL_PMP_COMPUTE_NORMAL_DEBUG_PP
+  std::cout << "<----- compute vertex normal at " << get(vpmap, v)
+            << ", must compute face normals? " << must_compute_face_normals << std::endl;
+#endif
 
-  halfedge_descriptor he = halfedge(v, pmesh);
   // handle isolated vertices
-  if (he==boost::graph_traits<PolygonMesh>::null_halfedge()) return normal;
-  halfedge_descriptor end = he;
-  do
+  halfedge_descriptor he = halfedge(v, pmesh);
+  if(he == boost::graph_traits<PolygonMesh>::null_halfedge())
+    return CGAL::NULL_VECTOR;
+
+  if(must_compute_face_normals)
   {
-    if (!is_border(he, pmesh))
+    for(face_descriptor f : CGAL::faces_around_target(halfedge(v, pmesh), pmesh))
     {
-      Vector n = fnmap_valid ? get(fnmap, face(he, pmesh))
-                             : compute_face_normal(face(he, pmesh), pmesh, np);
-      normal = traits.construct_sum_of_vectors_3_object()(normal, n);
+      if(f == boost::graph_traits<PolygonMesh>::null_face())
+        continue;
+
+      put(face_normals, f, compute_face_normal(f, pmesh, np));
     }
-    he = opposite(next(he, pmesh), pmesh);
-  } while (he != end);
+  }
+
+#ifdef CGAL_PMP_COMPUTE_NORMAL_DEBUG_PP
+  std::cout << "Incident face normals:" << std::endl;
+  for(halfedge_descriptor h : CGAL::halfedges_around_target(v, pmesh))
+  {
+    if(!is_border(h, pmesh))
+      std::cout << "get normal at f " << face(h, pmesh) << " : " << get(face_normals, face(h, pmesh)) << std::endl;
+  }
+#endif
+
+  Vector_3 normal = internal::compute_vertex_normal_most_visible_min_circle(v, face_normals, pmesh, traits);
+  if(traits.equal_3_object()(normal, CGAL::NULL_VECTOR)) // can't always find a most visible normal
+  {
+#ifdef CGAL_PMP_COMPUTE_NORMAL_DEBUG_PP
+    std::cout << "Failed to find most visible normal, use weighted sum of normals" << std::endl;
+#endif
+    normal = internal::compute_vertex_normal_as_sum_of_weighted_normals(
+               v, internal::SIN_WEIGHT, face_normals, vpmap, pmesh, traits);
+  }
 
-  if ( ! traits.equal_3_object()(normal, CGAL::NULL_VECTOR))
+  if(!traits.equal_3_object()(normal, CGAL::NULL_VECTOR))
     internal::normalize(normal, traits);
+
   return normal;
 }
 
+template <typename PolygonMesh>
+typename GetGeomTraits<PolygonMesh>::type::Vector_3
+compute_vertex_normal(typename boost::graph_traits<PolygonMesh>::vertex_descriptor v,
+                      const PolygonMesh& pmesh)
+{
+  return compute_vertex_normal(v, pmesh, CGAL::parameters::all_default());
+}
+
 /**
 * \ingroup PMP_normal_grp
 * computes the outward unit vector normal for all vertices of the polygon mesh.
+*
 * @tparam PolygonMesh a model of `FaceListGraph`
 * @tparam VertexNormalMap a model of `WritablePropertyMap` with
-    `boost::graph_traits<PolygonMesh>::%vertex_descriptor` as key type and
-    the return type of `compute_vertex_normal()` as value type.
+*                         `boost::graph_traits<PolygonMesh>::%vertex_descriptor` as key type and
+*                         the return type of `compute_vertex_normal()` as value type.
 *
 * @param pmesh the polygon mesh
-* @param vnm the property map in which the normals are written
-* @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+* @param vertex_normals the property map in which the normals are written
+* @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 *
 * \cgalNamedParamsBegin
-*    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
-*   If this parameter is omitted, an internal property map for
-*   `CGAL::vertex_point_t` must be available in `PolygonMesh`\cgalParamEnd
-*    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+*   \cgalParamNBegin{vertex_point_map}
+*     \cgalParamDescription{a property map associating points to the vertices of `pmesh`}
+*     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+*                    as key type and `%Point_3` as value type}
+*     \cgalParamDefault{`boost::get(CGAL::vertex_point, pmesh)`}
+*     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+*                     must be available in `PolygonMesh`.}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{geom_traits}
+*     \cgalParamDescription{an instance of a geometric traits class}
+*     \cgalParamType{a class model of `Kernel`}
+*     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+*     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+*   \cgalParamNEnd
 * \cgalNamedParamsEnd
 *
 * \warning This function involves a square root computation.
-* If `Kernel::FT` does not have a `sqrt()` operation, the square root computation
-* will be done approximately.
+* If the field type (`FT`) of the traits does not support the `sqrt()` operation,
+* the square root computation will be performed approximately.
 */
-template <typename PolygonMesh
-          , typename VertexNormalMap
-          , typename NamedParameters
-          >
-void
-compute_vertex_normals(const PolygonMesh& pmesh
-                      , VertexNormalMap vnm
-                      , const NamedParameters& np
-                      )
+template <typename PolygonMesh, typename VertexNormalMap, typename NamedParameters>
+void compute_vertex_normals(const PolygonMesh& pmesh,
+                            VertexNormalMap vertex_normals,
+                            const NamedParameters& np)
 {
-  typedef typename GetGeomTraits<PolygonMesh,NamedParameters>::type Kernel;
+  using parameters::choose_parameter;
+  using parameters::is_default_parameter;
+  using parameters::get_parameter;
 
-  for(typename boost::graph_traits<PolygonMesh>::vertex_descriptor v : vertices(pmesh)){
-    typename Kernel::Vector_3 vec = compute_vertex_normal(v, pmesh, np);
-    put(vnm, v, vec);
+  typedef typename boost::graph_traits<PolygonMesh>::vertex_descriptor           vertex_descriptor;
+
+  typedef typename GetGeomTraits<PolygonMesh,NamedParameters>::type              GT;
+  typedef typename GT::Vector_3                                                  Vector_3;
+
+  typedef CGAL::dynamic_face_property_t<Vector_3>                                Face_normal_tag;
+  typedef typename boost::property_map<PolygonMesh, Face_normal_tag>::const_type Face_normal_dmap;
+
+#ifdef CGAL_PMP_COMPUTE_NORMAL_DEBUG_PP
+  GT traits = choose_parameter<GT>(get_parameter(np, internal_np::geom_traits));
+
+  typedef typename GetVertexPointMap<PolygonMesh, NamedParameters>::const_type   VPMap;
+  VPMap vpmap = choose_parameter(get_parameter(np, internal_np::vertex_point),
+                                 get_const_property_map(vertex_point, pmesh));
+#endif
+
+  typedef typename internal_np::Lookup_named_param_def<internal_np::face_normal_t,
+                                                       NamedParameters,
+                                                       Face_normal_dmap>::type   Face_normal_map;
+  Face_normal_map face_normals = choose_parameter(get_parameter(np, internal_np::face_normal),
+                                                  get(Face_normal_tag(), pmesh));
+  const bool must_compute_face_normals = is_default_parameter(get_parameter(np, internal_np::face_normal));
+
+  if(must_compute_face_normals)
+    compute_face_normals(pmesh, face_normals, np);
+
+#ifdef CGAL_PMP_COMPUTE_NORMAL_DEBUG_PP
+  std::ofstream out("computed_normals.cgal.polylines.txt");
+  const Bbox_3 bb = bbox(pmesh, np);
+  const typename GT::FT bbox_diagonal = CGAL::sqrt(CGAL::square(bb.xmax() - bb.xmin()) +
+                                                   CGAL::square(bb.ymax() - bb.ymin()) +
+                                                   CGAL::square(bb.zmax() - bb.zmin()));
+#endif
+
+  for(vertex_descriptor v : vertices(pmesh))
+  {
+    const Vector_3 n = compute_vertex_normal(v, pmesh, np.face_normal_map(face_normals));
+    put(vertex_normals, v, n);
+
+#ifdef CGAL_PMP_COMPUTE_NORMAL_DEBUG_PP
+    out << "2 " << get(vpmap, v) << " "
+                << get(vpmap, v) + traits.construct_scaled_vector_3_object()(n, 0.1 * bbox_diagonal) << "\n";
+#endif
   }
 }
 
+template <typename PolygonMesh, typename VertexNormalMap>
+void compute_vertex_normals(const PolygonMesh& pmesh, VertexNormalMap vertex_normals)
+{
+  compute_vertex_normals(pmesh, vertex_normals, CGAL::parameters::all_default());
+}
+
 /**
 * \ingroup PMP_normal_grp
 * computes the outward unit vector normal for all vertices and faces of the polygon mesh.
+*
 * @tparam PolygonMesh a model of `FaceListGraph`
 * @tparam VertexNormalMap a model of `WritablePropertyMap` with
-    `boost::graph_traits<PolygonMesh>::%vertex_descriptor` as key type and
-    `Kernel::Vector_3` as value type.
+*    `boost::graph_traits<PolygonMesh>::%vertex_descriptor` as key type and
+*    `Kernel::Vector_3` as value type.
 * @tparam FaceNormalMap a model of `ReadWritePropertyMap` with
-    `boost::graph_traits<PolygonMesh>::%face_descriptor` as key type and
-    `Kernel::Vector_3` as value type.
+*    `boost::graph_traits<PolygonMesh>::%face_descriptor` as key type and
+*    `Kernel::Vector_3` as value type.
 *
 * @param pmesh the polygon mesh
-* @param vnm the property map in which the vertex normals are written
-* @param fnm the property map in which the face normals are written
-* @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+* @param vertex_normals the property map in which the vertex normals are written
+* @param face_normals the property map in which the face normals are written
+* @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 *
 * \cgalNamedParamsBegin
-*    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
-*   If this parameter is omitted, an internal property map for
-*   `CGAL::vertex_point_t` must be available in `PolygonMesh`\cgalParamEnd
-*    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+*   \cgalParamNBegin{vertex_point_map}
+*     \cgalParamDescription{a property map associating points to the vertices of `pmesh`}
+*     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+*                    as key type and `%Point_3` as value type}
+*     \cgalParamDefault{`boost::get(CGAL::vertex_point, pmesh)`}
+*     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+*                     must be available in `PolygonMesh`.}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{geom_traits}
+*     \cgalParamDescription{an instance of a geometric traits class}
+*     \cgalParamType{a class model of `Kernel`}
+*     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+*     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+*   \cgalParamNEnd
 * \cgalNamedParamsEnd
 *
 * \warning This function involves a square root computation.
-* If `Kernel::FT` does not have a `sqrt()` operation, the square root computation
-* will be done approximately.
+* If the field type (`FT`) of the traits does not support the `sqrt()` operation,
+* the square root computation will be performed approximately.
 */
-template <typename PolygonMesh
-          , typename VertexNormalMap
-          , typename FaceNormalMap
-          , typename NamedParameters
-          >
-void
-compute_normals(const PolygonMesh& pmesh
-                , VertexNormalMap vnm
-                , FaceNormalMap fnm
-                , const NamedParameters& np
-                )
+template <typename PolygonMesh,
+          typename VertexNormalMap, typename FaceNormalMap,
+          typename NamedParameters>
+void compute_normals(const PolygonMesh& pmesh,
+                     VertexNormalMap vertex_normals,
+                     FaceNormalMap face_normals,
+                     const NamedParameters& np)
 {
-  compute_face_normals(pmesh, fnm, np);
-  compute_vertex_normals(pmesh, vnm, np.face_normal_map(fnm));
+  compute_face_normals(pmesh, face_normals, np);
+  compute_vertex_normals(pmesh, vertex_normals, np.face_normal_map(face_normals));
 }
 
-///\cond SKIP_IN_MANUAL
-// compute_vertex_normal overloads
-template <typename PolygonMesh>
-typename CGAL::Kernel_traits< typename property_map_value<PolygonMesh, CGAL::vertex_point_t>::type>::Kernel::Vector_3
-compute_vertex_normal(
-  typename boost::graph_traits<PolygonMesh>::vertex_descriptor v,
-  const PolygonMesh& pmesh)
-{
-  return compute_vertex_normal(v, pmesh,
-    CGAL::Polygon_mesh_processing::parameters::all_default());
-}
-
-// compute_vertex_normals overloads
-template <typename PolygonMesh, typename VertexNormalMap>
-void
-compute_vertex_normals(const PolygonMesh& pmesh,
-                      VertexNormalMap vnm)
-{
-  compute_vertex_normals(pmesh, vnm,
-    CGAL::Polygon_mesh_processing::parameters::all_default());
-}
-
-// compute_face_normal overload
-template <typename PolygonMesh>
-typename CGAL::Kernel_traits < typename property_map_value<PolygonMesh, CGAL::vertex_point_t>::type>::Kernel::Vector_3
-compute_face_normal(
-  typename boost::graph_traits<PolygonMesh>::face_descriptor f,
-  const PolygonMesh& pmesh)
-{
-  return compute_face_normal(f, pmesh,
-    CGAL::Polygon_mesh_processing::parameters::all_default());
-}
-
-// compute_face_normals overload
-template <typename PolygonMesh, typename FaceNormalMap>
-void
-compute_face_normals(const PolygonMesh& pmesh, FaceNormalMap fnm)
-{
-  compute_face_normals(pmesh, fnm,
-    CGAL::Polygon_mesh_processing::parameters::all_default());
-}
-
-// compute_normals overload
 template <typename PolygonMesh, typename VertexNormalMap, typename FaceNormalMap>
-void
-compute_normals(const PolygonMesh& pmesh,
-                VertexNormalMap vnm,
-                FaceNormalMap fnm)
+void compute_normals(const PolygonMesh& pmesh,
+                     VertexNormalMap vertex_normals,
+                     FaceNormalMap face_normals)
 {
-  compute_normals(pmesh, vnm, fnm,
-    CGAL::Polygon_mesh_processing::parameters::all_default());
-}
-
-/// \endcond
-
+  compute_normals(pmesh, vertex_normals, face_normals, CGAL::parameters::all_default());
 }
 
-} // end of namespace CGAL::Polygon_mesh_processing
+} // namespace Polygon_mesh_processing
+} // namespace CGAL
 
 #endif // CGAL_POLYGON_MESH_PROCESSING_COMPUTE_NORMAL_H
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/connected_components.h 5.1-1/include/CGAL/Polygon_mesh_processing/connected_components.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/connected_components.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/connected_components.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/connected_components.h $
-// $Id: connected_components.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/connected_components.h $
+// $Id: connected_components.h a84927d 2020-07-23T17:15:44+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -23,18 +23,23 @@
 #include <CGAL/boost/graph/Named_function_parameters.h>
 #include <CGAL/boost/graph/helpers.h>
 #include <boost/graph/graph_traits.hpp>
-#include <boost/graph/filtered_graph.hpp>
-#include <boost/graph/connected_components.hpp>
 #include <boost/property_map/vector_property_map.hpp>
 
+#include <CGAL/assertions.h>
 #include <CGAL/boost/graph/iterator.h>
 #include <CGAL/boost/graph/helpers.h>
+#include <CGAL/boost/graph/Face_filtered_graph.h>
+#include <CGAL/boost/graph/copy_face_graph.h>
+#include <CGAL/Container_helper.h>
 
 #include <CGAL/assertions.h>
 #include <CGAL/tuple.h>
+
 #include <CGAL/boost/graph/Dual.h>
-#include <CGAL/boost/graph/helpers.h>
 #include <CGAL/Default.h>
+#include <CGAL/Dynamic_property_map.h>
+#include <CGAL/iterator.h>
+#include <CGAL/tuple.h>
 
 #include <CGAL/Polygon_mesh_processing/internal/named_function_params.h>
 #include <CGAL/Polygon_mesh_processing/internal/named_params_helper.h>
@@ -87,7 +92,7 @@ namespace internal {
       EdgeConstraintMap ecm;
     };
 
-}// namespace internal
+} // namespace internal
 
 /*!
  * \ingroup keep_connected_components_grp
@@ -95,22 +100,25 @@ namespace internal {
  * `seed_face` will also be added in `out`.
  *
  *  \tparam PolygonMesh a model of `FaceGraph`
- *  \tparam FaceOutputIterator a model of `OutputIterator` that accepts
-        faces of type
-        `boost::graph_traits<PolygonMesh>::%face_descriptor`.
- *  \tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+ *  \tparam FaceOutputIterator a model of `OutputIterator` with value type `boost::graph_traits<PolygonMesh>::%face_descriptor`.
+ *  \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
  *
  *  \param seed_face a face of `pmesh` from which exploration starts to detect the connected component
            that contains it
  *  \param pmesh the polygon mesh
  *  \param out the output iterator that collects faces from the same connected component as `seed_face`
- *  \param np optional \ref pmp_namedparameters "Named Parameters" described below
+ *  \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{edge_is_constrained_map}  a property map containing the constrained-or-not status of each edge of `pmesh` \cgalParamEnd
+ *   \cgalParamNBegin{edge_is_constrained_map}
+ *     \cgalParamDescription{a property map containing the constrained-or-not status of each edge of `pmesh`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%edge_descriptor`
+ *                    as key type and `bool` as value type}
+ *     \cgalParamDefault{a constant property map returning `false` for any edge key}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  *
- *  \returns the output iterator.
+ * \returns the output iterator.
  *
  */
 template <typename PolygonMesh
@@ -132,8 +140,7 @@ connected_component(typename boost::grap
     internal::No_constraint<PolygonMesh>//default
   > ::type                                               EdgeConstraintMap;
   EdgeConstraintMap ecmap
-    = choose_parameter(get_parameter(np, internal_np::edge_is_constrained),
-                   internal::No_constraint<PolygonMesh>());
+    = choose_parameter<EdgeConstraintMap>(get_parameter(np, internal_np::edge_is_constrained));
 
   typedef typename boost::graph_traits<PolygonMesh>::face_descriptor face_descriptor;
   typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor halfedge_descriptor;
@@ -147,7 +154,7 @@ connected_component(typename boost::grap
       if (!already_processed.insert(seed_face).second) continue;
       *out++=seed_face;
       for(halfedge_descriptor hd :
-                    CGAL::halfedges_around_face(halfedge(seed_face, pmesh), pmesh) )
+                    halfedges_around_face(halfedge(seed_face, pmesh), pmesh) )
       {
         if(! get(ecmap, edge(hd, pmesh))){
           face_descriptor neighbor = face( opposite(hd, pmesh), pmesh );
@@ -173,25 +180,33 @@ connected_component(typename boost::grap
  * \ingroup keep_connected_components_grp
  *  computes for each face the index of the corresponding connected component.
  *
- *  A property map for `CGAL::face_index_t` must be either available as an internal property map
- *  to `pmesh` or provided as one of the \ref pmp_namedparameters "Named Parameters".
- *
  *  \tparam PolygonMesh a model of `FaceListGraph`
  *  \tparam FaceComponentMap a model of `WritablePropertyMap` with
         `boost::graph_traits<PolygonMesh>::%face_descriptor` as key type and
         `boost::graph_traits<PolygonMesh>::%faces_size_type` as value type.
- *  \tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+ *  \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 
  * \param pmesh the polygon mesh
  * \param fcm the property map with indices of components associated to faces in `pmesh`
- * \param np optional \ref pmp_namedparameters "Named Parameters" described below
+ * \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{edge_is_constrained_map} a property map containing the constrained-or-not status of each edge of `pmesh` \cgalParamEnd
- *    \cgalParamBegin{face_index_map} a property map containing the index of each face of `pmesh` \cgalParamEnd
+ *   \cgalParamNBegin{edge_is_constrained_map}
+ *     \cgalParamDescription{a property map containing the constrained-or-not status of each edge of `pmesh`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%edge_descriptor`
+ *                    as key type and `bool` as value type}
+ *     \cgalParamDefault{a constant property map returning `false` for any edge}
+ *   \cgalParamNEnd
+
+ *   \cgalParamNBegin{face_index_map}
+ *     \cgalParamDescription{a property map associating to each face of `pmesh` a unique index between `0` and `num_faces(pmesh) - 1`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%face_descriptor`
+ *                    as key type and `std::size_t` as value type}
+ *     \cgalParamDefault{an automatically indexed internal map}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  *
- *  \returns the number of connected components.
+ * \returns the number of connected components.
  */
 
 template <typename PolygonMesh
@@ -206,31 +221,52 @@ connected_components(const PolygonMesh&
   using parameters::choose_parameter;
   using parameters::get_parameter;
 
+  typedef boost::graph_traits<PolygonMesh> GT;
+  typedef typename GT::halfedge_descriptor halfedge_descriptor;
+  typedef typename GT::face_descriptor face_descriptor;
+
   typedef typename internal_np::Lookup_named_param_def <
     internal_np::edge_is_constrained_t,
     NamedParameters,
     internal::No_constraint<PolygonMesh>//default
   > ::type                                               EdgeConstraintMap;
+
   EdgeConstraintMap ecmap
-    = choose_parameter(get_parameter(np, internal_np::edge_is_constrained),
-                   internal::No_constraint<PolygonMesh>());
+    = choose_parameter<EdgeConstraintMap>(get_parameter(np, internal_np::edge_is_constrained));
+
+  typedef typename GetInitializedFaceIndexMap<PolygonMesh, NamedParameters>::const_type FaceIndexMap;
+  FaceIndexMap fimap = get_initialized_face_index_map(pmesh, np);
 
-  typedef Dual<PolygonMesh>                              Dual;
-  typedef boost::filtered_graph<Dual,
-    internal::No_border<PolygonMesh,EdgeConstraintMap> > FiniteDual;
-  Dual dual(pmesh);
-
-  FiniteDual finite_dual(dual,
-    internal::No_border<PolygonMesh, EdgeConstraintMap>(pmesh, ecmap));
-
-  typename GetFaceIndexMap<PolygonMesh, NamedParameters>::const_type
-    fimap = choose_parameter(get_parameter(np, internal_np::face_index),
-                         get_const_property_map(boost::face_index, pmesh));
-
-  return boost::connected_components(finite_dual,
-    fcm,
-    boost::vertex_index_map(fimap)
-  );
+  typename boost::property_traits<FaceComponentMap>::value_type i=0;
+  std::vector<bool> handled(num_faces(pmesh), false);
+  for (face_descriptor f : faces(pmesh))
+  {
+    if (handled[get(fimap,f)]) continue;
+    std::vector<face_descriptor> queue;
+    queue.push_back(f);
+    while(!queue.empty())
+    {
+      face_descriptor fq = queue.back();
+      queue.pop_back();
+      typename boost::property_traits<FaceIndexMap>::value_type  fq_id = get(fimap,fq);
+      if ( handled[fq_id]) continue;
+      handled[fq_id]=true;
+      put(fcm, fq, i);
+      for (halfedge_descriptor h : halfedges_around_face(halfedge(fq, pmesh), pmesh))
+      {
+        if ( get(ecmap, edge(h, pmesh)) ) continue;
+        halfedge_descriptor opp = opposite(h, pmesh);
+        face_descriptor fqo = face(opp, pmesh);
+        if ( fqo != GT::null_face() )
+        {
+          if ( !handled[get(fimap,fqo)] )
+            queue.push_back(fqo);
+        }
+      }
+    }
+    ++i;
+  }
+  return i;
 }
 
 template <typename PolygonMesh, typename FaceComponentMap>
@@ -238,12 +274,9 @@ typename boost::property_traits<FaceComp
 connected_components(const PolygonMesh& pmesh,
                      FaceComponentMap fcm)
 {
-
-  return CGAL::Polygon_mesh_processing::connected_components(pmesh, fcm,
-    CGAL::Polygon_mesh_processing::parameters::all_default());
+  return CGAL::Polygon_mesh_processing::connected_components(pmesh, fcm, CGAL::parameters::all_default());
 }
 
-
 template <typename PolygonMesh
         , typename ComponentRange
         , typename FaceComponentMap
@@ -253,6 +286,62 @@ void keep_connected_components(PolygonMe
                               , const FaceComponentMap& fcm
                               , const NamedParameters& np);
 
+namespace internal {
+
+//  /*!
+//  * \ingroup keep_connected_components_grp
+//  *  returns the number of connected components in the mesh.
+//  *
+//  *  A property map for `CGAL::face_index_t` must be either available as an internal property map
+//  *  to `pmesh` or provided as one of the \ref bgl_namedparameters "Named Parameters".
+//  *
+//  *  \tparam PolygonMesh a model of `FaceGraph`
+//  *  \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
+//  *
+//  *  \param pmesh the polygon mesh
+//  *  \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+//  *
+//  * \cgalNamedParamsBegin
+//  *   \cgalParamNBegin{edge_is_constrained_map}
+//  *     \cgalParamDescription{a property map containing the constrained-or-not status of each edge of `pmesh`}
+//  *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%edge_descriptor`
+//  *                    as key type and `bool` as value type}
+//  *     \cgalParamDefault{a constant property map returning `false` for any edge}
+//  *   \cgalParamNEnd
+//  *
+//  *   \cgalParamNBegin{face_index_map}
+//  *     \cgalParamDescription{a property map associating to each face of `pmesh` a unique index between `0` and `num_faces(pmesh) - 1`}
+//  *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%face_descriptor`
+//  *                    as key type and `std::size_t` as value type}
+//  *     \cgalParamDefault{an automatically indexed internal map}
+//  *   \cgalParamNEnd
+//  * \cgalNamedParamsEnd
+//  *
+//  * \returns the output iterator.
+//  *
+//  */
+template <typename PolygonMesh,
+          typename CGAL_PMP_NP_TEMPLATE_PARAMETERS>
+std::size_t number_of_connected_components(const PolygonMesh& pmesh,
+                                           const CGAL_PMP_NP_CLASS& np)
+{
+  typedef typename boost::graph_traits<PolygonMesh>::faces_size_type                faces_size_type;
+  typedef CGAL::dynamic_face_property_t<faces_size_type>                             Face_property_tag;
+  typedef typename boost::property_map<PolygonMesh, Face_property_tag >::const_type Patch_ids_map;
+
+  Patch_ids_map patch_ids_map = get(Face_property_tag(), pmesh);
+
+  return CGAL::Polygon_mesh_processing::connected_components(pmesh, patch_ids_map, np);
+}
+
+template <typename PolygonMesh>
+std::size_t number_of_connected_components(const PolygonMesh& pmesh)
+{
+  return internal::number_of_connected_components(pmesh, CGAL::parameters::all_default());
+}
+
+} // end namespace internal
+
 /*!
  * \ingroup keep_connected_components_grp
  *
@@ -262,29 +351,59 @@ void keep_connected_components(PolygonMe
  * By default, the size of a face is `1` (and thus the size of a connected component is the number
  * of faces it contains), but it is also possible to pass custom sizes, such as the area of the face.
  *
- * Property maps for `CGAL::face_index_t` and `CGAL::vertex_index_t`
- * must be either available as internal property maps
- * to `pmesh` or provided as \ref pmp_namedparameters "Named Parameters".
- *
  * \tparam PolygonMesh a model of `FaceListGraph` and `MutableFaceGraph`
- * \tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+ * \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
  *
  * \param pmesh the polygon mesh
- * \param nb_components_to_keep the number of components to be kept
- * \param np optional \ref pmp_namedparameters "Named Parameters", amongst those described below
+ * \param nb_components_to_keep the number of components to be kept. If this number is larger than
+ *                              the number of components in the mesh, all components are kept.
+ * \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{edge_is_constrained_map} a property map containing the constrained-or-not status of each edge of `pmesh` \cgalParamEnd
- *    \cgalParamBegin{face_index_map} a property map containing the index of each face of `pmesh` \cgalParamEnd
- *    \cgalParamBegin{vertex_index_map} a property map containing the index of each vertex of `pmesh` \cgalParamEnd
- *    \cgalParamBegin{face_size_map}
- *      a property map containing a size for each face of `pmesh`. The value type of this property map
- *      is chosen by the user, but must be constructible from `0` and support `operator+=()` and
- *      comparisons.
- *    \cgalParamEnd
+ *   \cgalParamNBegin{edge_is_constrained_map}
+ *     \cgalParamDescription{a property map containing the constrained-or-not status of each edge of `pmesh`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%edge_descriptor`
+ *                    as key type and `bool` as value type}
+ *     \cgalParamDefault{a constant property map returning `false` for any edge}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{vertex_index_map}
+ *     \cgalParamDescription{a property map associating to each vertex of `pmesh` a unique index between `0` and `num_vertices(pmesh) - 1`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+ *                    as key type and `std::size_t` as value type}
+ *     \cgalParamDefault{an automatically indexed internal map}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{face_index_map}
+ *     \cgalParamDescription{a property map associating to each face of `pmesh` a unique index between `0` and `num_faces(pmesh) - 1`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%face_descriptor`
+ *                    as key type and `std::size_t` as value type}
+ *     \cgalParamDefault{an automatically indexed internal map}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{face_size_map}
+ *     \cgalParamDescription{a property map associating to each face of `pmesh` a size}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%face_descriptor` as key type.
+ *                    The value type is chosen by the user, but must be constructible from `0` and support summation and comparisons.}
+ *     \cgalParamDefault{A constant property map returning `1` for any face}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{dry_run}
+ *     \cgalParamDescription{If set to `true`, the mesh will not be altered, but the number of components
+ *                           that would be removed is returned.}
+ *     \cgalParamType{Boolean}
+ *     \cgalParamDefault{`false`}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{output_iterator}
+ *     \cgalParamDescription{an output iterator to collect the faces that would be removed by the algorithm,
+ *                           when using the "dry run" mode (see parameter `dry_run`)}
+ *     \cgalParamType{a model of `OutputIterator` with value type `face_descriptor`}
+ *     \cgalParamDefault{unused}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  *
- *  \return the number of connected components removed (ignoring isolated vertices).
+ * \return the number of connected components removed (ignoring isolated vertices).
  */
 template <typename PolygonMesh,
           typename NamedParameters>
@@ -298,10 +417,8 @@ std::size_t keep_largest_connected_compo
   using parameters::choose_parameter;
   using parameters::get_parameter;
 
-  // FaceIndexMap
-  typedef typename GetFaceIndexMap<PM, NamedParameters>::type            FaceIndexMap;
-  FaceIndexMap fimap = choose_parameter(get_parameter(np, internal_np::face_index),
-                                        get_property_map(boost::face_index, pmesh));
+  typedef typename CGAL::GetInitializedFaceIndexMap<PolygonMesh, NamedParameters>::type FaceIndexMap;
+  FaceIndexMap fimap = CGAL::get_initialized_face_index_map(pmesh, np);
 
   // FaceSizeMap
   typedef typename internal_np::Lookup_named_param_def<internal_np::face_size_map_t,
@@ -313,6 +430,13 @@ std::size_t keep_largest_connected_compo
   FaceSizeMap face_size_pmap = choose_parameter(get_parameter(np, internal_np::face_size_map),
                                                 Constant_property_map<face_descriptor, std::size_t>(1));
 
+  const bool dry_run = choose_parameter(get_parameter(np, internal_np::dry_run), false);
+
+  typedef typename internal_np::Lookup_named_param_def<internal_np::output_iterator_t,
+                                                       NamedParameters,
+                                                       Emptyset_iterator>::type Output_iterator;
+  Output_iterator out = choose_parameter<Output_iterator>(get_parameter(np, internal_np::output_iterator));
+
   // vector_property_map
   boost::vector_property_map<std::size_t, FaceIndexMap> face_cc(fimap);
   std::size_t num = connected_components(pmesh, face_cc, np);
@@ -320,12 +444,13 @@ std::size_t keep_largest_connected_compo
   // Even if we do not want to keep anything we need to first
   // calculate the number of existing connected_components to get the
   // correct return value.
-  if(nb_components_to_keep == 0) {
+  if(nb_components_to_keep == 0)
+  {
     CGAL::clear(pmesh);
     return num;
   }
 
-  if((num == 1)|| (nb_components_to_keep > num) )
+  if(nb_components_to_keep >= num)
     return 0;
 
   std::vector<std::pair<std::size_t, Face_size> > component_size(num);
@@ -338,11 +463,25 @@ std::size_t keep_largest_connected_compo
 
   // we sort the range [0, num) by component size
   std::sort(component_size.begin(), component_size.end(), internal::MoreSecond());
-  std::vector<std::size_t> cc_to_keep;
-  for(std::size_t i=0; i<nb_components_to_keep; ++i)
-    cc_to_keep.push_back( component_size[i].first );
 
-  keep_connected_components(pmesh, cc_to_keep, face_cc, np);
+  if(dry_run)
+  {
+    std::vector<bool> is_to_be_removed(num, false);
+    for(std::size_t i=0; i<nb_components_to_keep; ++i)
+      is_to_be_removed[component_size[i].first] = true;
+
+    for(face_descriptor f : faces(pmesh))
+      if(is_to_be_removed[face_cc[f]])
+        *out++ = f;
+  }
+  else
+  {
+    std::vector<std::size_t> cc_to_keep;
+    for(std::size_t i=0; i<nb_components_to_keep; ++i)
+      cc_to_keep.push_back(component_size[i].first);
+
+    keep_connected_components(pmesh, cc_to_keep, face_cc, np);
+  }
 
   return num - nb_components_to_keep;
 }
@@ -364,33 +503,62 @@ std::size_t keep_largest_connected_compo
  * the size of a connected component is the number of faces it contains), but it is also possible
  * to pass custom sizes, such as the area of the face.
  *
- * Property maps for `CGAL::face_index_t` and `CGAL::vertex_index_t`
- * must be either available as internal property maps
- * to `pmesh` or provided as \ref pmp_namedparameters "Named Parameters".
- *
  * \tparam PolygonMesh a model of `FaceListGraph` and `MutableFaceGraph`
  * \tparam ThresholdValueType the type of the threshold value
- * \tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+ * \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
  *
  * \param pmesh the polygon mesh
  * \param threshold_value any connected component with a size (strictly) smaller than this value will be discarded
- * \param np optional \ref pmp_namedparameters "Named Parameters", amongst those described below
+ * \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{edge_is_constrained_map} a property map containing the constrained-or-not status of each edge of `pmesh` \cgalParamEnd
- *    \cgalParamBegin{face_index_map} a property map containing the index of each face of `pmesh` \cgalParamEnd
- *    \cgalParamBegin{vertex_index_map} a property map containing the index of each vertex of `pmesh` \cgalParamEnd
- *    \cgalParamBegin{face_size_map}
- *      a property map containing a size for each face of `pmesh`. The value type of this property map
- *      is chosen by the user, but must be constructible from `0` and support `operator+=()` and
- *      comparisons.
- *    \cgalParamEnd
+ *   \cgalParamNBegin{edge_is_constrained_map}
+ *     \cgalParamDescription{a property map containing the constrained-or-not status of each edge of `pmesh`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%edge_descriptor`
+ *                    as key type and `bool` as value type}
+ *     \cgalParamDefault{a constant property map returning `false` for any edge}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{vertex_index_map}
+ *     \cgalParamDescription{a property map associating to each vertex of `pmesh` a unique index between `0` and `num_vertices(pmesh) - 1`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+ *                    as key type and `std::size_t` as value type}
+ *     \cgalParamDefault{an automatically indexed internal map}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{face_index_map}
+ *     \cgalParamDescription{a property map associating to each face of `pmesh` a unique index between `0` and `num_faces(pmesh) - 1`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%face_descriptor`
+ *                    as key type and `std::size_t` as value type}
+ *     \cgalParamDefault{an automatically indexed internal map}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{face_size_map}
+ *     \cgalParamDescription{a property map associating to each face of `pmesh` a size}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%face_descriptor` as key type.
+ *                    The value type is chosen by the user, but must be constructible from `0` and support summation and comparisons.}
+ *     \cgalParamDefault{A constant property map returning `1` for any face}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{dry_run}
+ *     \cgalParamDescription{If set to `true`, the mesh will not be altered, but the number of components
+ *                           that would be removed is returned.}
+ *     \cgalParamType{Boolean}
+ *     \cgalParamDefault{`false`}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{output_iterator}
+ *     \cgalParamDescription{an output iterator to collect the faces that would be removed by the algorithm,
+ *                           when using the "dry run" mode (see parameter `dry_run`)}
+ *     \cgalParamType{a model of `OutputIterator` with value type `face_descriptor`}
+ *     \cgalParamDefault{unused}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  *
  * \pre If a face size property map is passed by the user, `ThresholdValueType` must be the same
  *      type as the value type of the property map. Otherwise, `ThresholdValueType` must be `std::size_t`.
  *
- *  \return the number of connected components removed (ignoring isolated vertices).
+ * \return the number of connected components removed (ignoring isolated vertices).
  */
 template <typename PolygonMesh,
           typename ThresholdValueType,
@@ -405,47 +573,65 @@ std::size_t keep_large_connected_compone
   using parameters::choose_parameter;
   using parameters::get_parameter;
 
-  // FaceIndexMap
-  typedef typename GetFaceIndexMap<PM, NamedParameters>::type             FaceIndexMap;
-  FaceIndexMap fim = choose_parameter(get_parameter(np, internal_np::face_index),
-                                      get_property_map(boost::face_index, pmesh));
+  typedef typename CGAL::GetInitializedFaceIndexMap<PolygonMesh, NamedParameters>::type FaceIndexMap;
+  FaceIndexMap fim = CGAL::get_initialized_face_index_map(pmesh, np);
 
   typedef typename internal_np::Lookup_named_param_def<internal_np::face_size_map_t,
-                                                 NamedParameters,
-                                                 Constant_property_map<face_descriptor, std::size_t> // default
-                                                >::type                   FaceSizeMap;
+                                                       NamedParameters,
+                                                       Constant_property_map<face_descriptor, std::size_t> // default
+                                                      >::type             FaceSizeMap;
   typedef typename boost::property_traits<FaceSizeMap>::value_type        Face_size;
 
   CGAL_static_assertion((std::is_convertible<ThresholdValueType, Face_size>::value));
 
+  typedef typename internal_np::Lookup_named_param_def<internal_np::output_iterator_t,
+                                                       NamedParameters,
+                                                       Emptyset_iterator>::type Output_iterator;
+
   FaceSizeMap face_size_pmap = choose_parameter(get_parameter(np, internal_np::face_size_map),
-                                           Constant_property_map<face_descriptor, std::size_t>(1));
+                                                Constant_property_map<face_descriptor, std::size_t>(1));
+  const bool dry_run = choose_parameter(get_parameter(np, internal_np::dry_run), false);
+  Output_iterator out = choose_parameter<Output_iterator>(get_parameter(np, internal_np::output_iterator));
 
   // vector_property_map
   boost::vector_property_map<std::size_t, FaceIndexMap> face_cc(fim);
   std::size_t num = connected_components(pmesh, face_cc, np);
-  std::vector<Face_size> component_size(num);
-
-  for(std::size_t i=0; i<num; ++i)
-    component_size[i] = Face_size(0);
+  std::vector<Face_size> component_size(num, 0);
 
   for(face_descriptor f : faces(pmesh))
     component_size[face_cc[f]] += get(face_size_pmap, f);
 
   const Face_size thresh = threshold_value;
+  std::vector<bool> is_to_be_kept(num, false);
+  std::size_t res = 0;
 
-  std::vector<std::size_t> cc_to_keep;
   for(std::size_t i=0; i<num; ++i)
   {
     if(component_size[i] >= thresh)
-      cc_to_keep.push_back(i);
+    {
+      is_to_be_kept[i] = true;
+      ++res;
+    }
   }
 
-  keep_connected_components(pmesh, cc_to_keep, face_cc, np);
+  if(dry_run)
+  {
+    for(face_descriptor f : faces(pmesh))
+      if(!is_to_be_kept[face_cc[f]])
+        *out++ = f;
+  }
+  else
+  {
+    std::vector<std::size_t> ccs_to_keep;
+    for(std::size_t i=0; i<num; ++i)
+      if(is_to_be_kept[i])
+        ccs_to_keep.push_back(i);
 
-  return num - cc_to_keep.size();
-}
+    keep_connected_components(pmesh, ccs_to_keep, face_cc, np);
+  }
 
+  return num - res;
+}
 
 template <typename PolygonMesh>
 std::size_t keep_large_connected_components(PolygonMesh& pmesh,
@@ -467,7 +653,6 @@ void keep_or_remove_connected_components
                                         , bool  keep
                                         , const NamedParameters& np)
 {
-  typedef PolygonMesh PM;
   using parameters::choose_parameter;
   using parameters::get_parameter;
 
@@ -479,10 +664,8 @@ void keep_or_remove_connected_components
   typedef typename boost::graph_traits<PolygonMesh>::edge_descriptor   edge_descriptor;
   typedef typename boost::graph_traits<PolygonMesh>::edge_iterator     edge_iterator;
 
-  //VertexIndexMap
-  typedef typename GetVertexIndexMap<PM, NamedParameters>::type VertexIndexMap;
-  VertexIndexMap vim = choose_parameter(get_parameter(np, internal_np::vertex_index),
-                                    get_const_property_map(boost::vertex_index, pmesh));
+  typedef typename GetInitializedVertexIndexMap<PolygonMesh, NamedParameters>::type VertexIndexMap;
+  VertexIndexMap vim = get_initialized_vertex_index_map(pmesh, np);
 
   std::set<std::size_t> cc_to_keep;
   for(std::size_t i : components_to_keep)
@@ -601,25 +784,26 @@ void keep_or_remove_connected_components
 * \note If the removal of the connected components makes `pmesh` a non-manifold surface,
 * then the behavior of this function is undefined.
 *
-* Property maps for `CGAL::vertex_index_t`
-* must be either available as internal property map
-* to `pmesh` or provided as \ref pmp_namedparameters "Named Parameters".
-*
 * \tparam PolygonMesh a model of `FaceListGraph` and `MutableFaceGraph`
-* \tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
 * \tparam ComponentRange a range of ids convertible to `std::size`
 * \tparam FaceComponentMap a model of `ReadWritePropertyMap` with
 *         `boost::graph_traits<PolygonMesh>::%face_descriptor` as key type and
 *         `boost::graph_traits<PolygonMesh>::%faces_size_type` as value type.
+* \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 *
 * \param components_to_keep the range of ids of connected components to keep
 * \param pmesh the polygon mesh
 * \param fcm the property map with indices of components associated to faces in `pmesh`.
 *        After calling this function, the values of `fcm` are undefined.
-* \param np optional \ref pmp_namedparameters "Named Parameters" described below
+* \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 *
 * \cgalNamedParamsBegin
-*    \cgalParamBegin{vertex_index_map} a property map containing the index of each vertex of `pmesh` \cgalParamEnd
+*   \cgalParamNBegin{vertex_index_map}
+*     \cgalParamDescription{a property map associating to each vertex of `pmesh` a unique index between `0` and `num_vertices(pmesh) - 1`}
+*     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+*                    as key type and `std::size_t` as value type}
+*     \cgalParamDefault{an automatically indexed internal map}
+*   \cgalParamNEnd
 * \cgalNamedParamsEnd
 *
 */
@@ -637,33 +821,33 @@ void keep_connected_components(PolygonMe
 
 /*!
 * \ingroup keep_connected_components_grp
-* Removes in `pmesh` the connected components designated by theirs ids
+* removes in `pmesh` the connected components designated by theirs ids
 * in `components_to_remove` as well as all isolated vertices.
 * The connected component id of a face is given by `fcm`.
 *
 * \note If the removal of the connected components makes `pmesh` a non-manifold surface,
 * then the behavior of this function is undefined.
 *
-* Property maps for `CGAL::vertex_index_t`
-* must be either available as internal property map
-* to `pmesh` or provided as \ref pmp_namedparameters "Named Parameters".
-*
-*
 * \tparam PolygonMesh a model of `FaceListGraph` and `MutableFaceGraph`
-* \tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
 * \tparam ComponentRange a range of ids convertible to `std::size`
 * \tparam FaceComponentMap a model of `ReadWritePropertyMap` with
 *         `boost::graph_traits<PolygonMesh>::%face_descriptor` as key type and
 *         `boost::graph_traits<PolygonMesh>::%faces_size_type` as value type.
+* \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 *
 * \param components_to_remove the range of ids of connected components to remove
 * \param pmesh the polygon mesh
 * \param fcm the property map with indices of components associated to faces in `pmesh`.
 *        After calling this function, the values of `fcm` are undefined.
-* \param np optional \ref pmp_namedparameters "Named Parameters" described below
+* \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 *
 * \cgalNamedParamsBegin
-*    \cgalParamBegin{vertex_index_map} a property map containing the index of each vertex of `pmesh` \cgalParamEnd
+*   \cgalParamNBegin{vertex_index_map}
+*     \cgalParamDescription{a property map associating to each vertex of `pmesh` a unique index between `0` and `num_vertices(pmesh) - 1`}
+*     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+*                    as key type and `std::size_t` as value type}
+*     \cgalParamDefault{an automatically indexed internal map}
+*   \cgalParamNEnd
 * \cgalNamedParamsEnd
 *
 */
@@ -685,26 +869,39 @@ void remove_connected_components(Polygon
 *  keeps the connected components not designated by the faces in `components_to_remove`,
 *  and removes the other connected components and all isolated vertices.
 *
-* Property maps for `CGAL::face_index_t` and `CGAL::vertex_index_t`
-* must be either available as internal property maps
-* to `pmesh` or provided as \ref pmp_namedparameters "Named Parameters".
-*
 * \note If the removal of the connected components makes `pmesh` a non-manifold surface,
 * then the behavior of this function is undefined.
 *
 * \tparam PolygonMesh a model of `FaceListGraph` and `MutableFaceGraph`
-* \tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+* \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 * \tparam FaceRange a range of `boost::graph_traits<PolygonMesh>::%face_descriptor`
 *         indicating the connected components to be removed.
 *
 * \param components_to_remove a face range, including one face or more on each component to be removed
 * \param pmesh the polygon mesh
-* \param np optional \ref pmp_namedparameters "Named Parameters", amongst those described below
+* \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 *
 * \cgalNamedParamsBegin
-*    \cgalParamBegin{edge_is_constrained_map} a property map containing the constrained-or-not status of each edge of `pmesh` \cgalParamEnd
-*    \cgalParamBegin{face_index_map} a property map containing the index of each face of `pmesh` \cgalParamEnd
-*    \cgalParamBegin{vertex_index_map} a property map containing the index of each vertex of `pmesh` \cgalParamEnd
+*   \cgalParamNBegin{edge_is_constrained_map}
+*     \cgalParamDescription{a property map containing the constrained-or-not status of each edge of `pmesh`}
+*     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%edge_descriptor`
+*                    as key type and `bool` as value type}
+*     \cgalParamDefault{a constant property map returning `false` for any edge}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{vertex_index_map}
+*     \cgalParamDescription{a property map associating to each vertex of `pmesh` a unique index between `0` and `num_vertices(pmesh) - 1`}
+*     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+*                    as key type and `std::size_t` as value type}
+*     \cgalParamDefault{an automatically indexed internal map}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{face_index_map}
+*     \cgalParamDescription{a property map associating to each face of `pmesh` a unique index between `0` and `num_faces(pmesh) - 1`}
+*     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%face_descriptor`
+*                    as key type and `std::size_t` as value type}
+*     \cgalParamDefault{an automatically indexed internal map}
+*   \cgalParamNEnd
 * \cgalNamedParamsEnd
 *
 */
@@ -715,20 +912,19 @@ void remove_connected_components(Polygon
                                 , const FaceRange& components_to_remove
                                 , const CGAL_PMP_NP_CLASS& np)
 {
-  if (components_to_remove.empty()) return;
-  typedef PolygonMesh PM;
-  typedef typename boost::graph_traits<PM>::face_descriptor face_descriptor;
   using parameters::choose_parameter;
   using parameters::get_parameter;
 
-  //FaceIndexMap
-  typedef typename GetFaceIndexMap<PM, CGAL_PMP_NP_CLASS>::type FaceIndexMap;
-  FaceIndexMap fim = choose_parameter(get_parameter(np, internal_np::face_index),
-                                  get_property_map(boost::face_index, pmesh));
+  if (components_to_remove.empty())
+    return;
 
-  //vector_property_map
-  boost::vector_property_map<std::size_t, FaceIndexMap> face_cc(fim);
+  typedef PolygonMesh PM;
+  typedef typename boost::graph_traits<PM>::face_descriptor face_descriptor;
+
+  typedef typename CGAL::GetInitializedFaceIndexMap<PolygonMesh, CGAL_PMP_NP_CLASS>::type FaceIndexMap;
+  FaceIndexMap fim = CGAL::get_initialized_face_index_map(pmesh, np);
 
+  boost::vector_property_map<std::size_t, FaceIndexMap> face_cc(fim);
   connected_components(pmesh, face_cc, np);
 
   std::vector<std::size_t> cc_to_remove;
@@ -743,26 +939,39 @@ void remove_connected_components(Polygon
 *  keeps the connected components designated by the faces in `components_to_keep`,
 *  and removes the other connected components and all isolated vertices.
 *
-* Property maps for `CGAL::face_index_t` and `CGAL::vertex_index_t`
-* must be either available as internal property maps
-* to `pmesh` or provided as \ref pmp_namedparameters "Named Parameters".
-*
 * \note If the removal of the connected components makes `pmesh` a non-manifold surface,
 * then the behavior of this function is undefined.
 *
 * \tparam PolygonMesh a model of `FaceListGraph` and `MutableFaceGraph`
-* \tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+* \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 * \tparam FaceRange a range of `boost::graph_traits<PolygonMesh>::%face_descriptor`
 *         indicating the connected components to be kept.
 *
 * \param pmesh the polygon mesh
 * \param components_to_keep a face range, including one face or more on each component to be kept
-* \param np optional \ref pmp_namedparameters "Named Parameters", amongst those described below
+* \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 *
 * \cgalNamedParamsBegin
-*    \cgalParamBegin{edge_is_constrained_map} a property map containing the constrained-or-not status of each edge of `pmesh` \cgalParamEnd
-*    \cgalParamBegin{face_index_map} a property map containing the index of each face of `pmesh` \cgalParamEnd
-*    \cgalParamBegin{vertex_index_map} a property map containing the index of each vertex of `pmesh` \cgalParamEnd
+*   \cgalParamNBegin{edge_is_constrained_map}
+*     \cgalParamDescription{a property map containing the constrained-or-not status of each edge of `pmesh`}
+*     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%edge_descriptor`
+*                    as key type and `bool` as value type}
+*     \cgalParamDefault{a constant property map returning `false` for any edge}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{vertex_index_map}
+*     \cgalParamDescription{a property map associating to each vertex of `pmesh` a unique index between `0` and `num_vertices(pmesh) - 1`}
+*     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+*                    as key type and `std::size_t` as value type}
+*     \cgalParamDefault{an automatically indexed internal map}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{face_index_map}
+*     \cgalParamDescription{a property map associating to each face of `pmesh` a unique index between `0` and `num_faces(pmesh) - 1`}
+*     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%face_descriptor`
+*                    as key type and `std::size_t` as value type}
+*     \cgalParamDefault{an automatically indexed internal map}
+*   \cgalParamNEnd
 * \cgalNamedParamsEnd
 *
 */
@@ -779,14 +988,10 @@ void keep_connected_components(PolygonMe
   using parameters::choose_parameter;
   using parameters::get_parameter;
 
-  //FaceIndexMap
-  typedef typename GetFaceIndexMap<PM, CGAL_PMP_NP_CLASS>::type FaceIndexMap;
-  FaceIndexMap fim = choose_parameter(get_parameter(np, internal_np::face_index),
-                                  get_property_map(boost::face_index, pmesh));
+  typedef typename CGAL::GetInitializedFaceIndexMap<PolygonMesh, CGAL_PMP_NP_CLASS>::type FaceIndexMap;
+  FaceIndexMap fim = CGAL::get_initialized_face_index_map(pmesh, np);
 
-  //vector_property_map
   boost::vector_property_map<std::size_t, FaceIndexMap> face_cc(fim);
-
   connected_components(pmesh, face_cc, np);
 
   std::vector<std::size_t> cc_to_keep;
@@ -836,8 +1041,159 @@ void keep_connected_components(PolygonMe
     CGAL::Polygon_mesh_processing::parameters::all_default());
 }
 
-} // namespace Polygon_mesh_processing
+namespace internal {
 
+template <typename G>
+struct No_mark
+{
+  friend bool get(No_mark<G>, typename boost::graph_traits<G>::edge_descriptor) { return false; }
+  friend void put(No_mark<G>, typename boost::graph_traits<G>::edge_descriptor, bool) { }
+};
+
+template < class PolygonMesh, class PolygonMeshRange,
+           class FIMap, class VIMap,
+           class HIMap, class Ecm, class NamedParameters >
+void split_connected_components_impl(FIMap fim,
+                                     HIMap him,
+                                     VIMap vim,
+                                     Ecm ecm,
+                                     PolygonMeshRange& range,
+                                     const PolygonMesh& tm,
+                                     const NamedParameters& np)
+{
+  typedef typename boost::graph_traits<PolygonMesh>::faces_size_type  faces_size_type;
+  typedef typename internal_np::Lookup_named_param_def <
+      internal_np::face_patch_t,
+      NamedParameters,
+      typename boost::template property_map<
+      PolygonMesh, CGAL::dynamic_face_property_t<faces_size_type > >::const_type> ::type
+      Fpm;
+
+  using parameters::choose_parameter;
+  using parameters::get_parameter;
+  using parameters::is_default_parameter;
+
+  Fpm pidmap = choose_parameter(get_parameter(np, internal_np::face_patch),
+                                get(CGAL::dynamic_face_property_t<faces_size_type>(), tm));
+
+  faces_size_type nb_patches = 0;
+  if(is_default_parameter(get_parameter(np, internal_np::face_patch)))
+  {
+    nb_patches = CGAL::Polygon_mesh_processing::connected_components(
+          tm, pidmap, CGAL::parameters::face_index_map(fim)
+          .edge_is_constrained_map(ecm));
+  }
+  else
+  {
+    for(const auto& f : faces(tm))
+    {
+      faces_size_type patch_id = get(pidmap, f);
+      if(patch_id > nb_patches)
+        nb_patches = patch_id;
+    }
+    nb_patches+=1;
+  }
+  CGAL::internal::reserve(range, nb_patches);
+  for(faces_size_type i=0; i<nb_patches; ++i)
+  {
+    CGAL::Face_filtered_graph<PolygonMesh, FIMap, VIMap, HIMap>
+        filter_graph(tm, i, pidmap, CGAL::parameters::face_index_map(fim)
+                                                     .halfedge_index_map(him)
+                                                     .vertex_index_map(vim));
+    range.push_back(PolygonMesh());
+    PolygonMesh& new_graph = range.back();
+    CGAL::copy_face_graph(filter_graph, new_graph);
+  }
+}
+
+}//internal
+
+/*!
+ * \ingroup keep_connected_components_grp
+ * identifies the connected components of `pmesh` and pushes back a new `PolygonMesh` for each connected component in `cc_meshes`.
+ *
+ *  \tparam PolygonMesh a model of `FaceListGraph`
+ *  \tparam PolygonMeshRange a model of `SequenceContainer` with `PolygonMesh` as value type.
+ *
+ *  \tparam NamedParameters a sequence of Named Parameters
+ *
+ * \param pmesh the polygon mesh
+ * \param cc_meshes container that is filled with the extracted connected components.
+ * \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+ *
+ * \cgalNamedParamsBegin
+ *   \cgalParamNBegin{edge_is_constrained_map}
+ *     \cgalParamDescription{a property map containing the constrained-or-not status of each edge of `pmesh`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%edge_descriptor`
+ *                    as key type and `bool` as value type}
+ *     \cgalParamDefault{a constant property map returning `false` for any edge}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{vertex_index_map}
+ *     \cgalParamDescription{a property map associating to each vertex of `pmesh` a unique index between `0` and `num_vertices(pmesh) - 1`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+ *                    as key type and `std::size_t` as value type}
+ *     \cgalParamDefault{an automatically indexed internal map}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{halfedge_index_map}
+ *     \cgalParamDescription{a property map associating to each halfedge of `pmesh` a unique index between `0` and `num_halfedges(pmesh) - 1`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%halfedge_descriptor`
+ *                    as key type and `std::size_t` as value type}
+ *     \cgalParamDefault{an automatically indexed internal map}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{face_index_map}
+ *     \cgalParamDescription{a property map associating to each face of `pmesh` a unique index between `0` and `num_faces(pmesh) - 1`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%face_descriptor`
+ *                    as key type and `std::size_t` as value type}
+ *     \cgalParamDefault{an automatically indexed internal map}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{face_patch_map}
+ *     \cgalParamDescription{a property map with the patch id's associated to the faces of `pmesh`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%face_descriptor`
+ *                    as key type and the desired property, model of `CopyConstructible` as value type.}
+ *     \cgalParamDefault{a default property map where each face is associated with the ID of
+ *                       the connected component it belongs to. Connected components are
+ *                       computed with respect to the constrained edges listed in the property map
+ *                       `edge_is_constrained_map`}
+ *     \cgalParamExtra{The map is updated during the remeshing process while new faces are created.}
+ *   \cgalParamNEnd
+ * \cgalNamedParamsEnd
+ *
+ */
+template <class PolygonMesh, class PolygonMeshRange, class NamedParameters>
+void split_connected_components(const PolygonMesh& pmesh,
+                                PolygonMeshRange& cc_meshes,
+                                const NamedParameters& np)
+{
+  typedef typename internal_np::Lookup_named_param_def <
+    internal_np::edge_is_constrained_t,
+    NamedParameters,
+    internal::No_mark<PolygonMesh>//default
+  > ::type Ecm;
+
+  using parameters::choose_parameter;
+  using parameters::get_parameter;
+
+  Ecm ecm = choose_parameter(get_parameter(np, internal_np::edge_is_constrained),
+                             internal::No_mark<PolygonMesh>());
+
+  internal::split_connected_components_impl(CGAL::get_initialized_face_index_map(pmesh, np),
+                                            CGAL::get_initialized_halfedge_index_map(pmesh, np),
+                                            CGAL::get_initialized_vertex_index_map(pmesh, np),
+                                            ecm, cc_meshes, pmesh, np);
+}
+
+template <class PolygonMesh, class PolygonMeshRange>
+void split_connected_components(const PolygonMesh& pmesh,
+                                PolygonMeshRange& cc_meshes)
+{
+  split_connected_components(pmesh, cc_meshes, parameters::all_default());
+}
+
+} // namespace Polygon_mesh_processing
 } // namespace CGAL
 
 #include <CGAL/enable_warnings.h>
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/corefinement.h 5.1-1/include/CGAL/Polygon_mesh_processing/corefinement.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/corefinement.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/corefinement.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/corefinement.h $
-// $Id: corefinement.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/corefinement.h $
+// $Id: corefinement.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -17,11 +17,12 @@
 
 #include <CGAL/disable_warnings.h>
 
+#include <CGAL/boost/graph/copy_face_graph.h>
+#include <CGAL/boost/graph/named_params_helper.h>
 #include <CGAL/Polygon_mesh_processing/intersection.h>
 #include <CGAL/Polygon_mesh_processing/internal/Corefinement/Visitor.h>
 #include <CGAL/Polygon_mesh_processing/internal/Corefinement/Face_graph_output_builder.h>
 #include <CGAL/Polygon_mesh_processing/internal/Corefinement/Output_builder_for_autorefinement.h>
-#include <CGAL/boost/graph/copy_face_graph.h>
 #include <CGAL/iterator.h>
 
 namespace CGAL {
@@ -34,95 +35,6 @@ using Polygon_mesh_processing::Corefinem
 
 namespace Polygon_mesh_processing {
 
-namespace internal {
-
-template <class Kernel, class TriangleMesh, class VD, class Fid_map, class Vpm>
-bool recursive_does_bound_a_volume(const TriangleMesh& tm,
-                                         Vpm& vpm,
-                                         Fid_map& fid_map,
-                                         const std::vector<VD>& xtrm_vertices,
-                                         boost::dynamic_bitset<>& cc_handled,
-                                         const std::vector<std::size_t>& face_cc,
-                                         std::size_t xtrm_cc_id,
-                                         bool is_parent_outward_oriented)
-{
-  typedef boost::graph_traits<TriangleMesh> GT;
-  typedef typename GT::face_descriptor face_descriptor;
-  typedef Side_of_triangle_mesh<TriangleMesh, Kernel, Vpm> Side_of_tm;
-// first check that the orientation of the current cc is consistant with its
-// parent cc containing it
-  bool new_is_parent_outward_oriented = internal::is_outward_oriented(
-         xtrm_vertices[xtrm_cc_id], tm, parameters::vertex_point_map(vpm));
-  if (new_is_parent_outward_oriented==is_parent_outward_oriented)
-    return false;
-  cc_handled.set(xtrm_cc_id);
-
-  std::size_t nb_cc = cc_handled.size();
-
-// get all cc that are inside xtrm_cc_id
-  std::vector<face_descriptor> cc_faces;
-  for(face_descriptor fd : faces(tm))
-  {
-    if(face_cc[get(fid_map, fd)]==xtrm_cc_id)
-      cc_faces.push_back(fd);
-  }
-
-  typename Side_of_tm::AABB_tree aabb_tree(cc_faces.begin(), cc_faces.end(),
-                                           tm, vpm);
-  Side_of_tm side_of_cc(aabb_tree);
-
-  std::vector<std::size_t> cc_inside;
-  for(std::size_t id=0; id<nb_cc; ++id)
-  {
-    if (cc_handled.test(id)) continue;
-    if (side_of_cc(get(vpm,xtrm_vertices[id]))==ON_BOUNDED_SIDE)
-      cc_inside.push_back(id);
-  }
-
-// check whether we need another recursion for cc inside xtrm_cc_id
-  if (!cc_inside.empty())
-  {
-    std::size_t new_xtrm_cc_id = cc_inside.front();
-    boost::dynamic_bitset<> new_cc_handled(nb_cc,0);
-    new_cc_handled.set();
-    new_cc_handled.reset(new_xtrm_cc_id);
-    cc_handled.set(new_xtrm_cc_id);
-
-    std::size_t nb_candidates = cc_inside.size();
-    for (std::size_t i=1;i<nb_candidates;++i)
-    {
-      std::size_t candidate = cc_inside[i];
-      if(get(vpm,xtrm_vertices[candidate]).z() >
-         get(vpm,xtrm_vertices[new_xtrm_cc_id]).z()) new_xtrm_cc_id=candidate;
-      new_cc_handled.reset(candidate);
-      cc_handled.set(candidate);
-    }
-
-    if ( !internal::recursive_does_bound_a_volume<Kernel>(
-           tm, vpm, fid_map, xtrm_vertices, new_cc_handled, face_cc,
-           new_xtrm_cc_id, new_is_parent_outward_oriented) ) return false;
-  }
-
-// now explore remaining cc included in the same cc as xtrm_cc_id
-  boost::dynamic_bitset<> cc_not_handled = ~cc_handled;
-  std::size_t new_xtrm_cc_id = cc_not_handled.find_first();
-  if (new_xtrm_cc_id == cc_not_handled.npos) return true;
-
-  for (std::size_t candidate = cc_not_handled.find_next(new_xtrm_cc_id);
-                   candidate < cc_not_handled.npos;
-                   candidate = cc_not_handled.find_next(candidate))
-  {
-     if(get(vpm,xtrm_vertices[candidate]).z() > get(vpm,xtrm_vertices[new_xtrm_cc_id]).z())
-        new_xtrm_cc_id = candidate;
-  }
-
-  return internal::recursive_does_bound_a_volume<Kernel>(
-            tm, vpm, fid_map, xtrm_vertices, cc_handled, face_cc,
-            new_xtrm_cc_id, is_parent_outward_oriented);
-}
-
-} //end of namespace internal
-
 namespace Corefinement
 {
 /** \ingroup PMP_corefinement_grp
@@ -143,101 +55,6 @@ enum Boolean_operation_type {UNION = 0,
 #endif
 }
 
-/** \ingroup PMP_corefinement_grp
- *
- * indicates if `tm` bounds a volume.
- * See \ref coref_def_subsec for details.
- *
- * @tparam TriangleMesh a model of `MutableFaceGraph`, `HalfedgeListGraph` and `FaceListGraph`.
- * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
- *
- * @param tm a closed triangulated surface mesh
- * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
- *
- * @pre `CGAL::is_closed(tm)`
- *
- * \cgalNamedParamsBegin
- *   \cgalParamBegin{vertex_point_map}
- *     the property map with the points associated to the vertices of `tm`.
- *     If this parameter is omitted, an internal property map for
- *     `CGAL::vertex_point_t` must be available in `TriangleMesh`
- *   \cgalParamEnd
- *   \cgalParamBegin{face_index_map}
- *     a property map containing the index of each face of `tm`.
- *   \cgalParamEnd
- * \cgalNamedParamsEnd
- *
- * \see `CGAL::Polygon_mesh_processing::orient_to_bound_a_volume()`
- */
-template <class TriangleMesh, class NamedParameters>
-bool does_bound_a_volume(const TriangleMesh& tm, const NamedParameters& np)
-{
-  typedef boost::graph_traits<TriangleMesh> GT;
-  typedef typename GT::vertex_descriptor vertex_descriptor;
-  typedef typename GetVertexPointMap<TriangleMesh,
-                                     NamedParameters>::const_type Vpm;
-  typedef typename GetFaceIndexMap<TriangleMesh,
-                                   NamedParameters>::const_type Fid_map;
-  typedef typename Kernel_traits<
-    typename boost::property_traits<Vpm>::value_type >::Kernel Kernel;
-
-  if (!is_closed(tm)) return false;
-  if (!is_triangle_mesh(tm)) return false;
-
-  Vpm vpm = parameters::choose_parameter(parameters::get_parameter(np, internal_np::vertex_point),
-                                get_const_property_map(boost::vertex_point, tm));
-
-  Fid_map fid_map = parameters::choose_parameter(parameters::get_parameter(np, internal_np::face_index),
-                                        get_const_property_map(boost::face_index, tm));
-
-  std::vector<std::size_t> face_cc(num_faces(tm), std::size_t(-1));
-
-  // set the connected component id of each face
-  std::size_t nb_cc = connected_components(tm,
-                                bind_property_maps(fid_map,make_property_map(face_cc)),
-                                parameters::face_index_map(fid_map));
-
-  if (nb_cc == 1)
-    return true;
-
-  boost::dynamic_bitset<> cc_handled(nb_cc, 0);
-
-  // extract a vertex with max z coordinate for each connected component
-  std::vector<vertex_descriptor> xtrm_vertices(nb_cc, GT::null_vertex());
-  for(vertex_descriptor vd : vertices(tm))
-  {
-    std::size_t cc_id = face_cc[get(fid_map, face(halfedge(vd, tm), tm))];
-    if (xtrm_vertices[cc_id]==GT::null_vertex())
-      xtrm_vertices[cc_id]=vd;
-    else
-      if (get(vpm, vd).z()>get(vpm,xtrm_vertices[cc_id]).z())
-        xtrm_vertices[cc_id]=vd;
-  }
-
-  //extract a vertex with max z amongst all components
-  std::size_t xtrm_cc_id=0;
-  for(std::size_t id=1; id<nb_cc; ++id)
-    if (get(vpm, xtrm_vertices[id]).z()>get(vpm,xtrm_vertices[xtrm_cc_id]).z())
-      xtrm_cc_id=id;
-
-  bool is_parent_outward_oriented =
-    !internal::is_outward_oriented(xtrm_vertices[xtrm_cc_id], tm, np);
-
-  return internal::recursive_does_bound_a_volume<Kernel>(tm, vpm, fid_map,
-                                                         xtrm_vertices,
-                                                         cc_handled,
-                                                         face_cc,
-                                                         xtrm_cc_id,
-                                                         is_parent_outward_oriented);
-}
-
-/// \cond SKIP_IN_MANUAL
-template <class TriangleMesh>
-bool does_bound_a_volume(const TriangleMesh& tm)
-{
-  return does_bound_a_volume(tm, parameters::all_default());
-}
-/// \endcond
 
 #define CGAL_COREF_SET_OUTPUT_EDGE_MARK_MAP(I) \
   typedef typename internal_np::Lookup_named_param_def < \
@@ -246,9 +63,7 @@ bool does_bound_a_volume(const TriangleM
     Corefinement::No_mark<TriangleMesh> \
   > ::type Ecm_out_##I; \
     Ecm_out_##I ecm_out_##I = \
-      parameters::choose_parameter( parameters::get_parameter(std::get<I>(nps_out), internal_np::edge_is_constrained),  \
-                                     Corefinement::No_mark<TriangleMesh>() );
-
+      parameters::choose_parameter<Ecm_out_##I>(parameters::get_parameter(std::get<I>(nps_out), internal_np::edge_is_constrained));
 
 /**
   * \ingroup PMP_corefinement_grp
@@ -274,56 +89,83 @@ bool does_bound_a_volume(const TriangleM
   * \pre \link CGAL::Polygon_mesh_processing::does_bound_a_volume() `CGAL::Polygon_mesh_processing::does_bound_a_volume(tm2)` \endlink
   *
   * @tparam TriangleMesh a model of `MutableFaceGraph`, `HalfedgeListGraph` and `FaceListGraph`
-  * @tparam NamedParameters1 a sequence of \ref pmp_namedparameters "Named Parameters"
-  * @tparam NamedParameters2 a sequence of \ref pmp_namedparameters "Named Parameters"
-  * @tparam NamedParametersOut0 a sequence of \ref pmp_namedparameters "Named Parameters" for computing the union of the volumes bounded by `tm1` and `tm2`
-  * @tparam NamedParametersOut1 a sequence of \ref pmp_namedparameters "Named Parameters" for computing the intersection of the volumes bounded by `tm1` and `tm2`
-  * @tparam NamedParametersOut2 a sequence of \ref pmp_namedparameters "Named Parameters" for computing the difference of the volumes bounded by `tm1` and `tm2`
-  * @tparam NamedParametersOut3 a sequence of \ref pmp_namedparameters "Named Parameters" for computing the difference of the volumes bounded by `tm2` and `tm1`
+  * @tparam NamedParameters1 a sequence of \ref bgl_namedparameters "Named Parameters"
+  * @tparam NamedParameters2 a sequence of \ref bgl_namedparameters "Named Parameters"
+  * @tparam NamedParametersOut0 a sequence of \ref bgl_namedparameters "Named Parameters" for computing the union of the volumes bounded by `tm1` and `tm2`
+  * @tparam NamedParametersOut1 a sequence of \ref bgl_namedparameters "Named Parameters" for computing the intersection of the volumes bounded by `tm1` and `tm2`
+  * @tparam NamedParametersOut2 a sequence of \ref bgl_namedparameters "Named Parameters" for computing the difference of the volumes bounded by `tm1` and `tm2`
+  * @tparam NamedParametersOut3 a sequence of \ref bgl_namedparameters "Named Parameters" for computing the difference of the volumes bounded by `tm2` and `tm1`
   *
   * @param tm1 first input triangulated surface mesh
   * @param tm2 second input triangulated surface mesh
   * @param output an array of output surface meshes
-  * @param np1 optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
-  * @param np2 optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+  * @param np1 an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+  * @param np2 an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
   *
   * \cgalNamedParamsBegin
-  *   \cgalParamBegin{vertex_point_map}
-  *     the property map with the points associated to the vertices of `tm1` (`tm2`).
-  *     If this parameter is omitted, an internal property map for
-  *     `CGAL::vertex_point_t` should be available in `TriangleMesh`
-  *   \cgalParamEnd
-  *   \cgalParamBegin{edge_is_constrained_map} a property map containing the
-  *     constrained-or-not status of each edge of `tm1` (`tm2`).
-  *   \cgalParamEnd
-  *   \cgalParamBegin{face_index_map} a property map containing the index of each face of `tm1` (`tm2`).
-  *     Note that if the property map is writable, the indices of the faces
-  *     of `tm1` and `tm2` will be set after the corefinement is done.
-  *   \cgalParamEnd
-  *   \cgalParamBegin{visitor} a class model of `PMPCorefinementVisitor`
-  *                            that is used to track the creation of new faces  (`np1` only)
-  *   \cgalParamEnd
-  *   \cgalParamBegin{throw_on_self_intersection} if `true`, for each input triangle mesh,
-  *      the set of triangles close to the intersection of `tm1` and `tm2` will be
-  *      checked for self-intersection and `CGAL::Polygon_mesh_processing::Corefinement::Self_intersection_exception`
-  *      will be thrown if at least one is found (`np1` only).
-  *   \cgalParamEnd
+  *   \cgalParamNBegin{vertex_point_map}
+  *     \cgalParamDescription{a property map associating points to the vertices of `tm1` (`tm2`)}
+  *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+  *                    as key type and `%Point_3` as value type}
+  *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm1 (tm2))`}
+  *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+  *                     must be available in `TriangleMesh`.}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{edge_is_constrained_map}
+  *     \cgalParamDescription{a property map containing the constrained-or-not status of each edge of `tm1` (`tm2`)}
+  *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%edge_descriptor`
+  *                    as key type and `bool` as value type}
+  *     \cgalParamDefault{a constant property map returning `false` for any edge}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{face_index_map}
+  *     \cgalParamDescription{a property map associating to each face of `tm1` (`tm2`) a unique index
+  *                           between `0` and `num_faces(tm1) - 1` (`num_faces(tm2) - 1`)}
+  *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%face_descriptor`
+  *                    as key type and `std::size_t` as value type}
+  *     \cgalParamDefault{an automatically indexed internal map}
+  *     \cgalParamExtra{If the property map is writable, the indices of the faces of `tm1` and `tm2`
+  *                     will be set after the corefinement is done.}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{visitor}
+  *     \cgalParamDescription{a visitor used to track the creation of new faces}
+  *     \cgalParamType{a class model of `PMPCorefinementVisitor`}
+  *     \cgalParamDefault{`Corefinement::Default_visitor<TriangleMesh>`}
+  *     \cgalParamExtra{`np1` only}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{throw_on_self_intersection}
+  *     \cgalParamDescription{If `true`, the set of triangles closed to the intersection of `tm1` and `tm2` will be
+  *                           checked for self-intersections and `Corefinement::Self_intersection_exception`
+  *                           will be thrown if at least one self-intersection is found.}
+  *     \cgalParamType{Boolean}
+  *     \cgalParamDefault{`false`}
+  *     \cgalParamExtra{`np1` only}
+  *   \cgalParamNEnd
   * \cgalNamedParamsEnd
   *
-  * @param nps_out tuple of optional sequences of \ref pmp_namedparameters "Named Parameters" each among the ones listed below
+  * @param nps_out an optional tuple of sequences of \ref bgl_namedparameters "Named Parameters" each among the ones listed below
   *        (`tm_out` being used to refer to the output surface mesh in `output` corresponding to a given named parameter sequence)
   *
   * \cgalNamedParamsBegin
-  *   \cgalParamBegin{vertex_point_map}
-  *     the property map with the points associated to the vertices of `tm_out`.
-  *     If this parameter is omitted, an internal property map for
-  *     `CGAL::vertex_point_t` must be available in `TriangleMesh`
-  *   \cgalParamEnd
-  *   \cgalParamBegin{edge_is_constrained_map} a property map containing the
-  *     constrained-or-not status of each edge of `tm_out`. An edge of `tm_out` is constrained
-  *     if it is on the intersection of `tm1` and `tm2`, or if the edge corresponds to a
-  *     constrained edge in `tm1` or `tm2`.
-  *   \cgalParamEnd
+  *   \cgalParamNBegin{vertex_point_map}
+  *     \cgalParamDescription{a property map associating points to the vertices of `tm_out`}
+  *     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+  *                    as key type and `%Point_3` as value type}
+  *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm_out)`}
+  *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+  *                     must be available in `TriangleMesh`.}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{edge_is_constrained_map}
+  *     \cgalParamDescription{a property map containing the constrained-or-not status of each edge of `tm_out`.
+  *                           An edge of `tm_out` is constrained if it is on the intersection of `tm1` and `tm2`,
+  *                           or if the edge corresponds to a constrained edge in `tm1` or `tm2`.}
+  *     \cgalParamType{a class model of `WritablePropertyMap` with `boost::graph_traits<TriangleMesh>::%edge_descriptor`
+  *                    as key type and `bool` as value type}
+  *   \cgalParamNEnd
   * \cgalNamedParamsEnd
   *
   * @return an array filled as follows: for each operation computed, the position in the array
@@ -351,8 +193,11 @@ corefine_and_compute_boolean_operations(
                      NamedParametersOut2,
                      NamedParametersOut3>& nps_out)
 {
+  using parameters::choose_parameter;
+  using parameters::get_parameter;
+
   const bool throw_on_self_intersection =
-    parameters::choose_parameter(parameters::get_parameter(np1, internal_np::throw_on_self_intersection), false);
+    choose_parameter(get_parameter(np1, internal_np::throw_on_self_intersection), false);
 
 // Vertex point maps
   //for input meshes
@@ -365,11 +210,11 @@ corefine_and_compute_boolean_operations(
     static const bool same_vpm = (boost::is_same<Vpm,Vpm2>::value); )
   CGAL_static_assertion(same_vpm);
 
-  Vpm vpm1 = parameters::choose_parameter(parameters::get_parameter(np1, internal_np::vertex_point),
-                                 get_property_map(boost::vertex_point, tm1));
+  Vpm vpm1 = choose_parameter(get_parameter(np1, internal_np::vertex_point),
+                              get_property_map(boost::vertex_point, tm1));
 
-  Vpm vpm2 = parameters::choose_parameter(parameters::get_parameter(np2, internal_np::vertex_point),
-                                 get_property_map(boost::vertex_point, tm2));
+  Vpm vpm2 = choose_parameter(get_parameter(np2, internal_np::vertex_point),
+                              get_property_map(boost::vertex_point, tm2));
 
   typedef typename boost::property_traits<Vpm>::value_type Point_3;
 
@@ -497,10 +342,8 @@ corefine_and_compute_boolean_operations(
     Corefinement::No_mark<TriangleMesh>//default
   > ::type Ecm2;
 
-  Ecm1 ecm1 = parameters::choose_parameter( parameters::get_parameter(np1, internal_np::edge_is_constrained),
-                                   Corefinement::No_mark<TriangleMesh>() );
-  Ecm2 ecm2 = parameters::choose_parameter( parameters::get_parameter(np2, internal_np::edge_is_constrained),
-                                   Corefinement::No_mark<TriangleMesh>() );
+  Ecm1 ecm1 = choose_parameter<Ecm1>(get_parameter(np1, internal_np::edge_is_constrained));
+  Ecm2 ecm2 = choose_parameter<Ecm2>(get_parameter(np2, internal_np::edge_is_constrained));
 
   typedef Corefinement::Ecm_bind<TriangleMesh, Ecm1, Ecm2> Ecm_in;
 
@@ -514,34 +357,28 @@ corefine_and_compute_boolean_operations(
   typedef std::tuple<Ecm_out_0, Ecm_out_1, Ecm_out_2, Ecm_out_3>
                                                             Edge_mark_map_tuple;
 
-// Face index point maps
-  typedef typename GetFaceIndexMap<TriangleMesh,
-                                   NamedParameters1>::type Fid_map;
-  typedef typename GetFaceIndexMap<TriangleMesh,
-                                   NamedParameters2>::type Fid_map2;
-  CGAL_USE_TYPE(Fid_map2);
-  CGAL_assertion_code(
-    static const bool same_fidmap = (boost::is_same<Fid_map,Fid_map2>::value);)
-  CGAL_static_assertion(same_fidmap);
+  // Face index point maps
+  typedef typename CGAL::GetInitializedFaceIndexMap<TriangleMesh, NamedParameters1>::type FaceIndexMap1;
+  typedef typename CGAL::GetInitializedFaceIndexMap<TriangleMesh, NamedParameters2>::type FaceIndexMap2;
 
-  Fid_map fid_map1 = parameters::choose_parameter(parameters::get_parameter(np1, internal_np::face_index),
-                                        get_property_map(boost::face_index, tm1));
-  Fid_map fid_map2 = parameters::choose_parameter(parameters::get_parameter(np2, internal_np::face_index),
-                                         get_property_map(boost::face_index, tm2));
-// User visitor
+  FaceIndexMap1 fid_map1 = get_initialized_face_index_map(tm1, np1);
+  FaceIndexMap2 fid_map2 = get_initialized_face_index_map(tm2, np2);
+
+
+  // User visitor
   typedef typename internal_np::Lookup_named_param_def <
     internal_np::graph_visitor_t,
     NamedParameters1,
     Corefinement::Default_visitor<TriangleMesh>//default
   > ::type User_visitor;
-  User_visitor uv( parameters::choose_parameter( parameters::get_parameter(np1, internal_np::graph_visitor),
-                   Corefinement::Default_visitor<TriangleMesh>() ) );
+  User_visitor uv(choose_parameter<User_visitor>(get_parameter(np1, internal_np::graph_visitor)));
 
   // surface intersection algorithm call
   typedef Corefinement::Face_graph_output_builder<TriangleMesh,
                                                   Vpm,
                                                   Vpm_out_tuple,
-                                                  Fid_map,
+                                                  FaceIndexMap1,
+                                                  FaceIndexMap2,
                                                   Default,
                                                   Ecm_in,
                                                   Edge_mark_map_tuple,
@@ -551,20 +388,17 @@ corefine_and_compute_boolean_operations(
     TriangleMesh, Vpm, Ob, Ecm_in, User_visitor> Algo_visitor;
   Ecm_in ecm_in(tm1,tm2,ecm1,ecm2);
   Edge_mark_map_tuple ecms_out(ecm_out_0, ecm_out_1, ecm_out_2, ecm_out_3);
-  Ob ob(tm1, tm2, vpm1, vpm2, fid_map1, fid_map2, ecm_in,
-        vpm_out_tuple, ecms_out, uv, output);
+  Ob ob(tm1, tm2, vpm1, vpm2, fid_map1, fid_map2, ecm_in, vpm_out_tuple, ecms_out, uv, output);
 
   // special case used for clipping open meshes
-  if (  parameters::choose_parameter( parameters::get_parameter(np1, internal_np::use_bool_op_to_clip_surface),
-                             false) )
+  if (choose_parameter(get_parameter(np1, internal_np::use_bool_op_to_clip_surface), false))
   {
     CGAL_assertion(output[Corefinement::INTERSECTION] != boost::none);
     CGAL_assertion(output[Corefinement::UNION] == boost::none);
     CGAL_assertion(output[Corefinement::TM1_MINUS_TM2] == boost::none);
     CGAL_assertion(output[Corefinement::TM2_MINUS_TM1] == boost::none);
     const bool use_compact_clipper =
-      parameters::choose_parameter( parameters::get_parameter(np1, internal_np::use_compact_clipper),
-                           true);
+      choose_parameter(get_parameter(np1, internal_np::use_compact_clipper), true);
     ob.setup_for_clipping_a_surface(use_compact_clipper);
   }
 
@@ -642,52 +476,79 @@ corefine_and_compute_boolean_operations(
   * \pre \link CGAL::Polygon_mesh_processing::does_bound_a_volume() `CGAL::Polygon_mesh_processing::does_bound_a_volume(tm2)` \endlink
   *
   * @tparam TriangleMesh a model of `MutableFaceGraph`, `HalfedgeListGraph` and `FaceListGraph`
-  * @tparam NamedParameters1 a sequence of \ref pmp_namedparameters "Named Parameters"
-  * @tparam NamedParameters2 a sequence of \ref pmp_namedparameters "Named Parameters"
-  * @tparam NamedParametersOut a sequence of \ref pmp_namedparameters "Named Parameters"
+  * @tparam NamedParameters1 a sequence of \ref bgl_namedparameters "Named Parameters"
+  * @tparam NamedParameters2 a sequence of \ref bgl_namedparameters "Named Parameters"
+  * @tparam NamedParametersOut a sequence of \ref bgl_namedparameters "Named Parameters"
   *
   * @param tm1 first input triangulated surface mesh
   * @param tm2 second input triangulated surface mesh
   * @param tm_out output surface mesh
-  * @param np1 optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
-  * @param np2 optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+  * @param np1 an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+  * @param np2 an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
   *
   * \cgalNamedParamsBegin
-  *   \cgalParamBegin{vertex_point_map}
-  *     the property map with the points associated to the vertices of `tm1` (`tm2`).
-  *     If this parameter is omitted, an internal property map for
-  *     `CGAL::vertex_point_t` must be available in `TriangleMesh`
-  *   \cgalParamEnd
-  *   \cgalParamBegin{edge_is_constrained_map} a property map containing the
-  *     constrained-or-not status of each edge of `tm1` (`tm2`).
-  *   \cgalParamEnd
-  *   \cgalParamBegin{face_index_map} a property map containing the index of each face of `tm1` (`tm2`).
-  *     Note that if the property map is writable, the indices of the faces
-  *     of `tm1` and `tm2` will be set after the corefinement is done.
-  *   \cgalParamEnd
-  *   \cgalParamBegin{visitor} a class model of `PMPCorefinementVisitor`
-  *                            that is used to track the creation of new faces  (`np1` only)
-  *   \cgalParamEnd
-  *   \cgalParamBegin{throw_on_self_intersection} if `true`, for each input triangle mesh,
-  *      the set of triangles close to the intersection of `tm1` and `tm2` will be
-  *      checked for self-intersection and `CGAL::Polygon_mesh_processing::Corefinement::Self_intersection_exception`
-  *      will be thrown if at least one is found (`np1` only).
-  *   \cgalParamEnd
+  *   \cgalParamNBegin{vertex_point_map}
+  *     \cgalParamDescription{a property map associating points to the vertices of `tm1` (`tm2`)}
+  *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+  *                    as key type and `%Point_3` as value type}
+  *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm1 (tm2))`}
+  *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+  *                     must be available in `TriangleMesh`.}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{edge_is_constrained_map}
+  *     \cgalParamDescription{a property map containing the constrained-or-not status of each edge of `tm1` (`tm2`)}
+  *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%edge_descriptor`
+  *                    as key type and `bool` as value type}
+  *     \cgalParamDefault{a constant property map returning `false` for any edge}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{face_index_map}
+  *     \cgalParamDescription{a property map associating to each face of `tm1` (`tm2`) a unique index
+  *                           between `0` and `num_faces(tm1) - 1` (`num_faces(tm2) - 1`)}
+  *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%face_descriptor`
+  *                    as key type and `std::size_t` as value type}
+  *     \cgalParamDefault{an automatically indexed internal map}
+  *     \cgalParamExtra{If the property map is writable, the indices of the faces of `tm1` and `tm2`
+  *                     will be set after the corefinement is done.}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{visitor}
+  *     \cgalParamDescription{a visitor used to track the creation of new faces}
+  *     \cgalParamType{a class model of `PMPCorefinementVisitor`}
+  *     \cgalParamDefault{`Corefinement::Default_visitor<TriangleMesh>`}
+  *     \cgalParamExtra{`np1` only}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{throw_on_self_intersection}
+  *     \cgalParamDescription{If `true` the set of triangles closed to the intersection of `tm1` and `tm2` will be
+  *                           checked for self-intersections and `Corefinement::Self_intersection_exception`
+  *                           will be thrown if at least one self-intersection is found.}
+  *     \cgalParamType{Boolean}
+  *     \cgalParamDefault{`false`}
+  *     \cgalParamExtra{`np1` only}
+  *   \cgalParamNEnd
   * \cgalNamedParamsEnd
   *
-  * @param np_out optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+  * @param np_out an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
   *
   * \cgalNamedParamsBegin
-  *   \cgalParamBegin{vertex_point_map}
-  *     the property map with the points associated to the vertices of `tm_out`.
-  *     If this parameter is omitted, an internal property map for
-  *     `CGAL::vertex_point_t` must be available in `TriangleMesh`
-  *   \cgalParamEnd
-  *   \cgalParamBegin{edge_is_constrained_map} a property map containing the
-  *     constrained-or-not status of each edge of `tm_out`. An edge of `tm_out` is constrained
-  *     if it is on the intersection of `tm1` and `tm2`, or if the edge corresponds to a
-  *     constrained edge in `tm1` or `tm2`.
-  *   \cgalParamEnd
+  *   \cgalParamNBegin{vertex_point_map}
+  *     \cgalParamDescription{a property map associating points to the vertices of `tm_out`}
+  *     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+  *                    as key type and `%Point_3` as value type}
+  *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm_out)`}
+  *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+  *                     must be available in `TriangleMesh`.}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{edge_is_constrained_map}
+  *     \cgalParamDescription{a property map containing the constrained-or-not status of each edge of `tm_out`.
+  *                           An edge of `tm_out` is constrained if it is on the intersection of `tm1` and `tm2`,
+  *                           or if the edge corresponds to a constrained edge in `tm1` or `tm2`.}
+  *     \cgalParamType{a class model of `WritablePropertyMap` with `boost::graph_traits<TriangleMesh>::%edge_descriptor`
+  *                    as key type and `bool` as value type}
+  *   \cgalParamNEnd
   * \cgalNamedParamsEnd
   *
   * @return `true` if the output surface mesh is manifold and is put into `tm_out`.
@@ -794,45 +655,63 @@ corefine_and_compute_difference(      Tr
  * \pre \link CGAL::Polygon_mesh_processing::does_self_intersect() `!CGAL::Polygon_mesh_processing::does_self_intersect(tm2)` \endlink
  *
  * @tparam TriangleMesh a model of `MutableFaceGraph`, `HalfedgeListGraph` and `FaceListGraph`
- * @tparam NamedParameters1 a sequence of \ref pmp_namedparameters "Named Parameters"
- * @tparam NamedParameters2 a sequence of \ref pmp_namedparameters "Named Parameters"
+ * @tparam NamedParameters1 a sequence of \ref bgl_namedparameters "Named Parameters"
+ * @tparam NamedParameters2 a sequence of \ref bgl_namedparameters "Named Parameters"
  *
  * @param tm1 first input triangulated surface mesh
  * @param tm2 second input triangulated surface mesh
- * @param np1 optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
- * @param np2 optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+ * @param np1 an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+ * @param np2 an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *   \cgalParamBegin{vertex_point_map}
- *     the property map with the points associated to the vertices of `tm1` (`tm2`).
- *     If this parameter is omitted, an internal property map for
- *     `CGAL::vertex_point_t` must be available in `TriangleMesh`
- *   \cgalParamEnd
- *   \cgalParamBegin{edge_is_constrained_map} a property map containing the
- *     constrained-or-not status of each edge of `tm1` (`tm2`)
- *   \cgalParamEnd
- *   \cgalParamBegin{visitor} a class model of `PMPCorefinementVisitor`
- *                            that is used to track the creation of new faces (`np1` only)
- *   \cgalParamEnd
- *   \cgalParamBegin{throw_on_self_intersection} if `true`, for each input triangle mesh,
- *      the set of triangles close to the intersection of `tm1` and `tm2` will be
- *      checked for self-intersection and `CGAL::Polygon_mesh_processing::Corefinement::Self_intersection_exception`
- *      will be thrown if at least one is found (`np1` only).
- *   \cgalParamEnd
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `tm1` (`tm2`)}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+ *                    as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm1 (tm2))`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     must be available in `TriangleMesh`.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{edge_is_constrained_map}
+ *     \cgalParamDescription{a property map containing the constrained-or-not status of each edge of `tm1` (`tm2`)}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%edge_descriptor`
+ *                    as key type and `bool` as value type}
+ *     \cgalParamDefault{a constant property map returning `false` for any edge}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{visitor}
+ *     \cgalParamDescription{a visitor used to track the creation of new faces}
+ *     \cgalParamType{a class model of `PMPCorefinementVisitor`}
+ *     \cgalParamDefault{`Corefinement::Default_visitor<TriangleMesh>`}
+ *     \cgalParamExtra{`np1` only}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{throw_on_self_intersection}
+ *     \cgalParamDescription{If `true` the set of triangles closed to the intersection of `tm1` and `tm2` will be
+ *                           checked for self-intersections and `Corefinement::Self_intersection_exception`
+ *                           will be thrown if at least one self-intersection is found.}
+ *     \cgalParamType{Boolean}
+ *     \cgalParamDefault{`false`}
+ *     \cgalParamExtra{`np1` only}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  *
  */
- template <class TriangleMesh,
-           class NamedParameters1,
-           class NamedParameters2>
- void
- corefine(      TriangleMesh& tm1,
-                TriangleMesh& tm2,
-          const NamedParameters1& np1,
-          const NamedParameters2& np2)
+template <class TriangleMesh,
+          class NamedParameters1,
+          class NamedParameters2>
+void
+corefine(      TriangleMesh& tm1,
+               TriangleMesh& tm2,
+         const NamedParameters1& np1,
+         const NamedParameters2& np2)
 {
+  using parameters::choose_parameter;
+  using parameters::get_parameter;
+
   const bool throw_on_self_intersection =
-    parameters::choose_parameter(parameters::get_parameter(np1, internal_np::throw_on_self_intersection), false);
+    choose_parameter(get_parameter(np1, internal_np::throw_on_self_intersection), false);
 
 // Vertex point maps
   typedef typename GetVertexPointMap<TriangleMesh,
@@ -844,11 +723,11 @@ corefine_and_compute_difference(      Tr
     static const bool same_vpm = (boost::is_same<Vpm,Vpm2>::value);)
   CGAL_static_assertion(same_vpm);
 
-  Vpm vpm1 = parameters::choose_parameter(parameters::get_parameter(np1, internal_np::vertex_point),
-                                 get_property_map(boost::vertex_point, tm1));
+  Vpm vpm1 = choose_parameter(get_parameter(np1, internal_np::vertex_point),
+                              get_property_map(boost::vertex_point, tm1));
 
-  Vpm vpm2 = parameters::choose_parameter(parameters::get_parameter(np2, internal_np::vertex_point),
-                                 get_property_map(boost::vertex_point, tm2));
+  Vpm vpm2 = choose_parameter(get_parameter(np2, internal_np::vertex_point),
+                              get_property_map(boost::vertex_point, tm2));
 
 // Edge is-constrained maps
   typedef typename internal_np::Lookup_named_param_def <
@@ -863,10 +742,8 @@ corefine_and_compute_difference(      Tr
     Corefinement::No_mark<TriangleMesh>//default
   > ::type Ecm2;
 
-  Ecm1 ecm1 = parameters::choose_parameter( parameters::get_parameter(np1, internal_np::edge_is_constrained),
-                                   Corefinement::No_mark<TriangleMesh>() );
-  Ecm2 ecm2 = parameters::choose_parameter( parameters::get_parameter(np2, internal_np::edge_is_constrained),
-                                   Corefinement::No_mark<TriangleMesh>() );
+  Ecm1 ecm1 = choose_parameter<Ecm1>(get_parameter(np1, internal_np::edge_is_constrained));
+  Ecm2 ecm2 = choose_parameter<Ecm2>(get_parameter(np2, internal_np::edge_is_constrained));
 
   typedef Corefinement::Ecm_bind<TriangleMesh, Ecm1, Ecm2> Ecm;
 
@@ -883,8 +760,7 @@ corefine_and_compute_difference(      Tr
     NamedParameters1,
     Corefinement::Default_visitor<TriangleMesh>//default
   > ::type User_visitor;
-  User_visitor uv( parameters::choose_parameter( parameters::get_parameter(np1, internal_np::graph_visitor),
-                   Corefinement::Default_visitor<TriangleMesh>() ) );
+  User_visitor uv(choose_parameter<User_visitor>(get_parameter(np1, internal_np::graph_visitor)));
 
 // surface intersection algorithm call
   typedef Corefinement::No_extra_output_from_corefinement<TriangleMesh> Ob;
@@ -909,35 +785,56 @@ namespace experimental {
  * @tparam NamedParameters a sequence of \ref namedparameters
  *
  * @param tm input triangulated surface mesh
- * @param np optional sequence of \ref namedparameters among the ones listed below
+ * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *   \cgalParamBegin{vertex_point_map}
- *     the property map with the points associated to the vertices of `tm`.
- *     If this parameter is omitted, an internal property map for
- *     `CGAL::vertex_point_t` must be available in `TriangleMesh`
- *   \cgalParamEnd
- *   \cgalParamBegin{edge_is_constrained_map} a property map containing the
- *     constrained-or-not status of each edge of `tm`
- *   \cgalParamEnd
- *   \cgalParamBegin{visitor} a class model of `PMPCorefinementVisitor`
- *                            that is used to track the creation of new faces
- *   \cgalParamEnd
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `tm`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+ *                    as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     must be available in `TriangleMesh`.}
+ *  \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{edge_is_constrained_map}
+ *     \cgalParamDescription{a property map containing the constrained-or-not status of each edge of `tm`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%edge_descriptor`
+ *                    as key type and `bool` as value type}
+ *     \cgalParamDefault{a constant property map returning `false` for any edge}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{face_index_map}
+ *     \cgalParamDescription{a property map associating to each face of `tm` a unique index between `0` and `num_faces(tm) - 1`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%face_descriptor`
+ *                    as key type and `std::size_t` as value type}
+ *     \cgalParamDefault{an automatically indexed internal map}
+ *     \cgalParamExtra{If the property map is writable, the indices of the faces of `tm1` and `tm2`
+ *                     will be set after the corefinement is done.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{visitor}
+ *     \cgalParamDescription{a visitor used to track the creation of new faces}
+ *     \cgalParamType{a class model of `PMPCorefinementVisitor`}
+ *     \cgalParamDefault{`Corefinement::Default_visitor<TriangleMesh>`}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  *
  */
- template <class TriangleMesh,
-           class NamedParameters>
- void
- autorefine(      TriangleMesh& tm,
-            const NamedParameters& np)
+template <class TriangleMesh,
+          class NamedParameters>
+void
+autorefine(      TriangleMesh& tm,
+           const NamedParameters& np)
 {
+  using parameters::choose_parameter;
+  using parameters::get_parameter;
+
 // Vertex point maps
-  typedef typename GetVertexPointMap<TriangleMesh,
-                                     NamedParameters>::type Vpm;
+  typedef typename GetVertexPointMap<TriangleMesh, NamedParameters>::type Vpm;
 
-  Vpm vpm = parameters::choose_parameter(parameters::get_parameter(np, internal_np::vertex_point),
-                                get_property_map(boost::vertex_point, tm));
+  Vpm vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
+                             get_property_map(boost::vertex_point, tm));
 
 // Edge is-constrained maps
   typedef typename internal_np::Lookup_named_param_def <
@@ -945,10 +842,7 @@ namespace experimental {
     NamedParameters,
     Corefinement::No_mark<TriangleMesh>//default
   > ::type Ecm;
-
-
-  Ecm ecm = parameters::choose_parameter( parameters::get_parameter(np, internal_np::edge_is_constrained),
-                                 Corefinement::No_mark<TriangleMesh>() );
+  Ecm ecm = choose_parameter<Ecm>(get_parameter(np, internal_np::edge_is_constrained));
 
 // User visitor
   typedef typename internal_np::Lookup_named_param_def <
@@ -956,8 +850,7 @@ namespace experimental {
     NamedParameters,
     Corefinement::Default_visitor<TriangleMesh>//default
   > ::type User_visitor;
-  User_visitor uv( parameters::choose_parameter( parameters::get_parameter(np, internal_np::graph_visitor),
-                   Corefinement::Default_visitor<TriangleMesh>() ) );
+  User_visitor uv(choose_parameter<User_visitor>(get_parameter(np, internal_np::graph_visitor)));
 
 
 // surface intersection algorithm call
@@ -984,56 +877,77 @@ namespace experimental {
  * @tparam NamedParameters a sequence of \ref namedparameters
  *
  * @param tm input triangulated surface mesh
- * @param np optional sequence of \ref namedparameters among the ones listed below
+ * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *   \cgalParamBegin{vertex_point_map}
- *     the property map with the points associated to the vertices of `tm`.
- *     If this parameter is omitted, an internal property map for
- *     `CGAL::vertex_point_t` must be available in `TriangleMesh`
- *   \cgalParamEnd
- *   \cgalParamBegin{edge_is_constrained_map} a property map containing the
- *     constrained-or-not status of each edge of `tm`
- *   \cgalParamEnd
- *   \cgalParamBegin{face_index_map} a property map containing the index of each face of `tm` \cgalParamEnd
- *   \cgalParamBegin{visitor} a class model of `PMPCorefinementVisitor`
- *                            that is used to track the creation of new faces
- *   \cgalParamEnd
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `tm`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+ *                    as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     must be available in `TriangleMesh`.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{edge_is_constrained_map}
+ *     \cgalParamDescription{a property map containing the constrained-or-not status of each edge of `tm`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%edge_descriptor`
+ *                    as key type and `bool` as value type}
+ *     \cgalParamDefault{a constant property map returning `false` for any edge}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{face_index_map}
+ *     \cgalParamDescription{a property map associating to each face of `tm` a unique index between `0` and `num_faces(tm) - 1`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%face_descriptor`
+ *                    as key type and `std::size_t` as value type}
+ *     \cgalParamDefault{an automatically indexed internal map}
+ *     \cgalParamExtra{If the property map is writable, the indices of the faces of `tm` will be set
+ *                     after the autorefinement is done.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{visitor}
+ *     \cgalParamDescription{a visitor used to track the creation of new faces}
+ *     \cgalParamType{a class model of `PMPCorefinementVisitor`}
+ *     \cgalParamDefault{`Corefinement::Default_visitor<TriangleMesh>`}
+ *   \cgalParamNEnd
+ *
  * \cgalNamedParamsEnd
  *
  */
- template <class TriangleMesh,
-           class NamedParameters>
- bool
- autorefine_and_remove_self_intersections(      TriangleMesh& tm,
-                                          const NamedParameters& np)
+template <class TriangleMesh,
+          class NamedParameters>
+bool
+autorefine_and_remove_self_intersections(      TriangleMesh& tm,
+                                         const NamedParameters& np)
 {
+  using parameters::choose_parameter;
+  using parameters::get_parameter;
+
 // Vertex point maps
-  typedef typename GetVertexPointMap<TriangleMesh,
-                                     NamedParameters>::type Vpm;
-  Vpm vpm = parameters::choose_parameter(parameters::get_parameter(np, internal_np::vertex_point),
-                                get_property_map(boost::vertex_point, tm));
+  typedef typename GetVertexPointMap<TriangleMesh, NamedParameters>::type Vpm;
+  Vpm vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
+                             get_property_map(boost::vertex_point, tm));
+
 // Face index map
-  typedef typename GetFaceIndexMap<TriangleMesh,
-                                   NamedParameters>::type Fid_map;
-  Fid_map fid_map = parameters::choose_parameter(parameters::get_parameter(np, internal_np::face_index),
-                                        get_property_map(boost::face_index, tm));
+  typedef typename GetInitializedFaceIndexMap<TriangleMesh, NamedParameters>::type Fid_map;
+  Fid_map fid_map = get_initialized_face_index_map(tm, np);
+
+
 // Edge is-constrained maps
   typedef typename internal_np::Lookup_named_param_def <
     internal_np::edge_is_constrained_t,
     NamedParameters,
     Corefinement::No_mark<TriangleMesh>//default
   > ::type Ecm;
-  Ecm ecm = parameters::choose_parameter( parameters::get_parameter(np, internal_np::edge_is_constrained),
-                                 Corefinement::No_mark<TriangleMesh>() );
+  Ecm ecm = choose_parameter<Ecm>(get_parameter(np, internal_np::edge_is_constrained));
+
 // User visitor
   typedef typename internal_np::Lookup_named_param_def <
     internal_np::graph_visitor_t,
     NamedParameters,
     Corefinement::Default_visitor<TriangleMesh>//default
   > ::type User_visitor;
-  User_visitor uv( parameters::choose_parameter( parameters::get_parameter(np, internal_np::graph_visitor),
-                   Corefinement::Default_visitor<TriangleMesh>() ) );
+  User_visitor uv(choose_parameter<User_visitor>(get_parameter(np, internal_np::graph_visitor)));
 
 // surface intersection algorithm call
   typedef Corefinement::Output_builder_for_autorefinement<TriangleMesh,
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/detect_features.h 5.1-1/include/CGAL/Polygon_mesh_processing/detect_features.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/detect_features.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/detect_features.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/detect_features.h $
-// $Id: detect_features.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/detect_features.h $
+// $Id: detect_features.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -138,23 +138,18 @@ detect_surface_patches(PolygonMesh& p,
                        EdgeIsFeatureMap eif,
                        const NamedParameters& np)
 {
-  //extract types from NPs
-  typename GetFaceIndexMap<PolygonMesh, NamedParameters>::const_type
-          fimap = parameters::choose_parameter(parameters::get_parameter(np, internal_np::face_index),
-                                      get_const_property_map(boost::face_index, p));
-
   int offset = static_cast<int>(
-          parameters::choose_parameter(parameters::get_parameter(np, internal_np::first_index),
-          1));
+          parameters::choose_parameter(parameters::get_parameter(np, internal_np::first_index), 1));
 
   internal::PatchIdMapWrapper<PatchIdMap,
                               typename boost::property_traits<PatchIdMap>::value_type>
           wrapmap(patch_id_map, offset);
+
   return connected_components(p, wrapmap,
                               parameters::edge_is_constrained_map(eif)
-                             .face_index_map(fimap));
-
+                                         .face_index_map(CGAL::get_initialized_face_index_map(p, np)));
 }
+
 template <typename PolygonMesh, typename EdgeIsFeatureMap, typename PatchIdMap>
 typename boost::graph_traits<PolygonMesh>::faces_size_type
 detect_surface_patches(PolygonMesh& p,
@@ -241,22 +236,31 @@ template<typename GT,
  *
  * \tparam PolygonMesh a model of `HalfedgeListGraph`
  * \tparam FT a number type. It is
- * either deduced from the `geom_traits` \ref pmp_namedparameters "Named Parameters" if provided,
+ * either deduced from the `geom_traits` \ref bgl_namedparameters "Named Parameters" if provided,
  * or from the geometric traits class deduced from the point property map
  * of `PolygonMesh`.
  * \tparam EdgeIsFeatureMap a model of `ReadWritePropertyMap` with `boost::graph_traits<PolygonMesh>::%edge_descriptor`
  *  as key type and `bool` as value type. It must be default constructible.
- * \tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+ * \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
  *
  * \param pmesh the polygon mesh
  * \param angle_in_deg the dihedral angle bound
  * \param edge_is_feature_map the property map that will contain the sharp-or-not status of each edge of `pmesh`
- * \param np optional \ref pmp_namedparameters "Named Parameters", amongst those described below
+ * \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
- *    \cgalParamBegin{vertex_feature_degree_map} a property map that will contain the number of adjacent feature edges for
- *  each vertex of `pmesh` \cgalParamEnd
+ *   \cgalParamNBegin{vertex_feature_degree_map}
+ *     \cgalParamDescription{a property map that will associate to each vertex of `pmesh` the number of incident feature edges}
+ *     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor` as key type and `int` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_feature_degree_t(), pmesh)`}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{geom_traits}
+ *     \cgalParamDescription{an instance of a geometric traits class}
+ *     \cgalParamType{a class model of `Kernel`}
+ *     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+ *     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  *
  */
@@ -300,6 +304,7 @@ void detect_sharp_edges(PolygonMesh& pme
    inserted several times.
  * \tparam EdgeIsFeatureMap a model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%edge_descriptor`
  *  as key type and `bool` as value type.
+ *
  * \param pmesh the polygon mesh
  * \param patch_id_map the property map containing the surface patch ids for the faces of `pmesh`. It must be already filled.
  * \param vertex_incident_patches_map a property map that will contain the patch ids of all the faces incident to each vertex of `pmesh`.
@@ -373,33 +378,59 @@ namespace internal
  * computing a
  * surface patch id for each face.
  *
- * A property map for `CGAL::face_index_t` must be either available
- * as an internal property map to `pmesh` or provided as one of the Named Parameters.
- *
  * \tparam PolygonMesh a model of `FaceGraph`
  * \tparam FT a number type. It is
- * either deduced from the `geom_traits` \ref pmp_namedparameters "Named Parameters" if provided,
+ * either deduced from the `geom_traits` \ref bgl_namedparameters "Named Parameters" if provided,
  * or from the geometric traits class deduced from the point property map
  * of `PolygonMesh`.
  * \tparam EdgeIsFeatureMap a model of `ReadWritePropertyMap` with `boost::graph_traits<PolygonMesh>::%edge_descriptor`
  * \tparam PatchIdMap a model of `ReadWritePropertyMap` with
    `boost::graph_traits<PolygonMesh>::%face_descriptor` as key type
    and the desired patch id, model of `CopyConstructible` as value type.
- * \tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+ * \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
  *
  * \param pmesh the polygon mesh
  * \param angle_in_deg the dihedral angle bound
  * \param edge_is_feature_map the property map that will contain the sharp-or-not status of each edge of `pmesh`
  * \param patch_id_map the property map that will contain the surface patch ids for the faces of `pmesh`.
- * \param np optional \ref pmp_namedparameters "Named Parameters", amongst those described below
+ * \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
- *    \cgalParamBegin{vertex_feature_degree_map} a property map that will contain the number of adjacent feature edges for each vertex of `pmesh` \cgalParamEnd
- *    \cgalParamBegin{first_index} a `std::size_t` containing the index of the first surface patch of `pmesh`.
- *      The patches will be numbered on [first_index; first_index + num_patches], where num_patches is the number of surface patches \cgalParamEnd
- *    \cgalParamBegin{face_index_map} a property map containing the index of each face of `pmesh` \cgalParamEnd
- *    \cgalParamBegin{vertex_incident_patches_map} a property map that will contain the patch ids of all the faces incident to each vertex of `pmesh`. \cgalParamEnd
+ *   \cgalParamNBegin{vertex_feature_degree_map}
+ *     \cgalParamDescription{a property map that will associate to each vertex of `pmesh` the number of incident feature edges}
+ *     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+ *                    as key type and `int` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_feature_degree_t(), pmesh)`}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{first_index}
+ *     \cgalParamDescription{the index of the first surface patch of `pmesh`}
+ *     \cgalParamType{`std::size_t`}
+ *     \cgalParamExtra{The patches will be numbered on `[first_index; first_index + num_patches]`,
+ *                     where `num_patches` is the number of surface patches.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{face_index_map}
+ *     \cgalParamDescription{a property map associating to each face of `pmesh` a unique index between `0` and `num_faces(pmesh) - 1`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%face_descriptor`
+ *                    as key type and `std::size_t` as value type}
+ *     \cgalParamDefault{an automatically indexed internal map}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{vertex_incident_patches_map}
+ *     \cgalParamDescription{a property map that will contain the patch ids of all the faces incident to each vertex of `pmesh`}
+ *     \cgalParamType{a model of mutable `LvaluePropertyMap` with
+ *                    `boost::graph_traits<PolygonMesh>::%vertex_descriptor` as key type. Its value type
+ *                    must be a container of `boost::property_traits<PatchIdMap>::%value_type` and have a function `insert()`.}
+ *     \cgalParamExtra{A `std::set` or a `boost::unordered_set` are recommended, as a patch index may be inserted several times.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{geom_traits}
+ *     \cgalParamDescription{an instance of a geometric traits class}
+ *     \cgalParamType{a class model of `Kernel`}
+ *     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+ *     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  * \returns the number of surface patches.
  *
@@ -458,9 +489,6 @@ sharp_edges_segmentation(PolygonMesh& p,
                                  parameters::all_default());
 }
 
-
-
-
 } // end namespace PMP
 } // end namespace CGAL
 
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/distance.h 5.1-1/include/CGAL/Polygon_mesh_processing/distance.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/distance.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/distance.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/distance.h $
-// $Id: distance.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/distance.h $
+// $Id: distance.h 26eb790 2020-06-18T12:46:46+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -15,9 +15,9 @@
 
 #include <CGAL/license/Polygon_mesh_processing/distance.h>
 
+#include <CGAL/Polygon_mesh_processing/internal/mesh_to_point_set_hausdorff_distance.h>
+#include <CGAL/Polygon_mesh_processing/measure.h>
 
-#include <algorithm>
-#include <cmath>
 #include <CGAL/AABB_tree.h>
 #include <CGAL/AABB_traits.h>
 #include <CGAL/AABB_triangle_primitive.h>
@@ -26,29 +26,31 @@
 #include <CGAL/Polygon_mesh_processing/internal/named_function_params.h>
 #include <CGAL/Polygon_mesh_processing/internal/named_params_helper.h>
 #include <CGAL/point_generators_3.h>
-
 #include <CGAL/spatial_sort.h>
-#include <CGAL/Polygon_mesh_processing/measure.h>
 
-#include <CGAL/Polygon_mesh_processing/internal/mesh_to_point_set_hausdorff_distance.h>
 #ifdef CGAL_LINKED_WITH_TBB
-#include <tbb/parallel_for.h>
+#include <tbb/parallel_reduce.h>
 #include <tbb/blocked_range.h>
 #include <atomic>
 #endif // CGAL_LINKED_WITH_TBB
 
 #include <boost/unordered_set.hpp>
 
-namespace CGAL{
+#include <algorithm>
+#include <array>
+#include <cmath>
+
+namespace CGAL {
 namespace Polygon_mesh_processing {
-namespace internal{
-template <class Kernel, class OutputIterator>
-OutputIterator
-triangle_grid_sampling( const typename Kernel::Point_3& p0,
-                        const typename Kernel::Point_3& p1,
-                        const typename Kernel::Point_3& p2,
-                        double distance,
-                        OutputIterator out)
+namespace internal {
+
+template <class Kernel, class PointOutputIterator>
+PointOutputIterator
+triangle_grid_sampling(const typename Kernel::Point_3& p0,
+                       const typename Kernel::Point_3& p1,
+                       const typename Kernel::Point_3& p2,
+                       double distance,
+                       PointOutputIterator out)
 {
   typename Kernel::Compute_squared_distance_3 squared_distance;
   const double d_p0p1 = to_double(approximate_sqrt( squared_distance(p0, p1) ));
@@ -57,16 +59,17 @@ triangle_grid_sampling( const typename K
   const double n = (std::max)(std::ceil( d_p0p1 / distance ),
                               std::ceil( d_p0p2 / distance ));
 
-  for (double i=1; i<n; ++i)
-    for (double j=1; j<n-i; ++j)
+  for(double i=1; i<n; ++i)
+  {
+    for(double j=1; j<n-i; ++j)
     {
       const double c0=(1-(i+j)/n), c1=i/n, c2=j/n;
-      *out++=typename Kernel::Point_3(
-              p0.x()*c0+p1.x()*c1+p2.x()*c2,
-              p0.y()*c0+p1.y()*c1+p2.y()*c2,
-              p0.z()*c0+p1.z()*c1+p2.z()*c2
-            );
+      *out++ = typename Kernel::Point_3(p0.x()*c0+p1.x()*c1+p2.x()*c2,
+                                        p0.y()*c0+p1.y()*c1+p2.y()*c2,
+                                        p0.z()*c0+p1.z()*c1+p2.z()*c2);
     }
+  }
+
   return out;
 }
 
@@ -78,21 +81,27 @@ struct Distance_computation{
   const AABB_tree& tree;
   const PointRange& sample_points;
   Point_3 initial_hint;
-  std::atomic<double>* distance;
-
+  double distance;
+  //constructor
   Distance_computation(
           const AABB_tree& tree,
           const Point_3& p,
-          const PointRange& sample_points,
-          std::atomic<double>* d)
+          const PointRange& sample_points)
     : tree(tree)
     , sample_points(sample_points)
     , initial_hint(p)
-    , distance(d)
+    , distance(-1)
+  {}
+  //split constructor
+  Distance_computation(Distance_computation& s, tbb::split )
+    : tree(s.tree)
+    , sample_points(s.sample_points)
+    , initial_hint(s.initial_hint)
+    , distance(-1)
   {}
 
   void
-  operator()(const tbb::blocked_range<std::size_t>& range) const
+  operator()(const tbb::blocked_range<std::size_t>& range)
   {
     Point_3 hint = initial_hint;
     double hdist = 0;
@@ -101,17 +110,14 @@ struct Distance_computation{
       hint = tree.closest_point(*(sample_points.begin() + i), hint);
       typename Kernel_traits<Point_3>::Kernel::Compute_squared_distance_3 squared_distance;
       double d = to_double(CGAL::approximate_sqrt( squared_distance(hint,*(sample_points.begin() + i)) ));
-      if (d>hdist) hdist=d;
-    }
-
-    // update max value stored in distance
-    double current_value = *distance;
-    while( current_value < hdist )
-    {
-      if(distance->compare_exchange_weak(current_value, hdist))
-        current_value = hdist;
+      if(d > hdist)
+        hdist=d;
     }
+    if(hdist > distance)
+      distance = hdist;
   }
+
+  void join( Distance_computation& rhs ) {distance = (std::max)(rhs.distance, distance); }
 };
 #endif
 
@@ -128,13 +134,13 @@ double approximate_Hausdorff_distance_im
   CGAL_static_assertion_msg (!(boost::is_convertible<Concurrency_tag, Parallel_tag>::value),
                              "Parallel_tag is enabled but TBB is unavailable.");
 #else
-  if (boost::is_convertible<Concurrency_tag,Parallel_tag>::value)
+  if(boost::is_convertible<Concurrency_tag,Parallel_tag>::value)
   {
     std::atomic<double> distance;
     distance=0;
-    Distance_computation<AABBTree, PointRange> f(tree, hint, sample_points, &distance);
-    tbb::parallel_for(tbb::blocked_range<std::size_t>(0, sample_points.size()), f);
-    return distance;
+    Distance_computation<AABBTree, PointRange> f(tree, hint, sample_points);
+    tbb::parallel_reduce(tbb::blocked_range<std::size_t>(0, sample_points.size()), f);
+    return f.distance;
   }
   else
 #endif
@@ -153,19 +159,170 @@ double approximate_Hausdorff_distance_im
   }
 }
 
-} //end of namespace internal
+template<typename PointOutputIterator,
+         typename GeomTraits,
+         typename NamedParameters,
+         typename TriangleIterator,
+         typename Randomizer,
+         typename Creator,
+         typename Derived>
+struct Triangle_structure_sampler_base
+{
+  const NamedParameters np;
+  GeomTraits gt;
+  PointOutputIterator& out;
+
+  Triangle_structure_sampler_base(PointOutputIterator& out,
+                                  const NamedParameters& np)
+    : np(np), out(out)
+  {}
+
+  void sample_points();
+  double get_minimum_edge_length();
+  template<typename Tr>
+  double get_tr_area(const Tr&);
+
+  template<typename Tr>
+  std::array<typename GeomTraits::Point_3, 3> get_tr_points(const Tr& tr);
+
+  void ms_edges_sample(const std::size_t& nb_points_per_edge,
+                       const std::size_t& nb_pts_l_u);
+  void ru_edges_sample();
+  void internal_sample_triangles(double, bool, bool);
+
+  Randomizer get_randomizer();
+  std::pair<TriangleIterator, TriangleIterator> get_range();
+  std::size_t get_points_size();
+
+  void procede()
+  {
+    using parameters::choose_parameter;
+    using parameters::get_parameter;
+    using parameters::is_default_parameter;
+
+    gt = choose_parameter<GeomTraits>(get_parameter(np, internal_np::geom_traits));
+
+
+    bool use_rs = choose_parameter(get_parameter(np, internal_np::random_uniform_sampling), true);
+    bool use_gs = choose_parameter(get_parameter(np, internal_np::grid_sampling), false);
+    bool use_ms = choose_parameter(get_parameter(np, internal_np::monte_carlo_sampling), false);
+
+    if(use_gs || use_ms)
+      if(is_default_parameter(get_parameter(np, internal_np::random_uniform_sampling)))
+        use_rs = false;
+
+    bool smpl_vrtcs = choose_parameter(get_parameter(np, internal_np::do_sample_vertices), true);
+    bool smpl_dgs = choose_parameter(get_parameter(np, internal_np::do_sample_edges), true);
+    bool smpl_fcs = choose_parameter(get_parameter(np, internal_np::do_sample_faces), true);
+    double nb_pts_a_u = choose_parameter(get_parameter(np, internal_np::nb_points_per_area_unit), 0.);
+    double nb_pts_l_u = choose_parameter(get_parameter(np, internal_np::nb_points_per_distance_unit), 0.);
+
+    // sample vertices
+    if(smpl_vrtcs)
+      static_cast<Derived*>(this)->sample_points();
+
+    // grid sampling
+    if(use_gs)
+    {
+      double grid_spacing_ = choose_parameter(get_parameter(np, internal_np::grid_spacing), 0.);
+
+      if(grid_spacing_ == 0.)
+      {
+        // set grid spacing to the shortest edge length
+        grid_spacing_ = static_cast<Derived*>(this)->get_minimum_edge_length();
+      }
+
+      static_cast<Derived*>(this)->internal_sample_triangles(grid_spacing_, smpl_fcs, smpl_dgs);
+    }
+
+    // monte carlo sampling
+    if(use_ms)
+    {
+      double min_sq_edge_length = (std::numeric_limits<double>::max)();
+
+      std::size_t nb_points_per_face =
+          choose_parameter(get_parameter(np, internal_np::number_of_points_per_face), 0);
+
+      std::size_t nb_points_per_edge =
+          choose_parameter(get_parameter(np, internal_np::number_of_points_per_edge), 0);
+
+      if((nb_points_per_face == 0 && nb_pts_a_u == 0.) ||
+         (nb_points_per_edge == 0 && nb_pts_l_u == 0.))
+      {
+        min_sq_edge_length = static_cast<Derived*>(this)->get_minimum_edge_length();
+      }
+
+      // sample faces
+      if(smpl_fcs)
+      {
+        // set default value
+        if(nb_points_per_face == 0 && nb_pts_a_u == 0.)
+          nb_pts_a_u = 2. / min_sq_edge_length;
+
+        for(const auto& tr : make_range(static_cast<Derived*>(this)->get_range()))
+        {
+          std::size_t nb_points = nb_points_per_face;
+          if(nb_points == 0)
+          {
+            nb_points = (std::max)(
+                  static_cast<std::size_t>(
+                    std::ceil(static_cast<Derived*>(this)->get_tr_area(tr))
+                    *nb_pts_a_u), std::size_t(1));
+          }
+
+          // extract triangle face points
+          std::array<typename GeomTraits::Point_3, 3>points = static_cast<Derived*>(this)->get_tr_points(tr);
+
+          Random_points_in_triangle_3<typename GeomTraits::Point_3, Creator> g(points[0], points[1], points[2]);
+          out = std::copy_n(g, nb_points, out);
+        }
+      }
+
+      // sample edges
+      if(smpl_dgs)
+        static_cast<Derived*>(this)->ms_edges_sample(nb_points_per_edge, nb_pts_l_u);
+    }
+
+    // random uniform sampling
+    if(use_rs)
+    {
+      // sample faces
+      if(smpl_fcs)
+      {
+        std::size_t nb_points
+            = choose_parameter(get_parameter(np, internal_np::number_of_points_on_faces), 0);
+
+        typename Derived::Randomizer g = static_cast<Derived*>(this)->get_randomizer();
+        if(nb_points == 0)
+        {
+          if(nb_pts_a_u == 0.)
+            nb_points = static_cast<Derived*>(this)->get_points_size();
+          else
+            nb_points = static_cast<std::size_t>(std::ceil(g.sum_of_weights()*nb_pts_a_u));
+        }
+        out = std::copy_n(g, nb_points, out);
+      }
+
+      // sample edges
+      if(smpl_dgs)
+        static_cast<Derived*>(this)->ru_edges_sample(nb_pts_l_u,nb_pts_a_u);
+    }
+  }
+};
+
+} // namespace internal
 
 template <class Kernel,
           class FaceRange,
           class TriangleMesh,
           class VertexPointMap,
-          class OutputIterator>
-OutputIterator
+          class PointOutputIterator>
+PointOutputIterator
 sample_triangles(const FaceRange& triangles,
                  const TriangleMesh& tm,
                  VertexPointMap vpm,
                  double distance,
-                 OutputIterator out,
+                 PointOutputIterator out,
                  bool sample_faces,
                  bool sample_edges,
                  bool add_vertices)
@@ -183,346 +340,672 @@ sample_triangles(const FaceRange& triang
   {
     // sample edges but skip endpoints
     halfedge_descriptor hd = halfedge(fd, tm);
-    for (int i=0;i<3; ++i)
+    for(int i=0;i<3; ++i)
     {
-      if (sample_edges && sampled_edges.insert(edge(hd, tm)).second )
+      if(sample_edges && sampled_edges.insert(edge(hd, tm)).second )
       {
         Point_ref p0 = get(vpm, source(hd, tm));
         Point_ref p1 = get(vpm, target(hd, tm));
         typename Kernel::Compute_squared_distance_3 squared_distance;
-        const double d_p0p1 = to_double(approximate_sqrt( squared_distance(p0, p1) ));
+        const double d_p0p1 = to_double(approximate_sqrt(squared_distance(p0, p1)));
 
         const double nb_pts = std::ceil( d_p0p1 / distance );
         const Vector_3 step_vec =  typename Kernel::Construct_scaled_vector_3()(
           typename Kernel::Construct_vector_3()(p0, p1),
           typename Kernel::FT(1)/typename Kernel::FT(nb_pts));
-        for (double i=1; i<nb_pts; ++i)
+        for(double i=1; i<nb_pts; ++i)
         {
           *out++=typename Kernel::Construct_translated_point_3()(p0,
             typename Kernel::Construct_scaled_vector_3()(step_vec ,
               typename Kernel::FT(i)));
         }
       }
+
       //add endpoints once
-      if ( add_vertices && endpoints.insert(target(hd, tm)).second )
-        *out++=get(vpm, target(hd, tm));
-      hd=next(hd, tm);
+      if(add_vertices && endpoints.insert(target(hd, tm)).second)
+        *out++ = get(vpm, target(hd, tm));
+
+      hd = next(hd, tm);
     }
 
     // sample triangles
-    if (sample_faces)
+    if(sample_faces)
     {
       Point_ref p0 = get(vpm, source(hd, tm));
       Point_ref p1 = get(vpm, target(hd, tm));
       Point_ref p2 = get(vpm, target(next(hd, tm), tm));
-      out=internal::triangle_grid_sampling<Kernel>(p0, p1, p2, distance, out);
+      out = internal::triangle_grid_sampling<Kernel>(p0, p1, p2, distance, out);
     }
   }
   return out;
 }
 
-/** \ingroup PMP_distance_grp
- * generates points taken on `tm` and outputs them to `out`, the sampling method
- * is selected using named parameters.
- * @tparam TriangleMesh a model of the concept `FaceListGraph`
- * @tparam OutputIterator a model of `OutputIterator`
- *  holding objects of the same point type as
- *  the value type of the internal vertex point map of `tm`
- *
- * @param tm the triangle mesh that will be sampled
- * @param out output iterator to be filled with sampled points
- * @param np an optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
- *
- * \cgalNamedParamsBegin
- *    \cgalParamBegin{vertex_point_map} the property map with the points
- *      associated to the vertices of `tm`. If this parameter is omitted,
- *      an internal property map for `CGAL::vertex_point_t`
- *      must be available for `TriangleMesh`.
- *    \cgalParamEnd
- *    \cgalParamBegin{geom_traits} a model of `PMPDistanceTraits`. \cgalParamEnd
- *    \cgalParamBegin{use_random_uniform_sampling}
- *      if `true` is passed (the default), points are generated in a random
- *      and uniform way on the surface of `tm`, and/or on edges of `tm`.
- *      For faces, the number of sample points is the value passed to the named
- *      parameter `number_of_points_on_faces()`. If not set,
- *      the value passed to the named parameter `number_of_points_per_area_unit()`
- *      is multiplied by the area of `tm` to get the number of sample points.
- *      If none of these parameters is set, the number of points sampled is `num_vertices(tm)`.
- *      For edges, the number of the number of sample points is the value passed to the named
- *      parameter `number_of_points_on_edges()`. If not set,
- *      the value passed to the named parameter `number_of_points_per_distance_unit()`
- *      is multiplied by the sum of the length of edges of `tm` to get the number of sample points.
- *      If none of these parameters is set, the number of points sampled is `num_vertices(tm)`.
- *    \cgalParamEnd
- *    \cgalParamBegin{use_grid_sampling}
- *      if `true` is passed, points are generated on a grid in each triangle,
- *      with a minimum of one point per triangle. The distance between
- *      two consecutive points in the grid is that of the length of the
- *      smallest non-null edge of `tm` or the value passed to
- *      the named parameter `grid_spacing()`. Edges are also split using the
- *      same distance, if requested.
- *    \cgalParamEnd
- *    \cgalParamBegin{use_monte_carlo_sampling}
- *      if `true` is passed, points are generated randomly in each triangle and/or
- *      on each edge.
- *      For faces, the number of points per triangle is the value passed to the named
- *      parameter `number_of_points_per_face()`. If not set, the value passed
- *      to the named parameter `number_of_points_per_area_unit()` is
- *      used to pick a number of points per face proportional to the triangle
- *      area with a minimum of one point per face. If none of these parameters
- *      is set, 2 divided by the square of the length of the smallest non-null
- *      edge of `tm` is used as if it was passed to
- *      `number_of_points_per_area_unit()`.
- *      For edges, the number of points per edge is the value passed to the named
- *      parameter `number_of_points_per_edge()`. If not set, the value passed
- *      to the named parameter `number_of_points_per_distance_unit()` is
- *      used to pick a number of points per edge proportional to the length of
- *      the edge with a minimum of one point per face. If none of these parameters
- *      is set, 1 divided by the length of the smallest non-null edge of `tm`
- *      is used as if it was passed to `number_of_points_per_distance_unit()`.
- *    \cgalParamEnd
- *    \cgalParamBegin{sample_vertices} if `true` is passed (default value),
- *    vertices of `tm` are put into `out`.\cgalParamEnd
- *    \cgalParamBegin{sample_edges} if `true` is passed (default value),
- *    edges of `tm` are sampled.\cgalParamEnd
- *    \cgalParamBegin{sample_faces} if `true` is passed (default value),
- *    faces of `tm` are sampled.\cgalParamEnd
- *    \cgalParamBegin{grid_spacing} a double value used as the grid spacing
- *      for the grid sampling method.
- *    \cgalParamEnd
- *    \cgalParamBegin{number_of_points_on_edges} an unsigned integral value used
- *      for the random sampling method as the number of points to pick exclusively
- *      on edges.
- *    \cgalParamEnd
- *    \cgalParamBegin{number_of_points_on_faces} an unsigned integral value used
- *      for the random sampling method as the number of points to pick on the surface.
- *    \cgalParamEnd
- *    \cgalParamBegin{number_of_points_per_distance_unit} a double value
- *       used for the random sampling and the Monte Carlo sampling methods to
- *       repectively determine the total number of points on edges and the
- *       number of points per edge.
- *    \cgalParamEnd
- *    \cgalParamBegin{number_of_points_per_edge} an unsigned integral value
- *      used by the Monte-Carlo sampling method as the number of points per edge
- *      to pick.
- *    \cgalParamEnd
- *    \cgalParamBegin{number_of_points_per_face} an unsigned integral value
- *      used by the Monte-Carlo sampling method as the number of points per face
- *      to pick.
- *    \cgalParamEnd
- *    \cgalParamBegin{number_of_points_per_area_unit} a double value
- *       used for the random sampling and the Monte Carlo sampling methods to
- *       repectively determine the total number of points inside faces
- *       and the number of points per face.
- *    \cgalParamEnd
- * \cgalNamedParamsEnd
- */
-template<class OutputIterator, class TriangleMesh, class NamedParameters>
-OutputIterator
-sample_triangle_mesh(const TriangleMesh& tm,
-                           OutputIterator out,
-                           NamedParameters np)
-{
-  typedef typename GetGeomTraits<TriangleMesh,
-          NamedParameters>::type Geom_traits;
+namespace internal {
 
-  typedef typename GetVertexPointMap<TriangleMesh,
-          NamedParameters>::const_type Vpm;
+template<typename Mesh,
+         typename PointOutputIterator,
+         typename GeomTraits,
+         typename Creator,
+         typename Vpm,
+         typename NamedParameters>
+struct Triangle_structure_sampler_for_triangle_mesh
+    : Triangle_structure_sampler_base<PointOutputIterator,
+                                      GeomTraits,
+                                      NamedParameters,
+                                      typename boost::graph_traits<Mesh>::face_iterator,
+                                      Random_points_in_triangle_mesh_3<Mesh, Vpm, Creator>,
+                                      Creator,
+                                      Triangle_structure_sampler_for_triangle_mesh<Mesh,
+                                                                                   PointOutputIterator,
+                                                                                   GeomTraits,
+                                                                                   Creator,
+                                                                                   Vpm,
+                                                                                   NamedParameters> >
+{
+  typedef Triangle_structure_sampler_for_triangle_mesh<Mesh,
+                                                       PointOutputIterator,
+                                                       GeomTraits,
+                                                       Creator, Vpm,
+                                                       NamedParameters>     Self;
+  typedef Triangle_structure_sampler_base<PointOutputIterator,
+                                          GeomTraits,
+                                          NamedParameters,
+                                          typename boost::graph_traits<Mesh>::face_iterator,
+                                          Random_points_in_triangle_mesh_3<Mesh, Vpm, Creator>,
+                                          Creator,
+                                          Self>                             Base;
+
+  typedef boost::graph_traits<Mesh>                                         GT;
+  typedef typename GT::halfedge_descriptor                                  halfedge_descriptor;
+  typedef typename GT::edge_descriptor                                      edge_descriptor;
+  typedef typename GT::face_descriptor                                      face_descriptor;
+
+  typedef Random_points_in_triangle_mesh_3<Mesh, Vpm,Creator>               Randomizer;
+  typedef typename boost::graph_traits<Mesh>::face_iterator                 TriangleIterator;
+
+  Vpm pmap;
+  double min_sq_edge_length;
+  const Mesh& tm;
+
+  Triangle_structure_sampler_for_triangle_mesh(const Mesh& m,
+                                               PointOutputIterator& out,
+                                               const NamedParameters& np)
+    : Base(out, np), tm(m)
+  {
+    using parameters::choose_parameter;
+    using parameters::get_parameter;
 
-  typedef boost::graph_traits<TriangleMesh> GT;
-  typedef typename GT::face_descriptor face_descriptor;
-  typedef typename GT::halfedge_descriptor halfedge_descriptor;
-  typedef typename GT::edge_descriptor edge_descriptor;
+    pmap = choose_parameter(get_parameter(np, internal_np::vertex_point),
+                            get_const_property_map(vertex_point, tm));
+    min_sq_edge_length = (std::numeric_limits<double>::max)();
+  }
 
-  using parameters::choose_parameter;
-  using parameters::get_parameter;
-  using parameters::is_default_parameter;
+  std::pair<TriangleIterator, TriangleIterator> get_range()
+  {
+    return std::make_pair(faces(tm).begin(), faces(tm).end());
+  }
 
-  Vpm pmap = choose_parameter(get_parameter(np, internal_np::vertex_point),
-                          get_const_property_map(vertex_point, tm));
-  typedef Creator_uniform_3<typename Geom_traits::FT,
-                            typename Geom_traits::Point_3> Creator;
+  void sample_points()
+  {
+    Property_map_to_unary_function<Vpm> unary(pmap);
+    this->out = std::copy(boost::make_transform_iterator(boost::begin(vertices(tm)), unary),
+                          boost::make_transform_iterator(boost::end(vertices(tm)), unary),
+                          this->out);
+  }
 
-  Geom_traits geomtraits = choose_parameter(get_parameter(np, internal_np::geom_traits), Geom_traits());
+  double get_minimum_edge_length()
+  {
+    typedef typename boost::graph_traits<Mesh>::edge_descriptor edge_descriptor;
 
+    if(min_sq_edge_length != (std::numeric_limits<double>::max)())
+      return min_sq_edge_length;
 
-  bool use_rs = choose_parameter(get_parameter(np, internal_np::random_uniform_sampling), true);
-  bool use_gs = choose_parameter(get_parameter(np, internal_np::grid_sampling), false);
-  bool use_ms = choose_parameter(get_parameter(np, internal_np::monte_carlo_sampling), false);
+    for(edge_descriptor ed : edges(tm))
+    {
+      const double sq_el = CGAL::to_double(
+                             typename GeomTraits::Compute_squared_distance_3()(get(pmap, source(ed, tm)),
+                                                                               get(pmap, target(ed, tm))));
 
-  if (use_gs || use_ms)
-    if (is_default_parameter(get_parameter(np, internal_np::random_uniform_sampling)))
-      use_rs=false;
+      if(sq_el > 0. && sq_el < min_sq_edge_length)
+        min_sq_edge_length = sq_el;
+    }
 
-  bool smpl_vrtcs = choose_parameter(get_parameter(np, internal_np::do_sample_vertices), true);
-  bool smpl_dgs = choose_parameter(get_parameter(np, internal_np::do_sample_edges), true);
-  bool smpl_fcs = choose_parameter(get_parameter(np, internal_np::do_sample_faces), true);
+    return min_sq_edge_length;
+  }
 
-  double nb_pts_a_u = choose_parameter(get_parameter(np, internal_np::nb_points_per_area_unit), 0.);
-  double nb_pts_l_u = choose_parameter(get_parameter(np, internal_np::nb_points_per_distance_unit), 0.);
+  double get_tr_area(const typename boost::graph_traits<Mesh>::face_descriptor& tr)
+  {
+    return to_double(face_area(tr,tm,parameters::geom_traits(this->gt)));
+  }
 
-  // sample vertices
-  if (smpl_vrtcs)
+  template<typename Tr>//tr = face_descriptor here
+  std::array<typename GeomTraits::Point_3, 3> get_tr_points(const Tr& tr)
   {
-    Property_map_to_unary_function<Vpm> unary(pmap);
-    out = std::copy(
-      boost::make_transform_iterator(boost::begin(vertices(tm)), unary),
-      boost::make_transform_iterator(boost::end(vertices(tm)), unary),
-      out);
-  }
-
-  // grid sampling
-  if (use_gs)
-  {
-    double grid_spacing_ = choose_parameter(get_parameter(np, internal_np::grid_spacing), 0.);
-    if (grid_spacing_==0.)
-    {
-      // set grid spacing to the shortest edge length
-      double grid_spacing_ = (std::numeric_limits<double>::max)();
-      typedef typename boost::graph_traits<TriangleMesh>
-        ::edge_descriptor edge_descriptor;
-      for(edge_descriptor ed : edges(tm))
-      {
-        double el = std::sqrt(
-          to_double( typename Geom_traits::Compute_squared_distance_3()(
-            get(pmap, source(ed, tm)), get(pmap, target(ed, tm)) )));
-        if (el > 0 && el < grid_spacing_)
-          grid_spacing_ = el;
-      }
+    std::array<typename GeomTraits::Point_3, 3> points;
+    halfedge_descriptor hd(halfedge(tr,tm));
+    for(int i=0; i<3; ++i)
+    {
+      points[i] = get(pmap, target(hd, tm));
+      hd = next(hd, tm);
     }
-    out=sample_triangles<Geom_traits>(
-          faces(tm), tm, pmap, grid_spacing_, out,smpl_fcs, smpl_dgs, false);
+    return points;
   }
 
-  // monte carlo sampling
-  if (use_ms)
+  void ms_edges_sample(std::size_t nb_points_per_edge,
+                       double nb_pts_l_u)
   {
-    typename Geom_traits::Compute_squared_distance_3 squared_distance;
-    double min_edge_length = (std::numeric_limits<double>::max)();
+    typename GeomTraits::Compute_squared_distance_3 squared_distance = this->gt.compute_squared_distance_3_object();
 
-    std::size_t nb_points_per_face =
-      choose_parameter(get_parameter(np, internal_np::number_of_points_per_face), 0);
-    std::size_t nb_points_per_edge =
-      choose_parameter(get_parameter(np, internal_np::number_of_points_per_edge), 0);
+    if(nb_points_per_edge == 0 && nb_pts_l_u == 0.)
+      nb_pts_l_u = 1. / CGAL::sqrt(min_sq_edge_length);
 
-    if ((nb_points_per_face == 0 && nb_pts_a_u ==0.) ||
-        (nb_points_per_edge == 0 && nb_pts_l_u ==0.) )
+    for(edge_descriptor ed : edges(tm))
     {
-      typedef typename boost::graph_traits<TriangleMesh>
-        ::edge_descriptor edge_descriptor;
-      for(edge_descriptor ed : edges(tm))
+      std::size_t nb_points = nb_points_per_edge;
+      if(nb_points == 0)
       {
-        double el = std::sqrt(
-          to_double( squared_distance(get(pmap, source(ed, tm)),
-                                      get(pmap, target(ed, tm)) )));
-        if (min_edge_length > 0 && el < min_edge_length)
-          min_edge_length = el;
+        nb_points = (std::max)(
+          static_cast<std::size_t>(std::ceil(std::sqrt(to_double(
+           squared_distance(get(pmap, source(ed, tm)),
+                            get(pmap, target(ed, tm))))) * nb_pts_l_u)),
+          std::size_t(1));
       }
+
+      // now do the sampling of the edge
+      Random_points_on_segment_3<typename GeomTraits::Point_3, Creator>
+        g(get(pmap, source(ed,tm)), get(pmap, target(ed, tm)));
+      this->out = std::copy_n(g, nb_points, this->out);
     }
+  }
+  void ru_edges_sample(double nb_pts_l_u,
+                       double nb_pts_a_u)
+  {
+    using parameters::choose_parameter;
+    using parameters::get_parameter;
 
-    // sample faces
-    if (smpl_fcs)
+    std::size_t nb_points = choose_parameter(get_parameter(this->np, internal_np::number_of_points_on_edges), 0);
+    Random_points_on_edge_list_graph_3<Mesh, Vpm, Creator> g(tm, pmap);
+    if(nb_points == 0)
     {
-      // set default value
-      if (nb_points_per_face == 0 && nb_pts_a_u ==0.)
-        nb_pts_a_u = 2. / CGAL::square(min_edge_length);
-
-      for(face_descriptor f : faces(tm))
-      {
-        std::size_t nb_points = nb_points_per_face;
-        if (nb_points == 0)
-        {
-          nb_points = (std::max)(
-            static_cast<std::size_t>(
-              std::ceil(to_double(
-                face_area(f,tm,parameters::geom_traits(geomtraits)))*nb_pts_a_u))
-          ,std::size_t(1));
-        }
-        // extract triangle face points
-        typename Geom_traits::Point_3 points[3];
-        halfedge_descriptor hd(halfedge(f,tm));
-        for(int i=0; i<3; ++i)
-        {
-          points[i] = get(pmap, target(hd, tm));
-          hd = next(hd, tm);
-        }
-        // sample the triangle face
-        Random_points_in_triangle_3<typename Geom_traits::Point_3, Creator>
-          g(points[0], points[1], points[2]);
-        out=std::copy_n(g, nb_points, out);
-      }
+      if(nb_pts_l_u == 0)
+        nb_points = num_vertices(tm);
+      else
+        nb_points = static_cast<std::size_t>(std::ceil(g.mesh_length() * nb_pts_a_u));
     }
-    // sample edges
-    if (smpl_dgs)
+    this->out = std::copy_n(g, nb_points, this->out);
+  }
+
+  Randomizer get_randomizer()
+  {
+    return Randomizer(tm, pmap);
+  }
+
+  void internal_sample_triangles(double grid_spacing_, bool smpl_fcs, bool smpl_dgs)
+  {
+    this->out = sample_triangles<GeomTraits>(faces(tm), tm, pmap, grid_spacing_, this->out, smpl_fcs, smpl_dgs, false);
+  }
+
+  std::size_t get_points_size()
+  {
+    return num_vertices(tm);
+  }
+};
+
+template<typename PointRange,
+         typename TriangleRange,
+         typename PointOutputIterator,
+         typename GeomTraits,
+         typename Creator,
+         typename NamedParameters>
+struct Triangle_structure_sampler_for_triangle_soup
+    : Triangle_structure_sampler_base<PointOutputIterator,
+                                      GeomTraits,
+                                      NamedParameters,
+                                      typename TriangleRange::const_iterator,
+                                      Random_points_in_triangle_soup<PointRange,
+                                                                     typename TriangleRange::value_type,
+                                                                     Creator>,
+                                      Creator,
+                                      Triangle_structure_sampler_for_triangle_soup<PointRange,
+                                                                                   TriangleRange,
+                                                                                   PointOutputIterator,
+                                                                                   GeomTraits,
+                                                                                   Creator,
+                                                                                   NamedParameters> >
+{
+  typedef typename TriangleRange::value_type                                TriangleType;
+  typedef Triangle_structure_sampler_for_triangle_soup<PointRange,
+                                                       TriangleRange,
+                                                       PointOutputIterator,
+                                                       GeomTraits,
+                                                       Creator,
+                                                       NamedParameters>     Self;
+
+  typedef Triangle_structure_sampler_base<PointOutputIterator,
+                                          GeomTraits,
+                                          NamedParameters,
+                                          typename TriangleRange::const_iterator,
+                                          Random_points_in_triangle_soup<PointRange, TriangleType, Creator>,
+                                          Creator,
+                                          Self>                             Base;
+
+  typedef typename GeomTraits::Point_3                                      Point_3;
+
+  typedef Random_points_in_triangle_soup<PointRange, TriangleType, Creator> Randomizer;
+  typedef typename TriangleRange::const_iterator                            TriangleIterator;
+
+  double min_sq_edge_length;
+  const PointRange& points;
+  const TriangleRange& triangles;
+
+  Triangle_structure_sampler_for_triangle_soup(const PointRange& pts,
+                                               const TriangleRange& trs,
+                                               PointOutputIterator& out,
+                                               const NamedParameters& np)
+    : Base(out, np), points(pts), triangles(trs)
+  {
+    min_sq_edge_length = (std::numeric_limits<double>::max)();
+  }
+
+  std::pair<TriangleIterator, TriangleIterator> get_range()
+  {
+    return std::make_pair(triangles.begin(), triangles.end());
+  }
+
+  void sample_points()
+  {
+    this->out = std::copy(points.begin(), points.end(), this->out);
+  }
+
+  double get_minimum_edge_length()
+  {
+    if(min_sq_edge_length != (std::numeric_limits<double>::max)())
+      return min_sq_edge_length;
+
+    for(const auto& tr : triangles)
     {
-      if (nb_points_per_edge == 0 && nb_pts_l_u == 0)
-        nb_pts_l_u = 1. / min_edge_length;
-      for(edge_descriptor ed : edges(tm))
+      for(std::size_t i = 0; i< 3; ++i)
       {
-        std::size_t nb_points = nb_points_per_edge;
-        if (nb_points == 0)
-        {
-          nb_points = (std::max)(
-            static_cast<std::size_t>( std::ceil( std::sqrt( to_double(
-             squared_distance(get(pmap, source(ed, tm)),
-                              get(pmap, target(ed, tm)) )) )*nb_pts_l_u ) ),
-            std::size_t(1));
-        }
-        // now do the sampling of the edge
-        Random_points_on_segment_3<typename Geom_traits::Point_3, Creator>
-          g(get(pmap, source(ed,tm)), get(pmap, target(ed,tm)));
-        out=std::copy_n(g, nb_points, out);
+        const Point_3& a = points[tr[i]];
+        const Point_3& b = points[tr[(i+1)%3]];
+
+        const double sq_el = CGAL::to_double(typename GeomTraits::Compute_squared_distance_3()(a, b));
+        if(sq_el > 0. && sq_el < min_sq_edge_length)
+          min_sq_edge_length = sq_el;
       }
     }
+
+    return min_sq_edge_length;
+  }
+
+  template<typename Tr>
+  double get_tr_area(const Tr& tr)
+  {
+    return to_double(approximate_sqrt(
+                       this->gt.compute_squared_area_3_object()(
+                         points[tr[0]], points[tr[1]], points[tr[2]])));
   }
 
-  // random uniform sampling
-  if (use_rs)
+  template<typename Tr>
+  std::array<Point_3, 3> get_tr_points(const Tr& tr)
   {
-    // sample faces
-    if(smpl_fcs)
+    std::array<Point_3, 3> points;
+    for(int i=0; i<3; ++i)
     {
-      std::size_t nb_points = choose_parameter(get_parameter(np, internal_np::number_of_points_on_faces), 0);
-      Random_points_in_triangle_mesh_3<TriangleMesh, Vpm, Creator> g(tm, pmap);
-      if (nb_points == 0)
-      {
-        if (nb_pts_a_u == 0.)
-          nb_points = num_vertices(tm);
-        else
-          nb_points = static_cast<std::size_t>(
-            std::ceil(g.mesh_area()*nb_pts_a_u) );
-      }
-      out = std::copy_n(g, nb_points, out);
+      points[i] = this->points[tr[i]];
     }
-    // sample edges
-    if (smpl_dgs)
+    return points;
+  }
+
+  void ms_edges_sample(std::size_t, double)
+  {
+    // don't sample edges in soup.
+  }
+
+  void ru_edges_sample(double, double)
+  {
+    // don't sample edges in soup.
+  }
+
+  Randomizer get_randomizer()
+  {
+    return Randomizer(triangles, points);
+  }
+
+  void internal_sample_triangles(double distance, bool, bool)
+  {
+    for(const auto& tr : triangles)
     {
-      std::size_t nb_points =
-        choose_parameter(get_parameter(np, internal_np::number_of_points_on_edges), 0);
-      Random_points_on_edge_list_graph_3<TriangleMesh, Vpm, Creator> g(tm, pmap);
-      if (nb_points == 0)
-      {
-        if (nb_pts_l_u == 0)
-          nb_points = num_vertices(tm);
-        else
-          nb_points = static_cast<std::size_t>(
-            std::ceil( g.mesh_length()*nb_pts_a_u) );
-      }
-      out = std::copy_n(g, nb_points, out);
+      const Point_3& p0 = points[tr[0]];
+      const Point_3& p1 = points[tr[1]];
+      const Point_3& p2 = points[tr[2]];
+
+      this->out = internal::triangle_grid_sampling<GeomTraits>(p0, p1, p2, distance, this->out);
     }
   }
 
-  return out;
+  std::size_t get_points_size()
+  {
+    return points.size();
+  }
+};
+
+} // namespace internal
+
+/** \ingroup PMP_distance_grp
+ *
+ * generates points on `tm` and outputs them to `out`; the sampling method
+ * is selected using named parameters.
+ *
+ * @tparam TriangleMesh a model of the concepts `EdgeListGraph` and `FaceListGraph`
+ * @tparam PointOutputIterator a model of `OutputIterator`
+ *  holding objects of the same point type as
+ *  the value type of the point type associated to the mesh `tm`, i.e. the value type of the vertex
+ *  point map property map, if provided, or the value type of the internal point property map otherwise
+ * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
+ *
+ * @param tm the triangle mesh to be sampled
+ * @param out output iterator to be filled with sample points
+ * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+ *
+ * \cgalNamedParamsBegin
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `tm`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+ *                    as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     must be available in `TriangleMesh`.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{geom_traits}
+ *     \cgalParamDescription{an instance of a geometric traits class}
+ *     \cgalParamType{a class model of `PMPDistanceTraits`}
+ *     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+ *     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{use_random_uniform_sampling}
+ *     \cgalParamDescription{If `true` is passed, points are generated in a random and uniform way
+ *                           on the surface of `tm`, and/or on edges of `tm`.}
+ *     \cgalParamType{Boolean}
+ *     \cgalParamType{`true`}
+ *     \cgalParamExtra{For faces, the number of sample points is the value passed to the named
+ *                     parameter `number_of_points_on_faces`. If not set,
+ *                     the value passed to the named parameter `number_of_points_per_area_unit`
+ *                     is multiplied by the area of `tm` to get the number of sample points.
+ *                     If none of these parameters is set, the number of points sampled is `num_vertices(tm)`.
+ *                     For edges, the number of the number of sample points is the value passed to the named
+ *                     parameter `number_of_points_on_edges`. If not set,
+ *                     the value passed to the named parameter `number_of_points_per_distance_unit`
+ *                     is multiplied by the sum of the length of edges of `tm` to get the number of sample points.
+ *                     If none of these parameters is set, the number of points sampled is `num_vertices(tm)`.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{use_grid_sampling}
+ *     \cgalParamDescription{If `true` is passed, points are generated on a grid in each triangle,
+ *                           with a minimum of one point per triangle.}
+ *     \cgalParamType{Boolean}
+ *     \cgalParamDefault{`false`}
+ *     \cgalParamExtra{The distance between two consecutive points in the grid is that of the length
+ *                     of the smallest non-null edge of `tm` or the value passed to the named parameter
+ *                     `grid_spacing`. Edges are also split using the same distance, if requested.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{use_monte_carlo_sampling}
+ *     \cgalParamDescription{if `true` is passed, points are generated randomly in each triangle and/or on each edge.}
+ *     \cgalParamType{Boolean}
+ *     \cgalParamDefault{`false`}
+ *     \cgalParamExtra{For faces, the number of points per triangle is the value passed to the named
+ *                     parameter `number_of_points_per_face`. If not set, the value passed
+ *                     to the named parameter `number_of_points_per_area_unit` is
+ *                     used to pick a number of points per face proportional to the triangle
+ *                     area with a minimum of one point per face. If none of these parameters
+ *                     is set, 2 divided by the square of the length of the smallest non-null
+ *                     edge of `tm` is used as if it was passed to
+ *                     `number_of_points_per_area_unit`.
+ *                     For edges, the number of points per edge is the value passed to the named
+ *                     parameter `number_of_points_per_edge`. If not set, the value passed
+ *                     to the named parameter `number_of_points_per_distance_unit` is
+ *                     used to pick a number of points per edge proportional to the length of
+ *                     the edge with a minimum of one point per face. If none of these parameters
+ *                     is set, 1 divided by the length of the smallest non-null edge of `tm`
+ *                     is used as if it was passed to `number_of_points_per_distance_unit`.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{sample_vertices}
+ *     \cgalParamDescription{If `true` is passed, the vertices of `tm` are part of the sample.}
+ *     \cgalParamType{Boolean}
+ *     \cgalParamDefault{`true`}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{sample_edges}
+ *     \cgalParamDescription{If `true` is passed, edges of `tm` are sampled.}
+ *     \cgalParamType{Boolean}
+ *     \cgalParamDefault{`true`}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{sample_faces}
+ *     \cgalParamDescription{If `true` is passed, faces of `tm` are sampled.}
+ *     \cgalParamType{Boolean}
+ *     \cgalParamDefault{`true`}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{grid_spacing}
+ *     \cgalParamDescription{a value used as the grid spacing for the grid sampling method}
+ *     \cgalParamType{double}
+ *     \cgalParamDefault{the length of the shortest, non-degenerate edge of `tm`}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{number_of_points_on_edges}
+ *     \cgalParamDescription{a value used for the random sampling method as the number of points to pick exclusively on edges}
+ *     \cgalParamType{unsigned int}
+ *     \cgalParamDefault{`num_vertices(tm)` or a value based on `nb_points_per_distance_unit`, if it is defined}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{number_of_points_on_faces}
+ *     \cgalParamDescription{a value used for the random sampling method as the number of points to pick on the surface}
+ *     \cgalParamType{unsigned int}
+ *     \cgalParamDefault{`num_vertices(tm)` or a value based on `nb_points_per_area_unit`, if it is defined}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{number_of_points_per_distance_unit}
+ *     \cgalParamDescription{a value used for the random sampling and the Monte Carlo sampling methods to
+ *                           respectively determine the total number of points on edges and the number of points per edge}
+ *     \cgalParamType{double}
+ *     \cgalParamDefault{`1` divided by the length of the shortest, non-degenerate edge of `tm`}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{number_of_points_per_edge}
+ *     \cgalParamDescription{a value used by the Monte-Carlo sampling method as the number of points per edge to pick}
+ *     \cgalParamType{unsigned int}
+ *     \cgalParamDefault{`0`}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{number_of_points_per_area_unit}
+ *     \cgalParamDescription{a value used for the random sampling and the Monte Carlo sampling methods to
+ *                           respectively determine the total number of points inside faces and the number of points per face}
+ *     \cgalParamType{double}
+ *     \cgalParamDefault{`2` divided by the squared length of the shortest, non-degenerate edge of `tm`}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{number_of_points_per_face}
+ *     \cgalParamDescription{a value used by the Monte-Carlo sampling method as the number of points per face to pick}
+ *     \cgalParamType{unsigned int}
+ *     \cgalParamDefault{`0`}
+ *   \cgalParamNEnd
+ * \cgalNamedParamsEnd
+ *
+ * @see `CGAL::Polygon_mesh_processing::sample_triangle_soup()`
+ */
+template<class PointOutputIterator, class TriangleMesh, class NamedParameters>
+PointOutputIterator
+sample_triangle_mesh(const TriangleMesh& tm,
+                     PointOutputIterator out,
+                     const NamedParameters& np)
+{
+  typedef typename GetGeomTraits<TriangleMesh, NamedParameters>::type             GeomTraits;
+  typedef typename GetVertexPointMap<TriangleMesh, NamedParameters>::const_type   Vpm;
+
+  internal::Triangle_structure_sampler_for_triangle_mesh<TriangleMesh,
+      PointOutputIterator,
+      GeomTraits,
+      Creator_uniform_3<typename GeomTraits::FT,
+      typename GeomTraits::Point_3>,
+      Vpm,
+      NamedParameters> performer(tm, out, np);
+  performer.procede();
+
+  return performer.out;
 }
 
-template<class OutputIterator, class TriangleMesh>
-OutputIterator
+/** \ingroup PMP_distance_grp
+ *
+ * generates points on a triangle soup and puts them to `out`; the sampling method
+ * is selected using named parameters.
+ *
+ * @tparam PointRange a model of the concept `RandomAccessContainer` whose value type is the point type.
+ * @tparam TriangleRange a model of the concept `RandomAccessContainer`
+ *                      whose value_type is itself a model of the concept `RandomAccessContainer`
+ *                      whose value_type is an unsigned integral value.
+ * @tparam PointOutputIterator a model of `OutputIterator` holding objects of the same type as `PointRange`'s value type
+ * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
+ *
+ * @param points the points of the soup
+ * @param triangles a `TriangleRange` containing the triangles of the soup to be sampled
+ * @param out output iterator to be filled with sample points
+ * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+ *
+ * \cgalNamedParamsBegin
+ *   \cgalParamNBegin{geom_traits}
+ *     \cgalParamDescription{an instance of a geometric traits class}
+ *     \cgalParamType{a class model of `PMPDistanceTraits`}
+ *     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+ *     \cgalParamExtra{The geometric traits class must be compatible with the point range's point type.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{use_random_uniform_sampling}
+ *     \cgalParamDescription{If `true` is passed, points are generated in a random and uniform way
+ *                           over the triangles of the soup.}
+ *     \cgalParamType{Boolean}
+ *     \cgalParamType{`true`}
+ *     \cgalParamExtra{The number of sample points is the value passed to the named
+ *                     parameter `number_of_points_on_faces`. If not set,
+ *                     the value passed to the named parameter `number_of_points_per_area_unit`
+ *                     is multiplied by the area of the soup to get the number of sample points.
+ *                     If none of these parameters is set, the number of points sampled is `points.size()`.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{use_grid_sampling}
+ *     \cgalParamDescription{If `true` is passed, points are generated on a grid in each triangle,
+ *                           with a minimum of one point per triangle.}
+ *     \cgalParamType{Boolean}
+ *     \cgalParamDefault{`false`}
+ *     \cgalParamExtra{The distance between two consecutive points in the grid is that of the length
+ *                     of the smallest non-null edge of the soup or the value passed to the named parameter
+ *                     `grid_spacing`.}
+ *   \cgalParamNEnd
+ * *   \cgalParamNBegin{use_monte_carlo_sampling}
+ *     \cgalParamDescription{if `true` is passed, points are generated randomly in each triangle.}
+ *     \cgalParamType{Boolean}
+ *     \cgalParamDefault{`false`}
+ *     \cgalParamExtra{The number of points per triangle is the value passed to the named
+ *                     parameter `number_of_points_per_face`. If not set, the value passed
+ *                     to the named parameter `number_of_points_per_area_unit` is
+ *                     used to pick a number of points per face proportional to the triangle
+ *                     area with a minimum of one point per face. If none of these parameters
+ *                     is set, the number of points per area unit is set to 2 divided
+ *                     by the square of the length of the smallest non-null edge of the soup.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{sample_vertices}
+ *     \cgalParamDescription{If `true` is passed, the points of `points` are part of the sample.}
+ *     \cgalParamType{Boolean}
+ *     \cgalParamDefault{`true`}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{sample_faces}
+ *     \cgalParamDescription{If `true` is passed, faces of the soup are sampled.}
+ *     \cgalParamType{Boolean}
+ *     \cgalParamDefault{`true`}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{grid_spacing}
+ *     \cgalParamDescription{a value used as the grid spacing for the grid sampling method}
+ *     \cgalParamType{double}
+ *     \cgalParamDefault{the length of the shortest, non-degenerate edge of the soup}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{number_of_points_on_faces}
+ *     \cgalParamDescription{a value used for the random sampling method as the number of points to pick on the surface}
+ *     \cgalParamType{unsigned int}
+ *     \cgalParamDefault{`points.size()` or a value based on `nb_points_per_area_unit`, if it is defined}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{number_of_points_per_face}
+ *     \cgalParamDescription{a value used by the Monte-Carlo sampling method as the number of points per face to pick}
+ *     \cgalParamType{unsigned int}
+ *     \cgalParamDefault{`0`}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{number_of_points_per_area_unit}
+ *     \cgalParamDescription{a value used for the random sampling and the Monte Carlo sampling methods to
+ *                           respectively determine the total number of points inside faces and the number of points per face}
+ *     \cgalParamType{double}
+ *     \cgalParamDefault{`2` divided by the squared length of the shortest, non-degenerate edge of the soup}
+ *   \cgalParamNEnd
+ * \cgalNamedParamsEnd
+ *
+ * \attention Contrary to `sample_triangle_mesh()`, this method does not allow to sample edges.
+ *
+ * @see `CGAL::Polygon_mesh_processing::sample_triangle_mesh()`
+ */
+template<class PointOutputIterator,
+         class TriangleRange,
+         class PointRange,
+         class NamedParameters>
+PointOutputIterator
+sample_triangle_soup(const PointRange& points,
+                     const TriangleRange& triangles,
+                     PointOutputIterator out,
+                     const NamedParameters& np)
+{
+  typedef typename PointRange::value_type         Point_3;
+  typedef typename Kernel_traits<Point_3>::Kernel GeomTraits;
+
+  static_assert(std::is_same<Point_3, typename GeomTraits::Point_3>::value, "Wrong point type.");
+
+  internal::Triangle_structure_sampler_for_triangle_soup<PointRange,
+      TriangleRange,
+      PointOutputIterator,
+      GeomTraits,
+      Creator_uniform_3<typename GeomTraits::FT,
+      typename GeomTraits::Point_3>,
+      NamedParameters> performer(points, triangles, out, np);
+  performer.procede();
+
+  return performer.out;
+}
+
+template<class PointOutputIterator, class TriangleMesh>
+PointOutputIterator
 sample_triangle_mesh(const TriangleMesh& tm,
-                           OutputIterator out)
+                     PointOutputIterator out)
 {
   return sample_triangle_mesh(tm, out, parameters::all_default());
 }
 
+template<class PointOutputIterator,
+         class TriangleRange,
+         class PointRange>
+PointOutputIterator
+sample_triangle_soup(const PointRange& points,
+                     const TriangleRange& triangles,
+                     PointOutputIterator out)
+{
+  return sample_triangle_soup(points, triangles, out, parameters::all_default());
+}
+
 template <class Concurrency_tag,
           class Kernel,
           class PointRange,
@@ -550,7 +1033,6 @@ double approximate_Hausdorff_distance(
 
   Tree tree( faces(tm).first, faces(tm).second, tm);
   tree.build();
-  tree.accelerate_distance_queries();
   Point_3 hint = get(vpm, *vertices(tm).first);
 
   return internal::approximate_Hausdorff_distance_impl<Concurrency_tag, Kernel>
@@ -563,15 +1045,13 @@ template <class Concurrency_tag, class K
 double approximate_Hausdorff_distance(
    const TriangleMesh& tm1,
    const TriangleMesh& tm2,
-   NamedParameters np,
+   const NamedParameters& np,
    VertexPointMap vpm_2)
 {
-    std::vector<typename Kernel::Point_3> sample_points;
-    sample_triangle_mesh(
-                tm1,
-                std::back_inserter(sample_points),
-                np);
-    return approximate_Hausdorff_distance<Concurrency_tag, Kernel>(sample_points, tm2, vpm_2);
+  std::vector<typename Kernel::Point_3> sample_points;
+  sample_triangle_mesh(tm1, std::back_inserter(sample_points), np);
+
+  return approximate_Hausdorff_distance<Concurrency_tag, Kernel>(sample_points, tm2, vpm_2);
 }
 
 // documented functions
@@ -590,26 +1070,30 @@ double approximate_Hausdorff_distance(
  * for more details.
  *
  * @tparam Concurrency_tag enables sequential versus parallel algorithm.
- *                         Possible values are `Sequential_tag`
- *                         and `Parallel_tag`.
- * @tparam TriangleMesh a model of the concept `FaceListGraph`
- * @tparam NamedParameters1 a sequence of \ref pmp_namedparameters "Named Parameters" for `tm1`
- * @tparam NamedParameters2 a sequence of \ref pmp_namedparameters "Named Parameters" for `tm2`
+ *                         Possible values are `Sequential_tag`, `Parallel_tag`, and `Parallel_if_available_tag`.
+ * @tparam TriangleMesh a model of the concepts `EdgeListGraph` and `FaceListGraph`
+ * @tparam NamedParameters1 a sequence of \ref bgl_namedparameters "Named Parameters" for `tm1`
+ * @tparam NamedParameters2 a sequence of \ref bgl_namedparameters "Named Parameters" for `tm2`
  *
  * @param tm1 the triangle mesh that will be sampled
  * @param tm2 the triangle mesh to compute the distance to
- * @param np1 optional sequence of \ref pmp_namedparameters "Named Parameters" for `tm1` passed to `sample_triangle_mesh()`.
+ * @param np1 an optional sequence of \ref bgl_namedparameters "Named Parameters" forwarded to `sample_triangle_mesh()`
  *
- * @param np2 optional sequence of \ref pmp_namedparameters "Named Parameters" for `tm2` among the ones listed below
+ * @param np2 an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `tm2`
- *      If this parameter is omitted, an internal property map for `CGAL::vertex_point_t` must be available in `TriangleMesh`
- *      and in all places where `vertex_point_map` is used.
- *    \cgalParamEnd
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `tm2`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+ *                    as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm2)`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     must be available in `TriangleMesh`.}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
- * The function `CGAL::parameters::all_default()` can be used to indicate to use the default values for
- * `np1` and specify custom values for `np2`
+ *
+ * The function `CGAL::parameters::all_default()` can be used to indicate to use the default values
+ * for `np1` and specify custom values for `np2`.
  */
 template< class Concurrency_tag,
           class TriangleMesh,
@@ -621,9 +1105,9 @@ double approximate_Hausdorff_distance( c
                                        const NamedParameters2& np2)
 {
   typedef typename GetGeomTraits<TriangleMesh,
-                                 NamedParameters1>::type Geom_traits;
+                                 NamedParameters1>::type GeomTraits;
 
-  return approximate_Hausdorff_distance<Concurrency_tag, Geom_traits>(
+  return approximate_Hausdorff_distance<Concurrency_tag, GeomTraits>(
     tm1, tm2, np1, parameters::choose_parameter(parameters::get_parameter(np2, internal_np::vertex_point),
                                 get_const_property_map(vertex_point, tm2)));
 }
@@ -652,21 +1136,32 @@ double approximate_symmetric_Hausdorff_d
 
 /**
  * \ingroup PMP_distance_grp
- * returns the distance to `tm` of the point from `points`
- * that is the furthest from `tm`.
+ * returns the distance to `tm` of the point from `points` that is the furthest from `tm`.
+ *
  * @tparam PointRange a range of `Point_3`, model of `Range`. Its iterator type is `RandomAccessIterator`.
- * @tparam TriangleMesh a model of the concept `FaceListGraph`
- * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+ * @tparam TriangleMesh a model of the concepts `EdgeListGraph` and `FaceListGraph`
+ * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
+ *
  * @param points the range of points of interest
  * @param tm the triangle mesh to compute the distance to
- * @param np an optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+ * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{vertex_point_map}
- *    the property map with the points associated to the vertices of `tm`. If this parameter is omitted,
- *    an internal property map for `CGAL::vertex_point_t` must be available for the
-      vertices of `tm` \cgalParamEnd
- *    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `PMPDistanceTraits`\cgalParamEnd
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `tm`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+ *                    as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     must be available in `TriangleMesh`.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{geom_traits}
+ *     \cgalParamDescription{an instance of a geometric traits class}
+ *     \cgalParamType{a class model of `PMPDistanceTraits`}
+ *     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+ *     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  */
 template< class Concurrency_tag,
@@ -678,9 +1173,9 @@ double max_distance_to_triangle_mesh(con
                                      const NamedParameters& np)
 {
   typedef typename GetGeomTraits<TriangleMesh,
-                                 NamedParameters>::type Geom_traits;
+                                 NamedParameters>::type GeomTraits;
 
-  return approximate_Hausdorff_distance<Concurrency_tag, Geom_traits>
+  return approximate_Hausdorff_distance<Concurrency_tag, GeomTraits>
      (points,tm,parameters::choose_parameter(parameters::get_parameter(np, internal_np::vertex_point),
                              get_const_property_map(vertex_point, tm)));
 }
@@ -688,22 +1183,34 @@ double max_distance_to_triangle_mesh(con
 /*!
  *\ingroup PMP_distance_grp
  * returns an approximation of the distance between `points` and the point lying on `tm` that is the farthest from `points`
+ *
  * @tparam PointRange a range of `Point_3`, model of `Range`.
  * @tparam TriangleMesh a model of the concept `FaceListGraph`
- * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+ * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
+ *
  * @param tm a triangle mesh
  * @param points a range of points
  * @param precision for each triangle of `tm`, the distance of its farthest point from `points` is bounded.
  *                  A triangle is subdivided into sub-triangles so that the difference of its distance bounds
  *                  is smaller than `precision`. `precision` must be strictly positive to avoid infinite loops.
- * @param np an optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+ * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{vertex_point_map}
- *    the property map with the points associated to the vertices of `tm`. If this parameter is omitted,
- *    an internal property map for `CGAL::vertex_point_t` must be available for the
-      vertices of `tm` \cgalParamEnd
- *    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `PMPDistanceTraits`. \cgalParamEnd
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `tm`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+ *                    as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     must be available in `TriangleMesh`.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{geom_traits}
+ *     \cgalParamDescription{an instance of a geometric traits class}
+ *     \cgalParamType{a class model of `PMPDistanceTraits`}
+ *     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+ *     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  */
 template< class TriangleMesh,
@@ -715,16 +1222,16 @@ double approximate_max_distance_to_point
                                              const NamedParameters& np)
 {
   typedef typename GetGeomTraits<TriangleMesh,
-                                 NamedParameters>::type Geom_traits;
+                                 NamedParameters>::type GeomTraits;
   typedef boost::graph_traits<TriangleMesh> GT;
 
-  typedef Orthogonal_k_neighbor_search<Search_traits_3<Geom_traits> > Knn;
+  typedef Orthogonal_k_neighbor_search<Search_traits_3<GeomTraits> > Knn;
   typedef typename Knn::Tree Tree;
   Tree tree(points.begin(), points.end());
-  CRefiner<Geom_traits> ref;
+  CRefiner<GeomTraits> ref;
   for(typename GT::face_descriptor f : faces(tm))
   {
-    typename Geom_traits::Point_3 points[3];
+    typename GeomTraits::Point_3 points[3];
     typename GT::halfedge_descriptor hd(halfedge(f,tm));
     for(int i=0; i<3; ++i)
     {
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/extrude.h 5.1-1/include/CGAL/Polygon_mesh_processing/extrude.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/extrude.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/extrude.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/extrude.h $
-// $Id: extrude.h ed2067f 2020-03-17T17:08:32+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/extrude.h $
+// $Id: extrude.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -103,6 +103,7 @@ struct Identity_functor
 
 /**
  * \ingroup PMP_meshing_grp
+ *
  * \brief performs a generalized extrusion of `input` and puts it in `output`.
  *
  * This function extrudes the open surface mesh `input` and puts the result in `output`. The mesh generated is a closed
@@ -112,11 +113,13 @@ struct Identity_functor
  * vertices of the bottom and top part are first initialized to the same value as the corresponding
  * vertices of `input`. Then for each vertex, a call to `bot` and `top` is done for the vertices of the
  * bottom part and the top part, respectively.
+ *
  * \attention `output` may be self intersecting.
+ *
  * @tparam InputMesh a model of `FaceListGraph`
  * @tparam OutputMesh a model of `FaceListGraph` and `MutableFaceGraph`
- * @tparam NamedParameters1 a sequence of \ref pmp_namedparameters "Named Parameters" for `InputMesh`
- * @tparam NamedParameters2 a sequence of \ref pmp_namedparameters "Named Parameters" for `OutputMesh`
+ * @tparam NamedParameters1 a sequence of \ref bgl_namedparameters "Named Parameters" for `InputMesh`
+ * @tparam NamedParameters2 a sequence of \ref bgl_namedparameters "Named Parameters" for `OutputMesh`
  * @tparam BottomFunctor a functor providing
  * \code {.cpp}
  * void operator()`(boost::graph_traits<InputMesh>::vertex_descriptor input_v,boost::graph_traits<OutputMesh>::vertex_descriptor output_v)
@@ -124,30 +127,36 @@ struct Identity_functor
  *  where `output_v` is the copy of `input_v` from `input` into the bottom part of `output`.
  *
  * @tparam TopFunctor a functor providing a similar `operator()` as `BottomFunctor`.
+ *
  * @param input an open surface mesh to extrude.
  * @param output a surface mesh that will contain the result of the extrusion.
  * @param bot functor that will transform all points copied from
  * `input` in order to shape the bottom part of the extrusion.
  * @param top functor that will transform all points copied from
  * `input` in order to shape the top part of the extrusion.
- * @param np_in an optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+ * @param np_in an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- * \cgalParamBegin{vertex_point_map}
- *   the property map that contains the points associated to the vertices of `input`.
- *   If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
- *   should be available for the vertices of `input`
- * \cgalParamEnd
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `input`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<InputMesh>::%vertex_descriptor` as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, input)`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     should be available for the vertices of `input`.}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  *
- * @param np_out an optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+ * @param np_out an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- * \cgalParamBegin{vertex_point_map}
- *   the property map that will contain the points associated to the vertices of `output`.
- *   If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
- *   should be available for the vertices of `output`
- * \cgalParamEnd
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `ouput`}
+ *     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<OutputMesh>::%vertex_descriptor`
+ *                    as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, ouput)`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     should be available for the vertices of `ouput`.}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  */
 template <class InputMesh,
@@ -178,9 +187,9 @@ void extrude_mesh(const InputMesh& input
   using parameters::choose_parameter;
 
   VPMap output_vpm = choose_parameter(get_parameter(np_out, internal_np::vertex_point),
-                                  get_property_map(vertex_point, output));
+                                      get_property_map(vertex_point, output));
   IVPMap input_vpm = choose_parameter(get_parameter(np_in, internal_np::vertex_point),
-                                  get_const_property_map(vertex_point, input));
+                                      get_const_property_map(vertex_point, input));
 
   std::vector<std::pair<input_vertex_descriptor, output_vertex_descriptor> > bottom_v2v;
   std::vector<std::pair<input_halfedge_descriptor, output_halfedge_descriptor> > bottom_h2h;
@@ -228,36 +237,46 @@ void extrude_mesh(const InputMesh& input
 
 /**
  * \ingroup PMP_meshing_grp
+ *
  * fills `output` with a closed mesh bounding the volume swept by `input` when translating its
  * vertices by `v`. The mesh is oriented so that the faces corresponding to `input`
  * in `output` have the same orientation.
+ *
  * \attention `output` may be self intersecting.
+ *
  * @tparam InputMesh a model of the concept `FaceListGraph`
  * @tparam OutputMesh a model of the concept `FaceListGraph` and `MutableFaceGraph`
  * @tparam Vector_3 vector type from the same CGAL kernel as the point of the vertex point map used for `OutputMesh`.
- * @tparam NamedParameters1 a sequence of \ref pmp_namedparameters "Named Parameters" for `InputMesh`
- * @tparam NamedParameters2 a sequence of \ref pmp_namedparameters "Named Parameters" for `OutputMesh`
+ * @tparam NamedParameters1 a sequence of \ref bgl_namedparameters "Named Parameters" for `InputMesh`
+ * @tparam NamedParameters2 a sequence of \ref bgl_namedparameters "Named Parameters" for `OutputMesh`
+ *
  * @param input an open surface mesh to extrude.
  * @param output a surface mesh that will contain the result of the extrusion.
  * @param v the vector defining the direction of the extrusion
- * @param np_in an optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+ * @param np_in an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- * \cgalParamBegin{vertex_point_map}
- *   the property map that contains the points associated to the vertices of `input`.
- *   If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
- *   should be available for the vertices of `input`
- * \cgalParamEnd
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `input`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<InputMesh>::%vertex_descriptor`
+ *                    as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, input)`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     should be available for the vertices of `input`.}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  *
- * @param np_out an optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+ * @param np_out an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- * \cgalParamBegin{vertex_point_map}
- *   the property map that will contain the points associated to the vertices of `output`.
- *   If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
- *   should be available for the vertices of `output`
- * \cgalParamEnd
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `output`}
+ *     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<OutputMesh>::%vertex_descriptor`
+ *                    as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, output)`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     should be available for the vertices of `output`.}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  */
 template <class InputMesh,
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/fair.h 5.1-1/include/CGAL/Polygon_mesh_processing/fair.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/fair.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/fair.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/fair.h $
-// $Id: fair.h 86ae4be 2020-05-18T12:22:21+02:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/fair.h $
+// $Id: fair.h 0715654 2020-06-03T19:01:46+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -78,18 +78,39 @@ namespace internal {
   @tparam TriangleMesh a model of `FaceGraph` and `MutableFaceGraph`
   @tparam VertexRange a range of vertex descriptors of `TriangleMesh`, model of `Range`.
           Its iterator type is `InputIterator`.
-  @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+  @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 
   @param tmesh the triangle mesh with patches to be faired
   @param vertices the vertices of the patches to be faired (the positions of only those vertices will be changed)
-  @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+  @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
   \cgalNamedParamsBegin
-    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `tmesh`.
-        If this parameter is omitted, an internal property map for
-      `CGAL::vertex_point_t` must be available in `TriangleMesh`\cgalParamEnd
-    \cgalParamBegin{fairing_continuity} tangential continuity of the output surface patch. The larger `fairing_continuity` gets, the more fixed vertices are required \cgalParamEnd
-    \cgalParamBegin{sparse_linear_solver} an instance of the sparse linear solver used for fairing \cgalParamEnd
+    \cgalParamNBegin{vertex_point_map}
+      \cgalParamDescription{a property map associating points to the vertices of `tmesh`}
+      \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+                     as key type and `%Point_3` as value type}
+      \cgalParamDefault{`boost::get(CGAL::vertex_point, tmesh)`}
+      \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+                      should be available for the vertices of `tmesh`.}
+    \cgalParamNEnd
+
+    \cgalParamNBegin{fairing_continuity}
+      \cgalParamDescription{A value controling the tangential continuity of the output surface patch.
+                            The possible values are 0, 1 and 2, refering to the  C<sup>0</sup>, C<sup>1</sup>
+                            and C<sup>2</sup> continuity.}
+      \cgalParamType{unsigned int}
+      \cgalParamDefault{`1`}
+      \cgalParamExtra{The larger `fairing_continuity` gets, the more fixed vertices are required.}
+    \cgalParamNEnd
+
+    \cgalParamNBegin{sparse_linear_solver}
+      \cgalParamDescription{an instance of the sparse linear solver used for fairing}
+      \cgalParamType{a class model of `SparseLinearAlgebraWithFactorTraits_d`}
+      \cgalParamDefault{If \ref thirdpartyEigen "Eigen" 3.2 (or greater) is available and
+                        `CGAL_EIGEN3_ENABLED` is defined, then the following overload of `Eigen_solver_traits`
+                        is provided as default value:\n
+                        `CGAL::Eigen_solver_traits<Eigen::SparseLU<CGAL::Eigen_sparse_matrix<double>::%EigenType, Eigen::COLAMDOrdering<int> > >`}
+    \cgalParamNEnd
   \cgalNamedParamsEnd
 
   @return `true` if fairing is successful, otherwise no vertices are relocated
@@ -145,14 +166,13 @@ namespace internal {
       Default_Weight_calculator;
 
     VPMap vpmap_ = choose_parameter(get_parameter(np, internal_np::vertex_point),
-                                get_property_map(vertex_point, tmesh));
+                                    get_property_map(vertex_point, tmesh));
 
     return internal::fair(tmesh, vertices,
-      choose_parameter(get_parameter(np, internal_np::sparse_linear_solver), Default_solver()),
+      choose_parameter<Default_solver>(get_parameter(np, internal_np::sparse_linear_solver)),
       choose_parameter(get_parameter(np, internal_np::weight_calculator), Default_Weight_calculator(tmesh, vpmap_)),
       choose_parameter(get_parameter(np, internal_np::fairing_continuity), 1),
-      vpmap_
-      );
+      vpmap_);
   }
 
   template<typename TriangleMesh, typename VertexRange>
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/AABB_traversal_traits_with_transformation.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/AABB_traversal_traits_with_transformation.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/AABB_traversal_traits_with_transformation.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/AABB_traversal_traits_with_transformation.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/AABB_traversal_traits_with_transformation.h $
-// $Id: AABB_traversal_traits_with_transformation.h 5b2d46c 2020-04-21T10:03:00+02:00 Maxime Gimeno
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/AABB_traversal_traits_with_transformation.h $
+// $Id: AABB_traversal_traits_with_transformation.h e9d41d7 2020-04-21T10:03:00+02:00 Maxime Gimeno
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -26,7 +26,6 @@
 #include <CGAL/Filtered_predicate.h>
 #include <CGAL/Simple_cartesian.h>
 
-#include <CGAL/Box_intersection_d/Box_with_info_d.h>
 #include <CGAL/Aff_transformation_3.h>
 #include <boost/mpl/if.hpp>
 
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/Face_graph_output_builder.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/Face_graph_output_builder.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/Face_graph_output_builder.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/Face_graph_output_builder.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Corefinement/Face_graph_output_builder.h $
-// $Id: Face_graph_output_builder.h b9a079b 2019-11-04T11:34:53+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Corefinement/Face_graph_output_builder.h $
+// $Id: Face_graph_output_builder.h 12eb23c 2020-05-05T11:57:09+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -60,7 +60,8 @@ namespace params=PMP::parameters;
 template <class TriangleMesh,
           class VertexPointMap,
           class VpmOutTuple,
-          class FaceIdMap,
+          class FaceIdMap1,
+          class FaceIdMap2,
           class Kernel_=Default,
           class EdgeMarkMapBind_  = Default,
           class EdgeMarkMapTuple_ = Default,
@@ -110,8 +111,10 @@ class Face_graph_output_builder
 //Data members
   TriangleMesh &tm1, &tm2;
   // property maps of input meshes
-  const VertexPointMap &vpm1, &vpm2;
-  const FaceIdMap &fids1, &fids2;
+  const VertexPointMap vpm1;
+  const VertexPointMap vpm2;
+  FaceIdMap1 fids1;
+  FaceIdMap2 fids2;
   EdgeMarkMapBind& marks_on_input_edges;
   // property maps of output meshes
   const VpmOutTuple& output_vpms;
@@ -215,7 +218,7 @@ class Face_graph_output_builder
   // detect if a polyline is incident to two patches that won't be imported
   // for the current operation (polylines skipt are always incident to a
   // coplanar patch)
-  template <class TM, class FIM>
+  template <class TM, class FIM1, class FIM2>
   static
   void fill_polylines_to_skip(
     Intersection_polylines& polylines,
@@ -223,8 +226,8 @@ class Face_graph_output_builder
     const std::vector<std::size_t>& tm2_patch_ids,
     const boost::dynamic_bitset<>& patches_of_tm1_used,
     const boost::dynamic_bitset<>& patches_of_tm2_used,
-    const FIM& fids1,
-    const FIM& fids2,
+    const FIM1 fids1,
+    const FIM2 fids2,
     const TM& tm1,
     const TM& tm2)
   {
@@ -341,18 +344,17 @@ class Face_graph_output_builder
 
 public:
 
-  Face_graph_output_builder(      TriangleMesh& tm1,
-                                  TriangleMesh& tm2,
-                            const VertexPointMap &vpm1,
-                            const VertexPointMap &vpm2,
-                            const FaceIdMap& fids1,
-                            const FaceIdMap& fids2,
-                                  EdgeMarkMapBind& marks_on_input_edges,
+  Face_graph_output_builder(TriangleMesh& tm1,
+                            TriangleMesh& tm2,
+                            const VertexPointMap vpm1,
+                            const VertexPointMap vpm2,
+                            FaceIdMap1 fids1,
+                            FaceIdMap2 fids2,
+                            EdgeMarkMapBind& marks_on_input_edges,
                             const VpmOutTuple& output_vpms,
-                                  EdgeMarkMapTuple& out_edge_mark_maps,
-                                  UserVisitor& user_visitor,
-                            const std::array<
-                              boost::optional<TriangleMesh*>, 4 >& requested_output)
+                            EdgeMarkMapTuple& out_edge_mark_maps,
+                            UserVisitor& user_visitor,
+                            const std::array<boost::optional<TriangleMesh*>, 4 >& requested_output)
     : tm1(tm1), tm2(tm2)
     , vpm1(vpm1), vpm2(vpm2)
     , fids1(fids1), fids2(fids2)
@@ -460,9 +462,20 @@ public:
     Intersection_edge_map& intersection_edges1 = mesh_to_intersection_edges[&tm1];
     Intersection_edge_map& intersection_edges2 = mesh_to_intersection_edges[&tm2];
 
-    // this will initialize face indices if the face index map is writable.
-    helpers::init_face_indices(tm1, fids1);
-    helpers::init_face_indices(tm2, fids2);
+    // The property map must be either writable or well-initialized
+    if( CGAL::internal::Is_writable_property_map<FaceIdMap1>::value &&
+        !BGL::internal::is_index_map_valid(fids1, num_faces(tm1), faces(tm1)) )
+    {
+      BGL::internal::initialize_face_index_map(fids1, tm1);
+    }
+    CGAL_assertion(BGL::internal::is_index_map_valid(fids1, num_faces(tm1), faces(tm1)));
+
+    if( CGAL::internal::Is_writable_property_map<FaceIdMap2>::value &&
+        !BGL::internal::is_index_map_valid(fids2, num_faces(tm2), faces(tm2)) )
+    {
+      BGL::internal::initialize_face_index_map(fids2, tm2);
+    }
+    CGAL_assertion(BGL::internal::is_index_map_valid(fids2, num_faces(tm2), faces(tm2)));
 
     // bitset to identify coplanar faces
     boost::dynamic_bitset<> tm1_coplanar_faces(num_faces(tm1), 0);
@@ -620,9 +633,8 @@ public:
     std::size_t nb_patches_tm1 =
       PMP::connected_components(tm1,
                                 bind_property_maps(fids1,make_property_map(&tm1_patch_ids[0])),
-                                params::edge_is_constrained_map(
-                                    is_marked_1)
-                                .face_index_map(fids1));
+                                params::edge_is_constrained_map(is_marked_1)
+                                       .face_index_map(fids1));
 
     std::vector <std::size_t> tm1_patch_sizes(nb_patches_tm1, 0);
     for(std::size_t i : tm1_patch_ids)
@@ -634,9 +646,8 @@ public:
     std::size_t nb_patches_tm2 =
       PMP::connected_components(tm2,
                                 bind_property_maps(fids2,make_property_map(&tm2_patch_ids[0])),
-                                params::edge_is_constrained_map(
-                                    is_marked_2)
-                                .face_index_map(fids2));
+                                params::edge_is_constrained_map(is_marked_2)
+                                       .face_index_map(fids2));
 
     std::vector <std::size_t> tm2_patch_sizes(nb_patches_tm2, 0);
     for(Node_id i : tm2_patch_ids)
@@ -671,6 +682,10 @@ public:
       halfedge_descriptor h1 = it->second.first[&tm1];
       halfedge_descriptor h2 = it->second.first[&tm2];
 
+#ifdef CGAL_COREFINEMENT_DEBUG
+      std::cout << "Looking at triangles around edge " << tm1.point(source(h1, tm1)) << " " << tm1.point(target(h1, tm1)) << "\n";
+#endif
+
       CGAL_assertion(ids.first==vertex_to_node_id1[source(h1,tm1)]);
       CGAL_assertion(ids.second==vertex_to_node_id1[target(h1,tm1)]);
       CGAL_assertion(ids.first==vertex_to_node_id2[source(h2,tm2)]);
@@ -702,6 +717,9 @@ public:
             //Nothing allowed
             if (!used_to_clip_a_surface)
             {
+#ifdef CGAL_COREFINEMENT_DEBUG
+              std::cout << "  Non-manifold edge case 1\n";
+#endif
               impossible_operation.set();
               return;
             }
@@ -712,6 +730,9 @@ public:
           //Ambiguous, we can do nothing
           if (!used_to_clip_a_surface)
           {
+#ifdef CGAL_COREFINEMENT_DEBUG
+              std::cout << "  Non-manifold edge case 2\n";
+#endif
             impossible_operation.set();
             return;
           }
@@ -765,6 +786,9 @@ public:
         {
           CGAL_assertion(!used_to_clip_a_surface);
           //Ambiguous, we do nothing
+#ifdef CGAL_COREFINEMENT_DEBUG
+              std::cout << "  Non-manifold edge case 3\n";
+#endif
           impossible_operation.set();
           return;
         }
@@ -960,6 +984,9 @@ public:
                 // poly_second - poly_first             = {0}
                 // poly_first \cap poly_second          = q1q2
                 // opposite( poly_first U poly_second ) = p2p1
+#ifdef CGAL_COREFINEMENT_DEBUG
+              std::cout << "  Non-manifold edge case 4\n";
+#endif
                 impossible_operation.set(TM1_MINUS_TM2); // tm1-tm2 is non-manifold
               }
               else{
@@ -971,7 +998,12 @@ public:
                 is_patch_inside_tm2.set(patch_id_p1);
                 is_patch_inside_tm2.set(patch_id_p2);
                 if (!used_to_clip_a_surface)
+                {
+#ifdef CGAL_COREFINEMENT_DEBUG
+              std::cout << "  Non-manifold edge case 5\n";
+#endif
                   impossible_operation.set(INTERSECTION); // tm1 n tm2 is non-manifold
+                }
               }
             }
             else
@@ -986,6 +1018,9 @@ public:
               {
                 if (!used_to_clip_a_surface)
                 {
+#ifdef CGAL_COREFINEMENT_DEBUG
+              std::cout << "  Non-manifold edge case 6\n";
+#endif
                   impossible_operation.set();
                   return;
                 }
@@ -1007,6 +1042,9 @@ public:
               {
                 if (!used_to_clip_a_surface)
                 {
+#ifdef CGAL_COREFINEMENT_DEBUG
+              std::cout << "  Non-manifold edge case 7\n";
+#endif
                   impossible_operation.set();
                   return;
                 }
@@ -1028,6 +1066,9 @@ public:
                 // poly_second - poly_first             = q1q2
                 // poly_first \cap poly_second          = {0}
                 // opposite( poly_first U poly_second ) = p2q1 U q2p1
+#ifdef CGAL_COREFINEMENT_DEBUG
+              std::cout << "  Non-manifold edge case 8\n";
+#endif
                 impossible_operation.set(UNION); // tm1 U tm2 is non-manifold
               }
               else{
@@ -1038,6 +1079,9 @@ public:
                 // poly_second - poly_first             = q1p1 U p2q2
                 // poly_first \cap poly_second          = p1p2
                 // opposite( poly_first U poly_second ) = q2q1
+#ifdef CGAL_COREFINEMENT_DEBUG
+              std::cout << "  Non-manifold edge case 9\n";
+#endif
                 impossible_operation.set(TM2_MINUS_TM1); // tm2 - tm1 is non-manifold
               }
             }
@@ -1245,9 +1289,8 @@ public:
       polyline_lengths.push_back(polyline_info.second+1);
     }
 
-    typedef Patch_container<TriangleMesh,
-                            FaceIdMap,
-                            Intersection_edge_map> Patches;
+    typedef Patch_container<TriangleMesh, FaceIdMap1, Intersection_edge_map> Patches1;
+    typedef Patch_container<TriangleMesh, FaceIdMap2, Intersection_edge_map> Patches2;
 
     boost::unordered_set<vertex_descriptor> border_nm_vertices; // only used if used_to_clip_a_surface == true
     if (used_to_clip_a_surface)
@@ -1279,8 +1322,8 @@ public:
     }
 
     //store the patch description in a container to avoid recomputing it several times
-    Patches patches_of_tm1( tm1, tm1_patch_ids, fids1, intersection_edges1, nb_patches_tm1),
-            patches_of_tm2( tm2, tm2_patch_ids, fids2, intersection_edges2, nb_patches_tm2);
+    Patches1 patches_of_tm1(tm1, tm1_patch_ids, fids1, intersection_edges1, nb_patches_tm1);
+    Patches2 patches_of_tm2(tm2, tm2_patch_ids, fids2, intersection_edges2, nb_patches_tm2);
 
     // for each boolean operation, define two bitsets of patches contributing
     // to the result
@@ -1460,11 +1503,12 @@ public:
 
         // operation in tm1 with removal (and optionally inside-out) delayed
         // First backup the border edges of patches to be used
-        Patches tmp_patches_of_tm1(tm1,
-          patches_of_tm1.patch_ids,
-          patches_of_tm1.fids,
-          patches_of_tm1.is_intersection_edge,
-          patches_of_tm1.patches.size());
+        Patches1 tmp_patches_of_tm1(tm1,
+                                    patches_of_tm1.patch_ids,
+                                    patches_of_tm1.fids,
+                                    patches_of_tm1.is_intersection_edge,
+                                    patches_of_tm1.patches.size());
+
         boost::dynamic_bitset<> patches_of_tm1_removed =
             ~patches_of_tm1_used[inplace_operation_tm1];
         for (std::size_t i = patches_of_tm1_removed.find_first();
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/face_graph_utils.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/face_graph_utils.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/face_graph_utils.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/face_graph_utils.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Corefinement/face_graph_utils.h $
-// $Id: face_graph_utils.h b7c2375 2020-02-14T14:46:19+01:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Corefinement/face_graph_utils.h $
+// $Id: face_graph_utils.h 84186fc 2020-03-09T09:55:58+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -424,7 +424,7 @@ template <class PolygonMesh, class FaceI
 void extract_patch_simplices(
   std::size_t patch_id,
   PolygonMesh& pm,
-  const FaceIndexMap& fids,
+  const FaceIndexMap fids,
   const std::vector<std::size_t>& patch_ids,
   std::vector<typename boost::graph_traits<PolygonMesh>::face_descriptor>& patch_faces,
   std::set<typename boost::graph_traits<PolygonMesh>::vertex_descriptor>& interior_vertices,
@@ -480,13 +480,13 @@ struct Patch_container{
 // external data members
   PolygonMesh& pm;
   const std::vector<std::size_t>& patch_ids;
-  const FaceIndexMap& fids;
+  const FaceIndexMap fids;
   const IsIntersectionEdge& is_intersection_edge;
 // constructor
   Patch_container(
     PolygonMesh& pm,
     const std::vector<std::size_t>& patch_ids,
-    const FaceIndexMap& fids,
+    const FaceIndexMap fids,
     const IsIntersectionEdge& is_intersection_edge,
     std::size_t nb_patches
   ) : patches(nb_patches)
@@ -1010,14 +1010,15 @@ template < class TriangleMesh,
            class EdgeMarkMap2,
            class EdgeMarkMapOut,
            class IntersectionPolylines,
-           class PatchContainer,
+           class PatchContainer1,
+           class PatchContainer2,
            class UserVisitor>
 void fill_new_triangle_mesh(
   TriangleMesh& output,
   const boost::dynamic_bitset<>& patches_of_tm1_to_import,
   const boost::dynamic_bitset<>& patches_of_tm2_to_import,
-  PatchContainer& patches_of_tm1,
-  PatchContainer& patches_of_tm2,
+  PatchContainer1& patches_of_tm1,
+  PatchContainer2& patches_of_tm2,
   bool reverse_orientation_of_patches_from_tm1,
   bool reverse_orientation_of_patches_from_tm2,
   const IntersectionPolylines& polylines,
@@ -1256,7 +1257,8 @@ void disconnect_patches(
 }
 
 template <class TriangleMesh,
-          class PatchContainer,
+          class PatchContainer1,
+          class PatchContainer2,
           class IntersectionPolylines,
           class EdgeMap,
           class VertexPointMap,
@@ -1269,8 +1271,8 @@ void compute_inplace_operation_delay_rem
   TriangleMesh& tm2,
   const boost::dynamic_bitset<>& patches_of_tm1_to_keep,
   const boost::dynamic_bitset<>& patches_of_tm2_to_import,
-  PatchContainer& patches_of_tm1,
-  PatchContainer& patches_of_tm2,
+  PatchContainer1& patches_of_tm1,
+  PatchContainer2& patches_of_tm2,
   bool reverse_patch_orientation_tm2,
   const IntersectionPolylines& polylines,
   const VertexPointMap& vpm1,
@@ -1416,7 +1418,8 @@ remove_patches(TriangleMesh& tm,
 }
 
 template <class TriangleMesh,
-          class PatchContainer,
+          class PatchContainer1,
+          class PatchContainer2,
           class VertexPointMap,
           class EdgeMarkMapIn1,
           class EdgeMarkMapIn2,
@@ -1427,8 +1430,8 @@ void compute_inplace_operation(
   const TriangleMesh& /*tm2*/,
   const boost::dynamic_bitset<>& patches_of_tm1_to_keep,
   const boost::dynamic_bitset<>& patches_of_tm2_to_import,
-  PatchContainer& patches_of_tm1,
-  PatchContainer& patches_of_tm2,
+  PatchContainer1& patches_of_tm1,
+  PatchContainer2& patches_of_tm2,
   bool reverse_patch_orientation_tm1,
   bool reverse_patch_orientation_tm2,
   const VertexPointMap& vpm1,
@@ -1485,14 +1488,15 @@ void compute_inplace_operation(
 
 template <class TriangleMesh,
           class IntersectionPolylines,
-          class PatchContainer,
+          class PatchContainer1,
+          class PatchContainer2,
           class EdgeMap>
 void compute_border_edge_map(
   const TriangleMesh& tm1,
   const TriangleMesh& tm2,
   const IntersectionPolylines& polylines,
-  PatchContainer& patches_of_tm1,
-  PatchContainer& patches_of_tm2,
+  PatchContainer1& patches_of_tm1,
+  PatchContainer2& patches_of_tm2,
   EdgeMap& tm2_edge_to_tm1_edge)
 {
   typedef boost::graph_traits<TriangleMesh> GT;
@@ -1520,7 +1524,8 @@ void compute_border_edge_map(
 
 
 template <class TriangleMesh,
-          class PatchContainer,
+          class PatchContainer1,
+          class PatchContainer2,
           class IntersectionPolylines,
           class VertexPointMap,
           class EdgeMarkMapIn1,
@@ -1532,8 +1537,8 @@ void compute_inplace_operation(
   const TriangleMesh& tm2,
   const boost::dynamic_bitset<>& patches_of_tm1_to_keep,
   const boost::dynamic_bitset<>& patches_of_tm2_to_import,
-  PatchContainer& patches_of_tm1,
-  PatchContainer& patches_of_tm2,
+  PatchContainer1& patches_of_tm1,
+  PatchContainer2& patches_of_tm2,
   bool reverse_patch_orientation_tm1,
   bool reverse_patch_orientation_tm2,
   const VertexPointMap& vpm1,
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/intersection_callbacks.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/intersection_callbacks.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/intersection_callbacks.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/intersection_callbacks.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Corefinement/intersection_callbacks.h $
-// $Id: intersection_callbacks.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Corefinement/intersection_callbacks.h $
+// $Id: intersection_callbacks.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -15,7 +15,7 @@
 
 #include <CGAL/license/Polygon_mesh_processing/corefinement.h>
 
-
+#include <CGAL/Box_intersection_d/Box_with_info_d.h>
 #include <CGAL/property_map.h>
 #include <CGAL/enum.h>
 #include <CGAL/Polygon_mesh_processing/self_intersections.h>
@@ -38,7 +38,9 @@ protected:
   typedef boost::graph_traits<TriangleMesh> Graph_traits;
   typedef typename Graph_traits::face_descriptor face_descriptor;
   typedef typename Graph_traits::halfedge_descriptor halfedge_descriptor;
-  typedef typename CGAL::Box_intersection_d::Box_with_info_d<double, 3, halfedge_descriptor> Box;
+
+  typedef CGAL::Box_intersection_d::ID_FROM_BOX_ADDRESS Box_policy;
+  typedef CGAL::Box_intersection_d::Box_with_info_d<double, 3, halfedge_descriptor, Box_policy> Box;
 
 public:
   Collect_face_bbox_per_edge_bbox(
@@ -80,9 +82,12 @@ protected:
   typedef boost::graph_traits<TriangleMesh> Graph_traits;
   typedef typename Graph_traits::face_descriptor face_descriptor;
   typedef typename Graph_traits::halfedge_descriptor halfedge_descriptor;
-  typedef typename CGAL::Box_intersection_d::Box_with_info_d<double, 3, halfedge_descriptor> Box;
+
   typedef typename boost::property_traits<VertexPointMap>::reference Point;
 
+  typedef CGAL::Box_intersection_d::ID_FROM_BOX_ADDRESS Box_policy;
+  typedef CGAL::Box_intersection_d::Box_with_info_d<double, 3, halfedge_descriptor, Box_policy> Box;
+
 public:
   Collect_face_bbox_per_edge_bbox_with_coplanar_handling(
     const TriangleMesh& tm_faces,
@@ -162,7 +167,10 @@ protected:
   typedef typename Graph_traits::face_descriptor face_descriptor;
   typedef typename Graph_traits::halfedge_descriptor halfedge_descriptor;
   typedef typename Graph_traits::vertex_descriptor vertex_descriptor;
-  typedef typename CGAL::Box_intersection_d::Box_with_info_d<double, 3, halfedge_descriptor> Box;
+
+  typedef CGAL::Box_intersection_d::ID_FROM_BOX_ADDRESS Box_policy;
+  typedef CGAL::Box_intersection_d::Box_with_info_d<double, 3, halfedge_descriptor, Box_policy> Box;
+
   typedef typename boost::property_traits<VertexPointMap>::reference Point;
 
   bool is_edge_target_incident_to_face(halfedge_descriptor hd,
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/intersection_impl.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/intersection_impl.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/intersection_impl.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/intersection_impl.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Corefinement/intersection_impl.h $
-// $Id: intersection_impl.h 2c5826d 2020-01-17T16:29:49+01:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Corefinement/intersection_impl.h $
+// $Id: intersection_impl.h 9c0e99a 2020-03-24T14:29:45+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -18,7 +18,6 @@
 
 #include <boost/graph/graph_traits.hpp>
 #include <CGAL/box_intersection_d.h>
-#include <CGAL/Box_intersection_d/Box_with_info_d.h>
 #include <CGAL/Polygon_mesh_processing/internal/Corefinement/intersection_callbacks.h>
 #include <CGAL/Polygon_mesh_processing/internal/Corefinement/Intersection_type.h>
 #include <CGAL/Polygon_mesh_processing/internal/Corefinement/intersection_of_coplanar_triangles_3.h>
@@ -156,7 +155,8 @@ class Intersection_of_triangle_meshes
   typedef typename graph_traits::halfedge_descriptor halfedge_descriptor;
   typedef typename graph_traits::vertex_descriptor vertex_descriptor;
 
-  typedef typename CGAL::Box_intersection_d::Box_with_info_d<double, 3, halfedge_descriptor> Box;
+  typedef CGAL::Box_intersection_d::ID_FROM_BOX_ADDRESS Box_policy;
+  typedef CGAL::Box_intersection_d::Box_with_info_d<double, 3, halfedge_descriptor, Box_policy> Box;
 
   typedef boost::unordered_set<face_descriptor> Face_set;
   typedef boost::unordered_map<edge_descriptor, Face_set> Edge_to_faces;
@@ -250,10 +250,11 @@ class Intersection_of_triangle_meshes
         if (callback_si.self_intersections_found())
          throw Self_intersection_exception();
     }
-    else
-        CGAL::box_intersection_d( face_boxes_ptr.begin(), face_boxes_ptr.end(),
-                              edge_boxes_ptr.begin(), edge_boxes_ptr.end(),
-                              callback, cutoff );
+    else {
+      CGAL::box_intersection_d( face_boxes_ptr.begin(), face_boxes_ptr.end(),
+                                edge_boxes_ptr.begin(), edge_boxes_ptr.end(),
+                                callback, cutoff );
+    }
   }
 
   // for autorefinement
@@ -1267,6 +1268,8 @@ public:
   : nodes(tm1, tm2, vpm1, vpm2)
   , visitor(v)
   {
+    CGAL_precondition(is_triangle_mesh(tm1));
+    CGAL_precondition(is_triangle_mesh(tm2));
     CGAL_assertion_code( doing_autorefinement=false; )
   }
 
@@ -1277,6 +1280,7 @@ public:
   : nodes(tm, tm, vpm, vpm)
   , visitor(v)
   {
+    CGAL_precondition(is_triangle_mesh(tm));
     CGAL_assertion_code( doing_autorefinement=true; )
   }
 
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/intersection_nodes.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/intersection_nodes.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/intersection_nodes.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/intersection_nodes.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Corefinement/intersection_nodes.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Corefinement/intersection_nodes.h $
 // $Id: intersection_nodes.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/intersection_of_coplanar_triangles_3.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/intersection_of_coplanar_triangles_3.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/intersection_of_coplanar_triangles_3.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/intersection_of_coplanar_triangles_3.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Corefinement/intersection_of_coplanar_triangles_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Corefinement/intersection_of_coplanar_triangles_3.h $
 // $Id: intersection_of_coplanar_triangles_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/Intersection_type.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/Intersection_type.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/Intersection_type.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/Intersection_type.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Corefinement/Intersection_type.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Corefinement/Intersection_type.h $
 // $Id: Intersection_type.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/intersect_triangle_and_segment_3.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/intersect_triangle_and_segment_3.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/intersect_triangle_and_segment_3.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/intersect_triangle_and_segment_3.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Corefinement/intersect_triangle_and_segment_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Corefinement/intersect_triangle_and_segment_3.h $
 // $Id: intersect_triangle_and_segment_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/Output_builder_for_autorefinement.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/Output_builder_for_autorefinement.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/Output_builder_for_autorefinement.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/Output_builder_for_autorefinement.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Corefinement/Output_builder_for_autorefinement.h $
-// $Id: Output_builder_for_autorefinement.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Corefinement/Output_builder_for_autorefinement.h $
+// $Id: Output_builder_for_autorefinement.h ea80a64 2020-03-24T14:30:45+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -214,9 +214,6 @@ public:
     const boost::dynamic_bitset<>& is_node_of_degree_one,
     const Mesh_to_map_node&)
   {
-    // this will initialize face indices if the face index map is writable.
-    helpers::init_face_indices(tm, fids);
-
     // first build an unordered_map mapping a vertex to its node id + a set
     // of all intersection edges
     typedef boost::unordered_set<edge_descriptor> Intersection_edge_map;
@@ -239,8 +236,13 @@ public:
       intersection_edges.insert(edge(p.second.h2, tm));
     }
 
-    // this will initialize face indices if the face index map is writable.
-    helpers::init_face_indices(tm, fids);
+    // The property map must be either writable or well-initialized
+    if( CGAL::internal::Is_writable_property_map<FaceIdMap>::value &&
+        !BGL::internal::is_index_map_valid(fids, num_faces(tm), faces(tm)) )
+    {
+      BGL::internal::initialize_face_index_map(fids, tm);
+    }
+    CGAL_assertion(BGL::internal::is_index_map_valid(fids, num_faces(tm), faces(tm)));
 
     // bitset to identify coplanar faces
     boost::dynamic_bitset<> tm_coplanar_faces(num_faces(tm), 0);
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/predicates.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/predicates.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/predicates.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/predicates.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Corefinement/predicates.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Corefinement/predicates.h $
 // $Id: predicates.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/Visitor.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/Visitor.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/Visitor.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Corefinement/Visitor.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Corefinement/Visitor.h $
-// $Id: Visitor.h 2c5826d 2020-01-17T16:29:49+01:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Corefinement/Visitor.h $
+// $Id: Visitor.h fe91249 2020-01-17T16:27:13+01:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/do_no_use_CDT2.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/do_no_use_CDT2.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/do_no_use_CDT2.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/do_no_use_CDT2.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/do_no_use_CDT2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/do_no_use_CDT2.h $
 // $Id: do_no_use_CDT2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/fair_impl.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/fair_impl.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/fair_impl.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/fair_impl.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/fair_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/fair_impl.h $
 // $Id: fair_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Hole_filling/do_not_use_DT3.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Hole_filling/do_not_use_DT3.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Hole_filling/do_not_use_DT3.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Hole_filling/do_not_use_DT3.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Hole_filling/do_not_use_DT3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Hole_filling/do_not_use_DT3.h $
 // $Id: do_not_use_DT3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Hole_filling/experimental/experimental_code.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Hole_filling/experimental/experimental_code.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Hole_filling/experimental/experimental_code.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Hole_filling/experimental/experimental_code.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Hole_filling/experimental/experimental_code.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Hole_filling/experimental/experimental_code.h $
 // $Id: experimental_code.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Hole_filling/Triangulate_hole_polygon_mesh.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Hole_filling/Triangulate_hole_polygon_mesh.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Hole_filling/Triangulate_hole_polygon_mesh.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Hole_filling/Triangulate_hole_polygon_mesh.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Hole_filling/Triangulate_hole_polygon_mesh.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Hole_filling/Triangulate_hole_polygon_mesh.h $
 // $Id: Triangulate_hole_polygon_mesh.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Hole_filling/Triangulate_hole_polyline.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Hole_filling/Triangulate_hole_polyline.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Hole_filling/Triangulate_hole_polyline.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Hole_filling/Triangulate_hole_polyline.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Hole_filling/Triangulate_hole_polyline.h $
-// $Id: Triangulate_hole_polyline.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Hole_filling/Triangulate_hole_polyline.h $
+// $Id: Triangulate_hole_polyline.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -35,6 +35,7 @@
 #include <CGAL/boost/iterator/transform_iterator.hpp>
 #include <boost/next_prior.hpp>
 #include <boost/unordered_set.hpp>
+#include <boost/next_prior.hpp>
 
 namespace CGAL {
 namespace internal {
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Isotropic_remeshing/AABB_filtered_projection_traits.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Isotropic_remeshing/AABB_filtered_projection_traits.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Isotropic_remeshing/AABB_filtered_projection_traits.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Isotropic_remeshing/AABB_filtered_projection_traits.h	2020-09-08 12:15:23.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Isotropic_remeshing/AABB_filtered_projection_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Isotropic_remeshing/AABB_filtered_projection_traits.h $
 // $Id: AABB_filtered_projection_traits.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Isotropic_remeshing/remesh_impl.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Isotropic_remeshing/remesh_impl.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Isotropic_remeshing/remesh_impl.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Isotropic_remeshing/remesh_impl.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Isotropic_remeshing/remesh_impl.h $
-// $Id: remesh_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Isotropic_remeshing/remesh_impl.h $
+// $Id: remesh_impl.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -372,7 +372,6 @@ namespace internal {
       }
       for(std::size_t i=0; i < trees.size(); ++i){
         trees[i]->build();
-        trees[i]->accelerate_distance_queries();
       }
     }
 
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/mesh_to_point_set_hausdorff_distance.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/mesh_to_point_set_hausdorff_distance.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/mesh_to_point_set_hausdorff_distance.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/mesh_to_point_set_hausdorff_distance.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/mesh_to_point_set_hausdorff_distance.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/mesh_to_point_set_hausdorff_distance.h $
 // $Id: mesh_to_point_set_hausdorff_distance.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/named_function_params.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/named_function_params.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/named_function_params.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/named_function_params.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/named_function_params.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/named_function_params.h $
 // $Id: named_function_params.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/named_params_helper.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/named_params_helper.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/named_params_helper.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/named_params_helper.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/named_params_helper.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/named_params_helper.h $
 // $Id: named_params_helper.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Polygon_mesh_slicer/Axis_parallel_plane_traits.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Polygon_mesh_slicer/Axis_parallel_plane_traits.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Polygon_mesh_slicer/Axis_parallel_plane_traits.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Polygon_mesh_slicer/Axis_parallel_plane_traits.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Polygon_mesh_slicer/Axis_parallel_plane_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Polygon_mesh_slicer/Axis_parallel_plane_traits.h $
 // $Id: Axis_parallel_plane_traits.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Polygon_mesh_slicer/Traversal_traits.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Polygon_mesh_slicer/Traversal_traits.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Polygon_mesh_slicer/Traversal_traits.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Polygon_mesh_slicer/Traversal_traits.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Polygon_mesh_slicer/Traversal_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Polygon_mesh_slicer/Traversal_traits.h $
 // $Id: Traversal_traits.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/refine_impl.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/refine_impl.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/refine_impl.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/refine_impl.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/refine_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/refine_impl.h $
 // $Id: refine_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/repair_extra.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/repair_extra.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/repair_extra.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/repair_extra.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/repair_extra.h $
-// $Id: repair_extra.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/repair_extra.h $
+// $Id: repair_extra.h 41bb11c 2019-11-19T17:08:25+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -110,7 +110,8 @@ void collect_close_stitchable_boundary_e
   typedef boost::unordered_map<halfedge_descriptor, int> Halfedge_multiplicity;
   typedef std::vector<std::pair<halfedge_descriptor, halfedge_descriptor> > Halfedge_pairs;
 
-  typedef typename Box_intersection_d::Box_with_info_d<double, 3, edge_descriptor> Box;
+  typedef CGAL::Box_intersection_d::ID_FROM_BOX_ADDRESS Box_policy;
+  typedef CGAL::Box_intersection_d::Box_with_info_d<double, 3, edge_descriptor, Box_policy> Box;
 
   typedef Union_find<vertex_descriptor> UF_vertices;
   typedef std::map<vertex_descriptor, typename UF_vertices::handle> Handle_map;
@@ -141,7 +142,6 @@ void collect_close_stitchable_boundary_e
   for(Box& b : boxes)
     box_ptrs.push_back(&b);
 
-
   Halfedge_multiplicity multiplicity;
   Halfedge_pairs matching_hedges;
 
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Side_of_triangle_mesh/Point_inside_vertical_ray_cast.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Side_of_triangle_mesh/Point_inside_vertical_ray_cast.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Side_of_triangle_mesh/Point_inside_vertical_ray_cast.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Side_of_triangle_mesh/Point_inside_vertical_ray_cast.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Side_of_triangle_mesh/Point_inside_vertical_ray_cast.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Side_of_triangle_mesh/Point_inside_vertical_ray_cast.h $
 // $Id: Point_inside_vertical_ray_cast.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Side_of_triangle_mesh/Ray_3_Triangle_3_traversal_traits.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Side_of_triangle_mesh/Ray_3_Triangle_3_traversal_traits.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Side_of_triangle_mesh/Ray_3_Triangle_3_traversal_traits.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Side_of_triangle_mesh/Ray_3_Triangle_3_traversal_traits.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Side_of_triangle_mesh/Ray_3_Triangle_3_traversal_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Side_of_triangle_mesh/Ray_3_Triangle_3_traversal_traits.h $
 // $Id: Ray_3_Triangle_3_traversal_traits.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Smoothing/ceres_support.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Smoothing/ceres_support.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Smoothing/ceres_support.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Smoothing/ceres_support.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Smoothing/ceres_support.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Smoothing/ceres_support.h $
 // $Id: ceres_support.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Smoothing/curvature_flow_impl.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Smoothing/curvature_flow_impl.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Smoothing/curvature_flow_impl.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Smoothing/curvature_flow_impl.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Smoothing/curvature_flow_impl.h $
-// $Id: curvature_flow_impl.h 1641310 2020-06-15T17:12:04+02:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Smoothing/curvature_flow_impl.h $
+// $Id: curvature_flow_impl.h 7e12992 2020-06-15T17:12:04+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -171,7 +171,7 @@ public:
     // calls compute once to factorize with the preconditioner
     if(!solver.factor(A, D))
     {
-#ifdef CGAL_PMP_SMOOTHING_VERBOSE
+#ifdef CGAL_PMP_SMOOTHING_DEBUG
       std::cerr << "Could not factorize linear system with preconditioner." << std::endl;
 #endif
       return false;
@@ -181,7 +181,7 @@ public:
        !solver.linear_solver(by, Xy) ||
        !solver.linear_solver(bz, Xz))
     {
-#ifdef CGAL_PMP_SMOOTHING_VERBOSE
+#ifdef CGAL_PMP_SMOOTHING_DEBUG
       std::cerr << "Could not solve linear system." << std::endl;
 #endif
       return false;
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Smoothing/mesh_smoothing_impl.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Smoothing/mesh_smoothing_impl.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Smoothing/mesh_smoothing_impl.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Smoothing/mesh_smoothing_impl.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Smoothing/mesh_smoothing_impl.h $
-// $Id: mesh_smoothing_impl.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Smoothing/mesh_smoothing_impl.h $
+// $Id: mesh_smoothing_impl.h 29ddd67 2020-02-06T17:14:16+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -21,7 +21,7 @@
 #endif
 
 #include <CGAL/Polygon_mesh_processing/compute_normal.h>
-#include <CGAL/Polygon_mesh_processing/repair.h>
+#include <CGAL/Polygon_mesh_processing/shape_predicates.h>
 
 #include <CGAL/AABB_tree.h>
 #include <CGAL/AABB_traits.h>
@@ -50,9 +50,11 @@ namespace Polygon_mesh_processing {
 namespace internal {
 
 template <typename V, typename GT>
-double get_radian_angle(const V& v1, const V& v2, const GT& gt)
+typename GT::FT get_radian_angle(const V& v1, const V& v2, const GT& gt)
 {
-  return gt.compute_approximate_angle_3_object()(v1, v2) * CGAL_PI / 180.;
+  typedef typename GT::FT FT;
+
+  return gt.compute_approximate_angle_3_object()(v1, v2) * CGAL_PI / FT(180);
 }
 
 // super naive for now. Not sure it even makes sense to do something like that for surfaces
@@ -68,6 +70,7 @@ class Delaunay_edge_flipper
   typedef typename boost::graph_traits<TriangleMesh>::face_descriptor      face_descriptor;
 
   typedef typename boost::property_traits<VertexPointMap>::reference       Point_ref;
+  typedef typename GeomTraits::FT                                          FT;
   typedef typename GeomTraits::Vector_3                                    Vector;
 
 public:
@@ -86,20 +89,13 @@ public:
     const halfedge_descriptor h = halfedge(e, mesh_);
     const halfedge_descriptor opp_h = opposite(h, mesh_);
 
-    vertex_descriptor v0 = source(h, mesh_);
-    vertex_descriptor v1 = target(h, mesh_);
-    vertex_descriptor v2 = target(next(h, mesh_), mesh_);
-    vertex_descriptor v3 = target(next(opp_h, mesh_), mesh_);
-    const Point_ref p0 = get(vpmap_, v0);
-    const Point_ref p1 = get(vpmap_, v1);
-    const Point_ref p2 = get(vpmap_, v2);
-    const Point_ref p3 = get(vpmap_, v3);
-
-    double alpha = get_radian_angle(Vector(p0 - p2), Vector(p1 - p2), traits_);
-    double beta = get_radian_angle(Vector(p1 - p3), Vector(p0 - p3), traits_);
+    const vertex_descriptor v0 = source(h, mesh_);
+    const vertex_descriptor v1 = target(h, mesh_);
+    const vertex_descriptor v2 = target(next(h, mesh_), mesh_);
+    const vertex_descriptor v3 = target(next(opp_h, mesh_), mesh_);
 
-    // not local Delaunay if the sum of the angles is greater than pi
-    if(alpha + beta <= CGAL_PI)
+    std::set<vertex_descriptor> unique_vs { v0, v1, v2, v3 };
+    if(unique_vs.size() != 4)
       return false;
 
     // Don't want to flip if the other diagonal already exists
@@ -108,7 +104,16 @@ public:
     if(other_hd_already_exists.second)
       return false;
 
-    return true;
+    // not local Delaunay := sum of the opposite angles is greater than pi
+    const Point_ref p0 = get(vpmap_, v0);
+    const Point_ref p1 = get(vpmap_, v1);
+    const Point_ref p2 = get(vpmap_, v2);
+    const Point_ref p3 = get(vpmap_, v3);
+
+    FT alpha = get_radian_angle(Vector(p0 - p2), Vector(p1 - p2), traits_);
+    FT beta = get_radian_angle(Vector(p1 - p3), Vector(p0 - p3), traits_);
+
+    return (alpha + beta > CGAL_PI);
   }
 
   template <typename Marked_edges_map, typename EdgeRange>
@@ -127,6 +132,10 @@ public:
   template <typename FaceRange>
   void operator()(const FaceRange& face_range)
   {
+#ifdef CGAL_PMP_SMOOTHING_DEBUG
+    std::cout << "Flipping edges" << std::endl;
+#endif
+
     // edges to consider
     std::vector<edge_descriptor> edge_range;
     edge_range.reserve(3 * face_range.size());
@@ -166,6 +175,10 @@ public:
         ++flipped_n;
 
         halfedge_descriptor h = halfedge(e, mesh_);
+
+#ifdef CGAL_PMP_SMOOTHING_DEBUG_PP
+        std::cout << "Flipping " << edge(h, mesh_) << std::endl;
+#endif
         Euler::flip_edge(h, mesh_);
 
         add_to_stack_if_unmarked(edge(next(h, mesh_), mesh_), marks, edge_range);
@@ -174,6 +187,10 @@ public:
         add_to_stack_if_unmarked(edge(prev(opposite(h, mesh_), mesh_), mesh_), marks, edge_range);
       }
     }
+
+#ifdef CGAL_PMP_SMOOTHING_DEBUG
+    std::cout << flipped_n << " flips" << std::endl;
+#endif
   }
 
 private:
@@ -190,6 +207,7 @@ class Angle_smoother
   typedef typename boost::graph_traits<TriangleMesh>::halfedge_descriptor  halfedge_descriptor;
 
   typedef typename boost::property_traits<VertexPointMap>::reference       Point_ref;
+  typedef typename GeomTraits::FT                                          FT;
   typedef typename GeomTraits::Vector_3                                    Vector;
 
   typedef std::pair<halfedge_descriptor, halfedge_descriptor>              He_pair;
@@ -231,7 +249,7 @@ public:
   Vector operator()(const vertex_descriptor v) const
   {
     Vector move = CGAL::NULL_VECTOR;
-    double weights_sum = 0.;
+    FT weights_sum = FT(0);
 
     for(halfedge_descriptor main_he : halfedges_around_source(v, mesh_))
     {
@@ -249,26 +267,30 @@ public:
       Vector right_v(pt, right_pt);
 
       // rotate
-      double angle = get_radian_angle(right_v, left_v, traits_);
-      CGAL_warning(angle != 0.); // no degenerate faces is a precondition
-      if(angle == 0.)
-        continue;
-
       Vector bisector = rotate_edge(main_he, incident_pair);
-      double scaling_factor = CGAL::approximate_sqrt(
-                                traits_.compute_squared_distance_3_object()(get(vpmap_, source(main_he, mesh_)),
-                                                                            get(vpmap_, target(main_he, mesh_))));
+      FT scaling_factor = CGAL::approximate_sqrt(
+                            traits_.compute_squared_distance_3_object()(get(vpmap_, source(main_he, mesh_)),
+                                                                        get(vpmap_, target(main_he, mesh_))));
       bisector = traits_.construct_scaled_vector_3_object()(bisector, scaling_factor);
       Vector ps_psi(ps, traits_.construct_translated_point_3_object()(pt, bisector));
 
+      FT angle = get_radian_angle(right_v, left_v, traits_);
+      if(angle == FT(0))
+      {
+        // no degenerate faces is a precondition, angle can be 0 but it should be a numerical error
+        CGAL_warning(!is_degenerate_triangle_face(face(main_he, mesh_), mesh_));
+
+        return ps_psi; // since a small angle gives more weight, a null angle give priority (?)
+      }
+
       // small angles carry more weight
-      double weight = 1. / (angle*angle);
+      FT weight = 1. / CGAL::square(angle);
       weights_sum += weight;
 
       move += weight * ps_psi;
     }
 
-    if(weights_sum != 0.)
+    if(weights_sum != FT(0))
      move /= weights_sum;
 
     return move;
@@ -288,6 +310,7 @@ class Area_smoother
 
   typedef typename boost::property_traits<VertexPointMap>::value_type      Point;
   typedef typename boost::property_traits<VertexPointMap>::reference       Point_ref;
+  typedef typename GeomTraits::FT                                          FT;
   typedef typename GeomTraits::Vector_3                                    Vector;
 
 public:
@@ -298,27 +321,27 @@ public:
   { }
 
 private:
-  double element_area(const vertex_descriptor v1,
-                      const vertex_descriptor v2,
-                      const vertex_descriptor v3) const
+  FT element_area(const vertex_descriptor v1,
+                  const vertex_descriptor v2,
+                  const vertex_descriptor v3) const
   {
-    return CGAL::to_double(CGAL::approximate_sqrt(traits_.compute_squared_area_3_object()(get(vpmap_, v1),
-                                                                                          get(vpmap_, v2),
-                                                                                          get(vpmap_, v3))));
+    return CGAL::approximate_sqrt(traits_.compute_squared_area_3_object()(get(vpmap_, v1),
+                                                                          get(vpmap_, v2),
+                                                                          get(vpmap_, v3)));
   }
 
-  double element_area(const Point& P,
-                      const vertex_descriptor v2,
-                      const vertex_descriptor v3) const
+  FT element_area(const Point& P,
+                  const vertex_descriptor v2,
+                  const vertex_descriptor v3) const
   {
-    return CGAL::to_double(CGAL::approximate_sqrt(traits_.compute_squared_area_3_object()(P,
-                                                                                          get(vpmap_, v2),
-                                                                                          get(vpmap_, v3))));
+    return CGAL::approximate_sqrt(traits_.compute_squared_area_3_object()(P,
+                                                                          get(vpmap_, v2),
+                                                                          get(vpmap_, v3)));
   }
 
-  double compute_average_area_around(const vertex_descriptor v) const
+  FT compute_average_area_around(const vertex_descriptor v) const
   {
-    double sum_areas = 0.;
+    FT sum_areas = 0;
     unsigned int number_of_edges = 0;
 
     for(halfedge_descriptor h : halfedges_around_source(v, mesh_))
@@ -327,7 +350,7 @@ private:
       vertex_descriptor vi = source(next(h, mesh_), mesh_);
       vertex_descriptor vj = target(next(h, mesh_), mesh_);
 
-      double S = element_area(v, vi, vj);
+      FT S = element_area(v, vi, vj);
       sum_areas += S;
       ++number_of_edges;
     }
@@ -337,7 +360,7 @@ private:
 
   struct Face_energy
   {
-    Face_energy(const Point& pi, const Point& pj, const double s_av)
+    Face_energy(const Point& pi, const Point& pj, const FT s_av)
       :
         qx(pi.x()), qy(pi.y()), qz(pi.z()),
         rx(pj.x()), ry(pj.y()), rz(pj.z()),
@@ -346,7 +369,7 @@ private:
 
     // next two functions are just for convencience, the only thing ceres cares about is the operator()
     template <typename T>
-    double area(const T x, const T y, const T z) const
+    FT area(const T x, const T y, const T z) const
     {
       return CGAL::approximate_sqrt(CGAL::squared_area(Point(x, y, z),
                                                        Point(qx, qy, qz),
@@ -354,7 +377,7 @@ private:
     }
 
     template <typename T>
-    double evaluate(const T x, const T y, const T z) const { return area(x, y, z) - s_av; }
+    FT evaluate(const T x, const T y, const T z) const { return area(x, y, z) - s_av; }
 
     template <typename T>
     bool operator()(const T* const x, const T* const y, const T* const z,
@@ -390,9 +413,9 @@ private:
     }
 
   private:
-    const double qx, qy, qz;
-    const double rx, ry, rz;
-    const double s_av;
+    const FT qx, qy, qz;
+    const FT rx, ry, rz;
+    const FT s_av;
   };
 
 public:
@@ -401,12 +424,12 @@ public:
 #ifdef CGAL_PMP_USE_CERES_SOLVER
     const Point_ref vp = get(vpmap_, v);
 
-    const double S_av = compute_average_area_around(v);
+    const FT S_av = compute_average_area_around(v);
 
-    const double initial_x = vp.x();
-    const double initial_y = vp.y();
-    const double initial_z = vp.z();
-    double x = initial_x, y = initial_y, z = initial_z;
+    const FT initial_x = vp.x();
+    const FT initial_y = vp.y();
+    const FT initial_z = vp.z();
+    FT x = initial_x, y = initial_y, z = initial_z;
 
     ceres::Problem problem;
 
@@ -523,7 +546,8 @@ public:
     Optimizer compute_move(mesh_, vpmap_, traits_);
 
 #ifdef CGAL_PMP_SMOOTHING_DEBUG
-    double total_displacement = 0;
+    FT total_displacement = 0;
+    std::cout << "apply_moves_in_single_batch: " << apply_moves_in_single_batch << std::endl;
 #endif
 
     std::size_t moved_points = 0;
@@ -532,6 +556,10 @@ public:
       if(is_border(v, mesh_) || is_constrained(v))
         continue;
 
+#ifdef CGAL_PMP_SMOOTHING_DEBUG_PP
+      std::cout << "Considering " << v << " pos: " << get(vpmap_, v) << std::endl;
+#endif
+
       // compute normal to v
       Vector vn = compute_vertex_normal(v, mesh_, CGAL::parameters::vertex_point_map(vpmap_)
                                                                    .geom_traits(traits_));
@@ -542,17 +570,17 @@ public:
 
       // Gram Schmidt so that the new location is on the tangent plane of v (i.e. do mv -= (mv*n)*n)
       const FT sp = traits_.compute_scalar_product_3_object()(vn, move);
-      move = traits_.construct_sum_of_vectors_3_object()(move,
-                                                         traits_.construct_scaled_vector_3_object()(vn, - sp));
-
-      Point new_pos = pos + move;
+      move = traits_.construct_sum_of_vectors_3_object()(
+               move, traits_.construct_scaled_vector_3_object()(vn, - sp));
 
-      if((!use_sanity_checks || !does_move_create_bad_faces(v, new_pos)) &&
+      const Point new_pos = pos + move;
+      if(move != CGAL::NULL_VECTOR &&
+         !does_move_create_degenerate_faces(v, new_pos) &&
+         (!use_sanity_checks || !does_move_create_bad_faces(v, new_pos)) &&
          (!enforce_no_min_angle_regression || does_improve_min_angle_in_star(v, new_pos)))
       {
-#ifdef CGAL_PMP_SMOOTHING_DEBUG
+#ifdef CGAL_PMP_SMOOTHING_DEBUG_PP
         std::cout << "moving " << get(vpmap_, v) << " to " << new_pos << std::endl;
-        total_displacement += CGAL::approximate_sqrt(traits_.compute_squared_length_3_object()(move));
 #endif
 
         if(apply_moves_in_single_batch)
@@ -560,11 +588,15 @@ public:
         else
           put(vpmap_, v, new_pos);
 
+#ifdef CGAL_PMP_SMOOTHING_DEBUG
+        total_displacement += CGAL::approximate_sqrt(traits_.compute_squared_length_3_object()(move));
+#endif
+
         ++moved_points;
       }
       else // some sanity check failed
       {
-#ifdef CGAL_PMP_SMOOTHING_DEBUG
+#ifdef CGAL_PMP_SMOOTHING_DEBUG_PP
         std::cout << "move is rejected!" << std::endl;
 #endif
         if(apply_moves_in_single_batch)
@@ -607,6 +639,10 @@ public:
 
       Point_ref p_query = get(vpmap_, v);
       const Point projected = tree.closest_point(p_query);
+#ifdef CGAL_PMP_SMOOTHING_DEBUG_PP
+      std::cout << p_query << " to " << projected << std::endl;
+#endif
+
       put(vpmap_, v, projected);
     }
   }
@@ -617,19 +653,35 @@ private:
     return get(vcmap_, v);
   }
 
+  // Null faces are bad because they make normal computation difficult
+  bool does_move_create_degenerate_faces(const vertex_descriptor v,
+                                         const Point& new_pos) const
+  {
+    for(halfedge_descriptor main_he : halfedges_around_source(v, mesh_))
+    {
+      const halfedge_descriptor prev_he = prev(main_he, mesh_);
+      const Point_ref lpt = get(vpmap_, target(main_he, mesh_));
+      const Point_ref rpt = get(vpmap_, source(prev_he, mesh_));
+
+      if(traits_.collinear_3_object()(lpt, rpt, new_pos))
+        return true;
+    }
+
+    return false;
+  }
+
   // check for degenerate or inversed faces
   bool does_move_create_bad_faces(const vertex_descriptor v,
                                   const Point& new_pos) const
   {
-    // check for null faces and face inversions
+    // check for face inversions
     for(halfedge_descriptor main_he : halfedges_around_source(v, mesh_))
     {
       const halfedge_descriptor prev_he = prev(main_he, mesh_);
       const Point_ref lpt = get(vpmap_, target(main_he, mesh_));
       const Point_ref rpt = get(vpmap_, source(prev_he, mesh_));
 
-      if(traits_.collinear_3_object()(lpt, rpt, new_pos))
-        return true;
+      CGAL_assertion(!traits_.collinear_3_object()(lpt, rpt, new_pos)); // checked above
 
       const Point_ref old_pos = get(vpmap_, v);
       Vector ov_1 = traits_.construct_vector_3_object()(old_pos, lpt);
@@ -641,7 +693,7 @@ private:
 
       if(!is_positive(traits_.compute_scalar_product_3_object()(old_n, new_n)))
       {
-#ifdef CGAL_PMP_SMOOTHING_DEBUG
+#ifdef CGAL_PMP_SMOOTHING_DEBUG_PP
       std::cout << "Moving vertex would result in the inversion of a face normal!" << std::endl;
 #endif
         return true;
@@ -655,7 +707,7 @@ private:
                                       const Point& new_pos) const
   {
     // check if the minimum angle of the star has not deteriorated
-    double old_min_angle = CGAL_PI;
+    FT old_min_angle = CGAL_PI;
     for(halfedge_descriptor main_he : halfedges_around_source(v, mesh_))
     {
       const Point_ref old_pos = get(vpmap_, v);
@@ -683,7 +735,7 @@ private:
          get_radian_angle(Vector(lpt, rpt), Vector(lpt, new_pos), traits_) < old_min_angle ||
          get_radian_angle(Vector(rpt, new_pos), Vector(rpt, lpt), traits_) < old_min_angle)
       {
-#ifdef CGAL_PMP_SMOOTHING_DEBUG
+#ifdef CGAL_PMP_SMOOTHING_DEBUG_PP
         const Point_ref old_pos = get(vpmap_, v);
 
         std::cout << "deterioration of min angle in the star!" << std::endl;
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Smoothing/smoothing_evaluation.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Smoothing/smoothing_evaluation.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Smoothing/smoothing_evaluation.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Smoothing/smoothing_evaluation.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Smoothing/smoothing_evaluation.h $
-// $Id: smoothing_evaluation.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Smoothing/smoothing_evaluation.h $
+// $Id: smoothing_evaluation.h f33618e 2020-02-06T09:54:35+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -61,7 +61,7 @@ public:
       angles_.push_back(traits_.compute_approximate_angle_3_object()(a, b, c));
     }
 
-#ifdef CGAL_PMP_SMOOTHING_VERBOSE
+#ifdef CGAL_PMP_SMOOTHING_DEBUG
     std::cout << "angles_ size = " << angles_.size() << std::endl;
 #endif
   }
@@ -79,7 +79,7 @@ public:
     for(face_descriptor f : faces(mesh_))
       areas_.push_back(face_area(f, mesh_));
 
-#ifdef CGAL_PMP_SMOOTHING_VERBOSE
+#ifdef CGAL_PMP_SMOOTHING_DEBUG
     std::cout << "areas_ size = " << areas_.size() << std::endl;
 #endif
   }
@@ -97,7 +97,7 @@ public:
     for(face_descriptor f : faces(mesh_))
       aspect_ratios_.push_back(CGAL::Polygon_mesh_processing::face_aspect_ratio(f, mesh_));
 
-#ifdef CGAL_PMP_SMOOTHING_VERBOSE
+#ifdef CGAL_PMP_SMOOTHING_DEBUG
     std::cout << "aspect_ratios_ size = " << aspect_ratios_.size() << std::endl;
 #endif
   }
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Snapping/helper.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Snapping/helper.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Snapping/helper.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Snapping/helper.h	2020-09-08 12:15:23.000000000 +0000
@@ -0,0 +1,136 @@
+// Copyright (c) 2018, 2019 GeometryFactory (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Snapping/helper.h $
+// $Id: helper.h 0133e9f 2020-03-06T11:41:31+01:00 Mael Rouxel-Labbé
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Mael Rouxel-Labbé
+
+#ifndef CGAL_POLYGON_MESH_PROCESSING_INTERNAL_SNAPPING_HELPER_H
+#define CGAL_POLYGON_MESH_PROCESSING_INTERNAL_SNAPPING_HELPER_H
+
+#include <CGAL/license/Polygon_mesh_processing/repair.h>
+
+#include <CGAL/boost/graph/iterator.h>
+#include <CGAL/number_utils.h>
+#include <CGAL/Polygon_mesh_processing/internal/named_function_params.h>
+#include <CGAL/Polygon_mesh_processing/internal/named_params_helper.h>
+
+namespace CGAL {
+namespace Polygon_mesh_processing {
+namespace internal {
+
+template <typename VertexRange,
+          typename HalfedgeOutputIterator,
+          typename PolygonMesh>
+void vertices_as_halfedges(const VertexRange& vertex_range,
+                           const PolygonMesh& pmesh,
+                           HalfedgeOutputIterator out)
+{
+  typedef typename boost::graph_traits<PolygonMesh>::vertex_descriptor                vertex_descriptor;
+
+  for(vertex_descriptor v : vertex_range)
+    *out++ = halfedge(v, pmesh);
+}
+
+// Assigns at each vertex the 'tolerance' value as tolerance, but bounded by a percentage of the length of its shortest incident edge
+template <typename HalfedgeRange,
+          typename ToleranceMap,
+          typename PolygonMesh,
+          typename NamedParameters>
+void assign_tolerance_with_local_edge_length_bound(const HalfedgeRange& halfedge_range,
+                                                   ToleranceMap& tolerance_map,
+                                                   const typename GetGeomTraits<PolygonMesh, NamedParameters>::type::FT tolerance,
+                                                   PolygonMesh& mesh,
+                                                   const NamedParameters& np)
+{
+  typedef typename boost::graph_traits<PolygonMesh>::vertex_descriptor                vertex_descriptor;
+  typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor              halfedge_descriptor;
+
+  typedef typename GetVertexPointMap<PolygonMesh, NamedParameters>::type              VPM;
+  typedef typename GetGeomTraits<PolygonMesh, NamedParameters>::type                  GT;
+  typedef typename GT::FT                                                             FT;
+
+  using parameters::get_parameter;
+  using parameters::choose_parameter;
+
+  GT gt = choose_parameter<GT>(get_parameter(np, internal_np::geom_traits));
+  VPM vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
+                             get_property_map(vertex_point, mesh));
+
+  for(halfedge_descriptor hd : halfedge_range)
+  {
+    const vertex_descriptor vd = target(hd, mesh);
+    CGAL::Halfedge_around_target_iterator<PolygonMesh> hit, hend;
+    boost::tie(hit, hend) = CGAL::halfedges_around_target(vd, mesh);
+    CGAL_assertion(hit != hend);
+
+    FT sq_length = gt.compute_squared_distance_3_object()(get(vpm, source(*hit, mesh)),
+                                                          get(vpm, target(*hit, mesh)));
+    FT min_sq_dist = sq_length;
+    ++hit;
+
+    for(; hit!=hend; ++hit)
+    {
+      sq_length = gt.compute_squared_distance_3_object()(get(vpm, source(*hit, mesh)),
+                                                         get(vpm, target(*hit, mesh)));
+
+      if(sq_length < min_sq_dist)
+        min_sq_dist = sq_length;
+    }
+
+#ifdef CGAL_PMP_SNAP_DEBUG_PP
+    std::cout << "tolerance at vd: " << vd << " [" << get(vpm, vd) << "]: min of "
+              << 0.9 * CGAL::approximate_sqrt(min_sq_dist) << " AND " << tolerance << std::endl;
+#endif
+    put(tolerance_map, vd, CGAL::min<FT>(0.9 * CGAL::approximate_sqrt(min_sq_dist), tolerance));
+  }
+}
+
+template <typename HalfedgeRange,
+          typename ToleranceMap,
+          typename PolygonMesh>
+void assign_tolerance_with_local_edge_length_bound(const HalfedgeRange& halfedge_range,
+                                                   ToleranceMap& tolerance_map,
+                                                   const typename GetGeomTraits<PolygonMesh>::type::FT tolerance,
+                                                   PolygonMesh& mesh)
+{
+  return assign_tolerance_with_local_edge_length_bound(halfedge_range, tolerance_map, tolerance, mesh, CGAL::parameters::all_default());
+}
+
+template <typename GeomTraits>
+bool is_collinear_with_tolerance(const typename GeomTraits::Point_3& p, // va == vb
+                                 const typename GeomTraits::Point_3& pa,
+                                 const typename GeomTraits::Point_3& pb,
+                                 const GeomTraits& gt)
+{
+  typedef typename GeomTraits::FT                                                     FT;
+  typedef typename GeomTraits::Vector_3                                               Vector_3;
+
+  const Vector_3 va = gt.construct_vector_3_object()(p, pa);
+  const Vector_3 vb = gt.construct_vector_3_object()(p, pb);
+  const FT sp = gt.compute_scalar_product_3_object()(va, vb);
+
+  // To avoid junctions like:
+  //  ----va vb-----  from being locked since it could be needed in ------------- later
+  //       | |                                                      ----va  vb---
+  //                                                                    |   |
+  if(sp < FT(0))
+    return false;
+
+  const FT sq_va_l = gt.compute_squared_distance_3_object()(p, pa);
+  const FT sq_vb_l = gt.compute_squared_distance_3_object()(p, pb);
+  const FT sq_cos = 0.99999228397046306; // CGAL::square(std::cos(1°));
+
+  return (CGAL::square(sp) >= sq_va_l * sq_vb_l * sq_cos);
+}
+
+} // namespace internal
+} // namespace Polygon_mesh_processing
+} // namespace CGAL
+
+#endif // CGAL_POLYGON_MESH_PROCESSING_INTERNAL_SNAPPING_HELPER_H
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Snapping/snap.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Snapping/snap.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Snapping/snap.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Snapping/snap.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,43 +3,54 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Snapping/snap.h $
-// $Id: snap.h 5b2d46c 2020-04-21T10:03:00+02:00 Maxime Gimeno
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Snapping/snap.h $
+// $Id: snap.h 038869c 2020-05-12T15:12:21+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
-// Author(s)     : Sebastien Loriot
-//                 Mael Rouxel-Labbé
+// Author(s)     : Mael Rouxel-Labbé
+//                 Sebastien Loriot
 
-#ifndef CGAL_POLYGON_MESH_PROCESSING_INTERNAL_SNAP_H
-#define CGAL_POLYGON_MESH_PROCESSING_INTERNAL_SNAP_H
+#ifndef CGAL_POLYGON_MESH_PROCESSING_SNAPPING_SNAP_H
+#define CGAL_POLYGON_MESH_PROCESSING_SNAPPING_SNAP_H
 
 #include <CGAL/license/Polygon_mesh_processing/repair.h>
 
+#ifdef CGAL_PMP_SNAP_DEBUG_PP
+ #ifndef CGAL_PMP_SNAP_DEBUG
+  #define CGAL_PMP_SNAP_DEBUG
+ #endif
+#endif
+
+#include <CGAL/Polygon_mesh_processing/internal/Snapping/helper.h>
+#include <CGAL/Polygon_mesh_processing/internal/Snapping/snap_vertices.h>
+
 #include <CGAL/Polygon_mesh_processing/internal/named_function_params.h>
 #include <CGAL/Polygon_mesh_processing/internal/named_params_helper.h>
 #include <CGAL/Polygon_mesh_processing/border.h>
+#include <CGAL/Polygon_mesh_processing/compute_normal.h>
 
 #include <CGAL/AABB_tree.h>
 #include <CGAL/AABB_traits.h>
 #include <CGAL/AABB_halfedge_graph_segment_primitive.h>
 #include <CGAL/assertions.h>
-#include <CGAL/Bbox_3.h>
 #include <CGAL/boost/graph/helpers.h>
 #include <CGAL/boost/graph/iterator.h>
-#include <CGAL/box_intersection_d.h>
+#include <CGAL/boost/graph/selection.h>
 #include <CGAL/circulator.h>
 #include <CGAL/Dynamic_property_map.h>
 #include <CGAL/Kernel/global_functions.h>
 #include <CGAL/number_utils.h>
+#include <CGAL/Real_timer.h>
+#include <CGAL/utility.h>
 
-#include <boost/multi_index_container.hpp>
-#include <boost/multi_index/ordered_index.hpp>
-#include <boost/multi_index/member.hpp>
+#ifdef CGAL_EIGEN3_ENABLED
+#include <Eigen/Dense>
+#endif
 
 #ifdef CGAL_LINKED_WITH_TBB
-# include <tbb/concurrent_vector.h>
-# include <tbb/parallel_for.h>
+#include <tbb/concurrent_vector.h>
+#include <tbb/parallel_for.h>
 #endif
 
 #include <functional>
@@ -48,983 +59,1147 @@
 #include <limits>
 #include <map>
 #include <type_traits>
-#include <utility>
 #include <unordered_set>
+#include <utility>
 #include <vector>
 
+#ifdef CGAL_PMP_SNAP_DEBUG_OUTPUT
+#include <fstream>
+#endif
+
 namespace CGAL {
 namespace Polygon_mesh_processing {
 namespace internal {
 
-// Assigns at each vertex the 'tolerance' value as tolerance, but bounded by a percentage of the length of its shortest incident edge
-template <typename HalfedgeRange,
-          typename ToleranceMap,
-          typename PolygonMesh,
-          typename SourceNamedParameters>
-void assign_tolerance_with_local_edge_length_bound(const HalfedgeRange& hrange,
-                                                   ToleranceMap& tol_pmap,
-                                                   const typename GetGeomTraits<PolygonMesh, SourceNamedParameters>::type::FT tolerance,
-                                                   PolygonMesh& mesh,
-                                                   const SourceNamedParameters& snp)
-{
-  using parameters::get_parameter;
-  using parameters::choose_parameter;
-
-  typedef typename boost::graph_traits<PolygonMesh>::vertex_descriptor                vertex_descriptor;
-  typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor              halfedge_descriptor;
+template <typename HalfedgeRange, typename TriangleMesh,
+          typename ToleranceMap, typename NamedParameters>
+void simplify_range(HalfedgeRange& halfedge_range,
+                    TriangleMesh& tm,
+                    ToleranceMap tolerance_map,
+                    const NamedParameters& np)
+{
+  // @todo the simplification below is too naive and will fail to treat complicated zigzaging
+  // with long edges.
+  // A real simplification would create a fuzzy area around the border (say the dilation
+  // of the border polyline by a sphere of radius that continuously interpolates the tolerance
+  // at each vertex). The border is then simplified to use the least amount of edges possible
+  // in that fuzzy zone. Need to look into cartography papers, probably.
 
-  typedef typename GetVertexPointMap<PolygonMesh, SourceNamedParameters>::type        SVPM;
-  typedef typename GetGeomTraits<PolygonMesh, SourceNamedParameters>::type            GT;
-  typedef typename GT::FT                                                             FT;
-
-  GT gt = choose_parameter(get_parameter(snp, internal_np::geom_traits), GT());
-  SVPM svpm = choose_parameter(get_parameter(snp, internal_np::vertex_point),
-                               get_property_map(vertex_point, mesh));
-
-  for(halfedge_descriptor hd : hrange)
-  {
-    const vertex_descriptor vd = target(hd, mesh);
-    CGAL::Halfedge_around_target_iterator<PolygonMesh> hit, hend;
-    boost::tie(hit, hend) = CGAL::halfedges_around_target(vd, mesh);
-    CGAL_assertion(hit != hend);
-
-    FT sq_length = gt.compute_squared_distance_3_object()(get(svpm, source(*hit, mesh)),
-                                                          get(svpm, target(*hit, mesh)));
-    FT min_sq_dist = sq_length;
-    ++hit;
+  typedef typename boost::graph_traits<TriangleMesh>::vertex_descriptor           vertex_descriptor;
+  typedef typename boost::graph_traits<TriangleMesh>::halfedge_descriptor         halfedge_descriptor;
 
-    for(; hit!=hend; ++hit)
-    {
-      sq_length = gt.compute_squared_distance_3_object()(get(svpm, source(*hit, mesh)),
-                                                         get(svpm, target(*hit, mesh)));
+  typedef typename GetGeomTraits<TriangleMesh, NamedParameters>::type             GT;
+  typedef typename GT::FT                                                         FT;
 
-      if(sq_length < min_sq_dist)
-        min_sq_dist = sq_length;
-    }
-
-#ifdef CGAL_PMP_SNAP_DEBUG
-    std::cout << "tolerance at vd: " /*<< vd */ << " [" << get(svpm, vd) << "]: min of "
-              << 0.9 * CGAL::approximate_sqrt(min_sq_dist) << " AND " << tolerance << std::endl;
-#endif
-    put(tol_pmap, vd, CGAL::min<FT>(0.9 * CGAL::approximate_sqrt(min_sq_dist), tolerance));
-  }
-}
-
-template <typename HalfedgeRange,
-          typename ToleranceMap,
-          typename PolygonMesh>
-void assign_tolerance_with_local_edge_length_bound(const HalfedgeRange& hrange,
-                                                   ToleranceMap& tol_pmap,
-                                                   const typename GetGeomTraits<PolygonMesh>::type::FT tolerance,
-                                                   PolygonMesh& mesh)
-{
-  return assign_tolerance_with_local_edge_length_bound(hrange, tol_pmap, tolerance, mesh, CGAL::parameters::all_default());
-}
+  typedef typename GetVertexPointMap<TriangleMesh, NamedParameters>::type         VPM;
+  typedef typename boost::property_traits<VPM>::reference                         Point_ref;
 
-template <typename PolygonMesh, typename GeomTraits>
-struct Snapping_pair
-{
-  typedef typename boost::graph_traits<PolygonMesh>::vertex_descriptor       vertex_descriptor;
-  typedef typename GeomTraits::FT                                            FT;
+  using parameters::get_parameter;
+  using parameters::choose_parameter;
 
-  Snapping_pair(const vertex_descriptor vs_, const vertex_descriptor vt_, const FT sq_dist_)
-    : vs(vs_), vt(vt_), sq_dist(sq_dist_)
-  { }
+  const GT gt = choose_parameter<GT>(get_parameter(np, internal_np::geom_traits));
+  VPM vpm = choose_parameter(get_parameter(np, internal_np::vertex_point), get_property_map(vertex_point, tm));
 
-  vertex_descriptor vs;
-  vertex_descriptor vt;
-  FT sq_dist;
-};
+  typedef CGAL::dynamic_halfedge_property_t<bool>                                 Halfedge_bool_tag;
+  typedef typename boost::property_map<TriangleMesh, Halfedge_bool_tag>::type     Range_halfedges;
 
-template <typename PolygonMesh, typename GeomTraits,
-          typename DistanceMultiIndexContainer,
-          typename SVPM, typename TVPM,
-          typename ToleranceMap,
-          typename Box>
-struct Vertex_proximity_report
-{
-  typedef typename boost::graph_traits<PolygonMesh>::vertex_descriptor       vertex_descriptor;
-  typedef typename boost::graph_traits<PolygonMesh>::edge_descriptor         edge_descriptor;
+  Range_halfedges range_halfedges = get(Halfedge_bool_tag(), tm);
+  for(halfedge_descriptor h : halfedge_range)
+    put(range_halfedges, h, true);
 
-  typedef typename GeomTraits::FT                                            FT;
-  typedef typename boost::property_traits<SVPM>::value_type                  Point;
+  CGAL_postcondition_code(const std::size_t initial_n = halfedge_range.size();)
 
-  Vertex_proximity_report(DistanceMultiIndexContainer& snapping_pairs,
-                          const SVPM& svpm, const PolygonMesh& smesh,
-                          const TVPM& tvpm, const PolygonMesh& tmesh,
-                          const ToleranceMap& tol_pmap,
-                          const GeomTraits& gt)
-    :
-      m_snapping_pairs(snapping_pairs),
-      is_same_mesh((&smesh == &tmesh)),
-      svpm(svpm), tvpm(tvpm),
-      tol_pmap(tol_pmap),
-      gt(gt)
-  { }
+  std::set<halfedge_descriptor> edges_to_test(halfedge_range.begin(), halfedge_range.end());
 
-  void operator()(const Box& a, const Box& b)
+  int collapsed_n = 0;
+  while(!edges_to_test.empty())
   {
-    vertex_descriptor va = a.info();
-    vertex_descriptor vb = b.info();
-
-    if(is_same_mesh && va == vb)
-      return;
+    const halfedge_descriptor h = *(edges_to_test.begin());
+    edges_to_test.erase(edges_to_test.begin());
 
-    const Point sp = get(svpm, va);
-    const Point tp = get(tvpm, vb);
-    const FT tol = get(tol_pmap, va);
-
-    // Don't reject a '0' distance, it still needs to lock the points in place
-    const FT sq_dist = gt.compute_squared_distance_3_object()(sp, tp);
-    CGAL::Comparison_result res = CGAL::compare(sq_dist, tol * tol);
-
-#ifdef CGAL_PMP_SNAP_DEBUG_PP
-    std::cout << "distance between " /*<< va*/ << " [" << sp << "] and "
-                                     /*<< vb*/ << " [" << tp << "]: " << sq_dist
-                                     << " (bound: " << tol*tol << ") larger? " << (res == CGAL::LARGER)
-                                     << std::endl;
-#endif
+    const vertex_descriptor vs = source(h, tm);
+    const vertex_descriptor vt = target(h, tm);
+    const Point_ref ps = get(vpm, vs);
+    const Point_ref pt = get(vpm, vt);
+
+    // @fixme what if the source vertex is not to be snapped? Tolerance cannot be obtained...
+    // and where should the post-collapse vertex be since we can't move the source vertex...
+    // --> simply don't collapse?
+    const FT min_tol = (std::min)(get(tolerance_map, vs), get(tolerance_map, vt));
+    const FT max_tol = (std::max)(get(tolerance_map, vs), get(tolerance_map, vt));
 
-    if(res == CGAL::LARGER)
-      return;
-
-    m_snapping_pairs.insert(Snapping_pair<PolygonMesh, GeomTraits>(va, vb, sq_dist));
-  }
-
-private:
-  DistanceMultiIndexContainer& m_snapping_pairs;
+    if(gt.compare_squared_distance_3_object()(ps,pt,CGAL::square(max_tol))==SMALLER)
+    {
+      const halfedge_descriptor prev_h = prev(h, tm);
+      const halfedge_descriptor next_h = next(h, tm);
 
-  const bool is_same_mesh;
-  const SVPM& svpm;
-  const TVPM& tvpm;
-  const ToleranceMap& tol_pmap;
-  const GeomTraits& gt;
-};
+      // check that the border has at least 4 edges not to create degenerate volumes
+      if(border_size(h, tm) >= 4)
+      {
+        const FT h_sq_length = gt.compute_squared_distance_3_object()(ps, pt);
+        vertex_descriptor v = Euler::collapse_edge(edge(h, tm), tm);
 
-} // namespace internal
+        put(vpm, v, gt.construct_midpoint_3_object()(ps, pt));
+        put(tolerance_map, v, min_tol + 0.5 * CGAL::approximate_sqrt(h_sq_length));
 
-namespace experimental {
+        if(get(range_halfedges, prev_h))
+          edges_to_test.insert(prev_h);
+        if(get(range_halfedges, next_h))
+          edges_to_test.insert(next_h);
 
-// This is the function if you know what you're doing with the ranges
-//
-// \ingroup PMP_repairing_grp
-//
-// Attempts to snap the vertices in `source_hrange` onto the vertices in `target_hrange`.
-// A vertex of the source range is only snapped to a vertex of the target mesh
-// if its distance to the target mesh vertex is smaller than a user-chosen bound.
-// If a source vertex can be snapped onto multiple vertices of the target
-// range, the closest target vertex is used.
-// If multiple vertices within the source range could be snapped onto the same target vertex,
-// the source vertex closest to the target vertex will be snapped. Other source vertices will try to snap
-// to free target vertices until they find a valid target, or until there is no more target
-// to snap to (within the tolerance).
-//
-// @warning This function does not give any guarantee on the conformity between the source and target meshes after the snapping.
-// @warning This function does not merge vertices together.
-//
-// @tparam PolygonMesh a model of `FaceListGraph` and `MutableFaceGraph`
-// @tparam SourceHalfedgeRange a model of `Range` with value type `boost::graph_traits<PolygonMesh>::%halfedge_descriptor`
-// @tparam TargetHalfedgeRange a model of `Range` with value type `boost::graph_traits<PolygonMesh>::%halfedge_descriptor`
-// @tparam ToleranceMap a model of `ReadablePropertyMap` with key type `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
-//                      and value type `GetGeomTraits<PolygonMesh, SourceNamedParameters>::type::FT`
-// @tparam SourceNamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
-// @tparam TargetNamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
-//
-// @param source_hrange a range of vertices of the source mesh whose positions can be changed.
-//                      the vertices must be border vertices of `smesh`.
-// @param smesh the source mesh whose border vertices might be moved
-// @param target_hrange a range of vertices of the target mesh which are potential new positions
-//                      for the vertices in the source range
-// @param tmesh the target mesh to which the vertices in `target_hrange` belong
-// @param tol_pmap a tolerance map associating to each vertex of the source range a tolerance value:
-//               potential projection targets are sought in a sphere centered at the vertex and
-//               whose radius is the tolerance value.
-// @param snp optional \ref pmp_namedparameters "Named Parameters" related to the source mesh,
-//            amongst those described below:
-//
-// \cgalNamedParamsBegin
-//    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of the source mesh.
-//                                      The type of this map is model of `ReadWritePropertyMap`.
-//                                      If this parameter is omitted, an internal property map for
-//                                      `CGAL::vertex_point_t` must be available in `PolygonMesh`
-//    \cgalParamEnd
-//    \cgalParamBegin{geom_traits} a geometric traits class instance.
-//       The traits class must provide the nested types `Point_3` and `Vector_3`,
-//       and the nested functors :
-//         - `Construct_bbox_3` to construct a bounding box of a point,
-//         - `Compute_squared_distance_3` to compute the distance between two points,
-//
-//       and, for each functor `Foo`, a function `Foo foo_object()`
-//   \cgalParamEnd
-// \cgalNamedParamsEnd
-//
-// @param tnp optional \ref pmp_namedparameters "Named Parameters" related to the target mesh,
-//            amongst those described below:
-//
-// \cgalNamedParamsBegin
-//    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of the target mesh.
-//                                      The type of this map is model of `ReadablePropertyMap`.
-//                                      If this parameter is omitted, an internal property map for
-//                                      `CGAL::vertex_point_t` must be available in `PolygonMesh`
-//    \cgalParamEnd
-// \cgalNamedParamsEnd
-//
-// @return the number of snapped vertices
-//
-// @sa `merge_duplicated_vertices_in_boundary_cycles()`
-//
-template <typename PolygonMesh,
-          typename SourceHalfedgeRange, typename TargetHalfedgeRange,
-          typename ToleranceMap,
-          typename SourceNamedParameters, typename TargetNamedParameters>
-std::size_t snap_vertex_range_onto_vertex_range(const SourceHalfedgeRange& source_hrange,
-                                                PolygonMesh& smesh,
-                                                const TargetHalfedgeRange& target_hrange,
-                                                const PolygonMesh& tmesh,
-                                                const ToleranceMap& tol_pmap,
-                                                const SourceNamedParameters& snp,
-                                                const TargetNamedParameters& tnp)
-{
-  using parameters::get_parameter;
-  using parameters::choose_parameter;
-
-  typedef typename boost::graph_traits<PolygonMesh>::vertex_descriptor      vertex_descriptor;
-  typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor    halfedge_descriptor;
+        ++collapsed_n;
+      }
+    }
+  }
 
-  typedef CGAL::Box_intersection_d::Box_with_info_d<double, 3, vertex_descriptor>     Box;
+#ifdef CGAL_PMP_SNAP_DEBUG
+  std::cout << "collapsed " << collapsed_n << " edges" << std::endl;
+#endif
 
-  typedef typename GetVertexPointMap<PolygonMesh, SourceNamedParameters>::type        SVPM;
-  typedef typename GetVertexPointMap<PolygonMesh, TargetNamedParameters>::const_type  TVPM;
-  typedef typename boost::property_traits<TVPM>::value_type                           Point;
+  std::vector<halfedge_descriptor> new_range;
+  new_range.reserve(halfedge_range.size());
 
-  typedef typename GetGeomTraits<PolygonMesh, SourceNamedParameters>::type            GT;
-  typedef typename GT::FT                                                             FT;
+  for(halfedge_descriptor h : halfedges(tm))
+    if(get(range_halfedges, h))
+      new_range.push_back(h);
 
-  if(is_empty_range(source_hrange.begin(), source_hrange.end()) ||
-     is_empty_range(target_hrange.begin(), target_hrange.end()))
-    return 0;
+  halfedge_range = HalfedgeRange(new_range.begin(), new_range.end());
 
-  CGAL_static_assertion((std::is_same<Point, typename GT::Point_3>::value));
+  CGAL_postcondition(halfedge_range.size() == initial_n - collapsed_n);
+}
 
-  GT gt = choose_parameter(get_parameter(snp, internal_np::geom_traits), GT());
+// Adapted from <CGAL/internal/AABB_tree/AABB_traversal_traits.h>
+template <typename TriangleMesh,
+          typename VPMS, typename VPMT, typename FacePatchMap,
+          typename AABBTraits>
+class Projection_traits
+{
+  typedef typename AABBTraits::FT                                          FT;
+  typedef typename AABBTraits::Point_3                                     Point;
+  typedef typename AABBTraits::Primitive                                   Primitive;
+  typedef typename AABBTraits::Bounding_box                                Bounding_box;
+  typedef typename AABBTraits::Primitive::Id                               Primitive_id;
+  typedef typename AABBTraits::Point_and_primitive_id                      Point_and_primitive_id;
+  typedef typename AABBTraits::Object_and_primitive_id                     Object_and_primitive_id;
+
+  typedef CGAL::AABB_node<AABBTraits>                                      Node;
+
+  typedef typename AABBTraits::Geom_traits                                 Geom_traits;
+  typedef typename Geom_traits::Vector_3                                   Vector;
+  typedef typename Geom_traits::Plane_3                                    Plane;
 
-  SVPM svpm = choose_parameter(get_parameter(snp, internal_np::vertex_point),
-                               get_property_map(vertex_point, smesh));
-  TVPM tvpm = choose_parameter(get_parameter(tnp, internal_np::vertex_point),
-                               get_const_property_map(vertex_point, tmesh));
+  typedef typename boost::graph_traits<TriangleMesh>::halfedge_descriptor  halfedge_descriptor;
+  typedef typename boost::graph_traits<TriangleMesh>::edge_descriptor      edge_descriptor;
 
-#ifdef CGAL_PMP_SNAP_DEBUG
-  std::cout << "Snapping vertices to vertices. Range sizes: "
-            << std::distance(source_hrange.begin(), source_hrange.end()) << " and "
-            << std::distance(target_hrange.begin(), target_hrange.end()) << std::endl;
-
-  if(&smesh == &tmesh)
-    std::cout << "same mesh!" << std::endl;
-#endif
-
-  // Try to snap vertices
-  std::vector<Box> boxes;
-  std::unordered_set<vertex_descriptor> unique_vertices;
-  for(halfedge_descriptor hd : source_hrange)
-  {
-    const vertex_descriptor vd = target(hd, smesh);
-    if(!unique_vertices.insert(vd).second)
-      continue; // if 'vd' appears multiple times on the border, move it only once
-
-    // only making the box a little larger, but the final tolerance is not changed
-    const double eps = 1.01 * CGAL::to_double(get(tol_pmap, vd));
-
-    const Bbox_3 pb = gt.construct_bbox_3_object()(get(svpm, vd));
-    const Bbox_3 b(pb.xmin() - eps, pb.ymin() - eps, pb.zmin() - eps,
-                   pb.xmax() + eps, pb.ymax() + eps, pb.zmax() + eps);
-    boxes.push_back(Box(b, vd));
-  }
-
-  std::vector<Box> target_boxes;
-  for(halfedge_descriptor hd : target_hrange)
-  {
-    const vertex_descriptor vd = target(hd, tmesh);
-    const Point& p = get(tvpm, vd);
-    target_boxes.push_back(Box(gt.construct_bbox_3_object()(p), vd));
-  }
-
-  // Use a multi index to sort easily by sources, targets, AND distance.
-  // Then, look up the distances in increasing order, and snap whenever the source and the target
-  // have both not been snapped yet.
-  typedef internal::Snapping_pair<PolygonMesh, GT>                                    Snapping_pair;
-  typedef boost::multi_index::multi_index_container<
-    Snapping_pair,
-    boost::multi_index::indexed_by<
-      boost::multi_index::ordered_non_unique<
-        BOOST_MULTI_INDEX_MEMBER(Snapping_pair, vertex_descriptor, vs)>,
-      boost::multi_index::ordered_non_unique<
-        BOOST_MULTI_INDEX_MEMBER(Snapping_pair, vertex_descriptor, vt)>,
-      boost::multi_index::ordered_non_unique<
-        BOOST_MULTI_INDEX_MEMBER(Snapping_pair, FT, sq_dist)>
-    >
-  >                                                                                   Snapping_pair_container;
-
-  typedef internal::Vertex_proximity_report<PolygonMesh, GT, Snapping_pair_container,
-                                            SVPM, TVPM, ToleranceMap, Box>            Reporter;
-
-  Snapping_pair_container snapping_pairs;
-  Reporter vpr(snapping_pairs, svpm, smesh, tvpm, tmesh, tol_pmap, gt);
-
-  // Shenanigans to pass a reference as callback (which is copied by value by 'box_intersection_d')
-  std::function<void(const Box&, const Box&)> callback(std::ref(vpr));
-
-  CGAL::box_intersection_d(boxes.begin(), boxes.end(),
-                           target_boxes.begin(), target_boxes.end(),
-                           callback);
+#ifdef CGAL_PMP_SNAP_USE_ANISOTROPIC_DISTANCE
+  void build_metric()
+  {
+    Vector hv(get(m_vpm_S, source(m_h, m_tm_S)), get(m_vpm_S, target(m_h, m_tm_S)));
+    Vector n = CGAL::Polygon_mesh_processing::compute_face_normal(face(opposite(m_h, m_tm_S), m_tm_S), m_tm_S,
+                                                                  CGAL::parameters::geom_traits(m_gt)
+                                                                                   .vertex_point_map(m_vpm_S));
+    Vector pn = m_gt.construct_cross_product_vector_3_object()(hv, n);
+    Plane pl(get(m_vpm_S, target(m_h, m_tm_S)), pn);
+
+    Vector b1 = pl.base1();
+    Vector b2 = pl.base2();
+
+    internal::normalize(b1, m_gt);
+    internal::normalize(b2, m_gt);
+    internal::normalize(pn, m_gt);
+
+    Eigen::Matrix3d eigen_m;
+    eigen_m(0,0) = b1.x(); eigen_m(0,1) = b2.x(); eigen_m(0,2) = pn.x();
+    eigen_m(1,0) = b1.y(); eigen_m(1,1) = b2.y(); eigen_m(1,2) = pn.y();
+    eigen_m(2,0) = b1.z(); eigen_m(2,1) = b2.z(); eigen_m(2,2) = pn.z();
+
+    Eigen::Matrix3d eigen_diag = Eigen::Matrix3d::Zero();
+    eigen_diag(0,0) = 1;
+    eigen_diag(1,1) = 1;
+    eigen_diag(2,2) = 100; // we scale by 1/sqrt(lambda_0) in the direction of n
 
-#ifdef CGAL_PMP_SNAP_DEBUG
-    std::cout << snapping_pairs.size() << " snappable pair(s)!" << std::endl;
+    Eigen::Matrix3d eigen_mtransp = eigen_m.transpose();
+    m_metric = eigen_m * eigen_diag * eigen_mtransp;
+  }
 #endif
 
-  if(snapping_pairs.empty())
-    return 0;
-
-  // Sorted views of the container
-  typedef typename Snapping_pair_container::template nth_index<0>::type   Container_by_source;
-  typedef typename Snapping_pair_container::template nth_index<1>::type   Container_by_target;
-  typedef typename Snapping_pair_container::template nth_index<2>::type   Container_by_distance;
-
-  Container_by_source& container_by_source = snapping_pairs.template get<0>();
-  Container_by_target& container_by_target = snapping_pairs.template get<1>();
-  Container_by_distance& container_by_dist = snapping_pairs.template get<2>();
-
-  std::size_t counter = 0;
-
-  CGAL_assertion_code(FT prev = -1;)
-  while(!container_by_dist.empty())
+  FT squared_anisotropic_distance(const Point& p, const Point& q) const
   {
-    const Snapping_pair& sp = *(container_by_dist.begin());
-    const vertex_descriptor vs = sp.vs;
-    const vertex_descriptor vt = sp.vt;
-    CGAL_assertion(sp.sq_dist >= prev);
-    CGAL_assertion_code(prev = sp.sq_dist;)
+#ifdef CGAL_PMP_SNAP_USE_ANISOTROPIC_DISTANCE
+    Vector pq(p, q);
+    Eigen::Vector3d ev;
+    ev(0) = pq.x();
+    ev(1) = pq.y();
+    ev(2) = pq.z();
+
+    return ev.transpose() * m_metric * ev;
+#else
+    return CGAL::squared_distance(p, q);
+#endif
+  }
 
+  // The idea behind this function is to give priority to projections that are parallel to the edge 'e0'
+  // whom the query is the target of. This is done by considering an (anisotropic) distance that
+  // increases the distance if we are not projecting along the vector orthogonal to both the edge 'e0'
+  // and to the normal of the face, and also by checking the scalar product between the edges.
+  // The latter is because the distance to the projection point does not entirely reflect
+  // the alignment of the edges.
+  //
+  // This is WIP and likely to evolve...
+  bool is_better_than_current_best(const FT sq_dist,
+                                   const FT scalar_product)
+  {
 #ifdef CGAL_PMP_SNAP_DEBUG_PP
-    std::cout << "Snapping " /*<< vs*/ << " (" << get(svpm, vs) << ") "
-              << " to " /*<< vt*/ << " (" << get(tvpm, vt) << ") at dist: " << sp.sq_dist << std::endl;
+    std::cout << "is_better_than_current_best()" << std::endl;
+    std::cout << "sq_dist: " << sq_dist << std::endl;
+    std::cout << "m_sq_adist: " << m_sq_adist << std::endl;
+    std::cout << "scalar products: " << scalar_product << " " << m_sp_with_closest_edge << std::endl;
 #endif
 
-    // Collect all the source vertices projecting onto that target vertex
-   ++counter;
-    put(svpm, vs, get(tvpm, vt));
+    // Automatically accept much closer tentative targets; automatically reject much farther tentative targets
+    const FT lambda = 4; // comparing squared distances
+    if(lambda * sq_dist < m_sq_adist)
+      return true;
+    if(lambda * m_sq_adist < sq_dist)
+      return false;
 
-    // 'vs' and 'vt' cannot be used anymore, remove them from the container
-    container_by_source.erase(vs);
-    container_by_target.erase(vt);
+    return (scalar_product > m_sp_with_closest_edge);
   }
 
-  return counter;
-}
-
-template <typename PolygonMesh, typename SourceHalfedgeRange, typename TargetHalfedgeRange,
-          typename SourceNamedParameters, typename TargetNamedParameters>
-std::size_t snap_vertex_range_onto_vertex_range(const SourceHalfedgeRange& source_hrange,
-                                                PolygonMesh& smesh,
-                                                const TargetHalfedgeRange& target_hrange,
-                                                const PolygonMesh& tmesh,
-                                                const SourceNamedParameters& snp,
-                                                const TargetNamedParameters& tnp)
-{
-  typedef typename GetGeomTraits<PolygonMesh, SourceNamedParameters>::type            GT;
-  typedef typename GT::FT                                                             FT;
-  typedef CGAL::dynamic_vertex_property_t<FT>                                         Vertex_property_tag;
-  typedef typename boost::property_map<PolygonMesh, Vertex_property_tag>::type        Tolerance_map;
-
-  Tolerance_map tol_pmap = get(Vertex_property_tag(), smesh);
-  const FT tol_mx(std::numeric_limits<double>::max());
-  internal::assign_tolerance_with_local_edge_length_bound(source_hrange, tol_pmap, tol_mx, smesh, snp);
-
-  return snap_vertex_range_onto_vertex_range(source_hrange, smesh, target_hrange, tmesh, tol_pmap, snp, tnp);
-}
-
-template <typename PolygonMesh, typename SourceHalfedgeRange, typename TargetHalfedgeRange, typename ToleranceMap>
-std::size_t snap_vertex_range_onto_vertex_range(const SourceHalfedgeRange& source_hrange,
-                                                PolygonMesh& smesh,
-                                                const TargetHalfedgeRange& target_hrange,
-                                                const PolygonMesh& tmesh,
-                                                const ToleranceMap& tol_pmap)
-{
-  return snap_vertex_range_onto_vertex_range(source_hrange, smesh, target_hrange, tmesh, tol_pmap,
-                                             CGAL::parameters::all_default(),
-                                             CGAL::parameters::all_default());
-}
-
-template <typename PolygonMesh, typename SourceHalfedgeRange, typename TargetHalfedgeRange>
-std::size_t snap_vertex_range_onto_vertex_range(const SourceHalfedgeRange& source_hrange,
-                                                PolygonMesh& smesh,
-                                                const TargetHalfedgeRange& target_hrange,
-                                                const PolygonMesh& tmesh)
-{
-  return snap_vertex_range_onto_vertex_range(source_hrange, smesh, target_hrange, tmesh,
-                                             CGAL::parameters::all_default(),
-                                             CGAL::parameters::all_default());
-}
-
-template <typename PolygonMesh>
-std::size_t snap_vertex_range_onto_vertex_range(PolygonMesh& smesh, const PolygonMesh& tmesh)
-{
-  return snap_vertex_range_onto_vertex_range(halfedges(smesh), smesh, halfedges(tmesh), tmesh);
-}
-
-template <typename PolygonMesh, typename HalfedgeRange, typename ToleranceMap>
-std::size_t snap_border_vertices_onto_vertex_range(PolygonMesh& smesh,
-                                                   const HalfedgeRange& target_hrange,
-                                                   const PolygonMesh& tmesh,
-                                                   const ToleranceMap& tol_pmap)
-{
-  typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor    halfedge_descriptor;
-
-  std::vector<halfedge_descriptor> border_vertices;
-  border_halfedges(smesh, std::back_inserter(border_vertices));
-
-  return snap_vertex_range_onto_vertex_range(border_vertices, smesh, target_hrange, tmesh, tol_pmap);
-}
-
-template <typename PolygonMesh, typename HalfedgeRange>
-std::size_t snap_border_vertices_onto_vertex_range(PolygonMesh& smesh,
-                                                   const HalfedgeRange& target_hrange,
-                                                   const PolygonMesh& tmesh)
-{
-  typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor    halfedge_descriptor;
-
-  std::vector<halfedge_descriptor> border_vertices;
-  border_halfedges(smesh, std::back_inserter(border_vertices));
-
-  return snap_vertex_range_onto_vertex_range(border_vertices, smesh, target_hrange, tmesh);
-}
-
-template <typename PolygonMesh, typename ToleranceMap>
-std::size_t snap_border_vertices_onto_border_vertices(PolygonMesh& smesh,
-                                                      const PolygonMesh& tmesh,
-                                                      const ToleranceMap& tol_pmap)
-{
-  typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor    halfedge_descriptor;
-
-  std::vector<halfedge_descriptor> sborder_vertices;
-  border_halfedges(smesh, std::back_inserter(sborder_vertices));
+public:
+  explicit Projection_traits(const halfedge_descriptor h,
+                             const std::size_t patch_id,
+                             const FT sq_tolerance,
+                             const TriangleMesh& tm_S,
+                             const VPMS vpm_S,
+                             const TriangleMesh& tm_T,
+                             const FacePatchMap face_patch_map_T,
+                             const VPMT vpm_T,
+                             const AABBTraits& aabb_traits)
+    :
+      m_h(h),
+      m_patch_id(patch_id),
+      m_sq_tol(sq_tolerance),
+      m_tm_S(tm_S), m_vpm_S(vpm_S),
+      m_tm_T(tm_T), m_face_patch_map_T(face_patch_map_T), m_vpm_T(vpm_T),
+      m_is_same_mesh((&tm_S == &tm_T)),
+      m_continue(true),
+      m_closest_point_initialized(false),
+      m_traits(aabb_traits),
+      m_gt(Geom_traits()) // blame AABB's traits management
+  {
+    // not fully convinced that it is still useful after adding scalar product shenanigans
+// #define CGAL_PMP_SNAP_USE_ANISOTROPIC_DISTANCE
+#ifdef CGAL_PMP_SNAP_USE_ANISOTROPIC_DISTANCE
+    build_metric();
+#endif
 
-  std::vector<halfedge_descriptor> tborder_vertices;
-  border_halfedges(tmesh, std::back_inserter(tborder_vertices));
+    Vector hv(get(vpm_S, source(h, tm_S)), get(vpm_S, target(h, tm_S)));
+    m_direction = hv;
+    CGAL::Polygon_mesh_processing::internal::normalize(m_direction, m_gt);
+  }
 
-  return snap_vertex_range_onto_vertex_range(sborder_vertices, smesh, tborder_vertices, tmesh, tol_pmap);
-}
+  bool go_further() const { return m_continue; }
 
-// \ingroup PMP_repairing_grp
-//
-// Attempts to snap the border vertices of the source mesh onto the vertices of the target mesh.
-//
-// A vertex of the source range is only snapped to a vertex of the target mesh
-// if its distance to the target mesh vertex is smaller than a user-chosen bound.
-// If any source vertex can be snapped onto multiple vertices of the target
-// range, the closest one is chosen.
-// If multiple vertices within the source range could be snapped onto the same target vertex,
-// the source vertex closest to the target vertex will be snapped. Other source vertices will try to snap
-// to free target vertices until they find a valid target, or until there is no more target
-// to snap to (within the tolerance).
-//
-// @tparam PolygonMesh a model of `FaceListGraph` and `MutableFaceGraph`
-// @tparam ToleranceMap a model of `ReadablePropertyMap` with key type `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
-//                      and value type the number type associated with the traits of the mesh.
-//
-// @param smesh the source mesh whose border vertices might be moved
-// @param tmesh the target mesh whose vertices are potential projection targets
-// @param tol_pmap a tolerance map associating to each vertex of the source range a tolerance value:
-//               potential projection targets are sought in a sphere centered at the vertex and
-//               whose radius is the tolerance value.
-//
-// @pre `smesh` and `tmesh` are different meshes
-//
-// \return the number of snapped vertices
-//
-template <typename PolygonMesh, typename ToleranceMap>
-std::size_t snap_border_vertices_onto_vertex_range(PolygonMesh& smesh,
-                                                   const PolygonMesh& tmesh,
-                                                   const ToleranceMap& tol_pmap)
-{
-  return snap_border_vertices_onto_vertex_range(smesh, halfedges(tmesh), tmesh, tol_pmap);
-}
+  void intersection(const Point& query, const Primitive& primitive)
+  {
+#ifdef CGAL_PMP_SNAP_DEBUG_PP
+    std::cout << "~~~~ intersection with primitive: " << primitive.id() << std::endl;
+    std::cout << get(m_vpm_T, source(primitive.id(), m_tm_T)) << std::endl;
+    std::cout << get(m_vpm_T, target(primitive.id(), m_tm_T)) << std::endl;
+#endif
 
-template <typename PolygonMesh>
-std::size_t snap_border_vertices_onto_vertex_range(PolygonMesh& smesh, const PolygonMesh& tmesh)
-{
-  return snap_border_vertices_onto_vertex_range(smesh, halfedges(tmesh), tmesh);
-}
+    halfedge_descriptor h = halfedge(primitive.id(), m_tm_T);
+    if(is_border(h, m_tm_T))
+      h = opposite(h, m_tm_T);
 
-} // namespace experimental
+#ifdef CGAL_PMP_SNAP_DEBUG_PP
+      std::cout << "patches: " << get(m_face_patch_map_T, face(h, m_tm_T)) << " " << m_patch_id << std::endl;
+#endif
 
-////////////////////////////////////////////////////////////////////////////////////////////////////
-////////////////////////////////////////////////////////////////////////////////////////////////////
-////////////////////////////////////////////////////////////////////////////////////////////////////
-////////////////////////////////////////////////////////////////////////////////////////////////////
+    if(get(m_face_patch_map_T, face(h, m_tm_T)) != m_patch_id)
+      return;
 
-namespace internal {
+    const typename Primitive::Datum& s = primitive.datum(m_traits.shared_data());
+    if(m_traits.equal_3_object()(s[0], query) || m_traits.equal_3_object()(s[1], query))
+    {
+      // If we are NOT using the same mesh and the query is (geometrically) equal to one extremity
+      // of the target edge, we don't want to move the source point away from the target point
+      // (because we cannot move the target point).
+      if(!m_is_same_mesh)
+      {
+#ifdef CGAL_PMP_SNAP_DEBUG_PP
+        std::cout << "This vertex is stuck because it is equal to a vertex on the target mesh" << std::endl;
+#endif
+        m_closest_point_initialized = false;
+        m_continue = false;
+      }
 
-// Adapted from <CGAL/internal/AABB_tree/AABB_traversal_traits.h>
-template <typename AABBTraits>
-class Projection_traits
-{
-  typedef typename AABBTraits::FT                               FT;
-  typedef typename AABBTraits::Point_3                          Point;
-  typedef typename AABBTraits::Primitive                        Primitive;
-  typedef typename AABBTraits::Bounding_box                     Bounding_box;
-  typedef typename AABBTraits::Primitive::Id                    Primitive_id;
-  typedef typename AABBTraits::Point_and_primitive_id           Point_and_primitive_id;
-  typedef typename AABBTraits::Object_and_primitive_id          Object_and_primitive_id;
+      // skip the primitive if one of its endpoints is the query
+      return;
+    }
 
-  typedef CGAL::AABB_node<AABBTraits>                           Node;
+    // We are searching for a point on the target border that is close to target(h).
+    // To try and avoid foldings, we penalize potential matches that are roughly in the direction of 'h'
+    // by using an anisotropic distance that is the Euclidean on the plane orthogonal to the direction,
+    // and much larger when traveling in the rough direction of 'h'.
+    //
+    // Note that we apply this to points that fall within the tolerance ball, so if there is
+    // only a single candidate that is in a direction we don't like, we still take it.
+    //
+    // @todo should the construct_projected_point_3 be anisotropic too?
+    const Point new_closest_point = m_gt.construct_projected_point_3_object()(
+      CGAL::internal::Primitive_helper<AABBTraits>::get_datum(primitive, m_traits), query);
 
-public:
-  explicit Projection_traits(const AABBTraits& traits,
-                             const FT sq_tolerance)
-    : m_traits(traits),
-      m_closest_point_initialized(false),
-      m_sq_dist(sq_tolerance)
-  {}
+#ifdef CGAL_PMP_SNAP_DEBUG_PP
+    std::cout << "closest point to primitive: " << new_closest_point << std::endl;
+#endif
 
-  bool go_further() const { return true; }
+    const FT sq_ad_to_tentative_closest_pt = squared_anisotropic_distance(query, new_closest_point);
 
-  void intersection(const Point& query, const Primitive& primitive)
-  {
-    // skip the primitive if one of its endpoints is the query
-    const typename Primitive::Datum& s = primitive.datum(m_traits.shared_data());
-    if(m_traits.equal_3_object()(s[0], query) || m_traits.equal_3_object()(s[1], query))
-      return;
+    Vector tentative_dir(get(m_vpm_T, source(primitive.id(), m_tm_T)),
+                         get(m_vpm_T, target(primitive.id(), m_tm_T)));
+    CGAL::Polygon_mesh_processing::internal::normalize(tentative_dir, m_gt);
+    const FT sp_with_tentative = CGAL::abs(tentative_dir * m_direction);
 
-    if(!m_closest_point_initialized)
+    if(!m_closest_point_initialized ||
+       is_better_than_current_best(sq_ad_to_tentative_closest_pt, sp_with_tentative))
     {
+#ifdef CGAL_PMP_SNAP_DEBUG_PP
+      std::cout << "is better!" << std::endl;
+#endif
+
       m_closest_point_initialized = true;
-      m_closest_primitive = primitive.id();
-      m_closest_point = primitive.reference_point(m_traits.shared_data());
-      m_sq_dist = m_traits.squared_distance_object()(query, m_closest_point);
-    }
 
-    Point new_closest_point = m_traits.closest_point_object()(query, primitive, m_closest_point);
-    if(!m_traits.equal_3_object()(new_closest_point, m_closest_point))
-    {
       m_closest_primitive = primitive.id();
       m_closest_point = new_closest_point;
-      m_sq_dist = m_traits.squared_distance_object()(query, m_closest_point);
+      m_sq_adist = sq_ad_to_tentative_closest_pt;
+      m_sp_with_closest_edge = sp_with_tentative;
     }
   }
 
   bool do_intersect(const Point& query, const Node& node) const
   {
-    return m_traits.compare_distance_object()(query, node.bbox(), m_sq_dist) == CGAL::SMALLER;
+    // This should NOT be the anisotropic distance, as we want to test all targets within the tolerance
+    return m_traits.compare_distance_object()(query, node.bbox(), m_sq_tol) == CGAL::SMALLER;
   }
 
   const Point& closest_point() const { return m_closest_point; }
   typename Primitive::Id closest_primitive_id() const { return m_closest_primitive; }
   bool closest_point_initialized() const { return m_closest_point_initialized; }
+  FT scalar_product_with_best() const { return m_sp_with_closest_edge; }
 
 private:
-  const AABBTraits& m_traits;
+  halfedge_descriptor m_h;
+  const std::size_t m_patch_id;
+  const FT m_sq_tol;
+  Vector m_direction;
+#ifdef CGAL_PMP_SNAP_USE_ANISOTROPIC_DISTANCE
+  Eigen::Matrix3d m_metric;
+#endif
+
+  const TriangleMesh& m_tm_S;
+  VPMS m_vpm_S;
+  const TriangleMesh& m_tm_T;
+  FacePatchMap m_face_patch_map_T;
+  VPMT m_vpm_T;
+  const bool m_is_same_mesh;
+
+  bool m_continue;
   bool m_closest_point_initialized;
+  typename Primitive::Id m_closest_primitive;
   Point m_closest_point;
-  FT m_sq_dist;
+  FT m_sq_adist;
+  FT m_sp_with_closest_edge; // scalar product between m_h and the current best edge candidate
 
-  typename Primitive::Id m_closest_primitive;
+  const AABBTraits& m_traits;
+  Geom_traits m_gt;
 };
 
-template <typename PolygonMesh, typename GeomTraits, typename VPM>
-struct Compare_points_along_edge
+template <typename ConcurrencyTag>
+struct Edges_to_split_map_inserter // Parallel
 {
-  typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor    halfedge_descriptor;
-  typedef typename GeomTraits::Point_3                                      Point;
-
-  Compare_points_along_edge(halfedge_descriptor support,
-                            const GeomTraits& gt, const VPM& vpm, const PolygonMesh& pm)
-    : gt(gt), m_src(get(vpm, source(support, pm)))
-  { }
-
-  bool operator()(const Point& p1, const Point& p2) const {
-    return gt.less_distance_to_point_3_object()(m_src, p1, p2);
+#ifdef CGAL_LINKED_WITH_TBB
+  template <typename EdgeToSplitMap, typename HalfedgeDescriptor, typename Point>
+  void operator()(EdgeToSplitMap& edges_to_split,
+                  const HalfedgeDescriptor closest_h,
+                  const HalfedgeDescriptor h,
+                  const Point& closest_p)
+  {
+    typename EdgeToSplitMap::accessor acc;
+    edges_to_split.insert(acc, closest_h);
+    acc->second.emplace_back(h, closest_p);
   }
+#endif
+};
 
-  const GeomTraits& gt;
-  Point m_src;
+template <>
+struct Edges_to_split_map_inserter<CGAL::Sequential_tag>
+{
+  template <typename EdgeToSplitMap, typename HalfedgeDescriptor, typename Point>
+  void operator()(EdgeToSplitMap& edges_to_split,
+                  const HalfedgeDescriptor closest_h,
+                  const HalfedgeDescriptor h,
+                  const Point& closest_p)
+  {
+    edges_to_split[closest_h].emplace_back(h, closest_p);
+  }
 };
 
-template <typename TriangleMesh, typename EdgeToSplitContainer, typename AABBTree, typename VPM, typename GT>
-void find_splittable_edge(const std::pair<typename boost::property_traits<VPM>::value_type,
-                                          typename GT::FT>& point_with_tolerance,
-                          EdgeToSplitContainer& edges_to_split,
+// The UniqueVertex is a pair of a container of vertex_descriptor and FT, representing
+// vertices with the same geometric position and their associated snapping tolerance
+// (defined as the minimum of the tolerances of the vertices of the bunch)
+template <typename ConcurrencyTag = CGAL::Sequential_tag,
+          typename VertexWithTolerance, typename TriangleMesh,
+          typename EdgeToSplitMap, typename AABBTree,
+          typename VertexPatchMap_S, typename FacePatchMap_T,
+          typename VPMS, typename VPMT, typename GT>
+void find_splittable_edge(const VertexWithTolerance& vertex_with_tolerance,
+                          EdgeToSplitMap& edges_to_split,
                           const AABBTree* aabb_tree_ptr,
-                          const TriangleMesh& /* pms */,
-                          const VPM& /* vpms */,
-                          const TriangleMesh& pmt,
-                          const VPM& vpmt,
+                          const TriangleMesh& tm_S,
+                          VertexPatchMap_S vertex_patch_map_S,
+                          VPMS vpm_S,
+                          const TriangleMesh& tm_T,
+                          FacePatchMap_T face_patch_map_T,
+                          VPMT vpm_T,
                           const GT& gt)
 {
-  CGAL_USE(vpmt);
-  typedef typename boost::graph_traits<TriangleMesh>::halfedge_descriptor   halfedge_descriptor;
-  typedef typename boost::graph_traits<TriangleMesh>::edge_descriptor       edge_descriptor;
-
-  typedef typename GT::FT                                                   FT;
-  typedef typename boost::property_traits<VPM>::value_type                  Point;
-
-  typedef typename AABBTree::AABB_traits                                    AABB_traits;
+  typedef typename boost::graph_traits<TriangleMesh>::vertex_descriptor           vertex_descriptor;
+  typedef typename boost::graph_traits<TriangleMesh>::halfedge_descriptor         halfedge_descriptor;
+  typedef typename boost::graph_traits<TriangleMesh>::edge_descriptor             edge_descriptor;
+
+  typedef typename GT::FT                                                         FT;
+  typedef typename boost::property_traits<VPMS>::value_type                       Point;
+  typedef typename boost::property_traits<VPMS>::reference                        Point_ref;
+
+  typedef typename AABBTree::AABB_traits                                          AABB_traits;
+
+  typedef internal::Projection_traits<TriangleMesh, VPMS, VPMT, FacePatchMap_T, AABB_traits> Projection_traits;
+
+  // by construction the whole range has the same position
+  const halfedge_descriptor h = vertex_with_tolerance.first;
+  const vertex_descriptor v = target(h, tm_S);
+  const Point_ref query = get(vpm_S, v);
+  const FT sq_tolerance = CGAL::square(vertex_with_tolerance.second);
+  const std::size_t patch_id = get(vertex_patch_map_S, v);
 
-  const Point& query = point_with_tolerance.first;
-  const FT sq_tolerance = CGAL::square(point_with_tolerance.second);
+#ifdef CGAL_PMP_SNAP_DEBUG_PP
+  std::cout << "--------------------------- Query: " << v << " (" << query << ")" << std::endl;
+#endif
 
-  // use the current halfedge as hint
-  internal::Projection_traits<AABB_traits> traversal_traits(aabb_tree_ptr->traits(), sq_tolerance);
+  Projection_traits traversal_traits(h, patch_id, sq_tolerance, tm_S, vpm_S,
+                                     tm_T, face_patch_map_T, vpm_T, aabb_tree_ptr->traits());
   aabb_tree_ptr->traversal(query, traversal_traits);
 
   if(!traversal_traits.closest_point_initialized())
+  {
+#ifdef CGAL_PMP_SNAP_DEBUG_PP
+  std::cout << "Couldn't find any single projection point" << std::endl;
+#endif
     return;
+  }
 
   const Point& closest_p = traversal_traits.closest_point();
 
   // The filtering in the AABB tree checks the dist query <-> node bbox, which might be smaller than
   // the actual distance between the query <-> closest point
-  const FT sq_dist_to_closest = gt.compute_squared_distance_3_object()(query, closest_p);
-  bool is_close_enough = (sq_dist_to_closest <= sq_tolerance);
+  edge_descriptor closest_e = traversal_traits.closest_primitive_id();
+  bool is_close_enough =
+    gt.compare_squared_distance_3_object()(query,
+                                           gt.construct_segment_3_object()(get(vpm_T, source(closest_e, tm_T)),
+                                                                           get(vpm_T, target(closest_e, tm_T))),
+                                           sq_tolerance) != LARGER;
 
 #ifdef CGAL_PMP_SNAP_DEBUG_PP
   std::cout << "  Closest point: (" << closest_p << ")" << std::endl
-            << "  at distance " << gt.compute_squared_distance_3_object()(query, closest_p)
-            << " with a tolerance of " << point_with_tolerance.second << " at vertex (squared tol: " << sq_tolerance
-            << " && close enough? " << is_close_enough << ")" << std::endl;
+            << "  on edge: " << traversal_traits.closest_primitive_id()
+            << "  at sq distance " << gt.compute_squared_distance_3_object()(query, closest_p)
+            << " with squared tolerance: " << sq_tolerance
+            << " && close enough? " << is_close_enough << std::endl;
 #endif
 
   if(!is_close_enough)
     return;
 
-#ifdef CGAL_PMP_SNAP_DEBUG_PP
-  std::cout << "\t and is thus beneath tolerance" << std::endl;
-#endif
-
-  edge_descriptor closest = traversal_traits.closest_primitive_id();
-  CGAL_assertion(get(vpmt, source(closest, pmt)) != query &&
-                                                    get(vpmt, target(closest, pmt)) != query);
 
-  halfedge_descriptor clos_hd = halfedge(closest, pmt);
-  CGAL_assertion(is_border(edge(clos_hd, pmt), pmt));
+  CGAL_assertion(get(vpm_T, source(closest_e, tm_T)) != query &&
+                 get(vpm_T, target(closest_e, tm_T)) != query);
 
-  if(!is_border(clos_hd, pmt))
-    clos_hd = opposite(clos_hd, pmt);
+  halfedge_descriptor closest_h = halfedge(closest_e, tm_T);
+  CGAL_assertion(is_border(edge(closest_h, tm_T), tm_T));
 
-#ifdef CGAL_PMP_SNAP_DEBUG_PP
-  std::cout << "splitting position: " << query << std::endl;
-#endif
+  if(!is_border(closest_h, tm_T))
+    closest_h = opposite(closest_h, tm_T);
 
-  edges_to_split.push_back(std::make_pair(query, clos_hd));
+  // Using a map because we need to know if the same halfedge is split multiple times
+  Edges_to_split_map_inserter<ConcurrencyTag>()(edges_to_split, closest_h, vertex_with_tolerance.first, closest_p);
 }
 
 #ifdef CGAL_LINKED_WITH_TBB
 template <typename PointWithToleranceContainer,
-          typename TriangleMesh, typename EdgeToSplitContainer,
-          typename AABBTree, typename VPM, typename GT>
+          typename TriangleMesh, typename EdgeToSplitMap, typename AABBTree,
+          typename VertexPatchMap_S, typename FacePatchMap_T,
+          typename VPMS, typename VPMT, typename GT>
 struct Find_splittable_edge_for_parallel_for
 {
   Find_splittable_edge_for_parallel_for(const PointWithToleranceContainer& points_with_tolerance,
-                                        EdgeToSplitContainer& edges_to_split,
+                                        EdgeToSplitMap& edges_to_split,
                                         const AABBTree* aabb_tree_ptr,
-                                        const TriangleMesh& pms,
-                                        const VPM& vpms,
-                                        const TriangleMesh& pmt,
-                                        const VPM& vpmt,
+                                        const TriangleMesh& tm_S,
+                                        const VertexPatchMap_S vertex_patch_map_S,
+                                        const VPMS vpm_S,
+                                        const TriangleMesh& tm_T,
+                                        const FacePatchMap_T face_patch_map_T,
+                                        const VPMT vpm_T,
                                         const GT& gt)
     :
       m_points_with_tolerance(points_with_tolerance),
       m_edges_to_split(edges_to_split), m_aabb_tree_ptr(aabb_tree_ptr),
-      m_pms(pms), m_vpms(vpms), m_pmt(pmt), m_vpmt(vpmt), m_gt(gt)
+      m_tm_S(tm_S), m_vertex_patch_map_S(vertex_patch_map_S), m_vpm_S(vpm_S),
+      m_tm_T(tm_T), m_face_patch_map_T(face_patch_map_T), m_vpm_T(vpm_T),
+      m_gt(gt)
   { }
 
   void operator()(const tbb::blocked_range<size_t>& r) const
   {
     for(std::size_t i=r.begin(); i!=r.end(); ++i)
     {
-      find_splittable_edge(m_points_with_tolerance[i], m_edges_to_split,
-                           m_aabb_tree_ptr,
-                           m_pms, m_vpms, m_pmt, m_vpmt,
-                           m_gt);
+      find_splittable_edge<CGAL::Parallel_tag>(m_points_with_tolerance[i], m_edges_to_split, m_aabb_tree_ptr,
+                                               m_tm_S, m_vertex_patch_map_S, m_vpm_S,
+                                               m_tm_T, m_face_patch_map_T, m_vpm_T, m_gt);
     }
   }
 
 private:
   const PointWithToleranceContainer& m_points_with_tolerance;
-  EdgeToSplitContainer& m_edges_to_split;
+  EdgeToSplitMap& m_edges_to_split;
   const AABBTree* m_aabb_tree_ptr;
-  const TriangleMesh& m_pms;
-  const VPM m_vpms;
-  const TriangleMesh& m_pmt;
-  const VPM m_vpmt;
+  const TriangleMesh& m_tm_S;
+  const VertexPatchMap_S m_vertex_patch_map_S;
+  const VPMS m_vpm_S;
+  const TriangleMesh& m_tm_T;
+  const FacePatchMap_T m_face_patch_map_T;
+  const VPMT m_vpm_T;
   const GT& m_gt;
 };
 #endif
 
-} // namespace internal
-
-namespace experimental {
-
-// Try to merge vertices in 'hrange' (vd = target(hd, pm)) onto the edges in the 'erange',
-// non-conformingly
-template <typename HalfedgeRange, typename TriangleMesh,
-          typename ToleranceMap, typename NamedParameters>
-std::size_t snap_vertex_range_onto_vertex_range_non_conforming(const HalfedgeRange& source_hrange,
-                                                               TriangleMesh& pms,
-                                                               const HalfedgeRange& target_hrange,
-                                                               TriangleMesh& pmt,
-                                                               const ToleranceMap& tol_pmap,
-                                                               const NamedParameters& nps,
-                                                               const NamedParameters& npt)
+template <typename EdgesToSplitContainer,
+          typename TriangleMesh, typename GeomTraits,
+          typename VPMS, typename VPMT>
+std::size_t split_edges(EdgesToSplitContainer& edges_to_split,
+                        TriangleMesh& tm_S,
+                        VPMS vpm_S,
+                        TriangleMesh& tm_T,
+                        VPMT vpm_T,
+                        const GeomTraits& gt,
+                        const bool is_source_mesh_fixed) // when snapping is B --> A and the mesh B is fixed
 {
-  typedef typename boost::graph_traits<TriangleMesh>::vertex_descriptor     vertex_descriptor;
-  typedef typename boost::graph_traits<TriangleMesh>::halfedge_descriptor   halfedge_descriptor;
+#ifdef CGAL_PMP_SNAP_DEBUG
+  std::cout << "split " << edges_to_split.size() << " edges" << std::endl;
+#endif
 
-  typedef typename GetVertexPointMap<TriangleMesh, NamedParameters>::type   VPM;
-  typedef typename boost::property_traits<VPM>::value_type                  Point;
-  typedef typename boost::property_traits<VPM>::reference                   Point_ref;
-
-  typedef typename GetGeomTraits<TriangleMesh, NamedParameters>::type       GT;
-  typedef typename GT::FT                                                   FT;
-  typedef typename GT::Vector_3                                             Vector;
-
-  typedef internal::Compare_points_along_edge<TriangleMesh, GT, VPM>        Point_along_edge_comparer;
-
-  typedef CGAL::AABB_halfedge_graph_segment_primitive<TriangleMesh, VPM>    Primitive;
-  typedef CGAL::AABB_traits<GT, Primitive>                                  AABB_Traits;
-  typedef CGAL::AABB_tree<AABB_Traits>                                      AABB_tree;
+  typedef typename boost::graph_traits<TriangleMesh>::vertex_descriptor           vertex_descriptor;
+  typedef typename boost::graph_traits<TriangleMesh>::halfedge_descriptor         halfedge_descriptor;
 
-  using parameters::get_parameter;
-  using parameters::choose_parameter;
+  typedef typename boost::property_traits<VPMS>::value_type                       Point;
+  typedef typename boost::property_traits<VPMT>::reference                        Point_ref;
+  typedef typename GeomTraits::Vector_3                                           Vector;
 
-  // Need to have a triangle mesh to ensure that the refinement point of a border edge 'e' has visibility
-  // of the third p oint of the face 'f' incident to 'e'
-  CGAL_precondition(is_triangle_mesh(pmt));
+  typedef std::pair<halfedge_descriptor, Point>                                   Vertex_with_new_position;
+  typedef std::vector<Vertex_with_new_position>                                   Vertices_with_new_position;
+  typedef std::pair<const halfedge_descriptor, Vertices_with_new_position>        Edge_and_splitters;
 
   std::size_t snapped_n = 0;
 
-  VPM vpms = choose_parameter(get_parameter(nps, internal_np::vertex_point), get_property_map(vertex_point, pms));
-  VPM vpmt = choose_parameter(get_parameter(npt, internal_np::vertex_point), get_property_map(vertex_point, pmt));
+  CGAL::Real_timer timer;
+  timer.start();
 
-  const GT gt = choose_parameter(get_parameter(nps, internal_np::geom_traits), GT());
+  for(Edge_and_splitters& es : edges_to_split)
+  {
+    halfedge_descriptor h_to_split = es.first;
+    CGAL_assertion(is_border(h_to_split, tm_T));
 
-  // start by snapping vertices together to simplify things
-  snapped_n = snap_vertex_range_onto_vertex_range(source_hrange, pms, target_hrange, pmt, tol_pmap, nps, npt);
+    Vertices_with_new_position& splitters = es.second;
 
-#ifdef CGAL_PMP_SNAP_DEBUG
-  std::cout << "Snapping vertices to edges. Range sizes: "
-            << std::distance(source_hrange.begin(), source_hrange.end()) << " and "
-            << std::distance(target_hrange.begin(), target_hrange.end()) << std::endl;
+    if(splitters.size() > 1)
+    {
+#ifdef CGAL_PMP_SNAP_DEBUG_PP
+      std::cout << " _______ Multiple splitting points on the same halfedge" << std::endl;
+#endif
+
+      const Point_ref hsp = get(vpm_T, source(h_to_split, tm_T));
+      std::sort(splitters.begin(), splitters.end(),
+                [&](const Vertex_with_new_position& l, const Vertex_with_new_position& r) -> bool
+                {
+                  return gt.less_distance_to_point_3_object()(hsp, l.second, r.second);
+                });
+    }
+
+    // Inserting new points ordered from the source to the target of (the initial) 'h'
+    bool first_split = true;
+    Point previous_split_position = get(vpm_S, *(vertices(tm_S).begin())); // dummy value to avoid "used uninitialized" warning
+    for(const Vertex_with_new_position& vnp : splitters)
+    {
+      const halfedge_descriptor splitter_h = vnp.first;
+      const vertex_descriptor splitter_v = target(splitter_h, tm_S);
+      const Point new_position = is_source_mesh_fixed ? get(vpm_S, splitter_v) : vnp.second;
+
+      bool do_split = true;
+
+      // Some splits can create degenerate faces, avoid that
+      if((new_position == get(vpm_T, target(h_to_split, tm_T))) ||
+         (new_position == get(vpm_T, source(h_to_split, tm_T))))
+        do_split = false;
+
+      if(!first_split && new_position == previous_split_position)
+        do_split = false;
+
+#ifdef CGAL_PMP_SNAP_DEBUG_PP
+      std::cout << " -.-.-. Splitting " << edge(h_to_split, tm_T) << " |||| "
+                << " Vs " << source(h_to_split, tm_T) << " (" << tm_T.point(source(h_to_split, tm_T)) << ")"
+                << " --- Vt " << target(h_to_split, tm_T) << " (" << tm_T.point(target(h_to_split, tm_T)) << ")" << std::endl;
+      std::cout << "With point: " << vnp.second << std::endl;
+      std::cout << "Actually split? " << do_split << std::endl;
+#endif
+
+      // Split and update positions
+      vertex_descriptor new_v = boost::graph_traits<TriangleMesh>::null_vertex();
+      if(do_split)
+      {
+        CGAL::Euler::split_edge(h_to_split, tm_T);
+        new_v = source(h_to_split, tm_T);
+        put(vpm_T, new_v, new_position); // position of the new point on the target mesh
+      }
+
+      if(!is_source_mesh_fixed)
+        put(vpm_S, splitter_v, new_position);
+
+      first_split = false;
+      previous_split_position = new_position;
+      ++snapped_n;
+
+      // Everything below is choosing the diagonal to triangulate the quad formed by the edge split
+      // So, it's only relevant if splitting has been performed
+      if(!do_split)
+        continue;
+
+      /*          new_p
+       *         /   \
+       *    res /     \ h_to_split
+       *       /       \
+       *      /         \
+       *    left       right
+       *     |         /
+       *     |        /
+       *     |       /
+       *     |      /
+       *     |     /
+       *     |    /
+       *      opp
+       */
+
+      const halfedge_descriptor res = prev(h_to_split, tm_T);
+      const Point_ref left_pt = get(vpm_T, source(res, tm_T));
+      const Point_ref right_pt = get(vpm_T, target(h_to_split, tm_T));
+      const Point_ref opp = get(vpm_T, target(next(opposite(res, tm_T), tm_T), tm_T));
+
+      // Check if 'p' is "visible" from 'opp' (i.e. its projection on the plane 'Pl(left, opp, right)'
+      // falls in the cone with apex 'opp' and sides given by 'left' and 'right')
+      const Vector n = gt.construct_orthogonal_vector_3_object()(right_pt, left_pt, opp);
+      const Point trans_left_pt = gt.construct_translated_point_3_object()(left_pt, n);
+      const Point trans_right_pt = gt.construct_translated_point_3_object()(right_pt, n);
+
+      const Point_ref new_p = get(vpm_T, new_v);
+      const bool left_of_left = (gt.orientation_3_object()(trans_left_pt, left_pt, opp, new_p) == CGAL::POSITIVE);
+      const bool right_of_right = (gt.orientation_3_object()(right_pt, trans_right_pt, opp, new_p) == CGAL::POSITIVE);
+
+      const bool is_visible = (!left_of_left && !right_of_right);
+
+#ifdef CGAL_PMP_SNAP_DEBUG_PP
+      std::cout << "Left/Right: " << left_of_left << " " << right_of_right << std::endl;
+      std::cout << "visible from " << opp << " ? " << is_visible << std::endl;
 #endif
 
-  // Collect border points that can be projected onto a border edge
-  typedef std::map<Point, FT>                       Unique_vertices_with_tolerance;  // unordered map is empirically slower
+      // h_to_split is equal to 'next(res)' after splitting
+      const halfedge_descriptor h_to_split_opp = opposite(h_to_split, tm_T);
 
-  // If the point appears multiple times on the source border, use it only once to snap target edges
-  // use the smallest tolerance from all source vertices
+      if(is_visible)
+      {
+        halfedge_descriptor new_hd = CGAL::Euler::split_face(h_to_split_opp,
+                                                             prev(prev(h_to_split_opp, tm_T), tm_T), tm_T);
+        h_to_split = opposite(prev(new_hd, tm_T), tm_T);
+      }
+      else
+      {
+        halfedge_descriptor new_hd = CGAL::Euler::split_face(opposite(res, tm_T),
+                                                             prev(h_to_split_opp, tm_T), tm_T);
+        h_to_split = opposite(next(new_hd, tm_T), tm_T);
+      }
+    }
+  }
+
+  return snapped_n;
+}
+
+// Collect border points that can be projected onto a border edge
+//
+// If multiple vertices on the source border have the same geometric position, they are moved
+// all at once and we use the smallest tolerance from all source vertices with that position.
+// This is done to solve situation such as:
+//          ||
+//          ||
+//      ____||____
+//      ----------
+// (vertex-vertex snapping into non-conformal snapping) in one go.
+template <typename ConcurrencyTag = CGAL::Sequential_tag,
+          typename HalfedgeRange, typename TriangleMesh,
+          typename LockedVertexMap, typename LockedHalfedgeMap, typename ToleranceMap,
+          typename VertexPatchMap_S, typename FacePatchMap_T,
+          typename SourceNamedParameters, typename TargetNamedParameters>
+std::size_t snap_non_conformal_one_way(const HalfedgeRange& halfedge_range_S,
+                                       TriangleMesh& tm_S,
+                                       ToleranceMap tolerance_map_S,
+                                       VertexPatchMap_S vertex_patch_map_S,
+                                       LockedVertexMap locked_vertices_S,
+                                       const HalfedgeRange& halfedge_range_T,
+                                       TriangleMesh& tm_T,
+                                       FacePatchMap_T face_patch_map_T,
+                                       LockedHalfedgeMap locked_halfedges_T,
+                                       const bool is_source_mesh_fixed,
+                                       const SourceNamedParameters& snp,
+                                       const TargetNamedParameters& tnp)
+{
+  typedef typename boost::graph_traits<TriangleMesh>::vertex_descriptor           vertex_descriptor;
+  typedef typename boost::graph_traits<TriangleMesh>::halfedge_descriptor         halfedge_descriptor;
+
+  typedef typename GetGeomTraits<TriangleMesh, TargetNamedParameters>::type       GT;
+  typedef typename GT::FT                                                         FT;
+
+  typedef typename GetVertexPointMap<TriangleMesh, SourceNamedParameters>::type   VPMS;
+  typedef typename GetVertexPointMap<TriangleMesh, TargetNamedParameters>::type   VPMT;
+
+  typedef typename boost::property_traits<VPMT>::value_type                       Point;
+
+  typedef CGAL::AABB_halfedge_graph_segment_primitive<TriangleMesh, VPMT>         Primitive;
+  typedef CGAL::AABB_traits<GT, Primitive>                                        AABB_Traits;
+  typedef CGAL::AABB_tree<AABB_Traits>                                            AABB_tree;
+
+  typedef std::pair<halfedge_descriptor, Point>                                   Vertex_with_new_position;
+  typedef std::vector<Vertex_with_new_position>                                   Vertices_with_new_position;
+
+  using parameters::get_parameter;
+  using parameters::choose_parameter;
+
+  VPMS vpm_S = choose_parameter(get_parameter(snp, internal_np::vertex_point), get_property_map(vertex_point, tm_S));
+  VPMT vpm_T = choose_parameter(get_parameter(tnp, internal_np::vertex_point), get_property_map(vertex_point, tm_T));
+  const GT gt = choose_parameter<GT>(get_parameter(snp, internal_np::geom_traits));
 
 #ifdef CGAL_PMP_SNAP_DEBUG
-  std::cout << "gather unique points in source range" << std::endl;
+  std::cout << "Gather unique points in source range..." << std::endl;
 #endif
 
-  Unique_vertices_with_tolerance unique_source_vertices;
-  for(halfedge_descriptor hd : source_hrange)
+  typedef std::pair<halfedge_descriptor, FT>                                      Vertex_with_tolerance;
+  typedef std::vector<Vertex_with_tolerance>                                      Vertices_with_tolerance;
+
+  Vertices_with_tolerance vertices_to_snap;
+  vertices_to_snap.reserve(halfedge_range_S.size()); // ensures that iterators stay valid
+
+  // Take the min tolerance for all points that have the same coordinates
+  std::map<Point, FT> point_tolerance_map;
+
+  for(halfedge_descriptor h : halfedge_range_S)
   {
+    if(get(locked_vertices_S, target(h, tm_S)))
+      continue;
+
     // Skip the source vertex if its two incident halfedges are geometrically identical (it means that
     // the two halfedges are already stitchable and we don't want this common vertex to be used
     // to split a halfedge somewhere else)
-    if(get(vpms, source(hd, pms)) == get(vpms, target(next(hd, pms), pms)))
+    if(get(vpm_S, source(h, tm_S)) == get(vpm_S, target(next(h, tm_S), tm_S)))
       continue;
 
-    const vertex_descriptor vd = target(hd, pms);
-    const Point_ref query = get(vpms, vd);
-    const FT tolerance = get(tol_pmap, vd);
+    const vertex_descriptor v = target(h, tm_S);
+    const FT tolerance = get(tolerance_map_S, v);
 
-    std::pair<typename Unique_vertices_with_tolerance::iterator, bool> is_insert_successful =
-      unique_source_vertices.insert(std::make_pair(query, tolerance));
+    vertices_to_snap.emplace_back(h, tolerance);
 
-    // Keep the lowest tolerance out of all the source vertices with the same position
-    is_insert_successful.first->second = (std::min)(is_insert_successful.first->second, tolerance);
+    std::pair<Point, FT> entry(get(vpm_S, v), tolerance);
+    std::pair<typename std::map<Point, FT>::iterator, bool> is_insert_successful =
+      point_tolerance_map.insert(entry);
+    if(!is_insert_successful.second)
+      is_insert_successful.first->second = (std::min)(is_insert_successful.first->second, tolerance);
 
-#ifdef CGAL_PMP_SNAP_DEBUG_PP
-    if(is_insert_successful.second)
-      std::cout << "Query: " << vd << " (" << query << ")" << std::endl;
+    #ifdef CGAL_PMP_SNAP_DEBUG_PP
+    std::cout << "Non-conformal query: " << v << " (" << get(vpm_S, v) << "), tolerance: " << tolerance << std::endl;
 #endif
   }
 
+  for(auto& p : vertices_to_snap)
+    p.second = point_tolerance_map[get(vpm_S, target(p.first, tm_S))];
+
   // Since we're inserting primitives one by one, we can't pass this shared data in the constructor of the tree
   AABB_Traits aabb_traits;
-  aabb_traits.set_shared_data(pmt, vpmt);
+  aabb_traits.set_shared_data(tm_T, vpm_T);
   AABB_tree aabb_tree(aabb_traits);
 
-  for(halfedge_descriptor hd : target_hrange)
+  for(halfedge_descriptor h : halfedge_range_T)
   {
-    CGAL_precondition(is_border(edge(hd, pmt), pmt));
-    aabb_tree.insert(Primitive(edge(hd, pmt), pmt, vpmt));
+    CGAL_precondition(is_border(edge(h, tm_T), tm_T));
+    if(get(locked_halfedges_T, h))
+    {
+#ifdef CGAL_PMP_SNAP_DEBUG_PP
+      std::cout << edge(h, tm_T) << " is locked and not a valid target" << std::endl;
+#endif
+      continue;
+    }
+
+    aabb_tree.insert(Primitive(edge(h, tm_T), tm_T, vpm_T));
   }
 
   // Now, check which edges of the target range ought to be split by source vertices
 #ifdef CGAL_PMP_SNAP_DEBUG_PP
-  std::cout << "Collect edges to split w/ " << unique_source_vertices.size() << " unique vertices" << std::endl;
+  std::cout << "Collect edges to split with " << vertices_to_snap.size() << " vertices" << std::endl;
 #endif
 
-#ifdef CGAL_LINKED_WITH_TBB
+#ifndef CGAL_LINKED_WITH_TBB
+  CGAL_static_assertion_msg (!(std::is_convertible<ConcurrencyTag, Parallel_tag>::value),
+                             "Parallel_tag is enabled but TBB is unavailable.");
+#else
+  if(std::is_convertible<ConcurrencyTag, Parallel_tag>::value)
+  {
 #ifdef CGAL_PMP_SNAP_DEBUG
-  std::cout << "Parallel find splittable edges!" << std::endl;
+    std::cout << "Parallel find splittable edges!" << std::endl;
 #endif
 
-  typedef tbb::concurrent_vector<std::pair<Point, halfedge_descriptor> >        Concurrent_edge_to_split_container;
+    typedef tbb::concurrent_hash_map<halfedge_descriptor,
+                                     Vertices_with_new_position>                 Concurrent_edge_to_split_container;
+    typedef internal::Find_splittable_edge_for_parallel_for<
+              Vertices_with_tolerance, TriangleMesh,
+              Concurrent_edge_to_split_container, AABB_tree,
+              VertexPatchMap_S, FacePatchMap_T, VPMS, VPMT, GT>                  Functor;
 
-  // Can't parallel for a map...
-  std::vector<std::pair<Point, FT> > unique_vertices_with_tolerance_vector(unique_source_vertices.begin(),
-                                                                           unique_source_vertices.end());
+    CGAL::Real_timer timer;
+    timer.start();
 
-  typedef internal::Find_splittable_edge_for_parallel_for<std::vector<std::pair<Point, FT> >,
-                                                          TriangleMesh,
-                                                          Concurrent_edge_to_split_container,
-                                                          AABB_tree, VPM, GT>    Functor;
+    Concurrent_edge_to_split_container edges_to_split;
+    Functor f(vertices_to_snap, edges_to_split, &aabb_tree,
+              tm_S, vertex_patch_map_S, vpm_S, tm_T, face_patch_map_T, vpm_T, gt);
+    tbb::parallel_for(tbb::blocked_range<std::size_t>(0, vertices_to_snap.size()), f);
 
-  Concurrent_edge_to_split_container edges_to_split;
-  Functor f(unique_vertices_with_tolerance_vector, edges_to_split, &aabb_tree, pms, vpms, pmt, vpmt, gt);
-  tbb::parallel_for(tbb::blocked_range<std::size_t>(0, unique_vertices_with_tolerance_vector.size()), f);
-#else // CGAL_LINKED_WITH_TBB
+    std::cout << "Time to gather edges: " << timer.time() << std::endl;
+
+    return split_edges(edges_to_split, tm_S, vpm_S,  tm_T, vpm_T, gt, is_source_mesh_fixed);
+  }
+  else
+#endif // CGAL_LINKED_WITH_TBB
+  {
 #ifdef CGAL_PMP_SNAP_DEBUG
-  std::cout << "Sequential find splittable edges!" << std::endl;
+    std::cout << "Sequential find splittable edges!" << std::endl;
 #endif
-  typedef std::pair<Point, FT>                      Point_with_tolerance;
 
-  std::vector<std::pair<Point, halfedge_descriptor> > edges_to_split;
-  for(const Point_with_tolerance& point_with_tolerance : unique_source_vertices)
-  {
-    internal::find_splittable_edge(point_with_tolerance, edges_to_split, &aabb_tree,
-                                   pms, vpms, pmt, vpmt, gt);
+    std::map<halfedge_descriptor, Vertices_with_new_position> edges_to_split;
+    for(const Vertex_with_tolerance& vt : vertices_to_snap)
+    {
+      internal::find_splittable_edge(vt, edges_to_split, &aabb_tree,
+                                     tm_S, vertex_patch_map_S, vpm_S,
+                                     tm_T, face_patch_map_T, vpm_T, gt);
+    }
+
+    return split_edges(edges_to_split, tm_S, vpm_S, tm_T, vpm_T,  gt, is_source_mesh_fixed);
   }
+}
+
+// \ingroup PMP_repairing_grp
+//
+// Attempts to snap the vertices in `halfedge_range_A` onto edges of `halfedge_range_B`, and reciprocally.
+// A vertex from the first range is only snapped to an edge of the second range if the distance to
+// the edge is smaller than the tolerance prescribed at the vertex.
+//
+// \warning This function does not give any guarantee on the conformity between the two meshes after the snapping.
+// \warning This function does not merge vertices or the meshes, it is purely geometric.
+//
+// \tparam TriangleMesh a model of `FaceListGraph` and `MutableFaceGraph`
+// \tparam HalfedgeRange_A a model of `Range` with value type `boost::graph_traits<TriangleMesh>::%halfedge_descriptor`
+// \tparam HalfedgeRange_B a model of `Range` with value type `boost::graph_traits<TriangleMesh>::%halfedge_descriptor`
+// \tparam ToleranceMap_A a model of `ReadablePropertyMap` with key type `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+//                        and value type `GetGeomTraits<TriangleMesh, NamedParameters_A>::type::FT`
+// \tparam ToleranceMap_B a model of `ReadablePropertyMap` with key type `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+//                        and value type `GetGeomTraits<TriangleMesh, NamedParameters_A>::type::FT`
+// \tparam NamedParameters_A a sequence of \ref bgl_namedparameters "Named Parameters"
+// \tparam NamedParameters_B a sequence of \ref bgl_namedparameters "Named Parameters"
+//
+// \param halfedge_range_A a range of halfedges of the first mesh defining a set of vertices (as targets of the halfeges)
+// \param tm_A the first mesh to which the vertices in `halfedge_range_A` belong
+// \param tolerance_map_A a tolerance map associating to each vertex of the first range a tolerance value
+// \param halfedge_range_B a range of vertices of the second mesh defining a set of vertices (as targets of the halfeges)
+// \param tolerance_map_B a tolerance map associating to each vertex of the second range a tolerance value
+// \param tm_B the target mesh to which the vertices in `halfedge_range_B` belong
+// \param np_A an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+//
+// \cgalNamedParamsBegin
+//   \cgalParamBegin{vertex_point_map}
+//     the property map with the points associated to the vertices of the source mesh.
+//     The type of this map is model of `ReadWritePropertyMap`. If this parameter is omitted,
+//     an internal property map for `CGAL::vertex_point_t` must be available in `TriangleMesh`.
+//   \cgalParamEnd
+//   \cgalParamBegin{geom_traits}
+//     a geometric traits class instance, must be a model of `Kernel`
+//   \cgalParamEnd
+// \cgalNamedParamsEnd
+//
+// \param np_B an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+//
+// \cgalNamedParamsBegin
+//   \cgalParamBegin{vertex_point_map}
+//     the property map with the points associated to the vertices of the target mesh.
+//     The type of this map is model of `ReadablePropertyMap`. If this parameter is omitted,
+//     an internal property map for `CGAL::vertex_point_t` must be available in `TriangleMesh`.
+//   \cgalParamEnd
+// \cgalNamedParamsEnd
+//
+// \return the number of snapped vertex pairs
+//
+template <typename ConcurrencyTag = CGAL::Sequential_tag,
+          typename HalfedgeRange, typename TriangleMesh,
+          typename ToleranceMap_A, typename ToleranceMap_B,
+          typename NamedParameters_A, typename NamedParameters_B>
+std::size_t snap_non_conformal(HalfedgeRange& halfedge_range_A,
+                               TriangleMesh& tm_A,
+                               ToleranceMap_A tolerance_map_A,
+                               HalfedgeRange& halfedge_range_B,
+                               TriangleMesh& tm_B,
+                               ToleranceMap_B tolerance_map_B,
+                               const bool is_self_snapping, // == true if range and meshes are equal
+                               const NamedParameters_A& np_A,
+                               const NamedParameters_B& np_B)
+{
+#ifdef CGAL_PMP_SNAP_DEBUG
+  std::cout.precision(17);
+  std::cerr.precision(17);
 #endif
 
-  // Sort points falling on the same edge and split the edge
-  std::map<halfedge_descriptor, std::vector<Point> > points_per_edge;
-  typedef std::pair<Point, halfedge_descriptor> Pair_type;
-  for(const Pair_type& p : edges_to_split)
-    points_per_edge[p.second].push_back(p.first);
+  typedef typename boost::graph_traits<TriangleMesh>::vertex_descriptor           vertex_descriptor;
+  typedef typename boost::graph_traits<TriangleMesh>::halfedge_descriptor         halfedge_descriptor;
+  typedef typename boost::graph_traits<TriangleMesh>::face_descriptor             face_descriptor;
+
+  typedef CGAL::dynamic_vertex_property_t<bool>                                   Vertex_bool_tag;
+  typedef CGAL::dynamic_vertex_property_t<std::size_t>                            Vertex_size_t_tag;
+  typedef CGAL::dynamic_halfedge_property_t<bool>                                 Halfedge_bool_tag;
+
+  typedef typename boost::property_map<TriangleMesh, Vertex_bool_tag>::type       Locked_vertices;
+  typedef typename boost::property_map<TriangleMesh, Halfedge_bool_tag>::type     Locked_halfedges;
+  typedef typename boost::property_map<TriangleMesh, Vertex_size_t_tag>::type     Vertex_patch;
+
+  typedef typename internal_np::Lookup_named_param_def <
+    internal_np::face_patch_t, NamedParameters_A,
+      Constant_property_map<face_descriptor, std::size_t> /*default*/ >::type     Face_patch_map_A;
+  typedef typename internal_np::Lookup_named_param_def <
+    internal_np::face_patch_t, NamedParameters_B,
+      Constant_property_map<face_descriptor, std::size_t> /*default*/ >::type     Face_patch_map_B;
+
+  using CGAL::parameters::choose_parameter;
+  using CGAL::parameters::is_default_parameter;
+  using CGAL::parameters::get_parameter;
+
+  const bool is_same_mesh = (&tm_A == &tm_B);
+  const bool simplify_first_mesh = choose_parameter(get_parameter(np_A, internal_np::do_simplify_border), false);
+  const bool simplify_second_mesh = choose_parameter(get_parameter(np_B, internal_np::do_simplify_border), false);
+  const bool is_second_mesh_fixed = choose_parameter(get_parameter(np_B, internal_np::do_lock_mesh), false);
+
+  // vertex-vertex and vertex-edge snapping is only considered within compatible patches
+  Face_patch_map_A face_patch_map_A = choose_parameter(get_parameter(np_A, internal_np::face_patch),
+                                                       Constant_property_map<face_descriptor, std::size_t>(-1));
+  Face_patch_map_B face_patch_map_B = choose_parameter(get_parameter(np_B, internal_np::face_patch),
+                                                       Constant_property_map<face_descriptor, std::size_t>(-1));
+
+  Vertex_patch vertex_patch_map_A = get(Vertex_size_t_tag(), tm_A);
+  Vertex_patch vertex_patch_map_B = get(Vertex_size_t_tag(), tm_B);
+
+  for(const halfedge_descriptor h : halfedge_range_A)
+  {
+    CGAL_precondition(is_border(h, tm_A));
+    halfedge_descriptor h_opp = opposite(h, tm_A);
+    for(const vertex_descriptor v : vertices_around_face(h_opp, tm_A))
+      put(vertex_patch_map_A, v, get(face_patch_map_A, face(h_opp, tm_A)));
+  }
+
+  // @todo avoid that when 'self_snapping' is true
+  for(const halfedge_descriptor h : halfedge_range_B)
+  {
+    CGAL_precondition(is_border(h, tm_B));
+    halfedge_descriptor h_opp = opposite(h, tm_B);
+    for(const vertex_descriptor v : vertices_around_face(h_opp, tm_B))
+      put(vertex_patch_map_B, v, get(face_patch_map_B, face(h_opp, tm_B)));
+  }
 
 #ifdef CGAL_PMP_SNAP_DEBUG
-  std::cout << "split " << points_per_edge.size() << " edges" << std::endl;
+  std::cout << "Non-conformal snapping... Range sizes: "
+            << std::distance(halfedge_range_A.begin(), halfedge_range_A.end()) << " and "
+            << std::distance(halfedge_range_B.begin(), halfedge_range_B.end()) << std::endl;
+#endif
+
+  CGAL_expensive_precondition(is_valid_polygon_mesh(tm_S) && is_triangle_mesh(tm_S));
+  CGAL_expensive_precondition(is_valid_polygon_mesh(tm_T) && is_triangle_mesh(tm_T));
+
+  // Steps:
+  // - #1 Simplify the source range
+  // - #1bis Simplify the target range (if it's not locked)
+  // - #2 two-way vertex-vertex snapping
+  // - #3 two-way vertex-edge snapping
+
+  //////////////////////////////////////////////////////////////////////////////////////////////////
+  /// #1 and #1bis (Simplification of borders)
+  //////////////////////////////////////////////////////////////////////////////////////////////////
+
+#ifdef CGAL_PMP_SNAP_DEBUG
+  std::cout << "Simplify ranges (" << simplify_first_mesh << " " << simplify_second_mesh << ")..." << std::endl;
 #endif
 
-  typedef std::pair<const halfedge_descriptor, std::vector<Point> > Map_type;
-  for(Map_type& mt : points_per_edge)
+  if(simplify_first_mesh)
   {
-    const halfedge_descriptor mt_hd = mt.first;
-    const halfedge_descriptor mt_hd_opp = opposite(mt_hd, pmt);
-    CGAL_assertion(!is_border(mt_hd_opp, pmt));
+    internal::simplify_range(halfedge_range_A, tm_A, tolerance_map_A, np_A);
 
-    if(mt.second.size() > 1)
-    {
-#ifdef CGAL_PMP_SNAP_DEBUG_PP
-      std::cout << " MUST SORT ON BORDER!" << std::endl;
+#ifdef CGAL_PMP_SNAP_DEBUG_OUTPUT
+    std::ofstream("results/simplified_A.off") << std::setprecision(17) << tm_A;
 #endif
-      /// \todo Sorting the projected positions is too simple (for example, this doesn't work
-      ///       for a zigzaging polyline). Rather, points should be sorted following
-      ///       the order along the matching border. Note that this requires identifying
-      ///       matching polylines because a sequence of zigzaging points might not all project
-      ///       onto the same halfedge.
-      Point_along_edge_comparer cmp(mt_hd, gt, vpmt, pmt);
-      std::sort(mt.second.begin(), mt.second.end(), cmp);
-    }
+  }
 
-    halfedge_descriptor hd_to_split = mt_hd;
-    for(const Point& p : mt.second)
-    {
-#ifdef CGAL_PMP_SNAP_DEBUG_PP
-      std::cout << "SPLIT " << edge(hd_to_split, pmt) << " |||| "
-                << " vs " << source(hd_to_split, pmt) << " (" << pmt.point(source(hd_to_split, pmt)) << ")"
-                << " --- vt " << target(hd_to_split, pmt) << " (" << pmt.point(target(hd_to_split, pmt)) << ")" << std::endl;
-      std::cout << "  with pos " << p << std::endl;
+  if(!is_self_snapping && !is_second_mesh_fixed && simplify_second_mesh)
+  {
+    internal::simplify_range(halfedge_range_B, tm_B, tolerance_map_B, np_B);
+
+#ifdef CGAL_PMP_SNAP_DEBUG_OUTPUT
+    std::ofstream("results/simplified_B.off") << std::setprecision(17) << tm_B;
 #endif
+  }
 
-      halfedge_descriptor res = CGAL::Euler::split_edge(hd_to_split, pmt);
-      put(vpmt, target(res, pmt), p);
-      ++snapped_n;
+  //////////////////////////////////////////////////////////////////////////////////////////////////
+  /// #2 (Two-way vertex-vertex snapping)
+  //////////////////////////////////////////////////////////////////////////////////////////////////
+
+  // We keep in memory pairs of source/target edges that are stitchable after vertex-vertex snapping
+  // --> these halfedges should not be considered as targets in non-conformal snapping
+  // Similarly, matching vertices whose incident edges have matching directions are also locked
+  Locked_vertices locked_vertices_A = get(Vertex_bool_tag(), tm_A);
+  Locked_vertices locked_vertices_B = get(Vertex_bool_tag(), tm_B);
+  Locked_halfedges locked_halfedges_A = get(Halfedge_bool_tag(), tm_A);
+  Locked_halfedges locked_halfedges_B = get(Halfedge_bool_tag(), tm_B);
 
-      // @todo ensuring hd_to_split is safe... but is it necessary?
-      hd_to_split = next(res, pmt);
-      halfedge_descriptor hd_to_split_opp = opposite(hd_to_split, pmt);
+  std::size_t snapped_n = 0;
 
-      // Look at the geometry to determine which diagonal is better to use to split this new quad face
+  std::vector<std::pair<vertex_descriptor, vertex_descriptor> > locked_vertices;
+  std::vector<halfedge_descriptor> locked_halfedges_A_vector, locked_halfedges_B_vector;
 
-      /*           p
-       *         /   \
-       *    res /     \ hd_to_split
-       *       /       \
-       *      /         \
-       *    left       right
-       *     |         /
-       *     |        /
-       *     |       /
-       *     |      /
-       *     |     /
-       *       opp
-       */
+  snapped_n += internal::snap_vertices_two_way<ConcurrencyTag>(
+                 halfedge_range_A, tm_A, tolerance_map_A, vertex_patch_map_A,
+                 halfedge_range_B, tm_B, tolerance_map_B, vertex_patch_map_B,
+                 std::back_inserter(locked_vertices),
+                 std::back_inserter(locked_halfedges_A_vector),
+                 std::back_inserter(locked_halfedges_B_vector),
+                 is_second_mesh_fixed, np_A, np_B);
 
-      const Point_ref left_pt = get(vpmt, source(res, pmt));
-      const Point_ref right_pt = get(vpmt, target(hd_to_split, pmt));
-      const Point_ref opp = get(vpmt, target(next(opposite(res, pmt), pmt), pmt));
+  for(const auto& vpair : locked_vertices)
+  {
+    put(locked_vertices_A, vpair.first, true);
+    put(locked_vertices_B, vpair.second, true);
+    if(is_same_mesh)
+    {
+      put(locked_vertices_B, vpair.first, true);
+      put(locked_vertices_A, vpair.second, true);
+    }
+  }
 
-      // Check if 'p' is "visible" from 'opp' (i.e. its projection on the plane 'Pl(left, opp, right)'
-      // falls in the cone with apex 'opp' and sides given by 'left' and 'right')
-      const Vector n = gt.construct_orthogonal_vector_3_object()(right_pt, left_pt, opp);
+  for(const halfedge_descriptor h : locked_halfedges_A_vector)
+    put(locked_halfedges_A, h, true);
+  for(const halfedge_descriptor h : locked_halfedges_B_vector)
+    put(locked_halfedges_B, h, true);
 
-      const Point trans_left_pt = gt.construct_translated_point_3_object()(left_pt, n);
-      const Point trans_right_pt = gt.construct_translated_point_3_object()(right_pt, n);
+  if(is_same_mesh)
+  {
+    for(const halfedge_descriptor h : locked_halfedges_A_vector)
+      put(locked_halfedges_B, h, true);
+    for(const halfedge_descriptor h : locked_halfedges_B_vector)
+      put(locked_halfedges_A, h, true);
+  }
 
-      const bool left_of_left = (gt.orientation_3_object()(trans_left_pt, left_pt, opp, p) == CGAL::POSITIVE);
-      const bool right_of_right = (gt.orientation_3_object()(right_pt, trans_right_pt, opp, p) == CGAL::POSITIVE);
+#ifdef CGAL_PMP_SNAP_DEBUG_OUTPUT
+  std::ofstream("results/vertex_vertex_A.off") << std::setprecision(17) << tm_A;
+  std::ofstream("results/vertex_vertex_B.off") << std::setprecision(17) << tm_B;
+#endif
 
-      const bool is_visible = (!left_of_left && !right_of_right);
+  //////////////////////////////////////////////////////////////////////////////////////////////////
+  /// #3 (Two one-way vertex-edge snapping)
+  //////////////////////////////////////////////////////////////////////////////////////////////////
 
-#ifdef CGAL_PMP_SNAP_DEBUG_PP
-      std::cout << "Left/Right: " << left_of_left << " " << right_of_right << std::endl;
-      std::cout << "visible from " << opp << " ? " << is_visible << std::endl;
+#ifdef CGAL_PMP_SNAP_DEBUG
+  std::cout << " ///////////// Two one-way vertex-edge snapping (A --> B) " << std::endl;
 #endif
 
-      if(is_visible)
-      {
-        halfedge_descriptor new_hd = CGAL::Euler::split_face(hd_to_split_opp,
-                                                             prev(prev(mt_hd_opp, pmt), pmt), pmt);
-        hd_to_split = opposite(prev(new_hd, pmt), pmt);
-      }
-      else
-      {
-        halfedge_descriptor new_hd = CGAL::Euler::split_face(opposite(res, pmt),
-                                                             prev(hd_to_split_opp, pmt), pmt);
-        hd_to_split = opposite(next(new_hd, pmt), pmt);
-      }
-    }
+  snapped_n += internal::snap_non_conformal_one_way<ConcurrencyTag>(
+                 halfedge_range_A, tm_A, tolerance_map_A, vertex_patch_map_A, locked_vertices_A,
+                 halfedge_range_B, tm_B, face_patch_map_B, locked_halfedges_B,
+                 false /*source is never fixed*/, np_A, np_B);
+
+#ifdef CGAL_PMP_SNAP_DEBUG_OUTPUT
+  std::ofstream("results/vertex_edge_A.off") << std::setprecision(17) << tm_A;
+  std::ofstream("results/vertex_edge_B.off") << std::setprecision(17) << tm_B;
+#endif
+
+  if(!is_self_snapping)
+  {
+#ifdef CGAL_PMP_SNAP_DEBUG
+  std::cout << " ///////////// Two one-way vertex-edge snapping (B --> A) " << std::endl;
+#endif
+
+    snapped_n += internal::snap_non_conformal_one_way<ConcurrencyTag>(
+                   halfedge_range_B, tm_B, tolerance_map_B, vertex_patch_map_B, locked_vertices_B,
+                   halfedge_range_A, tm_A, face_patch_map_A, locked_halfedges_A,
+                   is_second_mesh_fixed, np_B, np_A);
   }
 
   return snapped_n;
 }
 
-template <typename HalfedgeRange, typename TriangleMesh, typename ToleranceMap>
-std::size_t snap_vertex_range_onto_vertex_range_non_conforming(const HalfedgeRange& source_hrange,
-                                                               TriangleMesh& pms,
-                                                               const HalfedgeRange& target_hrange,
-                                                               TriangleMesh& pmt,
-                                                               const ToleranceMap& tol_pmap)
-{
-  return snap_vertex_range_onto_vertex_range_non_conforming(source_hrange, pms, target_hrange, pmt,
-                                                            tol_pmap, parameters::all_default(),
-                                                            parameters::all_default());
-}
+} // namespace internal
+
+namespace experimental {
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
-////////////////////////////////////////////////////////////////////////////////////////////////////
+/// Convenience overloads
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 
-// Attempt to snap the vertices of the border of 'pm1' onto border edges of 'pm2'.
+template <typename ConcurrencyTag = CGAL::Sequential_tag,
+          typename TriangleMesh,
+          typename ToleranceMap_A, typename ToleranceMap_B,
+          typename NamedParameters_A, typename NamedParameters_B>
+std::size_t snap_borders(TriangleMesh& tm_A,
+                         ToleranceMap_A tolerance_map_A,
+                         TriangleMesh& tm_B,
+                         ToleranceMap_B tolerance_map_B,
+                         const NamedParameters_A& np_A,
+                         const NamedParameters_B& np_B)
+{
+  typedef typename boost::graph_traits<TriangleMesh>::halfedge_descriptor        halfedge_descriptor;
 
-template <typename TriangleMesh>
-std::size_t snap_border_vertices_non_conforming(TriangleMesh& pm1,
-                                                TriangleMesh& pm2)
+  std::vector<halfedge_descriptor> border_vertices_A;
+  border_halfedges(tm_A, std::back_inserter(border_vertices_A));
+  std::vector<halfedge_descriptor> border_vertices_B;
+  border_halfedges(tm_B, std::back_inserter(border_vertices_B));
+
+  return internal::snap_non_conformal<ConcurrencyTag>(border_vertices_A, tm_A, tolerance_map_A,
+                                                      border_vertices_B, tm_B, tolerance_map_B,
+                                                      false /*not self snapping*/, np_A, np_B);
+}
+
+template <typename ConcurrencyTag = CGAL::Sequential_tag,
+          typename TriangleMesh,
+          typename NamedParameters_A, typename NamedParameters_B>
+std::size_t snap_borders(TriangleMesh& tm_A,
+                         TriangleMesh& tm_B,
+                         const NamedParameters_A& np_A,
+                         const NamedParameters_B& np_B)
 {
   typedef typename boost::graph_traits<TriangleMesh>::halfedge_descriptor        halfedge_descriptor;
 
@@ -1034,84 +1209,109 @@ std::size_t snap_border_vertices_non_con
   typedef CGAL::dynamic_vertex_property_t<FT>                                    Vertex_property_tag;
   typedef typename boost::property_map<TriangleMesh, Vertex_property_tag>::type  Tolerance_map;
 
-  std::vector<halfedge_descriptor> border_vertices1;
-  border_halfedges(pm1, std::back_inserter(border_vertices1));
-
-  std::vector<halfedge_descriptor> border_vertices2;
-  if(&pm1 == &pm2)
-    border_vertices2 = border_vertices1;
-  else
-    border_halfedges(pm2, std::back_inserter(border_vertices2));
+  std::vector<halfedge_descriptor> border_vertices_A;
+  std::vector<halfedge_descriptor> border_vertices_B;
+  border_halfedges(tm_A, std::back_inserter(border_vertices_A));
+  border_halfedges(tm_B, std::back_inserter(border_vertices_B));
 
-  Tolerance_map tol_pmap = get(Vertex_property_tag(), pm1);
   const FT tol_mx((std::numeric_limits<double>::max)());
-  internal::assign_tolerance_with_local_edge_length_bound(border_vertices1, tol_pmap, tol_mx, pm1);
-
-  return snap_vertex_range_onto_vertex_range_non_conforming(border_vertices1, pm1,
-                                                            border_vertices2, pm2, tol_pmap,
-                                                            CGAL::parameters::all_default(),
-                                                            CGAL::parameters::all_default());
+  Tolerance_map tolerance_map_A = get(Vertex_property_tag(), tm_A);
+  internal::assign_tolerance_with_local_edge_length_bound(border_vertices_A, tolerance_map_A, tol_mx, tm_A, np_A);
+  Tolerance_map tolerance_map_B = get(Vertex_property_tag(), tm_B);
+  internal::assign_tolerance_with_local_edge_length_bound(border_vertices_B, tolerance_map_B, tol_mx, tm_B, np_B);
+
+  return internal::snap_non_conformal<ConcurrencyTag>(border_vertices_A, tm_A, tolerance_map_A,
+                                                      border_vertices_B, tm_B, tolerance_map_B,
+                                                      false /*no self snapping*/, np_A, np_B);
+}
+
+template <typename ConcurrencyTag = CGAL::Sequential_tag,
+          typename TriangleMesh,
+          typename ToleranceMap_A, typename ToleranceMap_B>
+std::size_t snap_borders(TriangleMesh& tm_A,
+                         ToleranceMap_A tolerance_map_A,
+                         TriangleMesh& tm_B,
+                         ToleranceMap_B tolerance_map_B)
+{
+  return snap_borders<ConcurrencyTag>(tm_A, tolerance_map_A, tm_B, tolerance_map_B,
+                                      CGAL::parameters::all_default(), CGAL::parameters::all_default());
+}
+
+template <typename ConcurrencyTag = CGAL::Sequential_tag,
+          typename TriangleMesh>
+std::size_t snap_borders(TriangleMesh& tm_A,
+                         TriangleMesh& tm_B)
+{
+  return snap_borders<ConcurrencyTag>(tm_A, tm_B, CGAL::parameters::all_default(), CGAL::parameters::all_default());
 }
 
-template <typename TriangleMesh, typename ToleranceMap, typename NamedParameters>
-std::size_t snap_border_vertices_non_conforming(TriangleMesh& pm1,
-                                                TriangleMesh& pm2,
-                                                const ToleranceMap& tol_pmap,
-                                                const NamedParameters& np1,
-                                                const NamedParameters& np2)
+////////////////////////////////////////////////////////////////////////////////////////////////////
+////////////////////////////////////////////////////////////////////////////////////////////////////
+////////////////////////////////////////////////////////////////////////////////////////////////////
+// Same as above, but with a single mesh
+
+template <typename ConcurrencyTag = CGAL::Sequential_tag,
+          typename TriangleMesh,
+          typename ToleranceMap,
+          typename CGAL_PMP_NP_TEMPLATE_PARAMETERS>
+std::size_t snap_borders(TriangleMesh& tm,
+                         ToleranceMap tolerance_map,
+                         const CGAL_PMP_NP_CLASS& np)
 {
   typedef typename boost::graph_traits<TriangleMesh>::halfedge_descriptor        halfedge_descriptor;
 
-  std::vector<halfedge_descriptor> border_vertices1;
-  border_halfedges(pm1, std::back_inserter(border_vertices1));
-
-  std::vector<halfedge_descriptor> border_vertices2;
-  border_halfedges(pm2, std::back_inserter(border_vertices2));
+  std::vector<halfedge_descriptor> border_vertices;
+  border_halfedges(tm, std::back_inserter(border_vertices));
 
-  return snap_vertex_range_onto_vertex_range_non_conforming(border_vertices1, pm1,
-                                                            border_vertices2, pm2,
-                                                            tol_pmap, np1, np2);
+  return internal::snap_non_conformal<ConcurrencyTag>(border_vertices, tm, tolerance_map,
+                                                      border_vertices, tm, tolerance_map,
+                                                      true /*self snapping*/, np, np);
 }
 
-template <typename TriangleMesh, typename ToleranceMap>
-std::size_t snap_border_vertices_non_conforming(TriangleMesh& pm1,
-                                                TriangleMesh& pm2,
-                                                const ToleranceMap& tol_pmap)
+template <typename ConcurrencyTag = CGAL::Sequential_tag,
+          typename TriangleMesh,
+          typename ToleranceMap>
+std::size_t snap_borders(TriangleMesh& tm,
+                         ToleranceMap tolerance_map)
 {
-  return snap_border_vertices_non_conforming(pm1, pm2, tol_pmap,
-                                             CGAL::parameters::all_default(), CGAL::parameters::all_default());
+  return snap_borders<ConcurrencyTag>(tm, tolerance_map, CGAL::parameters::all_default());
 }
 
-////////////////////////////////////////////////////////////////////////////////////////////////////
-////////////////////////////////////////////////////////////////////////////////////////////////////
-////////////////////////////////////////////////////////////////////////////////////////////////////
-
-template <typename TriangleMesh>
-std::size_t snap_border_vertices_non_conforming(TriangleMesh& pm)
+template <typename ConcurrencyTag = CGAL::Sequential_tag,
+          typename TriangleMesh,
+          typename CGAL_PMP_NP_TEMPLATE_PARAMETERS>
+std::size_t snap_borders(TriangleMesh& tm,
+                         const CGAL_PMP_NP_CLASS& np)
 {
-  return snap_border_vertices_non_conforming(pm, pm);
-}
+  typedef typename boost::graph_traits<TriangleMesh>::halfedge_descriptor        halfedge_descriptor;
 
-template <typename TriangleMesh, typename ToleranceMap, typename NamedParameters>
-std::size_t snap_border_vertices_non_conforming(TriangleMesh& pm,
-                                                const ToleranceMap& tol_pmap,
-                                                const NamedParameters& np)
-{
-  return snap_border_vertices_non_conforming(pm, pm, tol_pmap, np, np);
+  typedef typename GetGeomTraits<TriangleMesh>::type                             GT;
+  typedef typename GT::FT                                                        FT;
+
+  typedef CGAL::dynamic_vertex_property_t<FT>                                    Vertex_property_tag;
+  typedef typename boost::property_map<TriangleMesh, Vertex_property_tag>::type  Tolerance_map;
+
+  std::vector<halfedge_descriptor> border_vertices;
+  border_halfedges(tm, std::back_inserter(border_vertices));
+
+  const FT tol_mx((std::numeric_limits<double>::max)());
+  Tolerance_map tolerance_map = get(Vertex_property_tag(), tm);
+  internal::assign_tolerance_with_local_edge_length_bound(border_vertices, tolerance_map, tol_mx, tm, np);
+
+  return internal::snap_non_conformal<ConcurrencyTag>(border_vertices, tm, tolerance_map,
+                                                      border_vertices, tm, tolerance_map,
+                                                      true /*self snapping*/, np, np);
 }
 
-template <typename TriangleMesh, typename ToleranceMap>
-std::size_t snap_border_vertices_non_conforming(TriangleMesh& pm,
-                                                const ToleranceMap& tol_pmap)
+template <typename ConcurrencyTag = CGAL::Sequential_tag,
+          typename TriangleMesh>
+std::size_t snap_borders(TriangleMesh& tm)
 {
-  return snap_border_vertices_non_conforming(pm, pm, tol_pmap,
-                                             CGAL::parameters::all_default(), CGAL::parameters::all_default());
+  return snap_borders<ConcurrencyTag>(tm, CGAL::parameters::all_default());
 }
 
 } // end namespace experimental
-
 } // end namespace Polygon_mesh_processing
-
 } // end namespace CGAL
 
-#endif // CGAL_POLYGON_MESH_PROCESSING_INTERNAL_SNAP_H
+#endif // CGAL_POLYGON_MESH_PROCESSING_SNAPPING_SNAP_H
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Snapping/snap_vertices.h 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Snapping/snap_vertices.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/internal/Snapping/snap_vertices.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/internal/Snapping/snap_vertices.h	2020-09-08 12:15:23.000000000 +0000
@@ -0,0 +1,941 @@
+// Copyright (c) 2018, 2019 GeometryFactory (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Snapping/snap_vertices.h $
+// $Id: snap_vertices.h 038869c 2020-05-12T15:12:21+02:00 Mael Rouxel-Labbé
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Mael Rouxel-Labbé
+
+#ifndef CGAL_POLYGON_MESH_PROCESSING_SNAPPING_SNAP_VERTICES_H
+#define CGAL_POLYGON_MESH_PROCESSING_SNAPPING_SNAP_VERTICES_H
+
+#include <CGAL/license/Polygon_mesh_processing/repair.h>
+
+#ifdef CGAL_PMP_SNAP_DEBUG_PP
+ #ifndef CGAL_PMP_SNAP_DEBUG
+  #define CGAL_PMP_SNAP_DEBUG
+ #endif
+#endif
+
+#include <CGAL/Polygon_mesh_processing/internal/Snapping/helper.h>
+#include <CGAL/Polygon_mesh_processing/internal/named_function_params.h>
+#include <CGAL/Polygon_mesh_processing/internal/named_params_helper.h>
+#include <CGAL/Polygon_mesh_processing/border.h>
+
+#include <CGAL/assertions.h>
+#include <CGAL/Bbox_3.h>
+#include <CGAL/boost/graph/helpers.h>
+#include <CGAL/boost/graph/iterator.h>
+#include <CGAL/boost/graph/selection.h>
+#include <CGAL/box_intersection_d.h>
+#include <CGAL/circulator.h>
+#include <CGAL/Dynamic_property_map.h>
+#include <CGAL/iterator.h>
+#include <CGAL/Kernel/global_functions.h>
+#include <CGAL/number_utils.h>
+#include <CGAL/utility.h>
+#include <CGAL/Real_timer.h>
+#include <CGAL/tags.h>
+
+#include <boost/multi_index_container.hpp>
+#include <boost/multi_index/ordered_index.hpp>
+#include <boost/multi_index/member.hpp>
+
+#ifdef CGAL_LINKED_WITH_TBB
+# include <tbb/blocked_range.h>
+# include <tbb/concurrent_vector.h>
+# include <tbb/parallel_for.h>
+#endif
+
+#include <functional>
+#ifdef CGAL_PMP_SNAP_DEBUG_OUTPUT
+#include <fstream>
+#endif
+#include <iostream>
+#include <iterator>
+#include <limits>
+#include <map>
+#include <type_traits>
+#include <unordered_set>
+#include <utility>
+#include <vector>
+
+namespace CGAL {
+namespace Polygon_mesh_processing {
+namespace internal {
+
+// Data entry of the multi-index container
+template <typename PolygonMesh, typename GeomTraits>
+struct Snapping_pair
+{
+  typedef typename GeomTraits::FT                                                     FT;
+  typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor              halfedge_descriptor;
+  typedef std::vector<halfedge_descriptor>                                            Vertex_container;
+  typedef std::pair<Vertex_container, FT>                                             Unique_vertex;
+  typedef const Unique_vertex*                                                        Unique_vertex_ptr;
+
+  Snapping_pair(Unique_vertex_ptr uv_a_, Unique_vertex_ptr uv_b_, const FT sq_dist_)
+    : uv_a(uv_a_), uv_b(uv_b_), sq_dist(sq_dist_)
+  { }
+
+  Unique_vertex_ptr uv_a;
+  Unique_vertex_ptr uv_b;
+  FT sq_dist;
+};
+
+// Functor that just forwards the pair of the two intersecting boxes
+template <class OutputIterator>
+struct Intersecting_boxes_pairs_report
+{
+  Intersecting_boxes_pairs_report(OutputIterator it) :  m_iterator(it) { }
+
+  template <class Box>
+  void operator()(const Box* b, const Box* c) const {
+    *m_iterator++ = std::make_pair(b->info(), c->info());
+  }
+
+  mutable OutputIterator m_iterator;
+};
+
+template <typename SnappingPairContainer,
+          typename PolygonMesh, typename GeomTraits,
+          typename VPM_A, typename VPM_B,
+          typename ToleranceMap_A, typename ToleranceMap_B,
+          typename VertexPatchMap_A, typename VertexPatchMap_B,
+          typename Box
+#ifdef CGAL_LINKED_WITH_TBB
+          , typename UniqueVertexPairContainer = void
+          , typename ToKeepContainer = void
+#endif
+          >
+struct Vertex_proximity_report
+{
+  typedef typename boost::graph_traits<PolygonMesh>::vertex_descriptor                vertex_descriptor;
+  typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor              halfedge_descriptor;
+  typedef typename boost::graph_traits<PolygonMesh>::edge_descriptor                  edge_descriptor;
+
+  typedef typename GeomTraits::FT                                                     FT;
+
+  typedef std::vector<halfedge_descriptor>                                            Vertex_container;
+  typedef std::pair<Vertex_container, FT>                                             Unique_vertex;
+  typedef const Unique_vertex*                                                        Unique_vertex_ptr;
+  typedef std::pair<Unique_vertex_ptr, Unique_vertex_ptr>                             Unique_vertex_pair;
+
+  Vertex_proximity_report(SnappingPairContainer& snapping_pairs,
+                          const PolygonMesh& tm_A,
+                          ToleranceMap_A tolerance_map_A,
+                          VertexPatchMap_A vertex_patch_map_A,
+                          const VPM_A& vpm_A,
+                          const PolygonMesh& tm_B,
+                          ToleranceMap_B tolerance_map_B,
+                          VertexPatchMap_B vertex_patch_map_B,
+                          const VPM_B& vpm_B,
+                          const GeomTraits& gt
+#ifdef CGAL_LINKED_WITH_TBB
+                          , const UniqueVertexPairContainer* uv_pairs = nullptr
+                          , ToKeepContainer* to_keep = nullptr
+#endif
+                          )
+    :
+      m_tm_A(tm_A),
+      m_tolerance_map_A(tolerance_map_A),
+      m_vertex_patch_map_A(vertex_patch_map_A),
+      m_vpm_A(vpm_A),
+      m_tm_B(tm_B),
+      m_tolerance_map_B(tolerance_map_B),
+      m_vertex_patch_map_B(vertex_patch_map_B),
+      m_vpm_B(vpm_B),
+      m_gt(gt),
+      m_snapping_pairs(snapping_pairs)
+#ifdef CGAL_LINKED_WITH_TBB
+    , m_uv_pairs(uv_pairs)
+    , m_to_keep(to_keep)
+#endif
+  { }
+
+  bool are_equal_vertices(Unique_vertex_ptr uv_a, Unique_vertex_ptr uv_b) const
+  {
+    if(&m_tm_A != &m_tm_B) // must be the same mesh
+      return false;
+
+    if(uv_a->first.size() != uv_b->first.size())
+      return false;
+
+    CGAL_assertion(!uv_a->first.empty() && !uv_b->first.empty());
+    const halfedge_descriptor ha = uv_a->first.front();
+    return (std::find(uv_b->first.begin(), uv_b->first.end(), ha) != uv_b->first.end());
+  }
+
+  std::pair<FT, bool> do_keep(Unique_vertex_ptr uv_a, Unique_vertex_ptr uv_b) const
+  {
+    const Vertex_container& vs_a = uv_a->first;
+    const Vertex_container& vs_b = uv_b->first;
+    const vertex_descriptor va = target(vs_a.front(), m_tm_A);
+    const vertex_descriptor vb = target(vs_b.front(), m_tm_B);
+
+    // Check for patch compatibility
+    if(get(m_vertex_patch_map_A, va) != get(m_vertex_patch_map_B, vb))
+      return std::make_pair(-1, false); // ignore
+
+    // Reject if same mesh and already grouped together
+    if(are_equal_vertices(uv_a, uv_b))
+      return std::make_pair(-1, false);
+
+    const FT tol_a = uv_a->second;
+    const FT tol_b = uv_b->second;
+    CGAL_assertion(tol_a >= FT(0));
+    CGAL_assertion(tol_b >= FT(0));
+
+    const FT upper_bound_squared = CGAL::square(0.5*(tol_a + tol_b));
+    CGAL::Comparison_result res =
+      m_gt.compare_squared_distance_3_object()(get(m_vpm_A, va), get(m_vpm_B, vb), upper_bound_squared);
+
+#ifdef CGAL_PMP_SNAP_DEBUG_PP
+    std::cout << "squared distance between "
+              << va << " [" << get(m_vpm_A, va) << "] and "
+              << vb << " [" << get(m_vpm_B, vb) << "]: " << sq_dist
+              << " (tol a/b: " << tol_a << " " << tol_b << ") larger? " << (res == CGAL::LARGER)
+              << std::endl;
+#endif
+
+    if(res == CGAL::LARGER)
+      return std::make_pair(-1, false); // ignore
+    else
+    {
+      const FT sq_dist = (min)(m_gt.compute_squared_distance_3_object()(get(m_vpm_A, va), get(m_vpm_B, vb)),
+                               upper_bound_squared);
+      return std::make_pair(sq_dist, true); // keep
+    }
+  }
+
+  // that's the sequential version
+  void operator()(const Box* a, const Box* b)
+  {
+    Unique_vertex_ptr uv_a = a->info(), uv_b = b->info();
+    const std::pair<FT, bool> res = do_keep(uv_a, uv_b);
+    if(res.second)
+      m_snapping_pairs.insert(Snapping_pair<PolygonMesh, GeomTraits>(uv_a, uv_b, res.first));
+  }
+
+#ifdef CGAL_LINKED_WITH_TBB
+  // that's the parallel version
+  void operator()(const tbb::blocked_range<std::size_t>& r) const
+  {
+    CGAL_assertion(m_uv_pairs != nullptr);
+    CGAL_assertion(m_to_keep != nullptr);
+
+    for(std::size_t ri = r.begin(); ri != r.end(); ++ri)
+    {
+      const Unique_vertex_pair& hp = (*m_uv_pairs)[ri];
+      m_to_keep->operator[](ri) = do_keep(hp.first, hp.second);
+    }
+  }
+#endif
+
+private:
+  const PolygonMesh& m_tm_A;
+  ToleranceMap_A m_tolerance_map_A;
+  VertexPatchMap_A m_vertex_patch_map_A;
+  VPM_A m_vpm_A;
+  const PolygonMesh& m_tm_B;
+  ToleranceMap_B m_tolerance_map_B;
+  VertexPatchMap_B m_vertex_patch_map_B;
+  VPM_B m_vpm_B;
+  const GeomTraits& m_gt;
+
+  SnappingPairContainer& m_snapping_pairs; // will only be filled here in the sequential setting
+
+#ifdef CGAL_LINKED_WITH_TBB
+  const UniqueVertexPairContainer* m_uv_pairs;
+  ToKeepContainer* m_to_keep;
+#endif
+};
+
+template <typename ConcurrencyTag = CGAL::Sequential_tag,
+          typename HalfedgeRange_A, typename HalfedgeRange_B, typename PolygonMesh,
+          typename ToleranceMap_A, typename ToleranceMap_B,
+          typename VertexPatchMap_A, typename VertexPatchMap_B,
+          typename LockableVerticesOutputIterator, typename LockableHalfedgesOutputIterator,
+          typename NamedParameters_A, typename NamedParameters_B>
+std::size_t snap_vertices_two_way(const HalfedgeRange_A& halfedge_range_A,
+                                  PolygonMesh& tm_A,
+                                  ToleranceMap_A tolerance_map_A,
+                                  VertexPatchMap_A vertex_patch_map_A,
+                                  const HalfedgeRange_B& halfedge_range_B,
+                                  PolygonMesh& tm_B,
+                                  ToleranceMap_B tolerance_map_B,
+                                  VertexPatchMap_B vertex_patch_map_B,
+                                  LockableVerticesOutputIterator lockable_vps_out,
+                                  LockableHalfedgesOutputIterator lockable_ha_out,
+                                  LockableHalfedgesOutputIterator lockable_hb_out,
+                                  const bool is_second_mesh_fixed,
+                                  const NamedParameters_A& np_A,
+                                  const NamedParameters_B& np_B)
+{
+  typedef typename boost::graph_traits<PolygonMesh>::vertex_descriptor                vertex_descriptor;
+  typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor              halfedge_descriptor;
+
+  typedef typename GetVertexPointMap<PolygonMesh, NamedParameters_A>::type            VPM_A;
+  typedef typename GetVertexPointMap<PolygonMesh, NamedParameters_B>::type            VPM_B;
+
+  typedef typename GetGeomTraits<PolygonMesh, NamedParameters_A>::type                GT;
+  typedef typename GT::FT                                                             FT;
+  typedef typename boost::property_traits<VPM_B>::value_type                          Point;
+
+  typedef std::vector<halfedge_descriptor>                                            Vertex_container;
+  typedef std::pair<Vertex_container, FT>                                             Unique_vertex;
+  typedef const Unique_vertex*                                                        Unique_vertex_ptr;
+  typedef std::map<std::pair<Point, std::size_t>, Unique_vertex>                      Unique_positions;
+
+  typedef Box_intersection_d::ID_FROM_BOX_ADDRESS                                             Box_policy;
+  typedef CGAL::Box_intersection_d::Box_with_info_d<double, 3, Unique_vertex_ptr, Box_policy> Box;
+
+  using parameters::get_parameter;
+  using parameters::choose_parameter;
+
+  CGAL_static_assertion((std::is_same<Point, typename GT::Point_3>::value));
+
+  GT gt = choose_parameter<GT>(get_parameter(np_A, internal_np::geom_traits));
+  VPM_A vpm_A = choose_parameter(get_parameter(np_A, internal_np::vertex_point),
+                                 get_property_map(vertex_point, tm_A));
+  VPM_B vpm_B = choose_parameter(get_parameter(np_B, internal_np::vertex_point),
+                                 get_property_map(vertex_point, tm_B));
+
+#ifdef CGAL_PMP_SNAP_DEBUG
+  std::cout << "Finding snappables vertices. Range sizes: "
+            << std::distance(halfedge_range_A.begin(), halfedge_range_A.end()) << " and "
+            << std::distance(halfedge_range_B.begin(), halfedge_range_B.end()) << std::endl;
+#endif
+
+  if(is_empty_range(halfedge_range_A.begin(), halfedge_range_A.end()) ||
+     is_empty_range(halfedge_range_B.begin(), halfedge_range_B.end()))
+    return 0;
+
+  // Vertex-Vertex snapping is performed as follows:
+  // - Identify points which are already equal and group them together so that they are moved together
+  // - Create a single box for these points
+
+  std::vector<Box> boxes_A;
+  boxes_A.reserve(halfedge_range_A.size());
+  std::vector<Box> boxes_B;
+  boxes_B.reserve(halfedge_range_B.size());
+
+  Unique_positions unique_positions_A;
+  for(halfedge_descriptor h : halfedge_range_A)
+  {
+    const vertex_descriptor v = target(h, tm_A);
+    const FT tolerance = get(tolerance_map_A, v);
+
+#ifdef CGAL_PMP_SNAP_DEBUG_PP
+    std::cout << "Pos (A): " << v << " (" << get(vpm_A, v) << ")" << std::endl;
+#endif
+
+    Vertex_container nvc {{ h }};
+    std::pair<typename Unique_positions::iterator, bool> is_insert_successful =
+      unique_positions_A.insert(std::make_pair(
+                                  std::make_pair(get(vpm_A, v), get(vertex_patch_map_A, v)), // point and patch id
+                                  std::make_pair(nvc, tolerance)));
+
+    if(!is_insert_successful.second) // point was already met
+    {
+      Unique_vertex& uv = is_insert_successful.first->second;
+      Vertex_container& vc = uv.first; // second is the tolerance
+      CGAL_assertion(std::find(vc.begin(), vc.end(), h) == vc.end());
+      vc.push_back(h);
+      uv.second = (std::min)(uv.second, tolerance);
+    }
+  }
+
+  // same for tm_B (@todo avoid all that for self snapping + use self_intersection_d)
+  Unique_positions unique_positions_B;
+  for(halfedge_descriptor h : halfedge_range_B)
+  {
+    const vertex_descriptor v = target(h, tm_B);
+    const FT tolerance = get(tolerance_map_B, v);
+
+#ifdef CGAL_PMP_SNAP_DEBUG_PP
+    std::cout << "Pos (B): " << v << " (" << get(vpm_B, v) << ")" << std::endl;
+#endif
+
+    Vertex_container nvc {{ h }};
+    std::pair<typename Unique_positions::iterator, bool> is_insert_successful =
+      unique_positions_B.insert(std::make_pair(
+                                  std::make_pair(get(vpm_B, v), get(vertex_patch_map_B, v)), // point and patch id
+                                  std::make_pair(nvc, tolerance)));
+
+    if(!is_insert_successful.second) // point was already met
+    {
+      Unique_vertex& uv = is_insert_successful.first->second;
+      Vertex_container& vc = uv.first; // second is the tolerance
+      CGAL_assertion(std::find(vc.begin(), vc.end(), h) == vc.end());
+      vc.push_back(h);
+      uv.second = (std::min)(uv.second, tolerance);
+    }
+  }
+
+  // Actually build the boxes now
+  for(const auto& p : unique_positions_A)
+  {
+#ifdef CGAL_PMP_SNAP_DEBUG_PP
+    std::cout << "Unique_vertex (A), pos: " << p.first.first << " vertices:";
+    for(const halfedge_descriptor h : p.second.first)
+      std::cout << " " << target(h, tm_A);
+    std::cout << std::endl;
+#endif
+
+    const Unique_vertex& ev = p.second;
+    CGAL_assertion(!ev.first.empty());
+
+    // this only makes the box a little larger to ease intersection computations,
+    // the final tolerance is not changed
+    const double eps = 1.01 * CGAL::to_double(ev.second);
+    const Bbox_3 pb = gt.construct_bbox_3_object()(p.first.first);
+    const Bbox_3 b(pb.xmin() - eps, pb.ymin() - eps, pb.zmin() - eps,
+                   pb.xmax() + eps, pb.ymax() + eps, pb.zmax() + eps);
+    boxes_A.push_back(Box(b, &ev));
+  }
+
+  for(const auto& p : unique_positions_B)
+  {
+#ifdef CGAL_PMP_SNAP_DEBUG_PP
+    std::cout << "Unique_vertex (B), pos: " << p.first.first << " vertices:";
+    for(const halfedge_descriptor h : p.second.first)
+      std::cout << " " << target(h, tm_B);
+    std::cout << std::endl;
+#endif
+
+    const Unique_vertex& ev = p.second;
+    CGAL_assertion(!ev.first.empty());
+
+    const double eps = 1.01 * CGAL::to_double(ev.second);
+    const Bbox_3 pb = gt.construct_bbox_3_object()(p.first.first);
+    const Bbox_3 b(pb.xmin() - eps, pb.ymin() - eps, pb.zmin() - eps,
+                   pb.xmax() + eps, pb.ymax() + eps, pb.zmax() + eps);
+    boxes_B.push_back(Box(b, &ev));
+  }
+
+  // @fixme bench and don't use ptrs if not useful
+  std::vector<const Box*> boxes_A_ptr;
+  boxes_A_ptr.reserve(boxes_A.size());
+  for(const Box& b : boxes_A)
+    boxes_A_ptr.push_back(&b);
+
+  std::vector<const Box*> boxes_B_ptr;
+  boxes_B_ptr.reserve(boxes_B.size());
+  for(const Box& b : boxes_B)
+    boxes_B_ptr.push_back(&b);
+
+  // Use a multi index to sort easily by sources, targets, AND distance.
+  // Then, look up the distances in increasing order, and snap whenever the source and the target
+  // have both not been snapped yet.
+  typedef internal::Snapping_pair<PolygonMesh, GT>                                    Snapping_pair;
+  typedef boost::multi_index::multi_index_container<
+    Snapping_pair,
+    boost::multi_index::indexed_by<
+      boost::multi_index::ordered_non_unique<
+        BOOST_MULTI_INDEX_MEMBER(Snapping_pair, Unique_vertex_ptr, uv_a)>,
+      boost::multi_index::ordered_non_unique<
+        BOOST_MULTI_INDEX_MEMBER(Snapping_pair, Unique_vertex_ptr, uv_b)>,
+      boost::multi_index::ordered_non_unique<
+        BOOST_MULTI_INDEX_MEMBER(Snapping_pair, FT, sq_dist)>
+    >
+  >                                                                                   Snapping_pair_container;
+
+  Snapping_pair_container snapping_pairs;
+
+#if !defined(CGAL_LINKED_WITH_TBB)
+  CGAL_static_assertion_msg (!(std::is_convertible<ConcurrencyTag, Parallel_tag>::value),
+                             "Parallel_tag is enabled but TBB is unavailable.");
+#else
+  if(std::is_convertible<ConcurrencyTag, Parallel_tag>::value)
+  {
+    typedef std::pair<Unique_vertex_ptr, Unique_vertex_ptr>                           Unique_vertex_pair;
+    typedef tbb::concurrent_vector<Unique_vertex_pair>                                Unique_vertex_pairs;
+    typedef std::back_insert_iterator<Unique_vertex_pairs>                            UVP_output_iterator;
+
+    Unique_vertex_pairs uv_pairs;
+    Intersecting_boxes_pairs_report<UVP_output_iterator> callback(std::back_inserter(uv_pairs));
+
+    CGAL::Real_timer timer;
+    timer.start();
+
+    // Grab the boxes that are interesecting but don't do any extra filtering (in parallel)
+    CGAL::box_intersection_d<CGAL::Parallel_tag>(boxes_A_ptr.begin(), boxes_A_ptr.end(),
+                                                 boxes_B_ptr.begin(), boxes_B_ptr.end(),
+                                                 callback);
+
+    std::cout << "time for box_d: " << timer.time() << std::endl;
+
+    // Actually filter the range of intersecting boxes now (in parallel)
+    typedef std::vector<std::pair<FT, bool> >                                         Filters;
+    typedef Vertex_proximity_report<Snapping_pair_container,
+                                    PolygonMesh, GT, VPM_A, VPM_B,
+                                    ToleranceMap_A, ToleranceMap_B,
+                                    VertexPatchMap_A, VertexPatchMap_B,
+                                    Box, Unique_vertex_pairs, Filters>                Reporter;
+
+    Filters to_keep(uv_pairs.size());
+    Reporter proximity_filterer(snapping_pairs, tm_A, tolerance_map_A, vertex_patch_map_A, vpm_A,
+                                                tm_B, tolerance_map_B, vertex_patch_map_B, vpm_B,
+                                gt, &uv_pairs, &to_keep);
+    tbb::parallel_for(tbb::blocked_range<std::size_t>(0, uv_pairs.size()), proximity_filterer);
+
+    // Now fill the multi index, sequentially
+    for(std::size_t i=0, uvps = uv_pairs.size(); i<uvps; ++i)
+      if(to_keep[i].second)
+        snapping_pairs.insert(Snapping_pair(uv_pairs[i].first, uv_pairs[i].second, to_keep[i].first));
+  }
+  else
+#endif
+  {
+    typedef Vertex_proximity_report<Snapping_pair_container, PolygonMesh, GT,
+                                    VPM_A, VPM_B, ToleranceMap_A, ToleranceMap_B,
+                                    VertexPatchMap_A, VertexPatchMap_B, Box>          Reporter;
+
+    Reporter vpr(snapping_pairs, tm_A, tolerance_map_A, vertex_patch_map_A, vpm_A,
+                                 tm_B, tolerance_map_B, vertex_patch_map_B, vpm_B, gt);
+
+    // Shenanigans to pass a reference as callback (which is copied by value by 'box_intersection_d')
+    std::function<void(const Box*, const Box*)> callback(std::ref(vpr));
+
+    CGAL::box_intersection_d<CGAL::Sequential_tag>(boxes_A_ptr.begin(), boxes_A_ptr.end(),
+                                                   boxes_B_ptr.begin(), boxes_B_ptr.end(),
+                                                   callback);
+  }
+
+  //////////////////////////////////////////////////////////////////////////////////////////////////
+  /// Done collecting; start matching
+  //////////////////////////////////////////////////////////////////////////////////////////////////
+
+  if(snapping_pairs.empty())
+    return 0;
+
+  typedef std::pair<Unique_vertex_ptr, Unique_vertex_ptr>                             Unique_vertex_pair;
+  std::vector<Unique_vertex_pair> snappable_vertices_pairs;
+
+  // Sorted views of the container
+  typedef typename Snapping_pair_container::template nth_index<0>::type               Container_by_source;
+  typedef typename Snapping_pair_container::template nth_index<1>::type               Container_by_target;
+  typedef typename Snapping_pair_container::template nth_index<2>::type               Container_by_distance;
+
+  Container_by_source& container_by_source = snapping_pairs.template get<0>();
+  Container_by_target& container_by_target = snapping_pairs.template get<1>();
+  Container_by_distance& container_by_dist = snapping_pairs.template get<2>();
+
+  // @todo this could be a best-match algorithm rather than a greedy algorithm,
+  // but is the increased complexity -and thus runtime- worth it?
+  CGAL_assertion_code(FT prev = -1;)
+  while(!container_by_dist.empty())
+  {
+    const Snapping_pair& sp = *(container_by_dist.begin());
+    Unique_vertex_ptr uv_a = sp.uv_a;
+    Unique_vertex_ptr uv_b = sp.uv_b;
+    CGAL_assertion(sp.sq_dist >= prev);
+    CGAL_assertion_code(prev = sp.sq_dist;)
+
+#ifdef CGAL_PMP_SNAP_DEBUG_PP
+    const Point& pa = get(vpm_A, target(uv_a->first.front(), tm_A));
+    const Point& pb = get(vpm_B, target(uv_b->first.front(), tm_B));
+    std::cout << "Snapping (" << pa << ") to (" << pb << ") at dist: " << sp.sq_dist << std::endl;
+    std::cout << "#verts A: " << uv_a->first.size() << " #verts B: " << uv_b->first.size() << std::endl;
+#endif
+
+    snappable_vertices_pairs.emplace_back(uv_a, uv_b);
+
+    // 'va' and 'vb' cannot be used anymore, remove them from the container
+    container_by_source.erase(uv_a);
+    container_by_target.erase(uv_b);
+
+    if((&tm_A == &tm_B))
+    {
+      container_by_source.erase(uv_b);
+      container_by_target.erase(uv_a);
+    }
+  }
+
+#ifdef CGAL_PMP_SNAP_DEBUG
+  std::cout << snappable_vertices_pairs.size() << " snappable pair(s)" << std::endl;
+#endif
+
+  //////////////////////////////////////////////////////////////////////////////////////////////////
+  /// Done matching; start snapping
+  //////////////////////////////////////////////////////////////////////////////////////////////////
+
+  std::size_t counter = 0;
+
+#ifdef CGAL_PMP_SNAP_DEBUG_OUTPUT
+  std::ofstream out_edges("results/snappable.polylines.txt");
+  out_edges.precision(17);
+#endif
+
+  for(const Unique_vertex_pair& uvp : snappable_vertices_pairs)
+  {
+    Unique_vertex_ptr uv_a = uvp.first;
+    Unique_vertex_ptr uv_b = uvp.second;
+    const Vertex_container& vs_a = uv_a->first;
+    const Vertex_container& vs_b = uv_b->first;
+    const vertex_descriptor va = target(vs_a.front(), tm_A);
+    const vertex_descriptor vb = target(vs_b.front(), tm_B);
+
+#ifdef CGAL_PMP_SNAP_DEBUG_OUTPUT
+    out_edges << "2 " << tm_A.point(va) << " " << tm_B.point(vb) << std::endl;
+#endif
+
+    if(!gt.equal_3_object()(get(vpm_A, va), get(vpm_B, vb)))
+    {
+      if(is_second_mesh_fixed)
+      {
+        for(const halfedge_descriptor ha : vs_a)
+          put(vpm_A, target(ha, tm_A), get(vpm_B, vb));
+      }
+      else
+      {
+        // Pick a point that is on the segment [va; vb], with a ratio based on the respective tolerances
+        const FT tol_s = uv_a->second;
+        const FT tol_t = uv_b->second;
+        CGAL_assertion(tol_s != FT(0) || tol_t != FT(0));
+
+        const FT lambda = tol_t / (tol_s + tol_t);
+        const Point new_p = get(vpm_A, va) + lambda * (get(vpm_B, vb) - get(vpm_A, va));
+#ifdef CGAL_PMP_SNAP_DEBUG_PP
+        std::cout << "new position of " << va << " " << vb << " --> " << new_p << std::endl;
+#endif
+
+        for(const halfedge_descriptor ha : vs_a)
+          put(vpm_A, target(ha, tm_A), new_p);
+
+        for(const halfedge_descriptor hb : vs_b)
+          put(vpm_B, target(hb, tm_B), new_p);
+      }
+
+      ++counter;
+    }
+  }
+
+#ifdef CGAL_PMP_SNAP_DEBUG_OUTPUT
+  out_edges.close();
+#endif
+
+#ifdef CGAL_PMP_SNAP_DEBUG
+  std::cout << "Snapped " << counter << " pair(s)!" << std::endl;
+#endif
+
+  //////////////////////////////////////////////////////////////////////////////////////////////////
+  /// Done snapping; start analyzing
+  //////////////////////////////////////////////////////////////////////////////////////////////////
+
+  // Below is used in non-conformal snapping
+  // @todo could avoid doing it if not required
+  //
+  // Now that vertex-vertex snapping has been performed, look around to see if we can already
+  // lock some vertices and halfedges...
+  //
+  // #1 : If a pair of edges (1 incident to va, the other to vb) are already fully matching,
+  //      we don't want to project anything more onto any of those 2 edges
+  // #2 : If pairs on either side of the two matching vertices are compatible (not necessary fully matching),
+  //      then the two vertices should be locked as no better match can be obtained
+  // #3 : If a pair of incident edges are not fully matching, but still have compatible directions
+  //      (i.e. collinear and opposite directions), then we don't want to project onto the shorter
+  //      of the two
+  for(const Unique_vertex_pair& uvp : snappable_vertices_pairs)
+  {
+    Unique_vertex_ptr uv_a = uvp.first;
+    Unique_vertex_ptr uv_b = uvp.second;
+    const Vertex_container& vs_a = uv_a->first;
+    const Vertex_container& vs_b = uv_b->first;
+
+    // Quadratic, but all halfedges in vs_a and vs_b point to the same point. There shouldn't be many.
+    // @fixme this assumes compatible orientation...
+    for(const halfedge_descriptor ha : vs_a)
+    {
+      for(const halfedge_descriptor hb : vs_b)
+      {
+        if(!is_border(ha, tm_A) || !is_border(hb, tm_B))
+          continue;
+
+        const vertex_descriptor va = target(ha, tm_A);
+        const vertex_descriptor vb = target(hb, tm_B);
+
+        // The two folloing halfedges might not be in the range to snap, but it doesn't matter
+        const halfedge_descriptor nha = next(ha, tm_A);
+        const halfedge_descriptor nhb = next(hb, tm_B);
+        const bool is_stitchable_left = gt.equal_3_object()(get(vpm_A, source(ha, tm_A)),
+                                                            get(vpm_B, target(nhb, tm_B)));
+        const bool is_stitchable_right = gt.equal_3_object()(get(vpm_A, target(nha, tm_A)),
+                                                             get(vpm_B, source(hb, tm_B)));
+        bool is_collinear_left = false;
+        bool is_collinear_right = false;
+
+        if(is_stitchable_left)
+        {
+          *lockable_ha_out++ = ha; // #1
+          *lockable_hb_out++ = nhb; // #1
+          is_collinear_left = true;
+        }
+        else
+        {
+          const vertex_descriptor lva = source(ha, tm_A);
+          const vertex_descriptor lvb = target(nhb, tm_B);
+
+          is_collinear_left = is_collinear_with_tolerance(get(vpm_A, va), get(vpm_A, lva), get(vpm_B, lvb), gt);
+          if(is_collinear_left) // but not stitchable
+          {
+            if(gt.less_distance_to_point_3_object()(get(vpm_A, va), get(vpm_A, lva), get(vpm_B, lvb)))
+              *lockable_ha_out++ = ha; // #2
+            else
+              *lockable_hb_out++ = nhb; // #2
+          }
+        }
+
+        if(is_stitchable_right)
+        {
+          *lockable_ha_out++ = nha; // #1
+          *lockable_hb_out++ = hb; // #1
+          is_collinear_right = true;
+        }
+        else
+        {
+          const vertex_descriptor rva = target(nha, tm_A);
+          const vertex_descriptor rvb = source(hb, tm_B);
+
+          is_collinear_right = is_collinear_with_tolerance(get(vpm_A, va), get(vpm_A, rva), get(vpm_B, rvb), gt);
+          if(is_collinear_right) // but not stitchable
+          {
+            if(gt.less_distance_to_point_3_object()(get(vpm_A, va), get(vpm_A, rva), get(vpm_B, rvb)))
+              *lockable_ha_out++ = nha; // #2
+            else
+              *lockable_hb_out++ = hb; // #2
+          }
+        }
+
+        if(is_collinear_left && is_collinear_right) // #3
+        {
+#ifdef CGAL_PMP_SNAP_DEBUG
+          std::cout << va << " (" << get(vpm_A, va) << ") and " << vb << " (" << get(vpm_B, vb) << ") are locked vertices" << std::endl;
+#endif
+          *lockable_vps_out++ = std::make_pair(va, vb);
+        }
+      }
+    }
+  }
+
+  return counter;
+}
+
+// Convenience overload for snap_borders
+template <typename ConcurrencyTag = CGAL::Sequential_tag,
+          typename HalfedgeRange_A, typename HalfedgeRange_B, typename PolygonMesh,
+          typename ToleranceMap_A, typename ToleranceMap_B,
+          typename LockableVerticesOutputIterator, typename LockableHalfedgesOutputIterator,
+          typename NamedParameters_A, typename NamedParameters_B>
+std::size_t snap_vertices_two_way(const HalfedgeRange_A& halfedge_range_A,
+                                  PolygonMesh& tm_A,
+                                  ToleranceMap_A tolerance_map_A,
+                                  const HalfedgeRange_B& halfedge_range_B,
+                                  PolygonMesh& tm_B,
+                                  ToleranceMap_B tolerance_map_B,
+                                  LockableVerticesOutputIterator lockable_vps_out,
+                                  LockableHalfedgesOutputIterator lockable_ha_out,
+                                  LockableHalfedgesOutputIterator lockable_hb_out,
+                                  const NamedParameters_A& np_A,
+                                  const NamedParameters_B& np_B)
+{
+  typedef typename boost::graph_traits<PolygonMesh>::vertex_descriptor             vertex_descriptor;
+
+  using CGAL::parameters::choose_parameter;
+  using CGAL::parameters::get_parameter;
+
+  return snap_vertices_two_way(halfedge_range_A, tm_A, tolerance_map_A,
+                               choose_parameter(get_parameter(np_A, internal_np::vertex_incident_patches),
+                                                Constant_property_map<vertex_descriptor, std::size_t>(-1)),
+                               halfedge_range_B, tm_B, tolerance_map_B,
+                               choose_parameter(get_parameter(np_B, internal_np::vertex_incident_patches),
+                                                Constant_property_map<vertex_descriptor, std::size_t>(-1)),
+                               lockable_vps_out, lockable_ha_out, lockable_hb_out,
+                               choose_parameter(get_parameter(np_B, internal_np::do_lock_mesh), false),
+                               np_A, np_B);
+}
+
+} // namespace internal
+
+namespace experimental {
+
+// \ingroup PMP_repairing_grp
+//
+// Attempts to snap the vertices in `halfedge_range_A` and `halfedge_range_B`.
+// A vertex from the first range and a vertex from the second range are only snapped
+// if the distance between both vertices is smaller than the sum of their user-prescribed tolerance.
+// All such pairs are collected and processed in a greedy order: the two vertices are moved
+// to a common vertex (more on this below), and both vertices are locked and cannot be used
+// in any other matches.
+//
+// \warning This function does not give any guarantee on the conformity between the two meshes after the snapping.
+// \warning This function does not merge vertices or the meshes, it is purely geometric.
+//
+// \tparam PolygonMesh a model of `FaceListGraph` and `MutableFaceGraph`
+// \tparam HalfedgeRange_A a model of `Range` with value type `boost::graph_traits<PolygonMesh>::%halfedge_descriptor`
+// \tparam HalfedgeRange_B a model of `Range` with value type `boost::graph_traits<PolygonMesh>::%halfedge_descriptor`
+// \tparam ToleranceMap_A a model of `ReadablePropertyMap` with key type `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+//                        and value type `GetGeomTraits<PolygonMesh, NamedParameters_A>::type::FT`
+// \tparam ToleranceMap_B a model of `ReadablePropertyMap` with key type `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+//                        and value type `GetGeomTraits<PolygonMesh, NamedParameters_A>::type::FT`
+// \tparam NamedParameters_A a sequence of \ref bgl_namedparameters "Named Parameters"
+// \tparam NamedParameters_B a sequence of \ref bgl_namedparameters "Named Parameters"
+//
+// \param halfedge_range_A a range of halfedges of the first mesh defining a set of vertices (as targets of the halfeges)
+// \param tm_A the first mesh to which the vertices in `halfedge_range_A` belong
+// \param tolerance_map_A a tolerance map associating to each vertex of the first range a tolerance value
+// \param halfedge_range_B a range of vertices of the second mesh defining a set of vertices (as targets of the halfeges)
+// \param tolerance_map_B a tolerance map associating to each vertex of the second range a tolerance value
+// \param tm_B the target mesh to which the vertices in `halfedge_range_B` belong
+// \param np_A an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+//
+// \cgalNamedParamsBegin
+//   \cgalParamBegin{vertex_point_map}
+//     the property map with the points associated to the vertices of the source mesh.
+//     The type of this map is model of `ReadWritePropertyMap`. If this parameter is omitted,
+//     an internal property map for `CGAL::vertex_point_t` must be available in `PolygonMesh`.
+//   \cgalParamEnd
+//   \cgalParamBegin{geom_traits}
+//     a geometric traits class instance, must be a model of `Kernel`
+//   \cgalParamEnd
+// \cgalNamedParamsEnd
+//
+// \param np_B an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+//
+// \cgalNamedParamsBegin
+//   \cgalParamBegin{vertex_point_map}
+//     the property map with the points associated to the vertices of the target mesh.
+//     The type of this map is model of `ReadablePropertyMap`. If this parameter is omitted,
+//     an internal property map for `CGAL::vertex_point_t` must be available in `PolygonMesh`.
+//   \cgalParamEnd
+// \cgalNamedParamsEnd
+//
+// \return the number of snapped vertex pairs
+//
+template <typename HalfedgeRange_A, typename HalfedgeRange_B, typename PolygonMesh,
+          typename ToleranceMap_A, typename ToleranceMap_B,
+          typename NamedParameters_A, typename NamedParameters_B>
+std::size_t snap_vertices(const HalfedgeRange_A& halfedge_range_A,
+                          PolygonMesh& tm_A,
+                          ToleranceMap_A tolerance_map_A,
+                          const HalfedgeRange_B& halfedge_range_B,
+                          PolygonMesh& tm_B,
+                          ToleranceMap_B tolerance_map_B,
+                          const NamedParameters_A& np_A,
+                          const NamedParameters_B& np_B)
+{
+  CGAL::Emptyset_iterator unused_output_iterator;
+
+  return internal::snap_vertices_two_way(halfedge_range_A, tm_A, tolerance_map_A,
+                                         halfedge_range_B, tm_B, tolerance_map_B,
+                                         unused_output_iterator, unused_output_iterator,
+                                         unused_output_iterator, np_A, np_B);
+}
+
+template <typename HalfedgeRange_A, typename HalfedgeRange_B, typename PolygonMesh,
+          typename ToleranceMap_A, typename ToleranceMap_B>
+std::size_t snap_vertices(const HalfedgeRange_A& halfedge_range_A,
+                          PolygonMesh& tm_A,
+                          ToleranceMap_A tolerance_map_A,
+                          const HalfedgeRange_B& halfedge_range_B,
+                          PolygonMesh& tm_B,
+                          ToleranceMap_B tolerance_map_B)
+{
+  return snap_vertices(halfedge_range_A, tm_A, tolerance_map_A, halfedge_range_B, tm_B, tolerance_map_B,
+                       CGAL::parameters::all_default(), CGAL::parameters::all_default());
+}
+
+template <typename HalfedgeRange_A, typename HalfedgeRange_B, typename PolygonMesh,
+          typename T_A, typename Tag_A, typename Base_A,
+          typename T_B, typename Tag_B, typename Base_B>
+std::size_t snap_vertices(const HalfedgeRange_A& halfedge_range_A,
+                          PolygonMesh& tm_A,
+                          const HalfedgeRange_B& halfedge_range_B,
+                          PolygonMesh& tm_B,
+                          const CGAL::Named_function_parameters<T_A, Tag_A, Base_A>& np_A,
+                          const CGAL::Named_function_parameters<T_B, Tag_B, Base_B>& np_B)
+{
+  typedef CGAL::Named_function_parameters<T_A, Tag_A, Base_A>                         NamedParameters_A;
+  typedef typename GetGeomTraits<PolygonMesh, NamedParameters_A>::type                GT;
+  typedef typename GT::FT                                                             FT;
+  typedef CGAL::dynamic_vertex_property_t<FT>                                         Vertex_property_tag;
+  typedef typename boost::property_map<PolygonMesh, Vertex_property_tag>::type        Tolerance_map;
+
+  const FT max_tol((std::numeric_limits<double>::max)());
+
+  Tolerance_map tolerance_map_A = get(Vertex_property_tag(), tm_A);
+  internal::assign_tolerance_with_local_edge_length_bound(halfedge_range_A, tolerance_map_A, max_tol, tm_A, np_A);
+  Tolerance_map tolerance_map_B = get(Vertex_property_tag(), tm_B);
+  internal::assign_tolerance_with_local_edge_length_bound(halfedge_range_B, tolerance_map_B, max_tol, tm_B, np_B);
+
+  return snap_vertices(halfedge_range_A, tm_A, tolerance_map_A, halfedge_range_B, tm_B, tolerance_map_B, np_A, np_B);
+}
+
+template <typename HalfedgeRange_A, typename HalfedgeRange_B, typename PolygonMesh>
+std::size_t snap_vertices(const HalfedgeRange_A& halfedge_range_A,
+                          PolygonMesh& tm_A,
+                          const HalfedgeRange_B& halfedge_range_B,
+                          PolygonMesh& tm_B)
+{
+  return snap_vertices(halfedge_range_A, tm_A, halfedge_range_B, tm_B,
+                       parameters::all_default(), parameters::all_default());
+}
+
+///////////////////////////////////////////////////////////////////////////////////////////////////
+/// Border convenience overloads
+///////////////////////////////////////////////////////////////////////////////////////////////////
+
+template <typename PolygonMesh, typename ToleranceMap_A, typename ToleranceMap_B>
+std::size_t snap_border_vertices(PolygonMesh& tm_A,
+                                 ToleranceMap_A tolerance_map_A,
+                                 PolygonMesh& tm_B,
+                                 ToleranceMap_B tolerance_map_B)
+{
+  typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor    halfedge_descriptor;
+
+  std::vector<halfedge_descriptor> border_A;
+  border_halfedges(tm_A, std::back_inserter(border_A));
+  std::vector<halfedge_descriptor> border_B;
+  border_halfedges(tm_B, std::back_inserter(border_B));
+
+  return snap_vertices(border_A, tm_A, tolerance_map_A,
+                       border_B, tm_B, tolerance_map_B);
+}
+
+template <typename PolygonMesh>
+std::size_t snap_border_vertices(PolygonMesh& tm_A,
+                                 PolygonMesh& tm_B)
+{
+  typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor    halfedge_descriptor;
+
+  std::vector<halfedge_descriptor> border_vertices_A;
+  border_halfedges(tm_A, std::back_inserter(border_vertices_A));
+
+  std::vector<halfedge_descriptor> border_vertices_B;
+  border_halfedges(tm_B, std::back_inserter(border_vertices_B));
+
+  return snap_vertices(border_vertices_A, tm_A, border_vertices_B, tm_B);
+}
+
+template <typename PolygonMesh, typename ToleranceMap>
+std::size_t snap_border_vertices(PolygonMesh& tm, ToleranceMap tolerance_map)
+{
+  return snap_border_vertices(tm, tolerance_map, tm, tolerance_map);
+}
+
+template <typename PolygonMesh>
+std::size_t snap_border_vertices(PolygonMesh& tm)
+{
+  return snap_border_vertices(tm, tm);
+}
+
+} // namespace experimental
+} // namespace Polygon_mesh_processing
+} // namespace CGAL
+
+#endif // CGAL_POLYGON_MESH_PROCESSING_SNAPPING_SNAP_VERTICES_H
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/intersection.h 5.1-1/include/CGAL/Polygon_mesh_processing/intersection.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/intersection.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/intersection.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/intersection.h $
-// $Id: intersection.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/intersection.h $
+// $Id: intersection.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -17,20 +17,27 @@
 
 #include <CGAL/disable_warnings.h>
 
-#include <CGAL/Polygon_mesh_processing/internal/Corefinement/intersection_impl.h>
-#include <boost/type_traits/is_same.hpp>
-#include <boost/utility/enable_if.hpp>
-#include <CGAL/Polygon_mesh_processing/bbox.h>
-#include <CGAL/boost/iterator/counting_iterator.hpp>
-#include <boost/mpl/if.hpp>
-#include <CGAL/Polygon_mesh_processing/bbox.h>
-#include <CGAL/Polygon_mesh_processing/internal/named_params_helper.h>
-#include <CGAL/Polygon_mesh_processing/connected_components.h>
 #include <CGAL/AABB_face_graph_triangle_primitive.h>
 #include <CGAL/AABB_traits.h>
 #include <CGAL/AABB_tree.h>
+#include <CGAL/boost/iterator/counting_iterator.hpp>
+#include <CGAL/box_intersection_d.h>
+#include <CGAL/Polygon_mesh_processing/internal/Corefinement/intersection_impl.h>
+#include <CGAL/Polygon_mesh_processing/internal/named_params_helper.h>
+#include <CGAL/Polygon_mesh_processing/bbox.h>
+#include <CGAL/Polygon_mesh_processing/connected_components.h>
 #include <CGAL/Side_of_triangle_mesh.h>
 
+#include <boost/function_output_iterator.hpp>
+#include <boost/type_traits/is_same.hpp>
+#include <boost/utility/enable_if.hpp>
+#include <boost/mpl/if.hpp>
+
+#include <exception>
+#include <iterator>
+#include <utility>
+#include <vector>
+
 namespace CGAL {
 namespace Polygon_mesh_processing{
 namespace internal {
@@ -43,7 +50,6 @@ template<class TM,
          class VertexPointMap2>
 struct Intersect_faces
 {
-
   // typedefs
   typedef typename Kernel::Segment_3    Segment;
   typedef typename Kernel::Triangle_3   Triangle;
@@ -61,10 +67,6 @@ struct Intersect_faces
   typename Kernel::Construct_triangle_3 triangle_functor;
   typename Kernel::Do_intersect_3       do_intersect_3_functor;
 
-
-
-
-
   Intersect_faces(const TM& tm1, const TM& tm2,
                    OutputIterator it,
                    VertexPointMap1 vpmap1, VertexPointMap2 vpmap2,
@@ -130,9 +132,6 @@ struct Intersect_face_polyline
   typename Kernel::Construct_triangle_3 triangle_functor;
   typename Kernel::Do_intersect_3       do_intersect_3_functor;
 
-
-
-
   Intersect_face_polyline(const TM& tm,
                            const std::vector<face_descriptor>& faces,
                            const Polyline& polyline,
@@ -186,7 +185,6 @@ struct Intersect_face_polylines
   typedef typename boost::property_map<TM, boost::vertex_point_t>::const_type Ppmap;
   typedef typename boost::property_traits<Ppmap>::value_type Point;
 
-
   // members
   const TM& m_tm;
   const std::vector<face_descriptor>& faces;
@@ -198,9 +196,6 @@ struct Intersect_face_polylines
   typename Kernel::Construct_triangle_3 triangle_functor;
   typename Kernel::Do_intersect_3       do_intersect_3_functor;
 
-
-
-
   Intersect_face_polylines(const TM& tm,
                            const std::vector<face_descriptor>& faces,
                            const PolylineRange& polylines,
@@ -242,8 +237,6 @@ template<class Polyline,
          class OutputIterator>
 struct Intersect_polylines
 {
-
-
   // typedefs
   typedef typename Kernel::Segment_3    Segment;
   typedef typename Kernel::Point_3 Point;
@@ -257,9 +250,6 @@ struct Intersect_polylines
   typename Kernel::Construct_segment_3  segment_functor;
   typename Kernel::Do_intersect_3       do_intersect_3_functor;
 
-
-
-
   Intersect_polylines(const Polyline& polyline1,
                       const Polyline& polyline2,
                       OutputIterator it,
@@ -292,8 +282,6 @@ template<class PolylineRange,
          class OutputIterator>
 struct Intersect_polyline_ranges
 {
-
-
   // typedefs
   typedef typename Kernel::Segment_3    Segment;
   typedef typename Kernel::Point_3 Point;
@@ -307,9 +295,6 @@ struct Intersect_polyline_ranges
   typename Kernel::Construct_segment_3  segment_functor;
   typename Kernel::Do_intersect_3       do_intersect_3_functor;
 
-
-
-
   Intersect_polyline_ranges(const PolylineRange& polyline1,
                             const PolylineRange& polyline2,
                             OutputIterator it,
@@ -360,7 +345,7 @@ struct Throw_at_first_output {
  *  model of `RandomAccessRange`.
  * \tparam OutputIterator a model of `OutputIterator` holding objects of type
  *   `std::pair<boost::graph_traits<TriangleMesh>::%face_descriptor, boost::graph_traits<TriangleMesh>::%face_descriptor>`
- * \tparam NamedParameters a sequence of \ref pmp_namedparameters
+ * \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
  *
  * \param face_range1 the range of faces of `tm1` to check for intersections.
  * \param face_range2 the range of faces of `tm2` to check for intersections.
@@ -368,15 +353,29 @@ struct Throw_at_first_output {
  * \param tm2 the second triangulated surface mesh.
  * \param out output iterator to be filled with all pairs of faces that intersect.
  *  First and second element in the pairs correspond to faces of `tm1` and `tm2` respectively
- * \param np1 optional sequence of \ref pmp_namedparameters for `tm1`, among the ones listed below
- * \param np2 optional sequence of \ref pmp_namedparameters for `tm2`, among the ones listed below
+ * \param np1 an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+ * \param np2 an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
- *   If this parameter is omitted, an internal property map for
- *   `CGAL::vertex_point_t` must be available in `TriangleMesh`\cgalParamEnd
- *    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `PMPSelfIntersectionTraits` \cgalParamEnd
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `tm1` (`tm2`)}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+ *                    as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm1 (tm2))`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     should be available for the vertices of `tm1` (`tm2`)}
+ *     \cgalParamExtra{Both vertex point maps must have the same value type}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{geom_traits}
+ *     \cgalParamDescription{an instance of a geometric traits class}
+ *     \cgalParamType{a class model of `PMPSelfIntersectionTraits`}
+ *     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+ *     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+ *     \cgalParamExtra{np1 only}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
+ *
  * \return `out`
  */
 template <class TriangleMesh,
@@ -401,7 +400,9 @@ compute_face_face_intersection(const Fac
 
   typedef TriangleMesh TM;
   typedef typename boost::graph_traits<TM>::face_descriptor face_descriptor;
-  typedef typename CGAL::Box_intersection_d::Box_with_info_d<double, 3, face_descriptor> Box;
+
+  typedef CGAL::Box_intersection_d::ID_FROM_BOX_ADDRESS Box_policy;
+  typedef CGAL::Box_intersection_d::Box_with_info_d<double, 3, face_descriptor, Box_policy> Box;
 
   CGAL::Bbox_3 b1 = CGAL::Polygon_mesh_processing::bbox(tm1, np1),
                b2 = CGAL::Polygon_mesh_processing::bbox(tm2, np2);
@@ -414,12 +415,8 @@ compute_face_face_intersection(const Fac
   // make one box per facet
   std::vector<Box> boxes1;
   std::vector<Box> boxes2;
-  boxes1.reserve(
-        std::distance( boost::begin(face_range1), boost::end(face_range1) )
-        );
-  boxes2.reserve(
-        std::distance( boost::begin(face_range2), boost::end(face_range2) )
-        );
+  boxes1.reserve(std::distance(boost::begin(face_range1), boost::end(face_range1)));
+  boxes2.reserve(std::distance(boost::begin(face_range2), boost::end(face_range2)));
 
   typedef typename GetVertexPointMap<TM, NamedParameters1>::const_type VertexPointMap1;
   typedef typename GetVertexPointMap<TM, NamedParameters2>::const_type VertexPointMap2;
@@ -433,6 +430,7 @@ compute_face_face_intersection(const Fac
        typename boost::property_traits<VertexPointMap1>::value_type,
        typename boost::property_traits<VertexPointMap2>::value_type
        >::value) );
+
   for(face_descriptor f : face_range1)
   {
     boxes1.push_back(Box(Polygon_mesh_processing::face_bbox(f, tm1), f));
@@ -449,20 +447,16 @@ compute_face_face_intersection(const Fac
   std::vector<const Box*> box2_ptr(boost::make_counting_iterator<const Box*>(&boxes2[0]),
                                    boost::make_counting_iterator<const Box*>(&boxes2[0]+boxes2.size()));
 
-
   // compute intersections filtered out by boxes
   typedef typename GetGeomTraits<TM, NamedParameters1>::type GeomTraits;
-  GeomTraits gt = choose_parameter(get_parameter(np1, internal_np::geom_traits), GeomTraits());
+  GeomTraits gt = choose_parameter<GeomTraits>(get_parameter(np1, internal_np::geom_traits));
 
   internal::Intersect_faces<TM,
                             GeomTraits,
                             Box,
                             OutputIterator,
                             VertexPointMap1,
-                            VertexPointMap2> Intersect_faces(tm1, tm2,
-                                                             out,
-                                                             vpmap1, vpmap2,
-                                                             gt);
+                            VertexPointMap2> Intersect_faces(tm1, tm2, out, vpmap1, vpmap2, gt);
 
   std::ptrdiff_t cutoff = 2000;
   CGAL::box_intersection_d(box1_ptr.begin(), box1_ptr.end(),
@@ -491,20 +485,31 @@ compute_face_face_intersection(const Fac
  *  elements in their respective range. In the case of the polyline, this position is the index of
  * the segment intersecting the face (which is the index  of the first point of the
  * segment following the range order.)
- * \tparam NamedParameters a sequence of \ref pmp_namedparameters
+ * \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
  *
  * \param face_range the range of faces of `tm` to check for intersections.
  * \param polyline the polyline to check for intersections.
  * \param tm the triangulated surface mesh to check for intersections.
  * \param out output iterator to be filled with all pairs of face-segment that intersect
- * \param np optional sequence of \ref pmp_namedparameters for `tm`, among the ones listed below
+ * \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `tm`.
- *   If this parameter is omitted, an internal property map for
- *   `CGAL::vertex_point_t` must be available in `TriangleMesh`\cgalParamEnd
- *    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `PMPSelfIntersectionTraits` \cgalParamEnd
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `tm`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor` as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     should be available for the vertices of `tm`.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{geom_traits}
+ *     \cgalParamDescription{an instance of a geometric traits class}
+ *     \cgalParamType{a class model of `PMPSelfIntersectionTraits`}
+ *     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+ *     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
+ *
  * \return `out`
  */
 template <class TriangleMesh,
@@ -541,21 +546,16 @@ compute_face_polyline_intersection( cons
         typename boost::range_value<Polyline>::type>::value));
 
   std::vector<face_descriptor> faces;
-  faces.reserve(std::distance( boost::begin(face_range), boost::end(face_range) ));
+  faces.reserve(std::distance(boost::begin(face_range), boost::end(face_range)));
 
-  typedef typename CGAL::Box_intersection_d::Box_with_info_d<double, 3, std::size_t> Box;
+  typedef CGAL::Box_intersection_d::ID_FROM_BOX_ADDRESS Box_policy;
+  typedef CGAL::Box_intersection_d::Box_with_info_d<double, 3, std::size_t, Box_policy> Box;
 
   // make one box per facet
   std::vector<Box> boxes1;
   std::vector<Box> boxes2;
-  boxes1.reserve(
-        std::distance( boost::begin(face_range), boost::end(face_range) )
-        );
-
-  boxes2.reserve(
-        std::distance( boost::begin(polyline), boost::end(polyline) ) - 1
-        );
-
+  boxes1.reserve(std::distance(boost::begin(face_range), boost::end(face_range)));
+  boxes2.reserve(std::distance(boost::begin(polyline), boost::end(polyline)) - 1);
 
   for(face_descriptor f : face_range)
   {
@@ -571,7 +571,6 @@ compute_face_polyline_intersection( cons
   }
 
   // generate box pointers
-
   std::vector<const Box*> box1_ptr(boost::make_counting_iterator<const Box*>(&boxes1[0]),
                                    boost::make_counting_iterator<const Box*>(&boxes1[0]+boxes1.size()));
   std::vector<const Box*> box2_ptr(boost::make_counting_iterator<const Box*>(&boxes2[0]),
@@ -579,7 +578,7 @@ compute_face_polyline_intersection( cons
 
   // compute intersections filtered out by boxes
   typedef typename GetGeomTraits<TM, NamedParameters>::type GeomTraits;
-  GeomTraits gt = choose_parameter(get_parameter(np, internal_np::geom_traits), GeomTraits());
+  GeomTraits gt = choose_parameter<GeomTraits>(get_parameter(np, internal_np::geom_traits));
 
   internal::Intersect_face_polyline<TM,
                                     GeomTraits,
@@ -587,12 +586,7 @@ compute_face_polyline_intersection( cons
                                     OutputIterator,
                                     Polyline,
                                     VertexPointMap>
-                                    Intersect_face_polyline(tm,
-                                                            faces,
-                                                            polyline,
-                                                            out,
-                                                            vpmap,
-                                                            gt);
+                                    Intersect_face_polyline(tm, faces, polyline, out, vpmap, gt);
 
   std::ptrdiff_t cutoff = 2000;
   CGAL::box_intersection_d(box1_ptr.begin(), box1_ptr.end(),
@@ -620,19 +614,30 @@ compute_face_polyline_intersection( cons
  *   `std::pair<std::size_t, std::pair<std::size_t, std::size_t> >`.
  * Each pair holds the index of the face and a pair containing the index of the polyline in the range and the index of
  * the first point of the segment in the polyline.
- * \tparam NamedParameters a sequence of \ref pmp_namedparameters
+ * \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
  *
  * \param face_range the range of `tm` faces to check for intersections.
  * \param polyline_range the range of polylines to check for intersections.
  * \param tm the triangulated surface mesh to check for intersections.
  * \param out output iterator to be filled with all pairs of face-segment that intersect
- * \param np optional sequence of \ref pmp_namedparameters for `tm`, among the ones listed below
+ * \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `tm`.
- *   If this parameter is omitted, an internal property map for
- *   `CGAL::vertex_point_t` must be available in `TriangleMesh`\cgalParamEnd
- *    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `PMPSelfIntersectionTraits` \cgalParamEnd
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `tm`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+ *                    as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     should be available for the vertices of `tm`.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{geom_traits}
+ *     \cgalParamDescription{an instance of a geometric traits class}
+ *     \cgalParamType{a class model of `PMPSelfIntersectionTraits`}
+ *     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+ *     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  * \return `out`
  */
@@ -672,34 +677,32 @@ compute_face_polylines_intersection(cons
                                           get_const_property_map(boost::vertex_point, tm));
   typedef typename boost::property_traits<VertexPointMap>::value_type Point;
   typedef typename boost::range_value<PolylineRange>::type Polyline;
-  CGAL_static_assertion(
-        (boost::is_same<Point,
-        typename boost::range_value<Polyline>::type>::value));
+  CGAL_static_assertion((boost::is_same<Point, typename boost::range_value<Polyline>::type>::value));
 
   std::vector<face_descriptor> faces;
   faces.reserve(std::distance( boost::begin(face_range), boost::end(face_range) ));
 
-  typedef typename CGAL::Box_intersection_d::Box_with_info_d<double, 3, std::pair<std::size_t, std::size_t> > Box;
+  typedef CGAL::Box_intersection_d::ID_FROM_BOX_ADDRESS Box_policy;
+  typedef CGAL::Box_intersection_d::Box_with_info_d<double, 3, std::pair<std::size_t, std::size_t>, Box_policy> Box;
 
   // make one box per facet
   std::vector<Box> boxes1;
   std::vector<Box> boxes2;
-  boxes1.reserve(
-        std::distance( boost::begin(face_range), boost::end(face_range) )
-        );
+  boxes1.reserve(std::distance(boost::begin(face_range), boost::end(face_range)));
 
   std::size_t polylines_size = 0;
   for(Polyline poly : polyline_range)
   {
     polylines_size += std::distance( boost::begin(poly), boost::end(poly) ) -1;
   }
-  boxes2.reserve( polylines_size );
+  boxes2.reserve(polylines_size);
 
   for(face_descriptor f : face_range)
   {
     faces.push_back(f);
     boxes1.push_back(Box(Polygon_mesh_processing::face_bbox(f, tm), std::make_pair(0, faces.size()-1)));
   }
+
   std::size_t range_size = std::distance( boost::begin(polyline_range), boost::end(polyline_range) );
   for(std::size_t j = 0; j < range_size; ++j)
   {
@@ -722,7 +725,7 @@ compute_face_polylines_intersection(cons
 
   // compute intersections filtered out by boxes
   typedef typename GetGeomTraits<TM, NamedParameters>::type GeomTraits;
-  GeomTraits gt = choose_parameter(get_parameter(np, internal_np::geom_traits), GeomTraits());
+  GeomTraits gt = choose_parameter<GeomTraits>(get_parameter(np, internal_np::geom_traits));
 
   internal::Intersect_face_polylines<TM,
                                      GeomTraits,
@@ -730,12 +733,7 @@ compute_face_polylines_intersection(cons
                                      PolylineRange,
                                      OutputIterator,
                                      VertexPointMap>
-                                     Intersect_face_polyline(tm,
-                                                             faces,
-                                                             polyline_range,
-                                                             out,
-                                                             vpmap,
-                                                             gt);
+                                     Intersect_face_polyline(tm, faces, polyline_range, out, vpmap, gt);
 
   std::ptrdiff_t cutoff = 2000;
   CGAL::box_intersection_d(box1_ptr.begin(), box1_ptr.end(),
@@ -773,18 +771,15 @@ compute_polyline_polyline_intersection(c
                                        OutputIterator out,
                                        const Kernel& K)
 {
-  typedef typename CGAL::Box_intersection_d::Box_with_info_d<double, 3, std::size_t> Box;
+  typedef CGAL::Box_intersection_d::ID_FROM_BOX_ADDRESS Box_policy;
+  typedef CGAL::Box_intersection_d::Box_with_info_d<double, 3, std::size_t, Box_policy> Box;
+
   typedef typename Kernel::Point_3 Point;
   // make one box per facet
   std::vector<Box> boxes1;
   std::vector<Box> boxes2;
-  boxes1.reserve(
-        std::distance( boost::begin(polyline1), boost::end(polyline1) ) - 1
-        );
-
-  boxes2.reserve(
-        std::distance( boost::begin(polyline2), boost::end(polyline2) ) - 1
-        );
+  boxes1.reserve(std::distance(boost::begin(polyline1), boost::end(polyline1)) - 1);
+  boxes2.reserve(std::distance(boost::begin(polyline2), boost::end(polyline2)) - 1);
 
   for(std::size_t i =0; i< polyline1.size()-1; ++i)
   {
@@ -813,10 +808,7 @@ compute_polyline_polyline_intersection(c
                                 Kernel,
                                 Box,
                                 OutputIterator>
-                                intersect_polylines(polyline1,
-                                                    polyline2,
-                                                    out,
-                                                    K);
+                                intersect_polylines(polyline1, polyline2, out, K);
 
   std::ptrdiff_t cutoff = 2000;
   CGAL::box_intersection_d(box1_ptr.begin(), box1_ptr.end(),
@@ -856,7 +848,9 @@ compute_polylines_polylines_intersection
                                          const Kernel& K)
 {
   //info.first is the index of the polyline in the range, info.second is the index of the point in the polyline
-  typedef typename CGAL::Box_intersection_d::Box_with_info_d<double, 3, std::pair<std::size_t, std::size_t> > Box;
+  typedef CGAL::Box_intersection_d::ID_FROM_BOX_ADDRESS Box_policy;
+  typedef CGAL::Box_intersection_d::Box_with_info_d<double, 3, std::pair<std::size_t, std::size_t>, Box_policy> Box;
+
   typedef typename Kernel::Point_3 Point;
   typedef typename boost::range_value<PolylineRange>::type Polyline;
 
@@ -871,6 +865,7 @@ compute_polylines_polylines_intersection
     b1 += CGAL::bbox_3(poly.begin(), poly.end());
   }
   boxes1.reserve( polylines_size );
+
   polylines_size = 0;
   for(Polyline poly : polylines2)
   {
@@ -881,6 +876,7 @@ compute_polylines_polylines_intersection
 
   if(!CGAL::do_overlap(b1,b2))
     return out;
+
   std::size_t range_size = std::distance( boost::begin(polylines1), boost::end(polylines1) );
   for(std::size_t j = 0; j < range_size; ++j)
   {
@@ -915,15 +911,11 @@ compute_polylines_polylines_intersection
 
 
   // compute intersections filtered out by boxes
-
   internal::Intersect_polyline_ranges<PolylineRange,
                                       Kernel,
                                       Box,
                                       OutputIterator>
-                                      intersect_polylines(polylines1,
-                                                          polylines2,
-                                                          out,
-                                                          K);
+                                      intersect_polylines(polylines1, polylines2, out, K);
 
   std::ptrdiff_t cutoff = 2000;
   CGAL::box_intersection_d(box1_ptr.begin(), box1_ptr.end(),
@@ -943,20 +935,33 @@ compute_polylines_polylines_intersection
  * \tparam TriangleMesh a model of `FaceListGraph`
  * \tparam OutputIterator a model of `OutputIterator` holding objects of type
  *   `std::pair<boost::graph_traits<TriangleMesh>::%face_descriptor, boost::graph_traits<TriangleMesh>::%face_descriptor>`
- * \tparam NamedParameters a sequence of \ref pmp_namedparameters
+ * \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
  *
  * \param tm1 the first triangulated surface mesh to check for intersections
  * \param tm2 the second triangulated surface mesh to check for intersections
  * \param out output iterator to be filled with all pairs of faces that intersect
- * \param np1 optional sequence of \ref pmp_namedparameters for `tm1`, among the ones listed below
- * \param np2 optional sequence of \ref pmp_namedparameters for `tm2`, among the ones listed below
+ * \param np1 an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+ * \param np2 an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
- *   If this parameter is omitted, an internal property map for
- *   `CGAL::vertex_point_t` must be available in `TriangleMesh`\cgalParamEnd
- *    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `PMPSelfIntersectionTraits` \cgalParamEnd
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `tm1` (`tm2`)}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+ *                    as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm1 (tm2))`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     should be available for the vertices of `tm1` (`tm2`)}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{geom_traits}
+ *     \cgalParamDescription{an instance of a geometric traits class}
+ *     \cgalParamType{a class model of `PMPSelfIntersectionTraits`}
+ *     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+ *     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+ *     \cgalParamExtra{np1 only}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
+ *
  * \return `out`
  */
 template <class TriangleMesh,
@@ -993,19 +998,30 @@ compute_face_face_intersection(const Tri
  *  elements in their respective range. In the case of the polyline, this position is the index
  * of the segment that holds the intersection, so it is the index of the first point of the
  * segment following the range order.
- * \tparam NamedParameters a sequence of \ref pmp_namedparameters
+ * \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
  *
  * \param tm the triangulated surface mesh to check for intersections.
  * \param polyline the polyline to check for intersections.
  * \param out output iterator to be filled with all pairs of face-segment that intersect
- * \param np optional sequence of \ref pmp_namedparameters for `tm`, among the ones listed below
+ * \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
- *   If this parameter is omitted, an internal property map for
- *   `CGAL::vertex_point_t` must be available in `TriangleMesh`\cgalParamEnd
- *    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `PMPSelfIntersectionTraits` \cgalParamEnd
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `tm`}
+ *     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor` as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     should be available for the vertices of `tm`.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{geom_traits}
+ *     \cgalParamDescription{an instance of a geometric traits class}
+ *     \cgalParamType{a class model of `PMPSelfIntersectionTraits`}
+ *     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+ *     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
+ *
  * \return `out`
  */
 template <class TriangleMesh,
@@ -1198,24 +1214,39 @@ bool do_intersect(const Polyline& polyli
  * @pre `!do_overlap_test_of_bounded_sides || CGAL::is_closed(tm2)`
  *
  * @tparam TriangleMesh a model of `FaceListGraph`
- * @tparam NamedParameters1 a sequence of \ref pmp_namedparameters for `tm1`
- * @tparam NamedParameters2 a sequence of \ref pmp_namedparameters for `tm2`
+ * @tparam NamedParameters1 a sequence of \ref bgl_namedparameters "Named Parameters" for `tm1`
+ * @tparam NamedParameters2 a sequence of \ref bgl_namedparameters "Named Parameters" for `tm2`
  *
  * @param tm1 the first triangulated surface mesh to check for intersections
  * @param tm2 the second triangulated surface mesh to check for intersections
- * @param np1 optional sequence of \ref pmp_namedparameters for `tm1`, among the ones listed below
- * @param np2 optional sequence of \ref pmp_namedparameters for `tm2`, among the ones listed below
+ * @param np1 an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+ * @param np2 an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `tm1` (`tm2`).
- *   \attention The two property maps must have the same `value_type`.
- *
- *   If this parameter is omitted, an internal property map for
- *   `CGAL::vertex_point_t` must be available in `TriangleMesh`\cgalParamEnd
- *    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `PMPSelfIntersectionTraits` \cgalParamEnd
- *    \cgalParamBegin{do_overlap_test_of_bounded_sides} if set to `true` tests also the overlap of the bounded sides of `tm1` and `tm2`.
- *                                                      If `false` (default), only the intersection of surface triangles are tested.
- *     \cgalParamEnd
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `tm1` (`tm2`)}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+ *                    as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm1 (tm2))`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     should be available for the vertices of `tm1` (`tm2`)}
+ *     \cgalParamExtra{Both vertex point maps must have the same value type}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{geom_traits}
+ *     \cgalParamDescription{an instance of a geometric traits class}
+ *     \cgalParamType{a class model of `PMPSelfIntersectionTraits`}
+ *     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+ *     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+ *     \cgalParamExtra{np1 only}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{do_overlap_test_of_bounded_sides}
+ *     \cgalParamDescription{If `true`, also tests the overlap of the bounded sides of `tm1` and `tm2`.
+ *                           If `false`, only the intersection of surface triangles is tested.}
+ *     \cgalParamType{Boolean}
+ *     \cgalParamDefault{`false`}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  *
  */
@@ -1229,6 +1260,7 @@ bool do_intersect(const TriangleMesh& tm
 {
   using parameters::choose_parameter;
   using parameters::get_parameter;
+
   bool test_overlap =  choose_parameter(get_parameter(np1, internal_np::overlap_test),false) ||
                        choose_parameter(get_parameter(np2, internal_np::overlap_test),false);
 
@@ -1254,7 +1286,7 @@ bool do_intersect(const TriangleMesh& tm
     VertexPointMap2 vpm2 = choose_parameter(get_parameter(np2, internal_np::vertex_point),
                                         get_const_property_map(boost::vertex_point, tm2));
     typedef typename GetGeomTraits<TriangleMesh, NamedParameters1>::type GeomTraits;
-    GeomTraits gt = choose_parameter(get_parameter(np1, internal_np::geom_traits), GeomTraits());
+    GeomTraits gt = choose_parameter<GeomTraits>(get_parameter(np1, internal_np::geom_traits));
 
     return internal::is_mesh2_in_mesh1(tm1, tm2, vpm1, vpm2, gt) ||
            internal::is_mesh2_in_mesh1(tm2, tm1, vpm2, vpm1, gt);
@@ -1286,17 +1318,27 @@ bool do_intersect(const TriangleMesh& tm
  * \tparam PolylineRange a `RandomAccessRange` of `RandomAccessRange` of points. The point type of the range must be the
  *  same as the value type of the vertex point map.
  * \cgalDescribePolylineType
- * @tparam NamedParameters a sequence of \ref pmp_namedparameters
+ * @tparam NamedParameters a sequence of \ref bgl_namedparameters
  *
  * @param tm the triangulated surface mesh to check for intersections
  * @param polylines the range of polylines to check for intersections.
- * @param np optional sequence of \ref pmp_namedparameters among the ones listed below
+ * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `tm`.
- *   If this parameter is omitted, an internal property map for
- *   `CGAL::vertex_point_t` must be available in `TriangleMesh`\cgalParamEnd
- *    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `PMPSelfIntersectionTraits` \cgalParamEnd
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `tm`}
+ *     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor` as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     should be available for the vertices of `tm`.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{geom_traits}
+ *     \cgalParamDescription{an instance of a geometric traits class}
+ *     \cgalParamType{a class model of `PMPSelfIntersectionTraits`}
+ *     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+ *     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  *
  */
@@ -1341,17 +1383,27 @@ bool do_intersect(const TriangleMesh& tm
  * \tparam Polyline a `RandomAccessRange` of points. The point type of the range must be the
  *  same as the value type of the vertex point map.
  * \cgalDescribePolylineType
- * @tparam NamedParameters a sequence of \ref pmp_namedparameters
+ * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
  *
  * @param tm the triangulated surface mesh to check for intersections
  * @param polyline the polyline to check for intersections.
- * @param np optional sequence of \ref pmp_namedparameters among the ones listed below
+ * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `tn`.
- *   If this parameter is omitted, an internal property map for
- *   `CGAL::vertex_point_t` must be available in `TriangleMesh`\cgalParamEnd
- *    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `PMPSelfIntersectionTraits` \cgalParamEnd
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `tm`}
+ *     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor` as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     should be available for the vertices of `tm`.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{geom_traits}
+ *     \cgalParamDescription{an instance of a geometric traits class}
+ *     \cgalParamType{a class model of `PMPSelfIntersectionTraits`}
+ *     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+ *     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  *
  */
@@ -1551,31 +1603,41 @@ struct Mesh_callback
  *
  * \tparam TriangleMeshRange a model of `RandomAccessRange` of triangulated surface meshes model of `FaceListGraph`.
  * \tparam OutputIterator an output iterator in which `std::pair<std::size_t, std::size_t>` can be put.
- * \tparam NamedParameters a sequence of \ref pmp_namedparameters for the algorithm
- * \tparam NamedParametersRange a range of named parameters for the meshes.
+ * \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters" for the algorithm
+ * \tparam NamedParametersRange a range of \ref bgl_namedparameters "Named Parameters" for the meshes.
  *
  * \param range the range of triangulated surface meshes to be checked for intersections.
  * \param out output iterator used to collect pairs of intersecting meshes.
- * \param np an optional sequence named parameters among the one listed below
+ * \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `PMPSelfIntersectionTraits`.
- *                                 The default value for `geom_traits` is `CGAL::Kernel_traits<Point>::Kernel`, where `Point` is the
- *                                 value type of the vertex point map of the meshes.
- * \cgalParamEnd
- *    \cgalParamBegin{do_overlap_test_of_bounded_sides} if set to `true` reports also overlap of bounded sides of meshes.
- *                                                      If `false` (default), only the intersection of surface triangles are tested.
- *     \cgalParamEnd
+ *   \cgalParamNBegin{geom_traits}
+ *     \cgalParamDescription{an instance of a geometric traits class}
+ *     \cgalParamType{a class model of `PMPSelfIntersectionTraits`}
+ *     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`,
+ *                       where `Point` is the value type of the vertex point map of the meshes}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{do_overlap_test_of_bounded_sides}
+ *     \cgalParamDescription{If `true`, reports also overlap of bounded sides of meshes.
+ *                           If `false`, only the intersection of surface triangles are tested.}
+ *     \cgalParamType{Boolean}
+ *     \cgalParamDefault{`false`}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
- * \param nps an optional range of `vertex_point_map` named parameters containing the `VertexPointMap` of each mesh in `range`, in the same order.
- * If this parameter is omitted, then an internal property map for `CGAL::vertex_point_t` must be available for every mesh in the range.
- * All the vertex point maps must be of the same type.
+ *
+ * \param nps an optional range of sequences of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of a mesh.
- *   If this parameter is omitted, an internal property map for
- *   `CGAL::vertex_point_t` must be available in the triangle mesh type used in the range
- *     \cgalParamEnd
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of a mesh `tm`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+ *                    as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     should be available for the vertices of `tm`.}
+ *     \cgalParamExtra{All vertex point maps must have the same value type}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  */
 
@@ -1595,7 +1657,9 @@ OutputIterator intersecting_meshes(const
 
   bool report_overlap =  choose_parameter(get_parameter(np, internal_np::overlap_test),false);
 
-  typedef CGAL::Box_intersection_d::Box_with_info_d<double, 3, TriangleMeshIterator> Mesh_box;
+  typedef CGAL::Box_intersection_d::ID_FROM_BOX_ADDRESS Box_policy;
+  typedef CGAL::Box_intersection_d::Box_with_info_d<double, 3, TriangleMeshIterator, Box_policy> Mesh_box;
+
   std::vector<Mesh_box> boxes;
   boxes.reserve(std::distance(range.begin(), range.end()));
 
@@ -1604,21 +1668,18 @@ OutputIterator intersecting_meshes(const
     boxes.push_back( Mesh_box(Polygon_mesh_processing::bbox(*it), it) );
   }
 
-  std::vector<Mesh_box*> boxes_ptr(
-        boost::make_counting_iterator(&boxes[0]),
-    boost::make_counting_iterator(&boxes[0]+boxes.size()));
+  std::vector<Mesh_box*> boxes_ptr(boost::make_counting_iterator(&boxes[0]),
+                                   boost::make_counting_iterator(&boxes[0]+boxes.size()));
 
   typedef typename boost::range_value<NamedParametersRange>::type NP_rng;
   typedef typename boost::range_value<TriangleMeshRange>::type TriangleMesh;
   typedef typename GetGeomTraits<TriangleMesh, NamedParameters, NP_rng>::type GT;
-  GT gt = choose_parameter(get_parameter(np, internal_np::geom_traits), GT());
-
+  GT gt = choose_parameter<GT>(get_parameter(np, internal_np::geom_traits));
 
   //get all the pairs of meshes intersecting (no strict inclusion test)
   std::ptrdiff_t cutoff = 2000;
   internal::Mesh_callback<TriangleMeshRange, GT, OutputIterator, NamedParametersRange> callback(range, out, report_overlap, gt, nps);
-  CGAL::box_self_intersection_d(boxes_ptr.begin(), boxes_ptr.end(),
-                                callback, cutoff);
+  CGAL::box_self_intersection_d(boxes_ptr.begin(), boxes_ptr.end(), callback, cutoff);
   return callback.m_iterator;
 }
 
@@ -1634,7 +1695,7 @@ OutputIterator intersecting_meshes(const
 
 template <class TriangleMeshRange, class OutputIterator>
 OutputIterator intersecting_meshes(const TriangleMeshRange& range,
-                                          OutputIterator out)
+                                         OutputIterator out)
 {
   return intersecting_meshes(range, out, parameters::all_default());
 }
@@ -1648,8 +1709,8 @@ OutputIterator intersecting_meshes(const
  * \pre \link CGAL::Polygon_mesh_processing::does_self_intersect() `!CGAL::Polygon_mesh_processing::does_self_intersect(tm2)` \endlink
  *
  * @tparam TriangleMesh a model of `MutableFaceGraph`, `HalfedgeListGraph` and `FaceListGraph`
- * @tparam NamedParameters1 a sequence of \ref pmp_namedparameters "Named Parameters"
- * @tparam NamedParameters2 a sequence of \ref pmp_namedparameters "Named Parameters"
+ * @tparam NamedParameters1 a sequence of \ref bgl_namedparameters "Named Parameters"
+ * @tparam NamedParameters2 a sequence of \ref bgl_namedparameters "Named Parameters"
  * @tparam OutputIterator an output iterator in which `std::vector` of points
  *                        can be put. The point type is the one from the
  *                        vertex property map
@@ -1658,19 +1719,28 @@ OutputIterator intersecting_meshes(const
  * @param tm2 second input triangulated surface mesh
  * @param polyline_output output iterator of polylines. Each polyline will be
  *        given as a vector of points
- * @param np1 optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
- * @param np2 optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+ * @param np1 an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+ * @param np2 an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *   \cgalParamBegin{vertex_point_map}
- *    a property map with the points associated to the vertices of `tm1`
- *    (`tm2`). The two property map types must be the same.
- *    \cgalParamEnd
- *   \cgalParamBegin{throw_on_self_intersection} if `true`, for each input triangle mesh,
- *      the set of triangles close to the intersection of `tm1` and `tm2` will be
- *      checked for self-intersection and `CGAL::Polygon_mesh_processing::Corefinement::Self_intersection_exception`
- *      will be thrown if at least one is found (`np1` only).
- *   \cgalParamEnd
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `tm1` (`tm2`)}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+ *                    as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm1 (tm2))`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     should be available for the vertices of `tm1` (`tm2`)}
+ *     \cgalParamExtra{Both vertex point maps must have the same value type}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{throw_on_self_intersection}
+ *     \cgalParamDescription{If `true`, the set of triangles closed to the intersection of `tm1` and `tm2` will be
+ *                           checked for self-intersections and `Corefinement::Self_intersection_exception`
+ *                           will be thrown if at least one self-intersection is found.}
+ *     \cgalParamType{Boolean}
+ *     \cgalParamDefault{`false`}
+ *     \cgalParamExtra{`np1` only}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  *
  */
@@ -1710,10 +1780,10 @@ surface_intersection(const TriangleMesh&
 namespace experimental {
 /**
  * \ingroup PMP_corefinement_grp
+ *
  * computes the autointersection of triangles of `tm`. The output is a
  * set of polylines with all vertices but endpoints being of degree 2.
  *
- *
  * @tparam TriangleMesh a model of `HalfedgeListGraph` and `FaceListGraph`
  * @tparam NamedParameters a sequence of \ref namedparameters
  * @tparam OutputIterator an output iterator in which `std::vector` of points
@@ -1723,12 +1793,15 @@ namespace experimental {
  * @param tm input triangulated surface mesh
  * @param polyline_output output iterator of polylines. Each polyline will be
  *        given as a vector of points
- * @param np optional sequence of \ref namedparameters among the ones listed below
+ * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *   \cgalParamBegin{vertex_point_map}
- *    a property map with the points associated to the vertices of `tm`
- *    \cgalParamEnd
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `tm`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+ *                    as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  *
  */
@@ -1827,9 +1900,10 @@ surface_self_intersection(const Triangle
     CGAL::Polygon_mesh_processing::parameters::all_default()
   );
 }
-} //end of namespace experimental
 
-} } //end of namespace CGAL::Polygon_mesh_processing
+} //end of namespace experimental
+} //end of namespace Polygon_mesh_processing
+} //end of namespace CGAL
 
 #include <CGAL/enable_warnings.h>
 
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/locate.h 5.1-1/include/CGAL/Polygon_mesh_processing/locate.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/locate.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/locate.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/locate.h $
-// $Id: locate.h 416cbbc 2019-10-29T15:23:41+01:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/locate.h $
+// $Id: locate.h a84927d 2020-07-23T17:15:44+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Mael Rouxel-Labbé,
@@ -391,7 +391,7 @@ barycentric_coordinates(const Point& p,
 
 /// \ingroup PMP_locate_grp
 ///
-/// \brief Returns a random point over the halfedge `hd`, as a location.
+/// \brief returns a random point over the halfedge `hd`, as a location.
 ///
 /// \details The random point is chosen on the halfedge, meaning that all
 ///          its barycentric coordinates are positive. It is constructed by uniformly generating
@@ -426,7 +426,7 @@ random_location_on_halfedge(typename boo
 
 /// \ingroup PMP_locate_grp
 ///
-/// \brief Returns a random point over the face `fd`, as a location.
+/// \brief returns a random point over the face `fd`, as a location.
 ///
 /// \details The random point is on the face, meaning that all its barycentric coordinates
 ///          are positive.  It is constructed by uniformly picking a value `u` between `0` and `1`,
@@ -459,7 +459,7 @@ random_location_on_face(typename boost::
 
 /// \ingroup PMP_locate_grp
 ///
-/// \brief Returns a random point over the mesh `tm`.
+/// \brief returns a random point over the mesh `tm`.
 ///
 /// \details The returned location is obtained by choosing a random face of the mesh and
 ///          a random point on that face. The barycentric coordinates of the point in the face
@@ -562,22 +562,28 @@ get_descriptor_from_location(const std::
 ///
 /// \tparam FT must be a model of `FieldNumberType`
 /// \tparam TriangleMesh must be a model of `FaceGraph`
-/// \tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+/// \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 ///
 /// \param loc the location from which a point is constructed
 /// \param tm a triangulated surface mesh
-/// \param np an optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below:
+/// \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 ///
 /// \cgalNamedParamsBegin
-///   \cgalParamBegin{vertex_point_map}
-///     the property map with the points associated to the vertices of `tm`.
-///     If this parameter is omitted, an internal property map for
-///     `boost::vertex_point_t` must be available in `TriangleMesh`.
-///   \cgalParamEnd
-///   \cgalParamBegin{geom_traits}
-///     a geometric traits class instance, model of `Kernel`. If such traits class is provided,
-///     its type `FT` must be identical to the template parameter `FT` of this function.
-///   \cgalParamEnd
+///   \cgalParamNBegin{vertex_point_map}
+///     \cgalParamDescription{a property map associating points to the vertices of `tm`}
+///     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+///                    as key type and `%Point_3` as value type}
+///     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{geom_traits}
+///     \cgalParamDescription{an instance of a geometric traits class}
+///     \cgalParamType{a class model of `Kernel`}
+///     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+///     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+///     \cgalParamExtra{If such traits class is provided, its type `FT` must be identical
+///                     to the template parameter `FT` of this function.}
+///   \cgalParamNEnd
 /// \cgalNamedParamsEnd
 ///
 /// \pre `loc.first` is a face descriptor corresponding to a face of `tm`.
@@ -611,7 +617,7 @@ construct_point(const std::pair<typename
 
   VertexPointMap vpm = parameters::choose_parameter(parameters::get_parameter(np, internal_np::vertex_point),
                                                     get_const_property_map(boost::vertex_point, tm));
-  Geom_traits gt = choose_parameter(get_parameter(np, internal_np::geom_traits), Geom_traits());
+  Geom_traits gt = choose_parameter<Geom_traits>(get_parameter(np, internal_np::geom_traits));
 
   halfedge_descriptor hd = halfedge(loc.first, tm);
   const Point_reference p0 = get(vpm, source(hd, tm));
@@ -913,7 +919,7 @@ is_on_mesh_border(const std::pair<typena
 
 /// \ingroup PMP_locate_grp
 ///
-/// \brief Returns the location of the given vertex `vd` as a location,
+/// \brief returns the location of the given vertex `vd` as a location,
 ///        that is an ordered pair specifying a face incident to `vd`
 ///        and the barycentric coordinates of the vertex `vd` in that face.
 ///
@@ -975,7 +981,7 @@ locate_vertex(typename boost::graph_trai
 
 /// \ingroup PMP_locate_grp
 ///
-/// \brief Returns the location of a given vertex as a location in `fd`,
+/// \brief returns the location of a given vertex as a location in `fd`,
 ///        that is an ordered pair composed of `fd` and of the barycentric coordinates
 ///        of the vertex in `fd`.
 ///
@@ -1066,32 +1072,41 @@ locate_on_halfedge(const typename boost:
 ///          - `w2` corresponds to `target(next(halfedge(f, tm), tm), tm)`
 ///
 /// \tparam TriangleMesh must be a model of `FaceGraph`
-/// \tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+/// \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 ///
 /// \param query a point, whose type is equal to the value type of the vertex point property map
 ///              (either user-provided via named parameters or the internal point map of the mesh `tm`)
 /// \param fd a face of `tm`
 /// \param tm a triangulated surface mesh
-/// \param np an optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below:
+/// \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 ///
 /// \cgalNamedParamsBegin
-///   \cgalParamBegin{vertex_point_map}
-///     the property map with the points associated to the vertices of `tm`.
-///     If this parameter is omitted, an internal property map for
-///     `boost::vertex_point_t` must be available in `TriangleMesh`.
-///   \cgalParamEnd
-///   \cgalParamBegin{geom_traits}
-///     a geometric traits class instance, model of `Kernel`. If provided, the types `FT` and `Kernel::FT`
-///     must be identical and the traits class must be compatible with the value type of the vertex point
-///     property map.
-///   \cgalParamEnd
-///   \cgalParamBegin{snapping_tolerance}
-///     a tolerance value used to snap barycentric coordinates. Depending on the geometric traits used,
-///     the computation of the barycentric coordinates might be an inexact construction, thus leading
-///     to sometimes surprising values (e.g. a triplet `[0.5, 0.5, -1-e17]` for a point at the middle
-///     of an edge). The coordinates will be snapped towards `0` and `1` if the difference is smaller than the tolerance value, while
-///     still ensuring that the total sum of the coordinates is `1`. By default, the tolerance is `0`.
-///   \cgalParamEnd
+///   \cgalParamNBegin{vertex_point_map}
+///     \cgalParamDescription{a property map associating points to the vertices of `tm`}
+///     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+///                    as key type and `%Point_3` as value type}
+///     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{geom_traits}
+///     \cgalParamDescription{an instance of a geometric traits class}
+///     \cgalParamType{a class model of `Kernel`}
+///     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+///     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+///     \cgalParamExtra{If such traits class is provided, its type `FT` must be identical
+///                     to the template parameter `FT` of this function.}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{snapping_tolerance}
+///     \cgalParamDescription{a tolerance value used to snap barycentric coordinates}
+///     \cgalParamType{double}
+///     \cgalParamDefault{`0`}
+///     \cgalParamExtra{Depending on the geometric traits used, the computation of the barycentric coordinates
+///                     might be an inexact construction, thus leading to sometimes surprising values
+///                     (e.g. a triplet `[0.5, 0.5, -1-e17]` for a point at the middle of an edge).
+///                     The coordinates will be snapped towards `0` and `1` if the difference is smaller
+///                     than the tolerance value, while still ensuring that the total sum of the coordinates is `1`.}
+///   \cgalParamNEnd
 /// \cgalNamedParamsEnd
 ///
 /// \pre `fd` is not the null face
@@ -1126,7 +1141,7 @@ locate_in_face(const typename internal::
 
   VertexPointMap vpm = parameters::choose_parameter(parameters::get_parameter(np, internal_np::vertex_point),
                                                     get_const_property_map(boost::vertex_point, tm));
-  Geom_traits gt = choose_parameter(get_parameter(np, internal_np::geom_traits), Geom_traits());
+  Geom_traits gt = choose_parameter<Geom_traits>(get_parameter(np, internal_np::geom_traits));
 
   FT snap_tolerance = choose_parameter(get_parameter(np, internal_np::snapping_tolerance), 0);
 
@@ -1556,23 +1571,27 @@ void build_AABB_tree(const TriangleMesh&
 /// \tparam TriangleMesh must be a model of `FaceListGraph`
 /// \tparam Point3VPM must be a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
 ///                   as key type and the \cgal 3D point type (your traits' `%Point_3`) as value type.
-/// \tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+/// \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 ///
 /// \param tm a triangulated surface mesh
 /// \param outTree output parameter that stores the computed `AABB_tree`
-/// \param np an optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below:
+/// \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 ///
 /// \cgalNamedParamsBegin
-///   \cgalParamBegin{vertex_point_map}
-///     the property map with the points associated to the vertices of `tm`.
-///     If this parameter is omitted, an internal property map for
-///     `boost::vertex_point_t` must be available in `TriangleMesh`.
-///   \cgalParamEnd
-///   \cgalParamBegin{geom_traits}
-///     a geometric traits class instance, model of `Kernel` compatible with the point type held
-///     in the vertex point property map (either user-provided or internal to the mesh).
-///     Must be identical to the traits used in the template parameter of the `AABB_traits`.
-///   \cgalParamEnd
+///   \cgalParamNBegin{vertex_point_map}
+///     \cgalParamDescription{a property map associating points to the vertices of `tm`}
+///     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+///                    as key type and `%Point_3` as value type}
+///     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{geom_traits}
+///     \cgalParamDescription{an instance of a geometric traits class}
+///     \cgalParamType{a class model of `Kernel`}
+///     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+///     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+///     \cgalParamExtra{Must be identical to the traits used in the template parameter of the `AABB_traits`.}
+///   \cgalParamNEnd
 /// \cgalNamedParamsEnd
 ///
 template <typename TriangleMesh, typename Point3VPM, typename NamedParameters>
@@ -1618,31 +1637,39 @@ void build_AABB_tree(const TriangleMesh&
 /// \tparam TriangleMesh must be a model of `FaceListGraph`
 /// \tparam Point3VPM must be a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
 ///                   as key type and the \cgal 3D point type (your traits' `%Point_3`) as value type.
-/// \tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+/// \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 ///
 /// \param p the point to locate on the input triangulated surface mesh
 /// \param tree an AABB tree containing the triangular faces of the input surface mesh to perform the point location with
 /// \param tm a triangulated surface mesh
-/// \param np an optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below:
+/// \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 ///
 /// \cgalNamedParamsBegin
-///   \cgalParamBegin{vertex_point_map}
-///     the property map with the points associated to the vertices of `tm`.
-///     If this parameter is omitted, an internal property map for
-///     `boost::vertex_point_t` must be available in `TriangleMesh`.
-///   \cgalParamEnd
-///   \cgalParamBegin{geom_traits}
-///     a geometric traits class instance, model of `Kernel` compatible with the point type held
-///     in the vertex point property map (either user-provided or internal to the mesh).
-///     Must be identical to the traits used in the template parameter of the `AABB_traits`.
-///   \cgalParamEnd
-///   \cgalParamBegin{snapping_tolerance}
-///     a tolerance value used to snap barycentric coordinates. Depending on the geometric traits used,
-///     the computation of the barycentric coordinates might be an inexact construction, thus leading
-///     to sometimes surprising values (e.g. a triplet `[0.5, 0.5, -1-e17]` for a point at the middle
-///     of an edge). The coordinates will be snapped towards `0` and `1` if the difference is smaller than the tolerance value, while
-///     still ensuring that the total sum of the coordinates is `1`. By default, the tolerance is `0`.
-///   \cgalParamEnd
+///   \cgalParamNBegin{vertex_point_map}
+///     \cgalParamDescription{a property map associating points to the vertices of `tm`}
+///     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+///                    as key type and `%Point_3` as value type}
+///     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{geom_traits}
+///     \cgalParamDescription{an instance of a geometric traits class}
+///     \cgalParamType{a class model of `Kernel`}
+///     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+///     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+///     \cgalParamExtra{Must be identical to the traits used in the template parameter of the `AABB_traits`.}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{snapping_tolerance}
+///     \cgalParamDescription{a tolerance value used to snap barycentric coordinates}
+///     \cgalParamType{double}
+///     \cgalParamDefault{`0`}
+///     \cgalParamExtra{Depending on the geometric traits used, the computation of the barycentric coordinates
+///                     might be an inexact construction, thus leading to sometimes surprising values
+///                     (e.g. a triplet `[0.5, 0.5, -1-e17]` for a point at the middle of an edge).
+///                     The coordinates will be snapped towards `0` and `1` if the difference is smaller
+///                     than the tolerance value, while still ensuring that the total sum of the coordinates is `1`.}
+///   \cgalParamNEnd
 /// \cgalNamedParamsEnd
 ///
 /// \returns a face location. The type `FT` is deduced from the geometric traits, either provided by
@@ -1715,29 +1742,37 @@ locate_with_AABB_tree(const typename int
 ///          an AABB tree that you can store and use the function `locate_with_AABB_tree()`.
 ///
 /// \tparam TriangleMesh must be a model of `FaceListGraph`.
-/// \tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+/// \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 ///
 /// \param p the point to locate on the input triangulated surface mesh
 /// \param tm a triangulated surface mesh
-/// \param np an optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below:
+/// \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 ///
 /// \cgalNamedParamsBegin
-///   \cgalParamBegin{vertex_point_map}
-///     the property map with the points associated to the vertices of `tm`.
-///     If this parameter is omitted, an internal property map for
-///     `boost::vertex_point_t` must be available in `TriangleMesh`.
-///   \cgalParamEnd
-///   \cgalParamBegin{geom_traits}
-///     a geometric traits class instance, model of `Kernel` compatible with the point type held
-///     in the vertex point property map (either user-provided or internal to the mesh).
-///   \cgalParamEnd
-///   \cgalParamBegin{snapping_tolerance}
-///     a tolerance value used to snap barycentric coordinates. Depending on the geometric traits used,
-///     the computation of the barycentric coordinates might be an inexact construction, thus leading
-///     to sometimes surprising values (e.g. a triplet `[0.5, 0.5, -1-e17]` for a point at the middle
-///     of an edge). The coordinates will be snapped towards `0` and `1` if the difference is smaller than the tolerance value, while
-///     still ensuring that the total sum of the coordinates is `1`. By default, the tolerance is `0`.
-///   \cgalParamEnd
+///   \cgalParamNBegin{vertex_point_map}
+///     \cgalParamDescription{a property map associating points to the vertices of `tm`}
+///     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+///                    as key type and `%Point_3` as value type}
+///     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{geom_traits}
+///     \cgalParamDescription{an instance of a geometric traits class}
+///     \cgalParamType{a class model of `Kernel`}
+///     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+///     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{snapping_tolerance}
+///     \cgalParamDescription{a tolerance value used to snap barycentric coordinates}
+///     \cgalParamType{double}
+///     \cgalParamDefault{`0`}
+///     \cgalParamExtra{Depending on the geometric traits used, the computation of the barycentric coordinates
+///                     might be an inexact construction, thus leading to sometimes surprising values
+///                     (e.g. a triplet `[0.5, 0.5, -1-e17]` for a point at the middle of an edge).
+///                     The coordinates will be snapped towards `0` and `1` if the difference is smaller
+///                     than the tolerance value, while still ensuring that the total sum of the coordinates is `1`.}
+///   \cgalParamNEnd
 /// \cgalNamedParamsEnd
 ///
 template <typename TriangleMesh, typename NamedParameters>
@@ -1793,38 +1828,46 @@ locate(const typename property_map_value
 
 /// \ingroup PMP_locate_grp
 ///
-/// \brief Returns the face location along `ray` nearest to its source point.
+/// \brief returns the face location along `ray` nearest to its source point.
 ///
 /// If the ray does not intersect the mesh, a default constructed location is returned.
 ///
 /// \tparam TriangleMesh must be a model of `FaceListGraph`.
 /// \tparam Point3VPM must be a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
 ///                   as key type and the \cgal 3D point type (your traits' `%Point_3`) as value type.
-/// \tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+/// \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 ///
 /// \param ray a ray to intersect with the input triangulated surface mesh
 /// \param tree an AABB tree containing the triangular faces of the input surface mesh to perform the point location with
 /// \param tm a triangulated surface mesh
-/// \param np an optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below:
+/// \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 ///
 /// \cgalNamedParamsBegin
-///   \cgalParamBegin{vertex_point_map}
-///     the property map with the points associated to the vertices of `tm`.
-///     If this parameter is omitted, an internal property map for
-///     `boost::vertex_point_t` must be available in `TriangleMesh`.
-///   \cgalParamEnd
-///   \cgalParamBegin{geom_traits}
-///     a geometric traits class instance, model of `Kernel` compatible with the point type held
-///     in the vertex point property map (either user-provided or internal to the mesh).
-///     Must be identical to the traits used in the template parameter of the `AABB_traits`.
-///   \cgalParamEnd
-///   \cgalParamBegin{snapping_tolerance}
-///     a tolerance value used to snap barycentric coordinates. Depending on the geometric traits used,
-///     the computation of the barycentric coordinates might be an inexact construction, thus leading
-///     to sometimes surprising values (e.g. a triplet `[0.5, 0.5, -1-e17]` for a point at the middle
-///     of an edge). The coordinates will be snapped towards `0` and `1` if the difference is smaller than the tolerance value, while
-///     still ensuring that the total sum of the coordinates is `1`. By default, the tolerance is `0`.
-///   \cgalParamEnd
+///   \cgalParamNBegin{vertex_point_map}
+///     \cgalParamDescription{a property map associating points to the vertices of `tm`}
+///     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+///                    as key type and `%Point_3` as value type}
+///     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{geom_traits}
+///     \cgalParamDescription{an instance of a geometric traits class}
+///     \cgalParamType{a class model of `Kernel`}
+///     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+///     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+///     \cgalParamExtra{Must be identical to the traits used in the template parameter of the `AABB_traits`.}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{snapping_tolerance}
+///     \cgalParamDescription{a tolerance value used to snap barycentric coordinates}
+///     \cgalParamType{double}
+///     \cgalParamDefault{`0`}
+///     \cgalParamExtra{Depending on the geometric traits used, the computation of the barycentric coordinates
+///                     might be an inexact construction, thus leading to sometimes surprising values
+///                     (e.g. a triplet `[0.5, 0.5, -1-e17]` for a point at the middle of an edge).
+///                     The coordinates will be snapped towards `0` and `1` if the difference is smaller
+///                     than the tolerance value, while still ensuring that the total sum of the coordinates is `1`.}
+///   \cgalParamNEnd
 /// \cgalNamedParamsEnd
 ///
 /// \pre `ray` is an object with the same ambient dimension as the point type (the value type of the vertex point map).
@@ -1926,7 +1969,7 @@ locate_with_AABB_tree(const typename int
 
 /// \ingroup PMP_locate_grp
 ///
-/// \brief Returns the face location along `ray` nearest to its source point.
+/// \brief returns the face location along `ray` nearest to its source point.
 ///
 /// If the ray does not intersect the mesh, a default constructed location is returned.
 ///
@@ -1936,29 +1979,37 @@ locate_with_AABB_tree(const typename int
 ///          that accept a reference to an AABB tree as input.
 ///
 /// \tparam TriangleMesh must be a model of `FaceListGraph`.
-/// \tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+/// \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 ///
 /// \param ray a ray to intersect with the input triangulated surface mesh
 /// \param tm the input triangulated surface mesh
-/// \param np an optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below:
+/// \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 ///
 /// \cgalNamedParamsBegin
-///   \cgalParamBegin{vertex_point_map}
-///     the property map with the points associated to the vertices of `tm`.
-///     If this parameter is omitted, an internal property map for
-///     `boost::vertex_point_t` must be available in `TriangleMesh`.
-///   \cgalParamEnd
-///   \cgalParamBegin{geom_traits}
-///     a geometric traits class instance, model of `Kernel` compatible with the point type held
-///     in the vertex point property map (either user-provided or internal to the mesh).
-///   \cgalParamEnd
-///   \cgalParamBegin{snapping_tolerance}
-///     a tolerance value used to snap barycentric coordinates. Depending on the geometric traits used,
-///     the computation of the barycentric coordinates might be an inexact construction, thus leading
-///     to sometimes surprising values (e.g. a triplet `[0.5, 0.5, -1-e17]` for a point at the middle
-///     of an edge). The coordinates will be snapped towards `0` and `1` if the difference is smaller than the tolerance value, while
-///     still ensuring that the total sum of the coordinates is `1`. By default, the tolerance is `0`.
-///   \cgalParamEnd
+///   \cgalParamNBegin{vertex_point_map}
+///     \cgalParamDescription{a property map associating points to the vertices of `tm`}
+///     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+///                    as key type and `%Point_3` as value type}
+///     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{geom_traits}
+///     \cgalParamDescription{an instance of a geometric traits class}
+///     \cgalParamType{a class model of `Kernel`}
+///     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+///     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{snapping_tolerance}
+///     \cgalParamDescription{a tolerance value used to snap barycentric coordinates}
+///     \cgalParamType{double}
+///     \cgalParamDefault{`0`}
+///     \cgalParamExtra{Depending on the geometric traits used, the computation of the barycentric coordinates
+///                     might be an inexact construction, thus leading to sometimes surprising values
+///                     (e.g. a triplet `[0.5, 0.5, -1-e17]` for a point at the middle of an edge).
+///                     The coordinates will be snapped towards `0` and `1` if the difference is smaller
+///                     than the tolerance value, while still ensuring that the total sum of the coordinates is `1`.}
+///   \cgalParamNEnd
 /// \cgalNamedParamsEnd
 ///
 /// \pre `ray` is an object with the same ambient dimension as the point type (the value type of the vertex point map).
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/manifoldness.h 5.1-1/include/CGAL/Polygon_mesh_processing/manifoldness.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/manifoldness.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/manifoldness.h	2020-09-08 12:15:23.000000000 +0000
@@ -0,0 +1,472 @@
+// Copyright (c) 2015-2019 GeometryFactory (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/manifoldness.h $
+// $Id: manifoldness.h a84927d 2020-07-23T17:15:44+02:00 Laurent Rineau
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Sebastien Loriot,
+//                 Mael Rouxel-Labbé
+//
+#ifndef CGAL_POLYGON_MESH_PROCESSING_MANIFOLDNESS_H
+#define CGAL_POLYGON_MESH_PROCESSING_MANIFOLDNESS_H
+
+#include <CGAL/license/Polygon_mesh_processing/repair.h>
+
+#include <CGAL/Polygon_mesh_processing/internal/named_function_params.h>
+#include <CGAL/Polygon_mesh_processing/internal/named_params_helper.h>
+
+#include <CGAL/algorithm.h>
+#include <CGAL/assertions.h>
+#include <CGAL/boost/graph/helpers.h>
+#include <CGAL/Dynamic_property_map.h>
+#include <CGAL/iterator.h>
+#include <CGAL/property_map.h>
+
+#include <iterator>
+#include <map>
+#include <utility>
+#include <vector>
+
+namespace CGAL {
+namespace Polygon_mesh_processing {
+
+/// \ingroup PMP_repairing_grp
+/// returns whether a vertex of a polygon mesh is non-manifold.
+///
+/// @tparam PolygonMesh a model of `HalfedgeListGraph`
+///
+/// @param v a vertex of `pm`
+/// @param pm a triangle mesh containing `v`
+///
+/// \warning This function has linear runtime with respect to the size of the mesh.
+///
+/// \sa `duplicate_non_manifold_vertices()`
+///
+/// \return `true` if the vertex is non-manifold, `false` otherwise.
+template <typename PolygonMesh>
+bool is_non_manifold_vertex(typename boost::graph_traits<PolygonMesh>::vertex_descriptor v,
+                            const PolygonMesh& pm)
+{
+  typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor halfedge_descriptor;
+
+  typedef CGAL::dynamic_halfedge_property_t<bool>                                       Halfedge_property_tag;
+  typedef typename boost::property_map<PolygonMesh, Halfedge_property_tag>::const_type  Visited_halfedge_map;
+
+  // Dynamic pmaps do not have default initialization values (yet)
+  Visited_halfedge_map visited_halfedges = get(Halfedge_property_tag(), pm);
+  for(halfedge_descriptor h : halfedges(pm))
+    put(visited_halfedges, h, false);
+
+  std::size_t incident_null_faces_counter = 0;
+  for(halfedge_descriptor h : halfedges_around_target(v, pm))
+  {
+    put(visited_halfedges, h, true);
+    if(CGAL::is_border(h, pm))
+      ++incident_null_faces_counter;
+  }
+
+  if(incident_null_faces_counter > 1)
+  {
+    // The vertex is the sole connection between two connected components --> non-manifold
+    return true;
+  }
+
+  for(halfedge_descriptor h : halfedges(pm))
+  {
+    if(v == target(h, pm))
+    {
+      // Haven't seen that halfedge yet ==> more than one umbrella incident to 'v' ==> non-manifold
+      if(!get(visited_halfedges, h))
+        return true;
+    }
+  }
+
+  return false;
+}
+
+namespace internal {
+
+template <typename G>
+struct Vertex_collector
+{
+  typedef typename boost::graph_traits<G>::vertex_descriptor      vertex_descriptor;
+
+  bool has_old_vertex(const vertex_descriptor v) const { return collections.count(v) != 0; }
+  void tag_old_vertex(const vertex_descriptor v)
+  {
+    CGAL_precondition(!has_old_vertex(v));
+    collections[v];
+  }
+
+  void collect_vertices(vertex_descriptor v1, vertex_descriptor v2)
+  {
+    std::vector<vertex_descriptor>& verts = collections[v1];
+    if(verts.empty())
+      verts.push_back(v1);
+    verts.push_back(v2);
+  }
+
+  template<typename OutputIterator>
+  void dump(OutputIterator out)
+  {
+    typedef std::pair<const vertex_descriptor, std::vector<vertex_descriptor> > Pair_type;
+    for(const Pair_type& p : collections)
+      *out++ = p.second;
+  }
+
+  void dump(Emptyset_iterator) { }
+
+  std::map<vertex_descriptor, std::vector<vertex_descriptor> > collections;
+};
+
+template <typename PolygonMesh, typename VPM, typename ConstraintMap>
+typename boost::graph_traits<PolygonMesh>::vertex_descriptor
+create_new_vertex_for_sector(typename boost::graph_traits<PolygonMesh>::halfedge_descriptor sector_begin_h,
+                             typename boost::graph_traits<PolygonMesh>::halfedge_descriptor sector_last_h,
+                             PolygonMesh& pm,
+                             const VPM& vpm,
+                             const ConstraintMap& cmap)
+{
+  typedef typename boost::graph_traits<PolygonMesh>::vertex_descriptor    vertex_descriptor;
+  typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor  halfedge_descriptor;
+
+  vertex_descriptor old_vd = target(sector_begin_h, pm);
+  vertex_descriptor new_vd = add_vertex(pm);
+  put(vpm, new_vd, get(vpm, old_vd));
+
+  put(cmap, new_vd, true);
+
+  set_halfedge(new_vd, sector_begin_h, pm);
+  halfedge_descriptor h = sector_begin_h;
+  do
+  {
+    set_target(h, new_vd, pm);
+
+    if(h == sector_last_h)
+      break;
+    else
+      h = prev(opposite(h, pm), pm);
+  }
+  while(h != sector_begin_h); // for safety
+  CGAL_assertion(h != sector_begin_h);
+
+  return new_vd;
+}
+
+template <typename PolygonMesh, typename NamedParameters>
+std::size_t make_umbrella_manifold(typename boost::graph_traits<PolygonMesh>::halfedge_descriptor h,
+                                   PolygonMesh& pm,
+                                   internal::Vertex_collector<PolygonMesh>& dmap,
+                                   const NamedParameters& np)
+{
+  typedef typename boost::graph_traits<PolygonMesh>::vertex_descriptor    vertex_descriptor;
+  typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor  halfedge_descriptor;
+
+  using parameters::get_parameter;
+  using parameters::choose_parameter;
+
+  typedef typename GetVertexPointMap<PolygonMesh, NamedParameters>::type VertexPointMap;
+  VertexPointMap vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
+                                        get_property_map(vertex_point, pm));
+
+  typedef typename internal_np::Lookup_named_param_def<internal_np::vertex_is_constrained_t,
+                                                       NamedParameters,
+                                                       Constant_property_map<vertex_descriptor, bool> // default (no constraint pmap)
+                                                       >::type                  VerticesMap;
+  VerticesMap cmap = choose_parameter(get_parameter(np, internal_np::vertex_is_constrained),
+                                      Constant_property_map<vertex_descriptor, bool>(false));
+
+  std::size_t nb_new_vertices = 0;
+
+  vertex_descriptor old_v = target(h, pm);
+  put(cmap, old_v, true); // store the duplicates
+
+  // count the number of borders
+  int border_counter = 0;
+  halfedge_descriptor ih = h, done = ih, border_h = h;
+  do
+  {
+    if(is_border(ih, pm))
+    {
+      border_h = ih;
+      ++border_counter;
+    }
+
+    ih = prev(opposite(ih, pm), pm);
+  }
+  while(ih != done);
+
+  bool is_non_manifold_within_umbrella = (border_counter > 1);
+  if(!is_non_manifold_within_umbrella)
+  {
+    const bool first_time_meeting_v = !dmap.has_old_vertex(old_v);
+    if(first_time_meeting_v)
+    {
+      // The star is manifold, so if it is the first time we have met that vertex,
+      // there is nothing to do, we just keep the same vertex.
+      set_halfedge(old_v, h, pm); // to ensure halfedge(old_v, pm) stays valid
+      dmap.tag_old_vertex(old_v); // so that we know we have met old_v already, next time, we'll have to duplicate
+    }
+    else
+    {
+      // This is not the canonical star associated to 'v'.
+      // Create a new vertex, and move the whole star to that new vertex
+      halfedge_descriptor last_h = opposite(next(h, pm), pm);
+      vertex_descriptor new_v = create_new_vertex_for_sector(h, last_h, pm, vpm, cmap);
+      dmap.collect_vertices(old_v, new_v);
+      nb_new_vertices = 1;
+    }
+  }
+  // if there is more than one sector, look at each sector and split them away from the main one
+  else
+  {
+    // the first manifold sector, described by two halfedges
+    halfedge_descriptor sector_start_h = border_h;
+    CGAL_assertion(is_border(border_h, pm));
+
+    bool should_stop = false;
+    bool is_main_sector = true;
+    do
+    {
+      CGAL_assertion(is_border(sector_start_h, pm));
+
+      // collect the sector and split it away if it must be
+      halfedge_descriptor sector_last_h = sector_start_h;
+      do
+      {
+        halfedge_descriptor next_h = prev(opposite(sector_last_h, pm), pm);
+
+        if(is_border(next_h, pm))
+          break;
+
+        sector_last_h = next_h;
+      }
+      while(sector_last_h != sector_start_h);
+      CGAL_assertion(!is_border(sector_last_h, pm));
+      CGAL_assertion(sector_last_h != sector_start_h);
+
+      halfedge_descriptor next_start_h = prev(opposite(sector_last_h, pm), pm);
+
+      // there are multiple CCs incident to this particular vertex, and we should create a new vertex
+      // if it's not the first umbrella around 'old_v' or not the first sector, but only not if it's
+      // both the first umbrella and first sector.
+      bool must_create_new_vertex = (!is_main_sector || dmap.has_old_vertex(old_v));
+
+      // In any case, we must set up the next pointer correctly
+      set_next(sector_start_h, opposite(sector_last_h, pm), pm);
+
+      if(must_create_new_vertex)
+      {
+        vertex_descriptor new_v = create_new_vertex_for_sector(sector_start_h, sector_last_h, pm, vpm, cmap);
+        dmap.collect_vertices(old_v, new_v);
+        ++nb_new_vertices;
+      }
+      else
+      {
+        // We are in the first sector and first star, ensure that halfedge(old_v, pm) stays valid
+        set_halfedge(old_v, sector_start_h, pm);
+      }
+
+      is_main_sector = false;
+      sector_start_h = next_start_h;
+      should_stop = (sector_start_h == border_h);
+    }
+    while(!should_stop);
+  }
+
+  return nb_new_vertices;
+}
+
+} // end namespace internal
+
+/// \ingroup PMP_repairing_grp
+/// collects the non-manifold vertices (if any) present in the mesh. A non-manifold vertex `v` is returned
+/// via one incident halfedge `h` such that `target(h, pm) = v` for all the umbrellas that `v` apppears in
+/// (an <i>umbrella</i> being the set of faces incident to all the halfedges reachable by walking around `v`
+/// using `hnext = prev(opposite(h, pm), pm)`, starting from `h`).
+///
+/// @tparam PolygonMesh a model of `HalfedgeListGraph`
+/// @tparam OutputIterator a model of `OutputIterator` holding objects of type
+///                         `boost::graph_traits<PolygonMesh>::%halfedge_descriptor`
+///
+/// @param pm a triangle mesh
+/// @param out the output iterator that collects halfedges incident to `v`
+///
+/// \sa `is_non_manifold_vertex()`
+/// \sa `duplicate_non_manifold_vertices()`
+///
+/// \return the output iterator.
+template <typename PolygonMesh, typename OutputIterator>
+OutputIterator non_manifold_vertices(const PolygonMesh& pm,
+                                     OutputIterator out)
+{
+  // Non-manifoldness can appear either:
+  // - if 'pm' is pinched at a vertex. While traversing the incoming halfedges at this vertex,
+  //   we will meet strictly more than one border halfedge.
+  // - if there are multiple umbrellas around a vertex. In that case, we will find a non-visited
+  //   halfedge that has for target a vertex that is already visited.
+
+  typedef typename boost::graph_traits<PolygonMesh>::vertex_descriptor                  vertex_descriptor;
+  typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor                halfedge_descriptor;
+
+  typedef CGAL::dynamic_vertex_property_t<bool>                                         Vertex_bool_tag;
+  typedef typename boost::property_map<PolygonMesh, Vertex_bool_tag>::const_type        Known_manifold_vertex_map;
+  typedef CGAL::dynamic_vertex_property_t<halfedge_descriptor>                          Vertex_halfedge_tag;
+  typedef typename boost::property_map<PolygonMesh, Vertex_halfedge_tag>::const_type    Visited_vertex_map;
+  typedef CGAL::dynamic_halfedge_property_t<bool>                                       Halfedge_property_tag;
+  typedef typename boost::property_map<PolygonMesh, Halfedge_property_tag>::const_type  Visited_halfedge_map;
+
+  Known_manifold_vertex_map known_nm_vertices = get(Vertex_bool_tag(), pm);
+  Visited_vertex_map visited_vertices = get(Vertex_halfedge_tag(), pm);
+  Visited_halfedge_map visited_halfedges = get(Halfedge_property_tag(), pm);
+
+  halfedge_descriptor null_h = boost::graph_traits<PolygonMesh>::null_halfedge();
+
+  // Dynamic pmaps do not have default initialization values (yet)
+  for(vertex_descriptor v : vertices(pm))
+  {
+    put(known_nm_vertices, v, false);
+    put(visited_vertices, v, null_h);
+  }
+  for(halfedge_descriptor h : halfedges(pm))
+    put(visited_halfedges, h, false);
+
+  for(halfedge_descriptor h : halfedges(pm))
+  {
+    // If 'h' is not visited yet, we walk around the target of 'h' and mark these
+    // halfedges as visited. Thus, if we are here and the target is already marked as visited,
+    // it means that the vertex is non manifold.
+    if(!get(visited_halfedges, h))
+    {
+      put(visited_halfedges, h, true);
+      bool is_non_manifold = false;
+
+      vertex_descriptor v = target(h, pm);
+
+      if(get(visited_vertices, v) != null_h) // already seen this vertex, but not from this star
+      {
+        is_non_manifold = true;
+
+        // if this is the second time we visit that vertex and the first star was manifold, we have
+        // never reported the first star, but we must now
+        if(!get(known_nm_vertices, v))
+          *out++ = get(visited_vertices, v); // that's a halfedge of the first star we've seen 'v' in
+      }
+      else
+      {
+        // first time we meet this vertex, just mark it so, and keep the halfedge we found the vertex with in memory
+        put(visited_vertices, v, h);
+      }
+
+      // While walking the star of this halfedge, if we meet a border halfedge more than once,
+      // it means the mesh is pinched and we are also in the case of a non-manifold situation
+      halfedge_descriptor ih = h, done = ih;
+      int border_counter = 0;
+      do
+      {
+        put(visited_halfedges, ih, true);
+        if(is_border(ih, pm))
+          ++border_counter;
+
+        ih = prev(opposite(ih, pm), pm);
+      }
+      while(ih != done);
+
+      if(border_counter > 1)
+        is_non_manifold = true;
+
+      if(is_non_manifold)
+      {
+        *out++ = h;
+        put(known_nm_vertices, v, true);
+      }
+    }
+  }
+
+  return out;
+}
+
+/// \ingroup PMP_repairing_grp
+/// duplicates all the non-manifold vertices of the input mesh.
+///
+/// @tparam PolygonMesh a model of `HalfedgeListGraph` and `MutableHalfedgeGraph`
+/// @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
+///
+/// @param pm the surface mesh to be repaired
+/// @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+///
+/// \cgalNamedParamsBegin
+///   \cgalParamNBegin{vertex_point_map}
+///     \cgalParamDescription{a property map associating points to the vertices of `pm`}
+///     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+///                    as key type and `%Point_3` as value type}
+///     \cgalParamDefault{`boost::get(CGAL::vertex_point, pm)`}
+///     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+///                     must be available in `PolygonMesh`.}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{vertex_is_constrained_map}
+///     \cgalParamDescription{a property map containing the constrained-or-not status of each vertex of `pm`.}
+///     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+///                    as key type and `bool` as value type. It must be default constructible.}
+///     \cgalParamDefault{a default property map where no vertex is constrained}
+///     \cgalParamExtra{`put(vcm, v, true)` will be called for each duplicated
+///                     vertices, as well as the original non-manifold vertex in the input mesh.}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{output_iterator}
+///     \cgalParamDescription{an output iterator to collect the duplicated vertices}
+///     \cgalParamType{a model of `OutputIterator` with value type `std::vector<vertex_descriptor>`}
+///     \cgalParamDefault{unused}
+///     \cgalParamExtra{The first vertex of each vector is a non-manifold vertex of the input mesh,
+///                     followed by the new vertices that were created to fix the given non-manifold configuration.}
+///   \cgalParamNEnd
+/// \cgalNamedParamsEnd
+///
+/// \return the number of vertices created.
+template <typename PolygonMesh, typename NamedParameters>
+std::size_t duplicate_non_manifold_vertices(PolygonMesh& pm,
+                                            const NamedParameters& np)
+{
+  using parameters::get_parameter;
+  using parameters::choose_parameter;
+
+  typedef boost::graph_traits<PolygonMesh>                            GT;
+  typedef typename GT::halfedge_descriptor                            halfedge_descriptor;
+
+  typedef typename internal_np::Lookup_named_param_def<internal_np::output_iterator_t,
+                                                       NamedParameters,
+                                                       Emptyset_iterator>::type Output_iterator;
+
+  Output_iterator out = choose_parameter(get_parameter(np, internal_np::output_iterator),
+                                         Emptyset_iterator());
+
+  std::vector<halfedge_descriptor> non_manifold_cones;
+  non_manifold_vertices(pm, std::back_inserter(non_manifold_cones));
+
+  internal::Vertex_collector<PolygonMesh> dmap;
+  std::size_t nb_new_vertices = 0;
+  if(!non_manifold_cones.empty())
+  {
+    for(halfedge_descriptor h : non_manifold_cones)
+      nb_new_vertices += internal::make_umbrella_manifold(h, pm, dmap, np);
+
+    dmap.dump(out);
+  }
+
+  return nb_new_vertices;
+}
+
+template <class PolygonMesh>
+std::size_t duplicate_non_manifold_vertices(PolygonMesh& pm)
+{
+  return duplicate_non_manifold_vertices(pm, parameters::all_default());
+}
+
+} // namespace Polygon_mesh_processing
+} // namespace CGAL
+
+#endif // CGAL_POLYGON_MESH_PROCESSING_MANIFOLDNESS_H
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/measure.h 5.1-1/include/CGAL/Polygon_mesh_processing/measure.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/measure.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/measure.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/measure.h $
-// $Id: measure.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/measure.h $
+// $Id: measure.h 80f6f65 2020-04-05T17:56:50+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -40,9 +40,7 @@
 
 namespace CGAL {
 
-namespace Polygon_mesh_processing {
-
-// workaround for area(face_range,tm) overload
+// workaround for area(face_range, tm) overload
 template<typename CGAL_PMP_NP_TEMPLATE_PARAMETERS, typename NP>
 class GetGeomTraits<CGAL_PMP_NP_CLASS, NP>
 {
@@ -50,27 +48,38 @@ public:
   struct type{};
 };
 
+namespace Polygon_mesh_processing {
+
 /**
   * \ingroup measure_grp
   * computes the length of an edge of a given polygon mesh.
   * The edge is given by one of its halfedges, or the edge itself.
   *
   * @tparam PolygonMesh a model of `HalfedgeGraph`
-  * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+  * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
   *
   * @param h one halfedge of the edge to compute the length
   * @param pmesh the polygon mesh to which `h` belongs
-  * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+  * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
   *
   * \cgalNamedParamsBegin
-  *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
-  *   If this parameter is omitted, an internal property map for
-  *   `CGAL::vertex_point_t` must be available in `PolygonMesh`\cgalParamEnd
-  *    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+  *   \cgalParamNBegin{vertex_point_map}
+  *     \cgalParamDescription{a property map associating points to the vertices of `pmesh`}
+  *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+  *                    as key type and `%Point_3` as value type}
+  *     \cgalParamDefault{`boost::get(CGAL::vertex_point, pmesh)`}
+  *   \cgalParamNEnd
+  *
+ *   \cgalParamNBegin{geom_traits}
+ *     \cgalParamDescription{an instance of a geometric traits class}
+ *     \cgalParamType{a class model of `Kernel`}
+ *     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+ *     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+ *   \cgalParamNEnd
   * \cgalNamedParamsEnd
   *
   * @return the length of `h`. The return type `FT` is a number type. It is
-  * either deduced from the `geom_traits` \ref pmp_namedparameters "Named Parameters" if provided,
+  * either deduced from the `geom_traits` \ref bgl_namedparameters "Named Parameters" if provided,
   * or the geometric traits class deduced from the point property map
   * of `pmesh`.
   *
@@ -136,22 +145,31 @@ edge_length(typename boost::graph_traits
   * that contains a given halfedge.
   *
   * @tparam PolygonMesh a model of `HalfedgeGraph`
-  * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+  * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
   *
   * @param h a halfedge of the border polyline of which the length is computed
   * @param pmesh the polygon mesh to which `h` belongs
-  * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+  * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
   *
   * \cgalNamedParamsBegin
-  *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
-  *   If this parameter is omitted, an internal property map for
-  *   `CGAL::vertex_point_t` must be available in `PolygonMesh`\cgalParamEnd
-  *    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+  *   \cgalParamNBegin{vertex_point_map}
+  *     \cgalParamDescription{a property map associating points to the vertices of `pmesh`}
+  *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+  *                    as key type and `%Point_3` as value type}
+  *     \cgalParamDefault{`boost::get(CGAL::vertex_point, pmesh)`}
+  *   \cgalParamNEnd
+  *
+ *   \cgalParamNBegin{geom_traits}
+ *     \cgalParamDescription{an instance of a geometric traits class}
+ *     \cgalParamType{a class model of `Kernel`}
+ *     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+ *     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+ *   \cgalParamNEnd
   * \cgalNamedParamsEnd
   *
   * @return the length of the sequence of border edges of `face(h, pmesh)`.
   * The return type `FT` is a number type. It is
-  * either deduced from the `geom_traits` \ref pmp_namedparameters "Named Parameters" if provided,
+  * either deduced from the `geom_traits` \ref bgl_namedparameters "Named Parameters" if provided,
   * or the geometric traits class deduced from the point property map
   * of `pmesh`.
   *
@@ -198,16 +216,25 @@ face_border_length(typename boost::graph
   * a halfedge that is part of this border and the length of this border.
   *
   * @tparam PolygonMesh a model of `HalfedgeGraph`
-  * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+  * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
   *
   * @param pmesh the polygon mesh
-  * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+  * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
   *
   * \cgalNamedParamsBegin
-  *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
-  *   If this parameter is omitted, an internal property map for
-  *   `CGAL::vertex_point_t` must be available in `PolygonMesh`\cgalParamEnd
-  *    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+  *   \cgalParamNBegin{vertex_point_map}
+  *     \cgalParamDescription{a property map associating points to the vertices of `pmesh`}
+  *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+  *                    as key type and `%Point_3` as value type}
+  *     \cgalParamDefault{`boost::get(CGAL::vertex_point, pmesh)`}
+  *   \cgalParamNEnd
+  *
+ *   \cgalParamNBegin{geom_traits}
+ *     \cgalParamDescription{an instance of a geometric traits class}
+ *     \cgalParamType{a class model of `Kernel`}
+ *     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+ *     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+ *   \cgalParamNEnd
   * \cgalNamedParamsEnd
   *
   * @return a pair composed of two members:
@@ -216,7 +243,7 @@ face_border_length(typename boost::graph
   *     deduced from the graph traits corresponding to the type `PolygonMesh`.
   *   - `second`: the length of the longest border
   *     The return type `FT` is a number type. It is
-  *     either deduced from the `geom_traits` \ref pmp_namedparameters "Named Parameters" if provided,
+  *     either deduced from the `geom_traits` \ref bgl_namedparameters "Named Parameters" if provided,
   *     or the geometric traits class deduced from the point property map
   *     of `pmesh`
   *
@@ -278,24 +305,33 @@ longest_border(const PolygonMesh& pmesh)
   * triangulated surface mesh.
   *
   * @tparam TriangleMesh a model of `HalfedgeGraph`
-  * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+  * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
   *
   * @param f the face of which the area is computed
   * @param tmesh the triangulated surface mesh to which `f` belongs
-  * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+  * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
   *
   * \cgalNamedParamsBegin
-  *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
-  *   If this parameter is omitted, an internal property map for
-  *   `CGAL::vertex_point_t` must be available in `TriangleMesh`\cgalParamEnd
-  *  \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+  *   \cgalParamNBegin{vertex_point_map}
+  *     \cgalParamDescription{a property map associating points to the vertices of `tmesh`}
+  *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+  *                    as key type and `%Point_3` as value type}
+  *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tmesh)`}
+  *   \cgalParamNEnd
+  *
+ *   \cgalParamNBegin{geom_traits}
+ *     \cgalParamDescription{an instance of a geometric traits class}
+ *     \cgalParamType{a class model of `Kernel`}
+ *     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+ *     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+ *   \cgalParamNEnd
   * \cgalNamedParamsEnd
   *
   * @pre `f != boost::graph_traits<TriangleMesh>::%null_face()`
   *
   * @return the area of `f`.
   * The return type `FT` is a number type. It is
-  * either deduced from the `geom_traits` \ref pmp_namedparameters "Named Parameters" if provided,
+  * either deduced from the `geom_traits` \ref bgl_namedparameters "Named Parameters" if provided,
   * or the geometric traits class deduced from the point property map
   * of `tmesh`.
   *
@@ -326,7 +362,8 @@ face_area(typename boost::graph_traits<T
   halfedge_descriptor hd = halfedge(f, tmesh);
   halfedge_descriptor nhd = next(hd, tmesh);
 
-  typename GetGeomTraits<TriangleMesh, CGAL_PMP_NP_CLASS>::type traits;
+  typedef typename GetGeomTraits<TriangleMesh, CGAL_PMP_NP_CLASS>::type GT;
+  GT traits = choose_parameter<GT>(get_parameter(np, internal_np::geom_traits));
 
   return approximate_sqrt(traits.compute_squared_area_3_object()(get(vpm, source(hd, tmesh)),
                                                                  get(vpm, target(hd, tmesh)),
@@ -351,22 +388,31 @@ face_area(typename boost::graph_traits<T
           model of `Range`.
           Its iterator type is `InputIterator`.
   * @tparam TriangleMesh a model of `HalfedgeGraph`
-  * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+  * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
   *
   * @param face_range the range of faces of which the area is computed
   * @param tmesh the triangulated surface mesh to which the faces of `face_range` belong
-  * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+  * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
   *
   * \cgalNamedParamsBegin
-  *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
-  *   If this parameter is omitted, an internal property map for
-  *   `CGAL::vertex_point_t` must be available in `TriangleMesh`\cgalParamEnd
-  *  \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel` \cgalParamEnd
+  *   \cgalParamNBegin{vertex_point_map}
+  *     \cgalParamDescription{a property map associating points to the vertices of `tmesh`}
+  *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+  *                    as key type and `%Point_3` as value type}
+  *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tmesh)`}
+  *   \cgalParamNEnd
+  *
+ *   \cgalParamNBegin{geom_traits}
+ *     \cgalParamDescription{an instance of a geometric traits class}
+ *     \cgalParamType{a class model of `Kernel`}
+ *     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+ *     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+ *   \cgalParamNEnd
   * \cgalNamedParamsEnd
   *
   * @return sum of face areas of `faces`.
   * The return type `FT` is a number type. It is
-  * either deduced from the `geom_traits` \ref pmp_namedparameters "Named Parameters" if provided,
+  * either deduced from the `geom_traits` \ref bgl_namedparameters "Named Parameters" if provided,
   * or the geometric traits class deduced from the point property map
   * of `tmesh`.
   *
@@ -412,21 +458,30 @@ area(FaceRange face_range, const Triangl
   * computes the surface area of a triangulated surface mesh.
   *
   * @tparam TriangleMesh a model of `HalfedgeGraph`
-  * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+  * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
   *
   * @param tmesh the triangulated surface mesh
-  * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+  * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
   *
   * \cgalNamedParamsBegin
-  *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
-  *   If this parameter is omitted, an internal property map for
-  *   `CGAL::vertex_point_t` must be available in `TriangleMesh`\cgalParamEnd
-  *  \cgalParamBegin{geom_traits}an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+  *   \cgalParamNBegin{vertex_point_map}
+  *     \cgalParamDescription{a property map associating points to the vertices of `tmesh`}
+  *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+  *                    as key type and `%Point_3` as value type}
+  *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tmesh)`}
+  *   \cgalParamNEnd
+  *
+ *   \cgalParamNBegin{geom_traits}
+ *     \cgalParamDescription{an instance of a geometric traits class}
+ *     \cgalParamType{a class model of `Kernel`}
+ *     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+ *     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+ *   \cgalParamNEnd
   * \cgalNamedParamsEnd
   *
   * @return the surface area of `tmesh`.
   * The return type `FT` is a number type. It is
-  * either deduced from the `geom_traits` \ref pmp_namedparameters "Named Parameters" if provided,
+  * either deduced from the `geom_traits` \ref bgl_namedparameters "Named Parameters" if provided,
   * or the geometric traits class deduced from the point property map
   * of `tmesh`.
   *
@@ -462,23 +517,32 @@ area(const TriangleMesh& tmesh)
   * a closed triangulated surface mesh.
   *
   * @tparam TriangleMesh a model of `HalfedgeGraph`
-  * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+  * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
   *
   * @param tmesh the closed triangulated surface mesh bounding the volume
-  * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+  * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
   *
   * @pre `tmesh` is closed
   *
   * \cgalNamedParamsBegin
-  *  \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
-  *   If this parameter is omitted, an internal property map for
-  *   `CGAL::vertex_point_t` must be available in `TriangleMesh`\cgalParamEnd
-  *  \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+  *   \cgalParamNBegin{vertex_point_map}
+  *     \cgalParamDescription{a property map associating points to the vertices of `tmesh`}
+  *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+  *                    as key type and `%Point_3` as value type}
+  *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tmesh)`}
+  *   \cgalParamNEnd
+  *
+ *   \cgalParamNBegin{geom_traits}
+ *     \cgalParamDescription{an instance of a geometric traits class}
+ *     \cgalParamType{a class model of `Kernel`}
+ *     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+ *     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+ *   \cgalParamNEnd
   * \cgalNamedParamsEnd
   *
   * @return the volume bounded by `tmesh`.
   * The return type `FT` is a number type. It is
-  * either deduced from the `geom_traits` \ref pmp_namedparameters "Named Parameters" if provided,
+  * either deduced from the `geom_traits` \ref bgl_namedparameters "Named Parameters" if provided,
   * or the geometric traits class deduced from the point property map
   * of `tmesh`.
   */
@@ -500,7 +564,7 @@ volume(const TriangleMesh& tmesh,
 
   typename GetVertexPointMap<TriangleMesh, CGAL_PMP_NP_CLASS>::const_type
       vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
-                         get_const_property_map(CGAL::vertex_point, tmesh));
+                             get_const_property_map(CGAL::vertex_point, tmesh));
   typename GetGeomTraits<TriangleMesh, CGAL_PMP_NP_CLASS>::type::Point_3 origin(0, 0, 0);
 
   typedef typename boost::graph_traits<TriangleMesh>::face_descriptor face_descriptor;
@@ -534,23 +598,32 @@ volume(const TriangleMesh& tmesh)
   * computes the aspect ratio of a face of a given triangulated surface mesh.
   *
   * @tparam TriangleMesh a model of `HalfedgeGraph`
-  * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+  * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
   *
   * @param f the face of which the aspect ratio is computed
   * @param tmesh the triangulated surface mesh to which `f` belongs
-  * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+  * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
   *
   * \cgalNamedParamsBegin
-  *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
-  *   If this parameter is omitted, an internal property map for
-  *   `CGAL::vertex_point_t` must be available in `TriangleMesh`\cgalParamEnd
-  *  \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+  *   \cgalParamNBegin{vertex_point_map}
+  *     \cgalParamDescription{a property map associating points to the vertices of `tmesh`}
+  *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+  *                    as key type and `%Point_3` as value type}
+  *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tmesh)`}
+  *   \cgalParamNEnd
+  *
+ *   \cgalParamNBegin{geom_traits}
+ *     \cgalParamDescription{an instance of a geometric traits class}
+ *     \cgalParamType{a class model of `Kernel`}
+ *     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+ *     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+ *   \cgalParamNEnd
   * \cgalNamedParamsEnd
   *
   * @pre `f != boost::graph_traits<TriangleMesh>::%null_face()`
   *
   * @return the aspect ratio of `f`. The return type `FT` is a number type. It is
-  * either deduced from the `geom_traits` \ref pmp_namedparameters "Named Parameters" if provided,
+  * either deduced from the `geom_traits` \ref bgl_namedparameters "Named Parameters" if provided,
   * or the geometric traits class deduced from the point property map of `tmesh`.
   *
   */
@@ -577,11 +650,11 @@ face_aspect_ratio(typename boost::graph_
 
   typename GetVertexPointMap<TriangleMesh, CGAL_PMP_NP_CLASS>::const_type
       vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
-                         get_const_property_map(CGAL::vertex_point, tmesh));
+                             get_const_property_map(CGAL::vertex_point, tmesh));
 
   halfedge_descriptor h = halfedge(f, tmesh);
 
-  Geom_traits gt = choose_parameter(get_parameter(np, internal_np::geom_traits), Geom_traits());
+  Geom_traits gt = choose_parameter<Geom_traits>(get_parameter(np, internal_np::geom_traits));
 
 #if 0
   const FT sq_triangle_area = gt.compute_squared_area_3_object()(get(vpm, source(h, tmesh)),
@@ -645,18 +718,27 @@ face_aspect_ratio(typename boost::graph_
   * a closed triangulated surface mesh.
   *
   * @tparam TriangleMesh a model of `FaceListGraph`
-  * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+  * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
   *
   * @param tmesh the closed triangulated surface mesh bounding the volume
-  * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+  * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
   *
   * @pre `tmesh` is closed
   *
   * \cgalNamedParamsBegin
-  *  \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
-  *   If this parameter is omitted, an internal property map for
-  *   `CGAL::vertex_point_t` must be available in `TriangleMesh`\cgalParamEnd
-  *  \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+  *   \cgalParamNBegin{vertex_point_map}
+  *     \cgalParamDescription{a property map associating points to the vertices of `tmesh`}
+  *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+  *                    as key type and `%Point_3` as value type}
+  *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tmesh)`}
+  *   \cgalParamNEnd
+  *
+ *   \cgalParamNBegin{geom_traits}
+ *     \cgalParamDescription{an instance of a geometric traits class}
+ *     \cgalParamType{a class model of `Kernel`}
+ *     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+ *     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+ *   \cgalParamNEnd
   * \cgalNamedParamsEnd
   *
   * @return the centroid of the domain bounded by `tmesh`.
@@ -679,7 +761,7 @@ centroid(const TriangleMesh& tmesh, cons
 
   typedef typename GetVertexPointMap<TriangleMesh, CGAL_PMP_NP_CLASS>::const_type Vpm;
   Vpm vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
-                         get_const_property_map(CGAL::vertex_point, tmesh));
+                             get_const_property_map(CGAL::vertex_point, tmesh));
 
   typedef typename GetGeomTraits<TriangleMesh, CGAL_PMP_NP_CLASS>::type Kernel;
   typedef typename Kernel::Point_3                                      Point_3;
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/merge_border_vertices.h 5.1-1/include/CGAL/Polygon_mesh_processing/merge_border_vertices.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/merge_border_vertices.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/merge_border_vertices.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,12 +3,13 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/merge_border_vertices.h $
-// $Id: merge_border_vertices.h 38abfdf 2020-03-18T08:51:09+01:00 Mael
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/merge_border_vertices.h $
+// $Id: merge_border_vertices.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
 // Author(s)     : Sebastien Loriot
+//                 Mael Rouxel-Labbé
 
 #ifndef CGAL_POLYGON_MESH_PROCESSING_MERGE_BORDER_VERTICES_H
 #define CGAL_POLYGON_MESH_PROCESSING_MERGE_BORDER_VERTICES_H
@@ -261,19 +262,21 @@ void merge_vertices_in_range(const Halfe
 /// merges identical vertices around a cycle of boundary edges.
 ///
 /// @tparam PolygonMesh a model of `FaceListGraph` and `MutableFaceGraph`.
-/// @tparam NamedParameter a sequence of \ref pmp_namedparameters "Named Parameters".
+/// @tparam NamedParameter a sequence of \ref bgl_namedparameters "Named Parameters".
 ///
 /// @param h a halfedge that belongs to a boundary cycle.
 /// @param pm the polygon mesh which contains the boundary cycle.
-/// @param np optional parameter of \ref pmp_namedparameters "Named Parameters" listed below.
+/// @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 ///
 /// \cgalNamedParamsBegin
-/// \cgalParamBegin{vertex_point_map}
-///   the property map with the points associated to the vertices of `pm`.
-///    If this parameter is omitted, an internal property map for
-///  `CGAL::vertex_point_t` should be available in `PolygonMesh`
-/// \cgalParamEnd
+///   \cgalParamNBegin{vertex_point_map}
+///     \cgalParamDescription{a property map associating points to the vertices of `pm`}
+///     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+///                    as key type and `%Point_3` as value type}
+///     \cgalParamDefault{`boost::get(CGAL::vertex_point, pm)`}
+///   \cgalParamNEnd
 /// \cgalNamedParamsEnd
+///
 template <class PolygonMesh, class NamedParameter>
 void merge_duplicated_vertices_in_boundary_cycle(
         typename boost::graph_traits<PolygonMesh>::halfedge_descriptor h,
@@ -315,17 +318,18 @@ void merge_duplicated_vertices_in_bounda
 /// extracts boundary cycles and merges the duplicated vertices of each cycle.
 ///
 /// @tparam PolygonMesh a model of `FaceListGraph` and `MutableFaceGraph`.
-/// @tparam NamedParameter a sequence of \ref pmp_namedparameters "Named Parameters".
+/// @tparam NamedParameter a sequence of \ref bgl_namedparameters "Named Parameters".
 ///
 /// @param pm the polygon mesh which contains the cycles.
-/// @param np optional parameter of \ref pmp_namedparameters "Named Parameters" listed below.
+/// @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 ///
 /// \cgalNamedParamsBegin
-/// \cgalParamBegin{vertex_point_map}
-///   the property map with the points associated to the vertices of `pm`.
-///    If this parameter is omitted, an internal property map for
-///  `CGAL::vertex_point_t` should be available in `PolygonMesh`
-/// \cgalParamEnd
+///   \cgalParamNBegin{vertex_point_map}
+///     \cgalParamDescription{a property map associating points to the vertices of `pm`}
+///     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+///                    as key type and `%Point_3` as value type}
+///     \cgalParamDefault{`boost::get(CGAL::vertex_point, pm)`}
+///   \cgalParamNEnd
 /// \cgalNamedParamsEnd
 ///
 /// \sa `merge_duplicated_vertices_in_boundary_cycle()`
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/orientation.h 5.1-1/include/CGAL/Polygon_mesh_processing/orientation.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/orientation.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/orientation.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,12 +3,12 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/orientation.h $
-// $Id: orientation.h 2c5826d 2020-01-17T16:29:49+01:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/orientation.h $
+// $Id: orientation.h d567a0d 2020-07-16T09:52:32+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
-// Author(s)     : Ilker O. Yaz
+// Author(s)     : Sebastien Loriot and Ilker O. Yaz
 
 
 #ifndef CGAL_ORIENT_POLYGON_MESH_H
@@ -19,16 +19,22 @@
 
 #include <algorithm>
 #include <CGAL/Polygon_mesh_processing/connected_components.h>
+#include <CGAL/Polygon_mesh_processing/stitch_borders.h>
 #include <CGAL/Polygon_mesh_processing/compute_normal.h>
 #include <CGAL/Polygon_mesh_processing/internal/named_function_params.h>
 #include <CGAL/Polygon_mesh_processing/internal/named_params_helper.h>
+#include <CGAL/Polygon_mesh_processing/self_intersections.h>
 #include <CGAL/Side_of_triangle_mesh.h>
 #include <CGAL/Projection_traits_xy_3.h>
 #include <CGAL/boost/graph/helpers.h>
 #include <CGAL/boost/graph/iterator.h>
+#include <CGAL/utility.h>
 
 #include <boost/unordered_set.hpp>
 #include <boost/dynamic_bitset.hpp>
+#include <boost/ref.hpp>
+
+#include <functional>
 namespace CGAL {
 
 namespace Polygon_mesh_processing {
@@ -60,18 +66,18 @@ namespace internal{
                            const PolygonMesh& pmesh,
                            const NamedParameters& np)
   {
-
+    using parameters::choose_parameter;
     using parameters::get_parameter;
 
     CGAL_assertion(halfedge(v_max, pmesh)!=boost::graph_traits<PolygonMesh>::null_halfedge());
 
     //VertexPointMap
     typedef typename GetVertexPointMap<PolygonMesh, NamedParameters>::const_type VPMap;
-    VPMap vpmap = CGAL::parameters::choose_parameter(get_parameter(np, internal_np::vertex_point),
-                               get_const_property_map(vertex_point, pmesh));
+    VPMap vpmap = choose_parameter(get_parameter(np, internal_np::vertex_point),
+                                   get_const_property_map(vertex_point, pmesh));
     //Kernel
     typedef typename GetGeomTraits<PolygonMesh, NamedParameters>::type GT;
-    GT gt = CGAL::parameters::choose_parameter(get_parameter(np, internal_np::geom_traits), GT());
+    GT gt = choose_parameter<GT>(get_parameter(np, internal_np::geom_traits));
 
     //among the incoming edges of `v_max`, find one edge `e` with the minimal slope
     typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor halfedge_descriptor;
@@ -135,85 +141,101 @@ namespace internal{
 
 /**
  * \ingroup PMP_orientation_grp
- * tests whether a closed polygon mesh has a positive orientation.
- * A closed polygon mesh is considered to have a positive orientation if the normal vectors
- * to all its faces point outside the domain bounded by the polygon mesh.
+ * tests whether a closed triangle mesh has a positive orientation.
+ * A closed triangle mesh is considered to have a positive orientation if the normal vectors
+ * to all its faces point outside the domain bounded by the triangle mesh.
  * The normal vector to each face is chosen pointing on the side of the face
  * where its sequence of vertices is seen counterclockwise.
- * @pre `CGAL::is_closed(pmesh)`
- * @pre If `pmesh` contains several connected components, they are oriented consistently.
+ * @pre `CGAL::is_closed(tm)`
+ * @pre `CGAL::is_triangle_mesh(tm)`
+ * @pre If `tm` contains several connected components, they are oriented consistently.
  *      In other words, the answer to this predicate would be the same for each
  *      isolated connected component.
  *
- * @tparam PolygonMesh a model of `FaceListGraph`
- * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+ * @tparam TriangleMesh a model of `FaceListGraph`
+ * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
  *
- * @param pmesh the closed polygon mesh to be tested
- * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+ * @param tm the closed triangle mesh free from self-intersections to be tested
+ * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh` \cgalParamEnd
- *    \cgalParamBegin{geom_traits} a geometric traits class instance \cgalParamEnd
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `tm`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+ *                    as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{geom_traits}
+ *     \cgalParamDescription{an instance of a geometric traits class}
+ *     \cgalParamType{a class model of `Kernel`}
+ *     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+ *     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  *
- * \todo code : The following only handles polyhedron with one connected component
- *       the code, the sample example and the plugin must be updated.
+ * \note This function is only doing an orientation test for one connected component of `tm`.
+ *       For performance reasons, it is left to the user to call the function `does_bound_a_volume()`
+ *       on a triangulated version of `tm` to ensure the result returned is relevant.
+ *       For advanced usages, the function `volume_connected_components()` should be used instead.
  *
  * \sa `CGAL::Polygon_mesh_processing::reverse_face_orientations()`
  */
-template<typename PolygonMesh, typename NamedParameters>
-bool is_outward_oriented(const PolygonMesh& pmesh,
+template<typename TriangleMesh, typename NamedParameters>
+bool is_outward_oriented(const TriangleMesh& tm,
                          const NamedParameters& np)
 {
-  CGAL_warning(CGAL::is_closed(pmesh));
-  CGAL_precondition(CGAL::is_valid_polygon_mesh(pmesh));
+  CGAL_warning(CGAL::is_closed(tm));
+  CGAL_warning(CGAL::is_triangle_mesh(tm));
+  CGAL_precondition(CGAL::is_valid_polygon_mesh(tm));
 
 #ifdef CGAL_PMP_DEBUG_CODE
-  //check for empty pmesh
-  CGAL_warning(faces(pmesh).first != faces(pmesh).second);
+  //check for empty tm
+  CGAL_warning(faces(tm).first != faces(tm).second);
 #endif
 
-  if (faces(pmesh).first == faces(pmesh).second)
+  if (faces(tm).first == faces(tm).second)
     return true;
 
 
+  using parameters::choose_parameter;
   using parameters::get_parameter;
 
   //VertexPointMap
-  typedef typename GetVertexPointMap<PolygonMesh, NamedParameters>::const_type VPMap;
-  VPMap vpmap = CGAL::parameters::choose_parameter(get_parameter(np, internal_np::vertex_point),
-                             get_const_property_map(vertex_point, pmesh));
+  typedef typename GetVertexPointMap<TriangleMesh, NamedParameters>::const_type VPMap;
+  VPMap vpmap = choose_parameter(get_parameter(np, internal_np::vertex_point),
+                                 get_const_property_map(vertex_point, tm));
   //Kernel
-  typedef typename GetGeomTraits<PolygonMesh, NamedParameters>::type GT;
-  GT gt = CGAL::parameters::choose_parameter(get_parameter(np, internal_np::geom_traits), GT());
+  typedef typename GetGeomTraits<TriangleMesh, NamedParameters>::type GT;
+  GT gt = choose_parameter<GT>(get_parameter(np, internal_np::geom_traits));
 
   //find the vertex with maximal z coordinate
   internal::Compare_vertex_points_z_3<GT, VPMap> less_z(vpmap, gt);
-  typename boost::graph_traits<PolygonMesh>::vertex_descriptor v_max = *(vertices(pmesh).first);
-  for (typename boost::graph_traits<PolygonMesh>::vertex_iterator
-          vit=std::next(vertices(pmesh).first), vit_end = vertices(pmesh).second;
+  typename boost::graph_traits<TriangleMesh>::vertex_descriptor v_max = *(vertices(tm).first);
+  for (typename boost::graph_traits<TriangleMesh>::vertex_iterator
+          vit=std::next(vertices(tm).first), vit_end = vertices(tm).second;
           vit!=vit_end; ++vit)
   {
     // skip isolated vertices
-    if (halfedge(*vit, pmesh)==boost::graph_traits<PolygonMesh>::null_halfedge())
+    if (halfedge(*vit, tm)==boost::graph_traits<TriangleMesh>::null_halfedge())
       continue;
     if( less_z(v_max, *vit) )
       v_max=*vit;
   }
 
   // only isolated vertices
-  if (halfedge(v_max, pmesh)==boost::graph_traits<PolygonMesh>::null_halfedge())
+  if (halfedge(v_max, tm)==boost::graph_traits<TriangleMesh>::null_halfedge())
     return true;
 
-  return internal::is_outward_oriented(v_max, pmesh, np);
+  return internal::is_outward_oriented(v_max, tm, np);
 }
 
 ///\cond SKIP_IN_MANUAL
 
-template<typename PolygonMesh>
-bool is_outward_oriented(const PolygonMesh& pmesh)
+template<typename TriangleMesh>
+bool is_outward_oriented(const TriangleMesh& tm)
 {
-  return is_outward_oriented(pmesh,
+  return is_outward_oriented(tm,
     CGAL::Polygon_mesh_processing::parameters::all_default());
 }
 
@@ -340,97 +362,6 @@ void reverse_face_orientations(const Fac
       }
     }
 }
-namespace internal {
-
-template <class Kernel, class TriangleMesh, class VD, class Fid_map, class Vpm>
-void recursive_orient_volume_ccs( TriangleMesh& tm,
-                                  Vpm& vpm,
-                                  Fid_map& fid_map,
-                                  const std::vector<VD>& xtrm_vertices,
-                                  boost::dynamic_bitset<>& cc_handled,
-                                  const std::vector<std::size_t>& face_cc,
-                                  std::size_t xtrm_cc_id,
-                                  bool is_parent_outward_oriented)
-{
-  typedef boost::graph_traits<TriangleMesh> Graph_traits;
-  typedef typename Graph_traits::face_descriptor face_descriptor;
-  typedef Side_of_triangle_mesh<TriangleMesh, Kernel, Vpm> Side_of_tm;
-  std::vector<face_descriptor> cc_faces;
-  for(face_descriptor fd : faces(tm))
-  {
-    if(face_cc[get(fid_map, fd)]==xtrm_cc_id)
-      cc_faces.push_back(fd);
-  }
-// first check that the orientation of the current cc is consistant with its
-// parent cc containing it
-  bool new_is_parent_outward_oriented = internal::is_outward_oriented(
-         xtrm_vertices[xtrm_cc_id], tm, parameters::vertex_point_map(vpm));
-  if (new_is_parent_outward_oriented==is_parent_outward_oriented)
-  {
-    Polygon_mesh_processing::reverse_face_orientations(cc_faces, tm);
-    new_is_parent_outward_oriented = !new_is_parent_outward_oriented;
-  }
-  cc_handled.set(xtrm_cc_id);
-
-  std::size_t nb_cc = cc_handled.size();
-
-// get all cc that are inside xtrm_cc_id
-
-  typename Side_of_tm::AABB_tree aabb_tree(cc_faces.begin(), cc_faces.end(),
-                                           tm, vpm);
-  Side_of_tm side_of_cc(aabb_tree);
-
-  std::vector<std::size_t> cc_inside;
-  for(std::size_t id=0; id<nb_cc; ++id)
-  {
-    if (cc_handled.test(id)) continue;
-    if (side_of_cc(get(vpm,xtrm_vertices[id]))==ON_BOUNDED_SIDE)
-      cc_inside.push_back(id);
-  }
-
-// check whether we need another recursion for cc inside xtrm_cc_id
-  if (!cc_inside.empty())
-  {
-    std::size_t new_xtrm_cc_id = cc_inside.front();
-    boost::dynamic_bitset<> new_cc_handled(nb_cc,0);
-    new_cc_handled.set();
-    new_cc_handled.reset(new_xtrm_cc_id);
-    cc_handled.set(new_xtrm_cc_id);
-
-    std::size_t nb_candidates = cc_inside.size();
-    for (std::size_t i=1;i<nb_candidates;++i)
-    {
-      std::size_t candidate = cc_inside[i];
-      if(get(vpm,xtrm_vertices[candidate]).z() >
-         get(vpm,xtrm_vertices[new_xtrm_cc_id]).z()) new_xtrm_cc_id=candidate;
-      new_cc_handled.reset(candidate);
-      cc_handled.set(candidate);
-    }
-
-    internal::recursive_orient_volume_ccs<Kernel>(
-           tm, vpm, fid_map, xtrm_vertices, new_cc_handled, face_cc,
-           new_xtrm_cc_id, new_is_parent_outward_oriented);
-  }
-
-// now explore remaining cc included in the same cc as xtrm_cc_id
-  boost::dynamic_bitset<> cc_not_handled = ~cc_handled;
-  std::size_t new_xtrm_cc_id = cc_not_handled.find_first();
-  if (new_xtrm_cc_id == cc_not_handled.npos) return ;
-
-  for (std::size_t candidate = cc_not_handled.find_next(new_xtrm_cc_id);
-                   candidate < cc_not_handled.npos;
-                   candidate = cc_not_handled.find_next(candidate))
-  {
-     if(get(vpm,xtrm_vertices[candidate]).z() > get(vpm,xtrm_vertices[new_xtrm_cc_id]).z())
-        new_xtrm_cc_id = candidate;
-  }
-
-  internal::recursive_orient_volume_ccs<Kernel>(
-            tm, vpm, fid_map, xtrm_vertices, cc_handled, face_cc,
-            new_xtrm_cc_id, is_parent_outward_oriented);
-}
-
-}//end internal
 
 /**
 * \ingroup PMP_orientation_grp
@@ -438,54 +369,66 @@ void recursive_orient_volume_ccs( Triang
 * inward or outward oriented.
 *
 * @tparam TriangleMesh a model of `FaceListGraph` and `MutableFaceGraph` .
-*                      If `TriangleMesh` has an internal property map for `CGAL::face_index_t`,
-*                      as a named parameter, then it must be initialized.
-* @tparam NamedParameters a sequence of \ref pmp_namedparameters
+* @tparam NamedParameters a sequence of \ref bgl_namedparameters
 *
 * @param tm a closed triangulated surface mesh
-* @param np optional sequence of \ref pmp_namedparameters among the ones listed below
+* @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+*
+* \pre `CGAL::is_closed(tm)`
 *
 * \cgalNamedParamsBegin
-*   \cgalParamBegin{vertex_point_map}
-*     the property map with the points associated to the vertices of `tm`.
-*     If this parameter is omitted, an internal property map for
-*     `CGAL::vertex_point_t` must be available in `TriangleMesh`
-*   \cgalParamEnd
-*   \cgalParamBegin{face_index_map}
-*     a property map containing the index of each face of `tm`.
-*   \cgalParamEnd
-*   \cgalParamBegin{outward_orientation}
-*     if set to `true` (default) indicates that each connected component will be outward oriented,
-*     (inward oriented if `false`).
-*   \cgalParamEnd
+*   \cgalParamNBegin{vertex_point_map}
+*     \cgalParamDescription{a property map associating points to the vertices of `tm`}
+*     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+*                    as key type and `%Point_3` as value type}
+*     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{geom_traits}
+*     \cgalParamDescription{an instance of a geometric traits class}
+*     \cgalParamType{a class model of `Kernel`}
+*     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+*     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{face_index_map}
+*     \cgalParamDescription{a property map associating to each face of `tm` a unique index between `0` and `num_faces(tm) - 1`}
+*     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%face_descriptor`
+*                    as key type and `std::size_t` as value type}
+*     \cgalParamDefault{an automatically indexed internal map}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{outward_orientation}
+*     \cgalParamDescription{If `true`, each connected component will be outward oriented (and inward oriented if `false`).}
+*     \cgalParamType{Boolean}
+*     \cgalParamDefault{`true`}
+*   \cgalParamNEnd
 * \cgalNamedParamsEnd
 */
 template<class TriangleMesh, class NamedParameters>
-void orient(TriangleMesh& tm, const NamedParameters& np)
+void orient(TriangleMesh& tm,
+            const NamedParameters& np)
 {
-  typedef boost::graph_traits<TriangleMesh> Graph_traits;
-  typedef typename Graph_traits::vertex_descriptor vertex_descriptor;
-  typedef typename Graph_traits::face_descriptor face_descriptor;
-  typedef typename Graph_traits::halfedge_descriptor halfedge_descriptor;
-  typedef typename GetVertexPointMap<TriangleMesh,
-      NamedParameters>::const_type Vpm;
-  typedef typename GetFaceIndexMap<TriangleMesh,
-      NamedParameters>::const_type Fid_map;
+  typedef boost::graph_traits<TriangleMesh>                                        Graph_traits;
+  typedef typename Graph_traits::vertex_descriptor                                 vertex_descriptor;
+  typedef typename Graph_traits::face_descriptor                                   face_descriptor;
+  typedef typename Graph_traits::halfedge_descriptor                               halfedge_descriptor;
+  typedef typename GetVertexPointMap<TriangleMesh, NamedParameters>::const_type    Vpm;
+  typedef typename GetInitializedFaceIndexMap<TriangleMesh, NamedParameters>::type FaceIndexMap;
 
   CGAL_assertion(is_triangle_mesh(tm));
   CGAL_assertion(is_valid_polygon_mesh(tm));
   CGAL_assertion(is_closed(tm));
 
+  using parameters::choose_parameter;
   using parameters::get_parameter;
 
-  bool orient_outward = CGAL::parameters::choose_parameter(
-                          get_parameter(np, internal_np::outward_orientation),true);
+  bool orient_outward = choose_parameter(get_parameter(np, internal_np::outward_orientation),true);
 
-  Vpm vpm = CGAL::parameters::choose_parameter(get_parameter(np, internal_np::vertex_point),
-                         get_const_property_map(boost::vertex_point, tm));
+  Vpm vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
+                             get_const_property_map(boost::vertex_point, tm));
 
-  Fid_map fid_map = CGAL::parameters::choose_parameter(get_parameter(np, internal_np::face_index),
-                                 get_const_property_map(boost::face_index, tm));
+  FaceIndexMap fid_map = CGAL::get_initialized_face_index_map(tm, np);
 
   std::vector<std::size_t> face_cc(num_faces(tm), std::size_t(-1));
 
@@ -535,6 +478,832 @@ void orient(TriangleMesh& tm)
   orient(tm, parameters::all_default());
 }
 
+/*!
+ * \ingroup PMP_orientation_grp
+ * Enumeration type used to indicate the status of a set of faces
+ * classified by the function `volume_connected_components()`.
+ * The set of faces defines either a volume connected connected component
+ * in the case of `VALID_VOLUME` or a surface connected component otherwise.
+ */
+enum Volume_error_code { VALID_VOLUME, ///< The set of faces bounds a volume
+                         SURFACE_WITH_SELF_INTERSECTIONS, ///< The set of faces is self-intersecting
+                         VOLUME_INTERSECTION, ///< The set of faces intersects another surface connected component
+                         INCOMPATIBLE_ORIENTATION ///< The set of faces is included in a volume but has an incompatible orientation
+                       };
+namespace internal {
+
+// helper function to copy data
+template<class T, class RefToContainer>
+void copy_container_content(
+  const std::vector<T>& vec,
+  RefToContainer ref_wrapper)
+{
+  ref_wrapper.get().reserve(vec.size());
+  for(const T& t : vec)
+  {
+   ref_wrapper.get().push_back(t);
+  }
+}
+
+template<class T>
+void copy_container_content(
+  std::vector<T>& vec,
+  std::reference_wrapper<std::vector<T> > ref_wrapper)
+{
+  vec.swap(ref_wrapper.get());
+}
+
+template<class T>
+void copy_container_content(
+  std::vector<T>& vec,
+  boost::reference_wrapper<std::vector<T> > ref_wrapper)
+{
+  vec.swap(ref_wrapper.get());
+}
+
+template<class T>
+inline
+void copy_container_content(
+  std::vector<T>&,
+  internal_np::Param_not_found)
+{}
+
+template <class RefToContainer>
+void copy_nested_parents(
+  const std::vector< std::vector<std::size_t> >& nested_parents,
+  RefToContainer ref_to_vector)
+{
+  typedef typename RefToContainer::type Container;
+  typedef typename Container::value_type Container_value;
+
+  ref_to_vector.get().reserve(nested_parents.size());
+  for(const auto& t : nested_parents)
+  {
+    Container_value c;
+    c.reserve(t.size());
+    for(const std::size_t& val : t)
+      c.push_back(val);
+    ref_to_vector.get().push_back(c);
+  }
+}
+
+inline
+void copy_nested_parents(
+  std::vector< std::vector<std::size_t> >& nested_parents,
+  std::reference_wrapper<std::vector< std::vector<std::size_t> > > ref_to_vector)
+{
+  nested_parents.swap(ref_to_vector.get());
+}
+
+inline
+void copy_nested_parents(
+  std::vector< std::vector<std::size_t> >& nested_parents,
+  boost::reference_wrapper<std::vector< std::vector<std::size_t> > > ref_to_vector)
+{
+  nested_parents.swap(ref_to_vector.get());
+}
+
+inline
+void copy_nested_parents(
+  std::vector< std::vector<std::size_t> >&,
+  internal_np::Param_not_found)
+{}
+
+
+// helper function for setting id maps
+template <class TriangleMesh, class FaceIndexMap, class FaceCCIdMap>
+void set_f_cc_id(
+  const std::vector<std::size_t>& f_cc,
+  FaceIndexMap face_index_map,
+  FaceCCIdMap face_cc_map,
+  const TriangleMesh& tm)
+{
+  for(typename boost::graph_traits<TriangleMesh>::face_descriptor fd : faces(tm))
+    put(face_cc_map, fd, f_cc[ get(face_index_map, fd) ]);
+}
+
+template <class TriangleMesh, class FaceIndexMap>
+void set_f_cc_id(
+  const std::vector<std::size_t>&,
+  FaceIndexMap,
+  internal_np::Param_not_found,
+  const TriangleMesh&)
+{}
+
+template <class RefToVector>
+void copy_cc_to_volume_id(
+  std::vector<std::size_t>& cc_volume_ids,
+  RefToVector ref_to_vector)
+{
+  ref_to_vector.get().swap( cc_volume_ids );
+}
+
+inline
+void copy_cc_to_volume_id(
+  std::vector<std::size_t>&,
+  internal_np::Param_not_found)
+{}
+
+template <class RefToVector>
+void copy_nesting_levels(
+  std::vector<std::size_t>& nesting_levels,
+  RefToVector ref_to_vector)
+{
+  ref_to_vector.get().swap( nesting_levels );
+}
+
+inline
+void copy_nesting_levels(
+  std::vector<std::size_t>&,
+  internal_np::Param_not_found)
+{}
+
+template <class RefToBitset>
+void copy_orientation_bitset(
+  const std::vector<bool>& is_cc_outward_oriented,
+  RefToBitset ref_to_bs)
+{
+  ref_to_bs.get() = is_cc_outward_oriented;
+}
+
+inline
+void copy_orientation_bitset(
+  const std::vector<bool>&,
+  internal_np::Param_not_found)
+{}
+
+template <class OutputIterator>
+void set_cc_intersecting_pairs(
+  const std::set< std::pair<std::size_t, std::size_t> >& self_intersecting_cc,
+  OutputIterator out)
+{
+  for (const std::pair<std::size_t, std::size_t>& p : self_intersecting_cc)
+    *out++=p;
+}
+
+inline
+void set_cc_intersecting_pairs(
+  const std::set< std::pair<std::size_t, std::size_t> >&,
+  internal_np::Param_not_found)
+{}
+
+} // internal
+
+
+/*!
+ * \ingroup PMP_orientation_grp
+ * assigns to each face of `tm` an id corresponding to the volume connected component
+ * it contributes to.
+ *
+ * Using the adjacency relation of two faces along an edge, a triangle mesh can be split
+ * into connected components (*surface components* in the following).
+ * A surface component without boundary separates the 3D space into an infinite and
+ * a finite volume. We say that the finite volume is <i>enclosed</i> by this surface
+ * component.
+ *
+ * The volume connected components (*volume components* in the following) are defined as follows:
+ * Each surface component `S` that is outside any volume enclosed by
+ * another surface component defines the *outer boundary* of a volume component.
+ * Each surface component that is inside the volume enclosed by `S`
+ * defines a *hole* if it is included in no other volume enclosed by a surface component
+ * but `S`. Ignoring the identified volume component, the same procedure is recursively
+ * repeated for all surface components in each hole.
+ *
+ * There are some special cases:
+ * - a non-closed surface component is reported as a volume component ignoring any inclusion test
+ * - a self-intersecting surface component is reported as a volume component ignoring any inclusion test
+ * - a surface component intersecting another surface component
+ *   is reported as a volume component, and so are all the surface components inside its
+ *   enclosed volume
+ * - if `do_orientation_tests` is set to `true`, if the holes are not all equally oriented
+ *   (all inward or all outward) or if the holes and the outer boundary are equally
+ *   oriented, each surface component is reported as a volume component,
+ *   and so are all the surface components inside the corresponding enclosed volumes
+ * - If `do_orientation_tests` is set to `true` and the surface components that are
+ *   outside all enclosed volumes are inward oriented, they are then considered as holes
+ *   of the unbounded volume (that has no outer boundary)
+ *
+ * A property map for `CGAL::vertex_point_t` must be either available as an internal property map
+ * of `tm` or provided as one of the \ref bgl_namedparameters "Named Parameters".
+ *
+ * @tparam TriangleMesh a model of `FaceListGraph`
+ * @tparam VolumeFaceIndexMap a model of `WritablePropertyMap` with
+ *                      `boost::graph_traits<TriangleMesh>::%face_descriptor` as key type and
+ *                      `boost::graph_traits<TriangleMesh>::%faces_size_type` as value type.
+ * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
+ *
+ * @param tm the input triangle mesh
+ * @param volume_id_map the property map filled by this function with indices of volume components associated to the faces of `tm`
+ * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+ *
+ * @pre `CGAL::is_closed(tm)`
+ *
+ * \cgalNamedParamsBegin
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `tm`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+ *                    as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{geom_traits}
+ *     \cgalParamDescription{an instance of a geometric traits class}
+ *     \cgalParamType{a class model of `Kernel`}
+ *     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+ *     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{face_index_map}
+ *     \cgalParamDescription{a property map associating to each face of `tm` a unique index between `0` and `num_faces(tm) - 1`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%face_descriptor`
+ *                    as key type and `std::size_t` as value type}
+ *     \cgalParamDefault{an automatically indexed internal map}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{face_connected_component_map}
+ *     \cgalParamDescription{a property map filled by this function and that will contain for each face the id
+ *                           of its surface component in the range `[0, number of surface components - 1]`}
+ *     \cgalParamType{a class model of `WritablePropertyMap` with `boost::graph_traits<TriangleMesh>::%face_descriptor`
+ *                    as key type and `std::size_t` as value type}
+ *     \cgalParamDefault{an automatically indexed internal map}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{volume_inclusions}
+ *     \cgalParamDescription{a container, which contains at position `k` the ids of all the
+ *                           surface components that are the first intersected by any ray with source on
+ *                           the surface component `k` and directed outside the volume enclosed by the
+ *                           surface component `k`. There is only one such id but when some surface components intersect.
+ *                           The size of the container is exactly the number of surface components of `tm`.}
+ *     \cgalParamType{a `reference_wrapper` (either from `boost` or the standard library) containing
+ *                    a reference to an object that must be a model of the `BackInsertionSequence` concept,
+ *                    with a value type being a model of `BackInsertionSequence` of `std::size_t`,
+ *                    both types having the functions `reserve()` and `push_back()`}
+ *     \cgalParamDefault{unused}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{do_orientation_tests}
+ *     \cgalParamDescription{If `true`, the orientation of the faces of each surface components
+ *                           will be taken into account for the definition of the volume.
+ *                           If `false`, the face orientation is ignored and the volumes are defined
+ *                           only by the nesting of surface components.}
+ *     \cgalParamType{Boolean}
+ *     \cgalParamDefault{`true`}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{error_codes}
+ *     \cgalParamDescription{a container which indicates the status of a volume assigned to a set of faces.
+ *                           The description of the value type is given in the documentation of the enumeration type.
+ *                           The size of the container is exactly the number of volume components.}
+ *     \cgalParamType{a `reference_wrapper` (either from `boost` or the standard library) containing
+ *                    a reference to a container that must be a model of the `BackInsertionSequence` concept,
+ *                    with value type \link PMP_orientation_grp `Volume_error_code` \endlink}
+ *     \cgalParamDefault{unused}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{do_self_intersection_tests}
+ *     \cgalParamDescription{If `false`, it is assumed that `tm` does not contains any self-intersections.
+ *                           If `true`, the input might contain some self-intersections and a test is done prior to the volume decomposition.}
+ *     \cgalParamType{Boolean}
+ *     \cgalParamDefault{`false`}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{connected_component_id_to_volume_id}
+ *     \cgalParamDescription{For each connected component identified using its id `ccid`, the id
+ *                           of the volume it contributes to describe is the value at the position
+ *                           `ccid` in the container. The size of the container is exactly the number
+ *                           of connected components.}
+ *     \cgalParamType{a `reference_wrapper` (either from `boost` or the standard library) containing
+ *                    a reference to a container that must be a model of the `BackInsertionSequence` concept,
+ *                    with value type `std::size_t`}
+ *     \cgalParamDefault{unused}
+ *   \cgalParamNEnd
+ *
+ *
+ *   \cgalParamNBegin{nesting_levels}
+ *     \cgalParamDescription{For each connected component identified using its id `ccid`, the container contains the number of
+ *                           connected components containing on its bounded side this component.
+ *                           The size of the container is exactly the number of connected components.}
+ *     \cgalParamType{a `reference_wrapper` (either from `boost` or the standard library) containing
+ *                    a reference to a container that must be a model of the `BackInsertionSequence` concept,
+ *                    with value type `std::size_t`}
+ *     \cgalParamDefault{unused}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{is_cc_outward_oriented}
+ *     \cgalParamDescription{For each connected component identified using its id `ccid`, the output of `is_outward_oriented()`
+ *                           called on the triangle mesh corresponding to this connected component
+ *                           is the value at the position `ccid` in the container.
+ *                           The size of the container is exactly the number of connected components.}
+ *     \cgalParamType{a `reference_wrapper` (either from `boost` or the standard library) containing
+ *                    a reference to a container that must be a model of the `BackInsertionSequence` concept,
+ *                    with value type `bool`}
+ *     \cgalParamDefault{unused}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{intersecting_volume_pairs_output_iterator}
+ *     \cgalParamDescription{Output iterator into which pairs of ids (id must be convertible to `std::size_t`) can be put.
+ *                           Each pair of connected components intersecting will be reported using their ids.
+ *                           If `do_self_intersection_tests` named parameter is set to `false`, nothing will be reported.}
+ *     \cgalParamType{a model of `OutputIterator`}
+ *     \cgalParamDefault{unused}
+ *   \cgalParamNEnd
+ * \cgalNamedParamsEnd
+ *
+ * \return the number of volume components defined by `tm`
+ */
+template <class TriangleMesh, class VolumeFaceIndexMap, class NamedParameters>
+std::size_t
+volume_connected_components(const TriangleMesh& tm,
+                                  VolumeFaceIndexMap volume_id_map,
+                            const NamedParameters& np)
+{
+  CGAL_precondition(is_triangle_mesh(tm));
+  CGAL_precondition(is_closed(tm));
+
+  typedef boost::graph_traits<TriangleMesh> GT;
+  typedef typename GT::vertex_descriptor vertex_descriptor;
+  typedef typename GT::face_descriptor face_descriptor;
+  typedef typename GT::halfedge_descriptor halfedge_descriptor;
+  typedef typename GetVertexPointMap<TriangleMesh,
+                                     NamedParameters>::const_type Vpm;
+  typedef typename GetInitializedFaceIndexMap<TriangleMesh,
+                                              NamedParameters>::type FaceIndexMap;
+
+  typedef typename Kernel_traits<
+    typename boost::property_traits<Vpm>::value_type >::Kernel Kernel;
+
+  Vpm vpm = parameters::choose_parameter(parameters::get_parameter(np, internal_np::vertex_point),
+                                get_const_property_map(boost::vertex_point, tm));
+
+  FaceIndexMap fid_map = CGAL::get_initialized_face_index_map(tm, np);
+
+  std::vector<std::size_t> face_cc(num_faces(tm), std::size_t(-1));
+
+// set the connected component id of each face
+  const std::size_t nb_cc = connected_components(tm,
+                              bind_property_maps(fid_map,make_property_map(face_cc)),
+                              parameters::face_index_map(fid_map));
+
+  // contains for each CC the CC that are in its bounded side
+  std::vector<std::vector<std::size_t> > nested_cc_per_cc(nb_cc);
+
+  // copy cc-id info
+  internal::set_f_cc_id(face_cc, fid_map, parameters::get_parameter(np, internal_np::face_connected_component_map), tm);
+
+  const bool do_self_intersection_tests =
+        parameters::choose_parameter(parameters::get_parameter(np, internal_np::do_self_intersection_tests),
+                            false);
+  const bool ignore_orientation_of_cc =
+    !parameters::choose_parameter(parameters::get_parameter(np, internal_np::do_orientation_tests),
+                         true);
+
+  const bool used_as_a_predicate =
+    parameters::choose_parameter(parameters::get_parameter(np, internal_np::i_used_as_a_predicate),
+                        false); // indicate if the function is called by does_bound_a_volume
+
+  CGAL_assertion(!used_as_a_predicate || !ignore_orientation_of_cc);
+
+  std::vector<Volume_error_code> error_codes;
+  std::vector<bool> is_cc_outward_oriented;
+  std::vector<std::size_t> cc_volume_ids(nb_cc, -1);
+  std::vector < std::size_t > nesting_levels(nb_cc, 0); // indicates for each CC its nesting level
+
+  boost::dynamic_bitset<> cc_handled(nb_cc, 0);
+  std::size_t next_volume_id = 0;
+
+// Handle self-intersecting connected components
+  typedef std::pair<face_descriptor, face_descriptor> Face_pair;
+  std::vector<Face_pair> si_faces;
+  std::set< std::pair<std::size_t, std::size_t> > self_intersecting_cc; // due to self-intersections
+  if (do_self_intersection_tests)
+    self_intersections(tm, std::back_inserter(si_faces));
+  std::vector<bool> is_involved_in_self_intersection(nb_cc, false);
+
+  if (!si_faces.empty() && used_as_a_predicate)
+    return 0;
+
+  for(const Face_pair& fp : si_faces)
+  {
+    std::size_t first_cc_id = face_cc[ get(fid_map, fp.first) ];
+    std::size_t second_cc_id = face_cc[ get(fid_map, fp.second) ];
+
+    if (first_cc_id==second_cc_id)
+    {
+      if ( !cc_handled.test(first_cc_id) )
+      {
+        cc_handled.set(first_cc_id);
+        cc_volume_ids[first_cc_id]=next_volume_id++;
+        error_codes.push_back(SURFACE_WITH_SELF_INTERSECTIONS);
+      }
+    }
+    else
+    {
+      is_involved_in_self_intersection[first_cc_id] = true;
+      is_involved_in_self_intersection[second_cc_id] = true;
+      self_intersecting_cc.insert( make_sorted_pair(first_cc_id, second_cc_id) );
+    }
+  }
+
+  std::vector< std::vector<std::size_t> > nesting_parents(nb_cc);
+  if (!cc_handled.all())
+  {
+  // extract a vertex with max z coordinate for each connected component
+    std::vector<vertex_descriptor> xtrm_vertices(nb_cc, GT::null_vertex());
+    for(vertex_descriptor vd : vertices(tm))
+    {
+      halfedge_descriptor h = halfedge(vd, tm);
+      if (is_border(h, tm)) h = opposite(h, tm);
+      std::size_t cc_id = face_cc[get(fid_map, face(h, tm))];
+      if (cc_handled.test(cc_id)) continue;
+      if (xtrm_vertices[cc_id]==GT::null_vertex())
+        xtrm_vertices[cc_id]=vd;
+      else
+        if (get(vpm, vd).z()>get(vpm,xtrm_vertices[cc_id]).z())
+          xtrm_vertices[cc_id]=vd;
+    }
+
+  // fill orientation vector for each surface CC
+    if (!ignore_orientation_of_cc)
+    {
+      is_cc_outward_oriented.resize(nb_cc);
+      for(std::size_t cc_id=0; cc_id<nb_cc; ++cc_id)
+      {
+        if (cc_handled.test(cc_id)) continue;
+        is_cc_outward_oriented[cc_id] = internal::is_outward_oriented(xtrm_vertices[cc_id], tm, np);
+      }
+    }
+
+  //collect faces per CC
+    std::vector< std::vector<face_descriptor> > faces_per_cc(nb_cc);
+    std::vector< std::size_t > nb_faces_per_cc(nb_cc, 0);
+    for(face_descriptor fd : faces(tm))
+    {
+      std::size_t cc_id = face_cc[ get(fid_map, fd) ];
+      ++nb_faces_per_cc[ cc_id ];
+    }
+    for (std::size_t i=0; i<nb_cc; ++i)
+      if (!cc_handled.test(i))
+        faces_per_cc[i].reserve( nb_faces_per_cc[i] );
+    for(face_descriptor fd : faces(tm))
+    {
+      std::size_t cc_id = face_cc[ get(fid_map, fd) ];
+      faces_per_cc[ cc_id ].push_back(fd);
+    }
+
+  // init the main loop
+    // similar as above but exclusively contains cc ids included by more that one CC.
+    // The result will be then merged with nested_cc_per_cc but temporarilly we need
+    // another container to not more than once the inclusion testing (in case a CC is
+    // included by more than 2 CC) + associate such CC to only one volume
+    std::vector<std::vector<std::size_t> > nested_cc_per_cc_shared(nb_cc);
+    std::vector < boost::dynamic_bitset<> > level_k_nestings; // container containing CCs in the same volume (one bitset per volume) at level k
+    level_k_nestings.push_back( ~cc_handled );
+
+  // the following loop is exploring the nesting level by level (0 -> max_level)
+    std::size_t k = 0;
+    while (!level_k_nestings.empty())
+    {
+      std::vector < boost::dynamic_bitset<> > level_k_plus_1_nestings;
+      for(boost::dynamic_bitset<> cc_to_handle : level_k_nestings)
+      {
+        CGAL_assertion( cc_to_handle.any() );
+        while(cc_to_handle.any())
+        {
+        //extract a vertex with max z amongst all components
+          std::size_t xtrm_cc_id=cc_to_handle.find_first();
+          for(std::size_t id  = cc_to_handle.find_next(xtrm_cc_id);
+                          id != cc_to_handle.npos;
+                          id  = cc_to_handle.find_next(id))
+          {
+            if (get(vpm, xtrm_vertices[id]).z()>get(vpm,xtrm_vertices[xtrm_cc_id]).z())
+              xtrm_cc_id=id;
+          }
+          cc_to_handle.reset(xtrm_cc_id);
+          nesting_levels[xtrm_cc_id] = k;
+
+        // collect id inside xtrm_cc_id CC
+          typedef Side_of_triangle_mesh<TriangleMesh, Kernel, Vpm> Side_of_tm;
+          typename Side_of_tm::AABB_tree aabb_tree(faces_per_cc[xtrm_cc_id].begin(),
+                                                   faces_per_cc[xtrm_cc_id].end(),
+                                                   tm, vpm);
+          Side_of_tm side_of_cc(aabb_tree);
+
+          std::vector<std::size_t> cc_intersecting; // contains id of CC intersecting xtrm_cc_id
+
+          boost::dynamic_bitset<> nested_cc_to_handle(nb_cc, 0);
+          for(std::size_t id  = cc_to_handle.find_first();
+                          id != cc_to_handle.npos;
+                          id  = cc_to_handle.find_next(id))
+          {
+            if (self_intersecting_cc.count( make_sorted_pair(xtrm_cc_id, id) )!= 0)
+            {
+              cc_intersecting.push_back(id);
+              continue; // to not dot inclusion test for intersecting CCs
+            }
+
+            if (side_of_cc(get(vpm,xtrm_vertices[id]))==ON_BOUNDED_SIDE)
+            {
+              nested_cc_per_cc[xtrm_cc_id].push_back(id);
+              // mark nested CC as handle and collect them for the handling of the next level
+              nested_cc_to_handle.set(id);
+              cc_to_handle.reset(id);
+            }
+          }
+
+        //for each CC intersecting xtrm_cc_id, find the CCs included in both
+          for(std::size_t id : cc_intersecting)
+          {
+            typename Side_of_tm::AABB_tree aabb_tree(faces_per_cc[id].begin(),
+                                                     faces_per_cc[id].end(),
+                                                     tm, vpm);
+            Side_of_tm side_of_cc(aabb_tree);
+            for(std::size_t ncc_id : nested_cc_per_cc[xtrm_cc_id])
+            {
+              if (self_intersecting_cc.count( make_sorted_pair(ncc_id, id) )!= 0)
+                continue;
+              if (side_of_cc(get(vpm,xtrm_vertices[ncc_id]))==ON_BOUNDED_SIDE)
+                nested_cc_per_cc_shared[id].push_back(ncc_id);
+            }
+          }
+
+          if ( nested_cc_per_cc[xtrm_cc_id].empty() ) continue;
+          level_k_plus_1_nestings.push_back(nested_cc_to_handle);
+        }
+      }
+      ++k;
+      level_k_nestings.swap(level_k_plus_1_nestings);
+    }
+
+    // early return for orient_to_bound_a_volume
+    if (parameters::choose_parameter(parameters::get_parameter(np, internal_np::i_used_for_volume_orientation),false))
+    {
+      internal::copy_container_content(nesting_levels, parameters::get_parameter(np, internal_np::nesting_levels));
+      internal::copy_container_content(is_cc_outward_oriented, parameters::get_parameter(np, internal_np::is_cc_outward_oriented));
+      return 0;
+    }
+
+  // detect inconsistencies of the orientation at the level 0
+  // and check if all CC at level 0 are in the same volume
+    std::size_t ref_cc_id = nb_cc;
+    std::size_t FIRST_LEVEL = 0; // used to know if even or odd nesting is the top level
+    if(!ignore_orientation_of_cc)
+    {
+      for(std::size_t cc_id=0; cc_id<nb_cc; ++cc_id)
+      {
+        if (cc_handled.test(cc_id)) continue;
+        if( nesting_levels[cc_id]==0 )
+        {
+          if(ref_cc_id==nb_cc)
+            ref_cc_id=cc_id;
+          else
+            if( is_cc_outward_oriented[cc_id] != is_cc_outward_oriented[ref_cc_id] )
+            {
+              if (used_as_a_predicate) return 0;
+              // all is indefinite
+              for(std::size_t id=0; id<nb_cc; ++id)
+              {
+                if (cc_handled.test(cc_id)) continue;
+                cc_volume_ids[id] = next_volume_id++;
+                error_codes.push_back(INCOMPATIBLE_ORIENTATION);
+              }
+              cc_handled.set();
+              break;
+            }
+        }
+      }
+
+      if (!cc_handled.all() && !is_cc_outward_oriented[ref_cc_id])
+      {
+        // all level 0 CC are in the same volume
+        for(std::size_t cc_id=0; cc_id<nb_cc; ++cc_id)
+        {
+          if (cc_handled.test(cc_id)) continue;
+          if( nesting_levels[cc_id]==0 )
+          {
+            cc_handled.set(cc_id);
+            cc_volume_ids[cc_id]=next_volume_id;
+          }
+        }
+        ++next_volume_id;
+        error_codes.push_back(VALID_VOLUME);
+        FIRST_LEVEL = 1;
+      }
+    }
+
+  // apply volume classification using level 0 nesting
+    for(std::size_t cc_id=0; (!cc_handled.all()) && cc_id<nb_cc; ++cc_id)
+    {
+      if (cc_handled.test(cc_id)) continue;
+      CGAL_assertion( nesting_levels[cc_id]!=0 || ignore_orientation_of_cc || is_cc_outward_oriented[cc_id] );
+      if( nesting_levels[cc_id]%2 != FIRST_LEVEL ) continue; // we look for outer boundaries of volume only
+      cc_handled.set(cc_id);
+      cc_volume_ids[cc_id] = next_volume_id++;
+
+      //if the CC is involved in a self-intersection all nested CC are put in a separate volumes
+      if (is_involved_in_self_intersection[cc_id])
+      {
+        error_codes.push_back(VOLUME_INTERSECTION);
+        for(std::size_t ncc_id : nested_cc_per_cc[cc_id])
+        {
+          cc_handled.set(ncc_id);
+          cc_volume_ids[ncc_id] = next_volume_id++;
+          error_codes.push_back(VOLUME_INTERSECTION);
+        }
+        continue;
+      }
+      else
+      {
+        if (!ignore_orientation_of_cc && !is_cc_outward_oriented[cc_id])
+        {
+          // invalid orientation, all children are marked as incorrectly oriented
+          if (used_as_a_predicate) return 0;
+          cc_handled.set(cc_id);
+          cc_volume_ids[cc_id] = next_volume_id++;
+          error_codes.push_back(INCOMPATIBLE_ORIENTATION);
+          for(std::size_t ncc_id : nested_cc_per_cc[cc_id])
+          {
+            cc_handled.set(ncc_id);
+            cc_volume_ids[ncc_id] = next_volume_id++;
+            error_codes.push_back(INCOMPATIBLE_ORIENTATION);
+          }
+          continue;
+        }
+        else
+          error_codes.push_back(VALID_VOLUME);
+      }
+
+      for(std::size_t ncc_id : nested_cc_per_cc[cc_id])
+      {
+        if (nesting_levels[ncc_id]==nesting_levels[cc_id]+1)
+        {
+          cc_handled.set(ncc_id);
+          if (!ignore_orientation_of_cc)
+          {
+            if (is_cc_outward_oriented[cc_id]==is_cc_outward_oriented[ncc_id])
+            {
+              // the surface component has an incorrect orientation wrt to its parent:
+              // we dump it and all included surface components as independant volumes.
+              cc_volume_ids[ncc_id] = next_volume_id++;
+              error_codes.push_back(INCOMPATIBLE_ORIENTATION);
+              if (used_as_a_predicate) return 0;
+              for(std::size_t nncc_id : nested_cc_per_cc[ncc_id])
+              {
+                cc_handled.set(nncc_id);
+                cc_volume_ids[nncc_id] = next_volume_id++;
+                error_codes.push_back(INCOMPATIBLE_ORIENTATION);
+              }
+              continue;
+            }
+          }
+          if (is_involved_in_self_intersection[ncc_id])
+          {
+            cc_volume_ids[ncc_id] = next_volume_id++;
+            error_codes.push_back(VOLUME_INTERSECTION);
+            for(std::size_t nncc_id : nested_cc_per_cc[ncc_id])
+            {
+              if (cc_handled.test(nncc_id))
+              {
+                error_codes[ cc_volume_ids[nncc_id] ] = VOLUME_INTERSECTION;
+                continue;
+              }
+              cc_handled.set(nncc_id);
+              cc_volume_ids[nncc_id] = next_volume_id++;
+              error_codes.push_back(VOLUME_INTERSECTION);
+            }
+            continue;
+          }
+          cc_volume_ids[ncc_id] = cc_volume_ids[cc_id];
+        }
+      }
+    }
+    if (used_as_a_predicate)
+    {
+      internal::copy_container_content(is_cc_outward_oriented, parameters::get_parameter(np, internal_np::is_cc_outward_oriented));
+      return 1;
+    }
+
+  // merge nested_cc_per_cc and nested_cc_per_cc_shared
+  // (done after the volume creation to assign a CC to a unique volume)
+    for(std::size_t id=0; id<nb_cc; ++id)
+    {
+      if (!nested_cc_per_cc_shared[id].empty())
+        nested_cc_per_cc[id].insert(nested_cc_per_cc[id].end(),
+                                    nested_cc_per_cc_shared[id].begin(),
+                                    nested_cc_per_cc_shared[id].end());
+    }
+
+
+  // extract direct nested parent (more than one in case of self-intersection)
+    for(std::size_t cc_id=0; cc_id<nb_cc; ++cc_id)
+    {
+      for(std::size_t ncc_id : nested_cc_per_cc[cc_id])
+      {
+        if (nesting_levels[cc_id]+1 == nesting_levels[ncc_id])
+          nesting_parents[ncc_id].push_back(cc_id);
+      }
+    }
+
+  // update volume id map
+    for(std::size_t cc_id=0; cc_id<nb_cc; ++cc_id)
+    {
+      for(face_descriptor fd : faces_per_cc[cc_id])
+        put(volume_id_map, fd, cc_volume_ids[cc_id]);
+    }
+  }
+  else
+  {
+    for(face_descriptor fd : faces(tm))
+    {
+      std::size_t cc_id = face_cc[ get(fid_map, fd) ];
+      put(volume_id_map, fd, cc_volume_ids[cc_id]);
+    }
+  }
+
+  CGAL_assertion(next_volume_id == error_codes.size());
+  internal::copy_container_content(error_codes, parameters::get_parameter(np, internal_np::error_codes));
+  internal::copy_nested_parents(nesting_parents, parameters::get_parameter(np, internal_np::volume_inclusions));
+  internal::copy_container_content(nesting_levels, parameters::get_parameter(np, internal_np::nesting_levels));
+  internal::copy_container_content(cc_volume_ids, parameters::get_parameter(np, internal_np::connected_component_id_to_volume_id));
+  internal::copy_container_content(is_cc_outward_oriented, parameters::get_parameter(np, internal_np::is_cc_outward_oriented));
+  internal::set_cc_intersecting_pairs(self_intersecting_cc, parameters::get_parameter(np, internal_np::intersecting_volume_pairs_output_iterator));
+
+  return next_volume_id;
+}
+
+/** \ingroup PMP_orientation_grp
+ *
+ * indicates if `tm` bounds a volume.
+ * See \ref coref_def_subsec for details.
+ *
+ * @tparam TriangleMesh a model of `MutableFaceGraph`, `HalfedgeListGraph` and `FaceListGraph`.
+ * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
+ *
+ * @param tm a closed triangulated surface mesh
+ * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+ *
+ * @pre `CGAL::is_closed(tm)`
+ *
+ * \cgalNamedParamsBegin
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `tm`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+ *                    as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{face_index_map}
+ *     \cgalParamDescription{a property map associating to each face of `tm` a unique index between `0` and `num_faces(tm) - 1`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%face_descriptor`
+ *                    as key type and `std::size_t` as value type}
+ *     \cgalParamDefault{an automatically indexed internal map}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{is_cc_outward_oriented}
+ *     \cgalParamDescription{For each connected component identified using its id `ccid`, the output of `is_outward_oriented()`
+ *                           called on the triangle mesh corresponding to this connected component
+ *                           is the value at the position `ccid` in the container.
+ *                           The size of the container is exactly the number of connected components.}
+ *     \cgalParamType{a `reference_wrapper` (either from `boost` or the standard library) containing
+ *                    a reference to a container that must be a model of the `BackInsertionSequence` concept,
+ *                    with value type `bool`}
+ *     \cgalParamDefault{unused}
+ *   \cgalParamNEnd
+ * \cgalNamedParamsEnd
+ *
+ * \see `CGAL::Polygon_mesh_processing::orient_to_bound_a_volume()`
+ */
+template <class TriangleMesh, class NamedParameters>
+bool does_bound_a_volume(const TriangleMesh& tm, const NamedParameters& np)
+{
+  typedef boost::graph_traits<TriangleMesh> GT;
+  typedef typename GT::face_descriptor face_descriptor;
+
+  CGAL_precondition(is_closed(tm));
+  CGAL_precondition(is_triangle_mesh(tm));
+
+  Static_property_map<face_descriptor, std::size_t> vidmap(0); // dummy map not used
+  std::size_t res =
+    volume_connected_components(tm, vidmap, np.do_orientation_tests(true)
+                                              .i_used_as_a_predicate(true));
+  CGAL_assertion(res==0 || res==1);
+
+  return res!=0;
+}
+
+/// \cond SKIP_IN_MANUAL
+template <class TriangleMesh>
+bool does_bound_a_volume(const TriangleMesh& tm)
+{
+  return does_bound_a_volume(tm, parameters::all_default());
+}
+
+template <class TriangleMesh, class VolumeFaceIndexMap>
+std::size_t volume_connected_components(const TriangleMesh& tm, VolumeFaceIndexMap volume_id_map)
+{
+  return volume_connected_components(tm, volume_id_map, parameters::all_default());
+}
+/// \endcond
+
 
 /** \ingroup PMP_orientation_grp
  *
@@ -542,103 +1311,101 @@ void orient(TriangleMesh& tm)
  * See \ref coref_def_subsec for a precise definition.
  *
  * @tparam TriangleMesh a model of `MutableFaceGraph`, `HalfedgeListGraph` and `FaceListGraph`.
- *                      If `TriangleMesh` has an internal property map for `CGAL::face_index_t`,
- *                      as a named parameter, then it must be initialized.
- * @tparam NamedParameters a sequence of \ref pmp_namedparameters
+ * @tparam NamedParameters a sequence of \ref bgl_namedparameters
  *
  * @param tm a closed triangulated surface mesh
- * @param np optional sequence of \ref pmp_namedparameters among the ones listed below
+ * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+ *
+ * @pre `CGAL::is_closed(tm)`
  *
  * \cgalNamedParamsBegin
- *   \cgalParamBegin{vertex_point_map}
- *     the property map with the points associated to the vertices of `tm`.
- *     If this parameter is omitted, an internal property map for
- *     `CGAL::vertex_point_t` must be available in `TriangleMesh`
- *   \cgalParamEnd
- *   \cgalParamBegin{face_index_map}
- *     a property map containing the index of each face of `tm`.
- *   \cgalParamEnd
- *   \cgalParamBegin{outward_orientation}
- *     if set to `true` (default) the outer connected components will be outward oriented (inward oriented if set to `false`).
- *     If the outer connected components are inward oriented, it means that the infinity will be considered
- *     as part of the volume bounded by `tm`.
- *   \cgalParamEnd
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `tm`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+ *                    as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{face_index_map}
+ *     \cgalParamDescription{a property map associating to each face of `tm` a unique index between `0` and `num_faces(tm) - 1`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%face_descriptor`
+ *                    as key type and `std::size_t` as value type}
+ *     \cgalParamDefault{an automatically indexed internal map}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{outward_orientation}
+ *     \cgalParamDescription{If `true`, each connected component will be outward oriented (and inward oriented if `false`).}
+ *     \cgalParamType{Boolean}
+ *     \cgalParamDefault{`true`}
+ *     \cgalParamExtra{If the outer connected components are inward oriented,
+ *                     it means that the infinity will be considered as part of the volume bounded by `tm`.}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  *
  * \see `CGAL::Polygon_mesh_processing::does_bound_a_volume()`
  */
 template <class TriangleMesh, class NamedParameters>
 void orient_to_bound_a_volume(TriangleMesh& tm,
-                                        const NamedParameters& np)
+                              const NamedParameters& np)
 {
-  typedef boost::graph_traits<TriangleMesh> Graph_traits;
-  typedef typename Graph_traits::vertex_descriptor vertex_descriptor;
-  typedef typename GetVertexPointMap<TriangleMesh,
-      NamedParameters>::const_type Vpm;
-  typedef typename GetFaceIndexMap<TriangleMesh,
-      NamedParameters>::const_type Fid_map;
-  typedef typename Kernel_traits<
-      typename boost::property_traits<Vpm>::value_type >::Kernel Kernel;
-  if (!is_closed(tm)) return;
-  if (!is_triangle_mesh(tm)) return;
+  typedef boost::graph_traits<TriangleMesh>                                        Graph_traits;
+  typedef typename Graph_traits::face_descriptor                                   face_descriptor;
 
+  typedef typename GetVertexPointMap<TriangleMesh, NamedParameters>::const_type    Vpm;
+  typedef typename GetInitializedFaceIndexMap<TriangleMesh, NamedParameters>::type FaceIndexMap;
 
+  CGAL_assertion(is_closed(tm));
+  CGAL_assertion(is_triangle_mesh(tm));
+
+  using parameters::choose_parameter;
   using parameters::get_parameter;
 
-  bool orient_outward = CGAL::parameters::choose_parameter(
-                          get_parameter(np, internal_np::outward_orientation),true);
+  bool orient_outward = choose_parameter(get_parameter(np, internal_np::outward_orientation),true);
 
-  Vpm vpm = CGAL::parameters::choose_parameter(get_parameter(np, internal_np::vertex_point),
-                         get_const_property_map(boost::vertex_point, tm));
+  Vpm vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
+                             get_const_property_map(boost::vertex_point, tm));
 
-  Fid_map fid_map = CGAL::parameters::choose_parameter(get_parameter(np, internal_np::face_index),
-                                 get_const_property_map(boost::face_index, tm));
+  FaceIndexMap fid_map = CGAL::get_initialized_face_index_map(tm, np);
 
   std::vector<std::size_t> face_cc(num_faces(tm), std::size_t(-1));
-
+  std::vector<std::size_t> nesting_levels;
+  std::vector<bool> is_cc_outward_oriented;
+  Static_property_map<face_descriptor, std::size_t> vidmap(0); // dummy map not used
+
+  volume_connected_components(tm, vidmap,
+                              parameters::vertex_point_map(vpm)
+                                          .nesting_levels(boost::ref(nesting_levels))
+                                          .face_connected_component_map(bind_property_maps(fid_map,make_property_map(face_cc)))
+                                          .i_used_for_volume_orientation(true)
+                                          .do_orientation_tests(true)
+                                          .is_cc_outward_oriented(boost::ref(is_cc_outward_oriented))
+                                          );
 
   // set the connected component id of each face
-  std::size_t nb_cc = connected_components(tm,
-                                           bind_property_maps(fid_map,make_property_map(face_cc)),
-                                           parameters::face_index_map(fid_map));
 
-  if (nb_cc == 1)
+
+  if (nesting_levels.empty()) //case 1 cc
   {
-    if( orient_outward != is_outward_oriented(tm))
+    if( orient_outward != is_cc_outward_oriented[0])
       reverse_face_orientations(faces(tm), tm);
     return ;
   }
-
-
-  boost::dynamic_bitset<> cc_handled(nb_cc, 0);
-
-  // extract a vertex with max z coordinate for each connected component
-  std::vector<vertex_descriptor> xtrm_vertices(nb_cc, Graph_traits::null_vertex());
-  for(vertex_descriptor vd : vertices(tm))
+  std::size_t nb_cc = nesting_levels.size();
+  boost::dynamic_bitset<> cc_to_reverse(nb_cc, 0);
+  for(std::size_t i=0; i<nb_cc; ++i)
   {
-    std::size_t cc_id = face_cc[get(fid_map, face(halfedge(vd, tm), tm))];
-    if (xtrm_vertices[cc_id]==Graph_traits::null_vertex())
-      xtrm_vertices[cc_id]=vd;
-    else
-      if (get(vpm, vd).z()>get(vpm,xtrm_vertices[cc_id]).z())
-        xtrm_vertices[cc_id]=vd;
+    if ( ((nesting_levels[i]%2==0) == orient_outward) != is_cc_outward_oriented[i] )
+    {
+      cc_to_reverse.set(i);
+    }
   }
 
-  //extract a vertex with max z amongst all components
-  std::size_t xtrm_cc_id=0;
-  for(std::size_t id=1; id<nb_cc; ++id)
-    if (get(vpm, xtrm_vertices[id]).z()>get(vpm,xtrm_vertices[xtrm_cc_id]).z())
-      xtrm_cc_id=id;
-
-  bool is_parent_outward_oriented =
-      ! orient_outward;
-
-  internal::recursive_orient_volume_ccs<Kernel>(tm, vpm, fid_map,
-                                                xtrm_vertices,
-                                                cc_handled,
-                                                face_cc,
-                                                xtrm_cc_id,
-                                                is_parent_outward_oriented);
+  std::vector<face_descriptor> faces_to_reverse;
+  for (face_descriptor f : faces(tm))
+    if ( cc_to_reverse.test( face_cc[get(fid_map, f)] ) )
+      faces_to_reverse.push_back(f);
+
+  reverse_face_orientations(faces_to_reverse, tm);
 }
 
 template <class TriangleMesh>
@@ -646,6 +1413,218 @@ void orient_to_bound_a_volume(TriangleMe
 {
   orient_to_bound_a_volume(tm, parameters::all_default());
 }
+
+/*!
+ * \ingroup PMP_orientation_grp
+ * reverses the connected components of `tm` having compatible boundary cycles
+ * that could be merged if their orientation were made compatible, and stitches them.
+ * Connected components are examined by increasing number of faces.
+ *
+ * @tparam PolygonMesh a model of `MutableFaceGraph`, `HalfedgeListGraph` and `FaceListGraph`.
+ * @tparam NamedParameters a sequence of \ref bgl_namedparameters
+ *
+ * @param pm a surface mesh
+ * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+ *
+ * \cgalNamedParamsBegin
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `pm`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+ *                    as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, pm)`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     should be available for the vertices of `pm`.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{face_index_map}
+ *     \cgalParamDescription{a property map associating to each face of `pm` a unique index
+ *                           between `0` and `num_faces(pm) - 1`)}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%face_descriptor`
+ *                    as key type and `std::size_t` as value type}
+ *     \cgalParamDefault{an automatically indexed internal map}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{maximum_number_of_faces}
+ *     \cgalParamDescription{If not `0`, a connected component is considered reversible only
+ *                           if it has no more faces than the value given.
+ *                           Otherwise, it is always considered reversible.}
+ *     \cgalParamType{`std::size_t`}
+ *     \cgalParamDefault{`0`}
+ *   \cgalParamNEnd
+ * \cgalNamedParamsEnd
+ */
+template <class PolygonMesh, class NamedParameters>
+void merge_reversible_connected_components(PolygonMesh& pm,
+                                           const NamedParameters& np)
+{
+  typedef boost::graph_traits<PolygonMesh> GrT;
+  typedef typename GrT::face_descriptor face_descriptor;
+  typedef typename GrT::halfedge_descriptor halfedge_descriptor;
+
+  typedef typename GetVertexPointMap<PolygonMesh, NamedParameters>::const_type Vpm;
+
+  typedef typename boost::property_traits<Vpm>::value_type Point_3;
+  Vpm vpm = parameters::choose_parameter(parameters::get_parameter(np, internal_np::vertex_point),
+                                         get_const_property_map(vertex_point, pm));
+
+  typedef std::size_t F_cc_id;
+  typedef std::size_t B_cc_id;
+
+
+  typedef typename CGAL::GetInitializedFaceIndexMap<PolygonMesh, NamedParameters>::const_type Fidmap;
+  Fidmap fim = CGAL::get_initialized_face_index_map(pm, np);
+
+  typedef dynamic_face_property_t<F_cc_id>                   Face_property_tag;
+  typedef typename boost::property_map<PolygonMesh, Face_property_tag>::type   Face_cc_map;
+  Face_cc_map f_cc_ids  = get(Face_property_tag(), pm);
+  F_cc_id nb_cc = connected_components(pm, f_cc_ids, parameters::face_index_map(fim));
+
+  std::vector<std::size_t> nb_faces_per_cc(nb_cc, 0);
+  for (face_descriptor f : faces(pm))
+    nb_faces_per_cc[ get(f_cc_ids, f) ]+=1;
+
+  std::map< std::pair<Point_3, Point_3>, std::vector<halfedge_descriptor> > border_hedges_map;
+  std::vector<halfedge_descriptor> border_hedges;
+  typedef typename boost::property_map<PolygonMesh, dynamic_halfedge_property_t<B_cc_id> >::type H_to_bcc_id;
+  H_to_bcc_id h_bcc_ids = get(dynamic_halfedge_property_t<B_cc_id>(), pm);
+  const B_cc_id base_value(-1);
+  const B_cc_id FILTERED_OUT(-2);
+
+  // collect border halfedges
+  for (halfedge_descriptor h : halfedges(pm))
+    if ( is_border(h, pm) )
+    {
+      put(h_bcc_ids, h, base_value);
+      border_hedges.push_back(h);
+    }
+
+  // compute the border cc id of all halfedges and mark those duplicated in their own cycle
+  B_cc_id bcc_id=0;
+  for (halfedge_descriptor h : border_hedges)
+  {
+    if (get(h_bcc_ids,h) == base_value)
+    {
+      typedef std::map< std::pair<Point_3, Point_3>, halfedge_descriptor> Hmap;
+      Hmap hmap;
+      for (halfedge_descriptor hh : halfedges_around_face(h, pm))
+      {
+        std::pair< typename Hmap::iterator, bool > insert_res =
+            hmap.insert(
+              std::make_pair(
+                make_sorted_pair(get(vpm, source(hh, pm)),
+                                 get(vpm, target(hh,pm))), hh) );
+        if (insert_res.second)
+          put(h_bcc_ids, hh, bcc_id);
+        else
+        {
+          put(h_bcc_ids, hh, FILTERED_OUT);
+          put(h_bcc_ids, insert_res.first->second, FILTERED_OUT);
+        }
+      }
+      ++bcc_id;
+    }
+  }
+
+  // fill endpoints -> hedges
+  for (halfedge_descriptor h : border_hedges)
+  {
+    if ( get(h_bcc_ids, h) != FILTERED_OUT)
+      border_hedges_map[std::make_pair(get(vpm, source(h, pm)), get(vpm, target(h, pm)))].push_back(h);
+  }
+
+  // max nb of faces for a CC to be reversed
+  const std::size_t threshold =
+    parameters::choose_parameter( parameters::get_parameter(np, internal_np::maximum_number_of_faces), 0);
+
+  std::vector<bool> border_cycle_to_ignore(bcc_id, false);
+  std::vector<F_cc_id> cycle_f_cc_id(bcc_id);
+  std::vector< std::vector<F_cc_id> > patch_neighbors(nb_cc);
+
+  for (const auto& p : border_hedges_map)
+  {
+    const std::vector<halfedge_descriptor>& hedges = p.second;
+    switch(hedges.size())
+    {
+      case 1:
+        // isolated border hedge nothing to do
+      break;
+      case 2:
+      {
+        F_cc_id cc_id_0 = get(f_cc_ids, face(opposite(hedges[0], pm), pm)),
+                cc_id_1 = get(f_cc_ids, face(opposite(hedges[1], pm), pm));
+
+        if (cc_id_0!=cc_id_1)
+        {
+          cycle_f_cc_id[ get(h_bcc_ids, hedges[0]) ] = cc_id_0;
+          cycle_f_cc_id[ get(h_bcc_ids, hedges[1]) ] = cc_id_1;
+          // WARNING: we might have duplicates here but it is not important for our usage
+          patch_neighbors[cc_id_0].push_back(cc_id_1);
+          patch_neighbors[cc_id_1].push_back(cc_id_0);
+          break;
+        }
+        CGAL_FALLTHROUGH;
+      }
+      default:
+        for (halfedge_descriptor h : hedges)
+          border_cycle_to_ignore[get(h_bcc_ids, h)]=true;
+    }
+  }
+
+  // sort the connected components with potential matches using their number
+  // of faces (sorted by decreasing number of faces)
+  std::set<F_cc_id> ccs_to_reverse;
+  std::vector<bool> reversible(nb_cc, false);
+  std::set< F_cc_id, std::function<bool(F_cc_id,F_cc_id)> > queue(
+    [&nb_faces_per_cc](F_cc_id i, F_cc_id j)
+    {return nb_faces_per_cc[i]==nb_faces_per_cc[j] ? i<j : nb_faces_per_cc[i]>nb_faces_per_cc[j];}
+  );
+
+  for (B_cc_id i=0; i<bcc_id; ++i)
+  {
+    if ( !border_cycle_to_ignore[i] )
+    {
+      reversible[ cycle_f_cc_id[i] ] = true;
+      queue.insert(cycle_f_cc_id[i]);
+    }
+  }
+
+  // consider largest CC selected and reverse the neighbor patches if
+  // not already reversed or not marked as reversible
+  while( !queue.empty() )
+  {
+    F_cc_id f_cc_id = *queue.begin();
+    queue.erase( queue.begin() );
+    CGAL_assertion( reversible[f_cc_id] );
+    for (F_cc_id id : patch_neighbors[f_cc_id])
+    {
+      if (reversible[id] && (threshold==0 || threshold >= nb_faces_per_cc[id]))
+      {
+        CGAL_assertion( nb_faces_per_cc[f_cc_id] >= nb_faces_per_cc[id] );
+        ccs_to_reverse.insert(id);
+        reversible[id]=false;
+        queue.erase(id);
+      }
+    }
+  }
+
+  // reverse ccs and stitches boundaries
+  std::vector<face_descriptor> faces_to_reverse;
+  for (face_descriptor f : faces(pm))
+    if ( ccs_to_reverse.count( get(f_cc_ids, f) ) != 0 )
+      faces_to_reverse.push_back(f);
+
+  if ( !faces_to_reverse.empty() )
+  {
+    reverse_face_orientations(faces_to_reverse, pm);
+    stitch_borders(pm, np);
+  }
+}
+
+template <class PolygonMesh>
+void merge_reversible_connected_components(PolygonMesh& pm)
+{
+  merge_reversible_connected_components(pm, parameters::all_default());
+}
 } // namespace Polygon_mesh_processing
 } // namespace CGAL
 #endif // CGAL_ORIENT_POLYGON_MESH_H
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/orient_polygon_soup_extension.h 5.1-1/include/CGAL/Polygon_mesh_processing/orient_polygon_soup_extension.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/orient_polygon_soup_extension.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/orient_polygon_soup_extension.h	2020-09-08 12:15:23.000000000 +0000
@@ -0,0 +1,202 @@
+// Copyright (c) 2019 GeometryFactory (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/orient_polygon_soup_extension.h $
+// $Id: orient_polygon_soup_extension.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Sebastien Loriot and Maxime Gimeno
+
+#ifndef CGAL_ORIENT_POLYGON_SOUP_EXTENSION_H
+#define CGAL_ORIENT_POLYGON_SOUP_EXTENSION_H
+
+#include <CGAL/license/Polygon_mesh_processing/repair.h>
+
+#include <CGAL/AABB_tree.h>
+#include <CGAL/AABB_face_graph_triangle_primitive.h>
+#include <CGAL/AABB_traits.h>
+
+#include <CGAL/Polygon_mesh_processing/shape_predicates.h>
+#include <CGAL/Polygon_mesh_processing/compute_normal.h>
+#include <CGAL/Polygon_mesh_processing/orient_polygon_soup.h>
+
+#include <boost/iterator/counting_iterator.hpp>
+#include <boost/iterator/filter_iterator.hpp>
+
+#ifdef CGAL_LINKED_WITH_TBB
+#include <tbb/parallel_for.h>
+#endif // CGAL_LINKED_WITH_TBB
+
+
+namespace CGAL {
+
+namespace Polygon_mesh_processing {
+
+/*!
+ * \ingroup PMP_orientation_grp
+ * duplicates each point \a p at which the intersection
+ * of an infinitesimally small ball centered at \a p
+ * with the polygons incident to it is not a topological disk.
+ *
+ * @tparam PointRange a model of the concepts `RandomAccessContainer`
+ * and `BackInsertionSequence` whose `value_type` is the point type.
+ * @tparam PolygonRange a model of the concept `RandomAccessContainer`
+ * whose `value_type` is a model of the concept `RandomAccessContainer`
+ * whose `value_type` is `std::size_t`, and is also a model of `BackInsertionSequence`.
+ *
+ * @param points points of the soup of polygons. Some additional points might be pushed back to resolve
+ *               non-manifoldness or non-orientability issues.
+ * @param polygons each element in the vector describes a polygon using the indices of the points in `points`.
+ *                 If needed the order of the indices of a polygon might be reversed.
+ * @return `false` if some points were duplicated, thus producing a self-intersecting surface mesh.
+ * @return `true` otherwise.
+ * @sa `orient_polygon_soup()`
+ */
+template <class PointRange, class PolygonRange>
+bool
+duplicate_non_manifold_edges_in_polygon_soup(PointRange& points,
+                                            PolygonRange& polygons)
+{
+  std::size_t inital_nb_pts = points.size();
+  typedef CGAL::Polygon_mesh_processing::internal::
+    Polygon_soup_orienter<PointRange, PolygonRange> Orienter;
+
+  Orienter orienter(points, polygons);
+  orienter.fill_edge_map();
+  // make edges to duplicate
+  for(std::size_t i1=0;i1<points.size();++i1)
+    for(const typename Orienter::Internal_map_type::value_type& i2_and_pids : orienter.edges[i1])
+      if (i2_and_pids.second.size() > 1)
+        orienter.set_edge_marked(i1,i2_and_pids.first,orienter.marked_edges);
+  orienter.duplicate_singular_vertices();
+
+  return inital_nb_pts==points.size();
+}
+
+/*!
+ * \ingroup PMP_orientation_grp
+ *
+ * orients each triangle of a triangle soup using the orientation of its
+ * closest non degenerate triangle in `tm_ref`.
+ *
+ * \tparam Concurrency_tag enables sequential versus parallel orientation.
+                        Possible values are `Sequential_tag` (the default),
+                        `Parallel_if_available_tag`, and `Parallel_tag`.
+ * \tparam PointRange a model of the concepts `RandomAccessContainer`
+ * and `BackInsertionSequence` whose value type is the point type.
+ * @tparam TriangleRange a model of the concept `RandomAccessContainer`
+ * whose `value_type` is a model of the concept `RandomAccessContainer`
+ * whose `value_type` is `std::size_t`and of size 3.
+ * @tparam TriangleMesh a model of `FaceListGraph` and `MutableFaceGraph` .
+ *
+ * \param tm_ref the reference triangle_mesh.
+ * \param points the points of the soup.
+ * \param triangles the triangles of the soup.
+ * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+ *
+ * \cgalNamedParamsBegin
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `tm_ref`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+ *                    as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm_ref)`}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{geom_traits}
+ *     \cgalParamDescription{an instance of a geometric traits class}
+ *     \cgalParamType{The traits class must provide the nested functor `Collinear_3` to check whether three points are collinear. }
+ *     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+ *     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+ *   \cgalParamNEnd
+ * \cgalNamedParamsEnd
+ *
+ * \attention The types of points in `PointRange`, `geom_traits` and `vertex_point_map` must be the same.
+ */
+
+template <class Concurrency_tag = Sequential_tag, class PointRange, class TriangleRange,
+          class TriangleMesh, class NamedParameters>
+void
+orient_triangle_soup_with_reference_triangle_mesh(
+    const TriangleMesh& tm_ref,
+    PointRange& points,
+    TriangleRange& triangles,
+    const NamedParameters& np)
+{
+  namespace PMP = CGAL::Polygon_mesh_processing;
+
+  typedef boost::graph_traits<TriangleMesh> GrT;
+  typedef typename GrT::face_descriptor face_descriptor;
+  typedef typename PointRange::value_type Point_3;
+  typedef typename GetGeomTraits<TriangleMesh, NamedParameters>::type K;
+  typedef typename
+  GetVertexPointMap<TriangleMesh, NamedParameters>::const_type Vpm;
+
+  Vpm vpm = parameters::choose_parameter(parameters::get_parameter(np, internal_np::vertex_point),
+                                         get_const_property_map(CGAL::vertex_point, tm_ref));
+
+
+  typedef std::function<bool(face_descriptor)> Face_predicate;
+  Face_predicate is_not_deg =
+      [&tm_ref, np](face_descriptor f)
+  {
+    return !PMP::is_degenerate_triangle_face(f, tm_ref, np);
+  };
+
+  // build a tree filtering degenerate faces
+  typedef CGAL::AABB_face_graph_triangle_primitive<TriangleMesh, Vpm> Primitive;
+  typedef CGAL::AABB_traits<K, Primitive> Tree_traits;
+
+  boost::filter_iterator<Face_predicate, typename GrT::face_iterator>
+      begin(is_not_deg, faces(tm_ref).begin(), faces(tm_ref).end()),
+      end(is_not_deg, faces(tm_ref).end(), faces(tm_ref).end());
+
+  CGAL::AABB_tree<Tree_traits> tree(begin, end, tm_ref, vpm);
+
+  // now orient the faces
+  tree.build();
+  tree.accelerate_distance_queries();
+  auto process_facet =
+      [&points, &tree, &tm_ref, &triangles](std::size_t fid) {
+    const Point_3& p0 = points[triangles[fid][0]];
+    const Point_3& p1 = points[triangles[fid][1]];
+    const Point_3& p2 = points[triangles[fid][2]];
+    const Point_3 mid = CGAL::centroid(p0, p1, p2);
+    std::pair<Point_3, face_descriptor> pt_and_f =
+        tree.closest_point_and_primitive(mid);
+    auto face_ref_normal = PMP::compute_face_normal(pt_and_f.second, tm_ref);
+    if(face_ref_normal * cross_product(p1-p0, p2-p0) < 0) {
+      std::swap(triangles[fid][1], triangles[fid][2]);
+    }
+  };
+
+#if !defined(CGAL_LINKED_WITH_TBB)
+  CGAL_static_assertion_msg (!(boost::is_convertible<Concurrency_tag, CGAL::Parallel_tag>::value),
+                             "Parallel_tag is enabled but TBB is unavailable.");
+#else
+  if (boost::is_convertible<Concurrency_tag,CGAL::Parallel_tag>::value)
+    tbb::parallel_for(std::size_t(0), triangles.size(), std::size_t(1), process_facet);
+  else
+#endif
+    std::for_each(
+          boost::counting_iterator<std::size_t> (0),
+          boost::counting_iterator<std::size_t> (triangles.size()),
+          process_facet);
+}
+
+
+template <class Concurrency_tag = Sequential_tag, class PointRange, class TriangleRange,
+          class TriangleMesh>
+void
+orient_triangle_soup_with_reference_triangle_mesh(
+  const TriangleMesh& tm_ref,
+  PointRange& points,
+  TriangleRange& triangles)
+{
+  orient_triangle_soup_with_reference_triangle_mesh<Concurrency_tag>(tm_ref, points, triangles, CGAL::parameters::all_default());
+}
+
+}}//end namespace CGAL::Polygon_mesh_processing
+#endif // CGAL_ORIENT_POLYGON_SOUP_EXTENSION_H
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/orient_polygon_soup.h 5.1-1/include/CGAL/Polygon_mesh_processing/orient_polygon_soup.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/orient_polygon_soup.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/orient_polygon_soup.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/orient_polygon_soup.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/orient_polygon_soup.h $
 // $Id: orient_polygon_soup.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/polygon_mesh_to_polygon_soup.h 5.1-1/include/CGAL/Polygon_mesh_processing/polygon_mesh_to_polygon_soup.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/polygon_mesh_to_polygon_soup.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/polygon_mesh_to_polygon_soup.h	2020-09-08 12:15:23.000000000 +0000
@@ -0,0 +1,130 @@
+// Copyright (c) 2020 GeometryFactory (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/polygon_mesh_to_polygon_soup.h $
+// $Id: polygon_mesh_to_polygon_soup.h a84927d 2020-07-23T17:15:44+02:00 Laurent Rineau
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Mael Rouxel-Labbé
+
+#ifndef CGAL_POLYGON_MESH_PROCESSING_POLYGON_MESH_TO_POLYGON_SOUP_H
+#define CGAL_POLYGON_MESH_PROCESSING_POLYGON_MESH_TO_POLYGON_SOUP_H
+
+#include <CGAL/license/Polygon_mesh_processing/repair.h>
+
+#include <CGAL/algorithm.h>
+#include <CGAL/boost/graph/iterator.h>
+#include <CGAL/boost/graph/named_params_helper.h>
+#include <CGAL/Container_helper.h>
+#include <CGAL/Dynamic_property_map.h>
+#include <CGAL/property_map.h>
+
+#include <boost/range/value_type.hpp>
+#include <boost/range/reference.hpp>
+
+namespace CGAL {
+namespace Polygon_mesh_processing {
+
+/// \ingroup PMP_repairing_grp
+///
+/// adds the vertices and faces of a mesh into a (possibly non-empty) polygon soup.
+///
+/// \tparam PolygonMesh a model of `FaceListGraph`
+/// \tparam PointRange a model of the concepts `RandomAccessContainer` and
+///                    `BackInsertionSequence` whose value type can be constructed from
+///                    the point type of the polygon mesh
+/// \tparam PolygonRange a model of the concepts `RandomAccessContainer` and `BackInsertionSequence` whose
+///                      value type is itself a model of the concepts `RandomAccessContainer` and
+///                      `BackInsertionSequence` whose value type is `std::size_t`
+/// \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
+///
+/// \param mesh the mesh whose faces are being put in the polygon soup
+/// \param points points making the polygons of the soup
+/// \param polygons each element in the vector describes a polygon using the indices of the points in `points`
+/// \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+///
+/// \cgalNamedParamsBegin
+///   \cgalParamNBegin{vertex_point_map}
+///     \cgalParamDescription{a property map associating points to the vertices of `mesh`}
+///     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+///                    as key type and `%Point_3` as value type}
+///     \cgalParamDefault{`boost::get(CGAL::vertex_point, mesh)`}
+///   \cgalParamNEnd
+/// \cgalNamedParamsEnd
+///
+/// \cgalAdvancedBegin
+/// `PolygonRange` can also be a model of the concepts `RandomAccessContainer` and `BackInsertionSequence`
+/// whose value type is an array, but it is the user's responsability to ensure that
+/// all faces have the same number of vertices, and that this number is equal to the size of the array.
+/// \cgalAdvancedEnd
+///
+/// \sa `CGAL::Polygon_mesh_processing::orient_polygon_soup()`
+/// \sa `CGAL::Polygon_mesh_processing::is_polygon_soup_a_polygon_mesh()`
+/// \sa `CGAL::Polygon_mesh_processing::polygon_soup_to_polygon_mesh()`
+///
+template<typename PolygonMesh,
+         typename PointRange, typename PolygonRange,
+         typename NamedParameters>
+void polygon_mesh_to_polygon_soup(const PolygonMesh& mesh,
+                                  PointRange& points,
+                                  PolygonRange& polygons,
+                                  const NamedParameters& np)
+{
+  typedef typename boost::graph_traits<PolygonMesh>::vertex_descriptor              vertex_descriptor;
+  typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor            halfedge_descriptor;
+  typedef typename boost::graph_traits<PolygonMesh>::face_descriptor                face_descriptor;
+
+  using parameters::choose_parameter;
+  using parameters::get_parameter;
+
+  typedef typename GetVertexPointMap<PolygonMesh, NamedParameters>::const_type      VPM;
+  VPM vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
+                             get_const_property_map(vertex_point, mesh));
+
+  typedef CGAL::dynamic_vertex_property_t<std::size_t>                              Vertex_index;
+  typedef typename boost::property_map<PolygonMesh, Vertex_index>::const_type       VIM;
+  VIM vim = get(Vertex_index(), mesh);
+
+  typedef typename boost::range_value<PolygonRange>::type                           Polygon;
+
+  std::size_t index = points.size(); // so that multiple meshes can be put into the same soup
+  CGAL::internal::reserve(points, points.size() + vertices(mesh).size());
+  CGAL::internal::reserve(polygons, polygons.size() + faces(mesh).size());
+
+  for(const vertex_descriptor v : vertices(mesh))
+  {
+    points.emplace_back(get(vpm, v));
+    put(vim, v, index++);
+  }
+
+  for(const face_descriptor f : faces(mesh))
+  {
+    CGAL::Iterator_range<CGAL::Halfedge_around_face_iterator<PolygonMesh> > incident_halfedges =
+      CGAL::halfedges_around_face(halfedge(f, mesh), mesh);
+
+    Polygon polygon;
+    CGAL::internal::resize(polygon, incident_halfedges.size());
+
+    std::size_t pos = 0;
+    for(halfedge_descriptor h : incident_halfedges)
+      polygon[pos++] = get(vim, target(h, mesh));
+
+    polygons.push_back(polygon);
+  }
+}
+
+template<typename PolygonMesh, typename PointRange, typename PolygonRange>
+void polygon_mesh_to_polygon_soup(const PolygonMesh& mesh,
+                                  PointRange& points,
+                                  PolygonRange& polygons)
+{
+  return polygon_mesh_to_polygon_soup(mesh, points, polygons, CGAL::parameters::all_default());
+}
+
+} // namespace Polygon_mesh_processing
+} // namespace CGAL
+
+#endif // CGAL_POLYGON_MESH_PROCESSING_POLYGON_MESH_TO_POLYGON_SOUP_H
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/polygon_soup_to_polygon_mesh.h 5.1-1/include/CGAL/Polygon_mesh_processing/polygon_soup_to_polygon_mesh.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/polygon_soup_to_polygon_mesh.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/polygon_soup_to_polygon_mesh.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,53 +3,61 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/polygon_soup_to_polygon_mesh.h $
-// $Id: polygon_soup_to_polygon_mesh.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/polygon_soup_to_polygon_mesh.h $
+// $Id: polygon_soup_to_polygon_mesh.h 2780fa0 2020-08-05T10:49:14+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
 // Author(s)     : Laurent Rineau and Ilker O. Yaz
 
-#ifndef CGAL_POLYGON_MESH_PROCESSING_POLYGON_SOUP_TO_POLYGON_MESH
-#define CGAL_POLYGON_MESH_PROCESSING_POLYGON_SOUP_TO_POLYGON_MESH
+#ifndef CGAL_POLYGON_MESH_PROCESSING_POLYGON_SOUP_TO_POLYGON_MESH_H
+#define CGAL_POLYGON_MESH_PROCESSING_POLYGON_SOUP_TO_POLYGON_MESH_H
 
 #include <CGAL/license/Polygon_mesh_processing/repair.h>
 
-#include <CGAL/disable_warnings.h>
+#include <CGAL/Polygon_mesh_processing/orient_polygon_soup.h>
 
+#include <CGAL/algorithm.h>
 #include <CGAL/boost/graph/Euler_operations.h>
+#include <CGAL/boost/graph/iterator.h>
+#include <CGAL/boost/graph/named_params_helper.h>
+#include <CGAL/Dynamic_property_map.h>
 #include <CGAL/property_map.h>
-#include <CGAL/Polygon_mesh_processing/orient_polygon_soup.h>
-#include <CGAL/algorithm.h>
-#include <set>
-#include <boost/dynamic_bitset.hpp>
 
+#include <boost/dynamic_bitset.hpp>
 #include <boost/range/size.hpp>
 #include <boost/range/value_type.hpp>
 #include <boost/range/reference.hpp>
 
-namespace CGAL
-{
-namespace Polygon_mesh_processing
-{
-namespace internal
+#include <array>
+#include <set>
+#include <type_traits>
+#include <vector>
+
+namespace CGAL {
+namespace Polygon_mesh_processing {
+namespace internal {
+
+template <typename PM_Point, typename PS_Point>
+PM_Point convert_to_pm_point(const PS_Point& p)
 {
-template <typename PM
-        , typename PointRange
-        , typename PolygonRange>
-class Polygon_soup_to_polygon_mesh
+  CGAL_static_assertion((std::is_convertible<PS_Point, PM_Point>::value));
+  return PM_Point(p);
+}
+
+// just for backward compatibility reasons
+template <typename PM_Point, typename PS_FT>
+PM_Point convert_to_pm_point(const std::array<PS_FT, 3>& p)
 {
-  const PointRange& _points;
-  const PolygonRange& _polygons;
+  return PM_Point(p[0], p[1], p[2]);
+}
 
-  typedef typename boost::property_map<PM, CGAL::vertex_point_t>::type Vpmap;
-  typedef typename boost::property_traits<Vpmap>::value_type Point_3;
-
-  typedef typename boost::graph_traits<PM>::vertex_descriptor vertex_descriptor;
-  typedef typename boost::graph_traits<PM>::halfedge_descriptor halfedge_descriptor;
-  typedef typename boost::graph_traits<PM>::face_descriptor face_descriptor;
-  typedef typename PolygonRange::value_type Polygon;
-  typedef typename PointRange::value_type Point;
+template <typename PointRange,
+          typename PolygonRange,
+          typename PointMap = typename CGAL::GetPointMap<PointRange>::const_type>
+class PS_to_PM_converter
+{
+  typedef typename boost::range_value<PolygonRange>::type                 Polygon;
 
 public:
   /**
@@ -57,178 +65,240 @@ public:
   * @param points points of the soup of polygons.
   * @param polygons each element in the range describes a polygon using the index of the points in the range.
   */
-  Polygon_soup_to_polygon_mesh(const PointRange& points,
-                               const PolygonRange& polygons)
-    : _points(points),
-      _polygons(polygons)
+  PS_to_PM_converter(const PointRange& points,
+                     const PolygonRange& polygons,
+                     const PointMap pm = PointMap())
+    : m_points(points),
+      m_polygons(polygons),
+      m_pm(pm)
   { }
 
-  void operator()(PM& pmesh, const bool insert_isolated_vertices = true)
+  template <typename PolygonMesh, typename VertexPointMap>
+  void operator()(PolygonMesh& pmesh,
+                  VertexPointMap vpm,
+                  const bool insert_isolated_vertices = true)
   {
-    reserve(pmesh, static_cast<typename boost::graph_traits<PM>::vertices_size_type>(_points.size()),
-                   static_cast<typename boost::graph_traits<PM>::edges_size_type>(2*_polygons.size()),
-                   static_cast<typename boost::graph_traits<PM>::faces_size_type>(_polygons.size()) );
+    typedef typename boost::graph_traits<PolygonMesh>::vertex_descriptor    vertex_descriptor;
 
-    Vpmap vpmap = get(CGAL::vertex_point, pmesh);
+    typedef typename boost::property_traits<VertexPointMap>::value_type     PM_Point;
+
+    reserve(pmesh, static_cast<typename boost::graph_traits<PolygonMesh>::vertices_size_type>(m_points.size()),
+            static_cast<typename boost::graph_traits<PolygonMesh>::edges_size_type>(2*m_polygons.size()),
+            static_cast<typename boost::graph_traits<PolygonMesh>::faces_size_type>(m_polygons.size()));
 
     boost::dynamic_bitset<> not_isolated;
-    if (!insert_isolated_vertices)
+    if(!insert_isolated_vertices)
     {
-      not_isolated.resize(_points.size());
-      for (std::size_t i = 0, end = _polygons.size(); i < end; ++i)
+      not_isolated.resize(m_points.size());
+      for(std::size_t i = 0, end = m_polygons.size(); i < end; ++i)
       {
-        const Polygon& polygon = _polygons[i];
+        const Polygon& polygon = m_polygons[i];
         const std::size_t size = polygon.size();
-        for (std::size_t j = 0; j < size; ++j)
+        for(std::size_t j = 0; j < size; ++j)
           not_isolated.set(polygon[j], true);
       }
     }
 
-    std::vector<vertex_descriptor> vertices(_points.size());
-    for (std::size_t i = 0, end = _points.size(); i < end; ++i)
+    std::vector<vertex_descriptor> vertices(m_points.size());
+    for(std::size_t i = 0, end = m_points.size(); i < end; ++i)
     {
-      if (!insert_isolated_vertices && !not_isolated.test(i))
+      if(!insert_isolated_vertices && !not_isolated.test(i))
         continue;
 
-      Point_3 pi(_points[i][0], _points[i][1], _points[i][2]);
       vertices[i] = add_vertex(pmesh);
-      put(vpmap, vertices[i], pi);
+      PM_Point pi = convert_to_pm_point<PM_Point>(get(m_pm, m_points[i]));
+      put(vpm, vertices[i], pi);
     }
 
-    for (std::size_t i = 0, end = _polygons.size(); i < end; ++i)
+    for(std::size_t i = 0, end = m_polygons.size(); i < end; ++i)
     {
-      const Polygon& polygon = _polygons[i];
+      const Polygon& polygon = m_polygons[i];
       const std::size_t size = polygon.size();
 
       std::vector<vertex_descriptor> vr(size); //vertex range
       vr.resize(size);
-      for (std::size_t j = 0; j < size; ++j)
+      for(std::size_t j = 0; j < size; ++j)
         vr[j] = vertices[polygon[j] ];
 
-      CGAL_assertion_code(face_descriptor fd = )
-      CGAL::Euler::add_face(vr, pmesh);
-      CGAL_assertion(fd != boost::graph_traits<PM>::null_face());
+      CGAL_assertion_code(typename boost::graph_traits<PolygonMesh>::face_descriptor fd =)
+          CGAL::Euler::add_face(vr, pmesh);
+      CGAL_assertion(fd != boost::graph_traits<PolygonMesh>::null_face());
     }
   }
+
+  template <typename PolygonMesh>
+  void operator()(PolygonMesh& pmesh,
+                  const bool insert_isolated_vertices = true)
+  {
+    return operator()(pmesh, get(CGAL::vertex_point, pmesh), insert_isolated_vertices);
+  }
+
+private:
+  const PointRange& m_points;
+  const PolygonRange& m_polygons;
+  const PointMap m_pm;
 };
-}//end namespace internal
 
+} // namespace internal
 
-  /**
-   * \ingroup PMP_repairing_grp
-   *
-   * returns `true` if the soup of polygons defines a valid polygon
-   * mesh that can be handled by
-   * `CGAL::Polygon_mesh_processing::polygon_soup_to_polygon_mesh()`.
-   * It checks that each edge has at most two incident faces and such an edge
-   * is visited in opposite direction along the two face boundaries,
-   * no polygon has twice the same vertex,
-   * and the polygon soup describes a manifold surface.
-   * This function does not require a range of points as an argument
-   * since the check is purely topological. To each vertex of the mesh
-   * is associated an index that is used in the description of the
-   * boundaries of the polygons provided in `polygons`.
-   *
-   * @tparam PolygonRange a model of the concept `RandomAccessContainer`
-   * whose value_type is a model of the concept `RandomAccessContainer`
-   * whose value_type is `std::size_t`.
-   *
-   * @param polygons each element in the range describes a polygon
-   * using the indices of the vertices.
-   *
-   * @sa `orient_polygon_soup()`
-   */
-  template<class PolygonRange>
-  bool is_polygon_soup_a_polygon_mesh(const PolygonRange& polygons)
+/**
+* \ingroup PMP_repairing_grp
+*
+* returns `true` if the soup of polygons defines a valid polygon
+* mesh that can be handled by
+* `CGAL::Polygon_mesh_processing::polygon_soup_to_polygon_mesh()`.
+* It checks that each edge has at most two incident faces and such an edge
+* is visited in opposite direction along the two face boundaries,
+* no polygon has twice the same vertex,
+* and the polygon soup describes a manifold surface.
+* This function does not require a range of points as an argument
+* since the check is purely topological. To each vertex of the mesh
+* is associated an index that is used in the description of the
+* boundaries of the polygons provided in `polygons`.
+*
+* @tparam PolygonRange a model of the concept `RandomAccessContainer`
+* whose value_type is a model of the concept `RandomAccessContainer`
+* whose value_type is `std::size_t`.
+*
+* @param polygons each element in the range describes a polygon
+* using the indices of the vertices.
+*
+* @sa `orient_polygon_soup()`
+*/
+template<typename PolygonRange>
+bool is_polygon_soup_a_polygon_mesh(const PolygonRange& polygons)
+{
+  typedef typename boost::range_value<PolygonRange>::type           Polygon;
+  typedef typename boost::range_value<Polygon>::type                V_ID;
+
+  if(boost::begin(polygons) == boost::end(polygons))
+    return true;
+
+  //check there is no duplicated ordered edge, and
+  //check there is no polygon with twice the same vertex
+  std::set<std::pair<V_ID, V_ID> > edge_set;
+  V_ID max_id = 0;
+  for(const Polygon& polygon : polygons)
   {
-    typedef typename boost::range_value<
-      typename boost::range_value<
-        PolygonRange>::type >::type V_ID;
-    typedef typename boost::range_value<
-      PolygonRange>::type           Polygon;
+    std::size_t nb_edges = boost::size(polygon);
+    if(nb_edges < 3)
+      return false;
 
-    if(boost::begin(polygons) == boost::end(polygons)){
-      return true;
-    }
-    //check there is no duplicated ordered edge, and
-    //check there is no polygon with twice the same vertex
-    std::set< std::pair<V_ID, V_ID> > edge_set;
-    V_ID max_id=0;
-    for(const Polygon& polygon : polygons)
+    std::set<V_ID> polygon_vertices;
+    V_ID prev = *std::prev(boost::end(polygon));
+    for(V_ID id : polygon)
     {
-      std::size_t nb_edges = boost::size(polygon);
-      if (nb_edges<3) return false;
+      if(max_id<id)
+        max_id = id;
 
-      std::set<V_ID> polygon_vertices;
-      V_ID prev= *std::prev(boost::end(polygon));
-      for(V_ID id : polygon)
-      {
-        if (max_id<id) max_id=id;
-        if (! edge_set.insert(std::pair<V_ID, V_ID>(prev,id)).second )
-          return false;
-        else
-          prev=id;
+      if(! edge_set.insert(std::pair<V_ID, V_ID>(prev,id)).second)
+        return false;
+      else
+        prev = id;
 
-        if (!polygon_vertices.insert(id).second)
-          return false;//vertex met twice in the same polygon
-      }
+      if(!polygon_vertices.insert(id).second)
+        return false; // vertex met twice in the same polygon
     }
-
-    //check manifoldness
-    typedef std::vector<V_ID> PointRange;
-    typedef internal::Polygon_soup_orienter<PointRange, PolygonRange> Orienter;
-    typename Orienter::Edge_map edges(max_id+1);
-    typename Orienter::Marked_edges marked_edges;
-    Orienter::fill_edge_map(edges, marked_edges, polygons);
-    //returns false if duplication is necessary
-    if (!marked_edges.empty())
-      return false;
-    return Orienter::has_singular_vertices(static_cast<std::size_t>(max_id+1),polygons,edges,marked_edges);
   }
 
-  /**
-  * \ingroup PMP_repairing_grp
-  * builds a polygon mesh from a soup of polygons.
-  * @pre the input polygon soup describes a consistently oriented
-  * polygon mesh.
-  *
-  * @tparam PolygonMesh a model of `MutableFaceGraph` with an internal point
-  * property map
-  * @tparam PointRange a model of the concepts `RandomAccessContainer` and
-  * `BackInsertionSequence` whose value type is the point type
-  * @tparam PolygonRange a model of the concept `RandomAccessContainer` whose
-  * `value_type` is a model of the concept `RandomAccessContainer` whose `value_type` is `std::size_t`.
-
-  *
-  * @param points points of the soup of polygons
-  * @param polygons each element in the vector describes a polygon using the index of the points in `points`
-  * @param out the polygon mesh to be built
-  *
-  * @pre \link CGAL::Polygon_mesh_processing::is_polygon_soup_a_polygon_mesh()
-  *            CGAL::Polygon_mesh_processing::is_polygon_soup_a_polygon_mesh(polygons) \endlink
-  *
-  * \sa `CGAL::Polygon_mesh_processing::orient_polygon_soup()`
-  * \sa `CGAL::Polygon_mesh_processing::is_polygon_soup_a_polygon_mesh()`
-  *
-  */
-  template<class PolygonMesh, class PointRange, class PolygonRange>
-  void polygon_soup_to_polygon_mesh(
-    const PointRange& points,
-    const PolygonRange& polygons,
-    PolygonMesh& out)
-  {
-    CGAL_precondition_msg(is_polygon_soup_a_polygon_mesh(polygons),
-                          "Input soup needs to be a polygon mesh!");
+  //check manifoldness
+  typedef std::vector<V_ID>                                           PointRange;
+  typedef internal::Polygon_soup_orienter<PointRange, PolygonRange>   Orienter;
+
+  typename Orienter::Edge_map edges(max_id+1);
+  typename Orienter::Marked_edges marked_edges;
+  Orienter::fill_edge_map(edges, marked_edges, polygons);
+
+  //returns false if duplication is necessary
+  if(!marked_edges.empty())
+    return false;
+
+  return Orienter::has_singular_vertices(static_cast<std::size_t>(max_id+1),polygons,edges,marked_edges);
+}
+
+/**
+* \ingroup PMP_repairing_grp
+* builds a polygon mesh from a soup of polygons.
+*
+* @pre the input polygon soup describes a consistently oriented
+* polygon mesh. This can be checked using the function
+* \link CGAL::Polygon_mesh_processing::is_polygon_soup_a_polygon_mesh()
+* `CGAL::Polygon_mesh_processing::is_polygon_soup_a_polygon_mesh(polygons)` \endlink.
+*
+* @tparam PolygonMesh a model of `MutableFaceGraph`
+* @tparam PointRange a model of the concept `RandomAccessContainer`
+* whose value type is the point type
+* @tparam PolygonRange a model of the concept `RandomAccessContainer` whose
+* value type is a model of the concept `RandomAccessContainer` whose value type is `std::size_t`
+* @tparam NamedParameters_PS a sequence of \ref bgl_namedparameters "Named Parameters"
+* @tparam NamedParameters_PM a sequence of \ref bgl_namedparameters "Named Parameters"
+*
+* @param points points of the soup of polygons
+* @param polygons each element in the vector describes a polygon using the indices of the points in `points`
+* @param out the polygon mesh to be built
+* @param np_ps an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+*
+* \cgalNamedParamsBegin
+*   \cgalParamNBegin{point_map}
+*     \cgalParamDescription{a property map associating points to the elements of the range `points`}
+*     \cgalParamType{a model of `ReadablePropertyMap` whose value type is a point type convertible to the point type
+*                    of the vertex point map associated to the polygon mesh}
+*     \cgalParamDefault{`CGAL::Identity_property_map`}
+*   \cgalParamNEnd
+* \cgalNamedParamsEnd
+*
+* @param np_pm an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+*
+* \cgalNamedParamsBegin
+*   \cgalParamNBegin{vertex_point_map}
+*     \cgalParamDescription{a property map associating points to the vertices of `out`}
+*     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+*                    as key type and `%Point_3` as value type}
+*     \cgalParamDefault{`boost::get(CGAL::vertex_point, out)`}
+*     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+*                     must be available in `PolygonMesh`.}
+*   \cgalParamNEnd
+* \cgalNamedParamsEnd
+*
+* \sa `CGAL::Polygon_mesh_processing::orient_polygon_soup()`
+* \sa `CGAL::Polygon_mesh_processing::is_polygon_soup_a_polygon_mesh()`
+* \sa `CGAL::Polygon_mesh_processing::polygon_mesh_to_polygon_soup()`
+*/
+template<typename PolygonMesh,
+         typename PointRange, typename PolygonRange,
+         typename NamedParameters_PS, typename NamedParameters_PM>
+void polygon_soup_to_polygon_mesh(const PointRange& points,
+                                  const PolygonRange& polygons,
+                                  PolygonMesh& out,
+                                  const NamedParameters_PS& np_ps,
+                                  const NamedParameters_PM& np_pm)
+{
+  CGAL_precondition_msg(is_polygon_soup_a_polygon_mesh(polygons),
+                        "Input soup needs to define a valid polygon mesh! See is_polygon_soup_a_polygon_mesh() for further information.");
 
-    internal::Polygon_soup_to_polygon_mesh<PolygonMesh, PointRange, PolygonRange>
-      converter(points, polygons);
-    converter(out);
-  }
+  using parameters::choose_parameter;
+  using parameters::get_parameter;
 
-}//end namespace Polygon_mesh_processing
+  typedef typename CGAL::GetPointMap<PointRange, NamedParameters_PS>::const_type    Point_map;
+  Point_map pm = choose_parameter<Point_map>(get_parameter(np_ps, internal_np::point_map));
 
-}// end namespace CGAL
+  typedef typename CGAL::GetVertexPointMap<PolygonMesh, NamedParameters_PM>::type   Vertex_point_map;
+  Vertex_point_map vpm = choose_parameter(get_parameter(np_pm, internal_np::vertex_point),
+                                          get_property_map(CGAL::vertex_point, out));
+
+  internal::PS_to_PM_converter<PointRange, PolygonRange, Point_map> converter(points, polygons, pm);
+  converter(out, vpm);
+}
+
+template<typename PolygonMesh, typename PointRange, typename PolygonRange>
+void polygon_soup_to_polygon_mesh(const PointRange& points,
+                                  const PolygonRange& polygons,
+                                  PolygonMesh& out)
+{
+  return polygon_soup_to_polygon_mesh(points, polygons, out, parameters::all_default(), parameters::all_default());
+}
 
-#include <CGAL/enable_warnings.h>
+} // namespace Polygon_mesh_processing
+} // namespace CGAL
 
-#endif // CGAL_POLYGON_MESH_PROCESSING_POLYGON_SOUP_TO_POLYGON_MESH
+#endif // CGAL_POLYGON_MESH_PROCESSING_POLYGON_SOUP_TO_POLYGON_MESH_H
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/random_perturbation.h 5.1-1/include/CGAL/Polygon_mesh_processing/random_perturbation.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/random_perturbation.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/random_perturbation.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/random_perturbation.h $
-// $Id: random_perturbation.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/random_perturbation.h $
+// $Id: random_perturbation.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -74,7 +74,6 @@ namespace internal {
     if(do_project)
     {
       tree.rebuild(faces(tmesh).first, faces(tmesh).second, tmesh);
-      tree.accelerate_distance_queries();
     }
     typename GT::Construct_translated_point_3 translate
       = gt.construct_translated_point_3_object();
@@ -107,30 +106,49 @@ namespace internal {
 *         vertices of type `boost::graph_traits<TriangleMesh>::%vertex_descriptor`.
 *         Its iterator type is `ForwardIterator`.
 * @tparam TriangleMesh model of `MutableFaceGraph`.
-* @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+* @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 *
 * @param vertices the range of vertices to be perturbed
 * @param tmesh the triangulated surface mesh
 * @param perturbation_max_size the maximal length of moves that can be applied to
 *        vertices of `tmesh`.
-* @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+* @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 *
 * \cgalNamedParamsBegin
-*  \cgalParamBegin{geom_traits} a geometric traits class instance, model of `Kernel`.
-*  \cgalParamEnd
-*  \cgalParamBegin{vertex_point_map} the property map with the points associated
-*    to the vertices of `tmesh`. Instance of a class model of `ReadWritePropertyMap`.
-*  \cgalParamEnd
-*  \cgalParamBegin{vertex_is_constrained_map} a property map containing the
-*    constrained-or-not status of each vertex of `tmesh`. A constrained vertex
-*    cannot be modified at all during perturbation
-*  \cgalParamEnd
-*  \cgalParamBegin{do_project} a boolean that sets whether vertices are reprojected
-*    on the input surface after their coordinates random perturbation
-*  \cgalParamEnd
-*  \cgalParamBegin{random_seed} a non-negative integer value to seed the random
-      number generator, and make the perturbation deterministic
-*  \cgalParamEnd
+*   \cgalParamNBegin{vertex_point_map}
+*     \cgalParamDescription{a property map associating points to the vertices of `tmesh`}
+*     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+*                    as key type and `%Point_3` as value type}
+*     \cgalParamDefault{`boost::get(CGAL::vertex_point, tmesh)`}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{geom_traits}
+*     \cgalParamDescription{an instance of a geometric traits class}
+*     \cgalParamType{a class model of `Kernel`}
+*     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+*     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{vertex_is_constrained_map}
+*     \cgalParamDescription{a property map containing the constrained-or-not status of each vertex of `tmesh`}
+*     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+*                    as key type and `bool` as value type. It must be default constructible.}
+*     \cgalParamDefault{a default property map where no vertex is constrained}
+*     \cgalParamExtra{A constrained vertex cannot be modified at all during perturbation}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{do_project}
+*     \cgalParamDescription{indicates whether vertices are reprojected on the input surface
+*                           after their coordinates random perturbation}
+*     \cgalParamType{Boolean}
+*     \cgalParamDefault{`true`}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{random_seed}
+*     \cgalParamDescription{a value to seed the random number generator, and make the perturbation deterministic}
+*     \cgalParamType{unsigned int}
+*     \cgalParamDefault{`unsigned int(-1)`}
+*   \cgalParamNEnd
 * \cgalNamedParamsEnd
 *
 */
@@ -155,11 +173,11 @@ void random_perturbation(VertexRange ver
 #endif
 
   typedef typename GetGeomTraits<PM, NamedParameters>::type GT;
-  GT gt = choose_parameter(get_parameter(np, internal_np::geom_traits), GT());
+  GT gt = choose_parameter<GT>(get_parameter(np, internal_np::geom_traits));
 
   typedef typename GetVertexPointMap<PM, NamedParameters>::type VPMap;
   VPMap vpmap = choose_parameter(get_parameter(np, internal_np::vertex_point),
-                             get_property_map(vertex_point, tmesh));
+                                 get_property_map(vertex_point, tmesh));
 
   typedef typename internal_np::Lookup_named_param_def <
       internal_np::vertex_is_constrained_t,
@@ -226,4 +244,3 @@ void random_perturbation(TriangleMesh& t
 #include <CGAL/enable_warnings.h>
 
 #endif //CGAL_POLYGON_MESH_PROCESSING_RANDOM_PERTURBATION_H
-
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/refine.h 5.1-1/include/CGAL/Polygon_mesh_processing/refine.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/refine.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/refine.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/refine.h $
-// $Id: refine.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/refine.h $
+// $Id: refine.h 48bd92b 2020-04-13T13:03:05+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -36,26 +36,32 @@ namespace Polygon_mesh_processing {
     holding `boost::graph_traits<TriangleMesh>::%face_descriptor` for patch faces
   @tparam VertexOutputIterator model of `OutputIterator`
     holding `boost::graph_traits<TriangleMesh>::%vertex_descriptor` for patch vertices
-  @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+  @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 
   @param tmesh triangle mesh with patches to be refined
   @param faces the range of faces defining the patches to refine
   @param faces_out output iterator into which descriptors of new faces are recorded
   @param vertices_out output iterator into which descriptors of new vertices are recorded
-  @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+  @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
   \cgalNamedParamsBegin
-    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `tmesh`
-      Instance of a class model of `ReadWritePropertyMap`.
-      If this parameter is omitted, an internal property map for
-     `CGAL::vertex_point_t` must be available in `TriangleMesh`
-     \cgalParamEnd
-    \cgalParamBegin{density_control_factor} factor to control density of the output mesh,
-      where larger values lead to denser refinements.
-      The density of vertices of `faces_out` is this factor times higher than the vertices of `faces.` \cgalParamEnd
+    \cgalParamNBegin{vertex_point_map}
+      \cgalParamDescription{a property map associating points to the vertices of `tmesh`}
+      \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+                     as key type and `%Point_3` as value type}
+      \cgalParamDefault{`boost::get(CGAL::vertex_point, tmesh)`}
+      \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+                      must be available in `TriangleMesh`.}
+    \cgalParamNEnd
+
+    \cgalParamNBegin{density_control_factor}
+      \cgalParamDescription{a factor to control density of the output mesh, where larger values lead to denser refinements}
+      \cgalParamType{double}
+      \cgalParamDefault{\f$ \sqrt{2}\f$}
+      \cgalParamExtra{The density of vertices of `faces_out` is this factor times higher than the vertices of `faces`.}
+    \cgalParamNEnd
   \cgalNamedParamsEnd
 
-
   @return pair of `faces_out` and `vertices_out`
 
   \pre `is_triangle_mesh(tmesh)`
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/remesh.h 5.1-1/include/CGAL/Polygon_mesh_processing/remesh.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/remesh.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/remesh.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/remesh.h $
-// $Id: remesh.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/remesh.h $
+// $Id: remesh.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -41,77 +41,124 @@ namespace Polygon_mesh_processing {
 *         The descriptor types `boost::graph_traits<PolygonMesh>::%face_descriptor`
 *         and `boost::graph_traits<PolygonMesh>::%halfedge_descriptor` must be
 *         models of `Hashable`.
-*         If `PolygonMesh` has an internal property map for `CGAL::face_index_t`,
-*         and no `face_index_map` is given
-*         as a named parameter, then the internal one must be initialized
 * @tparam FaceRange range of `boost::graph_traits<PolygonMesh>::%face_descriptor`,
           model of `Range`. Its iterator type is `ForwardIterator`.
-* @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+* @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 *
 * @param pmesh a polygon mesh with triangulated surface patches to be remeshed
 * @param faces the range of triangular faces defining one or several surface patches to be remeshed
 * @param target_edge_length the edge length that is targeted in the remeshed patch.
 *        If `0` is passed then only the edge-flip, tangential relaxation, and projection steps will be done.
-* @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+* @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 *
 * @pre if constraints protection is activated, the constrained edges must
 * not be longer than 4/3*`target_edge_length`
 *
 * \cgalNamedParamsBegin
-*  \cgalParamBegin{geom_traits} a geometric traits class instance, model of `Kernel`.
-*    Exact constructions kernels are not supported by this function.
-*  \cgalParamEnd
-*  \cgalParamBegin{vertex_point_map} the property map with the points associated
-*    to the vertices of `pmesh`. Instance of a class model of `ReadWritePropertyMap`.
-*  \cgalParamEnd
-*  \cgalParamBegin{face_index_map} a property map containing the index of each face of `pmesh`
-*  \cgalParamEnd
-*  \cgalParamBegin{number_of_iterations} the number of iterations for the
-*    sequence of atomic operations performed (listed in the above description)
-*  \cgalParamEnd
-*  \cgalParamBegin{edge_is_constrained_map} a property map containing the
-*    constrained-or-not status of each edge of `pmesh`. A constrained edge can be split
-*    or collapsed, but not flipped, nor its endpoints moved by smoothing.
-*    Sub-edges generated by splitting are set to be constrained.
-*    Note that patch boundary edges (i.e. incident to only one face in the range)
-*    are always considered as constrained edges.
-*  \cgalParamEnd
-*  \cgalParamBegin{vertex_is_constrained_map} a property map containing the
-*    constrained-or-not status of each vertex of `pmesh`. A constrained vertex
-*    cannot be modified at all during remeshing
-*  \cgalParamEnd
-*  \cgalParamBegin{protect_constraints} If `true`, the edges set as constrained
-*     in `edge_is_constrained_map` (or by default the boundary edges)
-*     are not split nor collapsed during remeshing.
-*     Note that around constrained edges that have their length higher than
-*     twice `target_edge_length`, remeshing will fail to provide
-*     good quality results. It can even fail to terminate because of cascading vertex
-*     insertions.
-*  \cgalParamEnd
-*  \cgalParamBegin{collapse_constraints} If `true`, the edges set as constrained
-*     in `edge_is_constrained_map` (or by default the boundary edges)
-*     are collapsed during remeshing. This value is ignored if `protect_constraints` is true;
-*  \cgalParamEnd
-*  \cgalParamBegin{face_patch_map} a property map with the patch id's associated to the
-     faces of `faces`. Instance of a class model of `ReadWritePropertyMap`. It gets
-     updated during the remeshing process while new faces are created.
-*  \cgalParamEnd
-*  \cgalParamBegin{number_of_relaxation_steps} the number of iterations of tangential
-*    relaxation that are performed at each iteration of the remeshing process
-*  \cgalParamEnd
-*  \cgalParamBegin{relax_constraints} If `true`, the end vertices of the edges set as
-*    constrained in `edge_is_constrained_map` and boundary edges move along the
-*    constrained polylines they belong to.
-*  \cgalParamEnd
-*  \cgalParamBegin{do_project} a boolean that sets whether vertices should be reprojected
-*    on the input surface after creation or displacement.
-*  \cgalParamEnd
-*  \cgalParamBegin{projection_functor}
-*  A function object used to project input vertices (moved by the smoothing) and created vertices.
-*  It must have `%Point_3 operator()(vertex_descriptor)`, `%Point_3` being the value type
-*  of the vertex point map.
-*  If not provided, vertices are projected on the input surface mesh.
-*  \cgalParamEnd
+*   \cgalParamNBegin{vertex_point_map}
+*     \cgalParamDescription{a property map associating points to the vertices of `pmesh`}
+*     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+*                    as key type and `%Point_3` as value type}
+*     \cgalParamDefault{`boost::get(CGAL::vertex_point, pmesh)`}
+*     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+*                     must be available in `PolygonMesh`.}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{geom_traits}
+*     \cgalParamDescription{an instance of a geometric traits class}
+*     \cgalParamType{a class model of `Kernel`}
+*     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+*     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+*     \cgalParamExtra{Exact constructions kernels are not supported by this function.}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{face_index_map}
+*     \cgalParamDescription{a property map associating to each face of `pmesh` a unique index between `0` and `num_faces(pmesh) - 1`}
+*     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%face_descriptor`
+*                    as key type and `std::size_t` as value type}
+*     \cgalParamDefault{an automatically indexed internal map}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{number_of_iterations}
+*     \cgalParamDescription{the number of iterations for the sequence of atomic operations performed (listed in the above description)}
+*     \cgalParamType{unsigned int}
+*     \cgalParamDefault{`1`}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{edge_is_constrained_map}
+*     \cgalParamDescription{a property map containing the constrained-or-not status of each edge of `pmesh`}
+*     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<PolygonMesh>::%edge_descriptor`
+*                    as key type and `bool` as value type. It must be default constructible.}
+*     \cgalParamDefault{a default property map where no edge is constrained}
+*     \cgalParamExtra{A constrained edge can be split or collapsed, but not flipped, nor its endpoints moved by smoothing.}
+*     \cgalParamExtra{Sub-edges generated by splitting are set to be constrained.}
+*     \cgalParamExtra{Patch boundary edges (i.e. incident to only one face in the range) are always considered as constrained edges.}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{vertex_is_constrained_map}
+*     \cgalParamDescription{a property map containing the constrained-or-not status of each vertex of `pmesh`.}
+*     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+*                    as key type and `bool` as value type. It must be default constructible.}
+*     \cgalParamDefault{a default property map where no vertex is constrained}
+*     \cgalParamExtra{A constrained vertex cannot be modified during remeshing.}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{protect_constraints}
+*     \cgalParamDescription{If `true`, the edges set as constrained in `edge_is_constrained_map`
+*                           (or by default the boundary edges) are not split nor collapsed during remeshing.}
+*     \cgalParamType{Boolean}
+*     \cgalParamDefault{`false`}
+*     \cgalParamExtra{Note that around constrained edges that have their length higher than
+*                     twice `target_edge_length`, remeshing will fail to provide good quality results.
+*                     It can even fail to terminate because of cascading vertex insertions.}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{collapse_constraints}
+*     \cgalParamDescription{If `true`, the edges set as constrained in `edge_is_constrained_map`
+*                           (or by default the boundary edges) are collapsed during remeshing.}
+*     \cgalParamType{Boolean}
+*     \cgalParamDefault{`true`}
+*     \cgalParamExtra{This value is ignored if `protect_constraints` is `true`.}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{face_patch_map}
+*     \cgalParamDescription{a property map with the patch id's associated to the faces of `faces`}
+*     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<PolygonMesh>::%face_descriptor`
+*                    as key type and the desired property, model of `CopyConstructible` as value type.}
+*     \cgalParamDefault{a default property map where each face is associated with the ID of
+*                       the connected component it belongs to. Connected components are
+*                       computed with respect to the constrained edges listed in the property map
+*                       `edge_is_constrained_map`}
+*     \cgalParamExtra{The map is updated during the remeshing process while new faces are created.}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{number_of_relaxation_steps}
+*     \cgalParamDescription{the number of iterations of tangential relaxation that are performed
+*                           at each iteration of the remeshing process}
+*     \cgalParamType{unsigned int}
+*     \cgalParamDefault{`1`}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{relax_constraints}
+*     \cgalParamDescription{If `true`, the end vertices of the edges set as constrained
+*                           in `edge_is_constrained_map` and boundary edges move along the}
+*                           constrained polylines they belong to.}
+*     \cgalParamType{Boolean}
+*     \cgalParamDefault{`false`}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{do_project}
+*     \cgalParamDescription{whether vertices should be reprojected on the input surface after creation or displacement}
+*     \cgalParamType{Boolean}
+*     \cgalParamDefault{`true`}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{projection_functor}
+*     \cgalParamDescription{A function object used to project input vertices (moved by the smoothing) and created vertices}
+*     \cgalParamType{Unary functor that provides `%Point_3 operator()(vertex_descriptor)`, `%Point_3` being the value type
+*                    of the vertex point map.}
+*     \cgalParamDefault{If not provided, vertices are projected on the input surface mesh.}
+*   \cgalParamNEnd
 * \cgalNamedParamsEnd
 *
 * @sa `split_long_edges()`
@@ -139,6 +186,7 @@ void isotropic_remeshing(const FaceRange
   typedef PolygonMesh PM;
   typedef typename boost::graph_traits<PM>::vertex_descriptor vertex_descriptor;
   typedef typename boost::graph_traits<PM>::edge_descriptor edge_descriptor;
+
   using parameters::get_parameter;
   using parameters::choose_parameter;
 
@@ -154,15 +202,14 @@ void isotropic_remeshing(const FaceRange
     parameters::is_default_parameter(get_parameter(np, internal_np::projection_functor));
 
   typedef typename GetGeomTraits<PM, NamedParameters>::type GT;
-  GT gt = choose_parameter(get_parameter(np, internal_np::geom_traits), GT());
+  GT gt = choose_parameter<GT>(get_parameter(np, internal_np::geom_traits));
 
   typedef typename GetVertexPointMap<PM, NamedParameters>::type VPMap;
   VPMap vpmap = choose_parameter(get_parameter(np, internal_np::vertex_point),
-                             get_property_map(vertex_point, pmesh));
+                                 get_property_map(vertex_point, pmesh));
 
-  typedef typename GetFaceIndexMap<PM, NamedParameters>::type FIMap;
-  FIMap fimap = choose_parameter(get_parameter(np, internal_np::face_index),
-                           get_property_map(face_index, pmesh));
+  typedef typename GetInitializedFaceIndexMap<PolygonMesh, NamedParameters>::type FIMap;
+  FIMap fimap = CGAL::get_initialized_face_index_map(pmesh, np);
 
   typedef typename internal_np::Lookup_named_param_def <
       internal_np::edge_is_constrained_t,
@@ -294,24 +341,38 @@ void isotropic_remeshing(
 *         has an internal property map for `CGAL::vertex_point_t`.
 * @tparam EdgeRange range of `boost::graph_traits<PolygonMesh>::%edge_descriptor`,
 *   model of `Range`. Its iterator type is `InputIterator`.
-* @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+* @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 *
 * @param pmesh a polygon mesh
 * @param edges the range of edges to be split if they are longer than given threshold
 * @param max_length the edge length above which an edge from `edges` is split
 *        into to sub-edges
-* @param np optional \ref pmp_namedparameters "Named Parameters", amongst those described below
+* @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
 * \cgalNamedParamsBegin
-*  \cgalParamBegin{vertex_point_map} the property map with the points associated
-*    to the vertices of `pmesh`. Instance of a class model of `ReadWritePropertyMap`.
-*  \cgalParamEnd
-*  \cgalParamBegin{face_index_map} a property map containing the index of each face of `pmesh`
-*  \cgalParamEnd
-*  \cgalParamBegin{edge_is_constrained_map} a property map containing the
-*    constrained-or-not status of each edge of `pmesh`. A constrained edge can be split,
-*    and the sub-edges are set to be constrained.
-*  \cgalParamEnd
+*   \cgalParamNBegin{vertex_point_map}
+*     \cgalParamDescription{a property map associating points to the vertices of `pmesh`}
+*     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+*                    as key type and `%Point_3` as value type}
+*     \cgalParamDefault{`boost::get(CGAL::vertex_point, pmesh)`}
+*     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+*                     must be available in `PolygonMesh`.}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{face_index_map}
+*     \cgalParamDescription{a property map associating to each face of `pmesh` a unique index between `0` and `num_faces(pmesh) - 1`}
+*     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%face_descriptor`
+*                    as key type and `std::size_t` as value type}
+*     \cgalParamDefault{an automatically indexed internal map}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{edge_is_constrained_map}
+*     \cgalParamDescription{a property map containing the constrained-or-not status of each edge of `pmesh`}
+*     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<PolygonMesh>::%edge_descriptor`
+*                    as key type and `bool` as value type. It must be default constructible.}
+*     \cgalParamDefault{a default property map where no edge is constrained}
+*     \cgalParamExtra{A constrained edge can be split or collapsed, but not flipped, nor its endpoints moved by smoothing.}
+*   \cgalParamNEnd
 * \cgalNamedParamsEnd
 *
 * @sa `isotropic_remeshing()`
@@ -332,15 +393,14 @@ void split_long_edges(const EdgeRange& e
   using parameters::get_parameter;
 
   typedef typename GetGeomTraits<PM, NamedParameters>::type GT;
-  GT gt = choose_parameter(get_parameter(np, internal_np::geom_traits), GT());
+  GT gt = choose_parameter<GT>(get_parameter(np, internal_np::geom_traits));
 
   typedef typename GetVertexPointMap<PM, NamedParameters>::type VPMap;
   VPMap vpmap = choose_parameter(get_parameter(np, internal_np::vertex_point),
-                             get_property_map(vertex_point, pmesh));
+                                 get_property_map(vertex_point, pmesh));
 
-  typedef typename GetFaceIndexMap<PM, NamedParameters>::type FIMap;
-  FIMap fimap = choose_parameter(get_parameter(np, internal_np::face_index),
-                             get_property_map(face_index, pmesh));
+  typedef typename GetInitializedFaceIndexMap<PolygonMesh, NamedParameters>::type FIMap;
+  FIMap fimap = CGAL::get_initialized_face_index_map(pmesh, np);
 
   typedef typename internal_np::Lookup_named_param_def <
         internal_np::edge_is_constrained_t,
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/repair_degeneracies.h 5.1-1/include/CGAL/Polygon_mesh_processing/repair_degeneracies.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/repair_degeneracies.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/repair_degeneracies.h	2020-09-08 12:15:23.000000000 +0000
@@ -0,0 +1,2404 @@
+// Copyright (c) 2015-2019 GeometryFactory (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/repair_degeneracies.h $
+// $Id: repair_degeneracies.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Sebastien Loriot,
+//                 Mael Rouxel-Labbé
+//
+#ifndef CGAL_POLYGON_MESH_PROCESSING_REPAIR_DEGENERACIES_H
+#define CGAL_POLYGON_MESH_PROCESSING_REPAIR_DEGENERACIES_H
+
+#include <CGAL/license/Polygon_mesh_processing/repair.h>
+
+#include <CGAL/Polygon_mesh_processing/shape_predicates.h>
+#include <CGAL/Polygon_mesh_processing/measure.h>
+
+#include <CGAL/boost/graph/selection.h>
+#include <CGAL/boost/graph/Euler_operations.h>
+#include <CGAL/boost/graph/Named_function_parameters.h>
+#include <CGAL/Dynamic_property_map.h>
+#include <CGAL/property_map.h>
+#include <CGAL/Union_find.h>
+
+#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
+#include <CGAL/Polygon_mesh_processing/polygon_soup_to_polygon_mesh.h>
+#include <CGAL/IO/OFF_reader.h>
+#endif
+
+#include <boost/algorithm/minmax_element.hpp>
+#include <boost/utility/enable_if.hpp>
+
+#include <algorithm>
+#include <fstream>
+#include <iostream>
+#include <iterator>
+#include <map>
+#include <set>
+#include <sstream>
+#include <utility>
+#include <vector>
+
+// First part of the file: remove_ALMOST_degenerate_faces (needles/caps)
+// Second part of the file: remove_degenerate_edges/faces
+
+namespace CGAL {
+namespace Polygon_mesh_processing {
+namespace internal {
+
+template <typename TriangleMesh, typename VPM, typename ECM, typename Traits>
+std::array<typename boost::graph_traits<TriangleMesh>::halfedge_descriptor, 2>
+is_badly_shaped(const typename boost::graph_traits<TriangleMesh>::face_descriptor f,
+                TriangleMesh& tmesh,
+                const VPM& vpm,
+                const ECM& ecm,
+                const Traits& gt,
+                const double cap_threshold, // angle over 160° ==> cap
+                const double needle_threshold, // longest edge / shortest edge over this ratio ==> needle
+                const double collapse_length_threshold) // max length of edges allowed to be collapsed
+{
+  namespace PMP = CGAL::Polygon_mesh_processing;
+
+  typedef typename boost::graph_traits<TriangleMesh>::halfedge_descriptor       halfedge_descriptor;
+
+  const halfedge_descriptor null_h = boost::graph_traits<TriangleMesh>::null_halfedge();
+
+  halfedge_descriptor res = PMP::is_needle_triangle_face(f, tmesh, needle_threshold,
+                                                         parameters::vertex_point_map(vpm)
+                                                                    .geom_traits(gt));
+  if(res != null_h && !get(ecm, edge(res, tmesh)))
+  {
+    // don't want to collapse edges that are too large
+    if(collapse_length_threshold == 0  ||
+       edge_length(res, tmesh, parameters::vertex_point_map(vpm).geom_traits(gt)) <= collapse_length_threshold)
+    {
+      return make_array(res, null_h);
+    }
+  }
+  else // let's not make it possible to have a face be both a cap and a needle (for now)
+  {
+    res = PMP::is_cap_triangle_face(f, tmesh, cap_threshold, parameters::vertex_point_map(vpm).geom_traits(gt));
+    if(res != null_h && !get(ecm, edge(res, tmesh)))
+      return make_array(null_h, res);
+  }
+
+  return make_array(null_h, null_h);
+}
+
+template <typename TriangleMesh, typename EdgeContainer,
+          typename VPM, typename ECM, typename Traits>
+void collect_badly_shaped_triangles(const typename boost::graph_traits<TriangleMesh>::face_descriptor f,
+                                    TriangleMesh& tmesh,
+                                    const VPM& vpm,
+                                    const ECM& ecm,
+                                    const Traits& gt,
+                                    const double cap_threshold, // angle over this threshold (as a cosine) ==> cap
+                                    const double needle_threshold, // longest edge / shortest edge over this ratio ==> needle
+                                    const double collapse_length_threshold, // max length of edges allowed to be collapsed
+                                    EdgeContainer& edges_to_collapse,
+                                    EdgeContainer& edges_to_flip)
+{
+  typedef typename boost::graph_traits<TriangleMesh>::halfedge_descriptor       halfedge_descriptor;
+
+  std::array<halfedge_descriptor, 2> res = is_badly_shaped(f, tmesh, vpm, ecm, gt, cap_threshold,
+                                                           needle_threshold, collapse_length_threshold);
+
+  if(res[0] != boost::graph_traits<TriangleMesh>::null_halfedge())
+  {
+#ifdef CGAL_PMP_DEBUG_REMOVE_DEGENERACIES
+    std::cout << "add new needle: " << edge(res[0], tmesh) << std::endl;
+#endif
+    edges_to_collapse.insert(edge(res[0], tmesh));
+  }
+  else // let's not make it possible to have a face be both a cap and a needle (for now)
+  {
+    if(res[1] != boost::graph_traits<TriangleMesh>::null_halfedge())
+    {
+#ifdef CGAL_PMP_DEBUG_REMOVE_DEGENERACIES
+      std::cout << "add new cap: " << edge(res[1],tmesh) << std::endl;
+#endif
+      edges_to_flip.insert(edge(res[1], tmesh));
+    }
+  }
+}
+
+/*
+// Following Ronfard et al. 96 we look at variation of the normal after the collapse
+// the collapse must be topologically valid
+template <class TriangleMesh, class NamedParameters>
+bool is_collapse_geometrically_valid(typename boost::graph_traits<TriangleMesh>::halfedge_descriptor h,
+                                     const TriangleMesh& tmesh,
+                                     const NamedParameters& np)
+{
+  using CGAL::parameters::choose_parameter;
+  using CGAL::parameters::get_parameter;
+
+  typedef typename boost::graph_traits<TriangleMesh>::halfedge_descriptor         halfedge_descriptor;
+  typedef typename GetVertexPointMap<TriangleMesh, NamedParameters>::const_type   VPM;
+  typedef typename boost::property_traits<VPM>::reference                         Point_ref;
+  typedef typename GetGeomTraits<TriangleMesh, NamedParameters>::type             Traits;
+
+  VPM vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
+                             get_const_property_map(vertex_point, tmesh));
+  Traits gt = choose_parameter(get_parameter(np, internal_np::geom_traits), Traits());
+
+  // @todo handle boundary edges
+
+  h = opposite(h, tmesh); // Euler::collapse edge keeps the target and removes the source
+
+  // source is kept, target is removed
+  CGAL_assertion(target(h, tmesh) == vertex_removed);
+  Point_ref kept = get(vpm, source(h, tmesh));
+  Point_ref removed= get(vpm, target(h, tmesh));
+
+  // consider triangles incident to the vertex removed
+  halfedge_descriptor stop = prev(opposite(h, tmesh), tmesh);
+  halfedge_descriptor hi = opposite(next(h, tmesh), tmesh);
+
+  std::vector<halfedge_descriptor> triangles;
+  while(hi != stop)
+  {
+    if(!is_border(hi, tmesh))
+    {
+      Point_ref a = get(vpm, target(next(hi, tmesh), tmesh));
+      Point_ref b = get(vpm, source(hi, tmesh));
+
+      //ack a-b-point_remove and a-b-point_kept has a compatible orientation
+      // @todo use a predicate
+      typename Traits::Vector_3 n1 = gt.construct_cross_product_vector_3_object()(removed-a, b-a);
+      typename Traits::Vector_3 n2 = gt.construct_cross_product_vector_3_object()(kept-a, b-a);
+      if(gt.compute_scalar_product_3_object()(n1, n2) <= 0)
+        return false;
+    }
+
+    hi = opposite(next(hi, tmesh), tmesh);
+  }
+
+  return true;
+}
+*/
+
+template <class TriangleMesh, typename VPM, typename Traits>
+boost::optional<double> get_collapse_volume(typename boost::graph_traits<TriangleMesh>::halfedge_descriptor h,
+                                            const TriangleMesh& tmesh,
+                                            const VPM& vpm,
+                                            const Traits& gt)
+{
+  typedef typename boost::graph_traits<TriangleMesh>::halfedge_descriptor         halfedge_descriptor;
+
+  typedef typename boost::property_traits<VPM>::reference                         Point_ref;
+  typedef typename Traits::Vector_3                                               Vector_3;
+
+  const typename Traits::Point_3 origin(ORIGIN);
+
+// @todo handle boundary edges
+
+  h = opposite(h, tmesh); // Euler::collapse edge keeps the target and removes the source
+
+  // source is kept, target is removed
+  Point_ref kept = get(vpm, source(h, tmesh));
+  Point_ref removed= get(vpm, target(h, tmesh));
+
+  // init volume with incident triangles (reversed orientation
+  double delta_vol = volume(removed, kept, get(vpm, target(next(h, tmesh), tmesh)), origin) +
+                     volume(kept, removed, get(vpm, target(next(opposite(h, tmesh), tmesh), tmesh)), origin);
+
+  // consider triangles incident to the vertex removed
+  halfedge_descriptor stop = prev(opposite(h, tmesh), tmesh);
+  halfedge_descriptor hi = opposite(next(h, tmesh), tmesh);
+
+  std::vector<halfedge_descriptor> triangles;
+  while(hi != stop)
+  {
+    if(!is_border(hi, tmesh))
+    {
+      Point_ref a = get(vpm, target(next(hi, tmesh), tmesh));
+      Point_ref b = get(vpm, source(hi, tmesh));
+
+      //ack a-b-point_remove and a-b-point_kept has a compatible orientation
+      // @todo use a predicate
+      Vector_3 v_ab = gt.construct_vector_3_object()(a, b);
+      Vector_3 v_ar = gt.construct_vector_3_object()(a, removed);
+      Vector_3 v_ak = gt.construct_vector_3_object()(a, kept);
+
+      Vector_3 n1 = gt.construct_cross_product_vector_3_object()(v_ar, v_ab);
+      Vector_3 n2 = gt.construct_cross_product_vector_3_object()(v_ak, v_ab);
+      if(gt.compute_scalar_product_3_object()(n1, n2) <= 0)
+        return boost::none;
+
+      delta_vol += volume(b, a, removed, origin) + volume(a, b, kept, origin); // opposite orientation
+    }
+
+    hi = opposite(next(hi, tmesh), tmesh);
+  }
+
+  return CGAL::abs(delta_vol);
+}
+
+template <typename TriangleMesh, typename VPM, typename VCM, typename Traits>
+typename boost::graph_traits<TriangleMesh>::halfedge_descriptor
+get_best_edge_orientation(typename boost::graph_traits<TriangleMesh>::edge_descriptor e,
+                          const TriangleMesh& tmesh,
+                          const VPM& vpm,
+                          const VCM& vcm,
+                          const Traits& gt)
+{
+  typedef typename boost::graph_traits<TriangleMesh>::halfedge_descriptor halfedge_descriptor;
+
+  halfedge_descriptor h = halfedge(e, tmesh), ho = opposite(h, tmesh);
+
+  CGAL_assertion(!get(vcm, source(h, tmesh)) || !get(vcm, target(h, tmesh)));
+
+  boost::optional<double> dv1 = get_collapse_volume(h, tmesh, vpm, gt);
+  boost::optional<double> dv2 = get_collapse_volume(ho, tmesh, vpm, gt);
+
+  // the resulting point of the collapse of a halfedge is the target of the halfedge before collapse
+  if(get(vcm, source(h, tmesh)))
+     return dv2 != boost::none ? ho
+                               : boost::graph_traits<TriangleMesh>::null_halfedge();
+
+  if(get(vcm, target(h, tmesh)))
+     return dv1 != boost::none ? h
+                               : boost::graph_traits<TriangleMesh>::null_halfedge();
+
+  if(dv1 != boost::none)
+  {
+    if(dv2 != boost::none)
+      return (*dv1 < *dv2) ? h : ho;
+
+    return h;
+  }
+
+  if(dv2 != boost::none)
+    return ho;
+
+  return boost::graph_traits<TriangleMesh>::null_halfedge();
+}
+
+// adapted from triangulate_faces
+template <typename TriangleMesh, typename VPM, typename Traits>
+bool should_flip(typename boost::graph_traits<TriangleMesh>::edge_descriptor e,
+                 const TriangleMesh& tmesh,
+                 const VPM& vpm,
+                 const Traits& gt)
+{
+  typedef typename boost::graph_traits<TriangleMesh>::halfedge_descriptor halfedge_descriptor;
+
+  typedef typename boost::property_traits<VPM>::reference                 Point_ref;
+  typedef typename Traits::Vector_3                                       Vector_3;
+
+  CGAL_precondition(!is_border(e, tmesh));
+
+  halfedge_descriptor h = halfedge(e, tmesh);
+
+  Point_ref p0 = get(vpm, target(h, tmesh));
+  Point_ref p1 = get(vpm, target(next(h, tmesh), tmesh));
+  Point_ref p2 = get(vpm, source(h, tmesh));
+  Point_ref p3 = get(vpm, target(next(opposite(h, tmesh), tmesh), tmesh));
+
+  /* Chooses the diagonal that will split the quad in two triangles that maximize
+   * the scalar product of of the un-normalized normals of the two triangles.
+   * The lengths of the un-normalized normals (computed using cross-products of two vectors)
+   *  are proportional to the area of the triangles.
+   * Maximize the scalar product of the two normals will avoid skinny triangles,
+   * and will also taken into account the cosine of the angle between the two normals.
+   * In particular, if the two triangles are oriented in different directions,
+   * the scalar product will be negative.
+   */
+
+//  CGAL::cross_product(p2-p1, p3-p2) * CGAL::cross_product(p0-p3, p1-p0);
+//  CGAL::cross_product(p1-p0, p1-p2) * CGAL::cross_product(p3-p2, p3-p0);
+
+  const Vector_3 v01 = gt.construct_vector_3_object()(p0, p1);
+  const Vector_3 v12 = gt.construct_vector_3_object()(p1, p2);
+  const Vector_3 v23 = gt.construct_vector_3_object()(p2, p3);
+  const Vector_3 v30 = gt.construct_vector_3_object()(p3, p0);
+
+  const double p1p3 = gt.compute_scalar_product_3_object()(
+                        gt.construct_cross_product_vector_3_object()(v12, v23),
+                        gt.construct_cross_product_vector_3_object()(v30, v01));
+
+  const Vector_3 v21 = gt.construct_opposite_vector_3_object()(v12);
+  const Vector_3 v03 = gt.construct_opposite_vector_3_object()(v30);
+
+  const double p0p2 = gt.compute_scalar_product_3_object()(
+                        gt.construct_cross_product_vector_3_object()(v01, v21),
+                        gt.construct_cross_product_vector_3_object()(v23, v03));
+
+  return p0p2 <= p1p3;
+}
+
+} // namespace internal
+
+namespace experimental {
+
+// @todo check what to use as priority queue with removable elements, set might not be optimal
+template <typename FaceRange, typename TriangleMesh, typename NamedParameters>
+bool remove_almost_degenerate_faces(const FaceRange& face_range,
+                                    TriangleMesh& tmesh,
+                                    const double cap_threshold,
+                                    const double needle_threshold,
+                                    const double collapse_length_threshold,
+                                    const NamedParameters& np)
+{
+  using CGAL::parameters::choose_parameter;
+  using CGAL::parameters::get_parameter;
+
+  typedef typename boost::graph_traits<TriangleMesh>::vertex_descriptor         vertex_descriptor;
+  typedef typename boost::graph_traits<TriangleMesh>::halfedge_descriptor       halfedge_descriptor;
+  typedef typename boost::graph_traits<TriangleMesh>::edge_descriptor           edge_descriptor;
+  typedef typename boost::graph_traits<TriangleMesh>::face_descriptor           face_descriptor;
+
+  typedef Constant_property_map<vertex_descriptor, bool>                        Default_VCM;
+  typedef typename internal_np::Lookup_named_param_def<internal_np::vertex_is_constrained_t,
+                                                       NamedParameters,
+                                                       Default_VCM>::type       VCM;
+  VCM vcm_np = choose_parameter(get_parameter(np, internal_np::vertex_is_constrained), Default_VCM(false));
+
+  typedef Constant_property_map<edge_descriptor, bool>                          Default_ECM;
+  typedef typename internal_np::Lookup_named_param_def<internal_np::edge_is_constrained_t,
+                                                       NamedParameters,
+                                                       Default_ECM>::type       ECM;
+  ECM ecm = choose_parameter(get_parameter(np, internal_np::edge_is_constrained), Default_ECM(false));
+
+  typedef typename GetVertexPointMap<TriangleMesh, NamedParameters>::const_type VPM;
+  VPM vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
+                             get_const_property_map(vertex_point, tmesh));
+
+  typedef typename GetGeomTraits<TriangleMesh, NamedParameters>::type           Traits;
+  Traits gt = choose_parameter(get_parameter(np, internal_np::geom_traits), Traits());
+
+  // Vertex property map that combines the VCM and the fact that extremities of a constrained edge should be constrained
+  typedef CGAL::dynamic_vertex_property_t<bool>                                 Vertex_property_tag;
+  typedef typename boost::property_map<TriangleMesh, Vertex_property_tag>::type DVCM;
+  DVCM vcm = get(Vertex_property_tag(), tmesh);
+
+  for(face_descriptor f : face_range)
+  {
+    if(f == boost::graph_traits<TriangleMesh>::null_face())
+      continue;
+
+    for(halfedge_descriptor h : CGAL::halfedges_around_face(halfedge(f, tmesh), tmesh))
+    {
+      if(get(ecm, edge(h, tmesh)))
+      {
+        put(vcm, source(h, tmesh), true);
+        put(vcm, target(h, tmesh), true);
+      }
+      else if(get(vcm_np, target(h, tmesh)))
+      {
+        put(vcm, target(h, tmesh), true);
+      }
+    }
+  }
+
+  // Start the process of removing bad elements
+  std::set<edge_descriptor> edges_to_collapse;
+  std::set<edge_descriptor> edges_to_flip;
+
+  // @todo could probably do something a bit better by looping edges, consider the incident faces
+  // f1 / f2 and look at f1 if f1<f2, and the edge is smaller than the two other edges...
+  for(face_descriptor f : face_range)
+    internal::collect_badly_shaped_triangles(f, tmesh, vpm, ecm, gt,
+                                             cap_threshold, needle_threshold, collapse_length_threshold,
+                                             edges_to_collapse, edges_to_flip);
+
+#ifdef CGAL_PMP_DEBUG_REMOVE_DEGENERACIES
+  int iter = 0;
+#endif
+
+  for(;;)
+  {
+    bool something_was_done = false;
+
+#ifdef CGAL_PMP_DEBUG_REMOVE_DEGENERACIES
+    std::cout << edges_to_collapse.size() << " needles and " << edges_to_flip.size() << " caps" << std::endl;
+    std::ostringstream oss;
+    oss << "degen_cleaning_iter_" << iter++ << ".off";
+    std::ofstream out(oss.str().c_str());
+    out << std::setprecision(17);
+    out << tmesh;
+    out.close();
+#endif
+
+    if(edges_to_collapse.empty() && edges_to_flip.empty())
+      return true;
+
+    // @todo maybe using a priority queue handling the more almost degenerate elements should be used
+    std::set<edge_descriptor> next_edges_to_collapse;
+    std::set<edge_descriptor> next_edges_to_flip;
+
+    // treat needles
+#ifdef CGAL_PMP_DEBUG_REMOVE_DEGENERACIES_EXTRA
+    int kk=0;
+    std::ofstream(std::string("tmp/n-00000.off")) << tmesh;
+#endif
+    while(!edges_to_collapse.empty())
+    {
+      edge_descriptor e = *edges_to_collapse.begin();
+      edges_to_collapse.erase(edges_to_collapse.begin());
+
+      CGAL_assertion(!get(ecm, e));
+
+      if(get(vcm, source(e, tmesh)) && get(vcm, target(e, tmesh)))
+        continue;
+
+#ifdef CGAL_PMP_DEBUG_REMOVE_DEGENERACIES
+      std::cout << "  treat needle: " << e << " (" << tmesh.point(source (e, tmesh))
+                                        << " --- " << tmesh.point(target(e, tmesh)) << ")" << std::endl;
+#endif
+      if(CGAL::Euler::does_satisfy_link_condition(e, tmesh))
+      {
+        // the following edges are removed by the collapse
+        halfedge_descriptor h = halfedge(e, tmesh);
+        CGAL_assertion(!is_border(h, tmesh)); // because extracted from a face
+
+        std::array<halfedge_descriptor, 2> nc =
+          internal::is_badly_shaped(face(h, tmesh), tmesh, vpm, ecm, gt,
+                                    cap_threshold, needle_threshold, collapse_length_threshold);
+
+        if(nc[0] != h)
+        {
+#ifdef CGAL_PMP_DEBUG_REMOVE_DEGENERACIES
+          std::cerr << "Warning: Needle criteria no longer verified " << tmesh.point(source(e, tmesh)) << " "
+                                                                      << tmesh.point(target(e, tmesh)) << std::endl;
+#endif
+          // the opposite edge might also have been inserted in the set and might still be a needle
+          h = opposite(h, tmesh);
+          if(is_border(h, tmesh))
+            continue;
+
+          nc = internal::is_badly_shaped(face(h, tmesh), tmesh, vpm, ecm, gt,
+                                         cap_threshold, needle_threshold,
+                                         collapse_length_threshold);
+          if(nc[0] != h)
+            continue;
+        }
+
+        for(int i=0; i<2; ++i)
+        {
+          if(!is_border(h, tmesh))
+          {
+            edge_descriptor pe = edge(prev(h, tmesh), tmesh);
+            edges_to_flip.erase(pe);
+            next_edges_to_collapse.erase(pe);
+            edges_to_collapse.erase(pe);
+          }
+
+          h = opposite(h, tmesh);
+        }
+
+        // pick the orientation of edge to keep the vertex minimizing the volume variation
+        h = internal::get_best_edge_orientation(e, tmesh, vpm, vcm, gt);
+
+        if(h == boost::graph_traits<TriangleMesh>::null_halfedge())
+        {
+#ifdef CGAL_PMP_DEBUG_REMOVE_DEGENERACIES
+            std::cerr << "Warning: geometrically invalid edge collapse! "
+                      << tmesh.point(source(e, tmesh)) << " "
+                      << tmesh.point(target(e, tmesh)) << std::endl;
+#endif
+          next_edges_to_collapse.insert(e);
+          continue;
+        }
+
+        edges_to_flip.erase(e);
+        next_edges_to_collapse.erase(e); // for edges added in faces incident to a vertex kept after a collapse
+#ifdef CGAL_PMP_DEBUG_REMOVE_DEGENERACIES_EXTRA
+        std::cerr << "  " << kk << " -- Collapsing " << tmesh.point(source(h, tmesh)) << "  "
+                                                     << tmesh.point(target(h, tmesh)) << std::endl;
+#endif
+        // moving to the midpoint is not a good idea. On a circle for example you might endpoint with
+        // a bad geometry because you iteratively move one point
+        // auto mp = midpoint(tmesh.point(source(h, tmesh)), tmesh.point(target(h, tmesh)));
+
+        vertex_descriptor v = Euler::collapse_edge(edge(h, tmesh), tmesh);
+
+        //tmesh.point(v) = mp;
+        // examine all faces incident to the vertex kept
+        for(halfedge_descriptor hv : halfedges_around_target(v, tmesh))
+        {
+          if(!is_border(hv, tmesh))
+          {
+            internal::collect_badly_shaped_triangles(face(hv, tmesh), tmesh, vpm, ecm, gt,
+                                                     cap_threshold, needle_threshold, collapse_length_threshold,
+                                                     edges_to_collapse, edges_to_flip);
+          }
+        }
+
+#ifdef CGAL_PMP_DEBUG_REMOVE_DEGENERACIES_EXTRA
+        std::string nb = std::to_string(++kk);
+        if(kk<10) nb = std::string("0")+nb;
+        if(kk<100) nb = std::string("0")+nb;
+        if(kk<1000) nb = std::string("0")+nb;
+        if(kk<10000) nb = std::string("0")+nb;
+        std::ofstream(std::string("tmp/n-")+nb+std::string(".off")) << tmesh;
+#endif
+        something_was_done = true;
+      }
+      else
+      {
+#ifdef CGAL_PMP_DEBUG_REMOVE_DEGENERACIES
+        std::cerr << "Warning: uncollapsable edge! " << tmesh.point(source(e, tmesh)) << " "
+                                                     << tmesh.point(target(e, tmesh)) << std::endl;
+#endif
+        next_edges_to_collapse.insert(e);
+      }
+    }
+
+    // treat caps
+#ifdef CGAL_PMP_DEBUG_REMOVE_DEGENERACIES_EXTRA
+    kk=0;
+    std::ofstream(std::string("tmp/c-000.off")) << tmesh;
+#endif
+    while(!edges_to_flip.empty())
+    {
+      edge_descriptor e = *edges_to_flip.begin();
+      edges_to_flip.erase(edges_to_flip.begin());
+
+      CGAL_assertion(!get(ecm, e));
+
+      if(get(vcm, source(e, tmesh)) && get(vcm, target(e, tmesh)))
+        continue;
+
+#ifdef CGAL_PMP_DEBUG_REMOVE_DEGENERACIES
+      std::cout << "treat cap: " << e << " (" << tmesh.point(source(e, tmesh))
+                                   << " --- " << tmesh.point(target(e, tmesh)) << ")" << std::endl;
+#endif
+
+      halfedge_descriptor h = halfedge(e, tmesh);
+      std::array<halfedge_descriptor,2> nc = internal::is_badly_shaped(face(h, tmesh), tmesh, vpm, ecm, gt,
+                                                                       cap_threshold, needle_threshold,
+                                                                       collapse_length_threshold);
+      // First check the triangle is still a cap
+      if(nc[1] != h)
+      {
+#ifdef CGAL_PMP_DEBUG_REMOVE_DEGENERACIES
+        std::cerr << "Warning: Cap criteria no longer verified " << tmesh.point(source(e, tmesh)) << " --- "
+                                                                 << tmesh.point(target(e, tmesh)) << std::endl;
+#endif
+        // the opposite edge might also have been inserted in the set and might still be a cap
+        h = opposite(h, tmesh);
+        if(is_border(h, tmesh))
+          continue;
+
+        nc = internal::is_badly_shaped(face(h, tmesh), tmesh, vpm, ecm, gt,
+                                       cap_threshold, needle_threshold, collapse_length_threshold);
+        if(nc[1] != h)
+          continue;
+      }
+
+      // special case on the border
+      if(is_border(opposite(h, tmesh), tmesh))
+      {
+        // remove the triangle
+        edges_to_flip.erase(edge(prev(h, tmesh), tmesh));
+        edges_to_flip.erase(edge(next(h, tmesh), tmesh));
+        next_edges_to_collapse.erase(edge(prev(h, tmesh), tmesh));
+        next_edges_to_collapse.erase(edge(next(h, tmesh), tmesh));
+        Euler::remove_face(h, tmesh);
+        something_was_done = true;
+        continue;
+      }
+
+      // condition for the flip to be valid (the edge to be created does not already exist)
+      if(!halfedge(target(next(h, tmesh), tmesh),
+                   target(next(opposite(h, tmesh), tmesh), tmesh), tmesh).second)
+      {
+
+        if(!internal::should_flip(e, tmesh, vpm, gt))
+        {
+#ifdef CGAL_PMP_DEBUG_REMOVE_DEGENERACIES
+          std::cout << "Flipping prevented: not the best diagonal" << std::endl;
+#endif
+          next_edges_to_flip.insert(e);
+          continue;
+        }
+
+#ifdef CGAL_PMP_DEBUG_REMOVE_DEGENERACIES
+        std::cout << "Flipping" << std::endl;
+#endif
+#ifdef CGAL_PMP_DEBUG_REMOVE_DEGENERACIES_EXTRA
+        std::cerr << "step " << kk << "\n";
+        std::cerr << "  Flipping " << tmesh.point(source(h, tmesh)) << "  "
+                                                   << tmesh.point(target(h, tmesh)) << std::endl;
+#endif
+        Euler::flip_edge(h, tmesh);
+        CGAL_assertion(edge(h, tmesh) == e);
+
+        // handle face updates
+        for(int i=0; i<2; ++i)
+        {
+          CGAL_assertion(!is_border(h, tmesh));
+          std::array<halfedge_descriptor, 2> nc =
+            internal::is_badly_shaped(face(h, tmesh), tmesh, vpm, ecm, gt,
+                                      cap_threshold, needle_threshold, collapse_length_threshold);
+
+          if(nc[1] != boost::graph_traits<TriangleMesh>::null_halfedge())
+          {
+            if(edge(nc[1], tmesh) != e)
+              next_edges_to_flip.insert(edge(nc[1], tmesh));
+          }
+          else
+          {
+            if(nc[0] != boost::graph_traits<TriangleMesh>::null_halfedge())
+            {
+              next_edges_to_collapse.insert(edge(nc[0], tmesh));
+            }
+          }
+          h = opposite(h, tmesh);
+        }
+        something_was_done = true;
+      }
+#ifdef CGAL_PMP_DEBUG_REMOVE_DEGENERACIES
+      else
+      {
+        std::cerr << "Warning: unflippable edge! " << tmesh.point(source(h, tmesh)) << " --- "
+                                                   << tmesh.point(target(h, tmesh)) << std::endl;
+        next_edges_to_flip.insert(e);
+      }
+#endif
+#ifdef CGAL_PMP_DEBUG_REMOVE_DEGENERACIES_EXTRA
+      std::string nb = std::to_string(++kk);
+      if(kk<10) nb = std::string("0")+nb;
+      if(kk<100) nb = std::string("0")+nb;
+      if(kk<1000) nb = std::string("0")+nb;
+      if(kk<10000) nb = std::string("0")+nb;
+      std::ofstream(std::string("tmp/c-")+nb+std::string(".off")) << tmesh;
+#endif
+    }
+
+    std::swap(edges_to_collapse, next_edges_to_collapse);
+    std::swap(edges_to_flip, next_edges_to_flip);
+
+    if(!something_was_done)
+      return false;
+  }
+
+  return false;
+}
+
+template <typename FaceRange, typename TriangleMesh>
+bool remove_almost_degenerate_faces(const FaceRange& face_range,
+                                    TriangleMesh& tmesh,
+                                    const double cap_threshold,
+                                    const double needle_threshold,
+                                    const double collapse_length_threshold)
+{
+  return remove_almost_degenerate_faces(face_range, tmesh,
+                                        cap_threshold, needle_threshold, collapse_length_threshold,
+                                        CGAL::parameters::all_default());
+}
+
+template <typename TriangleMesh, typename CGAL_PMP_NP_TEMPLATE_PARAMETERS>
+bool remove_almost_degenerate_faces(TriangleMesh& tmesh,
+                                    const double cap_threshold,
+                                    const double needle_threshold,
+                                    const double collapse_length_threshold,
+                                    const CGAL_PMP_NP_CLASS& np)
+{
+  return remove_almost_degenerate_faces(faces(tmesh), tmesh, cap_threshold, needle_threshold,
+                                        collapse_length_threshold, np);
+}
+
+template<class TriangleMesh>
+bool remove_almost_degenerate_faces(TriangleMesh& tmesh,
+                                    const double cap_threshold,
+                                    const double needle_threshold,
+                                    const double collapse_length_threshold)
+{
+  return remove_almost_degenerate_faces(faces(tmesh), tmesh,
+                                        cap_threshold, needle_threshold, collapse_length_threshold,
+                                        CGAL::parameters::all_default());
+}
+
+} // namespace experimental
+
+////////////////////////////////////////////////////////////////////////////////////////////////////
+////////////////////////////////////////////////////////////////////////////////////////////////////
+////////////////////////////////////////////////////////////////////////////////////////////////////
+////////////////////////////////////////////////////////////////////////////////////////////////////
+/// Remove degenerate_edges/faces
+
+namespace internal {
+
+template <typename HalfedgeGraph, typename VertexPointMap, typename Traits>
+struct Less_vertex_point
+{
+  typedef typename boost::graph_traits<HalfedgeGraph>::vertex_descriptor    vertex_descriptor;
+
+  Less_vertex_point(const Traits& traits, const VertexPointMap& vpmap)
+    : m_traits(traits), m_vpmap(vpmap)
+  {}
+
+  bool operator()(vertex_descriptor v1, vertex_descriptor v2) const {
+    return m_traits.less_xyz_3_object()(get(m_vpmap, v1), get(m_vpmap, v2));
+  }
+
+  const Traits& m_traits;
+  const VertexPointMap& m_vpmap;
+};
+
+} // end namespace internal
+
+// this function removes a border edge even if it does not satisfy the link condition.
+// null_vertex() is returned if the removal changes the topology of the input
+template <typename TriangleMesh, typename EdgeSet, typename FaceSet>
+typename boost::graph_traits<TriangleMesh>::vertex_descriptor
+remove_a_border_edge(typename boost::graph_traits<TriangleMesh>::edge_descriptor ed,
+                     TriangleMesh& tm,
+                     EdgeSet& edge_set,
+                     FaceSet& face_set)
+{
+  typedef boost::graph_traits<TriangleMesh>                             GT;
+  typedef typename GT::vertex_descriptor                                vertex_descriptor;
+  typedef typename GT::halfedge_descriptor                              halfedge_descriptor;
+  typedef typename GT::edge_descriptor                                  edge_descriptor;
+  typedef typename GT::face_descriptor                                  face_descriptor;
+
+  halfedge_descriptor h = halfedge(ed, tm);
+
+  if(is_border(h, tm))
+    h = opposite(h, tm);
+
+  halfedge_descriptor opp_h = opposite(h, tm);
+  CGAL_assertion(is_border(opp_h, tm));
+  CGAL_assertion(!is_border(h, tm));
+
+  CGAL_assertion(next(next(opp_h, tm), tm) != opp_h); // not working for a hole made of 2 edges
+  CGAL_assertion(next(next(next(opp_h, tm), tm), tm) != opp_h); // not working for a hole make of 3 edges
+
+  if(CGAL::Euler::does_satisfy_link_condition(edge(h, tm), tm))
+  {
+    edge_set.erase(ed);
+    halfedge_descriptor h = halfedge(ed, tm);
+    if(is_border(h, tm))
+      h = opposite(h, tm);
+
+    edge_set.erase(edge(prev(h, tm), tm));
+    face_set.erase(face(h, tm));
+
+    return CGAL::Euler::collapse_edge(ed, tm);
+  }
+
+  // collect edges that have one vertex in the link of
+  // the vertices of h and one of the vertex of h as other vertex
+  std::set<edge_descriptor> common_incident_edges;
+  for(halfedge_descriptor hos : halfedges_around_source(h, tm))
+  {
+    for(halfedge_descriptor hot : halfedges_around_target(h, tm))
+    {
+      if(target(hos, tm) == source(hot, tm))
+      {
+        common_incident_edges.insert(edge(hot, tm));
+        common_incident_edges.insert(edge(hos, tm));
+      }
+    }
+  }
+
+  CGAL_assertion(common_incident_edges.size() >= 2);
+
+  // in the following loop, we visit define a connected component of
+  // faces bounded by edges in common_incident_edges and h. We look
+  // for the maximal one. This set of faces is the one that will
+  // disappear while collapsing ed
+  std::set<face_descriptor> marked_faces;
+
+  std::vector<halfedge_descriptor> queue;
+  queue.push_back(opposite(next(h, tm), tm));
+  queue.push_back(opposite(prev(h, tm), tm));
+  marked_faces.insert(face(h, tm));
+
+  do
+  {
+    std::vector<halfedge_descriptor> boundary;
+    while(!queue.empty())
+    {
+      halfedge_descriptor back=queue.back();
+      queue.pop_back();
+      face_descriptor fback=face(back, tm);
+      if(common_incident_edges.count(edge(back, tm)))
+      {
+        boundary.push_back(back);
+        continue;
+      }
+
+      if(fback==GT::null_face() || !marked_faces.insert(fback).second)
+        continue;
+
+      queue.push_back(opposite(next(back, tm), tm));
+      if(is_border(queue.back(), tm))
+      {
+#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
+        std::cout << "Boundary reached during exploration, the region to be removed is not a topological disk, not handled for now.\n";
+#endif
+        return GT::null_vertex();
+      }
+
+      queue.push_back(opposite(prev(back, tm), tm));
+      if(is_border(queue.back(), tm))
+      {
+#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
+        std::cout << "Boundary reached during exploration, the region to be removed is not a topological disk, not handled for now.\n";
+#endif
+        return GT::null_vertex();
+      }
+    }
+
+    CGAL_assertion(boundary.size() == 2);
+    common_incident_edges.erase(edge(boundary[0], tm));
+    common_incident_edges.erase(edge(boundary[1], tm));
+    if(!is_border(boundary[0], tm) || common_incident_edges.empty())
+      queue.push_back(boundary[0]);
+    if(!is_border(boundary[1], tm) || common_incident_edges.empty())
+      queue.push_back(boundary[1]);
+  }
+  while(!common_incident_edges.empty());
+
+  // hk1 and hk2 are bounding the region that will be removed.
+  // The edge of hk2 will be removed and hk2 will be replaced
+  // by the opposite edge of hk1
+  halfedge_descriptor hk1 = queue.front();
+  halfedge_descriptor hk2 = queue.back();
+  if(target(hk1, tm)!=source(hk2, tm))
+    std::swap(hk1, hk2);
+
+  CGAL_assertion(target(hk1, tm) == source(hk2, tm));
+  CGAL_assertion(source(hk1, tm) == source(h, tm));
+  CGAL_assertion(target(hk2, tm) == target(h, tm));
+
+  CGAL_assertion(is_valid_polygon_mesh(tm));
+  if(!is_selection_a_topological_disk(marked_faces, tm))
+  {
+#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
+    std::cout << "The region to be removed is not a topological disk, not handled for now.\n";
+#endif
+    return GT::null_vertex();
+  }
+
+  if(is_border(hk1, tm) && is_border(hk2, tm))
+  {
+#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
+    std::cout << "The region to be removed is an isolated region, not handled for now.\n";
+#endif
+    return GT::null_vertex();
+  }
+
+  // collect vertices and edges to remove and do remove faces
+  std::set<edge_descriptor> edges_to_remove;
+  std::set<vertex_descriptor> vertices_to_remove;
+  for(face_descriptor fd : marked_faces)
+  {
+    halfedge_descriptor hd = halfedge(fd, tm);
+    for(int i=0; i<3; ++i)
+    {
+      edges_to_remove.insert(edge(hd, tm));
+      vertices_to_remove.insert(target(hd, tm));
+      hd = next(hd, tm);
+    }
+  }
+
+  vertex_descriptor vkept = source(hk1, tm);
+
+  //back-up next, prev halfedge pointers to be restored after removal
+  halfedge_descriptor hp = prev(opp_h, tm);
+  halfedge_descriptor hn = next(opp_h, tm);
+  halfedge_descriptor hk1_opp_next = next(hk2, tm);
+  halfedge_descriptor hk1_opp_prev = prev(hk2, tm);
+  face_descriptor hk1_opp_face = face(hk2, tm);
+
+  // we will remove the target of hk2, update vertex pointers
+  for(halfedge_descriptor hot : halfedges_around_target(hk2, tm))
+    set_target(hot, vkept, tm);
+
+  // update halfedge pointers since hk2 will be removed
+  set_halfedge(vkept, opposite(hk1, tm), tm);
+  set_halfedge(target(hk1, tm), hk1, tm);
+
+  // do not remove hk1 and its vertices
+  vertices_to_remove.erase(vkept);
+  vertices_to_remove.erase(target(hk1, tm));
+  edges_to_remove.erase(edge(hk1, tm));
+
+  bool hk2_equals_hp = (hk2 == hp);
+  CGAL_assertion(is_border(hk2, tm) == hk2_equals_hp);
+
+  /*
+  - case hk2!=hp
+
+         /\      /
+     hk1/  \hk2 /
+       /    \  /
+  ____/______\/____
+  hn   h_opp   hp
+
+  - case hk2==hp
+
+         /\
+     hk1/  \hk2 == hp
+       /    \
+  ____/______\
+  hn   h_opp
+  */
+
+  // remove vertices
+  for(vertex_descriptor vd : vertices_to_remove)
+    remove_vertex(vd, tm);
+
+  // remove edges
+  for(edge_descriptor ed : edges_to_remove)
+  {
+    edge_set.erase(ed);
+    remove_edge(ed, tm);
+  }
+
+  // remove faces
+  for(face_descriptor fd : marked_faces)
+  {
+    face_set.erase(fd);
+    remove_face(fd, tm);
+  }
+
+  // now update pointers
+  set_face(opposite(hk1, tm), hk1_opp_face, tm);
+  if(!hk2_equals_hp)
+  {
+    set_next(hp, hn, tm);
+    set_next(opposite(hk1, tm), hk1_opp_next, tm);
+    set_next(hk1_opp_prev, opposite(hk1, tm), tm);
+    set_halfedge(hk1_opp_face, opposite(hk1, tm), tm);
+  }
+  else
+  {
+    set_next(hk1_opp_prev, opposite(hk1, tm), tm);
+    set_next(opposite(hk1, tm), hn, tm);
+  }
+
+  CGAL_assertion(is_valid_polygon_mesh(tm));
+  return vkept;
+}
+
+template <typename TriangleMesh>
+typename boost::graph_traits<TriangleMesh>::vertex_descriptor
+remove_a_border_edge(typename boost::graph_traits<TriangleMesh>::edge_descriptor ed,
+                     TriangleMesh& tm)
+{
+  std::set<typename boost::graph_traits<TriangleMesh>::edge_descriptor> edge_set;
+  std::set<typename boost::graph_traits<TriangleMesh>::face_descriptor> face_set;
+
+  return remove_a_border_edge(ed, tm, edge_set, face_set);
+}
+
+template <typename EdgeRange, typename TriangleMesh, typename NamedParameters, typename FaceSet>
+bool remove_degenerate_edges(const EdgeRange& edge_range,
+                             TriangleMesh& tmesh,
+                             FaceSet& face_set,
+                             const NamedParameters& np)
+{
+  CGAL_assertion(CGAL::is_triangle_mesh(tmesh));
+  CGAL_assertion(CGAL::is_valid_polygon_mesh(tmesh));
+
+  using parameters::get_parameter;
+  using parameters::choose_parameter;
+
+  typedef TriangleMesh                                                            TM;
+  typedef typename boost::graph_traits<TriangleMesh>                              GT;
+  typedef typename GT::vertex_descriptor                                          vertex_descriptor;
+  typedef typename GT::halfedge_descriptor                                        halfedge_descriptor;
+  typedef typename GT::edge_descriptor                                            edge_descriptor;
+  typedef typename GT::face_descriptor                                            face_descriptor;
+
+  typedef typename GetVertexPointMap<TM, NamedParameters>::type                   VertexPointMap;
+  VertexPointMap vpmap = choose_parameter(get_parameter(np, internal_np::vertex_point),
+                                          get_property_map(vertex_point, tmesh));
+
+  typedef typename GetGeomTraits<TM, NamedParameters>::type                       Traits;
+
+  std::size_t nb_deg_faces = 0;
+  bool all_removed = false;
+  bool some_removed = true;
+  bool preserve_genus = choose_parameter(get_parameter(np, internal_np::preserve_genus), true);
+
+  // collect edges of length 0
+  while(some_removed && !all_removed)
+  {
+    some_removed = false;
+    all_removed = true;
+    std::set<edge_descriptor> degenerate_edges_to_remove;
+    degenerate_edges(edge_range, tmesh, std::inserter(degenerate_edges_to_remove,
+                                                      degenerate_edges_to_remove.end()));
+
+#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
+    std::cout << "Found " << degenerate_edges_to_remove.size() << " null edges.\n";
+#endif
+
+    // first try to remove all collapsable edges
+    typename std::set<edge_descriptor>::iterator it = degenerate_edges_to_remove.begin();
+    while(it != degenerate_edges_to_remove.end())
+    {
+      edge_descriptor e = *it;
+      if(CGAL::Euler::does_satisfy_link_condition(e, tmesh))
+      {
+        halfedge_descriptor h = halfedge(e, tmesh);
+        degenerate_edges_to_remove.erase(it);
+
+        // remove edges that could also be set for removal
+        if(face(h, tmesh) != GT::null_face())
+        {
+          ++nb_deg_faces;
+          degenerate_edges_to_remove.erase(edge(prev(h, tmesh), tmesh));
+          face_set.erase(face(h, tmesh));
+        }
+
+        if(face(opposite(h, tmesh), tmesh) != GT::null_face())
+        {
+          ++nb_deg_faces;
+          degenerate_edges_to_remove.erase(edge(prev(opposite(h, tmesh), tmesh), tmesh));
+          face_set.erase(face(opposite(h, tmesh), tmesh));
+        }
+
+        //now remove the edge
+        CGAL::Euler::collapse_edge(e, tmesh);
+
+        // some_removed is not updated on purpose because if nothing
+        //  happens below then nothing can be done
+        it = degenerate_edges_to_remove.begin();
+      }
+      else // link condition not satisfied
+      {
+        ++it;
+      }
+    }
+
+    CGAL_assertion(is_valid_polygon_mesh(tmesh));
+#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
+    std::cout << "Remaining " << degenerate_edges_to_remove.size() << " null edges to be handled.\n";
+#endif
+
+    while(!degenerate_edges_to_remove.empty())
+    {
+      edge_descriptor ed = *degenerate_edges_to_remove.begin();
+      degenerate_edges_to_remove.erase(degenerate_edges_to_remove.begin());
+      halfedge_descriptor h = halfedge(ed, tmesh);
+
+      if(CGAL::Euler::does_satisfy_link_condition(ed, tmesh))
+      {
+        // remove edges that could also be set for removal
+        if(face(h, tmesh) != GT::null_face())
+        {
+          ++nb_deg_faces;
+          degenerate_edges_to_remove.erase(edge(prev(h, tmesh), tmesh));
+          face_set.erase(face(h, tmesh));
+        }
+
+        if(face(opposite(h, tmesh), tmesh)!=GT::null_face())
+        {
+          ++nb_deg_faces;
+          degenerate_edges_to_remove.erase(edge(prev(opposite(h, tmesh), tmesh), tmesh));
+          face_set.erase(face(opposite(h, tmesh), tmesh));
+        }
+
+        //now remove the edge
+        CGAL::Euler::collapse_edge(ed, tmesh);
+        some_removed = true;
+      }
+      else // link condition not satisfied
+      {
+        // handle the case when the edge is incident to a triangle hole
+        // we first fill the hole and try again
+        if(is_border(ed, tmesh))
+        {
+          halfedge_descriptor hd = halfedge(ed, tmesh);
+          if(!is_border(hd, tmesh))
+            hd = opposite(hd, tmesh);
+
+          if(is_triangle(hd, tmesh))
+          {
+            if(!preserve_genus)
+            {
+              Euler::fill_hole(hd, tmesh);
+              degenerate_edges_to_remove.insert(ed); // reinsert the edge for future processing
+            }
+            else
+            {
+              all_removed=false;
+            }
+
+            continue;
+          }
+
+#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
+          std::cout << "Calling remove_a_border_edge\n";
+#endif
+
+          vertex_descriptor vd = remove_a_border_edge(ed, tmesh, degenerate_edges_to_remove, face_set);
+          if(vd == GT::null_vertex())
+          {
+            // @todo: if some border edges are later removed, the edge might be processable later
+            // for example if it belongs to  boundary cycle of edges where the number of non-degenerate
+            // edges is 2. That's what happen with fused_vertices.off in the testsuite where the edges
+            // are not processed the same way with Polyhedron and Surface_mesh. In the case of Polyhedron
+            // more degenerate edges could be removed.
+            all_removed = false;
+          }
+          else
+            some_removed = true;
+
+          continue;
+        }
+        else
+        {
+          halfedge_descriptor hd = halfedge(ed, tmesh);
+          // if both vertices are boundary vertices we can't do anything
+          bool impossible = false;
+          for(halfedge_descriptor h : halfedges_around_target(hd, tmesh))
+          {
+            if(is_border(h, tmesh))
+            {
+              impossible = true;
+              break;
+            }
+          }
+
+          if(impossible)
+          {
+            impossible = false;
+            for(halfedge_descriptor h : halfedges_around_source(hd, tmesh))
+            {
+              if(is_border(h, tmesh))
+              {
+                impossible = true;
+                break;
+              }
+            }
+
+            if(impossible)
+            {
+              all_removed = false;
+              continue;
+            }
+          }
+        }
+
+        // When the edge does not satisfy the link condition, it means that it cannot be
+        // collapsed as is. In the following if there is a topological issue
+        // with contracting the edge (component or geometric feature that disappears),
+        // nothing is done.
+        // We start by marking the faces that are incident to an edge endpoint.
+        // If the set of marked faces is a topologically disk, then we simply remove all the simplicies
+        // inside the disk and star the hole with the edge vertex kept.
+        // If the set of marked faces is not a topological disk, it has some non-manifold vertices
+        // on its boundary. We need to mark additional faces to make it a topological disk.
+        // We can then apply the star hole procedure.
+        // Right now we additionally mark the smallest connected components of non-marked faces
+        // (using the number of faces)
+
+        //backup central point
+        typename Traits::Point_3 pt = get(vpmap, source(ed, tmesh));
+
+        // mark faces of the link of each endpoints of the edge which collapse is not topologically valid
+        std::set<face_descriptor> marked_faces;
+
+        //   first endpoint
+        for(halfedge_descriptor hd : CGAL::halfedges_around_target(halfedge(ed, tmesh), tmesh))
+          if(!is_border(hd, tmesh))
+            marked_faces.insert(face(hd, tmesh));
+
+        //   second endpoint
+        for(halfedge_descriptor hd : CGAL::halfedges_around_target(opposite(halfedge(ed, tmesh), tmesh), tmesh))
+          if(!is_border(hd, tmesh))
+            marked_faces.insert(face(hd, tmesh));
+
+        // extract the halfedges on the boundary of the marked region
+        std::vector<halfedge_descriptor> border;
+        for(face_descriptor fd : marked_faces)
+        {
+          for(halfedge_descriptor hd : CGAL::halfedges_around_face(halfedge(fd, tmesh), tmesh))
+          {
+            halfedge_descriptor hd_opp = opposite(hd, tmesh);
+            if(is_border(hd_opp, tmesh) || marked_faces.count(face(hd_opp, tmesh)) == 0)
+            {
+              border.push_back(hd);
+            }
+          }
+        }
+
+        if(border.empty())
+        {
+          // a whole connected component (without boundary) got selected and will disappear (not handled for now)
+#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
+          std::cout << "Trying to remove a whole connected component, not handled yet\n";
+#endif
+          all_removed = false;
+          continue;
+        }
+
+        // define cc of border halfedges: two halfedges are in the same cc
+        // if they are on the border of the cc of non-marked faces.
+        typedef CGAL::Union_find<halfedge_descriptor>                     UF_ds;
+        UF_ds uf;
+        std::map<halfedge_descriptor, typename UF_ds::handle> handles;
+
+        // one cc per border halfedge
+        for(halfedge_descriptor hd : border)
+          handles.insert(std::make_pair(hd, uf.make_set(hd)));
+
+        // join cc's
+        for(halfedge_descriptor hd : border)
+        {
+          CGAL_assertion(marked_faces.count(face(hd, tmesh)) > 0);
+          CGAL_assertion(marked_faces.count(face(opposite(hd, tmesh), tmesh)) == 0);
+          halfedge_descriptor candidate = hd;
+
+          do
+          {
+            candidate = prev(opposite(candidate, tmesh), tmesh);
+          }
+          while(!marked_faces.count(face(opposite(candidate, tmesh), tmesh)));
+
+          uf.unify_sets(handles[hd], handles[opposite(candidate, tmesh)]);
+        }
+
+        std::size_t nb_cc = uf.number_of_sets();
+        if(nb_cc != 1)
+        {
+          // if more than one connected component is found then the patch
+          // made of marked faces contains "non-manifold" vertices.
+          // The smallest components need to be marked so that the patch
+          // made of marked faces is a topological disk
+
+          // we will explore in parallel the connected components and will stop
+          // when all but one connected component have been entirely explored.
+          // We add one face at a time for each cc in order to not explore a
+          // potentially very large cc.
+          std::vector<std::vector<halfedge_descriptor> > stacks_per_cc(nb_cc);
+          std::vector<std::set<face_descriptor> > faces_per_cc(nb_cc);
+          std::vector<bool> exploration_finished(nb_cc, false);
+
+          // init the stacks of halfedges using the cc of the boundary
+          std::size_t index = 0;
+          std::map<halfedge_descriptor, std::size_t > ccs;
+
+          typedef std::pair<const halfedge_descriptor, typename UF_ds::handle> Pair_type;
+          for(const Pair_type& p : handles)
+          {
+            halfedge_descriptor opp_hedge = opposite(p.first, tmesh);
+            if(is_border(opp_hedge, tmesh))
+              continue; // nothing to do on the boundary
+
+            typedef typename std::map<halfedge_descriptor, std::size_t >::iterator Map_it;
+            std::pair<Map_it, bool> insert_res = ccs.insert(std::make_pair(*uf.find(p.second), index));
+
+            if(insert_res.second)
+              ++index;
+
+            stacks_per_cc[insert_res.first->second].push_back(prev(opp_hedge, tmesh));
+            stacks_per_cc[insert_res.first->second].push_back(next(opp_hedge, tmesh));
+            faces_per_cc[insert_res.first->second].insert(face(opp_hedge, tmesh));
+          }
+
+          if(index != nb_cc)
+          {
+            // most probably, one cc is a cycle of border edges
+#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
+            std::cout << "Trying to remove a component with a cycle of halfedges (nested hole or whole component), not handled yet.\n";
+#endif
+            all_removed = false;
+            continue;
+          }
+
+          std::size_t nb_ccs_to_be_explored = nb_cc;
+          index = 0;
+          //explore the cc's
+          do
+          {
+            // try to extract one more face for a given cc
+            do
+            {
+              CGAL_assertion(!exploration_finished[index]);
+              CGAL_assertion(!stacks_per_cc.empty());
+              CGAL_assertion(!stacks_per_cc[index].empty());
+
+              halfedge_descriptor hd = stacks_per_cc[index].back();
+              stacks_per_cc[index].pop_back();
+              hd = opposite(hd, tmesh);
+
+              if(!is_border(hd, tmesh) && !marked_faces.count(face(hd, tmesh)))
+              {
+                if(faces_per_cc[index].insert(face(hd, tmesh)).second)
+                {
+                  stacks_per_cc[index].push_back(next(hd, tmesh));
+                  stacks_per_cc[index].push_back(prev(hd, tmesh));
+                  break;
+                }
+              }
+
+              if(stacks_per_cc[index].empty())
+                break;
+            }
+            while(true);
+
+            // the exploration of a cc is finished when its stack is empty
+            exploration_finished[index]=stacks_per_cc[index].empty();
+            if(exploration_finished[index])
+              --nb_ccs_to_be_explored;
+
+            if(nb_ccs_to_be_explored==1)
+              break;
+
+            while(exploration_finished[(++index)%nb_cc]);
+
+            index = index%nb_cc;
+          }
+          while(true);
+
+          // @todo use the area criteria? this means maybe continue exploration of larger cc
+          // mark faces of completetly explored cc
+          for(index=0; index<nb_cc; ++index)
+          {
+            if(exploration_finished[index])
+            {
+              for(face_descriptor fd : faces_per_cc[index])
+                marked_faces.insert(fd);
+            }
+          }
+        }
+
+        // make sure the selection is a topological disk (otherwise we need another treatment)
+        if(!is_selection_a_topological_disk(marked_faces, tmesh))
+        {
+#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
+          std::cout << "Trying to handle a non-topological disk, do nothing\n";
+#endif
+          all_removed = false;
+          continue;
+        }
+
+        some_removed = true;
+
+        // collect simplices to be removed
+        std::set<vertex_descriptor> vertices_to_keep;
+        std::set<halfedge_descriptor> halfedges_to_keep;
+        for(halfedge_descriptor hd : border)
+        {
+          if(!marked_faces.count(face(opposite(hd, tmesh), tmesh)))
+          {
+            halfedges_to_keep.insert(hd);
+            vertices_to_keep.insert(target(hd, tmesh));
+          }
+        }
+
+        // backup next,prev relationships to set after patch removal
+        std::vector<std::pair<halfedge_descriptor, halfedge_descriptor> > next_prev_halfedge_pairs;
+        halfedge_descriptor first_border_hd = *(halfedges_to_keep.begin());
+        halfedge_descriptor current_border_hd = first_border_hd;
+        do
+        {
+          halfedge_descriptor prev_border_hd = current_border_hd;
+          current_border_hd = next(current_border_hd, tmesh);
+
+          while(marked_faces.count(face(opposite(current_border_hd, tmesh), tmesh)))
+            current_border_hd=next(opposite(current_border_hd, tmesh), tmesh);
+
+          next_prev_halfedge_pairs.push_back(std::make_pair(prev_border_hd, current_border_hd));
+        }
+        while(current_border_hd!=first_border_hd);
+
+        // collect vertices and edges to remove and do remove faces
+        std::set<edge_descriptor> edges_to_remove;
+        std::set<vertex_descriptor> vertices_to_remove;
+        for(face_descriptor fd : marked_faces)
+        {
+          halfedge_descriptor hd=halfedge(fd, tmesh);
+          for(int i=0; i<3; ++i)
+          {
+            if(!halfedges_to_keep.count(hd))
+              edges_to_remove.insert(edge(hd, tmesh));
+
+            if(!vertices_to_keep.count(target(hd, tmesh)))
+              vertices_to_remove.insert(target(hd, tmesh));
+
+            hd = next(hd, tmesh);
+          }
+
+          remove_face(fd, tmesh);
+          face_set.erase(fd);
+        }
+
+        // remove vertices
+        for(vertex_descriptor vd : vertices_to_remove)
+          remove_vertex(vd, tmesh);
+
+        // remove edges
+        for(edge_descriptor ed : edges_to_remove)
+        {
+          degenerate_edges_to_remove.erase(ed);
+          remove_edge(ed, tmesh);
+        }
+
+        // add a new face, set all border edges pointing to it
+        // and update halfedge vertex of patch boundary vertices
+        face_descriptor new_face = add_face(tmesh);
+        typedef std::pair<halfedge_descriptor, halfedge_descriptor> Pair_type;
+        for(const Pair_type& p : next_prev_halfedge_pairs)
+        {
+          set_face(p.first, new_face, tmesh);
+          set_next(p.first, p.second, tmesh);
+          set_halfedge(target(p.first, tmesh), p.first, tmesh);
+        }
+
+        set_halfedge(new_face, first_border_hd, tmesh);
+
+        // triangulate the new face and update the coordinate of the central vertex
+        halfedge_descriptor new_hd=Euler::add_center_vertex(first_border_hd, tmesh);
+        put(vpmap, target(new_hd, tmesh), pt);
+
+        for(halfedge_descriptor hd : halfedges_around_target(new_hd, tmesh))
+        {
+          if(is_degenerate_edge(edge(hd, tmesh), tmesh, np))
+            degenerate_edges_to_remove.insert(edge(hd, tmesh));
+
+          if(face(hd, tmesh) != GT::null_face() && is_degenerate_triangle_face(face(hd, tmesh), tmesh))
+            face_set.insert(face(hd, tmesh));
+        }
+
+        CGAL_assertion(is_valid_polygon_mesh(tmesh));
+      }
+    }
+  }
+
+  return all_removed;
+}
+
+template <typename EdgeRange, typename TriangleMesh, typename CGAL_PMP_NP_TEMPLATE_PARAMETERS>
+bool remove_degenerate_edges(const EdgeRange& edge_range,
+                             TriangleMesh& tmesh,
+                             const CGAL_PMP_NP_CLASS& np)
+{
+  std::set<typename boost::graph_traits<TriangleMesh>::face_descriptor> face_set;
+  return remove_degenerate_edges(edge_range, tmesh, face_set, np);
+}
+
+template <typename TriangleMesh, typename CGAL_PMP_NP_TEMPLATE_PARAMETERS>
+bool remove_degenerate_edges(TriangleMesh& tmesh,
+                             const CGAL_PMP_NP_CLASS& np)
+{
+  std::set<typename boost::graph_traits<TriangleMesh>::face_descriptor> face_set;
+  return remove_degenerate_edges(edges(tmesh), tmesh, face_set, np);
+}
+
+template <typename EdgeRange, typename TriangleMesh>
+bool remove_degenerate_edges(const EdgeRange& edge_range,
+                             TriangleMesh& tmesh)
+{
+  std::set<typename boost::graph_traits<TriangleMesh>::face_descriptor> face_set;
+  return remove_degenerate_edges(edge_range, tmesh, face_set, parameters::all_default());
+}
+
+template <typename TriangleMesh>
+bool remove_degenerate_edges(TriangleMesh& tmesh)
+{
+  std::set<typename boost::graph_traits<TriangleMesh>::face_descriptor> face_set;
+  return remove_degenerate_edges(edges(tmesh), tmesh, face_set, parameters::all_default());
+}
+
+// \ingroup PMP_repairing_grp
+// removes the degenerate faces from a triangulated surface mesh.
+// A face is considered degenerate if two of its vertices share the same location,
+// or more generally if all its vertices are collinear.
+//
+// @pre `CGAL::is_triangle_mesh(tmesh)`
+//
+// @tparam TriangleMesh a model of `FaceListGraph` and `MutableFaceGraph`
+// @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
+//
+// @param tmesh the  triangulated surface mesh to be repaired
+// @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+//
+// \cgalNamedParamsBegin
+//   \cgalParamNBegin{vertex_point_map}
+//     \cgalParamDescription{a property map associating points to the vertices of `tmesh`}
+//     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+//                    as key type and `%Point_3` as value type}
+//     \cgalParamDefault{`boost::get(CGAL::vertex_point, tmesh)`}
+//     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+//                     must be available in `TriangleMesh`.}
+//   \cgalParamNEnd
+//
+//   \cgalParamNBegin{geom_traits}
+//     \cgalParamDescription{an instance of a geometric traits class}
+//     \cgalParamType{The traits class must provide the nested type `Point_3`,
+//                    and the nested functors:
+//                    - `Compare_distance_3` to compute the distance between 2 points
+//                    - `Collinear_3` to check whether 3 points are collinear
+//                    - `Less_xyz_3` to compare lexicographically two points
+//                    - `Equal_3` to check whether 2 points are identical.
+//                    For each functor Foo, a function `Foo foo_object()` must be provided.}
+//     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+//     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+//   \cgalParamNEnd
+// \cgalNamedParamsEnd
+//
+// \todo the function might not be able to remove all degenerate faces.
+//       We should probably do something with the return type.
+//
+// \return `true` if all degenerate faces were successfully removed, and `false` otherwise.
+template <typename FaceRange, typename TriangleMesh, typename NamedParameters>
+bool remove_degenerate_faces(const FaceRange& face_range,
+                             TriangleMesh& tmesh,
+                             const NamedParameters& np)
+{
+  CGAL_assertion(CGAL::is_triangle_mesh(tmesh));
+
+  using parameters::get_parameter;
+  using parameters::choose_parameter;
+
+  typedef TriangleMesh                                                              TM;
+  typedef typename boost::graph_traits<TriangleMesh>                                GT;
+  typedef typename GT::vertex_descriptor                                            vertex_descriptor;
+  typedef typename GT::halfedge_descriptor                                          halfedge_descriptor;
+  typedef typename GT::edge_descriptor                                              edge_descriptor;
+  typedef typename GT::face_descriptor                                              face_descriptor;
+
+  typedef typename GetVertexPointMap<TM, NamedParameters>::type                     VertexPointMap;
+  VertexPointMap vpmap = choose_parameter(get_parameter(np, internal_np::vertex_point),
+                                          get_property_map(vertex_point, tmesh));
+  typedef typename GetGeomTraits<TM, NamedParameters>::type                         Traits;
+  Traits traits = choose_parameter(get_parameter(np, internal_np::geom_traits), Traits());
+
+  typedef typename boost::property_traits<VertexPointMap>::value_type               Point_3;
+  typedef typename boost::property_traits<VertexPointMap>::reference                Point_ref;
+
+  std::set<face_descriptor> degenerate_face_set;
+  degenerate_faces(face_range, tmesh, std::inserter(degenerate_face_set, degenerate_face_set.begin()), np);
+
+  const std::size_t faces_size = faces(tmesh).size();
+
+  if(degenerate_face_set.empty())
+    return true;
+
+  if(degenerate_face_set.size() == faces_size)
+  {
+    clear(tmesh);
+    return true;
+  }
+
+  // Sanitize the face range by adding adjacent degenerate faces
+  const std::size_t range_size = face_range.size();
+  bool is_range_full_mesh = (range_size == faces_size);
+  if(!is_range_full_mesh)
+  {
+    std::list<face_descriptor> faces_to_visit(degenerate_face_set.begin(), degenerate_face_set.end());
+
+    while(!faces_to_visit.empty())
+    {
+      face_descriptor fd = faces_to_visit.front();
+      faces_to_visit.pop_front();
+
+      for(halfedge_descriptor hd : halfedges_around_face(halfedge(fd, tmesh), tmesh))
+      {
+        for(halfedge_descriptor inc_hd : halfedges_around_target(hd, tmesh))
+        {
+          face_descriptor adj_fd = face(inc_hd, tmesh);
+          if(adj_fd == GT::null_face() || adj_fd == fd)
+            continue;
+
+          if(is_degenerate_triangle_face(adj_fd, tmesh))
+          {
+            if(degenerate_face_set.insert(adj_fd).second)
+            {
+              // successful insertion means we did not know about this face before
+              faces_to_visit.push_back(adj_fd);
+            }
+          }
+        }
+      }
+    }
+  }
+
+  // Note that there can't be any null edge incident to the degenerate faces range,
+  // otherwise we would have a null face incident to the face range, and that is not possible
+  // after the sanitization above
+  std::set<edge_descriptor> edge_range;
+  for(face_descriptor fd : degenerate_face_set)
+    for(halfedge_descriptor hd : halfedges_around_face(halfedge(fd, tmesh), tmesh))
+      edge_range.insert(edge(hd, tmesh));
+
+  // First remove edges of length 0
+  bool all_removed = remove_degenerate_edges(edge_range, tmesh, degenerate_face_set, np);
+
+  CGAL_assertion_code(for(face_descriptor fd : degenerate_face_set) {)
+  CGAL_assertion(is_degenerate_triangle_face(fd, tmesh));
+  CGAL_assertion_code(})
+
+#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
+  {
+    std::cout <<"Done with null edges.\n";
+    std::ofstream output("/tmp/no_null_edges.off");
+    output << std::setprecision(17) << tmesh << "\n";
+    output.close();
+  }
+#endif
+
+  // Then, remove triangles made of 3 collinear points
+
+  // start by filtering out border faces
+  // @todo shall we avoid doing that in case a non-manifold vertex on the boundary or if a whole component disappear?
+  std::set<face_descriptor> border_deg_faces;
+  for(face_descriptor f : degenerate_face_set)
+  {
+    halfedge_descriptor h = halfedge(f, tmesh);
+    for(int i=0; i<3; ++i)
+    {
+      if(is_border(opposite(h, tmesh), tmesh))
+      {
+        border_deg_faces.insert(f);
+        break;
+      }
+
+      h = next(h, tmesh);
+    }
+  }
+
+  while(!border_deg_faces.empty())
+  {
+    face_descriptor f_to_rm = *border_deg_faces.begin();
+    border_deg_faces.erase(border_deg_faces.begin());
+
+    halfedge_descriptor h = halfedge(f_to_rm, tmesh);
+    for(int i=0; i<3; ++i)
+    {
+      face_descriptor f = face(opposite(h, tmesh), tmesh);
+      if(f!=GT::null_face())
+      {
+        if(is_degenerate_triangle_face(f, tmesh, np))
+          border_deg_faces.insert(f);
+      }
+
+      h = next(h, tmesh);
+    }
+
+    while(!is_border(opposite(h, tmesh), tmesh))
+    {
+      h = next(h, tmesh);
+    }
+
+    degenerate_face_set.erase(f_to_rm);
+    Euler::remove_face(h, tmesh);
+  }
+
+  // Ignore faces with null edges
+  if(!all_removed)
+  {
+    std::map<edge_descriptor, bool> are_degenerate_edges;
+
+    for(face_descriptor fd : degenerate_face_set)
+    {
+      for(halfedge_descriptor hd : halfedges_around_face(halfedge(fd, tmesh), tmesh))
+      {
+        edge_descriptor ed = edge(hd, tmesh);
+        std::pair<typename std::map<edge_descriptor, bool>::iterator, bool> is_insert_successful =
+            are_degenerate_edges.insert(std::make_pair(ed, false));
+
+        bool is_degenerate = false;
+        if(is_insert_successful.second)
+        {
+          // did not previously exist in the map, so actually have to check if it is degenerate
+          if(traits.equal_3_object()(get(vpmap, target(ed, tmesh)), get(vpmap, source(ed, tmesh))))
+            is_degenerate = true;
+        }
+
+        is_insert_successful.first->second = is_degenerate;
+
+        if(is_degenerate)
+        {
+          halfedge_descriptor h = halfedge(ed, tmesh);
+          if(!is_border(h, tmesh))
+            degenerate_face_set.erase(face(h, tmesh));
+
+          h = opposite(h, tmesh);
+          if(!is_border(h, tmesh))
+            degenerate_face_set.erase(face(h, tmesh));
+        }
+      }
+    }
+  }
+
+  // first remove degree 3 vertices that are part of a cap
+  // (only the vertex in the middle of the opposite edge)
+  // This removal does not change the shape of the mesh.
+  while(!degenerate_face_set.empty())
+  {
+    std::set<vertex_descriptor> vertices_to_remove;
+    for(face_descriptor fd : degenerate_face_set)
+    {
+      for(halfedge_descriptor hd : halfedges_around_face(halfedge(fd, tmesh), tmesh))
+      {
+        vertex_descriptor vd = target(hd, tmesh);
+        if(degree(vd, tmesh) == 3 && is_border(vd, tmesh)==GT::null_halfedge())
+        {
+          vertices_to_remove.insert(vd);
+          break;
+        }
+      }
+    }
+
+    for(vertex_descriptor vd : vertices_to_remove)
+    {
+      for(halfedge_descriptor hd2 : halfedges_around_target(vd, tmesh))
+        degenerate_face_set.erase(face(hd2, tmesh));
+
+      // remove the central vertex and check if the new face is degenerated
+      halfedge_descriptor hd = CGAL::Euler::remove_center_vertex(halfedge(vd, tmesh), tmesh);
+      if(is_degenerate_triangle_face(face(hd, tmesh), tmesh, np))
+      {
+        degenerate_face_set.insert(face(hd, tmesh));
+      }
+    }
+
+    if(vertices_to_remove.empty())
+      break;
+  }
+
+  while(!degenerate_face_set.empty())
+  {
+#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
+    std::cout << "Loop on removing deg faces\n";
+
+    // ensure the mesh is not broken
+    {
+      std::ofstream out("/tmp/out.off");
+      out << tmesh;
+      out.close();
+
+      std::vector<typename Traits::Point_3> points;
+      std::vector<std::vector<std::size_t> > triangles;
+      std::ifstream in("/tmp/out.off");
+      CGAL::read_OFF(in, points, triangles);
+      if(!CGAL::Polygon_mesh_processing::is_polygon_soup_a_polygon_mesh(triangles))
+      {
+        std::cerr << "Warning: got a polygon soup (may simply be a non-manifold vertex)!\n";
+      }
+    }
+#endif
+
+    face_descriptor fd = *degenerate_face_set.begin();
+
+    // look whether an incident triangle is also degenerate
+    bool detect_cc_of_degenerate_triangles = false;
+    for(halfedge_descriptor hd : halfedges_around_face(halfedge(fd, tmesh), tmesh))
+    {
+      face_descriptor adjacent_face = face(opposite(hd, tmesh), tmesh);
+      if(adjacent_face!=GT::null_face() && degenerate_face_set.count(adjacent_face))
+      {
+        detect_cc_of_degenerate_triangles = true;
+        break;
+      }
+    }
+
+    if(!detect_cc_of_degenerate_triangles)
+    {
+#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
+      std::cout << "  no degenerate neighbors, using a flip.\n";
+#endif
+      degenerate_face_set.erase(degenerate_face_set.begin());
+
+      // flip the longest edge of the triangle
+      Point_ref p1 = get(vpmap, target(halfedge(fd, tmesh), tmesh));
+      Point_ref p2 = get(vpmap, target(next(halfedge(fd, tmesh), tmesh), tmesh));
+      Point_ref p3 = get(vpmap, source(halfedge(fd, tmesh), tmesh));
+
+      CGAL_assertion(p1!=p2 && p1!=p3 && p2!=p3);
+
+      typename Traits::Compare_distance_3 compare_distance = traits.compare_distance_3_object();
+
+      halfedge_descriptor edge_to_flip;
+      if(compare_distance(p1,p2, p1,p3) != CGAL::SMALLER) // p1p2 > p1p3
+      {
+        if(compare_distance(p1,p2, p2,p3) != CGAL::SMALLER) // p1p2 > p2p3
+          // flip p1p2
+          edge_to_flip = next(halfedge(fd, tmesh), tmesh);
+        else
+          // flip p2p3
+          edge_to_flip = prev(halfedge(fd, tmesh), tmesh);
+      }
+      else
+      {
+        if(compare_distance(p1,p3, p2,p3) != CGAL::SMALLER) // p1p3>p2p3
+          //flip p3p1
+          edge_to_flip = halfedge(fd, tmesh);
+        else
+          //flip p2p3
+          edge_to_flip = prev(halfedge(fd, tmesh), tmesh);
+      }
+
+      face_descriptor opposite_face=face(opposite(edge_to_flip, tmesh), tmesh);
+      if(opposite_face == GT::null_face())
+      {
+        // simply remove the face
+        Euler::remove_face(edge_to_flip, tmesh);
+      }
+      else
+      {
+        // condition for the flip to be valid (the edge to be created do not already exists)
+        if(!halfedge(target(next(edge_to_flip, tmesh), tmesh),
+                       target(next(opposite(edge_to_flip, tmesh), tmesh), tmesh),
+                       tmesh).second)
+        {
+          Euler::flip_edge(edge_to_flip, tmesh);
+        }
+        else
+        {
+          all_removed = false;
+#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
+          std::cout << "  WARNING: flip is not possible\n";
+          // @todo Let p and q be the vertices opposite to `edge_to_flip`, and let
+          //       r be the vertex of `edge_to_flip` that is the furthest away from
+          //       the edge `pq`. In that case I think we should remove all the triangles
+          //       so that the triangle pqr is in the mesh.
+#endif
+        }
+      }
+    }
+    else
+    {
+      // Process a connected component of degenerate faces
+      // get all the faces from the connected component
+      // and the boundary edges
+      std::set<face_descriptor> cc_faces;
+      std::vector<face_descriptor> queue;
+      std::vector<halfedge_descriptor> boundary_hedges;
+      std::vector<halfedge_descriptor> inside_hedges;
+      queue.push_back(fd);
+      cc_faces.insert(fd);
+
+      while(!queue.empty())
+      {
+        face_descriptor top=queue.back();
+        queue.pop_back();
+        for(halfedge_descriptor hd : halfedges_around_face(halfedge(top, tmesh), tmesh))
+        {
+          face_descriptor adjacent_face = face(opposite(hd, tmesh), tmesh);
+          if(adjacent_face==GT::null_face() || degenerate_face_set.count(adjacent_face)==0)
+          {
+            boundary_hedges.push_back(hd);
+          }
+          else
+          {
+            if(cc_faces.insert(adjacent_face).second)
+              queue.push_back(adjacent_face);
+
+            if(hd < opposite(hd, tmesh))
+              inside_hedges.push_back(hd);
+          }
+        }
+      }
+
+#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
+      std::cout << "  Deal with a cc of " << cc_faces.size() << " degenerate faces.\n";
+      /// dump cc_faces
+      {
+        int id = 0;
+        std::map<vertex_descriptor, int> vids;
+        for(face_descriptor f : cc_faces)
+        {
+          if(vids.insert(std::make_pair(target(halfedge(f, tmesh), tmesh), id)).second) ++id;
+          if(vids.insert(std::make_pair(target(next(halfedge(f, tmesh), tmesh), tmesh), id)).second) ++id;
+          if(vids.insert(std::make_pair(target(next(next(halfedge(f, tmesh), tmesh), tmesh), tmesh), id)).second) ++id;
+        }
+
+        std::ofstream output("/tmp/cc_faces.off");
+        output << std::setprecision(17);
+        output << "OFF\n" << vids.size() << " " << cc_faces.size() << " 0\n";
+        std::vector<typename Traits::Point_3> points(vids.size());
+        typedef std::pair<const vertex_descriptor, int> Pair_type;
+        for(const Pair_type& p : vids)
+
+          points[p.second] = get(vpmap, p.first);
+        for(const Point_3& p : points)
+          output << p << "\n";
+
+        for(face_descriptor f : cc_faces)
+        {
+          output << "3 "
+                 << vids[target(halfedge(f, tmesh), tmesh)] << " "
+                 << vids[target(next(halfedge(f, tmesh), tmesh), tmesh)] << " "
+                 << vids[target(next(next(halfedge(f, tmesh), tmesh), tmesh), tmesh)] << "\n";
+        }
+
+        for(std::size_t pid=2; pid!=points.size(); ++pid)
+        {
+          CGAL_assertion(collinear(points[0], points[1], points[pid]));
+        }
+      }
+#endif
+
+      // find vertices strictly inside the cc
+      std::set<vertex_descriptor> boundary_vertices;
+      for(halfedge_descriptor hd : boundary_hedges)
+        boundary_vertices.insert(target(hd, tmesh));
+
+      std::set<vertex_descriptor> inside_vertices;
+      for(halfedge_descriptor hd : inside_hedges)
+      {
+        if(!boundary_vertices.count(target(hd, tmesh)))
+          inside_vertices.insert(target(hd, tmesh));
+        if(!boundary_vertices.count(source(hd, tmesh)))
+          inside_vertices.insert(source(hd, tmesh));
+      }
+
+      // v-e+f = 1 for a topological disk and e = (3f+#boundary_edges)/2
+      if(boundary_vertices.size()+inside_vertices.size() -
+          (cc_faces.size()+boundary_hedges.size())/2 != 1)
+      {
+        //cc_faces does not define a topological disk
+        // @todo Find to way to handle that case
+#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
+        std::cout << "  WARNING: Cannot remove the component of degenerate faces: not a topological disk.\n";
+#endif
+
+        for(face_descriptor f : cc_faces)
+          degenerate_face_set.erase(f);
+
+        continue;
+      }
+
+      // preliminary step to check if the operation is possible
+      // sort the boundary points along the common supporting line
+      //    we first need a reference point
+      typedef internal::Less_vertex_point<TriangleMesh, VertexPointMap, Traits> Less_vertex;
+      std::pair<
+          typename std::set<vertex_descriptor>::iterator,
+          typename std::set<vertex_descriptor>::iterator > ref_vertices =
+          boost::minmax_element(boundary_vertices.begin(),
+                                boundary_vertices.end(),
+                                Less_vertex(traits, vpmap));
+
+      //    and then we sort the vertices using this reference point
+      typedef std::set<Point_3>                                Sorted_point_set;
+      Sorted_point_set sorted_points;
+      for(vertex_descriptor v : boundary_vertices)
+        sorted_points.insert(get(vpmap, v));
+
+      CGAL_assertion(sorted_points.size()==
+                     std::set<typename Traits::Point_3>(sorted_points.begin(),
+                                                        sorted_points.end()).size());
+
+      CGAL_assertion(get(vpmap, *ref_vertices.first) == *sorted_points.begin());
+      CGAL_assertion(get(vpmap, *ref_vertices.second) == *std::prev(sorted_points.end()));
+
+      const Point_3& xtrm1 = *sorted_points.begin();
+      const Point_3& xtrm2 = *std::prev(sorted_points.end());
+
+      // recover halfedges on the hole, bounded by the reference vertices
+      std::vector<halfedge_descriptor> side_one, side_two;
+
+      // look for the outgoing border halfedge of the first extreme point
+      for(halfedge_descriptor hd : boundary_hedges)
+      {
+        if(get(vpmap, source(hd, tmesh)) == xtrm1)
+        {
+          side_one.push_back(hd);
+          break;
+        }
+      }
+
+      CGAL_assertion(side_one.size() == 1);
+
+      bool non_monotone_border = false;
+
+      while(get(vpmap, target(side_one.back(), tmesh)) != xtrm2)
+      {
+        vertex_descriptor prev_vertex = target(side_one.back(), tmesh);
+        for(halfedge_descriptor hd : boundary_hedges)
+        {
+          if(source(hd, tmesh) == prev_vertex)
+          {
+            if(get(vpmap, target(hd, tmesh)) < get(vpmap, prev_vertex))
+              non_monotone_border = true;
+
+            side_one.push_back(hd);
+            break;
+          }
+        }
+
+        if(non_monotone_border)
+          break;
+      }
+
+      if(non_monotone_border)
+      {
+#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
+        std::cout << "  WARNING: Cannot remove the component of degenerate faces: border not a monotonic cycle.\n";
+#endif
+
+        for(face_descriptor f : cc_faces)
+          degenerate_face_set.erase(f);
+
+        continue;
+      }
+
+      // look for the outgoing border halfedge of second extreme vertex
+      for(halfedge_descriptor hd : boundary_hedges)
+      {
+        if(source(hd, tmesh) == target(side_one.back(), tmesh))
+        {
+          side_two.push_back(hd);
+          break;
+        }
+      }
+
+      CGAL_assertion(side_two.size() == 1);
+
+      while(target(side_two.back(), tmesh) != source(side_one.front(), tmesh))
+      {
+        vertex_descriptor prev_vertex = target(side_two.back(), tmesh);
+        for(halfedge_descriptor hd : boundary_hedges)
+        {
+          if(source(hd, tmesh) == prev_vertex)
+          {
+            if(get(vpmap, target(hd, tmesh)) > get(vpmap, prev_vertex))
+              non_monotone_border = true;
+
+            side_two.push_back(hd);
+            break;
+          }
+        }
+
+        if(non_monotone_border)
+          break;
+      }
+
+      if(non_monotone_border)
+      {
+#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
+        std::cout << "  WARNING: Cannot remove the component of degenerate faces: border not a monotonic cycle.\n";
+#endif
+
+        for(face_descriptor f : cc_faces)
+          degenerate_face_set.erase(f);
+
+        continue;
+      }
+
+      CGAL_assertion(side_one.size()+side_two.size()==boundary_hedges.size());
+
+      // reverse the order of the second side so as to follow
+      // the same order than side one
+      std::reverse(side_two.begin(), side_two.end());
+      for(halfedge_descriptor& h : side_two)
+        h = opposite(h, tmesh);
+
+      //make sure the points of the vertices along side_one are correctly sorted
+      std::vector<Point_3> side_points;
+      side_points.reserve(side_one.size()+1);
+      side_points.push_back(get(vpmap, source(side_one.front(), tmesh)));
+
+      for(halfedge_descriptor h : side_one)
+        side_points.push_back(get(vpmap, target(h, tmesh)));
+
+      CGAL_assertion(get(vpmap,source(side_one.front(), tmesh)) == side_points.front());
+      CGAL_assertion(get(vpmap,target(side_one.back(), tmesh)) == side_points.back());
+
+      // @todo the reordering could lead to the apparition of null edges.
+      std::sort(side_points.begin(), side_points.end());
+
+      CGAL_assertion(std::unique(side_points.begin(), side_points.end())==side_points.end());
+      for(std::size_t i=0; i<side_one.size()-1; ++i)
+        put(vpmap, target(side_one[i], tmesh), side_points[i+1]);
+
+      //same thing for side_two
+      side_points.clear();
+      side_points.reserve(side_two.size()+1);
+      side_points.push_back(get(vpmap, source(side_two.front(), tmesh)));
+
+      for(halfedge_descriptor h : side_two)
+        side_points.push_back(get(vpmap,target(h, tmesh)));
+
+      CGAL_assertion(get(vpmap, source(side_two.front(), tmesh)) == side_points.front());
+      CGAL_assertion(get(vpmap, target(side_two.back(), tmesh)) == side_points.back());
+
+      // @todo the reordering could lead to the apparition of null edges.
+      std::sort(side_points.begin(), side_points.end());
+
+      CGAL_assertion(std::unique(side_points.begin(), side_points.end())==side_points.end());
+      for(std::size_t i=0; i<side_two.size()-1; ++i)
+        put(vpmap, target(side_two[i], tmesh), side_points[i+1]);
+
+      CGAL_assertion(source(side_one.front(), tmesh) == *ref_vertices.first);
+      CGAL_assertion(source(side_two.front(), tmesh) == *ref_vertices.first);
+      CGAL_assertion(target(side_one.back(), tmesh) == *ref_vertices.second);
+      CGAL_assertion(target(side_two.back(), tmesh) == *ref_vertices.second);
+
+      typename Sorted_point_set::iterator it_pt = std::next(sorted_points.begin()),
+                                          it_pt_end = std::prev(sorted_points.end());
+
+      bool non_collapsable = false;
+      typename std::vector<halfedge_descriptor>::iterator side_one_it = side_one.begin();
+      typename std::vector<halfedge_descriptor>::iterator side_two_it = side_two.begin();
+      for(;it_pt!=it_pt_end;++it_pt)
+      {
+        // check if it_pt is the point of the target of one or two halfedges
+        bool target_of_side_one = (get(vpmap, target(*side_one_it, tmesh)) == *it_pt);
+        bool target_of_side_two = (get(vpmap, target(*side_two_it, tmesh)) == *it_pt);
+
+        if(target_of_side_one && target_of_side_two)
+        {
+          for(halfedge_descriptor h : halfedges_around_target(*side_one_it, tmesh))
+          {
+            if(source(h, tmesh) == target(*side_two_it, tmesh))
+            {
+              non_collapsable = true;
+              break;
+            }
+          }
+        }
+        else
+        {
+          CGAL_assertion(target_of_side_one || target_of_side_two);
+          vertex_descriptor v1 = target_of_side_one ? target(*side_one_it, tmesh)
+                                                    : target(*side_two_it, tmesh);
+          vertex_descriptor v2 = target_of_side_two ? target(next(opposite(*side_one_it, tmesh), tmesh), tmesh)
+                                                    : target(next(*side_two_it, tmesh), tmesh);
+          for(halfedge_descriptor h : halfedges_around_target(v1, tmesh))
+          {
+            if(source(h, tmesh)==v2)
+            {
+              non_collapsable=true;
+              break;
+            }
+          }
+        }
+
+        if(non_collapsable) break;
+        if(target_of_side_one) ++side_one_it;
+        if(target_of_side_two) ++side_two_it;
+      }
+
+      if(non_collapsable)
+      {
+        for(face_descriptor f : cc_faces)
+          degenerate_face_set.erase(f);
+
+#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
+        std::cout << "  WARNING: cannot remove a connected components of degenerate faces.\n";
+#endif
+        continue;
+      }
+
+      // now proceed to the fix
+      // update the face and halfedge vertex pointers on the boundary
+      for(halfedge_descriptor h : boundary_hedges)
+      {
+        set_face(h, GT::null_face(), tmesh);
+        set_halfedge(target(h, tmesh), h, tmesh);
+      }
+
+      // update next/prev pointers of boundary_hedges
+      for(halfedge_descriptor h : boundary_hedges)
+      {
+        halfedge_descriptor next_candidate = next(h, tmesh);
+        while(face(next_candidate, tmesh)!=GT::null_face())
+          next_candidate = next(opposite(next_candidate, tmesh), tmesh);
+
+        set_next(h, next_candidate, tmesh);
+      }
+
+      // remove degenerate faces
+      for(face_descriptor f : cc_faces)
+      {
+        degenerate_face_set.erase(f);
+        remove_face(f, tmesh);
+      }
+
+      // remove interior edges
+      for(halfedge_descriptor h : inside_hedges)
+        remove_edge(edge(h, tmesh), tmesh);
+
+      // remove interior vertices
+      for(vertex_descriptor v : inside_vertices)
+        remove_vertex(v, tmesh);
+
+#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
+      std::cout << "  side_one.size() " << side_one.size() << "\n";
+      std::cout << "  side_two.size() " << side_two.size() << "\n";
+#endif
+
+      CGAL_assertion(source(side_one.front(), tmesh) == *ref_vertices.first);
+      CGAL_assertion(source(side_two.front(), tmesh) == *ref_vertices.first);
+      CGAL_assertion(target(side_one.back(), tmesh) == *ref_vertices.second);
+      CGAL_assertion(target(side_two.back(), tmesh) == *ref_vertices.second);
+
+      // now split each side to contains the same sequence of points
+      //    first side
+      int hi = 0;
+
+      for(typename Sorted_point_set::iterator it=std::next(sorted_points.begin()),
+                                              it_end=sorted_points.end(); it!=it_end; ++it)
+      {
+        CGAL_assertion(*std::prev(it) == get(vpmap, source(side_one[hi], tmesh)));
+
+        if(*it != get(vpmap, target(side_one[hi], tmesh)))
+        {
+          // split the edge and update the point
+          halfedge_descriptor h1 = next(opposite(side_one[hi], tmesh), tmesh);
+          put(vpmap, target(Euler::split_edge(side_one[hi], tmesh), tmesh), *it);
+
+          // split_edge updates the halfedge of the source vertex of h,
+          // since we reuse later the halfedge of the first refernce vertex
+          // we must set it as we need.
+          if(source(h1, tmesh) == *ref_vertices.first)
+            set_halfedge(*ref_vertices.first, prev(prev(side_one[hi], tmesh), tmesh), tmesh);
+
+          // retriangulate the opposite face
+          if(face(h1, tmesh) != GT::null_face())
+            Euler::split_face(h1, opposite(side_one[hi], tmesh), tmesh);
+        }
+        else
+        {
+          ++hi;
+        }
+      }
+
+      //    second side
+      hi = 0;
+      for(typename Sorted_point_set::iterator it=std::next(sorted_points.begin()),
+                                              it_end=sorted_points.end(); it!=it_end; ++it)
+      {
+        CGAL_assertion(*std::prev(it) == get(vpmap, source(side_two[hi], tmesh)));
+        if(*it != get(vpmap, target(side_two[hi], tmesh)))
+        {
+          // split the edge and update the point
+          halfedge_descriptor h2 = Euler::split_edge(side_two[hi], tmesh);
+          put(vpmap, target(h2, tmesh), *it);
+
+          // split_edge updates the halfedge of the source vertex of h,
+          // since we reuse later the halfedge of the first refernce vertex
+          // we must set it as we need.
+          if(source(h2, tmesh) == *ref_vertices.first)
+            set_halfedge(*ref_vertices.first, opposite(h2, tmesh), tmesh);
+
+          // retriangulate the face
+          if(face(h2, tmesh) != GT::null_face())
+            Euler::split_face(h2, next(side_two[hi], tmesh), tmesh);
+        }
+        else
+        {
+          ++hi;
+        }
+      }
+
+      CGAL_assertion(target(halfedge(*ref_vertices.first, tmesh), tmesh) == *ref_vertices.first);
+      CGAL_assertion(face(halfedge(*ref_vertices.first, tmesh), tmesh) == GT::null_face());
+
+#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
+      {
+        halfedge_descriptor h_side2 = halfedge(*ref_vertices.first, tmesh);
+        halfedge_descriptor h_side1 = next(h_side2, tmesh);
+
+        do
+        {
+          CGAL_assertion(get(vpmap, source(h_side1, tmesh)) == get(vpmap, target(h_side2, tmesh)));
+          CGAL_assertion(get(vpmap, target(h_side1, tmesh)) == get(vpmap, source(h_side2, tmesh)));
+
+          if(target(next(opposite(h_side1, tmesh), tmesh), tmesh) ==
+               target(next(opposite(h_side2, tmesh), tmesh), tmesh))
+          {
+            CGAL_assertion(!"Forbidden simplification");
+          }
+
+          h_side2 = prev(h_side2, tmesh);
+          h_side1 = next(h_side1, tmesh);
+        }
+        while(target(h_side1, tmesh) != *ref_vertices.second);
+      }
+#endif
+
+      // remove side1 and replace its opposite hedges by those of side2
+      halfedge_descriptor h_side2 = halfedge(*ref_vertices.first, tmesh);
+      halfedge_descriptor h_side1 = next(h_side2, tmesh);
+      for(;;)
+      {
+        CGAL_assertion(get(vpmap, source(h_side1, tmesh)) == get(vpmap, target(h_side2, tmesh)));
+        CGAL_assertion(get(vpmap, target(h_side1, tmesh)) == get(vpmap, source(h_side2, tmesh)));
+
+        // backup target vertex
+        vertex_descriptor vertex_to_remove = target(h_side1, tmesh);
+        if(vertex_to_remove!=*ref_vertices.second)
+        {
+          vertex_descriptor replacement_vertex = source(h_side2, tmesh);
+          // replace the incident vertex
+          for(halfedge_descriptor hd : halfedges_around_target(h_side1, tmesh))
+            set_target(hd, replacement_vertex, tmesh);
+        }
+
+        // prev side2 hedge for next loop
+        halfedge_descriptor h_side2_for_next_turn = prev(h_side2, tmesh);
+
+        // replace the opposite of h_side1 by h_side2
+        halfedge_descriptor opposite_h_side1 = opposite(h_side1, tmesh);
+        face_descriptor the_face = face(opposite_h_side1, tmesh);
+        set_face(h_side2, the_face, tmesh);
+
+        if(the_face!=GT::null_face())
+          set_halfedge(the_face, h_side2, tmesh);
+
+        set_next(h_side2, next(opposite_h_side1, tmesh), tmesh);
+        set_next(prev(opposite_h_side1, tmesh), h_side2, tmesh);
+
+        // take the next hedges
+        edge_descriptor edge_to_remove = edge(h_side1, tmesh);
+        h_side1 = next(h_side1, tmesh);
+
+        // now remove the extra edge
+        remove_edge(edge_to_remove, tmesh);
+
+        // ... and the extra vertex if it's not the second reference
+        if(vertex_to_remove == *ref_vertices.second)
+        {
+          // update the halfedge pointer of the last vertex (others were already from side 2)
+          CGAL_assertion(target(opposite(h_side2, tmesh), tmesh) == vertex_to_remove);
+          set_halfedge(vertex_to_remove, opposite(h_side2, tmesh), tmesh);
+          break;
+        }
+        else
+        {
+          remove_vertex(vertex_to_remove , tmesh);
+        }
+
+        h_side2 = h_side2_for_next_turn;
+      }
+    }
+  }
+
+  return all_removed;
+}
+
+template <typename FaceRange, typename TriangleMesh>
+bool remove_degenerate_faces(const FaceRange& face_range,
+                             TriangleMesh& tmesh)
+{
+  return remove_degenerate_faces(face_range, tmesh, CGAL::parameters::all_default());
+}
+
+template <typename TriangleMesh, typename CGAL_PMP_NP_TEMPLATE_PARAMETERS>
+bool remove_degenerate_faces(TriangleMesh& tmesh,
+                             const CGAL_PMP_NP_CLASS& np)
+{
+  return remove_degenerate_faces(faces(tmesh), tmesh, np);
+}
+
+template<typename TriangleMesh>
+bool remove_degenerate_faces(TriangleMesh& tmesh)
+{
+  return remove_degenerate_faces(tmesh, CGAL::parameters::all_default());
+}
+
+} // namespace Polygon_mesh_processing
+} // namespace CGAL
+
+#endif // CGAL_POLYGON_MESH_PROCESSING_REPAIR_DEGENERACIES_H
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/repair.h 5.1-1/include/CGAL/Polygon_mesh_processing/repair.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/repair.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/repair.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/repair.h $
-// $Id: repair.h 71f2474 2020-03-02T09:42:44+01:00 Mael Rouxel-Labbé
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/repair.h $
+// $Id: repair.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -16,2974 +16,336 @@
 
 #include <CGAL/license/Polygon_mesh_processing/repair.h>
 
-#include <CGAL/boost/graph/Euler_operations.h>
-#include <CGAL/Dynamic_property_map.h>
-#include <CGAL/Union_find.h>
-#include <CGAL/property_map.h>
-#include <CGAL/algorithm.h>
-#include <CGAL/array.h>
-
-// headers for self-intersection removal
-#include <CGAL/Polygon_mesh_processing/self_intersections.h>
-#include <CGAL/Polygon_mesh_processing/triangulate_hole.h>
-#include <CGAL/boost/graph/selection.h>
-
-#include <CGAL/Polygon_mesh_processing/connected_components.h>
-#include <CGAL/Polygon_mesh_processing/border.h>
-#include <CGAL/Polygon_mesh_processing/shape_predicates.h>
-
-#include <CGAL/Polygon_mesh_processing/internal/named_function_params.h>
-#include <CGAL/Polygon_mesh_processing/internal/named_params_helper.h>
-
-#include <boost/range/has_range_iterator.hpp>
-
-#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
-#include <CGAL/Polygon_mesh_processing/polygon_soup_to_polygon_mesh.h>
-#include <CGAL/IO/OFF_reader.h>
-#endif
+#include <CGAL/Polygon_mesh_processing/manifoldness.h>
+#include <CGAL/Polygon_mesh_processing/repair_degeneracies.h>
+#include <CGAL/Polygon_mesh_processing/repair_self_intersections.h>
+#include <CGAL/Polygon_mesh_processing/bbox.h>
+#include <CGAL/Polygon_mesh_processing/measure.h>
 
-#include <boost/algorithm/minmax_element.hpp>
-#include <boost/type_traits/is_same.hpp>
-#include <boost/utility/enable_if.hpp>
-
-#include <fstream>
-#include <iostream>
-#include <map>
-#include <set>
-#include <utility>
-#include <vector>
+#include <CGAL/boost/graph/iterator.h>
 
-#ifdef DOXYGEN_RUNNING
-#define CGAL_PMP_NP_TEMPLATE_PARAMETERS NamedParameters
-#define CGAL_PMP_NP_CLASS NamedParameters
-#endif
+#include <vector>
 
-namespace CGAL{
+namespace CGAL {
 namespace Polygon_mesh_processing {
-namespace debug{
-
-  template <class TriangleMesh, class VertexPointMap>
-  std::ostream& dump_edge_neighborhood(
-    typename boost::graph_traits<TriangleMesh>::edge_descriptor ed,
-    TriangleMesh& tmesh,
-    const VertexPointMap& vpmap,
-    std::ostream& out)
-  {
-    typedef boost::graph_traits<TriangleMesh> GT;
-    typedef typename GT::halfedge_descriptor halfedge_descriptor;
-    typedef typename GT::vertex_descriptor vertex_descriptor;
-    typedef typename GT::face_descriptor face_descriptor;
-
-    halfedge_descriptor h = halfedge(ed, tmesh);
-
-    std::map<vertex_descriptor, int> vertices;
-    std::set<face_descriptor> faces;
-    int vindex=0;
-    for(halfedge_descriptor hd : halfedges_around_target(h, tmesh))
-    {
-      if ( vertices.insert(std::make_pair(source(hd, tmesh), vindex)).second )
-        ++vindex;
-      if (!is_border(hd, tmesh))
-        faces.insert( face(hd, tmesh) );
-    }
-
-    h=opposite(h, tmesh);
-    for(halfedge_descriptor hd : halfedges_around_target(h, tmesh))
-    {
-      if ( vertices.insert(std::make_pair(source(hd, tmesh), vindex)).second )
-        ++vindex;
-      if (!is_border(hd, tmesh))
-        faces.insert( face(hd, tmesh) );
-    }
-
-    std::vector<vertex_descriptor> ordered_vertices(vertices.size());
-    typedef std::pair<const vertex_descriptor, int> Pair_type;
-    for(const Pair_type& p : vertices)
-      ordered_vertices[p.second]=p.first;
-
-    out << "OFF\n" << ordered_vertices.size() << " " << faces.size() << " 0\n";
-    for(vertex_descriptor vd : ordered_vertices)
-      out << get(vpmap, vd) << "\n";
-    for(face_descriptor fd : faces)
-    {
-      out << "3";
-      h=halfedge(fd,tmesh);
-      for(halfedge_descriptor hd : halfedges_around_face(h, tmesh))
-        out << " " << vertices[target(hd, tmesh)];
-      out << "\n";
-    }
-    return out;
-  }
-
-  template <class FaceRange, class TriangleMesh>
-  void dump_cc_faces(const FaceRange& cc_faces, const TriangleMesh& tm, std::ostream& output)
-  {
-    typedef typename boost::property_map<TriangleMesh, boost::vertex_point_t>::const_type Vpm;
-    typedef typename boost::property_traits<Vpm>::value_type Point_3;
-    typedef typename boost::graph_traits<TriangleMesh>::vertex_descriptor vertex_descriptor;
-    typedef typename boost::graph_traits<TriangleMesh>::face_descriptor face_descriptor;
-
-    Vpm vpm = get(boost::vertex_point, tm);
-
-    int id=0;
-    std::map<vertex_descriptor, int> vids;
-    for(face_descriptor f : cc_faces)
-    {
-      if ( vids.insert( std::make_pair( target(halfedge(f, tm), tm), id) ).second ) ++id;
-      if ( vids.insert( std::make_pair( target(next(halfedge(f, tm), tm), tm), id) ).second ) ++id;
-      if ( vids.insert( std::make_pair( target(next(next(halfedge(f, tm), tm), tm), tm), id) ).second ) ++id;
-    }
-    output << std::setprecision(17);
-    output << "OFF\n" << vids.size() << " " << cc_faces.size() << " 0\n";
-    std::vector<Point_3> points(vids.size());
-    typedef std::pair<const vertex_descriptor, int> Pair_type;
-    for(Pair_type p : vids)
-      points[p.second]=get(vpm, p.first);
-    for(Point_3 p : points)
-      output << p << "\n";
-    for(face_descriptor f : cc_faces)
-    {
-      output << "3 "
-             << vids[ target(halfedge(f, tm), tm) ] << " "
-             << vids[ target(next(halfedge(f, tm), tm), tm) ] << " "
-             << vids[ target(next(next(halfedge(f, tm), tm), tm), tm) ] << "\n";
-    }
-  }
-
-} //end of namespace debug
-
-namespace internal {
-
-template <class HalfedgeGraph, class VertexPointMap, class Traits>
-struct Less_vertex_point{
-  typedef typename boost::graph_traits<HalfedgeGraph>::vertex_descriptor vertex_descriptor;
-  const Traits& m_traits;
-  const VertexPointMap& m_vpmap;
-  Less_vertex_point(const Traits& traits, const VertexPointMap& vpmap)
-    : m_traits(traits)
-    , m_vpmap(vpmap) {}
-  bool operator()(vertex_descriptor v1, vertex_descriptor v2) const{
-    return m_traits.less_xyz_3_object()(get(m_vpmap, v1), get(m_vpmap, v2));
-  }
-};
-
-} // end namespace internal
 
 /// \ingroup PMP_repairing_grp
-/// collects the degenerate edges within a given range of edges.
+/// removes the isolated vertices from any polygon mesh.
+/// A vertex is considered isolated if it is not incident to any simplex
+/// of higher dimension.
 ///
-/// @tparam EdgeRange a model of `Range` with value type `boost::graph_traits<TriangleMesh>::%edge_descriptor`
-/// @tparam TriangleMesh a model of `HalfedgeGraph`
-/// @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+/// @tparam PolygonMesh a model of `FaceListGraph` and `MutableFaceGraph`
 ///
-/// @param edges a subset of edges of `tm`
-/// @param tm a triangle mesh
-/// @param out an output iterator in which the degenerate edges are written
-/// @param np optional \ref pmp_namedparameters "Named Parameters" described below
+/// @param pmesh the polygon mesh to be repaired
 ///
-/// \cgalNamedParamsBegin
-///    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `tm`.
-///                                      The type of this map is model of `ReadWritePropertyMap`.
-///                                      If this parameter is omitted, an internal property map for
-///                                      `CGAL::vertex_point_t` should be available in `TriangleMesh`
-///    \cgalParamEnd
-///    \cgalParamBegin{geom_traits} a geometric traits class instance.
-///                                The traits class must provide the nested type `Point_3`,
-///                                and the nested functor `Equal_3` to check whether two points are identical.
-///    \cgalParamEnd
-/// \cgalNamedParamsEnd
-template <class EdgeRange, class TriangleMesh, class OutputIterator, class NamedParameters>
-OutputIterator degenerate_edges(const EdgeRange& edges,
-                                const TriangleMesh& tm,
-                                OutputIterator out,
-                                const NamedParameters& np)
+/// @return number of removed isolated vertices
+///
+template <class PolygonMesh>
+std::size_t remove_isolated_vertices(PolygonMesh& pmesh)
 {
-  typedef typename boost::graph_traits<TriangleMesh>::edge_descriptor edge_descriptor;
+  typedef typename boost::graph_traits<PolygonMesh>::vertex_descriptor vertex_descriptor;
+  std::vector<vertex_descriptor> to_be_removed;
 
-  for(edge_descriptor ed : edges)
+  for(vertex_descriptor v : vertices(pmesh))
   {
-    if(is_degenerate_edge(ed, tm, np))
-      *out++ = ed;
+    if (CGAL::halfedges_around_target(v, pmesh).first
+      == CGAL::halfedges_around_target(v, pmesh).second)
+      to_be_removed.push_back(v);
   }
-  return out;
-}
-
-template <class EdgeRange, class TriangleMesh, class OutputIterator>
-OutputIterator degenerate_edges(const EdgeRange& edges,
-                                const TriangleMesh& tm,
-                                OutputIterator out,
-                                typename boost::enable_if<
-                                  typename boost::has_range_iterator<EdgeRange>
-                                         >::type* = 0)
-{
-  return degenerate_edges(edges, tm, out, CGAL::parameters::all_default());
+  std::size_t nb_removed = to_be_removed.size();
+  for(vertex_descriptor v : to_be_removed)
+  {
+    remove_vertex(v, pmesh);
+  }
+  return nb_removed;
 }
 
 /// \ingroup PMP_repairing_grp
-/// calls the function `degenerate_edges()` with the range: `edges(tm)`.
 ///
-/// See above for the comprehensive description of the parameters.
+/// removes connected components whose area or volume is under a certain threshold value.
 ///
-template <class TriangleMesh, class OutputIterator, class NamedParameters>
-OutputIterator degenerate_edges(const TriangleMesh& tm,
-                                OutputIterator out,
-                                const NamedParameters& np
-#ifndef DOXYGEN_RUNNING
-                                , typename boost::disable_if<
-                                    boost::has_range_iterator<TriangleMesh>
-                                      >::type* = 0
-#endif
-                                                     )
-{
-  return degenerate_edges(edges(tm), tm, out, np);
-}
-
-template <class TriangleMesh, class OutputIterator>
-OutputIterator
-degenerate_edges(const TriangleMesh& tm, OutputIterator out)
-{
-  return degenerate_edges(edges(tm), tm, out, CGAL::parameters::all_default());
-}
-
-/// \ingroup PMP_repairing_grp
-/// collects the degenerate faces within a given range of faces.
+/// Thresholds are provided via \ref bgl_namedparameters "Named Parameters". (see below).
+/// If thresholds are not provided by the user, default values are computed as follows:
+/// - the area threshold is taken as the square of one percent of the length of the diagonal
+///   of the bounding box of the mesh.
+/// - the volume threshold is taken as the third power of one percent of the length of the diagonal
+///   of the bounding box of the mesh.
+///
+/// The area and volume of a connected component will always be positive values (regardless
+/// of the orientation of the mesh).
+///
+/// As a consequence of the last sentence, the area or volume criteria can be disabled
+/// by passing zero (`0`) as threshold value.
 ///
-/// @tparam FaceRange a model of `Range` with value type `boost::graph_traits<TriangleMesh>::%face_descriptor`
-/// @tparam TriangleMesh a model of `FaceGraph`
-/// @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+/// \tparam TriangleMesh a model of `FaceListGraph` and `MutableFaceGraph`
+/// \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 ///
-/// @param faces a subset of faces of `tm`
-/// @param tm a triangle mesh
-/// @param out an output iterator in which the degenerate faces are put
-/// @param np optional \ref pmp_namedparameters "Named Parameters" described below
+/// \param tmesh the triangulated polygon mesh
+/// \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 ///
 /// \cgalNamedParamsBegin
-///    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `tm`.
-///                                      The type of this map is model of `ReadWritePropertyMap`.
-///                                      If this parameter is omitted, an internal property map for
-///                                      `CGAL::vertex_point_t` should be available in `TriangleMesh`
-///    \cgalParamEnd
-///    \cgalParamBegin{geom_traits} a geometric traits class instance.
-///                                 The traits class must provide the nested functor `Collinear_3`
-///                                 to check whether three points are collinear.
-///    \cgalParamEnd
+///   \cgalParamNBegin{vertex_point_map}
+///     \cgalParamDescription{a property map associating points to the vertices of `tmesh`}
+///     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+///                    as key type and `%Point_3` as value type}
+///     \cgalParamDefault{`boost::get(CGAL::vertex_point, tmesh)`}
+///     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+///                     must be available in `TriangleMesh`.}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{geom_traits}
+///     \cgalParamDescription{an instance of a geometric traits class}
+///     \cgalParamType{a class model of `Kernel`}
+///     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+///     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+///     \cgalParamExtra{Exact constructions kernels are not supported by this function.}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{face_index_map}
+///     \cgalParamDescription{a property map associating to each face of `tmesh` a unique index between `0` and `num_faces(tmesh) - 1`}
+///     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%face_descriptor`
+///                    as key type and `std::size_t` as value type}
+///     \cgalParamDefault{an automatically indexed internal map}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{area_threshold}
+///     \cgalParamDescription{a fixed value such that only connected components whose area is larger than this value are kept}
+///     \cgalParamType{`geom_traits::FT`}
+///     \cgalParamDefault{1\% of the length of the diagonal of the axis-aligned bounding box of the mesh, squared}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{volume_threshold}
+///     \cgalParamDescription{a fixed value such that only connected components whose volume is
+///                           larger than this value are kept (only applies to closed connected components)}
+///     \cgalParamType{`geom_traits::FT`}
+///     \cgalParamDefault{1\% of the length of the diagonal of the axis-aligned bounding box of the mesh, cubed}
+///     \cgalParamExtra{The mesh must be closed.}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{edge_is_constrained_map}
+///     \cgalParamDescription{a property map containing the constrained-or-not status of each edge of `tmesh`}
+///     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%edge_descriptor`
+///                    as key type and `bool` as value type. It must be default constructible.}
+///     \cgalParamDefault{a default property map where no edge is constrained}
+///     \cgalParamExtra{A constrained edge can be split or collapsed, but not flipped, nor its endpoints moved by smoothing.}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{dry_run}
+///     \cgalParamDescription{If `true`, the mesh will not be altered, but the number of components
+///                           that would be removed is returned.}
+///     \cgalParamType{Boolean}
+///     \cgalParamDefault{`false`}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{output_iterator}
+///     \cgalParamDescription{An output iterator to collect the faces that would be removed by the algorithm,
+///                           when using the "dry run" mode (see parameter `dry_run`)}
+///     \cgalParamType{a model of `OutputIterator` with value type `face_descriptor`}
+///     \cgalParamDefault{unused}
+///   \cgalParamNEnd
 /// \cgalNamedParamsEnd
 ///
-template <class FaceRange, class TriangleMesh, class OutputIterator, class NamedParameters>
-OutputIterator degenerate_faces(const FaceRange& faces,
-                                const TriangleMesh& tm,
-                                OutputIterator out,
-                                const NamedParameters& np)
-{
-  typedef typename boost::graph_traits<TriangleMesh>::face_descriptor face_descriptor;
-
-  for(face_descriptor fd : faces)
-  {
-    if(is_degenerate_triangle_face(fd, tm, np))
-      *out++ = fd;
-  }
-  return out;
-}
-
-template <class FaceRange, class TriangleMesh, class OutputIterator>
-OutputIterator degenerate_faces(const FaceRange& faces,
-                                const TriangleMesh& tm,
-                                OutputIterator out,
-                                typename boost::enable_if<
-                                  boost::has_range_iterator<FaceRange>
-                                    >::type* = 0)
-{
-  return degenerate_faces(faces, tm, out, CGAL::parameters::all_default());
-}
-
-/// \ingroup PMP_repairing_grp
-/// calls the function `degenerate_faces()` with the range: `faces(tm)`.
-///
-/// See above for the comprehensive description of the parameters.
+/// \return the number of connected components removed (ignoring isolated vertices).
 ///
-template <class TriangleMesh, class OutputIterator, class NamedParameters>
-OutputIterator degenerate_faces(const TriangleMesh& tm,
-                                OutputIterator out,
-                                const NamedParameters& np
-#ifndef DOXYGEN_RUNNING
-                                , typename boost::disable_if<
-                                    boost::has_range_iterator<TriangleMesh>
-                                      >::type* = 0
-#endif
-                                                     )
-{
-  return degenerate_faces(faces(tm), tm, out, np);
-}
-
-template <class TriangleMesh, class OutputIterator>
-OutputIterator degenerate_faces(const TriangleMesh& tm, OutputIterator out)
-{
-  return degenerate_faces(faces(tm), tm, out, CGAL::parameters::all_default());
-}
-
-// this function removes a border edge even if it does not satisfy the link condition.
-// null_vertex() is returned if the removal changes the topology of the input
-template <typename TriangleMesh, typename EdgeSet, typename FaceSet>
-typename boost::graph_traits<TriangleMesh>::vertex_descriptor
-remove_a_border_edge(typename boost::graph_traits<TriangleMesh>::edge_descriptor ed,
-                     TriangleMesh& tm,
-                     EdgeSet& edge_set,
-                     FaceSet& face_set)
+template <typename TriangleMesh,
+          typename NamedParameters>
+std::size_t remove_connected_components_of_negligible_size(TriangleMesh& tmesh,
+                                                           const NamedParameters& np)
 {
-  typedef boost::graph_traits<TriangleMesh> GT;
-  typedef typename GT::edge_descriptor edge_descriptor;
-  typedef typename GT::halfedge_descriptor halfedge_descriptor;
-  typedef typename GT::face_descriptor face_descriptor;
-  typedef typename GT::vertex_descriptor vertex_descriptor;
-
-  halfedge_descriptor h=halfedge(ed,tm);
-
-  if ( is_border(h,tm) )
-    h=opposite(h,tm);
-
-  halfedge_descriptor opp_h = opposite(h,tm);
-  CGAL_assertion(is_border(opp_h,tm));
-  CGAL_assertion(!is_border(h,tm));
-
-  CGAL_assertion(next(next(opp_h, tm), tm) !=opp_h); // not working for a hole made of 2 edges
-  CGAL_assertion(next(next(next(opp_h, tm), tm), tm) !=opp_h); // not working for a hole make of 3 edges
-
-  if (CGAL::Euler::does_satisfy_link_condition(edge(h,tm),tm))
-  {
-    edge_set.erase(ed);
-    halfedge_descriptor h=halfedge(ed, tm);
-    if ( is_border(h, tm) )
-      h = opposite(h, tm);
+  using parameters::choose_parameter;
+  using parameters::is_default_parameter;
+  using parameters::get_parameter;
 
-    edge_set.erase(edge(prev(h, tm), tm));
-    face_set.erase(face(h, tm));
+  typedef typename boost::graph_traits<TriangleMesh>::halfedge_descriptor          halfedge_descriptor;
+  typedef typename boost::graph_traits<TriangleMesh>::face_descriptor              face_descriptor;
 
-    return CGAL::Euler::collapse_edge(ed, tm);
-  }
+  typedef typename GetGeomTraits<TriangleMesh, NamedParameters>::type              GT;
+  typedef typename GT::FT                                                          FT;
+  const GT traits = choose_parameter<GT>(get_parameter(np, internal_np::vertex_point));
 
-  // collect edges that have one vertex in the link of
-  // the vertices of h and one of the vertex of h as other vertex
-  std::set<edge_descriptor> common_incident_edges;
-  for(halfedge_descriptor hos : halfedges_around_source(h, tm))
-    for(halfedge_descriptor hot : halfedges_around_target(h, tm))
-    {
-      if( target(hos, tm) == source(hot, tm) )
-      {
-        common_incident_edges.insert( edge(hot, tm) );
-        common_incident_edges.insert( edge(hos, tm) );
-      }
-    }
+  typedef typename GetVertexPointMap<TriangleMesh, NamedParameters>::const_type    VPM;
+  typedef typename boost::property_traits<VPM>::value_type                         Point_3;
+  const VPM vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
+                                   get_const_property_map(CGAL::vertex_point, tmesh));
 
-  CGAL_assertion(common_incident_edges.size() >=2 );
+  typedef typename GetInitializedFaceIndexMap<TriangleMesh, NamedParameters>::type FaceIndexMap;
+  FaceIndexMap fim = CGAL::get_initialized_face_index_map(tmesh, np);
 
-  // in the following loop, we visit define a connected component of
-  // faces bounded by edges in common_incident_edges and h. We look
-  // for the maximal one. This set of faces is the one that will
-  // disappear while collapsing ed
-  std::set<face_descriptor> marked_faces;
-
-  std::vector<halfedge_descriptor> queue;
-  queue.push_back( opposite(next(h,tm), tm) );
-  queue.push_back( opposite(prev(h,tm), tm) );
-  marked_faces.insert( face(h, tm) );
+  FT area_threshold = choose_parameter(get_parameter(np, internal_np::area_threshold), FT(-1));
+  FT volume_threshold = choose_parameter(get_parameter(np, internal_np::volume_threshold), FT(-1));
 
-  do
-  {
-    std::vector<halfedge_descriptor> boundary;
-    while(!queue.empty())
-    {
-      halfedge_descriptor back=queue.back();
-      queue.pop_back();
-      face_descriptor fback=face(back,tm);
-      if (common_incident_edges.count(edge(back,tm)))
-      {
-        boundary.push_back(back);
-        continue;
-      }
+  // If no threshold is provided, compute it as a % of the bbox
+  const bool is_default_area_threshold = is_default_parameter(get_parameter(np, internal_np::area_threshold));
+  const bool is_default_volume_threshold = is_default_parameter(get_parameter(np, internal_np::volume_threshold));
 
-      if ( fback==GT::null_face() || !marked_faces.insert(fback).second )
-        continue;
+  const bool dry_run = choose_parameter(get_parameter(np, internal_np::dry_run), false);
 
-      queue.push_back( opposite(next(back,tm), tm) );
-      if ( is_border(queue.back(), tm) )
-      {
-#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
-        std::cout << "Boundary reached during exploration, the region to be removed is not a topological disk, not handled for now.\n";
-#endif
-        return GT::null_vertex();
-      }
+  typedef typename internal_np::Lookup_named_param_def<internal_np::output_iterator_t,
+                                                       NamedParameters,
+                                                       Emptyset_iterator>::type Output_iterator;
+  Output_iterator out = choose_parameter<Output_iterator>(get_parameter(np, internal_np::output_iterator));
 
-      queue.push_back( opposite(prev(back,tm), tm) );
-      if ( is_border(queue.back(), tm) )
-      {
-#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
-        std::cout << "Boundary reached during exploration, the region to be removed is not a topological disk, not handled for now.\n";
+#ifdef CGAL_PMP_DEBUG_SMALL_CC_REMOVAL
+  std::cout << "default threshold? " << is_default_area_threshold << " " << is_default_volume_threshold << std::endl;
 #endif
-        return GT::null_vertex();
-      }
-    }
-
-    CGAL_assertion( boundary.size() == 2 );
-    common_incident_edges.erase( edge(boundary[0], tm) );
-    common_incident_edges.erase( edge(boundary[1], tm) );
-    if (!is_border(boundary[0], tm) || common_incident_edges.empty())
-      queue.push_back(boundary[0]);
-    if (!is_border(boundary[1], tm) || common_incident_edges.empty())
-      queue.push_back(boundary[1]);
-  }
-  while(!common_incident_edges.empty());
 
-  // hk1 and hk2 are bounding the region that will be removed.
-  // The edge of hk2 will be removed and hk2 will be replaced
-  // by the opposite edge of hk1
-  halfedge_descriptor hk1=queue.front();
-  halfedge_descriptor hk2=queue.back();
-  if ( target(hk1,tm)!=source(hk2,tm) )
-    std::swap(hk1, hk2);
-
-  CGAL_assertion( target(hk1,tm)==source(hk2,tm) );
-  CGAL_assertion( source(hk1,tm)==source(h,tm) );
-  CGAL_assertion( target(hk2,tm)==target(h,tm) );
+  FT bbox_diagonal = FT(0), threshold_value = FT(0);
 
-  CGAL_assertion(is_valid_polygon_mesh(tm));
-  if (!is_selection_a_topological_disk(marked_faces, tm))
+  if(is_default_area_threshold || is_default_volume_threshold)
   {
-#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
-    std::cout << "The region to be removed is not a topological disk, not handled for now.\n";
-#endif
-    return GT::null_vertex();
+    if(is_empty(tmesh))
+      return 0;
 
-  }
+    const Bbox_3 bb = bbox(tmesh, np);
 
-  if (is_border(hk1, tm) && is_border(hk2, tm))
-  {
-#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
-    std::cout << "The region to be removed is an isolated region, not handled for now.\n";
-#endif
-    return GT::null_vertex();
-  }
+    bbox_diagonal = FT(CGAL::sqrt(CGAL::square(bb.xmax() - bb.xmin()) +
+                                  CGAL::square(bb.ymax() - bb.ymin()) +
+                                  CGAL::square(bb.zmax() - bb.zmin())));
+    threshold_value = bbox_diagonal / FT(100); // default filter is 1%
 
-  // collect vertices and edges to remove and do remove faces
-  std::set<edge_descriptor> edges_to_remove;
-  std::set<vertex_descriptor> vertices_to_remove;
-  for(face_descriptor fd : marked_faces)
-  {
-    halfedge_descriptor hd=halfedge(fd, tm);
-    for(int i=0; i<3; ++i)
-    {
-      edges_to_remove.insert( edge(hd, tm) );
-      vertices_to_remove.insert( target(hd,tm) );
-      hd=next(hd, tm);
-    }
-  }
-
-  vertex_descriptor vkept=source(hk1,tm);
-
-  //back-up next, prev halfedge pointers to be restored after removal
-  halfedge_descriptor hp=prev(opp_h, tm);
-  halfedge_descriptor hn=next(opp_h, tm);
-  halfedge_descriptor hk1_opp_next = next(hk2, tm);
-  halfedge_descriptor hk1_opp_prev = prev(hk2, tm);
-  face_descriptor hk1_opp_face = face(hk2,tm);
-
-  // we will remove the target of hk2, update vertex pointers
-  for(halfedge_descriptor hot : halfedges_around_target(hk2, tm))
-    set_target(hot, vkept, tm);
-
-  // update halfedge pointers since hk2 will be removed
-  set_halfedge(vkept, opposite(hk1, tm), tm);
-  set_halfedge(target(hk1,tm), hk1, tm);
-
-  // do not remove hk1 and its vertices
-  vertices_to_remove.erase( vkept );
-  vertices_to_remove.erase( target(hk1, tm) );
-  edges_to_remove.erase( edge(hk1,tm) );
-
-  bool hk2_equals_hp = hk2==hp;
-  CGAL_assertion( is_border(hk2, tm) == hk2_equals_hp );
-
-  /*
-  - case hk2!=hp
-
-         /\      /
-     hk1/  \hk2 /
-       /    \  /
-  ____/______\/____
-  hn   h_opp   hp
-
-  - case hk2==hp
-
-         /\
-     hk1/  \hk2 == hp
-       /    \
-  ____/______\
-  hn   h_opp
-  */
-
-  // remove vertices
-  for(vertex_descriptor vd : vertices_to_remove)
-    remove_vertex(vd, tm);
-
-  // remove edges
-  for(edge_descriptor ed : edges_to_remove)
-  {
-    edge_set.erase(ed);
-    remove_edge(ed, tm);
-  }
-
-  // remove faces
-  for(face_descriptor fd : marked_faces)
-  {
-    face_set.erase(fd);
-    remove_face(fd, tm);
-  }
-
-  // now update pointers
-  set_face(opposite(hk1, tm), hk1_opp_face, tm);
-  if (!hk2_equals_hp)
-  {
-    set_next(hp, hn, tm);
-    set_next(opposite(hk1, tm), hk1_opp_next, tm);
-    set_next(hk1_opp_prev, opposite(hk1, tm), tm);
-    set_halfedge(hk1_opp_face, opposite(hk1, tm), tm);
-  }
-  else
-  {
-    set_next(hk1_opp_prev, opposite(hk1, tm), tm);
-    set_next(opposite(hk1, tm), hn, tm);
-  }
-
-  CGAL_assertion(is_valid_polygon_mesh(tm));
-  return vkept;
-}
-
-template <class TriangleMesh>
-typename boost::graph_traits<TriangleMesh>::vertex_descriptor
-remove_a_border_edge(typename boost::graph_traits<TriangleMesh>::edge_descriptor ed,
-                     TriangleMesh& tm)
-{
-  std::set<typename boost::graph_traits<TriangleMesh>::edge_descriptor> edge_set;
-  std::set<typename boost::graph_traits<TriangleMesh>::face_descriptor> face_set;
-
-  return remove_a_border_edge(ed, tm, edge_set, face_set);
-}
-
-template <typename EdgeRange, typename TriangleMesh, typename NamedParameters, typename FaceSet>
-bool remove_degenerate_edges(const EdgeRange& edge_range,
-                             TriangleMesh& tmesh,
-                             FaceSet& face_set,
-                             const NamedParameters& np)
-{
-  CGAL_assertion(CGAL::is_triangle_mesh(tmesh));
-  CGAL_assertion(CGAL::is_valid_polygon_mesh(tmesh));
-
-  using parameters::get_parameter;
-  using parameters::choose_parameter;
-
-  typedef TriangleMesh TM;
-  typedef typename boost::graph_traits<TriangleMesh> GT;
-  typedef typename GT::edge_descriptor edge_descriptor;
-  typedef typename GT::halfedge_descriptor halfedge_descriptor;
-  typedef typename GT::face_descriptor face_descriptor;
-  typedef typename GT::vertex_descriptor vertex_descriptor;
-
-  typedef typename GetVertexPointMap<TM, NamedParameters>::type VertexPointMap;
-  VertexPointMap vpmap = choose_parameter(get_parameter(np, internal_np::vertex_point),
-                                      get_property_map(vertex_point, tmesh));
-
-  typedef typename GetGeomTraits<TM, NamedParameters>::type Traits;
-
-  std::size_t nb_deg_faces = 0;
-  bool all_removed=false;
-  bool some_removed=true;
-
-  bool preserve_genus = parameters::choose_parameter(parameters::get_parameter(np, internal_np::preserve_genus), true);
-
-  // collect edges of length 0
-  while(some_removed && !all_removed)
-  {
-    some_removed=false;
-    all_removed=true;
-    std::set<edge_descriptor> degenerate_edges_to_remove;
-    degenerate_edges(edge_range, tmesh, std::inserter(degenerate_edges_to_remove,
-                                                      degenerate_edges_to_remove.end()));
-
-#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
-    std::cout << "Found " << degenerate_edges_to_remove.size() << " null edges.\n";
-#endif
-
-    // first try to remove all collapsable edges
-    typename std::set<edge_descriptor>::iterator it = degenerate_edges_to_remove.begin();
-    while (it!=degenerate_edges_to_remove.end())
-    {
-      edge_descriptor e = *it;
-      if (CGAL::Euler::does_satisfy_link_condition(e,tmesh))
-      {
-        halfedge_descriptor h = halfedge(e, tmesh);
-        degenerate_edges_to_remove.erase(it);
-
-        // remove edges that could also be set for removal
-        if ( face(h, tmesh)!=GT::null_face() )
-        {
-          ++nb_deg_faces;
-          degenerate_edges_to_remove.erase(edge(prev(h, tmesh), tmesh));
-          face_set.erase(face(h, tmesh));
-        }
-
-        if (face(opposite(h, tmesh), tmesh)!=GT::null_face())
-        {
-          ++nb_deg_faces;
-          degenerate_edges_to_remove.erase(edge(prev(opposite(h, tmesh), tmesh), tmesh));
-          face_set.erase(face(opposite(h, tmesh), tmesh));
-        }
-
-        //now remove the edge
-        CGAL::Euler::collapse_edge(e, tmesh);
-
-        // some_removed is not updated on purpose because if nothing
-        //  happens below then nothing can be done
-        it = degenerate_edges_to_remove.begin();
-      }
-      else
-        ++it;
-    }
-
-    CGAL_assertion( is_valid_polygon_mesh(tmesh) );
-#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
-    std::cout << "Remaining " << degenerate_edges_to_remove.size() << " null edges to be handled.\n";
-#endif
-
-    while (!degenerate_edges_to_remove.empty())
-    {
-      edge_descriptor ed = *degenerate_edges_to_remove.begin();
-      degenerate_edges_to_remove.erase(degenerate_edges_to_remove.begin());
-
-      halfedge_descriptor h = halfedge(ed, tmesh);
-
-      if (CGAL::Euler::does_satisfy_link_condition(ed,tmesh))
-      {
-        // remove edges that could also be set for removal
-        if ( face(h, tmesh)!=GT::null_face() )
-        {
-          ++nb_deg_faces;
-          degenerate_edges_to_remove.erase(edge(prev(h, tmesh), tmesh));
-          face_set.erase(face(h, tmesh));
-        }
-
-        if (face(opposite(h, tmesh), tmesh)!=GT::null_face())
-        {
-          ++nb_deg_faces;
-          degenerate_edges_to_remove.erase(edge(prev(opposite(h, tmesh), tmesh), tmesh));
-          face_set.erase(face(opposite(h, tmesh), tmesh));
-        }
-
-        //now remove the edge
-        CGAL::Euler::collapse_edge(ed, tmesh);
-        some_removed = true;
-      }
-      else
-      {
-        //handle the case when the edge is incident to a triangle hole
-        //we first fill the hole and try again
-        if ( is_border(ed, tmesh) )
-        {
-          halfedge_descriptor hd = halfedge(ed,tmesh);
-          if (!is_border(hd,tmesh))
-            hd=opposite(hd,tmesh);
-
-          if (is_triangle(hd, tmesh))
-          {
-            if (!preserve_genus)
-            {
-              Euler::fill_hole(hd, tmesh);
-              degenerate_edges_to_remove.insert(ed); // reinsert the edge for future processing
-            }
-            else
-            {
-              all_removed=false;
-            }
-            continue;
-          }
-
-#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
-          std::cout << "Calling remove_a_border_edge\n";
-#endif
-
-          vertex_descriptor vd = remove_a_border_edge(ed, tmesh, degenerate_edges_to_remove, face_set);
-          if (vd == GT::null_vertex())
-          {
-            // @todo: if some border edges are later removed, the edge might be processable later
-            // for example if it belongs to  boundary cycle of edges where the number of non-degenerate
-            // edges is 2. That's what happen with fused_vertices.off in the testsuite where the edges
-            // are not processed the same way with Polyhedron and Surface_mesh. In the case of Polyhedron
-            // more degenerate edges could be removed.
-            all_removed=false;
-          }
-          else
-            some_removed=true;
-
-          continue;
-        }
-        else
-        {
-          halfedge_descriptor hd = halfedge(ed,tmesh);
-          // if both vertices are boundary vertices we can't do anything
-          bool impossible = false;
-          for(halfedge_descriptor h : halfedges_around_target(hd, tmesh))
-          {
-            if (is_border(h, tmesh))
-            {
-              impossible = true;
-              break;
-            }
-          }
-
-          if (impossible)
-          {
-            impossible=false;
-            for(halfedge_descriptor h : halfedges_around_source(hd, tmesh))
-            {
-              if (is_border(h, tmesh))
-              {
-                impossible = true;
-                break;
-              }
-            }
-
-            if (impossible)
-            {
-              all_removed=false;
-              continue;
-            }
-          }
-        }
-
-        // When the edge does not satisfy the link condition, it means that it cannot be
-        // collapsed as is. In the following if there is a topological issue
-        // with contracting the edge (component or geometric feature that disappears),
-        // nothing is done.
-        // We start by marking the faces that are incident to an edge endpoint.
-        // If the set of marked faces is a topologically disk, then we simply remove all the simplicies
-        // inside the disk and star the hole with the edge vertex kept.
-        // If the set of marked faces is not a topological disk, it has some non-manifold vertices
-        // on its boundary. We need to mark additional faces to make it a topological disk.
-        // We can then apply the star hole procedure.
-        // Right now we additionally mark the smallest connected components of non-marked faces
-        // (using the number of faces)
-
-        //backup central point
-        typename Traits::Point_3 pt = get(vpmap, source(ed, tmesh));
-
-        // mark faces of the link of each endpoints of the edge which collapse is not topologically valid
-        std::set<face_descriptor> marked_faces;
-
-        //   first endpoint
-        for(halfedge_descriptor hd : CGAL::halfedges_around_target(halfedge(ed,tmesh), tmesh) )
-          if (!is_border(hd,tmesh)) marked_faces.insert( face(hd, tmesh) );
-
-        //   second endpoint
-        for(halfedge_descriptor hd : CGAL::halfedges_around_target(opposite(halfedge(ed, tmesh), tmesh), tmesh) )
-          if (!is_border(hd,tmesh)) marked_faces.insert( face(hd, tmesh) );
-
-        // extract the halfedges on the boundary of the marked region
-        std::vector<halfedge_descriptor> border;
-        for(face_descriptor fd : marked_faces)
-        {
-          for(halfedge_descriptor hd : CGAL::halfedges_around_face(halfedge(fd,tmesh), tmesh))
-          {
-            halfedge_descriptor hd_opp = opposite(hd, tmesh);
-            if ( is_border(hd_opp, tmesh) ||
-                 marked_faces.count( face(hd_opp, tmesh) ) == 0 )
-            {
-              border.push_back( hd );
-            }
-          }
-        }
-
-        if (border.empty() )
-        {
-          // a whole connected component (without boundary) got selected and will disappear (not handled for now)
-#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
-            std::cout << "Trying to remove a whole connected component, not handled yet\n";
+#ifdef CGAL_PMP_DEBUG_SMALL_CC_REMOVAL
+    std::cout << "bb xmin xmax: " << bb.xmin() << " " << bb.xmax() << std::endl;
+    std::cout << "bb ymin ymax: " << bb.ymin() << " " << bb.ymax() << std::endl;
+    std::cout << "bb zmin zmax: " << bb.zmin() << " " << bb.zmax() << std::endl;
+    std::cout << "bbox_diagonal: " << bbox_diagonal << std::endl;
+    std::cout << "threshold_value: " << threshold_value << std::endl;
 #endif
-          all_removed=false;
-          continue;
-        }
-
-        // define cc of border halfedges: two halfedges are in the same cc
-        // if they are on the border of the cc of non-marked faces.
-        typedef CGAL::Union_find<halfedge_descriptor> UF_ds;
-        UF_ds uf;
-        std::map<halfedge_descriptor, typename UF_ds::handle> handles;
-        // one cc per border halfedge
-        for(halfedge_descriptor hd : border)
-          handles.insert( std::make_pair(hd, uf.make_set(hd)) );
-
-        // join cc's
-        for(halfedge_descriptor hd : border)
-        {
-          CGAL_assertion( marked_faces.count( face( hd, tmesh) ) > 0);
-          CGAL_assertion( marked_faces.count( face( opposite(hd, tmesh), tmesh) ) == 0 );
-          halfedge_descriptor candidate = hd;
-
-          do {
-            candidate = prev( opposite(candidate, tmesh), tmesh );
-          } while( !marked_faces.count( face( opposite(candidate, tmesh), tmesh) ) );
-
-          uf.unify_sets( handles[hd], handles[opposite(candidate, tmesh)] );
-        }
-
-        std::size_t nb_cc = uf.number_of_sets();
-        if ( nb_cc != 1 )
-        {
-          // if more than one connected component is found then the patch
-          // made of marked faces contains "non-manifold" vertices.
-          // The smallest components need to be marked so that the patch
-          // made of marked faces is a topological disk
-
-          // we will explore in parallel the connected components and will stop
-          // when all but one connected component have been entirely explored.
-          // We add one face at a time for each cc in order to not explore a
-          // potentially very large cc.
-          std::vector< std::vector<halfedge_descriptor> > stacks_per_cc(nb_cc);
-          std::vector< std::set<face_descriptor> > faces_per_cc(nb_cc);
-          std::vector< bool > exploration_finished(nb_cc, false);
-
-          // init the stacks of halfedges using the cc of the boundary
-          std::size_t index=0;
-          std::map< halfedge_descriptor, std::size_t > ccs;
-          typedef std::pair<const halfedge_descriptor, typename UF_ds::handle> Pair_type;
-          for(Pair_type p : handles)
-          {
-            halfedge_descriptor opp_hedge = opposite(p.first, tmesh);
-            if (is_border(opp_hedge, tmesh)) continue; // nothing to do on the boundary
-
-            typedef typename std::map< halfedge_descriptor, std::size_t >::iterator Map_it;
-            std::pair<Map_it, bool> insert_res=
-              ccs.insert( std::make_pair(*uf.find( p.second ), index) );
-
-            if (insert_res.second)
-              ++index;
-
-            stacks_per_cc[ insert_res.first->second ].push_back( prev(opp_hedge, tmesh) );
-            stacks_per_cc[ insert_res.first->second ].push_back( next(opp_hedge, tmesh) );
-            faces_per_cc[ insert_res.first->second ].insert( face(opp_hedge, tmesh) );
-          }
-
-          if (index != nb_cc)
-          {
-            // most probably, one cc is a cycle of border edges
-#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
-            std::cout << "Trying to remove a component with a cycle of halfedges (nested hole or whole component), not handled yet.\n";
-#endif
-            all_removed=false;
-            continue;
-          }
-
-          std::size_t nb_ccs_to_be_explored = nb_cc;
-          index=0;
-          //explore the cc's
-          do
-          {
-            // try to extract one more face for a given cc
-            do
-            {
-              CGAL_assertion( !exploration_finished[index] );
-              CGAL_assertion( !stacks_per_cc.empty() );
-              CGAL_assertion( !stacks_per_cc[index].empty() );
-              halfedge_descriptor hd = stacks_per_cc[index].back();
-              stacks_per_cc[index].pop_back();
-              hd = opposite(hd, tmesh);
-              if ( !is_border(hd,tmesh) && !marked_faces.count(face(hd, tmesh) ) )
-              {
-                if ( faces_per_cc[index].insert( face(hd, tmesh) ).second )
-                {
-                  stacks_per_cc[index].push_back( next(hd, tmesh) );
-                  stacks_per_cc[index].push_back( prev(hd, tmesh) );
-                  break;
-                }
-              }
-              if (stacks_per_cc[index].empty()) break;
-            }
-            while(true);
-
-            // the exploration of a cc is finished when its stack is empty
-            exploration_finished[index]=stacks_per_cc[index].empty();
-            if ( exploration_finished[index] )
-              --nb_ccs_to_be_explored;
-            if ( nb_ccs_to_be_explored==1 )
-              break;
-            while ( exploration_finished[(++index)%nb_cc] );
-            index=index%nb_cc;
-          }
-          while(true);
-
-          // @todo use the area criteria? this means maybe continue exploration of larger cc
-          // mark faces of completetly explored cc
-          for (index=0; index< nb_cc; ++index)
-          {
-            if( exploration_finished[index] )
-            {
-              for(face_descriptor fd : faces_per_cc[index])
-                marked_faces.insert(fd);
-            }
-          }
-        }
-
-        // make sure the selection is a topological disk (otherwise we need another treatment)
-        if (!is_selection_a_topological_disk(marked_faces, tmesh))
-        {
-#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
-          std::cout << "Trying to handle a non-topological disk, do nothing\n";
-#endif
-          all_removed=false;
-          continue;
-        }
-
-        some_removed = true;
-        // collect simplices to be removed
-        std::set<vertex_descriptor> vertices_to_keep;
-        std::set<halfedge_descriptor> halfedges_to_keep;
-        for(halfedge_descriptor hd : border)
-        {
-          if ( !marked_faces.count(face(opposite(hd, tmesh), tmesh)) )
-          {
-            halfedges_to_keep.insert( hd );
-            vertices_to_keep.insert( target(hd, tmesh) );
-          }
-        }
-
-        // backup next,prev relationships to set after patch removal
-        std::vector< std::pair<halfedge_descriptor, halfedge_descriptor> > next_prev_halfedge_pairs;
-        halfedge_descriptor first_border_hd=*( halfedges_to_keep.begin() );
-        halfedge_descriptor current_border_hd=first_border_hd;
-        do
-        {
-          halfedge_descriptor prev_border_hd=current_border_hd;
-          current_border_hd=next(current_border_hd, tmesh);
-          while( marked_faces.count( face( opposite(current_border_hd, tmesh), tmesh) ) )
-            current_border_hd=next(opposite(current_border_hd, tmesh), tmesh);
-          next_prev_halfedge_pairs.push_back( std::make_pair(prev_border_hd, current_border_hd) );
-        }
-        while(current_border_hd!=first_border_hd);
-
-        // collect vertices and edges to remove and do remove faces
-        std::set<edge_descriptor> edges_to_remove;
-        std::set<vertex_descriptor> vertices_to_remove;
-        for(face_descriptor fd : marked_faces)
-        {
-          halfedge_descriptor hd=halfedge(fd, tmesh);
-          for(int i=0; i<3; ++i)
-          {
-            if ( !halfedges_to_keep.count(hd) )
-              edges_to_remove.insert( edge(hd, tmesh) );
-
-            if ( !vertices_to_keep.count(target(hd,tmesh)) )
-              vertices_to_remove.insert( target(hd,tmesh) );
-
-            hd=next(hd, tmesh);
-          }
-
-          remove_face(fd, tmesh);
-          face_set.erase(fd);
-        }
-
-        // remove vertices
-        for(vertex_descriptor vd : vertices_to_remove)
-          remove_vertex(vd, tmesh);
-
-        // remove edges
-        for(edge_descriptor ed : edges_to_remove)
-        {
-          degenerate_edges_to_remove.erase(ed);
-          remove_edge(ed, tmesh);
-        }
-
-        // add a new face, set all border edges pointing to it
-        // and update halfedge vertex of patch boundary vertices
-        face_descriptor new_face = add_face(tmesh);
-        typedef std::pair<halfedge_descriptor, halfedge_descriptor> Pair_type;
-        for(const Pair_type& p : next_prev_halfedge_pairs)
-        {
-          set_face(p.first, new_face, tmesh);
-          set_next(p.first, p.second, tmesh);
-          set_halfedge(target(p.first, tmesh), p.first, tmesh);
-        }
-
-        set_halfedge(new_face, first_border_hd, tmesh);
-        // triangulate the new face and update the coordinate of the central vertex
-        halfedge_descriptor new_hd=Euler::add_center_vertex(first_border_hd, tmesh);
-        put(vpmap, target(new_hd, tmesh), pt);
-
-        for(halfedge_descriptor hd : halfedges_around_target(new_hd, tmesh))
-        {
-          if(is_degenerate_edge(edge(hd, tmesh), tmesh, np))
-            degenerate_edges_to_remove.insert(edge(hd, tmesh));
-
-          if(face(hd, tmesh) != GT::null_face() && is_degenerate_triangle_face(face(hd, tmesh), tmesh))
-            face_set.insert(face(hd, tmesh));
-        }
-
-        CGAL_assertion( is_valid_polygon_mesh(tmesh) );
-      }
-    }
   }
 
-  return all_removed;
-}
-
-template <typename EdgeRange, typename TriangleMesh, typename CGAL_PMP_NP_TEMPLATE_PARAMETERS>
-bool remove_degenerate_edges(const EdgeRange& edge_range,
-                             TriangleMesh& tmesh,
-                             const CGAL_PMP_NP_CLASS& np)
-{
-  std::set<typename boost::graph_traits<TriangleMesh>::face_descriptor> face_set;
-  return remove_degenerate_edges(edge_range, tmesh, face_set, np);
-}
+  if(is_default_area_threshold)
+    area_threshold = CGAL::square(threshold_value);
 
-template <typename TriangleMesh, typename CGAL_PMP_NP_TEMPLATE_PARAMETERS>
-bool remove_degenerate_edges(TriangleMesh& tmesh,
-                             const CGAL_PMP_NP_CLASS& np)
-{
-  std::set<typename boost::graph_traits<TriangleMesh>::face_descriptor> face_set;
-  return remove_degenerate_edges(edges(tmesh), tmesh, face_set, np);
-}
-
-template <class EdgeRange, class TriangleMesh>
-bool remove_degenerate_edges(const EdgeRange& edge_range,
-                             TriangleMesh& tmesh)
-{
-  std::set<typename boost::graph_traits<TriangleMesh>::face_descriptor> face_set;
-  return remove_degenerate_edges(edge_range, tmesh, face_set, parameters::all_default());
-}
+  if(is_default_volume_threshold)
+    volume_threshold = CGAL::square(threshold_value);
 
-template <class TriangleMesh>
-bool remove_degenerate_edges(TriangleMesh& tmesh)
-{
-  std::set<typename boost::graph_traits<TriangleMesh>::face_descriptor> face_set;
-  return remove_degenerate_edges(edges(tmesh), tmesh, face_set, parameters::all_default());
-}
-
-// \ingroup PMP_repairing_grp
-// removes the degenerate faces from a triangulated surface mesh.
-// A face is considered degenerate if two of its vertices share the same location,
-// or more generally if all its vertices are collinear.
-//
-// @pre `CGAL::is_triangle_mesh(tmesh)`
-//
-// @tparam TriangleMesh a model of `FaceListGraph` and `MutableFaceGraph`
-// @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
-//
-// @param tmesh the  triangulated surface mesh to be repaired
-// @param np optional \ref pmp_namedparameters "Named Parameters" described below
-//
-// \cgalNamedParamsBegin
-//    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
-//                                      The type of this map is model of `ReadWritePropertyMap`.
-//                                      If this parameter is omitted, an internal property map for
-//                                      `CGAL::vertex_point_t` must be available in `TriangleMesh`
-//    \cgalParamEnd
-//    \cgalParamBegin{geom_traits} a geometric traits class instance.
-//       The traits class must provide the nested type `Point_3`,
-//       and the nested functors:
-//         - `Compare_distance_3` to compute the distance between 2 points
-//         - `Collinear_3` to check whether 3 points are collinear
-//         - `Less_xyz_3` to compare lexicographically two points
-//        - `Equal_3` to check whether 2 points are identical.
-//       For each functor Foo, a function `Foo foo_object()` must be provided.
-//   \cgalParamEnd
-// \cgalNamedParamsEnd
-//
-// @todo the function might not be able to remove all degenerate faces.
-//       We should probably do something with the return type.
-//
-// \return `true` if all degenerate faces were successfully removed, and `false` otherwise.
-template <typename FaceRange, typename TriangleMesh, typename NamedParameters>
-bool remove_degenerate_faces(const FaceRange& face_range,
-                             TriangleMesh& tmesh,
-                             const NamedParameters& np)
-{
-  CGAL_assertion(CGAL::is_triangle_mesh(tmesh));
+  const bool use_areas = (is_default_area_threshold || area_threshold > 0);
+  const bool use_volumes = (is_default_volume_threshold || volume_threshold > 0);
 
-  using parameters::get_parameter;
-  using parameters::choose_parameter;
+  if(!use_areas && !use_volumes)
+    return 0;
 
-  typedef TriangleMesh TM;
-  typedef typename boost::graph_traits<TriangleMesh> GT;
-  typedef typename GT::edge_descriptor edge_descriptor;
-  typedef typename GT::halfedge_descriptor halfedge_descriptor;
-  typedef typename GT::face_descriptor face_descriptor;
-  typedef typename GT::vertex_descriptor vertex_descriptor;
-
-  typedef typename GetVertexPointMap<TM, NamedParameters>::type VertexPointMap;
-  VertexPointMap vpmap = choose_parameter(get_parameter(np, internal_np::vertex_point),
-                                      get_property_map(vertex_point, tmesh));
-  typedef typename GetGeomTraits<TM, NamedParameters>::type Traits;
-  Traits traits = choose_parameter(get_parameter(np, internal_np::geom_traits), Traits());
-
-  typedef typename boost::property_traits<VertexPointMap>::value_type Point_3;
-  typedef typename boost::property_traits<VertexPointMap>::reference Point_ref;
-
-  std::set<face_descriptor> degenerate_face_set;
-  degenerate_faces(face_range, tmesh, std::inserter(degenerate_face_set, degenerate_face_set.begin()), np);
+  // Compute the connected components only once
+  boost::vector_property_map<std::size_t, FaceIndexMap> face_cc(fim);
+  std::size_t num = connected_components(tmesh, face_cc, np);
 
-  const std::size_t faces_size = faces(tmesh).size();
+#ifdef CGAL_PMP_DEBUG_SMALL_CC_REMOVAL
+  std::cout << num << " different connected components" << std::endl;
+#endif
 
-  if(degenerate_face_set.empty())
-    return true;
+  if(!dry_run)
+    CGAL::Polygon_mesh_processing::remove_isolated_vertices(tmesh);
 
-  if(degenerate_face_set.size() == faces_size)
-  {
-    clear(tmesh);
-    return true;
-  }
+  // Compute CC-wide and total areas/volumes
+  FT total_area = 0;
+  std::vector<FT> component_areas(num, 0);
 
-  // Sanitize the face range by adding adjacent degenerate faces
-  const std::size_t range_size = face_range.size();
-  bool is_range_full_mesh = (range_size == faces_size);
-  if(!is_range_full_mesh)
+  if(use_areas)
   {
-    std::list<face_descriptor> faces_to_visit(degenerate_face_set.begin(), degenerate_face_set.end());
-
-    while(!faces_to_visit.empty())
+    for(face_descriptor f : faces(tmesh))
     {
-      face_descriptor fd = faces_to_visit.front();
-      faces_to_visit.pop_front();
-
-      for(halfedge_descriptor hd : halfedges_around_face(halfedge(fd, tmesh), tmesh))
-      {
-        for(halfedge_descriptor inc_hd : halfedges_around_target(hd, tmesh))
-        {
-          face_descriptor adj_fd = face(inc_hd, tmesh);
-          if(adj_fd == GT::null_face() || adj_fd == fd)
-            continue;
-
-          if(is_degenerate_triangle_face(adj_fd, tmesh))
-          {
-            if(degenerate_face_set.insert(adj_fd).second)
-            {
-              // successful insertion means we did not know about this face before
-              faces_to_visit.push_back(adj_fd);
-            }
-          }
-        }
-      }
+      const FT fa = face_area(f, tmesh, np);
+      component_areas[face_cc[f]] += fa;
+      total_area += fa;
     }
-  }
-
-  // Note that there can't be any null edge incident to the degenerate faces range,
-  // otherwise we would have a null face incident to the face range, and that is not possible
-  // after the sanitization above
-  std::set<edge_descriptor> edge_range;
-  for(face_descriptor fd : degenerate_face_set)
-    for(halfedge_descriptor hd : halfedges_around_face(halfedge(fd, tmesh), tmesh))
-      edge_range.insert(edge(hd, tmesh));
-
-  // First remove edges of length 0
-  bool all_removed = remove_degenerate_edges(edge_range, tmesh, degenerate_face_set, np);
-
-  CGAL_assertion_code(for(face_descriptor fd : degenerate_face_set) {)
-    CGAL_assertion(is_degenerate_triangle_face(fd, tmesh));
-  CGAL_assertion_code(})
 
-#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
-  {
-  std::cout <<"Done with null edges.\n";
-  std::ofstream output("/tmp/no_null_edges.off");
-  output << std::setprecision(17) << tmesh << "\n";
-  output.close();
-  }
+#ifdef CGAL_PMP_DEBUG_SMALL_CC_REMOVAL
+  std::cout << "area threshold: " << area_threshold << std::endl;
+  std::cout << "total area: " << total_area << std::endl;
 #endif
-
-  // Then, remove triangles made of 3 collinear points
-
-  // start by filtering out border faces
-  // @todo: shall we avoid doing that in case a non-manifold vertex on the boundary or if a whole component disappear?
-  std::set<face_descriptor> border_deg_faces;
-  for(face_descriptor f : degenerate_face_set)
-  {
-    halfedge_descriptor h = halfedge(f, tmesh);
-    for (int i=0; i<3; ++i)
-    {
-      if ( is_border( opposite(h, tmesh), tmesh) )
-      {
-        border_deg_faces.insert(f);
-        break;
-      }
-      h = next(h, tmesh);
-    }
   }
 
-  while( !border_deg_faces.empty() )
-  {
-    face_descriptor f_to_rm = *border_deg_faces.begin();
-    border_deg_faces.erase(border_deg_faces.begin());
-
-    halfedge_descriptor h = halfedge(f_to_rm, tmesh);
-    for (int i=0; i<3; ++i)
-    {
-      face_descriptor f = face(opposite(h, tmesh), tmesh);
-      if ( f!=GT::null_face() )
-      {
-        if (is_degenerate_triangle_face(f, tmesh, np) )
-          border_deg_faces.insert(f);
-      }
-      h = next(h, tmesh);
-    }
-
-    while( !is_border(opposite(h, tmesh), tmesh) )
-    {
-      h = next(h, tmesh);
-    }
-
-    degenerate_face_set.erase(f_to_rm);
-    Euler::remove_face(h, tmesh);
-  }
+  // Volumes make no sense for CCs that are not closed
+  std::vector<bool> cc_closeness(num, true);
+  std::vector<FT> component_volumes(num);
 
-  // Ignore faces with null edges
-  if(!all_removed)
+  if(use_volumes)
   {
-    std::map<edge_descriptor, bool> are_degenerate_edges;
-
-    for(face_descriptor fd : degenerate_face_set)
+    for(halfedge_descriptor h : halfedges(tmesh))
     {
-      for(halfedge_descriptor hd : halfedges_around_face(halfedge(fd, tmesh), tmesh))
-      {
-        edge_descriptor ed = edge(hd, tmesh);
-        std::pair<typename std::map<edge_descriptor, bool>::iterator, bool> is_insert_successful =
-          are_degenerate_edges.insert(std::make_pair(ed, false));
-
-        bool is_degenerate = false;
-        if(is_insert_successful.second)
-        {
-          // did not previously exist in the map, so actually have to check if it is degenerate
-          if(traits.equal_3_object()(get(vpmap, target(ed, tmesh)), get(vpmap, source(ed, tmesh))))
-            is_degenerate = true;
-        }
-
-        is_insert_successful.first->second = is_degenerate;
-
-        if(is_degenerate)
-        {
-          halfedge_descriptor h = halfedge(ed, tmesh);
-          if(!is_border(h, tmesh))
-            degenerate_face_set.erase(face(h, tmesh));
-
-          h = opposite(h, tmesh);
-          if(!is_border(h, tmesh))
-            degenerate_face_set.erase(face(h, tmesh));
-        }
-      }
+      if(is_border(h, tmesh))
+        cc_closeness[face_cc[face(opposite(h, tmesh), tmesh)]] = false;
     }
-  }
 
-  // first remove degree 3 vertices that are part of a cap
-  // (only the vertex in the middle of the opposite edge)
-  // This removal does not change the shape of the mesh.
-  while (!degenerate_face_set.empty())
-  {
-    std::set<vertex_descriptor> vertices_to_remove;
-    for(face_descriptor fd : degenerate_face_set)
-    {
-      for(halfedge_descriptor hd : halfedges_around_face(halfedge(fd, tmesh), tmesh))
-      {
-        vertex_descriptor vd = target(hd, tmesh);
-        if (degree(vd, tmesh) == 3 && is_border(vd, tmesh)==GT::null_halfedge())
-        {
-          vertices_to_remove.insert(vd);
-          break;
-        }
-      }
-    }
+    typename GT::Compute_volume_3 cv3 = traits.compute_volume_3_object();
+    Point_3 origin(0, 0, 0);
 
-    for(vertex_descriptor vd : vertices_to_remove)
+    for(face_descriptor f : faces(tmesh))
     {
-      halfedge_descriptor hd=halfedge(vd, tmesh);
-      for(halfedge_descriptor hd2 : halfedges_around_target(hd, tmesh))
-        if (!is_border(hd2, tmesh))
-          degenerate_face_set.erase( face(hd2, tmesh) );
-
-      // remove the central vertex and check if the new face is degenerated
-      hd=CGAL::Euler::remove_center_vertex(hd, tmesh);
-      if (is_degenerate_triangle_face(face(hd, tmesh), tmesh, np))
-      {
-        degenerate_face_set.insert( face(hd, tmesh) );
-      }
-    }
-
-    if (vertices_to_remove.empty())
-      break;
-  }
-
-  while (!degenerate_face_set.empty())
-  {
-#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
-    std::cout << "Loop on removing deg faces\n";
-    // ensure the mesh is not broken
-    {
-      std::ofstream out("/tmp/out.off");
-      out << tmesh;
-      out.close();
-
-      std::vector<typename Traits::Point_3> points;
-      std::vector<std::vector<std::size_t> > triangles;
-      std::ifstream in("/tmp/out.off");
-      CGAL::read_OFF(in, points, triangles);
-      if (!CGAL::Polygon_mesh_processing::is_polygon_soup_a_polygon_mesh(triangles))
-      {
-        std::cerr << "Warning: got a polygon soup (may simply be a non-manifold vertex)!\n";
-      }
-    }
-#endif
-
-    face_descriptor fd = *degenerate_face_set.begin();
-
-    // look whether an incident triangle is also degenerate
-    bool detect_cc_of_degenerate_triangles = false;
-    for(halfedge_descriptor hd : halfedges_around_face(halfedge(fd, tmesh), tmesh) )
-    {
-      face_descriptor adjacent_face = face( opposite(hd, tmesh), tmesh );
-      if ( adjacent_face!=GT::null_face() &&
-           degenerate_face_set.count(adjacent_face) )
-      {
-        detect_cc_of_degenerate_triangles = true;
-        break;
-      }
-    }
-
-    if (!detect_cc_of_degenerate_triangles)
-    {
-#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
-      std::cout << "  no degenerate neighbors, using a flip.\n";
-#endif
-      degenerate_face_set.erase(degenerate_face_set.begin());
-
-      // flip the longest edge of the triangle
-      Point_ref p1 = get(vpmap, target( halfedge(fd, tmesh), tmesh) );
-      Point_ref p2 = get(vpmap, target(next(halfedge(fd, tmesh), tmesh), tmesh) );
-      Point_ref p3 = get(vpmap, source( halfedge(fd, tmesh), tmesh) );
-
-      CGAL_assertion(p1!=p2 && p1!=p3 && p2!=p3);
-
-      typename Traits::Compare_distance_3 compare_distance = traits.compare_distance_3_object();
-
-      halfedge_descriptor edge_to_flip;
-      if (compare_distance(p1,p2, p1,p3) != CGAL::SMALLER) // p1p2 > p1p3
-      {
-        if (compare_distance(p1,p2, p2,p3) != CGAL::SMALLER) // p1p2 > p2p3
-          // flip p1p2
-          edge_to_flip = next( halfedge(fd, tmesh), tmesh );
-        else
-          // flip p2p3
-          edge_to_flip = prev( halfedge(fd, tmesh), tmesh );
-      }
-      else
-        if (compare_distance(p1,p3, p2,p3) != CGAL::SMALLER) // p1p3>p2p3
-          //flip p3p1
-          edge_to_flip = halfedge(fd, tmesh);
-        else
-          //flip p2p3
-          edge_to_flip = prev( halfedge(fd, tmesh), tmesh );
-
-      face_descriptor opposite_face=face( opposite(edge_to_flip, tmesh), tmesh);
-      if ( opposite_face == GT::null_face() )
-      {
-        // simply remove the face
-        Euler::remove_face(edge_to_flip, tmesh);
-      }
-      else
-      {
-        // condition for the flip to be valid (the edge to be created do not already exists)
-        if ( !halfedge(target(next(edge_to_flip, tmesh), tmesh),
-                       target(next(opposite(edge_to_flip, tmesh), tmesh), tmesh),
-                       tmesh).second )
-        {
-          Euler::flip_edge(edge_to_flip, tmesh);
-        }
-        else
-        {
-          all_removed=false;
-#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
-          std::cout << "  WARNING: flip is not possible\n";
-          // @todo Let p and q be the vertices opposite to `edge_to_flip`, and let
-          //       r be the vertex of `edge_to_flip` that is the furthest away from
-          //       the edge `pq`. In that case I think we should remove all the triangles
-          //       so that the triangle pqr is in the mesh.
-#endif
-        }
-      }
-    }
-    else
-    {
-    // Process a connected component of degenerate faces
-      // get all the faces from the connected component
-      // and the boundary edges
-      std::set<face_descriptor> cc_faces;
-      std::vector<face_descriptor> queue;
-      std::vector<halfedge_descriptor> boundary_hedges;
-      std::vector<halfedge_descriptor> inside_hedges;
-      queue.push_back(fd);
-      cc_faces.insert(fd);
-
-      while(!queue.empty())
-      {
-        face_descriptor top=queue.back();
-        queue.pop_back();
-        for(halfedge_descriptor hd : halfedges_around_face(halfedge(top, tmesh), tmesh) )
-        {
-          face_descriptor adjacent_face = face( opposite(hd, tmesh), tmesh );
-          if ( adjacent_face==GT::null_face() ||
-               degenerate_face_set.count(adjacent_face)==0 )
-          {
-            boundary_hedges.push_back(hd);
-          }
-          else
-          {
-            if (cc_faces.insert(adjacent_face).second)
-              queue.push_back(adjacent_face);
-
-            if ( hd < opposite(hd, tmesh) )
-              inside_hedges.push_back(hd);
-          }
-        }
-      }
-
-#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
-      std::cout << "  Deal with a cc of " << cc_faces.size() << " degenerate faces.\n";
-      /// dump cc_faces
-      {
-        int id=0;
-        std::map<vertex_descriptor, int> vids;
-        for(face_descriptor f : cc_faces)
-        {
-          if ( vids.insert( std::make_pair( target(halfedge(f, tmesh), tmesh), id) ).second ) ++id;
-          if ( vids.insert( std::make_pair( target(next(halfedge(f, tmesh), tmesh), tmesh), id) ).second ) ++id;
-          if ( vids.insert( std::make_pair( target(next(next(halfedge(f, tmesh), tmesh), tmesh), tmesh), id) ).second ) ++id;
-        }
-
-        std::ofstream output("/tmp/cc_faces.off");
-        output << std::setprecision(17);
-        output << "OFF\n" << vids.size() << " " << cc_faces.size() << " 0\n";
-        std::vector<typename Traits::Point_3> points(vids.size());
-        typedef std::pair<const vertex_descriptor, int> Pair_type;
-        for(Pair_type p : vids)
-
-          points[p.second]=get(vpmap, p.first);
-        for(typename Traits::Point_3 p : points)
-          output << p << "\n";
-
-        for(face_descriptor f : cc_faces)
-        {
-          output << "3 "
-                 << vids[ target(halfedge(f, tmesh), tmesh) ] << " "
-                 << vids[ target(next(halfedge(f, tmesh), tmesh), tmesh) ] << " "
-                 << vids[ target(next(next(halfedge(f, tmesh), tmesh), tmesh), tmesh) ] << "\n";
-        }
-
-        for (std::size_t pid=2; pid!=points.size(); ++pid)
-        {
-          CGAL_assertion(collinear(points[0], points[1], points[pid]));
-        }
-      }
-#endif
-
-      // find vertices strictly inside the cc
-      std::set<vertex_descriptor> boundary_vertices;
-      for(halfedge_descriptor hd : boundary_hedges)
-        boundary_vertices.insert( target(hd, tmesh) );
-
-      std::set<vertex_descriptor> inside_vertices;
-      for(halfedge_descriptor hd : inside_hedges)
-      {
-        if (!boundary_vertices.count( target(hd, tmesh) ))
-          inside_vertices.insert( target(hd, tmesh) );
-        if (!boundary_vertices.count( source(hd, tmesh) ))
-          inside_vertices.insert( source(hd, tmesh) );
-      }
-
-      // v-e+f = 1 for a topological disk and e = (3f+#boundary_edges)/2
-      if (boundary_vertices.size()+inside_vertices.size() -
-          (cc_faces.size()+boundary_hedges.size())/2 != 1)
-      {
-        //cc_faces does not define a topological disk
-        // @todo Find to way to handle that case
-#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
-        std::cout << "  WARNING: Cannot remove the component of degenerate faces: not a topological disk.\n";
-#endif
-
-        for(face_descriptor f : cc_faces)
-          degenerate_face_set.erase(f);
-
-        continue;
-      }
-
-      // preliminary step to check if the operation is possible
-      // sort the boundary points along the common supporting line
-      //    we first need a reference point
-      typedef internal::Less_vertex_point<TriangleMesh, VertexPointMap, Traits> Less_vertex;
-      std::pair<
-        typename std::set<vertex_descriptor>::iterator,
-        typename std::set<vertex_descriptor>::iterator > ref_vertices =
-        boost::minmax_element( boundary_vertices.begin(),
-                               boundary_vertices.end(),
-                               Less_vertex(traits, vpmap) );
-
-      //    and then we sort the vertices using this reference point
-      typedef std::set<typename Traits::Point_3> Sorted_point_set;
-      Sorted_point_set sorted_points;
-      for(vertex_descriptor v : boundary_vertices)
-        sorted_points.insert( get(vpmap,v) );
-
-      CGAL_assertion(sorted_points.size()==
-                     std::set<typename Traits::Point_3>(sorted_points.begin(),
-                                                        sorted_points.end()).size());
-
-      CGAL_assertion( get( vpmap, *ref_vertices.first)==*sorted_points.begin() );
-      CGAL_assertion( get( vpmap, *ref_vertices.second)==*std::prev(sorted_points.end()) );
-
-      const typename Traits::Point_3& xtrm1 = *sorted_points.begin();
-      const typename Traits::Point_3& xtrm2 = *std::prev(sorted_points.end());
-
-      // recover halfedges on the hole, bounded by the reference vertices
-      std::vector<halfedge_descriptor> side_one, side_two;
-
-      // look for the outgoing border halfedge of the first extreme point
-      for(halfedge_descriptor hd : boundary_hedges)
-      {
-        if ( get(vpmap, source(hd, tmesh)) == xtrm1 )
-        {
-          side_one.push_back(hd);
-          break;
-        }
-      }
-      CGAL_assertion(side_one.size()==1);
-
-      bool non_monotone_border = false;
-
-      while( get(vpmap, target(side_one.back(), tmesh)) != xtrm2 )
-      {
-        vertex_descriptor prev_vertex = target(side_one.back(), tmesh);
-        for(halfedge_descriptor hd : boundary_hedges)
-        {
-          if ( source(hd, tmesh) == prev_vertex )
-          {
-            if ( get(vpmap, target(hd, tmesh)) < get(vpmap, prev_vertex) )
-              non_monotone_border = true;
-
-            side_one.push_back(hd);
-            break;
-          }
-        }
-
-        if (non_monotone_border)
-          break;
-      }
-
-      if (non_monotone_border)
-      {
-#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
-        std::cout << "  WARNING: Cannot remove the component of degenerate faces: border not a monotonic cycle.\n";
-#endif
-
-        for(face_descriptor f : cc_faces)
-          degenerate_face_set.erase(f);
-
-        continue;
-      }
-
-      // look for the outgoing border halfedge of second extreme vertex
-      for(halfedge_descriptor hd : boundary_hedges)
-      {
-        if ( source(hd, tmesh) == target(side_one.back(), tmesh) )
-        {
-          side_two.push_back(hd);
-          break;
-        }
-      }
-      CGAL_assertion(side_two.size()==1);
-
-      while( target(side_two.back(), tmesh) != source(side_one.front(), tmesh) )
-      {
-        vertex_descriptor prev_vertex = target(side_two.back(), tmesh);
-        for(halfedge_descriptor hd : boundary_hedges)
-        {
-          if ( source(hd, tmesh) == prev_vertex )
-          {
-            if ( get(vpmap, target(hd, tmesh)) > get(vpmap, prev_vertex) )
-              non_monotone_border = true;
-
-            side_two.push_back(hd);
-            break;
-          }
-        }
-
-        if (non_monotone_border)
-          break;
-      }
-
-      if (non_monotone_border)
-      {
-#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
-        std::cout << "  WARNING: Cannot remove the component of degenerate faces: border not a monotonic cycle.\n";
-#endif
-
-        for(face_descriptor f : cc_faces)
-          degenerate_face_set.erase(f);
-
+      const std::size_t i = face_cc[f];
+      if(!cc_closeness[i])
         continue;
-      }
-
-      CGAL_assertion( side_one.size()+side_two.size()==boundary_hedges.size() );
-
-      // reverse the order of the second side so as to follow
-      // the same order than side one
-      std::reverse(side_two.begin(), side_two.end());
-      for(halfedge_descriptor& h : side_two)
-        h=opposite(h, tmesh);
-
-      //make sure the points of the vertices along side_one are correctly sorted
-      std::vector<Point_3> side_points;
-      side_points.reserve(side_one.size()+1);
-      side_points.push_back(get(vpmap,source(side_one.front(), tmesh)));
-
-      for(halfedge_descriptor h : side_one)
-        side_points.push_back(get(vpmap,target(h, tmesh)));
-
-      CGAL_assertion(get(vpmap,source(side_one.front(), tmesh))==side_points.front());
-      CGAL_assertion(get(vpmap,target(side_one.back(), tmesh))==side_points.back());
-
-      // @todo the reordering could lead to the apparition of null edges.
-      std::sort(side_points.begin(), side_points.end());
-
-      CGAL_assertion(std::unique(side_points.begin(), side_points.end())==side_points.end());
-      for(std::size_t i=0;i<side_one.size()-1;++i)
-        put(vpmap,target(side_one[i], tmesh), side_points[i+1]);
-
-      //same thing for side_two
-      side_points.clear();
-      side_points.reserve(side_two.size()+1);
-      side_points.push_back(get(vpmap,source(side_two.front(), tmesh)));
-
-      for(halfedge_descriptor h : side_two)
-        side_points.push_back(get(vpmap,target(h, tmesh)));
-
-      CGAL_assertion(get(vpmap,source(side_two.front(), tmesh))==side_points.front());
-      CGAL_assertion(get(vpmap,target(side_two.back(), tmesh))==side_points.back());
-
-      // @todo the reordering could lead to the apparition of null edges.
-      std::sort(side_points.begin(), side_points.end());
-
-      CGAL_assertion(std::unique(side_points.begin(), side_points.end())==side_points.end());
-      for(std::size_t i=0;i<side_two.size()-1;++i)
-        put(vpmap,target(side_two[i], tmesh), side_points[i+1]);
-
-      CGAL_assertion( source(side_one.front(), tmesh) == *ref_vertices.first );
-      CGAL_assertion( source(side_two.front(), tmesh) == *ref_vertices.first );
-      CGAL_assertion( target(side_one.back(), tmesh) == *ref_vertices.second );
-      CGAL_assertion( target(side_two.back(), tmesh) == *ref_vertices.second );
-
-      typename Sorted_point_set::iterator it_pt = std::next(sorted_points.begin()),
-                                          it_pt_end = std::prev(sorted_points.end());
-
-      bool non_collapsable = false;
-      typename std::vector<halfedge_descriptor>::iterator side_one_it = side_one.begin();
-      typename std::vector<halfedge_descriptor>::iterator side_two_it = side_two.begin();
-      for(;it_pt!=it_pt_end;++it_pt)
-      {
-        // check if it_pt is the point of the target of one or two halfedges
-        bool target_of_side_one = get(vpmap, target(*side_one_it, tmesh))==*it_pt;
-        bool target_of_side_two = get(vpmap, target(*side_two_it, tmesh))==*it_pt;
-
-        if (target_of_side_one && target_of_side_two)
-        {
-          for(halfedge_descriptor h : halfedges_around_target(*side_one_it, tmesh))
-          {
-            if (source(h, tmesh)==target(*side_two_it, tmesh))
-            {
-              non_collapsable=true;
-              break;
-            }
-          }
-        }
-        else
-        {
-          CGAL_assertion(target_of_side_one || target_of_side_two);
-          vertex_descriptor v1 = target_of_side_one ? target(*side_one_it, tmesh)
-                                                    : target(*side_two_it, tmesh);
-          vertex_descriptor v2 = target_of_side_two ? target(next(opposite(*side_one_it, tmesh), tmesh), tmesh)
-                                                    : target(next(*side_two_it, tmesh), tmesh);
-          for(halfedge_descriptor h : halfedges_around_target(v1, tmesh))
-          {
-            if (source(h, tmesh)==v2)
-            {
-              non_collapsable=true;
-              break;
-            }
-          }
-        }
-
-        if(non_collapsable) break;
-        if (target_of_side_one) ++side_one_it;
-        if (target_of_side_two) ++side_two_it;
-      }
-
-      if (non_collapsable)
-      {
-        for(face_descriptor f : cc_faces)
-          degenerate_face_set.erase(f);
-
-#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
-        std::cout << "  WARNING: cannot remove a connected components of degenerate faces.\n";
-#endif
-        continue;
-      }
-
-    // now proceed to the fix
-      // update the face and halfedge vertex pointers on the boundary
-      for(halfedge_descriptor h : boundary_hedges)
-      {
-        set_face(h, GT::null_face(), tmesh);
-        set_halfedge(target(h,tmesh), h, tmesh);
-      }
-
-      // update next/prev pointers of boundary_hedges
-      for(halfedge_descriptor h : boundary_hedges)
-      {
-        halfedge_descriptor next_candidate = next( h, tmesh);
-        while (face(next_candidate, tmesh)!=GT::null_face())
-          next_candidate = next( opposite( next_candidate, tmesh), tmesh);
-
-        set_next(h, next_candidate, tmesh);
-      }
-
-      // remove degenerate faces
-      for(face_descriptor f : cc_faces)
-      {
-        degenerate_face_set.erase(f);
-        remove_face(f, tmesh);
-      }
-
-      // remove interior edges
-      for(halfedge_descriptor h : inside_hedges)
-        remove_edge(edge(h, tmesh), tmesh);
-
-      // remove interior vertices
-      for(vertex_descriptor v : inside_vertices)
-        remove_vertex(v, tmesh);
-
-#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
-      std::cout << "  side_one.size() " << side_one.size() << "\n";
-      std::cout << "  side_two.size() " << side_two.size() << "\n";
-#endif
-
-      CGAL_assertion( source(side_one.front(), tmesh) == *ref_vertices.first );
-      CGAL_assertion( source(side_two.front(), tmesh) == *ref_vertices.first );
-      CGAL_assertion( target(side_one.back(), tmesh) == *ref_vertices.second );
-      CGAL_assertion( target(side_two.back(), tmesh) == *ref_vertices.second );
-
-      // now split each side to contains the same sequence of points
-      //    first side
-      int hi=0;
-      for (typename Sorted_point_set::iterator it=std::next(sorted_points.begin()),
-                                               it_end=sorted_points.end(); it!=it_end; ++it)
-      {
-        CGAL_assertion( *std::prev(it) == get(vpmap, source(side_one[hi], tmesh) ) );
-        if( *it != get(vpmap, target(side_one[hi], tmesh) ) )
-        {
-          // split the edge and update the point
-          halfedge_descriptor h1 = next(opposite(side_one[hi], tmesh), tmesh);
-          put(vpmap,
-              target(Euler::split_edge(side_one[hi], tmesh), tmesh),
-              *it);
-
-          // split_edge updates the halfedge of the source vertex of h,
-          // since we reuse later the halfedge of the first refernce vertex
-          // we must set it as we need.
-          if ( source(h1,tmesh) == *ref_vertices.first)
-            set_halfedge(*ref_vertices.first, prev( prev(side_one[hi], tmesh), tmesh), tmesh );
-
-          // retriangulate the opposite face
-          if ( face(h1, tmesh) != GT::null_face())
-            Euler::split_face(h1, opposite(side_one[hi], tmesh), tmesh);
-        }
-        else
-        {
-          ++hi;
-        }
-      }
-
-      //    second side
-      hi=0;
-      for (typename Sorted_point_set::iterator it=std::next(sorted_points.begin()),
-                                               it_end=sorted_points.end(); it!=it_end; ++it)
-      {
-        CGAL_assertion( *std::prev(it) == get(vpmap, source(side_two[hi], tmesh) ) );
-        if( *it != get(vpmap, target(side_two[hi], tmesh) ) )
-        {
-          // split the edge and update the point
-          halfedge_descriptor h2 = Euler::split_edge(side_two[hi], tmesh);
-          put(vpmap, target(h2, tmesh), *it);
-
-          // split_edge updates the halfedge of the source vertex of h,
-          // since we reuse later the halfedge of the first refernce vertex
-          // we must set it as we need.
-          if ( source(h2,tmesh) == *ref_vertices.first)
-            set_halfedge(*ref_vertices.first, opposite( h2, tmesh), tmesh );
-
-          // retriangulate the face
-          if ( face(h2, tmesh) != GT::null_face())
-            Euler::split_face(h2, next(side_two[hi], tmesh), tmesh);
-        }
-        else
-        {
-          ++hi;
-        }
-      }
-
-      CGAL_assertion( target(halfedge(*ref_vertices.first, tmesh), tmesh) == *ref_vertices.first );
-      CGAL_assertion( face(halfedge(*ref_vertices.first, tmesh), tmesh) == GT::null_face() );
-
-#ifdef CGAL_PMP_REMOVE_DEGENERATE_FACES_DEBUG
-      {
-        halfedge_descriptor h_side2 = halfedge(*ref_vertices.first, tmesh);
-        halfedge_descriptor h_side1 = next(h_side2, tmesh);
-
-        do
-        {
-          CGAL_assertion( get(vpmap, source(h_side1, tmesh)) == get(vpmap, target(h_side2, tmesh)) );
-          CGAL_assertion( get(vpmap, target(h_side1, tmesh)) == get(vpmap, source(h_side2, tmesh)) );
-
-          if ( target(next(opposite(h_side1, tmesh), tmesh), tmesh) ==
-               target(next(opposite(h_side2, tmesh), tmesh), tmesh) )
-          {
-            CGAL_error_msg("Forbidden simplification");
-          }
-
-          h_side2 = prev(h_side2, tmesh);
-          h_side1 = next(h_side1, tmesh);
-        }
-        while( target(h_side1, tmesh) != *ref_vertices.second );
-      }
-#endif
-
-      // remove side1 and replace its opposite hedges by those of side2
-      halfedge_descriptor h_side2 = halfedge(*ref_vertices.first, tmesh);
-      halfedge_descriptor h_side1 = next(h_side2, tmesh);
-      while(true)
-      {
-        CGAL_assertion( get(vpmap, source(h_side1, tmesh)) == get(vpmap, target(h_side2, tmesh)) );
-        CGAL_assertion( get(vpmap, target(h_side1, tmesh)) == get(vpmap, source(h_side2, tmesh)) );
-
-        // backup target vertex
-        vertex_descriptor vertex_to_remove = target(h_side1, tmesh);
-        if (vertex_to_remove!=*ref_vertices.second)
-        {
-          vertex_descriptor replacement_vertex = source(h_side2, tmesh);
-          // replace the incident vertex
-          for(halfedge_descriptor hd : halfedges_around_target(h_side1, tmesh))
-            set_target(hd, replacement_vertex, tmesh);
-        }
-
-        // prev side2 hedge for next loop
-        halfedge_descriptor h_side2_for_next_turn = prev(h_side2, tmesh);
-
-        // replace the opposite of h_side1 by h_side2
-        halfedge_descriptor opposite_h_side1 = opposite( h_side1, tmesh);
-        face_descriptor the_face = face(opposite_h_side1, tmesh);
-        set_face(h_side2, the_face, tmesh);
-
-        if (the_face!=GT::null_face())
-          set_halfedge(the_face, h_side2, tmesh);
-
-        set_next(h_side2, next(opposite_h_side1, tmesh), tmesh);
-        set_next(prev(opposite_h_side1, tmesh), h_side2, tmesh);
-
-        // take the next hedges
-        edge_descriptor edge_to_remove = edge(h_side1, tmesh);
-        h_side1 = next(h_side1, tmesh);
-
-        // now remove the extra edge
-        remove_edge(edge_to_remove, tmesh);
-
-        // ... and the extra vertex if it's not the second reference
-        if (vertex_to_remove==*ref_vertices.second)
-        {
-          // update the halfedge pointer of the last vertex (others were already from side 2)
-          CGAL_assertion( target(opposite(h_side2, tmesh), tmesh) == vertex_to_remove );
-          set_halfedge(vertex_to_remove, opposite(h_side2, tmesh), tmesh);
-          break;
-        }
-        else
-        {
-          remove_vertex(vertex_to_remove , tmesh);
-        }
-
-        h_side2 = h_side2_for_next_turn;
-      }
-    }
-  }
-
-  return all_removed;
-}
-
-template <typename FaceRange, typename TriangleMesh>
-bool remove_degenerate_faces(const FaceRange& face_range,
-                             TriangleMesh& tmesh)
-{
-  return remove_degenerate_faces(face_range, tmesh, parameters::all_default());
-}
-
-template <typename TriangleMesh, typename CGAL_PMP_NP_TEMPLATE_PARAMETERS>
-bool remove_degenerate_faces(TriangleMesh& tmesh,
-                             const CGAL_PMP_NP_CLASS& np)
-{
-  return remove_degenerate_faces(faces(tmesh), tmesh, np);
-}
-
-template<class TriangleMesh>
-bool remove_degenerate_faces(TriangleMesh& tmesh)
-{
-  return remove_degenerate_faces(tmesh,
-    CGAL::Polygon_mesh_processing::parameters::all_default());
-}
-
-/// \ingroup PMP_repairing_grp
-/// checks whether a vertex of a polygon mesh is non-manifold.
-///
-/// @tparam PolygonMesh a model of `HalfedgeListGraph`
-///
-/// @param v a vertex of `pm`
-/// @param pm a triangle mesh containing `v`
-///
-/// \warning This function has linear runtime with respect to the size of the mesh.
-///
-/// \sa `duplicate_non_manifold_vertices()`
-///
-/// \return `true` if the vertex is non-manifold, `false` otherwise.
-template <typename PolygonMesh>
-bool is_non_manifold_vertex(typename boost::graph_traits<PolygonMesh>::vertex_descriptor v,
-                            const PolygonMesh& pm)
-{
-  typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor halfedge_descriptor;
-
-  typedef CGAL::dynamic_halfedge_property_t<bool>                                       Halfedge_property_tag;
-  typedef typename boost::property_map<PolygonMesh, Halfedge_property_tag>::const_type  Visited_halfedge_map;
-
-  // Dynamic pmaps do not have default initialization values (yet)
-  Visited_halfedge_map visited_halfedges = get(Halfedge_property_tag(), pm);
-  for(halfedge_descriptor h : halfedges(pm))
-    put(visited_halfedges, h, false);
-
-  std::size_t incident_null_faces_counter = 0;
-  for(halfedge_descriptor h : halfedges_around_target(v, pm))
-  {
-    put(visited_halfedges, h, true);
-    if(CGAL::is_border(h, pm))
-      ++incident_null_faces_counter;
-  }
-
-  if(incident_null_faces_counter > 1)
-  {
-    // The vertex is the sole connection between two connected components --> non-manifold
-    return true;
-  }
-
-  for(halfedge_descriptor h : halfedges(pm))
-  {
-    if(v == target(h, pm))
-    {
-      // Haven't seen that halfedge yet ==> more than one umbrella incident to 'v' ==> non-manifold
-      if(!get(visited_halfedges, h))
-        return true;
-    }
-  }
-
-  return false;
-}
-
-namespace internal {
-
-template <typename G>
-struct Vertex_collector
-{
-  typedef typename boost::graph_traits<G>::vertex_descriptor      vertex_descriptor;
-
-  bool has_old_vertex(const vertex_descriptor v) const { return collections.count(v) != 0; }
-  void tag_old_vertex(const vertex_descriptor v)
-  {
-    CGAL_precondition(!has_old_vertex(v));
-    collections[v];
-  }
-
-  void collect_vertices(vertex_descriptor v1, vertex_descriptor v2)
-  {
-    std::vector<vertex_descriptor>& verts = collections[v1];
-    if(verts.empty())
-      verts.push_back(v1);
-    verts.push_back(v2);
-  }
-
-  template<typename OutputIterator>
-  void dump(OutputIterator out)
-  {
-    typedef std::pair<const vertex_descriptor, std::vector<vertex_descriptor> > Pair_type;
-    for(const Pair_type& p : collections)
-      *out++ = p.second;
-  }
-
-  void dump(Emptyset_iterator) { }
-
-  std::map<vertex_descriptor, std::vector<vertex_descriptor> > collections;
-};
-
-template <typename PolygonMesh, typename VPM, typename ConstraintMap>
-typename boost::graph_traits<PolygonMesh>::vertex_descriptor
-create_new_vertex_for_sector(typename boost::graph_traits<PolygonMesh>::halfedge_descriptor sector_begin_h,
-                             typename boost::graph_traits<PolygonMesh>::halfedge_descriptor sector_last_h,
-                             PolygonMesh& pm,
-                             const VPM& vpm,
-                             const ConstraintMap& cmap)
-{
-  typedef typename boost::graph_traits<PolygonMesh>::vertex_descriptor    vertex_descriptor;
-  typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor  halfedge_descriptor;
-
-  vertex_descriptor old_vd = target(sector_begin_h, pm);
-  vertex_descriptor new_vd = add_vertex(pm);
-  put(vpm, new_vd, get(vpm, old_vd));
-
-  put(cmap, new_vd, true);
-
-  set_halfedge(new_vd, sector_begin_h, pm);
-  halfedge_descriptor h = sector_begin_h;
-  do
-  {
-    set_target(h, new_vd, pm);
 
-    if(h == sector_last_h)
-      break;
-    else
-      h = prev(opposite(h, pm), pm);
-  }
-  while(h != sector_begin_h); // for safety
-  CGAL_assertion(h != sector_begin_h);
+      const FT fv = cv3(origin,
+                        get(vpm, target(halfedge(f, tmesh), tmesh)),
+                        get(vpm, target(next(halfedge(f, tmesh), tmesh), tmesh)),
+                        get(vpm, target(prev(halfedge(f, tmesh), tmesh), tmesh)));
 
-  return new_vd;
-}
-
-template <typename PolygonMesh, typename NamedParameters>
-std::size_t make_umbrella_manifold(typename boost::graph_traits<PolygonMesh>::halfedge_descriptor h,
-                                   PolygonMesh& pm,
-                                   internal::Vertex_collector<PolygonMesh>& dmap,
-                                   const NamedParameters& np)
-{
-  typedef typename boost::graph_traits<PolygonMesh>::vertex_descriptor    vertex_descriptor;
-  typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor  halfedge_descriptor;
-
-  using parameters::get_parameter;
-  using parameters::choose_parameter;
-
-  typedef typename GetVertexPointMap<PolygonMesh, NamedParameters>::type VertexPointMap;
-  VertexPointMap vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
-                                    get_property_map(vertex_point, pm));
-
-  typedef typename internal_np::Lookup_named_param_def<internal_np::vertex_is_constrained_t,
-                                                 NamedParameters,
-                                                 Constant_property_map<vertex_descriptor, bool> // default (no constraint pmap)
-                                                 >::type                  VerticesMap;
-  VerticesMap cmap = choose_parameter(get_parameter(np, internal_np::vertex_is_constrained),
-                                  Constant_property_map<vertex_descriptor, bool>(false));
-
-  std::size_t nb_new_vertices = 0;
-
-  vertex_descriptor old_v = target(h, pm);
-  put(cmap, old_v, true); // store the duplicates
-
-  // count the number of borders
-  int border_counter = 0;
-  halfedge_descriptor ih = h, done = ih, border_h = h;
-  do
-  {
-    if(is_border(ih, pm))
-    {
-      border_h = ih;
-      ++border_counter;
+      component_volumes[i] += fv;
     }
 
-    ih = prev(opposite(ih, pm), pm);
-  }
-  while(ih != done);
-
-  const bool is_non_manifold_within_umbrella = (border_counter > 1);
-  if(!is_non_manifold_within_umbrella)
-  {
-    const bool first_time_meeting_v = !dmap.has_old_vertex(old_v);
-    if(first_time_meeting_v)
+    // negative volume means the CC was oriented inward
+    FT total_volume = 0;
+    for(std::size_t i=0; i<num; ++i)
     {
-      // The star is manifold, so if it is the first time we have met that vertex,
-      // there is nothing to do, we just keep the same vertex.
-      set_halfedge(old_v, h, pm); // to ensure halfedge(old_v, pm) stays valid
-      dmap.tag_old_vertex(old_v); // so that we know we have met old_v already, next time, we'll have to duplicate
+      component_volumes[i] = CGAL::abs(component_volumes[i]);
+      total_volume += component_volumes[i];
     }
-    else
-    {
-      // This is not the canonical star associated to 'v'.
-      // Create a new vertex, and move the whole star to that new vertex
-      halfedge_descriptor last_h = opposite(next(h, pm), pm);
-      vertex_descriptor new_v = create_new_vertex_for_sector(h, last_h, pm, vpm, cmap);
-      dmap.collect_vertices(old_v, new_v);
-      nb_new_vertices = 1;
-    }
-  }
-  // if there is more than one sector, look at each sector and split them away from the main one
-  else
-  {
-    // the first manifold sector, described by two halfedges
-    halfedge_descriptor sector_start_h = border_h;
-    CGAL_assertion(is_border(border_h, pm));
-
-    bool should_stop = false;
-    bool is_main_sector = true;
-    do
-    {
-      CGAL_assertion(is_border(sector_start_h, pm));
 
-      // collect the sector and split it away if it must be
-      halfedge_descriptor sector_last_h = sector_start_h;
-      do
-      {
-        halfedge_descriptor next_h = prev(opposite(sector_last_h, pm), pm);
-
-        if(is_border(next_h, pm))
-          break;
-
-        sector_last_h = next_h;
-      }
-      while(sector_last_h != sector_start_h);
-      CGAL_assertion(!is_border(sector_last_h, pm));
-      CGAL_assertion(sector_last_h != sector_start_h);
-
-      halfedge_descriptor next_start_h = prev(opposite(sector_last_h, pm), pm);
-
-      // there are multiple CCs incident to this particular vertex, and we should create a new vertex
-      // if it's not the first umbrella around 'old_v' or not the first sector, but only not if it's
-      // both the first umbrella and first sector.
-      const bool must_create_new_vertex = (!is_main_sector || dmap.has_old_vertex(old_v));
-
-      // In any case, we must set up the next pointer correctly
-      set_next(sector_start_h, opposite(sector_last_h, pm), pm);
-
-      if(must_create_new_vertex)
-      {
-        vertex_descriptor new_v = create_new_vertex_for_sector(sector_start_h, sector_last_h, pm, vpm, cmap);
-        dmap.collect_vertices(old_v, new_v);
-        ++nb_new_vertices;
-      }
-      else
-      {
-        // Ensure that halfedge(old_v, pm) stays valid
-        set_halfedge(old_v, sector_start_h, pm);
-      }
-
-      is_main_sector = false;
-      sector_start_h = next_start_h;
-      should_stop = (sector_start_h == border_h);
-    }
-    while(!should_stop);
+#ifdef CGAL_PMP_DEBUG_SMALL_CC_REMOVAL
+    std::cout << "volume threshold: " << volume_threshold << std::endl;
+    std::cout << "total volume: " << total_volume << std::endl;
+#endif
   }
 
-  return nb_new_vertices;
-}
-
-} // end namespace internal
+  std::size_t res = 0;
+  std::vector<bool> is_to_be_removed(num, false);
 
-/// \ingroup PMP_repairing_grp
-/// collects the non-manifold vertices (if any) present in the mesh. A non-manifold vertex `v` is returned
-/// via one incident halfedge `h` such that `target(h, pm) = v` for all the umbrellas that `v` apppears in
-/// (an <i>umbrella</i> being the set of faces incident to all the halfedges reachable by walking around `v`
-/// using `hnext = prev(opposite(h, pm), pm)`, starting from `h`).
-///
-/// @tparam PolygonMesh a model of `HalfedgeListGraph`
-/// @tparam OutputIterator a model of `OutputIterator` holding objects of type
-///                         `boost::graph_traits<PolygonMesh>::%halfedge_descriptor`
-///
-/// @param pm a triangle mesh
-/// @param out the output iterator that collects halfedges incident to `v`
-///
-/// \sa `is_non_manifold_vertex()`
-/// \sa `duplicate_non_manifold_vertices()`
-///
-/// \return the output iterator.
-template <typename PolygonMesh, typename OutputIterator>
-OutputIterator non_manifold_vertices(const PolygonMesh& pm,
-                                     OutputIterator out)
-{
-  // Non-manifoldness can appear either:
-  // - if 'pm' is pinched at a vertex. While traversing the incoming halfedges at this vertex,
-  //   we will meet strictly more than one border halfedge.
-  // - if there are multiple umbrellas around a vertex. In that case, we will find a non-visited
-  //   halfedge that has for target a vertex that is already visited.
-
-  typedef typename boost::graph_traits<PolygonMesh>::vertex_descriptor                  vertex_descriptor;
-  typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor                halfedge_descriptor;
-
-  typedef CGAL::dynamic_vertex_property_t<bool>                                         Vertex_bool_tag;
-  typedef typename boost::property_map<PolygonMesh, Vertex_bool_tag>::const_type        Known_manifold_vertex_map;
-  typedef CGAL::dynamic_vertex_property_t<halfedge_descriptor>                          Vertex_halfedge_tag;
-  typedef typename boost::property_map<PolygonMesh, Vertex_halfedge_tag>::const_type    Visited_vertex_map;
-  typedef CGAL::dynamic_halfedge_property_t<bool>                                       Halfedge_property_tag;
-  typedef typename boost::property_map<PolygonMesh, Halfedge_property_tag>::const_type  Visited_halfedge_map;
-
-  Known_manifold_vertex_map known_nm_vertices = get(Vertex_bool_tag(), pm);
-  Visited_vertex_map visited_vertices = get(Vertex_halfedge_tag(), pm);
-  Visited_halfedge_map visited_halfedges = get(Halfedge_property_tag(), pm);
-
-  const halfedge_descriptor null_h = boost::graph_traits<PolygonMesh>::null_halfedge();
-
-  // Dynamic pmaps do not have default initialization values (yet)
-  for(vertex_descriptor v : vertices(pm))
+  for(std::size_t i=0; i<num; ++i)
   {
-    put(known_nm_vertices, v, false);
-    put(visited_vertices, v, null_h);
-  }
-  for(halfedge_descriptor h : halfedges(pm))
-    put(visited_halfedges, h, false);
+#ifdef CGAL_PMP_DEBUG_SMALL_CC_REMOVAL
+    std::cout << "CC " << i << " has area: " << component_areas[i]
+              << " and volume: " << component_volumes[i] << std::endl;
+#endif
 
-  for(halfedge_descriptor h : halfedges(pm))
-  {
-    // If 'h' is not visited yet, we walk around the target of 'h' and mark these
-    // halfedges as visited. Thus, if we are here and the target is already marked as visited,
-    // it means that the vertex is non manifold.
-    if(!get(visited_halfedges, h))
+    if((use_volumes && cc_closeness[i] && component_volumes[i] <= volume_threshold) ||
+       (use_areas && component_areas[i] <= area_threshold))
     {
-      put(visited_halfedges, h, true);
-      bool is_non_manifold = false;
-
-      vertex_descriptor v = target(h, pm);
-      if(get(visited_vertices, v) != null_h) // already seen this vertex, but not from this star
-      {
-        is_non_manifold = true;
-        // if this is the second time we visit that vertex and the first star was manifold, we have
-        // never reported the first star, but we must now
-        if(!get(known_nm_vertices, v))
-          *out++ = get(visited_vertices, v); // that's a halfedge of the first star we've seen 'v' in
-      }
-      else
-      {
-        // first time we meet this vertex, just mark it so, and keep the halfedge we found the vertex with in memory
-        put(visited_vertices, v, h);
-      }
-
-      // While walking the star of this halfedge, if we meet a border halfedge more than once,
-      // it means the mesh is pinched and we are also in the case of a non-manifold situation
-      halfedge_descriptor ih = h, done = ih;
-      int border_counter = 0;
-      do
-      {
-        put(visited_halfedges, ih, true);
-        if(is_border(ih, pm))
-          ++border_counter;
-
-        ih = prev(opposite(ih, pm), pm);
-      }
-      while(ih != done);
-
-      if(border_counter > 1)
-        is_non_manifold = true;
-
-      if(is_non_manifold)
-      {
-        *out++ = h;
-        put(known_nm_vertices, v, true);
-      }
+      is_to_be_removed[i] = true;
+      ++res;
     }
   }
 
-  return out;
-}
-
-/// \ingroup PMP_repairing_grp
-/// duplicates all the non-manifold vertices of the input mesh.
-///
-/// @tparam PolygonMesh a model of `HalfedgeListGraph` and `MutableHalfedgeGraph`
-/// @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
-///
-/// @param pm the surface mesh to be repaired
-/// @param np optional \ref pmp_namedparameters "Named Parameters" described below
-///
-/// \cgalNamedParamsBegin
-///    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
-///       The type of this map is model of `ReadWritePropertyMap`.
-///       If this parameter is omitted, an internal property map for
-///       `CGAL::vertex_point_t` should be available in `PolygonMesh`
-///    \cgalParamEnd
-///   \cgalParamBegin{vertex_is_constrained_map} a writable property map with `vertex_descriptor`
-///     as key and `bool` as `value_type`. `put(pmap, v, true)` will be called for each duplicated
-///     vertices, as well as the original non-manifold vertex in the input mesh.
-///  \cgalParamEnd
-///   \cgalParamBegin{output_iterator} a model of `OutputIterator` with value type
-///      `std::vector<vertex_descriptor>`. The first vertex of each vector is a non-manifold vertex
-///       of the input mesh, followed by the new vertices that were created to fix this precise
-///       non-manifold configuration.
-///  \cgalParamEnd
-/// \cgalNamedParamsEnd
-///
-/// \return the number of vertices created.
-template <typename PolygonMesh, typename NamedParameters>
-std::size_t duplicate_non_manifold_vertices(PolygonMesh& pm,
-                                            const NamedParameters& np)
-{
-  using parameters::get_parameter;
-  using parameters::choose_parameter;
-
-  typedef boost::graph_traits<PolygonMesh>                            GT;
-  typedef typename GT::halfedge_descriptor                            halfedge_descriptor;
-
-  typedef typename internal_np::Lookup_named_param_def <
-    internal_np::output_iterator_t,
-    NamedParameters,
-    Emptyset_iterator
-  > ::type                                                            Output_iterator;
-
-  Output_iterator out = choose_parameter(get_parameter(np, internal_np::output_iterator), Emptyset_iterator());
-
-  std::vector<halfedge_descriptor> non_manifold_cones;
-  non_manifold_vertices(pm, std::back_inserter(non_manifold_cones));
-
-  internal::Vertex_collector<PolygonMesh> dmap;
-  std::size_t nb_new_vertices = 0;
-  if(!non_manifold_cones.empty())
-  {
-    for(halfedge_descriptor h : non_manifold_cones)
-      nb_new_vertices += internal::make_umbrella_manifold(h, pm, dmap, np);
-
-    dmap.dump(out);
-  }
-
-  return nb_new_vertices;
-}
-
-template <class PolygonMesh>
-std::size_t duplicate_non_manifold_vertices(PolygonMesh& pm)
-{
-  return duplicate_non_manifold_vertices(pm, parameters::all_default());
-}
-
-/// \ingroup PMP_repairing_grp
-/// removes the isolated vertices from any polygon mesh.
-/// A vertex is considered isolated if it is not incident to any simplex
-/// of higher dimension.
-///
-/// @tparam PolygonMesh a model of `FaceListGraph` and `MutableFaceGraph`
-///
-/// @param pmesh the polygon mesh to be repaired
-///
-/// @return number of removed isolated vertices
-///
-template <class PolygonMesh>
-std::size_t remove_isolated_vertices(PolygonMesh& pmesh)
-{
-  typedef typename boost::graph_traits<PolygonMesh>::vertex_descriptor vertex_descriptor;
-  std::vector<vertex_descriptor> to_be_removed;
-
-  for(vertex_descriptor v : vertices(pmesh))
-  {
-    if (CGAL::halfedges_around_target(v, pmesh).first
-      == CGAL::halfedges_around_target(v, pmesh).second)
-      to_be_removed.push_back(v);
-  }
-  std::size_t nb_removed = to_be_removed.size();
-  for(vertex_descriptor v : to_be_removed)
-  {
-    remove_vertex(v, pmesh);
-  }
-  return nb_removed;
-}
-
-/// \cond SKIP_IN_MANUAL
-template <class TriangleMesh,  class face_descriptor, class VertexPointMap>
-std::pair< bool, bool >
-remove_self_intersections_one_step(TriangleMesh& tm,
-                                   std::set<face_descriptor>& faces_to_remove,
-                                   VertexPointMap& vpmap,
-                                   int step,
-                                   bool preserve_genus,
-                                   bool verbose)
-{
-  std::set<face_descriptor> faces_to_remove_copy = faces_to_remove;
-
-  if (verbose)
-    std::cout << "DEBUG: running remove_self_intersections_one_step, step " << step
-              << " with " << faces_to_remove.size() << " intersecting faces\n";
-
-  CGAL_assertion(tm.is_valid());
-
-  typedef boost::graph_traits<TriangleMesh> graph_traits;
-  typedef typename graph_traits::vertex_descriptor vertex_descriptor;
-  typedef typename graph_traits::edge_descriptor edge_descriptor;
-  typedef typename graph_traits::halfedge_descriptor halfedge_descriptor;
-
-  bool something_was_done = false; // indicates if a region was successfully remeshed
-  bool all_fixed = true; // indicates if all removal went well
-  // indicates if a removal was not possible because the region handle has
-  // some boundary cycle of halfedges
-  bool topology_issue = false;
-
-  if (verbose)
-  {
-    std::cout << "  DEBUG: is_valid in one_step(tm)? ";
-    std::cout.flush();
-    std::cout << is_valid_polygon_mesh(tm) << "\n";
-  }
-
-  if(!faces_to_remove.empty()){
-
-    while(!faces_to_remove.empty())
-    {
-      // Process a connected component of faces to remove.
-      // collect all the faces from the connected component
-      std::set<face_descriptor> cc_faces;
-      std::vector<face_descriptor> queue(1, *faces_to_remove.begin()); // temporary queue
-      cc_faces.insert(queue.back());
-      while(!queue.empty())
-      {
-        face_descriptor top=queue.back();
-        queue.pop_back();
-        halfedge_descriptor h = halfedge(top,tm);
-        for (int i=0;i<3; ++i)
-        {
-          face_descriptor adjacent_face = face( opposite(h, tm), tm );
-          if ( adjacent_face!=boost::graph_traits<TriangleMesh>::null_face())
-          {
-            if (faces_to_remove.count(adjacent_face) != 0 &&
-                cc_faces.insert(adjacent_face).second)
-              queue.push_back(adjacent_face);
-          }
-          h = next(h, tm);
-        }
-      }
-
-      // expand the region to be filled
-      if (step > 0)
-        expand_face_selection(cc_faces, tm, step,
-                              make_boolean_property_map(cc_faces),
-                              Emptyset_iterator());
-
-      // try to compactify the selection region by also selecting all the faces included
-      // in the bounding box of the initial selection
-      std::vector<halfedge_descriptor> stack_for_expension;
-      Bbox_3 bb;
-      for(face_descriptor fd : cc_faces)
-      {
-        for(halfedge_descriptor h : halfedges_around_face(halfedge(fd, tm), tm))
-        {
-          bb += get(vpmap, target(h, tm)).bbox();
-          face_descriptor nf = face(opposite(h, tm), tm);
-          if (nf != boost::graph_traits<TriangleMesh>::null_face() &&
-              cc_faces.count(nf)==0)
-          {
-            stack_for_expension.push_back(opposite(h, tm));
-          }
-        }
-      }
-
-      while(!stack_for_expension.empty())
-      {
-        halfedge_descriptor h=stack_for_expension.back();
-        stack_for_expension.pop_back();
-        if ( cc_faces.count(face(h,tm))==1) continue;
-        if ( do_overlap(bb, get(vpmap, target(next(h, tm), tm)).bbox()) )
-        {
-          cc_faces.insert(face(h,tm));
-          halfedge_descriptor candidate = opposite(next(h, tm), tm);
-          if ( face(candidate, tm) != boost::graph_traits<TriangleMesh>::null_face() )
-            stack_for_expension.push_back( candidate );
-          candidate = opposite(prev(h, tm), tm);
-          if ( face(candidate, tm) != boost::graph_traits<TriangleMesh>::null_face() )
-            stack_for_expension.push_back( candidate );
-        }
-      }
-
-      // remove faces from the set to process
-      for(face_descriptor f : cc_faces)
-        faces_to_remove.erase(f);
-
-      if (cc_faces.size()==1) continue; // it is a triangle nothing better can be done
-
-      //Check for non-manifold vertices in the selection and remove them by selecting all incident faces:
-      //  extract the set of halfedges that is on the boundary of the holes to be
-      //  made. In addition, we make sure no hole to be created contains a vertex
-      //  visited more than once along a hole border (pinched surface)
-      //  We save the size of boundary_hedges to make sur halfedges added
-      //  from non_filled_hole are not removed.
-      bool non_manifold_vertex_remaining_in_selection = false;
-      do{
-        bool non_manifold_vertex_removed = false; //here non-manifold is for the 1D polyline
-        std::vector<halfedge_descriptor> boundary_hedges;
-        for(face_descriptor fh : cc_faces)
-        {
-          halfedge_descriptor h = halfedge(fh,tm);
-          for (int i=0;i<3; ++i)
-          {
-            if ( is_border( opposite(h, tm), tm) ||
-                 cc_faces.count( face( opposite(h, tm), tm) ) == 0)
-            {
-              boundary_hedges.push_back(h);
-            }
-            h=next(h, tm);
-          }
-        }
-
-        // detect vertices visited more than once along
-        // a hole border. We then remove all faces incident
-        // to such a vertex to force the removal of the vertex.
-        // Actually even if two holes are sharing a vertex, this
-        // vertex will be removed. It is not needed but since
-        // we do not yet have one halfedge per hole it is simpler
-        // and does not harm
-        std::set<vertex_descriptor> border_vertices;
-        for(halfedge_descriptor h : boundary_hedges)
-        {
-          if (!border_vertices.insert(target(h,tm)).second){
-            bool any_face_added = false;
-            for(halfedge_descriptor hh : halfedges_around_target(h,tm)){
-              if (!is_border(hh, tm))
-              {
-                any_face_added |= cc_faces.insert(face(hh, tm)).second; // add the face to the current selection
-                faces_to_remove.erase(face(hh, tm));
-              }
-            }
-            if (any_face_added)
-              non_manifold_vertex_removed=true;
-            else
-              non_manifold_vertex_remaining_in_selection=true;
-          }
-        }
-
-        if (!non_manifold_vertex_removed)
-          break;
-      }
-      while(true);
-
-      if (preserve_genus && non_manifold_vertex_remaining_in_selection)
-      {
-        topology_issue = true;
-        if(verbose)
-          std::cout << "  DEBUG: CC not handled due to the presence at least one non-manifold vertex\n";
-        continue; // cannot replace a patch containing a nm vertex by a disk
-      }
-
-      // before running this function if preserve_genus=false, we duplicated
-      // all of them
-      CGAL_assertion( !non_manifold_vertex_remaining_in_selection );
-
-
-      // Collect halfedges on the boundary of the region to be selected
-      // (pointing inside the domain to be remeshed)
-      std::vector<halfedge_descriptor> cc_border_hedges;
-      for(face_descriptor fd : cc_faces)
-      {
-        halfedge_descriptor h = halfedge(fd, tm);
-        for (int i=0; i<3;++i)
-        {
-          if ( is_border(opposite(h, tm), tm) ||
-               cc_faces.count( face(opposite(h, tm), tm) )== 0)
-          {
-            cc_border_hedges.push_back(h);
-          }
-          h=next(h, tm);
-        }
-      }
-
-      if(!is_selection_a_topological_disk(cc_faces, tm))
-      {
-        // check if the selection contains cycles of border halfedges
-        bool only_border_edges = true;
-        std::set<halfedge_descriptor> mesh_border_hedge;
-
-        for(halfedge_descriptor h : cc_border_hedges)
-        {
-          if ( !is_border(opposite(h, tm), tm) )
-            only_border_edges = false;
-          else
-            mesh_border_hedge.insert( opposite(h, tm) );
-        }
-        int nb_cycles=0;
-        while(!mesh_border_hedge.empty())
-        {
-          // we must count the number of cycle of boundary edges
-          halfedge_descriptor h_b = *mesh_border_hedge.begin(), h=h_b;
-          mesh_border_hedge.erase( mesh_border_hedge.begin() );
-          do{
-            h=next(h, tm);
-            if (h==h_b)
-            {
-              // found a cycle
-              ++nb_cycles;
-              break;
-            }
-            else
-            {
-              typename std::set<halfedge_descriptor>::iterator it =
-                mesh_border_hedge.find(h);
-              if ( it == mesh_border_hedge.end() )
-                break; // not a cycle
-              mesh_border_hedge.erase(it);
-            }
-          }while(true);
-        }
-
-        if(nb_cycles > (only_border_edges ? 1 : 0) )
-        {
-          if(verbose)
-            std::cout << "  DEBUG: CC not handled due to the presence of  "
-                      << nb_cycles << " of boundary edges\n";
-          topology_issue = true;
-          continue;
-        }
-        else
-        {
-          if (preserve_genus)
-          {
-            if(verbose)
-              std::cout << "  DEBUG: CC not handled because it is not a topological disk (preserve_genus=true)\n";
-            all_fixed = false;
-            continue;
-          }
-          // count the number of cycles of halfedges of the boundary
-          std::map<vertex_descriptor, vertex_descriptor> bhs;
-          for(halfedge_descriptor h : cc_border_hedges)
-          {
-            bhs[source(h, tm)]=target(h, tm);
-          }
-          int nbc=0;
-          while(!bhs.empty())
-          {
-            ++nbc;
-            std::pair<vertex_descriptor, vertex_descriptor > top=*bhs.begin();
-            bhs.erase(bhs.begin());
-            do
-            {
-              typename std::map<vertex_descriptor, vertex_descriptor>::iterator
-                it_find = bhs.find(top.second);
-              if (it_find == bhs.end()) break;
-              top = *it_find;
-              bhs.erase(it_find);
-            }
-            while(true);
-          }
-          if (nbc!=1){
-            if(verbose)
-              std::cout << "  DEBUG: CC not handled because it is not a topological disk("
-                        << nbc << " boundary cycles)\n";
-            all_fixed = false;
-            continue;
-          }
-          else
-          {
-            if(verbose)
-              std::cout << "  DEBUG: CC that is not a topological disk but has only one boundary cycle(preserve_genus=false)\n";
-          }
-        }
-      }
-
-      // sort halfedges so that they describe the sequence
-      // of halfedges of the hole to be made
-      CGAL_assertion( cc_border_hedges.size() > 2 );
-      for(std::size_t i=0; i < cc_border_hedges.size()-2; ++i)
-      {
-        vertex_descriptor tgt = target(cc_border_hedges[i], tm);
-        for(std::size_t j=i+1; j<cc_border_hedges.size(); ++j)
-        {
-          if(tgt == source(cc_border_hedges[j], tm))
-          {
-            std::swap(cc_border_hedges[i+1], cc_border_hedges[j]);
-            break;
-          }
-          CGAL_assertion(j!=cc_border_hedges.size()-1);
-        }
-      }
-      CGAL_assertion( source(cc_border_hedges.front(), tm) ==
-                      target(cc_border_hedges.back(), tm) );
-
-      // collect vertices and edges inside the current selection cc
-      std::set<vertex_descriptor> cc_interior_vertices;
-      std::set<edge_descriptor>  cc_interior_edges;
-
-      // first collect all vertices and edges incident to the faces to remove
-      for(face_descriptor fh : cc_faces)
-      {
-        for(halfedge_descriptor h : halfedges_around_face(halfedge(fh,tm),tm))
-        {
-          if (halfedge(target(h, tm), tm)==h) // limit the number of insertions
-            cc_interior_vertices.insert(target(h, tm));
-          cc_interior_edges.insert(edge(h,tm));
-        }
-      }
-      // and then remove those on the boundary
-      for(halfedge_descriptor h : cc_border_hedges)
-      {
-        cc_interior_vertices.erase(target(h, tm));
-        cc_interior_edges.erase(edge(h,tm));
-      }
-
-      if (verbose)
-      {
-        std::cout << "  DEBUG: is_valid(tm) in one_step, before mesh changes? ";
-        std::cout << is_valid_polygon_mesh(tm) << std::endl;
-      }
-
-      //try hole_filling.
-      typedef CGAL::Triple<int, int, int> Face_indices;
-      typedef typename boost::property_traits<VertexPointMap>::value_type Point;
-      std::vector<Point> hole_points, third_points;
-      hole_points.reserve(cc_border_hedges.size());
-      third_points.reserve(cc_border_hedges.size());
-      std::vector<vertex_descriptor> border_vertices;
-      for(halfedge_descriptor h : cc_border_hedges)
-      {
-        vertex_descriptor v = source(h, tm);
-        hole_points.push_back( get(vpmap, v) );
-        border_vertices.push_back(v);
-        third_points.push_back(get(vpmap, target(next(opposite(h, tm), tm), tm))); // @todo fix me for mesh border edges
-      }
-      CGAL_assertion(hole_points.size() >= 3);
-
-      // try to triangulate the hole using default parameters
-      //(using Delaunay search space if CGAL_HOLE_FILLING_DO_NOT_USE_DT3 is not defined)
-      std::vector<Face_indices> patch;
-      if (hole_points.size()>3)
-        triangulate_hole_polyline(hole_points,
-                                  third_points,
-                                  std::back_inserter(patch));
-      else
-        patch.push_back(Face_indices(0,1,2)); // trivial hole filling
-
-      if(patch.empty())
-      {
-#ifndef CGAL_HOLE_FILLING_DO_NOT_USE_DT3
-        if (verbose)
-          std::cout << "  DEBUG: Failed to fill a hole using Delaunay search space.\n";
-        triangulate_hole_polyline(hole_points,
-                                  third_points,
-                                  std::back_inserter(patch),
-                                  parameters::use_delaunay_triangulation(false));
+#ifdef CGAL_PMP_DEBUG_SMALL_CC_REMOVAL
+  std::cout << "Removing " << res << " CCs" << std::endl;
 #endif
-        if (patch.empty())
-        {
-          if (verbose)
-            std::cout << "  DEBUG: Failed to fill a hole using the whole search space.\n";
-          all_fixed = false;
-          continue;
-        }
-      }
-
-      // make sure that the hole filling is valid, we check that no
-      // edge already in the mesh is present in patch.
-      bool non_manifold_edge_found = false;
-      for(const Face_indices& triangle : patch)
-      {
-        std::array<int, 6> edges =
-          make_array(triangle.first, triangle.second,
-                     triangle.second, triangle.third,
-                     triangle.third, triangle.first);
-        for (int k=0; k<3; ++k)
-        {
-          int vi=edges[2*k], vj=edges[2*k+1];
-          // ignore boundary edges
-          if (vi+1==vj || (vj==0 && static_cast<std::size_t>(vi)==border_vertices.size()-1) )
-            continue;
-          halfedge_descriptor h = halfedge(border_vertices[vi], border_vertices[vj], tm).first;
-          if (h!=boost::graph_traits<TriangleMesh>::null_halfedge() &&
-              cc_interior_edges.count(edge(h, tm))==0)
-          {
-            non_manifold_edge_found=true;
-            break;
-          }
-        }
-        if (non_manifold_edge_found) break;
-      }
-      if (non_manifold_edge_found)
-      {
-        if (verbose)
-          std::cout << "  DEBUG: Triangulation produced is non-manifold when plugged into the mesh.\n";
-        all_fixed = false;
-        continue;
-      }
-
-      // plug the new triangles in the mesh, reusing previous edges and faces
-      std::vector<edge_descriptor> edge_stack(cc_interior_edges.begin(), cc_interior_edges.end());
-      std::vector<face_descriptor> face_stack(cc_faces.begin(), cc_faces.end());
-
-      std::map< std::pair<int, int>, halfedge_descriptor > halfedge_map;
-      int i=0;
-      // register border halfedges
-      for(halfedge_descriptor h : cc_border_hedges)
-      {
-        int j = static_cast<int>( std::size_t(i+1)%cc_border_hedges.size() );
-        halfedge_map.insert(std::make_pair( std::make_pair(i, j), h) );
-        set_halfedge(target(h, tm), h, tm); // update vertex halfedge pointer
-        CGAL_assertion( border_vertices[i] == source(h, tm) &&
-                        border_vertices[j] == target(h, tm) );
-        ++i;
-      }
-
-      std::vector<halfedge_descriptor> hedges;
-      hedges.reserve(4);
-      face_descriptor f = boost::graph_traits<TriangleMesh>::null_face();
-      for(const Face_indices& triangle : patch)
-      {
-        // get the new face
-        if (face_stack.empty())
-          f=add_face(tm);
-        else
-        {
-          f=face_stack.back();
-          face_stack.pop_back();
-        }
-
-        std::array<int, 4> indices =
-          make_array( triangle.first,
-                      triangle.second,
-                      triangle.third,
-                      triangle.first );
-        for (int i=0; i<3; ++i)
-        {
-          // get the corresponding halfedge (either a new one or an already created)
-          typename std::map< std::pair<int, int> , halfedge_descriptor >::iterator insert_res =
-            halfedge_map.insert(
-              std::make_pair( std::make_pair(indices[i], indices[i+1]),
-                              boost::graph_traits<TriangleMesh>::null_halfedge() ) ).first;
-          if (insert_res->second == boost::graph_traits<TriangleMesh>::null_halfedge())
-          {
-            if (edge_stack.empty())
-              insert_res->second = halfedge(add_edge(tm), tm);
-            else
-            {
-              insert_res->second = halfedge(edge_stack.back(), tm);
-              edge_stack.pop_back();
-            }
-
-            halfedge_map[std::make_pair(indices[i+1], indices[i])] =
-              opposite(insert_res->second, tm);
-          }
-          hedges.push_back(insert_res->second);
-        }
-        hedges.push_back(hedges.front());
-        // update halfedge connections + face pointers
-        for(int i=0; i<3;++i)
-        {
-          set_next(hedges[i], hedges[i+1], tm);
-          set_face(hedges[i], f, tm);
-          set_target(hedges[i], border_vertices[indices[i+1]], tm);
-        }
-        set_halfedge(f, hedges[0], tm);
-        hedges.clear();
-      }
-
-      // now remove remaining edges,
-      for(edge_descriptor e : edge_stack)
-        remove_edge(e, tm);
-      // vertices,
-      for(vertex_descriptor vh : cc_interior_vertices)
-        remove_vertex(vh, tm);
-      // and remaning faces
-      for(face_descriptor f : face_stack)
-        remove_face(f, tm);
-
-      if (verbose)
-        std::cout << "  DEBUG: " << cc_faces.size() << " triangles removed, "
-                  << patch.size() << " created\n";
-
-      CGAL_assertion(is_valid_polygon_mesh(tm));
 
-      something_was_done = true;
-    }
-  }
-  if (!something_was_done)
+  if(dry_run)
   {
-    faces_to_remove.swap(faces_to_remove_copy);
-    if (verbose)
-      std::cout<<"  DEBUG: Nothing was changed during this step, self-intersections won't be recomputed."<<std::endl;
+    for(face_descriptor f : faces(tmesh))
+      if(is_to_be_removed[face_cc[f]])
+        *out++ = f;
   }
-  return std::make_pair(all_fixed, topology_issue);
-}
-
-template <class TriangleMesh, class NamedParameters>
-bool remove_self_intersections(TriangleMesh& tm, const NamedParameters& np)
-{
-  typedef boost::graph_traits<TriangleMesh> graph_traits;
-  typedef typename graph_traits::face_descriptor face_descriptor;
-
-  // named parameter extraction
-  typedef typename GetVertexPointMap<TriangleMesh, NamedParameters>::type VertexPointMap;
-  VertexPointMap vpm = parameters::choose_parameter(parameters::get_parameter(np, internal_np::vertex_point),
-                                           get_property_map(vertex_point, tm));
-
-  const int max_steps = parameters::choose_parameter(parameters::get_parameter(np, internal_np::number_of_iterations), 7);
-  bool verbose = parameters::choose_parameter(parameters::get_parameter(np, internal_np::verbosity_level), 0) > 0;
-  bool preserve_genus = parameters::choose_parameter(parameters::get_parameter(np, internal_np::preserve_genus), true);
-
-  if (verbose)
-    std::cout << "DEBUG: Starting remove_self_intersections, is_valid(tm)? " << is_valid_polygon_mesh(tm) << "\n";
-
-  // first handle the removal of degenerate faces
-  remove_degenerate_faces(tm, np);
-
-  if (!preserve_genus)
-    duplicate_non_manifold_vertices(tm, np);
-
-  if (verbose)
-    std::cout << "DEBUG: After degenerate faces removal, is_valid(tm)? " << is_valid_polygon_mesh(tm) << "\n";
-
-  // Look for self-intersections in the polyhedron and remove them
-  int step=-1;
-  bool all_fixed = true; // indicates if the filling of all created holes went fine
-  bool topology_issue = false; // indicates if some boundary cycles of edges are blocking the fixing
-  std::set<face_descriptor> faces_to_remove;
-  while( ++step<max_steps )
+  else
   {
-    if (faces_to_remove.empty()) // the previous round might have been blocked due to topological constraints
-    {
-      typedef std::pair<face_descriptor, face_descriptor> Face_pair;
-      std::vector<Face_pair> self_inter;
-      // @todo : possible optimization to reduce the range to check with the bbox
-      // of the previous patches or something.
-      self_intersections(tm, std::back_inserter(self_inter));
-
-      for(Face_pair fp : self_inter)
-      {
-        faces_to_remove.insert(fp.first);
-        faces_to_remove.insert(fp.second);
-      }
-    }
-
-    if ( faces_to_remove.empty() && all_fixed){
-      if (verbose)
-        std::cout<<"DEBUG: There is no more face to remove."<<std::endl;
-      break;
-    }
+    std::vector<std::size_t> ccs_to_remove;
+    for(std::size_t i=0; i<num; ++i)
+      if(is_to_be_removed[i])
+        ccs_to_remove.push_back(i);
 
-    std::tie(all_fixed, topology_issue) =
-      remove_self_intersections_one_step(tm, faces_to_remove, vpm, step, preserve_genus, verbose);
-    if (all_fixed && topology_issue)
-    {
-      if (verbose)
-        std::cout<< "DEBUG: Process stopped because of boundary cycles"
-                    " of boundary edges involved in self-intersections.\n";
-      return false;
-    }
+    remove_connected_components(tmesh, ccs_to_remove, face_cc, np);
+    CGAL_expensive_postcondition(is_valid_polygon_mesh(tmesh));
   }
 
-  return step<max_steps;
+  return res;
 }
 
-template <class TriangleMesh>
-bool remove_self_intersections(TriangleMesh& tm)
+template <typename TriangleMesh>
+std::size_t remove_connected_components_of_negligible_size(TriangleMesh& tmesh)
 {
-  return remove_self_intersections(tm, parameters::all_default());
+  return remove_connected_components_of_negligible_size(tmesh, parameters::all_default());
 }
-/// \endcond
 
-} } // end of CGAL::Polygon_mesh_processing
+} // namespace Polygon_mesh_processing
+} // namespace CGAL
 
 #endif // CGAL_POLYGON_MESH_PROCESSING_REPAIR_H
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/repair_polygon_soup.h 5.1-1/include/CGAL/Polygon_mesh_processing/repair_polygon_soup.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/repair_polygon_soup.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/repair_polygon_soup.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/repair_polygon_soup.h $
-// $Id: repair_polygon_soup.h 6411035 2020-07-31T15:52:02+02:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/repair_polygon_soup.h $
+// $Id: repair_polygon_soup.h ae0c9fb 2020-07-31T15:52:53+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Mael Rouxel-Labbé
@@ -147,18 +147,21 @@ bool simplify_polygon(PointRange& points
 // \tparam PolygonRange a model of the concept `SequenceContainer`
 //                      whose value_type is itself a model of the concept `SequenceContainer`
 //                      whose value_type is `std::size_t`.
-// \tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+// \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 //
 // \param points points of the soup of polygons.
 // \param polygons a vector of polygons. Each element in the vector describes a polygon
 //        using the indices of the points in `points`.
-// \param np optional \ref pmp_namedparameters "Named Parameters" described below
+// \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 //
 // \cgalNamedParamsBegin
-//    \cgalParamBegin{geom_traits} a geometric traits class instance.
-//       The traits class must provide the nested functor `Equal_3`
-//       to compare lexicographically two points a function `Equal_3 equal_3_object()`.
-//   \cgalParamEnd
+//   \cgalParamNBegin{geom_traits}
+//     \cgalParamDescription{an instance of a geometric traits class}
+//     \cgalParamType{The traits class must provide the nested functor `Equal_3`
+//                    to compare lexicographically two points a function `Equal_3 equal_3_object()`.}
+//     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+//     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+//   \cgalParamNEnd
 // \cgalNamedParamsEnd
 //
 template <typename Traits, typename PointRange, typename PolygonRange>
@@ -191,25 +194,28 @@ std::size_t simplify_polygons_in_polygon
 
 // \ingroup PMP_repairing_grp
 //
-// Splits "pinched" polygons, that is polygons for which a point appears more than once,
+// splits "pinched" polygons, that is polygons for which a point appears more than once,
 // into multiple non-pinched polygons.
 //
 // \tparam PointRange a model of the concept `RandomAccessContainer` whose value type is the point type.
 // \tparam PolygonRange a model of the concept `SequenceContainer`
 //                      whose value_type is itself a model of the concepts `SequenceContainer`
 //                      and `Swappable` whose value_type is `std::size_t`.
-// \tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+// \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 //
 // \param points points of the soup of polygons.
 // \param polygons a vector of polygons. Each element in the vector describes a polygon
 //        using the indices of the points in `points`.
-// \param np optional \ref pmp_namedparameters "Named Parameters" described below
+// \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 //
 // \cgalNamedParamsBegin
-//    \cgalParamBegin{geom_traits} a geometric traits class instance.
-//       The traits class must provide the nested functor `Less_xyz_3`
-//       to compare lexicographically two points a function `Less_xyz_3 less_xyz_3_object()`.
-//   \cgalParamEnd
+//   \cgalParamNBegin{geom_traits}
+//     \cgalParamDescription{an instance of a geometric traits class}
+//     \cgalParamType{The traits class must provide the nested functor `Less_xyz_3`
+//                    to compare lexicographically two points a function `Less_xyz_3 less_xyz_3_object()`.}
+//     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+//     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+//   \cgalParamNEnd
 // \cgalNamedParamsEnd
 //
 template <typename Traits, typename PointRange, typename PolygonRange>
@@ -294,13 +300,13 @@ std::size_t split_pinched_polygons_in_po
 
 // \ingroup PMP_repairing_grp
 //
-// Removes polygons with fewer than 2 points from the soup.
+// removes polygons with fewer than 2 points from the soup.
 //
 // \tparam PointRange a model of the concept `Container` whose value type is the point type.
 // \tparam PolygonRange a model of the concept `SequenceContainer`
 //                      whose value_type is itself a model of the concept `Container`
 //                      whose value_type is `std::size_t`.
-// \tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+// \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 //
 // \param points points of the soup of polygons.
 // \param polygons a vector of polygons. Each element in the vector describes a polygon
@@ -351,7 +357,7 @@ std::size_t remove_degenerate_polygons_i
 
 /// \ingroup PMP_repairing_grp
 ///
-/// Removes the isolated points from a polygon soup.
+/// removes the isolated points from a polygon soup.
 /// A point is considered <i>isolated</i> if it does not appear in any polygon of the soup.
 ///
 /// \tparam PointRange a model of the concept `SequenceContainer` whose value type is the point type.
@@ -453,7 +459,7 @@ std::size_t remove_isolated_points_in_po
 
 /// \ingroup PMP_repairing_grp
 ///
-/// Merges the duplicate points in a polygon soup.
+/// merges the duplicate points in a polygon soup.
 /// Note that the index of a point that is merged with another point will thus change
 /// in all the polygons that the point appears in.
 ///
@@ -462,18 +468,21 @@ std::size_t remove_isolated_points_in_po
 /// \tparam PolygonRange a model of the concept `RandomAccessContainer`
 ///                      whose value_type is itself a model of the concept `RandomAccessContainer`
 ///                      whose value_type is `std::size_t`.
-/// \tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+/// \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 ///
 /// \param points points of the soup of polygons.
 /// \param polygons a vector of polygons. Each element in the vector describes a polygon
 ///        using the indices of the points in `points`.
-/// \param np optional \ref pmp_namedparameters "Named Parameters" described below
+/// \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 ///
 /// \cgalNamedParamsBegin
-///    \cgalParamBegin{geom_traits} a geometric traits class instance.
-///       The traits class must provide the nested functor `Less_xyz_3`
-///       to compare lexicographically two points a function `Less_xyz_3 less_xyz_3_object()`.
-///   \cgalParamEnd
+///   \cgalParamNBegin{geom_traits}
+///     \cgalParamDescription{an instance of a geometric traits class}
+///     \cgalParamType{The traits class must provide the nested functor `Less_xyz_3`
+///                    to compare lexicographically two points a function `Less_xyz_3 less_xyz_3_object()`.}
+///     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+///     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+///   \cgalParamNEnd
 /// \cgalNamedParamsEnd
 ///
 /// \returns the number of removed points
@@ -491,7 +500,7 @@ std::size_t merge_duplicate_points_in_po
   using parameters::choose_parameter;
 
   typedef typename internal::GetPolygonGeomTraits<PointRange, PolygonRange, NamedParameters>::type Traits;
-  Traits traits = choose_parameter(get_parameter(np, internal_np::geom_traits), Traits());
+  Traits traits = choose_parameter<Traits>(get_parameter(np, internal_np::geom_traits));
 
   typedef typename Traits::Less_xyz_3                                             Less_xyz_3;
 
@@ -545,7 +554,8 @@ std::size_t merge_duplicate_points_in_po
   const std::size_t removed_points_n = ini_points_n - points.size();
 
 #ifdef CGAL_PMP_REPAIR_POLYGON_SOUP_VERBOSE
-  std::cout << "Removed (merged) " << removed_points_n << " duplicate points" << std::endl;
+  if(removed_points_n > 0)
+    std::cout << "Removed (merged) " << removed_points_n << " duplicate points" << std::endl;
 #endif
 
   return removed_points_n;
@@ -763,7 +773,7 @@ struct Duplicate_collector<ValueType, CG
 
 // \ingroup PMP_repairing_grp
 //
-// Collects duplicate polygons in a polygon soup, that is polygons that share the same vertices in the same
+// collects duplicate polygons in a polygon soup, that is polygons that share the same vertices in the same
 // order.
 //
 // \tparam PointRange a model of the concept `RandomAccessContainer` whose value type is the point type.
@@ -840,7 +850,7 @@ DuplicateOutputIterator collect_duplicat
 
 /// \ingroup PMP_repairing_grp
 ///
-/// Merges the duplicate polygons in a polygon soup. Two polygons are duplicate if they share the same
+/// merges the duplicate polygons in a polygon soup. Two polygons are duplicate if they share the same
 /// vertices in the same order. Note that the first vertex of the polygon does not matter, that is
 /// the triangle `0,1,2` is a duplicate of the triangle `2,0,1`.
 ///
@@ -848,27 +858,37 @@ DuplicateOutputIterator collect_duplicat
 /// \tparam PolygonRange a model of the concept `SequenceContainer`
 ///                      whose value_type is itself a model of the concepts `RandomAccessContainer`
 ///                      and `ReversibleContainer` whose value_type is `std::size_t`.
-/// \tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+/// \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 ///
 /// \param points points of the soup of polygons.
 /// \param polygons a vector of polygons. Each element in the vector describes a polygon
 ///        using the indices of the points in `points`.
-/// \param np optional \ref pmp_namedparameters "Named Parameters", amongst those described below
+/// \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 ///
 /// \cgalNamedParamsBegin
-///   \cgalParamBegin{geom_traits} a geometric traits class instance.
-///     The traits class must provide the nested functor `Less_xyz_3`
-///     to compare lexicographically two points a function `Less_xyz_3 less_xyz_3_object()`.
-///   \cgalParamEnd
-///   \cgalParamBegin{erase_all_duplicates}
-///     Parameter to indicate, when multiple polygons are duplicates, whether all the duplicate polygons
-///     should be removed or if one (arbitrarily chosen) face should be kept. %Default is `false`.
-///   \cgalParamEnd
-///   \cgalParamBegin{require_same_orientation}
-///     Parameter to indicate if polygon orientation should be taken into account when determining
-///     whether two polygons are duplicates, that is, whether e.g. the triangles `0,1,2` and `0,2,1`
-///     are duplicates. %Default is `false`.
-///   \cgalParamEnd
+///   \cgalParamNBegin{geom_traits}
+///     \cgalParamDescription{an instance of a geometric traits class}
+///     \cgalParamType{The traits class must provide the nested functor `Less_xyz_3`
+///                    to compare lexicographically two points a function `Less_xyz_3 less_xyz_3_object()`.}
+///     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+///     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{erase_all_duplicates}
+///     \cgalParamDescription{Parameter to indicate, when multiple polygons are duplicates,
+///                           whether all the duplicate polygons should be removed
+///                           or if one (arbitrarily chosen) face should be kept.}
+///     \cgalParamType{Boolean}
+///     \cgalParamDefault{`false`}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{require_same_orientation}
+///     \cgalParamDescription{Parameter to indicate if polygon orientation should be taken
+///                           into account when determining whether two polygons are duplicates,
+///                           that is, whether e.g. the triangles `0,1,2` and `0,2,1` are duplicates.}
+///     \cgalParamType{Boolean}
+///     \cgalParamDefault{`false`}
+///   \cgalParamNEnd
 /// \cgalNamedParamsEnd
 ///
 /// \returns the number of removed polygons
@@ -892,7 +912,7 @@ std::size_t merge_duplicate_polygons_in_
 #endif
 
   typedef typename internal::GetPolygonGeomTraits<PointRange, PolygonRange, NamedParameters>::type Traits;
-  Traits traits = choose_parameter(get_parameter(np, internal_np::geom_traits), Traits());
+  Traits traits = choose_parameter<Traits>(get_parameter(np, internal_np::geom_traits));
 
   std::vector<std::vector<P_ID> > all_duplicate_polygons;
   internal::collect_duplicate_polygons(points, polygons, std::back_inserter(all_duplicate_polygons), traits, same_orientation);
@@ -981,7 +1001,7 @@ std::size_t merge_duplicate_polygons_in_
 
 /// \ingroup PMP_repairing_grp
 ///
-/// Cleans a given polygon soup through various repairing operations. More precisely, this function
+/// cleans a given polygon soup through various repairing operations. More precisely, this function
 /// carries out the following tasks, in the same order as they are listed:
 /// - merging of duplicate points, using the function
 ///   `CGAL::Polygon_mesh_processing::merge_duplicate_points_in_polygon_soup()`;
@@ -1002,32 +1022,38 @@ std::size_t merge_duplicate_polygons_in_
 /// \tparam PolygonRange a model of the concept `SequenceContainer`.
 ///                      whose value_type is itself a model of the concepts `SequenceContainer`,
 ///                      `Swappable`, and `ReversibleContainer` whose value_type is `std::size_t`.
-/// \tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+/// \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 ///
 /// \param points points of the soup of polygons.
 /// \param polygons a vector of polygons. Each element in the vector describes a polygon
 ///        using the indices of the points in `points`.
-/// \param np optional \ref pmp_namedparameters "Named Parameters", amongst those described below
+/// \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 ///
 /// \cgalNamedParamsBegin
-///    \cgalParamBegin{geom_traits} a geometric traits class instance.
-///       The traits class must provide the nested functors :
-///         - `Less_xyz_3` to compare lexicographically two points
-///         - `Equal_3` to check whether 2 points are identical
-///
-///       and, for each functor `Foo`, a function `Foo foo_object()`.
-///   \cgalParamEnd
-///   \cgalParamBegin{erase_all_duplicates}
-///     Parameter forwarded to the function `merge_duplicate_polygons_in_polygon_soup()` to indicate,
-///     when multiple polygons are duplicates, whether all the duplicate polygons
-///     should be removed or if one (arbitrarily chosen) face should be kept. %Default is `false`.
-///   \cgalParamEnd
-///   \cgalParamBegin{require_same_orientation}
-///     Parameter forwarded to the function `merge_duplicate_polygons_in_polygon_soup()`
-///     to indicate if polygon orientation should be taken into account when determining whether
-///     two polygons are duplicates, that is, whether e.g. the triangles `0,1,2` and `0,2,1` are duplicates.
-///     %Default is `false`.
-///   \cgalParamEnd
+///   \cgalParamNBegin{geom_traits}
+///     \cgalParamDescription{an instance of a geometric traits class}
+///     \cgalParamType{The traits class must provide the nested functors `Less_xyz_3` and `Equal_3`
+///                    to respectivelycompare lexicographically two points and to check if 2 points
+///                    are identical. For each functor `Foo`, a function `Foo foo_object()` must be provided.}
+///     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+///     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{erase_all_duplicates}
+///     \cgalParamDescription{Parameter to indicate, when multiple polygons are duplicates,
+///                           whether all the duplicate polygons should be removed
+///                           or if one (arbitrarily chosen) face should be kept.}
+///     \cgalParamType{Boolean}
+///     \cgalParamDefault{`false`}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{require_same_orientation}
+///     \cgalParamDescription{Parameter to indicate if polygon orientation should be taken
+///                           into account when determining whether two polygons are duplicates,
+///                           that is, whether e.g. the triangles `0,1,2` and `0,2,1` are duplicates.}
+///     \cgalParamType{Boolean}
+///     \cgalParamDefault{`false`}
+///   \cgalParamNEnd
 /// \cgalNamedParamsEnd
 ///
 template <typename PointRange, typename PolygonRange, typename NamedParameters>
@@ -1039,7 +1065,7 @@ void repair_polygon_soup(PointRange& poi
   using parameters::choose_parameter;
 
   typedef typename internal::GetPolygonGeomTraits<PointRange, PolygonRange, NamedParameters>::type Traits;
-  Traits traits = choose_parameter(get_parameter(np, internal_np::geom_traits), Traits());
+  Traits traits = choose_parameter<Traits>(get_parameter(np, internal_np::geom_traits));
 
 #ifdef CGAL_PMP_REPAIR_POLYGON_SOUP_VERBOSE
   std::cout << "Repairing soup with " << points.size() << " points and " << polygons.size() << " polygons" << std::endl;
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/repair_self_intersections.h 5.1-1/include/CGAL/Polygon_mesh_processing/repair_self_intersections.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/repair_self_intersections.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/repair_self_intersections.h	2020-09-08 12:15:23.000000000 +0000
@@ -0,0 +1,1981 @@
+// Copyright (c) 2015-2020 GeometryFactory (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/repair_self_intersections.h $
+// $Id: repair_self_intersections.h e720e6d 2020-06-16T16:41:04+02:00 Mael Rouxel-Labbé
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Sebastien Loriot,
+//                 Mael Rouxel-Labbé
+//
+#ifndef CGAL_POLYGON_MESH_PROCESSING_REPAIR_SELF_INTERSECTIONS_H
+#define CGAL_POLYGON_MESH_PROCESSING_REPAIR_SELF_INTERSECTIONS_H
+
+#include <CGAL/license/Polygon_mesh_processing/repair.h>
+
+#include <CGAL/Polygon_mesh_processing/detect_features.h>
+#include <CGAL/Polygon_mesh_processing/distance.h>
+#include <CGAL/Polygon_mesh_processing/manifoldness.h>
+#include <CGAL/Polygon_mesh_processing/orient_polygon_soup.h>
+#include <CGAL/Polygon_mesh_processing/polygon_soup_to_polygon_mesh.h>
+#include <CGAL/Polygon_mesh_processing/self_intersections.h>
+#include <CGAL/Polygon_mesh_processing/smooth_mesh.h>
+#include <CGAL/Polygon_mesh_processing/triangulate_hole.h>
+
+#include <CGAL/assertions.h>
+#include <CGAL/boost/graph/copy_face_graph.h>
+#include <CGAL/boost/graph/Face_filtered_graph.h>
+#include <CGAL/boost/graph/Named_function_parameters.h>
+#include <CGAL/boost/graph/named_params_helper.h>
+#include <CGAL/boost/graph/selection.h>
+#include <CGAL/box_intersection_d.h>
+#include <CGAL/utility.h>
+
+#include <array>
+#include <fstream>
+#include <iostream>
+#include <iterator>
+#include <map>
+#include <set>
+#include <type_traits>
+#include <utility>
+#include <vector>
+
+// #define CGAL_PMP_REMOVE_SELF_INTERSECTIONS_NO_SMOOTHING
+// #define CGAL_PMP_REMOVE_SELF_INTERSECTIONS_NO_CONSTRAINTS_IN_HOLE_FILLING
+
+// Self-intersection removal is done by making a big-enough hole and filling it
+//
+// Local self-intersection removal is more subtle and only considers self-intersections
+// within a connected component. It then tries to fix those by trying successively:
+// - smoothing with the sharp edges in the area being constrained
+// - smoothing without the sharp edges in the area being constrained
+// - hole-filling with the sharp edges in the area being constrained
+// - hole-filling without the sharp edges in the area being constrained
+//
+// The working area grows as long as we haven't been able to fix the self-intersection,
+// up to a user-defined number of times.
+
+namespace CGAL {
+namespace Polygon_mesh_processing {
+namespace internal {
+
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+static int unsolved_self_intersections = 0;
+static int self_intersections_solved_by_constrained_smoothing = 0;
+static int self_intersections_solved_by_unconstrained_smoothing = 0;
+static int self_intersections_solved_by_constrained_hole_filling = 0;
+static int self_intersections_solved_by_unconstrained_hole_filling = 0;
+#endif
+
+// -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-
+
+// @todo these could be extracted to somewhere else, it's useful in itself
+template <typename PolygonMesh, typename VPM, typename Point, typename FaceOutputIterator>
+FaceOutputIterator replace_faces_with_patch(const std::vector<typename boost::graph_traits<PolygonMesh>::vertex_descriptor>& border_vertices,
+                                            const std::set<typename boost::graph_traits<PolygonMesh>::vertex_descriptor>& interior_vertices,
+                                            const std::vector<typename boost::graph_traits<PolygonMesh>::halfedge_descriptor>& border_hedges,
+                                            const std::set<typename boost::graph_traits<PolygonMesh>::edge_descriptor>& interior_edges,
+                                            const std::set<typename boost::graph_traits<PolygonMesh>::face_descriptor>& faces,
+                                            const std::vector<std::vector<Point> >& patch,
+                                            PolygonMesh& pmesh,
+                                            VPM& vpm,
+                                            FaceOutputIterator out)
+{
+  CGAL_static_assertion((std::is_same<typename boost::property_traits<VPM>::value_type, Point>::value));
+
+  typedef typename boost::graph_traits<PolygonMesh>::vertex_descriptor      vertex_descriptor;
+  typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor    halfedge_descriptor;
+  typedef typename boost::graph_traits<PolygonMesh>::edge_descriptor        edge_descriptor;
+  typedef typename boost::graph_traits<PolygonMesh>::face_descriptor        face_descriptor;
+
+  typedef std::vector<Point>                                                Point_face;
+  typedef std::vector<vertex_descriptor>                                    Vertex_face;
+
+  CGAL_precondition(is_valid_polygon_mesh(pmesh));
+
+  // To be used to create new elements
+  std::vector<vertex_descriptor> vertex_stack(interior_vertices.begin(), interior_vertices.end());
+  std::vector<edge_descriptor> edge_stack(interior_edges.begin(), interior_edges.end());
+  std::vector<face_descriptor> face_stack(faces.begin(), faces.end());
+
+  // Introduce new vertices, convert the patch in vertex patches
+  std::vector<Vertex_face> patch_with_vertices;
+  patch_with_vertices.reserve(patch.size());
+
+  std::map<Point, vertex_descriptor> point_to_vs;
+
+  // first, add those for which the vertex will not change
+  for(const vertex_descriptor v : border_vertices)
+    point_to_vs[get(vpm, v)] = v;
+
+  // now build a correspondence map and the faces with vertices
+  const vertex_descriptor null_v = boost::graph_traits<PolygonMesh>::null_vertex();
+  for(const Point_face& face : patch)
+  {
+    Vertex_face vface;
+    vface.reserve(face.size());
+
+    for(const Point& p : face)
+    {
+      bool success;
+      typename std::map<Point, vertex_descriptor>::iterator it;
+      std::tie(it, success) = point_to_vs.insert(std::make_pair(p, null_v));
+      vertex_descriptor& v = it->second;
+
+      if(success) // first time we meet that point, means it`s an interior point and we need to make a new vertex
+      {
+        if(vertex_stack.empty())
+        {
+          v = add_vertex(pmesh);
+        }
+        else
+        {
+          v = vertex_stack.back();
+          vertex_stack.pop_back();
+        }
+
+        put(vpm, v, p);
+      }
+
+      vface.push_back(v);
+    }
+
+    patch_with_vertices.push_back(vface);
+  }
+
+  typedef std::pair<vertex_descriptor, vertex_descriptor>                        Vertex_pair;
+  typedef std::map<Vertex_pair, halfedge_descriptor>                             Vertex_pair_halfedge_map;
+
+  Vertex_pair_halfedge_map halfedge_map;
+
+  // register border halfedges
+  int i = 0;
+  for(halfedge_descriptor h : border_hedges)
+  {
+    const vertex_descriptor vs = source(h, pmesh);
+    const vertex_descriptor vt = target(h, pmesh);
+    halfedge_map.insert(std::make_pair(std::make_pair(vs, vt), h));
+
+    set_halfedge(target(h, pmesh), h, pmesh); // update vertex halfedge pointer
+    ++i;
+  }
+
+  face_descriptor f = boost::graph_traits<PolygonMesh>::null_face();
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+  std::vector<face_descriptor> new_faces;
+#endif
+
+  for(const Vertex_face& vface : patch_with_vertices)
+  {
+    if(face_stack.empty())
+    {
+      f = add_face(pmesh);
+    }
+    else
+    {
+      f = face_stack.back();
+      face_stack.pop_back();
+    }
+
+    *out++ = f;
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+    new_faces.push_back(f);
+#endif
+
+    std::vector<halfedge_descriptor> hedges;
+    hedges.reserve(vface.size());
+
+    for(std::size_t i=0, n=vface.size(); i<n; ++i)
+    {
+      const vertex_descriptor vi = vface[i];
+      const vertex_descriptor vj = vface[(i+1)%n];
+
+      // get the corresponding halfedge (either a new one or an already created)
+      bool success;
+      typename Vertex_pair_halfedge_map::iterator it;
+      std::tie(it, success) = halfedge_map.insert(std::make_pair(std::make_pair(vi, vj),
+                                                  boost::graph_traits<PolygonMesh>::null_halfedge()));
+      halfedge_descriptor& h = it->second;
+
+      if(success) // this halfedge is an interior halfedge
+      {
+        if(edge_stack.empty())
+        {
+          h = halfedge(add_edge(pmesh), pmesh);
+        }
+        else
+        {
+          h = halfedge(edge_stack.back(), pmesh);
+          edge_stack.pop_back();
+        }
+
+        halfedge_map[std::make_pair(vj, vi)] = opposite(h, pmesh);
+      }
+
+      hedges.push_back(h);
+    }
+
+    CGAL_assertion(vface.size() == hedges.size());
+
+    // update halfedge connections + face pointers
+    for(std::size_t i=0, n=vface.size(); i<n; ++i)
+    {
+      set_next(hedges[i], hedges[(i+1)%n], pmesh);
+      set_face(hedges[i], f, pmesh);
+
+      set_target(hedges[i], vface[(i+1)%n], pmesh);
+      set_halfedge(vface[(i+1)%n], hedges[i], pmesh);
+    }
+
+    set_halfedge(f, hedges[0], pmesh);
+  }
+
+  // now remove the remaining superfluous vertices, edges, faces
+  for(vertex_descriptor v : vertex_stack)
+    remove_vertex(v, pmesh);
+  for(edge_descriptor e : edge_stack)
+    remove_edge(e, pmesh);
+  for(face_descriptor f : face_stack)
+    remove_face(f, pmesh);
+
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_OUTPUT
+  std::ofstream res_out("results/last_patch_replacement.off");
+  res_out << std::setprecision(17) << pmesh;
+#endif
+
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+  std::cout << "  DEBUG: Replacing range with patch: ";
+  std::cout << faces.size() << " triangles removed, " << patch.size() << " created\n";
+#endif
+
+  CGAL_postcondition(is_valid_polygon_mesh(pmesh));
+
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+  CGAL_postcondition(!does_self_intersect(new_faces, pmesh));
+#endif
+
+  return out;
+}
+
+template <typename PolygonMesh, typename VPM, typename Point, typename FaceOutputIterator>
+FaceOutputIterator replace_faces_with_patch(const std::set<typename boost::graph_traits<PolygonMesh>::face_descriptor>& face_range,
+                                            const std::vector<std::vector<Point> >& patch,
+                                            PolygonMesh& pmesh,
+                                            VPM& vpm,
+                                            FaceOutputIterator out)
+{
+  typedef typename boost::graph_traits<PolygonMesh>::vertex_descriptor     vertex_descriptor;
+  typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor   halfedge_descriptor;
+  typedef typename boost::graph_traits<PolygonMesh>::edge_descriptor       edge_descriptor;
+  typedef typename boost::graph_traits<PolygonMesh>::face_descriptor       face_descriptor;
+
+  std::vector<vertex_descriptor> border_vertices;
+  std::set<vertex_descriptor> interior_vertices;
+  std::vector<halfedge_descriptor> border_hedges;
+  std::set<edge_descriptor> interior_edges;
+
+  for(face_descriptor fh : face_range)
+  {
+    for(halfedge_descriptor h : halfedges_around_face(halfedge(fh, pmesh), pmesh))
+    {
+      if(halfedge(target(h, pmesh), pmesh) == h) // limit the number of insertions
+        interior_vertices.insert(target(h, pmesh));
+    }
+  }
+
+  for(face_descriptor fh : face_range)
+  {
+    for(halfedge_descriptor h : halfedges_around_face(halfedge(fh, pmesh), pmesh))
+    {
+      CGAL_assertion(!is_border(h, pmesh));
+
+      const edge_descriptor e = edge(h, pmesh);
+      const halfedge_descriptor opp_h = opposite(h, pmesh);
+      const face_descriptor opp_f = face(opp_h, pmesh);
+
+      if(is_border(opp_h, pmesh) || face_range.count(opp_f) == 0)
+      {
+        vertex_descriptor v = target(h, pmesh);
+        interior_vertices.erase(v);
+        border_hedges.push_back(h);
+        border_vertices.push_back(v);
+      }
+      else
+      {
+        interior_edges.insert(e);
+      }
+    }
+  }
+
+  return replace_faces_with_patch(border_vertices, interior_vertices,
+                                  border_hedges, interior_edges, face_range, patch,
+                                  pmesh, vpm, out);
+}
+
+template <typename PolygonMesh, typename VPM, typename Point>
+void replace_faces_with_patch(const std::set<typename boost::graph_traits<PolygonMesh>::face_descriptor>& faces,
+                              const std::vector<std::vector<Point> >& patch,
+                              PolygonMesh& pmesh,
+                              VPM& vpm)
+{
+  CGAL::Emptyset_iterator out;
+  replace_faces_with_patch(faces, patch, pmesh, vpm, out);
+}
+
+// -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-
+
+template <typename Point, typename FaceRange, typename PolygonMesh, typename VertexPointMap>
+void back_up_face_range_as_point_patch(std::vector<std::vector<Point> >& point_patch,
+                                       const FaceRange& face_range,
+                                       const PolygonMesh& tmesh,
+                                       const VertexPointMap vpm)
+{
+  typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor    halfedge_descriptor;
+  typedef typename boost::graph_traits<PolygonMesh>::face_descriptor        face_descriptor;
+
+  point_patch.reserve(face_range.size());
+
+  for(const face_descriptor f : face_range)
+  {
+    std::vector<Point> face_points;
+    for(const halfedge_descriptor h : CGAL::halfedges_around_face(halfedge(f, tmesh), tmesh))
+      face_points.push_back(get(vpm, target(h, tmesh)));
+
+    point_patch.push_back(face_points);
+  }
+}
+
+// -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-
+
+template <typename FaceRange, typename EdgeConstrainMap,
+          typename TriangleMesh, typename VertexPointMap, typename GeomTraits>
+void constrain_sharp_and_border_edges(const FaceRange& faces,
+                                      TriangleMesh& tmesh,
+                                      EdgeConstrainMap& eif,
+                                      const bool constrain_sharp_edges,
+                                      const double dihedral_angle,
+                                      const double /*weak_DA*/,
+                                      VertexPointMap vpm,
+                                      const GeomTraits& gt)
+{
+  typedef typename boost::graph_traits<TriangleMesh>::halfedge_descriptor   halfedge_descriptor;
+  typedef typename boost::graph_traits<TriangleMesh>::edge_descriptor       edge_descriptor;
+  typedef typename boost::graph_traits<TriangleMesh>::face_descriptor       face_descriptor;
+
+  typedef typename GeomTraits::FT                                           FT;
+  typedef typename GeomTraits::Vector_3                                     Vector;
+
+  std::map<edge_descriptor, bool> is_border_of_selection;
+  for(face_descriptor f : faces)
+  {
+    // @fixme what about nm vertices
+    for(halfedge_descriptor h : CGAL::halfedges_around_face(halfedge(f, tmesh), tmesh))
+    {
+      // Default initialization is guaranteed to be `false`. Thus, meet it once will switch
+      // the value to `true` and meeting it twice will switch back to `false`.
+      const edge_descriptor e = edge(h, tmesh);
+      is_border_of_selection[e] = !(is_border_of_selection[e]);
+    }
+  }
+
+#if 0 // Until detect_features++ is integrated
+  CGAL::Polygon_mesh_processing::experimental::detect_sharp_edges_pp(faces, tmesh, dihedral_angle, eif,
+                                                                     parameters::weak_dihedral_angle(weak_DA));
+
+  // borders are also constrained
+  for(const auto& ep : is_border_of_selection)
+    if(ep.second)
+      put(eif, ep.first, true);
+#else
+  // this is basically the code that is in detect_features (at the very bottom)
+  // but we do not want a folding to be marked as a sharp feature so the dihedral angle is also
+  // bounded from above
+  const double bound = dihedral_angle;
+  const double cos_angle = std::cos(bound * CGAL_PI / 180.);
+
+  for(const auto& ep : is_border_of_selection)
+  {
+    bool flag = ep.second;
+    if(constrain_sharp_edges && !flag)
+    {
+      const halfedge_descriptor h = halfedge(ep.first, tmesh);
+      CGAL_assertion(!is_border(edge(h, tmesh), tmesh));
+
+      const face_descriptor f1 = face(h, tmesh);
+      const face_descriptor f2 = face(opposite(h, tmesh), tmesh);
+
+      // @todo cache normals
+      const Vector n1 = compute_face_normal(f1, tmesh, parameters::vertex_point_map(vpm).geom_traits(gt));
+      const Vector n2 = compute_face_normal(f2, tmesh, parameters::vertex_point_map(vpm).geom_traits(gt));
+      const FT c = gt.compute_scalar_product_3_object()(n1, n2);
+
+      // Do not mark as sharp edges with a dihedral angle that is almost `pi` because this is likely
+      // due to a foldness on the mesh rather than a sharp edge that we wish to preserve
+      // (Ideally this would be pre-treated as part of the flatness treatment)
+      flag = (c <= cos_angle && c >= -cos_angle);
+    }
+
+    is_border_of_selection[ep.first] = flag; // Only needed for output, really
+    put(eif, ep.first, flag);
+  }
+#endif
+
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_OUTPUT
+  std::ofstream out("results/constrained_edges.polylines.txt");
+  out << std::setprecision(17);
+  for(edge_descriptor e : edges(tmesh))
+    if(get(eif, e))
+       out << "2 " << tmesh.point(source(e, tmesh)) << " " << tmesh.point(target(e, tmesh)) << std::endl;
+  out.close();
+#endif
+}
+
+// -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-
+
+template <typename TriangleMesh, typename VertexPointMap, typename GeomTraits>
+bool remove_self_intersections_with_smoothing(std::set<typename boost::graph_traits<TriangleMesh>::face_descriptor>& face_range,
+                                              TriangleMesh& tmesh,
+                                              const bool constrain_sharp_edges,
+                                              const double dihedral_angle,
+                                              const double weak_DA,
+                                              VertexPointMap vpm,
+                                              const GeomTraits& gt)
+{
+  namespace CP = CGAL::parameters;
+
+  typedef typename boost::graph_traits<TriangleMesh>::halfedge_descriptor   halfedge_descriptor;
+  typedef typename boost::graph_traits<TriangleMesh>::face_descriptor       face_descriptor;
+
+  typedef typename boost::property_traits<VertexPointMap>::value_type       Point;
+
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+  std::cout << "  DEBUG: repair with smoothing... (constraining sharp edges: ";
+  std::cout << std::boolalpha << constrain_sharp_edges << ")" << std::endl;
+#endif
+
+  CGAL_precondition(does_self_intersect(face_range, tmesh));
+
+  // Rather than working directly on the mesh, copy a range and work on this instead
+  const CGAL::Face_filtered_graph<TriangleMesh> ffg(tmesh, face_range);
+  TriangleMesh local_mesh;
+  CGAL::copy_face_graph(ffg, local_mesh, CP::vertex_point_map(vpm));
+
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_OUTPUT
+  std::ofstream out_p("results/local_mesh.off");
+  out_p << std::setprecision(17) << local_mesh;
+  out_p.close();
+#endif
+
+  // Constrain sharp and border edges
+  typedef CGAL::dynamic_edge_property_t<bool>                                 Edge_property_tag;
+  typedef typename boost::property_map<TriangleMesh, Edge_property_tag>::type EIFMap;
+  EIFMap eif = get(Edge_property_tag(), local_mesh);
+
+  VertexPointMap local_vpm = get_property_map(vertex_point, local_mesh);
+
+  constrain_sharp_and_border_edges(faces(local_mesh), local_mesh, eif, constrain_sharp_edges,
+                                   dihedral_angle, weak_DA, local_vpm, gt);
+
+  // @todo choice of number of iterations? Till convergence && max of 100?
+  Polygon_mesh_processing::smooth_mesh(faces(local_mesh), local_mesh, CP::edge_is_constrained_map(eif)
+                                                                         .number_of_iterations(100)
+                                                                         .use_safety_constraints(false));
+
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_OUTPUT
+  std::ofstream out("results/post_smoothing_local_mesh.off");
+  out << std::setprecision(17) << local_mesh;
+  out.close();
+#endif
+
+  if(does_self_intersect(local_mesh))
+    return false;
+
+  std::vector<std::vector<Point> > patch;
+  for(const face_descriptor f : faces(local_mesh))
+  {
+    halfedge_descriptor h = halfedge(f, local_mesh);
+    patch.emplace_back(std::initializer_list<Point>{get(local_vpm, target(h, local_mesh)),
+                                                    get(local_vpm, target(next(h, local_mesh), local_mesh)),
+                                                    get(local_vpm, target(prev(h, local_mesh), local_mesh))});
+  }
+
+  std::set<face_descriptor> new_faces;
+  replace_faces_with_patch(face_range, patch, tmesh, vpm, std::inserter(new_faces, new_faces.end()));
+  CGAL_assertion(!does_self_intersect(new_faces, tmesh, parameters::vertex_point_map(vpm)));
+
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+  if(constrain_sharp_edges)
+    ++self_intersections_solved_by_constrained_smoothing;
+  else
+    ++self_intersections_solved_by_unconstrained_smoothing;
+#endif
+
+  return true;
+}
+
+// -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-
+
+template <typename TriangleMesh>
+bool order_border_halfedge_range(std::vector<typename boost::graph_traits<TriangleMesh>::halfedge_descriptor>& hrange,
+                                 const TriangleMesh& tmesh)
+{
+  typedef typename boost::graph_traits<TriangleMesh>::vertex_descriptor     vertex_descriptor;
+
+  CGAL_precondition(hrange.size() > 2);
+
+  for(std::size_t i=0; i<hrange.size()-2; ++i)
+  {
+    const vertex_descriptor tgt = target(hrange[i], tmesh);
+    for(std::size_t j=i+1; j<hrange.size(); ++j)
+    {
+      if(tgt == source(hrange[j], tmesh))
+      {
+        std::swap(hrange[i+1], hrange[j]);
+        break;
+      }
+
+      // something went wrong while ordering halfedge (e.g. hole has more than one boundary cycle)
+      if(j == hrange.size() - 1)
+        return false;
+    }
+  }
+
+  CGAL_postcondition(source(hrange.front(), tmesh) == target(hrange.back(), tmesh));
+  return true;
+}
+
+// -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-
+
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_OUTPUT
+
+template <typename FaceContainer, typename TriangleMesh>
+void dump_cc(const FaceContainer& cc_faces,
+             const TriangleMesh& mesh,
+             const std::string filename)
+{
+  typedef typename boost::graph_traits<TriangleMesh>::face_descriptor      face_descriptor;
+
+  typedef typename GetVertexPointMap<TriangleMesh>::const_type VertexPointMap;
+  VertexPointMap vpm =get_const_property_map(vertex_point, mesh);
+
+  std::ofstream out(filename);
+  out.precision(17);
+
+  out << "OFF\n";
+  out << 3*cc_faces.size() << " " << cc_faces.size() << " 0\n";
+
+  for(const face_descriptor f : cc_faces)
+  {
+    out << get(vpm, source(halfedge(f, mesh), mesh)) << "\n";
+    out << get(vpm, target(halfedge(f, mesh), mesh)) << "\n";
+    out << get(vpm, target(next(halfedge(f, mesh), mesh), mesh)) << "\n";
+  }
+
+  int id = 0;
+  for(const face_descriptor f : cc_faces)
+  {
+    CGAL_USE(f);
+    out << "3 " << id << " " << id+1 << " " << id+2 << "\n";
+    id += 3;
+  }
+
+  out.close();
+}
+
+template <typename Point>
+void dump_tentative_hole(std::vector<std::vector<Point> >& point_patch,
+                         const std::string filename)
+{
+  std::ofstream out(filename);
+  out << std::setprecision(17);
+
+  std::map<Point, int> unique_points_with_id;
+  for(const std::vector<Point>& face : point_patch)
+    for(const Point& p : face)
+      unique_points_with_id.insert(std::make_pair(p, 0));
+
+  out << "OFF\n";
+  out << unique_points_with_id.size() << " " << point_patch.size() << " 0\n";
+
+  int unique_id = 0;
+  for(auto& pp : unique_points_with_id)
+  {
+    out << pp.first << "\n";
+    pp.second = unique_id++;
+  }
+
+  for(const std::vector<Point>& face : point_patch)
+  {
+    out << face.size();
+    for(const Point& p : face)
+      out << " " << unique_points_with_id.at(p);
+    out << "\n";
+  }
+
+  out << std::endl;
+  out.close();
+}
+
+#endif // CGAL_PMP_REMOVE_SELF_INTERSECTION_OUTPUT
+
+// -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-
+
+// Hole filling can be influenced by setting a third point associated to an edge on the border of the hole.
+// This third point is supposed to represent how the mesh continues on the other side of the hole.
+// If that edge is a border edge, there is no third point (since the opposite face is the null face).
+// Similarly if the edge is an internal sharp edge, we don't really want to use the opposite face because
+// there is by definition a strong discontinuity and it might thus mislead the hole filling algorithm.
+//
+// Rather, we construct an artifical third point that is in the same plane as the face incident to `h`,
+// defined as the third point of the imaginary equilateral triangle incident to opp(h, tmesh)
+template <typename TriangleMesh, typename VertexPointMap, typename GeomTraits>
+typename boost::property_traits<VertexPointMap>::value_type
+construct_artificial_third_point(const typename boost::graph_traits<TriangleMesh>::halfedge_descriptor h,
+                                 const TriangleMesh& tmesh,
+                                 const VertexPointMap vpm,
+                                 const GeomTraits& gt)
+{
+  typedef typename GeomTraits::FT                                           FT;
+  typedef typename boost::property_traits<VertexPointMap>::value_type       Point;
+  typedef typename boost::property_traits<VertexPointMap>::reference        Point_ref;
+  typedef typename GeomTraits::Vector_3                                     Vector;
+
+  const Point_ref p1 = get(vpm, source(h, tmesh));
+  const Point_ref p2 = get(vpm, target(h, tmesh));
+  const Point_ref opp_p = get(vpm, target(next(h, tmesh), tmesh));
+
+  // sqrt(3)/2 to have an equilateral triangle with p1, p2, and third_point
+  const FT dist = 0.5 * CGAL::sqrt(3.) * CGAL::approximate_sqrt(gt.compute_squared_distance_3_object()(p1, p2));
+
+  const Vector ve1 = gt.construct_vector_3_object()(p1, p2);
+  const Vector ve2 = gt.construct_vector_3_object()(p1, opp_p);
+
+  // gram schmidt
+  const FT e1e2_sp = gt.compute_scalar_product_3_object()(ve1, ve2);
+  Vector orthogonalized_ve2 = gt.construct_sum_of_vectors_3_object()(
+                                ve2, gt.construct_scaled_vector_3_object()(ve1, - e1e2_sp));
+  Polygon_mesh_processing::internal::normalize(orthogonalized_ve2, gt);
+
+  const Point mid_p1p2 = gt.construct_midpoint_3_object()(p1, p2);
+  const Point third_p = gt.construct_translated_point_3_object()(
+                          mid_p1p2, gt.construct_scaled_vector_3_object()(orthogonalized_ve2, -dist));
+
+  return third_p;
+}
+
+template <typename TriangleMesh, typename VertexPointMap, typename Point, typename GeomTraits>
+bool construct_tentative_hole_patch(std::vector<typename boost::graph_traits<TriangleMesh>::vertex_descriptor>& cc_border_vertices,
+                                    std::set<typename boost::graph_traits<TriangleMesh>::vertex_descriptor>& cc_interior_vertices,
+                                    std::set<typename boost::graph_traits<TriangleMesh>::edge_descriptor>& cc_interior_edges,
+                                    const std::vector<Point>& hole_points,
+                                    const std::vector<Point>& third_points,
+                                    const std::vector<typename boost::graph_traits<TriangleMesh>::halfedge_descriptor>& cc_border_hedges,
+                                    const std::set<typename boost::graph_traits<TriangleMesh>::face_descriptor>& cc_faces,
+                                    std::vector<std::vector<Point> >& point_patch,
+                                    const TriangleMesh& tmesh,
+                                    VertexPointMap /*vpm*/,
+                                    const GeomTraits& /*gt*/)
+{
+  CGAL_static_assertion((std::is_same<typename boost::property_traits<VertexPointMap>::value_type, Point>::value));
+
+  typedef typename boost::graph_traits<TriangleMesh>::halfedge_descriptor   halfedge_descriptor;
+  typedef typename boost::graph_traits<TriangleMesh>::face_descriptor       face_descriptor;
+
+  typedef CGAL::Triple<int, int, int>                                       Face_indices;
+
+  CGAL_assertion(cc_border_hedges.size() == cc_border_hedges.size());
+  CGAL_assertion(hole_points.size() == third_points.size());
+
+  // Collect vertices and edges inside the current selection cc: first collect all vertices and
+  // edges incident to the faces to remove...
+  for(const face_descriptor f : cc_faces)
+  {
+    for(halfedge_descriptor h : halfedges_around_face(halfedge(f, tmesh), tmesh))
+    {
+      if(halfedge(target(h, tmesh), tmesh) == h) // limit the number of insertions
+        cc_interior_vertices.insert(target(h, tmesh));
+
+      cc_interior_edges.insert(edge(h, tmesh));
+    }
+  }
+
+  // ... and then remove those on the boundary
+  for(halfedge_descriptor h : cc_border_hedges)
+  {
+    cc_interior_vertices.erase(target(h, tmesh));
+    cc_interior_edges.erase(edge(h, tmesh));
+  }
+
+  // try to triangulate the hole using default parameters
+  // (using Delaunay search space if CGAL_HOLE_FILLING_DO_NOT_USE_DT3 is not defined)
+  std::vector<Face_indices> hole_faces;
+  if(hole_points.size() > 3)
+    triangulate_hole_polyline(hole_points, third_points, std::back_inserter(hole_faces));
+  else
+    hole_faces.emplace_back(0, 1, 2); // trivial hole filling
+
+  if(hole_faces.empty())
+  {
+#ifndef CGAL_HOLE_FILLING_DO_NOT_USE_DT3
+ #ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+    std::cout << "  DEBUG: Failed to fill a hole using Delaunay search space.\n";
+ #endif
+
+    triangulate_hole_polyline(hole_points, third_points, std::back_inserter(hole_faces),
+                              parameters::use_delaunay_triangulation(false));
+#endif
+    if(hole_faces.empty())
+    {
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+      std::cout << "  DEBUG: Failed to fill a hole using the whole search space.\n";
+#endif
+      return false;
+    }
+  }
+
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_OUTPUT
+  std::cout << hole_faces.size() << " faces in the patch" << std::endl;
+  std::vector<std::vector<Point> > to_dump;
+  for(const Face_indices& face : hole_faces)
+  {
+    to_dump.emplace_back(std::initializer_list<Point>{hole_points[face.first],
+                                                      hole_points[face.second],
+                                                      hole_points[face.third]});
+  }
+
+  CGAL_assertion(to_dump.size() == hole_faces.size());
+
+  static int hole_id = 0;
+  std::stringstream oss;
+  oss << "results/tentative_hole_" << hole_id++ << ".off" << std::ends;
+  const std::string filename = oss.str().c_str();
+
+  dump_tentative_hole(to_dump, filename);
+#endif
+
+  // make sure that the hole filling is valid, we check that no
+  // edge already in the mesh is present in hole_faces.
+  bool non_manifold_edge_found = false;
+  for(const Face_indices& triangle : hole_faces)
+  {
+    std::array<int, 6> edges = make_array(triangle.first, triangle.second,
+                                          triangle.second, triangle.third,
+                                          triangle.third, triangle.first);
+    for(int k=0; k<3; ++k)
+    {
+      const int vi = edges[2*k], vj = edges[2*k+1];
+
+      // ignore boundary edges
+      if(vi+1 == vj || (vj == 0 && static_cast<std::size_t>(vi) == cc_border_vertices.size()-1))
+        continue;
+
+      halfedge_descriptor h = halfedge(cc_border_vertices[vi], cc_border_vertices[vj], tmesh).first;
+      if(h != boost::graph_traits<TriangleMesh>::null_halfedge() &&
+         cc_interior_edges.count(edge(h, tmesh)) == 0)
+      {
+        non_manifold_edge_found = true;
+        break;
+      }
+    }
+
+    if(non_manifold_edge_found)
+      break;
+  }
+
+  if(non_manifold_edge_found)
+  {
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+    std::cout << "  DEBUG: Triangulation produced is non-manifold when plugged into the mesh.\n";
+#endif
+
+    return false;
+  }
+
+  point_patch.reserve(point_patch.size() + hole_faces.size());
+  for(const Face_indices& face : hole_faces)
+  {
+    point_patch.emplace_back(std::initializer_list<Point>{hole_points[face.first],
+                                                          hole_points[face.second],
+                                                          hole_points[face.third]});
+  }
+
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+  std::cout << "  DEBUG: Found acceptable hole-filling patch.\n";
+#endif
+
+  return true;
+}
+
+// This function constructs the ranges `hole_points` and `third_points`. Note that for a sub-hole,
+// these two ranges are constructed in another function because we don't want to set 'third_points'
+// for edges that are on the border of the sub-hole but not on the border of the (full) hole.
+template <typename TriangleMesh, typename VertexPointMap, typename GeomTraits>
+bool construct_tentative_hole_patch(std::vector<typename boost::graph_traits<TriangleMesh>::vertex_descriptor>& cc_border_vertices,
+                                    std::set<typename boost::graph_traits<TriangleMesh>::vertex_descriptor>& cc_interior_vertices,
+                                    std::set<typename boost::graph_traits<TriangleMesh>::edge_descriptor>& cc_interior_edges,
+                                    const std::vector<typename boost::graph_traits<TriangleMesh>::halfedge_descriptor>& cc_border_hedges,
+                                    const std::set<typename boost::graph_traits<TriangleMesh>::face_descriptor>& cc_faces,
+                                    std::vector<std::vector<typename boost::property_traits<VertexPointMap>::value_type> >& patch,
+                                    const TriangleMesh& tmesh,
+                                    VertexPointMap vpm,
+                                    const GeomTraits& gt)
+{
+  typedef typename boost::graph_traits<TriangleMesh>::vertex_descriptor     vertex_descriptor;
+  typedef typename boost::graph_traits<TriangleMesh>::halfedge_descriptor   halfedge_descriptor;
+
+  typedef typename boost::property_traits<VertexPointMap>::value_type       Point;
+
+  std::vector<Point> hole_points, third_points;
+  hole_points.reserve(cc_border_hedges.size());
+  third_points.reserve(cc_border_hedges.size());
+
+  for(const halfedge_descriptor h : cc_border_hedges)
+  {
+    const vertex_descriptor v = source(h, tmesh);
+    hole_points.push_back(get(vpm, v));
+    cc_border_vertices.push_back(v);
+
+    CGAL_assertion(!is_border(h, tmesh));
+
+    if(is_border_edge(h, tmesh))
+      third_points.push_back(construct_artificial_third_point(h, tmesh, vpm, gt));
+    else
+      third_points.push_back(get(vpm, target(next(opposite(h, tmesh), tmesh), tmesh)));
+  }
+
+  CGAL_postcondition(hole_points.size() >= 3);
+
+  return construct_tentative_hole_patch(cc_border_vertices, cc_interior_vertices, cc_interior_edges,
+                                        hole_points, third_points, cc_border_hedges, cc_faces,
+                                        patch, tmesh, vpm, gt);
+}
+
+// In that overload, we don't know the border of the patch because the face range is a sub-region
+// of the hole. We also construct `hole_points` and `third_points`, but with no third point for internal
+// sharp edges because a local self-intersection is usually caused by folding and thus we do not want
+// a third point resulting from folding to constrain the way we fill the hole in the wrong way.
+template <typename TriangleMesh, typename VertexPointMap, typename GeomTraits>
+bool construct_tentative_sub_hole_patch(std::vector<std::vector<typename boost::property_traits<VertexPointMap>::value_type> >& patch,
+                                        const std::set<typename boost::graph_traits<TriangleMesh>::face_descriptor>& sub_cc_faces,
+                                        const std::set<typename boost::graph_traits<TriangleMesh>::face_descriptor>& cc_faces,
+                                        TriangleMesh& tmesh,
+                                        VertexPointMap vpm,
+                                        const GeomTraits& gt)
+{
+  typedef typename boost::graph_traits<TriangleMesh>::vertex_descriptor     vertex_descriptor;
+  typedef typename boost::graph_traits<TriangleMesh>::halfedge_descriptor   halfedge_descriptor;
+  typedef typename boost::graph_traits<TriangleMesh>::edge_descriptor       edge_descriptor;
+  typedef typename boost::graph_traits<TriangleMesh>::face_descriptor       face_descriptor;
+
+  typedef typename boost::property_traits<VertexPointMap>::value_type       Point;
+
+  // Collect halfedges on the boundary of the region to be selected
+  // (pointing inside the domain to be remeshed)
+  std::set<halfedge_descriptor> internal_hedges;
+  std::vector<halfedge_descriptor> cc_border_hedges;
+  for(const face_descriptor fd : sub_cc_faces)
+  {
+    halfedge_descriptor h = halfedge(fd, tmesh);
+    for(int i=0; i<3;++i)
+    {
+      if(is_border(opposite(h, tmesh), tmesh))
+      {
+         cc_border_hedges.push_back(h);
+      }
+      else
+      {
+        const face_descriptor opp_f = face(opposite(h, tmesh), tmesh);
+        if(sub_cc_faces.count(opp_f) == 0)
+        {
+          cc_border_hedges.push_back(h);
+          if(cc_faces.count(opp_f) != 0)
+            internal_hedges.insert(h);
+        }
+      }
+
+      h = next(h, tmesh);
+    }
+  }
+
+  // Sort halfedges so that they describe the sequence of halfedges of the hole to be made
+  if(!order_border_halfedge_range(cc_border_hedges, tmesh))
+  {
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+    std::cout << "  DEBUG: More than one border in sub-hole. Not currently handled." << std::endl;
+#endif
+
+    return false;
+  }
+
+  // @todo we don't care about those sets, so instead there could be a system of output iterators
+  // in construct_tentative_hole_patch() instead (and here would be emptyset iterators).
+  std::set<vertex_descriptor> cc_interior_vertices;
+  std::set<edge_descriptor> cc_interior_edges;
+
+  std::vector<vertex_descriptor> cc_border_vertices;
+  cc_border_vertices.reserve(cc_border_hedges.size());
+
+  std::vector<Point> hole_points, third_points;
+  hole_points.reserve(cc_border_hedges.size());
+  third_points.reserve(cc_border_hedges.size());
+
+  for(const halfedge_descriptor h : cc_border_hedges)
+  {
+    const vertex_descriptor v = source(h, tmesh);
+    hole_points.push_back(get(vpm, v));
+    cc_border_vertices.push_back(v);
+
+    CGAL_assertion(!is_border(h, tmesh));
+
+    if(internal_hedges.count(h) == 0 && // `h` is on the border of the full CC
+       !is_border_edge(h, tmesh))
+    {
+      third_points.push_back(get(vpm, target(next(opposite(h, tmesh), tmesh), tmesh)));
+    }
+    else // `h` is on the border of the sub CC but not on the border of the full CC
+    {
+      const Point tp = construct_artificial_third_point(h, tmesh, vpm, gt);
+      third_points.push_back(tp);
+    }
+  }
+
+  return construct_tentative_hole_patch(cc_border_vertices, cc_interior_vertices, cc_interior_edges,
+                                        hole_points, third_points, cc_border_hedges, sub_cc_faces,
+                                        patch, tmesh, vpm, gt);
+}
+
+// -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-
+
+// Patch is not valid if:
+// - we insert the same face more than once
+// - insert (geometric) non-manifold edges
+template <typename TriangleMesh, typename Point>
+bool check_patch_sanity(const std::vector<std::vector<Point> >& patch)
+{
+  std::set<std::set<Point> > unique_faces;
+  std::map<std::set<Point>, int> unique_edges;
+
+  for(const std::vector<Point>& face : patch)
+  {
+    if(!unique_faces.emplace(face.begin(), face.end()).second) // this face had already been found
+      return false;
+
+    int i = (unique_edges.insert(std::make_pair(std::set<Point> { face[0], face[1] }, 0)).first->second)++;
+    if(i == 2) // non-manifold edge
+      return false;
+
+    i = (unique_edges.insert(std::make_pair(std::set<Point> { face[1], face[2] }, 0)).first->second)++;
+    if(i == 2) // non-manifold edge
+      return false;
+
+    i = (unique_edges.insert(std::make_pair(std::set<Point> { face[2], face[0] }, 0)).first->second)++;
+    if(i == 2) // non-manifold edge
+      return false;
+  }
+
+  // Check for self-intersections
+  // Don't know anything better than just making a mesh out of the soup for now...
+  std::vector<Point> points;
+  std::vector<std::vector<std::size_t> > faces;
+  std::map<Point, std::size_t> ids;
+
+  std::size_t c = 0;
+  for(const std::vector<Point>& face : patch)
+  {
+    std::vector<std::size_t> ps_f;
+    for(const Point& pt : face)
+    {
+      std::size_t id = c;
+      std::pair<typename std::map<Point, std::size_t>::iterator, bool> is_insert_successful =
+        ids.insert(std::make_pair(pt, c));
+      if(is_insert_successful.second) // first time we've seen that point
+      {
+        ++c;
+        points.push_back(pt);
+      }
+      else // already seen that point
+      {
+        id = is_insert_successful.first->second;
+      }
+
+      CGAL_assertion(id < points.size());
+      ps_f.push_back(id);
+    }
+
+    faces.push_back(ps_f);
+  }
+
+  TriangleMesh patch_mesh;
+  if(is_polygon_soup_a_polygon_mesh(faces))
+    polygon_soup_to_polygon_mesh(points, faces, patch_mesh);
+  else
+    return false;
+
+  if(does_self_intersect(patch_mesh))
+  {
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+    std::cout << "  DEBUG: Tentative patch has self-intersections." << std::endl;
+#endif
+
+    return false;
+  }
+
+  return true;
+}
+
+// This function is only called when the hole is NOT subdivided into smaller holes
+template <typename TriangleMesh, typename VertexPointMap, typename GeomTraits>
+bool fill_hole(std::vector<typename boost::graph_traits<TriangleMesh>::halfedge_descriptor>& cc_border_hedges,
+               std::set<typename boost::graph_traits<TriangleMesh>::face_descriptor>& cc_faces,
+               std::set<typename boost::graph_traits<TriangleMesh>::face_descriptor>& working_face_range,
+               TriangleMesh& tmesh,
+               VertexPointMap vpm,
+               const GeomTraits& gt)
+{
+  typedef typename boost::graph_traits<TriangleMesh>::vertex_descriptor     vertex_descriptor;
+  typedef typename boost::graph_traits<TriangleMesh>::edge_descriptor       edge_descriptor;
+  typedef typename boost::graph_traits<TriangleMesh>::face_descriptor       face_descriptor;
+
+  typedef typename boost::property_traits<VertexPointMap>::value_type       Point;
+
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+  std::cout << "  DEBUG: Attempting hole-filling (no constraints), " << cc_faces.size() << " faces\n";
+#endif
+
+  if(!order_border_halfedge_range(cc_border_hedges, tmesh))
+  {
+    CGAL_assertion(false); // we shouldn't fail to orient the boundary cycle of the complete hole
+    return false;
+  }
+
+  std::set<vertex_descriptor> cc_interior_vertices;
+  std::set<edge_descriptor> cc_interior_edges;
+
+  std::vector<vertex_descriptor> cc_border_vertices;
+  cc_border_vertices.reserve(cc_border_hedges.size());
+
+  std::vector<std::vector<Point> > patch;
+  if(!construct_tentative_hole_patch(cc_border_vertices, cc_interior_vertices, cc_interior_edges,
+                                     cc_border_hedges, cc_faces, patch, tmesh, vpm, gt) ||
+     !check_patch_sanity<TriangleMesh>(patch))
+  {
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+    std::cout << "  DEBUG: Failed to find acceptable hole patch\n";
+#endif
+
+    return false;
+  }
+
+  // Could renew the range directly within the patch replacement function
+  // to avoid erasing and re-adding the same face
+  for(const face_descriptor f : cc_faces)
+    working_face_range.erase(f);
+
+  // Plug the new triangles in the mesh, reusing previous edges and faces
+  replace_faces_with_patch(cc_border_vertices, cc_interior_vertices,
+                           cc_border_hedges, cc_interior_edges,
+                           cc_faces, patch, tmesh, vpm,
+                           std::inserter(working_face_range, working_face_range.end()));
+
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_OUTPUT
+  static int filed_hole_id = 0;
+  std::stringstream oss;
+  oss << "results/filled_basic_" << filed_hole_id++ << ".off" << std::ends;
+  std::ofstream(oss.str().c_str()) << std::setprecision(17) << tmesh;
+#endif
+
+  CGAL_postcondition(is_valid_polygon_mesh(tmesh));
+
+  return true;
+}
+
+// Same function as above but border of the hole is not known
+template <typename TriangleMesh, typename VertexPointMap, typename GeomTraits>
+bool fill_hole(std::set<typename boost::graph_traits<TriangleMesh>::face_descriptor>& cc_faces,
+               std::set<typename boost::graph_traits<TriangleMesh>::face_descriptor>& working_face_range,
+               TriangleMesh& tmesh,
+               VertexPointMap vpm,
+               const GeomTraits& gt)
+{
+  typedef typename boost::graph_traits<TriangleMesh>::halfedge_descriptor   halfedge_descriptor;
+  typedef typename boost::graph_traits<TriangleMesh>::face_descriptor       face_descriptor;
+
+  std::vector<halfedge_descriptor> cc_border_hedges;
+  for(face_descriptor fd : cc_faces)
+  {
+    halfedge_descriptor h = halfedge(fd, tmesh);
+    for(int i=0; i<3; ++i)
+    {
+      if(is_border(opposite(h, tmesh), tmesh) || cc_faces.count(face(opposite(h, tmesh), tmesh)) == 0)
+        cc_border_hedges.push_back(h);
+
+      h = next(h, tmesh);
+    }
+  }
+
+  if(order_border_halfedge_range(cc_border_hedges, tmesh))
+    return fill_hole(cc_border_hedges, cc_faces, working_face_range, tmesh, vpm, gt);
+  else
+    return false;
+}
+
+template <typename TriangleMesh, typename VertexPointMap, typename GeomTraits>
+bool fill_hole_with_constraints(std::vector<typename boost::graph_traits<TriangleMesh>::halfedge_descriptor>& cc_border_hedges,
+                                std::set<typename boost::graph_traits<TriangleMesh>::face_descriptor>& cc_faces,
+                                std::set<typename boost::graph_traits<TriangleMesh>::face_descriptor>& working_face_range,
+                                TriangleMesh& tmesh,
+                                const double dihedral_angle,
+                                const double weak_DA,
+                                VertexPointMap vpm,
+                                const GeomTraits& gt)
+{
+  typedef typename boost::graph_traits<TriangleMesh>::face_descriptor       face_descriptor;
+
+  typedef typename boost::property_traits<VertexPointMap>::value_type       Point;
+
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+  std::cout << "  DEBUG: Attempting local hole-filling with constrained sharp edges..." << std::endl;
+#endif
+
+  // If we are treating self intersections locally, first try to constrain sharp edges in the hole
+  typedef CGAL::dynamic_edge_property_t<bool>                                 Edge_property_tag;
+  typedef typename boost::property_map<TriangleMesh, Edge_property_tag>::type EIFMap;
+  EIFMap eif = get(Edge_property_tag(), tmesh);
+
+  constrain_sharp_and_border_edges(cc_faces, tmesh, eif, true /*constrain_sharp_edges*/, dihedral_angle, weak_DA, vpm, gt);
+
+  // Partition the hole using these constrained edges
+  std::set<face_descriptor> visited_faces;
+  std::vector<std::vector<Point> > patch;
+
+  int cc_counter = 0;
+  for(face_descriptor f : cc_faces)
+  {
+    if(!visited_faces.insert(f).second) // already visited that face
+      continue;
+
+    // gather the faces of the sub-hole
+    std::set<face_descriptor> sub_cc;
+    Polygon_mesh_processing::connected_component(f, tmesh, std::inserter(sub_cc, sub_cc.end()),
+                                                 CGAL::parameters::edge_is_constrained_map(eif));
+
+    visited_faces.insert(sub_cc.begin(), sub_cc.end());
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+    std::cout << "CC of size " << sub_cc.size() << " (total: " << cc_faces.size() << ")" << std::endl;
+#endif
+    ++cc_counter;
+
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_OUTPUT
+    dump_cc(sub_cc, tmesh, "results/current_cc.off");
+#endif
+
+    // The mesh is not modified, but 'patch' gets filled
+    if(!construct_tentative_sub_hole_patch(patch, sub_cc, cc_faces, tmesh, vpm, gt))
+    {
+      // Something went wrong while finding a potential cover for the a sub-hole --> use basic hole-filling
+      return fill_hole(cc_border_hedges, cc_faces, working_face_range, tmesh, vpm, gt);
+    }
+  }
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+  std::cout << cc_counter << " independent sub holes" << std::endl;
+#endif
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_OUTPUT
+  std::ofstream out("results/hole_fillers.off");
+  out.precision(17);
+
+  out << "OFF\n";
+  out << 3*patch.size() << " " << patch.size() << " 0\n";
+
+  for(const auto& f : patch)
+  {
+    for(const auto& pt : f)
+      out << pt << "\n";
+  }
+
+  int id = 0;
+  for(std::size_t i=0; i<patch.size(); ++i)
+  {
+    out << "3 " << id << " " << id+1 << " " << id+2 << "\n";
+    id += 3;
+  }
+  out.close();
+#endif
+
+  // We're assembling multiple patches so we could have the same face appearing multiple times...
+  if(!check_patch_sanity<TriangleMesh>(patch))
+  {
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+    std::cout << "Unhealthy patch, use base fill_hole" << std::endl;
+#endif
+    return fill_hole(cc_border_hedges, cc_faces, working_face_range, tmesh, vpm, gt);
+  }
+
+  // Plug the hole-filling patch in the mesh
+  std::set<face_descriptor> new_faces;
+  replace_faces_with_patch(cc_faces, patch, tmesh, vpm, std::inserter(new_faces, new_faces.end()));
+
+  // Otherwise it should have failed the sanity check
+  CGAL_assertion(!does_self_intersect(new_faces, tmesh, parameters::vertex_point_map(vpm)));
+
+  // Update working range with the new faces
+  for(const face_descriptor f : cc_faces)
+    working_face_range.erase(f);
+
+  working_face_range.insert(new_faces.begin(), new_faces.end());
+
+  return true;
+}
+
+template <class Box, class TM, class VPM, class GT, class OutputIterator>
+struct Strict_intersect_edges // "strict" as in "not sharing a vertex"
+{
+  typedef typename boost::graph_traits<TM>::halfedge_descriptor               halfedge_descriptor;
+  typedef typename GT::Segment_3                                              Segment;
+
+  mutable OutputIterator m_iterator;
+  const TM& m_tmesh;
+  const VPM m_vpmap;
+
+  typename GT::Construct_segment_3 m_construct_segment;
+  typename GT::Do_intersect_3 m_do_intersect;
+
+  Strict_intersect_edges(const TM& tmesh, VPM vpmap, const GT& gt, OutputIterator it)
+    :
+      m_iterator(it),
+      m_tmesh(tmesh),
+      m_vpmap(vpmap),
+      m_construct_segment(gt.construct_segment_3_object()),
+      m_do_intersect(gt.do_intersect_3_object())
+  {}
+
+  void operator()(const Box* b, const Box* c) const
+  {
+    const halfedge_descriptor h = b->info();
+    const halfedge_descriptor g = c->info();
+
+    if(source(h, m_tmesh) == target(g, m_tmesh) || target(h, m_tmesh) == source(g, m_tmesh))
+      return;
+
+    const Segment s1 = m_construct_segment(get(m_vpmap, source(h, m_tmesh)), get(m_vpmap, target(h, m_tmesh)));
+    const Segment s2 = m_construct_segment(get(m_vpmap, source(g, m_tmesh)), get(m_vpmap, target(g, m_tmesh)));
+
+    if(m_do_intersect(s1, s2))
+      *m_iterator++ = std::make_pair(b->info(), c->info());
+  }
+};
+
+template <typename TriangleMesh, typename VertexPointMap, typename GeomTraits>
+bool is_simple_3(const std::vector<typename boost::graph_traits<TriangleMesh>::halfedge_descriptor>& cc_border_hedges,
+                 const TriangleMesh& tmesh,
+                 VertexPointMap vpm,
+                 const GeomTraits& gt)
+{
+  typedef typename boost::graph_traits<TriangleMesh>::halfedge_descriptor                       halfedge_descriptor;
+
+  typedef typename boost::property_traits<VertexPointMap>::reference                            Point_ref;
+
+  typedef CGAL::Box_intersection_d::ID_FROM_BOX_ADDRESS                                         Box_policy;
+  typedef CGAL::Box_intersection_d::Box_with_info_d<double, 3, halfedge_descriptor, Box_policy> Box;
+
+  std::vector<Box> boxes;
+  boxes.reserve(cc_border_hedges.size());
+
+  for(halfedge_descriptor h : cc_border_hedges)
+  {
+    const Point_ref p = get(vpm, source(h, tmesh));
+    const Point_ref q = get(vpm, target(h, tmesh));
+    CGAL_assertion(!gt.equal_3_object()(p, q));
+
+    boxes.emplace_back(p.bbox() + q.bbox(), h);
+  }
+
+  // generate box pointers
+  std::vector<const Box*> box_ptr;
+  box_ptr.reserve(boxes.size());
+
+  for(Box& b : boxes)
+    box_ptr.push_back(&b);
+
+  typedef boost::function_output_iterator<CGAL::internal::Throw_at_output>          Throwing_output_iterator;
+  typedef internal::Strict_intersect_edges<Box, TriangleMesh, VertexPointMap,
+                                           GeomTraits, Throwing_output_iterator>    Throwing_filter;
+  Throwing_filter throwing_filter(tmesh, vpm, gt, Throwing_output_iterator());
+
+  try
+  {
+    const std::ptrdiff_t cutoff = 2000;
+    CGAL::box_self_intersection_d<Parallel_if_available_tag>(box_ptr.begin(), box_ptr.end(), throwing_filter, cutoff);
+  }
+  catch(CGAL::internal::Throw_at_output_exception&)
+  {
+    return false;
+  }
+
+  return true;
+}
+
+template <typename TriangleMesh, typename VertexPointMap, typename GeomTraits>
+bool remove_self_intersections_with_hole_filling(std::vector<typename boost::graph_traits<TriangleMesh>::halfedge_descriptor>& cc_border_hedges,
+                                                 std::set<typename boost::graph_traits<TriangleMesh>::face_descriptor>& cc_faces,
+                                                 std::set<typename boost::graph_traits<TriangleMesh>::face_descriptor>& working_face_range,
+                                                 TriangleMesh& tmesh,
+                                                 bool local_self_intersection_removal,
+                                                 const double strong_dihedral_angle,
+                                                 const double weak_dihedral_angle,
+                                                 VertexPointMap vpm,
+                                                 const GeomTraits& gt)
+{
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_OUTPUT
+  std::ofstream out("results/zone_border.polylines.txt");
+  out << std::setprecision(17);
+  for(const auto& h : cc_border_hedges)
+    out << "2 " << tmesh.point(source(h, tmesh)) << " " << tmesh.point(target(h, tmesh)) << std::endl;
+  out.close();
+#endif
+
+  if(!is_simple_3(cc_border_hedges, tmesh, vpm, gt))
+  {
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+    std::cout << "Hole filling cannot handle non-simple border" << std::endl;
+#endif
+    return false;
+  }
+
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTIONS_NO_CONSTRAINTS_IN_HOLE_FILLING
+  // Do not try to impose sharp edge constraints if we are not doing local-only self intersections removal
+  local_self_intersection_removal = false;
+#endif
+
+  bool success = false;
+  if(local_self_intersection_removal)
+  {
+    success = fill_hole_with_constraints(cc_border_hedges, cc_faces, working_face_range, tmesh,
+                                         strong_dihedral_angle, weak_dihedral_angle, vpm, gt);
+  }
+  else
+  {
+    success = fill_hole(cc_border_hedges, cc_faces, working_face_range, tmesh, vpm, gt);
+  }
+
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+  if(success)
+  {
+    if(local_self_intersection_removal)
+      ++self_intersections_solved_by_constrained_hole_filling;
+    else
+      ++self_intersections_solved_by_unconstrained_hole_filling;
+  }
+#endif
+
+  return success;
+}
+
+// the parameter `step` controls how many extra layers of faces we take around the range `faces_to_remove`
+template <typename TriangleMesh, typename VertexPointMap, typename GeomTraits>
+std::pair<bool, bool>
+remove_self_intersections_one_step(std::set<typename boost::graph_traits<TriangleMesh>::face_descriptor>& faces_to_remove,
+                                   std::set<typename boost::graph_traits<TriangleMesh>::face_descriptor>& working_face_range,
+                                   TriangleMesh& tmesh,
+                                   const int step,
+                                   const bool preserve_genus,
+                                   const bool only_treat_self_intersections_locally,
+                                   const double strong_dihedral_angle,
+                                   const double weak_dihedral_angle,
+                                   VertexPointMap vpm,
+                                   const GeomTraits& gt)
+{
+  typedef boost::graph_traits<TriangleMesh>                               graph_traits;
+  typedef typename graph_traits::vertex_descriptor                        vertex_descriptor;
+  typedef typename graph_traits::halfedge_descriptor                      halfedge_descriptor;
+  typedef typename graph_traits::face_descriptor                          face_descriptor;
+
+  std::set<face_descriptor> faces_to_remove_copy = faces_to_remove;
+
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+  std::cout << "##### running remove_self_intersections_one_step, step " << step
+            << " with " << faces_to_remove.size() << " intersecting faces\n";
+#endif
+
+  bool something_was_done = false; // indicates if a region was successfully remeshed
+  bool all_fixed = true; // indicates if all removal went well
+  // indicates if a removal was not possible because the region handle has
+  // some boundary cycle of halfedges
+  bool topology_issue = false;
+
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+  std::cout << "  DEBUG: is_valid in one_step(tmesh)? ";
+  std::cout.flush();
+
+  unsolved_self_intersections = 0;
+#endif
+
+  CGAL_precondition(is_valid_polygon_mesh(tmesh));
+
+  while(!faces_to_remove.empty())
+  {
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+    std::cout << "  DEBUG: --------------- " << faces_to_remove.size() << " faces to remove (step: " << step << ")\n";
+#endif
+
+    // Process a connected component of faces to remove.
+    // collect all the faces from the connected component
+    std::set<face_descriptor> cc_faces;
+    std::vector<face_descriptor> queue(1, *faces_to_remove.begin()); // temporary queue
+    cc_faces.insert(queue.back());
+    while(!queue.empty())
+    {
+      face_descriptor top = queue.back();
+      queue.pop_back();
+      halfedge_descriptor h = halfedge(top, tmesh);
+      for(int i=0; i<3; ++i)
+      {
+        face_descriptor adjacent_face = face(opposite(h, tmesh), tmesh);
+        if(adjacent_face!=boost::graph_traits<TriangleMesh>::null_face())
+        {
+          if(faces_to_remove.count(adjacent_face) != 0 && cc_faces.insert(adjacent_face).second)
+            queue.push_back(adjacent_face);
+        }
+
+        h = next(h, tmesh);
+      }
+    }
+
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+    std::cout << "  DEBUG: " << cc_faces.size() << " faces in CC\n";
+    std::cout << "  DEBUG: first face: " << get(vpm, source(halfedge(*(cc_faces.begin()), tmesh), tmesh)) << " "
+              << get(vpm, target(halfedge(*(cc_faces.begin()), tmesh), tmesh)) << " "
+              << get(vpm, target(next(halfedge(*(cc_faces.begin()), tmesh), tmesh), tmesh)) << "\n";
+#endif
+
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_OUTPUT
+    static int ini_cc_id = 0;
+    std::stringstream ini_oss, mesh_oss;
+    std::cout << "Output initial CC #" << ini_cc_id << std::endl;
+    ini_oss << "results/initial_cc_" << ini_cc_id << ".off" << std::ends;
+    dump_cc(cc_faces, tmesh, ini_oss.str().c_str());
+
+    mesh_oss << "results/mesh_at_cc_ " << ini_cc_id++ << ".off" << std::ends;
+    std::ofstream mout(mesh_oss.str().c_str());
+    mout << std::setprecision(17) << tmesh;
+    mout.close();
+#endif
+
+    // expand the region to be filled
+    if(step > 0)
+    {
+      expand_face_selection(cc_faces, tmesh, step,
+                            make_boolean_property_map(cc_faces),
+                            Emptyset_iterator());
+    }
+
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_OUTPUT
+    static int exp_cc_id = 0;
+    std::stringstream oss;
+    std::cout << "Output expanded CC #" << exp_cc_id << std::endl;
+    oss << "results/expanded_cc_" << exp_cc_id++ << ".off" << std::ends;
+    dump_cc(cc_faces, tmesh, oss.str().c_str());
+#endif
+
+    // try to compactify the selection region by also selecting all the faces included
+    // in the bounding box of the initial selection
+    std::vector<halfedge_descriptor> stack_for_expension;
+    Bbox_3 bb;
+    for(face_descriptor fd : cc_faces)
+    {
+      for(halfedge_descriptor h : halfedges_around_face(halfedge(fd, tmesh), tmesh))
+      {
+        bb += get(vpm, target(h, tmesh)).bbox();
+        face_descriptor nf = face(opposite(h, tmesh), tmesh);
+        if(nf != boost::graph_traits<TriangleMesh>::null_face() && cc_faces.count(nf) == 0)
+        {
+          stack_for_expension.push_back(opposite(h, tmesh));
+        }
+      }
+    }
+
+    while(!stack_for_expension.empty())
+    {
+      halfedge_descriptor h = stack_for_expension.back();
+      stack_for_expension.pop_back();
+      if(cc_faces.count(face(h, tmesh)) == 1)
+        continue;
+
+      if(do_overlap(bb, get(vpm, target(next(h, tmesh), tmesh)).bbox()))
+      {
+        cc_faces.insert(face(h, tmesh));
+        halfedge_descriptor candidate = opposite(next(h, tmesh), tmesh);
+        if(face(candidate, tmesh) != boost::graph_traits<TriangleMesh>::null_face())
+          stack_for_expension.push_back(candidate);
+
+        candidate = opposite(prev(h, tmesh), tmesh);
+        if(face(candidate, tmesh) != boost::graph_traits<TriangleMesh>::null_face())
+          stack_for_expension.push_back(candidate);
+      }
+    }
+
+    if(only_treat_self_intersections_locally)
+    {
+      if(!does_self_intersect(cc_faces, tmesh, parameters::vertex_point_map(vpm).geom_traits(gt)))
+      {
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+        std::cout << "  DEBUG: No self-intersection in CC\n";
+#endif
+
+        for(const face_descriptor f : cc_faces)
+          faces_to_remove.erase(f);
+
+        continue;
+      }
+    }
+
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+    std::cout << "  DEBUG: " << cc_faces.size() << " faces in expanded CC\n";
+#endif
+
+    // remove faces from the set to process
+    for(const face_descriptor f : cc_faces)
+      faces_to_remove.erase(f);
+
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_OUTPUT
+    std::stringstream ex_oss;
+    std::cout << "Output FULLY expanded CC #" << exp_cc_id-1 << std::endl;
+    ex_oss << "results/fully_expanded_cc_" << exp_cc_id-1 << ".off" << std::ends;
+    dump_cc(cc_faces, tmesh, ex_oss.str().c_str());
+#endif
+
+    //Check for non-manifold vertices in the selection and remove them by selecting all incident faces:
+    //  extract the set of halfedges that is on the boundary of the holes to be
+    //  made. In addition, we make sure no hole to be created contains a vertex
+    //  visited more than once along a hole border (pinched surface)
+    //  We save the size of boundary_hedges to make sur halfedges added
+    //  from non_filled_hole are not removed.
+    bool non_manifold_vertex_remaining_in_selection = false;
+    do
+    {
+      bool non_manifold_vertex_removed = false; //here non-manifold is for the 1D polyline
+      std::vector<halfedge_descriptor> boundary_hedges;
+      for(face_descriptor fh : cc_faces)
+      {
+        halfedge_descriptor h = halfedge(fh, tmesh);
+        for(int i=0; i<3; ++i)
+        {
+          if(is_border(opposite(h, tmesh), tmesh) || cc_faces.count(face(opposite(h, tmesh), tmesh)) == 0)
+            boundary_hedges.push_back(h);
+
+          h = next(h, tmesh);
+        }
+      }
+
+      // detect vertices visited more than once along
+      // a hole border. We then remove all faces incident
+      // to such a vertex to force the removal of the vertex.
+      // Actually even if two holes are sharing a vertex, this
+      // vertex will be removed. It is not needed but since
+      // we do not yet have one halfedge per hole it is simpler
+      // and does not harm
+      std::set<vertex_descriptor> border_vertices;
+      for(halfedge_descriptor h : boundary_hedges)
+      {
+        if(!border_vertices.insert(target(h, tmesh)).second)
+        {
+          bool any_face_added = false;
+          for(halfedge_descriptor hh : halfedges_around_target(h, tmesh))
+          {
+            if(!is_border(hh, tmesh))
+            {
+              // add the face to the current selection
+              any_face_added |= cc_faces.insert(face(hh, tmesh)).second;
+              faces_to_remove.erase(face(hh, tmesh));
+            }
+          }
+
+          if(any_face_added)
+            non_manifold_vertex_removed = true;
+          else
+            non_manifold_vertex_remaining_in_selection = true;
+        }
+      }
+
+      if(!non_manifold_vertex_removed)
+        break;
+    }
+    while(true);
+
+    if(preserve_genus && non_manifold_vertex_remaining_in_selection)
+    {
+      topology_issue = true;
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+      std::cout << "  DEBUG: CC not handled due to the presence at least one non-manifold vertex\n";
+
+      ++unsolved_self_intersections;
+#endif
+
+      continue; // cannot replace a patch containing a nm vertex by a disk
+    }
+
+    // before running this function if preserve_genus=false, we duplicated
+    // all of them
+    CGAL_assertion(!non_manifold_vertex_remaining_in_selection);
+
+    // Collect halfedges on the boundary of the region to be selected
+    // (pointing inside the domain to be remeshed)
+    std::vector<halfedge_descriptor> cc_border_hedges;
+    for(face_descriptor fd : cc_faces)
+    {
+      halfedge_descriptor h = halfedge(fd, tmesh);
+      for(int i=0; i<3; ++i)
+      {
+        if(is_border(opposite(h, tmesh), tmesh) || cc_faces.count(face(opposite(h, tmesh), tmesh)) == 0)
+          cc_border_hedges.push_back(h);
+
+        h = next(h, tmesh);
+      }
+    }
+
+    if(cc_faces.size() == 1) // it is a triangle nothing better can be done
+    {
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+     ++unsolved_self_intersections;
+#endif
+      continue;
+    }
+
+    working_face_range.insert(cc_faces.begin(), cc_faces.end());
+
+    // Now, we have a proper selection that we can work on.
+
+#ifndef CGAL_PMP_REMOVE_SELF_INTERSECTIONS_NO_SMOOTHING
+    // First, try to smooth if we only care about local self-intersections
+    // Two different approaches:
+    // - First, try to constrain edges that are in the zone to smooth and whose dihedral angle is large,
+    //   but not too large (we don't want to constrain edges that are created by foldings)
+    // - If that fails, try to smooth without any constraints, but make sure that the deviation from
+    //   the first zone is small
+    //
+    // If smoothing fails, the face patch is restored to its pre-smoothing state.
+    //
+    // Note that there is no need to update the working range because smoothing doesn`t change
+    // the number of faces (and old faces are re-used).
+    bool fixed_by_smoothing = false;
+
+    if(only_treat_self_intersections_locally)
+    {
+      fixed_by_smoothing = remove_self_intersections_with_smoothing(cc_faces, tmesh, true,
+                                                                    strong_dihedral_angle,
+                                                                    weak_dihedral_angle, vpm, gt);
+
+      if(!fixed_by_smoothing) // try again, but without constraining sharp edges
+      {
+ #ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+        std::cout << "  DEBUG: Could not be solved via smoothing with constraints\n";
+ #endif
+
+        fixed_by_smoothing = remove_self_intersections_with_smoothing(cc_faces, tmesh, false,
+                                                                      strong_dihedral_angle,
+                                                                      weak_dihedral_angle, vpm, gt);
+      }
+    }
+
+    if(fixed_by_smoothing)
+    {
+ #ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+      std::cout << "  DEBUG: Solved with smoothing!\n";
+ #endif
+
+      something_was_done = true;
+      continue;
+    }
+ #ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+    else
+    {
+      std::cout << "  DEBUG: Could not be solved via smoothing\n";
+    }
+ #endif
+#endif // ndef CGAL_PMP_REMOVE_SELF_INTERSECTIONS_NO_SMOOTHING
+
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+    std::cout << "  DEBUG: Trying hole-filling based approach\n";
+#endif
+
+    if(!is_selection_a_topological_disk(cc_faces, tmesh))
+    {
+      // check if the selection contains cycles of border halfedges
+      bool only_border_edges = true;
+      std::set<halfedge_descriptor> mesh_border_hedge;
+
+      for(halfedge_descriptor h : cc_border_hedges)
+      {
+        if(!is_border(opposite(h, tmesh), tmesh))
+          only_border_edges = false;
+        else
+          mesh_border_hedge.insert(opposite(h, tmesh));
+      }
+
+      int nb_cycles = 0;
+      while(!mesh_border_hedge.empty())
+      {
+        // we must count the number of cycle of boundary edges
+        halfedge_descriptor h_b = *mesh_border_hedge.begin(), h=h_b;
+        mesh_border_hedge.erase(mesh_border_hedge.begin());
+        do
+        {
+          h = next(h, tmesh);
+          if(h == h_b)
+          {
+            // found a cycle
+            ++nb_cycles;
+            break;
+          }
+          else
+          {
+            typename std::set<halfedge_descriptor>::iterator it = mesh_border_hedge.find(h);
+            if(it == mesh_border_hedge.end())
+              break; // not a cycle
+
+            mesh_border_hedge.erase(it);
+          }
+        }
+        while(true);
+      }
+
+      if(nb_cycles > (only_border_edges ? 1 : 0))
+      {
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+        std::cout << "  DEBUG: CC not handled due to the presence of "
+                  << nb_cycles << " of boundary edges\n";
+     ++unsolved_self_intersections;
+#endif
+
+        topology_issue = true;
+        continue;
+      }
+      else
+      {
+        if(preserve_genus)
+        {
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+          std::cout << "  DEBUG: CC not handled because it is not a topological disk (preserve_genus=true)\n";
+          ++unsolved_self_intersections;
+#endif
+
+          all_fixed = false;
+          continue;
+        }
+
+        // count the number of cycles of halfedges of the boundary
+        std::map<vertex_descriptor, vertex_descriptor> bhs;
+        for(halfedge_descriptor h : cc_border_hedges)
+          bhs[source(h, tmesh)] = target(h, tmesh);
+
+        int nbc=0;
+        while(!bhs.empty())
+        {
+          ++nbc;
+          std::pair<vertex_descriptor, vertex_descriptor > top=*bhs.begin();
+          bhs.erase(bhs.begin());
+
+          do
+          {
+            typename std::map<vertex_descriptor, vertex_descriptor>::iterator it_find = bhs.find(top.second);
+            if(it_find == bhs.end())
+              break;
+
+            top = *it_find;
+            bhs.erase(it_find);
+          }
+          while(true);
+        }
+
+        if(nbc != 1)
+        {
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+          std::cout << "  DEBUG: CC not handled because it is not a topological disk("
+                    << nbc << " boundary cycles)\n";
+          ++unsolved_self_intersections;
+#endif
+
+          all_fixed = false;
+          continue;
+        }
+        else
+        {
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+          std::cout << "  DEBUG: CC that is not a topological disk but has only one boundary cycle(preserve_genus=false)\n";
+#endif
+        }
+      }
+    }
+
+    if(!remove_self_intersections_with_hole_filling(cc_border_hedges, cc_faces, working_face_range,
+                                                    tmesh, only_treat_self_intersections_locally,
+                                                    strong_dihedral_angle, weak_dihedral_angle,
+                                                    vpm, gt))
+    {
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+      std::cout << "  DEBUG: Failed to fill hole\n";
+      ++unsolved_self_intersections;
+#endif
+
+      all_fixed = false;
+    }
+    else
+    {
+      something_was_done = true;
+    }
+  }
+
+  if(!something_was_done)
+  {
+    faces_to_remove.swap(faces_to_remove_copy);
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+    std::cout << "  DEBUG: Nothing was changed during this step, self-intersections won`t be recomputed." << std::endl;
+#endif
+  }
+
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_OUTPUT
+  std::stringstream oss;
+  oss << "results/after_step_" << step << ".off" << std::ends;
+  std::ofstream(oss.str().c_str()) << std::setprecision(17) << tmesh;
+#endif
+
+  return std::make_pair(all_fixed, topology_issue);
+}
+
+} // namespace internal
+
+namespace experimental {
+
+template <typename FaceRange, typename TriangleMesh, typename NamedParameters>
+bool remove_self_intersections(const FaceRange& face_range,
+                               TriangleMesh& tmesh,
+                               const NamedParameters& np)
+{
+  using parameters::choose_parameter;
+  using parameters::get_parameter;
+
+  typedef boost::graph_traits<TriangleMesh>                                 graph_traits;
+  typedef typename graph_traits::face_descriptor                            face_descriptor;
+
+  // named parameter extraction
+  typedef typename GetVertexPointMap<TriangleMesh, NamedParameters>::type   VertexPointMap;
+  VertexPointMap vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
+                                        get_property_map(vertex_point, tmesh));
+
+  typedef typename GetGeomTraits<TriangleMesh, NamedParameters>::type       GeomTraits;
+  GeomTraits gt = choose_parameter<GeomTraits>(get_parameter(np, internal_np::geom_traits));
+
+  bool preserve_genus = choose_parameter(get_parameter(np, internal_np::preserve_genus), true);
+  const bool only_treat_self_intersections_locally = choose_parameter(get_parameter(np, internal_np::apply_per_connected_component), false);
+
+  // When treating intersections locally, we don't want to grow the working range too much as
+  // either the solution is found fast, or it's too difficult and neither local smoothing or local
+  // hole filling are going to provide nice results.
+  const int default_max_step = only_treat_self_intersections_locally ? 2 : 7;
+  const int max_steps = choose_parameter(get_parameter(np, internal_np::number_of_iterations), default_max_step);
+
+  // @fixme give it its own named parameter rather than abusing 'with_dihedral_angle'?
+  const double strong_dihedral_angle = choose_parameter(get_parameter(np, internal_np::with_dihedral_angle), 60.);
+
+  // detect_feature_pp NP (unused for now)
+  const double weak_dihedral_angle = 0.; // choose_parameter(get_parameter(np, internal_np::weak_dihedral_angle), 20.);
+
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+  std::cout << "DEBUG: Starting remove_self_intersections, is_valid(tmesh)? " << is_valid_polygon_mesh(tmesh) << "\n";
+  std::cout << "\tpreserve_genus: " << preserve_genus << std::endl;
+  std::cout << "\tonly_treat_self_intersections_locally: " << only_treat_self_intersections_locally << std::endl;
+  std::cout << "\tmax_steps: " << max_steps << std::endl;
+  std::cout << "\tstrong_dihedral_angle: " << strong_dihedral_angle << std::endl;
+  std::cout << "\tweak_dihedral_angle: " << weak_dihedral_angle << std::endl;
+#endif
+
+  if(!preserve_genus)
+    duplicate_non_manifold_vertices(tmesh, np);
+
+  // Look for self-intersections in the mesh and remove them
+  int step = -1;
+  bool all_fixed = true; // indicates if the filling of all created holes went fine
+  bool topology_issue = false; // indicates if some boundary cycles of edges are blocking the fixing
+  std::set<face_descriptor> faces_to_remove;
+  std::set<face_descriptor> working_face_range(face_range.begin(), face_range.end());
+
+  while(++step < max_steps)
+  {
+    if(faces_to_remove.empty()) // the previous round might have been blocked due to topological constraints
+    {
+      typedef std::pair<face_descriptor, face_descriptor> Face_pair;
+      std::vector<Face_pair> self_inter;
+
+      // TODO : possible optimization to reduce the range to check with the bbox
+      // of the previous patches or something.
+      self_intersections(working_face_range, tmesh, std::back_inserter(self_inter));
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+      std::cout << self_inter.size() << " intersecting pairs" << std::endl;
+#endif
+      for(const Face_pair& fp : self_inter)
+      {
+        faces_to_remove.insert(fp.first);
+        faces_to_remove.insert(fp.second);
+      }
+    }
+
+    if(faces_to_remove.empty() && all_fixed)
+    {
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+      std::cout << "DEBUG: There are no more faces to remove." << std::endl;
+#endif
+      break;
+    }
+
+    std::tie(all_fixed, topology_issue) =
+      internal::remove_self_intersections_one_step(
+          faces_to_remove, working_face_range, tmesh,
+          step, preserve_genus, only_treat_self_intersections_locally,
+          strong_dihedral_angle, weak_dihedral_angle, vpm, gt);
+
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+    if(all_fixed && topology_issue)
+        std::cout << "DEBUG: boundary cycles of boundary edges involved in self-intersections.\n";
+#endif
+  }
+
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_DEBUG
+  std::cout << "solved by constrained smoothing: " << internal::self_intersections_solved_by_constrained_smoothing << std::endl;
+  std::cout << "solved by unconstrained smoothing: " << internal::self_intersections_solved_by_unconstrained_smoothing << std::endl;
+  std::cout << "solved by constrained hole-filling: " << internal::self_intersections_solved_by_constrained_hole_filling << std::endl;
+  std::cout << "solved by unconstrained hole-filling: " << internal::self_intersections_solved_by_unconstrained_hole_filling << std::endl;
+  std::cout << "unsolved: " << internal::unsolved_self_intersections << std::endl;
+#endif
+
+#ifdef CGAL_PMP_REMOVE_SELF_INTERSECTION_OUTPUT
+  std::ofstream("results/final.off") << std::setprecision(17) << tmesh;
+#endif
+
+  return step < max_steps;
+}
+
+template <typename FaceRange, typename TriangleMesh>
+bool remove_self_intersections(const FaceRange& face_range, TriangleMesh& tmesh)
+{
+  return remove_self_intersections(face_range, tmesh, parameters::all_default());
+}
+
+template <typename TriangleMesh, typename CGAL_PMP_NP_TEMPLATE_PARAMETERS>
+bool remove_self_intersections(TriangleMesh& tmesh, const CGAL_PMP_NP_CLASS& np)
+{
+  return remove_self_intersections(faces(tmesh), tmesh, np);
+}
+
+template <typename TriangleMesh>
+bool remove_self_intersections(TriangleMesh& tmesh)
+{
+  return remove_self_intersections(tmesh, parameters::all_default());
+}
+
+} // namespace experimental
+} // namespace Polygon_mesh_processing
+} // namespace CGAL
+
+#endif // CGAL_POLYGON_MESH_PROCESSING_REPAIR_SELF_INTERSECTIONS_H
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/self_intersections.h 5.1-1/include/CGAL/Polygon_mesh_processing/self_intersections.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/self_intersections.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/self_intersections.h	2020-09-08 12:15:23.000000000 +0000
@@ -4,8 +4,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/self_intersections.h $
-// $Id: self_intersections.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/self_intersections.h $
+// $Id: self_intersections.h 48bd92b 2020-04-13T13:03:05+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -21,24 +21,33 @@
 
 #include <CGAL/disable_warnings.h>
 
+#include <CGAL/Polygon_mesh_processing/internal/named_function_params.h>
+#include <CGAL/Polygon_mesh_processing/internal/named_params_helper.h>
+
+#include <CGAL/algorithm.h>
+#include <CGAL/Bbox_3.h>
+#include <CGAL/boost/graph/helpers.h>
+#include <CGAL/boost/graph/properties.h>
 #include <CGAL/box_intersection_d.h>
+#include <CGAL/exceptions.h>
 #include <CGAL/intersections.h>
-#include <CGAL/Bbox_3.h>
-
+#include <CGAL/iterator.h>
 #include <CGAL/Kernel/global_functions_3.h>
+#include <CGAL/Random.h>
+#include <CGAL/use.h>
 
-#include <vector>
-#include <exception>
-#include <boost/range.hpp>
+#ifdef CGAL_LINKED_WITH_TBB
+#include <tbb/parallel_for.h>
+#include <tbb/blocked_range.h>
+#include <tbb/concurrent_vector.h>
+#endif
 
 #include <boost/function_output_iterator.hpp>
-#include <boost/type_traits/is_const.hpp>
-#include <boost/graph/graph_traits.hpp>
-#include <CGAL/boost/graph/helpers.h>
-#include <CGAL/boost/graph/properties.h>
 
-#include <CGAL/Polygon_mesh_processing/internal/named_function_params.h>
-#include <CGAL/Polygon_mesh_processing/internal/named_params_helper.h>
+#include <exception>
+#include <sstream>
+#include <type_traits>
+#include <vector>
 
 #ifdef DOXYGEN_RUNNING
 #define CGAL_PMP_NP_TEMPLATE_PARAMETERS NamedParameters
@@ -46,442 +55,520 @@
 #endif
 
 namespace CGAL {
+namespace Polygon_mesh_processing {
 namespace internal {
-template <class TM,//TriangleMesh
-          class Kernel,
-          class Box,
-          class OutputIterator,
-          class VertexPointMap>
-struct Intersect_facets
+
+// Checks for 'real' intersections, i.e. not simply a shared vertex or edge
+template <class GT, class TM, class VPM>
+bool do_faces_intersect(typename boost::graph_traits<TM>::halfedge_descriptor h,
+                        typename boost::graph_traits<TM>::halfedge_descriptor g,
+                        const TM& tmesh,
+                        const VPM vpmap,
+                        const typename GT::Construct_segment_3& construct_segment,
+                        const typename GT::Construct_triangle_3& construct_triangle,
+                        const typename GT::Do_intersect_3& do_intersect)
 {
-  // wrapper to check whether anything is inserted to output iterator
-  struct Output_iterator_with_bool
+  typedef typename boost::graph_traits<TM>::vertex_descriptor                 vertex_descriptor;
+  typedef typename boost::graph_traits<TM>::halfedge_descriptor               halfedge_descriptor;
+
+  typedef typename GT::Segment_3                                              Segment;
+  typedef typename GT::Triangle_3                                             Triangle;
+
+  CGAL_assertion(!is_border(h, tmesh));
+  CGAL_assertion(!is_border(g, tmesh));
+
+  vertex_descriptor hv[3], gv[3];
+  hv[0] = target(h, tmesh);
+  hv[1] = target(next(h, tmesh), tmesh);
+  hv[2] = source(h, tmesh);
+
+  gv[0] = target(g, tmesh);
+  gv[1] = target(next(g, tmesh), tmesh);
+  gv[2] = source(g, tmesh);
+
+  // check for shared edge
+  for(unsigned int i=0; i<3; ++i)
   {
-    Output_iterator_with_bool(OutputIterator* out, bool* intersected)
-      : m_iterator(out), m_intersected(intersected) { }
+    halfedge_descriptor opp_h = opposite(h, tmesh);
+    if(face(opp_h, tmesh) == face(g, tmesh))
+    {
+      // there is an intersection if the four points are coplanar and the triangles overlap
+      if(CGAL::coplanar(get(vpmap, hv[i]),
+                        get(vpmap, hv[(i+1)%3]),
+                        get(vpmap, hv[(i+2)%3]),
+                        get(vpmap, target(next(opp_h, tmesh), tmesh))) &&
+         CGAL::coplanar_orientation(get(vpmap, hv[(i+2)%3]),
+                                    get(vpmap, hv[i]),
+                                    get(vpmap, hv[(i+1)%3]),
+                                    get(vpmap, target(next(opp_h, tmesh), tmesh)))
+           == CGAL::POSITIVE)
+      {
+        return true;
+      }
+      else
+      {
+        // there is a shared edge but no intersection
+        return false;
+      }
+    }
 
-    template<class T>
-    void operator()(const T& t) {
-      *m_intersected = true;
-      *(*m_iterator)++ = t;
+    h = next(h, tmesh);
+  }
+
+  // check for shared vertex --> maybe intersection, maybe not
+  int i(0), j(0);
+  bool shared = false;
+  for(; i<3 && (! shared); ++i)
+  {
+    for(j=0; j<3 && (! shared); ++j)
+    {
+      if(hv[i] == gv[j])
+      {
+        shared = true;
+        break;
+      }
     }
 
-    OutputIterator* m_iterator;
-    bool* m_intersected;
-  };
-// typedefs
-  typedef typename Kernel::Segment_3    Segment;
-  typedef typename Kernel::Triangle_3   Triangle;
-  typedef typename boost::graph_traits<TM>::halfedge_descriptor halfedge_descriptor;
-  typedef typename boost::property_map<TM, boost::vertex_point_t>::const_type Ppmap;
+    if(shared)
+      break;
+  }
 
-// members
-  const TM& m_tmesh;
-  const VertexPointMap m_vpmap;
-  mutable OutputIterator  m_iterator;
-  mutable bool            m_intersected;
-  mutable boost::function_output_iterator<Output_iterator_with_bool> m_iterator_wrapper;
+  if(shared)
+  {
+    // found shared vertex:
+    CGAL_assertion(hv[i] == gv[j]);
+
+    // geometric check if the opposite segments intersect the triangles
+    const Triangle t1 = construct_triangle(get(vpmap, hv[0]), get(vpmap, hv[1]), get(vpmap, hv[2]));
+    const Triangle t2 = construct_triangle(get(vpmap, gv[0]), get(vpmap, gv[1]), get(vpmap, gv[2]));
+
+    const Segment s1 = construct_segment(get(vpmap, hv[(i+1)%3]), get(vpmap, hv[(i+2)%3]));
+    const Segment s2 = construct_segment(get(vpmap, gv[(j+1)%3]), get(vpmap, gv[(j+2)%3]));
+
+    if(do_intersect(t1, s2))
+      return true;
+    else if(do_intersect(t2, s1))
+      return true;
+
+    return false;
+  }
 
-  typename Kernel::Construct_segment_3  segment_functor;
-  typename Kernel::Construct_triangle_3 triangle_functor;
-  typename Kernel::Do_intersect_3       do_intersect_3_functor;
+  // check for geometric intersection
+  const Triangle th = construct_triangle(get(vpmap, hv[0]), get(vpmap, hv[1]), get(vpmap, hv[2]));
+  const Triangle tg = construct_triangle(get(vpmap, gv[0]), get(vpmap, gv[1]), get(vpmap, gv[2]));
+  if(do_intersect(th, tg))
+    return true;
 
+  return false;
+};
 
-  Intersect_facets(const TM& tmesh, OutputIterator it, VertexPointMap vpmap, const Kernel& kernel)
+template <class Box, class TM, class VPM, class GT,
+          class OutputIterator>
+struct Strict_intersect_faces // "strict" as in "not sharing a subface"
+{
+  typedef typename boost::graph_traits<TM>::halfedge_descriptor halfedge_descriptor;
+
+  mutable OutputIterator m_iterator;
+  const TM& m_tmesh;
+  const VPM m_vpmap;
+  typename GT::Construct_segment_3 m_construct_segment;
+  typename GT::Construct_triangle_3 m_construct_triangle;
+  typename GT::Do_intersect_3 m_do_intersect;
+
+  Strict_intersect_faces(const TM& tmesh, VPM vpmap, const GT& gt, OutputIterator it)
     :
-    m_tmesh(tmesh),
-    m_vpmap(vpmap),
-    m_iterator(it),
-    m_intersected(false),
-    m_iterator_wrapper(Output_iterator_with_bool(&m_iterator, &m_intersected)),
-    segment_functor(kernel.construct_segment_3_object()),
-    triangle_functor(kernel.construct_triangle_3_object()),
-    do_intersect_3_functor(kernel.do_intersect_3_object())
-  { }
+      m_iterator(it),
+      m_tmesh(tmesh),
+      m_vpmap(vpmap),
+      m_construct_segment(gt.construct_segment_3_object()),
+      m_construct_triangle(gt.construct_triangle_3_object()),
+      m_do_intersect(gt.do_intersect_3_object())
+  {}
 
   void operator()(const Box* b, const Box* c) const
   {
-    halfedge_descriptor h = halfedge(b->info(), m_tmesh);
-    halfedge_descriptor opp_h;
+    const halfedge_descriptor h = halfedge(b->info(), m_tmesh);
+    const halfedge_descriptor g = halfedge(c->info(), m_tmesh);
 
-    // check for shared egde
-    for(unsigned int i=0; i<3; ++i){
-      opp_h = opposite(h, m_tmesh);
-      if(face(opp_h, m_tmesh) == c->info()){
-        // there is an intersection if the four points are coplanar and
-        // the triangles overlap
-        if(CGAL::coplanar(get(m_vpmap, target(h, m_tmesh)),
-                          get(m_vpmap, target(next(h, m_tmesh), m_tmesh)),
-                          get(m_vpmap, source(h, m_tmesh)),
-                          get(m_vpmap, target(next(opp_h, m_tmesh), m_tmesh))) &&
-           CGAL::coplanar_orientation(get(m_vpmap, source(h, m_tmesh)),
-                                      get(m_vpmap, target(h, m_tmesh)),
-                                      get(m_vpmap, target(next(h, m_tmesh), m_tmesh)),
-                                      get(m_vpmap, target(next(opp_h, m_tmesh), m_tmesh)))
-             == CGAL::POSITIVE){
-          *m_iterator_wrapper++ = std::make_pair(b->info(), c->info());
-          return;
-        } else { // there is a shared edge but no intersection
-          return;
-        }
-      }
-      h = next(h, m_tmesh);
-    }
+    if(do_faces_intersect<GT>(h, g, m_tmesh, m_vpmap, m_construct_segment, m_construct_triangle, m_do_intersect))
+      *m_iterator++ = std::make_pair(b->info(), c->info());
+  }
+};
 
-    // check for shared vertex --> maybe intersection, maybe not
-    halfedge_descriptor g = halfedge(c->info(),m_tmesh);
-    halfedge_descriptor v;
-
-    if(target(h,m_tmesh) == target(g,m_tmesh))
-      v = g;
-    if(target(h,m_tmesh) == target(next(g,m_tmesh),m_tmesh))
-      v = next(g,m_tmesh);
-    if(target(h,m_tmesh) == target(next(next(g,m_tmesh),m_tmesh),m_tmesh))
-      v = next(next(g,m_tmesh),m_tmesh);
-
-    if(v == halfedge_descriptor()){
-      h = next(h,m_tmesh);
-      if(target(h,m_tmesh) == target(g,m_tmesh))
-        v = g;
-      if(target(h,m_tmesh) == target(next(g,m_tmesh),m_tmesh))
-        v = next(g,m_tmesh);
-      if(target(h,m_tmesh) == target(next(next(g,m_tmesh),m_tmesh),m_tmesh))
-        v = next(next(g,m_tmesh),m_tmesh);
-      if(v == halfedge_descriptor()){
-        h = next(h,m_tmesh);
-        if(target(h,m_tmesh) == target(g,m_tmesh))
-          v = g;
-        if(target(h,m_tmesh) == target(next(g,m_tmesh),m_tmesh))
-          v = next(g,m_tmesh);
-        if(target(h,m_tmesh) == target(next(next(g,m_tmesh),m_tmesh),m_tmesh))
-          v = next(next(g,m_tmesh),m_tmesh);
-      }
-    }
+template <class ConcurrencyTag,
+          class TriangleMesh,
+          class FaceRange,
+          class FacePairOutputIterator,
+          class NamedParameters>
+FacePairOutputIterator
+self_intersections_impl(const FaceRange& face_range,
+                        const TriangleMesh& tmesh,
+                        FacePairOutputIterator out,
+                        const bool throw_on_SI,
+                        const NamedParameters& np)
+{
+  CGAL_precondition(CGAL::is_triangle_mesh(tmesh));
 
-    if(v != halfedge_descriptor()){
-      // found shared vertex:
-      CGAL_assertion(target(h,m_tmesh) == target(v,m_tmesh));
-      // geometric check if the opposite segments intersect the triangles
-      Triangle t1 = triangle_functor( get(m_vpmap,target(h,m_tmesh)),
-                                      get(m_vpmap, target(next(h,m_tmesh),m_tmesh)),
-                                      get(m_vpmap, target(next(next(h,m_tmesh),m_tmesh),m_tmesh)));
-      Triangle t2 = triangle_functor( get(m_vpmap, target(v,m_tmesh)),
-                                      get(m_vpmap, target(next(v,m_tmesh),m_tmesh)),
-                                      get(m_vpmap, target(next(next(v,m_tmesh),m_tmesh),m_tmesh)));
-
-      Segment s1 = segment_functor( get(m_vpmap, target(next(h,m_tmesh),m_tmesh)),
-                                    get(m_vpmap, target(next(next(h,m_tmesh),m_tmesh),m_tmesh)));
-      Segment s2 = segment_functor( get(m_vpmap, target(next(v,m_tmesh),m_tmesh)),
-                                    get(m_vpmap, target(next(next(v,m_tmesh),m_tmesh),m_tmesh)));
-
-      if(do_intersect_3_functor(t1,s2)){
-        *m_iterator_wrapper++ = std::make_pair(b->info(), c->info());
-      } else if(do_intersect_3_functor(t2,s1)){
-        *m_iterator_wrapper++ = std::make_pair(b->info(), c->info());
-      }
-      return;
-    }
+  using CGAL::parameters::choose_parameter;
+  using CGAL::parameters::get_parameter;
 
-    // check for geometric intersection
-    Triangle t1 = triangle_functor( get(m_vpmap, target(h,m_tmesh)),
-                                    get(m_vpmap, target(next(h,m_tmesh),m_tmesh)),
-                                    get(m_vpmap, target(next(next(h,m_tmesh),m_tmesh),m_tmesh)));
-    Triangle t2 = triangle_functor( get(m_vpmap, target(g,m_tmesh)),
-                                    get(m_vpmap, target(next(g,m_tmesh),m_tmesh)),
-                                    get(m_vpmap, target(next(next(g,m_tmesh),m_tmesh),m_tmesh)));
-    if(do_intersect_3_functor(t1, t2)){
-      *m_iterator_wrapper++ = std::make_pair(b->info(), c->info());
-    }
-  } // end operator ()
-}; // end struct Intersect_facets
+  typedef TriangleMesh                                                                   TM;
+  typedef typename boost::graph_traits<TM>::halfedge_descriptor                          halfedge_descriptor;
+  typedef typename boost::graph_traits<TM>::face_descriptor                              face_descriptor;
+
+  typedef CGAL::Box_intersection_d::ID_FROM_BOX_ADDRESS                                  Box_policy;
+  typedef CGAL::Box_intersection_d::Box_with_info_d<double, 3, face_descriptor, Box_policy> Box;
+
+  typedef typename GetGeomTraits<TM, NamedParameters>::type                              GT;
+  GT gt = choose_parameter<GT>(get_parameter(np, internal_np::geom_traits));
+
+  typedef typename GetVertexPointMap<TM, NamedParameters>::const_type                    VPM;
+  VPM vpmap = choose_parameter(get_parameter(np, internal_np::vertex_point),
+                               get_const_property_map(boost::vertex_point, tmesh));
+
+  const unsigned int seed = choose_parameter(get_parameter(np, internal_np::random_seed), 0);
+  CGAL_USE(seed); // used in the random shuffle of the range, which is only done to balance tasks in parallel
+
+  const std::ptrdiff_t cutoff = 2000;
 
-struct Throw_at_output {
-  class Throw_at_output_exception: public std::exception
-  { };
-
-  template<class T>
-  void operator()(const T& /* t */) const {
-    throw Throw_at_output_exception();
+  // make one box per face
+  std::vector<Box> boxes;
+  boxes.reserve(std::distance(std::begin(face_range), std::end(face_range)));
+
+  // This loop is very cheap, so there is hardly anything to gain from parallelizing it
+  for(face_descriptor f : face_range)
+  {
+    halfedge_descriptor h = halfedge(f, tmesh);
+    typename boost::property_traits<VPM>::reference
+      p = get(vpmap, target(h,tmesh)),
+      q = get(vpmap, target(next(h, tmesh), tmesh)),
+      r = get(vpmap, target(prev(h, tmesh), tmesh));
+
+    // tiny fixme: if f is degenerate, we might still have a real intersection between f
+    // and another face f', but right now we are not creating a box for f and thus not returning those
+    if(collinear(p, q, r))
+    {
+      if(throw_on_SI)
+        throw CGAL::internal::Throw_at_output_exception();
+      else
+        *out++= std::make_pair(f, f);
+    }
+    else
+    {
+      boxes.push_back(Box(p.bbox() + q.bbox() + r.bbox(), f));
+    }
   }
-};
 
-}// namespace internal
+  // generate box pointers
+  std::vector<const Box*> box_ptr;
+  box_ptr.reserve(boxes.size());
+
+  for(Box& b : boxes)
+    box_ptr.push_back(&b);
 
-namespace Polygon_mesh_processing {
+  // In case we are throwing, like in `does_self_intersect()`, we keep the geometric test to throw ASAP.
+  // This is obviously not optimal if there are no or few self-intersections: it would be a greater speed-up
+  // to do the same as for `self_intersections()`. However, doing like `self_intersections()` would
+  // be a major slow-down over sequential code if there are a lot of self-intersections...
+  typedef boost::function_output_iterator<CGAL::internal::Throw_at_output>               Throwing_output_iterator;
+  typedef internal::Strict_intersect_faces<Box, TM, VPM, GT, Throwing_output_iterator>   Throwing_filter;
+  Throwing_filter throwing_filter(tmesh, vpmap, gt, Throwing_output_iterator());
+
+#if !defined(CGAL_LINKED_WITH_TBB)
+  CGAL_static_assertion_msg (!(std::is_convertible<ConcurrencyTag, Parallel_tag>::value),
+                             "Parallel_tag is enabled but TBB is unavailable.");
+#else
+  if(std::is_convertible<ConcurrencyTag, Parallel_tag>::value)
+  {
+    // We are going to split the range into a number of smaller ranges. To handle
+    // smaller trees of roughly the same size, we first apply a random shuffle to the range
+    CGAL::Random rng(seed);
+    CGAL::cpp98::random_shuffle(box_ptr.begin(), box_ptr.end(), rng);
+
+    // Write in a concurrent vector all pairs that intersect
+    typedef tbb::concurrent_vector<std::pair<face_descriptor, face_descriptor> >         Face_pairs;
+    typedef std::back_insert_iterator<Face_pairs>                                        Face_pairs_back_inserter;
+    typedef internal::Strict_intersect_faces<Box, TM, VPM, GT, Face_pairs_back_inserter> Intersecting_faces_filter;
+
+    Face_pairs face_pairs;
+    Intersecting_faces_filter callback(tmesh, vpmap, gt, std::back_inserter(face_pairs));
+
+    if(throw_on_SI)
+      CGAL::box_self_intersection_d<ConcurrencyTag>(box_ptr.begin(), box_ptr.end(), throwing_filter, cutoff);
+    else
+      CGAL::box_self_intersection_d<ConcurrencyTag>(box_ptr.begin(), box_ptr.end(), callback, cutoff);
 
-#ifndef DOXYGEN_RUNNING
-template <class TriangleMesh
-        , class FaceRange
-        , class OutputIterator
-        , class NamedParameters
->
-OutputIterator
-self_intersections( const FaceRange& face_range,
-                    const TriangleMesh& tmesh,
-                    OutputIterator out,
-                    const NamedParameters& np);
+    // Sequentially write into the output iterator
+    for(std::size_t i=0; i<face_pairs.size(); ++i)
+      *out ++= face_pairs[i];
+
+    return out;
+  }
 #endif
 
-/**
+  // Sequential version of the code
+  // Compute self-intersections filtered out by boxes
+  typedef internal::Strict_intersect_faces<Box, TM, VPM, GT, FacePairOutputIterator> Intersecting_faces_filter;
+  Intersecting_faces_filter intersect_faces(tmesh, vpmap, gt, out);
+
+  if(throw_on_SI)
+    CGAL::box_self_intersection_d<CGAL::Sequential_tag>(box_ptr.begin(), box_ptr.end(), throwing_filter, cutoff);
+  else
+    CGAL::box_self_intersection_d<CGAL::Sequential_tag>(box_ptr.begin(), box_ptr.end(), intersect_faces, cutoff);
+
+  return intersect_faces.m_iterator;
+}
+
+} // namespace internal
+
+/*!
  * \ingroup PMP_intersection_grp
- * detects and records self-intersections of a triangulated surface mesh.
+ * collects intersections between a subset of faces of a triangulated surface mesh.
+ * Two faces are said to intersect if the corresponding triangles intersect
+ * and the intersection is not an edge nor a vertex incident to both faces.
+ *
  * This function depends on the package \ref PkgBoxIntersectionD
+ *
  * @pre `CGAL::is_triangle_mesh(tmesh)`
  *
+ * @tparam ConcurrencyTag enables sequential versus parallel algorithm.
+ *                        Possible values are `Sequential_tag`, `Parallel_tag`, and `Parallel_if_available_tag`.
+ * @tparam FaceRange a model of `ConstRange` with value type `boost::graph_traits<TriangleMesh>::%face_descriptor`.
  * @tparam TriangleMesh a model of `FaceListGraph`
- * @tparam OutputIterator a model of `OutputIterator` holding objects of type
+ * @tparam FacePairOutputIterator a model of `OutputIterator` holding objects of type
  *   `std::pair<boost::graph_traits<TriangleMesh>::%face_descriptor, boost::graph_traits<TriangleMesh>::%face_descriptor>`
- * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+ * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
  *
+ * @param face_range the range of faces to check for self-intersection.
  * @param tmesh the triangulated surface mesh to be checked
- * @param out output iterator to be filled with all pairs of non-adjacent faces that intersect.
-              In case `tmesh` contains some degenerate faces, for each degenerate face `f` a pair `(f,f)`
-              will be put in `out` before any other self intersection between non-degenerate faces.
-              These are the only pairs where degenerate faces will be reported.
- * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+ * @param out output iterator to be filled with all pairs of non-adjacent faces that intersect
+ * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
- *   If this parameter is omitted, an internal property map for
- *   `CGAL::vertex_point_t` must be available in `TriangleMesh`\cgalParamEnd
- *    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `PMPSelfIntersectionTraits` \cgalParamEnd
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `tmesh`}
+ *     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor` as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tmesh)`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     should be available for the vertices of `tmesh`.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{geom_traits}
+ *     \cgalParamDescription{an instance of a geometric traits class}
+ *     \cgalParamType{a class model of `PMPSelfIntersectionTraits`}
+ *     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+ *     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
- *
- * @return `out`
  */
-template <class TriangleMesh
-        , class OutputIterator
-#ifdef DOXYGEN_RUNNING
-        , class NamedParameters
-#else //avoid ambiguity with self_intersections(faces, tmesh, out)
-        , class P, class T, class R
-#endif
->
-OutputIterator
-self_intersections(const TriangleMesh& tmesh
-                 , OutputIterator out
-#ifdef DOXYGEN_RUNNING
-                 , const NamedParameters& np)
-#else
-                 , const Named_function_parameters<P,T,R>& np)
-#endif
+template < class ConcurrencyTag = Sequential_tag,
+           class TriangleMesh,
+           class FaceRange,
+           class FacePairOutputIterator,
+           class NamedParameters>
+FacePairOutputIterator
+self_intersections(const FaceRange& face_range,
+                   const TriangleMesh& tmesh,
+                         FacePairOutputIterator out,
+                   const NamedParameters& np)
 {
-  return self_intersections(faces(tmesh), tmesh, out, np);
+  return internal::self_intersections_impl<ConcurrencyTag>(face_range, tmesh, out, false /*don't throw*/, np);
 }
 
 /// \cond SKIP_IN_MANUAL
-template <class TriangleMesh, class OutputIterator>
-OutputIterator
-self_intersections(const TriangleMesh& tmesh, OutputIterator out)
+template <class ConcurrencyTag = Sequential_tag,
+          class TriangleMesh,
+          class FaceRange,
+          class FacePairOutputIterator>
+FacePairOutputIterator
+self_intersections(const FaceRange& face_range,
+                   const TriangleMesh& tmesh,
+                         FacePairOutputIterator out)
 {
-  return self_intersections(tmesh, out,
-    CGAL::Polygon_mesh_processing::parameters::all_default());
+  return self_intersections<ConcurrencyTag>(face_range, tmesh, out, CGAL::parameters::all_default());
 }
 /// \endcond
 
-/*!
+/**
  * \ingroup PMP_intersection_grp
- * Same as above but the self-intersections reported
- * are only limited to the faces in `face_range`.
+ * collects intersections between all the faces of a triangulated surface mesh.
+ * Two faces are said to intersect if the corresponding triangles intersect
+ * and the intersection is not an edge nor a vertex incident to both faces.
+ *
+ * This function depends on the package \ref PkgBoxIntersectionD
  *
  * @pre `CGAL::is_triangle_mesh(tmesh)`
  *
- * @tparam FaceRange range of `boost::graph_traits<TriangleMesh>::%face_descriptor`,
- *  model of `Range`.
- * Its iterator type is `RandomAccessIterator`.
+ * @tparam ConcurrencyTag enables sequential versus parallel algorithm.
+ *                         Possible values are `Sequential_tag`, `Parallel_tag`, and `Parallel_if_available_tag`.
  * @tparam TriangleMesh a model of `FaceListGraph`
- * @tparam OutputIterator a model of `OutputIterator` holding objects of type
+ * @tparam FacePairOutputIterator a model of `OutputIterator` holding objects of type
  *   `std::pair<boost::graph_traits<TriangleMesh>::%face_descriptor, boost::graph_traits<TriangleMesh>::%face_descriptor>`
- * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+ * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
  *
- * @param face_range the range of faces to check for self-intersection.
  * @param tmesh the triangulated surface mesh to be checked
- * @param out output iterator to be filled with all pairs of non-adjacent faces that intersect
- * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+ * @param out output iterator to be filled with all pairs of non-adjacent faces that intersect.
+              In case `tmesh` contains some degenerate faces, for each degenerate face `f` a pair `(f,f)`
+              will be put in `out` before any other self intersection between non-degenerate faces.
+              These are the only pairs where degenerate faces will be reported.
+ * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
- *   If this parameter is omitted, an internal property map for
- *   `CGAL::vertex_point_t` must be available in `TriangleMesh`\cgalParamEnd
- *    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `PMPSelfIntersectionTraits` \cgalParamEnd
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `tmesh`}
+ *     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor` as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tmesh)`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     should be available for the vertices of `tmesh`.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{geom_traits}
+ *     \cgalParamDescription{an instance of a geometric traits class}
+ *     \cgalParamType{a class model of `PMPSelfIntersectionTraits`}
+ *     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+ *     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
-
+ *
+ * @return `out`
  */
-template <class TriangleMesh
-        , class FaceRange
-        , class OutputIterator
-        , class NamedParameters
->
-OutputIterator
-self_intersections( const FaceRange& face_range,
-                    const TriangleMesh& tmesh,
-                    OutputIterator out,
-                    const NamedParameters& np)
+template <class ConcurrencyTag = Sequential_tag,
+          class TriangleMesh,
+          class FacePairOutputIterator,
+          class CGAL_PMP_NP_TEMPLATE_PARAMETERS>
+FacePairOutputIterator
+self_intersections(const TriangleMesh& tmesh,
+                         FacePairOutputIterator out,
+                   const CGAL_PMP_NP_CLASS& np)
 {
-  CGAL_precondition(CGAL::is_triangle_mesh(tmesh));
-
-  typedef TriangleMesh TM;
-  typedef typename boost::graph_traits<TM>::face_descriptor face_descriptor;
-  typedef typename CGAL::Box_intersection_d::Box_with_info_d<double, 3, face_descriptor> Box;
-
-  // make one box per facet
-  std::vector<Box> boxes;
-  boxes.reserve(
-    std::distance( boost::begin(face_range), boost::end(face_range) )
-  );
-
-  typedef typename GetVertexPointMap<TM, NamedParameters>::const_type VertexPointMap;
-  VertexPointMap vpmap = parameters::choose_parameter(parameters::get_parameter(np, internal_np::vertex_point),
-                                                      get_const_property_map(boost::vertex_point, tmesh));
-
-  for(face_descriptor f : face_range)
-  {
-    typename boost::property_traits<VertexPointMap>::reference
-      p = get(vpmap, target(halfedge(f,tmesh),tmesh)),
-      q = get(vpmap, target(next(halfedge(f, tmesh), tmesh), tmesh)),
-      r = get(vpmap, target(next(next(halfedge(f, tmesh), tmesh), tmesh), tmesh));
-
-    if ( collinear(p, q, r) )
-      *out++= std::make_pair(f,f);
-    else
-      boxes.push_back(Box(p.bbox() + q.bbox() + r.bbox(), f));
-  }
-  // generate box pointers
-  std::vector<const Box*> box_ptr;
-  box_ptr.reserve(num_faces(tmesh));
-
-  for(Box& b : boxes)
-    box_ptr.push_back(&b);
-
-  // compute self-intersections filtered out by boxes
-  typedef typename GetGeomTraits<TM, NamedParameters>::type GeomTraits;
-  CGAL::internal::Intersect_facets<TM,GeomTraits,Box,OutputIterator,VertexPointMap>
-    intersect_facets(tmesh, out, vpmap,
-      parameters::choose_parameter(parameters::get_parameter(np, internal_np::geom_traits), GeomTraits()));
-
-  std::ptrdiff_t cutoff = 2000;
-  CGAL::box_self_intersection_d(box_ptr.begin(), box_ptr.end(),intersect_facets,cutoff);
-  return intersect_facets.m_iterator;
+  return self_intersections<ConcurrencyTag>(faces(tmesh), tmesh, out, np);
 }
 
 /// \cond SKIP_IN_MANUAL
-template <class TriangleMesh
-        , class FaceRange
-        , class OutputIterator
->
-OutputIterator self_intersections(const FaceRange& face_range,
-                                  const TriangleMesh& tmesh,
-                                  OutputIterator out)
+template <class ConcurrencyTag = Sequential_tag, class TriangleMesh, class FacePairOutputIterator>
+FacePairOutputIterator
+self_intersections(const TriangleMesh& tmesh, FacePairOutputIterator out)
 {
-  return self_intersections(face_range, tmesh, out,
-    CGAL::Polygon_mesh_processing::parameters::all_default());
+  return self_intersections<ConcurrencyTag>(faces(tmesh), tmesh, out, parameters::all_default());
 }
 /// \endcond
 
 /**
  * \ingroup PMP_intersection_grp
- * tests if a triangulated surface mesh self-intersects.
+ * tests if a set of faces of a triangulated surface mesh self-intersects.
  * This function depends on the package \ref PkgBoxIntersectionD
  * @pre `CGAL::is_triangle_mesh(tmesh)`
  *
+ * @tparam ConcurrencyTag enables sequential versus parallel algorithm.
+ *                        Possible values are `Sequential_tag`, `Parallel_tag`, and `Parallel_if_available_tag`.
+ * @tparam FaceRange a range of `face_descriptor`
  * @tparam TriangleMesh a model of `FaceListGraph`
- * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+ * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
  *
+ * @param face_range the set of faces to test for self-intersection
  * @param tmesh the triangulated surface mesh to be tested
- * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+ * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `tmesh`.
- *   If this parameter is omitted, an internal property map for
- *   `CGAL::vertex_point_t` must be available in `TriangleMesh`\cgalParamEnd
- *    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `PMPSelfIntersectionTraits` \cgalParamEnd
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `tmesh`}
+ *     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor` as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tmesh)`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     should be available for the vertices of `tmesh`.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{geom_traits}
+ *     \cgalParamDescription{an instance of a geometric traits class}
+ *     \cgalParamType{a class model of `PMPSelfIntersectionTraits`}
+ *     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+ *     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  *
- * @return true if `tmesh` self-intersects
+ * @return `true` if the faces in `face_range` self-intersect
  */
-template <class TriangleMesh
-        , class CGAL_PMP_NP_TEMPLATE_PARAMETERS
-          >
-bool does_self_intersect(const TriangleMesh& tmesh
-                        , const CGAL_PMP_NP_CLASS& np)
+template <class ConcurrencyTag = Sequential_tag,
+          class FaceRange,
+          class TriangleMesh,
+          class NamedParameters>
+bool does_self_intersect(const FaceRange& face_range,
+                         const TriangleMesh& tmesh,
+                         const NamedParameters& np)
 {
   CGAL_precondition(CGAL::is_triangle_mesh(tmesh));
 
   try
   {
-    typedef boost::function_output_iterator<CGAL::internal::Throw_at_output> OutputIterator;
-    self_intersections(tmesh, OutputIterator(), np);
+    CGAL::Emptyset_iterator unused_out;
+    internal::self_intersections_impl<ConcurrencyTag>(face_range, tmesh, unused_out, true /*throw*/, np);
+  }
+  catch(CGAL::internal::Throw_at_output_exception&)
+  {
+    return true;
   }
-  catch( CGAL::internal::Throw_at_output::Throw_at_output_exception& )
-  { return true; }
 
   return false;
 }
 
 /**
  * \ingroup PMP_intersection_grp
- * tests if a set of faces of a triangulated surface mesh self-intersects.
+ * tests if a triangulated surface mesh self-intersects.
  * This function depends on the package \ref PkgBoxIntersectionD
  * @pre `CGAL::is_triangle_mesh(tmesh)`
  *
- * @tparam FaceRange a range of `face_descriptor`
+ * @tparam ConcurrencyTag enables sequential versus parallel algorithm.
+ *                        Possible values are `Sequential_tag`, `Parallel_tag`, and `Parallel_if_available_tag`.
  * @tparam TriangleMesh a model of `FaceListGraph`
- * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+ * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
  *
- * @param face_range the set of faces to test for self-intersection
  * @param tmesh the triangulated surface mesh to be tested
- * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+ * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `tmesh`.
- *   If this parameter is omitted, an internal property map for
- *   `CGAL::vertex_point_t` must be available in `TriangleMesh`\cgalParamEnd
- *    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `SelfIntersectionTraits` \cgalParamEnd
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `tmesh`}
+ *     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor` as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tmesh)`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     should be available for the vertices of `tmesh`.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{geom_traits}
+ *     \cgalParamDescription{an instance of a geometric traits class}
+ *     \cgalParamType{a class model of `PMPSelfIntersectionTraits`}
+ *     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+ *     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  *
- * @return true if the faces in `face_range` self-intersect
+ * @return `true` if `tmesh` self-intersects
  */
-template <class FaceRange,
+template <class ConcurrencyTag = Sequential_tag,
           class TriangleMesh,
-          class NamedParameters
-          >
-bool does_self_intersect(const FaceRange& face_range,
-                         const TriangleMesh& tmesh,
-                         const NamedParameters& np)
+          class CGAL_PMP_NP_TEMPLATE_PARAMETERS>
+bool does_self_intersect(const TriangleMesh& tmesh,
+                         const CGAL_PMP_NP_CLASS& np)
 {
-  CGAL_precondition(CGAL::is_triangle_mesh(tmesh));
-
-  try
-  {
-    typedef boost::function_output_iterator<CGAL::internal::Throw_at_output> OutputIterator;
-    self_intersections(face_range, tmesh, OutputIterator(), np);
-  }
-  catch( CGAL::internal::Throw_at_output::Throw_at_output_exception& )
-  { return true; }
-
-  return false;
+  return does_self_intersect<ConcurrencyTag>(faces(tmesh), tmesh, np);
 }
 
 /// \cond SKIP_IN_MANUAL
-template <class TriangleMesh>
+template <class ConcurrencyTag = Sequential_tag, class TriangleMesh>
 bool does_self_intersect(const TriangleMesh& tmesh)
 {
-  return does_self_intersect(tmesh,
-    CGAL::Polygon_mesh_processing::parameters::all_default());
+  return does_self_intersect<ConcurrencyTag>(faces(tmesh), tmesh, CGAL::parameters::all_default());
 }
 
-template <class FaceRange, class TriangleMesh>
+template <class ConcurrencyTag = Sequential_tag, class FaceRange, class TriangleMesh>
 bool does_self_intersect(const FaceRange& face_range,
                          const TriangleMesh& tmesh)
 {
-  return does_self_intersect(face_range, tmesh,
-    CGAL::Polygon_mesh_processing::parameters::all_default());
+  return does_self_intersect<ConcurrencyTag>(face_range, tmesh, CGAL::parameters::all_default());
 }
-
 /// \endcond
 
-}// end namespace Polygon_mesh_processing
-
+}// namespace Polygon_mesh_processing
 }// namespace CGAL
 
 #include <CGAL/enable_warnings.h>
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/shape_predicates.h 5.1-1/include/CGAL/Polygon_mesh_processing/shape_predicates.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/shape_predicates.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/shape_predicates.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/shape_predicates.h $
-// $Id: shape_predicates.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/shape_predicates.h $
+// $Id: shape_predicates.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -23,6 +23,7 @@
 #include <CGAL/boost/graph/iterator.h>
 #include <CGAL/boost/graph/helpers.h>
 
+#include <boost/range/has_range_iterator.hpp>
 #include <boost/graph/graph_traits.hpp>
 
 #include <limits>
@@ -39,22 +40,27 @@ namespace Polygon_mesh_processing {
 /// An edge is considered degenerate if the geometric positions of its two extremities are identical.
 ///
 /// @tparam PolygonMesh a model of `HalfedgeGraph`
-/// @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+/// @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 ///
 /// @param e an edge of `pm`
 /// @param pm polygon mesh containing `e`
-/// @param np optional \ref pmp_namedparameters "Named Parameters" described below
+/// @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 ///
 /// \cgalNamedParamsBegin
-///   \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pm`.
-///                                     The type of this map is model of `ReadWritePropertyMap`.
-///                                     If this parameter is omitted, an internal property map for
-///                                     `CGAL::vertex_point_t` should be available in `PolygonMesh`
-/// \cgalParamEnd
-///   \cgalParamBegin{geom_traits} a geometric traits class instance.
-///                                The traits class must provide the nested type `Point_3`,
-///                                and the nested functor `Equal_3` to check whether two points are identical.
-///   \cgalParamEnd
+///   \cgalParamNBegin{vertex_point_map}
+///     \cgalParamDescription{a property map associating points to the vertices of `pm`}
+///     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+///                    as key type and `%Point_3` as value type}
+///     \cgalParamDefault{`boost::get(CGAL::vertex_point, pm)`}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{geom_traits}
+///     \cgalParamDescription{an instance of a geometric traits class}
+///     \cgalParamType{The traits class must provide the nested type `Point_3`,
+///                    and the nested functor `Equal_3` to check whether two points are identical.}
+///     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+///     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+///   \cgalParamNEnd
 /// \cgalNamedParamsEnd
 ///
 /// \sa `degenerate_edges()`
@@ -70,10 +76,10 @@ bool is_degenerate_edge(typename boost::
 
   typedef typename GetVertexPointMap<PolygonMesh, NamedParameters>::const_type VertexPointMap;
   VertexPointMap vpmap = choose_parameter(get_parameter(np, internal_np::vertex_point),
-                                      get_const_property_map(vertex_point, pm));
+                                          get_const_property_map(vertex_point, pm));
 
   typedef typename GetGeomTraits<PolygonMesh, NamedParameters>::type Traits;
-  Traits traits = choose_parameter(get_parameter(np, internal_np::geom_traits), Traits());
+  Traits traits = choose_parameter<Traits>(get_parameter(np, internal_np::geom_traits));
 
   return traits.equal_3_object()(get(vpmap, source(e, pm)), get(vpmap, target(e, pm)));
 }
@@ -86,26 +92,113 @@ bool is_degenerate_edge(typename boost::
 }
 
 /// \ingroup PMP_repairing_grp
+/// collects the degenerate edges within a given range of edges.
+///
+/// @tparam EdgeRange a model of `Range` with value type `boost::graph_traits<TriangleMesh>::%edge_descriptor`
+/// @tparam TriangleMesh a model of `HalfedgeGraph`
+/// @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
+///
+/// @param edges a subset of edges of `tm`
+/// @param tm a triangle mesh
+/// @param out an output iterator in which the degenerate edges are written
+/// @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+///
+/// \cgalNamedParamsBegin
+///   \cgalParamNBegin{vertex_point_map}
+///     \cgalParamDescription{a property map associating points to the vertices of `tm`}
+///     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+///                    as key type and `%Point_3` as value type}
+///     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+///     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+///                     must be available in `TriangleMesh`.}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{geom_traits}
+///     \cgalParamDescription{an instance of a geometric traits class}
+///     \cgalParamType{The traits class must provide the nested type `Point_3`,
+///                    and the nested functor `Equal_3` to check whether two points are identical.}
+///     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+///     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+///   \cgalParamNEnd
+/// \cgalNamedParamsEnd
+template <typename EdgeRange, typename TriangleMesh, typename OutputIterator, typename NamedParameters>
+OutputIterator degenerate_edges(const EdgeRange& edges,
+                                const TriangleMesh& tm,
+                                OutputIterator out,
+                                const NamedParameters& np)
+{
+  typedef typename boost::graph_traits<TriangleMesh>::edge_descriptor edge_descriptor;
+
+  for(edge_descriptor ed : edges)
+    if(is_degenerate_edge(ed, tm, np))
+      *out++ = ed;
+
+  return out;
+}
+
+template <typename EdgeRange, typename TriangleMesh, typename OutputIterator>
+OutputIterator degenerate_edges(const EdgeRange& edges,
+                                const TriangleMesh& tm,
+                                OutputIterator out,
+                                typename boost::enable_if<
+                                  typename boost::has_range_iterator<EdgeRange>
+                                >::type* = 0)
+{
+  return degenerate_edges(edges, tm, out, CGAL::parameters::all_default());
+}
+
+/// \ingroup PMP_repairing_grp
+/// calls the function `degenerate_edges()` with the range: `edges(tm)`.
+///
+/// See above for the comprehensive description of the parameters.
+///
+template <typename TriangleMesh, typename OutputIterator, typename NamedParameters>
+OutputIterator degenerate_edges(const TriangleMesh& tm,
+                                OutputIterator out,
+                                const NamedParameters& np
+#ifndef DOXYGEN_RUNNING
+                                ,
+                                typename boost::disable_if<
+                                  boost::has_range_iterator<TriangleMesh>
+                                >::type* = 0
+#endif
+                               )
+{
+  return degenerate_edges(edges(tm), tm, out, np);
+}
+
+template <typename TriangleMesh, typename OutputIterator>
+OutputIterator degenerate_edges(const TriangleMesh& tm, OutputIterator out)
+{
+  return degenerate_edges(edges(tm), tm, out, CGAL::parameters::all_default());
+}
+
+/// \ingroup PMP_repairing_grp
 /// checks whether a triangle face is degenerate.
 /// A triangle face is considered degenerate if the geometric positions of its vertices are collinear.
 ///
 /// @tparam TriangleMesh a model of `FaceGraph`
-/// @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+/// @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 ///
 /// @param f a triangle face of `tm`
 /// @param tm a triangle mesh containing `f`
-/// @param np optional \ref pmp_namedparameters "Named Parameters" described below
+/// @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 ///
 /// \cgalNamedParamsBegin
-///    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `tm`.
-///                                      The type of this map is model of `ReadWritePropertyMap`.
-///                                      If this parameter is omitted, an internal property map for
-///                                      `CGAL::vertex_point_t` should be available in `TriangleMesh`
-/// \cgalParamEnd
-///    \cgalParamBegin{geom_traits} a geometric traits class instance.
-///                                 The traits class must provide the nested functor `Collinear_3`
-///                                 to check whether three points are collinear.
-///   \cgalParamEnd
+///   \cgalParamNBegin{vertex_point_map}
+///     \cgalParamDescription{a property map associating points to the vertices of `tm`}
+///     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+///                    as key type and `%Point_3` as value type}
+///     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{geom_traits}
+///     \cgalParamDescription{an instance of a geometric traits class}
+///     \cgalParamType{The traits class must provide the nested type `Point_3`,
+///                    and the nested functor `Collinear_3` to check whether three points are aligned.}
+///     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+///     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+///   \cgalParamNEnd
 /// \cgalNamedParamsEnd
 ///
 /// \sa `degenerate_faces()`
@@ -123,10 +216,10 @@ bool is_degenerate_triangle_face(typenam
 
   typedef typename GetVertexPointMap<TriangleMesh, NamedParameters>::const_type VertexPointMap;
   VertexPointMap vpmap = choose_parameter(get_parameter(np, internal_np::vertex_point),
-                                      get_const_property_map(vertex_point, tm));
+                                          get_const_property_map(vertex_point, tm));
 
   typedef typename GetGeomTraits<TriangleMesh, NamedParameters>::type Traits;
-  Traits traits = choose_parameter(get_parameter(np, internal_np::geom_traits), Traits());
+  Traits traits = choose_parameter<Traits>(get_parameter(np, internal_np::geom_traits));
 
   typename boost::graph_traits<TriangleMesh>::halfedge_descriptor h = halfedge(f, tm);
 
@@ -143,27 +236,116 @@ bool is_degenerate_triangle_face(typenam
 }
 
 /// \ingroup PMP_repairing_grp
+/// collects the degenerate faces within a given range of faces.
+///
+/// @tparam FaceRange a model of `Range` with value type `boost::graph_traits<TriangleMesh>::%face_descriptor`
+/// @tparam TriangleMesh a model of `FaceGraph`
+/// @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
+///
+/// @param faces a subset of faces of `tm`
+/// @param tm a triangle mesh
+/// @param out an output iterator in which the degenerate faces are put
+/// @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
+///
+/// \cgalNamedParamsBegin
+///   \cgalParamNBegin{vertex_point_map}
+///     \cgalParamDescription{a property map associating points to the vertices of `tm`}
+///     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+///                    as key type and `%Point_3` as value type}
+///     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+///     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+///                     must be available in `TriangleMesh`.}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{geom_traits}
+///     \cgalParamDescription{an instance of a geometric traits class}
+///     \cgalParamType{The traits class must provide the nested type `Point_3`,
+///                    and the nested functor `Collinear_3` to check whether three points are collinear.}
+///     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+///     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+///   \cgalParamNEnd
+/// \cgalNamedParamsEnd
+///
+template <typename FaceRange, typename TriangleMesh, typename OutputIterator, typename NamedParameters>
+OutputIterator degenerate_faces(const FaceRange& faces,
+                                const TriangleMesh& tm,
+                                OutputIterator out,
+                                const NamedParameters& np)
+{
+  typedef typename boost::graph_traits<TriangleMesh>::face_descriptor face_descriptor;
+
+  for(face_descriptor fd : faces)
+  {
+    if(is_degenerate_triangle_face(fd, tm, np))
+      *out++ = fd;
+  }
+  return out;
+}
+
+template <typename FaceRange, typename TriangleMesh, typename OutputIterator>
+OutputIterator degenerate_faces(const FaceRange& faces,
+                                const TriangleMesh& tm,
+                                OutputIterator out,
+                                typename boost::enable_if<
+                                boost::has_range_iterator<FaceRange>
+                                >::type* = 0)
+{
+  return degenerate_faces(faces, tm, out, CGAL::parameters::all_default());
+}
+
+/// \ingroup PMP_repairing_grp
+/// calls the function `degenerate_faces()` with the range: `faces(tm)`.
+///
+/// See above for the comprehensive description of the parameters.
+///
+template <typename TriangleMesh, typename OutputIterator, typename NamedParameters>
+OutputIterator degenerate_faces(const TriangleMesh& tm,
+                                OutputIterator out,
+                                const NamedParameters& np
+#ifndef DOXYGEN_RUNNING
+                                ,
+                                typename boost::disable_if<
+                                  boost::has_range_iterator<TriangleMesh>
+                                >::type* = 0
+#endif
+                                )
+{
+  return degenerate_faces(faces(tm), tm, out, np);
+}
+
+template <typename TriangleMesh, typename OutputIterator>
+OutputIterator degenerate_faces(const TriangleMesh& tm, OutputIterator out)
+{
+  return degenerate_faces(faces(tm), tm, out, CGAL::parameters::all_default());
+}
+
+/// \ingroup PMP_repairing_grp
 /// checks whether a triangle face is needle.
 /// A triangle is said to be a <i>needle</i> if its longest edge is much longer than its shortest edge.
 ///
 /// @tparam TriangleMesh a model of `FaceGraph`
-/// @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+/// @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 ///
 /// @param f a triangle face of `tm`
 /// @param tm triangle mesh containing `f`
 /// @param threshold a bound on the ratio of the longest edge length and the shortest edge length
-/// @param np optional \ref pmp_namedparameters "Named Parameters" described below
+/// @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 ///
 /// \cgalNamedParamsBegin
-///    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `tm`.
-///                                      The type of this map is model of `ReadWritePropertyMap`.
-///                                      If this parameter is omitted, an internal property map for
-///                                      `CGAL::vertex_point_t` should be available in `TriangleMesh`
-/// \cgalParamEnd
-///    \cgalParamBegin{geom_traits} a geometric traits class instance.
-///                                 The traits class must provide the nested type `FT` and
-///                                 the nested functor `Compute_squared_distance_3`.
-///   \cgalParamEnd
+///   \cgalParamNBegin{vertex_point_map}
+///     \cgalParamDescription{a property map associating points to the vertices of `tm`}
+///     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+///                    as key type and `%Point_3` as value type}
+///     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{geom_traits}
+///     \cgalParamDescription{an instance of a geometric traits class}
+///     \cgalParamType{The traits class must provide the nested type `FT`,
+///                    and the nested functor `Compute_squared_distance_3`.}
+///     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+///     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+///   \cgalParamNEnd
 /// \cgalNamedParamsEnd
 ///
 /// \return the shortest halfedge if the triangle face is a needle, and a null halfedge otherwise.
@@ -184,10 +366,10 @@ is_needle_triangle_face(typename boost::
 
   typedef typename GetVertexPointMap<TriangleMesh, NamedParameters>::const_type VertexPointMap;
   VertexPointMap vpmap = choose_parameter(get_parameter(np, internal_np::vertex_point),
-                                      get_const_property_map(vertex_point, tm));
+                                          get_const_property_map(vertex_point, tm));
 
   typedef typename GetGeomTraits<TriangleMesh, NamedParameters>::type       Traits;
-  Traits traits = choose_parameter(get_parameter(np, internal_np::geom_traits), Traits());
+  Traits traits = choose_parameter<Traits>(get_parameter(np, internal_np::geom_traits));
 
   typedef typename Traits::FT                                               FT;
 
@@ -245,26 +427,31 @@ is_needle_triangle_face(typename boost::
 /// A triangle is said to be a <i>cap</i> if one of the its angles is close to `180` degrees.
 ///
 /// @tparam TriangleMesh a model of `FaceGraph`
-/// @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+/// @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 ///
 /// @param f a triangle face of `tm`
 /// @param tm triangle mesh containing `f`
 /// @param threshold the cosine of a minimum angle such that if `f` has an angle greater than this bound,
 ///                  it is a cap. The threshold is in range `[-1 0]` and corresponds to an angle
 ///                  between `90` and `180` degrees.
-/// @param np optional \ref pmp_namedparameters "Named Parameters" described below
+/// @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 ///
 /// \cgalNamedParamsBegin
-///    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `tm`.
-///                                      The type of this map is model of `ReadWritePropertyMap`.
-///                                      If this parameter is omitted, an internal property map for
-///                                      `CGAL::vertex_point_t` should be available in `TriangleMesh`
-/// \cgalParamEnd
-///    \cgalParamBegin{geom_traits} a geometric traits class instance.
-///                                 The traits class must provide the nested type `Point_3` and
-///                                 the nested functors `Compute_squared_distance_3`, `Construct_vector_3`,
-///                                 and `Compute_scalar_product_3`.
-///   \cgalParamEnd
+///   \cgalParamNBegin{vertex_point_map}
+///     \cgalParamDescription{a property map associating points to the vertices of `tm`}
+///     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+///                    as key type and `%Point_3` as value type}
+///     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{geom_traits}
+///     \cgalParamDescription{an instance of a geometric traits class}
+///     \cgalParamType{The traits class must provide the nested type `Point_3`,
+///                    the nested functors `Compute_squared_distance_3`, `Construct_vector_3`,
+///                    and `Compute_scalar_product_3`.}
+///     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+///     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+///   \cgalParamNEnd
 /// \cgalNamedParamsEnd
 ///
 /// \return the halfedge opposite of the largest angle if the face is a cap, and a null halfedge otherwise.
@@ -287,10 +474,10 @@ is_cap_triangle_face(typename boost::gra
 
   typedef typename GetVertexPointMap<TriangleMesh, NamedParameters>::const_type VertexPointMap;
   VertexPointMap vpmap = choose_parameter(get_parameter(np, internal_np::vertex_point),
-                                      get_const_property_map(vertex_point, tm));
+                                          get_const_property_map(vertex_point, tm));
 
   typedef typename GetGeomTraits<TriangleMesh, NamedParameters>::type       Traits;
-  Traits traits = choose_parameter(get_parameter(np, internal_np::geom_traits), Traits());
+  Traits traits = choose_parameter<Traits>(get_parameter(np, internal_np::geom_traits));
 
   typedef typename Traits::FT                                               FT;
   typedef typename Traits::Vector_3                                         Vector_3;
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/smooth_mesh.h 5.1-1/include/CGAL/Polygon_mesh_processing/smooth_mesh.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/smooth_mesh.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/smooth_mesh.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/smooth_mesh.h $
-// $Id: smooth_mesh.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/smooth_mesh.h $
+// $Id: smooth_mesh.h a84927d 2020-07-23T17:15:44+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -36,7 +36,7 @@ namespace Polygon_mesh_processing {
 /*!
 * \ingroup PMP_meshing_grp
 *
-* \short smoothes a triangulated region of a polygon mesh.
+* \short smooths a triangulated region of a polygon mesh.
 *
 * This function attempts to make the triangle angle and area distributions as uniform as possible
 * by moving (non-constrained) vertices.
@@ -52,45 +52,81 @@ namespace Polygon_mesh_processing {
 * @tparam TriangleMesh model of `MutableFaceGraph`.
 * @tparam FaceRange range of `boost::graph_traits<TriangleMesh>::%face_descriptor`,
           model of `Range`. Its iterator type is `ForwardIterator`.
-* @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+* @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 *
 * @param tmesh a polygon mesh with triangulated surface patches to be smoothed.
 * @param faces the range of triangular faces defining one or several surface patches to be smoothed.
-* @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below.
+* @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 *
 * \cgalNamedParamsBegin
-*  \cgalParamBegin{use_angle_smoothing} Boolean value to indicate whether angle-based smoothing should be used.
-*    %Default is `true`.
-*  \cgalParamEnd
-*  \cgalParamBegin{use_area_smoothing} Boolean value to indicate whether area-based smoothing should be used.
-*    %Default is `true`.
-*  \cgalParamEnd
-*  \cgalParamBegin{number_of_iterations} the number of iterations for the
-*    sequence of the smoothing iterations performed (default is 1).
-*  \cgalParamEnd
-*  \cgalParamBegin{use_safety_constraints} if `true`, vertex moves that would worsen the mesh
-*    are ignored. %Default is `false`.
-*  \cgalParamEnd
-*  \cgalParamBegin{use_Delaunay_flips} if `true` (default value), area-based smoothing will be completed
-*    by a phase of Delaunay-based edge-flips to prevent the creation of elongated triangles.
-*  \cgalParamEnd
-*  \cgalParamBegin{do_project} if `true` (default value), points are projected onto the initial surface
-*   after each iteration.
-*  \cgalParamEnd
-*  \cgalParamBegin{vertex_is_constrained_map} a property map containing the
-*    constrained-or-not status of each vertex of `tmesh`. A constrained vertex
-*    cannot be modified at all during smoothing.
-*  \cgalParamEnd
-*  \cgalParamBegin{edge_is_constrained_map} a property map, model of `ReadWritePropertyMap`, containing the
-*    constrained-or-not status of each edge of `tmesh`. A constrained edge cannot be flipped and its extremities
-*    are tagged as constrained vertices.
-*  \cgalParamEnd
-*  \cgalParamBegin{vertex_point_map} the property map, model of `ReadWritePropertyMap`, with the points
-*    associated to the vertices of `tmesh`.
-*  \cgalParamEnd
-*  \cgalParamBegin{geom_traits} a geometric traits class instance, model of `Kernel`.
-*    Exact constructions kernels are not supported by this function.
-*  \cgalParamEnd
+*   \cgalParamNBegin{number_of_iterations}
+*     \cgalParamDescription{the number of iterations for the sequence of the smoothing iterations performed}
+*     \cgalParamType{unsigned int}
+*     \cgalParamDefault{`1`}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{use_angle_smoothing}
+*     \cgalParamDescription{value to indicate whether angle-based smoothing should be used}
+*     \cgalParamType{Boolean}
+*     \cgalParamDefault{`true`}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{use_area_smoothing}
+*     \cgalParamDescription{value to indicate whether area-based smoothing should be used}
+*     \cgalParamType{Boolean}
+*     \cgalParamDefault{`true`}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{vertex_point_map}
+*     \cgalParamDescription{a property map associating points to the vertices of `tmesh`}
+*     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+*                    as key type and `%Point_3` as value type}
+*     \cgalParamDefault{`boost::get(CGAL::vertex_point, tmesh)`}
+*     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+*                     must be available in `TriangleMesh`.}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{geom_traits}
+*     \cgalParamDescription{an instance of a geometric traits class}
+*     \cgalParamType{a class model of `Kernel`}
+*     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+*     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{use_safety_constraints}
+*     \cgalParamDescription{If `true`, vertex moves that would worsen the mesh are ignored.}
+*     \cgalParamType{Boolean}
+*     \cgalParamDefault{`false`}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{use_Delaunay_flips}
+*     \cgalParamDescription{If `true`, area-based smoothing will be completed by a phase of
+*                           Delaunay-based edge-flips to prevent the creation of elongated triangles.}
+*     \cgalParamType{Boolean}
+*     \cgalParamDefault{`true`}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{do_project}
+*     \cgalParamDescription{If `true`, points are projected onto the initial surface after each iteration.}
+*     \cgalParamType{Boolean}
+*     \cgalParamDefault{`true`}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{vertex_is_constrained_map}
+*     \cgalParamDescription{a property map containing the constrained-or-not status of each vertex of `tmesh`.}
+*     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+*                    as key type and `bool` as value type. It must be default constructible.}
+*     \cgalParamDefault{a default property map where no vertex is constrained}
+*     \cgalParamExtra{A constrained vertex cannot be modified at all during smoothing.}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{edge_is_constrained_map}
+*     \cgalParamDescription{a property map containing the constrained-or-not status of each edge of `tmesh`.}
+*     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<TriangleMesh>::%edge_descriptor`
+*                    as key type and `bool` as value type. It must be default constructible.}
+*     \cgalParamDefault{a default property map where no edge is constrained}
+*     \cgalParamExtra{A constrained edge cannot be modified at all during smoothing.}
+*   \cgalParamNEnd
 * \cgalNamedParamsEnd
 *
 * @warning The third party library \link thirdpartyCeres Ceres \endlink is required
@@ -150,10 +186,9 @@ void smooth_mesh(const FaceRange& faces,
   using parameters::get_parameter;
 
   // named parameters
-  GeomTraits gt = choose_parameter(get_parameter(np, internal_np::geom_traits),
-                               GeomTraits());
+  GeomTraits gt = choose_parameter<GeomTraits>(get_parameter(np, internal_np::geom_traits));
   VertexPointMap vpmap = choose_parameter(get_parameter(np, internal_np::vertex_point),
-                               get_property_map(CGAL::vertex_point, tmesh));
+                                          get_property_map(CGAL::vertex_point, tmesh));
 
   const bool use_angle_smoothing = choose_parameter(get_parameter(np, internal_np::use_angle_smoothing), true);
   bool use_area_smoothing = choose_parameter(get_parameter(np, internal_np::use_area_smoothing), true);
@@ -173,7 +208,7 @@ void smooth_mesh(const FaceRange& faces,
   const bool use_Delaunay_flips = choose_parameter(get_parameter(np, internal_np::use_Delaunay_flips), true);
 
   VCMap vcmap = choose_parameter(get_parameter(np, internal_np::vertex_is_constrained),
-                             get(Vertex_property_tag(), tmesh));
+                                 get(Vertex_property_tag(), tmesh));
 
   // If it's the default vcmap, manually set everything to false because the dynamic pmap has no default initialization
   if((std::is_same<VCMap, Default_VCMap>::value))
@@ -183,7 +218,7 @@ void smooth_mesh(const FaceRange& faces,
   }
 
   ECMap ecmap = choose_parameter(get_parameter(np, internal_np::edge_is_constrained),
-                             Constant_property_map<edge_descriptor, bool>(false));
+                                 Constant_property_map<edge_descriptor, bool>(false));
 
   // a constrained edge has constrained extremities
   for(face_descriptor f : faces)
@@ -221,7 +256,6 @@ void smooth_mesh(const FaceRange& faces,
   }
 
   Tree aabb_tree(input_triangles.begin(), input_triangles.end());
-  aabb_tree.accelerate_distance_queries();
 
   // Setup the working ranges and check some preconditions
   Angle_smoother angle_smoother(tmesh, vpmap, vcmap, gt);
@@ -236,8 +270,16 @@ void smooth_mesh(const FaceRange& faces,
 
   for(unsigned int i=0; i<nb_iterations; ++i)
   {
+#ifdef CGAL_PMP_SMOOTHING_DEBUG
+    std::cout << "Iteration #" << i << std::endl;
+#endif
+
     if(use_area_smoothing)
     {
+#ifdef CGAL_PMP_SMOOTHING_DEBUG
+      std::cout << "Smooth areas..." << std::endl;
+#endif
+
       // First apply area smoothing...
       area_smoother.optimize(use_safety_constraints /*check for bad faces*/,
                              false /*apply moves as soon as they're calculated*/,
@@ -246,7 +288,7 @@ void smooth_mesh(const FaceRange& faces,
       {
         if(use_safety_constraints && does_self_intersect(tmesh))
         {
-#ifdef CGAL_PMP_SMOOTHING_VERBOSE
+#ifdef CGAL_PMP_SMOOTHING_DEBUG
           std::cerr << "Cannot re-project as there are self-intersections in the mesh!\n";
 #endif
           break;
@@ -262,6 +304,10 @@ void smooth_mesh(const FaceRange& faces,
     // ... then angle smoothing
     if(use_angle_smoothing)
     {
+#ifdef CGAL_PMP_SMOOTHING_DEBUG
+      std::cout << "Smooth angles..." << std::endl;
+#endif
+
       angle_smoother.optimize(use_safety_constraints /*check for bad faces*/,
                               true /*apply all moves at once*/,
                               use_safety_constraints /*check if the min angle is improved*/);
@@ -270,7 +316,7 @@ void smooth_mesh(const FaceRange& faces,
       {
         if(use_safety_constraints && does_self_intersect(tmesh))
         {
-#ifdef CGAL_PMP_SMOOTHING_VERBOSE
+#ifdef CGAL_PMP_SMOOTHING_DEBUG
           std::cerr << "Can't do re-projection, there are self-intersections in the mesh!\n";
 #endif
           break;
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/smooth_shape.h 5.1-1/include/CGAL/Polygon_mesh_processing/smooth_shape.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/smooth_shape.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/smooth_shape.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/smooth_shape.h $
-// $Id: smooth_shape.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/smooth_shape.h $
+// $Id: smooth_shape.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -48,31 +48,54 @@ namespace Polygon_mesh_processing {
 * @tparam TriangleMesh model of `MutableFaceGraph`.
 * @tparam FaceRange range of `boost::graph_traits<TriangleMesh>::%face_descriptor`,
 *         model of `Range`. Its iterator type is `ForwardIterator`.
-* @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+* @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 *
 * @param tmesh a polygon mesh with triangulated surface patches to be smoothed.
 * @param faces the range of triangular faces defining one or several surface patches to be smoothed.
 * @param time a time step that corresponds to the speed by which the surface is smoothed.
 *        A larger time step results in faster convergence but details may be distorted to have a larger extent
 *        compared to more iterations with a smaller step. Typical values scale in the interval (1e-6, 1].
-* @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below.
+* @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 *
 * \cgalNamedParamsBegin
-*  \cgalParamBegin{vertex_point_map} the property map with the points associated
-*    to the vertices of `tmesh`. Instance of a class model of `ReadWritePropertyMap`.
-*  \cgalParamEnd
-*  \cgalParamBegin{geom_traits} a geometric traits class instance, model of `Kernel`.
-*  \cgalParamEnd
-*  \cgalParamBegin{vertex_is_constrained_map} a property map containing the
-*    constrained-or-not status of each vertex of `tmesh`. A constrained vertex
-*    cannot be modified at all during smoothing.
-*  \cgalParamEnd
-*  \cgalParamBegin{number_of_iterations} the number of iterations for the
-*    sequence of the smoothing iterations performed. Each iteration is performed
-*    with the given time step.
-*  \cgalParamEnd
-*  \cgalParamBegin{sparse_linear_solver} an instance of the sparse linear solver used for smoothing \cgalParamEnd
-*  \cgalParamEnd
+*   \cgalParamNBegin{number_of_iterations}
+*     \cgalParamDescription{the number of iterations for the sequence of the smoothing iterations performed}
+*     \cgalParamType{unsigned int}
+*     \cgalParamDefault{`1`}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{vertex_point_map}
+*     \cgalParamDescription{a property map associating points to the vertices of `tmesh`}
+*     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+*                    as key type and `%Point_3` as value type}
+*     \cgalParamDefault{`boost::get(CGAL::vertex_point, tmesh)`}
+*     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+*                     must be available in `TriangleMesh`.}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{geom_traits}
+*     \cgalParamDescription{an instance of a geometric traits class}
+*     \cgalParamType{a class model of `Kernel`}
+*     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+*     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{vertex_is_constrained_map}
+*     \cgalParamDescription{a property map containing the constrained-or-not status of each vertex of `tmesh`.}
+*     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+*                    as key type and `bool` as value type. It must be default constructible.}
+*     \cgalParamDefault{a default property map where no vertex is constrained}
+*     \cgalParamExtra{A constrained vertex cannot be modified at all during smoothing.}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{sparse_linear_solver}
+*     \cgalParamDescription{an instance of the sparse linear solver used for smoothing}
+*     \cgalParamType{a class model of `SparseLinearAlgebraWithFactorTraits_d`}
+*     \cgalParamDefault{if \ref thirdpartyEigen "Eigen" 3.2 (or greater) is available and
+*                       `CGAL_EIGEN3_ENABLED` is defined, then the following overload of `Eigen_solver_traits`
+*                       is provided as default value:
+*                       `CGAL::Eigen_solver_traits<Eigen::BiCGSTAB<CGAL::Eigen_sparse_matrix<double>::%EigenType, Eigen::IncompleteLUT<double> > >`}
+*   \cgalParamNEnd
 * \cgalNamedParamsEnd
 *
 *  @warning This function involves linear algebra, that is computed using a non-exact floating-point arithmetic.
@@ -97,11 +120,11 @@ void smooth_shape(const FaceRange& faces
   using parameters::choose_parameter;
   using parameters::get_parameter;
 
-  GeomTraits gt = choose_parameter(get_parameter(np, internal_np::geom_traits), GeomTraits());
+  GeomTraits gt = choose_parameter<GeomTraits>(get_parameter(np, internal_np::geom_traits));
   VertexPointMap vpmap = choose_parameter(get_parameter(np, internal_np::vertex_point),
-                                      get_property_map(CGAL::vertex_point, tmesh));
+                                          get_property_map(CGAL::vertex_point, tmesh));
   VCMap vcmap = choose_parameter(get_parameter(np, internal_np::vertex_is_constrained),
-                             Constant_property_map<vertex_descriptor, bool>(false));
+                                 Constant_property_map<vertex_descriptor, bool>(false));
   const unsigned int nb_iterations = choose_parameter(get_parameter(np, internal_np::number_of_iterations), 1);
 
 #if defined(CGAL_EIGEN3_ENABLED)
@@ -132,7 +155,7 @@ void smooth_shape(const FaceRange& faces
   typedef typename Sparse_solver::Matrix                                          Eigen_matrix;
   typedef typename Sparse_solver::Vector                                          Eigen_vector;
 
-  Sparse_solver solver = choose_parameter(get_parameter(np, internal_np::sparse_linear_solver), Default_solver());
+  Sparse_solver solver = choose_parameter<Default_solver>(get_parameter(np, internal_np::sparse_linear_solver));
 
   std::size_t n = vertices(tmesh).size();
   Eigen_matrix A(n, n);
@@ -149,7 +172,7 @@ void smooth_shape(const FaceRange& faces
 
   for(unsigned int iter=0; iter<nb_iterations; ++iter)
   {
-#ifdef CGAL_PMP_SMOOTHING_VERBOSE
+#ifdef CGAL_PMP_SMOOTHING_DEBUG
     std::cout << "iteration #" << iter << std::endl;
 #endif
 
@@ -161,7 +184,7 @@ void smooth_shape(const FaceRange& faces
     }
     else
     {
-#ifdef CGAL_PMP_SMOOTHING_VERBOSE
+#ifdef CGAL_PMP_SMOOTHING_DEBUG
       std::cerr << "Failed to solve system!" << std::endl;
 #endif
       break;
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/stitch_borders.h 5.1-1/include/CGAL/Polygon_mesh_processing/stitch_borders.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/stitch_borders.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/stitch_borders.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/stitch_borders.h $
-// $Id: stitch_borders.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/stitch_borders.h $
+// $Id: stitch_borders.h 0715654 2020-06-03T19:01:46+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -53,39 +53,32 @@
 #endif
 
 namespace CGAL{
-
 namespace Polygon_mesh_processing{
-
 namespace internal{
 
-template <typename PM, typename VertexPointMap>
+template <typename PolygonMesh, typename VertexPointMap>
 struct Less_for_halfedge
 {
-  typedef typename boost::graph_traits<PM>::halfedge_descriptor
-    halfedge_descriptor;
-  typedef typename boost::property_traits<VertexPointMap>::reference Point;
-
-  Less_for_halfedge(const PM& pmesh_,
-                    const VertexPointMap& vpmap_)
-    : pmesh(pmesh_),
-      vpmap(vpmap_)
+  typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor halfedge_descriptor;
+  typedef typename boost::property_traits<VertexPointMap>::reference     Point;
+
+  Less_for_halfedge(const PolygonMesh& pmesh_, const VertexPointMap& vpm_)
+    : pmesh(pmesh_), vpm(vpm_)
   {}
 
-  bool operator()(halfedge_descriptor h1,
-                  halfedge_descriptor h2) const
+  bool operator()(halfedge_descriptor h1, halfedge_descriptor h2) const
   {
-    Point s1 = get(vpmap,target(opposite(h1, pmesh), pmesh));
-    Point t1 = get(vpmap,target(h1, pmesh));
-    Point s2 = get(vpmap,target(opposite(h2, pmesh), pmesh));
-    Point t2 = get(vpmap,target(h2, pmesh));
-    return
-    ( s1 < t1?  std::make_pair(s1,t1) : std::make_pair(t1, s1) )
-    <
-    ( s2 < t2?  std::make_pair(s2,t2) : std::make_pair(t2, s2) );
+    Point s1 = get(vpm,target(opposite(h1, pmesh), pmesh));
+    Point t1 = get(vpm,target(h1, pmesh));
+    Point s2 = get(vpm,target(opposite(h2, pmesh), pmesh));
+    Point t2 = get(vpm,target(h2, pmesh));
+
+    return (s1 < t1 ? std::make_pair(s1,t1) : std::make_pair(t1, s1))
+             < (s2 < t2 ? std::make_pair(s2,t2) : std::make_pair(t2, s2));
   }
 
-  const PM& pmesh;
-  const VertexPointMap& vpmap;
+  const PolygonMesh& pmesh;
+  const VertexPointMap& vpm;
 };
 
 //add a pair of border halfedges to be stitched.
@@ -94,142 +87,181 @@ template<typename Halfedge,
          typename Border_halfedge_map,
          typename Halfedge_pair,
          typename Manifold_halfedge_pair,
-         typename VPMAP,
+         typename VPM,
          typename Mesh>
 void fill_pairs(const Halfedge& he,
                 Border_halfedge_map& border_halfedge_map,
                 Halfedge_pair& halfedge_pairs,
                 Manifold_halfedge_pair& manifold_halfedge_pairs,
-                VPMAP vpmap,
+                VPM vpm,
                 const Mesh& pmesh)
 {
   typename Border_halfedge_map::iterator set_it;
   bool insertion_ok;
-  std::tie(set_it, insertion_ok)
-      = border_halfedge_map.insert(std::make_pair(he,std::make_pair(1,0)));
+  std::tie(set_it, insertion_ok) = border_halfedge_map.insert(std::make_pair(he,std::make_pair(1,0)));
 
-  if ( !insertion_ok ){ // we found already a halfedge with the points
+  if(!insertion_ok) // we found already a halfedge with the points
+  {
     ++set_it->second.first; // increase the multiplicity
     if(set_it->second.first == 2)
     {
       set_it->second.second = halfedge_pairs.size(); // set the id of the pair in the vector
-      halfedge_pairs.push_back( std::make_pair(set_it->first, he) );
-      if ( get(vpmap, source(he,pmesh))==get(vpmap, target(set_it->first,pmesh)) &&
-           get(vpmap, target(he,pmesh))==get(vpmap, source(set_it->first,pmesh)) )
+      halfedge_pairs.push_back( std::make_pair(set_it->first, he));
+      if(get(vpm, source(he,pmesh)) == get(vpm, target(set_it->first, pmesh)) &&
+         get(vpm, target(he,pmesh)) == get(vpm, source(set_it->first, pmesh)))
       {
         manifold_halfedge_pairs.push_back(true);
       }
       else
-        manifold_halfedge_pairs.push_back(false);
-    }
-    else
-      if ( set_it->second.first > 2 )
       {
-        manifold_halfedge_pairs[ set_it->second.second ] = false;
+        manifold_halfedge_pairs.push_back(false);
       }
+    }
+    else if(set_it->second.first > 2)
+    {
+      manifold_halfedge_pairs[ set_it->second.second ] = false;
+    }
   }
 }
 
-template<typename Mesh,
-         typename CCMap,
-         typename FIMap>
-std::size_t num_component_wrapper(const Mesh& pmesh,
-                      CCMap cc,
-                      FIMap fim)
+template <typename Mesh>
+struct Default_halfedges_keeper
 {
-  return CGAL::Polygon_mesh_processing::connected_components(pmesh, cc,
-                                                             CGAL::Polygon_mesh_processing::parameters::face_index_map(fim));
-}
+  typedef typename boost::graph_traits<Mesh>::halfedge_descriptor       halfedge_descriptor;
+
+  Default_halfedges_keeper() { }
+
+  halfedge_descriptor operator()(const halfedge_descriptor h1, const halfedge_descriptor h2) const
+  {
+    return (h1 < h2) ? h1 : h2; // Arbitrary preference
+  }
+};
 
-//specialization if there is no default FIMap, create one
-template<typename Mesh,
-         typename CCMap>
-std::size_t num_component_wrapper(Mesh& pmesh,
-                      CCMap cc,
-                      boost::cgal_no_property::type)
+// Provided for convenience. If passed to stitch_borders(), this ensures that if any of the two
+// edges is marked, then the preserver edge is also marked.
+template <typename EMM, typename PolygonMesh>
+struct Halfedges_keeper_with_marked_edge_priority
 {
+  typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor       halfedge_descriptor;
 
-     boost::unordered_map<typename boost::graph_traits<Mesh>::face_descriptor, std::size_t> fim;
+  Halfedges_keeper_with_marked_edge_priority(const EMM& emm, const PolygonMesh& pmesh)
+    : emm(emm), pmesh(pmesh)
+  { }
 
-   //init the map
-   std::size_t i=-1;
-   for(typename boost::graph_traits<Mesh>::face_descriptor f : faces(pmesh))
-     fim[f]=++i;
+  halfedge_descriptor operator()(const halfedge_descriptor h1, const halfedge_descriptor h2) const
+  {
+    // If h2 is marked, then whether h1 is marked or not, we can just return h2
+    if(get(emm, edge(h2, pmesh)))
+      return h2;
 
-   return CGAL::Polygon_mesh_processing::connected_components(pmesh,
-                                                              cc,
-                                                              parameters::face_index_map(boost::make_assoc_property_map(fim)));
-}
+    // If only h1 is marked, return h1;
+    // If both or none are marked, it does not matter which we are keeping, so also return h1
+    return h1;
+  }
+
+private:
+  const EMM& emm;
+  const PolygonMesh& pmesh;
+};
 
-template <typename PM, typename OutputIterator, typename LessHedge, typename VertexPointMap
-          , class CGAL_PMP_NP_TEMPLATE_PARAMETERS>
+template <typename PolygonMesh,
+          typename OutputIterator,
+          class CGAL_PMP_NP_TEMPLATE_PARAMETERS>
 OutputIterator
-collect_duplicated_stitchable_boundary_edges
-(PM& pmesh, OutputIterator out, LessHedge less_hedge,
- const VertexPointMap& vpmap, const CGAL_PMP_NP_CLASS& np)
+collect_duplicated_stitchable_boundary_edges(PolygonMesh& pmesh,
+                                             OutputIterator out,
+                                             const CGAL_PMP_NP_CLASS& np)
 {
-  typedef typename boost::graph_traits<PM>::halfedge_descriptor halfedge_descriptor;
-  typedef std::map<halfedge_descriptor, std::pair<int, std::size_t>, LessHedge> Border_halfedge_map;
-  Border_halfedge_map border_halfedge_map(less_hedge);
-  std::vector< std::pair<halfedge_descriptor, halfedge_descriptor> > halfedge_pairs;
-  std::vector< bool > manifold_halfedge_pairs;
+  using parameters::choose_parameter;
+  using parameters::get_parameter;
+
+  typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor          halfedge_descriptor;
 
+  typedef typename GetVertexPointMap<PolygonMesh, CGAL_PMP_NP_CLASS>::const_type  VPM;
+  VPM vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
+                             get_const_property_map(vertex_point, pmesh));
+
+  typedef CGAL::dynamic_face_property_t<int>                                      Face_property_tag;
+  typedef typename boost::property_map<PolygonMesh, Face_property_tag>::type      Face_cc_map;
 
-  typedef CGAL::dynamic_face_property_t<int>                        Face_property_tag;
-  typedef typename boost::property_map<PM, Face_property_tag>::type Face_cc_map;
   Face_cc_map cc;
-  std::size_t num_component = 0;
+  std::size_t num_cc = 0;
   std::vector<std::vector<halfedge_descriptor> > border_edges_per_cc;
-  bool per_cc = parameters::choose_parameter(parameters::get_parameter(np, internal_np::apply_per_connected_component),
-                                             false);
+
+  typedef typename internal_np::Lookup_named_param_def<internal_np::halfedges_keeper_t,
+                                                       CGAL_PMP_NP_CLASS,
+                                                       Default_halfedges_keeper<PolygonMesh> >::type  Halfedge_keeper;
+  const Halfedge_keeper hd_kpr = choose_parameter(get_parameter(np, internal_np::halfedges_keeper),
+                                                  Default_halfedges_keeper<PolygonMesh>());
+
+  bool per_cc = choose_parameter(get_parameter(np, internal_np::apply_per_connected_component), false);
+
+  typedef Less_for_halfedge<PolygonMesh, VPM>                                     Less_hedge;
+  typedef std::map<halfedge_descriptor, std::pair<int, std::size_t>, Less_hedge>  Border_halfedge_map;
+
+  Less_hedge less_hedge(pmesh, vpm);
+  Border_halfedge_map border_halfedge_map(less_hedge);
+
+  std::vector<std::pair<halfedge_descriptor, halfedge_descriptor> > halfedge_pairs;
+  std::vector<bool> manifold_halfedge_pairs;
+
   if(per_cc)
   {
     cc = get(Face_property_tag(), pmesh);
-    typedef typename GetFaceIndexMap<PM, CGAL_PMP_NP_CLASS>::const_type FIMap;
-    FIMap fim = parameters::choose_parameter(parameters::get_parameter(np, internal_np::face_index),
-                                             get_const_property_map(face_index, pmesh));
-    num_component = num_component_wrapper(pmesh, cc, fim);
-    border_edges_per_cc.resize(num_component);
+    num_cc = connected_components(pmesh, cc, np);
+    border_edges_per_cc.resize(num_cc);
   }
 
   for(halfedge_descriptor he : halfedges(pmesh))
   {
-    if ( !CGAL::is_border(he, pmesh) )
+    if(!CGAL::is_border(he, pmesh))
       continue;
+
     if(per_cc)
-    {
       border_edges_per_cc[get(cc, face(opposite(he, pmesh), pmesh))].push_back(he);
-    }
     else
-    {
-      fill_pairs(he, border_halfedge_map, halfedge_pairs,
-                 manifold_halfedge_pairs, vpmap, pmesh);
-    }
+      fill_pairs(he, border_halfedge_map, halfedge_pairs, manifold_halfedge_pairs, vpm, pmesh);
   }
+
   if(per_cc)
   {
-    for(std::size_t i = 0; i < num_component; ++i)
+    for(std::size_t i=0; i<num_cc; ++i)
     {
-      Border_halfedge_map border_halfedge_map_in_cc(less_hedge);
       CGAL_assertion(halfedge_pairs.empty());
       CGAL_assertion(manifold_halfedge_pairs.empty());
-      for(int j = 0; j < static_cast<int>(border_edges_per_cc[i].size()); ++j)
+
+      Border_halfedge_map border_halfedge_map_in_cc(less_hedge);
+      for(std::size_t j=0; j<border_edges_per_cc[i].size(); ++j)
       {
         halfedge_descriptor he = border_edges_per_cc[i][j];
         fill_pairs(he, border_halfedge_map_in_cc, halfedge_pairs,
-                   manifold_halfedge_pairs, vpmap, pmesh);
+                   manifold_halfedge_pairs, vpm, pmesh);
       }
+
       // put in `out` only manifold edges from the set of edges to stitch.
       // We choose not to allow only a pair out of the whole set to be stitched
       // as we can produce inconsistent stitching along a sequence of non-manifold edges
-      std::size_t nb_pairs=halfedge_pairs.size();
-      for (std::size_t k=0; k<nb_pairs; ++k)
+      const std::size_t nb_pairs = halfedge_pairs.size();
+      for(std::size_t k=0; k<nb_pairs; ++k)
       {
-        if( manifold_halfedge_pairs[k] )
+        if(manifold_halfedge_pairs[k])
         {
-          *out++=halfedge_pairs[k];
+          // the first halfedge of the pair is kept
+          if(hd_kpr(halfedge_pairs[k].first, halfedge_pairs[k].second) == halfedge_pairs[k].second)
+            std::swap(halfedge_pairs[k].first, halfedge_pairs[k].second);
+
+          *out++ = halfedge_pairs[k];
+
+#ifdef CGAL_PMP_STITCHING_DEBUG
+          const halfedge_descriptor h = halfedge_pairs[k].first;
+          const halfedge_descriptor hn = halfedge_pairs[k].second;
+          std::cout << "Stitch "
+                    << edge(h, pmesh) << " (" << get(vpm, source(h, pmesh)) << ") - (" << get(vpm, target(h, pmesh)) << ") and "
+                    << edge(hn, pmesh) << " (" << get(vpm, source(hn, pmesh)) << ") - (" << get(vpm, target(hn, pmesh)) << ")" << std::endl;
+#endif
         }
       }
+
       halfedge_pairs.clear();
       manifold_halfedge_pairs.clear();
     }
@@ -239,11 +271,25 @@ collect_duplicated_stitchable_boundary_e
   // put in `out` only manifold edges from the set of edges to stitch.
   // We choose not to allow only a pair out of the whole set to be stitched
   // as we can produce inconsistent stitching along a sequence of non-manifold edges
-    std::size_t nb_pairs=halfedge_pairs.size();
-    for (std::size_t i=0; i<nb_pairs; ++i)
+    const std::size_t nb_pairs=halfedge_pairs.size();
+    for(std::size_t i=0; i<nb_pairs; ++i)
     {
-      if( manifold_halfedge_pairs[i] )
-        *out++=halfedge_pairs[i];
+      if(manifold_halfedge_pairs[i])
+      {
+        // the first halfedge of the pair is kept
+        if(hd_kpr(halfedge_pairs[i].first, halfedge_pairs[i].second) == halfedge_pairs[i].second)
+          std::swap(halfedge_pairs[i].first, halfedge_pairs[i].second);
+
+        *out++ = halfedge_pairs[i];
+
+#ifdef CGAL_PMP_STITCHING_DEBUG
+        const halfedge_descriptor h = halfedge_pairs[i].first;
+        const halfedge_descriptor hn = halfedge_pairs[i].second;
+        std::cout << "Stitch "
+                  << edge(h, pmesh) << " (" << get(vpm, source(h, pmesh)) << ") - (" << get(vpm, target(h, pmesh)) << ") and "
+                  << edge(hn, pmesh) << " (" << get(vpm, source(hn, pmesh)) << ") - (" << get(vpm, target(hn, pmesh)) << ")" << std::endl;
+#endif
+      }
     }
   }
 
@@ -251,16 +297,18 @@ collect_duplicated_stitchable_boundary_e
 }
 
 
-template <class PM>
-void update_target_vertex(typename boost::graph_traits<PM>::halfedge_descriptor h,
-                          typename boost::graph_traits<PM>::vertex_descriptor v_kept,
-                          PM& pmesh)
+template <class PolygonMesh>
+void update_target_vertex(typename boost::graph_traits<PolygonMesh>::halfedge_descriptor h,
+                          typename boost::graph_traits<PolygonMesh>::vertex_descriptor v_kept,
+                          PolygonMesh& pmesh)
 {
-  typename boost::graph_traits<PM>::halfedge_descriptor start = h;
-  do{
+  typename boost::graph_traits<PolygonMesh>::halfedge_descriptor start = h;
+  do
+  {
     set_target(h, v_kept, pmesh);
     h = opposite(next(h, pmesh), pmesh);
-  } while( h != start );
+  }
+  while(h != start);
 }
 
 template <class vertex_descriptor, class Handle_map>
@@ -270,11 +318,10 @@ uf_get_handle(vertex_descriptor v,
               Handle_map& handles)
 {
   std::pair<typename Handle_map::iterator, bool> insert_res =
-  handles.insert( std::make_pair(v, typename Union_find<vertex_descriptor>::handle()) );
-  if( insert_res.second )
-  {
-    insert_res.first->second=uf_vertices.make_set(v);
-  }
+    handles.insert(std::make_pair(v, typename Union_find<vertex_descriptor>::handle()));
+  if(insert_res.second)
+    insert_res.first->second = uf_vertices.make_set(v);
+
   return insert_res.first->second;
 }
 
@@ -289,20 +336,18 @@ void uf_join_vertices(vertex_descriptor
   uf_vertices.unify_sets(h1, h2);
 }
 
-
-
 // main functions (vertices to keep selected and halfedge pairs filtered)
-template <class PM, typename HalfedgePairsRange, typename VertexPointMap,
+template <typename PolygonMesh, typename HalfedgePairsRange, typename VertexPointMap,
           typename Uf_vertices, typename Uf_handles>
-void run_stitch_borders(PM& pmesh,
+void run_stitch_borders(PolygonMesh& pmesh,
                         const HalfedgePairsRange& to_stitch,
                         const VertexPointMap& vpm,
                         Uf_vertices& uf_vertices,
                         Uf_handles& uf_handles)
 {
-  typedef typename boost::graph_traits<PM>::vertex_descriptor vertex_descriptor;
-  typedef typename boost::graph_traits<PM>::halfedge_descriptor halfedge_descriptor;
-  typedef typename std::pair<halfedge_descriptor, halfedge_descriptor> halfedges_pair;
+  typedef typename boost::graph_traits<PolygonMesh>::vertex_descriptor      vertex_descriptor;
+  typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor    halfedge_descriptor;
+  typedef typename std::pair<halfedge_descriptor, halfedge_descriptor>      halfedges_pair;
 
   CGAL_USE(vpm);
 
@@ -327,13 +372,13 @@ void run_stitch_borders(PM& pmesh,
     vertex_descriptor v_to_keep =
       *uf_vertices.find(uf_get_handle(h1_tgt, uf_vertices, uf_handles));
 
-    if (v_to_keep!=h1_tgt)
+    if(v_to_keep!=h1_tgt)
     {
       vertices_to_delete.push_back(h1_tgt);
       update_target_vertex(h1, v_to_keep, pmesh);
     }
 
-    if (v_to_keep != h2_src && h1_tgt!=h2_src)
+    if(v_to_keep != h2_src && h1_tgt!=h2_src)
     {
       vertices_to_delete.push_back( h2_src );
       update_target_vertex(opposite(h2, pmesh), v_to_keep, pmesh);
@@ -345,19 +390,19 @@ void run_stitch_borders(PM& pmesh,
 
     //update vertex pointers: target of h1 vs source of h2
     v_to_keep = *uf_vertices.find(uf_get_handle(h2_tgt, uf_vertices, uf_handles));
-    if (v_to_keep!=h2_tgt)
+    if(v_to_keep!=h2_tgt)
     {
       vertices_to_delete.push_back( h2_tgt );
       update_target_vertex(h2, v_to_keep, pmesh);
     }
 
-    if (v_to_keep!=h1_src && h1_src!=h2_tgt)
+    if(v_to_keep!=h1_src && h1_src!=h2_tgt)
     {
       vertices_to_delete.push_back( h1_src );
       update_target_vertex(opposite(h1, pmesh), v_to_keep, pmesh);
     }
 
-    set_halfedge(v_to_keep, opposite(h1,pmesh), pmesh);
+    set_halfedge(v_to_keep, opposite(h1, pmesh), pmesh);
   }
 
   /// Update next/prev of neighbor halfedges (that are not set for stiching)
@@ -414,14 +459,14 @@ void run_stitch_borders(PM& pmesh,
   }
 }
 
-template <class PM, typename HalfedgePairsRange, typename VertexPointMap>
-std::size_t stitch_borders_impl(PM& pmesh,
+template <typename PolygonMesh, typename HalfedgePairsRange, typename VertexPointMap>
+std::size_t stitch_borders_impl(PolygonMesh& pmesh,
                                 const HalfedgePairsRange& to_stitch,
                                 const VertexPointMap& vpm)
 {
-  typedef typename boost::graph_traits<PM>::vertex_descriptor vertex_descriptor;
-  typedef typename boost::graph_traits<PM>::halfedge_descriptor halfedge_descriptor;
-  typedef typename std::pair<halfedge_descriptor, halfedge_descriptor> halfedges_pair;
+  typedef typename boost::graph_traits<PolygonMesh>::vertex_descriptor    vertex_descriptor;
+  typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor  halfedge_descriptor;
+  typedef typename std::pair<halfedge_descriptor, halfedge_descriptor>    halfedges_pair;
 
   // The first step of the algorithm is to filter halfedges to be stitched so that
   // after stitching no edges will be present more than once.
@@ -481,9 +526,9 @@ std::size_t stitch_borders_impl(PM& pmes
 
       typename Uf_handles::iterator it_res = uf_handles.find(other_vd);
 
-      if (it_res!=uf_handles.end()) // if the other vertex is also involved in a merge
+      if(it_res!=uf_handles.end()) // if the other vertex is also involved in a merge
       {
-        if (other_vd < vd)
+        if(other_vd < vd)
           continue; // avoid reporting twice the same edge
 
         typename Uf_vertices::handle src_handle=uf_vertices.find(it_res->second);
@@ -507,7 +552,7 @@ std::size_t stitch_borders_impl(PM& pmes
        break; // nothing to do
       case 2:
       {
-        if (is_border_edge(it->second.front(), pmesh) &&
+        if(is_border_edge(it->second.front(), pmesh) &&
             is_border_edge(it->second.back(), pmesh))
           break; // these are edges that are most possibly scheduled for stitching or will create a two halfedge loop
         CGAL_FALLTHROUGH;
@@ -526,7 +571,7 @@ std::size_t stitch_borders_impl(PM& pmes
   }
 
   // filter halfedges to stitch
-  if (!unstitchable_vertices.empty())
+  if(!unstitchable_vertices.empty())
   {
     std::vector<halfedges_pair> to_stitch_filtered;
     to_stitch_filtered.reserve( to_stitch.size());
@@ -534,7 +579,7 @@ std::size_t stitch_borders_impl(PM& pmes
     {
       // We test both halfedges because the previous test
       // might involve only one of the two halfedges
-      if ( unstitchable_vertices.count( source(hk.first, pmesh) )== 0 &&
+      if(unstitchable_vertices.count( source(hk.first, pmesh) )== 0 &&
            unstitchable_vertices.count( target(hk.first, pmesh) )== 0 &&
            unstitchable_vertices.count( source(hk.second, pmesh) )== 0 &&
            unstitchable_vertices.count( target(hk.second, pmesh) )== 0 )
@@ -577,16 +622,21 @@ std::size_t stitch_borders_impl(PM& pmes
 /// the same as those of the target and source vertices of `h2` respectively.
 ///
 /// \tparam PolygonMesh a model of `MutableFaceGraph`
-/// \tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+/// \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 ///
 /// \param h a border halfedge of the polygon mesh `pm`
 /// \param pm the polygon mesh to be stitched
-/// \param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+/// \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 ///
 /// \cgalNamedParamsBegin
-///   \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
-///     If this parameter is omitted, an internal property map for `CGAL::vertex_point_t` must be available in `PolygonMesh`.
-///   \cgalParamEnd
+///   \cgalParamNBegin{vertex_point_map}
+///     \cgalParamDescription{a property map associating points to the vertices of `pm`}
+///     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+///                    as key type and `%Point_3` as value type}
+///     \cgalParamDefault{`boost::get(CGAL::vertex_point, pm)`}
+///     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+///                     must be available in `PolygonMesh`.}
+///   \cgalParamNEnd
 /// \cgalNamedParamsEnd
 ///
 /// \returns the number of pairs of halfedges that were stitched.
@@ -604,9 +654,16 @@ std::size_t stitch_boundary_cycle(const
 
   typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor           halfedge_descriptor;
 
-  typedef typename GetVertexPointMap<PolygonMesh, CGAL_PMP_NP_CLASS>::const_type   VPMap;
-  VPMap vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
-                           get_const_property_map(vertex_point, pm));
+  typedef typename GetVertexPointMap<PolygonMesh, CGAL_PMP_NP_CLASS>::const_type   VPM;
+  VPM vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
+                             get_const_property_map(vertex_point, pm));
+
+  typedef internal::Default_halfedges_keeper<PolygonMesh>                          Default_halfedges_keeper;
+  typedef typename internal_np::Lookup_named_param_def<internal_np::halfedges_keeper_t,
+                                                       CGAL_PMP_NP_CLASS,
+                                                        Default_halfedges_keeper>::type  Halfedge_keeper;
+  const Halfedge_keeper hd_kpr = choose_parameter(get_parameter(np, internal_np::halfedges_keeper),
+                                                  Default_halfedges_keeper());
 
   std::size_t stitched_boundary_cycles_n = 0;
 
@@ -671,12 +728,14 @@ std::size_t stitch_boundary_cycle(const
 
     halfedge_descriptor curr_h = start_h;
     halfedge_descriptor curr_hn = next(curr_h, pm);
-    for(;;) // while we can expand the zipping range
+    for(;;) //while we can expand the zipping range
     {
       // Don't create an invalid polygon mesh, even if the geometry allows it
       if(face(opposite(curr_h, pm), pm) == face(opposite(curr_hn, pm), pm))
       {
         unstitchable_halfedges.insert(curr_h);
+        unstitchable_halfedges.insert(curr_hn);
+
         bh = curr_hn;
         break;
       }
@@ -684,7 +743,10 @@ std::size_t stitch_boundary_cycle(const
       CGAL_assertion(is_border(curr_h, pm));
       CGAL_assertion(is_border(curr_hn, pm));
 
-      hedges_to_stitch.push_back(std::make_pair(curr_h, curr_hn));
+      if(hd_kpr(curr_h, curr_hn) == curr_h)
+        hedges_to_stitch.push_back(std::make_pair(curr_h, curr_hn));
+      else
+        hedges_to_stitch.push_back(std::make_pair(curr_hn, curr_h));
 
 #ifdef CGAL_PMP_STITCHING_DEBUG_PP
       std::cout << "expand zip with:\n"
@@ -737,7 +799,10 @@ std::size_t stitch_boundary_cycle(const
 #endif
 
         for(const auto& hp : hedges_to_stitch)
+        {
           unstitchable_halfedges.insert(hp.first);
+          unstitchable_halfedges.insert(hp.second);
+        }
       }
     }
   }
@@ -760,15 +825,20 @@ std::size_t stitch_boundary_cycle(const
 /// the same as those of the target and source vertices of `h2` respectively.
 ///
 /// \tparam PolygonMesh a model of `MutableFaceGraph`
-/// \tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+/// \tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 ///
 /// \param pm the polygon mesh to be stitched
-/// \param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+/// \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 ///
 /// \cgalNamedParamsBegin
-///   \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
-///     If this parameter is omitted, an internal property map for `CGAL::vertex_point_t` must be available in `PolygonMesh`.
-///   \cgalParamEnd
+///   \cgalParamNBegin{vertex_point_map}
+///     \cgalParamDescription{a property map associating points to the vertices of `pm`}
+///     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+///                    as key type and `%Point_3` as value type}
+///     \cgalParamDefault{`boost::get(CGAL::vertex_point, pm)`}
+///     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+///                     must be available in `PolygonMesh`.}
+///   \cgalParamNEnd
 /// \cgalNamedParamsEnd
 ///
 /// \returns the number of pairs of halfedges that were stitched.
@@ -799,21 +869,24 @@ std::size_t stitch_boundary_cycles(Polyg
 }
 
 ///\cond SKIP_IN_MANUAL
+// The VPM is only used here for debugging info purposes as in this overload, the halfedges
+// to stitch are already provided and all further checks are combinatorial and not geometrical.
+// There is thus nothing interesting to pass via named parameters and this overload is not documented.
 template <typename PolygonMesh,
           typename HalfedgePairsRange,
           typename CGAL_PMP_NP_TEMPLATE_PARAMETERS>
-void stitch_borders(PolygonMesh& pmesh,
-                    const HalfedgePairsRange& hedge_pairs_to_stitch,
-                    const CGAL_PMP_NP_CLASS& np)
+std::size_t stitch_borders(PolygonMesh& pmesh,
+                           const HalfedgePairsRange& hedge_pairs_to_stitch,
+                           const CGAL_PMP_NP_CLASS& np)
 {
   using parameters::choose_parameter;
   using parameters::get_parameter;
 
-  typedef typename GetVertexPointMap<PolygonMesh, CGAL_PMP_NP_CLASS>::const_type  VPMap;
-  VPMap vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
-                               get_const_property_map(vertex_point, pmesh));
+  typedef typename GetVertexPointMap<PolygonMesh, CGAL_PMP_NP_CLASS>::const_type  VPM;
+  VPM vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
+                             get_const_property_map(vertex_point, pmesh));
 
-  internal::stitch_borders_impl(pmesh, hedge_pairs_to_stitch, vpm);
+  return internal::stitch_borders_impl(pmesh, hedge_pairs_to_stitch, vpm);
 }
 ///\endcond
 
@@ -834,13 +907,15 @@ void stitch_borders(PolygonMesh& pmesh,
 * @param pmesh the polygon mesh to be modified by stitching
 * @param hedge_pairs_to_stitch a range of `std::pair` of halfedges to be stitched together
 *
+* @return the number of pairs of halfedges that were stitched.
+*
 */
 template <typename PolygonMesh,
           typename HalfedgePairsRange>
-void stitch_borders(PolygonMesh& pmesh,
-                    const HalfedgePairsRange& hedge_pairs_to_stitch)
+std::size_t stitch_borders(PolygonMesh& pmesh,
+                           const HalfedgePairsRange& hedge_pairs_to_stitch)
 {
-  stitch_borders(pmesh, hedge_pairs_to_stitch, CGAL::parameters::all_default());
+  return stitch_borders(pmesh, hedge_pairs_to_stitch, CGAL::parameters::all_default());
 }
 
 /// \ingroup PMP_repairing_grp
@@ -851,40 +926,45 @@ void stitch_borders(PolygonMesh& pmesh,
 /// the same as those of the target and source vertices of `h2` respectively.
 ///
 /// @tparam PolygonMesh a model of `FaceListGraph` and `MutableFaceGraph`
-/// @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+/// @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 ///
 /// @param pmesh the polygon mesh to be modified by stitching
-/// @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+/// @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 ///
 /// \cgalNamedParamsBegin
-///    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
-/// If this parameter is omitted, an internal property map for
-/// `CGAL::vertex_point_t` must be available in `PolygonMesh`.\cgalParamEnd
-/// \cgalParamBegin{apply_per_connected_component}
-///  specifies if the borders should only be stitched inside their own connected component.
-/// In that case, a property map for `CGAL::face_index_t` should be either available as an internal property map
-/// to `pmesh` or provided as the \ref pmp_namedparameters "Named Parameter" `face_index_map`. If this is not the case,
-/// a default map will be created on the fly.
-/// Default value is `false`.\cgalParamEnd
-/// \cgalParamBegin{face_index_map} a property map containing the index of each face of `pmesh` \cgalParamEnd
+///   \cgalParamNBegin{vertex_point_map}
+///     \cgalParamDescription{a property map associating points to the vertices of `pmesh`}
+///     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+///                    as key type and `%Point_3` as value type}
+///     \cgalParamDefault{`boost::get(CGAL::vertex_point, pmesh)`}
+///     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+///                     must be available in `PolygonMesh`.}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{apply_per_connected_component}
+///     \cgalParamDescription{specifies if the borders should only be stitched only within their own connected component.}
+///     \cgalParamType{Boolean}
+///     \cgalParamDefault{`false`}
+///   \cgalParamNEnd
+///
+///   \cgalParamNBegin{face_index_map}
+///     \cgalParamDescription{a property map associating to each face of `pmesh` a unique index between `0` and `num_faces(pmesh) - 1`}
+///     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%face_descriptor`
+///                    as key type and `std::size_t` as value type}
+///     \cgalParamDefault{an automatically indexed internal map}
+///   \cgalParamNEnd
 /// \cgalNamedParamsEnd
 ///
+/// @return the number of pairs of halfedges that were stitched.
+///
 /// @sa `stitch_boundary_cycle()`
 /// @sa `stitch_boundary_cycles()`
 ///
 template <typename PolygonMesh, class CGAL_PMP_NP_TEMPLATE_PARAMETERS>
-void stitch_borders(PolygonMesh& pmesh, const CGAL_PMP_NP_CLASS& np)
+std::size_t stitch_borders(PolygonMesh& pmesh,
+                           const CGAL_PMP_NP_CLASS& np)
 {
-  using parameters::choose_parameter;
-  using parameters::get_parameter;
-
-  typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor
-    halfedge_descriptor;
-  std::vector< std::pair<halfedge_descriptor, halfedge_descriptor> > hedge_pairs_to_stitch;
-
-  typedef typename GetVertexPointMap<PolygonMesh, CGAL_PMP_NP_CLASS>::const_type VPMap;
-  VPMap vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
-                           get_const_property_map(vertex_point, pmesh));
+  typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor halfedge_descriptor;
 
 #ifdef CGAL_PMP_STITCHING_DEBUG
   std::cout << "------- Stitch cycles..." << std::endl;
@@ -897,31 +977,30 @@ void stitch_borders(PolygonMesh& pmesh,
   std::cout << "------- Stitch all..." << std::endl;
 #endif
 
-  internal::collect_duplicated_stitchable_boundary_edges(pmesh,
-                                                         std::back_inserter(hedge_pairs_to_stitch),
-                                                         internal::Less_for_halfedge<PolygonMesh, VPMap>(pmesh, vpm),
-                                                         vpm, np);
+  std::vector< std::pair<halfedge_descriptor, halfedge_descriptor> > hedge_pairs_to_stitch;
+  internal::collect_duplicated_stitchable_boundary_edges(pmesh, std::back_inserter(hedge_pairs_to_stitch), np);
 
-  stitch_borders(pmesh, hedge_pairs_to_stitch, np);
+  res += stitch_borders(pmesh, hedge_pairs_to_stitch, np);
 
 #ifdef CGAL_PMP_STITCHING_DEBUG
+  std::cout << "------- Stitched " << res << " after cycles & general" << std::endl;
   std::cout << "------- Stitch cycles (#2)..." << std::endl;
 #endif
 
   res += stitch_boundary_cycles(pmesh, np);
 
 #ifdef CGAL_PMP_STITCHING_DEBUG
-  std::cout << "------- Stitched " << res << " in boundary cycles" << std::endl;
+  std::cout << "------- Stitched " << res << " (total)" << std::endl;
 #endif
 
-  CGAL_USE(res);
+  return res;
 }
 
 ///\cond SKIP_IN_MANUAL
 template <typename PolygonMesh>
-void stitch_borders(PolygonMesh& pmesh)
+std::size_t stitch_borders(PolygonMesh& pmesh)
 {
-  stitch_borders(pmesh, CGAL::parameters::all_default());
+  return stitch_borders(pmesh, CGAL::parameters::all_default());
 }
 ///\endcond
 
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/transform.h 5.1-1/include/CGAL/Polygon_mesh_processing/transform.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/transform.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/transform.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/transform.h $
-// $Id: transform.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/transform.h $
+// $Id: transform.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -26,16 +26,21 @@ namespace Polygon_mesh_processing{
  * the `value_type` of `vertex_point_map` (see below). Such a functor can be
  * `CGAL::Aff_transformation_3` for example.
  * @tparam PolygonMesh a model of `VertexListGraph`
- * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+ * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
  *
  * @param transformation the transformation functor to apply to  the points of `mesh`.
  * @param mesh the `PolygonMesh` to transform.
- * @param np optional sequence of \ref pmp_namedparameters for `mesh`, among the ones listed below
+ * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
- * * \cgalNamedParamsBegin
- *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `mesh`.
- *   If this parameter is omitted, an internal property map for
- *   `CGAL::vertex_point_t` must be available in `PolygonMesh`\cgalParamEnd
+ * \cgalNamedParamsBegin
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `mesh`}
+ *     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+ *                    as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, mesh)`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     must be available in `PolygonMesh`.}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  *
  */
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/triangulate_faces.h 5.1-1/include/CGAL/Polygon_mesh_processing/triangulate_faces.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/triangulate_faces.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/triangulate_faces.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/triangulate_faces.h $
-// $Id: triangulate_faces.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/triangulate_faces.h $
+// $Id: triangulate_faces.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -400,18 +400,28 @@ public:
 * \ingroup PMP_meshing_grp
 * triangulates a single face of a polygon mesh. This function depends on the package \ref PkgTriangulation2
 * @tparam PolygonMesh a model of `FaceListGraph` and `MutableFaceGraph`
-* @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+* @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 *
 * @param f face to be triangulated
 * @param pmesh the polygon mesh to which the face to be triangulated belongs
-* @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
-*
+* @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 *
 * \cgalNamedParamsBegin
-*    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
-*   If this parameter is omitted, an internal property map for
-*   `CGAL::vertex_point_t` must be available in `PolygonMesh`\cgalParamEnd
-*    \cgalParamBegin{geom_traits} a geometric traits class instance \cgalParamEnd
+*   \cgalParamNBegin{vertex_point_map}
+*     \cgalParamDescription{a property map associating points to the vertices of `pmesh`}
+*     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+*                    as key type and `%Point_3` as value type}
+*     \cgalParamDefault{`boost::get(CGAL::vertex_point, pmesh)`}
+*     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+*                     must be available in `PolygonMesh`.}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{geom_traits}
+*     \cgalParamDescription{an instance of a geometric traits class}
+*     \cgalParamType{a class model of `Kernel`}
+*     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+*     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+*   \cgalParamNEnd
 * \cgalNamedParamsEnd
 *
 * @return `true` if the face has been triangulated.
@@ -431,7 +441,7 @@ bool triangulate_face(typename boost::gr
 
   //Kernel
   typedef typename GetGeomTraits<PolygonMesh, NamedParameters>::type Kernel;
-  Kernel traits = choose_parameter(get_parameter(np, internal_np::geom_traits), Kernel());
+  Kernel traits = choose_parameter<Kernel>(get_parameter(np, internal_np::geom_traits));
 
   //Option
   bool use_cdt = choose_parameter(get_parameter(np, internal_np::use_delaunay_triangulation), true);
@@ -455,20 +465,32 @@ bool triangulate_face(typename boost::gr
           model of `Range`.
           Its iterator type is `InputIterator`.
 * @tparam PolygonMesh a model of `FaceListGraph` and `MutableFaceGraph`
-* @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+* @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 *
 * @param face_range the range of faces to be triangulated
 * @param pmesh the polygon mesh to be triangulated
-* @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+* @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 *
 * \cgalNamedParamsBegin
-*    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
-*   If this parameter is omitted, an internal property map for
-*   `CGAL::vertex_point_t` must be available in `PolygonMesh`\cgalParamEnd
-*    \cgalParamBegin{geom_traits} a geometric traits class instance \cgalParamEnd
+*   \cgalParamNBegin{vertex_point_map}
+*     \cgalParamDescription{a property map associating points to the vertices of `pmesh`}
+*     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+*                    as key type and `%Point_3` as value type}
+*     \cgalParamDefault{`boost::get(CGAL::vertex_point, pmesh)`}
+*     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+*                     must be available in `PolygonMesh`.}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{geom_traits}
+*     \cgalParamDescription{an instance of a geometric traits class}
+*     \cgalParamType{a class model of `Kernel`}
+*     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+*     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+*   \cgalParamNEnd
 * \cgalNamedParamsEnd
 *
 * @return `true` if all the faces have been triangulated.
+*
 * @see triangulate_face()
 */
 template <typename FaceRange, typename PolygonMesh, typename NamedParameters>
@@ -486,7 +508,7 @@ bool triangulate_faces(FaceRange face_ra
 
   //Kernel
   typedef typename GetGeomTraits<PolygonMesh, NamedParameters>::type Kernel;
-  Kernel traits = choose_parameter(get_parameter(np, internal_np::geom_traits), Kernel());
+  Kernel traits = choose_parameter<Kernel>(get_parameter(np, internal_np::geom_traits));
 
   //Option
   bool use_cdt = choose_parameter(get_parameter(np, internal_np::use_delaunay_triangulation), true);
@@ -505,19 +527,31 @@ bool triangulate_faces(FaceRange face_ra
 * \ingroup PMP_meshing_grp
 * triangulates all faces of a polygon mesh. This function depends on the package \ref PkgTriangulation2
 * @tparam PolygonMesh a model of `FaceListGraph` and `MutableFaceGraph`
-* @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+* @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 *
 * @param pmesh the polygon mesh to be triangulated
-* @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+* @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 *
 * \cgalNamedParamsBegin
-*    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
-*   If this parameter is omitted, an internal property map for
-*   `CGAL::vertex_point_t` must be available in `PolygonMesh`\cgalParamEnd
-*    \cgalParamBegin{geom_traits} a geometric traits class instance \cgalParamEnd
+*   \cgalParamNBegin{vertex_point_map}
+*     \cgalParamDescription{a property map associating points to the vertices of `pmesh`}
+*     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+*                    as key type and `%Point_3` as value type}
+*     \cgalParamDefault{`boost::get(CGAL::vertex_point, pmesh)`}
+*     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+*                     must be available in `PolygonMesh`.}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{geom_traits}
+*     \cgalParamDescription{an instance of a geometric traits class}
+*     \cgalParamType{a class model of `Kernel`}
+*     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+*     \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+*   \cgalParamNEnd
 * \cgalNamedParamsEnd
 *
 * @return `true` if all the faces have been triangulated.
+*
 * @see triangulate_face()
 */
 template <typename PolygonMesh, typename NamedParameters>
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/triangulate_hole.h 5.1-1/include/CGAL/Polygon_mesh_processing/triangulate_hole.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/triangulate_hole.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/triangulate_hole.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/triangulate_hole.h $
-// $Id: triangulate_hole.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/triangulate_hole.h $
+// $Id: triangulate_hole.h a84927d 2020-07-23T17:15:44+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Ilker O. Yaz
@@ -50,21 +50,37 @@ namespace Polygon_mesh_processing {
   @tparam PolygonMesh a model of `MutableFaceGraph`
   @tparam OutputIterator a model of `OutputIterator`
     holding `boost::graph_traits<PolygonMesh>::%face_descriptor` for patch faces.
-  @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+  @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 
   @param pmesh polygon mesh containing the hole
   @param border_halfedge a border halfedge incident to the hole
   @param out iterator over patch faces
-  @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+  @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
   \cgalNamedParamsBegin
-     \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
-         If this parameter is omitted, an internal property map for
-         `CGAL::vertex_point_t` must be available in `PolygonMesh`\cgalParamEnd
-     \cgalParamBegin{use_delaunay_triangulation} if `true`, use the Delaunay triangulation facet search space.
-         If no valid triangulation can be found in this search space, the algorithm falls back to the
-         non-Delaunay triangulations search space to find a solution \cgalParamEnd
-     \cgalParamBegin{geom_traits} a geometric traits class instance \cgalParamEnd
+    \cgalParamNBegin{vertex_point_map}
+      \cgalParamDescription{a property map associating points to the vertices of `pmesh`}
+      \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+                     as key type and `%Point_3` as value type}
+      \cgalParamDefault{`boost::get(CGAL::vertex_point, pmesh)`}
+      \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+                      must be available in `PolygonMesh`.}
+    \cgalParamNEnd
+
+    \cgalParamNBegin{geom_traits}
+      \cgalParamDescription{an instance of a geometric traits class}
+      \cgalParamType{a class model of `Kernel`}
+      \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+      \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+    \cgalParamNEnd
+
+    \cgalParamNBegin{use_delaunay_triangulation}
+      \cgalParamDescription{If `true`, use the Delaunay triangulation facet search space.}
+      \cgalParamType{Boolean}
+      \cgalParamDefault{`true`}
+      \cgalParamExtra{If no valid triangulation can be found in this search space, the algorithm
+                      falls back to the non-Delaunay triangulations search space to find a solution.}
+    \cgalParamNEnd
   \cgalNamedParamsEnd
 
   @return `out`
@@ -89,6 +105,8 @@ namespace Polygon_mesh_processing {
     using parameters::choose_parameter;
     using parameters::get_parameter;
 
+    typedef typename GetGeomTraits<PolygonMesh,NamedParameters>::type         GeomTraits;
+
     bool use_dt3 =
 #ifdef CGAL_HOLE_FILLING_DO_NOT_USE_DT3
       false;
@@ -103,8 +121,7 @@ namespace Polygon_mesh_processing {
       out,
       choose_parameter(get_parameter(np, internal_np::vertex_point), get_property_map(vertex_point, pmesh)),
       use_dt3,
-      choose_parameter(get_parameter(np, internal_np::geom_traits), typename GetGeomTraits<PolygonMesh,NamedParameters>::type()))
-      .first;
+      choose_parameter<GeomTraits>(get_parameter(np, internal_np::geom_traits))).first;
   }
 
   template<typename PolygonMesh, typename OutputIterator>
@@ -142,24 +159,45 @@ namespace Polygon_mesh_processing {
      holding `boost::graph_traits<PolygonMesh>::%face_descriptor` for patch faces.
   @tparam VertexOutputIterator model of `OutputIterator`
      holding `boost::graph_traits<PolygonMesh>::%vertex_descriptor` for patch vertices.
-  @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+  @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 
   @param pmesh polygon mesh which has the hole
   @param border_halfedge a border halfedge incident to the hole
   @param face_out output iterator over patch faces
   @param vertex_out output iterator over patch vertices without including the boundary
-  @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+  @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
   \cgalNamedParamsBegin
-     \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
-         If this parameter is omitted, an internal property map for
-         `CGAL::vertex_point_t` should be available in `PolygonMesh`\cgalParamEnd
-     \cgalParamBegin{density_control_factor} factor to control density of the ouput mesh, where larger values
-         cause denser refinements, as in `refine()` \cgalParamEnd
-     \cgalParamBegin{use_delaunay_triangulation} if `true`, use the Delaunay triangulation facet search space.
-         If no valid triangulation can be found in this search space, the algorithm falls back to the
-         non-Delaunay triangulations search space to find a solution \cgalParamEnd
-     \cgalParamBegin{geom_traits} a geometric traits class instance \cgalParamEnd
+    \cgalParamNBegin{vertex_point_map}
+      \cgalParamDescription{a property map associating points to the vertices of `pmesh`}
+      \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+                     as key type and `%Point_3` as value type}
+      \cgalParamDefault{`boost::get(CGAL::vertex_point, pmesh)`}
+      \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+                      must be available in `PolygonMesh`.}
+    \cgalParamNEnd
+
+    \cgalParamNBegin{geom_traits}
+      \cgalParamDescription{an instance of a geometric traits class}
+      \cgalParamType{a class model of `Kernel`}
+      \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+      \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+    \cgalParamNEnd
+
+    \cgalParamNBegin{use_delaunay_triangulation}
+      \cgalParamDescription{If `true`, use the Delaunay triangulation facet search space.}
+      \cgalParamType{Boolean}
+      \cgalParamDefault{`true`}
+      \cgalParamExtra{If no valid triangulation can be found in this search space, the algorithm
+                      falls back to the non-Delaunay triangulations search space to find a solution.}
+    \cgalParamNEnd
+
+    \cgalParamNBegin{density_control_factor}
+      \cgalParamDescription{factor to control density of the ouput mesh,
+                            where larger values cause denser refinements, as in `refine()`}
+      \cgalParamType{double}
+      \cgalParamDefault{\f$ \sqrt{2}\f$}
+    \cgalParamNEnd
   \cgalNamedParamsEnd
 
   @return pair of `face_out` and `vertex_out`
@@ -212,27 +250,63 @@ namespace Polygon_mesh_processing {
       holding `boost::graph_traits<PolygonMesh>::%face_descriptor` for patch faces
   @tparam VertexOutputIterator model of `OutputIterator`
       holding `boost::graph_traits<PolygonMesh>::%vertex_descriptor` for patch vertices
-  @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+  @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 
   @param pmesh polygon mesh which has the hole
   @param border_halfedge a border halfedge incident to the hole
   @param face_out output iterator over patch faces
   @param vertex_out output iterator over patch vertices without including the boundary
-  @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+  @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
   \cgalNamedParamsBegin
-     \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
-         If this parameter is omitted, an internal property map for
-         `CGAL::vertex_point_t` should be available in `PolygonMesh`
-         \cgalParamEnd
-     \cgalParamBegin{use_delaunay_triangulation} if `true`, use the Delaunay triangulation facet search space.
-         If no valid triangulation can be found in this search space, the algorithm falls back to the
-         non-Delaunay triangulations search space to find a solution \cgalParamEnd
-     \cgalParamBegin{density_control_factor} factor to control density of the ouput mesh, where larger values
-         cause denser refinements, as in `refine()` \cgalParamEnd
-     \cgalParamBegin{fairing_continuity} tangential continuity of the output surface patch \cgalParamEnd
-     \cgalParamBegin{sparse_linear_solver} an instance of the sparse linear solver used for fairing \cgalParamEnd
-     \cgalParamBegin{geom_traits} a geometric traits class instance \cgalParamEnd
+    \cgalParamNBegin{vertex_point_map}
+      \cgalParamDescription{a property map associating points to the vertices of `pmesh`}
+      \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+                     as key type and `%Point_3` as value type}
+      \cgalParamDefault{`boost::get(CGAL::vertex_point, pmesh)`}
+      \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+                      must be available in `PolygonMesh`.}
+    \cgalParamNEnd
+
+    \cgalParamNBegin{geom_traits}
+      \cgalParamDescription{an instance of a geometric traits class}
+      \cgalParamType{a class model of `Kernel`}
+      \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+      \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+    \cgalParamNEnd
+
+    \cgalParamNBegin{use_delaunay_triangulation}
+      \cgalParamDescription{If `true`, use the Delaunay triangulation facet search space.}
+      \cgalParamType{Boolean}
+      \cgalParamDefault{`true`}
+      \cgalParamExtra{If no valid triangulation can be found in this search space, the algorithm
+                      falls back to the non-Delaunay triangulations search space to find a solution.}
+    \cgalParamNEnd
+
+    \cgalParamNBegin{density_control_factor}
+      \cgalParamDescription{factor to control density of the ouput mesh,
+                            where larger values cause denser refinements, as in `refine()`}
+      \cgalParamType{double}
+      \cgalParamDefault{\f$ \sqrt{2}\f$}
+    \cgalParamNEnd
+
+    \cgalParamNBegin{fairing_continuity}
+      \cgalParamDescription{A value controling the tangential continuity of the output surface patch.
+                            The possible values are 0, 1 and 2, refering to the  C<sup>0</sup>, C<sup>1</sup>
+                            and C<sup>2</sup> continuity.}
+      \cgalParamType{unsigned int}
+      \cgalParamDefault{`1`}
+      \cgalParamExtra{The larger `fairing_continuity` gets, the more fixed vertices are required.}
+    \cgalParamNEnd
+
+    \cgalParamNBegin{sparse_linear_solver}
+      \cgalParamDescription{an instance of the sparse linear solver used for fairing}
+      \cgalParamType{a class model of `SparseLinearAlgebraWithFactorTraits_d`}
+      \cgalParamDefault{If \ref thirdpartyEigen "Eigen" 3.2 (or greater) is available and
+                        `CGAL_EIGEN3_ENABLED` is defined, then the following overload of `Eigen_solver_traits`
+                        is provided as default value:\n
+                        `CGAL::Eigen_solver_traits<Eigen::SparseLU<CGAL::Eigen_sparse_matrix<double>::%EigenType, Eigen::COLAMDOrdering<int> > >`}
+    \cgalParamNEnd
   \cgalNamedParamsEnd
 
   @return tuple of
@@ -312,19 +386,29 @@ namespace Polygon_mesh_processing {
      and the corresponding value type `type` must have
      a constructor `type(int p0, int p1, int p2)` available.
      The indices correspond to the ones of input points in `points`.
-  @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+  @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
 
   @param points the range of input points
   @param third_points the range of third points
   @param out iterator over output patch triangles, described by indices of points
              in `points`
-  @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+  @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
   \cgalNamedParamsBegin
-     \cgalParamBegin{use_delaunay_triangulation} if `true`, use the Delaunay triangulation facet search space.
-         If no valid triangulation can be found in this search space, the algorithm falls back to the
-         non-Delaunay triangulations search space to find a solution \cgalParamEnd
-     \cgalParamBegin{geom_traits} a geometric traits class instance \cgalParamEnd
+    \cgalParamNBegin{geom_traits}
+      \cgalParamDescription{an instance of a geometric traits class}
+      \cgalParamType{a class model of `Kernel`}
+      \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+      \cgalParamExtra{The geometric traits class must be compatible with the vertex point type.}
+    \cgalParamNEnd
+
+    \cgalParamNBegin{use_delaunay_triangulation}
+      \cgalParamDescription{If `true`, use the Delaunay triangulation facet search space.}
+      \cgalParamType{Boolean}
+      \cgalParamDefault{`true`}
+      \cgalParamExtra{If no valid triangulation can be found in this search space, the algorithm
+                      falls back to the non-Delaunay triangulations search space to find a solution.}
+    \cgalParamNEnd
   \cgalNamedParamsEnd
 
   \todo handle islands
@@ -363,11 +447,11 @@ namespace Polygon_mesh_processing {
 
     typedef typename PointRange1::iterator InIterator;
     typedef typename std::iterator_traits<InIterator>::value_type Point;
+    typedef typename CGAL::Kernel_traits<Point>::Kernel Kernel;
 
     triangulate_hole_polyline(points, third_points, tracer, WC(),
-      use_dt3,
-      choose_parameter(get_parameter(np, internal_np::geom_traits),
-        typename CGAL::Kernel_traits<Point>::Kernel()));
+                              use_dt3,
+                              choose_parameter<Kernel>(get_parameter(np, internal_np::geom_traits)));
 
     CGAL_assertion(holes.empty());
     return tracer.out;
@@ -387,7 +471,7 @@ namespace Polygon_mesh_processing {
 
   /*!
   \ingroup  hole_filling_grp
-  same as above but the range of third points is omitted. They are not
+  Same as above but the range of third points is omitted. They are not
   taken into account in the cost computation that leads the hole filling.
 */
   template <typename PointRange,
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_processing/Weights.h 5.1-1/include/CGAL/Polygon_mesh_processing/Weights.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_processing/Weights.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_processing/Weights.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/Weights.h $
-// $Id: Weights.h 86ae4be 2020-05-18T12:22:21+02:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/Weights.h $
+// $Id: Weights.h f031add 2020-05-18T12:07:17+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
diff -pruN 5.0.3-1/include/CGAL/polygon_mesh_processing.h 5.1-1/include/CGAL/polygon_mesh_processing.h
--- 5.0.3-1/include/CGAL/polygon_mesh_processing.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/polygon_mesh_processing.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/polygon_mesh_processing.h $
-// $Id: polygon_mesh_processing.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/polygon_mesh_processing.h $
+// $Id: polygon_mesh_processing.h 489869c 2020-03-31T17:49:29+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -20,6 +20,7 @@
 #include <CGAL/Polygon_mesh_processing/orient_polygon_soup.h>
 #include <CGAL/Polygon_mesh_processing/self_intersections.h>
 #include <CGAL/Polygon_mesh_processing/polygon_soup_to_polygon_mesh.h>
+#include <CGAL/Polygon_mesh_processing/polygon_mesh_to_polygon_soup.h>
 #include <CGAL/Polygon_mesh_processing/triangulate_faces.h>
 #include <CGAL/Polygon_mesh_processing/triangulate_hole.h>
 #include <CGAL/Polygon_mesh_processing/compute_normal.h>
@@ -32,6 +33,8 @@
 #include <CGAL/Polygon_mesh_processing/bbox.h>
 #include <CGAL/Polygon_mesh_processing/border.h>
 #include <CGAL/Polygon_mesh_processing/repair.h>
+#include <CGAL/Polygon_mesh_processing/repair_degeneracies.h>
+#include <CGAL/Polygon_mesh_processing/repair_self_intersections.h>
 #include <CGAL/Polygon_mesh_processing/remesh.h>
 #include <CGAL/Polygon_mesh_processing/corefinement.h>
 #include <CGAL/Polygon_mesh_processing/detect_features.h>
@@ -45,6 +48,7 @@
 #include <CGAL/Polygon_mesh_processing/merge_border_vertices.h>
 #include <CGAL/Polygon_mesh_processing/smooth_mesh.h>
 #include <CGAL/Polygon_mesh_processing/smooth_shape.h>
+#include <CGAL/Polygon_mesh_processing/manifoldness.h>
 
 // the named parameter header being not documented the doc is put here for now
 #ifdef DOXYGEN_RUNNING
diff -pruN 5.0.3-1/include/CGAL/Polygon_mesh_slicer.h 5.1-1/include/CGAL/Polygon_mesh_slicer.h
--- 5.0.3-1/include/CGAL/Polygon_mesh_slicer.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_mesh_slicer.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_slicer.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Polygon_mesh_slicer.h $
 // $Id: Polygon_mesh_slicer.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_nop_decomposition_2.h 5.1-1/include/CGAL/Polygon_nop_decomposition_2.h
--- 5.0.3-1/include/CGAL/Polygon_nop_decomposition_2.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_nop_decomposition_2.h	2020-09-08 12:15:11.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Minkowski_sum_2/include/CGAL/Polygon_nop_decomposition_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Minkowski_sum_2/include/CGAL/Polygon_nop_decomposition_2.h $
 // $Id: Polygon_nop_decomposition_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_offset_builder_2.h 5.1-1/include/CGAL/Polygon_offset_builder_2.h
--- 5.0.3-1/include/CGAL/Polygon_offset_builder_2.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_offset_builder_2.h	2020-09-08 12:15:41.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Straight_skeleton_2/include/CGAL/Polygon_offset_builder_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Straight_skeleton_2/include/CGAL/Polygon_offset_builder_2.h $
 // $Id: Polygon_offset_builder_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_offset_builder_traits_2.h 5.1-1/include/CGAL/Polygon_offset_builder_traits_2.h
--- 5.0.3-1/include/CGAL/Polygon_offset_builder_traits_2.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_offset_builder_traits_2.h	2020-09-08 12:15:41.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Straight_skeleton_2/include/CGAL/Polygon_offset_builder_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Straight_skeleton_2/include/CGAL/Polygon_offset_builder_traits_2.h $
 // $Id: Polygon_offset_builder_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_set_2.h 5.1-1/include/CGAL/Polygon_set_2.h
--- 5.0.3-1/include/CGAL/Polygon_set_2.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_set_2.h	2020-09-08 12:14:42.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Boolean_set_operations_2/include/CGAL/Polygon_set_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Boolean_set_operations_2/include/CGAL/Polygon_set_2.h $
 // $Id: Polygon_set_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_traits_2.h 5.1-1/include/CGAL/Polygon_traits_2.h
--- 5.0.3-1/include/CGAL/Polygon_traits_2.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_traits_2.h	2020-09-08 12:15:22.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon/include/CGAL/Polygon_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon/include/CGAL/Polygon_traits_2.h $
 // $Id: Polygon_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_triangulation_decomposition_2.h 5.1-1/include/CGAL/Polygon_triangulation_decomposition_2.h
--- 5.0.3-1/include/CGAL/Polygon_triangulation_decomposition_2.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_triangulation_decomposition_2.h	2020-09-08 12:15:11.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Minkowski_sum_2/include/CGAL/Polygon_triangulation_decomposition_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Minkowski_sum_2/include/CGAL/Polygon_triangulation_decomposition_2.h $
 // $Id: Polygon_triangulation_decomposition_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_vertical_decomposition_2.h 5.1-1/include/CGAL/Polygon_vertical_decomposition_2.h
--- 5.0.3-1/include/CGAL/Polygon_vertical_decomposition_2.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_vertical_decomposition_2.h	2020-09-08 12:15:11.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Minkowski_sum_2/include/CGAL/Polygon_vertical_decomposition_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Minkowski_sum_2/include/CGAL/Polygon_vertical_decomposition_2.h $
 // $Id: Polygon_vertical_decomposition_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polygon_with_holes_2.h 5.1-1/include/CGAL/Polygon_with_holes_2.h
--- 5.0.3-1/include/CGAL/Polygon_with_holes_2.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Polygon_with_holes_2.h	2020-09-08 12:15:22.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon/include/CGAL/Polygon_with_holes_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon/include/CGAL/Polygon_with_holes_2.h $
 // $Id: Polygon_with_holes_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polyhedral_complex_mesh_domain_3.h 5.1-1/include/CGAL/Polyhedral_complex_mesh_domain_3.h
--- 5.0.3-1/include/CGAL/Polyhedral_complex_mesh_domain_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Polyhedral_complex_mesh_domain_3.h	2020-09-08 12:15:09.000000000 +0000
@@ -4,8 +4,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Polyhedral_complex_mesh_domain_3.h $
-// $Id: Polyhedral_complex_mesh_domain_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Polyhedral_complex_mesh_domain_3.h $
+// $Id: Polyhedral_complex_mesh_domain_3.h c1a42c7 2020-03-26T18:58:21+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -750,8 +750,7 @@ detect_features(FT angle_in_degree,
   for(Polyhedron_type& p : poly)
   {
     initialize_ts(p);
-    using Mesh_3::internal::Get_face_index_pmap;
-    Get_face_index_pmap<Polyhedron_type> get_face_index_pmap(p);
+
 #ifdef CGAL_MESH_3_VERBOSE
     std::size_t poly_id = &p-&poly[0];
     std::cerr << "Polyhedron #" << poly_id << " :\n";
@@ -768,9 +767,9 @@ detect_features(FT angle_in_degree,
       , eif
       , pid_map
       , PMP::parameters::first_index(nb_of_patch_plus_one)
-      .face_index_map(get_face_index_pmap(p))
-      .vertex_incident_patches_map(vip_map)
-      .vertex_feature_degree_map(vertex_feature_degree_map));
+                        .face_index_map(get_initialized_face_index_map(p))
+                        .vertex_incident_patches_map(vip_map)
+                        .vertex_feature_degree_map(vertex_feature_degree_map));
 
     Mesh_3::internal::Is_featured_edge<Polyhedron_type> is_featured_edge(p);
 
@@ -990,7 +989,7 @@ add_featured_edges_to_graph(const Polyhe
     }
   }
 
-#if CGAL_MESH_3_PROTECTION_DEBUG > 1
+#if CGAL_MESH_3_PROTECTION_DEBUG & 2
   {// DEBUG
     Mesh_3::internal::dump_graph_edges("edges-graph.polylines.txt", g_copy);
   }
diff -pruN 5.0.3-1/include/CGAL/Polyhedral_mesh_domain_3.h 5.1-1/include/CGAL/Polyhedral_mesh_domain_3.h
--- 5.0.3-1/include/CGAL/Polyhedral_mesh_domain_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Polyhedral_mesh_domain_3.h	2020-09-08 12:15:09.000000000 +0000
@@ -4,8 +4,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Polyhedral_mesh_domain_3.h $
-// $Id: Polyhedral_mesh_domain_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Polyhedral_mesh_domain_3.h $
+// $Id: Polyhedral_mesh_domain_3.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -105,47 +105,6 @@ struct IGT_generator<Gt,CGAL::Tag_false>
 }  // end namespace details
 }  // end namespace Mesh_3
 
-namespace Mesh_3 {
-namespace internal {
-
-template <typename Polyhedron_type,
-          bool = CGAL::graph_has_property<Polyhedron_type,
-                                           CGAL::face_index_t>::value>
-class Get_face_index_pmap {
-public:
-  typedef typename boost::property_map<Polyhedron_type,
-                                       CGAL::face_index_t>::const_type Pmap;
-  Get_face_index_pmap(const Polyhedron_type&) {}
-  Pmap operator()(const Polyhedron_type& polyhedron) {
-    return get(CGAL::face_index, polyhedron);
-  }
-};
-
-template <typename Polyhedron_type>
-class Get_face_index_pmap<Polyhedron_type, false> {
-  typedef typename boost::graph_traits<Polyhedron_type>::face_descriptor
-                                                              face_descriptor;
-  typedef std::map<face_descriptor, int> Map;
-public:
-  Get_face_index_pmap(const Polyhedron_type& polyhedron) {
-    int id = 0;
-    for(face_descriptor f : faces(polyhedron))
-    {
-      face_ids[f] = id++;
-    }
-  }
-  typedef boost::associative_property_map<Map> Pmap;
-
-  Pmap operator()(const Polyhedron_type&) {
-    return Pmap(face_ids);
-  }
-private:
-  Map face_ids;
-};
-
-} // end namespace internal
-} // end namespace Mesh_3
-
 /**
  * @class Polyhedral_mesh_domain_3
  *
@@ -206,7 +165,7 @@ public:
   struct Primitive_type {
       //setting OneFaceGraphPerTree to false transforms the id type into
       //std::pair<FD, const FaceGraph*>.
-    typedef AABB_face_graph_triangle_primitive<P, typename boost::property_map<P,vertex_point_t>::type, CGAL::Tag_false> type;
+    typedef AABB_face_graph_triangle_primitive<P, typename boost::property_map<P,vertex_point_t>::const_type, CGAL::Tag_false> type;
 
     static
     typename IGT_::Triangle_3 datum(const typename type::Id primitive_id) {
diff -pruN 5.0.3-1/include/CGAL/Polyhedral_mesh_domain_with_features_3.h 5.1-1/include/CGAL/Polyhedral_mesh_domain_with_features_3.h
--- 5.0.3-1/include/CGAL/Polyhedral_mesh_domain_with_features_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Polyhedral_mesh_domain_with_features_3.h	2020-09-08 12:15:09.000000000 +0000
@@ -4,8 +4,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Polyhedral_mesh_domain_with_features_3.h $
-// $Id: Polyhedral_mesh_domain_with_features_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Polyhedral_mesh_domain_with_features_3.h $
+// $Id: Polyhedral_mesh_domain_with_features_3.h c1a42c7 2020-03-26T18:58:21+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -28,7 +28,6 @@
 #include <CGAL/Polyhedral_mesh_domain_3.h>
 #include <CGAL/Mesh_domain_with_polyline_features_3.h>
 #include <CGAL/Mesh_polyhedron_3.h>
-#include <CGAL/Mesh_3/Detect_polylines_in_polyhedra.h>
 #include <CGAL/Mesh_3/Polyline_with_context.h>
 
 #include <CGAL/IO/Polyhedron_iostream.h>
@@ -307,7 +306,7 @@ void dump_graph_edges(std::ostream& out,
   typedef typename boost::graph_traits<Graph>::edge_descriptor edge_descriptor;
 
   out.precision(17);
-  for(edge_descriptor e : edges(g))
+  for(edge_descriptor e : make_range(edges(g)))
   {
     vertex_descriptor s = source(e, g);
     vertex_descriptor t = target(e, g);
@@ -349,9 +348,6 @@ detect_features(FT angle_in_degree, std:
     typedef typename boost::property_map<Polyhedron,CGAL::vertex_incident_patches_t<P_id> >::type VIPMap;
     typedef typename boost::property_map<Polyhedron, CGAL::edge_is_feature_t>::type EIFMap;
 
-    using Mesh_3::internal::Get_face_index_pmap;
-    Get_face_index_pmap<Polyhedron> get_face_index_pmap(p);
-
     PIDMap pid_map = get(face_patch_id_t<Tag_>(), p);
     VIPMap vip_map = get(vertex_incident_patches_t<P_id>(), p);
     EIFMap eif_map = get(CGAL::edge_is_feature, p);
@@ -361,8 +357,8 @@ detect_features(FT angle_in_degree, std:
       , eif_map
       , pid_map
       , PMP::parameters::first_index(nb_of_patch_plus_one)
-      .face_index_map(get_face_index_pmap(p))
-      .vertex_incident_patches_map(vip_map));
+                        .face_index_map(get_initialized_face_index_map(p))
+                        .vertex_incident_patches_map(vip_map));
 
     Mesh_3::internal::Is_featured_edge<Polyhedron> is_featured_edge(p);
 
@@ -404,7 +400,7 @@ add_features_from_split_graph_into_polyl
   this->add_features_with_context(polylines.begin(),
                                   polylines.end());
 
-#if CGAL_MESH_3_PROTECTION_DEBUG > 1
+#if CGAL_MESH_3_PROTECTION_DEBUG & 2
   {//DEBUG
     std::ofstream og("polylines_graph.polylines.txt");
     og.precision(17);
@@ -417,7 +413,7 @@ add_features_from_split_graph_into_polyl
     }
     og.close();
   }
-#endif // CGAL_MESH_3_PROTECTION_DEBUG > 1
+#endif // CGAL_MESH_3_PROTECTION_DEBUG & 2
 
 }
 
@@ -476,7 +472,7 @@ add_featured_edges_to_graph(const Polyhe
     }
   }
 
-#if CGAL_MESH_3_PROTECTION_DEBUG > 1
+#if CGAL_MESH_3_PROTECTION_DEBUG & 2
   {// DEBUG
     dump_graph_edges("edges-graph.polylines.txt", g_copy);
   }
diff -pruN 5.0.3-1/include/CGAL/PolyhedralSurf_neighbors.h 5.1-1/include/CGAL/PolyhedralSurf_neighbors.h
--- 5.0.3-1/include/CGAL/PolyhedralSurf_neighbors.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/PolyhedralSurf_neighbors.h	2020-09-08 12:15:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Ridges_3/include/CGAL/PolyhedralSurf_neighbors.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Ridges_3/include/CGAL/PolyhedralSurf_neighbors.h $
 // $Id: PolyhedralSurf_neighbors.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polyhedron_3_fwd.h 5.1-1/include/CGAL/Polyhedron_3_fwd.h
--- 5.0.3-1/include/CGAL/Polyhedron_3_fwd.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Polyhedron_3_fwd.h	2020-09-08 12:14:54.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/Polyhedron_3_fwd.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/Polyhedron_3_fwd.h $
 // $Id: Polyhedron_3_fwd.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polyhedron_3.h 5.1-1/include/CGAL/Polyhedron_3.h
--- 5.0.3-1/include/CGAL/Polyhedron_3.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/Polyhedron_3.h	2020-09-08 12:15:28.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron/include/CGAL/Polyhedron_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron/include/CGAL/Polyhedron_3.h $
 // $Id: Polyhedron_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polyhedron_3_to_lcc.h 5.1-1/include/CGAL/Polyhedron_3_to_lcc.h
--- 5.0.3-1/include/CGAL/Polyhedron_3_to_lcc.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/Polyhedron_3_to_lcc.h	2020-09-08 12:15:28.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron/include/CGAL/Polyhedron_3_to_lcc.h $
-// $Id: Polyhedron_3_to_lcc.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron/include/CGAL/Polyhedron_3_to_lcc.h $
+// $Id: Polyhedron_3_to_lcc.h 9a50c0f 2019-11-26T13:22:03+01:00 Guillaume Damiand
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
@@ -13,6 +13,7 @@
 #ifndef CGAL_POLYHEDRON_3_TO_LCC_H
 #define CGAL_POLYHEDRON_3_TO_LCC_H
 
+#include <CGAL/Polyhedron_3.h>
 #include <CGAL/assertions.h>
 #include <iostream>
 #include <map>
diff -pruN 5.0.3-1/include/CGAL/Polyhedron_copy_3.h 5.1-1/include/CGAL/Polyhedron_copy_3.h
--- 5.0.3-1/include/CGAL/Polyhedron_copy_3.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/Polyhedron_copy_3.h	2020-09-08 12:15:28.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron/include/CGAL/Polyhedron_copy_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron/include/CGAL/Polyhedron_copy_3.h $
 // $Id: Polyhedron_copy_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/polyhedron_cut_plane_3.h 5.1-1/include/CGAL/polyhedron_cut_plane_3.h
--- 5.0.3-1/include/CGAL/polyhedron_cut_plane_3.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/polyhedron_cut_plane_3.h	2020-09-08 12:15:28.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron/include/CGAL/polyhedron_cut_plane_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron/include/CGAL/polyhedron_cut_plane_3.h $
 // $Id: polyhedron_cut_plane_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polyhedron_incremental_builder_3.h 5.1-1/include/CGAL/Polyhedron_incremental_builder_3.h
--- 5.0.3-1/include/CGAL/Polyhedron_incremental_builder_3.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/Polyhedron_incremental_builder_3.h	2020-09-08 12:15:28.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron/include/CGAL/Polyhedron_incremental_builder_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron/include/CGAL/Polyhedron_incremental_builder_3.h $
 // $Id: Polyhedron_incremental_builder_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polyhedron_items_3.h 5.1-1/include/CGAL/Polyhedron_items_3.h
--- 5.0.3-1/include/CGAL/Polyhedron_items_3.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Polyhedron_items_3.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/HalfedgeDS/include/CGAL/Polyhedron_items_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/HalfedgeDS/include/CGAL/Polyhedron_items_3.h $
 // $Id: Polyhedron_items_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polyhedron_items_with_id_3.h 5.1-1/include/CGAL/Polyhedron_items_with_id_3.h
--- 5.0.3-1/include/CGAL/Polyhedron_items_with_id_3.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/Polyhedron_items_with_id_3.h	2020-09-08 12:15:28.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron/include/CGAL/Polyhedron_items_with_id_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron/include/CGAL/Polyhedron_items_with_id_3.h $
 // $Id: Polyhedron_items_with_id_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polyhedron_min_items_3.h 5.1-1/include/CGAL/Polyhedron_min_items_3.h
--- 5.0.3-1/include/CGAL/Polyhedron_min_items_3.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/Polyhedron_min_items_3.h	2020-09-08 12:15:28.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron/include/CGAL/Polyhedron_min_items_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron/include/CGAL/Polyhedron_min_items_3.h $
 // $Id: Polyhedron_min_items_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polyhedron_traits_3.h 5.1-1/include/CGAL/Polyhedron_traits_3.h
--- 5.0.3-1/include/CGAL/Polyhedron_traits_3.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/Polyhedron_traits_3.h	2020-09-08 12:15:28.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron/include/CGAL/Polyhedron_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron/include/CGAL/Polyhedron_traits_3.h $
 // $Id: Polyhedron_traits_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polyhedron_traits_with_normals_3.h 5.1-1/include/CGAL/Polyhedron_traits_with_normals_3.h
--- 5.0.3-1/include/CGAL/Polyhedron_traits_with_normals_3.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/Polyhedron_traits_with_normals_3.h	2020-09-08 12:15:28.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyhedron/include/CGAL/Polyhedron_traits_with_normals_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyhedron/include/CGAL/Polyhedron_traits_with_normals_3.h $
 // $Id: Polyhedron_traits_with_normals_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polyline_simplification_2/Hybrid_squared_distance_cost.h 5.1-1/include/CGAL/Polyline_simplification_2/Hybrid_squared_distance_cost.h
--- 5.0.3-1/include/CGAL/Polyline_simplification_2/Hybrid_squared_distance_cost.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/Polyline_simplification_2/Hybrid_squared_distance_cost.h	2020-09-08 12:15:28.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyline_simplification_2/include/CGAL/Polyline_simplification_2/Hybrid_squared_distance_cost.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyline_simplification_2/include/CGAL/Polyline_simplification_2/Hybrid_squared_distance_cost.h $
 // $Id: Hybrid_squared_distance_cost.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polyline_simplification_2/Scaled_squared_distance_cost.h 5.1-1/include/CGAL/Polyline_simplification_2/Scaled_squared_distance_cost.h
--- 5.0.3-1/include/CGAL/Polyline_simplification_2/Scaled_squared_distance_cost.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/Polyline_simplification_2/Scaled_squared_distance_cost.h	2020-09-08 12:15:28.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyline_simplification_2/include/CGAL/Polyline_simplification_2/Scaled_squared_distance_cost.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyline_simplification_2/include/CGAL/Polyline_simplification_2/Scaled_squared_distance_cost.h $
 // $Id: Scaled_squared_distance_cost.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polyline_simplification_2/simplify.h 5.1-1/include/CGAL/Polyline_simplification_2/simplify.h
--- 5.0.3-1/include/CGAL/Polyline_simplification_2/simplify.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/Polyline_simplification_2/simplify.h	2020-09-08 12:15:28.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyline_simplification_2/include/CGAL/Polyline_simplification_2/simplify.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyline_simplification_2/include/CGAL/Polyline_simplification_2/simplify.h $
 // $Id: simplify.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polyline_simplification_2/Squared_distance_cost.h 5.1-1/include/CGAL/Polyline_simplification_2/Squared_distance_cost.h
--- 5.0.3-1/include/CGAL/Polyline_simplification_2/Squared_distance_cost.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/Polyline_simplification_2/Squared_distance_cost.h	2020-09-08 12:15:28.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyline_simplification_2/include/CGAL/Polyline_simplification_2/Squared_distance_cost.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyline_simplification_2/include/CGAL/Polyline_simplification_2/Squared_distance_cost.h $
 // $Id: Squared_distance_cost.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polyline_simplification_2/Stop_above_cost_threshold.h 5.1-1/include/CGAL/Polyline_simplification_2/Stop_above_cost_threshold.h
--- 5.0.3-1/include/CGAL/Polyline_simplification_2/Stop_above_cost_threshold.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/Polyline_simplification_2/Stop_above_cost_threshold.h	2020-09-08 12:15:28.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyline_simplification_2/include/CGAL/Polyline_simplification_2/Stop_above_cost_threshold.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyline_simplification_2/include/CGAL/Polyline_simplification_2/Stop_above_cost_threshold.h $
 // $Id: Stop_above_cost_threshold.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polyline_simplification_2/Stop_below_count_ratio_threshold.h 5.1-1/include/CGAL/Polyline_simplification_2/Stop_below_count_ratio_threshold.h
--- 5.0.3-1/include/CGAL/Polyline_simplification_2/Stop_below_count_ratio_threshold.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/Polyline_simplification_2/Stop_below_count_ratio_threshold.h	2020-09-08 12:15:28.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyline_simplification_2/include/CGAL/Polyline_simplification_2/Stop_below_count_ratio_threshold.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyline_simplification_2/include/CGAL/Polyline_simplification_2/Stop_below_count_ratio_threshold.h $
 // $Id: Stop_below_count_ratio_threshold.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polyline_simplification_2/Stop_below_count_threshold.h 5.1-1/include/CGAL/Polyline_simplification_2/Stop_below_count_threshold.h
--- 5.0.3-1/include/CGAL/Polyline_simplification_2/Stop_below_count_threshold.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/Polyline_simplification_2/Stop_below_count_threshold.h	2020-09-08 12:15:28.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyline_simplification_2/include/CGAL/Polyline_simplification_2/Stop_below_count_threshold.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyline_simplification_2/include/CGAL/Polyline_simplification_2/Stop_below_count_threshold.h $
 // $Id: Stop_below_count_threshold.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polyline_simplification_2/Vertex_base_2.h 5.1-1/include/CGAL/Polyline_simplification_2/Vertex_base_2.h
--- 5.0.3-1/include/CGAL/Polyline_simplification_2/Vertex_base_2.h	2020-08-01 19:02:45.000000000 +0000
+++ 5.1-1/include/CGAL/Polyline_simplification_2/Vertex_base_2.h	2020-09-08 12:15:28.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polyline_simplification_2/include/CGAL/Polyline_simplification_2/Vertex_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polyline_simplification_2/include/CGAL/Polyline_simplification_2/Vertex_base_2.h $
 // $Id: Vertex_base_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomial/Algebraic_structure_traits.h 5.1-1/include/CGAL/Polynomial/Algebraic_structure_traits.h
--- 5.0.3-1/include/CGAL/Polynomial/Algebraic_structure_traits.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomial/Algebraic_structure_traits.h	2020-09-08 12:15:29.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/Polynomial/Algebraic_structure_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/Polynomial/Algebraic_structure_traits.h $
 // $Id: Algebraic_structure_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomial/bezout_matrix.h 5.1-1/include/CGAL/Polynomial/bezout_matrix.h
--- 5.0.3-1/include/CGAL/Polynomial/bezout_matrix.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomial/bezout_matrix.h	2020-09-08 12:15:29.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/Polynomial/bezout_matrix.h $
-// $Id: bezout_matrix.h 5b2d46c 2020-04-21T10:03:00+02:00 Maxime Gimeno
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/Polynomial/bezout_matrix.h $
+// $Id: bezout_matrix.h e9d41d7 2020-04-21T10:03:00+02:00 Maxime Gimeno
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomial/Cached_extended_euclidean_algorithm.h 5.1-1/include/CGAL/Polynomial/Cached_extended_euclidean_algorithm.h
--- 5.0.3-1/include/CGAL/Polynomial/Cached_extended_euclidean_algorithm.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomial/Cached_extended_euclidean_algorithm.h	2020-09-08 12:15:29.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/Polynomial/Cached_extended_euclidean_algorithm.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/Polynomial/Cached_extended_euclidean_algorithm.h $
 // $Id: Cached_extended_euclidean_algorithm.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomial/Chinese_remainder_traits.h 5.1-1/include/CGAL/Polynomial/Chinese_remainder_traits.h
--- 5.0.3-1/include/CGAL/Polynomial/Chinese_remainder_traits.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomial/Chinese_remainder_traits.h	2020-09-08 12:15:29.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/Polynomial/Chinese_remainder_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/Polynomial/Chinese_remainder_traits.h $
 // $Id: Chinese_remainder_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomial/Coercion_traits.h 5.1-1/include/CGAL/Polynomial/Coercion_traits.h
--- 5.0.3-1/include/CGAL/Polynomial/Coercion_traits.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomial/Coercion_traits.h	2020-09-08 12:15:29.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/Polynomial/Coercion_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/Polynomial/Coercion_traits.h $
 // $Id: Coercion_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomial/Degree.h 5.1-1/include/CGAL/Polynomial/Degree.h
--- 5.0.3-1/include/CGAL/Polynomial/Degree.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomial/Degree.h	2020-09-08 12:15:29.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/Polynomial/Degree.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/Polynomial/Degree.h $
 // $Id: Degree.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomial/determinant.h 5.1-1/include/CGAL/Polynomial/determinant.h
--- 5.0.3-1/include/CGAL/Polynomial/determinant.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomial/determinant.h	2020-09-08 12:15:29.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/Polynomial/determinant.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/Polynomial/determinant.h $
 // $Id: determinant.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomial/Fraction_traits.h 5.1-1/include/CGAL/Polynomial/Fraction_traits.h
--- 5.0.3-1/include/CGAL/Polynomial/Fraction_traits.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomial/Fraction_traits.h	2020-09-08 12:15:29.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/Polynomial/Fraction_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/Polynomial/Fraction_traits.h $
 // $Id: Fraction_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomial/fwd.h 5.1-1/include/CGAL/Polynomial/fwd.h
--- 5.0.3-1/include/CGAL/Polynomial/fwd.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomial/fwd.h	2020-09-08 12:15:29.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/Polynomial/fwd.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/Polynomial/fwd.h $
 // $Id: fwd.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomial/Get_arithmetic_kernel.h 5.1-1/include/CGAL/Polynomial/Get_arithmetic_kernel.h
--- 5.0.3-1/include/CGAL/Polynomial/Get_arithmetic_kernel.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomial/Get_arithmetic_kernel.h	2020-09-08 12:15:29.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/Polynomial/Get_arithmetic_kernel.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/Polynomial/Get_arithmetic_kernel.h $
 // $Id: Get_arithmetic_kernel.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomial/hgdelta_update.h 5.1-1/include/CGAL/Polynomial/hgdelta_update.h
--- 5.0.3-1/include/CGAL/Polynomial/hgdelta_update.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomial/hgdelta_update.h	2020-09-08 12:15:29.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/Polynomial/hgdelta_update.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/Polynomial/hgdelta_update.h $
 // $Id: hgdelta_update.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomial/Interpolator.h 5.1-1/include/CGAL/Polynomial/Interpolator.h
--- 5.0.3-1/include/CGAL/Polynomial/Interpolator.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomial/Interpolator.h	2020-09-08 12:15:29.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/Polynomial/Interpolator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/Polynomial/Interpolator.h $
 // $Id: Interpolator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomial/misc.h 5.1-1/include/CGAL/Polynomial/misc.h
--- 5.0.3-1/include/CGAL/Polynomial/misc.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomial/misc.h	2020-09-08 12:15:29.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/Polynomial/misc.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/Polynomial/misc.h $
 // $Id: misc.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomial/modular_filter.h 5.1-1/include/CGAL/Polynomial/modular_filter.h
--- 5.0.3-1/include/CGAL/Polynomial/modular_filter.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomial/modular_filter.h	2020-09-08 12:15:29.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/Polynomial/modular_filter.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/Polynomial/modular_filter.h $
 // $Id: modular_filter.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomial/modular_gcd.h 5.1-1/include/CGAL/Polynomial/modular_gcd.h
--- 5.0.3-1/include/CGAL/Polynomial/modular_gcd.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomial/modular_gcd.h	2020-09-08 12:15:29.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/Polynomial/modular_gcd.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/Polynomial/modular_gcd.h $
 // $Id: modular_gcd.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomial/modular_gcd_utcf_algorithm_M.h 5.1-1/include/CGAL/Polynomial/modular_gcd_utcf_algorithm_M.h
--- 5.0.3-1/include/CGAL/Polynomial/modular_gcd_utcf_algorithm_M.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomial/modular_gcd_utcf_algorithm_M.h	2020-09-08 12:15:29.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/Polynomial/modular_gcd_utcf_algorithm_M.h $
-// $Id: modular_gcd_utcf_algorithm_M.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/Polynomial/modular_gcd_utcf_algorithm_M.h $
+// $Id: modular_gcd_utcf_algorithm_M.h fdb17cb 2020-03-26T19:26:10+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -109,7 +109,7 @@ Polynomial<NT> modular_gcd_utcf_algorith
     MScalar mg_;
     MPoly   mF1,mF2,mG_;
 
-    typename CRT::Scalar_type p,q,pq,s,t;
+    typename CRT::Scalar_type p(0),q(0),pq,s,t;
     Poly Gs,H1s,H2s, Gs_old; // s =^ star
 #ifdef CGAL_MODULAR_GCD_TIMER
     timer_init.stop();
diff -pruN 5.0.3-1/include/CGAL/Polynomial/modular_gcd_utcf_dfai.h 5.1-1/include/CGAL/Polynomial/modular_gcd_utcf_dfai.h
--- 5.0.3-1/include/CGAL/Polynomial/modular_gcd_utcf_dfai.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomial/modular_gcd_utcf_dfai.h	2020-09-08 12:15:29.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/Polynomial/modular_gcd_utcf_dfai.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/Polynomial/modular_gcd_utcf_dfai.h $
 // $Id: modular_gcd_utcf_dfai.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomial/modular_gcd_utils.h 5.1-1/include/CGAL/Polynomial/modular_gcd_utils.h
--- 5.0.3-1/include/CGAL/Polynomial/modular_gcd_utils.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomial/modular_gcd_utils.h	2020-09-08 12:15:29.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/Polynomial/modular_gcd_utils.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/Polynomial/modular_gcd_utils.h $
 // $Id: modular_gcd_utils.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomial/Modular_traits.h 5.1-1/include/CGAL/Polynomial/Modular_traits.h
--- 5.0.3-1/include/CGAL/Polynomial/Modular_traits.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomial/Modular_traits.h	2020-09-08 12:15:29.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/Polynomial/Modular_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/Polynomial/Modular_traits.h $
 // $Id: Modular_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomial/Monomial_representation.h 5.1-1/include/CGAL/Polynomial/Monomial_representation.h
--- 5.0.3-1/include/CGAL/Polynomial/Monomial_representation.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomial/Monomial_representation.h	2020-09-08 12:15:29.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/Polynomial/Monomial_representation.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/Polynomial/Monomial_representation.h $
 // $Id: Monomial_representation.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomial/polynomial_gcd.h 5.1-1/include/CGAL/Polynomial/polynomial_gcd.h
--- 5.0.3-1/include/CGAL/Polynomial/polynomial_gcd.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomial/polynomial_gcd.h	2020-09-08 12:15:29.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/Polynomial/polynomial_gcd.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/Polynomial/polynomial_gcd.h $
 // $Id: polynomial_gcd.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomial/polynomial_gcd_implementations.h 5.1-1/include/CGAL/Polynomial/polynomial_gcd_implementations.h
--- 5.0.3-1/include/CGAL/Polynomial/polynomial_gcd_implementations.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomial/polynomial_gcd_implementations.h	2020-09-08 12:15:29.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/Polynomial/polynomial_gcd_implementations.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/Polynomial/polynomial_gcd_implementations.h $
 // $Id: polynomial_gcd_implementations.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomial/polynomial_gcd_ntl.h 5.1-1/include/CGAL/Polynomial/polynomial_gcd_ntl.h
--- 5.0.3-1/include/CGAL/Polynomial/polynomial_gcd_ntl.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomial/polynomial_gcd_ntl.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/Polynomial/polynomial_gcd_ntl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/Polynomial/polynomial_gcd_ntl.h $
 // $Id: polynomial_gcd_ntl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomial/Polynomial_type.h 5.1-1/include/CGAL/Polynomial/Polynomial_type.h
--- 5.0.3-1/include/CGAL/Polynomial/Polynomial_type.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomial/Polynomial_type.h	2020-09-08 12:15:29.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/Polynomial/Polynomial_type.h $
-// $Id: Polynomial_type.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/Polynomial/Polynomial_type.h $
+// $Id: Polynomial_type.h 257a92d 2020-05-22T12:22:53+02:00 Marc Glisse
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -957,6 +957,52 @@ public:
     }
 
     friend Polynomial<NT> operator - <> (const Polynomial<NT>&);
+
+    //
+    // Comparison Operators
+    //
+
+    // polynomials only
+    friend bool operator == (const Polynomial& p1, const Polynomial& p2) {
+      CGAL_precondition(p1.degree() >= 0);
+      CGAL_precondition(p2.degree() >= 0);
+      if (p1.is_identical(p2)) return true;
+      if (p1.degree() != p2.degree()) return false;
+      for (int i = p1.degree(); i >= 0; i--) if (p1[i] != p2[i]) return false;
+      return true;
+    }
+    friend bool operator < (const Polynomial& p1, const Polynomial& p2)
+    { return ( p1.compare(p2) < 0 ); }
+
+    // operators NT
+    friend bool operator == (const Polynomial& p, const NT& num)  {
+      CGAL_precondition(p.degree() >= 0);
+      return p.degree() == 0 && p[0] == num;
+    }
+    friend bool operator < (const Polynomial& p,const NT& num)
+    { return ( p.compare(num) < 0 );}
+    friend bool operator > (const Polynomial& p,const NT& num)
+    { return ( p.compare(num) > 0 );}
+
+    // compare int #################################
+    friend bool operator == (const Polynomial& p, const CGAL_int(NT)& num)  {
+      CGAL_precondition(p.degree() >= 0);
+      return p.degree() == 0 && p[0] == NT(num);
+    }
+    friend bool operator < (const Polynomial& p, const CGAL_int(NT)& num)
+    { return ( p.compare(NT(num)) < 0 );}
+    friend bool operator > (const Polynomial& p, const CGAL_int(NT)& num)
+    { return ( p.compare(NT(num)) > 0 );}
+
+    // compare icoeff ###################################
+    friend bool operator == (const Polynomial& p, const CGAL_icoeff(NT)& num)  {
+      CGAL_precondition(p.degree() >= 0);
+      return p.degree() == 0 && p[0] == NT(num);
+    }
+    friend bool operator < (const Polynomial& p, const CGAL_icoeff(NT)& num)
+    { return ( p.compare(NT(num)) < 0 );}
+    friend bool operator > (const Polynomial& p, const CGAL_icoeff(NT)& num)
+    { return ( p.compare(NT(num)) > 0 );}
 }; // class Polynomial<NT_>
 
 // Arithmetic Operators, Part III:
@@ -1000,102 +1046,6 @@ Polynomial<NT> operator * (const Polynom
 
 
 //
-// Comparison Operators
-//
-
-// polynomials only
-template <class NT> inline
-bool operator == (const Polynomial<NT>& p1, const Polynomial<NT>& p2) {
-  CGAL_precondition(p1.degree() >= 0);
-  CGAL_precondition(p2.degree() >= 0);
-  if (p1.is_identical(p2)) return true;
-  if (p1.degree() != p2.degree()) return false;
-  for (int i = p1.degree(); i >= 0; i--) if (p1[i] != p2[i]) return false;
-  return true;
-}
-template <class NT> inline
-bool operator < (const Polynomial<NT>& p1, const Polynomial<NT>& p2)
-{ return ( p1.compare(p2) < 0 ); }
-template <class NT> inline
-bool operator > (const Polynomial<NT>& p1, const Polynomial<NT>& p2)
-{ return ( p1.compare(p2) > 0 ); }
-
-// operators NT
-template <class NT> inline
-bool operator == (const NT& num, const Polynomial<NT>& p) {
-  CGAL_precondition(p.degree() >= 0);
-  return p.degree() == 0 && p[0] == num;
-}
-template <class NT> inline
-bool operator == (const Polynomial<NT>& p, const NT& num)  {
-  CGAL_precondition(p.degree() >= 0);
-  return p.degree() == 0 && p[0] == num;
-}
-template <class NT> inline
-bool operator < (const NT& num, const Polynomial<NT>& p)
-{ return ( p.compare(num) > 0 );}
-template <class NT> inline
-bool operator < (const Polynomial<NT>& p,const NT& num)
-{ return ( p.compare(num) < 0 );}
-template <class NT> inline
-bool operator > (const NT& num, const Polynomial<NT>& p)
-{ return ( p.compare(num) < 0 );}
-template <class NT> inline
-bool operator > (const Polynomial<NT>& p,const NT& num)
-{ return ( p.compare(num) > 0 );}
-
-
-// compare int #################################
-template <class NT> inline
-bool operator == (const CGAL_int(NT)& num, const Polynomial<NT>& p)  {
-  CGAL_precondition(p.degree() >= 0);
-  return p.degree() == 0 && p[0] == NT(num);
-}
-template <class NT> inline
-bool operator == (const Polynomial<NT>& p, const CGAL_int(NT)& num)  {
-  CGAL_precondition(p.degree() >= 0);
-  return p.degree() == 0 && p[0] == NT(num);
-}
-template <class NT> inline
-bool operator < (const CGAL_int(NT)& num, const Polynomial<NT>& p)
-{ return ( p.compare(NT(num)) > 0 );}
-template <class NT> inline
-bool operator < (const Polynomial<NT>& p, const CGAL_int(NT)& num)
-{ return ( p.compare(NT(num)) < 0 );}
-template <class NT> inline
-bool operator > (const CGAL_int(NT)& num, const Polynomial<NT>& p)
-{ return ( p.compare(NT(num)) < 0 );}
-template <class NT> inline
-bool operator > (const Polynomial<NT>& p, const CGAL_int(NT)& num)
-{ return ( p.compare(NT(num)) > 0 );}
-
-// compare icoeff ###################################
-template <class NT> inline
-bool operator == (const CGAL_icoeff(NT)& num, const Polynomial<NT>& p)  {
-  CGAL_precondition(p.degree() >= 0);
-  return p.degree() == 0 && p[0] == NT(num);
-}
-template <class NT> inline
-bool operator == (const Polynomial<NT>& p, const CGAL_icoeff(NT)& num)  {
-  CGAL_precondition(p.degree() >= 0);
-  return p.degree() == 0 && p[0] == NT(num);
-}
-template <class NT> inline
-bool operator < (const CGAL_icoeff(NT)& num, const Polynomial<NT>& p)
-{ return ( p.compare(NT(num)) > 0 );}
-template <class NT> inline
-bool operator < (const Polynomial<NT>& p, const CGAL_icoeff(NT)& num)
-{ return ( p.compare(NT(num)) < 0 );}
-
-
-template <class NT> inline
-bool operator > (const CGAL_icoeff(NT)& num, const Polynomial<NT>& p)
-{ return ( p.compare(NT(num)) < 0 );}
-template <class NT> inline
-bool operator > (const Polynomial<NT>& p, const CGAL_icoeff(NT)& num)
-{ return ( p.compare(NT(num)) > 0 );}
-
-//
 // Algebraically non-trivial operations
 //
 
diff -pruN 5.0.3-1/include/CGAL/Polynomial/prs_resultant.h 5.1-1/include/CGAL/Polynomial/prs_resultant.h
--- 5.0.3-1/include/CGAL/Polynomial/prs_resultant.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomial/prs_resultant.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/Polynomial/prs_resultant.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/Polynomial/prs_resultant.h $
 // $Id: prs_resultant.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomial/Real_embeddable_traits.h 5.1-1/include/CGAL/Polynomial/Real_embeddable_traits.h
--- 5.0.3-1/include/CGAL/Polynomial/Real_embeddable_traits.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomial/Real_embeddable_traits.h	2020-09-08 12:15:29.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/Polynomial/Real_embeddable_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/Polynomial/Real_embeddable_traits.h $
 // $Id: Real_embeddable_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomial/resultant.h 5.1-1/include/CGAL/Polynomial/resultant.h
--- 5.0.3-1/include/CGAL/Polynomial/resultant.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomial/resultant.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/Polynomial/resultant.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/Polynomial/resultant.h $
 // $Id: resultant.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomial/Scalar_factor_traits.h 5.1-1/include/CGAL/Polynomial/Scalar_factor_traits.h
--- 5.0.3-1/include/CGAL/Polynomial/Scalar_factor_traits.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomial/Scalar_factor_traits.h	2020-09-08 12:15:29.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/Polynomial/Scalar_factor_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/Polynomial/Scalar_factor_traits.h $
 // $Id: Scalar_factor_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomial/square_free_factorize.h 5.1-1/include/CGAL/Polynomial/square_free_factorize.h
--- 5.0.3-1/include/CGAL/Polynomial/square_free_factorize.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomial/square_free_factorize.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/Polynomial/square_free_factorize.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/Polynomial/square_free_factorize.h $
 // $Id: square_free_factorize.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomial/sturm_habicht_sequence.h 5.1-1/include/CGAL/Polynomial/sturm_habicht_sequence.h
--- 5.0.3-1/include/CGAL/Polynomial/sturm_habicht_sequence.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomial/sturm_habicht_sequence.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/Polynomial/sturm_habicht_sequence.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/Polynomial/sturm_habicht_sequence.h $
 // $Id: sturm_habicht_sequence.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomial/subresultants.h 5.1-1/include/CGAL/Polynomial/subresultants.h
--- 5.0.3-1/include/CGAL/Polynomial/subresultants.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomial/subresultants.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/Polynomial/subresultants.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/Polynomial/subresultants.h $
 // $Id: subresultants.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomial.h 5.1-1/include/CGAL/Polynomial.h
--- 5.0.3-1/include/CGAL/Polynomial.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomial.h	2020-09-08 12:15:29.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/Polynomial.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/Polynomial.h $
 // $Id: Polynomial.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomials_1_2.h 5.1-1/include/CGAL/Polynomials_1_2.h
--- 5.0.3-1/include/CGAL/Polynomials_1_2.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomials_1_2.h	2020-09-08 12:14:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_for_circles/include/CGAL/Polynomials_1_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_for_circles/include/CGAL/Polynomials_1_2.h $
 // $Id: Polynomials_1_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomials_1_3.h 5.1-1/include/CGAL/Polynomials_1_3.h
--- 5.0.3-1/include/CGAL/Polynomials_1_3.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomials_1_3.h	2020-09-08 12:14:32.000000000 +0000
@@ -9,7 +9,7 @@
 // and a STREP (FET Open) Project under Contract No  IST-006413
 // (ACS -- Algorithms for Complex Shapes)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_for_spheres/include/CGAL/Polynomials_1_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_for_spheres/include/CGAL/Polynomials_1_3.h $
 // $Id: Polynomials_1_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomials_2_2.h 5.1-1/include/CGAL/Polynomials_2_2.h
--- 5.0.3-1/include/CGAL/Polynomials_2_2.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomials_2_2.h	2020-09-08 12:14:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_for_circles/include/CGAL/Polynomials_2_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_for_circles/include/CGAL/Polynomials_2_2.h $
 // $Id: Polynomials_2_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomials_2_3.h 5.1-1/include/CGAL/Polynomials_2_3.h
--- 5.0.3-1/include/CGAL/Polynomials_2_3.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomials_2_3.h	2020-09-08 12:14:32.000000000 +0000
@@ -9,7 +9,7 @@
 // and a STREP (FET Open) Project under Contract No  IST-006413
 // (ACS -- Algorithms for Complex Shapes)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_for_spheres/include/CGAL/Polynomials_2_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_for_spheres/include/CGAL/Polynomials_2_3.h $
 // $Id: Polynomials_2_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomials_for_line_3.h 5.1-1/include/CGAL/Polynomials_for_line_3.h
--- 5.0.3-1/include/CGAL/Polynomials_for_line_3.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomials_for_line_3.h	2020-09-08 12:14:32.000000000 +0000
@@ -9,7 +9,7 @@
 // and a STREP (FET Open) Project under Contract No  IST-006413
 // (ACS -- Algorithms for Complex Shapes)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_for_spheres/include/CGAL/Polynomials_for_line_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_for_spheres/include/CGAL/Polynomials_for_line_3.h $
 // $Id: Polynomials_for_line_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomial_traits_d.h 5.1-1/include/CGAL/Polynomial_traits_d.h
--- 5.0.3-1/include/CGAL/Polynomial_traits_d.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomial_traits_d.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/Polynomial_traits_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/Polynomial_traits_d.h $
 // $Id: Polynomial_traits_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polynomial_type_generator.h 5.1-1/include/CGAL/Polynomial_type_generator.h
--- 5.0.3-1/include/CGAL/Polynomial_type_generator.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polynomial_type_generator.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/Polynomial_type_generator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/Polynomial_type_generator.h $
 // $Id: Polynomial_type_generator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/polynomial_utils.h 5.1-1/include/CGAL/polynomial_utils.h
--- 5.0.3-1/include/CGAL/polynomial_utils.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/polynomial_utils.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polynomial/include/CGAL/polynomial_utils.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polynomial/include/CGAL/polynomial_utils.h $
 // $Id: polynomial_utils.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polytope_distance_d.h 5.1-1/include/CGAL/Polytope_distance_d.h
--- 5.0.3-1/include/CGAL/Polytope_distance_d.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polytope_distance_d.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polytope_distance_d/include/CGAL/Polytope_distance_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polytope_distance_d/include/CGAL/Polytope_distance_d.h $
 // $Id: Polytope_distance_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polytope_distance_d_traits_2.h 5.1-1/include/CGAL/Polytope_distance_d_traits_2.h
--- 5.0.3-1/include/CGAL/Polytope_distance_d_traits_2.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polytope_distance_d_traits_2.h	2020-09-08 12:15:30.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polytope_distance_d/include/CGAL/Polytope_distance_d_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polytope_distance_d/include/CGAL/Polytope_distance_d_traits_2.h $
 // $Id: Polytope_distance_d_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polytope_distance_d_traits_3.h 5.1-1/include/CGAL/Polytope_distance_d_traits_3.h
--- 5.0.3-1/include/CGAL/Polytope_distance_d_traits_3.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polytope_distance_d_traits_3.h	2020-09-08 12:15:30.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polytope_distance_d/include/CGAL/Polytope_distance_d_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polytope_distance_d/include/CGAL/Polytope_distance_d_traits_3.h $
 // $Id: Polytope_distance_d_traits_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Polytope_distance_d_traits_d.h 5.1-1/include/CGAL/Polytope_distance_d_traits_d.h
--- 5.0.3-1/include/CGAL/Polytope_distance_d_traits_d.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Polytope_distance_d_traits_d.h	2020-09-08 12:15:30.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polytope_distance_d/include/CGAL/Polytope_distance_d_traits_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polytope_distance_d/include/CGAL/Polytope_distance_d_traits_d.h $
 // $Id: Polytope_distance_d_traits_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/predicates/kernel_ftC2.h 5.1-1/include/CGAL/predicates/kernel_ftC2.h
--- 5.0.3-1/include/CGAL/predicates/kernel_ftC2.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/predicates/kernel_ftC2.h	2020-09-08 12:14:45.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/predicates/kernel_ftC2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/predicates/kernel_ftC2.h $
 // $Id: kernel_ftC2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/predicates/kernel_ftC3.h 5.1-1/include/CGAL/predicates/kernel_ftC3.h
--- 5.0.3-1/include/CGAL/predicates/kernel_ftC3.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/predicates/kernel_ftC3.h	2020-09-08 12:14:45.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/predicates/kernel_ftC3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/predicates/kernel_ftC3.h $
 // $Id: kernel_ftC3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/predicates/Polygon_offset_pred_ftC2.h 5.1-1/include/CGAL/predicates/Polygon_offset_pred_ftC2.h
--- 5.0.3-1/include/CGAL/predicates/Polygon_offset_pred_ftC2.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/predicates/Polygon_offset_pred_ftC2.h	2020-09-08 12:15:41.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Straight_skeleton_2/include/CGAL/predicates/Polygon_offset_pred_ftC2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Straight_skeleton_2/include/CGAL/predicates/Polygon_offset_pred_ftC2.h $
 // $Id: Polygon_offset_pred_ftC2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/predicates/predicates_for_mixed_complex_3.h 5.1-1/include/CGAL/predicates/predicates_for_mixed_complex_3.h
--- 5.0.3-1/include/CGAL/predicates/predicates_for_mixed_complex_3.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/predicates/predicates_for_mixed_complex_3.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Skin_surface_3/include/CGAL/predicates/predicates_for_mixed_complex_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Skin_surface_3/include/CGAL/predicates/predicates_for_mixed_complex_3.h $
 
 // $Id: predicates_for_mixed_complex_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
diff -pruN 5.0.3-1/include/CGAL/predicates/predicates_for_voronoi_intersection_cartesian_2_3.h 5.1-1/include/CGAL/predicates/predicates_for_voronoi_intersection_cartesian_2_3.h
--- 5.0.3-1/include/CGAL/predicates/predicates_for_voronoi_intersection_cartesian_2_3.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/predicates/predicates_for_voronoi_intersection_cartesian_2_3.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Interpolation/include/CGAL/predicates/predicates_for_voronoi_intersection_cartesian_2_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Interpolation/include/CGAL/predicates/predicates_for_voronoi_intersection_cartesian_2_3.h $
 // $Id: predicates_for_voronoi_intersection_cartesian_2_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/predicates/sign_of_determinant.h 5.1-1/include/CGAL/predicates/sign_of_determinant.h
--- 5.0.3-1/include/CGAL/predicates/sign_of_determinant.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/predicates/sign_of_determinant.h	2020-09-08 12:15:02.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/predicates/sign_of_determinant.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/predicates/sign_of_determinant.h $
 // $Id: sign_of_determinant.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/predicates/Straight_skeleton_pred_ftC2.h 5.1-1/include/CGAL/predicates/Straight_skeleton_pred_ftC2.h
--- 5.0.3-1/include/CGAL/predicates/Straight_skeleton_pred_ftC2.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/predicates/Straight_skeleton_pred_ftC2.h	2020-09-08 12:15:41.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Straight_skeleton_2/include/CGAL/predicates/Straight_skeleton_pred_ftC2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Straight_skeleton_2/include/CGAL/predicates/Straight_skeleton_pred_ftC2.h $
 // $Id: Straight_skeleton_pred_ftC2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/predicates_d.h 5.1-1/include/CGAL/predicates_d.h
--- 5.0.3-1/include/CGAL/predicates_d.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/predicates_d.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/predicates_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/predicates_d.h $
 // $Id: predicates_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/predicates_on_lines_2.h 5.1-1/include/CGAL/predicates_on_lines_2.h
--- 5.0.3-1/include/CGAL/predicates_on_lines_2.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/predicates_on_lines_2.h	2020-09-08 12:15:02.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/predicates_on_lines_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/predicates_on_lines_2.h $
 // $Id: predicates_on_lines_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/predicates_on_points_2.h 5.1-1/include/CGAL/predicates_on_points_2.h
--- 5.0.3-1/include/CGAL/predicates_on_points_2.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/predicates_on_points_2.h	2020-09-08 12:15:02.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/predicates_on_points_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/predicates_on_points_2.h $
 // $Id: predicates_on_points_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/predicates_on_points_3.h 5.1-1/include/CGAL/predicates_on_points_3.h
--- 5.0.3-1/include/CGAL/predicates_on_points_3.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/predicates_on_points_3.h	2020-09-08 12:15:02.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/predicates_on_points_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/predicates_on_points_3.h $
 // $Id: predicates_on_points_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/primes.h 5.1-1/include/CGAL/primes.h
--- 5.0.3-1/include/CGAL/primes.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/primes.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Modular_arithmetic/include/CGAL/primes.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Modular_arithmetic/include/CGAL/primes.h $
 // $Id: primes.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Profile_counter.h 5.1-1/include/CGAL/Profile_counter.h
--- 5.0.3-1/include/CGAL/Profile_counter.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Profile_counter.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Profiling_tools/include/CGAL/Profile_counter.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Profiling_tools/include/CGAL/Profile_counter.h $
 // $Id: Profile_counter.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Profile_timer.h 5.1-1/include/CGAL/Profile_timer.h
--- 5.0.3-1/include/CGAL/Profile_timer.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Profile_timer.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Profiling_tools/include/CGAL/Profile_timer.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Profiling_tools/include/CGAL/Profile_timer.h $
 // $Id: Profile_timer.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Projection_traits_xy_3.h 5.1-1/include/CGAL/Projection_traits_xy_3.h
--- 5.0.3-1/include/CGAL/Projection_traits_xy_3.h	2020-08-01 19:02:26.000000000 +0000
+++ 5.1-1/include/CGAL/Projection_traits_xy_3.h	2020-09-08 12:15:01.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Projection_traits_xy_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Projection_traits_xy_3.h $
 // $Id: Projection_traits_xy_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Projection_traits_xz_3.h 5.1-1/include/CGAL/Projection_traits_xz_3.h
--- 5.0.3-1/include/CGAL/Projection_traits_xz_3.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/Projection_traits_xz_3.h	2020-09-08 12:15:01.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Projection_traits_xz_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Projection_traits_xz_3.h $
 // $Id: Projection_traits_xz_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Projection_traits_yz_3.h 5.1-1/include/CGAL/Projection_traits_yz_3.h
--- 5.0.3-1/include/CGAL/Projection_traits_yz_3.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/Projection_traits_yz_3.h	2020-09-08 12:15:01.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Projection_traits_yz_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Projection_traits_yz_3.h $
 // $Id: Projection_traits_yz_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/property_map.h 5.1-1/include/CGAL/property_map.h
--- 5.0.3-1/include/CGAL/property_map.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/property_map.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Property_map/include/CGAL/property_map.h $
-// $Id: property_map.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Property_map/include/CGAL/property_map.h $
+// $Id: property_map.h c1d8fb6 2020-04-22T13:47:33+02:00 Simon Giraudot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Andreas Fabri and Laurent Saboret
@@ -27,6 +27,8 @@
 
 #include <utility> // defines std::pair
 
+#include <CGAL/boost/iterator/transform_iterator.hpp>
+#include <CGAL/Iterator_range.h>
 #include <CGAL/Cartesian_converter_fwd.h>
 #include <CGAL/Kernel_traits_fwd.h>
 #include <CGAL/assertions.h>
@@ -216,6 +218,22 @@ struct Identity_property_map
   /// @}
 };
 
+
+/// \cond SKIP_IN_MANUAL
+template <typename T>
+struct Identity_property_map_no_lvalue
+{
+  typedef T key_type; ///< typedef to `T`
+  typedef T value_type; ///< typedef to `T`
+  typedef T reference; ///< typedef to `T`
+  typedef boost::readable_property_map_tag category; ///< `boost::readable_property_map_tag`
+
+  typedef Identity_property_map_no_lvalue<T> Self;
+
+  friend reference get(const Self&, const key_type& k) {return k;}
+};
+/// \endcond
+
 /// Free function to create a `Identity_property_map` property map.
 ///
 /// \relates Identity_property_map
@@ -560,6 +578,38 @@ make_cartesian_converter_property_map(Vp
   return Cartesian_converter_property_map<GeomObject, Vpm>(vpm);
 }
 
+/// \cond SKIP_IN_MANUAL
+// Syntaxic sugar for transform_iterator+pmap_to_unary_function
+template <typename Iterator, typename Pmap>
+typename boost::transform_iterator<CGAL::Property_map_to_unary_function<Pmap>, Iterator>
+make_transform_iterator_from_property_map (Iterator it, Pmap pmap)
+{
+  return boost::make_transform_iterator (it, CGAL::Property_map_to_unary_function<Pmap>(pmap));
+}
+
+// Syntaxic sugar for make_range+transform_iterator+pmap_to_unary_function
+template <typename Range, typename Pmap>
+CGAL::Iterator_range<typename boost::transform_iterator<CGAL::Property_map_to_unary_function<Pmap>,
+                                                        typename Range::const_iterator> >
+make_transform_range_from_property_map (const Range& range, Pmap pmap)
+{
+  return CGAL::make_range
+    (make_transform_iterator_from_property_map (range.begin(), pmap),
+     make_transform_iterator_from_property_map (range.end(), pmap));
+}
+
+// Syntaxic sugar for make_range+transform_iterator+pmap_to_unary_function
+template <typename Range, typename Pmap>
+CGAL::Iterator_range<typename boost::transform_iterator<CGAL::Property_map_to_unary_function<Pmap>,
+                                                        typename Range::iterator> >
+make_transform_range_from_property_map (Range& range, Pmap pmap)
+{
+  return CGAL::make_range
+    (make_transform_iterator_from_property_map (range.begin(), pmap),
+     make_transform_iterator_from_property_map (range.end(), pmap));
+}
+/// \endcond
+
 } // namespace CGAL
 
 
diff -pruN 5.0.3-1/include/CGAL/QP_functions.h 5.1-1/include/CGAL/QP_functions.h
--- 5.0.3-1/include/CGAL/QP_functions.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/QP_functions.h	2020-09-08 12:15:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/QP_solver/include/CGAL/QP_functions.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/QP_solver/include/CGAL/QP_functions.h $
 // $Id: QP_functions.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/QP_models.h 5.1-1/include/CGAL/QP_models.h
--- 5.0.3-1/include/CGAL/QP_models.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/QP_models.h	2020-09-08 12:15:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/QP_solver/include/CGAL/QP_models.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/QP_solver/include/CGAL/QP_models.h $
 // $Id: QP_models.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/QP_options.h 5.1-1/include/CGAL/QP_options.h
--- 5.0.3-1/include/CGAL/QP_options.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/QP_options.h	2020-09-08 12:15:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/QP_solver/include/CGAL/QP_options.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/QP_solver/include/CGAL/QP_options.h $
 // $Id: QP_options.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/QP_solution.h 5.1-1/include/CGAL/QP_solution.h
--- 5.0.3-1/include/CGAL/QP_solution.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/QP_solution.h	2020-09-08 12:15:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/QP_solver/include/CGAL/QP_solution.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/QP_solver/include/CGAL/QP_solution.h $
 // $Id: QP_solution.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/QP_solver/assertions.h 5.1-1/include/CGAL/QP_solver/assertions.h
--- 5.0.3-1/include/CGAL/QP_solver/assertions.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/QP_solver/assertions.h	2020-09-08 12:15:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/QP_solver/include/CGAL/QP_solver/assertions.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/QP_solver/include/CGAL/QP_solver/assertions.h $
 // $Id: assertions.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/QP_solver/basic.h 5.1-1/include/CGAL/QP_solver/basic.h
--- 5.0.3-1/include/CGAL/QP_solver/basic.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/QP_solver/basic.h	2020-09-08 12:15:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/QP_solver/include/CGAL/QP_solver/basic.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/QP_solver/include/CGAL/QP_solver/basic.h $
 // $Id: basic.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/QP_solver/debug.h 5.1-1/include/CGAL/QP_solver/debug.h
--- 5.0.3-1/include/CGAL/QP_solver/debug.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/QP_solver/debug.h	2020-09-08 12:15:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/QP_solver/include/CGAL/QP_solver/debug.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/QP_solver/include/CGAL/QP_solver/debug.h $
 // $Id: debug.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/QP_solver/functors.h 5.1-1/include/CGAL/QP_solver/functors.h
--- 5.0.3-1/include/CGAL/QP_solver/functors.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/QP_solver/functors.h	2020-09-08 12:15:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/QP_solver/include/CGAL/QP_solver/functors.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/QP_solver/include/CGAL/QP_solver/functors.h $
 // $Id: functors.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/QP_solver/Initialization.h 5.1-1/include/CGAL/QP_solver/Initialization.h
--- 5.0.3-1/include/CGAL/QP_solver/Initialization.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/QP_solver/Initialization.h	2020-09-08 12:15:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/QP_solver/include/CGAL/QP_solver/Initialization.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/QP_solver/include/CGAL/QP_solver/Initialization.h $
 // $Id: Initialization.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/QP_solver/QP_basis_inverse.h 5.1-1/include/CGAL/QP_solver/QP_basis_inverse.h
--- 5.0.3-1/include/CGAL/QP_solver/QP_basis_inverse.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/QP_solver/QP_basis_inverse.h	2020-09-08 12:15:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/QP_solver/include/CGAL/QP_solver/QP_basis_inverse.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/QP_solver/include/CGAL/QP_solver/QP_basis_inverse.h $
 // $Id: QP_basis_inverse.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/QP_solver/QP_basis_inverse_impl.h 5.1-1/include/CGAL/QP_solver/QP_basis_inverse_impl.h
--- 5.0.3-1/include/CGAL/QP_solver/QP_basis_inverse_impl.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/QP_solver/QP_basis_inverse_impl.h	2020-09-08 12:15:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/QP_solver/include/CGAL/QP_solver/QP_basis_inverse_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/QP_solver/include/CGAL/QP_solver/QP_basis_inverse_impl.h $
 // $Id: QP_basis_inverse_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/QP_solver/QP_exact_bland_pricing.h 5.1-1/include/CGAL/QP_solver/QP_exact_bland_pricing.h
--- 5.0.3-1/include/CGAL/QP_solver/QP_exact_bland_pricing.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/QP_solver/QP_exact_bland_pricing.h	2020-09-08 12:15:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/QP_solver/include/CGAL/QP_solver/QP_exact_bland_pricing.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/QP_solver/include/CGAL/QP_solver/QP_exact_bland_pricing.h $
 // $Id: QP_exact_bland_pricing.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/QP_solver/QP__filtered_base.h 5.1-1/include/CGAL/QP_solver/QP__filtered_base.h
--- 5.0.3-1/include/CGAL/QP_solver/QP__filtered_base.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/QP_solver/QP__filtered_base.h	2020-09-08 12:15:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/QP_solver/include/CGAL/QP_solver/QP__filtered_base.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/QP_solver/include/CGAL/QP_solver/QP__filtered_base.h $
 // $Id: QP__filtered_base.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/QP_solver/QP__filtered_base_impl.h 5.1-1/include/CGAL/QP_solver/QP__filtered_base_impl.h
--- 5.0.3-1/include/CGAL/QP_solver/QP__filtered_base_impl.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/QP_solver/QP__filtered_base_impl.h	2020-09-08 12:15:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/QP_solver/include/CGAL/QP_solver/QP__filtered_base_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/QP_solver/include/CGAL/QP_solver/QP__filtered_base_impl.h $
 // $Id: QP__filtered_base_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/QP_solver/QP_full_exact_pricing.h 5.1-1/include/CGAL/QP_solver/QP_full_exact_pricing.h
--- 5.0.3-1/include/CGAL/QP_solver/QP_full_exact_pricing.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/QP_solver/QP_full_exact_pricing.h	2020-09-08 12:15:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/QP_solver/include/CGAL/QP_solver/QP_full_exact_pricing.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/QP_solver/include/CGAL/QP_solver/QP_full_exact_pricing.h $
 // $Id: QP_full_exact_pricing.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/QP_solver/QP_full_filtered_pricing.h 5.1-1/include/CGAL/QP_solver/QP_full_filtered_pricing.h
--- 5.0.3-1/include/CGAL/QP_solver/QP_full_filtered_pricing.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/QP_solver/QP_full_filtered_pricing.h	2020-09-08 12:15:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/QP_solver/include/CGAL/QP_solver/QP_full_filtered_pricing.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/QP_solver/include/CGAL/QP_solver/QP_full_filtered_pricing.h $
 // $Id: QP_full_filtered_pricing.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/QP_solver/QP_functions_impl.h 5.1-1/include/CGAL/QP_solver/QP_functions_impl.h
--- 5.0.3-1/include/CGAL/QP_solver/QP_functions_impl.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/QP_solver/QP_functions_impl.h	2020-09-08 12:15:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/QP_solver/include/CGAL/QP_solver/QP_functions_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/QP_solver/include/CGAL/QP_solver/QP_functions_impl.h $
 // $Id: QP_functions_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/QP_solver/QP__partial_base.h 5.1-1/include/CGAL/QP_solver/QP__partial_base.h
--- 5.0.3-1/include/CGAL/QP_solver/QP__partial_base.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/QP_solver/QP__partial_base.h	2020-09-08 12:15:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/QP_solver/include/CGAL/QP_solver/QP__partial_base.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/QP_solver/include/CGAL/QP_solver/QP__partial_base.h $
 // $Id: QP__partial_base.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/QP_solver/QP_partial_exact_pricing.h 5.1-1/include/CGAL/QP_solver/QP_partial_exact_pricing.h
--- 5.0.3-1/include/CGAL/QP_solver/QP_partial_exact_pricing.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/QP_solver/QP_partial_exact_pricing.h	2020-09-08 12:15:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/QP_solver/include/CGAL/QP_solver/QP_partial_exact_pricing.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/QP_solver/include/CGAL/QP_solver/QP_partial_exact_pricing.h $
 // $Id: QP_partial_exact_pricing.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/QP_solver/QP_partial_filtered_pricing.h 5.1-1/include/CGAL/QP_solver/QP_partial_filtered_pricing.h
--- 5.0.3-1/include/CGAL/QP_solver/QP_partial_filtered_pricing.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/QP_solver/QP_partial_filtered_pricing.h	2020-09-08 12:15:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/QP_solver/include/CGAL/QP_solver/QP_partial_filtered_pricing.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/QP_solver/include/CGAL/QP_solver/QP_partial_filtered_pricing.h $
 // $Id: QP_partial_filtered_pricing.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/QP_solver/QP_pricing_strategy.h 5.1-1/include/CGAL/QP_solver/QP_pricing_strategy.h
--- 5.0.3-1/include/CGAL/QP_solver/QP_pricing_strategy.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/QP_solver/QP_pricing_strategy.h	2020-09-08 12:15:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/QP_solver/include/CGAL/QP_solver/QP_pricing_strategy.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/QP_solver/include/CGAL/QP_solver/QP_pricing_strategy.h $
 // $Id: QP_pricing_strategy.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/QP_solver/QP_solution_impl.h 5.1-1/include/CGAL/QP_solver/QP_solution_impl.h
--- 5.0.3-1/include/CGAL/QP_solver/QP_solution_impl.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/QP_solver/QP_solution_impl.h	2020-09-08 12:15:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/QP_solver/include/CGAL/QP_solver/QP_solution_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/QP_solver/include/CGAL/QP_solver/QP_solution_impl.h $
 // $Id: QP_solution_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/QP_solver/QP_solver_bounds_impl.h 5.1-1/include/CGAL/QP_solver/QP_solver_bounds_impl.h
--- 5.0.3-1/include/CGAL/QP_solver/QP_solver_bounds_impl.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/QP_solver/QP_solver_bounds_impl.h	2020-09-08 12:15:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/QP_solver/include/CGAL/QP_solver/QP_solver_bounds_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/QP_solver/include/CGAL/QP_solver/QP_solver_bounds_impl.h $
 // $Id: QP_solver_bounds_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/QP_solver/QP_solver.h 5.1-1/include/CGAL/QP_solver/QP_solver.h
--- 5.0.3-1/include/CGAL/QP_solver/QP_solver.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/QP_solver/QP_solver.h	2020-09-08 12:15:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/QP_solver/include/CGAL/QP_solver/QP_solver.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/QP_solver/include/CGAL/QP_solver/QP_solver.h $
 // $Id: QP_solver.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/QP_solver/QP_solver_impl.h 5.1-1/include/CGAL/QP_solver/QP_solver_impl.h
--- 5.0.3-1/include/CGAL/QP_solver/QP_solver_impl.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/QP_solver/QP_solver_impl.h	2020-09-08 12:15:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/QP_solver/include/CGAL/QP_solver/QP_solver_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/QP_solver/include/CGAL/QP_solver/QP_solver_impl.h $
 // $Id: QP_solver_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/QP_solver/QP_solver_nonstandardform_impl.h 5.1-1/include/CGAL/QP_solver/QP_solver_nonstandardform_impl.h
--- 5.0.3-1/include/CGAL/QP_solver/QP_solver_nonstandardform_impl.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/QP_solver/QP_solver_nonstandardform_impl.h	2020-09-08 12:15:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/QP_solver/include/CGAL/QP_solver/QP_solver_nonstandardform_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/QP_solver/include/CGAL/QP_solver/QP_solver_nonstandardform_impl.h $
 // $Id: QP_solver_nonstandardform_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/QP_solver/Unbounded_direction.h 5.1-1/include/CGAL/QP_solver/Unbounded_direction.h
--- 5.0.3-1/include/CGAL/QP_solver/Unbounded_direction.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/QP_solver/Unbounded_direction.h	2020-09-08 12:15:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/QP_solver/include/CGAL/QP_solver/Unbounded_direction.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/QP_solver/include/CGAL/QP_solver/Unbounded_direction.h $
 // $Id: Unbounded_direction.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/AlphaShapeGraphicsItem.h 5.1-1/include/CGAL/Qt/AlphaShapeGraphicsItem.h
--- 5.0.3-1/include/CGAL/Qt/AlphaShapeGraphicsItem.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/AlphaShapeGraphicsItem.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/AlphaShapeGraphicsItem.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/AlphaShapeGraphicsItem.h $
 // $Id: AlphaShapeGraphicsItem.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/ApolloniusGraphGraphicsItem.h 5.1-1/include/CGAL/Qt/ApolloniusGraphGraphicsItem.h
--- 5.0.3-1/include/CGAL/Qt/ApolloniusGraphGraphicsItem.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/ApolloniusGraphGraphicsItem.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/ApolloniusGraphGraphicsItem.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/ApolloniusGraphGraphicsItem.h $
 // $Id: ApolloniusGraphGraphicsItem.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/Basic_viewer_qt.h 5.1-1/include/CGAL/Qt/Basic_viewer_qt.h
--- 5.0.3-1/include/CGAL/Qt/Basic_viewer_qt.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/Basic_viewer_qt.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/Basic_viewer_qt.h $
-// $Id: Basic_viewer_qt.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/Basic_viewer_qt.h $
+// $Id: Basic_viewer_qt.h 7ec9d57 2020-04-09T17:42:59+02:00 Guillaume Damiand
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -15,6 +15,8 @@
 
 #include <CGAL/license/GraphicsView.h>
 #include <iostream>
+#include <tuple>
+#include <string>
 
 #ifdef CGAL_USE_BASIC_VIEWER
 
@@ -43,6 +45,7 @@
 
 #include <vector>
 #include <cstdlib>
+#include <cfloat>
 
 #include <CGAL/Buffer_for_vao.h>
 #include <CGAL/Qt/CreateOpenGLContext.h>
@@ -91,7 +94,6 @@ const char fragment_source_color[] =
     "uniform float spec_power ; \n"
 
     "void main(void) { \n"
-
     "   highp vec3 L = light_pos.xyz - fP.xyz; \n"
     "   highp vec3 V = -fP.xyz; \n"
 
@@ -102,8 +104,7 @@ const char fragment_source_color[] =
     "   highp vec3 R = reflect(-L, N); \n"
     "   highp vec4 diffuse = max(dot(N,L), 0.0) * light_diff * fColor; \n"
     "   highp vec4 specular = pow(max(dot(R,V), 0.0), spec_power) * light_spec; \n"
-
-    "gl_FragColor = light_amb*fColor + diffuse  ; \n"
+    "   gl_FragColor = light_amb*fColor + diffuse  ; \n"
     "} \n"
     "\n"
   };
@@ -248,18 +249,28 @@ public:
                   bool draw_edges=true,
                   bool draw_faces=true,
                   bool use_mono_color=false,
-                  bool inverse_normal=false) :
+                  bool inverse_normal=false,
+                  bool draw_rays=true,
+                  bool draw_lines=true,
+                  bool draw_text=true) :
     CGAL::QGLViewer(parent),
     m_draw_vertices(draw_vertices),
     m_draw_edges(draw_edges),
+    m_draw_rays(draw_rays),
+    m_draw_lines(draw_lines),
     m_draw_faces(draw_faces),
     m_flatShading(true),
     m_use_mono_color(use_mono_color),
     m_inverse_normal(inverse_normal),
+    m_draw_text(draw_text),
     m_size_points(7.),
     m_size_edges(3.1),
+    m_size_rays(3.1),
+    m_size_lines(3.1),
     m_vertices_mono_color(200, 60, 60),
     m_edges_mono_color(0, 0, 0),
+    m_rays_mono_color(0, 0, 0),
+    m_lines_mono_color(0, 0, 0),
     m_faces_mono_color(60, 60, 200),
     m_ambient_color(0.6f, 0.5f, 0.5f, 0.5f),
     m_are_buffers_initialized(false),
@@ -281,6 +292,24 @@ public:
                                   &m_bounding_box,
                                   &arrays[COLOR_SEGMENTS],
                                   nullptr, nullptr),
+    m_buffer_for_mono_rays(&arrays[POS_MONO_RAYS],
+                            nullptr,
+                            &m_bounding_box,
+                            nullptr, nullptr),
+    m_buffer_for_colored_rays(&arrays[POS_COLORED_RAYS],
+                               nullptr,
+                               &m_bounding_box,
+                               &arrays[COLOR_RAYS],
+                               nullptr, nullptr),
+    m_buffer_for_mono_lines(&arrays[POS_MONO_RAYS],
+                           nullptr,
+                           &m_bounding_box,
+                           nullptr, nullptr),
+    m_buffer_for_colored_lines(&arrays[POS_COLORED_LINES],
+                              nullptr,
+                              &m_bounding_box,
+                              &arrays[COLOR_LINES],
+                              nullptr, nullptr),
     m_buffer_for_mono_faces(&arrays[POS_MONO_FACES],
                             nullptr,
                             &m_bounding_box,
@@ -300,6 +329,9 @@ public:
       setWindowTitle(title);
 
     resize(500, 450);
+
+    if (inverse_normal)
+    { negate_all_normals(); }
   }
 
   ~Basic_viewer_qt()
@@ -307,7 +339,7 @@ public:
     for (unsigned int i=0; i<NB_VBO_BUFFERS; ++i)
       buffers[i].destroy();
 
-    for (int i=0; i<NB_VAO_BUFFERS; ++i)
+    for (unsigned int i=0; i<NB_VAO_BUFFERS; ++i)
       vao[i].destroy();
   }
 
@@ -317,6 +349,7 @@ public:
     { arrays[i].clear(); }
 
     m_bounding_box=CGAL::Bbox_3();
+    m_texts.clear();
   }
 
   bool is_empty() const
@@ -325,6 +358,10 @@ public:
             m_buffer_for_colored_points.is_empty() &&
             m_buffer_for_mono_segments.is_empty() &&
             m_buffer_for_colored_segments.is_empty() &&
+            m_buffer_for_mono_rays.is_empty() &&
+            m_buffer_for_colored_rays.is_empty() &&
+            m_buffer_for_mono_lines.is_empty() &&
+            m_buffer_for_colored_lines.is_empty() &&
             m_buffer_for_mono_faces.is_empty() &&
             m_buffer_for_colored_faces.is_empty());
   }
@@ -340,7 +377,11 @@ public:
       m_buffer_for_mono_segments.has_zero_x() &&
       m_buffer_for_colored_segments.has_zero_x() &&
       m_buffer_for_mono_faces.has_zero_x() &&
-      m_buffer_for_colored_faces.has_zero_x();
+      m_buffer_for_colored_faces.has_zero_x() &&
+      m_buffer_for_mono_rays.has_zero_x() &&
+      m_buffer_for_colored_rays.has_zero_x() &&
+      m_buffer_for_mono_lines.has_zero_x() &&
+      m_buffer_for_colored_lines.has_zero_x();
   }
 
   bool has_zero_y() const
@@ -351,7 +392,11 @@ public:
       m_buffer_for_mono_segments.has_zero_y() &&
       m_buffer_for_colored_segments.has_zero_y() &&
       m_buffer_for_mono_faces.has_zero_y() &&
-      m_buffer_for_colored_faces.has_zero_y();
+      m_buffer_for_colored_faces.has_zero_y() &&
+      m_buffer_for_mono_rays.has_zero_y() &&
+      m_buffer_for_colored_rays.has_zero_y() &&
+      m_buffer_for_mono_lines.has_zero_y() &&
+      m_buffer_for_colored_lines.has_zero_y();
   }
 
   bool has_zero_z() const
@@ -362,7 +407,11 @@ public:
       m_buffer_for_mono_segments.has_zero_z() &&
       m_buffer_for_colored_segments.has_zero_z() &&
       m_buffer_for_mono_faces.has_zero_z() &&
-      m_buffer_for_colored_faces.has_zero_z();
+      m_buffer_for_colored_faces.has_zero_z() &&
+      m_buffer_for_mono_rays.has_zero_z() &&
+      m_buffer_for_colored_rays.has_zero_z() &&
+      m_buffer_for_mono_lines.has_zero_z() &&
+      m_buffer_for_colored_lines.has_zero_z();
   }
 
   template<typename KPoint>
@@ -382,6 +431,72 @@ public:
                    const CGAL::Color& acolor)
   { m_buffer_for_colored_segments.add_segment(p1, p2, acolor); }
 
+  template <typename KPoint, typename KVector>
+  void update_bounding_box_for_ray(const KPoint &p, const KVector &v)
+  {
+    Local_point lp = get_local_point(p);
+    Local_vector lv = get_local_vector(v);
+    CGAL::Bbox_3 b = (lp + lv).bbox();
+    m_bounding_box += b;
+  }
+
+  template <typename KPoint, typename KVector>
+  void update_bounding_box_for_line(const KPoint &p, const KVector &v,
+                                    const KVector &pv)
+  {
+    Local_point lp = get_local_point(p);
+    Local_vector lv = get_local_vector(v);
+    Local_vector lpv = get_local_vector(pv);
+
+    CGAL::Bbox_3 b = lp.bbox() + (lp + lv).bbox() + (lp + lpv).bbox();
+    m_bounding_box += b;
+  }
+
+  template <typename KPoint, typename KVector>
+  void add_ray(const KPoint &p, const KVector &v)
+  {
+    double bigNumber = 1e30;
+    m_buffer_for_mono_rays.add_ray_segment(p, (p + (bigNumber)*v));
+  }
+
+  template <typename KPoint, typename KVector>
+  void add_ray(const KPoint &p, const KVector &v, const CGAL::Color &acolor)
+  {
+    double bigNumber = 1e30;
+    m_buffer_for_colored_rays.add_ray_segment(p, (p + (bigNumber)*v), acolor);
+  }
+
+  template <typename KPoint, typename KVector>
+  void add_line(const KPoint &p, const KVector &v)
+  {
+    double bigNumber = 1e30;
+    m_buffer_for_mono_lines.add_line_segment((p - (bigNumber)*v),
+                                             (p + (bigNumber)*v));
+  }
+
+  template <typename KPoint, typename KVector>
+  void add_line(const KPoint &p, const KVector &v, const CGAL::Color &acolor)
+  {
+    double bigNumber = 1e30;
+    m_buffer_for_colored_lines.add_line_segment((p - (bigNumber)*v),
+                                                (p + (bigNumber)*v), acolor);
+  }
+
+  template<typename KPoint>
+  void add_text(const KPoint& kp, const QString& txt)
+  {
+    Local_point p=get_local_point(kp);
+    m_texts.push_back(std::make_tuple(p, txt));
+  }
+
+  template<typename KPoint>
+  void add_text(const KPoint& kp, const char* txt)
+  { add_text(kp, QString(txt)); }
+
+  template<typename KPoint>
+  void add_text(const KPoint& kp, const std::string& txt)
+  { add_text(kp, txt.c_str()); }
+
   bool is_a_face_started() const
   {
     return m_buffer_for_mono_faces.is_a_face_started() ||
@@ -463,7 +578,7 @@ protected:
       { std::cerr<<"VBO Creation number "<<i<<" FAILED"<<std::endl; }
     }
 
-    for (int i=0; i<NB_VAO_BUFFERS; ++i)
+    for (unsigned int i=0; i<NB_VAO_BUFFERS; ++i)
     {
       if(!vao[i].isCreated() && !vao[i].create())
       { std::cerr<<"VAO Creation number "<<i<<" FAILED"<<std::endl; }
@@ -585,7 +700,7 @@ protected:
 
     vao[VAO_MONO_SEGMENTS].release();
 
-    // 1.2) Color segments
+    // 2.1) Color segments
     vao[VAO_COLORED_SEGMENTS].bind();
 
     ++bufn;
@@ -611,13 +726,104 @@ protected:
 
     rendering_program_p_l.release();
 
-    // 3) FACE SHADER
+    // 3) RAYS SHADER
+
+    // 3.1) Mono rays
+    vao[VAO_MONO_RAYS].bind();
+
+    ++bufn;
+    assert(bufn<NB_VBO_BUFFERS);
+    buffers[bufn].bind();
+    buffers[bufn].allocate(arrays[POS_MONO_RAYS].data(),
+                           static_cast<int>(arrays[POS_MONO_RAYS].size()*sizeof(float)));
+    rendering_program_p_l.enableAttributeArray("vertex");
+    rendering_program_p_l.setAttributeArray("vertex",GL_FLOAT,0,3);
+
+    buffers[bufn].release();
+
+    rendering_program_p_l.disableAttributeArray("color");
+
+    vao[VAO_MONO_RAYS].release();
+
+    // 3.2) Color rays
+
+    vao[VAO_COLORED_RAYS].bind();
+
+    ++bufn;
+    assert(bufn<NB_VBO_BUFFERS);
+    buffers[bufn].bind();
+    buffers[bufn].allocate(arrays[POS_COLORED_RAYS].data(),
+                           static_cast<int>(arrays[POS_COLORED_RAYS].size()*sizeof(float)));
+    rendering_program_p_l.enableAttributeArray("vertex");
+    rendering_program_p_l.setAttributeBuffer("vertex",GL_FLOAT,0,3);
+
+    buffers[bufn].release();
+
+    ++bufn;
+    assert(bufn<NB_VBO_BUFFERS);
+    buffers[bufn].bind();
+    buffers[bufn].allocate(arrays[COLOR_RAYS].data(),
+                           static_cast<int>(arrays[COLOR_RAYS].size()*sizeof(float)));
+    rendering_program_p_l.enableAttributeArray("color");
+    rendering_program_p_l.setAttributeBuffer("color",GL_FLOAT,0,3);
+    buffers[bufn].release();
+
+    vao[VAO_COLORED_RAYS].release();
+
+    rendering_program_p_l.release();
+
+    // 4) LINES SHADER
+    // 4.1) Mono lines
+    vao[VAO_MONO_LINES].bind();
+
+    ++bufn;
+    assert(bufn<NB_VBO_BUFFERS);
+    buffers[bufn].bind();
+    buffers[bufn].allocate(arrays[POS_MONO_LINES].data(),
+                           static_cast<int>(arrays[POS_MONO_LINES].size()*sizeof(float)));
+    rendering_program_p_l.enableAttributeArray("vertex");
+    rendering_program_p_l.setAttributeArray("vertex",GL_FLOAT,0,3);
+
+    buffers[bufn].release();
+
+    rendering_program_p_l.disableAttributeArray("color");
+
+    vao[VAO_MONO_LINES].release();
+
+    // 4.2 Color lines
+
+    vao[VAO_COLORED_LINES].bind();
+
+    ++bufn;
+    assert(bufn<NB_VBO_BUFFERS);
+    buffers[bufn].bind();
+    buffers[bufn].allocate(arrays[POS_COLORED_LINES].data(),
+                           static_cast<int>(arrays[POS_COLORED_LINES].size()*sizeof(float)));
+    rendering_program_p_l.enableAttributeArray("vertex");
+    rendering_program_p_l.setAttributeBuffer("vertex",GL_FLOAT,0,3);
+
+    buffers[bufn].release();
+
+    ++bufn;
+    assert(bufn<NB_VBO_BUFFERS);
+    buffers[bufn].bind();
+    buffers[bufn].allocate(arrays[COLOR_LINES].data(),
+                           static_cast<int>(arrays[COLOR_LINES].size()*sizeof(float)));
+    rendering_program_p_l.enableAttributeArray("color");
+    rendering_program_p_l.setAttributeBuffer("color",GL_FLOAT,0,3);
+    buffers[bufn].release();
+
+    vao[VAO_COLORED_LINES].release();
+
+    rendering_program_p_l.release();
+
+    // 5) FACE SHADER
     rendering_program_face.bind();
 
-    // 3.1) Mono faces
+    // 5.1) Mono faces
     vao[VAO_MONO_FACES].bind();
 
-    // 3.1.1) points of the mono faces
+    // 5.1.1) points of the mono faces
     ++bufn;
     assert(bufn<NB_VBO_BUFFERS);
     buffers[bufn].bind();
@@ -628,7 +834,7 @@ protected:
 
     buffers[bufn].release();
 
-    // 3.1.2) normals of the mono faces
+    // 5.1.2) normals of the mono faces
     ++bufn;
     assert(bufn<NB_VBO_BUFFERS);
     buffers[bufn].bind();
@@ -649,14 +855,14 @@ protected:
 
     buffers[bufn].release();
 
-    // 3.1.3) color of the mono faces
+    // 5.1.3) color of the mono faces
     rendering_program_face.disableAttributeArray("color");
     vao[VAO_MONO_FACES].release();
 
-    // 3.2) Color faces
+    // 5.2) Color faces
     vao[VAO_COLORED_FACES].bind();
 
-    // 3.2.1) points of the color faces
+    // 5.2.1) points of the color faces
     ++bufn;
     assert(bufn<NB_VBO_BUFFERS);
     buffers[bufn].bind();
@@ -667,7 +873,7 @@ protected:
 
     buffers[bufn].release();
 
-    // 3.2.2) normals of the color faces
+    // 5.2.2) normals of the color faces
     ++bufn;
     assert(bufn<NB_VBO_BUFFERS);
     buffers[bufn].bind();
@@ -688,7 +894,7 @@ protected:
 
     buffers[bufn].release();
 
-    // 3.2.3) colors of the faces
+    // 5.2.3) colors of the faces
     ++bufn;
     assert(bufn<NB_VBO_BUFFERS);
     buffers[bufn].bind();
@@ -712,12 +918,12 @@ protected:
     QMatrix4x4 mvMatrix;
     double mat[16];
     viewer->camera()->getModelViewProjectionMatrix(mat);
-    for(int i=0; i < 16; i++)
+    for(unsigned int i=0; i < 16; i++)
     {
       mvpMatrix.data()[i] = (float)mat[i];
     }
     viewer->camera()->getModelViewMatrix(mat);
-    for(int i=0; i < 16; i++)
+    for(unsigned int i=0; i < 16; i++)
     {
       mvMatrix.data()[i] = (float)mat[i];
     }
@@ -771,6 +977,11 @@ protected:
     rendering_program_p_l.release();
   }
 
+  // Returns true if the data structure lies on a plane
+  bool is_two_dimensional() {
+    return (!is_empty() && (has_zero_x() || has_zero_y() || has_zero_z()));
+  }
+
   virtual void draw()
   {
     glEnable(GL_DEPTH_TEST);
@@ -846,6 +1057,83 @@ protected:
       rendering_program_p_l.release();
     }
 
+    if(m_draw_rays)
+    {
+        rendering_program_p_l.bind();
+
+        vao[VAO_MONO_RAYS].bind();
+        color.setRgbF((double)m_rays_mono_color.red()/(double)255,
+                      (double)m_rays_mono_color.green()/(double)255,
+                      (double)m_rays_mono_color.blue()/(double)255);
+        rendering_program_p_l.setAttributeValue("color",color);
+        glLineWidth(m_size_rays);
+        glDrawArrays(GL_LINES, 0, static_cast<GLsizei>(arrays[POS_MONO_RAYS].size()/3));
+        vao[VAO_MONO_RAYS].release();
+
+        vao[VAO_COLORED_RAYS].bind();
+        if (m_use_mono_color)
+        {
+            color.setRgbF((double)m_rays_mono_color.red()/(double)255,
+                          (double)m_rays_mono_color.green()/(double)255,
+                          (double)m_rays_mono_color.blue()/(double)255);
+            rendering_program_p_l.disableAttributeArray("color");
+            rendering_program_p_l.setAttributeValue("color",color);
+        }
+        else
+        {
+            rendering_program_p_l.enableAttributeArray("color");
+        }
+        glLineWidth(m_size_rays);
+        glDrawArrays(GL_LINES, 0, static_cast<GLsizei>(arrays[POS_COLORED_RAYS].size()/3));
+        vao[VAO_COLORED_RAYS].release();
+
+        rendering_program_p_l.release();
+    }
+
+    if(m_draw_lines)
+    {
+        rendering_program_p_l.bind();
+
+        vao[VAO_MONO_LINES].bind();
+        color.setRgbF((double)m_lines_mono_color.red()/(double)255,
+                      (double)m_lines_mono_color.green()/(double)255,
+                      (double)m_lines_mono_color.blue()/(double)255);
+        rendering_program_p_l.setAttributeValue("color",color);
+        glLineWidth(m_size_lines);
+        glDrawArrays(GL_LINES, 0, static_cast<GLsizei>(arrays[POS_MONO_LINES].size()/3));
+        vao[VAO_MONO_LINES].release();
+
+        rendering_program_p_l.release();
+
+        vao[VAO_COLORED_LINES].bind();
+        if (m_use_mono_color)
+        {
+            color.setRgbF((double)m_rays_mono_color.red()/(double)255,
+                          (double)m_rays_mono_color.green()/(double)255,
+                          (double)m_rays_mono_color.blue()/(double)255);
+            rendering_program_p_l.disableAttributeArray("color");
+            rendering_program_p_l.setAttributeValue("color",color);
+        }
+        else
+        {
+            rendering_program_p_l.enableAttributeArray("color");
+        }
+        glLineWidth(m_size_lines);
+        glDrawArrays(GL_LINES, 0, static_cast<GLsizei>(arrays[POS_COLORED_LINES].size()/3));
+        vao[VAO_COLORED_LINES].release();
+
+        rendering_program_p_l.release();
+    }
+
+    // Fix Z-fighting by drawing faces at a depth
+    GLfloat offset_factor;
+    GLfloat offset_units;
+    if (is_two_dimensional()) {
+      glGetFloatv(GL_POLYGON_OFFSET_FACTOR, &offset_factor);
+      glGetFloatv(GL_POLYGON_OFFSET_UNITS, &offset_units);
+      glPolygonOffset(0.1f, 0.9f);
+    }
+
     if (m_draw_faces)
     {
       rendering_program_face.bind();
@@ -874,10 +1162,13 @@ protected:
       glDrawArrays(GL_TRIANGLES, 0, static_cast<GLsizei>(arrays[POS_COLORED_FACES].size()/3));
       vao[VAO_COLORED_FACES].release();
 
+      if (is_two_dimensional())
+        glPolygonOffset(offset_factor, offset_units);
+
       rendering_program_face.release();
     }
 
-    if (!is_empty() && (has_zero_x() || has_zero_y() || has_zero_z()))
+    if (is_two_dimensional())
     {
       camera()->setType(CGAL::qglviewer::Camera::ORTHOGRAPHIC);
       //      Camera Constraint:
@@ -893,6 +1184,21 @@ protected:
       constraint.setRotationConstraintDirection(CGAL::qglviewer::Vec(cx, cy, cz));
       camera()->frame()->setConstraint(&constraint);
     }
+
+    if (m_draw_text)
+    {
+      glDisable(GL_LIGHTING);
+      for (std::size_t i=0; i<m_texts.size(); ++i)
+      {
+        CGAL::qglviewer::Vec screenPos=camera()->projectedCoordinatesOf
+          (CGAL::qglviewer::Vec(std::get<0>(m_texts[i]).x(),
+                                std::get<0>(m_texts[i]).y(),
+                                std::get<0>(m_texts[i]).z()));
+
+        drawText((int)screenPos[0], (int)screenPos[1], std::get<1>(m_texts[i]));
+      }
+      glEnable(GL_LIGHTING);
+    }
   }
 
   virtual void redraw()
@@ -912,11 +1218,13 @@ protected:
 
     // Add custom key description (see keyPressEvent).
     setKeyDescription(::Qt::Key_E, "Toggles edges display");
-    setKeyDescription(::Qt::Key_F, "Toggles faces display");
-    setKeyDescription(::Qt::Key_G, "Switch between flat/Gouraud shading display");
     setKeyDescription(::Qt::Key_M, "Toggles mono color");
     setKeyDescription(::Qt::Key_N, "Inverse direction of normals");
+    setKeyDescription(::Qt::Key_S, "Switch between flat/Gouraud shading display");
+    setKeyDescription(::Qt::Key_T, "Toggles text display");
+    setKeyDescription(::Qt::Key_U, "Move camera direction upside down");
     setKeyDescription(::Qt::Key_V, "Toggles vertices display");
+    setKeyDescription(::Qt::Key_W, "Toggles faces display");
     setKeyDescription(::Qt::Key_Plus, "Increase size of edges");
     setKeyDescription(::Qt::Key_Minus, "Decrease size of edges");
     setKeyDescription(::Qt::Key_Plus+::Qt::ControlModifier, "Increase size of vertices");
@@ -957,11 +1265,8 @@ protected:
 
   void negate_all_normals()
   {
-    for (unsigned int k=BEGIN_NORMAL; k<END_NORMAL; ++k)
-    {
-      for (std::size_t i=0; i<arrays[k].size(); ++i)
-      { arrays[k][i]=-arrays[k][i]; }
-    }
+    m_buffer_for_mono_faces.negate_normals();
+    m_buffer_for_colored_faces.negate_normals();
   }
 
   virtual void keyPressEvent(QKeyEvent *e)
@@ -974,13 +1279,7 @@ protected:
       displayMessage(QString("Draw edges=%1.").arg(m_draw_edges?"true":"false"));
       update();
     }
-    else if ((e->key()==::Qt::Key_F) && (modifiers==::Qt::NoButton))
-    {
-      m_draw_faces=!m_draw_faces;
-      displayMessage(QString("Draw faces=%1.").arg(m_draw_faces?"true":"false"));
-      update();
-    }
-    else if ((e->key()==::Qt::Key_G) && (modifiers==::Qt::NoButton))
+    else if ((e->key()==::Qt::Key_S) && (modifiers==::Qt::NoButton))
     {
       m_flatShading=!m_flatShading;
       if (m_flatShading)
@@ -1002,12 +1301,48 @@ protected:
       negate_all_normals();
       redraw();
     }
+    else if ((e->key()==::Qt::Key_T) && (modifiers==::Qt::NoButton))
+    {
+      m_draw_text=!m_draw_text;
+      displayMessage(QString("Draw text=%1.").arg(m_draw_text?"true":"false"));
+      update();
+    }
+    else if ((e->key()==::Qt::Key_U) && (modifiers==::Qt::NoButton))
+    {
+      if (is_two_dimensional())
+      {
+        displayMessage(QString("Move camera direction upside down."));
+        /* CGAL::qglviewer::Vec cur=camera()->viewDirection();
+        double cx=cur.x, cy=cur.y, cz=cur.z;
+        if (has_zero_x())      { cx=-cx; }
+        else if (has_zero_y()) { cy=-cy; }
+        else                   { cz=-cz; }
+        double cx=0., cy=0., cz=0.;
+        if (has_zero_x())      { cx=(cur.x<0?-1.:1); }
+        else if (has_zero_y()) { cy=(cur.y<0?-1.:1); }
+        else                   { cz=(cur.z<0?-1.:1); }*/
+
+        camera()->setUpVector(-camera()->upVector());
+        //camera()->frame()->setConstraint(NULL);
+        // camera()->setViewDirection(CGAL::qglviewer::Vec(-cx,-cy,-cz));
+        //constraint.setRotationConstraintDirection(CGAL::qglviewer::Vec(cx, cy, cz));
+        //camera()->frame()->setConstraint(&constraint);
+        //update();
+        redraw();
+      }
+    }
     else if ((e->key()==::Qt::Key_V) && (modifiers==::Qt::NoButton))
     {
       m_draw_vertices=!m_draw_vertices;
       displayMessage(QString("Draw vertices=%1.").arg(m_draw_vertices?"true":"false"));
       update();
     }
+    else if ((e->key()==::Qt::Key_W) && (modifiers==::Qt::NoButton))
+    {
+      m_draw_faces=!m_draw_faces;
+      displayMessage(QString("Draw faces=%1.").arg(m_draw_faces?"true":"false"));
+      update();
+    }
     else if ((e->key()==::Qt::Key_Plus) && (!modifiers.testFlag(::Qt::ControlModifier))) // No ctrl
     {
       m_size_edges+=.5;
@@ -1146,16 +1481,23 @@ protected:
 protected:
   bool m_draw_vertices;
   bool m_draw_edges;
+  bool m_draw_rays;
+  bool m_draw_lines;
   bool m_draw_faces;
   bool m_flatShading;
   bool m_use_mono_color;
   bool m_inverse_normal;
+  bool m_draw_text;
 
   double m_size_points;
   double m_size_edges;
+  double m_size_rays;
+  double m_size_lines;
 
   CGAL::Color m_vertices_mono_color;
   CGAL::Color m_edges_mono_color;
+  CGAL::Color m_rays_mono_color;
+  CGAL::Color m_lines_mono_color;
   CGAL::Color m_faces_mono_color;
   QVector4D   m_ambient_color;
 
@@ -1173,12 +1515,18 @@ protected:
     POS_COLORED_POINTS,
     POS_MONO_SEGMENTS,
     POS_COLORED_SEGMENTS,
+    POS_MONO_RAYS,
+    POS_COLORED_RAYS,
+    POS_MONO_LINES,
+    POS_COLORED_LINES,
     POS_MONO_FACES,
     POS_COLORED_FACES,
     END_POS,
     BEGIN_COLOR=END_POS,
     COLOR_POINTS=BEGIN_COLOR,
     COLOR_SEGMENTS,
+    COLOR_RAYS,
+    COLOR_LINES,
     COLOR_FACES,
     END_COLOR,
     BEGIN_NORMAL=END_COLOR,
@@ -1195,6 +1543,10 @@ protected:
   Buffer_for_vao<float> m_buffer_for_colored_points;
   Buffer_for_vao<float> m_buffer_for_mono_segments;
   Buffer_for_vao<float> m_buffer_for_colored_segments;
+  Buffer_for_vao<float> m_buffer_for_mono_rays;
+  Buffer_for_vao<float> m_buffer_for_colored_rays;
+  Buffer_for_vao<float> m_buffer_for_mono_lines;
+  Buffer_for_vao<float> m_buffer_for_colored_lines;
   Buffer_for_vao<float> m_buffer_for_mono_faces;
   Buffer_for_vao<float> m_buffer_for_colored_faces;
 
@@ -1209,6 +1561,10 @@ protected:
       VAO_COLORED_POINTS,
       VAO_MONO_SEGMENTS,
       VAO_COLORED_SEGMENTS,
+      VAO_MONO_RAYS,
+      VAO_COLORED_RAYS,
+      VAO_MONO_LINES,
+      VAO_COLORED_LINES,
       VAO_MONO_FACES,
       VAO_COLORED_FACES,
       NB_VAO_BUFFERS
@@ -1217,6 +1573,8 @@ protected:
 
   QOpenGLShaderProgram rendering_program_face;
   QOpenGLShaderProgram rendering_program_p_l;
+
+  std::vector<std::tuple<Local_point, QString> > m_texts;
 };
 
 } // End namespace CGAL
diff -pruN 5.0.3-1/include/CGAL/Qt/camera.h 5.1-1/include/CGAL/Qt/camera.h
--- 5.0.3-1/include/CGAL/Qt/camera.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/camera.h	2020-09-08 12:14:52.000000000 +0000
@@ -6,7 +6,7 @@
  This file is part of a fork of the QGLViewer library version 2.7.0.
 
 *****************************************************************************/
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/camera.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/camera.h $
 // $Id: camera.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-only
 
diff -pruN 5.0.3-1/include/CGAL/Qt/camera_impl.h 5.1-1/include/CGAL/Qt/camera_impl.h
--- 5.0.3-1/include/CGAL/Qt/camera_impl.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/camera_impl.h	2020-09-08 12:14:52.000000000 +0000
@@ -6,8 +6,8 @@
  This file is part of a fork of the QGLViewer library version 2.7.0.
 
 *****************************************************************************/
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/camera_impl.h $
-// $Id: camera_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/camera_impl.h $
+// $Id: camera_impl.h 66040cb 2020-07-20T17:13:01+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-only
 
 #ifdef CGAL_HEADER_ONLY
@@ -1725,6 +1725,11 @@ CGAL_INLINE_FUNCTION
    MAT(out, 3, 3) = r3[7];
    return 1;
   }
+
+#undef MAT
+#undef SWAP_ROWS_GLdouble
+#undef SWAP_ROWS_DOUBLE
+
 CGAL_INLINE_FUNCTION
 int unProject(GLdouble winx, GLdouble winy, GLdouble winz, GLdouble *modelview, GLdouble *projection, int *viewport,
               GLdouble *objX,GLdouble *objY,GLdouble *objZ)
diff -pruN 5.0.3-1/include/CGAL/Qt/CGAL_Qt_config.h 5.1-1/include/CGAL/Qt/CGAL_Qt_config.h
--- 5.0.3-1/include/CGAL/Qt/CGAL_Qt_config.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/CGAL_Qt_config.h	2020-09-08 12:14:52.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/CGAL_Qt_config.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/CGAL_Qt_config.h $
 // $Id: CGAL_Qt_config.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/CircularArcGraphicsItem.h 5.1-1/include/CGAL/Qt/CircularArcGraphicsItem.h
--- 5.0.3-1/include/CGAL/Qt/CircularArcGraphicsItem.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/CircularArcGraphicsItem.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/CircularArcGraphicsItem.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/CircularArcGraphicsItem.h $
 // $Id: CircularArcGraphicsItem.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/ConstrainedTriangulationGraphicsItem.h 5.1-1/include/CGAL/Qt/ConstrainedTriangulationGraphicsItem.h
--- 5.0.3-1/include/CGAL/Qt/ConstrainedTriangulationGraphicsItem.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/ConstrainedTriangulationGraphicsItem.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/ConstrainedTriangulationGraphicsItem.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/ConstrainedTriangulationGraphicsItem.h $
 // $Id: ConstrainedTriangulationGraphicsItem.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/constraint.h 5.1-1/include/CGAL/Qt/constraint.h
--- 5.0.3-1/include/CGAL/Qt/constraint.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/constraint.h	2020-09-08 12:14:52.000000000 +0000
@@ -6,7 +6,7 @@
  This file is part of a fork of the QGLViewer library version 2.7.0.
 
 *****************************************************************************/
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/constraint.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/constraint.h $
 // $Id: constraint.h 1ef976e 2019-10-19T16:09:56+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-only
 #ifndef QGLVIEWER_CONSTRAINT_H
diff -pruN 5.0.3-1/include/CGAL/Qt/constraint_impl.h 5.1-1/include/CGAL/Qt/constraint_impl.h
--- 5.0.3-1/include/CGAL/Qt/constraint_impl.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/constraint_impl.h	2020-09-08 12:14:52.000000000 +0000
@@ -6,7 +6,7 @@
  This file is part of a fork of the QGLViewer library version 2.7.0.
 
 *****************************************************************************/
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/constraint_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/constraint_impl.h $
 // $Id: constraint_impl.h 1ef976e 2019-10-19T16:09:56+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-only
 
diff -pruN 5.0.3-1/include/CGAL/Qt/Converter.h 5.1-1/include/CGAL/Qt/Converter.h
--- 5.0.3-1/include/CGAL/Qt/Converter.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/Converter.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/Converter.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/Converter.h $
 // $Id: Converter.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/CreateOpenGLContext.h 5.1-1/include/CGAL/Qt/CreateOpenGLContext.h
--- 5.0.3-1/include/CGAL/Qt/CreateOpenGLContext.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/CreateOpenGLContext.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/CreateOpenGLContext.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/CreateOpenGLContext.h $
 // $Id: CreateOpenGLContext.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/debug.h 5.1-1/include/CGAL/Qt/debug.h
--- 5.0.3-1/include/CGAL/Qt/debug.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/debug.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/debug.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/debug.h $
 // $Id: debug.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/debug_impl.h 5.1-1/include/CGAL/Qt/debug_impl.h
--- 5.0.3-1/include/CGAL/Qt/debug_impl.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/debug_impl.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/debug_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/debug_impl.h $
 // $Id: debug_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/DelaunayMeshTriangulationGraphicsItem.h 5.1-1/include/CGAL/Qt/DelaunayMeshTriangulationGraphicsItem.h
--- 5.0.3-1/include/CGAL/Qt/DelaunayMeshTriangulationGraphicsItem.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/DelaunayMeshTriangulationGraphicsItem.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/DelaunayMeshTriangulationGraphicsItem.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/DelaunayMeshTriangulationGraphicsItem.h $
 // $Id: DelaunayMeshTriangulationGraphicsItem.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/DemosMainWindow.h 5.1-1/include/CGAL/Qt/DemosMainWindow.h
--- 5.0.3-1/include/CGAL/Qt/DemosMainWindow.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/DemosMainWindow.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/DemosMainWindow.h $
-// $Id: DemosMainWindow.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/DemosMainWindow.h $
+// $Id: DemosMainWindow.h 624e8e8 2020-06-12T10:34:48+02:00 Maxime Gimeno
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -72,7 +72,7 @@ private:
   QMenu* getHelpMenu();
 
 protected:
-  DemosMainWindow (QWidget * parent = 0, ::Qt::WindowFlags flags = 0 );
+  DemosMainWindow (QWidget * parent = 0, ::Qt::WindowFlags flags = ::Qt::WindowType(0) );
   ~DemosMainWindow();
   void setupStatusBar();
   void addNavigation(QGraphicsView*);
diff -pruN 5.0.3-1/include/CGAL/Qt/DemosMainWindow_impl.h 5.1-1/include/CGAL/Qt/DemosMainWindow_impl.h
--- 5.0.3-1/include/CGAL/Qt/DemosMainWindow_impl.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/DemosMainWindow_impl.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/DemosMainWindow_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/DemosMainWindow_impl.h $
 // $Id: DemosMainWindow_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/domUtils.h 5.1-1/include/CGAL/Qt/domUtils.h
--- 5.0.3-1/include/CGAL/Qt/domUtils.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/domUtils.h	2020-09-08 12:14:52.000000000 +0000
@@ -6,7 +6,7 @@
  This file is part of a fork of the QGLViewer library version 2.7.0.
 
 *****************************************************************************/
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/domUtils.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/domUtils.h $
 // $Id: domUtils.h 1ef976e 2019-10-19T16:09:56+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-only
 #ifndef QGLVIEWER_DOMUTILS_H
diff -pruN 5.0.3-1/include/CGAL/Qt/frame.h 5.1-1/include/CGAL/Qt/frame.h
--- 5.0.3-1/include/CGAL/Qt/frame.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/frame.h	2020-09-08 12:14:52.000000000 +0000
@@ -6,7 +6,7 @@
  This file is part of a fork of the QGLViewer library version 2.7.0.
 
 *****************************************************************************/
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/frame.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/frame.h $
 // $Id: frame.h 1ef976e 2019-10-19T16:09:56+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-only
 #ifndef QGLVIEWER_FRAME_H
diff -pruN 5.0.3-1/include/CGAL/Qt/frame_impl.h 5.1-1/include/CGAL/Qt/frame_impl.h
--- 5.0.3-1/include/CGAL/Qt/frame_impl.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/frame_impl.h	2020-09-08 12:14:52.000000000 +0000
@@ -6,7 +6,7 @@
  This file is part of a fork of the QGLViewer library version 2.7.0.
 
 *****************************************************************************/
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/frame_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/frame_impl.h $
 // $Id: frame_impl.h 1ef976e 2019-10-19T16:09:56+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-only
 
diff -pruN 5.0.3-1/include/CGAL/Qt/GraphicsItem.h 5.1-1/include/CGAL/Qt/GraphicsItem.h
--- 5.0.3-1/include/CGAL/Qt/GraphicsItem.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/GraphicsItem.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/GraphicsItem.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/GraphicsItem.h $
 // $Id: GraphicsItem.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/GraphicsViewCircleInput.h 5.1-1/include/CGAL/Qt/GraphicsViewCircleInput.h
--- 5.0.3-1/include/CGAL/Qt/GraphicsViewCircleInput.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/GraphicsViewCircleInput.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/GraphicsViewCircleInput.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/GraphicsViewCircleInput.h $
 // $Id: GraphicsViewCircleInput.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/GraphicsViewCircularArcInput.h 5.1-1/include/CGAL/Qt/GraphicsViewCircularArcInput.h
--- 5.0.3-1/include/CGAL/Qt/GraphicsViewCircularArcInput.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/GraphicsViewCircularArcInput.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/GraphicsViewCircularArcInput.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/GraphicsViewCircularArcInput.h $
 // $Id: GraphicsViewCircularArcInput.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/GraphicsViewInput.h 5.1-1/include/CGAL/Qt/GraphicsViewInput.h
--- 5.0.3-1/include/CGAL/Qt/GraphicsViewInput.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/GraphicsViewInput.h	2020-09-08 12:14:52.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/GraphicsViewInput.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/GraphicsViewInput.h $
 // $Id: GraphicsViewInput.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/GraphicsViewIsoRectangleInput.h 5.1-1/include/CGAL/Qt/GraphicsViewIsoRectangleInput.h
--- 5.0.3-1/include/CGAL/Qt/GraphicsViewIsoRectangleInput.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/GraphicsViewIsoRectangleInput.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/GraphicsViewIsoRectangleInput.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/GraphicsViewIsoRectangleInput.h $
 // $Id: GraphicsViewIsoRectangleInput.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/GraphicsViewLineInput.h 5.1-1/include/CGAL/Qt/GraphicsViewLineInput.h
--- 5.0.3-1/include/CGAL/Qt/GraphicsViewLineInput.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/GraphicsViewLineInput.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/GraphicsViewLineInput.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/GraphicsViewLineInput.h $
 // $Id: GraphicsViewLineInput.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/GraphicsViewNavigation.h 5.1-1/include/CGAL/Qt/GraphicsViewNavigation.h
--- 5.0.3-1/include/CGAL/Qt/GraphicsViewNavigation.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/GraphicsViewNavigation.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/GraphicsViewNavigation.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/GraphicsViewNavigation.h $
 // $Id: GraphicsViewNavigation.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/GraphicsViewNavigation_impl.h 5.1-1/include/CGAL/Qt/GraphicsViewNavigation_impl.h
--- 5.0.3-1/include/CGAL/Qt/GraphicsViewNavigation_impl.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/GraphicsViewNavigation_impl.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/GraphicsViewNavigation_impl.h $
-// $Id: GraphicsViewNavigation_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/GraphicsViewNavigation_impl.h $
+// $Id: GraphicsViewNavigation_impl.h a9795c3 2020-06-11T13:11:36+02:00 Maxime Gimeno
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -134,15 +134,12 @@ namespace Qt {
     } // end case KeyRelease
     case QEvent::Wheel: {
       QWheelEvent *wheelEvent = static_cast<QWheelEvent*>(event);
-      if(wheelEvent->orientation() != ::Qt::Vertical) {
-        return false;
-      }
       double zoom_ratio = 240.0;
       if( (wheelEvent->modifiers() & ::Qt::ShiftModifier)
           || (wheelEvent->modifiers() & ::Qt::ControlModifier) ) {
         zoom_ratio = 120.0;
       }
-      scaleView(v, pow((double)2, -wheelEvent->delta() / zoom_ratio));
+      scaleView(v, pow((double)2, -wheelEvent->angleDelta().y() / zoom_ratio));
 
       //         display_parameters();
       return true;
@@ -311,12 +308,12 @@ namespace Qt {
       boost::format("matrix translation=(%1%, %2%)\n"
                     "       rotation=(%3% - %4% )\n"
                     "                (%5% - %6% )\n")
-      % v->matrix().dx()
-      % v->matrix().dy()
-      % v->matrix().m11()
-      % v->matrix().m12()
-      % v->matrix().m21()
-      % v->matrix().m22();
+      % v->transform().dx()
+      % v->transform().dy()
+      % v->transform().m11()
+      % v->transform().m12()
+      % v->transform().m21()
+      % v->transform().m22();
 
     QRect vp_rect = v->viewport()->rect();
     QPoint vp_top_left = vp_rect.topLeft();
diff -pruN 5.0.3-1/include/CGAL/Qt/GraphicsViewPointInput.h 5.1-1/include/CGAL/Qt/GraphicsViewPointInput.h
--- 5.0.3-1/include/CGAL/Qt/GraphicsViewPointInput.h	2020-08-01 19:02:13.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/GraphicsViewPointInput.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Qt/GraphicsViewPointInput.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Qt/GraphicsViewPointInput.h $
 // $Id: GraphicsViewPointInput.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 
diff -pruN 5.0.3-1/include/CGAL/Qt/GraphicsViewPolygonWithHolesInput.h 5.1-1/include/CGAL/Qt/GraphicsViewPolygonWithHolesInput.h
--- 5.0.3-1/include/CGAL/Qt/GraphicsViewPolygonWithHolesInput.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/GraphicsViewPolygonWithHolesInput.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/GraphicsViewPolygonWithHolesInput.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/GraphicsViewPolygonWithHolesInput.h $
 // $Id: GraphicsViewPolygonWithHolesInput.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/GraphicsViewPolylineInput.h 5.1-1/include/CGAL/Qt/GraphicsViewPolylineInput.h
--- 5.0.3-1/include/CGAL/Qt/GraphicsViewPolylineInput.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/GraphicsViewPolylineInput.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/GraphicsViewPolylineInput.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/GraphicsViewPolylineInput.h $
 // $Id: GraphicsViewPolylineInput.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/GraphicsViewPolylineInput_impl.h 5.1-1/include/CGAL/Qt/GraphicsViewPolylineInput_impl.h
--- 5.0.3-1/include/CGAL/Qt/GraphicsViewPolylineInput_impl.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/GraphicsViewPolylineInput_impl.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/GraphicsViewPolylineInput_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/GraphicsViewPolylineInput_impl.h $
 // $Id: GraphicsViewPolylineInput_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/image_interface.h 5.1-1/include/CGAL/Qt/image_interface.h
--- 5.0.3-1/include/CGAL/Qt/image_interface.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/image_interface.h	2020-09-08 12:14:52.000000000 +0000
@@ -6,7 +6,7 @@
  This file is part of a fork of the QGLViewer library version 2.7.0.
 
 *****************************************************************************/
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/image_interface.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/image_interface.h $
 // $Id: image_interface.h 1ef976e 2019-10-19T16:09:56+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-only
 
diff -pruN 5.0.3-1/include/CGAL/Qt/keyFrameInterpolator.h 5.1-1/include/CGAL/Qt/keyFrameInterpolator.h
--- 5.0.3-1/include/CGAL/Qt/keyFrameInterpolator.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/keyFrameInterpolator.h	2020-09-08 12:14:52.000000000 +0000
@@ -6,7 +6,7 @@
  This file is part of a fork of the QGLViewer library version 2.7.0.
 
 *****************************************************************************/
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/keyFrameInterpolator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/keyFrameInterpolator.h $
 // $Id: keyFrameInterpolator.h 1ef976e 2019-10-19T16:09:56+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-only
 #ifndef QGLVIEWER_KEY_FRAME_INTERPOLATOR_H
diff -pruN 5.0.3-1/include/CGAL/Qt/keyFrameInterpolator_impl.h 5.1-1/include/CGAL/Qt/keyFrameInterpolator_impl.h
--- 5.0.3-1/include/CGAL/Qt/keyFrameInterpolator_impl.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/keyFrameInterpolator_impl.h	2020-09-08 12:14:52.000000000 +0000
@@ -6,7 +6,7 @@
  This file is part of a fork of the QGLViewer library version 2.7.0.
 
 *****************************************************************************/
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/keyFrameInterpolator_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/keyFrameInterpolator_impl.h $
 // $Id: keyFrameInterpolator_impl.h 1ef976e 2019-10-19T16:09:56+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-only
 #ifdef CGAL_HEADER_ONLY
diff -pruN 5.0.3-1/include/CGAL/Qt/LineGraphicsItem.h 5.1-1/include/CGAL/Qt/LineGraphicsItem.h
--- 5.0.3-1/include/CGAL/Qt/LineGraphicsItem.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/LineGraphicsItem.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/LineGraphicsItem.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/LineGraphicsItem.h $
 // $Id: LineGraphicsItem.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/manipulatedCameraFrame.h 5.1-1/include/CGAL/Qt/manipulatedCameraFrame.h
--- 5.0.3-1/include/CGAL/Qt/manipulatedCameraFrame.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/manipulatedCameraFrame.h	2020-09-08 12:14:52.000000000 +0000
@@ -6,7 +6,7 @@
  This file is part of a fork of the QGLViewer library version 2.7.0.
 
 *****************************************************************************/
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/manipulatedCameraFrame.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/manipulatedCameraFrame.h $
 // $Id: manipulatedCameraFrame.h 1ef976e 2019-10-19T16:09:56+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-only
 
diff -pruN 5.0.3-1/include/CGAL/Qt/manipulatedCameraFrame_impl.h 5.1-1/include/CGAL/Qt/manipulatedCameraFrame_impl.h
--- 5.0.3-1/include/CGAL/Qt/manipulatedCameraFrame_impl.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/manipulatedCameraFrame_impl.h	2020-09-08 12:14:52.000000000 +0000
@@ -6,8 +6,8 @@
  This file is part of a fork of the QGLViewer library version 2.7.0.
 
 *****************************************************************************/
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/manipulatedCameraFrame_impl.h $
-// $Id: manipulatedCameraFrame_impl.h 1ef976e 2019-10-19T16:09:56+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/manipulatedCameraFrame_impl.h $
+// $Id: manipulatedCameraFrame_impl.h a9795c3 2020-06-11T13:11:36+02:00 Maxime Gimeno
 // SPDX-License-Identifier: GPL-3.0-only
 
 #ifdef CGAL_HEADER_ONLY
@@ -421,7 +421,7 @@ void ManipulatedCameraFrame::wheelEvent(
   case MOVE_BACKWARD:
     //#CONNECTION# mouseMoveEvent() MOVE_FORWARD case
     translate(
-        inverseTransformOf(Vec(0.0, 0.0, 0.2 * flySpeed() * event->delta())));
+        inverseTransformOf(Vec(0.0, 0.0, 0.2 * flySpeed() * event->angleDelta().y())));
     Q_EMIT manipulated();
     break;
   default:
diff -pruN 5.0.3-1/include/CGAL/Qt/manipulatedFrame.h 5.1-1/include/CGAL/Qt/manipulatedFrame.h
--- 5.0.3-1/include/CGAL/Qt/manipulatedFrame.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/manipulatedFrame.h	2020-09-08 12:14:52.000000000 +0000
@@ -6,7 +6,7 @@
  This file is part of a fork of the QGLViewer library version 2.7.0.
 
 *****************************************************************************/
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/manipulatedFrame.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/manipulatedFrame.h $
 // $Id: manipulatedFrame.h 1968f2c 2020-01-02T10:00:05+01:00 Maxime Gimeno
 // SPDX-License-Identifier: GPL-3.0-only
 
diff -pruN 5.0.3-1/include/CGAL/Qt/manipulatedFrame_impl.h 5.1-1/include/CGAL/Qt/manipulatedFrame_impl.h
--- 5.0.3-1/include/CGAL/Qt/manipulatedFrame_impl.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/manipulatedFrame_impl.h	2020-09-08 12:14:53.000000000 +0000
@@ -6,8 +6,8 @@
  This file is part of a fork of the QGLViewer library version 2.7.0.
 
 *****************************************************************************/
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/manipulatedFrame_impl.h $
-// $Id: manipulatedFrame_impl.h 1ef976e 2019-10-19T16:09:56+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/manipulatedFrame_impl.h $
+// $Id: manipulatedFrame_impl.h a9795c3 2020-06-11T13:11:36+02:00 Maxime Gimeno
 // SPDX-License-Identifier: GPL-3.0-only
 
 #ifdef CGAL_HEADER_ONLY
@@ -251,7 +251,7 @@ CGAL_INLINE_FUNCTION
 void ManipulatedFrame::computeMouseSpeed(const QMouseEvent *const e) {
   const QPoint delta = (e->pos() - prevPos_);
   const qreal dist = sqrt(qreal(delta.x() * delta.x() + delta.y() * delta.y()));
-  delay_ = last_move_time.restart();
+  delay_ = static_cast<int>(last_move_time.restart());
   if (delay_ == 0)
     // Less than a millisecond: assume delay = 1ms
     mouseSpeed_ = dist;
@@ -295,7 +295,7 @@ qreal ManipulatedFrame::deltaWithPrevPos
 CGAL_INLINE_FUNCTION
 qreal ManipulatedFrame::wheelDelta(const QWheelEvent *event) const {
   static const qreal WHEEL_SENSITIVITY_COEF = 8E-4;
-  return event->delta() * wheelSensitivity() * WHEEL_SENSITIVITY_COEF;
+  return event->angleDelta().y() * wheelSensitivity() * WHEEL_SENSITIVITY_COEF;
 }
 
 CGAL_INLINE_FUNCTION
diff -pruN 5.0.3-1/include/CGAL/Qt/mouseGrabber.h 5.1-1/include/CGAL/Qt/mouseGrabber.h
--- 5.0.3-1/include/CGAL/Qt/mouseGrabber.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/mouseGrabber.h	2020-09-08 12:14:53.000000000 +0000
@@ -6,7 +6,7 @@
  This file is part of a fork of the QGLViewer library version 2.7.0.
 
 *****************************************************************************/
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/mouseGrabber.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/mouseGrabber.h $
 // $Id: mouseGrabber.h 1ef976e 2019-10-19T16:09:56+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-only
 
diff -pruN 5.0.3-1/include/CGAL/Qt/mouseGrabber_impl.h 5.1-1/include/CGAL/Qt/mouseGrabber_impl.h
--- 5.0.3-1/include/CGAL/Qt/mouseGrabber_impl.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/mouseGrabber_impl.h	2020-09-08 12:14:53.000000000 +0000
@@ -6,7 +6,7 @@
  This file is part of a fork of the QGLViewer library version 2.7.0.
 
 *****************************************************************************/
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/mouseGrabber_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/mouseGrabber_impl.h $
 // $Id: mouseGrabber_impl.h 1ef976e 2019-10-19T16:09:56+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-only
 
diff -pruN 5.0.3-1/include/CGAL/Qt/PainterOstream.h 5.1-1/include/CGAL/Qt/PainterOstream.h
--- 5.0.3-1/include/CGAL/Qt/PainterOstream.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/PainterOstream.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/PainterOstream.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/PainterOstream.h $
 // $Id: PainterOstream.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/PointsGraphicsItem.h 5.1-1/include/CGAL/Qt/PointsGraphicsItem.h
--- 5.0.3-1/include/CGAL/Qt/PointsGraphicsItem.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/PointsGraphicsItem.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/PointsGraphicsItem.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/PointsGraphicsItem.h $
 // $Id: PointsGraphicsItem.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/PointsInKdTreeGraphicsItem.h 5.1-1/include/CGAL/Qt/PointsInKdTreeGraphicsItem.h
--- 5.0.3-1/include/CGAL/Qt/PointsInKdTreeGraphicsItem.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/PointsInKdTreeGraphicsItem.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/PointsInKdTreeGraphicsItem.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/PointsInKdTreeGraphicsItem.h $
 // $Id: PointsInKdTreeGraphicsItem.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/PolygonGraphicsItem.h 5.1-1/include/CGAL/Qt/PolygonGraphicsItem.h
--- 5.0.3-1/include/CGAL/Qt/PolygonGraphicsItem.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/PolygonGraphicsItem.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/PolygonGraphicsItem.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/PolygonGraphicsItem.h $
 // $Id: PolygonGraphicsItem.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/PolygonWithHolesGraphicsItem.h 5.1-1/include/CGAL/Qt/PolygonWithHolesGraphicsItem.h
--- 5.0.3-1/include/CGAL/Qt/PolygonWithHolesGraphicsItem.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/PolygonWithHolesGraphicsItem.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/PolygonWithHolesGraphicsItem.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/PolygonWithHolesGraphicsItem.h $
 // $Id: PolygonWithHolesGraphicsItem.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/PolylinesGraphicsItem.h 5.1-1/include/CGAL/Qt/PolylinesGraphicsItem.h
--- 5.0.3-1/include/CGAL/Qt/PolylinesGraphicsItem.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/PolylinesGraphicsItem.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/PolylinesGraphicsItem.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/PolylinesGraphicsItem.h $
 // $Id: PolylinesGraphicsItem.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/PowerdiagramGraphicsItem.h 5.1-1/include/CGAL/Qt/PowerdiagramGraphicsItem.h
--- 5.0.3-1/include/CGAL/Qt/PowerdiagramGraphicsItem.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/PowerdiagramGraphicsItem.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/PowerdiagramGraphicsItem.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/PowerdiagramGraphicsItem.h $
 // $Id: PowerdiagramGraphicsItem.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/qglviewer.h 5.1-1/include/CGAL/Qt/qglviewer.h
--- 5.0.3-1/include/CGAL/Qt/qglviewer.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/qglviewer.h	2020-09-08 12:14:53.000000000 +0000
@@ -6,8 +6,8 @@
  This file is part of a fork of the QGLViewer library version 2.7.0.
 
 *****************************************************************************/
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/qglviewer.h $
-// $Id: qglviewer.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/qglviewer.h $
+// $Id: qglviewer.h 624e8e8 2020-06-12T10:34:48+02:00 Maxime Gimeno
 // SPDX-License-Identifier: GPL-3.0-only
 
 #ifndef QGLVIEWER_QGLVIEWER_H
@@ -73,11 +73,11 @@ class CGAL_QT_EXPORT QGLViewer : public
 public:
   //todo check if this is used. If not remove it
   explicit QGLViewer(QGLContext* context, QWidget *parent = 0,
-                     ::Qt::WindowFlags flags = 0);
+                     ::Qt::WindowFlags flags = ::Qt::WindowType(0));
   explicit QGLViewer(QOpenGLContext* context, QWidget *parent = 0,
-                     ::Qt::WindowFlags flags = 0);
+                     ::Qt::WindowFlags flags = ::Qt::WindowType(0));
   explicit QGLViewer(QWidget *parent = 0,
-                     ::Qt::WindowFlags flags = 0);
+                     ::Qt::WindowFlags flags = ::Qt::WindowType(0));
 
   virtual ~QGLViewer();
 
diff -pruN 5.0.3-1/include/CGAL/Qt/qglviewer_impl.h 5.1-1/include/CGAL/Qt/qglviewer_impl.h
--- 5.0.3-1/include/CGAL/Qt/qglviewer_impl.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/qglviewer_impl.h	2020-09-08 12:14:53.000000000 +0000
@@ -6,8 +6,8 @@
  This file is part of a fork of the QGLViewer library version 2.7.0.
 
 *****************************************************************************/
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/qglviewer_impl.h $
-// $Id: qglviewer_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/qglviewer_impl.h $
+// $Id: qglviewer_impl.h 58b10a3 2020-03-26T18:58:50+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-only
 
 #ifdef CGAL_HEADER_ONLY
@@ -2248,7 +2248,7 @@ void CGAL::QGLViewer::keyPressEvent(QKey
     static QElapsedTimer doublePress;
 
     if (modifiers == playPathKeyboardModifiers()) {
-      int elapsed = doublePress.restart();
+      qint64 elapsed = doublePress.restart();
       if ((elapsed < 250) && (index == previousPathId_))
         camera()->resetPath(index);
       else {
@@ -2263,7 +2263,7 @@ void CGAL::QGLViewer::keyPressEvent(QKey
       }
       previousPathId_ = index;
     } else if (modifiers == addKeyFrameKeyboardModifiers()) {
-      int elapsed = doublePress.restart();
+      qint64 elapsed = doublePress.restart();
       if ((elapsed < 250) && (index == previousPathId_)) {
         if (camera()->keyFrameInterpolator(index)) {
           disconnect(camera()->keyFrameInterpolator(index),
diff -pruN 5.0.3-1/include/CGAL/Qt/qglviewer_impl_list.h 5.1-1/include/CGAL/Qt/qglviewer_impl_list.h
--- 5.0.3-1/include/CGAL/Qt/qglviewer_impl_list.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/qglviewer_impl_list.h	2020-09-08 12:14:53.000000000 +0000
@@ -6,7 +6,7 @@
  This file is part of a fork of the QGLViewer library version 2.7.0.
 
 *****************************************************************************/
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/qglviewer_impl_list.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/qglviewer_impl_list.h $
 // $Id: qglviewer_impl_list.h 1ef976e 2019-10-19T16:09:56+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-only
 
diff -pruN 5.0.3-1/include/CGAL/Qt/quaternion.h 5.1-1/include/CGAL/Qt/quaternion.h
--- 5.0.3-1/include/CGAL/Qt/quaternion.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/quaternion.h	2020-09-08 12:14:53.000000000 +0000
@@ -6,7 +6,7 @@
  This file is part of a fork of the QGLViewer library version 2.7.0.
 
 *****************************************************************************/
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/quaternion.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/quaternion.h $
 // $Id: quaternion.h 1ef976e 2019-10-19T16:09:56+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-only
 
diff -pruN 5.0.3-1/include/CGAL/Qt/quaternion_impl.h 5.1-1/include/CGAL/Qt/quaternion_impl.h
--- 5.0.3-1/include/CGAL/Qt/quaternion_impl.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/quaternion_impl.h	2020-09-08 12:14:53.000000000 +0000
@@ -6,7 +6,7 @@
  This file is part of a fork of the QGLViewer library version 2.7.0.
 
 *****************************************************************************/
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/quaternion_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/quaternion_impl.h $
 // $Id: quaternion_impl.h 1ef976e 2019-10-19T16:09:56+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-only
 
diff -pruN 5.0.3-1/include/CGAL/Qt/RegularGridGraphicsItem.h 5.1-1/include/CGAL/Qt/RegularGridGraphicsItem.h
--- 5.0.3-1/include/CGAL/Qt/RegularGridGraphicsItem.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/RegularGridGraphicsItem.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/RegularGridGraphicsItem.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/RegularGridGraphicsItem.h $
 // $Id: RegularGridGraphicsItem.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/RegularGridVectorFieldGraphicsItem.h 5.1-1/include/CGAL/Qt/RegularGridVectorFieldGraphicsItem.h
--- 5.0.3-1/include/CGAL/Qt/RegularGridVectorFieldGraphicsItem.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/RegularGridVectorFieldGraphicsItem.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/RegularGridVectorFieldGraphicsItem.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/RegularGridVectorFieldGraphicsItem.h $
 // $Id: RegularGridVectorFieldGraphicsItem.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/RegularTriangulationGraphicsItem.h 5.1-1/include/CGAL/Qt/RegularTriangulationGraphicsItem.h
--- 5.0.3-1/include/CGAL/Qt/RegularTriangulationGraphicsItem.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/RegularTriangulationGraphicsItem.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/RegularTriangulationGraphicsItem.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/RegularTriangulationGraphicsItem.h $
 // $Id: RegularTriangulationGraphicsItem.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/resources.h 5.1-1/include/CGAL/Qt/resources.h
--- 5.0.3-1/include/CGAL/Qt/resources.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/resources.h	2020-09-08 12:14:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/resources.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/resources.h $
 // $Id: resources.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/resources_impl.h 5.1-1/include/CGAL/Qt/resources_impl.h
--- 5.0.3-1/include/CGAL/Qt/resources_impl.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/resources_impl.h	2020-09-08 12:14:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/resources_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/resources_impl.h $
 // $Id: resources_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/SegmentDelaunayGraphGraphicsItem.h 5.1-1/include/CGAL/Qt/SegmentDelaunayGraphGraphicsItem.h
--- 5.0.3-1/include/CGAL/Qt/SegmentDelaunayGraphGraphicsItem.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/SegmentDelaunayGraphGraphicsItem.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/SegmentDelaunayGraphGraphicsItem.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/SegmentDelaunayGraphGraphicsItem.h $
 // $Id: SegmentDelaunayGraphGraphicsItem.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/SegmentDelaunayGraphLinfGraphicsItem.h 5.1-1/include/CGAL/Qt/SegmentDelaunayGraphLinfGraphicsItem.h
--- 5.0.3-1/include/CGAL/Qt/SegmentDelaunayGraphLinfGraphicsItem.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/SegmentDelaunayGraphLinfGraphicsItem.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/SegmentDelaunayGraphLinfGraphicsItem.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/SegmentDelaunayGraphLinfGraphicsItem.h $
 // $Id: SegmentDelaunayGraphLinfGraphicsItem.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/SegmentsGraphicsItem.h 5.1-1/include/CGAL/Qt/SegmentsGraphicsItem.h
--- 5.0.3-1/include/CGAL/Qt/SegmentsGraphicsItem.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/SegmentsGraphicsItem.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/SegmentsGraphicsItem.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/SegmentsGraphicsItem.h $
 // $Id: SegmentsGraphicsItem.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/StreamLinesGraphicsItem.h 5.1-1/include/CGAL/Qt/StreamLinesGraphicsItem.h
--- 5.0.3-1/include/CGAL/Qt/StreamLinesGraphicsItem.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/StreamLinesGraphicsItem.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/StreamLinesGraphicsItem.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/StreamLinesGraphicsItem.h $
 // $Id: StreamLinesGraphicsItem.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/TriangulationGraphicsItem.h 5.1-1/include/CGAL/Qt/TriangulationGraphicsItem.h
--- 5.0.3-1/include/CGAL/Qt/TriangulationGraphicsItem.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/TriangulationGraphicsItem.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/TriangulationGraphicsItem.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/TriangulationGraphicsItem.h $
 // $Id: TriangulationGraphicsItem.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/utility.h 5.1-1/include/CGAL/Qt/utility.h
--- 5.0.3-1/include/CGAL/Qt/utility.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/utility.h	2020-09-08 12:14:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/utility.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/utility.h $
 // $Id: utility.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/utility_impl.h 5.1-1/include/CGAL/Qt/utility_impl.h
--- 5.0.3-1/include/CGAL/Qt/utility_impl.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/utility_impl.h	2020-09-08 12:14:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/utility_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/utility_impl.h $
 // $Id: utility_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Qt/vec.h 5.1-1/include/CGAL/Qt/vec.h
--- 5.0.3-1/include/CGAL/Qt/vec.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/vec.h	2020-09-08 12:14:53.000000000 +0000
@@ -6,7 +6,7 @@
  This file is part of a fork of the QGLViewer library version 2.7.0.
 
 *****************************************************************************/
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/vec.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/vec.h $
 // $Id: vec.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-only
 
diff -pruN 5.0.3-1/include/CGAL/Qt/vec_impl.h 5.1-1/include/CGAL/Qt/vec_impl.h
--- 5.0.3-1/include/CGAL/Qt/vec_impl.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/vec_impl.h	2020-09-08 12:14:53.000000000 +0000
@@ -6,7 +6,7 @@
  This file is part of a fork of the QGLViewer library version 2.7.0.
 
 *****************************************************************************/
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/vec_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/vec_impl.h $
 // $Id: vec_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-only
 
diff -pruN 5.0.3-1/include/CGAL/Qt/viewer_actions.h 5.1-1/include/CGAL/Qt/viewer_actions.h
--- 5.0.3-1/include/CGAL/Qt/viewer_actions.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/viewer_actions.h	2020-09-08 12:14:53.000000000 +0000
@@ -6,7 +6,7 @@
  This file is part of a fork of the QGLViewer library version 2.7.0.
 
 *****************************************************************************/
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/viewer_actions.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/viewer_actions.h $
 // $Id: viewer_actions.h 1ef976e 2019-10-19T16:09:56+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-only
 #ifndef VIEWER_ACTIONS_H
diff -pruN 5.0.3-1/include/CGAL/Qt/VoronoiGraphicsItem.h 5.1-1/include/CGAL/Qt/VoronoiGraphicsItem.h
--- 5.0.3-1/include/CGAL/Qt/VoronoiGraphicsItem.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Qt/VoronoiGraphicsItem.h	2020-09-08 12:14:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/GraphicsView/include/CGAL/Qt/VoronoiGraphicsItem.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/GraphicsView/include/CGAL/Qt/VoronoiGraphicsItem.h $
 // $Id: VoronoiGraphicsItem.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Quotient_fwd.h 5.1-1/include/CGAL/Quotient_fwd.h
--- 5.0.3-1/include/CGAL/Quotient_fwd.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Quotient_fwd.h	2020-09-08 12:15:16.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Quotient_fwd.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Quotient_fwd.h $
 // $Id: Quotient_fwd.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Quotient.h 5.1-1/include/CGAL/Quotient.h
--- 5.0.3-1/include/CGAL/Quotient.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Quotient.h	2020-09-08 12:15:16.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Quotient.h $
-// $Id: Quotient.h 5b2d46c 2020-04-21T10:03:00+02:00 Maxime Gimeno
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Quotient.h $
+// $Id: Quotient.h 14c3b7e 2020-05-27T16:33:33+02:00 Laurent Rineau
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -129,6 +129,15 @@ class Quotient
   Quotient<NT>& operator*= (const CGAL_double(NT)& r);
   Quotient<NT>& operator/= (const CGAL_double(NT)& r);
 
+  friend bool operator==(const Quotient& x, const Quotient& y)
+  { return x.num * y.den == x.den * y.num; }
+  friend bool operator==(const Quotient& x, const NT& y)
+  { return x.den * y == x.num; }
+  friend inline bool operator==(const Quotient& x, const CGAL_int(NT) & y)
+  { return x.den * y == x.num; }
+  friend inline bool operator==(const Quotient& x, const CGAL_double(NT) & y)
+  { return x.den * y == x.num; } // Uh?
+
   Quotient<NT>&    normalize();
 
   const NT&   numerator()   const { return num; }
@@ -438,31 +447,6 @@ quotient_truncation(const Quotient<NT>&
 
 
 
-template <class NT>
-CGAL_MEDIUM_INLINE
-bool
-operator==(const Quotient<NT>& x, const Quotient<NT>& y)
-{ return x.num * y.den == x.den * y.num; }
-
-template <class NT>
-CGAL_MEDIUM_INLINE
-bool
-operator==(const Quotient<NT>& x, const NT& y)
-{ return x.den * y == x.num; }
-
-template <class NT>
-inline
-bool
-operator==(const Quotient<NT>& x, const CGAL_int(NT) & y)
-{ return x.den * y == x.num; }
-
-template <class NT>
-inline
-bool
-operator==(const Quotient<NT>& x, const CGAL_double(NT) & y)
-{ return x.den * y == x.num; }
-
-
 
 template <class NT>
 CGAL_MEDIUM_INLINE
diff -pruN 5.0.3-1/include/CGAL/radial_orient_normals.h 5.1-1/include/CGAL/radial_orient_normals.h
--- 5.0.3-1/include/CGAL/radial_orient_normals.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/radial_orient_normals.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_processing_3/include/CGAL/radial_orient_normals.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/radial_orient_normals.h $
 // $Id: radial_orient_normals.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Random_access_adaptor.h 5.1-1/include/CGAL/Random_access_adaptor.h
--- 5.0.3-1/include/CGAL/Random_access_adaptor.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Random_access_adaptor.h	2020-09-08 12:15:32.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Random_access_adaptor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Random_access_adaptor.h $
 // $Id: Random_access_adaptor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Random_access_value_adaptor.h 5.1-1/include/CGAL/Random_access_value_adaptor.h
--- 5.0.3-1/include/CGAL/Random_access_value_adaptor.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Random_access_value_adaptor.h	2020-09-08 12:15:32.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Random_access_value_adaptor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Random_access_value_adaptor.h $
 // $Id: Random_access_value_adaptor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/random_convex_hull_in_disc_2.h 5.1-1/include/CGAL/random_convex_hull_in_disc_2.h
--- 5.0.3-1/include/CGAL/random_convex_hull_in_disc_2.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/random_convex_hull_in_disc_2.h	2020-09-08 12:14:51.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Generator/include/CGAL/random_convex_hull_in_disc_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Generator/include/CGAL/random_convex_hull_in_disc_2.h $
 // $Id: random_convex_hull_in_disc_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Random_convex_hull_traits_2.h 5.1-1/include/CGAL/Random_convex_hull_traits_2.h
--- 5.0.3-1/include/CGAL/Random_convex_hull_traits_2.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Random_convex_hull_traits_2.h	2020-09-08 12:14:51.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Generator/include/CGAL/Random_convex_hull_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Generator/include/CGAL/Random_convex_hull_traits_2.h $
 // $Id: Random_convex_hull_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/random_convex_set_2.h 5.1-1/include/CGAL/random_convex_set_2.h
--- 5.0.3-1/include/CGAL/random_convex_set_2.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/random_convex_set_2.h	2020-09-08 12:14:51.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Generator/include/CGAL/random_convex_set_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Generator/include/CGAL/random_convex_set_2.h $
 // $Id: random_convex_set_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Random_convex_set_traits_2.h 5.1-1/include/CGAL/Random_convex_set_traits_2.h
--- 5.0.3-1/include/CGAL/Random_convex_set_traits_2.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Random_convex_set_traits_2.h	2020-09-08 12:14:51.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Generator/include/CGAL/Random_convex_set_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Generator/include/CGAL/Random_convex_set_traits_2.h $
 // $Id: Random_convex_set_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Random.h 5.1-1/include/CGAL/Random.h
--- 5.0.3-1/include/CGAL/Random.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Random.h	2020-09-08 12:15:31.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Random_numbers/include/CGAL/Random.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Random_numbers/include/CGAL/Random.h $
 // $Id: Random.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Random_impl.h 5.1-1/include/CGAL/Random_impl.h
--- 5.0.3-1/include/CGAL/Random_impl.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Random_impl.h	2020-09-08 12:15:31.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Random_numbers/include/CGAL/Random_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Random_numbers/include/CGAL/Random_impl.h $
 // $Id: Random_impl.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/random_polygon_2.h 5.1-1/include/CGAL/random_polygon_2.h
--- 5.0.3-1/include/CGAL/random_polygon_2.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/random_polygon_2.h	2020-09-08 12:14:51.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Generator/include/CGAL/random_polygon_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Generator/include/CGAL/random_polygon_2.h $
 // $Id: random_polygon_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Random_polygon_2_sweep.h 5.1-1/include/CGAL/Random_polygon_2_sweep.h
--- 5.0.3-1/include/CGAL/Random_polygon_2_sweep.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Random_polygon_2_sweep.h	2020-09-08 12:14:51.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Generator/include/CGAL/Random_polygon_2_sweep.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Generator/include/CGAL/Random_polygon_2_sweep.h $
 // $Id: Random_polygon_2_sweep.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Random_polygon_traits_2.h 5.1-1/include/CGAL/Random_polygon_traits_2.h
--- 5.0.3-1/include/CGAL/Random_polygon_traits_2.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/Random_polygon_traits_2.h	2020-09-08 12:14:51.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Generator/include/CGAL/Random_polygon_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Generator/include/CGAL/Random_polygon_traits_2.h $
 // $Id: Random_polygon_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/random_selection.h 5.1-1/include/CGAL/random_selection.h
--- 5.0.3-1/include/CGAL/random_selection.h	2020-08-01 19:02:21.000000000 +0000
+++ 5.1-1/include/CGAL/random_selection.h	2020-09-08 12:14:51.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Generator/include/CGAL/random_selection.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Generator/include/CGAL/random_selection.h $
 // $Id: random_selection.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/random_simplify_point_set.h 5.1-1/include/CGAL/random_simplify_point_set.h
--- 5.0.3-1/include/CGAL/random_simplify_point_set.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/random_simplify_point_set.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_processing_3/include/CGAL/random_simplify_point_set.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/random_simplify_point_set.h $
 // $Id: random_simplify_point_set.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/range_search_delaunay_2.h 5.1-1/include/CGAL/range_search_delaunay_2.h
--- 5.0.3-1/include/CGAL/range_search_delaunay_2.h	2020-08-01 19:02:39.000000000 +0000
+++ 5.1-1/include/CGAL/range_search_delaunay_2.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_2/include/CGAL/range_search_delaunay_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_2/include/CGAL/range_search_delaunay_2.h $
 // $Id: range_search_delaunay_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Range_segment_tree_traits.h 5.1-1/include/CGAL/Range_segment_tree_traits.h
--- 5.0.3-1/include/CGAL/Range_segment_tree_traits.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Range_segment_tree_traits.h	2020-09-08 12:15:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/SearchStructures/include/CGAL/Range_segment_tree_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/SearchStructures/include/CGAL/Range_segment_tree_traits.h $
 // $Id: Range_segment_tree_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Range_tree_d.h 5.1-1/include/CGAL/Range_tree_d.h
--- 5.0.3-1/include/CGAL/Range_tree_d.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Range_tree_d.h	2020-09-08 12:15:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/SearchStructures/include/CGAL/Range_tree_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/SearchStructures/include/CGAL/Range_tree_d.h $
 // $Id: Range_tree_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Range_tree_k.h 5.1-1/include/CGAL/Range_tree_k.h
--- 5.0.3-1/include/CGAL/Range_tree_k.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Range_tree_k.h	2020-09-08 12:15:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/SearchStructures/include/CGAL/Range_tree_k.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/SearchStructures/include/CGAL/Range_tree_k.h $
 // $Id: Range_tree_k.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/rational_rotation.h 5.1-1/include/CGAL/rational_rotation.h
--- 5.0.3-1/include/CGAL/rational_rotation.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/rational_rotation.h	2020-09-08 12:15:02.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/rational_rotation.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/rational_rotation.h $
 // $Id: rational_rotation.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Rational_traits.h 5.1-1/include/CGAL/Rational_traits.h
--- 5.0.3-1/include/CGAL/Rational_traits.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/Rational_traits.h	2020-09-08 12:14:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_foundations/include/CGAL/Rational_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_foundations/include/CGAL/Rational_traits.h $
 // $Id: Rational_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Ray_2.h 5.1-1/include/CGAL/Ray_2.h
--- 5.0.3-1/include/CGAL/Ray_2.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/Ray_2.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Ray_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Ray_2.h $
 // $Id: Ray_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Ray_3.h 5.1-1/include/CGAL/Ray_3.h
--- 5.0.3-1/include/CGAL/Ray_3.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/Ray_3.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Ray_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Ray_3.h $
 // $Id: Ray_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/read_vtk_image_data.h 5.1-1/include/CGAL/read_vtk_image_data.h
--- 5.0.3-1/include/CGAL/read_vtk_image_data.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/read_vtk_image_data.h	2020-09-08 12:14:44.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/read_vtk_image_data.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/read_vtk_image_data.h $
 // $Id: read_vtk_image_data.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Real_embeddable_traits.h 5.1-1/include/CGAL/Real_embeddable_traits.h
--- 5.0.3-1/include/CGAL/Real_embeddable_traits.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/Real_embeddable_traits.h	2020-09-08 12:14:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_foundations/include/CGAL/Real_embeddable_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_foundations/include/CGAL/Real_embeddable_traits.h $
 // $Id: Real_embeddable_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Real_timer.h 5.1-1/include/CGAL/Real_timer.h
--- 5.0.3-1/include/CGAL/Real_timer.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Real_timer.h	2020-09-08 12:15:30.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Profiling_tools/include/CGAL/Real_timer.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Profiling_tools/include/CGAL/Real_timer.h $
 // $Id: Real_timer.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Real_timer_impl.h 5.1-1/include/CGAL/Real_timer_impl.h
--- 5.0.3-1/include/CGAL/Real_timer_impl.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Real_timer_impl.h	2020-09-08 12:15:30.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Profiling_tools/include/CGAL/Real_timer_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Profiling_tools/include/CGAL/Real_timer_impl.h $
 // $Id: Real_timer_impl.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Reconstruction_triangulation_3.h 5.1-1/include/CGAL/Reconstruction_triangulation_3.h
--- 5.0.3-1/include/CGAL/Reconstruction_triangulation_3.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Reconstruction_triangulation_3.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Poisson_surface_reconstruction_3/include/CGAL/Reconstruction_triangulation_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Poisson_surface_reconstruction_3/include/CGAL/Reconstruction_triangulation_3.h $
 // $Id: Reconstruction_triangulation_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/rectangular_3_center_2.h 5.1-1/include/CGAL/rectangular_3_center_2.h
--- 5.0.3-1/include/CGAL/rectangular_3_center_2.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/rectangular_3_center_2.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/rectangular_3_center_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/rectangular_3_center_2.h $
 // $Id: rectangular_3_center_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/rectangular_p_center_2.h 5.1-1/include/CGAL/rectangular_p_center_2.h
--- 5.0.3-1/include/CGAL/rectangular_p_center_2.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/rectangular_p_center_2.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/rectangular_p_center_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/rectangular_p_center_2.h $
 // $Id: rectangular_p_center_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Rectangular_p_center_traits_2.h 5.1-1/include/CGAL/Rectangular_p_center_traits_2.h
--- 5.0.3-1/include/CGAL/Rectangular_p_center_traits_2.h	2020-08-01 19:02:16.000000000 +0000
+++ 5.1-1/include/CGAL/Rectangular_p_center_traits_2.h	2020-09-08 12:14:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Bounding_volumes/include/CGAL/Rectangular_p_center_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Bounding_volumes/include/CGAL/Rectangular_p_center_traits_2.h $
 // $Id: Rectangular_p_center_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Referenced_argument.h 5.1-1/include/CGAL/Referenced_argument.h
--- 5.0.3-1/include/CGAL/Referenced_argument.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/Referenced_argument.h	2020-09-08 12:15:04.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/Referenced_argument.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/Referenced_argument.h $
 // $Id: Referenced_argument.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/refine_mesh_3.h 5.1-1/include/CGAL/refine_mesh_3.h
--- 5.0.3-1/include/CGAL/refine_mesh_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/refine_mesh_3.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/refine_mesh_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/refine_mesh_3.h $
 // $Id: refine_mesh_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/refine_periodic_3_mesh_3.h 5.1-1/include/CGAL/refine_periodic_3_mesh_3.h
--- 5.0.3-1/include/CGAL/refine_periodic_3_mesh_3.h	2020-08-01 19:02:38.000000000 +0000
+++ 5.1-1/include/CGAL/refine_periodic_3_mesh_3.h	2020-09-08 12:15:20.000000000 +0000
@@ -4,8 +4,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Periodic_3_mesh_3/include/CGAL/refine_periodic_3_mesh_3.h $
-// $Id: refine_periodic_3_mesh_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Periodic_3_mesh_3/include/CGAL/refine_periodic_3_mesh_3.h $
+// $Id: refine_periodic_3_mesh_3.h 53d4c9b 2019-10-28T11:29:08+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Stephane Tayeb,
@@ -24,8 +24,7 @@
 #include <CGAL/Mesh_3/Dump_c3t3.h>
 #include <CGAL/Mesh_3/Triangulation_helpers.h>
 #include <CGAL/refine_mesh_3.h>
-
-#include <CGAL/Hash_handles_with_or_without_timestamps.h>
+#include <CGAL/Time_stamper.h>
 
 #include <boost/parameter/preprocessor.hpp>
 #include <boost/unordered_set.hpp>
diff -pruN 5.0.3-1/include/CGAL/Regular_complex_d.h 5.1-1/include/CGAL/Regular_complex_d.h
--- 5.0.3-1/include/CGAL/Regular_complex_d.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Regular_complex_d.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_d/include/CGAL/Regular_complex_d.h $
-// $Id: Regular_complex_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_d/include/CGAL/Regular_complex_d.h $
+// $Id: Regular_complex_d.h d1a323c 2020-03-26T19:24:14+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -87,7 +87,7 @@ public:
 
   void* pp;
   void*   for_compact_container() const { return pp; }
-  void* & for_compact_container()       { return pp; }
+  void for_compact_container(void *p) { pp = p; }
 
 #ifdef CGAL_USE_LEDA
   LEDA_MEMORY(RC_vertex_d)
@@ -153,7 +153,7 @@ public:
 
   void* pp;
   void*   for_compact_container() const { return pp; }
-  void* & for_compact_container()       { return pp; }
+  void for_compact_container(void *p) { pp = p; }
 
   #if 0
   struct Point_const_iterator {
diff -pruN 5.0.3-1/include/CGAL/Regular_grid_2.h 5.1-1/include/CGAL/Regular_grid_2.h
--- 5.0.3-1/include/CGAL/Regular_grid_2.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Regular_grid_2.h	2020-09-08 12:15:41.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_lines_2/include/CGAL/Regular_grid_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_lines_2/include/CGAL/Regular_grid_2.h $
 // $Id: Regular_grid_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Regularization/regularize_planes.h 5.1-1/include/CGAL/Regularization/regularize_planes.h
--- 5.0.3-1/include/CGAL/Regularization/regularize_planes.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Regularization/regularize_planes.h	2020-09-08 12:15:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Shape_detection/include/CGAL/Regularization/regularize_planes.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Shape_detection/include/CGAL/Regularization/regularize_planes.h $
 // $Id: regularize_planes.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Regularization.h 5.1-1/include/CGAL/Regularization.h
--- 5.0.3-1/include/CGAL/Regularization.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Regularization.h	2020-09-08 12:15:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Shape_detection/include/CGAL/Regularization.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Shape_detection/include/CGAL/Regularization.h $
 // $Id: Regularization.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/regular_neighbor_coordinates_2.h 5.1-1/include/CGAL/regular_neighbor_coordinates_2.h
--- 5.0.3-1/include/CGAL/regular_neighbor_coordinates_2.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/regular_neighbor_coordinates_2.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Interpolation/include/CGAL/regular_neighbor_coordinates_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Interpolation/include/CGAL/regular_neighbor_coordinates_2.h $
 // $Id: regular_neighbor_coordinates_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Regular_triangulation_2.h 5.1-1/include/CGAL/Regular_triangulation_2.h
--- 5.0.3-1/include/CGAL/Regular_triangulation_2.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Regular_triangulation_2.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/Regular_triangulation_2.h $
-// $Id: Regular_triangulation_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/Regular_triangulation_2.h $
+// $Id: Regular_triangulation_2.h 9f2eafd 2020-03-26T19:17:02+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Frederic Fichel, Mariette Yvinec, Julia Floetotto
@@ -20,7 +20,6 @@
 
 #include <CGAL/utility.h>
 #include <CGAL/Object.h>
-#include <CGAL/internal/boost/function_property_map.hpp>
 #include <CGAL/internal/Has_nested_type_Bare_point.h>
 
 #include <boost/bind.hpp>
@@ -34,6 +33,7 @@
 
 #include <boost/iterator/zip_iterator.hpp>
 #include <boost/mpl/and.hpp>
+#include <boost/property_map/function_property_map.hpp>
 
 #endif //CGAL_TRIANGULATION_2_DONT_INSERT_RANGE_OF_POINTS_WITH_INFO
 
@@ -414,12 +414,12 @@ public:
     // spatial sorting must use bare points, so we need an adapter
     typedef typename Geom_traits::Construct_point_2 Construct_point_2;
     typedef typename boost::result_of<const Construct_point_2(const Weighted_point&)>::type Ret;
-    typedef CGAL::internal::boost_::function_property_map<Construct_point_2, Weighted_point, Ret> fpmap;
+    typedef boost::function_property_map<Construct_point_2, Weighted_point, Ret> fpmap;
     typedef CGAL::Spatial_sort_traits_adapter_2<Geom_traits, fpmap> Search_traits_2;
 
     spatial_sort(points.begin(), points.end(),
                  Search_traits_2(
-                   CGAL::internal::boost_::make_function_property_map<Weighted_point, Ret, Construct_point_2>(
+                   boost::make_function_property_map<Weighted_point, Ret, Construct_point_2>(
                      geom_traits().construct_point_2_object()), geom_traits()));
 
     Face_handle hint;
@@ -482,13 +482,13 @@ private:
     typedef Index_to_Bare_point<Construct_point_2,
                                 std::vector<Weighted_point> > Access_bare_point;
     typedef typename boost::result_of<const Construct_point_2(const Weighted_point&)>::type Ret;
-    typedef CGAL::internal::boost_::function_property_map<Access_bare_point, std::size_t, Ret> fpmap;
+    typedef boost::function_property_map<Access_bare_point, std::size_t, Ret> fpmap;
     typedef CGAL::Spatial_sort_traits_adapter_2<Gt, fpmap> Search_traits_2;
 
     Access_bare_point accessor(points, geom_traits().construct_point_2_object());
     spatial_sort(indices.begin(), indices.end(),
                  Search_traits_2(
-                   CGAL::internal::boost_::make_function_property_map<
+                   boost::make_function_property_map<
                      std::size_t, Ret, Access_bare_point>(accessor),
                    geom_traits()));
 
diff -pruN 5.0.3-1/include/CGAL/Regular_triangulation_3.h 5.1-1/include/CGAL/Regular_triangulation_3.h
--- 5.0.3-1/include/CGAL/Regular_triangulation_3.h	2020-08-01 19:02:59.000000000 +0000
+++ 5.1-1/include/CGAL/Regular_triangulation_3.h	2020-09-08 12:15:52.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_3/include/CGAL/Regular_triangulation_3.h $
-// $Id: Regular_triangulation_3.h ad23db2 2020-06-30T16:36:58+02:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_3/include/CGAL/Regular_triangulation_3.h $
+// $Id: Regular_triangulation_3.h 07eacf5 2020-06-30T16:38:05+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -22,13 +22,6 @@
 
 #include <CGAL/basic.h>
 
-#include <set>
-
-#include <boost/bind.hpp>
-#include <boost/mpl/if.hpp>
-#include <boost/mpl/identity.hpp>
-#include <boost/utility/result_of.hpp>
-
 #ifdef CGAL_LINKED_WITH_TBB
 # include <CGAL/point_generators_3.h>
 # include <tbb/parallel_for.h>
@@ -41,7 +34,6 @@
 #include <CGAL/Regular_triangulation_vertex_base_3.h>
 #include <CGAL/Regular_triangulation_cell_base_3.h>
 #include <CGAL/internal/Has_nested_type_Bare_point.h>
-#include <CGAL/internal/boost/function_property_map.hpp>
 
 #include <CGAL/Cartesian_converter.h>
 #include <CGAL/Exact_predicates_exact_constructions_kernel.h>
@@ -63,6 +55,20 @@
 #include <CGAL/point_generators_3.h>
 #endif
 
+#include <boost/bind.hpp>
+#include <boost/mpl/if.hpp>
+#include <boost/mpl/identity.hpp>
+#include <boost/property_map/function_property_map.hpp>
+#include <boost/utility/result_of.hpp>
+
+#include <algorithm>
+#include <iostream>
+#include <iterator>
+#include <set>
+#include <thread>
+#include <utility>
+#include <vector>
+
 namespace CGAL {
 
 /************************************************
@@ -194,21 +200,38 @@ public:
     CGAL_triangulation_postcondition(is_valid());
   }
 
-  void swap(Regular_triangulation_3& tr)
+  Regular_triangulation_3(Regular_triangulation_3&& rt)
+    noexcept(noexcept(Tr_Base(std::move(rt))))
+    : Tr_Base(std::move(rt)), hidden_point_visitor(this)
   {
-    // The 'vertices' and 'hidden_points' members of 'hidden_point_visitor' should be empty
-    // as they are only filled (and cleared) during the insertion of a point.
-    // Hidden points are not stored there, but rather in cells. Thus, the only thing that must be set
-    // is the triangulation pointer.
-    Hidden_point_visitor<Concurrency_tag> new_hpv(this);
-    std::swap(hidden_point_visitor, new_hpv);
+    CGAL_triangulation_postcondition(is_valid());
+  }
 
+  ~Regular_triangulation_3() = default;
+
+  void swap(Regular_triangulation_3& tr)
+  {
+    // The 'vertices' and 'hidden_points' members of
+    // 'hidden_point_visitor' should be empty as they are only filled
+    // (and cleared) during the insertion of a point.  Hidden points
+    // are not stored there, but rather in cells. Thus, the only thing
+    // that must be set is the triangulation pointer, and it is
+    // already correctly set. There is nothing to do about
+    // 'hidden_point_visitor'.
     Tr_Base::swap(tr);
   }
 
-  Regular_triangulation_3& operator=(Regular_triangulation_3 tr)
+  Regular_triangulation_3& operator=(const Regular_triangulation_3& tr)
+  {
+    Regular_triangulation_3 copy(tr);
+    copy.swap(*this);
+    return *this;
+  }
+
+  Regular_triangulation_3& operator=(Regular_triangulation_3&& tr)
+    noexcept(noexcept(Regular_triangulation_3(std::move(tr))))
   {
-    swap(tr);
+    Tr_Base::operator=(std::move(tr));
     return *this;
   }
 
@@ -270,12 +293,12 @@ private:
       // Spatial sorting can only be applied to bare points, so we need an adaptor
       typedef typename Geom_traits::Construct_point_3 Construct_point_3;
       typedef typename boost::result_of<const Construct_point_3(const Weighted_point&)>::type Ret;
-      typedef CGAL::internal::boost_::function_property_map<Construct_point_3, Weighted_point, Ret> fpmap;
+      typedef boost::function_property_map<Construct_point_3, Weighted_point, Ret> fpmap;
       typedef CGAL::Spatial_sort_traits_adapter_3<Geom_traits, fpmap> Search_traits_3;
 
       spatial_sort(points_on_far_sphere.begin(), points_on_far_sphere.end(),
                    Search_traits_3(
-                     CGAL::internal::boost_::make_function_property_map<Weighted_point, Ret, Construct_point_3>(
+                     boost::make_function_property_map<Weighted_point, Ret, Construct_point_3>(
                        geom_traits().construct_point_3_object()), geom_traits()));
 
       typename std::vector<Weighted_point>::const_iterator it_p =
@@ -342,12 +365,12 @@ public:
     // kernel creates temporaries and prevent it.
     typedef typename Geom_traits::Construct_point_3 Construct_point_3;
     typedef typename boost::result_of<const Construct_point_3(const Weighted_point&)>::type Ret;
-    typedef CGAL::internal::boost_::function_property_map<Construct_point_3, Weighted_point, Ret> fpmap;
+    typedef boost::function_property_map<Construct_point_3, Weighted_point, Ret> fpmap;
     typedef CGAL::Spatial_sort_traits_adapter_3<Geom_traits, fpmap> Search_traits_3;
 
     spatial_sort(points.begin(), points.end(),
                  Search_traits_3(
-                   CGAL::internal::boost_::make_function_property_map<Weighted_point, Ret, Construct_point_3>(
+                   boost::make_function_property_map<Weighted_point, Ret, Construct_point_3>(
                      geom_traits().construct_point_3_object()), geom_traits()));
 
     // Parallel
@@ -467,14 +490,14 @@ private:
     typedef Index_to_Bare_point<Construct_point_3,
         std::vector<Weighted_point> > Access_bare_point;
     typedef typename boost::result_of<const Construct_point_3(const Weighted_point&)>::type Ret;
-    typedef CGAL::internal::boost_::function_property_map<Access_bare_point, std::size_t, Ret> fpmap;
+    typedef boost::function_property_map<Access_bare_point, std::size_t, Ret> fpmap;
     typedef CGAL::Spatial_sort_traits_adapter_3<Gt, fpmap> Search_traits_3;
 
     Access_bare_point accessor(points, geom_traits().construct_point_3_object());
     spatial_sort(indices.begin(), indices.end(),
                  Search_traits_3(
-                   CGAL::internal::boost_::make_function_property_map<
-                   std::size_t, Ret, Access_bare_point>(accessor),
+                   boost::make_function_property_map<
+                     std::size_t, Ret, Access_bare_point>(accessor),
                    geom_traits()));
 
 #ifdef CGAL_LINKED_WITH_TBB
@@ -1388,11 +1411,6 @@ protected:
       : m_rt(rt), m_points(points), m_tls_hint(tls_hint)
     {}
 
-    // Constructor
-    Insert_point(const Insert_point& ip)
-      : m_rt(ip.m_rt), m_points(ip.m_points), m_tls_hint(ip.m_tls_hint)
-    {}
-
     // operator()
     void operator()(const tbb::blocked_range<size_t>& r) const
     {
@@ -1499,12 +1517,6 @@ protected:
         m_tls_hint(tls_hint)
     {}
 
-    // Constructor
-    Insert_point_with_info(const Insert_point_with_info &ip)
-      : m_rt(ip.m_rt), m_points(ip.m_points), m_infos(ip.m_infos),
-        m_indices(ip.m_indices), m_tls_hint(ip.m_tls_hint)
-    {}
-
     // operator()
     void operator()(const tbb::blocked_range<size_t>& r) const
     {
@@ -1616,12 +1628,6 @@ protected:
         m_vertices_to_remove_sequentially(vertices_to_remove_sequentially)
     {}
 
-    // Constructor
-    Remove_point(const Remove_point& rp)
-      : m_rt(rp.m_rt), m_vertices(rp.m_vertices),
-        m_vertices_to_remove_sequentially(rp.m_vertices_to_remove_sequentially)
-    {}
-
     // operator()
     void operator()(const tbb::blocked_range<size_t>& r) const
     {
diff -pruN 5.0.3-1/include/CGAL/Regular_triangulation_adaptation_policies_2.h 5.1-1/include/CGAL/Regular_triangulation_adaptation_policies_2.h
--- 5.0.3-1/include/CGAL/Regular_triangulation_adaptation_policies_2.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Regular_triangulation_adaptation_policies_2.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Regular_triangulation_adaptation_policies_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Regular_triangulation_adaptation_policies_2.h $
 // $Id: Regular_triangulation_adaptation_policies_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Regular_triangulation_adaptation_traits_2.h 5.1-1/include/CGAL/Regular_triangulation_adaptation_traits_2.h
--- 5.0.3-1/include/CGAL/Regular_triangulation_adaptation_traits_2.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Regular_triangulation_adaptation_traits_2.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Regular_triangulation_adaptation_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Regular_triangulation_adaptation_traits_2.h $
 // $Id: Regular_triangulation_adaptation_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Regular_triangulation_cell_base_3.h 5.1-1/include/CGAL/Regular_triangulation_cell_base_3.h
--- 5.0.3-1/include/CGAL/Regular_triangulation_cell_base_3.h	2020-08-01 19:02:59.000000000 +0000
+++ 5.1-1/include/CGAL/Regular_triangulation_cell_base_3.h	2020-09-08 12:15:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_3/include/CGAL/Regular_triangulation_cell_base_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_3/include/CGAL/Regular_triangulation_cell_base_3.h $
 // $Id: Regular_triangulation_cell_base_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Regular_triangulation_cell_base_with_weighted_circumcenter_3.h 5.1-1/include/CGAL/Regular_triangulation_cell_base_with_weighted_circumcenter_3.h
--- 5.0.3-1/include/CGAL/Regular_triangulation_cell_base_with_weighted_circumcenter_3.h	2020-08-01 19:02:59.000000000 +0000
+++ 5.1-1/include/CGAL/Regular_triangulation_cell_base_with_weighted_circumcenter_3.h	2020-09-08 12:15:52.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_3/include/CGAL/Regular_triangulation_cell_base_with_weighted_circumcenter_3.h $
-// $Id: Regular_triangulation_cell_base_with_weighted_circumcenter_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_3/include/CGAL/Regular_triangulation_cell_base_with_weighted_circumcenter_3.h $
+// $Id: Regular_triangulation_cell_base_with_weighted_circumcenter_3.h dbbee66 2020-04-02T18:31:00+05:30 rathod-sahaab
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Monique Teillaud <Monique.Teillaud@sophia.inria.fr>
@@ -132,7 +132,7 @@ public:
       return *weighted_circumcenter_;
   }
 
-  void swap (Regular_triangulation_cell_base_with_weighted_circumcenter_3& other) throw()
+  void swap (Regular_triangulation_cell_base_with_weighted_circumcenter_3& other) noexcept
   {
     std::swap(static_cast<Cb&>(*this), static_cast<Cb&>(other));
     std::swap(weighted_circumcenter_, other.weighted_circumcenter_);
diff -pruN 5.0.3-1/include/CGAL/Regular_triangulation_euclidean_traits_3.h 5.1-1/include/CGAL/Regular_triangulation_euclidean_traits_3.h
--- 5.0.3-1/include/CGAL/Regular_triangulation_euclidean_traits_3.h	2020-08-01 19:02:59.000000000 +0000
+++ 5.1-1/include/CGAL/Regular_triangulation_euclidean_traits_3.h	2020-09-08 12:15:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_3/include/CGAL/Regular_triangulation_euclidean_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_3/include/CGAL/Regular_triangulation_euclidean_traits_3.h $
 // $Id: Regular_triangulation_euclidean_traits_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Regular_triangulation_face_base_2.h 5.1-1/include/CGAL/Regular_triangulation_face_base_2.h
--- 5.0.3-1/include/CGAL/Regular_triangulation_face_base_2.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Regular_triangulation_face_base_2.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/Regular_triangulation_face_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/Regular_triangulation_face_base_2.h $
 // $Id: Regular_triangulation_face_base_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Regular_triangulation.h 5.1-1/include/CGAL/Regular_triangulation.h
--- 5.0.3-1/include/CGAL/Regular_triangulation.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Regular_triangulation.h	2020-09-08 12:15:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation/include/CGAL/Regular_triangulation.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation/include/CGAL/Regular_triangulation.h $
 // $Id: Regular_triangulation.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Regular_triangulation_traits_adapter.h 5.1-1/include/CGAL/Regular_triangulation_traits_adapter.h
--- 5.0.3-1/include/CGAL/Regular_triangulation_traits_adapter.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Regular_triangulation_traits_adapter.h	2020-09-08 12:15:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation/include/CGAL/Regular_triangulation_traits_adapter.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation/include/CGAL/Regular_triangulation_traits_adapter.h $
 // $Id: Regular_triangulation_traits_adapter.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Regular_triangulation_vertex_base_2.h 5.1-1/include/CGAL/Regular_triangulation_vertex_base_2.h
--- 5.0.3-1/include/CGAL/Regular_triangulation_vertex_base_2.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Regular_triangulation_vertex_base_2.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/Regular_triangulation_vertex_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/Regular_triangulation_vertex_base_2.h $
 // $Id: Regular_triangulation_vertex_base_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Regular_triangulation_vertex_base_3.h 5.1-1/include/CGAL/Regular_triangulation_vertex_base_3.h
--- 5.0.3-1/include/CGAL/Regular_triangulation_vertex_base_3.h	2020-08-01 19:02:59.000000000 +0000
+++ 5.1-1/include/CGAL/Regular_triangulation_vertex_base_3.h	2020-09-08 12:15:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_3/include/CGAL/Regular_triangulation_vertex_base_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_3/include/CGAL/Regular_triangulation_vertex_base_3.h $
 // $Id: Regular_triangulation_vertex_base_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/remove_far_points_in_mesh_3.h 5.1-1/include/CGAL/remove_far_points_in_mesh_3.h
--- 5.0.3-1/include/CGAL/remove_far_points_in_mesh_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/remove_far_points_in_mesh_3.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/remove_far_points_in_mesh_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/remove_far_points_in_mesh_3.h $
 // $Id: remove_far_points_in_mesh_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/remove_outliers.h 5.1-1/include/CGAL/remove_outliers.h
--- 5.0.3-1/include/CGAL/remove_outliers.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/remove_outliers.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_processing_3/include/CGAL/remove_outliers.h $
-// $Id: remove_outliers.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/remove_outliers.h $
+// $Id: remove_outliers.h 0715654 2020-06-03T19:01:46+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s) : Laurent Saboret and Nader Salman and Pierre Alliez
@@ -16,9 +16,9 @@
 
 #include <CGAL/disable_warnings.h>
 
-#include <CGAL/Search_traits_3.h>
-#include <CGAL/Orthogonal_k_neighbor_search.h>
-#include <CGAL/Point_set_processing_3/internal/neighbor_query.h>
+#include <CGAL/Point_set_processing_3/internal/Neighbor_query.h>
+#include <CGAL/Point_set_processing_3/internal/Callback_wrapper.h>
+#include <CGAL/for_each.h>
 #include <CGAL/property_map.h>
 #include <CGAL/point_set_processing_assertions.h>
 #include <functional>
@@ -26,6 +26,8 @@
 #include <CGAL/boost/graph/Named_function_parameters.h>
 #include <CGAL/boost/graph/named_params_helper.h>
 
+#include <boost/iterator/zip_iterator.hpp>
+
 #include <iterator>
 #include <algorithm>
 #include <map>
@@ -49,22 +51,21 @@ namespace internal {
 /// @tparam Tree KD-tree.
 ///
 /// @return computed distance.
-template < typename Kernel,
-           typename Tree >
-typename Kernel::FT
+template <typename NeighborQuery>
+typename NeighborQuery::Kernel::FT
 compute_avg_knn_sq_distance_3(
-    const typename Kernel::Point_3& query, ///< 3D point to project
-    Tree& tree,                            ///< KD-tree
+  const typename NeighborQuery::Kernel::Point_3& query, ///< 3D point to project
+    NeighborQuery& neighbor_query,                            ///< KD-tree
     unsigned int k,                        ///< number of neighbors
-    typename Kernel::FT neighbor_radius)
+    typename NeighborQuery::Kernel::FT neighbor_radius)
 {
     // geometric types
+    typedef typename NeighborQuery::Kernel Kernel;
     typedef typename Kernel::FT FT;
     typedef typename Kernel::Point_3 Point;
 
     std::vector<Point> points;
-    CGAL::Point_set_processing_3::internal::neighbor_query
-      (query, tree, k, neighbor_radius, points);
+    neighbor_query.get_points (query, k, neighbor_radius, std::back_inserter(points));
 
     // compute average squared distance
     typename Kernel::Compute_squared_distance_3 sqd;
@@ -87,7 +88,9 @@ compute_avg_knn_sq_distance_3(
    \ingroup PkgPointSetProcessing3Algorithms
    Removes outliers:
    - computes average squared distance to the nearest neighbors,
-   - and sorts the points in increasing order of average distance.
+   - and partitions the points either using a threshold on the of
+     average distance or selecting a fixed percentage of points with
+     the highest average distances
 
    This method modifies the order of input points so as to pack all remaining points first,
    and returns an iterator over the first point to remove (see erase-remove idiom).
@@ -95,35 +98,65 @@ compute_avg_knn_sq_distance_3(
 
    \pre `k >= 2`
 
+   \tparam ConcurrencyTag enables sequential versus parallel algorithm. Possible values are `Sequential_tag`,
+                          `Parallel_tag`, and `Parallel_if_available_tag`.
    \tparam PointRange is a model of `Range`. The value type of
    its iterator is the key type of the named parameter `point_map`.
 
    \param points input point range.
    \param k number of neighbors
-   \param np optional sequence of \ref psp_namedparameters "Named Parameters" among the ones listed below.
+   \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
    \cgalNamedParamsBegin
-     \cgalParamBegin{point_map} a model of `ReadablePropertyMap` with value type `geom_traits::Point_3`.
-     If this parameter is omitted, `CGAL::Identity_property_map<geom_traits::Point_3>` is used.\cgalParamEnd
-     \cgalParamBegin{neighbor_radius} spherical neighborhood
-     radius. If provided, the neighborhood of a query point is
-     computed with a fixed spherical radius instead of a fixed number
-     of neighbors. In that case, the parameter `k` is used as a limit
-     on the number of points returned by each spherical query (to
-     avoid overly large number of points in high density areas). If no
-     limit is wanted, use `k=0`.\cgalParamEnd
-     \cgalParamBegin{threshold_percent} maximum percentage of points to remove.\cgalParamEnd
-     \cgalParamBegin{threshold_distance} minimum distance for a point to be considered as outlier
-     (distance here is the square root of the average squared distance to K nearest neighbors).\cgalParamEnd
-     \cgalParamBegin{callback} an instance of
-      `std::function<bool(double)>`. It is called regularly when the
-      algorithm is running: the current advancement (between 0. and
-      1.) is passed as parameter. If it returns `true`, then the
-      algorithm continues its execution normally; if it returns
-      `false`, the algorithm is stopped, all points are left unchanged
-      and the function return `points.end()`.\cgalParamEnd
-     \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
-   \cgalNamedParamsEnd
+     \cgalParamNBegin{point_map}
+       \cgalParamDescription{a property map associating points to the elements of the point set `points`}
+       \cgalParamType{a model of `ReadablePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange` and whose value type is `geom_traits::Point_3`}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{neighbor_radius}
+       \cgalParamDescription{the spherical neighborhood radius}
+       \cgalParamType{floating scalar value}
+       \cgalParamDefault{`0` (no limit)}
+       \cgalParamExtra{If provided, the neighborhood of a query point is computed with a fixed spherical
+                       radius instead of a fixed number of neighbors. In that case, the parameter
+                       `k` is used as a limit on the number of points returned by each spherical
+                       query (to avoid overly large number of points in high density areas).}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{threshold_percent}
+       \cgalParamDescription{the maximum percentage of points to remove}
+       \cgalParamType{double}
+       \cgalParamDefault{`10`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{threshold_distance}
+       \cgalParamDescription{the minimum distance for a point to be considered as outlier}
+       \cgalParamType{double}
+       \cgalParamDefault{`0`}
+       \cgalParamExtra{Distance here is the square root of the average squared distance to K-nearest neighbors}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{callback}
+       \cgalParamDescription{a mechanism to get feedback on the advancement of the algorithm
+                             while it's running and to interrupt it if needed}
+       \cgalParamType{an instance of `std::function<bool(double)>`.}
+       \cgalParamDefault{unused}
+       \cgalParamExtra{It is called regularly when the
+                       algorithm is running: the current advancement (between 0. and 1.)
+                       is passed as parameter. If it returns `true`, then the
+                       algorithm continues its execution normally; if it returns
+                       `false`, the algorithm is stopped, all points are left unchanged
+                       and the function return `points.size()`.}
+       \cgalParamExtra{The callback will be copied and therefore needs to be lightweight.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{geom_traits}
+       \cgalParamDescription{an instance of a geometric traits class}
+       \cgalParamType{a model of `Kernel`}
+       \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+     \cgalParamNEnd\cgalNamedParamsEnd
 
    \return iterator over the first point to remove.
 
@@ -135,7 +168,8 @@ compute_avg_knn_sq_distance_3(
    account; if `threshold_distance=0` only `threshold_percent` is
    taken into account.
 */
-template <typename PointRange,
+template <typename ConcurrencyTag,
+          typename PointRange,
           typename NamedParameters
 >
 typename PointRange::iterator
@@ -148,29 +182,25 @@ remove_outliers(
   using parameters::get_parameter;
 
   // geometric types
-  typedef typename Point_set_processing_3::GetPointMap<PointRange, NamedParameters>::type PointMap;
+  typedef typename CGAL::GetPointMap<PointRange, NamedParameters>::type PointMap;
   typedef typename Point_set_processing_3::GetK<PointRange, NamedParameters>::Kernel Kernel;
 
-  PointMap point_map = choose_parameter(get_parameter(np, internal_np::point_map), PointMap());
+  PointMap point_map = choose_parameter<PointMap>(get_parameter(np, internal_np::point_map));
   typename Kernel::FT neighbor_radius = choose_parameter(get_parameter(np, internal_np::neighbor_radius),
                                                          typename Kernel::FT(0));
   double threshold_percent = choose_parameter(get_parameter(np, internal_np::threshold_percent), 10.);
   double threshold_distance = choose_parameter(get_parameter(np, internal_np::threshold_distance), 0.);
   const std::function<bool(double)>& callback = choose_parameter(get_parameter(np, internal_np::callback),
-                                                               std::function<bool(double)>());
+                                                                 std::function<bool(double)>());
 
   typedef typename Kernel::FT FT;
 
   // basic geometric types
-  typedef typename Kernel::Point_3 Point;
-
-  // actual type of input points
-  typedef typename std::iterator_traits<typename PointRange::iterator>::value_type Enriched_point;
+  typedef typename PointRange::iterator iterator;
+  typedef typename iterator::value_type value_type;
 
   // types for K nearest neighbors search structure
-  typedef typename CGAL::Search_traits_3<Kernel> Tree_traits;
-  typedef typename CGAL::Orthogonal_k_neighbor_search<Tree_traits> Neighbor_search;
-  typedef typename Neighbor_search::Tree Tree;
+  typedef Point_set_processing_3::internal::Neighbor_query<Kernel, PointRange&, PointMap> Neighbor_query;
 
   // precondition: at least one element in the container.
   // to fix: should have at least three distinct points
@@ -182,57 +212,81 @@ remove_outliers(
 
   CGAL_point_set_processing_precondition(threshold_percent >= 0 && threshold_percent <= 100);
 
-  typename PointRange::iterator it;
+  Neighbor_query neighbor_query (points, point_map);
 
-  // Instanciate a KD-tree search.
-  // Note: We have to convert each input iterator to Point_3.
-  std::vector<Point> kd_tree_points;
-  for(it = points.begin(); it != points.end(); it++)
-    kd_tree_points.push_back( get(point_map, *it) );
-  Tree tree(kd_tree_points.begin(), kd_tree_points.end());
+  std::size_t nb_points = points.size();
 
   // iterate over input points and add them to multimap sorted by distance to k
-  std::multimap<FT,Enriched_point> sorted_points;
-  std::size_t nb = 0;
-  for(it = points.begin(); it != points.end(); it++, ++ nb)
+  std::vector<std::pair<FT, value_type> > sorted_points;
+  sorted_points.reserve (nb_points);
+  for (const value_type& p : points)
+    sorted_points.push_back(std::make_pair (FT(0), p));
+
+  Point_set_processing_3::internal::Callback_wrapper<ConcurrencyTag>
+    callback_wrapper (callback, nb_points);
+
+  CGAL::for_each<ConcurrencyTag>
+    (sorted_points,
+     [&](std::pair<FT, value_type>& p) -> bool
+     {
+       if (callback_wrapper.interrupted())
+         return false;
+
+       p.first = internal::compute_avg_knn_sq_distance_3(
+         get(point_map, p.second),
+         neighbor_query, k, neighbor_radius);
+
+       ++ callback_wrapper.advancement();
+       return true;
+     });
+
+  std::size_t first_index_to_remove = std::size_t(double(sorted_points.size()) * ((100.0-threshold_percent)/100.0));
+
+  typename std::vector<std::pair<FT, value_type> >::iterator f2r
+    = sorted_points.begin();
+
+  if (threshold_distance != FT(0))
+    f2r = std::partition (sorted_points.begin(), sorted_points.end(),
+                          [&threshold_distance](const std::pair<FT, value_type>& p) -> bool
+                          {
+                            return p.first < threshold_distance * threshold_distance;
+                          });
+
+  if (static_cast<std::size_t>(std::distance (sorted_points.begin(), f2r)) < first_index_to_remove)
   {
-    FT sq_distance = internal::compute_avg_knn_sq_distance_3<Kernel>(
-      get(point_map,*it),
-      tree, k, neighbor_radius);
-    sorted_points.insert( std::make_pair(sq_distance, *it) );
-    if (callback && !callback ((nb+1) / double(kd_tree_points.size())))
-      return points.end();
+    std::nth_element (f2r,
+                      sorted_points.begin() + first_index_to_remove,
+                      sorted_points.end());
+    f2r = sorted_points.begin() + first_index_to_remove;
   }
 
-  // Replaces [points.begin(), points.end()) range by the multimap content.
-  // Returns the iterator after the (100-threshold_percent) % best points.
-  typename PointRange::iterator first_point_to_remove = points.begin();
-  typename PointRange::iterator dst = points.begin();
-  int first_index_to_remove = int(double(sorted_points.size()) * ((100.0-threshold_percent)/100.0));
-  typename std::multimap<FT,Enriched_point>::iterator src;
-  int index;
-  for (src = sorted_points.begin(), index = 0;
-       src != sorted_points.end();
-       ++src, ++index)
+  // Replaces [points.begin(), points.end()) range by the sorted content.
+  iterator pit = points.begin();
+  iterator out = points.begin();
+
+  for (auto sit = sorted_points.begin(); sit != sorted_points.end(); ++ sit)
   {
-    *dst++ = src->second;
-    if (index <= first_index_to_remove ||
-        src->first < threshold_distance * threshold_distance)
-      first_point_to_remove = dst;
+    *pit = sit->second;
+    if (sit == f2r)
+      out = pit;
+    ++ pit;
   }
 
-  return first_point_to_remove;
+  callback_wrapper.join();
+
+  // Returns the iterator on the first point to remove
+  return out;
 }
 
 /// \cond SKIP_IN_MANUAL
 // variant with default NP
-template <typename PointRange>
+template <typename ConcurrencyTag, typename PointRange>
 typename PointRange::iterator
 remove_outliers(
   PointRange& points,
   unsigned int k) ///< number of neighbors.
 {
-  return remove_outliers (points, k, CGAL::Point_set_processing_3::parameters::all_default(points));
+  return remove_outliers<ConcurrencyTag> (points, k, CGAL::Point_set_processing_3::parameters::all_default(points));
 }
 /// \endcond
 
diff -pruN 5.0.3-1/include/CGAL/representation_tags.h 5.1-1/include/CGAL/representation_tags.h
--- 5.0.3-1/include/CGAL/representation_tags.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/representation_tags.h	2020-09-08 12:15:02.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/representation_tags.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/representation_tags.h $
 // $Id: representation_tags.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Residue.h 5.1-1/include/CGAL/Residue.h
--- 5.0.3-1/include/CGAL/Residue.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/Residue.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Modular_arithmetic/include/CGAL/Residue.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Modular_arithmetic/include/CGAL/Residue.h $
 // $Id: Residue.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/result_of.h 5.1-1/include/CGAL/result_of.h
--- 5.0.3-1/include/CGAL/result_of.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/result_of.h	2020-09-08 12:15:32.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/result_of.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/result_of.h $
 // $Id: result_of.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Ridges.h 5.1-1/include/CGAL/Ridges.h
--- 5.0.3-1/include/CGAL/Ridges.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Ridges.h	2020-09-08 12:15:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Ridges_3/include/CGAL/Ridges.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Ridges_3/include/CGAL/Ridges.h $
 // $Id: Ridges.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Rigid_triangle_mesh_collision_detection.h 5.1-1/include/CGAL/Rigid_triangle_mesh_collision_detection.h
--- 5.0.3-1/include/CGAL/Rigid_triangle_mesh_collision_detection.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Rigid_triangle_mesh_collision_detection.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Rigid_triangle_mesh_collision_detection.h $
-// $Id: Rigid_triangle_mesh_collision_detection.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Rigid_triangle_mesh_collision_detection.h $
+// $Id: Rigid_triangle_mesh_collision_detection.h 48bd92b 2020-04-13T13:03:05+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -188,45 +188,50 @@ public:
  /*!
   * adds mesh `tm` to the set of meshes to be considered for intersection.
   *
-  * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+  * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
   *
   * \return the id of `tm` used to refer to that mesh.
   *
   * @param tm triangulated surface mesh to add
-  * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+  * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
   *
   * \cgalNamedParamsBegin
-  *   \cgalParamBegin{vertex_point_map}
-  *     the property map with the points associated to the vertices of `tm`.
-  *     If this parameter is omitted, an internal property map for
-  *     `CGAL::vertex_point_t` must be available in `TriangleMesh`
-  *   \cgalParamEnd
-  *   \cgalParamBegin{face_index_map}
-  *      a property map containing the index of each face of `tm`. It must be initialized
-  *      and the value must be unique per face and in the range `[0, num_faces(tm)[`.
-  *   \cgalParamEnd
-  *   \cgalParamBegin{apply_per_connected_component}
-  *     if `false`, `tm` is assumed to have only one connected component, avoiding
-  *     the extraction of connected components. Default is `true`.
-  *   \cgalParamEnd
+  *   \cgalParamNBegin{vertex_point_map}
+  *     \cgalParamDescription{a property map associating points to the vertices of `tm`}
+  *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor` as key type and `%Point_3` as value type}
+  *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+  *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+  *                     should be available for the vertices of `tm`.}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{face_index_map}
+  *     \cgalParamDescription{a property map associating to each face of `tm` a unique index between `0` and `num_faces(tm) - 1`}
+  *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%face_descriptor` as key type and `std::size_t` as value type}
+  *     \cgalParamDefault{an automatically indexed internal map}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{apply_per_connected_component}
+  *     \cgalParamDescription{If `false`, `tm` is assumed to have only one connected component, avoiding the extraction of connected components.}
+  *     \cgalParamType{Boolean}
+  *     \cgalParamDefault{`true`}
+  *   \cgalParamNEnd
   * \cgalNamedParamsEnd
   */
   template <class NamedParameters>
   std::size_t add_mesh(const TriangleMesh& tm,
                        const NamedParameters& np)
   {
-  // handle vpm
-    using Polygon_mesh_processing::GetVertexPointMap;
-    typedef typename GetVertexPointMap<TriangleMesh,
-                                       NamedParameters>::const_type Local_vpm;
+    // handle vpm
+    typedef typename CGAL::GetVertexPointMap<TriangleMesh, NamedParameters>::const_type Local_vpm;
     CGAL_USE_TYPE(Local_vpm);
+
     CGAL_assertion_code(
       static const bool same_vpm = (boost::is_same<Local_vpm,Vpm>::value); )
     CGAL_static_assertion(same_vpm);
 
     Vpm vpm =
       parameters::choose_parameter(parameters::get_parameter(np, internal_np::vertex_point),
-                          get_const_property_map(boost::vertex_point, tm) );
+                                   get_const_property_map(boost::vertex_point, tm) );
   // now add the mesh
     std::size_t id = get_id_for_new_mesh();
     CGAL_assertion( m_aabb_trees[id] == nullptr );
@@ -245,28 +250,34 @@ public:
   * adds an instance of a triangulated surface mesh using an external tree of its faces.
   * \warning The tree is not copied and the lifetime of `tree` must be longer than that of this class.
   *
-  * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+  * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
   *
   * \return the id of `tm` used to refer to that mesh.
   *
   * @param tree an AABB-tree of faces of a mesh
   * @param tm triangulated surface mesh
-  * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+  * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
   *
   * \cgalNamedParamsBegin
-  *   \cgalParamBegin{vertex_point_map}
-  *     the property map with the points associated to the vertices of `tm`.
-  *     If this parameter is omitted, an internal property map for
-  *     `CGAL::vertex_point_t` must be available in `TriangleMesh`
-  *   \cgalParamEnd
-  *   \cgalParamBegin{face_index_map}
-  *      a property map containing the index of each face of `tm`. It must be initialized
-  *      and the value must be unique per face and in the range `[0, num_faces(tm)[`.
-  *   \cgalParamEnd
-  *   \cgalParamBegin{apply_per_connected_component}
-  *     if `false`, `tm` is assumed to have only one connected component, avoiding
-  *     the extraction of connected components. Default is `true`.
-  *   \cgalParamEnd
+  *   \cgalParamNBegin{vertex_point_map}
+  *     \cgalParamDescription{a property map associating points to the vertices of `tm`}
+  *     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor` as key type and `%Point_3` as value type}
+  *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+  *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+  *                     should be available for the vertices of `tm`.}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{face_index_map}
+  *     \cgalParamDescription{a property map associating to each face of `tm` a unique index between `0` and `num_faces(tm) - 1`}
+  *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%face_descriptor` as key type and `std::size_t` as value type}
+  *     \cgalParamDefault{an automatically indexed internal map}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{apply_per_connected_component}
+  *     \cgalParamDescription{If `false`, `tm` is assumed to have only one connected component, avoiding the extraction of connected components}
+  *     \cgalParamType{Boolean}
+  *     \cgalParamDefault{`true`}
+  *   \cgalParamNEnd
   * \cgalNamedParamsEnd
   */
   template <class NamedParameters>
@@ -488,26 +499,32 @@ public:
   * intended to be used before calling the `add_mesh()` overload taking an AABB-tree instead of a mesh
   * as input parameter.
   *
-  * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+  * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
   *
   * @param tm input triangulated surface mesh
   * @param [out] points will contain one point per connected component of `tm`
-  * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+  * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
   *
   * \cgalNamedParamsBegin
-  *   \cgalParamBegin{vertex_point_map}
-  *     the property map with the points associated to the vertices of `tm`.
-  *     If this parameter is omitted, an internal property map for
-  *     `CGAL::vertex_point_t` must be available in `TriangleMesh`
-  *   \cgalParamEnd
-  *   \cgalParamBegin{face_index_map}
-  *      a property map containing the index of each face of `tm`. It must be initialized
-  *      and the value must be unique per face and in the range `[0, num_faces(tm)[`.
-  *   \cgalParamEnd
-  *   \cgalParamBegin{apply_per_connected_component}
-  *     if `false`, `tm` is assumed to have only one connected component, avoiding
-  *     the extraction of connected components. %Default is `true`.
-  *   \cgalParamEnd
+  *   \cgalParamNBegin{vertex_point_map}
+  *     \cgalParamDescription{a property map associating points to the vertices of `tm`}
+  *     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor` as key type and `%Point_3` as value type}
+  *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+  *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+  *                     should be available for the vertices of `tm`.}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{face_index_map}
+  *     \cgalParamDescription{a property map associating to each face of `tm` a unique index between `0` and `num_faces(tm) - 1`}
+  *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<PolygonMesh>::%face_descriptor` as key type and `std::size_t` as value type}
+  *     \cgalParamDefault{an automatically indexed internal map}
+  *   \cgalParamNEnd
+  *
+  *   \cgalParamNBegin{apply_per_connected_component}
+  *     \cgalParamDescription{If `false`, `tm` is assumed to have only one connected component, avoiding the extraction of connected components}
+  *     \cgalParamType{Boolean}
+  *     \cgalParamDefault{`true`}
+  *   \cgalParamNEnd
   * \cgalNamedParamsEnd
   */
   template <class NamedParameters>
@@ -517,15 +534,11 @@ public:
           std::vector<Point_3>& points,
     const NamedParameters& np)
   {
-    using Polygon_mesh_processing::GetVertexPointMap;
-    using Polygon_mesh_processing::GetFaceIndexMap;
-
     const bool maybe_several_cc =
       parameters::choose_parameter(
         parameters::get_parameter(np, internal_np::apply_per_connected_component), true);
 
-    typedef typename GetVertexPointMap<TriangleMesh,
-                                       NamedParameters>::const_type Local_vpm;
+    typedef typename CGAL::GetVertexPointMap<TriangleMesh, NamedParameters>::const_type Local_vpm;
     CGAL_USE_TYPE(Local_vpm);
 
     CGAL_assertion_code(
@@ -534,7 +547,7 @@ public:
 
     Vpm vpm =
       parameters::choose_parameter(parameters::get_parameter(np, internal_np::vertex_point),
-                          get_const_property_map(boost::vertex_point, tm) );
+                                   get_const_property_map(boost::vertex_point, tm) );
 
     if (maybe_several_cc)
     {
@@ -542,12 +555,8 @@ public:
       std::vector<std::size_t> cc_ids(num_faces(tm));
 
       // face index map
-      typedef typename GetFaceIndexMap<TriangleMesh,
-                                       NamedParameters>::type Fid_map;
-
-      Fid_map fid_map =
-        parameters::choose_parameter(parameters::get_parameter(np, internal_np::face_index),
-                            get_const_property_map(boost::face_index, tm));
+      typedef typename GetInitializedFaceIndexMap<TriangleMesh, NamedParameters>::const_type FaceIndexMap;
+      FaceIndexMap fid_map = CGAL::get_initialized_face_index_map(tm, np);
 
       std::size_t nb_cc =
         Polygon_mesh_processing::connected_components(
diff -pruN 5.0.3-1/include/CGAL/Robust_circumcenter_filtered_traits_3.h 5.1-1/include/CGAL/Robust_circumcenter_filtered_traits_3.h
--- 5.0.3-1/include/CGAL/Robust_circumcenter_filtered_traits_3.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Robust_circumcenter_filtered_traits_3.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Poisson_surface_reconstruction_3/include/CGAL/Robust_circumcenter_filtered_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Poisson_surface_reconstruction_3/include/CGAL/Robust_circumcenter_filtered_traits_3.h $
 // $Id: Robust_circumcenter_filtered_traits_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Robust_circumcenter_traits_3.h 5.1-1/include/CGAL/Robust_circumcenter_traits_3.h
--- 5.0.3-1/include/CGAL/Robust_circumcenter_traits_3.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Robust_circumcenter_traits_3.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Robust_circumcenter_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Robust_circumcenter_traits_3.h $
 // $Id: Robust_circumcenter_traits_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Robust_construction.h 5.1-1/include/CGAL/Robust_construction.h
--- 5.0.3-1/include/CGAL/Robust_construction.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Robust_construction.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/include/CGAL/Robust_construction.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/Robust_construction.h $
 // $Id: Robust_construction.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Robust_weighted_circumcenter_filtered_traits_3.h 5.1-1/include/CGAL/Robust_weighted_circumcenter_filtered_traits_3.h
--- 5.0.3-1/include/CGAL/Robust_weighted_circumcenter_filtered_traits_3.h	2020-08-01 19:02:59.000000000 +0000
+++ 5.1-1/include/CGAL/Robust_weighted_circumcenter_filtered_traits_3.h	2020-09-08 12:15:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_3/include/CGAL/Robust_weighted_circumcenter_filtered_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_3/include/CGAL/Robust_weighted_circumcenter_filtered_traits_3.h $
 // $Id: Robust_weighted_circumcenter_filtered_traits_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Root_for_circles_2_2.h 5.1-1/include/CGAL/Root_for_circles_2_2.h
--- 5.0.3-1/include/CGAL/Root_for_circles_2_2.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Root_for_circles_2_2.h	2020-09-08 12:14:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_for_circles/include/CGAL/Root_for_circles_2_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_for_circles/include/CGAL/Root_for_circles_2_2.h $
 // $Id: Root_for_circles_2_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Root_for_spheres_2_3.h 5.1-1/include/CGAL/Root_for_spheres_2_3.h
--- 5.0.3-1/include/CGAL/Root_for_spheres_2_3.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Root_for_spheres_2_3.h	2020-09-08 12:14:32.000000000 +0000
@@ -9,7 +9,7 @@
 // and a STREP (FET Open) Project under Contract No  IST-006413
 // (ACS -- Algorithms for Complex Shapes)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_for_spheres/include/CGAL/Root_for_spheres_2_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_for_spheres/include/CGAL/Root_for_spheres_2_3.h $
 // $Id: Root_for_spheres_2_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Root_of_traits.h 5.1-1/include/CGAL/Root_of_traits.h
--- 5.0.3-1/include/CGAL/Root_of_traits.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Root_of_traits.h	2020-09-08 12:15:16.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Root_of_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Root_of_traits.h $
 // $Id: Root_of_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Root_of_traits_specializations.h 5.1-1/include/CGAL/Root_of_traits_specializations.h
--- 5.0.3-1/include/CGAL/Root_of_traits_specializations.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Root_of_traits_specializations.h	2020-09-08 12:15:16.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Root_of_traits_specializations.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Root_of_traits_specializations.h $
 // $Id: Root_of_traits_specializations.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Rotational_sweep_visibility_2.h 5.1-1/include/CGAL/Rotational_sweep_visibility_2.h
--- 5.0.3-1/include/CGAL/Rotational_sweep_visibility_2.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Rotational_sweep_visibility_2.h	2020-09-08 12:15:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Visibility_2/include/CGAL/Rotational_sweep_visibility_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Visibility_2/include/CGAL/Rotational_sweep_visibility_2.h $
 // $Id: Rotational_sweep_visibility_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/RS/ak_1.h 5.1-1/include/CGAL/RS/ak_1.h
--- 5.0.3-1/include/CGAL/RS/ak_1.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/RS/ak_1.h	2020-09-08 12:14:31.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/RS/ak_1.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/RS/ak_1.h $
 // $Id: ak_1.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/RS/ak_z_1.h 5.1-1/include/CGAL/RS/ak_z_1.h
--- 5.0.3-1/include/CGAL/RS/ak_z_1.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/RS/ak_z_1.h	2020-09-08 12:14:31.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/RS/ak_z_1.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/RS/ak_z_1.h $
 // $Id: ak_z_1.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/RS/algebraic_1.h 5.1-1/include/CGAL/RS/algebraic_1.h
--- 5.0.3-1/include/CGAL/RS/algebraic_1.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/RS/algebraic_1.h	2020-09-08 12:14:31.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/RS/algebraic_1.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/RS/algebraic_1.h $
 // $Id: algebraic_1.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/RS/algebraic_z_1.h 5.1-1/include/CGAL/RS/algebraic_z_1.h
--- 5.0.3-1/include/CGAL/RS/algebraic_z_1.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/RS/algebraic_z_1.h	2020-09-08 12:14:31.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/RS/algebraic_z_1.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/RS/algebraic_z_1.h $
 // $Id: algebraic_z_1.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/RS/bisection_refiner_1.h 5.1-1/include/CGAL/RS/bisection_refiner_1.h
--- 5.0.3-1/include/CGAL/RS/bisection_refiner_1.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/RS/bisection_refiner_1.h	2020-09-08 12:14:31.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/RS/bisection_refiner_1.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/RS/bisection_refiner_1.h $
 // $Id: bisection_refiner_1.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/RS/comparator_1.h 5.1-1/include/CGAL/RS/comparator_1.h
--- 5.0.3-1/include/CGAL/RS/comparator_1.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/RS/comparator_1.h	2020-09-08 12:14:31.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/RS/comparator_1.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/RS/comparator_1.h $
 // $Id: comparator_1.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/RS/dyadic.h 5.1-1/include/CGAL/RS/dyadic.h
--- 5.0.3-1/include/CGAL/RS/dyadic.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/RS/dyadic.h	2020-09-08 12:14:31.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/RS/dyadic.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/RS/dyadic.h $
 // $Id: dyadic.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/RS/exact_signat_1.h 5.1-1/include/CGAL/RS/exact_signat_1.h
--- 5.0.3-1/include/CGAL/RS/exact_signat_1.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/RS/exact_signat_1.h	2020-09-08 12:14:31.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/RS/exact_signat_1.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/RS/exact_signat_1.h $
 // $Id: exact_signat_1.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/RS/functors_1.h 5.1-1/include/CGAL/RS/functors_1.h
--- 5.0.3-1/include/CGAL/RS/functors_1.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/RS/functors_1.h	2020-09-08 12:14:31.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/RS/functors_1.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/RS/functors_1.h $
 // $Id: functors_1.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/RS/functors_z_1.h 5.1-1/include/CGAL/RS/functors_z_1.h
--- 5.0.3-1/include/CGAL/RS/functors_z_1.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/RS/functors_z_1.h	2020-09-08 12:14:31.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/RS/functors_z_1.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/RS/functors_z_1.h $
 // $Id: functors_z_1.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/RS/Gmpfr_make_unique.h 5.1-1/include/CGAL/RS/Gmpfr_make_unique.h
--- 5.0.3-1/include/CGAL/RS/Gmpfr_make_unique.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/RS/Gmpfr_make_unique.h	2020-09-08 12:14:31.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/RS/Gmpfr_make_unique.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/RS/Gmpfr_make_unique.h $
 // $Id: Gmpfr_make_unique.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/RS/polynomial_converter_1.h 5.1-1/include/CGAL/RS/polynomial_converter_1.h
--- 5.0.3-1/include/CGAL/RS/polynomial_converter_1.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/RS/polynomial_converter_1.h	2020-09-08 12:14:31.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/RS/polynomial_converter_1.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/RS/polynomial_converter_1.h $
 // $Id: polynomial_converter_1.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/RS/rs23_k_isolator_1.h 5.1-1/include/CGAL/RS/rs23_k_isolator_1.h
--- 5.0.3-1/include/CGAL/RS/rs23_k_isolator_1.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/RS/rs23_k_isolator_1.h	2020-09-08 12:14:31.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/RS/rs23_k_isolator_1.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/RS/rs23_k_isolator_1.h $
 // $Id: rs23_k_isolator_1.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/RS/rs2_calls.h 5.1-1/include/CGAL/RS/rs2_calls.h
--- 5.0.3-1/include/CGAL/RS/rs2_calls.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/RS/rs2_calls.h	2020-09-08 12:14:31.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/RS/rs2_calls.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/RS/rs2_calls.h $
 // $Id: rs2_calls.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/RS/rs2_isolator_1.h 5.1-1/include/CGAL/RS/rs2_isolator_1.h
--- 5.0.3-1/include/CGAL/RS/rs2_isolator_1.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/RS/rs2_isolator_1.h	2020-09-08 12:14:31.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/RS/rs2_isolator_1.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/RS/rs2_isolator_1.h $
 // $Id: rs2_isolator_1.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/RS/rs3_k_refiner_1.h 5.1-1/include/CGAL/RS/rs3_k_refiner_1.h
--- 5.0.3-1/include/CGAL/RS/rs3_k_refiner_1.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/RS/rs3_k_refiner_1.h	2020-09-08 12:14:31.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/RS/rs3_k_refiner_1.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/RS/rs3_k_refiner_1.h $
 // $Id: rs3_k_refiner_1.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/RS/rs3_refiner_1.h 5.1-1/include/CGAL/RS/rs3_refiner_1.h
--- 5.0.3-1/include/CGAL/RS/rs3_refiner_1.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/RS/rs3_refiner_1.h	2020-09-08 12:14:31.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/RS/rs3_refiner_1.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/RS/rs3_refiner_1.h $
 // $Id: rs3_refiner_1.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/RS/signat_1.h 5.1-1/include/CGAL/RS/signat_1.h
--- 5.0.3-1/include/CGAL/RS/signat_1.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/RS/signat_1.h	2020-09-08 12:14:31.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_kernel_d/include/CGAL/RS/signat_1.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_kernel_d/include/CGAL/RS/signat_1.h $
 // $Id: signat_1.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Runge_kutta_integrator_2.h 5.1-1/include/CGAL/Runge_kutta_integrator_2.h
--- 5.0.3-1/include/CGAL/Runge_kutta_integrator_2.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Runge_kutta_integrator_2.h	2020-09-08 12:15:41.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_lines_2/include/CGAL/Runge_kutta_integrator_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_lines_2/include/CGAL/Runge_kutta_integrator_2.h $
 // $Id: Runge_kutta_integrator_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Scalar_factor_traits.h 5.1-1/include/CGAL/Scalar_factor_traits.h
--- 5.0.3-1/include/CGAL/Scalar_factor_traits.h	2020-08-01 19:02:08.000000000 +0000
+++ 5.1-1/include/CGAL/Scalar_factor_traits.h	2020-09-08 12:14:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Algebraic_foundations/include/CGAL/Scalar_factor_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Algebraic_foundations/include/CGAL/Scalar_factor_traits.h $
 // $Id: Scalar_factor_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Scale_space_reconstruction_3/Advancing_front_mesher.h 5.1-1/include/CGAL/Scale_space_reconstruction_3/Advancing_front_mesher.h
--- 5.0.3-1/include/CGAL/Scale_space_reconstruction_3/Advancing_front_mesher.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Scale_space_reconstruction_3/Advancing_front_mesher.h	2020-09-08 12:15:33.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Scale_space_reconstruction_3/include/CGAL/Scale_space_reconstruction_3/Advancing_front_mesher.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Scale_space_reconstruction_3/include/CGAL/Scale_space_reconstruction_3/Advancing_front_mesher.h $
 // $Id: Advancing_front_mesher.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Scale_space_reconstruction_3/Alpha_shape_mesher.h 5.1-1/include/CGAL/Scale_space_reconstruction_3/Alpha_shape_mesher.h
--- 5.0.3-1/include/CGAL/Scale_space_reconstruction_3/Alpha_shape_mesher.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Scale_space_reconstruction_3/Alpha_shape_mesher.h	2020-09-08 12:15:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Scale_space_reconstruction_3/include/CGAL/Scale_space_reconstruction_3/Alpha_shape_mesher.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Scale_space_reconstruction_3/include/CGAL/Scale_space_reconstruction_3/Alpha_shape_mesher.h $
 // $Id: Alpha_shape_mesher.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Scale_space_reconstruction_3/internal/Auto_count.h 5.1-1/include/CGAL/Scale_space_reconstruction_3/internal/Auto_count.h
--- 5.0.3-1/include/CGAL/Scale_space_reconstruction_3/internal/Auto_count.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Scale_space_reconstruction_3/internal/Auto_count.h	2020-09-08 12:15:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Scale_space_reconstruction_3/include/CGAL/Scale_space_reconstruction_3/internal/Auto_count.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Scale_space_reconstruction_3/include/CGAL/Scale_space_reconstruction_3/internal/Auto_count.h $
 // $Id: Auto_count.h 8c4f590 2019-10-19T16:20:43+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Scale_space_reconstruction_3/Jet_smoother.h 5.1-1/include/CGAL/Scale_space_reconstruction_3/Jet_smoother.h
--- 5.0.3-1/include/CGAL/Scale_space_reconstruction_3/Jet_smoother.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Scale_space_reconstruction_3/Jet_smoother.h	2020-09-08 12:15:33.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Scale_space_reconstruction_3/include/CGAL/Scale_space_reconstruction_3/Jet_smoother.h $
-// $Id: Jet_smoother.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Scale_space_reconstruction_3/include/CGAL/Scale_space_reconstruction_3/Jet_smoother.h $
+// $Id: Jet_smoother.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s):      Simon Giraudot
@@ -39,17 +39,15 @@ namespace Scale_space_reconstruction_3
  *  `RealEmbeddable` field number type. Generally,
  *  `Exact_predicates_inexact_constructions_kernel` is preferred.
  *  \tparam ConcurrencyTag indicates whether to use concurrent
- *  processing. It can be omitted: if TBB (or greater) is available
+ *  processing. It can be omitted: if \ref thirdpartyTBB is available
  *  and `CGAL_LINKED_WITH_TBB` is defined then `Parallel_tag` is
  *  used. Otherwise, `Sequential_tag` is used.
  */
 template <typename Geom_traits,
 #ifdef DOXYGEN_RUNNING
           typename ConcurrencyTag>
-#elif CGAL_LINKED_WITH_TBB
-          typename ConcurrencyTag = CGAL::Parallel_tag>
 #else
-          typename ConcurrencyTag = CGAL::Sequential_tag>
+          typename ConcurrencyTag = CGAL::Parallel_if_available_tag>
 #endif
 class Jet_smoother
 {
diff -pruN 5.0.3-1/include/CGAL/Scale_space_reconstruction_3/Shape_construction_3.h 5.1-1/include/CGAL/Scale_space_reconstruction_3/Shape_construction_3.h
--- 5.0.3-1/include/CGAL/Scale_space_reconstruction_3/Shape_construction_3.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Scale_space_reconstruction_3/Shape_construction_3.h	2020-09-08 12:15:33.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Scale_space_reconstruction_3/include/CGAL/Scale_space_reconstruction_3/Shape_construction_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Scale_space_reconstruction_3/include/CGAL/Scale_space_reconstruction_3/Shape_construction_3.h $
 // $Id: Shape_construction_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Scale_space_reconstruction_3/Weighted_PCA_smoother.h 5.1-1/include/CGAL/Scale_space_reconstruction_3/Weighted_PCA_smoother.h
--- 5.0.3-1/include/CGAL/Scale_space_reconstruction_3/Weighted_PCA_smoother.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Scale_space_reconstruction_3/Weighted_PCA_smoother.h	2020-09-08 12:15:33.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Scale_space_reconstruction_3/include/CGAL/Scale_space_reconstruction_3/Weighted_PCA_smoother.h $
-// $Id: Weighted_PCA_smoother.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Scale_space_reconstruction_3/include/CGAL/Scale_space_reconstruction_3/Weighted_PCA_smoother.h $
+// $Id: Weighted_PCA_smoother.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s):      Thijs van Lankveld, Simon Giraudot
@@ -55,21 +55,17 @@ namespace Scale_space_reconstruction_3
  *  that case, an overload using `Eigen_diagonalize_traits` is
  *  provided.
  *  \tparam ConcurrencyTag indicates whether to use concurrent
- *  processing. It can be omitted: if TBB (or greater) is available
+ *  processing. It can be omitted: if \ref thirdpartyTBB is available
  *  and `CGAL_LINKED_WITH_TBB` is defined then `Parallel_tag` is
  *  used. Otherwise, `Sequential_tag` is used.
  */
 template <typename Geom_traits,
 #ifdef DOXYGEN_RUNNING
-          typename DiagonalizeTraits, typename ConcurrencyTag>
+          typename DiagonalizeTraits,
+          typename ConcurrencyTag>
 #else // DOXYGEN_RUNNING
-          typename DiagonalizeTraits
-          = CGAL::Default_diagonalize_traits<typename Geom_traits::FT, 3>,
-#ifdef CGAL_LINKED_WITH_TBB
-          typename ConcurrencyTag = CGAL::Parallel_tag>
-#else
-          typename ConcurrencyTag = CGAL::Sequential_tag>
-#endif // CGAL_LINKED_WITH_TBB
+          typename DiagonalizeTraits = CGAL::Default_diagonalize_traits<typename Geom_traits::FT, 3>,
+          typename ConcurrencyTag = CGAL::Parallel_if_available_tag>
 #endif // DOXYGEN_RUNNING
 class Weighted_PCA_smoother
 {
diff -pruN 5.0.3-1/include/CGAL/Scale_space_surface_reconstruction_3.h 5.1-1/include/CGAL/Scale_space_surface_reconstruction_3.h
--- 5.0.3-1/include/CGAL/Scale_space_surface_reconstruction_3.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Scale_space_surface_reconstruction_3.h	2020-09-08 12:15:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Scale_space_reconstruction_3/include/CGAL/Scale_space_surface_reconstruction_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Scale_space_reconstruction_3/include/CGAL/Scale_space_surface_reconstruction_3.h $
 // $Id: Scale_space_surface_reconstruction_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/SCIP_mixed_integer_program_traits.h 5.1-1/include/CGAL/SCIP_mixed_integer_program_traits.h
--- 5.0.3-1/include/CGAL/SCIP_mixed_integer_program_traits.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/SCIP_mixed_integer_program_traits.h	2020-09-08 12:15:39.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Solver_interface/include/CGAL/SCIP_mixed_integer_program_traits.h $
-// $Id: SCIP_mixed_integer_program_traits.h 202430d 2020-04-20T15:51:48+02:00 Dmitry Anisimov
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Solver_interface/include/CGAL/SCIP_mixed_integer_program_traits.h $
+// $Id: SCIP_mixed_integer_program_traits.h 3b6c100 2020-04-29T14:12:29+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s) : Liangliang Nan
@@ -13,225 +13,217 @@
 
 #include <CGAL/Mixed_integer_program_traits.h>
 
-#ifdef CGAL_USE_SCIP
+#if defined(CGAL_USE_SCIP) || defined(DOXYGEN_RUNNING)
 
 #include "scip/scip.h"
 #include "scip/scipdefplugins.h"
 
+#include <cmath>
+#include <iostream>
+#include <string>
+#include <vector>
+
 namespace CGAL {
 
-        /// \ingroup PkgSolverInterfaceRef
-        ///
-        /// This class provides an interface for formulating and solving
-        /// mixed integer programs (constrained or unconstrained) using
-        /// \ref thirdpartySCIP.
-        ///
-        /// \ref thirdpartySCIP must be available on the system.
-        ///
-        /// \cond SKIP_IN_MANUAL
-        /// \tparam FT Number type
-        /// \endcond
-        ///
-        /// \cgalModels `MixedIntegerProgramTraits`
-        ///
-        /// \sa `GLPK_mixed_integer_program_traits`
-        template <typename FT>
-        class SCIP_mixed_integer_program_traits : public Mixed_integer_program_traits<FT>
-        {
-                /// \cond SKIP_IN_MANUAL
-        public:
-                typedef CGAL::Mixed_integer_program_traits<FT>                Base_class;
-                typedef typename Base_class::Variable                        Variable;
-                typedef typename Base_class::Linear_constraint          Linear_constraint;
-                typedef typename Base_class::Linear_objective           Linear_objective;
-                typedef typename Linear_objective::Sense                Sense;
-                typedef typename Variable::Variable_type                Variable_type;
-
-        public:
-
-                /// Solves the program. Returns false if fails.
-                virtual bool solve();
-                /// \endcond
-        };
-
-        //////////////////////////////////////////////////////////////////////////
-
-        // implementation
-
-        template<typename FT>
-        bool SCIP_mixed_integer_program_traits<FT>::solve()
-        {
-                Base_class::error_message_.clear();
-
-                Scip* scip = 0;
-                SCIP_CALL(SCIPcreate(&scip));
-                SCIP_CALL(SCIPincludeDefaultPlugins(scip));
-
-                // Disables scip output to stdout
-                SCIPmessagehdlrSetQuiet(SCIPgetMessagehdlr(scip), TRUE);
-
-                // Uses wall clock time because getting CPU user seconds
-                // involves calling times() which is very expensive
-                SCIP_CALL(SCIPsetIntParam(scip, "timing/clocktype", SCIP_CLOCKTYPE_WALL));
-
-                // Creates empty problem
-                SCIP_CALL(SCIPcreateProbBasic(scip, "Solver_interface"));
-
-                // Creates variables
-                std::vector<SCIP_VAR*> scip_variables;
-                for (std::size_t i = 0; i < Base_class::variables_.size(); ++i) {
-                        const Variable* var = Base_class::variables_[i];
-                        SCIP_VAR* v = 0;
-
-                        double lb, ub;
-                        var->get_bounds(lb, ub);
-
-                        switch (var->variable_type())
-                        {
-                        case Variable::CONTINUOUS:
-                                SCIP_CALL(SCIPcreateVar(scip, &v, var->name().c_str(), lb, ub, 0.0, SCIP_VARTYPE_CONTINUOUS, TRUE, FALSE, 0, 0, 0, 0, 0));
-                                break;
-                        case Variable::INTEGER:
-                                SCIP_CALL(SCIPcreateVar(scip, &v, var->name().c_str(), lb, ub, 0.0, SCIP_VARTYPE_INTEGER, TRUE, FALSE, 0, 0, 0, 0, 0));
-                                break;
-                        case Variable::BINARY:
-                                SCIP_CALL(SCIPcreateVar(scip, &v, var->name().c_str(), 0, 1, 0.0, SCIP_VARTYPE_BINARY, TRUE, FALSE, 0, 0, 0, 0, 0));
-                                break;
-                        }
-                        // Adds the SCIP_VAR object to the scip problem
-                        SCIP_CALL(SCIPaddVar(scip, v));
-
-                        // Stores the SCIP_VAR pointer for later access
-                        scip_variables.push_back(v);
-                }
-
-                // Adds constraints
-
-                std::vector<SCIP_CONS*> scip_constraints;
-                for (std::size_t i = 0; i < Base_class::constraints_.size(); ++i) {
-                        const Linear_constraint* c = Base_class::constraints_[i];
-                        const std::unordered_map<const Variable*, double>& coeffs = c->coefficients();
-                        typename std::unordered_map<const Variable*, double>::const_iterator cur = coeffs.begin();
-
-                        std::vector<SCIP_VAR*>        cstr_variables(coeffs.size());
-                        std::vector<double>                cstr_values(coeffs.size());
-                        std::size_t idx = 0;
-                        for (; cur != coeffs.end(); ++cur) {
-                                std::size_t var_idx = cur->first->index();
-                                double coeff = cur->second;
-                                cstr_variables[idx] = scip_variables[var_idx];
-                                cstr_values[idx] = coeff;
-                                ++idx;
-                        }
-
-                        // Creates SCIP_CONS object
-                        SCIP_CONS* cons = 0;
-                        const std::string& name = c->name();
-
-                        double lb, ub;
-                        c->get_bounds(lb, ub);
-
-                        SCIP_CALL(SCIPcreateConsLinear(scip, &cons, name.c_str(), int(coeffs.size()), cstr_variables.data(), cstr_values.data(), lb, ub, TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, FALSE, FALSE, FALSE, FALSE));
-
-                        // Adds the constraint to scip
-                        SCIP_CALL(SCIPaddCons(scip, cons));
-
-                        // Stores the constraint for later on
-                        scip_constraints.push_back(cons);
-                }
-
-                // Sets objective
-
-                // Determines the coefficient of each variable in the objective function
-                const std::unordered_map<const Variable*, double>& obj_coeffs = Base_class::objective_->coefficients();
-                typename std::unordered_map<const Variable*, double>::const_iterator cur = obj_coeffs.begin();
-                for (; cur != obj_coeffs.end(); ++cur) {
-                        const Variable* var = cur->first;
-                        double coeff = cur->second;
-                        SCIP_CALL(SCIPchgVarObj(scip, scip_variables[var->index()], coeff));
-                }
-
-                // Sets the objective sense
-                bool minimize = (Base_class::objective_->sense() == Linear_objective::MINIMIZE);
-                SCIP_CALL(SCIPsetObjsense(scip, minimize ? SCIP_OBJSENSE_MINIMIZE : SCIP_OBJSENSE_MAXIMIZE));
-
-                // Turns presolve on (it's the SCIP default).
-                bool presolve = true;
-                if (presolve)
-                        SCIP_CALL(SCIPsetIntParam(scip, "presolving/maxrounds", -1)); // maximal number of presolving rounds (-1: unlimited, 0: off)
-                else
-                        SCIP_CALL(SCIPsetIntParam(scip, "presolving/maxrounds", 0));  // disable presolve
-
-                bool status = false;
-                // This tells scip to start the solution process
-                if (SCIPsolve(scip) == SCIP_OKAY) {
-                        // Gets the best found solution from scip
-                        SCIP_SOL* sol = SCIPgetBestSol(scip);
-                        if (sol) {
-                                // If optimal or feasible solution is found.
-                                Base_class::result_.resize(Base_class::variables_.size());
-                                for (std::size_t i = 0; i < Base_class::variables_.size(); ++i) {
-                                        FT x = SCIPgetSolVal(scip, sol, scip_variables[i]);
-
-                                        Variable* v = Base_class::variables_[i];
-                                        if (v->variable_type() != Variable::CONTINUOUS)
-                                                x = static_cast<int>(std::round(x));
-
-                                        v->set_solution_value(x);
-                                        Base_class::result_[i] = x;
-                                }
-                                status = true;
-                        }
-                }
-
-                // Reports the status: optimal, infeasible, etc.
-                SCIP_STATUS scip_status = SCIPgetStatus(scip);
-                switch (scip_status) {
-                case SCIP_STATUS_OPTIMAL:
-                        // Provides info only if fails.
-                        break;
-                case SCIP_STATUS_GAPLIMIT:
-                        // To be consistent with the other solvers.
-                        // Provides info only if fails.
-                        break;
-                case SCIP_STATUS_INFEASIBLE:
-                        Base_class::error_message_ = "model was infeasible";
-                        break;
-                case SCIP_STATUS_UNBOUNDED:
-                        Base_class::error_message_ = "model was unbounded";
-                        break;
-                case SCIP_STATUS_INFORUNBD:
-                        Base_class::error_message_ = "model was either infeasible or unbounded";
-                        break;
-                case SCIP_STATUS_TIMELIMIT:
-                        Base_class::error_message_ = "aborted due to time limit";
-                        break;
-                default:
-                        Base_class::error_message_ = "aborted with status: " + std::to_string(scip_status);
-                        break;
-                }
-
-                SCIP_CALL(SCIPresetParams(scip));
-
-                // Since the SCIPcreateVar captures all variables, we have to release them now
-                for (std::size_t i = 0; i < scip_variables.size(); ++i)
-                        SCIP_CALL(SCIPreleaseVar(scip, &scip_variables[i]));
-                scip_variables.clear();
-
-                // The same for the constraints
-                for (std::size_t i = 0; i < scip_constraints.size(); ++i)
-                        SCIP_CALL(SCIPreleaseCons(scip, &scip_constraints[i]));
-                scip_constraints.clear();
-
-                // After releasing all vars and cons we can free the scip problem.
-                // Remember this has always to be the last call to scip
-                SCIP_CALL(SCIPfree(&scip));
+/// \ingroup PkgSolverInterfaceRef
+///
+/// This class provides an interface for formulating and solving
+/// constrained or unconstrained mixed integer programs using
+/// \ref thirdpartySCIP (which must be available on the system).
+///
+/// \cgalModels `MixedIntegerProgramTraits`
+///
+/// \sa `GLPK_mixed_integer_program_traits`
+template <typename FT>
+class SCIP_mixed_integer_program_traits
+  : public Mixed_integer_program_traits<FT>
+{
+  /// \cond SKIP_IN_MANUAL
+public:
+  typedef CGAL::Mixed_integer_program_traits<FT>          Base_class;
+  typedef typename Base_class::Variable                   Variable;
+  typedef typename Base_class::Linear_constraint          Linear_constraint;
+  typedef typename Base_class::Linear_objective           Linear_objective;
+  typedef typename Linear_objective::Sense                Sense;
+  typedef typename Variable::Variable_type                Variable_type;
+
+public:
+  /// Solves the program. Returns `false` if fails.
+  virtual bool solve()
+  {
+    Base_class::error_message_.clear();
+
+    Scip* scip = 0;
+    SCIP_CALL(SCIPcreate(&scip));
+    SCIP_CALL(SCIPincludeDefaultPlugins(scip));
+
+    // Disables scip output to stdout
+    SCIPmessagehdlrSetQuiet(SCIPgetMessagehdlr(scip), TRUE);
+
+    // Uses wall clock time because getting CPU user seconds
+    // involves calling times() which is very expensive
+    SCIP_CALL(SCIPsetIntParam(scip, "timing/clocktype", SCIP_CLOCKTYPE_WALL));
+
+    // Creates empty problem
+    SCIP_CALL(SCIPcreateProbBasic(scip, "Solver_interface"));
+
+    // Creates variables
+    std::vector<SCIP_VAR*> scip_variables;
+    for (std::size_t i = 0; i < Base_class::variables_.size(); ++i) {
+      const Variable* var = Base_class::variables_[i];
+      SCIP_VAR* v = 0;
+
+      double lb, ub;
+      var->get_bounds(lb, ub);
+
+      switch (var->variable_type())
+      {
+        case Variable::CONTINUOUS:
+          SCIP_CALL(SCIPcreateVar(scip, &v, var->name().c_str(), lb, ub, 0.0, SCIP_VARTYPE_CONTINUOUS, TRUE, FALSE, 0, 0, 0, 0, 0));
+          break;
+        case Variable::INTEGER:
+          SCIP_CALL(SCIPcreateVar(scip, &v, var->name().c_str(), lb, ub, 0.0, SCIP_VARTYPE_INTEGER, TRUE, FALSE, 0, 0, 0, 0, 0));
+          break;
+        case Variable::BINARY:
+          SCIP_CALL(SCIPcreateVar(scip, &v, var->name().c_str(), 0, 1, 0.0, SCIP_VARTYPE_BINARY, TRUE, FALSE, 0, 0, 0, 0, 0));
+          break;
+      }
+      // Adds the SCIP_VAR object to the scip problem
+      SCIP_CALL(SCIPaddVar(scip, v));
+
+      // Stores the SCIP_VAR pointer for later access
+      scip_variables.push_back(v);
+    }
+
+    // Adds constraints
+
+    std::vector<SCIP_CONS*> scip_constraints;
+    for (std::size_t i = 0; i < Base_class::constraints_.size(); ++i) {
+      const Linear_constraint* c = Base_class::constraints_[i];
+      const std::unordered_map<const Variable*, double>& coeffs = c->coefficients();
+      typename std::unordered_map<const Variable*, double>::const_iterator cur = coeffs.begin();
+
+      std::vector<SCIP_VAR*>  cstr_variables(coeffs.size());
+      std::vector<double>    cstr_values(coeffs.size());
+      std::size_t idx = 0;
+      for (; cur != coeffs.end(); ++cur) {
+        std::size_t var_idx = cur->first->index();
+        double coeff = cur->second;
+        cstr_variables[idx] = scip_variables[var_idx];
+        cstr_values[idx] = coeff;
+        ++idx;
+      }
+
+      // Creates SCIP_CONS object
+      SCIP_CONS* cons = 0;
+      const std::string& name = c->name();
+
+      double lb, ub;
+      c->get_bounds(lb, ub);
+
+      SCIP_CALL(SCIPcreateConsLinear(scip, &cons, name.c_str(), int(coeffs.size()), cstr_variables.data(), cstr_values.data(), lb, ub, TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, FALSE, FALSE, FALSE, FALSE));
+
+      // Adds the constraint to scip
+      SCIP_CALL(SCIPaddCons(scip, cons));
+
+      // Stores the constraint for later on
+      scip_constraints.push_back(cons);
+    }
+
+    // Sets objective
+
+    // Determines the coefficient of each variable in the objective function
+    const std::unordered_map<const Variable*, double>& obj_coeffs = Base_class::objective_->coefficients();
+    typename std::unordered_map<const Variable*, double>::const_iterator cur = obj_coeffs.begin();
+    for (; cur != obj_coeffs.end(); ++cur) {
+      const Variable* var = cur->first;
+      double coeff = cur->second;
+      SCIP_CALL(SCIPchgVarObj(scip, scip_variables[var->index()], coeff));
+    }
+
+    // Sets the objective sense
+    bool minimize = (Base_class::objective_->sense() == Linear_objective::MINIMIZE);
+    SCIP_CALL(SCIPsetObjsense(scip, minimize ? SCIP_OBJSENSE_MINIMIZE : SCIP_OBJSENSE_MAXIMIZE));
+
+    // Turns presolve on (it's the SCIP default).
+    bool presolve = true;
+    if (presolve)
+      SCIP_CALL(SCIPsetIntParam(scip, "presolving/maxrounds", -1)); // maximal number of presolving rounds (-1: unlimited, 0: off)
+    else
+      SCIP_CALL(SCIPsetIntParam(scip, "presolving/maxrounds", 0));  // disable presolve
+
+    bool status = false;
+    // This tells scip to start the solution process
+    if (SCIPsolve(scip) == SCIP_OKAY) {
+      // Gets the best found solution from scip
+      SCIP_SOL* sol = SCIPgetBestSol(scip);
+      if (sol) {
+        // If optimal or feasible solution is found.
+        Base_class::result_.resize(Base_class::variables_.size());
+        for (std::size_t i = 0; i < Base_class::variables_.size(); ++i) {
+          FT x = SCIPgetSolVal(scip, sol, scip_variables[i]);
+
+          Variable* v = Base_class::variables_[i];
+          if (v->variable_type() != Variable::CONTINUOUS)
+            x = static_cast<int>(std::round(x));
 
-                return status;
+          v->set_solution_value(x);
+          Base_class::result_[i] = x;
         }
+        status = true;
+      }
+    }
+
+    // Reports the status: optimal, infeasible, etc.
+    SCIP_STATUS scip_status = SCIPgetStatus(scip);
+    switch (scip_status) {
+      case SCIP_STATUS_OPTIMAL:
+        // Provides info only if fails.
+        break;
+      case SCIP_STATUS_GAPLIMIT:
+        // To be consistent with the other solvers.
+        // Provides info only if fails.
+        break;
+      case SCIP_STATUS_INFEASIBLE:
+        Base_class::error_message_ = "model was infeasible";
+        break;
+      case SCIP_STATUS_UNBOUNDED:
+        Base_class::error_message_ = "model was unbounded";
+        break;
+      case SCIP_STATUS_INFORUNBD:
+        Base_class::error_message_ = "model was either infeasible or unbounded";
+        break;
+      case SCIP_STATUS_TIMELIMIT:
+        Base_class::error_message_ = "aborted due to time limit";
+        break;
+      default:
+        Base_class::error_message_ = "aborted with status: " + std::to_string(scip_status);
+        break;
+    }
+
+    SCIP_CALL(SCIPresetParams(scip));
+
+    // Since the SCIPcreateVar captures all variables, we have to release them now
+    for (std::size_t i = 0; i < scip_variables.size(); ++i)
+      SCIP_CALL(SCIPreleaseVar(scip, &scip_variables[i]));
+    scip_variables.clear();
+
+    // The same for the constraints
+    for (std::size_t i = 0; i < scip_constraints.size(); ++i)
+      SCIP_CALL(SCIPreleaseCons(scip, &scip_constraints[i]));
+    scip_constraints.clear();
+
+    // After releasing all vars and cons we can free the scip problem.
+    // Remember this has always to be the last call to scip
+    SCIP_CALL(SCIPfree(&scip));
+
+    return status;
+  }
+  /// \endcond
+};
 
 } // namespace CGAL
 
-#endif // CGAL_USE_SCIP
+#endif // CGAL_USE_SCIP or DOXYGEN_RUNNING
 
 #endif // CGAL_SCIP_MIXED_INTEGER_PROGRAM_TRAITS_H
diff -pruN 5.0.3-1/include/CGAL/Search_traits_2.h 5.1-1/include/CGAL/Search_traits_2.h
--- 5.0.3-1/include/CGAL/Search_traits_2.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Search_traits_2.h	2020-09-08 12:15:39.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_searching/include/CGAL/Search_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_searching/include/CGAL/Search_traits_2.h $
 // $Id: Search_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Search_traits_3.h 5.1-1/include/CGAL/Search_traits_3.h
--- 5.0.3-1/include/CGAL/Search_traits_3.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Search_traits_3.h	2020-09-08 12:15:39.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_searching/include/CGAL/Search_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_searching/include/CGAL/Search_traits_3.h $
 // $Id: Search_traits_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Search_traits_adapter.h 5.1-1/include/CGAL/Search_traits_adapter.h
--- 5.0.3-1/include/CGAL/Search_traits_adapter.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Search_traits_adapter.h	2020-09-08 12:15:40.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_searching/include/CGAL/Search_traits_adapter.h $
-// $Id: Search_traits_adapter.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_searching/include/CGAL/Search_traits_adapter.h $
+// $Id: Search_traits_adapter.h 2e180ac 2020-03-26T19:29:44+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -182,7 +182,11 @@ public:
     }
 
     Dereference_type&
-    dereference() const { return const_cast<Dereference_type&>((*point)[idx]); }
+    dereference() const
+    {
+      // Point::operator[] takes an int as parameter...
+      return const_cast<Dereference_type&>((*point)[static_cast<int>(idx)]);
+    }
 
   };
 
@@ -269,7 +273,6 @@ public:
 template <class Point_with_info,class PointPropertyMap,class Base_distance>
 class Distance_adapter : public Base_distance {
   PointPropertyMap ppmap;
-  typedef typename Base_distance::FT FT;
 
 public:
 
@@ -279,6 +282,7 @@ public:
 
   using Base_distance::transformed_distance;
 
+  typedef typename Base_distance::FT FT;
   typedef Point_with_info Point_d;
   typedef typename Base_distance::Query_item Query_item;
 
diff -pruN 5.0.3-1/include/CGAL/Search_traits_d.h 5.1-1/include/CGAL/Search_traits_d.h
--- 5.0.3-1/include/CGAL/Search_traits_d.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Search_traits_d.h	2020-09-08 12:15:40.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_searching/include/CGAL/Search_traits_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_searching/include/CGAL/Search_traits_d.h $
 // $Id: Search_traits_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Search_traits.h 5.1-1/include/CGAL/Search_traits.h
--- 5.0.3-1/include/CGAL/Search_traits.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Search_traits.h	2020-09-08 12:15:39.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_searching/include/CGAL/Search_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_searching/include/CGAL/Search_traits.h $
 // $Id: Search_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_2.h 5.1-1/include/CGAL/Segment_2.h
--- 5.0.3-1/include/CGAL/Segment_2.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_2.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Segment_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Segment_2.h $
 // $Id: Segment_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_3.h 5.1-1/include/CGAL/Segment_3.h
--- 5.0.3-1/include/CGAL/Segment_3.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_3.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Segment_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Segment_3.h $
 // $Id: Segment_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Are_parallel_C2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Are_parallel_C2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Are_parallel_C2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Are_parallel_C2.h	2020-09-08 12:15:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Are_parallel_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Are_parallel_C2.h $
 // $Id: Are_parallel_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Are_same_points_C2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Are_same_points_C2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Are_same_points_C2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Are_same_points_C2.h	2020-09-08 12:15:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Are_same_points_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Are_same_points_C2.h $
 // $Id: Are_same_points_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Are_same_segments_C2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Are_same_segments_C2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Are_same_segments_C2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Are_same_segments_C2.h	2020-09-08 12:15:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Are_same_segments_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Are_same_segments_C2.h $
 // $Id: Are_same_segments_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Arrangement_enum.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Arrangement_enum.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Arrangement_enum.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Arrangement_enum.h	2020-09-08 12:15:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Arrangement_enum.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Arrangement_enum.h $
 // $Id: Arrangement_enum.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Arrangement_type_C2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Arrangement_type_C2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Arrangement_type_C2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Arrangement_type_C2.h	2020-09-08 12:15:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Arrangement_type_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Arrangement_type_C2.h $
 // $Id: Arrangement_type_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Arrangement_type_non_intersecting_C2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Arrangement_type_non_intersecting_C2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Arrangement_type_non_intersecting_C2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Arrangement_type_non_intersecting_C2.h	2020-09-08 12:15:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Arrangement_type_non_intersecting_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Arrangement_type_non_intersecting_C2.h $
 // $Id: Arrangement_type_non_intersecting_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/basic.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/basic.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/basic.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/basic.h	2020-09-08 12:15:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/basic.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/basic.h $
 // $Id: basic.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Basic_predicates_C2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Basic_predicates_C2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Basic_predicates_C2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Basic_predicates_C2.h	2020-09-08 12:15:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Basic_predicates_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Basic_predicates_C2.h $
 // $Id: Basic_predicates_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Cartesian_converter.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Cartesian_converter.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Cartesian_converter.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Cartesian_converter.h	2020-09-08 12:15:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Cartesian_converter.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Cartesian_converter.h $
 // $Id: Cartesian_converter.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Compare_x_2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Compare_x_2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Compare_x_2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Compare_x_2.h	2020-09-08 12:15:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Compare_x_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Compare_x_2.h $
 // $Id: Compare_x_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Compare_y_2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Compare_y_2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Compare_y_2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Compare_y_2.h	2020-09-08 12:15:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Compare_y_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Compare_y_2.h $
 // $Id: Compare_y_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Constructions_C2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Constructions_C2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Constructions_C2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Constructions_C2.h	2020-09-08 12:15:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Constructions_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Constructions_C2.h $
 // $Id: Constructions_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Construct_storage_site_2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Construct_storage_site_2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Construct_storage_site_2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Construct_storage_site_2.h	2020-09-08 12:15:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Construct_storage_site_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Construct_storage_site_2.h $
 // $Id: Construct_storage_site_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Construct_storage_site_with_info_2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Construct_storage_site_with_info_2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Construct_storage_site_with_info_2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Construct_storage_site_with_info_2.h	2020-09-08 12:15:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Construct_storage_site_with_info_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Construct_storage_site_with_info_2.h $
 // $Id: Construct_storage_site_with_info_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Filtered_traits_base_2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Filtered_traits_base_2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Filtered_traits_base_2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Filtered_traits_base_2.h	2020-09-08 12:15:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Filtered_traits_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Filtered_traits_base_2.h $
 // $Id: Filtered_traits_base_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Filtered_traits_concept_check_tags.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Filtered_traits_concept_check_tags.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Filtered_traits_concept_check_tags.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Filtered_traits_concept_check_tags.h	2020-09-08 12:15:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Filtered_traits_concept_check_tags.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Filtered_traits_concept_check_tags.h $
 // $Id: Filtered_traits_concept_check_tags.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Finite_edge_interior_conflict_C2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Finite_edge_interior_conflict_C2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Finite_edge_interior_conflict_C2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Finite_edge_interior_conflict_C2.h	2020-09-08 12:15:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Finite_edge_interior_conflict_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Finite_edge_interior_conflict_C2.h $
 // $Id: Finite_edge_interior_conflict_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Infinite_edge_interior_conflict_C2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Infinite_edge_interior_conflict_C2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Infinite_edge_interior_conflict_C2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Infinite_edge_interior_conflict_C2.h	2020-09-08 12:15:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Infinite_edge_interior_conflict_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Infinite_edge_interior_conflict_C2.h $
 // $Id: Infinite_edge_interior_conflict_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/in_place_edge_list.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/in_place_edge_list.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/in_place_edge_list.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/in_place_edge_list.h	2020-09-08 12:15:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/in_place_edge_list.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/in_place_edge_list.h $
 // $Id: in_place_edge_list.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Is_degenerate_edge_C2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Is_degenerate_edge_C2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Is_degenerate_edge_C2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Is_degenerate_edge_C2.h	2020-09-08 12:15:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Is_degenerate_edge_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Is_degenerate_edge_C2.h $
 // $Id: Is_degenerate_edge_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Kernel_wrapper_2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Kernel_wrapper_2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Kernel_wrapper_2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Kernel_wrapper_2.h	2020-09-08 12:15:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Kernel_wrapper_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Kernel_wrapper_2.h $
 // $Id: Kernel_wrapper_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Orientation_C2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Orientation_C2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Orientation_C2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Orientation_C2.h	2020-09-08 12:15:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Orientation_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Orientation_C2.h $
 // $Id: Orientation_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Oriented_side_C2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Oriented_side_C2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Oriented_side_C2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Oriented_side_C2.h	2020-09-08 12:15:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Oriented_side_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Oriented_side_C2.h $
 // $Id: Oriented_side_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Oriented_side_of_bisector_C2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Oriented_side_of_bisector_C2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Oriented_side_of_bisector_C2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Oriented_side_of_bisector_C2.h	2020-09-08 12:15:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Oriented_side_of_bisector_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Oriented_side_of_bisector_C2.h $
 // $Id: Oriented_side_of_bisector_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Predicates_C2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Predicates_C2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Predicates_C2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Predicates_C2.h	2020-09-08 12:15:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Predicates_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Predicates_C2.h $
 // $Id: Predicates_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Segment_Delaunay_graph_2_impl.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Segment_Delaunay_graph_2_impl.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Segment_Delaunay_graph_2_impl.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Segment_Delaunay_graph_2_impl.h	2020-09-08 12:15:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Segment_Delaunay_graph_2_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Segment_Delaunay_graph_2_impl.h $
 // $Id: Segment_Delaunay_graph_2_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Segment_Delaunay_graph_hierarchy_2_impl.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Segment_Delaunay_graph_hierarchy_2_impl.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Segment_Delaunay_graph_hierarchy_2_impl.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Segment_Delaunay_graph_hierarchy_2_impl.h	2020-09-08 12:15:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Segment_Delaunay_graph_hierarchy_2_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Segment_Delaunay_graph_hierarchy_2_impl.h $
 // $Id: Segment_Delaunay_graph_hierarchy_2_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Sqrt_extension_2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Sqrt_extension_2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Sqrt_extension_2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Sqrt_extension_2.h	2020-09-08 12:15:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Sqrt_extension_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Sqrt_extension_2.h $
 // $Id: Sqrt_extension_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Traits_base_2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Traits_base_2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Traits_base_2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Traits_base_2.h	2020-09-08 12:15:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Traits_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Traits_base_2.h $
 // $Id: Traits_base_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Traits_wrapper_2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Traits_wrapper_2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Traits_wrapper_2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Traits_wrapper_2.h	2020-09-08 12:15:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Traits_wrapper_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Traits_wrapper_2.h $
 // $Id: Traits_wrapper_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Triangulation_face_base_with_edges_2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Triangulation_face_base_with_edges_2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Triangulation_face_base_with_edges_2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Triangulation_face_base_with_edges_2.h	2020-09-08 12:15:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Triangulation_face_base_with_edges_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Triangulation_face_base_with_edges_2.h $
 // $Id: Triangulation_face_base_with_edges_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Vertex_conflict_C2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Vertex_conflict_C2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Vertex_conflict_C2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Vertex_conflict_C2.h	2020-09-08 12:15:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Vertex_conflict_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Vertex_conflict_C2.h $
 // $Id: Vertex_conflict_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Voronoi_vertex_C2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Voronoi_vertex_C2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Voronoi_vertex_C2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Voronoi_vertex_C2.h	2020-09-08 12:15:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Voronoi_vertex_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Voronoi_vertex_C2.h $
 // $Id: Voronoi_vertex_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Voronoi_vertex_ring_C2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Voronoi_vertex_ring_C2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Voronoi_vertex_ring_C2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Voronoi_vertex_ring_C2.h	2020-09-08 12:15:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Voronoi_vertex_ring_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Voronoi_vertex_ring_C2.h $
 // $Id: Voronoi_vertex_ring_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Voronoi_vertex_sqrt_field_C2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Voronoi_vertex_sqrt_field_C2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Voronoi_vertex_sqrt_field_C2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Voronoi_vertex_sqrt_field_C2.h	2020-09-08 12:15:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Voronoi_vertex_sqrt_field_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Voronoi_vertex_sqrt_field_C2.h $
 // $Id: Voronoi_vertex_sqrt_field_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Voronoi_vertex_sqrt_field_new_C2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Voronoi_vertex_sqrt_field_new_C2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2/Voronoi_vertex_sqrt_field_new_C2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2/Voronoi_vertex_sqrt_field_new_C2.h	2020-09-08 12:15:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Voronoi_vertex_sqrt_field_new_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Voronoi_vertex_sqrt_field_new_C2.h $
 // $Id: Voronoi_vertex_sqrt_field_new_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_2.h	2020-09-08 12:15:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2.h $
 // $Id: Segment_Delaunay_graph_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_adaptation_policies_2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_adaptation_policies_2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_adaptation_policies_2.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_adaptation_policies_2.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Segment_Delaunay_graph_adaptation_policies_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Segment_Delaunay_graph_adaptation_policies_2.h $
 // $Id: Segment_Delaunay_graph_adaptation_policies_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_adaptation_traits_2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_adaptation_traits_2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_adaptation_traits_2.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_adaptation_traits_2.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Segment_Delaunay_graph_adaptation_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Segment_Delaunay_graph_adaptation_traits_2.h $
 // $Id: Segment_Delaunay_graph_adaptation_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_face_base_2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_face_base_2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_face_base_2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_face_base_2.h	2020-09-08 12:15:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_face_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_face_base_2.h $
 // $Id: Segment_Delaunay_graph_face_base_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_filtered_traits_2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_filtered_traits_2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_filtered_traits_2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_filtered_traits_2.h	2020-09-08 12:15:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_filtered_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_filtered_traits_2.h $
 // $Id: Segment_Delaunay_graph_filtered_traits_2.h 9a17c8d 2020-04-16T16:28:01+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_hierarchy_2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_hierarchy_2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_hierarchy_2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_hierarchy_2.h	2020-09-08 12:15:34.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_hierarchy_2.h $
-// $Id: Segment_Delaunay_graph_hierarchy_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_hierarchy_2.h $
+// $Id: Segment_Delaunay_graph_hierarchy_2.h cc18ed3 2020-04-06T18:33:26+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -246,11 +246,11 @@ public:
 
     if ( t.is_segment() ) {
       Point_handle_pair php =
-        this->register_input_site(t.source(), t.target());
+        this->register_input_site(t.source_of_supporting_site(), t.target_of_supporting_site());
       Storage_site_2 ss =
         this->st_.construct_storage_site_2_object()(php.first, php.second);
       Vertex_handle v =
-        insert_segment(t.source(), t.target(), ss, UNDEFINED_LEVEL);
+        insert_segment(t.source_of_supporting_site(), t.target_of_supporting_site(), ss, UNDEFINED_LEVEL);
       if ( v == Vertex_handle() ) {
         this->unregister_input_site( php.first, php.second );
       }
@@ -282,12 +282,12 @@ public:
 
     if ( t.is_segment() ) {
       Point_handle_pair php =
-        this->register_input_site(t.source(), t.target());
+        this->register_input_site(t.source_of_supporting_site(), t.target_of_supporting_site());
       Storage_site_2 ss =
         this->st_.construct_storage_site_2_object()(php.first, php.second);
       ss.set_info(info);
       Vertex_handle v =
-        insert_segment(t.source(), t.target(), ss, UNDEFINED_LEVEL);
+        insert_segment(t.source_of_supporting_site(), t.target_of_supporting_site(), ss, UNDEFINED_LEVEL);
       if ( v == Vertex_handle() ) {
         this->unregister_input_site( php.first, php.second );
       }
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_hierarchy_vertex_base_2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_hierarchy_vertex_base_2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_hierarchy_vertex_base_2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_hierarchy_vertex_base_2.h	2020-09-08 12:15:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_hierarchy_vertex_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_hierarchy_vertex_base_2.h $
 // $Id: Segment_Delaunay_graph_hierarchy_vertex_base_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/basic.h 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/basic.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/basic.h	2020-08-01 19:02:50.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/basic.h	2020-09-08 12:15:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/basic.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/basic.h $
 // $Id: basic.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Basic_predicates_C2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Basic_predicates_C2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Basic_predicates_C2.h	2020-08-01 19:02:50.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Basic_predicates_C2.h	2020-09-08 12:15:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/Basic_predicates_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/Basic_predicates_C2.h $
 // $Id: Basic_predicates_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Bisector_Linf.h 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Bisector_Linf.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Bisector_Linf.h	2020-08-01 19:02:50.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Bisector_Linf.h	2020-09-08 12:15:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/Bisector_Linf.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/Bisector_Linf.h $
 // $Id: Bisector_Linf.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Constructions_C2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Constructions_C2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Constructions_C2.h	2020-08-01 19:02:50.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Constructions_C2.h	2020-09-08 12:15:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/Constructions_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/Constructions_C2.h $
 // $Id: Constructions_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Filtered_traits_base_2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Filtered_traits_base_2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Filtered_traits_base_2.h	2020-08-01 19:02:50.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Filtered_traits_base_2.h	2020-09-08 12:15:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/Filtered_traits_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/Filtered_traits_base_2.h $
 // $Id: Filtered_traits_base_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Finite_edge_interior_conflict_C2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Finite_edge_interior_conflict_C2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Finite_edge_interior_conflict_C2.h	2020-08-01 19:02:50.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Finite_edge_interior_conflict_C2.h	2020-09-08 12:15:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/Finite_edge_interior_conflict_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/Finite_edge_interior_conflict_C2.h $
 // $Id: Finite_edge_interior_conflict_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Infinite_edge_interior_conflict_C2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Infinite_edge_interior_conflict_C2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Infinite_edge_interior_conflict_C2.h	2020-08-01 19:02:50.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Infinite_edge_interior_conflict_C2.h	2020-09-08 12:15:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/Infinite_edge_interior_conflict_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/Infinite_edge_interior_conflict_C2.h $
 // $Id: Infinite_edge_interior_conflict_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Orientation_Linf_C2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Orientation_Linf_C2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Orientation_Linf_C2.h	2020-08-01 19:02:50.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Orientation_Linf_C2.h	2020-09-08 12:15:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/Orientation_Linf_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/Orientation_Linf_C2.h $
 // $Id: Orientation_Linf_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Oriented_side_C2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Oriented_side_C2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Oriented_side_C2.h	2020-08-01 19:02:50.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Oriented_side_C2.h	2020-09-08 12:15:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/Oriented_side_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/Oriented_side_C2.h $
 // $Id: Oriented_side_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Oriented_side_of_bisector_C2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Oriented_side_of_bisector_C2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Oriented_side_of_bisector_C2.h	2020-08-01 19:02:50.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Oriented_side_of_bisector_C2.h	2020-09-08 12:15:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/Oriented_side_of_bisector_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/Oriented_side_of_bisector_C2.h $
 // $Id: Oriented_side_of_bisector_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Predicates_C2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Predicates_C2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Predicates_C2.h	2020-08-01 19:02:50.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Predicates_C2.h	2020-09-08 12:15:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/Predicates_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/Predicates_C2.h $
 // $Id: Predicates_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Segment_Delaunay_graph_Linf_2_impl.h 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Segment_Delaunay_graph_Linf_2_impl.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Segment_Delaunay_graph_Linf_2_impl.h	2020-08-01 19:02:50.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Segment_Delaunay_graph_Linf_2_impl.h	2020-09-08 12:15:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/Segment_Delaunay_graph_Linf_2_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/Segment_Delaunay_graph_Linf_2_impl.h $
 // $Id: Segment_Delaunay_graph_Linf_2_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Segment_Delaunay_graph_Linf_hierarchy_2_impl.h 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Segment_Delaunay_graph_Linf_hierarchy_2_impl.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Segment_Delaunay_graph_Linf_hierarchy_2_impl.h	2020-08-01 19:02:50.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Segment_Delaunay_graph_Linf_hierarchy_2_impl.h	2020-09-08 12:15:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/Segment_Delaunay_graph_Linf_hierarchy_2_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/Segment_Delaunay_graph_Linf_hierarchy_2_impl.h $
 // $Id: Segment_Delaunay_graph_Linf_hierarchy_2_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Traits_base_2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Traits_base_2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Traits_base_2.h	2020-08-01 19:02:50.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Traits_base_2.h	2020-09-08 12:15:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/Traits_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/Traits_base_2.h $
 // $Id: Traits_base_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Vertex_conflict_C2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Vertex_conflict_C2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Vertex_conflict_C2.h	2020-08-01 19:02:50.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Vertex_conflict_C2.h	2020-09-08 12:15:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/Vertex_conflict_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/Vertex_conflict_C2.h $
 // $Id: Vertex_conflict_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Voronoi_vertex_C2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Voronoi_vertex_C2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Voronoi_vertex_C2.h	2020-08-01 19:02:50.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Voronoi_vertex_C2.h	2020-09-08 12:15:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/Voronoi_vertex_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/Voronoi_vertex_C2.h $
 // $Id: Voronoi_vertex_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Voronoi_vertex_ring_C2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Voronoi_vertex_ring_C2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Voronoi_vertex_ring_C2.h	2020-08-01 19:02:50.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Voronoi_vertex_ring_C2.h	2020-09-08 12:15:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/Voronoi_vertex_ring_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/Voronoi_vertex_ring_C2.h $
 // $Id: Voronoi_vertex_ring_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Voronoi_vertex_sqrt_field_new_C2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Voronoi_vertex_sqrt_field_new_C2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Voronoi_vertex_sqrt_field_new_C2.h	2020-08-01 19:02:50.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2/Voronoi_vertex_sqrt_field_new_C2.h	2020-09-08 12:15:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/Voronoi_vertex_sqrt_field_new_C2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/Voronoi_vertex_sqrt_field_new_C2.h $
 // $Id: Voronoi_vertex_sqrt_field_new_C2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_2.h	2020-08-01 19:02:50.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_2.h	2020-09-08 12:15:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2.h $
 // $Id: Segment_Delaunay_graph_Linf_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_filtered_traits_2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_filtered_traits_2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_filtered_traits_2.h	2020-08-01 19:02:50.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_filtered_traits_2.h	2020-09-08 12:15:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_filtered_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_filtered_traits_2.h $
 // $Id: Segment_Delaunay_graph_Linf_filtered_traits_2.h 9a17c8d 2020-04-16T16:28:01+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_hierarchy_2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_hierarchy_2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_hierarchy_2.h	2020-08-01 19:02:50.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_hierarchy_2.h	2020-09-08 12:15:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_hierarchy_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_hierarchy_2.h $
 // $Id: Segment_Delaunay_graph_Linf_hierarchy_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_traits_2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_traits_2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_Linf_traits_2.h	2020-08-01 19:02:50.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_Linf_traits_2.h	2020-09-08 12:15:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_traits_2.h $
 // $Id: Segment_Delaunay_graph_Linf_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_simple_site_2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_simple_site_2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_simple_site_2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_simple_site_2.h	2020-09-08 12:15:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_simple_site_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_simple_site_2.h $
 // $Id: Segment_Delaunay_graph_simple_site_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_simple_storage_site_2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_simple_storage_site_2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_simple_storage_site_2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_simple_storage_site_2.h	2020-09-08 12:15:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_simple_storage_site_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_simple_storage_site_2.h $
 // $Id: Segment_Delaunay_graph_simple_storage_site_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_site_2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_site_2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_site_2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_site_2.h	2020-09-08 12:15:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_site_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_site_2.h $
 // $Id: Segment_Delaunay_graph_site_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_storage_site_2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_storage_site_2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_storage_site_2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_storage_site_2.h	2020-09-08 12:15:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_storage_site_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_storage_site_2.h $
 // $Id: Segment_Delaunay_graph_storage_site_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_storage_site_with_info_2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_storage_site_with_info_2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_storage_site_with_info_2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_storage_site_with_info_2.h	2020-09-08 12:15:34.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_storage_site_with_info_2.h $
-// $Id: Segment_Delaunay_graph_storage_site_with_info_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_storage_site_with_info_2.h $
+// $Id: Segment_Delaunay_graph_storage_site_with_info_2.h 980adeb 2020-04-09T13:27:13+02:00 Maxime Gimeno
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -119,7 +119,7 @@ public:
   // ACCESS TO INFO
   //---------------
   inline const Info& info() const {
-    CGAL_precondition( info_has_been_set() );
+    CGAL_assume(info_has_been_set());
     return info_;
   }
 
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_storage_traits_2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_storage_traits_2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_storage_traits_2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_storage_traits_2.h	2020-09-08 12:15:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_storage_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_storage_traits_2.h $
 // $Id: Segment_Delaunay_graph_storage_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_storage_traits_with_info_2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_storage_traits_with_info_2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_storage_traits_with_info_2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_storage_traits_with_info_2.h	2020-09-08 12:15:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_storage_traits_with_info_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_storage_traits_with_info_2.h $
 // $Id: Segment_Delaunay_graph_storage_traits_with_info_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_traits_2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_traits_2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_traits_2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_traits_2.h	2020-09-08 12:15:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_traits_2.h $
 // $Id: Segment_Delaunay_graph_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_Delaunay_graph_vertex_base_2.h 5.1-1/include/CGAL/Segment_Delaunay_graph_vertex_base_2.h
--- 5.0.3-1/include/CGAL/Segment_Delaunay_graph_vertex_base_2.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_Delaunay_graph_vertex_base_2.h	2020-09-08 12:15:34.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_vertex_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_vertex_base_2.h $
 // $Id: Segment_Delaunay_graph_vertex_base_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/segment_intersection_points_2.h 5.1-1/include/CGAL/segment_intersection_points_2.h
--- 5.0.3-1/include/CGAL/segment_intersection_points_2.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/segment_intersection_points_2.h	2020-09-08 12:14:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Convex_hull_2/include/CGAL/segment_intersection_points_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Convex_hull_2/include/CGAL/segment_intersection_points_2.h $
 // $Id: segment_intersection_points_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_tree_d.h 5.1-1/include/CGAL/Segment_tree_d.h
--- 5.0.3-1/include/CGAL/Segment_tree_d.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_tree_d.h	2020-09-08 12:15:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/SearchStructures/include/CGAL/Segment_tree_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/SearchStructures/include/CGAL/Segment_tree_d.h $
 // $Id: Segment_tree_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Segment_tree_k.h 5.1-1/include/CGAL/Segment_tree_k.h
--- 5.0.3-1/include/CGAL/Segment_tree_k.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Segment_tree_k.h	2020-09-08 12:15:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/SearchStructures/include/CGAL/Segment_tree_k.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/SearchStructures/include/CGAL/Segment_tree_k.h $
 // $Id: Segment_tree_k.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/SEP_header.h 5.1-1/include/CGAL/SEP_header.h
--- 5.0.3-1/include/CGAL/SEP_header.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/SEP_header.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/SEP_header.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/SEP_header.h $
 // $Id: SEP_header.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/SEP_to_ImageIO.h 5.1-1/include/CGAL/SEP_to_ImageIO.h
--- 5.0.3-1/include/CGAL/SEP_to_ImageIO.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/SEP_to_ImageIO.h	2020-09-08 12:14:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ImageIO/include/CGAL/SEP_to_ImageIO.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/CGAL_ImageIO/include/CGAL/SEP_to_ImageIO.h $
 // $Id: SEP_to_ImageIO.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Set_movable_separability_2/internal/Circle_arrangment.h 5.1-1/include/CGAL/Set_movable_separability_2/internal/Circle_arrangment.h
--- 5.0.3-1/include/CGAL/Set_movable_separability_2/internal/Circle_arrangment.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Set_movable_separability_2/internal/Circle_arrangment.h	2020-09-08 12:15:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Set_movable_separability_2/include/CGAL/Set_movable_separability_2/internal/Circle_arrangment.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Set_movable_separability_2/include/CGAL/Set_movable_separability_2/internal/Circle_arrangment.h $
 // $Id: Circle_arrangment.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Set_movable_separability_2/internal/Utils.h 5.1-1/include/CGAL/Set_movable_separability_2/internal/Utils.h
--- 5.0.3-1/include/CGAL/Set_movable_separability_2/internal/Utils.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Set_movable_separability_2/internal/Utils.h	2020-09-08 12:15:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Set_movable_separability_2/include/CGAL/Set_movable_separability_2/internal/Utils.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Set_movable_separability_2/include/CGAL/Set_movable_separability_2/internal/Utils.h $
 // $Id: Utils.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Set_movable_separability_2/Single_mold_translational_casting/is_pullout_direction.h 5.1-1/include/CGAL/Set_movable_separability_2/Single_mold_translational_casting/is_pullout_direction.h
--- 5.0.3-1/include/CGAL/Set_movable_separability_2/Single_mold_translational_casting/is_pullout_direction.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Set_movable_separability_2/Single_mold_translational_casting/is_pullout_direction.h	2020-09-08 12:15:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Set_movable_separability_2/include/CGAL/Set_movable_separability_2/Single_mold_translational_casting/is_pullout_direction.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Set_movable_separability_2/include/CGAL/Set_movable_separability_2/Single_mold_translational_casting/is_pullout_direction.h $
 // $Id: is_pullout_direction.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Set_movable_separability_2/Single_mold_translational_casting/pullout_directions.h 5.1-1/include/CGAL/Set_movable_separability_2/Single_mold_translational_casting/pullout_directions.h
--- 5.0.3-1/include/CGAL/Set_movable_separability_2/Single_mold_translational_casting/pullout_directions.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Set_movable_separability_2/Single_mold_translational_casting/pullout_directions.h	2020-09-08 12:15:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Set_movable_separability_2/include/CGAL/Set_movable_separability_2/Single_mold_translational_casting/pullout_directions.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Set_movable_separability_2/include/CGAL/Set_movable_separability_2/Single_mold_translational_casting/pullout_directions.h $
 // $Id: pullout_directions.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Set_movable_separability_2/Single_mold_translational_casting/top_edges.h 5.1-1/include/CGAL/Set_movable_separability_2/Single_mold_translational_casting/top_edges.h
--- 5.0.3-1/include/CGAL/Set_movable_separability_2/Single_mold_translational_casting/top_edges.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Set_movable_separability_2/Single_mold_translational_casting/top_edges.h	2020-09-08 12:15:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Set_movable_separability_2/include/CGAL/Set_movable_separability_2/Single_mold_translational_casting/top_edges.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Set_movable_separability_2/include/CGAL/Set_movable_separability_2/Single_mold_translational_casting/top_edges.h $
 // $Id: top_edges.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Shape_detection/deprecated/Region_growing.h 5.1-1/include/CGAL/Shape_detection/deprecated/Region_growing.h
--- 5.0.3-1/include/CGAL/Shape_detection/deprecated/Region_growing.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Shape_detection/deprecated/Region_growing.h	2020-09-08 12:15:38.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Shape_detection/include/CGAL/Shape_detection/deprecated/Region_growing.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Shape_detection/include/CGAL/Shape_detection/deprecated/Region_growing.h $
 // $Id: Region_growing.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Shape_detection/deprecated/Shape_detection_traits.h 5.1-1/include/CGAL/Shape_detection/deprecated/Shape_detection_traits.h
--- 5.0.3-1/include/CGAL/Shape_detection/deprecated/Shape_detection_traits.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Shape_detection/deprecated/Shape_detection_traits.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Shape_detection/include/CGAL/Shape_detection/deprecated/Shape_detection_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Shape_detection/include/CGAL/Shape_detection/deprecated/Shape_detection_traits.h $
 // $Id: Shape_detection_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Shape_detection/Efficient_RANSAC/Cone.h 5.1-1/include/CGAL/Shape_detection/Efficient_RANSAC/Cone.h
--- 5.0.3-1/include/CGAL/Shape_detection/Efficient_RANSAC/Cone.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Shape_detection/Efficient_RANSAC/Cone.h	2020-09-08 12:15:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Shape_detection/include/CGAL/Shape_detection/Efficient_RANSAC/Cone.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Shape_detection/include/CGAL/Shape_detection/Efficient_RANSAC/Cone.h $
 // $Id: Cone.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Shape_detection/Efficient_RANSAC/Cylinder.h 5.1-1/include/CGAL/Shape_detection/Efficient_RANSAC/Cylinder.h
--- 5.0.3-1/include/CGAL/Shape_detection/Efficient_RANSAC/Cylinder.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Shape_detection/Efficient_RANSAC/Cylinder.h	2020-09-08 12:15:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Shape_detection/include/CGAL/Shape_detection/Efficient_RANSAC/Cylinder.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Shape_detection/include/CGAL/Shape_detection/Efficient_RANSAC/Cylinder.h $
 // $Id: Cylinder.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Shape_detection/Efficient_RANSAC/Efficient_RANSAC.h 5.1-1/include/CGAL/Shape_detection/Efficient_RANSAC/Efficient_RANSAC.h
--- 5.0.3-1/include/CGAL/Shape_detection/Efficient_RANSAC/Efficient_RANSAC.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Shape_detection/Efficient_RANSAC/Efficient_RANSAC.h	2020-09-08 12:15:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Shape_detection/include/CGAL/Shape_detection/Efficient_RANSAC/Efficient_RANSAC.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Shape_detection/include/CGAL/Shape_detection/Efficient_RANSAC/Efficient_RANSAC.h $
 // $Id: Efficient_RANSAC.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Shape_detection/Efficient_RANSAC/Efficient_RANSAC_traits.h 5.1-1/include/CGAL/Shape_detection/Efficient_RANSAC/Efficient_RANSAC_traits.h
--- 5.0.3-1/include/CGAL/Shape_detection/Efficient_RANSAC/Efficient_RANSAC_traits.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Shape_detection/Efficient_RANSAC/Efficient_RANSAC_traits.h	2020-09-08 12:15:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Shape_detection/include/CGAL/Shape_detection/Efficient_RANSAC/Efficient_RANSAC_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Shape_detection/include/CGAL/Shape_detection/Efficient_RANSAC/Efficient_RANSAC_traits.h $
 // $Id: Efficient_RANSAC_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Shape_detection/Efficient_RANSAC/Octree.h 5.1-1/include/CGAL/Shape_detection/Efficient_RANSAC/Octree.h
--- 5.0.3-1/include/CGAL/Shape_detection/Efficient_RANSAC/Octree.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Shape_detection/Efficient_RANSAC/Octree.h	2020-09-08 12:15:37.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Shape_detection/include/CGAL/Shape_detection/Efficient_RANSAC/Octree.h $
-// $Id: Octree.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Shape_detection/include/CGAL/Shape_detection/Efficient_RANSAC/Octree.h $
+// $Id: Octree.h fdb17cb 2020-03-26T19:26:10+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -204,7 +204,7 @@ namespace CGAL {
       FT get_z(const Point_3& p){ return m_traits.compute_z_3_object()(p); }
       FT get_coord(const Point_3& p, unsigned int d)
       {
-        CGAL_assertion(d >= 0 && d < 3);
+        CGAL_assertion(d < 3);
         switch (d)
         {
           case 0: return get_x(p);
diff -pruN 5.0.3-1/include/CGAL/Shape_detection/Efficient_RANSAC/Plane.h 5.1-1/include/CGAL/Shape_detection/Efficient_RANSAC/Plane.h
--- 5.0.3-1/include/CGAL/Shape_detection/Efficient_RANSAC/Plane.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Shape_detection/Efficient_RANSAC/Plane.h	2020-09-08 12:15:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Shape_detection/include/CGAL/Shape_detection/Efficient_RANSAC/Plane.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Shape_detection/include/CGAL/Shape_detection/Efficient_RANSAC/Plane.h $
 // $Id: Plane.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Shape_detection/Efficient_RANSAC/property_map.h 5.1-1/include/CGAL/Shape_detection/Efficient_RANSAC/property_map.h
--- 5.0.3-1/include/CGAL/Shape_detection/Efficient_RANSAC/property_map.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Shape_detection/Efficient_RANSAC/property_map.h	2020-09-08 12:15:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Shape_detection/include/CGAL/Shape_detection/Efficient_RANSAC/property_map.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Shape_detection/include/CGAL/Shape_detection/Efficient_RANSAC/property_map.h $
 // $Id: property_map.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Shape_detection/Efficient_RANSAC/Shape_base.h 5.1-1/include/CGAL/Shape_detection/Efficient_RANSAC/Shape_base.h
--- 5.0.3-1/include/CGAL/Shape_detection/Efficient_RANSAC/Shape_base.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Shape_detection/Efficient_RANSAC/Shape_base.h	2020-09-08 12:15:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Shape_detection/include/CGAL/Shape_detection/Efficient_RANSAC/Shape_base.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Shape_detection/include/CGAL/Shape_detection/Efficient_RANSAC/Shape_base.h $
 // $Id: Shape_base.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Shape_detection/Efficient_RANSAC/Sphere.h 5.1-1/include/CGAL/Shape_detection/Efficient_RANSAC/Sphere.h
--- 5.0.3-1/include/CGAL/Shape_detection/Efficient_RANSAC/Sphere.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Shape_detection/Efficient_RANSAC/Sphere.h	2020-09-08 12:15:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Shape_detection/include/CGAL/Shape_detection/Efficient_RANSAC/Sphere.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Shape_detection/include/CGAL/Shape_detection/Efficient_RANSAC/Sphere.h $
 // $Id: Sphere.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Shape_detection/Efficient_RANSAC/Torus.h 5.1-1/include/CGAL/Shape_detection/Efficient_RANSAC/Torus.h
--- 5.0.3-1/include/CGAL/Shape_detection/Efficient_RANSAC/Torus.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Shape_detection/Efficient_RANSAC/Torus.h	2020-09-08 12:15:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Shape_detection/include/CGAL/Shape_detection/Efficient_RANSAC/Torus.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Shape_detection/include/CGAL/Shape_detection/Efficient_RANSAC/Torus.h $
 // $Id: Torus.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Shape_detection/Efficient_RANSAC.h 5.1-1/include/CGAL/Shape_detection/Efficient_RANSAC.h
--- 5.0.3-1/include/CGAL/Shape_detection/Efficient_RANSAC.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Shape_detection/Efficient_RANSAC.h	2020-09-08 12:15:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Shape_detection/include/CGAL/Shape_detection/Efficient_RANSAC.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Shape_detection/include/CGAL/Shape_detection/Efficient_RANSAC.h $
 // $Id: Efficient_RANSAC.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Shape_detection/Region_growing/internal/property_map.h 5.1-1/include/CGAL/Shape_detection/Region_growing/internal/property_map.h
--- 5.0.3-1/include/CGAL/Shape_detection/Region_growing/internal/property_map.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Shape_detection/Region_growing/internal/property_map.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Shape_detection/include/CGAL/Shape_detection/Region_growing/internal/property_map.h $
-// $Id: property_map.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Shape_detection/include/CGAL/Shape_detection/Region_growing/internal/property_map.h $
+// $Id: property_map.h 553d3c2 2020-04-10T13:03:50+02:00 Maxime Gimeno
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -51,8 +51,6 @@ namespace internal {
     { }
 
     reference operator[](const key_type item_index) const {
-
-      CGAL_precondition(item_index >= 0);
       CGAL_precondition(item_index < m_item_range.size());
 
       const auto& key = *(m_item_range.begin() + item_index);
diff -pruN 5.0.3-1/include/CGAL/Shape_detection/Region_growing/internal/utils.h 5.1-1/include/CGAL/Shape_detection/Region_growing/internal/utils.h
--- 5.0.3-1/include/CGAL/Shape_detection/Region_growing/internal/utils.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Shape_detection/Region_growing/internal/utils.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Shape_detection/include/CGAL/Shape_detection/Region_growing/internal/utils.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Shape_detection/include/CGAL/Shape_detection/Region_growing/internal/utils.h $
 // $Id: utils.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Shape_detection/Region_growing/Region_growing.h 5.1-1/include/CGAL/Shape_detection/Region_growing/Region_growing.h
--- 5.0.3-1/include/CGAL/Shape_detection/Region_growing/Region_growing.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Shape_detection/Region_growing/Region_growing.h	2020-09-08 12:15:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Shape_detection/include/CGAL/Shape_detection/Region_growing/Region_growing.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Shape_detection/include/CGAL/Shape_detection/Region_growing/Region_growing.h $
 // $Id: Region_growing.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/K_neighbor_query.h 5.1-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/K_neighbor_query.h
--- 5.0.3-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/K_neighbor_query.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/K_neighbor_query.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Shape_detection/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/K_neighbor_query.h $
-// $Id: K_neighbor_query.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Shape_detection/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/K_neighbor_query.h $
+// $Id: K_neighbor_query.h 553d3c2 2020-04-10T13:03:50+02:00 Maxime Gimeno
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -95,7 +95,7 @@ namespace Point_set {
     CGAL::Sliding_midpoint<Search_traits>;
 
     using Search_tree =
-    CGAL::Kd_tree<Search_traits, Splitter, CGAL::Tag_true>;
+      CGAL::Kd_tree<Search_traits, Splitter, CGAL::Tag_true, CGAL::Tag_true>;
 
     using Neighbor_search =
     CGAL::Orthogonal_k_neighbor_search<
@@ -171,7 +171,6 @@ namespace Point_set {
       const std::size_t query_index,
       std::vector<std::size_t>& neighbors) const {
 
-      CGAL_precondition(query_index >= 0);
       CGAL_precondition(query_index < m_input_range.size());
 
       Neighbor_search neighbor_search(
diff -pruN 5.0.3-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/Least_squares_line_fit_region.h 5.1-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/Least_squares_line_fit_region.h
--- 5.0.3-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/Least_squares_line_fit_region.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/Least_squares_line_fit_region.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Shape_detection/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/Least_squares_line_fit_region.h $
-// $Id: Least_squares_line_fit_region.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Shape_detection/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/Least_squares_line_fit_region.h $
+// $Id: Least_squares_line_fit_region.h 9a8b48c 2020-04-10T17:13:26+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -196,8 +196,6 @@ namespace Point_set {
       const std::size_t,
       const std::size_t query_index,
       const std::vector<std::size_t>&) const {
-
-      CGAL_precondition(query_index >= 0);
       CGAL_precondition(query_index < m_input_range.size());
 
       const auto& key = *(m_input_range.begin() + query_index);
@@ -246,8 +244,6 @@ namespace Point_set {
 
       CGAL_precondition(region.size() > 0);
       if (region.size() == 1) { // create new reference line and normal
-
-        CGAL_precondition(region[0] >= 0);
         CGAL_precondition(region[0] < m_input_range.size());
 
         // The best fit line will be a line through this point with
@@ -272,8 +268,6 @@ namespace Point_set {
         points.reserve(region.size());
 
         for (std::size_t i = 0; i < region.size(); ++i) {
-
-          CGAL_precondition(region[i] >= 0);
           CGAL_precondition(region[i] < m_input_range.size());
 
           const auto& key = *(m_input_range.begin() + region[i]);
diff -pruN 5.0.3-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/Least_squares_line_fit_sorting.h 5.1-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/Least_squares_line_fit_sorting.h
--- 5.0.3-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/Least_squares_line_fit_sorting.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/Least_squares_line_fit_sorting.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Shape_detection/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/Least_squares_line_fit_sorting.h $
-// $Id: Least_squares_line_fit_sorting.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Shape_detection/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/Least_squares_line_fit_sorting.h $
+// $Id: Least_squares_line_fit_sorting.h 553d3c2 2020-04-10T13:03:50+02:00 Maxime Gimeno
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -178,8 +178,6 @@ namespace Point_set {
 
         points.clear();
         for (std::size_t j = 0; j < neighbors.size(); ++j) {
-
-          CGAL_precondition(neighbors[j] >= 0);
           CGAL_precondition(neighbors[j] < m_input_range.size());
 
           const auto& key = *(m_input_range.begin() + neighbors[j]);
diff -pruN 5.0.3-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/Least_squares_plane_fit_region.h 5.1-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/Least_squares_plane_fit_region.h
--- 5.0.3-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/Least_squares_plane_fit_region.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/Least_squares_plane_fit_region.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Shape_detection/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/Least_squares_plane_fit_region.h $
-// $Id: Least_squares_plane_fit_region.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Shape_detection/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/Least_squares_plane_fit_region.h $
+// $Id: Least_squares_plane_fit_region.h 553d3c2 2020-04-10T13:03:50+02:00 Maxime Gimeno
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -197,7 +197,6 @@ namespace Point_set {
       const std::size_t query_index,
       const std::vector<std::size_t>&) const {
 
-      CGAL_precondition(query_index >= 0);
       CGAL_precondition(query_index < m_input_range.size());
 
       const auto& key = *(m_input_range.begin() + query_index);
@@ -246,8 +245,6 @@ namespace Point_set {
 
       CGAL_precondition(region.size() > 0);
       if (region.size() == 1) { // create new reference plane and normal
-
-        CGAL_precondition(region[0] >= 0);
         CGAL_precondition(region[0] < m_input_range.size());
 
         // The best fit plane will be a plane through this point with
@@ -272,8 +269,6 @@ namespace Point_set {
         points.reserve(region.size());
 
         for (std::size_t i = 0; i < region.size(); ++i) {
-
-          CGAL_precondition(region[i] >= 0);
           CGAL_precondition(region[i] < m_input_range.size());
 
           const auto& key = *(m_input_range.begin() + region[i]);
diff -pruN 5.0.3-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/Least_squares_plane_fit_sorting.h 5.1-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/Least_squares_plane_fit_sorting.h
--- 5.0.3-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/Least_squares_plane_fit_sorting.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/Least_squares_plane_fit_sorting.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Shape_detection/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/Least_squares_plane_fit_sorting.h $
-// $Id: Least_squares_plane_fit_sorting.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Shape_detection/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/Least_squares_plane_fit_sorting.h $
+// $Id: Least_squares_plane_fit_sorting.h d8789f0 2020-04-15T09:06:36+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -178,8 +178,6 @@ namespace Point_set {
 
         points.clear();
         for (std::size_t j = 0; j < neighbors.size(); ++j) {
-
-          CGAL_precondition(neighbors[j] >= 0);
           CGAL_precondition(neighbors[j] < m_input_range.size());
 
           const auto& key = *(m_input_range.begin() + neighbors[j]);
diff -pruN 5.0.3-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/Sphere_neighbor_query.h 5.1-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/Sphere_neighbor_query.h
--- 5.0.3-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/Sphere_neighbor_query.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/Sphere_neighbor_query.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Shape_detection/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/Sphere_neighbor_query.h $
-// $Id: Sphere_neighbor_query.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Shape_detection/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set/Sphere_neighbor_query.h $
+// $Id: Sphere_neighbor_query.h 553d3c2 2020-04-10T13:03:50+02:00 Maxime Gimeno
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -101,7 +101,7 @@ namespace Point_set {
     = CGAL::Fuzzy_sphere<Search_traits>;
 
     using Tree
-    = CGAL::Kd_tree<Search_traits, Splitter, CGAL::Tag_true>;
+    = CGAL::Kd_tree<Search_traits, Splitter, CGAL::Tag_true, CGAL::Tag_true>;
     /// \endcond
 
     /// @}
@@ -169,8 +169,6 @@ namespace Point_set {
     void operator()(
       const std::size_t query_index,
       std::vector<std::size_t>& neighbors) const {
-
-      CGAL_precondition(query_index >= 0);
       CGAL_precondition(query_index < m_input_range.size());
 
       const std::size_t sphere_center = query_index;
diff -pruN 5.0.3-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set.h 5.1-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set.h
--- 5.0.3-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Shape_detection/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Shape_detection/include/CGAL/Shape_detection/Region_growing/Region_growing_on_point_set.h $
 // $Id: Region_growing_on_point_set.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_polygon_mesh/Least_squares_plane_fit_region.h 5.1-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_polygon_mesh/Least_squares_plane_fit_region.h
--- 5.0.3-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_polygon_mesh/Least_squares_plane_fit_region.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_polygon_mesh/Least_squares_plane_fit_region.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Shape_detection/include/CGAL/Shape_detection/Region_growing/Region_growing_on_polygon_mesh/Least_squares_plane_fit_region.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Shape_detection/include/CGAL/Shape_detection/Region_growing/Region_growing_on_polygon_mesh/Least_squares_plane_fit_region.h $
 // $Id: Least_squares_plane_fit_region.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_polygon_mesh/Least_squares_plane_fit_sorting.h 5.1-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_polygon_mesh/Least_squares_plane_fit_sorting.h
--- 5.0.3-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_polygon_mesh/Least_squares_plane_fit_sorting.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_polygon_mesh/Least_squares_plane_fit_sorting.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Shape_detection/include/CGAL/Shape_detection/Region_growing/Region_growing_on_polygon_mesh/Least_squares_plane_fit_sorting.h $
-// $Id: Least_squares_plane_fit_sorting.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Shape_detection/include/CGAL/Shape_detection/Region_growing/Region_growing_on_polygon_mesh/Least_squares_plane_fit_sorting.h $
+// $Id: Least_squares_plane_fit_sorting.h d8789f0 2020-04-15T09:06:36+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -193,8 +193,6 @@ namespace Polygon_mesh {
 
         points.clear();
         for (std::size_t j = 0; j < neighbors.size(); ++j) {
-
-          CGAL_precondition(neighbors[j] >= 0);
           CGAL_precondition(neighbors[j] < m_face_range.size());
 
           const auto face = *(m_face_range.begin() + neighbors[j]);
diff -pruN 5.0.3-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_polygon_mesh/One_ring_neighbor_query.h 5.1-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_polygon_mesh/One_ring_neighbor_query.h
--- 5.0.3-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_polygon_mesh/One_ring_neighbor_query.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_polygon_mesh/One_ring_neighbor_query.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Shape_detection/include/CGAL/Shape_detection/Region_growing/Region_growing_on_polygon_mesh/One_ring_neighbor_query.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Shape_detection/include/CGAL/Shape_detection/Region_growing/Region_growing_on_polygon_mesh/One_ring_neighbor_query.h $
 // $Id: One_ring_neighbor_query.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_polygon_mesh.h 5.1-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_polygon_mesh.h
--- 5.0.3-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_polygon_mesh.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Shape_detection/Region_growing/Region_growing_on_polygon_mesh.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Shape_detection/include/CGAL/Shape_detection/Region_growing/Region_growing_on_polygon_mesh.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Shape_detection/include/CGAL/Shape_detection/Region_growing/Region_growing_on_polygon_mesh.h $
 // $Id: Region_growing_on_polygon_mesh.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Shape_detection/Region_growing.h 5.1-1/include/CGAL/Shape_detection/Region_growing.h
--- 5.0.3-1/include/CGAL/Shape_detection/Region_growing.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Shape_detection/Region_growing.h	2020-09-08 12:15:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Shape_detection/include/CGAL/Shape_detection/Region_growing.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Shape_detection/include/CGAL/Shape_detection/Region_growing.h $
 // $Id: Region_growing.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Shape_detection_3.h 5.1-1/include/CGAL/Shape_detection_3.h
--- 5.0.3-1/include/CGAL/Shape_detection_3.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Shape_detection_3.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Shape_detection/include/CGAL/Shape_detection_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Shape_detection/include/CGAL/Shape_detection_3.h $
 // $Id: Shape_detection_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Shape_detection.h 5.1-1/include/CGAL/Shape_detection.h
--- 5.0.3-1/include/CGAL/Shape_detection.h	2020-08-01 19:02:51.000000000 +0000
+++ 5.1-1/include/CGAL/Shape_detection.h	2020-09-08 12:15:37.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Shape_detection/include/CGAL/Shape_detection.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Shape_detection/include/CGAL/Shape_detection.h $
 // $Id: Shape_detection.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/sibson_gradient_fitting.h 5.1-1/include/CGAL/sibson_gradient_fitting.h
--- 5.0.3-1/include/CGAL/sibson_gradient_fitting.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/sibson_gradient_fitting.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Interpolation/include/CGAL/sibson_gradient_fitting.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Interpolation/include/CGAL/sibson_gradient_fitting.h $
 // $Id: sibson_gradient_fitting.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Side_of_bounded_square_2.h 5.1-1/include/CGAL/Side_of_bounded_square_2.h
--- 5.0.3-1/include/CGAL/Side_of_bounded_square_2.h	2020-08-01 19:02:50.000000000 +0000
+++ 5.1-1/include/CGAL/Side_of_bounded_square_2.h	2020-09-08 12:15:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_Linf_2/include/CGAL/Side_of_bounded_square_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_Linf_2/include/CGAL/Side_of_bounded_square_2.h $
 // $Id: Side_of_bounded_square_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Side_of_oriented_square_2.h 5.1-1/include/CGAL/Side_of_oriented_square_2.h
--- 5.0.3-1/include/CGAL/Side_of_oriented_square_2.h	2020-08-01 19:02:50.000000000 +0000
+++ 5.1-1/include/CGAL/Side_of_oriented_square_2.h	2020-09-08 12:15:36.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Segment_Delaunay_graph_Linf_2/include/CGAL/Side_of_oriented_square_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Segment_Delaunay_graph_Linf_2/include/CGAL/Side_of_oriented_square_2.h $
 // $Id: Side_of_oriented_square_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Side_of_triangle_mesh.h 5.1-1/include/CGAL/Side_of_triangle_mesh.h
--- 5.0.3-1/include/CGAL/Side_of_triangle_mesh.h	2020-08-01 19:02:41.000000000 +0000
+++ 5.1-1/include/CGAL/Side_of_triangle_mesh.h	2020-09-08 12:15:23.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polygon_mesh_processing/include/CGAL/Side_of_triangle_mesh.h $
-// $Id: Side_of_triangle_mesh.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polygon_mesh_processing/include/CGAL/Side_of_triangle_mesh.h $
+// $Id: Side_of_triangle_mesh.h f0c8dcb 2020-04-23T17:16:43+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -19,6 +19,7 @@
 #include <CGAL/disable_warnings.h>
 
 #include <CGAL/Polygon_mesh_processing/internal/Side_of_triangle_mesh/Point_inside_vertical_ray_cast.h>
+#include <CGAL/Polygon_mesh_processing/bbox.h>
 #include <CGAL/AABB_face_graph_triangle_primitive.h>
 
 #include <CGAL/AABB_tree.h>
@@ -66,7 +67,7 @@ namespace CGAL {
  */
 template <class TriangleMesh,
           class GeomTraits,
-          class VertexPointMap = Default
+          class VertexPointMap_ = Default
 #ifndef DOXYGEN_RUNNING
           , class AABBTree = Default
 #endif
@@ -76,25 +77,37 @@ class Side_of_triangle_mesh
   // typedefs
   template <typename TriangleMesh_,
             typename GeomTraits_,
-            typename VertexPointMap_>
+            typename VertexPointMap__>
   struct AABB_tree_default {
     typedef CGAL::AABB_face_graph_triangle_primitive<TriangleMesh_,
-                                                     VertexPointMap_> Primitive;
+                                                     VertexPointMap__> Primitive;
     typedef CGAL::AABB_traits<GeomTraits_, Primitive> Traits;
     typedef CGAL::AABB_tree<Traits> type;
   };
   typedef typename Default::Lazy_get<AABBTree,
                                      AABB_tree_default<TriangleMesh,
                                                        GeomTraits,
-                                                       VertexPointMap>
+                                                       VertexPointMap_>
                                      >::type AABB_tree_;
+  typedef typename Default::Get<VertexPointMap_,
+                                typename boost::property_map<TriangleMesh,
+                                                             vertex_point_t>::const_type>::type
+                                  VertexPointMap;
   typedef typename GeomTraits::Point_3 Point;
 
   //members
   typename GeomTraits::Construct_ray_3     ray_functor;
   typename GeomTraits::Construct_vector_3  vector_functor;
-  const AABB_tree_* tree_ptr;
+  const TriangleMesh* tm_ptr;
+  boost::optional<VertexPointMap> opt_vpm;
   bool own_tree;
+  CGAL::Bbox_3 box;
+#ifdef CGAL_HAS_THREADS
+  mutable CGAL_MUTEX tree_mutex;
+  mutable std::atomic<const AABB_tree_*> atomic_tree_ptr;
+#else
+  mutable const AABB_tree_* tree_ptr;
+#endif
 
 public:
 
@@ -118,14 +131,18 @@ public:
                         const GeomTraits& gt=GeomTraits())
   : ray_functor(gt.construct_ray_3_object())
   , vector_functor(gt.construct_vector_3_object())
+  , tm_ptr(&tmesh)
+  , opt_vpm(vpmap)
   , own_tree(true)
+#ifdef CGAL_HAS_THREADS
+  , atomic_tree_ptr(nullptr)
+#else
+  , tree_ptr(nullptr)
+#endif
   {
     CGAL_assertion(CGAL::is_triangle_mesh(tmesh));
     CGAL_assertion(CGAL::is_closed(tmesh));
-
-    tree_ptr = new AABB_tree(faces(tmesh).first,
-                             faces(tmesh).second,
-                             tmesh, vpmap);
+    box = Polygon_mesh_processing::bbox(tmesh, parameters::vertex_point_map(vpmap));
   }
 
   /**
@@ -138,17 +155,8 @@ public:
   */
   Side_of_triangle_mesh(const TriangleMesh& tmesh,
                         const GeomTraits& gt=GeomTraits())
-  : ray_functor(gt.construct_ray_3_object())
-  , vector_functor(gt.construct_vector_3_object())
-  , own_tree(true)
-  {
-    CGAL_assertion(CGAL::is_triangle_mesh(tmesh));
-    CGAL_assertion(CGAL::is_closed(tmesh));
-
-    tree_ptr = new AABB_tree(faces(tmesh).first,
-                             faces(tmesh).second,
-                             tmesh);
-  }
+  : Side_of_triangle_mesh(tmesh, get(vertex_point, tmesh), gt)
+  {}
 
   /**
   * Constructor that takes a pre-built \cgal `AABB_tree`
@@ -163,15 +171,24 @@ public:
                         const GeomTraits& gt = GeomTraits())
   : ray_functor(gt.construct_ray_3_object())
   , vector_functor(gt.construct_vector_3_object())
-  , tree_ptr(&tree)
   , own_tree(false)
+#ifdef CGAL_HAS_THREADS
+  , atomic_tree_ptr(&tree)
+#else
+  , tree_ptr(&tree)
+#endif
   {
+    box = tree.bbox();
   }
 
   ~Side_of_triangle_mesh()
   {
     if (own_tree)
+#ifdef CGAL_HAS_THREADS
+      delete atomic_tree_ptr.load();
+#else
       delete tree_ptr;
+#endif
   }
 
 public:
@@ -186,8 +203,43 @@ public:
    */
   Bounded_side operator()(const Point& point) const
   {
-    return internal::Point_inside_vertical_ray_cast<GeomTraits, AABB_tree>()(
-      point, *tree_ptr, ray_functor, vector_functor);
+    if(point.x() < box.xmin()
+       || point.x() > box.xmax()
+       || point.y() < box.ymin()
+       || point.y() > box.ymax()
+       || point.z() < box.zmin()
+       || point.z() > box.zmax())
+    {
+      return CGAL::ON_UNBOUNDED_SIDE;
+    }
+    else
+    {
+#ifdef CGAL_HAS_THREADS
+      AABB_tree_* tree_ptr =
+        const_cast<AABB_tree_*>(atomic_tree_ptr.load(std::memory_order_acquire));
+#endif
+      // Lazily build the tree only when needed
+      if (tree_ptr==nullptr)
+      {
+#ifdef CGAL_HAS_THREADS
+        CGAL_SCOPED_LOCK(tree_mutex);
+        tree_ptr = const_cast<AABB_tree_*>(atomic_tree_ptr.load(std::memory_order_relaxed));
+#endif
+        CGAL_assertion(tm_ptr != nullptr && opt_vpm!=boost::none);
+        if (tree_ptr==nullptr)
+        {
+          tree_ptr = new AABB_tree(faces(*tm_ptr).first,
+                                   faces(*tm_ptr).second,
+                                   *tm_ptr, *opt_vpm);
+          const_cast<AABB_tree_*>(tree_ptr)->build();
+#ifdef CGAL_HAS_THREADS
+        atomic_tree_ptr.store(tree_ptr, std::memory_order_release);
+#endif
+        }
+      }
+      return internal::Point_inside_vertical_ray_cast<GeomTraits, AABB_tree>()(
+            point, *tree_ptr, ray_functor, vector_functor);
+    }
   }
 
 };
diff -pruN 5.0.3-1/include/CGAL/Simple_cartesian.h 5.1-1/include/CGAL/Simple_cartesian.h
--- 5.0.3-1/include/CGAL/Simple_cartesian.h	2020-08-01 19:02:17.000000000 +0000
+++ 5.1-1/include/CGAL/Simple_cartesian.h	2020-09-08 12:14:45.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Cartesian_kernel/include/CGAL/Simple_cartesian.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Cartesian_kernel/include/CGAL/Simple_cartesian.h $
 // $Id: Simple_cartesian.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Simple_circular_kernel_2.h 5.1-1/include/CGAL/Simple_circular_kernel_2.h
--- 5.0.3-1/include/CGAL/Simple_circular_kernel_2.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Simple_circular_kernel_2.h	2020-09-08 12:14:46.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_2/include/CGAL/Simple_circular_kernel_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_2/include/CGAL/Simple_circular_kernel_2.h $
 // $Id: Simple_circular_kernel_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Simple_homogeneous.h 5.1-1/include/CGAL/Simple_homogeneous.h
--- 5.0.3-1/include/CGAL/Simple_homogeneous.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Simple_homogeneous.h	2020-09-08 12:14:54.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Homogeneous_kernel/include/CGAL/Simple_homogeneous.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Homogeneous_kernel/include/CGAL/Simple_homogeneous.h $
 // $Id: Simple_homogeneous.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Simple_polygon_visibility_2.h 5.1-1/include/CGAL/Simple_polygon_visibility_2.h
--- 5.0.3-1/include/CGAL/Simple_polygon_visibility_2.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Simple_polygon_visibility_2.h	2020-09-08 12:15:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Visibility_2/include/CGAL/Simple_polygon_visibility_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Visibility_2/include/CGAL/Simple_polygon_visibility_2.h $
 // $Id: Simple_polygon_visibility_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Simple_spherical_kernel_3.h 5.1-1/include/CGAL/Simple_spherical_kernel_3.h
--- 5.0.3-1/include/CGAL/Simple_spherical_kernel_3.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Simple_spherical_kernel_3.h	2020-09-08 12:14:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_3/include/CGAL/Simple_spherical_kernel_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_3/include/CGAL/Simple_spherical_kernel_3.h $
 // $Id: Simple_spherical_kernel_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/simplest_rational_in_interval.h 5.1-1/include/CGAL/simplest_rational_in_interval.h
--- 5.0.3-1/include/CGAL/simplest_rational_in_interval.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/simplest_rational_in_interval.h	2020-09-08 12:15:17.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/simplest_rational_in_interval.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/simplest_rational_in_interval.h $
 // $Id: simplest_rational_in_interval.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/simple_transformations_d.h 5.1-1/include/CGAL/simple_transformations_d.h
--- 5.0.3-1/include/CGAL/simple_transformations_d.h	2020-08-01 19:02:28.000000000 +0000
+++ 5.1-1/include/CGAL/simple_transformations_d.h	2020-09-08 12:15:04.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_d/include/CGAL/simple_transformations_d.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_d/include/CGAL/simple_transformations_d.h $
 // $Id: simple_transformations_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Sixtuple.h 5.1-1/include/CGAL/Sixtuple.h
--- 5.0.3-1/include/CGAL/Sixtuple.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Sixtuple.h	2020-09-08 12:15:32.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Sixtuple.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Sixtuple.h $
 // $Id: Sixtuple.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Skin_surface_3.h 5.1-1/include/CGAL/Skin_surface_3.h
--- 5.0.3-1/include/CGAL/Skin_surface_3.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Skin_surface_3.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Skin_surface_3/include/CGAL/Skin_surface_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Skin_surface_3/include/CGAL/Skin_surface_3.h $
 // $Id: Skin_surface_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Skin_surface_base_3.h 5.1-1/include/CGAL/Skin_surface_base_3.h
--- 5.0.3-1/include/CGAL/Skin_surface_base_3.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Skin_surface_base_3.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Skin_surface_3/include/CGAL/Skin_surface_base_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Skin_surface_3/include/CGAL/Skin_surface_base_3.h $
 // $Id: Skin_surface_base_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Skin_surface_filtered_traits_3.h 5.1-1/include/CGAL/Skin_surface_filtered_traits_3.h
--- 5.0.3-1/include/CGAL/Skin_surface_filtered_traits_3.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Skin_surface_filtered_traits_3.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Skin_surface_3/include/CGAL/Skin_surface_filtered_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Skin_surface_3/include/CGAL/Skin_surface_filtered_traits_3.h $
 // $Id: Skin_surface_filtered_traits_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Skin_surface_marching_tetrahedra_observer_3.h 5.1-1/include/CGAL/Skin_surface_marching_tetrahedra_observer_3.h
--- 5.0.3-1/include/CGAL/Skin_surface_marching_tetrahedra_observer_3.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Skin_surface_marching_tetrahedra_observer_3.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Skin_surface_3/include/CGAL/Skin_surface_marching_tetrahedra_observer_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Skin_surface_3/include/CGAL/Skin_surface_marching_tetrahedra_observer_3.h $
 // $Id: Skin_surface_marching_tetrahedra_observer_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Skin_surface_polyhedral_items_3.h 5.1-1/include/CGAL/Skin_surface_polyhedral_items_3.h
--- 5.0.3-1/include/CGAL/Skin_surface_polyhedral_items_3.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Skin_surface_polyhedral_items_3.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Skin_surface_3/include/CGAL/Skin_surface_polyhedral_items_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Skin_surface_3/include/CGAL/Skin_surface_polyhedral_items_3.h $
 // $Id: Skin_surface_polyhedral_items_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Skin_surface_polyhedral_items_with_face_information.h 5.1-1/include/CGAL/Skin_surface_polyhedral_items_with_face_information.h
--- 5.0.3-1/include/CGAL/Skin_surface_polyhedral_items_with_face_information.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Skin_surface_polyhedral_items_with_face_information.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Skin_surface_3/include/CGAL/Skin_surface_polyhedral_items_with_face_information.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Skin_surface_3/include/CGAL/Skin_surface_polyhedral_items_with_face_information.h $
 // $Id: Skin_surface_polyhedral_items_with_face_information.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Skin_surface_quadratic_surface_3.h 5.1-1/include/CGAL/Skin_surface_quadratic_surface_3.h
--- 5.0.3-1/include/CGAL/Skin_surface_quadratic_surface_3.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Skin_surface_quadratic_surface_3.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Skin_surface_3/include/CGAL/Skin_surface_quadratic_surface_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Skin_surface_3/include/CGAL/Skin_surface_quadratic_surface_3.h $
 // $Id: Skin_surface_quadratic_surface_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Skin_surface_refinement_policy_3.h 5.1-1/include/CGAL/Skin_surface_refinement_policy_3.h
--- 5.0.3-1/include/CGAL/Skin_surface_refinement_policy_3.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Skin_surface_refinement_policy_3.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Skin_surface_3/include/CGAL/Skin_surface_refinement_policy_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Skin_surface_3/include/CGAL/Skin_surface_refinement_policy_3.h $
 // $Id: Skin_surface_refinement_policy_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Skin_surface_traits_3.h 5.1-1/include/CGAL/Skin_surface_traits_3.h
--- 5.0.3-1/include/CGAL/Skin_surface_traits_3.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Skin_surface_traits_3.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Skin_surface_3/include/CGAL/Skin_surface_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Skin_surface_3/include/CGAL/Skin_surface_traits_3.h $
 // $Id: Skin_surface_traits_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Skiplist.h 5.1-1/include/CGAL/Skiplist.h
--- 5.0.3-1/include/CGAL/Skiplist.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Skiplist.h	2020-09-08 12:15:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Skiplist.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Skiplist.h $
 // $Id: Skiplist.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Small_side_angle_bisector_decomposition_2.h 5.1-1/include/CGAL/Small_side_angle_bisector_decomposition_2.h
--- 5.0.3-1/include/CGAL/Small_side_angle_bisector_decomposition_2.h	2020-08-01 19:02:33.000000000 +0000
+++ 5.1-1/include/CGAL/Small_side_angle_bisector_decomposition_2.h	2020-09-08 12:15:11.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Minkowski_sum_2/include/CGAL/Small_side_angle_bisector_decomposition_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Minkowski_sum_2/include/CGAL/Small_side_angle_bisector_decomposition_2.h $
 // $Id: Small_side_angle_bisector_decomposition_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Small_unordered_map.h 5.1-1/include/CGAL/Small_unordered_map.h
--- 5.0.3-1/include/CGAL/Small_unordered_map.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Small_unordered_map.h	2020-09-08 12:15:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Small_unordered_map.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Small_unordered_map.h $
 // $Id: Small_unordered_map.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Snap_rounding_2.h 5.1-1/include/CGAL/Snap_rounding_2.h
--- 5.0.3-1/include/CGAL/Snap_rounding_2.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Snap_rounding_2.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Snap_rounding_2/include/CGAL/Snap_rounding_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Snap_rounding_2/include/CGAL/Snap_rounding_2.h $
 // $Id: Snap_rounding_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Snap_rounding_kd_2.h 5.1-1/include/CGAL/Snap_rounding_kd_2.h
--- 5.0.3-1/include/CGAL/Snap_rounding_kd_2.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Snap_rounding_kd_2.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Snap_rounding_2/include/CGAL/Snap_rounding_kd_2.h $
-// $Id: Snap_rounding_kd_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Snap_rounding_2/include/CGAL/Snap_rounding_kd_2.h $
+// $Id: Snap_rounding_kd_2.h 3c7eb2c 2020-01-07T15:31:07+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -25,6 +25,7 @@
 #include <CGAL/utility.h>
 #include <CGAL/assertions.h>
 #include <CGAL/Dimension.h>
+#include <CGAL/number_type_config.h>
 
 #include <boost/type_traits/is_pointer.hpp>
 
@@ -394,7 +395,7 @@ public:
   Multiple_kd_tree(const Point_saved_pair_list & inp_points_list,
                    int inp_number_of_trees,
                    const Segment_list & seg_list) :
-    pi(3.1415), half_pi(1.57075),
+    pi(CGAL_PI), half_pi(0.5 * CGAL_PI),
     number_of_trees(inp_number_of_trees), input_points_list(inp_points_list)
   {
 
diff -pruN 5.0.3-1/include/CGAL/Snap_rounding_traits_2.h 5.1-1/include/CGAL/Snap_rounding_traits_2.h
--- 5.0.3-1/include/CGAL/Snap_rounding_traits_2.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Snap_rounding_traits_2.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Snap_rounding_2/include/CGAL/Snap_rounding_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Snap_rounding_2/include/CGAL/Snap_rounding_traits_2.h $
 // $Id: Snap_rounding_traits_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/sorted_matrix_search.h 5.1-1/include/CGAL/sorted_matrix_search.h
--- 5.0.3-1/include/CGAL/sorted_matrix_search.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/sorted_matrix_search.h	2020-09-08 12:15:06.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Matrix_search/include/CGAL/sorted_matrix_search.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Matrix_search/include/CGAL/sorted_matrix_search.h $
 // $Id: sorted_matrix_search.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Sorted_matrix_search_traits_adaptor.h 5.1-1/include/CGAL/Sorted_matrix_search_traits_adaptor.h
--- 5.0.3-1/include/CGAL/Sorted_matrix_search_traits_adaptor.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Sorted_matrix_search_traits_adaptor.h	2020-09-08 12:15:06.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Matrix_search/include/CGAL/Sorted_matrix_search_traits_adaptor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Matrix_search/include/CGAL/Sorted_matrix_search_traits_adaptor.h $
 // $Id: Sorted_matrix_search_traits_adaptor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Spatial_lock_grid_3.h 5.1-1/include/CGAL/Spatial_lock_grid_3.h
--- 5.0.3-1/include/CGAL/Spatial_lock_grid_3.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Spatial_lock_grid_3.h	2020-09-08 12:15:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Spatial_lock_grid_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Spatial_lock_grid_3.h $
 // $Id: Spatial_lock_grid_3.h 3560053 2020-01-21T10:05:19+01:00 Maxime Gimeno
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/spatial_sort.h 5.1-1/include/CGAL/spatial_sort.h
--- 5.0.3-1/include/CGAL/spatial_sort.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/spatial_sort.h	2020-09-08 12:15:40.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_sorting/include/CGAL/spatial_sort.h $
-// $Id: spatial_sort.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_sorting/include/CGAL/spatial_sort.h $
+// $Id: spatial_sort.h 7508a6f 2020-02-12T17:05:49+01:00 Laurent Rineau
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Christophe Delage
@@ -29,85 +29,78 @@ namespace CGAL {
 
 namespace internal {
 
-    template <class RandomAccessIterator, class Policy, class Kernel>
-    void spatial_sort (
-                       RandomAccessIterator begin, RandomAccessIterator end,
-                       const Kernel &k,
-                       Policy /*policy*/,
-                       typename Kernel::Point_2 *,
-                       std::ptrdiff_t threshold_hilbert,
-                       std::ptrdiff_t threshold_multiscale,
-                       double ratio)
-    {
-      typedef std::iterator_traits<RandomAccessIterator> Iterator_traits;
-      typedef typename Iterator_traits::difference_type Diff_t;
-      typedef Hilbert_sort_2<Kernel, Policy> Sort;
-        boost::rand48 random;
-        boost::random_number_generator<boost::rand48, Diff_t> rng(random);
-        CGAL::cpp98::random_shuffle(begin,end,rng);
-
-        if (threshold_hilbert==0) threshold_hilbert=4;
-        if (threshold_multiscale==0) threshold_multiscale=16;
-        if (ratio==0.0) ratio=0.25;
-
-        (Multiscale_sort<Sort> (Sort (k, threshold_hilbert),
-                                threshold_multiscale, ratio)) (begin, end);
-    }
-
-    template <class RandomAccessIterator, class Policy, class Kernel>
-    void spatial_sort (
-                       RandomAccessIterator begin, RandomAccessIterator end,
-                       const Kernel &k,
-                       Policy /*policy*/,
-                       typename Kernel::Point_3 *,
-                       std::ptrdiff_t threshold_hilbert,
-                       std::ptrdiff_t threshold_multiscale,
-                       double ratio)
-    {
-      typedef std::iterator_traits<RandomAccessIterator> Iterator_traits;
-      typedef typename Iterator_traits::difference_type Diff_t;
-      typedef Hilbert_sort_3<Kernel, Policy> Sort;
-        boost::rand48 random;
-        boost::random_number_generator<boost::rand48, Diff_t> rng(random);
-        CGAL::cpp98::random_shuffle(begin,end, rng);
-
-        if (threshold_hilbert==0) threshold_hilbert=8;
-        if (threshold_multiscale==0) threshold_multiscale=64;
-        if (ratio==0.0) ratio=0.125;
-
-        (Multiscale_sort<Sort> (Sort (k, threshold_hilbert),
-                                threshold_multiscale, ratio)) (begin, end);
-    }
-
-    template <class RandomAccessIterator, class Policy, class Kernel>
-    void spatial_sort (
-                       RandomAccessIterator begin, RandomAccessIterator end,
-                       const Kernel &k,
-                       Policy /*policy*/,
-                       typename Kernel::Point_d *,
-                       std::ptrdiff_t threshold_hilbert,
-                       std::ptrdiff_t threshold_multiscale,
-                       double ratio)
-    {
-      typedef std::iterator_traits<RandomAccessIterator> Iterator_traits;
-      typedef typename Iterator_traits::difference_type Diff_t;
-      typedef Hilbert_sort_d<Kernel, Policy> Sort;
-        boost::rand48 random;
-        boost::random_number_generator<boost::rand48, Diff_t> rng(random);
-        CGAL::cpp98::random_shuffle(begin,end, rng);
-
-        if (threshold_hilbert==0) threshold_hilbert=10;
-        if (threshold_multiscale==0) threshold_multiscale=500;
-        if (ratio==0.0) ratio=0.05;
-
-        (Multiscale_sort<Sort> (Sort (k, threshold_hilbert),
-                                threshold_multiscale, ratio)) (begin, end);
-    }
+template <class ConcurrencyTag = Sequential_tag, class RandomAccessIterator, class Policy, class Kernel>
+void spatial_sort (RandomAccessIterator begin, RandomAccessIterator end,
+                   const Kernel &k,
+                   Policy /*policy*/,
+                   typename Kernel::Point_2 *,
+                   std::ptrdiff_t threshold_hilbert,
+                   std::ptrdiff_t threshold_multiscale,
+                   double ratio)
+{
+  typedef std::iterator_traits<RandomAccessIterator> Iterator_traits;
+  typedef typename Iterator_traits::difference_type Diff_t;
+  typedef Hilbert_sort_2<Kernel, Policy, ConcurrencyTag> Sort;
+  boost::rand48 random;
+  boost::random_number_generator<boost::rand48, Diff_t> rng(random);
+  CGAL::cpp98::random_shuffle(begin,end,rng);
 
+  if (threshold_hilbert==0) threshold_hilbert=4;
+  if (threshold_multiscale==0) threshold_multiscale=16;
+  if (ratio==0.0) ratio=0.25;
+
+  (Multiscale_sort<Sort> (Sort (k, threshold_hilbert), threshold_multiscale, ratio)) (begin, end);
 }
 
+template <class ConcurrencyTag = Sequential_tag, class RandomAccessIterator, class Policy, class Kernel>
+void spatial_sort (RandomAccessIterator begin, RandomAccessIterator end,
+                   const Kernel &k,
+                   Policy /*policy*/,
+                   typename Kernel::Point_3 *,
+                   std::ptrdiff_t threshold_hilbert,
+                   std::ptrdiff_t threshold_multiscale,
+                   double ratio)
+{
+  typedef std::iterator_traits<RandomAccessIterator> Iterator_traits;
+  typedef typename Iterator_traits::difference_type Diff_t;
+  typedef Hilbert_sort_3<Kernel, Policy, ConcurrencyTag> Sort;
+  boost::rand48 random;
+  boost::random_number_generator<boost::rand48, Diff_t> rng(random);
+  CGAL::cpp98::random_shuffle(begin,end, rng);
+
+  if (threshold_hilbert==0) threshold_hilbert=8;
+  if (threshold_multiscale==0) threshold_multiscale=64;
+  if (ratio==0.0) ratio=0.125;
+
+  (Multiscale_sort<Sort> (Sort (k, threshold_hilbert), threshold_multiscale, ratio)) (begin, end);
+}
 
-template <class RandomAccessIterator, class Policy, class Kernel>
+template <class ConcurrencyTag = Sequential_tag, class RandomAccessIterator, class Policy, class Kernel>
+void spatial_sort (RandomAccessIterator begin, RandomAccessIterator end,
+                   const Kernel &k,
+                   Policy /*policy*/,
+                   typename Kernel::Point_d *,
+                   std::ptrdiff_t threshold_hilbert,
+                   std::ptrdiff_t threshold_multiscale,
+                   double ratio)
+{
+  typedef std::iterator_traits<RandomAccessIterator> Iterator_traits;
+  typedef typename Iterator_traits::difference_type Diff_t;
+  typedef Hilbert_sort_d<Kernel, Policy> Sort;
+  boost::rand48 random;
+  boost::random_number_generator<boost::rand48, Diff_t> rng(random);
+  CGAL::cpp98::random_shuffle(begin,end, rng);
+
+  if (threshold_hilbert==0) threshold_hilbert=10;
+  if (threshold_multiscale==0) threshold_multiscale=500;
+  if (ratio==0.0) ratio=0.05;
+
+  (Multiscale_sort<Sort> (Sort (k, threshold_hilbert), threshold_multiscale, ratio)) (begin, end);
+}
+
+} //namespace internal
+
+template <class ConcurrencyTag = Sequential_tag, class RandomAccessIterator, class Policy, class Kernel>
 void spatial_sort (RandomAccessIterator begin, RandomAccessIterator end,
                    const Kernel &k,
                    Policy policy,
@@ -115,66 +108,67 @@ void spatial_sort (RandomAccessIterator
                    std::ptrdiff_t threshold_multiscale=0,
                    double ratio=0.0)
 {
-    typedef std::iterator_traits<RandomAccessIterator> ITraits;
-    typedef typename ITraits::value_type               value_type;
+  typedef std::iterator_traits<RandomAccessIterator> ITraits;
+  typedef typename ITraits::value_type               value_type;
 
-    internal::spatial_sort(begin, end, k, policy, static_cast<value_type *> (0),
-                           threshold_hilbert,threshold_multiscale,ratio);
+  internal::spatial_sort<ConcurrencyTag>(begin, end, k, policy, static_cast<value_type *> (nullptr),
+                                         threshold_hilbert,threshold_multiscale,ratio);
 }
 
-template <class RandomAccessIterator>
+template <class ConcurrencyTag = Sequential_tag, class RandomAccessIterator>
 void spatial_sort (RandomAccessIterator begin, RandomAccessIterator end,
                    Hilbert_sort_median_policy policy,
                    std::ptrdiff_t threshold_hilbert=0,
                    std::ptrdiff_t threshold_multiscale=0,
                    double ratio=0.0)
 {
-    typedef std::iterator_traits<RandomAccessIterator> ITraits;
-    typedef typename ITraits::value_type               value_type;
-    typedef CGAL::Kernel_traits<value_type>            KTraits;
-    typedef typename KTraits::Kernel                   Kernel;
+  typedef std::iterator_traits<RandomAccessIterator> ITraits;
+  typedef typename ITraits::value_type               value_type;
+  typedef CGAL::Kernel_traits<value_type>            KTraits;
+  typedef typename KTraits::Kernel                   Kernel;
 
-    spatial_sort (begin, end, Kernel(), policy,
-                  threshold_hilbert,threshold_multiscale,ratio);
+  spatial_sort<ConcurrencyTag> (begin, end, Kernel(), policy,
+                                threshold_hilbert,threshold_multiscale,ratio);
 }
-template <class RandomAccessIterator>
+
+template <class ConcurrencyTag = Sequential_tag, class RandomAccessIterator>
 void spatial_sort (RandomAccessIterator begin, RandomAccessIterator end,
                    Hilbert_sort_middle_policy policy,
                    std::ptrdiff_t threshold_hilbert=0,
                    std::ptrdiff_t threshold_multiscale=0,
                    double ratio=0.0)
 {
-    typedef std::iterator_traits<RandomAccessIterator> ITraits;
-    typedef typename ITraits::value_type               value_type;
-    typedef CGAL::Kernel_traits<value_type>            KTraits;
-    typedef typename KTraits::Kernel                   Kernel;
+  typedef std::iterator_traits<RandomAccessIterator> ITraits;
+  typedef typename ITraits::value_type               value_type;
+  typedef CGAL::Kernel_traits<value_type>            KTraits;
+  typedef typename KTraits::Kernel                   Kernel;
 
-    spatial_sort (begin, end, Kernel(), policy,
-                  threshold_hilbert,threshold_multiscale,ratio);
+  spatial_sort<ConcurrencyTag> (begin, end, Kernel(), policy,
+                                threshold_hilbert,threshold_multiscale,ratio);
 }
 
 
-template <class RandomAccessIterator, class Kernel>
+template <class ConcurrencyTag = Sequential_tag, class RandomAccessIterator, class Kernel>
 void spatial_sort (RandomAccessIterator begin, RandomAccessIterator end,
                    const Kernel &k,
                    std::ptrdiff_t threshold_hilbert=0,
                    std::ptrdiff_t threshold_multiscale=0,
                    double ratio=0.0)
 {
-    spatial_sort (begin, end, k,
-                  Hilbert_sort_median_policy(),
-                  threshold_hilbert,threshold_multiscale,ratio);
+  spatial_sort<ConcurrencyTag> (begin, end, k,
+                                Hilbert_sort_median_policy(),
+                                threshold_hilbert,threshold_multiscale,ratio);
 }
 
-template <class RandomAccessIterator>
+template <class ConcurrencyTag = Sequential_tag, class RandomAccessIterator>
 void spatial_sort (RandomAccessIterator begin, RandomAccessIterator end,
                    std::ptrdiff_t threshold_hilbert=0,
                    std::ptrdiff_t threshold_multiscale=0,
                    double ratio=0.0)
 {
-    spatial_sort (begin, end,
-                  Hilbert_sort_median_policy(),
-                  threshold_hilbert,threshold_multiscale,ratio);
+  spatial_sort<ConcurrencyTag> (begin, end,
+                                Hilbert_sort_median_policy(),
+                                threshold_hilbert,threshold_multiscale,ratio);
 }
 
 } // namespace CGAL
diff -pruN 5.0.3-1/include/CGAL/spatial_sort_on_sphere.h 5.1-1/include/CGAL/spatial_sort_on_sphere.h
--- 5.0.3-1/include/CGAL/spatial_sort_on_sphere.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/spatial_sort_on_sphere.h	2020-09-08 12:15:40.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_sorting/include/CGAL/spatial_sort_on_sphere.h $
-// $Id: spatial_sort_on_sphere.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_sorting/include/CGAL/spatial_sort_on_sphere.h $
+// $Id: spatial_sort_on_sphere.h 5c41b10 2020-01-02T10:26:44+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Olivier Devillers
@@ -22,116 +22,115 @@ namespace CGAL {
 namespace internal {
 
 template <class RandomAccessIterator, class Policy, class Kernel>
-void spatial_sort_on_sphere (
-                   RandomAccessIterator begin, RandomAccessIterator end,
-                   const Kernel &k,
-                   Policy /*policy*/,
-                   typename Kernel::Point_3 *,
-                   double sq_r,
-                   const typename Kernel::Point_3 &p,
-                   std::ptrdiff_t threshold_hilbert,
-                   std::ptrdiff_t threshold_multiscale,
-                   double ratio)
+void spatial_sort_on_sphere (RandomAccessIterator begin, RandomAccessIterator end,
+                             const Kernel &k,
+                             Policy /*policy*/,
+                             typename Kernel::Point_3 *,
+                             double sq_r,
+                             const typename Kernel::Point_3 &p,
+                             std::ptrdiff_t threshold_hilbert,
+                             std::ptrdiff_t threshold_multiscale,
+                             double ratio)
 {
   typedef Hilbert_sort_on_sphere_3<Kernel, Policy> Sort;
-    typedef std::iterator_traits<RandomAccessIterator> ITraits;
-    typedef typename ITraits::difference_type Diff_t;
+  typedef std::iterator_traits<RandomAccessIterator> ITraits;
+  typedef typename ITraits::difference_type Diff_t;
 
-    boost::rand48 random;
-    boost::random_number_generator<boost::rand48, Diff_t> rng(random);
-    CGAL::cpp98::random_shuffle(begin,end, rng);
-
-            if (threshold_hilbert==0) threshold_hilbert=4;
-            if (threshold_multiscale==0) threshold_multiscale=16;
-            if (ratio==0.0) ratio=0.25;
+  boost::rand48 random;
+  boost::random_number_generator<boost::rand48, Diff_t> rng(random);
+  CGAL::cpp98::random_shuffle(begin,end, rng);
+
+  if (threshold_hilbert==0) threshold_hilbert=4;
+  if (threshold_multiscale==0) threshold_multiscale=16;
+  if (ratio==0.0) ratio=0.25;
 
-    (Multiscale_sort<Sort> (Sort (k, sq_r, p, threshold_hilbert),
-                            threshold_multiscale, ratio)) (begin, end);
+  (Multiscale_sort<Sort> (Sort (k, sq_r, p, threshold_hilbert),
+                          threshold_multiscale, ratio)) (begin, end);
 }
 
 } // end of namespace internal
 
 template <class RandomAccessIterator, class Policy, class Kernel>
 void spatial_sort_on_sphere (RandomAccessIterator begin, RandomAccessIterator end,
-                   const Kernel &k,
-                   Policy policy,
-                   double sq_r=1.0,
-                   const typename Kernel::Point_3 &p = typename Kernel::Point_3(0,0,0),
-                   std::ptrdiff_t threshold_hilbert=0,
-                   std::ptrdiff_t threshold_multiscale=0,
-                   double ratio=0.0)
+                             const Kernel &k,
+                             Policy policy,
+                             double sq_r=1.0,
+                             const typename Kernel::Point_3 &p = typename Kernel::Point_3(0,0,0),
+                             std::ptrdiff_t threshold_hilbert=0,
+                             std::ptrdiff_t threshold_multiscale=0,
+                             double ratio=0.0)
 {
-    typedef std::iterator_traits<RandomAccessIterator> ITraits;
-    typedef typename ITraits::value_type               value_type;
+  typedef std::iterator_traits<RandomAccessIterator> ITraits;
+  typedef typename ITraits::value_type               value_type;
 
-    internal::spatial_sort_on_sphere(begin, end, k, policy, static_cast<value_type *> (0),
-                           sq_r,p, threshold_hilbert,threshold_multiscale,ratio);
+  internal::spatial_sort_on_sphere(begin, end, k, policy, static_cast<value_type *> (0),
+                                   sq_r,p, threshold_hilbert,threshold_multiscale,ratio);
 }
 
 template <class RandomAccessIterator>
 void spatial_sort_on_sphere (RandomAccessIterator begin, RandomAccessIterator end,
-                   Hilbert_sort_median_policy policy,
-                   double sq_r=1.0,
-                   const typename CGAL::Kernel_traits<typename std::iterator_traits<RandomAccessIterator>::value_type>::Kernel::Point_3 &p =
-                     typename CGAL::Kernel_traits<typename std::iterator_traits<RandomAccessIterator>::value_type>::Kernel::Point_3(0,0,0),
-                   std::ptrdiff_t threshold_hilbert=0,
-                   std::ptrdiff_t threshold_multiscale=0,
-                   double ratio=0.0)
-{
-    typedef std::iterator_traits<RandomAccessIterator> ITraits;
-    typedef typename ITraits::value_type               value_type;
-    typedef CGAL::Kernel_traits<value_type>            KTraits;
-    typedef typename KTraits::Kernel                   Kernel;
+                             Hilbert_sort_median_policy policy,
+                             double sq_r=1.0,
+                             const typename CGAL::Kernel_traits<typename std::iterator_traits<RandomAccessIterator>::value_type>::Kernel::Point_3 &p =
+    typename CGAL::Kernel_traits<typename std::iterator_traits<RandomAccessIterator>::value_type>::Kernel::Point_3(0,0,0),
+                             std::ptrdiff_t threshold_hilbert=0,
+                             std::ptrdiff_t threshold_multiscale=0,
+                             double ratio=0.0)
+{
+  typedef std::iterator_traits<RandomAccessIterator> ITraits;
+  typedef typename ITraits::value_type               value_type;
+  typedef CGAL::Kernel_traits<value_type>            KTraits;
+  typedef typename KTraits::Kernel                   Kernel;
 
-    spatial_sort_on_sphere (begin, end, Kernel(), policy, sq_r, p,
-                  threshold_hilbert,threshold_multiscale,ratio);
+  spatial_sort_on_sphere (begin, end, Kernel(), policy, sq_r, p,
+                          threshold_hilbert,threshold_multiscale,ratio);
 }
 template <class RandomAccessIterator>
 void spatial_sort_on_sphere (RandomAccessIterator begin, RandomAccessIterator end,
-                   Hilbert_sort_middle_policy policy,
-                   double sq_r=1.0,
-                   const typename CGAL::Kernel_traits<typename std::iterator_traits<RandomAccessIterator>::value_type>::Kernel::Point_3 &p =
-                     typename CGAL::Kernel_traits<typename std::iterator_traits<RandomAccessIterator>::value_type>::Kernel::Point_3(0,0,0),
-                   std::ptrdiff_t threshold_hilbert=0,
-                   std::ptrdiff_t threshold_multiscale=0,
-                   double ratio=0.0)
-{
-    typedef std::iterator_traits<RandomAccessIterator> ITraits;
-    typedef typename ITraits::value_type               value_type;
-    typedef CGAL::Kernel_traits<value_type>            KTraits;
-    typedef typename KTraits::Kernel                   Kernel;
+                             Hilbert_sort_middle_policy policy,
+                             double sq_r=1.0,
+                             const typename CGAL::Kernel_traits<typename std::iterator_traits<RandomAccessIterator>::value_type>::Kernel::Point_3 &p =
+    typename CGAL::Kernel_traits<typename std::iterator_traits<RandomAccessIterator>::value_type>::Kernel::Point_3(0,0,0),
+                             std::ptrdiff_t threshold_hilbert=0,
+                             std::ptrdiff_t threshold_multiscale=0,
+                             double ratio=0.0)
+{
+  typedef std::iterator_traits<RandomAccessIterator> ITraits;
+  typedef typename ITraits::value_type               value_type;
+  typedef CGAL::Kernel_traits<value_type>            KTraits;
+  typedef typename KTraits::Kernel                   Kernel;
 
-    spatial_sort_on_sphere (begin, end, Kernel(), policy, sq_r,p,
-                  threshold_hilbert,threshold_multiscale,ratio);
+  spatial_sort_on_sphere (begin, end, Kernel(), policy, sq_r,p,
+                          threshold_hilbert,threshold_multiscale,ratio);
 }
 
 
 template <class RandomAccessIterator, class Kernel>
 void spatial_sort_on_sphere (RandomAccessIterator begin, RandomAccessIterator end,
-                   const Kernel &k,
-                   double sq_r = 1.0,
-           const typename Kernel::Point_3 &p = typename Kernel::Point_3(0,0,0),
-           std::ptrdiff_t threshold_hilbert=0,
-                   std::ptrdiff_t threshold_multiscale=0,
-                   double ratio=0.0)
-{
-    spatial_sort_on_sphere (begin, end, k,
-                  Hilbert_sort_median_policy(), sq_r, p,
-                  threshold_hilbert,threshold_multiscale,ratio);
+                             const Kernel &k,
+                             double sq_r = 1.0,
+                             const typename Kernel::Point_3 &p = typename Kernel::Point_3(0,0,0),
+                             std::ptrdiff_t threshold_hilbert=0,
+                             std::ptrdiff_t threshold_multiscale=0,
+                             double ratio=0.0)
+{
+  spatial_sort_on_sphere (begin, end, k,
+                          Hilbert_sort_median_policy(), sq_r, p,
+                          threshold_hilbert,threshold_multiscale,ratio);
 }
 
 template <class RandomAccessIterator>
 void spatial_sort_on_sphere (RandomAccessIterator begin, RandomAccessIterator end,
-           double sq_r = 1.0,
-                   const typename CGAL::Kernel_traits<typename std::iterator_traits<RandomAccessIterator>::value_type>::Kernel::Point_3 &p =
-               typename CGAL::Kernel_traits<typename std::iterator_traits<RandomAccessIterator>::value_type>::Kernel::Point_3(0,0,0),
-                   std::ptrdiff_t threshold_hilbert=0,
-                   std::ptrdiff_t threshold_multiscale=0,
-                   double ratio=0.0)
-{
-    spatial_sort_on_sphere (begin, end,
-                  Hilbert_sort_median_policy(), sq_r,p,
-                  threshold_hilbert,threshold_multiscale,ratio);
+                             double sq_r = 1.0,
+                             const typename CGAL::Kernel_traits<typename std::iterator_traits<RandomAccessIterator>::value_type>::Kernel::Point_3 &p =
+    typename CGAL::Kernel_traits<typename std::iterator_traits<RandomAccessIterator>::value_type>::Kernel::Point_3(0,0,0),
+                             std::ptrdiff_t threshold_hilbert=0,
+                             std::ptrdiff_t threshold_multiscale=0,
+                             double ratio=0.0)
+{
+  spatial_sort_on_sphere (begin, end,
+                          Hilbert_sort_median_policy(), sq_r,p,
+                          threshold_hilbert,threshold_multiscale,ratio);
 }
 
 } // end of namespace CGAL
diff -pruN 5.0.3-1/include/CGAL/Spatial_sort_traits_adapter_2.h 5.1-1/include/CGAL/Spatial_sort_traits_adapter_2.h
--- 5.0.3-1/include/CGAL/Spatial_sort_traits_adapter_2.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Spatial_sort_traits_adapter_2.h	2020-09-08 12:15:40.000000000 +0000
@@ -3,13 +3,12 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_sorting/include/CGAL/Spatial_sort_traits_adapter_2.h $
-// $Id: Spatial_sort_traits_adapter_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_sorting/include/CGAL/Spatial_sort_traits_adapter_2.h $
+// $Id: Spatial_sort_traits_adapter_2.h 5c41b10 2020-01-02T10:26:44+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Sebastien Loriot
 
-
 #ifndef CGAL_SPATIAL_SORT_TRAITS_ADAPTER_2_H
 #define CGAL_SPATIAL_SORT_TRAITS_ADAPTER_2_H
 
@@ -19,47 +18,51 @@
 
 #include <CGAL/property_map.h>
 
-
 namespace CGAL{
 
 using ::get;
 
-template<class Base_traits,class PointPropertyMap>
-class Spatial_sort_traits_adapter_2:public Base_traits{
+template<class Base_traits, class PointPropertyMap>
+class Spatial_sort_traits_adapter_2
+  : public Base_traits
+{
   PointPropertyMap ppmap_;
 public:
   Spatial_sort_traits_adapter_2(Base_traits base=Base_traits()):Base_traits(base){}
 
-  Spatial_sort_traits_adapter_2(const PointPropertyMap& ppmap,Base_traits base=Base_traits())
-  :Base_traits(base),ppmap_(ppmap){}
+  Spatial_sort_traits_adapter_2(const PointPropertyMap& ppmap, Base_traits base=Base_traits())
+    :Base_traits(base), ppmap_(ppmap){}
 
   typedef Base_traits Gt;
   typedef typename boost::property_traits<PointPropertyMap>::key_type Point_2;
   typedef typename boost::call_traits<Point_2>::param_type Arg_type;
 
-  struct Less_x_2 : public Base_traits::Less_x_2{
-    Less_x_2(const PointPropertyMap& ppmap,const typename Base_traits::Less_x_2& base):
-      Base_traits::Less_x_2(base),ppmap_(ppmap){}
+  struct Less_x_2
+    : public Base_traits::Less_x_2
+  {
+    Less_x_2(const PointPropertyMap& ppmap, const typename Base_traits::Less_x_2& base):
+      Base_traits::Less_x_2(base), ppmap_(ppmap){}
     const PointPropertyMap& ppmap_;
-    bool operator()(Arg_type p,Arg_type q) const {
-      return static_cast<const typename Base_traits::Less_x_2*>(this)->operator()(get(ppmap_,p),get(ppmap_,q));
+    bool operator()(Arg_type p, Arg_type q) const {
+      return static_cast<const typename Base_traits::Less_x_2*>(this)->operator()(get(ppmap_,p), get(ppmap_,q));
     }
   };
 
-  struct Less_y_2 : public Base_traits::Less_y_2{
-    Less_y_2(const PointPropertyMap& ppmap,const typename Base_traits::Less_y_2& base):
-      Base_traits::Less_y_2(base),ppmap_(ppmap){}
+  struct Less_y_2
+    : public Base_traits::Less_y_2
+  {
+    Less_y_2(const PointPropertyMap& ppmap, const typename Base_traits::Less_y_2& base):
+      Base_traits::Less_y_2(base), ppmap_(ppmap){}
     const PointPropertyMap& ppmap_;
-    bool operator()(Arg_type p,Arg_type q) const {
-      return static_cast<const typename Base_traits::Less_y_2*>(this)->operator()(get(ppmap_,p),get(ppmap_,q));
+    bool operator()(Arg_type p, Arg_type q) const {
+      return static_cast<const typename Base_traits::Less_y_2*>(this)->operator()(get(ppmap_,p), get(ppmap_,q));
     }
   };
 
-  Less_x_2 less_x_2_object () const {return Less_x_2(ppmap_,static_cast<const Gt*>(this)->less_x_2_object() );}
-  Less_y_2 less_y_2_object () const {return Less_y_2(ppmap_,static_cast<const Gt*>(this)->less_y_2_object() );}
+  Less_x_2 less_x_2_object () const {return Less_x_2(ppmap_, static_cast<const Gt*>(this)->less_x_2_object() );}
+  Less_y_2 less_y_2_object () const {return Less_y_2(ppmap_, static_cast<const Gt*>(this)->less_y_2_object() );}
 
   const PointPropertyMap& point_property_map() const {return ppmap_;}
-
 };
 
 } //namespace CGAL
diff -pruN 5.0.3-1/include/CGAL/Spatial_sort_traits_adapter_3.h 5.1-1/include/CGAL/Spatial_sort_traits_adapter_3.h
--- 5.0.3-1/include/CGAL/Spatial_sort_traits_adapter_3.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Spatial_sort_traits_adapter_3.h	2020-09-08 12:15:40.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_sorting/include/CGAL/Spatial_sort_traits_adapter_3.h $
-// $Id: Spatial_sort_traits_adapter_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_sorting/include/CGAL/Spatial_sort_traits_adapter_3.h $
+// $Id: Spatial_sort_traits_adapter_3.h 5c41b10 2020-01-02T10:26:44+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Sebastien Loriot
@@ -17,57 +17,63 @@
 
 #include <CGAL/property_map.h>
 
-
 namespace CGAL{
 
 using ::get;
 
-template<class Base_traits,class PointPropertyMap>
-class Spatial_sort_traits_adapter_3:public Base_traits{
+template<class Base_traits, class PointPropertyMap>
+class Spatial_sort_traits_adapter_3
+  : public Base_traits
+{
   PointPropertyMap ppmap_;
 public:
   Spatial_sort_traits_adapter_3(Base_traits base=Base_traits()):Base_traits(base){}
 
-  Spatial_sort_traits_adapter_3(const PointPropertyMap& ppmap,Base_traits base=Base_traits())
-  :Base_traits(base),ppmap_(ppmap){}
+  Spatial_sort_traits_adapter_3(const PointPropertyMap& ppmap, Base_traits base=Base_traits())
+    :Base_traits(base), ppmap_(ppmap){}
 
   typedef Base_traits Gt;
   typedef typename boost::property_traits<PointPropertyMap>::key_type Point_3;
   typedef typename boost::call_traits<Point_3>::param_type Arg_type;
 
-  struct Less_x_3 : public Base_traits::Less_x_3{
-    Less_x_3(const PointPropertyMap& ppmap,const typename Base_traits::Less_x_3& base):
-      Base_traits::Less_x_3(base),ppmap_(ppmap){}
+  struct Less_x_3
+    : public Base_traits::Less_x_3
+  {
+    Less_x_3(const PointPropertyMap& ppmap, const typename Base_traits::Less_x_3& base):
+      Base_traits::Less_x_3(base), ppmap_(ppmap){}
     const PointPropertyMap& ppmap_;
-    bool operator()(Arg_type p,Arg_type q) const {
-      return static_cast<const typename Base_traits::Less_x_3*>(this)->operator()(get(ppmap_,p),get(ppmap_,q));
+    bool operator()(Arg_type p, Arg_type q) const {
+      return static_cast<const typename Base_traits::Less_x_3*>(this)->operator()(get(ppmap_,p), get(ppmap_,q));
     }
   };
 
-  struct Less_y_3 : public Base_traits::Less_y_3{
-    Less_y_3(const PointPropertyMap& ppmap,const typename Base_traits::Less_y_3& base):
+  struct Less_y_3
+    : public Base_traits::Less_y_3
+  {
+    Less_y_3(const PointPropertyMap& ppmap, const typename Base_traits::Less_y_3& base):
       Base_traits::Less_y_3(base),ppmap_(ppmap){}
     const PointPropertyMap& ppmap_;
-    bool operator()(Arg_type p,Arg_type q) const {
-      return static_cast<const typename Base_traits::Less_y_3*>(this)->operator()(get(ppmap_,p),get(ppmap_,q));
+    bool operator()(Arg_type p, Arg_type q) const {
+      return static_cast<const typename Base_traits::Less_y_3*>(this)->operator()(get(ppmap_,p), get(ppmap_,q));
     }
   };
 
-  struct Less_z_3 : public Base_traits::Less_z_3{
-    Less_z_3(const PointPropertyMap& ppmap,const typename Base_traits::Less_z_3& base):
-      Base_traits::Less_z_3(base),ppmap_(ppmap){}
+  struct Less_z_3
+    : public Base_traits::Less_z_3
+  {
+    Less_z_3(const PointPropertyMap& ppmap, const typename Base_traits::Less_z_3& base):
+      Base_traits::Less_z_3(base), ppmap_(ppmap){}
     const PointPropertyMap& ppmap_;
-    bool operator()(Arg_type p,Arg_type q) const {
-      return static_cast<const typename Base_traits::Less_z_3*>(this)->operator()(get(ppmap_,p),get(ppmap_,q));
+    bool operator()(Arg_type p, Arg_type q) const {
+      return static_cast<const typename Base_traits::Less_z_3*>(this)->operator()(get(ppmap_,p), get(ppmap_,q));
     }
   };
 
-  Less_x_3 less_x_3_object () const {return Less_x_3(ppmap_,static_cast<const Gt*>(this)->less_x_3_object() );}
-  Less_y_3 less_y_3_object () const {return Less_y_3(ppmap_,static_cast<const Gt*>(this)->less_y_3_object() );}
-  Less_z_3 less_z_3_object () const {return Less_z_3(ppmap_,static_cast<const Gt*>(this)->less_z_3_object() );}
+  Less_x_3 less_x_3_object () const {return Less_x_3(ppmap_, static_cast<const Gt*>(this)->less_x_3_object() );}
+  Less_y_3 less_y_3_object () const {return Less_y_3(ppmap_, static_cast<const Gt*>(this)->less_y_3_object() );}
+  Less_z_3 less_z_3_object () const {return Less_z_3(ppmap_, static_cast<const Gt*>(this)->less_z_3_object() );}
 
   const PointPropertyMap& point_property_map() const {return ppmap_;}
-
 };
 
 } //namespace CGAL
diff -pruN 5.0.3-1/include/CGAL/Spatial_sort_traits_adapter_d.h 5.1-1/include/CGAL/Spatial_sort_traits_adapter_d.h
--- 5.0.3-1/include/CGAL/Spatial_sort_traits_adapter_d.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Spatial_sort_traits_adapter_d.h	2020-09-08 12:15:40.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_sorting/include/CGAL/Spatial_sort_traits_adapter_d.h $
-// $Id: Spatial_sort_traits_adapter_d.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_sorting/include/CGAL/Spatial_sort_traits_adapter_d.h $
+// $Id: Spatial_sort_traits_adapter_d.h 5c41b10 2020-01-02T10:26:44+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Sebastien Loriot
@@ -21,57 +21,59 @@ namespace CGAL{
 
 using ::get;
 
-template<class Base_traits,class PointPropertyMap>
-class Spatial_sort_traits_adapter_d:public Base_traits{
+template<class Base_traits, class PointPropertyMap>
+class Spatial_sort_traits_adapter_d
+  : public Base_traits
+{
   PointPropertyMap ppmap_;
 public:
-  Spatial_sort_traits_adapter_d(Base_traits base=Base_traits()):Base_traits(base){}
+  Spatial_sort_traits_adapter_d(Base_traits base=Base_traits()) : Base_traits(base){}
 
-  Spatial_sort_traits_adapter_d(const PointPropertyMap& ppmap,Base_traits base=Base_traits())
-  :Base_traits(base),ppmap_(ppmap){}
+  Spatial_sort_traits_adapter_d(const PointPropertyMap& ppmap, Base_traits base=Base_traits())
+    :Base_traits(base), ppmap_(ppmap){}
 
   typedef Base_traits Gt;
   typedef typename boost::property_traits<PointPropertyMap>::key_type Point_d;
   typedef typename boost::call_traits<Point_d>::param_type Arg_type;
 
-
-
-  struct Point_dimension_d: public Base_traits::Point_dimension_d{
-    Point_dimension_d(const PointPropertyMap& ppmap,const typename Base_traits::Point_dimension_d& base):
-      Base_traits::Point_dimension_d(base),ppmap_(ppmap){}
+  struct Point_dimension_d
+    : public Base_traits::Point_dimension_d
+  {
+    Point_dimension_d(const PointPropertyMap& ppmap, const typename Base_traits::Point_dimension_d& base):
+      Base_traits::Point_dimension_d(base), ppmap_(ppmap){}
     const PointPropertyMap& ppmap_;
     int operator()(Arg_type p) const {
-      return static_cast<const typename Base_traits::Point_dimension_d*>(this)->operator()(get(ppmap_,p));
+      return static_cast<const typename Base_traits::Point_dimension_d*>(this)->operator()(get(ppmap_, p));
     }
   };
 
-  struct Less_coordinate_d: public Base_traits::Less_coordinate_d{
-    Less_coordinate_d(const PointPropertyMap& ppmap,const typename Base_traits::Less_coordinate_d& base):
-      Base_traits::Less_coordinate_d(base),ppmap_(ppmap){}
+  struct Less_coordinate_d
+    : public Base_traits::Less_coordinate_d
+  {
+    Less_coordinate_d(const PointPropertyMap& ppmap, const typename Base_traits::Less_coordinate_d& base):
+      Base_traits::Less_coordinate_d(base), ppmap_(ppmap){}
     const PointPropertyMap& ppmap_;
-    bool operator()(Arg_type p,Arg_type q,int i) const {
-      return static_cast<const typename Base_traits::Less_coordinate_d*>(this)->operator()(get(ppmap_,p),get(ppmap_,q),i);
+    bool operator()(Arg_type p, Arg_type q, int i) const {
+      return static_cast<const typename Base_traits::Less_coordinate_d*>(this)->operator()(get(ppmap_,p), get(ppmap_,q), i);
     }
   };
 
-
-  struct Compute_coordinate_d: public Base_traits::Compute_coordinate_d{
-    Compute_coordinate_d(const PointPropertyMap& ppmap,const typename Base_traits::Compute_coordinate_d& base):
-      Base_traits::Compute_coordinate_d(base),ppmap_(ppmap){}
+  struct Compute_coordinate_d
+    : public Base_traits::Compute_coordinate_d
+  {
+    Compute_coordinate_d(const PointPropertyMap& ppmap, const typename Base_traits::Compute_coordinate_d& base):
+      Base_traits::Compute_coordinate_d(base), ppmap_(ppmap){}
     const PointPropertyMap& ppmap_;
-    bool operator()(Arg_type p,int i) const {
-      return static_cast<const typename Base_traits::Compute_coordinate_d*>(this)->operator()(get(ppmap_,p),i);
+    bool operator()(Arg_type p, int i) const {
+      return static_cast<const typename Base_traits::Compute_coordinate_d*>(this)->operator()(get(ppmap_,p), i);
     }
   };
 
-
-
-  Point_dimension_d point_dimension_d_object () const {return Point_dimension_d(ppmap_,static_cast<const Gt*>(this)->point_dimension_d_object() );}
-  Less_coordinate_d less_coordinate_d_object () const {return Less_coordinate_d(ppmap_,static_cast<const Gt*>(this)->less_coordinate_d_object() );}
-  Compute_coordinate_d compute_coordinate_d_object () const {return Compute_coordinate_d(ppmap_,static_cast<const Gt*>(this)->compute_coordinate_d_object() );}
+  Point_dimension_d point_dimension_d_object () const {return Point_dimension_d(ppmap_, static_cast<const Gt*>(this)->point_dimension_d_object() );}
+  Less_coordinate_d less_coordinate_d_object () const {return Less_coordinate_d(ppmap_, static_cast<const Gt*>(this)->less_coordinate_d_object() );}
+  Compute_coordinate_d compute_coordinate_d_object () const {return Compute_coordinate_d(ppmap_, static_cast<const Gt*>(this)->compute_coordinate_d_object() );}
 
   const PointPropertyMap& point_property_map() const {return ppmap_;}
-
 };
 
 } //namespace CGAL
diff -pruN 5.0.3-1/include/CGAL/Sphere_3.h 5.1-1/include/CGAL/Sphere_3.h
--- 5.0.3-1/include/CGAL/Sphere_3.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/Sphere_3.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Sphere_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Sphere_3.h $
 // $Id: Sphere_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Spherical_kernel_3.h 5.1-1/include/CGAL/Spherical_kernel_3.h
--- 5.0.3-1/include/CGAL/Spherical_kernel_3.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Spherical_kernel_3.h	2020-09-08 12:14:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_3/include/CGAL/Spherical_kernel_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_3/include/CGAL/Spherical_kernel_3.h $
 // $Id: Spherical_kernel_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Spherical_kernel_intersections.h 5.1-1/include/CGAL/Spherical_kernel_intersections.h
--- 5.0.3-1/include/CGAL/Spherical_kernel_intersections.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Spherical_kernel_intersections.h	2020-09-08 12:14:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_3/include/CGAL/Spherical_kernel_intersections.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_3/include/CGAL/Spherical_kernel_intersections.h $
 // $Id: Spherical_kernel_intersections.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Spherical_kernel_type_equality_wrapper.h 5.1-1/include/CGAL/Spherical_kernel_type_equality_wrapper.h
--- 5.0.3-1/include/CGAL/Spherical_kernel_type_equality_wrapper.h	2020-08-01 19:02:18.000000000 +0000
+++ 5.1-1/include/CGAL/Spherical_kernel_type_equality_wrapper.h	2020-09-08 12:14:47.000000000 +0000
@@ -9,7 +9,7 @@
 // and a STREP (FET Open) Project under Contract No  IST-006413
 // (ACS -- Algorithms for Complex Shapes)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Circular_kernel_3/include/CGAL/Spherical_kernel_type_equality_wrapper.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Circular_kernel_3/include/CGAL/Spherical_kernel_type_equality_wrapper.h $
 // $Id: Spherical_kernel_type_equality_wrapper.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Splitters.h 5.1-1/include/CGAL/Splitters.h
--- 5.0.3-1/include/CGAL/Splitters.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Splitters.h	2020-09-08 12:15:40.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_searching/include/CGAL/Splitters.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_searching/include/CGAL/Splitters.h $
 // $Id: Splitters.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Sqrt_extension/Algebraic_extension_traits.h 5.1-1/include/CGAL/Sqrt_extension/Algebraic_extension_traits.h
--- 5.0.3-1/include/CGAL/Sqrt_extension/Algebraic_extension_traits.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Sqrt_extension/Algebraic_extension_traits.h	2020-09-08 12:15:16.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Sqrt_extension/Algebraic_extension_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Sqrt_extension/Algebraic_extension_traits.h $
 // $Id: Algebraic_extension_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Sqrt_extension/Algebraic_structure_traits.h 5.1-1/include/CGAL/Sqrt_extension/Algebraic_structure_traits.h
--- 5.0.3-1/include/CGAL/Sqrt_extension/Algebraic_structure_traits.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Sqrt_extension/Algebraic_structure_traits.h	2020-09-08 12:15:16.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Sqrt_extension/Algebraic_structure_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Sqrt_extension/Algebraic_structure_traits.h $
 // $Id: Algebraic_structure_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Sqrt_extension/Chinese_remainder_traits.h 5.1-1/include/CGAL/Sqrt_extension/Chinese_remainder_traits.h
--- 5.0.3-1/include/CGAL/Sqrt_extension/Chinese_remainder_traits.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Sqrt_extension/Chinese_remainder_traits.h	2020-09-08 12:15:16.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Sqrt_extension/Chinese_remainder_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Sqrt_extension/Chinese_remainder_traits.h $
 // $Id: Chinese_remainder_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Sqrt_extension/Coercion_traits.h 5.1-1/include/CGAL/Sqrt_extension/Coercion_traits.h
--- 5.0.3-1/include/CGAL/Sqrt_extension/Coercion_traits.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Sqrt_extension/Coercion_traits.h	2020-09-08 12:15:16.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Sqrt_extension/Coercion_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Sqrt_extension/Coercion_traits.h $
 // $Id: Coercion_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Sqrt_extension/convert_to_bfi.h 5.1-1/include/CGAL/Sqrt_extension/convert_to_bfi.h
--- 5.0.3-1/include/CGAL/Sqrt_extension/convert_to_bfi.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Sqrt_extension/convert_to_bfi.h	2020-09-08 12:15:16.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Sqrt_extension/convert_to_bfi.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Sqrt_extension/convert_to_bfi.h $
 // $Id: convert_to_bfi.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Sqrt_extension/Eigen_NumTraits.h 5.1-1/include/CGAL/Sqrt_extension/Eigen_NumTraits.h
--- 5.0.3-1/include/CGAL/Sqrt_extension/Eigen_NumTraits.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Sqrt_extension/Eigen_NumTraits.h	2020-09-08 12:15:16.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Sqrt_extension/Eigen_NumTraits.h $
-// $Id: Eigen_NumTraits.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Sqrt_extension/Eigen_NumTraits.h $
+// $Id: Eigen_NumTraits.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -19,13 +19,6 @@ namespace Eigen {
   template <class NT,class ROOT, class ACDE_TAG, class FP_TAG>
   struct NumTraits<CGAL::Sqrt_extension<NT, ROOT, ACDE_TAG, FP_TAG> >
   {
-    typedef CGAL::Sqrt_extension<NT, ROOT, ACDE_TAG, FP_TAG> Real;
-    typedef Real NonInteger;
-    typedef Real Nested;
-    typedef Real Literal;
-
-    static inline Real epsilon() { return NumTraits<NT>::epsilon(); }
-
     enum {
       IsInteger = 0,
       IsSigned = 1,
@@ -35,6 +28,19 @@ namespace Eigen {
       AddCost = 2*NumTraits<NT>::AddCost+NumTraits<ROOT>::ReadCost,
       MulCost = 5*NumTraits<NT>::MulCost+2*NumTraits<NT>::AddCost
     };
+
+    typedef CGAL::Sqrt_extension<NT, ROOT, ACDE_TAG, FP_TAG> Real;
+    typedef Real NonInteger;
+    typedef Real Nested;
+    typedef Real Literal;
+
+    static inline Real epsilon() { return NumTraits<NT>::epsilon(); }
+    static inline int digits10() { return NumTraits<NT>::digits10(); }
+    static inline Real dummy_precision() { return NumTraits<NT>::dummy_precision(); }
+    static inline Real highest() { return NumTraits<NT>::highest(); }
+    static inline Real lowest() { return NumTraits<NT>::lowest(); }
+    static inline Real infinity() { return NumTraits<NT>::infinity(); }
+    static inline Real quiet_NaN() { return NumTraits<NT>::quiet_NaN(); }
   };
 }
 
diff -pruN 5.0.3-1/include/CGAL/Sqrt_extension/Fraction_traits.h 5.1-1/include/CGAL/Sqrt_extension/Fraction_traits.h
--- 5.0.3-1/include/CGAL/Sqrt_extension/Fraction_traits.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Sqrt_extension/Fraction_traits.h	2020-09-08 12:15:16.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Sqrt_extension/Fraction_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Sqrt_extension/Fraction_traits.h $
 // $Id: Fraction_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Sqrt_extension/Get_arithmetic_kernel.h 5.1-1/include/CGAL/Sqrt_extension/Get_arithmetic_kernel.h
--- 5.0.3-1/include/CGAL/Sqrt_extension/Get_arithmetic_kernel.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Sqrt_extension/Get_arithmetic_kernel.h	2020-09-08 12:15:16.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Sqrt_extension/Get_arithmetic_kernel.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Sqrt_extension/Get_arithmetic_kernel.h $
 // $Id: Get_arithmetic_kernel.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Sqrt_extension/io.h 5.1-1/include/CGAL/Sqrt_extension/io.h
--- 5.0.3-1/include/CGAL/Sqrt_extension/io.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Sqrt_extension/io.h	2020-09-08 12:15:16.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Sqrt_extension/io.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Sqrt_extension/io.h $
 // $Id: io.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Sqrt_extension/Modular_traits.h 5.1-1/include/CGAL/Sqrt_extension/Modular_traits.h
--- 5.0.3-1/include/CGAL/Sqrt_extension/Modular_traits.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Sqrt_extension/Modular_traits.h	2020-09-08 12:15:16.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Sqrt_extension/Modular_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Sqrt_extension/Modular_traits.h $
 // $Id: Modular_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Sqrt_extension/Real_embeddable_traits.h 5.1-1/include/CGAL/Sqrt_extension/Real_embeddable_traits.h
--- 5.0.3-1/include/CGAL/Sqrt_extension/Real_embeddable_traits.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Sqrt_extension/Real_embeddable_traits.h	2020-09-08 12:15:16.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Sqrt_extension/Real_embeddable_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Sqrt_extension/Real_embeddable_traits.h $
 // $Id: Real_embeddable_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Sqrt_extension/Scalar_factor_traits.h 5.1-1/include/CGAL/Sqrt_extension/Scalar_factor_traits.h
--- 5.0.3-1/include/CGAL/Sqrt_extension/Scalar_factor_traits.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Sqrt_extension/Scalar_factor_traits.h	2020-09-08 12:15:16.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Sqrt_extension/Scalar_factor_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Sqrt_extension/Scalar_factor_traits.h $
 // $Id: Scalar_factor_traits.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Sqrt_extension/Sqrt_extension_type.h 5.1-1/include/CGAL/Sqrt_extension/Sqrt_extension_type.h
--- 5.0.3-1/include/CGAL/Sqrt_extension/Sqrt_extension_type.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Sqrt_extension/Sqrt_extension_type.h	2020-09-08 12:15:16.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Sqrt_extension/Sqrt_extension_type.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Sqrt_extension/Sqrt_extension_type.h $
 // $Id: Sqrt_extension_type.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Sqrt_extension/Wang_traits.h 5.1-1/include/CGAL/Sqrt_extension/Wang_traits.h
--- 5.0.3-1/include/CGAL/Sqrt_extension/Wang_traits.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Sqrt_extension/Wang_traits.h	2020-09-08 12:15:16.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Sqrt_extension/Wang_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Sqrt_extension/Wang_traits.h $
 // $Id: Wang_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Sqrt_extension_fwd.h 5.1-1/include/CGAL/Sqrt_extension_fwd.h
--- 5.0.3-1/include/CGAL/Sqrt_extension_fwd.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Sqrt_extension_fwd.h	2020-09-08 12:15:16.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Sqrt_extension_fwd.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Sqrt_extension_fwd.h $
 // $Id: Sqrt_extension_fwd.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Sqrt_extension.h 5.1-1/include/CGAL/Sqrt_extension.h
--- 5.0.3-1/include/CGAL/Sqrt_extension.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/Sqrt_extension.h	2020-09-08 12:15:16.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/Sqrt_extension.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/Sqrt_extension.h $
 // $Id: Sqrt_extension.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/squared_distance_2_1.h 5.1-1/include/CGAL/squared_distance_2_1.h
--- 5.0.3-1/include/CGAL/squared_distance_2_1.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/squared_distance_2_1.h	2020-09-08 12:14:50.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Distance_2/include/CGAL/squared_distance_2_1.h $
-// $Id: squared_distance_2_1.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Distance_2/include/CGAL/squared_distance_2_1.h $
+// $Id: squared_distance_2_1.h 4527b1f 2020-03-26T19:01:49+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -188,7 +188,108 @@ namespace internal {
   typename K::FT
   squared_distance(const typename K::Segment_2 &seg1,
                    const typename K::Segment_2 &seg2,
-                   const K& k)
+                   const K& k,
+                   const Cartesian_tag&)
+  {
+    typedef typename K::RT RT;
+    typedef typename K::FT FT;
+    bool crossing1, crossing2;
+    RT c1s, c1e, c2s, c2e;
+    if (seg1.source() == seg1.target())
+      return internal::squared_distance(seg1.source(), seg2, k);
+    if (seg2.source() == seg2.target())
+      return internal::squared_distance(seg2.source(), seg1, k);
+
+    Orientation o1s = orientation(seg2.source(), seg2.target(), seg1.source());
+    Orientation o1e = orientation(seg2.source(), seg2.target(), seg1.target());
+    if (o1s == RIGHT_TURN) {
+      crossing1 = (o1e != RIGHT_TURN);
+    } else {
+      if (o1e != LEFT_TURN) {
+        if (o1s == COLLINEAR && o1e == COLLINEAR)
+          return internal::squared_distance_parallel(seg1, seg2, k);
+        crossing1 = true;
+      } else {
+        crossing1 = (o1s == COLLINEAR);
+      }
+    }
+
+    Orientation o2s = orientation(seg1.source(), seg1.target(), seg2.source());
+    Orientation o2e = orientation(seg1.source(), seg1.target(), seg2.target());
+    if (o2s == RIGHT_TURN) {
+      crossing2 = (o2e != RIGHT_TURN);
+    } else {
+      if (o2e != LEFT_TURN) {
+        if (o2s == COLLINEAR && o2e == COLLINEAR)
+          return internal::squared_distance_parallel(seg1, seg2, k);
+        crossing2 = true;
+      } else {
+        crossing2 = (o2s == COLLINEAR);
+      }
+    }
+
+    if (crossing1) {
+      if (crossing2)
+        return (FT)0;
+
+      c2s = CGAL::abs(wcross(seg1.source(), seg1.target(), seg2.source(), k));
+      c2e = CGAL::abs(wcross(seg1.source(), seg1.target(), seg2.target(), k));
+      Comparison_result dm = compare(c2s,c2e);
+
+      if (dm == SMALLER) {
+        return internal::squared_distance(seg2.source(), seg1, k);
+      } else {
+        if (dm == LARGER) {
+          return internal::squared_distance(seg2.target(), seg1, k);
+        } else {
+          // parallel, should not happen (no crossing)
+          return internal::squared_distance_parallel(seg1, seg2, k);
+        }
+      }
+    } else {
+      c1s = CGAL::abs(wcross(seg2.source(), seg2.target(), seg1.source(), k));
+      c1e = CGAL::abs(wcross(seg2.source(), seg2.target(), seg1.target(), k));
+      Comparison_result dm = compare(c1s,c1e);
+      if (crossing2) {
+        if (dm == SMALLER) {
+          return internal::squared_distance(seg1.source(), seg2, k);
+        } else {
+          if (dm == LARGER) {
+            return internal::squared_distance(seg1.target(), seg2, k);
+          } else {
+            // parallel, should not happen (no crossing)
+            return internal::squared_distance_parallel(seg1, seg2, k);
+          }
+        }
+      } else {
+        FT min1, min2;
+
+        if (dm == EQUAL)
+          return internal::squared_distance_parallel(seg1, seg2, k);
+        min1 = (dm == SMALLER) ?
+                 internal::squared_distance(seg1.source(), seg2, k):
+                 internal::squared_distance(seg1.target(), seg2, k);
+
+        c2s = CGAL::abs(wcross(seg1.source(), seg1.target(), seg2.source(), k));
+        c2e = CGAL::abs(wcross(seg1.source(), seg1.target(), seg2.target(), k));
+        dm = compare(c2s,c2e);
+
+        if (dm == EQUAL)  // should not happen.
+          return internal::squared_distance_parallel(seg1, seg2, k);
+        min2 = (dm == SMALLER) ?
+                 internal::squared_distance(seg2.source(), seg1, k):
+                 internal::squared_distance(seg2.target(), seg1, k);
+        return (min1 < min2) ? min1 : min2;
+      }
+    }
+  }
+
+  template <class K>
+  typename K::FT
+  squared_distance(const typename K::Segment_2 &seg1,
+                   const typename K::Segment_2 &seg2,
+                   const K& k,
+                   const Homogeneous_tag&)
   {
     typedef typename K::RT RT;
     typedef typename K::FT FT;
@@ -277,6 +378,7 @@ namespace internal {
     }
   }
 
+
   template <class K>
   inline typename K::RT
   _distance_measure_sub(const typename K::RT &startwcross,
@@ -690,7 +792,8 @@ template <class K>
 inline typename K::FT
 squared_distance(const Segment_2<K> &seg1, const Segment_2<K> &seg2)
 {
-  return internal::squared_distance(seg1, seg2, K());
+  typedef typename K::Kernel_tag Tag;
+  return internal::squared_distance(seg1, seg2, K(), Tag());
 }
 
 template <class K>
diff -pruN 5.0.3-1/include/CGAL/squared_distance_2_2.h 5.1-1/include/CGAL/squared_distance_2_2.h
--- 5.0.3-1/include/CGAL/squared_distance_2_2.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/squared_distance_2_2.h	2020-09-08 12:14:50.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Distance_2/include/CGAL/squared_distance_2_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Distance_2/include/CGAL/squared_distance_2_2.h $
 // $Id: squared_distance_2_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/squared_distance_2.h 5.1-1/include/CGAL/squared_distance_2.h
--- 5.0.3-1/include/CGAL/squared_distance_2.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/squared_distance_2.h	2020-09-08 12:14:50.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Distance_2/include/CGAL/squared_distance_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Distance_2/include/CGAL/squared_distance_2.h $
 // $Id: squared_distance_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/squared_distance_3_0.h 5.1-1/include/CGAL/squared_distance_3_0.h
--- 5.0.3-1/include/CGAL/squared_distance_3_0.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/squared_distance_3_0.h	2020-09-08 12:14:50.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Distance_3/include/CGAL/squared_distance_3_0.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Distance_3/include/CGAL/squared_distance_3_0.h $
 // $Id: squared_distance_3_0.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/squared_distance_3_1.h 5.1-1/include/CGAL/squared_distance_3_1.h
--- 5.0.3-1/include/CGAL/squared_distance_3_1.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/squared_distance_3_1.h	2020-09-08 12:14:50.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Distance_3/include/CGAL/squared_distance_3_1.h $
-// $Id: squared_distance_3_1.h f483506 2020-06-23T18:47:55+02:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Distance_3/include/CGAL/squared_distance_3_1.h $
+// $Id: squared_distance_3_1.h 0c94ff4 2020-06-23T18:49:46+02:00 Laurent Rineau
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
diff -pruN 5.0.3-1/include/CGAL/squared_distance_3_2.h 5.1-1/include/CGAL/squared_distance_3_2.h
--- 5.0.3-1/include/CGAL/squared_distance_3_2.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/squared_distance_3_2.h	2020-09-08 12:14:50.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Distance_3/include/CGAL/squared_distance_3_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Distance_3/include/CGAL/squared_distance_3_2.h $
 // $Id: squared_distance_3_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/squared_distance_3.h 5.1-1/include/CGAL/squared_distance_3.h
--- 5.0.3-1/include/CGAL/squared_distance_3.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/squared_distance_3.h	2020-09-08 12:14:50.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Distance_3/include/CGAL/squared_distance_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Distance_3/include/CGAL/squared_distance_3.h $
 // $Id: squared_distance_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/squared_distance_utils.h 5.1-1/include/CGAL/squared_distance_utils.h
--- 5.0.3-1/include/CGAL/squared_distance_utils.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/squared_distance_utils.h	2020-09-08 12:14:50.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Distance_2/include/CGAL/squared_distance_utils.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Distance_2/include/CGAL/squared_distance_utils.h $
 // $Id: squared_distance_utils.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/sse2.h 5.1-1/include/CGAL/sse2.h
--- 5.0.3-1/include/CGAL/sse2.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/sse2.h	2020-09-08 12:15:17.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/sse2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/sse2.h $
 // $Id: sse2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Static_filtered_predicate.h 5.1-1/include/CGAL/Static_filtered_predicate.h
--- 5.0.3-1/include/CGAL/Static_filtered_predicate.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/Static_filtered_predicate.h	2020-09-08 12:14:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Filtered_kernel/include/CGAL/Static_filtered_predicate.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Filtered_kernel/include/CGAL/Static_filtered_predicate.h $
 // $Id: Static_filtered_predicate.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/stddef.h 5.1-1/include/CGAL/stddef.h
--- 5.0.3-1/include/CGAL/stddef.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/stddef.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/stddef.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/stddef.h $
 // $Id: stddef.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 
diff -pruN 5.0.3-1/include/CGAL/Straight_skeleton_2/assertions.h 5.1-1/include/CGAL/Straight_skeleton_2/assertions.h
--- 5.0.3-1/include/CGAL/Straight_skeleton_2/assertions.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Straight_skeleton_2/assertions.h	2020-09-08 12:15:41.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Straight_skeleton_2/include/CGAL/Straight_skeleton_2/assertions.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Straight_skeleton_2/include/CGAL/Straight_skeleton_2/assertions.h $
 // $Id: assertions.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Straight_skeleton_2/debug.h 5.1-1/include/CGAL/Straight_skeleton_2/debug.h
--- 5.0.3-1/include/CGAL/Straight_skeleton_2/debug.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Straight_skeleton_2/debug.h	2020-09-08 12:15:41.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Straight_skeleton_2/include/CGAL/Straight_skeleton_2/debug.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Straight_skeleton_2/include/CGAL/Straight_skeleton_2/debug.h $
 // $Id: debug.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Straight_skeleton_2/Polygon_offset_builder_2_impl.h 5.1-1/include/CGAL/Straight_skeleton_2/Polygon_offset_builder_2_impl.h
--- 5.0.3-1/include/CGAL/Straight_skeleton_2/Polygon_offset_builder_2_impl.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Straight_skeleton_2/Polygon_offset_builder_2_impl.h	2020-09-08 12:15:41.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Straight_skeleton_2/include/CGAL/Straight_skeleton_2/Polygon_offset_builder_2_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Straight_skeleton_2/include/CGAL/Straight_skeleton_2/Polygon_offset_builder_2_impl.h $
 // $Id: Polygon_offset_builder_2_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Straight_skeleton_2/Straight_skeleton_aux.h 5.1-1/include/CGAL/Straight_skeleton_2/Straight_skeleton_aux.h
--- 5.0.3-1/include/CGAL/Straight_skeleton_2/Straight_skeleton_aux.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Straight_skeleton_2/Straight_skeleton_aux.h	2020-09-08 12:15:41.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Straight_skeleton_2/include/CGAL/Straight_skeleton_2/Straight_skeleton_aux.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Straight_skeleton_2/include/CGAL/Straight_skeleton_2/Straight_skeleton_aux.h $
 // $Id: Straight_skeleton_aux.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Straight_skeleton_2/Straight_skeleton_builder_2_impl.h 5.1-1/include/CGAL/Straight_skeleton_2/Straight_skeleton_builder_2_impl.h
--- 5.0.3-1/include/CGAL/Straight_skeleton_2/Straight_skeleton_builder_2_impl.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Straight_skeleton_2/Straight_skeleton_builder_2_impl.h	2020-09-08 12:15:41.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Straight_skeleton_2/include/CGAL/Straight_skeleton_2/Straight_skeleton_builder_2_impl.h $
-// $Id: Straight_skeleton_builder_2_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Straight_skeleton_2/include/CGAL/Straight_skeleton_2/Straight_skeleton_builder_2_impl.h $
+// $Id: Straight_skeleton_builder_2_impl.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Fernando Cacciola <fernando_cacciola@ciudad.com.ar>
@@ -22,9 +22,10 @@
 #include <boost/type_traits/detail/ice_not.hpp>
 #endif
 #include <boost/graph/adjacency_matrix.hpp>
-#include <CGAL/Unique_hash_map.h>
 
+#include <CGAL/number_type_config.h>
 #include <CGAL/Real_timer.h>
+#include <CGAL/Unique_hash_map.h>
 
 #if defined(BOOST_MSVC)
 #  pragma warning(push)
@@ -1421,8 +1422,8 @@ double angle_wrt_X ( Point const& a, Poi
   double dx = to_double(b.x() - a.x() ) ;
   double dy = to_double(b.y() - a.y() ) ;
   double atan = std::atan2(dy,dx);
-  double rad  = atan >= 0.0 ? atan : 2.0 * 3.141592 + atan ;
-  double deg  = rad * 180.0 / 3.141592 ;
+  double rad  = atan >= 0.0 ? atan : 2.0 * CGAL_PI + atan ;
+  double deg  = rad * 180.0 / CGAL_PI;
   return deg ;
 }
 
diff -pruN 5.0.3-1/include/CGAL/Straight_skeleton_2/Straight_skeleton_builder_events_2.h 5.1-1/include/CGAL/Straight_skeleton_2/Straight_skeleton_builder_events_2.h
--- 5.0.3-1/include/CGAL/Straight_skeleton_2/Straight_skeleton_builder_events_2.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Straight_skeleton_2/Straight_skeleton_builder_events_2.h	2020-09-08 12:15:41.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Straight_skeleton_2/include/CGAL/Straight_skeleton_2/Straight_skeleton_builder_events_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Straight_skeleton_2/include/CGAL/Straight_skeleton_2/Straight_skeleton_builder_events_2.h $
 // $Id: Straight_skeleton_builder_events_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Straight_skeleton_2/Straight_skeleton_builder_traits_2_aux.h 5.1-1/include/CGAL/Straight_skeleton_2/Straight_skeleton_builder_traits_2_aux.h
--- 5.0.3-1/include/CGAL/Straight_skeleton_2/Straight_skeleton_builder_traits_2_aux.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Straight_skeleton_2/Straight_skeleton_builder_traits_2_aux.h	2020-09-08 12:15:41.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Straight_skeleton_2/include/CGAL/Straight_skeleton_2/Straight_skeleton_builder_traits_2_aux.h $
-// $Id: Straight_skeleton_builder_traits_2_aux.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Straight_skeleton_2/include/CGAL/Straight_skeleton_2/Straight_skeleton_builder_traits_2_aux.h $
+// $Id: Straight_skeleton_builder_traits_2_aux.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Fernando Cacciola <fernando_cacciola@ciudad.com.ar>
@@ -106,98 +106,23 @@ public:
 
   Exceptionless_filtered_construction() {}
 
-  template <class A1>
+  template <class ... A>
   result_type
-  operator()(const A1 &a1) const
+  operator()(A&& ... a) const
   {
     try
     {
       Protect_FPU_rounding<Protection> P;
-      FC_result_type fr = Filter_construction(To_Filtered(a1));
+      FC_result_type fr = Filter_construction(To_Filtered(std::forward<A>(a))...);
       if ( fr )
         return From_Filtered(fr);
     }
     catch (Uncertain_conversion_exception&) {}
 
     Protect_FPU_rounding<!Protection> P(CGAL_FE_TONEAREST);
-    EC_result_type er = Exact_construction(To_Exact(a1)) ;
+    EC_result_type er = Exact_construction(To_Exact(std::forward<A>(a))...) ;
     return From_Exact(er);
   }
-
-  template <class A1, class A2>
-  result_type
-  operator()(const A1 &a1, const A2 &a2) const
-  {
-    try
-    {
-      Protect_FPU_rounding<Protection> P;
-      FC_result_type fr = Filter_construction(To_Filtered(a1),To_Filtered(a2));
-      if ( fr )
-        return From_Filtered(fr);
-    }
-    catch (Uncertain_conversion_exception&) {}
-
-    Protect_FPU_rounding<!Protection> P(CGAL_FE_TONEAREST);
-    EC_result_type er = Exact_construction(To_Exact(a1), To_Exact(a2)) ;
-    return From_Exact(er);
-  }
-
-  template <class A1, class A2, class A3>
-  result_type
-  operator()(const A1 &a1, const A2 &a2, const A3 &a3) const
-  {
-    try
-    {
-      Protect_FPU_rounding<Protection> P;
-      FC_result_type fr = Filter_construction(To_Filtered(a1),To_Filtered(a2),To_Filtered(a3));
-      if ( fr )
-        return From_Filtered(fr);
-    }
-    catch (Uncertain_conversion_exception&) {}
-
-    Protect_FPU_rounding<!Protection> P(CGAL_FE_TONEAREST);
-    EC_result_type er = Exact_construction(To_Exact(a1), To_Exact(a2), To_Exact(a3)) ;
-    return From_Exact(er);
-
-  }
-
-  template <class A1, class A2, class A3, class A4>
-  result_type
-  operator()(const A1 &a1, const A2 &a2, const A3 &a3, const A4 &a4) const
-  {
-    try
-    {
-      Protect_FPU_rounding<Protection> P;
-      FC_result_type fr = Filter_construction(To_Filtered(a1),To_Filtered(a2),To_Filtered(a3),To_Filtered(a4));
-      if ( fr )
-        return From_Filtered(fr);
-    }
-    catch (Uncertain_conversion_exception&) {}
-
-    Protect_FPU_rounding<!Protection> P(CGAL_FE_TONEAREST);
-    EC_result_type er = Exact_construction(To_Exact(a1), To_Exact(a2), To_Exact(a3), To_Exact(a4)) ;
-    return From_Exact(er);
-
-  }
-
-  template <class A1, class A2, class A3, class A4, class A5>
-  result_type
-  operator()(const A1 &a1, const A2 &a2, const A3 &a3, const A4 &a4, const A5 &a5) const
-  {
-    try
-    {
-      Protect_FPU_rounding<Protection> P;
-      FC_result_type fr = Filter_construction(To_Filtered(a1),To_Filtered(a2),To_Filtered(a3),To_Filtered(a4),To_Filtered(a5));
-      if ( fr )
-        return From_Filtered(fr);
-    }
-    catch (Uncertain_conversion_exception&) {}
-
-    Protect_FPU_rounding<!Protection> P(CGAL_FE_TONEAREST);
-    EC_result_type er = Exact_construction(To_Exact(a1), To_Exact(a2), To_Exact(a3), To_Exact(a4), To_Exact(a5)) ;
-    return From_Exact(er);
-
-  }
 };
 
 
diff -pruN 5.0.3-1/include/CGAL/Straight_skeleton_2/test.h 5.1-1/include/CGAL/Straight_skeleton_2/test.h
--- 5.0.3-1/include/CGAL/Straight_skeleton_2/test.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Straight_skeleton_2/test.h	2020-09-08 12:15:41.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Straight_skeleton_2/include/CGAL/Straight_skeleton_2/test.h $
-// $Id: test.h 5b2d46c 2020-04-21T10:03:00+02:00 Maxime Gimeno
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Straight_skeleton_2/include/CGAL/Straight_skeleton_2/test.h $
+// $Id: test.h e9d41d7 2020-04-21T10:03:00+02:00 Maxime Gimeno
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Fernando Cacciola <fernando_cacciola@ciudad.com.ar>
diff -pruN 5.0.3-1/include/CGAL/Straight_skeleton_2.h 5.1-1/include/CGAL/Straight_skeleton_2.h
--- 5.0.3-1/include/CGAL/Straight_skeleton_2.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Straight_skeleton_2.h	2020-09-08 12:15:41.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Straight_skeleton_2/include/CGAL/Straight_skeleton_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Straight_skeleton_2/include/CGAL/Straight_skeleton_2.h $
 // $Id: Straight_skeleton_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Straight_skeleton_builder_2.h 5.1-1/include/CGAL/Straight_skeleton_builder_2.h
--- 5.0.3-1/include/CGAL/Straight_skeleton_builder_2.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Straight_skeleton_builder_2.h	2020-09-08 12:15:41.000000000 +0000
@@ -3,7 +3,7 @@
 // This file is part of CGAL (www.cgal.org).
 //
 
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Straight_skeleton_2/include/CGAL/Straight_skeleton_builder_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Straight_skeleton_2/include/CGAL/Straight_skeleton_builder_2.h $
 // $Id: Straight_skeleton_builder_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Straight_skeleton_builder_traits_2.h 5.1-1/include/CGAL/Straight_skeleton_builder_traits_2.h
--- 5.0.3-1/include/CGAL/Straight_skeleton_builder_traits_2.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Straight_skeleton_builder_traits_2.h	2020-09-08 12:15:41.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Straight_skeleton_2/include/CGAL/Straight_skeleton_builder_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Straight_skeleton_2/include/CGAL/Straight_skeleton_builder_traits_2.h $
 // $Id: Straight_skeleton_builder_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Straight_skeleton_converter_2.h 5.1-1/include/CGAL/Straight_skeleton_converter_2.h
--- 5.0.3-1/include/CGAL/Straight_skeleton_converter_2.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Straight_skeleton_converter_2.h	2020-09-08 12:15:41.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Straight_skeleton_2/include/CGAL/Straight_skeleton_converter_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Straight_skeleton_2/include/CGAL/Straight_skeleton_converter_2.h $
 // $Id: Straight_skeleton_converter_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Straight_skeleton_face_base_2.h 5.1-1/include/CGAL/Straight_skeleton_face_base_2.h
--- 5.0.3-1/include/CGAL/Straight_skeleton_face_base_2.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Straight_skeleton_face_base_2.h	2020-09-08 12:15:41.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Straight_skeleton_2/include/CGAL/Straight_skeleton_face_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Straight_skeleton_2/include/CGAL/Straight_skeleton_face_base_2.h $
 // $Id: Straight_skeleton_face_base_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Straight_skeleton_halfedge_base_2.h 5.1-1/include/CGAL/Straight_skeleton_halfedge_base_2.h
--- 5.0.3-1/include/CGAL/Straight_skeleton_halfedge_base_2.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Straight_skeleton_halfedge_base_2.h	2020-09-08 12:15:41.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Straight_skeleton_2/include/CGAL/Straight_skeleton_halfedge_base_2.h $
-// $Id: Straight_skeleton_halfedge_base_2.h 6e4eb71 2020-06-23T18:48:04+02:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Straight_skeleton_2/include/CGAL/Straight_skeleton_halfedge_base_2.h $
+// $Id: Straight_skeleton_halfedge_base_2.h 3d78df1 2020-06-23T18:49:55+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Fernando Cacciola <fernando_cacciola@ciudad.com.ar>
diff -pruN 5.0.3-1/include/CGAL/Straight_skeleton_items_2.h 5.1-1/include/CGAL/Straight_skeleton_items_2.h
--- 5.0.3-1/include/CGAL/Straight_skeleton_items_2.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Straight_skeleton_items_2.h	2020-09-08 12:15:41.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Straight_skeleton_2/include/CGAL/Straight_skeleton_items_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Straight_skeleton_2/include/CGAL/Straight_skeleton_items_2.h $
 // $Id: Straight_skeleton_items_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Straight_skeleton_vertex_base_2.h 5.1-1/include/CGAL/Straight_skeleton_vertex_base_2.h
--- 5.0.3-1/include/CGAL/Straight_skeleton_vertex_base_2.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Straight_skeleton_vertex_base_2.h	2020-09-08 12:15:41.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Straight_skeleton_2/include/CGAL/Straight_skeleton_vertex_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Straight_skeleton_2/include/CGAL/Straight_skeleton_vertex_base_2.h $
 // $Id: Straight_skeleton_vertex_base_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Stream_lines_2.h 5.1-1/include/CGAL/Stream_lines_2.h
--- 5.0.3-1/include/CGAL/Stream_lines_2.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Stream_lines_2.h	2020-09-08 12:15:41.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_lines_2/include/CGAL/Stream_lines_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_lines_2/include/CGAL/Stream_lines_2.h $
 // $Id: Stream_lines_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/streamlines_assertions.h 5.1-1/include/CGAL/streamlines_assertions.h
--- 5.0.3-1/include/CGAL/streamlines_assertions.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/streamlines_assertions.h	2020-09-08 12:15:41.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_lines_2/include/CGAL/streamlines_assertions.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_lines_2/include/CGAL/streamlines_assertions.h $
 // $Id: streamlines_assertions.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/structure_point_set.h 5.1-1/include/CGAL/structure_point_set.h
--- 5.0.3-1/include/CGAL/structure_point_set.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/structure_point_set.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_processing_3/include/CGAL/structure_point_set.h $
-// $Id: structure_point_set.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/structure_point_set.h $
+// $Id: structure_point_set.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -164,22 +164,42 @@ public:
     \param points input point range.
     \param planes input plane range.
     \param epsilon size parameter.
-    \param np optional sequence of \ref psp_namedparameters "Named Parameters" among the ones listed below.
+    \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
     \cgalNamedParamsBegin
-      \cgalParamBegin{point_map} a model of `ReadablePropertyMap` with value type `Kernel::Point_3`.
-      If this parameter is omitted, `CGAL::Identity_property_map<Kernel::Point_3>` is used.\cgalParamEnd
-      \cgalParamBegin{normal_map} a model of `ReadablePropertyMap` with value type
-      `Kernel::Vector_3`.\cgalParamEnd
-      \cgalParamBegin{plane_index_map} a model of `ReadablePropertyMap` with value type `int`.
-      Associates the index of a point in the input range to the index of plane (-1 if point does is not assigned to
-      a plane).\cgalParamEnd
-      \cgalParamBegin{plane_map} a model of `ReadablePropertyMap` with value type
-      `Kernel::Plane_3`. If this parameter is omitted, `CGAL::Identity_property_map<Kernel::Plane_3>`
-      is used.\cgalParamEnd
-      \cgalParamBegin{attraction_factor} multiple of `epsilon` used to connect simplices.\cgalParamEnd
+      \cgalParamNBegin{point_map}
+        \cgalParamDescription{a property map associating points to the elements of the point set `points`}
+        \cgalParamType{a model of `ReadablePropertyMap` whose key type is the value type
+                       of the iterator of `PointRange` and whose value type is `geom_traits::Point_3`}
+        \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>`}
+      \cgalParamNEnd
+
+      \cgalParamNBegin{normal_map}
+        \cgalParamDescription{a property map associating normals to the elements of the point set `points`}
+        \cgalParamType{a model of `ReadablePropertyMap` whose key type is the value type
+                       of the iterator of `PointRange` and whose value type is `geom_traits::Vector_3`}
+      \cgalParamNEnd
+
+      \cgalParamNBegin{plane_index_map}
+        \cgalParamDescription{a property map associating the index of a point in the input range
+                              to the index of plane (`-1` if the point is not assigned to a plane)}
+        \cgalParamType{a class model of `ReadablePropertyMap` with `std::size_t` as key type and `int` as value type}
+        \cgalParamDefault{unused}
+      \cgalParamNEnd
+
+      \cgalParamNBegin{plane_map}
+        \cgalParamDescription{a property map containing the planes associated to the elements of the plane range `planes`}
+         \cgalParamType{a class model of `ReadablePropertyMap` with `PlaneRange::iterator::value_type`
+                        as key type and `geom_traits::Plane_3` as value type}
+        \cgalParamDefault{`CGAL::Identity_property_map<Kernel::Plane_3>`}
+      \cgalParamNEnd
+
+      \cgalParamNBegin{attraction_factor}
+        \cgalParamDescription{multiple of a tolerance `epsilon` used to connect simplices}
+        \cgalParamType{floating scalar value}
+        \cgalParamDefault{`3`}
+      \cgalParamNEnd
     \cgalNamedParamsEnd
-
   */
   template <typename PointRange,
             typename PlaneRange,
@@ -206,7 +226,7 @@ public:
     using parameters::get_parameter;
 
     // basic geometric types
-    typedef typename Point_set_processing_3::GetPointMap<PointRange, NamedParameters>::type PointMap;
+    typedef typename CGAL::GetPointMap<PointRange, NamedParameters>::type PointMap;
     typedef typename Point_set_processing_3::GetNormalMap<PointRange, NamedParameters>::type NormalMap;
     typedef typename Point_set_processing_3::GetPlaneMap<PlaneRange, NamedParameters>::type PlaneMap;
     typedef typename Point_set_processing_3::GetPlaneIndexMap<NamedParameters>::type PlaneIndexMap;
@@ -218,10 +238,10 @@ public:
                                 typename Point_set_processing_3::GetPlaneIndexMap<NamedParameters>::NoMap>::value),
                               "Error: no plane index map");
 
-    PointMap point_map = choose_parameter(get_parameter(np, internal_np::point_map), PointMap());
-    NormalMap normal_map = choose_parameter(get_parameter(np, internal_np::normal_map), NormalMap());
-    PlaneMap plane_map = choose_parameter(get_parameter(np, internal_np::plane_map), PlaneMap());
-    PlaneIndexMap index_map = choose_parameter(get_parameter(np, internal_np::plane_index_map), PlaneIndexMap());
+    PointMap point_map = choose_parameter<PointMap>(get_parameter(np, internal_np::point_map));
+    NormalMap normal_map = choose_parameter<NormalMap>(get_parameter(np, internal_np::normal_map));
+    PlaneMap plane_map = choose_parameter<PlaneMap>(get_parameter(np, internal_np::plane_map));
+    PlaneIndexMap index_map = choose_parameter<PlaneIndexMap>(get_parameter(np, internal_np::plane_index_map));
     double attraction_factor = choose_parameter(get_parameter(np, internal_np::attraction_factor), 3.);
 
     m_points.reserve(points.size());
@@ -1491,21 +1511,47 @@ private:
    \param planes input plane range.
    \param output output iterator where output points are written
    \param epsilon size parameter.
-   \param np optional sequence of \ref psp_namedparameters "Named Parameters" among the ones listed below.
+   \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
    \cgalNamedParamsBegin
-     \cgalParamBegin{point_map} a model of `ReadablePropertyMap` with value type `geom_traits::Point_3`.
-     If this parameter is omitted, `CGAL::Identity_property_map<geom_traits::Point_3>` is used.\cgalParamEnd
-     \cgalParamBegin{normal_map} a model of `ReadablePropertyMap` with value type
-     `geom_traits::Vector_3`.\cgalParamEnd
-     \cgalParamBegin{plane_index_map} a model of `ReadablePropertyMap` with value type `int`.
-     Associates the index of a point in the input range to the index of plane (-1 if point does is not assigned to
-     a plane).\cgalParamEnd
-     \cgalParamBegin{plane_map} a model of `ReadablePropertyMap` with value type
-     `geom_traits::Plane_3`. If this parameter is omitted, `CGAL::Identity_property_map<geom_traits::Plane_3>`
-     is used.\cgalParamEnd
-     \cgalParamBegin{attraction_factor} multiple of `epsilon` used to connect simplices.\cgalParamEnd
-     \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+      \cgalParamNBegin{point_map}
+        \cgalParamDescription{a property map associating points to the elements of the point set `points`}
+        \cgalParamType{a model of `ReadablePropertyMap` whose key type is the value type
+                       of the iterator of `PointRange` and whose value type is `geom_traits::Point_3`}
+        \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>`}
+      \cgalParamNEnd
+
+      \cgalParamNBegin{normal_map}
+        \cgalParamDescription{a property map associating normals to the elements of the point set `points`}
+        \cgalParamType{a model of `ReadablePropertyMap` whose key type is the value type
+                       of the iterator of `PointRange` and whose value type is `geom_traits::Vector_3`}
+      \cgalParamNEnd
+
+      \cgalParamNBegin{plane_index_map}
+        \cgalParamDescription{a property map associating the index of a point in the input range
+                              to the index of plane (`-1` if the point is not assigned to a plane)}
+        \cgalParamType{a class model of `ReadablePropertyMap` with `std::size_t` as key type and `int` as value type}
+        \cgalParamDefault{unused}
+      \cgalParamNEnd
+
+      \cgalParamNBegin{plane_map}
+        \cgalParamDescription{a property map containing the planes associated to the elements of the plane range `planes`}
+         \cgalParamType{a class model of `ReadablePropertyMap` with `PlaneRange::iterator::value_type`
+                        as key type and `geom_traits::Plane_3` as value type}
+        \cgalParamDefault{`CGAL::Identity_property_map<Kernel::Plane_3>`}
+      \cgalParamNEnd
+
+      \cgalParamNBegin{attraction_factor}
+        \cgalParamDescription{multiple of a tolerance `epsilon` used to connect simplices}
+        \cgalParamType{floating scalar value}
+        \cgalParamDefault{`3`}
+      \cgalParamNEnd
+
+     \cgalParamNBegin{geom_traits}
+       \cgalParamDescription{an instance of a geometric traits class}
+       \cgalParamType{a model of `Kernel`}
+       \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+     \cgalParamNEnd
    \cgalNamedParamsEnd
 
 */
@@ -1557,4 +1603,3 @@ structure_point_set (const PointRange& p
 #include <CGAL/enable_warnings.h>
 
 #endif // CGAL_STRUCTURE_POINT_SET_3_H
-
diff -pruN 5.0.3-1/include/CGAL/subdivide_skin_surface_mesh_3.h 5.1-1/include/CGAL/subdivide_skin_surface_mesh_3.h
--- 5.0.3-1/include/CGAL/subdivide_skin_surface_mesh_3.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/subdivide_skin_surface_mesh_3.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Skin_surface_3/include/CGAL/subdivide_skin_surface_mesh_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Skin_surface_3/include/CGAL/subdivide_skin_surface_mesh_3.h $
 // $Id: subdivide_skin_surface_mesh_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/subdivide_union_of_balls_mesh_3.h 5.1-1/include/CGAL/subdivide_union_of_balls_mesh_3.h
--- 5.0.3-1/include/CGAL/subdivide_union_of_balls_mesh_3.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/subdivide_union_of_balls_mesh_3.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Skin_surface_3/include/CGAL/subdivide_union_of_balls_mesh_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Skin_surface_3/include/CGAL/subdivide_union_of_balls_mesh_3.h $
 // $Id: subdivide_union_of_balls_mesh_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Subdivision_method_3/internal/Euler_extensions.h 5.1-1/include/CGAL/Subdivision_method_3/internal/Euler_extensions.h
--- 5.0.3-1/include/CGAL/Subdivision_method_3/internal/Euler_extensions.h	2020-08-01 19:02:54.000000000 +0000
+++ 5.1-1/include/CGAL/Subdivision_method_3/internal/Euler_extensions.h	2020-09-08 12:15:42.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Subdivision_method_3/include/CGAL/Subdivision_method_3/internal/Euler_extensions.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Subdivision_method_3/include/CGAL/Subdivision_method_3/internal/Euler_extensions.h $
 // $Id: Euler_extensions.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Subdivision_method_3/internal/subdivision_hosts_impl_3.h 5.1-1/include/CGAL/Subdivision_method_3/internal/subdivision_hosts_impl_3.h
--- 5.0.3-1/include/CGAL/Subdivision_method_3/internal/subdivision_hosts_impl_3.h	2020-08-01 19:02:54.000000000 +0000
+++ 5.1-1/include/CGAL/Subdivision_method_3/internal/subdivision_hosts_impl_3.h	2020-09-08 12:15:42.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Subdivision_method_3/include/CGAL/Subdivision_method_3/internal/subdivision_hosts_impl_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Subdivision_method_3/include/CGAL/Subdivision_method_3/internal/subdivision_hosts_impl_3.h $
 // $Id: subdivision_hosts_impl_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Subdivision_method_3/subdivision_hosts_3.h 5.1-1/include/CGAL/Subdivision_method_3/subdivision_hosts_3.h
--- 5.0.3-1/include/CGAL/Subdivision_method_3/subdivision_hosts_3.h	2020-08-01 19:02:54.000000000 +0000
+++ 5.1-1/include/CGAL/Subdivision_method_3/subdivision_hosts_3.h	2020-09-08 12:15:42.000000000 +0000
@@ -4,8 +4,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Subdivision_method_3/include/CGAL/Subdivision_method_3/subdivision_hosts_3.h $
-// $Id: subdivision_hosts_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Subdivision_method_3/include/CGAL/Subdivision_method_3/subdivision_hosts_3.h $
+// $Id: subdivision_hosts_3.h 48bd92b 2020-04-13T13:03:05+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -58,18 +58,27 @@ void PQQ(PolygonMesh& pmesh, Mask mask,
  *
  * @tparam PolygonMesh a model of `MutableFaceGraph`
  * @tparam Mask a model of `PQQMask_3`
- * @tparam NamedParameters a sequence of \ref sm_namedparameters "Named Parameters"
+ * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
  *
  * @param pmesh a polygon mesh
  * @param mask a geometry policy mask
- * @param np optional sequence of \ref sm_namedparameters "Named Parameters" among the ones listed below
+ * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
- *       If this parameter is omitted, an internal property map for
- *       `CGAL::vertex_point_t` must be available in `PolygonMesh`\cgalParamEnd
- *  \cgalParamBegin{number_of_iterations} the number of subdivision steps, by default 1.
- *     \cgalParamEnd
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `pmesh`}
+ *     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+ *                    as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, pmesh)`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     should be available for the vertices of `pmesh`.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{number_of_iterations}
+ *     \cgalParamDescription{the number of subdivision steps}
+ *     \cgalParamType{unsigned int}
+ *     \cgalParamDefault{`1`}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  **/
 template <class PolygonMesh, class Mask, class NamedParameters>
@@ -78,7 +87,7 @@ void PQQ(PolygonMesh& pmesh, Mask mask,
   using parameters::choose_parameter;
   using parameters::get_parameter;
 
-  typedef typename Polygon_mesh_processing::GetVertexPointMap<PolygonMesh, NamedParameters>::type Vpm;
+  typedef typename CGAL::GetVertexPointMap<PolygonMesh, NamedParameters>::type Vpm;
   Vpm vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
                          get_property_map(CGAL::vertex_point, pmesh));
 
@@ -105,18 +114,27 @@ void PTQ(PolygonMesh& pmesh, Mask mask,
  *
  * @tparam PolygonMesh a model of `MutableFaceGraph`
  * @tparam Mask a model of `PTQMask_3`
- * @tparam NamedParameters a sequence of \ref sm_namedparameters "Named Parameters"
+ * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
  *
  * @param pmesh a polygon mesh
  * @param mask a geometry policy mask
- * @param np optional sequence of \ref sm_namedparameters "Named Parameters" among the ones listed below
+ * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
- *       If this parameter is omitted, an internal property map for
- *       `CGAL::vertex_point_t` must be available in `PolygonMesh`\cgalParamEnd
- *  \cgalParamBegin{number_of_iterations} the number of subdivision steps, by default 1.
- *     \cgalParamEnd
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `pmesh`}
+ *     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+ *                    as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, pmesh)`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     should be available for the vertices of `pmesh`.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{number_of_iterations}
+ *     \cgalParamDescription{the number of subdivision steps}
+ *     \cgalParamType{unsigned int}
+ *     \cgalParamDefault{`1`}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  **/
 template <class PolygonMesh, class Mask, class NamedParameters>
@@ -125,7 +143,7 @@ void PTQ(PolygonMesh& pmesh, Mask mask,
   using parameters::choose_parameter;
   using parameters::get_parameter;
 
-  typedef typename Polygon_mesh_processing::GetVertexPointMap<PolygonMesh, NamedParameters>::type Vpm;
+  typedef typename CGAL::GetVertexPointMap<PolygonMesh, NamedParameters>::type Vpm;
   Vpm vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
                          get_property_map(CGAL::vertex_point, pmesh));
 
@@ -153,18 +171,27 @@ void DQQ(PolygonMesh& pmesh, Mask mask,
  *
  * @tparam PolygonMesh a model of `MutableFaceGraph`
  * @tparam Mask a model of `DQQMask_3`
- * @tparam NamedParameters a sequence of \ref sm_namedparameters "Named Parameters"
+ * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
  *
  * @param pmesh a polygon mesh
  * @param mask a geometry policy mask
- * @param np optional sequence of \ref sm_namedparameters "Named Parameters" among the ones listed below
+ * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
- *       If this parameter is omitted, an internal property map for
- *       `CGAL::vertex_point_t` must be available in `PolygonMesh`\cgalParamEnd
- *  \cgalParamBegin{number_of_iterations} the number of subdivision steps, by default 1.
- *     \cgalParamEnd
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `pmesh`}
+ *     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+ *                    as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, pmesh)`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     should be available for the vertices of `pmesh`.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{number_of_iterations}
+ *     \cgalParamDescription{the number of subdivision steps}
+ *     \cgalParamType{unsigned int}
+ *     \cgalParamDefault{`1`}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  *
  * \pre `pmesh` must be a triangle mesh.
@@ -175,7 +202,7 @@ void DQQ(PolygonMesh& pmesh, Mask mask,
   using parameters::choose_parameter;
   using parameters::get_parameter;
 
-  typedef typename Polygon_mesh_processing::GetVertexPointMap<PolygonMesh, NamedParameters>::type Vpm;
+  typedef typename CGAL::GetVertexPointMap<PolygonMesh, NamedParameters>::type Vpm;
   Vpm vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
                          get_property_map(CGAL::vertex_point, pmesh));
 
@@ -205,18 +232,27 @@ void Sqrt3(PolygonMesh& pmesh, Mask mask
  *
  * @tparam PolygonMesh a model of `MutableFaceGraph`
  * @tparam Mask a model of `Sqrt3Mask_3`
- * @tparam NamedParameters a sequence of \ref sm_namedparameters "Named Parameters"
+ * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
  *
  * @param pmesh a polygon mesh
  * @param mask a geometry policy mask
- * @param np optional sequence of \ref sm_namedparameters "Named Parameters" among the ones listed below
+ * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
- *       If this parameter is omitted, an internal property map for
- *       `CGAL::vertex_point_t` must be available in `PolygonMesh`\cgalParamEnd
- *  \cgalParamBegin{number_of_iterations} the number of subdivision steps, by default 1.
- *     \cgalParamEnd
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `pmesh`}
+ *     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+ *                    as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, pmesh)`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     should be available for the vertices of `pmesh`.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{number_of_iterations}
+ *     \cgalParamDescription{the number of subdivision steps}
+ *     \cgalParamType{unsigned int}
+ *     \cgalParamDefault{`1`}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  *
  * \pre `pmesh` must be a triangle mesh.
@@ -227,7 +263,7 @@ void Sqrt3(PolygonMesh& pmesh, Mask mask
   using parameters::choose_parameter;
   using parameters::get_parameter;
 
-  typedef typename Polygon_mesh_processing::GetVertexPointMap<PolygonMesh, NamedParameters>::type Vpm;
+  typedef typename CGAL::GetVertexPointMap<PolygonMesh, NamedParameters>::type Vpm;
   Vpm vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
                          get_property_map(CGAL::vertex_point, pmesh));
 
diff -pruN 5.0.3-1/include/CGAL/Subdivision_method_3/subdivision_masks_3.h 5.1-1/include/CGAL/Subdivision_method_3/subdivision_masks_3.h
--- 5.0.3-1/include/CGAL/Subdivision_method_3/subdivision_masks_3.h	2020-08-01 19:02:54.000000000 +0000
+++ 5.1-1/include/CGAL/Subdivision_method_3/subdivision_masks_3.h	2020-09-08 12:15:42.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Subdivision_method_3/include/CGAL/Subdivision_method_3/subdivision_masks_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Subdivision_method_3/include/CGAL/Subdivision_method_3/subdivision_masks_3.h $
 // $Id: subdivision_masks_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Subdivision_method_3/subdivision_methods_3.h 5.1-1/include/CGAL/Subdivision_method_3/subdivision_methods_3.h
--- 5.0.3-1/include/CGAL/Subdivision_method_3/subdivision_methods_3.h	2020-08-01 19:02:54.000000000 +0000
+++ 5.1-1/include/CGAL/Subdivision_method_3/subdivision_methods_3.h	2020-09-08 12:15:42.000000000 +0000
@@ -4,8 +4,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Subdivision_method_3/include/CGAL/Subdivision_method_3/subdivision_methods_3.h $
-// $Id: subdivision_methods_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Subdivision_method_3/include/CGAL/Subdivision_method_3/subdivision_methods_3.h $
+// $Id: subdivision_methods_3.h 48bd92b 2020-04-13T13:03:05+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -102,17 +102,26 @@ void CatmullClark_subdivision(PolygonMes
  * This function overwrites the control mesh `pmesh` with the subdivided mesh.
  *
  * @tparam PolygonMesh a model of `MutableFaceGraph`
- * @tparam NamedParameters a sequence of \ref sm_namedparameters "Named Parameters"
+ * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
  *
  * @param pmesh a polygon mesh
- * @param np optional sequence of \ref sm_namedparameters "Named Parameters" among the ones listed below
+ * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
- *       If this parameter is omitted, an internal property map for
- *       `CGAL::vertex_point_t` must be available in `PolygonMesh`\cgalParamEnd
- *  \cgalParamBegin{number_of_iterations} the number of subdivision steps, by default 1.
- *     \cgalParamEnd
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `pmesh`}
+ *     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+ *                    as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, pmesh)`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     should be available for the vertices of `pmesh`.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{number_of_iterations}
+ *     \cgalParamDescription{the number of subdivision steps}
+ *     \cgalParamType{unsigned int}
+ *     \cgalParamDefault{`1`}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  *
  * \pre `pmesh` must be a triangle mesh.
@@ -121,7 +130,8 @@ template <class PolygonMesh, class Named
 void CatmullClark_subdivision(PolygonMesh& pmesh, const NamedParameters& np) {
   using parameters::choose_parameter;
   using parameters::get_parameter;
-  typedef typename Polygon_mesh_processing::GetVertexPointMap<PolygonMesh, NamedParameters>::type Vpm;
+
+  typedef typename CGAL::GetVertexPointMap<PolygonMesh, NamedParameters>::type Vpm;
   Vpm vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
                          get_property_map(CGAL::vertex_point, pmesh));
 
@@ -157,24 +167,34 @@ void Loop_subdivision(PolygonMesh& pmesh
  * This function overwrites the control mesh `pmesh` with the subdivided mesh.
 
  * @tparam PolygonMesh a model of `MutableFaceGraph`
- * @tparam NamedParameters a sequence of \ref sm_namedparameters "Named Parameters"
+ * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
  *
  * @param pmesh a polygon mesh
- * @param np optional sequence of \ref sm_namedparameters "Named Parameters" among the ones listed below
+ * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
- *       If this parameter is omitted, an internal property map for
- *       `CGAL::vertex_point_t` must be available in `PolygonMesh`\cgalParamEnd
- *  \cgalParamBegin{number_of_iterations} the number of subdivision steps, by default 1.
- *     \cgalParamEnd
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `pmesh`}
+ *     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+ *                    as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, pmesh)`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     should be available for the vertices of `pmesh`.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{number_of_iterations}
+ *     \cgalParamDescription{the number of subdivision steps}
+ *     \cgalParamType{unsigned int}
+ *     \cgalParamDefault{`1`}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  **/
 template <class PolygonMesh, class NamedParameters>
 void Loop_subdivision(PolygonMesh& pmesh, const NamedParameters& np) {
   using parameters::choose_parameter;
   using parameters::get_parameter;
-  typedef typename Polygon_mesh_processing::GetVertexPointMap<PolygonMesh, NamedParameters>::type Vpm;
+
+  typedef typename CGAL::GetVertexPointMap<PolygonMesh, NamedParameters>::type Vpm;
   Vpm vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
                          get_property_map(CGAL::vertex_point, pmesh));
 
@@ -210,24 +230,34 @@ void DooSabin_subdivision(PolygonMesh& p
  * This function overwrites the control mesh `pmesh` with the subdivided mesh.
 
  * @tparam PolygonMesh a model of `MutableFaceGraph`
- * @tparam NamedParameters a sequence of \ref sm_namedparameters "Named Parameters"
+ * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
  *
  * @param pmesh a polygon mesh
- * @param np optional sequence of \ref sm_namedparameters "Named Parameters" among the ones listed below
+ * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
- *       If this parameter is omitted, an internal property map for
- *       `CGAL::vertex_point_t` must be available in `PolygonMesh`\cgalParamEnd
- *  \cgalParamBegin{number_of_iterations} the number of subdivision steps, by default 1.
- *     \cgalParamEnd
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `pmesh`}
+ *     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+ *                    as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, pmesh)`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     should be available for the vertices of `pmesh`.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{number_of_iterations}
+ *     \cgalParamDescription{the number of subdivision steps}
+ *     \cgalParamType{unsigned int}
+ *     \cgalParamDefault{`1`}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  **/
 template <class PolygonMesh, class NamedParameters>
 void DooSabin_subdivision(PolygonMesh& pmesh, const NamedParameters& np) {
   using parameters::choose_parameter;
   using parameters::get_parameter;
-  typedef typename Polygon_mesh_processing::GetVertexPointMap<PolygonMesh, NamedParameters>::type Vpm;
+
+  typedef typename CGAL::GetVertexPointMap<PolygonMesh, NamedParameters>::type Vpm;
   Vpm vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
                          get_property_map(CGAL::vertex_point, pmesh));
 
@@ -266,17 +296,26 @@ void Sqrt3_subdivision(PolygonMesh& pmes
  *            during a <em>single</em> call of this function.
  *
  * @tparam PolygonMesh a model of `MutableFaceGraph`
- * @tparam NamedParameters a sequence of \ref sm_namedparameters "Named Parameters"
+ * @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
  *
  * @param pmesh a polygon mesh
- * @param np optional sequence of \ref sm_namedparameters "Named Parameters" among the ones listed below
+ * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
- *       If this parameter is omitted, an internal property map for
- *       `CGAL::vertex_point_t` must be available in `PolygonMesh`\cgalParamEnd
- *  \cgalParamBegin{number_of_iterations} the number of subdivision steps, by default 1.
- *     \cgalParamEnd
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `pmesh`}
+ *     \cgalParamType{a class model of `ReadWritePropertyMap` with `boost::graph_traits<PolygonMesh>::%vertex_descriptor`
+ *                    as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, pmesh)`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     should be available for the vertices of `pmesh`.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{number_of_iterations}
+ *     \cgalParamDescription{the number of subdivision steps}
+ *     \cgalParamType{unsigned int}
+ *     \cgalParamDefault{`1`}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  *
  * \pre `pmesh` must be a triangle mesh.
@@ -285,7 +324,8 @@ template <class PolygonMesh, class Named
 void Sqrt3_subdivision(PolygonMesh& pmesh, const NamedParameters& np) {
   using parameters::choose_parameter;
   using parameters::get_parameter;
-  typedef typename Polygon_mesh_processing::GetVertexPointMap<PolygonMesh, NamedParameters>::type Vpm;
+
+  typedef typename CGAL::GetVertexPointMap<PolygonMesh, NamedParameters>::type Vpm;
   Vpm vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
                          get_property_map(CGAL::vertex_point, pmesh));
 
diff -pruN 5.0.3-1/include/CGAL/subdivision_method_3.h 5.1-1/include/CGAL/subdivision_method_3.h
--- 5.0.3-1/include/CGAL/subdivision_method_3.h	2020-08-01 19:02:54.000000000 +0000
+++ 5.1-1/include/CGAL/subdivision_method_3.h	2020-09-08 12:15:42.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Subdivision_method_3/include/CGAL/subdivision_method_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Subdivision_method_3/include/CGAL/subdivision_method_3.h $
 // $Id: subdivision_method_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh/IO/3mf.h 5.1-1/include/CGAL/Surface_mesh/IO/3mf.h
--- 5.0.3-1/include/CGAL/Surface_mesh/IO/3mf.h	2020-08-01 19:02:54.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh/IO/3mf.h	2020-09-08 12:15:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh/include/CGAL/Surface_mesh/IO/3mf.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh/include/CGAL/Surface_mesh/IO/3mf.h $
 // $Id: 3mf.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh/IO/PLY.h 5.1-1/include/CGAL/Surface_mesh/IO/PLY.h
--- 5.0.3-1/include/CGAL/Surface_mesh/IO/PLY.h	2020-08-01 19:02:54.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh/IO/PLY.h	2020-09-08 12:15:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh/include/CGAL/Surface_mesh/IO/PLY.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh/include/CGAL/Surface_mesh/IO/PLY.h $
 // $Id: PLY.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh/IO.h 5.1-1/include/CGAL/Surface_mesh/IO.h
--- 5.0.3-1/include/CGAL/Surface_mesh/IO.h	2020-08-01 19:02:54.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh/IO.h	2020-09-08 12:15:43.000000000 +0000
@@ -5,7 +5,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh/include/CGAL/Surface_mesh/IO.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh/include/CGAL/Surface_mesh/IO.h $
 // $Id: IO.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh/Properties.h 5.1-1/include/CGAL/Surface_mesh/Properties.h
--- 5.0.3-1/include/CGAL/Surface_mesh/Properties.h	2020-08-01 19:02:54.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh/Properties.h	2020-09-08 12:15:43.000000000 +0000
@@ -5,8 +5,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh/include/CGAL/Surface_mesh/Properties.h $
-// $Id: Properties.h 5b2d46c 2020-04-21T10:03:00+02:00 Maxime Gimeno
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh/include/CGAL/Surface_mesh/Properties.h $
+// $Id: Properties.h e9d41d7 2020-04-21T10:03:00+02:00 Maxime Gimeno
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 
@@ -486,6 +486,7 @@ public:
     void swap (Property_container& other)
     {
       this->parrays_.swap (other.parrays_);
+      std::swap(this->size_, other.size_);
     }
 
 private:
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh/Surface_mesh_fwd.h 5.1-1/include/CGAL/Surface_mesh/Surface_mesh_fwd.h
--- 5.0.3-1/include/CGAL/Surface_mesh/Surface_mesh_fwd.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh/Surface_mesh_fwd.h	2020-09-08 12:14:54.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/Surface_mesh/Surface_mesh_fwd.h $
-// $Id: Surface_mesh_fwd.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/Surface_mesh/Surface_mesh_fwd.h $
+// $Id: Surface_mesh_fwd.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 
@@ -17,7 +17,7 @@
 namespace CGAL {
 
 // fwdS for the public interface
-template<typename K>
+template<typename P>
 class Surface_mesh;
 
 
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh/Surface_mesh.h 5.1-1/include/CGAL/Surface_mesh/Surface_mesh.h
--- 5.0.3-1/include/CGAL/Surface_mesh/Surface_mesh.h	2020-08-01 19:02:54.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh/Surface_mesh.h	2020-09-08 12:15:43.000000000 +0000
@@ -4,8 +4,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh/include/CGAL/Surface_mesh/Surface_mesh.h $
-// $Id: Surface_mesh.h fa1c724 2020-06-23T12:55:31+01:00 Andreas Fabri
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh/include/CGAL/Surface_mesh/Surface_mesh.h $
+// $Id: Surface_mesh.h b3225cd 2020-07-03T16:21:29+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 
@@ -80,21 +80,6 @@ namespace CGAL {
           return idx_ != inf;
         }
 
-        /// are two indices equal?
-        bool operator==(const T& _rhs) const {
-            return idx_ == _rhs.idx_;
-        }
-
-        /// are two indices different?
-        bool operator!=(const T& _rhs) const {
-            return idx_ != _rhs.idx_;
-        }
-
-        /// Comparison by index.
-        bool operator<(const T& _rhs) const {
-            return idx_ < _rhs.idx_;
-        }
-
         // Compatibility with OpenMesh handle
         size_type idx() const {
           return idx_;
@@ -120,7 +105,7 @@ namespace CGAL {
 
         SM_Index operator+=(std::ptrdiff_t n) { idx_ = size_type(std::ptrdiff_t(idx_) + n); return *this; }
 
-    private:
+    protected:
         size_type idx_;
     };
 
@@ -131,8 +116,8 @@ namespace CGAL {
     return ret;
   }
 
-    // Implementation for Surface_mesh::Vertex_index
 
+    // Implementation for Surface_mesh::Vertex_index
     class SM_Vertex_index
  : public SM_Index<SM_Vertex_index>
     {
@@ -142,6 +127,25 @@ namespace CGAL {
 
         explicit SM_Vertex_index(size_type _idx) : SM_Index<SM_Vertex_index>(_idx) {}
 
+        template<class T> bool operator==(const T&) const = delete;
+        template<class T> bool operator!=(const T&) const = delete;
+        template<class T> bool operator<(const T&) const = delete;
+
+        /// are two indices equal?
+        bool operator==(const SM_Vertex_index& _rhs) const {
+            return this->idx_ == _rhs.idx_;
+        }
+
+        /// are two indices different?
+        bool operator!=(const SM_Vertex_index& _rhs) const {
+            return this->idx_ != _rhs.idx_;
+        }
+
+        /// Comparison by index.
+        bool operator<(const SM_Vertex_index& _rhs) const {
+          return this->idx_ < _rhs.idx_;
+        }
+
 
         friend std::ostream& operator<<(std::ostream& os, SM_Vertex_index const& v)
         {
@@ -168,6 +172,25 @@ namespace CGAL {
 
         explicit SM_Halfedge_index(size_type _idx) : SM_Index<SM_Halfedge_index>(_idx) {}
 
+        template<class T> bool operator==(const T&) const = delete;
+        template<class T> bool operator!=(const T&) const = delete;
+        template<class T> bool operator<(const T&) const = delete;
+
+        /// are two indices equal?
+        bool operator==(const SM_Halfedge_index& _rhs) const {
+            return this->idx_ == _rhs.idx_;
+        }
+
+        /// are two indices different?
+        bool operator!=(const SM_Halfedge_index& _rhs) const {
+            return this->idx_ != _rhs.idx_;
+        }
+
+        /// Comparison by index.
+        bool operator<(const SM_Halfedge_index& _rhs) const {
+          return this->idx_ < _rhs.idx_;
+        }
+
         friend std::ostream& operator<<(std::ostream& os, SM_Halfedge_index const& h)
         {
           return (os << 'h' << (size_type)h );
@@ -184,6 +207,24 @@ namespace CGAL {
 
         explicit SM_Face_index(size_type _idx) : SM_Index<SM_Face_index>(_idx) {}
 
+        template<class T> bool operator==(const T&) const = delete;
+        template<class T> bool operator!=(const T&) const = delete;
+        template<class T> bool operator<(const T&) const = delete;
+
+        /// are two indices equal?
+        bool operator==(const SM_Face_index& _rhs) const {
+            return this->idx_ == _rhs.idx_;
+        }
+
+        /// are two indices different?
+        bool operator!=(const SM_Face_index& _rhs) const {
+            return this->idx_ != _rhs.idx_;
+        }
+
+        /// Comparison by index.
+        bool operator<(const SM_Face_index& _rhs) const {
+          return this->idx_ < _rhs.idx_;
+        }
 
         friend std::ostream& operator<<(std::ostream& os, SM_Face_index const& f)
         {
@@ -218,6 +259,11 @@ namespace CGAL {
         // returns whether the index is valid, i.e., the index is not equal to (std::numeric_limits<size_type>::max)().
         bool is_valid() const { return halfedge_.is_valid(); }
 
+
+        template<class T> bool operator==(const T&) const = delete;
+        template<class T> bool operator!=(const T&) const = delete;
+        template<class T> bool operator<(const T&) const = delete;
+
         // Are two indices equal?
         bool operator==(const SM_Edge_index& other) const { return (size_type)(*this) == (size_type)other; }
 
@@ -1083,6 +1129,8 @@ public:
   }
 
     /// removes all vertices, halfedge, edges and faces. Collects garbage and clears all properties.
+    ///
+    /// After calling this method, the object is the same as a newly constructed object. The additional properties (such as normal vectors) are also removed and must thus be re-added if needed.
     void clear();
 
 
@@ -2112,9 +2160,9 @@ private: //-----------------------------
     else
       os << "COFF\n" << sm.number_of_vertices() << " " << sm.number_of_faces() << " 0\n";
     std::vector<int> reindex;
-    typename Polygon_mesh_processing::GetVertexPointMap<Surface_mesh<P>, NamedParameters>::const_type
+    typename CGAL::GetVertexPointMap<Surface_mesh<P>, NamedParameters>::const_type
         vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
-                           get_const_property_map(CGAL::vertex_point, sm));
+                               get_const_property_map(CGAL::vertex_point, sm));
     reindex.resize(sm.num_vertices());
     int n = 0;
     for(Vertex_index v : sm.vertices()){
@@ -2400,9 +2448,9 @@ private: //-----------------------------
    using parameters::choose_parameter;
    using parameters::get_parameter;
 
-    typename CGAL::Polygon_mesh_processing::GetVertexPointMap<Surface_mesh<P>, NamedParameters>::type
+    typename CGAL::GetVertexPointMap<Surface_mesh<P>, NamedParameters>::type
         vpm = choose_parameter(get_parameter(np, CGAL::internal_np::vertex_point),
-                           get_property_map(CGAL::vertex_point, sm));
+                               get_property_map(CGAL::vertex_point, sm));
     int n, f, e;
     std::string off;
     is >> sm_skip_comments;
@@ -2792,20 +2840,33 @@ void
 Surface_mesh<P>::
 clear()
 {
-    vprops_.resize(0);
-    hprops_.resize(0);
-    eprops_.resize(0);
-    fprops_.resize(0);
-
-    vprops_.shrink_to_fit();
-    hprops_.shrink_to_fit();
-    eprops_.shrink_to_fit();
-    fprops_.shrink_to_fit();
-
-    removed_vertices_ = removed_edges_ = removed_faces_ = 0;
-    vertices_freelist_ = edges_freelist_ = faces_freelist_ = (std::numeric_limits<size_type>::max)();
-    garbage_ = false;
-    anonymous_property_ = 0;
+  vprops_.clear();
+  hprops_.clear();
+  eprops_.clear();
+  fprops_.clear();
+
+  vprops_.resize(0);
+  hprops_.resize(0);
+  eprops_.resize(0);
+  fprops_.resize(0);
+
+  vprops_.shrink_to_fit();
+  hprops_.shrink_to_fit();
+  eprops_.shrink_to_fit();
+  fprops_.shrink_to_fit();
+
+  vconn_    = add_property_map<Vertex_index, Vertex_connectivity>("v:connectivity").first;
+  hconn_    = add_property_map<Halfedge_index, Halfedge_connectivity>("h:connectivity").first;
+  fconn_    = add_property_map<Face_index, Face_connectivity>("f:connectivity").first;
+  vpoint_   = add_property_map<Vertex_index, Point>("v:point").first;
+  vremoved_ = add_property_map<Vertex_index, bool>("v:removed", false).first;
+  eremoved_ = add_property_map<Edge_index, bool>("e:removed", false).first;
+  fremoved_ = add_property_map<Face_index, bool>("f:removed", false).first;
+
+  removed_vertices_ = removed_edges_ = removed_faces_ = 0;
+  vertices_freelist_ = edges_freelist_ = faces_freelist_ = (std::numeric_limits<size_type>::max)();
+  garbage_ = false;
+  anonymous_property_ = 0;
 }
 
 //-----------------------------------------------------------------------------
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_approximation/approximate_triangle_mesh.h 5.1-1/include/CGAL/Surface_mesh_approximation/approximate_triangle_mesh.h
--- 5.0.3-1/include/CGAL/Surface_mesh_approximation/approximate_triangle_mesh.h	2020-08-01 19:02:54.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_approximation/approximate_triangle_mesh.h	2020-09-08 12:15:43.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_approximation/include/CGAL/Surface_mesh_approximation/approximate_triangle_mesh.h $
-// $Id: approximate_triangle_mesh.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_approximation/include/CGAL/Surface_mesh_approximation/approximate_triangle_mesh.h $
+// $Id: approximate_triangle_mesh.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -47,7 +47,7 @@ enum Verbose_level {
 #ifdef DOXYGEN_RUNNING
 namespace parameters {
 
-/*! \ingroup vsa_namedparameters
+/*! \ingroup bgl_namedparameters
  * This function is used when default parameters are just fine for approximation or meshing.
  */
 unspecified_type all_default();
@@ -62,57 +62,130 @@ unspecified_type all_default();
  * to approximate a triangle surface mesh, with indexed triangles as output.
  *
  * @tparam TriangleMesh model of `FaceListGraph`
- * @tparam NamedParameters a sequence of \ref vsa_namedparameters
+ * @tparam NamedParameters a sequence of \ref bgl_namedparameters
  *
  * @param tm triangle surface mesh to be approximated
- * @param np optional sequence of \ref vsa_namedparameters among the ones listed below
+ * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
  * @return `true` if the indexed triangles represent a 2-manifold, oriented surface mesh, and `false` otherwise.
  *
  * \cgalNamedParamsBegin{Approximation Named Parameters}
- *  \cgalParamBegin{geom_traits} a geometric traits class instance, model of Kernel.
- *    Exact constructions kernels are not supported by this function.
- *  \cgalParamEnd
- *  \cgalParamBegin{vertex_point_map} the property map with the points associated
- *    to the vertices of `tm`. Instance of a class model of `ReadablePropertyMap`.
- *  \cgalParamEnd
- *  \cgalParamBegin{verbose_level} set verbose level.
- *  \cgalParamEnd
- *  \cgalParamBegin{seeding_method} selection of seeding method.
- *  \cgalParamEnd
- *  \cgalParamBegin{max_number_of_proxies} maximum number of proxies to approximate the input mesh.
- *  \cgalParamEnd
- *  \cgalParamBegin{min_error_drop} minimum error drop of the approximation, expressed in ratio between two iterations of proxy addition.
- *  \cgalParamEnd
- *  \cgalParamBegin{number_of_relaxations} number of relaxation iterations interleaved within seeding.
- *  \cgalParamEnd
- *  \cgalParamBegin{number_of_iterations} number of partitioning and fitting iterations after seeding.
- *  \cgalParamEnd
+ *   \cgalParamNBegin{geom_traits}
+ *     \cgalParamDescription{an instance of a geometric traits class}
+ *     \cgalParamType{a model of `Kernel`}
+ *     \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+ *     \cgalParamExtra{Exact constructions kernels are not supported by this function.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{vertex_point_map}
+ *     \cgalParamDescription{a property map associating points to the vertices of `tm`}
+ *     \cgalParamType{a class model of `ReadablePropertyMap` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor`
+ *                    as key type and `%Point_3` as value type}
+ *     \cgalParamDefault{`boost::get(CGAL::vertex_point, tm)`}
+ *     \cgalParamExtra{If this parameter is omitted, an internal property map for `CGAL::vertex_point_t`
+ *                     must be available in `TriangleMesh`.}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{verbose_level}
+ *     \cgalParamDescription{the verbose level}
+ *     \cgalParamType{`CGAL::Surface_mesh_approximation::Verbose_level`}
+ *     \cgalParamDefault{`CGAL::Surface_mesh_approximation::SILENT`}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{seeding_method}
+ *     \cgalParamDescription{the selection of seeding method}
+ *     \cgalParamType{`CGAL::Surface_mesh_approximation::Seeding_method`}
+ *     \cgalParamDefault{`CGAL::Surface_mesh_approximation::HIERARCHICAL`}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{max_number_of_proxies}
+ *     \cgalParamDescription{the maximum number of proxies used to approximate the input mesh}
+ *     \cgalParamType{`std::size_t`}
+ *     \cgalParamDefault{`num_faces(tm) / 3`, used when `min_error_drop` is also not provided}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{min_error_drop}
+ *     \cgalParamDescription{the minimum error drop of the approximation, expressed as
+ *                           the ratio between two iterations of proxy addition}
+ *     \cgalParamType{`geom_traits::FT`}
+ *     \cgalParamDefault{`0.1`, used when `max_number_of_proxies` is also not provided}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{number_of_relaxations}
+ *     \cgalParamDescription{the number of relaxation iterations interleaved within seeding}
+ *     \cgalParamType{`std::size_t`}
+ *     \cgalParamDefault{`5`}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{number_of_iterations}
+ *     \cgalParamDescription{the number of partitioning and fitting iterations after seeding}
+ *     \cgalParamType{`std::size_t`}
+ *     \cgalParamDefault{`std::min(std::max(number_of_faces / max_number_of_proxies, 20), 60)`}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
+ *
+ *
  * \cgalNamedParamsBegin{Meshing Named Parameters}
- *  \cgalParamBegin{subdivision_ratio} chord subdivision ratio threshold to the chord length or average edge length.
- *  \cgalParamEnd
- *  \cgalParamBegin{relative_to_chord} if `true` the `subdivision_ratio` is the ratio of the
- *    furthest vertex distance to the chord length, otherwise is the average edge length.
- *  \cgalParamEnd
- *  \cgalParamBegin{with_dihedral_angle} if set to `true` the `subdivision_ratio` is weighted by dihedral angle.
- *  \cgalParamEnd
- *  \cgalParamBegin{optimize_anchor_location} if set to `true`, optimize the anchor locations.
- *  \cgalParamEnd
- *  \cgalParamBegin{pca_plane} set `true` if use PCA plane fitting, otherwise use the default area averaged plane parameters.
- *  \cgalParamEnd
+ *   \cgalParamNBegin{subdivision_ratio}
+ *     \cgalParamDescription{the chord subdivision ratio threshold to the chord length or average edge length}
+ *     \cgalParamType{`geom_traits::FT`}
+ *     \cgalParamDefault{`5.0`}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{relative_to_chord}
+ *     \cgalParamDescription{If `true`, the `subdivision_ratio` is the ratio of the furthest vertex distance
+ *                           to the chord length, otherwise is the average edge length}
+ *     \cgalParamType{`Boolean`}
+ *     \cgalParamDefault{`false`}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{with_dihedral_angle}
+ *     \cgalParamDescription{If `true`, the `subdivision_ratio` is weighted by dihedral angle}
+ *     \cgalParamType{`Boolean`}
+ *     \cgalParamDefault{`false`}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{optimize_anchor_location}
+ *     \cgalParamDescription{If `true`, optimize the anchor locations}
+ *     \cgalParamType{`Boolean`}
+ *     \cgalParamDefault{`true`}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{pca_plane}
+ *     \cgalParamDescription{If `true`, use PCA plane fitting, otherwise use the default area averaged plane parameters}
+ *     \cgalParamType{`Boolean`}
+ *     \cgalParamDefault{`false`}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
+ *
+ *
  * \cgalNamedParamsBegin{Output Named Parameters}
- *  \cgalParamBegin{face_proxy_map} a `WritablePropertyMap` with
- * `boost::graph_traits<TriangleMesh>::%face_descriptor` as key and `std::size_t` as value type.
- * A proxy is a set of connected faces which are placed under the same proxy patch (see \cgalFigureRef{iterations}).
- * The proxy-ids are contiguous in range [0, number_of_proxies - 1].
- *  \cgalParamEnd
- *  \cgalParamBegin{proxies} output iterator over proxies.
- *  \cgalParamEnd
- *  \cgalParamBegin{anchors} output iterator over anchor points.
- *  \cgalParamEnd
- *  \cgalParamBegin{triangles} output iterator over indexed triangles.
- *  \cgalParamEnd
+ *   \cgalParamNBegin{face_proxy_map}
+ *     \cgalParamDescription{a property map to output the proxy index of each face of the input polygon mesh}
+ *     \cgalParamType{a model of `WritablePropertyMap` with `boost::graph_traits<TriangleMesh>::%face_descriptor`
+ *                    as key and `std::size_t` as value type}
+ *     \cgalParamDefault{no output operation is performed}
+ *     \cgalParamExtra{A proxy is a set of connected faces which are placed under the same proxy patch (see \cgalFigureRef{iterations})}
+ *     \cgalParamExtra{The proxy-ids are contiguous in range `[0, number_of_proxies - 1]`}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{proxies}
+ *     \cgalParamDescription{an `OutputIterator` to put proxies in}
+ *     \cgalParamType{a class model of `OutputIterator` with
+ *                    `CGAL::Surface_mesh_approximation::L21_metric_vector_proxy_no_area_weighting::Proxy` value type}
+ *     \cgalParamDefault{no output operation is performed}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{anchors}
+ *     \cgalParamDescription{an `OutputIterator` to put anchor points in}
+ *     \cgalParamType{a class model of `OutputIterator` with `geom_traits::%Point_3` value type}
+ *     \cgalParamDefault{no output operation is performed}
+ *   \cgalParamNEnd
+ *
+ *   \cgalParamNBegin{triangles}
+ *     \cgalParamDescription{an `OutputIterator` to put indexed triangles in}
+ *     \cgalParamType{a class model of `OutputIterator` with `std::array<std::size_t, 3>` value type}
+ *     \cgalParamDefault{no output operation is performed}
+ *   \cgalParamNEnd
  * \cgalNamedParamsEnd
  */
 template <typename TriangleMesh, typename NamedParameters>
@@ -122,10 +195,10 @@ bool approximate_triangle_mesh(const Tri
   using parameters::choose_parameter;
   using parameters::is_default_parameter;
 
-  typedef typename Polygon_mesh_processing::GetGeomTraits<TriangleMesh, NamedParameters>::type Geom_traits;
+  typedef typename CGAL::GetGeomTraits<TriangleMesh, NamedParameters>::type Geom_traits;
   typedef typename Geom_traits::FT FT;
 
-  typedef typename Polygon_mesh_processing::GetVertexPointMap<TriangleMesh, NamedParameters>::type Vertex_point_map;
+  typedef typename CGAL::GetVertexPointMap<TriangleMesh, NamedParameters>::type Vertex_point_map;
   Vertex_point_map point_pmap = choose_parameter(get_parameter(np, internal_np::vertex_point),
     get_property_map(vertex_point, const_cast<TriangleMesh &>(tm)));
 
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_approximation/L21_metric_plane_proxy.h 5.1-1/include/CGAL/Surface_mesh_approximation/L21_metric_plane_proxy.h
--- 5.0.3-1/include/CGAL/Surface_mesh_approximation/L21_metric_plane_proxy.h	2020-08-01 19:02:54.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_approximation/L21_metric_plane_proxy.h	2020-09-08 12:15:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_approximation/include/CGAL/Surface_mesh_approximation/L21_metric_plane_proxy.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_approximation/include/CGAL/Surface_mesh_approximation/L21_metric_plane_proxy.h $
 // $Id: L21_metric_plane_proxy.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_approximation/L2_metric_plane_proxy.h 5.1-1/include/CGAL/Surface_mesh_approximation/L2_metric_plane_proxy.h
--- 5.0.3-1/include/CGAL/Surface_mesh_approximation/L2_metric_plane_proxy.h	2020-08-01 19:02:54.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_approximation/L2_metric_plane_proxy.h	2020-09-08 12:15:43.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_approximation/include/CGAL/Surface_mesh_approximation/L2_metric_plane_proxy.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_approximation/include/CGAL/Surface_mesh_approximation/L2_metric_plane_proxy.h $
 // $Id: L2_metric_plane_proxy.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_cell_base_3.h 5.1-1/include/CGAL/Surface_mesh_cell_base_3.h
--- 5.0.3-1/include/CGAL/Surface_mesh_cell_base_3.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_cell_base_3.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Surface_mesh_cell_base_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Surface_mesh_cell_base_3.h $
 // $Id: Surface_mesh_cell_base_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_complex_2_in_triangulation_3.h 5.1-1/include/CGAL/Surface_mesh_complex_2_in_triangulation_3.h
--- 5.0.3-1/include/CGAL/Surface_mesh_complex_2_in_triangulation_3.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_complex_2_in_triangulation_3.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Surface_mesh_complex_2_in_triangulation_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Surface_mesh_complex_2_in_triangulation_3.h $
 // $Id: Surface_mesh_complex_2_in_triangulation_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_default_criteria_3.h 5.1-1/include/CGAL/Surface_mesh_default_criteria_3.h
--- 5.0.3-1/include/CGAL/Surface_mesh_default_criteria_3.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_default_criteria_3.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Surface_mesh_default_criteria_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Surface_mesh_default_criteria_3.h $
 // $Id: Surface_mesh_default_criteria_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_default_edges_criteria_3.h 5.1-1/include/CGAL/Surface_mesh_default_edges_criteria_3.h
--- 5.0.3-1/include/CGAL/Surface_mesh_default_edges_criteria_3.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_default_edges_criteria_3.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Surface_mesh_default_edges_criteria_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Surface_mesh_default_edges_criteria_3.h $
 // $Id: Surface_mesh_default_edges_criteria_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_default_triangulation_3.h 5.1-1/include/CGAL/Surface_mesh_default_triangulation_3.h
--- 5.0.3-1/include/CGAL/Surface_mesh_default_triangulation_3.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_default_triangulation_3.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Surface_mesh_default_triangulation_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Surface_mesh_default_triangulation_3.h $
 // $Id: Surface_mesh_default_triangulation_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_deformation.h 5.1-1/include/CGAL/Surface_mesh_deformation.h
--- 5.0.3-1/include/CGAL/Surface_mesh_deformation.h	2020-08-01 19:02:54.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_deformation.h	2020-09-08 12:15:43.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_deformation/include/CGAL/Surface_mesh_deformation.h $
-// $Id: Surface_mesh_deformation.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_deformation/include/CGAL/Surface_mesh_deformation.h $
+// $Id: Surface_mesh_deformation.h 05f0f95 2020-03-09T10:23:16+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Yin Xu, Andreas Fabri and Ilker O. Yaz
@@ -16,10 +16,11 @@
 
 #include <CGAL/disable_warnings.h>
 
+#include <CGAL/assertions.h>
+#include <CGAL/boost/graph/named_params_helper.h>
 #include <CGAL/config.h>
 #include <CGAL/Default.h>
 #include <CGAL/tuple.h>
-
 #include <CGAL/Polygon_mesh_processing/Weights.h>
 #include <CGAL/Simple_cartesian.h>
 
@@ -215,13 +216,9 @@ public:
 // Index maps
 #ifndef DOXYGEN_RUNNING
   typedef typename Default::Get<
-    VIM,
-    typename boost::property_map<Triangle_mesh, boost::vertex_index_t>::type
-  >::type Vertex_index_map;
+    VIM, typename CGAL::GetInitializedVertexIndexMap<Triangle_mesh>::type>::type Vertex_index_map;
   typedef typename Default::Get<
-    HIM,
-    typename boost::property_map<Triangle_mesh, boost::halfedge_index_t>::type
-  >::type Hedge_index_map;
+    HIM, typename CGAL::GetInitializedHalfedgeIndexMap<Triangle_mesh>::type>::type Hedge_index_map;
 #else
   /// vertex index map type
   typedef VIM Vertex_index_map;
@@ -356,9 +353,10 @@ public:
   //vertex_point_map set by default
   Surface_mesh_deformation(Triangle_mesh& triangle_mesh,
                            Vertex_index_map vertex_index_map,
-                           Hedge_index_map hedge_index_map
-                          )
-    : m_triangle_mesh(triangle_mesh), vertex_index_map(vertex_index_map), hedge_index_map(hedge_index_map),
+                           Hedge_index_map hedge_index_map)
+    : m_triangle_mesh(triangle_mesh),
+      vertex_index_map(vertex_index_map),
+      hedge_index_map(hedge_index_map),
       ros_id_map(std::vector<std::size_t>(num_vertices(triangle_mesh), (std::numeric_limits<std::size_t>::max)() )),
       is_roi_map(std::vector<bool>(num_vertices(triangle_mesh), false)),
       is_ctrl_map(std::vector<bool>(num_vertices(triangle_mesh), false)),
@@ -374,10 +372,10 @@ public:
 
   //vertex_point_map and hedge_index_map set by default
   Surface_mesh_deformation(Triangle_mesh& triangle_mesh,
-                           Vertex_index_map vertex_index_map
-                          )
-    : m_triangle_mesh(triangle_mesh), vertex_index_map(vertex_index_map),
-      hedge_index_map(get(boost::halfedge_index, triangle_mesh)),
+                           Vertex_index_map vertex_index_map)
+    : m_triangle_mesh(triangle_mesh),
+      vertex_index_map(vertex_index_map),
+      hedge_index_map(CGAL::get_initialized_halfedge_index_map(triangle_mesh)),
       ros_id_map(std::vector<std::size_t>(num_vertices(triangle_mesh), (std::numeric_limits<std::size_t>::max)() )),
       is_roi_map(std::vector<bool>(num_vertices(triangle_mesh), false)),
       is_ctrl_map(std::vector<bool>(num_vertices(triangle_mesh), false)),
@@ -393,8 +391,8 @@ public:
   //vertex_point_map, hedge_index_map and vertex_index_map set by default
   Surface_mesh_deformation(Triangle_mesh& triangle_mesh)
     : m_triangle_mesh(triangle_mesh),
-      vertex_index_map(get(boost::vertex_index, triangle_mesh)),
-      hedge_index_map(get(boost::halfedge_index, triangle_mesh)),
+      vertex_index_map(CGAL::get_initialized_vertex_index_map(triangle_mesh)),
+      hedge_index_map(CGAL::get_initialized_halfedge_index_map(triangle_mesh)),
       ros_id_map(std::vector<std::size_t>(num_vertices(triangle_mesh), (std::numeric_limits<std::size_t>::max)() )),
       is_roi_map(std::vector<bool>(num_vertices(triangle_mesh), false)),
       is_ctrl_map(std::vector<bool>(num_vertices(triangle_mesh), false)),
@@ -413,18 +411,19 @@ public:
                            Vertex_index_map vertex_index_map,
                            Hedge_index_map hedge_index_map,
                            Vertex_point_map vertex_point_map,
-                           Weight_calculator weight_calculator = Weight_calculator()
-                          )
-    : m_triangle_mesh(triangle_mesh), vertex_index_map(vertex_index_map), hedge_index_map(hedge_index_map),
-    ros_id_map(std::vector<std::size_t>(num_vertices(triangle_mesh), (std::numeric_limits<std::size_t>::max)() )),
-    is_roi_map(std::vector<bool>(num_vertices(triangle_mesh), false)),
-    is_ctrl_map(std::vector<bool>(num_vertices(triangle_mesh), false)),
-    m_iterations(5), m_tolerance(1e-4),
-    need_preprocess_factorization(true),
-    need_preprocess_region_of_solution(true),
-    last_preprocess_successful(false),
-    weight_calculator(weight_calculator),
-    vertex_point_map(vertex_point_map)
+                           Weight_calculator weight_calculator = Weight_calculator())
+    : m_triangle_mesh(triangle_mesh),
+      vertex_index_map(vertex_index_map),
+      hedge_index_map(hedge_index_map),
+      ros_id_map(std::vector<std::size_t>(num_vertices(triangle_mesh), (std::numeric_limits<std::size_t>::max)() )),
+      is_roi_map(std::vector<bool>(num_vertices(triangle_mesh), false)),
+      is_ctrl_map(std::vector<bool>(num_vertices(triangle_mesh), false)),
+      m_iterations(5), m_tolerance(1e-4),
+      need_preprocess_factorization(true),
+      need_preprocess_region_of_solution(true),
+      last_preprocess_successful(false),
+      weight_calculator(weight_calculator),
+      vertex_point_map(vertex_point_map)
   {
     init();
   }
@@ -433,21 +432,23 @@ public:
 /// \name Construction
 /// @{
     /**
-   * The constructor of a deformation object
+   * The constructor of a deformation object.
    *
    * @pre `triangle_mesh` consists of only triangular facets
    * @param triangle_mesh triangulated surface mesh to deform
-   * @param vertex_index_map property map which associates an id to each vertex, from `0` to `num_vertices(triangle_mesh)-1`.
-   * @param hedge_index_map property map which associates an id to each halfedge, from `0` to `2*num_edges(triangle_mesh)-1`.
+   * @param vertex_index_map a property map which associates a unique id to each vertex,
+   *                         between `0` to `num_vertices(triangle_mesh)-1`.
+   * @param hedge_index_map property map which associates a unique id to each halfedge,
+   *                        between `0` to `2*num_edges(triangle_mesh)-1`.
    * @param vertex_point_map property map which associates a point to each vertex of the triangle mesh.
    * @param weight_calculator function object or pointer for weight calculation
+   *
    */
   Surface_mesh_deformation(Triangle_mesh& triangle_mesh,
-    Vertex_index_map vertex_index_map=get(boost::vertex_index, triangle_mesh),
-    Hedge_index_map hedge_index_map=get(boost::halfedge_index, triangle_mesh),
-    Vertex_point_map vertex_point_map=get(boost::vertex_point, triangle_mesh),
-    Weight_calculator weight_calculator = Weight_calculator()
-    );
+                           Vertex_index_map vertex_index_map = unspecified_internal_vertex_index_map,
+                           Hedge_index_map hedge_index_map = unspecified_internal_halfedge_index_map,
+                           Vertex_point_map vertex_point_map = get(boost::vertex_point, triangle_mesh),
+                           Weight_calculator weight_calculator = Weight_calculator());
 /// @}
   #endif
 
diff -pruN 5.0.3-1/include/CGAL/Surface_mesher/Combining_oracle.h 5.1-1/include/CGAL/Surface_mesher/Combining_oracle.h
--- 5.0.3-1/include/CGAL/Surface_mesher/Combining_oracle.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesher/Combining_oracle.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Surface_mesher/Combining_oracle.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Surface_mesher/Combining_oracle.h $
 // $Id: Combining_oracle.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesher/Has_edges.h 5.1-1/include/CGAL/Surface_mesher/Has_edges.h
--- 5.0.3-1/include/CGAL/Surface_mesher/Has_edges.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesher/Has_edges.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Surface_mesher/Has_edges.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Surface_mesher/Has_edges.h $
 // $Id: Has_edges.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesher/Implicit_surface_oracle_3.h 5.1-1/include/CGAL/Surface_mesher/Implicit_surface_oracle_3.h
--- 5.0.3-1/include/CGAL/Surface_mesher/Implicit_surface_oracle_3.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesher/Implicit_surface_oracle_3.h	2020-09-08 12:15:47.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Surface_mesher/Implicit_surface_oracle_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Surface_mesher/Implicit_surface_oracle_3.h $
 // $Id: Implicit_surface_oracle_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesher/Intersection_data_structure_3.h 5.1-1/include/CGAL/Surface_mesher/Intersection_data_structure_3.h
--- 5.0.3-1/include/CGAL/Surface_mesher/Intersection_data_structure_3.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesher/Intersection_data_structure_3.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Surface_mesher/Intersection_data_structure_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Surface_mesher/Intersection_data_structure_3.h $
 // $Id: Intersection_data_structure_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesher/Null_oracle_visitor.h 5.1-1/include/CGAL/Surface_mesher/Null_oracle_visitor.h
--- 5.0.3-1/include/CGAL/Surface_mesher/Null_oracle_visitor.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesher/Null_oracle_visitor.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Surface_mesher/Null_oracle_visitor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Surface_mesher/Null_oracle_visitor.h $
 // $Id: Null_oracle_visitor.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesher/Point_surface_indices_oracle_visitor.h 5.1-1/include/CGAL/Surface_mesher/Point_surface_indices_oracle_visitor.h
--- 5.0.3-1/include/CGAL/Surface_mesher/Point_surface_indices_oracle_visitor.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesher/Point_surface_indices_oracle_visitor.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Surface_mesher/Point_surface_indices_oracle_visitor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Surface_mesher/Point_surface_indices_oracle_visitor.h $
 // $Id: Point_surface_indices_oracle_visitor.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesher/Poisson_implicit_surface_oracle_3.h 5.1-1/include/CGAL/Surface_mesher/Poisson_implicit_surface_oracle_3.h
--- 5.0.3-1/include/CGAL/Surface_mesher/Poisson_implicit_surface_oracle_3.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesher/Poisson_implicit_surface_oracle_3.h	2020-09-08 12:15:22.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Poisson_surface_reconstruction_3/include/CGAL/Surface_mesher/Poisson_implicit_surface_oracle_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Poisson_surface_reconstruction_3/include/CGAL/Surface_mesher/Poisson_implicit_surface_oracle_3.h $
 // $Id: Poisson_implicit_surface_oracle_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesher/Polyhedral_oracle.h 5.1-1/include/CGAL/Surface_mesher/Polyhedral_oracle.h
--- 5.0.3-1/include/CGAL/Surface_mesher/Polyhedral_oracle.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesher/Polyhedral_oracle.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Surface_mesher/Polyhedral_oracle.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Surface_mesher/Polyhedral_oracle.h $
 // $Id: Polyhedral_oracle.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesher/Profile_counter.h 5.1-1/include/CGAL/Surface_mesher/Profile_counter.h
--- 5.0.3-1/include/CGAL/Surface_mesher/Profile_counter.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesher/Profile_counter.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Surface_mesher/Profile_counter.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Surface_mesher/Profile_counter.h $
 // $Id: Profile_counter.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesher/Profile_timer.h 5.1-1/include/CGAL/Surface_mesher/Profile_timer.h
--- 5.0.3-1/include/CGAL/Surface_mesher/Profile_timer.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesher/Profile_timer.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Surface_mesher/Profile_timer.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Surface_mesher/Profile_timer.h $
 // $Id: Profile_timer.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesher/Sphere_oracle_3.h 5.1-1/include/CGAL/Surface_mesher/Sphere_oracle_3.h
--- 5.0.3-1/include/CGAL/Surface_mesher/Sphere_oracle_3.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesher/Sphere_oracle_3.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Surface_mesher/Sphere_oracle_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Surface_mesher/Sphere_oracle_3.h $
 // $Id: Sphere_oracle_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesher/Standard_criteria.h 5.1-1/include/CGAL/Surface_mesher/Standard_criteria.h
--- 5.0.3-1/include/CGAL/Surface_mesher/Standard_criteria.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesher/Standard_criteria.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Surface_mesher/Standard_criteria.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Surface_mesher/Standard_criteria.h $
 // $Id: Standard_criteria.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesher/Surface_mesher_edges_level.h 5.1-1/include/CGAL/Surface_mesher/Surface_mesher_edges_level.h
--- 5.0.3-1/include/CGAL/Surface_mesher/Surface_mesher_edges_level.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesher/Surface_mesher_edges_level.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Surface_mesher/Surface_mesher_edges_level.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Surface_mesher/Surface_mesher_edges_level.h $
 // $Id: Surface_mesher_edges_level.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesher/Surface_mesher_edges_level_visitor.h 5.1-1/include/CGAL/Surface_mesher/Surface_mesher_edges_level_visitor.h
--- 5.0.3-1/include/CGAL/Surface_mesher/Surface_mesher_edges_level_visitor.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesher/Surface_mesher_edges_level_visitor.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Surface_mesher/Surface_mesher_edges_level_visitor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Surface_mesher/Surface_mesher_edges_level_visitor.h $
 // $Id: Surface_mesher_edges_level_visitor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesher/Surface_mesher.h 5.1-1/include/CGAL/Surface_mesher/Surface_mesher.h
--- 5.0.3-1/include/CGAL/Surface_mesher/Surface_mesher.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesher/Surface_mesher.h	2020-09-08 12:15:47.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Surface_mesher/Surface_mesher.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Surface_mesher/Surface_mesher.h $
 // $Id: Surface_mesher.h 627a7b2 2020-04-27T17:08:35+02:00 Maxime Gimeno
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesher/Surface_mesher_manifold.h 5.1-1/include/CGAL/Surface_mesher/Surface_mesher_manifold.h
--- 5.0.3-1/include/CGAL/Surface_mesher/Surface_mesher_manifold.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesher/Surface_mesher_manifold.h	2020-09-08 12:15:47.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Surface_mesher/Surface_mesher_manifold.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Surface_mesher/Surface_mesher_manifold.h $
 // $Id: Surface_mesher_manifold.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesher/Surface_mesher_regular_edges.h 5.1-1/include/CGAL/Surface_mesher/Surface_mesher_regular_edges.h
--- 5.0.3-1/include/CGAL/Surface_mesher/Surface_mesher_regular_edges.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesher/Surface_mesher_regular_edges.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Surface_mesher/Surface_mesher_regular_edges.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Surface_mesher/Surface_mesher_regular_edges.h $
 // $Id: Surface_mesher_regular_edges.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesher/Surface_mesher_visitor.h 5.1-1/include/CGAL/Surface_mesher/Surface_mesher_visitor.h
--- 5.0.3-1/include/CGAL/Surface_mesher/Surface_mesher_visitor.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesher/Surface_mesher_visitor.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Surface_mesher/Surface_mesher_visitor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Surface_mesher/Surface_mesher_visitor.h $
 // $Id: Surface_mesher_visitor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesher/Types_generators.h 5.1-1/include/CGAL/Surface_mesher/Types_generators.h
--- 5.0.3-1/include/CGAL/Surface_mesher/Types_generators.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesher/Types_generators.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Surface_mesher/Types_generators.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Surface_mesher/Types_generators.h $
 // $Id: Types_generators.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesher/Verbose_flag.h 5.1-1/include/CGAL/Surface_mesher/Verbose_flag.h
--- 5.0.3-1/include/CGAL/Surface_mesher/Verbose_flag.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesher/Verbose_flag.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Surface_mesher/Verbose_flag.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Surface_mesher/Verbose_flag.h $
 // $Id: Verbose_flag.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesher/Vertices_on_the_same_psc_element_criterion.h 5.1-1/include/CGAL/Surface_mesher/Vertices_on_the_same_psc_element_criterion.h
--- 5.0.3-1/include/CGAL/Surface_mesher/Vertices_on_the_same_psc_element_criterion.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesher/Vertices_on_the_same_psc_element_criterion.h	2020-09-08 12:15:47.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Surface_mesher/Vertices_on_the_same_psc_element_criterion.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Surface_mesher/Vertices_on_the_same_psc_element_criterion.h $
 // $Id: Vertices_on_the_same_psc_element_criterion.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesher/Vertices_on_the_same_surface_criterion.h 5.1-1/include/CGAL/Surface_mesher/Vertices_on_the_same_surface_criterion.h
--- 5.0.3-1/include/CGAL/Surface_mesher/Vertices_on_the_same_surface_criterion.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesher/Vertices_on_the_same_surface_criterion.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Surface_mesher/Vertices_on_the_same_surface_criterion.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Surface_mesher/Vertices_on_the_same_surface_criterion.h $
 // $Id: Vertices_on_the_same_surface_criterion.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesher_generator.h 5.1-1/include/CGAL/Surface_mesher_generator.h
--- 5.0.3-1/include/CGAL/Surface_mesher_generator.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesher_generator.h	2020-09-08 12:15:47.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Surface_mesher_generator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Surface_mesher_generator.h $
 // $Id: Surface_mesher_generator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh.h 5.1-1/include/CGAL/Surface_mesh.h
--- 5.0.3-1/include/CGAL/Surface_mesh.h	2020-08-01 19:02:54.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh.h	2020-09-08 12:15:43.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh/include/CGAL/Surface_mesh.h $
-// $Id: Surface_mesh.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh/include/CGAL/Surface_mesh.h $
+// $Id: Surface_mesh.h 2f4e49b 2020-03-23T09:47:01+01:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Phlipp Moeller
@@ -21,7 +21,7 @@
 namespace CGAL {
 
 // fwdS for the public interface
-template<typename K>
+template<typename P>
 class Surface_mesh;
 
 }
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_parameterization/ARAP_parameterizer_3.h 5.1-1/include/CGAL/Surface_mesh_parameterization/ARAP_parameterizer_3.h
--- 5.0.3-1/include/CGAL/Surface_mesh_parameterization/ARAP_parameterizer_3.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_parameterization/ARAP_parameterizer_3.h	2020-09-08 12:15:43.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/ARAP_parameterizer_3.h $
-// $Id: ARAP_parameterizer_3.h e872a79 2020-01-15T17:31:47+01:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/ARAP_parameterizer_3.h $
+// $Id: ARAP_parameterizer_3.h 4a9eea9 2020-01-15T17:19:19+01:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Mael Rouxel-Labbé
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_parameterization/Barycentric_mapping_parameterizer_3.h 5.1-1/include/CGAL/Surface_mesh_parameterization/Barycentric_mapping_parameterizer_3.h
--- 5.0.3-1/include/CGAL/Surface_mesh_parameterization/Barycentric_mapping_parameterizer_3.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_parameterization/Barycentric_mapping_parameterizer_3.h	2020-09-08 12:15:43.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/Barycentric_mapping_parameterizer_3.h $
-// $Id: Barycentric_mapping_parameterizer_3.h e872a79 2020-01-15T17:31:47+01:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/Barycentric_mapping_parameterizer_3.h $
+// $Id: Barycentric_mapping_parameterizer_3.h 4a9eea9 2020-01-15T17:19:19+01:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Laurent Saboret, Pierre Alliez, Bruno Levy
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_parameterization/Circular_border_parameterizer_3.h 5.1-1/include/CGAL/Surface_mesh_parameterization/Circular_border_parameterizer_3.h
--- 5.0.3-1/include/CGAL/Surface_mesh_parameterization/Circular_border_parameterizer_3.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_parameterization/Circular_border_parameterizer_3.h	2020-09-08 12:15:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/Circular_border_parameterizer_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/Circular_border_parameterizer_3.h $
 // $Id: Circular_border_parameterizer_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_parameterization/Discrete_authalic_parameterizer_3.h 5.1-1/include/CGAL/Surface_mesh_parameterization/Discrete_authalic_parameterizer_3.h
--- 5.0.3-1/include/CGAL/Surface_mesh_parameterization/Discrete_authalic_parameterizer_3.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_parameterization/Discrete_authalic_parameterizer_3.h	2020-09-08 12:15:44.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/Discrete_authalic_parameterizer_3.h $
-// $Id: Discrete_authalic_parameterizer_3.h e872a79 2020-01-15T17:31:47+01:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/Discrete_authalic_parameterizer_3.h $
+// $Id: Discrete_authalic_parameterizer_3.h 4a9eea9 2020-01-15T17:19:19+01:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Laurent Saboret, Pierre Alliez, Bruno Levy
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_parameterization/Discrete_conformal_map_parameterizer_3.h 5.1-1/include/CGAL/Surface_mesh_parameterization/Discrete_conformal_map_parameterizer_3.h
--- 5.0.3-1/include/CGAL/Surface_mesh_parameterization/Discrete_conformal_map_parameterizer_3.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_parameterization/Discrete_conformal_map_parameterizer_3.h	2020-09-08 12:15:44.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/Discrete_conformal_map_parameterizer_3.h $
-// $Id: Discrete_conformal_map_parameterizer_3.h e872a79 2020-01-15T17:31:47+01:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/Discrete_conformal_map_parameterizer_3.h $
+// $Id: Discrete_conformal_map_parameterizer_3.h 4a9eea9 2020-01-15T17:19:19+01:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Laurent Saboret, Pierre Alliez, Bruno Levy
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_parameterization/Error_code.h 5.1-1/include/CGAL/Surface_mesh_parameterization/Error_code.h
--- 5.0.3-1/include/CGAL/Surface_mesh_parameterization/Error_code.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_parameterization/Error_code.h	2020-09-08 12:15:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/Error_code.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/Error_code.h $
 // $Id: Error_code.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_parameterization/Fixed_border_parameterizer_3.h 5.1-1/include/CGAL/Surface_mesh_parameterization/Fixed_border_parameterizer_3.h
--- 5.0.3-1/include/CGAL/Surface_mesh_parameterization/Fixed_border_parameterizer_3.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_parameterization/Fixed_border_parameterizer_3.h	2020-09-08 12:15:44.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/Fixed_border_parameterizer_3.h $
-// $Id: Fixed_border_parameterizer_3.h e872a79 2020-01-15T17:31:47+01:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/Fixed_border_parameterizer_3.h $
+// $Id: Fixed_border_parameterizer_3.h 4a9eea9 2020-01-15T17:19:19+01:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Laurent Saboret, Pierre Alliez, Bruno Levy
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_parameterization/internal/angles.h 5.1-1/include/CGAL/Surface_mesh_parameterization/internal/angles.h
--- 5.0.3-1/include/CGAL/Surface_mesh_parameterization/internal/angles.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_parameterization/internal/angles.h	2020-09-08 12:15:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/internal/angles.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/internal/angles.h $
 // $Id: angles.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_parameterization/internal/Bool_property_map.h 5.1-1/include/CGAL/Surface_mesh_parameterization/internal/Bool_property_map.h
--- 5.0.3-1/include/CGAL/Surface_mesh_parameterization/internal/Bool_property_map.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_parameterization/internal/Bool_property_map.h	2020-09-08 12:15:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/internal/Bool_property_map.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/internal/Bool_property_map.h $
 // $Id: Bool_property_map.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_parameterization/internal/Containers_filler.h 5.1-1/include/CGAL/Surface_mesh_parameterization/internal/Containers_filler.h
--- 5.0.3-1/include/CGAL/Surface_mesh_parameterization/internal/Containers_filler.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_parameterization/internal/Containers_filler.h	2020-09-08 12:15:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/internal/Containers_filler.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/internal/Containers_filler.h $
 // $Id: Containers_filler.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_parameterization/internal/kernel_traits.h 5.1-1/include/CGAL/Surface_mesh_parameterization/internal/kernel_traits.h
--- 5.0.3-1/include/CGAL/Surface_mesh_parameterization/internal/kernel_traits.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_parameterization/internal/kernel_traits.h	2020-09-08 12:15:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/internal/kernel_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/internal/kernel_traits.h $
 // $Id: kernel_traits.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_parameterization/internal/orbifold_cone_helper.h 5.1-1/include/CGAL/Surface_mesh_parameterization/internal/orbifold_cone_helper.h
--- 5.0.3-1/include/CGAL/Surface_mesh_parameterization/internal/orbifold_cone_helper.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_parameterization/internal/orbifold_cone_helper.h	2020-09-08 12:15:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/internal/orbifold_cone_helper.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/internal/orbifold_cone_helper.h $
 // $Id: orbifold_cone_helper.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_parameterization/internal/validity.h 5.1-1/include/CGAL/Surface_mesh_parameterization/internal/validity.h
--- 5.0.3-1/include/CGAL/Surface_mesh_parameterization/internal/validity.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_parameterization/internal/validity.h	2020-09-08 12:15:44.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/internal/validity.h $
-// $Id: validity.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/internal/validity.h $
+// $Id: validity.h 41bb11c 2019-11-19T17:08:25+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Mael Rouxel-Labbé
@@ -113,7 +113,8 @@ class Intersect_facets
   typename Kernel::Construct_triangle_2 triangle_functor;
   typename Kernel::Do_intersect_2 do_intersect_2_functor;
 
-  typedef CGAL::Box_intersection_d::Box_with_info_d<NT, 2, face_descriptor> Box;
+  typedef CGAL::Box_intersection_d::ID_FROM_BOX_ADDRESS Box_policy;
+  typedef CGAL::Box_intersection_d::Box_with_info_d<NT, 2, face_descriptor, Box_policy> Box;
 
   const TriangleMesh& mesh;
   const VertexUVMap uvmap;
@@ -250,7 +251,8 @@ bool is_one_to_one_mapping(const Triangl
   typedef typename Kernel::FT                                         NT;
   typedef typename Kernel::Point_2                                    Point_2;
 
-  typedef CGAL::Box_intersection_d::Box_with_info_d<NT, 2, face_descriptor> Box;
+  typedef CGAL::Box_intersection_d::ID_FROM_BOX_ADDRESS Box_policy;
+  typedef CGAL::Box_intersection_d::Box_with_info_d<NT, 2, face_descriptor, Box_policy> Box;
 
   // Create the corresponding vector of bounding boxes
   std::vector<Box> boxes;
@@ -283,8 +285,7 @@ bool is_one_to_one_mapping(const Triangl
   unsigned int counter = 0;
   Intersect_facets<TriangleMesh, VertexUVMap> intersect_facets(mesh, uvmap, counter);
   std::ptrdiff_t cutoff = 2000;
-  CGAL::box_self_intersection_d(boxes_ptr.begin(), boxes_ptr.end(),
-                                intersect_facets, cutoff);
+  CGAL::box_self_intersection_d(boxes_ptr.begin(), boxes_ptr.end(), intersect_facets, cutoff);
   return (counter == 0);
 }
 
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_parameterization/IO/File_off.h 5.1-1/include/CGAL/Surface_mesh_parameterization/IO/File_off.h
--- 5.0.3-1/include/CGAL/Surface_mesh_parameterization/IO/File_off.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_parameterization/IO/File_off.h	2020-09-08 12:15:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/IO/File_off.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/IO/File_off.h $
 // $Id: File_off.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_parameterization/LSCM_parameterizer_3.h 5.1-1/include/CGAL/Surface_mesh_parameterization/LSCM_parameterizer_3.h
--- 5.0.3-1/include/CGAL/Surface_mesh_parameterization/LSCM_parameterizer_3.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_parameterization/LSCM_parameterizer_3.h	2020-09-08 12:15:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/LSCM_parameterizer_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/LSCM_parameterizer_3.h $
 // $Id: LSCM_parameterizer_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_parameterization/Mean_value_coordinates_parameterizer_3.h 5.1-1/include/CGAL/Surface_mesh_parameterization/Mean_value_coordinates_parameterizer_3.h
--- 5.0.3-1/include/CGAL/Surface_mesh_parameterization/Mean_value_coordinates_parameterizer_3.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_parameterization/Mean_value_coordinates_parameterizer_3.h	2020-09-08 12:15:44.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/Mean_value_coordinates_parameterizer_3.h $
-// $Id: Mean_value_coordinates_parameterizer_3.h e872a79 2020-01-15T17:31:47+01:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/Mean_value_coordinates_parameterizer_3.h $
+// $Id: Mean_value_coordinates_parameterizer_3.h 4a9eea9 2020-01-15T17:19:19+01:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Laurent Saboret, Pierre Alliez, Bruno Levy
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_parameterization/MVC_post_processor_3.h 5.1-1/include/CGAL/Surface_mesh_parameterization/MVC_post_processor_3.h
--- 5.0.3-1/include/CGAL/Surface_mesh_parameterization/MVC_post_processor_3.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_parameterization/MVC_post_processor_3.h	2020-09-08 12:15:44.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/MVC_post_processor_3.h $
-// $Id: MVC_post_processor_3.h e872a79 2020-01-15T17:31:47+01:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/MVC_post_processor_3.h $
+// $Id: MVC_post_processor_3.h 4a9eea9 2020-01-15T17:19:19+01:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Mael Rouxel-Labbé
@@ -127,17 +127,16 @@ private:
 
   // Types used for the convexification of the mesh
     // Each triangulation vertex is associated its corresponding vertex_descriptor
-  typedef CGAL::Triangulation_vertex_base_with_info_2<vertex_descriptor,
-                                                      Kernel>       Vb;
-    // Each triangultaion face is associated a color (inside/outside information)
-  typedef CGAL::Triangulation_face_base_with_info_2<int, Kernel>    Fb;
-  typedef CGAL::Constrained_triangulation_face_base_2<Kernel, Fb>   Cfb;
-  typedef CGAL::Triangulation_data_structure_2<Vb, Cfb>             TDS;
-  typedef CGAL::No_intersection_tag                                 Itag;
+  typedef CGAL::Triangulation_vertex_base_with_info_2<vertex_descriptor, Kernel>  Vb;
+    // Each triangulation face is associated a color (inside/outside information)
+  typedef CGAL::Triangulation_face_base_with_info_2<int, Kernel>                  Fb;
+  typedef CGAL::Constrained_triangulation_face_base_2<Kernel, Fb>                 Cfb;
+  typedef CGAL::Triangulation_data_structure_2<Vb, Cfb>                           TDS;
+  typedef CGAL::No_constraint_intersection_requiring_constructions_tag            Itag;
 
     // Can choose either a triangulation or a Delaunay triangulation
-  typedef CGAL::Constrained_triangulation_2<Kernel, TDS, Itag>                CT;
-//    typedef CGAL::Constrained_Delaunay_triangulation_2<Kernel, TDS, Itag>   CT;
+  typedef CGAL::Constrained_triangulation_2<Kernel, TDS, Itag>                    CT;
+//  typedef CGAL::Constrained_Delaunay_triangulation_2<Kernel, TDS, Itag>           CT;
 
 // Private fields
 private:
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_parameterization/orbifold_enums.h 5.1-1/include/CGAL/Surface_mesh_parameterization/orbifold_enums.h
--- 5.0.3-1/include/CGAL/Surface_mesh_parameterization/orbifold_enums.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_parameterization/orbifold_enums.h	2020-09-08 12:15:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/orbifold_enums.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/orbifold_enums.h $
 // $Id: orbifold_enums.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_parameterization/orbifold_shortest_path.h 5.1-1/include/CGAL/Surface_mesh_parameterization/orbifold_shortest_path.h
--- 5.0.3-1/include/CGAL/Surface_mesh_parameterization/orbifold_shortest_path.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_parameterization/orbifold_shortest_path.h	2020-09-08 12:15:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/orbifold_shortest_path.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/orbifold_shortest_path.h $
 // $Id: orbifold_shortest_path.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_parameterization/Orbifold_Tutte_parameterizer_3.h 5.1-1/include/CGAL/Surface_mesh_parameterization/Orbifold_Tutte_parameterizer_3.h
--- 5.0.3-1/include/CGAL/Surface_mesh_parameterization/Orbifold_Tutte_parameterizer_3.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_parameterization/Orbifold_Tutte_parameterizer_3.h	2020-09-08 12:15:44.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/Orbifold_Tutte_parameterizer_3.h $
-// $Id: Orbifold_Tutte_parameterizer_3.h e872a79 2020-01-15T17:31:47+01:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/Orbifold_Tutte_parameterizer_3.h $
+// $Id: Orbifold_Tutte_parameterizer_3.h fba9a19 2020-03-09T10:03:31+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Mael Rouxel-Labbé
@@ -25,6 +25,7 @@
 #include <CGAL/Surface_mesh_parameterization/Error_code.h>
 #include <CGAL/Surface_mesh_parameterization/orbifold_shortest_path.h>
 
+#include <CGAL/assertions.h>
 #include <CGAL/Polygon_mesh_processing/Weights.h>
 
 #include <CGAL/assertions.h>
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_parameterization/parameterize.h 5.1-1/include/CGAL/Surface_mesh_parameterization/parameterize.h
--- 5.0.3-1/include/CGAL/Surface_mesh_parameterization/parameterize.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_parameterization/parameterize.h	2020-09-08 12:15:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/parameterize.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/parameterize.h $
 // $Id: parameterize.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_parameterization/Square_border_parameterizer_3.h 5.1-1/include/CGAL/Surface_mesh_parameterization/Square_border_parameterizer_3.h
--- 5.0.3-1/include/CGAL/Surface_mesh_parameterization/Square_border_parameterizer_3.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_parameterization/Square_border_parameterizer_3.h	2020-09-08 12:15:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/Square_border_parameterizer_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/Square_border_parameterizer_3.h $
 // $Id: Square_border_parameterizer_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_parameterization/Two_vertices_parameterizer_3.h 5.1-1/include/CGAL/Surface_mesh_parameterization/Two_vertices_parameterizer_3.h
--- 5.0.3-1/include/CGAL/Surface_mesh_parameterization/Two_vertices_parameterizer_3.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_parameterization/Two_vertices_parameterizer_3.h	2020-09-08 12:15:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/Two_vertices_parameterizer_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_parameterization/include/CGAL/Surface_mesh_parameterization/Two_vertices_parameterizer_3.h $
 // $Id: Two_vertices_parameterizer_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/surface_mesh_parameterization.h 5.1-1/include/CGAL/surface_mesh_parameterization.h
--- 5.0.3-1/include/CGAL/surface_mesh_parameterization.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/surface_mesh_parameterization.h	2020-09-08 12:15:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_parameterization/include/CGAL/surface_mesh_parameterization.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_parameterization/include/CGAL/surface_mesh_parameterization.h $
 // $Id: surface_mesh_parameterization.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_shortest_path/barycentric.h 5.1-1/include/CGAL/Surface_mesh_shortest_path/barycentric.h
--- 5.0.3-1/include/CGAL/Surface_mesh_shortest_path/barycentric.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_shortest_path/barycentric.h	2020-09-08 12:15:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_shortest_path/include/CGAL/Surface_mesh_shortest_path/barycentric.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_shortest_path/include/CGAL/Surface_mesh_shortest_path/barycentric.h $
 // $Id: barycentric.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_shortest_path/function_objects.h 5.1-1/include/CGAL/Surface_mesh_shortest_path/function_objects.h
--- 5.0.3-1/include/CGAL/Surface_mesh_shortest_path/function_objects.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_shortest_path/function_objects.h	2020-09-08 12:15:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_shortest_path/include/CGAL/Surface_mesh_shortest_path/function_objects.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_shortest_path/include/CGAL/Surface_mesh_shortest_path/function_objects.h $
 // $Id: function_objects.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_shortest_path/internal/Cone_expansion_event.h 5.1-1/include/CGAL/Surface_mesh_shortest_path/internal/Cone_expansion_event.h
--- 5.0.3-1/include/CGAL/Surface_mesh_shortest_path/internal/Cone_expansion_event.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_shortest_path/internal/Cone_expansion_event.h	2020-09-08 12:15:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_shortest_path/include/CGAL/Surface_mesh_shortest_path/internal/Cone_expansion_event.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_shortest_path/include/CGAL/Surface_mesh_shortest_path/internal/Cone_expansion_event.h $
 // $Id: Cone_expansion_event.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_shortest_path/internal/Cone_tree.h 5.1-1/include/CGAL/Surface_mesh_shortest_path/internal/Cone_tree.h
--- 5.0.3-1/include/CGAL/Surface_mesh_shortest_path/internal/Cone_tree.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_shortest_path/internal/Cone_tree.h	2020-09-08 12:15:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_shortest_path/include/CGAL/Surface_mesh_shortest_path/internal/Cone_tree.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_shortest_path/include/CGAL/Surface_mesh_shortest_path/internal/Cone_tree.h $
 // $Id: Cone_tree.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_shortest_path/internal/misc_functions.h 5.1-1/include/CGAL/Surface_mesh_shortest_path/internal/misc_functions.h
--- 5.0.3-1/include/CGAL/Surface_mesh_shortest_path/internal/misc_functions.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_shortest_path/internal/misc_functions.h	2020-09-08 12:15:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_shortest_path/include/CGAL/Surface_mesh_shortest_path/internal/misc_functions.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_shortest_path/include/CGAL/Surface_mesh_shortest_path/internal/misc_functions.h $
 // $Id: misc_functions.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_shortest_path/Surface_mesh_shortest_path.h 5.1-1/include/CGAL/Surface_mesh_shortest_path/Surface_mesh_shortest_path.h
--- 5.0.3-1/include/CGAL/Surface_mesh_shortest_path/Surface_mesh_shortest_path.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_shortest_path/Surface_mesh_shortest_path.h	2020-09-08 12:15:44.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_shortest_path/include/CGAL/Surface_mesh_shortest_path/Surface_mesh_shortest_path.h $
-// $Id: Surface_mesh_shortest_path.h 5b2d46c 2020-04-21T10:03:00+02:00 Maxime Gimeno
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_shortest_path/include/CGAL/Surface_mesh_shortest_path/Surface_mesh_shortest_path.h $
+// $Id: Surface_mesh_shortest_path.h f4fec8c 2020-05-28T11:06:30+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Stephen Kiazyk
@@ -67,7 +67,7 @@ Refer to those respective papers for the
 If index property maps are not provided through the constructor of the class, internal property maps must
 be available and initialized.
 
-\sa \link PkgBGLHelper `CGAL::set_halfedgeds_items_id()`\endlink
+\sa \link BGLGraphExternalIndices `CGAL::set_halfedgeds_items_id()`\endlink
 */
 
 template<class Traits,
@@ -257,12 +257,12 @@ public:
       return temp;
     }
 
-    bool operator==(const Source_point_iterator& other)
+    bool operator==(const Source_point_iterator& other) const
     {
       return m_iterator == other.m_iterator;
     }
 
-    bool operator!=(const Source_point_iterator& other)
+    bool operator!=(const Source_point_iterator& other) const
     {
       return m_iterator != other.m_iterator;
     }
@@ -1992,7 +1992,8 @@ private:
 
       for (boost::tie(current,end) = vertices(m_graph); current != end; ++current)
       {
-        std::cout << "Vertex#" << numVertices << ": p = " << get(m_vertexPointMap,*current)
+        std::cout << "Vertex#" << numVertices
+                  << ": p = " << get(m_vertexPointMap,*current)
                   << " , Saddle Vertex: " << (is_saddle_vertex(*current) ? "yes" : "no")
                   << " , Boundary Vertex: " << (is_boundary_vertex(*current) ? "yes" : "no") << std::endl;
         ++numVertices;
@@ -2105,7 +2106,8 @@ private:
             {
               std::cout << "PseudoSource Expansion: Parent = " << parent
                         << " , Vertex = " << get(m_vertexIndexMap, event->m_parent->target_vertex())
-                        << " , Distance = " << event->m_distanceEstimate << " , Level = " << event->m_parent->level() + 1 << std::endl;
+                        << " , Distance = " << event->m_distanceEstimate
+                        << " , Level = " << event->m_parent->level() + 1 << std::endl;
             }
 
             expand_pseudo_source(parent);
@@ -2116,7 +2118,8 @@ private:
               std::cout << "Left Expansion: Parent = " << parent
                         << " Edge = (" << get(m_vertexIndexMap, source(event->m_parent->left_child_edge(), m_graph))
                         << "," << get(m_vertexIndexMap, target(event->m_parent->left_child_edge(), m_graph))
-                        << ") , Distance = " << event->m_distanceEstimate << " , Level = " << event->m_parent->level() + 1 << std::endl;
+                        << ") , Distance = " << event->m_distanceEstimate
+                        << " , Level = " << event->m_parent->level() + 1 << std::endl;
             }
 
             expand_left_child(parent, event->m_windowSegment);
@@ -2127,7 +2130,8 @@ private:
               std::cout << "Right Expansion: Parent = " << parent
                         << " , Edge = (" << get(m_vertexIndexMap, source(event->m_parent->right_child_edge(), m_graph))
                         << "," << get(m_vertexIndexMap, target(event->m_parent->right_child_edge(), m_graph))
-                        << ") , Distance = " << event->m_distanceEstimate << " , Level = " << event->m_parent->level() + 1 << std::endl;
+                        << ") , Distance = " << event->m_distanceEstimate
+                        << " , Level = " << event->m_parent->level() + 1 << std::endl;
             }
 
             expand_right_child(parent, event->m_windowSegment);
@@ -2195,7 +2199,7 @@ public:
 
   Internal property maps must be available and initialized.
 
-  \sa \link PkgBGLHelper `CGAL::set_halfedgeds_items_id()`\endlink
+  \sa \link BGLGraphExternalIndices `CGAL::set_halfedgeds_items_id()`\endlink
   */
   Surface_mesh_shortest_path(const Triangle_mesh& tm,
                              const Traits& traits = Traits())
@@ -2227,7 +2231,7 @@ public:
 
   \param traits Optional instance of the traits class to use.
   */
-  Surface_mesh_shortest_path(Triangle_mesh& tm,
+  Surface_mesh_shortest_path(const Triangle_mesh& tm,
                              Vertex_index_map vertexIndexMap,
                              Halfedge_index_map halfedgeIndexMap,
                              Face_index_map faceIndexMap,
@@ -2833,8 +2837,7 @@ public:
     that accept a reference to an `AABB_tree` as input.
 
   \details The following static overload is also available:
-    - `static Face_location locate(const Point_3& p, const Triangle_mesh& tm,
-                                   Vertex_point_map vertexPointMap, const Traits& traits = Traits())`
+    - `static Face_location locate(const %Point_3& p, const Triangle_mesh& tm, Vertex_point_map vertexPointMap, const Traits& traits = Traits())`
 
   \tparam AABBTraits A model of `AABBTraits` used to define a \cgal `AABB_tree`.
 
@@ -2865,8 +2868,7 @@ public:
   \brief Returns the face location nearest to the given point.
 
   \details The following static overload is also available:
-    - static Face_location locate(const Point_3& p, const AABB_tree<AABBTraits>& tree, const Triangle_mesh& tm,
-                                  Vertex_point_map vertexPointMap, const Traits& traits = Traits())
+    - static Face_location locate(const %Point_3& p, const AABB_tree<AABBTraits>& tree, const Triangle_mesh& tm, Vertex_point_map vertexPointMap, const Traits& traits = Traits())
 
   \tparam AABBTraits A model of `AABBTraits` used to define a \cgal `AABB_tree`.
 
@@ -2907,7 +2909,7 @@ public:
     that accept a reference to an `AABB_tree` as input.
 
   \details The following static overload is also available:
-    - `static Face_location locate(const Ray_3& ray, const Triangle_mesh& tm, Vertex_point_map vertexPointMap, const Traits& traits = Traits())`
+    - `static Face_location locate(const %Ray_3& ray, const Triangle_mesh& tm, Vertex_point_map vertexPointMap, const Traits& traits = Traits())`
 
   \tparam AABBTraits A model of `AABBTraits` used to define an `AABB_tree`.
 
@@ -2939,8 +2941,7 @@ public:
     its source point.
 
   \details The following static overload is also available:
-    - static Face_location locate(const Ray_3& ray, const AABB_tree<AABBTraits>& tree, const Triangle_mesh& tm,
-                                  Vertex_point_map vertexPointMap, const Traits& traits = Traits())
+    - static Face_location locate(const %Ray_3& ray, const AABB_tree<AABBTraits>& tree, const Triangle_mesh& tm, Vertex_point_map vertexPointMap, const Traits& traits = Traits())
 
   \tparam AABBTraits A model of `AABBTraits` used to define a \cgal `AABB_tree`.
 
@@ -3013,9 +3014,7 @@ public:
 
   /// \endcond
 
-  /// @}
-
-  /*
+  /*!
   \brief Creates an `AABB_tree` suitable for use with `locate`.
 
   \details The following static overload is also available:
@@ -3031,6 +3030,8 @@ public:
     build_aabb_tree(m_graph, outTree, m_vertexPointMap);
   }
 
+  /// \cond
+
   template <class AABBTraits>
   void build_aabb_tree(AABB_tree<AABBTraits>& outTree,
                        Vertex_point_map vertexPointMap) const
@@ -3038,8 +3039,6 @@ public:
     build_aabb_tree(m_graph, outTree, vertexPointMap);
   }
 
-  /// \cond
-
   template <class AABBTraits>
   static void build_aabb_tree(const Triangle_mesh& tm,
                               AABB_tree<AABBTraits>& outTree,
@@ -3052,6 +3051,7 @@ public:
   }
   /// \endcond
 
+  /// @}
 };
 
 } // namespace CGAL
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_shortest_path/Surface_mesh_shortest_path_traits.h 5.1-1/include/CGAL/Surface_mesh_shortest_path/Surface_mesh_shortest_path_traits.h
--- 5.0.3-1/include/CGAL/Surface_mesh_shortest_path/Surface_mesh_shortest_path_traits.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_shortest_path/Surface_mesh_shortest_path_traits.h	2020-09-08 12:15:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_shortest_path/include/CGAL/Surface_mesh_shortest_path/Surface_mesh_shortest_path_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_shortest_path/include/CGAL/Surface_mesh_shortest_path/Surface_mesh_shortest_path_traits.h $
 // $Id: Surface_mesh_shortest_path_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_shortest_path.h 5.1-1/include/CGAL/Surface_mesh_shortest_path.h
--- 5.0.3-1/include/CGAL/Surface_mesh_shortest_path.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_shortest_path.h	2020-09-08 12:15:44.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_shortest_path/include/CGAL/Surface_mesh_shortest_path.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_shortest_path/include/CGAL/Surface_mesh_shortest_path.h $
 // $Id: Surface_mesh_shortest_path.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_simplification/Detail/Common.h 5.1-1/include/CGAL/Surface_mesh_simplification/Detail/Common.h
--- 5.0.3-1/include/CGAL/Surface_mesh_simplification/Detail/Common.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_simplification/Detail/Common.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,176 +0,0 @@
-// Copyright (c) 2006  GeometryFactory (France). All rights reserved.
-//
-// This file is part of CGAL (www.cgal.org).
-//
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Detail/Common.h $
-// $Id: Common.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
-// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
-//
-// Author(s)     : Fernando Cacciola <fernando.cacciola@geometryfactory.com>
-//
-#ifndef CGAL_SURFACE_MESH_SIMPLIFICATION_DETAIL_COMMON_H
-#define CGAL_SURFACE_MESH_SIMPLIFICATION_DETAIL_COMMON_H 1
-
-#include <CGAL/license/Surface_mesh_simplification.h>
-
-
-#include <functional>
-#include <utility>
-#include <vector>
-#include <vector>
-#include <set>
-
-#include <boost/config.hpp>
-#include <boost/shared_ptr.hpp>
-#include <boost/scoped_ptr.hpp>
-#include <boost/iterator_adaptors.hpp>
-#include <boost/optional/optional.hpp>
-#include <boost/none.hpp>
-#include <boost/tuple/tuple.hpp>
-#include <boost/format.hpp>
-#include <boost/scoped_array.hpp>
-#include <boost/graph/graph_traits.hpp>
-#include <boost/graph/properties.hpp>
-#include <boost/graph/adjacency_list.hpp>
-
-#include <CGAL/algorithm.h>
-#include <CGAL/Cartesian/MatrixC33.h>
-#include <CGAL/Modifiable_priority_queue.h>
-#include <CGAL/boost/graph/properties.h>
-#include <CGAL/boost/graph/iterator.h>
-
-namespace CGAL {
-
-namespace Surface_mesh_simplification
-{
-
-using boost::num_edges ;
-using boost::num_vertices ;
-using boost::edges ;
-using boost::out_edges ;
-using boost::in_edges ;
-using boost::source ;
-using boost::target ;
-
-using boost::shared_ptr ;
-using boost::optional ;
-using boost::none ;
-using boost::put_get_helper ;
-using boost::get ;
-using boost::put ;
-using boost::addressof ;
-
-using namespace boost::tuples ;
-
-template<class Handle>
-inline bool handle_assigned( Handle h ) { Handle null ; return h != null ; }
-
-template<class Iterator, class Handle>
-bool handle_exists ( Iterator begin, Iterator end, Handle h )
-{
- if ( handle_assigned(h) )
- {
-   while ( begin != end )
-     if ( begin++ == h )
-       return true ;
- }
- return false ;
-}
-
-template <class TM>
-struct No_constrained_edge_map{
-  typedef typename boost::graph_traits<TM>::edge_descriptor key_type;
-  typedef bool value_type;
-  typedef value_type reference;
-  typedef boost::readable_property_map_tag category;
-  friend bool get(No_constrained_edge_map, key_type) {
-    return false;
-  }
-};
-
-} // namespace Surface_mesh_simplification
-
-template<class N>
-inline std::string n_to_string( N const& n )
-{
-  return boost::str( boost::format("%|5.19g|") % n ) ;
-}
-
-template<class XYZ>
-inline std::string xyz_to_string( XYZ const& xyz )
-{
-  return boost::str( boost::format("(%|5.19g|,%|5.19g|,%|5.19g|)") % xyz.x() % xyz.y() % xyz.z() ) ;
-}
-
-template<class Matrix>
-inline std::string matrix_to_string( Matrix const& m )
-{
-  return boost::str( boost::format("[%1%|%2%|%3%]") % xyz_to_string(m.r0()) % xyz_to_string(m.r1()) % xyz_to_string(m.r2()) ) ;
-}
-
-template<class T>
-inline std::string optional_to_string( boost::optional<T> const& o )
-{
-  if ( o )
-       return boost::str( boost::format("%1%") % *o ) ;
-  else return std::string("NONE");
-}
-
-
-} //namespace CGAL
-
-#if   defined(CGAL_SURFACE_SIMPLIFICATION_ENABLE_TRACE)    \
-   || defined(CGAL_SURFACE_SIMPLIFICATION_ENABLE_LT_TRACE)
-#define CGAL_SMS_ENABLE_TRACE
-#endif
-
-#ifdef CGAL_SMS_ENABLE_TRACE
-
-#  include<string>
-#  include<iostream>
-#  include<sstream>
-namespace internal { namespace  { bool cgal_enable_sms_trace = true ; } }
-#  define CGAL_SMS_TRACE_IMPL(m) \
-     if ( ::internal::cgal_enable_sms_trace ) { \
-       std::ostringstream ss ; ss << m ; std::string s = ss.str(); \
-       /*Surface_simplification_external_trace(s)*/ std::cerr << s << std::endl; \
-     }
-
-#  define CGAL_SMS_DEBUG_CODE(code) code
-
-#else
-
-#  define CGAL_SMS_DEBUG_CODE(code)
-
-#endif
-
-#ifdef CGAL_SURFACE_SIMPLIFICATION_ENABLE_LT_TRACE
-#  define CGAL_SMS_LT_TRACE(l,m) if ( (l) <= CGAL_SURFACE_SIMPLIFICATION_ENABLE_LT_TRACE ) CGAL_SMS_TRACE_IMPL(m)
-#else
-#  define CGAL_SMS_LT_TRACE(l,m)
-#endif
-
-#ifdef CGAL_SURFACE_SIMPLIFICATION_ENABLE_TRACE
-#  define CGAL_SMS_TRACE_IF(c,l,m) if ( (c) && ( (l) <= CGAL_SURFACE_SIMPLIFICATION_ENABLE_TRACE) ) CGAL_SMS_TRACE_IMPL(m)
-#  define CGAL_SMS_TRACE(l,m)      if ( (l) <= CGAL_SURFACE_SIMPLIFICATION_ENABLE_TRACE ) CGAL_SMS_TRACE_IMPL(m)
-#else
-#  define CGAL_SMS_TRACE_IF(c,l,m)
-#  define CGAL_SMS_TRACE(l,m)
-#endif
-
-#undef CGAL_SMS_ENABLE_TRACE
-
-#ifdef CGAL_TESTING_SURFACE_MESH_SIMPLIFICATION
-#  define CGAL_SURF_SIMPL_TEST_assertion(EX)         CGAL_assertion(EX)
-#  define CGAL_SURF_SIMPL_TEST_assertion_msg(EX,MSG) CGAL_assertion_msg(EX,MSG)
-#  define CGAL_SURF_SIMPL_TEST_assertion_code(CODE)  CGAL_assertion_code(CODE)
-#else
-#  define CGAL_SURF_SIMPL_TEST_assertion(EX)
-#  define CGAL_SURF_SIMPL_TEST_assertion_msg(EX,MSG)
-#  define CGAL_SURF_SIMPL_TEST_assertion_code(CODE)
-#endif
-
-
-#endif // CGAL_SURFACE_MESH_SIMPLIFICATION_DETAIL_COMMON_H //
-// EOF //
-
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_simplification/Detail/Edge_collapse.h 5.1-1/include/CGAL/Surface_mesh_simplification/Detail/Edge_collapse.h
--- 5.0.3-1/include/CGAL/Surface_mesh_simplification/Detail/Edge_collapse.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_simplification/Detail/Edge_collapse.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,401 +0,0 @@
-// Copyright (c) 2006  GeometryFactory (France). All rights reserved.
-//
-// This file is part of CGAL (www.cgal.org).
-//
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Detail/Edge_collapse.h $
-// $Id: Edge_collapse.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
-// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
-//
-// Author(s)     : Fernando Cacciola <fernando.cacciola@geometryfactory.com>
-//
-#ifndef CGAL_SURFACE_MESH_SIMPLIFICATION_DETAIL_EDGE_COLLAPSE_H
-#define CGAL_SURFACE_MESH_SIMPLIFICATION_DETAIL_EDGE_COLLAPSE_H 1
-
-#include <CGAL/license/Surface_mesh_simplification.h>
-
-
-#include <CGAL/Surface_mesh_simplification/Detail/Common.h>
-#include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_profile.h>
-#include <CGAL/boost/graph/Euler_operations.h>
-#include <CGAL/boost/graph/helpers.h>
-namespace CGAL {
-
-namespace Surface_mesh_simplification
-{
-
-//
-// Implementation of the vertex-pair collapse triangulated surface mesh simplification algorithm
-//
-template<class TM_
-        ,class ShouldStop_
-        ,class VertexIndexMap_
-        ,class VertexPointMap_
-        ,class EdgeIndexMap_
-        ,class EdgeIsConstrainedMap_
-        ,class GetCost_
-        ,class GetPlacement_
-        ,class VisitorT_
-        >
-class EdgeCollapse
-{
-public:
-
-  typedef TM_              TM ;
-  typedef ShouldStop_       ShouldStop ;
-  typedef VertexIndexMap_   VertexIndexMap ;
-  typedef VertexPointMap_   VertexPointMap ;
-  typedef EdgeIndexMap_     EdgeIndexMap ;
-  typedef EdgeIsConstrainedMap_ EdgeIsConstrainedMap;
-  typedef GetCost_          GetCost ;
-  typedef GetPlacement_     GetPlacement ;
-  typedef VisitorT_         VisitorT ;
-
-  typedef EdgeCollapse Self ;
-
-  typedef Edge_profile<TM,VertexPointMap> Profile ;
-
-  typedef boost::graph_traits  <TM>       GraphTraits ;
-  typedef boost::graph_traits  <TM const> ConstGraphTraits ;
-
-  typedef typename GraphTraits::vertex_descriptor      vertex_descriptor ;
-  typedef typename GraphTraits::vertex_iterator        vertex_iterator ;
-  typedef typename GraphTraits::halfedge_descriptor    halfedge_descriptor ;
-  typedef typename GraphTraits::halfedge_iterator      halfedge_iterator ;
-  typedef CGAL::Halfedge_around_source_iterator<TM> out_edge_iterator ;
-  typedef CGAL::Halfedge_around_target_iterator<TM> in_edge_iterator ;
-  typedef typename GraphTraits::traversal_category     traversal_category ;
-  typedef typename GraphTraits::edges_size_type        size_type ;
-
-  typedef typename GraphTraits::edge_iterator edge_iterator ;
-  typedef VertexPointMap Vertex_point_pmap;
-  typedef typename boost::property_traits<Vertex_point_pmap>::value_type Point;
-  typedef typename Kernel_traits<Point>::Kernel Traits;
-
-  typedef typename Traits::Equal_3 Equal_3 ;
-
-  typedef typename Traits::Vector_3 Vector ;
-  typedef typename Traits::FT       FT ;
-
-  typedef optional<FT> Cost_type ;
-  typedef optional<Point> Placement_type ;
-
-  struct Compare_id
-  {
-    Compare_id() : mAlgorithm(0) {}
-
-    Compare_id( Self const* aAlgorithm ) : mAlgorithm(aAlgorithm) {}
-
-    bool operator() ( halfedge_descriptor const& a, halfedge_descriptor const& b ) const
-    {
-      return mAlgorithm->get_halfedge_id(a) < mAlgorithm->get_halfedge_id(b);
-    }
-
-    Self const* mAlgorithm ;
-  } ;
-
-  struct Compare_cost
-  {
-    Compare_cost() : mAlgorithm(0) {}
-
-    Compare_cost( Self const* aAlgorithm ) : mAlgorithm(aAlgorithm) {}
-
-    bool operator() ( halfedge_descriptor const& a, halfedge_descriptor const& b ) const
-    {
-      // NOTE: A cost is an optional<> value.
-      // Absent optionals are ordered first; that is, "none < T" and "T > none" for any defined T != none.
-      // In consequence, edges with undefined costs will be promoted to the top of the priority queue and poped out first.
-      return mAlgorithm->get_data(a).cost() < mAlgorithm->get_data(b).cost();
-    }
-
-    Self const* mAlgorithm ;
-  } ;
-
-  struct edge_id : boost::put_get_helper<size_type, edge_id>
-  {
-    typedef boost::readable_property_map_tag category;
-    typedef size_type                        value_type;
-    typedef size_type                        reference;
-    typedef halfedge_descriptor                  key_type;
-
-    edge_id() : mAlgorithm(0) {}
-
-    edge_id( Self const* aAlgorithm ) : mAlgorithm(aAlgorithm) {}
-
-    size_type operator[] ( halfedge_descriptor const& e ) const { return mAlgorithm->get_edge_id(e); }
-
-    Self const* mAlgorithm ;
-  } ;
-
-
-  typedef Modifiable_priority_queue<halfedge_descriptor,Compare_cost,edge_id> PQ ;
-  typedef typename PQ::handle pq_handle ;
-
-  // An Edge_data is associated with EVERY _ edge in the mesh (collapsable or not).
-  // It relates the edge with the PQ-handle needed to update the priority queue
-  // It also relates the edge with a policy-based cache
-  class Edge_data
-  {
-  public :
-
-    Edge_data() : mPQHandle() {}
-
-    Cost_type const& cost() const { return mCost ; }
-    Cost_type      & cost()       { return mCost ; }
-
-    pq_handle PQ_handle() const { return mPQHandle ;}
-
-    bool is_in_PQ() const { return mPQHandle != PQ::null_handle() ; }
-
-    void set_PQ_handle( pq_handle h ) { mPQHandle = h ; }
-
-    void reset_PQ_handle() { mPQHandle = PQ::null_handle() ; }
-
-  private:
-
-    Cost_type mCost ;
-    pq_handle mPQHandle ;
-  } ;
-  typedef Edge_data* Edge_data_ptr ;
-  typedef boost::scoped_array<Edge_data> Edge_data_array ;
-
-
-public:
-
-  EdgeCollapse( TM&                        aSurface
-              , ShouldStop           const& aShouldStop
-              , VertexIndexMap       const& aVertex_index_map
-              , VertexPointMap       const& aVertex_point_map
-              , EdgeIndexMap         const& aEdge_index_map
-              , EdgeIsConstrainedMap const& aEdge_is_constrained_map
-              , GetCost              const& aGetCost
-              , GetPlacement         const& aGetPlacement
-              , VisitorT                    aVisitor
-              ) ;
-
-  int run() ;
-
-private:
-
-  void Collect();
-  void Loop();
-  bool Is_collapse_topologically_valid( Profile const& aProfile ) ;
-  bool Is_tetrahedron( halfedge_descriptor const& h1 ) ;
-  bool Is_open_triangle( halfedge_descriptor const& h1 ) ;
-  bool Is_collapse_geometrically_valid( Profile const& aProfile, Placement_type aPlacement ) ;
-  void Collapse( Profile const& aProfile, Placement_type aPlacement ) ;
-  void Update_neighbors( vertex_descriptor const& aKeptV ) ;
-
-  Profile create_profile ( halfedge_descriptor const& aEdge )
-  {
-    return Profile(aEdge,mSurface,Vertex_index_map,Vertex_point_map,Edge_index_map, m_has_border);
-  }
-
-  size_type get_halfedge_id   ( halfedge_descriptor const& aEdge ) const { return Edge_index_map[aEdge]; }
-  size_type get_edge_id ( halfedge_descriptor const& aEdge ) const { return get_halfedge_id(aEdge) / 2 ; }
-
-  bool is_primary_edge ( halfedge_descriptor const& aEdge ) const { return ( get_halfedge_id(aEdge) % 2 ) == 0 ; }
-
-  halfedge_descriptor primary_edge ( halfedge_descriptor const& aEdge )
-  {
-    return is_primary_edge(aEdge) ? aEdge : opposite(aEdge,mSurface) ;
-  }
-
-  bool is_border ( halfedge_descriptor const& aEdge ) const { return face(aEdge,mSurface) == boost::graph_traits<TM>::null_face() ; }
-
-  bool is_constrained( halfedge_descriptor const& aEdge ) const { return get(Edge_is_constrained_map,edge(aEdge,mSurface)); }
-  bool is_constrained( vertex_descriptor const& aVertex ) const;
-
-  bool is_border_or_constrained( halfedge_descriptor const& aEdge ) const { return is_border(aEdge) || is_constrained(aEdge); }
-
-  bool is_edge_a_border ( halfedge_descriptor const& aEdge ) const
-  {
-    return is_border(aEdge) || is_border(opposite(aEdge,mSurface)) ;
-  }
-
-  bool is_border ( vertex_descriptor const& aV ) const ;
-
-  bool is_border_or_constrained ( vertex_descriptor const& aV ) const ;
-
-  bool are_shared_triangles_valid( Point const& p0, Point const& p1, Point const& p2, Point const& p3 ) const ;
-
-  halfedge_descriptor find_connection ( vertex_descriptor const& v0, vertex_descriptor const& v1 ) const ;
-
-  vertex_descriptor find_exterior_link_triangle_3rd_vertex ( halfedge_descriptor const& e, vertex_descriptor const& v0, vertex_descriptor const& v1 ) const ;
-
-  Edge_data& get_data ( halfedge_descriptor const& aEdge ) const
-  {
-    CGAL_assertion( is_primary_edge(aEdge) ) ;
-    return mEdgeDataArray[get_edge_id(aEdge)];
-  }
-
-  typename boost::property_traits<VertexPointMap>::reference
-  get_point ( vertex_descriptor const& aV ) const
-  {
-    return get(Vertex_point_map,aV);
-  }
-
-  boost::tuple<vertex_descriptor,vertex_descriptor> get_vertices( halfedge_descriptor const& aEdge ) const
-  {
-    vertex_descriptor p,q ;
-    p = source(aEdge,mSurface);
-    q = target(aEdge,mSurface);
-    return boost::make_tuple(p,q);
-  }
-
-  boost::tuple<vertex_descriptor,vertex_descriptor> get_vertices( halfedge_descriptor const& aEdge )
-  {
-    vertex_descriptor p,q ;
-    p = source(aEdge,mSurface);
-    q = target(aEdge,mSurface);
-    return boost::make_tuple(p,q);
-  }
-
-  std::string vertex_to_string( vertex_descriptor const& v ) const
-  {
-    Point const& p = get_point(v);
-    return boost::str( boost::format("[V%1%:%2%]") % get(Vertex_index_map,v) % xyz_to_string(p) ) ;
-  }
-
-  std::string edge_to_string ( halfedge_descriptor const& aEdge ) const
-  {
-    vertex_descriptor p,q ; boost::tie(p,q) = get_vertices(aEdge);
-    return boost::str( boost::format("{E%1% %2%->%3%}%4%") % get(Edge_index_map,aEdge) % vertex_to_string(p) % vertex_to_string(q) % ( is_border(aEdge) ? " (BORDER)" : ( is_border(opposite(aEdge,mSurface)) ? " (~BORDER)": "" ) ) ) ;
-  }
-
-  Cost_type get_cost ( Profile const& aProfile ) const
-  {
-    return Get_cost(aProfile, get_placement(aProfile) );
-  }
-
-  Placement_type get_placement( Profile const& aProfile ) const
-  {
-    return Get_placement(aProfile);
-  }
-
-  void insert_in_PQ( halfedge_descriptor const& aEdge, Edge_data& aData )
-  {
-    CGAL_SURF_SIMPL_TEST_assertion(is_primary_edge(aEdge)) ;
-    CGAL_SURF_SIMPL_TEST_assertion(!aData.is_in_PQ());
-    CGAL_SURF_SIMPL_TEST_assertion(!mPQ->contains(aEdge) ) ;
-
-    aData.set_PQ_handle(mPQ->push(aEdge));
-
-    CGAL_SURF_SIMPL_TEST_assertion(aData.is_in_PQ());
-    CGAL_SURF_SIMPL_TEST_assertion(mPQ->contains(aEdge) ) ;
-  }
-
-  void update_in_PQ( halfedge_descriptor const& aEdge, Edge_data& aData )
-  {
-    CGAL_SURF_SIMPL_TEST_assertion(is_primary_edge(aEdge)) ;
-    CGAL_SURF_SIMPL_TEST_assertion(aData.is_in_PQ());
-    CGAL_SURF_SIMPL_TEST_assertion(mPQ->contains(aEdge) ) ;
-
-    aData.set_PQ_handle(mPQ->update(aEdge,aData.PQ_handle())) ;
-
-    CGAL_SURF_SIMPL_TEST_assertion(aData.is_in_PQ());
-    CGAL_SURF_SIMPL_TEST_assertion(mPQ->contains(aEdge) ) ;
-  }
-
-  void remove_from_PQ( halfedge_descriptor const& aEdge, Edge_data& aData )
-  {
-    CGAL_SURF_SIMPL_TEST_assertion(is_primary_edge(aEdge)) ;
-    CGAL_SURF_SIMPL_TEST_assertion(aData.is_in_PQ());
-    CGAL_SURF_SIMPL_TEST_assertion(mPQ->contains(aEdge) ) ;
-
-    aData.set_PQ_handle(mPQ->erase(aEdge,aData.PQ_handle()));
-
-    CGAL_SURF_SIMPL_TEST_assertion(!aData.is_in_PQ());
-    CGAL_SURF_SIMPL_TEST_assertion(!mPQ->contains(aEdge) ) ;
-  }
-
-  optional<halfedge_descriptor> pop_from_PQ()
-  {
-    optional<halfedge_descriptor> rEdge = mPQ->extract_top();
-    if ( rEdge )
-    {
-      CGAL_SURF_SIMPL_TEST_assertion(is_primary_edge(*rEdge) ) ;
-      CGAL_SURF_SIMPL_TEST_assertion(get_data(*rEdge).is_in_PQ()) ;
-
-      get_data(*rEdge).reset_PQ_handle();
-
-      CGAL_SURF_SIMPL_TEST_assertion(!get_data(*rEdge).is_in_PQ() ) ;
-      CGAL_SURF_SIMPL_TEST_assertion(!mPQ->contains(*rEdge)) ;
-    }
-    return rEdge ;
-  }
-
-  /// Functions to ensure the backward compatibility before addition of the constrained edge map
-  template<class AEdgeIsConstrainedMap>
-  vertex_descriptor
-  halfedge_collapse_bk_compatibility(
-    halfedge_descriptor const& pq, AEdgeIsConstrainedMap aEdge_is_constrained_map)
-  {
-    return CGAL::Euler::collapse_edge(edge(pq,mSurface), mSurface, aEdge_is_constrained_map);
-  }
-
-
-  template<class TM>
-  vertex_descriptor
-  halfedge_collapse_bk_compatibility(
-    halfedge_descriptor const& pq, No_constrained_edge_map<TM> )
-  {
-    vertex_descriptor vd = CGAL::Euler::collapse_edge(edge(pq,mSurface), mSurface);
-    return vd;
-  }
-
-
-  /// We wrap this test to avoid penalizing runtime when no constraints are present
-  template<class AEdgeIsConstrainedMap>
-  bool
-  is_edge_adjacent_to_a_constrained_edge(
-    Profile const& aProfile, AEdgeIsConstrainedMap)
-  {
-    return is_constrained(aProfile.v0()) && is_constrained(aProfile.v1());
-  }
-
-  template<class TM>
-  bool
-  is_edge_adjacent_to_a_constrained_edge(
-    halfedge_descriptor const&, No_constrained_edge_map<TM> )
-  {
-    return false;
-  }
-  ///
-
-private:
-
-  TM&                   mSurface ;
-
-  ShouldStop           const& Should_stop ;
-  VertexIndexMap       const& Vertex_index_map ;
-  VertexPointMap       const& Vertex_point_map ;
-  EdgeIndexMap         const& Edge_index_map ;
-  EdgeIsConstrainedMap const& Edge_is_constrained_map;
-  GetCost              const& Get_cost ;
-  GetPlacement         const& Get_placement ;
-  VisitorT                    Visitor ;
-  bool                        m_has_border ;
-
-private:
-
-  Edge_data_array mEdgeDataArray ;
-
-  boost::scoped_ptr<PQ> mPQ ;
-
-  size_type mInitialEdgeCount ;
-  size_type mCurrentEdgeCount ;
-
-  FT          mcMaxDihedralAngleCos2 ;
-
-  CGAL_SMS_DEBUG_CODE ( unsigned mStep ; )
-} ;
-
-} // namespace Surface_mesh_simplification
-
-} //namespace CGAL
-
-#include <CGAL/Surface_mesh_simplification/Detail/Edge_collapse_impl.h>
-
-#endif // CGAL_SURFACE_MESH_SIMPLIFICATION_I_EDGE_COLLAPSE_H //
-// EOF //
-
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_simplification/Detail/Edge_collapse_impl.h 5.1-1/include/CGAL/Surface_mesh_simplification/Detail/Edge_collapse_impl.h
--- 5.0.3-1/include/CGAL/Surface_mesh_simplification/Detail/Edge_collapse_impl.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_simplification/Detail/Edge_collapse_impl.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,930 +0,0 @@
-// Copyright (c) 2006  GeometryFactory (France). All rights reserved.
-//
-// This file is part of CGAL (www.cgal.org).
-//
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Detail/Edge_collapse_impl.h $
-// $Id: Edge_collapse_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
-// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
-//
-// Author(s)     : Fernando Cacciola <fernando.cacciola@geometryfactory.com>
-//
-#ifndef CGAL_SURFACE_MESH_SIMPLIFICATION_DETAIL_EDGE_COLLAPSE_IMPL_H
-#define CGAL_SURFACE_MESH_SIMPLIFICATION_DETAIL_EDGE_COLLAPSE_IMPL_H
-
-#include <CGAL/license/Surface_mesh_simplification.h>
-
-
-namespace CGAL {
-
-namespace Surface_mesh_simplification
-{
-
-  template<class M, class SP, class VIM, class VPM,class EIM, class ECTM, class CF, class PF, class V>
-  EdgeCollapse<M,SP,VIM,VPM,EIM,ECTM,CF,PF,V>::EdgeCollapse( TM&                         aSurface
-                                                    , ShouldStop            const& aShould_stop
-                                                    , VertexIndexMap        const& aVertex_index_map
-                                                    , VertexPointMap        const& aVertex_point_map
-                                                    , EdgeIndexMap          const& aEdge_index_map
-                                                    , EdgeIsConstrainedMap  const& aEdge_is_constrained_map
-                                                    , GetCost               const& aGet_cost
-                                                    , GetPlacement          const& aGet_placement
-                                                    , VisitorT                     aVisitor
-                                                    )
-  :
-   mSurface           (aSurface)
-  ,Should_stop        (aShould_stop)
-  ,Vertex_index_map   (aVertex_index_map)
-  ,Vertex_point_map   (aVertex_point_map)
-  ,Edge_index_map     (aEdge_index_map)
-  ,Edge_is_constrained_map (aEdge_is_constrained_map)
-  ,Get_cost           (aGet_cost)
-  ,Get_placement      (aGet_placement)
-  ,Visitor            (aVisitor)
-  ,m_has_border       (false)
-{
-  const FT cMaxDihedralAngleCos = std::cos( 1.0 * CGAL_PI / 180.0 ) ;
-
-  mcMaxDihedralAngleCos2 = cMaxDihedralAngleCos * cMaxDihedralAngleCos ;
-
-  halfedge_iterator eb, ee ;
-  for ( boost::tie(eb,ee) = halfedges(mSurface); eb!=ee; ++eb )
-    {
-      halfedge_descriptor ed = *eb;
-      if(is_border(ed)){
-        m_has_border = true;
-        break;
-      }
-    }
-
-  CGAL_SMS_TRACE(0,"EdgeCollapse of TM with " << (num_edges(aSurface)/2) << " edges" );
-
-  CGAL_SMS_DEBUG_CODE ( mStep = 0 ; )
-
-#ifdef CGAL_SURFACE_SIMPLIFICATION_ENABLE_TRACE
-    vertex_iterator vb, ve ;
-    for ( boost::tie(vb,ve) = vertices(mSurface) ; vb != ve ; ++ vb )
-      CGAL_SMS_TRACE(1, vertex_to_string(*vb) ) ;
-
-    for ( boost::tie(eb,ee) = halfedges(mSurface); eb!=ee; ++eb )
-      CGAL_SMS_TRACE(1, edge_to_string(*eb) ) ;
-#endif
-}
-
-  template<class M,class SP, class VIM, class VPM,class EIM,class ECTM, class CF,class PF,class V>
-  int EdgeCollapse<M,SP,VIM,VPM,EIM,ECTM,CF,PF,V>::run()
-{
-  CGAL_SURF_SIMPL_TEST_assertion( mSurface.is_valid() && mSurface.is_pure_triangle() ) ;
-
-  Visitor.OnStarted(mSurface);
-
-  // First collect all candidate edges in a PQ
-  Collect();
-
-  // Then proceed to collapse each edge in turn
-  Loop();
-
-  CGAL_SMS_TRACE(0,"Finished: " << (mInitialEdgeCount - mCurrentEdgeCount) << " edges removed." ) ;
-
-  int r = (int)(mInitialEdgeCount - mCurrentEdgeCount) ;
-
-  Visitor.OnFinished(mSurface);
-
-  return r ;
-}
-
-  template<class M,class SP, class VIM, class VPM,class EIM,class ECTM, class CF,class PF,class V>
-  void EdgeCollapse<M,SP,VIM,VPM,EIM,ECTM,CF,PF,V>::Collect()
-{
-  CGAL_SMS_TRACE(0,"Collecting edges...");
-
-  //
-  // Loop over all the _undirected_ edges in the surface putting them in the PQ
-  //
-
-  Equal_3 equal_points = Traits().equal_3_object();
-
-  size_type lSize = num_edges(mSurface);
-
-  mInitialEdgeCount = mCurrentEdgeCount = static_cast<size_type>(
-                      std::distance( boost::begin(edges(mSurface)),
-                                     boost::end(edges(mSurface)) ) );;
-
-  mEdgeDataArray.reset( new Edge_data[lSize] ) ;
-
-  mPQ.reset( new PQ (lSize, Compare_cost(this), edge_id(this) ) ) ;
-
-  std::size_t id = 0 ;
-
-  CGAL_SURF_SIMPL_TEST_assertion_code ( size_type lInserted    = 0 ) ;
-  CGAL_SURF_SIMPL_TEST_assertion_code ( size_type lNotInserted = 0 ) ;
-
-  std::set<halfedge_descriptor> zero_length_edges;
-
-  edge_iterator eb, ee ;
-  for ( boost::tie(eb,ee) = edges(mSurface); eb!=ee; ++eb, id+=2 )
-  {
-    halfedge_descriptor lEdge = halfedge(*eb,mSurface) ;
-
-    if ( is_constrained(lEdge) ) continue;//no not insert constrainted edges
-
-    // AF CGAL_assertion( get_halfedge_id(lEdge) == id ) ;
-    // AF CGAL_assertion( get_halfedge_id(opposite(lEdge,mSurface)) == id+1 ) ;
-
-    Profile const& lProfile = create_profile(lEdge);
-    if ( !equal_points(lProfile.p0(),lProfile.p1()) )
-    {
-      Edge_data& lData = get_data(lEdge);
-
-      lData.cost() = get_cost(lProfile) ;
-      insert_in_PQ(lEdge,lData);
-
-      Visitor.OnCollected(lProfile,lData.cost());
-
-      CGAL_SURF_SIMPL_TEST_assertion_code ( ++ lInserted ) ;
-    }
-    else
-    {
-      zero_length_edges.insert(primary_edge(lEdge));
-      CGAL_SURF_SIMPL_TEST_assertion_code ( ++ lNotInserted ) ;
-    }
-
-
-    CGAL_SMS_TRACE(2,edge_to_string(lEdge));
-  }
-
-  CGAL_SURF_SIMPL_TEST_assertion ( lInserted + lNotInserted == mInitialEdgeCount ) ;
-
-  for (typename std::set<halfedge_descriptor>::iterator it=zero_length_edges.begin(),
-        it_end=zero_length_edges.end();it!=it_end;++it)
-  {
-    Profile const& lProfile = create_profile(*it);
-
-    if (!Is_collapse_topologically_valid(lProfile) ) continue;
-
-    // edges of length 0 removed no longer need to be treated
-    if ( lProfile.left_face_exists() )
-    {
-      halfedge_descriptor lEdge_to_remove = is_constrained(lProfile.vL_v0()) ?
-                                          primary_edge(lProfile.v1_vL()) :
-                                          primary_edge(lProfile.vL_v0()) ;
-      zero_length_edges.erase( lEdge_to_remove );
-      Edge_data& lData = get_data(lEdge_to_remove) ;
-      if ( lData.is_in_PQ() ){
-        CGAL_SMS_TRACE(2,"Removing E" << get(Edge_index_map,lEdge_to_remove) << " from PQ" );
-        remove_from_PQ(lEdge_to_remove,lData);
-      }
-      --mCurrentEdgeCount;
-    }
-
-    if ( lProfile.right_face_exists() )
-    {
-      halfedge_descriptor lEdge_to_remove = is_constrained(lProfile.vR_v1()) ?
-                                          primary_edge(lProfile.v0_vR()) :
-                                          primary_edge(lProfile.vR_v1()) ;
-      zero_length_edges.erase( lEdge_to_remove );
-      Edge_data& lData = get_data(lEdge_to_remove) ;
-      if ( lData.is_in_PQ() ){
-        CGAL_SMS_TRACE(2,"Removing E" << get(Edge_index_map,lEdge_to_remove) << " from PQ" );
-        remove_from_PQ(lEdge_to_remove,lData);
-      }
-      --mCurrentEdgeCount;
-    }
-
-    --mCurrentEdgeCount;
-
-    //the placement is trivial, it's always the point itself
-    Placement_type lPlacement = lProfile.p0();
-    vertex_descriptor rResult
-      = halfedge_collapse_bk_compatibility(lProfile.v0_v1(), Edge_is_constrained_map);
-    put(Vertex_point_map,rResult,*lPlacement);
-    Visitor.OnCollapsed(lProfile,rResult);
-  }
-
-  CGAL_SMS_TRACE(0,"Initial edge count: " << mInitialEdgeCount ) ;
-}
-
-  template<class M,class SP, class VIM, class VPM,class EIM,class ECTM, class CF,class PF,class V>
-  void EdgeCollapse<M,SP,VIM,VPM,EIM,ECTM,CF,PF,V>::Loop()
-{
-  CGAL_SMS_TRACE(0,"Collapsing edges...") ;
-
-  CGAL_SURF_SIMPL_TEST_assertion_code ( size_type lLoop_watchdog = 0 ) ;
-  CGAL_SURF_SIMPL_TEST_assertion_code ( size_type lNonCollapsableCount = 0 ) ;
-
-  //
-  // Pops and processes each edge from the PQ
-  //
-  optional<halfedge_descriptor> lEdge ;
-  #ifdef CGAL_SURF_SIMPL_INTERMEDIATE_STEPS_PRINTING
-  int i_rm=0;
-  #endif
-  while ( (lEdge = pop_from_PQ()) )
-  {
-    CGAL_SURF_SIMPL_TEST_assertion ( lLoop_watchdog ++ < mInitialEdgeCount ) ;
-
-    CGAL_SMS_TRACE(1,"Popped " << edge_to_string(*lEdge) ) ;
-
-    CGAL_assertion( !is_constrained(*lEdge) );
-
-    Profile const& lProfile = create_profile(*lEdge);
-
-    Cost_type lCost = get_data(*lEdge).cost();
-
-    Visitor.OnSelected(lProfile,lCost,mInitialEdgeCount,mCurrentEdgeCount);
-
-    if ( lCost )
-    {
-      if ( Should_stop(*lCost,lProfile,mInitialEdgeCount,mCurrentEdgeCount) )
-      {
-        Visitor.OnStopConditionReached(lProfile);
-
-        CGAL_SMS_TRACE(0,"Stop condition reached with InitialEdgeCount=" << mInitialEdgeCount
-                       << " CurrentEdgeCount=" << mCurrentEdgeCount
-                       << " Current Edge: " << edge_to_string(*lEdge)
-                       );
-        break ;
-      }
-
-      if ( Is_collapse_topologically_valid(lProfile) )
-      {
-        // The external function Get_new_vertex_point() is allowed to return an absent point if there is no way to place the vertex
-        // satisfying its constraints. In that case the remaining vertex is simply left unmoved.
-        Placement_type lPlacement = get_placement(lProfile);
-
-        if ( lPlacement!=boost::none && Is_collapse_geometrically_valid(lProfile,lPlacement) )
-        {
-          #ifdef CGAL_SURF_SIMPL_INTERMEDIATE_STEPS_PRINTING
-          std::cout << "step " << i_rm << " " << source(*lEdge,mSurface)->point() << " " << target(*lEdge,mSurface)->point() << "\n";
-          #endif
-          Collapse(lProfile,lPlacement);
-          #ifdef CGAL_SURF_SIMPL_INTERMEDIATE_STEPS_PRINTING
-          std::stringstream sstr;
-          sstr << "debug/P-";
-          if (i_rm<10) sstr << "0";
-          if (i_rm<100) sstr << "0";
-          sstr <<  i_rm  << ".off";
-          std::ofstream out(sstr.str().c_str());
-          out << mSurface;
-          ++i_rm;
-          #endif
-        }
-      }
-      else
-      {
-        CGAL_SURF_SIMPL_TEST_assertion_code ( lNonCollapsableCount++ ) ;
-
-        Visitor.OnNonCollapsable(lProfile);
-
-        CGAL_SMS_TRACE(1,edge_to_string(*lEdge) << " NOT Collapsable"  );
-      }
-    }
-    else
-    {
-      CGAL_SMS_TRACE(1,edge_to_string(*lEdge) << " uncomputable cost."  );
-    }
-  }
-}
-
-template<class M, class SP, class VIM, class VPM,class EIM,class ECTM, class CF,class PF,class V>
-bool EdgeCollapse<M,SP,VIM,VPM,EIM,ECTM,CF,PF,V>::is_border( vertex_descriptor const& aV ) const
-{
-  bool rR = false ;
-
-  in_edge_iterator eb, ee ;
-  for ( boost::tie(eb,ee) = halfedges_around_target(aV,mSurface) ; eb != ee ; ++ eb )
-  {
-    halfedge_descriptor lEdge = *eb ;
-    if ( is_edge_a_border(lEdge) )
-    {
-      rR = true ;
-      break ;
-    }
-  }
-
-  return rR ;
-}
-
-template<class M, class SP, class VIM, class VPM,class EIM,class ECTM, class CF,class PF,class V>
-bool EdgeCollapse<M,SP,VIM,VPM,EIM,ECTM,CF,PF,V>::is_border_or_constrained( vertex_descriptor const& aV ) const
-{
-  in_edge_iterator eb, ee ;
-  for ( boost::tie(eb,ee) = halfedges_around_target(aV,mSurface) ; eb != ee ; ++ eb )
-  {
-   halfedge_descriptor lEdge = *eb ;
-    if ( is_edge_a_border(lEdge) || is_constrained(lEdge) )
-      return true;
-  }
-  return false;
-}
-
-template<class M, class SP, class VIM, class VPM,class EIM,class ECTM, class CF,class PF,class V>
-bool EdgeCollapse<M,SP,VIM,VPM,EIM,ECTM,CF,PF,V>::is_constrained( vertex_descriptor const& aV ) const
-{
-  in_edge_iterator eb, ee ;
-  for ( boost::tie(eb,ee) = halfedges_around_target(aV,mSurface) ; eb != ee ; ++ eb )
-    if ( is_constrained(*eb) ) return true;
-  return false;
-}
-
-// Some edges are NOT collapsable: doing so would break the topological consistency of the mesh.
-// This function returns true if a edge 'p->q' can be collapsed.
-//
-// An edge p->q can be collapsed iff it satisfies the "link condition"
-// (as described in the "Mesh Optimization" article of Hoppe et al (1993))
-//
-// The link condition is as follows: for every vertex 'k' adjacent to both 'p and 'q',
-// "p,k,q" is a facet of the mesh.
-//
-template<class M, class SP, class VIM, class VPM,class EIM,class ECTM, class CF,class PF,class V>
-bool EdgeCollapse<M,SP,VIM,VPM,EIM,ECTM,CF,PF,V>::Is_collapse_topologically_valid( Profile const& aProfile )
-{
-  bool rR = true ;
-
-  CGAL_SMS_TRACE(3,"Testing topological collapsabilty of p_q=V" << get(Vertex_index_map,aProfile.v0()) << "(%" << degree(aProfile.v0(),mSurface) << ")"
-                  << "->V" << get(Vertex_index_map,aProfile.v1()) << "(%" << degree(aProfile.v1(),mSurface) << ")"
-                 );
-
-  CGAL_SMS_TRACE(4, "is p_q border:" << aProfile.is_v0_v1_a_border() );
-  CGAL_SMS_TRACE(4, "is q_q border:" << aProfile.is_v1_v0_a_border() );
-
-  out_edge_iterator eb1, ee1 ;
-  out_edge_iterator eb2, ee2 ;
-
-  CGAL_SMS_TRACE(4,"  t=V"
-                   << ( aProfile.left_face_exists() ? get(Vertex_index_map,aProfile.vL()) : -1 )
-                   << "(%"
-                  << ( aProfile.left_face_exists() ? degree(aProfile.vL(),mSurface) : 0 )
-                   << ")"
-                 );
-  CGAL_SMS_TRACE(4,"  b=V"
-                   << ( aProfile.right_face_exists() ? get(Vertex_index_map,aProfile.vR()) : -1 )
-                   << "(%"
-                  << ( aProfile.right_face_exists() ? degree(aProfile.vR(),mSurface) :0 )
-                   << ")"
-                   );
-
-  // Simple tests handling the case of non-manifold situations at a vertex or edge (pinching)
-  // (even if we advertise one should not use a surface mesh with such features)
-  if ( aProfile.left_face_exists () )
-  {
-    if ( CGAL::is_border( opposite(aProfile.v1_vL(), mSurface), mSurface ) &&
-         CGAL::is_border( opposite(aProfile.vL_v0(), mSurface), mSurface )
-        ) return false;
-
-    if ( aProfile.right_face_exists () &&
-         CGAL::is_border( opposite(aProfile.vR_v1(), mSurface), mSurface ) &&
-         CGAL::is_border( opposite(aProfile.v0_vR(), mSurface), mSurface )
-        ) return false;
-  }
-  else{
-    if ( aProfile.right_face_exists () )
-    {
-      if ( CGAL::is_border( opposite(aProfile.vR_v1(), mSurface), mSurface ) &&
-           CGAL::is_border( opposite(aProfile.v0_vR(), mSurface), mSurface )
-          ) return false;
-    }
-    else
-      return false;
-  }
-
-  // The following loop checks the link condition for v0_v1.
-  // Specifically, that for every vertex 'k' adjacent to both 'p and 'q', 'pkq' is a face of the mesh.
-  //
-  for ( boost::tie(eb1,ee1) = halfedges_around_source(aProfile.v0(),mSurface) ; rR && eb1 != ee1 ; ++ eb1 )
-  {
-    halfedge_descriptor v0_k = *eb1 ;
-
-    if ( v0_k != aProfile.v0_v1() )
-    {
-      vertex_descriptor k = target(v0_k,mSurface);
-
-      for ( boost::tie(eb2,ee2) =  halfedges_around_source(k,mSurface) ; rR && eb2 != ee2 ; ++ eb2 )
-      {
-        halfedge_descriptor k_v1 = *eb2 ;
-
-        if ( target(k_v1,mSurface) == aProfile.v1() )
-        {
-          // At this point we know p-q-k are connected and we need to determine if this triangle is a face of the mesh.
-          //
-          // Since the mesh is known to be triangular there are at most two faces sharing the edge p-q.
-          //
-          // If p->q is NOT a border edge, the top face is p->q->t where t is target(next(p->q))
-          // If q->p is NOT a border edge, the bottom face is q->p->b where b is target(next(q->p))
-          //
-          // If k is either t or b then p-q-k *might* be a face of the mesh. It won't be if k==t but p->q is border
-          // or k==b but q->b is a border (because in that case even though there exists triangles p->q->t (or q->p->b)
-          // they are holes, not faces)
-          //
-
-          bool lIsFace =   ( aProfile.vL() == k && aProfile.left_face_exists () )
-                        || ( aProfile.vR() == k && aProfile.right_face_exists() ) ;
-
-          CGAL_SURF_SIMPL_TEST_assertion_code
-          (
-            if ( lIsFace )
-            {
-              // Is k_v1 the halfedge bounding the face 'k-v1-v0'?
-              if ( ! is_border(k_v1) && target(next(k_v1,mSurface),mSurface) == aProfile.v0() )
-              {
-                CGAL_SURF_SIMPL_TEST_assertion( ! is_border(k_v1) ) ;
-                CGAL_SURF_SIMPL_TEST_assertion(  target(k_v1,mSurface) == aProfile.v1() ) ;
-                CGAL_SURF_SIMPL_TEST_assertion(  target(next(k_v1,mSurface),mSurface) == aProfile.v0() ) ;
-                CGAL_SURF_SIMPL_TEST_assertion(  target(next(next(k_v1,mSurface),mSurface),mSurface) == k ) ;
-              }
-              else // or is it the opposite?
-              {
-                halfedge_descriptor v1_k = opposite(k_v1,mSurface);
-                CGAL_SURF_SIMPL_TEST_assertion( ! is_border(v1_k) ) ;
-                CGAL_SURF_SIMPL_TEST_assertion(  target(v1_k,mSurface) == k ) ;
-                CGAL_SURF_SIMPL_TEST_assertion(  target(next(v1_k, mSurface),mSurface) == aProfile.v0() ) ;
-                CGAL_SURF_SIMPL_TEST_assertion(  target(next(next(v1_k, mSurface),mSurface),mSurface) == aProfile.v1() ) ;
-              }
-            }
-          );
-
-          if ( !lIsFace )
-          {
-            CGAL_SMS_TRACE(3,"  k=V" << get(Vertex_index_map,k) << " IS NOT in a face with p-q. NON-COLLAPSABLE edge." ) ;
-            rR = false ;
-            break ;
-          }
-          else
-          {
-            CGAL_SMS_TRACE(4,"  k=V" << get(Vertex_index_map,k) << " is in a face with p-q") ;
-          }
-        }
-      }
-    }
-  }
-
-  if ( rR )
-  {
-    /// ensure two constrained edges cannot get merged
-    if ( is_edge_adjacent_to_a_constrained_edge(
-          aProfile, Edge_is_constrained_map) ) return false ;
-
-    if ( aProfile.is_v0_v1_a_border() )
-    {
-      if ( Is_open_triangle(aProfile.v0_v1()) )
-      {
-        rR = false ;
-        CGAL_SMS_TRACE(3,"  p-q belongs to an open triangle. NON-COLLAPSABLE edge." ) ;
-      }
-    }
-    else if ( aProfile.is_v1_v0_a_border() )
-    {
-      if ( Is_open_triangle(aProfile.v1_v0()) )
-      {
-        rR = false ;
-        CGAL_SMS_TRACE(3,"  p-q belongs to an open triangle. NON-COLLAPSABLE edge." ) ;
-      }
-    }
-    else
-    {
-      if ( is_border(aProfile.v0()) && is_border(aProfile.v1()) )
-      {
-        rR = false ;
-        CGAL_SMS_TRACE(3,"  both p and q are boundary vertices but p-q is not. NON-COLLAPSABLE edge." ) ;
-      }
-      else
-      {
-        bool lTetra = Is_tetrahedron(aProfile.v0_v1());
-
-        //CGAL_SURF_SIMPL_TEST_assertion( lTetra == mSurface.is_tetrahedron(aProfile.v0_v1()) ) ;
-
-        if ( lTetra )
-        {
-          rR = false ;
-          CGAL_SMS_TRACE(3,"  p-q belongs to a tetrahedron. NON-COLLAPSABLE edge." ) ;
-        }
-
-        if ( next(aProfile.v0_v1(), mSurface) == opposite(prev(aProfile.v1_v0(), mSurface), mSurface) &&
-             prev(aProfile.v0_v1(), mSurface) == opposite(next(aProfile.v1_v0(), mSurface), mSurface) )
-        {
-          CGAL_SMS_TRACE(3,"  degenerate volume." ) ;
-          return false ;
-        }
-      }
-    }
-  }
-
-  return rR ;
-}
-
-template<class M, class SP, class VIM, class VPM,class EIM,class ECTM, class CF,class PF,class V>
-bool EdgeCollapse<M,SP,VIM,VPM,EIM,ECTM,CF,PF,V>::Is_tetrahedron( halfedge_descriptor const& h1 )
-{
-  return is_tetrahedron(h1,mSurface);
-}
-
-template<class M, class SP, class VIM, class VPM,class EIM,class ECTM, class CF,class PF,class V>
-bool EdgeCollapse<M,SP,VIM,VPM,EIM,ECTM,CF,PF,V>::Is_open_triangle( halfedge_descriptor const& h1 )
-{
-  bool rR = false ;
-
-  halfedge_descriptor h2 = next(h1,mSurface);
-  halfedge_descriptor h3 = next(h2,mSurface);
-
-  // First check if it is a triangle
-  if ( next(h3,mSurface) == h1 )
-  {
-    // Now check if it is open
-    CGAL_SMS_TRACE(4,"  p-q is a border edge... checking E" << get(Edge_index_map,h2) << " and E" << get(Edge_index_map,h3) ) ;
-
-    rR = is_border(h2) && is_border(h3);
-
-    CGAL_SURF_SIMPL_TEST_assertion( rR == (  is_border(h1)
-                                             && is_border(next(h1,mSurface))
-                                             && is_border(next(next(h1,mSurface),mSurface))
-                                          )
-                                  ) ;
-  }
-
-  return rR ;
-}
-
-// Given triangles 'p0,p1,p2' and 'p0,p2,p3', both shared along edge 'v0-v2',
-// determine if they are geometrically valid: that is, the ratio of their
-// respective areas is no greater than a max value and the internal
-// dihedral angle formed by their supporting planes is no greater than
-// a given threshold
-template<class M, class SP, class VIM, class VPM,class EIM,class ECTM, class CF,class PF,class V>
-bool EdgeCollapse<M,SP,VIM,VPM,EIM,ECTM,CF,PF,V>::are_shared_triangles_valid( Point const& p0, Point const& p1, Point const& p2, Point const& p3 ) const
-{
-  bool rR = false ;
-
-  Vector e01 = Traits().construct_vector_3_object()(p0,p1) ;
-  Vector e02 = Traits().construct_vector_3_object()(p0,p2) ;
-  Vector e03 = Traits().construct_vector_3_object()(p0,p3) ;
-
-  Vector n012 = Traits().construct_cross_product_vector_3_object()(e01,e02);
-  Vector n023 = Traits().construct_cross_product_vector_3_object()(e02,e03);
-
-  FT l012 = Traits().compute_scalar_product_3_object()(n012, n012) ;
-  FT l023 = Traits().compute_scalar_product_3_object()(n023, n023) ;
-
-  FT larger  = (std::max)(l012,l023);
-  FT smaller = (std::min)(l012,l023);
-
-  const double cMaxAreaRatio = 1e8 ;
-
-  CGAL_SMS_TRACE(4,"    Testing validity of shared triangles:"
-                  << "\n      p0=" << xyz_to_string(p0) << "\n      p1=" << xyz_to_string(p1) << "\n      p2=" << xyz_to_string(p2) << "\n      p3=" << xyz_to_string(p3)
-                  << "\n      e01=" << xyz_to_string(e01) << "\n      e02=" << xyz_to_string(e02) << "\n      e03=" << xyz_to_string(e03)
-                  << "\n      n012=" << xyz_to_string(n012) << "\n      n023=" << xyz_to_string(n023)
-                  << "\n      l012=" << n_to_string(l012) << "\n      l023=" << n_to_string(l023)
-                 );
-
-  if ( larger < cMaxAreaRatio * smaller )
-  {
-    FT l0123 = Traits().compute_scalar_product_3_object()(n012, n023) ;
-    CGAL_SMS_TRACE(4,"\n      l0123=" << n_to_string(l0123) );
-
-    if ( CGAL_NTS is_positive(l0123) )
-    {
-      rR = true ;
-    }
-    else
-    {
-      CGAL_SMS_TRACE(4,"\n      lhs: " << n_to_string(( l0123 * l0123 ) / ( l012 * l023 )) << " <= rhs: " << mcMaxDihedralAngleCos2 ) ;
-
-      if ( ( l0123 * l0123 ) <= mcMaxDihedralAngleCos2 * ( l012 * l023 ) )
-      {
-        rR = true ;
-      }
-    }
-  }
-
-  return rR ;
-}
-
-
-// Returns the directed halfedge connecting v0 to v1, if exists.
-template<class M, class SP, class VIM, class VPM,class EIM,class ECTM, class CF,class PF,class V>
-typename EdgeCollapse<M,SP,VIM,VPM,EIM,ECTM,CF,PF,V>::halfedge_descriptor
-EdgeCollapse<M,SP,VIM,VPM,EIM,ECTM,CF,PF,V>::find_connection ( vertex_descriptor const& v0, vertex_descriptor const& v1 ) const
-{
-  out_edge_iterator eb, ee ;
-  for ( boost::tie(eb,ee) = halfedges_around_source(v0,mSurface) ; eb != ee ; ++ eb )
-  {
-    halfedge_descriptor out = *eb ;
-    if ( target(out,mSurface) == v1 )
-      return out ;
-  }
-
-  return halfedge_descriptor() ;
-}
-
-// Given the edge 'e' around the link for the collapsinge edge "v0-v1", finds the vertex that makes a triangle adjacent to 'e' but exterior to the link (i.e not containing v0 nor v1)
-// If 'e' is a null handle OR 'e' is a border edge, there is no such triangle and a null handle is returned.
-template<class M, class SP, class VIM, class VPM,class EIM,class ECTM, class CF,class PF,class V>
-typename EdgeCollapse<M,SP,VIM,VPM,EIM,ECTM,CF,PF,V>::vertex_descriptor
-EdgeCollapse<M,SP,VIM,VPM,EIM,ECTM,CF,PF,V>::find_exterior_link_triangle_3rd_vertex ( halfedge_descriptor const& e, vertex_descriptor const& v0, vertex_descriptor const& v1 ) const
-{
-  vertex_descriptor r ;
-
-  if ( handle_assigned(e) )
-  {
-    vertex_descriptor ra = target(next(e, mSurface), mSurface);
-    vertex_descriptor rb = source(prev(e, mSurface), mSurface);
-
-    if ( ra == rb && ra != v0 && ra != v1 )
-    {
-      r = ra ;
-    }
-    else
-    {
-      ra = target(next(opposite(e,mSurface), mSurface), mSurface);
-      rb = source(prev(opposite(e,mSurface), mSurface), mSurface);
-
-      if ( ra == rb && ra != v0 && ra != v1 )
-      {
-        r = ra ;
-      }
-    }
-  }
-
-  return r ;
-}
-
-
-// A collapse is geometrically valid if, in the resulting local mesh no two adjacent triangles form an internal dihedral angle
-// greater than a fixed threshold (i.e. triangles do not "fold" into each other)
-//
-template<class M, class SP, class VIM, class VPM,class EIM,class ECTM, class CF,class PF,class V>
-bool EdgeCollapse<M,SP,VIM,VPM,EIM,ECTM,CF,PF,V>::Is_collapse_geometrically_valid( Profile const& aProfile, Placement_type k0)
-{
-  bool rR = true ;
-
-
-
-  CGAL_SMS_TRACE(3,"Testing geometrical collapsabilty of v0-v1=E" << get(Edge_index_map,aProfile.v0_v1()) );
-  if ( k0 )
-  {
-    // Use the current link to extract all local triangles incident to 'vx' in the collapsed mesh (which at this point doesn't exist yet)
-    //
-    typedef typename Profile::vertex_descriptor_vector::const_iterator link_iterator ;
-    link_iterator linkb = aProfile.link().begin();
-    link_iterator linke = aProfile.link().end  ();
-    link_iterator linkl = std::prev(linke) ;
-
-    for ( link_iterator l = linkb ; l != linke && rR ; ++ l )
-    {
-      link_iterator pv = ( l == linkb ? linkl : std::prev (l) );
-      link_iterator nx = ( l == linkl ? linkb : std::next  (l) ) ;
-
-      // k0,k1 and k3 are three consecutive vertices along the link.
-      vertex_descriptor k1 = *pv ;
-      vertex_descriptor k2 = * l ;
-      vertex_descriptor k3 = *nx ;
-
-      CGAL_SMS_TRACE(4,"  Screening link vertices k1=V" << get(Vertex_index_map,k1) << " k2=V" << get(Vertex_index_map,k2) << " k3=V" << get(Vertex_index_map,k3) ) ;
-
-      halfedge_descriptor e12 = find_connection(k1,k2);
-      halfedge_descriptor e23 = k3 != k1 ? find_connection(k2,k3) : halfedge_descriptor() ;
-
-      // If 'k1-k2-k3' are connected there will be two adjacent triangles 'k0,k1,k2' and 'k0,k2,k3' after the collapse.
-      if ( handle_assigned(e12) && handle_assigned(e23) )
-      {
-        CGAL_SMS_TRACE(4,"    Link triangles shared" ) ;
-
-        if ( !are_shared_triangles_valid( *k0, get_point(k1), get_point(k2), get_point(k3) ) )
-        {
-          CGAL_SMS_TRACE(3,"    Triangles VX-V" << get(Vertex_index_map,k1) << "-V" << get(Vertex_index_map,k2) << " and VX-V" << get(Vertex_index_map,k3) << " are not geometrically valid. Collapse rejected");
-          rR = false ;
-        }
-      }
-
-      if ( rR )
-      {
-        // Also check the triangles 'k0,k1,k2' and it's adjacent along e12: 'k4,k2,k1', if exist
-        vertex_descriptor k4 = find_exterior_link_triangle_3rd_vertex(e12,aProfile.v0(),aProfile.v1());
-
-        // There is indeed a triangle shared along e12
-        if ( handle_assigned(k4) )
-        {
-          CGAL_SMS_TRACE(4,"    Found exterior link triangle shared along E" << get(Edge_index_map,e12) << " with third vertex: V" << get(Vertex_index_map,k4) ) ;
-
-          if ( !are_shared_triangles_valid( get_point(k1), get_point(k4), get_point(k2), *k0 ) )
-          {
-            CGAL_SMS_TRACE(3,"    Triangles V" << get(Vertex_index_map,k1) << "-V" << get(Vertex_index_map,k4) << " and V" << get(Vertex_index_map,k2) << "-VX are not geometrically valid. Collapse rejected");
-            rR = false ;
-          }
-        }
-      }
-
-      if ( rR )
-      {
-        // And finally, check the triangles 'k0,k2,k3' and it's adjacent e23: 'k5,k3,k2' if exist
-        vertex_descriptor k5 = find_exterior_link_triangle_3rd_vertex(e23,aProfile.v0(),aProfile.v1());
-
-        // There is indeed a triangle shared along e12
-        if ( handle_assigned(k5) )
-        {
-          CGAL_SMS_TRACE(4,"    Found exterior link triangle shared along E" << get(Edge_index_map,e23) << " with third vertex: V" << get(Vertex_index_map,k5) ) ;
-
-          if ( !are_shared_triangles_valid( get_point(k2), get_point(k5), get_point(k3), *k0 ) )
-          {
-            CGAL_SMS_TRACE(3,"    Triangles V" << get(Vertex_index_map,k2) << "-V" << get(Vertex_index_map,k5) << " and V" << get(Vertex_index_map,k3) << "-VX are not geometrically valid. Collapse rejected");
-            rR = false ;
-          }
-        }
-      }
-    }
-
-  }
-
-  return rR ;
-}
-
-template<class M, class SP, class VIM, class VPM,class EIM,class ECTM, class CF,class PF,class V>
-void EdgeCollapse<M,SP,VIM,VPM,EIM,ECTM,CF,PF,V>::Collapse( Profile const& aProfile, Placement_type aPlacement )
-{
-
-  CGAL_SMS_TRACE(1,"S" << mStep << ". Collapsing " << edge_to_string(aProfile.v0_v1()) ) ;
-
-  vertex_descriptor rResult ;
-
-  Visitor.OnCollapsing(aProfile,aPlacement);
-
-  -- mCurrentEdgeCount ;
-
-  CGAL_SURF_SIMPL_TEST_assertion_code( size_type lResultingVertexCount = mSurface.size_of_vertices() ;
-                                       size_type lResultingEdgeCount   = mSurface.size_of_halfedges() / 2 ;
-                                     ) ;
-
-  // If the top/bottom facets exists, they are removed and the edges v0vt and Q-B along with them.
-  // In that case their corresponding pairs must be pop off the queue
-
-  if ( aProfile.left_face_exists() )
-  {
-    halfedge_descriptor lV0VL = primary_edge(aProfile.vL_v0());
-    if ( is_constrained(lV0VL) ) //make sure a constrained edge will not disappear
-      lV0VL=primary_edge(aProfile.v1_vL());
-
-    CGAL_SMS_TRACE(3,"V0VL E" << get(Edge_index_map,lV0VL)
-                    << "(V" <<  get(Vertex_index_map, source(lV0VL,mSurface)) << "->V" << get(Vertex_index_map,target(lV0VL,mSurface)) << ")"
-                   ) ;
-
-    Edge_data& lData = get_data(lV0VL) ;
-    if ( lData.is_in_PQ() )
-    {
-      CGAL_SMS_TRACE(2,"Removing E" << get(Edge_index_map,lV0VL) << " from PQ" ) ;
-      remove_from_PQ(lV0VL,lData) ;
-    }
-
-    -- mCurrentEdgeCount ;
-    CGAL_SURF_SIMPL_TEST_assertion_code( -- lResultingEdgeCount ) ;
-  }
-
-  if ( aProfile.right_face_exists() )
-  {
-    halfedge_descriptor lVRV1 = primary_edge(aProfile.vR_v1());
-    if ( is_constrained(lVRV1) ) //make sure a constrained edge will not disappear
-      lVRV1=primary_edge(aProfile.v0_vR());
-
-    CGAL_SMS_TRACE(3,"V1VRE" << get(Edge_index_map,lVRV1)
-                    << "(V" <<  get(Vertex_index_map, source(lVRV1,mSurface)) << "->V" << get(Vertex_index_map, target(lVRV1,mSurface)) << ")"
-                   ) ;
-
-    Edge_data& lData = get_data(lVRV1) ;
-    if ( lData.is_in_PQ() )
-    {
-      CGAL_SMS_TRACE(2,"Removing E" << get(Edge_index_map,lVRV1) << " from PQ") ;
-      remove_from_PQ(lVRV1,lData) ;
-    }
-    -- mCurrentEdgeCount ;
-    CGAL_SURF_SIMPL_TEST_assertion_code( -- lResultingEdgeCount ) ;
-  }
-
-  CGAL_SMS_TRACE(1,"Removing:\n  v0v1: E" << get(Edge_index_map,aProfile.v0_v1()) << "(V" << get(Vertex_index_map,aProfile.v0()) << "->V" << get(Vertex_index_map,aProfile.v1()) << ")" );
-
-
-  // Perform the actuall collapse.
-  // This is an external function.
-  // It's REQUIRED to remove ONLY 1 vertex (P or Q) and edges PQ,PT and QB
-  // (PT and QB are removed if they are not null).
-  // All other edges must be kept.
-  // All directed edges incident to vertex removed are relink to the vertex kept.
-  rResult = halfedge_collapse_bk_compatibility(aProfile.v0_v1(), Edge_is_constrained_map);
-
-  CGAL_SURF_SIMPL_TEST_assertion_code( -- lResultingEdgeCount ) ;
-
-  CGAL_SURF_SIMPL_TEST_assertion_code( -- lResultingVertexCount ) ;
-
-  CGAL_SURF_SIMPL_TEST_assertion( lResultingEdgeCount * 2 == mSurface.size_of_halfedges() ) ;
-
-  CGAL_SURF_SIMPL_TEST_assertion( lResultingVertexCount == mSurface.size_of_vertices() ) ;
-
-  CGAL_SURF_SIMPL_TEST_assertion( mSurface.is_valid() && mSurface.is_pure_triangle() ) ;
-
-  CGAL_SMS_TRACE(1,"V" << get(Vertex_index_map,rResult) << " kept." ) ;
-
-#ifdef CGAL_SURFACE_SIMPLIFICATION_ENABLE_TRACE
-  out_edge_iterator eb1, ee1 ;
-  for ( boost::tie(eb1,ee1) = halfedges_around_source(rResult,mSurface) ; eb1 != ee1 ; ++ eb1 )
-    CGAL_SMS_TRACE(2, edge_to_string(*eb1) ) ;
-#endif
-
-  if ( aPlacement )
-  {
-    CGAL_SMS_TRACE(1,"New vertex point: " << xyz_to_string(*aPlacement) ) ;
-    put(Vertex_point_map,rResult,*aPlacement) ;
-  }
-
-  Visitor.OnCollapsed(aProfile,rResult);
-
-  Update_neighbors(rResult) ;
-
-  CGAL_SMS_DEBUG_CODE ( ++mStep ; )
-}
-
-template<class M, class SP, class VIM, class VPM,class EIM,class ECTM, class CF,class PF,class V>
-void EdgeCollapse<M,SP,VIM,VPM,EIM,ECTM,CF,PF,V>::Update_neighbors( vertex_descriptor const& aKeptV )
-{
-  CGAL_SMS_TRACE(3,"Updating cost of neighboring edges..." ) ;
-
-  //
-  // (A) Collect all edges to update their cost: all those around each vertex adjacent to the vertex kept
-  //
-
-  typedef std::set<halfedge_descriptor,Compare_id> edges ;
-
-  edges lToUpdate(Compare_id(this)) ;
-  edges lToInsert(Compare_id(this)) ;
-
-  // (A.1) Loop around all vertices adjacent to the vertex kept
-  in_edge_iterator eb1, ee1 ;
-  for ( boost::tie(eb1,ee1) = halfedges_around_target(aKeptV,mSurface) ; eb1 != ee1 ; ++ eb1 )
-  {
-    halfedge_descriptor lEdge1 = *eb1 ;
-
-    vertex_descriptor lAdj_k = source(lEdge1,mSurface);
-
-    // (A.2) Loop around all edges incident on each adjacent vertex
-    in_edge_iterator eb2, ee2 ;
-    for ( boost::tie(eb2,ee2) = halfedges_around_target(lAdj_k,mSurface) ; eb2 != ee2 ; ++ eb2 )
-    {
-      halfedge_descriptor lEdge2 = primary_edge(*eb2) ;
-
-      Edge_data& lData2 = get_data(lEdge2);
-      CGAL_SMS_TRACE(4,"Inedge around V" << get(Vertex_index_map,lAdj_k) << edge_to_string(lEdge2) ) ;
-
-      // Only edges still in the PQ needs to be updated, the other needs to be re-inserted
-      if ( lData2.is_in_PQ() )
-        lToUpdate.insert(lEdge2) ;
-      else
-        lToInsert.insert(lEdge2) ;
-    }
-  }
-
-  //
-  // (B) Proceed to update the costs.
-  //
-
-  for ( typename edges::iterator it = lToUpdate.begin(), eit = lToUpdate.end() ; it != eit ; ++ it )
-  {
-    halfedge_descriptor lEdge = *it;
-
-    Edge_data& lData = get_data(lEdge);
-
-    Profile const& lProfile = create_profile(lEdge);
-
-    lData.cost() = get_cost(lProfile) ;
-
-    CGAL_SMS_TRACE(3, edge_to_string(lEdge) << " updated in the PQ") ;
-
-    update_in_PQ(lEdge,lData);
-  }
-
-  //
-  // (C) Insert ignored edges
-  //
-  // I think that this should be done for edges eliminated because of the geometric criteria
-  // and not the topological one.However maintaining such a set might be more expensive
-  // and hard to be safe ...
-  for ( typename edges::iterator it = lToInsert.begin(),
-                                 eit = lToInsert.end() ; it != eit ; ++ it )
-  {
-    halfedge_descriptor lEdge = *it;
-    if ( is_constrained(lEdge) ) continue; //do not insert constrained edges
-    Edge_data& lData = get_data(lEdge);
-
-    Profile const& lProfile = create_profile(lEdge);
-
-    lData.cost() = get_cost(lProfile) ;
-
-    CGAL_SMS_TRACE(3, edge_to_string(lEdge) << " re-inserted in the PQ") ;
-
-    insert_in_PQ(lEdge,lData);
-  }
-}
-
-
-} // namespace Surface_mesh_simplification
-
-} //namespace CGAL
-
-#endif // CGAL_SURFACE_MESH_SIMPLIFICATION_DETAIL_EDGE_COLLAPSE_IMPL_H //
-// EOF //
-
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_simplification/edge_collapse.h 5.1-1/include/CGAL/Surface_mesh_simplification/edge_collapse.h
--- 5.0.3-1/include/CGAL/Surface_mesh_simplification/edge_collapse.h	2020-08-01 19:02:56.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_simplification/edge_collapse.h	2020-09-08 12:15:45.000000000 +0000
@@ -2,161 +2,112 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/edge_collapse.h $
-// $Id: edge_collapse.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/edge_collapse.h $
+// $Id: edge_collapse.h 618a72b 2020-03-17T20:00:31+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Fernando Cacciola <fernando.cacciola@geometryfactory.com>
 //
 #ifndef CGAL_SURFACE_MESH_SIMPLIFICATION_EDGE_COLLAPSE_H
-#define CGAL_SURFACE_MESH_SIMPLIFICATION_EDGE_COLLAPSE_H 1
+#define CGAL_SURFACE_MESH_SIMPLIFICATION_EDGE_COLLAPSE_H
 
 #include <CGAL/license/Surface_mesh_simplification.h>
 
-
 #include <CGAL/boost/graph/properties.h>
 #include <CGAL/boost/graph/Named_function_parameters.h>
+#include <CGAL/boost/graph/named_params_helper.h>
 
-#include <CGAL/Surface_mesh_simplification/Detail/Edge_collapse.h>
-#include <CGAL/Surface_mesh_simplification/Detail/Common.h>
+#include <CGAL/Surface_mesh_simplification/internal/Common.h>
+#include <CGAL/Surface_mesh_simplification/internal/Edge_collapse.h>
 #include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/LindstromTurk.h>
 
 namespace CGAL {
+namespace Surface_mesh_simplification {
+namespace internal {
 
-namespace Surface_mesh_simplification
-{
-
-template<class TM
-        ,class ShouldStop
-        ,class VertexIndexMap
-        ,class VertexPointMap
-        ,class EdgeIndexMap
-        ,class EdgeIsConstrainedMap
-        ,class GetCost
-        ,class GetPlacement
-        ,class Visitor
-        >
-int edge_collapse ( TM&                       aSurface
-                  , ShouldStop           const& aShould_stop
+template<class TM,
+         class GT,
+         class ShouldStop,
+         class VertexIndexMap,
+         class VertexPointMap,
+         class HalfedgeIndexMap,
+         class EdgeIsConstrainedMap,
+         class GetCost,
+         class GetPlacement,
+         class Visitor>
+int edge_collapse(TM& tmesh,
+                  const ShouldStop& should_stop,
                   // optional mesh information policies
-                  , VertexIndexMap       const& aVertex_index_map     // defaults to get(vertex_index,aSurface)
-                  , VertexPointMap       const& aVertex_point_map     // defaults to get(vertex_point,aSurface)
-                  , EdgeIndexMap         const& aEdge_index_map       // defaults to get(edge_index,aSurface)
-                  , EdgeIsConstrainedMap const& aEdge_is_constrained_map   // defaults to No_constrained_edge_map<TM>()
-
+                  const GT& traits,
+                  const VertexIndexMap& vim, // defaults to get(vertex_index, tmesh)
+                  const VertexPointMap& vpm, // defaults to get(vertex_point, tmesh)
+                  const HalfedgeIndexMap& him, // defaults to get(edge_index, tmesh)
+                  const EdgeIsConstrainedMap& ecm, // defaults to No_constrained_edge_map<TM>()
                   // optional strategy policies - defaults to LindstomTurk
-                  , GetCost              const& aGet_cost
-                  , GetPlacement         const& aGet_placement
+                  const GetCost& get_cost,
+                  const GetPlacement& get_placement,
+                  Visitor visitor)
+{
+  typedef EdgeCollapse<TM, GT, ShouldStop,
+                       VertexIndexMap, VertexPointMap, HalfedgeIndexMap, EdgeIsConstrainedMap,
+                       GetCost, GetPlacement, Visitor> Algorithm;
 
-                  , Visitor                     aVisitor
-                  )
-{
-  typedef EdgeCollapse< TM
-                      , ShouldStop
-                      , VertexIndexMap
-                      , VertexPointMap
-                      , EdgeIndexMap
-                      , EdgeIsConstrainedMap
-                      , GetCost
-                      , GetPlacement
-                      , Visitor
-                      >
-                      Algorithm;
-
-  Algorithm algorithm( aSurface
-                     , aShould_stop
-                     , aVertex_index_map
-                     , aVertex_point_map
-                     , aEdge_index_map
-                     , aEdge_is_constrained_map
-                     , aGet_cost
-                     , aGet_placement
-                     , aVisitor
-                     ) ;
+  Algorithm algorithm(tmesh, traits, should_stop, vim, vpm, him, ecm, get_cost, get_placement, visitor);
 
   return algorithm.run();
 }
 
-
 struct Dummy_visitor
 {
-  template<class TM>                                 void OnStarted( TM& ) const {}
-  template<class TM>                                 void OnFinished ( TM& ) const {}
-  template<class Profile>                             void OnStopConditionReached( Profile const& ) const {}
-  template<class Profile, class OFT>                  void OnCollected( Profile const&, OFT const& ) const {}
-  template<class Profile, class OFT, class Size_type> void OnSelected( Profile const&, OFT const&, Size_type, Size_type ) const {}
-  template<class Profile, class OPoint>               void OnCollapsing(Profile const&, OPoint const& ) const {}
-  template<class Profile, class VH>                   void OnCollapsed( Profile const&, VH ) const {}
-  template<class Profile>                             void OnNonCollapsable(Profile const& ) const {}
-} ;
-
-template<class TM, class ShouldStop, class P, class T, class R>
-int edge_collapse ( TM& aSurface
-                  , ShouldStop const& aShould_stop
-                  , Named_function_parameters<P,T,R> const& aParams
-                  )
+  template<class TM>
+  void OnStarted(TM&) const {}
+  template<class TM>
+  void OnFinished(TM&) const {}
+  template<class Profile>
+  void OnStopConditionReached(const Profile&) const {}
+  template<class Profile, class OFT>
+  void OnCollected(const Profile&, const OFT&) const {}
+  template<class Profile, class OFT, class Size_type>
+  void OnSelected(const Profile&, const OFT&, Size_type, Size_type) const {}
+  template<class Profile, class OPoint>
+  void OnCollapsing(const Profile&, const OPoint&) const {}
+  template<class Profile, class VH>
+  void OnCollapsed(const Profile&, VH) const {}
+  template<class Profile>
+  void OnNonCollapsable(const Profile&) const {}
+};
+
+} // namespace internal
+
+template<class TM, class ShouldStop, class NamedParameters>
+int edge_collapse(TM& tmesh,
+                  const ShouldStop& should_stop,
+                  const NamedParameters& np)
 {
   using parameters::choose_parameter;
   using parameters::get_parameter;
 
-  LindstromTurk_params lPolicyParams ;
-
-  internal_np::graph_visitor_t vis = internal_np::graph_visitor_t() ;
-
-  return edge_collapse(aSurface
-                      ,aShould_stop
-                      ,choose_parameter(get_parameter(aParams,internal_np::vertex_index), get_const_property_map(boost::vertex_index, aSurface))
-                      ,choose_parameter(get_parameter(aParams,internal_np::vertex_point),get_property_map(vertex_point, aSurface))
-                      ,choose_parameter(get_parameter(aParams,internal_np::halfedge_index),get_const_property_map(boost::halfedge_index, aSurface))
-                      ,choose_parameter(get_parameter(aParams,internal_np::edge_is_constrained),No_constrained_edge_map<TM>())
-                      ,choose_parameter(get_parameter(aParams,internal_np::get_cost_policy), LindstromTurk_cost<TM>())
-                      ,choose_parameter(get_parameter(aParams,internal_np::get_placement_policy), LindstromTurk_placement<TM>())
-                      ,choose_parameter(get_parameter(aParams,vis), Dummy_visitor())
-                      );
-
-}
-  template<class TM, class ShouldStop, class GT, class P, class T, class R>
-int edge_collapse ( TM& aSurface
-                  , ShouldStop const& aShould_stop
-                  , Named_function_parameters<P,T,R> const& aParams
-                  )
-{
-  using parameters::choose_parameter;
-  using parameters::get_parameter;
-
-  LindstromTurk_params lPolicyParams ;
-
-  internal_np::graph_visitor_t vis = internal_np::graph_visitor_t() ;
-
-  return edge_collapse(aSurface
-                      ,aShould_stop
-                      ,choose_parameter(get_parameter(aParams,internal_np::vertex_index), get_const_property_map(boost::vertex_index, aSurface))
-                      ,choose_parameter(get_parameter(aParams,internal_np::vertex_point),get_property_map(vertex_point, aSurface))
-                      ,choose_parameter(get_parameter(aParams,internal_np::halfedge_index),get_const_property_map(boost::halfedge_index, aSurface))
-                      ,choose_parameter(get_parameter(aParams,internal_np::edge_is_constrained),No_constrained_edge_map<TM>())
-                      ,choose_parameter(get_parameter(aParams,internal_np::get_cost_policy), LindstromTurk_cost<TM>())
-                      ,choose_parameter(get_parameter(aParams,internal_np::get_placement_policy), LindstromTurk_placement<TM>())
-                      ,choose_parameter(get_parameter(aParams,vis), Dummy_visitor())
-                      );
+  typedef typename GetGeomTraits<TM, NamedParameters>::type                   Geom_traits;
 
+  return internal::edge_collapse(tmesh, should_stop,
+                                 choose_parameter<Geom_traits>(get_parameter(np, internal_np::geom_traits)),
+                                 CGAL::get_initialized_vertex_index_map(tmesh, np),
+                                 choose_parameter(get_parameter(np, internal_np::vertex_point),
+                                                  get_property_map(vertex_point, tmesh)),
+                                 CGAL::get_initialized_halfedge_index_map(tmesh, np),
+                                 choose_parameter<No_constrained_edge_map<TM> >(get_parameter(np, internal_np::edge_is_constrained)),
+                                 choose_parameter<LindstromTurk_cost<TM> >(get_parameter(np, internal_np::get_cost_policy)),
+                                 choose_parameter<LindstromTurk_placement<TM> >(get_parameter(np, internal_np::get_placement_policy)),
+                                 choose_parameter<internal::Dummy_visitor>(get_parameter(np, internal_np::graph_visitor)));
 }
 
 template<class TM, class ShouldStop>
-int edge_collapse ( TM& aSurface, ShouldStop const& aShould_stop )
+int edge_collapse(TM& tmesh, const ShouldStop& should_stop)
 {
-  return edge_collapse(aSurface,aShould_stop, CGAL::parameters::halfedge_index_map(get(boost::halfedge_index,aSurface)));
-}
-
-  template<class TM, class ShouldStop, class GT>
-  int edge_collapse ( TM& aSurface, ShouldStop const& aShould_stop)
-{
-  return edge_collapse(aSurface,aShould_stop, CGAL::parameters::halfedge_index_map(get(boost::halfedge_index,aSurface)));
+  return edge_collapse(tmesh, should_stop, CGAL::parameters::all_default());
 }
 
 } // namespace Surface_mesh_simplification
+} // namespace CGAL
 
-} //namespace CGAL
-
-#endif // CGAL_SURFACE_MESH_SIMPLIFICATION_EDGE_COLLAPSE_H //
-// EOF //
-
+#endif // CGAL_SURFACE_MESH_SIMPLIFICATION_EDGE_COLLAPSE_H
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_simplification/Edge_collapse_visitor_base.h 5.1-1/include/CGAL/Surface_mesh_simplification/Edge_collapse_visitor_base.h
--- 5.0.3-1/include/CGAL/Surface_mesh_simplification/Edge_collapse_visitor_base.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_simplification/Edge_collapse_visitor_base.h	2020-09-08 12:15:45.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Edge_collapse_visitor_base.h $
-// $Id: Edge_collapse_visitor_base.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Edge_collapse_visitor_base.h $
+// $Id: Edge_collapse_visitor_base.h ff09c5d 2019-10-25T16:35:53+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Fernando Cacciola <fernando.cacciola@geometryfactory.com>
@@ -13,52 +13,47 @@
 
 #include <CGAL/license/Surface_mesh_simplification.h>
 
-
-#include <CGAL/Surface_mesh_simplification/Detail/Common.h>
+#include <CGAL/Surface_mesh_simplification/internal/Common.h>
 #include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_profile.h>
 
 namespace CGAL {
+namespace Surface_mesh_simplification {
 
-namespace Surface_mesh_simplification
-{
-
-template<class TM_>
+template <class TM_,
+          class VertexPointMap_ = typename boost::property_map<TM_, CGAL::vertex_point_t>::type,
+          class GeomTraits_ = typename Kernel_traits<typename boost::property_traits<VertexPointMap_>::value_type>::type>
 struct Edge_collapse_visitor_base
 {
-  typedef TM_ TM ;
-
-  typedef Edge_profile<TM> Profile ;
-
-  typedef boost::graph_traits  <TM> GraphTraits ;
-
-  typedef typename GraphTraits::edges_size_type   size_type ;
-  typedef typename GraphTraits::vertex_descriptor vertex_descriptor ;
-  typedef typename boost::property_map<TM, CGAL::vertex_point_t>::type Vertex_point_pmap;
-  typedef typename boost::property_traits<Vertex_point_pmap>::value_type Point;
-  typedef typename Kernel_traits<Point>::Kernel   Kernel ;
-  typedef typename Kernel::FT                     FT ;
-
-  void OnStarted( TM& ) {}
-
-  void OnFinished ( TM& ) {}
-
-  void OnStopConditionReached( Profile const& ) {}
-
-  void OnCollected( Profile const&, boost::optional<FT> const& ) {}
-
-  void OnSelected( Profile const&, boost::optional<FT> const&, size_type, size_type ) {}
-
-  void OnCollapsing(Profile const&, boost::optional<Point> const& ) {}
-
-  void OnCollapsed( Profile const&, vertex_descriptor const& ) {}
-
-   void OnNonCollapsable(Profile const& ) {}
-} ;
+  typedef TM_                                                                   TM; // backward compatibility
+  typedef TM_                                                                   TriangleMesh; // for the concept
+  typedef TM_                                                                   Triangle_mesh;
+
+  typedef boost::graph_traits<TM>                                               GraphTraits;
+  typedef typename GraphTraits::edges_size_type                                 size_type;
+  typedef typename GraphTraits::vertex_descriptor                               vertex_descriptor;
+  typedef typename GraphTraits::halfedge_descriptor                             halfedge_descriptor;
+
+  typedef VertexPointMap_                                                       Vertex_point_pmap; // backward compatibility
+  typedef VertexPointMap_                                                       Vertex_point_map;
+  typedef typename boost::property_traits<Vertex_point_map>::value_type         Point;
+
+  typedef GeomTraits_                                                           Kernel; // backward compatibility
+  typedef GeomTraits_                                                           Geom_traits;
+  typedef typename Geom_traits::FT                                              FT;
+
+  typedef Edge_profile<Triangle_mesh, Vertex_point_map, Geom_traits>            Profile;
+
+  void OnStarted(Triangle_mesh&) {}
+  void OnFinished(Triangle_mesh&) {}
+  void OnStopConditionReached(const Profile&) {}
+  void OnCollected(const Profile&, const boost::optional<FT>&) {}
+  void OnSelected(const Profile&, const boost::optional<FT>&, size_type, size_type) {}
+  void OnCollapsing(const Profile&, const boost::optional<Point>&) {}
+  void OnCollapsed(const Profile&, const vertex_descriptor&) {}
+  void OnNonCollapsable(const Profile&) {}
+};
 
 } // namespace Surface_mesh_simplification
+} // namespace CGAL
 
-} //namespace CGAL
-
-#endif // CGAL_SURFACE_MESH_SIMPLIFICATION_EDGE_COLLAPSE_VISITOR_BASE_H //
-// EOF //
-
+#endif // CGAL_SURFACE_MESH_SIMPLIFICATION_EDGE_COLLAPSE_VISITOR_BASE_H
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_simplification/HalfedgeGraph_Polyhedron_3.h 5.1-1/include/CGAL/Surface_mesh_simplification/HalfedgeGraph_Polyhedron_3.h
--- 5.0.3-1/include/CGAL/Surface_mesh_simplification/HalfedgeGraph_Polyhedron_3.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_simplification/HalfedgeGraph_Polyhedron_3.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,23 +0,0 @@
-// Copyright (c) 2006  GeometryFactory (France). All rights reserved.
-//
-// This file is part of CGAL (www.cgal.org).
-//
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/HalfedgeGraph_Polyhedron_3.h $
-// $Id: HalfedgeGraph_Polyhedron_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
-// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
-//
-// Author(s)     : Fernando Cacciola <fernando.cacciola@geometryfactory.com>
-//
-#ifndef CGAL_SURFACE_MESH_SIMPLIFICATION_HALFEDGEGRAPH_POLYHEDRON_3_H
-#define CGAL_SURFACE_MESH_SIMPLIFICATION_HALFEDGEGRAPH_POLYHEDRON_3_H
-
-#include <CGAL/license/Surface_mesh_simplification.h>
-
-
-#include <CGAL/Polyhedron_3.h>
-#include <CGAL/boost/graph/graph_traits_Polyhedron_3.h>
-
-#endif // CGAL_SURFACE_MESH_SIMPLIFICATION_HALFEDGEGRAPH_POLYHEDRON_3_H
-// EOF //
-
-
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_simplification/internal/Common.h 5.1-1/include/CGAL/Surface_mesh_simplification/internal/Common.h
--- 5.0.3-1/include/CGAL/Surface_mesh_simplification/internal/Common.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_simplification/internal/Common.h	2020-09-08 12:15:45.000000000 +0000
@@ -0,0 +1,127 @@
+// Copyright (c) 2006  GeometryFactory (France). All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/internal/Common.h $
+// $Id: Common.h 6f96731 2019-10-28T09:41:51+01:00 Mael Rouxel-Labbé
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Fernando Cacciola <fernando.cacciola@geometryfactory.com>
+//
+#ifndef CGAL_SURFACE_MESH_SIMPLIFICATION_DETAIL_COMMON_H
+#define CGAL_SURFACE_MESH_SIMPLIFICATION_DETAIL_COMMON_H
+
+#include <CGAL/license/Surface_mesh_simplification.h>
+
+#include <CGAL/algorithm.h>
+#include <CGAL/boost/graph/properties.h>
+#include <CGAL/boost/graph/iterator.h>
+
+#include <boost/config.hpp>
+#include <boost/iterator_adaptors.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/tuple/tuple.hpp>
+#include <boost/format.hpp>
+#include <boost/graph/graph_traits.hpp>
+#include <boost/graph/properties.hpp>
+#include <boost/graph/adjacency_list.hpp>
+
+#include <functional>
+#include <utility>
+#include <vector>
+#include <vector>
+#include <set>
+
+namespace CGAL {
+namespace Surface_mesh_simplification {
+
+template<class Handle>
+inline bool handle_assigned(Handle h) { Handle null; return h != null; }
+
+template<class Iterator, class Handle>
+bool handle_exists(Iterator begin, Iterator end, Handle h)
+{
+  if(handle_assigned(h))
+  {
+    while(begin != end)
+      if(begin++ == h)
+        return true;
+  }
+
+  return false;
+}
+
+template <class TM>
+struct No_constrained_edge_map
+{
+  typedef typename boost::graph_traits<TM>::edge_descriptor         key_type;
+  typedef bool                                                      value_type;
+  typedef value_type                                                reference;
+  typedef boost::readable_property_map_tag                          category;
+  friend bool get(No_constrained_edge_map, key_type) { return false; }
+};
+
+} // namespace Surface_mesh_simplification
+
+template<class N>
+inline std::string n_to_string(const N& n) {
+  return boost::str(boost::format("%|5.19g|") % n);
+}
+
+template<class XYZ>
+inline std::string xyz_to_string(const XYZ& xyz) {
+  return boost::str(boost::format("(%|5.19g|,%|5.19g|,%|5.19g|)") % xyz.x() % xyz.y() % xyz.z());
+}
+
+template<class Matrix>
+inline std::string matrix_to_string(const Matrix& m) {
+  return boost::str(boost::format("[%1%|%2%|%3%]") % xyz_to_string(m.r0()) % xyz_to_string(m.r1()) % xyz_to_string(m.r2()));
+}
+
+template<class T>
+inline std::string optional_to_string(const boost::optional<T>& o) {
+  if(o)
+    return boost::str(boost::format("%1%") % *o);
+  else return std::string("NONE");
+}
+
+} // namespace CGAL
+
+#if defined(CGAL_SURFACE_SIMPLIFICATION_ENABLE_TRACE) \
+  || defined(CGAL_SURFACE_SIMPLIFICATION_ENABLE_LT_TRACE)
+#define CGAL_SMS_ENABLE_TRACE
+#endif
+
+#ifdef CGAL_SMS_ENABLE_TRACE
+  #include<string>
+  #include<iostream>
+  #include<sstream>
+
+  namespace internal { namespace { bool cgal_enable_sms_trace = true; } }
+  #define CGAL_SMS_TRACE_IMPL(m) \
+    if(::internal::cgal_enable_sms_trace) { \
+    std::ostringstream ss; ss << m; std::string s = ss.str(); \
+    /*Surface_simplification_external_trace(s)*/ std::cerr << s << std::endl; \
+    }
+
+  #define CGAL_SMS_DEBUG_CODE(code) code
+#else
+  #define CGAL_SMS_DEBUG_CODE(code)
+#endif
+
+#ifdef CGAL_SURFACE_SIMPLIFICATION_ENABLE_LT_TRACE
+  #define CGAL_SMS_LT_TRACE(l,m) if((l) <= CGAL_SURFACE_SIMPLIFICATION_ENABLE_LT_TRACE) CGAL_SMS_TRACE_IMPL(m)
+#else
+  #define CGAL_SMS_LT_TRACE(l,m)
+#endif
+
+#ifdef CGAL_SURFACE_SIMPLIFICATION_ENABLE_TRACE
+  #define CGAL_SMS_TRACE_IF(c,l,m) if((c) && ((l) <= CGAL_SURFACE_SIMPLIFICATION_ENABLE_TRACE)) CGAL_SMS_TRACE_IMPL(m)
+  #define CGAL_SMS_TRACE(l,m)      if((l) <= CGAL_SURFACE_SIMPLIFICATION_ENABLE_TRACE) CGAL_SMS_TRACE_IMPL(m)
+#else
+  #define CGAL_SMS_TRACE_IF(c,l,m)
+  #define CGAL_SMS_TRACE(l,m)
+#endif
+#undef CGAL_SMS_ENABLE_TRACE
+
+#endif // CGAL_SURFACE_MESH_SIMPLIFICATION_DETAIL_COMMON_H
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_simplification/internal/Edge_collapse.h 5.1-1/include/CGAL/Surface_mesh_simplification/internal/Edge_collapse.h
--- 5.0.3-1/include/CGAL/Surface_mesh_simplification/internal/Edge_collapse.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_simplification/internal/Edge_collapse.h	2020-09-08 12:15:45.000000000 +0000
@@ -0,0 +1,1270 @@
+// Copyright (c) 2006  GeometryFactory (France). All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/internal/Edge_collapse.h $
+// $Id: Edge_collapse.h 0813fd7 2019-11-19T11:04:14+01:00 Mael Rouxel-Labbé
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Fernando Cacciola <fernando.cacciola@geometryfactory.com>
+//
+#ifndef CGAL_SURFACE_MESH_SIMPLIFICATION_DETAIL_EDGE_COLLAPSE_H
+#define CGAL_SURFACE_MESH_SIMPLIFICATION_DETAIL_EDGE_COLLAPSE_H
+
+#include <CGAL/license/Surface_mesh_simplification.h>
+
+#include <CGAL/Surface_mesh_simplification/internal/Common.h>
+#include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_profile.h>
+
+#include <CGAL/boost/graph/Euler_operations.h>
+#include <CGAL/boost/graph/helpers.h>
+#include <CGAL/Modifiable_priority_queue.h>
+
+#include <boost/scoped_array.hpp>
+
+namespace CGAL {
+namespace Surface_mesh_simplification {
+namespace internal {
+
+BOOST_MPL_HAS_XXX_TRAIT_DEF(Update_tag)
+
+template <typename Cost_oracle,
+          bool has_Update_tag = has_Update_tag<Cost_oracle>::value>
+struct Oracles_require_updates :
+  public CGAL::Boolean_tag<Cost_oracle::Update_tag::value>
+  // when Mesh_domain has the nested type Has_features
+{ };
+
+template <typename Cost_oracle>
+struct Oracles_require_updates<Cost_oracle, false> : public CGAL::Tag_false { };
+
+template <typename EdgeCollapse,
+          bool Tag = internal::Oracles_require_updates<typename EdgeCollapse::Get_cost>::value>
+struct Oracles_initializer
+{
+  Oracles_initializer(const EdgeCollapse& e) : e(e) { }
+  void operator()() const { }
+  const EdgeCollapse& e;
+};
+
+template <typename EdgeCollapse>
+struct Oracles_initializer<EdgeCollapse, true>
+{
+  Oracles_initializer(const EdgeCollapse& e) : e(e) { }
+  void operator()() const { e.get_cost().initialize(e.mesh(), e.vpm(), e.geom_traits()); }
+  const EdgeCollapse& e;
+};
+
+template <typename EdgeCollapse,
+          bool Tag = internal::Oracles_require_updates<typename EdgeCollapse::Get_cost>::value>
+struct After_collapse_oracles_updater
+{
+  After_collapse_oracles_updater(const EdgeCollapse& e) : e(e) { }
+  void operator()(const typename EdgeCollapse::Profile& /*profile*/,
+                  const typename boost::graph_traits<typename EdgeCollapse::Triangle_mesh>::vertex_descriptor /*v_kept*/) const { };
+  const EdgeCollapse& e;
+};
+
+template <typename EdgeCollapse>
+struct After_collapse_oracles_updater<EdgeCollapse, true>
+{
+  After_collapse_oracles_updater(const EdgeCollapse& e) : e(e) { }
+  void operator()(const typename EdgeCollapse::Profile& profile,
+                  const typename boost::graph_traits<typename EdgeCollapse::Triangle_mesh>::vertex_descriptor v_kept) const {
+    e.get_cost().update_after_collapse(profile, v_kept);
+  }
+  const EdgeCollapse& e;
+};
+
+} // namespace internal
+
+// Implementation of the vertex-pair collapse triangulated surface mesh simplification algorithm
+template<class TM_,
+         class GeomTraits_,
+         class ShouldStop_,
+         class VertexIndexMap_,
+         class VertexPointMap_,
+         class HalfedgeIndexMap_,
+         class EdgeIsConstrainedMap_,
+         class GetCost_,
+         class GetPlacement_,
+         class VisitorT_>
+class EdgeCollapse
+{
+  typedef EdgeCollapse                                                    Self;
+
+public:
+  typedef TM_                                                             Triangle_mesh;
+  typedef GeomTraits_                                                     Geom_traits;
+  typedef ShouldStop_                                                     Should_stop;
+  typedef VertexIndexMap_                                                 Vertex_index_map;
+  typedef VertexPointMap_                                                 Vertex_point_map;
+  typedef HalfedgeIndexMap_                                               Halfedge_index_map;
+  typedef EdgeIsConstrainedMap_                                           Edge_is_constrained_map;
+  typedef GetCost_                                                        Get_cost;
+  typedef GetPlacement_                                                   Get_placement;
+  typedef VisitorT_                                                       Visitor;
+
+  typedef Edge_profile<Triangle_mesh, Vertex_point_map, Geom_traits>      Profile;
+
+  typedef boost::graph_traits<Triangle_mesh>                              Graph_traits;
+  typedef typename Graph_traits::vertex_descriptor                        vertex_descriptor;
+  typedef typename Graph_traits::vertex_iterator                          vertex_iterator;
+  typedef typename Graph_traits::halfedge_descriptor                      halfedge_descriptor;
+  typedef typename Graph_traits::halfedge_iterator                        halfedge_iterator;
+  typedef typename Graph_traits::edge_descriptor                          edge_descriptor;
+  typedef typename Graph_traits::edge_iterator                            edge_iterator;
+  typedef CGAL::Halfedge_around_source_iterator<Triangle_mesh>            out_edge_iterator;
+  typedef typename Graph_traits::edges_size_type                          size_type;
+
+  typedef typename boost::property_traits<Vertex_point_map>::value_type   Point;
+
+  typedef typename Geom_traits::FT                                        FT;
+  typedef typename Geom_traits::Vector_3                                  Vector;
+  typedef typename Geom_traits::Equal_3                                   Equal_3;
+
+  typedef boost::optional<FT>                                             Cost_type;
+  typedef boost::optional<Point>                                          Placement_type;
+
+  struct Compare_id
+  {
+    Compare_id() : m_algorithm(0) {}
+    Compare_id(const Self* algorithm) : m_algorithm(algorithm) {}
+
+    bool operator()(const halfedge_descriptor a, const halfedge_descriptor b) const {
+      return m_algorithm->get_halfedge_id(a) < m_algorithm->get_halfedge_id(b);
+    }
+
+    const Self* m_algorithm;
+  };
+
+  struct Compare_cost
+  {
+    Compare_cost() : m_algorithm(0) {}
+    Compare_cost(const Self* algorithm) : m_algorithm(algorithm) {}
+
+    bool operator()(const halfedge_descriptor a, const halfedge_descriptor b) const
+    {
+      // NOTE: A cost is a boost::optional<> value.
+      // Absent optionals are ordered first; that is, "none < T" and "T > none" for any defined T != none.
+      // In consequence, edges with undefined costs will be promoted to the top of the priority queue and poped out first.
+      return m_algorithm->get_data(a).cost() < m_algorithm->get_data(b).cost();
+    }
+
+    const Self* m_algorithm;
+  };
+
+  struct edge_id
+    : boost::put_get_helper<size_type, edge_id>
+  {
+    typedef boost::readable_property_map_tag category;
+    typedef size_type                        value_type;
+    typedef size_type                        reference;
+    typedef halfedge_descriptor              key_type;
+
+    edge_id() : m_algorithm(0) {}
+    edge_id(const Self* algorithm) : m_algorithm(algorithm) {}
+
+    size_type operator[](const halfedge_descriptor e) const { return m_algorithm->get_edge_id(e); }
+
+    const Self* m_algorithm;
+  };
+
+  typedef Modifiable_priority_queue<halfedge_descriptor, Compare_cost, edge_id>     PQ;
+  typedef typename PQ::handle                                                       PQ_handle;
+
+  // An Edge_data is associated with EVERY _ edge in the mesh (collapsable or not).
+  // It relates the edge with the PQ-handle needed to update the priority queue
+  // It also relates the edge with a policy-based cache
+  class Edge_data
+  {
+  public :
+    Edge_data() : m_PQ_h() {}
+
+    const Cost_type& cost() const { return m_cost; }
+    Cost_type& cost() { return m_cost; }
+
+    PQ_handle queue_handle() const { return m_PQ_h;}
+    bool is_in_PQ() const { return m_PQ_h != PQ::null_handle(); }
+    void set_PQ_handle(PQ_handle h) { m_PQ_h = h; }
+    void reset_queue_handle() { m_PQ_h = PQ::null_handle(); }
+
+  private:
+    Cost_type m_cost;
+    PQ_handle m_PQ_h;
+  };
+
+  typedef Edge_data*                                                                Edge_data_ptr;
+  typedef boost::scoped_array<Edge_data>                                            Edge_data_array;
+
+public:
+  EdgeCollapse(Triangle_mesh& tmesh,
+               const Geom_traits& traits,
+               const Should_stop& should_stop,
+               const Vertex_index_map& vim,
+               const Vertex_point_map& vpm,
+               const Halfedge_index_map& him,
+               const Edge_is_constrained_map& ecm,
+               const Get_cost& aGetCost,
+               const Get_placement& aGetPlacement,
+               Visitor visitor);
+
+  int run();
+
+public:
+  const Triangle_mesh& mesh() const { return m_tm; }
+  const Geom_traits& geom_traits() const { return m_traits; }
+  const Get_cost& get_cost() const { return m_get_cost; }
+  const Vertex_point_map& vpm() const { return m_vpm; }
+
+private:
+  void collect();
+  void loop();
+
+  bool is_collapse_topologically_valid(const Profile& profile);
+  bool is_tetrahedron(const halfedge_descriptor h);
+  bool is_open_triangle(const halfedge_descriptor h1);
+  bool is_collapse_geometrically_valid(const Profile& profile, Placement_type placement);
+  void collapse(const Profile& profile, Placement_type placement);
+  void update_neighbors(const vertex_descriptor v_kept);
+
+  Profile create_profile(const halfedge_descriptor h) {
+    return Profile(h, m_tm, m_traits, m_vim, m_vpm, m_him, m_has_border);
+  }
+
+  size_type get_halfedge_id(const halfedge_descriptor h) const { return get(m_him, h); }
+  size_type get_edge_id(const halfedge_descriptor h) const { return get_halfedge_id(h) / 2; }
+
+  bool is_primary_edge(const halfedge_descriptor h) const { return (get_halfedge_id(h) % 2) == 0; }
+  halfedge_descriptor primary_edge(const halfedge_descriptor h) {
+    return is_primary_edge(h) ? h : opposite(h, m_tm);
+  }
+
+  bool is_constrained(const vertex_descriptor v) const;
+  bool is_constrained(const halfedge_descriptor h) const {
+    return get(m_ecm, edge(h, m_tm));
+  }
+
+  bool is_border_or_constrained(const vertex_descriptor v) const;
+  bool is_border_or_constrained(const halfedge_descriptor h) const { return is_border(h) || is_constrained(h); }
+
+  bool are_shared_triangles_valid(const Point& p0, const Point& p1, const Point& p2, const Point& p3) const;
+
+  halfedge_descriptor find_connection(const vertex_descriptor v0, const vertex_descriptor v1) const;
+  vertex_descriptor find_exterior_link_triangle_3rd_vertex(const halfedge_descriptor e, const vertex_descriptor v0, const vertex_descriptor v1) const;
+
+  Edge_data& get_data(const halfedge_descriptor h) const
+  {
+    CGAL_assertion(is_primary_edge(h));
+    return m_edge_data[get_edge_id(h)];
+  }
+
+  typename boost::property_traits<Vertex_point_map>::reference
+  get_point(const vertex_descriptor v) const { return get(m_vpm, v); }
+
+  boost::tuple<vertex_descriptor, vertex_descriptor> get_vertices(const halfedge_descriptor h) const
+  {
+    vertex_descriptor p, q;
+    p = source(h, m_tm);
+    q = target(h, m_tm);
+    return boost::make_tuple(p, q);
+  }
+
+  std::string vertex_to_string(const vertex_descriptor v) const
+  {
+    const Point& p = get_point(v);
+    return boost::str(boost::format("[V%1%:%2%]") % get(m_vim,v) % xyz_to_string(p));
+  }
+
+  std::string edge_to_string(const halfedge_descriptor h) const
+  {
+    vertex_descriptor p, q;
+    boost::tie(p,q) = get_vertices(h);
+    return boost::str(boost::format("{E%1% %2%->%3%}%4%") % get_edge_id(h) % vertex_to_string(p) % vertex_to_string(q) % (is_border(h, m_tm) ? " (BORDER)" : (is_border(opposite(h, m_tm), m_tm) ? " (~BORDER)": "")));
+  }
+
+  Cost_type cost(const Profile& profile) const {
+    return m_get_cost(profile, get_placement(profile));
+  }
+
+  Placement_type get_placement(const Profile& profile) const {
+    return m_get_placement(profile);
+  }
+
+  void insert_in_PQ(const halfedge_descriptor h, Edge_data& data)
+  {
+    CGAL_assertion(is_primary_edge(h));
+    CGAL_expensive_assertion(!data.is_in_PQ());
+    CGAL_expensive_assertion(!mPQ->contains(h));
+
+    data.set_PQ_handle(mPQ->push(h));
+
+    CGAL_expensive_assertion(data.is_in_PQ());
+    CGAL_expensive_assertion(mPQ->contains(h));
+  }
+
+  void update_in_PQ(const halfedge_descriptor h, Edge_data& data)
+  {
+    CGAL_assertion(is_primary_edge(h));
+    CGAL_expensive_assertion(data.is_in_PQ());
+    CGAL_expensive_assertion(mPQ->contains(h));
+
+    data.set_PQ_handle(mPQ->update(h, data.queue_handle()));
+
+    CGAL_assertion(data.is_in_PQ());
+    CGAL_expensive_assertion(mPQ->contains(h));
+  }
+
+  void remove_from_PQ(const halfedge_descriptor h, Edge_data& data)
+  {
+    CGAL_assertion(is_primary_edge(h));
+    CGAL_expensive_assertion(data.is_in_PQ());
+    CGAL_expensive_assertion(mPQ->contains(h));
+
+    data.set_PQ_handle(mPQ->erase(h, data.queue_handle()));
+
+    CGAL_expensive_assertion(!data.is_in_PQ());
+    CGAL_expensive_assertion(!mPQ->contains(h));
+  }
+
+  boost::optional<halfedge_descriptor> pop_from_PQ()
+  {
+    boost::optional<halfedge_descriptor> opt_h = mPQ->extract_top();
+    if(opt_h)
+    {
+      CGAL_assertion(is_primary_edge(*opt_h));
+      CGAL_expensive_assertion(get_data(*opt_h).is_in_PQ());
+
+      get_data(*opt_h).reset_queue_handle();
+
+      CGAL_expensive_assertion(!get_data(*opt_h).is_in_PQ());
+      CGAL_expensive_assertion(!mPQ->contains(*opt_h));
+    }
+    return opt_h;
+  }
+
+  /// Functions to ensure the backward compatibility before addition of the constrained edge map
+  template<class IsConstrainedMap_>
+  vertex_descriptor halfedge_collapse_bk_compatibility(const halfedge_descriptor h,
+                                                       const IsConstrainedMap_& ecm)
+  {
+    return CGAL::Euler::collapse_edge(edge(h, m_tm), m_tm, ecm);
+  }
+  template<class TM>
+  vertex_descriptor halfedge_collapse_bk_compatibility(const halfedge_descriptor h,
+                                                       const No_constrained_edge_map<TM>&)
+  {
+    return CGAL::Euler::collapse_edge(edge(h, m_tm), m_tm);
+  }
+
+  /// We wrap this test to avoid penalizing runtime when no constraints are present
+  template<class ECM_>
+  bool is_edge_adjacent_to_a_constrained_edge(const Profile& profile, const ECM_&)
+  {
+    return is_constrained(profile.v0()) && is_constrained(profile.v1());
+  }
+  template<class TM>
+  bool is_edge_adjacent_to_a_constrained_edge(const halfedge_descriptor, const No_constrained_edge_map<TM>&)
+  {
+    return false;
+  }
+
+private:
+  Triangle_mesh& m_tm;
+  const Geom_traits& m_traits;
+  const Should_stop& m_should_stop;
+  const Vertex_index_map& m_vim;
+  const Vertex_point_map& m_vpm;
+  const Halfedge_index_map& m_him;
+  const Edge_is_constrained_map& m_ecm;
+  const Get_cost& m_get_cost;
+  const Get_placement& m_get_placement;
+  Visitor m_visitor;
+  bool m_has_border;
+
+private:
+  Edge_data_array m_edge_data;
+
+  boost::scoped_ptr<PQ> mPQ;
+
+  size_type m_initial_edge_count;
+  size_type m_current_edge_count;
+
+  FT m_max_dihedral_angle_squared_cos;
+
+  CGAL_SMS_DEBUG_CODE(unsigned m_step;)
+};
+
+template<class TM, class GT, class SP, class VIM, class VPM,class HIM, class ECM, class CF, class PF, class V>
+EdgeCollapse<TM,GT,SP,VIM,VPM,HIM,ECM,CF,PF,V>::
+EdgeCollapse(Triangle_mesh& tmesh,
+             const Geom_traits& traits,
+             const Should_stop& should_stop,
+             const Vertex_index_map& vim,
+             const Vertex_point_map& vpm,
+             const Halfedge_index_map& him,
+             const Edge_is_constrained_map& ecm,
+             const Get_cost& get_cost,
+             const Get_placement& get_placement,
+             Visitor visitor)
+  :
+    m_tm(tmesh),
+    m_traits(traits),
+    m_should_stop(should_stop),
+    m_vim(vim),
+    m_vpm(vpm),
+    m_him(him),
+    m_ecm(ecm),
+    m_get_cost(get_cost),
+    m_get_placement(get_placement),
+    m_visitor(visitor),
+    m_has_border(!is_closed(tmesh))
+{
+  m_max_dihedral_angle_squared_cos = CGAL::square(std::cos(1.0 * CGAL_PI / 180.0));
+
+  CGAL_SMS_TRACE(0, "EdgeCollapse of TM with " << (num_edges(tmesh)/2) << " edges");
+
+  CGAL_SMS_DEBUG_CODE(m_step = 0;)
+
+#ifdef CGAL_SURFACE_SIMPLIFICATION_ENABLE_TRACE
+  for(vertex_descriptor vd : vertices(m_tm))
+    CGAL_SMS_TRACE(1, vertex_to_string(vd));
+
+  for(halfedge_descriptor ed : halfedges(m_tm))
+    CGAL_SMS_TRACE(1, edge_to_string(ed));
+#endif
+}
+
+template<class TM, class GT, class SP, class VIM, class VPM,class HIM, class ECM, class CF, class PF, class V>
+int
+EdgeCollapse<TM,GT,SP,VIM,VPM,HIM,ECM,CF,PF,V>::
+run()
+{
+  CGAL_precondition(is_valid_polygon_mesh(m_tm) && CGAL::is_triangle_mesh(m_tm));
+
+  m_visitor.OnStarted(m_tm);
+
+  // this is similar to the visitor, but for the cost/stop/placement oracles
+  internal::Oracles_initializer<Self>(*this)();
+
+  // First collect all candidate edges in a PQ
+  collect();
+
+  // Then proceed to collapse each edge in turn
+  loop();
+
+  CGAL_SMS_TRACE(0, "Finished: " << (m_initial_edge_count - m_current_edge_count) << " edges removed.");
+
+  int r = int(m_initial_edge_count - m_current_edge_count);
+
+  m_visitor.OnFinished(m_tm);
+
+  return r;
+}
+
+template<class TM, class GT, class SP, class VIM, class VPM,class HIM, class ECM, class CF, class PF, class V>
+void
+EdgeCollapse<TM,GT,SP,VIM,VPM,HIM,ECM,CF,PF,V>::
+collect()
+{
+  CGAL_SMS_TRACE(0, "collecting edges...");
+
+  // loop over all the _undirected_ edges in the surface putting them in the PQ
+
+  const size_type ne = num_edges(m_tm); // if the mesh has garbage, you might have "ne > edges(tm).size()"
+  m_initial_edge_count = m_current_edge_count = size_type(edges(m_tm).size());
+
+  m_edge_data.reset(new Edge_data[ne]);
+  mPQ.reset(new PQ(ne, Compare_cost(this), edge_id(this)));
+
+  CGAL_assertion_code(size_type num_inserted = 0);
+  CGAL_assertion_code(size_type num_not_inserted = 0);
+
+  std::set<halfedge_descriptor> zero_length_edges;
+
+  for(edge_descriptor e : edges(m_tm))
+  {
+    const halfedge_descriptor h = halfedge(e, m_tm);
+
+    if(is_constrained(h))
+    {
+      CGAL_assertion_code(++num_not_inserted);
+      continue; // no not insert constrainted edges
+    }
+
+    const Profile profile = create_profile(h);
+    if(!m_traits.equal_3_object()(profile.p0(), profile.p1()))
+    {
+      Edge_data& data = get_data(h);
+
+      data.cost() = cost(profile);
+      insert_in_PQ(h, data);
+
+      m_visitor.OnCollected(profile, data.cost());
+
+      CGAL_assertion_code(++num_inserted);
+    }
+    else
+    {
+      zero_length_edges.insert(primary_edge(h));
+      CGAL_assertion_code(++num_not_inserted);
+    }
+
+    CGAL_SMS_TRACE(2, edge_to_string(h));
+  }
+
+  CGAL_assertion(num_inserted + num_not_inserted == m_initial_edge_count);
+
+  for(halfedge_descriptor hd : zero_length_edges)
+  {
+    const Profile profile = create_profile(hd);
+
+    if(!is_collapse_topologically_valid(profile))
+      continue;
+
+    // edges of length 0 removed no longer need to be treated
+    if(profile.left_face_exists())
+    {
+      halfedge_descriptor h_to_remove = is_constrained(profile.vL_v0()) ?
+                                          primary_edge(profile.v1_vL()) :
+                                          primary_edge(profile.vL_v0());
+      zero_length_edges.erase(h_to_remove);
+      Edge_data& data = get_data(h_to_remove);
+
+      if(data.is_in_PQ())
+      {
+        CGAL_SMS_TRACE(2, "Removing E" << get_edge_id(h_to_remove) << " from PQ");
+        remove_from_PQ(h_to_remove, data);
+      }
+
+      --m_current_edge_count;
+    }
+
+    if(profile.right_face_exists())
+    {
+      halfedge_descriptor h_to_remove = is_constrained(profile.vR_v1()) ?
+                                          primary_edge(profile.v0_vR()) :
+                                          primary_edge(profile.vR_v1());
+      zero_length_edges.erase(h_to_remove);
+      Edge_data& data = get_data(h_to_remove);
+
+      if(data.is_in_PQ())
+      {
+        CGAL_SMS_TRACE(2, "Removing E" << get_edge_id(h_to_remove) << " from PQ");
+        remove_from_PQ(h_to_remove, data);
+      }
+
+      --m_current_edge_count;
+    }
+
+    --m_current_edge_count;
+
+    //the placement is trivial, it's always the point itself
+    Placement_type placement = profile.p0();
+    vertex_descriptor v = halfedge_collapse_bk_compatibility(profile.v0_v1(), m_ecm);
+    put(m_vpm, v, *placement);
+
+    m_visitor.OnCollapsed(profile, v);
+    internal::After_collapse_oracles_updater<Self>(*this)(profile, v);
+  }
+
+  CGAL_SMS_TRACE(0, "Initial edge count: " << m_initial_edge_count);
+}
+
+template<class TM, class GT, class SP, class VIM, class VPM,class HIM, class ECM, class CF, class PF, class V>
+void
+EdgeCollapse<TM,GT,SP,VIM,VPM,HIM,ECM,CF,PF,V>::
+loop()
+{
+  CGAL_SMS_TRACE(0, "Collapsing edges...");
+
+  CGAL_assertion_code(size_type non_collapsable_count = 0);
+
+  // Pops and processes each edge from the PQ
+
+  boost::optional<halfedge_descriptor> opt_h;
+
+#ifdef CGAL_SURF_SIMPL_INTERMEDIATE_STEPS_PRINTING
+  int i_rm = 0;
+#endif
+
+  while((opt_h = pop_from_PQ()))
+  {
+    CGAL_SMS_TRACE(1, "Popped " << edge_to_string(*opt_h));
+    CGAL_assertion(!is_constrained(*opt_h));
+
+    const Profile profile = create_profile(*opt_h);
+    Cost_type cost = get_data(*opt_h).cost();
+
+    m_visitor.OnSelected(profile, cost, m_initial_edge_count, m_current_edge_count);
+
+    if(cost)
+    {
+      if(m_should_stop(*cost, profile, m_initial_edge_count, m_current_edge_count))
+      {
+        m_visitor.OnStopConditionReached(profile);
+
+        CGAL_SMS_TRACE(0, "Stop condition reached with initial edge count=" << m_initial_edge_count
+                            << " current edge count=" << m_current_edge_count
+                            << " current edge: " << edge_to_string(*opt_h));
+        break;
+      }
+
+      if(is_collapse_topologically_valid(profile))
+      {
+        Placement_type placement = get_placement(profile);
+
+        if(is_collapse_geometrically_valid(profile, placement))
+        {
+#ifdef CGAL_SURF_SIMPL_INTERMEDIATE_STEPS_PRINTING
+          std::cout << "step " << i_rm << " " << get(m_vpm, source(*h, m_tm))
+                                       << " " << get(m_vpm, target(*h, m_tm)) << "\n";
+#endif
+          collapse(profile, placement);
+
+#ifdef CGAL_SURF_SIMPL_INTERMEDIATE_STEPS_PRINTING
+          std::stringstream sstr;
+          sstr << "debug/P-";
+          if(i_rm<10) sstr << "0";
+          if(i_rm<100) sstr << "0";
+          sstr << i_rm << ".off";
+          std::ofstream out(sstr.str().c_str());
+          out << m_tm;
+          ++i_rm;
+#endif
+        }
+      }
+      else
+      {
+        CGAL_assertion_code(++non_collapsable_count);
+
+        m_visitor.OnNonCollapsable(profile);
+
+        CGAL_SMS_TRACE(1, edge_to_string(*opt_h) << " NOT Collapsable" );
+      }
+    }
+    else
+    {
+      CGAL_SMS_TRACE(1, edge_to_string(*opt_h) << " uncomputable cost." );
+    }
+  }
+}
+
+template<class TM, class GT, class SP, class VIM, class VPM, class HIM, class ECM, class CF, class PF, class V>
+bool
+EdgeCollapse<TM,GT,SP,VIM,VPM,HIM,ECM,CF,PF,V>::
+is_border_or_constrained(const vertex_descriptor v) const
+{
+  for(halfedge_descriptor h : halfedges_around_target(v, m_tm))
+  {
+    if(is_border_edge(h) || is_constrained(h))
+      return true;
+  }
+
+  return false;
+}
+
+template<class TM, class GT, class SP, class VIM, class VPM, class HIM, class ECM, class CF, class PF, class V>
+bool
+EdgeCollapse<TM,GT,SP,VIM,VPM,HIM,ECM,CF,PF,V>::
+is_constrained(const vertex_descriptor v) const
+{
+  for(halfedge_descriptor h : halfedges_around_target(v, m_tm))
+    if(is_constrained(h))
+      return true;
+
+  return false;
+}
+
+// Some edges are NOT collapsable: doing so would break the topological consistency of the mesh.
+// This function returns true if a edge 'p->q' can be collapsed.
+//
+// An edge p->q can be collapsed iff it satisfies the "link condition"
+// (as described in the "Mesh Optimization" article of Hoppe et al (1993))
+//
+// The link condition is as follows: for every vertex 'k' adjacent to both 'p and 'q',
+// "p,k,q" is a facet of the mesh.
+//
+template<class TM, class GT, class SP, class VIM, class VPM, class HIM, class ECM, class CF, class PF, class V>
+bool
+EdgeCollapse<TM,GT,SP,VIM,VPM,HIM,ECM,CF,PF,V>::
+is_collapse_topologically_valid(const Profile& profile)
+{
+  bool res = true;
+
+  CGAL_SMS_TRACE(3,"Testing topological collapsabilty of p_q=V" << get(m_vim,profile.v0()) << "(%" << degree(profile.v0(), m_tm) << ")"
+                 << "->V" << get(m_vim, profile.v1()) << "(%" << degree(profile.v1(), m_tm) << ")");
+
+  CGAL_SMS_TRACE(4, "is p_q border:" << profile.is_v0_v1_a_border());
+  CGAL_SMS_TRACE(4, "is q_q border:" << profile.is_v1_v0_a_border());
+
+  out_edge_iterator eb1, ee1;
+  out_edge_iterator eb2, ee2;
+
+  CGAL_SMS_TRACE(4, "  t=V" << (profile.left_face_exists() ? get(m_vim,profile.vL()) : -1)
+                            << "(%" << (profile.left_face_exists() ? degree(profile.vL(), m_tm) : 0) << ")");
+  CGAL_SMS_TRACE(4, "  b=V" << (profile.right_face_exists() ? get(m_vim,profile.vR()) : -1)
+                            << "(%" << (profile.right_face_exists() ? degree(profile.vR(), m_tm) :0) << ")");
+
+  // Simple tests handling the case of non-manifold situations at a vertex or edge (pinching)
+  // (even if we advertise one should not use a surface mesh with such features)
+  if(profile.left_face_exists())
+  {
+    if(CGAL::is_border(opposite(profile.v1_vL(), m_tm), m_tm) &&
+       CGAL::is_border(opposite(profile.vL_v0(), m_tm), m_tm))
+      return false;
+
+    if(profile.right_face_exists() &&
+       CGAL::is_border(opposite(profile.vR_v1(), m_tm), m_tm) &&
+       CGAL::is_border(opposite(profile.v0_vR(), m_tm), m_tm))
+      return false;
+  }
+  else
+  {
+    if(profile.right_face_exists())
+    {
+      if(CGAL::is_border(opposite(profile.vR_v1(), m_tm), m_tm) &&
+         CGAL::is_border(opposite(profile.v0_vR(), m_tm), m_tm))
+        return false;
+    }
+    else
+      return false;
+  }
+
+  // The following loop checks the link condition for v0_v1.
+  // Specifically, that for every vertex 'k' adjacent to both 'p and 'q', 'pkq' is a face of the mesh.
+  //
+  for(boost::tie(eb1,ee1) = halfedges_around_source(profile.v0(), m_tm); res && eb1 != ee1; ++eb1)
+  {
+    halfedge_descriptor v0_k = *eb1;
+
+    if(v0_k != profile.v0_v1())
+    {
+      vertex_descriptor k = target(v0_k, m_tm);
+
+      for(boost::tie(eb2,ee2) = halfedges_around_source(k, m_tm); res && eb2 != ee2; ++eb2)
+      {
+        halfedge_descriptor k_v1 = *eb2;
+
+        if(target(k_v1, m_tm) == profile.v1())
+        {
+          // At this point we know p-q-k are connected and we need to determine if this triangle is a face of the mesh.
+          //
+          // Since the mesh is known to be triangular there are at most two faces sharing the edge p-q.
+          //
+          // If p->q is NOT a border edge, the top face is p->q->t where t is target(next(p->q))
+          // If q->p is NOT a border edge, the bottom face is q->p->b where b is target(next(q->p))
+          //
+          // If k is either t or b then p-q-k *might* be a face of the mesh. It won't be if k==t but p->q is border
+          // or k==b but q->b is a border (because in that case even though there exists triangles p->q->t (or q->p->b)
+          // they are holes, not faces)
+          //
+
+          bool is_face = (profile.vL() == k && profile.left_face_exists()) ||
+                         (profile.vR() == k && profile.right_face_exists());
+
+          CGAL_assertion_code(
+            if(is_face)
+            {
+              // Is k_v1 the halfedge bounding the face 'k-v1-v0'?
+              if(!is_border(k_v1, m_tm) && target(next(k_v1, m_tm), m_tm) == profile.v0())
+              {
+                CGAL_assertion(target(k_v1, m_tm) == profile.v1());
+                CGAL_assertion(target(next(k_v1, m_tm), m_tm) == profile.v0());
+                CGAL_assertion(target(next(next(k_v1, m_tm), m_tm), m_tm) == k);
+              }
+              else // or is it the opposite?
+              {
+                halfedge_descriptor v1_k = opposite(k_v1, m_tm);
+                CGAL_assertion(!is_border(v1_k, m_tm));
+                CGAL_assertion(target(v1_k, m_tm) == k);
+                CGAL_assertion(target(next(v1_k, m_tm), m_tm) == profile.v0());
+                CGAL_assertion(target(next(next(v1_k, m_tm), m_tm), m_tm) == profile.v1());
+              }
+            }
+          );
+
+          if(!is_face)
+          {
+            CGAL_SMS_TRACE(3,"  k=V" << get(m_vim,k) << " IS NOT in a face with p-q. NON-COLLAPSABLE edge.");
+            res = false;
+            break;
+          }
+          else
+          {
+            CGAL_SMS_TRACE(4,"  k=V" << get(m_vim,k) << " is in a face with p-q");
+          }
+        }
+      }
+    }
+  }
+
+  if(res)
+  {
+    /// ensure two constrained edges cannot get merged
+    if(is_edge_adjacent_to_a_constrained_edge(profile, m_ecm))
+      return false;
+
+    if(profile.is_v0_v1_a_border())
+    {
+      if(is_open_triangle(profile.v0_v1()))
+      {
+        res = false;
+        CGAL_SMS_TRACE(3,"  p-q belongs to an open triangle. NON-COLLAPSABLE edge.");
+      }
+    }
+    else if(profile.is_v1_v0_a_border())
+    {
+      if(is_open_triangle(profile.v1_v0()))
+      {
+        res = false;
+        CGAL_SMS_TRACE(3,"  p-q belongs to an open triangle. NON-COLLAPSABLE edge.");
+      }
+    }
+    else
+    {
+      if(is_border(profile.v0(), m_tm) && is_border(profile.v1(), m_tm))
+      {
+        res = false;
+        CGAL_SMS_TRACE(3,"  both p and q are boundary vertices but p-q is not. NON-COLLAPSABLE edge.");
+      }
+      else
+      {
+        bool tetra = is_tetrahedron(profile.v0_v1());
+
+        //CGAL_assertion(tetra == m_tm.is_tetrahedron(profile.v0_v1()));
+
+        if(tetra)
+        {
+          res = false;
+          CGAL_SMS_TRACE(3,"  p-q belongs to a tetrahedron. NON-COLLAPSABLE edge.");
+        }
+
+        if(next(profile.v0_v1(), m_tm) == opposite(prev(profile.v1_v0(), m_tm), m_tm) &&
+           prev(profile.v0_v1(), m_tm) == opposite(next(profile.v1_v0(), m_tm), m_tm))
+        {
+          CGAL_SMS_TRACE(3,"  degenerate volume.");
+          return false;
+        }
+      }
+    }
+  }
+
+  return res;
+}
+
+template<class TM, class GT, class SP, class VIM, class VPM, class HIM, class ECM, class CF, class PF, class V>
+bool
+EdgeCollapse<TM,GT,SP,VIM,VPM,HIM,ECM,CF,PF,V>::
+is_tetrahedron(const halfedge_descriptor h)
+{
+  return CGAL::is_tetrahedron(h, m_tm);
+}
+
+template<class TM, class GT, class SP, class VIM, class VPM, class HIM, class ECM, class CF, class PF, class V>
+bool
+EdgeCollapse<TM,GT,SP,VIM,VPM,HIM,ECM,CF,PF,V>::
+is_open_triangle(const halfedge_descriptor h1)
+{
+  bool res = false;
+
+  halfedge_descriptor h2 = next(h1, m_tm);
+  halfedge_descriptor h3 = next(h2, m_tm);
+
+  // First check if it is a triangle
+  if(next(h3, m_tm) == h1)
+  {
+    // Now check if it is open
+    CGAL_SMS_TRACE(4,"  p-q is a border edge... checking E" << get_edge_id(h2) << " and E" << get_edge_id(h3));
+
+    res = is_border(h2, m_tm) && is_border(h3, m_tm);
+
+    CGAL_assertion(res == (is_border(h1, m_tm) &&
+                           is_border(next(h1, m_tm), m_tm) &&
+                           is_border(next(next(h1, m_tm), m_tm), m_tm)));
+  }
+
+  return res;
+}
+
+// Given triangles 'p0,p1,p2' and 'p0,p2,p3', both shared along edge 'v0-v2',
+// determine if they are geometrically valid: that is, the ratio of their
+// respective areas is no greater than a max value and the internal
+// dihedral angle formed by their supporting planes is no greater than
+// a given threshold
+template<class TM, class GT, class SP, class VIM, class VPM, class HIM, class ECM, class CF, class PF, class V>
+bool
+EdgeCollapse<TM,GT,SP,VIM,VPM,HIM,ECM,CF,PF,V>::
+are_shared_triangles_valid(const Point& p0, const Point& p1, const Point& p2, const Point& p3) const
+{
+  bool res = false;
+
+  Vector e01 = m_traits.construct_vector_3_object()(p0, p1);
+  Vector e02 = m_traits.construct_vector_3_object()(p0, p2);
+  Vector e03 = m_traits.construct_vector_3_object()(p0, p3);
+
+  Vector n012 = m_traits.construct_cross_product_vector_3_object()(e01, e02);
+  Vector n023 = m_traits.construct_cross_product_vector_3_object()(e02, e03);
+
+  FT l012 = m_traits.compute_scalar_product_3_object()(n012, n012);
+  FT l023 = m_traits.compute_scalar_product_3_object()(n023, n023);
+
+  FT larger = (std::max)(l012, l023);
+  FT smaller = (std::min)(l012, l023);
+
+  const FT max_area_ratio = 1e8;
+
+  CGAL_SMS_TRACE(4,"    Testing validity of shared triangles:"
+                 << "\n      p0=" << xyz_to_string(p0) << "\n      p1=" << xyz_to_string(p1) << "\n      p2=" << xyz_to_string(p2) << "\n      p3=" << xyz_to_string(p3)
+                 << "\n      e01=" << xyz_to_string(e01) << "\n      e02=" << xyz_to_string(e02) << "\n      e03=" << xyz_to_string(e03)
+                 << "\n      n012=" << xyz_to_string(n012) << "\n      n023=" << xyz_to_string(n023)
+                 << "\n      l012=" << n_to_string(l012) << "\n      l023=" << n_to_string(l023));
+
+  if(larger < max_area_ratio * smaller)
+  {
+    FT l0123 = m_traits.compute_scalar_product_3_object()(n012, n023);
+    CGAL_SMS_TRACE(4,"\n      l0123=" << n_to_string(l0123));
+
+    if(CGAL_NTS is_positive(l0123))
+    {
+      res = true;
+    }
+    else
+    {
+      CGAL_SMS_TRACE(4,"\n      lhs: " << n_to_string((l0123 * l0123) / (l012 * l023)) << " <= rhs: " << m_max_dihedral_angle_squared_cos);
+
+      if((l0123 * l0123) <= m_max_dihedral_angle_squared_cos * (l012 * l023))
+      {
+        res = true;
+      }
+    }
+  }
+
+  return res;
+}
+
+// Returns the directed halfedge connecting v0 to v1, if exists.
+template<class TM, class GT, class SP, class VIM, class VPM, class HIM, class ECM, class CF, class PF, class V>
+typename EdgeCollapse<TM,GT,SP,VIM,VPM,HIM,ECM,CF,PF,V>::halfedge_descriptor
+EdgeCollapse<TM,GT,SP,VIM,VPM,HIM,ECM,CF,PF,V>::
+find_connection(const vertex_descriptor v0,
+                const vertex_descriptor v1) const
+{
+  for(halfedge_descriptor out : halfedges_around_source(v0, m_tm))
+  {
+    if(target(out, m_tm) == v1)
+      return out;
+  }
+
+  return Graph_traits::null_halfedge();
+}
+
+// Given the edge 'e' around the link for the collapsinge edge "v0-v1", finds the vertex that makes a triangle adjacent to 'e' but exterior to the link (i.e not containing v0 nor v1)
+// If 'e' is a null handle OR 'e' is a border edge, there is no such triangle and a null handle is returned.
+template<class TM, class GT, class SP, class VIM, class VPM, class HIM, class ECM, class CF, class PF, class V>
+typename EdgeCollapse<TM,GT,SP,VIM,VPM,HIM,ECM,CF,PF,V>::vertex_descriptor
+EdgeCollapse<TM,GT,SP,VIM,VPM,HIM,ECM,CF,PF,V>::
+find_exterior_link_triangle_3rd_vertex(const halfedge_descriptor e,
+                                       const vertex_descriptor v0,
+                                       const vertex_descriptor v1) const
+{
+  vertex_descriptor r;
+
+  if(handle_assigned(e))
+  {
+    vertex_descriptor ra = target(next(e, m_tm), m_tm);
+    vertex_descriptor rb = source(prev(e, m_tm), m_tm);
+
+    if(ra == rb && ra != v0 && ra != v1)
+    {
+      r = ra;
+    }
+    else
+    {
+      ra = target(next(opposite(e, m_tm), m_tm), m_tm);
+      rb = source(prev(opposite(e, m_tm), m_tm), m_tm);
+
+      if(ra == rb && ra != v0 && ra != v1)
+      {
+        r = ra;
+      }
+    }
+  }
+
+  return r;
+}
+
+// A collapse is geometrically valid if, in the resulting local mesh no two adjacent triangles form an internal dihedral angle
+// greater than a fixed threshold (i.e. triangles do not "fold" into each other)
+//
+template<class TM, class GT, class SP, class VIM, class VPM, class HIM, class ECM, class CF, class PF, class V>
+bool
+EdgeCollapse<TM,GT,SP,VIM,VPM,HIM,ECM,CF,PF,V>::
+is_collapse_geometrically_valid(const Profile& profile, Placement_type k0)
+{
+  bool res = false;
+
+  CGAL_SMS_TRACE(3,"Testing geometrical collapsabilty of v0-v1=E" << get_edge_id(profile.v0_v1()));
+  if(k0)
+  {
+    res = true;
+
+    // Use the current link to extract all local triangles incident to 'vx' in the collapsed mesh
+    // (which at this point doesn't exist yet)
+    typedef typename std::vector<vertex_descriptor>::const_iterator link_iterator;
+
+    link_iterator linkb = profile.link().begin();
+    link_iterator linke = profile.link().end();
+    link_iterator linkl = std::prev(linke);
+
+    for(link_iterator l=linkb; l!=linke && res; ++l)
+    {
+      link_iterator pv = (l == linkb ? linkl : std::prev(l));
+      link_iterator nx = (l == linkl ? linkb : std::next(l));
+
+      // k0,k1 and k3 are three consecutive vertices along the link.
+      vertex_descriptor k1 = *pv;
+      vertex_descriptor k2 = *l;
+      vertex_descriptor k3 = *nx;
+
+      CGAL_SMS_TRACE(4, "  Screening link vertices k1=V" << get(m_vim, k1)
+                          << " k2=V" << get(m_vim, k2)
+                          << " k3=V" << get(m_vim, k3));
+
+      halfedge_descriptor e12 = find_connection(k1,k2);
+      halfedge_descriptor e23 = (k3 != k1) ? find_connection(k2,k3) : Graph_traits::null_halfedge();
+
+      // If 'k1-k2-k3' are connected there will be two adjacent triangles 'k0,k1,k2' and 'k0,k2,k3' after the collapse.
+      if(handle_assigned(e12) && handle_assigned(e23))
+      {
+        CGAL_SMS_TRACE(4,"    Link triangles shared");
+
+        if(!are_shared_triangles_valid(*k0, get_point(k1), get_point(k2), get_point(k3)))
+        {
+          CGAL_SMS_TRACE(3, "    Triangles VX-V" << get(m_vim, k1)
+                              << "-V" << get(m_vim, k2)
+                              << " and VX-V" << get(m_vim, k3)
+                              << " are not geometrically valid. Collapse rejected");
+          res = false;
+        }
+      }
+
+      if(res)
+      {
+        // Also check the triangles 'k0,k1,k2' and it's adjacent along e12: 'k4,k2,k1', if exist
+        vertex_descriptor k4 = find_exterior_link_triangle_3rd_vertex(e12, profile.v0(), profile.v1());
+
+        // There is indeed a triangle shared along e12
+        if(handle_assigned(k4))
+        {
+          CGAL_SMS_TRACE(4, "    Found exterior link triangle shared along E" << get_edge_id(e12)
+                              << " with third vertex: V" << get(m_vim, k4));
+
+          if(!are_shared_triangles_valid(get_point(k1), get_point(k4), get_point(k2), *k0))
+          {
+            CGAL_SMS_TRACE(3, "    Triangles V" << get(m_vim, k1)
+                                << "-V" << get(m_vim, k4) << " and V"
+                                << get(m_vim, k2) << "-VX are not geometrically valid. Collapse rejected");
+            res = false;
+          }
+        }
+      }
+
+      if(res)
+      {
+        // And finally, check the triangles 'k0,k2,k3' and it's adjacent e23: 'k5,k3,k2' if exist
+        vertex_descriptor k5 = find_exterior_link_triangle_3rd_vertex(e23, profile.v0(), profile.v1());
+
+        // There is indeed a triangle shared along e12
+        if(handle_assigned(k5))
+        {
+          CGAL_SMS_TRACE(4, "    Found exterior link triangle shared along E" << get_edge_id(e23)
+                             << " with third vertex: V" << get(m_vim, k5));
+
+          if(!are_shared_triangles_valid(get_point(k2), get_point(k5), get_point(k3), *k0))
+          {
+            CGAL_SMS_TRACE(3, "    Triangles V" << get(m_vim, k2)
+                                << "-V" << get(m_vim, k5)
+                                << " and V" << get(m_vim, k3) << "-VX are not geometrically valid. Collapse rejected");
+            res = false;
+          }
+        }
+      }
+    }
+  }
+
+  return res;
+}
+
+template<class TM, class GT, class SP, class VIM, class VPM, class HIM, class ECM, class CF, class PF, class V>
+void
+EdgeCollapse<TM,GT,SP,VIM,VPM,HIM,ECM,CF,PF,V>::
+collapse(const Profile& profile,
+         Placement_type placement)
+{
+  CGAL_SMS_TRACE(1, "S" << m_step << ". Collapsing " << edge_to_string(profile.v0_v1()));
+
+  vertex_descriptor v_res;
+
+  m_visitor.OnCollapsing(profile, placement);
+
+  --m_current_edge_count;
+
+  CGAL_assertion_code(
+    size_type resulting_vertex_count = size_type(vertices(m_tm).size());
+    size_type result_edge_count = size_type(edges(m_tm).size());
+  );
+
+  // If the top/bottom facets exists, they are removed and the edges v0vt and Q-B along with them.
+  // In that case their corresponding pairs must be pop off the queue
+  if(profile.left_face_exists())
+  {
+    halfedge_descriptor h_v0vL = primary_edge(profile.vL_v0());
+    if(is_constrained(h_v0vL)) // make sure a constrained edge will not disappear
+      h_v0vL = primary_edge(profile.v1_vL());
+
+    CGAL_SMS_TRACE(3, "V0VL E" << get_edge_id(h_v0vL)
+                       << "(V" << get(m_vim, source(h_v0vL, m_tm))
+                       << "->V" << get(m_vim,target(h_v0vL, m_tm)) << ")");
+
+    Edge_data& data = get_data(h_v0vL);
+    if(data.is_in_PQ())
+    {
+      CGAL_SMS_TRACE(2, "Removing E" << get_edge_id(h_v0vL) << " from PQ");
+      remove_from_PQ(h_v0vL, data);
+    }
+
+    --m_current_edge_count;
+    CGAL_assertion_code(--result_edge_count);
+  }
+
+  if(profile.right_face_exists())
+  {
+    halfedge_descriptor lVRV1 = primary_edge(profile.vR_v1());
+    if(is_constrained(lVRV1)) // make sure a constrained edge will not disappear
+      lVRV1 = primary_edge(profile.v0_vR());
+
+    CGAL_SMS_TRACE(3, "V1VRE" << get_edge_id(lVRV1)
+                        << "(V" << get(m_vim, source(lVRV1, m_tm))
+                        << "->V" << get(m_vim, target(lVRV1, m_tm)) << ")");
+
+    Edge_data& data = get_data(lVRV1);
+    if(data.is_in_PQ())
+    {
+      CGAL_SMS_TRACE(2, "Removing E" << get_edge_id(lVRV1) << " from PQ");
+      remove_from_PQ(lVRV1, data);
+    }
+
+    --m_current_edge_count;
+    CGAL_assertion_code(--result_edge_count);
+  }
+
+  CGAL_SMS_TRACE(1, "Removing:\n  v0v1: E" << get_edge_id(profile.v0_v1())
+                      << "(V" << get(m_vim, profile.v0())
+                      << "->V" << get(m_vim, profile.v1()) << ")");
+
+  // Perform the actuall collapse.
+  // This is an external function.
+  // It's REQUIRED to remove ONLY 1 vertex (P or Q) and edges PQ, PT and QB
+  // (PT and QB are removed if they are not null).
+  // All other edges must be kept.
+  // All directed edges incident to vertex removed are relink to the vertex kept.
+  v_res = halfedge_collapse_bk_compatibility(profile.v0_v1(), m_ecm);
+
+  CGAL_assertion_code(--result_edge_count);
+  CGAL_assertion_code(--resulting_vertex_count);
+
+  CGAL_assertion(result_edge_count == edges(m_tm).size());
+  CGAL_assertion(resulting_vertex_count == vertices(m_tm).size());
+  CGAL_expensive_assertion(is_valid_polygon_mesh(m_tm) && CGAL::is_triangle_mesh(m_tm));
+
+  CGAL_SMS_TRACE(1, "V" << get(m_vim, v_res) << " kept.");
+
+#ifdef CGAL_SURFACE_SIMPLIFICATION_ENABLE_TRACE
+  for(halfedge_descriptor hd : halfedges_around_source(v_res, m_tm))
+    CGAL_SMS_TRACE(2, edge_to_string(hd));
+#endif
+
+  if(placement)
+  {
+    CGAL_SMS_TRACE(1, "New vertex point: " << xyz_to_string(*placement));
+    put(m_vpm, v_res, *placement);
+  }
+
+  m_visitor.OnCollapsed(profile, v_res);
+  internal::After_collapse_oracles_updater<Self>(*this)(profile, v_res);
+
+  update_neighbors(v_res);
+
+  CGAL_SMS_DEBUG_CODE(++m_step;)
+}
+
+template<class TM, class GT, class SP, class VIM, class VPM, class HIM, class ECM, class CF, class PF, class V>
+void
+EdgeCollapse<TM,GT,SP,VIM,VPM,HIM,ECM,CF,PF,V>::
+update_neighbors(const vertex_descriptor v_kept)
+{
+  CGAL_SMS_TRACE(3,"Updating cost of neighboring edges...");
+
+  // (A) collect all edges to update their cost: all those around each vertex adjacent to the vertex kept
+  typedef std::set<halfedge_descriptor, Compare_id>                       Edge_set;
+
+  Edge_set edges_to_update(Compare_id(this));
+  Edge_set edges_to_insert(Compare_id(this));
+
+  // (A.1) loop around all vertices adjacent to the vertex kept
+  for(halfedge_descriptor h : halfedges_around_target(v_kept, m_tm))
+  {
+    vertex_descriptor v_adj = source(h, m_tm);
+
+    // (A.2) loop around all edges incident on each adjacent vertex
+    for(halfedge_descriptor h2 : halfedges_around_target(v_adj, m_tm))
+    {
+      h2 = primary_edge(h2);
+
+      Edge_data& data2 = get_data(h2);
+      CGAL_SMS_TRACE(4,"Inedge around V" << get(m_vim, v_adj) << edge_to_string(h2));
+
+      // Only edges still in the PQ needs to be updated, the other needs to be re-inserted
+      if(data2.is_in_PQ())
+        edges_to_update.insert(h2);
+      else
+        edges_to_insert.insert(h2);
+    }
+  }
+
+  // (B) Proceed to update the costs.
+  for(halfedge_descriptor h : edges_to_update)
+  {
+    Edge_data& data = get_data(h);
+    const Profile& profile = create_profile(h);
+    data.cost() = cost(profile);
+
+    CGAL_SMS_TRACE(3, edge_to_string(h) << " updated in the PQ");
+
+    update_in_PQ(h, data);
+  }
+
+  // (C) Insert ignored edges
+  //
+  // I think that this should be done for edges eliminated because of the geometric criteria
+  // and not the topological one.However maintaining such a set might be more expensive
+  // and hard to be safe ...
+  for(halfedge_descriptor h : edges_to_insert)
+  {
+    if(is_constrained(h))
+      continue; //do not insert constrained edges
+
+    Edge_data& data = get_data(h);
+    const Profile& profile = create_profile(h);
+    data.cost() = cost(profile);
+
+    CGAL_SMS_TRACE(3, edge_to_string(h) << " re-inserted in the PQ");
+    insert_in_PQ(h, data);
+  }
+}
+
+} // namespace Surface_mesh_simplification
+} // namespace CGAL
+
+#endif // CGAL_SURFACE_MESH_SIMPLIFICATION_I_EDGE_COLLAPSE_H //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Bounded_distance_placement.h 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Bounded_distance_placement.h
--- 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Bounded_distance_placement.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Bounded_distance_placement.h	2020-09-08 12:15:45.000000000 +0000
@@ -0,0 +1,192 @@
+// Copyright (c) 2019  GeometryFactory (France). All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Bounded_distance_placement.h $
+// $Id: Bounded_distance_placement.h d6d8fbe 2020-04-27T16:06:43+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Maxime Gimeno,
+//                 Mael Rouxel-Labbé
+//
+#ifndef CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_BOUNDED_DISTANCE_PLACEMENT_H
+#define CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_BOUNDED_DISTANCE_PLACEMENT_H
+
+#include <CGAL/license/Surface_mesh_simplification.h>
+
+#include <CGAL/AABB_tree.h>
+#include <CGAL/AABB_traits.h>
+#include <CGAL/AABB_triangle_primitive.h>
+#include <CGAL/assertions.h>
+#include <CGAL/Default.h>
+#include <CGAL/intersections.h>
+#include <CGAL/boost/graph/named_params_helper.h>
+
+#include <boost/optional.hpp>
+
+#include <vector>
+#include <type_traits>
+
+namespace CGAL {
+namespace Surface_mesh_simplification {
+
+// An AABB tree can also be passed to the placement instead, see undocumented specialization below
+template<typename BasePlacement, typename GeomTraits>
+class Bounded_distance_placement
+{
+  typedef GeomTraits                                                          Geom_traits;
+  typedef typename Geom_traits::FT                                            FT;
+
+  typedef typename GeomTraits::Triangle_3                                     Triangle;
+  typedef std::vector<Triangle>                                               Triangle_container;
+  typedef typename Triangle_container::iterator                               TC_iterator;
+
+  typedef CGAL::AABB_triangle_primitive<GeomTraits, TC_iterator>              Primitive;
+  typedef CGAL::AABB_traits<GeomTraits, Primitive>                            Traits;
+  typedef CGAL::AABB_tree<Traits>                                             AABB_tree;
+
+private:
+  template <typename Profile>
+  void initialize_tree(const Profile& profile) const
+  {
+    CGAL_static_assertion((std::is_same<GeomTraits, typename Profile::Geom_traits>::value));
+
+    typedef typename Profile::Triangle_mesh                                   Triangle_mesh;
+    typedef typename boost::graph_traits<Triangle_mesh>::halfedge_descriptor  halfedge_descriptor;
+    typedef typename boost::graph_traits<Triangle_mesh>::face_descriptor      face_descriptor;
+
+    CGAL_precondition(m_tree_ptr == nullptr);
+
+    const Triangle_mesh& tm = profile.surface_mesh();
+    const Geom_traits& gt = profile.geom_traits();
+
+    m_input_triangles.reserve(faces(tm).size());
+    for(face_descriptor f : faces(profile.surface_mesh()))
+    {
+      halfedge_descriptor h = halfedge(f, tm);
+      CGAL_assertion(!is_border(h, tm));
+
+      m_input_triangles.emplace_back(gt.construct_triangle_3_object()(
+                                       get(profile.vertex_point_map(), source(h, tm)),
+                                       get(profile.vertex_point_map(), target(h, tm)),
+                                       get(profile.vertex_point_map(), target(next(h, tm), tm))));
+    }
+
+    m_tree_ptr = new AABB_tree(m_input_triangles.begin(), m_input_triangles.end());
+    const_cast<AABB_tree*>(m_tree_ptr)->build();
+    const_cast<AABB_tree*>(m_tree_ptr)->accelerate_distance_queries();
+  }
+
+public:
+  Bounded_distance_placement(const FT dist,
+                             const BasePlacement& placement = BasePlacement())
+    :
+      m_sq_threshold_dist(CGAL::square(dist)),
+      m_tree_ptr(nullptr),
+      m_base_placement(placement)
+  { }
+
+  ~Bounded_distance_placement()
+  {
+    if(m_tree_ptr != nullptr)
+      delete m_tree_ptr;
+  }
+
+  template <typename Profile>
+  boost::optional<typename Profile::Point>
+  operator()(const Profile& profile) const
+  {
+    typedef typename Profile::Point                                           Point;
+
+    boost::optional<typename Profile::Point> op = m_base_placement(profile);
+    if(op)
+    {
+      if(m_tree_ptr == nullptr)
+        initialize_tree(profile);
+
+      CGAL_assertion(m_tree_ptr != nullptr);
+      CGAL_assertion(!m_tree_ptr->empty());
+
+      const Point& p = *op;
+
+      const Point& cp = m_tree_ptr->best_hint(p).first;
+
+      // We could do better by having access to the internal kd-tree
+      // and call search_any_point with a fuzzy_sphere.
+
+      // if no input vertex is closer than the threshold, then
+      // any face closer than the threshold is intersected by
+      // the sphere (avoid the inclusion of the mesh into the threshold sphere)
+      if(CGAL::compare_squared_distance(p, cp, m_sq_threshold_dist) != LARGER ||
+         m_tree_ptr->do_intersect(CGAL::Sphere_3<Geom_traits>(p, m_sq_threshold_dist)))
+        return op;
+
+      return boost::optional<Point>();
+    }
+
+    return op;
+  }
+
+private:
+  const FT m_sq_threshold_dist;
+  mutable const AABB_tree* m_tree_ptr;
+  mutable std::vector<Triangle> m_input_triangles;
+
+  const BasePlacement m_base_placement;
+};
+
+// Undocumented specizalization where an _already built_ AABB tree is passed
+template<typename BasePlacement, typename AABBTraits>
+class Bounded_distance_placement<BasePlacement, CGAL::AABB_tree<AABBTraits> >
+{
+  typedef CGAL::AABB_tree<AABBTraits>                                         AABB_tree;
+  typedef typename AABB_tree::AABB_traits::FT                                 FT;
+
+public:
+  Bounded_distance_placement(const FT dist,
+                             const AABB_tree& tree,
+                             const BasePlacement& placement = BasePlacement())
+    :
+      m_sq_threshold_dist(CGAL::square(dist)),
+      m_tree_ptr(&tree),
+      m_base_placement(placement)
+  { }
+
+  template <typename Profile>
+  boost::optional<typename Profile::Point>
+  operator()(const Profile& profile) const
+  {
+    typedef typename Profile::Geom_traits                                     Geom_traits;
+    typedef typename Profile::Point                                           Point;
+
+    boost::optional<typename Profile::Point> op = m_base_placement(profile);
+    if(op)
+    {
+      CGAL_assertion(m_tree_ptr != nullptr);
+      CGAL_assertion(!m_tree_ptr->empty());
+
+      const Point& p = *op;
+
+      const Point& cp = m_tree_ptr->best_hint(p).first;
+
+      if(CGAL::compare_squared_distance(p, cp, m_sq_threshold_dist) != LARGER ||
+         m_tree_ptr->do_intersect(CGAL::Sphere_3<Geom_traits>(p, m_sq_threshold_dist)))
+        return op;
+
+      return boost::optional<Point>();
+    }
+
+    return op;
+  }
+
+private:
+  const FT m_sq_threshold_dist;
+  mutable const AABB_tree* m_tree_ptr;
+
+  const BasePlacement m_base_placement;
+};
+
+} // namespace Surface_mesh_simplification
+} // namespace CGAL
+
+#endif // CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_BOUNDED_DISTANCE_PLACEMENT_H
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Bounded_normal_change_placement.h 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Bounded_normal_change_placement.h
--- 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Bounded_normal_change_placement.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Bounded_normal_change_placement.h	2020-09-08 12:15:45.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Bounded_normal_change_placement.h $
-// $Id: Bounded_normal_change_placement.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Bounded_normal_change_placement.h $
+// $Id: Bounded_normal_change_placement.h ff09c5d 2019-10-25T16:35:53+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Andreas Fabri
@@ -12,83 +12,85 @@
 #define CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_BOUNDED_NORMAL_CHANGE_PLACEMENT_H
 
 #include <CGAL/license/Surface_mesh_simplification.h>
+
+#include <CGAL/property_map.h>
+
 #include <boost/optional.hpp>
 
 namespace CGAL {
+namespace Surface_mesh_simplification {
 
-namespace Surface_mesh_simplification
-{
-
-template<class Placement>
+template<class GetPlacement>
 class Bounded_normal_change_placement
 {
 public:
-
-  typedef typename Placement::TM TM ;
-
-public:
-
-  Bounded_normal_change_placement(const Placement& placement = Placement() )
-    : mPlacement(placement)
+  Bounded_normal_change_placement(const GetPlacement& get_placement = GetPlacement())
+    : m_get_placement(get_placement)
   {}
 
   template <typename Profile>
   boost::optional<typename Profile::Point>
-  operator()( Profile const& aProfile) const
+  operator()(const Profile& profile) const
   {
-    boost::optional<typename Profile::Point> op = mPlacement(aProfile);
-    if(op){
+    typedef typename Profile::VertexPointMap                              Vertex_point_map;
+
+    typedef typename Profile::Geom_traits                                 Geom_traits;
+    typedef typename Geom_traits::Vector_3                                Vector;
+
+    typedef typename boost::property_traits<Vertex_point_map>::value_type Point;
+    typedef typename boost::property_traits<Vertex_point_map>::reference  Point_reference;
+
+    const Geom_traits& gt = profile.geom_traits();
+    const Vertex_point_map& vpm = profile.vertex_point_map();
+
+    boost::optional<typename Profile::Point> op = m_get_placement(profile);
+    if(op)
+    {
       // triangles returns the triangles of the star of the vertices of the edge to collapse
       // First the two trianges incident to the edge, then the other triangles
       // The second vertex of each triangle is the vertex that gets placed
-       const typename Profile::Triangle_vector& triangles = aProfile.triangles();
-       if(triangles.size()>2){
-         typedef typename Profile::Point Point;
-         typedef typename Profile::Kernel Traits;
-         typedef typename Traits::Vector_3 Vector;
-         typename Profile::VertexPointMap ppmap = aProfile.vertex_point_map();
+       const typename Profile::Triangle_vector& triangles = profile.triangles();
+       if(triangles.size() > 2)
+       {
          typename Profile::Triangle_vector::const_iterator it = triangles.begin();
-         if(aProfile.left_face_exists()){
+
+         if(profile.left_face_exists())
            ++it;
-         }
-         if(aProfile.right_face_exists()){
+         if(profile.right_face_exists())
            ++it;
-         }
-         while(it!= triangles.end()){
+
+         while(it!= triangles.end())
+         {
            const typename Profile::Triangle& t = *it;
-           Point p = get(ppmap,t.v0);
-           Point q = get(ppmap,t.v1);
-           Point r = get(ppmap,t.v2);
-           Point q2 = *op;
-
-           Vector eqp = Traits().construct_vector_3_object()(q,p) ;
-           Vector eqr = Traits().construct_vector_3_object()(q,r) ;
-           Vector eq2p = Traits().construct_vector_3_object()(q2,p) ;
-           Vector eq2r = Traits().construct_vector_3_object()(q2,r) ;
-
-           Vector n1 = Traits().construct_cross_product_vector_3_object()(eqp,eqr);
-           Vector n2 = Traits().construct_cross_product_vector_3_object()(eq2p,eq2r);
-           if(! is_positive(Traits().compute_scalar_product_3_object()(n1, n2))){
+           Point_reference p = get(vpm, t.v0);
+           Point_reference q = get(vpm, t.v1);
+           Point_reference r = get(vpm, t.v2);
+           const Point& q2 = *op;
+
+           Vector eqp = gt.construct_vector_3_object()(q, p);
+           Vector eqr = gt.construct_vector_3_object()(q, r);
+           Vector eq2p = gt.construct_vector_3_object()(q2, p);
+           Vector eq2r = gt.construct_vector_3_object()(q2, r);
+
+           Vector n1 = gt.construct_cross_product_vector_3_object()(eqp, eqr);
+           Vector n2 = gt.construct_cross_product_vector_3_object()(eq2p, eq2r);
+
+           if(!is_positive(gt.compute_scalar_product_3_object()(n1, n2)))
              return boost::optional<typename Profile::Point>();
-           }
+
            ++it;
          }
        }
     }
+
     return op;
   }
 
 private:
-
-  Placement  mPlacement ;
-
+  const GetPlacement m_get_placement;
 };
 
-
 } // namespace Surface_mesh_simplification
-
-} //namespace CGAL
+} // namespace CGAL
 
 #endif // CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_BOUNDED_NORMAL_CHANGE_PLACEMENT_H
-
-
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Constrained_placement.h 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Constrained_placement.h
--- 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Constrained_placement.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Constrained_placement.h	2020-09-08 12:15:45.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Constrained_placement.h $
-// $Id: Constrained_placement.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Constrained_placement.h $
+// $Id: Constrained_placement.h ff09c5d 2019-10-25T16:35:53+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Sebastien Loriot <sebastien.loriot@cgal.org>
@@ -13,58 +13,49 @@
 
 #include <CGAL/license/Surface_mesh_simplification.h>
 
-
-#include <CGAL/Surface_mesh_simplification/Detail/Common.h>
+#include <CGAL/Surface_mesh_simplification/internal/Common.h>
 #include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_profile.h>
 
 namespace CGAL {
-
-namespace Surface_mesh_simplification
-{
+namespace Surface_mesh_simplification {
 
 template<class BasePlacement, class EdgeIsConstrainedMap>
-class Constrained_placement : public BasePlacement
+class Constrained_placement
+  : public BasePlacement
 {
 public:
-
-  EdgeIsConstrainedMap Edge_is_constrained_map;
-
-public:
-  Constrained_placement(
-    EdgeIsConstrainedMap map=EdgeIsConstrainedMap(),
-    BasePlacement base=BasePlacement() )
-  : BasePlacement(base)
-  , Edge_is_constrained_map(map)
+  Constrained_placement(const EdgeIsConstrainedMap map = EdgeIsConstrainedMap(),
+                        const BasePlacement& base = BasePlacement())
+    : BasePlacement(base),
+      m_ecm(map)
   {}
 
   template <typename Profile>
-  optional<typename Profile::Point> operator()( Profile const& aProfile ) const
+  boost::optional<typename Profile::Point> operator()(const Profile& profile) const
   {
-    typedef typename Profile::TM                                TM;
-    typedef typename CGAL::Halfedge_around_target_iterator<TM>  in_edge_iterator;
+    typedef typename Profile::TM                                    TM;
+    typedef typename boost::graph_traits<TM>::halfedge_descriptor   halfedge_descriptor;
 
-    in_edge_iterator eb, ee ;
-    for ( boost::tie(eb,ee) = halfedges_around_target(aProfile.v0(),aProfile.surface_mesh());
-      eb != ee ; ++ eb )
+    for(halfedge_descriptor h : halfedges_around_target(profile.v0(), profile.surface_mesh()))
     {
-      if( get(Edge_is_constrained_map, edge(*eb,aProfile.surface_mesh())) )
-        return get(aProfile.vertex_point_map(),
-                   aProfile.v0());
+      if(get(m_ecm, edge(h, profile.surface_mesh())))
+        return get(profile.vertex_point_map(), profile.v0());
     }
-    for ( boost::tie(eb,ee) = halfedges_around_target(aProfile.v1(),aProfile.surface_mesh());
-      eb != ee ; ++ eb )
+
+    for(halfedge_descriptor h : halfedges_around_target(profile.v1(), profile.surface_mesh()))
     {
-      if( get(Edge_is_constrained_map, edge(*eb,aProfile.surface_mesh())) )
-        return get(aProfile.vertex_point_map(),
-                   aProfile.v1());
+      if(get(m_ecm, edge(h, profile.surface_mesh())))
+        return get(profile.vertex_point_map(), profile.v1());
     }
 
-    return static_cast<const BasePlacement*>(this)->operator()(aProfile);
+    return static_cast<const BasePlacement*>(this)->operator()(profile);
   }
+
+private:
+  EdgeIsConstrainedMap m_ecm;
 };
 
 } // namespace Surface_mesh_simplification
-
-} //namespace CGAL
+} // namespace CGAL
 
 #endif // CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_CONSTRAINED_PLACEMENT_H
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Count_ratio_stop_predicate.h 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Count_ratio_stop_predicate.h
--- 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Count_ratio_stop_predicate.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Count_ratio_stop_predicate.h	2020-09-08 12:15:45.000000000 +0000
@@ -2,25 +2,22 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Count_ratio_stop_predicate.h $
-// $Id: Count_ratio_stop_predicate.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Count_ratio_stop_predicate.h $
+// $Id: Count_ratio_stop_predicate.h ff09c5d 2019-10-25T16:35:53+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Fernando Cacciola <fernando.cacciola@geometryfactory.com>
 //
 #ifndef CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_COUNT_RATIO_STOP_PREDICATE_H
-#define CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_COUNT_RATIO_STOP_PREDICATE_H 1
+#define CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_COUNT_RATIO_STOP_PREDICATE_H
 
 #include <CGAL/license/Surface_mesh_simplification.h>
 
-
-#include <CGAL/Surface_mesh_simplification/Detail/Common.h>
+#include <CGAL/Surface_mesh_simplification/internal/Common.h>
 #include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_profile.h>
 
 namespace CGAL {
-
-namespace Surface_mesh_simplification
-{
+namespace Surface_mesh_simplification {
 
 //*******************************************************************************************************************
 //                                -= stopping condition predicate =-
@@ -30,42 +27,34 @@ namespace Surface_mesh_simplification
 //
 //*******************************************************************************************************************
 
-//
 // Stops when the ratio of initial to current vertex pairs is below some value.
-//
 template<class TM_>
 class Count_ratio_stop_predicate
 {
 public:
+  typedef TM_                                                 TM;
+  typedef typename boost::graph_traits<TM>::edges_size_type   size_type;
 
-  typedef TM_ TM ;
-
-  typedef Edge_profile<TM> Profile ;
-
-  typedef typename boost::graph_traits<TM>::edge_descriptor edge_descriptor ;
-  typedef typename boost::graph_traits<TM>::edges_size_type size_type ;
-
-  Count_ratio_stop_predicate( double aRatio ) : mRatio(aRatio) {}
+  Count_ratio_stop_predicate(const double ratio)
+    : m_ratio(ratio)
+  {
+    CGAL_warning(0. < ratio && ratio <= 1.);
+  }
 
-  template <typename F>
-  bool operator()( F const&       // aCurrentCost
-                 , Profile const& // aEdgeProfile
-                 , size_type         aInitialCount
-                 , size_type         aCurrentCount
-                 ) const
+  template <typename F, typename Profile>
+  bool operator()(const F& /*current_cost*/,
+                  const Profile& /*profile*/,
+                  size_type initial_edge_count,
+                  size_type current_edge_count) const
   {
-    return ( static_cast<double>(aCurrentCount) / static_cast<double>(aInitialCount) ) < mRatio ;
+    return (static_cast<double>(current_edge_count) / static_cast<double>(initial_edge_count)) < m_ratio;
   }
 
 private:
-
-  double mRatio ;
+  double m_ratio;
 };
 
 } // namespace Surface_mesh_simplification
+} // namespace CGAL
 
-} //namespace CGAL
-
-#endif // CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_COUNT_RATIO_STOP_PREDICATE_H //
-// EOF //
-
+#endif // CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_COUNT_RATIO_STOP_PREDICATE_H
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Count_stop_predicate.h 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Count_stop_predicate.h
--- 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Count_stop_predicate.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Count_stop_predicate.h	2020-09-08 12:15:45.000000000 +0000
@@ -2,25 +2,22 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Count_stop_predicate.h $
-// $Id: Count_stop_predicate.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Count_stop_predicate.h $
+// $Id: Count_stop_predicate.h ff09c5d 2019-10-25T16:35:53+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Fernando Cacciola <fernando.cacciola@geometryfactory.com>
 //
 #ifndef CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_COUNT_STOP_PREDICATE_H
-#define CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_COUNT_STOP_PREDICATE_H 1
+#define CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_COUNT_STOP_PREDICATE_H
 
 #include <CGAL/license/Surface_mesh_simplification.h>
 
-
-#include <CGAL/Surface_mesh_simplification/Detail/Common.h>
+#include <CGAL/Surface_mesh_simplification/internal/Common.h>
 #include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_profile.h>
 
 namespace CGAL {
-
-namespace Surface_mesh_simplification
-{
+namespace Surface_mesh_simplification {
 
 //*******************************************************************************************************************
 //                                -= stopping condition predicate =-
@@ -30,45 +27,32 @@ namespace Surface_mesh_simplification
 //
 //*******************************************************************************************************************
 
-//
 // Stops when the number of edges left falls below a given number.
-//
 template<class TM_>
 class Count_stop_predicate
 {
 public:
+  typedef TM_                                                                   TM;
+  typedef typename boost::graph_traits<TM>::edges_size_type                     size_type;
 
-  typedef TM_ TM ;
-
-  //  typedef Edge_profile<TM> Profile ;
-
-  typedef typename boost::graph_traits<TM>::edges_size_type size_type ;
-
-  // typedef typename Kernel::FT FT ;
-
-public :
-
-  Count_stop_predicate( std::size_t aThres ) : mThres(aThres) {}
+  Count_stop_predicate(const std::size_t edge_count_threshold)
+    : m_edge_count_threshold(edge_count_threshold)
+  { }
 
   template <typename F, typename Profile>
-  bool operator()( F const&         // aCurrentCost
-                 , Profile const& // aEdgeProfile
-                 , std::size_t    // aInitialCount
-                 , std::size_t       aCurrentCount
-                 ) const
+  bool operator()(const F& /*current_cost*/,
+                  const Profile& /*profile*/,
+                  std::size_t /*initial_edge_count*/,
+                  std::size_t current_edge_count) const
   {
-    return aCurrentCount < mThres ;
+    return current_edge_count < m_edge_count_threshold;
   }
 
 private:
-
-  std::size_t mThres ;
+  std::size_t m_edge_count_threshold;
 };
 
 } // namespace Surface_mesh_simplification
+} // namespace CGAL
 
-} //namespace CGAL
-
-#endif // CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_COUNT_STOP_PREDICATE_H //
-// EOF //
-
+#endif // CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_COUNT_STOP_PREDICATE_H
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Detail/Lindstrom_Turk_core.h 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Detail/Lindstrom_Turk_core.h
--- 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Detail/Lindstrom_Turk_core.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Detail/Lindstrom_Turk_core.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,192 +0,0 @@
-// Copyright (c) 2005, 2006 Fernando Luis Cacciola Carballal. All rights reserved.
-//
-// This file is part of CGAL (www.cgal.org).
-//
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Detail/Lindstrom_Turk_core.h $
-// $Id: Lindstrom_Turk_core.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
-// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
-//
-// Author(s)     : Fernando Cacciola <fernando_cacciola@ciudad.com.ar>
-//
-#ifndef CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_DETAIL_LINDSTROM_TURK_CORE_H
-#define CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_DETAIL_LINDSTROM_TURK_CORE_H 1
-
-#include <CGAL/license/Surface_mesh_simplification.h>
-
-
-#include <vector>
-
-#include <CGAL/Cartesian_converter.h>
-
-#include <CGAL/Surface_mesh_simplification/Detail/Common.h>
-#include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_profile.h>
-#include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/LindstromTurk_params.h>
-
-namespace CGAL {
-
-//
-// This should be in
-//
-// Implementation of the collapsing cost and placement strategy from:
-//
-//  "Fast and Memory Efficient Polygonal Symplification"
-//  Peter Lindstrom, Greg Turk
-//
-
-namespace Surface_mesh_simplification
-{
-
-  template<class TM_, class Profile_>
-class LindstromTurkCore
-{
-public:
-
-  typedef TM_    TM ;
-  typedef Profile_ Profile ;
-
-  typedef boost::graph_traits<TM> GraphTraits ;
-
-  typedef typename GraphTraits::vertex_descriptor vertex_descriptor ;
-  typedef typename GraphTraits::halfedge_descriptor   halfedge_descriptor ;
-
-  typedef LindstromTurk_params Params ;
-
-  typedef typename Profile::Point Point ;
-
-  typedef typename Profile::VertexPointMap Vertex_point_pmap;
-  typedef typename boost::property_traits<Vertex_point_pmap>::value_type TM_Point;
-
-  typedef typename Kernel_traits<TM_Point>::Kernel TM_Kernel ;
-
-  typedef typename Kernel_traits<Point>::Kernel Kernel;
-  typedef typename Kernel::Vector_3 Vector ;
-  typedef typename Kernel::FT       FT ;
-
-  typedef optional<FT>     Optional_FT ;
-  typedef optional<Point>  Optional_point ;
-  typedef optional<Vector> Optional_vector ;
-
-  typedef MatrixC33<Kernel> Matrix ;
-
-  typedef typename Profile::Triangle                 Triangle ;
-  typedef typename Profile::vertex_descriptor_vector vertex_descriptor_vector ;
-
-  typedef typename Profile::Triangle_vector       ::const_iterator const_triangle_iterator ;
-  typedef typename Profile::halfedge_descriptor_vector::const_iterator const_border_edge_iterator ;
-
-public:
-
-  LindstromTurkCore( Params const& aParams, Profile const& aProfile ) ;
-
-  Optional_point compute_placement() ;
-  Optional_FT    compute_cost( Optional_point const& p ) ;
-
-private :
-
-  struct Triangle_data
-  {
-    Triangle_data( Vector const& aNormalV, FT const& aNormalL ) : NormalV(aNormalV), NormalL(aNormalL) {}
-
-    Vector NormalV ;
-    FT     NormalL ;
-  } ;
-  struct Boundary_data
-  {
-    Boundary_data ( Point s_, Point t_, Vector const& v_, Vector const& n_ ) : s(s_), t(t_), v(v_), n(n_) {}
-
-    Point  s, t ;
-    Vector v, n ;
-  } ;
-  typedef std::vector<Triangle_data> Triangle_data_vector ;
-  typedef std::vector<Boundary_data> Boundary_data_vector ;
-
-private :
-
-  void Extract_triangle_data();
-  void Extract_boundary_data();
-
-  void Add_boundary_preservation_constraints( Boundary_data_vector const& aBdry ) ;
-  void Add_volume_preservation_constraints( Triangle_data_vector const& aTriangles );
-  void Add_boundary_and_volume_optimization_constraints( Boundary_data_vector const& aBdry, Triangle_data_vector const& aTriangles ) ;
-  void Add_shape_optimization_constraints( vertex_descriptor_vector const& aLink ) ;
-
-  FT Compute_boundary_cost( Vector const& v, Boundary_data_vector const& aBdry ) ;
-  FT Compute_volume_cost  ( Vector const& v, Triangle_data_vector const& aTriangles ) ;
-  FT Compute_shape_cost   ( Point  const& p, vertex_descriptor_vector const& aLink ) ;
-
-  Point get_point ( vertex_descriptor const& v ) const
-  {
-    return convert(get(mProfile.vertex_point_map(),v));
-  }
-
-  static Vector Point_cross_product ( Point const& a, Point const& b )
-  {
-    return cross_product(a-ORIGIN,b-ORIGIN);
-  }
-
-  // This is the (uX)(Xu) product described in the Lindstrom-Turk paper
-  static Matrix LT_product( Vector const& u )
-  {
-    FT a00 = ( u.y()*u.y() ) + ( u.z()*u.z() ) ;
-    FT a01 = -u.x()*u.y();
-    FT a02 = -u.x()*u.z();
-
-    FT a10 = a01 ;
-    FT a11 = ( u.x()*u.x() ) + ( u.z()*u.z() ) ;
-    FT a12 = - u.y() * u.z();
-
-    FT a20 = a02 ;
-    FT a21 = a12 ;
-    FT a22 =  ( u.x()*u.x() ) + ( u.y()*u.y() ) ;
-
-    return Matrix(a00,a01,a02
-                 ,a10,a11,a12
-                 ,a20,a21,a22
-                 );
-  }
-
-  static FT big_value() { return static_cast<FT>((std::numeric_limits<double>::max)()) ; }
-
-  static bool is_finite ( FT     const& n ) { return CGAL_NTS is_finite(n) ; }
-  static bool is_finite ( Point  const& p ) { return is_finite(p.x())  && is_finite(p.y())  && is_finite(p.z()) ;  }
-  static bool is_finite ( Vector const& v ) { return is_finite(v.x())  && is_finite(v.y())  && is_finite(v.z()) ;  }
-  static bool is_finite ( Matrix const& m ) { return is_finite(m.r0()) && is_finite(m.r1()) && is_finite(m.r2()) ; }
-
-  template<class T>
-  static optional<T> filter_infinity ( T const& n ) { return is_finite(n) ? optional<T>(n) : optional<T>() ; }
-
-  TM& surface() const { return mProfile.surface() ; }
-
-private:
-
-  Params const&  mParams ;
-  Profile const& mProfile ;
-
-    void Add_constraint_if_alpha_compatible( Vector const& Ai, FT const& bi ) ;
-
-    void Add_constraint_from_gradient ( Matrix const& H, Vector const& c ) ;
-
-private:
-
-  Triangle_data_vector mTriangle_data ;
-  Boundary_data_vector mBdry_data ;
-
-  int    mConstraints_n ;
-  Matrix mConstraints_A ;
-  Vector mConstraints_b ;
-
-  Cartesian_converter<TM_Kernel,Kernel> convert ;
-
-  FT mSquared_cos_alpha;
-  FT mSquared_sin_alpha;
-};
-
-} // namespace Surface_mesh_simplification
-
-} //namespace CGAL
-
-#include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Detail/Lindstrom_Turk_core_impl.h>
-
-#endif // CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_DETAIL_LINDSTROM_TURK_CORE_H //
-// EOF //
-
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Detail/Lindstrom_Turk_core_impl.h 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Detail/Lindstrom_Turk_core_impl.h
--- 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Detail/Lindstrom_Turk_core_impl.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Detail/Lindstrom_Turk_core_impl.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,611 +0,0 @@
-// Copyright (c) 2005, 2006 Fernando Luis Cacciola Carballal. All rights reserved.
-//
-// This file is part of CGAL (www.cgal.org).
-//
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Detail/Lindstrom_Turk_core_impl.h $
-// $Id: Lindstrom_Turk_core_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
-// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
-//
-// Author(s)     : Fernando Cacciola <fernando_cacciola@ciudad.com.ar>
-//
-#ifndef CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_DETAIL_LINDSTROM_TURK_CORE_IMPL_H
-#define CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_DETAIL_LINDSTROM_TURK_CORE_IMPL_H 1
-
-#include <CGAL/license/Surface_mesh_simplification.h>
-
-
-namespace CGAL {
-
-
-//
-// Implementation of the strategy from:
-//
-//  "Fast and Memory Efficient Polygonal Symplification"
-//  Peter Lindstrom, Greg Turk
-//
-
-namespace Surface_mesh_simplification
-{
-
-template<class TM, class K>
-LindstromTurkCore<TM,K>::LindstromTurkCore( Params const& aParams, Profile const& aProfile )
-  :
-   mParams(aParams)
-  ,mProfile(aProfile)
-  ,mConstraints_n(0)
-  ,mConstraints_A(NULL_MATRIX)
-  ,mConstraints_b(NULL_VECTOR)
-{
-  double alpha = 1.0 * CGAL_PI / 180.0;
-
-  FT cos_alpha = std::cos(alpha);
-  FT sin_alpha = std::sin(alpha);
-
-  mSquared_cos_alpha = cos_alpha * cos_alpha ;
-  mSquared_sin_alpha = sin_alpha * sin_alpha ;
-
-  Extract_triangle_data();
-  Extract_boundary_data();
-}
-
-template<class TM, class K>
-void LindstromTurkCore<TM,K>::Extract_boundary_data()
-{
-  mBdry_data.reserve(mProfile.border_edges().size());
-  for ( const_border_edge_iterator it = mProfile.border_edges().begin(), eit = mProfile.border_edges().end() ; it != eit ; ++ it )
-  {
-    halfedge_descriptor border_edge = *it ;
-
-    halfedge_descriptor face_edge = opposite(border_edge,surface()) ;
-
-    vertex_descriptor sv = source(face_edge,surface());
-    vertex_descriptor tv = target(face_edge,surface());
-
-    Point const& sp = get_point(sv);
-    Point const& tp = get_point(tv);
-
-    Vector v = tp - sp ;
-    Vector n = Point_cross_product(tp,sp) ;
-
-    CGAL_SMS_LT_TRACE(1,"  Boundary edge. S:" << xyz_to_string(sp) << " T:" << xyz_to_string(tp)
-                      << " V:" << xyz_to_string(v) << " N:" << xyz_to_string(n)
-                      ) ;
-
-    mBdry_data.push_back( Boundary_data(sp,tp,v,n) ) ;
-  }
-}
-
-template<class TM, class K>
-void LindstromTurkCore<TM,K>::Extract_triangle_data()
-{
-  mTriangle_data.reserve(mProfile.triangles().size());
-  for ( const_triangle_iterator it = mProfile.triangles().begin(), eit = mProfile.triangles().end() ; it != eit ; ++ it )
-  {
-    Triangle const& tri = *it ;
-
-    Point const& p0 = get_point(tri.v0);
-    Point const& p1 = get_point(tri.v1);
-    Point const& p2 = get_point(tri.v2);
-
-    Vector v01 = p1 - p0 ;
-    Vector v02 = p2 - p0 ;
-
-    Vector lNormalV = cross_product(v01,v02);
-
-    FT lNormalL = Point_cross_product(p0,p1) * (p2-ORIGIN);
-
-    CGAL_SMS_LT_TRACE(1,"  Extracting triangle v" << tri.v0 << "->v" << tri.v1 << "->v" << tri.v2
-                      << " N:" << xyz_to_string(lNormalV) << " L:" << n_to_string(lNormalL)
-                      );
-
-    mTriangle_data.push_back(Triangle_data(lNormalV,lNormalL));
-  }
-}
-
-template<class TM, class K>
-typename LindstromTurkCore<TM,K>::Optional_point LindstromTurkCore<TM,K>::compute_placement()
-{
-  Optional_point  rPlacementP ;
-  Optional_vector lPlacementV ;
-
-  CGAL_SMS_LT_TRACE(0,"Computing LT placement for E" << mProfile.v0_v1() << " (V" << mProfile.v0() << "->V" << mProfile.v1() << ")" );
-
-  //
-  // Each vertex constraint is an equation of the form: Ai * v = bi
-  // Where 'v' is a vector representing the vertex,
-  // 'Ai' is a (row) vector and 'bi' a scalar.
-  //
-  // The vertex is completely determined with 3 such constraints,
-  // so is the solution to the folloing system:
-  //
-  //  A.r0(). * v = b0
-  //  A1 * v = b1
-  //  A2 * v = b2
-  //
-  // Which in matrix form is :  A * v = b
-  //
-  // (with 'A' a 3x3 matrix and 'b' a vector)
-  //
-  // The member variable mConstrinas contains A and b. Indidivual constraints (Ai,bi) can be added to it.
-  // Once 3 such constraints have been added 'v' is directly solved a:
-  //
-  //  v = b*inverse(A)
-  //
-  // A constraint (Ai,bi) must be alpha-compatible with the previously added constraints (see Paper); if it's not, is discarded.
-  //
-  if ( mBdry_data.size() > 0 )
-    Add_boundary_preservation_constraints(mBdry_data);
-
-  if ( mConstraints_n < 3 )
-    Add_volume_preservation_constraints(mTriangle_data);
-
-  if ( mConstraints_n < 3 )
-    Add_boundary_and_volume_optimization_constraints(mBdry_data,mTriangle_data);
-
-  if ( mConstraints_n < 3 )
-    Add_shape_optimization_constraints(mProfile.link());
-
-  // It might happen that there were not enough alpha-compatible constraints.
-  // In that case there is simply no good vertex placement
-  if ( mConstraints_n == 3 )
-  {
-    // If the matrix is singular it's inverse cannot be computed so an 'absent' value is returned.
-    optional<Matrix> lOptional_Ai = inverse_matrix(mConstraints_A);
-    if ( lOptional_Ai )
-    {
-      Matrix const& lAi = *lOptional_Ai ;
-
-      CGAL_SMS_LT_TRACE(2,"       b: " << xyz_to_string(mConstraints_b) );
-      CGAL_SMS_LT_TRACE(2,"  inv(A): " << matrix_to_string(lAi) );
-
-      lPlacementV = filter_infinity(mConstraints_b * lAi) ;
-
-      CGAL_SMS_LT_TRACE(0,"  New vertex point: " << xyz_to_string(*lPlacementV) );
-    }
-    else
-    {
-      CGAL_SMS_LT_TRACE(1,"  Can't solve optimization, singular system.");
-    }
-  }
-  else
-  {
-    CGAL_SMS_LT_TRACE(1,"  Can't solve optimization, not enough alpha-compatible constraints.");
-  }
-
-  if ( lPlacementV )
-    rPlacementP = ORIGIN + (*lPlacementV) ;
-
-  return rPlacementP;
-}
-
-template<class TM, class K>
-typename LindstromTurkCore<TM,K>::Optional_FT LindstromTurkCore<TM,K>::compute_cost( Optional_point const& aP )
-{
-  Optional_FT rCost ;
-
-  if ( aP )
-  {
-    CGAL_SMS_LT_TRACE(0,"Computing LT cost for E" << mProfile.v0_v1() );
-    Vector lV = (*aP) - ORIGIN ;
-
-    FT lSquaredLength = squared_distance(mProfile.p0(),mProfile.p1());
-
-    FT lBdryCost   = Compute_boundary_cost(lV ,mBdry_data);
-    FT lVolumeCost = Compute_volume_cost  (lV ,mTriangle_data);
-    FT lShapeCost  = Compute_shape_cost   (*aP,mProfile.link());
-
-    FT lTotalCost =   FT(mParams.VolumeWeight)   * lVolumeCost
-                    + FT(mParams.BoundaryWeight) * lBdryCost   * lSquaredLength
-                    + FT(mParams.ShapeWeight)    * lShapeCost  * lSquaredLength * lSquaredLength ;
-
-    rCost = filter_infinity(lTotalCost);
-
-    CGAL_SMS_LT_TRACE(0,  "    Squared edge length: " << n_to_string(lSquaredLength)
-                      << "\n    Boundary cost: " << n_to_string(lBdryCost) << " weight: " << mParams.BoundaryWeight
-                      << "\n    Volume cost: " << n_to_string(lVolumeCost) << " weight: " << mParams.VolumeWeight
-                      << "\n    Shape cost: " << n_to_string(lShapeCost)   << " weight: " << mParams.ShapeWeight
-                      << "\n  TOTAL COST: " << n_to_string(lTotalCost)
-                      );
-
-  }
-
-  return rCost;
-}
-
-
-template<class TM, class K>
-void LindstromTurkCore<TM,K>::Add_boundary_preservation_constraints( Boundary_data_vector const& aBdry )
-{
-
-  if ( aBdry.size() > 0 )
-  {
-    Vector e1 = NULL_VECTOR ;
-    Vector e2 = NULL_VECTOR ;
-
-    FT e1x = FT(0.0), e1y = FT(0.0), e1z = FT(0.0) ;
-
-    for ( typename Boundary_data_vector::const_iterator it = aBdry.begin() ; it != aBdry.end() ; ++ it )
-    {
-      e1 = e1 + it->v ;
-      e2 = e2 + it->n ;
-
-      FT vx = it->v.x() ;
-      FT vy = it->v.y() ;
-      FT vz = it->v.z() ;
-
-      e1x = e1x + vx;
-      e1y = e1y + vy;
-      e1z = e1z + vz;
-
-      CGAL_SMS_LT_TRACE(1,"    vx:" << n_to_string(vx) << " vy:" << n_to_string(vy) << " vz:" << n_to_string(vz) << " e1x:"
-                            << n_to_string(e1x) << " e1y:" << n_to_string(e1y) << " e1z:" << n_to_string(e1z) );
-    }
-
-    Matrix H = LT_product(e1);
-
-    Vector c = cross_product(e1,e2);
-
-    CGAL_SMS_LT_TRACE(1
-                      ,"  Adding boundary preservation constraint."
-                       << "\n      SumV:" << xyz_to_string(e1)
-                       << "\n      SumN:" << xyz_to_string(e2)
-                       << "\n      H:" << matrix_to_string(H)
-                       << "\n      c:" << xyz_to_string(c)
-                      );
-
-    Add_constraint_from_gradient(H,c);
-  }
-}
-
-template<class TM, class K>
-void LindstromTurkCore<TM,K>::Add_volume_preservation_constraints( Triangle_data_vector const& aTriangles )
-{
-  CGAL_SMS_LT_TRACE(1,"  Adding volume preservation constraint. " << aTriangles.size() << " triangles.");
-
-  Vector lSumV = NULL_VECTOR ;
-  FT     lSumL(0) ;
-
-  for( typename Triangle_data_vector::const_iterator it = aTriangles.begin(), eit = aTriangles.end() ; it != eit ; ++it )
-  {
-    lSumV = lSumV + it->NormalV ;
-    lSumL = lSumL + it->NormalL ;
-  }
-
-  CGAL_SMS_LT_TRACE(1, "      SumV:" << xyz_to_string(lSumV) << "\n      SumL:" << n_to_string(lSumL) );
-
-  Add_constraint_if_alpha_compatible(lSumV,lSumL);
-
-}
-
-template<class TM, class K>
-void LindstromTurkCore<TM,K>::Add_boundary_and_volume_optimization_constraints( Boundary_data_vector const& aBdry
-                                                                            , Triangle_data_vector const& aTriangles
-                                                                            )
-{
-  CGAL_SMS_LT_TRACE(1,"  Adding boundary and volume optimization constraints. ");
-
-  Matrix H = NULL_MATRIX ;
-  Vector c = NULL_VECTOR ;
-
-  //
-  // Volume optimization
-  //
-  for( typename Triangle_data_vector::const_iterator it = aTriangles.begin(), eit = aTriangles.end() ; it != eit ; ++it )
-  {
-    Triangle_data const& lTri = *it ;
-
-    H += direct_product(lTri.NormalV,lTri.NormalV) ;
-
-    c = c - ( lTri.NormalL * lTri.NormalV ) ;
-  }
-
-  CGAL_SMS_LT_TRACE(2,"      Hv:" << matrix_to_string(H) << "\n      cv:" << xyz_to_string(c) ) ;
-
-
-  if ( aBdry.size() > 0 )
-  {
-    //
-    // Boundary optimization
-    //
-    Matrix Hb = NULL_MATRIX ;
-    Vector cb = NULL_VECTOR ;
-
-    for ( typename Boundary_data_vector::const_iterator it = aBdry.begin() ; it != aBdry.end() ; ++ it )
-    {
-      Matrix H = LT_product(it->v);
-      Vector c = cross_product(it->v,it->n);
-
-      Hb += H ;
-      cb = cb + c ;
-    }
-
-    CGAL_SMS_LT_TRACE(2,"      Hb:" << matrix_to_string(Hb) << "\n      cb:" << xyz_to_string(cb) ) ;
-
-    //
-    // Weighted average
-    //
-    FT lScaledBoundaryWeight = FT(9) * FT(mParams.BoundaryWeight) * squared_distance(mProfile.p0(),mProfile.p1())  ;
-
-    H *= mParams.VolumeWeight ;
-    c = c * mParams.VolumeWeight ;
-
-    H += lScaledBoundaryWeight * Hb ;
-    c = c + ( lScaledBoundaryWeight * cb ) ;
-
-    CGAL_SMS_LT_TRACE(2,"      H:" << matrix_to_string(H) << "\n      c:" << xyz_to_string(c) ) ;
-    CGAL_SMS_LT_TRACE(2,"      VolW:" << mParams.VolumeWeight << " BdryW:" << mParams.BoundaryWeight << " ScaledBdryW:" << lScaledBoundaryWeight ) ;
-
-  }
-
-  Add_constraint_from_gradient(H,c);
-}
-
-template<class TM, class K>
-void LindstromTurkCore<TM,K>::Add_shape_optimization_constraints( vertex_descriptor_vector const& aLink )
-{
-  FT s((double)aLink.size());
-
-  Matrix H (  s,0.0,0.0
-           ,0.0,  s,0.0
-           ,0.0,0.0,  s
-           );
-
-  Vector c = NULL_VECTOR ;
-
-  for( typename vertex_descriptor_vector::const_iterator it = aLink.begin(), eit = aLink.end() ; it != eit ; ++it )
-    c = c + (ORIGIN - get_point(*it)) ;
-
-  CGAL_SMS_LT_TRACE(1,"  Adding shape optimization constraint. Shape vector: " << xyz_to_string(c) );
-
-  Add_constraint_from_gradient(H,c);
-}
-
-template<class TM, class K>
-typename LindstromTurkCore<TM,K>::FT
-LindstromTurkCore<TM,K>::Compute_boundary_cost( Vector const& v, Boundary_data_vector const& aBdry )
-{
-  FT rCost(0);
-  for ( typename Boundary_data_vector::const_iterator it = aBdry.begin() ; it != aBdry.end() ; ++ it )
-  {
-    Vector u = (it->t - ORIGIN ) - v ;
-    Vector c = cross_product(it->v,u);
-    rCost += c*c;
-  }
-  return rCost / FT(4) ;
-}
-
-template<class TM, class K>
-typename LindstromTurkCore<TM,K>::FT
-LindstromTurkCore<TM,K>::Compute_volume_cost( Vector const& v, Triangle_data_vector const& aTriangles )
-{
-  FT rCost(0);
-
-  for( typename Triangle_data_vector::const_iterator it = aTriangles.begin(), eit = aTriangles.end() ; it != eit ; ++it )
-  {
-    Triangle_data const& lTri = *it ;
-
-    FT lF = lTri.NormalV * v - lTri.NormalL ;
-
-    rCost += ( lF * lF ) ;
-
-  }
-
-  return rCost / FT(36) ;
-}
-
-template<class TM, class K>
-typename LindstromTurkCore<TM,K>::FT
-LindstromTurkCore<TM,K>::Compute_shape_cost( Point const& p, vertex_descriptor_vector const& aLink )
-{
-  FT rCost(0);
-
-  for( typename vertex_descriptor_vector::const_iterator it = aLink.begin(), eit = aLink.end() ; it != eit ; ++it )
-    rCost += squared_distance(p,get_point(*it)) ;
-
-  return rCost ;
-}
-
-template<class TM, class K>
-void LindstromTurkCore<TM,K>::Add_constraint_if_alpha_compatible( Vector const& Ai, FT const& bi )
-{
-  CGAL_SMS_LT_TRACE(3,"    Adding new constraints if alpha-compatible.\n      Ai: " << xyz_to_string(Ai) << "\n      bi:" << n_to_string(bi) << ")" );
-
-  FT slai = Ai*Ai ;
-
-  CGAL_SMS_LT_TRACE(3,"\n      slai: " << n_to_string(slai) << ")" );
-
-  if ( is_finite(slai) )
-  {
-    FT l = CGAL_NTS sqrt( slai ) ;
-
-    CGAL_SMS_LT_TRACE(3,"      l: " << n_to_string(l) );
-
-    if ( !CGAL_NTS is_zero(l) )
-    {
-      Vector Ain = Ai / l ;
-      FT     bin = bi / l ;
-
-      CGAL_SMS_LT_TRACE(3,"      Ain: " << xyz_to_string(Ain) << " bin:" << n_to_string(bin) );
-
-      bool lAddIt = true ;
-
-      if ( mConstraints_n == 1 )
-      {
-        FT d01 = mConstraints_A.r0() * Ai  ;
-
-        FT sla0 = mConstraints_A.r0() * mConstraints_A.r0() ;
-        FT sd01 = d01 * d01 ;
-
-        FT max = sla0 * slai * mSquared_cos_alpha ;
-
-        CGAL_SMS_LT_TRACE(3,"      Second constraint. d01: " << n_to_string(d01) << " sla0:" << n_to_string(sla0) << " sd01:" << n_to_string(sd01) << " max:" << n_to_string(max) );
-
-        if ( sd01 > max )
-          lAddIt = false ;
-      }
-      else if ( mConstraints_n == 2 )
-      {
-        Vector N = cross_product(mConstraints_A.r0(),mConstraints_A.r1());
-
-        FT dc012 = N * Ai ;
-
-        FT slc01  = N * N ;
-        FT sdc012 = dc012 * dc012;
-
-        FT min = slc01 * slai * mSquared_sin_alpha ;
-
-        CGAL_SMS_LT_TRACE(3,"      Third constraint. N: " << xyz_to_string(N) << " dc012:" << n_to_string(dc012) << " slc01:" << n_to_string(slc01)
-                          << " sdc012:" << n_to_string(sdc012) << " min:" << n_to_string(min) );
-
-        if ( sdc012 <= min )
-          lAddIt = false ;
-      }
-
-      if ( lAddIt )
-      {
-        switch ( mConstraints_n )
-        {
-          case 0 :
-            mConstraints_A.r0() = Ain ;
-            mConstraints_b = Vector(bin,mConstraints_b.y(),mConstraints_b.z());
-            break ;
-          case 1 :
-            mConstraints_A.r1() = Ain ;
-            mConstraints_b = Vector(mConstraints_b.x(),bin,mConstraints_b.z());
-            break ;
-          case 2 :
-            mConstraints_A.r2() = Ain ;
-            mConstraints_b = Vector(mConstraints_b.x(),mConstraints_b.y(),bin);
-            break ;
-        }
-
-        CGAL_SMS_LT_TRACE(3,"      Accepting # " << mConstraints_n << " A:" << matrix_to_string(mConstraints_A) << " b:" << xyz_to_string(mConstraints_b) ) ;
-
-        ++ mConstraints_n ;
-
-      }
-      else
-      {
-        CGAL_SMS_LT_TRACE(3,"      INCOMPATIBLE. Discarded" ) ;
-      }
-    }
-    else
-    {
-      CGAL_SMS_LT_TRACE(3,"        l is ZERO. Discarded" );
-    }
-  }
-  else
-  {
-    CGAL_SMS_LT_TRACE(3,"      OVERFLOW. Discarded" ) ;
-  }
-}
-
-template<class V>
-int index_of_max_component ( V const& v )
-{
-  typedef typename Kernel_traits<V>::Kernel::FT FT ;
-
-  int i = 0 ;
-
-  FT max = v.x();
-
-  if ( max < v.y() )
-  {
-    max = v.y();
-    i = 1 ;
-  }
-  if ( max < v.z() )
-  {
-    max = v.z();
-    i = 2 ;
-  }
-  return i ;
-}
-
-template<class TM, class K>
-void LindstromTurkCore<TM,K>::Add_constraint_from_gradient ( Matrix const& H, Vector const& c )
-{
-  CGAL_SMS_LT_TRACE(3,"    Adding constraint from gradient. Current n=" << mConstraints_n ) ;
-
-  CGAL_precondition(mConstraints_n >= 0 && mConstraints_n<=2 );
-
-  switch(mConstraints_n)
-  {
-    case 0 :
-
-      Add_constraint_if_alpha_compatible(H.r0(),-c.x());
-      Add_constraint_if_alpha_compatible(H.r1(),-c.y());
-      Add_constraint_if_alpha_compatible(H.r2(),-c.z());
-
-      break;
-
-    case 1 :
-      {
-        Vector const& A0 = mConstraints_A.r0();
-
-        CGAL_assertion( A0 != NULL_VECTOR ) ;
-
-        Vector AbsA0( CGAL::abs(A0.x())
-                      , CGAL::abs(A0.y())
-                      , CGAL::abs(A0.z())
-                    );
-
-        Vector Q0;
-        switch ( index_of_max_component(AbsA0) )
-        {
-          // Since A0 is guaranteed to be non-zero, the denominators here are known to be non-zero too.
-
-          case 0: Q0 = Vector(- A0.z()/A0.x(),0              ,1              ); break;
-          case 1: Q0 = Vector(0              ,- A0.z()/A0.y(),1              ); break;
-          case 2: Q0 = Vector(1              ,0              ,- A0.x()/A0.z()); break;
-
-           default : Q0 = NULL_VECTOR ; // This should never happen!
-        }
-
-        CGAL_SMS_LT_TRACE(3,"      Q0:" << xyz_to_string(Q0) ) ;
-
-        CGAL_assertion( Q0 != NULL_VECTOR ) ;
-
-        Vector Q1 = cross_product(A0,Q0);
-
-        Vector A1 = H * Q0 ;
-        Vector A2 = H * Q1 ;
-
-        FT b1 = - ( Q0 * c ) ;
-        FT b2 = - ( Q1 * c ) ;
-
-        CGAL_SMS_LT_TRACE(3,"      Q1:" << xyz_to_string(Q1) << "\n      A1: " << xyz_to_string(A1) << "\n      A2:" << xyz_to_string(A2)
-                          << "\n      b1:" << n_to_string(b1) << "\n      b2:" << n_to_string(b2) ) ;
-
-        Add_constraint_if_alpha_compatible(A1,b1);
-        Add_constraint_if_alpha_compatible(A2,b2);
-
-      }
-      break ;
-
-    case 2:
-      {
-
-        Vector Q = cross_product(mConstraints_A.r0(),mConstraints_A.r1());
-
-        Vector A2 = H * Q ;
-
-        FT b2 = - ( Q * c ) ;
-
-        CGAL_SMS_LT_TRACE(3,"      Q:" << xyz_to_string(Q) << "\n      A2: " << xyz_to_string(A2) << "\n      b2:" << n_to_string(b2) ) ;
-
-        Add_constraint_if_alpha_compatible(A2,b2);
-
-      }
-      break ;
-
-  }
-}
-
-} // namespace Surface_mesh_simplification
-
-} //namespace CGAL
-
-#endif // CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_DETAIL_LINDSTROMTURK_CORE_IMPL_H //
-// EOF //
-
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_length_cost.h 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_length_cost.h
--- 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_length_cost.h	2020-08-01 19:02:55.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_length_cost.h	2020-09-08 12:15:45.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_length_cost.h $
-// $Id: Edge_length_cost.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_length_cost.h $
+// $Id: Edge_length_cost.h ff09c5d 2019-10-25T16:35:53+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Fernando Cacciola <fernando.cacciola@geometryfactory.com>
@@ -13,51 +13,28 @@
 
 #include <CGAL/license/Surface_mesh_simplification.h>
 
-
-#include <CGAL/Surface_mesh_simplification/Detail/Common.h>
+#include <CGAL/Surface_mesh_simplification/internal/Common.h>
 #include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_profile.h>
 
 namespace CGAL {
+namespace Surface_mesh_simplification {
 
-namespace Surface_mesh_simplification
-{
-
-//
 // Edge-length cost: the squared length of the collapsing edge
-//
-  template<class TM>
+template<class TM>
 class Edge_length_cost
 {
 public:
-  /*
-  typedef TM_ TM ;
-
-  typedef Edge_profile<TM> Profile ;
-  typedef typename Profile::Point Point;
-  typedef typename Kernel_traits<Point>::Kernel Kernel ;
-  typedef typename Kernel::FT FT ;
-  typedef optional<FT> result_type ;
-  */
-public:
-
-  Edge_length_cost()
-  {}
+  Edge_length_cost() {}
 
   template <typename Profile, typename T>
-  optional<typename Profile::FT> operator()( Profile const& aProfile, T const& /*aPlacement*/ ) const
+  boost::optional<typename Profile::FT> operator()(const Profile& profile, const T& /*placement*/) const
   {
-    typedef optional<typename Profile::FT> result_type;
-    return result_type(squared_distance(aProfile.p0(),aProfile.p1()));
+    typedef boost::optional<typename Profile::FT>                     result_type;
+    return result_type(profile.geom_traits().compute_squared_distance_3_object()(profile.p0(), profile.p1()));
   }
-
 };
 
-
 } // namespace Surface_mesh_simplification
-
-
-} //namespace CGAL
+} // namespace CGAL
 
 #endif // CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_EDGE_LENGHT_COST_H
-// EOF //
-
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_length_stop_predicate.h 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_length_stop_predicate.h
--- 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_length_stop_predicate.h	2020-08-01 19:02:56.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_length_stop_predicate.h	2020-09-08 12:15:45.000000000 +0000
@@ -2,47 +2,44 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_length_stop_predicate.h $
-// $Id: Edge_length_stop_predicate.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_length_stop_predicate.h $
+// $Id: Edge_length_stop_predicate.h ff09c5d 2019-10-25T16:35:53+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Sebastien Loriot <sebastien.loriot@geometryfactory.com>
 //
 #ifndef CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_EDGE_LENGTH_STOP_PREDICATE_H
-#define CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_EDGE_LENGTH_STOP_PREDICATE_H 1
+#define CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_EDGE_LENGTH_STOP_PREDICATE_H
 
 #include <CGAL/license/Surface_mesh_simplification.h>
 #include <CGAL/squared_distance_3.h>
 
 namespace CGAL {
-
-namespace Surface_mesh_simplification
-{
+namespace Surface_mesh_simplification {
 
 template <class FT>
 class Edge_length_stop_predicate
 {
-  FT m_edge_sq_length_threshold;
 public:
-  Edge_length_stop_predicate( double edge_length_threshold )
-    : m_edge_sq_length_threshold(edge_length_threshold*edge_length_threshold)
+  Edge_length_stop_predicate(const FT edge_length_threshold)
+    : m_edge_sq_length_threshold(edge_length_threshold * edge_length_threshold)
   {}
 
-  template <typename F, typename Profile>
-  bool operator()( F const&
-                 , Profile const&  profile
-                 , std::size_t
-                 , std::size_t
-                 ) const
+  template <typename Profile>
+  bool operator()(const FT& /*current_cost*/,
+                  const Profile& profile,
+                  std::size_t /*initial_edge_count*/,
+                  std::size_t /*current_edge_count*/) const
   {
-    return  CGAL::squared_distance(profile.p0(), profile.p1()) >
-            m_edge_sq_length_threshold;
+    const typename Profile::Geom_traits& gt = profile.geom_traits();
+    return gt.compute_squared_distance_3_object()(profile.p0(), profile.p1()) > m_edge_sq_length_threshold;
   }
+
+private:
+  const FT m_edge_sq_length_threshold;
 };
 
 } // namespace Surface_mesh_simplification
+} // namespace CGAL
 
-} //namespace CGAL
-
-#endif // CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_EDGE_LENGTH_STOP_PREDICATE_H //
-
+#endif // CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_EDGE_LENGTH_STOP_PREDICATE_H
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_profile.h 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_profile.h
--- 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_profile.h	2020-08-01 19:02:56.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_profile.h	2020-09-08 12:15:45.000000000 +0000
@@ -2,190 +2,353 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_profile.h $
-// $Id: Edge_profile.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_profile.h $
+// $Id: Edge_profile.h 9a7c98f 2019-11-29T11:42:18+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Fernando Cacciola <fernando.cacciola@geometryfactory.com>
 //
 #ifndef CGAL_SURFACE_MESH_SIMPLIFICATION_DETAIL_EDGE_PROFILE_H
-#define CGAL_SURFACE_MESH_SIMPLIFICATION_DETAIL_EDGE_PROFILE_H 1
+#define CGAL_SURFACE_MESH_SIMPLIFICATION_DETAIL_EDGE_PROFILE_H
 
 #include <CGAL/license/Surface_mesh_simplification.h>
 
+#include <CGAL/Surface_mesh_simplification/internal/Common.h>
+
+#include <CGAL/Kernel_traits.h>
 
 #include <vector>
 #include <set>
 
-#include <CGAL/Surface_mesh_simplification/Detail/Common.h>
-
 namespace CGAL {
+namespace Surface_mesh_simplification {
 
-namespace Surface_mesh_simplification
-{
-
-  template<class TM_, class VertexPointMap_ = typename boost::property_map<TM_, CGAL::vertex_point_t>::type>
+template<class TM_,
+         class VertexPointMap_ = typename boost::property_map<TM_, CGAL::vertex_point_t>::type,
+         class GeomTraits_ = typename Kernel_traits<typename boost::property_traits<VertexPointMap_>::value_type>::type>
 class Edge_profile
 {
 public:
+  typedef TM_                                                                 TM; // backward compatibility
+  typedef TM_                                                                 Triangle_mesh;
+  typedef VertexPointMap_                                                     VertexPointMap; // backward compatibility
+  typedef VertexPointMap_                                                     Vertex_point_map;
+
+  typedef typename boost::property_traits<VertexPointMap>::value_type         Point;
+  typedef typename boost::property_traits<VertexPointMap>::reference          Point_reference;
+
+  typedef boost::graph_traits<TM>                                             GraphTraits;
+  typedef typename GraphTraits::vertex_descriptor                             vertex_descriptor;
+  typedef typename GraphTraits::halfedge_descriptor                           halfedge_descriptor;
+  typedef typename GraphTraits::edge_descriptor                               edge_descriptor;
+  typedef typename GraphTraits::face_descriptor                               face_descriptor;
+  typedef typename GraphTraits::edges_size_type                               edges_size_type;
 
-  typedef TM_ TM ;
-  typedef VertexPointMap_ VertexPointMap;
-  typedef boost::graph_traits<TM>       GraphTraits ;
-
-  typedef typename GraphTraits::vertex_descriptor vertex_descriptor ;
-  typedef typename GraphTraits::face_descriptor face_descriptor ;
-  typedef typename GraphTraits::halfedge_descriptor halfedge_descriptor ;
+  typedef std::vector<vertex_descriptor>                                      vertex_descriptor_vector;
+  typedef std::vector<halfedge_descriptor>                                    halfedge_descriptor_vector;
 
-
-  //typedef typename boost::property_map<TM, CGAL::vertex_point_t>::type Vertex_point_pmap;
-  typedef typename boost::property_traits<VertexPointMap>::value_type Point;
-  typedef typename Kernel_traits<Point>::Kernel Kernel;
-  typedef typename Kernel::FT FT;
+  typedef GeomTraits_                                                         Geom_traits;
+  typedef typename Geom_traits::FT                                            FT;
 
 public:
-
   struct Triangle
   {
     Triangle() {}
 
-    Triangle( vertex_descriptor const& a_v0
-            , vertex_descriptor const& a_v1
-            , vertex_descriptor const& a_v2
-            )
-            : v0(a_v0), v1(a_v1), v2(a_v2)
+    Triangle(const vertex_descriptor v0_,
+             const vertex_descriptor v1_,
+             const vertex_descriptor v2_)
+      : v0(v0_), v1(v1_), v2(v2_)
     {
-      CGAL_SURF_SIMPL_TEST_assertion( handle_assigned(v0) && handle_assigned(v1) && handle_assigned(v2) ) ;
-      CGAL_SURF_SIMPL_TEST_assertion( v0 != v1 && v1 != v2 && v2 != v0 ) ;
+      CGAL_assertion(handle_assigned(v0) && handle_assigned(v1) && handle_assigned(v2));
+      CGAL_assertion(v0 != v1 && v1 != v2 && v2 != v0);
     }
 
-    vertex_descriptor v0 ;
-    vertex_descriptor v1 ;
-    vertex_descriptor v2 ;
-  } ;
-
-  typedef std::vector<vertex_descriptor> vertex_descriptor_vector ;
-  typedef std::vector<halfedge_descriptor>   halfedge_descriptor_vector ;
+    vertex_descriptor v0;
+    vertex_descriptor v1;
+    vertex_descriptor v2;
+  };
 
-  typedef std::vector<Triangle> Triangle_vector ;
+  typedef std::vector<Triangle>                                               Triangle_vector;
 
 public :
-
-  template<class VertexIdxMap
-          ,class EdgeIdxMap
-          >
-  Edge_profile ( halfedge_descriptor  const& aV0V1
-               , TM&                    aSurface
-               , VertexIdxMap     const& aVertex_index_map
-               , VertexPointMap   const& aVertex_point_map
-               , EdgeIdxMap       const& aEdge_index_map
-               , bool has_border
-               ) ;
+  template<class VertexIndexMap, class HalfedgeIndexMap>
+  Edge_profile(const halfedge_descriptor h_v0v1,
+               TM& tmesh,
+               const Geom_traits& traits,
+               const VertexIndexMap& vim,
+               const VertexPointMap& vpm,
+               const HalfedgeIndexMap& him,
+               bool has_border);
 
 public :
+  const halfedge_descriptor v0_v1() const { return m_h_v0v1; }
+  const halfedge_descriptor v1_v0() const { return m_h_v1v0; }
 
-  halfedge_descriptor const& v0_v1() const { return mV0V1; }
-  halfedge_descriptor const& v1_v0() const { return mV1V0; }
-
-  vertex_descriptor const& v0() const { return mV0; }
-  vertex_descriptor const& v1() const { return mV1; }
+  const vertex_descriptor v0() const { return m_v0; }
+  const vertex_descriptor v1() const { return m_v1; }
 
   // These are null if v0v1 is a border (thius there is no face to its left)
-  vertex_descriptor const&  vL() const { return mVL; }
-  halfedge_descriptor const& v1_vL() const { return mV1VL; }
-  halfedge_descriptor const& vL_v0() const { return mVLV0; }
+  const vertex_descriptor vL() const { return m_vL; }
+  const halfedge_descriptor v1_vL() const { return m_v1vL; }
+  const halfedge_descriptor vL_v0() const { return m_vLv0; }
 
   // These are null if v1v0 is a border (thius there is no face to its left)
-  vertex_descriptor const&  vR() const { return mVR; }
-  halfedge_descriptor const& v0_vR() const { return mV0VR; }
-  halfedge_descriptor const& vR_v1() const { return mVRV1; }
+  const vertex_descriptor vR() const { return m_vR; }
+  const halfedge_descriptor v0_vR() const { return m_v0vR; }
+  const halfedge_descriptor vR_v1() const { return m_vRv1; }
 
-  Triangle_vector const& triangles() const {
+  const Triangle_vector& triangles() const
+  {
+    if(m_triangles.empty())
+      const_cast<Edge_profile*>(this)->extract_triangles_and_link();
 
-    if(mTriangles.empty()){
-      const_cast<Edge_profile*>(this)->Extract_triangles_and_link();
-    }
-    CGAL_HISTOGRAM_PROFILER("triangles.size()", mTriangles.size());
-    return mTriangles ; }
+    CGAL_HISTOGRAM_PROFILER("triangles.size()", m_triangles.size());
+    return m_triangles;
+  }
 
   // The cycle of vertices around the edge
-  vertex_descriptor_vector const& link() const {
+  const vertex_descriptor_vector& link() const
+  {
     CGAL_PROFILER("link calls");
-    if(mLink.empty()){
+    if(m_link_vertices.empty())
+      const_cast<Edge_profile*>(this)->extract_triangles_and_link();
 
-      const_cast<Edge_profile*>(this)->Extract_triangles_and_link();
-    }
-    return mLink ; }
-
-  halfedge_descriptor_vector const& border_edges() const {
-    return mBorderEdges ;
+    return m_link_vertices;
   }
-  TM& surface() const { return *mSurface ; }
-  TM& surface_mesh() const { return *mSurface ; }
 
-  VertexPointMap vertex_point_map() const { return mvpm ; }
+  const halfedge_descriptor_vector& border_edges() const { return m_border_edges; }
+  const TM& surface() const { return m_tm; }
+  const TM& surface_mesh() const { return m_tm; }
+  const VertexPointMap& vertex_point_map() const { return m_vpm; }
+  const Geom_traits& geom_traits() const { return m_traits; }
 
 public :
+  const Point& p0() const { return m_p0; }
+  const Point& p1() const { return m_p1; }
 
-  Point const& p0() const { return mP0; }
-  Point const& p1() const { return mP1; }
+  bool is_v0_v1_a_border() const { return m_is_v0v1_border; }
+  bool is_v1_v0_a_border() const { return m_is_v1v0_border; }
 
-  bool is_v0_v1_a_border() const { return mIsBorderV0V1 ; }
-  bool is_v1_v0_a_border() const { return mIsBorderV1V0 ; }
+  bool left_face_exists() const { return !m_is_v0v1_border; }
+  bool right_face_exists() const { return !m_is_v1v0_border; }
 
-  bool left_face_exists () const { return !mIsBorderV0V1 ; }
-  bool right_face_exists() const { return !mIsBorderV1V0 ; }
+private:
+  void extract_borders();
+  void extract_triangles_and_link();
 
 private:
+  const halfedge_descriptor m_h_v0v1;
+  const halfedge_descriptor m_h_v1v0;
+
+  bool m_is_v0v1_border;
+  bool m_is_v1v0_border;
+
+  const vertex_descriptor m_v0;
+  const vertex_descriptor m_v1;
+
+  const Point_reference m_p0;
+  const Point_reference m_p1;
+
+  vertex_descriptor m_vL;
+  vertex_descriptor m_vR;
+
+  halfedge_descriptor m_v1vL;
+  halfedge_descriptor m_vLv0;
+  halfedge_descriptor m_v0vR;
+  halfedge_descriptor m_vRv1;
+
+  vertex_descriptor_vector m_link_vertices;
+  halfedge_descriptor_vector m_border_edges;
+  Triangle_vector m_triangles;
+
+  const TM& m_tm;
+  const VertexPointMap& m_vpm;
+  const Geom_traits& m_traits;
+};
+
+template<class TM, class VertexPointMap, class GeomTraits>
+template<class VertexIndexMap, class HalfedgeIndexMap>
+Edge_profile<TM, VertexPointMap, GeomTraits>::
+Edge_profile(const halfedge_descriptor h_v0v1,
+             TM& tmesh,
+             const Geom_traits& traits,
+             const VertexIndexMap& /*vim*/,
+             const VertexPointMap& vpm,
+             const HalfedgeIndexMap& /*him*/,
+             bool has_border)
+  :
+    m_h_v0v1(h_v0v1),
+    m_h_v1v0(opposite(h_v0v1, tmesh)),
+    m_is_v0v1_border(is_border(h_v0v1, tmesh)),
+    m_is_v1v0_border(is_border(m_h_v1v0, tmesh)),
+    m_v0(source(h_v0v1, tmesh)),
+    m_v1(target(h_v0v1, tmesh)),
+    m_p0(get(vpm, m_v0)),
+    m_p1(get(vpm, m_v1)),
+    m_tm(tmesh),
+    m_vpm(vpm),
+    m_traits(traits)
+{
+  CGAL_PROFILER("Edge_profile constructor calls");
+
+  CGAL_precondition(m_v0 != m_v1);
+
+  m_link_vertices.reserve(12);
+  m_triangles.reserve(16);
+
+  if(left_face_exists())
+  {
+    CGAL_assertion(! is_border(m_h_v0v1, surface_mesh()));
+
+    m_vLv0 = prev(v0_v1(), surface_mesh());
+    m_v1vL = next(v0_v1(), surface_mesh());
+    m_vL = target(v1_vL(), surface_mesh());
 
-  //  typedef typename GraphTraits::in_edge_iterator  in_edge_iterator ;
+    CGAL_assertion(m_v0 != m_vL);
+    CGAL_assertion(m_vL == source(vL_v0(), surface_mesh()));
+  }
+  else
+  {
+    CGAL_assertion(is_border(m_h_v0v1, surface_mesh()));
+  }
 
-  bool is_border(halfedge_descriptor e) const
+  if(right_face_exists())
+  {
+    CGAL_assertion(! is_border(m_h_v1v0, surface_mesh()));
+
+    m_v0vR = next(v1_v0(), surface_mesh());
+    m_vRv1 = prev(v1_v0(), surface_mesh());
+    m_vR = target(v0_vR(), surface_mesh());
+
+    CGAL_assertion(m_v0 != m_vR);
+    CGAL_assertion(m_vR == source(vR_v1(), surface_mesh()));
+  }
+  else
   {
-    return face(e,*mSurface) == boost::graph_traits<TM>::null_face();
+    CGAL_assertion(is_border(m_h_v1v0, surface_mesh()));
   }
 
+  if(has_border)
+    extract_borders();
+}
+
+template<class TM, class VertexPointMap, class GeomTraits>
+void
+Edge_profile<TM, VertexPointMap, GeomTraits>::
+extract_borders()
+{
+  halfedge_descriptor e = m_h_v0v1;
+  halfedge_descriptor oe = opposite(e, surface_mesh());
 
-  void Extract_borders() ;
+  bool b;
+  if((b = is_border(e, surface_mesh())) || is_border(oe, surface_mesh()))
+    m_border_edges.push_back(b?e:oe);
+
+  e = next(oe, surface_mesh());
+  oe = opposite(e, surface_mesh());
+  while(e != m_h_v0v1)
+  {
+    if((b = is_border(e, surface_mesh())) || is_border(oe, surface_mesh()))
+      m_border_edges.push_back(b?e:oe);
 
-  void Extract_triangles_and_link() ;
+    e = next(oe, surface_mesh());
+    oe = opposite(e, surface_mesh());
+  }
 
-private:
+  e = opposite(next(e, surface_mesh()), surface_mesh());
+  oe = opposite(e, surface_mesh());
+  while(e != m_h_v0v1)
+  {
+    if((b = is_border(e, surface_mesh())) || is_border(oe, surface_mesh()))
+      m_border_edges.push_back(b?e:oe);
 
-  halfedge_descriptor mV0V1;
-  halfedge_descriptor mV1V0;
+    e = opposite(next(e, surface_mesh()), surface_mesh());
+    oe = opposite(e, surface_mesh());
+  }
+}
 
-  bool mIsBorderV0V1 ;
-  bool mIsBorderV1V0 ;
+// Extract all triangles (its normals) and vertices (the link) around the collapsing edge p_q
+template<class TM, class VertexPointMap, class GeomTraits>
+void
+Edge_profile<TM, VertexPointMap, GeomTraits>::
+extract_triangles_and_link()
+{
+#ifdef CGAL_SMS_EDGE_PROFILE_ALWAYS_NEED_UNIQUE_VERTEX_IN_LINK
+  std::set<vertex_descriptor> vertex_already_inserted;
+#endif
+  // look at the two faces or holes adjacent to edge (v0,v1)
+  // and at the opposite vertex if it exists
+  halfedge_descriptor endleft = next(v1_v0(), surface_mesh());
+  halfedge_descriptor endright = next(v0_v1(), surface_mesh());
+
+  if(left_face_exists())
+    m_triangles.push_back(Triangle(v0(), v1(), vL()));
+  if(right_face_exists())
+    m_triangles.push_back(Triangle(v1(), v0(), vR()));
+
+  // counterclockwise around v0
+  halfedge_descriptor e02 = opposite(prev(v0_v1(), surface_mesh()), surface_mesh());
+  vertex_descriptor v = target(e02, surface_mesh()), v2 = v;
 
-  vertex_descriptor mV0;
-  vertex_descriptor mV1;
+  while(e02 != endleft)
+  {
+#ifdef CGAL_SMS_EDGE_PROFILE_ALWAYS_NEED_UNIQUE_VERTEX_IN_LINK
+    if(vertex_already_inserted.insert(v2).second)
+#endif
+      m_link_vertices.push_back(v2);
+
+    bool is_b = is_border(e02, surface_mesh());
+    e02 = opposite(prev(e02, surface_mesh()), surface_mesh());
+    v = target(e02, surface_mesh());
+    if(!is_b)
+      m_triangles.push_back(Triangle(v, v0(), v2));
 
-  Point mP0 ;
-  Point mP1 ;
+    v2 = v;
+  }
 
-  vertex_descriptor mVL;
-  vertex_descriptor mVR;
+  e02 = opposite(prev(v1_v0(), surface_mesh()), surface_mesh());
+  if(target(e02, surface_mesh()) != v) // add the vertex if it is not added in the following loop
+  {
+#ifdef CGAL_SMS_EDGE_PROFILE_ALWAYS_NEED_UNIQUE_VERTEX_IN_LINK
+    if(vertex_already_inserted.insert(v).second)
+#endif
+      m_link_vertices.push_back(v);
+  }
 
-  halfedge_descriptor mV1VL;
-  halfedge_descriptor mVLV0;
-  halfedge_descriptor mV0VR;
-  halfedge_descriptor mVRV1;
+  // counterclockwise around v1
+  v2 = target(e02, surface_mesh());
+  v = v2;
+  while(e02 != endright)
+  {
+#ifdef CGAL_SMS_EDGE_PROFILE_ALWAYS_NEED_UNIQUE_VERTEX_IN_LINK
+    if(vertex_already_inserted.insert(v2).second)
+#endif
+      m_link_vertices.push_back(v2);
+
+    bool is_b = is_border(e02, surface_mesh());
+    e02 = opposite(prev(e02, surface_mesh()), surface_mesh());
+    v = target(e02, surface_mesh());
 
-  vertex_descriptor_vector mLink ;
-  halfedge_descriptor_vector   mBorderEdges ;
-  Triangle_vector          mTriangles ;
+    if(!is_b)
+      m_triangles.push_back(Triangle(v,v1(),v2));
 
-  TM* mSurface ;
-  VertexPointMap mvpm;
-} ;
+    v2 = v;
+  }
 
-} // namespace Surface_mesh_simplification
+  if(m_link_vertices.empty() || // handle link of an isolated triangle
+     target(opposite(prev(v0_v1(), surface_mesh()), surface_mesh()), surface_mesh())!=v)
+  {
+#ifdef CGAL_SMS_EDGE_PROFILE_ALWAYS_NEED_UNIQUE_VERTEX_IN_LINK
+    if(vertex_already_inserted.insert(v).second)
+#endif
+      m_link_vertices.push_back(v);
+  }
 
-} //namespace CGAL
+  CGAL_postcondition(!m_link_vertices.empty());
+}
 
-#include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_profile_impl.h>
+} // namespace Surface_mesh_simplification
+} // namespace CGAL
 
 #endif // CGAL_SURFACE_MESH_SIMPLIFICATION_DETAIL_EDGE_PROFILE_H
-// EOF //
-
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_profile_impl.h 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_profile_impl.h
--- 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_profile_impl.h	2020-08-01 19:02:56.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_profile_impl.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,205 +0,0 @@
-// Copyright (c) 2006  GeometryFactory (France). All rights reserved.
-//
-// This file is part of CGAL (www.cgal.org).
-//
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_profile_impl.h $
-// $Id: Edge_profile_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
-// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
-//
-// Author(s)     : Fernando Cacciola <fernando.cacciola@geometryfactory.com>
-//
-#ifndef CGAL_SURFACE_MESH_SIMPLIFICATION_DETAIL_EDGE_PROFILE_IMPL_H
-#define CGAL_SURFACE_MESH_SIMPLIFICATION_DETAIL_EDGE_PROFILE_IMPL_H 1
-
-#include <CGAL/license/Surface_mesh_simplification.h>
-
-
-namespace CGAL {
-
-namespace Surface_mesh_simplification
-{
-
-  template<class TM, class VertexPointMap>
-
-template<class VertexIdxMap
-        ,class EdgeIdxMap
-        >
-  Edge_profile<TM,VertexPointMap>::Edge_profile ( halfedge_descriptor  const& aV0V1
-                                , TM&                    aSurface
-                                , VertexIdxMap     const&
-                                , VertexPointMap const& aVertex_point_map
-                                , EdgeIdxMap       const&
-                                , bool has_border
-
-                                )
-  :
-   mV0V1(aV0V1)
-  ,mSurface(boost::addressof(aSurface))
-    , mvpm(aVertex_point_map)
-
-{
-  CGAL_PROFILER("Edge_profile constructor calls");
-
-  mLink.reserve(12);
-  mTriangles.reserve(16);
-  mV1V0 = opposite(v0_v1(),surface_mesh());
-
-  mV0 = source(v0_v1(),surface_mesh());
-  mV1 = target(v0_v1(),surface_mesh());
-
-  CGAL_assertion( mV0 != mV1 );
-
-  mP0 = get(vertex_point_map(),mV0);
-  mP1 = get(vertex_point_map(),mV1);
-
-  mIsBorderV0V1 = is_border(v0_v1());
-  mIsBorderV1V0 = is_border(v1_v0());
-
-  if ( left_face_exists() )
-  {
-    CGAL_SURF_SIMPL_TEST_assertion( ! is_border(mV0V1) ) ;
-
-    mVLV0 = prev(v0_v1(),surface_mesh());
-    mV1VL = next(v0_v1(),surface_mesh());
-    mVL   = target(v1_vL(),surface_mesh());
-
-    CGAL_SURF_SIMPL_TEST_assertion( mV0 != mVL );
-    CGAL_SURF_SIMPL_TEST_assertion( mVL == source(vL_v0(),surface_mesh()) );
-  }
-  else
-  {
-    CGAL_SURF_SIMPL_TEST_assertion( is_border(mV0V1) ) ;
-  }
-
-  if ( right_face_exists() )
-  {
-    CGAL_SURF_SIMPL_TEST_assertion( ! is_border(mV1V0) ) ;
-
-    mV0VR = next(v1_v0(),surface_mesh());
-    mVRV1 = prev(v1_v0(),surface_mesh());
-    mVR   = target(v0_vR(),surface_mesh());
-
-    CGAL_SURF_SIMPL_TEST_assertion( mV0 != mVR );
-    CGAL_SURF_SIMPL_TEST_assertion( mVR == source(vR_v1(),surface_mesh()) );
-  }
-  else
-  {
-    CGAL_SURF_SIMPL_TEST_assertion( is_border(mV1V0) ) ;
-  }
-
-  if(has_border){
-    Extract_borders();
-  }
-}
-
-
-  template<class TM, class VertexPointMap>
-  void Edge_profile<TM,VertexPointMap>::Extract_borders()
-{
-  halfedge_descriptor e = mV0V1;
-  halfedge_descriptor oe = opposite(e, surface_mesh());
-  bool b;
-  if((b = is_border(e)) || is_border(oe)){
-    mBorderEdges.push_back(b?e:oe);
-  }
-  e = next(oe,surface_mesh());
-  oe = opposite(e,surface_mesh());
-  while(e != mV0V1){
-    if((b = is_border(e)) || is_border(oe)){
-      mBorderEdges.push_back(b?e:oe);
-    }
-    e = next(oe,surface_mesh());
-    oe = opposite(e,surface_mesh());
-  }
-  e = opposite(next(e,surface_mesh()),surface_mesh());
-  oe = opposite(e,surface_mesh());
-    while(e != mV0V1){
-    if((b = is_border(e)) || is_border(oe)){
-      mBorderEdges.push_back(b?e:oe);
-    }
-    e = opposite(next(e,surface_mesh()),surface_mesh());
-    oe = opposite(e,surface_mesh());
-    }
-}
-
-
-
-// Extract all triangles (its normals) and vertices (the link) around the collapsing edge p_q
-//
-  template<class TM, class VertexPointMap>
-  void Edge_profile<TM,VertexPointMap>::Extract_triangles_and_link()
-{
-  #ifdef CGAL_SMS_EDGE_PROFILE_ALWAYS_NEED_UNIQUE_VERTEX_IN_LINK
-  std::set<vertex_descriptor> vertex_already_inserted;
-  #endif
-  // look at the two faces or holes adjacent to edge (v0,v1)
-  // and at the opposite vertex if it exists
-  halfedge_descriptor endleft = next(v1_v0(), surface_mesh());
-  halfedge_descriptor endright = next(v0_v1(), surface_mesh());
-
-  if( left_face_exists() )
-    mTriangles.push_back(Triangle(v0(),v1(),vL()) ) ;
-  if( right_face_exists() )
-    mTriangles.push_back(Triangle(v1(),v0(),vR()) ) ;
-
-  // counterclockwise around v0
-  halfedge_descriptor e02 = opposite(prev(v0_v1(),surface_mesh()), surface_mesh());
-  vertex_descriptor v=target(e02,surface_mesh()), v2=v;
-  while(e02 != endleft) {
-    #ifdef CGAL_SMS_EDGE_PROFILE_ALWAYS_NEED_UNIQUE_VERTEX_IN_LINK
-    if (vertex_already_inserted.insert(v2).second)
-    #endif
-    mLink.push_back(v2);
-    bool is_b = is_border(e02);
-    e02 = opposite(prev(e02,surface_mesh()), surface_mesh());
-    v = target(e02,surface_mesh());
-    if( !is_b )
-      mTriangles.push_back(Triangle(v,v0(),v2) ) ;
-    v2 = v;
-  }
-
-  e02 = opposite(prev(v1_v0(),surface_mesh()), surface_mesh());
-  if(target(e02, surface_mesh())!=v ) // add the vertex if it is not added in the following loop
-  {
-    #ifdef CGAL_SMS_EDGE_PROFILE_ALWAYS_NEED_UNIQUE_VERTEX_IN_LINK
-    if (vertex_already_inserted.insert(v).second)
-    #endif
-    mLink.push_back(v);
-  }
-
-  // counterclockwise around v1
-  v2 = target(e02,surface_mesh());
-  v = v2;
-  while(e02 != endright) {
-    #ifdef CGAL_SMS_EDGE_PROFILE_ALWAYS_NEED_UNIQUE_VERTEX_IN_LINK
-    if (vertex_already_inserted.insert(v2).second)
-    #endif
-    mLink.push_back(v2);
-    bool is_b = is_border(e02);
-    e02 = opposite(prev(e02,surface_mesh()), surface_mesh());
-    v = target(e02,surface_mesh());
-    if( !is_b ){
-      mTriangles.push_back(Triangle(v,v1(),v2) ) ;
-    }
-    v2 = v;
-  }
-
-  if(mLink.empty() || //handle link of an isolated triangle
-     target(opposite(prev(v0_v1(),surface_mesh()), surface_mesh()), surface_mesh())!=v)
-  {
-    #ifdef CGAL_SMS_EDGE_PROFILE_ALWAYS_NEED_UNIQUE_VERTEX_IN_LINK
-    if (vertex_already_inserted.insert(v).second)
-    #endif
-    mLink.push_back(v);
-  }
-
-  CGAL_assertion(!mLink.empty());
-}
-
-} // namespace Surface_mesh_simplification
-
-} //namespace CGAL
-
-#endif // CGAL_SURFACE_MESH_SIMPLIFICATION_DETAIL_EDGE_PROFILE_IMPL_H
-// EOF //
-
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/GarlandHeckbert_policies.h 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/GarlandHeckbert_policies.h
--- 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/GarlandHeckbert_policies.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/GarlandHeckbert_policies.h	2020-09-08 12:15:45.000000000 +0000
@@ -0,0 +1,191 @@
+// Copyright (c) 2019  GeometryFactory (France). All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/GarlandHeckbert_policies.h $
+// $Id: GarlandHeckbert_policies.h 9a7c98f 2019-11-29T11:42:18+01:00 Mael Rouxel-Labbé
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Baskin Burak Senbaslar,
+//                 Mael Rouxel-Labbé
+//
+#ifndef CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_GARLANDHECKBERT_POLICIES_H
+#define CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_GARLANDHECKBERT_POLICIES_H
+
+#include <CGAL/license/Surface_mesh_simplification.h>
+
+#include <CGAL/Surface_mesh_simplification/internal/Common.h>
+#include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/internal/GarlandHeckbert_core.h>
+
+#include <CGAL/tags.h>
+
+#include <Eigen/Dense>
+
+#include <boost/optional/optional.hpp>
+
+#include <utility>
+
+namespace CGAL {
+namespace Surface_mesh_simplification {
+namespace internal {
+
+template <typename VCM_, typename FT_>
+class GarlandHeckbert_cost
+{
+public:
+  typedef VCM_                                                     Vertex_cost_map;
+  typedef FT_                                                      FT;
+
+  // Tells the main function of `Edge_collapse` that these policies must call "initialize"
+  // and "update" functions. A bit awkward, but still better than abusing visitors.
+  typedef CGAL::Tag_true                                           Update_tag;
+
+  GarlandHeckbert_cost() { }
+  GarlandHeckbert_cost(Vertex_cost_map vcm,
+                       const FT discontinuity_multiplier = FT(100)) // abusing FT(double)
+    :
+      m_cost_matrices(vcm),
+      m_discontinuity_multiplier(discontinuity_multiplier)
+  { }
+
+  template <typename TM_, typename VPM_, typename GT_>
+  void initialize(const TM_& tmesh, const VPM_& vpm, const GT_& gt) const
+  {
+    typedef internal::GarlandHeckbert_core<TM_, VPM_, GT_>                                 GH_core;
+
+    GH_core::fundamental_error_quadrics(m_cost_matrices, tmesh, vpm, gt, m_discontinuity_multiplier);
+  }
+
+  template <typename Profile>
+  boost::optional<typename Profile::FT>
+  operator()(const Profile& profile,
+             const boost::optional<typename Profile::Point>& placement) const
+  {
+    typedef typename Profile::Triangle_mesh                                                Triangle_mesh;
+    typedef typename Profile::Vertex_point_map                                             Vertex_point_map;
+    typedef typename Profile::Geom_traits                                                  Geom_traits;
+    typedef internal::GarlandHeckbert_core<Triangle_mesh, Vertex_point_map, Geom_traits>   GH_core;
+
+    typedef typename GH_core::Matrix4x4                                                    Matrix4x4;
+    typedef typename GH_core::Col4                                                         Col4;
+    typedef boost::optional<typename Profile::FT>                                          Optional_FT;
+
+    if(!placement)
+      return boost::optional<typename Profile::FT>();
+
+    CGAL_precondition(get(m_cost_matrices, profile.v0()) != Matrix4x4());
+    CGAL_precondition(get(m_cost_matrices, profile.v1()) != Matrix4x4());
+
+    const Matrix4x4 combined_matrix = GH_core::combine_matrices(get(m_cost_matrices, profile.v0()),
+                                                                get(m_cost_matrices, profile.v1()));
+    const Col4 pt = GH_core::point_to_homogenous_column(*placement);
+    const Optional_FT cost = (pt.transpose() * combined_matrix * pt)(0, 0);
+
+    return cost;
+  }
+
+  template <typename Profile, typename vertex_descriptor>
+  void update_after_collapse(const Profile& profile,
+                             const vertex_descriptor new_v) const
+  {
+    typedef typename Profile::Triangle_mesh                                                Triangle_mesh;
+    typedef typename Profile::Vertex_point_map                                             Vertex_point_map;
+    typedef typename Profile::Geom_traits                                                  Geom_traits;
+    typedef internal::GarlandHeckbert_core<Triangle_mesh, Vertex_point_map, Geom_traits>   GH_core;
+
+    put(m_cost_matrices, new_v,
+        GH_core::combine_matrices(get(m_cost_matrices, profile.v0()),
+                                  get(m_cost_matrices, profile.v1())));
+  }
+
+private:
+  Vertex_cost_map m_cost_matrices;
+  FT m_discontinuity_multiplier;
+};
+
+template<typename VCM_>
+class GarlandHeckbert_placement
+{
+public:
+  typedef VCM_                                                                             Vertex_cost_map;
+
+  GarlandHeckbert_placement() { }
+  GarlandHeckbert_placement(Vertex_cost_map cost_matrices)
+    : m_cost_matrices(cost_matrices)
+  { }
+
+  template <typename Profile>
+  boost::optional<typename Profile::Point>
+  operator()(const Profile& profile) const
+  {
+    typedef typename Profile::Triangle_mesh                                                Triangle_mesh;
+    typedef typename Profile::Vertex_point_map                                             Vertex_point_map;
+    typedef typename Profile::Geom_traits                                                  Geom_traits;
+    typedef internal::GarlandHeckbert_core<Triangle_mesh, Vertex_point_map, Geom_traits>   GH_core;
+
+    typedef typename GH_core::Matrix4x4                                                    Matrix4x4;
+    typedef typename GH_core::Col4                                                         Col4;
+
+    CGAL_precondition(get(m_cost_matrices, profile.v0()) != Matrix4x4());
+    CGAL_precondition(get(m_cost_matrices, profile.v1()) != Matrix4x4());
+
+    // the combined matrix has already been computed in the evaluation of the cost...
+    const Matrix4x4 combinedMatrix = GH_core::combine_matrices(
+                                       get(m_cost_matrices, profile.v0()),
+                                       get(m_cost_matrices, profile.v1()));
+
+    const Col4 p0 = GH_core::point_to_homogenous_column(profile.p0());
+    const Col4 p1 = GH_core::point_to_homogenous_column(profile.p1());
+    const Col4 opt = GH_core::optimal_point(combinedMatrix, p0, p1);
+
+    boost::optional<typename Profile::Point> pt = typename Profile::Point(opt(0) / opt(3),
+                                                                          opt(1) / opt(3),
+                                                                          opt(2) / opt(3));
+
+    return pt;
+  }
+
+private:
+  Vertex_cost_map m_cost_matrices;
+};
+
+} // namespace internal
+
+template <typename TriangleMesh, typename GeomTraits>
+class GarlandHeckbert_policies
+{
+public:
+  typedef TriangleMesh                                                          Triangle_mesh;
+  typedef GeomTraits                                                            Geom_traits;
+  typedef typename Geom_traits::FT                                              FT;
+
+  typedef typename internal::GarlandHeckbert_matrix_type<GeomTraits>::type      Cost_matrix;
+  typedef CGAL::dynamic_vertex_property_t<Cost_matrix>                          Cost_property;
+  typedef typename boost::property_map<TriangleMesh, Cost_property>::type       Vertex_cost_map;
+
+  typedef internal::GarlandHeckbert_cost<Vertex_cost_map, FT>                   Get_cost;
+  typedef internal::GarlandHeckbert_placement<Vertex_cost_map>                  Get_placement;
+
+  GarlandHeckbert_policies(TriangleMesh& tmesh,
+                           const FT discontinuity_multiplier = FT(100))
+  {
+    vcm_ = get(Cost_property(), tmesh);
+    get_cost_ = Get_cost(vcm_, discontinuity_multiplier);
+    get_placement_ = Get_placement(vcm_);
+  }
+
+  Get_cost& get_cost() { return get_cost_; }
+  const Get_cost& get_cost() const { return get_cost_; }
+  Get_placement& get_placement() { return get_placement_; }
+  const Get_placement& get_placement() const { return get_placement_; }
+
+private:
+  Vertex_cost_map vcm_;
+  Get_cost get_cost_;
+  Get_placement get_placement_;
+};
+
+} // namespace Surface_mesh_simplification
+} // namespace CGAL
+
+#endif // CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_GARLANDHECKBERT_POLICIES_H
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/internal/GarlandHeckbert_core.h 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/internal/GarlandHeckbert_core.h
--- 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/internal/GarlandHeckbert_core.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/internal/GarlandHeckbert_core.h	2020-09-08 12:15:45.000000000 +0000
@@ -0,0 +1,307 @@
+// Copyright (c) 2019  GeometryFactory (France). All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/internal/GarlandHeckbert_core.h $
+// $Id: GarlandHeckbert_core.h 9a7c98f 2019-11-29T11:42:18+01:00 Mael Rouxel-Labbé
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Baskin Burak Senbaslar,
+//                 Mael Rouxel-Labbé
+//
+
+#ifndef CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_INTERNAL_GARLAND_HECKBERT_CORE_H
+#define CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_INTERNAL_GARLAND_HECKBERT_CORE_H
+
+#include <CGAL/license/Surface_mesh_simplification.h>
+
+#include <CGAL/Surface_mesh_simplification/internal/Common.h>
+#include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_profile.h>
+
+#include <Eigen/Dense>
+
+namespace CGAL {
+namespace Surface_mesh_simplification {
+namespace internal {
+
+template<typename GT_>
+struct GarlandHeckbert_matrix_type
+{
+  typedef typename GT_::FT                                                      FT;
+  typedef Eigen::Matrix<FT, 4, 4, Eigen::DontAlign>                             type;
+};
+
+template<class TM_, class VPM_, typename GT_>
+struct GarlandHeckbert_core
+{
+  typedef TM_                                                                   Triangle_mesh;
+  typedef boost::graph_traits<Triangle_mesh>                                    GraphTraits;
+  typedef typename GraphTraits::vertex_descriptor                               vertex_descriptor;
+  typedef typename GraphTraits::halfedge_descriptor                             halfedge_descriptor;
+  typedef typename GraphTraits::face_descriptor                                 face_descriptor;
+
+  typedef VPM_                                                                  Vertex_point_pmap;
+  typedef typename boost::property_traits<Vertex_point_pmap>::value_type        Point;
+  typedef typename boost::property_traits<Vertex_point_pmap>::reference         Point_reference;
+
+  typedef GT_                                                                   Geom_traits;
+  typedef typename Geom_traits::FT                                              FT;
+  typedef typename Geom_traits::Line_3                                          Line_3;
+  typedef typename Geom_traits::Plane_3                                         Plane_3;
+  typedef typename Geom_traits::Vector_3                                        Vector_3;
+
+  typedef typename GarlandHeckbert_matrix_type<GT_>::type                       Matrix4x4;
+  typedef Eigen::Matrix<FT, 1, 4>                                               Row4;
+  typedef Eigen::Matrix<FT, 4, 1>                                               Col4;
+
+  static Col4 point_to_homogenous_column(const Point& pt)
+  {
+    return Col4(pt.x(), pt.y(), pt.z(), FT(1));
+  }
+
+  /**
+  * Combines two Q matrices.
+  * It is simply the addition of two matrices
+  */
+  inline static Matrix4x4 combine_matrices(const Matrix4x4& aFirst, const Matrix4x4& aSecond)
+  {
+    return aFirst + aSecond;
+  }
+
+  /**
+  * Returns `true` if the target of h is a discontinuity vertex in tmesh
+  *
+  * Currently, only checks if vertex belongs to a border.
+  */
+  static bool is_discontinuity_vertex(const halfedge_descriptor h, const Triangle_mesh& tmesh)
+  {
+    if(is_border(target(h, tmesh), tmesh))
+      return true;
+
+    return false;
+  }
+
+  static bool is_discontinuity_edge(const halfedge_descriptor h, const Triangle_mesh& tmesh)
+  {
+    return is_border_edge(h, tmesh);
+  }
+
+  /*
+  * fundamental error quadric for the target vertex of h in tmesh
+  * Unused, but leaving it as it might still be useful somehow
+  */
+  static Matrix4x4 fundamental_error_quadric(const halfedge_descriptor h,
+                                             const Triangle_mesh& tmesh,
+                                             const Vertex_point_pmap& vpm,
+                                             const Geom_traits& gt,
+                                             const FT discontinuity_multiplier = FT(100))
+  {
+    Matrix4x4 quadric = Matrix4x4::Zero();
+
+    const vertex_descriptor target_vd = target(h, tmesh);
+    const Vector_3 target_vertex_vector = gt.construct_vector_3_object()(CGAL::ORIGIN, get(vpm, target_vd));
+
+    // const bool discontinuity_vertex = is_discontinuity_vertex(h, tmesh);
+
+    for(const halfedge_descriptor hd : halfedges_around_target(target_vd, tmesh))
+    {
+      const face_descriptor fd = face(hd, tmesh);
+      if(fd == GraphTraits::null_face())
+        continue;
+
+      Plane_3 plane(get(vpm, source(hd, tmesh)),
+                    get(vpm, target(hd, tmesh)),
+                    get(vpm, target(next(hd, tmesh), tmesh)));
+
+      Row4 plane_mtr;
+      const FT norm = CGAL::sqrt(CGAL::square(plane.a()) +
+                                 CGAL::square(plane.b()) +
+                                 CGAL::square(plane.c()));
+      const FT den = FT(1) / norm;
+
+      plane_mtr << den * plane.a(),
+                   den * plane.b(),
+                   den * plane.c(),
+                   den * plane.d();
+      quadric += plane_mtr.transpose() * plane_mtr;
+
+      if(is_discontinuity_edge(hd, tmesh))
+      {
+        const vertex_descriptor source_vd = source(hd, tmesh);
+
+        const Vector_3 p1p2 = gt.construct_vector_3_object()(get(vpm, source_vd), get(vpm, target_vd));
+        const Vector_3 normal = gt.construct_cross_product_vector_3_object()(
+                                  p1p2, gt.construct_orthogonal_vector_3_object()(plane));
+
+        const FT d = - normal * target_vertex_vector;
+        const FT norm = CGAL::sqrt(gt.compute_squared_length_3_object()(normal));
+        const FT den = FT(1) / norm;
+
+        plane_mtr << den * normal.x(),
+                     den * normal.y(),
+                     den * normal.z(),
+                     den * d;
+        quadric += discontinuity_multiplier * plane_mtr.transpose() * plane_mtr;
+      }
+
+      const halfedge_descriptor shd = next(hd, tmesh);
+      if(is_discontinuity_edge(shd, tmesh))
+      {
+        const Vector_3 p1p2 = gt.construct_vector_3_object()(get(vpm, target_vd), get(vpm, target(shd, tmesh)));
+        const Vector_3 normal = gt.construct_cross_product_vector_3_object()(
+                                  p1p2, gt.construct_orthogonal_vector_3_object()(plane));
+
+        const FT d = - normal * target_vertex_vector;
+        const FT norm = CGAL::sqrt(gt.compute_squared_length_3_object()(normal));
+        const FT den = FT(1) / norm;
+
+        plane_mtr << den * normal.x(),
+                     den * normal.y(),
+                     den * normal.z(),
+                     den * d;
+        quadric += discontinuity_multiplier * plane_mtr.transpose() * plane_mtr;
+      }
+    }
+
+    return quadric;
+  }
+
+  template <typename VCM>
+  static void fundamental_error_quadrics(VCM& vcm, // quadrics container
+                                         const Triangle_mesh& tmesh,
+                                         const Vertex_point_pmap& vpm,
+                                         const Geom_traits& gt,
+                                         const FT discontinuity_multiplier = FT(100))
+  {
+    Matrix4x4 nq = Matrix4x4::Zero();
+    for(vertex_descriptor v : vertices(tmesh))
+      put(vcm, v, nq);
+
+    for(face_descriptor f : faces(tmesh))
+    {
+      const halfedge_descriptor h = halfedge(f, tmesh);
+
+      const Point_reference p = get(vpm, source(h, tmesh));
+      const Point_reference q = get(vpm, target(h, tmesh));
+      const Point_reference r = get(vpm, target(next(h, tmesh), tmesh));
+
+      const FT rpx = p.x() - r.x();
+      const FT rpy = p.y() - r.y();
+      const FT rpz = p.z() - r.z();
+      const FT rqx = q.x() - r.x();
+      const FT rqy = q.y() - r.y();
+      const FT rqz = q.z() - r.z();
+
+      // Cross product rp * rq
+      const FT a = rpy*rqz - rqy*rpz;
+      const FT b = rpz*rqx - rqz*rpx;
+      const FT c = rpx*rqy - rqx*rpy;
+      const FT d = - a*r.x() - b*r.y() - c*r.z();
+
+      const Vector_3 plane_n = gt.construct_vector_3_object()(a, b, c);
+      const FT norm = CGAL::sqrt(CGAL::square(a) + CGAL::square(b) + CGAL::square(c));
+      const FT den = FT(1) / norm;
+
+      Row4 plane_mtr_r;
+      plane_mtr_r << den * a, den * b, den * c, den * d;
+      const Matrix4x4 plane_mtr = plane_mtr_r.transpose() * plane_mtr_r;
+
+      for(halfedge_descriptor shd : halfedges_around_face(h, tmesh))
+      {
+        const vertex_descriptor vs = source(shd, tmesh);
+        const vertex_descriptor vt = target(shd, tmesh);
+
+        put(vcm, vt, combine_matrices(get(vcm, vt), plane_mtr));
+
+        if(!is_discontinuity_edge(shd, tmesh))
+          continue;
+
+        const Vector_3 pspt = gt.construct_vector_3_object()(get(vpm, vs), get(vpm, vt));
+        const Vector_3 disc_plane_n = gt.construct_cross_product_vector_3_object()(pspt, plane_n);
+
+        // the plane contains the edge, so taking 'vs' or 'vt' will yield the same 'd'
+        const Vector_3 vvt = gt.construct_vector_3_object()(CGAL::ORIGIN, get(vpm, vt));
+        const FT disc_d = - gt.compute_scalar_product_3_object()(disc_plane_n, vvt);
+
+        const FT disc_norm = CGAL::sqrt(gt.compute_squared_length_3_object()(disc_plane_n));
+        const FT disc_den = FT(1) / disc_norm;
+
+        Row4 disc_mtr_r;
+        disc_mtr_r << disc_den * disc_plane_n.x(),
+                      disc_den * disc_plane_n.y(),
+                      disc_den * disc_plane_n.z(),
+                      disc_den * disc_d;
+
+        const Matrix4x4 disc_mtr = discontinuity_multiplier * disc_mtr_r.transpose() * disc_mtr_r;
+
+        put(vcm, vs, combine_matrices(get(vcm, vs), disc_mtr));
+        put(vcm, vt, combine_matrices(get(vcm, vt), disc_mtr));
+      }
+    }
+  }
+
+
+  /*
+  * Return the point p that minimizes p' Q p where p is free.
+  * p0, and p1 are the points that are being collapsed.
+  * aQuadric is the matrix that is the combination of matrices
+  * of p0 and p1.
+  */
+  static Col4 optimal_point(const Matrix4x4& aQuadric,
+                            const Col4& p0,
+                            const Col4& p1)
+  {
+    Matrix4x4 X;
+    X << aQuadric.block(0, 0, 3, 4), 0, 0, 0, 1;
+
+    Col4 opt_pt;
+
+    if(X.determinant() == 0)
+    {
+      // not invertible
+      const Col4 p1mp0 = std::move(p1 - p0);
+      const FT a = (p1mp0.transpose() * aQuadric * p1mp0)(0, 0);
+      const FT b = 2 * (p0.transpose() * aQuadric * p1mp0)(0, 0);
+
+      if(a == 0)
+      {
+        if(b < 0)
+          opt_pt = p1;
+        else if(b == 0)
+          opt_pt = 0.5 * (p0 + p1);
+        else
+          opt_pt = p0;
+      }
+      else
+      {
+        FT ext_t = -b/(2*a);
+        if(ext_t < 0 || ext_t > 1 || a < 0)
+        {
+          // one of endpoints
+          FT p0_cost = (p0.transpose() * aQuadric * p0)(0, 0);
+          FT p1_cost = (p1.transpose() * aQuadric * p1)(0, 0);
+          if(p0_cost > p1_cost)
+            opt_pt = p1;
+          else
+            opt_pt = p0;
+        }
+        else
+        {
+          // extremum of the parabola
+          opt_pt = p0 + ext_t * (p1 - p0);
+        }
+      }
+    }
+    else // invertible
+    {
+      opt_pt = X.inverse().col(3); // == X.inverse() * (0 0 0 1)
+    }
+    return opt_pt;
+  }
+};
+
+} // namespace internal
+} // namespace Surface_mesh_simplification
+} // namespace CGAL
+
+#endif // CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_INTERNAL_GARLAND_HECKBERT_CORE_H
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/internal/Lindstrom_Turk_core.h 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/internal/Lindstrom_Turk_core.h
--- 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/internal/Lindstrom_Turk_core.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/internal/Lindstrom_Turk_core.h	2020-09-08 12:15:45.000000000 +0000
@@ -0,0 +1,746 @@
+// Copyright (c) 2005, 2006 Fernando Luis Cacciola Carballal. All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/internal/Lindstrom_Turk_core.h $
+// $Id: Lindstrom_Turk_core.h 6f96731 2019-10-28T09:41:51+01:00 Mael Rouxel-Labbé
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Fernando Cacciola <fernando_cacciola@ciudad.com.ar>
+//
+#ifndef CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_DETAIL_LINDSTROM_TURK_CORE_H
+#define CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_DETAIL_LINDSTROM_TURK_CORE_H
+
+#include <CGAL/license/Surface_mesh_simplification.h>
+
+#include <CGAL/Surface_mesh_simplification/internal/Common.h>
+#include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/internal/LindstromTurk_params.h>
+#include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_profile.h>
+
+#include <CGAL/Cartesian/MatrixC33.h>
+
+#include <limits>
+#include <vector>
+
+// This should be in
+//
+// Implementation of the collapsing cost and placement strategy from:
+//
+//  "Fast and Memory Efficient Polygonal Symplification"
+//  Peter Lindstrom, Greg Turk
+
+namespace CGAL {
+namespace Surface_mesh_simplification {
+namespace internal {
+
+template<class TM_, class Profile_>
+class LindstromTurkCore
+{
+public:
+  typedef TM_                                                            TM;
+  typedef Profile_                                                       Profile;
+
+  typedef boost::graph_traits<TM>                                        GraphTraits;
+  typedef typename GraphTraits::vertex_descriptor                        vertex_descriptor;
+  typedef typename GraphTraits::halfedge_descriptor                      halfedge_descriptor;
+
+  typedef internal::LindstromTurk_params                                 LT_params;
+
+  typedef typename Profile::VertexPointMap                               Vertex_point_pmap;
+  typedef typename Profile::VertexPointMap                               Vertex_point_map;
+  typedef typename boost::property_traits<Vertex_point_pmap>::value_type Point;
+  typedef typename boost::property_traits<Vertex_point_pmap>::reference  Point_reference;
+
+  typedef typename Profile::Geom_traits                                  Geom_traits;
+  typedef typename Geom_traits::FT                                       FT;
+  typedef typename Geom_traits::Vector_3                                 Vector;
+
+  typedef boost::optional<FT>                                            Optional_FT;
+  typedef boost::optional<Point>                                         Optional_point;
+  typedef boost::optional<Vector>                                        Optional_vector;
+
+  typedef MatrixC33<Geom_traits>                                         Matrix;
+
+  typedef typename Profile::Triangle                                     Triangle;
+  typedef std::vector<vertex_descriptor>                                 vertex_descriptor_vector;
+  typedef std::vector<halfedge_descriptor>                               halfedge_descriptor_vector;
+
+  typedef typename Profile::Triangle_vector::const_iterator              const_triangle_iterator;
+  typedef typename halfedge_descriptor_vector::const_iterator            const_border_edge_iterator;
+
+public:
+  LindstromTurkCore(const LT_params& aParams, const Profile& aProfile);
+
+  Optional_point compute_placement();
+  Optional_FT compute_cost(const Optional_point& p);
+
+private :
+  struct Triangle_data
+  {
+    Triangle_data(const Vector& aNormalV, const FT& aNormalL)
+      : NormalV(aNormalV), NormalL(aNormalL)
+    {}
+
+    Vector NormalV;
+    FT NormalL;
+  };
+
+  struct Boundary_data
+  {
+    Boundary_data(const Point& s_, const Point& t_, const Vector& v_, const Vector& n_)
+      : s(s_), t(t_), v(v_), n(n_) {}
+
+    const Point s, t;
+    const Vector v, n;
+  };
+
+  typedef std::vector<Triangle_data>                                       Triangle_data_vector;
+  typedef std::vector<Boundary_data>                                       Boundary_data_vector;
+
+private :
+  void extract_triangle_data();
+  void extract_boundary_data();
+
+  void add_boundary_preservation_constraints(const Boundary_data_vector& aBdry);
+  void add_volume_preservation_constraints(const Triangle_data_vector& triangles);
+  void add_boundary_and_volume_optimization_constraints(const Boundary_data_vector& aBdry,
+                                                        const Triangle_data_vector& triangles);
+  void add_shape_optimization_constraints(const vertex_descriptor_vector& link);
+
+  FT compute_boundary_cost(const Vector& v, const Boundary_data_vector& aBdry);
+  FT compute_volume_cost(const Vector& v, const Triangle_data_vector& triangles);
+  FT compute_shape_cost(const Point& p, const vertex_descriptor_vector& link);
+
+  Point_reference get_point(const vertex_descriptor v) const
+  {
+    return get(mProfile.vertex_point_map(), v);
+  }
+
+  const Geom_traits& geom_traits() const { return mProfile.geom_traits(); }
+  const TM& surface() const { return mProfile.surface(); }
+
+  static Vector point_cross_product(const Point& a, const Point& b)
+  {
+    return cross_product(a-ORIGIN, b-ORIGIN);
+  }
+
+  // This is the (uX)(Xu) product described in the Lindstrom-Turk paper
+  static Matrix LT_product(const Vector& u)
+  {
+    FT a00 = (u.y()*u.y()) + (u.z()*u.z());
+    FT a01 = -u.x()*u.y();
+    FT a02 = -u.x()*u.z();
+
+    FT a10 = a01;
+    FT a11 = (u.x()*u.x()) + (u.z()*u.z());
+    FT a12 = - u.y() * u.z();
+
+    FT a20 = a02;
+    FT a21 = a12;
+    FT a22 = (u.x()*u.x()) + (u.y()*u.y());
+
+    return Matrix(a00, a01, a02,
+                  a10, a11, a12,
+                  a20, a21, a22);
+  }
+
+  static FT big_value() { return static_cast<FT>((std::numeric_limits<double>::max)()); }
+
+  static bool is_finite(const FT& n) { return CGAL_NTS is_finite(n); }
+  static bool is_finite(const Point& p) { return is_finite(p.x())  && is_finite(p.y())  && is_finite(p.z()); }
+  static bool is_finite(const Vector& v) { return is_finite(v.x())  && is_finite(v.y())  && is_finite(v.z()); }
+  static bool is_finite(const Matrix& m) { return is_finite(m.r0()) && is_finite(m.r1()) && is_finite(m.r2()); }
+
+  template<class T>
+  static boost::optional<T> filter_infinity(const T& n) { return is_finite(n) ? boost::optional<T>(n) : boost::optional<T>(); }
+
+
+private:
+  const LT_params& mParams;
+  const Profile& mProfile;
+
+  void add_constraint_if_alpha_compatible(const Vector& Ai, const FT& bi);
+  void add_constraint_from_gradient(const Matrix& H, const Vector& c);
+
+private:
+  Triangle_data_vector mTriangle_data;
+  Boundary_data_vector mBdry_data;
+
+  int mConstraints_n;
+  Matrix mConstraints_A;
+  Vector mConstraints_b;
+
+  FT mSquared_cos_alpha;
+  FT mSquared_sin_alpha;
+};
+
+template<class TM, class K>
+LindstromTurkCore<TM,K>::
+LindstromTurkCore(const LT_params& aParams, const Profile& aProfile)
+  :
+    mParams(aParams),
+    mProfile(aProfile),
+    mConstraints_n(0),
+    mConstraints_A(NULL_MATRIX),
+    mConstraints_b(NULL_VECTOR)
+{
+  double alpha = 1.0 * CGAL_PI / 180.0;
+
+  FT cos_alpha = std::cos(alpha);
+  FT sin_alpha = std::sin(alpha);
+
+  mSquared_cos_alpha = cos_alpha * cos_alpha;
+  mSquared_sin_alpha = sin_alpha * sin_alpha;
+
+  extract_triangle_data();
+  extract_boundary_data();
+}
+
+template<class TM, class K>
+void
+LindstromTurkCore<TM,K>::
+extract_boundary_data()
+{
+  mBdry_data.reserve(mProfile.border_edges().size());
+  for(halfedge_descriptor border_edge : mProfile.border_edges())
+  {
+    halfedge_descriptor face_edge = opposite(border_edge, surface());
+
+    vertex_descriptor sv = source(face_edge, surface());
+    vertex_descriptor tv = target(face_edge, surface());
+
+    const Point_reference sp = get_point(sv);
+    const Point_reference tp = get_point(tv);
+
+    Vector v = tp - sp;
+    Vector n = point_cross_product(tp,sp);
+
+    CGAL_SMS_LT_TRACE(1, "  Boundary edge. S:" << xyz_to_string(sp) << " T:" << xyz_to_string(tp)
+                          << " V:" << xyz_to_string(v) << " N:" << xyz_to_string(n));
+
+    mBdry_data.push_back(Boundary_data(sp,tp,v,n));
+  }
+}
+
+template<class TM, class K>
+void
+LindstromTurkCore<TM,K>::
+extract_triangle_data()
+{
+  mTriangle_data.reserve(mProfile.triangles().size());
+
+  for(const Triangle& tri : mProfile.triangles())
+  {
+    const Point_reference p0 = get_point(tri.v0);
+    const Point_reference p1 = get_point(tri.v1);
+    const Point_reference p2 = get_point(tri.v2);
+
+    Vector v01 = p1 - p0;
+    Vector v02 = p2 - p0;
+
+    Vector lNormalV = cross_product(v01,v02);
+    FT lNormalL = point_cross_product(p0,p1) * (p2 - ORIGIN);
+
+    CGAL_SMS_LT_TRACE(1, "  Extracting triangle v" << tri.v0 << "->v" << tri.v1 << "->v" << tri.v2
+                           << " N:" << xyz_to_string(lNormalV) << " L:" << n_to_string(lNormalL));
+
+    mTriangle_data.push_back(Triangle_data(lNormalV,lNormalL));
+  }
+}
+
+template<class TM, class K>
+typename LindstromTurkCore<TM,K>::Optional_point
+LindstromTurkCore<TM,K>::
+compute_placement()
+{
+  Optional_point rPlacementP;
+  Optional_vector lPlacementV;
+
+  CGAL_SMS_LT_TRACE(0, "Computing LT placement for E" << mProfile.v0_v1()
+                          << " (V" << mProfile.v0() << "->V" << mProfile.v1() << ")");
+
+  // Each vertex constraint is an equation of the form: Ai * v = bi
+  // Where 'v' is a vector representing the vertex,
+  // 'Ai' is a (row) vector and 'bi' a scalar.
+  //
+  // The vertex is completely determined with 3 such constraints,
+  // so is the solution to the folloing system:
+  //
+  //  A.r0(). * v = b0
+  //  A1 * v = b1
+  //  A2 * v = b2
+  //
+  // Which in matrix form is :  A * v = b
+  //
+  // (with 'A' a 3x3 matrix and 'b' a vector)
+  //
+  // The member variable mConstrinas contains A and b. Indidivual constraints (Ai,bi) can be added to it.
+  // Once 3 such constraints have been added 'v' is directly solved a:
+  //
+  //  v = b*inverse(A)
+  //
+  // A constraint (Ai,bi) must be alpha-compatible with the previously added constraints (see Paper); if it's not, is discarded.
+  //
+  if(mBdry_data.size() > 0)
+    add_boundary_preservation_constraints(mBdry_data);
+
+  if(mConstraints_n < 3)
+    add_volume_preservation_constraints(mTriangle_data);
+
+  if(mConstraints_n < 3)
+    add_boundary_and_volume_optimization_constraints(mBdry_data,mTriangle_data);
+
+  if(mConstraints_n < 3)
+    add_shape_optimization_constraints(mProfile.link());
+
+  // It might happen that there were not enough alpha-compatible constraints.
+  // In that case there is simply no good vertex placement
+  if(mConstraints_n == 3)
+  {
+    // If the matrix is singular it's inverse cannot be computed so an 'absent' value is returned.
+    boost::optional<Matrix> lOptional_Ai = inverse_matrix(mConstraints_A);
+    if(lOptional_Ai)
+    {
+      const Matrix& lAi = *lOptional_Ai;
+
+      CGAL_SMS_LT_TRACE(2,"       b: " << xyz_to_string(mConstraints_b));
+      CGAL_SMS_LT_TRACE(2,"  inv(A): " << matrix_to_string(lAi));
+
+      lPlacementV = filter_infinity(mConstraints_b * lAi);
+
+      CGAL_SMS_LT_TRACE(0,"  New vertex point: " << xyz_to_string(*lPlacementV));
+    }
+    else
+    {
+      CGAL_SMS_LT_TRACE(1,"  Can't solve optimization, singular system.");
+    }
+  }
+  else
+  {
+    CGAL_SMS_LT_TRACE(1,"  Can't solve optimization, not enough alpha-compatible constraints.");
+  }
+
+  if(lPlacementV)
+    rPlacementP = ORIGIN + (*lPlacementV);
+
+  return rPlacementP;
+}
+
+template<class TM, class K>
+typename LindstromTurkCore<TM,K>::Optional_FT
+LindstromTurkCore<TM,K>::
+compute_cost(const Optional_point& aP)
+{
+  Optional_FT rCost;
+
+  if(aP)
+  {
+    CGAL_SMS_LT_TRACE(0,"Computing LT cost for E" << mProfile.v0_v1());
+    Vector lV = (*aP) - ORIGIN;
+
+    FT lSquaredLength = squared_distance(mProfile.p0(), mProfile.p1());
+
+    FT lBdryCost = compute_boundary_cost(lV, mBdry_data);
+    FT lVolumeCost = compute_volume_cost(lV, mTriangle_data);
+    FT lShapeCost  = compute_shape_cost(*aP, mProfile.link());
+
+    FT lTotalCost = FT(mParams.m_volume_weight) * lVolumeCost
+                    + FT(mParams.m_boundary_weight) * lBdryCost * lSquaredLength
+                    + FT(mParams.m_shape_weight) * lShapeCost  * lSquaredLength * lSquaredLength;
+
+    rCost = filter_infinity(lTotalCost);
+
+    CGAL_SMS_LT_TRACE(0, "    Squared edge length: " << n_to_string(lSquaredLength)
+                           << "\n    Boundary cost: " << n_to_string(lBdryCost) << " weight: " << mParams.m_boundary_weight
+                           << "\n    Volume cost: " << n_to_string(lVolumeCost) << " weight: " << mParams.m_volume_weight
+                           << "\n    Shape cost: " << n_to_string(lShapeCost)   << " weight: " << mParams.m_shape_weight
+                           << "\n  TOTAL COST: " << n_to_string(lTotalCost));
+
+  }
+
+  return rCost;
+}
+
+template<class TM, class K>
+void
+LindstromTurkCore<TM,K>::
+add_boundary_preservation_constraints(const Boundary_data_vector& aBdry)
+{
+  if(aBdry.size() > 0)
+  {
+    Vector e1 = NULL_VECTOR;
+    Vector e2 = NULL_VECTOR;
+
+    FT e1x = FT(0),
+       e1y = FT(0),
+       e1z = FT(0);
+
+    for(typename Boundary_data_vector::const_iterator it = aBdry.begin(); it != aBdry.end(); ++it)
+    {
+      e1 = e1 + it->v;
+      e2 = e2 + it->n;
+
+      FT vx = it->v.x();
+      FT vy = it->v.y();
+      FT vz = it->v.z();
+
+      e1x = e1x + vx;
+      e1y = e1y + vy;
+      e1z = e1z + vz;
+
+      CGAL_SMS_LT_TRACE(1,"    vx:" << n_to_string(vx) << " vy:" << n_to_string(vy) << " vz:" << n_to_string(vz) << " e1x:"
+                        << n_to_string(e1x) << " e1y:" << n_to_string(e1y) << " e1z:" << n_to_string(e1z));
+    }
+
+    Matrix H = LT_product(e1);
+    Vector c = cross_product(e1,e2);
+
+    CGAL_SMS_LT_TRACE(1, "  Adding boundary preservation constraint."
+                           << "\n      SumV:" << xyz_to_string(e1)
+                           << "\n      SumN:" << xyz_to_string(e2)
+                           << "\n      H:" << matrix_to_string(H)
+                           << "\n      c:" << xyz_to_string(c));
+
+    add_constraint_from_gradient(H,c);
+  }
+}
+
+template<class TM, class K>
+void
+LindstromTurkCore<TM,K>::
+add_volume_preservation_constraints(const Triangle_data_vector& triangles)
+{
+  CGAL_SMS_LT_TRACE(1,"  Adding volume preservation constraint. " << triangles.size() << " triangles.");
+
+  Vector lSumV = NULL_VECTOR;
+  FT lSumL(0);
+
+  for(typename Triangle_data_vector::const_iterator it = triangles.begin(), eit = triangles.end(); it != eit; ++it)
+  {
+    lSumV = lSumV + it->NormalV;
+    lSumL = lSumL + it->NormalL;
+  }
+
+  CGAL_SMS_LT_TRACE(1, "      SumV:" << xyz_to_string(lSumV) << "\n      SumL:" << n_to_string(lSumL));
+
+  add_constraint_if_alpha_compatible(lSumV,lSumL);
+}
+
+template<class TM, class K>
+void
+LindstromTurkCore<TM,K>::
+add_boundary_and_volume_optimization_constraints(const Boundary_data_vector& aBdry,
+                                                 const Triangle_data_vector& triangles)
+{
+  CGAL_SMS_LT_TRACE(1,"  Adding boundary and volume optimization constraints. ");
+
+  Matrix H = NULL_MATRIX;
+  Vector c = NULL_VECTOR;
+
+  // Volume optimization
+  for(const Triangle_data& lTri : triangles)
+  {
+    H += direct_product(lTri.NormalV, lTri.NormalV);
+    c = c - (lTri.NormalL * lTri.NormalV);
+  }
+
+  CGAL_SMS_LT_TRACE(2,"      Hv:" << matrix_to_string(H) << "\n      cv:" << xyz_to_string(c));
+
+  if(aBdry.size() > 0)
+  {
+    // Boundary optimization
+    Matrix Hb = NULL_MATRIX;
+    Vector cb = NULL_VECTOR;
+
+    for(typename Boundary_data_vector::const_iterator it = aBdry.begin(); it != aBdry.end(); ++it)
+    {
+      Matrix H = LT_product(it->v);
+      Vector c = cross_product(it->v, it->n);
+
+      Hb += H;
+      cb = cb + c;
+    }
+
+    CGAL_SMS_LT_TRACE(2,"      Hb:" << matrix_to_string(Hb) << "\n      cb:" << xyz_to_string(cb));
+
+    // Weighted average
+    FT scaled_boundary_weight = FT(9) * FT(mParams.m_boundary_weight) * squared_distance(mProfile.p0(),mProfile.p1()) ;
+
+    H *= mParams.m_volume_weight;
+    c = c * mParams.m_volume_weight;
+
+    H += scaled_boundary_weight * Hb;
+    c = c + (scaled_boundary_weight * cb);
+
+    CGAL_SMS_LT_TRACE(2, "      H:" << matrix_to_string(H) << "\n      c:" << xyz_to_string(c));
+    CGAL_SMS_LT_TRACE(2, "      VolW:" << mParams.m_volume_weight << " BdryW:" << mParams.m_boundary_weight << " ScaledBdryW:" << scaled_boundary_weight);
+  }
+
+  add_constraint_from_gradient(H,c);
+}
+
+template<class TM, class K>
+void
+LindstromTurkCore<TM,K>::
+add_shape_optimization_constraints(const vertex_descriptor_vector& link)
+{
+  FT s(double(link.size()));
+
+  Matrix H(s, 0.0, 0.0,
+           0.0, s, 0.0,
+           0.0, 0.0, s);
+
+  Vector c = NULL_VECTOR;
+  for(const vertex_descriptor v : link)
+    c = c + (ORIGIN - get_point(v));
+
+  CGAL_SMS_LT_TRACE(1,"  Adding shape optimization constraint. Shape vector: " << xyz_to_string(c));
+
+  add_constraint_from_gradient(H,c);
+}
+
+template<class TM, class K>
+typename LindstromTurkCore<TM,K>::FT
+LindstromTurkCore<TM,K>::
+compute_boundary_cost(const Vector& v,
+                      const Boundary_data_vector& aBdry)
+{
+  FT rCost(0);
+  for(typename Boundary_data_vector::const_iterator it = aBdry.begin(); it != aBdry.end(); ++it)
+  {
+    Vector u = (it->t - ORIGIN) - v;
+    Vector c = cross_product(it->v, u);
+    rCost += c*c;
+  }
+
+  return rCost / FT(4);
+}
+
+template<class TM, class K>
+typename LindstromTurkCore<TM,K>::FT
+LindstromTurkCore<TM,K>::
+compute_volume_cost(const Vector& v,
+                    const Triangle_data_vector& triangles)
+{
+  FT rCost(0);
+
+  for(const Triangle_data& lTri : triangles)
+  {
+    FT lF = lTri.NormalV * v - lTri.NormalL;
+    rCost += (lF * lF);
+  }
+
+  return rCost / FT(36);
+}
+
+template<class TM, class K>
+typename LindstromTurkCore<TM,K>::FT
+LindstromTurkCore<TM,K>::
+compute_shape_cost(const Point& p,
+                   const vertex_descriptor_vector& link)
+{
+  FT rCost(0);
+  for(const vertex_descriptor v : link)
+    rCost += squared_distance(p, get_point(v));
+
+  return rCost;
+}
+
+template<class TM, class K>
+void
+LindstromTurkCore<TM,K>::
+add_constraint_if_alpha_compatible(const Vector& Ai,
+                                   const FT& bi)
+{
+  CGAL_SMS_LT_TRACE(3, "    Adding new constraints if alpha-compatible.\n      Ai: " << xyz_to_string(Ai) << "\n      bi:" << n_to_string(bi) << ")");
+
+  FT slai = Ai*Ai;
+  CGAL_SMS_LT_TRACE(3, "\n      slai: " << n_to_string(slai) << ")");
+
+  if(is_finite(slai))
+  {
+    FT l = CGAL_NTS sqrt(slai);
+    CGAL_SMS_LT_TRACE(3, "      l: " << n_to_string(l));
+
+    if(!CGAL_NTS is_zero(l))
+    {
+      Vector Ain = Ai / l;
+      FT bin = bi / l;
+      CGAL_SMS_LT_TRACE(3, "      Ain: " << xyz_to_string(Ain) << " bin:" << n_to_string(bin));
+
+      bool lAddIt = true;
+
+      if(mConstraints_n == 1)
+      {
+        FT d01 = mConstraints_A.r0() * Ai ;
+        FT sla0 = mConstraints_A.r0() * mConstraints_A.r0();
+        FT sd01 = d01 * d01;
+        FT max = sla0 * slai * mSquared_cos_alpha;
+
+        CGAL_SMS_LT_TRACE(3, "      Second constraint. d01: " << n_to_string(d01) << " sla0:" << n_to_string(sla0)
+                                                 << " sd01:" << n_to_string(sd01) << " max:" << n_to_string(max));
+
+        if(sd01 > max)
+          lAddIt = false;
+      }
+      else if(mConstraints_n == 2)
+      {
+        Vector N = cross_product(mConstraints_A.r0(),mConstraints_A.r1());
+
+        FT dc012 = N * Ai;
+        FT slc01 = N * N;
+        FT sdc012 = dc012 * dc012;
+
+        FT min = slc01 * slai * mSquared_sin_alpha;
+
+        CGAL_SMS_LT_TRACE(3, "      Third constraint. N: " << xyz_to_string(N)
+                               << " dc012:" << n_to_string(dc012) << " slc01:" << n_to_string(slc01)
+                               << " sdc012:" << n_to_string(sdc012) << " min:" << n_to_string(min));
+
+        if(sdc012 <= min)
+          lAddIt = false;
+      }
+
+      if(lAddIt)
+      {
+        switch(mConstraints_n)
+        {
+          case 0:
+            mConstraints_A.r0() = Ain;
+            mConstraints_b = Vector(bin,mConstraints_b.y(),mConstraints_b.z());
+            break;
+          case 1:
+            mConstraints_A.r1() = Ain;
+            mConstraints_b = Vector(mConstraints_b.x(),bin,mConstraints_b.z());
+            break;
+          case 2:
+            mConstraints_A.r2() = Ain;
+            mConstraints_b = Vector(mConstraints_b.x(),mConstraints_b.y(),bin);
+            break;
+        }
+
+        CGAL_SMS_LT_TRACE(3, "      Accepting # " << mConstraints_n << " A:" << matrix_to_string(mConstraints_A) << " b:" << xyz_to_string(mConstraints_b));
+
+        ++mConstraints_n;
+      }
+      else
+      {
+        CGAL_SMS_LT_TRACE(3, "      INCOMPATIBLE. Discarded");
+      }
+    }
+    else
+    {
+      CGAL_SMS_LT_TRACE(3, "        l is ZERO. Discarded");
+    }
+  }
+  else
+  {
+    CGAL_SMS_LT_TRACE(3, "      OVERFLOW. Discarded");
+  }
+}
+
+template<class K>
+int index_of_max_component(const typename K::Vector_3& v)
+{
+  typedef typename K::FT                     FT;
+
+  int i = 0;
+  FT max = v.x();
+
+  if(max < v.y())
+  {
+    max = v.y();
+    i = 1;
+  }
+
+  if(max < v.z())
+  {
+    max = v.z();
+    i = 2;
+  }
+
+  return i;
+}
+
+template<class TM, class K>
+void
+LindstromTurkCore<TM,K>::
+add_constraint_from_gradient(const Matrix& H,
+                             const Vector& c)
+{
+  CGAL_SMS_LT_TRACE(3, "    Adding constraint from gradient. Current n=" << mConstraints_n);
+
+  CGAL_precondition(mConstraints_n >= 0 && mConstraints_n <= 2);
+
+  switch(mConstraints_n)
+  {
+    case 0:
+      add_constraint_if_alpha_compatible(H.r0(), -c.x());
+      add_constraint_if_alpha_compatible(H.r1(), -c.y());
+      add_constraint_if_alpha_compatible(H.r2(), -c.z());
+      break;
+    case 1:
+    {
+      const Vector& A0 = mConstraints_A.r0();
+
+      CGAL_assertion(A0 != NULL_VECTOR);
+
+      Vector AbsA0(CGAL::abs(A0.x()),
+                   CGAL::abs(A0.y()),
+                   CGAL::abs(A0.z()));
+
+      Vector Q0;
+      switch(index_of_max_component<Geom_traits>(AbsA0))
+      {
+        // Since A0 is guaranteed to be non-zero, the denominators here are known to be non-zero too.
+
+        case 0:
+          Q0 = Vector(- A0.z()/A0.x(), 0, 1);
+          break;
+        case 1:
+          Q0 = Vector(0,- A0.z()/A0.y(), 1);
+          break;
+        case 2:
+          Q0 = Vector(1, 0, - A0.x()/A0.z());
+          break;
+        default:
+          Q0 = NULL_VECTOR; // This should never happen!
+      }
+
+      CGAL_SMS_LT_TRACE(3, "      Q0:" << xyz_to_string(Q0));
+
+      CGAL_assertion(Q0 != NULL_VECTOR);
+
+      Vector Q1 = cross_product(A0, Q0);
+      Vector A1 = H * Q0;
+      Vector A2 = H * Q1;
+
+      FT b1 = - (Q0 * c);
+      FT b2 = - (Q1 * c);
+
+      CGAL_SMS_LT_TRACE(3, "      Q1:" << xyz_to_string(Q1) << "\n      A1: " << xyz_to_string(A1) << "\n      A2:" << xyz_to_string(A2)
+                             << "\n      b1:" << n_to_string(b1) << "\n      b2:" << n_to_string(b2));
+
+      add_constraint_if_alpha_compatible(A1,b1);
+      add_constraint_if_alpha_compatible(A2,b2);
+    }
+      break;
+
+    case 2:
+    {
+      Vector Q = cross_product(mConstraints_A.r0(),mConstraints_A.r1());
+      Vector A2 = H * Q;
+      FT b2 = - (Q * c);
+      CGAL_SMS_LT_TRACE(3, "      Q:" << xyz_to_string(Q) << "\n      A2: " << xyz_to_string(A2) << "\n      b2:" << n_to_string(b2));
+
+      add_constraint_if_alpha_compatible(A2,b2);
+    }
+      break;
+  }
+}
+
+} // namespace internal
+} // namespace Surface_mesh_simplification
+} // namespace CGAL
+
+#endif // CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_DETAIL_LINDSTROM_TURK_CORE_H
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/internal/LindstromTurk_params.h 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/internal/LindstromTurk_params.h
--- 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/internal/LindstromTurk_params.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/internal/LindstromTurk_params.h	2020-09-08 12:15:45.000000000 +0000
@@ -0,0 +1,45 @@
+// Copyright (c) 2006  GeometryFactory (France). All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/internal/LindstromTurk_params.h $
+// $Id: LindstromTurk_params.h ff09c5d 2019-10-25T16:35:53+02:00 Mael Rouxel-Labbé
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Fernando Cacciola <fernando.cacciola@geometryfactory.com>
+//
+#ifndef CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_INTERNAL_LINDSTROMTURK_PARAMS_H
+#define CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_INTERNAL_LINDSTROMTURK_PARAMS_H
+
+#include <CGAL/license/Surface_mesh_simplification.h>
+
+namespace CGAL {
+namespace Surface_mesh_simplification {
+namespace internal {
+
+struct LindstromTurk_params
+{
+  LindstromTurk_params()
+    :
+      m_volume_weight(0.5),
+      m_boundary_weight(0.5),
+      m_shape_weight(0)
+  {}
+
+  LindstromTurk_params(double volume_weight, double boundary_weight, double shape_weight)
+    :
+      m_volume_weight(volume_weight),
+      m_boundary_weight(boundary_weight),
+      m_shape_weight(shape_weight)
+  {}
+
+  double m_volume_weight;
+  double m_boundary_weight;
+  double m_shape_weight;
+};
+
+} // namespace internal
+} // namespace Surface_mesh_simplification
+} // namespace CGAL
+
+#endif // CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_INTERNAL_LINDSTROMTURK_PARAMS_H
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/LindstromTurk_cost.h 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/LindstromTurk_cost.h
--- 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/LindstromTurk_cost.h	2020-08-01 19:02:56.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/LindstromTurk_cost.h	2020-09-08 12:15:45.000000000 +0000
@@ -2,60 +2,48 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/LindstromTurk_cost.h $
-// $Id: LindstromTurk_cost.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/LindstromTurk_cost.h $
+// $Id: LindstromTurk_cost.h ff09c5d 2019-10-25T16:35:53+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Fernando Cacciola <fernando.cacciola@geometryfactory.com>
 //
 #ifndef CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_LINDSTROMTURK_COST_H
-#define CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_LINDSTROMTURK_COST_H 1
+#define CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_LINDSTROMTURK_COST_H
 
 #include <CGAL/license/Surface_mesh_simplification.h>
 
-
-#include <CGAL/Surface_mesh_simplification/Detail/Common.h>
-#include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Detail/Lindstrom_Turk_core.h>
+#include <CGAL/Surface_mesh_simplification/internal/Common.h>
+#include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/internal/Lindstrom_Turk_core.h>
+#include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/internal/LindstromTurk_params.h>
 
 namespace CGAL {
+namespace Surface_mesh_simplification {
 
-namespace Surface_mesh_simplification
-{
-
-  template<class TM_>
+template<class TM_>
 class LindstromTurk_cost
 {
 public:
+  typedef TM_                                                                 TM;
+  typedef internal::LindstromTurk_params                                      LindstromTurk_params;
 
-  typedef TM_ TM ;
-  /*
-
-  typedef Edge_profile<TM> Profile ;
-  typedef typename Traits::Point_3 Point;
-  typedef typename Traits::FT FT ;
-
-  typedef optional<FT> result_type ;
-  */
-public:
-
-  LindstromTurk_cost( LindstromTurk_params const& aParams = LindstromTurk_params() ) : mParams(aParams) {}
+  LindstromTurk_cost(const LindstromTurk_params& LT_params = LindstromTurk_params())
+    : m_LT_params(LT_params)
+  {}
 
   template <typename Profile>
-  optional<typename Profile::FT>
-  operator()( Profile const& aProfile, optional<typename Profile::Point> const& aPlacement ) const
+  boost::optional<typename Profile::FT>
+  operator()(const Profile& profile,
+             const boost::optional<typename Profile::Point>& placement) const
   {
-    return LindstromTurkCore<TM,Profile>(mParams,aProfile).compute_cost(aPlacement) ;
+    return internal::LindstromTurkCore<TM, Profile>(m_LT_params, profile).compute_cost(placement);
   }
 
 private:
-
-  LindstromTurk_params mParams ;
+  LindstromTurk_params m_LT_params;
 };
 
 } // namespace Surface_mesh_simplification
+} // namespace CGAL
 
-} //namespace CGAL
-
-#endif // CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_LINDSTROMTURK_COST_H //
-// EOF //
-
+#endif // CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_LINDSTROMTURK_COST_H
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/LindstromTurk.h 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/LindstromTurk.h
--- 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/LindstromTurk.h	2020-08-01 19:02:56.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/LindstromTurk.h	2020-09-08 12:15:45.000000000 +0000
@@ -2,22 +2,18 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/LindstromTurk.h $
-// $Id: LindstromTurk.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/LindstromTurk.h $
+// $Id: LindstromTurk.h ff09c5d 2019-10-25T16:35:53+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Fernando Cacciola <fernando.cacciola@geometryfactory.com>
 //
 #ifndef CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_LINDSTROMTURK_H
-#define CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_LINDSTROMTURK_H 1
+#define CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_LINDSTROMTURK_H
 
 #include <CGAL/license/Surface_mesh_simplification.h>
 
-
-#include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/LindstromTurk_params.h>
 #include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/LindstromTurk_cost.h>
 #include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/LindstromTurk_placement.h>
 
-#endif // CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_LINDSTROMTURK_H //
-// EOF //
-
+#endif // CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_LINDSTROMTURK_H
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/LindstromTurk_params.h 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/LindstromTurk_params.h
--- 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/LindstromTurk_params.h	2020-08-01 19:02:56.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/LindstromTurk_params.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,49 +0,0 @@
-// Copyright (c) 2006  GeometryFactory (France). All rights reserved.
-//
-// This file is part of CGAL (www.cgal.org).
-//
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/LindstromTurk_params.h $
-// $Id: LindstromTurk_params.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
-// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
-//
-// Author(s)     : Fernando Cacciola <fernando.cacciola@geometryfactory.com>
-//
-#ifndef CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_LINDSTROMTURK_PARAMS_H
-#define CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_LINDSTROMTURK_PARAMS_H
-
-#include <CGAL/license/Surface_mesh_simplification.h>
-
-
-namespace CGAL {
-
-namespace Surface_mesh_simplification
-{
-
-struct LindstromTurk_params
-{
-  LindstromTurk_params()
-    :
-    VolumeWeight  (0.5)
-   ,BoundaryWeight(0.5)
-   ,ShapeWeight   (0)
-  {}
-
-  LindstromTurk_params( double aVolumeWeight, double aBoundaryWeight, double aShapeWeight )
-    :
-    VolumeWeight  (aVolumeWeight)
-   ,BoundaryWeight(aBoundaryWeight)
-   ,ShapeWeight   (aShapeWeight)
-  {}
-
-  double VolumeWeight ;
-  double BoundaryWeight ;
-  double ShapeWeight ;
-};
-
-} // namespace Surface_mesh_simplification
-
-} //namespace CGAL
-
-#endif // CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_LINDSTROMTURK_PARAMS_H
-// EOF //
-
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/LindstromTurk_placement.h 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/LindstromTurk_placement.h
--- 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/LindstromTurk_placement.h	2020-08-01 19:02:56.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/LindstromTurk_placement.h	2020-09-08 12:15:45.000000000 +0000
@@ -2,55 +2,45 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/LindstromTurk_placement.h $
-// $Id: LindstromTurk_placement.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/LindstromTurk_placement.h $
+// $Id: LindstromTurk_placement.h ff09c5d 2019-10-25T16:35:53+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Fernando Cacciola <fernando.cacciola@geometryfactory.com>
 //
 #ifndef CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_LINDSTROMTURK_PLACEMENT_H
-#define CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_LINDSTROMTURK_PLACEMENT_H 1
+#define CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_LINDSTROMTURK_PLACEMENT_H
 
 #include <CGAL/license/Surface_mesh_simplification.h>
 
-
-#include <CGAL/Surface_mesh_simplification/Detail/Common.h>
-#include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Detail/Lindstrom_Turk_core.h>
+#include <CGAL/Surface_mesh_simplification/internal/Common.h>
+#include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/internal/Lindstrom_Turk_core.h>
 
 namespace CGAL {
+namespace Surface_mesh_simplification {
 
-namespace Surface_mesh_simplification
-{
-
-  template<class TM_>
+template<class TM_>
 class LindstromTurk_placement
 {
 public:
+  typedef TM_                                                             TM;
+  typedef internal::LindstromTurk_params                                  LindstromTurk_params;
 
-  typedef TM_ TM ;
-
-public:
-
-  LindstromTurk_placement( LindstromTurk_params const& aParams = LindstromTurk_params() ) : mParams(aParams) {}
+  LindstromTurk_placement(const LindstromTurk_params& LT_params = LindstromTurk_params())
+    : m_LT_params(LT_params)
+  {}
 
   template <typename Profile>
-  optional<typename Profile::Point>
-  operator()( Profile const& aProfile) const
+  boost::optional<typename Profile::Point> operator()(const Profile& profile) const
   {
-    return LindstromTurkCore<TM,Profile>(mParams,aProfile).compute_placement() ;
+    return internal::LindstromTurkCore<TM,Profile>(m_LT_params, profile).compute_placement();
   }
 
 private:
-
-  LindstromTurk_params mParams ;
-
+  LindstromTurk_params m_LT_params;
 };
 
-
 } // namespace Surface_mesh_simplification
+} // namespace CGAL
 
-} //namespace CGAL
-
-#endif // CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_LINDSTROMTURK_PLACEMENT_H //
-// EOF //
-
+#endif // CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_LINDSTROMTURK_PLACEMENT_H
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Midpoint_and_length.h 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Midpoint_and_length.h
--- 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Midpoint_and_length.h	2020-08-01 19:02:56.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Midpoint_and_length.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,22 +0,0 @@
-// Copyright (c) 2006  GeometryFactory (France). All rights reserved.
-//
-// This file is part of CGAL (www.cgal.org).
-//
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Midpoint_and_length.h $
-// $Id: Midpoint_and_length.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
-// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
-//
-// Author(s)     : Fernando Cacciola <fernando.cacciola@geometryfactory.com>
-//
-#ifndef CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_MIDPOINT_AND_LENGTH_H
-#define CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_MIDPOINT_AND_LENGTH_H
-
-#include <CGAL/license/Surface_mesh_simplification.h>
-
-
-#include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_length_cost.h>
-#include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Midpoint_placement.h>
-
-#endif // CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_MIDPOINT_AND_LENGTH_H
-// EOF //
-
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Midpoint_placement.h 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Midpoint_placement.h
--- 5.0.3-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Midpoint_placement.h	2020-08-01 19:02:56.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Midpoint_placement.h	2020-09-08 12:15:45.000000000 +0000
@@ -2,48 +2,41 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Midpoint_placement.h $
-// $Id: Midpoint_placement.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_simplification/include/CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Midpoint_placement.h $
+// $Id: Midpoint_placement.h ff09c5d 2019-10-25T16:35:53+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Fernando Cacciola <fernando.cacciola@geometryfactory.com>
 //
 #ifndef CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_MIDPOINT_PLACEMENT_H
-#define CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_MIDPOINT_PLACEMENT_H 1
+#define CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_MIDPOINT_PLACEMENT_H
 
 #include <CGAL/license/Surface_mesh_simplification.h>
 
 
-#include <CGAL/Surface_mesh_simplification/Detail/Common.h>
+#include <CGAL/Surface_mesh_simplification/internal/Common.h>
 #include <CGAL/Surface_mesh_simplification/Policies/Edge_collapse/Edge_profile.h>
 
 namespace CGAL {
+namespace Surface_mesh_simplification {
 
-namespace Surface_mesh_simplification
-{
-
-  template<class TM_>
+template<class TM_>
 class Midpoint_placement
 {
 public:
+  typedef TM_                                                     TM;
 
-  typedef TM_ TM;
-
-  Midpoint_placement()
-  {}
+  Midpoint_placement() {}
 
   template <typename Profile>
-  optional<typename Profile::Point> operator()( Profile const& aProfile ) const
+  boost::optional<typename Profile::Point> operator()(const Profile& profile) const
   {
-    return optional<typename Profile::Point> (midpoint(aProfile.p0(),aProfile.p1()));
+    typedef boost::optional<typename Profile::Point>              result_type;
+    return result_type(profile.geom_traits().construct_midpoint_3_object()(profile.p0(), profile.p1()));
   }
-
 };
 
 } // namespace Surface_mesh_simplification
+} // namespace CGAL
 
-} //namespace CGAL
-
-#endif // CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_MIDPOINT_PLACEMENT_H //
-// EOF //
-
+#endif // CGAL_SURFACE_MESH_SIMPLIFICATION_POLICIES_EDGE_COLLAPSE_MIDPOINT_PLACEMENT_H
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_topology/internal/Edge_weight_functor.h 5.1-1/include/CGAL/Surface_mesh_topology/internal/Edge_weight_functor.h
--- 5.0.3-1/include/CGAL/Surface_mesh_topology/internal/Edge_weight_functor.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_topology/internal/Edge_weight_functor.h	2020-09-08 12:15:45.000000000 +0000
@@ -0,0 +1,56 @@
+// Copyright (c) 2020 CNRS and LIRIS' Establishments (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_topology/include/CGAL/Surface_mesh_topology/internal/Edge_weight_functor.h $
+// $Id: Edge_weight_functor.h 19d7e19 2020-04-06T09:13:47+02:00 Guillaume Damiand
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Thien Hoang <thienvhoang99@gmail.com>
+//
+#ifndef CGAL_EDGE_WEIGHT_FUNCTOR_H
+#define CGAL_EDGE_WEIGHT_FUNCTOR_H
+
+#include <CGAL/license/Surface_mesh_topology.h>
+
+#include <CGAL/Face_graph_wrapper.h>
+#include <CGAL/squared_distance_3.h>
+
+namespace CGAL {
+namespace Surface_mesh_topology {
+
+// Unit weight, all edges have the same weight: 1.
+struct Unit_weight_functor
+{
+  using Weight_t=unsigned int;
+  template <class T>
+  Weight_t operator() (T) const { return 1; }
+};
+
+// Euclidean distance weight functor: each edge has as weight its Euclidean length.
+template<typename Mesh>
+struct Euclidean_length_weight_functor
+{
+  using Weight_t=double;
+  using Dart_const_handle=typename Get_map<Mesh, Mesh>::type::Dart_const_handle;
+
+  Euclidean_length_weight_functor(const Mesh& m) : m_mesh(m), m_map(m)
+  {}
+
+  Weight_t operator() (Dart_const_handle dh) const
+  {
+    return CGAL::sqrt(CGAL::squared_distance
+                      (Get_traits<Mesh>::get_point(m_mesh, dh),
+                       Get_traits<Mesh>::get_point(m_mesh, m_map.other_extremity(dh))));
+  }
+
+protected:
+  const Mesh& m_mesh;
+  const typename Get_map<Mesh, Mesh>::storage_type m_map;
+};
+
+} // namespace Surface_mesh_topology
+} // namespace CGAL
+
+#endif // CGAL_EDGE_WEIGHT_FUNCTOR_H
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_topology/internal/Facewidth.h 5.1-1/include/CGAL/Surface_mesh_topology/internal/Facewidth.h
--- 5.0.3-1/include/CGAL/Surface_mesh_topology/internal/Facewidth.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_topology/internal/Facewidth.h	2020-09-08 12:15:46.000000000 +0000
@@ -0,0 +1,156 @@
+// Copyright (c) 2020 CNRS and LIRIS' Establishments (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_topology/include/CGAL/Surface_mesh_topology/internal/Facewidth.h $
+// $Id: Facewidth.h a09b8b1 2020-04-21T08:49:10+02:00 Guillaume Damiand
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Thien Hoang <thienvhoang99@gmail.com>
+//
+#ifndef CGAL_FACEWIDTH_H
+#define CGAL_FACEWIDTH_H
+
+#include <CGAL/license/Surface_mesh_topology.h>
+
+#include <CGAL/Path_on_surface.h>
+#include <CGAL/Timer.h>
+#include <CGAL/Surface_mesh_topology/internal/Generic_map_selector.h>
+#include <CGAL/Surface_mesh_topology/internal/Shortest_noncontractible_cycle.h>
+
+namespace CGAL {
+namespace Surface_mesh_topology {
+namespace internal {
+
+template <class Mesh_>
+class Facewidth
+{
+public:
+  using Self=Facewidth<Mesh_>;
+  using Mesh=Mesh_;
+
+  using Original_map_wrapper=internal::Generic_map_selector<Mesh, Items_for_shortest_noncontractible_cycle>;
+  using Original_dart_const_handle=typename Original_map_wrapper::Dart_const_handle_original;
+
+  using Local_map        =typename Original_map_wrapper::Generic_map;
+  using Dart_handle      =typename Local_map::Dart_handle;
+  using Dart_const_handle=typename Local_map::Dart_const_handle;
+  using size_type        = typename Local_map::size_type;
+
+  using Path             = CGAL::Surface_mesh_topology::Path_on_surface<Mesh>;
+  using SNC              = Shortest_noncontractible_cycle<Local_map, false>;
+
+  Facewidth(const Mesh& amesh, bool display_time=false):
+    m_snc_to_find_facewidth(nullptr)
+  {
+    CGAL::Timer t;
+    if (display_time)
+    { t.start(); }
+
+    typename Original_map_wrapper::Origin_to_copy_map origin_to_radial;
+    typename Original_map_wrapper::Copy_to_origin_map local_radial_to_origin;
+
+    m_is_perforated=m_radial_map.get_new_mark();
+    Original_map_wrapper::copy(m_radial_map, amesh, origin_to_radial,
+                               local_radial_to_origin, m_is_perforated);
+
+    m_radial_map.negate_mark(m_is_perforated);
+    // Remove all boundary by adding faces, marked with m_is_perforated
+    m_radial_map.template close<2>();
+    m_radial_map.negate_mark(m_is_perforated);
+
+    m_mark_new_vertex=m_radial_map.get_new_mark();
+    size_type face_treated=m_radial_map.get_new_mark();
+    size_type original_darts=m_radial_map.get_new_mark();
+    m_radial_map.negate_mark(original_darts); // All original darts are marked
+
+    for (typename Local_map::Dart_range::iterator it=m_radial_map.darts().begin();
+         it!=m_radial_map.darts().end(); ++it)
+    {
+      if (!m_radial_map.is_marked(it, m_is_perforated) &&
+          !m_radial_map.is_marked(it, face_treated) &&
+          m_radial_map.is_marked(it, original_darts))
+      {
+        m_radial_map.template mark_cell<2>(it, face_treated);
+        Dart_handle new_vertex=m_radial_map.insert_cell_0_in_cell_2(it);
+        for (auto itv=m_radial_map.template darts_of_cell_basic<0>(new_vertex, m_mark_new_vertex).begin(),
+             itvend=m_radial_map.template darts_of_cell_basic<0>(new_vertex, m_mark_new_vertex).end();
+             itv!=itvend; ++itv)
+        {
+          m_radial_map.mark(itv, m_mark_new_vertex);
+          m_radial_to_original[m_radial_map.opposite2(itv)]=
+              local_radial_to_origin[m_radial_map.next(itv)];
+        }
+      }
+    }
+
+    // Remove the original edges of m_radial_map, and create vertex info.
+    for (auto it=m_radial_map.darts().begin(), itend=m_radial_map.darts().end();
+         it!=itend; ++it)
+    {
+      if (m_radial_map.template attribute<0>(it)==nullptr)
+      { m_radial_map.template set_attribute<0>
+          (it, m_radial_map.template create_attribute<0>(-1)); }
+
+      if (m_radial_map.is_marked(it, original_darts))
+      { m_radial_map.template remove_cell<1>(it); }
+    }
+
+    CGAL_assertion(m_radial_map.is_whole_map_unmarked(original_darts));
+    CGAL_assertion(m_radial_map.is_whole_map_unmarked(face_treated));
+
+    m_radial_map.free_mark(original_darts);
+    m_radial_map.free_mark(face_treated);
+
+    m_snc_to_find_facewidth=std::make_unique<SNC>(&m_radial_map, m_is_perforated);
+
+    if (display_time)
+    {
+      t.stop();
+      std::cout<<"[TIME] Facewidth constructor: "<<t.time()<<" seconds."<<std::endl;
+    }
+  }
+
+  std::vector<Original_dart_const_handle> compute_face_width(bool display_time=false)
+  {
+    CGAL::Timer t;
+    if (display_time)
+    { t.start(); }
+
+    // Find edgewidth of the radial map
+    Path_on_surface<Local_map> edgewidth_of_radial_map=
+        m_snc_to_find_facewidth->compute_edge_width();
+
+    std::vector<Original_dart_const_handle> cycle;
+    cycle.reserve(edgewidth_of_radial_map.length());
+    for (std::size_t i=0, n=edgewidth_of_radial_map.length(); i<n; i++)
+    {
+      Dart_handle dh=m_radial_map.dart_handle
+          (const_cast<typename Local_map::Dart&>(*edgewidth_of_radial_map.get_ith_real_dart(i)));
+      if (!m_radial_map.is_marked(dh, m_mark_new_vertex))
+      { cycle.push_back(m_radial_to_original[dh]); }
+    }
+
+    if (display_time)
+    {
+      t.stop();
+      std::cout<<"[TIME] compute_face_width: "<<t.time()<<" seconds."<<std::endl;
+    }
+
+    return cycle;
+  }
+
+protected:
+  Local_map m_radial_map;
+  size_type m_is_perforated;   /// mark for perforated darts
+  size_type m_mark_new_vertex; /// mark for new vertices
+  typename Original_map_wrapper::Copy_to_origin_map m_radial_to_original;
+  std::unique_ptr<SNC> m_snc_to_find_facewidth;
+};
+
+} // namespace internal
+} // namespace Surface_mesh_topology
+} // namespace CGAL
+
+#endif
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_topology/internal/Functors_for_face_graph_wrapper.h 5.1-1/include/CGAL/Surface_mesh_topology/internal/Functors_for_face_graph_wrapper.h
--- 5.0.3-1/include/CGAL/Surface_mesh_topology/internal/Functors_for_face_graph_wrapper.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_topology/internal/Functors_for_face_graph_wrapper.h	2020-09-08 12:15:46.000000000 +0000
@@ -0,0 +1,149 @@
+// Copyright (c) 2019 CNRS and LIRIS' Establishments (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_topology/include/CGAL/Surface_mesh_topology/internal/Functors_for_face_graph_wrapper.h $
+// $Id: Functors_for_face_graph_wrapper.h 3fb644e 2020-04-10T17:40:30+02:00 Guillaume Damiand
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
+//
+#ifndef CGAL_FUNCTORS_FOR_FACE_GRAPH_WRAPPER_H
+#define CGAL_FUNCTORS_FOR_FACE_GRAPH_WRAPPER_H 1
+
+#include <CGAL/license/Surface_mesh_topology.h>
+
+#include <boost/graph/graph_traits.hpp>
+#include <CGAL/boost/graph/helpers.h>
+
+////////////////////////////////////////////////////////////////////////////////
+/** This file contains the following functors for Face_graph_wrapper:
+ * Get_beta<typename HEG, unsigned int i>::
+         operator() (const HEG& heg, Dart_const_handle dh)
+ * Index_from_halfedge_descriptor<Mesh>::run(m, h)
+ * Halfedge_descriptor_from_index<Mesh>::run(m, i)
+ * Is_index_used<Mesh>::run(m, i)
+*/
+////////////////////////////////////////////////////////////////////////////////
+namespace CGAL {
+////////////////////////////////////////////////////////////////////////////////
+template<typename P>
+class Surface_mesh;
+////////////////////////////////////////////////////////////////////////////////
+namespace internal {
+////////////////////////////////////////////////////////////////////////////////
+/// Get_beta
+template<typename HEG, unsigned int i>
+struct Get_beta
+{
+  typedef typename boost::graph_traits<HEG>::halfedge_descriptor Dart_const_handle;
+
+  static Dart_const_handle value(const HEG& /*heg*/, Dart_const_handle /*dh*/)
+  {
+    std::cout<<"ERROR Get_beta<HEG, "<<i<<">"<<std::endl;
+    CGAL_assertion(false);
+    return nullptr;
+  }
+};
+template<typename HEG>
+struct Get_beta<HEG, 0>
+{
+  typedef typename boost::graph_traits<HEG>::halfedge_descriptor Dart_const_handle;
+  static Dart_const_handle value(const HEG& heg, Dart_const_handle dh)
+  { return prev(dh, heg); }
+};
+template<typename HEG>
+struct Get_beta<HEG, 1>
+{
+  typedef typename boost::graph_traits<HEG>::halfedge_descriptor Dart_const_handle;
+  static Dart_const_handle value(const HEG& heg, Dart_const_handle dh)
+  { return next(dh, heg); }
+};
+template<typename HEG>
+struct Get_beta<HEG, 2>
+{
+  typedef typename boost::graph_traits<HEG>::halfedge_descriptor Dart_const_handle;
+  static Dart_const_handle value(const HEG& heg, Dart_const_handle dh)
+  { return opposite(dh, heg); }
+};
+////////////////////////////////////////////////////////////////////////////////
+template<typename Mesh>
+struct Index_from_halfedge_descriptor
+{
+  typedef boost::uint32_t size_type;
+  typedef typename boost::template graph_traits<Mesh>::halfedge_descriptor
+  halfedge_descriptor;
+
+  static size_type run(const Mesh& m, halfedge_descriptor h)
+  {
+    size_type res=0;
+    for (typename boost::template graph_traits<Mesh>::halfedge_iterator
+           it=halfedges(m).begin(), itend=halfedges(m).end(); it!=itend; ++it, ++res)
+    { if ((*it)==h) { return res; } }
+    return (std::numeric_limits<size_type>::max)();
+  }
+};
+template<typename P>
+struct Index_from_halfedge_descriptor<CGAL::Surface_mesh<P> >
+{
+  using Mesh=CGAL::Surface_mesh<P>;
+  typedef boost::uint32_t size_type;
+  typedef typename boost::template graph_traits<Mesh>::halfedge_descriptor
+  halfedge_descriptor;
+
+  static size_type run(const Mesh& /*m*/, halfedge_descriptor h)
+  { return (size_type)(h); }
+};
+////////////////////////////////////////////////////////////////////////////////
+template<typename Mesh>
+struct Halfedge_descriptor_from_index
+{
+  typedef boost::uint32_t size_type;
+  typedef typename boost::template graph_traits<Mesh>::halfedge_descriptor
+  halfedge_descriptor;
+
+  static halfedge_descriptor run(const Mesh& m, size_type i)
+  {
+    for (typename boost::template graph_traits<Mesh>::halfedge_iterator
+           it=halfedges(m).begin(), itend=halfedges(m).end(); it!=itend; ++it, --i)
+    { if (i==0) { return *it; } }
+    return halfedge_descriptor(nullptr);
+  }
+};
+template<typename P>
+struct Halfedge_descriptor_from_index<CGAL::Surface_mesh<P> >
+{
+  using Mesh=CGAL::Surface_mesh<P>;
+  typedef boost::uint32_t size_type;
+  typedef typename boost::template graph_traits<Mesh>::halfedge_descriptor
+  halfedge_descriptor;
+
+  static halfedge_descriptor run(const Mesh& /*m*/, size_type i)
+  { return halfedge_descriptor(i); }
+};
+////////////////////////////////////////////////////////////////////////////////
+template<typename Mesh>
+struct Is_index_used
+{
+  typedef boost::uint32_t size_type;
+
+  static bool run(const Mesh& m, size_type i)
+  { return i<m.size_of_halfedges(); }
+};
+template<typename P>
+struct Is_index_used<CGAL::Surface_mesh<P> >
+{
+  using Mesh=CGAL::Surface_mesh<P>;
+  typedef boost::uint32_t size_type;
+
+  static bool run(const Mesh& m, size_type i)
+  { return i<(m.number_of_halfedges()+m.number_of_removed_halfedges()) &&
+             !m.is_removed(typename Mesh::Halfedge_index(i)); }
+};
+////////////////////////////////////////////////////////////////////////////////
+} // namespace internal
+} // namespace CGAL
+
+#endif // CGAL_FUNCTORS_FOR_FACE_GRAPH_WRAPPER_H //
+// EOF //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_topology/internal/Generic_map_selector.h 5.1-1/include/CGAL/Surface_mesh_topology/internal/Generic_map_selector.h
--- 5.0.3-1/include/CGAL/Surface_mesh_topology/internal/Generic_map_selector.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_topology/internal/Generic_map_selector.h	2020-09-08 12:15:46.000000000 +0000
@@ -0,0 +1,147 @@
+// Copyright (c) 2020 CNRS and LIRIS' Establishments (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_topology/include/CGAL/Surface_mesh_topology/internal/Generic_map_selector.h $
+// $Id: Generic_map_selector.h e6536aa 2020-06-09T20:15:16+02:00 Laurent Rineau
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Thien Hoang <thienvhoang99@gmail.com>
+//
+#ifndef CGAL_GENERIC_MAP_SELECTOR_H
+#define CGAL_GENERIC_MAP_SELECTOR_H
+
+#include <CGAL/license/Surface_mesh_topology.h>
+
+// #include <CGAL/Polyhedron_3.h>
+// #include <CGAL/Surface_mesh/Surface_mesh.h>
+#include <CGAL/Combinatorial_map.h>
+#include <CGAL/Generalized_map.h>
+#include <CGAL/Linear_cell_complex_for_combinatorial_map.h>
+#include <CGAL/Linear_cell_complex_for_generalized_map.h>
+#include <CGAL/Polygonal_schema.h>
+
+#include <tuple>
+
+namespace CGAL {
+namespace Surface_mesh_topology {
+namespace internal {
+
+  struct Items_for_shortest_noncontractible_cycle
+  {
+    template <class Map>
+    struct Dart_wrapper
+    {
+      using Vertex_attribute = CGAL::Cell_attribute<Map, int>;
+      using Attributes       = std::tuple<Vertex_attribute>;
+    };
+  };
+
+  template <class Mesh_, class Items_>
+  struct SNC_for_generalized_map
+  {
+    using Mesh_original              = Mesh_;
+    using Generic_map                = CGAL::Generalized_map<2, Items_>;
+    using Dart_const_handle_original = typename Mesh_original::Dart_const_handle;
+    using Copy_to_origin_map         = boost::unordered_map<typename Generic_map::Dart_handle,
+                                                            Dart_const_handle_original>;
+    using Origin_to_copy_map         = boost::unordered_map<Dart_const_handle_original,
+                                                            typename Generic_map::Dart_handle>;
+
+    static void copy(Generic_map& target, const Mesh_original& source,
+                     Origin_to_copy_map& origin_to_copy,
+                     Copy_to_origin_map& copy_to_origin,
+                     typename Generic_map::size_type mark_perforated)
+    {
+      target.copy_from_const(source, &origin_to_copy, &copy_to_origin, true, mark_perforated);
+    }
+  };
+
+  template <class Mesh_, class Items_>
+  struct SNC_for_combinatorial_map
+  {
+    using Mesh_original              = Mesh_;
+    using Generic_map                = CGAL::Combinatorial_map<2, Items_>;
+    using Dart_const_handle_original = typename Mesh_original::Dart_const_handle;
+    using Copy_to_origin_map         = boost::unordered_map<typename Generic_map::Dart_handle,
+                                                            Dart_const_handle_original>;
+    using Origin_to_copy_map         = boost::unordered_map<Dart_const_handle_original,
+                                                            typename Generic_map::Dart_handle>;
+
+    static void copy(Generic_map& target, const Mesh_original& source,
+                     Origin_to_copy_map& origin_to_copy, Copy_to_origin_map& copy_to_origin,
+                     typename Generic_map::size_type mark_perforated)
+    {
+      target.copy_from_const(source, &origin_to_copy, &copy_to_origin, true, mark_perforated);
+    }
+  };
+
+  template <class Mesh_, class Items_>
+  struct Generic_map_selector
+  {
+    using Mesh_original              = Mesh_;
+    using Generic_map                = CGAL::Combinatorial_map<2, Items_>;
+    using Dart_const_handle_original = typename boost::graph_traits<Mesh_original>::halfedge_descriptor;
+    using Copy_to_origin_map         = boost::unordered_map<typename Generic_map::Dart_handle,
+                                                            Dart_const_handle_original>;
+    using Origin_to_copy_map         = boost::unordered_map<Dart_const_handle_original,
+                                                            typename Generic_map::Dart_handle>;
+
+    static void copy(Generic_map& target, const Mesh_original& source,
+                     Origin_to_copy_map& origin_to_copy, Copy_to_origin_map& copy_to_origin,
+                     typename Generic_map::size_type mark_perforated)
+    {
+      target.import_from_halfedge_graph(source, &origin_to_copy, &copy_to_origin, true, mark_perforated);
+    }
+  };
+
+  template <unsigned int d, class Refs, class Items, class Alloc, class Storage, class Items2>
+  struct Generic_map_selector<CGAL::Generalized_map_base<d, Refs, Items, Alloc, Storage>, Items2 >
+    : SNC_for_generalized_map< CGAL::Generalized_map_base<d, Refs, Items, Alloc, Storage>, Items2 > {};
+
+  template <unsigned int d, class Items, class Alloc, class Storage, class Items2>
+  struct Generic_map_selector< CGAL::Generalized_map<d, Items, Alloc, Storage>, Items2 >
+    : SNC_for_generalized_map< CGAL::Generalized_map<d, Items, Alloc, Storage>, Items2 > {};
+
+  template <unsigned int d, unsigned int d2, class Traits, class Items,
+            class Alloc, template<unsigned int,class,class,class,class> class Map,
+            class Storage, class Items2>
+  struct Generic_map_selector< CGAL::Linear_cell_complex_for_generalized_map
+      <d, d2, Traits, Items, Alloc, Map, Storage>, Items2 >
+      : SNC_for_generalized_map< CGAL::Linear_cell_complex_for_generalized_map
+      <d, d2, Traits, Items, Alloc, Map, Storage>, Items2 > {};
+
+  template <class Items_, class Alloc_, class Storage_, class Items2>
+  struct Generic_map_selector< CGAL::Surface_mesh_topology::Polygonal_schema_with_generalized_map
+                               <Items_, Alloc_, Storage_>, Items2 > : SNC_for_generalized_map
+  <CGAL::Surface_mesh_topology::Polygonal_schema_with_generalized_map
+      <Items_, Alloc_, Storage_>, Items2 > {};
+
+  template <unsigned int d, class Refs, class Items, class Alloc, class Storage, class Items2>
+  struct Generic_map_selector< CGAL::Combinatorial_map_base<d, Refs, Items, Alloc, Storage>, Items2 >
+  : SNC_for_combinatorial_map< CGAL::Combinatorial_map_base<d, Refs, Items, Alloc, Storage>, Items2 > {};
+
+  template <unsigned int d, class Items, class Alloc, class Storage, class Items2>
+  struct Generic_map_selector< CGAL::Combinatorial_map<d, Items, Alloc, Storage>, Items2 >
+  : SNC_for_combinatorial_map< CGAL::Combinatorial_map<d, Items, Alloc, Storage>, Items2 > {};
+
+  template <unsigned int d, unsigned int d2, class Traits, class Items,
+            class Alloc, template<unsigned int,class,class,class,class> class Map,
+            class Storage, class Items2>
+  struct Generic_map_selector< CGAL::Linear_cell_complex_for_combinatorial_map
+      <d, d2, Traits, Items, Alloc, Map, Storage>, Items2 >
+      : SNC_for_combinatorial_map< CGAL::Linear_cell_complex_for_combinatorial_map
+      <d, d2, Traits, Items, Alloc, Map, Storage>, Items2 > {};
+
+  template <class Items_, class Alloc_, class Storage_, class Items2>
+  struct Generic_map_selector< CGAL::Surface_mesh_topology::Polygonal_schema_with_combinatorial_map
+                               <Items_, Alloc_, Storage_>, Items2 > :
+      SNC_for_combinatorial_map<CGAL::Surface_mesh_topology::
+      Polygonal_schema_with_combinatorial_map<Items_, Alloc_, Storage_>, Items2 > {};
+
+} // namespace internal
+} // namespace Surface_mesh_topology
+} // namespace CGAL
+
+#endif
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_topology/internal/Iterators_for_face_graph_wrapper.h 5.1-1/include/CGAL/Surface_mesh_topology/internal/Iterators_for_face_graph_wrapper.h
--- 5.0.3-1/include/CGAL/Surface_mesh_topology/internal/Iterators_for_face_graph_wrapper.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_topology/internal/Iterators_for_face_graph_wrapper.h	2020-09-08 12:15:46.000000000 +0000
@@ -0,0 +1,345 @@
+// Copyright (c) 2019 CNRS and LIRIS' Establishments (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_topology/include/CGAL/Surface_mesh_topology/internal/Iterators_for_face_graph_wrapper.h $
+// $Id: Iterators_for_face_graph_wrapper.h 1de5666 2020-04-20T14:39:05+02:00 Guillaume Damiand
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
+//
+#ifndef CGAL_ITERATORS_FOR_FACE_GRAPH_WRAPPER_H
+#define CGAL_ITERATORS_FOR_FACE_GRAPH_WRAPPER_H 1
+
+#include <CGAL/license/Surface_mesh_topology.h>
+
+#include <boost/graph/graph_traits.hpp>
+#include <CGAL/boost/graph/helpers.h>
+#include <CGAL/Surface_mesh_topology/internal/Functors_for_face_graph_wrapper.h>
+#include <stack>
+
+namespace CGAL {
+namespace internal {
+
+  //****************************************************************************
+  /* Class CMap_dart_iterator_basic_of_all: to iterate onto all the
+   * darts of the face graph.
+   */
+  template <typename Map_,bool Const=false>
+  class FGW_dart_iterator_basic_of_all
+  {
+  public:
+    typedef FGW_dart_iterator_basic_of_all Self;
+
+    typedef Map_ Map;
+    typedef typename Map::Dart_handle Dart_handle;
+    typedef typename Map::size_type size_type;
+
+  public:
+    /// Main constructor.
+    FGW_dart_iterator_basic_of_all(const Map& amap):
+      mmap(amap),
+      m_it(halfedges(amap.get_fg()).begin())
+    {}
+
+    /// Constructor with a dart in parameter (for end iterator).
+    FGW_dart_iterator_basic_of_all(const Map& amap, Dart_handle /*adart*/):
+      mmap(amap),
+      m_it(halfedges(amap.get_fg()).end())
+    {}
+
+    FGW_dart_iterator_basic_of_all(const FGW_dart_iterator_basic_of_all& other):
+      mmap(other.mmap),
+      m_it(other.m_it)
+    {}
+
+    Self& operator=(const Self& other)
+    {
+      CGAL_assertion(&mmap==&(other.mmap));
+      m_it=other.m_it;
+      return *this;
+    }
+
+    operator Dart_handle() const
+    { return operator*(); }
+
+    bool operator==(const Self& other) const
+    { return &mmap==&(other.mmap) && m_it==other.m_it; }
+
+    bool operator!=(const Self& other) const
+    { return !(operator==(other)); }
+
+    /// Prefix ++ operator.
+    Self& operator++()
+    {
+      CGAL_assertion(m_it!=halfedges(this->mmap.get_fg()).end());
+      ++m_it;
+      return *this;
+    }
+
+    /// Postfix ++ operator.
+    Self operator++(int)
+    { Self res=*this; operator ++(); return res; }
+
+    /// Prefix -- operator.
+    Self& operator--()
+    {
+      CGAL_assertion(m_it!=halfedges(this->mmap.get_fg()).begin());
+      --m_it;
+      return *this;
+    }
+
+    /// Postfix -- operator.
+    Self operator--(int)
+    { Self res=*this; operator --(); return res; }
+
+
+    Dart_handle operator*() const
+    {
+      CGAL_assertion(m_it!=halfedges(this->mmap.get_fg()).end());
+      return *m_it;
+    }
+
+  protected:
+    const Map& mmap;
+    typename boost::graph_traits<typename Map_::HEG>::halfedge_iterator m_it;
+  };
+
+////////////////////////////////////////////////////////////////////////////////
+  template <typename Map_>
+  class FGW_basis_for_cell_iterator
+  {
+  public:
+    typedef FGW_basis_for_cell_iterator Self;
+    typedef Map_ Map;
+    typedef typename Map::Dart_handle Dart_handle;
+    typedef typename Map::size_type size_type;
+
+    /// Main constructor.
+    FGW_basis_for_cell_iterator(const Map& amap, Dart_handle adart):
+      mmap(amap),
+      m_firstdart(adart),
+      m_curdart(adart)
+    {}
+
+    /// Constructor with two darts in parameter (for end iterator).
+    FGW_basis_for_cell_iterator(const Map& amap, Dart_handle adart,
+                                Dart_handle /* d2 */):
+      mmap(amap),
+      m_firstdart(adart),
+      m_curdart(Dart_handle())
+    {}
+
+    bool operator==(const Self& other) const
+    { return &mmap==&(other.mmap) && m_firstdart==other.m_firstdart &&
+        m_curdart==other.m_curdart; }
+
+    bool operator!=(const Self& other) const
+    { return !(this->operator==(other)); }
+
+    operator Dart_handle() const
+    { return operator*(); }
+
+    Dart_handle operator*() const
+    {
+      CGAL_assertion(m_curdart!=Dart_handle());
+      return m_curdart;
+    }
+
+  protected:
+    const Map& mmap;
+    Dart_handle m_firstdart, m_curdart;
+  };
+////////////////////////////////////////////////////////////////////////////////
+  template<typename HEG, unsigned int i>
+  class FGW_cell_iterator
+  {};
+////////////////////////////////////////////////////////////////////////////////
+/// Vertex iterator
+  template<typename Map_>
+  class FGW_cell_iterator<Map_, 0>: public FGW_basis_for_cell_iterator<Map_> // Vertex
+  {
+  public:
+    typedef FGW_cell_iterator Self;
+    typedef FGW_basis_for_cell_iterator<Map_> Base;
+    typedef Map_ Map;
+    typedef typename Map::Dart_handle Dart_handle;
+    typedef typename Map::size_type size_type;
+
+    FGW_cell_iterator(const Map& amap, Dart_handle adart) : Base(amap, adart),
+                                                            m_second_way(false)
+    {}
+
+    /// Constructor with two darts in parameter (for end iterator).
+    FGW_cell_iterator(const Map& amap, Dart_handle adart,
+                      Dart_handle d2): Base(amap, adart, d2)
+    {}
+
+    /// Prefix ++ operator.
+    Self& operator++()
+    {
+      if (!m_second_way)
+      {
+        if (this->mmap.template is_free<2>(this->m_curdart))
+        {
+          m_second_way=true;
+          this->m_curdart=this->mmap.template beta<0>(this->m_firstdart);
+          if (this->mmap.template is_free<2>(this->m_curdart))
+          { this->m_curdart=Dart_handle(); }
+          else { this->m_curdart=this->mmap.template beta<2>(this->m_curdart); }
+        }
+        else
+        {
+          this->m_curdart=this->mmap.template beta<2, 1>(this->m_curdart);
+          if (this->m_curdart==this->m_firstdart)
+          { this->m_curdart=Dart_handle(); }
+        }
+      }
+      else
+      {
+        this->m_curdart=this->mmap.template beta<0>(this->m_curdart);
+        if (this->mmap.template is_free<2>(this->m_curdart))
+        { this->m_curdart=Dart_handle(); }
+        else { this->m_curdart=this->mmap.template beta<2>(this->m_curdart); }
+      }
+
+      return *this;
+    }
+
+    /// Postfix ++ operator.
+    Self operator++(int)
+    { Self res=*this; operator ++(); return res; }
+
+protected:
+  /// True if we already found a border dart, and thus turn in the second way
+  bool m_second_way;
+};
+template<typename Map_>
+class FGW_cell_iterator<Map_, 1>: public FGW_basis_for_cell_iterator<Map_>  // Edge
+{
+  public:
+    typedef FGW_cell_iterator Self;
+    typedef FGW_basis_for_cell_iterator<Map_> Base;
+    typedef Map_ Map;
+    typedef typename Map::Dart_handle Dart_handle;
+    typedef typename Map::size_type size_type;
+
+    FGW_cell_iterator(const Map& amap, Dart_handle adart) : Base(amap, adart)
+    {}
+
+    /// Constructor with two darts in parameter (for end iterator).
+    FGW_cell_iterator(const Map& amap, Dart_handle adart,
+                      Dart_handle d2): Base(amap, adart, d2)
+    {}
+
+    /// Prefix ++ operator.
+    Self& operator++()
+    {
+      if (this->m_curdart==this->m_firstdart)
+      {
+        if (this->mmap.template is_free<2>(this->m_curdart))
+        { this->m_curdart=Dart_handle(); }
+        else { this->m_curdart=this->mmap.template beta<2>(this->m_curdart); }
+      }
+      else
+      { this->m_curdart=Dart_handle(); }
+
+      return *this;
+    }
+
+    /// Postfix ++ operator.
+    Self operator++(int)
+    { Self res=*this; operator ++(); return res; }
+};
+template<typename Map_>
+class FGW_cell_iterator<Map_, 2>: public FGW_basis_for_cell_iterator<Map_> // Face
+{
+public:
+  typedef FGW_cell_iterator Self;
+  typedef FGW_basis_for_cell_iterator<Map_> Base;
+  typedef Map_ Map;
+  typedef typename Map::Dart_handle Dart_handle;
+  typedef typename Map::size_type size_type;
+
+  FGW_cell_iterator(const Map& amap, Dart_handle adart) : Base(amap, adart)
+  {}
+
+  /// Constructor with two darts in parameter (for end iterator).
+  FGW_cell_iterator(const Map& amap, Dart_handle adart,
+                    Dart_handle d2): Base(amap, adart, d2)
+  {}
+
+  /// Prefix ++ operator.
+  Self& operator++()
+  {
+    this->m_curdart=this->mmap.template beta<1>(this->m_curdart);
+    if (this->m_curdart==this->m_firstdart)
+    { this->m_curdart=Dart_handle(); }
+
+    return *this;
+  }
+
+  /// Postfix ++ operator.
+  Self operator++(int)
+  { Self res=*this; operator ++(); return res; }
+};
+template<typename Map_>
+class FGW_cell_iterator<Map_, 3>: public FGW_basis_for_cell_iterator<Map_> // CC
+{
+public:
+  typedef FGW_cell_iterator Self;
+  typedef FGW_basis_for_cell_iterator<Map_> Base;
+  typedef Map_ Map;
+  typedef typename Map::Dart_handle Dart_handle;
+  typedef typename Map::size_type size_type;
+
+  FGW_cell_iterator(const Map& amap, Dart_handle adart) : Base(amap, adart)
+  { m_mark=this->mmap.get_new_mark(); }
+
+  /// Constructor with two darts in parameter (for end iterator).
+  FGW_cell_iterator(const Map& amap, Dart_handle adart,
+                    Dart_handle d2): Base(amap, adart, d2)
+  { m_mark=this->mmap.get_new_mark(); }
+
+  ~FGW_cell_iterator()
+  { this->mmap.free_mark(m_mark); }
+
+  /// Prefix ++ operator.
+  Self& operator++()
+  {
+    if (!this->mmap.is_marked(this->mmap.template beta<1>(this->m_curdart), m_mark))
+    {
+      m_to_treat.push(this->mmap.template beta<1>(this->m_curdart));
+      this->mmap.mark(this->mmap.template beta<1>(this->m_curdart), m_mark);
+    }
+    if (!this->mmap.template is_free<2>(this->m_curdart) &&
+        !this->mmap.is_marked(this->mmap.template beta<2>(this->m_curdart), m_mark))
+    {
+      m_to_treat.push(this->mmap.template beta<2>(this->m_curdart));
+      this->mmap.mark(this->mmap.template beta<2>(this->m_curdart), m_mark);
+    }
+
+    if (m_to_treat.empty())
+    { this->m_curdart=Dart_handle(); }
+    else
+    { this->m_curdart=m_to_treat.top(); m_to_treat.pop(); }
+
+    return *this;
+  }
+
+  /// Postfix ++ operator.
+  Self operator++(int)
+  { Self res=*this; operator ++(); return res; }
+
+protected:
+  typename Map_::size_type m_mark;
+  std::stack<Dart_handle> m_to_treat;
+};
+////////////////////////////////////////////////////////////////////////////////
+
+} // namespace internal
+} // namespace CGAL
+
+#endif // CGAL_ITERATORS_FOR_FACE_GRAPH_WRAPPER_H //
+// EOF //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_topology/internal/Minimal_quadrangulation.h 5.1-1/include/CGAL/Surface_mesh_topology/internal/Minimal_quadrangulation.h
--- 5.0.3-1/include/CGAL/Surface_mesh_topology/internal/Minimal_quadrangulation.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_topology/internal/Minimal_quadrangulation.h	2020-09-08 12:15:46.000000000 +0000
@@ -0,0 +1,1604 @@
+// Copyright (c) 2019 CNRS and LIRIS' Establishments (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_topology/include/CGAL/Surface_mesh_topology/internal/Minimal_quadrangulation.h $
+// $Id: Minimal_quadrangulation.h e6536aa 2020-06-09T20:15:16+02:00 Laurent Rineau
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
+//
+#ifndef CGAL_MINIMAL_QUADRANGULATION_H
+#define CGAL_MINIMAL_QUADRANGULATION_H 1
+
+// Should be defined before to include Path_on_surface_with_rle.h
+// If nothing is defined, use V1
+// #define CGAL_PWRLE_TURN_V1  // Compute turns by turning (method of CMap)
+// #define CGAL_PWRLE_TURN_V2  // Compute turns by using an id of darts, given by an hash-table (built and given by Minimal_quadrangulation)
+#define CGAL_PWRLE_TURN_V3  // Compute turns by using an id of darts, associated in Info of Darts (build by Minimal_quadrangulation)
+
+#include <CGAL/license/Surface_mesh_topology.h>
+
+#include <CGAL/Face_graph_wrapper.h>
+#include <CGAL/Path_on_surface.h>
+#include <CGAL/Surface_mesh_topology/internal/Path_on_surface_with_rle.h>
+#include <CGAL/Surface_mesh_topology/internal/Path_generators.h>
+#include <CGAL/Combinatorial_map_operations.h>
+#include <CGAL/Cell_attribute.h>
+#include <CGAL/Random.h>
+#include <CGAL/Timer.h>
+#include <unordered_map>
+#include <queue>
+#include <tuple>
+#include <iostream>
+
+namespace CGAL {
+namespace Surface_mesh_topology {
+namespace internal {
+
+struct Minimal_quadrangulation_local_map_items
+{
+  template <class CMap>
+  struct Dart_wrapper
+  {
+#ifdef CGAL_PWRLE_TURN_V3
+    typedef std::size_t Dart_info;
+#endif // CGAL_PWRLE_TURN_V3
+    typedef CGAL::Cell_attribute<CMap, int32_t> Vertex_attribute;
+    typedef std::tuple<Vertex_attribute, void, void> Attributes;
+  };
+};
+
+template<typename Mesh_>
+class Minimal_quadrangulation
+{
+public:
+  typedef Minimal_quadrangulation<Mesh_> Self;
+  typedef Mesh_                          Mesh;
+
+  typedef typename Get_map<Mesh, Mesh>::type       Original_map; // Mesh seen as a 2-map
+  typedef typename Original_map::Dart_const_handle Original_dart_const_handle;
+  typedef typename Original_map::size_type         Original_size_type;
+
+  typedef CGAL::Combinatorial_map<2, Minimal_quadrangulation_local_map_items> Local_map;
+  typedef typename Local_map::Dart_handle             Dart_handle;
+  typedef typename Local_map::Dart_const_handle       Dart_const_handle;
+  typedef typename Local_map::size_type               size_type;
+
+  // Associate each dart of the original map, not removed, a pair of darts in
+  // the reduced map.
+  typedef std::unordered_map<Original_dart_const_handle,
+  std::pair<Dart_handle, Dart_handle> > TPaths;
+
+  // Associations between original darts and their copy.
+  // Use only locally during the construction of the minimal quadrangulation.
+  typedef std::unordered_map<Original_dart_const_handle, Dart_handle> Origin_to_copy;
+  typedef std::unordered_map<Dart_handle, Original_dart_const_handle> Copy_to_origin;
+
+#ifdef CGAL_PWRLE_TURN_V2
+  typedef std::unordered_map<Original_dart_const_handle, std::size_t> TDartIds;
+#endif //CGAL_PWRLE_TURN_V2
+
+  /// @return the original map
+  const Original_map& get_original_map() const
+  { return m_original_map; }
+
+  /// @return the reduced map
+  const Local_map& get_local_map() const
+  { return m_local_map; }
+
+  /// @return the reduced map
+  Local_map& get_local_map()
+  { return m_local_map; }
+
+  /// Constructor taking a mesh as parameter.
+  Minimal_quadrangulation(const Mesh& amesh, bool display_time=false) :
+    m_original_map(amesh)
+  {
+    if (!get_original_map().is_without_boundary(1))
+    {
+      std::cerr<<"ERROR: the given amap has 1-boundaries; "
+               <<"such a surface is not possible to process here."
+               <<std::endl;
+      return;
+    }
+
+    CGAL::Timer t, t2;
+    if (display_time)
+    { t.start(); t2.start(); }
+
+    // The mapping between darts of the original map into the copied map.
+    Origin_to_copy origin_to_copy;
+
+    // The mapping between darts of the copy into darts of the original map.
+    Copy_to_origin copy_to_origin;
+
+    // We reserve three marks: m_mart_T and m_mark_L to mark darts in
+    // get_original_map() that belong to T or to L (i.e. edges contracted or
+    // removed), m_mark_perforated to mark perforated faces.
+    m_mark_T=get_original_map().get_new_mark();
+    m_mark_L=get_original_map().get_new_mark();
+    m_mark_perforated=get_local_map().get_new_mark();
+
+    // 1) We create m_local_map as a copy of amap, by contracting all
+    //    non loop.
+    surface_simplification_in_one_vertex(origin_to_copy, copy_to_origin);
+
+    if (display_time)
+    {
+      t2.stop();
+      std::cout<<"[TIME] Simplification in one vertex: "
+               <<t2.time()<<" seconds"<<std::endl;
+
+      t2.reset(); t2.start();
+    }
+
+#ifdef CGAL_TRACE_CMAP_TOOLS
+    std::cout<<"Creation of reduced map with non loops contracted: ";
+    get_local_map().display_characteristics(std::cout);
+    std::cout<<", valid="<<get_local_map().is_valid()<<std::endl;
+#endif
+
+    // 2) We simplify m_local_map in a surface with only one face
+    //    (or maybe more if we have boundaries!)
+    surface_simplification_in_one_face(origin_to_copy, copy_to_origin);
+
+    // 3) We remove all non perforated loops
+    remove_non_perforated_loops(copy_to_origin);
+
+    if (display_time)
+    {
+      t2.stop();
+      std::cout<<"[TIME] Simplification in one face: "
+               <<t2.time()<<" seconds"<<std::endl;
+      t2.reset(); t2.start();
+    }
+
+#ifdef CGAL_TRACE_CMAP_TOOLS
+    std::cout<<"All faces merges: ";
+    get_local_map().display_characteristics(std::cout);
+    std::cout<<", valid="<<get_local_map().is_valid()<<std::endl;
+#endif
+
+    if (!get_local_map().is_empty()) // is_empty if the surface is a sphere
+    {
+      // 4) We quadrangulate the face, except for the torus surfaces
+      if (!local_map_is_a_torus())
+      {
+        surface_quadrangulate();
+
+        if (display_time)
+        {
+          t2.stop();
+          std::cout<<"[TIME] Face quadrangulation: "
+                  <<t2.time()<<" seconds"<<std::endl;
+          t2.reset(); t2.start();
+        }
+      }
+
+#if defined(CGAL_PWRLE_TURN_V2) || defined(CGAL_PWRLE_TURN_V3)
+      // Now we label all the darts of the reduced map, to allow the computation
+      // of turns in constant time: only for methods V2 and V3
+      initialize_vertices_attributes();
+      initialize_ids();
+#endif // defined(CGAL_PWRLE_TURN_V2) || defined(CGAL_PWRLE_TURN_V3)
+    }
+
+#ifdef CGAL_TRACE_CMAP_TOOLS
+    std::cout<<"After quadrangulation: ";
+    get_local_map().display_characteristics(std::cout);
+    std::cout<<", valid="<<get_local_map().is_valid()<<std::endl;
+
+    std::cout<<"Paths are all valid ? "<<(are_paths_valid()?"YES":"NO")
+             <<std::endl;
+    auto marktemp=get_local_map().get_new_mark();
+    Dart_handle dh2=nullptr;
+    for (auto it=get_local_map().darts().begin();
+         it!=get_local_map().darts().end(); ++it)
+    {
+      if (!get_local_map().is_marked(it, marktemp))
+      {
+        std::cout<<"Degree="<<CGAL::template
+                   degree<Local_map, 0>(get_local_map(), it)
+            <<std::endl;
+        std::cout<<"Co-degree="<<CGAL::template
+                   codegree<Local_map, 2>(get_local_map(), it)
+            <<std::endl;
+        dh2=it;
+        do
+        {
+          get_local_map().mark(dh2, marktemp);
+          std::cout<<get_local_map().darts().index(dh2)<<"   "
+                   <<get_local_map().darts().
+                     index(get_local_map().template beta<0>(dh2))
+                   <<std::endl;
+          dh2=get_local_map().template beta<0,2>(dh2);
+        }
+        while(dh2!=it);
+      }
+    }
+    get_local_map().free_mark(marktemp);
+    get_local_map().display_darts(std::cout);
+#endif
+
+    if (display_time)
+    {
+      t.stop();
+      std::cout<<"[TIME] Total time for computation of reduced map: "
+               <<t.time()<<" seconds"<<std::endl;
+    }
+
+    CGAL_assertion(perforated_faces_correctly_marked());
+    CGAL_assertion(local_map_is_a_torus() || are_paths_valid()); // Because torus is a special case
+  }
+
+  ~Minimal_quadrangulation()
+  {
+    get_original_map().free_mark(m_mark_T);
+    get_original_map().free_mark(m_mark_L);
+    get_local_map().free_mark(m_mark_perforated);
+  }
+
+  /// @return true iff 'p' is contractible.
+  bool is_contractible(const Path_on_surface<Mesh>& p,
+                       bool display_time=false) const
+  {
+    if (p.is_empty())
+    { return true; }
+
+    if (!p.is_closed())
+    {
+      std::cerr<<"Error: is_contractible requires a closed path."<<std::endl;
+      return false;
+    }
+
+    if (get_local_map().is_empty())
+    { return true; } // A closed path on a sphere is always contractible.
+
+    CGAL::Timer t;
+    if (display_time)
+    { t.start(); }
+
+    bool res=false;
+    if (local_map_is_a_torus())
+    { // Case of torus
+      Path_on_surface<Local_map>
+        pt=transform_original_path_into_quad_surface_for_torus(p);
+
+      int a, b;
+      count_edges_of_path_on_torus(pt, a, b);
+      res=(a==0 && b==0);
+    }
+    else
+    {
+      internal::Path_on_surface_with_rle<Self>
+        pt=transform_original_path_into_quad_surface_with_rle(p);
+
+      pt.canonize();
+      res=pt.is_empty();
+    }
+
+    if (display_time)
+    {
+      t.stop();
+      std::cout<<"[TIME] is_contractible: "<<t.time()<<" seconds"
+               <<std::endl;
+    }
+
+    return res;
+  }
+
+  /// @return true iff 'p1' and 'p2' are freely homotopic.
+  bool are_freely_homotopic(const Path_on_surface<Mesh>& p1,
+                            const Path_on_surface<Mesh>& p2,
+                            bool display_time=false) const
+  {
+    if (p1.is_empty() && p2.is_empty()) { return true; }
+
+    if ((!p1.is_empty() && !p1.is_closed()) ||
+        (!p2.is_empty() && !p2.is_closed()))
+    {
+      std::cerr<<"Error: are_freely_homotopic requires two closed paths."
+               <<std::endl;
+      return false;
+    }
+
+    // Here we have two non empty closed paths.
+    if (get_local_map().is_empty())
+    { return true; } // Two closed paths on a sphere are always homotopic.
+
+    CGAL::Timer t;
+    if (display_time)
+    { t.start(); }
+
+    bool res=false;
+    if (local_map_is_a_torus())
+    { // Case of torus
+      Path_on_surface<Local_map>
+        pt1=transform_original_path_into_quad_surface_for_torus(p1);
+      Path_on_surface<Local_map>
+        pt2=transform_original_path_into_quad_surface_for_torus(p2);
+
+      int a1, a2, b1, b2;
+      count_edges_of_path_on_torus(pt1, a1, b1);
+      count_edges_of_path_on_torus(pt2, a2, b2);
+      res=(a1==a2 && b1==b2);
+    }
+    else
+    {
+      internal::Path_on_surface_with_rle<Self>
+        pt1=transform_original_path_into_quad_surface_with_rle(p1);
+      internal::Path_on_surface_with_rle<Self>
+        pt2=transform_original_path_into_quad_surface_with_rle(p2);
+      pt1.canonize();
+      pt2.canonize();
+      res=(pt1==pt2); // Do here to be counted in the computation time
+
+#ifdef CGAL_TRACE_PATH_TESTS
+      std::cout<<"Length of reduced paths: "<<pt1.length()<<" and "
+               <<pt2.length()<<std::endl;
+#endif
+    }
+
+    if (display_time)
+    {
+      t.stop();
+      std::cout<<"[TIME] are_freely_homotopic: "<<t.time()<<" seconds"<<std::endl;
+    }
+
+    return res;
+  }
+
+  /// @return true iff 'p1' and 'p2' are base point freely homotopic.
+  bool are_base_point_homotopic(const Path_on_surface<Mesh>& p1,
+                                const Path_on_surface<Mesh>& p2,
+                                bool display_time=false) const
+  {
+    if (p1.is_empty() && p2.is_empty()) { return true; }
+    if (p1.is_empty() || p2.is_empty()) { return false; }
+
+    if (!get_original_map().template belong_to_same_cell<0>
+        (p1.front_flip()?get_original_map().template beta<1>(p1.front()):p1.front(),
+         p2.front_flip()?get_original_map().template beta<1>(p2.front()):p2.front()) ||
+        !get_original_map().template belong_to_same_cell<0>
+        (p1.back_flip()?p1.back():get_original_map().template beta<1>(p1.back()),
+         p2.back_flip()?p2.back():get_original_map().template beta<1>(p2.back())))
+    {
+      std::cerr<<"Error: are_base_point_homotopic requires two paths that"
+               <<" share the same vertices as extremities."<<std::endl;
+      return false;
+    }
+
+    if (get_local_map().is_empty())
+    { return true; } // Two paths on a sphere are always base_point_homotopic.
+
+    CGAL::Timer t;
+    if (display_time)
+    { t.start(); }
+
+    Path_on_surface<Mesh> path=p1;
+    Path_on_surface<Mesh> path2=p2; path2.reverse();
+    path+=path2;
+
+    bool res=is_contractible(path);
+
+    if (display_time)
+    {
+      t.stop();
+      std::cout<<"[TIME] are_base_point_homotopic: "<<t.time()<<" seconds."
+               <<std::endl;
+    }
+
+    return res;
+  }
+
+  /// @return the positive turn between the two given darts.
+  /// i.e. turning right if the face bounded by a dart lays on its right
+  /// and left otherwise
+  std::size_t positive_turn(Dart_const_handle dh1,
+                            Dart_const_handle dh2) const
+  {
+#if defined(CGAL_PWRLE_TURN_V2) || defined(CGAL_PWRLE_TURN_V3)
+    return compute_positive_turn_given_ids
+        (get_local_map().template beta<2>(dh1), dh2);
+#else // defined(CGAL_PWRLE_TURN_V2) || defined(CGAL_PWRLE_TURN_V3)
+
+    CGAL_assertion((!get_local_map().template is_free<1>(dh1)));
+    CGAL_assertion((!get_local_map().template is_free<2>(dh1)));
+
+    if (dh2==get_local_map().template beta<2>(dh1) &&
+        dh2==get_local_map().template beta<1>(dh1))
+    { return 0; }
+
+    if (get_local_map().is_marked(dh1, m_mark_hole))
+    { return (std::numeric_limits<std::size_t>::max)(); }
+
+    Dart_const_handle ddh1=dh1;
+    std::size_t res=1;
+    while (get_local_map().template beta<1>(ddh1)!=dh2)
+    {
+      CGAL_assertion(!get_local_map().template is_free<2>
+                     (get_local_map().template beta<1>(ddh1)));
+
+      ++res;
+      ddh1=get_local_map().template beta<1, 2>(ddh1);
+      if (get_local_map().is_marked(ddh1, m_mark_hole))
+      { return (std::numeric_limits<std::size_t>::max)(); }
+
+      CGAL_assertion(!get_local_map().template is_free<1>(ddh1));
+      CGAL_assertion(get_local_map().template beta<1>(ddh1)==dh2 || ddh1!=dh1);
+    }
+    return res;
+#endif // defined(CGAL_PWRLE_TURN_V2) || defined(CGAL_PWRLE_TURN_V3)
+  }
+
+  /// @return the negative turn between the two given darts.
+  /// i.e. turning right if the face bounded by a dart lays on its left
+  /// and left otherwise
+  std::size_t negative_turn(Dart_const_handle dh1,
+                            Dart_const_handle dh2) const
+  {
+#if defined(CGAL_PWRLE_TURN_V2) || defined(CGAL_PWRLE_TURN_V3)
+    return compute_negative_turn_given_ids
+        (get_local_map().template beta<2>(dh1), dh2);
+#else // defined(CGAL_PWRLE_TURN_V2) || defined(CGAL_PWRLE_TURN_V3)
+
+    CGAL_assertion((!get_local_map().template is_free<1>(dh1)));
+    CGAL_assertion((!get_local_map().template is_free<2>(dh1)));
+
+    if (dh2==get_local_map().template beta<2>(dh1) &&
+        dh1==get_local_map().template beta<0>(dh2))
+    { return 0; }
+
+    if (get_local_map().is_marked
+        (get_local_map().template beta <2>(dh1), m_mark_hole))
+    { return (std::numeric_limits<std::size_t>::max)(); }
+
+    dh1=get_local_map().template beta<2>(dh1);
+    dh2=get_local_map().template beta<2>(dh2);
+    Dart_const_handle ddh1=dh1;
+    std::size_t res=1;
+    while (get_local_map().template beta<0>(ddh1)!=dh2)
+    {
+      CGAL_assertion(!get_local_map().template is_free<2>
+                     (get_local_map().template beta<0>(ddh1)));
+
+      ++res;
+      ddh1=get_local_map().template beta<0, 2>(ddh1);
+      if (get_local_map().is_marked(ddh1, m_mark_hole))
+      { return (std::numeric_limits<std::size_t>::max)(); }
+
+      CGAL_assertion(!get_local_map().template is_free<0>(ddh1));
+      CGAL_assertion(get_local_map().template beta<0>(ddh1)==dh2 || ddh1!=dh1);
+    }
+    return res;
+#endif // defined(CGAL_PWRLE_TURN_V2) || defined(CGAL_PWRLE_TURN_V3)
+  }
+
+protected:
+
+  /// @return true iff the dart dh is contracted, i.e. if it belongs to T
+  bool is_contracted(Original_dart_const_handle dh) const
+  { return get_original_map().is_marked(dh, m_mark_T); }
+
+  void count_edges_of_path_on_torus
+  (const Path_on_surface<Local_map>& path,
+   int& a, int& b) const
+  {
+    CGAL_assertion(local_map_is_a_torus());
+
+    Dart_const_handle dha=get_local_map().darts().begin();
+    Dart_const_handle dhb=get_local_map().template beta<1>(dha);
+
+    a=0; b=0;
+    for (std::size_t i=0; i<path.length(); ++i)
+    {
+      if(path.get_ith_flip(i))
+      {
+        if (path[i]==dha) { --a; }
+        else if (path[i]==get_local_map().template beta<2>(dha)) { ++a; }
+        else if (path[i]==dhb) { --b; }
+        else if (path[i]==get_local_map().template beta<2>(dhb)) { ++b; }
+      }
+      else
+      {
+        if (path[i]==dha) { ++a; }
+        else if (path[i]==get_local_map().template beta<2>(dha)) { --a; }
+        else if (path[i]==dhb) { ++b; }
+        else if (path[i]==get_local_map().template beta<2>(dhb)) { --b; }
+      }
+    }
+  }
+
+  Path_on_surface<Local_map>
+  transform_original_path_into_quad_surface_for_torus
+  (const Path_on_surface<Mesh>& path) const
+  {
+    CGAL_assertion(local_map_is_a_torus());
+
+    Path_on_surface<Local_map> res(get_local_map());
+    if (path.is_empty()) return res;
+
+    Dart_const_handle cur;
+    for (std::size_t i=0; i<path.length(); ++i)
+    {
+      if (!is_contracted(path[i]))// here flip doesn't matter
+      {
+        cur=get_first_dart_of_the_path(path[i], path.get_ith_flip(i), false);
+        CGAL_assertion(!get_local_map().is_marked(cur, m_mark_perforated));
+        while(cur!=get_second_dart_of_the_path(path[i], path.get_ith_flip(i), false))
+        {
+          res.push_back(cur, false, false);
+          cur=get_local_map().template beta<1>(cur);
+        }
+      }
+    }
+    res.update_is_closed();
+    CGAL_assertion(res.is_empty() || res.is_closed());
+    CGAL_assertion(res.is_valid());
+    return res;
+  }
+
+  internal::Path_on_surface_with_rle<Self>
+  transform_original_path_into_quad_surface_with_rle
+  (const Path_on_surface<Mesh>& path) const
+  {
+    internal::Path_on_surface_with_rle<Self>
+      res(*this
+#ifdef CGAL_PWRLE_TURN_V2
+          , m_dart_ids
+#endif //CGAL_PWRLE_TURN_V2
+          );
+
+    if (path.is_empty()) return res;
+
+    for (std::size_t i=0; i<path.length(); ++i)
+    {
+      if (!is_contracted(path[i]))
+      {
+        res.push_back(get_first_dart_of_the_path
+                      (path[i], path.get_ith_flip(i)), true);
+        if (!edge_path_has_only_one_dart(path[i]))
+        { res.push_back(get_second_dart_of_the_path
+                        (path[i], path.get_ith_flip(i)), true); }
+      }
+    }
+    res.update_is_closed();
+    if (!res.is_empty())
+    { res.merge_last_flat_with_next_if_possible(); }
+    CGAL_assertion(res.is_closed() || res.is_empty());
+    CGAL_assertion(res.is_valid());
+    return res;
+  }
+
+  /// Mark the edge containing adart in the original map.
+  void mark_original_edge(Original_dart_const_handle adart,
+                          Original_size_type amark)
+  {
+    get_original_map().mark(adart, amark);
+    if (!get_original_map().template is_free<2>(adart))
+    { get_original_map().mark(get_original_map().template beta<2>(adart),
+                              amark); }
+  }
+
+  /// Mark the edge containing adart in the reduced map (which is 2-closed)
+  void mark_reduced_edge(Dart_const_handle adart, size_type amark)
+  {
+    get_local_map().mark(adart, amark);
+    get_local_map().mark(get_local_map().template beta<2>(adart), amark);
+  }
+
+  /// Erase the edge given by adart (which belongs to the map m_local_map, i.e. the
+  /// copy) from the associative array copy_to_origin, and erase the
+  /// corresponding edge (which belongs to the map get_original_map()) from the
+  /// array origin_to_copy
+  void erase_edge_from_associative_arrays(Dart_handle adart,
+                                          Origin_to_copy& origin_to_copy,
+                                          Copy_to_origin & copy_to_origin)
+  {
+    if (!get_original_map().template is_free<2>(copy_to_origin[adart]))
+    {
+      origin_to_copy.erase(get_original_map().template beta<2>
+                           (copy_to_origin[adart]));
+      copy_to_origin.erase(get_local_map().template beta<2>(adart));
+    }
+
+    origin_to_copy.erase(copy_to_origin[adart]);
+    copy_to_origin.erase(adart);
+  }
+
+  Original_dart_const_handle prev_in_boundary(Original_dart_const_handle d)
+  {
+    CGAL_assertion(get_original_map().template is_free<2>(d));
+    Original_dart_const_handle res=get_original_map().template beta<0>(d);
+    while(!get_original_map().template is_free<2>(res))
+    { res=get_original_map().template beta<2, 0>(res); }
+    return res;
+  }
+
+  /// Step 1) Transform m_local_map into an equivalent surface having only one
+  /// vertex. All edges contracted during this step belong to the spanning
+  /// tree T, and thus corresponding edges in get_original_map() are marked.
+
+  /// Marks all darts belonging to T using a BFS
+  void compute_T()
+  {
+    Original_dart_const_handle dh;
+    auto grey=get_original_map().get_new_mark();
+    std::queue<Original_dart_const_handle> queue;
+    get_original_map().template mark_cell<0>
+        (get_original_map().darts().begin(), grey);
+    queue.push(get_original_map().darts().begin());
+
+    while (!queue.empty())
+    {
+      dh=queue.front();
+      queue.pop();
+      for (auto it=get_original_map().template darts_of_cell<0>(dh).begin(),
+             itend=get_original_map().template darts_of_cell<0>(dh).end();
+               it!=itend; ++it)
+      {
+        if (!get_original_map().is_marked
+            (get_original_map().template beta<1>(it), grey))
+        {
+          mark_original_edge(it, m_mark_T);
+          get_original_map().template mark_cell<0>
+              (get_original_map().template beta<1>(it), grey);
+          queue.push(get_original_map().template beta<1>(it));
+        }
+      }
+    }
+
+    CGAL_assertion(get_original_map().is_whole_map_marked(grey));
+    get_original_map().free_mark(grey);
+  }
+
+  /// Step 1) Create the copy of the input mesh, while contracting all edges of
+  /// THE spanning tree. Initializes also m_path, the pair of darts of the edge
+  /// in m_copy. This pair of edges will be updated later
+  /// (in surface_simplification_in_one_face() and in surface_quadrangulate()).
+  void surface_simplification_in_one_vertex
+  (Origin_to_copy& origin_to_copy,Copy_to_origin& copy_to_origin)
+  {
+    m_paths.clear();
+    compute_T();
+
+    Dart_handle d1, d2;
+    for (typename Original_map::Dart_range::const_iterator
+         it=get_original_map().darts().begin(),
+         itend=get_original_map().darts().end(); it!=itend; ++it)
+    {
+      if (!get_original_map().is_marked(it, m_mark_T))
+      {
+        if (get_original_map().template is_free<2>(it))
+        {// case of a boundary
+          d1=get_local_map().create_dart();
+          d2=get_local_map().create_dart();
+          get_local_map().template basic_link_beta_for_involution<2>(d1, d2);
+
+          origin_to_copy[it]=d1;
+          copy_to_origin[d1]=it;
+          get_local_map().mark(d2, m_mark_perforated);
+          if (get_original_map().is_perforated(it))
+          { get_local_map().mark(d1, m_mark_perforated); }
+
+          m_paths[it]=std::make_pair(d1, nullptr); // Initialize m_paths
+        }
+        else if (Original_dart_const_handle(it)<
+                 get_original_map().template beta<2>(it))
+        {
+          d1=get_local_map().create_dart();
+          d2=get_local_map().create_dart();
+          get_local_map().template basic_link_beta_for_involution<2>(d1, d2);
+
+          origin_to_copy[it]=d1;
+          origin_to_copy[get_original_map().template beta<2>(it)]=d2;
+          copy_to_origin[d1]=it;
+          copy_to_origin[d2]=get_original_map().template beta<2>(it);
+
+          if (get_original_map().is_perforated(it))
+          { get_local_map().mark(d1, m_mark_perforated); }
+          if (get_original_map().is_perforated
+              (get_original_map().template beta<2>(it)))
+          { get_local_map().mark(d2, m_mark_perforated); }
+
+          m_paths[it]=std::make_pair(d1, nullptr); // Initialize m_paths
+        }
+      }
+    }
+
+    /// Now we only need to do the basic_link_beta_1
+    Original_dart_const_handle dd1;
+    for (typename Original_map::Dart_range::const_iterator
+         it=get_original_map().darts().begin(),
+         itend=get_original_map().darts().end(); it!=itend; ++it)
+    {
+      if (!get_original_map().is_marked(it, m_mark_T))
+      {
+        dd1=get_original_map().template beta<1>(it);
+        while(get_original_map().is_marked(dd1, m_mark_T))
+        { dd1=get_original_map().template beta<1>(dd1); }
+        get_local_map().basic_link_beta_1(origin_to_copy[it],
+                                            origin_to_copy[dd1]); // let's link both
+
+        if (get_original_map().template is_free<2>(it))
+        {
+          dd1=prev_in_boundary(it);
+          while(get_original_map().is_marked(dd1, m_mark_T))
+          { dd1=prev_in_boundary(dd1); }
+          get_local_map().basic_link_beta_1
+              (get_local_map().template beta<2>(origin_to_copy[it]),
+               get_local_map().template beta<2>(origin_to_copy[dd1]));
+        }
+      }
+    }
+  }
+
+  /// Step 2) Transform the 2-map into an equivalent surface having only
+  /// one face. All edges removed during this step belong to the
+  /// dual spanning tree L (spanning tree of the dual 2-map).
+
+  /// Marks all darts belonging to L using a BFS
+  void compute_L(size_type toremove, Copy_to_origin& copy_to_origin)
+  {
+    Dart_handle dh;
+    Dart_handle ddh;
+    auto grey=get_local_map().get_new_mark();
+    std::queue<Dart_handle> queue;
+
+    for (auto it=get_local_map().darts().begin(),
+         itend=get_local_map().darts().end(); it!=itend; ++it)
+    {
+      if (!get_local_map().is_marked(it, grey))
+      {
+        get_local_map().template mark_cell<2>(it, grey);
+        if (!get_local_map().is_marked(it, m_mark_perforated))
+        {
+          queue.push(it);
+
+          while (!queue.empty())
+          {
+            dh=queue.front(); // face(dh) is not perforated (and thus all its darts)
+            queue.pop();
+            ddh=dh;
+            do
+            {
+              if (!get_local_map().is_marked
+                  (get_local_map().template beta<2>(ddh), grey) &&
+                  !get_local_map().is_marked
+                  (get_local_map().template beta<2>(ddh), m_mark_perforated))
+              {
+                mark_reduced_edge(ddh, toremove);
+                mark_original_edge(copy_to_origin[ddh], m_mark_L);
+                get_local_map().template mark_cell<2>
+                    (get_local_map().template beta<2>(ddh), grey);
+                queue.push(get_local_map().template beta<2>(ddh));
+              }
+              ddh=get_local_map().template beta<1>(ddh);
+            }
+            while (dh!=ddh);
+          }
+        }
+      }
+    }
+
+    CGAL_assertion(get_local_map().is_whole_map_marked(grey));
+    get_local_map().free_mark(grey);
+  }
+
+  /// Update all length two paths, before edge removal. Edges that will be
+  /// removed are marked with toremove mark. Dart mark as perforated are not
+  /// updates, since they are associated only with their corresponding dart
+  /// in the original map (and thus with only one dart and not two).
+  void update_length_two_paths_before_edge_removals(size_type toremove,
+                                                    Copy_to_origin& copy_to_origin)
+  {
+    // std::cout<<"************************************************"<<std::endl;
+    Dart_handle initdart, curdart;
+    Original_dart_const_handle d1, d2;
+    for (typename Local_map::Dart_range::iterator
+         it=get_local_map().darts().begin();
+         it!=get_local_map().darts().end(); ++it)
+    {
+      initdart=it;
+      if (!get_local_map().is_marked(initdart, toremove) &&
+          !get_local_map().is_marked(initdart, m_mark_perforated))
+      { // Here we are on a border edge of a "real" face (i.e. non perforated)
+        curdart=get_local_map().template beta<0, 2>(initdart);
+        while(get_local_map().is_marked(curdart, toremove))
+        { // Here, all edges marked to remove are between two real faces.
+          CGAL_assertion(copy_to_origin.count(curdart)==1);
+          set_first_dart_of_the_path(copy_to_origin.find(curdart)->second,
+                                     initdart);
+          curdart=get_local_map().template beta<0, 2>(curdart);
+        }
+
+        if (!get_local_map().is_marked
+            (get_local_map().template beta<2>(curdart), m_mark_perforated))
+        {
+          d1=copy_to_origin.find(get_local_map().template beta<2>(curdart))->second;
+          if (get_original_map().template is_free<2>(d1) ||
+              d1<get_original_map().template beta<2>(d1))
+          { m_paths[d1].second=initdart; }
+        }
+      }
+    }
+
+#ifndef NDEBUG
+    for (auto it=m_paths.begin(), itend=m_paths.end(); it!=itend; ++it)
+    {
+      CGAL_assertion(!get_local_map().is_marked(it->second.first, toremove));
+      CGAL_assertion(it->second.second==nullptr ||
+                     !get_local_map().is_marked(it->second.second, toremove));
+    }
+#endif
+  }
+
+  /// Remove all loops after having merge all real faces in one.
+  /// Erase from m_paths all darts linked with such a dart.
+  void remove_non_perforated_loops(Copy_to_origin& copy_to_origin)
+  {
+    get_local_map().set_automatic_attributes_management(false);
+
+    Original_dart_const_handle origin_dart;
+    for (typename Local_map::Dart_range::iterator
+         it=get_local_map().darts().begin(),
+         itend=get_local_map().darts().end(); it!=itend; ++it)
+    {
+      if (get_local_map().is_dart_used(it) &&
+          !get_local_map().is_marked(it, m_mark_perforated) &&
+          get_local_map().template beta<1>(it)==it)
+        { // A non perforated loop.
+          origin_dart=copy_to_origin[it];
+          if (!get_original_map().template is_free<2>(origin_dart) &&
+              origin_dart>get_original_map().template beta<2>(origin_dart))
+          { origin_dart=get_original_map().template beta<2>(origin_dart); }
+
+          mark_original_edge(origin_dart, m_mark_T);
+
+          if (get_local_map().is_marked(get_local_map().template beta<2>(it),
+                                  m_mark_perforated))
+          { get_local_map().template mark_cell<2>(it, m_mark_perforated); }
+
+          get_local_map().template remove_cell<1>(it);
+        }
+      }
+
+    get_local_map().set_automatic_attributes_management(true);
+
+    /// We remove from m_paths all associations having a removed dart
+    for (auto it=m_paths.begin(), itend=m_paths.end(); it!=itend; )
+    {
+      if (!get_local_map().is_dart_used(it->second.first))
+      {
+        mark_original_edge(it->first, m_mark_T);
+        it=m_paths.erase(it);
+      }
+      else if (it->second.second!=nullptr &&
+               !get_local_map().is_dart_used(it->second.second))
+      {
+        mark_original_edge(it->first, m_mark_T);
+        it=m_paths.erase(it);
+      }
+      else { ++it; }
+    }
+  }
+
+  /// Simplify the reduced map by merging all adjacent non perforated faces.
+  void surface_simplification_in_one_face
+  (Origin_to_copy& origin_to_copy, Copy_to_origin& copy_to_origin)
+  {
+    get_local_map().set_automatic_attributes_management(false);
+
+    size_type toremove=get_local_map().get_new_mark();
+    compute_L(toremove, copy_to_origin);
+
+    if (get_local_map().number_of_marked_darts(toremove)==
+        get_local_map().number_of_darts())
+    { // Case of sphere; all darts are removed.
+      m_paths.clear();
+      get_local_map().clear();
+    }
+    else
+    {
+      // Update the length two paths before to remove the edges.
+      update_length_two_paths_before_edge_removals(toremove, copy_to_origin);
+
+      // Then remove all the edges marekd to remove.
+      for (typename Local_map::Dart_range::iterator
+             it=get_local_map().darts().begin(),
+           itend=get_local_map().darts().end(); it!=itend; ++it)
+      {
+        if (get_local_map().is_dart_used(it) &&
+            get_local_map().is_marked(it, toremove))
+        {
+          CGAL_assertion(get_local_map().is_marked
+                         (get_local_map().template beta<2>(it), toremove));
+          erase_edge_from_associative_arrays(it, origin_to_copy, copy_to_origin);
+          get_local_map().template remove_cell<1>(it);
+        }
+      }
+    }
+
+    get_local_map().set_automatic_attributes_management(true);
+    get_local_map().free_mark(toremove);
+
+#ifndef NDEBUG
+    for (auto it=m_paths.begin(), itend=m_paths.end(); it!=itend; ++it)
+    {
+      CGAL_assertion(get_local_map().is_dart_used(it->second.first));
+      CGAL_assertion(it->second.second==nullptr ||
+                     get_local_map().is_dart_used(it->second.second));
+    }
+#endif
+  }
+
+  /// Step 4) quadrangulate the surface.
+  void surface_quadrangulate()
+  {
+    // Here the map has only one vertex and one face if we have a closed surface,
+    // and maybe several faces if the surface has boundaries
+    size_type oldedges=get_local_map().get_new_mark();
+    get_local_map().negate_mark(oldedges); // now all edges are marked
+
+    // 1) We insert a vertex in each face which is not perforated.
+    //    New edges created by the operation are not marked oldedges.
+    size_type treated=get_local_map().get_new_mark();
+
+    for (typename Local_map::Dart_range::iterator
+         it=get_local_map().darts().begin();
+         it!=get_local_map().darts().end(); ++it)
+    {
+      if (!get_local_map().is_marked(it, treated))
+      {
+        get_local_map().template mark_cell<2>(it, treated);
+        if (get_local_map().is_marked(it, oldedges) &&
+            !get_local_map().is_marked(it, m_mark_perforated))
+        {
+          CGAL_assertion(get_local_map().template beta<1>(it)!=it);
+          get_local_map().negate_mark(treated); // To mark new darts treated
+          get_local_map().insert_cell_0_in_cell_2(it);
+          get_local_map().negate_mark(treated);
+        }
+      }
+    }
+    CGAL_assertion(get_local_map().is_whole_map_marked(treated));
+    get_local_map().free_mark(treated);
+
+#ifdef NDEBUG
+    for (typename Local_map::Dart_range::iterator
+         it=get_local_map().darts().begin();
+         it!=get_local_map().darts().end(); ++it)
+    {
+      if (!get_local_map().is_marked(it, m_mark_perforated))
+      {
+        CGAL_assertion(get_local_map().template beta<1>(it)!=it);
+        CGAL_assertion((get_local_map().template beta<1,1,1>(it)==it));
+      }
+    }
+#endif
+
+    // 2) We update the pair of darts
+    for (typename TPaths::iterator itp=m_paths.begin(), itpend=m_paths.end();
+         itp!=itpend; ++itp)
+    {
+      std::pair<Dart_handle, Dart_handle>& p=itp->second;
+      if (p.second!=nullptr)
+      { // Edge between two real faces, removed during the quadrangulation
+        CGAL_assertion(!get_local_map().is_marked(p.first, m_mark_perforated));
+        CGAL_assertion(!get_local_map().is_marked(p.second, m_mark_perforated));
+        CGAL_assertion(get_local_map().template beta<1>(p.first)!=p.first);
+        CGAL_assertion(get_local_map().template beta<1>(p.second)!=p.second);
+        CGAL_assertion(get_local_map().is_marked(p.first, oldedges));
+        CGAL_assertion(get_local_map().is_marked(p.second, oldedges));
+        CGAL_assertion((get_local_map().template beta<1,1,1>(p.first)==p.first));
+        CGAL_assertion((get_local_map().template beta<1,1,1>(p.second)==p.second));
+        p.first=get_local_map().template beta<0, 2>(p.first);
+        p.second=get_local_map().template beta<0>(p.second);
+        CGAL_assertion(!get_local_map().is_marked(p.first, oldedges));
+        CGAL_assertion(!get_local_map().is_marked(p.second, oldedges));
+      }
+      else if (!get_local_map().is_marked(p.first, m_mark_perforated))
+      { // Edge between a real face and a perforated one, not updated during update_length_two_paths_before_edge_removals
+        CGAL_assertion(get_local_map().is_marked(p.first, oldedges));
+        CGAL_assertion(get_local_map().template beta<1>(p.first)!=p.first);
+        CGAL_assertion((get_local_map().template beta<1,1,1>(p.first)==p.first));
+        p.second=get_local_map().template beta<1, 2>(p.first);
+        p.first=get_local_map().template beta<0, 2>(p.first);
+        CGAL_assertion(!get_local_map().is_marked(p.first, oldedges));
+        CGAL_assertion(!get_local_map().is_marked(p.second, oldedges));
+      }
+      else if (!get_local_map().is_marked
+               (get_local_map().template beta<2>(p.first), m_mark_perforated))
+      { // Edge between a perforated face and a real one, not updated during update_length_two_paths_before_edge_removals
+        CGAL_assertion(get_local_map().is_marked(p.first, oldedges));
+        CGAL_assertion((get_local_map().template beta<2,1,1,1>(p.first)==
+                        get_local_map().template beta<2>(p.first)));
+        CGAL_assertion((get_local_map().template beta<2,1>(p.first)!=
+            get_local_map().template beta<2>(p.first)));
+        p.first=get_local_map().template beta<2, 1>(p.first);
+        p.second=get_local_map().template beta<1>(p.first);
+        CGAL_assertion(!get_local_map().is_marked(p.first, oldedges));
+        CGAL_assertion(!get_local_map().is_marked(p.second, oldedges));
+      }
+    }
+
+#ifdef NDEBUG
+    for (auto it=m_paths.begin(), itend=m_paths.end(); it!=itend; ++it)
+    {
+      if (it->second.second==nullptr)
+      {
+        CGAL_assertion(get_local_map().is_marked(it->second.first, oldedges));
+        CGAL_assertion(get_local_map().is_marked(it->second.first, m_mark_perforated) &&
+                       get_local_map().is_marked
+                       (get_local_map().template beta<2>(it->second.first), m_mark_perforated));
+      }
+      else
+      {
+        CGAL_assertion(!get_local_map().is_marked(it->second.first, oldedges));
+        CGAL_assertion(!get_local_map().is_marked(it->second.second, oldedges));
+      }
+    }
+#endif
+
+    // 3) We remove all the old edges, and extend the perforated faces
+    //    (i.e. we remove edges between real and perforated faces).
+    for (typename Local_map::Dart_range::iterator
+         it=get_local_map().darts().begin(),
+         itend=get_local_map().darts().end(); it!=itend; ++it)
+    {
+      if (get_local_map().is_dart_used(it) &&
+          get_local_map().is_marked(it, oldedges) &&
+          !get_local_map().is_marked(it, m_mark_perforated))
+      {
+        get_local_map().unmark(it, oldedges);
+        if (get_local_map().is_marked
+            (get_local_map().template beta<2>(it), m_mark_perforated))
+        { get_local_map().template mark_cell<2>(it, m_mark_perforated); }
+        get_local_map().template remove_cell<1>(it);
+      }
+      else { get_local_map().unmark(it, oldedges); }
+    }
+
+    CGAL_assertion(get_local_map().is_whole_map_unmarked(oldedges));
+    get_local_map().free_mark(oldedges);
+
+#ifdef NDEBUG
+    for (auto it=m_paths.begin(), itend=m_paths.end(); it!=itend; ++it)
+    {
+      CGAL_assertion(get_local_map().is_dart_used(it->second.first));
+      CGAL_assertion(it->second.second==nullptr ||
+                     get_local_map().is_dart_used(it->second.second));
+    }
+#endif
+  }
+
+#if defined(CGAL_PWRLE_TURN_V2) || defined(CGAL_PWRLE_TURN_V3)
+  // Initialize all vertices attributes.
+  // For all vertex v, the attribute of v is set to deg(v) if there is a
+  // hole around v and -deg(v) if there are no holes around v.
+  void initialize_vertices_attributes()
+  {
+    int deg;
+    bool hole_detected;
+    Dart_handle dh;
+    for (auto it=get_local_map().darts().begin(),
+         itend=get_local_map().darts().end(); it!=itend; ++it)
+    {
+      if (get_local_map().template attribute<0>(it)==NULL)
+      {
+        // We count the degree, while testing if there is a perforated face
+        // incident to the vertex.
+        deg=0;
+        hole_detected=false;
+        dh=it;
+        do
+        {
+          ++deg;
+          if (get_local_map().is_marked(dh, m_mark_perforated))
+          { hole_detected=true; }
+          dh=get_local_map().template beta<2, 1>(dh);
+        }
+        while(dh!=it);
+
+        // Then we set the vertex attribute to deg id there is a
+        // hole and -deg otherwise
+        if (!hole_detected) { deg=-deg; }
+
+        // We create the 0-attribute
+        get_local_map().template set_attribute<0>
+            (it, get_local_map().template create_attribute<0>(deg));
+      }
+    }
+  }
+
+  void initialize_ids()
+  {
+    std::size_t id;
+    Dart_handle dh, ddh;
+    auto treated=get_local_map().get_new_mark();
+
+    for (auto it=get_local_map().darts().begin(),
+         itend=get_local_map().darts().end(); it!=itend; ++it)
+    {
+      if (!get_local_map().is_marked(it, treated))
+      {
+        id=0;
+        if (get_local_map().template info<0>(it)<0)
+        {// there are no holes around this vertex
+         // we just set the ids from 0 to deg(v)-1
+          dh=it;
+          do
+          {
+#ifdef CGAL_PWRLE_TURN_V2
+            m_dart_ids[dh]=id;
+#else // CGAL_PWRLE_TURN_V2
+            // this is for the turn V3
+            get_local_map().info(dh)=id;
+          #endif // CGAL_PWRLE_TURN_V2
+            ++id;
+            get_local_map().mark(dh, treated);
+            dh=get_local_map().template beta<2, 1>(dh);
+          }
+          while(dh!=it);
+        }
+        else
+        {// there is at least a hole around the vertex
+         // we set the 0-th dart just after a hole
+         // then we add 1 to the next dart if we dont cross a hole
+         // and we add deg(v)+1 if we cross a hole
+          dh=it;
+          while(!get_local_map().is_marked(dh, m_mark_perforated))
+          {
+            dh=get_local_map().template beta<2, 1>(dh);
+          }
+          // now dh is right after a hole
+          ddh=dh;
+          do
+          {
+#ifdef CGAL_PWRLE_TURN_V2
+            m_dart_ids[ddh]=id;
+#else // CGAL_PWRLE_TURN_V2
+            // this is for the turn V3
+            get_local_map().info(ddh)=id;
+          #endif // CGAL_PWRLE_TURN_V2
+            if (get_local_map().is_marked(get_local_map().template beta<2>(ddh), m_mark_perforated))
+            { id+=get_local_map().template info<0>(ddh)+1; }
+            else
+            { id+=1; }
+            get_local_map().mark(ddh, treated);
+            ddh=get_local_map().template beta<2, 1>(ddh);
+          }
+          while(ddh!=dh);
+        }
+      }
+    }
+    get_local_map().free_mark(treated);
+  }
+
+  std::size_t get_dart_id(Dart_const_handle dh) const
+  {
+#ifdef CGAL_PWRLE_TURN_V2
+    return m_dart_ids.at(dh);
+#else //  CGAL_PWRLE_TURN_V2
+    // this is for the turn V3
+    return get_local_map().info(dh);
+#endif // CGAL_PWRLE_TURN_V2
+    std::cerr<<"Error: impossible to get dart id without method V2 or V3."<<std::endl;
+    return (std::numeric_limits<std::size_t>::max)();
+  }
+
+  /// @return the positive turn given two darts using their ids (unsed for CGAL_PWRLE_TURN_V2 and V3)
+  std::size_t compute_positive_turn_given_ids(Dart_const_handle dh1,
+                                              Dart_const_handle dh2) const
+  {
+    if (get_local_map().template info<0>(dh1)<0)
+    {// there is no hole around dh1 and dh2
+      if (get_dart_id(dh1)<=get_dart_id(dh2))
+      {
+        return get_dart_id(dh2)-get_dart_id(dh1);
+      }
+      // here we have to add the degree (i.e. substract the vertex info)
+      return get_dart_id(dh2)-get_local_map().template info<0>(dh1)-get_dart_id(dh1);
+    }
+    // here we know there is a hole just before the dart 0 (plus maybe other ones)
+    if (get_dart_id(dh1)>get_dart_id(dh2) || // we crossed dart 0, so we crossed a hole
+        get_dart_id(dh2)-get_dart_id(dh1)>
+        static_cast<std::size_t>(get_local_map().template info<0>(dh1))) // the gap is more than the degree, so we crossed a hole
+    {// so we return an "infinite" value
+      return (std::numeric_limits<std::size_t>::max)();
+    }
+    return get_dart_id(dh2)-get_dart_id(dh1);
+  }
+
+  /// @return the negative turn given two darts using their ids (unsed for CGAL_PWRLE_TURN_V2 and V3)
+  std::size_t compute_negative_turn_given_ids(Dart_const_handle dh1,
+                                              Dart_const_handle dh2) const
+  {
+    if (get_local_map().template info<0>(dh1)<0)
+    {// there is no hole around dh1 and dh2
+      if (get_dart_id(dh1)>=get_dart_id(dh2))
+      {
+        return get_dart_id(dh1)-get_dart_id(dh2);
+      }
+      // here we have to add the degree (i.e. substract the vertex info)
+      return get_dart_id(dh1)-get_local_map().template info<0>(dh1)-get_dart_id(dh2);
+    }
+    // here we know there is a hole just before the dart 0 (plus maybe other ones)
+    if (get_dart_id(dh1)<get_dart_id(dh2) || // we crossed dart 0, so we crossed a hole
+        get_dart_id(dh1)-get_dart_id(dh2)>
+        static_cast<std::size_t>(get_local_map().template info<0>(dh1))) // the gap is more than the degree, so we crossed a hole
+    {// so we return an "infinite" value
+      return (std::numeric_limits<std::size_t>::max)();
+    }
+    return get_dart_id(dh1)-get_dart_id(dh2);
+  }
+#endif // defined(CGAL_PWRLE_TURN_V2) || defined(CGAL_PWRLE_TURN_V3)
+
+  /// @return true iff the edge containing adart is associated with a path.
+  ///         (used for debug purpose because we are suppose to be able to
+  ///          test this by using directly the mark m_mark_T).
+  bool is_edge_has_path(Original_dart_const_handle adart) const
+  {
+    if (get_original_map().template is_free<2>(adart) ||
+        adart<get_original_map().template beta<2>(adart))
+    { return m_paths.find(adart)!=m_paths.end(); }
+    return m_paths.find(get_original_map().template beta<2>(adart))!=m_paths.end();
+  }
+
+  /// @return true iff the edge containing adart is associated with a path
+  ///         of only 1 dart (case of an edge bewteen two perforated faces)
+  bool edge_path_has_only_one_dart(Original_dart_const_handle adart) const
+  {
+    return
+        (get_original_map().is_perforated(adart) &&
+         (get_original_map().template is_free<2>(adart) ||
+          get_original_map().is_perforated
+          (get_original_map().template beta<2>(adart))));
+  }
+
+  /// @return the pair of darts associated with the edge containing adart
+  ///         in get_original_map().
+  /// @pre the edge containing adart must not belong to T.
+  std::pair<Dart_handle, Dart_handle>& get_pair_of_darts
+  (Original_dart_const_handle adart)
+  {
+    CGAL_assertion(!is_contracted(adart));
+    CGAL_assertion(is_edge_has_path(adart));
+
+    if (get_original_map().template is_free<2>(adart) ||
+        adart<get_original_map().template beta<2>(adart))
+    return m_paths.find(get_original_map().template beta<2>(adart))->second;
+  }
+
+  /** @return the first dart of the length 2 path associated with adart.
+   *  @param adart a dart in the original map
+   *  @param flip to return the first dart of beta2(adart)
+   *  @param isquadrangulation true if the minimal map is a quadrangulation,
+   *                           false otherwise (special case of a torus)
+   *  There is a special case for an edge between two perforated faces. In this
+   *  case there is only one dart in the path.
+   */
+  Dart_const_handle get_first_dart_of_the_path(Original_dart_const_handle adart,
+                                               bool flip=false,
+                                               bool isquadrangulation=true) const
+  {
+    CGAL_assertion(is_edge_has_path(adart));
+
+    if (isquadrangulation && get_original_map().is_perforated(adart) &&
+        (get_original_map().template is_free<2>(adart) ||
+         get_original_map().is_perforated
+         (get_original_map().template beta<2>(adart))))
+    {
+      if (get_original_map().template is_free<2>(adart) ||
+          adart<get_original_map().template beta<2>(adart))
+      {
+        const std::pair<Dart_handle, Dart_handle>&
+          p=m_paths.find(adart)->second;
+        return flip?get_local_map().template beta<2>(p.first):p.first;
+      }
+      else
+      {
+        const std::pair<Dart_handle, Dart_handle>&
+            p=m_paths.find(get_original_map().template beta<2>(adart))->second;
+        return flip?p.first:get_local_map().template beta<2>(p.first);
+      }
+    }
+
+    CGAL_assertion(!get_original_map().is_perforated(adart) ||
+                   (!get_original_map().template is_free<2>(adart) &&
+                    !get_original_map().is_perforated
+                    (get_original_map().template beta<2>(adart))));
+
+    if (get_original_map().template is_free<2>(adart) ||
+        adart<get_original_map().template beta<2>(adart))
+    {
+      const std::pair<Dart_handle, Dart_handle>&
+        p=m_paths.find(adart)->second;
+      return flip?
+            (isquadrangulation?get_local_map().template beta<2>(p.second):p.second):
+            p.first;
+    }
+
+    const std::pair<Dart_handle, Dart_handle>&
+        p=m_paths.find(get_original_map().template beta<2>(adart))->second;
+
+    return flip?
+          p.first:
+          (isquadrangulation?get_local_map().template beta<2>(p.second):p.second);
+  }
+
+  /** @return the second dart of the length 2 path associated with adart.
+   *  @param adart a dart in the original map
+   *  @param flip to return the first dart of beta2(adart)
+   *  @param isquadrangulation true if the minimal map is a quadrangulation,
+   *                           false otherwise (special case of a torus)
+   *  @pre edge containing adart should be incident to at least one non
+   *       perforated face.
+   */
+  Dart_const_handle get_second_dart_of_the_path(Original_dart_const_handle adart,
+                                                bool flip=false,
+                                                bool isquadrangulation=true) const
+  {
+    CGAL_assertion(is_edge_has_path(adart));
+    CGAL_assertion(!get_original_map().is_perforated(adart) ||
+                   (!get_original_map().template is_free<2>(adart) &&
+                    !get_original_map().is_perforated
+                    (get_original_map().template beta<2>(adart))));
+
+    if (get_original_map().template is_free<2>(adart) ||
+        adart<get_original_map().template beta<2>(adart))
+    {
+      const std::pair<Dart_handle, Dart_handle>&
+          p=m_paths.find(adart)->second;
+      return flip?
+            (isquadrangulation?get_local_map().template beta<2>(p.first):p.first):
+            p.second;
+    }
+
+    const std::pair<Dart_handle, Dart_handle>&
+        p=m_paths.find(get_original_map().template beta<2>(adart))->second;
+    return flip?
+          p.second:
+          (isquadrangulation?get_local_map().template beta<2>(p.first):p.first);
+  }
+
+  /// @return the first dart of the path, direct version without taking into
+  /// account possible flip and the two case of torus/quadrangulation.
+  Dart_handle get_first_dart_of_the_path_direct(Original_dart_const_handle adart) const
+  {
+    CGAL_assertion(is_edge_has_path(adart));
+
+    if (get_original_map().template is_free<2>(adart) ||
+        adart<get_original_map().template beta<2>(adart))
+    { return m_paths.find(adart)->second.first; }
+    return m_paths.find(get_original_map().template beta<2>(adart))->second.second;
+  }
+
+  void set_first_dart_of_the_path(Original_dart_const_handle adart,
+                                  Dart_handle d)
+  {
+    CGAL_assertion(is_edge_has_path(adart));
+
+    if (get_original_map().template is_free<2>(adart) ||
+        adart<get_original_map().template beta<2>(adart))
+    { m_paths.find(adart)->second.first=d; }
+    else
+    { m_paths.find(get_original_map().template beta<2>(adart))->second.second=d; }
+  }
+
+  void set_second_dart_of_the_path(Original_dart_const_handle adart,
+                                   Dart_handle d)
+  {
+    CGAL_assertion(is_edge_has_path(adart));
+
+    if (get_original_map().template is_free<2>(adart) ||
+        adart<get_original_map().template beta<2>(adart))
+    { m_paths.find(adart)->second.second=d; }
+    else { m_paths.find(adart)->second.first=d; }
+  }
+
+  bool local_map_is_a_torus() const
+  {
+    if (get_local_map().number_of_darts()!=4)
+    { return false; }
+    return (get_local_map().number_of_marked_darts(m_mark_perforated)==0);
+  }
+
+  /// @return true iff the perforated faces are correctly marked
+  /// (i.e. either fully marked or fully unmarked).
+  bool perforated_faces_correctly_marked() const
+  {
+    for (typename Local_map::Dart_range::const_iterator
+         it=get_local_map().darts().begin(),
+         itend=get_local_map().darts().end(); it!=itend; ++it)
+    {
+      CGAL_assertion(get_local_map().is_without_boundary(1));
+      if (get_local_map().is_marked(it, m_mark_perforated))
+      {
+        if (!get_local_map().template is_whole_cell_marked<2>
+            (it, m_mark_perforated))
+        {
+          std::cout<<"[ERROR] perforated_faces_correctly_marked: it is marked, "
+                     <<"but its face is not fully marked."<<std::endl;
+          return false;
+        }
+      }
+      else
+      {
+        if (!get_local_map().template is_whole_cell_unmarked<2>
+            (it, m_mark_perforated))
+        {
+          std::cout<<"[ERROR] perforated_faces_correctly_marked: it is not marked, "
+                     <<"but its face is not fully unmarked."<<std::endl;
+          return false;
+        }
+      }
+    }
+    return true;
+  }
+
+  /// Test if m_paths are valid, i.e.:
+  /// 1) all the darts of get_original_map() that do not belong to T are
+  ///    associated with a pair of darts;
+  /// 2) all the darts of m_paths belong to m_local_map;
+  /// 3) the origin of the second dart of the pair is the extremity of the
+  ///    first dart.
+  /// 4) all the darts of m_local_map are not free (both for beta 1 and 2)
+  /// 5) The two darts in a pair are different
+  bool are_paths_valid() const
+  {
+    if (m_paths.empty()) { return true; }
+
+    bool res=true;
+    for (auto it=get_original_map().darts().begin(),
+           itend=get_original_map().darts().end(); it!=itend; ++it)
+    {
+
+      if (!is_contracted(it))
+      {
+        if (!is_edge_has_path(it))
+        {
+          std::cout<<"ERROR: an edge that is not contracted "
+                   <<"has no associated path."
+                   <<"BTW is_marked(it)="
+                   <<get_original_map().is_marked(it, m_mark_T)<<std::endl;
+          res=false;
+        }
+      }
+      else
+      {
+        if (is_edge_has_path(it))
+        {
+          std::cout<<"ERROR: an edge that is contracted"
+                   <<" has an associated path."
+                   <<"BTW is_marked(it)="
+                   <<get_original_map().is_marked(it, m_mark_T)<<std::endl;
+          res=false;
+        }
+      }
+    }
+
+    for (auto it=get_local_map().darts().begin(),
+           itend=get_local_map().darts().end(); it!=itend; ++it)
+    {
+      if (get_local_map().is_free(it, 1))
+      {
+        std::cout<<"ERROR: a dart of the quandrangulated map is 1-free"
+                 <<std::endl;
+        res=false;
+      }
+      if (get_local_map().is_free(it, 2))
+      {
+        std::cout<<"ERROR: a dart of the quandrangulated map is 2-free"
+                 <<std::endl;
+        res=false;
+      }
+    }
+
+    for (auto it=m_paths.begin(); it!=m_paths.end(); ++it)
+    {
+      if (!get_local_map().is_dart_used(it->second.first))
+      {
+        std::cout<<"ERROR: first dart in m_paths does not exist anymore in m_local_map."
+                 <<std::endl;
+        res=false;
+      }
+      else if (!get_local_map().darts().owns(it->second.first))
+      {
+        std::cout<<"ERROR: first dart in m_paths does not belong to m_local_map."
+                 <<std::endl;
+        res=false;
+      }
+
+      if (!edge_path_has_only_one_dart(it->first))
+      {
+        if (!get_local_map().is_dart_used(it->second.second))
+        {
+          std::cout<<"ERROR: second dart in m_paths does not exist anymore in m_local_map."
+                  <<std::endl;
+          res=false;
+        }
+        else if (!get_local_map().darts().owns(it->second.second))
+        {
+          std::cout<<"ERROR: second dart in m_paths does not belong to m_local_map."
+                  <<std::endl;
+          res=false;
+        }
+      }
+
+      if (it->second.first==it->second.second)
+      {
+        std::cout<<"ERROR: two darts in the same pair are equal."
+                 <<std::endl;
+        res=false;
+      }
+    }
+
+    for (auto it=get_original_map().darts().begin(),
+           itend=get_original_map().darts().end(); it!=itend; ++it)
+    {
+      if (!is_contracted(it))
+      {
+        Dart_const_handle d1=get_first_dart_of_the_path(it);
+        if (d1==nullptr)
+        {
+          std::cout<<"ERROR: an edge is associated with a null dart in m_paths"
+                   <<" for its first dart."<<std::endl;
+          res=false;
+        }
+        else if (!edge_path_has_only_one_dart(it))
+        {
+          Dart_const_handle d2=get_second_dart_of_the_path(it);
+          if (d2==nullptr)
+          {
+            std::cout<<"ERROR: an edge is associated with a null dart in m_paths"
+                     <<" for its second dart."<<std::endl;
+            res=false;
+          }
+          else
+          {
+            Dart_const_handle dd1=get_local_map().other_extremity(d1);
+            CGAL_assertion(dd1!=NULL);
+            if (!CGAL::belong_to_same_cell<Local_map,0>
+                (get_local_map(), dd1, d2))
+            {
+              std::cout<<"ERROR: the two darts in a path are not consecutive."
+                      <<std::endl;
+              res=false;
+            }
+          }
+        }
+      }
+    }
+
+    return res;
+  }
+
+protected:
+  /// The original map (the mesh seen as a 2-map)
+  const typename Get_map<Mesh, Mesh>::storage_type m_original_map;
+  Local_map m_local_map; /// the reduced map
+  TPaths m_paths; /// Pair of edges associated with each edge of get_original_map()
+                  /// (except the edges that belong to the spanning tree T).
+  Original_size_type m_mark_T;   /// mark each edge of get_original_map() that belong to the spanning tree T
+  Original_size_type m_mark_L;   /// mark each edge of get_original_map() that belong to the dual spanning tree L
+  size_type m_mark_perforated; /// mark each edge of m_local_map that bounds a hole
+
+#ifdef CGAL_PWRLE_TURN_V2
+  TDartIds m_dart_ids; /// Ids of each dart of the transformed map, between 0 and n-1 (n being the number of darts)
+                       /// so that darts between 0...(n/2)-1 belong to the same vertex and
+                       /// d1=beta<1, 2>(d0), d2=beta<1, 2>(d1)...
+                       /// The same for darts between n/2...n-1 for the second vertex
+                       /// Thanks to these ids, we can compute in constant time the positive and
+                       /// negative turns between two consecutive darts
+#endif // CGAL_PWRLE_TURN_V2
+};
+
+} // namespace internal
+} // namespace Surface_mesh_topology
+} // namespace CGAL
+
+#endif // CGAL_MINIMAL_QUADRANGULATION_H //
+// EOF //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_topology/internal/Path_generators.h 5.1-1/include/CGAL/Surface_mesh_topology/internal/Path_generators.h
--- 5.0.3-1/include/CGAL/Surface_mesh_topology/internal/Path_generators.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_topology/internal/Path_generators.h	2020-09-08 12:15:46.000000000 +0000
@@ -0,0 +1,231 @@
+// Copyright (c) 2019 CNRS and LIRIS' Establishments (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_topology/include/CGAL/Surface_mesh_topology/internal/Path_generators.h $
+// $Id: Path_generators.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
+//
+#ifndef CGAL_PATH_GENERATORS_H
+#define CGAL_PATH_GENERATORS_H 1
+
+#include <CGAL/license/Surface_mesh_topology.h>
+
+#include <CGAL/Random.h>
+#include <unordered_set>
+#include <unordered_map>
+#include <cstdlib>
+
+namespace CGAL {
+namespace Surface_mesh_topology {
+namespace internal {
+
+template<typename Path>
+void create_braket_positive(Path& p, std::size_t length, CGAL::Random& random,
+                            bool update_isclosed=true)
+{
+  if (p.is_empty())
+  { p.initialize_random_starting_dart(random, false); }
+
+  p.extend_positive_turn(1, false);
+  p.extend_straight_positive(length, false);
+  p.extend_positive_turn(1, false);
+  if (update_isclosed) { p.update_is_closed(); }
+}
+
+template<typename Path>
+void create_braket_positive(Path& p, std::size_t length,
+                            bool update_isclosed=true)
+{
+  CGAL::Random random;
+  create_braket_positive(p, length, random, update_isclosed);
+}
+
+template<typename Path>
+void create_braket_negative(Path& p, std::size_t length, CGAL::Random& random,
+                            bool update_isclosed=true)
+{
+  if (p.is_empty())
+  { p.initialize_random_starting_dart(random, false); }
+
+  p.extend_negative_turn(1, false);
+  p.extend_straight_negative(length, false);
+  p.extend_negative_turn(1, false);
+  if (update_isclosed) { p.update_is_closed(); }
+}
+
+template<typename Path>
+void create_braket_negative(Path& p, std::size_t length,
+                            bool update_isclosed=true)
+{
+  CGAL::Random random;
+  create_braket_negative(p, length, random, update_isclosed);
+}
+
+template<typename Path>
+void generate_random_positive_bracket(Path& path,
+                                      std::size_t nb1,
+                                      std::size_t nb2, std::size_t nb3,
+                                      CGAL::Random& random)
+{
+  path.clear();
+  path.initialize_random_starting_dart(random);
+  path.extend_straight_positive(nb1-1);
+  create_braket_positive(path, nb2);
+  path.extend_straight_positive(nb3);
+  path.generate_random_path(random.get_int(0, 15), random);
+}
+
+template<typename LCC>
+typename LCC::Dart_const_handle
+generate_random_connected_set_of_faces(const LCC& lcc, std::size_t nb,
+                                       CGAL::Random& random,
+                                       std::unordered_set
+                                       <typename LCC::Dart_const_handle>& set,
+                                       typename LCC::size_type amark)
+{
+  set.clear();
+  if (lcc.is_empty()) { return NULL; }
+
+  std::unordered_map<std::size_t, typename LCC::Dart_const_handle> border_faces;
+
+  std::size_t index=static_cast<std::size_t>
+    (random.get_int(0, static_cast<int>(lcc.darts().capacity())));
+  while (!lcc.darts().is_used(index))
+  {
+    ++index;
+    if (index==lcc.darts().capacity()) { index=0; }
+  }
+
+  typename LCC::Dart_const_handle dh1=lcc.darts().iterator_to(lcc.darts()[index]);
+  border_faces[0]=dh1;
+  set.insert(dh1);
+  lcc.template mark_cell<2>(dh1, amark);
+
+  for (std::size_t i=1; i<nb; ++i)
+  {
+    std::size_t facenumber=static_cast<std::size_t>
+                           (random.get_int
+                            (0, static_cast<int>(border_faces.size())));
+    int nbborder=0;
+
+    typename LCC::Dart_const_handle dh1_init=border_faces[facenumber];
+    dh1=dh1_init;
+    do
+    {
+      if (!lcc.template is_free<2>(dh1) &&
+          !lcc.is_marked(lcc.template beta<2>(dh1), amark))
+      { ++nbborder; }
+      dh1=lcc.template beta<1>(dh1);
+    }
+    while (dh1!=dh1_init);
+
+    while(lcc.template is_free<2>(dh1) ||
+          lcc.is_marked(lcc.template beta<2>(dh1), amark))
+    { dh1=lcc.template beta<1>(dh1); }
+
+    std::size_t dartnumber=static_cast<std::size_t>(random.get_int(0, nbborder));
+    for (std::size_t j=0; j<dartnumber;)
+    {
+      if (!lcc.template is_free<2>(dh1) &&
+          !lcc.is_marked(lcc.template beta<2>(dh1), amark))
+      { ++j; }
+      dh1=lcc.template beta<1>(dh1);
+      while(lcc.template is_free<2>(dh1) ||
+            lcc.is_marked(lcc.template beta<2>(dh1), amark))
+      { dh1=lcc.template beta<1>(dh1); }
+    }
+
+    // Here we have a new face
+    set.insert(lcc.template beta<2>(dh1));
+    lcc.template mark_cell<2>(lcc.template beta<2>(dh1), amark);
+
+    // We add it in the list of borders faces
+    border_faces[border_faces.size()]=lcc.template beta<2>(dh1);
+
+    // Then we update the list of border faces (because some of them could be
+    // no more border due to the adding of the new face)
+    std::unordered_map<std::size_t, typename LCC::Dart_const_handle>
+        border_faces_new;
+    for (typename std::unordered_map<std::size_t,
+         typename LCC::Dart_const_handle>::iterator it=border_faces.begin(),
+         itend=border_faces.end(); it!=itend; ++it)
+    {
+      bool isborder=false;
+      dh1=it->second;
+      do
+      {
+        if (!lcc.template is_free<2>(dh1) &&
+            !lcc.is_marked(lcc.template beta<2>(dh1), amark))
+        { isborder=true; }
+        else
+        { dh1=lcc.template beta<1>(dh1); }
+      }
+      while(!isborder && dh1!=it->second);
+      if (isborder)
+      { border_faces_new[border_faces_new.size()]=dh1; }
+    }
+    std::swap(border_faces, border_faces_new);
+
+    if (border_faces.size()==0)
+    { return NULL; }
+  }
+
+  assert (border_faces.size()!=0);
+  typename LCC::Dart_const_handle dhres=border_faces[0];
+  while(lcc.template is_free<2>(dhres) ||
+        lcc.is_marked(lcc.template beta<2>(dhres), amark))
+  { dhres=lcc.template beta<1>(dhres); }
+
+  return dhres;
+}
+
+template<typename Path>
+void generate_random_closed_path(Path& p, std::size_t nb,
+                                 CGAL::Random& random)
+{
+  std::unordered_set<typename Path::Map::Dart_const_handle> faces;
+  typename Path::Map::size_type amark=p.get_map().get_new_mark();
+
+  typename Path::Map::Dart_const_handle dhi=
+      generate_random_connected_set_of_faces(p.get_map(), nb, random,
+                                             faces, amark);
+
+  if (dhi==NULL)
+  {
+    p.get_map().free_mark(amark);
+    return;  // We have selected all the faces.
+  }
+
+  typename Path::Map::Dart_const_handle dh=dhi;
+  do
+  {
+    CGAL_assertion(p.get_map().template is_free<2>(dh) ||
+                   !p.get_map().is_marked(p.get_map().template beta<2>(dh), amark));
+    p.push_back(dh, false);
+    dh=p.get_map().template beta<1>(dh);
+    while(!p.get_map().template is_free<2>(dh) &&
+          p.get_map().is_marked(p.get_map().template beta<2>(dh), amark))
+    { dh=p.get_map().template beta<2, 1>(dh); }
+  }
+  while(dh!=dhi);
+
+  for (typename std::template unordered_set<typename Path::Map::Dart_const_handle>::iterator
+       it=faces.begin(), itend=faces.end(); it!=itend; ++it)
+  { p.get_map().template unmark_cell<2>(*it, amark); }
+
+  p.get_map().free_mark(amark);
+
+  p.update_is_closed();
+  CGAL_assertion(p.is_closed());
+}
+
+} // namespace internal
+} // namespace Surface_mesh_topology
+} // namespace CGAL
+
+#endif // CGAL_PATH_GENERATORS_H //
+// EOF //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_topology/internal/Path_on_surface_with_rle.h 5.1-1/include/CGAL/Surface_mesh_topology/internal/Path_on_surface_with_rle.h
--- 5.0.3-1/include/CGAL/Surface_mesh_topology/internal/Path_on_surface_with_rle.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_topology/internal/Path_on_surface_with_rle.h	2020-09-08 12:15:46.000000000 +0000
@@ -0,0 +1,1583 @@
+// Copyright (c) 2019 CNRS and LIRIS' Establishments (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_topology/include/CGAL/Surface_mesh_topology/internal/Path_on_surface_with_rle.h $
+// $Id: Path_on_surface_with_rle.h f4a6f06 2020-04-07T15:02:33+02:00 Guillaume Damiand
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
+//
+#ifndef CGAL_PATH_ON_SURFACE_WITH_RLE_H
+#define CGAL_PATH_ON_SURFACE_WITH_RLE_H 1
+
+#include <CGAL/license/Surface_mesh_topology.h>
+
+#include <list>
+#include <utility>
+#include <iostream>
+#include <iterator>
+#include <vector>
+#include <set>
+#include <CGAL/assertions.h>
+#include <unordered_map>
+#include <unordered_set>
+
+namespace CGAL {
+namespace Surface_mesh_topology {
+
+template<typename Map_>
+class Path_on_surface;
+
+template<typename Mesh_>
+class Minimal_quadrangulation;
+
+namespace internal {
+
+// A flat is a sequence of darts given by its two extremities: begin and end,
+// with +2 turns (if length>0) or -2 turns (if length<0).
+// length==0 => begin==end.
+template<typename Map_>
+class CFlat
+{
+  typedef Map_                            Map;
+  typedef typename Map::Dart_const_handle Dart_const_handle;
+  typedef CFlat<Map>                      Self;
+
+public:
+  CFlat(Dart_const_handle dh) : begin(dh), end(dh), length(0)
+  {}
+
+  CFlat(Dart_const_handle dh1, Dart_const_handle dh2, int l) :
+    begin(dh1), end(dh2), length(l)
+  {}
+
+  bool operator==(const Self& other) const
+  { return begin==other.begin && end==other.end && length==other.length; }
+
+  bool operator!=(const Self& other) const
+  { return !(operator==(other)); }
+
+  friend std::ostream& operator<< (std::ostream& os, const Self& flat)
+  {
+    os<<"["<<&*(flat.begin)<<", "<<&*(flat.end)<<" (l="<<flat.length<<")]";
+    return os;
+  }
+
+  Dart_const_handle begin, end;
+  int length; // Length of the flat, positive flat if >0, negative flat if <0
+};
+
+// Small wrapper allowing to use directly a combinatorial map as if it is a
+// minimal quadrangulation.
+template<class Map_>
+class Light_MQ  // MQ for minimal quadrangulation
+{
+public:
+  typedef Map_                              Local_map;
+  typedef Map_                              Mesh;
+  typedef typename Map_::Dart_const_handle  Dart_const_handle;
+
+  Light_MQ(const Local_map& m): m_map(m)
+  {}
+
+  const Local_map& get_local_map() const
+  { return m_map; }
+
+  std::size_t positive_turn(Dart_const_handle d1, Dart_const_handle d2) const
+  { return m_map.positive_turn(d1, d2); }
+
+  std::size_t negative_turn(Dart_const_handle d1, Dart_const_handle d2) const
+  { return m_map.negative_turn(d1, d2); }
+
+protected:
+  const Local_map& m_map;
+};
+
+template<typename MQ> // MQ for minimal quadrangulation
+class Path_on_surface_with_rle
+{
+public:
+  typedef Path_on_surface_with_rle<MQ>                       Self;
+  typedef typename MQ::Local_map                             Map;
+  typedef typename MQ::Mesh                                  Mesh;
+  typedef typename Map::Dart_handle                          Dart_handle;
+  typedef typename Map::Dart_const_handle                    Dart_const_handle;
+  typedef CFlat<Map>                                         Flat;
+  typedef std::list<Flat>                                    List_of_flats;
+  typedef typename List_of_flats::iterator                   List_iterator;
+  // TODO typedef typename List_of_dart_length::const_iterator List_const_iterator;
+
+  friend class Path_on_surface<Map>;
+
+  struct List_iterator_hash
+  {
+    std::size_t operator() (const List_iterator& lit) const
+    {
+      return std::hash<const void*>()(&*lit);
+    }
+  };
+
+  typedef std::unordered_set<List_iterator, List_iterator_hash> Set_of_it;
+
+#ifdef CGAL_PWRLE_TURN_V2
+    typedef std::unordered_map<Dart_const_handle, std::size_t> TDartIds;
+#endif //CGAL_PWRLE_TURN_V2
+
+  /// Constructor
+  Path_on_surface_with_rle(const MQ& aMQ
+                         #ifdef CGAL_PWRLE_TURN_V2
+                           , const TDartIds & darts_ids
+                         #endif //CGAL_PWRLE_TURN_V2
+                           )
+    : m_MQ(aMQ),
+      m_is_closed(false),
+      m_length(0),
+      m_use_only_positive(false),
+      m_use_only_negative(false)
+  #ifdef CGAL_PWRLE_TURN_V2
+    , m_darts_ids(darts_ids)
+  #endif //CGAL_PWRLE_TURN_V2
+  {}
+
+  /// Creates a Path_on_surface_with_rle from a Path_on_surface.
+  /// If use_only_positive, consider only positive flats and not negative ones.
+  /// If use_only_negative, consider only negative flats and not positive ones.
+  /// If both are false, consider both positive and negative flats.
+  /// Both cannot be true at the same time.
+  /// Note that for a minimal surface of genus>=2, we cannot have both -2 and
+  /// +2 as turn, and thus these parameters are useless.
+  /// However, this case can occured for our unit tests on the cube, this is
+  /// the reason of these parameters.
+  Path_on_surface_with_rle(const MQ& aMQ, const Path_on_surface<Map>& apath,
+                           bool use_only_positive=false,
+                           bool use_only_negative=false)
+  : m_MQ(aMQ),
+    m_is_closed(apath.is_closed()),
+    m_length(0),
+    m_use_only_positive(use_only_positive),
+    m_use_only_negative(use_only_negative)
+  {
+    if (apath.is_empty()) return;
+
+    std::size_t i=0, starti=0;
+    bool positive_flat=false;
+    bool negative_flat=false;
+
+    if (apath.is_closed())
+    {
+      if (!use_only_negative && apath.next_positive_turn(i)==2)
+      { positive_flat=true; negative_flat=false; }
+      else if (!use_only_positive && apath.next_negative_turn(i)==2)
+      { positive_flat=false; negative_flat=true; }
+
+      while ((positive_flat && apath.next_positive_turn(i)==2) ||
+             (negative_flat && apath.next_negative_turn(i)==2))
+      {
+        i=apath.next_index(i);
+        if (i==0) // Case of a closed path, made of only one flat part.
+        {
+          m_path.push_back(Flat(apath.real_front(), apath.real_back(),
+                                (positive_flat?(static_cast<int>(apath.length()-1)):
+                                 -(static_cast<int>(apath.length()-1)))));
+          m_length=apath.length();
+          CGAL_assertion(is_valid());
+          return;
+        }
+      }
+      // Here i is the last dart of a flat
+      i=apath.next_index(i); // Now we are sure that i is the beginning of a flat
+    }
+
+    starti=i;
+    do
+    {
+      // Here dart i is the beginning of a flat part (maybe of length 0)
+        push_back(apath.get_ith_real_dart(i), false);
+        i=apath.next_index(i);
+    }
+    while(i<apath.length() && i!=starti);
+
+    CGAL_assertion(is_valid(true));
+  }
+
+  void swap(Self& p2)
+  {
+    if (this!=&p2)
+    {
+      CGAL_assertion(&get_map()==&(p2.get_map()));
+#ifdef CGAL_PWRLE_TURN_V2
+      CGAL_assertion(&m_darts_ids==&(p2.m_darts_ids));
+#endif // CGAL_PWRLE_TURN_V2
+      m_path.swap(p2.m_path);
+      std::swap(m_is_closed, p2.m_is_closed);
+      std::swap(m_length, p2.m_length);
+      std::swap(m_use_only_negative, p2.m_use_only_negative);
+      std::swap(m_use_only_positive, p2.m_use_only_positive);
+    }
+  }
+
+  Self& operator=(const Self& other)
+  {
+    CGAL_assertion(&get_map()==&(other.get_map()));
+#ifdef CGAL_PWRLE_TURN_V2
+    CGAL_assertion(&m_darts_ids==&(other.m_darts_ids));
+#endif // CGAL_PWRLE_TURN_V2
+    if (this!=&other)
+    {
+      m_path=other.m_path;
+      m_is_closed=other.m_is_closed;
+      m_length=other.m_length;
+      m_use_only_negative=other.m_use_only_negative;
+      m_use_only_positive=other.m_use_only_positive;
+    }
+    return *this;
+  }
+
+  Self& operator+=(Self& other)
+  {
+    // Be careful to the special case when *this==other.
+    // This is the reason of the litend computed with std::prev.
+    for (List_iterator lit=other.m_path.begin(),
+         litend=std::prev(other.m_path.end()); lit!=litend; ++lit)
+    { m_path.push_back(*lit); }
+    m_path.push_back(other.m_path.back()); // Last element
+    update_is_closed();
+    // TODO: List of flat should be simplify when possible
+    // TODO2: what to do if different values for m_use_only_positive and m_use_only_negative ??
+    //    (probably nothing since these bools are used only for our tests)
+    return *this;
+  }
+
+  Self operator+(const Self& other) const
+  {
+    Self res=*this;
+    res+=other;
+    return res;
+  }
+
+  /// @return true if this path is equal to other path. For closed paths, test
+  ///         all possible starting darts.
+  bool operator==(Self& other)
+  {
+    if (is_closed()!=other.is_closed() ||
+        length()!=other.length())
+      return false;
+
+    if (is_empty() && other.is_empty())
+    { return true; }
+
+    // Note that we need to transform the Path_on_surface_with_rle into
+    // the correspondings Path_on_surface, because a same path can have several
+    // different rle representations.
+    Path_on_surface<Map> p1(*this);
+    Path_on_surface<Map> p2(other);
+    return p1==p2;
+  }
+
+  bool operator!=(Self&  other)
+  { return !(operator==(other)); }
+
+  /// @return true iff the path is empty
+  bool is_empty() const
+  { return m_path.empty(); }
+
+  /// @return the length of the path, i.e. its number of darts
+  std::size_t length() const
+  { return m_length; }
+
+  /// @return the number of darts in the double linked list.
+  /// note that size_of_list()<=length().
+  std::size_t size_of_list() const
+  { return m_path.size(); }
+
+  /// @return true iff the path is closed (update after each path modification).
+  bool is_closed() const
+  { return m_is_closed; }
+
+  /// @return the underlying map.
+  const Map& get_map() const
+  { return m_MQ.get_local_map(); }
+
+  /// clear the path.
+  void clear()
+  {
+    m_path.clear();
+    m_is_closed=false;
+    m_length=0;
+  }
+
+  /// @return true if the given flat is valid
+  bool is_flat_valid(const List_iterator& flat) const
+  {
+    CGAL_assertion(is_valid_iterator(flat));
+    Dart_const_handle dhend=flat->begin;
+    int nb=0;
+    while(nb!=flat->length)
+    {
+      if (flat->length>0)
+      { dhend=get_map().template beta<1, 2, 1>(dhend); ++nb; }
+      else
+      { dhend=get_map().template beta<2, 0, 2, 0, 2>(dhend); --nb; }
+    }
+
+    return dhend==flat->end;
+  }
+
+  /// Display the given flat
+  void display_flat(std::ostream& os, const List_iterator& flat)
+  {
+    CGAL_assertion(is_valid_iterator(flat));
+    os<<"["<<get_map().darts().index(flat->begin)<<", "
+      <<get_map().darts().index(flat->end)<<" (l="<<flat->length<<")]";
+  }
+
+  /// @return true iff the path is valid; i.e. a sequence of flats two by
+  ///              two adjacent.
+  /// if test_minimal is true, test that there is no two consecutive flats
+  /// that can be merged. If test_minimal is false, this test is not done.
+  bool is_valid(bool test_minimal=true)
+  {
+    if (is_empty()) { return !is_closed(); } // an empty past is not closed
+
+    unsigned int nbdarts=0,i=0;
+    Dart_const_handle prev=(is_closed()?back():nullptr); // Last dart of the path
+    for (auto it=m_path.begin(); it!=m_path.end(); ++it)
+    {
+      if (prev!=nullptr && !get_map().template belong_to_same_cell<0>
+          (get_map().template beta<1>(prev), begin_of_flat(it)))
+      {
+        std::cerr<<"ERROR: The path is not valid: flat in position "<<i
+                 <<" does not follow the previous dart."<<std::endl;
+        return false;
+      }
+
+      if (!is_flat_valid(it))
+      {
+        display();
+        std::cout<<"ERROR: The path is not valid: flat at position "<<i
+                <<" with length "<<flat_length(it)
+               <<" is not correct: its end does not"
+              <<" correspond to the flat."<<std::endl;
+        return false;
+      }
+
+      if (test_minimal && can_merge_with_next_flat(it))
+      {
+        std::cout<<"ERROR: The path is not valid: flat at position "<<i
+                <<" with length "<<flat_length(it)<<" is not correct: it can be merged "
+               <<"with the next flat with length "
+              <<flat_length(next_iterator(it))<<" - turns between the two flats = +"
+             <<next_positive_turn(it)<<" -"<<next_negative_turn(it)<<std::endl;
+        return false;
+      }
+      prev=end_of_flat(it); // end of the previous flat
+      nbdarts+=std::abs(flat_length(it))+1; // Number of darts of the flat
+      ++i;
+    }
+
+    if (is_closed())
+    {
+      if (prev==nullptr)
+      {
+        std::cout<<"ERROR: The path is not valid: it is empty and closed."
+                <<std::endl;
+        return false;
+      }
+      if (!get_map().template belong_to_same_cell<0>
+          (get_map().template beta<1>(prev), begin_of_flat(m_path.begin())))
+      {
+        std::cerr<<"ERROR: The path is not valid: first flat "
+                 <<" does not follow the last dart for a closed path."<<std::endl;
+        return false;
+      }
+    }
+
+    if (length()!=nbdarts)
+    {
+      std::cerr<<"ERROR: The path is not valid: store length=="<<length()
+               <<" different of real length=="<<nbdarts<<std::endl;
+      return false;
+    }
+
+    return true;
+  }
+
+  /// For debugging purpose, test if 'it' is a valid iterator.
+  bool is_valid_iterator(const List_iterator& ittotest) const
+  {
+    if (ittotest==m_path.end()) { return false; }
+    return true;
+    // Assert too long; uncomment in case of bug.
+    /* for (auto it=m_path.begin(); it!=m_path.end(); ++it)
+    {
+      if (it==ittotest) { return true; }
+    }
+    return false; */
+  }
+
+  /// @return the first dart of the path
+  /// @pre !is_empty()
+  Dart_const_handle front()
+  {
+    CGAL_assertion(!is_empty());
+    return begin_of_flat(m_path.begin());
+  }
+
+  /// @return the last dart of the path
+  /// @pre !is_empty()
+  Dart_const_handle back()
+  {
+    CGAL_assertion(!is_empty());
+    return end_of_flat(std::prev(m_path.end()));
+  }
+
+  /// @return true iff df can be added at the end of the path.
+  bool can_be_pushed(Dart_const_handle dh)
+  {
+    // This assert is too long
+    // CGAL_assertion(get_map().darts().owns(dh));
+    if (is_empty()) return true;
+
+    return get_map().template belong_to_same_cell<0>
+      (get_map().template beta<1>(back()), dh);
+  }
+
+  /// Add the given dart at the end of this path.
+  /// @pre can_be_pushed(dh)
+  void push_back(Dart_const_handle dh, bool update_isclosed=true)
+  {
+    CGAL_assertion(dh!=Map::null_handle);
+    // This assert is too long, it is tested in the is_valid method.
+    // CGAL_assertion(can_be_pushed(dh));
+
+    List_iterator itlast=m_path.end();
+
+    bool positive_flat, negative_flat;
+    if (is_empty() ||
+        !is_prev_flat_can_be_extended_at_end(itlast, dh,
+                                             positive_flat, negative_flat))
+    { m_path.push_back(Flat(dh)); } // Create a new flat
+    else
+    {
+      itlast=std::prev(itlast);
+      set_end_of_flat(itlast, dh); // Move the last dart of the last flat
+      if (positive_flat && flat_length(itlast)>=0)
+      {
+        set_flat_length(itlast, flat_length(itlast)+1); // Increment the length of the flat
+      }
+      else
+      {
+        CGAL_assertion(negative_flat && flat_length(itlast)<=0);
+        set_flat_length(itlast, flat_length(itlast)-1); // Increment the length of the flat
+      }
+    }
+
+    ++m_length;
+    if (update_isclosed) { update_is_closed(); }
+  }
+
+  // Update m_is_closed to true iff the path is closed (i.e. the second
+  //   extremity of the last dart of the path is the same vertex than the one
+  //   of the first dart of the path).
+  void update_is_closed()
+  {
+    if (is_empty()) { m_is_closed=false; }
+    else
+    {
+      Dart_const_handle pend=get_map().other_extremity(back());
+      if (pend==Map::null_handle) { m_is_closed=false; }
+      else
+      { m_is_closed=get_map().template belong_to_same_cell<0>(front(), pend); }
+    }
+  }
+
+  /// @return true iff there is a flat after it
+  bool next_flat_exist(const List_iterator& it) const
+  {
+    CGAL_assertion(it!=m_path.end());
+    return !is_empty() && (is_closed() || std::next(it)!=m_path.end());
+  }
+
+  /// @return true iff there is a flat before it
+  bool prev_flat_exist(const List_iterator& it) const
+  {
+    return !is_empty() && (is_closed() || it!=m_path.begin());
+  }
+
+  void advance_iterator(List_iterator& it)
+  {
+    CGAL_assertion(it!=m_path.end());
+    it=std::next(it);
+    if (it==m_path.end())
+    {
+      if (is_closed())
+      { it=m_path.begin(); } // Here the path is closed, and it is the last element of the list
+    }
+  }
+
+  void retreat_iterator(List_iterator& it)
+  {
+    if (it==m_path.begin())
+    {
+      if (is_closed())
+      { it=m_path.end(); }
+      else { return; }
+    }
+    it=std::prev(it);
+  }
+
+  List_iterator next_iterator(const List_iterator& it)
+  {
+    List_iterator res=it;
+    advance_iterator(res);
+    return res;
+  }
+
+  List_iterator prev_iterator(const List_iterator& it)
+  {
+    List_iterator res=it;
+    retreat_iterator(res);
+    return res;
+  }
+
+  /// @return the beginning of the flat
+  Dart_const_handle begin_of_flat(const List_iterator& it)
+  {
+    CGAL_assertion(is_valid_iterator(it));
+    return it->begin;
+  }
+
+  /// @return the end of the flat
+  Dart_const_handle end_of_flat(const List_iterator& it)
+  {
+    CGAL_assertion(is_valid_iterator(it));
+    return it->end;
+  }
+
+  /// @return the length of the flat
+  int flat_length(const List_iterator& it)
+  {
+    CGAL_assertion(is_valid_iterator(it));
+    return it->length;
+  }
+
+  void set_flat_length(const List_iterator& it, int i)
+  {
+    CGAL_assertion(is_valid_iterator(it));
+    it->length=i;
+  }
+
+  void decrease_flat_length(const List_iterator& it)
+  {
+    CGAL_assertion(is_valid_iterator(it));
+    CGAL_assertion(flat_length(it)!=0);
+    if (flat_length(it)>0) { --(it->length); }
+    else                   { ++(it->length); }
+  }
+
+  void increase_flat_length(const List_iterator& it)
+  {
+    CGAL_assertion(is_valid_iterator(it));
+    if (flat_length(it)>=0) { ++(it->length); }
+    else                    { --(it->length); }
+  }
+
+  void set_begin_of_flat(const List_iterator& it, Dart_const_handle dh)
+  {
+    CGAL_assertion(is_valid_iterator(it));
+    it->begin=dh;
+  }
+
+  void set_end_of_flat(const List_iterator& it, Dart_const_handle dh)
+  {
+    CGAL_assertion(is_valid_iterator(it));
+    it->end=dh;
+  }
+
+  /// @return the second dart of the given flat.
+  Dart_const_handle second_dart_of_a_flat(const List_iterator& it)
+  {
+    if (flat_length(it)>0)
+    { return get_map().template beta<1, 2, 1>(begin_of_flat(it)); }
+
+    CGAL_assertion(flat_length(it)<0);
+    return get_map().template beta<2, 0, 2, 0, 2>(begin_of_flat(it));
+  }
+
+  /// @return the dart before the last dart of the given flat.
+  Dart_const_handle before_last_dart_of_a_flat(const List_iterator& it)
+  {
+    if (flat_length(it)>0)
+    { return get_map().template beta<0, 2, 0>(end_of_flat(it)); }
+
+    CGAL_assertion(flat_length(it)<0);
+    return get_map().template beta<2, 1, 2, 1, 2>(end_of_flat(it));
+  }
+
+  /// @return the turn between dart it and next dart.
+  ///         (turn is position of the second edge in the cyclic ordering of
+  ///          edges starting from the first edge around the second extremity
+  ///          of the first dart)
+  std::size_t next_positive_turn(const List_iterator& it)
+  {
+    CGAL_assertion(next_flat_exist(it));
+    return m_MQ.positive_turn(end_of_flat(it), begin_of_flat(next_iterator(it)));
+  }
+
+  /// Same than next_positive_turn but turning in reverse orientation around vertex.
+  std::size_t next_negative_turn(const List_iterator& it)
+  {
+    CGAL_assertion(next_flat_exist(it));
+    return m_MQ.negative_turn(end_of_flat(it), begin_of_flat(next_iterator(it)));
+  }
+
+  std::vector<std::size_t> compute_positive_turns()
+  {
+    std::vector<std::size_t> res;
+    if (is_empty()) return res;
+    for (auto it=m_path.begin(), itend=m_path.end(); it!=itend; ++it)
+    {
+      if (next_flat_exist(it))
+      { res.push_back(next_positive_turn(it)); }
+    }
+    return res;
+  }
+
+  std::vector<std::size_t> compute_negative_turns()
+  {
+    std::vector<std::size_t> res;
+    if (is_empty()) return res;
+    for (auto it=m_path.begin(), itend=m_path.end(); it!=itend; ++it)
+    {
+      if (next_flat_exist(it))
+      { res.push_back(next_negative_turn(it)); }
+    }
+    return res;
+  }
+
+  std::vector<std::size_t> compute_turns(bool positive)
+  { return (positive?compute_positive_turns():compute_negative_turns()); }
+
+  void flat_modified(const List_iterator& it,
+                     Set_of_it& modified_flats)
+  {
+    modified_flats.insert(it);
+    if (prev_flat_exist(it))
+    { modified_flats.insert(prev_iterator(it)); }
+    if (next_flat_exist(it))
+    { modified_flats.insert(next_iterator(it)); }
+  }
+
+  void erase_flat(List_iterator& it, Set_of_it& modified_flats)
+  {
+    if (next_flat_exist(it))
+    { modified_flats.insert(next_iterator(it)); }
+    if (prev_flat_exist(it))
+    { modified_flats.insert(prev_iterator(it)); }
+
+    modified_flats.erase(it);
+
+    it=m_path.erase(it); // after the erase, it is the element after the erased one
+    if (prev_flat_exist(it))
+    {
+      retreat_iterator(it); // this is why we move it backward here
+    }
+  }
+
+  List_iterator merge_modified_flats_when_possible(Set_of_it& modified_flats)
+  {
+    if (m_path.empty())
+    {
+      m_is_closed=false;
+      return m_path.end();
+    }
+
+    List_iterator smallest_it=m_path.end();
+    for (auto it=modified_flats.begin(), itend=modified_flats.end();
+         it!=itend; ++it)
+    {
+      if (prev_flat_exist(*it) && modified_flats.count(prev_iterator(*it))==0)
+      { smallest_it=*it; }
+    }
+
+    List_iterator itcur;
+    if (smallest_it==m_path.end())
+    { // Case of a closed path, with all paths modified
+      for (auto it=modified_flats.begin(); it!=modified_flats.end(); ++it)
+      {
+        itcur=*it;
+        while(merge_with_next_flat_if_possible(itcur, modified_flats))
+        {}
+      }
+      return m_path.begin();
+    }
+
+    for (auto it=modified_flats.begin(); it!=modified_flats.end(); ++it)
+    {
+      itcur=*it;
+      while(merge_with_next_flat_if_possible(itcur, modified_flats))
+      {}
+    }
+
+    // Note that smallest is not necessarily the smallest, this is a flat
+    // such that the previous flat is not modified.
+    return smallest_it;
+  }
+
+  /// Reduce the length of the flat part starting at 'it' from its beginning
+  /// 'it' moves to the previous flat if the current flat disapeared.
+  /// The path could be not valid after this operation (consistency with next
+  /// element should be ensure, by possibly updating the next flat part).
+  /// @return true iff the flat disapeared after its reduction.
+  bool reduce_flat_from_beginning(List_iterator& it,
+                                  Set_of_it& modified_flats)
+  {
+    CGAL_assertion(is_valid_iterator(it));
+
+    CGAL_assertion(m_length>0);
+    --m_length;
+
+    flat_modified(it, modified_flats);
+
+    if (flat_length(it)==0)
+    {
+      erase_flat(it, modified_flats);
+      return true;
+    }
+
+    set_begin_of_flat(it, second_dart_of_a_flat(it));
+    decrease_flat_length(it);
+    return false;
+  }
+
+  /// Reduce the length of the flat part starting at 'it' from its end.
+  /// 'it' moves to the previous flat if the current flat disapeared.
+  /// The path could be not valid after this operation (consistency with next
+  /// element should be ensure, by possibly updating the next flat part).
+  /// @return true iff the flat disapeared after its reduction.
+  bool reduce_flat_from_end(List_iterator& it,
+                            Set_of_it& modified_flats)
+  {
+    CGAL_assertion(is_valid_iterator(it));
+
+    CGAL_assertion(m_length>0);
+    --m_length;
+
+    flat_modified(it, modified_flats);
+
+    if (flat_length(it)==0)
+    {
+      erase_flat(it, modified_flats);
+      return true;
+    }
+
+    set_end_of_flat(it, before_last_dart_of_a_flat(it));
+    decrease_flat_length(it);
+    return false;
+  }
+
+  /// @return true iff the flat 'it' can be merged with its next flat.
+  bool can_merge_with_next_flat(const List_iterator& it,
+                                bool& positive2, bool& negative2)
+  {
+    if (m_path.size()<=1) { return false; }
+    if (!next_flat_exist(it)) { return false; }
+
+    List_iterator it2=next_iterator(it);
+    CGAL_assertion(m_path.size()>1 || it==it2);
+    if (it==it2) { return false; } // only one flat in the path
+
+    positive2=(!m_use_only_negative && (next_positive_turn(it)==2));
+    negative2=(!m_use_only_positive && (next_negative_turn(it)==2));
+    CGAL_assertion(!(positive2 && negative2));
+
+    if (!positive2 && !negative2) { return false; } // the middle turn is not a flat
+
+    bool positive1=(flat_length(it)>=0);
+    bool negative1=(flat_length(it)<=0);
+    bool positive3=(flat_length(it2)>=0);
+    bool negative3=(flat_length(it2)<=0);
+
+    return ((positive1 && positive2 && positive3) ||
+            (negative1 && negative2 && negative3));
+  }
+
+  /// @return true iff the flat 'it' can be merged with its next flat.
+  bool can_merge_with_next_flat(const List_iterator& it)
+  {
+    bool dummy1, dummy2;
+    return can_merge_with_next_flat(it, dummy1, dummy2);
+  }
+
+  /// Merge flat 'it' with its next flat if it is possible.
+  /// @return true if a merging was done.
+  bool merge_with_next_flat_if_possible(const List_iterator& it,
+                                        Set_of_it& modified_flats)
+  {
+    bool positive2=false, negative2=false;
+    if (!can_merge_with_next_flat(it, positive2, negative2))
+    { return false; }
+
+#ifdef CGAL_TRACE_PATH_TESTS
+    std::cout<<"Merge with next flat: ";
+    display_flat(std::cout, it); std::cout<<" and ";
+    display_flat(std::cout, next_iterator(it)); std::cout<<std::endl;
+#endif
+
+    List_iterator it2=next_iterator(it);
+    set_flat_length(it, flat_length(it)+flat_length(it2)+
+                    (positive2?1:-1));
+    set_end_of_flat(it, end_of_flat(it2));
+
+    if (modified_flats.count(it2)==1)
+      modified_flats.erase(it2);
+
+    m_path.erase(it2);
+
+    // CGAL_assertion(is_flat_valid(it));
+    return true;
+  }
+
+  /// Merge flat 'it' with its next flat if it is possible.
+  /// @return true if a merging was done.
+  bool merge_with_next_flat_if_possible(const List_iterator& it)
+  {
+    Set_of_it dummy;
+    return merge_with_next_flat_if_possible(it, dummy);
+  }
+
+  /// Merge the last flat of this path with its next flat if it is possible.
+  /// @return true if a merging was done.
+  bool merge_last_flat_with_next_if_possible()
+  {
+    List_iterator lastit=std::prev(m_path.end());
+    return merge_with_next_flat_if_possible(lastit);
+  }
+
+  /// Return true if it is the beginning of a spur.
+  bool is_spur(const List_iterator& it)
+  {
+    CGAL_assertion(is_valid_iterator(it));
+    return next_flat_exist(it) &&
+        get_map().template beta<2>(end_of_flat(it))==
+        begin_of_flat(next_iterator(it));
+  }
+
+  /// Remove the spur given by 'it'.
+  /// After the operation, either 'it' stay on the same element (if the flat
+  /// still exist), or 'it' move to the previous element if the flat containing
+  /// the spur is removed.
+  /// ('it' will be equal to m_path.end() if the path becomes empty).
+  void remove_spur(List_iterator& it)
+  {
+#ifdef CGAL_TRACE_PATH_TESTS
+        std::cout<<"Remove spur between flats: ";
+        display_flat(std::cout, it); std::cout<<" and ";
+        display_flat(std::cout, next_iterator(it)); std::cout<<std::endl;
+#endif
+
+    CGAL_assertion(is_spur(it));
+    Set_of_it modified_flats;
+    List_iterator it2=next_iterator(it); // it2 is the next flat
+
+    // We reduce the first flat
+    reduce_flat_from_end(it, modified_flats); // If flat 'it' is erased, it is moved to the previous flat
+
+    // We reduce the second flat
+    reduce_flat_from_beginning(it2, modified_flats);
+
+    // We merge adjacent flats if possible
+    it=merge_modified_flats_when_possible(modified_flats);
+
+    // CGAL_assertion(is_valid_iterator(it));
+    // CGAL_assertion(is_valid());
+  }
+
+  /// Move 'it' to the next spur after it. Go to m_path.end() if there is no
+  /// spur in the path.
+  void move_to_next_spur(List_iterator& it)
+  {
+    CGAL_assertion(is_valid_iterator(it));
+    List_iterator itend=(is_closed()?it:m_path.end());
+    do
+    {
+      advance_iterator(it);
+      if (it!=m_path.end() && is_spur(it)) { return; }
+    }
+    while(it!=itend);
+    it=m_path.end(); // Here there is no spur in the whole path
+  }
+
+  /// Simplify the path by removing all spurs, if all==true; otherwise
+  /// remove only one spur.
+  /// @return true iff at least one spur was removed
+  bool remove_spurs(bool all=true)
+  {
+    bool res=false;
+    List_iterator it=m_path.begin();
+    while(it!=m_path.end())
+    {
+      if (is_spur(it))
+      {
+        remove_spur(it); res=true;
+        // CGAL_assertion(is_valid_iterator(it));
+        // CGAL_assertion(is_valid());
+        if (!all) { return true; }
+      }
+      else { move_to_next_spur(it); }
+    }
+    // CGAL_assertion(is_valid());
+    return res;
+  }
+
+  /// Return true if 'it' is the beginning of a positive bracket.
+  /// If true, 'itend' is updated to be the end of the bracket
+  bool is_positive_bracket(const List_iterator& it,
+                           List_iterator& itend)
+  {
+    CGAL_assertion(is_valid_iterator(it));
+    if (m_use_only_negative || !next_flat_exist(it) ||
+        next_positive_turn(it)!=1)
+    { return false; }
+    // Here first positive turn is 1
+
+    itend=next_iterator(it);
+    // Here itend is the beginning of the second flat
+    if (flat_length(itend)<0)
+    { return false; } // This is not a positive bracket, we have some -2
+
+    if (!next_flat_exist(itend) || next_positive_turn(itend)!=1)
+    { return false; }
+
+    // Now we move itend to the beginning of the third flat.
+    advance_iterator(itend);
+    return true;
+  }
+
+  /// Return true if it is the beginning of a negative bracket.
+  /// If true, itend is updated to be the end of the bracket
+  bool is_negative_bracket(const List_iterator& it,
+                           List_iterator& itend)
+  {
+    CGAL_assertion(is_valid_iterator(it));
+    if (m_use_only_positive || !next_flat_exist(it) ||
+        next_negative_turn(it)!=1)
+    { return false; }
+    // Here first negative turn is 1
+
+    itend=next_iterator(it);
+    // Here itend is the beginning of the second flat
+    if (flat_length(itend)>0)
+    { return false; } // This is not a negative bracket, we have some +2
+
+    if (!next_flat_exist(itend) || next_negative_turn(itend)!=1)
+    { return false; }
+
+    // Now we move itend to the beginning of the third flat.
+    advance_iterator(itend);
+    return true;
+  }
+
+  /// Move 'it' to the next bracket after 'it'. Go to m_path.end() if there
+  /// is no bracket in the path.
+  void move_to_next_bracket(List_iterator& it)
+  {
+    CGAL_assertion(is_valid_iterator(it));
+    List_iterator itend=(is_closed()?it:m_path.end());
+    List_iterator it2;
+    do
+    {
+      advance_iterator(it);
+      if (it!=m_path.end() &&
+          (is_positive_bracket(it, it2) || is_negative_bracket(it, it2)))
+      { return; }
+    }
+    while(it!=itend);
+    it=m_path.end(); // Here there is no bracket in the whole path
+  }
+
+  /// Remove the given negative bracket. it1 is the flat beginning
+  /// of the bracket; it3 is the flat end of the bracket.
+  void remove_negative_bracket(List_iterator& it1, List_iterator& it3)
+  {
+#ifdef CGAL_TRACE_PATH_TESTS
+    std::cout<<"Remove negative bracket: ";
+    display_flat(std::cout, it1); std::cout<<", ";
+    display_flat(std::cout, next_iterator(it1)); std::cout<<" and ";
+    display_flat(std::cout, it3); std::cout<<std::endl;
+#endif
+
+    Set_of_it modified_flats;
+    List_iterator it2;
+    CGAL_assertion(is_negative_bracket(it1, it2)); // Here it2 is unused
+
+    if (size_of_list()==1) // it1==it3
+    { // Case of cyclic bracket
+      CGAL_assertion(size_of_list()==1);
+      CGAL_assertion(flat_length(it1)<0);
+      set_begin_of_flat(it1, get_map().template  beta<2,0,2,1>(begin_of_flat(it1)));
+      set_end_of_flat(it1, get_map().template  beta<2,1,2,0>(end_of_flat(it1)));
+      set_flat_length(it1, (-flat_length(it1))-2);
+      flat_modified(it1, modified_flats);
+      it1=merge_modified_flats_when_possible(modified_flats);
+      CGAL_assertion(m_length>1);
+      m_length-=2;
+      return;
+    }
+
+    it2=next_iterator(it1); // it2 is the the next flat after it1
+
+    reduce_flat_from_end(it1, modified_flats); // decrease also m_length
+    reduce_flat_from_beginning(it3, modified_flats);
+
+    it1=it2; // Beginning of the second flat, we are sure it still exists
+
+    set_begin_of_flat(it2, get_map().template  beta<2,1,1>(begin_of_flat(it2)));
+    set_end_of_flat(it2, get_map().template  beta<2,1,1>(end_of_flat(it2)));
+    set_flat_length(it2, -flat_length(it2));
+
+    flat_modified(it2, modified_flats);
+
+    it1=merge_modified_flats_when_possible(modified_flats);
+
+    // CGAL_assertion(is_valid());
+  }
+
+  /// Remove the given positive bracket. 'it1' is the flat beginning
+  /// of the bracket; 'it3' is the flat end of the bracket.
+  void remove_positive_bracket(List_iterator& it1,
+                               List_iterator& it3)
+  {
+#ifdef CGAL_TRACE_PATH_TESTS
+    std::cout<<"Remove positive bracket: ";
+    display_flat(std::cout, it1); std::cout<<", ";
+    display_flat(std::cout, next_iterator(it1)); std::cout<<" and ";
+    display_flat(std::cout, it3); std::cout<<std::endl;
+#endif
+
+    Set_of_it modified_flats;
+    List_iterator it2;
+    CGAL_assertion(is_positive_bracket(it1, it2)); // Here it2 is unused
+
+    if (size_of_list()==1) // it1==it3
+    { // Case of cyclic bracket
+      CGAL_assertion(size_of_list()==1);
+      CGAL_assertion(flat_length(it1)>0);
+      set_begin_of_flat(it1, get_map().template  beta<1,2,0,2>(begin_of_flat(it1)));
+      set_end_of_flat(it1, get_map().template  beta<0,2,1,2>(end_of_flat(it1)));
+      set_flat_length(it1, -(flat_length(it1)-2));
+      flat_modified(it1, modified_flats);
+      it1=merge_modified_flats_when_possible(modified_flats);
+      CGAL_assertion(m_length>1);
+      m_length-=2;
+      return;
+    }
+
+    it2=next_iterator(it1); // it2 is the next flat after it1
+
+    reduce_flat_from_end(it1, modified_flats); // decrease also m_length
+    reduce_flat_from_beginning(it3, modified_flats);
+
+    it1=it2; // Beginning of the second flat, we are sure it exists
+
+    set_begin_of_flat(it2, get_map().template  beta<1,1,2>(begin_of_flat(it2)));
+    set_end_of_flat(it2, get_map().template  beta<1,1,2>(end_of_flat(it2)));
+    set_flat_length(it2, -flat_length(it2));
+
+    flat_modified(it2, modified_flats);
+
+    it1=merge_modified_flats_when_possible(modified_flats);
+
+    // CGAL_assertion(is_valid());
+  }
+
+  /// Simplify the path by removing all brackets, if all==true (default),
+  /// or by removing only one bracket, if all==false.
+  /// @return true iff at least one bracket was removed
+  bool remove_brackets(bool all=true)
+  {
+    // CGAL_assertion(is_valid());
+    bool res=false;
+    List_iterator it1=m_path.begin();
+    List_iterator it2;
+    while(it1!=m_path.end())
+    {
+      // CGAL_assertion(is_valid());
+      if (is_positive_bracket(it1, it2))
+      {
+        remove_positive_bracket(it1, it2); res=true;
+        // CGAL_assertion(is_valid_iterator(it1));
+        // CGAL_assertion(is_valid());
+      }
+      else if (is_negative_bracket(it1, it2))
+      {
+        remove_negative_bracket(it1, it2); res=true;
+        // CGAL_assertion(is_valid_iterator(it1));
+        // CGAL_assertion(is_valid());
+      }
+      else { move_to_next_bracket(it1); }
+      if (!all && res) { return true; }
+    }
+    // CGAL_assertion(is_valid());
+    return res;
+  }
+
+  /// @return true iff 'it' is the beginning of a l-shape.
+  bool is_l_shape(const List_iterator& it)
+  {
+    CGAL_assertion(is_valid_iterator(it));
+    if (!next_flat_exist(it)) { return false; }
+    std::size_t t=next_negative_turn(it);
+    return (t==1 ||
+            (flat_length(it)<0 && size_of_list()==1 && t==2));
+  }
+
+  /// Move it to the next l_shape after it. Go to m_path.end() if there is no
+  /// l_shape in the path.
+  void move_to_next_l_shape(List_iterator& it)
+  {
+    // CGAL_assertion(is_valid());
+    CGAL_assertion(is_valid_iterator(it));
+    List_iterator itend=(is_closed()?it:m_path.end());
+    do
+    {
+      advance_iterator(it);
+      if (it!=m_path.end() && is_l_shape(it)) { return; }
+    }
+    while(it!=itend);
+    it=m_path.end(); // Here there is no spur in the whole path
+  }
+
+  /// @return true iff the flat before flat 'it' can be extended by adding
+  ///              dart 'dh' to its end.
+  bool is_prev_flat_can_be_extended_at_end(const List_iterator& it,
+                                           Dart_const_handle dh,
+                                           bool& positive_flat,
+                                           bool& negative_flat)
+  {
+    positive_flat=false; negative_flat=false;
+    if (!prev_flat_exist(it)) { return false; }
+    List_iterator ittemp=prev_iterator(it);
+    if (!m_use_only_negative && m_MQ.positive_turn(end_of_flat(ittemp), dh)==2)
+    { positive_flat=true; }
+    if (!m_use_only_positive && m_MQ.negative_turn(end_of_flat(ittemp), dh)==2)
+    { negative_flat=true; }
+
+    if (flat_length(ittemp)==0) // Case of flat lengh 0
+    { return positive_flat || negative_flat; }
+
+    return (flat_length(ittemp)>0 && positive_flat) ||
+      (flat_length(ittemp)<0 && negative_flat);
+  }
+
+  /// @return true iff the flat before flat 'it' can be extended by adding
+  ///              dart 'dh' to its end.
+  bool is_prev_flat_can_be_extended_at_end(const List_iterator& it,
+                                           Dart_const_handle dh)
+  {
+    bool dummy1, dummy2;
+    return is_prev_flat_can_be_extended_at_end(it, dh, dummy1, dummy2);
+  }
+
+  /// @return true iff the flat after flat 'it' can be extended by adding
+  ///              dart 'dh' to its beginning.
+  bool is_next_flat_can_be_extended_at_beginning(const List_iterator& it,
+                                                 Dart_const_handle dh,
+                                                 bool& positive_flat,
+                                                 bool& negative_flat)
+  {
+     CGAL_assertion(is_valid_iterator(it));
+
+     positive_flat=false; negative_flat=false;
+     if (!next_flat_exist(it)) { return false; }
+     List_iterator ittemp=next_iterator(it);
+     if (!m_use_only_negative && m_MQ.positive_turn(dh, begin_of_flat(ittemp))==2)
+     { positive_flat=true; }
+     if (!m_use_only_positive && m_MQ.negative_turn(dh, begin_of_flat(ittemp))==2)
+     { negative_flat=true; }
+
+     if (flat_length(ittemp)==0)  // Case of flat lengh 0
+     { return positive_flat || negative_flat; }
+
+     return (flat_length(ittemp)>0 && positive_flat) ||
+       (flat_length(ittemp)<0 && negative_flat);
+  }
+
+  /// @return true iff the flat after flat 'it' can be extended by adding
+  ///              dart 'dh' to its beginning.
+  bool is_next_flat_can_be_extended_at_beginning(const List_iterator& it,
+                                                 Dart_const_handle dh)
+  {
+    bool dummy1, dummy2;
+    return is_next_flat_can_be_extended_at_beginning(it, dh, dummy1, dummy2);
+  }
+
+  /// Add the given dart 'dh' before the flat 'it'.
+  void add_dart_before(const List_iterator& it, Dart_const_handle dh,
+                       Set_of_it& modified_flats)
+  {
+    CGAL_assertion(is_valid_iterator(it));
+
+    bool positive_flat, negative_flat;
+    if (is_prev_flat_can_be_extended_at_end(it, dh,
+                                            positive_flat, negative_flat))
+    {
+      List_iterator ittemp=prev_iterator(it);
+      set_end_of_flat(ittemp, dh); // Move the last dart of the previous flat
+      set_flat_length(ittemp, flat_length(ittemp)+(positive_flat?+1:-1)); // Increment the length of the flat
+      flat_modified(ittemp, modified_flats);
+    }
+    else
+    {
+      // insert the new element before 'it'
+      flat_modified(m_path.insert(it, Flat(dh)), modified_flats);
+    }
+    ++m_length;
+  }
+
+  /// Add the given dart 'dh' after the flat 'it'.
+  void add_dart_after(const List_iterator& it, Dart_const_handle dh,
+                      Set_of_it& modified_flats)
+  {
+    CGAL_assertion(is_valid_iterator(it));
+    bool positive_flat, negative_flat;
+    List_iterator ittemp=next_iterator(it);
+    if (is_next_flat_can_be_extended_at_beginning(it, dh,
+                                                  positive_flat, negative_flat))
+    {
+      set_begin_of_flat(ittemp, dh); // Move the first dart of the flat
+      set_flat_length(ittemp, flat_length(ittemp)+(positive_flat?+1:-1)); // Increment the length of the flat
+      flat_modified(ittemp, modified_flats);
+    }
+    else
+    {
+      // insert the new element before 'ittemp', thus after 'it'
+      flat_modified(m_path.insert(ittemp, Flat(dh)), modified_flats);
+    }
+    ++m_length;
+  }
+
+  /// Right push the given l-shape.
+  void right_push_l_shape(List_iterator& it1)
+  {
+#ifdef CGAL_TRACE_PATH_TESTS
+        std::cout<<"Right push l-shape: ";
+        display_flat(std::cout, it1); std::cout<<" and ";
+        display_flat(std::cout, next_iterator(it1)); std::cout<<std::endl;
+#endif
+
+    // it is the beginning of a flat
+    CGAL_assertion(is_l_shape(it1));
+    // CGAL_assertion(is_valid());
+    Set_of_it modified_flats;
+
+    if (m_path.size()==1)
+    {
+      if (next_negative_turn(it1)==2)
+      { // Special case of a global shift
+        set_flat_length(it1, -flat_length(it1));
+        set_begin_of_flat(it1, get_map().template beta<2,1,1>(begin_of_flat(it1)));
+        set_end_of_flat(it1, get_map().template beta<2,1,1>(end_of_flat(it1)));
+        // CGAL_assertion(is_valid());
+        return;
+      }
+      else // Here negative turn is 1
+      {
+        if (flat_length(it1)>0) // Case where the first flat is positive
+        { // We split the flat in two parts
+          CGAL_assertion(flat_length(it1)>=1);
+          Dart_const_handle dh1=begin_of_flat(it1);
+          Dart_const_handle dh2=end_of_flat(it1);
+          if (flat_length(it1)==1) // only one flat with two darts
+          {
+            reduce_flat_from_end(it1, modified_flats);
+            it1=m_path.insert(it1, Flat(dh2)); // insert dh1 before it1
+            flat_modified(it1, modified_flats);
+            ++m_length;
+          }
+          else
+          {
+            reduce_flat_from_beginning(it1, modified_flats);
+            reduce_flat_from_end(it1, modified_flats);
+            flat_modified(m_path.insert(it1, Flat(dh1)), modified_flats); // insert dh1 before it1
+            it1=m_path.insert(next_iterator(it1), Flat(dh2)); // insert dh2 after it1
+            flat_modified(it1, modified_flats);
+            m_length+=2;
+          }
+          // Now we can continue with the normal case because we have 3 flats
+          CGAL_assertion(flat_length(it1)==0);
+          CGAL_assertion(flat_length(next_iterator(it1))==0);
+        }
+        else
+        {
+          // Here we have only one flat, with some -2
+          // This case is not supposed possible.
+          CGAL_assertion(false);
+          return;
+        }
+      }
+    }
+
+    List_iterator it2=next_iterator(it1);  // Beginning of the second flat
+    CGAL_assertion(it1!=it2);
+
+    if (flat_length(it1)>0) // Case where the first flat is positive
+    { // We split the flat in two parts
+      Dart_const_handle dh=end_of_flat(it1); // last dart of the first flat
+      reduce_flat_from_end(it1, modified_flats);
+      it1=m_path.insert(it2, Flat(dh)); // insert dh before it2
+      ++m_length;
+      flat_modified(it1, modified_flats);
+    }
+
+    if (flat_length(it2)>0) // Case where the second flat is positive, we need to
+    { // split it into two parts
+      Dart_const_handle dh=begin_of_flat(it2); // first dart of the second flat
+      reduce_flat_from_beginning(it2, modified_flats);
+      it2=m_path.insert(it2, Flat(dh)); // insert dh before the second flat
+      ++m_length;
+      flat_modified(it2, modified_flats);
+    }
+
+    // CGAL_assertion(is_valid(false));
+    // CGAL_assertion(is_l_shape(it1));
+
+    Dart_const_handle dh1=get_map().template beta<2,1>(begin_of_flat(it1));
+    Dart_const_handle dh2=get_map().template beta<1>(dh1);
+    Dart_const_handle dh3=get_map().template beta<2,1,2,0>(end_of_flat(it1));
+    Dart_const_handle dh4=get_map().template beta<2,0,2,1>(begin_of_flat(it2));
+    Dart_const_handle dh5=get_map().template beta<2,0,0>(end_of_flat(it2));
+    Dart_const_handle dh6=get_map().template beta<1>(dh5);
+
+    bool first_flat_zero=(flat_length(it1)==0);
+    bool second_flat_zero=(flat_length(it2)==0);
+
+    if (first_flat_zero)
+    {
+      if (second_flat_zero)
+      { // Special case of a l-shape with only 2 darts
+        set_begin_of_flat(it1, dh1);
+        set_end_of_flat(it1, dh1);
+        set_begin_of_flat(it2, dh6);
+        set_end_of_flat(it2, dh6);
+
+        flat_modified(it1, modified_flats);
+        flat_modified(it2, modified_flats);
+
+        it1=merge_modified_flats_when_possible(modified_flats);
+
+        // CGAL_assertion(is_valid());
+        return;
+      }
+      else
+      { // Here first flat length is 0, while second flat not
+        set_begin_of_flat(it1, dh1);
+        set_end_of_flat(it1, dh5);
+        set_begin_of_flat(it2, dh6);
+        set_end_of_flat(it2, dh6);
+        set_flat_length(it1, -flat_length(it2));
+        set_flat_length(it2, 0);
+
+        flat_modified(it1, modified_flats);
+        flat_modified(it2, modified_flats);
+
+        it1=merge_modified_flats_when_possible(modified_flats);
+
+        // CGAL_assertion(is_valid());
+        return;
+      }
+    }
+    else
+    {
+      if (second_flat_zero)
+      { // Here first flat length is non zero, while second flat length is 0
+        set_begin_of_flat(it1, dh1);
+        set_end_of_flat(it1, dh1);
+        set_begin_of_flat(it2, dh2);
+        set_end_of_flat(it2, dh6);
+        set_flat_length(it2, -flat_length(it1));
+        set_flat_length(it1, 0);
+
+        flat_modified(it1, modified_flats);
+        flat_modified(it2, modified_flats);
+
+        it1=merge_modified_flats_when_possible(modified_flats);
+
+        // CGAL_assertion(is_valid());
+        return;
+      }
+    }
+
+    // General case, with two flats with non zero length
+    if (next_iterator(it2)!=it1 || next_negative_turn(it2)!=3)
+    {
+      // 1) Add the first dart before flat 'it'
+      add_dart_before(it1, dh1, modified_flats);
+
+      // 2) Add the last dart after flat 'it2'
+      add_dart_after(it2, dh6, modified_flats);
+    }
+    else
+    { // Case "-2^s -1 -2^t -3" is a special case
+      dh2=get_map().template beta<2,0>(dh1);
+      dh5=get_map().template beta<2,1>(dh6);
+      increase_flat_length(it1);
+      increase_flat_length(it2);
+      m_length+=2;
+      flat_modified(it1, modified_flats);
+      flat_modified(it2, modified_flats);
+    }
+
+    // 3) Move the first flat
+    CGAL_assertion(flat_length(it1)<0);
+    set_begin_of_flat(it1, dh2);
+    set_flat_length(it1, -(flat_length(it1))-1);
+    if (flat_length(it1)==0) { set_end_of_flat(it1, dh2); }
+    else { set_end_of_flat(it1, dh3); } // End of the moved flat
+    flat_modified(it1, modified_flats);
+
+    // 4) Move the second flat
+    CGAL_assertion(flat_length(it2)<0);
+    set_begin_of_flat(it2, dh4);
+    set_flat_length(it2, -(flat_length(it2))-1);
+    if (flat_length(it2)==0) { set_end_of_flat(it2, dh4); }
+    else { set_end_of_flat(it2, dh5); } // End of the moved flat
+    flat_modified(it2, modified_flats);
+
+    CGAL_assertion(m_length>1);
+    m_length-=2;
+
+    it1=merge_modified_flats_when_possible(modified_flats);
+  }
+
+  /// Right push the path, if all all l-shape are pushed, otherwise only one.
+  /// @return true iff the path was pushed
+  bool right_push(bool all=true)
+  {
+    bool res=false;
+    List_iterator it=m_path.begin();
+    while(it!=m_path.end())
+    {
+      if (is_l_shape(it))
+      {
+        right_push_l_shape(it); res=true;
+        // CGAL_assertion(is_valid_iterator(it));
+        // CGAL_assertion(is_valid());
+        if (!all) { return true; }
+      }
+      else { move_to_next_l_shape(it); }
+    }
+    // CGAL_assertion(is_valid());
+    return res;
+  }
+
+  /// Canonize the path
+  void canonize()
+  {
+    CGAL_assertion(is_valid());
+
+    if (is_empty()) { return; }
+
+    CGAL_assertion(is_closed());
+
+     bool modified=false;
+     remove_spurs();
+
+     do
+     {
+       modified=remove_brackets();
+       modified=modified || remove_spurs();
+     }
+     while(modified);
+
+     right_push();
+
+     CGAL_assertion(remove_brackets()==false);
+     CGAL_assertion(remove_spurs()==false);
+     CGAL_assertion(is_valid());
+  }
+
+  void display_positive_turns()
+  {
+    std::cout<<"+(";
+    for (List_iterator it=m_path.begin(), itend=m_path.end();
+         it!=itend; ++it)
+    {
+      if (next_flat_exist(it))
+      { std::cout<<next_positive_turn(it)<<" "; }
+    }
+    std::cout<<")";
+  }
+
+  void display_negative_turns()
+  {
+    std::cout<<"-(";
+    for (List_iterator it=m_path.begin(), itend=m_path.end();
+         it!=itend; ++it)
+    {
+      if (next_flat_exist(it))
+      { std::cout<<next_negative_turn(it)<<" "; }
+    }
+    std::cout<<")";
+  }
+
+  void display_pos_and_neg_turns()
+  {
+    display_positive_turns();
+    std::cout<<"  ";
+    display_negative_turns();
+  }
+
+  void display()
+  {
+    if (!is_empty())
+    {
+      for (List_iterator it=m_path.begin(), itend=m_path.end();
+           it!=itend; ++it)
+      {
+        std::cout<<"[ ";
+        std::cout<<get_map().darts().index(begin_of_flat(it))
+                 <<", "<<get_map().darts().index(end_of_flat(it))
+                 <<"("<<flat_length(it)<<") ] ";
+      }
+      if (is_closed())
+      { std::cout<<" c "; }
+      std::cout<<" length("<<length()<<")";
+    }
+  }
+
+  friend std::ostream& operator<<(std::ostream& os, const Self& p)
+  {
+    const_cast<Self&>(p).display(); // Problem of const correctness: todo solve
+    return os;
+  }
+
+  void set_m_use_only_positive(bool UOP)
+  { m_use_only_positive=UOP; }
+
+  void set_m_use_only_negative(bool UON)
+  { m_use_only_negative=UON; }
+
+protected:
+  const MQ& m_MQ; // The underlying map (a minimal quadrangulation)
+  List_of_flats m_path; // The sequence of flats (a flat part is a pair of darts
+         //  with positive or negative turn == 2). If negative value k, -k is the
+         //  length of the flat part, for negative turns (-2).
+  bool m_is_closed; // True iff the path is a cycle
+  std::size_t m_length;
+  bool m_use_only_positive;
+  bool m_use_only_negative;
+
+#ifdef CGAL_PWRLE_TURN_V2
+  const TDartIds& m_darts_ids;
+#endif //CGAL_PWRLE_TURN_V2
+};
+
+} // namespace internal
+} // namespace Surface_mesh_topology
+} // namespace CGAL
+
+#endif // CGAL_PATH_ON_SURFACE_WITH_RLE_H //
+// EOF //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_topology/internal/Shortest_noncontractible_cycle.h 5.1-1/include/CGAL/Surface_mesh_topology/internal/Shortest_noncontractible_cycle.h
--- 5.0.3-1/include/CGAL/Surface_mesh_topology/internal/Shortest_noncontractible_cycle.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_topology/internal/Shortest_noncontractible_cycle.h	2020-09-08 12:15:46.000000000 +0000
@@ -0,0 +1,596 @@
+// Copyright (c) 2020 CNRS and LIRIS' Establishments (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org).
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_topology/include/CGAL/Surface_mesh_topology/internal/Shortest_noncontractible_cycle.h $
+// $Id: Shortest_noncontractible_cycle.h 52186a0 2020-05-14T11:38:15+02:00 Guillaume Damiand
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+// Author(s)     : Thien Hoang <thienvhoang99@gmail.com>
+//
+#ifndef CGAL_SHORTEST_NONCONTRACTIBLE_CYCLE_H
+#define CGAL_SHORTEST_NONCONTRACTIBLE_CYCLE_H
+
+#include <CGAL/license/Surface_mesh_topology.h>
+
+#include <CGAL/Path_on_surface.h>
+#include <CGAL/Timer.h>
+#include <CGAL/Cell_attribute.h>
+#include <CGAL/Surface_mesh_topology/internal/Generic_map_selector.h>
+#include <CGAL/Surface_mesh_topology/internal/Edge_weight_functor.h>
+#include <queue>
+#include <tuple>
+#include <unordered_map>
+#include <boost/heap/fibonacci_heap.hpp>
+
+namespace CGAL {
+namespace Surface_mesh_topology {
+namespace internal {
+
+template<class SNC, bool Copy>
+struct Get_original_dart
+{
+  static typename SNC::Original_dart_const_handle
+  run(SNC* snc, typename SNC::Dart_handle dh)
+  { return snc->m_copy_to_origin[dh]; }
+};
+
+template<class SNC>
+struct Get_original_dart<SNC, false>
+{
+  static typename SNC::Original_dart_const_handle
+  run(SNC* /* snc */, typename SNC::Dart_handle dh)
+  { return dh; }
+};
+
+template <class Mesh_, bool Copy>
+class Shortest_noncontractible_cycle
+{
+public:
+  using Self=Shortest_noncontractible_cycle<Mesh_, Copy>;
+  using Mesh=Mesh_;
+
+  friend struct Get_original_dart<Self, Copy>;
+
+  using Original_map_wrapper=internal::Generic_map_selector<Mesh, Items_for_shortest_noncontractible_cycle>;
+  using Original_dart_const_handle=typename Original_map_wrapper::Dart_const_handle_original;
+
+  using Local_map        =typename Original_map_wrapper::Generic_map;
+  using Dart_handle      =typename Local_map::Dart_handle;
+  using Dart_const_handle=typename Local_map::Dart_const_handle;
+  using size_type        = typename Local_map::size_type;
+
+  using Dart_container=std::vector<Dart_handle>;
+  using Path          =CGAL::Surface_mesh_topology::Path_on_surface<Mesh>;
+
+  // Associations between original darts and their copy.
+  using Origin_to_copy=boost::unordered_map<Original_dart_const_handle, Dart_handle>;
+  using Copy_to_origin=boost::unordered_map<Dart_handle, Original_dart_const_handle>;
+
+  /// @return the local map
+  const Local_map& get_local_map() const
+  { return *m_local_map; }
+
+  /// @return the local map
+  Local_map& get_local_map()
+  { return *m_local_map; }
+
+  Shortest_noncontractible_cycle(const Mesh& amesh, bool display_time=false) :
+    m_cycle(amesh)
+  {
+    CGAL::Timer t;
+    if (display_time)
+    { t.start(); }
+
+    m_local_map=new Local_map;
+
+    // Initialize m_is_perforated
+    m_is_perforated=get_local_map().get_new_mark();
+
+    Original_map_wrapper::copy(*m_local_map, amesh,
+                               m_origin_to_copy, m_copy_to_origin, m_is_perforated);
+
+    get_local_map().negate_mark(m_is_perforated);
+    // Remove all boundary by adding faces, marked with m_is_perforated
+    get_local_map().template close<2>();
+    get_local_map().negate_mark(m_is_perforated);
+
+    create_vertex_info();
+
+    if (display_time)
+    {
+      t.stop();
+      std::cout<<"[TIME] Shortest_noncontractible_cycle constructor: "
+               <<t.time()<<" seconds."<<std::endl;
+    }
+  }
+
+  // Here amesh is already closed, and have 0-attributes with int.
+  // Thus the mesh is not copied.
+  Shortest_noncontractible_cycle(Mesh* amesh, size_type perforated_mark,
+                                 bool /*display_time*/=false) :
+    m_local_map(amesh),
+    m_is_perforated(perforated_mark),
+    m_cycle(*amesh)
+  {}
+
+  ~Shortest_noncontractible_cycle()
+  {
+    if (Copy)
+    {
+      delete m_local_map;
+    }
+  }
+
+  template <class WeightFunctor>
+  Path compute_cycle(Original_dart_const_handle root_vertex,
+                     typename WeightFunctor::Weight_t* length,
+                     const WeightFunctor& wf,
+                     bool display_time=false)
+  {
+    CGAL::Timer t;
+    if (display_time)
+    { t.start(); }
+
+    m_cycle.clear();
+    Dart_handle root=m_origin_to_copy[root_vertex];
+    this->compute_cycle(root, m_cycle, length, nullptr, wf);
+
+    if (display_time)
+    {
+      t.stop();
+      std::cout<<"[TIME] compute_cycle: "<<t.time()<<" seconds."<<std::endl;
+    }
+
+    return m_cycle;
+  }
+
+  template <class WeightFunctor=Unit_weight_functor>
+  Path compute_cycle(Original_dart_const_handle root_vertex,
+                     typename WeightFunctor::Weight_t* length,
+                     bool display_time=false)
+  { return compute_cycle(root_vertex, length, WeightFunctor(), display_time); }
+
+  template <class WeightFunctor=Unit_weight_functor>
+  Path compute_cycle(Original_dart_const_handle root_vertex,
+                     bool display_time=false)
+  { return compute_cycle(root_vertex, nullptr, display_time); }
+
+  template <class WeightFunctor>
+  Path compute_shortest_non_contractible_cycle(typename WeightFunctor::Weight_t* length,
+                                               const WeightFunctor& wf,
+                                               bool display_time=false)
+  {
+    CGAL::Timer t;
+    if (display_time)
+    { t.start(); }
+
+    m_cycle.clear();
+    bool first_check = true;
+    typename WeightFunctor::Weight_t min_length=0;
+    for (auto it=get_local_map().template attributes<0>().begin(),
+         itend=get_local_map().template attributes<0>().end(); it!=itend; ++it)
+    {
+      Dart_handle dh=get_local_map().template dart_of_attribute<0>(it);
+      typename WeightFunctor::Weight_t temp_length;
+      if (first_check)
+      {
+        if (compute_cycle(dh, m_cycle, &temp_length, nullptr, wf))
+        {
+          min_length = temp_length;
+          first_check = false;
+        }
+      }
+      else
+      {
+        if (compute_cycle(dh, m_cycle, &temp_length, &min_length, wf))
+        { min_length = temp_length; }
+      }
+    }
+    if (length!=nullptr) { *length=min_length; }
+
+    if (display_time)
+    {
+      t.stop();
+      std::cout<<"[TIME] compute_shortest_non_contractible_cycle: "<<t.time()<<" seconds."<<std::endl;
+    }
+
+    return m_cycle;
+  }
+
+  template <class WeightFunctor=Unit_weight_functor>
+  Path compute_shortest_non_contractible_cycle(typename WeightFunctor::Weight_t* length,
+                                              bool display_time=false)
+  { return compute_shortest_non_contractible_cycle(length, WeightFunctor(), display_time); }
+
+  Path compute_edge_width(bool display_time=false)
+  { return compute_shortest_non_contractible_cycle(nullptr, display_time); }
+
+protected:
+  int vertex_info(Dart_handle dh) const
+  { return get_local_map().template info<0>(dh); }
+  int& vertex_info(Dart_handle dh)
+  { return get_local_map().template info<0>(dh); }
+
+  void create_vertex_info()
+  {
+    for (auto it=get_local_map().darts().begin(),
+         itend=get_local_map().darts().end(); it!=itend; ++it)
+    {
+      if (get_local_map().template attribute<0>(it)==nullptr)
+      { get_local_map().template set_attribute<0>
+          (it, get_local_map().template create_attribute<0>(-1)); }
+    }
+  }
+
+  void initialize_vertex_info()
+  {
+    for (auto it=get_local_map().template attributes<0>().begin(),
+         itend = get_local_map().template attributes<0>().end(); it != itend; ++it)
+    { get_local_map().template info_of_attribute<0>(it)=-1; }
+  }
+
+  template <class WeightFunctor, class Distance_>
+  void compute_spanning_tree(Dart_handle root, Dart_container& spanning_tree,
+                             std::vector<Distance_>& distance_from_root,
+                             std::vector<int>& trace_index,
+                             const WeightFunctor& wf)
+  {
+    if (std::is_same<WeightFunctor, Unit_weight_functor>::value)
+    { compute_BFS_tree(root, spanning_tree, distance_from_root, trace_index); }
+    else
+    { compute_Dijkstra_tree(root, spanning_tree, distance_from_root, trace_index, wf); }
+  }
+
+  template <class Distance_>
+  struct Dijkstra_comparator
+  {
+    using Distance_type=Distance_;
+    Dijkstra_comparator(const std::vector<Distance_type>& distance_from_root) :
+      m_distance(distance_from_root) {}
+    bool operator()(const int x, const int y) const
+    { return m_distance[x]>m_distance[y]; }
+  private:
+    const std::vector<Distance_type>& m_distance;
+  };
+
+  /// Create a spanning tree using Dijkstra
+  template <class WeightFunctor, class Distance_>
+  void compute_Dijkstra_tree(Dart_handle root, Dart_container& spanning_tree,
+                             std::vector<Distance_>& distance_from_root,
+                             std::vector<int>& trace_index,
+                             const WeightFunctor& wf)
+  {
+    // Preparation
+    Dijkstra_comparator<Distance_> dc(distance_from_root);
+    //std::priority_queue<int, std::vector<int>, Dijkstra_comparator<Distance_> > pq(dc);
+    typedef boost::heap::fibonacci_heap<int, boost::heap::compare<Dijkstra_comparator<Distance_>>> Heap;
+    typedef typename Heap::handle_type heap_handle;
+
+    Heap pq(dc);
+    std::unordered_map<int, heap_handle> inqueue;
+
+    int vertex_index=0;
+    size_type vertex_visited=get_local_map().get_new_mark();
+    // Begin Dijkstra
+    distance_from_root.reserve(get_local_map().template attributes<0>().size());
+    distance_from_root.push_back(0);
+    inqueue[0]=pq.push(0);
+    vertex_info(root)=vertex_index;
+    get_local_map().template mark_cell<0>(root, vertex_visited);
+
+    while (!pq.empty())
+    {
+      int u_index=pq.top();
+      pq.pop();
+
+      inqueue.erase(u_index);
+
+      Dart_handle u=(u_index==0)?root:get_local_map().next(spanning_tree[u_index-1]);
+      CGAL_assertion(u_index==vertex_info(u));
+      Dart_handle it=u;
+      do
+      {
+        Dart_handle v=get_local_map().next(it);
+        Distance_ w=wf(Get_original_dart<Self, Copy>::run(this, nonhole_dart_of_same_edge(it)));
+        if (!get_local_map().is_marked(v, vertex_visited))
+        {
+          int v_index=++vertex_index;
+          CGAL_assertion(v_index==static_cast<int>(distance_from_root.size()));
+          CGAL_assertion(inqueue.count(v_index)==0);
+          distance_from_root.push_back(distance_from_root[u_index]+w);
+          spanning_tree.push_back(it);
+          trace_index.push_back(u_index-1);
+          vertex_info(v)=v_index;
+          get_local_map().template mark_cell<0>(v, vertex_visited);
+          inqueue[v_index]=pq.push(v_index);
+        }
+        else
+        {
+          int v_index=vertex_info(v);
+          if (distance_from_root[v_index]>distance_from_root[u_index]+w)
+          {
+            CGAL_assertion(v_index>0);
+            distance_from_root[v_index]=distance_from_root[u_index]+w;
+            spanning_tree[v_index-1]=it;
+            trace_index[v_index-1]=u_index-1;
+
+            auto it_inqueue=inqueue.find(v_index);
+            if (it_inqueue==inqueue.end())
+            { inqueue[v_index]=pq.push(v_index); }
+            else
+            { pq.decrease(it_inqueue->second, v_index); }
+          }
+        }
+        it=get_local_map().next(get_local_map().opposite2(it));
+      }
+      while(it!=u);
+    }
+    get_local_map().free_mark(vertex_visited);
+    // End Dijkstra
+  }
+
+  /// Create a spanning tree using BFS
+  template <class Distance_>
+  void compute_BFS_tree(Dart_handle root, Dart_container& spanning_tree,
+                        std::vector<Distance_>& distance_from_root,
+                        std::vector<int>& trace_index)
+  {
+    // Preparation
+    std::queue<int> q;
+    int vertex_index=0;
+    size_type vertex_visited=get_local_map().get_new_mark();
+    // Begin BFS
+    q.push(0);
+    vertex_info(root)=vertex_index;
+    get_local_map().template mark_cell<0>(root, vertex_visited);
+    distance_from_root.reserve(get_local_map().template attributes<0>().size());
+    distance_from_root.push_back(0);
+    while (!q.empty())
+    {
+      int u_index=q.front();
+      q.pop();
+      Dart_handle u=(u_index==0)?root:get_local_map().next(spanning_tree[u_index-1]);
+      CGAL_assertion(u_index==vertex_info(u));
+      Dart_handle it=u;
+      do
+      {
+        Dart_handle v=get_local_map().next(it);
+        if (!get_local_map().is_marked(v, vertex_visited))
+        {
+          int v_index=++vertex_index;
+          CGAL_assertion(v_index==static_cast<int>(distance_from_root.size()));
+          distance_from_root.push_back(1+distance_from_root[u_index]);
+          spanning_tree.push_back(it);
+          // `it` will lead to v
+          q.push(v_index);
+          trace_index.push_back(u_index-1);
+          vertex_info(v)=v_index;
+          get_local_map().template mark_cell<0>(v, vertex_visited);
+        }
+        it=get_local_map().next(get_local_map().opposite2(it));
+      }
+      while(it!=u);
+    }
+    get_local_map().free_mark(vertex_visited);
+    // End BFS
+  }
+
+  /// Check if there is only one unmarked edge around a face.
+  /// If there is, let dh_only_edge=the edge separating it and its only adjacent face.
+  bool is_degree_one_face(Dart_handle dh_face, Dart_handle& dh_only_edge,
+                          size_type edge_deleted)
+  {
+    Dart_handle dh=dh_face;
+    dh_only_edge=nullptr;
+    do
+    {
+      if (!get_local_map().is_marked(dh, edge_deleted))
+      {
+        if (dh_only_edge!=nullptr)
+        { dh_only_edge=nullptr; return false; }
+        dh_only_edge=dh;
+      }
+      dh=get_local_map().next(dh);
+    }
+    while(dh!=dh_face);
+    return (dh_only_edge!=nullptr);
+  }
+
+  /// Find E_nc
+  void compute_noncon_edges(const Dart_container& spanning_tree,
+                            Dart_container& noncon_edges)
+  {
+    noncon_edges.clear();
+    size_type face_deleted=get_local_map().get_new_mark();
+    size_type edge_deleted=get_local_map().get_new_mark();
+    size_type tested=get_local_map().get_new_mark();
+    for (auto dh : spanning_tree)
+    {
+      if (!get_local_map().is_marked(dh, edge_deleted))
+      { get_local_map().template mark_cell<1>(dh, edge_deleted); }
+    }
+    std::queue<Dart_handle> degree_one_faces;
+    // Add to queue the degree-1 faces
+    for (auto it=get_local_map().darts().begin(), itend=get_local_map().darts().end();
+         it!=itend; ++it)
+    {
+      if (get_local_map().is_marked(it, m_is_perforated))
+      {
+        if (!get_local_map().is_marked(it, face_deleted))
+        { get_local_map().template mark_cell<2>(it, face_deleted); }
+      }
+      else if (!get_local_map().is_marked(it, tested))
+      {
+        Dart_handle dh=it, dh_only_edge=nullptr;
+        bool degree_one=true;
+        do // Here we do not use is_degree_one_face method because we want to
+        {  // mark tested all darts of the face in the same loop.
+          get_local_map().template mark_cell<1,1>(dh, tested); // For CMap and GMap
+          if (degree_one)
+          {
+            if (!get_local_map().is_marked(dh, edge_deleted))
+            {
+              if (dh_only_edge!=nullptr) { degree_one=false; }
+              else { dh_only_edge=dh; }
+            }
+          }
+          dh=get_local_map().next(dh);
+        }
+        while(dh!=it);
+
+        if (degree_one && dh_only_edge!=nullptr)
+        {
+          degree_one_faces.push(dh_only_edge);
+          CGAL_assertion(!get_local_map().is_marked(dh_only_edge, edge_deleted));
+          CGAL_assertion(!get_local_map().is_marked(dh_only_edge, face_deleted));
+        }
+      }
+    }
+
+    // Remove the degree-1 faces
+    while (!degree_one_faces.empty())
+    {
+      Dart_handle dh_face=degree_one_faces.front();
+      degree_one_faces.pop();
+      if (!get_local_map().is_marked(dh_face, edge_deleted))
+      { get_local_map().template mark_cell<1>(dh_face, edge_deleted); }
+      if (!get_local_map().is_marked(dh_face, face_deleted))
+      { get_local_map().template mark_cell<2>(dh_face, face_deleted); }
+      Dart_handle dh_adj_face=get_local_map().opposite2(dh_face);
+      if (!get_local_map().is_marked(dh_adj_face, face_deleted))
+      {
+        Dart_handle dh_only_edge=nullptr;
+        if (is_degree_one_face(dh_adj_face, dh_only_edge, edge_deleted))
+        {
+          degree_one_faces.push(dh_only_edge);
+          CGAL_assertion(!get_local_map().is_marked(dh_only_edge, edge_deleted));
+          CGAL_assertion(!get_local_map().is_marked(dh_only_edge, face_deleted));
+        }
+      }
+    }
+    for (auto it=get_local_map().darts().begin(),
+         itend=get_local_map().darts().end(); it!=itend; ++it)
+    {
+      if (it<get_local_map().opposite2(it) && vertex_info(it)>=0 &&
+          !get_local_map().is_marked(it, edge_deleted))
+      { noncon_edges.push_back(it); }
+
+      get_local_map().unmark(it, tested);
+      get_local_map().unmark(it, edge_deleted);
+      get_local_map().unmark(it, face_deleted);
+    }
+
+    get_local_map().free_mark(tested);
+    get_local_map().free_mark(edge_deleted);
+    get_local_map().free_mark(face_deleted);
+  }
+
+  Dart_handle nonhole_dart_of_same_edge(Dart_handle dh)
+  {
+    CGAL_assertion(dh!=nullptr);
+    if (get_local_map().is_marked(dh, m_is_perforated))
+    { dh = get_local_map().opposite2(dh); }
+    CGAL_assertion(!get_local_map().is_marked(dh, m_is_perforated));
+    return dh;
+  }
+
+  Dart_handle nonhole_dart_of_same_edge(Dart_handle dh, bool& flip)
+  {
+    CGAL_assertion(dh!=nullptr);
+    if (get_local_map().is_marked(dh, m_is_perforated))
+    {
+      dh=get_local_map().opposite2(dh);
+      flip=!flip;
+    }
+    CGAL_assertion(!get_local_map().is_marked(dh, m_is_perforated));
+    return dh;
+  }
+
+  void add_to_cycle(Dart_handle dh, Path& cycle, bool flip)
+  {
+    dh=nonhole_dart_of_same_edge(dh, flip);
+    Original_dart_const_handle
+        dh_original=Get_original_dart<Self, Copy>::run(this, dh);
+    if (cycle.can_be_pushed(dh_original, flip))
+    { cycle.push_back(dh_original, flip, false); }
+    else
+    {
+      CGAL_assertion(cycle.can_be_pushed(dh_original, !flip));
+      cycle.push_back(dh_original, !flip, false);
+    }
+  }
+
+  template <class WeightFunctor>
+  bool compute_cycle(Dart_handle root, Path& cycle,
+                     typename WeightFunctor::Weight_t* length,
+                     const typename WeightFunctor::Weight_t* max_length,
+                     const WeightFunctor& wf)
+  {
+    std::vector<typename WeightFunctor::Weight_t> distance_from_root;
+    m_spanning_tree.clear();
+    m_trace_index.clear();
+    initialize_vertex_info();
+    compute_spanning_tree(root, m_spanning_tree, distance_from_root, m_trace_index, wf);
+    compute_noncon_edges(m_spanning_tree, m_noncon_edges);
+
+    bool first_check=true;
+    typename WeightFunctor::Weight_t min_distance=0;
+    Dart_handle min_noncon_edge;
+    int min_a=-1, min_b=-1;
+    for (auto dh : m_noncon_edges)
+    {
+      Dart_handle a=dh, b=get_local_map().next(dh);
+      int index_a=vertex_info(a), index_b=vertex_info(b);
+      typename WeightFunctor::Weight_t sum_distance=
+        distance_from_root[index_a]+distance_from_root[index_b]
+        +wf(Get_original_dart<Self, Copy>::run(this, nonhole_dart_of_same_edge(dh)));
+      if (first_check || sum_distance<min_distance)
+      {
+        min_distance=sum_distance;
+        min_noncon_edge=dh;
+        min_a=index_a;
+        min_b=index_b;
+        first_check=false;
+      }
+    }
+
+    if (first_check) { return false; } // no cycle found
+
+    if (max_length!=nullptr && min_distance>=*max_length)
+    { return false; } // Here the cycle is too long
+
+    cycle.clear();
+    // Trace back the path from `a` to root
+    for (int ind=min_a-1; ind!=-1; ind=m_trace_index[ind])
+    {  add_to_cycle(m_spanning_tree[ind], cycle, true); }
+    // Reverse: now it is the path from root to `a`
+    cycle.reverse();
+    add_to_cycle(min_noncon_edge, cycle, false);
+    // Trace back the path from `b` to root
+    for (int ind=min_b-1; ind!=-1; ind=m_trace_index[ind])
+    { add_to_cycle(m_spanning_tree[ind], cycle, true); }
+
+    cycle.update_is_closed();
+    CGAL_assertion(cycle.is_closed());
+    if (length!=nullptr) { *length=(min_distance<0?0:min_distance); }
+
+    return true;
+  }
+
+protected:
+  Local_map*       m_local_map; /// the local map
+  size_type        m_is_perforated;   /// mark for perforated darts
+  Origin_to_copy   m_origin_to_copy; /// associative array from original darts to copies
+  Copy_to_origin   m_copy_to_origin; /// associative array from copies to original darts
+  Dart_container   m_spanning_tree, m_noncon_edges; // Darts in the spanning tree and in E_nc
+  std::vector<int> m_trace_index;
+  Path             m_cycle;
+};
+
+} // namespace internal
+} // namespace Surface_mesh_topology
+} // namespace CGAL
+
+#endif
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_traits_generator_3.h 5.1-1/include/CGAL/Surface_mesh_traits_generator_3.h
--- 5.0.3-1/include/CGAL/Surface_mesh_traits_generator_3.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_traits_generator_3.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Surface_mesh_traits_generator_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Surface_mesh_traits_generator_3.h $
 // $Id: Surface_mesh_traits_generator_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_triangulation_generator_3.h 5.1-1/include/CGAL/Surface_mesh_triangulation_generator_3.h
--- 5.0.3-1/include/CGAL/Surface_mesh_triangulation_generator_3.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_triangulation_generator_3.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Surface_mesh_triangulation_generator_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Surface_mesh_triangulation_generator_3.h $
 // $Id: Surface_mesh_triangulation_generator_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_mesh_vertex_base_3.h 5.1-1/include/CGAL/Surface_mesh_vertex_base_3.h
--- 5.0.3-1/include/CGAL/Surface_mesh_vertex_base_3.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_mesh_vertex_base_3.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/Surface_mesh_vertex_base_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/Surface_mesh_vertex_base_3.h $
 // $Id: Surface_mesh_vertex_base_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/surface_neighbor_coordinates_3.h 5.1-1/include/CGAL/surface_neighbor_coordinates_3.h
--- 5.0.3-1/include/CGAL/surface_neighbor_coordinates_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/surface_neighbor_coordinates_3.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Interpolation/include/CGAL/surface_neighbor_coordinates_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Interpolation/include/CGAL/surface_neighbor_coordinates_3.h $
 // $Id: surface_neighbor_coordinates_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/surface_neighbors_3.h 5.1-1/include/CGAL/surface_neighbors_3.h
--- 5.0.3-1/include/CGAL/surface_neighbors_3.h	2020-08-01 19:02:24.000000000 +0000
+++ 5.1-1/include/CGAL/surface_neighbors_3.h	2020-09-08 12:14:57.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Interpolation/include/CGAL/surface_neighbors_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Interpolation/include/CGAL/surface_neighbors_3.h $
 // $Id: surface_neighbors_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/surface_reconstruction_points_assertions.h 5.1-1/include/CGAL/surface_reconstruction_points_assertions.h
--- 5.0.3-1/include/CGAL/surface_reconstruction_points_assertions.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/surface_reconstruction_points_assertions.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Poisson_surface_reconstruction_3/include/CGAL/surface_reconstruction_points_assertions.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Poisson_surface_reconstruction_3/include/CGAL/surface_reconstruction_points_assertions.h $
 // $Id: surface_reconstruction_points_assertions.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_sweep_2/Arr_basic_insertion_traits_2.h 5.1-1/include/CGAL/Surface_sweep_2/Arr_basic_insertion_traits_2.h
--- 5.0.3-1/include/CGAL/Surface_sweep_2/Arr_basic_insertion_traits_2.h	2020-08-01 19:02:13.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_sweep_2/Arr_basic_insertion_traits_2.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Surface_sweep_2/Arr_basic_insertion_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Surface_sweep_2/Arr_basic_insertion_traits_2.h $
 // $Id: Arr_basic_insertion_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_sweep_2/Arr_batched_pl_ss_visitor.h 5.1-1/include/CGAL/Surface_sweep_2/Arr_batched_pl_ss_visitor.h
--- 5.0.3-1/include/CGAL/Surface_sweep_2/Arr_batched_pl_ss_visitor.h	2020-08-01 19:02:13.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_sweep_2/Arr_batched_pl_ss_visitor.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Surface_sweep_2/Arr_batched_pl_ss_visitor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Surface_sweep_2/Arr_batched_pl_ss_visitor.h $
 // $Id: Arr_batched_pl_ss_visitor.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_sweep_2/Arr_construction_event_base.h 5.1-1/include/CGAL/Surface_sweep_2/Arr_construction_event_base.h
--- 5.0.3-1/include/CGAL/Surface_sweep_2/Arr_construction_event_base.h	2020-08-01 19:02:13.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_sweep_2/Arr_construction_event_base.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Surface_sweep_2/Arr_construction_event_base.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Surface_sweep_2/Arr_construction_event_base.h $
 // $Id: Arr_construction_event_base.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_sweep_2/Arr_construction_event.h 5.1-1/include/CGAL/Surface_sweep_2/Arr_construction_event.h
--- 5.0.3-1/include/CGAL/Surface_sweep_2/Arr_construction_event.h	2020-08-01 19:02:13.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_sweep_2/Arr_construction_event.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Surface_sweep_2/Arr_construction_event.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Surface_sweep_2/Arr_construction_event.h $
 // $Id: Arr_construction_event.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_sweep_2/Arr_construction_ss_visitor.h 5.1-1/include/CGAL/Surface_sweep_2/Arr_construction_ss_visitor.h
--- 5.0.3-1/include/CGAL/Surface_sweep_2/Arr_construction_ss_visitor.h	2020-08-01 19:02:13.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_sweep_2/Arr_construction_ss_visitor.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Surface_sweep_2/Arr_construction_ss_visitor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Surface_sweep_2/Arr_construction_ss_visitor.h $
 // $Id: Arr_construction_ss_visitor.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_sweep_2/Arr_construction_subcurve.h 5.1-1/include/CGAL/Surface_sweep_2/Arr_construction_subcurve.h
--- 5.0.3-1/include/CGAL/Surface_sweep_2/Arr_construction_subcurve.h	2020-08-01 19:02:13.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_sweep_2/Arr_construction_subcurve.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Surface_sweep_2/Arr_construction_subcurve.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Surface_sweep_2/Arr_construction_subcurve.h $
 // $Id: Arr_construction_subcurve.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_sweep_2/Arr_default_overlay_traits_base.h 5.1-1/include/CGAL/Surface_sweep_2/Arr_default_overlay_traits_base.h
--- 5.0.3-1/include/CGAL/Surface_sweep_2/Arr_default_overlay_traits_base.h	2020-08-01 19:02:13.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_sweep_2/Arr_default_overlay_traits_base.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Surface_sweep_2/Arr_default_overlay_traits_base.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Surface_sweep_2/Arr_default_overlay_traits_base.h $
 // $Id: Arr_default_overlay_traits_base.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_sweep_2/Arr_insertion_ss_visitor.h 5.1-1/include/CGAL/Surface_sweep_2/Arr_insertion_ss_visitor.h
--- 5.0.3-1/include/CGAL/Surface_sweep_2/Arr_insertion_ss_visitor.h	2020-08-01 19:02:13.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_sweep_2/Arr_insertion_ss_visitor.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Surface_sweep_2/Arr_insertion_ss_visitor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Surface_sweep_2/Arr_insertion_ss_visitor.h $
 // $Id: Arr_insertion_ss_visitor.h 092cf8e 2019-10-25T16:26:17+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_sweep_2/Arr_insertion_traits_2.h 5.1-1/include/CGAL/Surface_sweep_2/Arr_insertion_traits_2.h
--- 5.0.3-1/include/CGAL/Surface_sweep_2/Arr_insertion_traits_2.h	2020-08-01 19:02:13.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_sweep_2/Arr_insertion_traits_2.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Surface_sweep_2/Arr_insertion_traits_2.h $
-// $Id: Arr_insertion_traits_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Surface_sweep_2/Arr_insertion_traits_2.h $
+// $Id: Arr_insertion_traits_2.h 4158542 2020-04-01T12:31:51+03:00 Efi Fogel
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -80,17 +80,13 @@ public:
   protected:
     //! The base operators.
     Base_intersect_2 m_base_intersect;
-    Halfedge_handle invalid_he;
 
     /*! Constructor.
      * The constructor is declared private to allow only the functor
      * obtaining function, which is a member of the nesting class,
      * constructing it.
      */
-    Intersect_2(const Base_intersect_2& base) :
-      m_base_intersect (base),
-      invalid_he()
-    {}
+    Intersect_2(const Base_intersect_2& base) :  m_base_intersect (base) {}
 
     //! Allow its functor obtaining function calling the private constructor.
     friend class Arr_insertion_traits_2<Gt2, Arrangement_2>;
@@ -101,6 +97,14 @@ public:
                               const X_monotone_curve_2& cv2,
                               OutputIterator oi)
     {
+      typedef std::pair<Point_2, Multiplicity>          Intersection_point;
+      typedef boost::variant<Intersection_point, X_monotone_curve_2>
+                                                        Intersection_result;
+      typedef boost::variant<Intersection_point, Base_x_monotone_curve_2>
+                                                        Intersection_base_result;
+
+      Halfedge_handle invalid_he;
+
       if ((cv1.halfedge_handle() != invalid_he) &&
           (cv2.halfedge_handle() != invalid_he) &&
           (cv1.halfedge_handle() != cv2.halfedge_handle()))
@@ -110,40 +114,32 @@ public:
         return oi;
       }
 
-      OutputIterator oi_end = m_base_intersect(cv1.base(), cv2.base(), oi);
-      const Base_x_monotone_curve_2* base_overlap_cv;
-      const std::pair<Base_point_2, unsigned int>* intersect_p;
-
+      std::vector<Intersection_base_result> xections;
+      m_base_intersect(cv1.base(), cv2.base(), std::back_inserter(xections));
       // convert objects that are associated with Base_x_monotone_curve_2 to
       // X_monotone_curve_2
-      for(; oi != oi_end; ++oi) {
-        base_overlap_cv = object_cast<Base_x_monotone_curve_2>(&(*oi));
-        if (base_overlap_cv != nullptr) {
-          // Add halfedge handles to the resulting curve.
-          Halfedge_handle he;
-
-          if (cv1.halfedge_handle() != invalid_he) he = cv1.halfedge_handle();
-          else if (cv2.halfedge_handle() != invalid_he)
-            he = cv2.halfedge_handle();
-
-          X_monotone_curve_2 overlap_cv (*base_overlap_cv, he);
-
-          overlap_cv.set_overlapping();
-          *oi = make_object (overlap_cv);
-        }
-        else {
-          intersect_p =
-            object_cast<std::pair<Base_point_2, unsigned int> >(&(*oi));
-
-          CGAL_assertion (intersect_p != nullptr);
-
-          *oi = make_object(std::make_pair(Point_2(intersect_p->first),
-                                           intersect_p->second));
+      for (const auto& xection : xections) {
+        const Intersection_point*
+          p_p = boost::get<Intersection_point>(&xection);
+        if (p_p != nullptr) {
+          *oi++ = Intersection_result(xection);
+          continue;
         }
+        const Base_x_monotone_curve_2* base_cv_p =
+          boost::get<Base_x_monotone_curve_2>(&xection);
+        CGAL_assertion(base_cv_p);
+
+        // Add halfedge handles to the resulting curve.
+        Halfedge_handle he;
+        if (cv1.halfedge_handle() != invalid_he) he = cv1.halfedge_handle();
+        else if (cv2.halfedge_handle() != invalid_he)
+          he = cv2.halfedge_handle();
+        X_monotone_curve_2 cv(*base_cv_p, he);
+        cv.set_overlapping();
+        *oi++ = Intersection_result(cv);
       }
-
-      // Return a past-the-end iterator.
-      return oi_end;
+      xections.clear();
+      return oi;
     }
   };
 
diff -pruN 5.0.3-1/include/CGAL/Surface_sweep_2/Arr_no_intersection_insertion_ss_visitor.h 5.1-1/include/CGAL/Surface_sweep_2/Arr_no_intersection_insertion_ss_visitor.h
--- 5.0.3-1/include/CGAL/Surface_sweep_2/Arr_no_intersection_insertion_ss_visitor.h	2020-08-01 19:02:13.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_sweep_2/Arr_no_intersection_insertion_ss_visitor.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Surface_sweep_2/Arr_no_intersection_insertion_ss_visitor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Surface_sweep_2/Arr_no_intersection_insertion_ss_visitor.h $
 // $Id: Arr_no_intersection_insertion_ss_visitor.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_sweep_2/Arr_overlay_event.h 5.1-1/include/CGAL/Surface_sweep_2/Arr_overlay_event.h
--- 5.0.3-1/include/CGAL/Surface_sweep_2/Arr_overlay_event.h	2020-08-01 19:02:13.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_sweep_2/Arr_overlay_event.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Surface_sweep_2/Arr_overlay_event.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Surface_sweep_2/Arr_overlay_event.h $
 // $Id: Arr_overlay_event.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_sweep_2/Arr_overlay_ss_visitor.h 5.1-1/include/CGAL/Surface_sweep_2/Arr_overlay_ss_visitor.h
--- 5.0.3-1/include/CGAL/Surface_sweep_2/Arr_overlay_ss_visitor.h	2020-08-01 19:02:13.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_sweep_2/Arr_overlay_ss_visitor.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Surface_sweep_2/Arr_overlay_ss_visitor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Surface_sweep_2/Arr_overlay_ss_visitor.h $
 // $Id: Arr_overlay_ss_visitor.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_sweep_2/Arr_overlay_subcurve.h 5.1-1/include/CGAL/Surface_sweep_2/Arr_overlay_subcurve.h
--- 5.0.3-1/include/CGAL/Surface_sweep_2/Arr_overlay_subcurve.h	2020-08-01 19:02:13.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_sweep_2/Arr_overlay_subcurve.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Surface_sweep_2/Arr_overlay_subcurve.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Surface_sweep_2/Arr_overlay_subcurve.h $
 // $Id: Arr_overlay_subcurve.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_sweep_2/Arr_overlay_traits_2.h 5.1-1/include/CGAL/Surface_sweep_2/Arr_overlay_traits_2.h
--- 5.0.3-1/include/CGAL/Surface_sweep_2/Arr_overlay_traits_2.h	2020-08-01 19:02:13.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_sweep_2/Arr_overlay_traits_2.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Surface_sweep_2/Arr_overlay_traits_2.h $
-// $Id: Arr_overlay_traits_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Surface_sweep_2/Arr_overlay_traits_2.h $
+// $Id: Arr_overlay_traits_2.h 4158542 2020-04-01T12:31:51+03:00 Efi Fogel
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -345,24 +345,31 @@ public:
   class Intersect_2 {
   protected:
     //! The base traits.
-    const Arr_overlay_traits_2* m_traits;
+    const Arr_overlay_traits_2& m_traits;
 
     /*! Constructor.
      * The constructor is declared protected to allow only the functor
      * obtaining function, which is a member of the nesting class,
      * constructing it.
      */
-    Intersect_2(const Arr_overlay_traits_2* traits) : m_traits(traits) {}
+    Intersect_2(const Arr_overlay_traits_2& traits) : m_traits(traits) {}
 
     //! Allow its functor obtaining function calling the protected constructor.
     friend class Arr_overlay_traits_2<Gt2, Ar2, Ab2>;
 
   public:
-    template<class OutputIterator>
+    template <typename OutputIterator>
     OutputIterator operator()(const X_monotone_curve_2& xcv1,
                               const X_monotone_curve_2& xcv2,
                               OutputIterator oi)
     {
+      typedef std::pair<Point_2, Multiplicity>          Intersection_point;
+      typedef boost::variant<Intersection_point, X_monotone_curve_2>
+                                                        Intersection_result;
+      typedef std::pair<Base_point_2, Multiplicity>     Intersection_base_point;
+      typedef boost::variant<Intersection_base_point, Base_x_monotone_curve_2>
+                                                        Intersection_base_result;
+
       // In case the curves originate from the same arrangement, they are
       // obviously interior-disjoint.
       if (xcv1.color() == xcv2.color()) return oi;
@@ -396,19 +403,16 @@ public:
       // Note that we do not bother with curves whose left ends are open,
       // since such curved did not intersect before.
 
-      const std::pair<Base_point_2, unsigned int>* base_ipt;
-      const Base_x_monotone_curve_2* overlap_xcv;
       bool send_xcv1_first = true;
-      OutputIterator oi_end;
 
-      Parameter_space_in_x_2 ps_x_op = m_traits->parameter_space_in_x_2_object();
-      Parameter_space_in_y_2 ps_y_op = m_traits->parameter_space_in_y_2_object();
-      const Arr_parameter_space bx1 = ps_x_op(xcv1, ARR_MIN_END);
-      const Arr_parameter_space by1 = ps_y_op(xcv1, ARR_MIN_END);
-      const Arr_parameter_space bx2 = ps_x_op(xcv2, ARR_MIN_END);
-      const Arr_parameter_space by2 = ps_y_op(xcv2, ARR_MIN_END);
+      auto ps_x_op = m_traits.parameter_space_in_x_2_object();
+      auto ps_y_op = m_traits.parameter_space_in_y_2_object();
+      Arr_parameter_space bx1 = ps_x_op(xcv1, ARR_MIN_END);
+      Arr_parameter_space by1 = ps_y_op(xcv1, ARR_MIN_END);
+      Arr_parameter_space bx2 = ps_x_op(xcv2, ARR_MIN_END);
+      Arr_parameter_space by2 = ps_y_op(xcv2, ARR_MIN_END);
 
-      const Gt2* m_base_tr = m_traits->base_traits();
+      const Gt2* m_base_tr = m_traits.base_traits();
 
       if ((bx1 == ARR_INTERIOR) && (by1 == ARR_INTERIOR) &&
           (bx2 == ARR_INTERIOR) && (by2 == ARR_INTERIOR))
@@ -419,15 +423,17 @@ public:
             m_base_tr->construct_min_vertex_2_object()(xcv2.base())) == LARGER);
       }
 
-      oi_end = (send_xcv1_first) ?
-        m_base_tr->intersect_2_object()(xcv1.base(), xcv2.base(), oi) :
-        m_base_tr->intersect_2_object()(xcv2.base(), xcv1.base(), oi);
+      auto intersector = m_base_tr->intersect_2_object();
+      std::vector<Intersection_base_result> xections;
+      (send_xcv1_first) ?
+        intersector(xcv1.base(), xcv2.base(), std::back_inserter(xections)) :
+        intersector(xcv2.base(), xcv1.base(), std::back_inserter(xections));
 
       // Convert objects that are associated with Base_x_monotone_curve_2 to
       // the exteneded X_monotone_curve_2.
-      while (oi != oi_end) {
-        base_ipt = object_cast<std::pair<Base_point_2, unsigned int> >(&(*oi));
-
+      for (const auto& xection : xections) {
+        const Intersection_base_point* base_ipt =
+          boost::get<Intersection_base_point>(&xection);
         if (base_ipt != nullptr) {
           // We have a red-blue intersection point, so we attach the
           // intersecting red and blue halfedges to it.
@@ -451,42 +457,44 @@ public:
 
           // Create the extended point and add the multiplicity.
           Point_2 ex_point(base_ipt->first, red_cell, blue_cell);
-          *oi++ = CGAL::make_object(std::make_pair(ex_point, base_ipt->second));
+          *oi++ =
+            Intersection_result(std::make_pair(ex_point, base_ipt->second));
+          continue;
         }
-        else {
-          overlap_xcv = object_cast<Base_x_monotone_curve_2>(&(*oi));
-          CGAL_assertion(overlap_xcv != nullptr);
-
-          // We have a red-blue overlap, so we mark the curve accordingly.
-          Halfedge_handle_red        red_he;
-          Halfedge_handle_blue       blue_he;
-
-          if (xcv1.color() == RED) {
-            red_he = xcv1.red_halfedge_handle();
-
-            // Overlap can occur only between curves from a different color.
-            CGAL_assertion(xcv2.color() == BLUE);
-            blue_he = xcv2.blue_halfedge_handle();
-          }
-          else {
-            CGAL_assertion((xcv1.color() == BLUE) && (xcv2.color() == RED));
 
-            red_he = xcv2.red_halfedge_handle();
-            blue_he = xcv1.blue_halfedge_handle();
-          }
+        const Base_x_monotone_curve_2* overlap_xcv =
+          boost::get<Base_x_monotone_curve_2>(&xection);
+        CGAL_assertion(overlap_xcv != nullptr);
+
+        // We have a red-blue overlap, so we mark the curve accordingly.
+        Halfedge_handle_red red_he;
+        Halfedge_handle_blue blue_he;
+
+        if (xcv1.color() == RED) {
+          red_he = xcv1.red_halfedge_handle();
+
+          // Overlap can occur only between curves from a different color.
+          CGAL_assertion(xcv2.color() == BLUE);
+          blue_he = xcv2.blue_halfedge_handle();
+        }
+        else {
+          CGAL_assertion((xcv1.color() == BLUE) && (xcv2.color() == RED));
 
-          *oi++ = CGAL::make_object(X_monotone_curve_2(*overlap_xcv,
-                                                       red_he, blue_he));
+          red_he = xcv2.red_halfedge_handle();
+          blue_he = xcv1.blue_halfedge_handle();
         }
+
+        X_monotone_curve_2 cv(*overlap_xcv, red_he, blue_he);
+        *oi++ = Intersection_result(cv);
       }
 
       // Return the past-the-end iterator.
-      return oi_end;
+      return oi;
     }
   };
 
   /*! Obtain an Intersect_2 functor object. */
-  Intersect_2 intersect_2_object() const { return Intersect_2(this); }
+  Intersect_2 intersect_2_object() const { return Intersect_2(*this); }
 
   /*! A functor that splits an arc at a point. */
   class Split_2 {
diff -pruN 5.0.3-1/include/CGAL/Surface_sweep_2/Arr_vert_decomp_ss_visitor.h 5.1-1/include/CGAL/Surface_sweep_2/Arr_vert_decomp_ss_visitor.h
--- 5.0.3-1/include/CGAL/Surface_sweep_2/Arr_vert_decomp_ss_visitor.h	2020-08-01 19:02:13.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_sweep_2/Arr_vert_decomp_ss_visitor.h	2020-09-08 12:14:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Arrangement_on_surface_2/include/CGAL/Surface_sweep_2/Arr_vert_decomp_ss_visitor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Arrangement_on_surface_2/include/CGAL/Surface_sweep_2/Arr_vert_decomp_ss_visitor.h $
 // $Id: Arr_vert_decomp_ss_visitor.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_sweep_2/Curve_comparer.h 5.1-1/include/CGAL/Surface_sweep_2/Curve_comparer.h
--- 5.0.3-1/include/CGAL/Surface_sweep_2/Curve_comparer.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_sweep_2/Curve_comparer.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_sweep_2/include/CGAL/Surface_sweep_2/Curve_comparer.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_sweep_2/include/CGAL/Surface_sweep_2/Curve_comparer.h $
 // $Id: Curve_comparer.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_sweep_2/Curve_pair.h 5.1-1/include/CGAL/Surface_sweep_2/Curve_pair.h
--- 5.0.3-1/include/CGAL/Surface_sweep_2/Curve_pair.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_sweep_2/Curve_pair.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_sweep_2/include/CGAL/Surface_sweep_2/Curve_pair.h $
-// $Id: Curve_pair.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_sweep_2/include/CGAL/Surface_sweep_2/Curve_pair.h $
+// $Id: Curve_pair.h f4fec8c 2020-05-28T11:06:30+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Baruch Zukerman <baruchzu@post.tau.ac.il>
@@ -157,19 +157,19 @@ public:
     return (temp);
   }
 
-  bool operator==(const Self& other)
+  bool operator==(const Self& other) const
   {
     CGAL_precondition(m_container == other.m_container);
     return (m_index == other.m_index);
   }
 
-  bool operator!=(const Self& other)
+  bool operator!=(const Self& other) const
   {
     CGAL_precondition(m_container == other.m_container);
     return !(*this == other);
   }
 
-  unsigned int operator-(const Self& other)
+  unsigned int operator-(const Self& other) const
   {
     CGAL_precondition(m_container == other.m_container);
     return (m_index - other.m_index);
diff -pruN 5.0.3-1/include/CGAL/Surface_sweep_2/Default_event_base.h 5.1-1/include/CGAL/Surface_sweep_2/Default_event_base.h
--- 5.0.3-1/include/CGAL/Surface_sweep_2/Default_event_base.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_sweep_2/Default_event_base.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_sweep_2/include/CGAL/Surface_sweep_2/Default_event_base.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_sweep_2/include/CGAL/Surface_sweep_2/Default_event_base.h $
 // $Id: Default_event_base.h 092cf8e 2019-10-25T16:26:17+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_sweep_2/Default_event.h 5.1-1/include/CGAL/Surface_sweep_2/Default_event.h
--- 5.0.3-1/include/CGAL/Surface_sweep_2/Default_event.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_sweep_2/Default_event.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_sweep_2/include/CGAL/Surface_sweep_2/Default_event.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_sweep_2/include/CGAL/Surface_sweep_2/Default_event.h $
 // $Id: Default_event.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_sweep_2/Default_subcurve.h 5.1-1/include/CGAL/Surface_sweep_2/Default_subcurve.h
--- 5.0.3-1/include/CGAL/Surface_sweep_2/Default_subcurve.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_sweep_2/Default_subcurve.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_sweep_2/include/CGAL/Surface_sweep_2/Default_subcurve.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_sweep_2/include/CGAL/Surface_sweep_2/Default_subcurve.h $
 // $Id: Default_subcurve.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_sweep_2/Default_visitor_base.h 5.1-1/include/CGAL/Surface_sweep_2/Default_visitor_base.h
--- 5.0.3-1/include/CGAL/Surface_sweep_2/Default_visitor_base.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_sweep_2/Default_visitor_base.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_sweep_2/include/CGAL/Surface_sweep_2/Default_visitor_base.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_sweep_2/include/CGAL/Surface_sweep_2/Default_visitor_base.h $
 // $Id: Default_visitor_base.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_sweep_2/Default_visitor.h 5.1-1/include/CGAL/Surface_sweep_2/Default_visitor.h
--- 5.0.3-1/include/CGAL/Surface_sweep_2/Default_visitor.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_sweep_2/Default_visitor.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_sweep_2/include/CGAL/Surface_sweep_2/Default_visitor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_sweep_2/include/CGAL/Surface_sweep_2/Default_visitor.h $
 // $Id: Default_visitor.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_sweep_2/Do_interior_intersect_visitor.h 5.1-1/include/CGAL/Surface_sweep_2/Do_interior_intersect_visitor.h
--- 5.0.3-1/include/CGAL/Surface_sweep_2/Do_interior_intersect_visitor.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_sweep_2/Do_interior_intersect_visitor.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_sweep_2/include/CGAL/Surface_sweep_2/Do_interior_intersect_visitor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_sweep_2/include/CGAL/Surface_sweep_2/Do_interior_intersect_visitor.h $
 // $Id: Do_interior_intersect_visitor.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_sweep_2/Event_comparer.h 5.1-1/include/CGAL/Surface_sweep_2/Event_comparer.h
--- 5.0.3-1/include/CGAL/Surface_sweep_2/Event_comparer.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_sweep_2/Event_comparer.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_sweep_2/include/CGAL/Surface_sweep_2/Event_comparer.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_sweep_2/include/CGAL/Surface_sweep_2/Event_comparer.h $
 // $Id: Event_comparer.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_sweep_2/Intersection_points_visitor.h 5.1-1/include/CGAL/Surface_sweep_2/Intersection_points_visitor.h
--- 5.0.3-1/include/CGAL/Surface_sweep_2/Intersection_points_visitor.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_sweep_2/Intersection_points_visitor.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_sweep_2/include/CGAL/Surface_sweep_2/Intersection_points_visitor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_sweep_2/include/CGAL/Surface_sweep_2/Intersection_points_visitor.h $
 // $Id: Intersection_points_visitor.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_sweep_2/No_intersection_surface_sweep_2_impl.h 5.1-1/include/CGAL/Surface_sweep_2/No_intersection_surface_sweep_2_impl.h
--- 5.0.3-1/include/CGAL/Surface_sweep_2/No_intersection_surface_sweep_2_impl.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_sweep_2/No_intersection_surface_sweep_2_impl.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_sweep_2/include/CGAL/Surface_sweep_2/No_intersection_surface_sweep_2_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_sweep_2/include/CGAL/Surface_sweep_2/No_intersection_surface_sweep_2_impl.h $
 // $Id: No_intersection_surface_sweep_2_impl.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_sweep_2/No_overlap_event_base.h 5.1-1/include/CGAL/Surface_sweep_2/No_overlap_event_base.h
--- 5.0.3-1/include/CGAL/Surface_sweep_2/No_overlap_event_base.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_sweep_2/No_overlap_event_base.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_sweep_2/include/CGAL/Surface_sweep_2/No_overlap_event_base.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_sweep_2/include/CGAL/Surface_sweep_2/No_overlap_event_base.h $
 // $Id: No_overlap_event_base.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_sweep_2/No_overlap_event.h 5.1-1/include/CGAL/Surface_sweep_2/No_overlap_event.h
--- 5.0.3-1/include/CGAL/Surface_sweep_2/No_overlap_event.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_sweep_2/No_overlap_event.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_sweep_2/include/CGAL/Surface_sweep_2/No_overlap_event.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_sweep_2/include/CGAL/Surface_sweep_2/No_overlap_event.h $
 // $Id: No_overlap_event.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_sweep_2/No_overlap_subcurve.h 5.1-1/include/CGAL/Surface_sweep_2/No_overlap_subcurve.h
--- 5.0.3-1/include/CGAL/Surface_sweep_2/No_overlap_subcurve.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_sweep_2/No_overlap_subcurve.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_sweep_2/include/CGAL/Surface_sweep_2/No_overlap_subcurve.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_sweep_2/include/CGAL/Surface_sweep_2/No_overlap_subcurve.h $
 // $Id: No_overlap_subcurve.h 092cf8e 2019-10-25T16:26:17+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_sweep_2/Subcurves_visitor.h 5.1-1/include/CGAL/Surface_sweep_2/Subcurves_visitor.h
--- 5.0.3-1/include/CGAL/Surface_sweep_2/Subcurves_visitor.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_sweep_2/Subcurves_visitor.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_sweep_2/include/CGAL/Surface_sweep_2/Subcurves_visitor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_sweep_2/include/CGAL/Surface_sweep_2/Subcurves_visitor.h $
 // $Id: Subcurves_visitor.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_sweep_2/Surface_sweep_2_debug.h 5.1-1/include/CGAL/Surface_sweep_2/Surface_sweep_2_debug.h
--- 5.0.3-1/include/CGAL/Surface_sweep_2/Surface_sweep_2_debug.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_sweep_2/Surface_sweep_2_debug.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_sweep_2/include/CGAL/Surface_sweep_2/Surface_sweep_2_debug.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_sweep_2/include/CGAL/Surface_sweep_2/Surface_sweep_2_debug.h $
 // $Id: Surface_sweep_2_debug.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_sweep_2/Surface_sweep_2_impl.h 5.1-1/include/CGAL/Surface_sweep_2/Surface_sweep_2_impl.h
--- 5.0.3-1/include/CGAL/Surface_sweep_2/Surface_sweep_2_impl.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_sweep_2/Surface_sweep_2_impl.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_sweep_2/include/CGAL/Surface_sweep_2/Surface_sweep_2_impl.h $
-// $Id: Surface_sweep_2_impl.h 4b7a97c 2019-12-05T11:01:08+01:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_sweep_2/include/CGAL/Surface_sweep_2/Surface_sweep_2_impl.h $
+// $Id: Surface_sweep_2_impl.h 4158542 2020-04-01T12:31:51+03:00 Efi Fogel
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s) : Baruch Zukerman <baruchzu@post.tau.ac.il>
@@ -514,9 +514,10 @@ template <typename Vis>
   CGAL_SS_PRINT_CURVE(c2);
   CGAL_SS_PRINT_EOL();
 
-  CGAL_assertion(event_for_overlap==NULL || event_for_overlap==this->m_currentEvent);
+  CGAL_assertion((event_for_overlap == nullptr) ||
+                 (event_for_overlap == this->m_currentEvent));
 
-  typedef typename Geometry_traits_2::Multiplicity      Multiplicity;
+  auto ctr_min = this->m_traits->construct_min_vertex_2_object();;
 
   CGAL_assertion(c1 != c2);
 
@@ -529,12 +530,13 @@ template <typename Vis>
 
   // handle overlapping curves with common ancesters
   Subcurve_vector all_leaves_diff;
-  Subcurve* first_parent=nullptr;
-  if (c1->originating_subcurve1()!=nullptr || c2->originating_subcurve2()!=nullptr)
+  Subcurve* first_parent = nullptr;
+  if ((c1->originating_subcurve1() != nullptr) ||
+      (c2->originating_subcurve2() != nullptr))
   {
-    // get the subcurve leaves of c1 and of c2. Then extract from the smallest set
-    // the subcurves leaves that are not in the other one. If empty, it means that
-    // a subcurves is completely contained in another one.
+    // get the subcurve leaves of c1 and of c2. Then extract from the smallest
+    // set the subcurves leaves that are not in the other one. If empty, it
+    // means that a subcurves is completely contained in another one.
     first_parent = c1;
     Subcurve* second_parent = c2;
 
@@ -542,8 +544,7 @@ template <typename Vis>
     Subcurve_vector all_leaves_second;
     first_parent->all_leaves(std::back_inserter(all_leaves_first));
     second_parent->all_leaves(std::back_inserter(all_leaves_second));
-    if (all_leaves_second.size() > all_leaves_first.size())
-    {
+    if (all_leaves_second.size() > all_leaves_first.size()) {
       std::swap(first_parent,second_parent);
       std::swap(all_leaves_first,all_leaves_second);
     }
@@ -553,105 +554,108 @@ template <typename Vis>
     std::sort(all_leaves_first.begin(), all_leaves_first.end());
     std::sort(all_leaves_second.begin(), all_leaves_second.end());
 
+    // copies elements from all_leaves_second that are not in all_leaves_first
     std::set_difference(all_leaves_second.begin(), all_leaves_second.end(),
                         all_leaves_first.begin(), all_leaves_first.end(),
-                        std::back_inserter(all_leaves_diff)); // copies elements from all_leaves_second that are not in all_leaves_first
+                        std::back_inserter(all_leaves_diff));
 
-    if (all_leaves_second.size()==all_leaves_diff.size())
-    {
+    if (all_leaves_second.size() == all_leaves_diff.size()) {
       // first_parent has no common parent with second_parent
-      all_leaves_diff.clear(); // clear so that it is not used by _create_overlapping_curve()
+      // clear so that it is not used by _create_overlapping_curve()
+      all_leaves_diff.clear();
     }
-    else
-      if (all_leaves_diff.empty())
-      {
-        // first_parent entirely contains second_parent
-        CGAL_SS_PRINT_TEXT("One overlapping curve entirely contains the other one");
-        CGAL_SS_PRINT_EOL();
+    else if (all_leaves_diff.empty()) {
+      // first_parent entirely contains second_parent
+      CGAL_SS_PRINT_TEXT("One overlapping curve entirely contains the other one");
+      CGAL_SS_PRINT_EOL();
 
-        Event* left_event = (Event*) first_parent->left_event();
-        Event* right_event = (Event*) first_parent->right_event();
+      Event* left_event = (Event*) first_parent->left_event();
+      Event* right_event = (Event*) first_parent->right_event();
 
-        if (!second_parent->is_start_point(left_event))
-          left_event->add_curve_to_left(second_parent);
-        else
-          left_event->remove_curve_from_right(second_parent);
-
-        CGAL_SS_PRINT_CURVE(c1);
-        CGAL_SS_PRINT_TEXT(" + ");
-        CGAL_SS_PRINT_CURVE(c2);
-        CGAL_SS_PRINT_TEXT(" => ");
-        CGAL_SS_PRINT_EOL();
-        CGAL_SS_PRINT_TEXT("  ");
-        CGAL_SS_PRINT_CURVE(first_parent);
-        CGAL_SS_PRINT_EOL();
+      if (! second_parent->is_start_point(left_event))
+        left_event->add_curve_to_left(second_parent);
+      else
+        left_event->remove_curve_from_right(second_parent);
 
-        // Remove second_parent from the left curves of the right end
-        // and add it on the right otherwise
-        if (second_parent->is_end_point(right_event))
-          right_event->remove_curve_from_left(second_parent);
-        else
-          _add_curve_to_right(right_event, second_parent);
-
-        // add the overlapping curve kept of the right of the left end
-        right_event->add_curve_to_left(first_parent);
-        _add_curve_to_right(left_event, first_parent);
+      CGAL_SS_PRINT_CURVE(c1);
+      CGAL_SS_PRINT_TEXT(" + ");
+      CGAL_SS_PRINT_CURVE(c2);
+      CGAL_SS_PRINT_TEXT(" => ");
+      CGAL_SS_PRINT_EOL();
+      CGAL_SS_PRINT_TEXT("  ");
+      CGAL_SS_PRINT_CURVE(first_parent);
+      CGAL_SS_PRINT_EOL();
 
-        this->m_visitor->found_overlap(c1, c2, first_parent);
+      // Remove second_parent from the left curves of the right end
+      // and add it on the right otherwise
+      if (second_parent->is_end_point(right_event))
+        right_event->remove_curve_from_left(second_parent);
+      else
+        _add_curve_to_right(right_event, second_parent);
 
-        CGAL_SS_PRINT_END_EOL("computing intersection");
-        return;
-      }
-      else{
-        CGAL_SS_PRINT_TEXT("Overlap with common ancestors (all_leaves_diff.size() = ");
-        CGAL_SS_PRINT_TEXT(std::to_string(all_leaves_diff.size()).c_str());
-        CGAL_SS_PRINT_TEXT(")");
-        CGAL_SS_PRINT_EOL();
+      // add the overlapping curve kept of the right of the left end
+      right_event->add_curve_to_left(first_parent);
+      _add_curve_to_right(left_event, first_parent);
 
-        // iteratively create the final overlapping (geometric) curve.
-        // This is needed rather than simply computing the intersection of
-        // the last curves of first_parent and second_parent as some traits
-        // classes (such as Arr_curve_data_traits_2) override the Intersect_2
-        // functor and expects the curve to have no common ancesters
-        // (Arr_curve_data_traits_2 is used in the testsuite to sum up
-        //  the overlapping degree of a curve)
-        CGAL_SS_PRINT_TEXT("First parent is: ");
-        CGAL_SS_PRINT_CURVE(first_parent);
-        CGAL_SS_PRINT_EOL();
-        X_monotone_curve_2 xc = first_parent->last_curve();
-        for (typename Subcurve_vector::iterator sc_it=all_leaves_diff.begin();
-             sc_it!=all_leaves_diff.end(); ++sc_it)
-        {
-          CGAL_SS_PRINT_TEXT("Inter with curve: ");
-          CGAL_SS_PRINT_CURVE((*sc_it));
-          CGAL_SS_PRINT_EOL();
+      this->m_visitor->found_overlap(c1, c2, first_parent);
 
-          std::vector<CGAL::Object> inter_res;
+      CGAL_SS_PRINT_END_EOL("computing intersection");
+      return;
+    }
+    else {
+      CGAL_SS_PRINT_TEXT("Overlap with common ancestors (all_leaves_diff.size() = ");
+      CGAL_SS_PRINT_TEXT(std::to_string(all_leaves_diff.size()).c_str());
+      CGAL_SS_PRINT_TEXT(")");
+      CGAL_SS_PRINT_EOL();
 
-          this->m_traits->intersect_2_object()(xc,
-                                               (*sc_it)->last_curve(),
-                                               vector_inserter(inter_res));
-          CGAL_assertion(inter_res.size()==1);
-          CGAL_assertion( CGAL::object_cast< X_monotone_curve_2 >(&inter_res.front())!=nullptr );
-          xc = *CGAL::object_cast< X_monotone_curve_2 >(&inter_res.front());
-        }
+      // iteratively create the final overlapping (geometric) curve.
+      // This is needed rather than simply computing the intersection of
+      // the last curves of first_parent and second_parent as some traits
+      // classes (such as Arr_curve_data_traits_2) override the Intersect_2
+      // functor and expects the curve to have no common ancesters
+      // (Arr_curve_data_traits_2 is used in the testsuite to sum up
+      //  the overlapping degree of a curve)
+      CGAL_SS_PRINT_TEXT("First parent is: ");
+      CGAL_SS_PRINT_CURVE(first_parent);
+      CGAL_SS_PRINT_EOL();
+      X_monotone_curve_2 xc = first_parent->last_curve();
+      for (auto sc_it = all_leaves_diff.begin();
+           sc_it != all_leaves_diff.end(); ++sc_it)
+      {
+        CGAL_SS_PRINT_TEXT("Inter with curve: ");
+        CGAL_SS_PRINT_CURVE((*sc_it));
+        CGAL_SS_PRINT_EOL();
 
-        CGAL_assertion( this->m_queueEventLess(this->m_traits->construct_min_vertex_2_object()(xc),
-                                                event_for_overlap==NULL ? this->m_currentEvent : event_for_overlap)
-          == EQUAL);
-
-        _create_overlapping_curve(xc, c1 , c2, all_leaves_diff, first_parent, event_for_overlap);
-        CGAL_SS_PRINT_END_EOL("computing intersection (overlap with common ancestors)");
-        return;
+        Intersection_vector xections;
+        auto intersector = this->m_traits->intersect_2_object();
+        intersector(xc, (*sc_it)->last_curve(), vector_inserter(xections));
+        CGAL_assertion(xections.size() == 1);
+        auto& item = xections.front();
+        xc = *boost::get<X_monotone_curve_2>(&item);
       }
+
+      CGAL_assertion
+        (this->m_queueEventLess(ctr_min(xc),
+                                event_for_overlap == nullptr ?
+                                this->m_currentEvent : event_for_overlap) ==
+         EQUAL);
+
+      _create_overlapping_curve(xc, c1 , c2, all_leaves_diff, first_parent,
+                                event_for_overlap);
+      CGAL_SS_PRINT_END_EOL("computing intersection (overlap with common ancestors)");
+      return;
+    }
   }
 
+  auto ps_x_fnc = this->m_traits->parameter_space_in_x_2_object();
+  auto ps_y_fnc = this->m_traits->parameter_space_in_y_2_object();
+
   // do compute the intersection of the two curves
   vector_inserter vi(m_x_objects) ;
   vector_inserter vi_end(m_x_objects);
 
-  vi_end =
-    this->m_traits->intersect_2_object()(c1->last_curve(), c2->last_curve(), vi);
+  auto intersector = this->m_traits->intersect_2_object();
+  vi_end = intersector(c1->last_curve(), c2->last_curve(), vi);
 
   if (vi == vi_end) {
     CGAL_SS_PRINT_END_EOL("Computing intersection (no intersection)");
@@ -660,29 +664,19 @@ template <typename Vis>
 
   // The two subCurves may start at the same point, in that case we ignore the
   // first intersection point.
-
-  const Arr_parameter_space ps_x1 =
-    this->m_traits->parameter_space_in_x_2_object()(c1->last_curve(),
-                                                    ARR_MIN_END);
-  const Arr_parameter_space ps_y1 =
-    this->m_traits->parameter_space_in_y_2_object()(c1->last_curve(),
-                                                    ARR_MIN_END);
-  const Arr_parameter_space ps_x2 =
-    this->m_traits->parameter_space_in_x_2_object()(c2->last_curve(),
-                                                    ARR_MIN_END);
-  const Arr_parameter_space ps_y2 =
-    this->m_traits->parameter_space_in_y_2_object()(c2->last_curve(),
-                                                    ARR_MIN_END);
+  Arr_parameter_space ps_x1 = ps_x_fnc(c1->last_curve(), ARR_MIN_END);
+  Arr_parameter_space ps_y1 = ps_y_fnc(c1->last_curve(), ARR_MIN_END);
+  Arr_parameter_space ps_x2 = ps_x_fnc(c2->last_curve(), ARR_MIN_END);
+  Arr_parameter_space ps_y2 = ps_y_fnc(c2->last_curve(), ARR_MIN_END);
 
   if ((ps_x1 == ps_x2) && (ps_y1 == ps_y2) &&
       ((ps_x1 != ARR_INTERIOR) || (ps_y1 != ARR_INTERIOR)) &&
       this->m_traits->is_closed_2_object()(c1->last_curve(), ARR_MIN_END) &&
       this->m_traits->is_closed_2_object()(c2->last_curve(), ARR_MIN_END))
   {
-    if ( object_cast<std::pair<Point_2, Multiplicity> >(&(*vi)) != nullptr
-         && this->m_traits->equal_2_object()
-        (this->m_traits->construct_min_vertex_2_object()(c1->last_curve()),
-         this->m_traits->construct_min_vertex_2_object()(c2->last_curve())))
+    if ((boost::get<Intersection_point>(&(*vi)) != nullptr) &&
+        this->m_traits->equal_2_object()(ctr_min(c1->last_curve()),
+                                         ctr_min(c2->last_curve())))
     {
       CGAL_SS_PRINT_TEXT("Skipping common left endpoint on boundary ...");
       CGAL_SS_PRINT_EOL();
@@ -698,7 +692,7 @@ template <typename Vis>
     vector_inserter vi_last = vi_end;
 
     --vi_last;
-    if (object_cast<std::pair<Point_2, Multiplicity> >(&(*vi_last)) != nullptr) {
+    if (boost::get<Intersection_point>(&(*vi_last)) != nullptr) {
       CGAL_SS_PRINT_TEXT("Skipping common right endpoint...");
       CGAL_SS_PRINT_EOL();
       --vi_end;
@@ -708,18 +702,10 @@ template <typename Vis>
     // In case both right curve-ends have boundary conditions and are not
     // open, check whether the right endpoints are the same. If they are,
     // skip the last intersection point.
-    const Arr_parameter_space ps_x1 =
-      this->m_traits->parameter_space_in_x_2_object()(c1->last_curve(),
-                                                      ARR_MAX_END);
-    const Arr_parameter_space ps_y1 =
-      this->m_traits->parameter_space_in_y_2_object()(c1->last_curve(),
-                                                      ARR_MAX_END);
-    const Arr_parameter_space ps_x2 =
-      this->m_traits->parameter_space_in_x_2_object()(c2->last_curve(),
-                                                      ARR_MAX_END);
-    const Arr_parameter_space ps_y2 =
-      this->m_traits->parameter_space_in_y_2_object()(c2->last_curve(),
-                                                      ARR_MAX_END);
+    Arr_parameter_space ps_x1 = ps_x_fnc(c1->last_curve(), ARR_MAX_END);
+    Arr_parameter_space ps_y1 = ps_y_fnc(c1->last_curve(), ARR_MAX_END);
+    Arr_parameter_space ps_x2 = ps_x_fnc(c2->last_curve(), ARR_MAX_END);
+    Arr_parameter_space ps_y2 = ps_y_fnc(c2->last_curve(), ARR_MAX_END);
 
     if ((ps_x1 == ps_x2) && (ps_y1 == ps_y2) &&
         ((ps_x1 != ARR_INTERIOR) || (ps_y2 != ARR_INTERIOR)) &&
@@ -733,8 +719,7 @@ template <typename Vis>
         vector_inserter vi_last = vi_end;
 
         --vi_last;
-        if (object_cast<std::pair<Point_2, Multiplicity> >(&(*vi_last)) != nullptr)
-        {
+        if (boost::get<Intersection_point>(&(*vi_last)) != nullptr) {
           CGAL_SS_PRINT_TEXT("Skipping common right endpoint on boundary...");
           CGAL_SS_PRINT_EOL();
           --vi_end;
@@ -743,12 +728,11 @@ template <typename Vis>
     }
   }
 
-  const std::pair<Point_2, Multiplicity>* xp_point;
-
   // Efi: why not skipping in a loop?check only one (that is, why not in a loop)?
-  // SL: curves are split and no event strictly before the current event should be reported
+  // SL: curves are split and no event strictly before the current event should
+  //     be reported
   if (vi != vi_end) {
-    xp_point = object_cast<std::pair<Point_2, Multiplicity> >(&(*vi));
+    const Intersection_point* xp_point = boost::get<Intersection_point>(&(*vi));
     if (xp_point != nullptr) {
       // Skip the intersection point if it is not larger than the current
       // event.
@@ -762,9 +746,8 @@ template <typename Vis>
 
   bool first_i = true;
   for (; vi != vi_end; ++vi) {
-    unsigned int multiplicity = 0;
-
-    xp_point = object_cast<std::pair<Point_2, Multiplicity> >(&(*vi));
+    Multiplicity multiplicity = 0;
+    const Intersection_point* xp_point = boost::get<Intersection_point>(&(*vi));
     if (xp_point != nullptr) {
       Point_2 xp = xp_point->first;
       multiplicity = xp_point->second;
@@ -773,13 +756,14 @@ template <typename Vis>
       _create_intersection_point(xp, multiplicity, c1, c2);
     }
     else {
-      X_monotone_curve_2 icv = *object_cast<X_monotone_curve_2>(&(*vi));
+      const X_monotone_curve_2 icv = *boost::get<X_monotone_curve_2>(&(*vi));
       // CGAL_assertion(icv != nullptr);
 
       CGAL_SS_PRINT_TEXT("Found an overlap");
       CGAL_SS_PRINT_EOL();
+      // event_for_overlap is only valid for the first intersection
       _create_overlapping_curve(icv, c1 , c2, all_leaves_diff, first_parent,
-                                first_i ? event_for_overlap:NULL); // event_for_overlap is only valid for the first intersection
+                                first_i ? event_for_overlap : NULL);
     }
     first_i = false;
   }
diff -pruN 5.0.3-1/include/CGAL/Surface_sweep_2/Surface_sweep_2_utils.h 5.1-1/include/CGAL/Surface_sweep_2/Surface_sweep_2_utils.h
--- 5.0.3-1/include/CGAL/Surface_sweep_2/Surface_sweep_2_utils.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_sweep_2/Surface_sweep_2_utils.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_sweep_2/include/CGAL/Surface_sweep_2/Surface_sweep_2_utils.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_sweep_2/include/CGAL/Surface_sweep_2/Surface_sweep_2_utils.h $
 // $Id: Surface_sweep_2_utils.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_sweep_2_algorithms.h 5.1-1/include/CGAL/Surface_sweep_2_algorithms.h
--- 5.0.3-1/include/CGAL/Surface_sweep_2_algorithms.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_sweep_2_algorithms.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_sweep_2/include/CGAL/Surface_sweep_2_algorithms.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_sweep_2/include/CGAL/Surface_sweep_2_algorithms.h $
 // $Id: Surface_sweep_2_algorithms.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Surface_sweep_2.h 5.1-1/include/CGAL/Surface_sweep_2.h
--- 5.0.3-1/include/CGAL/Surface_sweep_2.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Surface_sweep_2.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_sweep_2/include/CGAL/Surface_sweep_2.h $
-// $Id: Surface_sweep_2.h 092cf8e 2019-10-25T16:26:17+02:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_sweep_2/include/CGAL/Surface_sweep_2.h $
+// $Id: Surface_sweep_2.h 4158542 2020-04-01T12:31:51+03:00 Efi Fogel
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s) : Baruch Zukerman <baruchzu@post.tau.ac.il>
@@ -90,6 +90,7 @@ public:
   typedef typename Base::Traits_adaptor_2               Traits_adaptor_2;
   typedef typename Traits_adaptor_2::Point_2            Point_2;
   typedef typename Traits_adaptor_2::X_monotone_curve_2 X_monotone_curve_2;
+  typedef typename Traits_adaptor_2::Multiplicity       Multiplicity;
 
   typedef typename Base::Event_queue_iterator           Event_queue_iterator;
   typedef typename Event::Subcurve_iterator             Event_subcurve_iterator;
@@ -106,11 +107,14 @@ public:
   typedef CGAL::Surface_sweep_2::Equal_curve_pair<Subcurve>
                                                         Equal_curve_pair;
   typedef boost::unordered_set<Curve_pair, Curve_pair_hasher, Equal_curve_pair>
-                                                      Curve_pair_set;
-
-  typedef std::vector<Object>                           Object_vector;
-  typedef random_access_input_iterator<Object_vector>   vector_inserter;
+                                                        Curve_pair_set;
 
+  typedef std::pair<Point_2, Multiplicity>              Intersection_point;
+  typedef boost::variant<Intersection_point, X_monotone_curve_2>
+                                                        Intersection_result;
+  typedef std::vector<Intersection_result>              Intersection_vector;
+  typedef random_access_input_iterator<Intersection_vector>
+                                                        vector_inserter;
   typedef typename Base::Subcurve_alloc                 Subcurve_alloc;
 protected:
   // Data members:
@@ -121,7 +125,7 @@ protected:
   Curve_pair_set m_curves_pair_set;  // A lookup table of pairs of Subcurves
                                      // that have been intersected.
 
-  std::vector<Object> m_x_objects;   // Auxiliary vector for storing the
+  Intersection_vector m_x_objects;   // Auxiliary vector for storing the
                                      // intersection objects.
 
   X_monotone_curve_2 sub_cv1;        // Auxiliary varibales
diff -pruN 5.0.3-1/include/CGAL/Sweep_line_2_algorithms.h 5.1-1/include/CGAL/Sweep_line_2_algorithms.h
--- 5.0.3-1/include/CGAL/Sweep_line_2_algorithms.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Sweep_line_2_algorithms.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_sweep_2/include/CGAL/Sweep_line_2_algorithms.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_sweep_2/include/CGAL/Sweep_line_2_algorithms.h $
 // $Id: Sweep_line_2_algorithms.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/sweep_observer.h 5.1-1/include/CGAL/sweep_observer.h
--- 5.0.3-1/include/CGAL/sweep_observer.h	2020-08-01 19:02:34.000000000 +0000
+++ 5.1-1/include/CGAL/sweep_observer.h	2020-09-08 12:15:12.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Nef_2/include/CGAL/sweep_observer.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Nef_2/include/CGAL/sweep_observer.h $
 // $Id: sweep_observer.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/tags.h 5.1-1/include/CGAL/tags.h
--- 5.0.3-1/include/CGAL/tags.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/tags.h	2020-09-08 12:15:32.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/tags.h $
-// $Id: tags.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/tags.h $
+// $Id: tags.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -26,7 +26,7 @@ namespace CGAL {
 struct Void {};
 
 // Boolean_tag<bool> is a model of the Boost Integral Constant concept.
-//   https://www.boost.org/libs/mpl/doc/refmanual/integral-constant.html
+// https://www.boost.org/libs/mpl/doc/refmanual/integral-constant.html
 template <bool b>
 struct Boolean_tag {
   typedef boost::mpl::integral_c_tag tag;
@@ -58,6 +58,12 @@ struct Null_functor {
 struct Sequential_tag {};
 struct Parallel_tag : public Sequential_tag {};
 
+#ifdef CGAL_LINKED_WITH_TBB
+typedef CGAL::Parallel_tag Parallel_if_available_tag;
+#else
+typedef CGAL::Sequential_tag Parallel_if_available_tag;
+#endif
+
 // A function that asserts a specific compile time tag
 // forcing its two arguments to have equal type.
 template <class Base>
diff -pruN 5.0.3-1/include/CGAL/TDS_full_cell_default_storage_policy.h 5.1-1/include/CGAL/TDS_full_cell_default_storage_policy.h
--- 5.0.3-1/include/CGAL/TDS_full_cell_default_storage_policy.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/TDS_full_cell_default_storage_policy.h	2020-09-08 12:15:49.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation/include/CGAL/TDS_full_cell_default_storage_policy.h $
-// $Id: TDS_full_cell_default_storage_policy.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation/include/CGAL/TDS_full_cell_default_storage_policy.h $
+// $Id: TDS_full_cell_default_storage_policy.h 378554e 2020-03-06T16:42:23+01:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)    : Samuel Hornus
@@ -44,7 +44,7 @@ struct TFC_data< Vertex_handle, Full_cel
     : vertices_(dmax+1), neighbors_(dmax+1)
     {}
     void*   for_compact_container() const { return vertices_.for_compact_container(); }
-    void* & for_compact_container()       { return vertices_.for_compact_container(); }
+    void    for_compact_container(void *p){ vertices_.for_compact_container(p); }
     int dimension() const { return ( vertices_.size() - 1 ); }
     void set_mirror_index(const int, const int) {}
 #ifdef BOOST_NO_INT64_T
diff -pruN 5.0.3-1/include/CGAL/TDS_full_cell_mirror_storage_policy.h 5.1-1/include/CGAL/TDS_full_cell_mirror_storage_policy.h
--- 5.0.3-1/include/CGAL/TDS_full_cell_mirror_storage_policy.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/TDS_full_cell_mirror_storage_policy.h	2020-09-08 12:15:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation/include/CGAL/TDS_full_cell_mirror_storage_policy.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation/include/CGAL/TDS_full_cell_mirror_storage_policy.h $
 // $Id: TDS_full_cell_mirror_storage_policy.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/test_FPU_rounding_mode_impl.h 5.1-1/include/CGAL/test_FPU_rounding_mode_impl.h
--- 5.0.3-1/include/CGAL/test_FPU_rounding_mode_impl.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/test_FPU_rounding_mode_impl.h	2020-09-08 12:15:17.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/test_FPU_rounding_mode_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/test_FPU_rounding_mode_impl.h $
 // $Id: test_FPU_rounding_mode_impl.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Tetrahedral_remeshing/internal/collapse_short_edges.h 5.1-1/include/CGAL/Tetrahedral_remeshing/internal/collapse_short_edges.h
--- 5.0.3-1/include/CGAL/Tetrahedral_remeshing/internal/collapse_short_edges.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Tetrahedral_remeshing/internal/collapse_short_edges.h	2020-09-08 12:15:49.000000000 +0000
@@ -0,0 +1,1315 @@
+// Copyright (c) 2020 GeometryFactory (France) and Telecom Paris (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Tetrahedral_remeshing/include/CGAL/Tetrahedral_remeshing/internal/collapse_short_edges.h $
+// $Id: collapse_short_edges.h 36386b0 2020-07-06T15:08:33+02:00 Jane Tournois
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Jane Tournois, Noura Faraj, Jean-Marc Thiery, Tamy Boubekeur
+
+#ifndef CGAL_INTERNAL_COLLAPSE_SHORT_EDGES_H
+#define CGAL_INTERNAL_COLLAPSE_SHORT_EDGES_H
+
+#include <CGAL/license/Tetrahedral_remeshing.h>
+
+#include <boost/bimap.hpp>
+#include <boost/bimap/set_of.hpp>
+#include <boost/bimap/multiset_of.hpp>
+#include <boost/array.hpp>
+#include <boost/unordered_set.hpp>
+#include <boost/container/small_vector.hpp>
+
+#include <vector>
+#include <algorithm>
+#include <unordered_map>
+#include <utility>
+#include <unordered_set>
+
+#include <CGAL/Mesh_3/tet_soup_to_c3t3.h>
+#include <CGAL/utility.h>
+#include <CGAL/Tetrahedral_remeshing/internal/tetrahedral_remeshing_helpers.h>
+
+
+namespace CGAL
+{
+namespace Tetrahedral_remeshing
+{
+namespace internal
+{
+enum Edge_type     { FEATURE, BOUNDARY, INSIDE, MIXTE,
+                     NO_COLLAPSE, INVALID, IMAGINARY, MIXTE_IMAGINARY, HULL_EDGE };
+enum Collapse_type { TO_MIDPOINT, TO_V0, TO_V1, IMPOSSIBLE };
+enum Result_type   { VALID,
+                     V_PROBLEM, C_PROBLEM, E_PROBLEM,
+                     TOPOLOGICAL_PROBLEM, ORIENTATION_PROBLEM, SHARED_NEIGHBOR_PROBLEM };
+
+template<typename C3t3>
+class CollapseTriangulation
+{
+  typedef typename C3t3::Triangulation                        Tr;
+  typedef typename C3t3::Edge                                 Edge;
+  typedef typename C3t3::Cell_handle                          Cell_handle;
+  typedef typename C3t3::Vertex_handle                        Vertex_handle;
+  typedef typename C3t3::Subdomain_index                      Subdomain_index;
+  typedef typename C3t3::Surface_patch_index                  Surface_patch_index;
+  typedef typename C3t3::Triangulation::Point                 Point_3;
+  typedef typename C3t3::Triangulation::Geom_traits::Vector_3 Vector_3;
+
+public:
+  CollapseTriangulation(const Edge& e,
+                        const std::unordered_set<Cell_handle>& cells_to_insert,
+                        Collapse_type _collapse_type)
+    : collapse_type(_collapse_type)
+    , v0_init(e.first->vertex(e.second))
+    , v1_init(e.first->vertex(e.third))
+  {
+    typedef std::array<int, 3> Facet; // 3 = id
+    typedef std::array<int, 5> Tet_with_ref; // first 4 = id, fifth = reference
+
+    std::unordered_set<Vertex_handle> vertices_to_insert;
+    for (Cell_handle ch : cells_to_insert)
+    {
+      for(int i = 0; i < 4; ++i)
+        vertices_to_insert.insert(ch->vertex(i));
+    }
+
+    CGAL_assertion(vertices_to_insert.end()
+      != std::find(vertices_to_insert.begin(), vertices_to_insert.end(), v1_init));
+    CGAL_assertion(vertices_to_insert.end()
+      != std::find(vertices_to_insert.begin(), vertices_to_insert.end(), v0_init));
+
+    std::unordered_map<Vertex_handle, int> v2i;/*vertex of main tr - vertex of collapse tr*/
+
+    //To add the vertices only once
+    std::vector<Point_3> points;
+    int index = 0;
+    for (Vertex_handle vh : vertices_to_insert)
+    {
+      if (v2i.find(vh) == v2i.end())
+      {
+        points.push_back(vh->point());
+        v2i.insert(std::make_pair(vh, index++));
+      }
+    }
+
+    std::vector<Tet_with_ref> finite_cells;
+    for (Cell_handle ch : cells_to_insert)
+    {
+      Tet_with_ref t = { { v2i.at(ch->vertex(0)),
+                           v2i.at(ch->vertex(1)),
+                           v2i.at(ch->vertex(2)),
+                           v2i.at(ch->vertex(3)),
+                           ch->subdomain_index() } };
+      finite_cells.push_back(t);
+    }
+
+    // finished
+    std::vector<Vertex_handle> new_vertices;
+    std::map<Facet, typename C3t3::Surface_patch_index> border_facets;
+     if (CGAL::build_triangulation<Tr, false>(triangulation,
+                                              points, finite_cells, border_facets,
+                                              new_vertices, false/*verbose*/))
+    {
+      CGAL_assertion(triangulation.tds().is_valid());
+      CGAL_assertion(triangulation.infinite_vertex() == new_vertices[0]);
+
+      // update()
+      vh0 = new_vertices[v2i.at(v0_init) + 1];
+      vh1 = new_vertices[v2i.at(v1_init) + 1];
+
+      Cell_handle ch;
+      int i0, i1;
+      not_an_edge = true;
+      CGAL_assertion(triangulation.tds().is_vertex(vh0));
+      CGAL_assertion(triangulation.tds().is_vertex(vh1));
+      if (triangulation.is_edge(vh0, vh1, ch, i0, i1))
+      {
+        edge = Edge(ch, i0, i1);
+        not_an_edge = false;
+      }
+    }
+#ifdef CGAL_TETRAHEDRAL_REMESHING_VERBOSE
+    else
+      std::cout << "Warning : CollapseTriangulation is not valid!" << std::endl;
+#endif
+  }
+
+  Result_type collapse()
+  {
+    if (not_an_edge)
+    {
+#ifdef CGAL_TETRAHEDRAL_REMESHING_VERBOSE
+      std::cout << "CollapseTriangulation::Not an edge..." << std::endl;
+#endif
+      return E_PROBLEM;
+    }
+    else
+    {
+      Vector_3 v0_new_pos = vec(vh0->point());
+
+      if (collapse_type == TO_MIDPOINT){
+        v0_new_pos = v0_new_pos + 0.5 * Vector_3(point(vh0->point()), point(vh1->point()));
+      }
+      else if (collapse_type == TO_V1){
+        v0_new_pos = vec(point(vh1->point()));
+      }
+
+      boost::unordered_set<Cell_handle> invalid_cells;
+
+      typedef typename Tr::Cell_circulator Cell_circulator;
+      Cell_circulator circ = triangulation.incident_cells(edge);
+      Cell_circulator done = circ;
+
+      std::vector<Cell_handle> cells_to_remove;
+
+      //Update the vertex before removing it
+      std::vector<Cell_handle> find_incident;
+      triangulation.incident_cells(vh0, std::back_inserter(find_incident));
+
+      std::vector<Cell_handle> cells_to_update;
+      triangulation.incident_cells(vh1, std::back_inserter(cells_to_update));
+
+      do
+      {
+        int v0_id = circ->index(vh0);
+        int v1_id = circ->index(vh1);
+
+        Cell_handle n0_ch = circ->neighbor(v0_id);
+        Cell_handle n1_ch = circ->neighbor(v1_id);
+
+        int ch_id_in_n0 = n0_ch->index(circ);
+        int ch_id_in_n1 = n1_ch->index(circ);
+
+        if (n0_ch->has_neighbor(n1_ch))
+          return SHARED_NEIGHBOR_PROBLEM;
+
+        //Update neighbors before removing cell
+        n0_ch->set_neighbor(ch_id_in_n0, n1_ch);
+        n1_ch->set_neighbor(ch_id_in_n1, n0_ch);
+
+        Subdomain_index si_n0 = n0_ch->subdomain_index();
+        Subdomain_index si_n1 = n1_ch->subdomain_index();
+        Subdomain_index si = circ->subdomain_index();
+
+        if (si_n0 != si && si_n1 != si)
+          return TOPOLOGICAL_PROBLEM;
+
+        if ( triangulation.is_infinite(n0_ch->vertex(ch_id_in_n0))
+             && triangulation.is_infinite(n1_ch->vertex(ch_id_in_n1)))
+          return TOPOLOGICAL_PROBLEM;
+
+        if ( triangulation.is_infinite(n0_ch)
+             && triangulation.is_infinite(n1_ch)
+             && !triangulation.is_infinite(circ))
+          return TOPOLOGICAL_PROBLEM;
+
+        cells_to_remove.push_back(circ);
+
+        invalid_cells.insert(circ);
+
+      } while (++circ != done);
+
+
+      vh0->set_point(Point_3(v0_new_pos.x(), v0_new_pos.y(), v0_new_pos.z()));
+      vh1->set_point(Point_3(v0_new_pos.x(), v0_new_pos.y(), v0_new_pos.z()));
+
+      Vertex_handle infinite_vertex = triangulation.infinite_vertex();
+
+      bool v0_updated = false;
+      for (const Cell_handle ch : find_incident)
+      {
+        if (invalid_cells.find(ch) == invalid_cells.end()) //valid cell
+        {
+          if (triangulation.is_infinite(ch))
+            infinite_vertex->set_cell(ch);
+          else {
+            vh0->set_cell(ch);
+            v0_updated = true;
+          }
+        }
+      }
+
+      //Update the vertex before removing it
+      for (Cell_handle ch : cells_to_update)
+      {
+        if (invalid_cells.find(ch) == invalid_cells.end()) //valid cell
+        {
+          ch->set_vertex(ch->index(vh1), vh0);
+
+          if (triangulation.is_infinite(ch))
+            infinite_vertex->set_cell(ch);
+          else {
+            if (!v0_updated) {
+              vh0->set_cell(ch);
+              v0_updated = true;
+            }
+          }
+        }
+      }
+
+      if (!v0_updated){
+        std::cout << "CollapseTriangulation::PB i cell not valid!!!" << std::endl;
+        return V_PROBLEM;
+      }
+      triangulation.tds().delete_vertex(vh1);
+
+      //Removing cells
+      for (Cell_handle ch : cells_to_remove){
+        triangulation.tds().delete_cell(ch);
+      }
+
+      for (Cell_handle cit : triangulation.finite_cell_handles())
+      {
+        if (!is_well_oriented(triangulation, cit))
+          return ORIENTATION_PROBLEM;
+        if (!triangulation.tds().is_valid(cit, true))
+          return C_PROBLEM;
+      }
+
+      for (Vertex_handle vit : triangulation.finite_vertex_handles())
+      {
+        if (!triangulation.tds().is_valid(vit, true))
+          return V_PROBLEM;
+      }
+
+      //int si_nb_vh0 = nb_incident_subdomains(vh0, c3t3);
+      //int si_nb_vh1 = nb_incident_subdomains(vh1, c3t3);
+      //int vertices_subdomain_nb_vh0 = std::max(si_nb_vh0, si_nb_vh1);
+      //bool is_on_hull_vh0 = is_on_convex_hull(vh0, c3t3) || is_on_convex_hull(vh1, c3t3);
+
+      //if( is_valid_for_domains() )
+      return VALID;
+
+      // return TOPOLOGICAL_PROBLEM;
+    }
+  }
+
+protected:
+  Tr triangulation;
+
+  const Collapse_type collapse_type;
+
+  const Vertex_handle v0_init;
+  const Vertex_handle v1_init;
+
+  Vertex_handle vh0;
+  Vertex_handle vh1;
+
+  Edge edge;
+
+  bool not_an_edge;
+};
+
+
+
+template<typename C3t3, typename CellSelector>
+Collapse_type get_collapse_type(const typename C3t3::Edge& edge,
+                                const C3t3& c3t3,
+                                CellSelector cell_selector)
+{
+  bool update_v0 = false;
+  bool update_v1 = false;
+  get_edge_info(edge, update_v0, update_v1, c3t3, cell_selector);
+
+  if (update_v0 && update_v1) return TO_MIDPOINT;
+  else if (update_v0)         return TO_V1;
+  else if (update_v1)         return TO_V0;
+  else                        return IMPOSSIBLE;
+}
+
+//template<typename C3t3>
+//Edge_type get_edge_type(const typename C3t3::Edge& edge,
+//                        const C3t3& c3t3)
+//{
+//  typedef typename C3t3::Vertex_handle Vertex_handle;
+//  typedef typename C3t3::Triangulation::Cell_circulator Cell_circulator;
+//  typedef typename C3t3::Subdomain_index Subdomain_index;
+
+//  const Vertex_handle & v0 = edge.first->vertex(edge.second);
+//  const Vertex_handle & v1 = edge.first->vertex(edge.third);
+
+//  const int dim0 = c3t3.in_dimension(v0);
+//  const int dim1 = c3t3.in_dimension(v1);
+
+//  const bool is_v0_on_hull = is_on_convex_hull(v0, c3t3);
+//  const bool is_v1_on_hull = is_on_convex_hull(v1, c3t3);
+
+//  if (c3t3.is_in_complex(edge))
+//    return FEATURE;
+
+//  else if (dim0 == 3 && dim1 == 3)
+//    return INSIDE;
+
+//  else if (dim0 == 2 && dim1 == 2)
+//  {
+//    Cell_circulator circ = c3t3.triangulation().incident_cells(edge);
+//    Cell_circulator done = circ;
+
+//    std::vector<Subdomain_index> indices;
+//    do
+//    {
+//      Subdomain_index current_si = circ->subdomain_index();
+
+//      if (std::find(indices.begin(), indices.end(), current_si) == indices.end()) {
+//        indices.push_back(current_si);
+//      }
+
+//      Subdomain_index si_n0 = circ->neighbor(circ->index(v0))->subdomain_index();
+//      Subdomain_index si_n1 = circ->neighbor(circ->index(v1))->subdomain_index();
+//      if (si_n0 == si_n1 && si_n0 != current_si)
+//        return NO_COLLAPSE;
+
+//    } while (++circ != done);
+
+//    const std::size_t nb_si_v0 = nb_incident_subdomains(v0, c3t3);
+//    const std::size_t nb_si_v1 = nb_incident_subdomains(v1, c3t3);
+
+//    if (indices.size() >= (std::min)(nb_si_v0, nb_si_v1)) {
+//      return BOUNDARY;
+//    }
+//  }
+
+//  //std::cerr << "ERROR : get_edge_type did not return anything valid!" << std::endl;
+//  return NO_COLLAPSE;
+//}
+
+template<typename C3t3>
+bool is_valid_collapse(const typename C3t3::Edge& edge,
+                       const C3t3& c3t3)
+{
+  typedef typename C3t3::Vertex_handle Vertex_handle;
+  typedef typename C3t3::Cell_handle   Cell_handle;
+  typedef typename C3t3::Triangulation::Cell_circulator Cell_circulator;
+
+  const Vertex_handle v0 = edge.first->vertex(edge.second);
+  const Vertex_handle v1 = edge.first->vertex(edge.third);
+
+  Cell_circulator circ = c3t3.triangulation().incident_cells(edge);
+  Cell_circulator done = circ;
+  do
+  {
+    int v0_id = circ->index(v0);
+    int v1_id = circ->index(v1);
+
+    Cell_handle n0_ch = circ->neighbor(v0_id);
+    Cell_handle n1_ch = circ->neighbor(v1_id);
+
+    if (n0_ch->has_vertex(v0)
+        || n1_ch->has_vertex(v1)
+        || n0_ch->has_neighbor(n1_ch))
+    {
+#ifdef CGAL_DEBUG_TET_REMESHING_IN_PLUGIN
+      if (c3t3.is_in_complex(edge))
+        ++nb_invalid_collapse_short;
+#endif
+      return false;
+    }
+  }
+  while (++circ != done);
+
+  return true;
+}
+
+template<typename C3t3>
+bool is_valid_collapse(const typename C3t3::Edge& edge,
+                       const Collapse_type& collapse_type,
+                       const typename C3t3::Triangulation::Point& new_pos,
+                       const C3t3& c3t3)
+{
+  typedef typename C3t3::Vertex_handle        Vertex_handle;
+  typedef typename C3t3::Cell_handle          Cell_handle;
+  typedef typename C3t3::Triangulation::Point Point;
+
+  const Vertex_handle v0 = edge.first->vertex(edge.second);
+  const Vertex_handle v1 = edge.first->vertex(edge.third);
+
+#ifdef CGAL_DEBUG_TET_REMESHING_IN_PLUGIN
+  const bool in_cx = c3t3.is_in_complex(edge);
+  if (in_cx)
+  {
+    if (collapse_type == TO_MIDPOINT)
+      nb_test_midpoint++;
+    else if (collapse_type == TO_V1)
+      nb_test_v1++;
+    else
+      nb_test_v0++;
+  }
+#endif
+
+  if (collapse_type == TO_V1 || collapse_type == TO_MIDPOINT)
+  {
+    std::vector<Cell_handle> cells_to_check;
+    c3t3.triangulation().finite_incident_cells(v0,
+        std::back_inserter(cells_to_check));
+
+    for (const Cell_handle ch : cells_to_check)
+    {
+      if (!ch->has_vertex(v1))
+      {
+        //check orientation
+        boost::array<Point, 4> pts = { ch->vertex(0)->point(),
+                                       ch->vertex(1)->point(),
+                                       ch->vertex(2)->point(),
+                                       ch->vertex(3)->point()};
+        pts[ch->index(v0)] = new_pos;
+        if (CGAL::orientation(point(pts[0]), point(pts[1]), point(pts[2]), point(pts[3]))
+            != CGAL::POSITIVE)
+        {
+#ifdef CGAL_DEBUG_TET_REMESHING_IN_PLUGIN
+          if (in_cx)
+          {
+            if (collapse_type == TO_MIDPOINT)
+              nb_orientation_midpoint++;
+            else
+              nb_orientation_v1++;
+          }
+#endif
+          return false;
+        }
+      }
+    }
+  }
+  if (collapse_type == TO_V0 || collapse_type == TO_MIDPOINT)
+  {
+    std::vector<Cell_handle> cells_to_check;
+    c3t3.triangulation().finite_incident_cells(v1,
+        std::back_inserter(cells_to_check));
+
+    for (const Cell_handle ch : cells_to_check)
+    {
+      if (!ch->has_vertex(v0))
+      {
+        //check orientation
+        boost::array<Point, 4> pts = { ch->vertex(0)->point(),
+                                       ch->vertex(1)->point(),
+                                       ch->vertex(2)->point(),
+                                       ch->vertex(3)->point() };
+        pts[ch->index(v1)] = new_pos;
+        if (CGAL::orientation(point(pts[0]), point(pts[1]), point(pts[2]), point(pts[3]))
+            != CGAL::POSITIVE)
+        {
+#ifdef CGAL_DEBUG_TET_REMESHING_IN_PLUGIN
+          if (in_cx)
+          {
+            if (collapse_type == TO_MIDPOINT)
+              nb_orientation_midpoint++;
+            else
+              nb_orientation_v0++;
+          }
+#endif
+          return false;
+        }
+      }
+    }
+  }
+
+  return is_valid_collapse(edge, c3t3);
+}
+
+template<typename Facet, typename Vh>
+bool facet_has_edge(const Facet& f, const Vh v0, const Vh v1)
+{
+  std::array<std::array<int, 2>, 3> edges = {{ {{1,2}}, {{2,3}}, {{3,1}} }};
+
+  for (int i = 0; i < 3; ++i)
+  {
+    const std::array<int, 2>& ei = edges[i];
+    if ( f.first->vertex((f.second + ei[0]) % 4) == v0
+      && f.first->vertex((f.second + ei[1]) % 4) == v1)
+      return true;
+    if ( f.first->vertex((f.second + ei[0]) % 4) == v1
+      && f.first->vertex((f.second + ei[1]) % 4) == v0)
+      return true;
+  }
+  return false;
+}
+
+template<typename C3t3, typename CellSelector>
+bool collapse_preserves_surface_star(const typename C3t3::Edge& edge,
+                                     const C3t3& c3t3,
+                                     const typename C3t3::Triangulation::Point& new_pos,
+                                     const CellSelector& cell_selector)
+{
+  typedef typename C3t3::Triangulation       Tr;
+  typedef typename C3t3::Vertex_handle       Vertex_handle;
+  typedef typename C3t3::Facet               Facet;
+  typedef typename Tr::Geom_traits::Vector_3 Vector_3;
+  typedef typename Tr::Geom_traits::Point_3  Point_3;
+
+  const Tr& tr = c3t3.triangulation();
+
+  const Vertex_handle v0 = edge.first->vertex(edge.second);
+  const Vertex_handle v1 = edge.first->vertex(edge.third);
+  if (c3t3.in_dimension(v0) != 2 || c3t3.in_dimension(v1) != 2)
+    return true;//other cases should not be treated here
+
+  typename Tr::Geom_traits gt = c3t3.triangulation().geom_traits();
+  typename Tr::Geom_traits::Construct_opposite_vector_3
+    opp = gt.construct_opposite_vector_3_object();
+  typename Tr::Geom_traits::Compute_scalar_product_3
+    product = gt.compute_scalar_product_3_object();
+  typename Tr::Geom_traits::Construct_normal_3
+    normal = gt.construct_normal_3_object();
+
+  boost::unordered_set<Facet> facets;
+  tr.finite_incident_facets(v0, std::inserter(facets, facets.end()));
+  tr.finite_incident_facets(v1, std::inserter(facets, facets.end()));
+
+// note : checking a 2nd ring of facets does not change the result
+//  boost::unordered_set<Facet> ring2;
+//  for (const Facet& f : facets)
+//  {
+//    for (int i = 1; i < 4; ++i)
+//    {
+//      Vertex_handle vi = f.first->vertex((f.second + i) % 4);
+//      tr.finite_incident_facets(vi, std::inserter(ring2, ring2.end()));
+//    }
+//  }
+//  facets.insert(ring2.begin(), ring2.end());
+
+  Vector_3 reference_normal = CGAL::NULL_VECTOR;
+  //Point_3 reference_c;
+  for (const Facet& f : facets)
+  {
+    if (!is_boundary(c3t3, f, cell_selector))
+      continue;
+    if (facet_has_edge(f, v0, v1))
+      continue; //this facet will collapse if collapse happens
+
+    std::array<Point_3, 3> pts = {{ point(f.first->vertex((f.second + 1) % 4)->point()),
+                                    point(f.first->vertex((f.second + 2) % 4)->point()),
+                                    point(f.first->vertex((f.second + 3) % 4)->point()) }};
+    if(f.second % 2 == 0)
+      std::swap(pts[0], pts[1]);
+
+    Vector_3 n_before_collapse = normal(pts[0], pts[1], pts[2]);
+
+    const Facet& mf = tr.mirror_facet(f);
+    bool do_opp = false;
+    if (  c3t3.triangulation().is_infinite(mf.first)
+      ||  c3t3.subdomain_index(mf.first) < c3t3.subdomain_index(f.first))
+    {
+      n_before_collapse = opp(n_before_collapse);
+      do_opp = true;
+    }
+
+    if (reference_normal == CGAL::NULL_VECTOR)
+    {
+      //reference_c = CGAL::centroid(pts[0], pts[1], pts[2]);
+      reference_normal = n_before_collapse;
+    }
+
+    // check after move
+    for (int i = 0; i < 3; ++i)
+    {
+      const Vertex_handle vi = f.first->vertex((f.second + i + 1) % 4);
+      if (vi == v0 || vi == v1)
+      {
+        if (f.second % 2 == 0)
+        {
+          if(i == 0)      pts[1] = point(new_pos);
+          else if(i == 1) pts[0] = point(new_pos);
+          else            pts[2] = point(new_pos);
+        }
+        else
+          pts[i] = point(new_pos);
+        break;
+      }
+    }
+
+    Vector_3 n_after_collapse = normal(pts[0], pts[1], pts[2]);
+    if(do_opp)
+      n_after_collapse = opp(n_after_collapse);
+
+    const double dotref = product(reference_normal, n_after_collapse);
+    if(dotref < 0)
+      return false;
+    const double dot = product(n_before_collapse, n_after_collapse);
+    if(dot < 0)
+      return false;
+
+//    if (dot * dotref < 0)
+//    {
+//      std::cout << "collapse edge : " << std::endl;
+//      std::cout << point(v0->point()) << " " << point(v1->point()) << std::endl;
+//      std::cout << "facet : " << std::endl;
+//      std::cout << pts[0] << " " << pts[1] << " " << pts[2] << std::endl;
+//
+//      Point_3 c = CGAL::centroid(pts[0], pts[1], pts[2]);
+//      std::cout << "n_before_collapse ";
+//      std::cout << c << " " << (c + n_before_collapse) << std::endl;
+//      std::cout << "n_after_collapse  ";
+//      std::cout << c << " " << (c + n_after_collapse) << std::endl;
+//      std::cout << "reference_normal  ";
+//      std::cout << reference_c << " " << (reference_c + reference_normal) << std::endl;
+//      std::cout << std::endl;
+//    }
+//    if (dotref < 0 || dot < 0)
+//      return false;
+  }
+
+  return true;
+}
+
+template<typename C3t3, typename CellSelector>
+bool are_edge_lengths_valid(const typename C3t3::Edge& edge,
+                            const C3t3& c3t3,
+                            const typename C3t3::Triangulation::Point& new_pos,
+                            const typename C3t3::Triangulation::Geom_traits::FT& sqhigh,
+                            const CellSelector& cell_selector,
+                            const bool /* adaptive */ = false)
+{
+  //SqLengthMap::key_type is Vertex_handle
+  //SqLengthMap::value_type is double
+  typedef typename C3t3::Triangulation::Geom_traits::FT FT;
+  typedef typename C3t3::Edge                           Edge;
+  typedef typename C3t3::Vertex_handle                  Vertex_handle;
+
+  const Vertex_handle v1 = edge.first->vertex(edge.second);
+  const Vertex_handle v2 = edge.first->vertex(edge.third);
+
+  boost::unordered_map<Vertex_handle, FT> edges_sqlength_after_collapse;
+
+  std::vector<Edge> inc_edges;
+  c3t3.triangulation().finite_incident_edges(v1,
+      std::back_inserter(inc_edges));
+  c3t3.triangulation().finite_incident_edges(v2,
+      std::back_inserter(inc_edges));
+
+  for (const Edge& ei : inc_edges)
+  {
+    if (is_outside(ei, c3t3, cell_selector))
+      continue;
+
+    Vertex_handle vh = ei.first->vertex(ei.second);
+    if (vh == v1 || vh == v2)
+      vh = ei.first->vertex(ei.third);
+    if (vh == v1 || vh == v2)
+      continue;
+
+    if (edges_sqlength_after_collapse.find(vh) == edges_sqlength_after_collapse.end())
+    {
+      const FT sqlen = CGAL::squared_distance(new_pos, point(vh->point()));
+
+      //if (adaptive){
+      //  if (is_boundary_edge(ei) || is_hull_edge(ei)){
+      //    if (sqlen_i > split_length)
+      //      return false;
+      //  }
+      //  else if (sqlen_i > 4.*getAimedLength(ei, aimed_length) / 3.){// && is_in_complex(ei)  ){
+      //    return false;
+      //  }
+      //}
+      //else {
+
+      if (sqlen > sqhigh) {
+        return false;
+      }
+      //}
+      edges_sqlength_after_collapse[vh] = sqlen;
+    }
+  }
+
+  return true;
+}
+
+template<typename C3t3>
+void merge_surface_patch_indices(const typename C3t3::Facet& f1,
+                                 const typename C3t3::Facet& f2,
+                                 C3t3& c3t3)
+{
+  const bool in_cx_f1 = c3t3.is_in_complex(f1);
+  const bool in_cx_f2 = c3t3.is_in_complex(f2);
+
+  if (in_cx_f1 && !in_cx_f2)
+  {
+    typename C3t3::Surface_patch_index patch = c3t3.surface_patch_index(f1);
+    f2.first->set_surface_patch_index(f2.second, patch);
+  }
+  else if (in_cx_f2 && !in_cx_f1)
+  {
+    typename C3t3::Surface_patch_index patch = c3t3.surface_patch_index(f2);
+    f1.first->set_surface_patch_index(f1.second, patch);
+  }
+  else if(in_cx_f1 && in_cx_f2)
+  {
+    CGAL_assertion(c3t3.surface_patch_index(f1) == c3t3.surface_patch_index(f2));
+
+    typename C3t3::Surface_patch_index patch = c3t3.surface_patch_index(f2);
+    c3t3.remove_from_complex(f2);
+    f2.first->set_surface_patch_index(f2.second, patch);
+  }
+}
+
+template<typename C3t3>
+typename C3t3::Vertex_handle
+collapse(const typename C3t3::Cell_handle ch,
+         const int to, const int from,
+         C3t3& c3t3)
+{
+  typedef typename C3t3::Triangulation Tr;
+  typedef typename C3t3::Vertex_handle Vertex_handle;
+  typedef typename C3t3::Cell_handle   Cell_handle;
+  typedef typename C3t3::Facet         Facet;
+  typedef typename Tr::Cell_circulator Cell_circulator;
+
+  Tr& tr = c3t3.triangulation();
+
+  Vertex_handle vh0 = ch->vertex(to);
+  Vertex_handle vh1 = ch->vertex(from);
+
+
+  //Update the vertex before removing it
+  std::vector<Cell_handle> find_incident;
+  tr.incident_cells(vh0, std::back_inserter(find_incident));
+
+  std::vector<Cell_handle> cells_to_update;
+  tr.incident_cells(vh1, std::back_inserter(cells_to_update));
+
+  boost::container::small_vector<Cell_handle, 30> inc_cells;
+  Cell_circulator circ = tr.incident_cells(ch, to, from);
+  Cell_circulator done = circ;
+  do
+  {
+    for (int i = 0; i < 4; ++i)
+    {
+      const Vertex_handle vi = circ->vertex(i);
+      if (vi != vh0 && vi != vh1)
+      {
+        const Facet fi(circ, i);
+        if (c3t3.is_in_complex(fi))
+          c3t3.remove_from_complex(fi);
+      }
+    }
+    inc_cells.push_back(circ);
+  }
+  while (++circ != done);
+
+  bool valid = true;
+  std::vector<Cell_handle> cells_to_remove;
+  boost::unordered_set<Cell_handle> invalid_cells;
+
+  for(const Cell_handle c : inc_cells)
+  {
+    const int v0_id = c->index(vh0);
+    const int v1_id = c->index(vh1);
+
+    Cell_handle n0_ch = c->neighbor(v0_id);
+    Cell_handle n1_ch = c->neighbor(v1_id);
+
+    const int ch_id_in_n0 = n0_ch->index(c);
+    const int ch_id_in_n1 = n1_ch->index(c);
+
+    //Merge surface patch indices
+    merge_surface_patch_indices(Facet(n0_ch, ch_id_in_n0),
+                                Facet(n1_ch, ch_id_in_n1),
+                                c3t3);
+
+    //Update neighbors before removing cell
+    n0_ch->set_neighbor(ch_id_in_n0, n1_ch);
+    n1_ch->set_neighbor(ch_id_in_n1, n0_ch);
+
+    //Update vertices cell pointer
+    for (int i = 0; i < 3; i++)
+    {
+      int vid = Tr::vertex_triple_index(ch_id_in_n0, i);
+      n0_ch->vertex(vid)->set_cell(n0_ch);
+    }
+    for (int i = 0; i < 3; i++)
+    {
+      int vid = Tr::vertex_triple_index(ch_id_in_n1, i);
+      n1_ch->vertex(vid)->set_cell(n1_ch);
+    }
+
+    if (tr.is_infinite(n0_ch->vertex(ch_id_in_n0))
+      && tr.is_infinite(n1_ch->vertex(ch_id_in_n1)))
+    {
+      std::cout << "Collapse infinite issue!" << std::endl;
+      return Vertex_handle();
+    }
+    cells_to_remove.push_back(c);
+
+    invalid_cells.insert(c);
+  }
+
+  const Vertex_handle infinite_vertex = tr.infinite_vertex();
+
+  bool v0_updated = false;
+  for (const Cell_handle c : find_incident)
+  {
+    if (invalid_cells.find(c) == invalid_cells.end())//valid cell
+    {
+      if (tr.is_infinite(c))
+        infinite_vertex->set_cell(c);
+      //else {
+      vh0->set_cell(c);
+      v0_updated = true;
+      //}
+    }
+  }
+
+  // update complex edges
+  const std::array<std::array<int, 2>, 6> edges
+    = { { {{0,1}}, {{0,2}}, {{0,3}}, {{1,2}}, {{1,3}}, {{2,3}} } }; //vertex indices in cells
+  const Vertex_handle vkept = vh0;
+  const Vertex_handle vdeleted = vh1;
+  for (const Cell_handle c : cells_to_update)
+  {
+    for (const std::array<int, 2>& ei : edges)
+    {
+      Vertex_handle eiv0 = c->vertex(ei[0]);
+      Vertex_handle eiv1 = c->vertex(ei[1]);
+      if (eiv1 == vdeleted && eiv0 != vkept) //replace eiv1 by vkept
+      {
+        if (c3t3.is_in_complex(eiv0, eiv1))
+        {
+          if (!c3t3.is_in_complex(eiv0, vkept))
+            c3t3.add_to_complex(eiv0, vkept, c3t3.curve_index(eiv0, eiv1));
+          c3t3.remove_from_complex(eiv0, eiv1);
+        }
+      }
+      else if (eiv0 == vdeleted && eiv1 != vkept) //replace eiv0 by vkept
+      {
+        if (c3t3.is_in_complex(eiv0, eiv1))
+        {
+          if (!c3t3.is_in_complex(vkept, eiv1))
+            c3t3.add_to_complex(vkept, eiv1, c3t3.curve_index(eiv0, eiv1));
+          c3t3.remove_from_complex(eiv0, eiv1);
+        }
+      }
+    }
+  }
+
+  // update complex facets
+
+  //Update the vertex before removing it
+  for (const Cell_handle c : cells_to_update)
+  {
+    if (invalid_cells.find(c) == invalid_cells.end()) //valid cell
+    {
+      c->set_vertex(c->index(vh1), vh0);
+
+      if (tr.is_infinite(c))
+        infinite_vertex->set_cell(c);
+      //else {
+      if (!v0_updated) {
+        vh0->set_cell(c);
+        v0_updated = true;
+      }
+      //}
+    }
+  }
+
+  if (!v0_updated)
+    std::cout << "PB i cell not valid!!!" << std::endl;
+
+  // Delete vertex
+  c3t3.triangulation().tds().delete_vertex(vh1);
+
+  // Delete cells
+  for (Cell_handle cell_to_remove : cells_to_remove)
+  {
+    // remove cell
+    if (c3t3.is_in_complex(cell_to_remove))
+      c3t3.remove_from_complex(cell_to_remove);
+    c3t3.triangulation().tds().delete_cell(cell_to_remove);
+  }
+
+  if (!valid){
+    std::cout << "Global triangulation collapse bug!!" << std::endl;
+    return Vertex_handle();
+  }
+
+  return vh0;
+}
+
+
+template<typename C3t3>
+typename C3t3::Vertex_handle collapse(typename C3t3::Edge& edge,
+                                      const Collapse_type& collapse_type,
+                                      C3t3& c3t3)
+{
+  typedef typename C3t3::Vertex_handle Vertex_handle;
+  typedef typename C3t3::Triangulation::Point Point_3;
+
+  Vertex_handle vh0 = edge.first->vertex(edge.second);
+  Vertex_handle vh1 = edge.first->vertex(edge.third);
+
+  const int dim_vh0 = c3t3.in_dimension(vh0);
+  const int dim_vh1 = c3t3.in_dimension(vh1);
+
+  Vertex_handle vh = Vertex_handle();
+
+  const Point_3 p0 = vh0->point();
+  const Point_3 p1 = vh1->point();
+
+  //Collapse at mid point
+  if (collapse_type == TO_MIDPOINT)
+  {
+    Point_3 new_position(CGAL::midpoint(point(vh0->point()), point(vh1->point())));
+    vh0->set_point(new_position);
+    vh1->set_point(new_position);
+
+    vh = collapse(edge.first, edge.second, edge.third, c3t3);
+    c3t3.set_dimension(vh, (std::min)(dim_vh0, dim_vh1));
+  }
+  else //Collapse at vertex
+  {
+    if (collapse_type == TO_V1)
+    {
+      vh0->set_point(p1);
+      vh = collapse(edge.first, edge.third, edge.second, c3t3);
+      c3t3.set_dimension(vh, (std::min)(dim_vh0, dim_vh1));
+    }
+    else //Collapse at v0
+    {
+      if (collapse_type == TO_V0)
+      {
+        vh1->set_point(p0);
+        vh = collapse(edge.first, edge.second, edge.third, c3t3);
+        c3t3.set_dimension(vh, (std::min)(dim_vh0, dim_vh1));
+      }
+      else
+        CGAL_assertion(false);
+    }
+  }
+  return vh;
+}
+
+template<typename C3t3>
+bool is_cells_set_manifold(const C3t3&,
+    std::unordered_set<typename C3t3::Cell_handle>& cells)
+{
+  typedef typename C3t3::Cell_handle Cell_handle;
+  typedef typename C3t3::Vertex_handle Vh;
+  typedef std::array<Vh, 3> FV;
+  typedef std::pair<Vh, Vh> EV;
+
+  boost::unordered_map<FV, int> facets;
+  for (Cell_handle c : cells)
+  {
+    for (int i = 0; i < 4; ++i)
+    {
+      const FV fvi = make_vertex_array(c->vertex((i + 1) % 4),
+        c->vertex((i + 2) % 4),
+        c->vertex((i + 3) % 4));
+      typename boost::unordered_map<FV, int>::iterator fit = facets.find(fvi);
+      if (fit == facets.end())
+        facets.insert(std::make_pair(fvi, 1));
+      else
+        fit->second++;
+    }
+  }
+
+  boost::unordered_map<EV, int> edges;
+  for (const std::pair<FV, int>& fvv : facets)
+  {
+    if (fvv.second != 1)
+      continue;
+
+    for (int i = 0; i < 3; ++i)
+    {
+      const EV evi = make_vertex_pair(fvv.first[i], fvv.first[(i + 1) % 3]);
+      typename boost::unordered_map<EV, int>::iterator eit = edges.find(evi);
+      if (eit == edges.end())
+        edges.insert(std::make_pair(evi, 1));
+      else
+        eit->second++;
+    }
+  }
+
+  for (const std::pair<EV, int>& evv : edges)
+    if (evv.second != 2)
+      return false;
+
+  return true;
+}
+
+template<typename C3t3, typename CellSelector, typename Visitor>
+typename C3t3::Vertex_handle collapse_edge(typename C3t3::Edge& edge,
+    C3t3& c3t3,
+    const typename C3t3::Triangulation::Geom_traits::FT& sqhigh,
+    const bool /* protect_boundaries */,
+    CellSelector cell_selector,
+    Visitor& )
+{
+  typedef typename C3t3::Triangulation   Tr;
+  typedef typename Tr::Point             Point;
+  typedef typename Tr::Vertex_handle     Vertex_handle;
+  typedef typename Tr::Cell_handle       Cell_handle;
+
+  const Vertex_handle v0 = edge.first->vertex(edge.second);
+  const Vertex_handle v1 = edge.first->vertex(edge.third);
+
+  Collapse_type collapse_type = get_collapse_type(edge, c3t3, cell_selector);
+
+#ifdef CGAL_DEBUG_TET_REMESHING_IN_PLUGIN
+  const bool in_cx = c3t3.is_in_complex(edge);
+  if (in_cx && collapse_type == IMPOSSIBLE)
+    nb_impossible++;
+#endif
+
+  if (collapse_type == IMPOSSIBLE)
+    return Vertex_handle();
+
+  Point new_pos;
+  switch(collapse_type)
+  {
+  case TO_V0:
+    new_pos = v0->point(); break;
+  case TO_V1:
+    new_pos = v1->point(); break;
+  default:
+    CGAL_assertion(collapse_type == TO_MIDPOINT);
+    new_pos = Point(CGAL::midpoint(point(v0->point()), point(v1->point())));
+  }
+
+  if (!is_valid_collapse(edge, collapse_type, new_pos, c3t3))
+  {
+#ifdef TET_REMESHING_COLLAPSE_FALLBACK_EXPERIMENTS
+    if (collapse_type == TO_MIDPOINT)
+    {
+      // with TO_MIDPOINT, we are authorized to test TO_V0 and TO_V1
+      if (is_valid_collapse(edge, TO_V0, v0->point(), c3t3))
+      {
+        collapse_type = TO_V0;
+        new_pos = v0->point();
+      }
+      else if (is_valid_collapse(edge, TO_V1, v1->point(), c3t3))
+      {
+        collapse_type = TO_V1;
+        new_pos = v1->point();
+      }
+      else
+      {
+#ifdef CGAL_DEBUG_TET_REMESHING_IN_PLUGIN
+        if (in_cx)
+          nb_invalid_collapse++;
+#endif
+        return Vertex_handle();
+      }
+    }
+    else
+#endif //TET_REMESHING_COLLAPSE_FALLBACK_EXPERIMENTS
+    {
+#ifdef CGAL_DEBUG_TET_REMESHING_IN_PLUGIN
+      if (in_cx)
+        nb_invalid_collapse++;
+#endif
+      return Vertex_handle();
+    }
+  }
+
+  if (are_edge_lengths_valid(edge, c3t3, new_pos, sqhigh, cell_selector/*, adaptive = false*/)
+    && collapse_preserves_surface_star(edge, c3t3, new_pos, cell_selector))
+  {
+    CGAL_assertion_code(typename Tr::Cell_handle dc);
+    CGAL_assertion_code(int di);
+    CGAL_assertion_code(int dj);
+    CGAL_assertion(c3t3.triangulation().is_edge(edge.first->vertex(edge.second),
+                                                edge.first->vertex(edge.third), dc, di, dj));
+
+    Vertex_handle v0_init = edge.first->vertex(edge.second);
+    Vertex_handle v1_init = edge.first->vertex(edge.third);
+
+    std::unordered_set<Cell_handle> cells_to_insert;
+    c3t3.triangulation().finite_incident_cells(v0_init,
+      std::inserter(cells_to_insert, cells_to_insert.end()));
+    c3t3.triangulation().finite_incident_cells(v1_init,
+      std::inserter(cells_to_insert, cells_to_insert.end()));
+
+    if(!is_cells_set_manifold(c3t3, cells_to_insert))
+      return Vertex_handle();
+
+    CollapseTriangulation<C3t3> local_tri(edge, cells_to_insert, collapse_type);
+
+    Result_type res = local_tri.collapse();
+    if (res == VALID)
+    {
+#ifdef CGAL_DEBUG_TET_REMESHING_IN_PLUGIN
+      if (in_cx)
+        nb_valid_collapse++;
+#endif
+      return collapse(edge, collapse_type, c3t3);
+    }
+  }
+#ifdef CGAL_DEBUG_TET_REMESHING_IN_PLUGIN
+  else if (in_cx)
+    nb_invalid_lengths++;
+#endif
+  return Vertex_handle();
+}
+
+template<typename C3T3, typename CellSelector>
+bool can_be_collapsed(const typename C3T3::Edge& e,
+                      const C3T3& c3t3,
+                      const bool protect_boundaries,
+                      CellSelector cell_selector)
+{
+  if (is_outside(e, c3t3, cell_selector))
+    return false;
+
+  if (protect_boundaries)
+  {
+    if (c3t3.is_in_complex(e))
+      return false;
+    else if (is_boundary(c3t3, e, cell_selector))
+      return false;
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+    if (!is_internal(e, c3t3, cell_selector))
+    {
+      std::cerr << "e is not inside!?" << std::endl;
+      typename C3T3::Vertex_handle v1 = e.first->vertex(e.second);
+      typename C3T3::Vertex_handle v2 = e.first->vertex(e.third);
+      std::cerr << v1->point() << " " << v2->point() << std::endl;
+    }
+#endif
+
+    CGAL_assertion(is_internal(e, c3t3, cell_selector));
+    return true;
+  }
+  else
+  {
+    return true;
+  }
+}
+
+template<typename C3T3, typename CellSelector, typename Visitor>
+void collapse_short_edges(C3T3& c3t3,
+                          const typename C3T3::Triangulation::Geom_traits::FT& low,
+                          const typename C3T3::Triangulation::Geom_traits::FT& high,
+                          const bool protect_boundaries,
+                          CellSelector cell_selector,
+                          Visitor& visitor)
+{
+  typedef typename C3T3::Triangulation       T3;
+  typedef typename T3::Cell_handle           Cell_handle;
+  typedef typename T3::Edge                  Edge;
+  typedef typename T3::Vertex_handle         Vertex_handle;
+  typedef typename std::pair<Vertex_handle, Vertex_handle> Edge_vv;
+
+  typedef typename T3::Geom_traits     Gt;
+  typedef typename T3::Geom_traits::FT FT;
+  typedef boost::bimap<
+  boost::bimaps::set_of<Edge_vv>,
+        boost::bimaps::multiset_of<FT, std::less<FT> > >  Boost_bimap;
+  typedef typename Boost_bimap::value_type            short_edge;
+
+  T3& tr = c3t3.triangulation();
+  typename Gt::Compute_squared_length_3 sql
+    = tr.geom_traits().compute_squared_length_3_object();
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_VERBOSE
+  std::cout << "Collapse short edges (" << low << ", " << high << ")...";
+  std::cout.flush();
+  std::size_t nb_collapses = 0;
+#endif
+  const FT sq_low = low*low;
+  const FT sq_high = high*high;
+
+  //collect long edges
+  Boost_bimap short_edges;
+  for (const Edge& e : tr.finite_edges())
+  {
+    if (!can_be_collapsed(e, c3t3, protect_boundaries, cell_selector))
+      continue;
+
+    FT sqlen = sql(tr.segment(e));
+    if (sqlen < sq_low)
+      short_edges.insert(short_edge(make_vertex_pair<T3>(e), sqlen));
+  }
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+  debug::dump_edges(short_edges, "short_edges.polylines.txt");
+
+  std::ofstream short_success("short_collapse_success.polylines.txt");
+  std::ofstream short_fail("short_collapse_fail.polylines.txt");
+  std::ofstream short_cancel("short_collapse_canceled.polylines.txt");
+#endif
+
+  while(!short_edges.empty())
+  {
+    //the edge with shortest length
+    typename Boost_bimap::right_map::iterator eit = short_edges.right.begin();
+    Edge_vv e = eit->second;
+    short_edges.right.erase(eit);
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_VERBOSE_PROGRESS
+    FT sqlen = eit->first;
+    std::cout << "\rCollapse... (" << short_edges.left.size() << " short edges, ";
+    std::cout << std::sqrt(sqlen) << ", ";
+    std::cout << nb_collapses << " collapses)";
+    std::cout.flush();
+#endif
+    Cell_handle cell;
+    int i1, i2;
+    if ( tr.tds().is_vertex(e.first)
+         && tr.tds().is_vertex(e.second)
+         && tr.tds().is_edge(e.first, e.second, cell, i1, i2)
+         && tr.segment(Edge(cell, i1, i2)).squared_length() < sq_low)
+    {
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+      const typename T3::Point p1 = e.first->point();
+      const typename T3::Point p2 = e.second->point();
+#endif
+
+      Edge edge(cell, i1, i2);
+
+      if (!can_be_collapsed(edge, c3t3, protect_boundaries, cell_selector))
+      {
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+        short_cancel << "2 " << point(p1) << " " << point(p2) << std::endl;
+#endif
+        continue;
+      }
+
+      Vertex_handle vh = collapse_edge(edge, c3t3, sq_high,
+                                       protect_boundaries, cell_selector,
+                                       visitor);
+      if (vh != Vertex_handle())
+      {
+        std::vector<Edge> incident_short;
+        c3t3.triangulation().finite_incident_edges(vh,
+            std::back_inserter(incident_short));
+        for (const Edge& eshort : incident_short)
+        {
+          if (!can_be_collapsed(eshort, c3t3, protect_boundaries, cell_selector))
+            continue;
+
+          const FT sqlen = sql(tr.segment(eshort));
+          if (sqlen < sq_low)
+            short_edges.insert(short_edge(make_vertex_pair<T3>(eshort), sqlen));
+        }
+
+        //debug::dump_c3t3(c3t3, "dump_after_collapse");
+        //CGAL_assertion(c3t3.triangulation().tds().is_valid());
+#ifdef CGAL_TETRAHEDRAL_REMESHING_VERBOSE
+        ++nb_collapses;
+#endif
+      }
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+      if (vh != Vertex_handle())
+        short_success << "2 " << point(p1) << " " << point(p2) << std::endl;
+      else
+        short_fail << "2 " << point(p1) << " " << point(p2) << std::endl;
+#endif
+    }
+  }//end loop on short_edges
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+  short_success.close();
+  short_fail.close();
+#endif
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_VERBOSE
+  std::cout << " done (" << nb_collapses << " collapses)." << std::endl;
+#endif
+}
+}
+}
+}
+
+#endif // CGAL_INTERNAL_COLLAPSE_SHORT_EDGES_H
diff -pruN 5.0.3-1/include/CGAL/Tetrahedral_remeshing/internal/compute_c3t3_statistics.h 5.1-1/include/CGAL/Tetrahedral_remeshing/internal/compute_c3t3_statistics.h
--- 5.0.3-1/include/CGAL/Tetrahedral_remeshing/internal/compute_c3t3_statistics.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Tetrahedral_remeshing/internal/compute_c3t3_statistics.h	2020-09-08 12:15:49.000000000 +0000
@@ -0,0 +1,214 @@
+// Copyright (c) 2020 GeometryFactory (France) and Telecom Paris (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Tetrahedral_remeshing/include/CGAL/Tetrahedral_remeshing/internal/compute_c3t3_statistics.h $
+// $Id: compute_c3t3_statistics.h ab05dde 2020-06-12T08:08:56+02:00 Jane Tournois
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Jane Tournois, Noura Faraj, Jean-Marc Thiery, Tamy Boubekeur
+
+#ifndef CGAL_TR_INTERNAL_COMPUTE_C3T3_STATISTICS_H
+#define CGAL_TR_INTERNAL_COMPUTE_C3T3_STATISTICS_H
+
+#include <CGAL/license/Tetrahedral_remeshing.h>
+
+#include <limits>
+#include <vector>
+#include <algorithm>
+#include <fstream>
+
+#include <boost/unordered_set.hpp>
+
+#include <CGAL/Tetrahedral_remeshing/internal/tetrahedral_remeshing_helpers.h>
+
+namespace CGAL
+{
+namespace Tetrahedral_remeshing
+{
+namespace internal
+{
+template<typename Triangulation, typename CellSelector>
+void compute_statistics(const Triangulation& tr,
+                        CellSelector cell_selector,
+                        const char* filename = "statistics_c3t3.txt")
+{
+  typedef Triangulation Tr;
+  typedef typename Tr::Geom_traits Gt;
+  typedef typename Tr::Cell_handle   Cell_handle;
+  typedef typename Tr::Vertex_handle Vertex_handle;
+  typedef typename Gt::Point_3       Point;
+  typedef typename Tr::Finite_facets_iterator Finite_facets_iterator;
+  typedef typename Tr::Finite_cells_iterator  Finite_cells_iterator;
+  typedef typename Tr::Cell::Subdomain_index  Subdomain_index;
+
+  std::size_t nb_edges = 0;
+  double total_edges = 0;
+  std::size_t nb_angle = 0;
+  double total_angle = 0;
+
+  double min_edges_length = (std::numeric_limits<double>::max)();
+  double max_edges_length = 0.;
+
+  double smallest_edge_radius = (std::numeric_limits<double>::max)();
+  double smallest_radius_radius = (std::numeric_limits<double>::max)();
+  double biggest_v_sma_cube = 0.;
+  double max_dihedral_angle = 0.;
+  double min_dihedral_angle = 180.;
+
+  for (Finite_facets_iterator fit = tr.finite_facets_begin();
+       fit != tr.finite_facets_end(); ++fit)
+  {
+    const Cell_handle cell = fit->first;
+    const int& index = fit->second;
+    if (!cell_selector(cell) || !cell_selector(cell->neighbor(index)))
+      continue;
+
+    const Point& pa = point(cell->vertex((index + 1) & 3)->point());
+    const Point& pb = point(cell->vertex((index + 2) & 3)->point());
+    const Point& pc = point(cell->vertex((index + 3) & 3)->point());
+
+    double edges[3];
+    edges[0] = (CGAL::sqrt(CGAL::squared_distance(pa, pb)));
+    edges[1] = (CGAL::sqrt(CGAL::squared_distance(pa, pc)));
+    edges[2] = (CGAL::sqrt(CGAL::squared_distance(pb, pc)));
+    for (int i = 0; i < 3; ++i)
+    {
+      if (edges[i] < min_edges_length){ min_edges_length = edges[i]; }
+      if (edges[i] > max_edges_length){ max_edges_length = edges[i]; }
+      total_edges += edges[i];
+      ++nb_edges;
+    }
+  }
+
+  double mean_edges_length = total_edges / (double)nb_edges;
+
+  typename Gt::Compute_approximate_dihedral_angle_3 approx_dihedral_angle
+    = tr.geom_traits().compute_approximate_dihedral_angle_3_object();
+
+  std::size_t nb_tets = 0;
+  boost::unordered_set<Vertex_handle> selected_vertices;
+  std::vector<Subdomain_index> sub_ids;
+  for (Finite_cells_iterator cit = tr.finite_cells_begin();
+       cit != tr.finite_cells_end();
+       ++cit)
+  {
+    const Subdomain_index& si = cit->subdomain_index();
+    if (si == Subdomain_index() || !cell_selector(cit))
+      continue;
+
+    ++nb_tets;
+    if (std::find(sub_ids.begin(), sub_ids.end(), si) == sub_ids.end())
+      sub_ids.push_back(cit->subdomain_index());
+    for (int i = 0; i < 4; ++i)
+      selected_vertices.insert(cit->vertex(i));
+
+    const Point& p0 = point(cit->vertex(0)->point());
+    const Point& p1 = point(cit->vertex(1)->point());
+    const Point& p2 = point(cit->vertex(2)->point());
+    const Point& p3 = point(cit->vertex(3)->point());
+    double v = CGAL::abs(tr.tetrahedron(cit).volume());
+    if (v == 0.)
+    {
+      std::cout << "degenerate cell :\n\t";
+      std::cout << p0 << "\n\t" << p1 << "\n\t" << p2 << "\n\t" << p3 << std::endl;
+    }
+    double circumradius = (v == 0.)
+                          ? CGAL::sqrt(CGAL::squared_radius(p0, p1, p2))
+                          : CGAL::sqrt(CGAL::squared_radius(p0, p1, p2, p3));
+
+    //find shortest edge
+    double edges[6];
+    edges[0] = CGAL::sqrt(CGAL::squared_distance(p0, p1));
+    edges[1] = CGAL::sqrt(CGAL::squared_distance(p0, p2));
+    edges[2] = CGAL::sqrt(CGAL::squared_distance(p0, p3));
+    edges[3] = CGAL::sqrt(CGAL::squared_distance(p2, p1));
+    edges[4] = CGAL::sqrt(CGAL::squared_distance(p2, p3));
+    edges[5] = CGAL::sqrt(CGAL::squared_distance(p1, p3));
+
+    double min_edge = edges[0];
+    for (int i = 1; i < 6; ++i)
+    {
+      if (edges[i] < min_edge)
+        min_edge = edges[i];
+    }
+
+    double sumar = CGAL::sqrt(CGAL::squared_area(p0, p1, p2))
+                   + CGAL::sqrt(CGAL::squared_area(p1, p2, p3))
+                   + CGAL::sqrt(CGAL::squared_area(p2, p3, p0))
+                   + CGAL::sqrt(CGAL::squared_area(p3, p1, p0));
+    double inradius = 3. * v / sumar;
+    double smallest_edge_radius_ = min_edge / circumradius*CGAL::sqrt(6.) / 4.;//*sqrt(6)/4 so that the perfect tet ratio is 1
+    double smallest_radius_radius_ = inradius / circumradius * 3.; //*3 so that the perfect tet ratio is 1 instead of 1/3
+    double biggest_v_sma_cube_ = v / std::pow(min_edge, 3) * 6. * CGAL::sqrt(2.);//*6*sqrt(2) so that the perfect tet ratio is 1 instead
+
+    if (smallest_edge_radius_ < smallest_edge_radius)
+      smallest_edge_radius = smallest_edge_radius_;
+
+    if (smallest_radius_radius_ < smallest_radius_radius)
+      smallest_radius_radius = smallest_radius_radius_;
+
+    if (biggest_v_sma_cube_ > biggest_v_sma_cube)
+      biggest_v_sma_cube = biggest_v_sma_cube_;
+
+    double a = CGAL::to_double(CGAL::abs(approx_dihedral_angle(p0, p1, p2, p3)));
+    if (a < min_dihedral_angle) { min_dihedral_angle = a; }
+    if (a > max_dihedral_angle) { max_dihedral_angle = a; }
+    total_angle += a;
+    ++nb_angle;
+    a = CGAL::to_double(CGAL::abs(approx_dihedral_angle(p0, p2, p1, p3)));
+    if (a < min_dihedral_angle) { min_dihedral_angle = a; }
+    if (a > max_dihedral_angle) { max_dihedral_angle = a; }
+    total_angle += a;
+    ++nb_angle;
+    a = CGAL::to_double(CGAL::abs(approx_dihedral_angle(p0, p3, p1, p2)));
+    if (a < min_dihedral_angle) { min_dihedral_angle = a; }
+    if (a > max_dihedral_angle) { max_dihedral_angle = a; }
+    total_angle += a;
+    ++nb_angle;
+    a = CGAL::to_double(CGAL::abs(approx_dihedral_angle(p1, p2, p0, p3)));
+    if (a < min_dihedral_angle) { min_dihedral_angle = a; }
+    if (a > max_dihedral_angle) { max_dihedral_angle = a; }
+    total_angle += a;
+    ++nb_angle;
+    a = CGAL::to_double(CGAL::abs(approx_dihedral_angle(p1, p3, p0, p2)));
+    if (a < min_dihedral_angle) { min_dihedral_angle = a; }
+    if (a > max_dihedral_angle) { max_dihedral_angle = a; }
+    total_angle += a;
+    ++nb_angle;
+    a = CGAL::to_double(CGAL::abs(approx_dihedral_angle(p2, p3, p0, p1)));
+    if (a < min_dihedral_angle) { min_dihedral_angle = a; }
+    if (a > max_dihedral_angle) { max_dihedral_angle = a; }
+    total_angle += a;
+    ++nb_angle;
+  }
+
+  std::size_t nb_subdomains = sub_ids.size();
+  //std::size_t nb_vertices = d->c3t3.number_of_vertices_in_complex();
+
+  std::ofstream ofs(filename);
+  if (!ofs)
+    return;
+
+  ofs << "Nb subdomains               : " << nb_subdomains << std::endl;
+  ofs << "Total number of vertices    : " << tr.number_of_vertices() << std::endl;
+  ofs << "Number of selected cells    : " << nb_tets << std::endl;
+  ofs << "Number of selected vertices : " << selected_vertices.size() << std::endl;
+  ofs << std::endl;
+  ofs << "Min dihedral angle : " << min_dihedral_angle << std::endl;
+  ofs << "Max dihedral angle : " << max_dihedral_angle << std::endl;
+  ofs << std::endl;
+  ofs << "Shortest edge       : " << min_edges_length << std::endl;
+  ofs << "Longest edge        : " << max_edges_length << std::endl;
+  ofs << "Average edge length : " << mean_edges_length << std::endl;
+
+  ofs.close();
+}
+
+}//end namespace internal
+}//end namespace Tetrahedral_remeshing
+}//end namespace CGAL
+
+#endif // CGAL_TR_INTERNAL_COMPUTE_C3T3_STATISTICS_H
diff -pruN 5.0.3-1/include/CGAL/Tetrahedral_remeshing/internal/flip_edges.h 5.1-1/include/CGAL/Tetrahedral_remeshing/internal/flip_edges.h
--- 5.0.3-1/include/CGAL/Tetrahedral_remeshing/internal/flip_edges.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Tetrahedral_remeshing/internal/flip_edges.h	2020-09-08 12:15:49.000000000 +0000
@@ -0,0 +1,1198 @@
+// Copyright (c) 2020 GeometryFactory (France) and Telecom Paris (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Tetrahedral_remeshing/include/CGAL/Tetrahedral_remeshing/internal/flip_edges.h $
+// $Id: flip_edges.h ab05dde 2020-06-12T08:08:56+02:00 Jane Tournois
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Jane Tournois, Noura Faraj, Jean-Marc Thiery, Tamy Boubekeur
+
+#ifndef CGAL_INTERNAL_FLIP_EDGES_H
+#define CGAL_INTERNAL_FLIP_EDGES_H
+
+#include <CGAL/license/Tetrahedral_remeshing.h>
+
+#include <CGAL/Triangulation_utils_3.h>
+#include <CGAL/utility.h>
+
+#include <CGAL/Tetrahedral_remeshing/internal/tetrahedral_remeshing_helpers.h>
+
+#include <boost/unordered_map.hpp>
+#include <boost/unordered_set.hpp>
+#include <boost/container/small_vector.hpp>
+
+#include <limits>
+#include <queue>
+
+namespace CGAL
+{
+namespace Tetrahedral_remeshing
+{
+namespace internal
+{
+enum Flip_Criterion{ MIN_ANGLE_BASED, AVERAGE_ANGLE_BASED,
+                     VALENCE_BASED, VALENCE_MIN_DH_BASED };
+
+//template<typename VertexPair>
+//void flip_inside_edges(std::vector<VertexPair>&)
+//{
+//  //TODO
+//}
+
+//outer_mirror_facets contains the set of facets of the outer hull
+//of the set of cells modified by the flip operation,
+//"seen from" outside
+//i.e. for each facet f among those, f.first has not been modified by flip
+template<typename C3t3, typename CellSet, typename FacetSet>
+void update_c3t3_facets(C3t3& c3t3,
+                        const CellSet& cells_to_update,
+                        const FacetSet& outer_mirror_facets)
+{
+  typedef typename C3t3::Facet       Facet;
+  typedef typename C3t3::Cell_handle Cell_handle;
+  typedef typename C3t3::Surface_patch_index Surface_patch_index;
+
+  for (Cell_handle c : cells_to_update)
+  {
+    //their subdomain indices have not been modified because we kept the same cells
+    //surface patch indices need to be fixed though
+    for (int i = 0; i < 4; ++i)
+    {
+      const Facet f(c, i);
+      const Facet mf = c3t3.triangulation().mirror_facet(f);
+      if (outer_mirror_facets.find(mf) != outer_mirror_facets.end())
+      {
+        //we are on the border of the modified zone, c3t3 info is valid outside,
+        //on mirror facet
+        const typename C3t3::Surface_patch_index patch = c3t3.surface_patch_index(mf);
+        if (c3t3.is_in_complex(mf))
+          f.first->set_surface_patch_index(f.second, patch);
+        else
+          f.first->set_surface_patch_index(f.second, Surface_patch_index());
+      }
+      else
+      {
+        //we are inside the modified zone, c3t3 info is not valid anymore
+        if (c3t3.is_in_complex(f) || c3t3.is_in_complex(mf))
+        {
+          f.first->set_surface_patch_index(f.second, Surface_patch_index());
+          mf.first->set_surface_patch_index(mf.second, Surface_patch_index());
+        }
+      }
+    }
+  }
+}
+
+template<typename C3t3>
+Sliver_removal_result flip_3_to_2(typename C3t3::Edge& edge,
+                                  C3t3& c3t3,
+                                  const std::vector<typename C3t3::Vertex_handle>& vertices_around_edge,
+                                  const Flip_Criterion& criterion)
+{
+  typedef typename C3t3::Triangulation Tr;
+  typedef typename C3t3::Facet         Facet;
+  typedef typename C3t3::Vertex_handle Vertex_handle;
+  typedef typename C3t3::Cell_handle   Cell_handle;
+  typedef typename Tr::Cell_circulator Cell_circulator;
+  typedef typename Tr::Geom_traits     Gt;
+  typedef typename Gt::FT              FT;
+
+  //Edge to face flip
+  Tr& tr = c3t3.triangulation();
+
+  Cell_circulator circ = tr.incident_cells(edge);
+  Cell_circulator done = circ;
+
+  Vertex_handle vh0 = edge.first->vertex(edge.second);
+  Vertex_handle vh1 = edge.first->vertex(edge.third);
+
+  //Select 2 cells to keep and update and one to remove
+  Cell_handle ch0 = Cell_handle(circ++);
+  Cell_handle ch1 = Cell_handle(circ++);
+  Cell_handle cell_to_remove = Cell_handle(circ++);
+  if (circ != done)
+  {
+    std::cout << "Wrong flip function" << std::endl;
+    return NOT_FLIPPABLE;
+  }
+
+  //Check structural validity
+  Cell_handle c;
+  int i0, i1, i3;
+  if (tr.is_facet(vertices_around_edge[0], vertices_around_edge[1], vertices_around_edge[2],
+                  c, i0, i1, i3))
+    return NOT_FLIPPABLE;
+
+  //Check topological validity
+  const typename C3t3::Subdomain_index subdomain = ch0->subdomain_index();
+  if ( subdomain != ch1->subdomain_index()
+       || subdomain != cell_to_remove->subdomain_index()
+       || ch1->subdomain_index() != cell_to_remove->subdomain_index())
+    return NOT_FLIPPABLE;
+
+  Vertex_handle vh2;
+  Vertex_handle vh3;
+
+  for (int i = 0; i < 3; ++i){
+    if (!ch0->has_vertex(vertices_around_edge[i]))
+      vh2 = vertices_around_edge[i];
+    else if (!ch1->has_vertex(vertices_around_edge[i]))
+      vh3 = vertices_around_edge[i];
+  }
+
+  int vh0_id = ch0->index(vh0);
+  int vh1_id = ch1->index(vh1);
+
+  //Check if flip valid
+  if (!is_well_oriented(tr, vh2,
+                        ch0->vertex(indices(vh0_id, 0)),
+                        ch0->vertex(indices(vh0_id, 1)),
+                        ch0->vertex(indices(vh0_id, 2)))
+      || !is_well_oriented(tr, vh3,
+                           ch1->vertex(indices(vh1_id, 0)),
+                           ch1->vertex(indices(vh1_id, 1)),
+                           ch1->vertex(indices(vh1_id, 2))))
+    return NOT_FLIPPABLE;
+
+  ///********************VALIDITY CHECK***************************/
+  //double curr_min_dh;
+  //bool check_validity = false;
+  //std::vector<typename Tr::Tetrahedron> pre_sliver_Removal_cells;
+  //if (check_validity){
+  //  pre_sliver_Removal_cells.clear();
+  //  pre_sliver_Removal_cells.push_back(K::Tetrahedron_3(ch0->vertex(0)->point(), ch0->vertex(1)->point(), ch0->vertex(2)->point(), ch0->vertex(3)->point()));
+  //  pre_sliver_Removal_cells.push_back(K::Tetrahedron_3(ch1->vertex(0)->point(), ch1->vertex(1)->point(), ch1->vertex(2)->point(), ch1->vertex(3)->point()));
+  //  pre_sliver_Removal_cells.push_back(K::Tetrahedron_3(cell_to_remove->vertex(0)->point(), cell_to_remove->vertex(1)->point(),
+  //    cell_to_remove->vertex(2)->point(), cell_to_remove->vertex(3)->point()));
+
+  //  curr_min_dh = min_dihedral_angle<Gt>(ch0);
+  //  curr_min_dh = std::min(curr_min_dh, min_dihedral_angle<Gt>(ch1));
+  //  curr_min_dh = std::min(curr_min_dh, min_dihedral_angle<Gt>(cell_to_remove));
+
+  //  pre_sliver_Removal_vertices.clear();
+  //  for (int i = 0; i < vertices_around_edge.size(); ++i){
+  //    pre_sliver_Removal_vertices.push_back(Point_3(vertices_around_edge[i]->point()));
+  //  }
+
+  //  previous_edges.clear();
+  //  previous_edges.push_back(std::make_pair(vh0->point(), vh1->point()));
+  //}
+  /*************************************************************/
+
+
+  if (criterion == MIN_ANGLE_BASED)
+  {
+    //Current worst dihedral angle
+    FT curr_min_dh = min_dihedral_angle(tr, ch0);
+    curr_min_dh = (std::min)(curr_min_dh, min_dihedral_angle(tr, ch1));
+    curr_min_dh = (std::min)(curr_min_dh, min_dihedral_angle(tr, cell_to_remove));
+
+    //Result worst dihedral angle
+    if (curr_min_dh > min_dihedral_angle(tr, vh2,
+                                         ch0->vertex(indices(vh0_id, 0)),
+                                         ch0->vertex(indices(vh0_id, 1)),
+                                         ch0->vertex(indices(vh0_id, 2)))
+        || curr_min_dh > min_dihedral_angle(tr, vh3,
+                                            ch1->vertex(indices(vh1_id, 0)),
+                                            ch1->vertex(indices(vh1_id, 1)),
+                                            ch1->vertex(indices(vh1_id, 2))))
+      return NO_BEST_CONFIGURATION;
+  }
+  else if (criterion == AVERAGE_ANGLE_BASED)
+  {
+    //Current worst dihedral angle
+    double average_min_dh = min_dihedral_angle(tr, ch0);
+    average_min_dh += min_dihedral_angle(tr, ch1);
+    average_min_dh += min_dihedral_angle(tr, cell_to_remove);
+
+    average_min_dh /= 3.;
+
+    FT new_average_min_dh = 0.5 *
+                            (min_dihedral_angle(tr, vh2, ch0->vertex(indices(vh0_id, 0)),
+                                ch0->vertex(indices(vh0_id, 1)),
+                                ch0->vertex(indices(vh0_id, 2)))
+                             + min_dihedral_angle(tr, vh3, ch1->vertex(indices(vh1_id, 0)),
+                                 ch1->vertex(indices(vh1_id, 1)),
+                                 ch1->vertex(indices(vh1_id, 2))));
+    //Result worst dihedral angle
+    if (average_min_dh > new_average_min_dh)
+      return NO_BEST_CONFIGURATION;
+  }
+
+  //Keep the facets
+  typedef CGAL::Triple<Vertex_handle, Vertex_handle, Vertex_handle> Facet_vvv;
+  typedef boost::unordered_map<Facet_vvv, std::size_t> FaceMapIndex;
+  boost::unordered_set<Facet> outer_mirror_facets;
+
+  FaceMapIndex facet_map_indices;
+  std::vector<Facet> mirror_facets;
+  circ = Cell_circulator(done);
+  do
+  {
+    // facet opposite to vh0
+    int curr_vh0_id = circ->index(vh0);
+    Facet n_vh0_facet = tr.mirror_facet(Facet(circ, curr_vh0_id));
+
+    outer_mirror_facets.insert(n_vh0_facet);
+
+    Facet_vvv face0 = make_vertex_triple(circ->vertex(indices(curr_vh0_id, 0)),
+                                         circ->vertex(indices(curr_vh0_id, 1)),
+                                         circ->vertex(indices(curr_vh0_id, 2)));
+
+    typename FaceMapIndex::iterator it = facet_map_indices.find(face0);
+    if (it == facet_map_indices.end())
+    {
+      facet_map_indices[face0] = mirror_facets.size();
+      mirror_facets.push_back(n_vh0_facet);
+    }
+
+    // facet opposite to vh1
+    int curr_vh1_id = circ->index(vh1);
+    Facet n_vh1_facet = tr.mirror_facet(Facet(circ, curr_vh1_id));
+
+    outer_mirror_facets.insert(n_vh1_facet);
+
+    Facet_vvv face1 = make_vertex_triple(circ->vertex(indices(curr_vh1_id, 0)),
+                                         circ->vertex(indices(curr_vh1_id, 1)),
+                                         circ->vertex(indices(curr_vh1_id, 2)));
+    it = facet_map_indices.find(face1);
+    if (it == facet_map_indices.end())
+    {
+      facet_map_indices[face1] = mirror_facets.size();
+      mirror_facets.push_back(n_vh1_facet);
+    }
+  }
+  while (++circ != done);
+
+  /*
+  c3t3.remove_from_complex( ch0 );
+  c3t3.remove_from_complex( ch1 );
+  c3t3.remove_from_complex( cell_to_remove );
+
+  tr.flip(edge);
+
+  for( int i = 0 ; i < facets.size() ; i ++ ){
+  Cell_handle new_cell = facets[i].first->neighbor( facets[i].second );
+  c3t3.add_to_complex( new_cell, si );
+  }
+  */
+
+  //Update cells
+  ch0->set_vertex(vh0_id, vh2);
+  ch1->set_vertex(vh1_id, vh3);
+
+  // "New" cells are not created, only modified/updated
+  std::vector<Cell_handle> cells_to_update;
+  cells_to_update.push_back(ch0);
+  cells_to_update.push_back(ch1);
+
+  //Update adjacencies and vertices' cells
+  for (Cell_handle ch : cells_to_update)
+  {
+    for (int v = 0; v < 4; ++v)
+    {
+      Facet_vvv face = make_vertex_triple(ch->vertex(indices(v, 0)),
+                                          ch->vertex(indices(v, 1)),
+                                          ch->vertex(indices(v, 2)));
+      typename FaceMapIndex::iterator it = facet_map_indices.find(face);
+      if (it == facet_map_indices.end())
+      {
+        facet_map_indices[face] = mirror_facets.size();
+        mirror_facets.push_back(Facet(ch, v));
+      }
+      else
+      {
+        Facet mirror_facet = mirror_facets[it->second];
+
+        //Update neighbor
+        mirror_facet.first->set_neighbor(mirror_facet.second, ch);
+        ch->set_neighbor(v, mirror_facet.first);
+      }
+      ch->vertex(v)->set_cell(ch);
+    }
+  }
+
+  // Update c3t3
+  update_c3t3_facets(c3t3, cells_to_update, outer_mirror_facets);
+
+  c3t3.remove_from_complex(cell_to_remove);
+  tr.tds().delete_cell(cell_to_remove);
+
+  /********************VALIDITY CHECK***************************/
+  //if (check_validity)
+  //{
+  //  post_sliver_Removal_cells.clear();
+  //  post_sliver_Removal_cells.push_back(ch0);
+  //  post_sliver_Removal_cells.push_back(ch1);
+
+  //  double new_min_dh = min_dihedral_angle<Gt>(ch0);
+  //  new_min_dh = std::min(new_min_dh, min_dihedral_angle<Gt>(ch1));
+
+  //  post_sliver_Removal_vertices.clear();
+  //  post_sliver_Removal_vertices.push_back(vh2);
+  //  post_sliver_Removal_vertices.push_back(vh3);
+
+  //  if (!is_well_oriented(ch0))
+  //    return INVALID_ORIENTATION;
+  //  if (!is_well_oriented(ch1))
+  //    return INVALID_ORIENTATION;
+  //  if (!tr.is_valid(ch0))
+  //    return INVALID_CELL;
+  //  if (!tr.is_valid(ch1))
+  //    return INVALID_CELL;
+
+  //  for (int i = 0; i < 4; ++i){
+  //    if (!tr.is_valid(ch0->neighbor(i)))
+  //      return INVALID_CELL;
+  //    if (!tr.is_valid(ch1->neighbor(i)))
+  //      return INVALID_CELL;
+  //    if (!tr.tds().is_valid(ch0->vertex(i)))
+  //      return INVALID_VERTEX;
+  //    if (!tr.tds().is_valid(ch1->vertex(i))){
+  //      return INVALID_VERTEX;
+  //    }
+  //  }
+
+  //  if ((curr_min_dh - new_min_dh) > 0.01){
+  //    std::cout << "Three_to_two_flip::Flip not improving the quality: " << curr_min_dh << " to " << new_min_dh << std::endl;
+  //    return INVALID_CELL;
+  //  }
+  //}
+  /***********************************************************/
+
+  return VALID_FLIP;
+}
+
+template<typename C3t3, typename CandidatesQueue>
+void find_best_flip_to_improve_dh(C3t3& c3t3,
+                                  typename C3t3::Edge& edge,
+                                  typename C3t3::Vertex_handle vh2,
+                                  typename C3t3::Vertex_handle vh3,
+                                  CandidatesQueue& candidates,
+                                  double curr_min_dh,
+                                  bool is_sliver_well_oriented = true,
+                                  int e_id = 0)
+{
+  typedef typename C3t3::Triangulation  Tr;
+  typedef typename C3t3::Vertex_handle  Vertex_handle;
+  typedef typename C3t3::Cell_handle    Cell_handle;
+  typedef typename C3t3::Facet          Facet;
+  typedef typename Tr::Facet_circulator Facet_circulator;
+  typedef typename Tr::Cell_circulator  Cell_circulator;
+  typedef typename Tr::Geom_traits      Gt;
+  typedef typename Gt::FT               FT;
+
+  // std::cout << "find_best_flip_to_improve_dh boundary " << std::endl;
+  Tr& tr = c3t3.triangulation();
+
+  Vertex_handle vh0 = edge.first->vertex(edge.second);
+  Vertex_handle vh1 = edge.first->vertex(edge.third);
+
+  Facet_circulator curr_fcirc = tr.incident_facets(edge);
+  Facet_circulator curr_fdone = curr_fcirc;
+
+  //Only keep the possible flips
+  std::vector<Vertex_handle> opposite_vertices;
+  int nb_cells_around_edge = 0;
+  do
+  {
+    Vertex_handle vh;
+    //Get the ids of the opposite vertices
+    for (int i = 0; i < 3; ++i)
+    {
+      Vertex_handle curr_vertex = curr_fcirc->first->vertex(indices(curr_fcirc->second, i));
+      if ( curr_vertex != vh0
+           && curr_vertex != vh1
+           && (curr_vertex == vh2 || curr_vertex == vh3))
+      {
+        vh = curr_vertex;
+        Facet_circulator facet_circulator(curr_fcirc);
+        Facet_circulator facet_done(curr_fcirc);
+
+        facet_done--;
+        facet_circulator++;
+        facet_circulator++;
+
+        bool is_edge = false;
+        do
+        {
+          //Get the ids of the opposite vertices
+          for (int j = 0; j < 3; ++j)
+          {
+            Vertex_handle curr = facet_circulator->first->vertex(
+                                          indices(facet_circulator->second, j));
+            if (curr != vh0  && curr != vh1)
+            {
+              Cell_handle ch;
+              int i0, i1;
+              if (tr.is_edge(curr, vh, ch, i0, i1))
+                is_edge = true;
+            }
+          }
+        } while (++facet_circulator != facet_done);
+
+        if (!is_edge && !tr.is_infinite(vh))
+          opposite_vertices.push_back(vh);
+      }
+    }
+    nb_cells_around_edge++;
+  }
+  while (++curr_fcirc != curr_fdone);
+
+  if (nb_cells_around_edge < 4)
+    return;
+
+  //Facets that will be used to create new cells i.e. all the facets opposite to vh1 and don't have vh
+  //Facets that will be used to update cells i.e. all the facets opposite to vh0 will be set to vh: facet.first->set_vertex( facet.second, vh )
+
+  Cell_circulator cell_circulator = tr.incident_cells(edge);
+  Cell_circulator done = cell_circulator;
+
+  for (std::size_t i = 0; i < opposite_vertices.size(); ++i)
+  {
+    Vertex_handle vh = opposite_vertices[i];
+    bool keep = true;
+
+    std::vector<Facet> facets;
+    do
+    {
+      //Store it if it do not have vh
+      if (!cell_circulator->has_vertex(vh))
+      {
+        //Facets opposite to vh0
+        Facet facet_vh0(cell_circulator, cell_circulator->index(vh0));
+
+        //Facets opposite to vh1
+        Facet facet_vh1(cell_circulator, cell_circulator->index(vh1));
+
+        facets.push_back(facet_vh1);
+        facets.push_back(facet_vh0);
+      }
+    } while (++cell_circulator != done);
+
+
+    FT min_flip_dihedral_angle = (std::numeric_limits<FT>::max)();
+    for (const Facet& fi : facets)
+    {
+      if (!tr.is_infinite(fi.first))
+      {
+        if (is_well_oriented(tr, vh, fi.first->vertex(indices(fi.second, 0)),
+                             fi.first->vertex(indices(fi.second, 1)),
+                             fi.first->vertex(indices(fi.second, 2))))
+        {
+          min_flip_dihedral_angle = (std::min)(min_flip_dihedral_angle,
+                                               min_dihedral_angle(tr, vh, fi.first->vertex(indices(fi.second, 0)),
+                                                   fi.first->vertex(indices(fi.second, 1)),
+                                                   fi.first->vertex(indices(fi.second, 2))));
+        }
+        else
+        {
+          keep = false;
+          break;
+        }
+      }
+    }
+
+    if (keep && (curr_min_dh  < min_flip_dihedral_angle || !is_sliver_well_oriented))
+    {
+      //std::cout << "vh " << vh->info() <<" old " << curr_min_dh << " min " << min_flip_dihedral_angle << std::endl;
+      candidates.push(std::make_pair(min_flip_dihedral_angle, std::make_pair(vh, e_id)));
+    }
+  }
+}
+
+template<typename C3t3, typename CandidatesQueue>
+void find_best_flip_to_improve_dh(C3t3& c3t3,
+                                  typename C3t3::Edge& edge,
+                                  CandidatesQueue& candidates,
+                                  double curr_min_dh,
+                                  bool is_sliver_well_oriented = true,
+                                  int e_id = 0)
+{
+  typedef typename C3t3::Triangulation  Tr;
+  typedef typename C3t3::Vertex_handle  Vertex_handle;
+  typedef typename C3t3::Cell_handle    Cell_handle;
+  typedef typename C3t3::Facet          Facet;
+  typedef typename Tr::Facet_circulator Facet_circulator;
+  typedef typename Tr::Cell_circulator  Cell_circulator;
+  typedef typename Tr::Geom_traits      Gt;
+  typedef typename Gt::FT               FT;
+
+  Tr& tr = c3t3.triangulation();
+
+  Vertex_handle vh0 = edge.first->vertex(edge.second);
+  Vertex_handle vh1 = edge.first->vertex(edge.third);
+
+  Facet_circulator curr_fcirc = tr.incident_facets(edge);
+  Facet_circulator curr_fdone = curr_fcirc;
+
+  //Only keep the possible flips
+  std::vector<Vertex_handle> opposite_vertices;
+  int nb_cells_around_edge = 0;
+  do
+  {
+    Vertex_handle vh;
+    //Get the ids of the opposite vertices
+    for (int i = 0; i < 3; ++i)
+    {
+      Vertex_handle curr_vertex = curr_fcirc->first->vertex(
+                                    indices(curr_fcirc->second, i));
+      if (curr_vertex != vh0 && curr_vertex != vh1)
+      {
+        vh = curr_vertex;
+        break;
+      }
+    }
+
+    Facet_circulator facet_circulator = curr_fcirc;
+    Facet_circulator facet_done = curr_fcirc;
+
+    facet_done--;
+    facet_circulator++;
+    facet_circulator++;
+    bool is_edge = false;
+    do
+    {
+      //Get the ids of the opposite vertices
+      for (int i = 0; i < 3; ++i)
+      {
+        Vertex_handle curr_vertex = facet_circulator->first->vertex(
+                                      indices(facet_circulator->second, i));
+        if (curr_vertex != vh0  && curr_vertex != vh1)
+        {
+          Cell_handle ch;
+          int i0, i1;
+          if (tr.is_edge(curr_vertex, vh, ch, i0, i1))
+            is_edge = true;
+        }
+      }
+    } while (++facet_circulator != facet_done);
+
+    if (!is_edge && !tr.is_infinite(vh))
+      opposite_vertices.push_back(vh);
+
+    nb_cells_around_edge++;
+  }
+  while (++curr_fcirc != curr_fdone);
+  if (nb_cells_around_edge < 4)
+    return;
+
+  //Facets that will be used to create new cells
+  //    i.e. all the facets opposite to vh1 and don't have vh
+  //Facets that will be used to update cells
+  //    i.e. all the facets opposite to vh0 will be set to vh:
+  //    facet.first->set_vertex( facet.second, vh )
+
+  Cell_circulator cell_circulator = tr.incident_cells(edge);
+  Cell_circulator done = cell_circulator;
+
+  for (std::size_t i = 0; i < opposite_vertices.size(); ++i)
+  {
+    Vertex_handle vh = opposite_vertices[i];
+    bool keep = true;
+
+    std::vector<Facet> facets;
+    do
+    {
+      //Store it if it do not have vh
+      if (!cell_circulator->has_vertex(vh))
+      {
+        //Facets opposite to vh0
+        Facet facet_vh0(cell_circulator, cell_circulator->index(vh0));
+
+        //Facets opposite to vh1
+        Facet facet_vh1(cell_circulator, cell_circulator->index(vh1));
+
+        facets.push_back(facet_vh1);
+        facets.push_back(facet_vh0);
+      }
+    }
+    while (++cell_circulator != done);
+
+    FT min_flip_dihedral_angle = (std::numeric_limits<FT>::max)();
+    for (const Facet& fi : facets)
+    {
+      if (!tr.is_infinite(fi.first))
+      {
+        if (is_well_oriented(tr, vh, fi.first->vertex(indices(fi.second, 0)),
+                             fi.first->vertex(indices(fi.second, 1)),
+                             fi.first->vertex(indices(fi.second, 2))))
+        {
+          min_flip_dihedral_angle = (std::min)(min_flip_dihedral_angle,
+                                               min_dihedral_angle(tr, vh, fi.first->vertex(indices(fi.second, 0)),
+                                                   fi.first->vertex(indices(fi.second, 1)),
+                                                   fi.first->vertex(indices(fi.second, 2))));
+        }
+        else
+        {
+          keep = false;
+          break;
+        }
+      }
+    }
+
+    if (keep && (curr_min_dh  < min_flip_dihedral_angle || !is_sliver_well_oriented))
+    {
+      //std::cout << "vh " << vh->info() <<" old " << curr_min_dh << " min " << min_flip_dihedral_angle << std::endl;
+      candidates.push(std::make_pair(min_flip_dihedral_angle, std::make_pair(vh, e_id)));
+    }
+  }
+}
+
+template<typename C3t3, typename Visitor>
+Sliver_removal_result flip_n_to_m(C3t3& c3t3,
+                                  typename C3t3::Edge& edge,
+                                  typename C3t3::Vertex_handle vh,
+                                  Visitor& visitor,
+                                  bool check_validity = false)
+{
+  CGAL_USE(check_validity);
+  // std::cout << "n_to_m_flip::start" << std::endl;
+  typedef typename C3t3::Triangulation  Tr;
+  typedef typename C3t3::Vertex_handle  Vertex_handle;
+  typedef typename C3t3::Cell_handle    Cell_handle;
+  typedef typename C3t3::Facet          Facet;
+  typedef typename Tr::Facet_circulator Facet_circulator;
+  typedef typename Tr::Cell_circulator  Cell_circulator;
+
+  Tr& tr = c3t3.triangulation();
+
+  Vertex_handle vh0 = edge.first->vertex(edge.second);
+  Vertex_handle vh1 = edge.first->vertex(edge.third);
+
+  //This vertex will have its valence augmenting a lot,
+  //TODO take the best one
+
+  //TODO!!!! Check that the created edges do not exist!!!
+
+  Facet_circulator facet_circulator = tr.incident_facets(edge);
+  Facet_circulator done_facet_circulator = facet_circulator;
+  bool look_for_vh_iterator = true;
+  do
+  {
+    facet_circulator++;
+
+    //Get the ids of the opposite vertices
+    for (int i = 0; i < 3; ++i)
+    {
+      if (facet_circulator->first->vertex(indices(facet_circulator->second, i)) == vh)
+        look_for_vh_iterator = false;
+    }
+  } while (facet_circulator != done_facet_circulator && look_for_vh_iterator);
+
+  if (look_for_vh_iterator) {
+    std::cout << "Vertex not an opposite of the edge!!" << std::endl;
+    return NOT_FLIPPABLE;
+  }
+
+  Facet_circulator facet_done(facet_circulator);
+  facet_done--;
+  facet_circulator++;
+  facet_circulator++;
+
+  std::vector<Vertex_handle> vertices_around_edge;
+  do
+  {
+    //Get the ids of the opposite vertices
+    for (int i = 0; i < 3; ++i)
+    {
+      Vertex_handle curr_vertex = facet_circulator->first->vertex(
+                                    indices(facet_circulator->second, i));
+      if (curr_vertex != vh0  && curr_vertex != vh1)
+      {
+        Cell_handle ch;
+        int i0, i1;
+        if (tr.is_edge(curr_vertex, vh, ch, i0, i1))
+          return NOT_FLIPPABLE;
+
+        vertices_around_edge.push_back(curr_vertex);
+      }
+    }
+  } while (++facet_circulator != facet_done);
+
+
+  std::vector<Cell_handle> cells_around_edge;
+  boost::container::small_vector<Cell_handle, 20> to_remove;
+
+  //Neighbors that will need to be updated after flip
+  boost::unordered_set<Facet> neighbor_facets;
+
+  //Facets that will be used to create new cells
+  // i.e. all the facets opposite to vh1 and don't have vh
+  std::vector<Facet> facets_for_new_cells;
+
+  //Facets that will be used to update cells
+  // i.e. all the facets opposite to vh0 will be set to vh :
+  // facet.first->set_vertex( facet.second, vh )
+  std::vector<Facet> facets_for_updated_cells;
+
+  Cell_circulator cell_circulator = tr.incident_cells(edge);
+  Cell_circulator done = cell_circulator;
+  do
+  {
+    cells_around_edge.push_back(cell_circulator);
+
+    //Facets opposite to vh0
+    Facet facet_vh0(cell_circulator, cell_circulator->index(vh0));
+    neighbor_facets.insert(tr.mirror_facet(facet_vh0));
+
+    //Facets opposite to vh1
+    Facet facet_vh1(cell_circulator, cell_circulator->index(vh1));
+    neighbor_facets.insert(tr.mirror_facet(facet_vh1));
+
+    //Store it if it do not have vh
+    if (cell_circulator->has_vertex(vh)) {
+      to_remove.push_back(cell_circulator);
+    }
+    else
+    {
+      facets_for_new_cells.push_back(facet_vh1);
+      facets_for_updated_cells.push_back(facet_vh0);
+    }
+    //
+    //        if( ! is_well_oriented( cell_circulator ) )
+    //            return WRONG;
+  }
+  while (++cell_circulator != done);
+
+  //Check that the result will be valid
+  for (const Facet& fi : facets_for_new_cells)
+  {
+    if ( !tr.is_infinite(fi.first)
+         && !is_well_oriented(tr, vh, fi.first->vertex(indices(fi.second, 0)),
+                              fi.first->vertex(indices(fi.second, 1)),
+                              fi.first->vertex(indices(fi.second, 2))))
+      return NOT_FLIPPABLE;
+  }
+  for (const Facet& fi : facets_for_updated_cells)
+  {
+    if ( !tr.is_infinite(fi.first)
+         && !is_well_oriented(tr, vh, fi.first->vertex(indices(fi.second, 0)),
+                              fi.first->vertex(indices(fi.second, 1)),
+                              fi.first->vertex(indices(fi.second, 2))))
+      return NOT_FLIPPABLE;
+  }
+
+  ///********************VALIDITY CHECK***************************/
+  //double current_min_dh = DBL_MAX;
+
+  //if (check_validity){
+
+  //  pre_sliver_Removal_cells.clear();
+  //  do{
+  //    pre_sliver_Removal_cells.push_back(K::Tetrahedron_3(cell_circulator->vertex(0)->point(), cell_circulator->vertex(1)->point(),
+  //      cell_circulator->vertex(2)->point(), cell_circulator->vertex(3)->point()));
+
+  //    if (!tr.is_infinite(cell_circulator))
+  //      current_min_dh = std::min(current_min_dh, min_dihedral_angle(cell_circulator));
+  //  } while (++cell_circulator != done);
+
+  //  pre_sliver_Removal_vertices.clear();
+  //  pre_sliver_Removal_vertices.push_back(vh->point());
+
+  //  previous_edges.clear();
+  //  previous_edges.push_back(std::make_pair(vh0->point(), vh1->point()));
+  //}
+  ///*************************************************************/
+
+  //Subdomain index?
+  typename C3t3::Subdomain_index subdomain = to_remove[0]->subdomain_index();
+  visitor.before_flip(to_remove[0]);
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+  for (std::size_t i = 1; i < to_remove.size(); ++i)
+    CGAL_assertion(subdomain == to_remove[i]->subdomain_index());
+#endif
+
+  std::vector<Cell_handle> cells_to_update;
+
+  //Create new cells
+  for (const Facet& fi : facets_for_new_cells)
+  {
+    Cell_handle new_cell = tr.tds().create_cell();
+
+    for (int v = 0; v < 4; v++){
+      new_cell->set_vertex(v, fi.first->vertex(v));
+    }
+
+    new_cell->set_vertex(fi.second, vh);
+
+    c3t3.add_to_complex(new_cell, subdomain);
+    visitor.after_flip(new_cell);
+    cells_to_update.push_back(new_cell);
+  }
+
+  //Update_existing cells
+  for (const Facet& fi : facets_for_updated_cells)
+  {
+    fi.first->set_vertex(fi.second, vh);
+    cells_to_update.push_back(fi.first);
+  }
+
+  typedef CGAL::Triple<Vertex_handle, Vertex_handle, Vertex_handle> Facet_vvv;
+  typedef boost::unordered_map<Facet_vvv, std::size_t> FaceMapIndex;
+
+  FaceMapIndex facet_map_indices;
+  std::vector<Facet> facets;
+
+  for (const Facet& f : neighbor_facets)
+  {
+    Cell_handle ch = f.first;
+    int v = f.second;
+
+    Facet_vvv face = make_vertex_triple(ch->vertex(indices(v,0)),
+                                        ch->vertex(indices(v,1)),
+                                        ch->vertex(indices(v,2)));
+    typename FaceMapIndex::iterator it = facet_map_indices.find(face);
+    if (it == facet_map_indices.end())
+    {
+      facet_map_indices[face] = facets.size();
+      facets.push_back(Facet(ch, v));
+    }
+  }
+
+  //Update adjacencies and vertices cells
+  for (Cell_handle ch : cells_to_update)
+  {
+    for (int v = 0; v < 4; v++)
+    {
+      Facet_vvv face = make_vertex_triple(ch->vertex(indices(v,0)),
+                                          ch->vertex(indices(v,1)),
+                                          ch->vertex(indices(v,2)));
+      typename FaceMapIndex::iterator it = facet_map_indices.find(face);
+      if (it == facet_map_indices.end())
+      {
+        facet_map_indices[face] = facets.size();
+        facets.push_back(Facet(ch, v));
+      }
+      else
+      {
+        Facet facet = facets[it->second];
+
+        //Update neighbor
+        facet.first->set_neighbor(facet.second, ch);
+        ch->set_neighbor(v, facet.first);
+      }
+      ch->vertex(v)->set_cell(ch);
+    }
+  }
+
+  // Update c3t3
+  update_c3t3_facets(c3t3, cells_to_update, neighbor_facets);
+
+  //Remove cells
+  for (Cell_handle ch : to_remove)
+  {
+    c3t3.remove_from_complex(ch);
+    tr.tds().delete_cell(ch);
+  }
+
+  ///********************VALIDITY CHECK***************************/
+  //if (check_validity){
+
+  //  double new_min_dh = DBL_MAX;
+
+  //  post_sliver_Removal_cells.clear();
+  //  for (unsigned int i = 0; i < cells_to_update.size(); ++i){
+  //    post_sliver_Removal_cells.push_back(cells_to_update[i]);
+
+  //    if (!tr.is_infinite(cells_to_update[i]))
+  //      new_min_dh = std::min(new_min_dh, min_dihedral_angle(cells_to_update[i]));
+  //  }
+
+  //  post_sliver_Removal_vertices.clear();
+  //  for (unsigned int i = 0; i < vertices_around_edge.size(); ++i){
+  //    post_sliver_Removal_vertices.push_back(vertices_around_edge[i]);
+  //  }
+
+  //  current_edges.clear();
+  //  for (unsigned int i = 0; i < vertices_around_edge.size(); ++i){
+  //    current_edges.push_back(std::make_pair(vertices_around_edge[i]->point(), vh->point()));
+  //  }
+
+
+  //  for (unsigned int i = 0; i < cells_to_update.size(); ++i){
+  //    if (!tr.is_valid(cells_to_update[i]))
+  //      return INVALID_CELL;
+
+  //    for (int v = 0; v < 4; v++){
+  //      if (!tr.is_valid(cells_to_update[i]->neighbor(v)))
+  //        return INVALID_CELL;
+
+  //      if (!tr.tds().is_valid(cells_to_update[i]->vertex(v)))
+  //        return INVALID_VERTEX;
+
+  //    }
+  //  }
+
+  //  if ((current_min_dh - new_min_dh) > 0.01){
+  //    std::cout << pre_sliver_Removal_cells.size() << " to " << post_sliver_Removal_cells.size() << " flip not improving the quality: " <<
+  //      current_min_dh << " to " << new_min_dh << std::endl;
+  //    return INVALID_CELL;
+  //  }
+
+  //}
+  ///***********************************************************/
+
+  // std::cout << "n_to_m_flip::end with success" << std::endl;
+
+  return VALID_FLIP;
+}
+
+
+template<typename C3t3, typename Visitor>
+Sliver_removal_result flip_n_to_m(typename C3t3::Edge& edge,
+                                  C3t3& c3t3,
+                                  std::vector<typename C3t3::Vertex_handle>& boundary_vertices,
+                                  const Flip_Criterion& criterion,
+                                  Visitor& visitor)
+{
+  typedef typename C3t3::Vertex_handle Vertex_handle;
+  typedef typename C3t3::Triangulation::Cell_circulator Cell_circulator;
+  typedef typename C3t3::Triangulation::Geom_traits Gt;
+  typedef typename Gt::FT FT;
+  typename C3t3::Triangulation& tr = c3t3.triangulation();
+
+  Sliver_removal_result result = NOT_FLIPPABLE;
+
+  typedef std::pair<double, std::pair<Vertex_handle, int> > Angle_and_vertex;
+
+  //std::cout << "n_to_m_flip " << boundary_vertices.size() << std::endl;
+  if (criterion == MIN_ANGLE_BASED)
+  {
+    std::priority_queue<Angle_and_vertex> candidates;
+
+    Cell_circulator circ = c3t3.triangulation().incident_cells(edge);
+    Cell_circulator done = circ;
+
+    FT curr_min_dh = min_dihedral_angle(tr, circ++);
+    while (circ != done)
+    {
+      curr_min_dh = (std::min)(curr_min_dh, min_dihedral_angle(tr, circ++));
+    }
+    if (boundary_vertices.size() == 2)
+      find_best_flip_to_improve_dh(c3t3, edge, boundary_vertices[0], boundary_vertices[1],
+                                   candidates, curr_min_dh);
+    else
+      find_best_flip_to_improve_dh(c3t3, edge, candidates, curr_min_dh);
+
+    bool flip_performed = false;
+    while (!candidates.empty() && !flip_performed)
+    {
+      Angle_and_vertex curr_cost_vpair = candidates.top();
+      candidates.pop();
+
+      //std::cout << curr_min_dh << " old, current " << curr_cost_vpair.second.first->info() <<" and angle " << curr_cost_vpair.first << std::endl;
+
+      if (curr_min_dh >= curr_cost_vpair.first)
+        return NO_BEST_CONFIGURATION;
+
+      result = flip_n_to_m(c3t3, edge, curr_cost_vpair.second.first, visitor);
+
+      if (result != NOT_FLIPPABLE)
+        flip_performed = true;
+    }
+  }
+
+  return result;
+}
+
+template<typename C3t3, typename Visitor>
+Sliver_removal_result find_best_flip(typename C3t3::Edge& edge,
+                                     C3t3& c3t3,
+                                     const Flip_Criterion& criterion,
+                                     Visitor& visitor)
+{
+  typedef typename C3t3::Triangulation        Tr;
+  typedef typename C3t3::Vertex_handle        Vertex_handle;
+  typedef typename Tr::Facet_circulator       Facet_circulator;
+
+  Tr& tr = c3t3.triangulation();
+
+  const Vertex_handle v0 = edge.first->vertex(edge.second);
+  const Vertex_handle v1 = edge.first->vertex(edge.third);
+
+  Facet_circulator circ = tr.incident_facets(edge);
+  Facet_circulator done = circ;
+
+  //Identify the vertices around this edge
+  boost::unordered_set<Vertex_handle> vertices_around_edge;
+  bool boundary_edge = false;
+  bool hull_edge = false;
+
+  boost::unordered_set<Vertex_handle> boundary_vertices;
+//  boost::unordered_set<Vertex_handle> hull_vertices;
+  do
+  {
+    //Get the ids of the opposite vertices
+    for (int i = 0; i < 3; ++i)
+    {
+      Vertex_handle vi = circ->first->vertex(indices(circ->second, i));
+      if (vi != v0 && vi != v1)
+      {
+        vertices_around_edge.insert(vi);
+
+        if ( circ->first->subdomain_index()
+             != circ->first->neighbor(circ->second)->subdomain_index())
+        {
+          boundary_edge = true;
+          boundary_vertices.insert(vi);
+        }
+
+        if ( tr.is_infinite(circ->first)
+             != tr.is_infinite(circ->first->neighbor(circ->second)))
+        {
+          hull_edge = true;
+          //hull_vertices.insert(vi);
+        }
+      }
+    }
+  }
+  while (++circ != done);
+
+
+  //Check if not feature edge
+  if (boundary_vertices.size() > 2)
+    return NOT_FLIPPABLE;
+
+  // perform flip when possible
+  Sliver_removal_result res = NOT_FLIPPABLE;
+  if (vertices_around_edge.size() == 3)
+  {
+    if (!boundary_edge && !hull_edge)
+    {
+      std::vector<Vertex_handle> vertices;
+      vertices.insert(vertices.end(), vertices_around_edge.begin(), vertices_around_edge.end());
+      res = flip_3_to_2(edge, c3t3, vertices, criterion);
+    }
+  }
+  else
+  {
+    //TODO fix for hull edges
+    // if( hull_edge )
+    //    return n_to_m_flip( edge, hull_vertices, flip_criterion, check_validity );
+    if (!hull_edge)
+    {
+      std::vector<Vertex_handle> vertices;
+      vertices.insert(vertices.end(), boundary_vertices.begin(), boundary_vertices.end());
+      res = flip_n_to_m(edge, c3t3, vertices, criterion, visitor);
+      //return n_to_m_flip(edge, boundary_vertices, flip_criterion);
+    }
+  }
+
+
+  return res;
+}
+
+
+template<typename VertexPair, typename C3t3, typename Visitor>
+std::size_t flip_all_edges(const std::vector<VertexPair>& edges,
+                           C3t3& c3t3,
+                           const Flip_Criterion& criterion,
+                           Visitor& visitor)
+{
+  typedef typename C3t3::Triangulation Tr;
+  typedef typename Tr::Cell_handle   Cell_handle;
+  typedef typename Tr::Edge          Edge;
+
+  Tr& tr = c3t3.triangulation();
+
+  std::size_t count = 0;
+  for (const VertexPair vp : edges)
+  {
+    Cell_handle ch;
+    int i0, i1;
+    if (tr.is_edge(vp.first, vp.second, ch, i0, i1))
+    {
+      Edge edge(ch, i0, i1);
+
+      Sliver_removal_result res = find_best_flip(edge, c3t3, criterion, visitor);
+      if (res == INVALID_CELL || res == INVALID_VERTEX || res == INVALID_ORIENTATION)
+      {
+        std::cout << "FLIP PROBLEM!!!!" << std::endl;
+        return count;
+      }
+      if (res == VALID_FLIP)
+      {
+        ++count;
+#ifdef CGAL_TETRAHEDRAL_REMESHING_VERBOSE_PROGRESS
+        std::cout << "\rFlip... (";
+        std::cout << count << " flips)";
+        std::cout.flush();
+#endif
+      }
+    }
+  }
+  return count;
+}
+
+template<typename C3T3, typename CellSelector, typename Visitor>
+void flip_edges(C3T3& c3t3,
+                const bool protect_boundaries,
+                CellSelector cell_selector,
+                Visitor& visitor)
+{
+  CGAL_USE(protect_boundaries);
+  typedef typename C3T3::Triangulation       T3;
+  typedef typename T3::Vertex_handle         Vertex_handle;
+  typedef typename std::pair<Vertex_handle, Vertex_handle> Edge_vv;
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_VERBOSE
+  std::cout << "Flip edges...";
+  std::cout.flush();
+  std::size_t nb_flips = 0;
+#endif
+
+  //const Flip_Criterion criterion = VALENCE_MIN_DH_BASED;
+
+  //collect long edges
+
+  //compute vertices normals map?
+
+  // typedef typename C3T3::Surface_patch_index Surface_patch_index;
+  // typedef boost::unordered_map<Surface_patch_index, unsigned int> Spi_map;
+  //if (!protect_boundaries)
+  //{
+  //  std::cout << "\tBoundary flips" << std::endl;
+  //  //Boundary flip
+  //  std::vector<Spi_map> boundary_vertices_valences;
+  //  std::vector<Edge_vv> boundary_edges;
+
+  //  collectBoundaryEdges(boundary_edges);
+
+  //  computeVerticesValences(boundary_vertices_valences);
+
+  //  if (criterion == VALENCE_BASED)
+  //    flipBoundaryEdges(boundary_edges, boundary_vertices_valences, VALENCE_BASED);
+  //  else
+  //    flipBoundaryEdges(boundary_edges, boundary_vertices_valences, MIN_ANGLE_BASED);
+  //}
+
+  std::vector<Edge_vv> inside_edges;
+  get_internal_edges(c3t3,
+                     cell_selector,
+                     std::back_inserter(inside_edges));
+
+  //if (criterion == VALENCE_BASED)
+  //  flip_inside_edges(inside_edges);
+  //else
+  //{
+#ifdef CGAL_TETRAHEDRAL_REMESHING_VERBOSE
+  nb_flips =
+#endif
+    flip_all_edges(inside_edges, c3t3, MIN_ANGLE_BASED, visitor);
+  //}
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_VERBOSE
+  std::cout << " done (" << nb_flips << " flips)." << std::endl;
+#endif
+}
+
+}//namespace internal
+}//namespace Tetrahedral_remeshing
+}//namespace CGAL
+
+#endif // CGAL_INTERNAL_FLIP_EDGES_H
diff -pruN 5.0.3-1/include/CGAL/Tetrahedral_remeshing/internal/FMLS.h 5.1-1/include/CGAL/Tetrahedral_remeshing/internal/FMLS.h
--- 5.0.3-1/include/CGAL/Tetrahedral_remeshing/internal/FMLS.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Tetrahedral_remeshing/internal/FMLS.h	2020-09-08 12:15:49.000000000 +0000
@@ -0,0 +1,815 @@
+// Copyright (c) 2020 GeometryFactory (France) and Telecom Paris (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Tetrahedral_remeshing/include/CGAL/Tetrahedral_remeshing/internal/FMLS.h $
+// $Id: FMLS.h ab05dde 2020-06-12T08:08:56+02:00 Jane Tournois
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Jane Tournois, Noura Faraj, Jean-Marc Thiery, Tamy Boubekeur
+
+#ifndef CGAL_TETRAHEDRAL_REMESHING_FMLS_H
+#define CGAL_TETRAHEDRAL_REMESHING_FMLS_H
+
+#include <CGAL/license/Tetrahedral_remeshing.h>
+
+// -------------------------------------------
+// FMLS
+// A Fast Moving Least Square operator for 3D
+// points sets.
+// -------------------------------------------
+
+#include <CGAL/number_utils.h>
+#include <CGAL/Point_3.h>
+#include <CGAL/Vector_3.h>
+
+#include <cmath>
+#include <vector>
+#include <unordered_set>
+
+#include <CGAL/Tetrahedral_remeshing/internal/tetrahedral_remeshing_helpers.h>
+
+#include <boost/functional/hash.hpp>
+
+
+namespace CGAL
+{
+namespace Tetrahedral_remeshing
+{
+namespace internal
+{
+// --------------------------------------------------------------
+//  MLS Projection
+// --------------------------------------------------------------
+
+inline double wendland(double x, double h)
+{
+  x = CGAL::abs(x);
+  if (x < h)
+    return CGAL::square(CGAL::square(1 - x / h)) * (4 * x / h + 1);
+  else
+    return 0.0;
+}
+
+//inline void setPNSample(std::vector<float>& p,
+//                        const std::size_t& i,
+//                        const float x, const float y, const float z,
+//                        const float nx, const float ny, const float nz)
+//{
+//  p[6 * i] = x;
+//  p[6 * i + 1] = y;
+//  p[6 * i + 2] = z;
+//  p[6 * i + 3] = nx;
+//  p[6 * i + 4] = ny;
+//  p[6 * i + 5] = nz;
+//}
+
+template<typename Gt>
+inline CGAL::Vector_3<Gt> projectOn(const CGAL::Vector_3<Gt>& x,
+                                    const CGAL::Vector_3<Gt>& N,
+                                    const CGAL::Vector_3<Gt>& P)
+{
+  typename Gt::Compute_scalar_product_3 scalar_product
+    = Gt().compute_scalar_product_3_object();
+
+  typename Gt::FT w = scalar_product((x - P), N);
+  return x - (N * w);
+}
+
+template<typename Gt>
+inline typename Gt::FT length(const CGAL::Vector_3<Gt>& v)
+{
+  return CGAL::approximate_sqrt(v.squared_length());
+}
+
+template<typename Gt>
+inline typename Gt::FT distance(const CGAL::Vector_3<Gt>& v1,
+                                const CGAL::Vector_3<Gt>& v2)
+{
+  CGAL::Vector_3<Gt> diff(v1.x() - v2.x(),
+                          v1.y() - v2.y(),
+                          v1.z() - v2.z());
+  return length(diff);
+}
+
+template<typename Gt>
+inline void weightedPointCombination(const CGAL::Vector_3<Gt>& x,
+                                     const CGAL::Vector_3<Gt>& pi,
+                                     const CGAL::Vector_3<Gt>& ni,
+                                     double sigma_s, bool bilateral, double sigma_r,
+                                     bool hermite,
+                                     CGAL::Vector_3<Gt>& c, CGAL::Vector_3<Gt>& nc, double& sumW)
+{
+  double w = wendland(distance(x, pi), sigma_s);
+  if (bilateral)
+    w *= wendland(length(x - projectOn(x, ni, pi)), sigma_r);
+  if (hermite)
+    c += w * projectOn(x, ni, pi);
+  else
+    c += w * pi;
+  nc += w * ni;
+  sumW += w;
+}
+
+template<typename Gt>
+class FMLS
+{
+  typedef typename Gt::Vector_3 Vector_3;
+  typedef typename Gt::FT       FT;
+
+public:
+  // FMLS provide MLS projection and filtering from a point set.
+  // The underlying data structure is a simple list of float in the PN format
+  // A PN object is a list of 6xfloat32 chunk :
+  // x0,y0,z0,nx0,ny0,nz0,x1,y1,z1,nx1,ny1,nz1,...
+  // with {xi,yi,zi} the position and {nxi,nyi,nzi} the normal vector of the
+  // i-th point sample. A PN can be read from and write to file directly
+  // (identity serialization) and is handled as a simple float32 pointer.
+  //
+  // Use the 'fast*' methods. Brute force methods are inserted only for comparison.
+  //
+  // Memory policy: a FMLS class manages itself al its belonging objects.
+  // Therefore, PN and filtered PN lists are the property of this class, and should
+  // be copied if modified outside the class.
+  FMLS()
+  {
+    PNSize = 0;
+    PNScale = 1.0f;
+    MLSRadius = 0.1f;
+    bilateralRange = 0.05f;
+    bilateral = false;
+    hermite = false;
+    numIter = 1;
+  }
+
+  // --------------------------------------------------------------
+  //  Main Interface
+  // --------------------------------------------------------------
+
+//  std::vector<double> createPN(std::size_t size)
+//  {
+//    return std::vector<double>(size * SURFEL_SIZE);
+//  }
+
+  void setPN(const std::vector<double>& newPN,
+             const std::size_t newPNSize,
+             const double& pointSpacing)
+  {
+    freeCPUMemory();
+    PN = newPN;
+    PNSize = newPNSize;
+    computePNScale();
+    MLSRadius = 3 * pointSpacing / PNScale;
+    grid.clear();
+    grid.init(PN, PNSize, MLSRadius * PNScale);
+  }
+
+  // Compute, according to the current point sampling stored in FMLS, the MLS projection
+  // of p and store the resulting position in q and normal in n.
+  void fastProjectionCPU(const Vector_3& p, Vector_3& q, Vector_3& n) const
+  {
+    double sigma_s = PNScale * MLSRadius;
+    double sigma_r = bilateralRange;
+
+    const Vector_3 g = (p - Vector_3(grid.getMinMax()[0], grid.getMinMax()[1], grid.getMinMax()[2])) / sigma_s;
+
+    std::array<std::size_t, 3> gxyz;
+    for (int j = 0; j < 3; ++j)
+    {
+      if (g[j] < 0.)
+        gxyz[j] = 0;
+      if (g[j] >= grid.getRes()[j])
+        gxyz[j] = grid.getRes()[j] - 1;
+      else
+        gxyz[j] = static_cast<std::size_t>(std::floor(g[j]));
+    }
+
+    std::array<std::size_t, 3> minIt;
+    std::array<std::size_t, 3> maxIt;
+    for (std::size_t j = 0; j < 3; ++j)
+    {
+      if (gxyz[j] == 0)
+        minIt[j] = 0;
+      else
+        minIt[j] = gxyz[j] - 1;
+      if (gxyz[j] == grid.getRes()[j] - 1)
+        maxIt[j] = (grid.getRes()[j] - 1);
+      else
+        maxIt[j] = gxyz[j] + 1;
+    }
+    Vector_3 c = CGAL::NULL_VECTOR;
+    double sumW = 0.f;
+    std::array<std::size_t, 3> it;
+    for (it[0] = minIt[0]; it[0] <= maxIt[0]; it[0]++)
+      for (it[1] = minIt[1]; it[1] <= maxIt[1]; it[1]++)
+        for (it[2] = minIt[2]; it[2] <= maxIt[2]; it[2]++) {
+          std::size_t gridIndex = grid.getLUTElement(it[0], it[1], it[2]);
+          if (gridIndex == 2 * PNSize)
+            continue;
+          std::size_t neigh = grid.getCellIndicesSize(it[0], it[1], it[2]);
+          for (std::size_t j = 0; j < neigh; j++) {
+            std::size_t k = grid.getIndicesElement(it[0], it[1], it[2], j);
+            Vector_3 pk(PN[6 * k], PN[6 * k + 1], PN[6 * k + 2]);
+            Vector_3 nk(PN[6 * k + 3], PN[6 * k + 4], PN[6 * k + 5]);
+            weightedPointCombination(p, pk, nk, sigma_s, bilateral, sigma_r, hermite, c, n, sumW);
+          }
+        }
+    if (sumW == 0.f) {
+      n = Vector_3(1.f, 0.f, 0.f);
+      q = p;
+    }
+    else {
+      c /= sumW;
+      normalize(n, Gt());
+      q = projectOn(p, n, c);
+    }
+  }
+
+//  // Compute the MLS projection of the list of point stored in pv and store the resulting
+//  // positions and normal in qv. qv must be preallocated to stroe 6*pvSize float32.
+//  // The strid indicates the offsets in qv (the defautl value of 3 means that the qv
+//  // is compact: pv={x0,y0,z0,x1,y1,z1...}. If pv contains also normals for instance,
+//  // the stride should be set to 6.
+//  void fastProjectionCPU(const std::vector<float>& pv,
+//                         const std::size_t pvSize,
+//                         std::vector<float>& qv,
+//                         std::size_t stride = 3) const
+//  {
+//    for (std::size_t i = 0; i < pvSize; i++)
+//    {
+//      Vector_3 p(pv[stride * i], pv[stride * i + 1], pv[stride * i + 2]);
+//      Vector_3 q, n;
+//      for (unsigned int j = 0; j < numIter; j++)
+//      {
+//        q = CGAL::NULL_VECTOR;
+//        n = CGAL::NULL_VECTOR;
+//        fastProjectionCPU(p, q, n);
+//        p = q;
+//      }
+//      setPNSample(qv, i, q[0], q[1], q[2], n[0], n[1], n[2]);
+//    }
+//  }
+
+//  // Brute force version. O(PNSize) complexity. For comparison only.
+//  void projectionCPU(const Vector_3& x, Vector_3& q, Vector_3& n)
+//  {
+//    float sigma_s = MLSRadius * PNScale;
+//    float sigma_r = bilateralRange;
+//    Vector_3 p(x);
+//    for (unsigned int k = 0; k < numIter; k++) {
+//      Vector_3 c = CGAL::NULL_VECTOR;
+//      n = CGAL::NULL_VECTOR;
+//      float sumW = 0.f;
+//      for (unsigned int j = 0; j < PNSize; j++) {
+//        Vector_3 pj(PN[6 * j], PN[6 * j + 1], PN[6 * j + 2]);
+//        Vector_3 nj(PN[6 * j + 3], PN[6 * j + 4], PN[6 * j + 5]);
+//        weightedPointCombination(p, pj, nj, sigma_s, bilateral, sigma_r, hermite, c, n, sumW);
+//      }
+//      c /= sumW;
+//      n.normalize();
+//      q = projectOn(p, n, c);
+//      p = q;
+//    }
+//  }
+//
+//  // Brute force version. O(pvSize*PNSize) complexity. For comparison only.
+//  void projectionCPU(const std::vector<float>& pv,
+//                     unsigned int pvSize,
+//                     std::vector<float>& qv,
+//                     unsigned int stride = 3)
+//  {
+//    for (int i = 0; i < int(pvSize); i++) {
+//      Vector_3 p(pv[stride * i], pv[stride * i + 1], pv[stride * i + 2]);
+//      Vector_3 q, n;
+//      for (unsigned int j = 0; j < numIter; j++) {
+//        q = CGAL::NULL_VECTOR;
+//        n = CGAL::NULL_VECTOR;
+//        projectionCPU(p, q, n);
+//        p = q;
+//      }
+//      setPNSample(qv, i, q[0], q[1], q[2], n[0], n[1], n[2]);
+//    }
+//  }
+
+  // --------------------------------------------------------------
+  //  Accessors
+  // --------------------------------------------------------------
+
+  // Number of elements of the PN. One elemnt is a 6-float32 chunk.
+  inline std::size_t getPNSize() const { return PNSize; }
+  inline std::vector<double>& getPN() { return PN; }
+  inline const std::vector<double>& getPN() const { return PN; }
+
+  // Min/Max corners of PN's bounding volume
+  inline const double* getMinMax() const { return grid.getMinMax(); }
+  // Radius of the bounding sphere of the PN
+  inline double getPNScale() const { return PNScale; }
+  // Normalized MLS support size
+  inline double getMLSRadius() const { return MLSRadius; }
+  inline void setMLSRadius(double s) { MLSRadius = s; grid.clear(); grid.init(PN, PNSize, MLSRadius * PNScale); }
+  // Bilateral weighting for feature preservation (inspired by [Jones 2003]).
+  inline bool isBilateral() const { return bilateral; }
+  inline void toggleBilateral(bool b) { bilateral = b; }
+  // Bilateral support size for the range weight
+  inline double getBilateralRange() const { return bilateralRange; }
+  inline void setBilateralRange(double r) { bilateralRange = r; }
+  // Hermite interpolation [Alexa 2009]
+  inline bool isHermite() const { return hermite; }
+  inline void toggleHermite(bool b) { hermite = b; }
+  // Fix number of iterations of the MLS projection
+  inline unsigned int getNumOfIter() const { return numIter; }
+  inline void setNumOfIter(unsigned int i) { numIter = i; }
+
+  // --------------------------------------------------------------
+  //  Misc.
+  // --------------------------------------------------------------
+
+  // Size of a point sample in bytes (6xfloat32: 3 for position and normal
+  static const unsigned int SURFEL_SIZE = 24;
+
+private:
+
+  void computePNScale()
+  {
+    Vector_3 c = CGAL::NULL_VECTOR;
+    for (std::size_t i = 0; i < PNSize; i++)
+      c += Vector_3(PN[6 * i], PN[6 * i + 1], PN[6 * i + 2]);
+    c /= (double)PNSize;
+    PNScale = 0.;
+    for (std::size_t i = 0; i < PNSize; i++) {
+      double r = distance(c, Vector_3(PN[6 * i], PN[6 * i + 1], PN[6 * i + 2]));
+      if (r > PNScale)
+        PNScale = r;
+    }
+  }
+
+  // --------------------------------------------------------------
+  //  3D Grid Structure
+  // --------------------------------------------------------------
+  // --------------------------------------------------------------
+  //  Grid data structure for fast r-ball neighborhood query
+  // --------------------------------------------------------------
+
+  class Grid
+  {
+  public:
+    Grid()
+      : minMax()
+      , cellSize(1.f)
+      , res()
+      , LUT()
+      , indices()
+    {}
+    ~Grid()
+    {
+      clear();
+    }
+
+    void init(const std::vector<double>& PN, std::size_t PNSize, double sigma_s)
+    {
+      cellSize = sigma_s;
+      for (std::size_t i = 0; i < 3; i++) {
+        minMax[i] = PN[i];
+        minMax[3 + i] = PN[i];
+      }
+      for (std::size_t i = 0; i < PNSize; i++)
+        for (std::size_t j = 0; j < 3; j++) {
+          if (PN[6 * i + j] < minMax[j])
+            minMax[j] = PN[6 * i + j];
+          if (PN[6 * i + j] > minMax[3 + j])
+            minMax[3 + j] = PN[6 * i + j];
+        }
+      for (std::size_t i = 0; i < 3; i++) {
+        minMax[i] -= 0.001f;
+        minMax[3 + i] += 0.001f;
+      }
+      for (std::size_t i = 0; i < 3; i++)
+        res[i] = (std::size_t)ceil((minMax[3 + i] - minMax[i]) / cellSize);
+      std::size_t LUTSize = res[0] * res[1] * res[2];
+      LUT.resize(LUTSize);
+      LUT.assign(LUTSize, 0);
+
+      std::size_t nonEmptyCells = 0;
+      Vector_3 gMin(minMax[0], minMax[1], minMax[2]);
+      Vector_3 gMax(minMax[3], minMax[4], minMax[5]);
+      for (std::size_t i = 0; i < PNSize; i++) {
+        std::size_t index = getLUTIndex(Vector_3(PN[6 * i], PN[6 * i + 1], PN[6 * i + 2]));
+        if (LUT[index] == 0)
+          nonEmptyCells++;
+        LUT[index]++;
+      }
+      std::size_t indicesSize = PNSize + nonEmptyCells;
+      indices.reserve(indicesSize);
+      indices.assign(indicesSize, 0);
+
+      std::size_t cpt = 0;
+      for (std::size_t i = 0; i < res[0]; i++)
+        for (std::size_t j = 0; j < res[1]; j++)
+          for (std::size_t k = 0; k < res[2]; k++) {
+            std::size_t index = getLUTIndex(i, j, k);
+            if (LUT[index] != 0) {
+              indices[cpt] = LUT[index];
+              LUT[index] = cpt;
+              cpt += indices[cpt] + 1;
+              indices[cpt - 1] = 0; // local iterator for subsequent filling
+            }
+            else
+              LUT[index] = 2 * PNSize;
+          }
+      for (std::size_t i = 0; i < PNSize; i++) {
+        Vector_3 p = Vector_3(PN[6 * i], PN[6 * i + 1], PN[6 * i + 2]);
+        std::size_t indicesIndex = getLUTElement(p);
+        std::size_t totalCount = indices[indicesIndex];
+        std::size_t countIndex = indicesIndex + totalCount;
+        std::size_t currentCount = indices[countIndex];
+        if (currentCount < indices[indicesIndex])
+          indices[countIndex]++;
+        std::size_t pIndex = indicesIndex + 1 + currentCount;
+        indices[pIndex] = i;
+      }
+    }
+
+    void clear()
+    {
+      cellSize = 1.f;
+    }
+
+    // Accessors
+
+    inline const std::array<double, 6> getMinMax() const { return minMax; }
+    inline const std::array<std::size_t, 3> getRes() const { return res; }
+    inline double getCellSize() const { return cellSize; }
+    inline std::vector<std::size_t>& getLUT() { return LUT; }
+    inline const std::vector<std::size_t>& getLUT() const { return LUT; }
+    inline std::size_t getLUTIndex(const std::size_t i,
+                                    const std::size_t j,
+                                    const std::size_t k) const
+    {
+      return k * res[0] * res[1] + j * res[0] + i;
+    }
+    inline std::size_t getLUTElement(const std::size_t i,
+                                      const std::size_t j,
+                                      const std::size_t k) const
+    {
+      return LUT[getLUTIndex(i, j, k)];
+    }
+    std::size_t getLUTIndex(const Vector_3& x) const
+    {
+      const Vector_3 vp = (x - Vector_3(minMax[0], minMax[1], minMax[2])) / cellSize;
+      std::array<std::size_t, 3> p;
+      for (int j = 0; j < 3; j++)
+      {
+        if (vp[j] < 0)
+          p[j] = 0;
+        if (vp[j] >= res[j])
+          p[j] = res[j] - 1;
+        else
+          p[j] = static_cast<std::size_t>(std::floor(vp[j]));
+      }
+      std::size_t index = p[2] * res[0] * res[1]
+                        + p[1] * res[0]
+                        + p[0];
+      return index;
+    }
+    inline std::size_t getLUTElement(const Vector_3& x) const {
+      return LUT[getLUTIndex(x)];
+    }
+    inline std::vector<std::size_t>& getIndices() { return indices; }
+    inline const std::vector<std::size_t>& getIndices() const { return indices; }
+    inline std::size_t getCellIndicesSize(std::size_t i,
+                                           std::size_t j,
+                                           std::size_t k) const {
+      return indices[getLUTElement(i, j, k)];
+    }
+    inline std::size_t getIndicesElement(std::size_t i,
+                                          std::size_t j,
+                                          std::size_t k,
+                                          std::size_t e) const {
+      return indices[getLUTElement(i, j, k) + 1 + e];
+    }
+
+  private:
+    std::array<double, 6> minMax;
+    double cellSize;
+    std::array<std::size_t, 3> res;
+    std::vector<std::size_t> LUT; // 3D Index Look-Up Table
+    std::vector<std::size_t> indices; // 3D Grid data
+  };
+
+
+  // --------------------------------------------------------------
+  //  Memory Managment
+  // --------------------------------------------------------------
+
+  void freeCPUMemory()
+  {
+    PN.clear();
+    PNSize = 0;
+  }
+
+  // --------------------------------------------------------------
+  //  CPU Data
+  // --------------------------------------------------------------
+
+  std::vector<double> PN;
+  std::size_t PNSize;
+  double PNScale; // size of the bounding sphere radius
+  double MLSRadius;
+  double bilateralRange;
+  bool bilateral;
+  bool hermite;
+  unsigned int numIter;
+  Grid grid;
+};
+
+template<typename Subdomain__FMLS,
+         typename Subdomain__FMLS_indices,
+         typename VerticesNormalsMap,
+         typename VerticesSurfaceIndices,
+         typename C3t3>
+void createMLSSurfaces(Subdomain__FMLS& subdomain_FMLS,
+                       Subdomain__FMLS_indices& subdomain_FMLS_indices,
+                       const VerticesNormalsMap& vertices_normals,
+                       const VerticesSurfaceIndices& vertices_surface_indices,
+                       const C3t3& c3t3,
+                       const int upsample = 2) // can be 0, 1 or 2
+{
+  typedef typename C3t3::Surface_patch_index Surface_index;
+  typedef typename C3t3::Triangulation       Tr;
+  typedef typename Tr::Edge                  Edge;
+  typedef typename Tr::Vertex_handle         Vertex_handle;
+  typedef typename Tr::Geom_traits           Gt;
+  typedef typename Gt::Point_3               Point_3;
+  typedef typename Gt::Vector_3              Vector_3;
+
+  typedef typename VerticesSurfaceIndices::mapped_type    VertexSurfaces;
+  typedef typename VerticesSurfaceIndices::const_iterator VerticesSurfaceIterator;
+
+  const Tr& tr = c3t3.triangulation();
+
+  //createAreaWeightedUpSampledMLSSurfaces(0);
+  //return ;
+  subdomain_FMLS.clear();
+  subdomain_FMLS_indices.clear();
+
+  typedef boost::unordered_map<Surface_index, std::size_t> SurfaceIndexMap;
+
+  SurfaceIndexMap current_subdomain_FMLS_indices;
+  SurfaceIndexMap subdomain_sample_numbers;
+
+  //Count the number of vertices for each boundary surface (i.e. one per label)
+  for (const Vertex_handle vit : tr.finite_vertex_handles())
+  {
+    VerticesSurfaceIterator sit = vertices_surface_indices.find(vit);
+    if (sit == vertices_surface_indices.end())
+      continue;
+
+    const VertexSurfaces& v_surface_indices = vertices_surface_indices.at(vit);
+    CGAL_assertion(vit->in_dimension() <= 2);
+
+    for(const Surface_index& si : v_surface_indices)
+    {
+      subdomain_sample_numbers[si]++;
+    }
+  }
+
+  if (upsample > 0)
+  {
+#ifdef CGAL_TETRAHEDRAL_REMESHING_VERBOSE
+    std::cout << "Up sampling MLS " << upsample << std::endl;
+#endif
+    for (typename C3t3::Facets_in_complex_iterator
+      fit = c3t3.facets_in_complex_begin();
+      fit != c3t3.facets_in_complex_end(); ++fit)
+    {
+      const Surface_index surf_i = c3t3.surface_patch_index(*fit);
+      if (upsample == 1)
+        subdomain_sample_numbers[surf_i] ++;
+      else if (upsample == 2)
+        subdomain_sample_numbers[surf_i] += 4;
+    }
+  }
+
+  std::vector< std::vector<double> > pns;
+
+  std::size_t count = 0;
+  //Memory allocation for the point plus normals of the point samples
+  for (typename SurfaceIndexMap::iterator it = subdomain_sample_numbers.begin();
+       it != subdomain_sample_numbers.end(); ++it)
+  {
+    current_subdomain_FMLS_indices[it->first] = count;
+    pns.push_back(std::vector<double>(it->second * 6, 0.));
+    count++;
+  }
+
+  std::vector<std::size_t> current_v_count(count, 0);
+  std::vector<double> point_spacing(count, 0.);
+  std::vector<int> point_spacing_count(count, 0);
+
+  //Allocation of the PN
+  for (Vertex_handle vit : tr.finite_vertex_handles())
+  {
+    VerticesSurfaceIterator sit = vertices_surface_indices.find(vit);
+    if (sit == vertices_surface_indices.end())
+      continue;
+
+    const VertexSurfaces& v_surface_indices = vertices_surface_indices.at(vit);
+    CGAL_assertion(vit->in_dimension() <= 2);
+
+    for (const Surface_index& surf_i : v_surface_indices)
+    {
+      const std::size_t& fmls_id = current_subdomain_FMLS_indices[surf_i];
+
+      const Point_3& p = point(vit->point());
+
+      pns[fmls_id][6 * current_v_count[fmls_id]] = p.x();
+      pns[fmls_id][6 * current_v_count[fmls_id] + 1] = p.y();
+      pns[fmls_id][6 * current_v_count[fmls_id] + 2] = p.z();
+
+      const Vector_3& normal = vertices_normals.at(vit).at(surf_i);
+
+      pns[fmls_id][6 * current_v_count[fmls_id] + 3] = normal.x();
+      pns[fmls_id][6 * current_v_count[fmls_id] + 4] = normal.y();
+      pns[fmls_id][6 * current_v_count[fmls_id] + 5] = normal.z();
+
+      current_v_count[fmls_id]++;
+    }
+  }
+
+  typedef std::pair<Vertex_handle, Vertex_handle> Edge_vv;
+  if (upsample == 0)
+  {
+    std::unordered_set<Edge_vv, boost::hash<Edge_vv> > edgeMap;
+
+    for (typename C3t3::Facets_in_complex_iterator
+         fit = c3t3.facets_in_complex_begin();
+         fit != c3t3.facets_in_complex_end(); ++fit)
+    {
+      for (int i = 0; i < 2; i++)
+      {
+        for (int j = i + 1; j < 3; j++)
+        {
+          Edge edge(fit->first, indices(fit->second,i), indices(fit->second,j));
+
+          Vertex_handle vh0 = edge.first->vertex(edge.second);
+          Vertex_handle vh1 = edge.first->vertex(edge.third);
+          Edge_vv e = make_vertex_pair(vh0, vh1);
+          if ( vertices_surface_indices.find(vh0) != vertices_surface_indices.end()
+               && vertices_surface_indices.find(vh1) != vertices_surface_indices.end()
+               && edgeMap.find(e) == edgeMap.end())
+          {
+            edgeMap.insert(e);
+
+            const Surface_index surf_i = c3t3.surface_patch_index(*fit);
+            const std::size_t fmls_id = current_subdomain_FMLS_indices[surf_i];
+
+            point_spacing[fmls_id] += CGAL::approximate_sqrt(
+                CGAL::squared_distance(point(vh0->point()), point(vh1->point())));
+            point_spacing_count[fmls_id] ++;
+          }
+        }
+      }
+    }
+  }
+
+  if (upsample > 0)
+  {
+    for (typename C3t3::Facets_in_complex_iterator
+         fit = c3t3.facets_in_complex_begin();
+         fit != c3t3.facets_in_complex_end(); ++fit)
+    {
+      const Surface_index surf_i = c3t3.surface_patch_index(*fit);
+
+      const std::size_t fmls_id = current_subdomain_FMLS_indices[surf_i];
+
+      Vertex_handle vhs[3] = { fit->first->vertex(indices(fit->second, 0)),
+                               fit->first->vertex(indices(fit->second, 1)),
+                               fit->first->vertex(indices(fit->second, 2)) };
+      Vector_3 points[3] = { Vector_3(CGAL::ORIGIN, point(vhs[0]->point())),
+                             Vector_3(CGAL::ORIGIN, point(vhs[1]->point())),
+                             Vector_3(CGAL::ORIGIN, point(vhs[2]->point())) };
+      Vector_3 normals[3] = { vertices_normals.at(vhs[0]).at(surf_i),
+                              vertices_normals.at(vhs[1]).at(surf_i),
+                              vertices_normals.at(vhs[2]).at(surf_i) };
+
+      std::vector<Vector_3> points_to_add;
+      std::vector<Vector_3> n_points_to_add;
+
+      //Add the barycenter of the facet
+      Vector_3 barycenter = (points[0] + points[1] + points[2]) / 3.;
+      Vector_3 n_barycenter = (normals[0] + normals[1] + normals[2]);
+
+      n_barycenter = n_barycenter / CGAL::approximate_sqrt((n_barycenter * n_barycenter));
+
+      points_to_add.push_back(barycenter);
+      n_points_to_add.push_back(n_barycenter);
+
+      if (upsample == 1)
+      {
+        for (int i = 0; i < 3; i++)
+        {
+          Vector_3 space_1 = barycenter - points[i];
+
+          point_spacing[fmls_id] += CGAL::approximate_sqrt(space_1 * space_1);
+          point_spacing_count[fmls_id] ++;
+        }
+      }
+      else if (upsample == 2)
+      {
+        for (int i = 0; i < 3; i++)
+        {
+          int i1 = (i + 1) % 3;
+          int i2 = (i + 2) % 3;
+
+          Vector_3 p = (barycenter + points[i1] + points[i2]) / 3.;
+          Vector_3 n = (n_barycenter + normals[i1] + normals[i2]);
+
+          n = n / CGAL::sqrt(n * n);
+
+          points_to_add.push_back(p);
+          n_points_to_add.push_back(n);
+
+          Vector_3 space_1 = p - barycenter;
+          Vector_3 space_2 = p - points[i1];
+          Vector_3 space_3 = p - points[i2];
+
+          point_spacing[fmls_id] += CGAL::approximate_sqrt(space_1 * space_1);
+          point_spacing[fmls_id] += CGAL::approximate_sqrt(space_2 * space_2);
+          point_spacing[fmls_id] += CGAL::approximate_sqrt(space_3 * space_3);
+
+          point_spacing_count[fmls_id] += 3;
+        }
+      }
+      for (std::size_t i = 0; i < points_to_add.size(); i++)
+      {
+        Vector_3& point = points_to_add[i];
+
+        pns[fmls_id][6 * current_v_count[fmls_id]] = point.x();
+        pns[fmls_id][6 * current_v_count[fmls_id] + 1] = point.y();
+        pns[fmls_id][6 * current_v_count[fmls_id] + 2] = point.z();
+
+        Vector_3& normal = n_points_to_add[i];
+
+        pns[fmls_id][6 * current_v_count[fmls_id] + 3] = normal.x();
+        pns[fmls_id][6 * current_v_count[fmls_id] + 4] = normal.y();
+        pns[fmls_id][6 * current_v_count[fmls_id] + 5] = normal.z();
+
+        current_v_count[fmls_id]++;
+      }
+    }
+  }
+
+
+  std::size_t nb_of_mls_to_create = 0;
+  double average_point_spacing = 0;
+
+  //Cretaing the actual MLS surfaces
+  for (typename SurfaceIndexMap::iterator it = current_subdomain_FMLS_indices.begin();
+       it != current_subdomain_FMLS_indices.end(); ++it)
+  {
+    if (current_v_count[it->second] > 3)
+    {
+      nb_of_mls_to_create++;
+
+      double current_point_spacing = point_spacing[it->second] / point_spacing_count[it->second];
+      point_spacing[it->second] = current_point_spacing;
+
+      average_point_spacing += current_point_spacing;
+    }
+  }
+
+  average_point_spacing = average_point_spacing / nb_of_mls_to_create;
+
+  subdomain_FMLS.resize(nb_of_mls_to_create, FMLS<Gt>());
+
+  count = 0;
+  //Creating the actual MLS surfaces
+  for (typename SurfaceIndexMap::iterator it = current_subdomain_FMLS_indices.begin();
+       it != current_subdomain_FMLS_indices.end(); ++it)
+  {
+    if (current_v_count[it->second] > 3)
+    {
+      const double current_point_spacing = point_spacing[it->second];
+
+      //subdomain_FMLS[count].toggleHermite(true);
+      subdomain_FMLS[count].setPN(pns[it->second],
+                                  current_v_count[it->second],
+                                  current_point_spacing);
+      //  subdomain_FMLS[count].toggleHermite(true);
+      subdomain_FMLS_indices[it->first] = count;
+
+      count++;
+    }
+    else {
+      std::cout << "Problem of number for MLS : " << current_v_count[it->second] << std::endl;
+    }
+  }
+}
+
+} // internal
+} // Tetrahedral_remeshing
+} // CGAL
+
+#endif //CGAL_TETRAHEDRAL_REMESHING_FMLS_H
diff -pruN 5.0.3-1/include/CGAL/Tetrahedral_remeshing/internal/smooth_vertices.h 5.1-1/include/CGAL/Tetrahedral_remeshing/internal/smooth_vertices.h
--- 5.0.3-1/include/CGAL/Tetrahedral_remeshing/internal/smooth_vertices.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Tetrahedral_remeshing/internal/smooth_vertices.h	2020-09-08 12:15:49.000000000 +0000
@@ -0,0 +1,722 @@
+// Copyright (c) 2020 GeometryFactory (France) and Telecom Paris (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Tetrahedral_remeshing/include/CGAL/Tetrahedral_remeshing/internal/smooth_vertices.h $
+// $Id: smooth_vertices.h 5698056 2020-07-24T11:19:51+02:00 Jane Tournois
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Jane Tournois, Noura Faraj, Jean-Marc Thiery, Tamy Boubekeur
+
+#ifndef CGAL_INTERNAL_SMOOTH_VERTICES_H
+#define CGAL_INTERNAL_SMOOTH_VERTICES_H
+
+#include <CGAL/license/Tetrahedral_remeshing.h>
+
+#include <CGAL/Vector_3.h>
+
+#include <CGAL/Tetrahedral_remeshing/internal/tetrahedral_remeshing_helpers.h>
+#include <CGAL/Tetrahedral_remeshing/internal/FMLS.h>
+
+#include <boost/unordered_map.hpp>
+#include <boost/optional.hpp>
+#include <boost/container/small_vector.hpp>
+
+#include <vector>
+#include <cmath>
+#include <list>
+
+namespace CGAL
+{
+namespace Tetrahedral_remeshing
+{
+namespace internal
+{
+template<typename C3t3>
+class Tetrahedral_remeshing_smoother
+{
+  typedef typename C3t3::Triangulation       Tr;
+  typedef typename C3t3::Surface_patch_index Surface_patch_index;
+  typedef typename Tr::Vertex_handle         Vertex_handle;
+  typedef typename Tr::Edge                  Edge;
+  typedef typename Tr::Facet                 Facet;
+
+  typedef typename Tr::Geom_traits           Gt;
+  typedef typename Gt::Vector_3              Vector_3;
+  typedef typename Gt::Point_3               Point_3;
+
+private:
+  typedef  CGAL::Tetrahedral_remeshing::internal::FMLS<Gt> FMLS;
+  std::vector<FMLS> subdomain_FMLS;
+  boost::unordered_map<Surface_patch_index, std::size_t> subdomain_FMLS_indices;
+  bool m_smooth_constrained_edges;
+
+public:
+  template<typename CellSelector>
+  void init(const C3t3& c3t3,
+            const CellSelector& cell_selector,
+            const bool smooth_constrained_edges)
+  {
+    //collect a map of vertices surface indices
+    boost::unordered_map<Vertex_handle, std::vector<Surface_patch_index> > vertices_surface_indices;
+    collect_vertices_surface_indices(c3t3, vertices_surface_indices);
+
+    //collect a map of normals at surface vertices
+    boost::unordered_map<Vertex_handle,
+          boost::unordered_map<Surface_patch_index, Vector_3> > vertices_normals;
+    compute_vertices_normals(c3t3, vertices_normals, cell_selector);
+
+    // Build MLS Surfaces
+    createMLSSurfaces(subdomain_FMLS,
+                      subdomain_FMLS_indices,
+                      vertices_normals,
+                      vertices_surface_indices,
+                      c3t3);
+
+    m_smooth_constrained_edges = smooth_constrained_edges;
+  }
+
+private:
+
+  Vector_3 project_on_tangent_plane(const Vector_3& gi,
+                                    const Vector_3& pi,
+                                    const Vector_3& normal)
+  {
+    Vector_3 diff = pi - gi;
+    return gi + (normal * diff) * normal;
+  }
+
+  template<typename CellSelector>
+  boost::optional<Facet>
+  find_adjacent_facet_on_surface(const Facet& f,
+                                 const Edge& edge,
+                                 const C3t3& c3t3,
+                                 const CellSelector& cell_selector)
+  {
+    CGAL_assertion(is_boundary(c3t3, f, cell_selector));
+
+    typedef typename Tr::Facet_circulator Facet_circulator;
+
+    if (c3t3.is_in_complex(edge))
+      return {}; //do not "cross" complex edges
+    //they are likely to be sharp and not to follow the > 0 dot product criterion
+
+    const Surface_patch_index& patch = c3t3.surface_patch_index(f);
+    const Facet& mf = c3t3.triangulation().mirror_facet(f);
+
+    Facet_circulator fcirc = c3t3.triangulation().incident_facets(edge);
+    Facet_circulator fend = fcirc;
+    do
+    {
+      const Facet fi = *fcirc;
+      if (f != fi
+          && mf != fi
+          && is_boundary(c3t3, fi, cell_selector)
+          && patch == c3t3.surface_patch_index(fi))
+      {
+        return canonical_facet(fi); //"canonical" is important
+      }
+    } while (++fcirc != fend);
+
+    return {};
+  }
+
+  template<typename Gt>
+  Vector_3 compute_normal(const Facet& f,
+                          const Vector_3& reference_normal,
+                          const Gt& gt)
+  {
+    typename Gt::Construct_opposite_vector_3
+      opp = gt.construct_opposite_vector_3_object();
+    typename Gt::Compute_scalar_product_3
+      scalar_product = gt.compute_scalar_product_3_object();
+
+    Vector_3 n = CGAL::Tetrahedral_remeshing::normal(f, gt);
+    if (scalar_product(n, reference_normal) < 0.)
+      n = opp(n);
+
+    return n;
+  }
+
+  template<typename VertexNormalsMap, typename CellSelector>
+  void compute_vertices_normals(const C3t3& c3t3,
+                                VertexNormalsMap& normals_map,
+                                const CellSelector& cell_selector)
+  {
+    typename Tr::Geom_traits gt = c3t3.triangulation().geom_traits();
+    typename Tr::Geom_traits::Construct_opposite_vector_3
+      opp = gt.construct_opposite_vector_3_object();
+
+    const Tr& tr = c3t3.triangulation();
+
+    //collect all facet normals
+    boost::unordered_map<Facet, Vector_3> fnormals;
+    for (const Facet& f : tr.finite_facets())
+    {
+      if (is_boundary(c3t3, f, cell_selector))
+      {
+        const Facet cf = canonical_facet(f);
+        fnormals[cf] = CGAL::NULL_VECTOR;
+      }
+    }
+
+    for (const std::pair<Facet, Vector_3>& fn : fnormals)
+    {
+      if(fn.second != CGAL::NULL_VECTOR)
+        continue;
+
+      const Facet& f = fn.first;
+      const Facet& mf = tr.mirror_facet(f);
+      CGAL_assertion(is_boundary(c3t3, f, cell_selector));
+
+      Vector_3 start_ref = CGAL::Tetrahedral_remeshing::normal(f, tr.geom_traits());
+      if (c3t3.triangulation().is_infinite(mf.first)
+          || c3t3.subdomain_index(mf.first) < c3t3.subdomain_index(f.first))
+        start_ref = opp(start_ref);
+      fnormals[f] = start_ref;
+
+      std::list<Facet> facets;
+      facets.push_back(f);
+      while (!facets.empty())
+      {
+        const Facet ff = facets.front();
+        facets.pop_front();
+
+        const typename C3t3::Cell_handle ch = f.first;
+        const std::array<std::array<int, 2>, 3> edges
+          = {{ {{(ff.second + 1) % 4, (ff.second + 2) % 4}}, //edge 1-2
+               {{(ff.second + 2) % 4, (ff.second + 3) % 4}}, //edge 2-3
+               {{(ff.second + 3) % 4, (ff.second + 1) % 4}}  //edge 3-1
+            }}; //vertex indices in cells
+
+        const Vector_3& ref = fnormals[f];
+        for (const std::array<int, 2>& ei : edges)
+        {
+          Edge edge(ch, ei[0], ei[1]);
+          if (boost::optional<Facet> neighbor
+              = find_adjacent_facet_on_surface(f, edge, c3t3, cell_selector))
+          {
+            const Facet neigh = *neighbor; //already a canonical_facet
+            if (fnormals[neigh] == CGAL::NULL_VECTOR) //check it's not already computed
+            {
+              fnormals[neigh] = compute_normal(neigh, ref, gt);
+              facets.push_back(neigh);
+            }
+          }
+        }
+      }
+    }
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+    std::ofstream osf("dump_facet_normals.polylines.txt");
+#endif
+    for (const auto& fn : fnormals)
+    {
+      const Facet& f = fn.first;
+      const Vector_3& n = fn.second;
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+      typename Tr::Geom_traits::Point_3 fc
+        = CGAL::centroid(point(f.first->vertex(indices(f.second, 0))->point()),
+                         point(f.first->vertex(indices(f.second, 1))->point()),
+                         point(f.first->vertex(indices(f.second, 2))->point()));
+      osf << "2 " << fc << " " << (fc + n) << std::endl;
+#endif
+      const Surface_patch_index& surf_i = c3t3.surface_patch_index(f);
+
+      for (int i = 0; i < 3; ++i)
+      {
+        const Vertex_handle vi = f.first->vertex(indices(f.second, i));
+        typename VertexNormalsMap::iterator patch_vector_it = normals_map.find(vi);
+
+        if (patch_vector_it == normals_map.end()
+            || patch_vector_it->second.find(surf_i) == patch_vector_it->second.end())
+        {
+          normals_map[vi][surf_i] = n;
+        }
+        else
+        {
+          normals_map[vi][surf_i] += n;
+        }
+      }
+    }
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+    osf.close();
+    std::ofstream os("dump_normals.polylines.txt");
+    boost::unordered_map<Surface_patch_index,
+          std::vector<typename Tr::Geom_traits::Segment_3 > > ons_map;
+#endif
+
+    //normalize the computed normals
+    for (typename VertexNormalsMap::iterator vnm_it = normals_map.begin();
+         vnm_it != normals_map.end(); ++vnm_it)
+    {
+      //value type is map<Surface_patch_index, Vector_3>
+      for (typename VertexNormalsMap::mapped_type::iterator it = vnm_it->second.begin();
+           it != vnm_it->second.end(); ++it)
+      {
+        Vector_3& n = it->second;
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+        auto p = point(vnm_it->first->point());
+        os << "2 " << p << " " << (p + n) << std::endl;
+#endif
+
+        CGAL::Tetrahedral_remeshing::normalize(n, c3t3.triangulation().geom_traits());
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+        const Surface_patch_index si = it->first;
+        if (ons_map.find(si) == ons_map.end())
+          ons_map[si] = std::vector<typename Tr::Geom_traits::Segment_3>();
+        ons_map[si].push_back(typename Tr::Geom_traits::Segment_3(p, p + n));
+#endif
+      }
+    }
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+    os.close();
+    for (auto& kv : ons_map)
+    {
+      std::ostringstream oss;
+      oss << "dump_normals_normalized_" << kv.first << ".polylines.txt";
+      std::ofstream ons(oss.str());
+      for (auto s : kv.second)
+        ons << "2 " << s.source() << " " << s.target() << std::endl;
+      ons.close();
+    }
+#endif
+  }
+
+  boost::optional<Vector_3> project(const Surface_patch_index& si,
+                                    const Vector_3& gi)
+  {
+    CGAL_assertion(subdomain_FMLS_indices.find(si) != subdomain_FMLS_indices.end());
+    CGAL_assertion(!std::isnan(gi.x()) && !std::isnan(gi.y()) && !std::isnan(gi.z()));
+
+    Vector_3 point(gi.x(), gi.y(), gi.z());
+    Vector_3 res_normal = CGAL::NULL_VECTOR;
+    Vector_3 result(point);
+
+    const FMLS& fmls = subdomain_FMLS[subdomain_FMLS_indices.at(si)];
+
+    int it_nb = 0;
+    const int max_it_nb = 5;
+    const double epsilon = fmls.getPNScale() / 1000.;
+    const double sq_eps = CGAL::square(epsilon);
+
+    do
+    {
+      point = result;
+
+      fmls.fastProjectionCPU(point, result, res_normal);
+
+      if (std::isnan(result[0]) || std::isnan(result[1]) || std::isnan(result[2])) {
+        std::cout << "MLS error detected si " //<< si
+                  << "\t(size : "       << fmls.getPNSize() << ")"
+                  << "\t(point = "      << point      << " )" << std::endl;
+        return {};
+      }
+    } while ((result - point).squared_length() > sq_eps && ++it_nb < max_it_nb);
+
+    return Vector_3(result[0], result[1], result[2]);
+  }
+
+  template<typename CellRange, typename Tr>
+  bool check_inversion_and_move(const typename Tr::Vertex_handle v,
+                                const typename Tr::Point& final_pos,
+                                const CellRange& inc_cells,
+                                const Tr& /* tr */)
+  {
+    const typename Tr::Point backup = v->point(); //backup v's position
+    const typename Tr::Geom_traits::Point_3 pv = point(backup);
+
+    bool valid_orientation = false;
+    double frac = 1.0;
+    typename Tr::Geom_traits::Vector_3 move(pv, point(final_pos));
+    do
+    {
+      v->set_point(typename Tr::Point(pv + frac * move));
+
+      bool valid_try = true;
+      for (const typename Tr::Cell_handle ci : inc_cells)
+      {
+        if (CGAL::POSITIVE != CGAL::orientation(point(ci->vertex(0)->point()),
+                                                point(ci->vertex(1)->point()),
+                                                point(ci->vertex(2)->point()),
+                                                point(ci->vertex(3)->point())))
+        {
+          frac = 0.9 * frac;
+          valid_try = false;
+          break;
+        }
+      }
+      valid_orientation = valid_try;
+
+//          std::cout << std::boolalpha << "valid orientation = " << valid_orientation
+//                    << "\tfrac = " << frac << std::endl;
+    }
+    while(!valid_orientation && frac > 0.1);
+
+    if (!valid_orientation) //move failed
+      v->set_point(backup);
+
+    return valid_orientation;
+  }
+
+  void collect_vertices_surface_indices(
+    const C3t3& c3t3,
+    boost::unordered_map<Vertex_handle,
+    std::vector<Surface_patch_index> >& vertices_surface_indices)
+  {
+    for (typename C3t3::Facets_in_complex_iterator
+         fit = c3t3.facets_in_complex_begin();
+         fit != c3t3.facets_in_complex_end(); ++fit)
+    {
+      const Surface_patch_index& surface_index = c3t3.surface_patch_index(*fit);
+
+      for (int i = 0; i < 3; i++)
+      {
+        const Vertex_handle vi = fit->first->vertex(indices(fit->second, i));
+
+        std::vector<Surface_patch_index>& v_surface_indices = vertices_surface_indices[vi];
+        if (std::find(v_surface_indices.begin(), v_surface_indices.end(), surface_index) == v_surface_indices.end())
+          v_surface_indices.push_back(surface_index);
+      }
+    }
+  }
+
+public:
+  template<typename C3T3, typename CellSelector>
+  void smooth_vertices(C3T3& c3t3,
+                       const bool protect_boundaries,
+                       const CellSelector& cell_selector)
+  {
+    typedef typename C3T3::Cell_handle            Cell_handle;
+    typedef typename Gt::FT              FT;
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+    std::ofstream os_surf("smooth_surfaces.polylines.txt");
+    std::ofstream os_surf0("smooth_surfaces0.polylines.txt");
+    std::ofstream os_vol("smooth_volume.polylines.txt");
+#endif
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_VERBOSE
+    std::cout << "Smooth vertices...";
+    std::cout.flush();
+#endif
+    std::size_t nb_done = 0;
+
+    Tr& tr = c3t3.triangulation();
+
+    //collect a map of vertices surface indices
+    boost::unordered_map<Vertex_handle, std::vector<Surface_patch_index> > vertices_surface_indices;
+    if(m_smooth_constrained_edges)
+      collect_vertices_surface_indices(c3t3, vertices_surface_indices);
+
+    //collect a map of normals at surface vertices
+    boost::unordered_map<Vertex_handle,
+          boost::unordered_map<Surface_patch_index, Vector_3> > vertices_normals;
+    compute_vertices_normals(c3t3, vertices_normals, cell_selector);
+
+    //smooth()
+    const std::size_t nbv = tr.number_of_vertices();
+    boost::unordered_map<Vertex_handle, std::size_t> vertex_id;
+    std::vector<Vector_3> smoothed_positions(nbv, CGAL::NULL_VECTOR);
+    std::vector<int> neighbors(nbv, -1);
+
+    //collect ids
+    std::size_t id = 0;
+    for (const Vertex_handle v : tr.finite_vertex_handles())
+    {
+      vertex_id[v] = id++;
+    }
+
+    //collect incident cells
+    std::vector<boost::container::small_vector<Cell_handle, 40> >
+    inc_cells(nbv, boost::container::small_vector<Cell_handle, 40>());
+    for (const Cell_handle c : tr.finite_cell_handles())
+    {
+      for (int i = 0; i < 4; ++i)
+      {
+        const std::size_t idi = vertex_id[c->vertex(i)];
+        inc_cells[idi].push_back(c);
+      }
+    }
+
+    if (!protect_boundaries && m_smooth_constrained_edges)
+    {
+      /////////////// EDGES IN COMPLEX //////////////////
+      //collect neighbors
+      for (const Edge& e : tr.finite_edges())
+      {
+        if (c3t3.is_in_complex(e))
+        {
+          const Vertex_handle vh0 = e.first->vertex(e.second);
+          const Vertex_handle vh1 = e.first->vertex(e.third);
+
+          const std::size_t& i0 = vertex_id.at(vh0);
+          const std::size_t& i1 = vertex_id.at(vh1);
+
+          const bool on_feature_v0 = is_on_feature(vh0);
+          const bool on_feature_v1 = is_on_feature(vh1);
+
+          if (!c3t3.is_in_complex(vh0))
+            neighbors[i0] = (std::max)(0, neighbors[i0]);
+          if (!c3t3.is_in_complex(vh1))
+            neighbors[i1] = (std::max)(0, neighbors[i1]);
+
+          if (!c3t3.is_in_complex(vh0) && on_feature_v1)
+          {
+            const Point_3& p1 = point(vh1->point());
+            smoothed_positions[i0] = smoothed_positions[i0] + Vector_3(p1.x(), p1.y(), p1.z());
+            neighbors[i0]++;
+          }
+          if (!c3t3.is_in_complex(vh1) && on_feature_v0)
+          {
+            const Point_3& p0 = point(vh0->point());
+            smoothed_positions[i1] = smoothed_positions[i1] + Vector_3(p0.x(), p0.y(), p0.z());
+            neighbors[i1]++;
+          }
+        }
+      }
+
+      // Smooth
+      for (Vertex_handle v : tr.finite_vertex_handles())
+      {
+        const std::size_t& vid = vertex_id.at(v);
+        if (neighbors[vid] > 1)
+        {
+          Vector_3 smoothed_position = smoothed_positions[vid] / neighbors[vid];
+          Vector_3 final_position = CGAL::NULL_VECTOR;
+
+          std::size_t count = 0;
+          const Vector_3 current_pos(CGAL::ORIGIN, point(v->point()));
+
+          const std::vector<Surface_patch_index>& v_surface_indices = vertices_surface_indices[v];
+          for (const Surface_patch_index& si : v_surface_indices)
+          {
+            Vector_3 normal_projection
+              = project_on_tangent_plane(smoothed_position, current_pos, vertices_normals[v][si]);
+
+            //Check if the mls surface exists to avoid degenerated cases
+            if (boost::optional<Vector_3> mls_projection = project(si, normal_projection)) {
+              final_position = final_position + *mls_projection;
+            }
+            else {
+              final_position = final_position + normal_projection;
+            }
+            count++;
+          }
+
+          if (count > 0)
+            final_position = final_position / static_cast<FT>(count);
+          else
+            final_position = smoothed_position;
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+          os_surf << "2 " << current_pos << " " << final_position << std::endl;
+#endif
+          // move vertex
+          const typename Tr::Point new_pos(final_position.x(), final_position.y(), final_position.z());
+          if(check_inversion_and_move(v, new_pos, inc_cells[vid], tr))
+            nb_done++;
+        }
+        else if (neighbors[vid] > 0)
+        {
+          Vector_3 final_position = CGAL::NULL_VECTOR;
+
+          int count = 0;
+          const Vector_3 current_pos(CGAL::ORIGIN, point(v->point()));
+
+          const std::vector<Surface_patch_index>& v_surface_indices = vertices_surface_indices[v];
+          for (const Surface_patch_index si : v_surface_indices)
+          {
+            //Check if the mls surface exists to avoid degenerated cases
+
+            if (boost::optional<Vector_3> mls_projection = project(si, current_pos)) {
+              final_position = final_position + *mls_projection;
+            }
+            else {
+              final_position = final_position + current_pos;
+            }
+            count++;
+          }
+
+          if (count > 0)
+            final_position = final_position / static_cast<FT>(count);
+          else
+            final_position = current_pos;
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+          os_surf << "2 " << current_pos << " " << final_position << std::endl;
+#endif
+          // move vertex
+          const typename Tr::Point new_pos(final_position.x(), final_position.y(), final_position.z());
+          if(check_inversion_and_move(v, new_pos, inc_cells[vid], tr))
+            nb_done++;
+        }
+      }
+    }
+
+    smoothed_positions.assign(nbv, CGAL::NULL_VECTOR);
+    neighbors.assign(nbv, -1);
+
+    /////////////// EDGES ON SURFACE, BUT NOT IN COMPLEX //////////////////
+    if (!protect_boundaries)
+    {
+      for (const Edge& e : tr.finite_edges())
+      {
+        if (is_boundary(c3t3, e, cell_selector) && !c3t3.is_in_complex(e))
+        {
+          const Vertex_handle vh0 = e.first->vertex(e.second);
+          const Vertex_handle vh1 = e.first->vertex(e.third);
+
+          const std::size_t& i0 = vertex_id.at(vh0);
+          const std::size_t& i1 = vertex_id.at(vh1);
+
+          const bool on_feature_v0 = is_on_feature(vh0);
+          const bool on_feature_v1 = is_on_feature(vh1);
+
+          if (!on_feature_v0)
+            neighbors[i0] = (std::max)(0, neighbors[i0]);
+          if (!on_feature_v1)
+            neighbors[i1] = (std::max)(0, neighbors[i1]);
+
+          if (!on_feature_v0)
+          {
+            const Point_3& p1 = point(vh1->point());
+            smoothed_positions[i0] = smoothed_positions[i0] + Vector_3(p1.x(), p1.y(), p1.z());
+            neighbors[i0]++;
+          }
+          if (!on_feature_v1)
+          {
+            const Point_3& p0 = point(vh0->point());
+            smoothed_positions[i1] = smoothed_positions[i1] + Vector_3(p0.x(), p0.y(), p0.z());
+            neighbors[i1]++;
+          }
+        }
+      }
+
+      for (Vertex_handle v : tr.finite_vertex_handles())
+      {
+        if (v->in_dimension() != 2)
+          continue;
+
+        const std::size_t& vid = vertex_id.at(v);
+        if (neighbors[vid] > 1)
+        {
+          Vector_3 smoothed_position = smoothed_positions[vid] / static_cast<FT>(neighbors[vid]);
+          const Vector_3 current_pos(CGAL::ORIGIN, point(v->point()));
+          Vector_3 final_position = CGAL::NULL_VECTOR;
+
+          const Surface_patch_index si = surface_patch_index(v, c3t3);
+          CGAL_assertion(si != Surface_patch_index());
+
+          Vector_3 normal_projection = project_on_tangent_plane(smoothed_position,
+                                       current_pos,
+                                       vertices_normals[v][si]);
+
+          if (boost::optional<Vector_3> mls_projection = project(si, normal_projection))
+            final_position = final_position + *mls_projection;
+          else
+            final_position = smoothed_position;
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+          os_surf << "2 " << current_pos << " " << final_position << std::endl;
+#endif
+          const typename Tr::Point new_pos(final_position.x(), final_position.y(), final_position.z());
+          if(check_inversion_and_move(v, new_pos, inc_cells[vid], tr))
+            nb_done++;
+        }
+        else if (neighbors[vid] > 0)
+        {
+          const Surface_patch_index si = surface_patch_index(v, c3t3);
+          CGAL_assertion(si != Surface_patch_index());
+
+          const Vector_3 current_pos(CGAL::ORIGIN, point(v->point()));
+
+          if (boost::optional<Vector_3> mls_projection = project(si, current_pos))
+          {
+            const typename Tr::Point new_pos(CGAL::ORIGIN + *mls_projection);
+            if(check_inversion_and_move(v, new_pos, inc_cells[vid], tr))
+              nb_done++;
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+            os_surf0 << "2 " << current_pos << " " << new_pos << std::endl;
+#endif
+          }
+        }
+      }
+    }
+    CGAL_assertion(CGAL::Tetrahedral_remeshing::debug::are_cell_orientations_valid(tr));
+    ////   end if(!protect_boundaries)
+
+    smoothed_positions.assign(nbv, CGAL::NULL_VECTOR);
+    neighbors.assign(nbv, 0/*for dim 3 vertices, start counting directly from 0*/);
+
+    ////////////// INTERNAL VERTICES ///////////////////////
+    for (const Edge& e : tr.finite_edges())
+    {
+      if (!is_outside(e, c3t3, cell_selector))
+      {
+        const Vertex_handle vh0 = e.first->vertex(e.second);
+        const Vertex_handle vh1 = e.first->vertex(e.third);
+
+        const std::size_t& i0 = vertex_id.at(vh0);
+        const std::size_t& i1 = vertex_id.at(vh1);
+
+        if (c3t3.in_dimension(vh0) == 3)
+        {
+          const Point_3& p1 = point(vh1->point());
+          smoothed_positions[i0] = smoothed_positions[i0] + Vector_3(CGAL::ORIGIN, p1);
+          neighbors[i0]++;
+        }
+        if (c3t3.in_dimension(vh1) == 3)
+        {
+          const Point_3& p0 = point(vh0->point());
+          smoothed_positions[i1] = smoothed_positions[i1] + Vector_3(CGAL::ORIGIN, p0);
+          neighbors[i1]++;
+        }
+      }
+    }
+
+    for (Vertex_handle v : tr.finite_vertex_handles())
+    {
+      const std::size_t& vid = vertex_id.at(v);
+      if (c3t3.in_dimension(v) == 3 && neighbors[vid] > 1)
+      {
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+        os_vol << "2 " << point(v->point());
+#endif
+        const Vector_3 p = smoothed_positions[vid] / static_cast<FT>(neighbors[vid]);
+        typename Tr::Point new_pos(p.x(), p.y(), p.z());
+        if(check_inversion_and_move(v, new_pos, inc_cells[vid], tr))
+          nb_done++;
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+        os_vol << " " << point(v->point()) << std::endl;
+#endif
+      }
+    }
+    CGAL_assertion(CGAL::Tetrahedral_remeshing::debug::are_cell_orientations_valid(tr));
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_VERBOSE
+    std::cout << " done (" << nb_done << " vertices smoothed)." << std::endl;
+#endif
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+    CGAL::Tetrahedral_remeshing::debug::dump_vertices_by_dimension(
+      c3t3.triangulation(), "c3t3_vertices_after_smoothing");
+    os_surf.close();
+    os_vol.close();
+#endif
+  }
+
+};//end class Tetrahedral_remeshing_smoother
+}//namespace internal
+}//namespace Tetrahedral_adaptive_remeshing
+}//namespace CGAL
+
+#endif //CGAL_INTERNAL_SMOOTH_VERTICES_H
diff -pruN 5.0.3-1/include/CGAL/Tetrahedral_remeshing/internal/split_long_edges.h 5.1-1/include/CGAL/Tetrahedral_remeshing/internal/split_long_edges.h
--- 5.0.3-1/include/CGAL/Tetrahedral_remeshing/internal/split_long_edges.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Tetrahedral_remeshing/internal/split_long_edges.h	2020-09-08 12:15:49.000000000 +0000
@@ -0,0 +1,330 @@
+// Copyright (c) 2020 GeometryFactory (France) and Telecom Paris (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Tetrahedral_remeshing/include/CGAL/Tetrahedral_remeshing/internal/split_long_edges.h $
+// $Id: split_long_edges.h 74835a9 2020-07-03T07:33:01+02:00 Jane Tournois
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Jane Tournois, Noura Faraj, Jean-Marc Thiery, Tamy Boubekeur
+
+#ifndef CGAL_INTERNAL_SPLIT_LONG_EDGES_H
+#define CGAL_INTERNAL_SPLIT_LONG_EDGES_H
+
+#include <CGAL/license/Tetrahedral_remeshing.h>
+
+#include <boost/bimap.hpp>
+#include <boost/bimap/set_of.hpp>
+#include <boost/bimap/multiset_of.hpp>
+#include <boost/unordered_map.hpp>
+#include <boost/container/small_vector.hpp>
+
+#include <CGAL/Tetrahedral_remeshing/internal/tetrahedral_remeshing_helpers.h>
+
+#include <functional>
+#include <utility>
+
+namespace CGAL
+{
+namespace Tetrahedral_remeshing
+{
+namespace internal
+{
+template<typename C3t3>
+typename C3t3::Vertex_handle split_edge(const typename C3t3::Edge& e,
+                                        C3t3& c3t3)
+{
+  typedef typename C3t3::Triangulation       Tr;
+  typedef typename C3t3::Subdomain_index     Subdomain_index;
+  typedef typename C3t3::Surface_patch_index Surface_patch_index;
+  typedef typename Tr::Geom_traits::Point_3 Point;
+  typedef typename Tr::Facet                Facet;
+  typedef typename Tr::Vertex_handle        Vertex_handle;
+  typedef typename Tr::Cell_handle          Cell_handle;
+  typedef typename Tr::Cell_circulator      Cell_circulator;
+
+  Tr& tr = c3t3.triangulation();
+  const Vertex_handle v1 = e.first->vertex(e.second);
+  const Vertex_handle v2 = e.first->vertex(e.third);
+
+  const Point m = tr.geom_traits().construct_midpoint_3_object()
+    (point(v1->point()), point(v2->point()));
+
+  //backup subdomain info of incident cells before making changes
+  short dimension = (std::max)((std::max)(1, c3t3.in_dimension(v1)), c3t3.in_dimension(v2));
+  boost::unordered_map<Facet, Subdomain_index> cells_info;
+  boost::unordered_map<Facet, std::pair<Vertex_handle, Surface_patch_index> > facets_info;
+
+  // check orientation and collect incident cells to avoid circulating twice
+  boost::container::small_vector<Cell_handle, 30> inc_cells;
+  Cell_circulator circ = tr.incident_cells(e);
+  Cell_circulator end = circ;
+  do
+  {
+    inc_cells.push_back(circ);
+    if (tr.is_infinite(circ))
+    {
+      ++circ;
+      continue;
+    }
+
+    //1st half-cell
+    std::array<Point, 4> pts = { point(circ->vertex(0)->point()),
+                                 point(circ->vertex(1)->point()),
+                                 point(circ->vertex(2)->point()),
+                                 point(circ->vertex(3)->point()) };
+    const int i1 = circ->index(v1);
+    const Point p1 = pts[i1];
+    pts[i1] = m;
+    if(CGAL::orientation(pts[0], pts[1], pts[2], pts[3]) != CGAL::POSITIVE)
+      return Vertex_handle();
+
+    //2nd half-cell
+    pts[i1] = p1;
+    pts[circ->index(v2)] = m;
+    if (CGAL::orientation(pts[0], pts[1], pts[2], pts[3]) != CGAL::POSITIVE)
+      return Vertex_handle();
+
+    ++circ;
+  }
+  while (circ != end);
+
+  for(Cell_handle c : inc_cells)
+  {
+    const int index_v1 = c->index(v1);
+    const int index_v2 = c->index(v2);
+
+    //keys are the opposite facets to the ones not containing e,
+    //because they will not be modified
+    const Subdomain_index subdomain = c3t3.subdomain_index(c);
+    const Facet opp_facet1 = tr.mirror_facet(Facet(c, index_v1));
+    const Facet opp_facet2 = tr.mirror_facet(Facet(c, index_v2));
+
+    // volume data
+    cells_info.insert(std::make_pair(opp_facet1, subdomain));
+    cells_info.insert(std::make_pair(opp_facet2, subdomain));
+    if (c3t3.is_in_complex(c))
+      c3t3.remove_from_complex(c);
+
+    // surface data for facets of the cells to be split
+    const int findex = CGAL::Triangulation_utils_3::next_around_edge(index_v1, index_v2);
+    Surface_patch_index patch = c3t3.surface_patch_index(c, findex);
+    Vertex_handle opp_vertex = c->vertex(findex);
+    facets_info.insert(std::make_pair(opp_facet1,
+                                      std::make_pair(opp_vertex, patch)));
+    facets_info.insert(std::make_pair(opp_facet2,
+                                      std::make_pair(opp_vertex, patch)));
+
+    if(c3t3.is_in_complex(c, findex))
+      c3t3.remove_from_complex(c, findex);
+  }
+
+  // insert midpoint
+  Vertex_handle new_v = tr.tds().insert_in_edge(e);
+  new_v->set_point(typename Tr::Point(m));
+  new_v->set_dimension(dimension);
+
+  // update c3t3 with subdomain and surface patch indices
+  std::vector<Cell_handle> new_cells;
+  tr.incident_cells(new_v, std::back_inserter(new_cells));
+  for (Cell_handle new_cell : new_cells)
+  {
+    const Facet fi(new_cell, new_cell->index(new_v));
+    const Facet mfi = tr.mirror_facet(fi);
+
+    //get subdomain info back
+    CGAL_assertion(cells_info.find(mfi) != cells_info.end());
+    Subdomain_index n_index = cells_info.at(mfi);
+    if (Subdomain_index() != n_index)
+      c3t3.add_to_complex(new_cell, n_index);
+    else
+      new_cell->set_subdomain_index(Subdomain_index());
+
+    // get surface info back
+    CGAL_assertion(facets_info.find(mfi) != facets_info.end());
+    const std::pair<Vertex_handle, Surface_patch_index> v_and_opp_patch = facets_info.at(mfi);
+
+    // facet opposite to new_v (status wrt c3t3 is unchanged)
+    new_cell->set_surface_patch_index(new_cell->index(new_v),
+                                      mfi.first->surface_patch_index(mfi.second));
+
+    // new half-facet (added or not to c3t3 depending on the stored surface patch index)
+    if (Surface_patch_index() == v_and_opp_patch.second)
+      new_cell->set_surface_patch_index(new_cell->index(v_and_opp_patch.first),
+                                        Surface_patch_index());
+    else
+      c3t3.add_to_complex(new_cell,
+                          new_cell->index(v_and_opp_patch.first),
+                          v_and_opp_patch.second);
+
+    // newly created internal facet
+    for (int i = 0; i < 4; ++i)
+    {
+      const Vertex_handle vi = new_cell->vertex(i);
+      if (vi == v1 || vi == v2)
+      {
+        new_cell->set_surface_patch_index(i, Surface_patch_index());
+        break;
+      }
+    }
+
+    //the 4th facet (new_v, v_and_opp_patch.first, v1 or v2)
+    // will have its patch tagged from the other side, if needed
+  }
+
+  set_index(new_v, c3t3);
+
+  return new_v;
+}
+
+template<typename C3T3, typename CellSelector>
+bool can_be_split(const typename C3T3::Edge& e,
+                  const C3T3& c3t3,
+                  const bool protect_boundaries,
+                  CellSelector cell_selector)
+{
+  if (is_outside(e, c3t3, cell_selector))
+    return false;
+
+  if (protect_boundaries)
+  {
+    if (c3t3.is_in_complex(e))
+      return false;
+    else if (is_boundary(c3t3, e, cell_selector))
+      return false;
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+    if (!is_internal(e, c3t3, cell_selector))
+    {
+      std::cerr << "e is not inside!?" << std::endl;
+      typename C3T3::Vertex_handle v1 = e.first->vertex(e.second);
+      typename C3T3::Vertex_handle v2 = e.first->vertex(e.third);
+      std::cerr << v1->point() << " " << v2->point() << std::endl;
+    }
+#endif
+
+    CGAL_assertion(is_internal(e, c3t3, cell_selector));
+    return true;
+  }
+  else
+  {
+    return true;
+  }
+}
+
+template<typename C3T3, typename CellSelector, typename Visitor>
+void split_long_edges(C3T3& c3t3,
+                      const typename C3T3::Triangulation::Geom_traits::FT& high,
+                      const bool protect_boundaries,
+                      CellSelector cell_selector,
+                      Visitor& visitor)
+{
+  typedef typename C3T3::Triangulation       T3;
+  typedef typename T3::Cell_handle           Cell_handle;
+  typedef typename T3::Edge                  Edge;
+  typedef typename T3::Finite_edges_iterator Finite_edges_iterator;
+  typedef typename T3::Vertex_handle         Vertex_handle;
+  typedef typename std::pair<Vertex_handle, Vertex_handle> Edge_vv;
+
+  typedef typename T3::Geom_traits     Gt;
+  typedef typename T3::Geom_traits::FT FT;
+  typedef boost::bimap<
+  boost::bimaps::set_of<Edge_vv>,
+        boost::bimaps::multiset_of<FT, std::greater<FT> > >  Boost_bimap;
+  typedef typename Boost_bimap::value_type               long_edge;
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_VERBOSE
+  std::cout << "Split long edges (" << high << ")...";
+  std::cout.flush();
+  std::size_t nb_splits = 0;
+#endif
+  const FT sq_high = high*high;
+
+  //collect long edges
+  T3& tr = c3t3.triangulation();
+  Boost_bimap long_edges;
+  for (Finite_edges_iterator eit = tr.finite_edges_begin();
+       eit != tr.finite_edges_end(); ++eit)
+  {
+    Edge e = *eit;
+    if (!can_be_split(e, c3t3, protect_boundaries, cell_selector))
+      continue;
+
+    typename Gt::Compute_squared_length_3 sql
+      = tr.geom_traits().compute_squared_length_3_object();
+    FT sqlen = sql(tr.segment(e));
+    if (sqlen > sq_high)
+      long_edges.insert(long_edge(make_vertex_pair<T3>(e), sqlen));
+  }
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+  debug::dump_edges(long_edges, "long_edges.polylines.txt");
+
+  std::ofstream ofs("midpoints.off");
+  ofs << "OFF" << std::endl;
+  ofs << long_edges.size() << " 0 0" << std::endl;
+#endif
+  while(!long_edges.empty())
+  {
+    //the edge with longest length
+    typename Boost_bimap::right_map::iterator eit = long_edges.right.begin();
+    Edge_vv e = eit->second;
+#ifdef CGAL_TETRAHEDRAL_REMESHING_VERBOSE_PROGRESS
+    const double sqlen = eit->first;
+#endif
+    long_edges.right.erase(eit);
+
+    Cell_handle cell;
+    int i1, i2;
+    if ( tr.tds().is_edge(e.first, e.second, cell, i1, i2))
+    {
+      Edge edge(cell, i1, i2);
+
+      //check that splittability has not changed
+      if (!can_be_split(edge, c3t3, protect_boundaries, cell_selector))
+        continue;
+
+      visitor.before_split(tr, edge);
+      Vertex_handle vh = split_edge(edge, c3t3);
+
+      if(vh != Vertex_handle())
+        visitor.after_split(tr, vh);
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+      if (vh != Vertex_handle())
+        ofs << vh->point() << std::endl;
+#endif
+
+#if  defined(CGAL_TETRAHEDRAL_REMESHING_VERBOSE_PROGRESS) \
+|| defined(CGAL_TETRAHEDRAL_REMESHING_VERBOSE)
+      if (vh != Vertex_handle())
+        ++nb_splits;
+#endif
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_VERBOSE_PROGRESS
+      std::cout << "\rSplit (" << high << ")... ("
+                << long_edges.left.size() << " long edges, "
+                << "length  = " << std::sqrt(sqlen) << ", "
+                << nb_splits << " splits)";
+      std::cout.flush();
+#endif
+    }
+  }//end loop on long_edges
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+  if(ofs.is_open())
+    ofs.close();
+#endif
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_VERBOSE
+  std::cout << " done (" << nb_splits << " splits)." << std::endl;
+#endif
+}
+
+} // internal
+} // Tetrahedral_remeshing
+} // CGAL
+
+#endif // CGAL_INTERNAL_SPLIT_LONG_EDGES_H
diff -pruN 5.0.3-1/include/CGAL/Tetrahedral_remeshing/internal/tetrahedral_adaptive_remeshing_impl.h 5.1-1/include/CGAL/Tetrahedral_remeshing/internal/tetrahedral_adaptive_remeshing_impl.h
--- 5.0.3-1/include/CGAL/Tetrahedral_remeshing/internal/tetrahedral_adaptive_remeshing_impl.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Tetrahedral_remeshing/internal/tetrahedral_adaptive_remeshing_impl.h	2020-09-08 12:15:49.000000000 +0000
@@ -0,0 +1,645 @@
+// Copyright (c) 2020 GeometryFactory (France) and Telecom Paris (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Tetrahedral_remeshing/include/CGAL/Tetrahedral_remeshing/internal/tetrahedral_adaptive_remeshing_impl.h $
+// $Id: tetrahedral_adaptive_remeshing_impl.h 962b681 2020-07-31T15:52:56+02:00 Laurent Rineau
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Jane Tournois, Noura Faraj, Jean-Marc Thiery, Tamy Boubekeur
+
+#ifndef TETRAHEDRAL_REMESHING_IMPL_H
+#define TETRAHEDRAL_REMESHING_IMPL_H
+
+#include <CGAL/license/Tetrahedral_remeshing.h>
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_VERBOSE_PROGRESS
+#define CGAL_TETRAHEDRAL_REMESHING_VERBOSE
+#endif
+
+#include <CGAL/Mesh_complex_3_in_triangulation_3.h>
+#include <CGAL/Triangulation_utils_3.h>
+
+#include <CGAL/Tetrahedral_remeshing/internal/split_long_edges.h>
+#include <CGAL/Tetrahedral_remeshing/internal/collapse_short_edges.h>
+#include <CGAL/Tetrahedral_remeshing/internal/flip_edges.h>
+#include <CGAL/Tetrahedral_remeshing/internal/smooth_vertices.h>
+
+#include <CGAL/Tetrahedral_remeshing/internal/tetrahedral_remeshing_helpers.h>
+#include <CGAL/Tetrahedral_remeshing/internal/compute_c3t3_statistics.h>
+
+#include <boost/optional.hpp>
+
+namespace CGAL
+{
+namespace Tetrahedral_remeshing
+{
+namespace internal
+{
+class Default_remeshing_visitor
+{
+public:
+  template<typename Tr>
+  void before_split(const Tr& /* tr */, const typename Tr::Edge& /* e */) {}
+  template<typename Tr>
+  void after_split(const Tr& /* tr */, const typename Tr::Vertex_handle /* new_v */) {}
+
+  template<typename CellHandleOld, typename CellHandleNew>
+  void after_add_cell(CellHandleOld /* co */, CellHandleNew /* cn */) const {}
+
+  template<typename CellHandle>
+  void before_flip(const CellHandle /* c */) {}
+  template<typename CellHandle>
+  void after_flip(CellHandle /* c */) {}
+};
+
+template<typename Tr>
+struct All_cells_selected
+{
+  typedef typename Tr::Cell_handle argument_type;
+  typedef typename Tr::Cell::Subdomain_index Subdomain_index;
+
+  typedef bool                     result_type;
+
+  result_type operator()(const argument_type c) const
+  {
+    return c->subdomain_index() != Subdomain_index();
+  }
+};
+
+template<typename Primitive>
+struct No_constraint_pmap
+{
+public:
+  typedef Primitive                           key_type;
+  typedef bool                                value_type;
+  typedef value_type&                         reference;
+  typedef boost::read_write_property_map_tag  category;
+
+  friend bool get(const No_constraint_pmap&, const key_type&) {
+    return false;
+  }
+  friend void put(No_constraint_pmap&, const key_type&, const bool) {}
+};
+
+template<typename Triangulation
+         , typename SizingFunction
+         , typename EdgeIsConstrainedMap
+         , typename FacetIsConstrainedMap
+         , typename CellSelector
+         , typename Visitor
+         , typename CornerIndex = int
+         , typename CurveIndex = int
+         >
+class Adaptive_remesher
+{
+  typedef Triangulation Tr;
+  typedef typename Tr::Geom_traits::FT FT;
+
+  typedef CGAL::Mesh_complex_3_in_triangulation_3<Tr, CornerIndex, CurveIndex> C3t3;
+
+  typedef typename C3t3::Cell_handle         Cell_handle;
+  typedef typename C3t3::Vertex_handle       Vertex_handle;
+  typedef typename C3t3::Subdomain_index     Subdomain_index;
+  typedef typename C3t3::Surface_patch_index Surface_patch_index;
+  typedef typename C3t3::Curve_index         Curve_index;
+  typedef typename C3t3::Corner_index        Corner_index;
+
+  typedef Tetrahedral_remeshing_smoother<C3t3> Smoother;
+
+private:
+  C3t3 m_c3t3;
+  const SizingFunction& m_sizing;
+  const bool m_protect_boundaries;
+  CellSelector m_cell_selector;
+  Visitor& m_visitor;
+  Smoother m_vertex_smoother;//initialized with initial surface
+
+  C3t3* m_c3t3_pbackup;
+  Triangulation* m_tr_pbackup; //backup to re-swap triangulations when done
+
+public:
+  Adaptive_remesher(Triangulation& tr
+                    , const SizingFunction& sizing
+                    , const bool protect_boundaries
+                    , EdgeIsConstrainedMap ecmap
+                    , FacetIsConstrainedMap fcmap
+                    , bool smooth_constrained_edges
+                    , CellSelector cell_selector
+                    , Visitor& visitor
+                   )
+    : m_c3t3()
+    , m_sizing(sizing)
+    , m_protect_boundaries(protect_boundaries)
+    , m_cell_selector(cell_selector)
+    , m_visitor(visitor)
+    , m_c3t3_pbackup(NULL)
+    , m_tr_pbackup(&tr)
+  {
+    m_c3t3.triangulation().swap(tr);
+
+    init_c3t3(ecmap, fcmap);
+    m_vertex_smoother.init(m_c3t3, m_cell_selector, smooth_constrained_edges);
+
+#ifdef CGAL_DUMP_REMESHING_STEPS
+    CGAL::Tetrahedral_remeshing::debug::dump_c3t3(m_c3t3, "00-init");
+    CGAL::Tetrahedral_remeshing::debug::dump_facets_in_complex(m_c3t3,
+      "00-facets_in_complex_after_init.off");
+#endif
+  }
+
+  Adaptive_remesher(C3t3& c3t3
+                    , const SizingFunction& sizing
+                    , const bool protect_boundaries
+                    , EdgeIsConstrainedMap ecmap
+                    , FacetIsConstrainedMap fcmap
+                    , bool smooth_constrained_edges
+                    , CellSelector cell_selector
+                    , Visitor& visitor
+                   )
+    : m_c3t3()
+    , m_sizing(sizing)
+    , m_protect_boundaries(protect_boundaries)
+    , m_cell_selector(cell_selector)
+    , m_visitor(visitor)
+    , m_c3t3_pbackup(&c3t3)
+    , m_tr_pbackup(NULL)
+  {
+    m_c3t3.swap(c3t3);
+
+    init_c3t3(ecmap, fcmap);
+    m_vertex_smoother.init(m_c3t3, m_cell_selector, smooth_constrained_edges);
+
+#ifdef CGAL_DUMP_REMESHING_STEPS
+    CGAL::Tetrahedral_remeshing::debug::dump_c3t3(m_c3t3, "00-init");
+    CGAL::Tetrahedral_remeshing::debug::dump_facets_in_complex(m_c3t3,
+      "00-facets_in_complex_after_init.off");
+#endif
+  }
+
+  bool input_is_c3t3() const
+  {
+    return m_c3t3_pbackup != NULL;
+  }
+
+  void split()
+  {
+    CGAL_assertion(check_vertex_dimensions());
+
+    const FT target_edge_length = m_sizing(CGAL::ORIGIN);
+    const FT emax = FT(4)/FT(3) * target_edge_length;
+    split_long_edges(m_c3t3, emax, m_protect_boundaries,
+                     m_cell_selector, m_visitor);
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+    CGAL_assertion(tr().tds().is_valid(true));
+    CGAL_assertion(debug::are_cell_orientations_valid(tr()));
+    CGAL::Tetrahedral_remeshing::debug::dump_facets_in_complex(m_c3t3,
+      "1-facets_in_complex_after_split.off");
+    CGAL::Tetrahedral_remeshing::debug::dump_vertices_by_dimension(
+      m_c3t3.triangulation(), "1-c3t3_vertices_after_split");
+#endif
+#ifdef CGAL_DUMP_REMESHING_STEPS
+    CGAL::Tetrahedral_remeshing::debug::dump_c3t3(m_c3t3, "1-split");
+#endif
+  }
+
+  void collapse()
+  {
+    CGAL_assertion(check_vertex_dimensions());
+
+    const FT target_edge_length = m_sizing(CGAL::ORIGIN);
+    FT emin = FT(4)/FT(5) * target_edge_length;
+    FT emax = FT(4)/FT(3) * target_edge_length;
+    collapse_short_edges(m_c3t3, emin, emax, m_protect_boundaries,
+                         m_cell_selector, m_visitor);
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+    CGAL_assertion(tr().tds().is_valid(true));
+    CGAL_assertion(debug::are_cell_orientations_valid(tr()));
+    CGAL::Tetrahedral_remeshing::debug::dump_vertices_by_dimension(
+      m_c3t3.triangulation(), "2-c3t3_vertices_after_collapse");
+#endif
+#ifdef CGAL_DUMP_REMESHING_STEPS
+    CGAL::Tetrahedral_remeshing::debug::dump_c3t3(m_c3t3, "2-collapse");
+#endif
+  }
+
+  void flip()
+  {
+    flip_edges(m_c3t3, m_protect_boundaries,
+               m_cell_selector, m_visitor);
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+    CGAL_assertion(tr().tds().is_valid(true));
+    CGAL_assertion(debug::are_cell_orientations_valid(tr()));
+    CGAL::Tetrahedral_remeshing::debug::dump_vertices_by_dimension(
+      m_c3t3.triangulation(), "3-c3t3_vertices_after_flip");
+#endif
+#ifdef CGAL_DUMP_REMESHING_STEPS
+    CGAL::Tetrahedral_remeshing::debug::dump_c3t3(m_c3t3, "3-flip");
+#endif
+  }
+
+  void smooth()
+  {
+    m_vertex_smoother.smooth_vertices(m_c3t3, m_protect_boundaries, m_cell_selector);
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+    CGAL_assertion(tr().tds().is_valid(true));
+    CGAL_assertion(debug::are_cell_orientations_valid(tr()));
+    CGAL::Tetrahedral_remeshing::debug::dump_vertices_by_dimension(
+      m_c3t3.triangulation(), "4-c3t3_vertices_after_smooth");
+#endif
+#ifdef CGAL_DUMP_REMESHING_STEPS
+    CGAL::Tetrahedral_remeshing::debug::dump_c3t3(m_c3t3, "4-smooth");
+#endif
+  }
+
+  bool resolution_reached()
+  {
+    const FT target_edge_length = m_sizing(CGAL::ORIGIN);
+
+    FT emax = FT(4) / FT(3) * target_edge_length;
+    FT emin = FT(4) / FT(5) * target_edge_length;
+
+    FT sqmax = emax * emax;
+    FT sqmin = emin * emin;
+
+    typedef typename Tr::Edge Edge;
+    for (Edge e : tr().finite_edges())
+    {
+      // skip protected edges
+      if (m_protect_boundaries)
+      {
+        if(  m_c3t3.is_in_complex(e)
+             || is_boundary(m_c3t3, e, m_cell_selector))
+          continue;
+      }
+
+      FT sqlen = tr().segment(e).squared_length();
+      if (sqlen < sqmin || sqlen > sqmax)
+        return false;
+    }
+    std::cout << "Resolution reached" << std::endl;
+    return true;
+  }
+
+  //peel off slivers
+  std::size_t postprocess(const double sliver_angle = 2.)
+  {
+    if (m_protect_boundaries)
+      return 0;
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_VERBOSE
+    std::cout << "Postprocess...";
+    std::cout.flush();
+#endif
+
+    std::size_t nb_slivers_peel = 0;
+    std::vector<std::pair<Cell_handle, std::array<bool, 4> > > peelable_cells;
+#ifdef CGAL_TETRAHEDRAL_REMESHING_VERBOSE
+    double mindh = 180.;
+#endif
+    for (Cell_handle cit : tr().finite_cell_handles())
+    {
+      std::array<bool, 4> facets_on_surface;
+      if (m_c3t3.is_in_complex(cit))
+      {
+        const double dh = min_dihedral_angle(tr(), cit);
+        if(dh < sliver_angle && is_peelable(m_c3t3, cit, facets_on_surface))
+          peelable_cells.push_back(std::make_pair(cit, facets_on_surface));
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_VERBOSE
+        mindh = (std::min)(dh, mindh);
+#endif
+      }
+    }
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_VERBOSE
+    std::cout << "Min dihedral angle : " << mindh << std::endl;
+    std::cout << "Peelable cells : " << peelable_cells.size() << std::endl;
+#endif
+
+    for (auto c_i : peelable_cells)
+    {
+      Cell_handle c = c_i.first;
+      const std::array<bool, 4>& f_on_surface = c_i.second;
+
+      boost::optional<Surface_patch_index> patch;
+      for (int i = 0; i < 4; ++i)
+      {
+        if (f_on_surface[i])
+        {
+          Surface_patch_index spi = m_c3t3.surface_patch_index(c, i);
+          if (patch != boost::none && patch != spi)
+          {
+            patch = boost::none;
+            break;
+          }
+          else
+          {
+            patch = spi;
+          }
+        }
+      }
+      if(patch == boost::none)
+        continue;
+
+      for (int i = 0; i < 4; ++i)
+      {
+        if(f_on_surface[i])
+          m_c3t3.remove_from_complex(c, i);
+        else
+          m_c3t3.add_to_complex(c, i, patch.get());
+      }
+
+      m_c3t3.remove_from_complex(c);
+      ++nb_slivers_peel;
+    }
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+    CGAL_assertion(tr().tds().is_valid(true));
+    CGAL_assertion(debug::are_cell_orientations_valid(tr()));
+#endif
+#ifdef CGAL_DUMP_REMESHING_STEPS
+    CGAL::Tetrahedral_remeshing::debug::dump_c3t3(m_c3t3, "99-postprocess");
+#endif
+#ifdef CGAL_TETRAHEDRAL_REMESHING_VERBOSE
+    std::cout << "(peeling removed " << nb_slivers_peel << " slivers)" << std::endl;
+    std::cout << "done." << std::endl;
+#endif
+    return nb_slivers_peel;
+  }
+
+  void finalize()
+  {
+    if (m_c3t3_pbackup != NULL)
+      m_c3t3_pbackup->swap(m_c3t3);
+    else
+      m_tr_pbackup->swap(m_c3t3.triangulation());
+  }
+
+private:
+  void init_c3t3(const EdgeIsConstrainedMap& ecmap,
+                 const FacetIsConstrainedMap& fcmap)
+  {
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+    debug_c3t3();
+    std::size_t nbc = 0;
+    std::size_t nbf = 0;
+    std::size_t nbe = 0;
+    std::size_t nbv = 0;
+#endif
+    //update number_of_cells and number_of_facets in c3t3
+    m_c3t3.rescan_after_load_of_triangulation();
+
+    //tag cells
+    for (Cell_handle cit : tr().finite_cell_handles())
+    {
+      if (m_cell_selector(cit))
+      {
+        const Subdomain_index index = cit->subdomain_index();
+        if(!input_is_c3t3())
+          m_c3t3.remove_from_complex(cit);
+        m_c3t3.add_to_complex(cit, index);
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+        ++nbc;
+#endif
+      }
+      if (!input_is_c3t3())
+      {
+        for (int i = 0; i < 4; ++i)
+        {
+          if (cit->vertex(i)->in_dimension() == -1)
+            cit->vertex(i)->set_dimension(3);
+        }
+      }
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+      else if (input_is_c3t3() && m_c3t3.is_in_complex(cit))
+        ++nbc;
+#endif
+    }
+
+    //tag facets
+    typedef typename Tr::Facet Facet;
+    for (const Facet& f : tr().finite_facets())
+    {
+      const Facet mf = tr().mirror_facet(f);
+      const Subdomain_index s1 = f.first->subdomain_index();
+      const Subdomain_index s2 = mf.first->subdomain_index();
+      if (s1 != s2
+          || get(fcmap, f)
+          || get(fcmap, mf)
+          || (m_c3t3_pbackup == NULL && f.first->is_facet_on_surface(f.second)))
+      {
+        Surface_patch_index patch = f.first->surface_patch_index(f.second);
+        if(patch == Surface_patch_index())
+          set_surface_patch_index_to_default(s1, s2, patch);
+        m_c3t3.add_to_complex(f, patch);
+
+        const int i = f.second;
+        for (int j = 0; j < 3; ++j)
+        {
+          Vertex_handle vij = f.first->vertex(Tr::vertex_triple_index(i, j));
+          if (vij->in_dimension() == -1 || vij->in_dimension() > 2)
+            vij->set_dimension(2);
+        }
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+        ++nbf;
+#endif
+      }
+    }
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+    CGAL::Tetrahedral_remeshing::debug::dump_facets_in_complex(m_c3t3, "facets_in_complex.off");
+#endif
+
+    //tag edges
+    typedef typename Tr::Edge Edge;
+    for (const Edge& e : tr().finite_edges())
+    {
+      if (m_c3t3.is_in_complex(e))
+      {
+        CGAL_assertion(m_c3t3.in_dimension(e.first->vertex(e.second)) <= 1);
+        CGAL_assertion(m_c3t3.in_dimension(e.first->vertex(e.third)) <= 1);
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+        ++nbe;
+#endif
+        continue;
+      }
+
+      if (get(ecmap, CGAL::Tetrahedral_remeshing::make_vertex_pair<Tr>(e))
+          || nb_incident_subdomains(e, m_c3t3) > 2
+          || nb_incident_surface_patches(e, m_c3t3) > 1)
+      {
+        m_c3t3.add_to_complex(e, 1);
+
+        Vertex_handle v = e.first->vertex(e.second);
+        if (v->in_dimension() == -1 || v->in_dimension() > 1)
+          v->set_dimension(1);
+
+        v = e.first->vertex(e.third);
+        if (v->in_dimension() == -1 || v->in_dimension() > 1)
+          v->set_dimension(1);
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+        ++nbe;
+#endif
+      }
+    }
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+    CGAL::Tetrahedral_remeshing::debug::dump_edges_in_complex(m_c3t3, "edges_in_complex.polylines.txt");
+#endif
+
+    //tag vertices
+    Corner_index corner_id = 0;
+    for (Vertex_handle vit : tr().finite_vertex_handles())
+    {
+      if ( vit->in_dimension() == 0
+           || nb_incident_complex_edges(vit, m_c3t3) > 2)
+      {
+        if(!m_c3t3.is_in_complex(vit))
+          m_c3t3.add_to_complex(vit, ++corner_id);
+
+        if (vit->in_dimension() == -1 || vit->in_dimension() > 0)
+          vit->set_dimension(0);
+
+        vit->set_index(corner_id);
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+        ++nbv;
+#endif
+      }
+    }
+
+    for (Vertex_handle v : tr().finite_vertex_handles())
+      set_index(v, m_c3t3);
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
+    std::cout << "C3t3 ready :" << std::endl;
+    std::cout << "\t cells    = " << nbc << std::endl;
+    std::cout << "\t facets   = " << nbf << std::endl;
+    std::cout << "\t edges    = " << nbe << std::endl;
+    std::cout << "\t vertices = " << nbv << std::endl;
+
+    CGAL::Tetrahedral_remeshing::debug::dump_vertices_by_dimension(
+      m_c3t3.triangulation(), "c3t3_vertices_");
+#endif
+  }
+
+private:
+  bool check_vertex_dimensions()
+  {
+    for (Vertex_handle vit : tr().finite_vertex_handles())
+    {
+      // dimension is -1 for Mesh_3 "far points"
+      // for other vertices, it is in [0; 3]
+      if (vit->in_dimension() < -1 || vit->in_dimension() > 3)
+        return false;
+    }
+    return true;
+  }
+  void debug_c3t3()
+  {
+    for (typename Tr::Facet f : tr().finite_facets())
+    {
+      typename Tr::Facet mf = tr().mirror_facet(f);
+      CGAL_assertion(m_c3t3.is_in_complex(f) == m_c3t3.is_in_complex(mf));
+    }
+  }
+
+  template<typename PatchIndex>
+  void set_surface_patch_index_to_default(const Subdomain_index&,
+                                          const Subdomain_index&,
+                                          PatchIndex& patch)
+  {
+    if(m_c3t3.number_of_facets() == 0)
+      patch = 1;
+    else
+      patch = m_c3t3.surface_patch_index(*m_c3t3.facets_in_complex_begin());
+  }
+
+  void set_surface_patch_index_to_default(const Subdomain_index& s1,
+                                          const Subdomain_index& s2,
+                                          std::pair<Subdomain_index, Subdomain_index>& patch)
+  {
+    patch = (s1 < s2) ? std::make_pair(s1, s2) : std::make_pair(s2, s1);
+  }
+
+public:
+  Tr& tr()
+  {
+    return m_c3t3.triangulation();
+  }
+  const Tr& tr() const
+  {
+    return m_c3t3.triangulation();
+  }
+
+  void remesh(const std::size_t& max_it,
+              const std::size_t& nb_extra_iterations)
+  {
+    std::size_t it_nb = 0;
+    while (it_nb < max_it)
+    {
+      ++it_nb;
+#ifdef CGAL_TETRAHEDRAL_REMESHING_VERBOSE
+      std::cout << "# Iteration " << it_nb << " #" << std::endl;
+#endif
+      if (!resolution_reached())
+      {
+        split();
+        collapse();
+      }
+      flip();
+      smooth();
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_VERBOSE
+      std::cout << "# Iteration " << it_nb << " done : "
+                << tr().number_of_vertices()
+                << " vertices #" << std::endl;
+#endif
+#ifdef CGAL_DUMP_REMESHING_STEPS
+      std::ostringstream ossi;
+      ossi << "statistics_" << it_nb << ".txt";
+      Tetrahedral_remeshing::internal::compute_statistics(
+        tr(), m_cell_selector, ossi.str().c_str());
+#endif
+    }
+
+    while (it_nb < max_it + nb_extra_iterations)
+    {
+      ++it_nb;
+
+      flip();
+      smooth();
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_VERBOSE
+      std::cout << "# Iteration " << it_nb << " (flip and smooth only) done : "
+                << tr().number_of_vertices()
+                << " vertices #" << std::endl;
+#endif
+#ifdef CGAL_DUMP_REMESHING_STEPS
+      std::ostringstream ossi;
+      ossi << "statistics_" << it_nb << ".txt";
+      Tetrahedral_remeshing::internal::compute_statistics(
+        tr(),  m_cell_selector, ossi.str().c_str());
+#endif
+    }
+
+    postprocess(); //peel off boundary slivers
+
+    finalize();
+    //Warning : triangulation() is now empty
+  }
+
+};//end class Adaptive_remesher
+
+}//end namespace internal
+}//end namespace Tetrahedral_remeshing
+}//end namespace CGAL
+
+#endif //TETRAHEDRAL_REMESHING_IMPL_H
diff -pruN 5.0.3-1/include/CGAL/Tetrahedral_remeshing/internal/tetrahedral_remeshing_helpers.h 5.1-1/include/CGAL/Tetrahedral_remeshing/internal/tetrahedral_remeshing_helpers.h
--- 5.0.3-1/include/CGAL/Tetrahedral_remeshing/internal/tetrahedral_remeshing_helpers.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Tetrahedral_remeshing/internal/tetrahedral_remeshing_helpers.h	2020-09-08 12:15:49.000000000 +0000
@@ -0,0 +1,1499 @@
+// Copyright (c) 2020 GeometryFactory (France) and Telecom Paris (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Tetrahedral_remeshing/include/CGAL/Tetrahedral_remeshing/internal/tetrahedral_remeshing_helpers.h $
+// $Id: tetrahedral_remeshing_helpers.h df044c4 2020-08-13T16:47:37+02:00 Jane Tournois
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Jane Tournois, Noura Faraj, Jean-Marc Thiery, Tamy Boubekeur
+
+#ifndef CGAL_INTERNAL_TET_REMESHING_HELPERS_H
+#define CGAL_INTERNAL_TET_REMESHING_HELPERS_H
+
+#include <CGAL/license/Tetrahedral_remeshing.h>
+
+#include <utility>
+#include <array>
+#include <iterator>
+
+#include <CGAL/Point_3.h>
+#include <CGAL/Weighted_point_3.h>
+#include <CGAL/Vector_3.h>
+#include <CGAL/utility.h>
+
+#include <CGAL/IO/File_binary_mesh_3.h>
+
+#include <boost/container/flat_set.hpp>
+#include <boost/container/small_vector.hpp>
+
+namespace CGAL
+{
+namespace Tetrahedral_remeshing
+{
+
+enum Subdomain_relation { EQUAL, DIFFERENT, INCLUDED, INCLUDES };
+enum Sliver_removal_result { INVALID_ORIENTATION, INVALID_CELL, INVALID_VERTEX,
+  NOT_FLIPPABLE, EDGE_PROBLEM, VALID_FLIP, NO_BEST_CONFIGURATION, EXISTING_EDGE };
+
+template<typename K>
+CGAL::Point_3<K> point(const CGAL::Point_3<K>& p)
+{
+  return p;
+}
+template<typename K>
+CGAL::Point_3<K> point(const CGAL::Weighted_point_3<K>& wp)
+{
+  typename K::Construct_point_3 pt = K().construct_point_3_object();
+  return pt(wp);
+}
+
+template <typename K>
+CGAL::Vector_3<K> vec(const CGAL::Point_3<K>& p)
+{
+  typename K::Construct_vector_3 v = K().construct_vector_3_object();
+  return v(CGAL::ORIGIN, p);
+}
+template <typename K>
+CGAL::Vector_3<K> vec(const CGAL::Weighted_point_3<K>& wp)
+{
+  return vec(point(wp));
+}
+
+
+const int indices_table[4][3] = { { 3, 1, 2 },
+                                  { 3, 2, 0 },
+                                  { 3, 0, 1 },
+                                  { 2, 1, 0 } };
+
+int indices(const int& i, const int& j)
+{
+  CGAL_assertion(i < 4 && j < 3);
+  if(i < 4 && j < 3)
+    return indices_table[i][j];
+  else
+    return -1;
+}
+
+template<typename Gt, typename Point>
+typename Gt::FT dihedral_angle(const Point& p,
+                               const Point& q,
+                               const Point& r,
+                               const Point& s,
+                               const Gt& gt)
+{
+  return gt.compute_approximate_dihedral_angle_3_object()(p, q, r, s);
+}
+
+template<typename Point, typename Geom_traits>
+typename Geom_traits::FT min_dihedral_angle(const Point& p,
+                                            const Point& q,
+                                            const Point& r,
+                                            const Point& s,
+                                            const Geom_traits& gt)
+{
+  typedef typename Geom_traits::FT FT;
+  FT a = CGAL::abs(dihedral_angle(p, q, r, s, gt));
+  FT min_dh = a;
+
+  a = CGAL::abs(dihedral_angle(p, r, q, s, gt));
+  min_dh = (std::min)(a, min_dh);
+
+  a = CGAL::abs(dihedral_angle(p, s, q, r, gt));
+  min_dh = (std::min)(a, min_dh);
+
+  a = CGAL::abs(dihedral_angle(q, r, p, s, gt));
+  min_dh = (std::min)(a, min_dh);
+
+  a = CGAL::abs(dihedral_angle(q, s, p, r, gt));
+  min_dh = (std::min)(a, min_dh);
+
+  a = CGAL::abs(dihedral_angle(r, s, p, q, gt));
+  min_dh = (std::min)(a, min_dh);
+
+  return min_dh;
+}
+
+template<typename Tr>
+typename Tr::Geom_traits::FT min_dihedral_angle(const Tr& tr,
+                                                const typename Tr::Vertex_handle v0,
+                                                const typename Tr::Vertex_handle v1,
+                                                const typename Tr::Vertex_handle v2,
+                                                const typename Tr::Vertex_handle v3)
+{
+  return min_dihedral_angle(point(v0->point()),
+                            point(v1->point()),
+                            point(v2->point()),
+                            point(v3->point()),
+                            tr.geom_traits());
+}
+
+template<typename Tr>
+typename Tr::Geom_traits::FT min_dihedral_angle(const Tr& tr,
+                                                const typename Tr::Cell_handle c)
+{
+  return min_dihedral_angle(tr,
+                            c->vertex(0),
+                            c->vertex(1),
+                            c->vertex(2),
+                            c->vertex(3));
+}
+
+template<typename C3t3>
+bool is_peelable(const C3t3& c3t3,
+                 const typename C3t3::Cell_handle ch,
+                 std::array<bool, 4>& facets_on_surface)
+{
+  typedef typename C3t3::Triangulation::Geom_traits::FT FT;
+  typedef typename C3t3::Facet                          Facet;
+
+  if(!c3t3.is_in_complex(ch))
+    return false;
+
+  bool on_surface = false;
+  for (int i = 0; i < 4; ++i)
+  {
+    facets_on_surface[i] = !c3t3.is_in_complex(ch->neighbor(i));
+    on_surface = on_surface || facets_on_surface[i];
+  }
+  if(!on_surface)
+    return false;
+
+  FT area_on_surface = 0.;
+  FT area_inside = 0.;
+  for (int i = 0; i < 4; ++i)
+  {
+    Facet f(ch, i);
+    const FT facet_area = CGAL::approximate_sqrt(c3t3.triangulation().triangle(f).squared_area());
+    if(facets_on_surface[i])
+      area_on_surface += facet_area;
+    else
+      area_inside += facet_area;
+  }
+
+  return (area_inside < 1.5 * area_on_surface);
+}
+
+template<typename Tr>
+typename Tr::Geom_traits::Vector_3 facet_normal(const Tr& tr,
+                                                const typename Tr::Facet& f)
+{
+  const typename Tr::Geom_traits gt = tr.geom_traits();
+  typename Tr::Geom_traits::Construct_normal_3 cn
+    = gt.construct_normal_3_object();
+  return cn(point(f.first->vertex((f.second + 1) % 4)->point()),
+            point(f.first->vertex((f.second + 2) % 4)->point()),
+            point(f.first->vertex((f.second + 3) % 4)->point()));
+}
+
+template<typename Vh>
+std::pair<Vh, Vh> make_vertex_pair(const Vh v1, const Vh v2)
+{
+  if (v2 < v1) return std::make_pair(v2, v1);
+  else         return std::make_pair(v1, v2);
+}
+
+template<typename Tr>
+std::pair<typename Tr::Vertex_handle, typename Tr::Vertex_handle>
+make_vertex_pair(const typename Tr::Edge& e)
+{
+  typedef typename Tr::Vertex_handle Vertex_handle;
+  Vertex_handle v1 = e.first->vertex(e.second);
+  Vertex_handle v2 = e.first->vertex(e.third);
+  return make_vertex_pair(v1, v2);
+}
+
+template<typename Vh>
+CGAL::Triple<Vh, Vh, Vh> make_vertex_triple(const Vh vh0, const Vh vh1, const Vh vh2)
+{
+  CGAL::Triple<Vh, Vh, Vh> ft(vh0, vh1, vh2);
+  if (ft.template get<1>() < ft.template get<0>()) std::swap(ft.template get<0>(), ft.template get<1>());
+  if (ft.template get<2>() < ft.template get<1>()) std::swap(ft.template get<1>(), ft.template get<2>());
+  if (ft.template get<1>() < ft.template get<0>()) std::swap(ft.template get<0>(), ft.template get<1>());
+  return ft;
+}
+
+template<typename Vh>
+std::array<Vh, 3> make_vertex_array(const Vh vh0, const Vh vh1, const Vh vh2)
+{
+  std::array<Vh, 3> ft = { {vh0, vh1, vh2} };
+  if (ft[1] < ft[0]) std::swap(ft[0], ft[1]);
+  if (ft[2] < ft[1]) std::swap(ft[1], ft[2]);
+  if (ft[1] < ft[0]) std::swap(ft[0], ft[1]);
+  return ft;
+}
+
+template<typename Facet>
+Facet canonical_facet(const Facet& f)
+{
+  const typename Facet::first_type c = f.first;
+  const int i = f.second;
+  const typename Facet::first_type c2 = c->neighbor(i);
+  return (c2 < c) ? std::make_pair(c2, c2->index(c)) : std::make_pair(c, i);
+}
+
+template<typename VertexHandle>
+bool is_on_feature(const VertexHandle v)
+{
+  return (v->in_dimension() == 1 || v->in_dimension() == 0);
+}
+
+template<typename Tr>
+bool is_well_oriented(const Tr& tr,
+                      const typename Tr::Vertex_handle v0,
+                      const typename Tr::Vertex_handle v1,
+                      const typename Tr::Vertex_handle v2,
+                      const typename Tr::Vertex_handle v3)
+{
+  return CGAL::POSITIVE == tr.geom_traits().orientation_3_object()(
+           point(v0->point()),
+           point(v1->point()),
+           point(v2->point()),
+           point(v3->point()));
+}
+
+template<typename Tr>
+bool is_well_oriented(const Tr& tr, const typename Tr::Cell_handle ch)
+{
+  return is_well_oriented(tr, ch->vertex(0), ch->vertex(1),
+                          ch->vertex(2), ch->vertex(3));
+}
+
+template<typename C3T3, typename CellSelector>
+bool is_boundary(const C3T3& c3t3,
+                 const typename C3T3::Facet& f,
+                 const CellSelector& cell_selector)
+{
+  return c3t3.is_in_complex(f)
+         || cell_selector(f.first) != cell_selector(f.first->neighbor(f.second));
+}
+
+template<typename C3T3, typename CellSelector>
+bool is_boundary(const C3T3& c3t3,
+                 const typename C3T3::Triangulation::Edge& e,
+                 CellSelector cell_selector)
+{
+  typedef typename C3T3::Triangulation   Tr;
+  typedef typename Tr::Facet_circulator  Facet_circulator;
+  typedef typename Tr::Facet             Facet;
+
+  Facet_circulator fcirc = c3t3.triangulation().incident_facets(e);
+  Facet_circulator fend = fcirc;
+
+  do
+  {
+    const Facet& f = *fcirc;
+    if (is_boundary(c3t3, f, cell_selector))
+      return true;
+  }
+  while (++fcirc != fend);
+
+  return false;
+}
+
+template<typename C3t3, typename CellSelector>
+bool is_boundary_edge(const typename C3t3::Vertex_handle& v0,
+                      const typename C3t3::Vertex_handle& v1,
+                      const C3t3& c3t3,
+                      const CellSelector& cell_selector)
+{
+  typedef typename C3t3::Edge        Edge;
+  typedef typename C3t3::Cell_handle Cell_handle;
+
+  Cell_handle cell;
+  int i0, i1;
+  if (c3t3.triangulation().tds().is_edge(v0, v1, cell, i0, i1))
+    return is_boundary(c3t3, Edge(cell, i0, i1), cell_selector);
+  else
+    return false;
+}
+
+template<typename C3t3, typename CellSelector>
+bool is_boundary_vertex(const typename C3t3::Vertex_handle& v,
+                        const C3t3& c3t3,
+                        CellSelector cell_selector)
+{
+  typedef typename C3t3::Facet Facet;
+  std::vector<Facet> facets;
+  c3t3.triangulation().incident_facets(v, std::back_inserter(facets));
+
+  for(const Facet& f : facets)
+  {
+    if (c3t3.is_in_complex(f))
+      return true;
+    if (cell_selector(f.first) ^ cell_selector(f.first->neighbor(f.second)))
+      return true;
+  }
+  return false;
+}
+
+template<typename C3t3>
+typename C3t3::Surface_patch_index surface_patch_index(const typename C3t3::Vertex_handle v,
+    const C3t3& c3t3)
+{
+  typedef typename C3t3::Surface_patch_index Surface_patch_index;
+  typedef typename C3t3::Facet Facet;
+  std::vector<Facet> facets;
+  c3t3.triangulation().incident_facets(v, std::back_inserter(facets));
+
+  for(const Facet& f : facets)
+  {
+    if (c3t3.is_in_complex(f))
+      return c3t3.surface_patch_index(f);
+  }
+  return Surface_patch_index();
+}
+
+template<typename C3t3>
+void set_index(typename C3t3::Vertex_handle v, const C3t3& c3t3)
+{
+  switch (v->in_dimension())
+  {
+  case 3:
+    v->set_index(v->cell()->subdomain_index());
+    break;
+  case 2:
+    v->set_index(surface_patch_index(v, c3t3));
+    break;
+  case 1:
+    v->set_index(typename C3t3::Curve_index(1));
+    break;
+  case 0:
+    v->set_index(boost::get<typename C3t3::Corner_index>(v->index()));
+    break;
+  default:
+    CGAL_assertion(false);
+  }
+}
+
+template<typename C3t3>
+bool is_edge_in_complex(const typename C3t3::Vertex_handle& v0,
+                        const typename C3t3::Vertex_handle& v1,
+                        const C3t3& c3t3)
+{
+  typedef typename C3t3::Edge        Edge;
+  typedef typename C3t3::Cell_handle Cell_handle;
+
+  Cell_handle cell;
+  int i0, i1;
+  if (c3t3.triangulation().tds().is_edge(v0, v1, cell, i0, i1))
+    return c3t3.is_in_complex(Edge(cell, i0, i1));
+  else
+    return false;
+}
+
+template<typename C3t3, typename OutputIterator>
+OutputIterator incident_subdomains(const typename C3t3::Vertex_handle v,
+                                   const C3t3& c3t3,
+                                   OutputIterator oit)
+{
+  typedef typename C3t3::Triangulation::Cell_handle Cell_handle;
+  std::vector<Cell_handle> cells;
+  c3t3.triangulation().incident_cells(v, std::back_inserter(cells));
+
+  for (std::size_t i = 0; i < cells.size(); ++i)
+    *oit++ = cells[i]->subdomain_index();
+
+  return oit;
+}
+
+template<typename C3t3, typename OutputIterator>
+OutputIterator incident_subdomains(const typename C3t3::Edge& e,
+                                   const C3t3& c3t3,
+                                   OutputIterator oit)
+{
+  typedef typename C3t3::Triangulation::Cell_circulator Cell_circulator;
+
+  Cell_circulator circ = c3t3.triangulation().incident_cells(e);
+  Cell_circulator end = circ;
+  do
+  {
+    *oit++ = circ->subdomain_index();
+  } while (++circ != end);
+
+  return oit;
+}
+
+template<typename C3t3, typename OutputIterator>
+OutputIterator incident_surface_patches(const typename C3t3::Edge& e,
+                                        const C3t3& c3t3,
+                                        OutputIterator oit)
+{
+  typedef typename C3t3::Triangulation::Facet_circulator Facet_circulator;
+  typedef typename C3t3::Triangulation::Facet Facet;
+
+  Facet_circulator circ = c3t3.triangulation().incident_facets(e);
+  Facet_circulator end = circ;
+  do
+  {
+    const Facet& f = *circ;
+    if(c3t3.is_in_complex(f))
+      *oit++ = c3t3.surface_patch_index(f);
+  }
+  while (++circ != end);
+
+  return oit;
+}
+
+template<typename C3t3>
+std::size_t nb_incident_subdomains(const typename C3t3::Vertex_handle v,
+                                   const C3t3& c3t3)
+{
+  typedef typename C3t3::Subdomain_index Subdomain_index;
+
+  boost::unordered_set<Subdomain_index> indices;
+  incident_subdomains(v, c3t3, std::inserter(indices, indices.begin()));
+
+  return indices.size();
+}
+
+template<typename C3t3>
+std::size_t nb_incident_subdomains(const typename C3t3::Edge& e,
+                                   const C3t3& c3t3)
+{
+  typedef typename C3t3::Subdomain_index Subdomain_index;
+
+  boost::unordered_set<Subdomain_index> indices;
+  incident_subdomains(e, c3t3, std::inserter(indices, indices.begin()));
+
+  return indices.size();
+}
+
+template <typename C3t3>
+std::size_t nb_incident_surface_patches(const typename C3t3::Edge& e,
+                                        const C3t3& c3t3)
+{
+  typedef typename C3t3::Surface_patch_index Surface_patch_index;
+
+  boost::unordered_set<Surface_patch_index> indices;
+  incident_surface_patches(e, c3t3, std::inserter(indices, indices.begin()));
+
+  return indices.size();
+}
+
+template<typename C3t3>
+std::size_t nb_incident_complex_edges(const typename C3t3::Vertex_handle v,
+                                      const C3t3& c3t3)
+{
+  typedef typename C3t3::Edge Edge;
+  boost::unordered_set<Edge> edges;
+  c3t3.triangulation().finite_incident_edges(v, std::inserter(edges, edges.begin()));
+
+  std::size_t count = 0;
+  for (const Edge& e : edges)
+  {
+    if (c3t3.is_in_complex(e))
+      ++count;
+  }
+  return count;
+}
+
+
+template<typename C3t3>
+bool is_feature(const typename C3t3::Vertex_handle v,
+                const typename C3t3::Vertex_handle neighbor,
+                const C3t3& c3t3)
+{
+  typename C3t3::Cell_handle ch;
+  int i0, i1;
+  if (c3t3.triangulation().is_edge(v, neighbor, ch, i0, i1))
+  {
+    typename C3t3::Edge edge(ch, i0, i1);
+    return c3t3.is_in_complex(edge);
+  }
+  return false;
+}
+
+template<typename C3t3>
+bool is_feature(const typename C3t3::Vertex_handle v, const C3t3& c3t3)
+{
+  typedef typename C3t3::Edge Edge;
+
+  if (c3t3.number_of_corners() > 0)
+  {
+    return c3t3.is_in_complex(v);
+  }
+  else if (nb_incident_subdomains(v, c3t3) > 3)
+  {
+    std::vector<Edge> edges;
+    c3t3.triangulation().finite_incident_edges(v, std::back_inserter(edges));
+
+    int feature_count = 0;
+    for(const Edge& ei : edges)
+    {
+      if (c3t3.is_in_complex(ei))
+      {
+        feature_count++;
+        if (feature_count >= 3)
+          return true;
+      }
+    }
+  }
+  return false;
+}
+
+/**
+* returns true iff `v` is on the outer hull of c3t3.triangulation()
+* i.e. finite and incident to at least one infinite cell
+*/
+template<typename C3t3>
+bool is_on_convex_hull(const typename C3t3::Vertex_handle v,
+                       const C3t3& c3t3)
+{
+  if (v == c3t3.triangulation().infinite_vertex())
+    return true;
+
+  //on hull == incident to infinite cell
+  typedef typename C3t3::Triangulation::Cell_handle Cell_handle;
+
+  std::vector<Cell_handle> cells;
+  c3t3.triangulation().incident_cells(v, std::back_inserter(cells));
+  for (Cell_handle ci : cells)
+  {
+    if (c3t3.triangulation().is_infinite(ci))
+      return true;
+  }
+  return false;
+}
+
+/**
+* returns true iff `edge` is on the outer hull
+* of c3t3.triangulation()
+* i.e. finite and incident to at least one infinite cell
+*/
+template<typename C3t3>
+bool is_on_convex_hull(const typename C3t3::Edge & edge,
+                       const C3t3& c3t3)
+{
+  typedef typename C3t3::Triangulation::Cell_circulator Cell_circulator;
+  Cell_circulator circ = c3t3.triangulation().incident_cells(edge);
+  Cell_circulator done = circ;
+  do
+  {
+    if (c3t3.triangulation().is_infinite(circ))
+      return true;
+  } while (++circ != done);
+
+  return false;
+}
+
+template<typename C3t3, typename CellSelector>
+bool is_outside(const typename C3t3::Edge & edge,
+                const C3t3& c3t3,
+                CellSelector cell_selector)
+{
+  typedef typename C3t3::Triangulation::Cell_circulator Cell_circulator;
+  Cell_circulator circ = c3t3.triangulation().incident_cells(edge);
+  Cell_circulator done = circ;
+  do
+  {
+    // is cell in complex?
+    if (c3t3.is_in_complex(circ))
+      return false;
+    // does circ belong to the selection?
+    if (cell_selector(circ))
+      return false;
+
+    ++circ;
+  } while (circ != done);
+
+  return true; //all incident cells are outside or infinite
+}
+
+template<typename C3t3, typename CellSelector>
+bool is_selected(const typename C3t3::Vertex_handle v,
+                 const C3t3& c3t3,
+                 CellSelector cell_selector)
+{
+  typedef typename C3t3::Triangulation::Cell_handle Cell_handle;
+
+  std::vector<Cell_handle> cells;
+  c3t3.triangulation().incident_cells(v, std::back_inserter(cells));
+
+  for(Cell_handle c : cells)
+  {
+    if (cell_selector(c))
+      return true;
+  }
+  return false;
+}
+
+template<typename C3t3, typename CellSelector>
+bool is_internal(const typename C3t3::Edge& edge,
+                 const C3t3& c3t3,
+                 CellSelector cell_selector)
+{
+  const typename C3t3::Vertex_handle vs = edge.first->vertex(edge.second);
+  const typename C3t3::Vertex_handle vt = edge.first->vertex(edge.third);
+
+  typedef typename C3t3::Triangulation::Cell_circulator Cell_circulator;
+  Cell_circulator circ = c3t3.triangulation().incident_cells(edge);
+  Cell_circulator done = circ;
+
+  const typename C3t3::Subdomain_index si = circ->subdomain_index();
+  do
+  {
+    if (c3t3.triangulation().is_infinite(circ))
+      return false;
+    if (si != circ->subdomain_index())
+      return false;
+    if (!cell_selector(circ))
+      return false;
+    if (c3t3.is_in_complex(
+          circ,
+          CGAL::Triangulation_utils_3::next_around_edge(circ->index(vs), circ->index(vt))))
+      return false;
+  } while (++circ != done);
+
+  return true;
+}
+
+template<typename Gt>
+void normalize(typename Gt::Vector_3& v, const Gt& gt)
+{
+  typedef typename Gt::FT       FT;
+
+  const FT norm = CGAL::approximate_sqrt(gt.compute_squared_length_3_object()(v));
+  if (norm != FT(0))
+    v = gt.construct_divided_vector_3_object()(v, norm);
+}
+
+template<typename Facet, typename Gt>
+typename Gt::Vector_3 normal(const Facet& f, const Gt& gt)
+{
+  typedef typename Gt::Vector_3 Vector_3;
+  typedef typename Gt::Point_3  Point;
+  typedef typename Gt::FT       FT;
+
+  Point p0 = point(f.first->vertex((f.second + 1) % 4)->point());
+  Point p1 = point(f.first->vertex((f.second + 2) % 4)->point());
+  const Point& p2 = point(f.first->vertex((f.second + 3) % 4)->point());
+
+  if (f.second % 2 == 0)//equivalent to the commented orientation test
+    std::swap(p0, p1);
+
+  Vector_3 n = gt.construct_cross_product_vector_3_object()(
+                 gt.construct_vector_3_object()(p1, p2),
+                 gt.construct_vector_3_object()(p1, p0));
+
+  //cross-product(AB, AC)'s norm is the area of the parallelogram
+  //formed by these 2 vectors.
+  //the triangle's area is half of it
+  return gt.construct_scaled_vector_3_object()(n, FT(1) / FT(2));
+}
+
+template<typename C3t3, typename CellSelector, typename OutputIterator>
+OutputIterator get_internal_edges(const C3t3& c3t3,
+                                  CellSelector cell_selector,
+                                  OutputIterator oit)/*holds Edges*/
+{
+  for (typename C3t3::Triangulation::Finite_edges_iterator
+       eit = c3t3.triangulation().finite_edges_begin();
+       eit != c3t3.triangulation().finite_edges_end();
+       ++eit)
+  {
+    const typename C3t3::Edge& e = *eit;
+    if (is_internal(e, c3t3, cell_selector))
+    {
+      *oit++ = make_vertex_pair<typename C3t3::Triangulation>(e);
+    }
+  }
+  return oit;
+}
+
+template<typename C3t3, typename CellSelector>
+bool topology_test(const typename C3t3::Edge& edge,
+                   const C3t3& c3t3,
+                   const CellSelector& cell_selector)
+{
+  typedef typename C3t3::Vertex_handle Vertex_handle;
+  typedef typename C3t3::Cell_handle   Cell_handle;
+  typedef typename C3t3::Edge          Edge;
+  typedef typename C3t3::Facet         Facet;
+  typedef typename C3t3::Triangulation::Facet_circulator Facet_circulator;
+
+  const Vertex_handle v0 = edge.first->vertex(edge.second);
+  const Vertex_handle v1 = edge.first->vertex(edge.third);
+
+  // the "topology test" checks that :
+  // no incident non-boundary facet has 3 boundary edges
+  // no incident boundary facet has 3 feature edges
+
+  Facet_circulator fcirc = c3t3.triangulation().incident_facets(edge);
+  Facet_circulator fdone = fcirc;
+  do
+  {
+    if (c3t3.triangulation().is_infinite(fcirc->first))
+      continue;
+
+    const Facet& f = *fcirc;
+    if (is_boundary(c3t3, f, cell_selector))
+      //boundary : check that facet does not have 3 feature edges
+    {
+      //Get the ids of the opposite vertices
+      for (int i = 1; i < 4; i++)
+      {
+        Vertex_handle vi = f.first->vertex((f.second + i) % 4);
+        if (vi != v0 && vi != v1 && nb_incident_subdomains(vi, c3t3) > 1)
+        {
+          if (is_edge_in_complex(v0, vi, c3t3)
+              && is_edge_in_complex(v1, vi, c3t3))
+            return false;
+        }
+      }
+    }
+    else //non-boundary : check that facet does not have 3 boundary edges
+    {
+      const Cell_handle circ = f.first;
+      const int i = f.second;
+      if (is_boundary(c3t3, Edge(circ, (i + 1) % 4, (i + 2) % 4), cell_selector)
+          && is_boundary(c3t3, Edge(circ, (i + 2) % 4, (i + 3) % 4), cell_selector)
+          && is_boundary(c3t3, Edge(circ, (i + 3) % 4, (i + 1) % 4), cell_selector))
+        return false;
+    }
+  } while (++fcirc != fdone);
+
+  return true;
+}
+
+template<typename C3t3>
+Subdomain_relation compare_subdomains(const typename C3t3::Vertex_handle v0,
+                                      const typename C3t3::Vertex_handle v1,
+                                      const C3t3& c3t3)
+{
+  typedef typename C3t3::Subdomain_index Subdomain_index;
+  typedef boost::container::flat_set<Subdomain_index,
+    std::less<Subdomain_index>,
+    boost::container::small_vector<Subdomain_index, 30> > Set_of_subdomains;
+
+  Set_of_subdomains subdomains_v0;
+  incident_subdomains(v0, c3t3,
+    std::inserter(subdomains_v0, subdomains_v0.begin()));
+
+  Set_of_subdomains subdomains_v1;
+  incident_subdomains(v1, c3t3,
+    std::inserter(subdomains_v1, subdomains_v1.begin()));
+
+  if (subdomains_v0.size() == subdomains_v1.size())
+  {
+    if(std::equal(subdomains_v0.begin(), subdomains_v0.end(), subdomains_v1.begin()))
+      return EQUAL;
+    else
+      return DIFFERENT;
+  }
+  else
+  {
+    boost::container::small_vector<Subdomain_index, 30>
+      intersection((std::min)(subdomains_v0.size(), subdomains_v1.size()), -1);
+    typename boost::container::small_vector<Subdomain_index, 30>::iterator
+    end_it = std::set_intersection(subdomains_v0.begin(), subdomains_v0.end(),
+                                   subdomains_v1.begin(), subdomains_v1.end(),
+                                   intersection.begin());
+    std::ptrdiff_t intersection_size =
+      std::distance(intersection.begin(), end_it);
+
+    if (subdomains_v0.size() > subdomains_v1.size()
+        && intersection_size == std::ptrdiff_t(subdomains_v1.size()))
+    {
+      return INCLUDES;
+    }
+    else if (intersection_size == std::ptrdiff_t(subdomains_v0.size())) {
+      return INCLUDED;
+    }
+  }
+  return DIFFERENT;
+}
+
+template<typename C3t3, typename CellSelector>
+void get_edge_info(const typename C3t3::Edge& edge,
+                   bool& update_v0,
+                   bool& update_v1,
+                   const C3t3& c3t3,
+                   const CellSelector& cell_selector)
+{
+  typedef typename C3t3::Vertex_handle Vertex_handle;
+
+  update_v0 = false;
+  update_v1 = false;
+
+  const Vertex_handle v0 = edge.first->vertex(edge.second);
+  const Vertex_handle v1 = edge.first->vertex(edge.third);
+
+  const int dim0 = c3t3.in_dimension(v0);
+  const int dim1 = c3t3.in_dimension(v1);
+
+  if (dim0 == 3)
+  {
+    CGAL_assertion(!is_on_convex_hull(v0, c3t3));
+    update_v0 = true;
+    if (dim1 == 3)
+    {
+      CGAL_assertion(!is_on_convex_hull(v1, c3t3));
+      update_v1 = true;
+      return;
+    }
+    else // dim1 is 2, 1, or 0
+      return;
+  }
+  else if (dim1 == 3)
+  {
+    update_v1 = true;
+    return;
+  }
+
+  // from now on, all cases lie on surfaces, or between surfaces
+  CGAL_assertion(dim0 != 3 && dim1 != 3);
+
+  //feature edges and feature vertices
+  if (dim0 < 2 || dim1 < 2)
+  {
+    if (!topology_test(edge, c3t3, cell_selector))
+    {
+#ifdef CGAL_DEBUG_TET_REMESHING_IN_PLUGIN
+      nb_topology_test++;
+#endif
+      return;
+    }
+
+    if (c3t3.is_in_complex(edge))
+    {
+      const std::size_t nb_si_v0 = nb_incident_subdomains(v0, c3t3);
+      const std::size_t nb_si_v1 = nb_incident_subdomains(v1, c3t3);
+
+      if (nb_si_v0 > nb_si_v1) {
+        if (!c3t3.is_in_complex(v1))
+          update_v1 = true;
+      }
+      else if (nb_si_v1 > nb_si_v0) {
+        if (!c3t3.is_in_complex(v0))
+          update_v0 = true;
+      }
+      else {
+        if (!c3t3.is_in_complex(v0))
+          update_v0 = true;
+        if (!c3t3.is_in_complex(v1))
+          update_v1 = true;
+      }
+    }
+    else
+    {
+      if (dim0 == 2 && is_boundary_edge(v0, v1, c3t3, cell_selector))
+      {
+        update_v0 = true;
+        return;
+      }
+      else if(dim1 == 2 && is_boundary_edge(v0, v1, c3t3, cell_selector))
+      {
+        update_v1 = true;
+        return;
+      }
+    }
+    return;
+  }
+
+  if (dim0 == 2 && dim1 == 2)
+  {
+    if (is_boundary(c3t3, edge, cell_selector))
+    {
+      if (!topology_test(edge, c3t3, cell_selector))
+        return;
+      Subdomain_relation subdomain_rel = compare_subdomains(v0, v1, c3t3);
+
+      //Vertices on the same surface
+      if (subdomain_rel == INCLUDES) {
+        update_v1 = true;
+      }
+      else if (subdomain_rel == INCLUDED) {
+        update_v0 = true;
+      }
+      else if (subdomain_rel == EQUAL)
+      {
+        if (c3t3.number_of_edges() == 0)
+        {
+          update_v0 = true;
+          update_v1 = true;
+        }
+        else
+        {
+          const bool v0_on_feature = is_on_feature(v0);
+          const bool v1_on_feature = is_on_feature(v1);
+
+          if (v0_on_feature && v1_on_feature) {
+            if (c3t3.is_in_complex(edge)) {
+              if (!c3t3.is_in_complex(v0))
+                update_v0 = true;
+              if (!c3t3.is_in_complex(v1))
+                update_v1 = true;
+            }
+          }
+          else {
+            if (!v0_on_feature) {
+              update_v0 = true;
+            }
+            if (!v1_on_feature) {
+              update_v1 = true;
+            }
+          }
+        }
+      }
+    }
+  }
+}
+
+namespace debug
+{
+
+// forward-declaration
+template<typename Tr, typename CellRange>
+void dump_cells(const CellRange& cells, const char* filename);
+
+template <typename Bimap>
+void dump_edges(const Bimap& edges, const char* filename)
+{
+  std::ofstream ofs(filename);
+  ofs.precision(17);
+
+  BOOST_FOREACH(typename Bimap::left_const_reference it, edges.left)
+  {
+    ofs << "2 " << point(it.first.first->point())
+        << " " << point(it.first.second->point()) << std::endl;
+  }
+  ofs.close();
+}
+
+template<typename Facet, typename OutputStream>
+void dump_facet(const Facet& f, OutputStream& os)
+{
+  os << "4 ";
+  os << point(f.first->vertex((f.second + 1) % 4)->point()) << " "
+     << point(f.first->vertex((f.second + 2) % 4)->point()) << " "
+     << point(f.first->vertex((f.second + 3) % 4)->point()) << " "
+     << point(f.first->vertex((f.second + 1) % 4)->point());
+  os << std::endl;
+}
+
+template<typename FacetRange>
+void dump_facets(const FacetRange& facets, const char* filename)
+{
+  std::ofstream os(filename);
+  for (typename FacetRange::value_type f : facets)
+  {
+    dump_facet(f, os);
+  }
+  os.close();
+}
+
+template<typename CellRange>
+void dump_polylines(const CellRange& cells, const char* filename)
+{
+  std::ofstream ofs(filename);
+  if (!ofs) return;
+
+  for (typename CellRange::const_iterator it = cells.begin();
+       it != cells.end(); ++it)
+  {
+    for (int i = 0; i < 4; ++i)
+      dump_facet(std::make_pair(*it, i), ofs);
+  }
+  ofs.close();
+}
+
+template<typename Tr>
+bool are_cell_orientations_valid(const Tr& tr)
+{
+  typedef typename Tr::Geom_traits::Point_3 Point_3;
+  typedef typename Tr::Facet                Facet;
+
+  std::set<Facet> facets;
+  for (const typename Tr::Cell_handle ch : tr.finite_cell_handles())
+  {
+    const Point_3& p0 = point(ch->vertex(0)->point());
+    const Point_3& p1 = point(ch->vertex(1)->point());
+    const Point_3& p2 = point(ch->vertex(2)->point());
+    const Point_3& p3 = point(ch->vertex(3)->point());
+
+    const CGAL::Orientation o = CGAL::orientation(p0, p1, p2, p3);
+    if (o != CGAL::POSITIVE)
+    {
+      facets.insert(canonical_facet(Facet(ch, 0)));
+      facets.insert(canonical_facet(Facet(ch, 1)));
+      facets.insert(canonical_facet(Facet(ch, 2)));
+      facets.insert(canonical_facet(Facet(ch, 3)));
+    }
+  }
+  if (!facets.empty())
+  {
+    std::cerr << "Warning : there are inverted cells!\n"
+              << "\tSee cells_with_negative_volume.polylines.txt" << std::endl;
+    dump_facets(facets, "cells_with_negative_volume.polylines.txt");
+  }
+  return facets.empty();
+}
+
+template<typename Tr>
+void dump_surface_off(const Tr& tr, const char* filename)
+{
+  typedef typename Tr::Vertex_handle              Vertex_handle;
+  typedef typename Tr::Cell_handle                Cell_handle;
+  typedef typename Tr::Finite_facets_iterator     Finite_facets_iterator;
+  typedef boost::bimap<Vertex_handle, int>                   Bimap_t;
+  typedef typename Bimap_t::left_map::value_type             value_type;
+
+  //collect vertices
+  Bimap_t vertices;
+  std::size_t nbf = 0;
+  int index = 0;
+  for (Finite_facets_iterator fit = tr.finite_facets_begin();
+       fit != tr.finite_facets_end(); ++fit)
+  {
+    Cell_handle c = fit->first;
+    int i = fit->second;
+    if (tr.is_infinite(c) || tr.is_infinite(c->neighbor(i)))
+    {
+      nbf++;
+      for (int j = 1; j < 4; ++j)
+      {
+        Vertex_handle vij = c->vertex((i + j) % 4);
+        if (vertices.left.find(vij) == vertices.left.end())
+          vertices.left.insert(value_type(vij, index++));
+      }
+    }
+  }
+
+  //write header
+  std::ofstream ofs(filename);
+  ofs.precision(17);
+  ofs << "OFF" << std::endl;
+  ofs << vertices.left.size() << " " << nbf << " 0" << std::endl << std::endl;
+
+  // write vertices
+  for (typename Bimap_t::right_iterator vit = vertices.right.begin();
+       vit != vertices.right.end(); ++vit)
+  {
+    ofs << point(vit->second->point()) << std::endl;
+  }
+
+  //write facets
+  std::size_t nbf_print = 0;
+  for (Finite_facets_iterator fit = tr.finite_facets_begin();
+       fit != tr.finite_facets_end(); ++fit)
+  {
+    Cell_handle c = fit->first;
+    int i = fit->second;
+    if (tr.is_infinite(c) || tr.is_infinite(c->neighbor(i)))
+    {
+      ofs << "3  " << vertices.left.at(c->vertex((i + 1) % 4)) << " "
+          << vertices.left.at(c->vertex((i + 2) % 4)) << " "
+          << vertices.left.at(c->vertex((i + 3) % 4)) << std::endl;
+      ++nbf_print;
+    }
+  }
+  CGAL_assertion(nbf == nbf_print);
+
+  ofs.close();
+}
+
+template<typename CellRange, typename Tr>
+void dump_cells_off(const CellRange& cells, const Tr& /*tr*/, const char* filename)
+{
+  typedef typename Tr::Vertex_handle              Vertex_handle;
+  typedef typename Tr::Cell_handle                Cell_handle;
+  typedef boost::bimap<Vertex_handle, int>        Bimap_t;
+  typedef typename Bimap_t::left_map::value_type  value_type;
+
+  Bimap_t vertices;
+  int index = 0;
+  boost::unordered_set<std::array<Vertex_handle, 3> > facets;
+
+  for (Cell_handle c : cells)
+  {
+    //collect vertices
+    for (int i = 0; i < 4; ++i)
+    {
+      Vertex_handle vi = c->vertex(i);
+      if (vertices.left.find(c->vertex(i)) == vertices.left.end())
+        vertices.left.insert(value_type(vi, index++));
+    }
+    //collect facets
+    for (int i = 0; i < 4; ++i)
+    {
+      //if (tr.is_infinite(c->neighbor(i)))
+      {
+        std::array<Vertex_handle, 3> fi = make_vertex_array(c->vertex((i + 1) % 4),
+          c->vertex((i + 2) % 4),
+          c->vertex((i + 3) % 4));
+          facets.insert(fi);
+      }
+    }
+  }
+
+  //write header
+  std::ofstream ofs(filename);
+  ofs.precision(17);
+  ofs << "OFF" << std::endl;
+  ofs << vertices.size() << " " << facets.size() << " 0" << std::endl << std::endl;
+
+  for(const typename Bimap_t::right_map::value_type& v : vertices.right)
+    ofs << v.second->point().x() << " "
+        << v.second->point().y() << " "
+        << v.second->point().z() << std::endl;
+
+  for(const std::array<Vertex_handle, 3>& f : facets)
+    ofs << "3  " << vertices.left.at(f[0]) << " "
+                 << vertices.left.at(f[1]) << " "
+                 << vertices.left.at(f[2]) << std::endl;
+
+  ofs.close();
+}
+
+template<typename Tr>
+void dump_cells_off(const Tr& tr, const char* filename)
+{
+  typedef typename Tr::Vertex_handle              Vertex_handle;
+  typedef typename Tr::Cell_handle                Cell_handle;
+  typedef typename Tr::Finite_facets_iterator     Finite_facets_iterator;
+  typedef typename Tr::Finite_vertices_iterator   Finite_vertices_iterator;
+  typedef boost::bimap<Vertex_handle, int>                   Bimap_t;
+  typedef typename Bimap_t::left_map::value_type             value_type;
+
+  //write header
+  std::ofstream ofs(filename);
+  ofs.precision(17);
+  ofs << "OFF" << std::endl;
+  ofs << tr.number_of_vertices()
+      << " " << tr.number_of_finite_facets() << " 0" << std::endl << std::endl;
+
+  //collect and write vertices
+  Bimap_t vertices;
+  int index = 0;
+  for (Finite_vertices_iterator vit = tr.finite_vertices_begin();
+       vit != tr.finite_vertices_end(); ++vit)
+  {
+    vertices.left.insert(value_type(vit, index++));
+    ofs << vit->point().x() << " "
+        << vit->point().y() << " "
+        << vit->point().z() << std::endl;
+  }
+
+  //write facets
+  for (Finite_facets_iterator fit = tr.finite_facets_begin();
+       fit != tr.finite_facets_end(); ++fit)
+  {
+    Cell_handle c = fit->first;
+    int i = fit->second;
+    ofs << "3  " << vertices.left.at(c->vertex((i + 1) % 4)) << " "
+        << vertices.left.at(c->vertex((i + 2) % 4)) << " "
+        << vertices.left.at(c->vertex((i + 3) % 4)) << std::endl;
+  }
+  ofs.close();
+}
+
+template<typename CellRange>
+void dump_cells_polylines(const CellRange& cells, const char* filename)
+{
+  std::ofstream ofs(filename);
+  ofs.precision(17);
+  for (auto c : cells)
+  {
+    ofs << "2 " << point(c->vertex(0)->point()) << " "
+                << point(c->vertex(1)->point()) <<std::endl;
+    ofs << "2 " << point(c->vertex(0)->point()) << " "
+                << point(c->vertex(2)->point()) << std::endl;
+    ofs << "2 " << point(c->vertex(0)->point()) << " "
+                << point(c->vertex(3)->point()) << std::endl;
+    ofs << "2 " << point(c->vertex(1)->point()) << " "
+                << point(c->vertex(2)->point()) << std::endl;
+    ofs << "2 " << point(c->vertex(1)->point()) << " "
+                << point(c->vertex(3)->point()) << std::endl;
+    ofs << "2 " << point(c->vertex(2)->point()) << " "
+                << point(c->vertex(3)->point()) << std::endl;
+  }
+  ofs.close();
+}
+
+template<typename Tr, typename CellRange, typename IndexRange>
+void dump_cells(const CellRange& cells,
+                const IndexRange& indices,
+                const char* filename)
+{
+  typedef typename Tr::Vertex_handle                         Vertex_handle;
+  typedef typename Tr::Point                                 Point;
+  typedef boost::bimap<Vertex_handle, int>                   Bimap_t;
+  typedef typename Bimap_t::left_map::value_type             value_type;
+
+  CGAL_assertion(indices.empty() || cells.size() == indices.size());
+
+  //collect vertices
+  Bimap_t vertices;
+  int index = 1;
+  for (typename CellRange::const_iterator cit = cells.begin();
+       cit != cells.end();
+       ++cit)
+  {
+    for (int i = 0; i < 4; ++i)
+    {
+      Vertex_handle vi = (*cit)->vertex(i);
+      if (vertices.left.find(vi) == vertices.left.end())
+        vertices.left.insert(value_type(vi, index++));
+    }
+  }
+
+  //write cells
+  std::ofstream ofs(filename);
+  ofs.precision(17);
+  ofs << "MeshVersionFormatted 1" << std::endl;
+  ofs << "Dimension 3" << std::endl;
+  ofs << "Vertices" << std::endl << vertices.size() << std::endl;
+  for (typename Bimap_t::right_const_iterator vit = vertices.right.begin();
+       vit != vertices.right.end();
+       ++vit)
+  {
+    const Point& p = vit->second->point();
+    ofs << p.x() << " " << p.y() << " " << p.z() << " 2" << std::endl;
+  }
+  ofs << "Tetrahedra " << std::endl << cells.size() << std::endl;
+  typename IndexRange::const_iterator iit = indices.begin();
+  for (typename CellRange::const_iterator cit = cells.begin();
+       cit != cells.end();
+       ++cit)
+  {
+    ofs << vertices.left.at((*cit)->vertex(0))
+        << " " << vertices.left.at((*cit)->vertex(1))
+        << " " << vertices.left.at((*cit)->vertex(2))
+        << " " << vertices.left.at((*cit)->vertex(3));
+
+    if (iit == indices.end())
+      ofs << " 1" << std::endl;
+    else
+    {
+      ofs << " " << (*iit) << std::endl;
+      ++iit;
+    }
+  }
+  ofs << "End" << std::endl;
+  ofs.close();
+}
+
+template<typename Tr, typename CellRange>
+void dump_cells(const CellRange& cells, const char* filename)
+{
+  std::vector<int> indices;
+  dump_cells<Tr>(cells, indices, filename);
+}
+
+template<typename Tr>
+void dump_cells_in_complex(const Tr& tr, const char* filename)
+{
+  std::vector<typename Tr::Cell_handle> cells;
+  std::vector<typename Tr::Cell::Subdomain_index> indices;
+
+  for (typename Tr::Finite_cells_iterator cit = tr.finite_cells_begin();
+       cit != tr.finite_cells_end(); ++cit)
+  {
+    if (cit->subdomain_index() > 0)
+    {
+      cells.push_back(cit);
+      indices.push_back(cit->subdomain_index());
+    }
+  }
+  dump_cells<Tr>(cells, indices, filename);
+}
+
+template<typename C3t3>
+void dump_facets_in_complex(const C3t3& c3t3, const char* filename)
+{
+  typedef typename C3t3::Triangulation              Tr;
+  typedef typename Tr::Vertex_handle                Vertex_handle;
+  typedef typename Tr::Cell_handle                  Cell_handle;
+  typedef typename C3t3::Facets_in_complex_iterator Facets_in_complex_iterator;
+  typedef boost::bimap<Vertex_handle, int>          Bimap_t;
+  typedef typename Bimap_t::left_map::value_type  value_type;
+
+  //collect vertices
+  Bimap_t vertices;
+  std::size_t nbf = 0;
+  int index = 0;
+  for (Facets_in_complex_iterator fit = c3t3.facets_in_complex_begin();
+       fit != c3t3.facets_in_complex_end(); ++fit)
+  {
+    Cell_handle c = fit->first;
+    int i = fit->second;
+
+    nbf++;
+    for (int j = 1; j < 4; ++j)
+    {
+      Vertex_handle vij = c->vertex((i + j) % 4);
+      if (vertices.left.find(vij) == vertices.left.end())
+        vertices.left.insert(value_type(vij, index++));
+    }
+  }
+
+  //write header
+  std::ofstream ofs(filename);
+  ofs.precision(17);
+  ofs << "OFF" << std::endl;
+  ofs << vertices.left.size() << " " << nbf << " 0" << std::endl << std::endl;
+
+  // write vertices
+  for (typename Bimap_t::right_iterator vit = vertices.right.begin();
+       vit != vertices.right.end(); ++vit)
+  {
+    ofs << point(vit->second->point()) << std::endl;
+  }
+
+  //write facets
+  std::size_t nbf_print = 0;
+  for (Facets_in_complex_iterator fit = c3t3.facets_in_complex_begin();
+       fit != c3t3.facets_in_complex_end(); ++fit)
+  {
+    Cell_handle c = fit->first;
+    int i = fit->second;
+    ofs << "3  " << vertices.left.at(c->vertex((i + 1) % 4)) << " "
+        << vertices.left.at(c->vertex((i + 2) % 4)) << " "
+        << vertices.left.at(c->vertex((i + 3) % 4)) << std::endl;
+    ++nbf_print;
+  }
+  CGAL_assertion(nbf == nbf_print);
+
+  ofs.close();
+}
+
+template<typename C3T3>
+void dump_edges_in_complex(const C3T3& c3t3, const char* filename)
+{
+  std::ofstream ofs(filename);
+  ofs.precision(17);
+  for (typename C3T3::Edges_in_complex_iterator eit = c3t3.edges_in_complex_begin();
+       eit != c3t3.edges_in_complex_end(); ++eit)
+  {
+    const typename C3T3::Edge& e = *eit;
+    ofs << "2 "
+        << point(e.first->vertex(e.second)->point()) << " "
+        << point(e.first->vertex(e.third)->point()) << "\n";
+  }
+  ofs.close();
+}
+
+template<typename Tr, typename CellSelector>
+void dump_cells_with_small_dihedral_angle(const Tr& tr,
+    const double angle_bound,
+    CellSelector cell_select,
+    const char* filename)
+{
+  typedef typename Tr::Cell_handle           Cell_handle;
+  typedef typename Tr::Cell::Subdomain_index Subdomain_index;
+  std::vector<Cell_handle>     cells;
+  std::vector<Subdomain_index> indices;
+
+  for (typename Tr::Finite_cells_iterator cit = tr.finite_cells_begin();
+       cit != tr.finite_cells_end(); ++cit)
+  {
+    Cell_handle c = cit;
+    if (c->subdomain_index() != Subdomain_index() && cell_select(c))
+    {
+      double dh = min_dihedral_angle(tr, c);
+      if (dh < angle_bound)
+      {
+        cells.push_back(c);
+        indices.push_back(c->subdomain_index());
+      }
+    }
+  }
+  std::cout << "bad cells : " << cells.size() << std::endl;
+  dump_cells<Tr>(cells, indices, filename);
+  dump_cells_off(cells, tr, "bad_cells.off");
+}
+
+template<typename Tr>
+void dump_vertices_by_dimension(const Tr& tr, const char* prefix)
+{
+  typedef typename Tr::Vertex_handle Vertex_handle;
+  std::vector< std::vector<Vertex_handle> > vertices_per_dimension(4);
+
+  for (typename Tr::Finite_vertices_iterator
+       vit = tr.finite_vertices_begin();
+       vit != tr.finite_vertices_end();
+       ++vit)
+  {
+    if (vit->in_dimension() == -1)
+      continue;//far point
+    CGAL_assertion(vit->in_dimension() >= 0 && vit->in_dimension() < 4);
+
+    vertices_per_dimension[vit->in_dimension()].push_back(vit);
+  }
+
+  for (int i = 0; i < 4; ++i)
+  {
+    //dimension is i
+    const std::vector<Vertex_handle>& vertices_di = vertices_per_dimension[i];
+
+    std::cout << "Dimension " << i << " : " << vertices_di.size() << std::endl;
+
+    std::ostringstream oss;
+    oss << prefix << "_dimension_" << i << ".off";
+
+    std::ofstream ofs(oss.str());
+    ofs.precision(17);
+    ofs << "OFF" << std::endl;
+    ofs << vertices_di.size() << " 0 0" << std::endl << std::endl;
+
+    for (Vertex_handle vj : vertices_di)
+    {
+      ofs << point(vj->point()) << std::endl;
+    }
+
+    ofs.close();
+  }
+}
+
+template<typename Tr>
+void dump_triangulation_cells(const Tr& tr, const char* filename)
+{
+  std::vector<typename Tr::Cell_handle> cells(tr.number_of_finite_cells());
+  std::vector<typename Tr::Cell::Subdomain_index> indices(tr.number_of_finite_cells());
+  int i = 0;
+  for (typename Tr::Finite_cells_iterator cit = tr.finite_cells_begin();
+       cit != tr.finite_cells_end(); ++cit)
+  {
+    cells[i] = cit;
+    indices[i++] = cit->subdomain_index();
+  }
+  dump_cells<Tr>(cells, indices, filename);
+}
+
+template<typename C3t3>
+void dump_binary(const C3t3& c3t3, const char* filename)
+{
+  std::ofstream os(filename, std::ios::binary | std::ios::out);
+  CGAL::Mesh_3::save_binary_file(os, c3t3);
+  os.close();
+}
+
+template<typename C3t3>
+void dump_medit(const C3t3& c3t3, const char* filename)
+{
+  std::ofstream os(filename, std::ios::out);
+  c3t3.output_to_medit(os, true, true);
+  os.close();
+}
+
+template<typename C3t3>
+void dump_c3t3(const C3t3& c3t3, const char* filename_no_extension)
+{
+  std::string filename_medit(filename_no_extension);
+  filename_medit.append(".mesh");
+  dump_medit(c3t3, filename_medit.c_str());
+
+  std::string filename_binary(filename_no_extension);
+  filename_binary.append(".binary.cgal");
+  dump_binary(c3t3, filename_binary.c_str());
+}
+
+
+} //namespace debug
+} //namespace Tetrahedral_remeshing
+} //namespace CGAL
+
+#endif //CGAL_INTERNAL_TET_REMESHING_HELPERS_H
diff -pruN 5.0.3-1/include/CGAL/Tetrahedral_remeshing/Remeshing_cell_base_3.h 5.1-1/include/CGAL/Tetrahedral_remeshing/Remeshing_cell_base_3.h
--- 5.0.3-1/include/CGAL/Tetrahedral_remeshing/Remeshing_cell_base_3.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Tetrahedral_remeshing/Remeshing_cell_base_3.h	2020-09-08 12:15:48.000000000 +0000
@@ -0,0 +1,65 @@
+// Copyright (c) 2020 GeometryFactory (France) and Telecom Paris (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Tetrahedral_remeshing/include/CGAL/Tetrahedral_remeshing/Remeshing_cell_base_3.h $
+// $Id: Remeshing_cell_base_3.h ab05dde 2020-06-12T08:08:56+02:00 Jane Tournois
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Jane Tournois, Noura Faraj, Jean-Marc Thiery, Tamy Boubekeur
+
+#ifndef CGAL_TET_ADAPTIVE_REMESHING_CELL_BASE_3_H
+#define CGAL_TET_ADAPTIVE_REMESHING_CELL_BASE_3_H
+
+#include <CGAL/license/Tetrahedral_remeshing.h>
+
+#include <CGAL/Mesh_cell_base_3.h>
+
+namespace CGAL
+{
+namespace Tetrahedral_remeshing
+{
+namespace internal
+{
+struct Fake_MD_C
+{
+  typedef int Subdomain_index;
+  typedef int Surface_patch_index;
+  typedef int Index;
+};
+}
+
+/*!
+\ingroup PkgTetrahedralRemeshingClasses
+
+The class `Remeshing_cell_base_3` is a model of the concept `MeshCellBase_3`.
+It is designed to serve as cell base class for the 3D triangulation
+used in the tetrahedral remeshing process.
+
+\tparam Gt is the geometric traits class.
+It has to be a model of the concept `RemeshingTriangulationTraits_3`.
+
+\tparam Cb is a cell base class from which `Remeshing_cell_base_3` derives.
+It must be a model of the `TriangulationCellBase_3` concept.
+It has the default value `Triangulation_cell_base_3<Gt>`.
+
+\cgalModels `MeshCellBase_3`
+
+*/
+#ifndef DOXYGEN_RUNNING
+template<typename Gt,
+         typename Cb = CGAL::Triangulation_cell_base_3<Gt> >
+using Remeshing_cell_base_3
+  = CGAL::Mesh_cell_base_3<Gt, internal::Fake_MD_C, Cb>;
+#else
+template<typename Gt,
+         typename Cb = CGAL::Triangulation_cell_base_3<Gt> >
+class Remeshing_cell_base_3;
+#endif
+
+}//end namespace Tetrahedral_remeshing
+}//end namespace CGAL
+
+#endif //CGAL_TET_ADAPTIVE_REMESHING_CELL_BASE_3_H
diff -pruN 5.0.3-1/include/CGAL/Tetrahedral_remeshing/Remeshing_triangulation_3.h 5.1-1/include/CGAL/Tetrahedral_remeshing/Remeshing_triangulation_3.h
--- 5.0.3-1/include/CGAL/Tetrahedral_remeshing/Remeshing_triangulation_3.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Tetrahedral_remeshing/Remeshing_triangulation_3.h	2020-09-08 12:15:48.000000000 +0000
@@ -0,0 +1,77 @@
+// Copyright (c) 2020 GeometryFactory (France) and Telecom Paris (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Tetrahedral_remeshing/include/CGAL/Tetrahedral_remeshing/Remeshing_triangulation_3.h $
+// $Id: Remeshing_triangulation_3.h ab05dde 2020-06-12T08:08:56+02:00 Jane Tournois
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Jane Tournois, Noura Faraj, Jean-Marc Thiery, Tamy Boubekeur
+
+#ifndef CGAL_TETRAHEDRAL_REMESHING_TRIANGULATION_H
+#define CGAL_TETRAHEDRAL_REMESHING_TRIANGULATION_H
+
+#include <CGAL/license/Tetrahedral_remeshing.h>
+
+#include <CGAL/Triangulation_data_structure_3.h>
+#include <CGAL/Triangulation_3.h>
+
+#include <CGAL/Tetrahedral_remeshing/Remeshing_cell_base_3.h>
+#include <CGAL/Tetrahedral_remeshing/Remeshing_vertex_base_3.h>
+
+#include <CGAL/Kernel_traits.h>
+#include <CGAL/Cartesian_converter.h>
+#include <CGAL/tags.h>
+
+#include <boost/static_assert.hpp>
+#include <boost/type_traits/is_same.hpp>
+
+namespace CGAL
+{
+namespace Tetrahedral_remeshing
+{
+/*!
+\ingroup PkgTetrahedralRemeshingClasses
+
+The class `Remeshing_triangulation_3`
+is a class template which provides a valid triangulation type
+that can be used as the 3D triangulation input for
+the tetrahedral remeshing process.
+
+\tparam Gt is the geometric traits class.
+It has to be a model of the concept `RemeshingTriangulationTraits_3`.
+
+\tparam Concurrency_tag enables sequential versus parallel implementation of the
+triangulation data structure.
+Possible values are `Sequential_tag` (the default), `Parallel_tag`,
+and `Parallel_if_available_tag`.
+
+\tparam Vb is a model of `RemeshingVertexBase_3`. It has the default value ` Remeshing_vertex_base_3<Gt>`.
+\tparam Cb is a model of `RemeshingCellBase_3`. It has the default value ` Remeshing_cell_base_3<Gt>`.
+
+*/
+template<typename Gt,
+         typename Concurrency_tag = CGAL::Sequential_tag,
+         typename Vb = Remeshing_vertex_base_3<Gt>,
+         typename Cb = Remeshing_cell_base_3<Gt>
+>
+class Remeshing_triangulation_3
+  : public CGAL::Triangulation_3<Gt, CGAL::Triangulation_data_structure_3<Vb,Cb> >
+{
+public:
+  typedef Vb Remeshing_Vb;
+  typedef Cb Remeshing_Cb;
+
+  typedef CGAL::Triangulation_data_structure_3<
+            Remeshing_Vb, Remeshing_Cb, Concurrency_tag>  Tds;
+  typedef CGAL::Triangulation_3<Gt, Tds>                  Base;
+
+  using Base::Base;
+};
+
+}//end namespace Tetrahedral_remeshing
+}//end namespace CGAL
+
+#endif // CGAL_TETRAHEDRAL_REMESHING_TRIANGULATION_H
diff -pruN 5.0.3-1/include/CGAL/Tetrahedral_remeshing/Remeshing_vertex_base_3.h 5.1-1/include/CGAL/Tetrahedral_remeshing/Remeshing_vertex_base_3.h
--- 5.0.3-1/include/CGAL/Tetrahedral_remeshing/Remeshing_vertex_base_3.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Tetrahedral_remeshing/Remeshing_vertex_base_3.h	2020-09-08 12:15:48.000000000 +0000
@@ -0,0 +1,67 @@
+// Copyright (c) 2020 GeometryFactory (France) and Telecom Paris (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Tetrahedral_remeshing/include/CGAL/Tetrahedral_remeshing/Remeshing_vertex_base_3.h $
+// $Id: Remeshing_vertex_base_3.h ab05dde 2020-06-12T08:08:56+02:00 Jane Tournois
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Jane Tournois, Noura Faraj, Jean-Marc Thiery, Tamy Boubekeur
+
+#ifndef CGAL_TET_ADAPTIVE_REMESHING_VERTEX_BASE_3_H
+#define CGAL_TET_ADAPTIVE_REMESHING_VERTEX_BASE_3_H
+
+#include <CGAL/license/Tetrahedral_remeshing.h>
+
+#include <CGAL/Mesh_vertex_base_3.h>
+
+namespace CGAL
+{
+namespace Tetrahedral_remeshing
+{
+namespace internal
+{
+
+struct Fake_MD_V
+{
+  typedef int Subdomain_index;
+  typedef int Surface_patch_index;
+  typedef int Index;
+};
+
+} // internal
+
+/*!
+\ingroup PkgTetrahedralRemeshingClasses
+
+The class `Remeshing_vertex_base_3` is a model of the concept `MeshVertexBase_3`.
+It is designed to serve as vertex base class for the 3D triangulation
+used in the tetrahedral remeshing process.
+
+\tparam Gt is the geometric traits class.
+It has to be a model of the concept `RemeshingTriangulationTraits_3`.
+
+\tparam Vb is a vertex base class from which `Remeshing_vertex_base_3` derives.
+It must be a model of the `TriangulationVertexBase_3` concept.
+It has the default value `Triangulation_vertex_base_3<Gt>`.
+
+\cgalModels `MeshVertexBase_3`
+*/
+#ifndef DOXYGEN_RUNNING
+template<typename GT,
+         typename Vb = CGAL::Triangulation_vertex_base_3<GT> >
+using Remeshing_vertex_base_3
+  = CGAL::Mesh_vertex_base_3<GT, internal::Fake_MD_V, Vb>;
+#else
+template<typename GT,
+         typename Vb = CGAL::Triangulation_vertex_base_3<GT> >
+class Remeshing_vertex_base_3;
+#endif
+
+}//end namespace Tetrahedral_remeshing
+
+}//end namespace CGAL
+
+#endif //CGAL_TET_ADAPTIVE_REMESHING_VERTEX_BASE_3_H
diff -pruN 5.0.3-1/include/CGAL/Tetrahedral_remeshing/Sizing_field.h 5.1-1/include/CGAL/Tetrahedral_remeshing/Sizing_field.h
--- 5.0.3-1/include/CGAL/Tetrahedral_remeshing/Sizing_field.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Tetrahedral_remeshing/Sizing_field.h	2020-09-08 12:15:48.000000000 +0000
@@ -0,0 +1,37 @@
+// Copyright (c) 2020 GeometryFactory (France) and Telecom Paris (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Tetrahedral_remeshing/include/CGAL/Tetrahedral_remeshing/Sizing_field.h $
+// $Id: Sizing_field.h ab05dde 2020-06-12T08:08:56+02:00 Jane Tournois
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Jane Tournois, Noura Faraj, Jean-Marc Thiery, Tamy Boubekeur
+
+#ifndef CGAL_SIZING_FIELD_H
+#define CGAL_SIZING_FIELD_H
+
+#include <CGAL/license/Tetrahedral_remeshing.h>
+
+namespace CGAL
+{
+/*!
+* Sizing field virtual class
+*/
+template <class Kernel>
+class Sizing_field
+{
+public:
+  typedef Kernel                      K;
+  typedef typename Kernel::FT         FT;
+  typedef typename Kernel::Point_3    Point_3;
+
+public:
+  virtual FT operator()(const Point_3& p) const = 0;
+};
+
+}//end namespace CGAL
+
+#endif //CGAL_SIZING_FIELD_H
diff -pruN 5.0.3-1/include/CGAL/Tetrahedral_remeshing/tetrahedral_remeshing_io.h 5.1-1/include/CGAL/Tetrahedral_remeshing/tetrahedral_remeshing_io.h
--- 5.0.3-1/include/CGAL/Tetrahedral_remeshing/tetrahedral_remeshing_io.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Tetrahedral_remeshing/tetrahedral_remeshing_io.h	2020-09-08 12:15:49.000000000 +0000
@@ -0,0 +1,55 @@
+// Copyright (c) 2020 GeometryFactory (France) and Telecom Paris (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Tetrahedral_remeshing/include/CGAL/Tetrahedral_remeshing/tetrahedral_remeshing_io.h $
+// $Id: tetrahedral_remeshing_io.h ab05dde 2020-06-12T08:08:56+02:00 Jane Tournois
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Jane Tournois, Noura Faraj, Jean-Marc Thiery, Tamy Boubekeur
+
+#include <CGAL/IO/io.h>
+
+#include <iostream>
+#include <fstream>
+
+namespace CGAL
+{
+template<typename T3>
+bool load_triangulation(std::istream& is, T3& t3)
+{
+  std::string s;
+  if (!(is >> s)) return false;
+  bool binary = (s == "binary");
+  if (binary) {
+    if (!(is >> s)) return false;
+  }
+  if (s != "CGAL" || !(is >> s) || s != "c3t3")
+    return false;
+
+  std::getline(is, s);
+  if (binary) CGAL::set_binary_mode(is);
+  else        CGAL::set_ascii_mode(is);
+  is >> t3;
+  return bool(is);
+}
+
+template<typename T3>
+bool save_binary_triangulation(std::ostream& os, const T3& t3)
+{
+  os << "binary CGAL c3t3\n";
+  CGAL::set_binary_mode(os);
+  return !!(os << t3);
+}
+
+template<typename T3>
+bool save_ascii_triangulation(std::ostream& os, const T3& t3)
+{
+  os << "CGAL c3t3\n";
+  CGAL::set_ascii_mode(os);
+  return !!(os << t3);
+}
+
+}
\ No newline at end of file
diff -pruN 5.0.3-1/include/CGAL/Tetrahedral_remeshing/Uniform_sizing_field.h 5.1-1/include/CGAL/Tetrahedral_remeshing/Uniform_sizing_field.h
--- 5.0.3-1/include/CGAL/Tetrahedral_remeshing/Uniform_sizing_field.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Tetrahedral_remeshing/Uniform_sizing_field.h	2020-09-08 12:15:49.000000000 +0000
@@ -0,0 +1,45 @@
+// Copyright (c) 2020 GeometryFactory (France) and Telecom Paris (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Tetrahedral_remeshing/include/CGAL/Tetrahedral_remeshing/Uniform_sizing_field.h $
+// $Id: Uniform_sizing_field.h ab05dde 2020-06-12T08:08:56+02:00 Jane Tournois
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Jane Tournois, Noura Faraj, Jean-Marc Thiery, Tamy Boubekeur
+
+#ifndef CGAL_UNIFORM_SIZING_FIELD_H
+#define CGAL_UNIFORM_SIZING_FIELD_H
+
+#include <CGAL/license/Tetrahedral_remeshing.h>
+
+#include <CGAL/Tetrahedral_remeshing/Sizing_field.h>
+
+namespace CGAL
+{
+template <class Kernel>
+class Uniform_sizing_field : Sizing_field<Kernel>
+{
+private:
+  typedef Sizing_field<Kernel>        Base;
+public:
+  typedef typename Base::FT         FT;
+  typedef typename Base::Point_3    Point_3;
+
+  Uniform_sizing_field(const FT& size)
+    : m_size(size)
+  {}
+
+  FT operator()(const Point_3&) const
+  {
+    return m_size;
+  }
+private:
+  FT m_size;
+};
+
+}//end namespace CGAL
+
+#endif //CGAL_UNIFORM_SIZING_FIELD_H
diff -pruN 5.0.3-1/include/CGAL/tetrahedral_remeshing.h 5.1-1/include/CGAL/tetrahedral_remeshing.h
--- 5.0.3-1/include/CGAL/tetrahedral_remeshing.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/tetrahedral_remeshing.h	2020-09-08 12:15:49.000000000 +0000
@@ -0,0 +1,497 @@
+// Copyright (c) 2020 GeometryFactory (France) and Telecom Paris (France).
+// All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Tetrahedral_remeshing/include/CGAL/tetrahedral_remeshing.h $
+// $Id: tetrahedral_remeshing.h 962b681 2020-07-31T15:52:56+02:00 Laurent Rineau
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Jane Tournois, Noura Faraj, Jean-Marc Thiery, Tamy Boubekeur
+
+#ifndef TETRAHEDRAL_REMESHING_H
+#define TETRAHEDRAL_REMESHING_H
+
+#include <CGAL/license/Tetrahedral_remeshing.h>
+
+#include <CGAL/Triangulation_3.h>
+#include <CGAL/Mesh_complex_3_in_triangulation_3.h>
+
+#include <CGAL/Tetrahedral_remeshing/Sizing_field.h>
+#include <CGAL/Tetrahedral_remeshing/Uniform_sizing_field.h>
+#include <CGAL/Tetrahedral_remeshing/internal/tetrahedral_adaptive_remeshing_impl.h>
+#include <CGAL/Tetrahedral_remeshing/internal/compute_c3t3_statistics.h>
+
+#include <CGAL/boost/graph/Named_function_parameters.h>
+#include <CGAL/boost/graph/named_params_helper.h>
+
+#ifdef CGAL_DUMP_REMESHING_STEPS
+#include <sstream>
+#endif
+
+namespace CGAL
+{
+///////////////////////////////////////////////////
+///////////////// TRIANGULATION_3 /////////////////
+///////////////////////////////////////////////////
+/*!
+* \ingroup PkgTetrahedralRemeshingRef
+* remeshes a tetrahedral mesh.
+*
+* It is recommended to use `CGAL::Tetrahedral_remeshing::Remeshing_triangulation_3`
+* for the first parameter, as it inherits from `Triangulation_3` with a `TDS` suitable for this function.
+*
+* This function takes as input a 3-dimensional triangulation
+* and performs a sequence of atomic operations
+* in order to generate as output a high quality mesh with a prescribed
+* uniform density.
+* These atomic operations are performed as follows:
+*   - edge splits, until all edges satisfy a prescribed length criterion,
+*   - edge collapses, until all edges satisfy a prescribed length criterion,
+*   - edge flips, to locally improve dihedral angles, until they can
+*     no longer be improved by flipping,
+*   - global smoothing by vertex relocations,
+*   - re-projection of boundary vertices to the initial surface.
+*
+* This remeshing function can deal with multi-domains, boundaries, and features.
+* It preserves the geometry of
+* subdomains throughout the remeshing process.
+*
+* Subdomains are defined by indices that
+* are stored in the cells of the input triangulation, following the `MeshCellBase_3`
+* concept (refined by `RemeshingCellBase_3`).
+* The surfacic interfaces between subdomains are formed by facets whose two incident cells
+* have different subdomain indices.
+* The edges where three or more subdomains meet form feature polylines,
+* and are considered as constrained edges.
+*
+*
+* @tparam Traits is the geometric traits, model of `RemeshingTriangulationTraits_3`
+* @tparam TDS is the triangulation data structure for `Triangulation_3`,
+*             model of ` TriangulationDataStructure_3`,
+*             with cell base model of `RemeshingCellBase_3`
+*             and vertex base model of `RemeshingVertexBase_3`.
+* @tparam SLDS is an optional parameter for `Triangulation_3`, that
+*             specifies the type of the spatial lock data structure.
+* @tparam NamedParameters a sequence of \ref bgl_namedparameters "Named Parameters"
+*
+* @param tr the triangulation to be remeshed, of type `Triangulation_3<Traits, TDS, SLDS>`.
+*           `Remeshing_triangulation` is a helper class that satisfies all the requirements
+*           of its template parameters.
+* @param target_edge_length the uniform target edge length. This parameter provides a
+*          mesh density target for the remeshing algorithm.
+* @param np optional sequence of \ref bgl_namedparameters "Named Parameters"
+*          among the ones listed below
+*
+* \cgalNamedParamsBegin
+*   \cgalParamNBegin{number_of_iterations}
+*     \cgalParamDescription{the number of iterations for the full sequence of atomic operations
+*                           performed (listed in the above description)}
+*     \cgalParamType{unsigned int}
+*     \cgalParamDefault{`1`}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{remesh_boundaries}
+*     \cgalParamDescription{If `false`, none of the input volume boundaries can be modified.
+*                           Otherwise, the topology is preserved, but atomic operations
+*                           can be performed on the surfaces, and along feature polylines,
+*                           such that boundaries are remeshed.}
+*     \cgalParamType{`bool`}
+*     \cgalParamDefault{`true`}
+*     \cgalParamExtra{Boundaries are between the exterior and the interior,
+*                     between two subdomains, between the areas selected or not for remeshing
+*                     (cf `Remeshing_cell_is_selected_map`), or defined
+*                     by `Remeshing_edge_is_constrained_map` and `Remeshing_facet_is_constrained_map`.}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{edge_is_constrained_map}
+*     \cgalParamDescription{a property map containing the constrained-or-not status of each edge of `tr`.}
+*     \cgalParamType{a class model of `ReadWritePropertyMap` with `std::pair<Triangulation_3::Vertex_handle, Triangulation_3::Vertex_handle>`
+*                    as key type and `bool` as value type. It must be default constructible.}
+*     \cgalParamDefault{a default property map where no edge is constrained}
+*     \cgalParamExtra{A constrained edge can be split or collapsed, but not flipped.}
+*     \cgalParamExtra{The pairs must be ordered to ensure consistency.}
+*     \cgalParamExtra{During the meshing process, the set of constrained edges evolves consistently
+*                     with edge splits and collapses, so the property map must be writable.}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{facet_is_constrained_map}
+*     \cgalParamDescription{a property map containing the constrained-or-not status of each facet of `tr`.}
+*     \cgalParamType{a class model of `ReadablePropertyMap` with `Triangulation_3::Facet`
+*                    as key type and `bool` as value type. It must be default constructible.}
+*     \cgalParamDefault{a default property map where no facet is constrained}
+*     \cgalParamExtra{A constrained facet can be split or collapsed, but not flipped.}
+*     \cgalParamExtra{This map, contrary to the others, is not updated throughout the remeshing process.}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{cell_is_selected_map}
+*     \cgalParamDescription{a property map containing the selected - or - not status for each cell of `tr` for remeshing.
+*                           Only selected cells are modified (and possibly their neighbors if surfaces are modified) by remeshing.}
+*     \cgalParamType{a class model of `ReadWritePropertyMap` with `Triangulation_3::Cell_handle`
+*                    as key type and `bool` as value type. It must be default constructible.}
+*     \cgalParamDefault{a default property map where all cells of the domain
+*                       (i.e. with a non-zero `Subdomain_index`) are selected.}
+*     \cgalParamExtra{During the meshing process, the set of selected cells evolves consistently with
+*                     the atomic operations that are performed, so the property map must be writable.}
+*   \cgalParamNEnd
+*
+*   \cgalParamNBegin{smooth_constrained_edges}
+*     \cgalParamDescription{If `true`, the end vertices of the edges set as
+*                           constrained in `edge_is_constrained_map` move along the
+*                           constrained polylines they belong to.}
+*     \cgalParamType{`bool`}
+*     \cgalParamDefault{`false`}
+*     \cgalParamExtra{The endvertices of constraints listed
+*                     by `edge_is_constrained_map`, and edges incident to at least three subdomains
+*                     are made eligible to one dimensional smoothing, along the constrained polylines they belong to.
+*                     Corners (i.e. vertices incident to more than 2 constrained edges) are not allowed
+*                     to move at all.\n
+*                     Note that activating the smoothing step on polyline constraints tends to reduce
+*                     the quality of the minimal dihedral angle in the mesh.\n
+*                     If `remesh_boundaries` is set to `false`, this parameter is ignored.}
+*   \cgalParamNEnd
+*
+*  \cgalNamedParamsEnd
+*
+* \sa `CGAL::Tetrahedral_remeshing::Remeshing_triangulation_3`
+*
+* @todo implement non-uniform sizing field instead of uniform target edge length
+*/
+template<typename Traits, typename TDS, typename SLDS,
+         typename NamedParameters>
+void tetrahedral_isotropic_remeshing(
+  CGAL::Triangulation_3<Traits, TDS, SLDS>& tr,
+  const double& target_edge_length,
+  const NamedParameters& np)
+{
+  typedef CGAL::Triangulation_3<Traits, TDS, SLDS> Triangulation;
+  tetrahedral_isotropic_remeshing(
+    tr,
+    [target_edge_length](const typename Triangulation::Point& /* p */)
+                        {return target_edge_length;},
+    np);
+}
+
+template<typename Traits, typename TDS, typename SLDS,
+         typename NamedParameters>
+void tetrahedral_isotropic_remeshing(
+  CGAL::Triangulation_3<Traits, TDS, SLDS>& tr,
+  const float& target_edge_length,
+  const NamedParameters& np)
+{
+  typedef CGAL::Triangulation_3<Traits, TDS, SLDS> Triangulation;
+  tetrahedral_isotropic_remeshing(
+    tr,
+    [target_edge_length](const typename Triangulation::Point& /* p */)
+                        {return target_edge_length; },
+    np);
+}
+
+template<typename Traits, typename TDS, typename SLDS,
+         typename SizingFunction,
+         typename NamedParameters>
+void tetrahedral_isotropic_remeshing(
+  CGAL::Triangulation_3<Traits, TDS, SLDS>& tr,
+  const SizingFunction& sizing,
+  const NamedParameters& np)
+{
+  CGAL_assertion(tr.is_valid(true));
+
+  typedef CGAL::Triangulation_3<Traits, TDS, SLDS> Tr;
+
+  using parameters::choose_parameter;
+  using parameters::get_parameter;
+
+  bool remesh_surfaces = choose_parameter(get_parameter(np, internal_np::remesh_boundaries),
+                                          true);
+  bool protect = !remesh_surfaces;
+  // bool adaptive = choose_parameter(get_parameter(np, internal_np::adaptive_size),
+  //                              false);
+  std::size_t max_it = choose_parameter(get_parameter(np, internal_np::number_of_iterations),
+                                        1);
+  bool smooth_constrained_edges
+    = choose_parameter(get_parameter(np, internal_np::smooth_constrained_edges),
+                       false);
+
+  typedef typename internal_np::Lookup_named_param_def <
+    internal_np::cell_selector_t,
+    NamedParameters,
+    Tetrahedral_remeshing::internal::All_cells_selected<Tr>//default
+  > ::type SelectionFunctor;
+  SelectionFunctor cell_select
+    = choose_parameter(get_parameter(np, internal_np::cell_selector),
+                       Tetrahedral_remeshing::internal::All_cells_selected<Tr>());
+
+  typedef std::pair<typename Tr::Vertex_handle, typename Tr::Vertex_handle> Edge_vv;
+  typedef Tetrahedral_remeshing::internal::No_constraint_pmap<Edge_vv> No_edge;
+  typedef typename internal_np::Lookup_named_param_def <
+    internal_np::edge_is_constrained_t,
+    NamedParameters,
+    No_edge//default
+  > ::type ECMap;
+  ECMap ecmap = choose_parameter(get_parameter(np, internal_np::edge_is_constrained),
+                                 No_edge());
+
+  typedef typename Tr::Facet Facet;
+  typedef Tetrahedral_remeshing::internal::No_constraint_pmap<Facet> No_facet;
+  typedef typename internal_np::Lookup_named_param_def <
+    internal_np::facet_is_constrained_t,
+    NamedParameters,
+    No_facet//default
+  > ::type FCMap;
+  FCMap fcmap = choose_parameter(get_parameter(np, internal_np::facet_is_constrained),
+                                 No_facet());
+
+  typedef typename internal_np::Lookup_named_param_def <
+    internal_np::remeshing_visitor_t,
+    NamedParameters,
+    Tetrahedral_remeshing::internal::Default_remeshing_visitor
+  > ::type Visitor;
+  Visitor visitor
+    = choose_parameter(get_parameter(np, internal_np::remeshing_visitor),
+                       Tetrahedral_remeshing::internal::Default_remeshing_visitor());
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_VERBOSE
+  std::cout << "Tetrahedral remeshing ("
+            << "nb_iter = " << max_it << ", "
+            << "protect = " << std::boolalpha << protect
+            << ")" << std::endl;
+
+  std::cout << "Init tetrahedral remeshing...";
+  std::cout.flush();
+#endif
+
+  typedef Tetrahedral_remeshing::internal::Adaptive_remesher<
+    Tr, SizingFunction, ECMap, FCMap, SelectionFunctor, Visitor> Remesher;
+  Remesher remesher(tr, sizing, protect
+                  , ecmap, fcmap
+                  , smooth_constrained_edges
+                  , cell_select
+                  , visitor);
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_VERBOSE
+  std::cout << "done." << std::endl;
+  Tetrahedral_remeshing::internal::compute_statistics(
+    remesher.tr(), cell_select, "statistics_begin.txt");
+#endif
+
+  // perform remeshing
+  std::size_t nb_extra_iterations = 3;
+  remesher.remesh(max_it, nb_extra_iterations);
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_VERBOSE
+  const double angle_bound = 5.0;
+  Tetrahedral_remeshing::debug::dump_cells_with_small_dihedral_angle(tr,
+    angle_bound, cell_select, "bad_cells.mesh");
+  Tetrahedral_remeshing::internal::compute_statistics(tr,
+    cell_select, "statistics_end.txt");
+#endif
+}
+
+template<typename Traits, typename TDS, typename SLDS>
+void tetrahedral_isotropic_remeshing(
+  CGAL::Triangulation_3<Traits, TDS, SLDS>& tr,
+  const double& target_edge_length)
+{
+  tetrahedral_isotropic_remeshing(tr, target_edge_length,
+                                 CGAL::parameters::all_default());
+}
+
+/*!
+* \ingroup PkgTetrahedralRemeshingRef
+* converts the triangulation contained in the input to a `Triangulation_3`.
+*
+* This function should be used to generate a valid triangulation
+* for tetrahedral remeshing, when the input triangulation is generated with the
+* tetrahedral mesh generation package.
+*
+* @tparam Tr is the underlying triangulation for `Mesh_complex_3_in_triangulation_3`.
+*            It can be instantiated with any 3D regular triangulation of CGAL provided
+*            that its vertex and cell base classes are models of the concepts
+*            `MeshVertexBase_3` (refined by `RemeshingCellBase_3`)
+*            and `MeshCellBase_3` (refined by `RemeshingVertexBase_3`), respectively.
+* @tparam CornerIndex is the type of the indices for feature corners.
+*            If `c3t3` has been generated using `CGAL::make_mesh_3()`, it must match
+*            the `Corner_index` type of the model of the `MeshDomainWithFeatures_3` concept used for mesh generation.
+* @tparam CurveIndex is the type of the indices for feature curves.
+*            If `c3t3` has been generated using `CGAL::make_mesh_3()`, it must match
+*            the `Curve_index` type of the model of the `MeshDomainWithFeatures_3` concept used for mesh generation.
+*
+* @param c3t3 the complex containing the triangulation to be remeshed.
+*/
+
+template<typename Tr,
+         typename CornerIndex,
+         typename CurveIndex>
+CGAL::Triangulation_3<typename Tr::Geom_traits,
+                      typename Tr::Triangulation_data_structure>
+convert_to_triangulation_3(
+  CGAL::Mesh_complex_3_in_triangulation_3<Tr, CornerIndex, CurveIndex> c3t3)
+{
+  using GT   = typename Tr::Geom_traits;
+  using TDS  = typename Tr::Triangulation_data_structure;
+
+  CGAL::Triangulation_3<GT, TDS> tr;
+  tr.swap(c3t3.triangulation());
+  return tr;
+}
+
+///////////////////////////////////////////////////
+/////// MESH_COMPLEX_3_IN_TRIANGULATION_3 /////////
+///////////////////////////////////////////////////
+
+///////
+////// Warning with using this version :
+////// the triangulation after remeshing is not regular anymore
+////// the empty-sphere property is not respected
+///////
+template<typename Tr,
+         typename CornerIndex, typename CurveIndex,
+         typename NamedParameters>
+void tetrahedral_isotropic_remeshing(
+  CGAL::Mesh_complex_3_in_triangulation_3<Tr, CornerIndex, CurveIndex>& c3t3,
+  const double& target_edge_length,
+  const NamedParameters& np)
+{
+  tetrahedral_isotropic_remeshing(
+    c3t3,
+    [target_edge_length](const typename Tr::Point& /* p */)
+                        {return target_edge_length; },
+    np);
+}
+
+template<typename Tr,
+         typename CornerIndex, typename CurveIndex,
+         typename NamedParameters>
+void tetrahedral_isotropic_remeshing(
+  CGAL::Mesh_complex_3_in_triangulation_3<Tr, CornerIndex, CurveIndex>& c3t3,
+  const float& target_edge_length,
+  const NamedParameters& np)
+{
+  tetrahedral_isotropic_remeshing(
+    c3t3,
+    [target_edge_length](const typename Tr::Point& /*p*/)
+                        {return target_edge_length; },
+    np);
+}
+
+template<typename Tr,
+         typename CornerIndex, typename CurveIndex>
+void tetrahedral_isotropic_remeshing(
+  CGAL::Mesh_complex_3_in_triangulation_3<Tr, CornerIndex, CurveIndex>& c3t3,
+  const double& target_edge_length)
+{
+  return tetrahedral_isotropic_remeshing(c3t3, target_edge_length,
+                                        CGAL::parameters::all_default());
+}
+
+template<typename Tr,
+         typename CornerIndex, typename CurveIndex,
+         typename SizingFunction,
+         typename NamedParameters>
+void tetrahedral_isotropic_remeshing(
+  CGAL::Mesh_complex_3_in_triangulation_3<Tr, CornerIndex, CurveIndex>& c3t3,
+  const SizingFunction& sizing,
+  const NamedParameters& np)
+{
+  CGAL_assertion(c3t3.triangulation().tds().is_valid(true));
+
+  using parameters::get_parameter;
+  using parameters::choose_parameter;
+
+  bool remesh_surfaces = choose_parameter(get_parameter(np, internal_np::remesh_boundaries),
+                                          true);
+  bool protect = !remesh_surfaces;
+  std::size_t max_it = choose_parameter(get_parameter(np, internal_np::number_of_iterations), 1);
+
+  bool smooth_constrained_edges
+    = choose_parameter(get_parameter(np, internal_np::smooth_constrained_edges),
+      false);
+
+  typedef typename internal_np::Lookup_named_param_def <
+  internal_np::cell_selector_t,
+              NamedParameters,
+              Tetrahedral_remeshing::internal::All_cells_selected<Tr>//default
+              > ::type SelectionFunctor;
+  SelectionFunctor cell_select
+    = choose_parameter(get_parameter(np, internal_np::cell_selector),
+                       Tetrahedral_remeshing::internal::All_cells_selected<Tr>());
+
+  typedef std::pair<typename Tr::Vertex_handle, typename Tr::Vertex_handle> Edge_vv;
+  typedef Tetrahedral_remeshing::internal::No_constraint_pmap<Edge_vv> No_edge;
+  typedef typename internal_np::Lookup_named_param_def <
+  internal_np::edge_is_constrained_t,
+              NamedParameters,
+              No_edge//default
+              > ::type ECMap;
+  ECMap ecmap = choose_parameter(get_parameter(np, internal_np::edge_is_constrained),
+                                 No_edge());
+
+  typedef typename Tr::Facet Facet;
+  typedef Tetrahedral_remeshing::internal::No_constraint_pmap<Facet> No_facet;
+  typedef typename internal_np::Lookup_named_param_def <
+  internal_np::facet_is_constrained_t,
+              NamedParameters,
+              No_facet//default
+              > ::type FCMap;
+  FCMap fcmap = choose_parameter(get_parameter(np, internal_np::facet_is_constrained),
+                                 No_facet());
+
+  typedef typename internal_np::Lookup_named_param_def <
+  internal_np::remeshing_visitor_t,
+              NamedParameters,
+              Tetrahedral_remeshing::internal::Default_remeshing_visitor
+              > ::type Visitor;
+  Visitor visitor
+    = choose_parameter(get_parameter(np, internal_np::remeshing_visitor),
+                       Tetrahedral_remeshing::internal::Default_remeshing_visitor());
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_VERBOSE
+  std::cout << "Tetrahedral remeshing ("
+            << "nb_iter = " << max_it << ", "
+            << "protect = " << std::boolalpha << protect
+            << ")" << std::endl;
+
+  std::cout << "Init tetrahedral remeshing...";
+  std::cout.flush();
+#endif
+
+  typedef Tetrahedral_remeshing::internal::Adaptive_remesher<
+  Tr, SizingFunction, ECMap, FCMap, SelectionFunctor,
+  Visitor,
+  CornerIndex, CurveIndex
+  > Remesher;
+  Remesher remesher(c3t3, sizing, protect
+                    , ecmap, fcmap
+                    , smooth_constrained_edges
+                    , cell_select
+                    , visitor);
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_VERBOSE
+  std::cout << "done." << std::endl;
+  Tetrahedral_remeshing::internal::compute_statistics(
+    remesher.tr(),
+    cell_select, "statistics_begin.txt");
+#endif
+
+  // perform remeshing
+  std::size_t nb_extra_iterations = 3;
+  remesher.remesh(max_it, nb_extra_iterations);
+
+#ifdef CGAL_TETRAHEDRAL_REMESHING_VERBOSE
+  const double angle_bound = 5.0;
+  Tetrahedral_remeshing::debug::dump_cells_with_small_dihedral_angle(
+    c3t3.triangulation(),
+    angle_bound, cell_select, "bad_cells.mesh");
+#endif
+#ifdef CGAL_TETRAHEDRAL_REMESHING_VERBOSE
+  Tetrahedral_remeshing::internal::compute_statistics(
+    c3t3.triangulation(),
+    cell_select, "statistics_end.txt");
+#endif
+}
+
+}//end namespace CGAL
+
+#endif //TETRAHEDRAL_REMESHING_H
diff -pruN 5.0.3-1/include/CGAL/Tetrahedron_3.h 5.1-1/include/CGAL/Tetrahedron_3.h
--- 5.0.3-1/include/CGAL/Tetrahedron_3.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/Tetrahedron_3.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Tetrahedron_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Tetrahedron_3.h $
 // $Id: Tetrahedron_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/thread.h 5.1-1/include/CGAL/thread.h
--- 5.0.3-1/include/CGAL/thread.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/thread.h	2020-09-08 12:15:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/thread.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/thread.h $
 // $Id: thread.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Three/Buffer_objects.h 5.1-1/include/CGAL/Three/Buffer_objects.h
--- 5.0.3-1/include/CGAL/Three/Buffer_objects.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Three/Buffer_objects.h	2020-09-08 12:15:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Three/include/CGAL/Three/Buffer_objects.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Three/include/CGAL/Three/Buffer_objects.h $
 // $Id: Buffer_objects.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Three/Edge_container.h 5.1-1/include/CGAL/Three/Edge_container.h
--- 5.0.3-1/include/CGAL/Three/Edge_container.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Three/Edge_container.h	2020-09-08 12:15:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Three/include/CGAL/Three/Edge_container.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Three/include/CGAL/Three/Edge_container.h $
 // $Id: Edge_container.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Three/exceptions.h 5.1-1/include/CGAL/Three/exceptions.h
--- 5.0.3-1/include/CGAL/Three/exceptions.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Three/exceptions.h	2020-09-08 12:15:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Three/include/CGAL/Three/exceptions.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Three/include/CGAL/Three/exceptions.h $
 // $Id: exceptions.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Three/Point_container.h 5.1-1/include/CGAL/Three/Point_container.h
--- 5.0.3-1/include/CGAL/Three/Point_container.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Three/Point_container.h	2020-09-08 12:15:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Three/include/CGAL/Three/Point_container.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Three/include/CGAL/Three/Point_container.h $
 // $Id: Point_container.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Three/Polyhedron_demo_io_plugin_interface.h 5.1-1/include/CGAL/Three/Polyhedron_demo_io_plugin_interface.h
--- 5.0.3-1/include/CGAL/Three/Polyhedron_demo_io_plugin_interface.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Three/Polyhedron_demo_io_plugin_interface.h	2020-09-08 12:15:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Three/include/CGAL/Three/Polyhedron_demo_io_plugin_interface.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Three/include/CGAL/Three/Polyhedron_demo_io_plugin_interface.h $
 // $Id: Polyhedron_demo_io_plugin_interface.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Three/Polyhedron_demo_plugin_helper.h 5.1-1/include/CGAL/Three/Polyhedron_demo_plugin_helper.h
--- 5.0.3-1/include/CGAL/Three/Polyhedron_demo_plugin_helper.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Three/Polyhedron_demo_plugin_helper.h	2020-09-08 12:15:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Three/include/CGAL/Three/Polyhedron_demo_plugin_helper.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Three/include/CGAL/Three/Polyhedron_demo_plugin_helper.h $
 // $Id: Polyhedron_demo_plugin_helper.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Three/Polyhedron_demo_plugin_interface.h 5.1-1/include/CGAL/Three/Polyhedron_demo_plugin_interface.h
--- 5.0.3-1/include/CGAL/Three/Polyhedron_demo_plugin_interface.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Three/Polyhedron_demo_plugin_interface.h	2020-09-08 12:15:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Three/include/CGAL/Three/Polyhedron_demo_plugin_interface.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Three/include/CGAL/Three/Polyhedron_demo_plugin_interface.h $
 // $Id: Polyhedron_demo_plugin_interface.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Three/Primitive_container.h 5.1-1/include/CGAL/Three/Primitive_container.h
--- 5.0.3-1/include/CGAL/Three/Primitive_container.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Three/Primitive_container.h	2020-09-08 12:15:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Three/include/CGAL/Three/Primitive_container.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Three/include/CGAL/Three/Primitive_container.h $
 // $Id: Primitive_container.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Three/Scene_draw_interface.h 5.1-1/include/CGAL/Three/Scene_draw_interface.h
--- 5.0.3-1/include/CGAL/Three/Scene_draw_interface.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Three/Scene_draw_interface.h	2020-09-08 12:15:49.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Three/include/CGAL/Three/Scene_draw_interface.h $
-// $Id: Scene_draw_interface.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Three/include/CGAL/Three/Scene_draw_interface.h $
+// $Id: Scene_draw_interface.h 755c044 2020-07-24T12:02:11+02:00 Maxime Gimeno
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -16,6 +16,7 @@
 #include <CGAL/license/Three.h>
 
 #include <QPoint>
+#include <QVector3D>
 class QKeyEvent;
 class QPoint;
 namespace CGAL
diff -pruN 5.0.3-1/include/CGAL/Three/Scene_group_item.h 5.1-1/include/CGAL/Three/Scene_group_item.h
--- 5.0.3-1/include/CGAL/Three/Scene_group_item.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Three/Scene_group_item.h	2020-09-08 12:15:49.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Three/include/CGAL/Three/Scene_group_item.h $
-// $Id: Scene_group_item.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Three/include/CGAL/Three/Scene_group_item.h $
+// $Id: Scene_group_item.h 9551708 2020-06-03T15:02:58+02:00 Maxime Gimeno
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -245,8 +245,12 @@ public Q_SLOTS:
     {
       for(int i = 0; i < children.size(); ++i)
       {
-        if(children[i] >= removed_id)
+        if(children[i] > removed_id)
           --children[i];
+        else if(children[i] == removed_id)//child has been removed from the scene, it doesn't exist anymore.
+        {
+          children.removeAll(removed_id);
+        }
       }
     }
 private:
diff -pruN 5.0.3-1/include/CGAL/Three/Scene_interface.h 5.1-1/include/CGAL/Three/Scene_interface.h
--- 5.0.3-1/include/CGAL/Three/Scene_interface.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Three/Scene_interface.h	2020-09-08 12:15:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Three/include/CGAL/Three/Scene_interface.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Three/include/CGAL/Three/Scene_interface.h $
 // $Id: Scene_interface.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Three/Scene_item_config.h 5.1-1/include/CGAL/Three/Scene_item_config.h
--- 5.0.3-1/include/CGAL/Three/Scene_item_config.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Three/Scene_item_config.h	2020-09-08 12:15:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Three/include/CGAL/Three/Scene_item_config.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Three/include/CGAL/Three/Scene_item_config.h $
 // $Id: Scene_item_config.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Three/Scene_item.h 5.1-1/include/CGAL/Three/Scene_item.h
--- 5.0.3-1/include/CGAL/Three/Scene_item.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Three/Scene_item.h	2020-09-08 12:15:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Three/include/CGAL/Three/Scene_item.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Three/include/CGAL/Three/Scene_item.h $
 // $Id: Scene_item.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Three/Scene_item_rendering_helper.h 5.1-1/include/CGAL/Three/Scene_item_rendering_helper.h
--- 5.0.3-1/include/CGAL/Three/Scene_item_rendering_helper.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Three/Scene_item_rendering_helper.h	2020-09-08 12:15:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Three/include/CGAL/Three/Scene_item_rendering_helper.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Three/include/CGAL/Three/Scene_item_rendering_helper.h $
 // $Id: Scene_item_rendering_helper.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Three/Scene_item_with_properties.h 5.1-1/include/CGAL/Three/Scene_item_with_properties.h
--- 5.0.3-1/include/CGAL/Three/Scene_item_with_properties.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Three/Scene_item_with_properties.h	2020-09-08 12:15:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Three/include/CGAL/Three/Scene_item_with_properties.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Three/include/CGAL/Three/Scene_item_with_properties.h $
 // $Id: Scene_item_with_properties.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Three/Scene_print_item_interface.h 5.1-1/include/CGAL/Three/Scene_print_item_interface.h
--- 5.0.3-1/include/CGAL/Three/Scene_print_item_interface.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Three/Scene_print_item_interface.h	2020-09-08 12:15:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Three/include/CGAL/Three/Scene_print_item_interface.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Three/include/CGAL/Three/Scene_print_item_interface.h $
 // $Id: Scene_print_item_interface.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Three/Scene_transparent_interface.h 5.1-1/include/CGAL/Three/Scene_transparent_interface.h
--- 5.0.3-1/include/CGAL/Three/Scene_transparent_interface.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Three/Scene_transparent_interface.h	2020-09-08 12:15:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Three/include/CGAL/Three/Scene_transparent_interface.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Three/include/CGAL/Three/Scene_transparent_interface.h $
 // $Id: Scene_transparent_interface.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Three/Scene_zoomable_item_interface.h 5.1-1/include/CGAL/Three/Scene_zoomable_item_interface.h
--- 5.0.3-1/include/CGAL/Three/Scene_zoomable_item_interface.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Three/Scene_zoomable_item_interface.h	2020-09-08 12:15:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Three/include/CGAL/Three/Scene_zoomable_item_interface.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Three/include/CGAL/Three/Scene_zoomable_item_interface.h $
 // $Id: Scene_zoomable_item_interface.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Three/TextRenderer.h 5.1-1/include/CGAL/Three/TextRenderer.h
--- 5.0.3-1/include/CGAL/Three/TextRenderer.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Three/TextRenderer.h	2020-09-08 12:15:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Three/include/CGAL/Three/TextRenderer.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Three/include/CGAL/Three/TextRenderer.h $
 // $Id: TextRenderer.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Three/Three.h 5.1-1/include/CGAL/Three/Three.h
--- 5.0.3-1/include/CGAL/Three/Three.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Three/Three.h	2020-09-08 12:15:49.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Three/include/CGAL/Three/Three.h $
-// $Id: Three.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Three/include/CGAL/Three/Three.h $
+// $Id: Three.h b4bba7f 2020-06-15T08:58:13+02:00 Maxime Gimeno
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -29,8 +29,15 @@
 #  define THREE_EXPORT Q_DECL_IMPORT
 #endif
 
+#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
+#define CGAL_QT_SKIP_EMPTY_PARTS QString::SkipEmptyParts
+#else
+#define CGAL_QT_SKIP_EMPTY_PARTS ::Qt::SkipEmptyParts
+#endif
+
 namespace CGAL{
 namespace Three{
+//define enum depending on Qt version
 class Polyhedron_demo_plugin_interface;
 class THREE_EXPORT Three{
 public:
@@ -73,15 +80,31 @@ public:
    * in the plugin.
    */
   static void autoConnectActions(CGAL::Three::Polyhedron_demo_plugin_interface* plugin);
+  /*!
+   * Displays in the console a blue text preceded by the mention
+   * "INFO: ".
+   */
   static void information(QString);
   /*!
-   * Displays a blue text preceded by the mention "WARNING :".
+   * Displays in the console an orange text preceded by the mention "WARNING: ".
    */
   static void warning(QString);
   /*!
-   * Displays a red text preceded by the mention "ERROR :".
+   * Displays in the console a red text preceded by the mention "ERROR: ".
    */
   static void error(QString);
+  /*!
+   * Displays an information popup.
+   */
+  static void information(QString title, QString message);
+  /*!
+   * Displays a warning popup.
+   */
+  static void warning(QString title, QString message);
+  /*!
+   * Displays an error popup.
+   */
+  static void error(QString title, QString message);
 protected:
   static QMainWindow* s_mainwindow;
   static Viewer_interface* s_mainviewer;
diff -pruN 5.0.3-1/include/CGAL/Three/Triangle_container.h 5.1-1/include/CGAL/Three/Triangle_container.h
--- 5.0.3-1/include/CGAL/Three/Triangle_container.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Three/Triangle_container.h	2020-09-08 12:15:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Three/include/CGAL/Three/Triangle_container.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Three/include/CGAL/Three/Triangle_container.h $
 // $Id: Triangle_container.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Three/Viewer_config.h 5.1-1/include/CGAL/Three/Viewer_config.h
--- 5.0.3-1/include/CGAL/Three/Viewer_config.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Three/Viewer_config.h	2020-09-08 12:15:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Three/include/CGAL/Three/Viewer_config.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Three/include/CGAL/Three/Viewer_config.h $
 // $Id: Viewer_config.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Three/Viewer_interface.h 5.1-1/include/CGAL/Three/Viewer_interface.h
--- 5.0.3-1/include/CGAL/Three/Viewer_interface.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Three/Viewer_interface.h	2020-09-08 12:15:49.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Three/include/CGAL/Three/Viewer_interface.h $
-// $Id: Viewer_interface.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Three/include/CGAL/Three/Viewer_interface.h $
+// $Id: Viewer_interface.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -238,6 +238,9 @@ public Q_SLOTS:
   //! If b is true, faces will be ligted from both internal and external side.
   //! If b is false, only the side that is exposed to the light source will be lighted.
   virtual void setTwoSides(bool b) = 0;
+  //! If b is true, then a special color mask is applied to points and meshes to differenciate
+  //! front-faced and back-faced elements.
+  virtual void setBackFrontShading(bool b) =0;
   //! \brief Sets the fast drawing mode
   //! @see inFastDrawing()
   virtual void setFastDrawing(bool b) = 0;
diff -pruN 5.0.3-1/include/CGAL/Threetuple.h 5.1-1/include/CGAL/Threetuple.h
--- 5.0.3-1/include/CGAL/Threetuple.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Threetuple.h	2020-09-08 12:15:32.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Threetuple.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Threetuple.h $
 // $Id: Threetuple.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Timer.h 5.1-1/include/CGAL/Timer.h
--- 5.0.3-1/include/CGAL/Timer.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Timer.h	2020-09-08 12:15:30.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Profiling_tools/include/CGAL/Timer.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Profiling_tools/include/CGAL/Timer.h $
 // $Id: Timer.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Timer_impl.h 5.1-1/include/CGAL/Timer_impl.h
--- 5.0.3-1/include/CGAL/Timer_impl.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Timer_impl.h	2020-09-08 12:15:30.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Profiling_tools/include/CGAL/Timer_impl.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Profiling_tools/include/CGAL/Timer_impl.h $
 // $Id: Timer_impl.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Time_stamper.h 5.1-1/include/CGAL/Time_stamper.h
--- 5.0.3-1/include/CGAL/Time_stamper.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Time_stamper.h	2020-09-08 12:15:32.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Time_stamper.h $
-// $Id: Time_stamper.h c08c632 2019-12-10T13:02:24+01:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Time_stamper.h $
+// $Id: Time_stamper.h e82ea5d 2020-01-29T17:13:11+01:00 Laurent Rineau
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Jane Tournois
@@ -28,26 +28,12 @@ constexpr size_t rounded_down_log2(size_
 template <typename T>
 struct Time_stamper
 {
-  Time_stamper()
-   : time_stamp_() {}
-
-  Time_stamper(const Time_stamper& ts)
-    : time_stamp_()
-  {
-    time_stamp_ = std::size_t(ts.time_stamp_);
-  }
-
-  Time_stamper& operator=(const Time_stamper& ts)
-  {
-    time_stamp_ = std::size_t(ts.time_stamp_);
-    return *this;
-  }
-
   static void initialize_time_stamp(T* pt) {
     pt->set_time_stamp(std::size_t(-1));
   }
 
-  void set_time_stamp(T* pt) {
+  template <typename time_stamp_t>
+  static void set_time_stamp(T* pt, time_stamp_t& time_stamp_) {
     if(pt->time_stamp() == std::size_t(-1)) {
       const std::size_t new_ts = time_stamp_++;
       pt->set_time_stamp(new_ts);
@@ -96,23 +82,14 @@ struct Time_stamper
       return time_stamp(p_t1) < time_stamp(p_t2);
     }
   }
-
-  void reset() {
-    time_stamp_ = 0;
-  }
-private:
-#ifdef CGAL_NO_ATOMIC
-  std::size_t time_stamp_;
-#else
-  CGAL::cpp11::atomic<std::size_t> time_stamp_;
-#endif
 }; // end class template Time_stamper<T>
 
 template <typename T>
 struct No_time_stamp
 {
 public:
-  void set_time_stamp(T*)  {}
+  template <typename time_stamp_t>
+  static void set_time_stamp(T*, time_stamp_t&)  {}
   static bool less(const T* p_t1,const T* p_t2) {
     return p_t1 < p_t2;
   }
@@ -130,8 +107,6 @@ public:
     constexpr std::size_t shift = internal::rounded_down_log2(sizeof(T));
     return reinterpret_cast<std::size_t>(p) >> shift;
   }
-
-  void reset()                {}
 }; // end class template No_time_stamp<T>
 
 // That class template is an auxiliary class.  It has a
@@ -159,6 +134,17 @@ struct Get_time_stamper<T,false>{
 template <class T>
 struct Time_stamper_impl : public Get_time_stamper<T>::type {};
 
+struct Hash_handles_with_or_without_timestamps
+{
+  template <typename Handle>
+  std::size_t operator()(const Handle h) const
+  {
+    typedef typename std::iterator_traits<Handle>::value_type Type;
+
+    return Get_time_stamper<Type>::type::hash_value(&*h);
+  }
+};
+
 } //end of namespace CGAL
 
 #endif // CGAL_TIME_STAMPER_H
diff -pruN 5.0.3-1/include/CGAL/Tools/chained_map.h 5.1-1/include/CGAL/Tools/chained_map.h
--- 5.0.3-1/include/CGAL/Tools/chained_map.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Tools/chained_map.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Hash_map/include/CGAL/Tools/chained_map.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Hash_map/include/CGAL/Tools/chained_map.h $
 // $Id: chained_map.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/to_rational.h 5.1-1/include/CGAL/to_rational.h
--- 5.0.3-1/include/CGAL/to_rational.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/to_rational.h	2020-09-08 12:15:17.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/to_rational.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/to_rational.h $
 // $Id: to_rational.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/transforming_iterator.h 5.1-1/include/CGAL/transforming_iterator.h
--- 5.0.3-1/include/CGAL/transforming_iterator.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/transforming_iterator.h	2020-09-08 12:15:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/transforming_iterator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/transforming_iterator.h $
 // $Id: transforming_iterator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/transforming_pair_iterator.h 5.1-1/include/CGAL/transforming_pair_iterator.h
--- 5.0.3-1/include/CGAL/transforming_pair_iterator.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/transforming_pair_iterator.h	2020-09-08 12:15:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/transforming_pair_iterator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/transforming_pair_iterator.h $
 // $Id: transforming_pair_iterator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Transform_iterator.h 5.1-1/include/CGAL/Transform_iterator.h
--- 5.0.3-1/include/CGAL/Transform_iterator.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Transform_iterator.h	2020-09-08 12:15:06.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Matrix_search/include/CGAL/Transform_iterator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Matrix_search/include/CGAL/Transform_iterator.h $
 // $Id: Transform_iterator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Tree_assertions.h 5.1-1/include/CGAL/Tree_assertions.h
--- 5.0.3-1/include/CGAL/Tree_assertions.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Tree_assertions.h	2020-09-08 12:15:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/SearchStructures/include/CGAL/Tree_assertions.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/SearchStructures/include/CGAL/Tree_assertions.h $
 // $Id: Tree_assertions.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Tree_base.h 5.1-1/include/CGAL/Tree_base.h
--- 5.0.3-1/include/CGAL/Tree_base.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Tree_base.h	2020-09-08 12:15:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/SearchStructures/include/CGAL/Tree_base.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/SearchStructures/include/CGAL/Tree_base.h $
 // $Id: Tree_base.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Tree_traits.h 5.1-1/include/CGAL/Tree_traits.h
--- 5.0.3-1/include/CGAL/Tree_traits.h	2020-08-01 19:02:48.000000000 +0000
+++ 5.1-1/include/CGAL/Tree_traits.h	2020-09-08 12:15:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/SearchStructures/include/CGAL/Tree_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/SearchStructures/include/CGAL/Tree_traits.h $
 // $Id: Tree_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangle_2.h 5.1-1/include/CGAL/Triangle_2.h
--- 5.0.3-1/include/CGAL/Triangle_2.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/Triangle_2.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Triangle_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Triangle_2.h $
 // $Id: Triangle_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangle_3.h 5.1-1/include/CGAL/Triangle_3.h
--- 5.0.3-1/include/CGAL/Triangle_3.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/Triangle_3.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Triangle_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Triangle_3.h $
 // $Id: Triangle_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangle_accessor_3.h 5.1-1/include/CGAL/Triangle_accessor_3.h
--- 5.0.3-1/include/CGAL/Triangle_accessor_3.h	2020-08-01 19:02:31.000000000 +0000
+++ 5.1-1/include/CGAL/Triangle_accessor_3.h	2020-09-08 12:15:09.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Triangle_accessor_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_3/include/CGAL/Triangle_accessor_3.h $
 // $Id: Triangle_accessor_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangular_expansion_visibility_2.h 5.1-1/include/CGAL/Triangular_expansion_visibility_2.h
--- 5.0.3-1/include/CGAL/Triangular_expansion_visibility_2.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Triangular_expansion_visibility_2.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Visibility_2/include/CGAL/Triangular_expansion_visibility_2.h $
-// $Id: Triangular_expansion_visibility_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Visibility_2/include/CGAL/Triangular_expansion_visibility_2.h $
+// $Id: Triangular_expansion_visibility_2.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -66,13 +66,13 @@ public:
   typedef CGAL::Tag_true                      Supports_simple_polygon_category;
 
 private:
-  typedef CGAL::Triangulation_vertex_base_2<K>                     Vb;
-  typedef CGAL::Constrained_triangulation_face_base_2<K>           Fb;
-  typedef CGAL::Triangulation_data_structure_2<Vb,Fb>              TDS;
-  typedef CGAL::No_intersection_tag                                Itag;
-  typedef CGAL::Constrained_Delaunay_triangulation_2<K, TDS, Itag> CDT;
+  typedef CGAL::Triangulation_vertex_base_2<K>                          Vb;
+  typedef CGAL::Constrained_triangulation_face_base_2<K>                Fb;
+  typedef CGAL::Triangulation_data_structure_2<Vb, Fb>                  TDS;
+  typedef CGAL::No_constraint_intersection_requiring_constructions_tag  Itag;
+  typedef CGAL::Constrained_Delaunay_triangulation_2<K, TDS, Itag>      CDT;
 
-  typedef std::pair<Point_2,Point_2>                               Constraint;
+  typedef std::pair<Point_2, Point_2>                                   Constraint;
 
   // Functor to create edge constraints for the CDT out of Halfedges
   struct Make_constraint
diff -pruN 5.0.3-1/include/CGAL/Triangular_field_2.h 5.1-1/include/CGAL/Triangular_field_2.h
--- 5.0.3-1/include/CGAL/Triangular_field_2.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Triangular_field_2.h	2020-09-08 12:15:41.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Stream_lines_2/include/CGAL/Triangular_field_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Stream_lines_2/include/CGAL/Triangular_field_2.h $
 // $Id: Triangular_field_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulated_mixed_complex_observer_3.h 5.1-1/include/CGAL/Triangulated_mixed_complex_observer_3.h
--- 5.0.3-1/include/CGAL/Triangulated_mixed_complex_observer_3.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulated_mixed_complex_observer_3.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Skin_surface_3/include/CGAL/Triangulated_mixed_complex_observer_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Skin_surface_3/include/CGAL/Triangulated_mixed_complex_observer_3.h $
 // $Id: Triangulated_mixed_complex_observer_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/triangulate_mixed_complex_3.h 5.1-1/include/CGAL/triangulate_mixed_complex_3.h
--- 5.0.3-1/include/CGAL/triangulate_mixed_complex_3.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/triangulate_mixed_complex_3.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Skin_surface_3/include/CGAL/triangulate_mixed_complex_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Skin_surface_3/include/CGAL/triangulate_mixed_complex_3.h $
 // $Id: triangulate_mixed_complex_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/triangulate_power_diagram_3.h 5.1-1/include/CGAL/triangulate_power_diagram_3.h
--- 5.0.3-1/include/CGAL/triangulate_power_diagram_3.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/triangulate_power_diagram_3.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Skin_surface_3/include/CGAL/triangulate_power_diagram_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Skin_surface_3/include/CGAL/triangulate_power_diagram_3.h $
 // $Id: triangulate_power_diagram_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulation_2/insert_constraints.h 5.1-1/include/CGAL/Triangulation_2/insert_constraints.h
--- 5.0.3-1/include/CGAL/Triangulation_2/insert_constraints.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_2/insert_constraints.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/Triangulation_2/insert_constraints.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/Triangulation_2/insert_constraints.h $
 // $Id: insert_constraints.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulation_2/internal/Constraint_hierarchy_2.h 5.1-1/include/CGAL/Triangulation_2/internal/Constraint_hierarchy_2.h
--- 5.0.3-1/include/CGAL/Triangulation_2/internal/Constraint_hierarchy_2.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_2/internal/Constraint_hierarchy_2.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/Triangulation_2/internal/Constraint_hierarchy_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/Triangulation_2/internal/Constraint_hierarchy_2.h $
 // $Id: Constraint_hierarchy_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulation_2/internal/CTP2_subconstraint_graph.h 5.1-1/include/CGAL/Triangulation_2/internal/CTP2_subconstraint_graph.h
--- 5.0.3-1/include/CGAL/Triangulation_2/internal/CTP2_subconstraint_graph.h	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_2/internal/CTP2_subconstraint_graph.h	2020-09-08 12:15:50.000000000 +0000
@@ -0,0 +1,126 @@
+// Copyright (c) 2020  GeometryFactory (France).  All rights reserved.
+//
+// This file is part of CGAL (www.cgal.org)
+//
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/Triangulation_2/internal/CTP2_subconstraint_graph.h $
+// $Id: CTP2_subconstraint_graph.h f841429 2020-03-26T19:27:08+01:00 Sébastien Loriot
+// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
+//
+//
+// Author(s)     : Simon Giraudot
+
+#ifndef CGAL_CTP2_SUBCONSTRAINT_GRAPH_H
+#define CGAL_CTP2_SUBCONSTRAINT_GRAPH_H
+
+#include <CGAL/license/Triangulation_2.h>
+
+#include <CGAL/boost/graph/internal/graph_traits_2D_triangulation_helper.h>
+#include <CGAL/boost/graph/split_graph_into_polylines.h>
+
+#include <CGAL/property_map.h>
+
+namespace CGAL
+{
+
+namespace internal
+{
+
+template <typename CTP2>
+class CTP2_subconstraint_graph
+{
+  CTP2& ctp2;
+public:
+
+  typedef typename CTP2::Vertex_handle vertex_descriptor;
+  typedef typename CTP2::Subconstraint edge_descriptor;
+  typedef boost::undirected_tag directed_category;
+  typedef boost::disallow_parallel_edge_tag edge_parallel_category;
+  struct CTP2_graph_traversal_category :
+    public virtual boost::bidirectional_graph_tag,
+    public virtual boost::adjacency_graph_tag,
+    public virtual boost::edge_list_graph_tag,
+    public virtual boost::vertex_list_graph_tag
+  { };
+  typedef CTP2_graph_traversal_category traversal_category;
+  typedef internal::Dereference_to_handle_enforcer<
+            CTP2,
+            typename CTP2::Finite_vertices_iterator,
+            vertex_descriptor>                                                vertex_iterator;
+
+  typedef typename CTP2::Subconstraint_iterator::value_type Subconstr_it_v_t;
+  typedef First_of_pair_property_map<Subconstr_it_v_t> Subconstr_map;
+  typedef Property_map_to_unary_function<Subconstr_map> Subconstr_uf;
+  typedef boost::transform_iterator<Subconstr_uf, typename CTP2::Subconstraint_iterator> edge_iterator;
+
+  CTP2_subconstraint_graph (CTP2& ctp2) : ctp2(ctp2) { }
+
+  friend Iterator_range<vertex_iterator> vertices (const CTP2_subconstraint_graph& g)
+  {
+    return make_range (vertex_iterator(g.ctp2.finite_vertices_begin()),
+                       vertex_iterator(g.ctp2.finite_vertices_end()));
+  }
+
+  friend Iterator_range<edge_iterator> edges (const CTP2_subconstraint_graph& g)
+  {
+    return make_range (boost::make_transform_iterator(g.ctp2.subconstraints_begin(), Subconstr_uf(Subconstr_map())),
+                       boost::make_transform_iterator(g.ctp2.subconstraints_end(), Subconstr_uf(Subconstr_map())));
+  }
+
+  friend vertex_descriptor source (edge_descriptor ed, const CTP2_subconstraint_graph&)
+  {
+    return ed.first;
+  }
+
+  friend vertex_descriptor target (edge_descriptor ed, const CTP2_subconstraint_graph&)
+  {
+    return ed.second;
+  }
+};
+
+
+template <typename CTP2>
+class CTP2_graph_visitor
+{
+private:
+  CTP2& ctp2;
+  std::vector<typename CTP2::Constraint_id> to_remove;
+  typename CTP2::Constraint_id current;
+  typename CTP2::Vertex_handle latest_vertex;
+
+public:
+
+  CTP2_graph_visitor (CTP2& ctp2) : ctp2 (ctp2) { }
+
+  void start_new_polyline()
+  {
+    latest_vertex = typename CTP2::Vertex_handle();
+    current = typename CTP2::Constraint_id();
+  }
+
+  void add_node (typename CTP2::Vertex_handle vh)
+  {
+    if (latest_vertex != typename CTP2::Vertex_handle())
+    {
+      to_remove.push_back (ctp2.context(latest_vertex, vh).id());
+      typename CTP2::Constraint_id cid = ctp2.insert_constraint(latest_vertex, vh);
+      if (current == typename CTP2::Constraint_id())
+        current = cid;
+      else
+        current = ctp2.concatenate (current, cid);
+    }
+    latest_vertex = vh;
+  }
+
+  void end_polyline()
+  {
+    for (typename CTP2::Constraint_id id : to_remove)
+      ctp2.remove_constraint(id);
+    to_remove.clear();
+  }
+};
+
+} // namespace internal
+
+} // namespace CGAL
+
+#endif // CGAL_CTP2_SUBCONSTRAINT_GRAPH_H
diff -pruN 5.0.3-1/include/CGAL/Triangulation_2/internal/Polyline_constraint_hierarchy_2.h 5.1-1/include/CGAL/Triangulation_2/internal/Polyline_constraint_hierarchy_2.h
--- 5.0.3-1/include/CGAL/Triangulation_2/internal/Polyline_constraint_hierarchy_2.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_2/internal/Polyline_constraint_hierarchy_2.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/Triangulation_2/internal/Polyline_constraint_hierarchy_2.h $
-// $Id: Polyline_constraint_hierarchy_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/Triangulation_2/internal/Polyline_constraint_hierarchy_2.h $
+// $Id: Polyline_constraint_hierarchy_2.h d1a323c 2020-03-26T19:24:14+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -28,12 +28,11 @@ namespace CGAL {
 
 // T               is expected to be Vertex_handle
 // Compare         is a comparison operator for type T
-// Data            is intended to store info on a Vertex
-template <class T, class Compare, class Data>
+// Point           the point type of vertices
+template <class T, class Compare, class Point>
 class Polyline_constraint_hierarchy_2
 {
 public:
-  typedef Data                                    Point;
   typedef T                                       Vertex_handle;
   typedef std::pair<T, T>                         Edge;
   typedef std::pair<T, T>                         Constraint;
@@ -43,14 +42,12 @@ private:
   class Node {
   public:
     explicit Node(Vertex_handle vh, bool input = false)
-      : vertex_(vh), point_(vh->point()), id(-1), input(input)
+      : vertex_(vh), id(-1), input(input)
     {}
-    Point& point() { return point_; }
-    const Point& point() const { return point_; }
+    const Point& point() const { return vertex_->point(); }
     Vertex_handle vertex() const { return vertex_; }
   private:
     Vertex_handle vertex_;
-    Point point_;
   public:
     int id;
     bool input;
@@ -65,7 +62,7 @@ public:
     : public boost::iterator_adaptor<
     Point_it
     , typename Vertex_list::all_iterator
-    , Point
+    , const Point
     >
   {
   public:
@@ -73,7 +70,7 @@ public:
     Point_it(typename Vertex_list::all_iterator it) : Point_it::iterator_adaptor_(it) {}
   private:
     friend class boost::iterator_core_access;
-    Point& dereference() const { return this->base()->point(); }
+    const Point& dereference() const { return this->base()->point(); }
   };
 
   // only nodes with a vertex_handle that is still in the triangulation
@@ -150,17 +147,13 @@ public:
   };
 
   class Context {
-    friend class Polyline_constraint_hierarchy_2<T,Compare,Data>;
+    friend class Polyline_constraint_hierarchy_2<T,Compare,Point>;
   private:
     Vertex_list*    enclosing;
     Vertex_it       pos;
   public:
     Context() : enclosing(nullptr) {}
 
-    Context(const Context& hc)
-      : enclosing(hc.enclosing), pos(hc.pos)
-    {}
-
     Vertex_it    vertices_begin()const { return enclosing->skip_begin();}
     Vertex_it    current()const {return pos;}
     Vertex_it    vertices_end()const {return enclosing->skip_end();}
@@ -190,9 +183,11 @@ public:
     , sc_to_c_map(Pair_compare(comp))
   { }
   Polyline_constraint_hierarchy_2(const Polyline_constraint_hierarchy_2& ch);
+  Polyline_constraint_hierarchy_2(Polyline_constraint_hierarchy_2&&) = default;
   ~Polyline_constraint_hierarchy_2(){ clear();}
   void clear();
   Polyline_constraint_hierarchy_2& operator=(const Polyline_constraint_hierarchy_2& ch);
+  Polyline_constraint_hierarchy_2& operator=(Polyline_constraint_hierarchy_2&& ch) = default;
 
   // Query
   bool is_subconstrained_edge(T va, T vb) const;
@@ -284,8 +279,8 @@ public:
   void   print() const;
 };
 
-template <class T, class Compare, class Data>
-Polyline_constraint_hierarchy_2<T,Compare,Data>::
+template <class T, class Compare, class Point>
+Polyline_constraint_hierarchy_2<T,Compare,Point>::
 Polyline_constraint_hierarchy_2(const Polyline_constraint_hierarchy_2& ch)
   : comp(ch.comp)
   , sc_to_c_map(Pair_compare(comp))
@@ -293,17 +288,17 @@ Polyline_constraint_hierarchy_2(const Po
   copy(ch);
 }
 
-template <class T, class Compare, class Data>
-Polyline_constraint_hierarchy_2<T,Compare,Data>&
-Polyline_constraint_hierarchy_2<T,Compare,Data>::
+template <class T, class Compare, class Point>
+Polyline_constraint_hierarchy_2<T,Compare,Point>&
+Polyline_constraint_hierarchy_2<T,Compare,Point>::
 operator=(const Polyline_constraint_hierarchy_2& ch){
   copy(ch);
   return *this;
 }
 
-template <class T, class Compare, class Data>
+template <class T, class Compare, class Point>
 void
-Polyline_constraint_hierarchy_2<T,Compare,Data>::
+Polyline_constraint_hierarchy_2<T,Compare,Point>::
 copy(const Polyline_constraint_hierarchy_2& ch1)
 {
   // create a identity transfer vertex map
@@ -318,9 +313,9 @@ copy(const Polyline_constraint_hierarchy
   copy(ch1, vmap);
 }
 
-template <class T, class Compare, class Data>
+template <class T, class Compare, class Point>
 void
-Polyline_constraint_hierarchy_2<T,Compare,Data>::
+Polyline_constraint_hierarchy_2<T,Compare,Point>::
 copy(const Polyline_constraint_hierarchy_2& ch1, std::map<Vertex_handle,Vertex_handle>& vmap)
   // copy with a transfer vertex map
 {
@@ -365,9 +360,9 @@ copy(const Polyline_constraint_hierarchy
 }
 
 
-template <class T, class Compare, class Data>
+template <class T, class Compare, class Point>
 void
-Polyline_constraint_hierarchy_2<T,Compare,Data>::
+Polyline_constraint_hierarchy_2<T,Compare,Point>::
 swap(Polyline_constraint_hierarchy_2& ch)
 {
   constraint_set.swap(ch.constraint_set);
@@ -376,16 +371,16 @@ swap(Polyline_constraint_hierarchy_2& ch
 
 
 /*
-template <class T, class Compare, class Data>
-bool Polyline_constraint_hierarchy_2<T,Compare,Data>::
+template <class T, class Compare, class Point>
+bool Polyline_constraint_hierarchy_2<T,Compare,Point>::
 is_constrained_edge(T va, T vb) const
 {
   return( c_to_sc_map.find(make_edge(va, vb)) != c_to_sc_map.end() );
 }
 */
 
-template <class T, class Compare, class Data>
-bool Polyline_constraint_hierarchy_2<T,Compare,Data>::
+template <class T, class Compare, class Point>
+bool Polyline_constraint_hierarchy_2<T,Compare,Point>::
 is_subconstrained_edge(T va, T vb) const
 {
   return( sc_to_c_map.find(make_edge(va, vb)) != sc_to_c_map.end() );
@@ -393,8 +388,8 @@ is_subconstrained_edge(T va, T vb) const
 
 
 // af: obsolete
-template <class T, class Compare, class Data>
-bool Polyline_constraint_hierarchy_2<T,Compare,Data>::
+template <class T, class Compare, class Point>
+bool Polyline_constraint_hierarchy_2<T,Compare,Point>::
 enclosing_constraint(Edge he, Constraint& hc) const
 {
   Context_iterator hcit, past;
@@ -405,8 +400,8 @@ enclosing_constraint(Edge he, Constraint
 
 
 // used by Constrained_triangulation_plus_2::intersect with Exact_intersection_tag
-template <class T, class Compare, class Data>
-bool Polyline_constraint_hierarchy_2<T,Compare,Data>::
+template <class T, class Compare, class Point>
+bool Polyline_constraint_hierarchy_2<T,Compare,Point>::
 enclosing_constraint(T  vaa, T  vbb, T& va, T& vb) const
 {
   Context_iterator hcit, past;
@@ -433,8 +428,8 @@ enclosing_constraint(T  vaa, T  vbb, T&
 }
 
 // af: obsolete
-template <class T, class Compare, class Data>
-bool Polyline_constraint_hierarchy_2<T,Compare,Data>::
+template <class T, class Compare, class Point>
+bool Polyline_constraint_hierarchy_2<T,Compare,Point>::
 enclosing_constraints(T vaa, T vbb , Constraint_list& hcl) const
 {
   Context_iterator hcit, past;
@@ -446,9 +441,9 @@ enclosing_constraints(T vaa, T vbb , Con
   return true;
 }
 
-template <class T, class Compare, class Data>
-typename Polyline_constraint_hierarchy_2<T,Compare,Data>::Context
-Polyline_constraint_hierarchy_2<T,Compare,Data>::
+template <class T, class Compare, class Point>
+typename Polyline_constraint_hierarchy_2<T,Compare,Point>::Context
+Polyline_constraint_hierarchy_2<T,Compare,Point>::
 context(T va, T vb)
 {
   Context_iterator hcit, past;
@@ -456,9 +451,9 @@ context(T va, T vb)
   return *hcit;
 }
 
-template <class T, class Compare, class Data>
+template <class T, class Compare, class Point>
 std::size_t
-Polyline_constraint_hierarchy_2<T,Compare,Data>::
+Polyline_constraint_hierarchy_2<T,Compare,Point>::
 number_of_enclosing_constraints(T va, T vb) const
 {
   Context_list* hcl = nullptr;
@@ -467,9 +462,9 @@ number_of_enclosing_constraints(T va, T
   return hcl->size();
 }
 
-template <class T, class Compare, class Data>
-typename Polyline_constraint_hierarchy_2<T,Compare,Data>::Context_iterator
-Polyline_constraint_hierarchy_2<T,Compare,Data>::
+template <class T, class Compare, class Point>
+typename Polyline_constraint_hierarchy_2<T,Compare,Point>::Context_iterator
+Polyline_constraint_hierarchy_2<T,Compare,Point>::
 contexts_begin(T va, T vb) const
 {
    Context_iterator first, last;
@@ -477,9 +472,9 @@ contexts_begin(T va, T vb) const
    return first;
 }
 
-template <class T, class Compare, class Data>
-typename Polyline_constraint_hierarchy_2<T,Compare,Data>::Context_iterator
-Polyline_constraint_hierarchy_2<T,Compare,Data>::
+template <class T, class Compare, class Point>
+typename Polyline_constraint_hierarchy_2<T,Compare,Point>::Context_iterator
+Polyline_constraint_hierarchy_2<T,Compare,Point>::
 contexts_end(T va, T vb) const
 {
    Context_iterator first, last;
@@ -487,9 +482,9 @@ contexts_end(T va, T vb) const
    return last;
 }
 
-template <class T, class Compare, class Data>
+template <class T, class Compare, class Point>
 void
-Polyline_constraint_hierarchy_2<T,Compare,Data>::
+Polyline_constraint_hierarchy_2<T,Compare,Point>::
 swap(Constraint_id first, Constraint_id second){
     // We have to look at all subconstraints
   for(Vertex_it it = first.vl_ptr()->skip_begin(), succ = it, end = first.vl_ptr()->skip_end();
@@ -502,7 +497,7 @@ swap(Constraint_id first, Constraint_id
     // and replace the context of the constraint
     for(Context_iterator ctit=hcl->begin(); ctit != hcl->end(); ctit++) {
       if(ctit->enclosing == first.vl_ptr()){
-        ctit->enclosing = 0;
+        ctit->enclosing = nullptr;
         break;
       }
     }
@@ -533,7 +528,7 @@ swap(Constraint_id first, Constraint_id
 
     // and replace the context of the constraint
     for(Context_iterator ctit=hcl->begin(); ctit != hcl->end(); ctit++) {
-      if(ctit->enclosing == 0){
+      if(ctit->enclosing == nullptr){
         ctit->enclosing = second.vl_ptr();
         break;
       }
@@ -543,9 +538,9 @@ swap(Constraint_id first, Constraint_id
 }
 
 
-template <class T, class Compare, class Data>
+template <class T, class Compare, class Point>
 void
-Polyline_constraint_hierarchy_2<T,Compare,Data>::
+Polyline_constraint_hierarchy_2<T,Compare,Point>::
 remove_constraint(Constraint_id cid){
   constraint_set.erase(cid);
 
@@ -580,8 +575,8 @@ remove_constraint(Constraint_id cid){
 // This function removes vertex v from the polyline constraint
 // It only works for one polyline passing through v
 // and for the case that the constrained edge u,w has no intersections
-template <class T, class Compare, class Data>
-void Polyline_constraint_hierarchy_2<T,Compare,Data>::simplify(Vertex_it uc,
+template <class T, class Compare, class Point>
+void Polyline_constraint_hierarchy_2<T,Compare,Point>::simplify(Vertex_it uc,
                                                        Vertex_it vc,
                                                        Vertex_it wc)
 
@@ -628,9 +623,9 @@ void Polyline_constraint_hierarchy_2<T,C
 }
 
 
-template <class T, class Compare, class Data>
+template <class T, class Compare, class Point>
 std::size_t
-Polyline_constraint_hierarchy_2<T,Compare,Data>::remove_points_without_corresponding_vertex(Constraint_id cid)
+Polyline_constraint_hierarchy_2<T,Compare,Point>::remove_points_without_corresponding_vertex(Constraint_id cid)
 {
   std::size_t n = 0;
   for(Point_it it = points_in_constraint_begin(cid);
@@ -643,9 +638,9 @@ Polyline_constraint_hierarchy_2<T,Compar
   return n;
 }
 
-template <class T, class Compare, class Data>
+template <class T, class Compare, class Point>
 std::size_t
-Polyline_constraint_hierarchy_2<T,Compare,Data>::remove_points_without_corresponding_vertex()
+Polyline_constraint_hierarchy_2<T,Compare,Point>::remove_points_without_corresponding_vertex()
 {
   std::size_t n = 0;
   for(C_iterator it = constraint_set.begin(); it!= constraint_set.end(); ++it){
@@ -655,9 +650,9 @@ Polyline_constraint_hierarchy_2<T,Compar
 }
 
 
-template <class T, class Compare, class Data>
-typename Polyline_constraint_hierarchy_2<T,Compare,Data>::Constraint_id
-Polyline_constraint_hierarchy_2<T,Compare,Data>::concatenate(Constraint_id first, Constraint_id second)
+template <class T, class Compare, class Point>
+typename Polyline_constraint_hierarchy_2<T,Compare,Point>::Constraint_id
+Polyline_constraint_hierarchy_2<T,Compare,Point>::concatenate(Constraint_id first, Constraint_id second)
 {
   constraint_set.erase(first);
   constraint_set.erase(second);
@@ -708,9 +703,9 @@ Polyline_constraint_hierarchy_2<T,Compar
   return first;
 }
 
-template <class T, class Compare, class Data>
-typename Polyline_constraint_hierarchy_2<T,Compare,Data>::Constraint_id
-Polyline_constraint_hierarchy_2<T,Compare,Data>::concatenate2(Constraint_id first, Constraint_id second)
+template <class T, class Compare, class Point>
+typename Polyline_constraint_hierarchy_2<T,Compare,Point>::Constraint_id
+Polyline_constraint_hierarchy_2<T,Compare,Point>::concatenate2(Constraint_id first, Constraint_id second)
 {
   constraint_set.erase(first);
   constraint_set.erase(second);
@@ -764,9 +759,9 @@ Polyline_constraint_hierarchy_2<T,Compar
   // split a constraint in two constraints, so that vcit becomes the first
   // vertex of the new constraint
   // returns the new constraint
-template <class T, class Compare, class Data>
-typename Polyline_constraint_hierarchy_2<T,Compare,Data>::Constraint_id
-Polyline_constraint_hierarchy_2<T,Compare,Data>::split(Constraint_id first, Vertex_it vcit)
+template <class T, class Compare, class Point>
+typename Polyline_constraint_hierarchy_2<T,Compare,Point>::Constraint_id
+Polyline_constraint_hierarchy_2<T,Compare,Point>::split(Constraint_id first, Vertex_it vcit)
 {
   constraint_set.erase(first);
   Vertex_list* second = new Vertex_list;
@@ -798,9 +793,9 @@ Polyline_constraint_hierarchy_2<T,Compar
   return second;
 }
 
-template <class T, class Compare, class Data>
-typename Polyline_constraint_hierarchy_2<T,Compare,Data>::Constraint_id
-Polyline_constraint_hierarchy_2<T,Compare,Data>::split2(Constraint_id first, Vertex_it vcit)
+template <class T, class Compare, class Point>
+typename Polyline_constraint_hierarchy_2<T,Compare,Point>::Constraint_id
+Polyline_constraint_hierarchy_2<T,Compare,Point>::split2(Constraint_id first, Vertex_it vcit)
 {
   constraint_set.erase(first);
   Vertex_list* second = new Vertex_list;
@@ -837,9 +832,9 @@ Polyline_constraint_hierarchy_2<T,Compar
 when a constraint is inserted,
 it is, at first, both  a constraint and a subconstraint
  */
-template <class T, class Compare, class Data>
-typename Polyline_constraint_hierarchy_2<T,Compare,Data>::Vertex_list*
-Polyline_constraint_hierarchy_2<T,Compare,Data>::
+template <class T, class Compare, class Point>
+typename Polyline_constraint_hierarchy_2<T,Compare,Point>::Vertex_list*
+Polyline_constraint_hierarchy_2<T,Compare,Point>::
 insert_constraint(T va, T vb){
   Edge        he = make_edge(va, vb);
   Vertex_list*  children = new Vertex_list;
@@ -865,9 +860,9 @@ insert_constraint(T va, T vb){
 }
 
 
-template <class T, class Compare, class Data>
-typename Polyline_constraint_hierarchy_2<T,Compare,Data>::Vertex_list*
-Polyline_constraint_hierarchy_2<T,Compare,Data>::
+template <class T, class Compare, class Point>
+typename Polyline_constraint_hierarchy_2<T,Compare,Point>::Vertex_list*
+Polyline_constraint_hierarchy_2<T,Compare,Point>::
 insert_constraint_old_API(T va, T vb){
   Edge        he = make_edge(va, vb);
 
@@ -894,9 +889,9 @@ insert_constraint_old_API(T va, T vb){
 }
 
 
-template <class T, class Compare, class Data>
+template <class T, class Compare, class Point>
 void
-Polyline_constraint_hierarchy_2<T,Compare,Data>::
+Polyline_constraint_hierarchy_2<T,Compare,Point>::
 append_constraint(Constraint_id cid, T va, T vb){
   Edge        he = make_edge(va, vb);
   Context_list* fathers;
@@ -919,8 +914,8 @@ append_constraint(Constraint_id cid, T v
 }
 
 
-template <class T, class Compare, class Data>
-void Polyline_constraint_hierarchy_2<T,Compare,Data>::
+template <class T, class Compare, class Point>
+void Polyline_constraint_hierarchy_2<T,Compare,Point>::
 clear()
 {
   C_iterator cit;
@@ -940,8 +935,8 @@ clear()
 }
 
 
-template <class T, class Compare, class Data>
-bool Polyline_constraint_hierarchy_2<T,Compare,Data>::
+template <class T, class Compare, class Point>
+bool Polyline_constraint_hierarchy_2<T,Compare,Point>::
 next_along_sc(T va, T vb, T& w) const
 {
   // find the next vertex after vb along any enclosing constrained
@@ -969,8 +964,8 @@ next_along_sc(T va, T vb, T& w) const
   Attention, le point v DOIT etre un point de Steiner,
   et les segments va,v et v,vb sont des sous contraintes.
 */
-template <class T, class Compare, class Data>
-void Polyline_constraint_hierarchy_2<T,Compare,Data>::
+template <class T, class Compare, class Point>
+void Polyline_constraint_hierarchy_2<T,Compare,Point>::
 remove_Steiner(T v, T va, T vb)
 {
   // remove a Steiner point
@@ -1001,16 +996,16 @@ remove_Steiner(T v, T va, T vb)
   same as add_Steiner
   precondition : va,vb est une souscontrainte.
 */
-template <class T, class Compare, class Data>
-void Polyline_constraint_hierarchy_2<T,Compare,Data>::
+template <class T, class Compare, class Point>
+void Polyline_constraint_hierarchy_2<T,Compare,Point>::
 split_constraint(T va, T vb, T vc){
   add_Steiner(va, vb, vc);
 }
 
 
-template <class T, class Compare, class Data>
+template <class T, class Compare, class Point>
 void
-Polyline_constraint_hierarchy_2<T,Compare,Data>::
+Polyline_constraint_hierarchy_2<T,Compare,Point>::
 add_Steiner(T va, T vb, T vc){
   Context_list* hcl=nullptr;
   if(!get_contexts(va,vb,hcl)) CGAL_triangulation_assertion(false);
@@ -1061,19 +1056,19 @@ add_Steiner(T va, T vb, T vc){
 }
 
 
-template <class T, class Compare, class Data>
+template <class T, class Compare, class Point>
 inline
-typename Polyline_constraint_hierarchy_2<T,Compare,Data>::Edge
-Polyline_constraint_hierarchy_2<T,Compare,Data>::
+typename Polyline_constraint_hierarchy_2<T,Compare,Point>::Edge
+Polyline_constraint_hierarchy_2<T,Compare,Point>::
 make_edge(T va, T vb) const
 {
   return comp(va, vb) ? Edge(va,vb) : Edge(vb,va);
 }
 
-template <class T, class Compare, class Data>
+template <class T, class Compare, class Point>
 inline
 bool
-Polyline_constraint_hierarchy_2<T,Compare,Data>::
+Polyline_constraint_hierarchy_2<T,Compare,Point>::
 get_contexts(T va, T vb, Context_list* & hcl) const
 {
   Sc_iterator sc_iter = sc_to_c_map.find(make_edge(va,vb));
@@ -1082,10 +1077,10 @@ get_contexts(T va, T vb, Context_list* &
   return true;
 }
 
-template <class T, class Compare, class Data>
+template <class T, class Compare, class Point>
 inline
 bool
-Polyline_constraint_hierarchy_2<T,Compare,Data>::
+Polyline_constraint_hierarchy_2<T,Compare,Point>::
 get_contexts(T va, T vb,
              Context_iterator& ctxt,
              Context_iterator& past) const
@@ -1099,19 +1094,19 @@ get_contexts(T va, T vb,
 
 
 
-template <class T, class Compare, class Data>
+template <class T, class Compare, class Point>
 inline
-typename Polyline_constraint_hierarchy_2<T,Compare,Data>::Vertex_it
-Polyline_constraint_hierarchy_2<T,Compare,Data>::
+typename Polyline_constraint_hierarchy_2<T,Compare,Point>::Vertex_it
+Polyline_constraint_hierarchy_2<T,Compare,Point>::
 get_pos(T va, T vb) const
   //return pos in the first context
 {
     return (*sc_to_c_map.find(make_edge(va,vb))).second->begin().pos;
 }
 
-template <class T, class Compare, class Data>
+template <class T, class Compare, class Point>
 void
-Polyline_constraint_hierarchy_2<T,Compare,Data>::
+Polyline_constraint_hierarchy_2<T,Compare,Point>::
 oriented_end(T va, T vb, T& vc) const
 {
   Context_iterator ctxt, past;
@@ -1123,9 +1118,9 @@ oriented_end(T va, T vb, T& vc) const
 }
 
 
-template <class T, class Compare, class Data>
+template <class T, class Compare, class Point>
 void
-Polyline_constraint_hierarchy_2<T,Compare,Data>::
+Polyline_constraint_hierarchy_2<T,Compare,Point>::
 print() const
 {
   C_iterator hcit;
diff -pruN 5.0.3-1/include/CGAL/Triangulation_2/internal/Triangulation_2_filtered_projection_traits_3.h 5.1-1/include/CGAL/Triangulation_2/internal/Triangulation_2_filtered_projection_traits_3.h
--- 5.0.3-1/include/CGAL/Triangulation_2/internal/Triangulation_2_filtered_projection_traits_3.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_2/internal/Triangulation_2_filtered_projection_traits_3.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/Triangulation_2/internal/Triangulation_2_filtered_projection_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/Triangulation_2/internal/Triangulation_2_filtered_projection_traits_3.h $
 // $Id: Triangulation_2_filtered_projection_traits_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulation_2/internal/Triangulation_2_projection_traits_base_3.h 5.1-1/include/CGAL/Triangulation_2/internal/Triangulation_2_projection_traits_base_3.h
--- 5.0.3-1/include/CGAL/Triangulation_2/internal/Triangulation_2_projection_traits_base_3.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_2/internal/Triangulation_2_projection_traits_base_3.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/Triangulation_2/internal/Triangulation_2_projection_traits_base_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/Triangulation_2/internal/Triangulation_2_projection_traits_base_3.h $
 // $Id: Triangulation_2_projection_traits_base_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulation_2/internal/Triangulation_line_face_circulator_2.h 5.1-1/include/CGAL/Triangulation_2/internal/Triangulation_line_face_circulator_2.h
--- 5.0.3-1/include/CGAL/Triangulation_2/internal/Triangulation_line_face_circulator_2.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_2/internal/Triangulation_line_face_circulator_2.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/Triangulation_2/internal/Triangulation_line_face_circulator_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/Triangulation_2/internal/Triangulation_line_face_circulator_2.h $
 // $Id: Triangulation_line_face_circulator_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulation_2.h 5.1-1/include/CGAL/Triangulation_2.h
--- 5.0.3-1/include/CGAL/Triangulation_2.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_2.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/Triangulation_2.h $
-// $Id: Triangulation_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/Triangulation_2.h $
+// $Id: Triangulation_2.h dcc4fb1 2020-05-20T09:46:26+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Olivier Devillers, Mariette Yvinec
@@ -37,7 +37,6 @@
 #include <CGAL/Spatial_sort_traits_adapter_2.h>
 
 #include <CGAL/double.h>
-#include <CGAL/internal/boost/function_property_map.hpp>
 
 #include <boost/utility/result_of.hpp>
 #include <boost/random/linear_congruential.hpp>
@@ -243,6 +242,7 @@ public:
   // CONSTRUCTORS
   Triangulation_2(const Geom_traits& geom_traits=Geom_traits());
   Triangulation_2(const Triangulation_2<Gt,Tds> &tr);
+  Triangulation_2(Triangulation_2&&) = default;
 
   template <class InputIterator>
   Triangulation_2(InputIterator first, InputIterator last,
@@ -255,6 +255,10 @@ public:
 
   //Assignement
   Triangulation_2 &operator=(const Triangulation_2 &tr);
+  Triangulation_2 &operator=(Triangulation_2 &&) = default;
+
+  // Destructor
+  ~Triangulation_2() = default;
 
   //Helping
   void copy_triangulation(const Triangulation_2 &tr);
@@ -629,7 +633,7 @@ std::ptrdiff_t insert(InputIterator firs
          typename std::iterator_traits<InputIterator>::value_type,
          Point
          >
-         >::type* = NULL)
+         >::type* = nullptr)
 #else
   template < class InputIterator >
   std::ptrdiff_t
@@ -1017,7 +1021,7 @@ includes_edge(Vertex_handle va, Vertex_h
   Orientation orient;
   int indv;
   Edge_circulator ec = incident_edges(va), done(ec);
-  if (ec != 0) {
+  if (ec != nullptr) {
     do {
       //find the index of the other vertex of *ec
       indv = 3 - ((*ec).first)->index(va) - (*ec).second ;
@@ -2598,7 +2602,7 @@ march_locate_2D_LFC(Face_handle start,
   }else {
     lfc = Line_face_circulator(start->vertex(0), this, t);
   }
-  if(lfc==0 || lfc.collinear_outside()){
+  if(lfc==nullptr || lfc.collinear_outside()){
     // point t lies outside or on the convex hull
     // we walk on the convex hull to find it out
     Face_circulator fc = incident_faces(infinite_vertex());
diff -pruN 5.0.3-1/include/CGAL/Triangulation_2_projection_traits_3.h 5.1-1/include/CGAL/Triangulation_2_projection_traits_3.h
--- 5.0.3-1/include/CGAL/Triangulation_2_projection_traits_3.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_2_projection_traits_3.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/Triangulation_2_projection_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/Triangulation_2_projection_traits_3.h $
 // $Id: Triangulation_2_projection_traits_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulation_2_to_lcc.h 5.1-1/include/CGAL/Triangulation_2_to_lcc.h
--- 5.0.3-1/include/CGAL/Triangulation_2_to_lcc.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_2_to_lcc.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/Triangulation_2_to_lcc.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/Triangulation_2_to_lcc.h $
 // $Id: Triangulation_2_to_lcc.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulation_2_traits_3.h 5.1-1/include/CGAL/Triangulation_2_traits_3.h
--- 5.0.3-1/include/CGAL/Triangulation_2_traits_3.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_2_traits_3.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/Triangulation_2_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/Triangulation_2_traits_3.h $
 // $Id: Triangulation_2_traits_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulation_3.h 5.1-1/include/CGAL/Triangulation_3.h
--- 5.0.3-1/include/CGAL/Triangulation_3.h	2020-08-01 19:02:59.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_3.h	2020-09-08 12:15:52.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_3/include/CGAL/Triangulation_3.h $
-// $Id: Triangulation_3.h 2e8a59d 2020-07-21T15:25:54+02:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_3/include/CGAL/Triangulation_3.h $
+// $Id: Triangulation_3.h 022b1a7 2020-07-21T15:27:49+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Monique Teillaud <Monique.Teillaud@sophia.inria.fr>
@@ -47,7 +47,6 @@
 #include <CGAL/function_objects.h>
 #include <CGAL/Iterator_project.h>
 #include <CGAL/Default.h>
-#include <CGAL/internal/boost/function_property_map.hpp>
 
 #include <CGAL/Bbox_3.h>
 #include <CGAL/Spatial_lock_grid_3.h>
@@ -57,6 +56,7 @@
 #include <boost/random/uniform_smallint.hpp>
 #include <boost/random/variate_generator.hpp>
 #include <boost/mpl/if.hpp>
+#include <boost/property_map/function_property_map.hpp>
 #include <boost/unordered_map.hpp>
 #include <boost/utility/result_of.hpp>
 #include <boost/container/small_vector.hpp>
@@ -179,7 +179,7 @@ public:
 
   void *get_lock_data_structure() const
   {
-    return 0;
+    return nullptr;
   }
 
   void set_lock_data_structure(void *) const {}
@@ -245,7 +245,7 @@ protected:
 public:
   bool is_parallel() const
   {
-    return m_lock_ds != 0;
+    return m_lock_ds != nullptr;
   }
 
   // LOCKS
@@ -731,6 +731,9 @@ public:
     CGAL_triangulation_expensive_postcondition(*this == tr);
   }
 
+  Triangulation_3(Triangulation_3&& tr) = default;
+  ~Triangulation_3() = default;
+
   template < typename InputIterator >
   Triangulation_3(InputIterator first, InputIterator last,
                   const GT& gt = GT(), Lock_data_structure *lock_ds = nullptr)
@@ -757,21 +760,21 @@ public:
     init_tds();
   }
 
-  Triangulation_3& operator=(Triangulation_3 tr)
+  Triangulation_3& operator=(const Triangulation_3& tr)
   {
-    // Because the parameter tr is passed by value, the triangulation passed
-    // as argument has been copied.
-    // The following 'swap' consumes the *copy* and the original triangulation
-    // is left untouched.
-    swap(tr);
+    Triangulation_3 copy(tr);
+    swap(copy);
     return *this;
   }
 
+  Triangulation_3& operator=(Triangulation_3&& tr) = default;
+
   // HELPING FUNCTIONS
-  void swap(Triangulation_3& tr)
+  void swap(Triangulation_3& tr) noexcept
   {
-    std::swap(tr._gt, _gt);
-    std::swap(tr.infinite, infinite);
+    using std::swap;
+    swap(tr._gt, _gt);
+    swap(tr.infinite, infinite);
     _tds.swap(tr._tds);
     Base::swap(tr);
   }
@@ -1139,7 +1142,7 @@ public:
                               typename std::iterator_traits<InputIterator>::value_type,
                               Point
                           >
-                        >::type* = NULL)
+                        >::type* = nullptr)
 #else
   template < class InputIterator >
   std::ptrdiff_t insert(InputIterator first, InputIterator last)
@@ -2202,8 +2205,80 @@ public:
   bool is_valid(bool verbose = false, int level = 0) const;
   bool is_valid(Cell_handle c, bool verbose = false, int level = 0) const;
   bool is_valid_finite(Cell_handle c, bool verbose = false, int level=0) const;
+
+  //IO
+  template <typename Tr_src,
+            typename ConvertVertex,
+            typename ConvertCell>
+  std::istream& file_input(std::istream& is,
+                           ConvertVertex convert_vertex = ConvertVertex(),
+                           ConvertCell convert_cell = ConvertCell())
+  {
+    // reads
+    // the dimension
+    // the number of finite vertices
+    // the non combinatorial information on vertices (point, etc)
+    // the number of cells
+    // the cells by the indices of their vertices in the preceding list
+    // of vertices, plus the non combinatorial information on each cell
+    // the neighbors of each cell by their index in the preceding list of cells
+    // when dimension < 3 : the same with faces of maximal dimension
+
+    // If this is used for a TDS, the vertices are processed from 0 to n.
+    // Else, we make V[0] the infinite vertex and work from 1 to n+1.
+
+    typedef Self Triangulation;
+    typedef typename Triangulation::Vertex_handle  Vertex_handle;
+    typedef typename Triangulation::Cell_handle    Cell_handle;
+
+    typedef typename Tr_src::Vertex Vertex1;
+    typedef typename Tr_src::Cell Cell1;
+
+    clear();
+    tds().cells().clear();
+
+    std::size_t n;
+    int d;
+    if(is_ascii(is))
+      is >> d >> n;
+    else {
+      read(is, d);
+      read(is, n);
+    }
+    if(!is) return is;
+    tds().set_dimension(d);
+
+    std::size_t V_size = n+1;
+    std::vector< Vertex_handle > V(V_size);
+
+    // the infinite vertex is numbered 0
+    V[0] = infinite_vertex();
+
+    for (std::size_t i = 1; i < V_size; ++i) {
+      Vertex1 v;
+      if(!(is >> v)) return is;
+      Vertex_handle vh=tds().create_vertex( convert_vertex(v) );
+      V[i] = vh;
+      convert_vertex(v, *V[i]);
+    }
+
+    std::vector< Cell_handle > C;
+
+    std::size_t m;
+    tds().read_cells(is, V, m, C);
+
+    for (std::size_t j=0 ; j < m; j++) {
+      Cell1 c;
+      if(!(is >> c)) return is;
+      convert_cell(c, *C[j]);
+    }
+
+    CGAL_triangulation_assertion( is_valid(false) );
+    return is;
+  }
 };
 
+
 template < class GT, class Tds, class Lds >
 std::istream& operator>> (std::istream& is, Triangulation_3<GT, Tds, Lds>& tr)
 {
@@ -6598,12 +6673,12 @@ _remove_cluster_3D(InputIterator first,
       // Spatial sorting can only be applied to bare points, so we need an adaptor
       typedef typename Geom_traits::Construct_point_3 Construct_point_3;
       typedef typename boost::result_of<const Construct_point_3(const Point&)>::type Ret;
-      typedef CGAL::internal::boost_::function_property_map<Construct_point_3, Point, Ret> fpmap;
+      typedef boost::function_property_map<Construct_point_3, Point, Ret> fpmap;
       typedef CGAL::Spatial_sort_traits_adapter_3<Geom_traits, fpmap> Search_traits_3;
 
       spatial_sort(vps.begin(), vps.end(),
                    Search_traits_3(
-                     CGAL::internal::boost_::make_function_property_map<Point, Ret, Construct_point_3>(
+                     boost::make_function_property_map<Point, Ret, Construct_point_3>(
                        geom_traits().construct_point_3_object()), geom_traits()));
 
       std::size_t svps = vps.size();
diff -pruN 5.0.3-1/include/CGAL/Triangulation_3_to_lcc.h 5.1-1/include/CGAL/Triangulation_3_to_lcc.h
--- 5.0.3-1/include/CGAL/Triangulation_3_to_lcc.h	2020-08-01 19:02:59.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_3_to_lcc.h	2020-09-08 12:15:52.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_3/include/CGAL/Triangulation_3_to_lcc.h $
-// $Id: Triangulation_3_to_lcc.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_3/include/CGAL/Triangulation_3_to_lcc.h $
+// $Id: Triangulation_3_to_lcc.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
@@ -15,9 +15,27 @@
 
 #include <CGAL/assertions.h>
 #include <map>
+#include <CGAL/Weighted_point_3.h>
 
 namespace CGAL {
 
+  namespace internal
+  {
+    template<typename Point>
+    struct Get_point
+    {
+      static const Point& run(const Point& p)
+      { return p; }
+    };
+
+    template<typename Kernel>
+    struct Get_point<CGAL::Weighted_point_3<Kernel> >
+    {
+      static const typename Kernel::Point_3& run(const CGAL::Weighted_point_3<Kernel>& p)
+      { return p.point(); }
+    };
+  }
+
   /** Convert a given Triangulation_3 into a 3D linear cell complex.
    * @param alcc the used linear cell complex.
    * @param atr the Triangulation_3.
@@ -52,7 +70,7 @@ namespace CGAL {
     for (TVertex_iterator itv = atr.vertices_begin();
          itv != atr.vertices_end(); ++itv)
     {
-      TV[itv] = alcc.create_vertex_attribute(itv->point());
+      TV[itv] = alcc.create_vertex_attribute(internal::Get_point<typename Triangulation::Point>::run(itv->point()));
     }
 
     // Create the tetrahedron and create a map to link Cell_iterator
diff -pruN 5.0.3-1/include/CGAL/triangulation_assertions.h 5.1-1/include/CGAL/triangulation_assertions.h
--- 5.0.3-1/include/CGAL/triangulation_assertions.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/triangulation_assertions.h	2020-09-08 12:15:32.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/triangulation_assertions.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/triangulation_assertions.h $
 // $Id: triangulation_assertions.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulation_cell_base_3.h 5.1-1/include/CGAL/Triangulation_cell_base_3.h
--- 5.0.3-1/include/CGAL/Triangulation_cell_base_3.h	2020-08-01 19:02:59.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_cell_base_3.h	2020-09-08 12:15:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_3/include/CGAL/Triangulation_cell_base_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_3/include/CGAL/Triangulation_cell_base_3.h $
 // $Id: Triangulation_cell_base_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulation_cell_base_with_info_3.h 5.1-1/include/CGAL/Triangulation_cell_base_with_info_3.h
--- 5.0.3-1/include/CGAL/Triangulation_cell_base_with_info_3.h	2020-08-01 19:02:59.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_cell_base_with_info_3.h	2020-09-08 12:15:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_3/include/CGAL/Triangulation_cell_base_with_info_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_3/include/CGAL/Triangulation_cell_base_with_info_3.h $
 // $Id: Triangulation_cell_base_with_info_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulation_conformer_2.h 5.1-1/include/CGAL/Triangulation_conformer_2.h
--- 5.0.3-1/include/CGAL/Triangulation_conformer_2.h	2020-08-01 19:02:29.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_conformer_2.h	2020-09-08 12:15:06.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_2/include/CGAL/Triangulation_conformer_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Mesh_2/include/CGAL/Triangulation_conformer_2.h $
 // $Id: Triangulation_conformer_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulation_data_structure_2.h 5.1-1/include/CGAL/Triangulation_data_structure_2.h
--- 5.0.3-1/include/CGAL/Triangulation_data_structure_2.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_data_structure_2.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/TDS_2/include/CGAL/Triangulation_data_structure_2.h $
-// $Id: Triangulation_data_structure_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/TDS_2/include/CGAL/Triangulation_data_structure_2.h $
+// $Id: Triangulation_data_structure_2.h 871c972 2020-06-03T16:23:22+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -105,8 +105,13 @@ protected:
 public:
   Triangulation_data_structure_2();
   Triangulation_data_structure_2(const Tds &tds);
+  Triangulation_data_structure_2(Triangulation_data_structure_2&& tds)
+    noexcept(noexcept(Face_range(std::move(tds._faces))) &&
+             noexcept(Vertex_range(std::move(tds._vertices))));
+
   ~Triangulation_data_structure_2();
   Tds& operator= (const Tds &tds);
+  Tds& operator= (Tds&& tds) noexcept(noexcept(Tds(std::move(tds))));
   void swap(Tds &tds);
 
   //ACCESS FUNCTIONS
@@ -642,9 +647,6 @@ public:
 
   Triangulation_default_data_structure_2(const Geom_traits& = Geom_traits())
     : Tds() {}
-
-  Triangulation_default_data_structure_2(const Tdds &tdds)
-    : Tds(tdds) {}
 };
 
 //for backward compatibility
@@ -657,8 +659,6 @@ public:
   typedef Triangulation_data_structure_using_list_2<Vb,Fb>  Tdsul;
 
   Triangulation_data_structure_using_list_2(): Tds() {}
-  Triangulation_data_structure_using_list_2(const Tdsul &tdsul)
-    : Tds(tdsul) {}
 };
 
 
@@ -677,12 +677,23 @@ Triangulation_data_structure_2(const Tds
 
 template < class Vb, class Fb>
 Triangulation_data_structure_2<Vb,Fb> ::
+Triangulation_data_structure_2(Tds &&tds)
+    noexcept(noexcept(Face_range(std::move(tds._faces))) &&
+             noexcept(Vertex_range(std::move(tds._vertices))))
+  : _dimension(std::exchange(tds._dimension, -2))
+  , _faces(std::move(tds._faces))
+  , _vertices(std::move(tds._vertices))
+{
+}
+
+template < class Vb, class Fb>
+Triangulation_data_structure_2<Vb,Fb> ::
 ~Triangulation_data_structure_2()
 {
   clear();
 }
 
-//assignement
+//copy-assignment
 template < class Vb, class Fb>
 Triangulation_data_structure_2<Vb,Fb>&
 Triangulation_data_structure_2<Vb,Fb> ::
@@ -692,6 +703,18 @@ operator= (const Tds &tds)
   return *this;
 }
 
+//move-assignment
+template < class Vb, class Fb>
+Triangulation_data_structure_2<Vb,Fb>&
+Triangulation_data_structure_2<Vb,Fb> ::
+operator= (Tds &&tds) noexcept(noexcept(Tds(std::move(tds))))
+{
+  _faces = std::move(tds._faces);
+  _vertices = std::move(tds._vertices);
+  _dimension = std::exchange(tds._dimension, -2);
+  return *this;
+}
+
 template <  class Vb, class Fb>
 void
 Triangulation_data_structure_2<Vb,Fb>::
@@ -799,7 +822,7 @@ is_edge(Vertex_handle va, Vertex_handle
 {
   Face_handle fc = va->face();
   Face_handle start = fc;
-  if (fc == 0) return false;
+  if (fc == nullptr) return false;
   int inda, indb;
   do {
     inda=fc->index(va);
@@ -1032,7 +1055,9 @@ insert_dim_up(Vertex_handle w,  bool ori
 
       for ( ; lfit != faces_list.end() ; ++lfit) {
         f = * lfit;
-        g = create_face(f); //calls copy constructor of face
+        g = create_face(f->vertex(0),f->vertex(1),f->vertex(2),
+                        f->neighbor(0),f->neighbor(1),f->neighbor(2));
+
         f->set_vertex(dim,v);
         g->set_vertex(dim,w);
         set_adjacency(f, dim, g, dim);
diff -pruN 5.0.3-1/include/CGAL/Triangulation_data_structure_3.h 5.1-1/include/CGAL/Triangulation_data_structure_3.h
--- 5.0.3-1/include/CGAL/Triangulation_data_structure_3.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_data_structure_3.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/TDS_3/include/CGAL/Triangulation_data_structure_3.h $
-// $Id: Triangulation_data_structure_3.h ad23db2 2020-06-30T16:36:58+02:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/TDS_3/include/CGAL/Triangulation_data_structure_3.h $
+// $Id: Triangulation_data_structure_3.h 07eacf5 2020-06-30T16:38:05+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Monique Teillaud <Monique.Teillaud@sophia.inria.fr>
@@ -189,6 +189,7 @@ public:
 
       return hf ^ 419 * hs;
     }
+
   };
 
   static const int maximal_nb_of_facets_of_small_hole = 128;
@@ -230,6 +231,15 @@ public:
     copy_tds(tds);
   }
 
+  Triangulation_data_structure_3(Tds && tds)
+    noexcept(noexcept(Cell_range(std::move(tds._cells))) &&
+             noexcept(Vertex_range(std::move(tds._vertices))))
+    : _dimension(std::exchange(tds._dimension, -2))
+    , _cells(std::move(tds._cells))
+    , _vertices(std::move(tds._vertices))
+  {
+  }
+
   Tds & operator= (const Tds & tds)
   {
     if (&tds != this) {
@@ -239,6 +249,17 @@ public:
     return *this;
   }
 
+  Tds & operator= (Tds && tds)
+    noexcept(noexcept(Tds(std::move(tds))))
+  {
+    _cells = std::move(tds._cells);
+    _vertices = std::move(tds._vertices);
+    _dimension = std::exchange(tds._dimension, -2);
+    return *this;
+  }
+
+  ~Triangulation_data_structure_3() = default; // for the rule-of-five
+
   size_type number_of_vertices() const { return vertices().size(); }
 
   int dimension() const {return _dimension;}
@@ -916,12 +937,6 @@ public:
         *output++ = e;
         return *this;
       }
-      Facet_it& operator=(const Facet_it& f) {
-        output = f.output;
-        filter = f.filter;
-        return *this;
-      }
-      Facet_it(const Facet_it&)=default;
     };
     Facet_it facet_it() {
       return Facet_it(output, filter);
@@ -1046,6 +1061,15 @@ public:
       }
     }
 
+    // Implement the rule-of-five, to please the diagnostic
+    // `cppcoreguidelines-special-member-functions` of clang-tidy.
+    // Instead of defaulting those special member functions, let's
+    // delete them, to prevent any misuse.
+    Vertex_extractor(const Vertex_extractor&) = delete;
+    Vertex_extractor(Vertex_extractor&&) = delete;
+    Vertex_extractor& operator=(const Vertex_extractor&) = delete;
+    Vertex_extractor& operator=(Vertex_extractor&&) = delete;
+
     ~Vertex_extractor()
     {
       for(std::size_t i=0; i < tmp_vertices.size(); ++i){
@@ -1567,6 +1591,73 @@ public:
     return s <= maximal_nb_of_facets_of_small_hole;
   }
 
+  //IO
+  template <typename TDS_src,
+            typename ConvertVertex,
+            typename ConvertCell>
+  std::istream& file_input(std::istream& is,
+                           ConvertVertex convert_vertex = ConvertVertex(),
+                           ConvertCell convert_cell = ConvertCell())
+  {
+    // reads
+    // the dimension
+    // the number of finite vertices
+    // the non combinatorial information on vertices (point, etc)
+    // the number of cells
+    // the cells by the indices of their vertices in the preceding list
+    // of vertices, plus the non combinatorial information on each cell
+    // the neighbors of each cell by their index in the preceding list of cells
+    // when dimension < 3 : the same with faces of maximal dimension
+
+    // If this is used for a TDS, the vertices are processed from 0 to n.
+    // Else, we make V[0] the infinite vertex and work from 1 to n+1.
+
+    typedef typename Tds::Vertex_handle  Vertex_handle;
+    typedef typename Tds::Cell_handle    Cell_handle;
+
+    typedef typename TDS_src::Vertex Vertex1;
+    typedef typename TDS_src::Cell Cell1;
+    clear();
+    cells().clear();
+
+    std::size_t n;
+    int d;
+    if(is_ascii(is))
+      is >> d >> n;
+    else {
+      read(is, d);
+      read(is, n);
+    }
+    if(!is) return is;
+    set_dimension(d);
+
+    std::size_t V_size = n;
+    std::vector< Vertex_handle > V(V_size);
+
+    // the infinite vertex is numbered 0
+    for (std::size_t i=0 ; i < V_size; ++i) {
+      Vertex1 v;
+      if(!(is >> v)) return is;
+      Vertex_handle vh=create_vertex( convert_vertex(v) );
+      V[i] = vh;
+      convert_vertex(v, *V[i]);
+    }
+
+    std::vector< Cell_handle > C;
+
+    std::size_t m;
+    read_cells(is, V, m, C);
+
+    for (std::size_t j=0 ; j < m; j++) {
+      Cell1 c;
+      if(!(is >> c)) return is;
+      convert_cell(c, *C[j]);
+    }
+
+    CGAL_triangulation_assertion(is_valid(false));
+    return is;
+  }
+
 private:
 
   // Change the orientation of the cell by swapping indices 0 and 1.
@@ -1596,6 +1687,7 @@ private:
   // counts but does not check
   bool count_cells(size_type &i, bool verbose = false, int level = 0) const;
   // counts AND checks the validity
+
 };
 
 #ifdef CGAL_TDS_USE_RECURSIVE_CREATE_STAR_3
diff -pruN 5.0.3-1/include/CGAL/Triangulation_data_structure.h 5.1-1/include/CGAL/Triangulation_data_structure.h
--- 5.0.3-1/include/CGAL/Triangulation_data_structure.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_data_structure.h	2020-09-08 12:15:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation/include/CGAL/Triangulation_data_structure.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation/include/CGAL/Triangulation_data_structure.h $
 // $Id: Triangulation_data_structure.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulation_data_structure_using_list_2.h 5.1-1/include/CGAL/Triangulation_data_structure_using_list_2.h
--- 5.0.3-1/include/CGAL/Triangulation_data_structure_using_list_2.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_data_structure_using_list_2.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/Triangulation_data_structure_using_list_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/Triangulation_data_structure_using_list_2.h $
 // $Id: Triangulation_data_structure_using_list_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulation_default_data_structure_2.h 5.1-1/include/CGAL/Triangulation_default_data_structure_2.h
--- 5.0.3-1/include/CGAL/Triangulation_default_data_structure_2.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_default_data_structure_2.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/Triangulation_default_data_structure_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/Triangulation_default_data_structure_2.h $
 // $Id: Triangulation_default_data_structure_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulation_ds_cell_base_3.h 5.1-1/include/CGAL/Triangulation_ds_cell_base_3.h
--- 5.0.3-1/include/CGAL/Triangulation_ds_cell_base_3.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_ds_cell_base_3.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/TDS_3/include/CGAL/Triangulation_ds_cell_base_3.h $
-// $Id: Triangulation_ds_cell_base_3.h fb83386 2020-05-12T16:58:32+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/TDS_3/include/CGAL/Triangulation_ds_cell_base_3.h $
+// $Id: Triangulation_ds_cell_base_3.h dcc4fb1 2020-05-20T09:46:26+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Monique Teillaud <Monique.Teillaud@sophia.inria.fr>
@@ -180,7 +180,7 @@ public:
 
   // For use by Compact_container.
   void * for_compact_container() const { return N[0].for_compact_container(); }
-  void * & for_compact_container()     { return N[0].for_compact_container(); }
+  void for_compact_container(void* p) { N[0].for_compact_container(p); }
 
   // TDS internal data access functions.
         TDS_data& tds_data()       { return _tds_data; }
diff -pruN 5.0.3-1/include/CGAL/Triangulation_ds_circulators_2.h 5.1-1/include/CGAL/Triangulation_ds_circulators_2.h
--- 5.0.3-1/include/CGAL/Triangulation_ds_circulators_2.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_ds_circulators_2.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/TDS_2/include/CGAL/Triangulation_ds_circulators_2.h $
-// $Id: Triangulation_ds_circulators_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/TDS_2/include/CGAL/Triangulation_ds_circulators_2.h $
+// $Id: Triangulation_ds_circulators_2.h 74d8922 2020-04-20T15:25:40+02:00 Guillaume Damiand
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -71,8 +71,8 @@ public:
   bool operator!=(const Face_handle &fh) const { return pos != fh; }
 
   bool is_empty() const;
-  bool operator==(std::nullptr_t CGAL_triangulation_assertion_code(n)) const;
-  bool operator!=(std::nullptr_t CGAL_triangulation_assertion_code(n)) const;
+  bool operator==(std::nullptr_t /*CGAL_triangulation_assertion_code(n)*/) const;
+  bool operator!=(std::nullptr_t /*CGAL_triangulation_assertion_code(n)*/) const;
 
   Face&
   operator*() const
@@ -152,8 +152,8 @@ public:
   { return pos->vertex(_ri) != vh; }
 
   bool is_empty() const;
-  bool operator==(std::nullptr_t CGAL_triangulation_assertion_code(n)) const;
-  bool operator!=(std::nullptr_t CGAL_triangulation_assertion_code(n)) const;
+  bool operator==(std::nullptr_t /*CGAL_triangulation_assertion_code(n)*/) const;
+  bool operator!=(std::nullptr_t /*CGAL_triangulation_assertion_code(n)*/) const;
 
   Vertex&
   operator*() const
@@ -231,8 +231,8 @@ public:
   bool operator==(const Edge_circulator &vc) const;
   bool operator!=(const Edge_circulator &vc) const;
   bool is_empty() const;
-  bool operator==(std::nullptr_t CGAL_triangulation_assertion_code(n)) const;
-  bool operator!=(std::nullptr_t CGAL_triangulation_assertion_code(n)) const;
+  bool operator==(std::nullptr_t /*CGAL_triangulation_assertion_code(n)*/) const;
+  bool operator!=(std::nullptr_t /*CGAL_triangulation_assertion_code(n)*/) const;
 
   Edge*  operator->() const {
     edge.first=pos;
@@ -338,18 +338,18 @@ return (_v == Vertex_handle() ||  pos ==
 template < class Tds >
 inline bool
 Triangulation_ds_face_circulator_2<Tds> ::
-operator==(std::nullptr_t CGAL_triangulation_assertion_code(n)) const
+operator==(std::nullptr_t /*CGAL_triangulation_assertion_code(n)*/) const
 {
-  CGAL_triangulation_assertion( n == nullptr);
+  //CGAL_triangulation_assertion( n == nullptr);
   return (_v == Vertex_handle() ||  pos == Face_handle() );
 }
 
 template < class Tds >
 inline bool
 Triangulation_ds_face_circulator_2<Tds> ::
-operator!=(std::nullptr_t CGAL_triangulation_assertion_code(n)) const
+operator!=(std::nullptr_t /*CGAL_triangulation_assertion_code(n)*/) const
 {
-  CGAL_triangulation_assertion( n == nullptr);
+  //CGAL_triangulation_assertion( n == nullptr);
   return ! (*this == nullptr);
 }
 
@@ -462,18 +462,18 @@ is_empty() const
 template < class Tds >
 inline bool
 Triangulation_ds_vertex_circulator_2<Tds> ::
-operator==(std::nullptr_t CGAL_triangulation_assertion_code(n)) const
+operator==(std::nullptr_t /*CGAL_triangulation_assertion_code(n)*/) const
 {
-  CGAL_triangulation_assertion( n == nullptr);
+  //CGAL_triangulation_assertion( n == nullptr);
   return (_v == Vertex_handle() || pos == Face_handle());
 }
 
 template < class Tds >
 inline bool
 Triangulation_ds_vertex_circulator_2<Tds> ::
-operator!=(std::nullptr_t CGAL_triangulation_assertion_code(n)) const
+operator!=(std::nullptr_t /*CGAL_triangulation_assertion_code(n)*/) const
 {
-  CGAL_triangulation_assertion( n == nullptr);
+  //CGAL_triangulation_assertion( n == nullptr);
   return !(*this == nullptr);
 }
 
@@ -584,18 +584,18 @@ is_empty() const
 template < class Tds >
 inline bool
 Triangulation_ds_edge_circulator_2<Tds> ::
-operator==(std::nullptr_t CGAL_triangulation_assertion_code(n)) const
+operator==(std::nullptr_t /*CGAL_triangulation_assertion_code(n)*/) const
 {
-  CGAL_triangulation_assertion( n == nullptr);
+  //CGAL_triangulation_assertion( n == nullptr);
   return (_v == Vertex_handle() || pos == Face_handle());
 }
 
 template < class Tds >
 inline bool
 Triangulation_ds_edge_circulator_2<Tds> ::
-operator!=(std::nullptr_t CGAL_triangulation_assertion_code(n)) const
+operator!=(std::nullptr_t /*CGAL_triangulation_assertion_code(n)*/) const
 {
-  CGAL_triangulation_assertion( n == nullptr);
+  //CGAL_triangulation_assertion( n == nullptr);
   return !(*this == nullptr);
 }
 
diff -pruN 5.0.3-1/include/CGAL/Triangulation_ds_face_2.h 5.1-1/include/CGAL/Triangulation_ds_face_2.h
--- 5.0.3-1/include/CGAL/Triangulation_ds_face_2.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_ds_face_2.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/TDS_2/include/CGAL/Triangulation_ds_face_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/TDS_2/include/CGAL/Triangulation_ds_face_2.h $
 // $Id: Triangulation_ds_face_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulation_ds_face_base_2.h 5.1-1/include/CGAL/Triangulation_ds_face_base_2.h
--- 5.0.3-1/include/CGAL/Triangulation_ds_face_base_2.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_ds_face_base_2.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/TDS_2/include/CGAL/Triangulation_ds_face_base_2.h $
-// $Id: Triangulation_ds_face_base_2.h dc12dea 2020-05-04T17:52:26+02:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/TDS_2/include/CGAL/Triangulation_ds_face_base_2.h $
+// $Id: Triangulation_ds_face_base_2.h dcc4fb1 2020-05-20T09:46:26+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -79,7 +79,7 @@ public:
 
    // For use by Compact_container.
   void * for_compact_container() const {return N[0].for_compact_container(); }
-  void * & for_compact_container()     { return N[0].for_compact_container();}
+  void for_compact_container(void* p) { N[0].for_compact_container(p);}
 
 
   static int ccw(int i) {return Triangulation_cw_ccw_2::ccw(i);}
diff -pruN 5.0.3-1/include/CGAL/Triangulation_ds_full_cell.h 5.1-1/include/CGAL/Triangulation_ds_full_cell.h
--- 5.0.3-1/include/CGAL/Triangulation_ds_full_cell.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_ds_full_cell.h	2020-09-08 12:15:49.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation/include/CGAL/Triangulation_ds_full_cell.h $
-// $Id: Triangulation_ds_full_cell.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation/include/CGAL/Triangulation_ds_full_cell.h $
+// $Id: Triangulation_ds_full_cell.h 378554e 2020-03-06T16:42:23+01:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)    : Samuel Hornus
@@ -204,7 +204,7 @@ public:
     TDS_data & tds_data() { return tds_data_; } /* Concept */
 
     void*   for_compact_container() const { return combinatorics_.for_compact_container(); }
-    void* & for_compact_container() { return combinatorics_.for_compact_container(); }
+    void    for_compact_container(void* p){ combinatorics_.for_compact_container(p); }
 
     bool is_valid(bool verbose = false, int = 0) const /* Concept */
     {
diff -pruN 5.0.3-1/include/CGAL/Triangulation_ds_iterators_2.h 5.1-1/include/CGAL/Triangulation_ds_iterators_2.h
--- 5.0.3-1/include/CGAL/Triangulation_ds_iterators_2.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_ds_iterators_2.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/TDS_2/include/CGAL/Triangulation_ds_iterators_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/TDS_2/include/CGAL/Triangulation_ds_iterators_2.h $
 // $Id: Triangulation_ds_iterators_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulation_ds_vertex_2.h 5.1-1/include/CGAL/Triangulation_ds_vertex_2.h
--- 5.0.3-1/include/CGAL/Triangulation_ds_vertex_2.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_ds_vertex_2.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/TDS_2/include/CGAL/Triangulation_ds_vertex_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/TDS_2/include/CGAL/Triangulation_ds_vertex_2.h $
 // $Id: Triangulation_ds_vertex_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulation_ds_vertex_base_2.h 5.1-1/include/CGAL/Triangulation_ds_vertex_base_2.h
--- 5.0.3-1/include/CGAL/Triangulation_ds_vertex_base_2.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_ds_vertex_base_2.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/TDS_2/include/CGAL/Triangulation_ds_vertex_base_2.h $
-// $Id: Triangulation_ds_vertex_base_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/TDS_2/include/CGAL/Triangulation_ds_vertex_base_2.h $
+// $Id: Triangulation_ds_vertex_base_2.h d1a323c 2020-03-26T19:24:14+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -49,7 +49,7 @@ public:
 
     // For use by the Compact_container.
   void *   for_compact_container() const { return _f.for_compact_container(); }
-  void * & for_compact_container()       { return _f.for_compact_container(); }
+  void for_compact_container(void* p) { _f.for_compact_container(p); }
 
 private:
   Face_handle _f;
diff -pruN 5.0.3-1/include/CGAL/Triangulation_ds_vertex_base_3.h 5.1-1/include/CGAL/Triangulation_ds_vertex_base_3.h
--- 5.0.3-1/include/CGAL/Triangulation_ds_vertex_base_3.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_ds_vertex_base_3.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/TDS_3/include/CGAL/Triangulation_ds_vertex_base_3.h $
-// $Id: Triangulation_ds_vertex_base_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/TDS_3/include/CGAL/Triangulation_ds_vertex_base_3.h $
+// $Id: Triangulation_ds_vertex_base_3.h d1a323c 2020-03-26T19:24:14+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Monique Teillaud <Monique.Teillaud@sophia.inria.fr>
@@ -59,8 +59,8 @@ public:
   // For use by the Compact_container.
   void *   for_compact_container() const
   { return _c.for_compact_container(); }
-  void * & for_compact_container()
-  { return _c.for_compact_container(); }
+  void for_compact_container(void* p)
+  { _c.for_compact_container(p); }
 
 private:
   Cell_handle _c;
diff -pruN 5.0.3-1/include/CGAL/Triangulation_ds_vertex.h 5.1-1/include/CGAL/Triangulation_ds_vertex.h
--- 5.0.3-1/include/CGAL/Triangulation_ds_vertex.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_ds_vertex.h	2020-09-08 12:15:49.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation/include/CGAL/Triangulation_ds_vertex.h $
-// $Id: Triangulation_ds_vertex.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation/include/CGAL/Triangulation_ds_vertex.h $
+// $Id: Triangulation_ds_vertex.h d1a323c 2020-03-26T19:24:14+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)    : Samuel Hornus
@@ -102,7 +102,7 @@ public:
 public: // FOR MEMORY MANAGEMENT
 
     void*   for_compact_container() const { return full_cell_.for_compact_container(); }
-    void* & for_compact_container()       { return full_cell_.for_compact_container(); }
+    void    for_compact_container(void *p){ full_cell_.for_compact_container(p); }
 
 };  // end of Triangulation_ds_vertex
 
diff -pruN 5.0.3-1/include/CGAL/Triangulation_face_base_2.h 5.1-1/include/CGAL/Triangulation_face_base_2.h
--- 5.0.3-1/include/CGAL/Triangulation_face_base_2.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_face_base_2.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/Triangulation_face_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/Triangulation_face_base_2.h $
 // $Id: Triangulation_face_base_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulation_face_base_with_id_2.h 5.1-1/include/CGAL/Triangulation_face_base_with_id_2.h
--- 5.0.3-1/include/CGAL/Triangulation_face_base_with_id_2.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_face_base_with_id_2.h	2020-09-08 12:15:50.000000000 +0000
@@ -2,8 +2,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/Triangulation_face_base_with_id_2.h $
-// $Id: Triangulation_face_base_with_id_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/Triangulation_face_base_with_id_2.h $
+// $Id: Triangulation_face_base_with_id_2.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -42,11 +42,15 @@ public:
     : Fb(v0, v1, v2, n0, n1, n2)
   { }
 
-  int& id() { return _id; }
-  int id() const { return _id; }
+  int& id() { return face_id; }
+  int id() const { return face_id; }
+
+  int& edge_id(const std::size_t i) { return edge_ids[i]; }
+  int edge_id(const std::size_t i) const { return edge_ids[i]; }
 
 private:
-  int _id;
+  int face_id;
+  std::array<int, 3> edge_ids;
 };
 
 } //namespace CGAL
diff -pruN 5.0.3-1/include/CGAL/Triangulation_face_base_with_info_2.h 5.1-1/include/CGAL/Triangulation_face_base_with_info_2.h
--- 5.0.3-1/include/CGAL/Triangulation_face_base_with_info_2.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_face_base_with_info_2.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/Triangulation_face_base_with_info_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/Triangulation_face_base_with_info_2.h $
 // $Id: Triangulation_face_base_with_info_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulation_face.h 5.1-1/include/CGAL/Triangulation_face.h
--- 5.0.3-1/include/CGAL/Triangulation_face.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_face.h	2020-09-08 12:15:49.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation/include/CGAL/Triangulation_face.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation/include/CGAL/Triangulation_face.h $
 // $Id: Triangulation_face.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulation_full_cell.h 5.1-1/include/CGAL/Triangulation_full_cell.h
--- 5.0.3-1/include/CGAL/Triangulation_full_cell.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_full_cell.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation/include/CGAL/Triangulation_full_cell.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation/include/CGAL/Triangulation_full_cell.h $
 // $Id: Triangulation_full_cell.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulation_geom_traits_3.h 5.1-1/include/CGAL/Triangulation_geom_traits_3.h
--- 5.0.3-1/include/CGAL/Triangulation_geom_traits_3.h	2020-08-01 19:02:59.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_geom_traits_3.h	2020-09-08 12:15:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_3/include/CGAL/Triangulation_geom_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_3/include/CGAL/Triangulation_geom_traits_3.h $
 // $Id: Triangulation_geom_traits_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulation.h 5.1-1/include/CGAL/Triangulation.h
--- 5.0.3-1/include/CGAL/Triangulation.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation.h	2020-09-08 12:15:49.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation/include/CGAL/Triangulation.h $
-// $Id: Triangulation.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation/include/CGAL/Triangulation.h $
+// $Id: Triangulation.h f8fd7a6 2020-04-18T13:53:46+02:00 Marc Glisse
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)    : Samuel Hornus
@@ -27,6 +27,7 @@
 #include <CGAL/Default.h>
 #include <CGAL/Random.h>
 
+#include <boost/container/small_vector.hpp>
 #include <boost/iterator/filter_iterator.hpp>
 #include <CGAL/boost/iterator/transform_iterator.hpp>
 
@@ -187,7 +188,6 @@ protected: // DATA MEMBERS
     Triangulation_ds                            tds_;
     const Geom_traits                           kernel_;
     Vertex_handle                               infinity_;
-    mutable std::vector<Oriented_side>          orientations_;
     mutable boost::optional<Flat_orientation_d> flat_orientation_;
     // The user can specify a Flat_orientation_d object to be used for
     // orienting simplices of a specific dimension
@@ -317,8 +317,6 @@ public:
             ++infinity_;
             ++inf2;
         }
-        // A full_cell has at most 1 + maximal_dimension() facets:
-        orientations_.resize(1 + maximal_dimension());
         // Our coaffine orientation predicates HAS state member variables
         reset_flat_orientation();
     }
@@ -613,8 +611,6 @@ public:
     {
         tds_.clear();
         infinity_ = tds().insert_increase_dimension();
-        // A full_cell has at most 1 + maximal_dimension() facets:
-        orientations_.resize(1 + maximal_dimension());
         // Our coaffine orientation predicates HAS state member variables
         reset_flat_orientation();
 #ifdef CGAL_TRIANGULATION_STATISTICS
@@ -1090,6 +1086,8 @@ Triangulation<TT, TDS>
     // of one |orientation| predicate
     Full_cell_handle previous = Full_cell_handle();
     bool full_cell_not_found = true;
+    // A full_cell has at most dimension + 1 facets:
+    boost::container::small_vector<signed char, 8> orientations_ (cur_dim + 1);
     while(full_cell_not_found) // we walk until we locate the query point |p|
     {
     #ifdef CGAL_TRIANGULATION_STATISTICS
diff -pruN 5.0.3-1/include/CGAL/Triangulation_hierarchy_2.h 5.1-1/include/CGAL/Triangulation_hierarchy_2.h
--- 5.0.3-1/include/CGAL/Triangulation_hierarchy_2.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_hierarchy_2.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/Triangulation_hierarchy_2.h $
-// $Id: Triangulation_hierarchy_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/Triangulation_hierarchy_2.h $
+// $Id: Triangulation_hierarchy_2.h dcc4fb1 2020-05-20T09:46:26+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -25,10 +25,9 @@
 #include <CGAL/spatial_sort.h>
 #include <CGAL/Spatial_sort_traits_adapter_2.h>
 
-#include <CGAL/internal/boost/function_property_map.hpp>
-
 #include <boost/mpl/identity.hpp>
 #include <boost/mpl/if.hpp>
+#include <boost/property_map/function_property_map.hpp>
 #include <boost/random/linear_congruential.hpp>
 #include <boost/random/geometric_distribution.hpp>
 #include <boost/random/variate_generator.hpp>
@@ -37,6 +36,7 @@
 #include <iostream>
 #include <map>
 #include <vector>
+#include <array>
 
 namespace CGAL {
 
@@ -83,13 +83,25 @@ public:
 
  private:
   // here is the stack of triangulations which form the hierarchy
-  Tr_Base*   hierarchy[Triangulation_hierarchy_2__maxlevel];
+  std::array<Tr_Base*,Triangulation_hierarchy_2__maxlevel> hierarchy;
   boost::rand48  random;
 
 public:
   Triangulation_hierarchy_2(const Geom_traits& traits = Geom_traits());
   Triangulation_hierarchy_2(const Triangulation_hierarchy_2& tr);
 
+  Triangulation_hierarchy_2(Triangulation_hierarchy_2&& other)
+    noexcept( noexcept(Tr_Base(std::move(other))) )
+    : Tr_Base(std::move(other))
+    , random(std::move(other.random))
+  {
+    hierarchy[0] = this;
+    for(int i=1; i<Triangulation_hierarchy_2__maxlevel; ++i) {
+      hierarchy[i] = other.hierarchy[i];
+      other.hierarchy[i] = nullptr;
+    }
+  }
+
   template<class InputIterator>
   Triangulation_hierarchy_2(InputIterator first, InputIterator beyond,
                             const Geom_traits& traits = Geom_traits())
@@ -103,6 +115,19 @@ public:
   }
 
   Triangulation_hierarchy_2 &operator=(const  Triangulation_hierarchy_2& tr);
+
+  Triangulation_hierarchy_2 & operator=(Triangulation_hierarchy_2&& other)
+    noexcept( noexcept(Triangulation_hierarchy_2(std::move(other))) )
+  {
+    static_cast<Tr_Base&>(*this) = std::move(other);
+    hierarchy[0] = this;
+    for(int i=1; i<Triangulation_hierarchy_2__maxlevel; ++i) {
+      hierarchy[i] = other.hierarchy[i];
+      other.hierarchy[i] = nullptr;
+    }
+    return *this;
+  }
+
   ~Triangulation_hierarchy_2();
 
   //Helping
@@ -130,12 +155,12 @@ public:
       // Spatial sort can only be used with Gt::Point_2: we need an adapter
       typedef typename Geom_traits::Construct_point_2 Construct_point_2;
       typedef typename boost::result_of<const Construct_point_2(const Point&)>::type Ret;
-      typedef CGAL::internal::boost_::function_property_map<Construct_point_2, Point, Ret> fpmap;
+      typedef boost::function_property_map<Construct_point_2, Point, Ret> fpmap;
       typedef CGAL::Spatial_sort_traits_adapter_2<Geom_traits, fpmap> Search_traits_2;
 
       spatial_sort(points.begin(), points.end(),
                    Search_traits_2(
-                     CGAL::internal::boost_::make_function_property_map<Point, Ret, Construct_point_2>(
+                     boost::make_function_property_map<Point, Ret, Construct_point_2>(
                        geom_traits().construct_point_2_object()), geom_traits()));
 
       // hints[i] is the face of the previously inserted point in level i.
@@ -391,8 +416,8 @@ void
 Triangulation_hierarchy_2<Tr_>::
 clear()
 {
-        for(int i=0;i<Triangulation_hierarchy_2__maxlevel;++i)
-        hierarchy[i]->clear();
+  for(int i=0;i<Triangulation_hierarchy_2__maxlevel;++i)
+    if(hierarchy[i]) hierarchy[i]->clear();
 }
 
 
@@ -724,7 +749,7 @@ locate_in_all(const Point& p,
     level--;
   }
 
-  for (int i=level+1; i<Triangulation_hierarchy_2__maxlevel;++i) pos[i]=0;
+  for (int i=level+1; i<Triangulation_hierarchy_2__maxlevel;++i) pos[i]=nullptr;
   while(level > 0) {
     pos[level]=position=hierarchy[level]->locate(p, position);
     // locate at that level from "position"
diff -pruN 5.0.3-1/include/CGAL/Triangulation_hierarchy_3.h 5.1-1/include/CGAL/Triangulation_hierarchy_3.h
--- 5.0.3-1/include/CGAL/Triangulation_hierarchy_3.h	2020-08-01 19:02:59.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_hierarchy_3.h	2020-09-08 12:15:52.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_3/include/CGAL/Triangulation_hierarchy_3.h $
-// $Id: Triangulation_hierarchy_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_3/include/CGAL/Triangulation_hierarchy_3.h $
+// $Id: Triangulation_hierarchy_3.h dcc4fb1 2020-05-20T09:46:26+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Olivier Devillers <Olivier.Devillers@sophia.inria.fr>
@@ -33,8 +33,7 @@
 #include <CGAL/Triangulation_hierarchy_vertex_base_3.h>
 #include <CGAL/Location_policy.h>
 
-#include <CGAL/internal/boost/function_property_map.hpp>
-
+#include <boost/property_map/function_property_map.hpp>
 #include <boost/random/linear_congruential.hpp>
 #include <boost/random/geometric_distribution.hpp>
 #include <boost/random/variate_generator.hpp>
@@ -51,6 +50,8 @@
 #include <boost/mpl/identity.hpp>
 #include <boost/mpl/if.hpp>
 
+#include <array>
+
 #endif //CGAL_TRIANGULATION_3_DONT_INSERT_RANGE_OF_POINTS_WITH_INFO
 
 namespace CGAL {
@@ -92,7 +93,7 @@ public:
 private:
 
   // here is the stack of triangulations which form the hierarchy
-  Tr_Base*       hierarchy[maxlevel];
+  std::array<Tr_Base*,maxlevel> hierarchy;
   boost::rand48  random;
 
   void set_up_down(Vertex_handle up, Vertex_handle down)
@@ -107,6 +108,18 @@ public:
 
   Triangulation_hierarchy_3(const Triangulation_hierarchy_3& tr);
 
+  Triangulation_hierarchy_3(Triangulation_hierarchy_3&& other)
+    noexcept( noexcept(Tr_Base(std::move(other))) )
+    : Tr_Base(std::move(other))
+    , random(std::move(other.random))
+  {
+    hierarchy[0] = this;
+    for(int i=1; i<maxlevel; ++i) {
+      hierarchy[i] = other.hierarchy[i];
+      other.hierarchy[i] = nullptr;
+    }
+  }
+
   template < typename InputIterator >
   Triangulation_hierarchy_3(InputIterator first, InputIterator last,
                             const Geom_traits& traits = Geom_traits())
@@ -125,9 +138,32 @@ public:
     return *this;
   }
 
-  ~Triangulation_hierarchy_3();
+  Triangulation_hierarchy_3 & operator=(Triangulation_hierarchy_3&& other)
+    noexcept( noexcept(Triangulation_hierarchy_3(std::move(other))) )
+  {
+    static_cast<Tr_Base&>(*this) = std::move(other);
+    hierarchy[0] = this;
+    for(int i=1; i<maxlevel; ++i) {
+      hierarchy[i] = other.hierarchy[i];
+      other.hierarchy[i] = nullptr;
+    }
+    return *this;
+  }
 
-  void swap(Triangulation_hierarchy_3 &tr);
+  ~Triangulation_hierarchy_3()
+  {
+    clear();
+    for(int i=1; i<maxlevel; ++i) {
+      delete hierarchy[i];
+    }
+  };
+
+  void swap(Triangulation_hierarchy_3 &tr)
+  {
+    Tr_Base::swap(tr);
+    for(int i=1; i<maxlevel; ++i)
+      std::swap(hierarchy[i], tr.hierarchy[i]);
+  };
 
   void clear();
 
@@ -169,12 +205,12 @@ public:
       // Spatial sort can only be used with Geom_traits::Point_3: we need an adapter
       typedef typename Geom_traits::Construct_point_3 Construct_point_3;
       typedef typename boost::result_of<const Construct_point_3(const Point&)>::type Ret;
-      typedef CGAL::internal::boost_::function_property_map<Construct_point_3, Point, Ret> fpmap;
+      typedef boost::function_property_map<Construct_point_3, Point, Ret> fpmap;
       typedef CGAL::Spatial_sort_traits_adapter_3<Geom_traits, fpmap> Search_traits_3;
 
       spatial_sort(points.begin(), points.end(),
                    Search_traits_3(
-                     CGAL::internal::boost_::make_function_property_map<Point, Ret, Construct_point_3>(
+                     boost::make_function_property_map<Point, Ret, Construct_point_3>(
                        geom_traits().construct_point_3_object()), geom_traits()));
 
       // hints[i] is the vertex of the previously inserted point in level i.
@@ -246,13 +282,13 @@ private:
     typedef Index_to_Bare_point<Construct_point_3,
                                 std::vector<Point> > Access_bare_point;
     typedef typename boost::result_of<const Construct_point_3(const Point&)>::type Ret;
-    typedef CGAL::internal::boost_::function_property_map<Access_bare_point, std::size_t, Ret> fpmap;
+    typedef boost::function_property_map<Access_bare_point, std::size_t, Ret> fpmap;
     typedef CGAL::Spatial_sort_traits_adapter_3<Geom_traits, fpmap> Search_traits_3;
 
     Access_bare_point accessor(points, geom_traits().construct_point_3_object());
     spatial_sort(indices.begin(), indices.end(),
                  Search_traits_3(
-                   CGAL::internal::boost_::make_function_property_map<
+                   boost::make_function_property_map<
                      std::size_t, Ret, Access_bare_point>(accessor),
                    geom_traits()));
 
@@ -462,26 +498,6 @@ Triangulation_hierarchy_3(const Triangul
   }
 }
 
-template <class Tr>
-void
-Triangulation_hierarchy_3<Tr>::
-swap(Triangulation_hierarchy_3<Tr> &tr)
-{
-  Tr_Base::swap(tr);
-  for(int i=1; i<maxlevel; ++i)
-      std::swap(hierarchy[i], tr.hierarchy[i]);
-}
-
-template <class Tr>
-Triangulation_hierarchy_3<Tr>::
-~Triangulation_hierarchy_3()
-{
-  clear();
-  for(int i=1; i<maxlevel; ++i) {
-    delete hierarchy[i];
-  }
-}
-
 template <class Tr>
 void
 Triangulation_hierarchy_3<Tr>::
diff -pruN 5.0.3-1/include/CGAL/Triangulation_hierarchy_vertex_base_2.h 5.1-1/include/CGAL/Triangulation_hierarchy_vertex_base_2.h
--- 5.0.3-1/include/CGAL/Triangulation_hierarchy_vertex_base_2.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_hierarchy_vertex_base_2.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/Triangulation_hierarchy_vertex_base_2.h $
-// $Id: Triangulation_hierarchy_vertex_base_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/Triangulation_hierarchy_vertex_base_2.h $
+// $Id: Triangulation_hierarchy_vertex_base_2.h d1a323c 2020-03-26T19:24:14+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -41,13 +41,13 @@ public:
   };
 
   Triangulation_hierarchy_vertex_base_2()
-    : Base(), _up(0), _down(0)
+    : Base()
     {}
   Triangulation_hierarchy_vertex_base_2(const Point & p, Face_handle f)
-    : Base(p,f), _up(0), _down(0)
+    : Base(p,f)
     {}
   Triangulation_hierarchy_vertex_base_2(const Point & p)
-    : Base(p), _up(0), _down(0)
+    : Base(p)
     {}
 
   Vertex_handle up() {return _up;}
@@ -57,8 +57,8 @@ public:
 
 
  private:
-  Vertex_handle  _up;    // same vertex one level above
-  Vertex_handle  _down;  // same vertex one level below
+  Vertex_handle  _up   = nullptr;    // same vertex one level above
+  Vertex_handle  _down = nullptr;  // same vertex one level below
 };
 
 } //namespace CGAL
diff -pruN 5.0.3-1/include/CGAL/Triangulation_hierarchy_vertex_base_3.h 5.1-1/include/CGAL/Triangulation_hierarchy_vertex_base_3.h
--- 5.0.3-1/include/CGAL/Triangulation_hierarchy_vertex_base_3.h	2020-08-01 19:02:59.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_hierarchy_vertex_base_3.h	2020-09-08 12:15:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_3/include/CGAL/Triangulation_hierarchy_vertex_base_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_3/include/CGAL/Triangulation_hierarchy_vertex_base_3.h $
 // $Id: Triangulation_hierarchy_vertex_base_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulation_incremental_builder_3.h 5.1-1/include/CGAL/Triangulation_incremental_builder_3.h
--- 5.0.3-1/include/CGAL/Triangulation_incremental_builder_3.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_incremental_builder_3.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Skin_surface_3/include/CGAL/Triangulation_incremental_builder_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Skin_surface_3/include/CGAL/Triangulation_incremental_builder_3.h $
 // $Id: Triangulation_incremental_builder_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulation_simplex_3.h 5.1-1/include/CGAL/Triangulation_simplex_3.h
--- 5.0.3-1/include/CGAL/Triangulation_simplex_3.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_simplex_3.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/TDS_3/include/CGAL/Triangulation_simplex_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/TDS_3/include/CGAL/Triangulation_simplex_3.h $
 // $Id: Triangulation_simplex_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulation_structural_filtering_traits.h 5.1-1/include/CGAL/Triangulation_structural_filtering_traits.h
--- 5.0.3-1/include/CGAL/Triangulation_structural_filtering_traits.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_structural_filtering_traits.h	2020-09-08 12:15:01.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Triangulation_structural_filtering_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Triangulation_structural_filtering_traits.h $
 // $Id: Triangulation_structural_filtering_traits.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulation_utils_2.h 5.1-1/include/CGAL/Triangulation_utils_2.h
--- 5.0.3-1/include/CGAL/Triangulation_utils_2.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_utils_2.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/TDS_2/include/CGAL/Triangulation_utils_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/TDS_2/include/CGAL/Triangulation_utils_2.h $
 // $Id: Triangulation_utils_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulation_utils_3.h 5.1-1/include/CGAL/Triangulation_utils_3.h
--- 5.0.3-1/include/CGAL/Triangulation_utils_3.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_utils_3.h	2020-09-08 12:15:48.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/TDS_3/include/CGAL/Triangulation_utils_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/TDS_3/include/CGAL/Triangulation_utils_3.h $
 // $Id: Triangulation_utils_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulation_vertex_base_2.h 5.1-1/include/CGAL/Triangulation_vertex_base_2.h
--- 5.0.3-1/include/CGAL/Triangulation_vertex_base_2.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_vertex_base_2.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/Triangulation_vertex_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/Triangulation_vertex_base_2.h $
 // $Id: Triangulation_vertex_base_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulation_vertex_base_3.h 5.1-1/include/CGAL/Triangulation_vertex_base_3.h
--- 5.0.3-1/include/CGAL/Triangulation_vertex_base_3.h	2020-08-01 19:02:59.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_vertex_base_3.h	2020-09-08 12:15:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_3/include/CGAL/Triangulation_vertex_base_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_3/include/CGAL/Triangulation_vertex_base_3.h $
 // $Id: Triangulation_vertex_base_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulation_vertex_base_with_id_2.h 5.1-1/include/CGAL/Triangulation_vertex_base_with_id_2.h
--- 5.0.3-1/include/CGAL/Triangulation_vertex_base_with_id_2.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_vertex_base_with_id_2.h	2020-09-08 12:15:50.000000000 +0000
@@ -2,7 +2,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/Triangulation_vertex_base_with_id_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/Triangulation_vertex_base_with_id_2.h $
 // $Id: Triangulation_vertex_base_with_id_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulation_vertex_base_with_info_2.h 5.1-1/include/CGAL/Triangulation_vertex_base_with_info_2.h
--- 5.0.3-1/include/CGAL/Triangulation_vertex_base_with_info_2.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_vertex_base_with_info_2.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_2/include/CGAL/Triangulation_vertex_base_with_info_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_2/include/CGAL/Triangulation_vertex_base_with_info_2.h $
 // $Id: Triangulation_vertex_base_with_info_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulation_vertex_base_with_info_3.h 5.1-1/include/CGAL/Triangulation_vertex_base_with_info_3.h
--- 5.0.3-1/include/CGAL/Triangulation_vertex_base_with_info_3.h	2020-08-01 19:02:59.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_vertex_base_with_info_3.h	2020-09-08 12:15:52.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation_3/include/CGAL/Triangulation_vertex_base_with_info_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation_3/include/CGAL/Triangulation_vertex_base_with_info_3.h $
 // $Id: Triangulation_vertex_base_with_info_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Triangulation_vertex.h 5.1-1/include/CGAL/Triangulation_vertex.h
--- 5.0.3-1/include/CGAL/Triangulation_vertex.h	2020-08-01 19:02:58.000000000 +0000
+++ 5.1-1/include/CGAL/Triangulation_vertex.h	2020-09-08 12:15:50.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Triangulation/include/CGAL/Triangulation_vertex.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Triangulation/include/CGAL/Triangulation_vertex.h $
 // $Id: Triangulation_vertex.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/tss.h 5.1-1/include/CGAL/tss.h
--- 5.0.3-1/include/CGAL/tss.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/tss.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/tss.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/tss.h $
 // $Id: tss.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 
diff -pruN 5.0.3-1/include/CGAL/tuple.h 5.1-1/include/CGAL/tuple.h
--- 5.0.3-1/include/CGAL/tuple.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/tuple.h	2020-09-08 12:15:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/tuple.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/tuple.h $
 // $Id: tuple.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Twotuple.h 5.1-1/include/CGAL/Twotuple.h
--- 5.0.3-1/include/CGAL/Twotuple.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Twotuple.h	2020-09-08 12:15:32.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Twotuple.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Twotuple.h $
 // $Id: Twotuple.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/typeset.h 5.1-1/include/CGAL/typeset.h
--- 5.0.3-1/include/CGAL/typeset.h	2020-08-01 19:02:35.000000000 +0000
+++ 5.1-1/include/CGAL/typeset.h	2020-09-08 12:15:15.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/NewKernel_d/include/CGAL/typeset.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/NewKernel_d/include/CGAL/typeset.h $
 // $Id: typeset.h 822bc55 2020-03-27T08:28:48+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/type_traits.h 5.1-1/include/CGAL/type_traits.h
--- 5.0.3-1/include/CGAL/type_traits.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/type_traits.h	2020-09-08 12:15:33.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/type_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/type_traits.h $
 // $Id: type_traits.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Umbilics.h 5.1-1/include/CGAL/Umbilics.h
--- 5.0.3-1/include/CGAL/Umbilics.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Umbilics.h	2020-09-08 12:15:31.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Ridges_3/include/CGAL/Umbilics.h $
-// $Id: Umbilics.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Ridges_3/include/CGAL/Umbilics.h $
+// $Id: Umbilics.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Marc Pouget and Frédéric Cazals
@@ -13,16 +13,18 @@
 
 #include <CGAL/license/Ridges_3.h>
 
-
-#include <list>
-#include <vector>
-#include <math.h>
 #include <CGAL/basic.h>
 #include <CGAL/boost/graph/helpers.h>
+#include <CGAL/number_type_config.h>
 #include <CGAL/PolyhedralSurf_neighbors.h>
 #include <CGAL/Kernel/global_functions_3.h>
+
 #include <boost/shared_ptr.hpp>
 
+#include <list>
+#include <vector>
+#include <math.h>
+
 namespace CGAL {
 
 enum Umbilic_type { NON_GENERIC_UMBILIC = 0, ELLIPTIC_UMBILIC, HYPERBOLIC_UMBILIC};
@@ -230,7 +232,6 @@ compute_type(Umbilic& umb)
 {
   Vector_3 dir, dirnext, normal;
   double cosinus, angle=0, angleSum=0;
-  const double  pi=3.141592653589793;
   vertex_descriptor v;
   typename std::list<halfedge_descriptor>::const_iterator itb = umb.contour_list().begin(),
     itlast = --umb.contour_list().end();
@@ -266,9 +267,9 @@ compute_type(Umbilic& umb)
   else angle = -acos(cosinus);
   angleSum += angle;
 
-  if ((angleSum > (pi/2)) && (angleSum < (3*pi/2))) umb.umbilic_type() = HYPERBOLIC_UMBILIC ;
-  else if ((angleSum < (-pi/2)) && (angleSum > (-3*pi/2))) umb.umbilic_type() = ELLIPTIC_UMBILIC;
-  else if ((angleSum <= (pi/2)) && (angleSum >= (-pi/2))) return 0;//is not considered as an umbilic
+  if ((angleSum > (CGAL_PI/2)) && (angleSum < (3*CGAL_PI/2))) umb.umbilic_type() = HYPERBOLIC_UMBILIC ;
+  else if ((angleSum < (-CGAL_PI/2)) && (angleSum > (-3*CGAL_PI/2))) umb.umbilic_type() = ELLIPTIC_UMBILIC;
+  else if ((angleSum <= (CGAL_PI/2)) && (angleSum >= (-CGAL_PI/2))) return 0;//is not considered as an umbilic
   else umb.umbilic_type() = NON_GENERIC_UMBILIC;
   return 1;
 }
diff -pruN 5.0.3-1/include/CGAL/Uncertain.h 5.1-1/include/CGAL/Uncertain.h
--- 5.0.3-1/include/CGAL/Uncertain.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Uncertain.h	2020-09-08 12:15:32.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/Uncertain.h $
-// $Id: Uncertain.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/Uncertain.h $
+// $Id: Uncertain.h 3302086 2020-06-13T23:41:39+05:30 Abhay Raj Singh
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Sylvain Pion
@@ -66,8 +66,6 @@ class Uncertain_conversion_exception
 public:
   Uncertain_conversion_exception(const std::string &s)
     : std::range_error(s) {}
-
-  ~Uncertain_conversion_exception() throw() {}
 };
 
 
diff -pruN 5.0.3-1/include/CGAL/Unfiltered_predicate_adaptor.h 5.1-1/include/CGAL/Unfiltered_predicate_adaptor.h
--- 5.0.3-1/include/CGAL/Unfiltered_predicate_adaptor.h	2020-08-01 19:02:53.000000000 +0000
+++ 5.1-1/include/CGAL/Unfiltered_predicate_adaptor.h	2020-09-08 12:15:41.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Straight_skeleton_2/include/CGAL/Unfiltered_predicate_adaptor.h $
-// $Id: Unfiltered_predicate_adaptor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Straight_skeleton_2/include/CGAL/Unfiltered_predicate_adaptor.h $
+// $Id: Unfiltered_predicate_adaptor.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Sylvain Pion, Fernando Cacciola <fernando_cacciola@ciudad.com.ar>
@@ -45,111 +45,14 @@ public:
     : Certified_approx_predicate(o1, o2)
   {}
 
-  template <class A1>
+  template <class ... A>
   result_type
-  operator()(const A1 &a1) const
+  operator()(A&& ... a) const
 #ifndef CGAL_CFG_OUTOFLINE_TEMPLATE_MEMBER_DEFINITION_BUG
   ;
 #else
   {
-    return static_cast<result_type>(Certified_approx_predicate(a1));
-  }
-#endif
-
-  template <class A1, class A2>
-  result_type
-  operator()(const A1 &a1, const A2 &a2) const
-#ifndef CGAL_CFG_OUTOFLINE_TEMPLATE_MEMBER_DEFINITION_BUG
-  ;
-#else
-  {
-    return static_cast<result_type>(Certified_approx_predicate(a1, a2));
-  }
-#endif
-
-  template <class A1, class A2, class A3>
-  result_type
-  operator()(const A1 &a1, const A2 &a2, const A3 &a3) const
-#ifndef CGAL_CFG_OUTOFLINE_TEMPLATE_MEMBER_DEFINITION_BUG
-  ;
-#else
-  {
-    return static_cast<result_type>(Certified_approx_predicate(a1, a2, a3));
-  }
-#endif
-
-  template <class A1, class A2, class A3, class A4>
-  result_type
-  operator()(const A1 &a1, const A2 &a2, const A3 &a3, const A4 &a4) const
-#ifndef CGAL_CFG_OUTOFLINE_TEMPLATE_MEMBER_DEFINITION_BUG
-  ;
-#else
-  {
-    return static_cast<result_type>(Certified_approx_predicate(a1, a2, a3,a4));
-  }
-#endif
-
-  template <class A1, class A2, class A3, class A4, class A5>
-  result_type
-  operator()(const A1 &a1, const A2 &a2, const A3 &a3, const A4 &a4,
-             const A5 &a5) const
-#ifndef CGAL_CFG_OUTOFLINE_TEMPLATE_MEMBER_DEFINITION_BUG
-  ;
-#else
-  {
-    return static_cast<result_type>(Certified_approx_predicate(a1, a2, a3, a4, a5));
-  }
-#endif
-
-  template <class A1, class A2, class A3, class A4, class A5, class A6>
-  result_type
-  operator()(const A1 &a1, const A2 &a2, const A3 &a3, const A4 &a4,
-             const A5 &a5, const A6 &a6) const
-#ifndef CGAL_CFG_OUTOFLINE_TEMPLATE_MEMBER_DEFINITION_BUG
-  ;
-#else
-  {
-    return static_cast<result_type>(Certified_approx_predicate(a1, a2, a3, a4, a5, a6));
-  }
-#endif
-
-  template <class A1, class A2, class A3, class A4, class A5, class A6,
-            class A7>
-  result_type
-  operator()(const A1 &a1, const A2 &a2, const A3 &a3, const A4 &a4,
-             const A5 &a5, const A6 &a6, const A7 &a7) const
-#ifndef CGAL_CFG_OUTOFLINE_TEMPLATE_MEMBER_DEFINITION_BUG
-  ;
-#else
-  {
-    return static_cast<result_type>(Certified_approx_predicate(a1, a2, a3, a4, a5, a6, a7));
-  }
-#endif
-
-  template <class A1, class A2, class A3, class A4, class A5, class A6,
-            class A7, class A8>
-  result_type
-  operator()(const A1 &a1, const A2 &a2, const A3 &a3, const A4 &a4,
-             const A5 &a5, const A6 &a6, const A7 &a7, const A8 &a8) const
-#ifndef CGAL_CFG_OUTOFLINE_TEMPLATE_MEMBER_DEFINITION_BUG
-  ;
-#else
-  {
-    return static_cast<result_type>(Certified_approx_predicate(a1, a2, a3, a4, a5, a6, a7, a8));
-  }
-#endif
-
-  template <class A1, class A2, class A3, class A4, class A5, class A6,
-            class A7, class A8, class A9>
-  result_type
-  operator()(const A1 &a1, const A2 &a2, const A3 &a3, const A4 &a4,
-             const A5 &a5, const A6 &a6, const A7 &a7, const A8 &a8,
-             const A9 &a9 ) const
-#ifndef CGAL_CFG_OUTOFLINE_TEMPLATE_MEMBER_DEFINITION_BUG
-  ;
-#else
-  {
-    return static_cast<result_type>(Certified_approx_predicate(a1, a2, a3, a4, a5, a6, a7, a8, a9));
+    return static_cast<result_type>(Certified_approx_predicate(std::forward<A>(a)...));
   }
 #endif
 
@@ -158,90 +61,12 @@ public:
 
 #ifndef CGAL_CFG_OUTOFLINE_TEMPLATE_MEMBER_DEFINITION_BUG
 template <class CAP>
-  template <class A1>
-typename Unfiltered_predicate_adaptor<CAP>::result_type
-Unfiltered_predicate_adaptor<CAP>::
-  operator()(const A1 &a1) const
-{
-  return static_cast<result_type>(Certified_approx_predicate(a1));
-}
-
-template <class CAP>
-  template <class A1, class A2>
-typename Unfiltered_predicate_adaptor<CAP>::result_type
-Unfiltered_predicate_adaptor<CAP>::
-  operator()(const A1 &a1, const A2 &a2) const
-{
-  return static_cast<result_type>(Certified_approx_predicate(a1, a2));
-}
-
-template <class CAP>
-  template <class A1, class A2, class A3>
-typename Unfiltered_predicate_adaptor<CAP>::result_type
-Unfiltered_predicate_adaptor<CAP>::
-  operator()(const A1 &a1, const A2 &a2, const A3 &a3) const
-{
-  return static_cast<result_type>(Certified_approx_predicate(a1, a2, a3));
-}
-
-template <class CAP>
-  template <class A1, class A2, class A3, class A4>
-typename Unfiltered_predicate_adaptor<CAP>::result_type
-Unfiltered_predicate_adaptor<CAP>::
-  operator()(const A1 &a1, const A2 &a2, const A3 &a3, const A4 &a4) const
-{
-  return static_cast<result_type>(Certified_approx_predicate(a1, a2, a3, a4));
-}
-
-template <class CAP>
-  template <class A1, class A2, class A3, class A4, class A5>
-typename Unfiltered_predicate_adaptor<CAP>::result_type
-Unfiltered_predicate_adaptor<CAP>::
-  operator()(const A1 &a1, const A2 &a2, const A3 &a3, const A4 &a4,
-             const A5 &a5) const
-{
-  return static_cast<result_type>(Certified_approx_predicate(a1, a2, a3, a4, a5));
-}
-
-template <class CAP>
-  template <class A1, class A2, class A3, class A4, class A5, class A6>
-typename Unfiltered_predicate_adaptor<CAP>::result_type
-Unfiltered_predicate_adaptor<CAP>::
-  operator()(const A1 &a1, const A2 &a2, const A3 &a3, const A4 &a4,
-             const A5 &a5, const A6 &a6) const
-{
-  return static_cast<result_type>(Certified_approx_predicate(a1, a2, a3, a4, a5, a6));
-}
-
-template <class CAP>
-  template <class A1, class A2, class A3, class A4, class A5, class A6, class A7>
-typename Unfiltered_predicate_adaptor<CAP>::result_type
-Unfiltered_predicate_adaptor<CAP>::
-  operator()(const A1 &a1, const A2 &a2, const A3 &a3, const A4 &a4,
-             const A5 &a5, const A6 &a6, const A7 &a7 ) const
-{
-  return static_cast<result_type>(Certified_approx_predicate(a1, a2, a3, a4, a5, a6, a7));
-}
-
-template <class CAP>
-  template <class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8>
-typename Unfiltered_predicate_adaptor<CAP>::result_type
-Unfiltered_predicate_adaptor<CAP>::
-  operator()(const A1 &a1, const A2 &a2, const A3 &a3, const A4 &a4,
-             const A5 &a5, const A6 &a6, const A7 &a7, const A8 &a8 ) const
-{
-  return static_cast<result_type>(Certified_approx_predicate(a1, a2, a3, a4, a5, a6, a7, a8));
-}
-
-template <class CAP>
-  template <class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8, class A9>
+  template <class ... A>
 typename Unfiltered_predicate_adaptor<CAP>::result_type
 Unfiltered_predicate_adaptor<CAP>::
-  operator()(const A1 &a1, const A2 &a2, const A3 &a3, const A4 &a4,
-             const A5 &a5, const A6 &a6, const A7 &a7, const A8 &a8,
-             const A9 &a9 ) const
+  operator()(A&& ... a) const
 {
-  return static_cast<result_type>(Certified_approx_predicate(a1, a2, a3, a4, a5, a6, a7, a8, a9));
+  return static_cast<result_type>(Certified_approx_predicate(std::forward<A>(a)...));
 }
 #endif
 
diff -pruN 5.0.3-1/include/CGAL/Union_find.h 5.1-1/include/CGAL/Union_find.h
--- 5.0.3-1/include/CGAL/Union_find.h	2020-08-01 19:02:59.000000000 +0000
+++ 5.1-1/include/CGAL/Union_find.h	2020-09-08 12:15:52.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Union_find/include/CGAL/Union_find.h $
-// $Id: Union_find.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Union_find/include/CGAL/Union_find.h $
+// $Id: Union_find.h fcd4e57 2020-05-22T11:29:03+02:00 Marc Glisse
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -114,8 +114,7 @@ public:
 #ifdef _MSC_VER
     typedef CGAL_ALLOCATOR(Union_find_struct)                allocator;
 #else
-    typedef typename A::template rebind<Union_find_struct>   Rebind;
-    typedef typename Rebind::other                           allocator;
+    typedef typename std::allocator_traits<A>::template rebind_alloc<Union_find_struct> allocator;
 #endif
 
 private:
diff -pruN 5.0.3-1/include/CGAL/Union_of_balls_3.h 5.1-1/include/CGAL/Union_of_balls_3.h
--- 5.0.3-1/include/CGAL/Union_of_balls_3.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Union_of_balls_3.h	2020-09-08 12:15:38.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Skin_surface_3/include/CGAL/Union_of_balls_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Skin_surface_3/include/CGAL/Union_of_balls_3.h $
 // $Id: Union_of_balls_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Unique_hash_map.h 5.1-1/include/CGAL/Unique_hash_map.h
--- 5.0.3-1/include/CGAL/Unique_hash_map.h	2020-08-01 19:02:22.000000000 +0000
+++ 5.1-1/include/CGAL/Unique_hash_map.h	2020-09-08 12:14:53.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Hash_map/include/CGAL/Unique_hash_map.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Hash_map/include/CGAL/Unique_hash_map.h $
 // $Id: Unique_hash_map.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/use.h 5.1-1/include/CGAL/use.h
--- 5.0.3-1/include/CGAL/use.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/use.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/use.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/use.h $
 // $Id: use.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/user_classes.h 5.1-1/include/CGAL/user_classes.h
--- 5.0.3-1/include/CGAL/user_classes.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/user_classes.h	2020-09-08 12:15:02.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/user_classes.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/user_classes.h $
 // $Id: user_classes.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/utility.h 5.1-1/include/CGAL/utility.h
--- 5.0.3-1/include/CGAL/utility.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/utility.h	2020-09-08 12:15:33.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/utility.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/utility.h $
 // $Id: utility.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/utils_classes.h 5.1-1/include/CGAL/utils_classes.h
--- 5.0.3-1/include/CGAL/utils_classes.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/utils_classes.h	2020-09-08 12:15:17.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/utils_classes.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/utils_classes.h $
 // $Id: utils_classes.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/utils.h 5.1-1/include/CGAL/utils.h
--- 5.0.3-1/include/CGAL/utils.h	2020-08-01 19:02:36.000000000 +0000
+++ 5.1-1/include/CGAL/utils.h	2020-09-08 12:15:17.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Number_types/include/CGAL/utils.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Number_types/include/CGAL/utils.h $
 // $Id: utils.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/value_type_traits.h 5.1-1/include/CGAL/value_type_traits.h
--- 5.0.3-1/include/CGAL/value_type_traits.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/value_type_traits.h	2020-09-08 12:15:33.000000000 +0000
@@ -4,7 +4,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/value_type_traits.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/value_type_traits.h $
 // $Id: value_type_traits.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Variational_shape_approximation.h 5.1-1/include/CGAL/Variational_shape_approximation.h
--- 5.0.3-1/include/CGAL/Variational_shape_approximation.h	2020-08-01 19:02:54.000000000 +0000
+++ 5.1-1/include/CGAL/Variational_shape_approximation.h	2020-09-08 12:15:43.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesh_approximation/include/CGAL/Variational_shape_approximation.h $
-// $Id: Variational_shape_approximation.h 5b2d46c 2020-04-21T10:03:00+02:00 Maxime Gimeno
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesh_approximation/include/CGAL/Variational_shape_approximation.h $
+// $Id: Variational_shape_approximation.h 038869c 2020-05-12T15:12:21+02:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -295,20 +295,36 @@ public:
    * @brief initializes the seeds with both maximum number of proxies and minimum error drop stop criteria.
    * The first criterion met stops the seeding.
    * Parameters out of range are ignored.
-   * @tparam NamedParameters a sequence of \ref vsa_namedparameters
+   * @tparam NamedParameters a sequence of \ref bgl_namedparameters
 
-   * @param np optional sequence of \ref vsa_namedparameters among the ones listed below
+   * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
    * @return number of proxies initialized
 
    * \cgalNamedParamsBegin{Seeding Named Parameters}
-   *  \cgalParamBegin{seeding_method} selection of seeding method.
-   *  \cgalParamEnd
-   *  \cgalParamBegin{max_number_of_proxies} maximum number of proxies to approximate the input mesh.
-   *  \cgalParamEnd
-   *  \cgalParamBegin{min_error_drop} minimum error drop of the approximation, expressed in ratio between two iterations of proxy addition.
-   *  \cgalParamEnd
-   *  \cgalParamBegin{number_of_relaxations} number of relaxation iterations interleaved within seeding.
-   *  \cgalParamEnd
+   *   \cgalParamNBegin{seeding_method}
+   *     \cgalParamDescription{the selection of seeding method}
+   *     \cgalParamType{`CGAL::Surface_mesh_approximation::Seeding_method`}
+   *     \cgalParamDefault{`CGAL::Surface_mesh_approximation::HIERARCHICAL`}
+   *   \cgalParamNEnd
+   *
+   *   \cgalParamNBegin{max_number_of_proxies}
+   *     \cgalParamDescription{the maximum number of proxies used to approximate the input mesh}
+   *     \cgalParamType{`std::size_t`}
+   *     \cgalParamDefault{`num_faces(tm) / 3`, used when `min_error_drop` is also not provided}
+   *   \cgalParamNEnd
+   *
+   *   \cgalParamNBegin{min_error_drop}
+   *     \cgalParamDescription{the minimum error drop of the approximation, expressed as
+   *                           the ratio between two iterations of proxy addition}
+   *     \cgalParamType{`geom_traits::FT`}
+   *     \cgalParamDefault{`0.1`, used when `max_number_of_proxies` is also not provided}
+   *   \cgalParamNEnd
+   *
+   *   \cgalParamNBegin{number_of_relaxations}
+   *     \cgalParamDescription{the number of relaxation iterations interleaved within seeding}
+   *     \cgalParamType{`std::size_t`}
+   *     \cgalParamDefault{`5`}
+   *   \cgalParamNEnd
    * \cgalNamedParamsEnd
    */
   template <typename NamedParameters>
@@ -767,23 +783,42 @@ public:
   /// @{
   /*!
    * @brief extracts the output mesh in the form of an indexed triangle set.
-   * @tparam NamedParameters a sequence of \ref vsa_namedparameters
+   * @tparam NamedParameters a sequence of \ref bgl_namedparameters
    *
-   * @param np optional sequence of \ref vsa_namedparameters among the ones listed below
+   * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
    * @return `true` if the extracted surface mesh is manifold, and `false` otherwise.
    *
    * \cgalNamedParamsBegin{Meshing Named Parameters}
-   *   \cgalParamBegin{subdivision_ratio} chord subdivision ratio threshold to the chord length or average edge length.
-   *   \cgalParamEnd
-   *   \cgalParamBegin{relative_to_chord} set `true` if the subdivision_ratio is the ratio of the
-   *     furthest vertex distance to the chord length, or to the average edge length otherwise.
-   *   \cgalParamEnd
-   *   \cgalParamBegin{with_dihedral_angle}  set `true` if subdivision_ratio is weighted by dihedral angle.
-   *   \cgalParamEnd
-   *   \cgalParamBegin{optimize_anchor_location}  if set to `true`, optimize the anchor locations.
-   *   \cgalParamEnd
-   *   \cgalParamBegin{pca_plane}  set `true` if use PCA plane fitting, otherwise use the default area averaged plane parameters.
-   *   \cgalParamEnd
+   *   \cgalParamNBegin{subdivision_ratio}
+   *     \cgalParamDescription{the chord subdivision ratio threshold to the chord length or average edge length}
+   *     \cgalParamType{`geom_traits::FT`}
+   *     \cgalParamDefault{`5.0`}
+   *   \cgalParamNEnd
+   *
+   *   \cgalParamNBegin{relative_to_chord}
+   *     \cgalParamDescription{If `true`, the `subdivision_ratio` is the ratio of the furthest vertex distance
+   *                           to the chord length, otherwise is the average edge length}
+   *     \cgalParamType{`Boolean`}
+   *     \cgalParamDefault{`false`}
+   *   \cgalParamNEnd
+   *
+   *   \cgalParamNBegin{with_dihedral_angle}
+   *     \cgalParamDescription{If `true`, the `subdivision_ratio` is weighted by dihedral angle}
+   *     \cgalParamType{`Boolean`}
+   *     \cgalParamDefault{`false`}
+   *   \cgalParamNEnd
+   *
+   *   \cgalParamNBegin{optimize_anchor_location}
+   *     \cgalParamDescription{If `true`, optimize the anchor locations}
+   *     \cgalParamType{`Boolean`}
+   *     \cgalParamDefault{`true`}
+   *   \cgalParamNEnd
+   *
+   *   \cgalParamNBegin{pca_plane}
+   *     \cgalParamDescription{If `true`, use PCA plane fitting, otherwise use the default area averaged plane parameters}
+   *     \cgalParamType{`Boolean`}
+   *     \cgalParamDefault{`false`}
+   *   \cgalParamNEnd
    * \cgalNamedParamsEnd
    */
   template <typename NamedParameters>
@@ -831,22 +866,38 @@ public:
   /// @{
   /*!
    * @brief outputs approximation results.
-   * @tparam NamedParameters a sequence of \ref vsa_namedparameters
+   * @tparam NamedParameters a sequence of \ref bgl_namedparameters
 
-   * @param np optional sequence of \ref vsa_namedparameters among the ones listed below
+   * @param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
    * \cgalNamedParamsBegin{Output Named Parameters}
-   *  \cgalParamBegin{face_proxy_map} a `WritePropertyMap` with
-   * `boost::graph_traits<TriangleMesh>::%face_descriptor` as key and `std::size_t` as value type.
-   * A proxy is a set of connected faces which are placed under the same proxy patch (see \cgalFigureRef{iterations}).
-   * The proxy-ids are contiguous in range `[0, number_of_proxies() - 1]`.
-   *  \cgalParamEnd
-   *  \cgalParamBegin{proxies} output iterator over proxies.
-   *  \cgalParamEnd
-   *  \cgalParamBegin{anchors} output iterator over anchor points.
-   *  \cgalParamEnd
-   *  \cgalParamBegin{triangles} output iterator over indexed triangles.
-   *  \cgalParamEnd
+   *   \cgalParamNBegin{face_proxy_map}
+   *     \cgalParamDescription{a property map to output the proxy index of each face of the input polygon mesh}
+   *     \cgalParamType{a model of `WritablePropertyMap` with `boost::graph_traits<TriangleMesh>::%face_descriptor`
+   *                    as key and `std::size_t` as value type}
+   *     \cgalParamDefault{no output operation is performed}
+   *     \cgalParamExtra{A proxy is a set of connected faces which are placed under the same proxy patch (see \cgalFigureRef{iterations})}
+   *     \cgalParamExtra{The proxy-ids are contiguous in range `[0, number_of_proxies - 1]`}
+   *   \cgalParamNEnd
+   *
+   *   \cgalParamNBegin{proxies}
+   *     \cgalParamDescription{an `OutputIterator` to put proxies in}
+   *     \cgalParamType{a class model of `OutputIterator` with
+   *                    `CGAL::Surface_mesh_approximation::L21_metric_vector_proxy_no_area_weighting::Proxy` value type}
+   *     \cgalParamDefault{no output operation is performed}
+   *   \cgalParamNEnd
+   *
+   *   \cgalParamNBegin{anchors}
+   *     \cgalParamDescription{an `OutputIterator` to put anchor points in}
+   *     \cgalParamType{a class model of `OutputIterator` with `geom_traits::%Point_3` value type}
+   *     \cgalParamDefault{no output operation is performed}
+   *   \cgalParamNEnd
+   *
+   *   \cgalParamNBegin{triangles}
+   *     \cgalParamDescription{an `OutputIterator` to put indexed triangles in}
+   *     \cgalParamType{a class model of `OutputIterator` with `std::array<std::size_t, 3>` value type}
+   *     \cgalParamDefault{no output operation is performed}
+   *   \cgalParamNEnd
    * \cgalNamedParamsEnd
    */
   template <typename NamedParameters>
diff -pruN 5.0.3-1/include/CGAL/vcm_estimate_edges.h 5.1-1/include/CGAL/vcm_estimate_edges.h
--- 5.0.3-1/include/CGAL/vcm_estimate_edges.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/vcm_estimate_edges.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_processing_3/include/CGAL/vcm_estimate_edges.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/vcm_estimate_edges.h $
 // $Id: vcm_estimate_edges.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/vcm_estimate_normals.h 5.1-1/include/CGAL/vcm_estimate_normals.h
--- 5.0.3-1/include/CGAL/vcm_estimate_normals.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/vcm_estimate_normals.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_processing_3/include/CGAL/vcm_estimate_normals.h $
-// $Id: vcm_estimate_normals.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/vcm_estimate_normals.h $
+// $Id: vcm_estimate_normals.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s) : Jocelyn Meyron and Quentin Mérigot
@@ -235,12 +235,21 @@ vcm_convolve (ForwardIterator first,
    \param ccov output range of covariance matrices.
    \param offset_radius offset_radius.
    \param convolution_radius convolution_radius.
-   \param np optional sequence of \ref psp_namedparameters "Named Parameters" among the ones listed below.
+   \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
    \cgalNamedParamsBegin
-     \cgalParamBegin{point_map} a model of `ReadablePropertyMap` with value type `geom_traits::Point_3`.
-     If this parameter is omitted, `CGAL::Identity_property_map<geom_traits::Point_3>` is used.\cgalParamEnd
-     \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+     \cgalParamNBegin{point_map}
+       \cgalParamDescription{a property map associating points to the elements of the point set `points`}
+       \cgalParamType{a model of `ReadWritePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange` and whose value type is `geom_traits::Point_3`}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{geom_traits}
+       \cgalParamDescription{an instance of a geometric traits class}
+       \cgalParamType{a model of `Kernel`}
+       \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+     \cgalParamNEnd
    \cgalNamedParamsEnd
 
    \sa `CGAL::vcm_is_on_feature_edge()`
@@ -260,10 +269,10 @@ compute_vcm (const PointRange& points,
     using parameters::get_parameter;
 
     // basic geometric types
-    typedef typename Point_set_processing_3::GetPointMap<PointRange, NamedParameters>::type PointMap;
+    typedef typename CGAL::GetPointMap<PointRange, NamedParameters>::type PointMap;
     typedef typename Point_set_processing_3::GetK<PointRange, NamedParameters>::Kernel Kernel;
 
-    PointMap point_map = choose_parameter(get_parameter(np, internal_np::point_map), PointMap());
+    PointMap point_map = choose_parameter<PointMap>(get_parameter(np, internal_np::point_map));
     Kernel kernel;
 
     // First, compute the VCM for each point
@@ -320,7 +329,7 @@ vcm_estimate_normals_internal (PointRang
     using parameters::get_parameter;
 
     // basic geometric types
-    typedef typename Point_set_processing_3::GetPointMap<PointRange, NamedParameters>::type PointMap;
+    typedef typename CGAL::GetPointMap<PointRange, NamedParameters>::type PointMap;
     typedef typename Point_set_processing_3::GetNormalMap<PointRange, NamedParameters>::type NormalMap;
     typedef typename Point_set_processing_3::GetK<PointRange, NamedParameters>::Kernel Kernel;
     typedef typename GetDiagonalizeTraits<NamedParameters, double, 3>::type DiagonalizeTraits;
@@ -329,8 +338,8 @@ vcm_estimate_normals_internal (PointRang
                                 typename Point_set_processing_3::GetNormalMap<PointRange, NamedParameters>::NoMap>::value),
                               "Error: no normal map");
 
-    PointMap point_map = choose_parameter(get_parameter(np, internal_np::point_map), PointMap());
-    NormalMap normal_map = choose_parameter(get_parameter(np, internal_np::normal_map), NormalMap());
+    PointMap point_map = choose_parameter<PointMap>(get_parameter(np, internal_np::point_map));
+    NormalMap normal_map = choose_parameter<NormalMap>(get_parameter(np, internal_np::normal_map));
 
     typedef std::array<double, 6> Covariance;
 
@@ -398,20 +407,36 @@ vcm_estimate_normals_internal (PointRang
    \param points input point range.
    \param offset_radius offset_radius.
    \param convolution_radius convolution_radius.
-   \param np optional sequence of \ref psp_namedparameters "Named Parameters" among the ones listed below.
+   \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
    \cgalNamedParamsBegin
-     \cgalParamBegin{point_map} a model of `ReadablePropertyMap` with value type `geom_traits::Point_3`.
-     If this parameter is omitted, `CGAL::Identity_property_map<geom_traits::Point_3>` is used.\cgalParamEnd
-     \cgalParamBegin{normal_map} a model of `WritablePropertyMap` with value type
-     `geom_traits::Vector_3`.\cgalParamEnd
-       \cgalParamBegin{diagonalize_traits} a model of `DiagonalizeTraits`. It can be omitted:
-       if Eigen 3 (or greater) is available and `CGAL_EIGEN3_ENABLED` is defined then an overload
-       using `Eigen_diagonalize_traits` is provided. Otherwise, the internal implementation
-       `CGAL::Diagonalize_traits` is used.\cgalParamEnd
-     \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+     \cgalParamNBegin{point_map}
+       \cgalParamDescription{a property map associating points to the elements of the point set `points`}
+       \cgalParamType{a model of `ReadWritePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange` and whose value type is `geom_traits::Point_3`}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{normal_map}
+       \cgalParamDescription{a property map associating normals to the elements of the point set `points`}
+       \cgalParamType{a model of `ReadWritePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange` and whose value type is `geom_traits::Vector_3`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{diagonalize_traits}
+       \cgalParamDescription{the solver used for diagonalizing covariance matrices}
+       \cgalParamType{a class model of `DiagonalizeTraits`}
+       \cgalParamDefault{If Eigen 3 (or greater) is available and `CGAL_EIGEN3_ENABLED` is defined
+                         then an overload using `Eigen_diagonalize_traits` is provided.
+                         Otherwise, the internal implementation `CGAL::Diagonalize_traits` is used}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{geom_traits}
+       \cgalParamDescription{an instance of a geometric traits class}
+       \cgalParamType{a model of `Kernel`}
+       \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+     \cgalParamNEnd
    \cgalNamedParamsEnd
-
 */
 template <typename PointRange,
           typename NamedParameters
@@ -457,18 +482,35 @@ vcm_estimate_normals (PointRange& points
    \param points input point range.
    \param offset_radius offset_radius.
    \param k number of neighbor points used for convolution.
-   \param np optional sequence of \ref psp_namedparameters "Named Parameters" among the ones listed below.
+   \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
    \cgalNamedParamsBegin
-     \cgalParamBegin{point_map} a model of `ReadablePropertyMap` with value type `geom_traits::Point_3`.
-     If this parameter is omitted, `CGAL::Identity_property_map<geom_traits::Point_3>` is used.\cgalParamEnd
-     \cgalParamBegin{normal_map} a model of `WritablePropertyMap` with value type
-     `geom_traits::Vector_3`.\cgalParamEnd
-       \cgalParamBegin{diagonalize_traits} a model of `DiagonalizeTraits`. It can be omitted:
-       if Eigen 3 (or greater) is available and `CGAL_EIGEN3_ENABLED` is defined then an overload
-       using `Eigen_diagonalize_traits` is provided. Otherwise, the internal implementation
-       `CGAL::Diagonalize_traits` is used.\cgalParamEnd
-     \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+     \cgalParamNBegin{point_map}
+       \cgalParamDescription{a property map associating points to the elements of the point set `points`}
+       \cgalParamType{a model of `ReadWritePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange` and whose value type is `geom_traits::Point_3`}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{normal_map}
+       \cgalParamDescription{a property map associating normals to the elements of the point set `points`}
+       \cgalParamType{a model of `ReadWritePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange` and whose value type is `geom_traits::Vector_3`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{diagonalize_traits}
+       \cgalParamDescription{the solver used for diagonalizing covariance matrices}
+       \cgalParamType{a class model of `DiagonalizeTraits`}
+       \cgalParamDefault{If Eigen 3 (or greater) is available and `CGAL_EIGEN3_ENABLED` is defined
+                         then an overload using `Eigen_diagonalize_traits` is provided.
+                         Otherwise, the internal implementation `CGAL::Diagonalize_traits` is used}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{geom_traits}
+       \cgalParamDescription{an instance of a geometric traits class}
+       \cgalParamType{a model of `Kernel`}
+       \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+     \cgalParamNEnd
    \cgalNamedParamsEnd
 */
 template < typename PointRange,
diff -pruN 5.0.3-1/include/CGAL/Vector_2.h 5.1-1/include/CGAL/Vector_2.h
--- 5.0.3-1/include/CGAL/Vector_2.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/Vector_2.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Vector_2.h $
-// $Id: Vector_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Vector_2.h $
+// $Id: Vector_2.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -54,12 +54,12 @@ public:
   typedef RVector_2 Rep;
   typedef typename R_::Cartesian_const_iterator_2 Cartesian_const_iterator;
 
-  const Rep& rep() const
+  const Rep& rep() const noexcept
   {
     return *this;
   }
 
-  Rep& rep()
+  Rep& rep() noexcept
   {
     return *this;
   }
@@ -93,6 +93,15 @@ public:
   Vector_2(const RT &x, const RT &y, const RT &w)
       : RVector_2(typename R::Construct_vector_2()(Return_base_tag(), x,y,w)) {}
 
+  friend void swap(Self& a, Self& b)
+#ifdef __cpp_lib_is_swappable
+    noexcept(std::is_nothrow_swappable_v<Rep>)
+#endif
+  {
+    using std::swap;
+    swap(a.rep(), b.rep());
+  }
+
 
   typename cpp11::result_of<typename R::Compute_x_2(Vector_2)>::type
   x() const
diff -pruN 5.0.3-1/include/CGAL/Vector_3.h 5.1-1/include/CGAL/Vector_3.h
--- 5.0.3-1/include/CGAL/Vector_3.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/Vector_3.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Vector_3.h $
-// $Id: Vector_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Vector_3.h $
+// $Id: Vector_3.h 8bb22d5 2020-03-26T14:23:37+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -93,6 +93,15 @@ public:
   Vector_3(const RT& x, const RT& y, const RT& z, const RT& w)
     : Rep(typename R::Construct_vector_3()(Return_base_tag(), x, y, z, w)) {}
 
+  friend void swap(Self& a, Self& b)
+#ifdef __cpp_lib_is_swappable
+    noexcept(std::is_nothrow_swappable_v<Rep>)
+#endif
+  {
+    using std::swap;
+    swap(a.rep(), b.rep());
+  }
+
   Direction_3 direction() const
   {
     return R().construct_direction_3_object()(*this);
diff -pruN 5.0.3-1/include/CGAL/vector.h 5.1-1/include/CGAL/vector.h
--- 5.0.3-1/include/CGAL/vector.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/vector.h	2020-09-08 12:15:33.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/STL_Extension/include/CGAL/vector.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/STL_Extension/include/CGAL/vector.h $
 // $Id: vector.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/version.h 5.1-1/include/CGAL/version.h
--- 5.0.3-1/include/CGAL/version.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/version.h	2020-09-08 12:15:53.000000000 +0000
@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/version.h $
-// $Id: version.h 2b01447 2020-02-25T09:46:40+01:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/version.h $
+// $Id: version.h b271ec5 2020-09-04T15:18:51+02:00 Laurent Rineau
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : -
@@ -17,12 +17,12 @@
 #define CGAL_VERSION_H
 
 #ifndef SWIG
-#define CGAL_VERSION 5.0.3
-#define CGAL_GIT_HASH ff74e7f6e89013c6b08d86369d90f7c35c330a2b
+#define CGAL_VERSION 5.1
+#define CGAL_GIT_HASH 301c5e0c5caffe894d1217257bad3218bf990f06
 #endif
-#define CGAL_VERSION_NR 1050031000
+#define CGAL_VERSION_NR 1050101000
 #define CGAL_SVN_REVISION 99999
-#define CGAL_RELEASE_DATE 20200801
+#define CGAL_RELEASE_DATE 20200908
 
 #include <CGAL/version_macros.h>
 
diff -pruN 5.0.3-1/include/CGAL/version_macros.h 5.1-1/include/CGAL/version_macros.h
--- 5.0.3-1/include/CGAL/version_macros.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/version_macros.h	2020-09-08 12:14:56.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Installation/include/CGAL/version_macros.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Installation/include/CGAL/version_macros.h $
 // $Id: version_macros.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Vertex2Data_Property_Map_with_std_map.h 5.1-1/include/CGAL/Vertex2Data_Property_Map_with_std_map.h
--- 5.0.3-1/include/CGAL/Vertex2Data_Property_Map_with_std_map.h	2020-08-01 19:02:47.000000000 +0000
+++ 5.1-1/include/CGAL/Vertex2Data_Property_Map_with_std_map.h	2020-09-08 12:15:31.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Ridges_3/include/CGAL/Vertex2Data_Property_Map_with_std_map.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Ridges_3/include/CGAL/Vertex2Data_Property_Map_with_std_map.h $
 // $Id: Vertex2Data_Property_Map_with_std_map.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Visibility_2/visibility_utils.h 5.1-1/include/CGAL/Visibility_2/visibility_utils.h
--- 5.0.3-1/include/CGAL/Visibility_2/visibility_utils.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Visibility_2/visibility_utils.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Visibility_2/include/CGAL/Visibility_2/visibility_utils.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Visibility_2/include/CGAL/Visibility_2/visibility_utils.h $
 // $Id: visibility_utils.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Voronoi_diagram_2/Accessor.h 5.1-1/include/CGAL/Voronoi_diagram_2/Accessor.h
--- 5.0.3-1/include/CGAL/Voronoi_diagram_2/Accessor.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Voronoi_diagram_2/Accessor.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Accessor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Accessor.h $
 // $Id: Accessor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Voronoi_diagram_2/Adaptation_traits_base_2.h 5.1-1/include/CGAL/Voronoi_diagram_2/Adaptation_traits_base_2.h
--- 5.0.3-1/include/CGAL/Voronoi_diagram_2/Adaptation_traits_base_2.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Voronoi_diagram_2/Adaptation_traits_base_2.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Adaptation_traits_base_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Adaptation_traits_base_2.h $
 // $Id: Adaptation_traits_base_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Voronoi_diagram_2/Adaptation_traits_functors.h 5.1-1/include/CGAL/Voronoi_diagram_2/Adaptation_traits_functors.h
--- 5.0.3-1/include/CGAL/Voronoi_diagram_2/Adaptation_traits_functors.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Voronoi_diagram_2/Adaptation_traits_functors.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Adaptation_traits_functors.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Adaptation_traits_functors.h $
 // $Id: Adaptation_traits_functors.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Voronoi_diagram_2/Apollonius_graph_degeneracy_testers.h 5.1-1/include/CGAL/Voronoi_diagram_2/Apollonius_graph_degeneracy_testers.h
--- 5.0.3-1/include/CGAL/Voronoi_diagram_2/Apollonius_graph_degeneracy_testers.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Voronoi_diagram_2/Apollonius_graph_degeneracy_testers.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Apollonius_graph_degeneracy_testers.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Apollonius_graph_degeneracy_testers.h $
 // $Id: Apollonius_graph_degeneracy_testers.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Voronoi_diagram_2/Apollonius_graph_nearest_site_2.h 5.1-1/include/CGAL/Voronoi_diagram_2/Apollonius_graph_nearest_site_2.h
--- 5.0.3-1/include/CGAL/Voronoi_diagram_2/Apollonius_graph_nearest_site_2.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Voronoi_diagram_2/Apollonius_graph_nearest_site_2.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Apollonius_graph_nearest_site_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Apollonius_graph_nearest_site_2.h $
 // $Id: Apollonius_graph_nearest_site_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Voronoi_diagram_2/basic.h 5.1-1/include/CGAL/Voronoi_diagram_2/basic.h
--- 5.0.3-1/include/CGAL/Voronoi_diagram_2/basic.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Voronoi_diagram_2/basic.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/basic.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/basic.h $
 // $Id: basic.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Voronoi_diagram_2/Cached_degeneracy_testers.h 5.1-1/include/CGAL/Voronoi_diagram_2/Cached_degeneracy_testers.h
--- 5.0.3-1/include/CGAL/Voronoi_diagram_2/Cached_degeneracy_testers.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Voronoi_diagram_2/Cached_degeneracy_testers.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Cached_degeneracy_testers.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Cached_degeneracy_testers.h $
 // $Id: Cached_degeneracy_testers.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Voronoi_diagram_2/Circulator_adaptors.h 5.1-1/include/CGAL/Voronoi_diagram_2/Circulator_adaptors.h
--- 5.0.3-1/include/CGAL/Voronoi_diagram_2/Circulator_adaptors.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Voronoi_diagram_2/Circulator_adaptors.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Circulator_adaptors.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Circulator_adaptors.h $
 // $Id: Circulator_adaptors.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Voronoi_diagram_2/Connected_components.h 5.1-1/include/CGAL/Voronoi_diagram_2/Connected_components.h
--- 5.0.3-1/include/CGAL/Voronoi_diagram_2/Connected_components.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Voronoi_diagram_2/Connected_components.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Connected_components.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Connected_components.h $
 // $Id: Connected_components.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Voronoi_diagram_2/Construct_dual_points.h 5.1-1/include/CGAL/Voronoi_diagram_2/Construct_dual_points.h
--- 5.0.3-1/include/CGAL/Voronoi_diagram_2/Construct_dual_points.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Voronoi_diagram_2/Construct_dual_points.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Construct_dual_points.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Construct_dual_points.h $
 // $Id: Construct_dual_points.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Voronoi_diagram_2/Default_site_inserters.h 5.1-1/include/CGAL/Voronoi_diagram_2/Default_site_inserters.h
--- 5.0.3-1/include/CGAL/Voronoi_diagram_2/Default_site_inserters.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Voronoi_diagram_2/Default_site_inserters.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Default_site_inserters.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Default_site_inserters.h $
 // $Id: Default_site_inserters.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Voronoi_diagram_2/Default_site_removers.h 5.1-1/include/CGAL/Voronoi_diagram_2/Default_site_removers.h
--- 5.0.3-1/include/CGAL/Voronoi_diagram_2/Default_site_removers.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Voronoi_diagram_2/Default_site_removers.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Default_site_removers.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Default_site_removers.h $
 // $Id: Default_site_removers.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Voronoi_diagram_2/Degeneracy_tester_binders.h 5.1-1/include/CGAL/Voronoi_diagram_2/Degeneracy_tester_binders.h
--- 5.0.3-1/include/CGAL/Voronoi_diagram_2/Degeneracy_tester_binders.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Voronoi_diagram_2/Degeneracy_tester_binders.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Degeneracy_tester_binders.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Degeneracy_tester_binders.h $
 // $Id: Degeneracy_tester_binders.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Voronoi_diagram_2/Delaunay_triangulation_degeneracy_testers.h 5.1-1/include/CGAL/Voronoi_diagram_2/Delaunay_triangulation_degeneracy_testers.h
--- 5.0.3-1/include/CGAL/Voronoi_diagram_2/Delaunay_triangulation_degeneracy_testers.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Voronoi_diagram_2/Delaunay_triangulation_degeneracy_testers.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Delaunay_triangulation_degeneracy_testers.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Delaunay_triangulation_degeneracy_testers.h $
 // $Id: Delaunay_triangulation_degeneracy_testers.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Voronoi_diagram_2/Delaunay_triangulation_nearest_site_2.h 5.1-1/include/CGAL/Voronoi_diagram_2/Delaunay_triangulation_nearest_site_2.h
--- 5.0.3-1/include/CGAL/Voronoi_diagram_2/Delaunay_triangulation_nearest_site_2.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Voronoi_diagram_2/Delaunay_triangulation_nearest_site_2.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Delaunay_triangulation_nearest_site_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Delaunay_triangulation_nearest_site_2.h $
 // $Id: Delaunay_triangulation_nearest_site_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Voronoi_diagram_2/Dummy_iterator.h 5.1-1/include/CGAL/Voronoi_diagram_2/Dummy_iterator.h
--- 5.0.3-1/include/CGAL/Voronoi_diagram_2/Dummy_iterator.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Voronoi_diagram_2/Dummy_iterator.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Dummy_iterator.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Dummy_iterator.h $
 // $Id: Dummy_iterator.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Voronoi_diagram_2/Edge_less.h 5.1-1/include/CGAL/Voronoi_diagram_2/Edge_less.h
--- 5.0.3-1/include/CGAL/Voronoi_diagram_2/Edge_less.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Voronoi_diagram_2/Edge_less.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Edge_less.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Edge_less.h $
 // $Id: Edge_less.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Voronoi_diagram_2/Face.h 5.1-1/include/CGAL/Voronoi_diagram_2/Face.h
--- 5.0.3-1/include/CGAL/Voronoi_diagram_2/Face.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Voronoi_diagram_2/Face.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Face.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Face.h $
 // $Id: Face.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Voronoi_diagram_2/Finder_classes.h 5.1-1/include/CGAL/Voronoi_diagram_2/Finder_classes.h
--- 5.0.3-1/include/CGAL/Voronoi_diagram_2/Finder_classes.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Voronoi_diagram_2/Finder_classes.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Finder_classes.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Finder_classes.h $
 // $Id: Finder_classes.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Voronoi_diagram_2/Halfedge.h 5.1-1/include/CGAL/Voronoi_diagram_2/Halfedge.h
--- 5.0.3-1/include/CGAL/Voronoi_diagram_2/Halfedge.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Voronoi_diagram_2/Halfedge.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Halfedge.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Halfedge.h $
 // $Id: Halfedge.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Voronoi_diagram_2/Handle_adaptor.h 5.1-1/include/CGAL/Voronoi_diagram_2/Handle_adaptor.h
--- 5.0.3-1/include/CGAL/Voronoi_diagram_2/Handle_adaptor.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Voronoi_diagram_2/Handle_adaptor.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Handle_adaptor.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Handle_adaptor.h $
 // $Id: Handle_adaptor.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Voronoi_diagram_2/Identity_rejectors.h 5.1-1/include/CGAL/Voronoi_diagram_2/Identity_rejectors.h
--- 5.0.3-1/include/CGAL/Voronoi_diagram_2/Identity_rejectors.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Voronoi_diagram_2/Identity_rejectors.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Identity_rejectors.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Identity_rejectors.h $
 // $Id: Identity_rejectors.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Voronoi_diagram_2/Iterator_adaptors.h 5.1-1/include/CGAL/Voronoi_diagram_2/Iterator_adaptors.h
--- 5.0.3-1/include/CGAL/Voronoi_diagram_2/Iterator_adaptors.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Voronoi_diagram_2/Iterator_adaptors.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Iterator_adaptors.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Iterator_adaptors.h $
 // $Id: Iterator_adaptors.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Voronoi_diagram_2/Policy_base.h 5.1-1/include/CGAL/Voronoi_diagram_2/Policy_base.h
--- 5.0.3-1/include/CGAL/Voronoi_diagram_2/Policy_base.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Voronoi_diagram_2/Policy_base.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Policy_base.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Policy_base.h $
 // $Id: Policy_base.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Voronoi_diagram_2/Regular_triangulation_degeneracy_testers.h 5.1-1/include/CGAL/Voronoi_diagram_2/Regular_triangulation_degeneracy_testers.h
--- 5.0.3-1/include/CGAL/Voronoi_diagram_2/Regular_triangulation_degeneracy_testers.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Voronoi_diagram_2/Regular_triangulation_degeneracy_testers.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Regular_triangulation_degeneracy_testers.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Regular_triangulation_degeneracy_testers.h $
 // $Id: Regular_triangulation_degeneracy_testers.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Voronoi_diagram_2/Regular_triangulation_nearest_site_2.h 5.1-1/include/CGAL/Voronoi_diagram_2/Regular_triangulation_nearest_site_2.h
--- 5.0.3-1/include/CGAL/Voronoi_diagram_2/Regular_triangulation_nearest_site_2.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Voronoi_diagram_2/Regular_triangulation_nearest_site_2.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Regular_triangulation_nearest_site_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Regular_triangulation_nearest_site_2.h $
 // $Id: Regular_triangulation_nearest_site_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Voronoi_diagram_2/Segment_Delaunay_graph_degeneracy_testers.h 5.1-1/include/CGAL/Voronoi_diagram_2/Segment_Delaunay_graph_degeneracy_testers.h
--- 5.0.3-1/include/CGAL/Voronoi_diagram_2/Segment_Delaunay_graph_degeneracy_testers.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Voronoi_diagram_2/Segment_Delaunay_graph_degeneracy_testers.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Segment_Delaunay_graph_degeneracy_testers.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Segment_Delaunay_graph_degeneracy_testers.h $
 // $Id: Segment_Delaunay_graph_degeneracy_testers.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Voronoi_diagram_2/Segment_Delaunay_graph_nearest_site_2.h 5.1-1/include/CGAL/Voronoi_diagram_2/Segment_Delaunay_graph_nearest_site_2.h
--- 5.0.3-1/include/CGAL/Voronoi_diagram_2/Segment_Delaunay_graph_nearest_site_2.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Voronoi_diagram_2/Segment_Delaunay_graph_nearest_site_2.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Segment_Delaunay_graph_nearest_site_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Segment_Delaunay_graph_nearest_site_2.h $
 // $Id: Segment_Delaunay_graph_nearest_site_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Voronoi_diagram_2/Site_accessors.h 5.1-1/include/CGAL/Voronoi_diagram_2/Site_accessors.h
--- 5.0.3-1/include/CGAL/Voronoi_diagram_2/Site_accessors.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Voronoi_diagram_2/Site_accessors.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Site_accessors.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Site_accessors.h $
 // $Id: Site_accessors.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Voronoi_diagram_2/Unbounded_edges.h 5.1-1/include/CGAL/Voronoi_diagram_2/Unbounded_edges.h
--- 5.0.3-1/include/CGAL/Voronoi_diagram_2/Unbounded_edges.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Voronoi_diagram_2/Unbounded_edges.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Unbounded_edges.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Unbounded_edges.h $
 // $Id: Unbounded_edges.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Voronoi_diagram_2/Unbounded_faces.h 5.1-1/include/CGAL/Voronoi_diagram_2/Unbounded_faces.h
--- 5.0.3-1/include/CGAL/Voronoi_diagram_2/Unbounded_faces.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Voronoi_diagram_2/Unbounded_faces.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Unbounded_faces.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Unbounded_faces.h $
 // $Id: Unbounded_faces.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Voronoi_diagram_2/Validity_testers.h 5.1-1/include/CGAL/Voronoi_diagram_2/Validity_testers.h
--- 5.0.3-1/include/CGAL/Voronoi_diagram_2/Validity_testers.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Voronoi_diagram_2/Validity_testers.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Validity_testers.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Validity_testers.h $
 // $Id: Validity_testers.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Voronoi_diagram_2/Vertex.h 5.1-1/include/CGAL/Voronoi_diagram_2/Vertex.h
--- 5.0.3-1/include/CGAL/Voronoi_diagram_2/Vertex.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Voronoi_diagram_2/Vertex.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Vertex.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2/Vertex.h $
 // $Id: Vertex.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Voronoi_diagram_2.h 5.1-1/include/CGAL/Voronoi_diagram_2.h
--- 5.0.3-1/include/CGAL/Voronoi_diagram_2.h	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/include/CGAL/Voronoi_diagram_2.h	2020-09-08 12:15:53.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Voronoi_diagram_2/include/CGAL/Voronoi_diagram_2.h $
 // $Id: Voronoi_diagram_2.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Voronoi_intersection_2_traits_3.h 5.1-1/include/CGAL/Voronoi_intersection_2_traits_3.h
--- 5.0.3-1/include/CGAL/Voronoi_intersection_2_traits_3.h	2020-08-01 19:02:23.000000000 +0000
+++ 5.1-1/include/CGAL/Voronoi_intersection_2_traits_3.h	2020-09-08 12:14:56.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Interpolation/include/CGAL/Voronoi_intersection_2_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Interpolation/include/CGAL/Voronoi_intersection_2_traits_3.h $
 // $Id: Voronoi_intersection_2_traits_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/vtkSurfaceMesherContourFilter.h 5.1-1/include/CGAL/vtkSurfaceMesherContourFilter.h
--- 5.0.3-1/include/CGAL/vtkSurfaceMesherContourFilter.h	2020-08-01 19:02:57.000000000 +0000
+++ 5.1-1/include/CGAL/vtkSurfaceMesherContourFilter.h	2020-09-08 12:15:47.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Surface_mesher/include/CGAL/vtkSurfaceMesherContourFilter.h $
-// $Id: vtkSurfaceMesherContourFilter.h 5b2d46c 2020-04-21T10:03:00+02:00 Maxime Gimeno
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Surface_mesher/include/CGAL/vtkSurfaceMesherContourFilter.h $
+// $Id: vtkSurfaceMesherContourFilter.h e9d41d7 2020-04-21T10:03:00+02:00 Maxime Gimeno
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
diff -pruN 5.0.3-1/include/CGAL/Weighted_alpha_shape_euclidean_traits_2.h 5.1-1/include/CGAL/Weighted_alpha_shape_euclidean_traits_2.h
--- 5.0.3-1/include/CGAL/Weighted_alpha_shape_euclidean_traits_2.h	2020-08-01 19:02:09.000000000 +0000
+++ 5.1-1/include/CGAL/Weighted_alpha_shape_euclidean_traits_2.h	2020-09-08 12:14:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Alpha_shapes_2/include/CGAL/Weighted_alpha_shape_euclidean_traits_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Alpha_shapes_2/include/CGAL/Weighted_alpha_shape_euclidean_traits_2.h $
 // $Id: Weighted_alpha_shape_euclidean_traits_2.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Weighted_alpha_shape_euclidean_traits_3.h 5.1-1/include/CGAL/Weighted_alpha_shape_euclidean_traits_3.h
--- 5.0.3-1/include/CGAL/Weighted_alpha_shape_euclidean_traits_3.h	2020-08-01 19:02:10.000000000 +0000
+++ 5.1-1/include/CGAL/Weighted_alpha_shape_euclidean_traits_3.h	2020-09-08 12:14:32.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Alpha_shapes_3/include/CGAL/Weighted_alpha_shape_euclidean_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Alpha_shapes_3/include/CGAL/Weighted_alpha_shape_euclidean_traits_3.h $
 // $Id: Weighted_alpha_shape_euclidean_traits_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Weighted_Minkowski_distance.h 5.1-1/include/CGAL/Weighted_Minkowski_distance.h
--- 5.0.3-1/include/CGAL/Weighted_Minkowski_distance.h	2020-08-01 19:02:52.000000000 +0000
+++ 5.1-1/include/CGAL/Weighted_Minkowski_distance.h	2020-09-08 12:15:40.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Spatial_searching/include/CGAL/Weighted_Minkowski_distance.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Spatial_searching/include/CGAL/Weighted_Minkowski_distance.h $
 // $Id: Weighted_Minkowski_distance.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Weighted_point_2.h 5.1-1/include/CGAL/Weighted_point_2.h
--- 5.0.3-1/include/CGAL/Weighted_point_2.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/Weighted_point_2.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Weighted_point_2.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Weighted_point_2.h $
 // $Id: Weighted_point_2.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Weighted_point_3.h 5.1-1/include/CGAL/Weighted_point_3.h
--- 5.0.3-1/include/CGAL/Weighted_point_3.h	2020-08-01 19:02:27.000000000 +0000
+++ 5.1-1/include/CGAL/Weighted_point_3.h	2020-09-08 12:15:01.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Kernel_23/include/CGAL/Weighted_point_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Kernel_23/include/CGAL/Weighted_point_3.h $
 // $Id: Weighted_point_3.h 52164b1 2019-10-19T15:34:59+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Width_3.h 5.1-1/include/CGAL/Width_3.h
--- 5.0.3-1/include/CGAL/Width_3.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Width_3.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polytope_distance_d/include/CGAL/Width_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polytope_distance_d/include/CGAL/Width_3.h $
 // $Id: Width_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/width_assertions.h 5.1-1/include/CGAL/width_assertions.h
--- 5.0.3-1/include/CGAL/width_assertions.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/width_assertions.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polytope_distance_d/include/CGAL/width_assertions.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polytope_distance_d/include/CGAL/width_assertions.h $
 // $Id: width_assertions.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Width_default_traits_3.h 5.1-1/include/CGAL/Width_default_traits_3.h
--- 5.0.3-1/include/CGAL/Width_default_traits_3.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Width_default_traits_3.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polytope_distance_d/include/CGAL/Width_default_traits_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polytope_distance_d/include/CGAL/Width_default_traits_3.h $
 // $Id: Width_default_traits_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/Width_polyhedron_3.h 5.1-1/include/CGAL/Width_polyhedron_3.h
--- 5.0.3-1/include/CGAL/Width_polyhedron_3.h	2020-08-01 19:02:46.000000000 +0000
+++ 5.1-1/include/CGAL/Width_polyhedron_3.h	2020-09-08 12:15:30.000000000 +0000
@@ -3,7 +3,7 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Polytope_distance_d/include/CGAL/Width_polyhedron_3.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Polytope_distance_d/include/CGAL/Width_polyhedron_3.h $
 // $Id: Width_polyhedron_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/include/CGAL/wlop_simplify_and_regularize_point_set.h 5.1-1/include/CGAL/wlop_simplify_and_regularize_point_set.h
--- 5.0.3-1/include/CGAL/wlop_simplify_and_regularize_point_set.h	2020-08-01 19:02:40.000000000 +0000
+++ 5.1-1/include/CGAL/wlop_simplify_and_regularize_point_set.h	2020-09-08 12:15:22.000000000 +0000
@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Point_set_processing_3/include/CGAL/wlop_simplify_and_regularize_point_set.h $
-// $Id: wlop_simplify_and_regularize_point_set.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Point_set_processing_3/include/CGAL/wlop_simplify_and_regularize_point_set.h $
+// $Id: wlop_simplify_and_regularize_point_set.h c253679 2020-04-18T16:27:58+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s) : Shihao Wu, Clement Jamin, Pierre Alliez
@@ -32,12 +32,8 @@
 #include <cmath>
 #include <ctime>
 
-#ifdef CGAL_LINKED_WITH_TBB
-#include <CGAL/Point_set_processing_3/internal/Parallel_callback.h>
-#include <tbb/parallel_for.h>
-#include <tbb/blocked_range.h>
-#include <tbb/scalable_allocator.h>
-#endif // CGAL_LINKED_WITH_TBB
+#include <CGAL/Point_set_processing_3/internal/Callback_wrapper.h>
+#include <CGAL/for_each.h>
 
 #include <CGAL/Simple_cartesian.h>
 #include <CGAL/Fuzzy_sphere.h>
@@ -329,67 +325,6 @@ compute_density_weight_for_sample_point(
 
 /// \endcond
 
-#ifdef CGAL_LINKED_WITH_TBB
-/// \cond SKIP_IN_MANUAL
-/// This is for parallelization of function: compute_denoise_projection()
-template <typename Kernel, typename Tree, typename RandomAccessIterator>
-class Sample_point_updater
-{
-  typedef typename Kernel::Point_3   Point;
-  typedef typename Kernel::FT        FT;
-
-  std::vector<Point> &update_sample_points;
-  std::vector<Point> &sample_points;
-  const Tree &original_kd_tree;
-  const Tree &sample_kd_tree;
-  const typename Kernel::FT radius;
-  const std::vector<typename Kernel::FT> &original_densities;
-  const std::vector<typename Kernel::FT> &sample_densities;
-  cpp11::atomic<std::size_t>& advancement;
-  cpp11::atomic<bool>& interrupted;
-
-public:
-  Sample_point_updater(
-    std::vector<Point> &out,
-    std::vector<Point> &in,
-    const Tree &_original_kd_tree,
-    const Tree &_sample_kd_tree,
-    const typename Kernel::FT _radius,
-    const std::vector<typename Kernel::FT> &_original_densities,
-    const std::vector<typename Kernel::FT> &_sample_densities,
-    cpp11::atomic<std::size_t>& advancement,
-    cpp11::atomic<bool>& interrupted):
-  update_sample_points(out),
-    sample_points(in),
-    original_kd_tree(_original_kd_tree),
-    sample_kd_tree(_sample_kd_tree),
-    radius(_radius),
-    original_densities(_original_densities),
-    sample_densities(_sample_densities),
-    advancement (advancement),
-    interrupted (interrupted) {}
-
-  void operator() ( const tbb::blocked_range<size_t>& r ) const
-  {
-    for (size_t i = r.begin(); i != r.end(); ++i)
-    {
-      if (interrupted)
-        break;
-      update_sample_points[i] = simplify_and_regularize_internal::
-        compute_update_sample_point<Kernel, Tree, RandomAccessIterator>(
-        sample_points[i],
-        original_kd_tree,
-        sample_kd_tree,
-        radius,
-        original_densities,
-        sample_densities);
-      ++ advancement;
-    }
-  }
-};
-/// \endcond
-#endif // CGAL_LINKED_WITH_TBB
-
 
 // ----------------------------------------------------------------------------
 // Public section
@@ -411,9 +346,8 @@ public:
    See the <a href="https://www.threadingbuildingblocks.org/documentation">TBB documentation</a>
    for more details.
 
-   \tparam ConcurrencyTag enables sequential versus parallel algorithm.
-   Possible values are `Sequential_tag`
-   and `Parallel_tag`.
+   \tparam ConcurrencyTag enables sequential versus parallel algorithm. Possible values are `Sequential_tag`,
+                          `Parallel_tag`, and `Parallel_if_available_tag`.
    \tparam PointRange is a model of `Range`. The value type of
    its iterator is the key type of the named parameter `point_map`.
    \tparam OutputIterator Type of the output iterator.
@@ -421,32 +355,74 @@ public:
 
    \param points input point range.
    \param output iterator where output points are put.
-   \param np optional sequence of \ref psp_namedparameters "Named Parameters" among the ones listed below.
+   \param np an optional sequence of \ref bgl_namedparameters "Named Parameters" among the ones listed below
 
    \cgalNamedParamsBegin
-     \cgalParamBegin{point_map} a model of `ReadWritePropertyMap` with value type `geom_traits::Point_3`.
-     If this parameter is omitted, `CGAL::Identity_property_map<geom_traits::Point_3>` is used.\cgalParamEnd
-     \cgalParamBegin{normal_map} a model of `ReadWritePropertyMap` with value type
-     `geom_traits::Vector_3`.\cgalParamEnd
-     \cgalParamBegin{select_percentage} percentage of points to retain. The default value is set to
-     5 (\%).\cgalParamEnd
-     \cgalParamBegin{neighbor_radius} spherical neighborhood radius. This is a key parameter that needs to be
-     finely tuned. The result will be irregular if too small, but a larger value will impact the runtime. In
-     practice, choosing a radius such that the neighborhood of each sample point includes at least two rings
-     of neighboring sample points gives satisfactory result. If this parameter is not provided, it is
-     automatically set to 8 times the average spacing of the point set.\cgalParamEnd
-     \cgalParamBegin{number_of_iterations} number of iterations to solve the optimsation problem. The default
-     value is 35. More iterations give a more regular result but increase the runtime.\cgalParamEnd
-     \cgalParamBegin{require_uniform_sampling} an optional preprocessing, which will give better result if the
-     distribution of the input points is highly non-uniform. The default value is `false`. \cgalParamEnd
-     \cgalParamBegin{callback} an instance of
-      `std::function<bool(double)>`. It is called regularly when the
-      algorithm is running: the current advancement (between 0. and
-      1.) is passed as parameter. If it returns `true`, then the
-      algorithm continues its execution normally; if it returns
-      `false`, the algorithm is stopped, no output points are
-      generated.\cgalParamEnd
-     \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+     \cgalParamNBegin{point_map}
+       \cgalParamDescription{a property map associating points to the elements of the point set `points`}
+       \cgalParamType{a model of `ReadWritePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange` and whose value type is `geom_traits::Point_3`}
+       \cgalParamDefault{`CGAL::Identity_property_map<geom_traits::Point_3>`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{normal_map}
+       \cgalParamDescription{a property map associating normals to the elements of the point set `points`}
+       \cgalParamType{a model of `ReadWritePropertyMap` whose key type is the value type
+                      of the iterator of `PointRange` and whose value type is `geom_traits::Vector_3`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{select_percentage}
+       \cgalParamDescription{percentage of points to retain}
+       \cgalParamType{floating scalar value}
+       \cgalParamDefault{`5`}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{neighbor_radius}
+       \cgalParamDescription{the spherical neighborhood radius}
+       \cgalParamType{floating scalar value}
+       \cgalParamDefault{8 times the average spacing of the point set}
+       \cgalParamExtra{This is a key parameter that needs to be finely tuned.
+                       The result will be irregular if too small, but a larger value will impact the runtime.
+                       In practice, choosing a radius such that the neighborhood of each sample point
+                       includes at least two rings of neighboring sample points gives satisfactory result.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{number_of_iterations}
+       \cgalParamDescription{number of iterations to solve the optimsation problem}
+       \cgalParamType{unsigned int}
+       \cgalParamDefault{`35`}
+       \cgalParamExtra{More iterations give a more regular result but increase the runtime}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{require_uniform_sampling}
+       \cgalParamDescription{If `true`, an optional preprocessing is applied, which will give
+                             better results if the distribution of the input points is highly non-uniform.}
+       \cgalParamType{Boolean}
+       \cgalParamDefault{`35`}
+       \cgalParamExtra{More iterations give a more regular result but increase the runtime}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{callback}
+       \cgalParamDescription{a mechanism to get feedback on the advancement of the algorithm
+                             while it's running and to interrupt it if needed}
+       \cgalParamType{an instance of `std::function<bool(double)>`.}
+       \cgalParamDefault{unused}
+       \cgalParamExtra{It is called regularly when the
+                       algorithm is running: the current advancement (between 0. and
+                       1.) is passed as parameter. If it returns `true`, then the
+                       algorithm continues its execution normally; if it returns
+                       `false`, the algorithm is stopped, no output points are
+                       generated.}
+       \cgalParamExtra{The callback will be copied and therefore needs to be lightweight.}
+       \cgalParamExtra{When `CGAL::Parallel_tag` is used, the `callback` mechanism is called asynchronously
+                       on a separate thread and shouldn't access or modify the variables that are parameters of the algorithm.}
+     \cgalParamNEnd
+
+     \cgalParamNBegin{geom_traits}
+       \cgalParamDescription{an instance of a geometric traits class}
+       \cgalParamType{a model of `Kernel`}
+       \cgalParamDefault{a \cgal Kernel deduced from the point type, using `CGAL::Kernel_traits`}
+     \cgalParamNEnd
    \cgalNamedParamsEnd
 
 */
@@ -465,10 +441,10 @@ wlop_simplify_and_regularize_point_set(
   using parameters::get_parameter;
 
   // basic geometric types
-  typedef typename Point_set_processing_3::GetPointMap<PointRange, NamedParameters>::type PointMap;
+  typedef typename CGAL::GetPointMap<PointRange, NamedParameters>::type PointMap;
   typedef typename Point_set_processing_3::GetK<PointRange, NamedParameters>::Kernel Kernel;
 
-  PointMap point_map = choose_parameter(get_parameter(np, internal_np::point_map), PointMap());
+  PointMap point_map = choose_parameter<PointMap>(get_parameter(np, internal_np::point_map));
   double select_percentage = choose_parameter(get_parameter(np, internal_np::select_percentage), 5.);
   double radius = choose_parameter(get_parameter(np, internal_np::neighbor_radius), -1);
   unsigned int iter_number = choose_parameter(get_parameter(np, internal_np::number_of_iterations), 35);
@@ -530,7 +506,6 @@ wlop_simplify_and_regularize_point_set(
 #endif
   }
 
-  FT radius2 = radius * radius;
   CGAL_point_set_processing_precondition(radius > 0);
 
   // Initiate a KD-tree search for original points
@@ -589,79 +564,50 @@ wlop_simplify_and_regularize_point_set(
       sample_density_weights.push_back(density);
     }
 
+    typedef boost::zip_iterator<boost::tuple<typename std::vector<Point>::iterator,
+                                             typename std::vector<Point>::iterator> > Zip_iterator;
 
-    typename std::vector<Point>::iterator update_iter = update_sample_points.begin();
-#ifndef CGAL_LINKED_WITH_TBB
-  CGAL_static_assertion_msg (!(boost::is_convertible<ConcurrencyTag, Parallel_tag>::value),
-                             "Parallel_tag is enabled but TBB is unavailable.");
-#else
-    //parallel
-    if (boost::is_convertible<ConcurrencyTag, Parallel_tag>::value)
-    {
-      Point_set_processing_3::internal::Parallel_callback
-        parallel_callback (callback, iter_number * number_of_sample, iter_n * number_of_sample);
+    Point_set_processing_3::internal::Callback_wrapper<ConcurrencyTag>
+      callback_wrapper (callback, iter_number * number_of_sample, iter_n * number_of_sample);
 
-      tbb::blocked_range<size_t> block(0, number_of_sample);
-      Sample_point_updater<Kernel, Kd_Tree, typename PointRange::iterator> sample_updater(
-                           update_sample_points,
-                           sample_points,
-                           original_kd_tree,
-                           sample_kd_tree,
-                           radius2,
-                           original_density_weights,
-                           sample_density_weights,
-                           parallel_callback.advancement(),
-                           parallel_callback.interrupted());
-
-       tbb::parallel_for(block, sample_updater);
-
-       bool interrupted = parallel_callback.interrupted();
-
-       // We interrupt by hand as counter only goes halfway and won't terminate by itself
-       parallel_callback.interrupted() = true;
-       parallel_callback.join();
-
-       // If interrupted during this step, nothing is computed, we return NaN
-       if (interrupted)
-         return output;
-    }else
-#endif
-    {
-      //sequential
-      std::size_t nb = iter_n * number_of_sample;
-      for (sample_iter = sample_points.begin();
-           sample_iter != sample_points.end(); ++sample_iter, ++update_iter, ++ nb)
-      {
-        *update_iter = simplify_and_regularize_internal::
-          compute_update_sample_point<Kernel,
-                                      Kd_Tree,
-                                      typename PointRange::iterator>
-                                      (*sample_iter,
-                                       original_kd_tree,
-                                       sample_kd_tree,
-                                       radius2,
-                                       original_density_weights,
-                                       sample_density_weights);
-        if (callback && !callback ((nb+1) / double(iter_number * number_of_sample)))
-          return output;
-      }
-    }
+    CGAL::for_each<ConcurrencyTag>
+      (CGAL::make_range (boost::make_zip_iterator (boost::make_tuple (sample_points.begin(), update_sample_points.begin())),
+                         boost::make_zip_iterator (boost::make_tuple (sample_points.end(), update_sample_points.end()))),
+       [&](const typename Zip_iterator::reference& t)
+       {
+         if (callback_wrapper.interrupted())
+           return false;
+
+         get<1>(t) = simplify_and_regularize_internal::
+           compute_update_sample_point<Kernel, Kd_Tree, typename PointRange::iterator>(
+             get<0>(t),
+             original_kd_tree,
+             sample_kd_tree,
+             radius,
+             original_density_weights,
+             sample_density_weights);
+         ++ callback_wrapper.advancement();
+
+         return true;
+       });
+
+    bool interrupted = callback_wrapper.interrupted();
+
+    // We interrupt by hand as counter only goes halfway and won't terminate by itself
+    callback_wrapper.interrupted() = true;
+    callback_wrapper.join();
+
+    // If interrupted during this step, nothing is computed, we return NaN
+    if (interrupted)
+      return output;
 
     sample_iter = sample_points.begin();
-    for (update_iter = update_sample_points.begin();
-         update_iter != update_sample_points.end();
-         ++update_iter, ++sample_iter)
-    {
-      *sample_iter = *update_iter;
-    }
+    for (std::size_t i = 0; i < sample_points.size(); ++ i)
+      sample_points[i] = update_sample_points[i];
   }
 
   // final output
-  for(sample_iter = sample_points.begin();
-      sample_iter != sample_points.end(); ++sample_iter)
-  {
-    *output++ = *sample_iter;
-  }
+  std::copy (sample_points.begin(), sample_points.end(), output);
 
   return output;
 }
diff -pruN 5.0.3-1/include/CGAL/wmult.h 5.1-1/include/CGAL/wmult.h
--- 5.0.3-1/include/CGAL/wmult.h	2020-08-01 19:02:20.000000000 +0000
+++ 5.1-1/include/CGAL/wmult.h	2020-09-08 12:14:50.000000000 +0000
@@ -7,7 +7,7 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Distance_2/include/CGAL/wmult.h $
+// $URL: https://github.com/CGAL/cgal/blob/v5.1/Distance_2/include/CGAL/wmult.h $
 // $Id: wmult.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
diff -pruN 5.0.3-1/lib/cmake/CGAL/CGALConfigBuildVersion.cmake 5.1-1/lib/cmake/CGAL/CGALConfigBuildVersion.cmake
--- 5.0.3-1/lib/cmake/CGAL/CGALConfigBuildVersion.cmake	1970-01-01 00:00:00.000000000 +0000
+++ 5.1-1/lib/cmake/CGAL/CGALConfigBuildVersion.cmake	2020-09-08 12:15:53.000000000 +0000
@@ -0,0 +1 @@
+set(CGAL_BUILD_VERSION 1000)
\ No newline at end of file
diff -pruN 5.0.3-1/lib/cmake/CGAL/CGALConfig.cmake 5.1-1/lib/cmake/CGAL/CGALConfig.cmake
--- 5.0.3-1/lib/cmake/CGAL/CGALConfig.cmake	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/lib/cmake/CGAL/CGALConfig.cmake	2020-09-08 12:12:19.000000000 +0000
@@ -150,6 +150,7 @@ foreach(cgal_lib ${CGAL_LIBRARIES})
   endif()
 endforeach()
 
+include(${CGAL_CONFIG_DIR}/CGALConfigVersion.cmake)
 
 #
 #
@@ -161,15 +162,6 @@ cgal_setup_module_path()
 set(CGAL_USE_FILE ${CGAL_MODULES_DIR}/UseCGAL.cmake)
 include(${CGAL_MODULES_DIR}/CGAL_target_use_TBB.cmake)
 
-include("${CGAL_MODULES_DIR}/CGAL_parse_version_h.cmake")
-cgal_parse_version_h( "${CGAL_INSTALLATION_PACKAGE_DIR}/include/CGAL/version.h"
-  "CGAL_VERSION_NAME"
-  "CGAL_MAJOR_VERSION"
-  "CGAL_MINOR_VERSION"
-  "CGAL_BUGFIX_VERSION"
-  "CGAL_BUILD_VERSION")
-set(CGAL_VERSION "${CGAL_MAJOR_VERSION}.${CGAL_MINOR_VERSION}.${CGAL_BUGFIX_VERSION}.${CGAL_BUILD_VERSION}")
-
 if( CGAL_DEV_MODE OR RUNNING_CGAL_AUTO_TEST )
   # Do not use -isystem for CGAL include paths
   set(CMAKE_NO_SYSTEM_FROM_IMPORTED TRUE)
diff -pruN 5.0.3-1/lib/cmake/CGAL/CGALConfigVersion.cmake 5.1-1/lib/cmake/CGAL/CGALConfigVersion.cmake
--- 5.0.3-1/lib/cmake/CGAL/CGALConfigVersion.cmake	2020-08-01 19:00:32.000000000 +0000
+++ 5.1-1/lib/cmake/CGAL/CGALConfigVersion.cmake	2020-09-08 12:12:19.000000000 +0000
@@ -1,7 +1,8 @@
 set(CGAL_MAJOR_VERSION 5)
-set(CGAL_MINOR_VERSION 0)
-set(CGAL_BUGFIX_VERSION 3)
-set(CGAL_VERSION_PUBLIC_RELEASE_VERSION "5.0.3")
+set(CGAL_MINOR_VERSION 1)
+set(CGAL_BUGFIX_VERSION 0)
+include(${CMAKE_CURRENT_LIST_DIR}/CGALConfigBuildVersion.cmake)
+set(CGAL_VERSION_PUBLIC_RELEASE_VERSION "5.1")
 set(CGAL_VERSION_PUBLIC_RELEASE_NAME "CGAL-${CGAL_VERSION_PUBLIC_RELEASE_VERSION}")
 
 if (CGAL_BUGFIX_VERSION AND CGAL_BUGFIX_VERSION GREATER 0)
diff -pruN 5.0.3-1/VERSION 5.1-1/VERSION
--- 5.0.3-1/VERSION	2020-08-01 19:03:00.000000000 +0000
+++ 5.1-1/VERSION	2020-09-08 12:15:53.000000000 +0000
@@ -1 +1 @@
-5.0.3
\ No newline at end of file
+5.1
\ No newline at end of file
